/******************************************************************************
*  Copyright(c)2002-03 : IFR Systems Inc : Wichita,KS : Design Engineering  
*******************************************************************************
* IFR 2975 P25 Radio Test Set                                               
*= IFR 2975 ===================================================================
* LabWindows/CVI 6.0 VXIpnp Instrument Driver                               
* Original Release: February 2003                                           
* By: Jeff Hand                                                             
******************************************************************************/

/******************************************************************************
*= IFR 2975 Include File ======================================================
*  Please do not use global variables or arrays in the include file of      
*  instrument drivers that will be submitted for inclusion into the         
*  LabWindows Instrument Driver Library.                                    
=============================================================================*/
#ifndef __ifr2975_HEADER

#define __ifr2975_HEADER

#include <vpptype.h>

#if defined(__cplusplus) || defined(__cplusplus__)
extern "C" {
#endif

/*****************************************************************************/
/*= Instrument Driver Specific Error/Warning Codes ==========================*/
/*****************************************************************************/
#define VI_ERROR_INSTR_FILE_OPEN                (_VI_ERROR+0x3FFC0800L)
#define VI_ERROR_INSTR_FILE_WRITE               (_VI_ERROR+0x3FFC0801L)
#define VI_ERROR_INSTR_INTERPRETING_RESPONSE    (_VI_ERROR+0x3FFC0803L)
#define VI_ERROR_INSTR_PARAMETER9               (_VI_ERROR+0x3FFC0809L)
#define VI_ERROR_INSTR_PARAMETER10              (_VI_ERROR+0x3FFC080AL)
#define VI_ERROR_INSTR_PARAMETER11              (_VI_ERROR+0x3FFC080BL)
#define VI_ERROR_INSTR_PARAMETER12              (_VI_ERROR+0x3FFC080CL)
#define VI_ERROR_INSTR_PARAMETER13              (_VI_ERROR+0x3FFC080DL)
#define VI_ERROR_INSTR_PARAMETER14              (_VI_ERROR+0x3FFC080EL)
#define VI_ERROR_INSTR_PARAMETER15              (_VI_ERROR+0x3FFC080FL)

/*****************************************************************************/
/*= Define Instrument Specific Error/Warning Codes Here =====================*/
/*****************************************************************************/
#define VI_WARNING_INSTR_OFFSET                         (0x3FFC0900L)
#define VI_ERROR_INSTR_OFFSET                   (_VI_ERROR+0x3FFC0900L)

#define ifr2975_ERROR_INSTRUMENT_ERROR           (VI_ERROR_INSTR_OFFSET + 0xF0L)

#define ifr2975_ERROR_INVALID_CONFIGURATION      (VI_ERROR_INSTR_OFFSET + 0xF1L)
#define ifr2975_ERROR_INSTRUMENT_OPTION          (VI_ERROR_INSTR_OFFSET + 0xF2L)
#define ifr2975_ERROR_INSTRUMENT_NSUP_MODEL      (VI_ERROR_INSTR_OFFSET + 0xF3L)
#define ifr2975_ERROR_SETTINGS_CONFLICT          (VI_ERROR_INSTR_OFFSET + 0xF4L)

/*****************************************************************************/
/*= Instrument specific defines =============================================*/
/*****************************************************************************/



/******************************************************************************
*                                                                           
******************************************************************************/


/******************************************************************************
=== Define Instrument Specific Error/Warning Codes Here =======================
******************************************************************************/

/******************************************************************************
=== GLOBAL USER-CALLABLE FUNCTION DECLARATIONS (Exportable Functions) =========
******************************************************************************/
ViStatus _VI_FUNC ifr2975_init (ViRsrc resourceName, ViBoolean IDQuery,
                               ViBoolean resetDevice, ViSession *instrumentHandle);

ViStatus _VI_FUNC ifr2975_Set_Avg_Count_AFC (ViSession instrumentHandle,
                                            ViInt16 samplestoAverageOver);

ViStatus _VI_FUNC ifr2975_Set_Trig_Mode_AFC (ViSession instrumentHandle,
                                            ViInt16 triggerMode);

ViStatus _VI_FUNC ifr2975_Set_Pk_Hold_State_AFC (ViSession instrumentHandle,
                                          ViInt16 peakHoldMode);

ViStatus _VI_FUNC ifr2975_Reset_Pk_Hold_AFC (ViSession instrumentHandle);

ViStatus _VI_FUNC ifr2975_Set_Alerts_AFC (ViSession instrumentHandle,
                                         ViInt16 alertMode);

ViStatus _VI_FUNC ifr2975_Set_UL_AFC (ViSession instrumentHandle,
                                     ViReal64 upperAlertLimit);

ViStatus _VI_FUNC ifr2975_Set_LL_AFC (ViSession instrumentHandle,
                                     ViReal64 lowerAlertLimit);

ViStatus _VI_FUNC ifr2975_Set_Pk_Hold_State_AFC (ViSession instrumentHandle,
                                         ViInt16 enableMode);

ViStatus _VI_FUNC ifr2975_Set_Enable_State_AFC (ViSession instrumentHandle,
                                               ViInt16 enableMode);

ViStatus _VI_FUNC ifr2975_Set_Run_State_AFC(ViSession instrumentHandle,
                                           ViInt16 runMode);

ViStatus _VI_FUNC ifr2975_Set_Gate_State_AFC (ViSession instrumentHandle,
                                            ViInt16 gateTime);

ViStatus _VI_FUNC ifr2975_Set_Avg_Count_FMDM (ViSession instrumentHandle,
                                             ViInt16 samplestoAverageOver);

ViStatus _VI_FUNC ifr2975_Set_Avg_Math_State_FMDM (ViSession instrumentHandle,
                                            ViInt16 averagingMode);

ViStatus _VI_FUNC ifr2975_Set_Trig_Mode_FMDM (ViSession instrumentHandle,
                                             ViInt16 triggerMode);

ViStatus _VI_FUNC ifr2975_Set_Pk_Hold_State_FMDM (ViSession instrumentHandle,
                                           ViInt16 peakHoldMode);

ViStatus _VI_FUNC ifr2975_Reset_Pk_Hold_FMDM (ViSession instrumentHandle);

ViStatus _VI_FUNC ifr2975_Set_Alerts_FMDM (ViSession instrumentHandle,
                                          ViInt16 alertMode);

ViStatus _VI_FUNC ifr2975_Set_UL_FMDM (ViSession instrumentHandle,
                                      ViReal64 upperAlertLimit);

ViStatus _VI_FUNC ifr2975_Set_LL_FMDM (ViSession instrumentHandle,
                                      ViReal64 lowerAlertLimit);

ViStatus _VI_FUNC ifr2975_Set_Enable_State_FMDM(ViSession instrumentHandle,
                                               ViInt16 enableMode);

ViStatus _VI_FUNC ifr2975_Set_Run_State_FMDM (ViSession instrumentHandle,
                                            ViInt16 runMode);

ViStatus _VI_FUNC ifr2975_Set_Filter_State_SINAD (ViSession instrumentHandle,
                                         ViInt16 filterSetting);

ViStatus _VI_FUNC ifr2975_Set_Avg_SINAD (ViSession instrumentHandle,
                                        ViInt16 samplestoAverageOver);

ViStatus _VI_FUNC ifr2975_Set_Avg_DSP_SINAD (ViSession instrumentHandle,
                                            ViInt16 samplestoAverageOver);

ViStatus _VI_FUNC ifr2975_Set_Trig_Mode_SINAD (ViSession instrumentHandle,
                                              ViInt16 triggerMode);

ViStatus _VI_FUNC ifr2975_Set_Pk_Hold_State_SINAD (ViSession instrumentHandle,
                                            ViInt16 peakHoldMode);

ViStatus _VI_FUNC ifr2975_Reset_Pk_Hold_SINAD (ViSession instrumentHandle);

ViStatus _VI_FUNC ifr2975_Set_Alerts_SINAD (ViSession instrumentHandle,
                                           ViInt16 alertMode);

ViStatus _VI_FUNC ifr2975_Set_UL_SINAD (ViSession instrumentHandle,
                                       ViReal64 upperAlertLimit);

ViStatus _VI_FUNC ifr2975_Set_LL_SINAD (ViSession instrumentHandle,
                                       ViReal64 lowerAlertLimit);

ViStatus _VI_FUNC ifr2975_Set_Enable_State_SINAD (ViSession instrumentHandle,
                                                ViInt16 enableMode);

ViStatus _VI_FUNC ifr2975_Set_Run_State_SINAD (ViSession instrumentHandle,
                                                ViInt16 runMode);

ViStatus _VI_FUNC ifr2975_Set_Filter_State_DIST (ViSession instrumentHandle,
                                        ViInt16 filterSetting);

ViStatus _VI_FUNC ifr2975_Set_Avg_DIST (ViSession instrumentHandle,
                                       ViInt16 samplestoAverageOver);

ViStatus _VI_FUNC ifr2975_Set_Avg_DSP_DIST (ViSession instrumentHandle,
                                           ViInt16 samplestoAverageOver);

ViStatus _VI_FUNC ifr2975_Set_Trig_Mode_DIST (ViSession instrumentHandle,
                                             ViInt16 triggerMode);

ViStatus _VI_FUNC ifr2975_Set_Pk_Hold_State_DIST (ViSession instrumentHandle,
                                           ViInt16 peakHoldMode);

ViStatus _VI_FUNC ifr2975_Reset_Pk_Hold_DIST (ViSession instrumentHandle);

ViStatus _VI_FUNC ifr2975_Set_Alerts_DIST (ViSession instrumentHandle,
                                          ViInt16 alertMode);

ViStatus _VI_FUNC ifr2975_Set_UL_DIST (ViSession instrumentHandle,
                                      ViReal64 upperAlertLimit);

ViStatus _VI_FUNC ifr2975_Set_LL_DIST (ViSession instrumentHandle,
                                      ViReal64 lowerAlertLimit);

ViStatus _VI_FUNC ifr2975_Set_Enable_State_DIST (ViSession instrumentHandle,
                                               ViInt16 enableMode);

ViStatus _VI_FUNC ifr2975_Set_Run_State_DIST (ViSession instrumentHandle,
                                            ViInt16 runMode);

ViStatus _VI_FUNC ifr2975_Set_Freq_FGEN (ViSession instrumentHandle,
                                        ViInt16 output, ViReal64 frequency);

ViStatus _VI_FUNC ifr2975_Set_Lev_FGEN (ViSession instrumentHandle,
                                       ViInt16 output, ViReal64 level);

ViStatus _VI_FUNC ifr2975_Set_Dev_FGEN (ViSession instrumentHandle,
                                       ViInt16 output, ViReal64 deviation);

ViStatus _VI_FUNC ifr2975_Set_Per_FGEN (ViSession instrumentHandle,
                                       ViInt16 output, ViReal64 percent);

ViStatus _VI_FUNC ifr2975_Set_Shape_FGEN (ViSession instrumentHandle,
                                         ViInt16 output, ViInt16 waveshape);

ViStatus _VI_FUNC ifr2975_Set_Dur_FGEN (ViSession instrumentHandle,
                                       ViInt16 output, ViInt32 duration);

ViStatus _VI_FUNC ifr2975_Set_Mode_FGEN (ViSession instrumentHandle,
                                        ViInt16 output, ViInt16 mode);

ViStatus _VI_FUNC ifr2975_Synchro_FGEN (ViSession instrumentHandle);

ViStatus _VI_FUNC ifr2975_Set_Demod_Filt_BBA (ViSession instrumentHandle,
                                             ViInt16 filterSetting);

ViStatus _VI_FUNC ifr2975_Set_FGen_Filt_BBA (ViSession instrumentHandle,
                                            ViInt16 filterSetting);

ViStatus _VI_FUNC ifr2975_Set_FGen_Bal_BBA (ViSession instrumentHandle,
                                           ViInt16 balanceSetting);

ViStatus _VI_FUNC ifr2975_Set_FGen_Lev_BBA (ViSession instrumentHandle,
                                           ViInt16 level);

ViStatus _VI_FUNC ifr2975_Set_SINAD_Bal_BBA (ViSession instrumentHandle,
                                            ViInt16 balanceSetting);

ViStatus _VI_FUNC ifr2975_Set_Mic_Bias_BBA (ViSession instrumentHandle,
                                           ViInt16 microphoneBias);

ViStatus _VI_FUNC ifr2975_Set_Aud_Mix_BBA (ViSession instrumentHandle,
                                          ViInt16 source, ViInt16 mixer,
                                          ViInt16 state);

ViStatus _VI_FUNC ifr2975_Clear_Aud_Mix_BBA (ViSession instrumentHandle, ViInt16 mixer);

ViStatus _VI_FUNC ifr2975_Set_Aud_Route_BBA (ViSession instrumentHandle,
                                            ViInt16 source, ViInt16 destination);

ViStatus _VI_FUNC ifr2975_Set_Avg_RFERR (ViSession instrumentHandle,
                                        ViInt16 samplestoAverageOver);

ViStatus _VI_FUNC ifr2975_Set_Pk_State_RFERR (ViSession instrumentHandle,
                                            ViInt16 peakHoldMode);

ViStatus _VI_FUNC ifr2975_Reset_Pk_Hold_RFERR (ViSession instrumentHandle);

ViStatus _VI_FUNC ifr2975_Set_Alerts_RFERR (ViSession instrumentHandle,
                                           ViInt16 alertMode);

ViStatus _VI_FUNC ifr2975_Set_UL_RFERR (ViSession instrumentHandle,
                                       ViReal64 upperAlertLimit);

ViStatus _VI_FUNC ifr2975_Set_LL_RFERR (ViSession instrumentHandle,
                                       ViReal64 lowerAlertLimit);

ViStatus _VI_FUNC ifr2975_Set_Enable_State_RFERR (ViSession instrumentHandle,
                                           ViInt16 enableMode);

ViStatus _VI_FUNC ifr2975_Set_Run_State_RFERR (ViSession instrumentHandle,
                                        ViInt16 runMode);

ViStatus _VI_FUNC ifr2975_Set_Gate_State_RFERR (ViSession instrumentHandle,
                                         ViInt16 gateTime);

ViStatus _VI_FUNC ifr2975_Set_Avg_DVM (ViSession instrumentHandle,
                                      ViInt16 samplestoAverageOver);

ViStatus _VI_FUNC ifr2975_Set_Pk_Hold_State_DVM (ViSession instrumentHandle,
                                          ViInt16 peakHoldMode);

ViStatus _VI_FUNC ifr2975_Reset_Pk_Hold_DVM (ViSession instrumentHandle);

ViStatus _VI_FUNC ifr2975_Set_Alerts_DVM (ViSession instrumentHandle,
                                         ViInt16 alertMode);

ViStatus _VI_FUNC ifr2975_Set_UL_DVM (ViSession instrumentHandle,
                                     ViReal64 upperAlertLimit);

ViStatus _VI_FUNC ifr2975_Set_LL_DVM (ViSession instrumentHandle,
                                     ViReal64 lowerAlertLimit);

ViStatus _VI_FUNC ifr2975_Set_Enable_DVM (ViSession instrumentHandle,
                                         ViInt16 enableMode);

ViStatus _VI_FUNC ifr2975_Set_Run_State_DVM (ViSession instrumentHandle,
                                      ViInt16 runMode);

ViStatus _VI_FUNC ifr2975_Set_Math_Mode_DVM (ViSession instrumentHandle,
                                            ViInt16 mathMode);

ViStatus _VI_FUNC ifr2975_Set_Imp_DVM (ViSession instrumentHandle,
                                      ViInt16 inputImpedance);

ViStatus _VI_FUNC ifr2975_Set_Hold_DVM (ViSession instrumentHandle,
                                       ViInt16 holdStatus);

ViStatus _VI_FUNC ifr2975_Set_Line_Filter_DVM (ViSession instrumentHandle,
                                              ViInt16 lineFilterSetting);

ViStatus _VI_FUNC ifr2975_Set_Range_DVM (ViSession instrumentHandle,
                                        ViInt16 range);

ViStatus _VI_FUNC ifr2975_Set_Source_DVM (ViSession instrumentHandle,
                                         ViInt16 inputSource);
                                         
ViStatus _VI_FUNC ifr2975_Set_Enable_State_DVM (ViSession instrumentHandle,
                                         ViInt16 enablemode);

ViStatus _VI_FUNC ifr2975_Set_Avg_C4FM (ViSession instrumentHandle,
                                       ViInt32 samplestoAverageOver);

ViStatus _VI_FUNC ifr2975_Set_Trig_Mode_C4FM (ViSession instrumentHandle,
                                             ViInt16 triggerMode);

ViStatus _VI_FUNC ifr2975_Set_Pk_Hold_State_C4FM (ViSession instrumentHandle,
                                           ViInt16 peakHoldMode);

ViStatus _VI_FUNC ifr2975_Reset_Pk_Hold_C4FM (ViSession instrumentHandle);

ViStatus _VI_FUNC ifr2975_Set_Alerts_C4FM (ViSession instrumentHandle,
                                          ViInt16 alertMode);

ViStatus _VI_FUNC ifr2975_Set_UL_C4FM (ViSession instrumentHandle,
                                      ViReal64 upperAlertLimit);

ViStatus _VI_FUNC ifr2975_Set_LL_C4FM (ViSession instrumentHandle,
                                      ViReal64 lowerAlertLimit);

ViStatus _VI_FUNC ifr2975_Set_Enable_State_C4FM (ViSession instrumentHandle,
                                          ViInt16 enableMode);

ViStatus _VI_FUNC ifr2975_Set_Run_State_C4FM (ViSession instrumentHandle,
                                       ViInt16 runMode);

ViStatus _VI_FUNC ifr2975_Set_Sweep_OSCOPE (ViSession instrumentHandle,
                                           ViReal64 sweepRate);

ViStatus _VI_FUNC ifr2975_Set_Vert_Scale_OSCOPE (ViSession instrumentHandle,
                                                ViInt16 channel,
                                                ViInt16 verticalScale);

ViStatus _VI_FUNC ifr2975_Set_Horiz_Pos_OSCOPE (ViSession instrumentHandle,
                                               ViReal64 offset);

ViStatus _VI_FUNC ifr2975_Set_Trig_Type_OSCOPE (ViSession instrumentHandle,
                                               ViInt16 triggerType);

ViStatus _VI_FUNC ifr2975_Set_Trig_Source_OSCOPE (ViSession instrumentHandle,
                                                 ViInt16 triggerSource);

ViStatus _VI_FUNC ifr2975_Set_Trig_Lev_OSCOPE (ViSession instrumentHandle,
                                              ViReal64 triggerLevel);

ViStatus _VI_FUNC ifr2975_Set_Trig_Pol_OSCOPE (ViSession instrumentHandle,
                                              ViInt16 triggerPolarity);

ViStatus _VI_FUNC ifr2975_Set_Pk_Hold_Reset_Flags_OSCOPE (ViSession instrumentHandle,
                                                         ViInt16 flagBits);

ViStatus _VI_FUNC ifr2975_Set_Avg_OSCOPE (ViSession instrumentHandle,
                                         ViInt16 samplestoAverageOver);

ViStatus _VI_FUNC ifr2975_Set_Input_Coupling_OSCOPE (ViSession instrumentHandle,
                                                    ViInt16 channel,
                                                    ViInt16 inputCoupling);

ViStatus _VI_FUNC ifr2975_Set_Trace_Math_Mode_OSCOPE (ViSession instrumentHandle,
                                                     ViInt16 mathType);

ViStatus _VI_FUNC ifr2975_Set_Run_State_OSCOPE (ViSession instrumentHandle, ViInt16 control,ViInt16 runMode);

ViStatus _VI_FUNC ifr2975_Set_Freq_REC (ViSession instrumentHandle,
                                       ViReal64 frequency);
                                       
ViStatus _VI_FUNC ifr2975_Set_RFIn_REC (ViSession instrumentHandle, ViInt16 input);

ViStatus _VI_FUNC ifr2975_Set_Attn_REC (ViSession instrumentHandle,
                                       ViInt16 attenuation);

ViStatus _VI_FUNC ifr2975_Set_Filter_REC (ViSession instrumentHandle,
                                         ViInt16 filter);

ViStatus _VI_FUNC ifr2975_Set_Demod_Rec (ViSession instrumentHandle,
                                        ViInt16 demodFormat);

ViStatus _VI_FUNC ifr2975_Set_Range_REC (ViSession instrumentHandle,
                                        ViInt16 range);

ViStatus _VI_FUNC ifr2975_Set_IFBW_REC (ViSession instrumentHandle,
                                       ViReal64 IFBandwidth);

ViStatus _VI_FUNC ifr2975_Set_IFGain_REC (ViSession instrumentHandle,
                                         ViReal64 IFGain);

ViStatus _VI_FUNC ifr2975_Set_HWAGC_REC (ViSession instrumentHandle,
                                        ViInt16 AGCSpeed);

ViStatus _VI_FUNC ifr2975_Set_SWAGC_State_REC (ViSession instrumentHandle,
                                        ViInt16 AGCMode);

ViStatus _VI_FUNC ifr2975_Set_Sweep_Lock_State_REC (ViSession instrumentHandle,
                                             ViInt16 setting);

ViStatus _VI_FUNC ifr2975_Set_Freq_GEN (ViSession instrumentHandle,
                                       ViReal64 frequency);

ViStatus _VI_FUNC ifr2975_Set_Level_GEN (ViSession instrumentHandle,
                                        ViReal64 level);

ViStatus _VI_FUNC ifr2975_Set_RFOut_GEN (ViSession instrumentHandle,
                                              ViInt16 outputPort);

ViStatus _VI_FUNC ifr2975_Set_HWALC_GEN (ViSession instrumentHandle,
                                        ViInt16 ALCSpeed);

ViStatus _VI_FUNC ifr2975_Set_BW_GEN (ViSession instrumentHandle,
                                     ViInt16 bandwidth);

ViStatus _VI_FUNC ifr2975_Set_Gen_State_GEN (ViSession instrumentHandle,
                                         ViBoolean output);

ViStatus _VI_FUNC ifr2975_Set_Yig_Speed_GEN (ViSession instrumentHandle,
                                            ViInt16 speed);

ViStatus _VI_FUNC ifr2975_Set_Mod_GEN (ViSession instrumentHandle,
                                      ViInt16 modulationMode);

ViStatus _VI_FUNC ifr2975_Set_P25_GEN (ViSession instrumentHandle, ViInt16 mode);

ViStatus _VI_FUNC ifr2975_Set_PTT_State_GEN (ViSession instrumentHandle, ViBoolean PTT);

ViStatus _VI_FUNC ifr2975_Set_Decode_State_APCO (ViSession instrumentHandle,
                                             ViBoolean decode);

ViStatus _VI_FUNC ifr2975_Set_Dec_Sample_Rate_APCO (ViSession instrumentHandle,
                                                   ViInt16 sampleRate);

ViStatus _VI_FUNC ifr2975_Set_Encrypt_State_APCO (ViSession instrumentHandle, ViInt16 Encrypt_State, ViInt16 ALGID);

ViStatus _VI_FUNC ifr2975_Set_C4FM_Mod_APCO (ViSession instrumentHandle,
                                            ViReal64 modulation);

ViStatus _VI_FUNC ifr2975_Set_Enable_State_WBPWR (ViSession instrumentHandle,
                                                ViInt16 enableMode);

ViStatus _VI_FUNC ifr2975_Zero_WBPWR (ViSession instrumentHandle);

ViStatus _VI_FUNC ifr2975_Set_Range_WBPWR (ViSession instrumentHandle,
                                          ViInt16 range);

ViStatus _VI_FUNC ifr2975_Set_Det_WBPWR (ViSession instrumentHandle,
                                        ViInt16 speed);

ViStatus _VI_FUNC ifr2975_Get_Enable_State_WBPWR (ViSession instrumentHandle,
                                           ViInt16 *enabledModes);

ViStatus _VI_FUNC ifr2975_Get_Range_WBPWR (ViSession instrumentHandle,
                                          ViInt16 *range);

ViStatus _VI_FUNC ifr2975_Get_Det_WBPWR (ViSession instrumentHandle,
                                        ViInt16 *detectorSetting);

ViStatus _VI_FUNC ifr2975_Set_Enable_State_RLMeter (ViSession instrumentHandle,
                                                  ViInt16 enableMode);

ViStatus _VI_FUNC ifr2975_Set_Update_Rate_RLMeter (ViSession instrumentHandle,
                                                  ViInt16 updateRate);

ViStatus _VI_FUNC ifr2975_Set_Cable_Loss_RLMeter (ViSession instrumentHandle,
                                                 ViReal64 cableLoss);

ViStatus _VI_FUNC ifr2975_Set_Vert_Scale_SA (ViSession instrumentHandle,
                                            ViInt16 scale);

ViStatus _VI_FUNC ifr2975_Set_TOS_SA (ViSession instrumentHandle,
                                     ViReal64 topofScreen);

ViStatus _VI_FUNC ifr2975_Set_Span_SA (ViSession instrumentHandle,
                                      ViReal64 startFrequency,
                                      ViReal64 stopFrequency);

ViStatus _VI_FUNC ifr2975_Set_RBW_SA (ViSession instrumentHandle,
                                     ViReal64 resolutionBandwidth);

ViStatus _VI_FUNC ifr2975_Set_VBW_SA (ViSession instrumentHandle,
                                     ViReal64 videoBandwidth);

ViStatus _VI_FUNC ifr2975_Set_IFGain_SA (ViSession instrumentHandle,
                                        ViReal64 IFGain);

ViStatus _VI_FUNC ifr2975_Set_Run_State_SA (ViSession instrumentHandle,
                                          ViInt16 runState);

ViStatus _VI_FUNC ifr2975_Set_Norm_State_SA (ViSession instrumentHandle,
                                             ViInt16 normalizeState);

ViStatus _VI_FUNC ifr2975_Set_Sweep_SA (ViSession instrumentHandle,
                                       ViReal64 speed);

ViStatus _VI_FUNC ifr2975_Set_Tracking_Gen_State_SA (ViSession instrumentHandle,
                                          ViInt16 tracking);


ViStatus _VI_FUNC ifr2975_Set_Screen_Mode (ViSession instrumentHandle,
                                          ViInt16 screenMode);

ViStatus _VI_FUNC ifr2975_Set_Time_Date_MISC (ViSession instrumentHandle,
                                             ViInt16 month, ViInt16 day,
                                             ViInt16 year, ViInt16 hour,
                                             ViInt16 minute);

ViStatus _VI_FUNC ifr2975_Set_Ref_MISC (ViSession instrumentHandle,
                                       ViBoolean referenceSource);

ViStatus _VI_FUNC ifr2975_Get_Avg_Count_AFC (ViSession instrumentHandle,
                                      ViInt16 *averagingCount);

ViStatus _VI_FUNC ifr2975_Get_Trig_Mode_AFC (ViSession instrumentHandle,
                                       ViInt16 *triggerMode);

ViStatus _VI_FUNC ifr2975_Get_Pk_Hold_State_AFC (ViSession instrumentHandle,
                                             ViInt16 *peakHoldMode);

ViStatus _VI_FUNC ifr2975_Get_Alert_Status_AFC (ViSession instrumentHandle,
                                               ViInt16 *alertEnableStatus);

ViStatus _VI_FUNC ifr2975_Get_UL_AFC (ViSession instrumentHandle,
                                     ViReal64 *upperAlertLimit);

ViStatus _VI_FUNC ifr2975_Get_LL_AFC (ViSession instrumentHandle,
                                     ViReal64 *lowerAlertLimit);

ViStatus _VI_FUNC ifr2975_Get_Enable_State_AFC(ViSession instrumentHandle,
                                              ViInt16 *enableMode);

ViStatus _VI_FUNC ifr2975_Get_Run_State_AFC (ViSession instrumentHandle,
                                            ViInt16 *runState);

ViStatus _VI_FUNC ifr2975_Get_Gate_State_AFC (ViSession instrumentHandle,
                                       ViInt16 *gateStatus);

ViStatus _VI_FUNC ifr2975_Get_Avg_Count_FMDM (ViSession instrumentHandle,
                                             ViInt16 *averagingCount);

ViStatus _VI_FUNC ifr2975_Get_Avg_Math_State_FMDM (ViSession instrumentHandle,
                                            ViInt16 *averagingMode);

ViStatus _VI_FUNC ifr2975_Get_Trig_Mode_FMDM (ViSession instrumentHandle,
                                             ViInt16 *triggerMode);

ViStatus _VI_FUNC ifr2975_Get_Pk_Hold_State_FMDM (ViSession instrumentHandle,
                                           ViInt16 *peakHoldMode);

ViStatus _VI_FUNC ifr2975_Get_Alert_Status_FMDM (ViSession instrumentHandle,
                                                ViInt16 *alertEnableStatus);

ViStatus _VI_FUNC ifr2975_Get_UL_FMDM (ViSession instrumentHandle,
                                      ViReal64 *upperAlertLimit);

ViStatus _VI_FUNC ifr2975_Get_LL_FMDM (ViSession instrumentHandle,
                                      ViReal64 *lowerAlertLimit);

ViStatus _VI_FUNC ifr2975_Get_Enable_State_FMDM(ViSession instrumentHandle,
                                               ViInt16 *enableMode);

ViStatus _VI_FUNC ifr2975_Get_Run_State_FMDM (ViSession instrumentHandle,
                                             ViInt16 *runState);

ViStatus _VI_FUNC ifr2975_Get_Filter_State_SINAD (ViSession instrumentHandle,
                                                ViInt16 *filteringStatus);

ViStatus _VI_FUNC ifr2975_Get_Avg_SINAD (ViSession instrumentHandle,
                                        ViInt16 *averagingCount);

ViStatus _VI_FUNC ifr2975_Get_Avg_DSP_SINAD (ViSession instrumentHandle,
                                            ViInt16 *averagingCount);

ViStatus _VI_FUNC ifr2975_Get_Trig_Mode_SINAD (ViSession instrumentHandle,
                                              ViInt16 *triggerMode);

ViStatus _VI_FUNC ifr2975_Get_Pk_Hold_State_SINAD (ViSession instrumentHandle,
                                            ViInt16 *peakHoldMode);

ViStatus _VI_FUNC ifr2975_Get_Alert_Status_SINAD (ViSession instrumentHandle,
                                                 ViInt16 *alertEnableStatus);

ViStatus _VI_FUNC ifr2975_Get_UL_SINAD (ViSession instrumentHandle,
                                       ViReal64 *upperAlertLimit);

ViStatus _VI_FUNC ifr2975_Get_LL_SINAD (ViSession instrumentHandle,
                                       ViReal64 *lowerAlertLimit);

ViStatus _VI_FUNC ifr2975_Get_Enable_State_SINAD (ViSession instrumentHandle,
                                                ViInt16 *enableMode);

ViStatus _VI_FUNC ifr2975_Get_Run_State_SINAD (ViSession instrumentHandle,
                                              ViInt16 *runState);


ViStatus _VI_FUNC ifr2975_Get_Filter_State_DIST (ViSession instrumentHandle,
                                               ViInt16 *filteringStatus);

ViStatus _VI_FUNC ifr2975_Get_Avg_DIST (ViSession instrumentHandle,
                                       ViInt16 *averagingCount);

ViStatus _VI_FUNC ifr2975_Get_Avg_DSP_DIST (ViSession instrumentHandle,
                                           ViInt16 *averagingCount);

ViStatus _VI_FUNC ifr2975_Get_Trig_Mode_DIST (ViSession instrumentHandle,
                                             ViInt16 *triggerMode);

ViStatus _VI_FUNC ifr2975_Get_Pk_Hold_State_DIST (ViSession instrumentHandle,
                                                ViInt16 *peakHoldMode);

ViStatus _VI_FUNC ifr2975_Get_Alert_Status_DIST (ViSession instrumentHandle,
                                                ViInt16 *alertEnableStatus);

ViStatus _VI_FUNC ifr2975_Get_UL_DIST (ViSession instrumentHandle,
                                      ViReal64 *upperAlertLimit);

ViStatus _VI_FUNC ifr2975_Get_LL_DIST (ViSession instrumentHandle,
                                      ViReal64 *lowerAlertLimit);

ViStatus _VI_FUNC ifr2975_Get_Enable_State_DIST (ViSession instrumentHandle,
                                               ViInt16 *enableMode);

ViStatus _VI_FUNC ifr2975_Get_Run_State_DIST (ViSession instrumentHandle,
                                             ViInt16 *runState);

ViStatus _VI_FUNC ifr2975_Get_Freq_FGEN (ViSession instrumentHandle,
                                        ViInt16 source, ViReal64 *frequency);

ViStatus _VI_FUNC ifr2975_Get_Lev_FGEN (ViSession instrumentHandle,
                                       ViInt16 source, ViReal64 *level);

ViStatus _VI_FUNC ifr2975_Get_Dev_FGEN (ViSession instrumentHandle,
                                       ViInt16 source, ViReal64 *deviation);
                                       
ViStatus _VI_FUNC ifr2975_Get_Per_FGEN (ViSession instrumentHandle,
                                       ViInt16 source, ViReal64 *percent);

ViStatus _VI_FUNC ifr2975_Get_Shape_FGEN (ViSession instrumentHandle,
                                         ViInt16 source, ViInt16 *shape);

ViStatus _VI_FUNC ifr2975_Get_Dur_FGEN (ViSession instrumentHandle,
                                       ViInt16 source, ViReal64 *duration);

ViStatus _VI_FUNC ifr2975_Get_Mode_FGEN (ViSession instrumentHandle,
                                        ViInt16 source, ViInt16 *mode);

ViStatus _VI_FUNC ifr2975_Get_Demod_Filt_BBA (ViSession instrumentHandle,
                                             ViInt16 *filter);

ViStatus _VI_FUNC ifr2975_Get_FGen_Filt_BBA (ViSession instrumentHandle,
                                            ViInt16 *filter);

ViStatus _VI_FUNC ifr2975_Get_FGen_Bal_BBA (ViSession instrumentHandle,
                                           ViInt16 *balance);

ViStatus _VI_FUNC ifr2975_Get_FGen_Level_BBA (ViSession instrumentHandle,
                                             ViInt16 *level);

ViStatus _VI_FUNC ifr2975_Get_SINAD_Bal_BBA (ViSession instrumentHandle,
                                            ViInt16 *balance);

ViStatus _VI_FUNC ifr2975_Get_Mic_Bias_BBA (ViSession instrumentHandle,
                                           ViInt16 *bias);

ViStatus _VI_FUNC ifr2975_Get_Avg_RFERR (ViSession instrumentHandle,
                                        ViInt16 *averagingCount);

ViStatus _VI_FUNC ifr2975_Get_Alerts_RFERR (ViSession instrumentHandle,
                                                 ViInt16 *alertEnableStatus);
                                                 
ViStatus _VI_FUNC ifr2975_Get_UL_RFERR (ViSession instrumentHandle,
                                       ViReal64 *upperAlertLimit);

ViStatus _VI_FUNC ifr2975_Get_LL_RFERR (ViSession instrumentHandle,
                                       ViReal64 *lowerAlertLimit);

ViStatus _VI_FUNC ifr2975_Get_Enable_State_RFERR (ViSession instrumentHandle,
                                                ViInt16 *enableMode);

ViStatus _VI_FUNC ifr2975_Get_Run_State_RFERR (ViSession instrumentHandle,
                                              ViInt16 *runState);

ViStatus _VI_FUNC ifr2975_Get_Gate_State_RFERR (ViSession instrumentHandle,
                                         ViInt16 *gateStatus);

ViStatus _VI_FUNC ifr2975_Get_Avg_DVM (ViSession instrumentHandle,
                                      ViInt16 *averagingCount);

ViStatus _VI_FUNC ifr2975_Get_Pk_Hold_State_DVM (ViSession instrumentHandle,
                                               ViInt16 *peakHoldMode);

ViStatus _VI_FUNC ifr2975_Get_Alert_Status_DVM (ViSession instrumentHandle,
                                               ViInt16 *alertEnableStatus);

ViStatus _VI_FUNC ifr2975_Get_UL_DVM (ViSession instrumentHandle,
                                     ViReal64 *upperAlertLimit);

ViStatus _VI_FUNC ifr2975_Get_LL_DVM (ViSession instrumentHandle,
                                     ViReal64 *lowerAlertLimit);

ViStatus _VI_FUNC ifr2975_Get_Enable_State_DVM (ViSession instrumentHandle,
                                               ViInt16 *inputSource);

ViStatus _VI_FUNC ifr2975_Get_Enable_DVM (ViSession instrumentHandle,
                                                 ViInt16 *enableMode);

ViStatus _VI_FUNC ifr2975_Get_Run_State_DVM (ViSession instrumentHandle,
                                            ViInt16 *runState);

ViStatus _VI_FUNC ifr2975_Get_Math_Mode_DVM (ViSession instrumentHandle,
                                            ViInt16 *mathMode);

ViStatus _VI_FUNC ifr2975_Get_Imp_DVM (ViSession instrumentHandle,
                                      ViInt16 *impedance);

ViStatus _VI_FUNC ifr2975_Get_Hold_DVM (ViSession instrumentHandle,
                                       ViInt16 *holdState);

ViStatus _VI_FUNC ifr2975_Get_Line_Filter_DVM (ViSession instrumentHandle,
                                              ViInt16 *lineFilterSetting);

ViStatus _VI_FUNC ifr2975_Get_Range_DVM (ViSession instrumentHandle,
                                        ViInt16 *range);

ViStatus _VI_FUNC ifr2975_Get_Source_DVM (ViSession instrumentHandle,
                                         ViInt16 *source);
                                    
                                        
ViStatus _VI_FUNC ifr2975_Get_Enable_DVM (ViSession instrumentHandle,
                                         ViInt16 *enablemode);                                         

ViStatus _VI_FUNC ifr2975_Get_Avg_C4FM (ViSession instrumentHandle,
                                       ViInt32 *averagingCount);

ViStatus _VI_FUNC ifr2975_Get_Trig_Mode_C4FM (ViSession instrumentHandle,
                                             ViInt16 *triggerMode);

ViStatus _VI_FUNC ifr2975_Get_Pk_Hold_State_C4FM (ViSession instrumentHandle,
                                                ViInt16 *peakHoldMode);

ViStatus _VI_FUNC ifr2975_Get_Alert_Status_C4FM (ViSession instrumentHandle,
                                                ViInt16 *alertEnableStatus);

ViStatus _VI_FUNC ifr2975_Get_Pk_Hold_State_RFERR (ViSession instrumentHandle,
                                                 ViInt16 *peakHoldMode);

ViStatus _VI_FUNC ifr2975_Get_UL_C4FM (ViSession instrumentHandle,
                                      ViReal64 *upperAlertLimit);

ViStatus _VI_FUNC ifr2975_Get_LL_C4FM (ViSession instrumentHandle,
                                      ViReal64 *lowerAlertLimit);

ViStatus _VI_FUNC ifr2975_Get_Enable_State_C4FM (ViSession instrumentHandle,
                                               ViInt16 *enableMode);

ViStatus _VI_FUNC ifr2975_Get_Run_State_OSCOPE (ViSession instrumentHandle,
                                               ViInt16 scope, ViInt16 *runState);

ViStatus _VI_FUNC ifr2975_Get_Run_State_C4FM (ViSession instrumentHandle,
                                             ViInt16 *runState);

ViStatus _VI_FUNC ifr2975_Get_Sweep_OSCOPE (ViSession instrumentHandle,
                                           ViReal64 *sweepSpeed);

ViStatus _VI_FUNC ifr2975_Get_Vert_Scale_OSCOPE (ViSession instrumentHandle,
                                                ViInt16 channel,
                                                ViInt16 *verticalScale);

ViStatus _VI_FUNC ifr2975_Get_Horiz_Pos_OSCOPE (ViSession instrumentHandle,
                                               ViReal64 *horizontalPosition);

ViStatus _VI_FUNC ifr2975_Get_Trig_Type_OSCOPE (ViSession instrumentHandle,
                                               ViInt16 *triggerType);

ViStatus _VI_FUNC ifr2975_Get_Trig_Source_OSCOPE (ViSession instrumentHandle,
                                                 ViInt16 *triggerSource);

ViStatus _VI_FUNC ifr2975_Get_Trig_Lev_OSCOPE (ViSession instrumentHandle,
                                              ViReal64 *triggerLevel);

ViStatus _VI_FUNC ifr2975_Get_Trig_Pol_OSCOPE (ViSession instrumentHandle,
                                              ViInt16 *triggerPolarity);

ViStatus _VI_FUNC ifr2975_Get_Pk_Hold_Reset_Flags_OSCOPE (ViSession instrumentHandle,
                                                         ViInt16 *flagBits);

ViStatus _VI_FUNC ifr2975_Get_Avg_OSCOPE (ViSession instrumentHandle,
                                         ViInt16 *averagingCount);

ViStatus _VI_FUNC ifr2975_Get_Input_Coupling_OSCOPE (ViSession instrumentHandle,
                                                    ViInt16 Channel, ViInt16 *inputCoupling);

ViStatus _VI_FUNC ifr2975_Get_Trace_Math_Mode_OSCOPE (ViSession instrumentHandle,
                                                     ViInt16 *mathMode);

ViStatus _VI_FUNC ifr2975_Get_Trace_OSCOPE (ViSession instrumentHandle, ViInt16 control, 
															ViInt32 *length_array, ViReal64 *TEMP_DATA);

ViStatus _VI_FUNC ifr2975_Get_Freq_REC (ViSession instrumentHandle,
                                       ViReal64 *frequency);

ViStatus _VI_FUNC ifr2975_Get_RFIn_REC (ViSession instrumentHandle,
                                       ViInt16 *RFInput);

ViStatus _VI_FUNC ifr2975_Get_Attn_REC (ViSession instrumentHandle,
                                       ViInt16 *attenuation);

ViStatus _VI_FUNC ifr2975_Get_Filter_REC (ViSession instrumentHandle,
                                         ViInt16 *filter);

ViStatus _VI_FUNC ifr2975_Get_Demod_REC (ViSession instrumentHandle,
                                        ViInt16 *demodulation);

ViStatus _VI_FUNC ifr2975_Get_Range_REC (ViSession instrumentHandle,
                                        ViInt16 *range);

ViStatus _VI_FUNC ifr2975_Get_IFBW_REC (ViSession instrumentHandle,
                                       ViReal64 *IFBandwidth);

ViStatus _VI_FUNC ifr2975_Get_IFGain_REC (ViSession instrumentHandle,
                                         ViReal64 *IFGain);

ViStatus _VI_FUNC ifr2975_Get_HWAGC_REC (ViSession instrumentHandle,
                                        ViInt16 *HWAGCSpeed);

ViStatus _VI_FUNC ifr2975_Get_SWAGC_State_REC (ViSession instrumentHandle,
                                        ViInt16 *softwareAGCMode);

ViStatus _VI_FUNC ifr2975_Get_Sweep_Lock_State_REC (ViSession instrumentHandle,
                                             ViInt16 *setting);

ViStatus _VI_FUNC ifr2975_Get_Freq_GEN (ViSession instrumentHandle,
                                       ViReal64 *frequency);

ViStatus _VI_FUNC ifr2975_Get_Level_GEN (ViSession instrumentHandle,
                                        ViReal64 *level);

ViStatus _VI_FUNC ifr2975_Get_RFOut_GEN (ViSession instrumentHandle,
                                              ViInt16 *outputPort);

ViStatus _VI_FUNC ifr2975_Get_HWALC_GEN (ViSession instrumentHandle,
                                        ViInt16 *ALCSetting);

ViStatus _VI_FUNC ifr2975_Get_BW_GEN (ViSession instrumentHandle,
                                     ViInt16 *bandwidth);

ViStatus _VI_FUNC ifr2975_Get_Gen_State_GEN (ViSession instrumentHandle,
                                         ViInt16 *onOff);

ViStatus _VI_FUNC ifr2975_Get_Yig_Speed_GEN (ViSession instrumentHandle,
                                            ViInt16 *yigSpeed);

ViStatus _VI_FUNC ifr2975_Get_Mod_GEN (ViSession instrumentHandle,
                                      ViInt16 *modulationMode);

ViStatus _VI_FUNC ifr2975_Get_P25_GEN (ViSession instrumentHandle,
                                      ViInt16 *p25Mode);

ViStatus _VI_FUNC ifr2975_Get_PTT_State_GEN (ViSession instrumentHandle,
                                      ViInt16 *PTTState);

ViStatus _VI_FUNC ifr2975_Get_Decode_State_APCO (ViSession instrumentHandle,
                                                 ViInt16 *decodeEnableStatus);

ViStatus _VI_FUNC ifr2975_Get_Dec_Sample_Rate_APCO (ViSession instrumentHandle,
                                                   ViInt16 *decodeSampleRate);

//ViStatus _VI_FUNC ifr2975_Get_Encrypt_State_APCO (ViSession instrumentHandle,
//                                                     ViInt16 *encryptionStatus);

//ViStatus _VI_FUNC ifr2975_Get_Encrypt_State_APCO (ViSession instrumentHandle,
//                                                     ViInt16 *encryptionStatus,
//                                                      ViInt16 *ALGID);

ViStatus _VI_FUNC ifr2975_Get_Encrypt_State_APCO (ViSession instrumentHandle,
                                                     ViString encryptionStatus,
                                                      ViString ALGID);

ViStatus _VI_FUNC ifr2975_Get_C4FM_Mod_APCO (ViSession instrumentHandle,
                                                   ViReal64 *C4FMModulation);

ViStatus _VI_FUNC ifr2975_Get_Enable_State_RLMeter (ViSession instrumentHandle,
                                                  ViInt16 *enableMode);

ViStatus _VI_FUNC ifr2975_Get_Update_Rate_RLMeter (ViSession instrumentHandle,
                                                  ViInt16 *updateRate);

ViStatus _VI_FUNC ifr2975_Get_Cable_Loss_RLMeter (ViSession instrumentHandle,
                                                 ViReal64 *cableLoss);

ViStatus _VI_FUNC ifr2975_Get_Vert_Scale_SA (ViSession instrumentHandle,
                                            ViInt16 *verticalScale);

ViStatus _VI_FUNC ifr2975_Get_TOS_SA (ViSession instrumentHandle,
                                     ViReal64 *topofScreen);

ViStatus _VI_FUNC ifr2975_Get_Span_SA (ViSession instrumentHandle,
                                              ViReal64 *startFreq,
                                              ViReal64 *stopFreq);

ViStatus _VI_FUNC ifr2975_Get_RBW_SA (ViSession instrumentHandle,
                                     ViReal64 *resolutionBandwidth);

ViStatus _VI_FUNC ifr2975_Get_VBW_SA (ViSession instrumentHandle,
                                     ViReal64 *videoBandwidth);

ViStatus _VI_FUNC ifr2975_Get_IFGain_SA (ViSession instrumentHandle,
                                         ViReal64 *IFGain);

ViStatus _VI_FUNC ifr2975_Get_Run_State_SA (ViSession instrumentHandle,
                                           ViInt16 *runState);

ViStatus _VI_FUNC ifr2975_Get_Norm_State_SA (ViSession instrumentHandle,
                                         ViInt16 *normalizeState);

ViStatus _VI_FUNC ifr2975_Get_Sweep_SA (ViSession instrumentHandle,
                                       ViReal64 *sweepSpeed);

ViStatus _VI_FUNC ifr2975_Get_Tracking_Gen_State_SA (ViSession instrumentHandle,
                                           ViInt16 *tracking);

ViStatus _VI_FUNC ifr2975_Query_Ver_RLMETER (ViSession instrumentHandle,
                                            ViInt16 system, ViChar version[]);

ViStatus _VI_FUNC ifr2975_Get_Trace_SA (ViSession instrumentHandle, 
										ViInt32 *length_array, ViReal64 *TEMP_DATA);

ViStatus _VI_FUNC ifr2975_Query_Ver_SA (ViSession instrumentHandle,
                                       ViChar version[], ViInt16 ID);

ViStatus _VI_FUNC ifr2975_Meas_Freq_AFC (ViSession instrumentHandle,
                                        ViReal64 *frequency);

ViStatus _VI_FUNC ifr2975_Query_Ver_AFC (ViSession instrumentHandle,
                                        ViInt16 system, ViChar version[]);

ViStatus _VI_FUNC ifr2975_Meas_Dev_FMDM (ViSession instrumentHandle,
                                        ViReal64 *deviation);

ViStatus _VI_FUNC ifr2975_Query_Ver_FMDM (ViSession instrumentHandle,
                                         ViInt16 system, ViChar version[]);

ViStatus _VI_FUNC ifr2975_Meas_SINAD_SINAD (ViSession instrumentHandle,
                                           ViReal64 *SINAD);

ViStatus _VI_FUNC ifr2975_Query_Ver_SINAD (ViSession instrumentHandle,
                                          ViInt16 system, ViChar version[]);

ViStatus _VI_FUNC ifr2975_Meas_Dist_DIST (ViSession instrumentHandle,
                                         ViReal64 *distortion);

ViStatus _VI_FUNC ifr2975_Query_Ver_DIST (ViSession instrumentHandle,
                                         ViInt16 system, ViChar version[]);

ViStatus _VI_FUNC ifr2975_Query_Ver_FGEN (ViSession instrumentHandle,
                                         ViInt16 system, ViChar version[]);

ViStatus _VI_FUNC ifr2975_Query_Ver_BBA (ViSession instrumentHandle,
                                        ViInt16 system, ViChar version[]);

ViStatus _VI_FUNC ifr2975_Meas_Freq_Err_RFERR (ViSession instrumentHandle,
                                              ViReal64 *frequencyError);

ViStatus _VI_FUNC ifr2975_Query_Ver_RFERR (ViSession instrumentHandle,
                                          ViInt16 system, ViChar version[]);

ViStatus _VI_FUNC ifr2975_Meas_Volt_DVM (ViSession instrumentHandle,
                                        ViReal64 *voltage);

ViStatus _VI_FUNC ifr2975_Query_Ver_DVM (ViSession instrumentHandle,
                                        ViInt16 system, ViChar version[]);

ViStatus _VI_FUNC ifr2975_Meas_Error_C4FM (ViSession instrumentHandle,
                                          ViReal64 *error);

ViStatus _VI_FUNC ifr2975_Query_Ver_C4FM (ViSession instrumentHandle,
                                         ViInt16 system, ViChar version[]);

ViStatus _VI_FUNC ifr2975_Trig_Arm_OSCOPE (ViSession instrumentHandle);

ViStatus _VI_FUNC ifr2975_Query_Ver_OSCOPE (ViSession instrumentHandle,
                                           ViInt16 system, ViChar version[]);

ViStatus _VI_FUNC ifr2975_Query_Ver_REC (ViSession instrumentHandle,
                                           ViInt16 system, ViChar version[]);

ViStatus _VI_FUNC ifr2975_Query_Ver_GEN (ViSession instrumentHandle,
                                           ViInt16 system, ViChar version[]);

ViStatus _VI_FUNC ifr2975_Query_Ver_APCO (ViSession instrumentHandle,
                                         ViInt16 system, ViChar version[]);

ViStatus _VI_FUNC ifr2975_Meas_Power_WBPWR (ViSession instrumentHandle,
                                           ViReal64 *reading);

ViStatus _VI_FUNC ifr2975_Meas_RLMeter (ViSession instrumentHandle,
                                       ViReal64 *receivedLevel);

ViStatus _VI_FUNC ifr2975_Reset (ViSession instrumentHandle);

ViStatus _VI_FUNC ifr2975_Error_Query (ViSession instrumentHandle,
                                      ViInt32 *errorCode, ViChar errorMessage[]);

ViStatus _VI_FUNC ifr2975_Error_Message (ViSession instrumentHandle,
                                        ViStatus statusCode, ViChar errMessage[]);

ViStatus _VI_FUNC ifr2975_Revision_Query (ViSession instrumentHandle,
                                         ViChar driverRev[], ViChar intrRev[]);

ViStatus _VI_FUNC ifr2975_readInstrData (ViSession instrumentHandle,
                                        ViInt32 numberBytesToRead,
                                        ViChar readBuffer[],
                                        ViInt32 *numBytesRead);

ViStatus _VI_FUNC ifr2975_writeInstrData (ViSession instrumentHandle,
                                         ViChar writeBuffer[]);

ViStatus _VI_FUNC ifr2975_Read_Write_From_Instrument (ViSession instrumentHandle,
                                                     ViChar writeData[],
                                                     ViChar readData[]);

ViStatus _VI_FUNC ifr2975_Self_Test (ViSession instrumentHandle,
                                    ViInt16 *testResult, ViChar testMessage[]);

ViStatus _VI_FUNC ifr2975_Close_Conn (ViSession instrumentHandle);

ViStatus _VI_FUNC ifr2975_Close (ViSession instrumentHandle);


ViStatus _VI_FUNC ifr2975_Set_Encode_Bits_APCO (ViSession instrumentHandle,
                                            ViInt16 encode, ViInt32 value);

#if defined(__cplusplus) || defined(__cplusplus__)
}
#endif

/******************************************************************************
===== END INCLUDE FILE ========================================================
******************************************************************************/

#endif
