/*****************************************************************************
*  Copyright(c)2002-03 : IFR Systems Inc : Wichita,KS : Design Engineering  
******************************************************************************
* IFR 2975 P25 Radio Test Set 
*= IFR 2975 ==================================================================
* LabWindows/CVI 6.0 VXIpnp Instrument Driver UI
* Original Release: February 2003                                           
*               By: Jeff Hand                                                             
*****************************************************************************/

#include <visa.h>
#include <ANSI_C.h>
#include <utility.h>
#include <formatio.h>
#include "ifr2975.h"
#include <cvirte.h>        /* Needed if linking in external compiler; harmless otherwise */
#include <userint.h>
#include "ifr_2975_uir.h"

/*****************************************************************************
* LabWindows/CVI 5.0 Instrument Driver User Interface
* By:   Jeff Hand, IFR SYSTEMS INC.
*****************************************************************************/

static int panelHandle[20], conn;

ViSession handle;

int abortApplication(void);

/*****************************************************************************
* Function: main
* Purpose:  This main function for the Instrument Driver UI.
*                                                           
*****************************************************************************/
int main (int argc, char *argv[]) {
    ViInt32 i;
    ViInt16 ViStatus = VI_SUCCESS;
    
    /*
    * Displays information about UI and Driver
    */
    MessagePopup ("IFR 2975 CVI User Interface - Example",
                  "This User Interface is for use with the IFR 2975.\n\n"
                  "This User Interface is only an example of how the CVI driver could be used.\n\n"
                  "It incorporates the use of GPIB, Ethernet, and Serial connections.\n\n"
                  "Aeroflex, Inc. \n\n"
                  "For RS232 please set Baud Speed to 115200 on unit.");
    
    /*
    * Creates all the Panels 
    */
    if ((panelHandle[PANEL] = LoadPanel (panelHandle[PANEL], "ifr_2975_uir.uir", PANEL)) < 0)
        return -1;
    if ((panelHandle[CONN] = LoadPanel (panelHandle[PANEL], "ifr_2975_uir.uir", CONN)) < 0)
        return -1;
    if ((panelHandle[GEN] = LoadPanel (panelHandle[PANEL], "ifr_2975_uir.uir", GEN)) < 0)
        return -1;
    if ((panelHandle[SA] = LoadPanel (panelHandle[PANEL], "ifr_2975_uir.uir", SA)) < 0)
        return -1;
    if ((panelHandle[SCOPE] = LoadPanel (panelHandle[PANEL], "ifr_2975_uir.uir", SCOPE)) < 0)
        return -1;
    if ((panelHandle[RECEIVER] = LoadPanel (panelHandle[PANEL], "ifr_2975_uir.uir", RECEIVER)) < 0)
        return -1;
    if ((panelHandle[METER] = LoadPanel (panelHandle[PANEL], "ifr_2975_uir.uir", METER)) < 0)
        return -1;
    if ((panelHandle[FGEN] = LoadPanel (panelHandle[PANEL], "ifr_2975_uir.uir", FGEN)) < 0)
        return -1;

    /*
    * Displays panels that will open on startup 
    */
    DisplayPanel (panelHandle[PANEL]);
    DisplayPanel (panelHandle[CONN]);
    //DisplayPanel (panelHandle[GEN]);
    //DisplayPanel (panelHandle[SA]);
    //DisplayPanel (panelHandle[SCOPE]);
    //DisplayPanel (panelHandle[RECEIVER]);
    //DisplayPanel (panelHandle[METER]);
    //DisplayPanel (panelHandle[FGEN]);
    
    /*
    * Starts the UI 
    */
    RunUserInterface ();
    
    /*
    * Closes all connections upon exiting program 
    */
    ifr2975_Close_Conn (handle);    
    
    return 0;
}

/*****************************************************************************
* Function: EXIT_BUTTON                                 
* Purpose:  This is the exit button for the main panel. 
*                                                       
*****************************************************************************/
int CVICALLBACK EXIT_BUTTON (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    ViInt16 result;
    ViInt16 ViStatus = VI_SUCCESS;
    
    switch (event)
        {
        case EVENT_COMMIT:
            /*
            * Checks to make sure the user wants to disconnect from box 
            */
            result = ConfirmPopup ("IFR 2975 Instrument Driver UI",
                                   "Would you like to Exit this program?");
            
            /*
            * If the user wants to exit 
            */
            if (result==1) 
                {
			    /*
			    * Disconnects all connections to 2975 
			    */
			    ifr2975_Close_Conn (handle);
    
    
				exit (EXIT_SUCCESS);
                }
            
            break;
        case EVENT_RIGHT_CLICK:

            break;
        }
    return 0;
}


/*****************************************************************************
* Function: ETH_CONN                                    
* Purpose:  This is the ethernet connect/disconnect switch for the UI.  
*                                                       
*****************************************************************************/
int CVICALLBACK ETH_CONN (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    ViInt32 toggle;
    char *ip;
    char ip_string[100];
    char *port;
    char port_string[100];
    ViInt32 ifr2975_status;
    ViInt32 protocol_toggle;
    ViByte serial_address[100];
    
    ip = &ip_string[0];
    port = &port_string[0];
    
    switch (event)
        {
        case EVENT_COMMIT:
            SetCtrlAttribute (panelHandle[GEN], GEN_UPDATE_GEN, ATTR_DIMMED, 1);
            SetCtrlAttribute (panelHandle[SCOPE], SCOPE_UPDATE_SCOPE, ATTR_DIMMED, 1);
            SetCtrlAttribute (panelHandle[FGEN], FGEN_UPDATE_FGEN, ATTR_DIMMED, 1);
            SetCtrlAttribute (panelHandle[SA], SA_UPDATE_SA, ATTR_DIMMED, 1);
            SetCtrlAttribute (panelHandle[METER], METER_UPDATE_METERS, ATTR_DIMMED, 1);
            SetCtrlAttribute (panelHandle[RECEIVER], RECEIVER_UPDATE_REC, ATTR_DIMMED, 1);
            SetCtrlAttribute (panelHandle[PANEL], PANEL_DEFAULTS_UI, ATTR_DIMMED, 1);
            SetCtrlAttribute (panelHandle[PANEL], PANEL_EXIT, ATTR_DIMMED, 1);
            SetCtrlAttribute (panelHandle[PANEL], PANEL_UPDATE_GO, ATTR_DIMMED, 1);
            SetCtrlAttribute (panelHandle[PANEL], PANEL_UPDATE_UI, ATTR_DIMMED, 1);

            
            /*
            * Tells user that the UI is communicating with the box 
            */
			SetWaitCursor(1);
        
            GetCtrlVal (panelHandle[CONN], CONN_ETH_CONN, &toggle);
            
            if (toggle == 1) {
                SetCtrlVal (panelHandle[CONN], CONN_ETH_BUSY, 0);
            
                /*
                * Get IP Address and Port number 
                */
                GetCtrlVal (panelHandle[CONN], CONN_IP_ADDR, ip);
                
                GetCtrlVal (panelHandle[CONN], CONN_connect_type, &protocol_toggle);            

                /*
                * Setup for TCPIP communication of the 2975 
                */
                Fmt (serial_address, "%s<TCPIP::%s::23::SOCKET",ip);
                if (ifr2975_init (serial_address, 0, 0, &handle)<0)
                    return -1;
                
                SetCtrlVal (panelHandle[CONN], CONN_connect_type, 1);            
                
                SetCtrlVal (panelHandle[CONN], CONN_ETH_BUSY, 1);
            } else {
                SetCtrlVal (panelHandle[CONN], CONN_ETH_BUSY, 1);
            
                /*
                * Disconnects from the box 
                */
                if ((ifr2975_status = ifr2975_Close_Conn (handle)) < 0){
                    SetCtrlVal (panelHandle[CONN], CONN_ETH_CONN, 0);
                    MessagePopup ("IFR 2975 Instrument Driver UI",
                              "Ethernet Disconnection has failed");
                    
                    /*
                    * Exits UI 
                    */
                    abortApplication();
                }
            
                SetCtrlVal (panelHandle[CONN], CONN_ETH_BUSY, 0);
                
                /*
                * resets ethernet handle 
                */
                handle = 0;
                
                GetCtrlVal (panelHandle[CONN], CONN_connect_type, &protocol_toggle);            
            
            }
            
            /*
            * Tells user that the UI is not longer communicating with the box
            */
            SetWaitCursor(0);

            SetCtrlAttribute (panelHandle[GEN], GEN_UPDATE_GEN, ATTR_DIMMED, 0);
            SetCtrlAttribute (panelHandle[SCOPE], SCOPE_UPDATE_SCOPE, ATTR_DIMMED, 0);
            SetCtrlAttribute (panelHandle[FGEN], FGEN_UPDATE_FGEN, ATTR_DIMMED, 0);
            SetCtrlAttribute (panelHandle[SA], SA_UPDATE_SA, ATTR_DIMMED, 0);
            SetCtrlAttribute (panelHandle[METER], METER_UPDATE_METERS, ATTR_DIMMED, 0);
            SetCtrlAttribute (panelHandle[RECEIVER], RECEIVER_UPDATE_REC, ATTR_DIMMED, 0);
            SetCtrlAttribute (panelHandle[PANEL], PANEL_DEFAULTS_UI, ATTR_DIMMED, 0);
            SetCtrlAttribute (panelHandle[PANEL], PANEL_EXIT, ATTR_DIMMED, 0);
            SetCtrlAttribute (panelHandle[PANEL], PANEL_UPDATE_GO, ATTR_DIMMED, 0);
            SetCtrlAttribute (panelHandle[PANEL], PANEL_UPDATE_UI, ATTR_DIMMED, 0);
            
            break;
        case EVENT_RIGHT_CLICK:
            /*
            * Help Message for given widget 
            */
            MessagePopup("HELP: Ethernet Connect/Disconnect Toggle Switch", "This button connects/disconnects ethernet from the unit."); 
            
            break;
        }
    return 0;
}

/*****************************************************************************
* Function: SERIAL_CONN                                 
* Purpose:  This is the serial connect/disconnect switch for the UI.        
*                                                       
*****************************************************************************/
int CVICALLBACK SERIAL_CONN (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    ViInt32 toggle;
    ViInt32 ifr2975_status;
    ViInt32 port;
    ViInt32 protocol_toggle;
    ViByte serial_address[100];
    
    switch (event)
        {
        case EVENT_COMMIT:
            
            SetCtrlAttribute (panelHandle[GEN], GEN_UPDATE_GEN, ATTR_DIMMED, 1);
            SetCtrlAttribute (panelHandle[SCOPE], SCOPE_UPDATE_SCOPE, ATTR_DIMMED, 1);
            SetCtrlAttribute (panelHandle[FGEN], FGEN_UPDATE_FGEN, ATTR_DIMMED, 1);
            SetCtrlAttribute (panelHandle[SA], SA_UPDATE_SA, ATTR_DIMMED, 1);
            SetCtrlAttribute (panelHandle[METER], METER_UPDATE_METERS, ATTR_DIMMED, 1);
            SetCtrlAttribute (panelHandle[RECEIVER], RECEIVER_UPDATE_REC, ATTR_DIMMED, 1);
            SetCtrlAttribute (panelHandle[PANEL], PANEL_DEFAULTS_UI, ATTR_DIMMED, 1);
            SetCtrlAttribute (panelHandle[PANEL], PANEL_EXIT, ATTR_DIMMED, 1);
            SetCtrlAttribute (panelHandle[PANEL], PANEL_UPDATE_GO, ATTR_DIMMED, 1);
            SetCtrlAttribute (panelHandle[PANEL], PANEL_UPDATE_UI, ATTR_DIMMED, 1);

			/*
            * Tells user that the UI is communicating with the box 
            */
            SetWaitCursor(1);
            
            GetCtrlVal (panelHandle[CONN], CONN_SERIAL_CONN, &toggle);
            
            if (toggle == 1) {
                SetCtrlVal (panelHandle[CONN], CONN_SERIAL_BUSY, 0);
                
                GetCtrlVal (panelHandle[CONN], CONN_SERIAL_PORT, &port);

                GetCtrlVal (panelHandle[CONN], CONN_connect_type, &protocol_toggle);            

                /*
                * Setup for GPIB communication of the 2975 
                */
                Fmt (serial_address, "%s<ASRL%d",port);
                if (ifr2975_init (serial_address, 0, 0, &handle)<0)
                    return -1;

                SetCtrlVal (panelHandle[CONN], CONN_connect_type, 2);            
                
                SetCtrlVal (panelHandle[CONN], CONN_SERIAL_BUSY, 1);
            } else {
                SetCtrlVal (panelHandle[CONN], CONN_SERIAL_BUSY, 1);
                
                /*
                * Disconnects from the box 
                */
                if ((ifr2975_status = ifr2975_Close_Conn (handle)) < 0){
                    SetCtrlVal (panelHandle[CONN], CONN_SERIAL_CONN, 0);
                    MessagePopup ("IFR 2975 Instrument Driver UI",
                              "Serial Disconnection has failed");
                    
                    /*
                    * Exits UI 
                    */
                    abortApplication();
                }
                
                /*
                * resets serial handle 
                */
                handle = 0;
                
                SetCtrlVal (panelHandle[CONN], CONN_SERIAL_BUSY, 0);
                
                GetCtrlVal (panelHandle[CONN], CONN_connect_type, &protocol_toggle);            
            
            }

            SetCtrlAttribute (panelHandle[GEN], GEN_UPDATE_GEN, ATTR_DIMMED, 0);
            SetCtrlAttribute (panelHandle[SCOPE], SCOPE_UPDATE_SCOPE, ATTR_DIMMED, 0);
            SetCtrlAttribute (panelHandle[FGEN], FGEN_UPDATE_FGEN, ATTR_DIMMED, 0);
            SetCtrlAttribute (panelHandle[SA], SA_UPDATE_SA, ATTR_DIMMED, 0);
            SetCtrlAttribute (panelHandle[METER], METER_UPDATE_METERS, ATTR_DIMMED, 0);
            SetCtrlAttribute (panelHandle[RECEIVER], RECEIVER_UPDATE_REC, ATTR_DIMMED, 0);
            SetCtrlAttribute (panelHandle[PANEL], PANEL_DEFAULTS_UI, ATTR_DIMMED, 0);
            SetCtrlAttribute (panelHandle[PANEL], PANEL_EXIT, ATTR_DIMMED, 0);
            SetCtrlAttribute (panelHandle[PANEL], PANEL_UPDATE_GO, ATTR_DIMMED, 0);
            SetCtrlAttribute (panelHandle[PANEL], PANEL_UPDATE_UI, ATTR_DIMMED, 0);

            /*
            * Tells user that the UI is not longer communicating with the box
            */
            SetWaitCursor(0);
            break;
        case EVENT_RIGHT_CLICK:
            /*
            * Help Message for given widget 
            */
            MessagePopup("HELP: Serial Connect/Disconnect Toggle Switch", "This button connects/disconnects serial from the unit."); 
            
            break;
        }
    return 0;
}

/*****************************************************************************
* Function: GPIB_CONN                                   
* Purpose:  This is the gpib connect/disconnect switch for the UI.          
*                                                       
*****************************************************************************/
int CVICALLBACK GPIB_CONN (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    ViInt32 toggle;
    ViInt32 ifr2975_status;
    ViInt32 num_address = 0;
    ViByte gpib_address[100];
    ViInt32 protocol_toggle;    
    
    switch (event)
        {
        case EVENT_COMMIT:
            SetCtrlAttribute (panelHandle[GEN], GEN_UPDATE_GEN, ATTR_DIMMED, 1);
            SetCtrlAttribute (panelHandle[SCOPE], SCOPE_UPDATE_SCOPE, ATTR_DIMMED, 1);
            SetCtrlAttribute (panelHandle[FGEN], FGEN_UPDATE_FGEN, ATTR_DIMMED, 1);
            SetCtrlAttribute (panelHandle[SA], SA_UPDATE_SA, ATTR_DIMMED, 1);
            SetCtrlAttribute (panelHandle[METER], METER_UPDATE_METERS, ATTR_DIMMED, 1);
            SetCtrlAttribute (panelHandle[RECEIVER], RECEIVER_UPDATE_REC, ATTR_DIMMED, 1);
            SetCtrlAttribute (panelHandle[PANEL], PANEL_DEFAULTS_UI, ATTR_DIMMED, 1);
            SetCtrlAttribute (panelHandle[PANEL], PANEL_EXIT, ATTR_DIMMED, 1);
            SetCtrlAttribute (panelHandle[PANEL], PANEL_UPDATE_GO, ATTR_DIMMED, 1);
            SetCtrlAttribute (panelHandle[PANEL], PANEL_UPDATE_UI, ATTR_DIMMED, 1);

            /*
            * Tells user that the UI is communicating with the box 
            */
            SetWaitCursor(1);
            
            GetCtrlVal (panelHandle[CONN], CONN_GPIB_CONN, &toggle);
            
            if (toggle == 1) {
                SetCtrlVal (panelHandle[CONN], CONN_GPIB_BUSY, 0);

                GetCtrlVal (panelHandle[CONN], CONN_GPIB_ADDR, &num_address);

                /*
                * Sets the Gpib address of the 2975 
                */
                Fmt (gpib_address, "%s<GPIB::%d",num_address);
                
                if (ifr2975_init (gpib_address, 0, 0, &handle)<0)
                    return -1;
                
                GetCtrlVal (panelHandle[CONN], CONN_connect_type, &protocol_toggle);            
            
                SetCtrlVal (panelHandle[CONN], CONN_connect_type, 0);            
            
                SetCtrlVal (panelHandle[CONN], CONN_GPIB_BUSY, 1);
            } else {
                SetCtrlVal (panelHandle[CONN], CONN_GPIB_BUSY, 1);
            
                /*
                * Disconnects from the box 
                */
                if ((ifr2975_status = ifr2975_Close_Conn (handle)) < 0){
                    SetCtrlVal (panelHandle[CONN], CONN_GPIB_CONN, 0);
                    MessagePopup ("IFR 2975 Instrument Driver UI",
                              "GPIB Disconnection has failed");
                    
                    /*
                    * Exits UI 
                    */
                    abortApplication();
                }
                
                /*
                * resets gpib handle 
                */
                handle = 0;
                
                SetCtrlVal (panelHandle[CONN], CONN_GPIB_BUSY, 0);
                
                GetCtrlVal (panelHandle[CONN], CONN_connect_type, &protocol_toggle);            
            
            }

            /*
            * Tells user that the UI is not longer communicating with the box
            */
            SetWaitCursor(0);

            SetCtrlAttribute (panelHandle[GEN], GEN_UPDATE_GEN, ATTR_DIMMED, 0);
            SetCtrlAttribute (panelHandle[SCOPE], SCOPE_UPDATE_SCOPE, ATTR_DIMMED, 0);
            SetCtrlAttribute (panelHandle[FGEN], FGEN_UPDATE_FGEN, ATTR_DIMMED, 0);
            SetCtrlAttribute (panelHandle[SA], SA_UPDATE_SA, ATTR_DIMMED, 0);
            SetCtrlAttribute (panelHandle[METER], METER_UPDATE_METERS, ATTR_DIMMED, 0);
            SetCtrlAttribute (panelHandle[RECEIVER], RECEIVER_UPDATE_REC, ATTR_DIMMED, 0);
            SetCtrlAttribute (panelHandle[PANEL], PANEL_DEFAULTS_UI, ATTR_DIMMED, 0);
            SetCtrlAttribute (panelHandle[PANEL], PANEL_EXIT, ATTR_DIMMED, 0);
            SetCtrlAttribute (panelHandle[PANEL], PANEL_UPDATE_GO, ATTR_DIMMED, 0);
            SetCtrlAttribute (panelHandle[PANEL], PANEL_UPDATE_UI, ATTR_DIMMED, 0);

            break;
        case EVENT_RIGHT_CLICK:
            /*
            * Help Message for given widget 
            */
            MessagePopup("HELP: GPIB Connect/Disconnect Toggle Switch", "This button connects/disconnects GPIB from the unit."); 
            
            break;
        }
    return 0;
}

/*****************************************************************************
* Function: connect_type                                
* Purpose:  This switches between being connected through gpib, eth, and    
*           serial.                                     
*****************************************************************************/
int CVICALLBACK connect_type (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    ViInt32 protocol_toggle;
    
    switch (event)
        {
        case EVENT_COMMIT:
            /*
            * Tells user that the UI is communicating with the box 
            */
            SetWaitCursor(1);
            
            GetCtrlVal (panelHandle[CONN], CONN_connect_type, &protocol_toggle);            
            
            /*
            * Tells user that the UI is not longer communicating with the box
            */
            SetWaitCursor(0);
            break;
        case EVENT_RIGHT_CLICK:
            /*
            * Help Message for given widget 
            */
            MessagePopup("HELP: Connection Type Switch", "This switch specifies which connection is being used."); 
            
            break;
        }
    return 0;
}

/*****************************************************************************
* Function: CLOSE_GEN                                   
* Purpose:  This closes the generator panel.            
*                                                       
*****************************************************************************/
int CVICALLBACK CLOSE_GEN (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event)
        {
        case EVENT_COMMIT:
            
            /*
            * Closes the given panel 
            */
            HidePanel (panelHandle[GEN]);
            
            SetCtrlVal (panelHandle[PANEL], PANEL_GEN_SWITCH, 0);
            
            break;
        case EVENT_RIGHT_CLICK:
            /*
            * Help Message for given widget 
            */
            MessagePopup("HELP: Generator Panel Close Button", "This button closes the Generator panel."); 
            
            break;
        }
    return 0;
}

/*****************************************************************************
* Function: CLOSE_CONN                                  
* Purpose:  This closes the connection panel.           
*                                                       
*****************************************************************************/
int CVICALLBACK CLOSE_CONN (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event)
        {
        case EVENT_COMMIT:
            /*
            * Closes the given panel 
            */
            HidePanel (panelHandle[CONN]);
            
            SetCtrlVal (panelHandle[PANEL], PANEL_CONN_SWITCH, 0);
            
            break;
        case EVENT_RIGHT_CLICK:
            /*
            * Help Message for given widget 
            */
            MessagePopup("HELP: Connection Panel Close Button", "This button closes the Connection panel."); 
            
            break;
        }
    return 0;
}

/*****************************************************************************
* Function: CLOSE_REC                                   
* Purpose:  This closes the receiver panel.             
*                                                       
*****************************************************************************/
int CVICALLBACK CLOSE_REC (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event)
        {
        case EVENT_COMMIT:
            
            /*
            * Closes the given panel 
            */
            HidePanel (panelHandle[RECEIVER]);
            
            SetCtrlVal (panelHandle[PANEL], PANEL_REC_SWITCH, 0);
            break;
        case EVENT_RIGHT_CLICK:
            /*
            * Help Message for given widget 
            */
            MessagePopup("HELP: Receiver Panel Close Button", "This button closes the Receiver panel."); 
            
            break;
        }
    return 0;
}

/*****************************************************************************
* Function: CLOSE_SA                                    
* Purpose:  This is closes the Spectrum Analyzer panel. 
*                                                       
*****************************************************************************/
int CVICALLBACK CLOSE_SA (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event)
        {
        case EVENT_COMMIT:
            
            /*
            * Closes the given panel 
            */
            HidePanel (panelHandle[SA]);
            
            SetCtrlVal (panelHandle[PANEL], PANEL_SA_SWITCH, 0);
            break;
        case EVENT_RIGHT_CLICK:
            /*
            * Help Message for given widget 
            */
            MessagePopup("HELP: Spectrum Analyzer Panel Close Button", "This button closes the Spectrum Analyzer panel."); 
            
            break;
        }
    return 0;
}

/*****************************************************************************
* Function: CLOSE_METER                                 
* Purpose:  This is closes the meter panel.             
*                                                       
*****************************************************************************/
int CVICALLBACK CLOSE_METER (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event)
        {
        case EVENT_COMMIT:
            /*
            * Closes the given panel 
            */
            HidePanel (panelHandle[METER]);
            
            SetCtrlVal (panelHandle[PANEL], PANEL_METER_SWITCH, 0);
            break;
        case EVENT_RIGHT_CLICK:
            /*
            * Help Message for given widget 
            */
            MessagePopup("HELP: Meter Panel Close Button", "This button closes the Meter panel."); 
            
            break;
        }
    return 0;
}

/*****************************************************************************
* Function: CLOSE_SCOPE                                 
* Purpose:  This is closes the scope panel.             
*                                                       
*****************************************************************************/
int CVICALLBACK CLOSE_SCOPE (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event)
        {
        case EVENT_COMMIT:
            /*
            * Closes the given panel 
            */
            HidePanel (panelHandle[SCOPE]);
            
            SetCtrlVal (panelHandle[PANEL], PANEL_SCOPE_SWITCH, 0);
            break;
        case EVENT_RIGHT_CLICK:
            /*
            * Help Message for given widget 
            */
            MessagePopup("HELP: Scope Panel Close Button", "This button closes the Scope panel."); 
            
            break;
        }
    return 0;
}


/*****************************************************************************
* Function: CONN_SWITCH                                 
* Purpose:  This is closes/opens the connection panel.  
*                                                       
*****************************************************************************/
int CVICALLBACK CONN_SWITCH (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    ViInt32 value1;
    
    switch (event)
        {
        case EVENT_COMMIT:
            GetCtrlVal (panelHandle[PANEL], PANEL_CONN_SWITCH, &value1);
            
            if (value1 == 1) {
                /*
                * Displays the given panel 
                */
                DisplayPanel (panelHandle[CONN]);
            } else {
                /*
                * Closes the given panel 
                */
                HidePanel (panelHandle[CONN]);
            }
            
            break;
        case EVENT_RIGHT_CLICK:
            /*
            * Help Message for given widget 
            */
            MessagePopup("HELP: Connection Toggle Switch", "This button open/closes the Connection panel."); 
            
            break;
        }
    return 0;
}

/*****************************************************************************
* Function: FGEN_SWITCH                                 
* Purpose:  This is closes/opens the function generator panel.              
*                                                       
*****************************************************************************/
int CVICALLBACK FGEN_SWITCH (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    ViInt32 value1;
    
    switch (event)
        {
        case EVENT_COMMIT:
            GetCtrlVal (panelHandle[PANEL], PANEL_FGEN_SWITCH, &value1);
            
            if (value1 == 1) {
                /*
                * Displays the given panel 
                */
                DisplayPanel (panelHandle[FGEN]);
            } else {
                /*
                * Closes the given panel 
                */
                HidePanel (panelHandle[FGEN]);
            }
            
            break;
        case EVENT_RIGHT_CLICK:
            /*
            * Help Message for given widget 
            */
            MessagePopup("HELP: Function Generator UI Toggle Switch", "This button open/closes the Function Generator panel."); 
            
            break;
        }
    return 0;
}

/*****************************************************************************
* Function: SA_SWITCH                                   
* Purpose:  This is closes/opens the spectrum analyzer panel.               
*                                                       
*****************************************************************************/
int CVICALLBACK SA_SWITCH (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    ViInt32 value1;
    
    switch (event)
        {
        case EVENT_COMMIT:
            GetCtrlVal (panelHandle[PANEL], PANEL_SA_SWITCH, &value1);
            
            if (value1 == 1) {
                /*
                * Displays the given panel 
                */
                DisplayPanel (panelHandle[SA]);
            } else {
                /*
                * Closes the given panel 
                */
                HidePanel (panelHandle[SA]);
            }
            
            break;
        case EVENT_RIGHT_CLICK:
            /*
            * Help Message for given widget 
            */
            MessagePopup("HELP: Spectrum Analyzer UI Toggle Switch", "This button open/closes the Spectrum Analyzer panel."); 
            
            break;
        }
    return 0;
}

/*****************************************************************************
* Function: SCOPE_SWITCH                                
* Purpose:  This is closes/opens the scope panel.       
*                                                       
*****************************************************************************/
int CVICALLBACK SCOPE_SWITCH (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    ViInt32 value1;
    
    switch (event)
        {
        case EVENT_COMMIT:
            GetCtrlVal (panelHandle[PANEL], PANEL_SCOPE_SWITCH, &value1);
            
            if (value1 == 1) {
                /*
                * Displays the given panel 
                */
                DisplayPanel (panelHandle[SCOPE]);
            } else {
                /*
                * Closes the given panel 
                */
                HidePanel (panelHandle[SCOPE]);
            }
            
            break;
        case EVENT_RIGHT_CLICK:
            /*
            * Help Message for given widget 
            */
            MessagePopup("HELP: Scope UI Toggle Switch", "This button open/closes the Scope panel.");
            
            break;
        }
    return 0;
}

/*****************************************************************************
* Function: GEN_SWITCH                                  
* Purpose:  This is closes/opens the generator panel.   
*                                                       
*****************************************************************************/
int CVICALLBACK GEN_SWITCH (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    ViInt32 value1;
    
    switch (event)
        {
        case EVENT_COMMIT:
            GetCtrlVal (panelHandle[PANEL], PANEL_GEN_SWITCH, &value1);
            
            if (value1 == 1) {
                /*
                * Displays the given panel 
                */
                DisplayPanel (panelHandle[GEN]);
            } else {
                /*
                * Closes the given panel 
                */
                HidePanel (panelHandle[GEN]);
            }
            
            break;
        case EVENT_RIGHT_CLICK:
            /*
            * Help Message for given widget 
            */
            MessagePopup("HELP: Generator UI Toggle Switch", "This button open/closes the Generator panel."); 
            break;
        }
    return 0;
}

/*****************************************************************************
* Function: REC_SWITCH                                  
* Purpose:  This is closes/opens the receiver panel.    
*                                                       
*****************************************************************************/
int CVICALLBACK REC_SWITCH (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    ViInt32 value1;
    
    switch (event)
        {
        case EVENT_COMMIT:
            GetCtrlVal (panelHandle[PANEL], PANEL_REC_SWITCH, &value1);
            
            if (value1 == 1) {
                /*
                * Displays the given panel 
                */
                DisplayPanel (panelHandle[RECEIVER]);
            } else {
                /*
                * Closes the given panel 
                */
                HidePanel (panelHandle[RECEIVER]);
            }    
            
            break;
        case EVENT_RIGHT_CLICK:
            /*
            * Help Message for given widget 
            */
            MessagePopup("HELP: Receiver UI Toggle Switch", "This button open/closes the Receiver panel."); 
            
            break;
        }
    return 0;
}

/*****************************************************************************
* Function: METER_SWITCH                                
* Purpose:  This is closes/opens the meter panel.       
*                                                       
*****************************************************************************/
int CVICALLBACK METER_SWITCH (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    ViInt32 value1;
    
    switch (event)
        {
        case EVENT_COMMIT:
            GetCtrlVal (panelHandle[PANEL], PANEL_METER_SWITCH, &value1);
            
            if (value1 == 1) {
                /*
                * Displays the given panel 
                */
                DisplayPanel (panelHandle[METER]);
            } else {
                /*
                * Closes the given panel 
                */
                HidePanel (panelHandle[METER]);
            }
            
            break;
        case EVENT_RIGHT_CLICK:
            /*
            * Help Message for given widget 
            */
            MessagePopup("HELP: Meters UI Toggle Switch", "This button open/closes the Meters panel."); 
            
            break;
        }
    return 0;
}


/*****************************************************************************
* Function: CLOSE_FGEN                                  
* Purpose:  This is closes the function generater panel.                    
*                                                       
*****************************************************************************/
int CVICALLBACK CLOSE_FGEN (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event)
        {
        case EVENT_COMMIT:
            /*
            * Closes the given panel 
            */
            HidePanel (panelHandle[FGEN]);
            
            SetCtrlVal (panelHandle[PANEL], PANEL_FGEN_SWITCH, 0);

            break;
        case EVENT_RIGHT_CLICK:
            /*
            * Help Message for given widget 
            */
            MessagePopup("HELP: Function Generator Panel Close Button", "This button closes the Function Generator panel."); 
            
            break;
        }
    return 0;
}


/*****************************************************************************
* Function: fgen_update                                 
* Purpose:  This is updates the function generator panel.                   
*                                                       
*****************************************************************************/
int fgen_update (void) {
    /*
    * Fgen variables 
    */
    ViInt32 fgen_channel_value;
    ViInt16 fgen_mode;
    short fgen_shape_value;
    ViReal64 fgen_level_value;
    ViReal64 fgen_freq_value;
    ViInt32 fgen_update_state;
    ViInt16 status;
    
    GetCtrlVal (panelHandle[PANEL], PANEL_FGEN_SWITCH,
                &fgen_update_state);
                
    if (fgen_update_state == 1) {
        
        if (status = ifr2975_Set_Screen_Mode (handle, 7)<0) {
            /*
            * Displays Driver Error 
            */
            MessagePopup("Driver Error", "ifr2975_Set_Screen_Mode failed !"); 
            
            /*
            * Exits UI 
            */
            abortApplication();
        }
        
        /*
        * Lets User interact with UI 
        */
        /*
        * It allow the user to turn off the continuous update function 
        */
        ProcessSystemEvents ();
        
        if (status = ifr2975_Get_Freq_FGEN (handle, 1, &fgen_freq_value)<0) {
            /*
            * Displays Driver Error 
            */
            MessagePopup("Driver Error", "ifr2975_Get_Freq_FGEN failed !"); 
            
            /*
            * Exits UI 
            */
            abortApplication();
        }
        
        SetCtrlVal(panelHandle[FGEN], FGEN_fgen_freq_1, fgen_freq_value);
        
        /*
        * Lets User interact with UI 
        */
        /*
        * It allow the user to turn off the continuous update function 
        */
        ProcessSystemEvents ();
        
        if (status = ifr2975_Get_Lev_FGEN (handle, 1, &fgen_level_value)<0) {
            /*
            * Displays Driver Error 
            */
            MessagePopup("Driver Error", "ifr2975_Get_Lev_FGEN failed !"); 
            
            /*
            * Exits UI 
            */
            abortApplication();
        }
        
        SetCtrlVal(panelHandle[FGEN], FGEN_fgen_level_1, fgen_level_value);
        
        /*
        * Lets User interact with UI 
        */
        /*
        * It allow the user to turn off the continuous update function 
        */
        ProcessSystemEvents ();
        
        if (status = ifr2975_Get_Shape_FGEN (handle, 1, &fgen_shape_value)<0) {
            /*
            * Displays Driver Error 
            */
            MessagePopup("Driver Error", "ifr2975_Get_Shape_FGEN failed !"); 
            
            /*
            * Exits UI 
            */
            abortApplication();
        }
        
        SetCtrlVal(panelHandle[FGEN], FGEN_fgen_shape_1, fgen_shape_value);
        
        /*
        * Lets User interact with UI 
        */
        /*
        * It allow the user to turn off the continuous update function 
        */
        ProcessSystemEvents ();
        
        if (status = ifr2975_Get_Mode_FGEN (handle, 1, &fgen_mode)<0) {
            /*
            * Displays Driver Error 
            */
            MessagePopup("Driver Error", "ifr2975_Get_Mode_FGEN failed !"); 
            
            /*
            * Exits UI 
            */
            abortApplication();
        }
        
        SetCtrlVal(panelHandle[FGEN], FGEN_fgen_type_1, fgen_mode);
        
        /*
        * Lets User interact with UI 
        */
        /*
        * It allow the user to turn off the continuous update function 
        */
        ProcessSystemEvents ();
        
        if (fgen_mode==0) {
            SetCtrlAttribute (panelHandle[FGEN], FGEN_fgen_freq_1, ATTR_DIMMED, 1);
            SetCtrlAttribute (panelHandle[FGEN], FGEN_fgen_level_1, ATTR_DIMMED, 1);
            SetCtrlAttribute (panelHandle[FGEN], FGEN_fgen_shape_1, ATTR_DIMMED, 1);
        } else {
            SetCtrlAttribute (panelHandle[FGEN], FGEN_fgen_freq_1, ATTR_DIMMED, 0);
            SetCtrlAttribute (panelHandle[FGEN], FGEN_fgen_level_1, ATTR_DIMMED, 0);
            SetCtrlAttribute (panelHandle[FGEN], FGEN_fgen_shape_1, ATTR_DIMMED, 0);
        }
        
        
        if (status = ifr2975_Get_Freq_FGEN (handle, 2, &fgen_freq_value)<0) {
            /*
            * Displays Driver Error 
            */
            MessagePopup("Driver Error", "ifr2975_Get_Freq_FGEN failed !"); 
            
            /*
            * Exits UI 
            */
            abortApplication();
        }
        
        SetCtrlVal(panelHandle[FGEN], FGEN_fgen_freq_2, fgen_freq_value);
        
        /*
        * Lets User interact with UI 
        */
        /*
        * It allow the user to turn off the continuous update function 
        */
        ProcessSystemEvents ();
        
        if (status = ifr2975_Get_Lev_FGEN (handle, 2, &fgen_level_value)<0) {
            /*
            * Displays Driver Error 
            */
            MessagePopup("Driver Error", "ifr2975_Get_Lev_FGEN failed !"); 
            
            /*
            * Exits UI 
            */
            abortApplication();
        }
        
        SetCtrlVal(panelHandle[FGEN], FGEN_fgen_level_2, fgen_level_value);
        
        /*
        * Lets User interact with UI 
        */
        /*
        * It allow the user to turn off the continuous update function 
        */
        ProcessSystemEvents ();
        
        if (status = ifr2975_Get_Shape_FGEN (handle, 2, &fgen_shape_value)<0) {
            /*
            * Displays Driver Error 
            */
            MessagePopup("Driver Error", "ifr2975_Get_Shape_FGEN failed !"); 
            
            /*
            * Exits UI 
            */
            abortApplication();
        }
        
        SetCtrlVal(panelHandle[FGEN], FGEN_fgen_shape_2, fgen_shape_value);
        
        /*
        * Lets User interact with UI 
        */
        /*
        * It allow the user to turn off the continuous update function 
        */
        ProcessSystemEvents ();
        
        if (status = ifr2975_Get_Mode_FGEN (handle, 2, &fgen_mode)<0) {
            /*
            * Displays Driver Error 
            */
            MessagePopup("Driver Error", "ifr2975_Get_Mode_FGEN failed !"); 
            
            /*
            * Exits UI 
            */
            abortApplication();
        }
        
        SetCtrlVal(panelHandle[FGEN], FGEN_fgen_type_2, fgen_mode);
        
        if (fgen_mode==0) {
            SetCtrlAttribute (panelHandle[FGEN], FGEN_fgen_freq_2, ATTR_DIMMED, 1);
            SetCtrlAttribute (panelHandle[FGEN], FGEN_fgen_level_2, ATTR_DIMMED, 1);
            SetCtrlAttribute (panelHandle[FGEN], FGEN_fgen_shape_2, ATTR_DIMMED, 1);
        } else {
            SetCtrlAttribute (panelHandle[FGEN], FGEN_fgen_freq_2, ATTR_DIMMED, 0);
            SetCtrlAttribute (panelHandle[FGEN], FGEN_fgen_level_2, ATTR_DIMMED, 0);
            SetCtrlAttribute (panelHandle[FGEN], FGEN_fgen_shape_2, ATTR_DIMMED, 0);
        }
    }
    
    return 0;
}


/*****************************************************************************
* Function: sa_update                                   
* Purpose:  This is updates the specturm analyzer panel.                    
*                                                       
*****************************************************************************/
int sa_update (void) {
    int div1;
    ViReal64 tos;
    ViReal64 bos;

    /*
    * SA Values 
    */
    ViInt32 length_sa_trace;
    ViReal64 sa_trace_data[10000];
        ViReal64 *sa_trace_data_ptr = &sa_trace_data[0];
    ViInt32 sa_sweep_speed;
    ViInt32 sa_on;
    ViInt32 *sa_on_ptr = &sa_on;
    ViInt16 status;
    ViReal64 top_screen;
    ViReal64 sweep_speed;
    ViReal64 span;
    ViInt16 vert_scale;

    GetCtrlVal(panelHandle[PANEL], PANEL_SA_SWITCH, sa_on_ptr);

    if (sa_on == 1) {
    
    
        if (status = ifr2975_Get_Vert_Scale_SA (handle, &vert_scale)<0) {
            /*
            * Displays Driver Error 
            */
            MessagePopup("Driver Error", "ifr2975_Get_Vert_Scale_SA failed !"); 
            
            /*
            * Exits UI 
            */
            abortApplication();
        }
        SetCtrlVal (panelHandle[SA], SA_vert_scale, vert_scale);
        
        /*
        * Lets User interact with UI 
        */
        /*
        * It allow the user to turn off the continuous update function 
        */
        ProcessSystemEvents ();
        
        if (status = ifr2975_Set_Screen_Mode (handle, 5)<0) {
            /*
            * Displays Driver Error 
            */
            MessagePopup("Driver Error", "ifr2975_Set_Screen_Mode failed !"); 
            
            /*
            * Exits UI 
            */
            abortApplication();
        }
        
        /*
        * Lets User interact with UI 
        */
        /*
        * It allow the user to turn off the continuous update function 
        */
        ProcessSystemEvents ();
        
        if (status = ifr2975_Get_Sweep_SA (handle, &sweep_speed)<0) {
            /*
            * Displays Driver Error 
            */
            MessagePopup("Driver Error", "ifr2975_Get_Sweep_SA failed !"); 
            
            /*
            * Exits UI 
            */
            abortApplication();
        }
        
        SetCtrlVal(panelHandle[SA], SA_sweep, sweep_speed);

        /*
        * Lets User interact with UI 
        */
        /*
        * It allow the user to turn off the continuous update function 
        */
        ProcessSystemEvents ();
        
        if (status = ifr2975_Get_TOS_SA (handle, &top_screen)<0) {
            /*
            * Displays Driver Error 
            */
            MessagePopup("Driver Error", "ifr2975_Get_TOS_SA failed !"); 
            
            /*
            * Exits UI 
            */
            abortApplication();
        }
        
        SetCtrlVal(panelHandle[SA], SA_TOP_Screen, top_screen);
        
        /*
        * Lets User interact with UI 
        */
        /*
        * It allow the user to turn off the continuous update function 
        */
        ProcessSystemEvents ();
        
        if (status = ifr2975_Get_Trace_SA (handle, &length_sa_trace, sa_trace_data_ptr)<0) {
            /*
            * Displays Driver Error 
            */
            MessagePopup("Driver Error", "ifr2975_Get_Trace_SA failed !"); 
            
            /*
            * Exits UI 
            */
            abortApplication();
        }
        
        GetCtrlVal (panelHandle[SA], SA_vert_scale, &div1);
        GetCtrlVal (panelHandle[SA], SA_TOP_Screen, &tos);
            
        bos=tos-8*div1;
            
        SetAxisRange (panelHandle[SA], SA_sa_trace, VAL_NO_CHANGE,-1.0, 1.0, VAL_MANUAL, bos, tos);
        
        if (!(length_sa_trace == 0)){
            DeleteGraphPlot(panelHandle[SA], SA_sa_trace, -1, 0);
            PlotY (panelHandle[SA], SA_sa_trace, sa_trace_data_ptr,
                   length_sa_trace, VAL_DOUBLE, VAL_THIN_LINE, VAL_SMALL_X,
                   VAL_SOLID, 20, VAL_WHITE);
        }
        
        /*
        * Lets User interact with UI 
        */
        /*
        * It allow the user to turn off the continuous update function 
        */
        ProcessSystemEvents ();
        
    }

    return 0;
}



/*****************************************************************************
* Function: scope_update                                
* Purpose:  This is updates the scope panel.            
*                                                       
*****************************************************************************/
int scope_update (void) {
    ViReal64 scope_sweep_speed;
    ViInt32 scope_channel_selection;
    ViReal64 scope_sweep_speed_conversion;
    ViInt32 scope_on;
    ViInt16 status;

    ViInt16 value_scope_v_ref_ch1;
    ViInt16 value_scope_v_div_ch1;
    
    ViInt16 value_scope_v_ref_ch2;
    ViInt16 value_scope_v_div_ch2;
    
    ViInt16 value_scope_v_ref_ext;
    ViInt16 value_scope_v_div_ext;
    
    ViInt32 ch1;
    ViInt32 ch2;
    ViInt32 ext;
    
    ViReal64 div1;
    ViReal64 div2;
    ViReal64 value1;
    ViReal64  value2;
    
    /*
    * Scope Values 
    */
    ViInt32 length_scope_trace;
    ViReal64 scope_trace_data[10000];
        ViReal64 *scope_trace_data_ptr = &scope_trace_data[0];
    
    ViInt16 value_scope_trigger;
    ViInt16 value_scope_source;
    ViReal64 value_scope_level_v;
    ViInt16 value_scope_slope;
    ViReal64 value_scope_h_pos;

    GetCtrlVal(panelHandle[PANEL], PANEL_SCOPE_SWITCH, &scope_on);
    
    if (scope_on == 1) {
        /*
        *   CHANGE SCREEN   
        */
        if (status = ifr2975_Set_Screen_Mode (handle, 4)<0) {
            /*
            * Displays Driver Error 
            */
            MessagePopup("Driver Error", "ifr2975_Set_Screen_Mode failed !"); 
            
            /*
            * Exits UI 
            */
            abortApplication();
        }
        
        /*
        * Lets User interact with UI 
        */
        /*
        * It allow the user to turn off the continuous update function 
        */
        ProcessSystemEvents ();
        
        if (status = ifr2975_Get_Sweep_OSCOPE (handle, &scope_sweep_speed)<0) {
            /*
            * Displays Driver Error 
            */
            MessagePopup("Driver Error", "ifr2975_Get_Sweep_OSCOPE failed !"); 
            
            /*
            * Exits UI 
            */
            abortApplication();
        }
        SetCtrlVal (panelHandle[SCOPE], SCOPE_scope_sweep, scope_sweep_speed);
        
        /*
        * Lets User interact with UI 
        */
        /*
        * It allow the user to turn off the continuous update function 
        */
        ProcessSystemEvents ();
        
        if (status = ifr2975_Get_Trig_Type_OSCOPE (handle, &value_scope_trigger)<0) {
            /*
            * Displays Driver Error 
            */
            MessagePopup("Driver Error", "ifr2975_Get_Trig_Type_OSCOPE failed !"); 
            
            /*
            * Exits UI 
            */
            abortApplication();
        }
        SetCtrlVal (panelHandle[SCOPE], SCOPE_scope_trigger, value_scope_trigger);
        
        /*
        * Lets User interact with UI 
        */
        /*
        * It allow the user to turn off the continuous update function 
        */
        ProcessSystemEvents ();
        
        if (status = ifr2975_Get_Trig_Source_OSCOPE (handle, &value_scope_source)<0) {
            /*
            * Displays Driver Error 
            */
            MessagePopup("Driver Error", "ifr2975_Get_Trig_Source_OSCOPE failed !"); 
            
            /*
            * Exits UI 
            */
            abortApplication();
        }
        SetCtrlVal (panelHandle[SCOPE], SCOPE_scope_source, value_scope_source);
        
        /*
        * Lets User interact with UI 
        */
        /*
        * It allow the user to turn off the continuous update function 
        */
        ProcessSystemEvents ();
        
        if (status = ifr2975_Get_Trig_Lev_OSCOPE (handle, &value_scope_level_v)<0) {
            /*
            * Displays Driver Error 
            */
            MessagePopup("Driver Error", "ifr2975_Get_Vert_Scale_OSCOPE failed !"); 
            
            /*
            * Exits UI 
            */
            abortApplication();
        }
        SetCtrlVal (panelHandle[SCOPE], SCOPE_scope_level_v, value_scope_level_v);
        
        /*
        * Lets User interact with UI 
        */
        /*
        * It allow the user to turn off the continuous update function 
        */
        ProcessSystemEvents ();
        
        if (status = ifr2975_Get_Trig_Pol_OSCOPE (handle, &value_scope_slope)<0) {
            /*
            * Displays Driver Error 
            */
            MessagePopup("Driver Error", "ifr2975_Get_Trig_Pol_OSCOPE failed !"); 
            
            /*
            * Exits UI 
            */
            abortApplication();
        }
        SetCtrlVal (panelHandle[SCOPE], SCOPE_scope_slope, value_scope_slope);
        
        /*
        * Lets User interact with UI 
        */
        /*
        * It allow the user to turn off the continuous update function 
        */
        ProcessSystemEvents ();
        
        if (status = ifr2975_Get_Horiz_Pos_OSCOPE (handle, &value_scope_h_pos)<0) {
            /*
            * Displays Driver Error 
            */
            MessagePopup("Driver Error", "ifr2975_Get_Horiz_Pos_OSCOPE failed !"); 
            
            /*
            * Exits UI 
            */
            abortApplication();
        }
        SetCtrlVal (panelHandle[SCOPE], SCOPE_scope_h_pos, value_scope_h_pos);
        
        DeleteGraphPlot(panelHandle[SCOPE], SCOPE_scope_trace, -1, 0);
        
        GetCtrlVal(panelHandle[SCOPE], SCOPE_scope_ch1, &ch1);

        GetCtrlVal (panelHandle[SCOPE], SCOPE_scope_div_cvi, &div1);

        /*
        * Lets User interact with UI 
        */
        /*
        * It allow the user to turn off the continuous update function 
        */
        ProcessSystemEvents ();
        
        value1=div1*10/2;
        value2=-1*div1*10/2;
        
        if (ch1 == 1) {
        
            /*
            * Graph Scope 
            */
            if (status = ifr2975_Get_Trace_OSCOPE (handle, 1, &length_scope_trace, scope_trace_data_ptr)<0) {
                /*
                * Displays Driver Error 
                */
                MessagePopup("Driver Error", "ifr2975_Get_Trace_OSCOPE failed ch 1!"); 
                
	            /*
	            * Exits UI 
	            */
	            abortApplication();
            }

            /*
            * Lets User interact with UI 
            */
            /*
            * It allow the user to turn off the continuous update function 
            */
            ProcessSystemEvents ();
        
            SetAxisRange (panelHandle[SCOPE], SCOPE_scope_trace, VAL_NO_CHANGE,-1.0, 1.0, VAL_MANUAL, value2, value1);
        
            /*
            * Lets User interact with UI 
            */
            /*
            * It allow the user to turn off the continuous update function 
            */
            ProcessSystemEvents ();
        
            if (!(length_scope_trace == 0)){
                PlotY (panelHandle[SCOPE], SCOPE_scope_trace, scope_trace_data_ptr,
                       length_scope_trace, VAL_DOUBLE, VAL_THIN_LINE, VAL_SMALL_X,
                       VAL_SOLID, 20, VAL_WHITE);
            }    
        
            
            /*
            * Lets User interact with UI 
            */
            /*
            * It allow the user to turn off the continuous update function 
            */
            ProcessSystemEvents ();
        
            if (status = ifr2975_Get_Input_Coupling_OSCOPE (handle, 1, &value_scope_v_ref_ch1)<0) {
                /*
                * Displays Driver Error 
                */
                MessagePopup("Driver Error", "ifr2975_Set_Input_Coupling_OSCOPE failed !"); 
                
                /*
                * Exits UI 
                */
                abortApplication();
            }
            SetCtrlVal (panelHandle[SCOPE], SCOPE_scope_v_ref_ch1, value_scope_v_ref_ch1);
            
            /*
            * Lets User interact with UI 
            */
            /*
            * It allow the user to turn off the continuous update function 
            */
            ProcessSystemEvents ();
        
            if (status = ifr2975_Get_Vert_Scale_OSCOPE (handle, 1, &value_scope_v_div_ch1)<0) {
                /*
                * Displays Driver Error 
                */
                MessagePopup("Driver Error", "ifr2975_Set_Vert_Scale_OSCOPE failed !"); 
                
                /*
                * Exits UI 
                */
                abortApplication();
            }
            SetCtrlVal (panelHandle[SCOPE], SCOPE_scope_v_div_ch1, value_scope_v_div_ch1);
        }
        
        GetCtrlVal(panelHandle[SCOPE], SCOPE_scope_ch2, &ch2);

        if (ch2 == 1) {
            /*
            * Graph Scope 
            */
            if (status = ifr2975_Get_Trace_OSCOPE (handle, 2, &length_scope_trace, scope_trace_data_ptr)<0) {
                /*
                * Displays Driver Error 
                */
                MessagePopup("Driver Error", "ifr2975_Get_Trace_OSCOPE ch2 failed !"); 
                
                /*
                * Exits UI 
                */
                abortApplication();
            }
            
            /*
            * Lets User interact with UI 
            */
            /*
            * It allow the user to turn off the continuous update function 
            */
            ProcessSystemEvents ();
        
            SetAxisRange (panelHandle[SCOPE], SCOPE_scope_trace, VAL_NO_CHANGE,-1.0, 1.0, VAL_MANUAL, value2, value1);
            
            if (!(length_scope_trace == 0)){
                PlotY (panelHandle[SCOPE], SCOPE_scope_trace, scope_trace_data_ptr,
                       length_scope_trace, VAL_DOUBLE, VAL_THIN_LINE, VAL_SMALL_X,
                       VAL_SOLID, 20, VAL_YELLOW);
            }    

            /*
            * Lets User interact with UI 
            */
            /*
            * It allow the user to turn off the continuous update function 
            */
            ProcessSystemEvents ();
        
            if (status = ifr2975_Get_Input_Coupling_OSCOPE (handle, 2, &value_scope_v_ref_ch2)<0) {
                /*
                * Displays Driver Error 
                */
                MessagePopup("Driver Error", "ifr2975_Set_Input_Coupling_OSCOPE failed !"); 
                
                /*
                * Exits UI 
                */
                abortApplication();
            }
            SetCtrlVal (panelHandle[SCOPE], SCOPE_scope_v_ref_ch2, value_scope_v_ref_ch2);
            
            /*
            * Lets User interact with UI 
            */
            /*
            * It allow the user to turn off the continuous update function 
            */
            ProcessSystemEvents ();
        
            if (status = ifr2975_Get_Vert_Scale_OSCOPE (handle, 2, &value_scope_v_div_ch2)<0) {
                /*
                * Displays Driver Error 
                */
                MessagePopup("Driver Error", "ifr2975_Set_Vert_Scale_OSCOPE failed !"); 
                
                /*
                * Exits UI 
                */
                abortApplication();
            }
            SetCtrlVal (panelHandle[SCOPE], SCOPE_scope_v_div_ch2, value_scope_v_div_ch2);
        }
        
    }
    return 0;
}

/*****************************************************************************
* Function: generator_update                            
* Purpose:  This is updates the generator panel.        
*                                                       
*****************************************************************************/
int generator_update (void) {
    /*
    * Generator Values 
    */
    ViReal64 generator_freq;
    ViReal64 generator_level;
    ViInt16 generator_RF;
    ViInt16 generator_mod_type;
    ViInt16 generator_output;
    ViInt16 status;
    ViInt32 generator_update_state;
    ViReal64 value_mod_freq_1;
    ViReal64 value_mod_freq_2;
    ViReal64 value_mod_level_1;
    ViReal64 value_mod_level_2;
    ViInt16 value_mod_type_1;
    ViInt16 value_mod_type_2;
    ViInt16 value_mod_shape_1;
    ViInt16 value_mod_shape_2;
    
    GetCtrlVal (panelHandle[PANEL], PANEL_GEN_SWITCH,
                &generator_update_state);
                
    if (generator_update_state == 1) {
    
        if (status = ifr2975_Set_Screen_Mode (handle, 1)<0) {
            /*
            * Displays Driver Error 
            */
            MessagePopup("Driver Error", "ifr2975_Set_Screen_Mode failed !"); 
            
            /*
            * Exits UI 
            */
            abortApplication();
        }
        
        /*
        * Lets User interact with UI 
        */
        /*
        * It allow the user to turn off the continuous update function 
        */
        ProcessSystemEvents ();
        
        if (status = ifr2975_Get_Freq_GEN (handle, &generator_freq)<0) {
            /*
            * Displays Driver Error 
            */
            MessagePopup("Driver Error", "ifr2975_Get_Freq_GEN failed !"); 
            
            /*
            * Exits UI 
            */
            abortApplication();
        }
        
        SetCtrlVal(panelHandle[GEN], GEN_GEN_FREQ, generator_freq);
        
        /*
        * Lets User interact with UI 
        */
        /*
        * It allow the user to turn off the continuous update function 
        */
        ProcessSystemEvents ();
        
        if (status = ifr2975_Get_Level_GEN (handle, &generator_level)<0) {
            /*
            * Displays Driver Error 
            */
            MessagePopup("Driver Error", "ifr2975_Get_Level_GEN failed !"); 
            
            /*
            * Exits UI 
            */
            abortApplication();
        }
        
        SetCtrlVal(panelHandle[GEN], GEN_Generator_level, generator_level);
        
        /*
        * Lets User interact with UI 
        */
        /*
        * It allow the user to turn off the continuous update function 
        */
        ProcessSystemEvents ();
        
        if (status = ifr2975_Get_RFOut_GEN (handle, &generator_output)<0) {
            /*
            * Displays Driver Error 
            */
            MessagePopup("Driver Error", "ifr2975_Get_RFOut_GEN failed !"); 
            
            /*
            * Exits UI 
            */
            abortApplication();
        }
        
        SetCtrlVal(panelHandle[GEN], GEN_Generator_Output, generator_output);
        
        /*
        * Lets User interact with UI 
        */
        /*
        * It allow the user to turn off the continuous update function 
        */
        ProcessSystemEvents ();

        if (status = ifr2975_Get_PTT_State_GEN (handle, &generator_RF)<0) {
            /*
            * Displays Driver Error 
            */
            MessagePopup("Driver Error", "ifr2975_Get_PTT_GEN failed !"); 
            
            /*
            * Exits UI 
            */
            abortApplication();
        }
        
        SetCtrlVal(panelHandle[GEN], GEN_Generator_RF, generator_RF);
        
        /*
        * Lets User interact with UI 
        */
        /*
        * It allow the user to turn off the continuous update function 
        */
        ProcessSystemEvents ();
        
        if (status = ifr2975_Get_Mod_GEN (handle, &generator_mod_type)<0) {
            /*
            * Displays Driver Error 
            */
            MessagePopup("Driver Error", "ifr2975_Get_Mod_GEN failed !"); 
            
            /*
            * Exits UI 
            */
            abortApplication();
        }
        
        SetCtrlVal(panelHandle[GEN], GEN_Mod_Type, generator_mod_type);
        
        /*
        * Lets User interact with UI 
        */
        /*
        * It allow the user to turn off the continuous update function 
        */
        ProcessSystemEvents ();
        
        /*
        * Modulation 1 
        */
        if (status = ifr2975_Get_Freq_FGEN (handle, 3, &value_mod_freq_1)<0) {
            /*
            * Displays Driver Error 
            */
            MessagePopup("Driver Error", "ifr2975_Get_Freq_FGEN failed !"); 
            
            /*
            * Exits UI 
            */
            abortApplication();
        }
        
        SetCtrlVal(panelHandle[GEN], GEN_mod_freq_1, value_mod_freq_1);
        
        /*
        * Lets User interact with UI 
        */
        /*
        * It allow the user to turn off the continuous update function 
        */
        ProcessSystemEvents ();
        
        if (status = ifr2975_Get_Per_FGEN (handle, 3, &value_mod_level_1)<0) {
            /*
            * Displays Driver Error 
            */
            MessagePopup("Driver Error", "ifr2975_Get_Per_FGEN failed !"); 
            
            /*
            * Exits UI 
            */
            abortApplication();
        }
        value_mod_level_1=value_mod_level_1/2; 
        
        SetCtrlVal(panelHandle[GEN], GEN_mod_level_1, value_mod_level_1);
        
        /*
        * Lets User interact with UI 
        */
        /*
        * It allow the user to turn off the continuous update function 
        */
        ProcessSystemEvents ();
        
        if (status = ifr2975_Get_Mode_FGEN (handle, 3, &value_mod_type_1)<0) {
            /*
            * Displays Driver Error 
            */
            MessagePopup("Driver Error", "ifr2975_Get_Mode_FGEN failed !"); 
            
            /*
            * Exits UI 
            */
            abortApplication();
        }
        
        SetCtrlVal(panelHandle[GEN], GEN_mod_type_1, value_mod_type_1);
        
        /*
        * Lets User interact with UI 
        */
        /*
        * It allow the user to turn off the continuous update function 
        */
        ProcessSystemEvents ();
        
        if (status = ifr2975_Get_Shape_FGEN (handle, 3, &value_mod_shape_1)<0) {
            /*
            * Displays Driver Error 
            */
            MessagePopup("Driver Error", "ifr2975_Get_Shape_FGEN failed !"); 
            
            /*
            * Exits UI 
            */
            abortApplication();
        }
        
        SetCtrlVal(panelHandle[GEN], GEN_mod_shape_1, value_mod_shape_1);
        
        if (value_mod_type_1==0) {
            SetCtrlAttribute (panelHandle[GEN], GEN_mod_freq_1, ATTR_DIMMED, 1);
            SetCtrlAttribute (panelHandle[GEN], GEN_mod_level_1, ATTR_DIMMED, 1);
            SetCtrlAttribute (panelHandle[GEN], GEN_mod_shape_1, ATTR_DIMMED, 1);
        } else {
            SetCtrlAttribute (panelHandle[GEN], GEN_mod_freq_1, ATTR_DIMMED, 0);
            SetCtrlAttribute (panelHandle[GEN], GEN_mod_level_1, ATTR_DIMMED, 0);
            SetCtrlAttribute (panelHandle[GEN], GEN_mod_shape_1, ATTR_DIMMED, 0);
        }
        
        /*
        * Modulation 2 
        */
        if (status = ifr2975_Get_Freq_FGEN (handle, 4, &value_mod_freq_2)<0) {
            /*
            * Displays Driver Error 
            */
            MessagePopup("Driver Error", "ifr2975_Get_Freq_FGEN failed !"); 
            
            /*
            * Exits UI 
            */
            abortApplication();
        }
        
        SetCtrlVal(panelHandle[GEN], GEN_mod_freq_2, value_mod_freq_2);
        
        /*
        * Lets User interact with UI 
        */
        /*
        * It allow the user to turn off the continuous update function 
        */
        ProcessSystemEvents ();
        
        if (status = ifr2975_Get_Per_FGEN (handle, 4, &value_mod_level_2)<0) {
            /*
            * Displays Driver Error 
            */
            MessagePopup("Driver Error", "ifr2975_Get_Per_FGEN failed !"); 
            
            /*
            * Exits UI 
            */
            abortApplication();
        }
        value_mod_level_2=value_mod_level_2/2; 
        
        SetCtrlVal(panelHandle[GEN], GEN_mod_level_2, value_mod_level_2);
        
        /*
        * Lets User interact with UI 
        */
        /*
        * It allow the user to turn off the continuous update function 
        */
        ProcessSystemEvents ();
        
        if (status = ifr2975_Get_Mode_FGEN (handle, 4, &value_mod_type_2)<0) {
            /*
            * Displays Driver Error 
            */
            MessagePopup("Driver Error", "ifr2975_Get_Mode_FGEN failed !"); 
            
            /*
            * Exits UI 
            */
            abortApplication();
        }
        
        SetCtrlVal(panelHandle[GEN], GEN_mod_type_2, value_mod_type_2);
        
        /*
        * Lets User interact with UI 
        */
        /*
        * It allow the user to turn off the continuous update function 
        */
        ProcessSystemEvents ();
                                                             
        if (status = ifr2975_Get_Shape_FGEN (handle, 4, &value_mod_shape_2)<0) {
            /*
            * Displays Driver Error 
            */
            MessagePopup("Driver Error", "ifr2975_Get_Shape_FGEN failed !"); 
            abortApplication();
        }
        
        SetCtrlVal(panelHandle[GEN], GEN_mod_shape_2, value_mod_shape_2);
        
        if (value_mod_type_2==0) {
            SetCtrlAttribute (panelHandle[GEN], GEN_mod_freq_2, ATTR_DIMMED, 1);
            SetCtrlAttribute (panelHandle[GEN], GEN_mod_level_2, ATTR_DIMMED, 1);
            SetCtrlAttribute (panelHandle[GEN], GEN_mod_shape_2, ATTR_DIMMED, 1);
        } else {
            SetCtrlAttribute (panelHandle[GEN], GEN_mod_freq_2, ATTR_DIMMED, 0);
            SetCtrlAttribute (panelHandle[GEN], GEN_mod_level_2, ATTR_DIMMED, 0);
            SetCtrlAttribute (panelHandle[GEN], GEN_mod_shape_2, ATTR_DIMMED, 0);
        }
        
    }

    return 0;
}

/*****************************************************************************
* Function: receiver_update                             
* Purpose:  This is updates the receiver panel.         
*                                                       
*****************************************************************************/
int receiver_update (void) {
     ViInt32 receiver_update_state;
    ViInt16 status;
    /*
    * Receiver Values 
    */
    ViReal64 receiver_freq;
    ViInt16 receiver_input;
    ViInt16 receiver_demod;
    ViInt16 receiver_atten;
    ViInt16 receiver_if;
    ViInt16 receiver_filter;
    
    GetCtrlVal (panelHandle[PANEL], PANEL_REC_SWITCH,
                &receiver_update_state);
    if (receiver_update_state == 1) {
        if (status = ifr2975_Set_Screen_Mode (handle, 2)<0) {
            /*
            * Displays Driver Error 
            */
            MessagePopup("Driver Error", "ifr2975_Set_Screen_Mode failed !"); 
            
            /*
            * Exits UI 
            */
            abortApplication();
        }
        
        /*
        * Lets User interact with UI 
        */
        /*
        * It allow the user to turn off the continuous update function 
        */
        ProcessSystemEvents ();
        
        if (status = ifr2975_Get_Freq_REC (handle, &receiver_freq)<0) {
            /*
            * Displays Driver Error 
            */
            MessagePopup("Driver Error", "ifr2975_Get_Freq_REC failed !"); 
            
            /*
            * Exits UI 
            */
            abortApplication();
        }
        
        SetCtrlVal(panelHandle[RECEIVER], RECEIVER_Receiver_freq, receiver_freq);
        
        /*
        * Lets User interact with UI 
        */
        /*
        * It allow the user to turn off the continuous update function 
        */
        ProcessSystemEvents ();
        
        if (status = ifr2975_Get_RFIn_REC (handle, &receiver_input)<0) {
            /*
            * Displays Driver Error 
            */
            MessagePopup("Driver Error", "ifr2975_Get_RFIn_REC failed !"); 
            
            /*
            * Exits UI 
            */
            abortApplication();
        }
        
        SetCtrlVal(panelHandle[RECEIVER], RECEIVER_Receiver_Input, receiver_input);
        
        /*
        * Lets User interact with UI 
        */
        /*
        * It allow the user to turn off the continuous update function 
        */
        ProcessSystemEvents ();
        
        if (status = ifr2975_Get_Attn_REC (handle, &receiver_atten)<0) {
            /*
            * Displays Driver Error 
            */
            MessagePopup("Driver Error", "ifr2975_Get_Attn_REC failed !"); 
            
            /*
            * Exits UI 
            */
            abortApplication();
        }
        
        SetCtrlVal(panelHandle[RECEIVER], RECEIVER_Receiver_Atten, receiver_atten);
        
        /*
        * Lets User interact with UI 
        */
        /*
        * It allow the user to turn off the continuous update function 
        */
        ProcessSystemEvents ();
        
        if (status = ifr2975_Get_Demod_REC (handle, &receiver_demod)<0) {
            /*
            * Displays Driver Error 
            */
            MessagePopup("Driver Error", "ifr2975_Get_Demod_REC failed !"); 
            
            /*
            * Exits UI 
            */
            abortApplication();
        }
        
        SetCtrlVal(panelHandle[RECEIVER], RECEIVER_Receiver_Demod, receiver_demod);
        
        if (receiver_demod==0) {
                SetCtrlAttribute (panelHandle[RECEIVER], RECEIVER_Receiver_FILTER, ATTR_DIMMED, 0);
        } else {
                SetCtrlAttribute (panelHandle[RECEIVER], RECEIVER_Receiver_FILTER, ATTR_DIMMED, 1);
        }
        
        if (status = ifr2975_Get_Filter_REC (handle, &receiver_filter)<0) {
            /*
            * Displays Driver Error 
            */
            MessagePopup("Driver Error", "ifr2975_Get_Filter_REC failed !"); 
            
            /*
            * Exits UI 
            */
            abortApplication();
        }
        
        SetCtrlVal(panelHandle[RECEIVER], RECEIVER_Receiver_FILTER, receiver_filter);
        
        /*
        * Lets User interact with UI 
        */
        /*
        * It allow the user to turn off the continuous update function 
        */
        ProcessSystemEvents ();
        
        if (status = ifr2975_Get_Demod_REC (handle, &receiver_if)<0) {
            /*
            * Displays Driver Error 
            */
            MessagePopup("Driver Error", "ifr2975_Get_Demod_REC failed !"); 
            
            /*
            * Exits UI 
            */
            abortApplication();
        }
        
        SetCtrlVal(panelHandle[RECEIVER], RECEIVER_Receiver_IF, receiver_if);
    }


    return 0;
}

/*****************************************************************************
* Function: meter_update                                
* Purpose:  This is updates the meter panel.            
*                                                       
*****************************************************************************/
int meter_update (void) {

    /*
    * Meters 
    */
    ViReal64 meas_sinad;
    ViReal64 meas_afcounter;
    ViReal64 meas_distortion;
    ViReal64 meas_dvm;
    ViReal64 meas_power;
    ViReal64 meas_dev;
    ViReal64 meas_rferror;
    ViReal64 meas_rssi;
    ViInt32 meter_on_off;
    ViInt16 status;

    /*
    * Meter Update 
    */
    GetCtrlVal (panelHandle[PANEL], PANEL_METER_SWITCH,
                &meter_on_off);
    if (meter_on_off == 1) {
        if (status = ifr2975_Meas_SINAD_SINAD (handle, &meas_sinad)<0) {
            /*
            * Displays Driver Error 
            */
            MessagePopup("Driver Error", "ifr2975_Meas_SINAD_SINAD failed !"); 
            
            /*
            * Exits UI 
            */
            abortApplication();
        }
        
        SetCtrlVal(panelHandle[METER], METER_sinad_meter, meas_sinad);
        
        /*
        * Lets User interact with UI 
        */
        /*
        * It allow the user to turn off the continuous update function 
        */
        ProcessSystemEvents ();
        
        if (status = ifr2975_Meas_Freq_AFC (handle, &meas_afcounter)<0) {
            /*
            * Displays Driver Error 
            */
            MessagePopup("Driver Error", "ifr2975_Meas_Freq_AFC failed !"); 
            
            /*
            * Exits UI 
            */
            abortApplication();
        }
        
        SetCtrlVal(panelHandle[METER], METER_afcounter_meter, meas_afcounter);
        
        /*
        * Lets User interact with UI 
        */
        /*
        * It allow the user to turn off the continuous update function 
        */
        ProcessSystemEvents ();
        
        if (status = ifr2975_Meas_Dist_DIST (handle, &meas_distortion)<0) {
            /*
            * Displays Driver Error 
            */
            MessagePopup("Driver Error", "ifr2975_Meas_dist_DIST failed !"); 
            
            /*
            * Exits UI 
            */
            abortApplication();
        }
        
        SetCtrlVal(panelHandle[METER], METER_distortion_meter, meas_distortion);
        
        /*
        * Lets User interact with UI 
        */
        /*
        * It allow the user to turn off the continuous update function 
        */
        ProcessSystemEvents ();
        
        if (status = ifr2975_Meas_Volt_DVM (handle, &meas_dvm)<0) {
            /*
            * Displays Driver Error 
            */
            MessagePopup("Driver Error", "ifr2975_Meas_Volt_DVM failed !"); 
            
            /*
            * Exits UI 
            */
            abortApplication();
        }
        
        SetCtrlVal(panelHandle[METER], METER_dvm_meter, meas_dvm);
        
        /*
        * Lets User interact with UI 
        */
        /*
        * It allow the user to turn off the continuous update function 
        */
        ProcessSystemEvents ();
        
        if (status = ifr2975_Meas_Power_WBPWR (handle, &meas_power)<0) {
            /*
            * Displays Driver Error 
            */
            MessagePopup("Driver Error", "ifr2975_Meas_Power_WBPWR failed !"); 
            
            /*
            * Exits UI 
            */
            abortApplication();
        }
        
        SetCtrlVal(panelHandle[METER], METER_power_meter, meas_power);
    
        /*
        * Lets User interact with UI 
        */
        /*
        * It allow the user to turn off the continuous update function 
        */
        ProcessSystemEvents ();
        
        if (status = ifr2975_Meas_Dev_FMDM (handle, &meas_dev)<0) {
            /*
            * Displays Driver Error 
            */
            MessagePopup("Driver Error", "ifr2975_Meas_Dev_FMDM failed !"); 
            
            /*
            * Exits UI 
            */
            abortApplication();
        }
        
        SetCtrlVal(panelHandle[METER], METER_deviation_meter, meas_dev);
        
        /*
        * Lets User interact with UI 
        */
        /*
        * It allow the user to turn off the continuous update function 
        */
        ProcessSystemEvents ();
        
        if (status = ifr2975_Meas_Freq_Err_RFERR (handle, &meas_rferror)<0) {
            /*
            * Displays Driver Error 
            */
            MessagePopup("Driver Error", "ifr2975_Meas_Freq_Err_RFERR failed !"); 
            
            /*
            * Exits UI 
            */
            abortApplication();
        }
        
        SetCtrlVal(panelHandle[METER], METER_rferror_meter, meas_rferror);    
        
        /*
        * Lets User interact with UI 
        */
        /*
        * It allow the user to turn off the continuous update function 
        */
        ProcessSystemEvents ();
        
        if (status = ifr2975_Meas_RLMeter (handle, &meas_rssi)<0) {
            /*
            * Displays Driver Error 
            */
            MessagePopup("Driver Error", "ifr2975_Meas_RLMeter failed !"); 
            
            /*
            * Exits UI 
            */
            abortApplication();
        }
        
        SetCtrlVal(panelHandle[METER], METER_rssi_meter, meas_rssi);
        
    }

    return 0;
}


/*****************************************************************************
* Function: UPDATE_UI                                   
* Purpose:  This is updates the all the panels.         
*                                                       
*****************************************************************************/
int CVICALLBACK UPDATE_UI (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    ViInt16 status;
    
    switch (event)
        {
        case EVENT_COMMIT:
            SetCtrlAttribute (panelHandle[GEN], GEN_UPDATE_GEN, ATTR_DIMMED, 1);
            SetCtrlAttribute (panelHandle[SCOPE], SCOPE_UPDATE_SCOPE, ATTR_DIMMED, 1);
            SetCtrlAttribute (panelHandle[FGEN], FGEN_UPDATE_FGEN, ATTR_DIMMED, 1);
            SetCtrlAttribute (panelHandle[SA], SA_UPDATE_SA, ATTR_DIMMED, 1);
            SetCtrlAttribute (panelHandle[METER], METER_UPDATE_METERS, ATTR_DIMMED, 1);
            SetCtrlAttribute (panelHandle[RECEIVER], RECEIVER_UPDATE_REC, ATTR_DIMMED, 1);
            SetCtrlAttribute (panelHandle[PANEL], PANEL_DEFAULTS_UI, ATTR_DIMMED, 1);
            SetCtrlAttribute (panelHandle[PANEL], PANEL_EXIT, ATTR_DIMMED, 1);
            SetCtrlAttribute (panelHandle[PANEL], PANEL_UPDATE_GO, ATTR_DIMMED, 1);
            //SetCtrlAttribute (panelHandle[PANEL], PANEL_UPDATE_UI, ATTR_DIMMED, 1);
            							  
            /*
            * Tells user that the UI is communicating with the box 
            */
            SetWaitCursor(1);
            
            if (status = fgen_update()<0) {
                /*
                * Displays Driver Error 
                */
                MessagePopup("Driver Error", "Function Generator Update failed !"); 
                
                /*
                * Exits UI 
                */
                abortApplication();
            }
            
            if (status = sa_update()<0) {
                /*
                * Displays Driver Error 
                */
                MessagePopup("Driver Error", "Spectrum Analyzer Update failed !"); 
                
                /*
                * Exits UI 
                */
                abortApplication();
            }
            
            if (status = scope_update()<0) {
                /*
                * Displays Driver Error 
                */
                MessagePopup("Driver Error", "Scope Update failed !"); 
                
                /*
                * Exits UI 
                */
                abortApplication();
            }
            
            if (status = generator_update()<0) {
                /*
                * Displays Driver Error 
                */
                MessagePopup("Driver Error", "Generator Update failed !"); 
                
                /*
                * Exits UI 
                */
                abortApplication();
            }
            
            if (status = receiver_update()<0) {
                /*
                * Displays Driver Error 
                */
                MessagePopup("Driver Error", "Receiver Update failed !"); 
                
                /*
                * Exits UI 
                */
                abortApplication();
            }
            
            if (status = meter_update()<0) {
                /*
                * Displays Driver Error 
                */
                MessagePopup("Driver Error", "Meter Update failed !"); 
                
                /*
                * Exits UI 
                */
                abortApplication();
            }
            
            /*
            * Tells user that the UI is not longer communicating with the box
            */
            SetWaitCursor(0);

            SetCtrlAttribute (panelHandle[GEN], GEN_UPDATE_GEN, ATTR_DIMMED, 0);
            SetCtrlAttribute (panelHandle[SCOPE], SCOPE_UPDATE_SCOPE, ATTR_DIMMED, 0);
            SetCtrlAttribute (panelHandle[FGEN], FGEN_UPDATE_FGEN, ATTR_DIMMED, 0);
            SetCtrlAttribute (panelHandle[SA], SA_UPDATE_SA, ATTR_DIMMED, 0);
            SetCtrlAttribute (panelHandle[METER], METER_UPDATE_METERS, ATTR_DIMMED, 0);
            SetCtrlAttribute (panelHandle[RECEIVER], RECEIVER_UPDATE_REC, ATTR_DIMMED, 0);
            SetCtrlAttribute (panelHandle[PANEL], PANEL_DEFAULTS_UI, ATTR_DIMMED, 0);
            SetCtrlAttribute (panelHandle[PANEL], PANEL_EXIT, ATTR_DIMMED, 0);
            SetCtrlAttribute (panelHandle[PANEL], PANEL_UPDATE_GO, ATTR_DIMMED, 0);
            //SetCtrlAttribute (panelHandle[PANEL], PANEL_UPDATE_UI, ATTR_DIMMED, 0);
            
            break;
        case EVENT_RIGHT_CLICK:
            
            /*
            * Help Message for given widget 
            */
            MessagePopup("HELP: Update UI Button", "This button updates the UI."); 
            
            break;
        }
    return 0;
}


/*****************************************************************************
* Function: UPDATE_GO                                   
* Purpose:  This is updates the all the panels continuously.               
*                                                       
*****************************************************************************/
int CVICALLBACK UPDATE_GO (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    ViInt32 status;
    ViInt32 state;
    
    switch (event)
        {
        case EVENT_COMMIT:
            /*
            * Tells user that the UI is communicating with the box 
            */
            SetWaitCursor(1);
            SetCtrlAttribute (panelHandle[GEN], GEN_UPDATE_GEN, ATTR_DIMMED, 1);
            SetCtrlAttribute (panelHandle[SCOPE], SCOPE_UPDATE_SCOPE, ATTR_DIMMED, 1);
            SetCtrlAttribute (panelHandle[FGEN], FGEN_UPDATE_FGEN, ATTR_DIMMED, 1);
            SetCtrlAttribute (panelHandle[SA], SA_UPDATE_SA, ATTR_DIMMED, 1);
            SetCtrlAttribute (panelHandle[METER], METER_UPDATE_METERS, ATTR_DIMMED, 1);
            SetCtrlAttribute (panelHandle[RECEIVER], RECEIVER_UPDATE_REC, ATTR_DIMMED, 1);
            SetCtrlAttribute (panelHandle[PANEL], PANEL_DEFAULTS_UI, ATTR_DIMMED, 1);
            SetCtrlAttribute (panelHandle[PANEL], PANEL_EXIT, ATTR_DIMMED, 1);
            //SetCtrlAttribute (panelHandle[PANEL], PANEL_UPDATE_GO, ATTR_DIMMED, 1);
            SetCtrlAttribute (panelHandle[PANEL], PANEL_UPDATE_UI, ATTR_DIMMED, 1);

            /*
            * Lets User interact with UI 
            */
            /*
            * It allow the user to turn off the continuous update function 
            */
            ProcessSystemEvents ();
            
            GetCtrlVal (panelHandle[PANEL], PANEL_UPDATE_GO, &state);
            
            for (;state==1;) {
                GetCtrlVal (panelHandle[PANEL], PANEL_UPDATE_GO, &state);
                
                /*
                * Lets User interact with UI 
                */
                /*
                * It allow the user to turn off the continuous update function 
                */
                ProcessSystemEvents ();
                
                if (state==1) {
                    if (status = fgen_update()<0) {
                        /*
                        * Displays Driver Error 
                        */
                        MessagePopup("Driver Error", "Function Generator Update failed !"); 
                        
                        /*
                        * Exits UI 
                        */
                        abortApplication();
                    }
                }
                
                GetCtrlVal (panelHandle[PANEL], PANEL_UPDATE_GO, &state);
                
                /*
                * Lets User interact with UI 
                */
                /*
                * It allow the user to turn off the continuous update function 
                */
                ProcessSystemEvents ();
                
                if (state==1) {
                    if (status = sa_update()<0) {
                        /*
                        * Displays Driver Error 
                        */
                        MessagePopup("Driver Error", "Spectrum Analyzer Update failed !"); 
                        
                        /*
                        * Exits UI 
                        */
                        abortApplication();
                    }
                }
                
                GetCtrlVal (panelHandle[PANEL], PANEL_UPDATE_GO, &state);
                
                /*
                * Lets User interact with UI 
                */
                /*
                * It allow the user to turn off the continuous update function 
                */
                ProcessSystemEvents ();
                
                if (state==1) {                    
                    if (status = scope_update()<0) {
                        /*
                        * Displays Driver Error 
                        */
                        MessagePopup("Driver Error", "Scope Update failed !"); 
                        
                        /*
                        * Exits UI 
                        */
                        abortApplication();
                    }
                }
                
                GetCtrlVal (panelHandle[PANEL], PANEL_UPDATE_GO, &state);
                
                /*
                * Lets User interact with UI 
                */
                /*
                * It allow the user to turn off the continuous update function 
                */
                ProcessSystemEvents ();
                
                if (state==1) {                    
                    if (status = generator_update()<0) {
                        /*
                        * Displays Driver Error 
                        */
                        MessagePopup("Driver Error", "Generator Update failed !"); 
                        
                        /*
                        * Exits UI 
                        */
                        abortApplication();
                    }
                }
                
                GetCtrlVal (panelHandle[PANEL], PANEL_UPDATE_GO, &state);
                
                /*
                * Lets User interact with UI 
                */
                /*
                * It allow the user to turn off the continuous update function 
                */
                ProcessSystemEvents ();
                
                if (state==1) {
                    if (status = receiver_update()<0) {
                        /*
                        * Displays Driver Error 
                        */
                        MessagePopup("Driver Error", "Receiver Update failed !"); 
                        
                        /*
                        * Exits UI 
                        */
                        abortApplication();
                    }
                }
                
                GetCtrlVal (panelHandle[PANEL], PANEL_UPDATE_GO, &state);
                
                /*
                * Lets User interact with UI 
                */
                /*
                * It allow the user to turn off the continuous update function 
                */
                ProcessSystemEvents ();
                
                if (state==1) {                    
                    if (status = meter_update()<0) {
                        /*
                        * Displays Driver Error 
                        */
                        MessagePopup("Driver Error", "Meter Update failed !"); 
                        
                        /*
                        * Exits UI 
                        */
                        abortApplication();
                    }
                }            
            }
            SetCtrlAttribute (panelHandle[GEN], GEN_UPDATE_GEN, ATTR_DIMMED, 0);
            SetCtrlAttribute (panelHandle[SCOPE], SCOPE_UPDATE_SCOPE, ATTR_DIMMED, 0);
            SetCtrlAttribute (panelHandle[FGEN], FGEN_UPDATE_FGEN, ATTR_DIMMED, 0);
            SetCtrlAttribute (panelHandle[SA], SA_UPDATE_SA, ATTR_DIMMED, 0);
            SetCtrlAttribute (panelHandle[METER], METER_UPDATE_METERS, ATTR_DIMMED, 0);
            SetCtrlAttribute (panelHandle[RECEIVER], RECEIVER_UPDATE_REC, ATTR_DIMMED, 0);
            SetCtrlAttribute (panelHandle[PANEL], PANEL_DEFAULTS_UI, ATTR_DIMMED, 0);
            SetCtrlAttribute (panelHandle[PANEL], PANEL_EXIT, ATTR_DIMMED, 0);
            //SetCtrlAttribute (panelHandle[PANEL], PANEL_UPDATE_GO, ATTR_DIMMED, 0);
            SetCtrlAttribute (panelHandle[PANEL], PANEL_UPDATE_UI, ATTR_DIMMED, 0);
            
            /*
            * Tells user that the UI is not longer communicating with the box
            */
            SetWaitCursor(0);
            
            break;
        case EVENT_VAL_CHANGED:
            GetCtrlVal (panelHandle[PANEL], PANEL_UPDATE_GO, &state);
            
            
            break;
        case EVENT_RIGHT_CLICK:
            /*
            * Help Message for given widget 
            */
            MessagePopup("HELP: Go UI Button", "This button continously updates the UI."); 
            
            
            break;
        }
    return 0;
}

/*****************************************************************************
* Function: CONN_PORT                                   
* Purpose:  This is the function that specifies the TCP/IP port address.    
*                                                       
*****************************************************************************/
int CVICALLBACK CONN_PORT (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event)
        {
        case EVENT_COMMIT:

            break;
        case EVENT_RIGHT_CLICK:
            /*
            * Help Message for given widget 
            */
            MessagePopup("HELP: Port Address", "This field specifies the port address of the unit."); 
            break;
        }
    return 0;
}

/*****************************************************************************
* Function: IP_ADDR                                     
* Purpose:  This is the function that specifies the IP address.             
*                                                       
*****************************************************************************/
int CVICALLBACK IP_ADDR (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event)
        {
        case EVENT_COMMIT:

            break;
        case EVENT_RIGHT_CLICK:
            /*
            * Help Message for given widget 
            */
            MessagePopup("HELP: IP Address", "This field specifies the IP address of the unit.\n\n This value can be found on the System --> Configuration Screen."); 
            break;
        }
    return 0;
}

/*****************************************************************************
* Function: GPIB_ADDR                                   
* Purpose:  This is the function that specifies the GPIB address.           
*                                                       
*****************************************************************************/
int CVICALLBACK GPIB_ADDR (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event)
        {
        case EVENT_COMMIT:

            break;
        case EVENT_RIGHT_CLICK:
            /*
            * Help Message for given widget 
            */
            MessagePopup("HELP: GPIB Address", "This field specifies the GPIB address of the unit.\n\n This value can be found on the System --> Configuration Screen.");
            
            break;
        }
    return 0;
}

/*****************************************************************************
* Function: SERIAL_PORT                                 
* Purpose:  This is the function that specifies the serial port.            
*                                                       
*****************************************************************************/
int CVICALLBACK SERIAL_PORT (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event)
        {
        case EVENT_COMMIT:

            break;
        case EVENT_RIGHT_CLICK:
            /*
            * Help Message for given widget 
            */
            MessagePopup("HELP: Serial Port", "This field specifies the serial port connected to the unit.");
            
            break;
        }
    return 0;
}


/*****************************************************************************
* Function: Generator_RF                                
* Purpose:  This is the function turns on/off the generator.                
*                                                       
*****************************************************************************/
int CVICALLBACK Generator_RF (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    ViInt16 rf_on_off;
    ViInt16 status;
    
    switch (event)
        {
        case EVENT_COMMIT:
            /*
            * Tells user that the UI is communicating with the box 
            */
            SetWaitCursor(1);
            
            GetCtrlVal (panelHandle[GEN], GEN_Generator_RF, &rf_on_off);
            
            if (status = ifr2975_Set_PTT_State_GEN (handle, rf_on_off)<0) {
                /*
                * Displays Driver Error 
                */
                MessagePopup("Driver Error", "ifr2975_Set_PTT_State_GEN failed !"); 
                
                
                /*
                * Exits UI 
                */
                abortApplication();
            }
            
            /*
            * Tells user that the UI is not longer communicating with the box
            */
            SetWaitCursor(0);
            
            break;
        case EVENT_RIGHT_CLICK:
            /*
            * Help Message for given widget 
            */
            MessagePopup("HELP: RF", "This field determines if the Generator is on/off.");
            break;
        }
    return 0;
}

/*****************************************************************************
* Function: Generator_Output                            
* Purpose:  This is the function selects the output port of the generator.  
*                                                       
*****************************************************************************/
int CVICALLBACK Generator_Output (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    ViInt16 output_port;
    ViInt16 status;
    
    switch (event)
        {
        case EVENT_COMMIT:
            /*
            * Tells user that the UI is communicating with the box 
            */
            SetWaitCursor(1);
            
            GetCtrlVal (panelHandle[GEN], GEN_Generator_Output, &output_port);
            
            if (status = ifr2975_Set_RFOut_GEN (handle, output_port)<0) {
                /*
                * Displays Driver Error 
                */
                MessagePopup("Driver Error", "ifr2975_Set_RFOut_GEN failed !"); 
                
                /*
                * Exits UI 
                */
                abortApplication();
            }
            /*
            * Tells user that the UI is not longer communicating with the box
            */
            SetWaitCursor(0);
            break;
        case EVENT_RIGHT_CLICK:

            /*
            * Help Message for given widget 
            */
            MessagePopup("HELP: Receiver Output", "This field shows the output port of the Generator.");
            
            break;
        }
    return 0;
}


/*****************************************************************************
* Function: GEN_FREQ                                    
* Purpose:  This is the function sets the frequency of the generator.       
*                                                       
*****************************************************************************/
int CVICALLBACK GEN_FREQ (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    ViReal64 freq;
    ViInt16 status;
    
    switch (event)
        {
        case EVENT_COMMIT:
            /*
            * Tells user that the UI is communicating with the box 
            */
            SetWaitCursor(1);
            
            GetCtrlVal (panelHandle[GEN], GEN_GEN_FREQ, &freq);
            
            if (status = ifr2975_Set_Freq_GEN (handle, freq)<0) {
                /*
                * Displays Driver Error 
                */
                MessagePopup("Driver Error", "ifr2975_Set_Freq_GEN failed !"); 
                
                /*
                * Exits UI 
                */
                abortApplication();
            }
            /*
            * Tells user that the UI is not longer communicating with the box
            */
            SetWaitCursor(0);
            break;
        case EVENT_RIGHT_CLICK:
            
            /*
            * Help Message for given widget 
            */
            MessagePopup("HELP: Generator Level", "This field shows the output frequency of the Generator.");
            
            break;
        }
    return 0;
}

/*****************************************************************************
* Function: Generator_level                             
* Purpose:  This is the function sets the level of the generator.           
*                                                       
*****************************************************************************/
int CVICALLBACK Generator_level (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    ViReal64 level;
    ViInt16 status;
    
    switch (event)
        {
        case EVENT_COMMIT:
            /*
            * Tells user that the UI is communicating with the box 
            */
            SetWaitCursor(1);
            
            GetCtrlVal (panelHandle[GEN], GEN_Generator_level, &level);
            
            if (status = ifr2975_Set_Level_GEN (handle, level)<0) {
                /*
                * Displays Driver Error 
                */
                MessagePopup("Driver Error", "ifr2975_Set_Level_GEN failed !"); \
                
                /*
                * Exits UI 
                */
                abortApplication();
            }
            
            /*
            * Tells user that the UI is not longer communicating with the box
            */
            SetWaitCursor(0);
            break;
        case EVENT_RIGHT_CLICK:

            /*
            * Help Message for given widget 
            */
            MessagePopup("HELP: Generator Level", "This field shows the output level of the Generator.");
            
            break;
        }
    return 0;
}

/*****************************************************************************
* Function: Mod_Type                                    
* Purpose:  This is the function sets the modulation type of the generator. 
*                                                       
*****************************************************************************/
int CVICALLBACK Mod_Type (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    ViInt16 mod;
    ViInt16 status;

    switch (event)
        {
        case EVENT_COMMIT:
            /*
            * Tells user that the UI is communicating with the box 
            */
            SetWaitCursor(1);
            
            GetCtrlVal (panelHandle[GEN], GEN_Mod_Type, &mod);
            
            if (status = ifr2975_Set_Mod_GEN (handle, mod)<0) {
                /*
                * Displays Driver Error 
                */
                MessagePopup("Driver Error", "ifr2975_Set_Level_GEN failed !"); 
                
                /*
                * Exits UI 
                */
                abortApplication();
            }
            
            /*
            * Tells user that the UI is not longer communicating with the box
            */
            SetWaitCursor(0);
            
            break;
        case EVENT_RIGHT_CLICK:
            
            /*
            * Help Message for given widget 
            */
            MessagePopup("HELP: Generator Level", "This field shows the modulation type of the Generator.");
            
            break;
        }
    return 0;
}


/*****************************************************************************
* Function: Receiver_freq                               
* Purpose:  This is the function sets the frequency of the receiver.        
*                                                       
*****************************************************************************/
int CVICALLBACK Receiver_freq (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    ViReal64 freq;
    ViInt16 status;
    
    switch (event)
        {
        case EVENT_COMMIT:
            
            /*
            * Tells user that the UI is communicating with the box 
            */
            SetWaitCursor(1);
            
            GetCtrlVal (panelHandle[RECEIVER], RECEIVER_Receiver_freq, &freq);
            
            if (status = ifr2975_Set_Freq_REC (handle, freq)<0) {
                /*
                * Displays Driver Error 
                */
                MessagePopup("Driver Error", "ifr2975_Set_Freq_REC failed !"); 
                
                /*
                * Exits UI 
                */
                abortApplication();
            }
            /*
            * Tells user that the UI is not longer communicating with the box
            */
            SetWaitCursor(0);
            break;
        case EVENT_RIGHT_CLICK:
            
            /*
            * Help Message for given widget 
            */
            MessagePopup("HELP: Receiver Frequency", "This field shows the input frequency of the Receiver.");
            
            break;
        }
    return 0;
}

/*****************************************************************************
* Function: Receiver_Input                              
* Purpose:  This is the function sets the input port of the receiver.       
*                                                       
*****************************************************************************/
int CVICALLBACK Receiver_Input (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    ViInt16 input;
    ViInt16 status;
    
    switch (event)
        {
        case EVENT_COMMIT:
            /*
            * Tells user that the UI is communicating with the box 
            */
            SetWaitCursor(1);
            GetCtrlVal (panelHandle[RECEIVER], RECEIVER_Receiver_Input, &input);
            
            if (status = ifr2975_Set_RFIn_REC (handle, input)<0) {
                /*
                * Displays Driver Error 
                */
                MessagePopup("Driver Error", "ifr2975_Set_IFGain_REC failed !"); 
                
                /*
                * Exits UI 
                */
                abortApplication();
            }
            /*
            * Tells user that the UI is not longer communicating with the box
            */
            SetWaitCursor(0);
            break;
        case EVENT_RIGHT_CLICK:

            /*
            * Help Message for given widget 
            */
            MessagePopup("HELP: Receiver Input Port", "This field shows the input port of the Receiver.");
            
            break;
        }
    return 0;
}

/*****************************************************************************
* Function: Receiver_Demod                              
* Purpose:  This is the function sets the demodulatoin of the receiver.     
*                                                       
*****************************************************************************/
int CVICALLBACK Receiver_Demod (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    ViInt16 demod;
    ViInt16 status;

    switch (event)
        {
        case EVENT_COMMIT:
            /*
            * Tells user that the UI is communicating with the box 
            */
            SetWaitCursor(1);
            GetCtrlVal (panelHandle[RECEIVER], RECEIVER_Receiver_Demod, &demod);
            
            if (status = ifr2975_Set_Demod_Rec (handle, demod)<0) {
                /*
                * Displays Driver Error 
                */
                MessagePopup("Driver Error", "ifr2975_Set_Demod_Rec failed !"); 
                
                /*
                * Exits UI 
                */
                abortApplication();
            }
            
            if (demod==0) {
                SetCtrlAttribute (panelHandle[RECEIVER], RECEIVER_Receiver_FILTER, ATTR_DIMMED, 0);
            } else {
                SetCtrlAttribute (panelHandle[RECEIVER], RECEIVER_Receiver_FILTER, ATTR_DIMMED, 1);
            }
        
            /*
            * Tells user that the UI is not longer communicating with the box
            */
            SetWaitCursor(0);
            break;
        case EVENT_RIGHT_CLICK:

            /*
            * Help Message for given widget 
            */
            MessagePopup("HELP: Receiver Demodulation", "This field shows the input demodulation type of the Receiver.");
            
            break;
        }
    return 0;
}

/*****************************************************************************
* Function: Receiver_Atten                              
* Purpose:  This is the function sets the attenuation of the receiver.      
*                                                       
*****************************************************************************/
int CVICALLBACK Receiver_Atten (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    ViInt16 status;
    ViInt16 atten;
    
    switch (event)
        {
        case EVENT_COMMIT:
            /*
            * Tells user that the UI is communicating with the box 
            */
            SetWaitCursor(1);
            
            GetCtrlVal (panelHandle[RECEIVER], RECEIVER_Receiver_Atten, &atten);
            
            if (status = ifr2975_Set_Attn_REC (handle, atten)<0) {
                /*
                * Displays Driver Error 
                */
                MessagePopup("Driver Error", "ifr2975_Set_Level_GEN failed !"); 
                
                /*
                * Exits UI 
                */
                abortApplication();
            }
            /*
            * Tells user that the UI is not longer communicating with the box
            */
            SetWaitCursor(0);
            break;
        case EVENT_RIGHT_CLICK:
            
            /*
            * Help Message for given widget 
            */
            MessagePopup("HELP: Receiver Attenuation", "This field shows the input attenuation of the Receiver.");
            
            break;
        }
    return 0;
}


/*****************************************************************************
* Function: scope_sweep                                 
* Purpose:  This is the function sets the sweep speed of the scope.         
*                                                       
*****************************************************************************/
int CVICALLBACK scope_sweep (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    ViReal64 sweep;
    ViInt16 status;
    
    switch (event)
        {
        case EVENT_COMMIT:
            /*
            * Tells user that the UI is communicating with the box 
            */
            SetWaitCursor(1);
            
            GetCtrlVal (panelHandle[SCOPE], SCOPE_scope_sweep, &sweep);
            
            if (status = ifr2975_Set_Sweep_OSCOPE (handle, sweep)<0) {
                /*
                * Displays Driver Error 
                */
                MessagePopup("Driver Error", "ifr2975_Set_Sweep_OSCOPE failed !"); 
                
                /*
                * Exits UI 
                */
                abortApplication();
            }
            /*
            * Tells user that the UI is not longer communicating with the box
            */
            SetWaitCursor(0);
            break;
        case EVENT_RIGHT_CLICK:
            
            /*
            * Help Message for given widget 
            */
            MessagePopup("HELP: Scope Sweep Speed", "This field shows the sweep speed of the Scope.");
            
            break;
        }
    return 0;
}

/*****************************************************************************
* Function: scope_sweep                                 
* Purpose:  This is the function sets the trigger type of the scope.        
*                                                       
*****************************************************************************/
int CVICALLBACK scope_trigger (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    ViInt32 trigger;
    ViInt16 status;
    
    switch (event)
        {
        case EVENT_COMMIT:
            /*
            * Tells user that the UI is communicating with the box 
            */
            SetWaitCursor(1);
            
            GetCtrlVal (panelHandle[SCOPE], SCOPE_scope_trigger, &trigger);
            
            if (status = ifr2975_Set_Trig_Type_OSCOPE (handle, trigger)<0) {
                /*
                * Displays Driver Error 
                */
                MessagePopup("Driver Error", "ifr2975_Set_Trig_Type_OSCOPE failed !"); 
                
                /*
                * Exits UI 
                */
                abortApplication();
            }
            /*
            * Tells user that the UI is not longer communicating with the box
            */
            SetWaitCursor(0);
            break;
        case EVENT_RIGHT_CLICK:

            /*
            * Help Message for given widget 
            */
            MessagePopup("HELP: Scope Trigger Mode", "This field shows the trigger mode of the Scope.");
            
            break;
        }
    return 0;
}

/*****************************************************************************
* Function: scope_source                                
* Purpose:  This is the function sets the trigger source of the scope.      
*                                                       
*****************************************************************************/
int CVICALLBACK scope_source (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    ViInt32 source;
    ViInt16 status;
    
    switch (event)
        {
        case EVENT_COMMIT:
            /*
            * Tells user that the UI is communicating with the box 
            */
            SetWaitCursor(1);
            
            GetCtrlVal (panelHandle[SCOPE], SCOPE_scope_source, &source);
            
            if (status = ifr2975_Set_Trig_Source_OSCOPE (handle, source)<0) {
                /*
                * Displays Driver Error 
                */
                MessagePopup("Driver Error", "ifr2975_Set_Trig_Source_OSCOPE failed !"); 
                
                /*
                * Exits UI 
                */
                abortApplication();
            }
            /*
            * Tells user that the UI is not longer communicating with the box
            */
            SetWaitCursor(0);
            break;
        case EVENT_RIGHT_CLICK:

            /*
            * Help Message for given widget 
            */
            MessagePopup("HELP: Scope Trigger Source", "This field shows the triggering source of the Scope.");
            
            break;
        }
    return 0;
}

/*****************************************************************************
* Function: scope_level_v                               
* Purpose:  This is the function sets the trigger level of the scope.       
*                                                       
*****************************************************************************/
int CVICALLBACK scope_level_v (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    ViReal64 level_v;
    ViInt16 status;
    
    switch (event)
        {
        case EVENT_COMMIT:
            /*
            * Tells user that the UI is communicating with the box 
            */
            SetWaitCursor(1);
            
            GetCtrlVal (panelHandle[SCOPE], SCOPE_scope_level_v, &level_v);
            
            if (status = ifr2975_Set_Trig_Lev_OSCOPE (handle, level_v)<0) {
                /*
                * Displays Driver Error 
                */
                MessagePopup("Driver Error", "ifr2975_Set_Trig_Lev_OSCOPE failed !"); 
                
                /*
                * Exits UI 
                */
                abortApplication();
            }
            /*
            * Tells user that the UI is not longer communicating with the box
            */
            SetWaitCursor(0);
            break;
        case EVENT_RIGHT_CLICK:

            /*
            * Help Message for given widget 
            */
            MessagePopup("HELP: Scope Trigger Voltage Level", "This field shows the trigger voltage level of the Scope.");
            
            break;
        }
    return 0;
}

/*****************************************************************************
* Function: scope_slope                                 
* Purpose:  This is the function sets the trigger slope of the scope.       
*                                                       
*****************************************************************************/
int CVICALLBACK scope_slope (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    ViInt32 slope;
    ViInt16 status;
    
    switch (event)
        {
        case EVENT_COMMIT:
            /*
            * Tells user that the UI is communicating with the box 
            */
            SetWaitCursor(1);
            
            GetCtrlVal (panelHandle[SCOPE], SCOPE_scope_slope, &slope);
            
            if (status = ifr2975_Set_Trig_Pol_OSCOPE (handle, slope)<0) {
                /*
                * Displays Driver Error 
                */
                MessagePopup("Driver Error", "ifr2975_Set_Trig_Pol_OSCOPE failed !"); 
                
                /*
                * Exits UI 
                */
                abortApplication();
            }
            
            /*
            * Tells user that the UI is not longer communicating with the box
            */
            SetWaitCursor(0);
            break;
        case EVENT_RIGHT_CLICK:

            /*
            * Help Message for given widget 
            */
            MessagePopup("HELP: Scope Triggering Slope", "This field shows the triggering slope of the Scope.");
            
            break;
        }
    return 0;
}

/*****************************************************************************
* Function: scope_h_pos                                 
* Purpose:  This is the function sets the horizontal position of the scope  
*           trace.                                      
*****************************************************************************/
int CVICALLBACK scope_h_pos (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    float pos;
    ViInt16 status;
    
    switch (event)
        {
        case EVENT_COMMIT:
            /*
            * Tells user that the UI is communicating with the box 
            */
            SetWaitCursor(1);
            
            GetCtrlVal (panelHandle[SCOPE], SCOPE_scope_h_pos, &pos);
            
            if (status = ifr2975_Set_Horiz_Pos_OSCOPE (handle, pos)<0) {
                /*
                * Displays Driver Error 
                */
                MessagePopup("Driver Error", "ifr2975_Set_Horiz_Pos_OSCOPE failed !"); 
                
                /*
                * Exits UI 
                */
                abortApplication();
            }
            
            /*
            * Tells user that the UI is not longer communicating with the box
            */
            SetWaitCursor(0);
            break;
        case EVENT_RIGHT_CLICK:

            /*
            * Help Message for given widget 
            */
            MessagePopup("HELP: Scope Horizontal Position", "This field shows the horizontal position of the Scope.");
            
            break;
        }
    return 0;
}


/*****************************************************************************
* Function: scope_ch1                                   
* Purpose:  This is the function turns on/off the scope channel 1 for the   
*           UI.                                         
*****************************************************************************/
int CVICALLBACK scope_ch1 (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    ViInt32 ch1_on;
    
    switch (event)
        {
        case EVENT_COMMIT:
            DeleteGraphPlot(panelHandle[SCOPE], SCOPE_scope_trace, -1, 0);
            
            GetCtrlVal (panelHandle[SCOPE], SCOPE_scope_ch1, &ch1_on);
            
            if(ch1_on) {
                SetCtrlAttribute (panelHandle[SCOPE], SCOPE_scope_v_ref_ch1, ATTR_DIMMED, 0);
                SetCtrlAttribute (panelHandle[SCOPE], SCOPE_scope_v_div_ch1, ATTR_DIMMED, 0);
            } else {
                 SetCtrlAttribute (panelHandle[SCOPE], SCOPE_scope_v_ref_ch1, ATTR_DIMMED, 1);
                 SetCtrlAttribute (panelHandle[SCOPE], SCOPE_scope_v_div_ch1, ATTR_DIMMED, 1);
            }
            
            break;
        case EVENT_RIGHT_CLICK:

            /*
            * Help Message for given widget 
            */
            MessagePopup("HELP: Scope Channel 1", "This field shows if scope channel 1 will be updated.");
            
            break;
        }
    return 0;
}

/*****************************************************************************
* Function: scope_ch2                                   
* Purpose:  This is the function turns on/off the scope channel 2 for the   
*           UI.                                         
*****************************************************************************/
int CVICALLBACK scope_ch2 (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    ViInt32 ch2_on;
    
    switch (event)
        {
        case EVENT_COMMIT:

            DeleteGraphPlot(panelHandle[SCOPE], SCOPE_scope_trace, -1, 0);
            
            GetCtrlVal (panelHandle[SCOPE], SCOPE_scope_ch2, &ch2_on);
            
            if(ch2_on) {
                SetCtrlAttribute (panelHandle[SCOPE], SCOPE_scope_v_ref_ch2, ATTR_DIMMED, 0);
                SetCtrlAttribute (panelHandle[SCOPE], SCOPE_scope_v_div_ch2, ATTR_DIMMED, 0);
            } else {
                 SetCtrlAttribute (panelHandle[SCOPE], SCOPE_scope_v_ref_ch2, ATTR_DIMMED, 1);
                 SetCtrlAttribute (panelHandle[SCOPE], SCOPE_scope_v_div_ch2, ATTR_DIMMED, 1);
            }
            
            break;
        case EVENT_RIGHT_CLICK:

            /*
            * Help Message for given widget 
            */
            MessagePopup("HELP: Scope Channel 2", "This field shows if scope channel 2 will be updated.");
            
            break;
        }
    return 0;
}

/*****************************************************************************
* Function: DEFAULTS_UI                                 
* Purpose:  This is the function sets up the defaults to the 2975.         
*                                                       
*****************************************************************************/
int CVICALLBACK DEFAULTS_UI (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    ViInt16 status;
    
    switch (event)
        {
        case EVENT_COMMIT:
            SetCtrlAttribute (panelHandle[GEN], GEN_UPDATE_GEN, ATTR_DIMMED, 1);
            SetCtrlAttribute (panelHandle[SCOPE], SCOPE_UPDATE_SCOPE, ATTR_DIMMED, 1);
            SetCtrlAttribute (panelHandle[FGEN], FGEN_UPDATE_FGEN, ATTR_DIMMED, 1);
            SetCtrlAttribute (panelHandle[SA], SA_UPDATE_SA, ATTR_DIMMED, 1);
            SetCtrlAttribute (panelHandle[METER], METER_UPDATE_METERS, ATTR_DIMMED, 1);
            SetCtrlAttribute (panelHandle[RECEIVER], RECEIVER_UPDATE_REC, ATTR_DIMMED, 1);
            //SetCtrlAttribute (panelHandle[PANEL], PANEL_DEFAULTS_UI, ATTR_DIMMED, 1);
            SetCtrlAttribute (panelHandle[PANEL], PANEL_EXIT, ATTR_DIMMED, 1);
            SetCtrlAttribute (panelHandle[PANEL], PANEL_UPDATE_GO, ATTR_DIMMED, 1);
            SetCtrlAttribute (panelHandle[PANEL], PANEL_UPDATE_UI, ATTR_DIMMED, 1);
            
            /*
            * Tells user that the UI is communicating with the box 
            */
            SetWaitCursor(1);
            
            /* Defaults for Generator */
            if (status = ifr2975_Set_RFOut_GEN (handle, 0)<0) {
                /*
                * Displays Driver Error 
                */
                MessagePopup("Driver Error", "ifr2975_Set_RFOut_GEN failed !"); 
                
                /*
                * Exits UI 
                */
                abortApplication();
            }
            
            if (status = ifr2975_Set_Freq_GEN (handle, 170)<0) {
                /*
                * Displays Driver Error 
                */
                MessagePopup("Driver Error", "ifr2975_Set_Freq_GEN failed !"); 
                
                /*
                * Exits UI 
                */
                abortApplication();
            }
            
            if (status = ifr2975_Set_Level_GEN (handle, -30)<0) {
                /*
                * Displays Driver Error 
                */
                MessagePopup("Driver Error", "ifr2975_Set_Level_GEN failed !"); 
                
                /*
                * Exits UI 
                */
                abortApplication();
            }
            
            if (status = ifr2975_Set_Mod_GEN (handle, 1)<0) {
                /*
                * Displays Driver Error 
                */
                MessagePopup("Driver Error", "ifr2975_Set_Mod_GEN failed !"); 
                
                /*
                * Exits UI 
                */
                abortApplication();
            }
            
            if (status = ifr2975_Set_PTT_State_GEN (handle, 1)<0) {
                /*
                * Displays Driver Error 
                */
                MessagePopup("Driver Error", "ifr2975_Set_PTT_State_GEN failed !"); 
                
                /*
                * Exits UI 
                */
                abortApplication();
            }
        
            /* Defaults for Receiver */
            if (status = ifr2975_Set_RFIn_REC (handle, 0)<0) {
                /*
                * Displays Driver Error 
                */
                MessagePopup("Driver Error", "ifr2975_Set_RFIn_REC failed !"); 
                
                /*
                * Exits UI 
                */
                abortApplication();
            }
            
            if (status = ifr2975_Set_Freq_REC (handle, 170)<0) {
                /*
                * Displays Driver Error 
                */
                MessagePopup("Driver Error", "ifr2975_Set_Freq_REC failed !"); 
                
                /*
                * Exits UI 
                */
                abortApplication();
            }
            
            if (status = ifr2975_Set_Demod_Rec (handle, 0)<0) {
                /*
                * Displays Driver Error 
                */
                MessagePopup("Driver Error", "ifr2975_Set_Demod_Rec failed !"); 
                
                /*
                * Exits UI 
                */
                abortApplication();
            }
            
            if (status = ifr2975_Set_Attn_REC (handle, 0)<0) {
                /*
                * Displays Driver Error 
                */
                MessagePopup("Driver Error", "ifr2975_Set_Attn_REC failed !"); 
                
                /*
                * Exits UI 
                */
                abortApplication();
            }
        
            /* Defaults for Fgen */
            if (status = ifr2975_Set_Mode_FGEN (handle, 1, 1)<0) {
                /*
                * Displays Driver Error 
                */
                MessagePopup("Driver Error", "ifr2975_Set_Mode_FGEN failed !"); 
                
                /*
                * Exits UI 
                */
                abortApplication();
            }
            
            if (status = ifr2975_Set_Freq_FGEN (handle, 1, 250)<0) {
                /*
                * Displays Driver Error 
                */
                MessagePopup("Driver Error", "ifr2975_Set_Freq_FGEN failed !"); 
                
                /*
                * Exits UI 
                */
                abortApplication();
            }
            
            if (status = ifr2975_Set_Lev_FGEN (handle, 1, 10)<0) {
                /*
                * Displays Driver Error 
                */
                MessagePopup("Driver Error", "ifr2975_Set_Lev_FGEN failed !"); 
                
                /*
                * Exits UI 
                */
                abortApplication();
            }
        
            if (status = ifr2975_Set_Mode_FGEN (handle, 2, 0)<0) {
                /*
                * Displays Driver Error 
                */
                MessagePopup("Driver Error", "ifr2975_Set_Mode_FGEN failed !"); 
                
                /*
                * Exits UI 
                */
                abortApplication();
            }
            
            /*
            * Tells user that the UI is not longer communicating with the box
            */
            SetWaitCursor(0);

            SetCtrlAttribute (panelHandle[GEN], GEN_UPDATE_GEN, ATTR_DIMMED, 0);
            SetCtrlAttribute (panelHandle[SCOPE], SCOPE_UPDATE_SCOPE, ATTR_DIMMED, 0);
            SetCtrlAttribute (panelHandle[FGEN], FGEN_UPDATE_FGEN, ATTR_DIMMED, 0);
            SetCtrlAttribute (panelHandle[SA], SA_UPDATE_SA, ATTR_DIMMED, 0);
            SetCtrlAttribute (panelHandle[METER], METER_UPDATE_METERS, ATTR_DIMMED, 0);
            SetCtrlAttribute (panelHandle[RECEIVER], RECEIVER_UPDATE_REC, ATTR_DIMMED, 0);
            //SetCtrlAttribute (panelHandle[PANEL], PANEL_DEFAULTS_UI, ATTR_DIMMED, 0);
            SetCtrlAttribute (panelHandle[PANEL], PANEL_EXIT, ATTR_DIMMED, 0);
            SetCtrlAttribute (panelHandle[PANEL], PANEL_UPDATE_GO, ATTR_DIMMED, 0);
            SetCtrlAttribute (panelHandle[PANEL], PANEL_UPDATE_UI, ATTR_DIMMED, 0);

            break;
        case EVENT_RIGHT_CLICK:

            break;
        }
    return 0;
}

/*****************************************************************************
* Function: fgen_freq_1                                 
* Purpose:  This is the function sets the frequency for fgen 1.             
*                                                       
*****************************************************************************/
int CVICALLBACK fgen_freq_1 (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    ViReal64 freq;
    ViInt16 status;
    
    switch (event)
        {
        case EVENT_COMMIT:
            /*
            * Tells user that the UI is communicating with the box 
            */
            SetWaitCursor(1);
            
            GetCtrlVal (panelHandle[FGEN], FGEN_fgen_freq_1, &freq);
            
            if (status = ifr2975_Set_Freq_FGEN (handle, 1, freq)<0) {
                /*
                * Displays Driver Error 
                */
                MessagePopup("Driver Error", "ifr2975_Set_Freq_FGEN failed !"); 
                
                /*
                * Exits UI 
                */
                abortApplication();    
            }

            /*
            * Tells user that the UI is not longer communicating with the box
            */
            SetWaitCursor(0);

            break;
        case EVENT_RIGHT_CLICK:

            /*
            * Help Message for given widget 
            */
            MessagePopup("HELP: Function Generator 1 Frequency", "This field shows the frequency for the Function Generator 1.");
            
            break;
        }
    return 0;
}

/*****************************************************************************
* Function: fgen_level_1                                
* Purpose:  This is the function sets the level for fgen 1.                 
*                                                       
*****************************************************************************/
int CVICALLBACK fgen_level_1 (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    ViReal64 level;
    ViInt16 status;
    
    switch (event)
        {
        case EVENT_COMMIT:
            /*
            * Tells user that the UI is communicating with the box 
            */
            SetWaitCursor(1);
            
            GetCtrlVal (panelHandle[FGEN], FGEN_fgen_level_1, &level);
            
            if (status = ifr2975_Set_Lev_FGEN (handle, 1, level)<0) {
                /*
                * Displays Driver Error 
                */
                MessagePopup("Driver Error", "ifr2975_Set_Lev_FGEN failed !"); 
                
                /*
                * Exits UI 
                */
                abortApplication();
            }
            
            /*
            * Tells user that the UI is not longer communicating with the box
            */
            SetWaitCursor(0);
            break;
        case EVENT_RIGHT_CLICK:

            /*
            * Help Message for given widget 
            */
            MessagePopup("HELP: Function Generator 1 Level", "This field shows the level for the Function Generator 1.");
            
            break;
        }
    return 0;
}

/*****************************************************************************
* Function: fgen_type_1                                 
* Purpose:  This is the function sets the modulation type for fgen 1.       
*                                                       
*****************************************************************************/
int CVICALLBACK fgen_type_1 (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    ViInt16 mode;
    ViInt16 status;
    
    switch (event)
        {
        case EVENT_COMMIT:
            /*
            * Tells user that the UI is communicating with the box 
            */
            SetWaitCursor(1);
            
            GetCtrlVal (panelHandle[FGEN], FGEN_fgen_type_1, &mode);
            
            if (status = ifr2975_Set_Mode_FGEN (handle, 1, mode)<0) {
                /*
                * Displays Driver Error 
                */
                MessagePopup("Driver Error", "ifr2975_Set_Mode_FGEN failed !"); 
                
                /*
                * Exits UI 
                */
                abortApplication();
            }
            
            if (mode==0) {
                SetCtrlAttribute (panelHandle[FGEN], FGEN_fgen_freq_1, ATTR_DIMMED, 1);
                SetCtrlAttribute (panelHandle[FGEN], FGEN_fgen_level_1, ATTR_DIMMED, 1);
                SetCtrlAttribute (panelHandle[FGEN], FGEN_fgen_shape_1, ATTR_DIMMED, 1);
            } else {
                SetCtrlAttribute (panelHandle[FGEN], FGEN_fgen_freq_1, ATTR_DIMMED, 0);
                SetCtrlAttribute (panelHandle[FGEN], FGEN_fgen_level_1, ATTR_DIMMED, 0);
                SetCtrlAttribute (panelHandle[FGEN], FGEN_fgen_shape_1, ATTR_DIMMED, 0);
            }

            /*
            * Tells user that the UI is not longer communicating with the box
            */
            SetWaitCursor(0);
            break;
        case EVENT_RIGHT_CLICK:

            /*
            * Help Message for given widget 
            */
            MessagePopup("HELP: Function Generator 1 Type", "This field shows the type for the Function Generator 1.");
            
            break;
        }
    return 0;
}

/*****************************************************************************
* Function: fgen_shape_1                                 
* Purpose:  This is the function sets the shape for fgen 1.                 
*                                                       
*****************************************************************************/
int CVICALLBACK fgen_shape_1 (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    ViInt16 shape;
    ViInt16 status;
    
    switch (event)
        {
        case EVENT_COMMIT:
            /*
            * Tells user that the UI is communicating with the box 
            */
            SetWaitCursor(1);
            
            GetCtrlVal (panelHandle[FGEN], FGEN_fgen_shape_1, &shape);
            
            if (status = ifr2975_Set_Shape_FGEN (handle, 1, shape)<0) {
                /*
                * Displays Driver Error 
                */
                MessagePopup("Driver Error", "ifr2975_Set_Shape_FGEN failed !"); 
                
                /*
                * Exits UI 
                */
                abortApplication();
            }
            
            /*
            * Tells user that the UI is not longer communicating with the box
            */
            SetWaitCursor(0);
            
            break;
        case EVENT_RIGHT_CLICK:

            /*
            * Help Message for given widget 
            */
            MessagePopup("HELP: Function Generator 1 Shape", "This field shows the shape for the Function Generator 1.");
            
            break;
        }
    return 0;
}

/*****************************************************************************
* Function: fgen_freq_2                                 
* Purpose:  This is the function sets the frequency for fgen 2.            
*                                                       
*****************************************************************************/
int CVICALLBACK fgen_freq_2 (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    ViReal64 freq;
    ViInt16 status;
    
    switch (event)
        {
        case EVENT_COMMIT:
            /*
            * Tells user that the UI is communicating with the box 
            */
            SetWaitCursor(1);
            
            GetCtrlVal (panelHandle[FGEN], FGEN_fgen_freq_2, &freq);
            
            if (status = ifr2975_Set_Freq_FGEN (handle, 2, freq)<0) {
                /*
                * Displays Driver Error 
                */
                MessagePopup("Driver Error", "ifr2975_Set_Freq_FGEN failed !"); 
                
                /*
                * Exits UI 
                */
                abortApplication();
            }
            
            /*
            * Tells user that the UI is not longer communicating with the box
            */
            SetWaitCursor(0);
            break;
        case EVENT_RIGHT_CLICK:

            /*
            * Help Message for given widget 
            */
            MessagePopup("HELP: Function Generator 2 Frequency", "This field shows the frequency for the Function Generator 2.");
            
            break;
        }
    return 0;
}

/*****************************************************************************
* Function: fgen_level_2                                
* Purpose:  This is the function sets the level for fgen 2.                
*                                                       
*****************************************************************************/
int CVICALLBACK fgen_level_2 (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    ViReal64 level;
    ViInt16 status;
    
    switch (event)
        {
        case EVENT_COMMIT:
            /*
            * Tells user that the UI is communicating with the box 
            */
            SetWaitCursor(1);
            
            GetCtrlVal (panelHandle[FGEN], FGEN_fgen_level_2, &level);
            
            if (status = ifr2975_Set_Lev_FGEN (handle, 2, level)<0) {
                /*
                * Displays Driver Error 
                */
                MessagePopup("Driver Error", "ifr2975_Set_Lev_FGEN failed !"); 
                
                /*
                * Exits UI 
                */
                abortApplication();
            }
            /*
            * Tells user that the UI is not longer communicating with the box
            */
            SetWaitCursor(0);
            break;
        case EVENT_RIGHT_CLICK:

            /*
            * Help Message for given widget 
            */
            MessagePopup("HELP: Function Generator 2 Level", "This field shows the level for the Function Generator 2.");
            
            break;
        }
    return 0;
}

/*****************************************************************************
* Function: fgen_type_2                                 
* Purpose:  This is the function sets the modulation type for fgen 2.      
*                                                       
*****************************************************************************/
int CVICALLBACK fgen_type_2 (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    ViInt16 mode;
    ViInt16 status;
    
    switch (event)
        {
        case EVENT_COMMIT:
            /*
            * Tells user that the UI is communicating with the box 
            */
            SetWaitCursor(1);
            
            GetCtrlVal (panelHandle[FGEN], FGEN_fgen_type_2, &mode);
            
            if (status = ifr2975_Set_Mode_FGEN (handle, 2, mode)<0) {
                /*
                * Displays Driver Error 
                */
                MessagePopup("Driver Error", "ifr2975_Set_Mode_FGEN failed !"); 
                
                /*
                * Exits UI 
                */
                abortApplication();
            }
            
            if (mode==0) {
                SetCtrlAttribute (panelHandle[FGEN], FGEN_fgen_freq_2, ATTR_DIMMED, 1);
                SetCtrlAttribute (panelHandle[FGEN], FGEN_fgen_level_2, ATTR_DIMMED, 1);
                SetCtrlAttribute (panelHandle[FGEN], FGEN_fgen_shape_2, ATTR_DIMMED, 1);
            } else {
                SetCtrlAttribute (panelHandle[FGEN], FGEN_fgen_freq_2, ATTR_DIMMED, 0);
                SetCtrlAttribute (panelHandle[FGEN], FGEN_fgen_level_2, ATTR_DIMMED, 0);
                SetCtrlAttribute (panelHandle[FGEN], FGEN_fgen_shape_2, ATTR_DIMMED, 0);
            }

            /*
            * Tells user that the UI is not longer communicating with the box
            */
            SetWaitCursor(0);
            
            break;
        case EVENT_RIGHT_CLICK:

            /*
            * Help Message for given widget 
            */
            MessagePopup("HELP: Function Generator 2 Type", "This field shows the type for the Function Generator 2.");
            
            break;
        }
    return 0;
}

/*****************************************************************************
* Function: fgen_shape_2                                
* Purpose:  This is the function sets the shape for fgen 2.                
*                                                       
*****************************************************************************/
int CVICALLBACK fgen_shape_2 (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    ViInt16 shape;
    ViInt16 status;
    
    switch (event)
        {
        case EVENT_COMMIT:
            /*
            * Tells user that the UI is communicating with the box 
            */
            SetWaitCursor(1);
            
            GetCtrlVal (panelHandle[FGEN], FGEN_fgen_shape_2, &shape);
            
            if (status = ifr2975_Set_Shape_FGEN (handle, 2, shape)<0) {
                /*
                * Displays Driver Error 
                */
                MessagePopup("Driver Error", "ifr2975_Set_Shape_FGEN failed !"); 
                
                /*
                * Exits UI 
                */
                abortApplication();
            }
            /*
            * Tells user that the UI is not longer communicating with the box
            */
            SetWaitCursor(0);
            
            break;
        case EVENT_RIGHT_CLICK:

            /*
            * Help Message for given widget 
            */
            MessagePopup("HELP: Function Generator 2 Shape", "This field shows the shape for the Function Generator 2.");
            
            break;
        }
    return 0;
}


/*****************************************************************************
* Function: mod_freq_1                                  
* Purpose:  This is the function sets the frequency for the generator's    
*           modulator 1.                                
*****************************************************************************/
int CVICALLBACK mod_freq_1 (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{    
    ViInt16 status;
    ViReal64 mod;
    
    switch (event)
        {
        case EVENT_COMMIT:
            /*
            * Tells user that the UI is communicating with the box 
            */
            SetWaitCursor(1);
            
            GetCtrlVal (panelHandle[GEN], GEN_mod_freq_1, &mod);
            
            if (status = ifr2975_Set_Freq_FGEN (handle, 3, mod)<0) {
                /*
                * Displays Driver Error 
                */
                MessagePopup("Driver Error", "ifr2975_Set_Freq_FGEN failed !"); 
                
                /*
                * Exits UI 
                */
                abortApplication();    
            }
            
            /*
            * Tells user that the UI is not longer communicating with the box
            */
            SetWaitCursor(0);
            
            break;
        case EVENT_RIGHT_CLICK:

            /*
            * Help Message for given widget 
            */
            MessagePopup("HELP: Generator Modulation Frequency 1", "This field shows the output modulation frequency of the Generator.");
            
            break;
        }
    return 0;
}

/*****************************************************************************
* Function: mod_level_1                                 
* Purpose:  This is the function sets the level for the generator's        
*           modulator 1.                                
*****************************************************************************/
int CVICALLBACK mod_level_1 (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    ViInt16 status;
    ViReal64 dev;
    
    switch (event)
        {
        case EVENT_COMMIT:
            /*
            * Tells user that the UI is communicating with the box 
            */
            SetWaitCursor(1);
            
            GetCtrlVal (panelHandle[GEN], GEN_mod_level_1, &dev);
            dev = dev*2;
            if (status = ifr2975_Set_Per_FGEN (handle, 3, dev)<0) {
                /*
                * Displays Driver Error 
                */
                MessagePopup("Driver Error", "ifr2975_Set_Dev_FGEN failed !"); 
                
                /*
                * Exits UI 
                */
                abortApplication();
            }

            /*
            * Tells user that the UI is not longer communicating with the box
            */
            SetWaitCursor(0);
            
            break;
        case EVENT_RIGHT_CLICK:

            /*
            * Help Message for given widget 
            */
            MessagePopup("HELP: Generator Modulation Level 1", "This field shows the output modulation level of the Generator.");
            
            break;
        }
    return 0;
}

/*****************************************************************************
* Function: mod_type_1                                  
* Purpose:  This is the function sets the modulation type for the           
*           generator's modulator 1.                    
*****************************************************************************/
int CVICALLBACK mod_type_1 (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    ViInt16 status;
    ViInt16 type;
    
    switch (event)
        {
        case EVENT_COMMIT:
            /*
            * Tells user that the UI is communicating with the box 
            */
            SetWaitCursor(1);
            
            GetCtrlVal (panelHandle[GEN], GEN_mod_type_1, &type);
            
            if (status = ifr2975_Set_Mode_FGEN (handle, 3, type)<0) {
                /*
                * Displays Driver Error 
                */
                MessagePopup("Driver Error", "ifr2975_Set_Mode_FGEN failed !"); 
                
                /*
                * Exits UI 
                */
                abortApplication();
            }

            if (type==0) {
                SetCtrlAttribute (panelHandle[GEN], GEN_mod_freq_1, ATTR_DIMMED, 1);
                SetCtrlAttribute (panelHandle[GEN], GEN_mod_level_1, ATTR_DIMMED, 1);
                SetCtrlAttribute (panelHandle[GEN], GEN_mod_shape_1, ATTR_DIMMED, 1);
            } else {
                SetCtrlAttribute (panelHandle[GEN], GEN_mod_freq_1, ATTR_DIMMED, 0);
                SetCtrlAttribute (panelHandle[GEN], GEN_mod_level_1, ATTR_DIMMED, 0);
                SetCtrlAttribute (panelHandle[GEN], GEN_mod_shape_1, ATTR_DIMMED, 0);
            }

            /*
            * Tells user that the UI is not longer communicating with the box
            */
            SetWaitCursor(0);
            
            break;
        case EVENT_RIGHT_CLICK:

            /*
            * Help Message for given widget 
            */
            MessagePopup("HELP: Generator Modulation Type 1", "This field shows the output modulation type of the Generator.");
            
            break;
        }
    return 0;
}

/*****************************************************************************
* Function: mod_shape_1                                 
* Purpose:  This is the function sets the shape for the generator's        
*           modulator 1.                                
*****************************************************************************/
int CVICALLBACK mod_shape_1 (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    ViInt16 status;
    ViInt16 shape;
    
    switch (event)
        {
        case EVENT_COMMIT:
            /*
            * Tells user that the UI is communicating with the box 
            */
            SetWaitCursor(1);    
            
            GetCtrlVal (panelHandle[GEN], GEN_mod_shape_1, &shape);
            
            if (status = ifr2975_Set_Shape_FGEN (handle, 3, shape)<0) {
                /*
                * Displays Driver Error 
                */
                MessagePopup("Driver Error", "ifr2975_Set_Shape_FGEN failed !"); 
                
                /*
                * Exits UI 
                */
                abortApplication();
            }
            
            /*
            * Tells user that the UI is not longer communicating with the box
            */
            SetWaitCursor(0);
            
            break;
        case EVENT_RIGHT_CLICK:

            /*
            * Help Message for given widget 
            */
            MessagePopup("HELP: Generator Modulation Shape 1", "This field shows the output modulation shape of the Generator.");
            
            break;
        }
    return 0;
}

/*****************************************************************************
* Function: mod_freq_2                                  
* Purpose:  This is the function sets the frequency for the generator's    
*           modulator 2.                                
*****************************************************************************/
int CVICALLBACK mod_freq_2 (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    ViInt16 status;
    ViReal64 mod;
    
    switch (event)
        {
        case EVENT_COMMIT:
            /*
            * Tells user that the UI is communicating with the box 
            */
            SetWaitCursor(1);
            
            GetCtrlVal (panelHandle[GEN], GEN_mod_freq_2, &mod);
            
            if (status = ifr2975_Set_Freq_FGEN (handle, 4, mod)<0) {
                /*
                * Displays Driver Error 
                */
                MessagePopup("Driver Error", "ifr2975_Set_Freq_FGEN failed !"); 
                
                /*
                * Exits UI 
                */
                abortApplication();
            }
            
            /*
            * Tells user that the UI is not longer communicating with the box
            */
            SetWaitCursor(0);
            
            break;
        case EVENT_RIGHT_CLICK:

            /*
            * Help Message for given widget 
            */
            MessagePopup("HELP: Generator Modulation Frequency 2", "This field shows the output modulation frequency of the Generator.");
            
            break;
        }
    return 0;
}

/*****************************************************************************
* Function: mod_level_2                                 
* Purpose:  This is the function sets the level for the generator's        
*           modulator 2.                                
*****************************************************************************/
int CVICALLBACK mod_level_2 (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    ViInt16 status;
    ViReal64 dev;
    
    switch (event)
        {
        case EVENT_COMMIT:
            /*
            * Tells user that the UI is communicating with the box 
            */
            SetWaitCursor(1);
            
            GetCtrlVal (panelHandle[GEN], GEN_mod_level_2, &dev);
            dev = dev*2;
            if (status = ifr2975_Set_Per_FGEN (handle, 4, dev)<0) {
                /*
                * Displays Driver Error 
                */
                MessagePopup("Driver Error", "ifr2975_Set_Dev_FGEN failed !"); 
                
                /*
                * Exits UI 
                */
                abortApplication();
            }
            
            /*
            * Tells user that the UI is not longer communicating with the box
            */
            SetWaitCursor(0);
            
            break;
        case EVENT_RIGHT_CLICK:

            /*
            * Help Message for given widget 
            */
            MessagePopup("HELP: Generator Modulation Level 2", "This field shows the output modulation level of the Generator.");
            
            break;
        }
    return 0;
}

/*****************************************************************************
* Function: mod_type_2                                  
* Purpose:  This is the function sets the modulation type for the             
*           generator's modulator 2.                    
*****************************************************************************/
int CVICALLBACK mod_type_2 (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    ViInt16 status;
    ViInt16 type;
    
    switch (event)
        {
        case EVENT_COMMIT:
            /*
            * Tells user that the UI is communicating with the box 
            */
            SetWaitCursor(1);
            
            GetCtrlVal (panelHandle[GEN], GEN_mod_type_2, &type);
            
            if (status = ifr2975_Set_Mode_FGEN (handle, 4, type)<0) {
                /*
                * Displays Driver Error 
                */
                MessagePopup("Driver Error", "ifr2975_Set_Mode_FGEN failed !"); 
                
                /*
                * Exits UI 
                */
                abortApplication();
            }
            
            if (type==0) {
                SetCtrlAttribute (panelHandle[GEN], GEN_mod_freq_2, ATTR_DIMMED, 1);
                SetCtrlAttribute (panelHandle[GEN], GEN_mod_level_2, ATTR_DIMMED, 1);
                SetCtrlAttribute (panelHandle[GEN], GEN_mod_shape_2, ATTR_DIMMED, 1);
            } else {
                SetCtrlAttribute (panelHandle[GEN], GEN_mod_freq_2, ATTR_DIMMED, 0);
                SetCtrlAttribute (panelHandle[GEN], GEN_mod_level_2, ATTR_DIMMED, 0);
                SetCtrlAttribute (panelHandle[GEN], GEN_mod_shape_2, ATTR_DIMMED, 0);
            }
            
            /*
            * Tells user that the UI is not longer communicating with the box
            */
            SetWaitCursor(0);

            break;
        case EVENT_RIGHT_CLICK:
            
            /*
            * Help Message for given widget 
            */
            MessagePopup("HELP: Generator Modulation Type 2", "This field shows the output modulation type of the Generator.");
            
            break;
        }
    return 0;
}

/*****************************************************************************
* Function: mod_shape_2                                 
* Purpose:  This is the function sets the shape for the generator's        
*           modulator 2.                                
*****************************************************************************/
int CVICALLBACK mod_shape_2 (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    ViInt16 status;
    ViInt16 shape;
    
    switch (event)
        {
        case EVENT_COMMIT:
            /*
            * Tells user that the UI is communicating with the box 
            */
            SetWaitCursor(1);
            
            GetCtrlVal (panelHandle[GEN], GEN_mod_shape_2, &shape);
            
            if (status = ifr2975_Set_Shape_FGEN (handle, 4, shape)<0) {
                /*
                * Displays Driver Error 
                */
                MessagePopup("Driver Error", "ifr2975_Set_Shape_FGEN failed !"); 
                
                /*
                * Exits UI 
                */
                abortApplication();
            }
            
            /*
            * Tells user that the UI is not longer communicating with the box
            */
            SetWaitCursor(0);

            break;
        case EVENT_RIGHT_CLICK:

            /*
            * Help Message for given widget 
            */
            MessagePopup("HELP: Generator Modulation Shape 2", "This field shows the output modulation shape of the Generator.");
            
            break;
        }
    return 0;
}


/*****************************************************************************
* Function: sweep                                       
* Purpose:  This is the function sets the sweep speed for the spectrum     
*           analyzer.                                   
*****************************************************************************/
int CVICALLBACK sweep (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    ViReal64 sweep_speed;
    ViInt16 status;
    
    switch (event)
        {
        case EVENT_COMMIT:
            /*
            * Tells user that the UI is communicating with the box 
            */
            SetWaitCursor(1);
            
            GetCtrlVal (panelHandle[SA], SA_sweep, &sweep_speed);
            
            if (status = ifr2975_Set_Sweep_SA (handle, sweep_speed)<0) {
                /*
                * Displays Driver Error 
                */
                MessagePopup("Driver Error", "ifr2975_Set_Sweep_SA failed !"); 
                
                /*
                * Exits UI 
                */
                abortApplication();
            }
            
            /*
            * Tells user that the UI is not longer communicating with the box
            */
            SetWaitCursor(0);
            
            break;
        case EVENT_RIGHT_CLICK:

            /*
            * Help Message for given widget 
            */
            MessagePopup("HELP: Spectrum Analyzer Sweep Speed", "This field shows the sweep speed for the Spectrum Analyzer.");
            
            break;
        }
    return 0;
}

/*****************************************************************************
* Function: TOP_Screen                                  
* Purpose:  This is the function sets the top-of-screen value for the         
*           spectrum analyzer.                          
*****************************************************************************/
int CVICALLBACK TOP_Screen (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    ViReal64 top_screen;
    ViInt16 status;
    
    switch (event)
        {
        case EVENT_COMMIT:
            /*
            * Tells user that the UI is communicating with the box 
            */
            SetWaitCursor(1);
            
            GetCtrlVal (panelHandle[SA], SA_TOP_Screen, &top_screen);
            
            if (status = ifr2975_Set_TOS_SA (handle, top_screen)<0) {
                /*
                * Displays Driver Error 
                */
                MessagePopup("Driver Error", "ifr2975_Set_TOS_SA failed !"); 
                
                /*
                * Exits UI 
                */
                abortApplication();
            }
            
            /*
            * Tells user that the UI is not longer communicating with the box
            */
            SetWaitCursor(0);
            
            break;
        case EVENT_RIGHT_CLICK:
            
            /*
            * Help Message for given widget 
            */
            MessagePopup("HELP: Spectrum Analyzer Top-of-Screen", "This field shows the top-of-screen value for the Spectrum Analyzer.");
            
            break;
        }
    return 0;
}

/*****************************************************************************
* Function: UPDATE_METERS                               
* Purpose:  This is the function updates the meters panel.                  
*                                                       
*****************************************************************************/
int CVICALLBACK UPDATE_METERS (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    ViInt16 status;
    
    switch (event)
        {
        case EVENT_COMMIT:
        
            /*
            * Tells user that the UI is communicating with the box 
            */
            SetWaitCursor(1);
            
            if (status = meter_update()<0) {
                /*
                * Displays Driver Error 
                */
                MessagePopup("Driver Error", "Meter Update failed !"); 
                
                /*
                * Exits UI 
                */
                abortApplication();
            }
            
            /*
            * Tells user that the UI is not longer communicating with the box
            */
            SetWaitCursor(0);
            
            break;
        case EVENT_RIGHT_CLICK:

            break;
        }
    return 0;
}

/*****************************************************************************
* Function: UPDATE_FGEN                                 
* Purpose:  This is the function updates the function generators panel.    
*                                                       
*****************************************************************************/
int CVICALLBACK UPDATE_FGEN (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    ViInt16 status;
    
    switch (event)
        {
        case EVENT_COMMIT:
            SetCtrlAttribute (panelHandle[GEN], GEN_UPDATE_GEN, ATTR_DIMMED, 1);
            SetCtrlAttribute (panelHandle[SCOPE], SCOPE_UPDATE_SCOPE, ATTR_DIMMED, 1);
            //SetCtrlAttribute (panelHandle[FGEN], FGEN_UPDATE_FGEN, ATTR_DIMMED, 1);
            SetCtrlAttribute (panelHandle[SA], SA_UPDATE_SA, ATTR_DIMMED, 1);
            SetCtrlAttribute (panelHandle[METER], METER_UPDATE_METERS, ATTR_DIMMED, 1);
            SetCtrlAttribute (panelHandle[RECEIVER], RECEIVER_UPDATE_REC, ATTR_DIMMED, 1);
            SetCtrlAttribute (panelHandle[PANEL], PANEL_DEFAULTS_UI, ATTR_DIMMED, 1);
            SetCtrlAttribute (panelHandle[PANEL], PANEL_EXIT, ATTR_DIMMED, 1);
            SetCtrlAttribute (panelHandle[PANEL], PANEL_UPDATE_GO, ATTR_DIMMED, 1);
            SetCtrlAttribute (panelHandle[PANEL], PANEL_UPDATE_UI, ATTR_DIMMED, 1);
        
            /*
            * Tells user that the UI is communicating with the box 
            */
            SetWaitCursor(1);
            
            if (status = fgen_update()<0) {
                /*
                * Displays Driver Error 
                */
                MessagePopup("Driver Error", "Function Generator Update failed !"); 
                
                /*
                * Exits UI 
                */
                abortApplication();
            }
            
            /*
            * Tells user that the UI is not longer communicating with the box
            */
            SetWaitCursor(0);

            SetCtrlAttribute (panelHandle[GEN], GEN_UPDATE_GEN, ATTR_DIMMED, 0);
            SetCtrlAttribute (panelHandle[SCOPE], SCOPE_UPDATE_SCOPE, ATTR_DIMMED, 0);
            //SetCtrlAttribute (panelHandle[FGEN], FGEN_UPDATE_FGEN, ATTR_DIMMED, 0);
            SetCtrlAttribute (panelHandle[SA], SA_UPDATE_SA, ATTR_DIMMED, 0);
            SetCtrlAttribute (panelHandle[METER], METER_UPDATE_METERS, ATTR_DIMMED, 0);
            SetCtrlAttribute (panelHandle[RECEIVER], RECEIVER_UPDATE_REC, ATTR_DIMMED, 0);
            SetCtrlAttribute (panelHandle[PANEL], PANEL_DEFAULTS_UI, ATTR_DIMMED, 0);
            SetCtrlAttribute (panelHandle[PANEL], PANEL_EXIT, ATTR_DIMMED, 0);
            SetCtrlAttribute (panelHandle[PANEL], PANEL_UPDATE_GO, ATTR_DIMMED, 0);
            SetCtrlAttribute (panelHandle[PANEL], PANEL_UPDATE_UI, ATTR_DIMMED, 0);
            
            break;
        case EVENT_RIGHT_CLICK:

            break;
        }
    return 0;
}

/*****************************************************************************
* Function: UPDATE_GEN                                  
* Purpose:  This is the function updates the generators panel.             
*                                                       
*****************************************************************************/
int CVICALLBACK UPDATE_GEN (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    ViInt16 status;
    
    switch (event)
        {
        case EVENT_COMMIT:
            //SetCtrlAttribute (panelHandle[GEN], GEN_UPDATE_GEN, ATTR_DIMMED, 1);
            SetCtrlAttribute (panelHandle[SCOPE], SCOPE_UPDATE_SCOPE, ATTR_DIMMED, 1);
            SetCtrlAttribute (panelHandle[FGEN], FGEN_UPDATE_FGEN, ATTR_DIMMED, 1);
            SetCtrlAttribute (panelHandle[SA], SA_UPDATE_SA, ATTR_DIMMED, 1);
            SetCtrlAttribute (panelHandle[METER], METER_UPDATE_METERS, ATTR_DIMMED, 1);
            SetCtrlAttribute (panelHandle[RECEIVER], RECEIVER_UPDATE_REC, ATTR_DIMMED, 1);
            SetCtrlAttribute (panelHandle[PANEL], PANEL_DEFAULTS_UI, ATTR_DIMMED, 1);
            SetCtrlAttribute (panelHandle[PANEL], PANEL_EXIT, ATTR_DIMMED, 1);
            SetCtrlAttribute (panelHandle[PANEL], PANEL_UPDATE_GO, ATTR_DIMMED, 1);
            SetCtrlAttribute (panelHandle[PANEL], PANEL_UPDATE_UI, ATTR_DIMMED, 1);
        
            /*
            * Tells user that the UI is communicating with the box 
            */
            SetWaitCursor(1);
            
            if (status = generator_update()<0) {
                /*
                * Displays Driver Error 
                */
                MessagePopup("Driver Error", "Generator Update failed !"); 
                
                /*
                * Exits UI 
                */
                abortApplication();
            }
            
            /*
            * Tells user that the UI is not longer communicating with the box
            */
            SetWaitCursor(0);

            //SetCtrlAttribute (panelHandle[GEN], GEN_UPDATE_GEN, ATTR_DIMMED, 0);
            SetCtrlAttribute (panelHandle[SCOPE], SCOPE_UPDATE_SCOPE, ATTR_DIMMED, 0);
            SetCtrlAttribute (panelHandle[FGEN], FGEN_UPDATE_FGEN, ATTR_DIMMED, 0);
            SetCtrlAttribute (panelHandle[SA], SA_UPDATE_SA, ATTR_DIMMED, 0);
            SetCtrlAttribute (panelHandle[METER], METER_UPDATE_METERS, ATTR_DIMMED, 0);
            SetCtrlAttribute (panelHandle[RECEIVER], RECEIVER_UPDATE_REC, ATTR_DIMMED, 0);
            SetCtrlAttribute (panelHandle[PANEL], PANEL_DEFAULTS_UI, ATTR_DIMMED, 0);
            SetCtrlAttribute (panelHandle[PANEL], PANEL_EXIT, ATTR_DIMMED, 0);
            SetCtrlAttribute (panelHandle[PANEL], PANEL_UPDATE_GO, ATTR_DIMMED, 0);
            SetCtrlAttribute (panelHandle[PANEL], PANEL_UPDATE_UI, ATTR_DIMMED, 0);
            
            break;
        case EVENT_RIGHT_CLICK:

            break;
        }
    return 0;
}

/*****************************************************************************
* Function: UPDATE_GEN                                  
* Purpose:  This is the function updates the receivers panel.              
*                                                       
*****************************************************************************/
int CVICALLBACK UPDATE_REC (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    ViInt16 status;
    
    switch (event)
        {
        case EVENT_COMMIT:
			SetCtrlAttribute (panelHandle[GEN], GEN_UPDATE_GEN, ATTR_DIMMED, 1);
			SetCtrlAttribute (panelHandle[SCOPE], SCOPE_UPDATE_SCOPE, ATTR_DIMMED, 1);
            SetCtrlAttribute (panelHandle[FGEN], FGEN_UPDATE_FGEN, ATTR_DIMMED, 1);
            SetCtrlAttribute (panelHandle[SA], SA_UPDATE_SA, ATTR_DIMMED, 1);
            SetCtrlAttribute (panelHandle[METER], METER_UPDATE_METERS, ATTR_DIMMED, 1);
            //SetCtrlAttribute (panelHandle[RECEIVER], RECEIVER_UPDATE_REC, ATTR_DIMMED, 1);
            SetCtrlAttribute (panelHandle[PANEL], PANEL_DEFAULTS_UI, ATTR_DIMMED, 1);
            SetCtrlAttribute (panelHandle[PANEL], PANEL_EXIT, ATTR_DIMMED, 1);
            SetCtrlAttribute (panelHandle[PANEL], PANEL_UPDATE_GO, ATTR_DIMMED, 1);
            SetCtrlAttribute (panelHandle[PANEL], PANEL_UPDATE_UI, ATTR_DIMMED, 1);

            /*
            * Tells user that the UI is communicating with the box 
            */
            SetWaitCursor(1);
            
            if (status = receiver_update()<0) {
                /*
                * Displays Driver Error 
                */
                MessagePopup("Driver Error", "Receiver Update failed !"); 
                
                /*
                * Exits UI 
                */
                abortApplication();
            }
            
            /*
            * Tells user that the UI is not longer communicating with the box
            */
            SetWaitCursor(0);

            SetCtrlAttribute (panelHandle[GEN], GEN_UPDATE_GEN, ATTR_DIMMED, 0);
            SetCtrlAttribute (panelHandle[SCOPE], SCOPE_UPDATE_SCOPE, ATTR_DIMMED, 0);
            SetCtrlAttribute (panelHandle[FGEN], FGEN_UPDATE_FGEN, ATTR_DIMMED, 0);
            SetCtrlAttribute (panelHandle[SA], SA_UPDATE_SA, ATTR_DIMMED, 0);
            SetCtrlAttribute (panelHandle[METER], METER_UPDATE_METERS, ATTR_DIMMED, 0);
            //SetCtrlAttribute (panelHandle[RECEIVER], RECEIVER_UPDATE_REC, ATTR_DIMMED, 0);
            SetCtrlAttribute (panelHandle[PANEL], PANEL_DEFAULTS_UI, ATTR_DIMMED, 0);
            SetCtrlAttribute (panelHandle[PANEL], PANEL_EXIT, ATTR_DIMMED, 0);
            SetCtrlAttribute (panelHandle[PANEL], PANEL_UPDATE_GO, ATTR_DIMMED, 0);
            SetCtrlAttribute (panelHandle[PANEL], PANEL_UPDATE_UI, ATTR_DIMMED, 0);
            
            break;
        case EVENT_RIGHT_CLICK:

            break;
        }
    return 0;
}

/*****************************************************************************
* Function: UPDATE_SCOPE                                
* Purpose:  This is the function updates the scope panel.                  
*                                                       
*****************************************************************************/
int CVICALLBACK UPDATE_SCOPE (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    ViInt16 status;
    
    switch (event)
        {
        case EVENT_COMMIT:
            SetCtrlAttribute (panelHandle[GEN], GEN_UPDATE_GEN, ATTR_DIMMED, 1);
            //SetCtrlAttribute (panelHandle[SCOPE], SCOPE_UPDATE_SCOPE, ATTR_DIMMED, 1);
            SetCtrlAttribute (panelHandle[FGEN], FGEN_UPDATE_FGEN, ATTR_DIMMED, 1);
            SetCtrlAttribute (panelHandle[SA], SA_UPDATE_SA, ATTR_DIMMED, 1);
            SetCtrlAttribute (panelHandle[METER], METER_UPDATE_METERS, ATTR_DIMMED, 1);
            SetCtrlAttribute (panelHandle[RECEIVER], RECEIVER_UPDATE_REC, ATTR_DIMMED, 1);
            SetCtrlAttribute (panelHandle[PANEL], PANEL_DEFAULTS_UI, ATTR_DIMMED, 1);
            SetCtrlAttribute (panelHandle[PANEL], PANEL_EXIT, ATTR_DIMMED, 1);
            SetCtrlAttribute (panelHandle[PANEL], PANEL_UPDATE_GO, ATTR_DIMMED, 1);
            SetCtrlAttribute (panelHandle[PANEL], PANEL_UPDATE_UI, ATTR_DIMMED, 1);
        
            /*
            * Tells user that the UI is communicating with the box 
            */
            SetWaitCursor(1);
            
            if (status = scope_update()<0) {
                /*
                * Displays Driver Error 
                */
                MessagePopup("Driver Error", "Scope Update failed !"); 
                
                /*
                * Exits UI 
                */
                abortApplication();
            }
            
            /*
            * Tells user that the UI is not longer communicating with the box
            */
            SetWaitCursor(0);

            SetCtrlAttribute (panelHandle[GEN], GEN_UPDATE_GEN, ATTR_DIMMED, 0);
            //SetCtrlAttribute (panelHandle[SCOPE], SCOPE_UPDATE_SCOPE, ATTR_DIMMED, 0);
            SetCtrlAttribute (panelHandle[FGEN], FGEN_UPDATE_FGEN, ATTR_DIMMED, 0);
            SetCtrlAttribute (panelHandle[SA], SA_UPDATE_SA, ATTR_DIMMED, 0);
            SetCtrlAttribute (panelHandle[METER], METER_UPDATE_METERS, ATTR_DIMMED, 0);
            SetCtrlAttribute (panelHandle[RECEIVER], RECEIVER_UPDATE_REC, ATTR_DIMMED, 0);
            SetCtrlAttribute (panelHandle[PANEL], PANEL_DEFAULTS_UI, ATTR_DIMMED, 0);
            SetCtrlAttribute (panelHandle[PANEL], PANEL_EXIT, ATTR_DIMMED, 0);
            SetCtrlAttribute (panelHandle[PANEL], PANEL_UPDATE_GO, ATTR_DIMMED, 0);
            SetCtrlAttribute (panelHandle[PANEL], PANEL_UPDATE_UI, ATTR_DIMMED, 0);

            break;
        case EVENT_RIGHT_CLICK:

            break;
        }
    return 0;
}

/*****************************************************************************
* Function: UPDATE_SA                                   
* Purpose:  This is the function updates the spectrum analyzer panel.       
*                                                       
*****************************************************************************/
int CVICALLBACK UPDATE_SA (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    ViInt16 status;
    
    switch (event)
        {
        case EVENT_COMMIT:
			SetCtrlAttribute (panelHandle[GEN], GEN_UPDATE_GEN, ATTR_DIMMED, 1);
			SetCtrlAttribute (panelHandle[SCOPE], SCOPE_UPDATE_SCOPE, ATTR_DIMMED, 1);
            SetCtrlAttribute (panelHandle[FGEN], FGEN_UPDATE_FGEN, ATTR_DIMMED, 1);
            //SetCtrlAttribute (panelHandle[SA], SA_UPDATE_SA, ATTR_DIMMED, 1);
            SetCtrlAttribute (panelHandle[METER], METER_UPDATE_METERS, ATTR_DIMMED, 1);
            SetCtrlAttribute (panelHandle[RECEIVER], RECEIVER_UPDATE_REC, ATTR_DIMMED, 1);
            SetCtrlAttribute (panelHandle[PANEL], PANEL_DEFAULTS_UI, ATTR_DIMMED, 1);
            SetCtrlAttribute (panelHandle[PANEL], PANEL_EXIT, ATTR_DIMMED, 1);
            SetCtrlAttribute (panelHandle[PANEL], PANEL_UPDATE_GO, ATTR_DIMMED, 1);
            SetCtrlAttribute (panelHandle[PANEL], PANEL_UPDATE_UI, ATTR_DIMMED, 1);
        
            /*
            * Tells user that the UI is communicating with the box 
            */
            SetWaitCursor(1);
            
            if (status = sa_update()<0) {
                /*
                * Displays Driver Error 
                */
                MessagePopup("Driver Error", "Spectrum Analyzer Update failed !"); 
                
                /*
                * Exits UI 
                */
                abortApplication();
            }
            
            /*
            * Tells user that the UI is not longer communicating with the box
            */
            SetWaitCursor(0);

            SetCtrlAttribute (panelHandle[GEN], GEN_UPDATE_GEN, ATTR_DIMMED, 0);
            SetCtrlAttribute (panelHandle[SCOPE], SCOPE_UPDATE_SCOPE, ATTR_DIMMED, 0);
            SetCtrlAttribute (panelHandle[FGEN], FGEN_UPDATE_FGEN, ATTR_DIMMED, 0);
            //SetCtrlAttribute (panelHandle[SA], SA_UPDATE_SA, ATTR_DIMMED, 0);
            SetCtrlAttribute (panelHandle[METER], METER_UPDATE_METERS, ATTR_DIMMED, 0);
            SetCtrlAttribute (panelHandle[RECEIVER], RECEIVER_UPDATE_REC, ATTR_DIMMED, 0);
            SetCtrlAttribute (panelHandle[PANEL], PANEL_DEFAULTS_UI, ATTR_DIMMED, 0);
            SetCtrlAttribute (panelHandle[PANEL], PANEL_EXIT, ATTR_DIMMED, 0);
            SetCtrlAttribute (panelHandle[PANEL], PANEL_UPDATE_GO, ATTR_DIMMED, 0);
            SetCtrlAttribute (panelHandle[PANEL], PANEL_UPDATE_UI, ATTR_DIMMED, 0);
            
            break;
        case EVENT_RIGHT_CLICK:

            break;
        }
    return 0;
}

/*****************************************************************************
* Function: scope_v_ref_ch1                             
* Purpose:  This is the function sets the scope reference for channel 1.    
*                                                       
*****************************************************************************/
int CVICALLBACK scope_v_ref_ch1 (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    ViInt32 value_scope_v_ref_ch1;
    ViInt16 status;
    
    switch (event)
        {
        case EVENT_COMMIT:
        
            /*
            * Tells user that the UI is communicating with the box 
            */
            SetWaitCursor(1);
            
            GetCtrlVal (panelHandle[SCOPE], SCOPE_scope_v_ref_ch1, &value_scope_v_ref_ch1);
            
            if (status = ifr2975_Set_Input_Coupling_OSCOPE (handle, 1, value_scope_v_ref_ch1)<0) {
                /*
                * Displays Driver Error 
                */
                MessagePopup("Driver Error", "ifr2975_Set_Input_Coupling_OSCOPE failed !"); 
                
                /*
                * Exits UI 
                */
                abortApplication();
            }
            
            /*
            * Tells user that the UI is not longer communicating with the box
            */
            SetWaitCursor(0);
            
            break;
        case EVENT_RIGHT_CLICK:

            /*
            * Help Message for given widget 
            */
            MessagePopup("HELP: Scope Voltage Reference", "This field shows the reference of channel 1.");

            break;
        }
    return 0;
}

/*****************************************************************************
* Function: scope_v_div_ch1                             
* Purpose:  This is the function sets the Volts/Div for channel 1.         
*                                                       
*****************************************************************************/
int CVICALLBACK scope_v_div_ch1 (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    ViInt32 value_scope_v_div_ch1;
    ViInt16 status;
    switch (event)
        {
        case EVENT_COMMIT:
        
            /*
            * Tells user that the UI is communicating with the box 
            */
            SetWaitCursor(1);
            
            GetCtrlVal (panelHandle[SCOPE], SCOPE_scope_v_div_ch1, &value_scope_v_div_ch1);
            
            if (status = ifr2975_Set_Vert_Scale_OSCOPE (handle, 1, value_scope_v_div_ch1)<0) {
                /*
                * Displays Driver Error 
                */
                MessagePopup("Driver Error", "ifr2975_Set_Vert_Scale_OSCOPE failed !"); 
                
                /*
                * Exits UI 
                */
                abortApplication();
            }
            
            /*
            * Tells user that the UI is not longer communicating with the box
            */
            SetWaitCursor(0);
            
            break;
        case EVENT_RIGHT_CLICK:

            /*
            * Help Message for given widget 
            */
            MessagePopup("HELP: Scope Voltage/Division Channel 1", "This field shows the voltage/division for channel 1.");

            break;
        }
    return 0;
}

/*****************************************************************************
* Function: scope_v_ref_ch2                             
* Purpose:  This is the function sets the voltage reference for channel 2.  
*                                                       
*****************************************************************************/
int CVICALLBACK scope_v_ref_ch2 (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    ViInt32 value_scope_v_ref_ch2;
    ViInt16 status;
    switch (event)
        {
        case EVENT_COMMIT:
            /*
            * Tells user that the UI is communicating with the box 
            */
            SetWaitCursor(1);
            
            GetCtrlVal (panelHandle[SCOPE], SCOPE_scope_v_ref_ch2, &value_scope_v_ref_ch2);
            
            if (status = ifr2975_Set_Input_Coupling_OSCOPE (handle, 2, value_scope_v_ref_ch2)<0) {
                /*
                * Displays Driver Error 
                */
                MessagePopup("Driver Error", "ifr2975_Set_Input_Coupling_OSCOPE failed !"); 
                
                /*
                * Exits UI 
                */
                abortApplication();
            }
            
            /*
            * Tells user that the UI is not longer communicating with the box
            */
            SetWaitCursor(0);
            
            break;
        case EVENT_RIGHT_CLICK:

            /*
            * Help Message for given widget 
            */
            MessagePopup("HELP: Scope Voltage Reference", "This field shows the reference of channel 2.");

            break;
        }
    return 0;
}

/*****************************************************************************
* Function: scope_v_div_ch2                             
* Purpose:  This is the function sets the volts/divsion for channel 2.     
*                                                       
*****************************************************************************/
int CVICALLBACK scope_v_div_ch2 (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    ViInt32 value_scope_v_div_ch2;
    ViInt16 status;

    switch (event)
        {
        case EVENT_COMMIT:
        
            /*
            * Tells user that the UI is communicating with the box 
            */
            SetWaitCursor(1);
            
            GetCtrlVal (panelHandle[SCOPE], SCOPE_scope_v_div_ch2, &value_scope_v_div_ch2);
            
            if (status = ifr2975_Set_Vert_Scale_OSCOPE (handle, 2, value_scope_v_div_ch2)<0) {
                /*
                * Displays Driver Error 
                */
                MessagePopup("Driver Error", "ifr2975_Set_Vert_Scale_OSCOPE failed !"); 
                
                /*
                * Exits UI 
                */
                abortApplication();
            }
            
            /*
            * Tells user that the UI is not longer communicating with the box
            */
            SetWaitCursor(0);
            
            break;
        case EVENT_RIGHT_CLICK:

            /*
            * Help Message for given widget 
            */
            MessagePopup("HELP: Scope Voltage/Division Channel 2", "This field shows the voltage/division for channel 2.");

            break;
        }
    return 0;
}

/*****************************************************************************
* Function: abortApplication                            
* Purpose:  This is the function exits the UI upon command failure.         
*                                                       
*****************************************************************************/
int abortApplication (void) {
    
    /*
    * Disconnects all connections to 2975 
    */
    ifr2975_Close_Conn (handle);
    
    /*
    * Displays Driver Error 
    */
    MessagePopup("Driver Error", "An Error has occured.  \n\nThe driver probably lost connection.");
    
    exit (EXIT_FAILURE);
    
    return 0;
}


/*****************************************************************************
* Function: vert_scale                                  
* Purpose:  This is the function sets units/division of the Spectrum       
*           Analyzer.                                   
*****************************************************************************/
int CVICALLBACK vert_scale (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    int vert_scale;
    
    switch (event)
        {
        case EVENT_COMMIT:
            /*
            * Tells user that the UI is communicating with the box 
            */
            SetWaitCursor(1);
            
            GetCtrlVal (panelHandle[SA], SA_vert_scale, &vert_scale);
            
            ifr2975_Set_Vert_Scale_SA (handle, vert_scale);
            
            /*
            * Tells user that the UI is not longer communicating with the box
            */
            SetWaitCursor(0);
            
            break;
        case EVENT_RIGHT_CLICK:

            /*
            * Help Message for given widget 
            */
            MessagePopup("HELP: Spectrum Analyzer Vertical Scale", "This field shows the vertical scale value for the Spectrum Analyzer on the 2975 UI.");
            
            break;
        }
    return 0;
}


/*****************************************************************************
* Function: Receiver_FILTER                             
* Purpose:  This is the function sets the filters on the Receiver.         
*                                                       
*****************************************************************************/
int CVICALLBACK Receiver_FILTER (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    ViInt32 filter;
    ViInt32 status;
    
    switch (event)
        {
        case EVENT_COMMIT:
            /*
            * Tells user that the UI is communicating with the box 
            */
            SetWaitCursor(1);
            
            GetCtrlVal (panelHandle[RECEIVER], RECEIVER_Receiver_FILTER, &filter);
            
            if (status = ifr2975_Set_Filter_REC (handle, filter)<0) {
                /*
                * Displays Driver Error 
                */
                MessagePopup("Driver Error", "ifr2975_Set_Filter_REC failed !"); 
                
                /*
                * Exits UI 
                */
                abortApplication();
            }
            
            /*
            * Tells user that the UI is not longer communicating with the box
            */
            SetWaitCursor(0);

            break;
        case EVENT_RIGHT_CLICK:
            
            /*
            * Help Message for given widget 
            */
            MessagePopup("HELP: Receiver Filter", "This field shows the input filter of the Receiver.");
            
            break;
        }
    return 0;
}


/*****************************************************************************
* Function: scope_div_cvi                               
* Purpose:  This is the function sets the volts/division for the scope.    
*                                                       
*****************************************************************************/
int CVICALLBACK scope_div_cvi (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    ViReal64 div1;
    ViReal64 value1;
    ViReal64 value2;
    
    switch (event)
        {
        case EVENT_COMMIT:
            
            GetCtrlVal (panelHandle[SCOPE], SCOPE_scope_div_cvi, &div1);
            
            /*
            * Determines the max and min values of the Scope 
            */
            value1=div1*10/2;
            value2=-1*div1*10/2;
            
            /*
            * Sets up the Scopes Y axis Scale 
            */
            SetAxisRange (panelHandle[SCOPE], SCOPE_scope_trace, VAL_NO_CHANGE,-1.0, 1.0, VAL_MANUAL, value2, value1);
            
            break;
        case EVENT_RIGHT_CLICK:

            /*
            * Help Message for given widget 
            */
            MessagePopup("HELP: Scope Voltage/Division Driver UI", "This field shows the voltage/division for the driver UI.");
            
            break;
        }
    return 0;
}

/*****************************************************************************
* Function: Receiver_IF                                 
* Purpose:  This is the function sets the IF filters for the receiver.     
*                                                       
*****************************************************************************/
int CVICALLBACK Receiver_IF (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event)
        {
        case EVENT_COMMIT:
            /*
            * Not implimented yet 
            */
            break;
        case EVENT_RIGHT_CLICK:
            
            /*
            * Help Message for given widget 
            */
            MessagePopup("HELP: Receiver IF Bandwidth", "This field shows the input IF Bandwidth of the Receiver.");
            
            break;
        }
    return 0;
}
