#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C) 2003      DESIGN ENGINEERING        #
# --------------------------------------------------------------------------#
# FILE        : BIN19200.tcl                                                #
# PRODUCT     : FITS                                                        #
# AUTHOR      : David Hagood                                                #
# DESCRIPTION : Define the BIN19200 format source for BER                   #
#############################################################################
namespace eval ::Ber::BIN19200 \
{
   variable Name "Binary 19200 bps"
   variable ShortName "BIN19200"
   variable fd ""
   variable out_fd ""
   
   proc Start { } \
   {
      Debug::Puts ::Ber::BIN19200 1 "Starting"
      variable fd
      variable out_fd
      Stop
      set fd [open /BBAudio/uart RDWR]
      BBAudio uutvoutput 30 -nowait
      fconfigure $fd -mode 19200,N,8,1 -buffering none -translation binary -blocking 0
      set out_fd [P25 symbol receive open]
      fconfigure $out_fd -buffering none -translation binary -blocking 0
      P25 symbol receive format bytes
      P25 decode symsrc 1
      fcopy $fd $out_fd -command ::Ber::BIN19200::Done
      Ber run 1 -nowait
   }
   proc Done { args } \
   {
      Debug::Puts ::Ber::BIN19200 1 "Done: $args"
      Stop
   } 
   
   proc Stop {} \
   {
      Debug::Puts ::Ber::BIN19200 1 "Stopping"
      Ber run 0 -nowait
      variable fd
      variable out_fd
      P25 decode symsrc 0
      P25 symbol receive close
      set out_fd ""
      if {$fd != ""} \
      {
         catch "close $fd"
      }
      set fd ""
   }
   proc Help {} \
   {
      return \
{BIN19200 - Symbols are taken from front panel UUT connector using the
following format:
<ul><li>RS-232 style signaling</li>
<li>19200 bits per second</li>
<li>8 data bits</li>
<li>1 stop bit</li>
<li>No parity</li>
<li>4 symbols per byte, most significant symbol sent first</li>
</ul>
}
   }
   proc Enabled {} { return [IFR_options is_installed RX_BER]}
}


if { [info exists  ::Ber::Sources] } \
{
  if { [lsearch  $::Ber::Sources  "::Ber::BIN19200"] == -1 } \
  {
    lappend ::Ber::Sources ::Ber::BIN19200
  }
} \
else \
{
  lappend ::Ber::Sources  ::Ber::BIN19200
}


