#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C) 2002      DESIGN ENGINEERING        #
# --------------------------------------------------------------------------#
# FILE        : dsc.tcl                                                     #
# PRODUCT     : FITS                                                        #
# AUTHOR      : David Hagood                                                #
# DESCRIPTION : Set up the proceedures and data structures to support DCS   #
#                                                                           #
#############################################################################

namespace eval DCS \
{
  #***************************************************************************
  #
  # Name:        Compose
  #
  # Parameters:  code   - digits to send
  #              invert - invert flag
  #
  # Description: Return list of sequence commands for a DCS code
  #
  #***************************************************************************
  proc Compose { code { invert 0 } } \
  {
     # make sure we interp the code as octal
     set code 0$code
     
     # only accept the bottom 9 bits and set the required octal 4000
     set code [expr ($code & 0x1ff) + 04000]
     set crc  [Golay $code]
     set bits [expr ($crc << 12) | $code]
     
     if { $invert != 0 } \
     {
        set bits [expr (~$bits) & 0x7FFFFF]
     }
     
     return [list $bits 0 DCS 0 ]
  }
  

  #***************************************************************************
  #
  # Name:        Compose_unkey
  #
  # Parameters:
  #
  # Description: Return list of sequence commands for a DCS unkey burst
  #
  #***************************************************************************
  proc Compose_unkey { } \
  {
     return { 134 0 SQUARE 180 }
  }
  

  #***************************************************************************
  #
  # Name:        Golay
  #
  # Parameters:  data - digits to send
  #
  # Description: compute the error correction bits for a code sequence
  #
  #***************************************************************************
  proc Golay { data } \
  {
     set DCSPOLY 0x475
  
     # X11+X10+X6+X5+X4+X2+1  = (1)10001110101 */
  
     set MASK11  0x7FF

     set c $data
     set kbits 12
  
     while {$kbits > 0 } \
     {
        incr kbits -1
        if {[expr $c & 0x800] != 0} \
        {
           set c [expr $c ^ $DCSPOLY]
        }
  
        set c [expr $c << 1]
     }
  
     return [expr ($c >> 1) & $MASK11]
  }

};  # End of DCS namespace


