#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C) 2001      DESIGN ENGINEERING        #
# --------------------------------------------------------------------------#
# FILE        : tone_remote.tcl                                             #
# PRODUCT     : FITS                                                        #
# AUTHOR      : David Hagood                                                #
# DESCRIPTION : Set up the proceedures and data structures to support       #
#               Tone Remote                                                 #
#############################################################################

namespace eval ::ToneRemote \
{
  #***************************************************************************
  #
  # Name:        Compose
  #
  # Parameters:  freq - frequency to send
  #
  # Description: Compose a frequency list to make a tone remote burst
  #
  #***************************************************************************
  proc Compose { freq } \
  {
    variable Timing
    variable Levels
    variable Freqs
    
    set seq {}

    if { $::ToneRemote::GuardDur == "CONT" } \
    {
      lappend seq [list $Freqs(MAX)   0 SINE $Timing(MAX)      "" "$Levels(MAX)      dB"]
      lappend seq [list $freq         0 SINE $Timing(FUNCTION) "" "$Levels(FUNCTION) dB"]
      lappend seq [list $Freqs(GUARD) 0 SINE 0                 "" "$Levels(GUARD)    dB"]
    }
    
    if { $::ToneRemote::GuardDur == "BURST" } \
    {
      lappend seq [list $Freqs(MAX)   0 SINE $Timing(MAX)      "" "$Levels(MAX)      dB"]
      lappend seq [list $freq         0 SINE $Timing(FUNCTION) "" "$Levels(FUNCTION) dB"]
      lappend seq [list $Freqs(GUARD) 0 SINE $Timing(GUARD)    "" "$Levels(GUARD)    dB"]
    }

    return $seq
  }
  
};  # End of ToneRemote namespace



