#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C) 2001      DESIGN ENGINEERING        #
# --------------------------------------------------------------------------#
# FILE        : 56tone.tcl                                                  #
# PRODUCT     : FITS                                                        #
# AUTHOR      : David Hagood                                                #
# DESCRIPTION : 56TONE tone signaling package                               #
#############################################################################

package require Tone_Signaling

namespace eval 56TONE \
{
  #**************************************************************************
  #
  # Name:        Init
  #
  # Parameters:
  #
  # Description: Loads the namespace
  #
  #**************************************************************************
  proc Init { } \
  {
    return
  }


  #**************************************************************************
  #
  # Name:        Version
  #
  # Parameters:
  #
  # Description: Returns the software version
  #
  #**************************************************************************
  proc Version { } \
  { 
    return "1.0" 
  }


  #**************************************************************************
  #
  # Name:        Compose
  #
  # Parameters:
  #
  # Description: Composes the sequence
  #
  #**************************************************************************
  proc Compose { args } \
  {
    return [Tone_Signaling::Parse_core  56TONE::Parse_string  $args]
  }


  #**************************************************************************
  #
  # Name:        Parse_string
  #
  # Parameters:
  #
  # Description: This routine parses tone string
  #
  #**************************************************************************
  proc Parse_string { string } \
  {
    variable tones
    variable mark_timing
    variable space_timing
    variable MaxTones 5
    
    variable ValidChars
    variable timing
    
    set result {}
    
    # Set up the begining silence
    if { [info exists timing(BEGIN)] } \
    {
      lappend result [Tone_Signaling::Tone  0  $timing(BEGIN)]
    }
          
    set char  ""
    
    set string [string toupper $string]
    
    # Clip to the max tones allowed
    set max  [string length $string]
    
    if { $max > $MaxTones } \
    {
      set max  $MaxTones
    }
    
    for { set i 0 } { $i < $max } { incr i } \
    {
      set new_char [string index $string $i]

      if { $new_char == "A" } \
      {
        # 'A' is invalid
        continue
      }

      if { $new_char == $char } \
      {
        # repeated digit - send an R
        set new_char  R
      }

      set freq  $tones($new_char)
      
      set char  $new_char

      # 2nd tone of 5/6 sequence is a gap of 45 mS
      if { $i == 1 } \
      {
        lappend result [Tone_Signaling::Tone  0  45]
      }

      # 1st tone is 630 mS
      # tones 6-10 are 52 mS
      if { $i == 0 } \
      {
        lappend result [Tone_Signaling::Tone  $freq  630]
      } \
      elseif { $i > 5 } \
      {
        lappend result [Tone_Signaling::Tone  $freq  52]
      } \
      else \
      {
        lappend result [Tone_Signaling::Tone  $freq  $mark_timing($char)]
      }

      lappend result [Tone_Signaling::Tone  0  $space_timing($char)]
    }

    return $result
  }


};  # End of 56TONE namespace

package provide 56TONE 1.0
