#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C) 2001      DESIGN ENGINEERING        #
# --------------------------------------------------------------------------#
# FILE        : ccirh4.tcl                                                  #
# PRODUCT     : FITS                                                        #
# AUTHOR      : David Hagood                                                #
# DESCRIPTION : CCIRH4 tone signaling package                               #
#############################################################################

package require Tone_Signaling

namespace eval CCIRH4 \
{
  #**************************************************************************
  #
  # Name:        Init
  #
  # Parameters:
  #
  # Description: Loads the namespace
  #
  #**************************************************************************
  proc Init { } \
  {
    return
  }


  #**************************************************************************
  #
  # Name:        Version
  #
  # Parameters:
  #
  # Description: Returns the software version
  #
  #**************************************************************************
  proc Version { } \
  { 
    return "1.0" 
  }


  #**************************************************************************
  #
  # Name:        Compose
  #
  # Parameters:
  #
  # Description: Composes the sequence
  #
  #**************************************************************************
  proc Compose { args } \
  {
    return [Tone_Signaling::Parse_core CCIRH4::Parse_string $args]
  }


  #**************************************************************************
  #
  # Name:        Parse_string
  #
  # Parameters:
  #
  # Description: This routine parses tone string
  #
  #**************************************************************************
  proc Parse_string { string } \
  {
     variable tones
     variable mark_timing
     variable space_timing
     variable MaxTones 5
     variable ValidChars
     variable timing
     
     set result {}
     
     # set up the begining silence
     if { [info exists timing(BEGIN)] } \
     {
       lappend result [Tone_Signaling::Tone  0  $timing(BEGIN)]
     }
           
     set char  ""
     
     # Clip to the max tones allowed
     set max [string length $string]

     if { $max > $MaxTones } \
     {
       set max $MaxTones
     }

     set string  [string toupper $string]

     for { set i 0 } { $i < $max } { incr i } \
     {
       set new_char  [string index $string $i]

       if { $new_char == "A" } \
       {
         # 'A' is invalid
         continue
       }

       if { $new_char == $char } \
       {
         # repeated digit - send an R
         set new_char  R
       }

       set freq  $tones($new_char)
	      
	   set char  $new_char
	
	   # doing 6th ASSIE tone, double duration
	   if { $i == 5 } \
	   {
         lappend result [Tone_Signaling::Tone  $freq  [expr 2*$mark_timing($char)]]
	   } \
	   else \
	   {
         lappend result [Tone_Signaling::Tone  $freq  $mark_timing($char)]
       }

       lappend result [Tone_Signaling::Tone  0  $space_timing($char)]

       # GAP tone before doing 5th tone
	   if { $i == 4 } \
       {
         lappend result [Tone_Signaling::Tone  0  [expr 2*$mark_timing($char)]]
	   }
     }

     return $result
  }


};  # End of CCIRH4 namespace

package provide CCIRH4 1.0
