#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C) 2002      DESIGN ENGINEERING        #
# --------------------------------------------------------------------------#
# FILE        : user.tcl                                                    #
# PRODUCT     : FITS                                                        #
# AUTHOR      : David Hagood                                                #
# DESCRIPTION : User defined tone signaling package                         #
#############################################################################

package require Tone_Signaling

namespace eval User_TS \
{
  #**************************************************************************
  #
  # Name:        Init
  #
  # Parameters:
  #
  # Description: Loads the namespace
  #
  #**************************************************************************
  proc Init { } \
  {
    return
  }


  #**************************************************************************
  #
  # Name:        Version
  #
  # Parameters:
  #
  # Description: Returns the software version
  #
  #**************************************************************************
  proc Version { } \
  { 
    return "1.0" 
  }


  #**************************************************************************
  #
  # Name:        Compose
  #
  # Parameters:
  #
  # Description: Composes the sequence
  #
  #**************************************************************************
  proc Compose { args } \
  {
    #puts "User_TS::Compose:  $args"
    
    set temp  [Tone_Signaling::Parse_core  User_TS::Parse_string  $args]
    
    #puts "temp:  $temp"
    
    return  $temp
  }
  
  
  #**************************************************************************
  #
  # Name:        Parse_string
  #
  # Parameters:
  #
  # Description: This routine parses tone string
  #
  #**************************************************************************
  proc Parse_string { string } \
  {
    #puts "Parse_string:  $string"
     
    variable timing
    variable tones
    variable mark_timing
    variable space_timing
     
    set result {}
     
    # set up the begining silence
    if { [info exists timing(BEGIN)] } \
    {
      lappend result [Tone_Signaling::Tone  0  $timing(BEGIN)]
    }
           
    set char  ""
     
    set string  [string toupper $string]
     
    set max  [string length $string]
    
    for { set i 0 } { $i < $max } { incr i } \
    {
      set char  [string index $string $i]
      
      #puts "char:  $char"
      
      if { [info exists tones($char)] } \
      {
        set freq  $tones($char)
      } \
      else \
      {
        # Undefined tone - generate no signal for a while...
        set freq  0
      }

      if { [info exists mark_timing($char)] } \
      {
        lappend result [Tone_Signaling::Tone  $freq  $mark_timing($char)]
      } \
      else \
      {
        # Undefined MARK timing
        lappend result [Tone_Signaling::Tone  $freq  0]
      }
      

      if { [info exists space_timing($char)] } \
      {
        lappend result [Tone_Signaling::Tone  0  $space_timing($char)]
      } \
      else \
      {
        # Undefined SPACE timing
        lappend result [Tone_Signaling::Tone  0  0]
      }
    }

    #puts "result:  $result"
    
    return $result
  }


  #****************************************************************************
  #
  # Name:        getSaveState
  #
  # Parameters:  none
  #
  # Description: This function returns a list of TCL commands which can be 
  #              used to set the user defined configuration tile to its 
  #              current state
  #
  #****************************************************************************
  proc getSaveState { } \
  {
    variable tones
    variable timing
    variable mark_timing
    variable space_timing
  
    set returnList \
    {
      set ::User_TS::tones(0)    "$tones(0)"
      set ::User_TS::tones(1)    "$tones(1)"
      set ::User_TS::tones(2)    "$tones(2)"
      set ::User_TS::tones(3)    "$tones(3)"
      set ::User_TS::tones(4)    "$tones(4)"
      set ::User_TS::tones(5)    "$tones(5)"
      set ::User_TS::tones(6)    "$tones(6)"
      set ::User_TS::tones(7)    "$tones(7)"
      set ::User_TS::tones(8)    "$tones(8)"
      set ::User_TS::tones(9)    "$tones(9)"
      set ::User_TS::tones(R)    "$tones(R)"
      set ::User_TS::tones(G)    "$tones(G)"
      set ::User_TS::tones(A)    "$tones(A)"

      set ::User_TS::timing(BEGIN_DEF)   "$timing(BEGIN_DEF)"
      set ::User_TS::timing(BEGIN)       "$timing(BEGIN)"

      set ::User_TS::mark_timing(0)    "$mark_timing(0)"
      set ::User_TS::mark_timing(1)    "$mark_timing(1)"
      set ::User_TS::mark_timing(2)    "$mark_timing(2)"
      set ::User_TS::mark_timing(3)    "$mark_timing(3)"
      set ::User_TS::mark_timing(4)    "$mark_timing(4)"
      set ::User_TS::mark_timing(5)    "$mark_timing(5)"
      set ::User_TS::mark_timing(6)    "$mark_timing(6)"
      set ::User_TS::mark_timing(7)    "$mark_timing(7)"
      set ::User_TS::mark_timing(8)    "$mark_timing(8)"
      set ::User_TS::mark_timing(9)    "$mark_timing(9)"
      set ::User_TS::mark_timing(R)    "$mark_timing(R)"
      set ::User_TS::mark_timing(G)    "$mark_timing(G)"
      set ::User_TS::mark_timing(A)    "$mark_timing(A)"

      set ::User_TS::space_timing(0)    "$space_timing(0)"
      set ::User_TS::space_timing(1)    "$space_timing(1)"
      set ::User_TS::space_timing(2)    "$space_timing(2)"
      set ::User_TS::space_timing(3)    "$space_timing(3)"
      set ::User_TS::space_timing(4)    "$space_timing(4)"
      set ::User_TS::space_timing(5)    "$space_timing(5)"
      set ::User_TS::space_timing(6)    "$space_timing(6)"
      set ::User_TS::space_timing(7)    "$space_timing(7)"
      set ::User_TS::space_timing(8)    "$space_timing(8)"
      set ::User_TS::space_timing(9)    "$space_timing(9)"
      set ::User_TS::space_timing(R)    "$space_timing(R)"
      set ::User_TS::space_timing(G)    "$space_timing(G)"
      set ::User_TS::space_timing(A)    "$space_timing(A)"
    }
     
    return [subst -nocommands $returnList]
  }


  #****************************************************************************
  #
  # Name:        Sync
  #
  # Parameters:  none
  #
  # Description: Write state variables for user defined tone signaling
  #
  #****************************************************************************
  proc Sync { } \
  {
    eval [getSaveState]
  }


};  # End of User_TS namespace


#****************************************************************************
# This registers the tiles with the list for saving instrument states
#****************************************************************************
namespace eval ::SavedStates \
{
  variable tiles
  
  lappend tiles User_TS
}


package provide User_TS 1.0

