#
# AUTOEXEC.TCL
#
# kick the file system into "fast" mode during load...
exec make_hda1_fast 1

. config -border 0 -bg white -cursor watch
wm title . IFR-2975
image create photo image_large_logo -file "/system/logo/logo.gif"
label .init_logo -image image_large_logo -bg white
pack .init_logo -fill both -expand true
wm geo . 640x480+0+0
update idle
#
# load Message bus driver, DSP driver and DSP engine
#
foreach driver_file [lsort [glob -nocomplain /system/drivers/*.*]] {
   exec ld 1 0 $driver_file
}


# Add places to search for packages
lappend auto_path /system/interface /system/Tone_signaling /system/Tone_signaling/Tone_Signaling
# Force the load of the Utils commands
catch {load /system/interface/Callback/Callback.out}
catch {load /system/interface/Utils/Utils.out}

# force load of all pkgIndex files in the background
proc preload_pkg { dir } \
{
   set f [file join $dir pkgIndex.tcl]
   if [file exists $f] "BGSource $f"
   set f [file join $dir tclIndex]
   if [file exists $f] "BGSource $f"
   foreach d [glob -nocomplain [file join $dir *]] \
   {
      if [file isdirectory $d] \
      {
         BGCommand "preload_pkg $d"
      }
   }
}
foreach d $auto_path \
{
   BGCommand "preload_pkg $d"
}
update idle

proc at_exit { cmd } \
{
   lappend ::sys(at_exit) $cmd
}
proc cancel_at_exit { cmd } \
{
   set index [lsearch -exact $::sys(at_exit) $cmd]
   if {$index != -1} \
   {
      set ::sys(at_exit) [lreplace $::sys(at_exit) $index $index]
   }
}

CreateLog /debug /usr/logfiles/errorLog 250

namespace eval Debug \
{
   array set enable \
   {
      MASTER 1
   }
   variable log_fd [open "/debug" WRONLY ]
   fconfigure $log_fd -buffering none -translation binary
   
   file delete /usr/errorLog.old
   file delete /usr/errorLog
   
   proc Puts { name level args } \
   {
      variable enable
      variable log_fd
      
      # don't print if master enable is off
      if {$enable(MASTER) == 0 } { return }
      if [info exists enable($name)] \
      {
         # is this level debug enabled?
         if {$enable($name) < $level } { return }
      } \
      else \
      {
         if { 0 < $level } { return }
      }
      if {$enable(MASTER) == "LOG" } \
      {
         puts $log_fd "$name:$level $args"
      }
      puts "$name:$level $args"
   }
   
   if [file exists /system/etc/debug_options.tcl] \
   {
      catch { source /system/etc/debug_options.tcl }
   }
}
interp alias {} debug_puts {} Debug::Puts "debug_puts" 0

# set up standard options
option readfile /system/etc/2975_ui_options

# read any other options
foreach f [glob -nocomplain /system/etc/*.rc] \
{
   puts "reading option file $f"
   option readfile $f
}


if {[option get . debug_shell_enable {}] != "" } \
{
   source /system/debug.tcl
} \
else \
{
 puts "no debugger for you"
}

proc is_vxworks { } {
  global tcl_platform
  if {$tcl_platform(os) == "VxWorks"} {return 1}
  return 0
}

proc isnt_vxworks { } {
  global tcl_platform
  if {$tcl_platform(os) == "VxWorks"} {return 0}
  return 1
}

# Do any needed restart cleanup
if [is_vxworks] {
  catch \
  {
     file delete /usr/reload/reload.tar
     file delete /usr/reload/reload.tar.gz
     file delete /usr/reload/options.new
  
     if [file exists /usr/reload/restart.tcl ] {
       BGSource /usr/reload/restart.tcl
     }
  }
  
}


# force a consistant scaling no matter the screen res
tk scaling [expr 75.0/72]


namespace eval ErrLog \
{
  variable state
  array set State \
  {
     file /usr/errorLog
     enable 1
     print 1
     sep "--------------------------------------------------------"
     err ""
  }
  
  set State(log_fd) [open "/debug" WRONLY ]
  fconfigure $State(log_fd) -buffering none

  
  #*************************************************
  # Name: myCatch
  # Parameters:  command to catch, optional var to catch to
  #Description:  Replacement for built-in catch:
  #              also logs data to a file
  #*************************************************  
  proc myCatch { cmd {var ::ErrLog::State(err)} } \
  {
    # NOTE: because we rename this out of its namespace,
    # we cannot access State normally. We have to
    # use the fully qualified name.
    
    upvar $var err
    if {[set r [oldCatch {uplevel 1 $cmd} err]] == 1} \
    {
       set level [info level ]
       set foo ""
       for {set i 0 } { $i <= $level } { incr i } \
       {
          append foo "\n    invoked from within\n[info level $i]"
       }

       append ::errorInfo $foo
       
       if {$::ErrLog::State(enable) != 0} \
       {
         set fileId $::ErrLog::State(log_fd)
         puts $fileId "Logged at [clock format [clock seconds]]"
         puts $fileId $::ErrLog::State(sep)
         puts $fileId "Error: command was"
         puts $fileId $cmd
         puts $fileId $::ErrLog::State(sep)

         # logger cannot handle huge writes, so break it up
         puts $fileId "errorInfo:\n"
         set foo [split $::errorInfo \n]
         foreach line $foo { puts $fileId $line } 
         puts $fileId $::ErrLog::State(sep)
         puts $fileId "err:\n$err"
         puts $fileId $::ErrLog::State(sep)
         puts $fileId "return code: $r"
       }
       if {$::ErrLog::State(print) != 0} \
       {
         puts $::ErrLog::State(sep)
         puts "Error: command was"
         puts $cmd
         puts $::ErrLog::State(sep)
         puts "errorInfo:\n$::errorInfo"
         puts $::ErrLog::State(sep)
         puts "err:\n$err"
         puts $::ErrLog::State(sep)
         puts "return code: $r"
       }
    }
    return $r
  }
  
 #*************************************************
 # Name: logErrors
 # Parameters:  message    Data to write to the file
 #              restart    if 1, clear log file
 #Description:  write error data to a file
 #*************************************************  
 proc Log { message {restart 0} } {
   variable State

   set fileId $::ErrLog::State(log_fd)
   
   puts $fileId "Logged at [clock format [clock seconds]]"
   puts $fileId $message
   
   if {$State(print) != 0} \
   {
      puts $message
   }
 }
 
 if {"[option get . logErrors {}]" != "" } \
 {
   set State(enable) 1
 }
 Log "Starting Autoexec.tcl [clock format [clock seconds]]" 1
}

 # hide the old catch, and replace it

rename catch oldCatch; rename ErrLog::myCatch catch

# Bring up DHCP system so we can get our address early
package require DHCP
DHCP broadcast

# CHANGE DEFAULT BINDING OF F10 TO AVOID ERROR IF F10 PRESSED
bind all <F10> {}

set LOAD_FLAG     0

# move the mouse out of the way.
package require mouse
mouse . 900 900

#*************************************************
#       Name: load_option
# Parameters: none
#Description: option to launch engineer applications,
#             or load DSPs and engines withou UI, 
#             read config_ui file & look for match
#             command on each line
#*************************************************
proc load_option { } {
     global LOAD_FLAG

   if [catch {open /system/config.ui} fileID] {
      puts stderr "Cannot open /system/config.ui"

   } else {
        while {[gets $fileID line] >= 0} {
           if {[lindex $line 0] == "ENGINEERING_APPLICATION"} {
              set LOAD_FLAG 1
           }

           if {[lindex $line 0] == "HALT"} {
              set LOAD_FLAG 2
              }
           if {[lindex $line 0] == "NEW_UI"} {
              set LOAD_FLAG 3
              }
        }
        close $fileID
     }
}

#*************************************************
# Name: quick_stack_dump
# Parameters: none
# Description: This is a debug function designed 
#     to display the current program stack
#*************************************************
proc quick_stack_dump { } {
   set level [info level ]
   set foo ""
   for {set i 0 } { $i <= $level } { incr i } \
   {
      append foo "\n    invoked from within\n[info level $i]"
   }
   puts "$foo"
}

if {$tcl_platform(os) == "VxWorks"} {
  load_option
}

#
# load DSPs and engine files only, no UI
#
if {$LOAD_FLAG == 2} {
   exec reboot_dsps
   foreach engine_file [lsort [glob -nocomplain /system/engines/*.*]] {
      exec ld 1 0 $engine_file
   }
   return
}


#*************************************************
#       Name: put
# Parameters: text
#Description: put text if debug_out is on
#*************************************************
proc put { text } {
  global debug_print
  
  if {$debug_print} {
    puts $text
  }
}

if {![info exists debug_print]} {
  set debug_print 0
}


# COMMENT NEXT LINE TO PREVENT DEBUG PRINTS
#set debug_print 1

# SET debug_meter TO 1 TO ALLOW DEBUG PRINTS FOR METER READINGS
set debug_meter 0


if {[is_vxworks]} {
  set sys(system_path) /system
  set sys(sysdata_path) /system
} else {
  set sys(system_path) [pwd]
  set sys(sysdata_path) [pwd]
}

proc Please_wait { } {
     toplevel .auto -class STARTUP -width 640 -height 480 -cursor watch
     wm geo .auto 640x480+0+0
     wm override .auto 1
     wm group .auto ""
     pack propagate .auto false
     bindtags .auto "autoONLY .auto STARTUP all"
     bind autoONLY <Map> \
     {
        Please_wait_finish
     }
     wm withdraw .auto
     
     # create our fonts
     set fonts [option get .auto fonts {}]
     foreach font $fonts \
     {
        eval font create $font [option get .auto $font {}]
     }
     pack propagate .auto false

     # TITLE BAR
     label .auto.title

     # Logo and status area
     frame .auto.sub

     # LOGO
     image create photo image1 -file "/system/logo/smalllogo.gif"
     label .auto.sub.pic -image image1
     bind .auto.sub.pic <Destroy> "after idle {image delete image1}"

     # status area 
     frame .auto.sub.status

     # copyright data frame
     frame .auto.copyright
     pack propagate .auto.copyright false
     pack .auto.title   -fill x -side top
     pack .auto.sub     -fill x -side top
     pack .auto.sub.pic    -side right
     pack .auto.sub.status -side left -fill both -expand true
     pack .auto.copyright -fill both -expand true
}

proc Wait_done {  } {
   Debug::Puts Startup 1 "Deiconify ." 
   wm deiconify .
   wm geo . 640x480+0+0
   Debug::Puts Startup 1 "Update" 
   update idle
   after idle "destroy .auto"
}

#*************************************************
#       Name: read_env
# Parameters: 
#Description: reads env file
#*************************************************
proc read_env {} {

  # Check for existence before trying to use file
  if {![file exists /system/etc/master_setup]} {
    return
  }

  catch {
    set fileId [open /system/etc/master_setup r] 
    set err ""
  } err

  if {$err != ""} {
    #puts "Couldn't open file: code was $err"
    return
  }

  if {[catch {set line [read $fileId]} err] } {
    close $fileId
    puts "couldn't read the setup: err was $err"
    return
  }

  if {[catch { array set ::sys $line } err] } {
    close $fileId  
    puts "Couldn't create Array $err"
    return
  }    

  close $fileId
}

read_env

# define stub routines for simulation. If we are real, they will be replaced

proc loadbar_update { name count total color } {
}
proc load_ui_message { which {color black}} {
}
proc setup_load_screen { name } {
}
proc setup_credits { name } {
}
proc wait_for_system {} {
}
proc Please_wait_middle {} \
{
   # set up main window
   ::Load::setup_credits .auto.copyright
   ::Load::setup_load_screen .auto.sub.status
   wm deiconify .auto
   wm geo .auto 640x480+0+0
   update idle
}

proc Please_wait_finish { } \
{
   global LOAD_FLAG
   wm withdraw .
   destroy .init_logo
   update idle

   # load option - normal startup or engineering applications
   if {$LOAD_FLAG == 0} \
   {
     source startup.tcl
     source main.tcl
   } \
   elseif {$LOAD_FLAG == 1} \
   {
      wait_for_system
      Wait_done
      cd /system/eng_appl
      source main.tcl
   }
}


BGCommand Please_wait

BGSource  load_it.tcl

BGCommand Please_wait_middle

