#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C)1998,99      DESIGN ENGINEERING      #
# --------------------------------------------------------------------------#
# FILE        : bbaudio.tcl                                                 #
# PRODUCT     : FITS                                                        #
# AUTHOR      : Steve Johnson                                               #
# DESCRIPTION : DemoDisplay for BaseBand Audio                              #
#                                                                           #
########################################################### ##################

package require BBAudio

#
# default hardware here
#
proc init_bbaudio_hardware { } {
  global HwUtilEngine

  ::BBAudio::clr_bbaudio_routes
  BBAudio speakervolume 0

  set_hwutil_tcxo $HwUtilEngine(TCXO)
  set_hwutil_backlight 2000
  set_hwutil_crypto OFF
  set_hwutil_10mhz_ref INT_REF
  set_default_volume 
}


#
# doHwUtilBacklight
#
proc doHwUtilBacklight { tg } {
  scale $tg.adj   -label "Backlight"  -from 0 -to 4095 \
                  -resolution 100 -length 4c -orient horizontal \
                  -command set_hwutil_backlight -font bbaudio_font

  pack $tg.adj    -side top  -fill both -expand true
}


#
# doBBLvlCal
#
proc doBBLvlCal { tg } {
  global HwUtilEngine
  global doBBLvlCalIndex
  global doBBLvlCalPath

  set doBBLvlCalIndex 1
  set doBBLvlCalPath  $tg

  entry $tg.max   -textvariable  HwUtilEngine(bbcal.max) -width 5 \
                  -font bbaudio_font -bg white
  entry $tg.index -textvariable  doBBLvlCalIndex -width 5 \
                  -font bbaudio_font -bg white
  entry $tg.name  -textvariable  HwUtilEngine(bbcal.$doBBLvlCalIndex.name) \
                  -width 14 -font bbaudio_font -bg white
  entry $tg.cal   -textvariable  HwUtilEngine(bbcal.$doBBLvlCalIndex.cal)  \
                  -width 10  -font bbaudio_font -bg white

  bind $tg.index  <Return> { 
    $doBBLvlCalPath.name  configure \
           -textvariable HwUtilEngine(bbcal.$doBBLvlCalIndex.name)
    $doBBLvlCalPath.cal  configure \
           -textvariable HwUtilEngine(bbcal.$doBBLvlCalIndex.cal)
  }

  bind $tg.name   <Return> { write_mixer_router_cal_data bb_audio_lvl }
  bind $tg.cal    <Return> { write_mixer_router_cal_data bb_audio_lvl }


  pack $tg.max    -side left -fill both -expand true
  pack $tg.index  -side left -fill both -expand true
  pack $tg.name   -side left -fill both -expand true
  pack $tg.cal    -side left -fill both -expand true
}

#
# doHwUtilTcxo
#
proc doHwUtilTcxo { tg } {
  global HwUtilEngine

  label $tg.msg   -text  "Adj TCXO"                   -font bbaudio_font 
  entry $tg.entry -textvariable  HwUtilEngine(TCXO)   -width 8 \
                  -font bbaudio_font -bg white 


  bind $tg.entry <Return> trace_hwutil_tcxo

  pack $tg.msg     -side left -fill both -expand true
  pack $tg.entry   -side left -fill both -expand true
}


#
# doBBaudioUUTDigOUt
#
proc doBBaudioUUTDigOUt { tg } {
  global BBAUDIO_ENGINE

  label $tg.msg  -text "UUT Digital Output" -font bbaudio_font
  checkbutton $tg.ck0 -text "Bit0, Dig Out, (Pin 9)" -font bbaudio_font \
        -command { set_bbaudio_uut_dig_out $UUT_STATE(Bit0) 0x01 } \
        -variable UUT_STATE(Bit0) -onvalue 0 -offvalue 1
  checkbutton $tg.ck1 -text "Bit1, Dig Out, (Pin 10)" -font bbaudio_font \
        -command { set_bbaudio_uut_dig_out $UUT_STATE(Bit1) 0x02 } \
        -variable UUT_STATE(Bit1) -onvalue 0 -offvalue 2
  checkbutton $tg.ck2 -text "Bit2, Dig Out, (Pin 11)" -font bbaudio_font \
        -command { set_bbaudio_uut_dig_out $UUT_STATE(Bit2) 0x04 } \
        -variable UUT_STATE(Bit2) -onvalue 0 -offvalue 4
  checkbutton $tg.ck3 -text "Bit3, Dig Out, (Pin 12)" -font bbaudio_font \
        -command { set_bbaudio_uut_dig_out $UUT_STATE(Bit3) 0x08 } \
        -variable UUT_STATE(Bit3) -onvalue 0 -offvalue 8
  checkbutton $tg.ck4 -text "Bit4, Serial Out, (Pin 13)" -font bbaudio_font \
        -command { set_bbaudio_uut_dig_out $UUT_STATE(Bit4) 0x10 } \
        -variable UUT_STATE(Bit4) -onvalue 0 -offvalue 16
  checkbutton $tg.ck5 -text "Bit5, Vout Disable, (Pin 8)" -font bbaudio_font \
        -command { set_bbaudio_uut_dig_out $UUT_STATE(Bit5) 0x20 } \
        -variable UUT_STATE(Bit5) -onvalue 0 -offvalue 32
  checkbutton $tg.ck6 -text "Bit5, PTT, (Mic Connector)" -font bbaudio_font \
        -command { set_bbaudio_uut_dig_out $UUT_STATE(Bit6) 0x40 } \
        -variable UUT_STATE(Bit6) -onvalue 0 -offvalue 64

  scale $tg.vdc -label "Output Voltage (rel)"  -from 0 -to 99 -font bbaudio_font \
        -orient horizontal -command set_bbaudio_uut_adj_test_voltage
  scale $tg.abs -label "Output Voltage (abs)" -from -99 -to 99 -font bbaudio_font \
        -orient horizontal -command set_bbaudio_uut_adj_test_voltage

  pack $tg.msg     -side top -fill both -expand true
  pack $tg.ck0     -side top -fill both -expand true
  pack $tg.ck1     -side top -fill both -expand true
  pack $tg.ck2     -side top -fill both -expand true
  pack $tg.ck3     -side top -fill both -expand true
  pack $tg.ck4     -side top -fill both -expand true
  pack $tg.ck5     -side top -fill both -expand true
  pack $tg.ck6     -side top -fill both -expand true

  pack $tg.vdc     -side top -fill both -expand true
  pack $tg.abs     -side top -fill both -expand true
}


#
# doBBaudioReadIO
#
proc doBBaudioReadIO { tg } {
  global BBAUDIO_ENGINE

  frame $tg.f1
  frame $tg.f2
  frame $tg.f3
  frame $tg.f4
  frame $tg.f5
  frame $tg.f6

  label $tg.f1.msg -text "Digital 1" -font bbaudio_font
  label $tg.f1.io  -textvariable BBAUDIO_ENGINE(DigIn1) -font bbaudio_font

  label $tg.f2.msg -text "Digital 2" -font bbaudio_font
  label $tg.f2.io  -textvariable BBAUDIO_ENGINE(DigIn2) -font bbaudio_font

  label $tg.f3.msg -text "Digital 3" -font bbaudio_font
  label $tg.f3.io  -textvariable BBAUDIO_ENGINE(DigIn3) -font bbaudio_font

  label $tg.f4.msg -text "Digital 4" -font bbaudio_font
  label $tg.f4.io  -textvariable BBAUDIO_ENGINE(DigIn4) -font bbaudio_font

  label $tg.f5.msg -text "Serial 1" -font bbaudio_font
  label $tg.f5.io  -textvariable BBAUDIO_ENGINE(SerIn1) -font bbaudio_font

  label $tg.f6.msg -text "DSP Sample Rate (mS)" -font bbaudio_font
  entry $tg.f6.entry -textvariable  BBAUDIO_ENGINE(dig_rate) -width 8 \
                  -font bbaudio_font -bg white

  bind $tg.f6.entry <Return> { set_bbaudio_uut_dig_in_auto \
                                 $BBAUDIO_ENGINE(dig_rate)  }

  pack $tg.f1.msg   -side left -fill both -expand true
  pack $tg.f1.io    -side left -fill both -expand true

  pack $tg.f2.msg   -side left -fill both -expand true
  pack $tg.f2.io    -side left -fill both -expand true

  pack $tg.f3.msg   -side left -fill both -expand true
  pack $tg.f3.io    -side left -fill both -expand true

  pack $tg.f4.msg   -side left -fill both -expand true
  pack $tg.f4.io    -side left -fill both -expand true

  pack $tg.f5.msg   -side left -fill both -expand true
  pack $tg.f5.io    -side left -fill both -expand true

  pack $tg.f6.msg   -side left -fill both -expand true
  pack $tg.f6.entry -side left -fill both -expand true

  pack $tg.f1       -side top -fill both -expand true
  pack $tg.f2       -side top -fill both -expand true
  pack $tg.f3       -side top -fill both -expand true
  pack $tg.f4       -side top -fill both -expand true
  pack $tg.f5       -side top -fill both -expand true
  pack $tg.f6       -side top -fill both -expand true
}


#
# doFgenGen1
#
proc doFgenGen1 { tg } {
  global BBFgen

  frame $tg.f0
  label $tg.f0.msg -text "Waveform Shape" -font bbaudio_font
  radiobutton $tg.f0.off   -text "Off" -font bbaudio_font \
                           -command { set_bbfgen_burst_mode 1 $BBFgen(Mode1) } \
                           -variable BBFgen(Mode1) \
                           -value OFF
  radiobutton $tg.f0.cont  -text "Continuous" -font bbaudio_font \
                           -command { set_bbfgen_burst_mode 1 $BBFgen(Mode1) } \
                           -variable BBFgen(Mode1) \
                           -value CONTINUOUS
  radiobutton $tg.f0.burs  -text "Burst" -font bbaudio_font \
                           -command { set_bbfgen_burst_mode 1 $BBFgen(Mode1) } \
                           -variable BBFgen(Mode1) \
                           -value BURST

  frame $tg.f00
  label $tg.f00.msg -text "Duration (1 to 100000 mS)" -font bbaudio_font
  entry $tg.f00.dur -textvariable BBFgen(Durat1) -font bbaudio_font -bg white
  bind $tg.f00.dur <Return> { set_bbfgen_burst_duration 1 $BBFgen(Durat1) }

  frame $tg.f1
  label $tg.f1.msg -text "Freq (0 to 20000)" -font bbaudio_font
  entry $tg.f1.freq -textvariable BBFgen(Freq1) -font bbaudio_font -bg white
  bind $tg.f1.freq <Return> { set_bbfgen_freq 1 $BBFgen(Freq1) }
  
  frame $tg.f2
  label $tg.f2.msg -text "AF lvl (0 to 10.0000 Volts)" -font bbaudio_font
  entry $tg.f2.aflvl -textvariable BBFgen(AFlvl1) -font bbaudio_font -bg white
  bind $tg.f2.aflvl <Return> { set_bbfgen_level 1 $BBFgen(AFlvl1) }

  frame $tg.f4
  label $tg.f4.msg -text "1V == 5 kHz = 16000" -font bbaudio_font
  
  frame $tg.f3
  label $tg.f3.msg -text "Waveform Shape" -font bbaudio_font
  radiobutton $tg.f3.sine  -text "Sine" -font bbaudio_font \
                           -command { set_bbfgen_shape 1 $BBFgen(Shape1) } \
                           -variable BBFgen(Shape1) \
                           -value SINE
  radiobutton $tg.f3.tri   -text "Triangle" -font bbaudio_font \
                           -command { set_bbfgen_shape 1 $BBFgen(Shape1) } \
                           -variable BBFgen(Shape1) \
                           -value TRIANGLE
  radiobutton $tg.f3.squ   -text "Square" -font bbaudio_font \
                           -command { set_bbfgen_shape 1 $BBFgen(Shape1) } \
                           -variable BBFgen(Shape1) \
                           -value SQUARE
  radiobutton $tg.f3.ramp  -text "Ramp" -font bbaudio_font \
                           -command { set_bbfgen_shape 1 $BBFgen(Shape1) } \
                           -variable BBFgen(Shape1) \
                           -value RAMP
  radiobutton $tg.f3.dc    -text "DC" -font bbaudio_font \
                           -command { set_bbfgen_shape 1 $BBFgen(Shape1) } \
                           -variable BBFgen(Shape1) \
                           -value DC
  radiobutton $tg.f3.arb   -text "Arbitrary" -font bbaudio_font \
                           -command { set_bbfgen_shape 1 $BBFgen(Shape1) } \
                           -variable BBFgen(Shape1) \
                           -value ARBITRARY

  pack $tg.f00.msg  -side left  -fill both -expand true
  pack $tg.f00.dur  -side left  -fill both -expand true

  pack $tg.f0.msg   -side top   -fill both -expand true
  pack $tg.f0.off   -side left  -fill both -expand true
  pack $tg.f0.cont  -side left  -fill both -expand true
  pack $tg.f0.burs  -side left  -fill both -expand true

  pack $tg.f1.msg   -side left  -fill both -expand true
  pack $tg.f1.freq  -side left  -fill both -expand true

  pack $tg.f2.msg   -side left  -fill both -expand true
  pack $tg.f2.aflvl -side left  -fill both -expand true

  pack $tg.f4.msg   -side left  -fill both -expand true

  pack $tg.f3.msg   -side top   -fill both -expand true
  pack $tg.f3.sine  -side left  -fill both -expand true
  pack $tg.f3.tri   -side left  -fill both -expand true
  pack $tg.f3.squ   -side left  -fill both -expand true
  pack $tg.f3.ramp  -side left  -fill both -expand true
  pack $tg.f3.dc    -side left  -fill both -expand true
  pack $tg.f3.arb   -side left  -fill both -expand true

  pack $tg.f0       -side top   -fill both -expand true
  pack $tg.f00      -side top   -fill both -expand true
  pack $tg.f1       -side top   -fill both -expand true
  pack $tg.f2       -side top   -fill both -expand true
  pack $tg.f4       -side top   -fill both -expand true
  pack $tg.f3       -side top   -fill both -expand true
}


#
# doFgenGen2
#
proc doFgenGen2 { tg } {
  global BBFgen

  frame $tg.f0
  label $tg.f0.msg -text "Waveform Shape" -font bbaudio_font
  radiobutton $tg.f0.off   -text "Off" -font bbaudio_font \
                           -command { set_bbfgen_burst_mode 2 $BBFgen(Mode2) } \
                           -variable BBFgen(Mode2) \
                           -value OFF
  radiobutton $tg.f0.cont  -text "Continuous" -font bbaudio_font \
                           -command { set_bbfgen_burst_mode 2 $BBFgen(Mode2) } \
                           -variable BBFgen(Mode2) \
                           -value CONTINUOUS
  radiobutton $tg.f0.burs  -text "Burst" -font bbaudio_font \
                           -command { set_bbfgen_burst_mode 2 $BBFgen(Mode2) } \
                           -variable BBFgen(Mode2) \
                           -value BURST

  frame $tg.f00
  label $tg.f00.msg -text "Duration (1 to 100000 mS)" -font bbaudio_font
  entry $tg.f00.dur -textvariable BBFgen(Durat2) -font bbaudio_font -bg white
  bind $tg.f00.dur <Return> { set_bbfgen_burst_duration 2 $BBFgen(Durat2) }


  frame $tg.f1
  label $tg.f1.msg -text "Freq (0 to 20000)" -font bbaudio_font

  entry $tg.f1.freq -textvariable BBFgen(Freq2) -font bbaudio_font -bg white
  bind $tg.f1.freq <Return> { set_bbfgen_freq 2 $BBFgen(Freq2) }

  frame $tg.f2
  label $tg.f2.msg -text "AF lvl (0 to 10.0000 Volts)" -font bbaudio_font
  entry $tg.f2.aflvl -textvariable BBFgen(AFlvl2) -font bbaudio_font -bg white
  bind $tg.f2.aflvl <Return> { set_bbfgen_level 2 $BBFgen(AFlvl2) }

  frame $tg.f4
  label $tg.f4.msg -text "1V == 5 kHz = 16000" -font bbaudio_font

  frame $tg.f3
  label $tg.f3.msg -text "Waveform Shape" -font bbaudio_font
  radiobutton $tg.f3.sine  -text "Sine" -font bbaudio_font \
                           -command { set_bbfgen_shape 2 $BBFgen(Shape2) } \
                           -variable BBFgen(Shape2) \
                           -value SINE
  radiobutton $tg.f3.tri   -text "Triangle" -font bbaudio_font \
                           -command { set_bbfgen_shape 2 $BBFgen(Shape2) } \
                           -variable BBFgen(Shape2) \
                           -value TRIANGLE
  radiobutton $tg.f3.squ   -text "Square" -font bbaudio_font \
                           -command { set_bbfgen_shape 2 $BBFgen(Shape2) } \
                           -variable BBFgen(Shape2) \
                           -value SQUARE
  radiobutton $tg.f3.ramp  -text "Ramp" -font bbaudio_font \
                           -command { set_bbfgen_shape 2 $BBFgen(Shape2) } \
                           -variable BBFgen(Shape2) \
                           -value RAMP
  radiobutton $tg.f3.dc    -text "DC" -font bbaudio_font \
                           -command { set_bbfgen_shape 2 $BBFgen(Shape2) } \
                           -variable BBFgen(Shape2) \
                           -value DC
  radiobutton $tg.f3.arb   -text "Arbitrary" -font bbaudio_font \
                           -command { set_bbfgen_shape 2 $BBFgen(Shape2) } \
                           -variable BBFgen(Shape2) \
                           -value ARBITRARY

  pack $tg.f00.msg  -side left  -fill both -expand true
  pack $tg.f00.dur  -side left  -fill both -expand true

  pack $tg.f0.msg   -side top   -fill both -expand true
  pack $tg.f0.off   -side left  -fill both -expand true
  pack $tg.f0.cont  -side left  -fill both -expand true
  pack $tg.f0.burs  -side left  -fill both -expand true

  pack $tg.f1.msg   -side left  -fill both -expand true
  pack $tg.f1.freq  -side left  -fill both -expand true

  pack $tg.f2.msg   -side left  -fill both -expand true
  pack $tg.f2.aflvl -side left  -fill both -expand true

  pack $tg.f4.msg   -side left  -fill both -expand true

  pack $tg.f3.msg   -side top   -fill both -expand true
  pack $tg.f3.sine  -side left  -fill both -expand true
  pack $tg.f3.tri   -side left  -fill both -expand true
  pack $tg.f3.squ   -side left  -fill both -expand true
  pack $tg.f3.ramp  -side left  -fill both -expand true
  pack $tg.f3.dc    -side left  -fill both -expand true
  pack $tg.f3.arb   -side left  -fill both -expand true

  pack $tg.f0       -side top   -fill both -expand true
  pack $tg.f00      -side top   -fill both -expand true
  pack $tg.f1       -side top   -fill both -expand true
  pack $tg.f2       -side top   -fill both -expand true
  pack $tg.f4       -side top   -fill both -expand true
  pack $tg.f3       -side top   -fill both -expand true
}

#
# doFgenGen3
#
proc doFgenGen3 { tg } {
  global BBFgen

  frame $tg.f0
  label $tg.f0.msg -text "Waveform Shape" -font bbaudio_font
  radiobutton $tg.f0.off   -text "Off" -font bbaudio_font \
                           -command { set_bbfgen_burst_mode 3 $BBFgen(Mode3) } \
                           -variable BBFgen(Mode3) \
                           -value OFF
  radiobutton $tg.f0.cont  -text "Continuous" -font bbaudio_font \
                           -command { set_bbfgen_burst_mode 3 $BBFgen(Mode3) } \
                           -variable BBFgen(Mode3) \
                           -value CONTINUOUS
  radiobutton $tg.f0.burs  -text "Burst" -font bbaudio_font \
                           -command { set_bbfgen_burst_mode 3 $BBFgen(Mode3) } \
                           -variable BBFgen(Mode3) \
                           -value BURST

  frame $tg.f00
  label $tg.f00.msg -text "Duration (1 to 100000 mS)" -font bbaudio_font
  entry $tg.f00.dur -textvariable BBFgen(Durat3) -font bbaudio_font -bg white
  bind $tg.f00.dur <Return> { set_bbfgen_burst_duration 3 $BBFgen(Durat3) }


  frame $tg.f1
  label $tg.f1.msg -text "Freq (0 to 20000)" -font bbaudio_font

  entry $tg.f1.freq -textvariable BBFgen(Freq3) -font bbaudio_font -bg white
  bind $tg.f1.freq <Return> { set_bbfgen_freq 3 $BBFgen(Freq3) }

  frame $tg.f2
  label $tg.f2.msg -text "AF lvl (0 to 10.0000 Volts)" -font bbaudio_font
  entry $tg.f2.aflvl -textvariable BBFgen(AFlvl3) -font bbaudio_font -bg white
  bind $tg.f2.aflvl <Return> { set_bbfgen_level 3 $BBFgen(AFlvl3) }

  frame $tg.f4
  label $tg.f4.msg -text "1V == 5 kHz = 16000" -font bbaudio_font

  frame $tg.f3
  label $tg.f3.msg -text "Waveform Shape" -font bbaudio_font
  radiobutton $tg.f3.sine  -text "Sine" -font bbaudio_font \
                           -command { set_bbfgen_shape 3 $BBFgen(Shape3) } \
                           -variable BBFgen(Shape3) \
                           -value SINE
  radiobutton $tg.f3.tri   -text "Triangle" -font bbaudio_font \
                           -command { set_bbfgen_shape 3 $BBFgen(Shape3) } \
                           -variable BBFgen(Shape3) \
                           -value TRIANGLE
  radiobutton $tg.f3.squ   -text "Square" -font bbaudio_font \
                           -command { set_bbfgen_shape 3 $BBFgen(Shape3) } \
                           -variable BBFgen(Shape3) \
                           -value SQUARE
  radiobutton $tg.f3.ramp  -text "Ramp" -font bbaudio_font \
                           -command { set_bbfgen_shape 3 $BBFgen(Shape3) } \
                           -variable BBFgen(Shape3) \
                           -value RAMP
  radiobutton $tg.f3.dc    -text "DC" -font bbaudio_font \
                           -command { set_bbfgen_shape 3 $BBFgen(Shape3) } \
                           -variable BBFgen(Shape3) \
                           -value DC
  radiobutton $tg.f3.arb   -text "Arbitrary" -font bbaudio_font \
                           -command { set_bbfgen_shape 3 $BBFgen(Shape3) } \
                           -variable BBFgen(Shape3) \
                           -value ARBITRARY

  pack $tg.f00.msg  -side left  -fill both -expand true
  pack $tg.f00.dur  -side left  -fill both -expand true

  pack $tg.f0.msg   -side top   -fill both -expand true
  pack $tg.f0.off   -side left  -fill both -expand true
  pack $tg.f0.cont  -side left  -fill both -expand true
  pack $tg.f0.burs  -side left  -fill both -expand true

  pack $tg.f1.msg   -side left  -fill both -expand true
  pack $tg.f1.freq  -side left  -fill both -expand true

  pack $tg.f2.msg   -side left  -fill both -expand true
  pack $tg.f2.aflvl -side left  -fill both -expand true

  pack $tg.f4.msg   -side left  -fill both -expand true

  pack $tg.f3.msg   -side top   -fill both -expand true
  pack $tg.f3.sine  -side left  -fill both -expand true
  pack $tg.f3.tri   -side left  -fill both -expand true
  pack $tg.f3.squ   -side left  -fill both -expand true
  pack $tg.f3.ramp  -side left  -fill both -expand true
  pack $tg.f3.dc    -side left  -fill both -expand true
  pack $tg.f3.arb   -side left  -fill both -expand true

  pack $tg.f0       -side top   -fill both -expand true
  pack $tg.f00      -side top   -fill both -expand true
  pack $tg.f1       -side top   -fill both -expand true
  pack $tg.f2       -side top   -fill both -expand true
  pack $tg.f4       -side top   -fill both -expand true
  pack $tg.f3       -side top   -fill both -expand true
}

#
# doFgenGen4
#
proc doFgenGen4 { tg } {
  global BBFgen

  frame $tg.f0
  label $tg.f0.msg -text "Waveform Shape" -font bbaudio_font
  radiobutton $tg.f0.off   -text "Off" -font bbaudio_font \
                           -command { set_bbfgen_burst_mode 4 $BBFgen(Mode4) } \
                           -variable BBFgen(Mode4) \
                           -value OFF
  radiobutton $tg.f0.cont  -text "Continuous" -font bbaudio_font \
                           -command { set_bbfgen_burst_mode 4 $BBFgen(Mode4) } \
                           -variable BBFgen(Mode4) \
                           -value CONTINUOUS
  radiobutton $tg.f0.burs  -text "Burst" -font bbaudio_font \
                           -command { set_bbfgen_burst_mode 4 $BBFgen(Mode4) } \
                           -variable BBFgen(Mode4) \
                           -value BURST

  frame $tg.f00
  label $tg.f00.msg -text "Duration (1 to 100000 mS)" -font bbaudio_font
  entry $tg.f00.dur -textvariable BBFgen(Durat4) -font bbaudio_font -bg white
  bind $tg.f00.dur <Return> { set_bbfgen_burst_duration 4 $BBFgen(Durat4) }


  frame $tg.f1
  label $tg.f1.msg -text "Freq (0 to 20000)" -font bbaudio_font

  entry $tg.f1.freq -textvariable BBFgen(Freq4) -font bbaudio_font -bg white
  bind $tg.f1.freq <Return> { set_bbfgen_freq 4 $BBFgen(Freq4) }

  frame $tg.f2
  label $tg.f2.msg -text "AF lvl (0 to 10.0000 Volts)" -font bbaudio_font
  entry $tg.f2.aflvl -textvariable BBFgen(AFlvl4) -font bbaudio_font -bg white
  bind $tg.f2.aflvl <Return> { set_bbfgen_level 4 $BBFgen(AFlvl4) }

  frame $tg.f4
  label $tg.f4.msg -text "1V == 5 kHz = 16000" -font bbaudio_font

  frame $tg.f3
  label $tg.f3.msg -text "Waveform Shape" -font bbaudio_font
  radiobutton $tg.f3.sine  -text "Sine" -font bbaudio_font \
                           -command { set_bbfgen_shape 4 $BBFgen(Shape4) } \
                           -variable BBFgen(Shape4) \
                           -value SINE
  radiobutton $tg.f3.tri   -text "Triangle" -font bbaudio_font \
                           -command { set_bbfgen_shape 4 $BBFgen(Shape4) } \
                           -variable BBFgen(Shape4) \
                           -value TRIANGLE
  radiobutton $tg.f3.squ   -text "Square" -font bbaudio_font \
                           -command { set_bbfgen_shape 4 $BBFgen(Shape4) } \
                           -variable BBFgen(Shape4) \
                           -value SQUARE
  radiobutton $tg.f3.ramp  -text "Ramp" -font bbaudio_font \
                           -command { set_bbfgen_shape 4 $BBFgen(Shape4) } \
                           -variable BBFgen(Shape4) \
                           -value RAMP
  radiobutton $tg.f3.dc    -text "DC" -font bbaudio_font \
                           -command { set_bbfgen_shape 4 $BBFgen(Shape4) } \
                           -variable BBFgen(Shape4) \
                           -value DC
  radiobutton $tg.f3.arb   -text "Arbitrary" -font bbaudio_font \
                           -command { set_bbfgen_shape 4 $BBFgen(Shape4) } \
                           -variable BBFgen(Shape4) \
                           -value ARBITRARY

  pack $tg.f00.msg  -side left  -fill both -expand true
  pack $tg.f00.dur  -side left  -fill both -expand true

  pack $tg.f0.msg   -side top   -fill both -expand true
  pack $tg.f0.off   -side left  -fill both -expand true
  pack $tg.f0.cont  -side left  -fill both -expand true
  pack $tg.f0.burs  -side left  -fill both -expand true

  pack $tg.f1.msg   -side left  -fill both -expand true
  pack $tg.f1.freq  -side left  -fill both -expand true

  pack $tg.f2.msg   -side left  -fill both -expand true
  pack $tg.f2.aflvl -side left  -fill both -expand true

  pack $tg.f4.msg   -side left  -fill both -expand true

  pack $tg.f3.msg   -side top   -fill both -expand true
  pack $tg.f3.sine  -side left  -fill both -expand true
  pack $tg.f3.tri   -side left  -fill both -expand true
  pack $tg.f3.squ   -side left  -fill both -expand true
  pack $tg.f3.ramp  -side left  -fill both -expand true
  pack $tg.f3.dc    -side left  -fill both -expand true
  pack $tg.f3.arb   -side left  -fill both -expand true

  pack $tg.f0       -side top   -fill both -expand true
  pack $tg.f00      -side top   -fill both -expand true
  pack $tg.f1       -side top   -fill both -expand true
  pack $tg.f2       -side top   -fill both -expand true
  pack $tg.f4       -side top   -fill both -expand true
  pack $tg.f3       -side top   -fill both -expand true
}


#
# doFgenGen0
#
proc doFgenGen0 { tg } {
  global BBFgen

  frame $tg.f0
  label $tg.f0.msg -text "Waveform Shape" -font bbaudio_font
  radiobutton $tg.f0.off   -text "Off" -font bbaudio_font \
                           -command { set_bbfgen_burst_mode 0 $BBFgen(Mode0) } \
                           -variable BBFgen(Mode0) \
                           -value OFF
  radiobutton $tg.f0.cont  -text "Continuous" -font bbaudio_font \
                           -command { set_bbfgen_burst_mode 0 $BBFgen(Mode0) } \
                           -variable BBFgen(Mode0) \
                           -value CONTINUOUS
  radiobutton $tg.f0.burs  -text "Burst" -font bbaudio_font \
                           -command { set_bbfgen_burst_mode 0 $BBFgen(Mode0) } \
                           -variable BBFgen(Mode0) \
                           -value BURST

  frame $tg.f00
  label $tg.f00.msg -text "Duration (1 to 100000 mS)" -font bbaudio_font
  entry $tg.f00.dur -textvariable BBFgen(Durat0) -font bbaudio_font -bg white
  bind $tg.f00.dur <Return> { set_bbfgen_burst_duration 0 $BBFgen(Durat0) }


  frame $tg.f1
  label $tg.f1.msg -text "Freq (0 to 20000)" -font bbaudio_font

  entry $tg.f1.freq -textvariable BBFgen(Freq0) -font bbaudio_font -bg white
  bind $tg.f1.freq <Return> { set_bbfgen_freq 0 $BBFgen(Freq0) }

  frame $tg.f2
  label $tg.f2.msg -text "AF lvl (0 to 10.0000 Volts)" -font bbaudio_font
  entry $tg.f2.aflvl -textvariable BBFgen(AFlvl0) -font bbaudio_font -bg white
  bind $tg.f2.aflvl <Return> { set_bbfgen_level 0 $BBFgen(AFlvl0) }

  frame $tg.f4
  label $tg.f4.msg -text "1V == 5 kHz = 16000" -font bbaudio_font

  frame $tg.f3
  label $tg.f3.msg -text "Waveform Shape" -font bbaudio_font
  radiobutton $tg.f3.sine  -text "Sine" -font bbaudio_font \
                           -command { set_bbfgen_shape 0 $BBFgen(Shape0) } \
                           -variable BBFgen(Shape0) \
                           -value SINE
  radiobutton $tg.f3.tri   -text "Triangle" -font bbaudio_font \
                           -command { set_bbfgen_shape 0 $BBFgen(Shape0) } \
                           -variable BBFgen(Shape0) \
                           -value TRIANGLE
  radiobutton $tg.f3.squ   -text "Square" -font bbaudio_font \
                           -command { set_bbfgen_shape 0 $BBFgen(Shape0) } \
                           -variable BBFgen(Shape0) \
                           -value SQUARE
  radiobutton $tg.f3.ramp  -text "Ramp" -font bbaudio_font \
                           -command { set_bbfgen_shape 0 $BBFgen(Shape0) } \
                           -variable BBFgen(Shape0) \
                           -value RAMP
  radiobutton $tg.f3.dc    -text "DC" -font bbaudio_font \
                           -command { set_bbfgen_shape 0 $BBFgen(Shape0) } \
                           -variable BBFgen(Shape0) \
                           -value DC
  radiobutton $tg.f3.arb   -text "Arbitrary" -font bbaudio_font \
                           -command { set_bbfgen_shape 0 $BBFgen(Shape0) } \
                           -variable BBFgen(Shape0) \
                           -value ARBITRARY

  pack $tg.f00.msg  -side left  -fill both -expand true
  pack $tg.f00.dur  -side left  -fill both -expand true

  pack $tg.f0.msg   -side top   -fill both -expand true
  pack $tg.f0.off   -side left  -fill both -expand true
  pack $tg.f0.cont  -side left  -fill both -expand true
  pack $tg.f0.burs  -side left  -fill both -expand true

  pack $tg.f1.msg   -side left  -fill both -expand true
  pack $tg.f1.freq  -side left  -fill both -expand true

  pack $tg.f2.msg   -side left  -fill both -expand true
  pack $tg.f2.aflvl -side left  -fill both -expand true

  pack $tg.f4.msg   -side left  -fill both -expand true

  pack $tg.f3.msg   -side top   -fill both -expand true
  pack $tg.f3.sine  -side left  -fill both -expand true
  pack $tg.f3.tri   -side left  -fill both -expand true
  pack $tg.f3.squ   -side left  -fill both -expand true
  pack $tg.f3.ramp  -side left  -fill both -expand true
  pack $tg.f3.dc    -side left  -fill both -expand true
  pack $tg.f3.arb   -side left  -fill both -expand true

  pack $tg.f0       -side top   -fill both -expand true
  pack $tg.f00      -side top   -fill both -expand true
  pack $tg.f1       -side top   -fill both -expand true
  pack $tg.f2       -side top   -fill both -expand true
  pack $tg.f4       -side top   -fill both -expand true
  pack $tg.f3       -side top   -fill both -expand true
}






#
# build debug BB Audio screen - except final pack
#
proc build_bbaudio_screen { name } {
  global BBAUDIO_ENGINE
  global HwUtilEngine
  global DemoColor
  global RBBA

  set RBBA $name

  #
  # define screen frames
  #
  frame $RBBA.mbar          -relief raised -bd 2
  frame $RBBA.mbar.bb       -relief raised -bd 2
  frame $RBBA.mbar.util     -relief raised -bd 2


  button $RBBA.mbar.reset   -text "Reset HW" -font bbaudio_font \
                            -command init_bbaudio_hardware \
                            -bg $DemoColor(menu)

  message $RBBA.mbar.label  -text "BB Audio" -font bbaudio_font \
                            -bg $DemoColor(title) \
                            -width $DemoColor(labelwidth)

  button $RBBA.mbar.clr     -text "Clear Routes" -font bbaudio_font \
                            -command ::BBAudio::clr_bbaudio_routes \
                            -bg $DemoColor(menu)

  #
  #  Define menu bars 
  #
  menubutton $RBBA.mbar.bb.in    -text  "Input"        -font bbaudio_font \
                                 -menu $RBBA.mbar.bb.in.menu \
                                 -bg $DemoColor(menu) -relief raised
  menubutton $RBBA.mbar.bb.out   -text  "Output"       -font bbaudio_font \
                                 -menu $RBBA.mbar.bb.out.menu \
                                 -bg $DemoColor(menu) -relief raised
  menubutton $RBBA.mbar.bb.fgen  -text  "Fgen"         -font bbaudio_font \
                                 -menu $RBBA.mbar.bb.fgen.menu \
                                 -bg $DemoColor(menu) -relief raised
  menubutton $RBBA.mbar.bb.cal   -text  "Cal Mixer Lvl"  -font bbaudio_font \
                                 -menu $RBBA.mbar.bb.cal.menu \
                                 -bg $DemoColor(menu) -relief raised
  menubutton $RBBA.mbar.bb.uut   -text  "UUT Port"     -font bbaudio_font \
                                 -menu $RBBA.mbar.bb.uut.menu \
                                 -bg $DemoColor(menu) -relief raised


  #
  #  Build Input Menus 
  #
  menu $RBBA.mbar.bb.in.menu
  $RBBA.mbar.bb.in.menu add command \
              -label "Mixer" -font bbaudio_font \
              -command { WidgetWindow build_BBmixer_screen \
                         $RBBA.mixer $RBBA "Mixer Control" }
  $RBBA.mbar.bb.in.menu add command \
              -label "Router" -font bbaudio_font \
              -command { WidgetWindow build_BBrouter_screen \
                         $RBBA.router $RBBA "Router Control" }
  $RBBA.mbar.bb.in.menu add cascade       -label "SINAD" \
                -font bbaudio_font -menu $RBBA.mbar.bb.in.menu.sinad
  menu $RBBA.mbar.bb.in.menu.sinad
  $RBBA.mbar.bb.in.menu.sinad add cascade -label "Input" \
                -font bbaudio_font -menu $RBBA.mbar.bb.in.menu.sinad.input

  $RBBA.mbar.bb.in.menu add cascade       -label "Mic"   \
                -font bbaudio_font -menu $RBBA.mbar.bb.in.menu.mic
  menu $RBBA.mbar.bb.in.menu.mic
  $RBBA.mbar.bb.in.menu.mic add cascade   -label "Bias"  \
                -font bbaudio_font -menu $RBBA.mbar.bb.in.menu.mic.bias
  $RBBA.mbar.bb.in.menu add command \
              -label "Squelch" -font bbaudio_font \
              -command { WidgetWindow build_squelch_screen \
                         $RBBA.squelch $RBBA "Squelch Control" }

  #
  #  Build UUT Menus 
  #
  menu $RBBA.mbar.bb.uut.menu
  $RBBA.mbar.bb.uut.menu      add command -label "Digital Output" \
         -font bbaudio_font -command { WidgetWindow doBBaudioUUTDigOUt \
                                      $RBBA.uut_out $RBBA "UUT output control"}
  $RBBA.mbar.bb.uut.menu      add command -label "Digital Input" \
         -font bbaudio_font -command { WidgetWindow doBBaudioReadIO \
                                     $RBBA.uut_in $RBBA "UUT Input status" }

  #
  #  Build Output Menus 
  #
  menu $RBBA.mbar.bb.out.menu
  $RBBA.mbar.bb.out.menu       add cascade -label "Demod"  \
         -font bbaudio_font -menu $RBBA.mbar.bb.out.menu.demod
  $RBBA.mbar.bb.out.menu      add cascade -label "Fgen"    \
         -font bbaudio_font -menu $RBBA.mbar.bb.out.menu.fgen


menu $RBBA.mbar.bb.out.menu.demod
$RBBA.mbar.bb.out.menu.demod add cascade -label "Filter" \
       -font bbaudio_font -menu $RBBA.mbar.bb.out.menu.demod.filt


menu $RBBA.mbar.bb.out.menu.fgen

$RBBA.mbar.bb.out.menu.fgen add cascade -label "Filter"  \
       -font bbaudio_font -menu $RBBA.mbar.bb.out.menu.fgen.filt
$RBBA.mbar.bb.out.menu.fgen add cascade -label "Output"  \
       -font bbaudio_font -menu $RBBA.mbar.bb.out.menu.fgen.out
$RBBA.mbar.bb.out.menu.fgen add cascade -label "Level"   \
       -font bbaudio_font -menu $RBBA.mbar.bb.out.menu.fgen.level


#
#  build input menu actions
#
menu $RBBA.mbar.bb.in.menu.sinad.input
$RBBA.mbar.bb.in.menu.sinad.input add radiobutton -label "Balanced" \
        -font bbaudio_font \
        -command { BBAudio sinadbalance $BBAUDIO_ENGINE(SINAD.input) } \
        -variable BBAUDIO_ENGINE(SINAD.input) -value BALANCED
$RBBA.mbar.bb.in.menu.sinad.input add radiobutton -label "Unbalanced" \
        -font bbaudio_font \
        -command { BBAudio sinadbalance $BBAUDIO_ENGINE(SINAD.input) } \
        -variable BBAUDIO_ENGINE(SINAD.input) -value UNBALANCED

# BIND BUTTON RELEASE SO DEFAULT BINDINGS DONT UNPOST
#
bind $RBBA.mbar.bb.in.menu.sinad.input <ButtonRelease> \
                                          {MenuInvokeRemain %W 1;break} 
bind $RBBA.mbar.bb.in.menu.sinad.input <Return> \
                                          {MenuInvokeRemain %W 0;break}


menu $RBBA.mbar.bb.in.menu.mic.bias
$RBBA.mbar.bb.in.menu.mic.bias add radiobutton -label "Normal" \
        -font bbaudio_font \
        -command { BBAudio microphonebias $BBAUDIO_ENGINE(MIC.bias) } \
        -variable BBAUDIO_ENGINE(MIC.bias) -value NORMAL
$RBBA.mbar.bb.in.menu.mic.bias add radiobutton -label "Power" \
        -font bbaudio_font \
        -command { BBAudio microphonebias $BBAUDIO_ENGINE(MIC.bias) } \
        -variable BBAUDIO_ENGINE(MIC.bias) -value POWER

# BIND BUTTON RELEASE SO DEFAULT BINDINGS DONT UNPOST
#
bind $RBBA.mbar.bb.in.menu.mic.bias <ButtonRelease> \
                                          {MenuInvokeRemain %W 1;break} 
bind $RBBA.mbar.bb.in.menu.mic.bias <Return> \
                                          {MenuInvokeRemain %W 0;break}


menu $RBBA.mbar.bb.out.menu.demod.filt
$RBBA.mbar.bb.out.menu.demod.filt add radiobutton -label "Square" \
        -font bbaudio_font \
        -command { BBAudio demodfilter $BBAUDIO_ENGINE(demod.filter) } \
        -variable BBAUDIO_ENGINE(demod.filter) -value SQUARE
$RBBA.mbar.bb.out.menu.demod.filt add radiobutton -label "Sine" \
        -font bbaudio_font \
        -command { BBAudio demodfilter $BBAUDIO_ENGINE(demod.filter) } \
        -variable BBAUDIO_ENGINE(demod.filter) -value SINE


# BIND BUTTON RELEASE SO DEFAULT BINDINGS DONT UNPOST
#
bind $RBBA.mbar.bb.out.menu.demod.filt <ButtonRelease> \
                                          {MenuInvokeRemain %W 1;break} 
bind $RBBA.mbar.bb.out.menu.demod.filt <Return> \
                                          {MenuInvokeRemain %W 0;break}


menu $RBBA.mbar.bb.out.menu.fgen.filt
$RBBA.mbar.bb.out.menu.fgen.filt add radiobutton -label "Speech" \
        -font bbaudio_font \
        -command { BBAudio fgenfilter $BBAUDIO_ENGINE(fgen.filter) } \
        -variable BBAUDIO_ENGINE(fgen.filter) -value SINE
$RBBA.mbar.bb.out.menu.fgen.filt add radiobutton -label "Data" \
        -font bbaudio_font \
        -command { BBAudio fgenfilter $BBAUDIO_ENGINE(fgen.filter) } \
        -variable BBAUDIO_ENGINE(fgen.filter) -value SQUARE


# BIND BUTTON RELEASE SO DEFAULT BINDINGS DONT UNPOST
#
bind $RBBA.mbar.bb.out.menu.fgen.filt <ButtonRelease> \
                                          {MenuInvokeRemain %W 1;break} 
bind $RBBA.mbar.bb.out.menu.fgen.filt <Return> \
                                          {MenuInvokeRemain %W 0;break}


menu $RBBA.mbar.bb.out.menu.fgen.out
$RBBA.mbar.bb.out.menu.fgen.out add radiobutton -label "Balanced" \
        -font bbaudio_font \
        -command { BBAudio fgenbalance $BBAUDIO_ENGINE(fgen.output) } \
        -variable BBAUDIO_ENGINE(fgen.output) -value BALANCED
$RBBA.mbar.bb.out.menu.fgen.out add radiobutton -label "Unbalanced" \
        -font bbaudio_font \
        -command { BBAudio fgenbalance $BBAUDIO_ENGINE(fgen.output) } \
        -variable BBAUDIO_ENGINE(fgen.output) -value UNBALANCED


# BIND BUTTON RELEASE SO DEFAULT BINDINGS DONT UNPOST
#
bind $RBBA.mbar.bb.out.menu.fgen.out <ButtonRelease> \
                                          {MenuInvokeRemain %W 1;break} 
bind $RBBA.mbar.bb.out.menu.fgen.out <Return> \
                                          {MenuInvokeRemain %W 0;break}


menu $RBBA.mbar.bb.out.menu.fgen.level
$RBBA.mbar.bb.out.menu.fgen.level add radiobutton -label "Normal" \
        -font bbaudio_font \
        -command { BBAudio fgenlevel $BBAUDIO_ENGINE(fgen.level) } \
        -variable BBAUDIO_ENGINE(fgen.level) -value NORMAL
$RBBA.mbar.bb.out.menu.fgen.level add radiobutton -label "One Tenth" \
        -font bbaudio_font \
        -command { BBAudio fgenlevel $BBAUDIO_ENGINE(fgen.level) } \
        -variable BBAUDIO_ENGINE(fgen.level) -value TENTH

# BIND BUTTON RELEASE SO DEFAULT BINDINGS DONT UNPOST
#
bind $RBBA.mbar.bb.out.menu.fgen.level <ButtonRelease> \
                                          {MenuInvokeRemain %W 1;break} 
bind $RBBA.mbar.bb.out.menu.fgen.level <Return> \
                                          {MenuInvokeRemain %W 0;break}


#
#  bb level cal menus
#
menu $RBBA.mbar.bb.cal.menu

$RBBA.mbar.bb.cal.menu add command \
              -label "Cal Router/Mixer Levels" -font bbaudio_font \
              -command { WidgetWindow doBBLvlCal \
                         $RBBA.callvl $RBBA "Cal Router/Mixer Levels"}

#
#  Fgen menus
#
menu $RBBA.mbar.bb.fgen.menu
$RBBA.mbar.bb.fgen.menu add command \
              -label "Fgen0" -font bbaudio_font \
              -command { WidgetWindow doFgenGen0 \
                         $RBBA.fgen0 $RBBA "Function generator 0"}
$RBBA.mbar.bb.fgen.menu add command \
              -label "Fgen1" -font bbaudio_font \
              -command { WidgetWindow doFgenGen1 \
                         $RBBA.fgen1 $RBBA "Function generator 1"}
$RBBA.mbar.bb.fgen.menu add command \
              -label "Fgen2" -font bbaudio_font \
              -command { WidgetWindow doFgenGen2 \
                         $RBBA.fgen2 $RBBA "Function generator 2"}
$RBBA.mbar.bb.fgen.menu add command \
              -label "Fgen3" -font bbaudio_font \
              -command { WidgetWindow doFgenGen3 \
                         $RBBA.fgen3 $RBBA "Function generator 3"}
$RBBA.mbar.bb.fgen.menu add command \
              -label "Fgen4" -font bbaudio_font \
              -command { WidgetWindow doFgenGen4 \
                         $RBBA.fgen4 $RBBA "Function generator 4"}
$RBBA.mbar.bb.fgen.menu add command \
              -label "DTMF" -font bbaudio_font \
              -command { WidgetWindow doDummy \
                         $RBBA.dtmf $RBBA "DTMF"}
$RBBA.mbar.bb.fgen.menu add command \
              -label "Tone Sequence" -font bbaudio_font \
              -command { WidgetWindow doDummy \
                         $RBBA.tone_signal $RBBA "Tone Signaling"}
$RBBA.mbar.bb.fgen.menu add command \
              -label "Synchronize" -font bbaudio_font \
              -command { bbfgen_sync }

#
#  HW Utility Volume 
#
#
#  Define menu bars 
#
menubutton $RBBA.mbar.util.menu    -text  "HW Utils" -font bbaudio_font \
                                   -menu $RBBA.mbar.util.menu.pull \
                                   -bg $DemoColor(menu) -relief raised

menu $RBBA.mbar.util.menu.pull
$RBBA.mbar.util.menu.pull add cascade       -label "Crypto" \
                         -font bbaudio_font \
                         -menu $RBBA.mbar.util.menu.pull.crypt
$RBBA.mbar.util.menu.pull add cascade       -label "10 MHz Input" \
                         -font bbaudio_font \
                         -menu $RBBA.mbar.util.menu.pull.10mhz
$RBBA.mbar.util.menu.pull add command       -label "Backlight" \
                         -font bbaudio_font \
                         -command { WidgetWindow doHwUtilBacklight \
                                    $RBBA.backlight $RBBA "Backlight"}
$RBBA.mbar.util.menu.pull add command       -label "TCXO Adj" \
                         -font bbaudio_font \
                         -command { WidgetWindow doHwUtilTcxo \
                                    $RBBA.tcxo $RBBA "TCXO Adj"}

menu $RBBA.mbar.util.menu.pull.crypt
$RBBA.mbar.util.menu.pull.crypt add radiobutton -label "Enable" \
        -font bbaudio_font \
        -command { set_hwutil_crypto $HwUtilEngine(crypto) } \
        -variable HwUtilEngine(crypto) -value ON
$RBBA.mbar.util.menu.pull.crypt add radiobutton -label "Disable" \
        -font bbaudio_font \
        -command { set_hwutil_crypto $HwUtilEngine(crypto) } \
        -variable HwUtilEngine(crypto) -value OFF

# BIND BUTTON RELEASE SO DEFAULT BINDINGS DONT UNPOST
#
bind $RBBA.mbar.util.menu.pull.crypt <ButtonRelease> \
                                          {MenuInvokeRemain %W 1;break} 
bind $RBBA.mbar.util.menu.pull.crypt <Return> \
                                          {MenuInvokeRemain %W 0;break}

menu $RBBA.mbar.util.menu.pull.10mhz

$RBBA.mbar.util.menu.pull.10mhz add radiobutton -label "Internal" \
        -font bbaudio_font \
        -command { set_hwutil_10mhz_ref $HwUtilEngine(10mhz) } \
        -variable HwUtilEngine(10mhz) -value INT_REF
$RBBA.mbar.util.menu.pull.10mhz add radiobutton -label "External" \
        -font bbaudio_font \
        -command { set_hwutil_10mhz_ref $HwUtilEngine(10mhz) } \
        -variable HwUtilEngine(10mhz) -value EXT_REF

# BIND BUTTON RELEASE SO DEFAULT BINDINGS DONT UNPOST
#
bind $RBBA.mbar.util.menu.pull.10mhz <ButtonRelease> \
                                          {MenuInvokeRemain %W 1;break} 
bind $RBBA.mbar.util.menu.pull.10mhz <Return> \
                                          {MenuInvokeRemain %W 0;break}


#
#  pack bb menu bar
#
pack $RBBA.mbar.bb.in      -side left -fill both -expand true
pack $RBBA.mbar.bb.out     -side left -fill both -expand true
pack $RBBA.mbar.bb.cal     -side left -fill both -expand true
pack $RBBA.mbar.bb.fgen    -side left -fill both -expand true
pack $RBBA.mbar.bb.uut     -side left -fill both -expand true

#
#  pack util menu bar
#
pack $RBBA.mbar.util.menu   -side left -fill both -expand true


#
#  pack main frame
#
pack $RBBA.mbar.label        -side left  -fill both
pack $RBBA.mbar.bb           -side left  -fill both -expand true
pack $RBBA.mbar.clr          -side left  -fill both
pack $RBBA.mbar.util         -side left  -fill both
pack $RBBA.mbar.reset        -side right -fill both


pack $RBBA.mbar              -side left  -fill both -expand true

}


font create bbmixer_font -family helvetica -size 8

#
#   build_BBmixer_screen
#
proc build_BBmixer_screen { name } {
  global bsinadms
  global SINAD_STATUS
  global SINAD_SCRN
  global DemoColor
  global sinads_font
  global BBAUDIO_ENGINE

set brmix $name

frame $brmix.head
frame $brmix.col

#
#  Widget header
#
label $brmix.head.label -text "Mixer Configuration" -bg $DemoColor(title)
pack $brmix.head.label   -side left -fill both -expand true

#
#  Column headers
#
label $brmix.col.label -text "OUT \\ IN" -font bbmixer_font -width 10 -bg "light yellow"
label $brmix.col.c1    -text "L1"  -font bbmixer_font -width 3 
label $brmix.col.c2    -text "Mic" -font bbmixer_font -width 3 
label $brmix.col.c3    -text "Sin" -font bbmixer_font -width 3 
label $brmix.col.c4    -text "Wav" -font bbmixer_font -width 3 
label $brmix.col.c5    -text "FGen0"  -font bbmixer_font -width  5 
label $brmix.col.c6    -text "FGen1"  -font bbmixer_font -width  5 
label $brmix.col.c7    -text "FGen2"  -font bbmixer_font -width  5 
label $brmix.col.c8    -text "FGen3"  -font bbmixer_font -width  5 
label $brmix.col.c9    -text "FGen4"  -font bbmixer_font -width  5
label $brmix.col.c10   -text "U0"  -font bbmixer_font -width  3 
label $brmix.col.c11   -text "U1"  -font bbmixer_font -width  3 
label $brmix.col.c12   -text "U2"  -font bbmixer_font -width  3 
label $brmix.col.c13   -text "U3"  -font bbmixer_font -width  3 
label $brmix.col.c14   -text "U4"  -font bbmixer_font -width  3 
label $brmix.col.c15   -text "U5"  -font bbmixer_font -width  3 


#
# Set up Speaker buttons
#
label $brmix.col.spkr_label -text "Speaker" -width 10 -bg $DemoColor(title)
foreach { source } { L1 MIC SINAD FILE FGEN0 FGEN1 FGEN2 FGEN3 FGEN4 USER0 USER1 USER2 USER3 USER4 USER5 } {
   set var BBAUDIO_ENGINE($source.SPKR)
   checkbutton $brmix.col.spkr_$source -font bbmixer_font \
               -command "::BBAudio::set_bbaudio_route $source SPKR $$var" \
               -variable BBAUDIO_ENGINE($source.SPKR) -onvalue 1 -offvalue 0
}


#
# Set up DEMOD buttons
#
label $brmix.col.demod_label -text "DEMOD" -width 10 -bg $DemoColor(title)
foreach { source } { L1 MIC SINAD FILE FGEN0 FGEN1 FGEN2 FGEN3 FGEN4 USER0 USER1 USER2 USER3 USER4 USER5 } {
   set var BBAUDIO_ENGINE($source.DEMOD)
   checkbutton $brmix.col.demod_$source -font bbmixer_font \
               -command "::BBAudio::set_bbaudio_route $source DEMOD $$var" \
               -variable BBAUDIO_ENGINE($source.DEMOD) -onvalue 1 -offvalue 0
}


#
# Set up FGEN buttons
#
label $brmix.col.fgen_label -text "FGEN" -width 10 -bg $DemoColor(title)
foreach { source } { L1 MIC SINAD FILE FGEN0 FGEN1 FGEN2 FGEN3 FGEN4 USER0 USER1 USER2 USER3 USER4 USER5 } {
   set var BBAUDIO_ENGINE($source.FGEN)
   checkbutton $brmix.col.fgen_$source -font bbmixer_font \
               -command "::BBAudio::set_bbaudio_route $source FGEN $$var" \
               -variable BBAUDIO_ENGINE($source.FGEN) -onvalue 1 -offvalue 0
}


#
# Set up L1 buttons
#
label $brmix.col.l1_label -text "L1" -width 10 -bg $DemoColor(title)
foreach { source } { L1 MIC SINAD FILE FGEN0 FGEN1 FGEN2 FGEN3 FGEN4 USER0 USER1 USER2 USER3 USER4 USER5 } {
   set var BBAUDIO_ENGINE($source.L1)
   checkbutton $brmix.col.l1_$source -font bbmixer_font \
               -command "::BBAudio::set_bbaudio_route $source L1 $$var" \
               -variable BBAUDIO_ENGINE($source.L1) -onvalue 1 -offvalue 0
}


#
# Set up USER 0 buttons
#
label $brmix.col.user0_label -text "User 0" -width 10 -bg $DemoColor(title)
foreach { source } { L1 MIC SINAD FILE FGEN0 FGEN1 FGEN2 FGEN3 FGEN4 USER0 USER1 USER2 USER3 USER4 USER5 } {
   set var BBAUDIO_ENGINE($source.USER0)
   checkbutton $brmix.col.user0_$source -font bbmixer_font \
               -command "::BBAudio::set_bbaudio_route $source USER0 $$var" \
               -variable BBAUDIO_ENGINE($source.USER0) -onvalue 1 -offvalue 0
}


#
# Set up USER 1 buttons
#
label $brmix.col.user1_label -text "User 1" -width 10 -bg $DemoColor(title)
foreach { source } { L1 MIC SINAD FILE FGEN0 FGEN1 FGEN2 FGEN3 FGEN4 USER0 USER1 USER2 USER3 USER4 USER5 } {
   set var BBAUDIO_ENGINE($source.USER1)
   checkbutton $brmix.col.user1_$source -font bbmixer_font \
               -command "::BBAudio::set_bbaudio_route $source USER1 $$var" \
               -variable BBAUDIO_ENGINE($source.USER1) -onvalue 1 -offvalue 0
}


#
# Set up layout of rows and columns
#
grid $brmix.col.label -row 0 -column 0 -sticky e
foreach { col } { 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 } {
   grid $brmix.col.c$col -row 0 -column $col -sticky e
}

set row_no 1
foreach { source } { spkr demod fgen l1 user0 user1 } {
   set col_no 0
   foreach { col } { label L1 MIC SINAD FILE FGEN0 FGEN1 FGEN2 FGEN3 FGEN4 USER0 USER1 USER2 USER3 USER4 USER5 } {
      grid $brmix.col.$source\_$col -row $row_no -column $col_no -sticky e
      incr col_no
   }
   incr row_no
}

 
#
#  Pack screen
#
pack $brmix.head        -side top
pack $brmix.col         -side top
}


set BBrouter(inputs) "light blue"
set BBrouter(mixers) "yellow2"
set BBrouter(meters) "LimeGreen"


#
#   build_BBrouter_screen
#
proc build_BBrouter_screen { name } {
  global bsinadms
  global SINAD_STATUS
  global SINAD_SCRN
  global DemoColor
  global sinads_font
  global BBAUDIO_ENGINE
  global BBrouter
  global BBAUDIO_ROUTE

set brroute $name

frame $brroute.head1
frame $brroute.blank1
frame $brroute.col
frame $brroute.blank2
frame $brroute.head2
frame $brroute.blank3
frame $brroute.col2

#
#  Widget header
#
label $brroute.head1.label -text "Select data source for user defined Mixer Inputs" -bg $DemoColor(title)
pack $brroute.head1.label   -side left -fill both -expand true

label $brroute.blank1.label -text " " 
pack $brroute.blank1.label   -side left -fill both -expand true

#
#  Column headers
#
label $brroute.col.label -text " "     -font bbmixer_font  
label $brroute.col.c1    -text "L1"    -font bbmixer_font -bg $BBrouter(inputs)
label $brroute.col.c2    -text "mic"   -font bbmixer_font -bg $BBrouter(inputs)
label $brroute.col.c3    -text "sin"   -font bbmixer_font -bg $BBrouter(inputs)
label $brroute.col.c4    -text "wav"   -font bbmixer_font -bg $BBrouter(inputs)
label $brroute.col.c5    -text "fg0"   -font bbmixer_font -bg $BBrouter(inputs)
label $brroute.col.c6    -text "fg1"   -font bbmixer_font -bg $BBrouter(inputs)
label $brroute.col.c7    -text "fg2"   -font bbmixer_font -bg $BBrouter(inputs)
label $brroute.col.c8    -text "fg3"   -font bbmixer_font -bg $BBrouter(inputs)
label $brroute.col.c9    -text "fg4"   -font bbmixer_font -bg $BBrouter(inputs)
label $brroute.col.c10   -text "spkr"  -font bbmixer_font -bg $BBrouter(mixers)
label $brroute.col.c11   -text "dmod"  -font bbmixer_font -bg $BBrouter(mixers)
label $brroute.col.c12   -text "fgen"  -font bbmixer_font -bg $BBrouter(mixers)
label $brroute.col.c13   -text "mxL1"  -font bbmixer_font -bg $BBrouter(mixers)
label $brroute.col.c14   -text "mxu0"  -font bbmixer_font -bg $BBrouter(mixers)
label $brroute.col.c15   -text "mxu1"  -font bbmixer_font -bg $BBrouter(mixers)
label $brroute.col.c16   -text "afc"   -font bbmixer_font -bg $BBrouter(meters)
label $brroute.col.c17   -text "sd"    -font bbmixer_font -bg $BBrouter(meters)
label $brroute.col.c18   -text "scope" -font bbmixer_font -bg $BBrouter(meters)
label $brroute.col.c19   -text "sclr"  -font bbmixer_font -bg $BBrouter(meters)
label $brroute.col.c20   -text "mf4"   -font bbmixer_font -bg $BBrouter(meters)
label $brroute.col.c21   -text "mf5"   -font bbmixer_font -bg $BBrouter(meters)
label $brroute.col.c22   -text "mf6"   -font bbmixer_font -bg $BBrouter(meters)
label $brroute.col.c23   -text "mf7"   -font bbmixer_font -bg $BBrouter(meters)
label $brroute.col.c24   -text "mf8"   -font bbmixer_font -bg $BBrouter(meters)
label $brroute.col.c25   -text "none"  -font bbmixer_font -bg "light yellow"


#
# Set up Mixer User Input # 0 to 5 buttons
#
foreach { i j } { u0 USER0 u1 USER1 u2 USER2 u3 USER3 u4 USER4 u5 USER5 } {
   label $brroute.col.$i\_label -text "Mix $i" -font bbmixer_font -bg $DemoColor(title)
   set var BBAUDIO_ROUTE($j)
   radiobutton $brroute.col.$i\_l1 -font bbmixer_font \
                           -command "BBAudio route $$var $j" \
                           -variable BBAUDIO_ROUTE($j) -value L1
   radiobutton $brroute.col.$i\_mic -font bbmixer_font \
                           -command "BBAudio route $$var $j" \
                           -variable BBAUDIO_ROUTE($j) -value MIC
   radiobutton $brroute.col.$i\_sinad -font bbmixer_font \
                           -command "BBAudio route $$var) $j" \
                           -variable BBAUDIO_ROUTE($j) -value SINAD
   radiobutton $brroute.col.$i\_file -font bbmixer_font \
                           -command "BBAudio route $$var) $j" \
                           -variable BBAUDIO_ROUTE($j) -value FILE
   radiobutton $brroute.col.$i\_fgen0 -font bbmixer_font \
                           -command "BBAudio route $$var $j" \
                           -variable BBAUDIO_ROUTE($j) -value FGEN0
   radiobutton $brroute.col.$i\_fgen1 -font bbmixer_font \
                           -command "BBAudio route $$var $j" \
                           -variable BBAUDIO_ROUTE($j) -value FGEN1
   radiobutton $brroute.col.$i\_fgen2 -font bbmixer_font \
                           -command "BBAudio route $$var $j" \
                           -variable BBAUDIO_ROUTE($j) -value FGEN2
   radiobutton $brroute.col.$i\_fgen3 -font bbmixer_font \
                           -command "BBAudio route $$var $j" \
                           -variable BBAUDIO_ROUTE($j) -value FGEN3
   radiobutton $brroute.col.$i\_fgen4 -font bbmixer_font \
                           -command "BBAudio route $$var $j" \
                           -variable BBAUDIO_ROUTE($j) -value FGEN4 
   radiobutton $brroute.col.$i\_spkr -font bbmixer_font \
                           -command "BBAudio route $$var $j" \
                           -variable BBAUDIO_ROUTE($j) -value SPKR
   radiobutton $brroute.col.$i\_demod -font bbmixer_font \
                           -command "BBAudio route $$var) $j" \
                           -variable BBAUDIO_ROUTE($j) -value DEMOD
   radiobutton $brroute.col.$i\_fgen -font bbmixer_font \
                           -command "BBAudio route $$var $j" \
                           -variable BBAUDIO_ROUTE($j) -value FGEN
   radiobutton $brroute.col.$i\_mixl1 -font bbmixer_font \
                           -command "BBAudio route $$var $j" \
                           -variable BBAUDIO_ROUTE($j) -value MIX_L1
   radiobutton $brroute.col.$i\_user0 -font bbmixer_font \
                           -command "BBAudio route $$var $j" \
                           -variable BBAUDIO_ROUTE($j) -value USER0
   radiobutton $brroute.col.$i\_user1 -font bbmixer_font \
                           -command "BBAudio route $$var $j" \
                           -variable BBAUDIO_ROUTE($j) -value USER1
   radiobutton $brroute.col.$i\_afctr -font bbmixer_font \
                           -command "BBAudio route $$var $j" \
                           -variable BBAUDIO_ROUTE($j) -value AFCTR
   radiobutton $brroute.col.$i\_msinad -font bbmixer_font \
                           -command "BBAudio route $$var $j" \
                           -variable BBAUDIO_ROUTE($j) -value MSINAD
   radiobutton $brroute.col.$i\_scope -font bbmixer_font \
                           -command "BBAudio route $var $j" \
                           -variable BBAUDIO_ROUTE($j) -value SCOPE
   radiobutton $brroute.col.$i\_sclr -font bbmixer_font \
                           -command "BBAudio route $var $j" \
                           -variable BBAUDIO_ROUTE($j) -value SCALAR 
   radiobutton $brroute.col.$i\_mf4 -font bbmixer_font \
                           -command "BBAudio route $$var $j" \
                           -variable BBAUDIO_ROUTE($j) -value M_F4
   radiobutton $brroute.col.$i\_mf5 -font bbmixer_font \
                           -command "BBAudio route $$var $j" \
                           -variable BBAUDIO_ROUTE($j) -value M_F5
   radiobutton $brroute.col.$i\_mf6 -font bbmixer_font \
                           -command "BBAudio route $$var $j" \
                           -variable BBAUDIO_ROUTE($j) -value M_F6
   radiobutton $brroute.col.$i\_mf7 -font bbmixer_font \
                           -command "BBAudio route $$var $j" \
                           -variable BBAUDIO_ROUTE($j) -value M_F7
   radiobutton $brroute.col.$i\_mf8 -font bbmixer_font \
                           -command "BBAudio route $$var $j" \
                           -variable BBAUDIO_ROUTE($j) -value M_F8
   radiobutton $brroute.col.$i\_none -font bbmixer_font -bg "light yellow" \
                           -command "BBAudio route $$var $j" \
                           -variable BBAUDIO_ROUTE($j) -value NONE
}


#
# Set up layout of rows and columns for Mixer User Input
#
grid $brroute.col.label -row 0 -column 0
grid $brroute.col.c25   -row 0 -column 1

foreach { col } { 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 } {   
   grid $brroute.col.c$col -row 0 -column [expr $col + 1]
}

set row_no 1
foreach { source } { u0 u1 u2 u3 u4 u5 } {
   set col_no 0
   foreach { col } { label none l1 mic sinad file fgen0 fgen1 fgen2 fgen3 fgen4 spkr demod fgen mixl1 user0 user1 \
                     afctr msinad scope sclr mf4 mf5 mf6 mf7 mf8 } {
      grid $brroute.col.$source\_$col -row $row_no -column $col_no
      incr col_no
   }
   incr row_no
}


#
#  Widget header
#
label $brroute.blank2.label -text " " 
pack $brroute.blank2.label   -side left -fill both -expand true

label $brroute.head2.label -text "Select data source for Meters (filters)" -bg $DemoColor(title)
pack $brroute.head2.label   -side left -fill both -expand true

label $brroute.blank3.label -text " " 
pack $brroute.blank3.label   -side left -fill both -expand true


#
#  Column headers
#
label $brroute.col2.label -text " "     -font bbmixer_font  
label $brroute.col2.c1    -text "L1"    -font bbmixer_font -bg $BBrouter(inputs)
label $brroute.col2.c2    -text "mic"   -font bbmixer_font -bg $BBrouter(inputs)
label $brroute.col2.c3    -text "sin"   -font bbmixer_font -bg $BBrouter(inputs)
label $brroute.col2.c4    -text "wav"   -font bbmixer_font -bg $BBrouter(inputs)
label $brroute.col2.c5    -text "fg0"   -font bbmixer_font -bg $BBrouter(inputs)
label $brroute.col2.c6    -text "fg1"   -font bbmixer_font -bg $BBrouter(inputs)
label $brroute.col2.c7    -text "fg2"   -font bbmixer_font -bg $BBrouter(inputs)
label $brroute.col2.c8    -text "fg3"   -font bbmixer_font -bg $BBrouter(inputs)
label $brroute.col2.c9    -text "fg4"   -font bbmixer_font -bg $BBrouter(inputs)                     
label $brroute.col2.c10   -text "spkr"  -font bbmixer_font -bg $BBrouter(mixers)
label $brroute.col2.c11   -text "dmod"  -font bbmixer_font -bg $BBrouter(mixers)
label $brroute.col2.c12   -text "fgen"  -font bbmixer_font -bg $BBrouter(mixers)
label $brroute.col2.c13   -text "mxL1"  -font bbmixer_font -bg $BBrouter(mixers)
label $brroute.col2.c14   -text "mxu0"  -font bbmixer_font -bg $BBrouter(mixers)
label $brroute.col2.c15   -text "mxu1"  -font bbmixer_font -bg $BBrouter(mixers)
label $brroute.col2.c16   -text "afc"   -font bbmixer_font -bg $BBrouter(meters)
label $brroute.col2.c17   -text "sd"    -font bbmixer_font -bg $BBrouter(meters)
label $brroute.col2.c18   -text "scope" -font bbmixer_font -bg $BBrouter(meters)
label $brroute.col2.c19   -text "sclr"  -font bbmixer_font -bg $BBrouter(meters)
label $brroute.col2.c20   -text "mf4"   -font bbmixer_font -bg $BBrouter(meters)
label $brroute.col2.c21   -text "mf5"   -font bbmixer_font -bg $BBrouter(meters)
label $brroute.col2.c22   -text "mf6"   -font bbmixer_font -bg $BBrouter(meters)
label $brroute.col2.c23   -text "mf7"   -font bbmixer_font -bg $BBrouter(meters)
label $brroute.col2.c24   -text "mf8"   -font bbmixer_font -bg $BBrouter(meters)
label $brroute.col2.c25   -text "none"  -font bbmixer_font -bg "light yellow"


#
# Set up Meter # 0 to 2 buttons
#
foreach { i meter } { mtr0 AFCTR mtr1 SINAD mtr2 SCOPE mtr3 SCALAR } {
   label $brroute.col2.$i\_label -text $meter -font bbmixer_font -bg $DemoColor(title)
   set var BBAUDIO_ROUTE($meter)
   radiobutton $brroute.col2.$i\_l1 -font bbmixer_font \
                           -command "BBAudio route $$var $meter" \
                           -variable BBAUDIO_ROUTE($meter) -value L1
   radiobutton $brroute.col2.$i\_mic -font bbmixer_font \
                           -command "BBAudio route $$var $meter" \
                           -variable BBAUDIO_ROUTE($meter) -value MIC
   radiobutton $brroute.col2.$i\_sinad -font bbmixer_font \
                           -command "BBAudio route $$var $meter" \
                           -variable BBAUDIO_ROUTE($meter) -value SINAD
   radiobutton $brroute.col2.$i\_file -font bbmixer_font \
                           -command "BBAudio route $$var $meter" \
                           -variable BBAUDIO_ROUTE($meter) -value FILE
   radiobutton $brroute.col2.$i\_fgen0 -font bbmixer_font \
                           -command "BBAudio route $$var $meter" \
                           -variable BBAUDIO_ROUTE($meter) -value FGEN0
   radiobutton $brroute.col2.$i\_fgen1 -font bbmixer_font \
                           -command "BBAudio route $$var $meter" \
                           -variable BBAUDIO_ROUTE($meter) -value FGEN1
   radiobutton $brroute.col2.$i\_fgen2 -font bbmixer_font \
                           -command "BBAudio route $$var $meter" \
                           -variable BBAUDIO_ROUTE($meter) -value FGEN2
   radiobutton $brroute.col2.$i\_fgen3 -font bbmixer_font \
                           -command "BBAudio route $$var $meter" \
                           -variable BBAUDIO_ROUTE($meter) -value FGEN3
   radiobutton $brroute.col2.$i\_fgen4 -font bbmixer_font \
                           -command "BBAudio route $$var $meter" \
                           -variable BBAUDIO_ROUTE($meter) -value FGEN4
   radiobutton $brroute.col2.$i\_spkr -font bbmixer_font \
                           -command "BBAudio route $$var $meter" \
                           -variable BBAUDIO_ROUTE($meter) -value SPKR
   radiobutton $brroute.col2.$i\_demod -font bbmixer_font \
                           -command "BBAudio route $$var $meter" \
                           -variable BBAUDIO_ROUTE($meter) -value DEMOD
   radiobutton $brroute.col2.$i\_fgen -font bbmixer_font \
                           -command "BBAudio route $$var $meter" \
                           -variable BBAUDIO_ROUTE($meter) -value FGEN
   radiobutton $brroute.col2.$i\_mixl1 -font bbmixer_font \
                           -command "BBAudio route $$var $meter" \
                           -variable BBAUDIO_ROUTE($meter) -value MIX_L1
   radiobutton $brroute.col2.$i\_user0 -font bbmixer_font \
                           -command "BBAudio route $$var $meter" \
                           -variable BBAUDIO_ROUTE($meter) -value USER0
   radiobutton $brroute.col2.$i\_user1 -font bbmixer_font \
                           -command "BBAudio route $$var $meter" \
                           -variable BBAUDIO_ROUTE($meter) -value USER1
   radiobutton $brroute.col2.$i\_afctr -font bbmixer_font \
                           -command "BBAudio route $$var $meter" \
                           -variable BBAUDIO_ROUTE($meter) -value AFCTR
   radiobutton $brroute.col2.$i\_msinad -font bbmixer_font \
                           -command "BBAudio route $$var $meter" \
                           -variable BBAUDIO_ROUTE($meter) -value MSINAD
   radiobutton $brroute.col2.$i\_scope -font bbmixer_font \
                           -command "BBAudio route $$var $meter" \
                           -variable BBAUDIO_ROUTE($meter) -value SCOPE
   radiobutton $brroute.col2.$i\_sclr -font bbmixer_font \
                           -command "BBAudio route $$var $meter" \
                           -variable BBAUDIO_ROUTE($meter) -value SCALAR 
   radiobutton $brroute.col2.$i\_mf4 -font bbmixer_font \
                           -command "BBAudio route $$var $meter" \
                           -variable BBAUDIO_ROUTE($meter) -value M_F4
   radiobutton $brroute.col2.$i\_mf5 -font bbmixer_font \
                           -command "BBAudio route $$var $meter" \
                           -variable BBAUDIO_ROUTE($meter) -value M_F5
   radiobutton $brroute.col2.$i\_mf6 -font bbmixer_font \
                           -command "BBAudio route $$var $meter" \
                           -variable BBAUDIO_ROUTE($meter) -value M_F6
   radiobutton $brroute.col2.$i\_mf7 -font bbmixer_font \
                           -command "BBAudio route $$var $meter" \
                           -variable BBAUDIO_ROUTE($meter) -value M_F7
   radiobutton $brroute.col2.$i\_mf8 -font bbmixer_font \
                           -command "BBAudio route $$var $meter" \
                           -variable BBAUDIO_ROUTE($meter) -value M_F8
   radiobutton $brroute.col2.$i\_none -font bbmixer_font -bg "light yellow" \
                           -command "BBAudio route $$var $meter" \
                           -variable BBAUDIO_ROUTE($meter) -value NONE
}


#
# Set up layout of rows and columns for Meter
#
grid $brroute.col2.label -row 0 -column 0
grid $brroute.col2.c25   -row 0 -column 1

foreach { col } { 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 } {
   grid $brroute.col2.c$col -row 0 -column [expr $col + 1]
}

set row_no 1

foreach { source } { mtr0 mtr1 mtr2 mtr3 } {
   set col_no 0
   foreach { col } { label none l1 mic sinad file fgen0 fgen1 fgen2 fgen3 fgen4 spkr demod fgen mixl1 user0 user1 \
                     afctr msinad scope sclr mf4 mf5 mf6 mf7 mf8 } { 
      grid $brroute.col2.$source\_$col -row $row_no -column $col_no 
      incr col_no
   }
   incr row_no
}


#
#  Pack screen
#
pack $brroute.head1       -side top
pack $brroute.blank1      -side top
pack $brroute.col         -side top
pack $brroute.blank2      -side top
pack $brroute.head2       -side top
pack $brroute.blank3      -side top
pack $brroute.col2        -side top
}


#
#   build_squelch_screen
#
proc build_squelch_screen { name } {
   global bbaudio_font
   global BBAUDIO_ENGINE

   set squelch $name

   frame $squelch.modeset
   frame $squelch.limitset
   frame $squelch.in_mode
   frame $squelch.in_range
   frame $squelch.in_status
   frame $squelch.reset

#
# Set up Squelch Mode buttons
#
radiobutton $squelch.modeset.auto  -text "Auto" -font bbaudio_font \
                      -command { set_bbaudio_squelch_mode $BBAUDIO_ENGINE(squelch_scrn_mode) } \
                      -variable BBAUDIO_ENGINE(squelch_scrn_mode) \
                      -value AUTO
radiobutton $squelch.modeset.squelch  -text "SQUELCHED" -font bbaudio_font \
                      -command { set_bbaudio_squelch_mode $BBAUDIO_ENGINE(squelch_scrn_mode) } \
                      -variable BBAUDIO_ENGINE(squelch_scrn_mode) \
                      -value SQUELCHED
radiobutton $squelch.modeset.nosquelch  -text "NO SQUELCH" -font bbaudio_font \
                      -command { set_bbaudio_squelch_mode $BBAUDIO_ENGINE(squelch_scrn_mode) } \
                      -variable BBAUDIO_ENGINE(squelch_scrn_mode) \
                      -value NOSQUELCH
radiobutton $squelch.modeset.query  -text "Query" -font bbaudio_font \
                      -command { set_bbaudio_squelch_mode $BBAUDIO_ENGINE(squelch_scrn_mode) } \
                      -variable BBAUDIO_ENGINE(squelch_scrn_mode) \
                      -value QUERY
 
pack $squelch.modeset.auto       -side left -fill both -expand true
pack $squelch.modeset.squelch    -side left -fill both -expand true
pack $squelch.modeset.nosquelch  -side left -fill both -expand true
pack $squelch.modeset.query      -side left -fill both -expand true

#
# Set up Squelch auto limit value
#
label $squelch.limitset.label -text "Auto Limit"
entry $squelch.limitset.entry -relief sunken  -width 8 \
                       -textvariable BBAUDIO_ENGINE(squelch_scrn_limit)

bind  $squelch.limitset.entry  <Return> \
           { ::Receiver::update_squelch_level $BBAUDIO_ENGINE(squelch_scrn_limit) }

pack $squelch.limitset.label   -side left -fill both -expand true
pack $squelch.limitset.entry   -side left -fill both -expand true

#
# Display Squelch output information
#
label $squelch.in_mode.label -text "Reply: Squelch Mode"
label $squelch.in_mode.entry -textvariable BBAUDIO_ENGINE(SquelchModeIn)
pack  $squelch.in_mode.label -side top
pack  $squelch.in_mode.entry -side top

label $squelch.in_range.label -text "Reply: Squelch Limit"
label $squelch.in_range.entry -textvariable BBAUDIO_ENGINE(SquelchLevelIn)
pack  $squelch.in_range.label -side top
pack  $squelch.in_range.entry -side top

label $squelch.in_status.label -text "Reply: Squelch Status"
label $squelch.in_status.entry -textvariable BBAUDIO_ENGINE(SquelchStatusIn)
pack  $squelch.in_status.label -side top
pack  $squelch.in_status.entry -side top

#
#  Pack screen
#
pack $squelch.modeset   -side top
pack $squelch.limitset  -side top
pack $squelch.in_mode   -side top
pack $squelch.in_range  -side top
pack $squelch.in_status -side top
pack $squelch.reset     -side top
}


#
#  Allow the enable or disable of a generic "meter"
#
proc invalidate_squelch_input { } {
    global BBAUDIO_ENGINE

    set BBAUDIO_ENGINE(SquelchModeIn)    -555
    set BBAUDIO_ENGINE(SquelchStatusIn)  -555
    set BBAUDIO_ENGINE(SquelchLevelIn)   -555
}

