#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C)1998,99      DESIGN ENGINEERING      #
# --------------------------------------------------------------------------#
# FILE        : to_fgen.tcl                                                 #
# PRODUCT     : FITS                                                        #
# AUTHOR      : Steve Johnson                                               #
# DESCRIPTION : Interface to Function Generator via Message Bus             #
#                                                                           #
#############################################################################


#
#  Hardware Utilities System Number
#
set BBFgen(SYSTEM_ID)          7

#
#  FGEN Sub-Systems
#
set BBFgen(FREQ)               128
set BBFgen(SHAPE)              129
set BBFgen(MODE)               130
set BBFgen(LEVEL)              131
set BBFgen(DURATION)           132
set BBFgen(FUNCTION)           133
set BBFgen(TIME)               134
set BBFgen(ENCODE)             135
set BBFgen(TONE_SEQ)           136
set BBFgen(SINGLE_TONE)        137
set BBFgen(TRIGGER)            138
set BBFgen(DEVIATION)          139
set BBFgen(INFO)               140
set BBFgen(SYNCALL)            141


set BBFgen(Mode0)              OFF
set BBFgen(Mode1)              OFF
set BBFgen(Mode2)              OFF
set BBFgen(Mode3)              OFF
set BBFgen(Mode4)              OFF

set BBFgen(Durat0)             500
set BBFgen(Durat1)             500
set BBFgen(Durat2)             500
set BBFgen(Durat3)             500
set BBFgen(Durat4)             500

set BBFgen(Freq0)              1000
set BBFgen(Freq1)              1000
set BBFgen(Freq2)              1000
set BBFgen(Freq3)              1000
set BBFgen(Freq4)              1000

set BBFgen(AFlvl0)             1.0
set BBFgen(AFlvl1)             1.0
set BBFgen(AFlvl2)             1.0
set BBFgen(AFlvl3)             1.0
set BBFgen(AFlvl4)             1.0

set BBFgen(RFDev0)             1.0
set BBFgen(RFDev1)             1.0
set BBFgen(RFDev2)             1.0
set BBFgen(RFDev3)             1.0
set BBFgen(RFDev4)             1.0

set BBFgen(Shape0)             SINE
set BBFgen(Shape1)             SINE
set BBFgen(Shape2)             SINE
set BBFgen(Shape3)             SINE
set BBFgen(Shape4)             SINE


font create bbfgen_font -family helvetica -size 8


#
#  Function Generator, Set Freq ( 0 to 20000 Hz)
#
proc set_bbfgen_freq { lun hz } {
  global BBFgen

  message_bus_double $BBFgen(SYSTEM_ID) $BBFgen(FREQ) $lun $hz
}


#
#  Function Generator, Set Shape
#
proc set_bbfgen_shape { lun shape } {
  global BBFgen

  switch $shape {
    SINE      { set x 0 }
    TRIANGLE  { set x 1 }
    SQUARE    { set x 2 }
    RAMP      { set x 3 }
    DC        { set x 4 }
    ARBITRARY { set x 5 }
    default   { return  }
   }

#puts "set_bbfgen_shape  $BBFgen(SYSTEM_ID) $BBFgen(SHAPE) $lun $x"
  message_bus_int $BBFgen(SYSTEM_ID) $BBFgen(SHAPE) $lun $x
}

#
#  Function Generator, Set AF level ( 0 to 10.0000 volts )
#
proc set_bbfgen_level { lun volts } {
  global BBFgen

  message_bus_double $BBFgen(SYSTEM_ID) $BBFgen(LEVEL) $lun $volts
}

#
#  Function Generator, Set Burst Mode
#
proc set_bbfgen_burst_mode { lun type } {
  global BBFgen

  switch $type {
    OFF         { set x 0 }
    CONTINUOUS  { set x 1 }
    BURST       { set x 2 }
    default     { return }
   }

  message_bus_int $BBFgen(SYSTEM_ID) $BBFgen(MODE) $lun $x
}

#
#  Function Generator, Set Burst Duration ( 1 to 100000 mS )
#
proc set_bbfgen_burst_duration { lun mS } {
  global BBFgen

  message_bus_int $BBFgen(SYSTEM_ID) $BBFgen(DURATION) $lun $mS
}

#
#  Function Generator, Set Arbitrary Shape for 'set_bbfgen_shape'
#
proc set_bbfgen_arbitrary_shape { lun mS } {
  global BBFgen

  # stubbed out for now
  message_bus_int $BBFgen(SYSTEM_ID) $BBFgen(FUNCTION) $lun $mS
}

#
#  Function Generator, Set DTMF timings (mS mS mS)
#
proc set_bbfgen_dtmf_timing { lun burst space end } {
  global BBFgen
  global Send_Flags


  # send <index> <cal>
  set msg " { $BBFgen(SYSTEM_ID)  $BBFgen(TIME) $lun $Send_Flags } \
            { inline UDWORD $burst $space $end } "
#puts $msg
  tclMsgbus send $msg
}


#
#  Function Generator, Set DTMF digits
#
proc set_bbfgen_dtmf_digits { lun mS } {
  global BBFgen

  # stubbed out for now
  message_bus_int $BBFgen(SYSTEM_ID) $BBFgen(ENCODE) $lun $mS
}

#
#  Function Generator, Set Tone Sequence Elements
#
proc set_bbfgen_ts_elements { lun mS } {
  global BBFgen

  # stubbed out for now
  message_bus_int $BBFgen(SYSTEM_ID) $BBFgen(TONE_SEQ) $lun $mS
}


#
#  Function Generator, Trigger one or both Generators ( 0 is current, 
#                                                       1 is both )
#
proc set_bbfgen_trigger { lun state } {
  global BBFgen

  switch $state {
    CURRENT     { set x 0 }
    BOTH        { set x 1 }
    default     { return }
   }

  message_bus_int $BBFgen(SYSTEM_ID) $BBFgen(TRIGGER) $lun $x
}


#
#  Function Generator, Set RF Gen Deviation ( 0 to 40.000 kHz )
#
proc set_bbfgen_deviation { lun kHz } {
  global BBFgen

  message_bus_double $BBFgen(SYSTEM_ID) $BBFgen(DEVIATION) $lun $kHz
}

#
#  Function Generator, Synchronize
#
proc bbfgen_sync { } {
  global BBFgen

  message_bus_int $BBFgen(SYSTEM_ID) $BBFgen(SYNCALL) 1 0
}

