#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C)1998,99      DESIGN ENGINEERING      #
# --------------------------------------------------------------------------#
# FILE        : to_utils.tcl                                                #
# PRODUCT     : FITS                                                        #
# AUTHOR      : Steve Johnson                                               #
# DESCRIPTION : Interface to Utilities via Message Bus                      #
#                                                                           #
#############################################################################


#
#  Hardware Utilities System Number
#
set HwUtilEngine(SYSTEM_ID)             17

#
#  Receiver Sub-Systems
#
#set HwUtilEngine(CMD_VOLUME)            1
set HwUtilEngine(CMD_BACKLIGHT)          2
set HwUtilEngine(CMD_TCXO)               3
set HwUtilEngine(CMD_CRYPTO)             4
set HwUtilEngine(CMD_YIG_LOCK)           5
set HwUtilEngine(CMD_ENABLE)             6
set HwUtilEngine(CMD_INFO)               7
set HwUtilEngine(REP_TEMP)               8
set HwUtilEngine(CMD_10MHZ_REF)          9
#set HwUtilEngine(CMD_DEFAULT_VOL)      10
set HwUtilEngine(CMD_PTRESET)           11
set HwUtilEngine(CMD_FP_LEDS)           12
set HwUtilEngine(CMD_DMA_STATUS)        13
set HwUtilEngine(CMD_MIXER_ROUTER_CAL)  14
set HwUtilEngine(CMD_YIG_CAL)           15
set HwUtilEngine(CMD_YIGCAL_RESULT)     16

#
#  State variables for engineering screen
#
set HwUtilEngine(10mhz)            INT_REF
set HwUtilEngine(crypto)               OFF
set HwUtilEngine(shift_state)          OFF

 
set HwUtilEngine(status_handle)          0
set HwUtilEngine(yigcal_handle)          0
set HwUtilEngine(temp_handle)            0


font create hwutil_font -family helvetica -size 8

#
#  TCXO Caligration Data
#
set HwUtilEngine(TCXO)                  123

#
#  Current IF/Video Temperature
#
set HwUtilEngine(IF_temp)   999

#
#  Current IF/Video Temperature
#
set HwUtilEngine(yigcal_status)   "Initial"

####################################################################
#
#   Get DMA2 status from MF DSP
#
#    status         status message from DSP
#
#    xx0000         find owner     255 none turned on
#                                    1 one owner for DMA
#                                    2 two requests for DMA
#    0000xx         Shut off owner 255 nothing shut off
#                                    0 analyzer
#                                    1 mic/sinad       
#    00xx00         Turn on        255 already on or none are on
#                                    0 analyzer
#                                    1 mic/sinad  
#
####################################################################
proc handle_dma_status {sys sub lun flags uid status } { 

  set off [ expr $status & 0xff]
  set on  [ expr $status & 0xff00]
  set own [ expr $status & 0xff0000]

  set on  [ expr $on  >> 8  ]
  set own [ expr $own >> 16 ]

#  puts "handle_dma_status: [ format " %5X %5X %5X %5X" $off $on $own $status ]"

  if { $own > 1 && $own != 255 } {
    if { $off } {
      puts "Conflict w/ Analyzer: Mic-Sinad Turned Off!"
    } else  {
      puts "Conflict w/ Mic-Sinad: Analyzer Turned Off!"
    }
  }
}
####################################################################
#
#   get current cals from DSP
#
####################################################################
proc handle_yigcal_status {resys sub lun flags uid status rcv0 rcv2800 gen0 gen2800 } { 
  global RcvrYigCals
  global RfGenYigCals
  global HwUtilEngine

#puts "handle_yigcal_status $status $rcv0 $rcv2800 $gen0 $gen2800"

  set math     [ expr {$status & 4095}]
  set err_gen  [ format "%03X" $math ]
  set math     [ expr {$status >> 12} ]
  set err_rcv  [ format "%03X" $math ]

  if { $status == 0 } {
   set RcvrYigCals(temp)           "Complete"
   set RfGenYigCals(temp)          "Complete"
   set HwUtilEngine(yigcal_status) "Complete"
  } else {
   set RcvrYigCals(temp)           "Err $err_rcv"
   set RfGenYigCals(temp)          "Err $err_gen"
   set HwUtilEngine(yigcal_status) "Err R:$err_rcv G:$err_gen"
  }

  
  set RcvrYigCals(0)     $rcv0
  set RcvrYigCals(2800)  $rcv2800

  set RfGenYigCals(0)    $gen0
  set RfGenYigCals(2800) $gen2800
}

####################################################################
#
#   get current temperature from DSP
#
####################################################################
proc handle_ifvideo_temp {resys sub lun flags uid temp } { 
  global HwUtilEngine

#puts "handle_ifvideo_temp $temp"

  set HwUtilEngine(IF_temp)  [ format "%2.1f" $temp ]
}


#############################################################
#       Name: create_utils_callback
#############################################################
proc create_utils_callback { } {
  global HwUtilEngine

  if {$HwUtilEngine(status_handle) == 0} {
    set HwUtilEngine(status_handle) [tclMsgbus rcv handle \
                                    -sys $HwUtilEngine(SYSTEM_ID) \
                                    -sub $HwUtilEngine(CMD_DMA_STATUS) \
                                    -lun 1 \
                                    -cmd "handle_dma_status" \
                                    -mask 8]
  }

  if {$HwUtilEngine(yigcal_handle) == 0} {
    set HwUtilEngine(yigcal_handle) [tclMsgbus rcv handle \
                                    -sys $HwUtilEngine(SYSTEM_ID) \
                                    -sub $HwUtilEngine(CMD_YIGCAL_RESULT) \
                                    -lun 1 \
                                    -cmd "handle_yigcal_status" \
                                    -mask 8]
  }

  if {$HwUtilEngine(temp_handle) == 0} {
    set HwUtilEngine(temp_handle) [tclMsgbus rcv handle \
                                    -sys $HwUtilEngine(SYSTEM_ID) \
                                    -sub $HwUtilEngine(REP_TEMP) -lun 1 \
                                    -cmd "handle_ifvideo_temp" -mask 8]
  }

}

#############################################################
#
#  Message bus interfaces - removed from use
#
#############################################################
proc destroy_utils_callback { } {
  global HwUtilEngine

  if {$HwUtilEngine(status_handle) != 0 } {
    tclMsgbus $HwUtilEngine(status_handle) destroy
  }

   set HwUtilEngine(status_handle) 0
}


#####################################################################
#
#
#     Interface to VxWorks Engine
#
#
######################################################################


#
#  Hardware Utilities backlight control ( 0 to 4095 )
#
proc set_hwutil_backlight { level } {
  global HwUtilEngine

  message_bus_int $HwUtilEngine(SYSTEM_ID) $HwUtilEngine(CMD_BACKLIGHT) \
                  1 $level
}

#
#  Hardware Utilities TCXO control ( 0 to 4095 )
#
proc set_hwutil_tcxo { set_value } {
  global HwUtilEngine

  message_bus_int $HwUtilEngine(SYSTEM_ID) $HwUtilEngine(CMD_TCXO) \
                  1 $set_value
}

proc trace_hwutil_tcxo { } {
  global HwUtilEngine

  set_hwutil_tcxo $HwUtilEngine(TCXO)
}

#
#  Hardware Utilities crypto control
#
proc set_hwutil_crypto { state } {
  global HwUtilEngine

  switch $state {
    ON  { set x 0 }
    OFF { set x 1 }
    default { return }
   } 

  message_bus_int $HwUtilEngine(SYSTEM_ID) $HwUtilEngine(CMD_CRYPTO) 1 $x
}

#
#  Hardware Utilities Receiver & RF Gen Yig & Synth Lock message output
#
proc set_rcvr_gen_yig_lock_msg { state } {
  global HwUtilEngine

  switch $state {
    ON  { set x 0 }
    OFF { set x 1 }
    default { return }
   } 

  message_bus_int $HwUtilEngine(SYSTEM_ID) $HwUtilEngine(CMD_YIG_LOCK) 1 $x
}


#
#  Hardware Utilities set 10 MHz reference source
#
proc set_hwutil_10mhz_ref { x } {
  global HwUtilEngine

  switch $x {
    INT_REF { set ref 1 }
    EXT_REF { set ref 0 }
    default { return }
   } 

  message_bus_int $HwUtilEngine(SYSTEM_ID) $HwUtilEngine(CMD_10MHZ_REF) \
                  1 $ref
}


#
#  Hardware Utilities - trace function for tcxo calibration
#
proc trace_tcxo_cal_data { name element op } {
  global HwUtilEngine

  set_hwutil_tcxo $HwUtilEngine(TCXO)
}


trace variable HwUtilEngine(TCXO) w trace_tcxo_cal_data


#
#  Hardware Utilities - default volume (called 1st after unit is powered up)
#
proc set_hwutil_default_volume { } {
  global HwUtilEngine

  message_bus_int $HwUtilEngine(SYSTEM_ID) $HwUtilEngine(CMD_DEFAULT_VOL) 1 0
}

#
#  reset power term
#
proc set_pt_reset {  } {
  global HwUtilEngine

  message_bus_int $HwUtilEngine(SYSTEM_ID) $HwUtilEngine(CMD_PTRESET) 1 0
}

#
#   Shift key light
#
proc set_shift_led { status } {
  global HwUtilEngine
  global Send_Flags

  if {$status == "ON"} {
    set HwUtilEngine(shift_state) 1
  } elseif {$status == "OFF"} {
    set HwUtilEngine(shift_state) 0
  } else {
    return
  }

  set mask 1
 
  # send <index> <cal>
  set msg " { $HwUtilEngine(SYSTEM_ID)  \
              $HwUtilEngine(CMD_FP_LEDS) 1 $Send_Flags } \
            { inline dw $HwUtilEngine(shift_state) $mask } "
#puts $msg
  tclMsgbus send $msg
}

proc toggle_shift_led { } {
  global HwUtilEngine

  if { $HwUtilEngine(shift_state) == 1 } {
    set_shift_led OFF
  } else {
    set_shift_led ON
  }
}

#*************************************************
#       Name: hw_utils_run
# Parameters: none
#Description: 
#*************************************************
proc hw_utils_run { x } {
  global HwUtilEngine

  switch $x {
    OFF     { message_bus_int $HwUtilEngine(SYSTEM_ID) \
                       $HwUtilEngine(CMD_ENABLE) 1 0x00 }
    DEMO     -
    ALL     { message_bus_int $HwUtilEngine(SYSTEM_ID) \
                       $HwUtilEngine(CMD_ENABLE) 1 0x0e }
   }
}


#
#   Send Cal Values to MF DSP
#
proc set_mixer_router_cal { name index cal } {
  global HwUtilEngine
  global Send_Flags

  set HwUtilEngine(bbcal.$index.name) $name
  set HwUtilEngine(bbcal.$index.cal)  $cal


  set msg " { $HwUtilEngine(SYSTEM_ID) $HwUtilEngine(CMD_MIXER_ROUTER_CAL) \
              1 $Send_Flags } \
            { inline sdword $index } \
            { inline double $cal } "
#puts $msg
  tclMsgbus send $msg
}




#############################################################################
#                                                                           #
#                       Calibration Interfaces                              #
#                                                                           #
#############################################################################

#
# parse tcxo cal 
#
# The TCXO Calibrations have the form:
# TCXOCAL   Value  25                      
#
proc parse_tcxo_cal { line } {
  global HwUtilEngine

# puts $line
  switch [ lindex $line 1 ] {
    Value { set HwUtilEngine(TCXO) [ lindex $line 2 ] }
  }

#puts $HwUtilEngine(TCXO)
}


#
# update tcl tcxo cal data
#
proc get_tcxo_cal_data { filename } {
  global CalFileName

  set file [ OpenCalFile $filename r ]

  while { [ gets $file line ] >= 0 } {
# puts $line
    if { [ lindex $line 0 ] == "TCXOCAL" } {
      parse_tcxo_cal $line
    }
  }

  close $file
}


#
# update mixer/route level cal data
#
proc get_mixer_router_cal_data { filename } {
  global CalFileName
  global HwUtilEngine

  set file [ OpenCalFile $filename r ]

  while { [ gets $file line ] >= 0 } {
# puts $line
    if { [ lindex $line 0 ] == "BB_LEVEL_CAL" } {
      set_mixer_router_cal [ lindex $line 1 ] [ lindex $line 2 ] \
                           [ lindex $line 4 ]
    }
    if { [ lindex $line 0 ] == "BB_LEVEL_MAX" } {
       set HwUtilEngine(bbcal.max) [ lindex $line 1 ]
    }
  }

  close $file
}

#
# update mixer/route level cal data
#
proc write_mixer_router_cal_data { filename } {
  global CalFileName
  global HwUtilEngine

  set file [ OpenCalFile $filename w ]

  puts $file "# Test Set Calibrations\n"
  puts $file "BB_LEVEL_MAX   $HwUtilEngine(bbcal.max)"

  for { set i 0 } { $i < $HwUtilEngine(bbcal.max) } { incr i } {
    set msg [ format "BB_LEVEL_CAL %10s %d   Value %f" \
                      $HwUtilEngine(bbcal.$i.name) $i \
                      $HwUtilEngine(bbcal.$i.cal) ]

    puts $file $msg

    set_mixer_router_cal $HwUtilEngine(bbcal.$i.name) \
                         $i $HwUtilEngine(bbcal.$i.cal)

  }

  flush $file
  close $file
}

proc update_tcxo_cal_data { } {
  global HwUtilEngine

  set_hwutil_tcxo $HwUtilEngine(TCXO)
}

#*************************************************
#       Name: set_start_yig_calibration
# Parameters: none
#Description: 
#*************************************************
proc set_start_yig_calibration { } {
  global HwUtilEngine
  global RcvrYigCals
  global RfGenYigCals

  set RcvrYigCals(temp)           "Running"
  set RfGenYigCals(temp)          "Running"
  set HwUtilEngine(yigcal_status) "Running"

  message_bus_int $HwUtilEngine(SYSTEM_ID) $HwUtilEngine(CMD_YIG_CAL) 1 1
}
