#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C)1998,99      DESIGN ENGINEERING      #
# --------------------------------------------------------------------------#
# FILE        : screen.tcl                                                  #
# PRODUCT     : FITS                                                        #
# AUTHOR      : Steve Johnson                                               #
# DESCRIPTION : channel analyzer demo screen                                #
#                                                                           #
#############################################################################

#
# Display state variables
#
set ChAnlzState(ifbw)  3KHZ  
set ChAnlzState(adc)   CHANLZ 
set ChAnlzState(vidbw) BW100K

set ANLZR_ENGINE(persist_state) 0


#
#  allow a manual reset of all hardware
#
proc reset_ch_anlzr_hardware { } {
  global ANLZR_ENGINE
  global ChAnlzState

  set_anlzr_adc_input CHANLZ
  set_anlzr_ifchbw $ChAnlzState(ifbw)
  set_anlzr_sweep $ANLZR_ENGINE(sweep.tr) 2
  set_anlzr_span 1 $ANLZR_ENGINE(span) -1.0
  set_anlzr_ifchgain 0
  set_anlzr_tos -10
  set_anlzr_scale 10
}

#
# set channel persistance
#
proc anlz_persistance { } {
  global ANLZR_ENGINE

  if { $ANLZR_ENGINE(trace1) != -1 } {
    $ANLZR_ENGINE(trace1) config -persist $ANLZR_ENGINE(persist_state)
  }
}


#
# set channel active state
#
proc anlz_onoff { } {
  global ANLZR_ENGINE

  if { $ANLZR_ENGINE(trace1) != -1 } {
    $ANLZR_ENGINE(trace1).trace1 config -active $ANLZR_ENGINE(trace_state)
  }
}


#
# doBWSpan
#
proc doBWSpan { tg } {
  global ChAnlzState
  global ANLZR_ENGINE

  message $tg.msg   -text "Span (MHz)"  -width 3c -font anlzr_font
  entry $tg.entry -textvariable  ANLZR_ENGINE(span) -width 5 \
                  -bg white -font anlzr_font

  bind $tg.entry <Return> { set_anlzr_span 1 $ANLZR_ENGINE(span) -1.0 } 

  pack $tg.msg      -side left -fill both -expand true
  pack $tg.entry
}


#
# doChanlzrSweep
#
proc doChanlzrSweep { tg } {
  global ChAnlzState
  global ANLZR_ENGINE

  message $tg.msg1  -text "Sweep (uS/pt)"     -width 3c -font anlzr_font
  entry $tg.entry1  -textvariable  ANLZR_ENGINE(sweep.pt) -width 8 \
                                -font anlzr_font -bg white
  message $tg.msg2  -text "Sweep (uS/trace)"  -width 3c -font anlzr_font
  entry $tg.entry2  -textvariable  ANLZR_ENGINE(sweep.tr) -width 8 \
                                -font anlzr_font -bg white

  bind $tg.entry1 <Return> { set_anlzr_sweep $ANLZR_ENGINE(sweep.pt) 1 }
  bind $tg.entry2 <Return> { set_anlzr_sweep $ANLZR_ENGINE(sweep.tr) 2 }

  pack $tg.msg1      -side top -fill both -expand true
  pack $tg.entry1    -side top
  pack $tg.msg2      -side top -fill both -expand true
  pack $tg.entry2    -side top
}


#
# doChanlzrDDSFreq
#
proc doChanlzrDDSFreq { tg } {
  global ChAnlzState
  global ANLZR_ENGINE

  message $tg.msg   -text "IF DDS Freq (MHz)"  -width 3c \
                    -font anlzr_font
  entry $tg.entry -textvariable  ANLZR_ENGINE(dds_freq) -width 5 \
                  -font anlzr_font -bg white


  bind $tg.entry <Return> { set_anlzr_if_dds_freq $ANLZR_ENGINE(dds_freq) } 

  pack $tg.msg      -side top  -fill both -expand true
  pack $tg.entry    -side top  -fill both -expand true
}


#
# doAnlzrIfgain
#
proc doAnlzrIfgain { tg } {
  global ChAnlzState

  label $tg.title -text "Sweep > 5MHz Main, else Channel"  -font anlzr_font
  scale $tg.adj   -label "IF Gain (dB)"  -from -14.0 -to 82.0 \
                  -resolution 0.5 -length 3c -orient horizontal \
                  -command set_anlzr_ifchgain -font anlzr_font

  pack $tg.title  -side top  -fill both -expand true
  pack $tg.adj    -side top  -fill both -expand true
}


#
# doChanlzrTraceLength
#
proc doChanlzrTraceLength { tg } {
  global ANLZR_ENGINE

  label $tg.title -text "Analyzer Trace Length" 
  label $tg.title2 -text "(re-Start trace after length change)" 

  label $tg.msg   -text "Trace Length (pts: 2-1000) "
  entry $tg.entry   -textvariable  ANLZR_ENGINE(trace_length) -width 6 \
                    -relief sunken -bg white

  bind $tg.entry <Return> { reset_anlzr_trace_length } 

  pack $tg.title    -side top  
  pack $tg.title2   -side top 
  pack $tg.msg      -side left
  pack $tg.entry    -side left
}


set AnlzrCalTableIndex 2
set AnlzrCalTableRoot  1
proc anlzr_update_sw_cal { } {
  global AnlzrCals
  global AnlzrCalTableIndex
  global AnlzrCalTableRoot
  global ANLZR_ENGINE

  do_anlzr_cals bwswitch $AnlzrCalTableIndex \
                $AnlzrCals($AnlzrCalTableIndex.khz) \
                $AnlzrCals($AnlzrCalTableIndex.norm) 0.0

  set_anlzr_span 1 $ANLZR_ENGINE(span) -1.0
  set_anlzr_tos  $ANLZR_ENGINE(tos)
}

#
# doChanlzrBWSwCalTable
#
proc doChanlzrBWSwCalTable { tg } {
  global AnlzrCals
  global AnlzrCalTableIndex
  global AnlzrCalTableRoot
  global ANLZR_ENGINE

  set AnlzrCalTableRoot $tg

  frame $tg.table
  frame $tg.save
  frame $tg.info

  frame $tg.table.index  -relief groove -borderwidth 2
  frame $tg.table.freq   -relief groove -borderwidth 2
  frame $tg.table.norm   -relief groove -borderwidth 2

  label $tg.table.index.msg   -text "IF BW Index (1-$AnlzrCals(max))" -font anlzr_font
  entry $tg.table.index.entry -textvariable AnlzrCalTableIndex \
                              -width 3 -relief sunken -bg white -font anlzr_font

  label $tg.table.freq.msg    -text "Freq (kHz)" -font anlzr_font
  entry $tg.table.freq.entry  -textvariable  AnlzrCals($AnlzrCalTableIndex.khz) \
                              -width 9 -relief sunken -bg white -font anlzr_font

  label $tg.table.norm.msg    -text "Normalize (dB)" -font anlzr_font
  entry $tg.table.norm.entry  -textvariable AnlzrCals($AnlzrCalTableIndex.norm) \
                              -width 9 -relief sunken -bg white -font anlzr_font

  entry $tg.info.entry1       -textvariable  AnlzrCals(date) \
                              -width 12 -relief sunken -bg white -font anlzr_font
  entry $tg.info.entry2       -textvariable AnlzrCals(time) \
                              -width 12 -relief sunken -bg white -font anlzr_font

  button $tg.save.put         -text  "Save to Cal File" -font anlzr_font \
                              -command { put_anlzr_cal_data analyzer; \
                                         anlzr_update_sw_cal } -relief raised
  button $tg.save.get         -text  "Retrieve Cal File" -font anlzr_font \
                              -command { get_anlzr_cal_data analyzer } -relief raised

  bind $AnlzrCalTableRoot.table.index.entry <Return> {
    $AnlzrCalTableRoot.table.freq.entry  configure \
           -textvariable  AnlzrCals($AnlzrCalTableIndex.khz)
    $AnlzrCalTableRoot.table.norm.entry  configure \
           -textvariable AnlzrCals($AnlzrCalTableIndex.norm)
  }

  bind $AnlzrCalTableRoot.table.freq.entry <Enter> {
    bind $AnlzrCalTableRoot.table.freq.entry <F27> { 
           set AnlzrCals($AnlzrCalTableIndex.khz) \
              [ expr $AnlzrCals($AnlzrCalTableIndex.khz) + 0.02]
           anlzr_update_sw_cal 
    }
    bind $AnlzrCalTableRoot.table.freq.entry <F26> { 
           set AnlzrCals($AnlzrCalTableIndex.khz) \
              [ expr $AnlzrCals($AnlzrCalTableIndex.khz) - 0.02]
           anlzr_update_sw_cal
    }
  }

  bind $AnlzrCalTableRoot.table.norm.entry <Enter> {
    bind $AnlzrCalTableRoot.table.norm.entry <F27> { 
           set AnlzrCals($AnlzrCalTableIndex.norm) \
              [ expr $AnlzrCals($AnlzrCalTableIndex.norm) + 0.1]
           anlzr_update_sw_cal
    }
    bind $AnlzrCalTableRoot.table.norm.entry <F26> { 
           set AnlzrCals($AnlzrCalTableIndex.norm) \
              [ expr $AnlzrCals($AnlzrCalTableIndex.norm) - 0.1]
           anlzr_update_sw_cal
    }
  }

  bind $AnlzrCalTableRoot.table.freq.entry <Return> { anlzr_update_sw_cal }
  bind $AnlzrCalTableRoot.table.norm.entry <Return> { anlzr_update_sw_cal }



  pack $tg.table.index.msg    -side top
  pack $tg.table.index.entry  -side top

  pack $tg.table.freq.msg     -side top
  pack $tg.table.freq.entry   -side top

  pack $tg.table.norm.msg     -side top
  pack $tg.table.norm.entry   -side top

  pack $tg.info.entry1        -side top
  pack $tg.info.entry2        -side top

  pack $tg.save.put           -side top
  pack $tg.save.get           -side top

  pack $tg.table.index        -side left
  pack $tg.table.freq         -side left
  pack $tg.table.norm         -side left

  pack $tg.table              -side top
  pack $tg.save               -side left
  pack $tg.info               -side left
}


#
# doAnlzrCfCalTable
#
set AnlzrCalCfTableIndex   1
set AnlzrCalCfTableRoot    1
proc doAnlzrCfCalTable { tg } {
  global AnlzrCals
  global AnlzrCalCfTableIndex
  global AnlzrCalCfTableRoot
  global ANLZR_ENGINE

  set AnlzrCalCfTableRoot $tg

  frame $tg.table
  frame $tg.save
  frame $tg.info

  frame $tg.table.index  -relief groove -borderwidth 2
  frame $tg.table.freq   -relief groove -borderwidth 2
  frame $tg.table.norm   -relief groove -borderwidth 2
  frame $tg.table.pt     -relief groove -borderwidth 2

  label $tg.table.index.msg   -text "Index (0-$AnlzrCals(cfmax))" -font anlzr_font
  entry $tg.table.index.entry -textvariable AnlzrCalCfTableIndex \
                              -width 3 -relief sunken -bg white -font anlzr_font

  label $tg.table.freq.msg    -text "Freq (MHz)" -font anlzr_font
  entry $tg.table.freq.entry  -textvariable  AnlzrCals($AnlzrCalCfTableIndex.cf.mhz) \
                              -width 9 -relief sunken -bg white -font anlzr_font

  label $tg.table.norm.msg    -text "CF Pwr (dBm)" -font anlzr_font
  entry $tg.table.norm.entry  -textvariable AnlzrCals($AnlzrCalCfTableIndex.cf.dbm) \
                              -width 9 -relief sunken -bg white -font anlzr_font

  label $tg.table.pt.msg      -text "Pwr Term (Watts)" -font anlzr_font
  entry $tg.table.pt.entry    -textvariable AnlzrCals($AnlzrCalCfTableIndex.cf.watts) \
                              -width 9 -relief sunken -bg white -font anlzr_font

  entry $tg.info.entry1       -textvariable  AnlzrCals(date) \
                              -width 12 -relief sunken -bg white -font anlzr_font
  entry $tg.info.entry2       -textvariable AnlzrCals(time) \
                              -width 12 -relief sunken -bg white -font anlzr_font

  button $tg.save.put         -text  "Save to Cal File" -font anlzr_font \
                              -command { put_anlzr_cal_data analyzer } -relief raised
  button $tg.save.get         -text  "Retrieve Cal File" -font anlzr_font \
                              -command { get_anlzr_cal_data analyzer } -relief raised

  bind $AnlzrCalCfTableRoot.table.index.entry <Return> {
    $AnlzrCalCfTableRoot.table.freq.entry  configure \
           -textvariable  AnlzrCals($AnlzrCalCfTableIndex.cf.mhz)
    $AnlzrCalCfTableRoot.table.norm.entry  configure \
           -textvariable AnlzrCals($AnlzrCalCfTableIndex.cf.dbm)
    $AnlzrCalCfTableRoot.table.pt.entry  configure \
           -textvariable AnlzrCals($AnlzrCalCfTableIndex.cf.watts)
    Receiver freq $AnlzrCals($AnlzrCalCfTableIndex.cf.mhz) MHz
  }

  pack $tg.table.index.msg    -side top
  pack $tg.table.index.entry  -side top

  pack $tg.table.freq.msg     -side top
  pack $tg.table.freq.entry   -side top

  pack $tg.table.norm.msg     -side top
  pack $tg.table.norm.entry   -side top

  pack $tg.table.pt.msg       -side top
  pack $tg.table.pt.entry     -side top

  pack $tg.info.entry1        -side top
  pack $tg.info.entry2        -side top

  pack $tg.save.put           -side top
  pack $tg.save.get           -side top

  pack $tg.table.index        -side left
  pack $tg.table.freq         -side left
  pack $tg.table.norm         -side left
  pack $tg.table.pt           -side left

  pack $tg.table              -side top
  pack $tg.save               -side left
  pack $tg.info               -side left
}


#
# doAnlzrAntInputCalTable
#
set AnlzrCalAntTableIndex   1
set AnlzrCalAntTableRoot   -1
proc doAnlzrAntInputCalTable { tg } {
  global AnlzrCals
  global AnlzrCalAntTableIndex
  global AnlzrCalAntTableRoot
  global ANLZR_ENGINE

  set AnlzrCalAntTableRoot $tg

  frame $tg.table
  frame $tg.save
  frame $tg.info

  frame $tg.table.index
  frame $tg.table.freq
  frame $tg.table.pwr

  label $tg.table.index.msg   -text "Index (0-$AnlzrCals(antmax)-1)" -font anlzr_font
  entry $tg.table.index.entry -textvariable AnlzrCalAntTableIndex \
                              -width 3 -relief sunken -bg white -font anlzr_font

  label $tg.table.freq.msg    -text "Freq (MHz)" -font anlzr_font
  entry $tg.table.freq.entry  -textvariable  AnlzrCals($AnlzrCalAntTableIndex.ant.mhz) \
                              -width 9 -relief sunken -bg white -font anlzr_font

  label $tg.table.pwr.msg     -text "Ant Correction (dB)" -font anlzr_font
  entry $tg.table.pwr.entry   -textvariable AnlzrCals($AnlzrCalAntTableIndex.ant.db) \
                              -width 9 -relief sunken -bg white -font anlzr_font


  entry $tg.info.entry1       -textvariable  AnlzrCals(date) \
                              -width 12 -relief sunken -bg white -font anlzr_font
  entry $tg.info.entry2       -textvariable AnlzrCals(time) \
                              -width 12 -relief sunken -bg white -font anlzr_font

  button $tg.save.put         -text  "Save to Cal File" -font anlzr_font \
                              -command { put_anlzr_cal_data analyzer } -relief raised
  button $tg.save.get         -text  "Retrieve Cal File" -font anlzr_font \
                              -command { get_anlzr_cal_data analyzer } -relief raised


  bind $AnlzrCalAntTableRoot.table.index.entry <Return> {
    $AnlzrCalAntTableRoot.table.freq.entry  configure \
           -textvariable  AnlzrCals($AnlzrCalAntTableIndex.ant.mhz)
    $AnlzrCalAntTableRoot.table.pwr.entry  configure \
           -textvariable AnlzrCals($AnlzrCalAntTableIndex.ant.db)
    Receiver freq $AnlzrCals($AnlzrCalAntTableIndex.ant.mhz) MHz
  }

  pack $tg.table.index.msg    -side top
  pack $tg.table.index.entry  -side top

  pack $tg.table.freq.msg     -side top
  pack $tg.table.freq.entry   -side top

  pack $tg.table.pwr.msg      -side top
  pack $tg.table.pwr.entry    -side top

  pack $tg.info.entry1        -side top
  pack $tg.info.entry2        -side top

  pack $tg.save.put           -side top
  pack $tg.save.get           -side top

  pack $tg.table.index        -side left
  pack $tg.table.freq         -side left
  pack $tg.table.pwr          -side left

  pack $tg.table              -side top
  pack $tg.save               -side left
  pack $tg.info               -side left
}


#
# doAnlzrTRInputCalTable
#
set AnlzrCalTRTableIndex   1
set AnlzrCalTRTableRoot   -1
proc doAnlzrTRInputCalTable { tg } {
  global AnlzrCals
  global AnlzrCalTRTableIndex
  global AnlzrCalTRTableRoot
  global ANLZR_ENGINE

  set AnlzrCalTRTableRoot $tg

  frame $tg.table
  frame $tg.save
  frame $tg.info

  frame $tg.table.index
  frame $tg.table.freq
  frame $tg.table.pwr

  label $tg.table.index.msg   -text "Index (0-$AnlzrCals(trmax)-1)" -font anlzr_font
  entry $tg.table.index.entry -textvariable AnlzrCalTRTableIndex \
                              -width 3 -relief sunken -bg white -font anlzr_font

  label $tg.table.freq.msg    -text "Freq (MHz)" -font anlzr_font
  entry $tg.table.freq.entry  -textvariable  AnlzrCals($AnlzrCalTRTableIndex.tr.mhz) \
                              -width 9 -relief sunken -bg white -font anlzr_font

  label $tg.table.pwr.msg     -text "TR Correction (dB)" -font anlzr_font
  entry $tg.table.pwr.entry   -textvariable AnlzrCals($AnlzrCalTRTableIndex.tr.db) \
                              -width 9 -relief sunken -bg white -font anlzr_font


  entry $tg.info.entry1       -textvariable  AnlzrCals(date) \
                              -width 12 -relief sunken -bg white -font anlzr_font
  entry $tg.info.entry2       -textvariable AnlzrCals(time) \
                              -width 12 -relief sunken -bg white -font anlzr_font

  button $tg.save.put         -text  "Save to Cal File" -font anlzr_font \
                              -command { put_anlzr_cal_data analyzer } -relief raised
  button $tg.save.get         -text  "Retrieve Cal File" -font anlzr_font \
                              -command { get_anlzr_cal_data analyzer } -relief raised


  bind $AnlzrCalTRTableRoot.table.index.entry <Return> {
    $AnlzrCalTRTableRoot.table.freq.entry  configure \
           -textvariable  AnlzrCals($AnlzrCalTRTableIndex.tr.mhz)
    $AnlzrCalTRTableRoot.table.pwr.entry  configure \
           -textvariable AnlzrCals($AnlzrCalTRTableIndex.tr.db)
    Receiver freq $AnlzrCals($AnlzrCalTRTableIndex.tr.mhz) MHz 
  }

  pack $tg.table.index.msg    -side top
  pack $tg.table.index.entry  -side top

  pack $tg.table.freq.msg     -side top
  pack $tg.table.freq.entry   -side top

  pack $tg.table.pwr.msg      -side top
  pack $tg.table.pwr.entry    -side top

  pack $tg.info.entry1        -side top
  pack $tg.info.entry2        -side top

  pack $tg.save.put           -side top
  pack $tg.save.get           -side top

  pack $tg.table.index        -side left
  pack $tg.table.freq         -side left
  pack $tg.table.pwr          -side left

  pack $tg.table              -side top
  pack $tg.save               -side left
  pack $tg.info               -side left
}

#
# doChanlzrTraceWidgetWidth
#
proc doChanlzrTraceWidgetWidth { tg } {
  global ANLZR_ENGINE

  label $tg.title -text "Analyzer TraceWidget Width" 

  label $tg.msg   -text "Width (pts: 2-600) "
  entry $tg.entry   -textvariable  ANLZR_ENGINE(WidgetWidth) -width 6 \
                    -relief sunken -bg white

  bind $tg.entry <Return> {
                            $ANLZR_ENGINE(trace1) config \
                                -width $ANLZR_ENGINE(WidgetWidth)
                          } 

  pack $tg.title    -side top
  pack $tg.msg      -side left
  pack $tg.entry    -side left
}


 #
 # doChanlzrVideoBW   
 #
 proc doChanlzrVideoBW { tg } {
   global ChAnlzState

 message $tg.msg -text "Video BW" -width 3c -font anlzr_font


 radiobutton $tg.bw10  -text "10" -font anlzr_font \
        -command { set_anlzr_video_bw $ChAnlzState(vidbw) } \
        -variable ChAnlzState(vidbw) \
        -value BW10   
 radiobutton $tg.bw30  -text "30" -font anlzr_font \
        -command { set_anlzr_video_bw $ChAnlzState(vidbw) } \
        -variable ChAnlzState(vidbw) \
        -value BW30   
 radiobutton $tg.bw100  -text "100" -font anlzr_font \
        -command { set_anlzr_video_bw $ChAnlzState(vidbw) } \
        -variable ChAnlzState(vidbw) \
        -value BW100   
 radiobutton $tg.bw300  -text "300" -font anlzr_font \
        -command { set_anlzr_video_bw $ChAnlzState(vidbw) } \
        -variable ChAnlzState(vidbw) \
        -value BW300   
 radiobutton $tg.bw1k   -text "1K" -font anlzr_font \
        -command { set_anlzr_video_bw $ChAnlzState(vidbw) } \
        -variable ChAnlzState(vidbw) \
        -value BW1K   
 radiobutton $tg.bw3k   -text "3K" -font anlzr_font \
        -command { set_anlzr_video_bw $ChAnlzState(vidbw) } \
        -variable ChAnlzState(vidbw) \
        -value BW3K   
 radiobutton $tg.bw10k  -text "10K" -font anlzr_font \
        -command { set_anlzr_video_bw $ChAnlzState(vidbw) } \
        -variable ChAnlzState(vidbw) \
        -value BW10K   
 radiobutton $tg.bw30k  -text "30K" -font anlzr_font \
        -command { set_anlzr_video_bw $ChAnlzState(vidbw) } \
        -variable ChAnlzState(vidbw) \
        -value BW30K   
 radiobutton $tg.bw100k -text "100K" -font anlzr_font \
        -command { set_anlzr_video_bw $ChAnlzState(vidbw) } \
        -variable ChAnlzState(vidbw) \
        -value BW100K   
 radiobutton $tg.bw300k -text "300K" -font anlzr_font \
        -command { set_anlzr_video_bw $ChAnlzState(vidbw) } \
        -variable ChAnlzState(vidbw) \
        -value BW300K   
 radiobutton $tg.bw1m   -text "1M" -font anlzr_font \
        -command { set_anlzr_video_bw $ChAnlzState(vidbw) } \
        -variable ChAnlzState(vidbw) \
        -value BW1M   
 radiobutton $tg.bw3m   -text "3M" -font anlzr_font \
        -command { set_anlzr_video_bw $ChAnlzState(vidbw) } \
        -variable ChAnlzState(vidbw) \
        -value BW3M   
 radiobutton $tg.bwoff  -text "Disabled" -font anlzr_font \
        -command { set_anlzr_video_bw $ChAnlzState(vidbw) } \
        -variable ChAnlzState(vidbw) \
        -value BW_OFF   
    
   pack $tg.msg     -side top  -fill both -expand true
   pack $tg.bw10    -side top  -fill both -expand true
   pack $tg.bw30    -side top  -fill both -expand true
   pack $tg.bw100   -side top  -fill both -expand true
   pack $tg.bw300   -side top  -fill both -expand true
   pack $tg.bw1k    -side top  -fill both -expand true
   pack $tg.bw3k    -side top  -fill both -expand true
   pack $tg.bw10k   -side top  -fill both -expand true
   pack $tg.bw30k   -side top  -fill both -expand true
   pack $tg.bw100k  -side top  -fill both -expand true
   pack $tg.bw300k  -side top  -fill both -expand true
   pack $tg.bw1m    -side top  -fill both -expand true
   pack $tg.bw3m    -side top  -fill both -expand true
   pack $tg.bwoff   -side top  -fill both -expand true
 }   



#
# doAnalyzeScale
#
proc doAnalyzeScale { tg } {
  global ANLZR_ENGINE

  message $tg.msg   -text "Vertical Scale (dB/Div)"  -width 3c \
                                -font anlzr_font
  entry $tg.entry   -textvariable  ANLZR_ENGINE(scale) -width 5 \
                                -font anlzr_font -bg white


  bind $tg.entry <Return> { set_anlzr_scale $ANLZR_ENGINE(scale) }

  pack $tg.msg      -side left -fill both -expand true
  pack $tg.entry    -side left -fill both -expand true
}

#
# doFullStartFreq
#   Set the starting RF Freq for the Full-Span Analyzer
#
proc doFullStartFreq { tg } {
  global ANLZR_ENGINE

  message $tg.msg -text "Start Freq (MHz)" -width 3c -font rcvr_font
  entry $tg.entry -textvariable ANLZR_ENGINE(full_start) -width 6 -bg white
  pack $tg.msg -side left -fill both -expand true
  pack $tg.entry -side left -fill both -expand true
}

#
# doFullStepSize
#   Set the RF Freq Step size (Increment) for the Full-Span Analyzer
#
proc doFullStepSize { tg } {
  global ANLZR_ENGINE

  message $tg.msg -text "Step Size (MHz)" -width 3c -font rcvr_font
  entry $tg.entry -textvariable ANLZR_ENGINE(full_incr) -width 6 -bg white
  pack $tg.msg -side left -fill both -expand true
  pack $tg.entry -side left -fill both -expand true
}

#
# doFullStepDelay
#   Set the RF Freq Step Delay for the Full-Span Analyzer
#
proc doFullStepDelay { tg } {
  global ANLZR_ENGINE

  label $tg.msg -text "Step Delay  (mS)" -font rcvr_font
  entry $tg.entry -textvariable ANLZR_ENGINE(full_delay) -width 5 -bg white
  pack $tg.msg -side left -fill both -expand true
  pack $tg.entry -side left -fill both -expand true
}


#
# doIFBW
#
proc doIFBW { tg } {
  global ChAnlzState
  global RcvrState

  message $tg.msg1 -text "IF BandWidth"  -width 3c -font anlzr_font

  radiobutton $tg.bw1  -text "1) 300 Hz" -font anlzr_font \
                       -command { set_anlzr_ifchbw $ChAnlzState(ifbw) } \
                       -variable ChAnlzState(ifbw) \
                       -value 300HZ
  radiobutton $tg.bw2  -text "2) 3 kHz" -font anlzr_font \
                       -command { set_anlzr_ifchbw $ChAnlzState(ifbw) } \
                       -variable ChAnlzState(ifbw) \
                       -value 3KHZ
  radiobutton $tg.bw3  -text "3) 60 kHz" -font anlzr_font \
                       -command { set_anlzr_ifchbw $ChAnlzState(ifbw) } \
                       -variable ChAnlzState(ifbw) \
                       -value 60KHZ
  radiobutton $tg.bw4  -text "4) 30 KHz" -font anlzr_font \
                       -command { set_anlzr_ifchbw $ChAnlzState(ifbw) } \
                       -variable ChAnlzState(ifbw) \
                       -value 30KHZ
  radiobutton $tg.bw5  -text "5) 300 kHz" -font anlzr_font \
                       -command { set_anlzr_ifchbw $ChAnlzState(ifbw) } \
                       -variable ChAnlzState(ifbw) \
                       -value 300KHZ
  radiobutton $tg.bw6  -text "6) 3 MHz" -font anlzr_font \
                       -command { set_anlzr_ifchbw $ChAnlzState(ifbw) } \
                       -variable ChAnlzState(ifbw) \
                       -value 3MHZ

  pack $tg.msg1      -side top -fill both -expand true
  pack $tg.bw1       -side top
  pack $tg.bw2       -side top
  pack $tg.bw3       -side top
  pack $tg.bw4       -side top
  pack $tg.bw5       -side top
  pack $tg.bw6       -side top
}



#
#  build channel analyzer screen
#
proc build_ch_anlzr_screen { name } {
  global chzrn
  global DemoColor

  #
  #  House keeping
  #
  set ANLZR_ENGINE(display_handle)  ".anlz_traces"

  # we have to save $chzrn because the -command {} will not rememver
  # local instance of $name
  set chzrn $name

  #
  # define screen frames
  #
  frame $chzrn.mbar          -relief raised -bd 2
  frame $chzrn.run           -relief raised -bd 2
  frame $chzrn.reading       -relief raised -bd 2
  frame $chzrn.norm          -relief raised -bd 2

  #
  # define single reading display
  #
  label $chzrn.reading.msg     -text "Sgl Pt Reading" \
                               -font anlzr_font -width 15
  label $chzrn.reading.sglpt   -textvariable  ANLZR_ENGINE(sglpt) \
                               -font anlzr_font -width 10 -bg white


  #
  #  Define menu bars  
  #
  message $name.mbar.label      -text "Analyzer" -font anlzr_font \
                                -bg $DemoColor(title) \
                                -width $DemoColor(labelwidth)

  menubutton $chzrn.mbar.ch     -text  "Settings"  -font anlzr_font \
                                -menu $chzrn.mbar.ch.menu \
                                -bg $DemoColor(menu) -relief raised
  menubutton $chzrn.mbar.disp   -text  "Display"     -font anlzr_font \
                                -menu $chzrn.mbar.disp.menu \
                                -bg $DemoColor(menu) -relief raised
  menubutton $chzrn.mbar.cal    -text  "Calibration"     -font anlzr_font \
                                -menu $chzrn.mbar.cal.menu \
                                -bg $DemoColor(menu) -relief raised

  #
  #  Build Channel Analyzer menu 
  #
  menu $chzrn.mbar.ch.menu
  $chzrn.mbar.ch.menu add command \
              -label "BW Span" -font anlzr_font \
              -command { WidgetWindow doBWSpan \
                         $chzrn.ch_spanbw $chzrn "BW Span" }
  $chzrn.mbar.ch.menu add command \
              -label "Sweep" -font anlzr_font \
              -command { WidgetWindow doChanlzrSweep \
                         $chzrn.ch_sweep $chzrn "Anlzr Sweep" }
  $chzrn.mbar.ch.menu add command \
              -label "IF Bandwidth" -font anlzr_font \
              -command { WidgetWindow doIFBW \
                         $chzrn.ch_bw $chzrn "Anlzr BW" }
  $chzrn.mbar.ch.menu add command \
              -label "IF Gain" -font anlzr_font \
              -command { WidgetWindow doAnlzrIfgain \
                         $chzrn.ch_ifgn $chzrn "Ch. Analz. IF Gain" }
  $chzrn.mbar.ch.menu add command \
              -label "DDS Freq" -font anlzr_font \
              -command { WidgetWindow doChanlzrDDSFreq \
                         $chzrn.ch_dds $chzrn "Ch. Analz. DDS Freq" }

  #
  #  Build Channel Analyzer menu 
  #
  menu $chzrn.mbar.disp.menu
  $chzrn.mbar.disp.menu add command \
              -label "Video Bandwidth" -font anlzr_font \
              -command { WidgetWindow doChanlzrVideoBW \
                         $chzrn.disp_vbw $chzrn "Video Bandwidth" }
  $chzrn.mbar.disp.menu add cascade \
              -label "Input Source" -font anlzr_font \
              -menu $chzrn.mbar.disp.menu.inp

  menu $chzrn.mbar.disp.menu.inp

  $chzrn.mbar.disp.menu.inp add radiobutton -label "CH" -font anlzr_font \
                    -command { set_anlzr_adc_input $ChAnlzState(adc) } \
                       -variable ChAnlzState(adc) \
                       -value CHANLZ   
  $chzrn.mbar.disp.menu.inp add radiobutton -label "Main" -font anlzr_font \
                       -command { set_anlzr_adc_input $ChAnlzState(adc) } \
                       -variable ChAnlzState(adc) \
                       -value MAINANLZ
  $chzrn.mbar.disp.menu.inp add radiobutton -label "Temp" -font anlzr_font \
                       -command { set_anlzr_adc_input $ChAnlzState(adc) } \
                       -variable ChAnlzState(adc) \
                       -value TEMP

  $chzrn.mbar.disp.menu add command \
              -label "Trace Length" -font anlzr_font \
              -command { WidgetWindow doChanlzrTraceLength \
                         $chzrn.disp_len $chzrn "Trace Length" }
  $chzrn.mbar.disp.menu add command \
              -label "Widget Width" -font anlzr_font \
              -command { WidgetWindow doChanlzrTraceWidgetWidth \
                         $chzrn.disp_width $chzrn "Widget width" }
  $chzrn.mbar.disp.menu add checkbutton \
              -label "Trace On/Off" -font anlzr_font \
              -command { anlz_onoff } \
              -variable ANLZR_ENGINE(trace_state)
  $chzrn.mbar.disp.menu add checkbutton \
              -label "Persistance On/Off" -font anlzr_font \
              -command { anlz_persistance } \
              -variable ANLZR_ENGINE(persist_state)


  #
  #  Build Channel Analyzer cal menu 
  #
  menu $chzrn.mbar.cal.menu
  $chzrn.mbar.cal.menu add command \
              -label "BW Switching Cal" -font anlzr_font \
              -command { WidgetWindow doChanlzrBWSwCalTable \
                         $chzrn.cal_table1 $chzrn "BW Switching Cal" }
  $chzrn.mbar.cal.menu add command \
              -label "Pwr Term Cross-Feed Cal" -font anlzr_font \
              -command { WidgetWindow doAnlzrCfCalTable \
                         $chzrn.cal_table2 $chzrn "Cross-Feed Cal" }
  $chzrn.mbar.cal.menu add command \
              -label "Pwr Term Ant Input Cal" -font anlzr_font \
              -command { WidgetWindow doAnlzrAntInputCalTable \
                         $chzrn.cal_table3 $chzrn "Ant Input Cal" }
  $chzrn.mbar.cal.menu add command \
              -label "Pwr Term T/R Input Cal" -font anlzr_font \
              -command { WidgetWindow doAnlzrTRInputCalTable \
                         $chzrn.cal_table4 $chzrn "T-R Input Cal" }


  button $chzrn.mbar.norm -text "Normalize" -command { set_anlzr_normalize 1 } \
                                        -font anlzr_font -bg $DemoColor(menu)
  label $chzrn.mbar.normmsg     -textvariable  ANLZR_ENGINE(status) \
                                -font anlzr_font -width 10 -bg white



  button $chzrn.run.ch    -text "Channel" -command { set_anlzr_run CHANNEL } \
                                        -font anlzr_font -bg $DemoColor(menu)  

  button $chzrn.run.full  -text "Full"   -command { set_anlzr_run FULL } \
                                        -font anlzr_font -bg $DemoColor(menu)      
  button $chzrn.run.stop  -text "Stop"  -command { set_anlzr_run STOP } \
                                        -font anlzr_font -bg $DemoColor(menu)

  label $chzrn.run.msg    -text "Run" -font anlzr_font



  #
  #  pack sub frames - run
  #
  pack $chzrn.run.msg          -side left  -fill both
  pack $chzrn.run.full         -side left  -fill both -padx 2
  pack $chzrn.run.ch           -side left  -fill both -padx 2
  pack $chzrn.run.stop         -side left  -fill both -padx 2

  #
  #  pack sub frames - mbar
  #
  pack $chzrn.mbar.label       -side left  -fill both
  pack $chzrn.mbar.ch          -side left  -fill both
  pack $chzrn.mbar.disp        -side left  -fill both
  pack $chzrn.mbar.cal         -side left  -fill both
  pack $chzrn.mbar.norm        -side left  -fill both -padx 2
  pack $chzrn.mbar.normmsg     -side left  -fill both

  #
  #  pack sub frames
  #
  pack $chzrn.reading.msg      -side left  -fill both
  pack $chzrn.reading.sglpt    -side left  -fill both


  #
  #  pack main frame
  #
  pack $chzrn.mbar             -side left  -fill both -expand true
  pack $chzrn.run              -side left  -fill both
 
  pack $chzrn.reading          -side right -fill both


  return $chzrn
}



#
#  build channle analyzer screen
#
proc build_ch_anlzr_trace { name } {
  global ANLZR_ENGINE

  frame $name.knob

  message $name.knob.msg   -text "TOS (dBm)" -font anlzr_font
  entry $name.knob.entry   -textvariable  ANLZR_ENGINE(tos) -width 5 \
                           -font anlzr_font -bg white
  message $name.knob.msg2  -text "Scale (dB/Div)" -font anlzr_font
  entry $name.knob.entry2  -textvariable  ANLZR_ENGINE(stat_db_step) -width 5 \
                           -font anlzr_font -bg white
  message $name.knob.msg3  -text "Atten (dB)" -font anlzr_font
  entry $name.knob.entry3  -textvariable  ANLZR_ENGINE(stat_atten) -width 3 \
                           -font anlzr_font -bg white
  entry $name.knob.entry4  -textvariable  ANLZR_ENGINE(stat_cal) -width 6 \
                           -font anlzr_font -bg white
  entry $name.knob.entry5  -textvariable  ANLZR_ENGINE(stat_ifgain) -width 5 \
                           -font anlzr_font -bg white

  bind $name.knob.entry   <Return> { set_anlzr_tos  $ANLZR_ENGINE(tos) } 
  bind $name.knob.entry2  <Return> { set_anlzr_scale $ANLZR_ENGINE(stat_db_step) }
  bind $name.knob.entry3  <Return> { Receiver atten $ANLZR_ENGINE(stat_atten) }


  set ANLZR_ENGINE(trace1)  $name.tr

  TraceWidget $ANLZR_ENGINE(trace1) -bg #893e4d \
                                    -width $ANLZR_ENGINE(WidgetWidth) \
                                    -height 290
  $ANLZR_ENGINE(trace1) trace create trace1 -sys $ANLZR_ENGINE(SYSTEM_ID) \
                                   -subsys $ANLZR_ENGINE(MTR_TRACE) \
                                   -cfgsubsys $ANLZR_ENGINE(MTR_TRACE_CONFIG) \
                                   -lun 1 -flags 0x3D -active 1
 
  $ANLZR_ENGINE(trace1) config -xgratdivs 10 -ygratdivs 8 -persist 0

  $ANLZR_ENGINE(trace1) config -fg        gray
  $ANLZR_ENGINE(trace1) config -graticule gray
  $ANLZR_ENGINE(trace1) config -grid      gray

  pack $name.knob.msg    -side left
  pack $name.knob.entry  -side left
  pack $name.knob.msg2   -side left
  pack $name.knob.entry2 -side left
  pack $name.knob.msg3   -side left
  pack $name.knob.entry3 -side left
  pack $name.knob.entry4 -side left -padx 2
  pack $name.knob.entry5 -side left -padx 2

  pack $name.knob  -side top  -fill both -expand true -padx 5
  pack $name.tr    -side left -fill both -expand true -padx 5
}

