#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C)1998,99      DESIGN ENGINEERING      #
# --------------------------------------------------------------------------#
# FILE        : to_anlzr.tcl                                                #
# PRODUCT     : FITS                                                        #
# AUTHOR      : Steve Johnson                                               #
# DESCRIPTION : Interface to Channel Analyzer via Message Bus               #
#                                                                           #
#############################################################################

#
#  Analyzer System Number
#
set ANLZR_ENGINE(SYSTEM_ID)            13

#
#  Analyzer Sub-Systems
#
set ANLZR_ENGINE(MTR_SCALE)           1
set ANLZR_ENGINE(MTR_TOPOFSCREEN)     2
set ANLZR_ENGINE(MTR_SPAN)            3
set ANLZR_ENGINE(MTR_IF_GAIN)         4
set ANLZR_ENGINE(MTR_CH_BW)           5
set ANLZR_ENGINE(MTR_ADC_INPUT)       6
set ANLZR_ENGINE(MTR_VIDEO_BW)        7
set ANLZR_ENGINE(MTR_MATH)            8
set ANLZR_ENGINE(MTR_PHRST)           9
set ANLZR_ENGINE(MTR_REFERENCE)       10
set ANLZR_ENGINE(MTR_TRIG_ARM)        11
set ANLZR_ENGINE(MTR_TRIG_SOURCE)     12
set ANLZR_ENGINE(MTR_CALIBRATIONS)    13
set ANLZR_ENGINE(MTR_ENABLE)          14
set ANLZR_ENGINE(MTR_INFO)            15
set ANLZR_ENGINE(MTR_TRACE)           16
set ANLZR_ENGINE(MTR_STATUS)          17
set ANLZR_ENGINE(MTR_RUN)             18
set ANLZR_ENGINE(MTR_SGL_PT)          19
set ANLZR_ENGINE(MTR_SWEEP)           20
set ANLZR_ENGINE(MTR_IF_DDS_FREQ)     21
set ANLZR_ENGINE(MTR_CENTER_FREQ)     22
set ANLZR_ENGINE(MTR_TG_STATE)        23
set ANLZR_ENGINE(MTR_TOS_INFO)        24
set ANLZR_ENGINE(MTR_NORMALIZE)       25
set ANLZR_ENGINE(MTR_CAL_DATA)        26
set ANLZR_ENGINE(MTR_SAVE_RECALL)     27
set ANLZR_ENGINE(MTR_REF_TRACE)       28
set ANLZR_ENGINE(MTR_MATH_TRACE)      29
set ANLZR_ENGINE(MTR_TRACE_CONFIG)    200

#
#  enable sgl point mode, traces and status messages
#
set ANLZR_ENGINE(MB_EVENTS)            0x66

# trace length may be between 2 to 2000 pts
# engine may shrink size to allow sweep to be set
set ANLZR_ENGINE(trace_length)        500
set ANLZR_ENGINE(WidgetWidth)         300

set ANLZR_ENGINE(sweep.pt)            400
set ANLZR_ENGINE(sweep.tr)         200000
set ANLZR_ENGINE(span)                0.1
set ANLZR_ENGINE(start)              50.0
set ANLZR_ENGINE(stop)              200.0
set ANLZR_ENGINE(sglpt)                -1
set ANLZR_ENGINE(dds_freq)             16
set ANLZR_ENGINE(scale)                10.0
set ANLZR_ENGINE(tos)                 -10.0
set ANLZR_ENGINE(status)                0

set ANLZR_ENGINE(full_start)          5.0
set ANLZR_ENGINE(full_incr)           5.0
set ANLZR_ENGINE(full_delay)          1.0
set ANLZR_ENGINE(full_mode)           continuous
set ANLZR_ENGINE(TrkGen)              disabled

set ANLZR_ENGINE(display_handle)       -1
set ANLZR_ENGINE(trace1)               -1
set ANLZR_ENGINE(color_ch1)         white
set ANLZR_ENGINE(trace_state)           1

set ANLZR_ENGINE(handler)              -1
set ANLZR_ENGINE(handle_sglpt)          0
set ANLZR_ENGINE(handle_cf)             0
set ANLZR_ENGINE(handle_tos)            0
set ANLZR_ENGINE(handle_status)         0

set ANLZR_ENGINE(stat_trig)            -1
set ANLZR_ENGINE(stat_llfail)          -1
set ANLZR_ENGINE(stat_ulfail)          -1
set ANLZR_ENGINE(stat_timfail)         -1
set ANLZR_ENGINE(stat_memfail)         -1
set ANLZR_ENGINE(stat_stop)            -1
set ANLZR_ENGINE(stat_length)          -1

set ANLZR_ENGINE(stat_db_step)         -1
set ANLZR_ENGINE(stat_atten)           -1
set ANLZR_ENGINE(stat_ifgain)          -1
set ANLZR_ENGINE(stat_cal)           Initial

set Send_Flags                        0x1

font create anlzr_font -family helvetica -size 8

#############################################################################
#                                                                           #
#                       Message Bus Interfaces                              #
#                                                                           #
#############################################################################


#
#  Message bus callback - fill in supplied structure
#
#
proc handle_anlzr_status {sys sub lun flags uid status } {
  global ANLZR_ENGINE
 
#puts "RLM reading: $status"

  if { $status == 1 } {
    set ANLZR_ENGINE(status)  "Set RF Gen"
  } elseif { $status == 2 } {
    set ANLZR_ENGINE(status)  "Set Power Term"
  } elseif { $status == 3 } {
    set ANLZR_ENGINE(status)  "Set Ref Level"
  } elseif { $status == 4 } {
    set ANLZR_ENGINE(status)  "Read RF Level"
  } elseif { $status == 5 } {
    set ANLZR_ENGINE(status)  "Adjust TOS"
  } elseif { $status == 6 } {
    set ANLZR_ENGINE(status)  "Set Compensation"
  } elseif { $status == 7 } {
    set ANLZR_ENGINE(status)  "Doing Next RBW"
  } elseif { $status == 8 } {
    set ANLZR_ENGINE(status)  "Cal Complete"
  } elseif { $status == 9 } {
    set ANLZR_ENGINE(status)  "User Aborted"
  } else {
    set ANLZR_ENGINE(status)  "Cal Error: $status"
  }
}

#
#  Message bus callback - fill in supplied structure
#
proc handle_anlzr_sglpt {sys sub lun flags uid reading } {
  global ANLZR_ENGINE

#puts "anlzr: $reading"

  set ANLZR_ENGINE(sglpt)        [ format "%3.2f" $reading ]
  set ANLZR_ENGINE(sglpt_format) [ format "%3.2f" $reading ]
}

#
#  Message bus callback - fill in supplied structure
#
proc handle_anlzr_tos {sys sub lun flags uid tos db_step atten ifgain state } {
  global ANLZR_ENGINE

#puts "handle_anlzr_tos: $tos $db_step $atten $ifgain $state"
  set ANLZR_ENGINE(tos)           [ format "%3.1f" $tos ]
  set ANLZR_ENGINE(stat_db_step)  [ format "%3.1f" $db_step ]
  set ANLZR_ENGINE(stat_ifgain)   [ format "%3.1f" $ifgain ]
  set ANLZR_ENGINE(stat_atten)    [ expr int($atten) ]


  if { $state & 1 } {
    set ANLZR_ENGINE(stat_cal)     CALed
  } else {
    set ANLZR_ENGINE(stat_cal)     UnCAL
  }
}


#
#  Message bus interfaces for messages being generated FROM anlzr engine
#
proc create_anlzr_callback { } {
  global ANLZR_ENGINE

  set ANLZR_ENGINE(handle_sglpt) [tclMsgbus rcv handle \
                                  -sys $ANLZR_ENGINE(SYSTEM_ID) \
                                  -sub $ANLZR_ENGINE(MTR_SGL_PT) \
                                  -lun 1 -cmd "handle_anlzr_sglpt" ]

  set ANLZR_ENGINE(handle_tos)    [tclMsgbus rcv handle \
                                  -sys $ANLZR_ENGINE(SYSTEM_ID) \
                                  -sub $ANLZR_ENGINE(MTR_TOS_INFO) \
                                  -lun 1 -cmd "handle_anlzr_tos" ]

  set ANLZR_ENGINE(handle_status) [tclMsgbus rcv handle \
                                  -sys $ANLZR_ENGINE(SYSTEM_ID) \
                                  -sub $ANLZR_ENGINE(MTR_STATUS) \
                                  -lun 1 -cmd "handle_anlzr_status" ]
}


#
#  Message bus interfaces - removed
#
proc destroy_anlzr_callback { } {
  global ANLZR_ENGINE

  if {$ANLZR_ENGINE(handle_sglpt) != 0 } {
    tclMsgbus $ANLZR_ENGINE(handle_sglpt) destroy
  }

  if {$ANLZR_ENGINE(handle_tos) != 0 } {
    tclMsgbus $ANLZR_ENGINE(handle_tos) destroy
  }

  if {$ANLZR_ENGINE(handle_status) != 0 } {
    tclMsgbus $ANLZR_ENGINE(handle_status) destroy
  }

  set ANLZR_ENGINE(handle_sglpt)  0
  set ANLZR_ENGINE(handle_tos)    0
  set ANLZR_ENGINE(handle_status) 0
}


#############################################################################
#                                                                           #
#                       Engine Interfaces                                   #
#                                                                           #
#############################################################################

#
#  Anlzr Tracking Generator State ( ON, OFF)
#
proc set_anlzr_tg_state { state } {
  global ANLZR_ENGINE

  switch $state {
    1       -
    ON      { set x 1; set_rfgen_ptt ON }
    0       -
    OFF     { set x 0; set_rfgen_ptt OFF }
    default { puts "set_anlzr_tg_state err: $state"; return }
  }

  message_bus_int $ANLZR_ENGINE(SYSTEM_ID) $ANLZR_ENGINE(MTR_TG_STATE) 1 $x
}


#
#  Set math operation - clear, average size..
#
proc set_anlzr_math { type {value 0.0} } {
  global Send_Flags
  global ANLZR_ENGINE

  switch $type {
    CLEAR-UP { set t 0  }   ; # for average
    CLEAR-DN { set t 1  }   ; # for peak
    AVERAGE  { set t 2  }
    default { puts "set_anlzr_math err: $type"; return }
  }

  set value [ expr double($value) ]

  set msg " { $ANLZR_ENGINE(SYSTEM_ID) $ANLZR_ENGINE(MTR_MATH) \
              1 $Send_Flags } \
            { inline sdword $t } \
            { inline double $value } "

  tclMsgbus send $msg
}


#
#  Set analyzer RF frequency
#
proc set_anlzr_freq { type freq } {
  global Send_Flags
  global ANLZR_ENGINE

  switch $type {
    RFGEN-INV { set t 2  }
    RFGEN     { set t 1  }
    RCVR      { set t 0  }
    default { puts "set_anlzr_freq err: $type"; return }
  }

  set msg " { $ANLZR_ENGINE(SYSTEM_ID) $ANLZR_ENGINE(MTR_CENTER_FREQ) \
              1 $Send_Flags } \
            { inline sdword $t } \
            { inline double $freq } "

  tclMsgbus send $msg
}

#
#  Anlzr IF channel gain ( -14 to +82 dB in 0.1 dB steps)
#
proc set_anlzr_ifchgain { db } {
  global ANLZR_ENGINE

  message_bus_double $ANLZR_ENGINE(SYSTEM_ID) \
                     $ANLZR_ENGINE(MTR_IF_GAIN) \
                     1 $db
}


#
#  Anlzr IF DDS sweep time for trace
#                             type : 1 uS/pt
#                                    2 uS/trace
#
proc set_anlzr_sweep { uS type } {
  global Send_Flags
  global ANLZR_ENGINE

  if { $type == 1 } {
    set ANLZR_ENGINE(sweep.tr) [ expr $ANLZR_ENGINE(sweep.pt) * $ANLZR_ENGINE(trace_length)]
  } else {
    set ANLZR_ENGINE(sweep.pt) [ expr $ANLZR_ENGINE(sweep.tr) / $ANLZR_ENGINE(trace_length)]
  }

  set msg " { $ANLZR_ENGINE(SYSTEM_ID) $ANLZR_ENGINE(MTR_SWEEP) \
              1 $Send_Flags } \
            { inline udword $uS} \
            { inline udword $type } "
#puts $msg
  tclMsgbus send $msg
}


#
#  Anlzr IF Channel BandWidth ( 300 Hz, 3 kHz or 60 kHz )
#
proc set_anlzr_ifchbw { x } {
  global ANLZR_ENGINE
  global RcvrState

  switch $x {
    300HZ   { set bw 300     }
    3KHZ    { set bw 3000    }
    60KHZ   { set bw 60000   }
    30KHZ   { set bw 30000   ; set RcvrState(ifbw) $bw }
    300KHZ  { set bw 300000  ; set RcvrState(ifbw) $bw }
    3MHZ    { set bw 3000000 ; set RcvrState(ifbw) $bw }
    default { puts "set_anlzr_ifchbw err: $x"; return }
  }

  message_bus_int $ANLZR_ENGINE(SYSTEM_ID) $ANLZR_ENGINE(MTR_CH_BW) 1 $bw
}



#
#  Anlzr Video BandWidth ( off, 10, 30, 100, 300, 1k, 3k, 10k, 30k, 
#                          100k, 300k, 1M, 3M )
#
proc set_anlzr_video_bw { x } {
  global ANLZR_ENGINE

  switch $x {
    BW_OFF   { set bw 0   }
    BW10     { set bw 10 }
    BW30     { set bw 30 }
    BW100    { set bw 100 }
    BW300    { set bw 300 }
    BW1K     { set bw 1000 }
    BW3K     { set bw 3000 }
    BW10K    { set bw 10000 }
    BW30K    { set bw 30000 }
    BW100K   { set bw 100000 }
    BW300K   { set bw 300000 }
    BW1M     { set bw 1000000 }
    BW3M     { set bw 3000000 }
    default  { return       }
  }

  message_bus_int $ANLZR_ENGINE(SYSTEM_ID) $ANLZR_ENGINE(MTR_VIDEO_BW) 1 $bw
}

#
#  Anlzr ADC Input Selection ( 1 is CH anlzr, 2 is Main anlzr, 3 is temp)
#
proc set_anlzr_adc_input { arg } {
  global ANLZR_ENGINE

  switch $arg {
    CHANLZ   { set adc 1 }
    MAINANLZ { set adc 2 }
    TEMP     { set adc 3 }
    default  { return    }
  }

 message_bus_int $ANLZR_ENGINE(SYSTEM_ID) $ANLZR_ENGINE(MTR_ADC_INPUT) 1 $adc
}


#
#  Anlzr Span Bandwidth for sweep (MHz)
#
#  Two Methods: Total Bandwidth & Start/Stop
#    
#  Total Bandwidth F1 = BW    F2 = -1.0
#  Start/Stop      F1 = Start F2 = Stop
#
#
proc set_anlzr_span { lun f1 f2 } {
  global ANLZR_ENGINE
  global Send_Flags

  set msg " { $ANLZR_ENGINE(SYSTEM_ID) $ANLZR_ENGINE(MTR_SPAN) $lun \
             $Send_Flags } { inline double $f1 } { inline double $f2 } "

  tclMsgbus send $msg
}

#
#  Anlzr Run Mode - 
#         state - START analyzer is running
#                 STOP          not running
#
proc set_anlzr_run { state } {
  global ANLZR_ENGINE

  switch $state {
    CHANNEL { set st 1 }
    FULL    { set st 2 }
    STOP    { set st 3 }
    default { return   }
  }

  message_bus_int $ANLZR_ENGINE(SYSTEM_ID) $ANLZR_ENGINE(MTR_RUN) 1 $st
}


#
#  enable MB messages from analyzer engine
#
proc set_anlzr_enable { modes } {
  global Send_Flags
  global ANLZR_ENGINE
  global set_anlzr_enable_delay

  set state 0

  foreach el $modes {
    switch $el {
      DISABLE {  }
      LIVE    { set state [ expr $state |  0x2 ] }
      PEAK    { set state [ expr $state | 0x08 ] }
      AVERAGE { set state [ expr $state | 0x10 ] }
      SGLPT   { set state [ expr $state | 0x20 ] }
      TOS     { set state [ expr $state | 0x40 ] }
      REF     { set state [ expr $state | 0x80 ] }
      default { puts "set_anlzr_enable: invalid mode $el" } 
    } 
  }

  set msg " { $ANLZR_ENGINE(SYSTEM_ID) $ANLZR_ENGINE(MTR_ENABLE) \
              1 $Send_Flags } \
            { inline udword $ANLZR_ENGINE(trace_length) } \
            { inline udword $state } "

  tclMsgbus send $msg
}

#
#  Anlzr IF DDS Freq ( 0 to 20 MHz )
#
proc set_anlzr_if_dds_freq { mhz } {
  global ANLZR_ENGINE

  message_bus_double $ANLZR_ENGINE(SYSTEM_ID) \
                     $ANLZR_ENGINE(MTR_IF_DDS_FREQ) \
                     1 $mhz
}


#
#  Change current analyzer trace length
#
proc reset_anlzr_trace_length { } {
  set_anlzr_run STOP
  set_anlzr_enable DISABLE
  set_anlzr_enable DEMO
}


#
#  Anlzr Vertical Scale ( 0.1 - 20.0 dB per division )
#
proc set_anlzr_scale { db_div } {
  global ANLZR_ENGINE

  message_bus_double $ANLZR_ENGINE(SYSTEM_ID) \
                     $ANLZR_ENGINE(MTR_SCALE) \
                     1 $db_div
}

#
#  Anlzr Tops of Scale ( 80 dBm to -70 dBm )
#           
#       various things main limit the actual top of 
#       scale actually used - refer to TOS callback
#       for what actually happened after sending this cmd
#
proc set_anlzr_tos { dbm } {
  global ANLZR_ENGINE

  message_bus_double $ANLZR_ENGINE(SYSTEM_ID) \
                     $ANLZR_ENGINE(MTR_TOPOFSCREEN) \
                     1 $dbm
}


#
#  Anlzr Display Normalize Interface 
#         bits: 
#         xxxx xxxx xxxx xxxx                                            
#              |||| |||| ++++-- start (num div 0 to 127) & abort 255     
#              |||| ++++------- cross-feed ( 0 T/R, 1 Gen-T/R, 2 T/R-Ant 
#              ||||                          3 Gen-Ant )                 
#              ++++------------ options 1 restore RF gen when done       
#                                       2                                
#
proc set_anlzr_normalize { bits } {
 global ANLZR_ENGINE

 message_bus_int $ANLZR_ENGINE(SYSTEM_ID) $ANLZR_ENGINE(MTR_NORMALIZE) 1 $bits
}


#
#  Set Anlzr Calibration Data
#
#           type  - cal data type to send to engine
#           index - 1 to 6
#                   1 to 3 is channel bw ( 300, 3k, 60k )
#                   4 to 6 is main bw ( 30k, 300k, 3M )
#           khz   - is amount of freq to change the YIG or DDS
#                      to center the trace because of filter
#           db    - move screen up/down this number of dB (+/- 0.1)
#           watts - power term reading in watts
#
proc do_anlzr_cals { type index khz db watts } {
  global ANLZR_ENGINE
  global Send_Flags

  switch $type {
    header   { set itype 1  }
    bwswitch { set itype 2  }
    cf       { set itype 3  }
    tr       { set itype 4  }
    ant      { set itype 5  }
    cfsize   { set itype 12 }
    trsize   { set itype 13 }
    antsize  { set itype 14 }
    delnorm  { set itype 100 }
    default  { return   }
  }

  set msg " { $ANLZR_ENGINE(SYSTEM_ID) $ANLZR_ENGINE(MTR_CAL_DATA) 1 \
              $Send_Flags } { inline udword $itype } \
                            { inline udword $index } { inline double $khz } \
                            { inline double $db }    { inline double $watts }"
#puts $msg
  tclMsgbus send $msg
}

#
#  Delete Normalize Calibration File
#
proc del_normalize_calibrations { } {

  do_anlzr_cals delnorm 0 0.0 0.0 0.0

  set code [ catch {
    cd /usr/analyzer
    exec rm AnalyzerTweeks
  } err ]

  if { $code } {
    puts "open_event_log failure: $err"
    return
  }
}


#
#  Set Anlzr Save/Recall Data
#
#           type      - SAVE, RECALL
#           operation - SETREF, CLEARREF
#                  
proc set_anlzr_save_recall { type operation } {
  global Send_Flags
  global ANLZR_ENGINE

  set code [file isdirectory "/usr/analyzer"]

  if { $code == 0 } {
    set code [ catch {
       cd /usr
       exec mkdir analyzer
    } err ]

    if { $code } {
      puts "set_anlzr_save_recall (mkdir) $err"
      return
    }
  }

  switch $type {
    SAVE     { set itype 1  }
    RECALL   -
    DONTCARE { set itype 0  }
    default  { return }
  }

  switch $operation {
    SETREF   { set ioperation 1  }
    CLEARREF { set ioperation 2  }
    USEREF   { set ioperation 3  }
    default  { return }
  }

  set msg " { $ANLZR_ENGINE(SYSTEM_ID) $ANLZR_ENGINE(MTR_SAVE_RECALL) \
              1 $Send_Flags } \
            { sdword $itype} \
            { sdword $ioperation } "

  tclMsgbus send $msg
}

#############################################################################
#                                                                           #
#                       Calibration Interfaces                              #
#                                                                           #
#############################################################################

set AnlzrCals(cfmax)   0
set AnlzrCals(max)     0
set AnlzrCals(date)   -1
set AnlzrCals(time)   -1

set AnlzrCals(1.khz)  -1
set AnlzrCals(1.norm) -1
set AnlzrCals(2.khz)  -1
set AnlzrCals(2.norm) -1
set AnlzrCals(3.khz)  -1
set AnlzrCals(3.norm) -1
set AnlzrCals(4.khz)  -1
set AnlzrCals(4.norm) -1
set AnlzrCals(5.khz)  -1
set AnlzrCals(5.norm) -1
set AnlzrCals(6.khz)  -1
set AnlzrCals(6.norm) -1

#
#   Parse Header Info
#
#
proc parse_anlz_header { line } {
  global AnlzrCals
 
  set AnlzrCals(date)   [ lindex $line 2 ]
  set AnlzrCals(time)   [ lindex $line 4 ]

  do_anlzr_cals header 0 0.0 0.0 0.0
}

#
#  Parse Switching Cal Data entry
#
#
proc parse_anlz_cal { line } {
  global AnlzrCals
 
  set index [ lindex $line 2 ]
  set khz   [ lindex $line 4 ]
  set norm  [ lindex $line 6 ]

  set AnlzrCals($index.khz)   $khz
  set AnlzrCals($index.norm)  $norm

  do_anlzr_cals bwswitch $index $khz $norm 0.0
}

#
#  Parse Cross-Feed Cal Data entry
#
#
proc parse_anlz_cf_cal { line } {
  global AnlzrCals
 
  set index [ lindex $line 2 ]
  set mhz   [ lindex $line 4 ]
  set dbm   [ lindex $line 6 ]
  set watts [ lindex $line 8 ]

  set AnlzrCals($index.cf.mhz)   $mhz
  set AnlzrCals($index.cf.dbm)   $dbm
  set AnlzrCals($index.cf.watts) $watts

  do_anlzr_cals cf $index $mhz $dbm $watts
}

#
#  Parse Input Signal Path Loss entry
#
#
proc parse_anlz_input_cal { type line } {
  global AnlzrCals
 
  set index [ lindex $line 2 ]
  set mhz   [ lindex $line 4 ]
  set db    [ lindex $line 6 ]

  set AnlzrCals($index.$type.mhz)   $mhz
  set AnlzrCals($index.$type.db)    $db

  do_anlzr_cals $type $index $mhz $db 0.0
}


#
# update tcl anlzr cal data
#
proc get_anlzr_cal_data { filename } {
  global AnlzrCals

  set file [ OpenCalFile $filename r ]

  while { [ gets $file line ] >= 0 } {
# puts $line
    if { [ lindex $line 0 ] == "ANALYZER_CHARACTERISTICS" } {
      parse_anlz_header $line
    }
    if { [ lindex $line 0 ] == "ANLZR_BW_SW_CAL_MAX" } {
      set AnlzrCals(max)    [ lindex $line 1 ]
    }
    if { [ lindex $line 0 ] == "ANLZR_BW_SW_CAL" } {
      parse_anlz_cal $line
    }
    if { [ lindex $line 0 ] == "ANLZR_FREQ_CF_CAL_MAX" } {
      set AnlzrCals(cfmax)  [ lindex $line 1 ]
      do_anlzr_cals cfsize $AnlzrCals(cfmax) 0.0 0.0 0.0
    }
    if { [ lindex $line 0 ] == "ANLZR_FREQ_CF_CAL" } {
      parse_anlz_cf_cal $line
    }
    if { [ lindex $line 0 ] == "ANLZR_FREQ_TR2REC_CAL_MAX" } {
      set AnlzrCals(trmax)  [ lindex $line 1 ]
      do_anlzr_cals trsize $AnlzrCals(trmax) 0.0 0.0 0.0
    }
    if { [ lindex $line 0 ] == "ANLZR_FREQ_TR2REC_CAL" } {
      parse_anlz_input_cal tr $line
    }
    if { [ lindex $line 0 ] == "ANLZR_FREQ_ANT2REC_CAL_MAX" } {
      set AnlzrCals(antmax)  [ lindex $line 1 ]
      do_anlzr_cals antsize $AnlzrCals(antmax) 0.0 0.0 0.0
    }
    if { [ lindex $line 0 ] == "ANLZR_FREQ_ANT2REC_CAL" } {
      parse_anlz_input_cal ant $line
    }
  }

  close $file
}


#
# update tcl anlzr cal data - write current data
#
proc put_anlzr_cal_data { filename } {
  global AnlzrCals

  set file [ OpenCalFile $filename w ]

  puts $file "#"
  puts $file "#  Spectrum Analyzer Calibration Table"
  puts $file "#====================================================================="
  set msg [ format "ANALYZER_CHARACTERISTICS   DATE %10s TIME %10s" \
                    $AnlzrCals(date) $AnlzrCals(time) ]
  puts $file $msg
  puts $file " "

  # normalize cal data - normalization occures using the middle filter
  set temp $AnlzrCals(2.norm)
  set AnlzrCals(1.norm) [ expr $AnlzrCals(1.norm) - $temp ]
  set AnlzrCals(2.norm) 0.0
  set AnlzrCals(3.norm) [ expr $AnlzrCals(3.norm) - $temp ]

  set AnlzrCals(4.norm) [ expr $AnlzrCals(4.norm) - $temp ]
  set AnlzrCals(5.norm) [ expr $AnlzrCals(5.norm) - $temp ]
  set AnlzrCals(6.norm) [ expr $AnlzrCals(6.norm) - $temp ]

  set msg [ format "ANLZR_BW_SW_CAL_MAX    %3d" $AnlzrCals(max) ]
  puts $file $msg
  for {set i 1 } { $i <= $AnlzrCals(max) } { incr i } { 
    set msg [ format "ANLZR_BW_SW_CAL  INDEX %2d  FREQADJ %7.3f  NORMALIZE %7.2f" \
               $i $AnlzrCals($i.khz) $AnlzrCals($i.norm) ]
    puts $file $msg
  }

  puts $file " "

  set msg [ format "ANLZR_FREQ_CF_CAL_MAX    %3d" $AnlzrCals(cfmax) ]
  puts $file $msg
  set i [ expr $AnlzrCals(cfmax) -1 ]
  for { } { $i >= 0 } { incr i -1 } { 
    set msg [ format "ANLZR_FREQ_CF_CAL INDEX %2d  FREQ %8.3f  RCVRPWR %6.2f  PT %7.4f" \
     $i $AnlzrCals($i.cf.mhz) $AnlzrCals($i.cf.dbm) $AnlzrCals($i.cf.watts)]
    puts $file $msg
  }

  puts $file " "

  set msg [ format "ANLZR_FREQ_TR2REC_CAL_MAX    %3d" $AnlzrCals(trmax) ]
  puts $file $msg
  set i [ expr $AnlzrCals(trmax) -1 ]
  for { } { $i >= 0 } { incr i -1 } { 
    set msg [ format "ANLZR_FREQ_TR2REC_CAL  INDEX %2d  FREQ %8.3f  LOSS %7.2f" \
     $i $AnlzrCals($i.tr.mhz) $AnlzrCals($i.tr.db)]
    puts $file $msg
  }

  puts $file " "

  set msg [ format "ANLZR_FREQ_ANT2REC_CAL_MAX    %3d" $AnlzrCals(antmax) ]
  puts $file $msg
  set i [ expr $AnlzrCals(antmax) -1 ]
  for { } { $i >= 0 } { incr i -1 } { 
    set msg [ format "ANLZR_FREQ_ANT2REC_CAL INDEX %3d  FREQ %8.3f  LOSS %7.2f" \
     $i $AnlzrCals($i.ant.mhz) $AnlzrCals($i.ant.db)]
    puts $file $msg
  }


  flush $file
  close $file
}
