#
# MAIN.TCL for Engineering Application
#


#
# load the TCL command extensions
#
package require Tracewidget
package require Tclmsgbus
package require BBAudio


# load the VxWorks Interface Modules
#
source to_dsp.tcl
source to_menu.tcl
source to_color.tcl

cd bbdemo
source to_utils.tcl
source to_bbaud.tcl
source to_fgen.tcl
source screen.tcl
cd ..

cd rcvrdemo
source to_rcvr.tcl
source screen.tcl
source P25_DEC.TCL
source TO_P25.TCL
cd ..

cd ch_anlzr_demo
source to_anlzr.tcl
source screen.tcl
cd ..

cd rfgen_demo
source to_rfgen.tcl
source screen.tcl
source p25parms.tcl
source to_p25e.tcl
set_rfgen_onoff ON
cd ..

cd scopedemo
source to_scope.tcl
source screen.tcl
cd ..

cd meterdemo
source to_rff.tcl
source screen_rff.tcl

source to_evm.tcl
source screen_evm.tcl

source to_dvm.tcl
source screen_dvm.tcl

source to_pwt.tcl
source screen_pt.tcl

source to_sinad.tcl
source screen_sinad.tcl

source to_fmdev.tcl
source screen_fmdev.tcl

source to_afctr.tcl
source screen_afctr.tcl

source to_rlm.tcl
source screen_rlm.tcl

source screen_info.tcl

source to_alert.tcl
source screen_alert.tcl

source screen.tcl
cd ..

#*************************************************
#       Name: TclFileCopy
# Parameters: filein - copy from
#             fileout - copy to
#Description: copy ascii file file
#*************************************************
proc TclFileCopy { filein fileout } {

  set fin  [open $filein  r ]
  set fout [open $fileout w ]

  while { [ gets $fin line ] >= 0 } {
    puts $fout $line
  }

  flush $fout

  close $fin
  close $fout
} 

#*************************************************
#       Name: OpenCalFile
# Parameters: filename - file name to use
#             attr     - attributes
#Description: open cal file, if not open then
#             find default file, copy into filename
#*************************************************
proc OpenCalFile { filename attr } {

  set code [ catch {
    set file [open /system/calibration/$filename $attr]
  } err ]

  if { $code } {
    set code [ catch {
      set file [open /system/calibration/$filename.default $attr]
    } err ]

    if { $code } {
      puts "OpenCalFile: $err"
    } else {
      set temp_path [ pwd ]
      cd /system/calibration
      TclFileCopy $filename.default $filename
      cd $temp_path 
    }
  } 

  return $file
}

#
#  Get cal data from calibration file & load Engines
#
get_tcxo_cal_data      tcxo
update_tcxo_cal_data

get_rcvr_yig_cal_data  rcvr_yig
update_rcvr_yig_cal_data

rcvrcal_freq_read_cal rcvr_freq_ant ANT
rcvrcal_set_engine_cal ANT

rcvrcal_freq_read_cal rcvr_freq_tr TR
rcvrcal_set_engine_cal TR

get_rfgen_yig_cal_data rfgen_yig
update_rfgen_yig_cal_data

gencal_alc_read_cal    rfgen_alc
gencal_set_engine_level

gencal_freq_read_cal   rfgen_freq_tr  TR
gencal_set_engine_freq TR

gencal_freq_read_cal   rfgen_freq_gen GEN
gencal_set_engine_freq GEN

get_scope_cal_data     scope
update_scope_calibration

# read CAL data from PT EEprom to PT Engine
Power enable on
read_pwt_eeprom

get_anlzr_cal_data analyzer
get_mixer_router_cal_data bb_audio_lvl


#
# all loaded, now start screen
#
frame .demorcvr
frame .demorfgen
frame .demobbaudio
frame .demochanlzr
frame .demoscope
frame .demometers

frame .demoscopedisplay
frame .demoanlzrdisplay


build_rcvr_screen      .demorcvr
build_bbaudio_screen   .demobbaudio
build_ch_anlzr_screen  .demochanlzr
build_rfgen_screen     .demorfgen
build_scope_screen     .demoscope
build_meter_screen     .demometers


pack .demorcvr     -side top -fill x
pack .demorfgen    -side top -fill x
pack .demochanlzr  -side top -fill x
pack .demobbaudio  -side top -fill x
pack .demoscope    -side top -fill x
pack .demometers   -side top -fill x


#
#  Open callbacks
#
create_bbaudio_callback
create_rcvr_callback
create_p25dec_callback
create_rfgen_callback
create_anlzr_callback
create_scope_callback

build_ch_anlzr_trace   .demoanlzrdisplay
build_scope_trace      .demoscopedisplay

pack .demoanlzrdisplay  -side left
pack .demoscopedisplay  -side right

create_dvm_callback
create_rlm_callback
create_pwt_callback
create_pwt_alert_callback
create_utils_callback

#
#  Set MB Enables (allow Vx Messages to appear on MB)
#
set_bbaudio_enable DEMO
set_rcvr_enable    DEMO
set_anlzr_enable   { LIVE TOS SGLPT }
set_rfgen_enable   DEMO
set_scope_enable 1 DEMO
set_scope_enable 2 DEMO
set_scope_enable 3 DEMO
hw_utils_run       DEMO
rlm_enable         ALL

pwt_run OFF
pwt_run READCALS
read_pwt_eeprom
pwt_run ALL



#
#  Set HW to match checkboxes
#
reset_rcvr_hw
init_bbaudio_hardware
reset_ch_anlzr_hardware
reset_rfgen_hw
init_scope_hardware
set_pt_reset
reset_dvm_hw
reset_pwt_hw
reset_sinad_hw
default_p25_parms



#
#  Global Key Bindings
#
#  help    is F1
#  softkey is F2-F7
#  return  is F8
#  mode    is F9
#  hold    is F10
#  power   is F11
#  spinner is F26,F27
#  volume  is F28,F29
#  squelch is F30,F31
#  shift   is F32
#
#bind . <Any-KeyPress> <puts "Keysym: %K" }
#
bind all <F29> { BBAudio speakervolrel 1 }
bind all <F28> { BBAudio speakervolrel -1 }
bind all <F32> { toggle_shift_led }
bind all <F11> { poweroff }

wm title . "IFR-2975"
bind . <Destroy> { after 100 {exec reboot } }

set_start_yig_calibration
