#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C)1998,99,2000   DESIGN ENGINEERING    #
# --------------------------------------------------------------------------#
# FILE        : screen.tcl                                                  #
# PRODUCT     : FITS                                                        #
# AUTHOR      : Steve Johnson, Bruce Butterfield                            #
# DESCRIPTION : Meters demo screen                                          #
#                                                                           #
#############################################################################
set METER_IN_ROUTING(Sinad)     -1
set METER_IN_ROUTING(AFctr)     -1
set METER_IN_ROUTING(Scope)     -1

set METER_ID(AFctr)             $BBAUDIO_ENGINE(SETARMROUTE_IN_DEVICE14)
set METER_ID(Sinad)             $BBAUDIO_ENGINE(SETARMROUTE_IN_DEVICE15)
set METER_ID(Scope)             $BBAUDIO_ENGINE(SETARMROUTE_IN_DEVICE16)

#
#  Meter Font Declaration
#
font create meter_font -family helvetica -size 8

#
#  allow a manual reset of all meters
#
proc reset_meter_hw { } {
    reset_rff_hw 
    reset_sinad_vars
    reset_sinad_hw
}

#
#  build meter screen
#
proc build_meter_screen { name } {
  global hbms
  global DemoColor
  global METER_IN_ROUTING
  global METER_ID
  global BBAUDIO_ENGINE

  # we have to save $hbms because the -command {} will not remember
  # local instance of $name
  set hbms $name

  #
  # define screen frames
  #
  frame $hbms.mbar          -relief raised -bd 2


  message $hbms.mbar.label        -text "Meters" -font meter_font \
                                  -bg $DemoColor(title) \
                                  -width $DemoColor(labelwidth)
#
# define reset hw button
#
button $name.changeui      -text  "Main UI" -font meter_font \
                           -command eng_yi_2_main_ui \
                           -bg white

#
#  Define menu bars 
#
menubutton $hbms.mbar.rff   -text  "RFF"  -font meter_font \
                            -menu $hbms.mbar.rff.menu \
                            -bg $DemoColor(menu) -relief raised
menubutton $hbms.mbar.evm   -text  "C4FM"         -font meter_font \
                            -menu $hbms.mbar.evm.menu \
                            -bg $DemoColor(menu) -relief raised
menubutton $hbms.mbar.sinad  -text  "Sinad"       -font meter_font \
                            -menu $hbms.mbar.sinad.menu \
                            -bg $DemoColor(menu) -relief raised
menubutton $hbms.mbar.dvm   -text  "DVM"          -font meter_font \
                            -menu $hbms.mbar.dvm.menu \
                            -bg $DemoColor(menu) -relief raised
menubutton $hbms.mbar.pwt   -text  "PT"          -font meter_font \
                            -menu $hbms.mbar.pwt.menu \
                            -bg $DemoColor(menu) -relief raised
menubutton $hbms.mbar.fmdev -text  "FMDEV"          -font meter_font \
                            -menu $hbms.mbar.fmdev.menu \
                            -bg $DemoColor(menu) -relief raised
menubutton $hbms.mbar.afctr -text  "AFCTR"          -font meter_font \
                            -menu $hbms.mbar.afctr.menu \
                            -bg $DemoColor(menu) -relief raised
menubutton $hbms.mbar.rlmtr -text  "RLM"          -font meter_font \
                            -menu $hbms.mbar.rlmtr.menu \
                            -bg $DemoColor(menu) -relief raised
menubutton $hbms.mbar.info  -text  "Info"         -font meter_font \
                            -menu $hbms.mbar.info.menu \
                            -bg $DemoColor(menu) -relief raised
menubutton $hbms.mbar.alert -text  "Alerts"       -font meter_font \
                            -menu $hbms.mbar.alert.menu \
                            -bg $DemoColor(menu) -relief raised


#
#  Build RF Frequency Error menu 
#
menu $hbms.mbar.rff.menu
$hbms.mbar.rff.menu add command \
              -label "RFF Meter" -font meter_font \
              -command { WidgetWindow build_RFF_meter_screen \
                         $hbms.rff $hbms "RF Error Meter"}

#
#  Build EVM menu 
#
menu $hbms.mbar.evm.menu
$hbms.mbar.evm.menu add command \
              -label "C4FM Meter" -font meter_font \
              -command { WidgetWindow build_EVM_meter_screen \
                         $hbms.evm $hbms "C4FM error meter"}

#
#  Build Sinad menu 
#
menu $hbms.mbar.sinad.menu
$hbms.mbar.sinad.menu add command \
              -label "Display Meter" -font meter_font \
              -command { WidgetWindow build_Sinad_meter_screen \
                         $hbms.sinad $hbms "Sinad Meter"}
$hbms.mbar.sinad.menu add command \
              -label "Remove Input" -font meter_font \
              -command { remove_sinad_routing }

#
#  Build DVM menu 
#
menu $hbms.mbar.dvm.menu
$hbms.mbar.dvm.menu add command \
              -label "DVM Meter" -font meter_font \
              -command { WidgetWindow create_dvm_screen \
                         $hbms.dvm $hbms "DVM"}

#
#  Build PowerTerm menu 
#
menu $hbms.mbar.pwt.menu
$hbms.mbar.pwt.menu add command \
              -label "Pwr Term Mtr" -font meter_font \
              -command { WidgetWindow create_pwt_screen \
                         $hbms.pwt $hbms "Power Term"}
$hbms.mbar.pwt.menu add command \
              -label "Reset Ant/Gen Port" -font meter_font \
              -command set_pt_reset

#
#  Build FM Deviation menu 
#
menu $hbms.mbar.fmdev.menu
$hbms.mbar.fmdev.menu add command \
              -label "FM Deviation" -font meter_font \
              -command { WidgetWindow build_FMDev_meter_screen \
                         $hbms.fmdev $hbms "FM Deviation"}

#
#  Build AF Counter menu 
#
menu $hbms.mbar.afctr.menu
$hbms.mbar.afctr.menu add command \
              -label "Display Meter" -font meter_font \
              -command { WidgetWindow build_AFCTR_meter_screen \
                         $hbms.afctr $hbms "AF Counter"}
$hbms.mbar.afctr.menu add command \
              -label "Remove Input" -font meter_font \
              -command { remove_afctr_routing }

#
#  Build Receive Level Meter menu 
#
menu $hbms.mbar.rlmtr.menu
$hbms.mbar.rlmtr.menu add command \
              -label "Rcv Level Meter" -font meter_font \
              -command { WidgetWindow build_RLM_meter_screen \
                         $hbms.rlmtr $hbms "Receive Level Meter"}
$hbms.mbar.rlmtr.menu add command \
              -label "Calibrate" -font meter_font \
              -command { WidgetWindow build_RLM_cal_screen \
                         $hbms.rlmtr_cal $hbms "Receive Level Meter Calibration"}

#
#  Build Meter Info menu 
#
menu $hbms.mbar.info.menu
$hbms.mbar.info.menu add command \
              -label "Meter Versions" -font meter_font \
              -command { WidgetWindow build_meter_INFO_screen \
                         $hbms.info $hbms "Meter Versions"}

#
#  Build Alert menu 
#
menu $hbms.mbar.alert.menu
$hbms.mbar.alert.menu add command \
              -label "Alerts" -font meter_font \
              -command { WidgetWindow build_ALERT_meter_screen \
                         $hbms.alert $hbms "Alerts"}



#
#  pack sub frames
#
pack $hbms.mbar.label        -side left  -fill both
pack $hbms.mbar.rff          -side left  -fill both -expand true
pack $hbms.mbar.evm          -side left  -fill both -expand true
pack $hbms.mbar.sinad        -side left  -fill both -expand true
pack $hbms.mbar.dvm          -side left  -fill both -expand true
pack $hbms.mbar.pwt          -side left  -fill both -expand true
pack $hbms.mbar.fmdev        -side left  -fill both -expand true
pack $hbms.mbar.afctr        -side left  -fill both -expand true
pack $hbms.mbar.rlmtr        -side left  -fill both -expand true
pack $hbms.mbar.info         -side left  -fill both -expand true
pack $hbms.mbar.alert        -side left  -fill both -expand true

#
#  pack main frame
#
pack $hbms.mbar              -side left  -fill both -expand true
pack $hbms.changeui          -side right -fill both

return $hbms
}

proc remove_afctr_routing { } {
    BBAudio route NONE AFCTR
}

proc remove_sinad_routing { } {
    BBAudio route NONE MSINAD
}
