#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C) 2000   DESIGN ENGINEERING           #
# --------------------------------------------------------------------------#
# FILE        : screen_afctr.tcl                                            #
# PRODUCT     : FITS                                                        #
# AUTHOR      : Bruce Butterfield                                           #
# DESCRIPTION : Audio Frequency (AF) Counter Meter                          #
#                                                                           #
#############################################################################

package require AFCounter

font create afctrs_font -family helvetica -size 10


#
#   build_AFCTR_meter_screen
#
proc build_AFCTR_meter_screen { name } {
  global bafctrms
  global DemoColor
  global afctrs_font

set bafctrms $name

frame $bafctrms.head
frame $bafctrms.enable
frame $bafctrms.gate
frame $bafctrms.trig
frame $bafctrms.avg
frame $bafctrms.freq
frame $bafctrms.freqerr
frame $bafctrms.state

#
#  Meter header
#
#
# Set up Meter Enable / Disable buttons
#

radiobutton $bafctrms.enable.on  -text "Enabled" -font afctrs_font \
                      -command { ::AFCounter::QuickStart } \
                      -variable ::AFCounter::State(enabled) \
                      -value ON
radiobutton $bafctrms.enable.off -text "Disabled" -font afctrs_font \
                      -command { ::AFCounter::QuickStop } \
                      -variable ::AFCounter::State(enabled) \
                      -value OFF
 
pack $bafctrms.enable.on   -side left -fill both -expand true
pack $bafctrms.enable.off  -side left -fill both -expand true

#
# Set up Meter Gate buttons
#

radiobutton $bafctrms.gate.tenth  -text "0.1 Second" -font afctrs_font \
                      -command { AFCounter gate $::AFCounter::State(Engine_gate) } \
                      -variable ::AFCounter::State(Engine_gate) \
                      -value 0
radiobutton $bafctrms.gate.second -text "1.0 Second" -font afctrs_font \
                      -command { AFCounter gate $::AFCounter::State(Engine_gate) } \
                      -variable ::AFCounter::State(Engine_gate) \
                      -value 1
 
pack $bafctrms.gate.tenth   -side left -fill both -expand true
pack $bafctrms.gate.second  -side left -fill both -expand true

#
# Set up meter trigger
#

radiobutton $bafctrms.trig.one -text "One-Shot" -font afctrs_font \
                      -command { AFCounter trig $::AFCounter::State(trig) } \
                      -variable ::AFCounter::State(trig) \
                      -value 2
radiobutton $bafctrms.trig.cont -text "Continuous" -font afctrs_font \
                      -command { AFCounter trig $::AFCounter::State(trig) } \
                      -variable ::AFCounter::State(trig) \
                      -value 1
 
pack $bafctrms.trig.one   -side left -fill both -expand true
pack $bafctrms.trig.cont  -side left -fill both -expand true

#
#  Average Widget
#
label $bafctrms.avg.label -text "No. Samples"
entry $bafctrms.avg.entry -relief sunken  -width 8 \
                       -textvariable ::AFCounter::State(average)
bind  $bafctrms.avg.entry  <Return> \
           { AFCounter avg $::AFCounter::State(average) }
pack $bafctrms.avg.label   -side left -fill both -expand true
pack $bafctrms.avg.entry   -side left -fill both -expand true


#
# Display the AF Counter meter output information
#
label $bafctrms.freq.label -text "Frequency"
label $bafctrms.freq.entry -textvariable ::AFCounter::State(reading) -relief sunken

pack $bafctrms.freq.label  -side top
pack $bafctrms.freq.entry  -side top


label $bafctrms.freqerr.label -text "Precision"
label $bafctrms.freqerr.entry -textvariable ::AFCounter::State(precision) -relief sunken

pack $bafctrms.freqerr.label  -side top
pack $bafctrms.freqerr.entry  -side top


label $bafctrms.state.label -text "Status"
label $bafctrms.state.entry -textvariable ::AFCounter::State(status) -relief sunken

pack $bafctrms.state.label  -side top
pack $bafctrms.state.entry  -side top


#
#  Pack screen
#
pack $bafctrms.head        -side top
pack $bafctrms.enable      -side top
pack $bafctrms.gate        -side top
pack $bafctrms.trig        -side top
pack $bafctrms.avg         -side top
pack $bafctrms.freq        -side top
pack $bafctrms.freqerr     -side top
pack $bafctrms.state       -side top
}

#
#  allow a manual reset of all hardware
#
proc reset_afctr_hw { } {
    ::AFCounter::DefaultSetup 
    ::AFCounter::QuickStop
}




