#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C) 2000   DESIGN ENGINEERING           #
# --------------------------------------------------------------------------#
# FILE        : screen_alert.tcl                                            #
# PRODUCT     : FITS                                                        #
# AUTHOR      : Bruce Butterfield                                           #
# DESCRIPTION : Alert Engine test screen                                    #
#                                                                           #
#############################################################################

package require BBAudio

set ALERT_SCRN(Freq)            800.0
set ALERT_SCRN(Duration)        1000
set ALERT_SCRN(Priority)        100
set ALERT_SCRN(Handle)          123456
set ALERT_SCRN(Level)           50.0
set ALERT_SCRN(Type)            NONE
set ALERT_SCRN(ToneSel)         RAW

set ALERT_SCRN(Type)            DONE

set ALERT_SCRN(Spkr_VolHW)      0
set ALERT_SCRN(Spkr_VolPercent) 0
set ALERT_SCRN(Spkr_VolTemp)    0

font create alert_font -family helvetica -size 10


#
#   build_ALERT_meter_screen
#
proc build_ALERT_meter_screen { name } {
  global balertms
  global ALERT_SCRN
  global DemoColor
  global alerts_font

set balertms $name

frame $balertms.head
frame $balertms.freq
frame $balertms.time
frame $balertms.priority
frame $balertms.handle
frame $balertms.level
frame $balertms.warn
frame $balertms.alarm
frame $balertms.prompt
frame $balertms.tonesel
frame $balertms.spkr_hw
frame $balertms.spkr_percent
frame $balertms.spkr_temp
frame $balertms.reset_vol
frame $balertms.tone_test
frame $balertms.reset

#
#  Meter header
#
label $balertms.head.label -text "Alert Manager Test" -bg $DemoColor(title)

pack $balertms.head.label   -side left -fill both -expand true

#
#  Frequency Setup
#
label $balertms.freq.label -text "Tone Frequency"
entry $balertms.freq.entry -relief sunken  -width 8 -textvariable ALERT_SCRN(Freq)
pack $balertms.freq.label   -side left -fill both -expand true
pack $balertms.freq.entry   -side left -fill both -expand true

#
#  Duration Setup
#
label $balertms.time.label -text "Duration (ms)"
entry $balertms.time.entry -relief sunken  -width 8 -textvariable ALERT_SCRN(Duration)
pack $balertms.time.label   -side left -fill both -expand true
pack $balertms.time.entry   -side left -fill both -expand true

#
#  Priority Setup
#
label $balertms.priority.label -text "Priority (0-255)"
entry $balertms.priority.entry -relief sunken  -width 8 -textvariable ALERT_SCRN(Priority)
pack $balertms.priority.label   -side left -fill both -expand true
pack $balertms.priority.entry   -side left -fill both -expand true

#
#  Handle Setup
#
label $balertms.handle.label -text "Handle (int)"
entry $balertms.handle.entry -relief sunken  -width 8 -textvariable ALERT_SCRN(Handle)
pack $balertms.handle.label   -side left -fill both -expand true
pack $balertms.handle.entry   -side left -fill both -expand true

#
#  Level Setup
#
label $balertms.level.label -text "Level (%)"
entry $balertms.level.entry -relief sunken  -width 8 -textvariable ALERT_SCRN(Level)
pack $balertms.level.label   -side left -fill both -expand true
pack $balertms.level.entry   -side left -fill both -expand true

#
# Set up Warnings
#

radiobutton $balertms.warn.std  -text "Std Warn" -font alerts_font \
                      -variable ALERT_SCRN(Type) \
                      -value STD_WARN
radiobutton $balertms.warn.crit -text "Crit Warn" -font alerts_font \
                      -variable ALERT_SCRN(Type) \
                      -value CRIT_WARN
 
pack $balertms.warn.std   -side left -fill both -expand true
pack $balertms.warn.crit  -side left -fill both -expand true

#
# Set up Alarms
#

radiobutton $balertms.alarm.lo  -text "Lo Alarm" -font alerts_font \
                      -variable ALERT_SCRN(Type) \
                      -value LO_ALARM
radiobutton $balertms.alarm.hi -text "Hi Alarm" -font alerts_font \
                      -variable ALERT_SCRN(Type) \
                      -value HI_ALARM
 
pack $balertms.alarm.lo   -side left -fill both -expand true
pack $balertms.alarm.hi  -side left -fill both -expand true

#
# Set up Prompts
#

radiobutton $balertms.prompt.enable  -text "Enabled" -font alerts_font \
                      -variable ALERT_SCRN(Type) \
                      -value ENABLED
radiobutton $balertms.prompt.info  -text "Info" -font alerts_font \
                      -variable ALERT_SCRN(Type) \
                      -value INFO
radiobutton $balertms.prompt.question  -text "Question" -font alerts_font \
                      -variable ALERT_SCRN(Type) \
                      -value QUESTION
radiobutton $balertms.prompt.done  -text "Done" -font alerts_font \
                      -variable ALERT_SCRN(Type) \
                      -value DONE
 
pack $balertms.prompt.enable     -side left -fill both -expand true
pack $balertms.prompt.info       -side left -fill both -expand true
pack $balertms.prompt.question   -side left -fill both -expand true
pack $balertms.prompt.done       -side left -fill both -expand true

#
# Set up Raw vs Standard tones
#
button $balertms.tonesel.raw  -text "Raw Tone" -font alerts_font \
                      -command { send_raw_tone } 
button $balertms.tonesel.std -text "Std Tone" -font alerts_font \
                      -command { send_std_tone } 
pack $balertms.tonesel.raw   -side left -fill both -expand true
pack $balertms.tonesel.std  -side left -fill both -expand true


#
#  Absolute setting of speaker volume
#
label $balertms.spkr_hw.label -text "Absolute Spkr Vol (0-31)"
entry $balertms.spkr_hw.entry -relief sunken  -width 8 -textvariable ALERT_SCRN(Spkr_VolHW)
bind  $balertms.spkr_hw.entry  <Return> \
           { BBAudio speakervolume $ALERT_SCRN(Spkr_VolHW) }
pack $balertms.spkr_hw.label   -side left -fill both -expand true
pack $balertms.spkr_hw.entry   -side left -fill both -expand true

#
#  Absolute setting of speaker volume
#
label $balertms.spkr_percent.label -text "Absolute Spkr Vol (0 - 100%)"
entry $balertms.spkr_percent.entry -relief sunken  -width 8 -textvariable ALERT_SCRN(Spkr_VolPercent)
bind  $balertms.spkr_percent.entry  <Return> \
           { BBAudio speakervolpercent $ALERT_SCRN(Spkr_VolPercent) }
pack $balertms.spkr_percent.label   -side left -fill both -expand true
pack $balertms.spkr_percent.entry   -side left -fill both -expand true

#
#  Absolute setting of speaker volume - Temporarily
#
label $balertms.spkr_temp.label -text "Temporary Spkr Vol (0 - 100%)"
entry $balertms.spkr_temp.entry -relief sunken  -width 8 -textvariable ALERT_SCRN(Spkr_VolTemp)
bind  $balertms.spkr_temp.entry  <Return> \
           { BBAudio debug speakervol_temp $ALERT_SCRN(Spkr_VolTemp) }
pack $balertms.spkr_temp.label   -side left -fill both -expand true
pack $balertms.spkr_temp.entry   -side left -fill both -expand true

#
# Reset speaker volume
#
button $balertms.reset_vol.sel  -text "Reset Volume" -font alerts_font \
                      -command { set_bbaudio_spkr_level_restore } 
pack $balertms.reset_vol.sel   -side left -fill both -expand true

#
# Tone Tests
#
button $balertms.tone_test.rise  -text "Rising Tones" -font alerts_font \
                      -command { rising_tones } 
button $balertms.tone_test.fall  -text "Falling Tones" -font alerts_font \
                      -command { falling_tones } 
button $balertms.tone_test.alert  -text "Alert Override" -font alerts_font \
                      -command { alert_tones } 
button $balertms.tone_test.alertall  -text "All Alerts" -font alerts_font \
                      -command { all_alert_tones } 
#pack $balertms.tone_test.rise   -side left -fill both -expand true
#pack $balertms.tone_test.fall   -side left -fill both -expand true
#pack $balertms.tone_test.alert  -side left -fill both -expand true
#pack $balertms.tone_test.alertall  -side left -fill both -expand true

#
# Reset Tests
#
button $balertms.reset.on  -text "Start Test" -font alerts_font \
                      -command { start_long_alert } 
button $balertms.reset.off  -text "Stop Test" -font alerts_font \
                      -command { stop_long_alert } 
#pack $balertms.reset.on   -side left -fill both -expand true
#pack $balertms.reset.off  -side left -fill both -expand true


#
#  Pack screen
#
pack $balertms.head        -side top
pack $balertms.freq        -side top
pack $balertms.time        -side top
pack $balertms.priority    -side top
pack $balertms.handle      -side top
pack $balertms.level       -side top
pack $balertms.warn        -side top
pack $balertms.alarm       -side top
pack $balertms.prompt      -side top
pack $balertms.tonesel     -side top
pack $balertms.spkr_hw     -side top
pack $balertms.spkr_percent  -side top
pack $balertms.spkr_temp   -side top
pack $balertms.reset_vol   -side top
#pack $balertms.tone_test   -side top
#pack $balertms.reset       -side top
}

#
#  allow a manual reset of all hardware
#
proc reset_alert_hw { } {
   global ALERT_SCRN

    # Initialize data settings of the engine
    set ALERT_SCRN(Freq)        800.0
    set ALERT_SCRN(Duration)    1000
    set ALERT_SCRN(Priority)    100
    set ALERT_SCRN(Handle)      123456
    set ALERT_SCRN(Level)       50.0
    set ALERT_SCRN(Type)        NONE
    set ALERT_SCRN(ToneSel)     RAW

    # Set up the Alert engine to the reset values
}

proc send_raw_tone { } {
   global ALERT_SCRN
   alert_raw_tone $ALERT_SCRN(Priority) $ALERT_SCRN(Handle) $ALERT_SCRN(Duration) $ALERT_SCRN(Freq) $ALERT_SCRN(Level) 
}

proc send_std_tone { } {
   global ALERT_SCRN
   alert_std_tone $ALERT_SCRN(Priority) $ALERT_SCRN(Handle) $ALERT_SCRN(Duration) $ALERT_SCRN(Type) 
}

proc rising_tones { } {
   alert_raw_tone 1 100 5000 880.0 50.0 
   alert_raw_tone 9 108 1000 554.4 50.0 
   alert_raw_tone 3 102 4000 784.0 50.0 
   alert_raw_tone 8 107 1500 587.3 50.0 
   alert_raw_tone 4 103 3500 740.0 50.0 
   alert_raw_tone 2 101 4500 830.6 50.0 
   alert_raw_tone 5 104 3000 698.5 50.0 
   alert_raw_tone 7 106 2000 622.3 50.0 
   alert_raw_tone 6 105 2500 659.3 50.0 
   alert_raw_tone 10 109 500 523.3 50.0 
}

proc falling_tones { } {
   alert_raw_tone 1 200 5000 440.0 50.0 
   alert_raw_tone 2 201 4000 466.2 50.0 
   alert_raw_tone 3 202 3000 493.9 50.0 
   alert_raw_tone 4 203 2000 523.3 50.0 
   alert_raw_tone 5 204 1000 554.4 50.0 
}

proc alert_tones { } {
   alert_raw_tone 10 301 8000 880.0 50.0 
   alert_std_tone 35 400 3500 CRIT_WARN 
   alert_raw_tone 20 302 7000 466.2 50.0 
   alert_raw_tone 30 303 6000 987.8 50.0 
   alert_raw_tone 40 304 2500 523.3 50.0 
   alert_raw_tone 50 305 2000 554.4 50.0 
   alert_raw_tone 60 306 1500 587.3 50.0 
   alert_raw_tone 70 307 1000 622.3 50.0 
   alert_raw_tone 80 308 500 659.3 50.0 
}

proc all_alert_tones { } {
   alert_std_tone 3 102 3000 LO_ALARM 
   alert_std_tone 1 100 5000 STD_WARN 
   alert_std_tone 5 104 2000 ENABLED 
   alert_std_tone 6 105 1500 INFO 
   alert_std_tone 2 101 4000 CRIT_WARN 
   alert_std_tone 7 106 1000 QUESTION 
   alert_std_tone 4 103 2500 HI_ALARM 
   alert_std_tone 8 107 500 DONE 
}

proc start_long_alert { } {
   global ALERT_SCRN
   alert_std_tone 2 101 9000 $ALERT_SCRN(Type) 
}

proc stop_long_alert { } {
   global ALERT_SCRN
   alert_std_tone 2 101 0 $ALERT_SCRN(Type) 
}

