#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C) 2000   DESIGN ENGINEERING           #
# --------------------------------------------------------------------------#
# FILE        : screen_dsptest.tcl                                          #
# PRODUCT     : FITS                                                        #
# AUTHOR      : Bruce Butterfield                                           #
# DESCRIPTION : Test of DSP messaging                                       #
#                                                                           #
#############################################################################

set DSPENUM(pentium)		0
set DSPENUM(parametric)		1
set DSPENUM(layer1)		2
set DSPENUM(multifunction)	3
set DSPENUM(scope)		4

set DSPTEST_SCRN(enable)	OFF
set DSPTEST_SCRN(dsp_id)	$DSPENUM(multifunction)
set DSPTEST_SCRN(delay)		50
set DSPTEST_SCRN(tag)		998


font create dsptest_font -family helvetica -size 8

#
#   build_DSP_meter_screen
#
proc build_DSP_meter_screen { name } {
  global bdsps
  global DSPENUM
  global DSPTEST_SCRN
  global DemoColor

set bdsps $name

frame $bdsps.head
frame $bdsps.enable
frame $bdsps.dspid
frame $bdsps.delay
frame $bdsps.tag


#
# Set up Meter Enable / Disable buttons
#

radiobutton $bdsps.enable.on  -text "Enabled" -font dsptest_font \
                      -command { set_enable_delay $DSPTEST_SCRN(enable) } \
                      -variable DSPTEST_SCRN(enable) \
                      -value ON
radiobutton $bdsps.enable.off -text "Disabled" -font dsptest_font \
                      -command { set_enable_delay $DSPTEST_SCRN(enable) } \
                      -variable DSPTEST_SCRN(enable) \
                      -value OFF
 
pack $bdsps.enable.on   -side left -fill both -expand true
pack $bdsps.enable.off  -side left -fill both -expand true

#
# Set up desired DSP
#
radiobutton $bdsps.dspid.parm  -text "Parm" -font dsptest_font \
                      -command { set_type_tag $DSPTEST_SCRN(dsp_id) } \
                      -variable DSPTEST_SCRN(dsp_id) \
                      -value $DSPENUM(parametric)
radiobutton $bdsps.dspid.L1  -text "L1" -font dsptest_font \
                      -command { set_type_tag $DSPTEST_SCRN(dsp_id) } \
                      -variable DSPTEST_SCRN(dsp_id) \
                      -value $DSPENUM(layer1)
radiobutton $bdsps.dspid.mf  -text "MF" -font dsptest_font \
                      -command { set_type_tag $DSPTEST_SCRN(dsp_id) } \
                      -variable DSPTEST_SCRN(dsp_id) \
                      -value $DSPENUM(multifunction)
radiobutton $bdsps.dspid.scope  -text "Scope" -font dsptest_font \
                      -command { set_type_tag $DSPTEST_SCRN(dsp_id) } \
                      -variable DSPTEST_SCRN(dsp_id) \
                      -value $DSPENUM(scope)

pack $bdsps.dspid.parm    -side left -fill both -expand true
pack $bdsps.dspid.L1      -side left -fill both -expand true
pack $bdsps.dspid.mf      -side left -fill both -expand true
pack $bdsps.dspid.scope   -side left -fill both -expand true

#
#  Set Delay
#
label $bdsps.delay.label -text "Delay (ms)"
entry $bdsps.delay.entry -relief sunken  -width 8 \
                       -textvariable DSPTEST_SCRN(delay)
bind  $bdsps.delay.entry  <Return> \
           { set_enable_delay $DSPTEST_SCRN(delay) }
pack $bdsps.delay.label   -side left -fill both -expand true
pack $bdsps.delay.entry   -side left -fill both -expand true

#
#  Set Tag
#
label $bdsps.tag.label -text "Message Tag"
entry $bdsps.tag.entry -relief sunken  -width 8 \
                       -textvariable DSPTEST_SCRN(tag)
bind  $bdsps.tag.entry  <Return> \
           { set_type_tag $DSPTEST_SCRN(tag) }
pack $bdsps.tag.label   -side left -fill both -expand true
pack $bdsps.tag.entry   -side left -fill both -expand true

#
#  Pack screen
#
pack $bdsps.head        -side top
pack $bdsps.enable      -side top
pack $bdsps.dspid       -side top
pack $bdsps.delay       -side top
pack $bdsps.tag         -side top
}

#
#  Set DSP type and tag
#
proc set_type_tag { } {
    global DSPTEST_SCRN

    T_SetDSPSim2 $DSPTEST_SCRN(dsp_id) $DSPTEST_SCRN(tag)
}

#
#  Set DSP enable and time delay
#
proc set_enable_delay { } {
    global DSPTEST_SCRN

    T_SetDSPSim1 $DSPTEST_SCRN(enable) $DSPTEST_SCRN(delay)
    if {$DSPTEST_SCRN(enable) == ON } {
        T_TestDSP
    }
}

#
#  Reset the meter
#
proc reset_dsptest_hw { } {
    global DSPTEST_SCRN

set DSPTEST_SCRN(enable)	OFF
set DSPTEST_SCRN(dsp_id)	$DSPENUM(multifunction)
set DSPTEST_SCRN(delay)		50
set DSPTEST_SCRN(tag)		998
}




