#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C)1998,99      DESIGN ENGINEERING      #
# --------------------------------------------------------------------------#
# FILE        : screen.dvm.tcl                                              #
# PRODUCT     : FITS                                                        #
# AUTHOR      : Steve Johnson                                               #
# DESCRIPTION : dvm demo screen                                             #
#                                                                           #
#############################################################################

#
# Display state variables
#
set DvmButton1 R40V
set DvmButton2 AC
set DvmButton3 DVM
set DvmButton4 LOADOFF
set DvmButton5 HZ60



#
#  allow a manual reset of all hardware
#
proc reset_dvm_hw { } {
  # these commands should not be used except in special conditions
  set_dvm_zero ZEROENABLE0
  set_dvm_hold HOLDNORMAL
  set_dvm_50hz HZ60

  # normal operation commands
  set_dvm_load LOADOFF
  set_dvm_range R40V
  set_dvm_mode AC
  set_dvm_source DVM
}

#
#  build dvm screen
#
proc create_dvm_screen { name } {
  global HCVS
#
# define screen frames
#
  set HCVS $name

frame $HCVS.info              -relief groove -bd 4

frame $HCVS.fctn              -relief groove -bd 4
frame $HCVS.fctn.range        -relief raised -bd 2
frame $HCVS.fctn.source       -relief raised -bd 2

frame $HCVS.oper              -relief groove -bd 4
frame $HCVS.oper.mode         -relief raised -bd 2
frame $HCVS.oper.hz           -relief groove -bd 2
frame $HCVS.oper.ohm          -relief groove -bd 2
frame $HCVS.oper.run          -relief groove -bd 2

frame $HCVS.mtr               -relief groove -bd 4
frame $HCVS.mtr.read          -relief raised -bd 2
frame $HCVS.mtr.stat          -relief raised -bd 2
frame $HCVS.mtr.hold          -relief raised -bd 2
frame $HCVS.mtr.zero          -relief raised -bd 2

#
#  Screen interface to hardware operations
#
message $HCVS.info.title        -text "Info:  "       -width 10c -font scope_font
button $HCVS.info.reset         -text Default -command { reset_dvm_all } -font dvm_font

message $HCVS.fctn.range.title  -text "Range"         -width 10c -font scope_font
message $HCVS.fctn.source.title -text "Input Signal"  -width 10c -font scope_font

message $HCVS.oper.mode.title   -text "Mode"          -width 10c -font scope_font
message $HCVS.oper.hz.title     -text "50/60 Hz Line" -width 10c -font scope_font
message $HCVS.oper.ohm.title    -text "Impedance"     -width 10c -font scope_font

message $HCVS.mtr.read.title    -text "Reading (Volts)"  -width 10c -font scope_font
message $HCVS.mtr.stat.title    -text "Status"           -width 10c -font scope_font
message $HCVS.mtr.hold.title    -text "Hold Reading"     -width 10c -font scope_font
message $HCVS.mtr.zero.title    -text "Zero Reading"     -width 10c -font scope_font


#
#  range selections
#
radiobutton $HCVS.fctn.range.bauto -text "AUTO" -font dvm_font \
       -command { set_dvm_range $DvmButton1 } -variable DvmButton1 -value RAUTO
radiobutton $HCVS.fctn.range.b400mv -text "400 mV" -font dvm_font \
       -command { set_dvm_range $DvmButton1 } -variable DvmButton1 -value R400MV
radiobutton $HCVS.fctn.range.b4v -text "4 V" -font dvm_font \
       -command { set_dvm_range $DvmButton1 } -variable DvmButton1 -value R4V
radiobutton $HCVS.fctn.range.b40v -text "40 V" -font dvm_font \
       -command { set_dvm_range $DvmButton1 } -variable DvmButton1 -value R40V
radiobutton $HCVS.fctn.range.b400v -text "400 V" -font dvm_font \
       -command { set_dvm_range $DvmButton1 } -variable DvmButton1 -value R400V

radiobutton $HCVS.fctn.source.bdvm -text "DVM" -font dvm_font \
       -command { set_dvm_source  $DvmButton3 } -variable DvmButton3 -value DVM
radiobutton $HCVS.fctn.source.bs1  -text "Ch 1" -font dvm_font \
       -command { set_dvm_source  $DvmButton3 } -variable DvmButton3 -value SCOPE1
radiobutton $HCVS.fctn.source.bs2  -text "Ch 2" -font dvm_font \
       -command { set_dvm_source  $DvmButton3 } -variable DvmButton3 -value SCOPE2

radiobutton $HCVS.oper.mode.bac -text "AC" -font dvm_font \
       -command { set_dvm_mode  $DvmButton2 } -variable DvmButton2 -value AC
radiobutton $HCVS.oper.mode.bdc -text "DC" -font dvm_font \
       -command { set_dvm_mode  $DvmButton2 } -variable DvmButton2 -value DC

radiobutton $HCVS.oper.ohm.oc  -text "OC" -font dvm_font \
       -command { set_dvm_load  $DvmButton4 } -variable DvmButton4 -value LOADOFF
radiobutton $HCVS.oper.ohm.150 -text "150" -font dvm_font \
       -command { set_dvm_load  $DvmButton4 } -variable DvmButton4 -value LOAD150
radiobutton $HCVS.oper.ohm.600 -text "600" -font dvm_font \
       -command { set_dvm_load  $DvmButton4 } -variable DvmButton4 -value LOAD600

radiobutton $HCVS.oper.hz.50 -text "50 Hz" -font dvm_font \
       -command { set_dvm_50hz  $DvmButton5 } -variable DvmButton5 -value HZ50
radiobutton $HCVS.oper.hz.60 -text "60 Hz" -font dvm_font \
       -command { set_dvm_50hz  $DvmButton5 } -variable DvmButton5 -value HZ60

button $HCVS.oper.run.on   -text Run  -command { dvm_run ON  } -font dvm_font
button $HCVS.oper.run.off  -text Stop -command { dvm_run OFF } -font dvm_font

label $HCVS.mtr.read.thing   -textvariable DVM_STATUS(reading)     -font dvm_font
label $HCVS.mtr.stat.thing1  -textvariable DVM_STATUS(holding)     -font dvm_font
label $HCVS.mtr.stat.thing2  -textvariable DVM_STATUS(zero)        -font dvm_font
label $HCVS.mtr.stat.thing3  -textvariable DVM_STATUS(range)       -font dvm_font
label $HCVS.mtr.stat.thing4  -textvariable DVM_STATUS(overvoltage) -font dvm_font
label $HCVS.mtr.stat.thing5  -textvariable DVM_STATUS(cmdchange)   -font dvm_font
label $HCVS.mtr.stat.thing6  -textvariable DVM_STATUS(disconnect)  -font dvm_font

button $HCVS.mtr.hold.on   -text Enable -command { set_dvm_hold HOLDENABLE } -font dvm_font
button $HCVS.mtr.hold.off  -text Normal -command { set_dvm_hold HOLDNORMAL } -font dvm_font

button $HCVS.mtr.zero.on   -text Enable -command { set_dvm_zero ZEROENABLE  } -font dvm_font
button $HCVS.mtr.zero.off  -text Normal -command { set_dvm_zero ZEROENABLE0 } -font dvm_font

#
# pack 
#
pack $HCVS.info.reset         -side left
pack $HCVS.info.title         -side left -fill both -expand true

pack $HCVS.fctn.range.title   -side top -fill both -expand true
pack $HCVS.fctn.range.bauto   -side top -fill both -expand true
pack $HCVS.fctn.range.b400mv  -side top -fill both -expand true
pack $HCVS.fctn.range.b4v     -side top -fill both -expand true
pack $HCVS.fctn.range.b40v    -side top -fill both -expand true
pack $HCVS.fctn.range.b400v   -side top -fill both -expand true

pack $HCVS.fctn.source.title  -side top  -fill both -expand true
pack $HCVS.fctn.source.bdvm   -side left -fill both -expand true
pack $HCVS.fctn.source.bs1    -side left -fill both -expand true
pack $HCVS.fctn.source.bs2    -side left -fill both -expand true

pack $HCVS.oper.mode.title    -side top  -fill both -expand true
pack $HCVS.oper.mode.bac      -side left -fill both -expand true
pack $HCVS.oper.mode.bdc      -side left -fill both -expand true

pack $HCVS.oper.ohm.title     -side top  -fill both -expand true
pack $HCVS.oper.ohm.oc        -side left -fill both -expand true
pack $HCVS.oper.ohm.150       -side left -fill both -expand true
pack $HCVS.oper.ohm.600       -side left -fill both -expand true

pack $HCVS.oper.hz.title      -side top  -fill both -expand true
pack $HCVS.oper.hz.50         -side left -fill both -expand true
pack $HCVS.oper.hz.60         -side left -fill both -expand true

pack $HCVS.oper.run.on        -side left -fill both -expand true
pack $HCVS.oper.run.off       -side left -fill both -expand true

pack $HCVS.mtr.read.title     -side top -fill both -expand true
pack $HCVS.mtr.read.thing     -side top -fill both -expand true

pack $HCVS.mtr.stat.title     -side top -fill both -expand true
pack $HCVS.mtr.stat.thing1    -side top -fill both -expand true
pack $HCVS.mtr.stat.thing2    -side top -fill both -expand true
pack $HCVS.mtr.stat.thing3    -side top -fill both -expand true
pack $HCVS.mtr.stat.thing4    -side top -fill both -expand true
pack $HCVS.mtr.stat.thing5    -side top -fill both -expand true
pack $HCVS.mtr.stat.thing6    -side top -fill both -expand true

pack $HCVS.mtr.hold.title     -side top  -fill both -expand true
pack $HCVS.mtr.hold.on        -side left -fill both -expand true
pack $HCVS.mtr.hold.off       -side left -fill both -expand true

pack $HCVS.mtr.zero.title     -side top  -fill both -expand true
pack $HCVS.mtr.zero.on        -side left -fill both -expand true
pack $HCVS.mtr.zero.off       -side left -fill both -expand true

#
#  pack subframes
#
pack $HCVS.fctn.range         -side top  -fill both -expand true
pack $HCVS.fctn.source        -side top  -fill both -expand true

pack $HCVS.oper.mode          -side top  -fill both -expand true
pack $HCVS.oper.ohm           -side top  -fill both -expand true
pack $HCVS.oper.hz            -side top  -fill both -expand true
pack $HCVS.oper.run           -side top  -fill both -expand true

pack $HCVS.mtr.read           -side top  -fill both -expand true
pack $HCVS.mtr.stat           -side top  -fill both -expand true
pack $HCVS.mtr.hold           -side top  -fill both -expand true
pack $HCVS.mtr.zero           -side top  -fill both -expand true

#
#  pack main frame
#
pack $HCVS.info               -side top  -fill both -expand true
pack $HCVS.fctn               -side left -fill both -expand true
pack $HCVS.oper               -side left -fill both -expand true
pack $HCVS.mtr                -side left -fill both -expand true
}
