#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C) 2000   DESIGN ENGINEERING           #
# --------------------------------------------------------------------------#
# FILE        : screen_fmdev.tcl                                            #
# PRODUCT     : FITS                                                        #
# AUTHOR      : Bruce Butterfield                                           #
# DESCRIPTION : FM Deviation Meter                                          #
#                                                                           #
#############################################################################

package require FMDev

font create fmdevs_font -family helvetica -size 10


#
#   build_FMDev_meter_screen
#
proc build_FMDev_meter_screen { name } {
  global bfmdevs
  global DemoColor
  global fmdevs_font

set bfmdevs $name

frame $bfmdevs.head
frame $bfmdevs.enable
frame $bfmdevs.trig
frame $bfmdevs.readmode
frame $bfmdevs.avg
frame $bfmdevs.pos_value
frame $bfmdevs.neg_value
frame $bfmdevs.avg_value

#
# Set up Meter Enable / Disable buttons
#

radiobutton $bfmdevs.enable.on  -text "Enabled" -font fmdevs_font \
                      -command { FMDev run ON } \
                      -variable ::FMDev::State(run) \
                      -value ON
radiobutton $bfmdevs.enable.off -text "Disabled" -font fmdevs_font \
                      -command { FMDev run OFF } \
                      -variable ::FMDev::State(run) \
                      -value OFF
 
pack $bfmdevs.enable.on   -side left -fill both -expand true
pack $bfmdevs.enable.off  -side left -fill both -expand true

#
# Set up meter trigger
#

radiobutton $bfmdevs.trig.one -text "One-Shot" -font fmdevs_font \
                      -command { FMDev trig ONE_SHOT } \
                      -variable ::FMDev::State(trigger) \
                      -value ONE_SHOT
radiobutton $bfmdevs.trig.cont -text "Continuous" -font fmdevs_font \
                      -command { FMDev trig CONTINUOUS } \
                      -variable ::FMDev::State(trigger) \
                      -value CONTINUOUS
 
pack $bfmdevs.trig.off   -side left -fill both -expand true
pack $bfmdevs.trig.one   -side left -fill both -expand true
pack $bfmdevs.trig.cont  -side left -fill both -expand true

#
# Set up meter read mode
#

radiobutton $bfmdevs.readmode.peak  -text "Peak" -font fmdevs_font \
                      -command { FMDev math PEAK } \
                      -variable ::FMDev::State(math) \
                      -value PEAK
radiobutton $bfmdevs.readmode.average -text "Average" -font fmdevs_font \
                      -command { FMDev math AVERAGE } \
                      -variable ::FMDev::State(math) \
                      -value AVERAGE
 
pack $bfmdevs.readmode.peak      -side left -fill both -expand true
pack $bfmdevs.readmode.average   -side left -fill both -expand true

#
#  Average Widget
#
label $bfmdevs.avg.label -text "No. Samples"
entry $bfmdevs.avg.entry -relief sunken  -width 8 \
                       -textvariable ::FMDev::State(average)
bind  $bfmdevs.avg.entry  <Return> \
           { FMDev avg $::FMDev::State(average) }
pack $bfmdevs.avg.label   -side left -fill both -expand true
pack $bfmdevs.avg.entry   -side left -fill both -expand true


#
# Display the FM Deviation meter output information
#
label $bfmdevs.pos_value.label -text "Positive Peak"
label $bfmdevs.pos_value.entry -textvariable ::FMDev::State(pos_value) -relief sunken
pack $bfmdevs.pos_value.label  -side top
pack $bfmdevs.pos_value.entry  -side top

label $bfmdevs.neg_value.label -text "Negative Peak"
label $bfmdevs.neg_value.entry -textvariable ::FMDev::State(neg_value) -relief sunken
pack $bfmdevs.neg_value.label  -side top
pack $bfmdevs.neg_value.entry  -side top

label $bfmdevs.avg_value.label -text "Average Value"
label $bfmdevs.avg_value.entry -textvariable ::FMDev::State(avg_value) -relief sunken
pack $bfmdevs.avg_value.label  -side top
pack $bfmdevs.avg_value.entry  -side top

#
#  Pack screen
#
pack $bfmdevs.head        -side top
pack $bfmdevs.enable      -side top
pack $bfmdevs.trig        -side top
pack $bfmdevs.readmode    -side top
pack $bfmdevs.avg         -side top
pack $bfmdevs.pos_value   -side top
pack $bfmdevs.neg_value   -side top
pack $bfmdevs.avg_value   -side top
}

#
#  allow a manual reset of all hardware
#
proc reset_fmdev_hw { } {
    ::FMDev::DefaultSetup
    ::FMDev::QuickStart
}


