#############################################################################
# IFR SYSTEMS, INC  COPYRIGHT (C)1998,99      DESIGN ENGINEERING            #
# --------------------------------------------------------------------------#
# FILE        : screen_pt.tcl                                               #
# PRODUCT     : FITS                                                        #
# AUTHOR      : 
# DESCRIPTION : power term diagnostics screen                               #
#                                                                           #
#############################################################################


proc reset_pwt_hw { } {
  read_pwt_task ON
  set_pwt_range AUTO
}


#*************************************************
#       Name: save_pt_meter_data
# Parameters: none
#Description: Saves the following data to a file:
#  Meter A/D reading
#  Meter A/D zero offset
#  Meter A/D reading after offset
#  System temp
#  PowerTerm temp
#  Raw power reading (after spline)
#  Frequency
#  Frequency compensation value
#  Frequency compensated power
#
# The file is /usr/powerterm_data.csv
#*************************************************
proc save_pt_meter_data { {filename /usr/powerterm_data.csv} } {
  global PWT_STATUS

  # If the file doesn't exists, create it and write
  # the header
  if {[file exists $filename] == 0} {
    set file [open $filename w]
    puts $file "Time,Meter A/D reading,Meter A/D zero offset,Meter A/D reading after offset,System temp,PowerTerm temp,Raw power reading (after spline),Frequency,Frequency compensation value,Frequency compensated power"
  } else {
    set file [open $filename a]
  }
  puts $file "[clock seconds],$PWT_STATUS(ad_value),$PWT_STATUS(zero_offset),$PWT_STATUS(ad_offset),$PWT_STATUS(temp1),$PWT_STATUS(temp2),$PWT_STATUS(raw_power),$PWT_STATUS(freq),$PWT_STATUS(pw_factor),$PWT_STATUS(rd_power)"
  close $file
}

proc clear_pt_meter_data { {filename /usr/powerterm_data.csv} } {
  file delete $filename
}
#*************************************************
#       Name: build_diagpower_screen
# Parameters: home, outer frame for screen
#Description: builds diagnostic power term screen
# Softkeys assigned:
# S1 - Zero meter
# S2 - 
# S6 - Write data to file
#*************************************************
proc create_pwt_screen {home} {
  global PWT_ENGINE
  global PWT_STATUS

  # Receiver's frame
  frame $home.center

  # Interface
  frame $home.center.interface
  frame $home.center.select
  frame $home.center.freq
  frame $home.center.ovld


  # Build interface of Receiver
  label $home.center.interface.adlabel   -text "AD value:"
  label $home.center.interface.advalue   -relief sunken \
               -textvariable PWT_STATUS(ad_value)  -width 13 -bg white

  label $home.center.interface.zerolabel       -text "Zero offset:"
  label $home.center.interface.zerovalue       -relief sunken \
               -textvariable PWT_STATUS(zero_offset) -width 13 -bg white

  label $home.center.interface.offsetlabel     -text "AD offset:"
  label $home.center.interface.offsetvalue     -relief sunken \
               -textvariable PWT_STATUS(ad_offset) -width 13 -bg white

  label $home.center.interface.rawpowerlabel   -text "Raw power:"
  label $home.center.interface.rawpowervalue    -relief sunken \
               -textvariable PWT_STATUS(raw_power) -width 13 -bg white

  label $home.center.interface.rdcondlabel   -text "Reading State:"
  label $home.center.interface.rdcondvalue   -relief sunken \
               -textvariable PWT_STATUS(rd_cond) -width 13 -bg white

  label $home.center.interface.freqfactorlabel -text "Freq factor:"
  label $home.center.interface.freqfactorvalue -relief sunken \
               -textvariable PWT_STATUS(pw_factor) -width 13 -bg white




  label $home.center.interface.finalpowerlbl   -text "RF Pwr (W):"
  label $home.center.interface.finalpowervalue -relief sunken \
               -textvariable PWT_STATUS(rd_power) -width 14 -bg white

  label $home.center.interface.freqlabel       -text "Frequency:"
  label $home.center.interface.freqvalue       -relief sunken \
               -textvariable PWT_STATUS(freq) -width 14 -bg white

  label $home.center.interface.temp1label      -text "Temp Det:"
  label $home.center.interface.temp1value      -relief sunken \
               -textvariable PWT_STATUS(temp1) -width 14 -bg white

  label $home.center.interface.temp2label      -text "Temp Ref:"
  label $home.center.interface.temp2value      -relief sunken \
               -textvariable PWT_STATUS(temp2) -width 14 -bg white

  label $home.center.interface.fanspeedlabel   -text "Fan speed:"
  label $home.center.interface.fanspeedvalue   -relief sunken \
               -textvariable PWT_STATUS(fan_speed) -width 14 -bg white

  label $home.center.interface.finalpowerlabel   -text "HW Range:"
  label $home.center.interface.finalpowerval     -relief sunken \
               -textvariable PWT_STATUS(rd_range) -bg white


  label $home.center.interface.precisionlbl     -text "Precision:"
  label $home.center.interface.precisionvalue   -relief sunken \
               -textvariable PWT_STATUS(rd_sig) -width 5 -bg white


  grid $home.center.interface.adlabel          -row 0 -column 0 -sticky e    -padx 4 -pady 4
  grid $home.center.interface.advalue          -row 0 -column 1 -sticky ew   -padx 4 -pady 4

  grid $home.center.interface.zerolabel        -row 1 -column 0 -sticky e    -padx 4 -pady 4
  grid $home.center.interface.zerovalue        -row 1 -column 1 -sticky ew   -padx 4 -pady 4

  grid $home.center.interface.offsetlabel      -row 2 -column 0 -sticky e    -padx 4 -pady 4
  grid $home.center.interface.offsetvalue      -row 2 -column 1 -sticky ew   -padx 4 -pady 4

  grid $home.center.interface.rawpowerlabel    -row 3 -column 0 -sticky e    -padx 4 -pady 4
  grid $home.center.interface.rawpowervalue    -row 3 -column 1 -sticky ew   -padx 4 -pady 4

  grid $home.center.interface.rdcondlabel      -row 4 -column 0 -sticky e    -padx 4 -pady 4
  grid $home.center.interface.rdcondvalue      -row 4 -column 1 -sticky ew   -padx 4 -pady 4

  grid $home.center.interface.freqfactorlabel  -row 5 -column 0 -sticky e    -padx 4 -pady 4
  grid $home.center.interface.freqfactorvalue  -row 5 -column 1 -sticky ew   -padx 4 -pady 4



  grid $home.center.interface.finalpowerlbl    -row 0 -column 2 -sticky e    -padx 4 -pady 4
  grid $home.center.interface.finalpowervalue  -row 0 -column 3 -sticky we   -padx 4 -pady 4

  grid $home.center.interface.freqlabel        -row 1 -column 2 -sticky e    -padx 4 -pady 4
  grid $home.center.interface.freqvalue        -row 1 -column 3 -sticky ew   -padx 4 -pady 4

  grid $home.center.interface.temp1label       -row 2 -column 2 -sticky e    -padx 4 -pady 4
  grid $home.center.interface.temp1value       -row 2 -column 3 -sticky ew   -padx 4 -pady 4

  grid $home.center.interface.temp2label       -row 3 -column 2 -sticky e    -padx 4 -pady 4
  grid $home.center.interface.temp2value       -row 3 -column 3 -sticky ew   -padx 4 -pady 4

  grid $home.center.interface.fanspeedlabel    -row 4 -column 2 -sticky e    -padx 4 -pady 4
  grid $home.center.interface.fanspeedvalue    -row 4 -column 3 -sticky ew   -padx 4 -pady 4

  grid $home.center.interface.finalpowerlabel  -row 5 -column 2 -sticky e    -padx 4 -pady 4
  grid $home.center.interface.finalpowerval    -row 5 -column 3 -sticky ew   -padx 4 -pady 4


  grid $home.center.interface.precisionlbl     -row 0 -column 4 -sticky e    -padx 4 -pady 4
  grid $home.center.interface.precisionvalue   -row 0 -column 5 -sticky we   -padx 4 -pady 4


  label $home.center.freq.label  -text "Rcvr Frequency (MHz):"
  entry $home.center.freq.entry  -relief sunken -textvariable RCVR_ENGINE(RF_Freq) -width 10 -bg white
  bind  $home.center.freq.entry  <Return> trace_rcvr_freq


  #
  #  Display Overload Information
  #
  label $home.center.ovld.lab1 -text "Ant:"
  label $home.center.ovld.ant -relief sunken \
                              -textvariable PWT_STATUS(Ant_ovld) \
                              -width 12 -bg white
  label $home.center.ovld.lab2 -text "Gen:"
  label $home.center.ovld.gen -relief sunken \
                              -textvariable PWT_STATUS(Gen_ovld) \
                              -width 12 -bg white
  label $home.center.ovld.lab3 -text "Temp Ovld:"
  label $home.center.ovld.temp -relief sunken \
                              -textvariable PWT_STATUS(temp_ovld) \
                              -width 12 -bg white


  #
  #  Build Defaults Menu 
  #
  menubutton $home.center.default -text "Default EEPROM" -menu $home.center.default.menu
  menu $home.center.default.menu
  $home.center.default.menu add command \
              -label "Factory Defaults" -font meter_font \
              -command { default_pwt_eeprom CALS; read_pwt_eeprom }
  $home.center.default.menu add command \
              -label "Zero" -font meter_font \
              -command { default_pwt_eeprom ZERO; read_pwt_eeprom }
  $home.center.default.menu add command \
              -label "Write Eeprom - image data" -font meter_font \
              -command { write_pwt_cals IMAGE }
  $home.center.default.menu add command \
              -label "Write Eeprom - driver data" -font meter_font \
              -command { write_pwt_cals DRIVER }
  $home.center.default.menu add command \
              -label "Save Image to Disk" -font meter_font \
              -command { pwt_disk_cal SAVE }
  $home.center.default.menu add command \
              -label "Read Image from Disk" -font meter_font \
              -command { pwt_disk_cal RECALL }


  entry $home.center.entry -relief sunken -textvariable PWT_ENGINE(zero_pattern) \
                           -width 3 -bg white


  #
  #  Build Range Selection 
  #
  menubutton $home.center.range -text "Range Selection" -menu $home.center.range.menu
  menu $home.center.range.menu
  $home.center.range.menu add radiobutton \
              -label AUTO -variable PWT_ENGINE(range) -value AUTO \
              -command { set_pwt_range  $PWT_ENGINE(range) }
  $home.center.range.menu add radiobutton \
              -label LOW  -variable PWT_ENGINE(range) -value LOW  \
              -command { set_pwt_range  $PWT_ENGINE(range) }
  $home.center.range.menu add radiobutton \
              -label HIGH -variable PWT_ENGINE(range) -value HI   \
              -command { set_pwt_range  $PWT_ENGINE(range) }


  #
  #  Build Message Enable menu 
  #
  menubutton $home.center.enable -text "MB Messages" -menu $home.center.enable.menu
  menu $home.center.enable.menu
  $home.center.enable.menu add command \
              -label "Disable Msgs" -font meter_font -command { Power enable off }
  $home.center.enable.menu add command \
              -label "Enable All Msgs" -font meter_font -command { Power enable on }
  $home.center.enable.menu add command \
              -label "Enable Alert Msgs" -font meter_font -command { Power enable on}


  frame $home.center.buttons
  button $home.center.buttons.zero      -text "Zero (SFTK1)" -command { Power zero }
  bind [winfo toplevel $home] <F2> { set_pwt_zero }
 
  button $home.center.buttons.log      -text "Log data (SFTK5)" -command { save_pt_meter_data }
  bind [winfo toplevel $home] <F7> { save_pt_meter_data } 

  button $home.center.buttons.clear    -text "Clear log (SFTK6)" -command { clear_pt_meter_data }
  bind [winfo toplevel $home] <F6> { clear_pt_meter_data } 

  checkbutton $home.center.buttons.task -text "PT Task" -offvalue OFF -onvalue ON \
                                -variable PWT_ENGINE(task_state) \
                                -command { read_pwt_task $PWT_ENGINE(task_state) }

  # pack sub-frames
  pack  $home.center.freq.label  -padx 5 -side left  -expand yes -fill both
  pack  $home.center.freq.entry  -padx 5 -side left  -expand yes -fill both

  pack $home.center.ovld.lab1    -padx 5 -side left  -expand yes -fill both
  pack  $home.center.ovld.ant    -padx 5 -side left  -expand yes -fill both
  pack $home.center.ovld.lab2    -padx 5 -side left  -expand yes -fill both
  pack  $home.center.ovld.gen    -padx 5 -side left  -expand yes -fill both
  pack $home.center.ovld.lab3    -padx 5 -side left  -expand yes -fill both
  pack  $home.center.ovld.temp   -padx 5 -side left  -expand yes -fill both

  pack  $home.center.interface       -side top   -expand yes -fill both
  pack  $home.center.ovld            -pady 2     -side top
  pack  $home.center.buttons         -side top
  pack  $home.center.buttons.zero    -side left 
  pack  $home.center.buttons.log     -side left
  pack  $home.center.buttons.clear   -side left
  pack  $home.center.buttons.task    -side left
  pack  $home.center.freq            -side top

  pack  $home.center.default     -padx 2 -side left  -expand yes -fill both
  pack  $home.center.entry       -padx 2 -side left  -expand yes -fill both
  pack  $home.center.enable      -padx 2 -side left  -expand yes -fill both
  pack  $home.center.range       -padx 2 -side left  -expand yes -fill both

  pack $home.center              -padx 5 -pady 5
}
