#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C)1998,99,2000   DESIGN ENGINEERING    #
# --------------------------------------------------------------------------#
# FILE        : screen_rlm.tcl                                              #
# PRODUCT     : FITS                                                        #
# AUTHOR      : Steve Johnson                                               #
# DESCRIPTION : Rcvr Level Meter                                            #
#                                                                           #
#############################################################################


set RLMMTR_SCRN(Enable)     OFF
set RLMMTR_SCRN(RawReading) ON
set RLMMTR_SCRN(cableloss)  0.0

#
#  allow a manual reset of all hardware
#
proc reset_rlm_hw { } {
}

#
#   build_RLM_meter_screen
#
proc build_RLM_meter_screen { name } {
  global RLM_ENGINE
  global RLMMTR_SCRN
  global RLM_STATUS
  global DemoColor
  global brlmms 

set brlmms $name


frame $brlmms.reading
frame $brlmms.precision
frame $brlmms.state


label $brlmms.label -text "Receive Level Meter" -bg $DemoColor(title)


#
# Set up Meter Enable - Disable buttons
#
checkbutton $brlmms.enable    -text "Enable Message (Readings)" \
                              -offvalue OFF -onvalue ON \
                              -variable RLMMTR_SCRN(Enable) -font rlm_font \
                              -command { rlm_run $RLMMTR_SCRN(Enable) }

label $brlmms.reading.label  -text "Reading:"
label $brlmms.reading.value  -relief sunken \
                  -textvariable RLM_STATUS(reading) -width 12 -bg white

label $brlmms.precision.label  -text "Precision:"
label $brlmms.precision.value  -relief sunken \
                  -textvariable RLM_STATUS(precision) -width 5 -bg white

label $brlmms.state.label  -text "Cal State:"
label $brlmms.state.value  -relief sunken \
                  -textvariable RLM_STATUS(state) -width 28 -bg white


#
#  Pack controls
#
pack $brlmms.reading.label    -side left -fill both -expand true
pack $brlmms.reading.value    -side left -fill both -expand true

pack $brlmms.precision.label  -side left -fill both -expand true
pack $brlmms.precision.value  -side left -fill both -expand true

pack $brlmms.state.label      -side left -fill both -expand true
pack $brlmms.state.value      -side left -fill both -expand true


#
#  Pack screen
#
pack $brlmms.label       -side top
pack $brlmms.enable      -side top
pack $brlmms.reading     -side top
pack $brlmms.precision   -side top
pack $brlmms.state       -side top
}



#
#   build_RLM_meter_screen
#
proc build_RLM_cal_screen { name } {
  global RLM_ENGINE
  global RLMMTR_SCRN
  global RLM_STATUS
  global DemoColor
  global brlmms 

set brlmms $name

frame $brlmms.size
frame $brlmms.start
frame $brlmms.step
frame $brlmms.loss
frame $brlmms.calstatus

label $brlmms.size.label  -text "Number Cal Pts:"
entry $brlmms.size.value  -relief sunken \
                  -textvariable RLM_ENGINE(num_cals) -width 5 -bg white

label $brlmms.start.label  -text "Starting RF Level (dBm):"
entry $brlmms.start.value  -relief sunken \
                  -textvariable RLM_ENGINE(start_lvl) -width 5 -bg white

label $brlmms.step.label  -text "Step size (dB):"
entry $brlmms.step.value  -relief sunken \
                  -textvariable RLM_ENGINE(step_lvl) -width 5 -bg white

label $brlmms.loss.label  -text "RF Cable Loss (dB):"
entry $brlmms.loss.value  -relief sunken \
                  -textvariable RLMMTR_SCRN(cableloss) -width 5 -bg white
bind  $brlmms.loss.value  <Return> do_rlm_cable_loss

label $brlmms.calstatus.label  -text "Cal Message:"
label $brlmms.calstatus.value  -relief sunken \
                 -textvariable RLM_STATUS(status) -width 15 -bg white
label $brlmms.calstatus.index  -relief sunken \
                 -textvariable RLM_STATUS(statusindex) -width 5 -bg white

checkbutton $brlmms.raw  -text "Raw Readings" -offvalue OFF -onvalue ON \
                         -variable RLMMTR_SCRN(RawReading) -font rlm_font \
                         -command { set_rlm_raw_mode $RLMMTR_SCRN(RawReading) }


button $brlmms.cal   -text "Do Calibration" -command { do_rl_calibration }
button $brlmms.abort -text "Abort Cal"      -command { abort_rl_calibration }


#
#  Pack sub-screens
#
pack $brlmms.size.label        -side left
pack $brlmms.size.value        -side left

pack $brlmms.start.label       -side left
pack $brlmms.start.value       -side left

pack $brlmms.step.label        -side left
pack $brlmms.step.value        -side left

pack $brlmms.loss.label        -side left
pack $brlmms.loss.value        -side left

pack $brlmms.calstatus.label   -side left
pack $brlmms.calstatus.value   -side left
pack $brlmms.calstatus.index   -side left

#
#  Pack screen
#
pack $brlmms.size        -side top
pack $brlmms.start       -side top
pack $brlmms.step        -side top
pack $brlmms.loss        -side top
pack $brlmms.calstatus   -side top
pack $brlmms.raw         -side top
pack $brlmms.cal         -side left
pack $brlmms.abort       -side left
}
