#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C) 2000   DESIGN ENGINEERING           #
# --------------------------------------------------------------------------#
# FILE        : screen_sinad.tcl                                            #
# PRODUCT     : FITS                                                        #
# AUTHOR      : Bruce Butterfield                                           #
# DESCRIPTION : Sinad / Distortion Error Meter                              #
#                                                                           #
#############################################################################

set SINAD_SCRN(NotchHi) 1100.00
set SINAD_SCRN(NotchLo) 900.00
set SINAD_SCRN(NoiseFreqHi) 10000.00
set SINAD_SCRN(NoiseFreqLo) 300.00
set SINAD_SCRN(Scaling) 50000
set SINAD_SCRN(FFT_Window) 1
set SINAD_SCRN(FFT_Alpha) 2.5
set SINAD_SCRN(FFT_Size) 2048
set SINAD_SCRN(FFT_Cal) 0


font create sinads_font -family helvetica -size 10


#
#   build_Sinad_meter_screen
#
proc build_Sinad_meter_screen { name } {
  global bsinadms
  global SINAD_STATUS
  global SINAD_SCRN
  global DemoColor
  global sinads_font

set bsinadms $name

frame $bsinadms.head
frame $bsinadms.enable
frame $bsinadms.trig
frame $bsinadms.avg
frame $bsinadms.filter
frame $bsinadms.notchhi
frame $bsinadms.notchlo
frame $bsinadms.noisehi
frame $bsinadms.noiselo
frame $bsinadms.scale
frame $bsinadms.fft_window
frame $bsinadms.fft_alpha
frame $bsinadms.fft_size
frame $bsinadms.fft_cal
frame $bsinadms.sigmag
frame $bsinadms.noisemag
frame $bsinadms.freq
frame $bsinadms.freqerr
frame $bsinadms.flags


#
# Set up Meter Enable / Disable buttons
#

radiobutton $bsinadms.enable.on  -text "Enabled" -font sinads_font \
                      -command { Sinad run ON } \
                      -variable ::Sinad::State(run) \
                      -value ON
radiobutton $bsinadms.enable.off -text "Disabled" -font sinads_font \
                      -command { Sinad run OFF } \
                      -variable ::Sinad::State(run) \
                      -value OFF
 
pack $bsinadms.enable.on   -side left -fill both -expand true
pack $bsinadms.enable.off  -side left -fill both -expand true

#
# Set up meter trigger
#

radiobutton $bsinadms.trig.one -text "One-Shot" -font sinads_font \
                      -command { Sinad trig ONE_SHOT } \
                      -variable ::Sinad::State(trigger) \
                      -value ONE_SHOT
radiobutton $bsinadms.trig.cont -text "Continuous" -font sinads_font \
                      -command { Sinad trig CONTINUOUS } \
                      -variable ::Sinad::State(trigger) \
                      -value CONTINUOUS
 
pack $bsinadms.trig.one   -side left -fill both -expand true
pack $bsinadms.trig.cont  -side left -fill both -expand true


#
#  Average Widget
#
label $bsinadms.avg.label -text "Average Samples"  -width 30
entry $bsinadms.avg.entry -relief sunken  -width 12 \
                       -textvariable ::Sinad::State(average)  
bind  $bsinadms.avg.entry  <Return> \
           { Sinad avg $::Sinad::State(average) }
label $bsinadms.avg.space -text " "  -width 3
pack $bsinadms.avg.label   -side left -fill both -expand true
pack $bsinadms.avg.entry   -side left -fill both -expand true
pack $bsinadms.avg.space   -side left -fill both -expand true

#
#  Filter Widget
#
label $bsinadms.filter.label -text "Filter Samples"  -width 30
entry $bsinadms.filter.entry -relief sunken  -width 12 \
                       -textvariable ::Sinad::State(filter)
bind  $bsinadms.filter.entry  <Return> \
           { Sinad filter $::Sinad::State(filter) }
label $bsinadms.filter.space -text " "  -width 3
pack $bsinadms.filter.label   -side left -fill both -expand true
pack $bsinadms.filter.entry   -side left -fill both -expand true
pack $bsinadms.filter.space   -side left -fill both -expand true

#
#  Notch Hi Freq Widget
#
label $bsinadms.notchhi.label -text "Notch Hi Frequency"  -width 30
entry $bsinadms.notchhi.entry -relief sunken  -width 12 \
                       -textvariable SINAD_SCRN(NotchHi)
bind  $bsinadms.notchhi.entry  <Return> \
           { Sinad debug hnotch $SINAD_SCRN(NotchHi) }
label $bsinadms.notchhi.space -text " "  -width 3
pack $bsinadms.notchhi.label   -side left -fill both -expand true
pack $bsinadms.notchhi.entry   -side left -fill both -expand true
pack $bsinadms.notchhi.space   -side left -fill both -expand true

#
#  Notch Lo Freq Widget
#
label $bsinadms.notchlo.label -text "Notch Lo Frequency"  -width 30
entry $bsinadms.notchlo.entry -relief sunken  -width 12 \
                       -textvariable SINAD_SCRN(NotchLo)
bind  $bsinadms.notchlo.entry  <Return> \
           { Sinad debug lnotch $SINAD_SCRN(NotchLo) }
label $bsinadms.notchlo.space -text " "  -width 3
pack $bsinadms.notchlo.label   -side left -fill both -expand true
pack $bsinadms.notchlo.entry   -side left -fill both -expand true
pack $bsinadms.notchlo.space   -side left -fill both -expand true

#
#  Noise Hi Freq Widget
#
label $bsinadms.noisehi.label -text "Noise Hi Frequency"  -width 30
entry $bsinadms.noisehi.entry -relief sunken  -width 12 \
                       -textvariable SINAD_SCRN(NoiseFreqHi)
bind  $bsinadms.noisehi.entry  <Return> \
           { Sinad debug hnoise $SINAD_SCRN(NoiseFreqHi) }
label $bsinadms.noisehi.space -text " "  -width 3
pack $bsinadms.noisehi.label   -side left -fill both -expand true
pack $bsinadms.noisehi.entry   -side left -fill both -expand true
pack $bsinadms.noisehi.space   -side left -fill both -expand true

#
#  Noise Lo Freq Widget
#
label $bsinadms.noiselo.label -text "Noise Lo Frequency"  -width 30
entry $bsinadms.noiselo.entry -relief sunken  -width 12 \
                       -textvariable SINAD_SCRN(NoiseFreqLo)
bind  $bsinadms.noiselo.entry  <Return> \
           { Sinad debug lnoise $SINAD_SCRN(NoiseFreqLo) }
label $bsinadms.noiselo.space -text " "  -width 3
pack $bsinadms.noiselo.label   -side left -fill both -expand true
pack $bsinadms.noiselo.entry   -side left -fill both -expand true
pack $bsinadms.noiselo.space   -side left -fill both -expand true


#
#  Scale Widget
#
label $bsinadms.scale.label -text "Scaling"  -width 30
entry $bsinadms.scale.entry -relief sunken  -width 12 \
                       -textvariable SINAD_SCRN(Scaling)
bind  $bsinadms.scale.entry  <Return> \
           { Sinad debug scale $SINAD_SCRN(Scaling) }
label $bsinadms.scale.space -text " "  -width 3
pack $bsinadms.scale.label   -side left -fill both -expand true
pack $bsinadms.scale.entry   -side left -fill both -expand true
pack $bsinadms.scale.space   -side left -fill both -expand true

#
#  FFT Window Widget
#
label $bsinadms.fft_window.label -text "FFT Window (1-10)"  -width 30
entry $bsinadms.fft_window.entry -relief sunken  -width 12 \
                       -textvariable SINAD_SCRN(FFT_Window)
bind  $bsinadms.fft_window.entry  <Return> \
           { Sinad debug window $SINAD_SCRN(FFT_Window) }
label $bsinadms.fft_window.space -text " "  -width 3
pack $bsinadms.fft_window.label   -side left -fill both -expand true
pack $bsinadms.fft_window.entry   -side left -fill both -expand true
pack $bsinadms.fft_window.space   -side left -fill both -expand true

#
#  FFT Alpha Widget
#
label $bsinadms.fft_alpha.label -text "FFT Alpha (1-100)"  -width 30
entry $bsinadms.fft_alpha.entry -relief sunken  -width 12 \
                       -textvariable SINAD_SCRN(FFT_Alpha)
bind  $bsinadms.fft_alpha.entry  <Return> \
           { Sinad debug alpha $SINAD_SCRN(FFT_Alpha) }
label $bsinadms.fft_alpha.space -text " "  -width 3
pack $bsinadms.fft_alpha.label   -side left -fill both -expand true
pack $bsinadms.fft_alpha.entry   -side left -fill both -expand true
pack $bsinadms.fft_alpha.space   -side left -fill both -expand true

#
#  FFT Size Widget
#
label $bsinadms.fft_size.label -text "FFT size (64-32768)"  -width 30
entry $bsinadms.fft_size.entry -relief sunken  -width 12 \
                       -textvariable SINAD_SCRN(FFT_Size)
bind  $bsinadms.fft_size.entry  <Return> \
           { Sinad debug size $SINAD_SCRN(FFT_Size) }
label $bsinadms.fft_size.space -text " "  -width 3
pack $bsinadms.fft_size.label   -side left -fill both -expand true
pack $bsinadms.fft_size.entry   -side left -fill both -expand true
pack $bsinadms.fft_size.space   -side left -fill both -expand true

#
#  FFT Calibration Widget
#
label $bsinadms.fft_cal.label -text "FFT cal (-100 to +100)"  -width 30
entry $bsinadms.fft_cal.entry -relief sunken  -width 12 \
                       -textvariable SINAD_SCRN(FFT_Cal)
bind  $bsinadms.fft_cal.entry  <Return> \
           { Sinad debug calibrate $SINAD_SCRN(FFT_Cal) }
label $bsinadms.fft_cal.space -text " "  -width 3
pack $bsinadms.fft_cal.label   -side left -fill both -expand true
pack $bsinadms.fft_cal.entry   -side left -fill both -expand true
pack $bsinadms.fft_cal.space   -side left -fill both -expand true

#
# Display the Sinad meter output information
#
label $bsinadms.sigmag.label -text "Signal Magnitude"  -width 30
label $bsinadms.sigmag.entry -textvariable ::Sinad::State(signalPower) -relief sunken  -width 12
label $bsinadms.sigmag.space -text " "  -width 3

pack $bsinadms.sigmag.label  -side left
pack $bsinadms.sigmag.entry  -side left
pack $bsinadms.sigmag.space  -side left

label $bsinadms.noisemag.label -text "Noise Magnitude"  -width 30
label $bsinadms.noisemag.entry -textvariable ::Sinad::State(noisePower) -relief sunken  -width 12
label $bsinadms.noisemag.space -text " "  -width 3

pack $bsinadms.noisemag.label  -side left
pack $bsinadms.noisemag.entry  -side left
pack $bsinadms.noisemag.space  -side left

label $bsinadms.freq.label -text "Sinad"  -width 30
label $bsinadms.freq.entry -textvariable ::Sinad::State(sinad) -relief sunken  -width 12
label $bsinadms.freq.space -text " "  -width 3

pack $bsinadms.freq.label  -side left
pack $bsinadms.freq.entry  -side left
pack $bsinadms.freq.space  -side left


label $bsinadms.freqerr.label -text "Distortion"  -width 30
label $bsinadms.freqerr.entry -textvariable ::Sinad::State(distortion) -relief sunken  -width 12
label $bsinadms.freqerr.space -text " "  -width 3

pack $bsinadms.freqerr.label  -side left
pack $bsinadms.freqerr.entry  -side left
pack $bsinadms.freqerr.space  -side left


#
#  Pack screen
#
pack $bsinadms.head        -side top
pack $bsinadms.enable      -side top
pack $bsinadms.trig        -side top
pack $bsinadms.avg         -side top
pack $bsinadms.filter      -side top
pack $bsinadms.notchhi     -side top
pack $bsinadms.notchlo     -side top
pack $bsinadms.noisehi     -side top
pack $bsinadms.noiselo     -side top
pack $bsinadms.scale       -side top
pack $bsinadms.fft_window  -side top
pack $bsinadms.fft_alpha   -side top
pack $bsinadms.fft_size    -side top
pack $bsinadms.fft_cal     -side top
pack $bsinadms.sigmag      -side top
pack $bsinadms.noisemag    -side top
pack $bsinadms.freq        -side top
pack $bsinadms.freqerr     -side top
}

#
#  allow a manual reset of all hardware
#
proc reset_sinad_vars { } {
    global SINAD_STATUS
    global SINAD_SCRN

    # Initialize data settings of the engine
    set SINAD_SCRN(NotchHi)     1100.00
    set SINAD_SCRN(NotchLo)     900.00
    set SINAD_SCRN(NoiseFreqHi) 10000.00
    set SINAD_SCRN(NoiseFreqLo) 300.00
    set SINAD_SCRN(Scaling)     50000
    set SINAD_SCRN(FFT_Window)  1
    set SINAD_SCRN(FFT_Alpha)   2.5
    set SINAD_SCRN(FFT_Size)    2048
}

proc reset_sinad_hw { } {
    global SINAD_SCRN

    # Set up the Sinad engine to default values
    ::Sinad::DefaultSetup 

    Sinad debug lnotch $SINAD_SCRN(NotchLo)
    Sinad debug hnotch $SINAD_SCRN(NotchHi)
    Sinad debug hnoise $SINAD_SCRN(NoiseFreqHi)
    Sinad debug lnoise $SINAD_SCRN(NoiseFreqLo)
    Sinad debug scale $SINAD_SCRN(Scaling)
    Sinad debug window $SINAD_SCRN(FFT_Window)
    Sinad debug alpha $SINAD_SCRN(FFT_Alpha)
    Sinad debug size $SINAD_SCRN(FFT_Size)
    Sinad debug calibrate $SINAD_SCRN(FFT_Cal)
}


