#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C) 2000    DESIGN ENGINEERING          #
# --------------------------------------------------------------------------#
# FILE        : to_alert.tcl                                                #
# PRODUCT     : FITS                                                        #
# AUTHOR      : Bruce Butterfield                                           #
# DESCRIPTION : Interface to the Alert manager                              #
#                                                                           #
#############################################################################

#
#  Alert Manager System Number
#
set ALERT_ENGINE(SYSTEM_ID)            27

#
#  Alert Manager Sub-Systems
#
set ALERT_ENGINE(MTR_SPECIFIC_START)    128
set ALERT_ENGINE(MTR_RAW_TONE)          129
set ALERT_ENGINE(MTR_STD_TONE)          130

set ALERT_STD_TYPE(ILLEGAL)		0
set ALERT_STD_TYPE(LOW_ALARM)		1
set ALERT_STD_TYPE(HI_ALARM)		2
set ALERT_STD_TYPE(WARN)		101
set ALERT_STD_TYPE(CRIT_WARN)		102
set ALERT_STD_TYPE(ENABLED)		201
set ALERT_STD_TYPE(INFO)		202
set ALERT_STD_TYPE(QUESTION)		203
set ALERT_STD_TYPE(DONE)		204


#############################################################################
#                                                                           #
#                       Engine Interfaces                                   #
#                                                                           #
#############################################################################

#
# Generate the Message-bus message to request a raw tone
#
proc alert_raw_tone { priority handle duration freq level } {
    global ALERT_ENGINE

    set int_freq [expr int($freq * 10)]
    set int_level [expr int($level * 10)]
    set var [list $priority $handle $duration $int_freq $int_level ]
    tclMsgbus send "{ $ALERT_ENGINE(SYSTEM_ID) $ALERT_ENGINE(MTR_RAW_TONE) 1 0xc1 } \
                  { ptr UDWORD $var }"
}

#
# Generate the Message-bus message to request a standard tone
#
proc alert_std_tone { priority handle duration type } {
    global ALERT_ENGINE
    global ALERT_STD_TYPE

    switch $type {
        STD_WARN   { set alarm_type $ALERT_STD_TYPE(WARN) }
        CRIT_WARN  { set alarm_type $ALERT_STD_TYPE(CRIT_WARN) }
        LO_ALARM   { set alarm_type $ALERT_STD_TYPE(LOW_ALARM) }
        HI_ALARM   { set alarm_type $ALERT_STD_TYPE(HI_ALARM) }
        ENABLED    { set alarm_type $ALERT_STD_TYPE(ENABLED) }
        INFO       { set alarm_type $ALERT_STD_TYPE(INFO) }
        QUESTION   { set alarm_type $ALERT_STD_TYPE(QUESTION) }
        DONE       { set alarm_type $ALERT_STD_TYPE(DONE) }
        default { return }
   }

    set var [list $priority $handle $duration $alarm_type ]
    tclMsgbus send "{ $ALERT_ENGINE(SYSTEM_ID) $ALERT_ENGINE(MTR_STD_TONE) 1 0xc1 } \
                  { ptr UDWORD $var }"
}


