#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C)1999,00      DESIGN ENGINEERING      #
# --------------------------------------------------------------------------#
# FILE        : to_pwt.tcl                                                  #
# PRODUCT     : FITS                                                        #
# AUTHOR      : Nghia Tran                                                  #
# DESCRIPTION : Interface to Power term via Message Bus                     #
#                                                                           #
#############################################################################

#
#  Power Term System Number
#

set PWT_SYSTEM_ID         6

#
#  Power Term Sub-Systems
#
set PWT_SUB_ENABLE        1                       
#set PWT_SUB_OVLDRESET     2           obsolete cmd  (sgj)      
#set PWT_SUB_DET_SLOW      3           obsolete cmd  (sgj)
#set PWT_SUB_FREQ          4           obsolete cmd  (sgj)
set PWT_SUB_READ_ENABLE   5 
set PWT_SUB_SETZERO       6
set PWT_SUB_READING       7
set PWT_SUB_DIAG          8
set PWT_SUB_TEMP          9
set PWT_SUB_CALS         10

#set PWT_SUB_ZERO         11           obsolete cmd  (sgj)
set PWT_SUB_OVLD         12
set PWT_SUB_SER_MAJOR    13
set PWT_SUB_SER_MINOR    14
set PWT_SUB_VER          15
set PWT_SUB_CALDAY       16
set PWT_SUB_CALMONTH     17
set PWT_SUB_CALYEARYY    18
set PWT_SUB_CALHOUR      19
set PWT_SUB_CALMINUTE    20

set PWT_SUB_CALAD        21
set PWT_SUB_CALPWR       22
set PWT_SUB_CALFREQ      23
#set PWT_SUB_SPLINE       24           obsolete cmd  (sgj)
#set PWT_SUB_CALREBUILD   25           obsolete cmd  (sgj)
set PWT_SUB_READCAL      26
set PWT_SUB_RANGE        27
set PWT_SUB_WRITECALS    28
set PWT_SUB_WRITESERIAL  29
set PWT_SUB_WRITEVERSION 30

set PWT_SUB_WRITETIME    31
set PWT_SUB_READEEPROM   32
set PWT_SUB_FREQPOINT    33
#set PWT_SUB_CANCELCAL    34           obsolete cmd  (sgj)
set PWT_SUB_INFOR        35
set PWT_SUB_CALINFOR     36
#set PWT_SUB_GETZERO      37           obsolete cmd  (sgj)
set PWT_SUB_TEMP_OVLD    38
set PWT_SUB_DEFAULT_CALS 39
set PWT_SUB_DISK_CALS    40

#
#  Power Term Call-back Data
#
set PWT_STATUS(rd_power)      -1
set PWT_STATUS(rd_range)      -1
set PWT_STATUS(rd_sig)        -1
set PWT_STATUS(rd_cond)       -1

set PWT_STATUS(significance)  -1
set PWT_STATUS(ad_value)      -1
set PWT_STATUS(low_zero)      -1
set PWT_STATUS(high_zero)     -1
set PWT_STATUS(zero_offset)   -1
set PWT_STATUS(ad_offset)     -1
set PWT_STATUS(raw_power)     -1
set PWT_STATUS(pw_factor)     -1
set PWT_STATUS(final_power)   -1
set PWT_STATUS(freq)          -1
set PWT_STATUS(temp1)         -1
set PWT_STATUS(temp2)         -1
set PWT_STATUS(fan_speed)     -1

set PWT_STATUS(range)         -1
set PWT_STATUS(Ant_ovld)      "Port Ok"
set PWT_STATUS(Gen_ovld)      "Port Ok"
set PWT_STATUS(major_SN)      -1
set PWT_STATUS(minor_SN)      -1
set PWT_STATUS(version)       -1
set PWT_STATUS(day)           -1
set PWT_STATUS(month)         -1
set PWT_STATUS(year)          -1
set PWT_STATUS(hour)          -1 
set PWT_STATUS(minute)        -1

set Pwt_Handler(reading)       0
set Pwt_Handler(diag)          0
set Pwt_Handler(cal)           0
set Pwt_Handler(temp)          0
set Pwt_Handler(zero)          0
set Pwt_Handler(ovld)          0
set Pwt_Handler(infor)         0
set Pwt_Handler(freq)          0

set PWT_ENGINE(task_state)     ON
set PWT_ENGINE(range)          AUTO
set PWT_ENGINE(zero_pattern)   256


####################################################################
#
#               HANDLE MESSAGE FROM VXWORKS
#
####################################################################



####################################################################
#
#   Get power readings from PT engine
#
#      power   - reading in mW
#      range   - current HW range  0 is auto, 1 is low, 2 is high
#      sig_fig - number of significant figures in reading
#      cond    - bit map of state information
#
####################################################################
proc handle_power_reading {sys sub lun flags uid power range sig_fig cond } { 

  global PWT_STATUS

  set PWT_STATUS(rd_power)      [ format "%10.7f" $power ]
  set PWT_STATUS(rd_range)      [ expr int($range)       ]
  set PWT_STATUS(rd_sig)        [ expr int($sig_fig)     ]
  set PWT_STATUS(rd_cond)       [ expr int($cond)        ]

  if { $range == 1 } {
    set PWT_STATUS(rd_range) Low
  } else {
    set PWT_STATUS(rd_range) High
  }
     
  
#puts "POWER READING: $PWT_STATUS(rd_power) W, range is $PWT_STATUS(rd_range), \
#                                              signif is $PWT_STATUS(rd_sig)"
}

####################################################################
#
#  Get diagnostic power readings from PT engine
#             AD_value    - raw PT dac value
#             low_zero    - low range delta value when user defines 'zero' reading
#             high_zero   - high range delta value when user defines 'zero' reading
#             AD_offset   - AD_value minus zero_offset value
#             raw_power   - spline value of AD_offset
#             freq_factor - frequency spline correction value
#             RF_Freq     - current receiver frequency (MHz)
#
####################################################################
proc handle_diag_reading {sys sub lun flags uid AD_value lowzero highzero AD_offset raw_power freq_factor RF_Freq } { 
  global PWT_STATUS

  set PWT_STATUS(ad_value)    [ expr int($AD_value)  ]
  set PWT_STATUS(low_zero)    [ expr int($lowzero)   ]
  set PWT_STATUS(high_zero)   [ expr int($highzero)  ]
  set PWT_STATUS(ad_offset)   [ expr int($AD_offset) ]
  set PWT_STATUS(raw_power)   [ format "%10.7f" $raw_power ]
  set PWT_STATUS(pw_factor)   [ format "%10.7f" $freq_factor ]
  set PWT_STATUS(freq)        [ format "%10.6f" $RF_Freq     ]

  if { $PWT_STATUS(rd_range) == "Low" } {
    set PWT_STATUS(zero_offset) $PWT_STATUS(low_zero)
  } else {
    set PWT_STATUS(zero_offset) $PWT_STATUS(high_zero)
  }

#  puts "handle_diag_reading: $PWT_STATUS(ad_value) $PWT_STATUS(pw_factor) \
#                             $PWT_STATUS(freq)"
}

###########################################
#
#  Get 2 temperatures and fan speed               
#
###########################################
proc handle_pwt_temp {sys sub lun flags uid temp1 temp2 fan_speed} {
  global PWT_STATUS

  set PWT_STATUS(temp1)   [ format "%4.1f" $temp1 ]
  set PWT_STATUS(temp2)   [ format "%4.1f" $temp2 ]

  set fan_speed [expr $fan_speed & 0xe]

# set PWT_STATUS(fan_speed) $fan_speed
  
  switch $fan_speed {
     14  { set PWT_STATUS(fan_speed)  STOP    }
     12  { set PWT_STATUS(fan_speed)  SLOW    }
     10  { set PWT_STATUS(fan_speed)  MEDIUM1 }
      8  { set PWT_STATUS(fan_speed)  MEDIUM2 }
      6  { set PWT_STATUS(fan_speed)  FAST    }
  }

#puts "handle_pwt_temp $PWT_STATUS(temp1) $PWT_STATUS(temp2) \
#                      $PWT_STATUS(fan_speed)"
}

####################################################
#
#  Get cals value from vxWorks
#
#                 low level       high level      frequency
#  type:             0                1               2
#  ---------------------------------------------------
#  pwr_or_freq:     power            power           factor
#  ---------------------------------------------------
#  ad:               AD               AD              N/A
#  ---------------------------------------------------
#  status:      Bit map of array-indexs cal/uncal status
#  ---------------------------------------------------
#  freq_index:  frequency reference index.  11 = 1100 MHz
#
####################################################
proc handle_pwt_cal {sys sub lun flags uid type pwr_or_freq ad_or_factor status freq_index} {
 set length [ llength $pwr_or_freq ]
puts "handle_pwt_cal $length"

 if { $type == 2 } {
    for { set i 0 } { $i < $length } { incr i } {

       set freq   [ lindex $pwr_or_freq $i ]
       set factor [ lindex $ad_or_factor  $i ]
       set cal    [ expr ( $status & (1<<$i) ) ? 1 : 0 ]

       puts "handle_pwt_cal $type $i $freq $factor $cal $freq_index"
    }
  } else {
    for { set i 0 } { $i < $length  } { incr i } {

       set pwr [ lindex $pwr_or_freq $i ]
       set ad  [ lindex $ad_or_factor  $i ]
       set cal [ expr ( $status & (1<<$i) ) ? 1 : 0 ]

       puts "handle_pwt_cal $type $i $pwr $ad $cal $freq_index"
    }    
  }
}


###########################################
#
#  Get Atn and Gen ovld status              
#
###########################################
proc handle_pwt_ovld { syst sub lun flags uid ovld } {
  global PWT_STATUS

# puts "OVLD MESSAGE is $ovld"
   
  if { $ovld & 0x1 } { 
    set PWT_STATUS(Ant_ovld) "Overload" 
  } else {
    set PWT_STATUS(Ant_ovld) "Port Ok"
  }

  if { $ovld & 0x2 } { 
    set PWT_STATUS(Gen_ovld) "Overload" 
  } else {
    set PWT_STATUS(Gen_ovld) "Port Ok"
  }
}

###########################################
#
#  Temperature overloaded             
#
###########################################
proc handle_pwt_temp_ovld {syst sub lun flags uid temp } { 
  global PWT_STATUS

#  puts "handle_pwt_temp_ovld: $temp"

  set PWT_STATUS(temp_ovld)  [ format "%4.1f" $temp ]
}

###########################################
#
#    Get calibration's information            
#
###########################################
proc handle_pwt_infor { sys sub lun flags uid majorSN  minorSN version \
                  day  month  year hour minute } {
  global PWT_STATUS
   
  set PWT_STATUS(major_SN)   $majorSN   
  set PWT_STATUS(minor_SN)   $minorSN
  set PWT_STATUS(version)    $version
  set PWT_STATUS(day)        $day
  set PWT_STATUS(month)      $month
  set PWT_STATUS(year)       $year
  set PWT_STATUS(hour)       $hour 
  set PWT_STATUS(minute)     $minute  
  
  # ASSEMBLE AND UPDATE CALIBRATION DATE
  receive_pwt_date

#  puts "$PWT_STATUS(major_SN) $PWT_STATUS(minor_SN) $PWT_STATUS(version) \
#        $PWT_STATUS(day) $PWT_STATUS(month) $PWT_STATUS(year) \
#        $PWT_STATUS(hour) $PWT_STATUS(minute)"  

}

###############################################################################
#
#  Message bus interfaces for messages being generated FROM powerterm engine
#
###############################################################################
proc create_pwt_callback { } {
  global PWT_SYSTEM_ID
  global PWT_SUB_READING
  global PWT_SUB_DIAG
  global PWT_SUB_TEMP
  global PWT_SUB_CALS
  global PWT_SUB_INFOR

  global Pwt_Handler


  set Pwt_Handler(reading) [tclMsgbus rcv handle -sys $PWT_SYSTEM_ID \
                         -sub $PWT_SUB_READING \
                         -lun 1 -cmd "handle_power_reading" ]

  set Pwt_Handler(diag)   [tclMsgbus rcv handle -sys $PWT_SYSTEM_ID \
                         -sub $PWT_SUB_DIAG \
                         -lun 1 -cmd "handle_diag_reading" ]

  set Pwt_Handler(temp) [tclMsgbus rcv handle -sys $PWT_SYSTEM_ID \
                         -sub $PWT_SUB_TEMP \
                         -lun 1 -cmd "handle_pwt_temp" ]

  set Pwt_Handler(cal) [tclMsgbus rcv handle -sys $PWT_SYSTEM_ID \
                         -sub $PWT_SUB_CALS \
                         -lun 1 -cmd "handle_pwt_cal" ]

  set Pwt_Handler(infor) [tclMsgbus rcv handle -sys $PWT_SYSTEM_ID \
                         -sub $PWT_SUB_INFOR \
                         -lun 1 -cmd "handle_pwt_infor" ]
}

#*************************************************
#       Name: create_pwt_alert_callback
# Parameters: 
#Description: creates message bus callback for overload and temperature
#*************************************************
proc create_pwt_alert_callback { } {
  global PWT_SYSTEM_ID
  global PWT_SUB_TEMP_OVLD       
  global PWT_SUB_OVLD   
  global Pwt_Handler

  set Pwt_Handler(temp_ovld) [tclMsgbus rcv handle -sys $PWT_SYSTEM_ID \
                  -sub $PWT_SUB_TEMP_OVLD -lun 1 -cmd "handle_pwt_temp_ovld" ]

  set Pwt_Handler(ovld) [tclMsgbus rcv handle -sys $PWT_SYSTEM_ID \
                  -sub $PWT_SUB_OVLD -lun 1 -cmd "handle_pwt_ovld" ]
}


##############################################################################
#
#  Message bus interfaces - removed from use
#
##############################################################################
proc destroy_pwt_callback { } {
  global Pwt_Handler

  if {$Pwt_Handler(reading) != 0 } {
    tclMsgbus $Pwt_Handler(reading) destroy
  }
  
  if {$Pwt_Handler(diag) != 0 } {
    tclMsgbus $Pwt_Handler(diag) destroy
  }

  if {$Pwt_Handler(temp) != 0 } {
    tclMsgbus $Pwt_Handler(temp) destroy
  }

  if {$Pwt_Handler(cal) != 0 } {
    tclMsgbus $Pwt_Handler(cal) destroy
  }

  if {$Pwt_Handler(infor) != 0 } {
    tclMsgbus $Pwt_Handler(infor) destroy
  }

   set Pwt_Handler(reading) 0
   set Pwt_Handler(diag)    0
   set Pwt_Handler(temp)    0
   set Pwt_Handler(cal)     0
   set Pwt_Handler(infor)   0
}


###############################################################
#
#      Command to control powerterm 
#
###############################################################

###########################################
#
#    Set reading_task  
#
#    $state ON (100 mS reading update ) or OFF
#       
#
###########################################
proc read_pwt_task { state } {
  global PWT_SYSTEM_ID
  global PWT_SUB_READ_ENABLE
  
  switch $state {
  OFF     { set value 0 }
  ON      { set value 200 }
  default { return }
  }
  message_bus_int $PWT_SYSTEM_ID $PWT_SUB_READ_ENABLE 1 $value
}

############################################
#
#   Set AD value at 0 Watts
#
############################################
proc set_pwt_zero { } {
   global PWT_SYSTEM_ID
   global PWT_SUB_SETZERO

   message_bus_int $PWT_SYSTEM_ID $PWT_SUB_SETZERO 1 0
}

###########################################
#
# Update serial number (major) into driver
# format:   XXYY        
#
###########################################
proc update_pwt_serial_major { major } {
   global PWT_SYSTEM_ID
   global PWT_SUB_SER_MAJOR
  
   message_bus_int $PWT_SYSTEM_ID $PWT_SUB_SER_MAJOR 1 $major
}

###########################################
#
# Update serial number (minor) into driver        
# format:    xxyy
#
###########################################
proc update_pwt_serial_minor { minor } {
   global PWT_SYSTEM_ID
   global PWT_SUB_SER_MINOR
  
   message_bus_int $PWT_SYSTEM_ID $PWT_SUB_SER_MINOR 1 $minor
}

###########################################
#
# Update version number into driver        
# format:  VV
#
###########################################
proc update_pwt_version { version } {
   global PWT_SYSTEM_ID
   global PWT_SUB_VER
  
   message_bus_int $PWT_SYSTEM_ID $PWT_SUB_VER 1 $version
}

###########################################
#
# Update calibration day into driver        
# format:  dd
#
###########################################
proc update_pwt_cal_day { date } {
   global PWT_SYSTEM_ID
   global PWT_SUB_CALDAY
  
   message_bus_int $PWT_SYSTEM_ID $PWT_SUB_CALDAY 1 $date
}

###########################################
#
# Update calibration month into driver        
# format:  mm
#
###########################################
proc update_pwt_cal_month { month } {
   global PWT_SYSTEM_ID
   global PWT_SUB_CALMONTH
  
   message_bus_int $PWT_SYSTEM_ID $PWT_SUB_CALMONTH 1 $month
}

###########################################
#
# Update calibration year into driver        
# format:  YYyy
#
###########################################
proc update_pwt_cal_year { year } {
   global PWT_SYSTEM_ID
   global PWT_SUB_CALYEARYY
  
   message_bus_int $PWT_SYSTEM_ID $PWT_SUB_CALYEARYY 1 $year
}

###########################################
#
# Update calibration hour into driver        
# format:  hh
#
###########################################
proc update_pwt_cal_hour { hour } {
   global PWT_SYSTEM_ID
   global PWT_SUB_CALHOUR
  
   message_bus_int $PWT_SYSTEM_ID $PWT_SUB_CALHOUR 1 $hour
}

###########################################
#
# Update calibration minute into driver        
# format:  mm
#
###########################################
proc update_pwt_cal_minute { minute } {
   global PWT_SYSTEM_ID
   global PWT_SUB_CALMINUTE
  
   message_bus_int $PWT_SYSTEM_ID $PWT_SUB_CALMINUTE 1 $minute
}

##################################################
#
#  Set Cal frequency
#index     0         1      ....
#      100.0Mhz   300.0Mhz  ....
#
##################################################
proc set_pwt_factor_init { index } {
    global PWT_SYSTEM_ID
    global PWT_SUB_FREQPOINT
    
  message_bus_int $PWT_SYSTEM_ID $PWT_SUB_FREQPOINT 1 $index 
}

###########################################
#
#             Update power        
#      low_range      high_range
#level    0              1
#index          0->10
#
###########################################
proc update_pwt_cal_power { level index data } {
   global PWT_SYSTEM_ID
   global PWT_SUB_CALPWR
   global Send_Flags
  
    set msg " { $PWT_SYSTEM_ID $PWT_SUB_CALPWR 1 $Send_Flags } { inline \
                  double $level $index $data } "
   tclMsgbus send $msg
}

###########################################
#
#             Update frequency factor        
#        frequency    factor
#level    not used      1
#index          0->13
###########################################
proc update_pwt_cal_freq { type index data } {
   global PWT_SYSTEM_ID
   global PWT_SUB_CALFREQ
   global Send_Flags
  
    set msg " { $PWT_SYSTEM_ID $PWT_SUB_CALFREQ 1 $Send_Flags } { inline \
                  double $type $index $data } "
   tclMsgbus send $msg

}

###########################################
#   Read calibration data
#          
#   This data is stored in the Engine/Driver
#    (to refresh the Engine/Driver use the
#      read_pwt_eeprom command)
#   This command is used to support the main
#      UI calibration screen.
#
#    type:  0   low range cals
#           1   high range cals
#           2   freq response cals
#  
#################################################
proc read_pwt_cal { type } {
   global PWT_SYSTEM_ID
   global PWT_SUB_READCAL

    switch $type {
    LOW   { set x 0 }
    HI    { set x 1 }
    FREQ  { set x 2 }
    default { return }
  }

   message_bus_int $PWT_SYSTEM_ID $PWT_SUB_READCAL 1 $x
}

###############################################
#
#         Set messurement range
#
###############################################
proc set_pwt_range { x } {
  global PWT_SYSTEM_ID
  global PWT_SUB_RANGE

  switch $x {
     AUTO  { set mode 0 }
     LOW   { set mode 1 }
     HI    { set mode 2 }
    default { return }
  }
  message_bus_int $PWT_SYSTEM_ID $PWT_SUB_RANGE 1 $mode
}

##################################################
#
#  Write every calibration's data into eeprom 
#
#        mode - IMAGE  simply write what is
#               stored in the image array to
#               power term EEPROM. After a read
#               from hard disk - use this..
#
#               DRIVER write driver cal struct
#               to power term EEPROM.  After PT
#               cal operation - use this..
#
##################################################
proc write_pwt_cals { mode } {
  global PWT_SYSTEM_ID
  global PWT_SUB_WRITECALS

  switch $mode {
     IMAGE    { set x 0 }
     DRIVER   { set x 1 }
     default  { return }
  }
    
  message_bus_int $PWT_SYSTEM_ID $PWT_SUB_WRITECALS 1 $x
}

##################################################
#
#  Write calibration's information into eerpom:
#  Serial, Version, date, time, year, hour, minute
#
##################################################
proc write_pwt_time { } {
    global PWT_SYSTEM_ID
    global PWT_SUB_WRITETIME
    
  message_bus_int $PWT_SYSTEM_ID $PWT_SUB_WRITETIME 1 0  
}

##################################################
#
#  Read data from eeprom
#
##################################################
proc read_pwt_eeprom { } {
    global PWT_SYSTEM_ID
    global PWT_SUB_READEEPROM
    
  message_bus_int $PWT_SYSTEM_ID $PWT_SUB_READEEPROM 1 0  
}



##################################################
#  Get calibration information: serial number,
#  version, day, month, year, hour, minute.
#  Must read EEPROM before executing this command
##################################################
proc get_pwt_cal_infor { } {
    global PWT_SYSTEM_ID
    global PWT_SUB_CALINFOR
    
    message_bus_int $PWT_SYSTEM_ID $PWT_SUB_CALINFOR 1 0
}

###########################################
# start,stop PWT engine
############################################
proc pwt_run { x } {
  global PWT_SYSTEM_ID
  global PWT_SUB_ENABLE

  switch $x {
    OFF     {   message_bus_int $PWT_SYSTEM_ID $PWT_SUB_ENABLE 1 0x00 }
    ALL     {   message_bus_int $PWT_SYSTEM_ID $PWT_SUB_ENABLE 1 0xff }
    ALERT   {   message_bus_int $PWT_SYSTEM_ID $PWT_SUB_ENABLE 1 0xa8 }
   }
}


##################################################
#
#  Write calibration's information into eerpom:
#  Serial, Version, date, time, year, hour, minute
#
##################################################
proc default_pwt_eeprom { type } {
  global PWT_SYSTEM_ID
  global PWT_SUB_DEFAULT_CALS
  global PWT_ENGINE


  switch $type {
    ZERO    { set x $PWT_ENGINE(zero_pattern) }
    CALS    { set x 0  }
    default { return }
  }
    
  message_bus_int $PWT_SYSTEM_ID $PWT_SUB_DEFAULT_CALS 1 $x
}


##################################################
#
#  save/recall power term EEPROM data
#
##################################################
proc pwt_disk_cal { state } {
  global PWT_SYSTEM_ID
  global PWT_SUB_DISK_CALS

  switch $state {
    RECALL  { set x 1  }
    SAVE    { set x 0  }
    default { return   }
  }

  message_bus_int $PWT_SYSTEM_ID $PWT_SUB_DISK_CALS 1 $x
}