#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C)1998,99      DESIGN ENGINEERING      #
# --------------------------------------------------------------------------#
# FILE        : to_rlm.tcl                                                  #
# PRODUCT     : FITS                                                        #
# AUTHOR      : Steve Johnson                                               #
# DESCRIPTION : Interface to receive level meter via Message Bus            #
#                                                                           #
#############################################################################

#
#  Rec Level System Number
#
set RLM_ENGINE(SYSTEM_ID)           25

#
#  Rec Level Sub-Systems
#
set RLM_ENGINE(MTR_ENABLE)           1
set RLM_ENGINE(MTR_READING)          2
set RLM_ENGINE(MTR_INFO)             3
set RLM_ENGINE(MTR_AVERAGE)          4
set RLM_ENGINE(MTR_TRIG)             5
set RLM_ENGINE(MTR_MATH)             6
set RLM_ENGINE(MTR_PHRST)            7
set RLM_ENGINE(MTR_ALARM)            8
set RLM_ENGINE(MTR_PEAKS)            9
set RLM_ENGINE(MTR_ULVAL)           10
set RLM_ENGINE(MTR_LLVAL)           11
set RLM_ENGINE(MTR_GATE)            12
set RLM_ENGINE(MTR_START_STOP)      13

set RLM_ENGINE(RLM_SUB_UPDATE_RATE) 128
set RLM_ENGINE(RLM_SUB_SET_REF)     129
set RLM_ENGINE(RLM_SUB_SET_CAL)     130
set RLM_ENGINE(MTR_STATUS)          131
set RLM_ENGINE(MTR_CABLE_LOSS)      132


#
#  Rec Level Cal Information
#
set RLM_ENGINE(num_cals)            14
set RLM_ENGINE(start_lvl)          -10
set RLM_ENGINE(step_lvl)            -5

font create rlm_font -family helvetica -size 10

#
#  Rec Level Call-back Data
#
set RLM_STATUS(reading)             -1
set RLM_STATUS(alarm)               -1
set RLM_STATUS(state)               "SW Init"
set RLM_STATUS(precision)           -1
set RLM_STATUS(status)              "SW Init"
set RLM_STATUS(statusindex)          0


#
#  Dvm Helpful globals
#
set RLM_ENGINE(Handler_reading)      0
set RLM_ENGINE(Handler_alarm)        0
set RLM_ENGINE(Handler_info)         0
set RLM_ENGINE(Handler_status)       0


#
#  Message bus callback - fill in supplied structure
#
#
proc handle_rlm_reading {sys sub lun flags uid reading precision state } {
  global RLM_STATUS
 
#puts "RLM reading: $reading $precision $state"

  set RLM_STATUS(reading)     [ format "%10.2f" $reading ]
  set RLM_STATUS(precision)   $precision

  if { $state == 0 } {
    set RLM_STATUS(state)  "Calibrated Level, dBm"
  } elseif { $state == -1 } {
    set RLM_STATUS(state)  "RF HW Chg!, Reading dBm"
  } elseif { $state == -2 } {
    set RLM_STATUS(state)  "Raw Readings, dB"
  } elseif { $state == -3 } {
    set RLM_STATUS(state)  "No Spline, Raw Level (dB)"
  } elseif { $state == -10 } {
    set RLM_STATUS(state)  "Level! Input Too Low"
  } elseif { $state == -11 } {
    set RLM_STATUS(state)  "Level! Input Too High"
  } else {
    set RLM_STATUS(state)  "Unknown Message"
  }
}


#
#  Message bus callback - fill in supplied structure
#
proc handle_rlm_alarm {sys sub lun flags uid alarm } {
  global RLM_STATUS

puts "RLM Alarm:  $alarm"

  set RLM_STATUS(alarm)  $alarm
}

#
#  Message bus callback - fill in supplied structure
#
proc handle_rlm_info {sys sub lun flags uid dvr eng } {
    global     METERDATA_SCRN

    # Ignore messages if the STATUS flag is not set
    if { $flags & 0x04 } {
        set METERDATA_SCRN(Rlm_Drv)   $dvr
        set METERDATA_SCRN(Rlm_Eng)   $eng
    }
}

#
#  Status Bits
#
#   11 10 9 8   8 7 6 5   4 3 2 1
#    |  | | |   | | | |   | | | |
#    |  | | |   | | | |   | | | +- Step Number        0 - 255
#    |  | | |   | | | |   | | +--- Step Number
#    |  | | |   | | | |   | +----- Step Number
#    |  | | |   | | | |   +------- Step Number
#    |  | | |   | | | +----------- Step Number
#    |  | | |   | | +------------- Step Number
#    |  | | |   | +--------------- Step Number
#    |  | | |   +----------------- Step Number
#    |  | | |
#    |  | | +---------------------Cal     1 Cal Task Armed, 2 Task Running 
#    |  | +-----------------------Cal     3 Cal Done      , 4 Cal Error
#    |  +-------------------------Cal     0 Unknown
#    +----------------------------
#
#
#
proc put_rlm_rfgen_back { } {
  global RFGEN_ENGINE
  set_rfgen_rflevel $RFGEN_ENGINE(RF_Level)
}

proc handle_rlm_status {sys sub lun flags uid status } {
  global RLM_STATUS

#puts "handle_rlm_status:  $status"

  set state [ expr { $status & 0xf00 } ]
  set state [ expr { $state >> 8 } ]

  if { $state == 1 } {
    set RLM_STATUS(status)  "Cal Waiting"
  } elseif { $state == 2 } {
    set RLM_STATUS(status)  "Cal Running"
  } elseif { $state == 5 } {
    set RLM_STATUS(status)  "Cal Done"
    put_rlm_rfgen_back
    set_rlm_raw_mode OFF
  } elseif { $state == 6 } {
    set RLM_STATUS(status)  "Cal Error"
    put_rlm_rfgen_back
  } elseif { $state == 7 } {
    set RLM_STATUS(status)  "Err! (Cable?)"
    put_rlm_rfgen_back
  } elseif { $state == 8 } {
    set RLM_STATUS(status)  "Aborted Cal"
    put_rlm_rfgen_back
  } else {
    set RLM_STATUS(status)  "Unknown Status"
    put_rlm_rfgen_back
  }

  set RLM_STATUS(statusindex)  [ expr { $status & 0xff } ]
}


#
#  Message bus interfaces for messages being generated FROM rcvr engine
#
proc create_rlm_callback { } {
  global RLM_ENGINE

  set RLM_ENGINE(Handler_reading)  [tclMsgbus rcv handle \
                                   -sys $RLM_ENGINE(SYSTEM_ID) \
                                   -sub $RLM_ENGINE(MTR_READING) \
                                   -lun 1 -cmd "handle_rlm_reading" ]

  set RLM_ENGINE(Handler_alarm)    [tclMsgbus rcv handle \
                                   -sys $RLM_ENGINE(SYSTEM_ID) \
                                   -sub $RLM_ENGINE(MTR_ALARM) \
                                   -lun 1 -cmd "handle_rlm_alarm" ]

  set RLM_ENGINE(Handler_info)     [tclMsgbus rcv handle \
                                   -sys $RLM_ENGINE(SYSTEM_ID) \
                                   -sub $RLM_ENGINE(MTR_INFO) \
                                   -lun 1 -cmd "handle_rlm_info" ]

  set RLM_ENGINE(Handler_status)   [tclMsgbus rcv handle \
                                   -sys $RLM_ENGINE(SYSTEM_ID) \
                                   -sub $RLM_ENGINE(MTR_STATUS) \
                                   -lun 1 -cmd "handle_rlm_status" ]
}

#
#  Message bus interfaces - removed from use
#
proc destroy_rlm_callback { } {
  global RLM_ENGINE

  if {$RLM_ENGINE(Handler_reading) != 0 } {
    tclMsgbus $RLM_ENGINE(Handler_reading) destroy
  }

  if {$RLM_ENGINE(Handler_alarm) != 0 } {
    tclMsgbus $RLM_ENGINE(Handler_alarm) destroy
  }

  if {$RLM_ENGINE(Handler_info) != 0 } {
    tclMsgbus $RLM_ENGINE(Handler_info) destroy
  }

  if {$RLM_ENGINE(Handler_status) != 0 } {
    tclMsgbus $RLM_ENGINE(Handler_status) destroy
  }

  set RLM_ENGINE(Handler_reading)    0
  set RLM_ENGINE(Handler_alarm)      0
  set RLM_ENGINE(Handler_info)       0
  set RLM_ENGINE(Handler_status)     0
}


#############################################################################
#                                                                           #
#                       Engine Interfaces                                   #
#                                                                           #
#############################################################################



#
# set speed of readings (default is 200 mS)
#
#    $rate is speed readings are sent from the DSP
#    1  is 10 mS per reading
#    10 is 100 mS per reading 
#
proc rlm_set_update_rate { rate } {
  global RLM_ENGINE

  message_bus_int $RLM_ENGINE(SYSTEM_ID) $RLM_ENGINE(RLM_SUB_UPDATE_RATE) 1 $rate
}

#
# Request access of Engine information:
#
proc get_rlm_info { } {
  global RLM_ENGINE
  global MSG_BUS_FLAGS
 
  set msg " { $RLM_ENGINE(SYSTEM_ID) $RLM_ENGINE(MTR_INFO) 1 \
              $MSG_BUS_FLAGS(QUERY) } { inline udword 1}  "

  tclMsgbus send $msg
}

#
# set_rlm_cals - set calibration information
#
#
proc set_rlm_cals { type index cal } {
  global RLM_ENGINE
  global Send_Flags

  set msg " { $RLM_ENGINE(SYSTEM_ID) $RLM_ENGINE(RLM_SUB_SET_CAL) \
              1 $Send_Flags } \
            { inline sdword $type } \
            { inline sdword $index } \
            { inline double $cal } "
#puts $msg
  tclMsgbus send $msg
}

#
# abort_rl_calibration
#
#
proc abort_rl_calibration { } {
  set_rlm_cals 5 0 0
}

#
# set_rl_num_cal_steps
#
#
proc set_rl_num_cal_steps { num } {
  set_rlm_cals 1 $num 0
}

#
# set_rl_cal_step_size
#
#     num   - number of cal steps  ( 2 to 30 )
#     step  - RF Gen level step size ( dB )
#
proc set_rl_cal_step_size { num step } {
  set_rlm_cals 2 $num $step
}

#
# start_rl_calibration
#
#     dBm - starting RF gen rf level
#
#
proc start_rl_calibration { dBm } {
  set_rlm_cals 3 0 $dBm
}


#
# set_rl_start_stop
#
#
proc set_rl_start_stop { state } {
  set_rlm_cals 100 $state 0
}


#
# set_rlm_raw_mode - 0 use splined values, 1 use uncalibrated values
#
#
proc set_rlm_raw_mode { state } {
  global RLMMTR_SCRN

  switch $state {
    ON      { set_rlm_cals 4 1 0  }
    OFF     { set_rlm_cals 4 0 0  }
    default { return }
  }

  set RLMMTR_SCRN(RawReading) $state
}


#
# set cable loss - cable used for calibration (default is 0.0 dB)
#
#
proc rlm_set_cable_loss { dB } {
  global RLM_ENGINE

  message_bus_double $RLM_ENGINE(SYSTEM_ID) $RLM_ENGINE(MTR_CABLE_LOSS) 1 $dB
}



proc do_rlm_cable_loss { } {
  global RLMMTR_SCRN

  rlm_set_cable_loss $RLMMTR_SCRN(cableloss)
}


#
# start,stop RLM readings
#
proc rlm_run { x } {
  global RLM_ENGINE
  global RLMMTR_SCRN

  switch $x {
    ON  { set_rl_start_stop 1 }
    OFF { set_rl_start_stop 0 }
    default { return }
  }

  set RLMMTR_SCRN(Enable) $x
}

#
# start,stop RLM readings
#
proc rlm_enable { x } {
  global RLM_ENGINE

  switch $x {
    ALL { message_bus_int $RLM_ENGINE(SYSTEM_ID) $RLM_ENGINE(MTR_ENABLE) 1 3 }
    OFF { message_bus_int $RLM_ENGINE(SYSTEM_ID) $RLM_ENGINE(MTR_ENABLE) 1 1 }
    default { return }
  }
}


#
# do_rl_calibration - set calibration information
#
#
proc do_rl_calibration { } {
  global RLM_ENGINE

  set_rlm_raw_mode ON
  rlm_run ON
  set_rl_num_cal_steps $RLM_ENGINE(num_cals)
  set_rl_cal_step_size $RLM_ENGINE(num_cals) $RLM_ENGINE(step_lvl)
  start_rl_calibration $RLM_ENGINE(start_lvl)
}