#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C)1998,99,2000   DESIGN ENGINEERING    #
# --------------------------------------------------------------------------#
# FILE        : p25_decode.tcl                                              #
# PRODUCT     : FITS                                                        #
# AUTHOR      : Steve White                                                 #
# DESCRIPTION : P-25 Decode Info Window                                     #
#                                                                           #
#############################################################################

#These are the actual Displayed decoded values
set P25DEC_INFO(count) "---"
set P25DEC_INFO(MFID) "---"
set P25DEC_INFO(TGID) "---"
set P25DEC_INFO(ALGID) "---"
set P25DEC_INFO(SID)   "---"
set P25DEC_INFO(NAC) "---"
set P25DEC_INFO(DID) "---"
set P25DEC_INFO(KID) "---"
set P25DEC_INFO(STATUS) "---"
set P25DEC_INFO(LCO) "--------"
set P25DEC_INFO(P) "---"
set P25DEC_INFO(SF) "---"
set P25DEC_INFO(EMG) "---"
set P25DEC_INFO(LSD) "--------"
set P25DEC_INFO(encrypt) "---"
set P25DEC_INFO(DUID) "--------"
set P25DEC_INFO(MI) "------------"

set p25state 0
set p25dec 0
set LDU_COUNT(1) 0
set LDU_COUNT(2) 0

#
#   Reset P25 Decoded display values
#
proc reset_p25_values { } {
global P25DEC_INFO
global p25state

   if { $p25state == 0 } {
     set P25DEC_INFO(count) "---"
   } else {
     set P25DEC_INFO(count) 0
     request_encrypt
   }
   set P25DEC_INFO(MFID) "---"
   set P25DEC_INFO(TGID) "---"
   set P25DEC_INFO(ALGID) "---"
   set P25DEC_INFO(SID)   "---"
   set P25DEC_INFO(NAC) "---"
   set P25DEC_INFO(DID) "---"
   set P25DEC_INFO(KID) "---"
   set P25DEC_INFO(STATUS) "---"
   set P25DEC_INFO(LCO) "--------"
   set P25DEC_INFO(P) "---"
   set P25DEC_INFO(SF) "---"
   set P25DEC_INFO(EMG) "---"
   set P25DEC_INFO(LSD) "--------"
   set P25DEC_INFO(encrypt) "---"
   set P25DEC_INFO(DUID) "--------"
   set P25DEC_INFO(MI) "------------"
}

#
#   Start the P-25 Decode Info
#
proc start_p25_decode { } {
global p25dec
global p25state
global P25DEC_INFO
global LDU_COUNT

   set p25state 1
   reset_p25_values
   $p25dec.f8.b1 configure -text STOP -command stop_p25_decode
   p25_info_dump 1
   $p25dec.f8.c1 create oval 3 3 12 12 -fill green
   set LDU_COUNT(1) 0
   set LDU_COUNT(2) 0
}

#
#   Stop the P-25 Decode Info
#
proc stop_p25_decode { } {
global p25dec
global p25state

   p25_info_dump 0
   set p25state 0
   $p25dec.f8.b1 configure -text START -command start_p25_decode
#   $p25dec.f8.c1 configure -bg red
   $p25dec.f8.c1 create oval 3 3 12 12 -fill red
}

#
#   Build the P-25 Decode Info Window (menu)
#    ENTRY: name - Base window name to build into
#
proc build_P25_decode_screen { name } {
  global p25dec 
  global p25state
  global DemoColor
  global P25DEC_INFO

  set p25dec $name

  frame $p25dec.count
  frame $p25dec.f1
  frame $p25dec.f2
  frame $p25dec.f3
  frame $p25dec.f4
  frame $p25dec.f5
  frame $p25dec.f6
  frame $p25dec.f7
  frame $p25dec.f8
  frame $p25dec.f9
  frame $p25dec.f10

  label $p25dec.label -text "P25 DECODED INFO" -width 32

label $p25dec.count.label -text "SuperFrame Count:"
label $p25dec.count.value -textvariable P25DEC_INFO(count) -width 6 -fg blue

label $p25dec.f1.mfidlab  -text " MFID:" -width 5
label $p25dec.f1.mfidval  -textvariable P25DEC_INFO(MFID) -width 4 -fg blue
label $p25dec.f1.tgidlab  -text "TGID:"
label $p25dec.f1.tgidval  -textvariable P25DEC_INFO(TGID) -width 6 -fg blue

label $p25dec.f2.algidlab  -text "ALGID:" -width 5
label $p25dec.f2.algidval -textvariable P25DEC_INFO(ALGID) -width 4 -fg blue
label $p25dec.f2.sidlab  -text "SID:"
label $p25dec.f2.sidval  -textvariable P25DEC_INFO(SID) -width 6 -fg blue

label $p25dec.f3.naclab  -text "  NAC:" -width 5
label $p25dec.f3.nacval -textvariable P25DEC_INFO(NAC) -width 4 -fg blue
label $p25dec.f3.didlab  -text "DID:"
label $p25dec.f3.didval -textvariable P25DEC_INFO(DID) -width 6 -fg blue

label $p25dec.f4.kidlab  -text "  KID:" -width 5
label $p25dec.f4.kidval -textvariable P25DEC_INFO(KID) -width 4 -fg blue
label $p25dec.f4.statlab  -text "Status:"
label $p25dec.f4.statval -textvariable P25DEC_INFO(STATUS) -width 6 -fg blue

label $p25dec.f5.plab  -text "P:" -width 2
label $p25dec.f5.pval -textvariable P25DEC_INFO(P) -width 2 -fg blue
label $p25dec.f5.dummy -text " " -width 5
label $p25dec.f5.sflab  -text "SF:"
label $p25dec.f5.sfval -textvariable P25DEC_INFO(SF) -width 2 -fg blue
label $p25dec.f5.emglab  -text "EMG:"
label $p25dec.f5.emgval -textvariable P25DEC_INFO(EMG) -width 2 -fg blue

label $p25dec.f6.lcolab  -text "  LCO:" -width 5
label $p25dec.f6.lcoval -textvariable P25DEC_INFO(LCO) -justify left -fg blue

label $p25dec.f9.duidlab  -text " DUID:" -width 5
label $p25dec.f9.duidval -textvariable P25DEC_INFO(DUID) -justify left -fg blue

label $p25dec.f10.milab  -text "    MI:" -width 5
label $p25dec.f10.mival -textvariable P25DEC_INFO(MI) -justify left -fg blue

label $p25dec.f7.lsdlab  -text "  LSD:" -width 5
label $p25dec.f7.lsdval -textvariable P25DEC_INFO(LSD) -justify left -fg blue
label $p25dec.f7.enclab -text "Encrypt:"
label $p25dec.f7.encval -textvariable P25DEC_INFO(encrypt) -width 6 -fg blue

if { $p25state == 0 } {
   set color red
   set lab START
   set cmd "start_p25_decode"
} else {
   set color green
   set lab STOP
   set cmd "stop_p25_decode"
}

button $p25dec.f8.b1 -text $lab -width 6 -command $cmd
button $p25dec.f8.b2 -text RESET -command reset_p25_values
# Run indicator Button (Red/Green)
canvas $p25dec.f8.c1 -width 16 -height 16
$p25dec.f8.c1 create oval 3 3 12 12 -fill $color

#
#  Pack controls (labels/values)
#
pack $p25dec.count.label    -side left -fill both -expand true
pack $p25dec.count.value    -side left -fill both -expand true

pack $p25dec.f1.mfidlab  -side left 
pack $p25dec.f1.mfidval  -side left 
pack $p25dec.f1.tgidval  -side right
pack $p25dec.f1.tgidlab  -side right

pack $p25dec.f2.algidlab  -side left -fill both 
pack $p25dec.f2.algidval  -side left -fill both 
pack $p25dec.f2.sidval  -side right
pack $p25dec.f2.sidlab  -side right

pack $p25dec.f3.naclab  -side left -fill both 
pack $p25dec.f3.nacval  -side left -fill both 
pack $p25dec.f3.didval  -side right
pack $p25dec.f3.didlab  -side right

pack $p25dec.f4.kidlab  -side left -fill both 
pack $p25dec.f4.kidval  -side left -fill both 
pack $p25dec.f4.statval  -side right
pack $p25dec.f4.statlab  -side right

pack $p25dec.f5.plab  -side left -fill both 
pack $p25dec.f5.pval -side left -fill both 
pack $p25dec.f5.dummy -side left -fill both
pack $p25dec.f5.sflab -side left -fill both
pack $p25dec.f5.sfval -side left -fill both
pack $p25dec.f5.emgval -side right -fill both -padx 6
pack $p25dec.f5.emglab -side right -fill both

pack $p25dec.f6.lcolab -side left -fill both 
pack $p25dec.f6.lcoval -side left 

pack $p25dec.f9.duidlab -side left -fill both 
pack $p25dec.f9.duidval -side left 

pack $p25dec.f10.milab -side left -fill both 
pack $p25dec.f10.mival -side left 

pack $p25dec.f7.lsdlab -side left -fill both 
pack $p25dec.f7.lsdval -side left -fill both 
pack $p25dec.f7.encval  -side right
pack $p25dec.f7.enclab  -side right

pack $p25dec.f8.b1 -side left -fill both -padx 8
pack $p25dec.f8.b2 -side right -fill both -padx 8
pack $p25dec.f8.c1 -side left -padx 2

#
#  Pack screen (frames)
#
pack $p25dec.label -side top -fill both -pady 5
pack $p25dec.count -side top -anchor w
pack $p25dec.f1 -side top -fill x -padx 2
pack $p25dec.f2 -side top -fill x -padx 2
pack $p25dec.f3 -side top -fill x -padx 2
pack $p25dec.f4 -side top -fill x -padx 2
pack $p25dec.f7 -side top -fill x -padx 2
pack $p25dec.f10 -side top -fill x -padx 2
pack $p25dec.f6 -side top -fill x -padx 2
pack $p25dec.f9 -side top -fill x -padx 2
pack $p25dec.f5 -side top -fill x -padx 2
pack $p25dec.f8 -side top -fill x -padx 2 -pady 4
}


