#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C)1998,99      DESIGN ENGINEERING      #
# --------------------------------------------------------------------------#
# FILE        : to_p25.tcl                                                  #
# PRODUCT     : FITS                                                        #
# AUTHOR      : Steve White                                                 #
# DESCRIPTION : Interface to P-25 Decode via Message Bus                    #
#                                                                           #
#############################################################################

#
#  P-25 Decode system Number
#
set P25DEC_ENGINE(SYSTEM_ID)         19

#
#  P-25 Decode Sub-Systems
#
set P25DEC_ENGINE(SAMPLE_RATE)          1
set P25DEC_ENGINE(INFO_DUMP)            2
set P25DEC_ENGINE(HDR_INFO)             3
set P25DEC_ENGINE(ENCRYPT_REQ)          4
set P25DEC_ENGINE(ENCRYPT_STATE)        5
set P25DEC_ENGINE(LCW_INFO)             6
set P25DEC_ENGINE(NID_INFO)             7
set P25DEC_ENGINE(LSD_INFO)             8

#
#  P-25 Decode Call-back Handles
#
set P25DEC_HANDLE(HDR_INFO)  0
set P25DEC_HANDLE(LCW_INFO)  0
set P25DEC_HANDLE(NID_INFO)  0
set P25DEC_HANDLE(LSD_INFO)  0
set P25DEC_HANDLE(ENCRYPT_STATE)  0

# LCO String array values
set LCO_STRING(0) "(0)Group Voice Chan User"
set LCO_STRING(1) "(1)(Undefined)"
set LCO_STRING(2) "(2)Group Voice Chan Update"
set LCO_STRING(3) "(3)Unit-Unit Voice Chan User"

# DUID String array values
set DUID_STRING(0) "(0)Header"
set DUID_STRING(3) "(3)Terminator (w/o Link)"
set DUID_STRING(5) "(5)LDU1"
set DUID_STRING(10) "(0A)LDU2"
set DUID_STRING(15) "(0F)Terminator w/Link"

#############################################################################
#                                                                           #
#                       Message Bus Interfaces                              #
#                                                                           #
#############################################################################

#
#  Message bus callback handlers 
#
#
# P-25 LCW Dump info handler.  Called by OS when a p25 LCW info
# msg is present on the Message Bus. Must be previously registered w/Tcl.
#
proc handle_lcw { sys sub lun flags uid pay } {
global P25DEC_INFO

#1st, bump the Super-frame count
   incr P25DEC_INFO(count)

   set LCO [lindex $pay 2]

   switch $LCO {
       3 {
          set DID [lindex $pay 5]
          set DID [expr $DID << 16]
          set DID [expr $DID + [expr [lindex $pay 6] << 8]]
          set DID [expr $DID + [lindex $pay 7]]
          set DID [format "%06X" $DID]
          set TGID "---"
         } 
       0 {
           set DID "---"
           set TGID [lindex $pay 6]
           set TGID [expr $TGID << 8]
           set TGID [expr $TGID + [lindex $pay 7]]
           set TGID [format "%04X" $TGID]
         }
      default {
               return
              }
   }

   set SID [lindex $pay 8]
   set SID [expr $SID << 16]
   set SID [expr $SID + [expr [lindex $pay 9] << 8]]
   set SID [expr $SID + [lindex $pay 10]]
   set SID [format "%04X" $SID]
   set EMG [expr [lindex $pay 4] >> 7]
   set P [lindex $pay 0]
   set SF [lindex $pay 1]
   set MFID [format "%02X" [lindex $pay 3]]
   set LCO [parse_lco [lindex $pay 2]]
 
   set P25DEC_INFO(MFID) $MFID
   set P25DEC_INFO(TGID) $TGID
   set P25DEC_INFO(P) $P
   set P25DEC_INFO(DID) $DID
   set P25DEC_INFO(SF) $SF
   set P25DEC_INFO(EMG) $EMG
   set P25DEC_INFO(SID) $SID
   set P25DEC_INFO(LCO) $LCO
}

#
# P-25 NID Dump info handler.  Called by OS when a p25 NID info
# msg is present on the Message Bus. Must be registered w/Tcl.
proc handle_nid { sys sub lun flags uid pay1 pay2 } {
global P25DEC_INFO

   set duid [parse_duid [lindex $pay1 0]]
   set nac [lindex $pay2 0]

   set P25DEC_INFO(NAC) [format "%03X" $nac]
   if { $duid != "" } {
     set P25DEC_INFO(DUID) $duid
   }
}

#
# P-25 LSD Dump info handler.  Called by OS when a p25 LSD info
# msg is present on the Message Bus. Must be registered w/Tcl.
proc handle_lsd { sys sub lun flags uid pay } {
global P25DEC_INFO

   set S1 [lindex $pay 1]
   set S2 [lindex $pay 2]
   set S3 [lindex $pay 3]

   set P25DEC_INFO(LSD) [format "%08X" [lindex $pay 0]]
   set P25DEC_INFO(STATUS) [chk_status $S1]
   set P25DEC_INFO(STATUS) [chk_status $S2]
   set P25DEC_INFO(STATUS) [chk_status $S3]
}

#
# P-25 Header Dump info handler.  Called by OS when a p25-header dump info
# cmd is present on the Message Bus. Must be registered w/Tcl.
proc handle_hdr { sys sub lun flags uid pay } {
global P25DEC_INFO

   set MI1 [format "%06X" [lindex $pay 0]]
   set MI2 [format "%06X" [lindex $pay 1]]
   set MI3 [format "%06X" [lindex $pay 2]]
   set MFID [format "%02X" [lindex $pay 3]]
   set ALGID [format "%02X" [lindex $pay 4]]
   set KID [format "%04X" [lindex $pay 5]]
   set TGID [format "%04X" [lindex $pay 6]]
   
   set P25DEC_INFO(MFID) $MFID
   set P25DEC_INFO(ALGID) $ALGID
   set P25DEC_INFO(TGID) $TGID
   set P25DEC_INFO(KID) $KID
   set P25DEC_INFO(MI) [concat $MI1$MI2$MI3]
}

#
# P-25 Encryption state update handler.  Called by OS when a p25 encryption state
# status msg is present on the Message Bus. Must be registered w/Tcl.
proc handle_enc { sys sub lun flags uid pay } {
global P25DEC_INFO
global p25state

   if { [lindex $pay 0] == 0} {
     set state OFF
   } else {
     set state ON
   }
   if { $p25state > 0 } {
     set P25DEC_INFO(encrypt) $state
   }
}

#
#  Register the handlers for messages
#  coming back from DSP's off the Msg Bus.
#
proc create_p25dec_callback { } {
  global P25DEC_ENGINE
  global P25DEC_HANDLE

  set P25DEC_HANDLE(LCW_INFO) [tclMsgbus rcv handle -sys $P25DEC_ENGINE(SYSTEM_ID) \
         -sub $P25DEC_ENGINE(LCW_INFO) -lun 1 -cmd "handle_lcw" ]
  set P25DEC_HANDLE(NID_INFO) [tclMsgbus rcv handle -sys $P25DEC_ENGINE(SYSTEM_ID) \
         -sub $P25DEC_ENGINE(NID_INFO) -lun 1 -cmd "handle_nid" ]
  set P25DEC_HANDLE(LSD_INFO) [tclMsgbus rcv handle -sys $P25DEC_ENGINE(SYSTEM_ID) \
        -sub $P25DEC_ENGINE(LSD_INFO) -lun 1 -cmd "handle_lsd"]
  set P25DEC_HANDLE(HDR_INFO) [tclMsgbus rcv handle -sys $P25DEC_ENGINE(SYSTEM_ID) \
        -sub $P25DEC_ENGINE(HDR_INFO) -lun 1 -cmd "handle_hdr"]
  set P25DEC_HANDLE(ENCRYPT_STATE) [tclMsgbus rcv handle -sys $P25DEC_ENGINE(SYSTEM_ID) \
        -sub $P25DEC_ENGINE(ENCRYPT_STATE) -lun 1 -cmd "handle_enc"]
}


#
#  Message bus interfaces - Un-register the P-25 decode handler's
#
proc destroy_p25_callback { } {
  global P25DEC_HANDLE

  if {$P25DEC_HANDLE(LCW_INFO) != 0 } {
    tclMsgbus $P25DEC_HANDLE(LCW_INFO) destroy
  }

  if {$P25DEC_HANDLE(NID_INFO) != 0 } {
    tclMsgbus $P25DEC_HANDLE(NID_INFO) destroy
  }

  if {$P25DEC_HANDLE(LSD_INFO) != 0 } {
    tclMsgbus $P25DEC_HANDLE(LSD_INFO) destroy
  }

  if {$P25DEC_HANDLE(HDR_INFO) != 0 } {
    tclMsgbus $P25DEC_HANDLE(HDR_INFO) destroy
  }

  if {$P25DEC_HANDLE(ENCRYPT_STATE) != 0 } {
    tclMsgbus $P25DEC_HANDLE(ENCRYPT_STATE) destroy
  }

  set P25DEC_HANDLE(LCW_INFO) 0
  set P25DEC_HANDLE(NID_INFO) 0
  set P25DEC_HANDLE(LSD_INFO) 0
  set P25DEC_HANDLE(HDR_INFO) 0
  set P25DEC_HANDLE(ENCRYPT_STATE) 0
}

#
# Parse the provided LCO Code, and return the corresponding string
#   ENTRY: code - LCO code
proc parse_lco { code } {
global LCO_STRING

   if { $code < 4 } {
      set value $LCO_STRING($code)
   } else {
      set value "Unknown"
   }
   return $value
}

#
# Parse the provided DUID Code, and return the corresponding string or NULL
# if not ready to update DUID yet.
#   ENTRY: code - DUID code
proc parse_duid { code } {
global DUID_STRING
global LDU_COUNT

   set str ""
   switch $code {
       0   { set str $DUID_STRING($code)}
       3   { set str $DUID_STRING($code)} 
       5   {
             incr LDU_COUNT(1)
             if { $LDU_COUNT(2) > 0 } {
                set str [concat $DUID_STRING(5) $DUID_STRING(10)]
                set LDU_COUNT(2) 0
                set LDU_COUNT(1) 0
             }
             if { $LDU_COUNT(1) > 1 } {
                set str $DUID_STRING($code)
                set LDU_COUNT(1) 0
             }
           }
       10  { 
             incr LDU_COUNT(2)
             if { $LDU_COUNT(1) > 0 } {
                set str [concat $DUID_STRING(5) $DUID_STRING(10)]
                set LDU_COUNT(2) 0
                set LDU_COUNT(1) 0
             }
             if { $LDU_COUNT(2) > 1 } {
                set str $DUID_STRING($code)
                set LDU_COUNT(2) 0
             }
           }
       15  { set str $DUID_STRING($code)}
       default { set str " ? ? ?" }
   }
   return $str
}

#
# Look at provided status symbols (32-bits) and if compare with the 
# current value of the Status-symbol INFO field, and if different,
# return the new status symbol (2-bits).
#   ENTRY: bits - 32-bit status
proc chk_status { bits } {
global P25DEC_INFO
   
   for { set i 0 } { $i < 16 } { incr i} {
     set val [expr {$bits & 3}]
     if { $val != $P25DEC_INFO(STATUS) } {
        return $val
     }
     set bits [expr {$bits >> 2}]
   }
   return $P25DEC_INFO(STATUS)
}


#############################################################################
#                                                                           #
#                       Engine Interfaces                                   #
#                                                                           #
#############################################################################


#
#  Request current Encryption State from the DSP
#
proc request_encrypt { } {
  global P25DEC_ENGINE

  set msg " { $P25DEC_ENGINE(SYSTEM_ID) $P25DEC_ENGINE(ENCRYPT_REQ) 1 2 } \
            { inline sdword 0 } "
  tclMsgbus send $msg
}

#
# Tell the DSP's to start Dumping P25 Decoded Info
#   ENTRY: state - 1=Start Dumping Info, 0=Stop Dumping Info
#
proc p25_info_dump { state } {
  global P25DEC_ENGINE

   message_bus_int $P25DEC_ENGINE(SYSTEM_ID) $P25DEC_ENGINE(INFO_DUMP) 1 $state
}

