#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C)1998,99      DESIGN ENGINEERING      #
# --------------------------------------------------------------------------#
# FILE        : screen.tcl                                                  #
# PRODUCT     : FITS                                                        #
# AUTHOR      : Steve Johnson                                               #
# DESCRIPTION : receiver demo screen                                        #
#                                                                           #
#############################################################################

#
# Display state variables
#
set RcvrState(input) ANT
set RcvrState(ifbw)  30KHZ

set RcvrState(range) NARROW
set RcvrState(lock)  LOCK
set RcvrState(agc)   SLOW
set RcvrState(demod) DEMOD_P25

set RcvrState(AgcSample)   300
set RcvrState(AgcLL)     -10.5
set RcvrState(AgcUL)      -9.5
set RcvrState(AgcFilter)    3
set RcvrState(AgcState)      1

set RCVR_STATUS(hop_freq1)   99.9
set RCVR_STATUS(hop_freq2)  100.1
set RCVR_STATUS(hop_delay)  500
set RCVR_STATUS(hop_bitmap) 2


#
#  allow a manual reset of all hardware
#
proc reset_rcvr_hw { } {
  # set defaults
  set_rcvr_default_ifatten 
  Receiver freq 500 MHz
  Receiver atten 0
  Receiver rfin ANT

  set_rcvr_ifbw 30KHZ 
  Receiver ifgain -14.0

  set_rcvr_enable LOCK_YIG_MSG
  set_rcvr_range NARROW
  set_rcvr_lock LOCK
  set_rcvr_hwagc SLOW
  set_rcvr_demod DEMOD_P25
  set_rcvr_agc_mode ON

  trace_rcvr_sw_agc_period
  trace_rcvr_sw_agc_ll
  trace_rcvr_sw_agc_ul
  trace_rcvr_sw_agc_filter 
  trace_rcvr_sw_agc_state
}

#
# doRcvrFreq
#
proc doRcvrFreq { tg } {
  global RCVR_ENGINE

  label $tg.check -textvariable RCVR_ENGINE(OutGoingFreq)  -font rcvr_font 
  label $tg.msg   -text  "Adj Rx Freq (MHz)"  -font rcvr_font 
  entry $tg.entry -textvariable  RCVR_ENGINE(RF_Freq) -width 10 -bg white

  bind $tg.entry <Return> {Receiver freq $RCVR_ENGINE(RF_Freq)}

  pack $tg.msg    -side top  -fill both -expand true
  pack $tg.check  -side left -fill both -expand true
  pack $tg.entry  -side left -fill both -expand true
}


#
# doRcvrAtten
#
proc doRcvrAtten { tg } {
  scale $tg.adj -label "Atten (dB)"  -from 0 -to 50 \
               -resolution 10 -length 3c -orient horizontal \
               -command {Receiver atten} -font rcvr_font

  pack $tg.adj    -side top  -fill both -expand true
}

#
# doRcvrHopTest
#
proc doRcvrHopTest { tg } {
  global do_rcvr_freq_hop_switch
  global RCVR_STATUS

  frame $tg.freq1
  frame $tg.freq2
  frame $tg.delay
  frame $tg.hw

  label $tg.freq1.msg   -text  "Freq1 (MHz)" -font rfgen_font 
  entry $tg.freq1.stat  -textvariable  RCVR_STATUS(hop_freq1) -bg white \
                      -font rfgen_font -relief sunken -width 8 -bg white

  label $tg.freq2.msg   -text  "Freq2 (MHz)" -font rfgen_font 
  entry $tg.freq2.stat  -textvariable  RCVR_STATUS(hop_freq2) -bg white \
                      -font rfgen_font -relief sunken -width 8 -bg white

  label $tg.delay.msg   -text  "Delay (mS)"  -font rfgen_font 
  entry $tg.delay.stat  -textvariable  RCVR_STATUS(hop_delay) -bg white \
                      -font rfgen_font -relief sunken -width 5 -bg white

  label $tg.hw.msg      -text  "Test Port Bitmap"  -font rfgen_font 
  entry $tg.hw.stat     -textvariable  RCVR_STATUS(hop_bitmap) -bg white \
                      -font rfgen_font -relief sunken -width 5 -bg white

  checkbutton $tg.state -text "Rcvr Hop Test (On/Off)" \
                      -variable do_rcvr_freq_hop_switch \
                      -font rfgen_font \
                      -command start_rcvr_freq_hop

  bind $tg.hw.stat <Return> { set_rcvr_hopstate $RCVR_STATUS(hop_bitmap) }


  pack $tg.freq1.msg    -side left  -fill both -expand true
  pack $tg.freq1.stat   -side left  -fill both -expand true

  pack $tg.freq2.msg    -side left  -fill both -expand true
  pack $tg.freq2.stat   -side left  -fill both -expand true

  pack $tg.delay.msg    -side left  -fill both -expand true
  pack $tg.delay.stat   -side left  -fill both -expand true

  pack $tg.hw.msg       -side left  -fill both -expand true
  pack $tg.hw.stat      -side left  -fill both -expand true

  pack $tg.state        -side top  -fill both -expand true
  pack $tg.freq1        -side top  -fill both -expand true
  pack $tg.freq2        -side top  -fill both -expand true
  pack $tg.hw           -side top  -fill both -expand true
  pack $tg.delay        -side top  -fill both -expand true
}





#
# doRcvrIfBw
#
proc doRcvrIfBw { tg } {
  global RcvrState

  message $tg.msg       -text "IF Bandwidth"  -width 3c -font rcvr_font 
  radiobutton $tg.30k   -text "30 k" -font rcvr_font \
                      -command { set_rcvr_ifbw $RcvrState(ifbw) } \
                      -variable RcvrState(ifbw) \
                      -value 30KHZ   
  radiobutton $tg.300k  -text "300 k" -font rcvr_font \
                      -command { set_rcvr_ifbw $RcvrState(ifbw) } \
                      -variable RcvrState(ifbw) \
                      -value 300KHZ
  radiobutton $tg.3m    -text "3 M" -font rcvr_font \
                      -command { set_rcvr_ifbw $RcvrState(ifbw) } \
                      -variable RcvrState(ifbw) \
                      -value 3MHZ

  pack $tg.msg     -side top  -fill both -expand true
  pack $tg.30k     -side left -fill both -expand true
  pack $tg.300k    -side left -fill both -expand true
  pack $tg.3m      -side left -fill both -expand true
}


#
# doRcvrIfGain
#
proc doRcvrIfGain { tg } {
  global RcvrState
  global RCVR_ENGINE

  scale $tg.adj -label "IF Gain (dB)"  -from -14.0 -to 82.0 \
                -resolution 0.1 -length 3c -orient horizontal \
                -command Receiver ifgain -font rcvr_font

  label $tg.check -textvariable RCVR_ENGINE(OutGoingFreq)  -font rcvr_font 
  label $tg.msg   -text  "Adj Rx Freq (MHz)"  -font rcvr_font 
  entry $tg.entry -textvariable  RCVR_ENGINE(RF_Freq) -width 10 -bg white


  bind $tg.entry <Return> {Receiver freq $RCVR_ENGINE(RF_Freq)}

  pack $tg.adj    -side top  -fill both -expand true
  pack $tg.msg    -side top  -fill both -expand true
  pack $tg.check  -side left -fill both -expand true
  pack $tg.entry  -side left -fill both -expand true
}


#
# doRcvrSwAgc
#
proc doRcvrSwAgc { tg } {
  global RcvrState

  frame $tg.adj
  frame $tg.ll
  frame $tg.ul
  frame $tg.filt

  label $tg.adj.msg   -text  "Sample Period (mS)"          -font rcvr_font 
  entry $tg.adj.entry -textvariable  RcvrState(AgcSample)  -width 10 -bg white
  bind $tg.adj.entry <Return> trace_rcvr_sw_agc_period

  label $tg.ll.msg   -text  "AGC LL (dBm)"                 -font rcvr_font 
  entry $tg.ll.entry -textvariable  RcvrState(AgcLL)       -width 10 -bg white
  bind $tg.ll.entry <Return> trace_rcvr_sw_agc_ll

  label $tg.ul.msg   -text  "AGC UL (dBm)"                 -font rcvr_font 
  entry $tg.ul.entry -textvariable  RcvrState(AgcUL)       -width 10 -bg white
  bind $tg.ul.entry <Return> trace_rcvr_sw_agc_ul

  label $tg.filt.msg   -text  "AGC Filter"                 -font rcvr_font 
  entry $tg.filt.entry -textvariable  RcvrState(AgcFilter) -width 10 -bg white
  bind $tg.filt.entry <Return> trace_rcvr_sw_agc_filter

  checkbutton $tg.state -text "State" -font rcvr_font \
                        -command trace_rcvr_sw_agc_state \
                        -variable RcvrState(AgcState)

  pack $tg.adj.msg   -side left -fill both -expand true
  pack $tg.adj.entry -side left -fill both -expand true

  pack $tg.ll.msg   -side left -fill both -expand true
  pack $tg.ll.entry -side left -fill both -expand true

  pack $tg.ul.msg   -side left -fill both -expand true
  pack $tg.ul.entry -side left -fill both -expand true

  pack $tg.filt.msg   -side left -fill both -expand true
  pack $tg.filt.entry -side left -fill both -expand true


  pack $tg.adj   -side top  -fill both -expand true
  pack $tg.ll    -side top  -fill both -expand true
  pack $tg.ul    -side top  -fill both -expand true
  pack $tg.filt  -side top  -fill both -expand true

  pack $tg.state -side top  -fill both -expand true
}


#
# doRcvrYigCals
#
proc doRcvrYigCals { tg } {
  global RcvrState
  global HwUtilEngine
  global RcvrYigCals

  label $tg.msg    -text  "Yig Cal Values:"  -font rcvr_font 
  entry $tg.entry1 -textvariable  RcvrYigCals(0) \
                 -width 5 -font rcvr_font  -bg white
  entry $tg.entry2 -textvariable  RcvrYigCals(2800) \
                 -width 5 -font rcvr_font  -bg white

  button $tg.button   -text "Do Yig Calibration" -font rcvr_font \
                      -command { set_start_yig_calibration }
  label  $tg.entry3   -textvariable  HwUtilEngine(yigcal_status) \
                      -width 5 -font rcvr_font  -bg white

  bind $tg.entry1 <Return> update_rcvr_yig_cal_data
  bind $tg.entry2 <Return> update_rcvr_yig_cal_data

  pack $tg.button  -side top  -fill both -expand true
  pack $tg.entry3  -side top  -fill both -expand true

  pack $tg.msg     -side top  -fill both -expand true
  pack $tg.entry1  -side left -fill both -expand true
  pack $tg.entry2  -side left -fill both -expand true
}


#
# doRcvr214pwrCal
#
proc doRcvr214pwrCal { tg } {
  global RcvrState

  label $tg.msg    -text  "Harris 214 IF Pwr Cal:"    -font rcvr_font 
  entry $tg.entry  -textvariable  RCVR_ENGINE(214pwrCal) \
                   -width 5 -font rcvr_font  -bg white

  bind $tg.entry <Return> { set_rcvr_cals 3 $RCVR_ENGINE(214pwrCal) 0 }

  pack $tg.msg     -side top  -fill both -expand true
  pack $tg.entry   -side left -fill both -expand true
}


#
# doRcvrPllStatus
#
proc doRcvrPllStatus { tg } {
  global RcvrState

  frame $tg.ref    
  frame $tg.main  
  frame $tg.clean 
  frame $tg.step  


  label $tg.msg        -text  "PLL Lock Status"  -font rcvr_font

  label $tg.ref.msg    -text  "Ref"              -font rcvr_font
  label $tg.ref.stat   -textvariable  RCVR_STATUS(lock_ref) \
                                 -font rcvr_font 
  label $tg.main.msg   -text  "Main"             -font rcvr_font 
  label $tg.main.stat  -textvariable  RCVR_STATUS(lock_main) \
                                 -font rcvr_font 
  label $tg.clean.msg  -text  "Clean"            -font rcvr_font 
  label $tg.clean.stat -textvariable  RCVR_STATUS(lock_clean) \
                                 -font rcvr_font 
  label $tg.step.msg   -text  "Step"             -font rcvr_font 
  label $tg.step.stat  -textvariable  RCVR_STATUS(lock_step) \
                                 -font rcvr_font 

  pack $tg.ref.msg     -side top  -fill both -expand true
  pack $tg.ref.stat    -side top  -fill both -expand true

  pack $tg.main.msg    -side top  -fill both -expand true
  pack $tg.main.stat   -side top  -fill both -expand true

  pack $tg.clean.msg   -side top  -fill both -expand true
  pack $tg.clean.stat  -side top  -fill both -expand true

  pack $tg.step.msg    -side top  -fill both -expand true
  pack $tg.step.stat   -side top  -fill both -expand true 

  pack $tg.msg     -side top  -fill both -expand true
  pack $tg.ref     -side left -fill both -expand true
  pack $tg.main    -side left -fill both -expand true
  pack $tg.clean   -side left -fill both -expand true
  pack $tg.step    -side left -fill both -expand true
}


#
# doRcvrYigStatus
#
proc doRcvrYigStatus { tg } {
  global RcvrState

  frame $tg.hold    
  frame $tg.dac  

  label $tg.hold.msg   -text  "Yig Hold"         -font rcvr_font 
  label $tg.hold.stat  -textvariable  RCVR_STATUS(yig_hold) \
                     -font rcvr_font 

  label $tg.dac.msg    -text  "Yig DAC"           -font rcvr_font 
  label $tg.dac.stat   -textvariable  RCVR_STATUS(yig_dac) \
                     -font rcvr_font 

  pack $tg.hold.msg   -side top  -fill both -expand true
  pack $tg.hold.stat  -side top  -fill both -expand true

  pack $tg.dac.msg    -side top  -fill both -expand true
  pack $tg.dac.stat   -side top  -fill both -expand true


  pack $tg.hold    -side left  -fill both -expand true
  pack $tg.dac     -side left  -fill both -expand true
}


#
#  build rcvr screen
#
proc build_rcvr_screen { name } {
  global hrn
  global DemoColor

  # we have to save $hrn because the -command {} will not rememver
  # local instance of $name
  set hrn $name

  #
  # define screen frames
  #
  frame $hrn.mbar          -relief raised -bd 2
  frame $hrn.locksummary   -relief raised -bd 2


  #
  # PLL lock summary
  #
  label $hrn.locksummary.stat  -textvariable  RCVR_STATUS(lock_summary) \
                               -font rcvr_font

  message $name.mbar.label    -text "Receiver" -font rcvr_font \
                              -bg $DemoColor(title) \
                              -width $DemoColor(labelwidth)

  #
  #  Define menu bars 
  #
  menubutton $hrn.mbar.rcvr   -text  "FrontEnd"    -font rcvr_font \
                          -menu $hrn.mbar.rcvr.menu \
                          -bg $DemoColor(menu) -relief raised
  menubutton $hrn.mbar.if     -text  "IF"          -font rcvr_font \
                          -menu $hrn.mbar.if.menu \
                          -bg $DemoColor(menu) -relief raised
  menubutton $hrn.mbar.demod  -text  "Demod"       -font rcvr_font \
                          -menu $hrn.mbar.demod.menu \
                          -bg $DemoColor(menu) -relief raised
  button $hrn.mbar.p25        -text "P25" -command { WidgetWindow build_P25_decode_screen \
                                                 $hrn.p25 $hrn "P25 Decode"} \
                          -font rcvr_font -bg $DemoColor(menu)
  menubutton $hrn.mbar.cals   -text  "Calibration" -font rcvr_font \
                          -menu $hrn.mbar.cals.menu \
                          -bg $DemoColor(menu) -relief raised
  menubutton $hrn.mbar.status -text  "Status"      -font rcvr_font \
                          -menu $hrn.mbar.status.menu \
                          -bg $DemoColor(menu) -relief raised

  #
  #  Build Receiver menu 
  #
  menu $hrn.mbar.rcvr.menu
  $hrn.mbar.rcvr.menu add command \
            -label "RF Freq" -font rcvr_font \
            -command { WidgetWindow doRcvrFreq \
                       $hrn.rcvr_freq $hrn "Receiver Frequency"}
  $hrn.mbar.rcvr.menu add cascade \
            -label "Input" -font rcvr_font \
            -menu $hrn.mbar.rcvr.menu.input
  menu $hrn.mbar.rcvr.menu.input
  $hrn.mbar.rcvr.menu.input add radiobutton -label "T/R" -font rcvr_font \
                    -command { Receiver rfin $RcvrState(input) } \
                    -variable RcvrState(input) \
                    -value TR
  $hrn.mbar.rcvr.menu.input add radiobutton -label "Ant" -font rcvr_font \
                    -command { Receiver rfin $RcvrState(input) } \
                    -variable RcvrState(input) \
                    -value ANT

  $hrn.mbar.rcvr.menu add command \
            -label "Atten" -font rcvr_font \
            -command { WidgetWindow doRcvrAtten \
                       $hrn.rcvr_atten $hrn "Receiver Attenuation" }
  $hrn.mbar.rcvr.menu add cascade \
            -label "Range" -font rcvr_font \
            -menu $hrn.mbar.rcvr.menu.range
  menu $hrn.mbar.rcvr.menu.range
  $hrn.mbar.rcvr.menu.range add radiobutton -label "BW Wide" -font rcvr_font \
                      -command { set_rcvr_range $RcvrState(range) } \
                      -variable RcvrState(range) \
                      -value WIDE   
  $hrn.mbar.rcvr.menu.range add radiobutton -label "BW Narrow" -font rcvr_font \
                      -command { set_rcvr_range $RcvrState(range) } \
                      -variable RcvrState(range) \
                      -value NARROW

  $hrn.mbar.rcvr.menu add cascade \
          -label "PLL Lock" -font rcvr_font \
          -menu $hrn.mbar.rcvr.menu.lock
  menu $hrn.mbar.rcvr.menu.lock
  $hrn.mbar.rcvr.menu.lock add radiobutton -label "Lock" -font rcvr_font \
                   -command { set_rcvr_lock $RcvrState(lock) } \
                   -variable RcvrState(lock) \
                   -value LOCK   
  $hrn.mbar.rcvr.menu.lock add radiobutton -label "Sweep" -font rcvr_font \
                   -command { set_rcvr_lock $RcvrState(lock) } \
                   -variable RcvrState(lock) \
                   -value SWEEP

  $hrn.mbar.rcvr.menu add cascade \
            -label "AGC" -font rcvr_font \
            -menu $hrn.mbar.rcvr.menu.agc
  menu $hrn.mbar.rcvr.menu.agc
  $hrn.mbar.rcvr.menu.agc add radiobutton -label "Fast" -font rcvr_font \
                    -command { set_rcvr_hwagc $RcvrState(agc) } \
                    -variable RcvrState(agc) \
                    -value FAST
  $hrn.mbar.rcvr.menu.agc add radiobutton -label "Slow" -font rcvr_font \
                    -command { set_rcvr_hwagc $RcvrState(agc) } \
                    -variable RcvrState(agc) \
                    -value SLOW

  $hrn.mbar.rcvr.menu add command \
            -label "Synth Hop Test" -font rcvr_font \
            -command { WidgetWindow doRcvrHopTest \
                       $hrn.rcvr_hop $hrn "Receiver Hop Test" }

  #
  #  Build IF menu 
  #
  menu $hrn.mbar.if.menu
  $hrn.mbar.if.menu add command \
            -label "Bandwidth" -font rcvr_font \
            -command { WidgetWindow doRcvrIfBw \
                       $hrn.if_bw $hrn "IF Bandwidth" }
  $hrn.mbar.if.menu add command \
            -label "Gain" -font rcvr_font \
            -command { WidgetWindow doRcvrIfGain \
                       $hrn.if_gain $hrn "IF Gain" }
  $hrn.mbar.if.menu add command \
            -label "SW AGC" -font rcvr_font \
            -command { WidgetWindow doRcvrSwAgc \
                       $hrn.if_sw_agc $hrn "Receiver SW AGC" }

  #
  #  Build Demod menu 
  #
  menu $hrn.mbar.demod.menu
  $hrn.mbar.demod.menu add cascade \
          -label "Demod" -font rcvr_font \
          -menu $hrn.mbar.demod.menu.demod
 
  menu $hrn.mbar.demod.menu.demod
  $hrn.mbar.demod.menu.demod add radiobutton -label "Off" -font rcvr_font \
                 -command { set_rcvr_demod $RcvrState(demod) } \
                 -variable RcvrState(demod) \
                 -value DEMOD_OFF
  $hrn.mbar.demod.menu.demod add radiobutton -label "FM Narr" -font rcvr_font \
                 -command { set_rcvr_demod $RcvrState(demod) } \
                 -variable RcvrState(demod) \
                 -value DEMOD_FM_NARROW
  $hrn.mbar.demod.menu.demod add radiobutton -label "FM Wide" -font rcvr_font \
                 -command { set_rcvr_demod $RcvrState(demod) } \
                 -variable RcvrState(demod) \
                 -value DEMOD_FM_WIDE
  $hrn.mbar.demod.menu.demod add radiobutton -label "FM WideBand" -font rcvr_font \
                 -command { set_rcvr_demod $RcvrState(demod) } \
                 -variable RcvrState(demod) \
                 -value DEMOD_FM_WB
  $hrn.mbar.demod.menu.demod add radiobutton -label "FM MediumBand" -font rcvr_font \
                 -command { set_rcvr_demod $RcvrState(demod) } \
                 -variable RcvrState(demod) \
                 -value DEMOD_FM_MEDIUM
  $hrn.mbar.demod.menu.demod add radiobutton -label "FM TV" -font rcvr_font \
                 -command { set_rcvr_demod $RcvrState(demod) } \
                 -variable RcvrState(demod) \
                 -value DEMOD_FM_TV
  $hrn.mbar.demod.menu.demod add radiobutton -label "AM" -font rcvr_font \
                 -command { set_rcvr_demod $RcvrState(demod) } \
                 -variable RcvrState(demod) \
                 -value DEMOD_AM
  $hrn.mbar.demod.menu.demod add radiobutton -label "P25" -font rcvr_font \
                 -command { set_rcvr_demod $RcvrState(demod) } \
                 -variable RcvrState(demod) \
                 -value DEMOD_P25

  #
  #  Build Calibration menu 
  #
  menu $hrn.mbar.cals.menu
  $hrn.mbar.cals.menu add command \
            -label "Yig Cal" -font rcvr_font \
            -command { WidgetWindow doRcvrYigCals \
                       $hrn.cals_rx_yig $hrn "YIG calibration" }
  $hrn.mbar.cals.menu add command \
            -label "214 Power" -font rcvr_font \
            -command { WidgetWindow doRcvr214pwrCal \
                       $hrn.cals_214 $hrn "50214 Power" }

  #
  #  Build Status menu 
  #
  menu $hrn.mbar.status.menu
  $hrn.mbar.status.menu add command \
            -label "PLL" -font rcvr_font \
            -command { WidgetWindow doRcvrPllStatus \
                       $hrn.status_rxpll $hrn "Receiver PLL status" }
  $hrn.mbar.status.menu add command \
            -label "Yig" -font rcvr_font \
            -command { WidgetWindow doRcvrYigStatus \
                       $hrn.status_rx_yig $hrn "Receiver YIG status" }



  #
  #  pack sub frames
  #
  pack $hrn.mbar.label        -side left  -fill both

  pack $hrn.mbar.rcvr         -side left  -fill both -expand true
  pack $hrn.mbar.if           -side left  -fill both -expand true
  pack $hrn.mbar.demod        -side left  -fill both -expand true
  pack $hrn.mbar.p25          -side left  -fill both -expand true
  pack $hrn.mbar.cals         -side left  -fill both -expand true
  pack $hrn.mbar.status       -side right -fill both -expand true


  #
  # Lock Subframe
  #
  pack $hrn.locksummary.stat  -side left -fill both -expand true



  #
  #  pack main frame
  #
  pack $hrn.mbar              -side left  -fill both -expand true

  pack $hrn.locksummary       -side right -fill both



  return $hrn
}

