#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C)1998,99      DESIGN ENGINEERING      #
# --------------------------------------------------------------------------#
# FILE        : to_rcvr.tcl                                                 #
# PRODUCT     : FITS                                                        #
# AUTHOR      : Steve Johnson                                               #
# DESCRIPTION : Interface to Receiver via Message Bus                       #
#                                                                           #
#############################################################################

#
#  Receiver System Number
#
set RCVR_ENGINE(SYSTEM_ID)            2

#
#  Receiver Sub-Systems
#
set RCVR_ENGINE(SUB_FREQ)             1
set RCVR_ENGINE(SUB_INPUT)            2
set RCVR_ENGINE(SUB_ATTEN)            3

set RCVR_ENGINE(SUB_IFGAIN)           4
set RCVR_ENGINE(SUB_IFBW)             5

set RCVR_ENGINE(SUB_RANGE)            6
set RCVR_ENGINE(SUB_DEMOD)            7

set RCVR_ENGINE(SUB_AGC_MODE)         8
set RCVR_ENGINE(SUB_AGC_LEVEL)        9
set RCVR_ENGINE(SUB_AGC_SPEED)       10

set RCVR_ENGINE(SUB_SQ_STAT)         11
set RCVR_ENGINE(SUB_SQ_LEVEL)        12

set RCVR_ENGINE(SUB_LOCK)            13
set RCVR_ENGINE(SUB_LOCK_STAT)       14

set RCVR_ENGINE(SUB_ENABLE)          15
set RCVR_ENGINE(SUB_INFO)            16

set RCVR_ENGINE(SUB_UPDATE_YIG_CAL)  17
set RCVR_ENGINE(SUB_DEFAULT_IFATTEN) 18
set RCVR_ENGINE(SUB_YIG_STAT)        19

set RCVR_ENGINE(SUB_DLOAD_CAL)       20
set RCVR_ENGINE(SUB_CALCONFIG)       21
set RCVR_ENGINE(SUB_AGC_ELEMENT)     22

set RCVR_ENGINE(SUB_HOPSTATE)        23
set RCVR_ENGINE(SUB_RCVRHWAGC)       24


#
#  Receiver Call-back Data
#
set RCVR_STATUS(lock_main)   -1
set RCVR_STATUS(lock_clean)  -1
set RCVR_STATUS(lock_step)   -1
set RCVR_STATUS(lock_ref)    -1
set RCVR_STATUS(alarm)        0
set RCVR_STATUS(yig_hold)    -1
set RCVR_STATUS(yig_dac)     -1
set RCVR_STATUS(lock_summary) "PLL Lock Init"


#
#  Receiver Helpful globals
#
set RCVR_ENGINE(RF_Freq)      500.0
set RCVR_ENGINE(OutGoingFreq) 500.0
set Rcvr_Handler(lock)          0
set Rcvr_Handler(yig)           0

set SET_TOP_OF_TABLE_RECOMPUTE 255

#
#  Receiver Font Declaration
#
font create rcvr_font -family helvetica -size 8

#
#  Receiver Yig Caligration Data
#
set RcvrYigCals(temp)            25
set RcvrYigCals(0)               100
set RcvrYigCals(2800)            2638

set RCVR_ENGINE(214pwrCal)       848


#############################################################################
#                                                                           #
#                       Dead Interfaces                                     #
#                                                                           #
#############################################################################


proc set_rcvr_input { x } {
  global RCVR_ENGINE

  # be sure to reset PT in switching (could be hung)
  set_pt_reset

  # no one should be calling this
  # we'll track down users with the following
  puts "set_rcvr_input obsolete!"
}

#############################################################################
#                                                                           #
#                       Message Bus Interfaces                              #
#                                                                           #
#############################################################################


#
#  RF Gen calibration interface
#
proc set_rcvr_cals { index thing1 thing2 } {
  global RCVR_ENGINE
  global Send_Flags

  set msg " { $RCVR_ENGINE(SYSTEM_ID) \
              $RCVR_ENGINE(SUB_DLOAD_CAL) 1 $Send_Flags } \
            { inline dw $index } \
            { inline double $thing1 } \
            { inline double $thing2 } "

  tclMsgbus send $msg
}

#
#  Message bus interfaces Rcvr calibration message
#
proc update_rcvr_cals { index cal } {
  global RCVR_ENGINE
  global Send_Flags
  
  # send <index> <cal>
  set msg " { $RCVR_ENGINE(SYSTEM_ID) $RCVR_ENGINE(SUB_UPDATE_YIG_CAL) 1 \
              $Send_Flags } { inline dw $index $cal } "
#puts $msg
  tclMsgbus send $msg
}


#
#  is_locked - printed message
#
proc is_locked { bool } { 
  if { $bool } {
    return "Unlock"
  } 

  return "Locked"
}


#
#  Message bus callback - fill in supplied structure
#
proc handle_rcvr_lock {sys sub lun flags uid thing } {
  global RCVR_STATUS

#set stuff [format "rcvr msg %x " $thing]
#puts $thing

  set RCVR_STATUS(lock_main)    [ is_locked [ expr { $thing & 0x1 } ] ]
  set RCVR_STATUS(lock_clean)   [ is_locked [ expr { $thing & 0x2 } ] ]
  set RCVR_STATUS(lock_step)    [ is_locked [ expr { $thing & 0x4 } ] ]
  set RCVR_STATUS(lock_ref)     [ is_locked [ expr { $thing & 0x8 } ] ]

  if { $thing } {
    set RCVR_STATUS(lock_summary) "PLL Unlocked"
  } else {
    set RCVR_STATUS(lock_summary) "PLL Locked"
  }
}

#
#  Message bus callback - fill in supplied structure
#
proc handle_rcvr_yig {sys sub lun flags uid hold dac } {
  global RCVR_STATUS

#puts "rcvr msg:  $uid   $hold $dac"

  set RCVR_STATUS(yig_hold)  $hold
  set RCVR_STATUS(yig_dac)   $dac
}

#
#  Message bus interfaces for messages being generated FROM rcvr engine
#
proc create_rcvr_callback { } {
  global RCVR_ENGINE
  global Rcvr_Handler

  set Rcvr_Handler(lock) [tclMsgbus rcv handle -sys $RCVR_ENGINE(SYSTEM_ID) \
         -sub $RCVR_ENGINE(SUB_LOCK_STAT) -lun 1 -cmd "handle_rcvr_lock" ]
  set Rcvr_Handler(yig)  [tclMsgbus rcv handle -sys $RCVR_ENGINE(SYSTEM_ID) \
         -sub $RCVR_ENGINE(SUB_YIG_STAT) -lun 1 -cmd "handle_rcvr_yig" ]
}


#
#  Message bus interfaces - removed from use
#
proc destroy_rcvr_callback { } {
  global Rcvr_Handler

  if {$Rcvr_Handler(lock) != 0 } {
    tclMsgbus $Rcvr_Handler(lock) destroy
  }

  if {$Rcvr_Handler(yig) != 0 } {
    tclMsgbus $Rcvr_Handler(yig) destroy
  }

  set Rcvr_Handler(lock) 0
  set Rcvr_Handler(yig)  0
}


#############################################################################
#                                                                           #
#                       Engine Interfaces                                   #
#                                                                           #
#############################################################################


#
#  IF BandWidth ( 30 kHz, 300 kHz or 3 MHz )
#
proc set_rcvr_ifbw { x } {
  global RCVR_ENGINE

  switch $x {
    30KHZ   { set bw 30000   }
    300KHZ  { set bw 300000  }
    3MHZ    { set bw 3000000 }
    default { return   }
  }

  message_bus_int $RCVR_ENGINE(SYSTEM_ID) $RCVR_ENGINE(SUB_IFBW) 1 $bw
}


#
#  enable alarm or lock messages from DSP
#
proc set_rcvr_enable { x } {
   global RCVR_ENGINE

  switch $x {
    DISABLE_MSG { message_bus_int $RCVR_ENGINE(SYSTEM_ID) \
                                  $RCVR_ENGINE(SUB_ENABLE) 1 0  }
       LOCK_MSG { message_bus_int $RCVR_ENGINE(SYSTEM_ID) \
                                  $RCVR_ENGINE(SUB_ENABLE) 1 2  }
      ALARM_MSG { message_bus_int $RCVR_ENGINE(SYSTEM_ID) \
                                  $RCVR_ENGINE(SUB_ENABLE) 1 4  }
        YIG_MSG { message_bus_int $RCVR_ENGINE(SYSTEM_ID) \
                                  $RCVR_ENGINE(SUB_ENABLE) 1 8  }
           DEMO -
   LOCK_YIG_MSG { message_bus_int $RCVR_ENGINE(SYSTEM_ID) \
                                  $RCVR_ENGINE(SUB_ENABLE) 1 10 }
        ALL_MSG { message_bus_int $RCVR_ENGINE(SYSTEM_ID) \
                                  $RCVR_ENGINE(SUB_ENABLE) 1 15 }
  }
}

#
#  Receiver Defaults - Set IF attenuation to 0 (called 1st after powerup)
#
proc set_rcvr_default_ifatten { } {
  global RCVR_ENGINE

  message_bus_int $RCVR_ENGINE(SYSTEM_ID) \
                  $RCVR_ENGINE(SUB_DEFAULT_IFATTEN) 1 0
}

#
#  send YIG cal table to DSP
#
#  the global "RcvrYigCals" is defined in "to_cals.tcl"
#
proc update_rcvr_yig_cal_data { } {
  global SET_TOP_OF_TABLE_RECOMPUTE 
  global RcvrYigCals

  update_rcvr_cals 0 $RcvrYigCals(0)
  update_rcvr_cals $SET_TOP_OF_TABLE_RECOMPUTE  $RcvrYigCals(2800)
}


#
#  Receiver hardware range - Improve phase noise
#          Wide for modulation accuracy tests (better residual noise)
#          Narrow for selectivitiy testing (better phase noise >= 10 kHz)
#
proc set_rcvr_range { x } {
  global RCVR_ENGINE

  switch $x {
    WIDE   { set range 0  }
    NARROW { set range 1  }
    default { return      }
  }

  message_bus_int $RCVR_ENGINE(SYSTEM_ID) $RCVR_ENGINE(SUB_RANGE) 1 $range
}

#
#  Receiver PLL lock condition
#           LOCK: for normal receiver operation
#           SWEEP: for spectrum analyzer operation
#
proc set_rcvr_lock { x } {
  global RCVR_ENGINE

  switch $x {
    LOCK    { set y 1  }
    SWEEP   { set y 0  }
    default { return   }
  }

  message_bus_int $RCVR_ENGINE(SYSTEM_ID) $RCVR_ENGINE(SUB_LOCK) 1 $y
}


#
#  Receiver AGC mode -  Fast for tracking gen, slow best phase noise
#
proc set_rcvr_hwagc { x } {
  global RCVR_ENGINE

  switch $x {
    FAST    { set agc 0  }
    SLOW    { set agc 1  }
    default { return     }
  }

  message_bus_int $RCVR_ENGINE(SYSTEM_ID) $RCVR_ENGINE(SUB_RCVRHWAGC) 1 $agc
}


#
#  Set Receiver SW AGC (ON/OFF)
#
proc set_rcvr_agc_mode { state } {
  global RCVR_ENGINE

  switch $state {
    ON      { set in 1  }
    OFF     { set in 0  }
    default { return    }
  }

  message_bus_int $RCVR_ENGINE(SYSTEM_ID) $RCVR_ENGINE(SUB_AGC_MODE) 1 $in
}


#
#  Receiver Demod - set demodulation type
#
proc set_rcvr_demod { type } {
  global RCVR_ENGINE

  switch $type {
    DEMOD_FM_WIDE    { set demod 0  }
    DEMOD_FM_NARROW  { set demod 1  }
    DEMOD_FM_WB      { set demod 2  }
    DEMOD_FM_TV      { set demod 3  }
    DEMOD_AM         { set demod 4  }
    DEMOD_P25        { set demod 5  }
    DEMOD_FM_MEDIUM  { set demod 10 }
    DEMOD_OFF        { set demod 11 }
    default          { return       }
  }

  message_bus_int $RCVR_ENGINE(SYSTEM_ID) $RCVR_ENGINE(SUB_DEMOD) 1 $demod
}


#
#  Rcvr Agc Element Interface
#       index     1   IF power sample period in 10 mS increments ( default 10)
#                 2   Adjustment low limit (0 to -45 dBm, default -12.5 dBm)
#                 3   Adjustment high limit (0 to -45 dBm default -11.5 dBm)
#                 4   Box-car filter size (minimum is 2)
#                 5   AGC state ( 1 is ON, 0 is disabled - default is 1)
#
proc set_rcvr_agc_elements { type value } {
  global RCVR_ENGINE
  global Send_Flags

  switch $type {
    PERIOD  { set x 1 }
    AGCLL   { set x 2 }
    AGCUL   { set x 3 }
    FILTER  { set x 4 }
    default { return  }
  }

  set msg " { $RCVR_ENGINE(SYSTEM_ID) \
              $RCVR_ENGINE(SUB_AGC_ELEMENT) 1 $Send_Flags } \
            { inline dw $x } \
            { inline sdword $value } "

  tclMsgbus send $msg
}


proc trace_rcvr_sw_agc_period { } {
  global RcvrState

  # mS to mS/10
  set x [ expr int( $RcvrState(AgcSample) / 10 ) ]

  if { $x <= 0 } { 
    set x 1 
    set RcvrState(AgcSample) 10
  }

  set_rcvr_agc_elements PERIOD $x 
}


proc trace_rcvr_sw_agc_ll { } { 
  global RcvrState

  # dBm to 10*dBm
  set x [ expr int( $RcvrState(AgcLL) * 10 ) ]

  set_rcvr_agc_elements AGCLL $x
}

proc trace_rcvr_sw_agc_ul { } { 
  global RcvrState

  # dBm to 10*dBm
  set x [ expr int( $RcvrState(AgcUL) * 10 )]

  set_rcvr_agc_elements AGCUL $x 
}

proc trace_rcvr_sw_agc_filter { } { 
  global RcvrState
  set_rcvr_agc_elements FILTER $RcvrState(AgcFilter) 
}

proc trace_rcvr_sw_agc_state { } { 
  global RcvrState
  global RCVR_ENGINE

  message_bus_int $RCVR_ENGINE(SYSTEM_ID) $RCVR_ENGINE(SUB_AGC_MODE) \
                  1 $RcvrState(AgcState)
}


#
#  Receiver State
#
#   this function will turn on the hardware signal capability that a
#   freq change has occured in the DSP code - settling time test purposes
#
#
#   bitmap -  which bit(s) to toggle 1 or 0
#     Bit0 = Digital Out 1 (DB15 Pin09): 1=Low, 0=Open Collector [DEFAULT].
#     Bit1 = Digital Out 2 (DB15 Pin10): 1=Low, 0=Open Collector [DEFAULT].
#     Bit2 = Digital Out 3 (DB15 Pin11): 1=Low, 0=Open Collector [DEFAULT].
#     Bit3 = Digital Out 4 (DB15 Pin12): 1=Low, 0=Open Collector [DEFAULT].
#     Bit4 = Serial  Out 1 (DB15 Pin13): 1=Low, 0=Open Collector [DEFAULT].
#              WARNING: Bit4 may be removed in the future.  A serial shift reg
#              might be added to the FPGA to send serial data out this pin.
#              NOTE: Depending on how this is done, the live state of this
#              pin might not be available.
#     Bit5 = Vout Disable  (DB15 Pin08): 1=Disable [DEFAULT], 0=Enable.
#
proc set_rcvr_hopstate { bitmap } {
  global RCVR_ENGINE

  message_bus_int $RCVR_ENGINE(SYSTEM_ID) $RCVR_ENGINE(SUB_HOPSTATE) 1 $bitmap
}


#
#  do_rcvr_freq_hop - test proc for hopping freqs
#
#         interval - time between freq changes (mS)
#         freq1    - 1st freq in MHz
#         freq2    - 2nd freq in MHz
#
#
set do_rcvr_freq_hop_switch 0

proc do_rcvr_freq_hop { interval freq1 freq2 orignal} {
  global do_rcvr_freq_hop_switch

  if { $do_rcvr_freq_hop_switch } {
#  puts $freq1
     Receiver freq $freq1 MHz
     after $interval [ list do_rcvr_freq_hop $interval $freq2 $freq1 $orignal ]
  } else {
     Receiver freq $orignal MHz
  }
}

#
#  Start freq hop test
#
proc start_rcvr_freq_hop { } {
  global do_rcvr_freq_hop_switch
  global RCVR_STATUS
  global RCVR_ENGINE

  if { $do_rcvr_freq_hop_switch } {
    do_rcvr_freq_hop $RCVR_STATUS(hop_delay) $RCVR_STATUS(hop_freq1) \
                     $RCVR_STATUS(hop_freq2) $RCVR_ENGINE(RF_Freq)
  }
}

#############################################################################
#                                                                           #
#                       Calibration Interfaces                              #
#                                                                           #
#############################################################################

#
# parse_rcvr_yig_cal
#
# The Receiver Calibrations have the form:
# RXYIGCAL   Temp   25                      
# RXYIGCAL   Freq  <MHz>    Value <cal number> 
#
proc parse_rcvr_yig_cal { line } {
  global RcvrYigCals

  switch [ lindex $line 1 ] {
    Temp    { set RcvrYigCals(temp)  [ lindex $line 2 ] }
    Freq    { set RcvrYigCals([ lindex $line 2 ])  [ lindex $line 4 ] }
    default { return       }
  }

# puts $RcvrYigCals([ lindex $line 2 ])
}


#
# update tcl yig cal data
#
proc get_rcvr_yig_cal_data { filename } {
  global CalFileName

  set file [ OpenCalFile $filename r ]

  while { [ gets $file line ] >= 0 } {
    if { [ lindex $line 0 ] == "RXYIGCAL" } {
      parse_rcvr_yig_cal $line
    }
  }

  close $file
}



#############################################################################
#                                                                           #
#                       System Calibration                                  #
#                                                                           #
#############################################################################

#
#  Read RCVR Freq Calibration Table
#
proc rcvrcal_freq_read_cal { filename rfport } {
   global RCVRCAL


   switch $rfport {
     ANT    {
             set token_chr RCVR_ANT_FREQ_CHARACTERISTICS
             set token_ele RCVR_ANT_FREQ_CAL
            }
     TR     {
             set token_chr RCVR_TR_FREQ_CHARACTERISTICS
             set token_ele RCVR_TR_FREQ_CAL
            }
     default { return }
   }


   set fileId [ OpenCalFile $filename r ]

   while { [ gets $fileId line ] >= 0} {

     if { [lindex $line 0] == $token_chr } {
        set RCVRCAL($rfport.FreqMaxIndex)    [lindex $line 2 ]
        set RCVRCAL($rfport.FreqRef.0)       [lindex $line 4 ]
        set RCVRCAL($rfport.FreqRef.10)      [lindex $line 6 ]
        set RCVRCAL($rfport.FreqRef.20)      [lindex $line 8 ]
        set RCVRCAL($rfport.FreqRef.30)      [lindex $line 10 ]
        set RCVRCAL($rfport.FreqRef.40)      [lindex $line 12 ]
        set RCVRCAL($rfport.FreqRef.50)      [lindex $line 14 ]

        continue
     }

     if { [lindex $line 0] == $token_ele } {
        set index                           [lindex $line 2  ]
        set RCVRCAL($rfport.freq.$index)    [lindex $line 4  ]
        set RCVRCAL($rfport.db.0.$index)    [lindex $line 6  ]
        set RCVRCAL($rfport.db.10.$index)   [lindex $line 8  ]
        set RCVRCAL($rfport.db.20.$index)   [lindex $line 10 ]
        set RCVRCAL($rfport.db.30.$index)   [lindex $line 12 ]
        set RCVRCAL($rfport.db.40.$index)   [lindex $line 14 ]
        set RCVRCAL($rfport.db.50.$index)   [lindex $line 16 ]

        rcvrcal_freq_db_2_dbm $index $rfport
        continue
     }
   }

  close $fileId
}


#
#  rcvrcal_freq_db_2_dbm
#
#
proc rcvrcal_freq_db_2_dbm { index rfport } {
   global RCVRCAL

   set steps { 0 10 20 30 40 50 }

   foreach s $steps {
     set RCVRCAL($rfport.dbm.$s.$index) \
        [ expr $RCVRCAL($rfport.db.$s.$index) + $RCVRCAL($rfport.FreqRef.$s) ]
   }
}

#
#  upload freq cals RF Gen engine
#
proc rcvrcal_set_engine_cal { rfport } {
   global RCVRCAL

   switch $rfport {
     ANT    {  set type 1 }
     TR     {  set type 2 }
     default { return }
   }

   # update ALC curve in ENGINE
#   gencal_set_calconfig $type $RCVR_ENGINE(FreqMaxIndex)

   set i $RCVRCAL($rfport.FreqMaxIndex)
   incr i -1

   set steps { 0 10 20 30 40 50 }
 
   # re-spline occures when sending index 0
   for {  } { $i >= 0 } { incr i -1 } {
      rcvrcal_freq_dbm_2_db $i $rfport
      foreach s $steps {

#        puts " rcvrcal_set_engine_cal $type $rfport $i $s \
#                          $RCVRCAL($rfport.freq.$i) \
#                          $RCVRCAL($rfport.db.$s.$i) "

        rcvrcal_to_engine $type $rfport $i $s \
                          $RCVRCAL($rfport.freq.$i) \
                          $RCVRCAL($rfport.db.$s.$i)
      }
   }
}

#
#  rcvrcal_freq_dbm_2_db
#
#  
#
proc rcvrcal_freq_dbm_2_db { index rfport } {
   global RCVRCAL

   set steps { 0 10 20 30 40 50 }

   foreach s $steps {
     set RCVRCAL($rfport.db.$s.$index) \
      [ expr $RCVRCAL($rfport.dbm.$s.$index) - $RCVRCAL($rfport.FreqRef.$s) ]
   }
}


#
#  RF Gen frequency calibration
#
#  args:   type          1 is freq calibrations
#          port          0 is TR, 256 is ANT
#          index         array index (0 to 255)
#          atten         rcvr attenuator (0 10 20 30 40 50 dB)
#          freq          in MHz
#          cal           RF level in dB
#                                                          
#      0xff0000  - attenuation (0 10 20 30 40 50 )         
#      0x00ff00  - index ( 0 to 255)                       
#      0x0000f0  - port  ( 0 is TR, 1 is ANT )             
#      0x00000f  - type  ( 1 is atten freq curve )         
#                          2 is IF cal data                
#
#  special note: spline curve is updated on index 0 only.  
#
proc rcvrcal_to_engine { type port index atten freq cal } {
  global RCVR_ENGINE
  global Send_Flags

  switch $port {
    TR  { set x 0 }
    ANT { set x 16 }
    default { return }
  }

  set x [ expr $x + ( 65536 * $atten ) ]
  set x [ expr $x + ( 256 * $index ) ]
  set x [ expr $x + $type ]

  set_rcvr_cals $x $freq $cal
}
