#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C)1998,99,2000   DESIGN ENGINEERING    #
# --------------------------------------------------------------------------#
# FILE        : P25parms.tcl                                                #
# PRODUCT     : FITS                                                        #
# AUTHOR      : Steve White                                                 #
# DESCRIPTION : P-25 Encode Parameters Window                               #
#                                                                           #
#############################################################################

#These hold the actual edited P25 parameter values
set P25_PARMS(MFID) 0
set P25_PARMS(TGID) 1
set P25_PARMS(KID) 0
set P25_PARMS(SID) 1
set P25_PARMS(NAC) 293
set P25_PARMS(DID) 1
set P25_PARMS(STATUS) 0 
set P25_PARMS(LCO) 0
set P25_PARMS(P) 0
set P25_PARMS(SF) 0
set P25_PARMS(EMG) 0
set P25_PARMS(LSD) 0
set P25_PARMS(encrypt) 0
set P25_PARMS(EVM) 0

#This will end up being the window base name
set p25enc 0

#
#   Default P25 parameters values
#
proc default_p25_parms { } {
global P25_PARMS

   set P25_PARMS(MFID) 0
   set P25_PARMS(TGID) 1
   set P25_PARMS(KID) 0
   set P25_PARMS(SID) 1
   set P25_PARMS(NAC) 293
   set P25_PARMS(DID) 1
   set P25_PARMS(STATUS) 0 
   set P25_PARMS(LCO) 0
   set P25_PARMS(P) 0
   set P25_PARMS(SF) 0
   set P25_PARMS(EMG) 0
   set P25_PARMS(LSD) 0
   set P25_PARMS(encrypt) 0
   set P25_PARMS(EVM) 0
   send_p25_parms
}

#
#   Do the P-25 encode parameters Window (menu)
#    ENTRY: name - Base window name to build into
#
proc doP25parms { name } {
  global p25enc 
  global DemoColor
  global P25_PARMS

  set p25enc $name

  frame $p25enc.f1
  frame $p25enc.f2
  frame $p25enc.f3
  frame $p25enc.f4
  frame $p25enc.f4a
  frame $p25enc.f5
  frame $p25enc.f6
  frame $p25enc.f7
#  frame $p25enc.f8
#  frame $p25enc.f9
#  frame $p25enc.f10

  label $p25enc.label -text "P25 ENCODE PARAMETERS" -width 32

  label $p25enc.f1.mfidlab  -text " MFID:" -width 5
  entry $p25enc.f1.mfidval  -textvariable P25_PARMS(MFID) -width 4 -bg white
  bind $p25enc.f1.mfidval <Return> validate_mfid
  label $p25enc.f1.tgidlab  -text "TGID:"
  entry $p25enc.f1.tgidval  -textvariable P25_PARMS(TGID) -width 7 -bg white
  bind $p25enc.f1.tgidval <Return> validate_tgid

  label $p25enc.f2.naclab  -text "  NAC:" -width 5
  entry $p25enc.f2.nacval -textvariable P25_PARMS(NAC) -width 4 -bg white
  bind $p25enc.f2.nacval <Return> validate_nac
  label $p25enc.f2.kidlab  -text "  KID:"
  entry $p25enc.f2.kidval -textvariable P25_PARMS(KID) -width 7 -bg white
  bind $p25enc.f2.kidval <Return> validate_kid

  label $p25enc.f3.lcolab  -text "  LCO:" -width 5
  entry $p25enc.f3.lcoval -textvariable P25_PARMS(LCO) -width 4 -bg white
  bind $p25enc.f3.lcoval <Return> validate_lco
  label $p25enc.f3.didlab  -text "  DID:"
  entry $p25enc.f3.didval -textvariable P25_PARMS(DID) -width 7 -bg white
  bind $p25enc.f3.didval <Return> validate_did

  label $p25enc.f4.lsdlab  -text "  LSD:" -width 5
  entry $p25enc.f4.lsdval -textvariable P25_PARMS(LSD) -width 9 -bg white
  bind $p25enc.f4.lsdval <Return> validate_lsd
  label $p25enc.f4.sidlab  -text "  SID:"
  entry $p25enc.f4.sidval -textvariable P25_PARMS(SID) -width 7 -bg white
  bind $p25enc.f4.sidval <Return> validate_sid

  label $p25enc.f4a.evmlab  -text " C4FM:" -width 5
  entry $p25enc.f4a.evmval -textvariable P25_PARMS(EVM) -width 9 -bg white
  bind $p25enc.f4a.evmval <Return> validate_EVM

  label $p25enc.f5.statlab  -text "Status:"
  radiobutton $p25enc.f5.stat0 -text "00" -command send_p25_parms \
              -variable P25_PARMS(STATUS) -value 0
  radiobutton $p25enc.f5.stat1 -text "01" -command send_p25_parms \
              -variable P25_PARMS(STATUS) -value 1
  radiobutton $p25enc.f5.stat2 -text "10" -command send_p25_parms \
              -variable P25_PARMS(STATUS) -value 2
  radiobutton $p25enc.f5.stat3 -text "11" -command send_p25_parms \
              -variable P25_PARMS(STATUS) -value 3

  label $p25enc.f6.plab -text "P:"
  checkbutton $p25enc.f6.p -variable P25_PARMS(P) -command send_p25_parms
  label $p25enc.f6.dummy -width 4
  label $p25enc.f6.sflab -text "SF:"
  checkbutton $p25enc.f6.sf -variable P25_PARMS(SF) -command send_p25_parms
  label $p25enc.f6.emglab -text "EMG:"
  checkbutton $p25enc.f6.emg -variable P25_PARMS(EMG) -command send_p25_parms

  label $p25enc.f7.enclab -text "Encryption:"
  checkbutton $p25enc.f7.enc -variable P25_PARMS(encrypt) -command enc_state
  button $p25enc.f7.b1 -text DEFAULT -command default_p25_parms
#
#  Pack controls (labels/values)
#
  pack $p25enc.f1.mfidlab  -side left 
  pack $p25enc.f1.mfidval  -side left 
  pack $p25enc.f1.tgidval  -side right
  pack $p25enc.f1.tgidlab  -side right

  pack $p25enc.f2.naclab  -side left
  pack $p25enc.f2.nacval  -side left 
  pack $p25enc.f2.kidval  -side right 
  pack $p25enc.f2.kidlab  -side right

  pack $p25enc.f3.lcolab  -side left
  pack $p25enc.f3.lcoval  -side left 
  pack $p25enc.f3.didval  -side right 
  pack $p25enc.f3.didlab  -side right

  pack $p25enc.f4.lsdlab  -side left
  pack $p25enc.f4.lsdval  -side left 
  pack $p25enc.f4.sidval  -side right 
  pack $p25enc.f4.sidlab  -side right

  pack $p25enc.f4a.evmlab  -side left
  pack $p25enc.f4a.evmval  -side left 

  pack $p25enc.f5.statlab -side left
  pack $p25enc.f5.stat0   -side left
  pack $p25enc.f5.stat1   -side left
  pack $p25enc.f5.stat2   -side left
  pack $p25enc.f5.stat3   -side left

  pack $p25enc.f6.plab    -side left
  pack $p25enc.f6.p       -side left
  pack $p25enc.f6.dummy -side left -fill both
  pack $p25enc.f6.sflab   -side left
  pack $p25enc.f6.sf      -side left
  pack $p25enc.f6.emg     -side right
  pack $p25enc.f6.emglab  -side right 

  pack $p25enc.f7.enclab    -side left
  pack $p25enc.f7.enc       -side left
  pack $p25enc.f7.b1        -side right -padx 4

#
#  Pack screen (frames)
#
  pack $p25enc.label -side top -fill both -pady 5
  pack $p25enc.f1 -side top -fill x -padx 2
  pack $p25enc.f2 -side top -fill x -padx 2
  pack $p25enc.f3 -side top -fill x -padx 2
  pack $p25enc.f4 -side top -fill x -padx 2
  pack $p25enc.f4a -side top -fill x -padx 2
  pack $p25enc.f5 -side top -fill x -padx 2
  pack $p25enc.f6 -side top -fill x -padx 2
  pack $p25enc.f7 -side top -fill x -padx 2
}

# Validate the MFID Parameter, and send parms
proc validate_mfid { } {
global P25_PARMS
global p25enc

   set result [validate_parm $P25_PARMS(MFID) hex 0xff]
   if { $result == 0 } {
      set P25_PARMS(MFID) 0
   }
   focus $p25enc
   send_p25_parms
}

# Validate the TGID Parameter, and send parms
proc validate_tgid { } {
global P25_PARMS
global p25enc

   set result [validate_parm $P25_PARMS(TGID) hex 0xffff]
   if { $result == 0 } {
      set P25_PARMS(TGID) 0
   }
   focus $p25enc
   send_p25_parms
}

# Validate the NAC Parameter, and send parms
proc validate_nac { } {
global P25_PARMS
global p25enc

   set result [validate_parm $P25_PARMS(NAC) hex 0xfff]
   if { $result == 0 } {
      set P25_PARMS(NAC) 0
   }
   focus $p25enc
   send_p25_parms
}

# Validate the KID Parameter, and send parms
proc validate_kid { } {
global P25_PARMS
global p25enc

   set result [validate_parm $P25_PARMS(KID) hex 0xffff]
   if { $result == 0 } {
      set P25_PARMS(KID) 0
   }
   focus $p25enc
   send_p25_parms
}

# Validate the LCO Parameter, and send parms
proc validate_lco { } {
global P25_PARMS
global p25enc

   set result [validate_parm $P25_PARMS(LCO) dec 63]
   if { $result == 0 } {
      set P25_PARMS(LCO) 0
   }
   focus $p25enc
   send_p25_parms
}

# Validate the DID Parameter, and send parms
proc validate_did { } {
global P25_PARMS
global p25enc

   set result [validate_parm $P25_PARMS(DID) hex 0xffffff]
   if { $result == 0 } {
      set P25_PARMS(DID) 0
   }
   focus $p25enc
   send_p25_parms
}

# Validate the LSD Parameter, and send parms
proc validate_lsd { } {
global P25_PARMS
global p25enc

   set result [validate_parm $P25_PARMS(LSD) hex 0xffffffff]
   if { $result == 0 } {
      set P25_PARMS(LSD) 0
   }
   focus $p25enc
   send_p25_parms
}

# Validate the SID Parameter, and send parms
proc validate_sid { } {
global P25_PARMS
global p25enc

   set result [validate_parm $P25_PARMS(SID) hex 0xffffff]
   if { $result == 0 } {
      set P25_PARMS(SID) 0
   }
   focus $p25enc
   send_p25_parms
}

# Validate the EVM Parameter, and send parms
proc validate_EVM { } {
   global P25_PARMS
   global p25enc

#   set result [validate_parm $P25_PARMS(EVM) dec 1000]
#   if { $result == 0 } {
#      set P25_PARMS(EVM) 0
#   }
   focus $p25enc
   set_P25EVM_level $P25_PARMS(EVM)
}


# Validate the provided Numeric Parameter
#   ENTRY: p - parameter to verify
#          type - hex or dec
#          upper - upper limit to allow
#   EXIT: returns 1=parameter OK, 0=Invalid Parameter
proc validate_parm { p type upper } {

  if { [string compare $type "hex"] == 0 && [string length $p] > 8 } {
    return 0
  }
  for { set i 0 } { [string index $p $i] != "" } { incr i } {
     set c [string index $p $i] 
     if { [string compare $type "hex"] == 0 } {
        if { [ishex $c] == 0 } {
           return 0
        }
     } else {
        if { [isdec $c] == 0 } {
           return 0
        }
     }  
  }
  if { [string compare $type "hex"] == 0 } {
    set temp 0x
    set p [concat $temp$p]
  }
  if { [expr $upper] == -1 && [string length $p] < 11 } {
    return 1
  }
  if { [expr $p] > [expr $upper] } {
    return 0
  }
  return 1
}

# Check provided character for valid hex char
#   ENTRY: c - char to check
#   EXIT: 0=FAIL, 1=OK
proc ishex { c } {
   if { $c < 0 } {
      return 0
   }
   if { $c > 9 } {
     if { [string toupper $c] < "A" } {
       return 0
     }
     if { [string toupper $c] > "F" } {
       return 0
     }
   }
   return 1
}

# Check provided character for valid decimal char
#   ENTRY: c - char to check
#   EXIT: 0=FAIL, 1=OK
proc isdec { c } {
   if { $c < 0 || $c > 9 } {
      return 0
   }
   return 1
}

