#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C)1998,99      DESIGN ENGINEERING      #
# --------------------------------------------------------------------------#
# FILE        : screen.tcl                                                  #
# PRODUCT     : FITS                                                        #
# AUTHOR      : Steve Johnson                                               #
# DESCRIPTION : RF Gen demo screen                                          #
#                                                                           #
#############################################################################

#
# Display state variables
#
set RfGenState(output) GEN
set RfGenState(alc)    SLOW
set RfGenState(bw)     NARROW
set RfGenState(state)  ON
set RfGenState(speed)  SLOW
set RfGenState(p25modemsg) DISABLE


set RFGEN_STATUS(hop_freq1)   99.9
set RFGEN_STATUS(hop_freq2)  100.1
set RFGEN_STATUS(hop_delay)  500
set RFGEN_STATUS(hop_bitmap) 1


#
#  allow a manual reset of all hardware
#
proc reset_rfgen_hw { } {
  global PWT_ENGINE

  set_rfgen_output GEN
  set_rfgen_freq 500.0
  set_rfgen_rflevel -30
  set_rfgen_alc SLOW
  set_rfgen_bw NARROW
  set_rfgen_yigspeed SLOW
  set_rfgen_ptt $PWT_ENGINE(ptt_state)
}



#
# doRfGenFreq
#
proc doRfGenFreq { tg } {
  global RfGenState
  global RFGEN_ENGINE

  label $tg.check -textvariable  RFGEN_ENGINE(OutGoingFreq)  \
                  -font rfgen_font 
  label $tg.msg   -text  "Adj Freq (MHz)"                -font rfgen_font 
  entry $tg.entry -textvariable  RFGEN_ENGINE(RF_Freq)   -width 10 \
                  -font rfgen_font  -bg white


  bind $tg.entry <Return> trace_rfgen_freq


  pack $tg.msg     -side top  -fill both -expand true
  pack $tg.check   -side left -fill both -expand true
  pack $tg.entry   -side left -fill both -expand true
}


#
# doRfGenRFlev
#
proc doRfGenRFlev { tg } {
  global RfGenState

  frame $tg.err

  label $tg.err.msg -text  "ALC Adjust Err:"             -font rfgen_font
  label $tg.err.alc -textvariable  RFGEN_STATUS(alcstat) -font rfgen_font

  pack $tg.err.msg  -side left  -fill both -expand true
  pack $tg.err.alc  -side left  -fill both -expand true

  label $tg.check -textvariable  RFGEN_ENGINE(OutGoingLevel)  \
                  -font rfgen_font
  label $tg.msg   -text  "Adj Input Level (dBm)"         -font rfgen_font 
  entry $tg.entry -textvariable  RFGEN_ENGINE(RF_Level)  -width 5 \
                  -font rfgen_font -bg white


  bind $tg.entry <Return> trace_rfgen_level


  pack $tg.err     -side top  -fill both -expand true
  pack $tg.check   -side top  -fill both -expand true
  pack $tg.msg     -side left -fill both -expand true
  pack $tg.entry   -side left -fill both -expand true
}


#
# doRfGenHopTest
#
proc doRfGenHopTest { tg } {
  global do_rfgen_freq_hop_switch
  global RFGEN_STATUS

  frame $tg.freq1
  frame $tg.freq2
  frame $tg.delay
  frame $tg.hw

  label $tg.freq1.msg   -text  "Freq1 (MHz)" -font rfgen_font 
  entry $tg.freq1.stat  -textvariable  RFGEN_STATUS(hop_freq1) \
                      -font rfgen_font -relief sunken -width 8 -bg white

  label $tg.freq2.msg   -text  "Freq2 (MHz)" -font rfgen_font 
  entry $tg.freq2.stat  -textvariable  RFGEN_STATUS(hop_freq2) \
                      -font rfgen_font -relief sunken -width 8 -bg white

  label $tg.delay.msg   -text  "Delay (mS)"  -font rfgen_font 
  entry $tg.delay.stat  -textvariable  RFGEN_STATUS(hop_delay) \
                      -font rfgen_font -relief sunken -width 5 -bg white

  label $tg.hw.msg      -text  "Test Port Bitmap"  -font rfgen_font 
  entry $tg.hw.stat     -textvariable  RFGEN_STATUS(hop_bitmap) \
                      -font rfgen_font -relief sunken -width 5 -bg white

  checkbutton $tg.state -text "RF Gen Hop Test (On/Off)" \
                      -variable do_rfgen_freq_hop_switch \
                      -font rfgen_font \
                      -command start_rfgen_freq_hop

  bind $tg.hw.stat <Return> { set_rfgen_hopstate $RFGEN_STATUS(hop_bitmap) }

  pack $tg.freq1.msg    -side left  -fill both -expand true
  pack $tg.freq1.stat   -side left  -fill both -expand true

  pack $tg.freq2.msg    -side left  -fill both -expand true
  pack $tg.freq2.stat   -side left  -fill both -expand true

  pack $tg.delay.msg    -side left  -fill both -expand true
  pack $tg.delay.stat   -side left  -fill both -expand true

  pack $tg.hw.msg       -side left  -fill both -expand true
  pack $tg.hw.stat      -side left  -fill both -expand true

  pack $tg.state        -side top  -fill both -expand true
  pack $tg.freq1        -side top  -fill both -expand true
  pack $tg.freq2        -side top  -fill both -expand true
  pack $tg.hw           -side top  -fill both -expand true
  pack $tg.delay        -side top  -fill both -expand true
}




#
# doRfGenPllStatus
#
proc doRfGenPllStatus { tg } {
  global RfGenState
  global RFGEN_STATUS

frame $tg.main  
frame $tg.clean 
frame $tg.step  

label $tg.msg        -text  "PLL Lock Status"  -font rfgen_font

label $tg.main.msg   -text  "Main"             -font rfgen_font 
label $tg.main.stat  -textvariable  RFGEN_STATUS(lock_main) \
                                 -font rfgen_font 
label $tg.clean.msg  -text  "Clean"            -font rfgen_font 
label $tg.clean.stat -textvariable  RFGEN_STATUS(lock_clean) \
                                 -font rfgen_font 
label $tg.step.msg   -text  "Step"             -font rfgen_font 
label $tg.step.stat  -textvariable  RFGEN_STATUS(lock_step) \
                                 -font rfgen_font 

  pack $tg.main.msg    -side top  -fill both -expand true
  pack $tg.main.stat   -side top  -fill both -expand true

  pack $tg.clean.msg   -side top  -fill both -expand true
  pack $tg.clean.stat  -side top  -fill both -expand true

  pack $tg.step.msg    -side top  -fill both -expand true
  pack $tg.step.stat   -side top  -fill both -expand true 

  pack $tg.msg     -side top  -fill both -expand true

  pack $tg.main    -side left -fill both -expand true
  pack $tg.clean   -side left -fill both -expand true
  pack $tg.step    -side left -fill both -expand true
}


#
# doRfGenYigStatus
#
proc doRfGenYigStatus { tg } {
  global RfGenState
  global RFGEN_STATUS

frame $tg.hold
frame $tg.dac
frame $tg.padetect

label $tg.hold.msg   -text  "Yig Hold"         -font rfgen_font 
label $tg.hold.stat  -textvariable  RFGEN_STATUS(yig_hold) \
                     -font rfgen_font

label $tg.dac.msg    -text  "Yig DAC"          -font rfgen_font 
label $tg.dac.stat   -textvariable  RFGEN_STATUS(yig_dac) \
                     -font rfgen_font

label $tg.padetect.msg  -text  "Pa Detect"     -font rfgen_font 
label $tg.padetect.stat -textvariable  RFGEN_STATUS(yig_padetect) \
                        -font rfgen_font


  pack $tg.hold.msg       -side top  -fill both -expand true
  pack $tg.hold.stat      -side top  -fill both -expand true

  pack $tg.dac.msg        -side top  -fill both -expand true
  pack $tg.dac.stat       -side top  -fill both -expand true

  pack $tg.padetect.msg   -side top  -fill both -expand true
  pack $tg.padetect.stat  -side top  -fill both -expand true


  pack $tg.hold     -side left  -fill both -expand true
  pack $tg.dac      -side left  -fill both -expand true
  pack $tg.padetect -side left  -fill both -expand true
}


#
# doRfGenYigCal
#
proc doRfGenYigCal { tg } {
  global RcvrState
  global HwUtilEngine
  global RfGenYigCals

  label $tg.msg    -text  "Yig Cal Values:"  -font rfgen_font 
  entry $tg.entry1 -textvariable  RfGenYigCals(0) \
                 -width 5 -font rfgen_font  -bg white
  entry $tg.entry2 -textvariable  RfGenYigCals(2800) \
                 -width 5 -font rfgen_font  -bg white

  button $tg.button   -text "Do Yig Calibration" -font rfgen_font \
                      -command { set_start_yig_calibration }
  label  $tg.entry3   -textvariable  HwUtilEngine(yigcal_status) \
                      -width 5 -font rfgen_font  -bg white

  bind $tg.entry1 <Return> update_rfgen_yig_cal_data
  bind $tg.entry2 <Return> update_rfgen_yig_cal_data

  pack $tg.button  -side top  -fill both -expand true
  pack $tg.entry3  -side top  -fill both -expand true

  pack $tg.msg     -side top  -fill both -expand true
  pack $tg.entry1  -side left -fill both -expand true
  pack $tg.entry2  -side left -fill both -expand true
}

#############################################################################
#                                                                           #
#                       Cal Screen ALC Curve                                #
#                                                                           #
#############################################################################

#
#   build_gencal_alc_screen
#
proc build_gencal_alc_screen { name } {
  global bgass
  global RFGEN_ENGINE
  global GENCAL

set bgass $name

frame $bgass.ref
frame $bgass.max
frame $bgass.lvl
frame $bgass.lvl.index -relief groove
frame $bgass.lvl.dbm   -relief groove
frame $bgass.lvl.alc   -relief groove
frame $bgass.hw

#
#  Read/Write Data Files
#
set RFGEN_ENGINE(Filename) rfgen_alc

button     $bgass.readcals -text "Read Calibration File" \
                -command { gencal_alc_read_cal $RFGEN_ENGINE(Filename) }
button     $bgass.savecals -text "Save Calibration File" \
                -command { gencal_alc_save_cal $RFGEN_ENGINE(Filename) }
button     $bgass.enginecals -text "Send Cals to Engine" \
                -command { gencal_set_engine_level }

#
#  Reference Widget
#
label $bgass.ref.label -text "Reference Level (0 dB)"
entry $bgass.ref.freq  -relief sunken  -width 8  -bg white \
                       -textvariable RFGEN_ENGINE(Normalize.Genalc)


#
#  Data Entry Stuff
#
label $bgass.lvl.index.label \
                -text "Index (0 - Max-1)"
entry $bgass.lvl.index.entry -relief sunken  -width 8 -bg white \
                -textvariable RFGEN_ENGINE(AlcIndex)

label $bgass.lvl.dbm.label \
                -text "RF Lvl (dBm)"
entry $bgass.lvl.dbm.entry  -relief sunken  -width 8 -bg white \
                -textvariable GENCAL(level,abs,$RFGEN_ENGINE(AlcIndex))

label $bgass.lvl.alc.label \
                -text "ALC DAC ( 0 - 4095)"
entry $bgass.lvl.alc.entry  -relief sunken  -width 8 -bg white \
                -textvariable GENCAL(level,alc,$RFGEN_ENGINE(AlcIndex))

label $bgass.lvl.label \
                    -text $GENCAL(level,status,$RFGEN_ENGINE(AlcIndex))

bind  $bgass.lvl.alc.entry  <Return> \
           { gencal_Vlevel $GENCAL(level,alc,$RFGEN_ENGINE(AlcIndex)) }


# changing indexs requires much grinding to be sensible for new index
bind $bgass.lvl.index.entry <Return> {
  $bgass.lvl.dbm.entry  config \
                -textvariable GENCAL(level,abs,$RFGEN_ENGINE(AlcIndex))
  $bgass.lvl.alc.entry config \
                -textvariable GENCAL(level,alc,$RFGEN_ENGINE(AlcIndex))
  $bgass.lvl.label config \
                    -text $GENCAL(level,status,$RFGEN_ENGINE(AlcIndex))
  bind $bgass.lvl.alc.entry <Return> {}

  bind $bgass.lvl.alc.entry <Return> \
           { gencal_Vlevel $GENCAL(level,alc,$RFGEN_ENGINE(AlcIndex)) }

  gencal_Vlevel $GENCAL(level,alc,$RFGEN_ENGINE(AlcIndex))
}

bind  $bgass.lvl.dbm.entry <Return> \
                         { gencal_alc_capture $RFGEN_ENGINE(AlcIndex) }


#
#  (Re) Set Array Boundary
#
label $bgass.max.label -text "Max Array Elements"
entry $bgass.max.value   -relief sunken  -width 8 -bg white \
                -textvariable RFGEN_ENGINE(AlcMaxIndex)

bind  $bgass.max.value <Return> { gencal_alc_set_max }


#
#  Control Step attenuator
#
scale $bgass.hw.adj -label "Step Atten"  -from 0 -to 110 -resolution 10 \
        -orient horizontal -command gencal_Astep

button $bgass.hw.reset  -text "Reset PT" -command set_pt_reset

#
#  sub-frames
#
pack $bgass.lvl.index.label -side top
pack $bgass.lvl.index.entry -side top

pack $bgass.lvl.dbm.label   -side top
pack $bgass.lvl.dbm.entry   -side top

pack $bgass.lvl.alc.label   -side top
pack $bgass.lvl.alc.entry   -side top


#
#  max elements
#
pack $bgass.max.label  -side top
pack $bgass.max.value  -side top

#
#  level entry stuff
#
pack $bgass.lvl.index  -side left
pack $bgass.lvl.alc    -side left
pack $bgass.lvl.dbm    -side left
pack $bgass.lvl.label  -side left


#
#  Reference Frequency
#
pack $bgass.ref.label  -side top
pack $bgass.ref.freq   -side top

pack $bgass.hw.adj     -side left
pack $bgass.hw.reset   -side left


#
#  Pack screen
#
pack $bgass.lvl        -side top
pack $bgass.hw         -side top
pack $bgass.max        -side top
pack $bgass.ref        -side top
pack $bgass.savecals   -side top
pack $bgass.readcals   -side top
pack $bgass.enginecals -side top
}



#############################################################################
#                                                                           #
#                       Cal Screen FREQ Curve                               #
#                                                                           #
#############################################################################


proc doRfGenFreqTrCurve { name } { 
  gencal_port TR
  build_gencal_freq_screen $name tr
}

proc doRfGenFreqGenCurve { name } {
  gencal_port GEN 
  build_gencal_freq_screen $name gen
}


#
#   build_gencal_freq_screen
#
proc build_gencal_freq_screen { name rfport } {
  global bgfss
  global RFGEN_ENGINE
  global GENCAL

set bgfss $name

frame $bgfss.ref
frame $bgfss.ref.a0    -relief groove
frame $bgfss.ref.a10   -relief groove
frame $bgfss.ref.a20   -relief groove
frame $bgfss.max
frame $bgfss.lvl
frame $bgfss.lvl.index -relief groove
frame $bgfss.lvl.freq  -relief groove
frame $bgfss.lvl.a0    -relief groove
frame $bgfss.lvl.a10   -relief groove
frame $bgfss.lvl.a20   -relief groove
frame $bgfss.hw


set RFGEN_ENGINE(Filename) rfgen_freq_$rfport

#
#  Read/Write Data Files
#
button  $bgfss.readcals -text "Read Calibration File" \
 -command { gencal_freq_read_cal $RFGEN_ENGINE(Filename) $RFGEN_ENGINE(Port) }
button  $bgfss.savecals -text "Save Calibration File" \
 -command { gencal_freq_save_cal $RFGEN_ENGINE(Filename) $RFGEN_ENGINE(Port) }
button  $bgfss.enginecals -text "Send Cals to Engine" \
 -command { gencal_set_engine_freq $RFGEN_ENGINE(Port) }

#
#  Reference Widget
#
label $bgfss.ref.a0.label -text "Ref Lvl (0 dB)"
entry $bgfss.ref.a0.freq  -relief sunken  -width 8 -bg white \
                       -textvariable RFGEN_ENGINE(Normalize.Genfreq.dB0)

label $bgfss.ref.a10.label -text "Ref Lvl (10 dB)"
entry $bgfss.ref.a10.freq  -relief sunken  -width 8 -bg white \
                       -textvariable RFGEN_ENGINE(Normalize.Genfreq.dB10)

label $bgfss.ref.a20.label -text "Ref Lvl (20 dB)"
entry $bgfss.ref.a20.freq  -relief sunken  -width 8 -bg white \
                       -textvariable RFGEN_ENGINE(Normalize.Genfreq.dB20)


#
#  Data Entry Stuff
#
label $bgfss.lvl.index.label \
   -text "Index (0 - Max-1)"
entry $bgfss.lvl.index.entry -relief sunken -width 8 -bg white \
   -textvariable RFGEN_ENGINE(FreqIndex)

label $bgfss.lvl.freq.label \
   -text "Freq (MHz)"
entry $bgfss.lvl.freq.entry -relief sunken  -width 8 -bg white \
   -textvariable GENCAL(freq,$RFGEN_ENGINE(Port),freq,$RFGEN_ENGINE(FreqIndex))

bind  $bgfss.lvl.freq.entry <Return> { gencal_freq \
    $GENCAL(freq,$RFGEN_ENGINE(Port),freq,$RFGEN_ENGINE(FreqIndex)) }

label $bgfss.lvl.a0.label \
   -text "Step 0 dB"
entry $bgfss.lvl.a0.entry -relief sunken  -width 8 -bg white \
   -textvariable GENCAL(freq,$RFGEN_ENGINE(Port),dB0,abs,$RFGEN_ENGINE(FreqIndex))

label $bgfss.lvl.a10.label \
   -text "Step 10 dB"
entry $bgfss.lvl.a10.entry  -relief sunken  -width 8 -bg white \
   -textvariable GENCAL(freq,$RFGEN_ENGINE(Port),dB10,abs,$RFGEN_ENGINE(FreqIndex))

label $bgfss.lvl.a20.label \
   -text "Step 20 dB"
entry $bgfss.lvl.a20.entry  -relief sunken  -width 8 -bg white \
   -textvariable GENCAL(freq,$RFGEN_ENGINE(Port),dB20,abs,$RFGEN_ENGINE(FreqIndex))

# changing indexs requires much grinding to be sensible for new index
bind $bgfss.lvl.index.entry <Return> {
  $bgfss.lvl.freq.entry config \
  -textvariable GENCAL(freq,$RFGEN_ENGINE(Port),freq,$RFGEN_ENGINE(FreqIndex))
  $bgfss.lvl.a0.entry   config \
  -textvariable GENCAL(freq,$RFGEN_ENGINE(Port),dB0,abs,$RFGEN_ENGINE(FreqIndex))
  $bgfss.lvl.a10.entry  config \
  -textvariable GENCAL(freq,$RFGEN_ENGINE(Port),dB10,abs,$RFGEN_ENGINE(FreqIndex))
  $bgfss.lvl.a20.entry  config \
  -textvariable GENCAL(freq,$RFGEN_ENGINE(Port),dB20,abs,$RFGEN_ENGINE(FreqIndex))
}


#
#  (Re) Set Array Boundary
#
label $bgfss.max.label -text "Max Array Elements"
entry $bgfss.max.value   -relief sunken  -width 8 -bg white \
                -textvariable RFGEN_ENGINE(FreqMaxIndex)

bind  $bgfss.max.value <Return> { gencal_freq_set_max $RFGEN_ENGINE(Port) }


#
#  Control Step attenuator
#
scale $bgfss.hw.step -label "Step Atten"  -from 0 -to 110 -resolution 10 \
        -orient horizontal -command gencal_Astep

#
#  Control ALC DAC
#
scale $bgfss.hw.alcdac -label "ALC DAC"  -from 0 -to 4095 \
        -orient horizontal -command gencal_Vlevel


button $bgfss.hw.reset  -text "Reset PT" -command set_pt_reset

#
#  sub-frames
#
pack $bgfss.lvl.index.label -side top
pack $bgfss.lvl.index.entry -side top

pack $bgfss.lvl.freq.label  -side top
pack $bgfss.lvl.freq.entry  -side top

pack $bgfss.lvl.a0.label   -side top
pack $bgfss.lvl.a0.entry   -side top

pack $bgfss.lvl.a10.label  -side top
pack $bgfss.lvl.a10.entry  -side top

pack $bgfss.lvl.a20.label  -side top
pack $bgfss.lvl.a20.entry  -side top


pack $bgfss.ref.a0.label   -side top
pack $bgfss.ref.a0.freq    -side top

pack $bgfss.ref.a10.label  -side top
pack $bgfss.ref.a10.freq   -side top

pack $bgfss.ref.a20.label  -side top
pack $bgfss.ref.a20.freq   -side top



#
#  max elements
#
pack $bgfss.max.label  -side top
pack $bgfss.max.value  -side top

#
#  freq entry stuff
#
pack $bgfss.lvl.index  -side left
pack $bgfss.lvl.freq   -side left

pack $bgfss.lvl.a0     -side left
pack $bgfss.lvl.a10    -side left
pack $bgfss.lvl.a20    -side left

pack $bgfss.ref.a0     -side left
pack $bgfss.ref.a10    -side left
pack $bgfss.ref.a20    -side left

pack $bgfss.hw.step    -side left
pack $bgfss.hw.alcdac  -side left
pack $bgfss.hw.reset   -side left


#
#  Pack screen
#
pack $bgfss.lvl        -side top
pack $bgfss.hw         -side top
pack $bgfss.max        -side top
pack $bgfss.ref        -side top
pack $bgfss.savecals   -side top
pack $bgfss.readcals   -side top
pack $bgfss.enginecals -side top
}






#############################################################################
#                                                                           #
#                       build RF Gen screen                                 #
#                                                                           #
#############################################################################
proc build_rfgen_screen { name } {
  global rfgrn
  global DemoColor

  # we have to save $rfgrn because the -command {} will not rememver
  # local instance of $name
  set rfgrn $name

  #
  # define screen frames
  #
  frame $rfgrn.mbar          -relief raised -bd 2
  frame $rfgrn.locksummary   -relief raised -bd 2


  #
  # PLL lock summary
  #
  label $rfgrn.locksummary.stat  -textvariable  RFGEN_STATUS(lock_summary) \
                                -font rfgen_font



  #
  #  Define menu bars  
  #
  message $name.mbar.label      -text "RF Gen" -font rfgen_font \
                                -bg $DemoColor(title) \
                                -width $DemoColor(labelwidth)

  menubutton $rfgrn.mbar.gen    -text  "FrontEnd"  -font rfgen_font \
                                -menu $rfgrn.mbar.gen.menu \
                                -bg $DemoColor(menu) -relief raised
  menubutton $rfgrn.mbar.mod    -text  "Modulation"  -font rfgen_font \
                                -menu $rfgrn.mbar.mod.menu \
                                -bg $DemoColor(menu) -relief raised
  menubutton $rfgrn.mbar.status -text  "Status"  -font rfgen_font \
                                -menu $rfgrn.mbar.status.menu \
                                -bg $DemoColor(menu) -relief raised
  menubutton $rfgrn.mbar.yig    -text  "Yig"  -font rfgen_font \
                                -menu $rfgrn.mbar.yig.menu \
                                -bg $DemoColor(menu) -relief raised
  menubutton $rfgrn.mbar.cal    -text  "Calibration"  -font rfgen_font \
                                -menu $rfgrn.mbar.cal.menu \
                                -bg $DemoColor(menu) -relief raised

  #
  #  Build RF Gen menu 
  #
  menu $rfgrn.mbar.gen.menu
  $rfgrn.mbar.gen.menu add cascade \
              -label "DC Power" -font rfgen_font \
              -menu $rfgrn.mbar.gen.menu.state
    menu  $rfgrn.mbar.gen.menu.state
    $rfgrn.mbar.gen.menu.state add radiobutton \
                             -label "On (re-run yig cal)" -font rfgen_font \
                             -command { set_rfgen_onoff $RfGenState(state) } \
                             -variable RfGenState(state) \
                             -value ON
    $rfgrn.mbar.gen.menu.state add radiobutton -label "Off" -font rfgen_font \
                             -command { set_rfgen_onoff $RfGenState(state) } \
                             -variable RfGenState(state) \
                             -value OFF
 
  $rfgrn.mbar.gen.menu add cascade \
              -label "Output" -font rfgen_font \
              -menu $rfgrn.mbar.gen.menu.output
    menu $rfgrn.mbar.gen.menu.output
    $rfgrn.mbar.gen.menu.output add radiobutton -label "T/R" -font rfgen_font \
                   -command { set_rfgen_output $RfGenState(output) } \
                   -variable RfGenState(output) \
                   -value TR
    $rfgrn.mbar.gen.menu.output add radiobutton -label "Gen" -font rfgen_font \
                   -command { set_rfgen_output $RfGenState(output) } \
                   -variable RfGenState(output) \
                   -value GEN
  $rfgrn.mbar.gen.menu add command \
              -label "RF Freq" -font rfgen_font \
              -command { WidgetWindow doRfGenFreq \
                         $rfgrn.gen_freq $rfgrn "RF Generator frequency" }
  $rfgrn.mbar.gen.menu add command \
              -label "RF Level" -font rfgen_font \
              -command { WidgetWindow doRfGenRFlev \
                         $rfgrn.gen_level $rfgrn "RF Generator level" }
  $rfgrn.mbar.gen.menu add command \
              -label "Synth Hop Test" -font rfgen_font \
              -command { WidgetWindow doRfGenHopTest \
                         $rfgrn.gen_hoptest $rfgrn "RF Generator hop test" }

  #
  #  Build RF Gen YIG menu 
  #
  menu $rfgrn.mbar.yig.menu
  $rfgrn.mbar.yig.menu add cascade \
              -label "PLL" -font rfgen_font \
              -menu $rfgrn.mbar.yig.menu.pll
    menu $rfgrn.mbar.yig.menu.pll
    $rfgrn.mbar.yig.menu.pll add radiobutton -label "Slow" -font rfgen_font \
                    -command { set_rfgen_alc $RfGenState(alc) } \
                    -variable RfGenState(alc) \
                    -value SLOW
    $rfgrn.mbar.yig.menu.pll add radiobutton -label "Fast" -font rfgen_font \
                     -command { set_rfgen_alc $RfGenState(alc) } \
                     -variable RfGenState(alc) \
                     -value FAST
    $rfgrn.mbar.yig.menu.pll add radiobutton -label "Digital" -font rfgen_font \
                     -command { set_rfgen_alc $RfGenState(alc) } \
                     -variable RfGenState(alc) \
                     -value DIGITAL

  $rfgrn.mbar.yig.menu add cascade \
              -label "Bandwidth" -font rfgen_font \
              -menu $rfgrn.mbar.yig.menu.bw
    menu $rfgrn.mbar.yig.menu.bw
    $rfgrn.mbar.yig.menu.bw add radiobutton -label "Narrow" -font rfgen_font \
       -command { set_rfgen_bw $RfGenState(bw) } \
       -variable RfGenState(bw) \
       -value NARROW
    $rfgrn.mbar.yig.menu.bw add radiobutton -label "Wide" -font rfgen_font \
       -command { set_rfgen_bw $RfGenState(bw) } \
       -variable RfGenState(bw) \
       -value WIDE

  $rfgrn.mbar.yig.menu add cascade \
              -label "Speed" -font rfgen_font \
              -menu $rfgrn.mbar.yig.menu.speed
    menu $rfgrn.mbar.yig.menu.speed
    $rfgrn.mbar.yig.menu.speed add radiobutton -label "Slow" -font rfgen_font \
       -command { set_rfgen_yigspeed $RfGenState(speed) } \
       -variable RfGenState(speed) \
       -value SLOW
    $rfgrn.mbar.yig.menu.speed add radiobutton -label "Fast" -font rfgen_font \
       -command { set_rfgen_yigspeed $RfGenState(speed) } \
       -variable RfGenState(speed) \
       -value FAST

  #
  #  Build RF Gen Modulation menu 
  #
  menu $rfgrn.mbar.mod.menu
  $rfgrn.mbar.mod.menu add cascade \
              -label "Type" -font rfgen_font \
              -menu $rfgrn.mbar.mod.menu.type
    menu $rfgrn.mbar.mod.menu.type
    $rfgrn.mbar.mod.menu.type add radiobutton -label "OFF" -font rfgen_font \
                         -command { set_rfgen_modulation $RfGenState(mod) } \
                         -variable RfGenState(mod) \
                         -value OFF
    $rfgrn.mbar.mod.menu.type add radiobutton -label "FM Narrow" -font rfgen_font \
                         -command { set_rfgen_modulation $RfGenState(mod) } \
                         -variable RfGenState(mod) \
                         -value FMN
    $rfgrn.mbar.mod.menu.type add radiobutton -label "FM Wide" -font rfgen_font \
                         -command { set_rfgen_modulation $RfGenState(mod) } \
                         -variable RfGenState(mod) \
                         -value FMW
    $rfgrn.mbar.mod.menu.type add radiobutton -label "PM" -font rfgen_font \
                         -command { set_rfgen_modulation $RfGenState(mod) } \
                         -variable RfGenState(mod) \
                         -value PM
    $rfgrn.mbar.mod.menu.type add radiobutton -label "AM" -font rfgen_font \
                         -command { set_rfgen_modulation $RfGenState(mod) } \
                         -variable RfGenState(mod) \
                         -value AM
    $rfgrn.mbar.mod.menu.type add radiobutton -label "PSK" -font rfgen_font \
                         -command { set_rfgen_modulation $RfGenState(mod) } \
                         -variable RfGenState(mod) \
                         -value PSK
    $rfgrn.mbar.mod.menu.type add radiobutton -label "USB" -font rfgen_font \
                         -command { set_rfgen_modulation $RfGenState(mod) } \
                         -variable RfGenState(mod) \
                         -value USB
    $rfgrn.mbar.mod.menu.type add radiobutton -label "LSB" -font rfgen_font \
                         -command { set_rfgen_modulation $RfGenState(mod) } \
                         -variable RfGenState(mod) \
                         -value LSB
    $rfgrn.mbar.mod.menu.type add radiobutton -label "P25 MIS" -font rfgen_font \
                         -command { set_rfgen_modulation $RfGenState(mod) } \
                         -variable RfGenState(mod) \
                         -value P25MIS
    $rfgrn.mbar.mod.menu.type add radiobutton -label "P25 MCC" -font rfgen_font \
                         -command { set_rfgen_modulation $RfGenState(mod) } \
                         -variable RfGenState(mod) \
                         -value P25MCC
    $rfgrn.mbar.mod.menu.type add radiobutton -label "P25 Std Xmt Test" -font rfgen_font \
                         -command { set_rfgen_modulation $RfGenState(mod) } \
                         -variable RfGenState(mod) \
                         -value P25STDMSG

  $rfgrn.mbar.mod.menu add cascade \
              -label "P25 Modes" -font rfgen_font \
              -menu $rfgrn.mbar.mod.menu.p25

    menu $rfgrn.mbar.mod.menu.p25

    $rfgrn.mbar.mod.menu.p25 add radiobutton -label "OFF" -font rfgen_font \
       -command { set_rfgen_p25mode $RfGenState(p25mode) } \
       -variable RfGenState(p25mode) \
       -value OFF
    $rfgrn.mbar.mod.menu.p25 add radiobutton -label "Speech" -font rfgen_font \
       -command { set_rfgen_p25mode $RfGenState(p25mode) } \
       -variable RfGenState(p25mode) \
       -value SPEECH
    $rfgrn.mbar.mod.menu.p25 add radiobutton -label "Silence" -font rfgen_font \
       -command { set_rfgen_p25mode $RfGenState(p25mode) } \
       -variable RfGenState(p25mode) \
       -value SILENCE
    $rfgrn.mbar.mod.menu.p25 add radiobutton -label "5% Error" -font rfgen_font \
       -command { set_rfgen_p25mode $RfGenState(p25mode) } \
       -variable RfGenState(p25mode) \
       -value ERR5PERCENT
    $rfgrn.mbar.mod.menu.p25 add radiobutton -label "1011Hz Tone" -font rfgen_font \
       -command { set_rfgen_p25mode $RfGenState(p25mode) } \
       -variable RfGenState(p25mode) \
       -value HZ1011
    $rfgrn.mbar.mod.menu.p25 add radiobutton -label "VxWorks File" -font rfgen_font \
       -command { set_rfgen_p25mode $RfGenState(p25mode) } \
       -variable RfGenState(p25mode) \
       -value FILE
    $rfgrn.mbar.mod.menu.p25 add cascade \
              -label "Standard Patterns" -font rfgen_font \
              -menu $rfgrn.mbar.mod.menu.p25.std_patterns

    menu $rfgrn.mbar.mod.menu.p25.std_patterns

    $rfgrn.mbar.mod.menu.p25.std_patterns add radiobutton -label "STD Tone" -font rfgen_font \
       -command { set_rfgen_p25mode $RfGenState(p25mode) } \
       -variable RfGenState(p25mode) \
       -value STD_TONE
    $rfgrn.mbar.mod.menu.p25.std_patterns add radiobutton -label "STD Silence" -font rfgen_font \
       -command { set_rfgen_p25mode $RfGenState(p25mode) } \
       -variable RfGenState(p25mode) \
       -value STD_SILENCE
    $rfgrn.mbar.mod.menu.p25.std_patterns add radiobutton -label "STD Interference" -font rfgen_font \
       -command { set_rfgen_p25mode $RfGenState(p25mode) } \
       -variable RfGenState(p25mode) \
       -value STD_INTERFER
    $rfgrn.mbar.mod.menu.p25.std_patterns add radiobutton -label "STD Busy" -font rfgen_font \
       -command { set_rfgen_p25mode $RfGenState(p25mode) } \
       -variable RfGenState(p25mode) \
       -value STD_BUSY
    $rfgrn.mbar.mod.menu.p25.std_patterns add radiobutton -label "STD Idle" -font rfgen_font \
       -command { set_rfgen_p25mode $RfGenState(p25mode) } \
       -variable RfGenState(p25mode) \
       -value STD_IDLE
    $rfgrn.mbar.mod.menu.p25.std_patterns add radiobutton -label "STD Calibration" -font rfgen_font \
       -command { set_rfgen_p25mode $RfGenState(p25mode) } \
       -variable RfGenState(p25mode) \
       -value STD_CAL
    $rfgrn.mbar.mod.menu.p25.std_patterns add radiobutton -label "STD LUD #1 Trigger" -font rfgen_font \
       -command { set_rfgen_p25mode $RfGenState(p25mode) } \
       -variable RfGenState(p25mode) \
       -value STD_LUD1TRIG
    $rfgrn.mbar.mod.menu.p25.std_patterns add radiobutton -label "STD LUD #2 Trigger" -font rfgen_font \
       -command { set_rfgen_p25mode $RfGenState(p25mode) } \
       -variable RfGenState(p25mode) \
       -value STD_LUD2TRIG
    $rfgrn.mbar.mod.menu.p25.std_patterns add radiobutton -label "STD No Trigger" -font rfgen_font \
       -command { set_rfgen_p25mode $RfGenState(p25mode) } \
       -variable RfGenState(p25mode) \
       -value STD_NOTRIG
    $rfgrn.mbar.mod.menu.p25.std_patterns add radiobutton -label "STD 511" -font rfgen_font \
       -command { set_rfgen_p25mode $RfGenState(p25mode) } \
       -variable RfGenState(p25mode) \
       -value STD_511
    $rfgrn.mbar.mod.menu.p25.std_patterns add radiobutton -label "STD Symbol Rate" -font rfgen_font \
       -command { set_rfgen_p25mode $RfGenState(p25mode) } \
       -variable RfGenState(p25mode) \
       -value STD_SYM_RATE
    $rfgrn.mbar.mod.menu.p25.std_patterns add radiobutton -label "STD Low Dev" -font rfgen_font \
       -command { set_rfgen_p25mode $RfGenState(p25mode) } \
       -variable RfGenState(p25mode) \
       -value STD_LOW_DEV
    $rfgrn.mbar.mod.menu.p25.std_patterns add radiobutton -label "STD Fidelity Pattern" -font rfgen_font \
       -command { set_rfgen_p25mode $RfGenState(p25mode) } \
       -variable RfGenState(p25mode) \
       -value STD_FID_PAT
    $rfgrn.mbar.mod.menu.p25.std_patterns add radiobutton -label "STD Fidelity Spectrum" -font rfgen_font \
       -command { set_rfgen_p25mode $RfGenState(p25mode) } \
       -variable RfGenState(p25mode) \
       -value STD_FID_SPECT


   # p25 ENCODE PARAMETERS
   $rfgrn.mbar.mod.menu add command \
              -label "P25 Parameters" -font rfgen_font \
              -command { WidgetWindow doP25parms \
                         $rfgrn.mod_p25 $rfgrn "P25 Protocol Encode"}

  #
  #  Build RF Gen Status menu 
  #
  menu $rfgrn.mbar.status.menu
  $rfgrn.mbar.status.menu add command \
              -label "PLL" -font rfgen_font \
              -command { WidgetWindow doRfGenPllStatus \
                         $rfgrn.pll_status $rfgrn "RF Gen PLL status" }
  $rfgrn.mbar.status.menu add command \
              -label "YIG" -font rfgen_font \
              -command { WidgetWindow doRfGenYigStatus \
                         $rfgrn.yig_status $rfgrn "RF GEN YIG status" }

  #
  #  Build RF Gen Calibration menu 
  #
  menu $rfgrn.mbar.cal.menu
  $rfgrn.mbar.cal.menu add command \
              -label "Yig" -font rfgen_font \
              -command { WidgetWindow doRfGenYigCal \
                         $rfgrn.cal_yig $rfgrn "RF Gen YIG cal"}
  $rfgrn.mbar.cal.menu add command \
              -label "ALC Curve" -font rfgen_font \
              -command { WidgetWindow build_gencal_alc_screen \
                         $rfgrn.cal_alc $rfgrn "RF Gen ALC curve" }
  $rfgrn.mbar.cal.menu add command \
              -label "T/R Freq Curve" -font rfgen_font \
              -command { WidgetWindow doRfGenFreqTrCurve \
                         $rfgrn.cal_tr $rfgrn "RF Gen T/R freq. response" }
  $rfgrn.mbar.cal.menu add command \
              -label "Gen Freq Curve" -font rfgen_font \
              -command { WidgetWindow doRfGenFreqGenCurve \
                         $rfgrn.cal_gen $rfgrn "RF Gen port freq. response" }

  checkbutton $rfgrn.mbar.ptt   -text "PTT"  -offvalue CLR -onvalue SET \
                                -font rfgen_font -bg $DemoColor(menu) \
                                -variable PWT_ENGINE(ptt_state) \
                                -command { set_rfgen_ptt $PWT_ENGINE(ptt_state) }



  #
  #  pack sub frames
  #
  pack $name.mbar.label         -side left -fill both

  pack $rfgrn.mbar.gen          -side left -fill both -expand true
  pack $rfgrn.mbar.yig          -side left -fill both -expand true
  pack $rfgrn.mbar.ptt          -side left -fill both -expand true
  pack $rfgrn.mbar.mod          -side left -fill both -expand true
  pack $rfgrn.mbar.cal          -side left -fill both -expand true
  pack $rfgrn.mbar.status       -side left -fill both -expand true

  pack $rfgrn.locksummary.stat  -side left -fill both


  #
  #  pack main frame
  #
  pack $rfgrn.mbar              -side left  -fill both -expand true
  pack $rfgrn.locksummary       -side right -fill both

  return $rfgrn
}

