#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C)1998,99      DESIGN ENGINEERING      #
# --------------------------------------------------------------------------#
# FILE        : to_p25e.tcl                                                 #
# PRODUCT     : FITS                                                        #
# AUTHOR      : Steve White                                                 #
# DESCRIPTION : Interface to P-25 Encode to Msg Bus                         #
#                                                                           #
#############################################################################

#
#  P-25 Encode system Number
#
set P25ENC_ENGINE(SYSTEM_ID)         20

#
#  P-25 Encode Sub-Systems
#
set P25ENC_ENGINE(PTT_STATE)            1
set P25ENC_ENGINE(P25_PARMS)            2
set P25ENC_ENGINE(P25_ENCRYPT)          3
set P25ENC_ENGINE(P25ENC_INFO)          4
set P25ENC_ENGINE(P25EVM_GEN)           5

# Turn On/Off Encryption state
proc enc_state { } {
global P25_PARMS
global P25ENC_ENGINE

   message_bus_int $P25ENC_ENGINE(SYSTEM_ID) $P25ENC_ENGINE(P25_ENCRYPT) 1 \
                   $P25_PARMS(encrypt)
}

#
# Set level of EVM
#
proc set_P25EVM_level { level } {
   global P25ENC_ENGINE

   message_bus_double $P25ENC_ENGINE(SYSTEM_ID) $P25ENC_ENGINE(P25EVM_GEN) 1 \
                   $level
}


#Send the P-25 Encode Parameters to the Layer-1 DSP
# NOTE: Some of the global parms are actually Hex #'s, but
#       dont have the "0x" in front of them.
#
proc send_p25_parms { } {
global P25_PARMS
global P25ENC_ENGINE

  set hex "0x"
  set NAC [concat $hex$P25_PARMS(NAC)]
  set NAC [expr $NAC]
  set MFID [concat $hex$P25_PARMS(MFID)]
  set MFID [expr $MFID]
  set TGID [concat $hex$P25_PARMS(TGID)]
  set TGID [expr $TGID]
  set LSD [format_lsd $P25_PARMS(LSD)]
  set LSD [concat $hex$LSD]
  set LSD2 [expr $LSD & 0xffff]
  set LSD1 [expr {$LSD >> 16} & 0xffff]
  set LCO $P25_PARMS(LCO)
  set DID [concat $hex$P25_PARMS(DID)]
  set DID [expr $DID]
  set SID [concat $hex$P25_PARMS(SID)]
  set SID [expr $SID]
  set EMG $P25_PARMS(EMG)
  set SF $P25_PARMS(SF)
  set P $P25_PARMS(P)
  set KID [concat $hex$P25_PARMS(KID)]
  set KID [expr $KID]
  set STATUS $P25_PARMS(STATUS)

  set var [list $NAC $MFID $TGID $LSD1 $LSD2 $LCO $DID $SID $EMG $SF $P $KID $STATUS ]
  tclMsgbus send "{ $P25ENC_ENGINE(SYSTEM_ID) $P25ENC_ENGINE(P25_PARMS) 1 1 } \
                  { ptr UDWORD $var }"

  set_P25EVM_level $P25_PARMS(EVM)
}

#Format the LSD string (hex w/o "0x") to read the hex digits
#from most sig to leas sig
#   ENTRY: Hex digit string without preceeding "0x"
#   EXIT: formated hex digit string without preceeding "0x"
proc format_lsd { lsd } {

   set val 0
   set hex 0x

   for { set i 0 } { $i < 8 } { incr i} {
     set val [expr $val << 4]
     set mask 0
     if { [string index $lsd $i] != "" } {
       set mask [string index $lsd $i]
       if { [ishex $mask] > 0 } {
          set mask [concat $hex$mask]
          set mask [expr $mask]
       }
     }
     set val [expr $val | $mask]
   }
   return [format "%x" $val]
}

