#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C)1998,99      DESIGN ENGINEERING      #
# --------------------------------------------------------------------------#
# FILE        : to_rfgen.tcl                                                #
# PRODUCT     : FITS                                                        #
# AUTHOR      : Steve Johnson                                               #
# DESCRIPTION : Interface to RF Gen via Message Bus                         #
#                                                                           #
#############################################################################

#
#  RF Gen System Number
#
set RFGEN_ENGINE(SYSTEM_ID)           10

#
#  RF Gen Sub-Systems (general operation)
#
set RFGEN_ENGINE(SUB_FREQ)             1
set RFGEN_ENGINE(SUB_PORT)             2
set RFGEN_ENGINE(SUB_LEVEL)            3
set RFGEN_ENGINE(SUB_MODULATION)       4
set RFGEN_ENGINE(SUB_P25MODE)          5
set RFGEN_ENGINE(SUB_PTT)              6
set RFGEN_ENGINE(SUB_STATUS)           7
set RFGEN_ENGINE(SUB_INFO)             8
set RFGEN_ENGINE(SUB_ENABLE)           9
set RFGEN_ENGINE(SUB_UPDATE_YIG_CAL)  10
set RFGEN_ENGINE(SUB_LOCK)            11
set RFGEN_ENGINE(SUB_ALC)             12
set RFGEN_ENGINE(SUB_BW)              13
set RFGEN_ENGINE(SUB_ONOFF)           14
set RFGEN_ENGINE(SUB_YIGSPEED)        15
set RFGEN_ENGINE(SUB_ALARM)           16

#
#  RF Gen Sub-Systems (calibration)
#
set RFGEN_ENGINE(SUB_READING)         17
set RFGEN_ENGINE(SUB_GEN_FREQ)        18
set RFGEN_ENGINE(SUB_LEVELER)         19
set RFGEN_ENGINE(SUB_OUTPUT)          20
set RFGEN_ENGINE(SUB_ATTEN_STEP)      21
set RFGEN_ENGINE(SUB_SET_REF_FREQ)    22
set RFGEN_ENGINE(SUB_SET_LVELER_CAL)  23
set RFGEN_ENGINE(SUB_LEVELER_CAL)     24
set RFGEN_ENGINE(SUB_FREQ_CAL)        25
set RFGEN_ENGINE(SUB_GET_CALS)        26
set RFGEN_ENGINE(SUB_READ_CALS)       27
set RFGEN_ENGINE(SUB_DLOAD_LVEL_CAL)  28
set RFGEN_ENGINE(SUB_DLOAD_FREQ_CAL)  29
set RFGEN_ENGINE(SUB_GET_READING)     30
set RFGEN_ENGINE(SUB_SET_FREQ_INDX)   31


set RFGEN_ENGINE(SUB_CALCONFIG)       32
set RFGEN_ENGINE(SUB_SPLINEINFO)      33
set RFGEN_ENGINE(SUB_ALCSTAT)         34
set RFGEN_ENGINE(SUB_HOPSTATE)        35


#
#  RF Generator Call-back Data
#
set RFGEN_STATUS(lock_main)           -1
set RFGEN_STATUS(lock_clean)          -1
set RFGEN_STATUS(lock_step)           -1
set RFGEN_STATUS(lock_summary)        "PLL Lock Init"
set RFGEN_STATUS(alarm)               0
set RFGEN_STATUS(yig_hold)            -1
set RFGEN_STATUS(yig_dac)             -1
set RFGEN_STATUS(yig_padetect)        -1
set RFGEN_STATUS(spline)              -1
set RFGEN_STATUS(alcstat)             "Ok"


set Rfgen_Handler(lock)                0
set Rfgen_Handler(yig)                 0
set Rfgen_Handler(spline)              0


set RFGEN_ENGINE(RF_Freq)              500.0
set RFGEN_ENGINE(OutGoingFreq)         500.0
set RFGEN_ENGINE(RF_Level)            -30
set RFGEN_ENGINE(OutGoingLevel)       -30

set PWT_ENGINE(ptt_state)              SET


#
#  Calibration Globals
#
set RFGEN_ENGINE(Filename)                 junk

set RFGEN_ENGINE(Port)                     GEN
set RFGEN_ENGINE(Step)                     0
set RFGEN_ENGINE(AlcDac)                   0

set RFGEN_ENGINE(Normalize.Genalc)        -30.00
set RFGEN_ENGINE(Normalize.Genfreq.dB0)   -30.00
set RFGEN_ENGINE(Normalize.Genfreq.dB10)  -40.00
set RFGEN_ENGINE(Normalize.Genfreq.dB20)  -50.00

set RFGEN_ENGINE(AlcIndex)                 0
set RFGEN_ENGINE(AlcMaxIndex)              14

set RFGEN_ENGINE(FreqIndex)                0
set RFGEN_ENGINE(FreqMaxIndex)             14



set SET_TOP_OF_TABLE_RECOMPUTE 255

#
#  RF Gen Font Declaration
#
font create rfgen_font -family helvetica -size 8

#
#  RF Gen Yig Caligration Data
#
set RfGenYigCals(temp)                 25
set RfGenYigCals(0)                    100
set RfGenYigCals(2800)                 2651


#############################################################################
#                                                                           #
#                       Message Bus Interfaces                              #
#                                                                           #
#############################################################################

#
#   Message bus interfaces RF Gen calibration message
#
proc update_rfgen_cals { index cal } {
  global RFGEN_ENGINE
  global Send_Flags
  
  # send <index> <cal>
  set msg " { $RFGEN_ENGINE(SYSTEM_ID)  \
              $RFGEN_ENGINE(SUB_UPDATE_YIG_CAL) 1 $Send_Flags } \
            { inline dw $index $cal } "
#puts $msg
  tclMsgbus send $msg
}

#
#  Message bus callback - fill in supplied structure
#
proc handle_rfgen_lock {sys sub lun flags uid thing } {
  global RFGEN_STATUS

#set stuff [format "rcvr msg %x " $thing]
#puts "rf gen lock: $thing"

  set RFGEN_STATUS(lock_main)    [ is_locked [ expr { $thing & 0x1 } ] ]
  set RFGEN_STATUS(lock_clean)   [ is_locked [ expr { $thing & 0x2 } ] ]
  set RFGEN_STATUS(lock_step)    [ is_locked [ expr { $thing & 0x4 } ] ]

  if { $thing } {
    set RFGEN_STATUS(lock_summary) "PLL Unlocked"
  } else {
    set RFGEN_STATUS(lock_summary) "PLL Locked"
  }
}

#
#  Message bus callback - fill in supplied structure
#
proc handle_rfgen_yig {sys sub lun flags uid hold dac padetect } {
  global RFGEN_STATUS

#puts "rf gen yig: $hold $dac $padetect"

  set RFGEN_STATUS(yig_hold)     $hold
  set RFGEN_STATUS(yig_dac)      $dac
  set RFGEN_STATUS(yig_padetect) $padetect
}

#
#  Message bus callback - fill in supplied structure
#
proc handle_rfgen_spline {sys sub lun flags uid type thing point } {
  global RFGEN_STATUS

  # only look at status message
  if { $flags & 4 } {
    set RFGEN_STATUS(spline)     $point

puts "handle_rfgen_spline: $type $thing $point"
  }
}


#
#  Message bus callback - fill in supplied structure
#
proc handle_rfgen_alcstat {sys sub lun flags uid msg } {
  global RFGEN_STATUS

  set RFGEN_STATUS(alcstat) $msg

  if {$msg == -1 } {
    set RFGEN_STATUS(alcstat) "Ok"
  } else {
    set RFGEN_STATUS(alcstat) $msg
  }

#puts "handle_rfgen_alcstat: $msg $RFGEN_STATUS(alcstat)"
}



#############################################################################
#                                                                           #
#                       Engine Interfaces                                   #
#                                                                           #
#############################################################################

#
#  RF Gen RF frequency (  2.7 GHz to 100 kHz in 1 Hz steps)
#
proc set_rfgen_freq { MHz } {
  global RFGEN_ENGINE

#puts $MHz

  message_bus_double $RFGEN_ENGINE(SYSTEM_ID) \
                     $RFGEN_ENGINE(SUB_FREQ) 1 $MHz
}

proc trace_rfgen_freq { } {
  global RFGEN_ENGINE

  set RFGEN_ENGINE(OutGoingFreq) $RFGEN_ENGINE(RF_Freq)

  set_rfgen_freq $RFGEN_ENGINE(RF_Freq)
}


#
#  RF Gen RF signal source (T/R or GEN output)
#
proc set_rfgen_output { x } {
  global RFGEN_ENGINE

  # be sure to reset PT in switching (could be hung)
  set_pt_reset

  switch $x {
    GEN     { set in 1 }
    TR      { set in 0 }
    default { return   }
  }

  message_bus_int $RFGEN_ENGINE(SYSTEM_ID) $RFGEN_ENGINE(SUB_PORT) 1 $in
}


#
#  RF Gen RF level (Gen: +10 dBm to -110 dBm, TR: -30 dBm to -137 dBm )
#
proc set_rfgen_rflevel { dbm } {
  global RFGEN_ENGINE

  message_bus_double $RFGEN_ENGINE(SYSTEM_ID) $RFGEN_ENGINE(SUB_LEVEL) 1 $dbm
}

proc trace_rfgen_level { } {
  global RFGEN_ENGINE

  set RFGEN_ENGINE(OutGoingLevel) $RFGEN_ENGINE(RF_Level)

  set_rfgen_rflevel $RFGEN_ENGINE(RF_Level)
}

#
#  RF Gen modulation ( OFF, FM, PM, ...)
#
proc set_rfgen_modulation { type } {
  global RFGEN_ENGINE

  switch $type {
    OFF       { set x 0  }
    FMN       { set x 1  }
    FMW       { set x 2  }
    PM        { set x 3  }
    AM        { set x 4  }
    PSK       { set x 5  }
    USB       { set x 6  }
    LSB       { set x 7  }
    P25MIS    { set x 8  }
    P25MCC    { set x 9  }
    P25STDMSG { set x 10 }
    default   { return }
  }


  message_bus_int $RFGEN_ENGINE(SYSTEM_ID) $RFGEN_ENGINE(SUB_MODULATION) 1 $x
}


#
#  RF Gen P25 ptt ( off, speech, silence, 5% error, 1011 Hz, file )
#
proc set_rfgen_p25mode { mode } {
  global RFGEN_ENGINE

  switch $mode {
    OFF          { set x 0 }
    SPEECH       { set x 1 }
    SILENCE      { set x 2 }
    ERR5PERCENT  { set x 3 }
    HZ1011       { set x 4 }
    FILE         { set x 5 }
    STD_TONE     { set x 6 }
    STD_SILENCE  { set x 7 }
    STD_INTERFER { set x 8 }
    STD_BUSY     { set x 9 }
    STD_IDLE     { set x 10 }
    STD_CAL      { set x 11 }
    STD_LUD1TRIG { set x 12 }
    STD_NOTRIG   { set x 13 }
    STD_LUD2TRIG { set x 14 }
    STD_511      { set x 15 }
    STD_SYM_RATE { set x 16 }
    STD_LOW_DEV  { set x 17 }
    STD_FID_PAT  { set x 18 }
    STD_FID_SPECT {set x 19 }
    default      { return }
  }

  message_bus_int $RFGEN_ENGINE(SYSTEM_ID) $RFGEN_ENGINE(SUB_P25MODE) 1 $x
}

#
#  RF Gen ptt control (requires level-mode to be PTT)
#
proc set_rfgen_ptt { x } {
  global RFGEN_ENGINE

  switch $x {
    SET   { set ptt 1  }
    CLR   { set ptt 0  }
    default { return }
  }

  message_bus_int $RFGEN_ENGINE(SYSTEM_ID) $RFGEN_ENGINE(SUB_PTT) 1 $ptt
}

#
#  enable alarm or lock messages from DSP
#
proc set_rfgen_enable { x } {
  global RFGEN_ENGINE 

  switch $x {
     DISABLE_MSG { message_bus_int $RFGEN_ENGINE(SYSTEM_ID) \
                                   $RFGEN_ENGINE(SUB_ENABLE) 1 0 }
        LOCK_MSG { message_bus_int $RFGEN_ENGINE(SYSTEM_ID) \
                                   $RFGEN_ENGINE(SUB_ENABLE) 1 2 }
       ALARM_MSG { message_bus_int $RFGEN_ENGINE(SYSTEM_ID) \
                                   $RFGEN_ENGINE(SUB_ENABLE) 1 4 }
         YIG_MSG { message_bus_int $RFGEN_ENGINE(SYSTEM_ID) \
                                   $RFGEN_ENGINE(SUB_ENABLE) 1 8 }
    LOCK_YIG_MSG { message_bus_int $RFGEN_ENGINE(SYSTEM_ID) \
                                   $RFGEN_ENGINE(SUB_ENABLE) 1 10 }
            DEMO { message_bus_int $RFGEN_ENGINE(SYSTEM_ID) \
                                   $RFGEN_ENGINE(SUB_ENABLE) 1 0x1A }
         default { return }
  }
}



#
#  send YIG cal table to DSP
#
#  the global "RfGenYigCals" is defined in "to_cals.tcl"
#
proc update_rfgen_yig_cal_data { } {
  global SET_TOP_OF_TABLE_RECOMPUTE 
  global RfGenYigCals

  update_rfgen_cals 0 $RfGenYigCals(0)
  update_rfgen_cals $SET_TOP_OF_TABLE_RECOMPUTE $RfGenYigCals(2800)
}


#
#  RF Gen ALC control
#
proc set_rfgen_alc { x } {
  global RFGEN_ENGINE

  switch $x {
    SLOW    { set alc 0  }
    FAST    { set alc 1  }
    DIGITAL { set alc 2  }
    default { return }
  }

  message_bus_int $RFGEN_ENGINE(SYSTEM_ID) $RFGEN_ENGINE(SUB_ALC) 1 $alc
}

#
#  RF Gen Bandwidth
#
proc set_rfgen_bw { x } {
  global RFGEN_ENGINE

  switch $x {
    NARROW   { set bw 1  }
    WIDE     { set bw 0  }
    default { return }
  }

  message_bus_int $RFGEN_ENGINE(SYSTEM_ID) $RFGEN_ENGINE(SUB_BW) 1 $bw
}


#
#  RF Gen On/Off
#
proc set_rfgen_onoff { x } {
  global RFGEN_ENGINE

  switch $x {
    ON   { set state 1  }
    OFF  { set state 0  }
    default { return }
  }

  message_bus_int $RFGEN_ENGINE(SYSTEM_ID) $RFGEN_ENGINE(SUB_ONOFF) 1 $state
}


#
#  RF Gen Yig Speed (how fast yig acquires lock)
#
proc set_rfgen_yigspeed { x } {
  global RFGEN_ENGINE

  switch $x {
    SLOW   { set state 1  }
    FAST   { set state 0  }
    default { return }
  }

  message_bus_int $RFGEN_ENGINE(SYSTEM_ID) \
                  $RFGEN_ENGINE(SUB_YIGSPEED) 1 $state
}



#
#  RF Gen Hop State
#
#   this function will turn on the hardware signal capability that a
#   freq change has occured in the DSP code - settling time test purposes
#
#
#   bitmap -  which bit(s) to toggle 1 or 0
#     Bit0 = Digital Out 1 (DB15 Pin09): 1=Low, 0=Open Collector [DEFAULT].
#     Bit1 = Digital Out 2 (DB15 Pin10): 1=Low, 0=Open Collector [DEFAULT].
#     Bit2 = Digital Out 3 (DB15 Pin11): 1=Low, 0=Open Collector [DEFAULT].
#     Bit3 = Digital Out 4 (DB15 Pin12): 1=Low, 0=Open Collector [DEFAULT].
#     Bit4 = Serial  Out 1 (DB15 Pin13): 1=Low, 0=Open Collector [DEFAULT].
#              WARNING: Bit4 may be removed in the future.  A serial shift reg
#              might be added to the FPGA to send serial data out this pin.
#              NOTE: Depending on how this is done, the live state of this
#              pin might not be available.
#     Bit5 = Vout Disable  (DB15 Pin08): 1=Disable [DEFAULT], 0=Enable.
#
proc set_rfgen_hopstate { bitmap } {
  global RFGEN_ENGINE

  message_bus_int $RFGEN_ENGINE(SYSTEM_ID) $RFGEN_ENGINE(SUB_HOPSTATE) 1 $bitmap
}


#
#  do_rfgen_freq_hop - test proc for hopping freqs
#
#         interval - time between freq changes (mS)
#         freq1    - 1st freq in MHz
#         freq2    - 2nd freq in MHz
#
#
set do_rfgen_freq_hop_switch 0

proc do_rfgen_freq_hop { interval freq1 freq2 orignal} {
  global do_rfgen_freq_hop_switch

  if { $do_rfgen_freq_hop_switch } {
#  puts $freq1
     set_rfgen_freq $freq1
     after $interval [ list do_rfgen_freq_hop $interval $freq2 $freq1 $orignal ]
  } else {
     set_rfgen_freq $orignal
  }
}

#
#  Start freq hop test
#
proc start_rfgen_freq_hop { } {
  global do_rfgen_freq_hop_switch
  global RFGEN_STATUS
  global RFGEN_ENGINE

  if { $do_rfgen_freq_hop_switch } {
    do_rfgen_freq_hop $RFGEN_STATUS(hop_delay) $RFGEN_STATUS(hop_freq1) \
                      $RFGEN_STATUS(hop_freq2) $RFGEN_ENGINE(RF_Freq)
  }
}




#############################################################################
#                                                                           #
#                       Calibration Interfaces                              #
#                                                                           #
#############################################################################

#
# parse rfgen yig cal
#
# The RF Gen Calibrations have the form:
# GENYIGCAL   Temp   25                      
# GENYIGCAL   Freq  <MHz>    Value <cal number> 
#
proc parse_rfgen_yig_cal { line } {
  global RfGenYigCals

  switch [ lindex $line 1 ] {
    Temp { set RfGenYigCals(temp)  [ lindex $line 2 ] }
    Freq { set RfGenYigCals([ lindex $line 2 ])  [ lindex $line 4 ] }
  }

# puts $RfGenYigCals([ lindex $line 2 ])
}


#
# update RF Gen YIG cal data
#
proc get_rfgen_yig_cal_data { filename } {
  global CalFileName

  set file [ OpenCalFile $filename r ]

  while { [ gets $file line ] >= 0 } {
    if { [ lindex $line 0 ] == "GENYIGCAL" } {
      parse_rfgen_yig_cal $line
    }
  }

  close $file
}



#############################################################################
#                                                                           #
#                       Auxilary Files                                      #
#                                                                           #
#############################################################################

source to_genca.tcl



#############################################################################
#                                                                           #
#                       Callback                                            #
#                                                                           #
#############################################################################

#
#  Message bus interfaces for messages being generated FROM rfgen engine
#
proc create_rfgen_callback { } {
  global RFGEN_ENGINE
  global Rfgen_Handler

  set RFGen_Handler(lock)    [tclMsgbus rcv handle \
                           -sys $RFGEN_ENGINE(SYSTEM_ID) \
                           -sub $RFGEN_ENGINE(SUB_LOCK) \
                           -lun 1 -cmd "handle_rfgen_lock" ]
  set RFGen_Handler(yig)     [tclMsgbus rcv handle \
                           -sys $RFGEN_ENGINE(SYSTEM_ID) \
                           -sub $RFGEN_ENGINE(SUB_STATUS) \
                           -lun 1 -cmd "handle_rfgen_yig" ]
  set RFGen_Handler(spline)  [tclMsgbus rcv handle \
                           -sys $RFGEN_ENGINE(SYSTEM_ID) \
                           -sub $RFGEN_ENGINE(SUB_SPLINEINFO) \
                           -lun 1 -cmd "handle_rfgen_spline" ]
  set RFGen_Handler(alcstat) [tclMsgbus rcv handle \
                           -sys $RFGEN_ENGINE(SYSTEM_ID) \
                           -sub $RFGEN_ENGINE(SUB_ALCSTAT) \
                           -lun 1 -cmd "handle_rfgen_alcstat" ]
}



#
#  Message bus interfaces - removed
#
proc destroy_rfgen_callback { } {
  global Rfgen_Handler

  if {$RFGen_Handler(lock) != 0 } {
    tclMsgbus $RFGen_Handler(lock) destroy
  }

  if {$RFGen_Handler(yig) != 0 } {
    tclMsgbus $RFGen_Handler(yig) destroy
  }

  if {$RFGen_Handler(spline) != 0 } {
    tclMsgbus $RFGen_Handler(spline) destroy
  }

  if {$RFGen_Handler(alcstat) != 0 } {
    tclMsgbus $RFGen_Handler(alcstat) destroy
  }

 
  set RFGen_Handler(lock)      0
  set RFGen_Handler(yig)       0
  set RFGen_Handler(spline)    0
  set RFGen_Handler(alcstat)   0
}
