#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C)1998,99      DESIGN ENGINEERING      #
# --------------------------------------------------------------------------#
# FILE        : scope.tcl                                                   #
# PRODUCT     : FITS                                                        #
# AUTHOR      : Steve Johnson                                               #
# DESCRIPTION : DemoDisplay for SCOPE                                       #
#                                                                           #
#############################################################################

# NOTE: METER_IN_ROUTING and METER_ID are used to define settings for the BBAUDIO
#    engine interface for setting up meter routings.  The constants are currently
#    defined in the meterdemo directory, in the screen.tcl file.

#
# Display state variables
#
set ScopeButton1 DC_T
set ScopeButton2 5V
set ScopeButton3 DC_T
set ScopeButton4 5V
set ScopeButton5 STRIG_CH1
set ScopeButton6 STRIG_AUTO
set ScopeButton7 0
set ScopeButton8 V1
set ScopeButton9 1000.0
set ScopeButton10 0
set ScopeButton11 0

set ScopeButton12 1
set ScopeButton13 1
set ScopeButton14 0
set ScopeButton15 POS
set ScopeButton16 STOP
set ScopeButton17 STOP

#
# default hardware here
#
proc init_scope_hardware { } {
  global SCOPE_ENGINE

  set_scope_scale 1 5V
  set_scope_scale 2 5V
  set_scope_scale 3 1V

  set_scope_coupling 1 DC_T
  set_scope_coupling 2 DC_T

  set_scope_trig_source STRIG_CH1
  set_scope_trig_type STRIG_AUTO
  scope_trigger_polarity POS

  set_scope_sweep 1000.0
  set_scope_horiz 1 0
}


#
# set channel offset position (-100 to 100)
#
proc ch1_offset { offset } {
  global SCOPE_ENGINE

  if { $SCOPE_ENGINE(name) != -1 } {
    set temp [ expr $offset / 100.0 ]
    # trace widget uses a -1 to 1 scale
    $SCOPE_ENGINE(name).trace1 config -voffs $temp
  }
}

proc ch2_offset { offset } {
  global SCOPE_ENGINE

  if { $SCOPE_ENGINE(name) != -1 } {
    set temp [ expr $offset / 100.0 ]
    # trace widget uses a -1 to 1 scale
    $SCOPE_ENGINE(name)trace2 config -voffs $temp
  }
}

proc ch3_offset { offset } {
  global SCOPE_ENGINE

  if { $SCOPE_ENGINE(name) != -1 } {
    set temp [ expr $offset / 100.0 ]
    # trace widget uses a -1 to 1 scale
    $SCOPE_ENGINE(name).trace3 config -voffs $temp
  }
}

#
# set channel persistance
#
proc ch_persistance { x } {
  global SCOPE_ENGINE

  if { $SCOPE_ENGINE(name) != -1 } {
    $SCOPE_ENGINE(name) config -persist $x
  }
}



#
# set channel active state
#
proc ch1_onoff { x } {
  global SCOPE_ENGINE

  if { $SCOPE_ENGINE(name) != -1 } {
    $SCOPE_ENGINE(name).trace1 config -active $x -fg $SCOPE_ENGINE(color_ch1)
  }
}

proc ch2_onoff { x } {
  global SCOPE_ENGINE

  if { $SCOPE_ENGINE(name) != -1 } {
    $SCOPE_ENGINE(name).trace2 config -active $x -fg $SCOPE_ENGINE(color_ch2)
  }
}

proc ch3_onoff { x } {
  global SCOPE_ENGINE

  if { $SCOPE_ENGINE(name) != -1 } {
    $SCOPE_ENGINE(name).trace3 config -active $x -fg $SCOPE_ENGINE(color_ch3)
  }
}

proc remove_scope_routing { } {
    global      METER_ID
    global      METER_IN_ROUTING
 
    set METER_IN_ROUTING(Scope) -1
    BBAudio route $METER_IN_ROUTING(Scope) $METER_ID(Scope)
}


#
# popup for displaying scope status
# messages from the DSP
#
proc scope_status_panel { } {
  global SCOPE_ENGINE

  set handle $SCOPE_ENGINE(stat_handle)

  toplevel $handle -bd 4 -relief ridge
  
  frame $handle.left
  frame $handle.right

  wm title $handle "Scope Status"

  button $handle.kill        -command { destroy $SCOPE_ENGINE(stat_handle) } -text "Kill"

  label $handle.left.msg1    -text "Trace Trigger   :"
  label $handle.right.trig    -textvariable SCOPE_ENGINE(stat_trig)

  label $handle.left.msg2    -text "Pre-Trig LL Fail:"
  label $handle.right.llfail  -textvariable SCOPE_ENGINE(stat_llfail)

  label $handle.left.msg3    -text "Pre-Trig UL Fail:"
  label $handle.right.ulfail  -textvariable SCOPE_ENGINE(stat_ulfail)

  label $handle.left.msg4    -text "Timer Alloc Fail:"
  label $handle.right.timfail -textvariable SCOPE_ENGINE(stat_timfail)

  label $handle.left.msg5    -text "Mem Alloc Fail  :"
  label $handle.right.memfail -textvariable SCOPE_ENGINE(stat_memfail)

  label $handle.left.msg6    -text "Halt Traces Msg :"
  label $handle.right.stop    -textvariable SCOPE_ENGINE(stat_stop)

  label $handle.left.msg7    -text "Trace Length Fail:"
  label $handle.right.length -textvariable SCOPE_ENGINE(stat_length)

  pack $handle.right.trig      -side top
  pack $handle.right.llfail    -side top
  pack $handle.right.ulfail    -side top
  pack $handle.right.timfail   -side top
  pack $handle.right.memfail   -side top
  pack $handle.right.stop      -side top
  pack $handle.right.length    -side top

  pack $handle.left.msg1       -side top
  pack $handle.left.msg2       -side top
  pack $handle.left.msg3       -side top
  pack $handle.left.msg4       -side top
  pack $handle.left.msg5       -side top
  pack $handle.left.msg6       -side top
  pack $handle.left.msg7       -side top

  pack $handle.kill            -side top
  pack $handle.left            -side left
  pack $handle.right           -side left
}



proc doScopeTriggerLevel { tg  } {
  global SCOPE_ENGINE
  global DemoColor
  global scope_font
  global dstl

  label $tg.msg     -text  "Trigger Level (mV)" -font scope_font
  label $tg.msg2    -text  "Range -500000 to 500000 mV" \
                    -font scope_font
  entry $tg.entry   -textvariable SCOPE_ENGINE(trig_level) \
                    -font scope_font -width 8 -bg white -relief sunken
  scale $tg.adj     -from 10000 -to -10000 -resolution 1 -font scope_font \
                    -command set_scope_trig_level

  set dstl $tg.adj

  bind $tg.entry <Return> { 
                            set_scope_trig_level $SCOPE_ENGINE(trig_level) 
                            $dstl configure -variable SCOPE_ENGINE(trig_level)
                          }

  pack $tg.msg    -side top
  pack $tg.msg2   -side top
  pack $tg.entry  -side top
  pack $tg.adj    -side top
}

proc doScopeVPos1 { tg  } {
  global SCOPE_ENGINE
  global DemoColor
  global scope_font

  label $tg.msg    -text  "Vertical Position" \
                   -font scope_font
  scale $tg.ch     -label "Ch1" -from 100 -to -100 \
                   -resolution 2 -font scope_font \
                   -orient vertical -command ch1_offset \
                   -bg $SCOPE_ENGINE(color_ch1)

  pack $tg.msg  -side top  -fill both -expand true
  pack $tg.ch   -side top  -fill both -expand true
}

proc doScopeVPos2 { tg  } {
  global SCOPE_ENGINE
  global DemoColor
  global scope_font

  label $tg.msg    -text  "Vertical Position" \
                   -font scope_font
  scale $tg.ch     -label "Ch2" -from 100 -to -100 \
                   -resolution 2 -font scope_font \
                   -orient vertical -command ch2_offset \
                   -bg $SCOPE_ENGINE(color_ch2)

  pack $tg.msg  -side top  -fill both -expand true
  pack $tg.ch   -side top  -fill both -expand true
}

proc doScopeVPos3 { tg  } {
  global SCOPE_ENGINE
  global DemoColor
  global scope_font

  label $tg.msg    -text  "Vertical Position" \
                   -font scope_font
  scale $tg.ch     -label "Ch3" -from 100 -to -100 \
                   -resolution 2 -font scope_font \
                   -orient vertical -command ch3_offset \
                   -bg $SCOPE_ENGINE(color_ch3)

  pack $tg.msg  -side top  -fill both -expand true
  pack $tg.ch   -side top  -fill both -expand true
}


proc doScopeHorizPos { tg  } {
  global SCOPE_ENGINE
  global DemoColor
  global scope_font

  label $tg.msg    -text  "Horizontal Position" \
                   -font scope_font
  scale $tg.ch     -label "Offset" -from -100 -to 100 \
                   -resolution 10 -font scope_font \
                   -orient horizontal -command set_scope_offset

  bind $tg <F26> { move_scope_horiz -10 }
  bind $tg <F27> { move_scope_horiz  10 }

  pack $tg.msg  -side top  -fill both -expand true
  pack $tg.ch   -side top  -fill both -expand true
}


proc doScopeCh1CalOffset { tg } {
  global DemoColor
  global scope_font

  label $tg.msg    -text  "Op Amp Offset Adjustment" \
                   -font scope_font


  button  $tg.up   -text  "Up" -font scope_font \
                   -command {set_scope_x5_cals U35 1}

  button  $tg.dn   -text  "Down" -font scope_font \
                   -command {set_scope_x5_cals U35 0}


  pack $tg.msg  -side top  -fill both -expand true
  pack $tg.up   -side left -fill both -expand true
  pack $tg.dn   -side left -fill both -expand true
}

proc doScopeCh2CalOffset { tg } {
  global DemoColor
  global scope_font

  label $tg.msg    -text  "Op Amp Offset Adjustment" \
                   -font scope_font


  button  $tg.up   -text  "Up" -font scope_font \
                   -command {set_scope_x5_cals U40 1}

  button  $tg.dn   -text  "Down" -font scope_font \
                   -command {set_scope_x5_cals U40 0}


  pack $tg.msg  -side top  -fill both -expand true
  pack $tg.up   -side left -fill both -expand true
  pack $tg.dn   -side left -fill both -expand true
}



#
# build debug scope screen - except final pack
#
proc build_scope_screen { name } {
  global SCOPE_ENGINE
  global DemoColor
  global HsNv
  global METER_IN_ROUTING
  global METER_ID
  global BBAUDIO_ENGINE

  set HsNv $name
#
# define screen frames
#
frame $HsNv.mbar              -relief raised -bd 2
frame $HsNv.run               -relief raised -bd 2



message $name.mbar.label        -text "Scope" -font scope_font \
                                -bg $DemoColor(title) \
                                -width $DemoColor(labelwidth)


#
# define reset hw button
#
button $HsNv.reset            -text  "Reset HW" -font scope_font \
                              -command reset_rfgen_hw \
                              -bg $DemoColor(menu)

#
#  Trig Status & One Shot Arm
#
button $HsNv.mbar.arm  -text "Arm" -command { scope_arm_trig } \
                       -font scope_font -bg $DemoColor(menu)


#
#  Build Channel Menus 
#
menubutton $HsNv.mbar.ch1 -text  "Channel 1" -font scope_font -menu $HsNv.mbar.ch1.menu \
               -bg $SCOPE_ENGINE(color_ch1) -relief raised
menubutton $HsNv.mbar.ch2 -text  "Channel 2" -font scope_font -menu $HsNv.mbar.ch2.menu \
               -bg $SCOPE_ENGINE(color_ch2) -relief raised
menubutton $HsNv.mbar.ch3 -text  "Channel 3" -font scope_font -menu $HsNv.mbar.ch3.menu \
               -bg $SCOPE_ENGINE(color_ch3) -relief raised

menu $HsNv.mbar.ch1.menu
$HsNv.mbar.ch1.menu add cascade -label "Scale" -font scope_font -menu $HsNv.mbar.ch1.menu.scale
$HsNv.mbar.ch1.menu add cascade -label "AC/DC" -font scope_font -menu $HsNv.mbar.ch1.menu.acdc
$HsNv.mbar.ch1.menu add separator

$HsNv.mbar.ch1.menu add command \
              -label "Position" -font scope_font \
              -command { WidgetWindow doScopeVPos1 \
                         $HsNv.ch1_pos $HsNv "Ch 1 VPOS" }
$HsNv.mbar.ch1.menu add checkbutton -label "Trace On" -font scope_font \
       -command { ch1_onoff  $ScopeButton12 } -variable ScopeButton12



menu $HsNv.mbar.ch2.menu
$HsNv.mbar.ch2.menu add cascade -label "Scale" -font scope_font -menu $HsNv.mbar.ch2.menu.scale
$HsNv.mbar.ch2.menu add cascade -label "AC/DC" -font scope_font -menu $HsNv.mbar.ch2.menu.acdc
$HsNv.mbar.ch2.menu add separator

$HsNv.mbar.ch2.menu add command \
              -label "Position" -font scope_font \
              -command { WidgetWindow doScopeVPos2 \
                         $HsNv.ch2_vpos $HsNv "Ch 2 VPOS" }
$HsNv.mbar.ch2.menu add checkbutton -label "Trace On" -font scope_font \
       -command { ch2_onoff  $ScopeButton13 } -variable ScopeButton13


#
#  Build Channel 3 Menus - Include scope meter routing
#
menu $HsNv.mbar.ch3.menu
$HsNv.mbar.ch3.menu add command \
              -label "Remove Input" -font scope_font \
              -command { remove_scope_routing }
$HsNv.mbar.ch3.menu add cascade -label "Select Hardware Input" \
                -font scope_font -menu $HsNv.mbar.ch3.menu.hw_routing
$HsNv.mbar.ch3.menu add cascade -label "Select FGen Input" \
                -font scope_font -menu $HsNv.mbar.ch3.menu.fgen_routing
$HsNv.mbar.ch3.menu add cascade -label "Select Mixer-Out Input" \
                -font scope_font -menu $HsNv.mbar.ch3.menu.mix_routing

# Set up Scope routing submenus 

menu $HsNv.mbar.ch3.menu.hw_routing
$HsNv.mbar.ch3.menu.hw_routing add radiobutton -label "L1_Out" \
        -font scope_font \
        -command { BBAudio route $METER_IN_ROUTING(Scope) $METER_ID(Scope) } \
        -variable METER_IN_ROUTING(Scope) -value 0
$HsNv.mbar.ch3.menu.hw_routing add radiobutton -label "MIC_In" \
        -font scope_font \
        -command { BBAudio route $METER_IN_ROUTING(Scope) $METER_ID(Scope) } \
        -variable METER_IN_ROUTING(Scope) -value 1
$HsNv.mbar.ch3.menu.hw_routing add radiobutton -label "Sinad_In" \
        -font scope_font \
        -command { BBAudio route $METER_IN_ROUTING(Scope) $METER_ID(Scope) } \
        -variable METER_IN_ROUTING(Scope) -value 2
$HsNv.mbar.ch3.menu.hw_routing add radiobutton -label "Waveform_In" \
        -font scope_font \
        -command { BBAudio route $METER_IN_ROUTING(Scope) $METER_ID(Scope) } \
        -variable METER_IN_ROUTING(Scope) -value 3


menu $HsNv.mbar.ch3.menu.fgen_routing
$HsNv.mbar.ch3.menu.fgen_routing add radiobutton -label "FGen #0" \
        -font scope_font \
        -command { BBAudio route $METER_IN_ROUTING(Scope) $METER_ID(Scope) } \
        -variable METER_IN_ROUTING(Scope) -value 4
$HsNv.mbar.ch3.menu.fgen_routing add radiobutton -label "FGen #1" \
        -font scope_font \
        -command { BBAudio route $METER_IN_ROUTING(Scope) $METER_ID(Scope) } \
        -variable METER_IN_ROUTING(Scope) -value 5
$HsNv.mbar.ch3.menu.fgen_routing add radiobutton -label "FGen #2" \
        -font scope_font \
        -command { BBAudio route $METER_IN_ROUTING(Scope) $METER_ID(Scope) } \
        -variable METER_IN_ROUTING(Scope) -value 6


menu $HsNv.mbar.ch3.menu.mix_routing
$HsNv.mbar.ch3.menu.mix_routing add radiobutton -label "Speaker" \
        -font scope_font \
        -command { BBAudio route $METER_IN_ROUTING(Scope) $METER_ID(Scope) } \
        -variable METER_IN_ROUTING(Scope) -value 8
$HsNv.mbar.ch3.menu.mix_routing add radiobutton -label "DeMod" \
        -font scope_font \
        -command { BBAudio route $METER_IN_ROUTING(Scope) $METER_ID(Scope) } \
        -variable METER_IN_ROUTING(Scope) -value 9
$HsNv.mbar.ch3.menu.mix_routing add radiobutton -label "FGen" \
        -font scope_font \
        -command { BBAudio route $METER_IN_ROUTING(Scope) $METER_ID(Scope) } \
        -variable METER_IN_ROUTING(Scope) -value 10
$HsNv.mbar.ch3.menu.mix_routing add radiobutton -label "L1-In" \
        -font scope_font \
        -command { BBAudio route $METER_IN_ROUTING(Scope) $METER_ID(Scope) } \
        -variable METER_IN_ROUTING(Scope) -value 11
$HsNv.mbar.ch3.menu.mix_routing add radiobutton -label "User #0" \
        -font scope_font \
        -command { BBAudio route $METER_IN_ROUTING(Scope) $METER_ID(Scope) } \
        -variable METER_IN_ROUTING(Scope) -value 12
$HsNv.mbar.ch3.menu.mix_routing add radiobutton -label "User #1" \
        -font scope_font \
        -command { BBAudio route $METER_IN_ROUTING(Scope) $METER_ID(Scope) } \
        -variable METER_IN_ROUTING(Scope) -value 13

$HsNv.mbar.ch3.menu add separator
$HsNv.mbar.ch3.menu add cascade -label "Scale" -font scope_font -menu $HsNv.mbar.ch3.menu.scale
$HsNv.mbar.ch3.menu add separator

$HsNv.mbar.ch3.menu add command \
              -label "Position" -font scope_font \
              -command { WidgetWindow doScopeVPos3 \
                         $HsNv.ch3_pos $HsNv "Ch 3 VPOS" }

$HsNv.mbar.ch3.menu add checkbutton -label "Trace On" -font scope_font \
       -command { ch3_onoff  $ScopeButton14 } -variable ScopeButton14
#
#  Build Trigger Menus 
#
menubutton $HsNv.mbar.trig -text  "Trigger" -font scope_font \
                           -menu $HsNv.mbar.trig.menu \
                           -bg $DemoColor(menu) -relief raised

menu $HsNv.mbar.trig.menu
$HsNv.mbar.trig.menu add cascade -label "Source"   -font scope_font \
                                 -menu $HsNv.mbar.trig.menu.source

$HsNv.mbar.trig.menu add cascade -label "Type"     -font scope_font \
                                 -menu $HsNv.mbar.trig.menu.type

$HsNv.mbar.trig.menu add cascade -label "Polarity" -font scope_font \
                                 -menu $HsNv.mbar.trig.menu.pol

$HsNv.mbar.trig.menu add separator 

$HsNv.mbar.trig.menu add command -label "Horiz Position" -font scope_font \
                                 -command { WidgetWindow doScopeHorizPos \
                                 $HsNv.trig_hpos $HsNv "Scope Horiz Position"}

$HsNv.mbar.trig.menu add command -label "Trigger Level" -font scope_font \
                                 -command { WidgetWindow doScopeTriggerLevel \
                                 $HsNv.trig_level $HsNv "Scope Trigger Level"}

$HsNv.mbar.trig.menu add separator 

$HsNv.mbar.trig.menu add command -label "Cal Ch1" -font scope_font \
                                 -command { WidgetWindow doScopeCh1CalOffset \
                                 $HsNv.trig_hpos $HsNv "Cal Ch1 Offset"}

$HsNv.mbar.trig.menu add command -label "Cal Ch2" -font scope_font \
                                 -command { WidgetWindow doScopeCh2CalOffset \
                                 $HsNv.trig_hpos $HsNv "Cal Ch2 Offset"}


#
#  Build Sweep Menus 
#
menubutton $HsNv.mbar.sweep -text  "Sweep"    -font scope_font \
                            -menu $HsNv.mbar.sweep.menu \
                            -bg $DemoColor(menu) -relief raised

menu $HsNv.mbar.sweep.menu
$HsNv.mbar.sweep.menu add cascade -label "Rate" -font scope_font \
                                  -menu $HsNv.mbar.sweep.menu.rate

$HsNv.mbar.sweep.menu add separator
$HsNv.mbar.sweep.menu add command -label "Status..." -command "scope_status_panel" -font scope_font
$HsNv.mbar.sweep.menu add checkbutton -label "Persistance" -font scope_font \
       -command { ch_persistance $ScopeButton10 } -variable ScopeButton10




#
#  Build Run Menus 
#
button $HsNv.run.hw    -text "HW" -command { scope_run RUN DSP_SC } \
                                  -font scope_font -bg $DemoColor(menu)
button $HsNv.run.sw    -text "SW" -command { scope_run RUN DSP_MF } \
                                  -font scope_font -bg $DemoColor(menu)
      
button $HsNv.run.stop  -text "Stop"  -command { scope_run STOP } \
                                     -font scope_font -bg $DemoColor(menu)

label $HsNv.run.msg    -text "Run" -font scope_font


#
#  channel 1
#
menu $HsNv.mbar.ch1.menu.acdc
$HsNv.mbar.ch1.menu.acdc add radiobutton -label "AC" -font scope_font \
       -command { set_scope_coupling 1 $ScopeButton1 } -variable ScopeButton1 -value AC_T
$HsNv.mbar.ch1.menu.acdc add radiobutton -label "DC" -font scope_font \
       -command { set_scope_coupling 1 $ScopeButton1 } -variable ScopeButton1 -value DC_T


menu $HsNv.mbar.ch1.menu.scale
$HsNv.mbar.ch1.menu.scale add radiobutton -label "GND" -font scope_font \
        -command { set_scope_scale 1 $ScopeButton2 } -variable ScopeButton2 -value GROUND
$HsNv.mbar.ch1.menu.scale add radiobutton -label "10 mV" -font scope_font \
       -command { set_scope_scale 1 $ScopeButton2 } -variable ScopeButton2 -value 10MV
$HsNv.mbar.ch1.menu.scale add radiobutton -label "20 mV" -font scope_font \
       -command { set_scope_scale 1 $ScopeButton2 } -variable ScopeButton2 -value 20MV
$HsNv.mbar.ch1.menu.scale add radiobutton -label "50 mV" -font scope_font \
       -command { set_scope_scale 1 $ScopeButton2 } -variable ScopeButton2 -value 50MV
$HsNv.mbar.ch1.menu.scale add radiobutton -label "100 mV" -font scope_font \
       -command { set_scope_scale 1 $ScopeButton2 } -variable ScopeButton2 -value 100MV
$HsNv.mbar.ch1.menu.scale add radiobutton -label "200 mV" -font scope_font \
       -command { set_scope_scale 1 $ScopeButton2 } -variable ScopeButton2 -value 200MV
$HsNv.mbar.ch1.menu.scale add radiobutton -label "500 mV" -font scope_font \
       -command { set_scope_scale 1 $ScopeButton2 } -variable ScopeButton2 -value 500MV
$HsNv.mbar.ch1.menu.scale add radiobutton -label "1 V" -font scope_font \
       -command { set_scope_scale 1 $ScopeButton2 } -variable ScopeButton2 -value 1V
$HsNv.mbar.ch1.menu.scale add radiobutton -label "2 V" -font scope_font \
       -command { set_scope_scale 1 $ScopeButton2 } -variable ScopeButton2 -value 2V
$HsNv.mbar.ch1.menu.scale add radiobutton -label "5 V" -font scope_font \
       -command { set_scope_scale 1 $ScopeButton2 } -variable ScopeButton2 -value 5V
$HsNv.mbar.ch1.menu.scale add radiobutton -label "10 V" -font scope_font \
       -command { set_scope_scale 1 $ScopeButton2 } -variable ScopeButton2 -value 10V
$HsNv.mbar.ch1.menu.scale add radiobutton -label "20 V" -font scope_font \
       -command { set_scope_scale 1 $ScopeButton2 } -variable ScopeButton2 -value 20V
$HsNv.mbar.ch1.menu.scale add radiobutton -label "50 V" -font scope_font \
       -command { set_scope_scale 1 $ScopeButton2 } -variable ScopeButton2 -value 50V

#
#  channel 2
#
menu $HsNv.mbar.ch2.menu.acdc
$HsNv.mbar.ch2.menu.acdc add radiobutton -label "AC"  -font scope_font \
       -command { set_scope_coupling 2 $ScopeButton3 } -variable ScopeButton3 -value AC_T
$HsNv.mbar.ch2.menu.acdc add radiobutton -label "DC"  -font scope_font \
       -command { set_scope_coupling 2 $ScopeButton3 } -variable ScopeButton3 -value DC_T


menu $HsNv.mbar.ch2.menu.scale
$HsNv.mbar.ch2.menu.scale add radiobutton -label "GND"  -font scope_font \
       -command { set_scope_scale 2 $ScopeButton4 } -variable ScopeButton4 -value GROUND
$HsNv.mbar.ch2.menu.scale add radiobutton -label "10 mV"  -font scope_font \
       -command { set_scope_scale 2 $ScopeButton4 } -variable ScopeButton4 -value 10MV
$HsNv.mbar.ch2.menu.scale add radiobutton -label "20 mV"  -font scope_font \
       -command { set_scope_scale 2 $ScopeButton4 } -variable ScopeButton4 -value 20MV
$HsNv.mbar.ch2.menu.scale add radiobutton -label "50 mV"  -font scope_font \
       -command { set_scope_scale 2 $ScopeButton4 } -variable ScopeButton4 -value 50MV
$HsNv.mbar.ch2.menu.scale add radiobutton -label "100 mV"  -font scope_font \
       -command { set_scope_scale 2 $ScopeButton4 } -variable ScopeButton4 -value 100MV
$HsNv.mbar.ch2.menu.scale add radiobutton -label "200 mV"  -font scope_font \
       -command { set_scope_scale 2 $ScopeButton4 } -variable ScopeButton4 -value 200MV
$HsNv.mbar.ch2.menu.scale add radiobutton -label "500 mV"  -font scope_font \
       -command { set_scope_scale 2 $ScopeButton4 } -variable ScopeButton4 -value 500MV
$HsNv.mbar.ch2.menu.scale add radiobutton -label "1 V" -font scope_font \
       -command { set_scope_scale 2 $ScopeButton4 } -variable ScopeButton4 -value 1V
$HsNv.mbar.ch2.menu.scale add radiobutton -label "2 V" -font scope_font \
       -command { set_scope_scale 2 $ScopeButton4 } -variable ScopeButton4 -value 2V
$HsNv.mbar.ch2.menu.scale add radiobutton -label "5 V" -font scope_font \
       -command { set_scope_scale 2 $ScopeButton4 } -variable ScopeButton4 -value 5V
$HsNv.mbar.ch2.menu.scale add radiobutton -label "10 V" -font scope_font \
       -command { set_scope_scale 2 $ScopeButton4 } -variable ScopeButton4 -value 10V
$HsNv.mbar.ch2.menu.scale add radiobutton -label "20 V" -font scope_font \
       -command { set_scope_scale 2 $ScopeButton4 } -variable ScopeButton4 -value 20V
$HsNv.mbar.ch2.menu.scale add radiobutton -label "50 V" -font scope_font \
       -command { set_scope_scale 2 $ScopeButton4 } -variable ScopeButton4 -value 50V


#
# trigger
#
menu $HsNv.mbar.trig.menu.source
$HsNv.mbar.trig.menu.source add radiobutton -label "Ext" -font scope_font \
       -command { set_scope_trig_source $ScopeButton5 } -variable ScopeButton5 -value STRIG_EXT
$HsNv.mbar.trig.menu.source add radiobutton -label "CH1" -font scope_font \
       -command { set_scope_trig_source $ScopeButton5 } -variable ScopeButton5 -value STRIG_CH1
$HsNv.mbar.trig.menu.source add radiobutton -label "CH2" -font scope_font \
       -command { set_scope_trig_source $ScopeButton5 } -variable ScopeButton5 -value STRIG_CH2


menu $HsNv.mbar.trig.menu.type
$HsNv.mbar.trig.menu.type add radiobutton -label "Auto" -font scope_font \
       -command { set_scope_trig_type $ScopeButton6 } -variable ScopeButton6 -value STRIG_AUTO
$HsNv.mbar.trig.menu.type add radiobutton -label "Norm" -font scope_font \
       -command { set_scope_trig_type $ScopeButton6 } -variable ScopeButton6 -value STRIG_NORMAL
$HsNv.mbar.trig.menu.type add radiobutton -label "1 Shot" -font scope_font \
       -command { set_scope_trig_type $ScopeButton6 } -variable ScopeButton6 -value STRIG_1SHOT

menu $HsNv.mbar.trig.menu.pol
$HsNv.mbar.trig.menu.pol add radiobutton -label "Rising" -font scope_font \
       -command { scope_trigger_polarity $ScopeButton15 } -variable ScopeButton15 -value POS
$HsNv.mbar.trig.menu.pol add radiobutton -label "Falling" -font scope_font \
       -command { scope_trigger_polarity $ScopeButton15 } -variable ScopeButton15 -value NEG


#
#  Sweep
#
menu $HsNv.mbar.sweep.menu.rate 
$HsNv.mbar.sweep.menu.rate add radiobutton -label "1 S"   -font scope_font \
       -command { set_scope_sweep $ScopeButton9 } -variable ScopeButton9 -value 1000000.0
$HsNv.mbar.sweep.menu.rate add radiobutton -label "500 mS"   -font scope_font \
       -command { set_scope_sweep $ScopeButton9 } -variable ScopeButton9 -value 500000.0
$HsNv.mbar.sweep.menu.rate add radiobutton -label "200 mS"   -font scope_font \
       -command { set_scope_sweep $ScopeButton9 } -variable ScopeButton9 -value 200000.0
$HsNv.mbar.sweep.menu.rate add radiobutton -label "100 mS"   -font scope_font \
       -command { set_scope_sweep $ScopeButton9 } -variable ScopeButton9 -value 100000.0
$HsNv.mbar.sweep.menu.rate add radiobutton -label "50 mS"  -font scope_font \
       -command { set_scope_sweep $ScopeButton9 } -variable ScopeButton9 -value 50000.0
$HsNv.mbar.sweep.menu.rate add radiobutton -label "20 mS"  -font scope_font \
       -command { set_scope_sweep $ScopeButton9 } -variable ScopeButton9 -value 20000.0
$HsNv.mbar.sweep.menu.rate add radiobutton -label "10 mS"  -font scope_font \
       -command { set_scope_sweep $ScopeButton9 } -variable ScopeButton9 -value 10000.0
$HsNv.mbar.sweep.menu.rate add radiobutton -label "5 mS"   -font scope_font \
       -command { set_scope_sweep $ScopeButton9 } -variable ScopeButton9 -value 5000.0
$HsNv.mbar.sweep.menu.rate add radiobutton -label "2 mS"   -font scope_font \
       -command { set_scope_sweep $ScopeButton9 } -variable ScopeButton9 -value 2000.0
$HsNv.mbar.sweep.menu.rate add radiobutton -label "1 mS"   -font scope_font \
       -command { set_scope_sweep $ScopeButton9 } -variable ScopeButton9 -value 1000.0
$HsNv.mbar.sweep.menu.rate add radiobutton -label "500 uS" -font scope_font \
       -command { set_scope_sweep $ScopeButton9 } -variable ScopeButton9 -value 500.0
$HsNv.mbar.sweep.menu.rate add radiobutton -label "200 uS" -font scope_font \
       -command { set_scope_sweep $ScopeButton9 } -variable ScopeButton9 -value 200.0
$HsNv.mbar.sweep.menu.rate add radiobutton -label "100 uS" -font scope_font \
       -command { set_scope_sweep $ScopeButton9 } -variable ScopeButton9 -value 100.0
$HsNv.mbar.sweep.menu.rate add radiobutton -label "50 uS" -font scope_font \
       -command { set_scope_sweep $ScopeButton9 } -variable ScopeButton9 -value 50.0
$HsNv.mbar.sweep.menu.rate add radiobutton -label "20 uS" -font scope_font \
       -command { set_scope_sweep $ScopeButton9 } -variable ScopeButton9 -value 20.0
$HsNv.mbar.sweep.menu.rate add radiobutton -label "10 uS" -font scope_font \
       -command { set_scope_sweep $ScopeButton9 } -variable ScopeButton9 -value 10.0
$HsNv.mbar.sweep.menu.rate add radiobutton -label "5 uS"   -font scope_font \
       -command { set_scope_sweep $ScopeButton9 } -variable ScopeButton9 -value 5.0
$HsNv.mbar.sweep.menu.rate add radiobutton -label "2 uS"   -font scope_font \
       -command { set_scope_sweep $ScopeButton9 } -variable ScopeButton9 -value 2.0
$HsNv.mbar.sweep.menu.rate add radiobutton -label "1 uS"   -font scope_font \
       -command { set_scope_sweep $ScopeButton9 } -variable ScopeButton9 -value 1.0
$HsNv.mbar.sweep.menu.rate add radiobutton -label "0.1 uS"   -font scope_font \
       -command { set_scope_sweep $ScopeButton9 } -variable ScopeButton9 -value 0.1


#
#  Multi-fctn interface
#
menu $HsNv.mbar.ch3.menu.scale
$HsNv.mbar.ch3.menu.scale add radiobutton -label "GND"  -font scope_font \
       -command { set_scope_scale 3 $ScopeButton8 } -variable ScopeButton8 -value GROUND
$HsNv.mbar.ch3.menu.scale add radiobutton -label "10 mV"  -font scope_font \
       -command { set_scope_scale 3 $ScopeButton8 } -variable ScopeButton8 -value 10MV
$HsNv.mbar.ch3.menu.scale add radiobutton -label "20 mV"  -font scope_font \
       -command { set_scope_scale 3 $ScopeButton8 } -variable ScopeButton8 -value 20MV
$HsNv.mbar.ch3.menu.scale add radiobutton -label "50 mV"  -font scope_font \
       -command { set_scope_scale 3 $ScopeButton8 } -variable ScopeButton8 -value 50MV
$HsNv.mbar.ch3.menu.scale add radiobutton -label "100 mV"  -font scope_font \
       -command { set_scope_scale 3 $ScopeButton8 } -variable ScopeButton8 -value 100MV
$HsNv.mbar.ch3.menu.scale add radiobutton -label "200 mV"  -font scope_font \
       -command { set_scope_scale 3 $ScopeButton8 } -variable ScopeButton8 -value 200MV
$HsNv.mbar.ch3.menu.scale add radiobutton -label "500 mV"  -font scope_font \
       -command { set_scope_scale 3 $ScopeButton8 } -variable ScopeButton8 -value 500MV
$HsNv.mbar.ch3.menu.scale add radiobutton -label "1 V" -font scope_font \
       -command { set_scope_scale 3 $ScopeButton8 } -variable ScopeButton8 -value 1V
$HsNv.mbar.ch3.menu.scale add radiobutton -label "2 V" -font scope_font \
       -command { set_scope_scale 3 $ScopeButton8 } -variable ScopeButton8 -value 2V
$HsNv.mbar.ch3.menu.scale add radiobutton -label "5 V" -font scope_font \
       -command { set_scope_scale 3 $ScopeButton8 } -variable ScopeButton8 -value 5V
$HsNv.mbar.ch3.menu.scale add radiobutton -label "10 V" -font scope_font \
       -command { set_scope_scale 3 $ScopeButton8 } -variable ScopeButton8 -value 10V
$HsNv.mbar.ch3.menu.scale add radiobutton -label "20 V" -font scope_font \
       -command { set_scope_scale 3 $ScopeButton8 } -variable ScopeButton8 -value 20V
$HsNv.mbar.ch3.menu.scale add radiobutton -label "50 V" -font scope_font \
       -command { set_scope_scale 3 $ScopeButton8 } -variable ScopeButton8 -value 50V


#
#  pack menu bar
#
pack $HsNv.mbar.label  -side left  -fill both

pack $HsNv.mbar.ch1    -side left  -fill both
pack $HsNv.mbar.ch2    -side left  -fill both
pack $HsNv.mbar.ch3    -side left  -fill both

pack $HsNv.mbar.trig   -side left  -fill both -expand true
pack $HsNv.mbar.sweep  -side left  -fill both -expand true

pack $HsNv.mbar.arm    -side left  -fill both


#
#  pack run bar
#
pack $HsNv.run.msg   -side left
pack $HsNv.run.hw    -side left
pack $HsNv.run.sw    -side left
pack $HsNv.run.stop  -side left


#
#  pack main frame
#
pack $HsNv.mbar                -side left -fill both -expand true
pack $HsNv.run                 -side left
pack $HsNv.reset               -side left
}



#
#  build channel scope screen
#
proc build_scope_trace { name } {
  global SCOPE_ENGINE

  frame $name.knob
  set SCOPE_ENGINE(name)  $name.tr

  set SCOPE_ENGINE(led_pretrig)  $name.knob.led1
  set SCOPE_ENGINE(led_trigarm)  $name.knob.led2
  set SCOPE_ENGINE(led_triged)   $name.knob.led3

  message $name.knob.msg   -text "Scale (uS/Div)" -font scope_font
  entry $name.knob.entry   -textvariable  ScopeButton9 -width 10 \
                           -font anlzr_font -bg white

  label  $name.knob.msg1 -text " Pre Trig" -font scope_font
  canvas $name.knob.led1 -width 14 -height 14
  $name.knob.led1 create oval 3 3 12 12 -fill blue -tag trig_led
  label  $name.knob.msg2 -text " Trig Armed" -font scope_font
  canvas $name.knob.led2 -width 14 -height 14
  $name.knob.led2 create oval 3 3 12 12 -fill blue -tag trig_led
  label  $name.knob.msg3 -text " Trace Trig" -font scope_font
  canvas $name.knob.led3 -width 14 -height 14
  $name.knob.led3 create oval 3 3 12 12 -fill blue -tag trig_led

  TraceWidget $SCOPE_ENGINE(name) -bg #000088 -fg #c0cad4 -width 290 -height 290
  $SCOPE_ENGINE(name) trace create trace1
  $SCOPE_ENGINE(name) trace create trace2
  $SCOPE_ENGINE(name) trace create trace3

  $SCOPE_ENGINE(name) config -xgratdivs 10 -ygratdivs 10 -persist 0

  $SCOPE_ENGINE(name) config -fg        gray
  $SCOPE_ENGINE(name) config -graticule gray
  $SCOPE_ENGINE(name) config -grid      gray

  $SCOPE_ENGINE(name).trace1 config -sys 14 -subsys 16 -lun 1 \
                                    -flags 0x3D -active 1 \
                                    -cfgsubsys $SCOPE_ENGINE(MTR_TRACE_CONFIG) \
                                    -fg $SCOPE_ENGINE(color_ch1)
  $SCOPE_ENGINE(name).trace2 config -sys 14 -subsys 16 -lun 2 \
                                    -flags 0x3D -active 1 \
                                    -cfgsubsys $SCOPE_ENGINE(MTR_TRACE_CONFIG) \
                                    -fg $SCOPE_ENGINE(color_ch2)
  $SCOPE_ENGINE(name).trace3 config -sys 14 -subsys 16 -lun 3 \
                                    -flags 0x3D -active 1 \
                                    -cfgsubsys $SCOPE_ENGINE(MTR_TRACE_CONFIG) \
                                    -fg $SCOPE_ENGINE(color_ch3)

  pack $name.knob.msg        -side left
  pack $name.knob.entry      -side left

  pack $name.knob.led3       -side right
  pack $name.knob.msg3       -side right
  pack $name.knob.led2       -side right
  pack $name.knob.msg2       -side right
  pack $name.knob.led1       -side right
  pack $name.knob.msg1       -side right

  pack $name.knob            -side top   -padx 5 -fill both -expand true
  pack $name.tr              -side left  -padx 5 -fill both -expand true
}

