#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C)1998,99      DESIGN ENGINEERING      #
# --------------------------------------------------------------------------#
# FILE        : to_scope.tcl                                                #
# PRODUCT     : FITS                                                        #
# AUTHOR      : Steve Johnson                                               #
# DESCRIPTION : Interface to SCOPE via Message Bus                          #
#                                                                           #
#############################################################################


#
#  Scope System Number
#
set SCOPE_ENGINE(SYSTEM_ID)      14

#
#  Scope Sub-Systems
#
set SCOPE_ENGINE(MTR_SWEEP)       1
set SCOPE_ENGINE(MTR_SCALE)       2
set SCOPE_ENGINE(MTR_RUN)         3
set SCOPE_ENGINE(MTR_HPOS)        4
set SCOPE_ENGINE(MTR_TRIG_TYPE)   5
set SCOPE_ENGINE(MTR_TRIG_ARM)    6 
set SCOPE_ENGINE(MTR_TRIG_SOURCE) 7
set SCOPE_ENGINE(MTR_ENABLE)      8
set SCOPE_ENGINE(MTR_PHRST)       9
set SCOPE_ENGINE(MTR_AVERAGE)     10
set SCOPE_ENGINE(MTR_COUPLING)    11
set SCOPE_ENGINE(MTR_MATH)        12
set SCOPE_ENGINE(MTR_STATUS)      13
set SCOPE_ENGINE(MTR_TRIG_LEVEL)  14
set SCOPE_ENGINE(MTR_INFO)        15
set SCOPE_ENGINE(MTR_TRACE)       16
set SCOPE_ENGINE(MTR_CAL_X5)      17
set SCOPE_ENGINE(MTR_TRIG_POL)    18
set SCOPE_ENGINE(MTR_CAL_RANGE)   19
set SCOPE_ENGINE(MTR_TRACE_CONFIG) 200

#
#  enable traces and status messages
#
set SCOPE_ENGINE(MB_EVENTS)      7


set SCOPE_ENGINE(trig_level)    0

set SCOPE_ENGINE(name)         -1
set SCOPE_ENGINE(color_ch1)    #ffcbc8
set SCOPE_ENGINE(color_ch2)    #ffff24
set SCOPE_ENGINE(color_ch3)    #82ff40

set SCOPE_ENGINE(stat_handle)     .status
set SCOPE_ENGINE(handler)         -1
set SCOPE_ENGINE(stat_triged)     -1
set SCOPE_ENGINE(stat_llfail)     -1
set SCOPE_ENGINE(stat_ulfail)     -1
set SCOPE_ENGINE(stat_timfail)    -1
set SCOPE_ENGINE(stat_memfail)    -1
set SCOPE_ENGINE(stat_stop)       -1
set SCOPE_ENGINE(stat_length)     -1
set SCOPE_ENGINE(stat_pretrig)    -1
set SCOPE_ENGINE(stat_trigarm)    -1

set SCOPE_ENGINE(U35CAL)          -1
set SCOPE_ENGINE(U38CAL)          -1
set SCOPE_ENGINE(U40CAL)          -1
set SCOPE_ENGINE(U43CAL)          -1

set SCOPE_ENGINE(20MVCH1)         -1
set SCOPE_ENGINE(50MVCH1)         -1
set SCOPE_ENGINE(100MVCH1)        -1
set SCOPE_ENGINE(200MVCH1)        -1
set SCOPE_ENGINE(500MVCH1)        -1
set SCOPE_ENGINE(50MVCH1)         -1
set SCOPE_ENGINE(1VCH1)           -1
set SCOPE_ENGINE(2VCH1)           -1
set SCOPE_ENGINE(5VCH1)           -1
set SCOPE_ENGINE(10VCH1)          -1
set SCOPE_ENGINE(20VCH1)          -1
set SCOPE_ENGINE(50VCH1)          -1

set SCOPE_ENGINE(20MVCH2)         -1
set SCOPE_ENGINE(50MVCH2)         -1
set SCOPE_ENGINE(100MVCH2)        -1
set SCOPE_ENGINE(200MVCH2)        -1
set SCOPE_ENGINE(500MVCH2)        -1
set SCOPE_ENGINE(50MVCH2)         -1
set SCOPE_ENGINE(1VCH2)           -1
set SCOPE_ENGINE(2VCH2)           -1
set SCOPE_ENGINE(5VCH2)           -1
set SCOPE_ENGINE(10VCH2)          -1
set SCOPE_ENGINE(20VCH2)          -1
set SCOPE_ENGINE(50VCH2)          -1


font create scope_font -family helvetica -size 8

#############################################################################
#                                                                           #
#                       Message Bus Interfaces                              #
#                                                                           #
#############################################################################


proc scope_cal_x5 { system subsystem lun refdes value } {
  global Send_Flags

  set msg " { $system $subsystem $lun $Send_Flags } \
            { inline udword $refdes } \
            { inline udword $value } "
#puts $msg
  tclMsgbus send $msg
}

#
#  scope_status_tf
#
proc scope_status_tf { stat bit} { 
  if { $stat &  $bit } {
    return "True"
  } 

  return "False"
}

#
#  Message bus callback - fill in supplied structure
#
# SC_STATUS_LLFAIL   = 0x000002, 1 is trig ll address was exceeded, 0 is ok
# SC_STATUS_ULFAIL   = 0x000004, 1 is trig ul address was exceeded, 0 is ok
# SC_STATUS_TIMFAIL  = 0x000008, 1 is reader task timer allocation failed  
# SC_STATUS_MEMFAIL  = 0x000010, 1 is trace buffer allocation failed       
# SC_STATUS_STOP     = 0x000020, 1 is reader task is stopped               
# SC_LENGTH_FAIL     = 0x000040, Trace length is too long for given sweep  
# SC_PRE_TRIG_RDY    = 0x000100, 1 is Pre-Trigger buffer filled            
# SC_TRIG_ARMED      = 0x000200, 1 is Trigger ISR is running               
# SC_TRACE_TRIGGERED = 0x000400, 1 is Post-Trigger trace is being captured 
 
# 
#
proc handle_scope_status {sys sub lun flags uid status } {
  global SCOPE_ENGINE

  set flags [ expr $flags & 3 ]

  if { $flags == 0 } {
    return
  }

#puts "scope status: $sys $sub $lun $flags $uid $status"

  set SCOPE_ENGINE(stat_llfail)   [ scope_status_tf $status  0x02 ]
  set SCOPE_ENGINE(stat_ulfail)   [ scope_status_tf $status  0x04 ]
  set SCOPE_ENGINE(stat_timfail)  [ scope_status_tf $status  0x08 ]
  set SCOPE_ENGINE(stat_memfail)  [ scope_status_tf $status  0x10 ]
  set SCOPE_ENGINE(stat_stop)     [ scope_status_tf $status  0x20 ]
  set SCOPE_ENGINE(stat_length)   [ scope_status_tf $status  0x40 ]

  set SCOPE_ENGINE(stat_pretrig)  [ scope_status_tf $status 0x100 ]
  set SCOPE_ENGINE(stat_trigarm)  [ scope_status_tf $status 0x200 ]
  set SCOPE_ENGINE(stat_triged)   [ scope_status_tf $status 0x400 ]

  set led [ expr $status & 0xf00 ]
  set led [ expr $led >> 8 ]

  if {$led & 1 } {
    $SCOPE_ENGINE(led_pretrig) itemconfigure trig_led -fill yellow
  } else {
    $SCOPE_ENGINE(led_pretrig) itemconfigure trig_led -fill #809d9a
  }

  if {$led & 2 } {
    $SCOPE_ENGINE(led_trigarm) itemconfigure trig_led -fill yellow
  } else {
    $SCOPE_ENGINE(led_trigarm) itemconfigure trig_led -fill #809d9a
  }

  if {$led & 4 } {
    $SCOPE_ENGINE(led_triged)  itemconfigure trig_led -fill yellow
  } else {
    $SCOPE_ENGINE(led_triged)  itemconfigure trig_led -fill #809d9a
  }
}


proc handle_scope_config {sys sub lun flags uid thing1 thing2 thing3 \
                                       thing4 thing5 thing6 thing7 } {

  puts "handle_scope_config: $flags $sys $lun $thing1 $thing2 $thing3 \
                                      $thing4 $thing5 $thing6 $thing7"
}


#
#  Message bus interfaces for messages being generated FROM scope engine
#
proc create_scope_callback { } {
  global SCOPE_ENGINE

  #
  #  both lun's will be sending the same status - ignore ch2 for now.
  #
  set SCOPE_ENGINE(handler) [tclMsgbus rcv handle -sys \
                            $SCOPE_ENGINE(SYSTEM_ID) -sub \
                            $SCOPE_ENGINE(MTR_STATUS) \
                            -lun 1 -cmd "handle_scope_status" ]
}


#
#  Message bus interfaces - removed from use
#
proc destroy_scope_callback { } {
  global SCOPE_ENGINE

  if {$SCOPE_ENGINE(handler) != 0 } {
    tclMsgbus $SCOPE_ENGINE(handler) destroy
  }

  set SCOPE_ENGINE(handler) 0
}

#############################################################################
#                                                                           #
#                       Engine Interfaces                                   #
#                                                                           #
#############################################################################


#
#  Set Scope sweep ( uS/Div... 1uS/div to 100mS/div )
#
#  this interface applies to all LUN's (actually could 
#  use any LUN to set sweep.
#
#
proc set_scope_sweep { uS } {
  global SCOPE_ENGINE

  set sweep [ expr $uS * 10 ]
  set sweep [ expr int($sweep) ]

  message_bus_int $SCOPE_ENGINE(SYSTEM_ID) $SCOPE_ENGINE(MTR_SWEEP) 1 $sweep
}


#
#  Set Scope vertical scale (lun = 1|2) (Volts/Div)
#
proc set_scope_scale { lun vdiv } {
  global SCOPE_ENGINE

  switch $vdiv {
    GROUND   { set scale 0  }
     10MV    { set scale 1  }
     20MV    { set scale 2  }
     50MV    { set scale 3  }
    100MV    { set scale 4  }
    200MV    { set scale 5  }
    500MV    { set scale 6  }
       1V    { set scale 7  }
       2V    { set scale 8  }
       5V    { set scale 9  }
      10V    { set scale 10 }
      20V    { set scale 11 }
      50V    { set scale 12 }
     default { return }
  }

  message_bus_int $SCOPE_ENGINE(SYSTEM_ID) $SCOPE_ENGINE(MTR_SCALE) $lun $scale
}


#
#  Set Scope input coupling ( AC or DC )
#
proc set_scope_coupling { lun acdc } {
  global SCOPE_ENGINE

  switch $acdc {
    AC_T   { set coupling 0 }
    DC_T   { set coupling 1 }
   default { return }
  }

  message_bus_int $SCOPE_ENGINE(SYSTEM_ID) $SCOPE_ENGINE(MTR_COUPLING) $lun $coupling
}


#
#  Set Scope horizontal offset from trigger point ( +/- trace length )
#
proc set_scope_horiz { lun horiz } {
  global SCOPE_ENGINE

  set SCOPE_ENGINE(horiz) $horiz

  message_bus_int $SCOPE_ENGINE(SYSTEM_ID) $SCOPE_ENGINE(MTR_HPOS) $lun $horiz
}

proc move_scope_horiz { step } {
  global SCOPE_ENGINE

  set horiz $SCOPE_ENGINE(horiz)
  incr horiz $step

  set_scope_horiz 1 $horiz
}

proc set_scope_offset { offset } {
  set_scope_horiz 1 $offset
}


#
#  Set Scope trigger type - normal, one-shot, auto
#
proc set_scope_trig_type { trigtype } {
  global SCOPE_ENGINE

  switch $trigtype {
    STRIG_AUTO   { set type 0 }
    STRIG_NORMAL { set type 1 }
    STRIG_1SHOT  { set type 2 }
        default  { return }
  }

  message_bus_int $SCOPE_ENGINE(SYSTEM_ID) $SCOPE_ENGINE(MTR_TRIG_TYPE) 1 $type
}


#
#  Set Scope trigger source - external input or scope channels 1/2
#
proc set_scope_trig_source { trigsource } {
  global SCOPE_ENGINE

  switch $trigsource {
    STRIG_EXT  { set source 0 }
    STRIG_CH1  { set source 1 }
    STRIG_CH2  { set source 2 }
     default   { return }
  }

  message_bus_int $SCOPE_ENGINE(SYSTEM_ID) $SCOPE_ENGINE(MTR_TRIG_SOURCE) 1 $source
}


#
#  Set Scope trigger level ( range is -500.0 to 500.0 Volts )
#
proc set_scope_trig_level { mV } {
  global SCOPE_ENGINE

  set volts [ expr { double( $mV ) / 1000 } ]

  message_bus_double $SCOPE_ENGINE(SYSTEM_ID) $SCOPE_ENGINE(MTR_TRIG_LEVEL) 1 $volts
}


#
#  Arm Scope Trigger
#
proc scope_arm_trig { } {
  global SCOPE_ENGINE

  message_bus_int $SCOPE_ENGINE(SYSTEM_ID) $SCOPE_ENGINE(MTR_TRIG_ARM) 1 1
}

#
#  Arm Scope Trigger Polarity Selection (positive or negative edge)
#
proc scope_trigger_polarity { polarity } {
  global SCOPE_ENGINE

  switch $polarity {
    POS  { set pol 1 }
    NEG  { set pol 0 }
 default { return }
  }

  message_bus_int $SCOPE_ENGINE(SYSTEM_ID) $SCOPE_ENGINE(MTR_TRIG_POL) 1 $pol
}


#
#  Supply Calibration to current Scope Vertical scale
#      (-2048 to 2047, default is 0)
#
proc scope_cal_range { ch index cal } {
  global SCOPE_ENGINE
  global Send_Flags

  set msg " { $SCOPE_ENGINE(SYSTEM_ID) $SCOPE_ENGINE(MTR_CAL_RANGE) \
                                       $ch $Send_Flags } \
            { inline udword $index } \
            { inline udword $cal } "
#puts $msg
  tclMsgbus send $msg

}


proc do_scope_color { path ch } {
  global SCOPE_ENGINE

  set temp [ $path configure -background ]
  set SCOPE_ENGINE($ch) [ lindex $temp 4 ]
}

#
#  Set Scope Run state - enable DSP & VxWorks Engine to send trace data to TCL
#
#  to stop traces, only need to issue a STOP, kills all running scopes
#
proc scope_run { state { dsp DSP_SC } } {
  global SCOPE_ENGINE

  do_scope_color .demoscope.mbar.ch1  color_ch1
  do_scope_color .demoscope.mbar.ch2  color_ch2
  do_scope_color .demoscope.mbar.ch3  color_ch3

  $SCOPE_ENGINE(name).trace1 config -fg $SCOPE_ENGINE(color_ch1)
  $SCOPE_ENGINE(name).trace2 config -fg $SCOPE_ENGINE(color_ch2)
  $SCOPE_ENGINE(name).trace3 config -fg $SCOPE_ENGINE(color_ch3)

  switch $state {
    RUN    { set st 1 }
    STOP   { set st 0 }
  default  { return }
  }

  switch $dsp {
    DSP_SC  { message_bus_int $SCOPE_ENGINE(SYSTEM_ID) \
                              $SCOPE_ENGINE(MTR_RUN) 1 $st
            }
    DSP_MF  { message_bus_int $SCOPE_ENGINE(SYSTEM_ID) \
                              $SCOPE_ENGINE(MTR_RUN) 3 $st }
  }
}


#
#  Set Scope engine MB messages to be sent
#
proc set_scope_enable { lun state } {
  global SCOPE_ENGINE

  switch $state {
    DEMO -
    TRACE_STAT_MSG  { set event $SCOPE_ENGINE(MB_EVENTS) }
               OFF  { set event 0 }
    default { return }
  }

  message_bus_int $SCOPE_ENGINE(SYSTEM_ID) $SCOPE_ENGINE(MTR_ENABLE) $lun $event
}


#
#  Set Scope calibration values - digital pots U35, U40 (the remaining 
#              are NOT installed)
#
proc set_scope_x5_cals { type value } {
  global SCOPE_ENGINE

  set refdes 0

  switch $type {
    U35  { set refdes 0x35 }
    U38  { set refdes 0x38 }
    U40  { set refdes 0x40 }
    U43  { set refdes 0x43 }
  default { return }
  }

  scope_cal_x5 $SCOPE_ENGINE(SYSTEM_ID) \
               $SCOPE_ENGINE(MTR_CAL_X5) 1 $refdes $value 
}

#############################################################################
#                                                                           #
#                       Calibration Interfaces                              #
#                                                                           #
#############################################################################


#
# Scope Calibration parser
#
proc parse_scope { line } {
  global SCOPE_ENGINE

  set refdes [ lindex $line 3 ]
  set ch     [ lindex $line 1 ]
  set value  [ lindex $line 5 ]

  switch [ lindex $line 1 ] {
    X5STAGE { set SCOPE_ENGINE($refdes)    $value }
    CH1     { set SCOPE_ENGINE($refdes$ch) $value }
    CH2     { set SCOPE_ENGINE($refdes$ch) $value }
  }

#puts "scope cal:  $refdes $ch $value "
}


#
# Scope Calibrations: get TCL data file
#
proc get_scope_cal_data { filename } {
  global CalFileName

  set file [ OpenCalFile $filename r ]

  while { [ gets $file line ] >= 0 } {
    if { [ lindex $line 0 ] == "SCOPECAL" } {
      parse_scope $line
    }
  }

  close $file
}


#
# Send Scope Cals to Vxworks scope engine:
#
proc update_scope_calibration { } {
  global SCOPE_ENGINE

#  set_scope_x5_cals U35 $SCOPE_ENGINE(U35CAL)
#  set_scope_x5_cals U40 $SCOPE_ENGINE(U40CAL)

  set ranges { 20MV 50MV 100MV 200MV 500MV 1V 2V 5V 10V 20V 50V }
  
  set index 0
  foreach i $ranges {
    set arg [ format "%sCH1" $i ]
#    scope_cal_range 1 $index $SCOPE_ENGINE($arg)
    incr index
  }

  set index 0
  foreach i $ranges {
    set arg [ format "%sCH2" $i ]
#    scope_cal_range 2 $index $SCOPE_ENGINE($arg)
    incr index
  }
}

