#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C) 2001      DESIGN ENGINEERING        #
# --------------------------------------------------------------------------#
# FILE        : alert.tcl                                                   #
# PRODUCT     : FITS                                                        #
# AUTHOR      : David Hagood                                                #
# DESCRIPTION : Tcl interface for the alert engine                          #
#                                                                           #
#############################################################################
trace vdelete ::Alert::State ruw "::Alert::Load"

#****************************************************************************
# Handle a load of the C++ interface
#****************************************************************************
if {"[info command Alert]" != "" } { return }

# Call the base class dummy command to force a load
package require Engtclbase
Engtclbase

# Load the object file
::load $::Alert::object

namespace eval ::Alert \
{
   variable Types
   variable State
   array set State \
   {
      status     0
      status,raw 0
      
      types \
        {
           NO_ALARM
           LOW_ALARM	
           HI_ALARM
           HI_LO_ALARM
           ALARM_ENABLED
           INFO
           QUESTION
           COMPLETE
           WARNING
           CRIT_WARNING
        }
   }
     
   proc updateState { state } \
   {
      variable State
      set State(status,raw) $state
      set State(status) [lindex $State(types) $state]
   }
#****************************************************************************
# Name:        getSaveState
# Parameters:  none
# Description: This function returns a list of TCL commands which can be used
#              to set the BB Audio engine to its current state
#****************************************************************************
 proc getSaveState { } \
 {
   variable State
   set returnList \
   {
      Alert vol $State(volume)
   }
   return [subst -nocommands $returnList]
 }
 
#****************************************************************************
# Name:        AddMeter
# Parameters:  Namespace of meter
# Description: add a possible alert source to the list of alert sources
#              Such namespaces are expected to provide the following items:
#              toggleAlarm - change alarm enable on/off
#              State(alarm_text) - shows alarm ENABLE state
#              State(alarm) - shows alarm ACTIVE/inactive
#              name - name string of meter
#****************************************************************************
 proc AddMeter { Namespace } \
 {
    variable Meter_list
    lappend $Namespace
 }
 
 Alert -onchange state { Alert::updateState %V}
 Alert -onchange vol   { set Alert::State(volume) %V}

}

#****************************************************************************
# This registers the engine with the list for the version screen
#****************************************************************************
namespace eval ::Version_Screen \
{
   variable version
   variable names
   
   array set names \
   {
     Alert_Eng "Alert Manager Engine"
   }
   set version(Alert_Eng) [Alert version]
}

#****************************************************************************
# This registers the engine with the list for saving instrument states
#****************************************************************************
namespace eval ::SavedStates {

   variable engines   
   lappend engines Alert
}
