#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C) 2001      DESIGN ENGINEERING        #
# --------------------------------------------------------------------------#
# FILE        : dhcp.tcl                                                    #
# PRODUCT     : FITS                                                        #
# AUTHOR      : David Hagood                                                #
# DESCRIPTION : DHCP interface routines                                     #
#############################################################################
package require Callback
Callback
package require DNS

namespace eval ::DHCP \
{
   ::load $object
   variable    interfaces {fet0 fei0}
   set State(LOGGING) 0
   set State(BOUND) 0
   
   proc Log { msg } \
   {
      variable State
      if {$State(LOGGING) != 0} \
      {
         catch \
         {
            Debug::Puts DHCP 0 $msg
         }
      }
   }
#****************************************************************************
# Name:        Response
# Parameters:  Packet data, as parsed by C interface
# Description: Handle a State packet from the DHCP server
#****************************************************************************
   proc Response { packet } \
   {
      variable State
      # stupid checks - I AM in DHCP mode, right?
      if {$State(Mode) != "DHCP" } \
      {
         Debug::Puts DHCP 2 "Not in DHCP mode: $packet"
         return
      }
      # and I'm running, right?
      if {$State(state) == "IDLE" } \
      {
         Debug::Puts DHCP 2 "Not running: $packet"
         return
      }
      # bust the packet into an array
      array set parts $packet
      # This is a State, right?
      if {$parts(OPCODE) != "BOOTREPLY" } \
      {
         Debug::Puts DHCP 2 "Not a boot reply: $packet"
         return
      }
      # For me, right?
      if {$parts(HW_ADDR) != [DHCP hw]} \
      {
         Debug::Puts DHCP 2 "Not for me: $packet"
         return
      }
      # my current transaction ID?
      if {$parts(TRANSACTION_ID) != [DHCP trans] } \
      {
         Debug::Puts DHCP 2 "Not my transaction ID: $packet"
         return
      }

      Debug::Puts DHCP 5 "Rx packet $packet"

      # OK, go to the state routine
      switch -- $parts(DHCP Message type) \
      {
         "OFFER" { Offer $packet}
         "ACK"   { Ack   $packet}
         "NACK"  { Nack  $packet}
         default \
         {
            Debug::Puts DHCP 2 "Unknown opcode: $packet"
            return
         } 
      }
   }
#****************************************************************************
# Name:        Offer
# Parameters:  
# Description: Handle a State packet from the DHCP server
#****************************************************************************
   proc Offer { packet } \
   {
      variable State
      
      if {$State(state) != "DISCOVER"} \
      {
         Debug::Puts DHCP 2 "Got OFFER when not in DISCOVER"
         return
      }
      # OK, got an offer. Send the request, start the timeout, and move on
      after cancel DHCP::Timeout
      after cancel DHCP::Timeout_core

      set State(state) "REQUEST"
      # copy data over to state variable
      array set State $packet
      after 1000 DHCP::Timeout
      if {$State(BOUND) != 0} \
      {
         DHCP request BOUND
      } \
      else \
      {
         DHCP request
      }
   }
#****************************************************************************
# Name:        Ack
# Parameters:  
# Description: Handle a State packet from the DHCP server
#****************************************************************************
   proc Ack { packet } \
   {
      variable State
      
      if {$State(state) != "REQUEST"} \
      {
         Debug::Puts DHCP 2 "Got ACK when not in REQUEST"
         return
      }
      
      # OK, got an address. Can we use it?
      after cancel DHCP::Timeout
      after cancel DHCP::Timeout_core
      
      # copy data over to working variable
      array set work $packet
      
      if {$work(YOUR_IP) == $State(CLIENT_IP)} \
      {
         Log "Got our old address back"
         set State(squatter_MAC) ""
         # copy data over to main array
         array set State $packet
         Accept
         return
      }

      set addr [DHCP arp $work(YOUR_IP)]

      if {$addr == "" } \
      {
         # this is what we WANT - nobody home. So accept this address
         set State(squatter_MAC) ""
         # copy data over to main array
         array set State $packet
         Accept
         return
      }
      Log "Damn squatter at $addr has this address"
      # No, somebody has that address already. Decline and restart
      Debug::Puts DHCP 1 "Damn squatter at $addr has this address"
      set State(squatter_MAC) $addr
      set State(state) "DECLINE"
      DHCP decline
      after 1000 DHCP::Start
   }
#****************************************************************************
# Name:        Nack
# Parameters:  
# Description: Handle a State packet from the DHCP server
#****************************************************************************
   proc Nack { packet  } \
   {
      variable State
      if {$State(state) != "REQUEST"} \
      {
         Debug::Puts DHCP 2 "Got NACK when not in REQUEST"
         return
      }
      after cancel DHCP::Timeout
      after cancel DHCP::Timeout_core

      # Server won't give us what we want... reset everything and try again
      Debug::Puts DHCP 1 "Got NACK"
      set State(state) "NACK"
      set State(CLIENT_IP) 0.0.0.0
      set State(YOUR_IP)   0.0.0.0
      Config
      after 1000 DHCP::Start
   }
#****************************************************************************
# Name:        Timeout
# Parameters:  
# Description: Handle timeout on receiving a DHCP response
#****************************************************************************
   proc Timeout {  } \
   {
      variable State
      # we may have received the packet, timed out, and been busy.
      after cancel DHCP::Timeout_core
      after idle DHCP::Timeout_core
   }
   proc Timeout_core {} \
   {
      Debug::Puts DHCP 1 "Timeout"
      DHCP timeout -1
      set State(state) "DECLINE"
      after 1000 DHCP::Start
   }
   
#****************************************************************************
# Name:        Accept
# Parameters:  
# Description: Accept IP address from server
#****************************************************************************
   proc Accept {  } \
   {
      variable State
      
      set State(state) "ACCEPT"
      
      Debug::Puts DHCP 1 "Accepted State"

      Log "accept client $State(CLIENT_IP) your $State(YOUR_IP)"
      set State(CLIENT_IP) $State(YOUR_IP)
      # start our timer to re-up
      set State(aquired) [clock seconds]
      set State(expires) [expr $State(aquired) + $State(Renew time)]
      set State(aquired,text) [clock format $State(aquired)]
      set State(expires,text) [clock format $State(expires)]
      DHCP timeout $State(Renew time)
      set State(BOUND) 1
      Config
      Write_config
      DNS servers $State(DNS)
   }
#****************************************************************************
# Name:        Config
# Parameters:  
# Description: Set ethernet up with our settings.
#****************************************************************************
   proc Config { } \
   {
      variable State
      # configure our interface, set the timeout, etc.
      # Force netmask to none first, to set route.
      oldCatch {DHCP config -netmask 0.0.0.0}
      oldCatch {DHCP config -hostname $State(Host Name) -address $State(CLIENT_IP) \
                  -netmask $State(Subnet) -gateway $State(Router)}
      DHCP broadcast
      oldCatch {DHCP config -hostname $State(Host Name) -address $State(CLIENT_IP) \
                  -netmask $State(Subnet) -gateway $State(Router)}
      DHCP broadcast
   }
   
#****************************************************************************
# Name:        Renew
# Parameters:  
# Description: Renew our lease
#****************************************************************************
   proc Renew { } \
   {
      variable State
      
      # Send the request, start the timeout, and move on
      after cancel DHCP::Timeout
      after cancel DHCP::Timeout_core
      after 1000 DHCP::Timeout
      set State(state) "REQUEST"
      if {$State(BOUND) != 0} \
      {
         DHCP request BOUND
      } \
      else \
      {
         DHCP request
      }
   }
#****************************************************************************
# Name:        Start
# Parameters:  
# Description: Start a DHCP transaction sequence
#****************************************************************************
   proc Start {  } \
   {
      variable State
      if {$State(Mode) == "DHCP" } \
      {
         set State(state) DISCOVER
         Log "Transaction is [DHCP transaction new], current IP is $State(CLIENT_IP)"
         DHCP timeout -1
         after 1000 DHCP::Timeout
         DHCP discover
      } \
      else \
      {
         set State(BOUND) 0
         set State(state) IDLE
         DHCP timeout -1
         Config
      }
   }
   
#****************************************************************************
# Name:        Read_config
# Parameters:  
# Description: Read the config file
#****************************************************************************
   proc Read_config { } \
   {
    variable State
     
    # OPEN FILE
    set line ""
    if ![file exists $State(filename)] \
    {
      Debug::Puts DHCP 1 "Config file $State(filename) does not exist"
    }
    Debug::Puts DHCP 1 "Config file exists"
    if [catch {set f [open $State(filename) r]} err] \
    {  
       Debug::Puts DHCP 1 "Couldn't open $State(filename) - $err"
       return
    }
    if [catch {set line [read $f]} err] \
    {
       Debug::Puts DHCP 1 "Couldn't read $State(filename) - $err"
    }
    close $f

    Debug::Puts DHCP 5  "line is $line"
    set State(CLIENT_IP) [lindex $line 1]
    set State(YOUR_IP)   $State(CLIENT_IP)
    set State(user_id)   [lindex $line 2]
    set State(group_id)  [lindex $line 3]
    set State(Subnet)    [lindex $line 4]
    set State(Router)    [lindex $line 5]
    set State(Mode)      [lindex $line 6]
    set {State(Host Name)} [lindex $line 0]
    set State(state) "FIXED CONFIG"
    if {$State(Mode) == ""} \
    {
       set State(Mode) "Fixed"
    }
    Config
  }
#****************************************************************************
# Name:        Write_config
# Parameters:  
# Description: Write the config file
#****************************************************************************
   proc Write_config { } \
   {
    variable State
     
    # OPEN FILE
    if [catch {set f [open $State(filename) w]} err] \
    {  
       Debug::Puts DHCP 1 "Couldn't open $State(filename) - $err"
       return
    }

     if [catch {puts $f "$State(Host Name) $State(CLIENT_IP) $State(user_id) $State(group_id) $State(Subnet) $State(Router) $State(Mode)"} err] \
    {
       Debug::Puts DHCP 1 "Couldn't write $State(filename) - $err"
    }
    close $f
  }
#****************************************************************************
# Name:        Toggle_mode
# Parameters:  
# Description: Switch between DHCP and Fixed modes
#****************************************************************************
  proc Toggle_mode { } \
  {
     variable State
     if {$State(Mode) == "Fixed"} \
     {
        set State(Mode) "DHCP"
        Renew
        return
     } \
     else \
     {
        set State(Mode) "Fixed"
        Write_config
     }
     after idle DHCP::Start
  }

#****************************************************************************
# Name:        At_exit
# Parameters:  
# Description: Release any lease we own at shutdown
#****************************************************************************
   proc At_exit { } \
   {
      variable State
      upvar reboot_flag reboot_flag
      if {$State(Mode) == "DHCP" } \
      {
         # if we don't know why we are going down, or if we know we are going down
         # cold, then release the license. If we are going down for a reboot, don't.
         if {($reboot_flag == "") || ($reboot_flag == 0)} \
         {
            DHCP release $State(Server ID)
         }
      }
   }
#****************************************************************************
# Name:        At_start
# Parameters:  
# Description: Startup code
#****************************************************************************
   proc At_start { } \
   {
     variable State
     variable interfaces
     
     DHCP cmd ::DHCP::Response
     DHCP renew ::DHCP::Renew
     foreach interface $interfaces \
     {
         if [DHCP find $interface] \
         {
            DHCP int $interface
            set State(interface) $interface
            break
         }
     }
     set State(HW_ADDR) [DHCP hwa]
     Read_config
     at_exit DHCP::At_exit
     if {$State(Mode) != "Fixed" }\
     {
       # attempt to renew our old lease - else pick up a new one
       Renew
     }
   }
   DHCP::At_start
}
