#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C) 2002      DESIGN ENGINEERING        #
# --------------------------------------------------------------------------#
# FILE        : dsploader.tcl                                               #
# PRODUCT     : FITS                                                        #
# AUTHOR      : Hieu Tran                                                   #
# DESCRIPTION : This file is sourced by TCL when the Dspload command is     #
#             : first used.  It loads the C++ interface file, provides a    #
#             : namespace for screen variables, and sets up callbacks to    #
#             : update the variables for the UI.                            #
#                                                                           #
#############################################################################
trace vdelete ::Dsploader::State ruw "::Dsploader::Load"

#****************************************************************************
# Handle a load of the C++ interface
#****************************************************************************
if {"[info command Dsploader]" != "" } { return }

# Call the base class dummy command to force a load
package require Engtclbase
Engtclbase


# Load the object file
::load $::Dsploader::object

package provide Dsploader 1.0

namespace eval ::Version_Screen \
{
   variable version
   variable names
   
   array set names \
   {
     DSP_DRIVER "CPU to DSP driver"
     DSP_MF "Audio DSP"
     DSP_PM "Parametric DSP"
     DSP_L1 "Layer 1 DSP"
     DSP_SC "Scope DSP"
   }
   set version(DSP_DRIVER) ""
   set version(DSP_MF) ""
   set version(DSP_PM) ""
   set version(DSP_L1) ""
   set version(DSP_SC) ""
}

namespace eval ::Dsploader \
{
   variable Errors
   variable OptionInfo
   variable Log_routine_errs 0
   
   array set Errors \
   {
      0 "INFORMATION"
      1 "BAD_CMD"
      2 "BAD_LEN"
      3 "BAD_ARG"
      4 "SLAVE_DEAD"
      5 "NO_MEM"
      6 "HW_FAIL"
      7 "SW_FAIL"
      8 "CMD_FULL"
   }
   variable names
   array set names \
   {
      0 MAIN
      1 Parametric
      2 "Layer 1"
      3 "Audio"
      4 "Scope"
      5 "Downconverter"
      6 "Upconverter"
      7 "FPGA"
   }
   
   # log errors from DSPs into error log
   proc LogError { v } \
   {
      variable Errors
      variable names
      variable Log_routine_errs
      set err [lindex $v 0]
      set cmd [lindex $v 1]
      set msg [lrange $v 2 end]
            
      set from [expr ($cmd>>20)&15]
      set to   [expr ($cmd>>16)&15]
      if [info exists Errors($err)] \
      {
         set errmsg $Errors($err)
      } \
      else \
      {
         set errmsg "Error($err)"
      }
      # Don't log "routine" comm failures unless told to
      if {!$Log_routine_errs} \
      {
          switch $errmsg \
          {
             SW_FAIL    -
             SLAVE_DEAD \
             {
                puts "DSP Error: $errmsg $$from->$to ([format %06x $cmd]) $msg"
                return 
             }
          }
      }
      if [info exists names($from)] {set from $names($from)}
      if [info exists names($to)] {set from $names($to)}
      ErrLog::Log "DSP Error: $errmsg $$from->$to ([format %06x $cmd]) $msg"
   }
   proc Version { msg } \
   {
      # we need to split this so CVS won't munge it
      set pattern {(.*), (.*), \$}
      append pattern {Name: (.*) \$}
      if {[regexp $pattern $msg x date time ver] == 1} \
      {
         set ::Version_Screen::version(DSP_DRIVER) "$date, $time, $ver"
      }
   }
   proc Vinfo { which msg } \
   {
      # we need to split this so CVS won't munge it
      set pattern {(.*);\$}
      append pattern {Name: (.*) \$\;((.*),(.*));(.*)}

      if {[regexp $pattern $msg x model ver datetime date time extra] == 1} \
      {
         set ::Version_Screen::version($which) "$which $date, $time, $ver, $extra"
      }
   }
   proc OptionVersion { msg } \
   {
      variable OptionInfo

      set mark [string first ":" $msg]
      set index [string range $msg 0 [expr $mark - 1] ]
      set value [string range $msg [expr $mark + 1] end]
      set OptionInfo($index) $value
      set OptionInfo(aes) $msg
   }
   Dsploader -onchange err {Dsploader::LogError %V}
   Dsploader -onchange version  "Dsploader::Version %V"
   Dsploader -onchange vinfo1   "Dsploader::Vinfo DSP_PM %V"
   Dsploader -onchange vinfo2   "Dsploader::Vinfo DSP_L1 %V"
   Dsploader -onchange vinfo3   "Dsploader::Vinfo DSP_MF %V"
   Dsploader -onchange vinfo4   "Dsploader::Vinfo DSP_SC %V"

   Dsploader -onchange optversion0   "Dsploader::OptionVersion %V"

   set ::Debug::enable(DSP) 10
}


