namespace eval ::GPIB {
 variable saveafter {}
 
 set file [GPIB file]
 
 # in case we have no setup, set the variables
   
 proc Input { } {
   variable slave
   variable file
   Debug::Puts GPIB 1 "Reading"
   set line [read $file]
   Debug::Puts GPIB 1 "Read $line"
   ::Slave::Input $slave $line
   Debug::Puts GPIB 1 "Did $line"
 }
 
 proc Puts { args } \
 {
     variable slave
     puts "GPIB: somebody called puts $args"
     CallTrace
     return [eval ::Slave::Puts $slave $args]
 }
 
 proc Init {} {
    variable file
    variable statefile
    variable myaddr
    variable mysubaddr
    variable slave
    GPIB myad    $myaddr:$mysubaddr
    GPIB timeout $::GPIB::timeout
    set slave [::Slave::New $file ]
    upvar 0 $slave  gpib_slave

    fconfigure $file -blocking false -buffering none -translation {auto cr}
    fileevent  $file readable "::GPIB::Input"
    ::Slave::Echo $::GPIB::slave OFF
    set gpib_slave(TRIM) 1
    interp alias $gpib_slave(NAME) puts {} ::GPIB::Puts
    after idle "$gpib_slave(NAME) eval {package require ALL; ALL}"
 }
 
 proc Save {} {
    variable saveafter
    if {"$saveafter"!=""} {
       after cancel $saveafter
    }
    set saveafter [after 100 { ::GPIB::DoSave }]
 }
 
 proc DoSave { } {
    variable statefile
    variable myaddr
    variable mysubaddr
    variable timeout
    variable saveafter
    set saveafter {}
    
    file mkdir [file dirname $statefile]
    set f [ open $statefile w]
    puts $f "set ::GPIB::timeout   $timeout"
    puts $f "set ::GPIB::myaddr    $myaddr"
    puts $f "set ::GPIB::mysubaddr $mysubaddr"
    # other commands here as needed...
    close $f
 }
 
 proc Addr { {addr {}} {subaddr {}}} {
    variable myaddr
    variable mysubaddr
    if {"$addr"!=""}    { set myaddr $addr}
    if {"$subaddr"!=""} { set mysubaddr $subaddr}
    set newaddr [GPIB myad $myaddr:$mysubaddr]
    set a [split $newaddr :]
    set myaddr [lindex $a 0]
    set mysubaddr [lindex $a 1]
    Save
 }
 proc Timeout { tout } {
    variable timeout
    if {$timeout != $tout } {
        set timeout [GPIB timeout $tout]
        Save
    }
 }
 
 proc Get_Srq { } {
    set srq [GPIB SRQ]
    return $srq
 }
 proc Set_Sre { mask } {
    set sre [GPIB SRE $mask]
    return $sre
 }
 proc Get_Sre { } {
    set sre [GPIB SRE]
    return $sre
 }
 
 
 proc Slave_Addr { slave {addr {}} {subaddr {}}} \
 {
    return [Addr $addr $subaddr]
 }
 proc Slave_Timeout { slave {tout {}}} \
 {
    return [Timeout $tout]
 }

 proc Slave_Get_Srq { slave } {
    return [Get_Srq]
 }

 proc Slave_Get_Sre { slave } {
    return [Get_Sre]
 }

 proc Slave_Set_Sre { slave value } {
    return [Set_Sre $value]
 }

  ::Slave::Add_alias ::GPIB::Addr ::GPIB::Slave_Addr
  ::Slave::Add_alias ::GPIB::Timeout ::GPIB::Slave_Timeout
  ::Slave::Add_alias *STB? ::GPIB::Slave_Get_Srq
  ::Slave::Add_alias *stb? ::GPIB::Slave_Get_Srq
  ::Slave::Add_alias *SRE? ::GPIB::Slave_Get_Sre
  ::Slave::Add_alias *sre? ::GPIB::Slave_Get_Sre
  ::Slave::Add_alias *SRE  ::GPIB::Slave_Set_Sre
  ::Slave::Add_alias *sre  ::GPIB::Slave_Set_Sre
}


