namespace eval ::Generator \
{
 variable SaveSortOrder AFTER(Analyzer)
 proc updateLockStatus { lockstat } \
 {
   variable State
   
   set State(lockstat,raw) $lockstat
   
   # if the generator is OFF, we pretend it is locked...
   if {$State(dcpower) != "ON" } { set lockstat 0}
   
   set State(lock,main)     [is_locked $lockstat 1]
   set State(lock,clean)    [is_locked $lockstat 2]
   set State(lock,step)     [is_locked $lockstat 4]
   set State(lock,ref)      [is_locked $lockstat 8]

   set State(lock_unstable) [expr $lockstat & 0x8000]
   set State(lockstat)      [expr $lockstat & 0x7FFF]
   
   # if we are unlocked and we've been unlocked for a while, start a cal
   if {$State(lock_unstable) == 0} \
   {
      if {$State(lockstat) != 0} \
      {
         # error log any unlocks
         if {$State(lockstat) != 0 } \
         {
            ::ErrLog::Log "Generator unlock detected: lock status $State(lockstat)"
         }
         if {$State(lock,cal_request) == 0} \
         {
            set State(lock,cal_request) 1
            ::HW_control::StartYigCal
         }
      } \
      else \
      {
         # locked now, so if we UNLOCK, we need to recal
         set State(lock,cal_request) 0
      }
   }
 }
 proc SyncAll { } \
 {
    variable State
    set State(dcpower) 1
    set State(enable) 27
    set State(alc) 0
    set State(hwbw_raw) 0
    HW_control::OnYigDone \
    {
        Generator dcpower on -nowait
    }
 }

 proc updateDCpower { state } {
   variable State
   switch $state {
      0       { set State(dcpower) OFF }
      1       { 
                 if {$State(dcpower) != "ON"} \
                 {
                    set State(dcpower) ON
                    HW_control::OnYigDone Generator::Sync
                 }
              }
      default { return }
   }
 }
 
  Load::on_hwinit_done  { ::Generator::SyncAll }

#****************************************************************************
# Name:        Get_HTML
# Parameters:  base_url - the base URL for the containing page
# Description: Return an string containing HTML showing the current state
#              of this system
#****************************************************************************
 proc Get_HTML { {base_url ""} } \
 {
    variable State
    set x [subst \
{<table align="left" border=1>
<tr><td nowrap colspan=2><center><h2>RF Generator setup</h2></center></td></tr>
<tr><td nowrap>Frequency</td>         <td nowrap>$State(freq) MHz</td></tr>
<tr><td nowrap>Output port</td>       <td nowrap>$State(rfout)</td></tr>
<tr><td nowrap>Modulation format</td> <td nowrap>$State(mod)</td></tr>
<tr><td nowrap>DC Power</td>          <td nowrap>$State(dcpower)</td></tr>
<tr><td nowrap>Carrier</td>           <td nowrap>$State(ptt)</td></tr>
<tr><td nowrap>RF Level</td>          <td nowrap>$State(rflev,dBm) dBm</td></tr>
}]
   # need to add modulator stuff here
   append x "</table>"
   return $x
 }

}
  
# Now, all the deprecated functions used by the CAL station code...
 proc ::gencal_freq { MHz } {
  puts "gencal_freq DEPRECATED: use Generator debug rawfreq"
  CallTrace
  Generator debug rawfreq $MHz -nowait
 }


#****************************************************************************
# This registers the engine with the list for saving instrument states
#****************************************************************************
namespace eval ::SavedStates {

   variable engines   
   lappend engines Generator
   
   variable pre_restore_cmds
   
   lappend pre_restore_cmds {Generator::Trunking_mode}
}


#****************************************************************************
# This registers the engine with the list for generating an HTML page
#****************************************************************************
namespace eval ::HTTPD \
{
   variable main_screen_items
   lappend  main_screen_items(1) Generator
}


#****************************************************************************
# This registers the engine with the list for the version screen
#****************************************************************************
HW_control::OnYigDone \
{
 namespace eval ::Version_Screen \
 {
   variable version
   variable names
   
   array set names \
   {
     Gen_Eng "RF Generator Engine"
     Gen_Drv "RF Generator Driver"
     Gen_HW  "RF Generator Hardware"
   }
   set version(Gen_Eng)   "[Generator ver 0]"
   set version(Gen_Drv)   "[Generator ver 1]"
   set version(Gen_HW)    "Revision [Generator ver 2]"
 }
}
