#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT  2002        DESIGN ENGINEERING        #
# --------------------------------------------------------------------------#
# FILE        : gzip.tcl                                                    #
# PRODUCT     : FITS                                                        #
# AUTHOR      : Hieu Tran                                                   #
# DESCRIPTION : This file is sourced by TCL when the Gzip command is        #
#             : first used.  It loads the C++ interface file, provides a    #
#             : namespace for screen variables, and sets up callbacks to    #
#             : update the variables for the UI.                            #
#                                                                           #
#############################################################################
trace vdelete ::Gzip::State ruw "::Gzip::Load"

#****************************************************************************
# Handle a load of the C++ interface
#****************************************************************************
if {"[info command Gzip]" != ""} { return }

# Call the base class dummy command to force a load
package require Engtclbase
Engtclbase

# Load the object file
::load $::Gzip::object

package provide Gzip 1.0


namespace eval ::Gzip {

   array set gzip_error_str \
   {
      0 "Open gzip file"
      1 "Create output file"
      2 "Read gzip file"
      3 "Write output file"
      4 "Not in gzip format"
      5 "Unknown suffix -- ignored"
      6 "Gzip file not found"
      7 "Couldn't start gzip"
      8 "Error decompressing"
      256 "Start"
      257 "Done"
   }

#****************************************************************************
# Name:        updateGzipError
# Parameters:  none
# Description: 
#****************************************************************************
 proc updateGzipError { code } \
 {
   variable State
   variable gzip_error_str
   
   switch -- $code \
   {
      256 { set State(gzip_done) 0; return }
      257 { set State(gzip_done) 1; return }
   }

   set State(gziperrcode) $code

   if [info exists gzip_error_str($code)] \
   {
      set State(gziperrstr) $gzip_error_str($code)
   }
 }


#****************************************************************************
# Initialize the namespace and activate the callbacks
#****************************************************************************
 Gzip -onchange error   { ::Gzip::updateGzipError %V; }
 Gzip -onchange percent { set ::Gzip::State(gzippercent) %V; }

}

