variable SaveSortOrder FIRST

#****************************************************************************
# This registers the engine with the list for saving instrument states
#****************************************************************************
namespace eval ::SavedStates {

   variable engines   
   lappend engines HW_control
}

#****************************************************************************
# This registers the engine with the list for the version screen
#****************************************************************************
namespace eval ::Version_Screen \
{
   variable version
   variable names
   
   array set names \
   {
     HW_Eng "HW Control Engine"
     HW_Drv "HW Control Driver"
   }
   set version(HW_Eng) [HW_control version 0]
   set version(HW_Drv) [HW_control version 1]
}

array set State \
{
   tcxo_cal    /system/calibration/tcxo
   rcv_yig_cal /system/calibration/rcvr_yig
   gen_yig_cal /system/calibration/rfgen_yig 
}

proc write_gen_yig_cals {  } \
{
   variable State
   set fileId [open $::HW_control::State(gen_yig_cal) w]
       
   puts $fileId "# Test Set Calibration file [clock format [clock seconds]]"
   puts $fileId " "
   puts $fileId "#"
   puts $fileId "#  RF Generator Calibrations"
   puts $fileId "#"
  
   puts $fileId "GENYIGCAL    Temp  $::HW_control::State(if_temp)"
   puts $fileId "GENYIGCAL    Freq  0      Value   $::HW_control::State(yig_gen0)"
   puts $fileId "GENYIGCAL    Freq  2800   Value   $::HW_control::State(yig_gen2800)"
  
   close $fileId
}

proc write_rcv_yig_cals {  } \
{
   variable State
   if ![info exists ::HW_control::State(if_temp)] { return }
   if ![info exists ::HW_control::State(yig_rec0)] { return }
   if ![info exists ::HW_control::State(yig_rec2800)] { return }
   
   set fileId [open $State(rcv_yig_cal) w]
       
   puts $fileId "# Test Set Calibration file [clock format [clock seconds]]"
   puts $fileId " "
   puts $fileId "#"
   puts $fileId "#  Receiver Calibrations"
   puts $fileId "#"
  
   puts $fileId "RXYIGCAL    Temp  $::HW_control::State(if_temp)"
   puts $fileId "RXYIGCAL    Freq  0      Value   $::HW_control::State(yig_rec0)"
   puts $fileId "RXYIGCAL    Freq  2800   Value   $::HW_control::State(yig_rec2800)"
  
   close $fileId
}

proc write_txco_cals {  } \
{
   variable State
   
   set fileId [open $State(tcxo_cal) w]
   
   puts $fileId "# Test Set Calibration file [clock format [clock seconds]]"
   puts $fileId " "
   puts $fileId "#"
   puts $fileId "#  TS TCXO Calibrations"
   puts $fileId "#"
  
   puts $fileId "TCXOCAL    Value  $::HW_control::State(tcxo)"
  
   close $fileId
}

proc Save_yig_cal_on_done { } \
{
   after idle "HW_control::Save_yig_cal_on_done_core"
}
proc Save_yig_cal_on_done_core { } \
{
   variable State
   if {$State(yigres) == 0} \
   {
      after idle HW_control::write_gen_yig_cals
      after idle HW_control::write_rcv_yig_cals
   }
}

proc Save_tcxo_cal_on_change { } \
{
   set cmd "HW_control::write_txco_cals"
   after cancel $cmd
   after 500 $cmd
}
proc Save_rx_yig_cal_on_change {} \
{
   set cmd "HW_control::write_rcv_yig_cals"
   after cancel $cmd
   after 500 $cmd
}

proc Save_gn_yig_cal_on_change {} \
{
   set cmd "HW_control::write_gen_yig_cals"
   after cancel $cmd
   after 500 $cmd
}

proc read_tcxo_cals { } \
{
   variable State
   # READ TCXO
   Debug::Puts HW_control 1 "Reading TXCO cal"
   
   if ![file exists $State(tcxo_cal)] \
   {
      Debug::Puts HW_control 1 "using default"
      set file [open $State(tcxo_cal).default r]
   } \
   else \
   {
      Debug::Puts HW_control 1 "using normal"
      set file [open $State(tcxo_cal) r]
   }
  
   while { [ gets $file line ] >= 0 } \
   {
      Debug::Puts HW_control 1 "Got $line"
      if { [ lindex $line 0 ] == "TCXOCAL" } \
      {
        set State(tcxo) [lindex $line 2]
        Debug::Puts HW_control 1 "set to $State(tcxo)"
        HW_control txco $State(tcxo) -nowait
        break
      }
   }
    Debug::Puts HW_control 1 "done"
  
   close $file
}

proc read_rec_yig_cals { } \
{
   variable State
   # READ YIG cals

   if ![file exists $State(rcv_yig_cal)] \
   {
      set file [open $State(rcv_yig_cal).default r]
   } \
   else \
   {
      set file [open $State(rcv_yig_cal) r]
   }
  
   while { [ gets $file line ] >= 0 } \
   {
      if { [ lindex $line 0 ] == "RXYIGCAL" } \
      {
        if { [ lindex $line 1 ] == "Freq" } \
        {
           set freq [lindex $line 2]
           set value [lindex $line 4]
           
           switch -- $freq \
           {
              0    { set ::HW_control::State(yig_rec0)     $value;Receiver yigcal 0   $value  -nowait}
              2800 { set ::HW_control::State(yig_rec2800)  $value;Receiver yigcal 255 $value  -nowait}
           }
        }
      }
   }
  
   close $file
}


proc read_gen_yig_cals { } \
{
   variable State
   # READ YIG cals
   if ![file exists $State(gen_yig_cal)] \
   {
      set file [open $State(gen_yig_cal).default r]
   } \
   else \
   {
      set file [open $State(gen_yig_cal) r]
   }
   
   while { [ gets $file line ] >= 0 } \
   {
      if { [ lindex $line 0 ] == "GENYIGCAL" } \
      {
        if { [ lindex $line 1 ] == "Freq" } \
        {
           set freq [lindex $line 2]
           set value [lindex $line 4]
           switch -- $freq \
           {
              0    { set ::HW_control::State(yig_gen0)    $value;Generator yigcal 0   $value  -nowait}
              2800 { set ::HW_control::State(yig_gen2800) $value;Generator yigcal 255 $value  -nowait}
           }
        }
      }
   }
  
   close $file
}
   HW_control enable    0x0e
   HW_control backlight 4095
   HW_control ref       INTERNAL

read_gen_yig_cals
read_rec_yig_cals 
read_tcxo_cals

HW_control -onchange yig_res HW_control::Save_yig_cal_on_done
HW_control -onchange tcxo    HW_control::Save_tcxo_cal_on_change
Receiver   -onchange yigcal  HW_control::Save_rx_yig_cal_on_change
Generator  -onchange yigcal  HW_control::Save_gn_yig_cal_on_change
HW_control ref internal
HW_control::StartYigCal
Load::update_hardware_loadbar 33
Load::load_hardware_message "Started Yig Cal"
