#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C) 2003      DESIGN ENGINEERING        #
# --------------------------------------------------------------------------#
# FILE        : keys.tcl                                                    #
# PRODUCT     : FITS                                                        #
# AUTHOR      : David Hagood                                                #
# DESCRIPTION : This file is sourced by TCL when the Keys command is        #
#             : first used.  It loads the C++ interface file, provides a    #
#             : namespace for screen variables, and sets up callbacks to    #
#             : update the variables for the UI.                            #
#                                                                           #
#############################################################################
trace vdelete ::Keys::State ruw "::Keys::Load"

#****************************************************************************
# Handle a load of the C++ interface
#****************************************************************************
if {"[info command Keys]" != "" } { return }

# Call the base class dummy command to force a load
package require Engtclbase
Engtclbase

# Load the object file
::load $::Keys::object

package provide Keys 1.0


namespace eval ::Keys \
{

}

namespace eval KEY_MGT \
{
   set Key_version [Keys version]
   set index [string first "\$Nam" $Key_version]
   if {$index > 0} \
   {
      set front [string range $Key_version 0 [expr $index - 1] ]
      set end   [string range $Key_version [expr $index + 6] end]
      set Key_version $front
      append Key_version $end
   }
   set Key_version [string trimright $Key_version "\$"]

   variable InfoString \
   {
     (using Key Management version - $Key_version)
     <br>
     This option allows the [IFR_options longmodel] to maintain a full keyset, to
     better test encryption and interface with APCO-25 key variable loaders.
     Without this option only a single fixed key is available for encryption.
   }
   set InfoString [subst $InfoString]
}

namespace eval MOTOROLA_ASN \
{
   set Key_version [Keys version]
   set index [string first "\$Nam" $Key_version]
   if {$index > 0} \
   {
      set front [string range $Key_version 0 [expr $index - 1] ]
      set end   [string range $Key_version [expr $index + 6] end]
      set Key_version $front
      append Key_version $end
   }
   set Key_version [string trimright $Key_version "\$"]

   variable InfoString \
   {
     (using Key Management version - $Key_version)
     <br>
     This option allows the use of older Key Fill Devices such as the 
     Motorola KVL-3000 to load encryption keys into the [IFR_options longmodel].
   }
   set InfoString [subst $InfoString]
}
