#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C) 2001      DESIGN ENGINEERING        #
# --------------------------------------------------------------------------#
# FILE        : Pxxv.tcl                                                    #
# PRODUCT     : FITS                                                        #
# AUTHOR      : Bruce Butterfield                                           #
# DESCRIPTION : This file is sourced by TCL when the P25 command is         #
#             : first used.  It loads the C++ interface file, provides a    #
#             : namespace for screen variables, and sets up callbacks to    #
#             : update the variables for the UI.                            #
#                                                                           #
#############################################################################
trace vdelete ::P25ns::State ruw "::P25ns::Load"

#****************************************************************************
# Handle a load of the C++ interface
#****************************************************************************
# Call the base class dummy command to force a load
if {"[info command P25]" != ""} { return }
package require Engtclbase
Engtclbase

# Load the object file
::load $::P25ns::object

package provide P25 1.0

#****************************************************************************
# Add functions to the P25 namespace
#****************************************************************************
#****************************************************************************
# Global variables replaced by this namespace
#****************************************************************************
#
# ::P25ns::DecodeCount                  Replaces        P25DEC_INFO(count)
# ::P25ns::DecodeState(DID)             Replaces        P25DEC_INFO(DID)
# ::P25ns::DecodeState(TGID)            Replaces        P25DEC_INFO(TGID)
# ::P25ns::DecodeState(MFID)            Replaces        P25DEC_INFO(MFID)
# ::P25ns::DecodeState(P)               Replaces        P25DEC_INFO(P)
# ::P25ns::DecodeState(SF)              Replaces        P25DEC_INFO(SF)
# ::P25ns::DecodeState(EMG)             Replaces        P25DEC_INFO(EMG)
# ::P25ns::DecodeState(SID)             Replaces        P25DEC_INFO(SID)
# ::P25ns::DecodeState(LCW_raw)         Replaces        P25DEC_INFO(raw$i)
# ::P25ns::DecodeState(LCO)             Replaces        P25DEC_INFO(LCO)
# ::P25ns::DecodeState(LCO_text)        Replaces        P25DEC_INFO(LCO_text)
# ::P25ns::DecodeState(LCO_short_text)  Replaces        P25DEC_INFO(LCO_short_text)
# ::P25ns::DecodeState(Status)          Replaces        P25DEC_INFO(STATUS)
# ::P25ns::DecodeState(NAC)             Replaces        P25DEC_INFO(NAC)
# ::P25ns::DecodeState(DUID)            Replaces        P25DEC_INFO(DUID)
# ::P25ns::DecodeState(DUID_text)       Replaces        P25DEC_INFO(DUID_text)
# ::P25ns::DecodeState(LSD)             Replaces        P25DEC_INFO(LSD)
# ::P25ns::DecodeState(ALGID)           Replaces        P25DEC_INFO(ALGID)
# ::P25ns::DecodeState(KID)             Replaces        P25DEC_INFO(KID)
# ::P25ns::DecodeState(MI)              Replaces        P25DEC_INFO(MI)
# ::P25ns::DecodeState(Status)          Replaces        P25DEC_INFO(STATUS)
# ::P25ns::DecodeState(StatusText)      Replaces        P25DEC_INFO(STATUS_text)
# ::P25ns::DecodeState(StatusShortText) Replaces        P25DEC_INFO(STATUS_short_text)
# ::P25ns::DecodeState(ALGID_text)      Replaces        P25DEC_INFO(ALGID_text)
# ::P25ns::DecodeState(header)
#
#****************************************************************************
#****************************************************************************
# P25 Array variables
#****************************************************************************
#
# Original P25 variables
#
# DecodeParm / EncodeParm(HDR_NAC)      // Network Access Code (NAC)  12-bits
# DecodeParm / EncodeParm(HDR_MFID)     // Manufacturer's ID           8-bits
# DecodeParm / EncodeParm(HDR_TGID)     // Talk Group ID              16-bits  
#            / EncodeParm(HDR_LSD1)     // Low Speed Data #1 (MSW)    16-bits  
#            / EncodeParm(HDR_LSD2)     // Low Speed Data #2 (LSW)    16-bits  
# DecodeParm(HDR_LSD)                   // Low Speed Data (LSD)       32-bits  
# DecodeParm / EncodeParm(LCO)	        // Link Control Opcode (LCO)   6-bits
# DecodeParm / EncodeParm(HDR_DID)      // Destination Logical Link ID 24-bits 
# DecodeParm / EncodeParm(HDR_SID)      // Source Logical Link ID     24-bits
# DecodeParm / EncodeParm(EMG)	        // Emergency bit               1-bit
# DecodeParm / EncodeParm(SF)	        // Standard Format bit         1-bit
# DecodeParm / EncodeParm(P)	        // Priority bit                1-bit
# DecodeParm / EncodeParm(HDR_KID)      // Key ID                     16-bits  
# DecodeParm / EncodeParm(HDR_SSYM)     // Status Symbol               2-bits  (was STATUS)
# DecodeParm / EncodeParm(HDR_ALGID)    // Algorithm ID 	       8-bits
#
# Additional P25 variables
#
# DecodeParm / EncodeParm(LCO_MFID)     // Manufacturer's ID           8-bits
# DecodeParm / EncodeParm(CHANID)       // Generic Channel identifier  4-bits
# DecodeParm / EncodeParm(CHANID)       // Generic Channel identifier  4-bits
# DecodeParm / EncodeParm(CHANNUM)      // Generic Channel number     12-bits  
# DecodeParm / EncodeParm(CHANID_A)     // Channel A identifier        4-bits
# DecodeParm / EncodeParm(CHANNUM_A)    // Channel A number	      12-bits
# DecodeParm / EncodeParm(CHANID_B)     // Channel B identifier        4-bits
# DecodeParm / EncodeParm(CHANNUM_B)    // Channel B number	      12-bits
# DecodeParm / EncodeParm(CHANID_R)     // Channel Recieve id          4-bits
# DecodeParm / EncodeParm(CHANNUM_R)    // Channel Recieve number     12-bits
# DecodeParm / EncodeParm(CHANID_T)     // Channel Transmit id         4-bits
# DecodeParm / EncodeParm(CHANNUM_T)    // Channel Transmit number    12-bits
# DecodeParm / EncodeParm(GRPADDR)      // Generic Group Address      16-bits 
# DecodeParm / EncodeParm(GRPADDR_A)    // Group Address - A          16-bits
# DecodeParm / EncodeParm(GRPADDR_B)    // Group Address - B          16-bits
# DecodeParm / EncodeParm(TARGETADDR)   // Target Address             24-bits 
# DecodeParm / EncodeParm(SOURCEADDR)   // Source Address	      24-bits
# DecodeParm / EncodeParm(CTIMER)       // Call Timer	  	      16-bits
# DecodeParm / EncodeParm(DIG0_4)       // Digits 0-4	        Range = 0 - 99999
# DecodeParm / EncodeParm(DIG5_9,       // Digits 5-9	        Range = 0 - 99999
# DecodeParm / EncodeParm(NETID)        // Network Identification     20-bits
# DecodeParm / EncodeParm(SYSID)        // System Identification      12-bits
# DecodeParm / EncodeParm(SRCID)        // Source Identification      24-bits
# DecodeParm / EncodeParm(PRIREQ)       // Requested Priority	       4-bits (3-bits ? )
# DecodeParm / EncodeParm(SSAVAIL)      // Available System Services  24-bits
# DecodeParm / EncodeParm(SSSUPPORT)    // Supported System Services  24-bits
# DecodeParm / EncodeParm(RFSUBID)      // RF Sub-System Id	       8-bits
# DecodeParm / EncodeParm(SITEID)       // Site Id		       8-bits
# DecodeParm / EncodeParm(SERVCLASS)    // Service Class	       8-bits
# DecodeParm / EncodeParm(SERVCLASS_A)  // Service Class - A	       8-bits
# DecodeParm / EncodeParm(SERVCLASS_B)  // Service Class - B	       8-bits
# DecodeParm / EncodeParm(LOCREGAREA)   // Location Registration Area  8-bits
# DecodeParm / EncodeParm(STSUPDATE)    // Status		      16-bits
# DecodeParm / EncodeParm(MSG)          // Data message	 	      16-bits
# DecodeParm / EncodeParm(EXTFUNCT_A)   // Ext. Function Octet 0-2    24-bits
# DecodeParm / EncodeParm(EXTFUNCT_B)   // Ext. Function Octet 3-4    16-bits
# DecodeParm / EncodeParm(BW)           // Bandwidth  		       9-bits
# DecodeParm / EncodeParm(XMITOFF)      // Transmit Offset  	       9-bits
# DecodeParm / EncodeParm(CHANSPACE)    // Channel Spacing  	      10-bits
# DecodeParm / EncodeParm(BASEFREQ)     // Base Frequency   	      32-bits
# DecodeParm / EncodeParm(ALGID)        // Algorithm ID 	       8-bits
# DecodeParm / EncodeParm(SERVICEOP)    // Service Options	       8-bits
# DecodeParm / EncodeParm(RESERVE_4)    // Reserved		       4-bits
# DecodeParm / EncodeParm(RESERVE_8)    // Reserved		       8-bits
# DecodeParm / EncodeParm(RESERVE_16)   // Reserved		      16-bits
# DecodeParm / EncodeParm(RESERVE_24)   // Reserved		      24-bits
# DecodeParm / EncodeParm(DIGIT1_3)
# DecodeParm / EncodeParm(DIGIT4_6)
# DecodeParm / EncodeParm(DIGIT7_10)
#
# P25 Trunking specific variables
#
# DecodeParm / EncodeParm(WACNID)       // Wide Area Communication Net 20-bits
# DecodeParm / EncodeParm(WUID)         // Working Unit ID             24-bits
# DecodeParm / EncodeParm(WGID)         // Working Group ID            16-bits
# DecodeParm / EncodeParm(CHANID_C)     // Control Channel ID          4-bits.
# DecodeParm / EncodeParm(CHANNUM_C)    // Control Channel number      12-bits
# DecodeParm / EncodeParm(CHANID_V)     // Voice Channel ID            4-bits.
# DecodeParm / EncodeParm(CHANNUM_V)    // Voice Channel number        12-bits
# DecodeParm / EncodeParm(ANNGRPADR)    // Announcement Group Address  16-bits
# DecodeParm / EncodeParm(T_PROT)       // Protected Trunking Flag     1-bit
# DecodeParm / EncodeParm(T_ANET)       // Active Network Flag         1-bit
# DecodeParm / EncodeParm(T_LG)         // Local/Global Flag           1-bit
# DecodeParm / EncodeParm(T_GAV)        // Group Affiliation Value     2-bits
# DecodeParm / EncodeParm(T_RV)         // Registration Value          2-bits
# DecodeParm(UID)                       // Unit ID                     24-bits
# DecodeParm(GID)                       // Group ID                    16-bits

#****************************************************************************
namespace eval ::P25ns {
  variable SaveSortOrder FIRST
  variable EncodeParm
  variable DecodeParm

  array set DUID_strings { 0   "Header Data"    \
                           3   "Term w/o LC"    \
                           5   "LDU 1"          \
                           7   "Trunk Sig Data" \
                           10  "LDU 2"          \
                           12  "Packet Data"    \
                           15  "Term w/ LC" }

  array set LCO_strings { 00  "Group Voice Channel User"               \
                           0  "Group Voice Channel User"               \
                          02  "Group Voice Channel Update"             \
                           2  "Group Voice Channel Update"             \
                          03  "Unit to Unit Voice Channel User"        \
                           3  "Unit to Unit Voice Channel User"        \
                          04  "Group Voice Channel Update - Explicit"  \
                           4  "Group Voice Channel Update - Explicit"  \
                          05  "Unit to Unit Answer Request"            \
                           5  "Unit to Unit Answer Request"            \
                          06  "Telephone Interconnect Voice Ch User"   \
                           6  "Telephone Interconnect Voice Ch User"   \
                          07  "Telephone Interconnect Answer Req"      \
                           7  "Telephone Interconnect Answer Req"      \
                          0F  "Call Termination/Cancellation"          \
                           F  "Call Termination/Cancellation"          \
                          10  "Group Affiliation Query"                \
                          11  "Unit Registration Command"              \
                          12  "Unit Authenticaion Command"             \
                          13  "Status Query"                           \
                          14  "Status Update"                          \
                          15  "Message Update"                         \
                          16  "Call Alert"                             \
                          17  "Extended Function Command"              \
                          18  "Channel Identifier Update"              \
                          20  "System Service Broadcast"               \
                          21  "Secondary Control Channel Broadcast"    \
                          22  "Adjacent Site Status Broadcast"         \
                          23  "RFSS Status Broadcast"                  \
                          24  "Network Status Broadcast"               \
                          25  "Protection Parameter Broadcast" }

  array set LCO_short_strings { 00  "Grp Voice Chan User"       \
                                 0  "Grp Voice Chan User"       \
                                02  "Grp Voice Chan Update"     \
                                 2  "Grp Voice Chan Update"     \
                                03  "Unit/Unit Voice Chan User" \
                                 3  "Unit/Unit Voice Chan User" \
                                04  "Grp Voice Chan - Explicit" \
                                 4  "Grp Voice Chan - Explicit" \
                                05  "Unit/Unit Answer Request"  \
                                 5  "Unit/Unit Answer Request"  \
                                06  "Tel Voice Chan User"       \
                                 6  "Tel Voice Chan User"       \
                                07  "Tel Answer Request"        \
                                 7  "Tel Answer Request"        \
                                0F  "Call Term/Cancel"          \
                                 F  "Call Term/Cancel"          \
                                10  "Group Affiliation Query"   \
                                11  "Unit Reg Command"          \
                                12  "Unit Auth Command"         \
                                13  "Status Query"              \
                                14  "Status Update"             \
                                15  "Message Update"            \
                                16  "Call Alert"                \
                                17  "Extended Func Command"     \
                                18  "Chan Identifier Update"    \
                                20  "System Service Bcast"      \
                                21  "Secondary Ctrl Chan Bcast" \
                                22  "Adjacent Site Stat Bcast"  \
                                23  "RFSS Stat Bcast"           \
                                24  "Network Status Bcast"      \
                                25  "Prot Parameter Bcast" }

  array set status_str { 0  "Unknown, for talk-around" \
                         1  "Inbound Channel Busy"     \
                         2  "Unknwn,Inbound&Outbound"  \
                         3  "Inbound Channel Idle" }

  array set status_short_str { 0  "Unknown,TalkAround" \
                               1  "Inbound Chan Busy"  \
                               2  "Unknwn,in&outbound" \
                               3  "Inbound Chan Idle"  }

  array set algid_strings { 80  "No Encryption"     \
                            81  "DES Encryption" }

  array set algid_short_str { 00  "CLEAR"  \
                              80  "CLEAR"  \
                              81  "DES"    \
                              84  "AES" }

  array set trunk_mode_type { 0  "800"      \
                              1  "UHF/VHF"  \
                              2  "700"      \
                              "800"      0  \
                              "UHF/VHF"  1  \
                              "700"      2 }

#****************************************************************************
# Name:        set_p25_callbacks
# Parameters:  none
# Description: Sets up the callbacks to update the P25 data
#****************************************************************************
 proc set_p25_callbacks { } {

   if { $::P25ns::callbackStatus != 0 } {
       return
   }


   P25 decode -onchange samplerate    { set ::P25ns::State(d_sampleRate) %V; }
   P25 decode -onchange enable        { set ::P25ns::State(d_enable) %V; }
   P25 decode -onchange run           { set ::P25ns::State(d_run) %V; }
   P25 decode -onchange encryptstate  { set ::P25ns::State(d_encryptState) %V; }

#   P25 decode -onchange header { ::P25ns::updateHeader %V; }
#   P25 decode -onchange link   { ::P25ns::updateLCW %V; }
#   P25 decode -onchange nid    { ::P25ns::updateNID %V; }
#   P25 decode -onchange lsd    { ::P25ns::updateLSD %V; }

   P25 encode -onchange encrypt { ::P25ns::updateEncodeEncrypt %V; }
   P25 encode -onchange parms_a { ::P25ns::updateEncodeParms BlockA %V; }
   P25 encode -onchange parms_b { ::P25ns::updateEncodeParms BlockB %V; }
   P25 encode -onchange parms_c { ::P25ns::updateEncodeParms BlockC %V; }
   P25 encode -onchange parms_d { ::P25ns::updateEncodeParms BlockD %V; }
   P25 encode -onchange parms_e { ::P25ns::updateEncodeParms BlockE %V; }
   P25 encode -onchange parms_f { ::P25ns::updateEncodeParms BlockF %V; }
   P25 encode -onchange parms_g { ::P25ns::updateEncodeParms BlockG %V; }
   P25 encode -onchange parms_h { ::P25ns::updateEncodeParms BlockH %V; }
   P25 encode -onchange parms_i { ::P25ns::updateEncodeParms BlockI %V; }
   P25 encode -onchange trunksim { ::P25ns::setTrunkSimState %V; }
   P25 encode -onchange trunkpttenable { ::P25ns::setTrunkPttCtrl %V; }
   P25 encode -onchange trunkctrlfreq  { ::P25ns::updateTrunkFreqs CTRL %V; }
   P25 encode -onchange trunkvoicefreq { ::P25ns::updateTrunkFreqs VOICE %V; }
   P25 encode -onchange trunkchantrfreq { ::P25ns::updateTrunkFreqs ADJCHANTR %V; }
   P25 encode -onchange trunksccb1freq { ::P25ns::updateTrunkFreqs SCCB1 %V; }
   P25 encode -onchange trunksccb2freq { ::P25ns::updateTrunkFreqs SCCB2 %V; }
   P25 encode -onchange ValidAlgids { ::P25ns::updateAlgidParms %V; }
   P25 encode -onchange chanaddr { ::P25ns::updateChannelAddr %V; }
   P25 encode -onchange TrunkModeIndex { ::P25ns::updateTrunkModeIndex %V; }
   P25 encode -onchange digits { set ::P25ns::EncodeParm(DIG0_9) [lindex %V 0]; }
   P25 encode -onchange autowrapdata { ::P25ns::updateAutocopy %V }
   P25 encode -onchange AdjStatusEnable { set ::P25ns::State(AdjStatusEnable) %V; }
   P25 encode -onchange obtctrl { ::P25ns::updateOBT_Ctrl %V; }
   P25 encode -onchange explicitmessages { ::P25ns::updateMsgFormatType %V; }

   P25 decode -onchange parms_a { ::P25ns::updateDecodeParms BlockA %V}
   P25 decode -onchange parms_b { ::P25ns::updateDecodeParms BlockB %V}
   P25 decode -onchange parms_c { ::P25ns::updateDecodeParms BlockC %V}
   P25 decode -onchange parms_d { ::P25ns::updateDecodeParms BlockD %V}
   P25 decode -onchange parms_e { ::P25ns::updateDecodeParms BlockE %V}
   P25 decode -onchange parms_f { ::P25ns::updateDecodeParms BlockF %V}
   P25 decode -onchange parms_g { ::P25ns::updateDecodeParms BlockG %V}
   P25 decode -onchange parms_h { ::P25ns::updateDecodeParms BlockH %V}
   P25 decode -onchange dtmf    { set ::P25ns::DecodeParm(DTMF) %V}

   P25 decode -onchange trunkpttrcv { ::P25ns::setTrunkRcvPtt %V}
   P25 decode -onchange trunkregrcv { ::P25ns::setTrunkRcvReg; }
   P25 decode -onchange trunkaffrcv { ::P25ns::setTrunkRcvAff; }
   P25 decode -onchange trunksigrcv { ::P25ns::setTrunkRcvSignal %V; }

   trace variable ::P25ns::EncodeParm(LCO) w  "::P25ns::updateEncodeLcoStrings"

   set ::P25ns::callbackStatus 1
 }

#****************************************************************************
# Name:        updateDecodeParms
# Parameters:  type	   Type of data message {BlockA | BlockB | BlockC | BlockD}
#              parmList	   Data from the engine
# Description: Records the settings of the P25 transmit variables
#****************************************************************************
 proc updateDecodeParms { type parmList } {
    variable DecodeParm
    variable algid_strings
    variable DUID_strings
    variable status_str
    variable status_short_str
    variable LCO_strings
    variable LCO_short_strings
    variable LCO_delay
    variable LCO_delayTime
    variable LCO_ignore

    if {$type == "BlockA"} {
        set DecodeParm(LCO_CNT) [lindex $parmList 0]
        set Lco [lindex $parmList 1]
        set Lco [expr $Lco & 0x3f]

        # This logic bypasses the normal variable update for a short period of
        # time so that a single LCO update remains visible on the P25 Uplink
        # tile for a few seconds before being overwritten by the continuous 
        # background LCOs  (0, 3, 6)
        #
        # THIS IS ONLY VALID FOR THE MASTER INTERPRETER. THE SLAVE SHOULD NOT
        # BE DELAYED.
        if {![interp issafe]} \
        {
           if { ($Lco == 0) || ($Lco == 3) || ($Lco == 6) } \
          {
              if { $LCO_delay == 1 } \
              {
                 set LCO_ignore 1
                 return
              }
           } \
           else \
           {
              set LCO_ignore 0
              if {$LCO_delayTime <= 0} \
              {
                 set LCO_delay 0
              } \
              else \
              {
                 set LCO_delay 1
                 after $LCO_delayTime {set ::P25ns::LCO_delay 0; set ::P25ns::LCO_ignore 0; }
              }
           }
        }
        set DecodeParm(OCTET_0) [format "%02X" [lindex $parmList 1] ]
        set DecodeParm(OCTET_1) [format "%02X" [lindex $parmList 2] ]
        set DecodeParm(OCTET_2) [format "%02X" [lindex $parmList 3] ]
        set DecodeParm(OCTET_3) [format "%02X" [lindex $parmList 4] ]
        set DecodeParm(OCTET_4) [format "%02X" [lindex $parmList 5] ]
        set DecodeParm(OCTET_5) [format "%02X" [lindex $parmList 6] ]
        set DecodeParm(OCTET_6) [format "%02X" [lindex $parmList 7] ]
        set DecodeParm(OCTET_7) [format "%02X" [lindex $parmList 8] ]
        set DecodeParm(OCTET_8) [format "%02X" [lindex $parmList 9] ]
#        puts "LCO=$Lco, Count=$DecodeParm(LCO_CNT)  ---  $DecodeParm(OCTET_0) $DecodeParm(OCTET_1) $DecodeParm(OCTET_2) \
#$DecodeParm(OCTET_3) $DecodeParm(OCTET_4) $DecodeParm(OCTET_5) $DecodeParm(OCTET_6) \
#$DecodeParm(OCTET_7) $DecodeParm(OCTET_8)"  
    }

    if {$LCO_ignore == 1} {return};

    if {$type == "BlockB"} {
        set DecodeParm(HDR_CNT) [lindex $parmList 0]]
        set MI1 [format "%06X"  [lindex $parmList 1]]
        set MI2 [format "%06X"  [lindex $parmList 2]]
        set MI3 [format "%06X"  [lindex $parmList 3]]
        set DecodeParm(HDR_MI)    [concat $MI1$MI2$MI3]
        set DecodeParm(HDR_MFID_dec)  [lindex $parmList 4]
        set DecodeParm(HDR_MFID)  [format "%02X" [lindex $parmList 4]]
        set algid                 [lindex $parmList 5]
        set DecodeParm(HDR_ALGID) [format "%02X" $algid]
        set DecodeParm(HDR_KID)   [format "%04X" [lindex $parmList 6]]
        set DecodeParm(HDR_TGID)  [format "%04X" [lindex $parmList 7]]
        set DecodeParm(HDR_DID)   [format "%06X" [lindex $parmList 8]]
        set DecodeParm(HDR_SID)   [format "%06X" [lindex $parmList 9]]

        if [info exists algid_strings($algid)] {
          set DecodeParm(ALGID_text) $algid_strings($algid)
        } else {
          set DecodeParm(ALGID_text) " "
        }

        set MI1 [format "%06X"   [lindex $parmList 10]]
        set MI2 [format "%06X"   [lindex $parmList 11]]
        set MI3 [format "%06X"   [lindex $parmList 12]]
        set DecodeParm(ES_MI)    [concat $MI1$MI2$MI3]
        set algid                [lindex $parmList 13]
        set DecodeParm(ES_ALGID) [format "%02X" $algid]
        set DecodeParm(ES_KID)   [format "%04X" [lindex $parmList 14]]

        if [info exists algid_strings($algid)] {
          set DecodeParm(ES_ALGID_text) $algid_strings($algid)
        } else {
          set DecodeParm(ES_ALGID_text) " "
        }

#        puts "HDR_CNT=$DecodeParm(HDR_CNT), $DecodeParm(HDR_MI) $DecodeParm(HDR_MFID) \
#$DecodeParm(HDR_ALGID) $DecodeParm(HDR_KID) $DecodeParm(HDR_TGID) $DecodeParm(HDR_DID) \
#$DecodeParm(HDR_SID)"
    }
    if {$type == "BlockC"} {
        set DecodeParm(NID_CNT)  [lindex $parmList 0]
        set DecodeParm(HDR_NAC)  [format "%03X" [lindex $parmList 1]]
        set DecodeParm(HDR_NAC_dec)  [format "%d" [lindex $parmList 1]]
        set duid                 [lindex $parmList 2]
        set DecodeParm(HDR_DUID) [format "%01X" $duid]
        set DecodeParm(LSD_CNT)  [lindex $parmList 3]
        set DecodeParm(HDR_LSD)  [format "%08X" [lindex $parmList 4]]
        set statusSymbol         [lindex $parmList 5] 
        set DecodeParm(HDR_SSYM) [format "%01X" $statusSymbol]

        # For known DUID values, identify the data
        if [info exists DUID_strings($duid)] {
           set DecodeParm(DUID_text) $DUID_strings($duid)
        } else {
           set DecodeParm(DUID_text) " "
        }

        # Set Status text and short text
        if [info exists status_str($statusSymbol)] {
           set DecodeParm(StatusText) $status_str($statusSymbol)
        } else {
           set DecodeParm(StatusText) " "
        }

        if [info exists status_short_str($statusSymbol)] {
           set DecodeParm(StatusShortText) $status_short_str($statusSymbol)
        } else {
           set DecodeParm(StatusShortText) " "
        }

#        puts "NID_CNT=$DecodeParm(NID_CNT), $DecodeParm(NAC) $DecodeParm(DUID),  \
#LSD_CNT=$DecodeParm(LSD_CNT) $DecodeParm(LSD) $DecodeParm(SSYM)"
    }
    if {$type == "BlockD"} {
        set DecodeParm(P)          [format "%01X" [lindex $parmList 0]]
        set DecodeParm(SF)         [format "%01X" [lindex $parmList 1]]
        set lco                    [lindex $parmList 2]
        set DecodeParm(LCO)        [format "%02X" $lco]
        set DecodeParm(MFID)       [format "%02X" [lindex $parmList 3]]
        set DecodeParm(MFID_dec)   [lindex $parmList 3]
        set DecodeParm(SERVICEOP_dec)  [lindex $parmList 4]
        set DecodeParm(SERVICEOP)  [format "%02X" [lindex $parmList 4]]
        set DecodeParm(EMG)        [format "%01X" [lindex $parmList 5]]
        set DecodeParm(RESERVE_8)  [format "%02X" [lindex $parmList 6]]
        set DecodeParm(GRPADDR)    [format "%04X" [lindex $parmList 7]]
        set DecodeParm(GRPADDR_dec)  [lindex $parmList 7]
        set DecodeParm(TARGETADDR) [format "%06X" [lindex $parmList 8]]
        set DecodeParm(TARGETADDR_dec) [lindex $parmList 8]
        set DecodeParm(SOURCEADDR) [format "%06X" [lindex $parmList 9]]
        set DecodeParm(SOURCEADDR_dec) [lindex $parmList 9]

        # Set up LCO string text
        if [info exists LCO_strings($DecodeParm(LCO))] {
           set DecodeParm(LCO_text) $LCO_strings($DecodeParm(LCO))
        } else {
           set DecodeParm(LCO_text) "<reserved>"
        }
   
        if [info exists LCO_short_strings($DecodeParm(LCO))] {
           set DecodeParm(LCO_short_text) $LCO_short_strings($DecodeParm(LCO))
        } else {
           set DecodeParm(LCO_short_text) "<reserved>"
        }

#        puts "BlockD, $DecodeParm(P) $DecodeParm(SF) $DecodeParm(LCO) \
#$DecodeParm(MFID) $DecodeParm(SERVICEOP) $DecodeParm(EMG) $DecodeParm(RESERVE_8) \
#$DecodeParm(GRPADDR) $DecodeParm(TARGETADDR) $DecodeParm(SOURCEADDR)"  
    }
    if {$type == "BlockE"} {
        set DecodeParm(CHANID)          [format "%01X" [lindex $parmList 0]]
        set DecodeParm(CHANID_dec)      [lindex $parmList 0]
        set DecodeParm(CHANNUM)         [format "%03X" [lindex $parmList 1]]
        set DecodeParm(CHANNUM_dec)     [lindex $parmList 1]
        set DecodeParm(CHANID_A)        [format "%01X" [lindex $parmList 2]]
        set DecodeParm(CHANNUM_A)       [format "%03X" [lindex $parmList 3]]
        set DecodeParm(CHANID_B)        [format "%01X" [lindex $parmList 4]]
        set DecodeParm(CHANNUM_B)       [format "%03X" [lindex $parmList 5]]
        set DecodeParm(CHANID_R)        [format "%01X" [lindex $parmList 6]]
        set DecodeParm(CHANNUM_R)       [format "%03X" [lindex $parmList 7]]
        set DecodeParm(CHANID_T)        [format "%01X" [lindex $parmList 8]]
        set DecodeParm(CHANNUM_T)       [format "%03X" [lindex $parmList 9]]
        set DecodeParm(GRPADDR_A)       [format "%04X" [lindex $parmList 10]]
        set DecodeParm(GRPADDR_B)       [format "%04X" [lindex $parmList 11]]

#        puts "BlockE, $DecodeParm(CHANID) $DecodeParm(CHANNUM) $DecodeParm(CHANID_A)\
#$DecodeParm(CHANNUM_A) $DecodeParm(CHANID_B) $DecodeParm(CHANNUM_B) $DecodeParm(CHANID_R)\
#$DecodeParm(CHANNUM_R) $DecodeParm(CHANID_T) $DecodeParm(CHANNUM_T)\
#$DecodeParm(GRPADDR_A) $DecodeParm(GRPADDR_B)"  
    }
    if {$type == "BlockF"} {
        set DecodeParm(CTIMER)          [format "%04X" [lindex $parmList 0]]
        set DecodeParm(DIG0_4)          [format "%05X" [lindex $parmList 1]]
        set DecodeParm(DIG5_9)          [format "%05X" [lindex $parmList 2]]
        set DecodeParm(NETID)           [format "%05X" [lindex $parmList 3]]
        set DecodeParm(SYSID)           [format "%03X" [lindex $parmList 4]]
        set DecodeParm(SYSID_dec)       [lindex $parmList 4]
        set DecodeParm(SRCID)           [format "%06X" [lindex $parmList 5]]
        set DecodeParm(SRCID_dec)       [lindex $parmList 5]
        set DecodeParm(PRIREQ)          [format "%01X" [lindex $parmList 6]]
        set DecodeParm(SSAVAIL)         [format "%06X" [lindex $parmList 7]]
        set DecodeParm(SSAVAIL_dec)     [lindex $parmList 7]
        set DecodeParm(SSSUPPORT)       [format "%06X" [lindex $parmList 8]]
        set DecodeParm(SSSUPPORT_dec)   [lindex $parmList 8]
        set DecodeParm(RFSUBID)         [format "%02X" [lindex $parmList 9]]
        set DecodeParm(RFSUBID_dec)     [lindex $parmList 9]
        set DecodeParm(SITEID)          [format "%02X" [lindex $parmList 10]]
        set DecodeParm(SITEID_dec)      [lindex $parmList 10]
        set DecodeParm(SERVCLASS)       [format "%02X" [lindex $parmList 11]]
        set DecodeParm(SERVCLASS_dec)   [lindex $parmList 11]
        set DecodeParm(SERVCLASS_A)     [format "%02X" [lindex $parmList 12]]
        set DecodeParm(SERVCLASS_B)     [format "%02X" [lindex $parmList 13]]

#        puts "BlockF, $DecodeParm(CTIMER) $DecodeParm(DIG0_4) $DecodeParm(DIG5_9)\
#$DecodeParm(NETID) $DecodeParm(SYSID) $DecodeParm(SRCID) $DecodeParm(PRIREQ)\
#$DecodeParm(SSAVAIL) $DecodeParm(SSSUPPORT) $DecodeParm(RFSUBID) $DecodeParm(SITEID)\
#$DecodeParm(SERVCLASS) $DecodeParm(SERVCLASS_A) $DecodeParm(SERVCLASS_B)"  
    }
    if {$type == "BlockG"} {
        set DecodeParm(LOCREGAREA)   [format "%02X" [lindex $parmList 0]]
        set DecodeParm(LOCREGAREA_dec)   [lindex $parmList 0]
        set DecodeParm(STSUPDATE)    [format "%04X" [lindex $parmList 1]]
        set DecodeParm(MSG)          [format "%04X" [lindex $parmList 2]]
        set DecodeParm(EXTFUNCT_A)   [format "%04X" [lindex $parmList 3]]
        set DecodeParm(EXTFUNCT_B)   [format "%06X" [lindex $parmList 4]]
        set DecodeParm(BW)           [format "%03X" [lindex $parmList 5]]
        set DecodeParm(XMITOFF)      [format "%03X" [lindex $parmList 6]]
        set DecodeParm(CHANSPACE)    [format "%03X" [lindex $parmList 7]]
        set DecodeParm(BASEFREQ)     [format "%08X" [lindex $parmList 8]]
        set DecodeParm(ALGID)        [format "%02X" [lindex $parmList 9]]
        set DecodeParm(KID)          [format "%04X" [lindex $parmList 10]]
        set DecodeParm(RESERVE_4)    [format "%01X" [lindex $parmList 11]]
        set DecodeParm(RESERVE_16)   [format "%04X" [lindex $parmList 12]]
        set DecodeParm(RESERVE_24)   [format "%06X" [lindex $parmList 13]]

#        puts "BlockG, $DecodeParm(LOCREGAREA) $DecodeParm(STSUPDATE) $DecodeParm(MSG)\
#$DecodeParm(EXTFUNCT_A) $DecodeParm(EXTFUNCT_B) $DecodeParm(BW) $DecodeParm(XMITOFF)\
#$DecodeParm(CHANSPACE) $DecodeParm(BASEFREQ) $DecodeParm(ALGID) $DecodeParm(KID)\
#$DecodeParm(RESERVE_4) $DecodeParm(RESERVE_16) $DecodeParm(RESERVE_24)"  
    }
    if {$type == "BlockH"} {
        set DecodeParm(OPCODE)       [format "%02X" [lindex $parmList 0]]
        set DecodeParm(OPCODE_dec)   [lindex $parmList 0]
        set DecodeParm(T_PROT)       [lindex $parmList 1]
        set DecodeParm(T_ANET)       [lindex $parmList 2]
        set DecodeParm(UID)          [format "%06X" [lindex $parmList 3]]
        set DecodeParm(UID_dec)      [lindex $parmList 3]
        set DecodeParm(GID)          [format "%04X" [lindex $parmList 4]]
        set DecodeParm(GID_dec)      [lindex $parmList 4]
        set DecodeParm(WACNID)       [format "%05X" [lindex $parmList 5]]
        set DecodeParm(WACNID_dec)   [lindex $parmList 5]
        set DecodeParm(ANNGRPADR)    [format "%04X" [lindex $parmList 6]]
        set DecodeParm(ANNGRPADR_dec)  [lindex $parmList 6]
        set DecodeParm(ANSRES)       [format "%02X" [lindex $parmList 7]]
        set DecodeParm(PSTNADDR)     [format "%02X" [lindex $parmList 8]]
        set DecodeParm(SERVTYPE)     [format "%02X" [lindex $parmList 9]]
        set DecodeParm(REASONCODE)   [format "%02X" [lindex $parmList 10]]
        set DecodeParm(IDFLAGS)      [format "%01X" [lindex $parmList 11]]
        set DecodeParm(IDUPDATE)     [format "%01X" [lindex $parmList 12]]
        set DecodeParm(TXMULT)       [lindex $parmList 13]
        set DecodeParm(ADD_24)       [format "%06X" [lindex $parmList 14]]
        set DecodeParm(RES_32)       [format "%08X" [lindex $parmList 15]]

#        puts "BlockH, $DecodeParm(OPCODE) $DecodeParm(T_PROT) $DecodeParm(T_ANET)\
#$DecodeParm(UID) $DecodeParm(GID) $DecodeParm(WACNID) $DecodeParm(ANNGRPADR)"
    }
 }
#****************************************************************************
# Name:        updateEncodeParms
# Parameters:  type	   Type of data message {BlockA | BlockB | BlockC | BlockD}
#              parmList	   Data from the engine
# Description: Records the settings of the P25 transmit variables
#****************************************************************************
 proc updateEncodeParms { type  parmList } \
 {
    variable EncodeParm
    variable algid_short_str
    variable status_str
    variable status_short_str
    variable TrunkChanArray
    variable TrunkModeCurrent
    variable State

    if {$type == "BlockA"} \
    {
        set EncodeParm(HDR_NAC_dec)   [format "%d" [lindex $parmList 0] ]
        set EncodeParm(HDR_NAC)   [format "%03X" [lindex $parmList 0] ]
        set EncodeParm(HDR_MFID_dec)  [format "%d" [lindex $parmList 1] ]
        set EncodeParm(HDR_MFID)  [format "%02X" [lindex $parmList 1] ]
        set EncodeParm(HDR_TGID)  [format "%04X" [lindex $parmList 2] ]
        set EncodeParm(HDR_LSD1)  [format "%04X" [lindex $parmList 3] ]
        set EncodeParm(HDR_LSD2)  [format "%04X" [lindex $parmList 4] ]

#####################################################################################
# The following code was intentionally disabled. This message block is a direct 
# match to an encode parameter structure in the engine, and it represents an early
# structure of data sent to the DSP. The ACTUAL variables were moved to the next
# block because of Header vs. LCO data update issues, but the array structure was
# left for DSP compatibility. Eventually the DSP structure may eliminate unused
# variables, allowing this code block to be permanently deleted.
#        set EncodeParm(LCO)       [format "%02X" [lindex $parmList 5] ]
#        set EncodeParm(HDR_DID)   [format "%06X" [lindex $parmList 6] ]
#        set EncodeParm(HDR_SID)   [format "%06X" [lindex $parmList 7] ]
#        set EncodeParm(EMG)       [format "%01X" [lindex $parmList 8] ]
#        set EncodeParm(SF)        [format "%01X" [lindex $parmList 9] ]
#        set EncodeParm(P)         [format "%01X" [lindex $parmList 10] ]
#####################################################################################
        set EncodeParm(HDR_KID)   [format "%04X" [lindex $parmList 11] ]

        set EncodeParm(HDR_SSYM)  [format "%01X" [lindex $parmList 12] ]
        
          if [info exists status_str($EncodeParm(HDR_SSYM))] \
          {
            set EncodeParm(StatusText)  $status_str($EncodeParm(HDR_SSYM))
          } \
          else \
          {
            set EncodeParm(StatusText)  ""
          }

          if [info exists status_short_str($EncodeParm(HDR_SSYM))] \
          {
            set EncodeParm(StatusShortText) $status_short_str($EncodeParm(HDR_SSYM))
          } \
          else \
          {
            set EncodeParm(StatusShortText) ""
          }

        set EncodeParm(HDR_ALGID) [format "%01X" [lindex $parmList 13]]
        
          if [info exists algid_short_str($EncodeParm(HDR_ALGID))] \
          {
            set EncodeParm(HDR_ALGID_txt) $algid_short_str($EncodeParm(HDR_ALGID))
          } \
          else \
          {
            set EncodeParm(HDR_ALGID_txt) "Unknown"
          }

#        puts "Encode BlockA, $EncodeParm(HDR_NAC) $EncodeParm(HDR_MFID) $EncodeParm(HDR_TGID)\
#$EncodeParm(HDR_LSD1) $EncodeParm(HDR_LSD2) $EncodeParm(LCO) $EncodeParm(HDR_DID)\
#$EncodeParm(HDR_SID) $EncodeParm(EMG) $EncodeParm(SF) $EncodeParm(P)\
#$EncodeParm(HDR_KID) $EncodeParm(HDR_SSYM) $EncodeParm(HDR_ALGID)"  

    } elseif {$type == "BlockB"} {
        set EncodeParm(LCO_MFID)   [format "%02X" [lindex $parmList 0] ]

        set EncodeParm(LCO)       [format "%02X" [lindex $parmList 1] ]
        set EncodeParm(HDR_DID)   [format "%06X" [lindex $parmList 2] ]
        set EncodeParm(HDR_SID)   [format "%06X" [lindex $parmList 3] ]
        set EncodeParm(EMG)       [format "%01X" [lindex $parmList 4] ]
        set EncodeParm(SF)        [format "%01X" [lindex $parmList 5] ]
        set EncodeParm(P)         [format "%01X" [lindex $parmList 6] ]

        set EncodeParm(CHANID)     [format "%01X" [lindex $parmList 7] ]
        set EncodeParm(CHANNUM)    [format "%03X" [lindex $parmList 8] ]
        set EncodeParm(CHANID_A)   [format "%01X" [lindex $parmList 9] ]
        set EncodeParm(CHANNUM_A)  [format "%03X" [lindex $parmList 10] ]
        set EncodeParm(CHANID_B)   [format "%01X" [lindex $parmList 11] ]
        set EncodeParm(CHANNUM_B)  [format "%03X" [lindex $parmList 12] ]
        set EncodeParm(CHANID_R_dec)   [lindex $parmList 13]
        set EncodeParm(CHANID_R)   [format "%01X" [lindex $parmList 13] ]
        set EncodeParm(CHANNUM_R_dec)  [lindex $parmList 14]
        set EncodeParm(CHANNUM_R)  [format "%03X" [lindex $parmList 14] ]
        set EncodeParm(CHANID_T_dec)   [lindex $parmList 15]
        set EncodeParm(CHANID_T)   [format "%01X" [lindex $parmList 15] ]
        set EncodeParm(CHANNUM_T_dec)  [lindex $parmList 16]
        set EncodeParm(CHANNUM_T)  [format "%03X" [lindex $parmList 16] ]
        set EncodeParm(GRPADDR)    [format "%04X" [lindex $parmList 17] ]
        set EncodeParm(GRPADDR_A)  [format "%04X" [lindex $parmList 18] ]
        set EncodeParm(GRPADDR_B)  [format "%04X" [lindex $parmList 19] ]
        set EncodeParm(TELE_DIDSID) [format "%06X" [lindex $parmList 20] ]
        set EncodeParm(TARGETADDR_dec) [lindex $parmList 21]
        set EncodeParm(TARGETADDR) [format "%06X" [lindex $parmList 21] ]
        set EncodeParm(SOURCEADDR_dec) [lindex $parmList 22]
        set EncodeParm(SOURCEADDR) [format "%06X" [lindex $parmList 22] ]
#        puts "Encode BlockB, $EncodeParm(LCO_MFID)"
    } elseif {$type == "BlockC"} {
        set EncodeParm(CTIMER_dec)  [lindex $parmList 0]
        set EncodeParm(CTIMER)      [format "%04X" [lindex $parmList 0] ]
        set EncodeParm(DIG0_4)      [format "%05X" [lindex $parmList 1] ]
        set EncodeParm(DIG5_9)      [format "%05X" [lindex $parmList 2] ]
        set EncodeParm(NETID)       [format "%05X" [lindex $parmList 3] ]
        set EncodeParm(SYSID_dec)   [lindex $parmList 4]
        set EncodeParm(SYSID)       [format "%03X" [lindex $parmList 4] ]
        set EncodeParm(SRCID_dec)   [lindex $parmList 5]
        set EncodeParm(SRCID)       [format "%06X" [lindex $parmList 5] ]
        set EncodeParm(PRIREQ)      [format "%01X" [lindex $parmList 6] ]
        set EncodeParm(SSAVAIL)     [format "%06X" [lindex $parmList 7] ]
        set EncodeParm(SSSUPPORT)   [format "%06X" [lindex $parmList 8] ]
        set EncodeParm(RFSUBID_dec) [lindex $parmList 9]
        set EncodeParm(RFSUBID)     [format "%02X" [lindex $parmList 9] ]
        set EncodeParm(SITEID_dec)  [lindex $parmList 10]
        set EncodeParm(SITEID)      [format "%02X" [lindex $parmList 10] ]
        set EncodeParm(SERVCLASS_dec)   [lindex $parmList 11]
        set EncodeParm(SERVCLASS)   [format "%02X" [lindex $parmList 11] ]
        set EncodeParm(SERVCLASS_A) [format "%02X" [lindex $parmList 12] ]
        set EncodeParm(SERVCLASS_B) [format "%02X" [lindex $parmList 13] ]
        set EncodeParm(LOCREGAREA_dec)  [lindex $parmList 14]
        set EncodeParm(LOCREGAREA)  [format "%02X" [lindex $parmList 14] ]
    } elseif {$type == "BlockD"} {
        set EncodeParm(STSUPDATE)   [format "%04X" [lindex $parmList 0] ]
        set EncodeParm(MSG)         [format "%04X" [lindex $parmList 1] ]
        set EncodeParm(EXTFUNCT_A)  [format "%04X" [lindex $parmList 2] ]
        set EncodeParm(EXTFUNCT_B)  [format "%06X" [lindex $parmList 3] ]
        set EncodeParm(BW)          [format "%03X" [lindex $parmList 4] ]
        set EncodeParm(XMITOFF)     [format "%03X" [lindex $parmList 5] ]
        set EncodeParm(CHANSPACE)   [format "%03X" [lindex $parmList 6] ]
        set EncodeParm(BASEFREQ)    [format "%08X" [lindex $parmList 7] ]
        set EncodeParm(LCO_ALGID)   [format "%02X" [lindex $parmList 8] ]
        if [info exists algid_short_str($EncodeParm(LCO_ALGID))] {
          set EncodeParm(LCO_ALGID_txt) $algid_short_str($EncodeParm(LCO_ALGID))
        } else {
          set EncodeParm(LCO_ALGID_txt) "Unknown"
        }
        set EncodeParm(LCO_KID)     [format "%04X" [lindex $parmList 9] ]
        set EncodeParm(SERVICEOP_dec)  [lindex $parmList 10]
        set EncodeParm(SERVICEOP)   [format "%02X" [lindex $parmList 10] ]
        set EncodeParm(RESERVE_4)   [format "%01X" [lindex $parmList 11] ]
        set EncodeParm(RESERVE_8)   [format "%02X" [lindex $parmList 12] ]
        set EncodeParm(RESERVE_16)  [format "%04X" [lindex $parmList 13] ]
        set EncodeParm(RESERVE_24)  [format "%06X" [lindex $parmList 14] ]
    } elseif {$type == "BlockE"} {
        set EncodeParm(WACNID_dec)  [lindex $parmList 0]
        set EncodeParm(WACNID)      [format "%05X" [lindex $parmList 0] ]
        set EncodeParm(WUID_dec)    [lindex $parmList 1]
        set EncodeParm(WUID)        [format "%06X" [lindex $parmList 1] ]
        set EncodeParm(WGID_dec)    [lindex $parmList 2]
        set EncodeParm(WGID)        [format "%04X" [lindex $parmList 2] ]
        set EncodeParm(CHANID_C_dec)    [lindex $parmList 3]
        set EncodeParm(CHANID_C)        [format "%01X" [lindex $parmList 3] ]
        set EncodeParm(CHANNUM_C_dec)   [lindex $parmList 4]
        set EncodeParm(CHANNUM_C)       [format "%03X" [lindex $parmList 4] ]
        set EncodeParm(CHANID_V_dec)    [lindex $parmList 5]
        set EncodeParm(CHANID_V)        [format "%01X" [lindex $parmList 5] ]
        set EncodeParm(CHANNUM_V_dec)   [lindex $parmList 6]
        set EncodeParm(CHANNUM_V)       [format "%03X" [lindex $parmList 6] ]
        set EncodeParm(ANNGRPADR_dec)   [lindex $parmList 7]
        set EncodeParm(ANNGRPADR)       [format "%04X" [lindex $parmList 7] ]
        set EncodeParm(T_PROT_dec)  [lindex $parmList 8]
        set EncodeParm(T_PROT)      [lindex $parmList 8]
        set EncodeParm(T_ANET_dec)  [lindex $parmList 9]
        set EncodeParm(T_ANET)      [lindex $parmList 9]
        set EncodeParm(T_LG_dec)    [lindex $parmList 10]
        set EncodeParm(T_LG)        [lindex $parmList 10]
        set EncodeParm(T_GAV_dec)   [lindex $parmList 11]
        set EncodeParm(T_GAV)       [lindex $parmList 11]
        set EncodeParm(T_RV_dec)    [lindex $parmList 12]
        set EncodeParm(T_RV)        [lindex $parmList 12]
    } elseif {$type == "BlockF"} {
        set EncodeParm(CHANIDRX_C_dec)    [lindex $parmList 0]
        set EncodeParm(CHANNUMRX_C_dec)   [lindex $parmList 1]
        set EncodeParm(CHANIDTX_C_dec)    [lindex $parmList 2]
        set EncodeParm(CHANNUMTX_C_dec)   [lindex $parmList 3]
        set EncodeParm(CHANIDRX_V_dec)    [lindex $parmList 4]
        set EncodeParm(CHANNUMRX_V_dec)   [lindex $parmList 5]
        set EncodeParm(CHANIDTX_V_dec)    [lindex $parmList 6]
        set EncodeParm(CHANNUMTX_V_dec)   [lindex $parmList 7]

        # Only set the channel numbers if in explicit mode
        if { ($TrunkModeCurrent == "UHF/VHF") && ($State(expmsg_txt) == "EXPLICIT") } \
        {
           set TrunkChanArray($TrunkModeCurrent,CHANIDRX_C) $EncodeParm(CHANIDRX_C_dec)
           set TrunkChanArray($TrunkModeCurrent,CHANIDTX_C) $EncodeParm(CHANIDTX_C_dec)
           set TrunkChanArray($TrunkModeCurrent,CHANIDRX_V) $EncodeParm(CHANIDRX_V_dec)
           set TrunkChanArray($TrunkModeCurrent,CHANIDTX_V) $EncodeParm(CHANIDTX_V_dec)
           set TrunkChanArray($TrunkModeCurrent,CHANNUMRX_C) $EncodeParm(CHANNUMRX_C_dec)
           set TrunkChanArray($TrunkModeCurrent,CHANNUMRX_V) $EncodeParm(CHANNUMRX_V_dec)
        }

        set TrunkChanArray($TrunkModeCurrent,CHANNUMTX_C) $EncodeParm(CHANNUMTX_C_dec)
        set TrunkChanArray($TrunkModeCurrent,CHANNUMTX_V) $EncodeParm(CHANNUMTX_V_dec)
    } elseif {$type == "BlockG"} {
        set EncodeParm(OCTET_0) [format "%02X" [lindex $parmList 0] ]
        set EncodeParm(OCTET_1) [format "%02X" [lindex $parmList 1] ]
        set EncodeParm(OCTET_2) [format "%02X" [lindex $parmList 2] ]
        set EncodeParm(OCTET_3) [format "%02X" [lindex $parmList 3] ]
        set EncodeParm(OCTET_4) [format "%02X" [lindex $parmList 4] ]
        set EncodeParm(OCTET_5) [format "%02X" [lindex $parmList 5] ]
        set EncodeParm(OCTET_6) [format "%02X" [lindex $parmList 6] ]
        set EncodeParm(OCTET_7) [format "%02X" [lindex $parmList 7] ]
        set EncodeParm(OCTET_8) [format "%02X" [lindex $parmList 8] ]
    } elseif {$type == "BlockH"} {
        set EncodeParm(ADJ_MFID_dec)   [lindex $parmList 0]
        set EncodeParm(ADJ_MFID)       [format "%02X" [lindex $parmList 0] ]
        set EncodeParm(ADJ_RFSSID_dec) [lindex $parmList 1]
        set EncodeParm(ADJ_RFSSID)     [format "%02X" [lindex $parmList 1] ]
        set EncodeParm(ADJ_SYSID_dec)  [lindex $parmList 2]
        set EncodeParm(ADJ_SYSID)      [format "%03X" [lindex $parmList 2] ]
        set EncodeParm(ADJ_SITEID_dec) [lindex $parmList 3]
        set EncodeParm(ADJ_SITEID)     [format "%02X" [lindex $parmList 3] ]
        set EncodeParm(ADJ_LRA_dec)    [lindex $parmList 4]
        set EncodeParm(ADJ_LRA)        [format "%02X" [lindex $parmList 4] ]
        set EncodeParm(ADJ_SCLASS_dec) [lindex $parmList 5]
        set EncodeParm(ADJ_SCLASS)     [format "%02X" [lindex $parmList 5] ]
        set EncodeParm(ADJ_CFLAG_dec)  [lindex $parmList 6]
        set EncodeParm(ADJ_FFLAG_dec)  [lindex $parmList 7]
        set EncodeParm(ADJ_VFLAG_dec)  [lindex $parmList 8]
        set EncodeParm(ADJ_AFLAG_dec)  [lindex $parmList 9]
        set EncodeParm(ADJ_CHANID_R_dec)  [lindex $parmList 10]
        set EncodeParm(ADJ_CHANID_R)      [format "%01X" [lindex $parmList 10] ]
        set EncodeParm(ADJ_CHANNUM_R_dec) [lindex $parmList 11]
        set EncodeParm(ADJ_CHANNUM_R)     [format "%03X" [lindex $parmList 11] ]
        set EncodeParm(ADJ_CHANID_T_dec)  [lindex $parmList 12]
        set EncodeParm(ADJ_CHANID_T)      [format "%01X" [lindex $parmList 12] ]
        set EncodeParm(ADJ_CHANNUM_T_dec) [lindex $parmList 13]
        set EncodeParm(ADJ_CHANNUM_T)     [format "%03X" [lindex $parmList 13] ]
    } elseif {$type == "BlockI"} {
        set EncodeParm(SCCB1_CHANID_R_dec)  [lindex $parmList 0]
        set EncodeParm(SCCB1_CHANID_R)      [format "%01X" [lindex $parmList 0] ]
        set EncodeParm(SCCB1_CHANNUM_R_dec) [lindex $parmList 1]
        set EncodeParm(SCCB1_CHANNUM_R)     [format "%03X" [lindex $parmList 1] ]
        set EncodeParm(SCCB1_CHANID_T_dec)  [lindex $parmList 2]
        set EncodeParm(SCCB1_CHANID_T)      [format "%01X" [lindex $parmList 2] ]
        set EncodeParm(SCCB1_CHANNUM_T_dec) [lindex $parmList 3]
        set EncodeParm(SCCB1_CHANNUM_T)     [format "%03X" [lindex $parmList 3] ]
        set EncodeParm(SCCB1_SSCLASS_dec)   [lindex $parmList 4]
        set EncodeParm(SCCB1_SSCLASS)       [format "%02X" [lindex $parmList 4] ]
        set EncodeParm(SCCB2_CHANID_R_dec)  [lindex $parmList 5]
        set EncodeParm(SCCB2_CHANID_R)      [format "%01X" [lindex $parmList 5] ]
        set EncodeParm(SCCB2_CHANNUM_R_dec) [lindex $parmList 6]
        set EncodeParm(SCCB2_CHANNUM_R)     [format "%03X" [lindex $parmList 6] ]
        set EncodeParm(SCCB2_CHANID_T_dec)  [lindex $parmList 7]
        set EncodeParm(SCCB2_CHANID_T)      [format "%01X" [lindex $parmList 7] ]
        set EncodeParm(SCCB2_CHANNUM_T_dec) [lindex $parmList 8]
        set EncodeParm(SCCB2_CHANNUM_T)     [format "%03X" [lindex $parmList 8] ]
        set EncodeParm(SCCB2_SSCLASS_dec)   [lindex $parmList 9]
        set EncodeParm(SCCB2_SSCLASS)       [format "%02X" [lindex $parmList 9] ]
    }
 }

#****************************************************************************
# Name:        updateEncodeLcoStrings
# Parameters:  encryptParm - encode encryption state, and algorithm ID
# Description: Sets up variable for the P25 encode encryption status
#****************************************************************************
 proc updateEncodeLcoStrings { args } \
 {
    variable EncodeParm
    variable LCO_strings
    variable LCO_short_strings

    if { [info exists LCO_strings($EncodeParm(LCO))] } \
    {
      set EncodeParm(LCO_text)  $LCO_strings($EncodeParm(LCO))
    } \
    else \
    {
      set EncodeParm(LCO_text) "<reserved>"
    }

    if { [info exists LCO_short_strings($EncodeParm(LCO))] } \
    {
      set EncodeParm(LCO_short_text)  $LCO_short_strings($EncodeParm(LCO))
    } \
    else \
    {
      set EncodeParm(LCO_short_text) "<reserved>"
    }
}   

#****************************************************************************
# Name:        updateAlgidParms
# Parameters:  encryptParm - encode encryption state, and algorithm ID
# Description: Sets up variable for the P25 encode encryption status
#****************************************************************************
 proc updateAlgidParms { algidParm } \
 {
    set len [llength $algidParm]

#    puts $algidParm
 }

#***************************************************************************
# Name: send_lco
# Parameters: none
# Description: Sends the selected LCO
#***************************************************************************
 proc send_lco { } \
 {
   variable  EncodeParm

   switch -- $EncodeParm(LCO) \
   {
     "00" { after idle "P25 encode lcomsg  0                      \
                                         0x$EncodeParm(P)         \
                                         0x$EncodeParm(SF)        \
                                         0x$EncodeParm(LCO_MFID)  \
                                         0x$EncodeParm(EMG)       \
                                         0x$EncodeParm(SERVICEOP) \
                                         0x0                      \
                                         0x$EncodeParm(GRPADDR)   \
                                         0x$EncodeParm(SOURCEADDR) -nowait" }

     "02" { after idle "P25 encode lcomsg  2                      \
                                         0x$EncodeParm(P)         \
                                         0x$EncodeParm(SF)        \
                                         0x$EncodeParm(CHANID_A)  \
                                         0x$EncodeParm(CHANNUM_A) \
                                         0x$EncodeParm(GRPADDR_A) \
	                                 0x$EncodeParm(CHANID_B)  \
	                                 0x$EncodeParm(CHANNUM_B) \
	                                 0x$EncodeParm(GRPADDR_B) -nowait" }

     "03" { after idle "P25 encode lcomsg  3                       \
                                         0x$EncodeParm(P)          \
                                         0x$EncodeParm(SF)         \
                                         0x$EncodeParm(LCO_MFID)   \
                                         0x$EncodeParm(EMG)        \
                                         0x$EncodeParm(SERVICEOP)  \
                                         0x$EncodeParm(TARGETADDR) \
                                         0x$EncodeParm(SOURCEADDR) -nowait" }

     "04" { after idle "P25 encode lcomsg  4                      \
                                         0x$EncodeParm(P)         \
                                         0x$EncodeParm(SF)        \
                                         0x0                      \
                                         0x$EncodeParm(EMG)       \
                                         0x$EncodeParm(SERVICEOP) \
                                         0x$EncodeParm(GRPADDR)   \
                                         0x$EncodeParm(CHANID_T)  \
                                         0x$EncodeParm(CHANNUM_T) \
                                         0x$EncodeParm(CHANID_R)  \
                                         0x$EncodeParm(CHANNUM_R) -nowait" }

     "05" { after idle "P25 encode lcomsg  5                       \
                                         0x$EncodeParm(P)          \
                                         0x$EncodeParm(SF)         \
                                         0x$EncodeParm(EMG)        \
                                         0x$EncodeParm(SERVICEOP)  \
                                         0x0                       \
                                         0x$EncodeParm(TARGETADDR) \
                                         0x$EncodeParm(SOURCEADDR) -nowait" }

     "06" { after idle "P25 encode lcomsg  6                      \
                                         0x$EncodeParm(P)         \
                                         0x$EncodeParm(SF)        \
                                         0x0                      \
                                         0x$EncodeParm(EMG)       \
                                         0x$EncodeParm(SERVICEOP) \
                                         0x0                      \
                                         0x$EncodeParm(CTIMER)    \
                                         0x$EncodeParm(TARGETADDR) -nowait" }

     "07" { after idle "P25 encode digits $EncodeParm(DIG0_9)" 
            after idle "P25 encode lcomsg  7                      \
                                         0x$EncodeParm(P)         \
                                         0x$EncodeParm(SF)        \
                                         0x$EncodeParm(TARGETADDR) -nowait" }
                                
            

     "0F" { after idle "P25 encode lcomsg  15                     \
                                         0x$EncodeParm(P)         \
                                         0x$EncodeParm(SF)        \
                                         0x0                      \
                                         0x0                      \
                                         0x$EncodeParm(TARGETADDR) -nowait" }

     "10" { after idle "P25 encode lcomsg  16                      \
                                         0x$EncodeParm(P)          \
                                         0x$EncodeParm(SF)         \
                                         0x0                       \
                                         0x$EncodeParm(TARGETADDR) \
                                         0x$EncodeParm(SOURCEADDR) -nowait" }

     "11" { after idle "P25 encode lcomsg  17                      \
                                         0x$EncodeParm(P)          \
                                         0x$EncodeParm(SF)         \
                                         0x$EncodeParm(NETID)      \
                                         0x$EncodeParm(SYSID)      \
                                         0x$EncodeParm(SRCID)      \
                                         0x0 -nowait" }

     "12" { after idle "P25 encode lcomsg  18                      \
                                         0x$EncodeParm(P)          \
                                         0x$EncodeParm(SF)         \
                                         0x$EncodeParm(NETID)      \
                                         0x$EncodeParm(SYSID)      \
                                         0x$EncodeParm(SRCID)      \
                                         0x0 -nowait" }

     "13" { after idle "P25 encode lcomsg  19                      \
                                         0x$EncodeParm(P)          \
                                         0x$EncodeParm(SF)         \
                                         0x0                       \
                                         0x$EncodeParm(TARGETADDR) \
                                         0x$EncodeParm(SOURCEADDR) -nowait" }

     "14" { after idle "P25 encode lcomsg  20                      \
                                         0x$EncodeParm(P)          \
                                         0x$EncodeParm(SF)         \
                                         0x$EncodeParm(STSUPDATE)  \
                                         0x$EncodeParm(TARGETADDR) \
                                         0x$EncodeParm(SOURCEADDR) -nowait" }

     "15" { after idle "P25 encode lcomsg  21                      \
                                         0x$EncodeParm(P)          \
                                         0x$EncodeParm(SF)         \
                                         0x$EncodeParm(MSG)        \
                                         0x$EncodeParm(TARGETADDR) \
                                         0x$EncodeParm(SOURCEADDR) -nowait" }

     "16" { after idle "P25 encode lcomsg  22                      \
                                         0x$EncodeParm(P)          \
                                         0x$EncodeParm(SF)         \
                                         0x0                       \
                                         0x$EncodeParm(TARGETADDR) \
                                         0x$EncodeParm(SOURCEADDR) -nowait" }

     "17" { after idle "P25 encode lcomsg  23                      \
                                         0x$EncodeParm(P)          \
                                         0x$EncodeParm(SF)         \
                                         0x$EncodeParm(EXTFUNCT_A) \
                                         0x$EncodeParm(EXTFUNCT_B) \
                                         0x$EncodeParm(TARGETADDR) -nowait" }

     "18" { after idle "P25 encode lcomsg  24                      \
                                         0x$EncodeParm(P)          \
                                         0x$EncodeParm(SF)         \
                                         0x$EncodeParm(CHANID)     \
                                         0x$EncodeParm(BW)         \
                                         0x$EncodeParm(XMITOFF)    \
                                         0x$EncodeParm(CHANSPACE)  \
                                         0x$EncodeParm(BASEFREQ) -nowait" }

     "20" { after idle "P25 encode lcomsg  32                      \
                                         0x$EncodeParm(P)          \
                                         0x$EncodeParm(SF)         \
                                         0x0                       \
                                         0x0                       \
                                         0x$EncodeParm(PRIREQ)     \
                                         0x$EncodeParm(SSAVAIL)    \
                                         0x$EncodeParm(SSSUPPORT) -nowait" }

     "21" { after idle "P25 encode lcomsg  33                       \
                                         0x$EncodeParm(P)           \
                                         0x$EncodeParm(SF)          \
                                         0x$EncodeParm(RFSUBID)     \
                                         0x$EncodeParm(SITEID)      \
                                         0x$EncodeParm(CHANID_A)    \
                                         0x$EncodeParm(CHANNUM_A)   \
                                         0x$EncodeParm(SERVCLASS_A) \
                                         0x$EncodeParm(CHANID_B)    \
                                         0x$EncodeParm(CHANNUM_B)   \
                                         0x$EncodeParm(SERVCLASS_B) -nowait" }

     "22" { after idle "P25 encode lcomsg  34                       \
                                         0x$EncodeParm(P)           \
                                         0x$EncodeParm(SF)          \
                                         0x$EncodeParm(LOCREGAREA)  \
                                         0x0                        \
                                         0x$EncodeParm(SYSID)       \
                                         0x$EncodeParm(RFSUBID)     \
                                         0x$EncodeParm(SITEID)      \
                                         0x$EncodeParm(CHANID)      \
                                         0x$EncodeParm(CHANNUM)     \
                                         0x$EncodeParm(SERVCLASS) -nowait" }

     "23" { after idle "P25 encode lcomsg  35                       \
                                         0x$EncodeParm(P)           \
                                         0x$EncodeParm(SF)          \
                                         0x$EncodeParm(LOCREGAREA)  \
                                         0x0                        \
                                         0x$EncodeParm(SYSID)       \
                                         0x$EncodeParm(RFSUBID)     \
                                         0x$EncodeParm(SITEID)      \
                                         0x$EncodeParm(CHANID)      \
                                         0x$EncodeParm(CHANNUM)     \
                                         0x$EncodeParm(SERVCLASS) -nowait" }

     "24" { after idle "P25 encode lcomsg  36                       \
                                         0x$EncodeParm(P)           \
                                         0x$EncodeParm(SF)          \
                                         0x0                        \
                                         0x$EncodeParm(NETID)       \
                                         0x$EncodeParm(SYSID)       \
                                         0x$EncodeParm(CHANID)      \
                                         0x$EncodeParm(CHANNUM)     \
                                         0x$EncodeParm(SERVCLASS) -nowait" }

     "25" { after idle "P25 encode lcomsg  37                       \
                                         0x$EncodeParm(P)           \
                                         0x$EncodeParm(SF)          \
                                         0x0                        \
                                         0x$EncodeParm(LCO_ALGID)   \
                                         0x$EncodeParm(LCO_KID)     \
                                         0x$EncodeParm(TARGETADDR) -nowait" }
    
   }

   return
 }


#***************************************************************************
# Name: send_lco_0
# Parameters: none
# Description: Sends the LCO 0 when the Generator modulation is set to 'P 25'
#***************************************************************************
 proc send_lco_0 { args } \
 {
   variable  EncodeParm
   
   switch -- $::Generator::State(mod) \
   {
     "P25 LSM" -
     "P22" \
     {
        after idle "P25 encode field  lco  0x0 -nowait"

        after idle "P25 encode lcomsg  0                        \
                                       0x$EncodeParm(P)         \
                                       0x$EncodeParm(SF)        \
                                       0x$EncodeParm(LCO_MFID)  \
                                       0x$EncodeParm(EMG)       \
                                       0x$EncodeParm(SERVICEOP) \
                                       0x0                      \
                                       0x$EncodeParm(GRPADDR)   \
                                       0x$EncodeParm(SOURCEADDR) -nowait"
      }
   }
   
   return
 }

#****************************************************************************
# Name:        updateOBT_Ctrl
# Parameters:  type     Current OBT control type
# Description: This saves and updates strings indicating the current method
#     being used to expand 2 block trunking packets to 3 block sized packets
#****************************************************************************
 proc updateOBT_Ctrl { type } \
 {
    variable State

    set State(obtctrl) $type
    if { $type == 2 } \
    {
       set State(obtctrl_Stxt) "TYPE 1"
    } elseif { $type == 6 } \
    {
       set State(obtctrl_Stxt) "TYPE 2"
    } else \
    {
       set State(obtctrl_Stxt) "TYPE 0"
    }
 }




#****************************************************************************
# Name:        updateMsgFormatType
# Parameters:  type     Current message format type
# Description: Saves the Explicit / Implicit message formatting type
#     This is used for outbound P25 trunking messages
#****************************************************************************
 proc updateMsgFormatType { type } \
 {
    variable State

    set State(expmsg) $type
    if { $type == 1 } \
    {
       set State(expmsg_txt) EXPLICIT
    } else \
    {
       set State(expmsg_txt) IMPLICIT
    }
 }

#****************************************************************************
# Name:        toggleMsgFormatType
# Parameters:  none
# Description: Saves the Tx and Rx trunking control frequencies
#****************************************************************************
 proc toggleMsgFormatType { } \
 {
    variable State

    if { $State(expmsg) == 1 } \
    {
       P25 encode explicitmessages 0 -nowait
    } else \
    {
       P25 encode explicitmessages 1 -nowait
    }
    EnableTrunking "Disable"
 }


#****************************************************************************
# Name:        updateTrunkCtrlFreqs
# Parameters:  parmList	   Data from the engine
# Description: Saves the Tx and Rx trunking control frequencies
#****************************************************************************
 proc updateTrunkFreqs { type parmList } \
 {
    variable TrunkFreqs
    if { $type == "CTRL" } \
    {
        set TrunkFreqs(ccTX)   [format "%5.5f" [lindex $parmList 0] ]
        set TrunkFreqs(ccRX)   [format "%5.5f" [lindex $parmList 1] ]
    } \
    elseif { $type == "VOICE" } \
    {
        set TrunkFreqs(vcTX)   [format "%5.5f" [lindex $parmList 0] ]
        set TrunkFreqs(vcRX)   [format "%5.5f" [lindex $parmList 1] ]
    } \
    elseif { $type == "ADJCHANTR" } \
    {
        set TrunkFreqs(adjchanT)   [format "%5.5f" [lindex $parmList 0] ]
        set TrunkFreqs(adjchanR)   [format "%5.5f" [lindex $parmList 1] ]
    } \
    elseif { $type == "SCCB1" } \
    {
        set TrunkFreqs(sccb1_chanT)   [format "%5.5f" [lindex $parmList 0] ]
        set TrunkFreqs(sccb1_chanR)   [format "%5.5f" [lindex $parmList 1] ]
    } \
    elseif { $type == "SCCB2" } \
    {
        set TrunkFreqs(sccb2_chanT)   [format "%5.5f" [lindex $parmList 0] ]
        set TrunkFreqs(sccb2_chanR)   [format "%5.5f" [lindex $parmList 1] ]
    }
 }


#****************************************************************************
# Name:        updateEncodeEncrypt
# Parameters:  encryptParm - encode encryption state, and algorithm ID
# Description: Sets up variable for the P25 encode encryption status
#****************************************************************************
 proc updateEncodeEncrypt { encryptParm } {

    set state [lindex $encryptParm 0]
    set algid [lindex $encryptParm 1]

#    if {$encrypt} {
#       set ::P25ns::State(e_encryptState) 1
#       set ::P25ns::EncodeState(ALGID) 81
#       set ::P25ns::EncodeState(ALGID_text) $::P25ns::algid_strings($::P25ns::EncodeState(ALGID))

#    } else {
#       set ::P25ns::State(e_encryptState) 0
#       set ::P25ns::EncodeState(ALGID) 80
#       set ::P25ns::EncodeState(ALGID_text) $::P25ns::algid_strings($::P25ns::EncodeState(ALGID))
#    }
 }
    
#****************************************************************************
# Name:        updateChannelAddr
# Parameters:  parmList	   Data from the engine
# Description: Updates an array element of the Channel Address array
#****************************************************************************
 proc updateChannelAddr { parmList } {
    variable EncodeParm
    variable trunk_mode_type
    variable ChannelAddr
    variable TrunkModeIndex
    variable TrunkModeCurrent
    variable ChannelConfig
    
    set band [lindex $parmList 0]
    if [info exists trunk_mode_type($band)] \
    {
       set band $trunk_mode_type($band)
    }

    if { $band == $TrunkModeCurrent } \
    {
        set ChannelAddr(INDEX)   [lindex $parmList 1]
        set i $ChannelAddr(INDEX)
        set ChannelAddr(BW)           [format "%2.3f" [lindex $parmList 2] ]
        set ChannelConfig($i,BW)      [format "%2.3f" [lindex $parmList 2] ]
        set ChannelAddr(OFFSET)       [format "%2.4f" [lindex $parmList 3] ]
        set ChannelConfig($i,OFFSET)  [format "%2.4f" [lindex $parmList 3] ]
        set ChannelAddr(SPACING)      [format "%3.3f" [lindex $parmList 4] ]
        set ChannelConfig($i,SPACING) [format "%3.3f" [lindex $parmList 4] ]
        set ChannelAddr(FREQ)       [format "%5.6f" [lindex $parmList 5] ]
        set ChannelConfig($i,FREQ) [format "%5.6f" [lindex $parmList 5] ]
    }
 }

#****************************************************************************
# Name:        set_channel_config_parms
# Parameters:  none
# Description: Loads the channel configuration variables with current band data
#****************************************************************************
 proc set_channel_config_parms { } \
 {
    variable ChannelConfig
    variable trunk_mode_type
    variable TrunkModeCurrent
    variable TrunkModeIndex

    set band $trunk_mode_type($TrunkModeCurrent)
    set index $TrunkModeIndex($TrunkModeCurrent)
    for { set i  0 } { $i <= 15 } { incr i  1 } \
    {
       set templist [P25 encode chanaddr $band $i]
       set ChannelConfig($i,BW)      [format "%2.3f" [lindex $templist 2] ]
       set ChannelConfig($i,OFFSET)  [format "%3.4f" [lindex $templist 3] ]
       set ChannelConfig($i,SPACING) [format "%2.3f" [lindex $templist 4] ]
       set ChannelConfig($i,FREQ)    [format "%3.6f" [lindex $templist 5] ]
    }
     P25 encode chanaddr $band $index -nowait


 }


#****************************************************************************
# Name:        updateTrunkModeIndex
# Parameters:  parmList	   Data from the engine
# Description: Updates an array element of the Channel Address array
#****************************************************************************
 proc updateTrunkModeIndex { parmList } {
    variable trunk_mode_type
    variable TrunkModeIndex
    variable TrunkModeCurrent
    variable TrunkChanArray
    variable State

    set mode [lindex $parmList 0]
    if [info exists trunk_mode_type($mode)] \
    {
       set TrunkModeCurrent $trunk_mode_type($mode)
       set TrunkModeIndex($TrunkModeCurrent) [lindex $parmList 1]

       # Bypass if we are loading a setup - sequence order problems
       if { ![info exists ::Recall_SetupsUI::State(recall_in_process)] } \
       {
          return
       } elseif { $::Recall_SetupsUI::State(recall_in_process) == 1 } \
       {
          return
       }
       
       # Switch the channel numbers to the ones last used by the new mode
       if { ($TrunkModeCurrent == "UHF/VHF") && ($State(expmsg_txt) == "EXPLICIT") } \
       {
          after idle P25 encode field CHANIDRX_C  $TrunkChanArray($TrunkModeCurrent,CHANIDRX_C) -nowait
          after idle P25 encode field CHANIDTX_C  $TrunkChanArray($TrunkModeCurrent,CHANIDTX_C) -nowait
          after idle P25 encode field CHANIDRX_V  $TrunkChanArray($TrunkModeCurrent,CHANIDRX_V) -nowait
          after idle P25 encode field CHANIDTX_V  $TrunkChanArray($TrunkModeCurrent,CHANIDTX_V) -nowait
       } else \
       {
          set chan_id $TrunkModeIndex($TrunkModeCurrent)
          after idle P25 encode field CHANIDRX_C  $chan_id -nowait
          after idle P25 encode field CHANIDTX_C  $chan_id -nowait
          after idle P25 encode field CHANIDRX_V  $chan_id -nowait
          after idle P25 encode field CHANIDTX_V  $chan_id -nowait
       }

       after idle P25 encode field CHANNUMRX_C $TrunkChanArray($TrunkModeCurrent,CHANNUMRX_C) -nowait
       after idle P25 encode field CHANNUMTX_C $TrunkChanArray($TrunkModeCurrent,CHANNUMTX_C) -nowait
       after idle P25 encode field CHANNUMRX_V $TrunkChanArray($TrunkModeCurrent,CHANNUMRX_V) -nowait
       after idle P25 encode field CHANNUMTX_V $TrunkChanArray($TrunkModeCurrent,CHANNUMTX_V) -nowait
    }

    # When the mode or index changes, send a query
    after idle P25 encode chanaddr $mode $TrunkModeIndex($TrunkModeCurrent)
 }

#****************************************************************************
# Name:        setTrunkModeIndex
# Parameters:  none
# Description: Sets both the trunking mode and channel index
#****************************************************************************
 proc setTrunkModeIndex { {newMode "NONE"} } {
    variable TrunkModeIndex
    variable TrunkModeCurrent

    if {$newMode == "NONE"} {
        set newMode $TrunkModeCurrent
    }

    if {$newMode == "800"} {
        P25 encode TrunkModeIndex 0 $TrunkModeIndex($newMode) -nowait
    } elseif {$newMode == "UHF/VHF"} {
        P25 encode TrunkModeIndex 1 $TrunkModeIndex($newMode) -nowait
    } elseif {$newMode == "700"} {
        P25 encode TrunkModeIndex 2 $TrunkModeIndex($newMode) -nowait
    }
 }

#****************************************************************************
# Name:        setTrunkModeIndexValue
# Parameters:  none
# Description: Updates the channel index for the current trunking mode
#****************************************************************************
 proc setTrunkModeIndexValue {  } {
    variable TrunkModeCurrent
    variable ChannelAddr
    variable trunk_mode_type

    if [info exists trunk_mode_type($TrunkModeCurrent)] \
    {
        set mode $trunk_mode_type($TrunkModeCurrent)
        P25 encode TrunkModeIndex $mode $ChannelAddr(INDEX) -nowait
    }
 }


 
#****************************************************************************
# Name:        setChannelParms
# Parameters:  none
# Description: Sets both the trunking mode and channel index
#****************************************************************************
 proc setChannelParms {  } {
 
    variable TrunkModeCurrent
    variable ChannelAddr
    variable trunk_mode_type

    if [info exists trunk_mode_type($TrunkModeCurrent)] \
    {
        set mode $trunk_mode_type($TrunkModeCurrent)
        
        P25 encode chanaddr $mode $ChannelAddr(INDEX) $ChannelAddr(BW) \
            $ChannelAddr(OFFSET) $ChannelAddr(SPACING) $ChannelAddr(FREQ) -nowait
    }
 }
 
#****************************************************************************
# Name:        setTrunkSimState
# Parameters:  none
# Description: This function is called when the trunking simulation is changed
#****************************************************************************
 proc setTrunkSimState { value } {
   variable State

   if { $value == 1 } \
   {
      set State(trunk_on_off) "ON"
   } else \
   {
      set State(trunk_on_off) "OFF"
   }
 }

#****************************************************************************
# Name:        setTrunkPttCtrl
# Parameters:  none
# Description: This function is called when the user changes the Trunking PTT
#****************************************************************************
 proc setTrunkPttCtrl { value } {
   variable State

   set State(trunk_ptt_state) $value
   if { $value == 1 } \
   {
      set State(trunk_ptt_on_off) "PTT ON"
   } else \
   {
      set State(trunk_ptt_on_off) "PTT OFF"
   }
 }

#****************************************************************************
# Name:        setTrunkRcvPtt
# Parameters:  none
# Description: This function is called when the radio requests a voice channel
#****************************************************************************
 proc setTrunkRcvPtt { value } {
   variable State

   if { $value == 0 } \
   {
      set State(trunk_ptt_rcv) 0
   } else \
   {
      set State(trunk_ptt_rcv) 1
   }
 }

#****************************************************************************
# Name:        updateAutocopy
# Parameters:  none
# Description: This function is called to update the TCL variables when the
#    engine reports a change in the Autocopy status
#****************************************************************************
 proc updateAutocopy { value } {
   variable State

   set State(AutoWrap) $value
   if { $value & 0x01 } { \
      set State(AutoCopy_Reg) 1
   } else { \
      set State(AutoCopy_Reg) 0
   }

   if { $value & 0x02 } { \
      set State(AutoCopy_Aff) 1
   } else { \
      set State(AutoCopy_Aff) 0
   }

   if { $value & 0x04 } { \
      set State(AutoCopy_Voice) 1
   } else { \
      set State(AutoCopy_Voice) 0
   }

   if { $value & 0x08 } { \
      set State(AutoCopy_UU) 1
   } else { \
      set State(AutoCopy_UU) 0
   }

   if { $value & 0x10 } { \
      set State(AutoCopy_Dial) 1
   } else { \
      set State(AutoCopy_Dial) 0
   }

 }

#****************************************************************************
# Name:        setAutocopy
# Parameters:  none
# Description: This function is called when the user changes the settings on
#    the variables which are copied from radio to repeater sim.
#****************************************************************************
 proc setAutocopy { } {
   variable State

   set temp  [expr $State(AutoCopy_Reg) +         \
                  ($State(AutoCopy_Aff) << 1) +   \
                  ($State(AutoCopy_Voice) << 2) + \
                  ($State(AutoCopy_UU) << 3) +    \
                  ($State(AutoCopy_Dial) << 4)]
   P25 encode autowrapdata $temp -nowait
 }

#****************************************************************************
# Name:        setLogMask
# Parameters:  none
# Description: This function is called when the user selects a new P25 data
#              log option. It deselects options which cannot be obtained at
#              the same time, and updates the log mask
#****************************************************************************
 proc setLogMask { type } {
   variable Log

   # Deselect options which are incompatible with the selected one 
   switch -- $type \
   {
     "lco_msg"     -
     "lco_octets"  { 
                     if { $Log(extended) != 1 } \
                     {
                        set Log(isp_octets)  0
                        set Log(isp_msg)     0 
                     }
                     set Log(osp_octets)  0
                     set Log(osp_msg)     0
                   }
                   
     "osp_msg"     -
     "osp_octets"  { set Log(lco_octets)  0
                     set Log(lco_msg)     0
                     set Log(isp_octets)  0
                     set Log(isp_msg)     0
                   }
                   
     "isp_msg"     -
     "isp_octets"  { 
                     if { $Log(extended) != 1 } \
                     {
                        set Log(lco_octets)  0
                        set Log(lco_msg)     0
                     }
                     set Log(osp_octets)  0
                     set Log(osp_msg)     0
                   }
                   
     "none"        { set Log(lco_octets)  0
                     set Log(lco_msg)     0
                     set Log(osp_octets)  0
                     set Log(osp_msg)     0
                     set Log(isp_octets)  0
                     set Log(isp_msg)     0
                   }
   }

   # NOTE: The logger uses some flag bits to control a dump of debug information
   #    which is not currently available to the user. Variables are:
   #        Log(debug_xmt),  Log(debug_rcv)
   #    These are initialized to ZERO in the pkgIndex.tcl file, and are never set 
   #    in the program. They can be hand modified without fear of the data being
   #    reset until power is cycled.
   set Log(mask)  [expr ($Log(raw) << 0) + \
                        ($Log(osp_octets) << 1) + ($Log(osp_msg) << 4) + \
                        ($Log(isp_octets) << 2) + ($Log(isp_msg) << 5) + \
                        ($Log(lco_octets) << 6) + ($Log(lco_msg) << 7) + \
                        ($Log(debug_xmt) << 12)  + ($Log(debug_rcv) << 13)]
 }



#****************************************************************************
# Name:        setTrunkRcvReg
# Parameters:  none
# Description: This function is called when the radio attempts to register
#****************************************************************************
 proc setTrunkRcvReg { } {
   variable State

   set State(trunk_reg_rcv) 1
   set cmd {set ::P25ns::State(trunk_reg_rcv) 0}
   after cancel $cmd
   after 2000 $cmd
 }

#****************************************************************************
# Name:        setTrunkRcvAff
# Parameters:  none
# Description: This function is called when the radio attempts to affiliate
#****************************************************************************
 proc setTrunkRcvAff { } {
   variable State

   set State(trunk_aff_rcv) 1
   set cmd {set ::P25ns::State(trunk_aff_rcv) 0}
   after cancel $cmd
   after 2000 $cmd
 }

#****************************************************************************
# Name:        setTrunkRcvSignal
# Parameters:  none
# Description: This function is called when the radio attempts a phone call
#****************************************************************************
 proc setTrunkRcvSignal { type } {
   variable State

   set State(trunk_signal_rcv) $type
#   set cmd {set ::P25ns::State(trunk_signal_rcv) 0}
#   after cancel $cmd
#   after 3000 $cmd
 }

#****************************************************************************
# Name:        setTrunkRcvDial
# Parameters:  none
# Description: This function is called when the radio attempts a phone call
#****************************************************************************
 proc setTrunkRcvDial { } {
   variable State

   set State(trunk_dial_rcv) 1
   set cmd {set ::P25ns::State(trunk_dial_rcv) 0}
   after cancel $cmd
   after 2000 $cmd
 }

#****************************************************************************
# Name:        setTrunkRcvPSTN
# Parameters:  none
# Description: This function is called when the radio attempts a phone call
#****************************************************************************
 proc setTrunkRcvPSTN { } {
   variable State

   set State(trunk_pstn_rcv) 1
   set cmd {set ::P25ns::State(trunk_pstn_rcv) 0}
   after cancel $cmd
   after 2000 $cmd
 }

#****************************************************************************
# Name:        sendTenDigits
# Parameters:  none
# Description: This function is called to send down the entered phone number
#****************************************************************************
 proc sendTenDigits { } {
   variable EncodeParm

   P25 encode digits  $EncodeParm(DIG0_9) -nowait
 }

#****************************************************************************
# Name:        setTxFrequency
# Parameters:  designator
# Description: Sets the TX Frequency then updates the channel to the channel
#              closest to this frequency
#****************************************************************************
 proc setTxFrequency { designator } {
   variable TrunkFreqs
   variable TrunkModeCurrent
   variable TrunkModeIndex
   variable trunk_mode_type
   
   if [info exists trunk_mode_type($TrunkModeCurrent)] \
   {
      set mode $trunk_mode_type($TrunkModeCurrent)
   } else \
   {
      return
   }

   switch -- $designator \
   {
     "ccTX"  { 
        set freqList [P25 encode freq2chan $TrunkFreqs(ccTX) $mode $TrunkModeIndex($TrunkModeCurrent)]
        set channel [lindex $freqList 0]
        set index   [lindex $freqList 2]
        P25 encode field channumtx_c $channel
#        P25 encode field chanidtx_c $index 
     }
     
     "vcTX"  { 
        set freqList [P25 encode freq2chan $TrunkFreqs(vcTX) $mode $TrunkModeIndex($TrunkModeCurrent)]
        set channel [lindex $freqList 0]
        set index   [lindex $freqList 2]
        P25 encode field channumtx_v $channel
#        P25 encode field chanidtx_v $index 
     }
   }

   return
 }



#****************************************************************************
# Name:        SnapToFrequency
# Parameters:  designator
# Description: This uses the specified frequency to look up the closest
#              channel to that frequency, and then updates the channel 
#              (which snaps the frequency to the closest value)
#****************************************************************************
 proc SnapToFrequency { designator } {
   variable EncodeParm
   variable TrunkModeCurrent
   variable TrunkFreqs
   variable trunk_mode_type

   if [info exists trunk_mode_type($TrunkModeCurrent)] \
   {
      set mode $trunk_mode_type($TrunkModeCurrent)
   } else \
   {
      return
   }

   switch -- $designator \
   {
     "ccTX"  { 
        set freqList [P25 encode freq2chan $TrunkFreqs(ccTX) $mode $EncodeParm(CHANIDTX_C_dec)]
        set channel [lindex $freqList 0]
        set index   [lindex $freqList 2]
        P25 encode field channumtx_c $channel
     }
     
     "ccRX"  { 
        set freqList [P25 encode freq2chan $TrunkFreqs(ccRX) $mode $EncodeParm(CHANIDRX_C_dec)]
        set channel [lindex $freqList 0]
        set index   [lindex $freqList 2]
        P25 encode field channumrx_c $channel
     }
     
     "vcTX"  { 
        set freqList [P25 encode freq2chan $TrunkFreqs(vcTX) $mode $EncodeParm(CHANIDTX_V_dec)]
        set channel [lindex $freqList 0]
        set index   [lindex $freqList 2]
        P25 encode field channumtx_v $channel
     }

     "vcRX"  { 
        set freqList [P25 encode freq2chan $TrunkFreqs(vcRX) $mode $EncodeParm(CHANIDRX_V_dec)]
        set channel [lindex $freqList 0]
        set index   [lindex $freqList 2]
        P25 encode field channumrx_v $channel
     }

     "adjchanT"  { 
        set freqList [P25 encode freq2chan $TrunkFreqs(adjchanT) $mode $EncodeParm(ADJ_CHANID_T_dec)]
        set channel [lindex $freqList 0]
        set index   [lindex $freqList 2]
        P25 encode field adj_channum_t $channel
     }

     "adjchanR"  { 
        set freqList [P25 encode freq2chan $TrunkFreqs(adjchanR) $mode $EncodeParm(ADJ_CHANID_R_dec)]
        set channel [lindex $freqList 0]
        set index   [lindex $freqList 2]
        P25 encode field adj_channum_r $channel
     }

     "sccb1_chanT"  { 
        set freqList [P25 encode freq2chan $TrunkFreqs(sccb1_chanT) $mode $EncodeParm(SCCB1_CHANID_T_dec)]
        set channel [lindex $freqList 0]
        set index   [lindex $freqList 2]
        P25 encode field sccb1_channum_t $channel
     }

     "sccb1_chanR"  { 
        set freqList [P25 encode freq2chan $TrunkFreqs(sccb1_chanR) $mode $EncodeParm(SCCB1_CHANID_R_dec)]
        set channel [lindex $freqList 0]
        set index   [lindex $freqList 2]
        P25 encode field sccb1_channum_r $channel
     }

     "sccb2_chanT"  { 
        set freqList [P25 encode freq2chan $TrunkFreqs(sccb2_chanT) $mode $EncodeParm(SCCB2_CHANID_T_dec)]
        set channel [lindex $freqList 0]
        set index   [lindex $freqList 2]
        P25 encode field sccb2_channum_t $channel
     }

     "sccb2_chanR"  { 
        set freqList [P25 encode freq2chan $TrunkFreqs(sccb2_chanR) $mode $EncodeParm(SCCB2_CHANID_R_dec)]
        set channel [lindex $freqList 0]
        set index   [lindex $freqList 2]
        P25 encode field sccb2_channum_r $channel
     }
   }
 }

#****************************************************************************
# Name:        getSaveState
# Parameters:  none
# Description: This function returns a list of TCL commands which can be used
#              to set the P25 engine to its current state
#****************************************************************************
 proc getSaveState { } \
 {
   variable  EncodeParm
   variable  State
   variable  TrunkChanArray
   variable  TrunkModeIndex
   variable  TrunkModeCurrent
   variable  Log

   # Save the P, SF, and LCO values in temporary variables
   set temp_P    $EncodeParm(P)
   set temp_SF   $EncodeParm(SF)
   set temp_LCO  $EncodeParm(LCO)
   

   set channelList \
   {
     # Restore P25 channel variables
     P25 encode field CHANID_C    0x$EncodeParm(CHANID_C) -nowait
     P25 encode field CHANID_V    0x$EncodeParm(CHANID_V) -nowait
     P25 encode field chanidtx_c  $EncodeParm(CHANIDTX_C_dec) -nowait
     P25 encode field chanidrx_c  $EncodeParm(CHANIDRX_C_dec) -nowait
     P25 encode field chanidtx_v  $EncodeParm(CHANIDTX_V_dec) -nowait
     P25 encode field chanidrx_v  $EncodeParm(CHANIDRX_V_dec) -nowait
     P25 encode field adj_chanid_r   0x$EncodeParm(ADJ_CHANID_R) -nowait
     P25 encode field adj_chanid_t   0x$EncodeParm(ADJ_CHANID_T) -nowait
     P25 encode field sccb1_chanid_t   $EncodeParm(SCCB1_CHANID_T_dec) -nowait
     P25 encode field sccb1_chanid_r   $EncodeParm(SCCB1_CHANID_R_dec) -nowait
     P25 encode field sccb2_chanid_t   $EncodeParm(SCCB2_CHANID_T_dec) -nowait
     P25 encode field sccb2_chanid_r   $EncodeParm(SCCB2_CHANID_R_dec) -nowait
     P25 encode field CHANNUM_C   0x$EncodeParm(CHANNUM_C) -nowait
     P25 encode field CHANNUM_V   0x$EncodeParm(CHANNUM_V) -nowait
     P25 encode field channumtx_c $EncodeParm(CHANNUMTX_C_dec) -nowait
     P25 encode field channumrx_c $EncodeParm(CHANNUMRX_C_dec) -nowait
     P25 encode field channumtx_v $EncodeParm(CHANNUMTX_V_dec) -nowait
     P25 encode field channumrx_v $EncodeParm(CHANNUMRX_V_dec) -nowait
     P25 encode field adj_channum_r  0x$EncodeParm(ADJ_CHANNUM_R) -nowait
     P25 encode field adj_channum_t  0x$EncodeParm(ADJ_CHANNUM_T) -nowait
     P25 encode field sccb1_channum_t  $EncodeParm(SCCB1_CHANNUM_T_dec) -nowait
     P25 encode field sccb1_channum_r  $EncodeParm(SCCB1_CHANNUM_R_dec) -nowait
     P25 encode field sccb2_channum_t  $EncodeParm(SCCB2_CHANNUM_T_dec) -nowait
     P25 encode field sccb2_channum_r  $EncodeParm(SCCB2_CHANNUM_R_dec) -nowait
   }     

   set returnList \
   {
     # Restore P25 mode variables
     P25 decode samplerate  $State(d_sampleRate) -nowait
     P25 decode enable      $State(d_enable)     -nowait
     P25 decode run         $State(d_run)        -nowait
     P25 encode encrypt     $State(e_encryptState) $State(e_encryptState) -nowait
     
     P25 encode obtctrl            $State(obtctrl) -nowait
     P25 encode explicitmessages   $State(expmsg) -nowait
     P25 encode autowrapdata       $State(AutoWrap) -nowait
     P25 encode AdjStatusEnable    $State(AdjStatusEnable) -nowait

     # Restore variables used for both P25 and P25 trunking
     P25 encode field rfsubid    0x$EncodeParm(RFSUBID) -nowait
     P25 encode field siteid     0x$EncodeParm(SITEID) -nowait
     P25 encode field lra        0x$EncodeParm(LOCREGAREA) -nowait
     P25 encode field sclass     0x$EncodeParm(SERVCLASS) -nowait

     # Restore settings for P25 trunking variables
     P25 encode field WACNID      0x$EncodeParm(WACNID) -nowait
     P25 encode field WUID        0x$EncodeParm(WUID) -nowait
     P25 encode field WGID        0x$EncodeParm(WGID) -nowait
     P25 encode field ANNGRPADR   0x$EncodeParm(ANNGRPADR) -nowait
     P25 encode field T_PROT      $EncodeParm(T_PROT) -nowait
     P25 encode field T_ANET      $EncodeParm(T_ANET) -nowait
     P25 encode field T_LG        $EncodeParm(T_LG) -nowait
     P25 encode field T_GAV       $EncodeParm(T_GAV) -nowait
     P25 encode field T_RV        $EncodeParm(T_RV) -nowait
     P25 encode field tgtaddr     $EncodeParm(TARGETADDR_dec) -nowait
     P25 encode field srcid       $EncodeParm(SRCID_dec) -nowait
     P25 encode field srcaddr     $EncodeParm(SOURCEADDR_dec) -nowait
     P25 encode field ctime       $EncodeParm(CTIMER_dec) -nowait
     P25 encode field adj_mfid       0x$EncodeParm(ADJ_MFID) -nowait
     P25 encode field adj_rfssid     0x$EncodeParm(ADJ_RFSSID) -nowait
     P25 encode field adj_sysid      0x$EncodeParm(ADJ_SYSID) -nowait
     P25 encode field adj_siteid     0x$EncodeParm(ADJ_SITEID) -nowait
     P25 encode field adj_lra        0x$EncodeParm(ADJ_LRA) -nowait
     P25 encode field adj_sclass     0x$EncodeParm(ADJ_SCLASS) -nowait
     P25 encode field adj_c_flag       $EncodeParm(ADJ_CFLAG_dec) -nowait
     P25 encode field adj_f_flag       $EncodeParm(ADJ_FFLAG_dec) -nowait
     P25 encode field adj_v_flag       $EncodeParm(ADJ_VFLAG_dec) -nowait
     P25 encode field adj_a_flag       $EncodeParm(ADJ_AFLAG_dec) -nowait
     P25 encode field sccb1_sclass   0x$EncodeParm(SCCB1_SSCLASS) -nowait
     P25 encode field sccb2_sclass   0x$EncodeParm(SCCB2_SSCLASS) -nowait

     # Set up the LCO message saves
     # These selected LCO messages save all of the user enterable fields
     P25 encode lcomsg  2  0x$EncodeParm(P) \
                           0x$EncodeParm(SF) \
                           0x$EncodeParm(CHANID_A) \
                           0x$EncodeParm(CHANNUM_A) \
                           0x$EncodeParm(GRPADDR_A) \
                           0x$EncodeParm(CHANID_B) \
                           0x$EncodeParm(CHANNUM_B) \
                           0x$EncodeParm(GRPADDR_B) -nowait \
  
     P25 encode lcomsg  3  0x$EncodeParm(P) \
                           0x$EncodeParm(SF) \
                           0x$EncodeParm(LCO_MFID) \
                           0x$EncodeParm(EMG) \
                           0x$EncodeParm(SERVICEOP) \
                           0x$EncodeParm(TARGETADDR) \
                           0x$EncodeParm(SOURCEADDR) -nowait \
  
     P25 encode lcomsg  4  0x$EncodeParm(P) \
                           0x$EncodeParm(SF) \
                           0x0 \
                           0x$EncodeParm(EMG) \
                           0x$EncodeParm(SERVICEOP) \
                           0x$EncodeParm(GRPADDR) \
                           0x$EncodeParm(CHANID_T) \
                           0x$EncodeParm(CHANNUM_T) \
                           0x$EncodeParm(CHANID_R) \
                           0x$EncodeParm(CHANNUM_R) -nowait \
  
     P25 encode lcomsg  6  0x$EncodeParm(P) \
                           0x$EncodeParm(SF) \
                           0x0 \
                           0x$EncodeParm(EMG) \
                           0x$EncodeParm(SERVICEOP) \
                           0x0 \
                           0x$EncodeParm(CTIMER) \
                           0x$EncodeParm(TARGETADDR) -nowait \
        
     P25 encode lcomsg  17  0x$EncodeParm(P) \
                            0x$EncodeParm(SF) \
                            0x$EncodeParm(NETID) \
                            0x$EncodeParm(SYSID) \
                            0x$EncodeParm(SRCID) \
                            0x0 -nowait \
  
     P25 encode lcomsg  20  0x$EncodeParm(P) \
                            0x$EncodeParm(SF) \
                            0x$EncodeParm(STSUPDATE) \
                            0x$EncodeParm(TARGETADDR) \
                            0x$EncodeParm(SOURCEADDR) -nowait \
     
     P25 encode lcomsg  21  0x$EncodeParm(P) \
                            0x$EncodeParm(SF) \
                            0x$EncodeParm(MSG) \
                            0x$EncodeParm(TARGETADDR) \
                            0x$EncodeParm(SOURCEADDR) -nowait\

  
     P25 encode lcomsg  23  0x$EncodeParm(P) \
                            0x$EncodeParm(SF) \
                            0x$EncodeParm(EXTFUNCT_A) \
                            0x$EncodeParm(EXTFUNCT_B) \
                            0x$EncodeParm(TARGETADDR) -nowait \
  
     P25 encode lcomsg  24  0x$EncodeParm(P) \
                            0x$EncodeParm(SF) \
                            0x$EncodeParm(CHANID) \
                            0x$EncodeParm(BW) \
                            0x$EncodeParm(XMITOFF) \
                            0x$EncodeParm(CHANSPACE) \
                            0x$EncodeParm(BASEFREQ) -nowait \
  
     P25 encode lcomsg  32  0x$EncodeParm(P) \
                            0x$EncodeParm(SF) \
                            0x0 \
                            0x0 \
                            0x$EncodeParm(PRIREQ) \
                            0x$EncodeParm(SSAVAIL) \
                            0x$EncodeParm(SSSUPPORT) -nowait \
  
     P25 encode lcomsg  33  0x$EncodeParm(P) \
                            0x$EncodeParm(SF) \
                            0x$EncodeParm(RFSUBID) \
                            0x$EncodeParm(SITEID) \
                            0x$EncodeParm(CHANID_A) \
                            0x$EncodeParm(CHANNUM_A) \
                            0x$EncodeParm(SERVCLASS_A) \
                            0x$EncodeParm(CHANID_B) \
                            0x$EncodeParm(CHANNUM_B) \
                            0x$EncodeParm(SERVCLASS_B) -nowait \
  
     P25 encode lcomsg  34  0x$EncodeParm(P) \
                            0x$EncodeParm(SF) \
                            0x$EncodeParm(LOCREGAREA) \
                            0x0 \
                            0x$EncodeParm(SYSID) \
                            0x$EncodeParm(RFSUBID) \
                            0x$EncodeParm(SITEID) \
                            0x$EncodeParm(CHANID) \
                            0x$EncodeParm(CHANNUM) \
                            0x$EncodeParm(SERVCLASS) -nowait \
  
     P25 encode lcomsg  37  0x$EncodeParm(P) \
                            0x$EncodeParm(SF) \
                            0x0 \
                            0x$EncodeParm(LCO_ALGID) \
                            0x$EncodeParm(LCO_KID) \
                            0x$EncodeParm(TARGETADDR) -nowait

     P25 encode lcomsg  0  0x$EncodeParm(P) \
                           0x$EncodeParm(SF) \
                           0x$EncodeParm(LCO_MFID) \
                           0x$EncodeParm(EMG) \
                           0x$EncodeParm(SERVICEOP) \
                           0x0 \
                           0x$EncodeParm(GRPADDR) \
                           0x$EncodeParm(SOURCEADDR) -nowait \
  
     # Give them time to take effect
     update idle
 
     # Restore P25 header information 
     # using the temporary values saved above for P, SF, and LCO
     P25 encode sendarray 0x$EncodeParm(HDR_NAC) \
                          0x$EncodeParm(HDR_MFID) \
                          0x$EncodeParm(HDR_TGID) \
                          0x$EncodeParm(HDR_LSD1) \
                          0x$EncodeParm(HDR_LSD2) \
                          0x$temp_LCO \
                          0x$EncodeParm(HDR_DID) \
                          0x$EncodeParm(HDR_SID) \
                          0x$EncodeParm(EMG) \
                          0x$temp_SF \
                          0x$temp_P \
                          0x$EncodeParm(HDR_KID) \
                          0x$EncodeParm(HDR_SSYM) -nowait

     P25 encode field  ALGID  0x$EncodeParm(HDR_ALGID)  -nowait

     P25 encode digits  $EncodeParm(DIG0_9) -nowait

     # Give them time to take effect
     update idle

     # Restore the current LCO value
     # using the temporary value saved above for LCO
     set ::P25ns::EncodeParm(LCO)  $temp_LCO

     # Restore the selected logging options
     set ::P25ns::Log(lco_octets)  $Log(lco_octets)
     set ::P25ns::Log(lco_msg)     $Log(lco_msg)
     set ::P25ns::Log(osp_octets)  $Log(osp_octets)
     set ::P25ns::Log(osp_msg)     $Log(osp_msg)
     set ::P25ns::Log(isp_octets)  $Log(isp_octets)
     set ::P25ns::Log(isp_msg)     $Log(isp_msg)
     set ::P25ns::Log(raw)         $Log(raw)
     set ::P25ns::Log(NextVersion) $Log(NextVersion)
     if [IFR_options is_installed P25_CTRL_LOG]  {
        P25 log setversion $Log(NextVersion)
     }
     # Set the mask for the above values
     ::P25ns::setLogMask  "raw"
   
     # Restore the current LCO message
     ::P25ns::send_lco
   }

   append returnList "\n\n"
     
   foreach { band } { 0 1 2 } \
   {
      foreach { index } { 0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 } \
      {
         append returnList "P25 encode chanaddr [P25 encode chanaddr $band $index] \n"
      }
   }

   append returnList [subst " array set ::P25ns::TrunkModeIndex {[array get TrunkModeIndex]} \n" ]
   append returnList [subst " array set ::P25ns::TrunkChanArray {[array get TrunkChanArray]} \n"]
   append returnList "::P25ns::setTrunkModeIndex $::P25ns::TrunkModeCurrent"

   append returnList "\n"
   append returnList $channelList
   append returnList "\n"
   
   if {$::P25ns::State(trunk_on_off) == "ON"} \
   {
      append returnList {SavedStates::Finally "::P25ns::EnableTrunking Enable"}
   }
   return [subst -nocommands $returnList]
 }


#****************************************************************************
# Name:        SyncFreq
# Parameters:  none
# Description: Write state variables to hardware to update channel frequencies
#****************************************************************************
 proc SyncFreq { } {
     variable EncodeParm

     # Bypass set of values if we are loading a setup - but query data
     if { ![info exists ::Recall_SetupsUI::State(recall_in_process)] } \
     {
        P25 encode field channumrx_c
        return
     } elseif { $::Recall_SetupsUI::State(recall_in_process) == 1 } \
     {
        P25 encode field channumrx_c
        return
     }

     P25 encode field CHANNUM_C   0x$EncodeParm(CHANNUM_C) -nowait
     P25 encode field CHANNUM_V   0x$EncodeParm(CHANNUM_V) -nowait
     P25 encode field channumtx_c $EncodeParm(CHANNUMTX_C_dec) -nowait
     P25 encode field channumrx_c $EncodeParm(CHANNUMRX_C_dec) -nowait
     P25 encode field channumtx_v $EncodeParm(CHANNUMTX_V_dec) -nowait
     P25 encode field channumrx_v $EncodeParm(CHANNUMRX_V_dec) -nowait
     P25 encode field adj_channum_r  0x$EncodeParm(ADJ_CHANNUM_R) -nowait
     P25 encode field adj_channum_t  0x$EncodeParm(ADJ_CHANNUM_T) -nowait
     P25 encode field sccb1_channum_t  $EncodeParm(SCCB1_CHANNUM_T_dec) -nowait
     P25 encode field sccb1_channum_r  $EncodeParm(SCCB1_CHANNUM_R_dec) -nowait
     P25 encode field sccb2_channum_t  $EncodeParm(SCCB2_CHANNUM_T_dec) -nowait
     P25 encode field sccb2_channum_r  $EncodeParm(SCCB2_CHANNUM_R_dec) -nowait
 }

#****************************************************************************
# Name:        Sync
# Parameters:  none
# Description: Write state variables to hardware
#****************************************************************************
 proc Sync { } {
    eval [getSaveState]
 }

#****************************************************************************
# Name:        SyncAll
# Parameters:  none
# Description: Insures that TCL, the engine and hardware are in sync
#****************************************************************************
 proc SyncAll { } {

    set_p25_callbacks

    if ![interp issafe] \
    {
       after 100 {::P25ns::Sync}
    }
 }

#****************************************************************************
# Name:        SetTrunkingDefault
# Parameters:  none
# Description: Set the P25 trunking to default values
#****************************************************************************
 proc SetTrunkingDefault { } \
 {
     variable DefaultTrunkEncode
     variable TrunkModeCurrent
     variable trunk_mode_type
     variable TrunkModeIndex
     variable EncodeParm
     variable Log

     P25 encode field mfid           0x$DefaultTrunkEncode(MFID) -nowait
     P25 encode field nac            0x$DefaultTrunkEncode(NAC) -nowait
     P25 encode field sysid          0x$DefaultTrunkEncode(SYSID) -nowait
     P25 encode field rfsubid        0x$DefaultTrunkEncode(RFSUBID) -nowait
     P25 encode field siteid         0x$DefaultTrunkEncode(SITEID) -nowait
     P25 encode field lra            0x$DefaultTrunkEncode(LRA) -nowait
     P25 encode field sclass         0x$DefaultTrunkEncode(SCLASS) -nowait
     P25 encode field servop         0x$DefaultTrunkEncode(SERVOP) -nowait
     P25 encode field WACNID         0x$DefaultTrunkEncode(WACNID) -nowait
     P25 encode field WUID           0x$DefaultTrunkEncode(WUID) -nowait
     P25 encode field WGID           0x$DefaultTrunkEncode(WGID) -nowait
     P25 encode field CHANID_C       0x$DefaultTrunkEncode(CHANID_C) -nowait
     P25 encode field CHANNUM_C      0x$DefaultTrunkEncode(CHANNUM_C) -nowait
     P25 encode field CHANID_V       0x$DefaultTrunkEncode(CHANID_V) -nowait
     P25 encode field CHANNUM_V      0x$DefaultTrunkEncode(CHANNUM_V) -nowait
     P25 encode field ANNGRPADR      0x$DefaultTrunkEncode(ANNGRPADR) -nowait
     P25 encode field T_PROT           $DefaultTrunkEncode(T_PROT) -nowait
     P25 encode field T_ANET           $DefaultTrunkEncode(T_ANET) -nowait
     P25 encode field T_LG             $DefaultTrunkEncode(T_LG) -nowait
     P25 encode field T_GAV            $DefaultTrunkEncode(T_GAV) -nowait
     P25 encode field T_RV             $DefaultTrunkEncode(T_RV) -nowait
     P25 encode field ctime          0x$DefaultTrunkEncode(CTIMER) -nowait
     P25 encode field srcid          0x$DefaultTrunkEncode(SRCID) -nowait
     P25 encode field tgtaddr        0x$DefaultTrunkEncode(TARGETADDR) -nowait
     P25 encode field srcaddr        0x$DefaultTrunkEncode(SOURCEADDR) -nowait
     P25 encode field adj_mfid       0x$DefaultTrunkEncode(ADJ_MFID) -nowait
     P25 encode field adj_rfssid     0x$DefaultTrunkEncode(ADJ_RFSSID) -nowait
     P25 encode field adj_sysid      0x$DefaultTrunkEncode(ADJ_SYSID) -nowait
     P25 encode field adj_siteid     0x$DefaultTrunkEncode(ADJ_SITEID) -nowait
     P25 encode field adj_lra        0x$DefaultTrunkEncode(ADJ_LRA) -nowait
     P25 encode field adj_sclass     0x$DefaultTrunkEncode(ADJ_SCLASS) -nowait
     P25 encode field adj_c_flag       $DefaultTrunkEncode(ADJ_CFLAG_dec) -nowait
     P25 encode field adj_f_flag       $DefaultTrunkEncode(ADJ_FFLAG_dec) -nowait
     P25 encode field adj_v_flag       $DefaultTrunkEncode(ADJ_VFLAG_dec) -nowait
     P25 encode field adj_a_flag       $DefaultTrunkEncode(ADJ_AFLAG_dec) -nowait
     P25 encode field adj_chanid_r   0x$DefaultTrunkEncode(ADJ_CHANID_R) -nowait
     P25 encode field adj_channum_r  0x$DefaultTrunkEncode(ADJ_CHANNUM_R) -nowait
     P25 encode field adj_chanid_t   0x$DefaultTrunkEncode(ADJ_CHANID_T) -nowait
     P25 encode field adj_channum_t  0x$DefaultTrunkEncode(ADJ_CHANNUM_T) -nowait
     P25 encode obtctrl                $DefaultTrunkEncode(obtctrl) -nowait
     P25 encode explicitmessages       $DefaultTrunkEncode(expmsg) -nowait
     P25 encode autowrapdata           999999 -forgive
     P25 encode AdjStatusEnable        $DefaultTrunkEncode(AdjStatusEnable) -nowait
     P25 encode field sccb1_sclass   0 -nowait
     P25 encode field sccb2_sclass   0 -nowait

     set EncodeParm(DIG0_9) $DefaultTrunkEncode(DIG0_9)
     P25ns::sendTenDigits

     P25 encode lcomsg  0  0x$EncodeParm(P)            \
                           0x$EncodeParm(SF)           \
                           0x$DefaultTrunkEncode(MFID) \
                           0x$EncodeParm(EMG)          \
                           0x$DefaultTrunkEncode(SERVOP)    \
                           0x0                         \
                           0x$DefaultTrunkEncode(WGID) \
                           0x$DefaultTrunkEncode(SOURCEADDR) -nowait

     foreach { band } { 0 1 2 } \
     {
        foreach { index } { 0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 } \
        {
           set templist [P25 encode chanaddr $band $index]
           P25 encode chanaddr $band $index [lindex $templist 2] \
                  [lindex $templist 3] 6.25 [lindex $templist 5] -nowait
        }
     }
     P25 encode chanaddr $trunk_mode_type($TrunkModeCurrent) $TrunkModeIndex($TrunkModeCurrent) -nowait

 }

#****************************************************************************
# Name:        EnableTrunking
# Parameters:  Desired trunking state
# Description: Enable / Disable P25 trunking simulation
#****************************************************************************
 proc EnableTrunking { state } \
 {
    variable EncodeParm
    variable State
    set State(trunk_state,raw) $state
    if { $state == "Enable" } \
    {
       ::Generator::Trunking_mode "::P25ns::EnableTrunking Disable"
       P25 encode lcomsg  0  0x$EncodeParm(P)         \
                             0x$EncodeParm(SF)        \
                             0x$EncodeParm(HDR_MFID)  \
                             0x$EncodeParm(EMG)       \
                             0x$EncodeParm(SERVICEOP) \
                             0x0                      \
                             0x$EncodeParm(WGID)      \
                             0x$EncodeParm(SOURCEADDR) -nowait

       P25 encode trunksim 1 -nowait
       P25 decode enable 3 -nowait
       P25 decode run 1 -nowait
       P25 encode trunkpttenable  0 -nowait

       Generator ptt 1 -nowait
       Generator mod p25mis -nowait
       if { $::Generator::State(p25mode) == "OFF" } \
       {
           Generator p25mode file -nowait
       }

       Receiver demod P25 -nowait
       # for reasons currently unknown, the previous command does not seem
       # to set everything up the FIRST time after a power cycle which does
       # not have a reload file (ex. after a hard shut down after a factory 
       # default. Therefore wait a second and hit the receiver again.
       after 1000 "Receiver demod P25 -nowait"

       set State(trunk_state) ON
       set State(trunk_state_short) ON

    } elseif { $state == "Disable" } \
    {
       Generator ptt 0 -nowait
       P25 encode trunksim 0 -nowait
       P25 encode trunkpttenable  0 -nowait

       # Clear the indicator values
       set ::P25ns::State(trunk_ptt_rcv)  0
       set ::P25ns::State(trunk_reg_rcv)  0
       set ::P25ns::State(trunk_aff_rcv)  0
 
       set State(trunk_state) OFF
       set State(trunk_state_short) OFF
   } 
 }

 #**************************************************************************
 #
 #  Name: readLoggerCtrlData
 #
 #  Parameters: none
 #
 # Description: This reads from the outbound control pipe of the P25 logger.
 #    Currently it is only used to obtain the next logger version to preserve
 #    this information over a power cycle
 #
 #**************************************************************************
 proc readLoggerCtrlData { } \
 {
     variable  Log
     variable  P25_log_ctrl_fd

     gets $Log(P25_log_ctrldata_fd) line
#     puts "Version line = $line"
     set index [string first "=" $line]
     set version [string range $line [expr $index + 1] end]
     set Log(NextVersion) $version
 }

#****************************************************************************
# Name:        Send_header
# Parameters:  none
# Description: Send the P25 header
#****************************************************************************
 proc Send_header { } \
 {
    variable EncodeParm

    # This is a BODGE. There is no DSP command to force a header to be sent,
    # but when we send an ALGID command down, it forces an update of the
    # header information which gets sent out.
    P25 encode field ALGID 0x$EncodeParm(HDR_ALGID) -nowait
 }
#****************************************************************************
# Initialize the namespace and activate the callbacks
#****************************************************************************

    variable Log

    Load::on_system_done  { ::P25ns::SyncAll }
    if ![interp issafe] \
    {
        if [IFR_options is_installed P25_CTRL_LOG] \
        {
           CreateLog "/P25_log" "" 1000
        
           # Initialize the logger
           P25 log maxtime 120
           P25 log maxsize 2.0
           P25 log limit size
           P25 log maxversion 99999
           P25 log versiontype increment
           P25 log setversion $Log(NextVersion)

           #puts "Setting up file event for P25 logger"
           if { $Log(P25_log_ctrldata_fd) == -1 } \
           {
              set Log(P25_log_ctrldata_fd) [open "/P25_log_ctrlOut" RDONLY]
              fileevent $Log(P25_log_ctrldata_fd) readable \
              { 
                 ::P25ns::readLoggerCtrlData 
              }
           }
        }
    }


}  ;# End of the P25 namespace

#****************************************************************************
# This registers the engine with the list for saving instrument states
#****************************************************************************
namespace eval ::SavedStates {

   variable engines   
   lappend engines P25ns
}


#****************************************************************************
# This registers the engine with the list for the version screen
#****************************************************************************
namespace eval ::Version_Screen \
{
   variable version
   variable names
   
   array set names \
   {
     P25t_Eng "P25 Encode Engine"
     P25t_Drv "P25 Encode driver"
   }
   #set version(P25r_Eng)  [P25 decode ver 0]
   #set version(P25r_Drv)  [P25 decode ver 0]
   set version(P25t_Eng)  [P25 encode ver 0]
   set version(P25t_Drv)  [P25 encode ver 1]
}

#****************************************************************************
# Define the P25_SCCB namespace, and set up our HTML option info
#****************************************************************************
namespace eval ::P25_SCCB \
{
  variable InfoString

  set P25_version [string trimleft [P25 encode ver 0] "P25Mtr"]
  set InfoString [subst \
  {
         <br>
         (using P25 SCCB version - $P25_version)
         <br>
         This option adds the broadcast of Secondary Control Channel messages
         to an existing P25 Repeater simulation.
  }]
}

#****************************************************************************
# Define the P25_Explicit namespace, and set up our HTML option info
#****************************************************************************
namespace eval ::P25_Explicit \
{
  variable InfoString

  set P25_version [string trimleft [P25 encode ver 0] "P25Mtr"]
  set InfoString [subst \
  {
         <br>
         (using P25 Explicit version - $P25_version)
         <br>
         This option allows the P25 Repeater simulation the use Explicit message
         packets.
  }]
}

#****************************************************************************
# Define the P25_UU_PSTN namespace, and set up our HTML option info
#****************************************************************************
namespace eval ::P25_UU_PSTN \
{
  variable InfoString

  set P25_version [string trimleft [P25 encode ver 0] "P25Mtr"]
  set InfoString [subst \
  {
         <br>
         (using P25 UU/PSTN version - $P25_version)
         <br>
         This option allows the P25 Repeater to generate and respond to 
         Unit to Unit and PSTN calls (explicit mode only).
  }]
}

#****************************************************************************
# Define the P25_Adj namespace, and set up our HTML option info
#****************************************************************************
namespace eval ::P25_Adj \
{
  variable InfoString

  set P25_version [string trimleft [P25 encode ver 0] "P25Mtr"]
  set InfoString [subst \
  {
         <br>
         (using P25 Adjacent Channel version - $P25_version)
         <br>
         This option allows the P25 Repeater simulation to transmit message 
         packets which notify radios of an Adjacent Control Channel (explicit mode only).
  }]
}

#****************************************************************************
# Define the P25_CTRL_LOG namespace, and set up our HTML option info
#****************************************************************************
namespace eval ::P25_CTRL_LOG \
{
  variable InfoString

  set P25_version [string trimleft [P25 encode ver 0] "P25Mtr"]
  set InfoString [subst \
  {
         <br>
         (using P25 Data Logger version - $P25_version)
         <br>
         This option allows the user to monitor an inbound or outbound P25 
         control or traffic channel, and log the information to a file.
  }]
}

#****************************************************************************
# Define the P25_LSM namespace, and set up our HTML option info
#****************************************************************************
namespace eval ::P25_LSM \
{
  variable InfoString

  set P25_version [string trimleft [P25 encode ver 0] "P25Mtr"]
  set InfoString [subst \
  {
         <br>
         (using P25 LSM version - $P25_version)
         <br>
         This option enables QPSK modulation, demodulation and measurement capabilities.
  }]
}

#****************************************************************************
# Define the P25_OBT namespace, and set up our HTML option info
#****************************************************************************
namespace eval ::P25_OBT \
{
  variable InfoString

  set P25_version [string trimleft [P25 encode ver 0] "P25Mtr"]
  set InfoString [subst \
  {
         <br>
         (using P25 Other Band Trunking version - $P25_version)
         <br>
         This option provides a basic P25 Repeater simulation, allowing it to
         communicate in the UHF/VHF and 700 MHz frequency bands.
  }]
}

#****************************************************************************
# Define the P25_trunk namespace, and set up our HTML option info
#****************************************************************************
namespace eval ::P25_trunk \
{
  variable InfoString

  set P25_version [string trimleft [P25 encode ver 0] "P25Mtr"]
  set InfoString [subst \
  {
         <br>
         (using P25 Trunking version - $P25_version)
         <br>
         This option provides a basic P25 Repeater simulation, allowing it to
         communicate in the 800 MHz frequency band.
  }]
}

#****************************************************************************
# Define the P25_trunk namespace, and set up our HTML option info
#****************************************************************************
namespace eval ::P25_WIDEPULSE \
{
  variable InfoString

  set P25_version [string trimleft [P25 encode ver 0] "P25Mtr"]
  set InfoString [subst \
  {
         <br>
         (using Widepulse version - $P25_version)
         <br>
         Wide Pulse Modulation for Motorola Smartnet/Smartzone systems using P25 Wide Pulse Simulcast Systems.
  }]
}

#****************************************************************************
# Define the P25_trunk namespace, and set up our HTML option info
#****************************************************************************
namespace eval ::LTD \
{
  variable InfoString

  set P25_version [string trimleft [P25 encode ver 0] "P25Mtr"]
  set InfoString [subst \
  {
         <br>
         (using LTD version - $P25_version)
         <br>
         Motorola Test Mode analysis suite for Motorola's next 
generation digital two-way professional radio platform.
  }]
}

