#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C) 2001      DESIGN ENGINEERING        #
# --------------------------------------------------------------------------#
# FILE        : parse_list.tcl                                              #
# PRODUCT     : FITS                                                        #
# AUTHOR      : David Hagood                                                #
# DESCRIPTION : This routine allows for traversing an arbitrarily complex   #
#               list (of lists (of lists)...                                #
#############################################################################

package provide Parse_list 1.0

#****************************************************************************
# Name:        Parse_list
# Parameters:  command to parse single "thing", zero or more lists
# Description: Given an arbitrarily complex list of (lists of lists..),
#              iterate over it until we find a simple string item, and then
#              call a routine to parse it.
#****************************************************************************
proc Parse_list { cmd list } {
  # if list length is 1, then expand it and try again
  if {[llength $list] == 1 } {
     set list [lindex $list 0]
  }
  
  # if the length of the first element of args is 1, then we've worked our way down
  # to the fundemental unit of data 
  if {[llength $list] == 1} {
     # OK, this SHOULD be the fundemental quantum of data
     return [eval $cmd $list]
  }
  # OK, this is still a list of lists... bust it out once more
  set seq {}
  foreach part $list {
     lappend seq [Parse_list $cmd $part]
  }
  return $seq
}
  
#****************************************************************************
# Name:        Parse_list_of_lists
# Parameters:  command to parse single "thing", zero or more lists
# Description: Given an arbitrarily complex list of (lists of lists..),
#              iterate over it until we find a simple string item, and then
#              call a routine to parse it.
#****************************************************************************
proc Parse_list_of_lists { cmd list } {
  # if list is length 1, then expand it and try again
  if {[llength $list] == 1 } {
     set list [lindex $list 0]
     if {[llength $list] == 1 } {
        return -code error "Expecting a list, not $list"
     }
  }
  
  set x [lindex $list 0]
  
  if {[llength $x] == 1} {
   # OK, this SHOULD be the fundemental quantum of data
    return [eval $cmd $list]
  }
  # OK, this is still a list of lists... bust it out once more
  set seq {}
  foreach part $list {
     lappend seq [Parse_list_of_lists $cmd $part]
  }
  return $seq
}
  
