namespace eval ::Power \
{
#****************************************************************************
# Name:        updateInfo
# Parameters:  state
# Description: handle info messages
#****************************************************************************
 proc updateInfo { v } \
 {
   variable State
   foreach { serial_major serial_minor version day month year hour minute } $v \
   {
      set State(serial_major) $serial_major
      set State(serial_minor) $serial_minor
      set State(version) $version
      set State(day) $day
      set State(month) $month
      set State(year) $year
      set State(hour) $hour
      set State(minute) $minute
   }
 }
 
#****************************************************************************
# Name:        updateCals
# Parameters:  state
# Description: handle diagnostic messages
#****************************************************************************
 proc updateCals { arg } \
 {
   # BODGE this should be handled in this file
    eval ::powerterm::receive_pwt_cal $arg
 }

   Sync
   Power enable on -nowait
   set State(overload,reset) 2000
   set State(overload,once) 0

   Power -onchange debug info        {::Power::updateInfo %V}
   Power -onchange debug calday      {set ::Power::State(day) %V}
   Power -onchange debug calmonth    {set ::Power::State(month) %V}
   Power -onchange debug calyear     {set ::Power::State(year) %V}
   Power -onchange debug calhour     {set ::Power::State(hour) %V}
   Power -onchange debug calmin      {set ::Power::State(minute) %V}
   Power -onchange debug snmajor     {set ::Power::State(serial_major) %V}
   Power -onchange debug snminor     {set ::Power::State(serial_minor) %V}
       
   set State(cal,handler) [Power debug -structure -queue cals {::Power::updateCals %V}]
 
   # BODGE this really should be handled by the power term engine....
   # if no disk file, then write data to disk file
   if ![file exists /system/calibration/ptcals ] \
   {
          Power debug diskcal 0 -nowait
   }

   proc hook_poweralerts { top } \
   {
      trace variable ::Power::State(temp_overload) w "::Power::handle_overtemp $top"
      trace variable ::Power::State(overload) w "::Power::handle_pwt_ovld $top"
   }

   proc unhook_poweralerts { top } \
   {
      trace vdelete ::Power::State(temp_overload) w "::Power::handle_overtemp $top"
      trace vdelete ::Power::State(overload) w "::Power::handle_pwt_ovld $top"
   }
   
   # register with screens to generate overload messages as needed
   View::Register_alert Power::hook_poweralerts
   View::Register_cleanup Power::unhook_poweralerts

   proc handle_overtemp { top args } \
   {
      set cmd "::Power::handle_overtemp_core $top"
      after cancel $cmd
      after idle $cmd
   }
   proc handle_overtemp_core { top } \
   {
      variable State
      set dialog [name_child $top pwtoverheat]
      
      if {!$State(temp_overload)} \
      {
         destroy $dialog
         return
      }
      if ![winfo exists $dialog] \
      {
          message_box $dialog WARNING
      }
   }
   proc handle_pwt_ovld { top args } \
   {
      set cmd "::Power::handle_pwt_ovld_core $top"
      after cancel $cmd
      after idle $cmd
   }
   
   bind PWT_RESET <Destroy> "Power debug olreset"
   
   proc handle_pwt_ovld_core { top } \
   {
     variable State
     set ovld $::Power::State(overload)
     set dialog [name_child $top pwtoverload]

     # DISPLAY OVERLOAD MESSAGE IF NEEDED

     if { !$ovld} \
     {
       destroy $dialog
       return
     }
    
     if {![winfo exists $dialog]} \
     {
        # create the message box
        message_box $dialog WARNING
     }

     # set up warning messages as needed
     switch $ovld \
     {
       1 {set which antText}
       2 {set which genText}
       3 {set which bothText}
     }
     $dialog.text config -text [subst [option get $dialog $which {}]]
   }
   
   proc updateOverload { overload } \
   {
      variable State
      
      set State(overload) $overload
      set cmd "set ::Power::State(overload,once) 0"
      if {$overload} \
      {
         # error log any overloads
         ::ErrLog::Log "Power term overload detected: state $overload"

         if {$State(overload,once) == 0} \
         {
           Generator ptt off -nowait
           after $State(overload,reset) "Power debug olreset"
         }
         set $State(overload,once) 1
         after cancel $cmd
      } \
      else \
      {
         if {$State(overload,once) == 1} \
         {
            set State(overload,once) 2
            after $State(overload,reset) $cmd
         }
      }
   }
   
   proc log_overtemp { overtemp } \
   {
      variable State
      if {$overtemp} \
      {
         ::ErrLog::Log "Power term overtemp detected: temps are $State(temp1,raw) $State(temp2,raw)"
      }
   }
   Power -onchange overtemp "::Power::log_overtemp %V"
   
   after idle {
       Power debug eeprom -nowait
       Power debug readcal  0 -nowait
       Power debug readcal  1 -nowait
       Power debug readcal  2 -nowait
       Power debug getcalinfo -nowait
       Load::on_system_done { after 6000 {Power zero -nowait}}
   }
}
