#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT © 2002        DESIGN ENGINEERING        #
# --------------------------------------------------------------------------#
# FILE        : receiver.tcl                                                #
# PRODUCT     : FITS                                                        #
# AUTHOR      : Bruce Butterfield                                           #
# DESCRIPTION : This file is sourced by TCL when the Receiver command is    #
#             : first used.  It loads the C++ interface file, provides a    #
#             : namespace for screen variables, and sets up callbacks to    #
#             : update the variables for the UI.                            #
#                                                                           #
#############################################################################
trace vdelete ::Receiver::State ruw "::Receiver::Load"

#****************************************************************************
# Handle a load of the C++ interface
#****************************************************************************
if {"[info command Receiver]" != ""} { return }

# Call the base class dummy command to force a load
package require Engtclbase
Engtclbase

package require Analyzer

# Load the object file
::load $::Receiver::object

package provide Receiver 1.0

#****************************************************************************
# Receiver namespace variables
#****************************************************************************
#    Status of commands sent to AFCounter
#       ::Receiver::State(freq_raw)             Mhz
#       ::Receiver::State(rfinput_raw)          0 | 1   {0 = T/R, 1 = ANT}
#       ::Receiver::State(rfinput)              "T/R" | "ANT"
#       ::Receiver::State(atten)                attenuation value
#       ::Receiver::State(max_atten)            max value for current input port
#       ::Receiver::State(ifbw)                 30 kHz | 300 kHz | 3 Mhz
#       ::Receiver::State(demod_raw)            {0-5, 10,11}
#       ::Receiver::State(demod)                See demod_map_decode
#       ::Receiver::State(range_raw)            0 = Wide, 1 = Narrow
#       ::Receiver::State(range)                {WIDE | NARROW}
#       ::Receiver::State(hwagc_raw)            0 = Fast, 1 = Slow
#       ::Receiver::State(hwagc)                {FAST | SLOW}
#       ::Receiver::State(agcmode)              {0 | 1}
#****************************************************************************


namespace eval ::Receiver \
{
   variable SaveSortOrder AFTER(Freq_ListUI)
   
   array set demod_map_set \
   {
      DEMOD_FM_WIDE    0 
      DEMOD_FM_NARROW  1
      DEMOD_FM_WB      2
      DEMOD_FM_TV      3
      DEMOD_AM         4
      DEMOD_P25        5
      DEMOD_FM_MEDIUM  10
      DEMOD_OFF        11
      DEMOD_P25_LSM    12
      DEMOD_P25_P2     13
      DEMOD_P25_WIDE   14
      DEMOD_AM_12      15
      DEMOD_AM_25      16
      DEMOD_AM_60      17
      DEMOD_AM_200     18
      DEMOD_LTD        19
      DEMOD_LTD_WIDE   20
      DEMOD_P25_P2_WIDE 21
      DEMOD_P25_LSM_WIDE 22
      DEMOD_P25_WIDEPULSE_NARROW 23
      DEMOD_P25_WIDEPULSE_WIDE 24
   }

   array set demod_map_decode \
   { 
      0  DEMOD_FM_WIDE
      1  DEMOD_FM_NARROW
      2  DEMOD_FM_WB 
      3  DEMOD_FM_TV
      4  DEMOD_AM
      5  DEMOD_P25 
      10 DEMOD_FM_MEDIUM
      11 DEMOD_OFF
      12 DEMOD_P25_LSM
      13 DEMOD_P25_P2
      14 DEMOD_P25_WIDE
      15 DEMOD_AM_12
      16 DEMOD_AM_25
      17 DEMOD_AM_60
      18 DEMOD_AM_200
      19 DEMOD_LTD
      20 DEMOD_LTD_WIDE
      21 DEMOD_P25_P2_WIDE
      22 DEMOD_P25_LSM_WIDE
      23 DEMOD_P25_WIDEPULSE_NARROW
      24 DEMOD_P25_WIDEPULSE_WIDE
   }
   variable demod_map_names
   array set demod_map_names \
   {
      0  {FM NARROW 12.5kHz "20kHz LP"}
      1  {FM NARROW 12.5kHz "3kHz LP"}
      2  {FM WIDE   200kHz "20kHz LP"}
      3  {FM "60 kHz" 60kHz "20kHz LP"}
      4  {AM NARROW 6kHz "3kHz LP"}
      5  {P25 NARROW 12.5kHz "4kHz LP"}
      10 {FM MEDIUM 25kHz "20kHz LP"}
      11 {OFF OFF OFF OFF} 
      12 {"P25 LSM" NARROW 12.5kHz "4kHz LP"} 
      13 {"P25 P2" NARROW 6.25kHz "4kHz LP"} 
      14 {"P25" NARROW 25kHz "4kHz LP"} 
      15 {AM MNARROW 12.5kHz "20kHz LP"}
      16 {AM MEDIUM  25kHz "20kHz LP"}
      17 {AM "60 kHz" 60kHz "20kHz LP"}
      18 {AM WIDE 200kHz "200kHz LP"}
      19 {LTD NARROW 12.5kHz "4kHz LP"}
      20 {LTD WIDE   25kHz   "4kHz LP"}
      21 {"P25 P2" WIDE 12.5kHz "4kHz LP"}
      22 {"P25 LSM" WIDE 25kHz "4kHz LP"}
      23 {"P25 WP" NARROW 25kHz "4kHz LP"}
      24 {"P25 WP" WIDE 50kHz "4kHz LP"}
   }
   
   array set demod_if_map \
   { 
      0  30000
      1  30000
      2  300000
      3  300000
      4  30000
      5  30000
      10 30000
      11 300000
      12 30000
      13 30000
      14 30000
      15 30000
      16 30000
      17 300000
      18 300000
      19 30000
      20 30000
      21 30000
      22 30000
      23 30000
      24 60000
   }
   
   array set State \
   {
      agc,samples 300
      agc,low     -10.5
      agc,hi      -9.5
      agc,filter  4
      if_gain 0
   }
   
   variable iir_filters
   set iir_filters \
   { 
     "NO FILTER" 
     "300 Hz HP"
     "300 Hz LP"
     "4 kHz BESSEL LP"
     "15 kHz LP"
     "20 kHz LP"
     "4 kHz HP"
     "300 Hz - 4 kHz BP"
     "4 kHz BUTTERWORTH LP"
     "80 Hz NARROW BP"
     "3 kHz NARROW BP"
     "APCO-25"
   }
   variable iir_filters_short
   set iir_filters_short \
   { 
     "NO FILTER" 
     "300 Hz HP"
     "300 Hz LP"
     "4 kHz LP"
     "15 kHz LP"
     "20 kHz LP"
     "4 kHz HP"
     "BANDPASS"
     "BUTTERWORTH"
     "80 Hz NBP"
     "3 kHz NBP"
     "APCO-25"
   }
   

#****************************************************************************
# Name:        set_receiver_callbacks
# Parameters:  none
# Description: Sets up the callbacks to update the AF Counter data
#****************************************************************************
 proc set_receiver_callbacks { } {

   if { $::Receiver::callbackStatus != 0 } {
       return
   }

   Receiver -onchange freq    { ::Receiver::update_freq %V }
   Receiver -onchange atten   { ::Receiver::updateAtten %V }
   Receiver -onchange ifbw    { set ::Receiver::State(ifbw) %V }
   Receiver -onchange filter  { ::Receiver::updateIIRfilter %V }
   Receiver -onchange range   { ::Receiver::updateRange %V }
   Receiver -onchange hwagc   { ::Receiver::updateHwAgc %V }
   Receiver -onchange agcmode { ::Receiver::updateAgcMode %V }
   Receiver -onchange squelchlevel  { ::Receiver::updateSquelch %V }
   Receiver -onchange squelchmode   { set ::Receiver::State(squelchmode) %V}
   Receiver -onchange squelchstatus { set ::Receiver::State(squelchstate) %V}
   Receiver -onchange maxatten      { set ::Receiver::State(maxatten) %V }
   Receiver -onchange lockstat { ::Receiver::updateLockStatus %V }
   Receiver -onchange lock     { set ::Receiver::State(lock,mode) %V }
   Receiver -onchange yigstat  { ::Receiver::updateYigStatus  %V }
   Receiver -onchange agc      { ::Receiver::updateAgc %V }
   Receiver -onchange ifgain   { ::Receiver::updateIFGain %V }

   Receiver -onchange find { ::Receiver::updateFindFreq %V }

   set ::Receiver::callbackStatus 1
   set ::Receiver::State(knob_squelch) 0
   
   # Only allow automatic command processing if master
   if ![interp issafe] \
   {
      Receiver -onchange rfinput { ::Receiver::updateInput %V }
      Receiver -onchange demod   { ::Receiver::updateDemod %V }
      Receiver squelchlevel
   }
 }
 proc update_freq { val } \
 {
    variable State
    set State(freq)    $val
    switch -- $State(fine_tune) \
    {
       "0"     { set State(freq_raw) [format %4.6f $val] }
       "3"     { set State(freq_raw) "<[format %4.4f $val]>" }
       "1"     { set State(freq_raw) "<Finding>" }
       default { set State(freq_raw) "<Fine Tune>" }
    }
 }
 
 proc updateSquelch { val } \
 {
    variable State
    set State(squelchlevel) $val
    after cancel ::Receiver::updateSquelch_core
    after 100    ::Receiver::updateSquelch_core
 }

 proc updateSquelch_core { } \
 {
    variable State
    set State(knob_squelch) $State(squelchlevel)
 }
 
#****************************************************************************
# Name:        updateIFGain
# Parameters:  Gain value
# Description: save the current IF gain
#****************************************************************************
 proc updateIFGain { gain } \
 {
    variable State
    set State(if_gain) $gain
 }

#****************************************************************************
# Name:        updateIIRfilter
# Parameters:  filter value
# Description: save the IIR post-detection filter
#****************************************************************************
 proc updateIIRfilter { filter } \
 {
    variable State
    variable iir_filters
    variable iir_filters_short
    set State(iirFilter) $filter
    set State(iirFilter,text) [lindex $iir_filters $filter]
    set State(iirFilter,text_short) [lindex $iir_filters_short $filter]
 }

#****************************************************************************
# Name:        setIIRfilter
# Parameters:  None
# Description: Set the IIR post-detection filter. The UI uses this function 
#              since it cannot avoid variable expansion at object construction
#****************************************************************************
 proc setIIRfilter { } \
 {
    variable State
    Receiver filter $State(iirFilter) -nowait
 }

#****************************************************************************
# Name:        updateYigStatus
# Parameters:  Yig status
# Description: Updates the YIG status
#****************************************************************************
 proc updateYigStatus { arg } \
 {
   variable State
   set State(yig,hold) [lindex $arg 0]
   set State(yig,dac)  [lindex $arg 1]
 }
 
#****************************************************************************
# Name:        updateLockStatus
# Parameters:  lockstat - engine lock status
# Description: Updates the lock status
#****************************************************************************
 proc is_locked { stat mask } \
 {
    if {$stat & $mask } \
    {
       return "UNLOCKED"
    }
    return "LOCKED"
 }
 
 proc updateLockStatus { lockstat } \
 {
   variable State
   
   set State(lockstat,raw) $lockstat
     
   set State(lock,main)     [is_locked $lockstat 1]
   set State(lock,clean)    [is_locked $lockstat 2]
   set State(lock,step)     [is_locked $lockstat 4]
   set State(lock,ref)      [is_locked $lockstat 8]

   set State(lock_unstable) [expr $lockstat & 0x8000]
   set State(lockstat)      [expr $lockstat & 0x7FFF]
   
   if [interp issafe] {return }
   
   # if we are unlocked and we've been unlocked for a while, start a cal
   if {$State(lock_unstable) == 0} \
   {
      # error log any unlocks
      if {$State(lockstat) & 8 } \
      {
         ::ErrLog::Log "Main reference unlocked: lock status $State(lockstat)"
      }
      if {$State(lockstat) != 0 } \
      {
         ::ErrLog::Log "Receiver unlock detected: lock status $State(lockstat)"
      }
      # If it is the YIG that is unlocked (and not the reference)
      # fire off a YIG cal
      if { ($State(lockstat) != 0) && (($State(lockstat) & 8) == 0)} \
      {
         if {$State(lock,cal_request) == 0} \
         {
            set State(lock,cal_request) 1
            ::HW_control::StartYigCal
         }
      } \
      else \
      {
         # locked now, so if we UNLOCK, we need to recal
         set State(lock,cal_request) 0
      }
   }
 }

#****************************************************************************
# Name:        updateInput
# Parameters:  input             0 = Off, 1 = On
# Description: Saves up the current hardware input selected on the Receiver
#****************************************************************************
 proc updateInput { input } {
   set ::Receiver::State(rfinput_raw) $input
   switch $input {
      0       { set ::Receiver::State(rfinput) "T/R" }
      1       { set ::Receiver::State(rfinput) "ANT" }
      default { puts "Receiver::updateInput arg err [$input]"; return }
   }

   # there may be a atten limit issue when switching ports
   Receiver atten $::Receiver::State(atten) -nowait
 }

#***************************************************************************
# Name:        toggle_input
# Parameters:  none
# Description: Switch the input for the Receiver
#***************************************************************************
 proc toggle_input { } \
 {
    if { $::Receiver::State(rfinput) == "ANT" } \
    {
        Receiver rfin "T/R" -nowait
    } else \
    {
        Receiver rfin "ANT" -nowait
    }
 }


#****************************************************************************
# Name:        updateAtten
# Parameters:  db             
# Description: Saves the current attenuation selected on the Receiver
#****************************************************************************
 proc updateAtten { db } {
 
   set ::Receiver::State(atten) $db
   
#   puts "Receiver::updateAtten $db $::Receiver::State(atten)"
 }

#****************************************************************************
# Name:        updateDemod
# Parameters:  demod             Demodulation state from engine
# Description: Saves the current demodulation selected on the Receiver
#****************************************************************************
 proc updateDemod { demod } {
   variable State
   variable demod_map_decode
   variable demod_map_names
    variable iir_filters
   variable demod_if_map
   
   set State(demod_raw) $demod
   if [info exists demod_map_decode($demod)] {
      set State(demod) $demod_map_decode($demod)
   }
   if [info exists demod_map_names($demod)] {
      set State(demod,text) [lindex $demod_map_names($demod) 0]
      set State(ifbw,text)  [lindex $demod_map_names($demod) 1]
      set State(ifbw,bw)    [lindex $demod_map_names($demod) 2]
      set State(post,text)  [lindex $demod_map_names($demod) 3]
   }
   # set the receiver hardware IF appropriately
   if [info exists demod_if_map($demod) ] \
   {
      Receiver ifbw $demod_if_map($demod) -nowait
   }
 }

#****************************************************************************
# Name:        setDemod
# Parameters:  demod             Demodulation state to configure the engine
# Description: Sets the current demodulation selected on the Receiver
#****************************************************************************
 proc setDemod { demod } {
   if [info exists ::Receiver::demod_map_set($demod)] {
      Receiver demod $::Receiver::demod_map_set($demod) -nowait
   }
 }

#****************************************************************************
# Name:        updateHwAgc
# Parameters:  input             0 = Fast, 1 = Slow
# Description: Saves up the current hardware agc selected on the Receiver
#****************************************************************************
 proc updateHwAgc { input } {
   set ::Receiver::State(hwagc_raw) $input
   switch $input {
      0       { set ::Receiver::State(hwagc) "FAST" }
      1       { set ::Receiver::State(hwagc) "SLOW" }
      default { return }
   }
 }

#****************************************************************************
# Name:        updateAgcMode
# Parameters:  input             0 = Fast, 1 = Slow
# Description: Currently the receiver engine does not validate the input to
#              be within a range since the driver forces all non-zero values
#              to be sent as 1.  Essentially this is a permanent forgive.
#****************************************************************************
 proc updateAgcMode { input } {
   switch -- $input {
      0       { set ::Receiver::State(agcmode) 0 }
      default { set ::Receiver::State(agcmode) 1 }
   }
 }


#****************************************************************************
# Name:        update_squelch_level
# Parameters:  direction        {up down}
# Description: Used by squelch knob to change the level where squelch occurs
#****************************************************************************
 proc update_squelch_level {direction} {
    variable State
    if {$direction == "up"} {
      incr State(knob_squelch) -36
    } else \
    {
      incr State(knob_squelch) 36
    }
    set cmd \
    {
       Receiver squelchmode auto -nowait
       Receiver squelchlevel $Receiver::State(knob_squelch) -nowait
    }
    after cancel $cmd
    after idle $cmd
 }


#****************************************************************************
# Name:        updateRange
# Parameters:  input             0 = Wide, 1 = Narrow
# Description: Saves up the current receiver range selected 
#****************************************************************************
 proc updateRange { input } {
   set ::Receiver::State(range_raw) $input
   switch $input {
      0       { set ::Receiver::State(range) "WIDE" }
      1       { set ::Receiver::State(range) "NARROW" }
      default { return }
   }
 }


#****************************************************************************
# Name:        Find
# Parameters:  none
# Description: Find a signal from DC to daylight
#****************************************************************************
  proc Find { } \
  {
      variable State
      if {$State(fine_tune) == 0} \
      {
         Debug::Puts FIND 3 "Start find from 0 MHz"
         # we need to make sure these are started before we return so we don't
         # queue up more than one
         set State(find_old_freq) $State(freq)
         set State(fine_tune) -1
         set State(freq_raw) "<Start>"
         Receiver freq 0
         after 10
         Receiver find 2800 -nowait
      }
  }

#****************************************************************************
# Name:        FindNext
# Parameters:  none
# Description: Find a signal from current freq to daylight
#****************************************************************************
  proc FindNext { } \
  {
      variable State
      if {$State(fine_tune) == 0}\
      {
         Debug::Puts FIND 3 "Start find from $State(freq_raw) MHz"
         set State(find_old_freq) $State(freq)
         set State(fine_tune) -1
         set State(freq_raw) "<Start>"
         # wait for the find to start to lock this out
         Receiver find 2800 -nowait
      }
  }
#****************************************************************************
# Name:        updateFindFreq
# Parameters:  
# Description:  
#****************************************************************************
 proc updateFindFreq { val } \
 {
    variable State

    set freq  [lindex $val 0]
    set state [lindex $val 1]
    set State(fine_tune) $state
    Debug::Puts FIND 3 "update find $val"
    switch -- $state \
    {
        0 \
        {
           # not running
        }
        1 \
        { 
           set State(freq_raw) "<Finding>"
           # just started
        }
        2 \
        {
           # find completed
           if ![interp issafe ] \
           {
              # Restore system hardware:
              Receiver agcmode $State(agcmode) -nowait
              Receiver ifgain $State(if_gain) -nowait
              if {$freq == 0} \
              {
                  Receiver ifbw $State(ifbw) -nowait
                  if [info exists State(find_old_freq)] \
                  {
                     Receiver freq $State(find_old_freq) -nowait
                  } \
                  else \
                  {
                     Receiver freq $State(freq) -nowait
                  }
                  set State(fine_tune) 0
              } \
              else \
              {
                  set State(fine_tune) 2
                  Receiver freq $freq -nowait
                  Fine_tune
              }
            }
        }
        3 \
        {
           # update on frequency being searched
           set State(freq_raw) "<[format %4.4f $freq]>"
        }
    }
 }
#****************************************************************************
# Name:        Fine_tune
# Parameters:  none
# Description: Using the RF error meter, fine-tune the receiver to the
#              received signal 
#****************************************************************************
 proc Fine_tune { {fine 0} } \
 {
    variable State
    set State(fine_tune) 2
    Debug::Puts FIND 3 "Fine tune $fine"
    if {$fine} \
    {
      set State(fine_tune) "FINE1"
    } \
    else \
    {
       set State(fine_tune) "BEGIN"
    }
    set State(fine_tune,avg) [RFError average]
    RFError average 1 -nowait
    trace vdelete  ::RFError::State(freqErrRaw) w ::Receiver::Fine_tune_core
    trace variable ::RFError::State(freqErrRaw) w ::Receiver::Fine_tune_core
    after 5000 "Receiver::Fine_tune_abort"
 }
 proc Fine_tune_core { args } \
 {
    set cmd "Receiver::Fine_tune_update"
    after cancel $cmd
    after idle $cmd
 }
 proc Fine_tune_abort { } \
 {
    variable State
    set State(fine_tune) "DONE"
    Debug::Puts Receiver 5 "Fine tune abort"
    Fine_tune_update
 }
 proc Fine_tune_update { } \
 {
    variable State
    set err [expr $::RFError::State(freqErrRaw)/1000000.0]
    Debug::Puts FIND 3 "Fine tune update $State(fine_tune)"
    switch -- $State(fine_tune) \
    {
      "BEGIN" \
      {
         Receiver freq -nowait
         # discard 5 readings as they might be bogus.
         set State(fine_tune,ifbw) $State(ifbw)
         Receiver ifbw 6000000 -nowait
         set State(fine_tune) "COARSE1"
         set State(fine_tune,discard) 5
      }
      "COARSE1" \
      {
         if {[incr State(fine_tune,discard) -1] <= 0} \
         {
            # counter valid? if not, bail
            if {($::RFError::State(status) & 0x101f) != 0 } \
            {
               Debug::Puts Receiver 5 "Coarse tune counter not good"
               set State(fine_tune) "DONE"
               after idle Receiver::Fine_tune_update
               return
            }
            set State(fine_tune) "FINE1"
            set new [Receiver freq [expr $State(freq) + $err] -forgive]
            Receiver ifbw $State(fine_tune,ifbw) -nowait
            Debug::Puts Receiver 5 "Coarse tune by $err to $new"
         }
      }
      "FINE1" \
      {
         Receiver freq -nowait
         set State(fine_tune,discard) 3
         set State(fine_tune) "FINE2"
      }
      "FINE2" \
      {
         if {[incr State(fine_tune,discard) -1] <= 0} \
         {
            set State(fine_tune) "DONE"
            # counter valid? if not, bail
            if {($::RFError::State(status) & 0x101f) != 0 } \
            {
               Debug::Puts Receiver 5 "Fine tune counter not good"
               after idle Receiver::Fine_tune_update
               return
            }
            set new [Receiver freq [expr $State(freq) + $err] -forgive]
            Debug::Puts Receiver 5 "Fine Tune by $err to $new"
            after idle Receiver::Fine_tune_update
         }
      }
      default -
      "DONE" \
      {
         after cancel "Receiver::Fine_tune_abort"
         trace vdelete ::RFError::State(freqErrRaw) w ::Receiver::Fine_tune_core
         RFError avg $State(fine_tune,avg) -nowait
         Receiver freq -nowait
         Debug::Puts Receiver 5 "Done"
         set State(fine_tune) 0
      }
    }
 }
 

#****************************************************************************
# Name:        Get_HTML
# Parameters:  base_url - the base URL for the containing page
# Description: Return an string containing HTML showing the current state
#              of this system
#****************************************************************************
 proc Get_HTML { {base_url ""} } \
 {
    variable State
    return [subst \
{<table align="left" border=1>
<tr><td nowrap colspan=2><center><h2>Receiver setup</h2></center></td></tr>
<tr><td nowrap>Frequency</td>         <td nowrap>$State(freq_raw) MHz</td></tr>
<tr><td nowrap>Input port</td>        <td nowrap>$State(rfinput)</td></tr>
<tr><td nowrap>Attenuation</td>       <td nowrap>$State(atten) dB</td></tr>
<tr><td nowrap>Demod format</td>      <td nowrap>$State(demod,text)</td></tr>
<tr><td nowrap>IF Bandwidth</td>      <td nowrap>$State(ifbw,bw)</td></tr>
<tr><td nowrap>Post-detect filter</td><td nowrap>$State(iirFilter,text)</td></tr>
</table>
}]
 }
 
#****************************************************************************
# Name:        getSaveState
# Parameters:  none
# Description: This function returns a list of TCL commands which can be used
#              to set the Receiver engine to its current state
#****************************************************************************
 proc getSaveState { } \
 {
   variable State

   set returnList \
   {
     Receiver freq         $State(freq)         -nowait
     Receiver rfinput      $State(rfinput_raw)  -nowait
     Receiver atten        $State(atten)        -nowait
     Receiver ifbw         $State(ifbw)         -nowait
     Receiver demod        $State(demod_raw)    -nowait
     Receiver range        $State(range)        -nowait
     Receiver hwagc        $State(hwagc_raw)    -nowait
     Receiver agcmode      $State(agcmode)      -nowait
     Receiver filter       $State(iirFilter)    -nowait
     Receiver squelchmode  $State(squelchmode)  -nowait
     Receiver squelchlevel $State(squelchlevel) -nowait
     Receiver lock         $State(lock,mode)    -nowait
     
     Receiver::Config_agc  $State(agc,samples) $State(agc,low) $State(agc,hi) $State(agc,filter)
   }

   return [subst -nocommands $returnList]
 }
#****************************************************************************
# Name:        Update_agc
# Parameters:  index and value
# Description: Update AGC parms from engine
#****************************************************************************
 proc updateAgc { v } \
 {
    if {[llength $v] != 4} { return }
    variable State
    set State(agc,samples) [expr [lindex $v 0] * 10]
    set State(agc,low)     [expr [lindex $v 1] /10.0]
    set State(agc,hi)      [expr [lindex $v 2] /10.0]
    set State(agc,filter)  [lindex $v 3]
 }
#****************************************************************************
# Name:        Config_agc
# Parameters:  # samples to average over
#              lower and upper limit values
#              filtering setting
# Description: sets the parms for the Receive IF AGC loop
#****************************************************************************
 proc Config_agc { samples low hi filter } \
 {
    if {$samples < 10 } { set samples 10 }

    Receiver agc PERIOD [expr int($samples/10)]  -nowait
    Receiver agc AGCLL  [expr int($low*10)] -nowait
    Receiver agc AGCUL  [expr int($hi*10)] -nowait
    Receiver agc FILTER $filter -nowait
 }
#****************************************************************************
# Name:        Sync
# Parameters:  none
# Description: Write state variables to hardware
#****************************************************************************
 proc Sync { } \
 {
    Receiver enable 63
    eval [::Receiver::getSaveState]
 }

#****************************************************************************
# Name:        SyncAll
# Parameters:  none
# Description: Insures that TCL, the engine and hardware are in sync
#****************************************************************************
 proc SyncAll { } {
    set_receiver_callbacks
    if ![interp issafe] \
    {
       ::Receiver::Sync
    }
 }

#****************************************************************************
# Initialize the namespace and activate the callbacks
#****************************************************************************
   if ![interp issafe] \
   {
      HW_control::OnYigDone { ::Receiver::SyncAll }
   }

} ;# End of Receiver namespace

#****************************************************************************
# This registers the engine with the list for saving instrument states
#****************************************************************************
namespace eval ::SavedStates {

   variable engines   
   lappend engines Receiver
}

#****************************************************************************
# This registers the engine with the list for generating an HTML page
#****************************************************************************
namespace eval ::HTTPD \
{
   variable main_screen_items
   lappend main_screen_items(1) Receiver
}

#****************************************************************************
# This registers the engine with the list for the version screen
#****************************************************************************
namespace eval ::Version_Screen \
{
   variable version
   variable names
   
   array set names \
   {
     Rec_Eng "Receiver Engine"
     Rec_Drv "Receiver Driver"
   }
   set version(Rec_Eng)   [Receiver  version 0]
   set version(Rec_Drv)   [Receiver  version 1]
}
