#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C) 2001      DESIGN ENGINEERING        #
# --------------------------------------------------------------------------#
# FILE        : Serial.tcl                                                  #
# PRODUCT     : FITS                                                        #
# AUTHOR      : David Hagood                                                #
# DESCRIPTION : This file is sourced by TCL when the Serial command is first#
#             : used.  It loads the C++ interface file, provides an Serial  #
#             : namespace for screen variables, and sets up callbacks to    #
#             : update the variables for the UI.                            #
#                                                                           #
#############################################################################
# remove the variable watches

foreach v $::Serial::vlist {
     trace vdelete ::Serial::$v rwu ::Serial::Load
}

# Load the object file
if {"[info command Serial]" != ""} { return}
load $::Serial::object
package provide Serial 1.0

# don't load the namespace commands if we are a slave interp..
if [interp issafe {}] { return }

namespace eval ::Serial {
  variable saveafter {}
  variable interp {}

  set file [Serial file]
  fconfigure $file -translation {crlf crlf}
  fconfigure $file -buffering none

  # In case we have no config file, set the vars
  set baud 115200
  set term Ansi
  set mode OFF
  set echo ON
  set handshake SOFT
  set cur_mode OFF

  proc Baud { {newbaud {}} } {
     variable baud
     if {"$newbaud" !="" } {
        if {$newbaud == $baud } {
           return
        }
        set baud $newbaud
     }
     
     set baud [Serial baud $baud]
  }
  proc Echo { {newecho {}} } {
     variable echo
     variable interp
     if {"$newecho" !=""} {
        if {$newecho == $echo} {
          return
        }
        set echo $newecho
     }
     set echo [::Slave::Echo $interp $echo]
   }
   proc Handshake { {newhs {}} } {
     variable handshake
     variable interp
     if {"$newhs" !=""} {
        if {$newhs == $handshake} {
           return
        }
        set handshake $newhs
     }
     catch { set handshake [Serial handshake $handshake] }
   }
   proc Mode { {newmode {}} } {
      variable mode
      variable interp
      variable file
      variable term
      variable echo
      variable baud
      variable cur_mode
      
      if {"$newmode" != "" } {
         if {$newmode == $mode } {
            return
         }
         set mode $newmode
      }
      if {$mode == $cur_mode} { return }
      
      switch -- $mode {
         "off" -
         "0" -
         "OFF" { 
           set mode OFF
           set cur_mode $mode
           if {"$interp"!=""} {           
              upvar 0 $interp slave
              $slave(NAME) eval exit
              set mode "OFF"
              set interp {}
              set slave(NAME) {}
              $slave(EDIT) delete
           }
          }
          
         "on" -
         "ON" -
         "tcl" -
         "TCL" -
         default {
            set mode "TCL"
            set cur_mode $mode
            update idle
            if {"$interp"==""} {
              Baud $baud
              set interp [::Slave::New $file "Serial"]
              upvar 0 $interp slave
              interp alias $slave(INTERP) echo {} Serial::Echo
              interp alias $slave(INTERP) RING {} Serial::Ring
              ::Slave::Connect $interp RS232 $term
              ::Slave::Echo    $interp $echo
              fconfigure $file -translation {crlf crlf}
              upvar 0 $interp slave
              puts $file "$slave(IDN) on RS-232 active"
             }
         }
      }
   }
   proc Ring {} \
   {
      variable interp
      upvar 0 $interp slave
      puts slave(FD) "ATA"
   }
   proc Term { {newterm {}} } {
      variable term
      if {"$newterm" != "" } {
         if {$newterm == $term} {
           return
         }
         set term $newterm
      }
   }
   
   proc Init {} {
      variable mode
      variable baud
      variable term
      variable handshake
      Baud
      Term
      Handshake
      Echo
   }
    
  #****************************************************************************
  # Name:        getSaveState
  # Parameters:  none
  # Description: This function returns a list of TCL commands which can be used
  #              to set the C4 FM engine to its current state
  #****************************************************************************
  proc getSaveState { } \
  {
    variable mode
    variable baud
    variable term
    variable echo
    variable handshake
    variable saveafter
   
   set returnList \
   {
      ::Serial::Baud $baud
      ::Serial::Term $term
      ::Serial::Echo $echo
      ::Serial::Handshake $handshake
      ::Serial::Mode $mode
   }
   return [subst $returnList]
 }

}

#****************************************************************************
# This registers the engine with the list for saving instrument states
#****************************************************************************
namespace eval ::SavedStates {

   variable engines   
   lappend engines Serial
}


