package provide Units 1.0

#*************************************************
#       Name: convert_W_dBm
# Parameters: watts
#Description: converts watts to dBm
#*************************************************
proc convert_W_dBm { watts } {
   if { $watts < .00001 } {set watts .00001}
   return [expr double(10*log10(double($watts/.001)))]
}

#*************************************************
#       Name: convert_dBm_W
# Parameters: dBm
#Description: converts dBm to watts
#*************************************************
proc convert_dBm_W { dBm } {

  return [expr double(.001*(pow(10,double(($dBm/10)))))]
}
#*************************************************
#       Name: convert_dBm_mW
# Parameters: dBm
#Description: converts dBm to milliwatts
#*************************************************
proc convert_dBm_mW { dBm } {

  return [expr double((pow(10,double(($dBm/10)))))]
}
#*************************************************
#       Name: convert_dBm_V
# Parameters: dBm, dBm level
#         : load, optional, assumes 50
#Description: converts dbm level to volts, returns volt value
#*************************************************
proc convert_dBm_V { dbm {load 50} } {

   return [expr sqrt( ((pow(10,(double($dbm)/10))*.001)*double($load)) )]
}
#*************************************************
#       Name: convert_dBm_mV
# Parameters: dBm, dBm level
#         : load, optional, assumes 50
#Description: converts dbm level to millivolts, returns volt value
#*************************************************
proc convert_dBm_mV { dbm {load 50} } {

   return [expr 1000.0*sqrt( ((pow(10,(double($dbm)/10))*.001)*double($load)) )]
}
#*************************************************
#       Name: convert_dBm_uV
# Parameters: dBm, dBm level
#         : load, optional, assumes 50
#Description: converts dbm level to microvolts, returns volt value
#*************************************************
proc convert_dBm_uV { dbm {load 50} } {

   return [expr 1000000.0*sqrt( ((pow(10,(double($dbm)/10))*.001)*double($load)) )]
}
#*************************************************
#       Name: convert_V_dbm
# Parameters: volt, voltage level rms
#         : load, optional, assumes 50
#Description: converts voltage level to dBm, returns dbm value
#*************************************************
proc convert_V_dBm { volt {load 50} } {

  if {$volt < .00000001} {set volt .00000001}

   return [expr log10([expr (double($volt*$volt)/$load)/.001])*10]
}
