#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT  2002        DESIGN ENGINEERING        #
# --------------------------------------------------------------------------#
# FILE        : untar.tcl                                                   #
# PRODUCT     : FITS                                                        #
# AUTHOR      : Hieu Tran                                                   #
# DESCRIPTION : This file is sourced by TCL when the untar command is       #
#             : first used.  It loads the C++ interface file, provides a    #
#             : namespace for screen variables, and sets up callbacks to    #
#             : update the variables for the UI.                            #
#                                                                           #
#############################################################################
trace vdelete ::Untar::State ruw "::Untar::Load"

#****************************************************************************
# Handle a load of the C++ interface
#****************************************************************************
if {"[info command Untar]" != ""} { return }

# Call the base class dummy command to force a load
package require Engtclbase
Engtclbase

# Load the object file
::load $::Untar::object

package provide Untar 1.0


namespace eval ::Untar {


   array set untar_error_str { \
      0 "Open tar file"        \
      1 "Not in tar format"    \
      2 "Read file"            \
      3 "Create file"          \
      4 "Write file"           \
      5 "Couldn't start untar" \
   }


#****************************************************************************
# Name:        set_untar_callbacks
# Parameters:  none
# Description: Sets up the callbacks to update the untar data
#****************************************************************************
 proc set_untar_callbacks { } \
 {

   if { $::Untar::callbackStatus != 0 } \
   {
       return
   }

   Untar -onchange error     { ::Untar::updateUntarError %V; }
   Untar -onchange filecount { set ::Untar::State(tarfilecount) %V; }
   Untar -onchange filename  { ::Untar::updateUntarFilename %V; }
   Untar -onchange percent   { set ::Untar::State(filepercent) %V; }
   Untar -onchange tarsize   { ::Untar::updateUntarTarsize %V; }
   Untar -onchange tardone   { set ::Untar::State(tardone) %V; }
   
   set ::Untar::callbackStatus 1
 }


#****************************************************************************
# Name:        updateUntarError
# Parameters:  none
# Description: 
#****************************************************************************
 proc updateUntarError { code } \
 {
   variable State
   variable untar_error_str

   set State(tarerrcode) $code

   if [info exists untar_error_str($code)] \
   {
      set State(tarerrstr) $untar_error_str($code)
   }
 }


#****************************************************************************
# Name:        updateUntarFilename
# Parameters:  none
# Description: 
#****************************************************************************
 proc updateUntarFilename { parmList } \
 {
   variable State

   set State(filename)  [lindex $parmList 0]
   set State(filecount) [lindex $parmList 1]
 }

#****************************************************************************
# Name:        updateUntarTarsize
# Parameters:  none
# Description: 
#****************************************************************************
 proc updateUntarTarsize { parmList } \
 {
   variable State

   set State(tarfileoffset) [lindex $parmList 0]
   set State(tarfilesize)   [lindex $parmList 1]
 }


#****************************************************************************
# Initialize the namespace and activate the callbacks
#****************************************************************************
   Load::on_system_done { ::Untar::set_untar_callbacks }

} ;# End of Untar namespace

