#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C) 2001      DESIGN ENGINEERING        #
# --------------------------------------------------------------------------#
# FILE        : bbaudio.tcl                                                 #
# PRODUCT     : FITS                                                        #
# AUTHOR      : Bruce Butterfield                                           #
# DESCRIPTION : This file is sourced by TCL when the BBAudio command is     #
#             : first used.  It loads the C++ interface file, provides a    #
#             : namespace for screen variables, and sets up callbacks to    #
#             : update the variables for the UI.                            #
#                                                                           #
#############################################################################
trace vdelete ::BBAudio::State ruw "::BBAudio::Load"

#****************************************************************************
# Handle a load of the C++ interface
#****************************************************************************
# Call the base class dummy command to force a load
if {"[info command BBAudio]" != "" } { return }
package require Engtclbase
Engtclbase

# Load the object file
::load $::BBAudio::object

package provide BBAudio 1.0

#****************************************************************************
# Variables in the BBAudio namespace
#****************************************************************************
#    Status of commands sent to BBAudio
#       ::BBAudio::State(micbias)               {0 | 1}
#       ::BBAudio::State(micbias_lname)         {DYNAMIC | ELECTRET}
#       ::BBAudio::State(micbias_sname)         {DYN | ELEC}
#       ::BBAudio::State(sinadbal)              {0 | 1}
#       ::BBAudio::State(sinadbal_lname)        {BALANCED | UNBALANCED}
#       ::BBAudio::State(sinadbal_sname)        {BALAN | UNBAL}
#       ::BBAudio::State(fgenbal)               {0 | 1}
#       ::BBAudio::State(fgenbal_lname)         {UNBALANCED | BALANCED}
#       ::BBAudio::State(fgenbal_sname)         {UNBAL | BALAN}
#       ::BBAudio::State(fgenfilter)            {DATA | TONE}
#       ::BBAudio::State(demodfilter)           {DATA | TONE}
#       ::BBAudio::State(fgenLevel)             { "x1" | "x0.1" }
#       ::BBAudio::State(Engine_fgenLevel)      { 0 | 1 }
#    Status of connections to mixers
#       ::BBAudio::Mixers($mixer.$source)       {OFF | ON}
#                                               {mixer = see mixer_target_map}
#                                               {source = see mixer_source_map}
#****************************************************************************
#
#   BBAudio::set_bbaudio_route  L1     FGEN    0/1;  # DEMOD to Audio Out #1
#   BBAudio::set_bbaudio_route  FGEN1  FGEN    0/1;  # FGen 1 to Audio Out #1
#   BBAudio::set_bbaudio_route  FGEN2  FGEN    0/1;  # FGen 2 to Audio Out #1
#   BBAudio::set_bbaudio_route  MIC    FGEN    0/1;  # MIC to Audio Out #1
#   BBAudio::set_bbaudio_route  SINAD  FGEN    0/1;  # Audio In to Audio Out #1
#
#   BBAudio::set_bbaudio_route  L1     DEMOD   0/1;  # DEMOD to Audio Out #2
#   BBAudio::set_bbaudio_route  FGEN1  DEMOD   0/1;  # FGen 1 to Audio Out #2
#   BBAudio::set_bbaudio_route  FGEN2  DEMOD   0/1;  # FGen 2 to Audio Out #2
#   BBAudio::set_bbaudio_route  MIC    DEMOD   0/1;  # MIC to Audio Out #2
#   BBAudio::set_bbaudio_route  SINAD  DEMOD   0/1;  # Audio In to Audio Out #2
#    
#   BBAudio::set_bbaudio_route  L1     SPKR    0/1;  # DEMOD to Speaker
#   BBAudio::set_bbaudio_route  FGEN1  SPKR    0/1;  # FGen 1 to Speaker
#   BBAudio::set_bbaudio_route  FGEN2  SPKR    0/1;  # FGen 2 to Speaker
#   BBAudio::set_bbaudio_route  MIC    SPKR    0/1;  # MIC to Speaker
#   BBAudio::set_bbaudio_route  SINAD  SPKR    0/1;  # Audio In to Speaker
#    
#
#   BBAudio  clearmixer  FGEN   # Clears the Audio Out #1 inputs
#   BBAudio  clearmixer  DEMOD  # Clears the Audio Out #2 inputs
#   BBAudio  clearmixer  SPKR   # Clears the Speaker inputs
#
#
#   Above must be followed these calls in order to update the variables:
#
#   BBAudio::update_mixers [BBAudio getmixer FGEN]
#   BBAudio::update_mixers [BBAudio getmixer DEMOD]
#   BBAudio::update_mixers [BBAudio getmixer SPKR]
#
#****************************************************************************
        

#****************************************************************************
# Set up a namespace for BBAudio
#****************************************************************************
namespace eval ::BBAudio {
   variable SaveSortOrder AFTER(Receiver)


#****************************************************************************
# Name:        set_bbaudio_callbacks
# Parameters:  none
# Description: Sets up the callbacks to update the BBAudio data
#****************************************************************************
 proc set_bbaudio_callbacks { } {

   BBAudio -onchange micbias       { ::BBAudio::update_mic_bias %V}
   BBAudio -onchange sinadbalance  { ::BBAudio::update_sinad_bal %V}
   BBAudio -onchange demodfilter   { ::BBAudio::update_demod_filter %V}
   BBAudio -onchange fgenfilter    { ::BBAudio::update_fgen_filter %V}
   BBAudio -onchange fgenbalance   { ::BBAudio::update_fgen_bal %V}
   BBAudio -onchange fgenlevel     { ::BBAudio::update_fgen_level %V}
   BBAudio -queue getmixer         { ::BBAudio::update_mixers %V}
   BBAudio -queue route            { ::BBAudio::update_routes %V}
   BBAudio -onchange diginauto     { ::BBAudio::update_uut_rate %V}
   BBAudio -onchange uutdigin      { ::BBAudio::update_uut_in %V}
   BBAudio -onchange uutdigout     { ::BBAudio::update_uut_out %V}
   BBAudio -onchange uutvoutput    { ::BBAudio::update_uut_volts %V}
   BBAudio -onchange uutvoutadj    { ::BBAudio::update_uut_volts %V}
   BBAudio -onchange speakervol        { set ::BBAudio::State(volume_raw) %V }
   BBAudio -onchange speakervolpercent { set ::BBAudio::State(volume) %V }
 }
 
#****************************************************************************
# Name:        
# Parameters:  
# Description: 
#****************************************************************************
 proc update_uut_in { v } \
 {
    variable State
    set State(uut_in)       [lindex $v 0]
    set State(uut_in_delta) [lindex $v 1]
    
    # Added a wrapper for the case when 'State(uut_in_delta)' is NULL
    # Getting an error at boot up
    # Preston, 12/12/02
    if { $State(uut_in_delta) != "" } \
    {
      if [expr $State(uut_in_delta) & 0x20] \
      {
        # Microphone PTT has changed state
        if [expr $State(uut_in) & 0x20] \
        {
          set State(PTT) 0
        } \
        else \
        {
           set State(PTT) 1
        }
      }
    }
    
    return
 }
 
#****************************************************************************
# Name:        
# Parameters:  
# Description: 
#****************************************************************************
 proc update_uut_rate { time } \
 {
    variable State
    set State(diginauto) $time
 }
 
#****************************************************************************
# Name:        
# Parameters:  
# Description: 
#****************************************************************************
 proc update_uut_out { out } \
 {
    variable State
    set State(uut_out) $out
 }
 
#****************************************************************************
# Name:        
# Parameters:  
# Description: 
#****************************************************************************
 proc update_uut_volts { volts } \
 {
    variable State
    set State(uut_vout) $volts
 }
 
#****************************************************************************
# Name:        update_routes
# Parameters:  Routing list from the engine
#              Parm #0 = Source ID connected to target
#              Parm #1 = Target of routing connection
# Description: Updates the array of routing connections
#****************************************************************************
 proc update_routes { routeList } \
 {
   set target  [lindex $routeList 1]
 
   set ::BBAudio::State(Route_$target)  [lindex $routeList 0]
    
   # OVERRIDE folks who try to change the input routing on squelch...
   if { $::BBAudio::State(Route_23) != 0 } \
   {
     BBAudio route L1 SQUELCHED
   }
   
   return
 }

#****************************************************************************
# Name:        update_mixers
# Parameters:  Mixer list from the engine
#              Parm #0 = Mixer
#              Parm #1 = Bit array of connections
# Description: Updates the array of Mixer connections
#****************************************************************************
 proc update_mixers { routeList } \
 {
   #puts "update_mixers:  $routeList"
   
   variable  Mixers
   variable  State
   variable  mixer_source_map
   variable  mixer_target_map
   

   set mixerID    [lindex $routeList 0]
   set mixerData  [lindex $routeList 1]
   
   if { [info exists mixer_target_map($mixerID)] } \
   {
     set mixer  $mixer_target_map($mixerID)
   } \
   else \
   {
     return
   }

   set bitValue 1
   
   for { set i  0 } { $i < 16 } { incr i  1 } \
   {
     if { [info exists mixer_source_map($i)] } \
     {
       set source  $mixer_source_map($i)
     } \
     else \
     {
       # Stop the loop
       break
     }

     if { [expr $bitValue & $mixerData] } \
     {
       set Mixers($mixer.$source)  "ON"
     } \
     else \
     {
       set Mixers($mixer.$source)  "OFF"
     }
     
     set bitValue  [expr $bitValue * 2]
   }
   
   # Check the Modulator connection
   if { $Mixers(L1.L1) == "OFF" && $Mixers(L1.SQUELCHED) == "OFF" } \
   {
     if { $State(demod_to_mod) != "OFF" } \
     {
       set State(demod_to_mod)  "OFF"
     }
   } \
   elseif { $Mixers(L1.L1) == "ON" && $Mixers(L1.SQUELCHED) == "OFF" } \
   {
     if { $State(demod_to_mod) != "DEMOD" } \
     {
       set State(demod_to_mod)  "DEMOD"
     }
   } \
   elseif { $Mixers(L1.L1) == "OFF" && $Mixers(L1.SQUELCHED) == "ON" } \
   {
     if { $State(demod_to_mod) != "SQUELCH" } \
     {
       set State(demod_to_mod)  "SQUELCH"
     }
   }

   return 
 }


#****************************************************************************
# Name:        set_bbaudio_route (MIXER)
# Parameters:  source          Single data source
#              destination     Target mixer
#              state           0 = OFF, 1 = ON (Send data to mixer)
# Description: This function has largely been replaced by the generic
#              "BBAudio mixer" command, but it exists for conditions where
#              we need to fake a connection.  It currently is used to allow 
#              the user to connect L1 to SPKR, while it actually inserts a
#              scalar meter connection when this occurs
#****************************************************************************
 proc set_bbaudio_route { source  destination  state } \
 {

   # The following intercepts a command to route L1 to the speaker, and converts
   # it to a command to send it through a SCALAR meter (filter) first.  This allows
   # the speaker volume to be adjusted based on the internal scaling of the data
   # in L1.   

    set t_source $source
    if { $source == "L1" } { set t_source 1 }

    set t_dest $destination
    if { $destination == "SPKR" } { set t_dest 1 }

    # When L1 is sent to the SPEAKER, the volume needs to be adjusted. When the 
    # routing is initially set up, L1 is sent first to the SCALAR meter, and then
    # the SCALAR meter is eventually connected to the SPEAKER. When disconnecting
    # the SPEAKER from L1, we have to break the connections set up.  This makes the
    # hardware issues transparent to the user who just makes the L1 to SPKR connection.
    if { $t_source == 1 && $t_dest == 1 } \
    {
       if { $state == 1 || $state == "ON" } \
       {
          # Route L1 to the SPKR through the SCALAR meter to rescale the volume
          BBAudio route L1 SCALAR    -nowait    ;# Input from L1 to scalar filter
          BBAudio route SCALAR USER0 -nowait    ;# Connect scalar output to mixer input
          BBAudio mixer USER0 SPKR 1 -nowait    ;# Send scalar data to speaker
          BBAudio enablemeter 3 ON   -nowait
       } \
       else \
       {
          # Disconnect the special routing to the speaker
          BBAudio route NONE SCALAR  -nowait  ;# Remove input to scalar filter
          BBAudio route NONE USER0   -nowait  ;# Remove connection to mixer input
          BBAudio mixer USER0 SPKR 0 -nowait  ;# Remove User0 from speaker mixer
          BBAudio enablemeter 3 OFF  -nowait
       }
    } \
    else \
    {
       BBAudio mixer $source $destination $state -nowait
    }
 }

#****************************************************************************
# Name:        clr_bbaudio_routes
# Parameters:  none
# Description: Removes ALL inputs from ALL of the mixers
#****************************************************************************
 proc clr_bbaudio_routes { } {
    BBAudio clearmixer SPKR  -nowait
    BBAudio clearmixer DEMOD -nowait
    BBAudio clearmixer FGEN  -nowait
    BBAudio clearmixer L1    -nowait
    BBAudio clearmixer USER0 -nowait
    BBAudio clearmixer USER1 -nowait
 }

#****************************************************************************
# Name:        default_audio_routes
# Parameters:  none
# Description: Sets the audio routes to a standard default setting
#****************************************************************************
proc default_audio_routes { } {
    ::BBAudio::clr_bbaudio_routes
    BBAudio demodfilter DATA     -nowait
    BBAudio fgenbalance BALANCED -nowait
    BBAudio fgenlevel   NORMAL   -nowait
    BBAudio fgenfilter  DATA     -nowait
}

#****************************************************************************
# Name:        update_mic_bias
# Parameters:  none
# Description: Updates the mic bias from an engine message
#****************************************************************************
 proc update_mic_bias { bias } {
    if {$bias == 0} {
        set ::BBAudio::State(micbias) 0
        set ::BBAudio::State(micbias_lname) DYNAMIC
        set ::BBAudio::State(micbias_sname) DYN
    } elseif {$bias == 1} {
        set ::BBAudio::State(micbias) 1
        set ::BBAudio::State(micbias_lname) ELECTRET
        set ::BBAudio::State(micbias_sname) ELEC
    }
 }

#****************************************************************************
# Name:        update_sinad_bal
# Parameters:  none
# Description: Updates the sinad balance/unbalance
#****************************************************************************
 proc update_sinad_bal { balance } {
    if {$balance == 0} {
        set ::BBAudio::State(sinadbal) 0
        set ::BBAudio::State(sinadbal_lname) BALANCED
        set ::BBAudio::State(sinadbal_sname) BALAN
    } elseif {$balance == 1} {
        set ::BBAudio::State(sinadbal) 1
        set ::BBAudio::State(sinadbal_lname) UNBALANCED
        set ::BBAudio::State(sinadbal_sname) UNBAL
    }
 } 

#****************************************************************************
# Name:        update_fgen_bal
# Parameters:  none
# Description: Updates the fgen balance/unbalance
#****************************************************************************
 proc update_fgen_bal { balance } {
    if {$balance == 1} {
        set ::BBAudio::State(fgenbal) 1
        set ::BBAudio::State(fgenbal_lname) BALANCED
        set ::BBAudio::State(fgenbal_sname) BALAN
    } elseif {$balance == 0} {
        set ::BBAudio::State(fgenbal) 0
        set ::BBAudio::State(fgenbal_lname) UNBALANCED
        set ::BBAudio::State(fgenbal_sname) UNBAL
    }
 } 

#****************************************************************************
# Name:        update_fgen_filter
# Parameters:  none
# Description: Updates the fgen filter type
#****************************************************************************
 proc update_fgen_filter { filter } {
    if {$filter == 0} {
        set ::BBAudio::State(fgenfilter) DATA
    } elseif {$filter == 1} {
        set ::BBAudio::State(fgenfilter) TONE
    }
 } 

#****************************************************************************
# Name:        update_fgen_level
# Parameters:  none
# Description: Updates the fgen level field
#****************************************************************************
 proc update_fgen_level { level } {
    if {$level == 0} {
        set ::BBAudio::State(fgenLevel) "x1"
    } elseif {$level == 1} {
        set ::BBAudio::State(fgenLevel) "x0.1"
    }
    set ::BBAudio::State(Engine_fgenLevel) $level
 } 

#****************************************************************************
# Name:        update_demod_filter
# Parameters:  none
# Description: Updates the demod filter type
#****************************************************************************
 proc update_demod_filter { filter } {
    if {$filter == 0} {
        set ::BBAudio::State(demodfilter) DATA
    } elseif {$filter == 1} {
        set ::BBAudio::State(demodfilter) TONE
    }
 } 

#****************************************************************************
# Name:        toggle_mixer_source
# Parameters:  none
# Description: Commands the hardware to toggle a source connection to a mixer 
#****************************************************************************
 proc toggle_mixer_source { source  mixer } \
 {
    if { $::BBAudio::Mixers($mixer.$source) == "OFF" } \
    {
       ::BBAudio::set_bbaudio_route $source $mixer 1
    } \
    else \
    {
       ::BBAudio::set_bbaudio_route $source $mixer 0
    }
 }

#****************************************************************************
# Name:        toggle_L1_to_SPKR
# Parameters:  none
# Description: This command is specific to the L1/SPKR connection, because 
#              the connection will be made through the SCALAR and USER0, and
#              the Mixers matrix will not show the connection of SPKR.L1
#****************************************************************************
 proc toggle_L1_to_SPKR { } \
 {
    if { $::BBAudio::Mixers(SPKR.USER0) == "OFF" } \
    {
       ::BBAudio::set_bbaudio_route L1 SPKR 1
    } \
    else \
    {
       ::BBAudio::set_bbaudio_route L1 SPKR 0
    }
 }

#****************************************************************************
# Name:        toggle_mic_bias
# Parameters:  none
# Description: Commands the hardware to toggle the mic bias 
#****************************************************************************
 proc toggle_mic_bias { } \
 {
    if {$::BBAudio::State(micbias) == 0 } \
    {
       BBAudio micbias 1 -nowait
    } \
    else \
    {
       BBAudio micbias 0 -nowait
    }
 }

#****************************************************************************
# Name:        toggle_sinad_balance
# Parameters:  none
# Description: Commands the hardware to toggle the sinad balance
#****************************************************************************
 proc toggle_sinad_balance { } \
 {
    if {$::BBAudio::State(sinadbal_sname) == "BALAN" } \
    {
       BBAudio sinadbal UNBAL -nowait
    } \
    else \
    {
       BBAudio sinadbal BALAN -nowait
    }
 }

#****************************************************************************
# Name:        toggle_fgen_balance
# Parameters:  none
# Description: Commands the hardware to toggle the fgen balance
#****************************************************************************
 proc toggle_fgen_balance { } \
 {
    if {$::BBAudio::State(fgenbal_sname) == "BALAN" } \
    {
       BBAudio fgenbalance UNBAL -nowait
    } \
    else \
    {
       BBAudio fgenbalance BALAN -nowait
    }
 }

#****************************************************************************
# Name:        toggle_demod_filter
# Parameters:  none
# Description: Commands the hardware to toggle the demod filter type
#****************************************************************************
 proc toggle_demod_filter { } {
    if { $::BBAudio::State(demodfilter) == "TONE" } \
    {
       BBAudio demodfilter DATA -nowait
    } \
    else \
    {
       BBAudio demodfilter TONE -nowait
    }
 } 

#****************************************************************************
# Name:        toggle_fgen_filter
# Parameters:  none
# Description: Commands the hardware to toggle the fgen filter type
#****************************************************************************
 proc toggle_fgen_filter { } {
    if { $::BBAudio::State(fgenfilter) == "TONE" } \
    {
       BBAudio fgenfilter DATA -nowait
    } \
    else \
    {
       BBAudio fgenfilter TONE -nowait
    }
 } 

#****************************************************************************
# Name:        toggle_fgen_level
# Parameters:  none
# Description: Commands the hardware to toggle the fgen output level
#****************************************************************************
 proc toggle_fgen_level { } {
    if { $::BBAudio::State(fgenLevel) == "x1" } \
    {
       BBAudio fgenlevel 1 -nowait
    } \
    else \
    {
       BBAudio fgenlevel 0 -nowait
    }
 } 



#****************************************************************************
# Name:        getSaveState
# Parameters:  none
# Description: This function returns a list of TCL commands which can be used
#              to set the BB Audio engine to its current state
#****************************************************************************
 proc getSaveState { } {
   variable State
   set returnList \
   { 
      BBAudio micbias      $State(micbias_sname)    -nowait
      BBAudio sinadbal     $State(sinadbal_sname)   -nowait
      BBAudio demodfilter  $State(demodfilter)      -nowait
      BBAudio fgenfilter   $State(fgenfilter)       -nowait
      BBAudio fgenbalance  $State(fgenbal_sname)    -nowait
      BBAudio fgenlevel    $State(Engine_fgenLevel) -nowait
   }
   
   foreach { mixer } { SPKR DEMOD FGEN L1 USER0 USER1 } \
   {
      append returnList "      BBAudio clearmixer $mixer -nowait\n"
      
      foreach source {L1 MIC SINAD FILE FGEN0 FGEN1 FGEN2 FGEN3 FGEN4 USER0 USER1 USER2 USER3 USER4 SQUELCHED} \
      {
         if {$::BBAudio::Mixers($mixer.$source) == "ON"} \
         {
            append returnList "      BBAudio mixer $source $mixer ON -nowait\n"
         }
      }
      append returnList "      BBAudio getmixer $mixer -nowait\n"
   }
   
   for {set i 9} {$i < 24} {incr i 1} \
   {
        append returnList "      BBAudio route $State(Route_$i) $i -nowait\n"
   }
   if { $::BBAudio::Mixers(SPKR.USER0) == "OFF" } \
   {
      append returnList "      ::BBAudio::set_bbaudio_route L1 SPKR 0\n"
   }\
   else \
   {
      append returnList "      ::BBAudio::set_bbaudio_route L1 SPKR 1\n"
   }
   append returnList  "      BBAudio uutdigout $State(uut_out) -nowait\n"
   append returnList  "      BBAudio uutvoutput $State(uut_vout) -nowait\n"
   append returnList  "      BBAudio speakervolpercent $State(volume) -nowait\n"
   
   return [subst -nocommands $returnList]
 }

#****************************************************************************
# Name:        Sync
# Parameters:  none
# Description: Write state variables to hardware
#****************************************************************************
 proc Sync { } {
    after idle {eval [::BBAudio::getSaveState]}
    BBAudio uutdigin -query
 }

#****************************************************************************
# Name:        SyncAll
# Parameters:  none
# Description: Insures that TCL, the engine and hardware are in sync
#****************************************************************************
 proc SyncAll { } {
    set_bbaudio_callbacks
    if ![interp issafe] \
    {
       Sync
       BBAudio enable 3           -nowait
       BBAudio diginauto 10       -nowait
       BBAudio route L1 SQUELCHED -nowait
    }
 }
 
 proc Factory_defaults {} \
 {
    BBAudio micbias      ELEC    -nowait
    BBAudio sinadbal     UNBAL   -nowait
    BBAudio demodfilter  TONE      -nowait
    BBAudio fgenfilter   TONE       -nowait
    BBAudio fgenbalance  UNBAL    -nowait
    BBAudio fgenlevel    0 -nowait
    BBAudio clearmixer SPKR -nowait
    BBAudio mixer FGEN0 SPKR ON -nowait
    BBAudio mixer USER0 SPKR ON -nowait
    update_mixers [BBAudio getmixer SPKR]
    BBAudio clearmixer DEMOD -nowait
    BBAudio mixer L1 DEMOD ON -nowait
    update_mixers [BBAudio getmixer DEMOD]
    BBAudio clearmixer FGEN -nowait
    BBAudio mixer FGEN1 FGEN ON -nowait
    BBAudio mixer FGEN2 FGEN ON -nowait
    update_mixers [BBAudio getmixer FGEN]
    BBAudio clearmixer L1 -nowait
    BBAudio mixer FGEN3 L1 ON -nowait
    BBAudio mixer FGEN4 L1 ON -nowait
    update_mixers [BBAudio getmixer L1]
    BBAudio clearmixer USER0 -nowait
    update_mixers [BBAudio getmixer USER0]
    BBAudio clearmixer USER1 -nowait
    update_mixers [BBAudio getmixer USER1]
    BBAudio route -1 9  -nowait
    BBAudio route -1 10  -nowait
    BBAudio route -1 11  -nowait
    BBAudio route -1 12  -nowait
    BBAudio route -1 13  -nowait
    BBAudio route -1 14  -nowait
    BBAudio route 2 15  -nowait
    BBAudio route 2 16  -nowait
    BBAudio route -1 17  -nowait
    BBAudio route -1 18  -nowait
    BBAudio route -1 19  -nowait
    BBAudio route -1 20  -nowait
    BBAudio route -1 21  -nowait
    BBAudio route -1 22  -nowait
    BBAudio route 0 23  -nowait
    set_bbaudio_route L1 SPKR 1
    BBAudio uutdigout 0 -nowait
    BBAudio uutvoutput 0 -nowait
    BBAudio speakervolpercent 0 -nowait
 }
#****************************************************************************
# Initialize the namespace and activate the callbacks
#****************************************************************************
   Load::on_system_done { ::BBAudio::SyncAll }
}  ;# End of the BBAudio namespace

#****************************************************************************
# This registers the engine with the list for saving instrument states
#****************************************************************************
namespace eval ::SavedStates {

   variable engines   
   lappend engines BBAudio
}

#****************************************************************************
# This registers the engine with the list for the version screen
#****************************************************************************
namespace eval ::Version_Screen \
{
   variable version
   variable names
   
   array set names \
   {
     BBa_Eng   "BaseBand Audio Engine"
     BBa_Drv   "BaseBand Audio Driver"
   }
   set version(BBa_Eng)   [BBAudio version 0]
   set version(BBa_Drv)   [BBAudio version 1]
}

