
namespace eval ::Ber \
{
   set State(Pattern) ::Ber::DEMOD


   # mapping from Generator modes into our modes
   # each element is a list of {pattern to map to, exact match flag}
   array set Pattern_Mapping \
   {
       4  {0 0}
       6  {0 1}
       7  {2 1}
       9  {4 1}
       3  {1 0}
       11 {1 1}
       15 {6 1}
   }
 # on generator pattern change, sync BER to gen pattern
 proc Sync2GenPattern { gen_pattern } \
 {
    variable State
    variable Pattern_Mapping
    if [info exists Pattern_Mapping($gen_pattern)] \
    {
       foreach {pattern match } $Pattern_Mapping($gen_pattern) {}
          
       Ber pattern $pattern -nowait
       set State(pattern_match) $match
    } \
    else \
    {
       set State(pattern_match) 0
    }
 }
 # on BER pattern change, see if we are in sync with gen
 proc validatePattern { } \
 {
    variable State
    variable Pattern_Mapping
    set gen_pattern $::Generator::State(p25mode_raw)
    if [info exists Pattern_Mapping($gen_pattern)] \
    {
       foreach {pattern match } $Pattern_Mapping($gen_pattern) {}
       set State(pattern_match) $match
    } \
    else \
    {
       set State(pattern_match) 0
    }
 }
 # add querying the generator patter when turned on, to force an update of our pattern
 proc updateMode { runState } \
 {
   switch $runState \
   {
     0        { set ::Ber::State(run)  OFF }
      
     1        { set ::Ber::State(run)  ON; Generator p25Mode }
      
     default  { return }
   }
   
   set ::Ber::State(Engine_run)  $runState
   
   return
 }

#****************************************************************************
# Name:        Get_HTML
# Parameters:  base_url - the base URL for the containing page
# Description: Return an string containing HTML showing the current state
#              of this system
#****************************************************************************
 proc Get_HTML { {base_url ""} } \
 {
    variable State
    if {$State(Engine_run) == 0 } { return }
    if {$State(status) & 2} { return }
    set x "<table align=left border=1>\n"
    append x "<tr><td nowrap colspan=2><center><h2>Bit Error Rate</h2></center></td></tr>\n"
    foreach {item name units } { total_bits "Number of bits received" bits
                                 total_error_bits "Number of bits in error" bits
                                 total_frames  "Number of frames received" frames
                                 total_error_frames "Number of frames with errors" frames
                                 ber_percent "BER" %
                                 ber_ppm "BER" ppm
                                 test_pattern "Pattern in use" ""
                               } \
    {
       append x "<tr><td nowrap>$name</td><td>$State($item) $units</td></tr>\n"
    }
    append x "</table>"
    return $x
 }
#****************************************************************************
# Name:        Sources_Help
# Parameters:  none
# Description: Return an string containing HTML help for all installed data
#              sources
#****************************************************************************
 proc Sources_Help { } \
 {
    set retval "<ul>"
    variable Sources
    foreach source $Sources \
    {
        append retval [subst {<li>[${source}::Help]</li>\n}]
    }
    append retval "</ul>"
    return $retval
 }

#****************************************************************************
# Name:        Rescan_sources
# Parameters:  none
# Description: Re-read the sources definition files
#****************************************************************************
 proc Rescan_sources { } \
 {
    variable Sources
    set Sources ""
    foreach file [glob -nocomplain /system/BER/*.tcl] \
    {
       catch { uplevel #0 "source $file"}
    }
 }
 
#********************************************************************
#
#        Name: source_core
#
#  Parameters: 
#
# Description: Sets the Pattern source
#
#********************************************************************
proc source_core { patt } \
 {
   variable  State
   set enabled 0
   if [catch {set enabled [${patt}::Enabled]} err] \
   {
      puts "error $errorInfo $err"
      set enabled 0
   }
   if {$enabled == 0} \
   {
      set patt ::Ber::DEMOD
   }
   # stop old input source
   catch "$State(Source)::Stop"
   ${patt}::Start
   set State(Source) $patt
   
   return $State(Source)
 }
 proc Slave_source_core { slave patt } \
 {
    return [source_core $patt]
 }
 
 ::Slave::Add_alias ::Ber::source_core ::Ber::Slave_source_core
 
 Rescan_sources
 Generator -onchange p25Mode { ::Ber::Sync2GenPattern %V}
 
};  # End of the BER namespace


#****************************************************************************
# This registers the engine with the list for generating an HTML page
#****************************************************************************
namespace eval ::HTTPD \
{
   variable main_screen_items
   lappend  main_screen_items(2) Ber
}

#****************************************************************************
# This registers the engine with the list for saving instrument states
#****************************************************************************
namespace eval ::SavedStates \
{
  variable engines   

  lappend engines  Ber
}


#****************************************************************************
# This registers the engine with the list for the version screen
#****************************************************************************
namespace eval ::Version_Screen \
{
  variable version
  variable names
   
  array set names \
  {
    Ber_Eng "BER Engine"
    Ber_Drv "BER Driver"
  }

  set version(Ber_Eng) [Ber version 0]

  set version(Ber_Drv) [Ber version 1]
}

