#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C) 2001      DESIGN ENGINEERING        #
# --------------------------------------------------------------------------#
# FILE        : sinad.tcl                                                   #
# PRODUCT     : FITS                                                        #
# AUTHOR      : Bruce Butterfield                                           #
# DESCRIPTION : This file is sourced by TCL when the Sinad/Dist command is  #
#             : first used.  It loads the C++ interface file, provides a    #
#             : namespace for screen variables, and sets up callbacks to    #
#             : update the variables for the UI.                            #
#                                                                           #
#############################################################################
trace vdelete ::Sinad::State ruw "::Sinad::Load"

#****************************************************************************
# Handle a load of the C++ interface
#****************************************************************************
if {"[info command Sinad]" != "" } { return }

# Call the base class dummy command to force a load
package require Engtclbase
Engtclbase
package require Alert

# Load the object file
::load $::Sinad::object

package provide Sinad 1.0
package provide Dist 1.0

package require BBAudio

#****************************************************************************
# Add functions to the Sinad namespace
#****************************************************************************
#    Status of commands sent to Sinad engine
#       ::Sinad::State(Engine_run)          {0 | 1}
#       ::Sinad::State(run)                 {OFF | ON}
#       ::Sinad::State(average)             {1 - 100}
#       ::Sinad::State(filter)              {1 - 100}
#       ::Sinad::State(Engine_enabled)      
#       ::Sinad::State(enabled)             {OFF | ON}
#       ::Sinad::State(Engine_trigger)      {0 | 1}
#       ::Sinad::State(trigger)             {OFF | ON}
#       ::Sinad::State(Engine_math)         {0 | 1}
#       ::Sinad::State(math)                {OFF | ON}
#       ::Sinad::State(driverId)            Driver ID information
#       ::Sinad::State(engineId)            Engine ID information
#       ::Sinad::State(CWTenabled)          {OFF | ON}
#    Status of data received from the engine
#       ::Sinad::State(sinad)               Sinad reading
#       ::Sinad::State(distortion)          Distortion reading
#       ::Sinad::State(signalPower)         Total signal power
#       ::Sinad::State(noisePower)          Total noise power
#    Status of alert limits and peak values
#       ::Sinad::State(Sinad.peakHi)        Maximum peak reading
#       ::Sinad::State(Sinad.peakLo)        Minimum peak reading
#       ::Sinad::State(Sinad.peakFlags)     Peak Hold status flags
#       ::Sinad::State(Sinad.peakEnable)    Peak Enable settings
#       ::Sinad::State(Sinad.alertHi)       Maximum valid reading limit
#       ::Sinad::State(Sinad.alertLo)       Minimum valid reading limit
#       ::Sinad::State(Sinad.alerts)        Alert flag status
#
#       ::Sinad::State(Dist.peakHi)         Maximum peak reading
#       ::Sinad::State(Dist.peakLo)         Minimum peak reading
#       ::Sinad::State(Dist.peakFlags)      Peak Hold status flags
#       ::Sinad::State(Dist.peakEnable)     Peak Enable settings
#       ::Sinad::State(Dist.alertHi)        Maximum valid reading limit
#       ::Sinad::State(Dist.alertLo)        Minimum valid reading limit
#       ::Sinad::State(Dist.alerts)         Alert flag status
#****************************************************************************
namespace eval ::Sinad {
   variable SaveSortOrder AFTER(BBAudio)
   variable name "SINAD"

   set State(sample_rate) 80000

   array set trigger_map { \
      CONTINUOUS  1 \
      ONE_SHOT    2 \
      1 CONTINUOUS  \
      2 ONE_SHOT    \
   }

   array set math_map { \
      OFF 0  \
      ON 1   \
      0 OFF  \
      1 ON   \
   }
   variable Tracking_sources
   array set Tracking_sources { 0 OFF 1 FGEN1 2 FGEN2 3 MOD1 4 MOD2}
   
   set State(tmath,modes) { "OFF" "ADD" "SUB" "INV"}
	foreach trace {tlive tavg tref tulim tllim tpkh tpkl tmath } \
	{
	   set State(tlimit,$trace) 0
	   set State(tlimit,$trace,enable) 0
	}
	set State(tlimit,alert) OFF
	
	set State(tracking_gen) 0
	
#****************************************************************************
# Name:        set_sinad_callbacks
# Parameters:  none
# Description: Sets up the callbacks to update the Sinad data
#****************************************************************************
 proc set_sinad_callbacks { } {

   Sinad -onchange reading   { ::Sinad::updateReadingSinad %V}


   Sinad -onchange readpeak  { ::Sinad::updatePeaks   "Sinad" %V}
   Sinad -onchange ulimit    { ::Sinad::updateAlertHi "Sinad" %V}
   Sinad -onchange llimit    { ::Sinad::updateAlertLo "Sinad" %V}
   Sinad -onchange alerts    { ::Sinad::updateAlerts  "Sinad" %V}

   Sinad -onchange peak      { ::Sinad::updatePeakEnable "Sinad" %V}

   Sinad -onchange log       { ::Sinad::updateLogMode %V}

   Sinad -onchange run       { ::Sinad::updateMode %V}
   Sinad -onchange math      { ::Sinad::updateMath %V}
   Sinad -onchange trig      { ::Sinad::updateTrig %V}
  
   Sinad -onchange avg       { set ::Sinad::State(average) %V}
   Sinad -onchange filter    { set ::Sinad::State(filter) %V}
  
   Sinad -onchange en        { ::Sinad::updateEnable %V}
   Sinad -onchange encwt     { ::Sinad::updateCWT %V}
   Sinad -onchange fread     { ::Sinad::updateReading_old %V}
  
   Sinad -onchange rate      { set ::Sinad::State(rate) %V }
  
   Sinad -onchange maxx      { ::Sinad::updateMaxX  %V}
   Sinad -onchange minx      { ::Sinad::updateMinX  %V}
   Sinad -onchange maxy      { ::Sinad::updateMaxY  %V}
   Sinad -onchange miny      { ::Sinad::updateMinY  %V}
   Sinad -onchange rangey    { ::Sinad::updateRangeY  %V}
  
   Sinad -onchange size      { set ::Sinad::State(size) %V }
   Sinad -onchange alpha     { set ::Sinad::State(alpha) %V }
   Sinad -onchange window    { Sinad::updateWindow %V }
  
   Sinad -onchange lnotch    { set ::Sinad::State(lnotch) %V }
   Sinad -onchange hnotch    { set ::Sinad::State(hnotch) %V }
   Sinad -onchange lnoise    { set ::Sinad::State(lnoise) %V }
   Sinad -onchange hnoise    { set ::Sinad::State(hnoise) %V }
   Sinad -onchange tmath_mode    { Sinad::update_tmath %V }
   Sinad -onchange tlimit_status  { Sinad::trace_limit %V }
   Sinad -onchange tlimit_enable  { Sinad::trace_enable %V }
   Sinad -onchange tavg_factor  { ::Sinad::updateTraceAvgFactor %V }
   Sinad -onchange sample_rate  { ::Sinad::update_samplerate %V}
   Sinad -onchange size { set ::Sinad::State(size) %V}

   Sinad -queue -structure marker       { ::Sinad::updateMarker %V }
   Sinad -onchange anlz_trig { ::Sinad::updateAnlzTrig %V }

   trace variable ::BBAudio::State(Route_16) w { ::Sinad::Track_input }
   trace variable ::BBAudio::State(micbias)  w { ::Sinad::Track_input }
   trace variable ::BBAudio::State(sinadbal) w { ::Sinad::Track_input }
}

#****************************************************************************
# Name:        update_samplerate
# Parameters:  new sample rate for Sinad
# Description: update our sample rate
#****************************************************************************
proc update_samplerate { rate } \
{
   variable State
   set State(sample_rate) $rate
}

if ![interp issafe] \
{
  proc update_samplerate { rate } \
  {
     variable State
     set State(sample_rate) $rate
     adjust_track_gen
  }

  proc adjust_track_gen_cb  { args } \
  {
     variable State
     variable Tracking_sources

     set lun $State(tracking_gen)
     if {$lun == 0} \
     {
         # LUN = ZERO means the tracking generator is OFF
         # Avoid any accidental setup of FGEN 0
         set State(tracking_gen,text) OFF
         return
     }

     if { ($lun > 2) || ($::Generator::State(is_P25_mode) == 0) } \
     {
        if {$lun == 3} {set GeneratorUI::fgen_mode(3) SINC}
        if {$lun == 4} {set GeneratorUI::fgen_mode(4) SINC}
        FGen::setMode $lun SINC
     }

     set cmd [namespace code adjust_track_gen]
     after cancel $cmd
     after idle $cmd
     set State(tracking_gen,text) $Tracking_sources($State(tracking_gen))
  }

  proc adjust_track_gen  { } \
  {
     variable State
     set rate $State(sample_rate) 
     set tg $State(tracking_gen)
     
     if {$tg == 0} \
     {
        Sinad anlz_trigger 0 -nowait
        return
     }
     set arb_len 16384
     Sinad anlz_trigger 1 -nowait
     
     #recompute the settings on whatever FGENs we are using for a "tracking generator"
     # Theory: the arbs are arb_len samples long. So we need to set the arb replay rate
     # to arb_len/(our sample length) times our sample rate to insure proper sync (and sinc)
     set multiplier [expr $arb_len/$State(size)]
     set fgen_rate [expr $rate*$multiplier]
     set ::FGen::State($tg,SINC,rate)  $fgen_rate
     
     # now compute the omega value - the max frequency of the data is 
     # omega * Frate/(arb len), and we want max frequency = indicated max on analyzer
     # (plus 10%, hence the 1.1 in the function)
     set Fmax $State(maxx)
     set omega [expr $arb_len*($Fmax/$fgen_rate)*1.1]
     set ::FGen::State($tg,SINC,omega) $omega

     # Generate the SINC arbitrary waveform and send it to FGEN
     ::SincUI::setup_sinc_waveform  $tg

  }
  trace variable ::Sinad::State(tracking_gen) w Sinad::adjust_track_gen_cb
}

#****************************************************************************
# Name:        update_tmath
# Parameters:  new trace limit status
# Description: Update audio analzyer trace math mode
#****************************************************************************
proc update_tmath { mode } \
{
   variable State
   set State(tmath,mode,raw)   [lindex $mode 0]
   set State(tmath,offset,raw) [lindex $mode 1]
   set State(tmath,offset)     [format %3.1f [lindex $mode 1]]
   set State(tmath,trace_a)    [lindex $mode 2]
   set State(tmath,trace_b)    [lindex $mode 3]

   set State(tmath,mode) [lindex $State(tmath,modes) $State(tmath,mode,raw)]
}
#****************************************************************************
# Name:        set_tmath
# Parameters:  none
# Description: set the trace math mode 
#****************************************************************************
proc set_tmath { } \
{
   variable State
   Sinad tmath_mode $State(tmath,mode,raw) $State(tmath,offset) \
   $State(tmath,trace_a) $State(tmath,trace_b) -nowait
}

#****************************************************************************
# Name:        trace_limit
# Parameters:  new trace limit status
# Description: Update audio analzyer trace limit exceeded status
#****************************************************************************
proc trace_limit { levels } \
{
	variable State
	set i 0
	set overall_state 0
	foreach trace {tlive tavg tref tulim tllim tpkh tpkl tmath } \
	{
	   set state [lindex $levels $i]
	   incr i
	   set State(tlimit,$trace) $state
	   set overall_state [expr $overall_state | $state]
	}
	set State(tlimit,state) $overall_state
}
proc set_trace_enable { } \
{
   variable State
   Sinad tlimit_enable $State(tlimit,tlive,enable) \
                       $State(tlimit,tavg,enable) \
                       $State(tlimit,tref,enable) \
                       $State(tlimit,tulim,enable) \
                       $State(tlimit,tllim,enable) \
                       $State(tlimit,tpkhi,enable) \
                       $State(tlimit,tmath,enable) -nowait
}
proc trace_enable { enables } \
{
	variable State
	set i 0
	set overall_state 0
	foreach trace {tlive tavg tref tulim tllim tpkh tpkl tmath } \
	{
	   set state [lindex $enables $i]
	   incr i
	   set State(tlimit,$trace,enable) $state
	   if {$state & 1} \
	   {
	     set State(tlimit,$trace,low) "ON"
	   } \
	   else \
	   {
	      set State(tlimit,$trace,low) "OFF"
	   }
	   if {$state & 2} \
	   {
	     set State(tlimit,$trace,hi) "ON"
	   } \
	   else \
	   {
	      set State(tlimit,$trace,hi) "OFF"
	   }
	   
	   if {$state & 4} \
	   {
	     set State(tlimit,$trace,alarm) "ON"
	   } \
	   else \
	   {
	      set State(tlimit,$trace,alarm) "OFF"
	   }
	   set overall_state [expr $overall_state | ($State(tlimit,$trace,enable) & $state)]
	}
	set State(tlimit,enable) $overall_state
}
#****************************************************************************
# Name:        updateLogMode
# Parameters:  new log mode
# Description: Update audio analzyer log mode
#****************************************************************************
proc updateLogMode { mode } \
{
   variable State
   
   set State(log_mode)  $mode
   
   if { $mode == 0 } \
   {
     set State(log_mode,text)  "LINEAR"
   } \
   else \
   {
     set State(log_mode,text)  "LOG"
   }
}
#****************************************************************************
# Name:        toggleLogMode
# Parameters:  none
# Description: change log mode on audio analyzer
#****************************************************************************
proc toggleLogMode { } \
{
   variable State
   if {$State(log_mode) == 0} \
   {
      Sinad log 1 -nowait
   } \
   else \
   {
      Sinad log 0 -nowait
   }
}

#****************************************************************************
# Name:        setWindow
# Parameters:  none
# Description: Update windowing mode based on menu var
#****************************************************************************
 proc setWindow {  } \
 {
    variable State
 
    set which  [lsearch $State(window_menu) $State(window,text)]
 
    Sinad window  $which  -nowait
 }
 
 
#****************************************************************************
# Name:        updateWindow
# Parameters:  New windowing mode
# Description: Update the SINAD windowing mode
#****************************************************************************
 proc updateWindow { window } \
 {                             
    variable State
    
    set State(window)  $window
    
    set work  [lindex $State(window_menu) $window]
    
    if { "$work" != "" } \
    {
      set State(window,text)  $work
    } \
    else \
    {
      set State(window,text)  ???
    }
    
    return
 }
 
#****************************************************************************
# Name:        updateAnlzTrig
# Parameters:  state of audio analyzer trigger
# Description: Update the analyzer trigger state
#****************************************************************************
 proc updateAnlzTrig { state } \
 {
   variable  State
    
   set State(anlz_trig,raw)  $state
    
   switch -- $state \
   {
     0  { set State(anlz_trig)  "OFF"    }
       
     1  { set State(anlz_trig)  "CENTER" }
   }
    
   return
 }
 
#****************************************************************************
# Name:        updateMarker
# Parameters:  marker list
# Description: Get the reading on a marker
#****************************************************************************
 proc updateMarker { m } \
 {
    #puts "updateMarker"
    
    variable State
 
    set which [lindex $m 0]
    set freq  [lindex $m 1]
    set levels [lindex $m 2]
    set State(marker,$which,pos)   [lindex $m 3]
    set live  [lindex $levels 0]
    set avg   [lindex $levels 1]
    set ref   [lindex $levels 2]
    set ul    [lindex $levels 3]
    set ll    [lindex $levels 4]
    set peak_h [lindex $levels 5]
    set peak_l [lindex $levels 6]
    set math   [lindex $levels 7]
    
    array set State [subst { marker,$which,freq  [format  "%.3f"  $freq]
                             marker,$which,live  [format  "%.3f"  $live]
                             marker,$which,avg   [format  "%.3f"  $avg]
                             marker,$which,ref   [format  "%.3f"  $ref]
                             marker,$which,ul    [format  "%.3f"  $ul]
                             marker,$which,ll    [format  "%.3f"  $ll]
                             marker,$which,ph    [format  "%.3f"  $peak_h]
                             marker,$which,pl    [format  "%.3f"  $peak_l]
                             marker,$which,math  [format  "%.3f"  $math]
                            }]
 }
 
#****************************************************************************
# Name:        setTrig
# Parameters:  trigState         ONE_SHOT | CONTINUOUS
# Description: Sets up the current trigger mode of the Sinad meter
#****************************************************************************
 proc setTrig { trigState } {
    if [info exists ::Sinad::trigger_map($trigState)] {
       Sinad trig $::Sinad::trigger_map($trigState)
    }
 }

#****************************************************************************
# Name:        updateTrig
# Parameters:  trigState         1 = One Shot, 2 = Continuous
# Description: Sets up the current trigger mode of the Sinad meter
#****************************************************************************
 proc updateTrig { trigState } \
 {
   variable State
   
   if [info exists ::Sinad::trigger_map($trigState)] \
   {
     set State(trigger) $::Sinad::trigger_map($trigState)
   }
   
   set State(Engine_trigger) $trigState
 }

#****************************************************************************
# Name:        setMath
# Parameters:  mathState         OFF | ON
# Description: Sets up the current math mode of the Sinad meter
#****************************************************************************
 proc setMath { mathState } \
 {
   if [info exists ::Sinad::math_map($mathState)] \
   {
     Sinad math $::Sinad::math_map($mathState)
   }
 }

#****************************************************************************
# Name:        updateMath
# Parameters:  trigState         0 = OFF, 1 = ON
# Description: Sets up the current math mode of the Sinad meter
#****************************************************************************
 proc updateMath { mathState } \
 {
   variable State
   
   if [info exists ::Sinad::math_map($mathState)] \
   {
     set State(math) $::Sinad::math_map($mathState)
   }
   
   set State(Engine_math) $mathState
 }


#****************************************************************************
# Name:        updateMode
# Parameters:  runState         0 = Off, 1 = On
# Description: Sets up the current run state of the Sinad
#****************************************************************************
 proc updateMode { runState } \
 {
   variable State
   
   switch $runState \
   {
     0       { set State(run) OFF }
     1       { set State(run) ON }
     default { return }
   }

   set State(Engine_run) $runState
 }

#****************************************************************************
# Name:        updateEnable
# Parameters:  enableBits       1 = Off, 3 = Enabled
# Description: Sets up the current enabled state of the Sinad
#****************************************************************************
 proc updateEnable { enableBits } \
 {
   variable State
   set State(Engine_enabled) $enableBits
   
   if {$enableBits & 2} \
   {
     set State(enabled) Enabled
   } \
   else \
   {
      set State(enabled) Disabled
   }
   
   # Get the traces presently displayed on the Audio Analyzer
   if { [expr $enableBits & 4] } \
   {
     set State(trace_state,live)  "ON"
   } \
   else \
   {
     set State(trace_state,live)  "OFF"
   }

   if { [expr $enableBits & 8] } \
   {
     set State(trace_state,avg)  "ON"
   } \
   else \
   {
     set State(trace_state,avg)  "OFF"
   }

   if { [expr $enableBits & 16] } \
   {
     set State(trace_state,stored)  "ON"
   } \
   else \
   {
     set State(trace_state,stored)  "OFF"
   }

   if { [expr $enableBits & 32] } \
   {
     set State(trace_state,ulim)  "ON"
   } \
   else \
   {
     set State(trace_state,ulim)  "OFF"
   }

   if { [expr $enableBits & 64] } \
   {
     set State(trace_state,llim)  "ON"
   } \
   else \
   {
     set State(trace_state,llim)  "OFF"
   }
   if { [expr $enableBits & 0x10000] } \
   {
     set State(trace_state,pkhi)  "ON"
   } \
   else \
   {
     set State(trace_state,pkhi)  "OFF"
   }
   
   if { [expr $enableBits & 0x20000] } \
   {
     set State(trace_state,pklo)  "ON"
   } \
   else \
   {
     set State(trace_state,pklo)  "OFF"
   }
   if { [expr $enableBits & 0x40000] } \
   {
     set State(trace_state,math)  "ON"
   } \
   else \
   {
     set State(trace_state,math)  "OFF"
   }
     
   # Get the state of the Audio Analyzer Markers
   if { [expr $enableBits & 128] } \
   {
     set State(marker,0,state)  "ON"
   } \
   else \
   {
     set State(marker,0,state)  "OFF"
   }
   
   if { [expr $enableBits & 256] } \
   {
     set State(marker,1,state)  "ON"
   } \
   else \
   {
     set State(marker,1,state)  "OFF"
   }

   if { [expr $enableBits & 512] } \
   {
     set State(marker,2,state)  "ON"
   } \
   else \
   {
     set State(marker,2,state)  "OFF"
   }

   if { [expr $enableBits & 1024] } \
   {
     set State(marker,3,state)  "ON"
   } \
   else \
   {
     set State(marker,3,state)  "OFF"
   }

   if { [expr $enableBits & 2048] } \
   {
     set State(marker,4,state)  "ON"
   } \
   else \
   {
     set State(marker,4,state)  "OFF"
   }

   if { [expr $enableBits & 4096] } \
   {
     set State(marker,5,state)  "ON"
   } \
   else \
   {
     set State(marker,5,state)  "OFF"
   }

   if { [expr $enableBits & 8192] } \
   {
     set State(marker,6,state)  "ON"
   } \
   else \
   {
     set State(marker,6,state)  "OFF"
   }

   if { [expr $enableBits & 16384] } \
   {
     set State(marker,7,state)  "ON"
   } \
   else \
   {
     set State(marker,7,state)  "OFF"
   }

   if { [expr $enableBits & 32768] } \
   {
     set State(marker,8,state)  "ON"
   } \
   else \
   {
     set State(marker,8,state)  "OFF"
   }

   if { [expr $enableBits & 65536] } \
   {
     set State(marker,9,state)  "ON"
   } \
   else \
   {
     set State(marker,9,state)  "OFF"
   }

   return
 }


#****************************************************************************
# Name:        updateCWT
# Parameters:  state       0 = Off, 1 = On
# Description: Sets up the current enabled state of the C-Weight filter
#****************************************************************************
 proc updateCWT { state } \
 {
   variable State
   
   switch $state \
   {
     1       { set State(CWTenabled) ON }
     default { set State(CWTenabled) OFF }
   }
 }


#****************************************************************************
# Name:        updateReading
# Parameters:  readingList      List of data from the engine
#                Sinad          Sinad reading
#                Sinad prec.    Precision of sinad
#                Distortion     Distortion reading
#                Dist prec.     Precision of distortion
#                Sig power      Signal power
#                Noise power    
#                Status
# Description: Sets up the current state of the Sinad reading
#****************************************************************************
 proc updateReading_old { readingList } \
 {
   variable State
   
   # Get the data from the meter reading
   set State(signalPower)     [lindex $readingList 0 ]
   set State(noisePower)      [lindex $readingList 1]
   set State(totalPower)      [lindex $readingList 2]
   set State(readStatus)      [lindex $readingList 3]
   set State(dc)              [lindex $readingList 4]
   set State(sig_level)       [lindex $readingList 5]
   set State(fft_time)        [lindex $readingList 6]
 }

#****************************************************************************
# Name:        updateReading
# Parameters:  readingList      List of data from the engine
#                Sinad          Sinad reading
#                Sinad prec.    Precision of sinad
#                Distortion     Distortion reading
#                Dist prec.     Precision of distortion
#                Sig power      Signal power
#                Noise power    
#                Status
# Description: Sets up the current state of the Sinad reading
#****************************************************************************
 proc updateReadingSinad { readingList } \
 {

   variable State
   set State(sinad,raw) [lindex $readingList 0]
   set State(sinad)  [format "%3.2f" $State(sinad,raw) ]
   set status        [lindex $readingList 2]
   set State(Sinad.alertStatus) $status
   if {$status & 0x2000} \
   {
      set State(alarm) "ACTIVE"
   } \
   else \
   {
      set State(alarm) "inactive"
   }
 }
 proc updateReadingDist { readingList } \
 {
    variable State
    set State(distortion,raw) [lindex $readingList 0]
    if { $State(distortion,raw) < 990.0 } \
    {
        set State(distortion)   [format "%4.2f" $State(distortion,raw) ]
    } else \
    {
        set State(distortion)   [format "%3.2g" $State(distortion,raw) ]
    }
    set status              [lindex $readingList 2]
    set State(Dist.alertStatus) $status
 }



#****************************************************************************
# Name:        updatePeaks
# Parameters:  meterType        "Sinad"  or  "Dist"
#              readingList      List of data from the engine
#                 Peak HI       Max peak value
#                 HI Precision  Max peak precision
#                 Peak LO       Min peak value
#                 LO Precision  Min peak precision
#                 Flags         Status flags
# Description: Sets up the current state of the peak readings
#****************************************************************************
 proc updatePeaks { meterType  readingList } \
 {
   variable State
   
   # Get the data from the peak readings
   set peakHi  [lindex $readingList 0]
   set peakLo  [lindex $readingList 2]
   
   set State($meterType.peakFlags) [lindex $readingList 4]

   set State($meterType.peakHi)  [format "%0.2f" $peakHi]
   set State($meterType.peakLo)  [format "%0.2f" $peakLo]  
   
 }


#****************************************************************************
# Name:        updatePeakEnable
# Parameters:  
# Description: Sets up the current state of the peak enable
#****************************************************************************
 proc updatePeakEnable { meterType  peakEnable } \
 {
   variable State
   
   set State($meterType.peakEnable)  $peakEnable

   if { $State($meterType.peakEnable) == 1 } \
   {
     set State($meterType.peakEnable_text)  "ON"
   }

   if { $State($meterType.peakEnable) == 0 } \
   {
     set State($meterType.peakEnable_text)  "OFF"
   }

 }


#****************************************************************************
# Name:        updateAlerts
# Parameters:  
# Description: Update the Alerts flags
#****************************************************************************
 proc updateAlerts { meterType  alerts } \
 {
   variable State
   
   set State($meterType.alerts)  $alerts
 
   if {$alerts & 2} \
   {
     set State($meterType.alertHi_text) "ON"
   } \
   else \
   {
     set State($meterType.alertHi_text) "OFF"
   }
   if {$alerts & 1} \
   {
     set State($meterType.alertLo_text) "ON"
   } \
   else \
   {
     set State($meterType.alertLo_text) "OFF"
   }
   if {$alerts & 4} \
   {
     set State(alarm_text)                  "ON"
     set State($meterType.BellEnabled_text) "ON"

     set State($meterType.BellEnabled) 1
   } \
   else \
   {
     set State(alarm_text)                  "OFF"
     set State($meterType.BellEnabled_text) "OFF"

     set State($meterType.BellEnabled) 0
   }
   
   if {$meterType == "Sinad"} \
   {
     set State(alerts)            $State(Sinad.alerts)
     set State(BellEnabled)       $State(Sinad.BellEnabled)
     set State(BellEnabled_text)  $State(Sinad.BellEnabled_text)
     set State(alarm_text)        $State(Sinad.alarm_text)
     set State(alertLo_text)      $State(Sinad.alertLo_text)
     set State(alertHi_text)      $State(Sinad.alertHi_text)
   }
   return
 }


#****************************************************************************
# Name:        updateAlertHi
# Parameters:  
# Description: Formats the Alert High value
#****************************************************************************
 proc updateAlertHi { meterType  alertHi } \
 {
   variable State
   
   set State($meterType.alertHi)  [format "%0.2f" $alertHi]
 }

#****************************************************************************
# Name:        updateAlertLo
# Parameters:  
# Description: Formats the Alert Low value
#****************************************************************************
 proc updateAlertLo { meterType  alertLo } \
 {
   variable State
   
   set State($meterType.alertLo)  [format "%0.2f" $alertLo]

 }



#****************************************************************************
# Name:        updateMaxX
# Parameters:  maxx 
# Description: Formats the maxx value
#****************************************************************************
 proc updateMaxX { maxx } \
 {
   variable State
   
   set State(maxx)  [expr int($maxx)]
   
   # Set the Span
   set State(span)  [expr int($State(maxx) - $State(minx))]

   # Set the center frequency
   set State(center_freq)  [expr int(($State(maxx) + $State(minx)) / 2.0)]
   
   if ![interp issafe] \
   {
      set cmd [namespace code adjust_track_gen]
      after cancel $cmd
      after 500 $cmd
   }
 }


#****************************************************************************
# Name:        updateMinX
# Parameters:  minx 
# Description: Formats the minx value
#****************************************************************************
 proc updateMinX { minx } \
 {
   variable State
   
   set State(minx)  [expr int($minx)]

   # Set the Span
   set State(span)  [expr int($State(maxx) - $State(minx))]

   # Set the center frequency
   set State(center_freq)  [expr int(($State(maxx) + $State(minx)) / 2.0)]

   return
 }


#****************************************************************************
# Name:        updateMaxY
# Parameters:  maxy 
# Description: Formats the maxy value
#****************************************************************************
 proc updateMaxY { maxy } \
 {
   variable State
   
   set State(maxy)  [format "%.1f" $maxy]
   set cmd [namespace code update_scale]
   after cancel $cmd
   after idle $cmd
 }


#****************************************************************************
# Name:        updateMinY
# Parameters:  miny 
# Description: Formats the miny value
#****************************************************************************
 proc updateMinY { miny } \
 {
   variable State

   set State(miny)  [format "%.1f" $miny]
   set cmd [namespace code update_scale]
   after cancel $cmd
   after idle $cmd
}
#****************************************************************************
# Name:        updateRangeY
# Parameters:  rangey 
# Description: Formats the miny value
#****************************************************************************
 proc updateRangeY { rangey } \
 {
   variable State

   set State(rangey) $rangey
   set cmd [namespace code update_scale]
   after cancel $cmd
   after idle $cmd
 }
proc update_scale {} \
{
   variable State
   # NOTE: we assume 8 division on screen
   # Update all of the scale labels
   set State(scale,raw) [expr $State(rangey)/8.0]
   for {set i 1} { $i < 8 } { incr i } \
   {
      set State(scale$i) [format "%.1f" [expr $State(maxy) - ($i *$State(scale,raw))]]
   }
   # Now, see if we have an appropriate set of units - if not, set range to --
   foreach possible $State(scale_menu) \
   {
      if {abs($possible-$State(scale,raw)) < .01} \
      {
        set State(scale) $possible
        set State(scale,raw) $possible
        return
      }
   }
   set State(scale) "--"
 }


#***************************************************************************
#        Name: setSpan
#  Parameters:
# Description: Sets the Start and Stop frequencies to the new Span.
#***************************************************************************
proc setSpan { } \
{
  #puts "setSpan"
  
  variable  State

  # Set the desired Start and Stop frequencies and span
  set desired_span $State(span)
  set desired_maxx  [expr $State(center_freq) + ($State(span) / 2.0)]
  set desired_minx  [expr $State(center_freq) - ($State(span) / 2.0)]
  
  # Attempt to use desired values
  set actual_maxx [Sinad  maxx $desired_maxx -forgive]
  set actual_minx [Sinad  minx $desired_minx -forgive]
  
  set delta_maxx [expr $actual_maxx + 0.1]
  set delta_minx [expr $actual_minx - 0.1]
  if { $desired_maxx > $delta_maxx } \
  {
     Sinad minx [expr $actual_maxx - $desired_span] -nowait
  } elseif { $desired_minx < $delta_minx } \
  {
     Sinad maxx [expr $actual_minx + $desired_span] -nowait
  }
}


#***************************************************************************
#        Name: setFreq
#  Parameters:
# Description: Sets the Start and Stop frequencies to the new center freq.
#***************************************************************************
proc setFreq { } \
{
  #puts "setFreq"
  
  variable  State

  # Set the Start frequency
  set maxx  [expr $State(center_freq) + ($State(span) / 2.0)]
  
  after idle "Sinad  maxx  $maxx  -nowait"
  
  # Set the Stop frequency
  set minx  [expr $State(center_freq) - ($State(span) / 2.0)]
  
  after idle "Sinad  minx  $minx  -nowait"

  return
}

#***************************************************************************
#        Name: setCenterFreq
#  Parameters:
# Description: Sets the Start and Stop frequencies to the new center freq.
#***************************************************************************
proc setCenterFreq { newFreq currentFreq } \
{
  variable  State
  #puts "setCenterFreq: new = $newFreq, old = $currentFreq, span = $State(span)"

  # Save the current settings
  set currentSpan $State(span)

  # We need to solve several problems related to the way the engine clips data
  #    1) If the center changes past the current min/max set by the span, we
  #       need to make sure to adjust the correct min/max first, or the engine
  #       will clip to force Max >= Min + 10
  #    2) If the center frequency is set too close to the absolute max or min
  #       frequency, the span will clip

  set desired_maxx  [expr $newFreq + ($currentSpan / 2.0)]
  set desired_minx  [expr $newFreq - ($currentSpan / 2.0)]

  # Set the order to update the min and max frequency, based on which way we
  # are moving the center
  if { $newFreq > $currentFreq } \
  {
     set actual_maxx [Sinad  maxx  $desired_maxx -forgive]
     set delta_maxx [expr $actual_maxx + 0.1 ]
     if { $desired_maxx > $delta_maxx } \
     {
        set currentSpan [expr $currentSpan - 2.0 * ($desired_maxx - $actual_maxx) ]
        set desired_minx [expr $actual_maxx - $currentSpan ]
     }
     Sinad  minx  $desired_minx -nowait
  } else \
  {
     set actual_minx [Sinad  minx  $desired_minx -forgive]
     set delta_minx [expr $actual_minx - 0.1 ]
     if { $desired_minx < $delta_minx } \
     {
        set currentSpan [expr $currentSpan - 2.0 * ($actual_minx - $desired_minx) ]
        set desired_maxx [expr $actual_minx + $currentSpan ]
     }
     Sinad  maxx  $desired_maxx -nowait
  }
}

#***************************************************************************
#        Name: setScale
#  Parameters:
# Description: Sets the Scale display values.
#***************************************************************************
proc setScale { } \
{
  #puts "setScale"
  
  variable  State
  
  if {$State(scale) != "--"} \
  {
     set State(scale,raw) $State(scale)
     set temp [expr 8 * $State(scale)]
     after idle "Sinad  rangey  $temp  -nowait"
  }
}


#****************************************************************************
# Name:        updateTraceAvgFactor
# Parameters:  tavg_factor 
# Description: Formats the tavg_factor value
#****************************************************************************
 proc updateTraceAvgFactor { tavg_factor } \
 {
   variable State
   
   set State(tavg_factor)  [format "%.1f" $tavg_factor]
 }



#********************************************************************
#
#       Name: setLimits
#
# Parameters:
#
# Decsription: Sets the limit values
#
#********************************************************************
proc setLimits { meterType  limit_type } \
{
  variable  State
  
  if { $meterType == "Dist" } \
  {
    if { $limit_type == "high" } \
    {
      set temp  [expr $State(Dist.alerts) & 0x02]
    
      # If the limit alert is turned OFF, turn it ON
      if { $temp == 0 } \
      {
        set temp  [expr $State(Dist.alerts) | 0x02]
        
        Dist alerts  $temp  -nowait
      }
      
      Dist ulimit  $State(Dist.alertHi)  -nowait
    }
   
    if { $limit_type == "low" } \
    {
      set temp  [expr $State(Dist.alerts) & 0x01]
      
      # If the limit alert is turned OFF, turn it ON
      if { $temp == 0 } \
      {
        set temp  [expr $State(Dist.alerts) | 0x01]
        
        Dist alerts  $temp  -nowait
      }
      
      Dist llimit  $State(Dist.alertLo)  -nowait
    }
    
  }

  if { $meterType == "Sinad" } \
  {
    if { $limit_type == "high" } \
    {
      set temp  [expr $State(Sinad.alerts) & 0x02]
    
      # If the limit alert is turned OFF, turn it ON
      if { $temp == 0 } \
      {
        set temp  [expr $State(Sinad.alerts) | 0x02]
        
        Sinad alerts  $temp  -nowait
      }
      
      Sinad ulimit  $State(Sinad.alertHi)  -nowait
    }
   
    if { $limit_type == "low" } \
    {
      set temp  [expr $State(Sinad.alerts) & 0x01]
      
      # If the limit alert is turned OFF, turn it ON
      if { $temp == 0 } \
      {
        set temp  [expr $State(Sinad.alerts) | 0x01]
        
        Sinad alerts  $temp  -nowait
      }
      
      Sinad llimit  $State(Sinad.alertLo)  -nowait
    }
    
  }

  return
}  


#********************************************************************
#
#       Name: toggleAlerts
#
# Parameters:
#
# Decsription: Toggles the state of the alerts
#
#********************************************************************
proc toggleAlerts { meterType  limit_type } \
 {
   variable  State
   
   if { $meterType == "Dist" } \
   {
     if { $limit_type == "high" } \
     {
       set new  [expr $State(Dist.alerts) ^ 2]
    
       Dist alerts  $new  -nowait
     }

     if { $limit_type == "low" } \
     {
       set new  [expr $State(Dist.alerts) ^ 1]
    
       Dist alerts  $new  -nowait
     }
   }

   if { $meterType == "Sinad" } \
   {
     if { $limit_type == "high" } \
     {
       set new  [expr $State(Sinad.alerts) ^ 2]
    
       Sinad alerts  $new  -nowait
     }

     if { $limit_type == "low" } \
     {
       set new  [expr $State(Sinad.alerts) ^ 1]
    
       Sinad alerts  $new  -nowait
     }
   }
  
   return
 }


#********************************************************************
#
#       Name: toggleAlarm
#
# Parameters:
#
# Decsription: Toggles the state of the alarm
#
#********************************************************************
proc toggleAlarm { {meterType Sinad} } \
{
  variable State
  $meterType alerts [expr $State($meterType.alerts)^4]
}


#********************************************************************
#
#       Name: togglePeakEnable
#
# Parameters: 
#
# Decsription: 
#
#********************************************************************
proc togglePeakEnable { meterType } \
 {
   variable State
   
   if { $meterType == "Dist" } \
   {
     if { $State(Dist.peakEnable) == 1 } \
     {
       after idle "Dist peak 0 -nowait"
     }
  
     if { $State(Dist.peakEnable) == 0 } \
     {
       after idle "Dist phreset  -nowait"
       after idle "Dist peak 1   -nowait"
     }
   }

   if { $meterType == "Sinad" } \
   {
     if { $State(Sinad.peakEnable) == 1 } \
     {
       after idle "Sinad peak 0 -nowait"
     }
  
     if { $State(Sinad.peakEnable) == 0 } \
     {
       after idle "Sinad phreset  -nowait"
       after idle "Sinad peak 1   -nowait"
     }
   }

   return    
 }

#********************************************************************
#
#       Name: setAverage
#
# Parameters: new_value   - requested new value 
#                           -1 - no new value requested
#
# Description: set the meter average
#
#********************************************************************
proc setAverage { meterType } \
 {
   set new_value  $::Sinad::State(filter)
      
   # Set the meter engine
   after idle "Sinad  filter  $new_value  -nowait"

   if { $new_value == 1 } \
   {
     after idle "Sinad math 0"
   } \
   else \
   {
     after idle "Sinad math 1"
   }

   return
 }


#********************************************************************
#
#       Name: toggleCWeight
#
# Parameters: 
#
# Description: set the meter C Weight filter
#
#********************************************************************
proc toggleCWeight { meterType } \
 {
   #puts "set_meter_c_weight:  $new_value"

   set new_value  $::Sinad::State(CWTenabled)
     
   if { $new_value == "ON" } \
   {
     set new_value  "OFF"
   } \
   else \
   {
     set new_value  "ON"
   }
     
   # Set the meter engine
   after idle "Sinad  encwt  $new_value  -nowait"
     
   return
     
 }


#****************************************************************************
# Name:        query_sinad_state
# Parameters:  none
# Description: This function sends down a request to the Sinad engine to 
#            : obtain the engine state 
#****************************************************************************
 proc query_sinad_state { } {
   variable State
   Sinad run  
   Sinad math 
   Sinad avg
   Sinad en
   Sinad trig
   Sinad filter
   Sinad ulim
   Sinad llim
   Sinad peak
   Sinad alerts
   Sinad encwt
   Sinad log

   Sinad rate  
   Sinad minx
   Sinad maxx
   Sinad miny
   Sinad maxy
   Sinad tavg_factor
   Sinad size
   Sinad alpha
   Sinad window
   Sinad lnotch
   Sinad hnotch
   Sinad lnoise
   Sinad hnoise
   
   Dist ulim
   Dist llim
   Dist peak
   Dist alerts

   set State(engineId) [format_version [Sinad ver 0] ]
   set State(driverId) [format_version [Sinad ver 1] ]
   
   if [IFR_options is_installed Audio_Analyzer] \
   {
      Sinad anlz_trig
      for {set i 0} { $i < 10} { incr i } { Sinad marker $i}
   }
 }

#****************************************************************************
# Name:        getSaveState
# Parameters:  none
# Description: This function returns a list of TCL commands which can be used
#              to set the Sinad engine to its current state
#****************************************************************************
 proc getSaveState { } {
   variable State 
   
   set returnList \
   {
     Sinad run     $State(Engine_run)       -nowait
     Sinad math    $State(Engine_math)      -nowait
     Sinad trig    $State(Engine_trigger)   -nowait
     Sinad avg     $State(average)          -nowait
     Sinad filter  $State(filter)           -nowait
     Sinad en      $State(Engine_enabled)   -nowait
     Sinad peak    $State(Sinad.peakEnable) -nowait
     Sinad alerts  $State(Sinad.alerts)     -nowait
     Sinad ulim    $State(Sinad.alertHi)    -nowait
     Sinad llim    $State(Sinad.alertLo)    -nowait
     Sinad encwt   $State(CWTenabled)       -nowait
     Sinad rate    $State(rate)             -nowait
     Sinad minx    $State(minx)             -nowait
     Sinad maxx    $State(maxx)             -nowait
     Sinad rangey  $State(rangey)           -nowait
     Sinad maxy    $State(maxy)             -nowait
     Sinad size    $State(size)             -nowait
     Sinad alpha   $State(alpha)            -nowait
     Sinad window  $State(window)           -nowait
     Sinad lnoise  $State(lnoise)           -nowait
     Sinad hnoise  $State(hnoise)           -nowait
     Sinad lnotch  $State(lnotch)           -nowait
     Sinad hnotch  $State(hnotch)           -nowait
     
     catch {Sinad log $State(log_mode) -nowait }
     catch {
        Sinad tmmode $State(tmath,mode,raw) $State(tmath,offset,raw) $State(tmath,trace_a) $State(tmath,trace_b) -nowait
     }
     
     Dist peak    $State(Dist.peakEnable)  -nowait
     Dist alerts  $State(Dist.alerts)      -nowait
     Dist ulim    $State(Dist.alertHi)     -nowait
     Dist llim    $State(Dist.alertLo)     -nowait

     set ::Sinad::State(Sinad.BellEnabled)        {$State(Sinad.BellEnabled)}
     set ::Sinad::State(Sinad.BellEnabled_text)   {$State(Sinad.BellEnabled_text)}
     set ::Sinad::State(Sinad.resolution)         {$State(Sinad.resolution)}
     set ::Sinad::State(Dist.BellEnabled)         {$State(Dist.BellEnabled)}
     set ::Sinad::State(Dist.BellEnabled_text)    {$State(Dist.BellEnabled_text)}
     set ::Sinad::State(Dist.resolution)          {$State(Dist.resolution)}
     
     Sinad::Input {$State(input)}
    
  }
  append returnList { if [IFR_options is_installed Audio_Analyzer] \{}
  lappend statevars [concat [array get State marker,*,*] \
                            [array get State trace_state,*]
                    ]

  append returnList \
  {
     catch { Sinad anlz_trig   $State(anlz_trig,raw) -nowait }
     catch { Sinad tavg_factor $State(tavg_factor)   -nowait }
     catch { array set ::Sinad::State $statevars}
     set ::Sinad::State(scale) {$::Sinad::State(scale)}
     
  }
  if [IFR_options is_installed Audio_Analyzer] \
  {
  
     if ![interp issafe] \
     {
        append returnList \
        {
           set Sinad::State(tracking_gen) $State(tracking_gen)
        }
     }
	  foreach trace {tref tulim tllim } \
	  {
	    set get_traces \
	    {
	      set x {[Sinad trace $trace]}
	      Sinad trace  $trace x
	      unset x
	    }
	    append returnList [subst $get_traces]
	  }
	  set x \
	  {
	     Sinad tlimit_enable [Sinad tlimit_enable]
	  }
	  append returnList [subst $x]
  }
  for { set i 0 } { $i < 10 } { incr i } \
  {
    append returnList "        catch {Sinad marker $i $State(marker,$i,freq)}\n"
  }
  append returnList {    \}}

  return [subst -nocommands $returnList]
}

#****************************************************************************
# Name:        Sync
# Parameters:  none
# Description: Write state variables to hardware
#****************************************************************************
 proc Sync { } {
    eval [getSaveState]
 }

#****************************************************************************
# Name:        DefaultSetup
# Parameters:  none
# Description: Sets the Sinad engine with default values
#****************************************************************************
 proc DefaultSetup { } {
    Sinad trig CONTINUOUS       ;# Set the meter for continuous readings
    Sinad math OFF              ;# Enable filtering
    Sinad avg 1                 ;# Set number of symbols to average over
    Sinad filter 15             ;# Set initial filter at 15
    Sinad debug lnotch 900.0    ;# Set the low notch frequency
    Sinad debug hnotch 1100.0   ;# Set the low notch frequency
    Sinad encwt 0               ;# Initially disable the C-Weight filter
 }
  #********************************************************************
  #
  #       Name: Input
  #
  # Parameters: new input source
  #
  # Description: sets meter input
  #
  #********************************************************************
  proc Input { { new_value "" } } \
  {
    #puts "set_meter_input:  $new_value"

    variable  State

    if { $new_value != "" } \
    {
      set State(input)  $new_value
    } 
      
    switch $State(input) \
    {
      "DEMOD"   { BBAudio route L1 MSINAD }
    
      "SQLCH"   { BBAudio route SQUELCHED MSINAD }
      
      "MIC dyn" { BBAudio route MIC MSINAD
                  BBAudio micbias Normal
                }

      "MIC elec" { BBAudio route MIC MSINAD
                   BBAudio micbias Powered
                }
    
      
      "AUDIO"   { BBAudio route SINAD MSINAD
                  BBAudio sinadbalance UNBALANCED
                }
              
      
      "Aud Bal"  { BBAudio route SINAD MSINAD
                   BBAudio sinadbalance BALANCED
                 }
      
      "FILE"    { BBAudio route FILE MSINAD}
      
      default { BBAudio route SINAD MSINAD
                BBAudio sinadbalance BALANCED
              }
    }
  }   

  #****************************************************************************
  # Name:        Track_input
  # Parameters:  args per trace variable
  # Description: Updates the engine's idea of where our input is routed
  #****************************************************************************
  proc Track_input {args } \
  {
     set cmd "::Sinad::Track_input_core"
     after cancel $cmd
     after idle $cmd
  }
  proc Track_input_core { } \
  {
     variable State
     
     switch -- $::BBAudio::State(Route_16) \
     {
        0 { set State(input) DEMOD }
        23 {set State(input) "SQLCH" } 
        1 { 
           if {$::BBAudio::State(micbias) == 0 } \
           { 
             set State(input)  "MIC dyn"
           } \
           else \
           {
             set State(input)  "MIC elec"
           }
          }
        2 { 
           if {$::BBAudio::State(sinadbal) == 0 } \
           { 
             set State(input)  "Aud Bal"
           } \
           else \
           {
             set State(input)  "AUDIO"
           }
          }
        3 { set State(input) FILE }
     }
  }

#****************************************************************************
# Name:        SyncAll
# Parameters:  none
# Description: Insures that TCL, the engine and hardware are in sync
#****************************************************************************
 proc SyncAll { } {
    set_sinad_callbacks
    query_sinad_state
    if ![interp issafe] \
    {
       Sync
       DefaultSetup
       after 100 "Sinad sendcwt $::Sinad::defaultCWT"
    }
 }

 
#****************************************************************************
# Name:        X_to_freq
# Parameters:  widget - trace widget name
#              x      - position within that widget
# Description: Given a trace widget and an offset in pixels from the left
#              edge of that widget, compute the frequency, ASSUMING that
#              widget really is showing the FFT trace
#****************************************************************************
 proc X_to_freq { widget  x } \
 {
   variable State
   
   if { ![winfo exists $widget] } { return 0 }
   
   if { [winfo class $widget] != "TraceWidget" } { return 0 }
   
   set width  [$widget cget -width]
   
   set width  [expr double($width)]
   set maxx   [expr double($State(maxx))]
   set minx   [expr double($State(minx))]
   
   set freq  [expr ( ( ( $maxx - $minx ) * $x ) / $width ) + $minx]
   
   return  $freq
 }
 
#****************************************************************************
# Name:        Marker_freq_to_x
# Parameters:  widget - trace widget path
#              marker #
# Description: Given a trace widget and a marker #, compute the offset in 
#              pixels from the left edge of that widget, ASSUMING that
#              widget really is showing the FFT trace
#****************************************************************************
 proc Marker_freq_to_x { widget marker } \
 {
   variable State
   if { ![winfo exists $widget] } { return 0 }
    
   if { [winfo class $widget] != "TraceWidget" } { return 0 }
    
   set width  [$widget cget -width]
   return [expr $width * $State(marker,$marker,pos)]
 }
 
#****************************************************************************
# Name:        freq_to_X
# Parameters:  widget - trace widget path
#              freq   - frequency value
# Description: Given a trace widget and a frequency, compute the offset in 
#              pixels from the left edge of that widget, ASSUMING that
#              widget really is showing the FFT trace
#****************************************************************************
 proc freq_to_X { widget  freq } \
 {
   variable State
    
   if { ![winfo exists $widget] } { return 0 }
    
   if { [winfo class $widget] != "TraceWidget" } { return 0 }
    
   set width  [$widget cget -width]
   
   # Convert to 'double' values for the ratio calculations
   if { $freq == "" } \
   {
     set freq  0.0
   } \
   else \
   {
     set freq   [expr double($freq)]
   }

   set width  [expr double($width)]
   set minx   [expr double($State(minx))]
   set maxx   [expr double($State(maxx))]

   set x  [expr (($freq - $minx)/($maxx - $minx)) * $width]
   
   set x  [expr int($x)]
   
   # Limit the movement
   if { $x < 0 } \
   {
     set x  0
   }
    
   if { $x > $width } \
   {
     set x  $width
   }

   return  $x
 }

#****************************************************************************
# Name:        Y_to_db
# Parameters:  Trace widget name, position within that widget
# Description: Given a trace widget and an offset in pixels from the top
#              edge of that widget, compute the level, ASSUMING that
#              widget really is showing the FFT trace
#****************************************************************************
 proc Y_to_db { widget  y } \
 {
   variable State
   
   if { ![winfo exists $widget] } { return 0 }
   
   if { [winfo class $widget] != "TraceWidget" } { return 0 }
   
   set height  [$widget cget -height]
   
   set height  [expr double($height)]
   set maxy    [expr double($State(maxy))]
   set miny    [expr double($State(miny))]

   set db  [expr $maxy - ( ( ( $maxy - $miny ) * $y ) / $height )]
   
   return  $db
 }


#****************************************************************************
# Name:        db_to_Y
# Parameters:  widget - trace widget name
#              db     - magnitude value
# Description: Given a trace widget and a level, compute the offset in 
#              pixels from the top edge of that widget, ASSUMING that
#              widget really is showing the FFT trace
#****************************************************************************
 proc db_to_Y { widget  db } \
 {
   variable State
    
   if { ![winfo exists $widget] } { return 0 }
    
   if { [winfo class $widget] != "TraceWidget" } { return 0 }
    
   set height  [$widget cget -height]
   
   if { $db == "" } \
   {
     set db  0.0
   } \
   else \
   {
     set db   [expr double($db)]
   }

   set height  [expr double($height)]
   set maxy    [expr double($State(maxy))]
   set miny    [expr double($State(miny))]
    
   set y  [expr ( ( $maxy - $db ) / ( $maxy - $miny ) ) * $height]
   
   set y  [expr int($y)]

   # Limit the movement
   if { $y < 0 } \
   {
     set y  0
   }
    
   if { $y > $height } \
   {
     set y  $height
   }

   return  $y
 }

#****************************************************************************
# Name:        Get_HTML_reading
# Parameters:  base_url - the base URL for the containing page
# Description: Return an string containing HTML showing the current state
#              of this system
#****************************************************************************
 proc Get_HTML_reading { {base_url ""} } \
 {
    variable State
    set x \
{<tr>
<td nowrap>$State(input) Distortion</td><td nowrap>$State(distortion)%</td>
</tr>
<tr>
<td nowrap>$State(input) SINAD</td><td nowrap>$State(sinad) dB</td></tr>
</tr>}
    return [subst $x]
 }
 
#****************************************************************************
# Initialize the namespace and activate the callbacks
#****************************************************************************
   Load::on_system_done  { ::Sinad::SyncAll }
   Alert::AddMeter Sinad
}  ;# End of the Sinad namespace

#****************************************************************************
# Beginning to split Distortion into its own namespace...
#****************************************************************************
namespace eval ::Distortion \
{
   variable State
   
   array set State \
   {
      alerts 0
      alertHi_text      ""
      alertLo_text      ""
      alarm_text        ""
      alarm             ""
      BellEnabled       0
      BellEnabled_text  "OFF"
   }

   variable name "Distortion"
   
#********************************************************************
#
#       Name: toggleAlarm
#
# Parameters:
#
# Decsription: Toggles the state of the alarm
#
#********************************************************************
proc toggleAlarm { } \
{
  variable State
  Dist alerts [expr $State(alerts)^4]
}

#****************************************************************************
# Name:        updateAlerts
# Parameters:  
# Description: Update the Alerts flags
#****************************************************************************
 proc updateAlerts { alerts } \
 {
   variable State
   
   set State(alerts)  $alerts
 
   if {$alerts & 2} \
   {
     set State(alertHi_text) "ON"
   } \
   else \
   {
     set State(alertHi_text) "OFF"
   }
   if {$alerts & 1} \
   {
     set State(alertLo_text) "ON"
   } \
   else \
   {
     set State(alertLo_text) "OFF"
   }
   if {$alerts & 4} \
   {
     set State(alarm_text)        "ON"
     set State(BellEnabled_text)  "ON"
     
     set State(BellEnabled)  1
   } \
   else \
   {
     set State(alarm_text)        "OFF"
     set State(BellEnabled_text)  "OFF"

     set State(BellEnabled)  0
   }
   return
 }

#****************************************************************************
# Name:        updateReading
# Parameters:  readingList      List of data from the engine
#                Sinad          Sinad reading
#                Sinad prec.    Precision of sinad
#                Distortion     Distortion reading
#                Dist prec.     Precision of distortion
#                Sig power      Signal power
#                Noise power    
#                Status
# Description: Sets up the current state of the Sinad reading
#****************************************************************************
 proc updateReading { readingList } {

   variable State
   set State(reading)   [format "%3.2f" [lindex $readingList 0] ]
   set status           [lindex $readingList 2]
   set State(Status) $status
   if {$status & 0x2000} \
   {
      set State(alarm) "ACTIVE"
   } \
   else \
   {
      set State(alarm) "inactive"
   }
 }
 Alert::AddMeter Distortion
 
 Dist -onchange reading   { ::Sinad::updateReadingDist %V}
 Dist -onchange readpeak   { ::Sinad::updatePeaks   "Dist" %V}
 Dist -onchange ulimit     { ::Sinad::updateAlertHi "Dist" %V}
 Dist -onchange llimit     { ::Sinad::updateAlertLo "Dist" %V}
 Dist -onchange alerts     { ::Sinad::updateAlerts  "Dist" %V}

 Dist -onchange peak       { ::Sinad::updatePeakEnable "Dist" %V}
 Dist -onchange alerts     { ::Distortion::updateAlerts  %V }

}
#****************************************************************************
# This registers the engine with the list for saving instrument states
#****************************************************************************
namespace eval ::SavedStates {

   variable engines   
   lappend engines Sinad
}

#****************************************************************************
# Define the Audio_Analyzer namespace, and set up our HTML option info
#****************************************************************************
namespace eval ::Audio_Analyzer \
{
   set Audio_version [string trimleft [Sinad version 0] "SinadMtr"]
   set InfoString [subst \
   {
         <br>
         (using Sinad system version - $Audio_version)
         <br>
         This option provides an FFT-based audio spectrum analyzer, capable
         of analyzing baseband or demodulated audio, with resolutions down
         to .25 Hz (depending upon the maximum frequency displayed.)
   }]
}

#****************************************************************************
# This registers the engine with the list for generating an HTML page
#****************************************************************************
namespace eval ::HTTPD \
{
   variable main_screen_readings
   lappend  main_screen_readings(1) Sinad
}

#****************************************************************************
# This registers the engine with the list for the version screen
#****************************************************************************
namespace eval ::Version_Screen \
{
   variable version
   variable names
   
   array set names \
   {
      Sinad_Eng "SINAD Engine"
      Sinad_Drv "SINAD Driver"
   }
   set version(Sinad_Eng) [Sinad version 0]
   set version(Sinad_Drv) [Sinad version 1]
}
