#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C) 2001, 2002   DESIGN ENGINEERING     #
# --------------------------------------------------------------------------#
# FILE        : SzSnet.tcl                                                  #
# PRODUCT     : FITS                                                        #
# AUTHOR      : Bradford Taylor                                             #
# DESCRIPTION : This file is sourced by TCL when the SzSnet command is      #
#             : first used.  It loads the C++ interface file, provides a    #
#             : namespace for screen variables, and sets up callbacks to    #
#             : update the variables for the UI.                            #
#                                                                           #
#############################################################################

#****************************************************************************
# Handle a load of the C++ interface
#****************************************************************************
# Call the base class dummy command to force a load
if {"[info command SzSnet]" != "" } { return }
package require Engtclbase
Engtclbase

# Load the object file
::load $::SzSnet::object

package provide SzSnet 1.0

#***************************************************************************
#  Add functions to SzSnet namespace                                       *
#***************************************************************************

namespace eval ::SzSnet {
  variable SaveSortOrder FIRST
  variable conn_tones
  variable Avail_Bands

  array set conn_tones \
  {
    105  0
     76  1
     83  2
     90  3
     97  4
    116  5
    128  6
    138  7

    0  105
    1   76
    2   83
    3   90
    4   97
    5  116
    6  128
    7  138
  }
  
  array set Avail_Bands \
  {
    0  "800"
    1  "800 SPLNT"
    2  "VHF/UHF"
    3  "900"
    4  "USER"
    5  "800 REBND"
  }
  

#***************************************************************************
#        Name:  set_szsnet_callbacks
#  Parameters:   none
# Description:  Sets up the callbacks to update the SzSnet data
#***************************************************************************
proc set_szsnet_callbacks {} {

  if { $::SzSnet::callbackStatus != 0 } \
  {
    return
  }
  
  SzSnet -onchange reading     { ::SzSnet::updateReading %V}
  
  # Added for the Scanner tile
  SzSnet -onchange currentvx   { set ::SzSnet::State(voice_chan)  %V}
  SzSnet -onchange onvx        { set ::SzSnet::State(onvx)  %V}
  SzSnet -onchange lsdvx       { set ::SzSnet::State(lsd)  [format "0x%03X" %V]; }
  SzSnet -onchange baudvx      { set ::SzSnet::State(baud)  %V}
  SzSnet -onchange found       { set ::SzSnet::State(found)  %V}
  SzSnet -onchange foundgrp    { set ::SzSnet::State(talk_group)  %V}
  SzSnet -onchange scanchan    { set ::SzSnet::State(control_chan)  %V}

  SzSnet -onchange tbits       { SzSnet::updateTBits %V}

  SzSnet -onchange sysid       { SzSnet::updateSystemId %V}
  SzSnet -onchange group       { SzSnet::updateGroupId  %V}
  SzSnet -onchange unitid      { SzSnet::updateUnitId  %V}

  SzSnet -onchange follow      { SzSnet::updateScanOnOff    %V}
  SzSnet -onchange foundid     { SzSnet::updateFoundID      %V}
  SzSnet -onchange followgroup { set SzSnet::State(grp_id) %V}
  SzSnet -onchange followid    { set SzSnet::State(unit_id) %V}
  SzSnet -onchange control     { SzSnet::update_CCTx_freq  %V}
  
  SzSnet -onchange foundsysid  { SzSnet::updateFoundSysID %V}

  
  # Added for the Repeat Sim tile
  SzSnet -onchange uutptt      { SzSnet::updateUUTPTT       %V}
  SzSnet -onchange uutaff      { SzSnet::updateUUTAffil     %V}
  SzSnet -onchange getcnx      { SzSnet::updateCNx          %V}
  SzSnet -onchange ptt         { SzSnet::updatePTT          %V}
  SzSnet -onchange digital     { SzSnet::updateCallType     %V}
  SzSnet -onchange cnxtone     { SzSnet::updateConnectTone  %V}
  SzSnet -onchange simcx       { SzSnet::updateSimOnOff     %V}
  
  SzSnet -onchange uhfcctx     { SzSnet::update_CCTx_freq  %V}
  SzSnet -onchange uhfccrx     { SzSnet::update_CCRx_freq  %V}
  SzSnet -onchange uhfvctx     { SzSnet::update_VCTx_freq  %V}
  SzSnet -onchange uhfvcrx     { SzSnet::update_VCRx_freq  %V}

  SzSnet -onchange voice       { SzSnet::update_VCTx_freq  %V}

  SzSnet -onchange aff         { set ::SzSnet::State(affil) %V}
  SzSnet -onchange band        { SzSnet::setBandText %V}
  
  set ::SzSnet::callbackStatus 1
  
}

#***************************************************************************
#        Name:  Init
#  Parameters:
# Description:  procedure to call to load the namespace
#***************************************************************************
proc Init { } \
{

}

#***************************************************************************
#        Name:  setBandText
#  Parameters:  parmList 	Data returned by the engine
# Description:  procedure to call to load the namespace
#***************************************************************************
proc setBandText { parmList } \
{
   variable  Avail_Bands
   variable  State

    set band [lindex $parmList 0]
    if [info exists Avail_Bands($band)] \
    {
       set State(band) $band 
       set State(band_text) $Avail_Bands($band)
    }

   # Setup the Control and voice channels when the band changes
   setChannels
}



#***************************************************************************
#        Name:  updateReading       
#  Parameters:   The layer 1 SmartZone/SmartNet data packet
# Description:  Updates the state parametets for SmartZone/SmartNet
#***************************************************************************
proc updateReading {readingList} {

   set rl $readingList

   set ::SzSnet::rlist $rl
   # Get the data from the packet
   set ::SzSnet::State(status)          [lindex $rl 0]
   set ::SzSnet::State(word)            [format "0x%08X" [lindex $rl 1]]
   set ::SzSnet::State(parity)          [format "0x%03X" [lindex $rl 2]]
   set ::SzSnet::State(corrections)     [lindex $rl 3]
   set ::SzSnet::State(dc_ave)          [lindex $rl 4]
   set ::SzSnet::State(standard_dev)    [lindex $rl 5]
   set ::SzSnet::State(symbolrate)      [lindex $rl 6]
   set ::SzSnet::State(baudrate)        [lindex $rl 7]
   set ::SzSnet::State(timestamp)       [lindex $rl 8]
   set ::SzSnet::State(id)              [format "0x%04X" [lindex $rl 9]]
   set ::SzSnet::State(i_bit)           [lindex $rl 10]
   set ::SzSnet::State(cmd)             [format "0x%03X" [lindex $rl 11]]
   set ::SzSnet::State(talkgroup)       [format "0x%03X" [lindex $rl 12]]
   
   set ::SzSnet::State(t_bits)          [lindex $rl 13]
   
   updateTBits  $::SzSnet::State(t_bits)
   
   set ::SzSnet::State(channel)         [lindex $rl 14]
   set ::SzSnet::State(type)            [lindex $rl 15]
   
   set ::SzSnet::State(inverted)        [expr $::SzSnet::State(status) & 0x1]

   set type          $::SzSnet::State(type)
   set monitor_type  $::SzSnet::State(monitor_type)
   
   if { $type == $monitor_type } \
   {
     set ::SzSnet::State(mon_word)          [format "0x%08X" $::SzSnet::State(word)]
     set ::SzSnet::State(mon_parity)        [format "0x%03X" $::SzSnet::State(parity)]
     set ::SzSnet::State(mon_corrections)   $::SzSnet::State(corrections)
     set ::SzSnet::State(mon_inverted)      $::SzSnet::State(inverted)
     set ::SzSnet::State(mon_timestamp)     $::SzSnet::State(timestamp)
     set ::SzSnet::State(mon_dc_ave)        [format %.5f $::SzSnet::State(dc_ave)]
     set ::SzSnet::State(mon_standard_dev)  [format %.5f $::SzSnet::State(standard_dev)]
     set ::SzSnet::State(mon_symbolrate)    $::SzSnet::State(symbolrate)
   }
   
}


# show data word
proc showWord {} {
  #puts  $::SzSnet::rlist
  #puts "ID = $::SzSnet::State(id) CMD = $::SzSnet::State(cmd) Channel = $::SzSnet::State(channel)";
  #puts "I = $::SzSnet::State(i_bit) TALK = $::SzSnet::State(talkgroup) Tbits = $::SzSnet::State(t_bits)";
  #puts "Type = $::SzSnet::State(type)  Ts = $::SzSnet::State(timestamp)"
}



#***************************************************************************
#
#        Name:  updateSystemId
#
#  Parameters:
#
# Description:  Updates the System ID decimal and hex values
#
#***************************************************************************
proc updateSystemId { sys_id } \
 {
   #puts "updateSystemId:  $sys_id"
   
   variable  State

   set State(sys_id)  $sys_id
   
   set State(sys_id_hex)  [format "%X"  $sys_id]

   return
 }


#***************************************************************************
#
#        Name:  updateGroupId
#
#  Parameters:
#
# Description:  Updates the Group ID decimal and hex values
#
#***************************************************************************
proc updateGroupId { grp_id } \
 {
   #puts "updateGroupId:  $grp_id"

   variable  State

   set State(grp_id)  $grp_id
   
   set State(grp_id_hex)  [format "%X"  $grp_id]

   return
 }


#***************************************************************************
#
#        Name:  updateUnitId
#
#  Parameters:
#
# Description:  Updates the Unit ID decimal and hex values
#
#***************************************************************************
proc updateUnitId { unit_id } \
 {
   #puts "updateUnitId:  $unit_id"

   variable  State

   set State(unit_id)  $unit_id
   
   set State(unit_id_hex)  [format "%X"  $unit_id]

   return
 }


#***************************************************************************
#
#        Name:  updateScanOnOff       
#
#  Parameters:
#
# Description:  Updates the Scanner ON / OFF
#
#***************************************************************************
proc updateScanOnOff { scan_on_off } \
 {
   variable  State

   if { $scan_on_off == 0 } \
   {
     set State(scan_on_off)       0
     set State(scan_on_off_text)  "OFF"

     # Clear the 'ACTIVE' indicator
     set ::SzSnet::State(onvx)  0
   }

   if { $scan_on_off == 1 } \
   {
     set State(scan_on_off)       1
     set State(scan_on_off_text)  "ON"

     if { ![interp issafe] } \
     {
       after idle "::SzSnet::After_Scan_turn_on"
     }
  }

   return
 }

 proc After_Scan_turn_on { } \
 {
   variable  State
   
   # Setup the Receiver
   after idle "Receiver rfin         ANT       -nowait"
   after idle "Receiver atten        0         -nowait"
   
   #after idle "Receiver demod        fmmedium  -nowait"
   
   after idle "Receiver hwagc        1         -nowait"
   after idle "Receiver agcmode      1         -nowait"
   after idle "Receiver squelchmode  AUTO      -nowait"

   return
 }


#***************************************************************************
#        Name:  updateFoundID       
#  Parameters:   The layer 1 SmartZone/SmartNet data packet
# Description:  Updates the Individual ID parameter for SmartZone/SmartNet
#***************************************************************************
proc updateFoundID { ind_id } \
 {
   variable  State

   if { $ind_id == -1 } \
   {
     set State(ind_id)  ""
   } \
   else \
   {
     set State(ind_id)  $ind_id
   }

 }


#***************************************************************************
#        Name:  updateUUTPTT       
#  Parameters:   The layer 1 SmartZone/SmartNet data packet
# Description:  Updates the UUT PTT parameter for SmartZone/SmartNet
#***************************************************************************
proc updateUUTPTT { uutptt } \
 {
   #puts "updateUUTPTT:  $uutptt"
   
   variable  State
  
   if { $uutptt == -1 } \
   {
     set State(uutptt)  ""
   } \
   else \
   {
     set State(uutptt)  $uutptt
     
     if { $State(uutptt) == 0 } \
     {
       set State(conn_tone_freq)  ""
     }
   }
   
   return
 }


#***************************************************************************
#        Name:  updateUUTAffil       
#  Parameters:   The layer 1 SmartZone/SmartNet data packet
# Description:  Updates the UUT Affiliation parameter for SmartZone/SmartNet
#***************************************************************************
proc updateUUTAffil { uutaff } \
 {
   #puts "updateUUTAffil:  $uutaff"
   
   variable  State

   if { $uutaff == -1 } \
   {
     set State(uutaff)  ""
   } \
   else \
   {
     if { $uutaff == 1 } \
     {
       set State(uutaff)  $uutaff
     
       after 5000 "set ::SzSnet::State(uutaff)  0"
     }
   }
   
   return
 }


#***************************************************************************
#        Name:  updateCNx       
#  Parameters:   The layer 1 SmartZone/SmartNet data packet
# Description:  Updates the UUT Connect Tone parameter for SmartZone/SmartNet
#***************************************************************************
proc updateCNx { cnx } \
 {
   #puts "updateCNx:  $cnx"
   
   variable  State
   
   set State(cnx)  $cnx
  
   set State(conn_tone_freq)  [expr floor($State(cnx))]  
   
   return
 }


#***************************************************************************
#
#        Name: updatePTT
#
#  Parameters:
#
# Description: Updates the PTT
#
#***************************************************************************
proc updatePTT { ptt } \
 {
   #puts "updatePTT:  $ptt"
  
   variable  State
  
   switch -- $ptt \
   {
     0  { set State(ptt_on_off)  0
          set State(ptt_on_off_text)  "PTT OFF" }
     
     1  { set State(ptt_on_off)  1
          set State(ptt_on_off_text)  "PTT ON" }
   }

   return
 }


#***************************************************************************
#
#        Name: updateCallType
#
#  Parameters:
#
# Description: Updates the call type:  ANALOG or DIGITAL
#
#***************************************************************************
proc updateCallType { call_type } \
 {
   #puts "updateCallType:  $call_type"
   
   variable  State
   
   switch -- $call_type \
   {
     0  { set State(call_type)  0
          set State(call_type_text)  "ANALOG" }

     1  { set State(call_type)  1
          set State(call_type_text)  "DIGITAL" }
   }

   return
 }


#***************************************************************************
#
#        Name: updateConnectTone
#
#  Parameters:
#
# Description: Sets the Connection Tone (0 - 7)
#
#***************************************************************************
proc updateConnectTone { conn_tone_val } \
 {
   #puts "updateConnectTone"
   
   variable  State
   variable  conn_tones

   set State(conn_tone)  $conn_tones($conn_tone_val)
  
   return
 }


#***************************************************************************
#
#        Name: updateSimOnOff
#
#  Parameters:
#
# Description: Updates the Simulate Control value 
#
#***************************************************************************
proc updateSimOnOff { sim_on_off } \
 {
   Debug::Puts SZSNET 3 "updateSimOnOff:  $sim_on_off"
  
   variable  State
  
   # Reset the AFFIL and PTT state
   set State(uutaff)  0
   set State(uutptt)  0   

   if { $sim_on_off == 0 } \
   {
     set State(sim_on_off)  0
     set State(sim_on_off_text)  "OFF"
     
     if ![interp issafe ] \
     {
        after idle "::SzSnet::After_Sim_turn_off"
     }
   }

   if { $sim_on_off == 4 } \
   {
     set State(sim_on_off)  1
     set State(sim_on_off_text)  "ON"

     if ![interp issafe ] \
     {
        after idle "::SzSnet::After_Sim_turn_on"
     }
   }

   return
 } 
 proc After_Sim_turn_off { } \
 {

 }
 proc After_Sim_turn_on { } \
 {
   Receiver filter 0 -nowait

   Generator mod  fmn

   Generator ptt  ON
 }



#***************************************************************************
#
#        Name: update_CCTx_freq
#
#  Parameters:
#
# Description: Updates the CCTx frequency for all bands
#
#***************************************************************************
proc update_CCTx_freq { { control_chan  "" } } \
 {
   #puts "update_CCTx_freq:  $control_chan"
  
   variable  State
   
   #puts "State(band_text):  $State(band_text)"
   
   switch -- $State(band_text) \
   {
     "800"        -
     "800 REBND"  -
     "800 SPLNT"  { if { $control_chan != "" } \
                    {
                      set State(control_chan)   $control_chan
                      set State(cctx_chan,800)  $control_chan
                    }
                    set State(cctx_freq,800)  [SzSnet txfreq  $State(cctx_chan,800) -forgive]
                    set State(cctx_freq,800)  [format "%.4f" $State(cctx_freq,800)]
                  }
    
     "900"        { if { $control_chan != "" } \
                    {
                      set State(control_chan)   $control_chan
                      set State(cctx_chan,900)  $control_chan
                    }
                    set State(cctx_freq,900)  [SzSnet txfreq  $State(cctx_chan,900) -forgive]
                    set State(cctx_freq,900)  [format "%.4f" $State(cctx_freq,900)]
                  }
    
     "VHF/UHF"    { if { $control_chan != "" } \
                    {
                      set State(cctx_chan,UV)  $control_chan
                    }
                    set State(cctx_freq,UV)  [SzSnet uhftxfreq  $State(cctx_chan,UV) -forgive]
                    set State(cctx_freq,UV)  [format "%.4f" $State(cctx_freq,UV)]
                  }

     "USER"       { if { $control_chan != "" } \
                    {
                      set State(control_chan)    $control_chan
     
                      set State(cctx_chan,USER)  $control_chan
                    }
                    
                    # Use TX channel
                    # TX and RX channels are the same
                    set chan  $State(cctx_chan,USER)
                    
                    #puts "State(cctx_chan,USER):  $State(cctx_chan,USER)"
                    #puts "chan:  $chan"
                    
                    # If the USER TX frequency is undefined for this channel, 
                    # initialize it to 100.0
                    if { ![info exists  ::SzSnet::State(cctx_freq,USER,$chan)] } \
                    {
                      set State(cctx_freq,USER,$chan)  100.0000
                    }
                     
                    #puts "State(cctx_freq,USER,$chan):  $State(cctx_freq,USER,$chan)"
                    
                    # Set and display this USER TX frequency value
                    set State(cctx_freq,USER)  [format "%.4f" $State(cctx_freq,USER,$chan)]

                    # If the USER RX frequency is undefined for this channel, 
                    # initialize it to 100.0
                    if { ![info exists  ::SzSnet::State(ccrx_freq,USER,$chan)] } \
                    {
                      set State(ccrx_freq,USER,$chan)  100.0000
                    }
                     
                    #puts "State(ccrx_freq,USER,$chan):  $State(ccrx_freq,USER,$chan)"

                    # Set and display this USER RX frequency value
                    set State(ccrx_freq,USER)  [format "%.4f" $State(ccrx_freq,USER,$chan)]
                  }
   }

   return
 }


#***************************************************************************
#
#        Name: update_CCRx_freq
#
#  Parameters:
#
# Description: Updates the CCRx frequency for all bands
#
#***************************************************************************
proc update_CCRx_freq { { control_chan  "" } } \
 {
   #puts "update_CCRx_freq:  $control_chan"
  
   variable  State
   
   #puts "State(band_text):  $State(band_text)"

   switch -- $State(band_text) \
   {
     "800"        -
     "800 REBND"  -
     "800 SPLNT"  { }
    
     "900"        { }
    
     "VHF/UHF"    { if { $control_chan != "" } \
                    {
                      set State(ccrx_chan,UV)  $control_chan
                    }
                    set State(ccrx_freq,UV)  [SzSnet uhfrxfreq  $State(ccrx_chan,UV) -forgive]
                    set State(ccrx_freq,UV)  [format "%.4f" $State(ccrx_freq,UV)]
                  }

     "USER"       { }
   }

   return
 }


#***************************************************************************
#
#        Name: update_VCTx_freq
#
#  Parameters:
#
# Description: Updates the VCTx frequency for all bands
#
#***************************************************************************
proc update_VCTx_freq { { voice_chan  "" } } \
 {
   #puts "update_VCTx_freq:  $voice_chan"
  
   variable  State
   
   #puts "State(band_text):  $State(band_text)"

   switch -- $State(band_text) \
   {
     "800"        -
     "800 REBND"  -
     "800 SPLNT"  { if { $voice_chan != "" } \
                    {
                      set State(voice_chan)     $voice_chan
                      set State(vctx_chan,800)  $voice_chan
                    }
                    set State(vctx_freq,800)  [SzSnet txfreq  $State(vctx_chan,800) -forgive]
                    set State(vctx_freq,800)  [format "%.4f" $State(vctx_freq,800)]
                  }

     "900"        { if { $voice_chan != "" } \
                    {
                      set State(voice_chan)     $voice_chan
                      set State(vctx_chan,900)  $voice_chan
                    }
                    
                    set State(vctx_freq,900)  [SzSnet txfreq  $State(vctx_chan,900) -forgive]
                    set State(vctx_freq,900)  [format "%.4f" $State(vctx_freq,900)]
                  }

     "VHF/UHF"    { if { $voice_chan != "" } \
                    {
                      set State(vctx_chan,UV)  $voice_chan
                    }
                    
                    set State(vctx_freq,UV)  [SzSnet uhftxfreq  $State(vctx_chan,UV) -forgive]
                    set State(vctx_freq,UV)  [format "%.4f" $State(vctx_freq,UV)]
                  }

     "USER"       { if { $voice_chan != "" } \
                    {
                      set State(voice_chan)      $voice_chan
     
                      set State(vctx_chan,USER)  $voice_chan
                    }
                    
                    # Use TX channel
                    # TX and RX channels are the same
                    set chan  $State(vctx_chan,USER)

                    # If the USER TX frequency is undefined for this channel, 
                    # initialize it to 100.0
                    if { ![info exists  ::SzSnet::State(vctx_freq,USER,$chan)] } \
                    {
                      set State(vctx_freq,USER,$chan)  100.0000
                    }
                    
                    # Set and display this USER TX frequency value
                    set State(vctx_freq,USER)  [format "%.4f" $State(vctx_freq,USER,$chan)]
                    
                    # If the USER RX frequency is undefined for this channel, 
                    # initialize it to 100.0
                    if { ![info exists  ::SzSnet::State(vcrx_freq,USER,$chan)] } \
                    {
                      set State(vcrx_freq,USER,$chan)  100.0000
                    }
                    
                    # Set and display this USER RX frequency value
                    set State(vcrx_freq,USER)  [format "%.4f" $State(vcrx_freq,USER,$chan)]
                  }
   }

   return
 }


#***************************************************************************
#
#        Name: update_VCRx_freq
#
#  Parameters:
#
# Description: Updates the VCRx frequency for all bands
#
#***************************************************************************
proc update_VCRx_freq { { voice_chan  "" } } \
 {
   #puts "update_VCRx_freq:  $voice_chan"
  
   variable  State
   
   #puts "State(band_text):  $State(band_text)"

   switch -- $State(band_text) \
   {
     "800"        -
     "800 REBND"  -
     "800 SPLNT"  { }
    
     "900"        { }
    
     "VHF/UHF"    { if { $voice_chan != "" } \
                    {
                      set State(vcrx_chan,UV)  $voice_chan
                    }
                    
                    set State(vcrx_freq,UV)  [SzSnet uhfrxfreq  $State(vcrx_chan,UV) -forgive]
                    set State(vcrx_freq,UV)  [format "%.4f" $State(vcrx_freq,UV)]
                  }

     "USER"       { }
   }

   return
 }


#***************************************************************************
#
#        Name: updateFoundSysID
#
#  Parameters:
#
# Description: Updates the Sys ID for the Control Channel on the Scanner
#
#***************************************************************************
proc updateFoundSysID { found_sys_id } \
 {
   #puts "updateFoundSysID:  $found_sys_id"
  
   variable  State
   
   set State(found_sys_id)  [format "%X" $found_sys_id]
   
   return
 }


#***************************************************************************
#
#        Name: updateTBits
#
#  Parameters:
#
# Description: Updates the T Bits variables for hex and binary display
#
#***************************************************************************
proc updateTBits { t_bits } \
 {
   #puts "updateTBits:  $t_bits"
  
   variable  State
   
   set State(t_bits)  $t_bits
   

   switch -- $State(t_bits) \
   {
     0   { set State(t_bits_hex)         "0"
           set State(t_bits_binary)      "0000" }
     
     1   { set State(t_bits_hex)         "1"
           set State(t_bits_binary)      "0001" }
     
     2   { set State(t_bits_hex)         "2"
           set State(t_bits_binary)      "0010" }
     
     3   { set State(t_bits_hex)         "3"
           set State(t_bits_binary)      "0011" }
     
     4   { set State(t_bits_hex)         "4"
           set State(t_bits_binary)      "0100" }
     
     5   { set State(t_bits_hex)         "5"
           set State(t_bits_binary)      "0101" }
     
     6   { set State(t_bits_hex)         "6"
           set State(t_bits_binary)      "0110" }
     
     7   { set State(t_bits_hex)         "7"
           set State(t_bits_binary)      "0111" }
     
     8   { set State(t_bits_hex)         "8"
           set State(t_bits_binary)      "1000" }
     
     9   { set State(t_bits_hex)         "9"
           set State(t_bits_binary)      "1001" }
     
     10  { set State(t_bits_hex)         "A"
           set State(t_bits_binary)      "1010" }
     
     11  { set State(t_bits_hex)         "B"
           set State(t_bits_binary)      "1011" }
     
     12  { set State(t_bits_hex)         "C"
           set State(t_bits_binary)      "1100" }
     
     13  { set State(t_bits_hex)         "D"
           set State(t_bits_binary)      "1101" }
     
     14  { set State(t_bits_hex)         "E"
           set State(t_bits_binary)      "1110" }
     
     15  { set State(t_bits_hex)         "F"
           set State(t_bits_binary)      "1111" }
     
     default  { }
   }

   return
 }


#***************************************************************************
#
#        Name: togglePTT
#
#  Parameters:
#
# Description: Toggles the PTT
#
#***************************************************************************
proc togglePTT { } \
 {
   #puts "togglePTT"
  
   variable  State
  
   if { $State(ptt_on_off) == 0 } \
   {
     after idle "SzSnet ptt 1 -nowait"
   } \
   else \
   {
     SzSnet ptt 0 -nowait
   }

   return
 }


#***************************************************************************
#
#        Name: toggleAffil
#
#  Parameters:
#
# Description: Toggles the Affiliation mode
#
#***************************************************************************
proc toggleAffil { } \
 {
   #puts "toggleAffil"
  
   variable  State
  
   if { $State(affil) == 0 } \
   {
     after idle "SzSnet affiliation  1 -nowait"
   } \
   else \
   {
     SzSnet affiliation  0 -nowait
   }

   return
 }


#***************************************************************************
#
#        Name: toggleCallType
#
#  Parameters:
#
# Description: Toggles the call type:  ANALOG or DIGITAL
#
#***************************************************************************
proc toggleCallType { } \
 {
   #puts "toggleCallType"
   
   variable  State
   
   # Reset the AFFIL and PTT state
   set State(uutaff)  0
   set State(uutptt)  0   
   
   if { $State(call_type) == 0 } \
   {
     after idle "SzSnet digital 1 -nowait"
   } \
   else \
   {
     SzSnet digital 0 -nowait
   }

   return
 }


#***************************************************************************
#
#        Name: toggleSimOnOff
#
#  Parameters:
#
# Description: Starts and Stops the simulator
#
#***************************************************************************
proc toggleSimOnOff { } \
 {
   #puts "toggleSimOnOff"
  
   variable  State
  
   if { $State(sim_on_off) == 0 } \
   {
     after idle "::SzSnet::Sim_turn_on"
   } \
   else \
   {
     after idle "::SzSnet::Sim_turn_off"
   }
   
   return
 }


#***************************************************************************
#
#        Name: Sim_turn_on
#
#  Parameters:
#
# Description: Starts the simulator
#
#***************************************************************************
proc Sim_turn_on { } \
 {
   # turn off any previous trunking state
   ::Generator::Trunking_mode "::SzSnet::Sim_turn_off"
   
   SzSnet simcx 1
 }
 

#***************************************************************************
#
#        Name: Sim_turn_off
#
#  Parameters:
#
# Description: Stops the simulator
#
#***************************************************************************
proc Sim_turn_off { } \
 {
   SzSnet ptt 0 -nowait
   
   SzSnet simcx 0
 }


#***************************************************************************
#
#        Name: toggleScanOnOff 
#
#  Parameters: 
#
# Description: Toggle the Scanner ON / OFF
#
#***************************************************************************
proc toggleScanOnOff { } \
 {
   #puts "toggleScanOnOff"

   variable  State
  
   if { $State(scan_on_off) == 0 } \
   {
     after idle "SzSnet::Scan_turn_on"
   } \
   else \
   {
     after idle "SzSnet::Scan_turn_off"
   }

   return
 }


#***************************************************************************
#
#        Name: Scan_turn_on
#
#  Parameters:
#
# Description: Starts the Scanner
#
#***************************************************************************
proc Scan_turn_on { } \
 {
   # turn off any previous trunking state
   ::Generator::Trunking_mode  "::SzSnet::Scan_turn_off"
  
   SzSnet follow  1  -nowait
 }


#***************************************************************************
#
#        Name: Scan_turn_off
#
#  Parameters:
#
# Description: Stops the Scanner
#
#***************************************************************************
proc Scan_turn_off { } \
 {
   variable  State
   
   SzSnet follow  0  -nowait
   
   # Clear the 'ACTIVE' indicator
   set State(onvx)  0

 }


#***************************************************************************
#
#        Name: setBand
#
#  Parameters:
#
# Description: Sets the OSW band
#
#***************************************************************************
proc setBand { } \
 {
   #puts "SzSnet_TCL::setBand"
   
   variable  State
   
   #puts "State(band_text):  $State(band_text)"

   switch -- $State(band_text) \
   {
     "800"        { SzSnet band  0 }
   
     "800 SPLNT"  { SzSnet band  1 }

     "VHF/UHF"    { SzSnet band  2 }
       
     "900"        { SzSnet band  3 }

     "USER"       { SzSnet band  4 }

     "800 REBND"  { SzSnet band  5 }
   }

   return
 }  


#***************************************************************************
#
#        Name: setChannels
#
#  Parameters:
#  Description: Sets the Control Channel and Voice Channel Transmit 
#              channel numbers
#***************************************************************************
proc setChannels { } \
 {
   #puts "SzSnet_TCL::setChannels"
   
   variable  State
   
   #puts "State(band_text):  $State(band_text)"

   switch -- $State(band_text) \
   {
     "800"        { SzSnet control $State(cctx_chan,800) -nowait
                    SzSnet voice   $State(vctx_chan,800) -nowait }

     "800 SPLNT"  { SzSnet control $State(cctx_chan,800) -nowait
                    SzSnet voice   $State(vctx_chan,800) -nowait }

     "VHF/UHF"    { SzSnet uhfcctx $State(cctx_chan,UV) -nowait 
                    SzSnet uhfccrx $State(ccrx_chan,UV) -nowait
                    SzSnet uhfvctx $State(vctx_chan,UV) -nowait
                    SzSnet uhfvcrx $State(vcrx_chan,UV) -nowait  }
       
     "900"        { SzSnet control $State(cctx_chan,900) -nowait
                    SzSnet voice   $State(vctx_chan,900) -nowait }
     
     "USER"       { SzSnet control $State(cctx_chan,USER) -nowait
                    SzSnet voice   $State(vctx_chan,USER) -nowait
                    set_user_freq 
                    return
                  }
   }

   if { $State(sim_on_off) == 1 } \
   {
     after idle "::SzSnet::cycleSimOnOff"
   }
   
   return
 }


#***************************************************************************
#
#        Name: setFrequency
#
#  Parameters:
#
# Description: Sets the frequency as entered by the user
#
#***************************************************************************
proc setFrequency { designator } \
 {
   #puts "SzSnet_TCL::setFrequency:  $designator"

   variable  State

   # For USER mode, the engine does not maintain a method of going from
   # frequency to channel, so drive the current USER settings to the engine
   switch -- $designator \
   {
     "cctx_freq,USER"  -
     "ccrx_freq,USER"  -
     "vctx_freq,USER"  -
     "vcrx_freq,USER"  { set_user_freq
                         return 
                       }
                       
     default           { }
   }
   
   # Set the designator channel to the result
   switch -- $designator \
   {
     "cctx_freq,800"   -
     "cctx_freq,900"   { # Get the TX channel closest to the frequency
                         set channel  [SzSnet  freqtochantx  $State($designator)]
   
                         SzSnet control  $channel -nowait 
                       }
     
     "vctx_freq,800"   -
     "vctx_freq,900"   { # Get the TX channel closest to the frequency
                         set channel  [SzSnet  freqtochantx  $State($designator)]
   
                         SzSnet voice    $channel -nowait 
                       }

     "cctx_freq,UV"    { # Get the TX channel closest to the frequency
                         set channel  [SzSnet  freqtochantx  $State($designator)]
                         
                         SzSnet uhfcctx  $channel -nowait 
                       }
                         
     "ccrx_freq,UV"    { # Get the RX channel closest to the frequency
                         set channel  [SzSnet  freqtochanrx  $State($designator)]
                         
                         SzSnet uhfccrx  $channel -nowait 
                       }
                         
     "vctx_freq,UV"    { # Get the TX channel closest to the frequency
                         set channel  [SzSnet  freqtochantx  $State($designator)]
                         
                         SzSnet uhfvctx  $channel -nowait 
                       }
                         
     "vcrx_freq,UV"    { # Get the RX channel closest to the frequency
                         set channel  [SzSnet  freqtochanrx  $State($designator)]
                         
                         SzSnet uhfvcrx  $channel -nowait 
                       }
     
     default           { }
   }
   
   if { $State(sim_on_off) == 1 } \
   {
     after idle "::SzSnet::cycleSimOnOff"
   }

   return
 }
 
  #***************************************************************************
  #  Name:        set_user_freq
  #  Parameters:
  #  Description: Sets the user frequency value
  #***************************************************************************
  proc set_user_freq { } \
  {
    variable State
    #puts "SzSnet_TCL::set_user_freq"
    
    if { $State(band_text) == "USER" } \
    {
      # Control Channels
      set chan  $State(cctx_chan,USER)
      
      # Set the TX frequency to the user entered value
      SzSnet  txfreq  $chan  $State(cctx_freq,USER)
      
      # Set the USER TX frequency for this channel to the user entered frequency
      set State(cctx_freq,USER,$chan)  $State(cctx_freq,USER)
      
      # Set the RX frequency to the user entered value
      SzSnet  rxfreq  $chan  $State(ccrx_freq,USER)
    
      # Set the USER RX frequency for this channel to the user entered frequency
      set State(ccrx_freq,USER,$chan)  $State(ccrx_freq,USER)
    
      # Voice Channels
      set chan  $State(vctx_chan,USER)
      
      # Set the TX frequency to the user entered value
      SzSnet  txfreq  $chan  $State(vctx_freq,USER)
      
      # Set the USER TX frequency for this channel to the user entered frequency
      set State(vctx_freq,USER,$chan)  $State(vctx_freq,USER)
      
      # Set the RX frequency to the user entered value
      SzSnet  rxfreq  $chan  $State(vcrx_freq,USER)
    
      # Set the USER RX frequency for this channel to the user entered frequency
      set State(vcrx_freq,USER,$chan)  $State(vcrx_freq,USER)
    }
    
    # Cycle the mode
    if { $State(sim_on_off) == 1 } \
    {
      after idle "::SzSnet::cycleSimOnOff"
    }
    
    return
  }

#***************************************************************************
#
#        Name: setConnectTone
#
#  Parameters:
#
# Description: Sets the Connection Tone (0 - 7)
#
#***************************************************************************
proc setConnectTone { } \
 {
   #puts "setConnectTone"
   
   variable  State
   variable  conn_tones

   set conn_tone_val  $conn_tones($State(conn_tone))
  
   #puts "conn_tone_val:  $conn_tone_val"
  
   after idle "SzSnet connecttone $conn_tone_val -nowait"
  
   return
 }


#***************************************************************************
#
#        Name: setSystemID
#
#  Parameters:
#
# Description: Sets the System ID
#
#***************************************************************************
proc setSystemID { value_type } \
 {
   #puts "setSystemID"
  
   variable  State
   
   switch -- $value_type \
   {
     "HEX"  { if { $State(sys_id_hex) == "" } \
              {
                set State(sys_id_hex)  0
              }
             
              set temp  [string toupper $State(sys_id_hex)]
              
              set temp  0x$State(sys_id_hex)
            }
   
     "DEC"  { if { $State(sys_id) == "" } \
              {
                set State(sys_id)  0
              }
             
              set temp  [format  "%X"  $State(sys_id)]
              
              set temp  [string toupper $temp]
             
              set temp  0x$temp
            }
   }

   after idle "SzSnet  sysid  $temp  -nowait"

   return
 }


#***************************************************************************
#
#        Name: setGroupID
#
#  Parameters:
#
# Description: Sets the Group ID
#
#***************************************************************************
proc setGroupID { format_type } \
 {
   #puts "setGroupID:  $format_type"
  
   variable  State
  
   switch -- $format_type \
   {
     "HEX"  { if { $State(grp_id_hex) == "" } \
              {
                set State(grp_id_hex)  0
              }
             
              set State(grp_id_hex)  [string toupper $State(grp_id_hex)]
             
              # Set the decimal value variable
              set State(grp_id)  [format "%d" 0x$State(grp_id_hex)]
            }
   
     "DEC"  { if { $State(grp_id) == "" } \
              {
                set State(grp_id)  0
              }
             
              # Set the hexadecimal value variable
              set  State(grp_id_hex)  [format "%X" $State(grp_id)]
            }
   }

   after idle "SzSnet  grp  $State(grp_id)  -nowait"
  
   return
 }


#***************************************************************************
#
#        Name: setUnitID
#
#  Parameters:
#
# Description: Sets the Unit ID
#
#***************************************************************************
proc setUnitID { format_type } \
 {
   #puts "setUnitID:  $format_type"
    
   variable  State

   switch -- $format_type \
   {
     "HEX"  { if { $State(unit_id_hex) == "" } \
              {
                set State(unit_id_hex)  0
              }
             
              set State(unit_id_hex)  [string toupper $State(unit_id_hex)]
             
              # Set the decimal value variable
              set  State(unit_id)  [format "%d" 0x$State(unit_id_hex)]
            }
   
     "DEC"  { if { $State(unit_id) == "" } \
              {
                set State(unit_id)  0
              }
             
              # Set the hexadecimal value variable
              set  State(unit_id_hex)  [format "%X" $State(unit_id)]
            }
   }

   catch { SzSnet  unitid  $State(unit_id)  -nowait } result

   return
 }


#***************************************************************************
#
#        Name: cycleSimOnOff
#
#  Parameters:
#
# Description: Cycles the state of the Simulator OFF and ON
#
#***************************************************************************
proc cycleSimOnOff { } \
 {
   #puts "cycleSimOnOff"
  
   variable  State
  
   # Turn OFF the Simulator
   SzSnet simulatecontrol  0

   after 1000

   # Restore the ON/OFF state of the Simulator
   SzSnet simulatecontrol  1

   # Reset the Affiliation Received status
   set State(uutaff)  0

   return
 }


#***************************************************************************
#
#        Name: cycleScanOnOff
#
#  Parameters:
#
# Description: Cycles the state of the Scanner OFF and ON
#
#***************************************************************************
proc cycleScanOnOff { } \
 {
   #puts "cycleScanOnOff"
  
   SzSnet follow 0
      
   after 500
      
   SzSnet follow 1

   return
 }


#****************************************************************************
#        Name: getSaveState
#  Parameters:  none
# Description: This function returns a list of TCL commands which can be used
#              to set the SzSnet engine to its current state
#****************************************************************************
 proc getSaveState { } \
 {
   variable  State
   variable  conn_tones


   SzSnet::Init
   
   set returnList \
   {
       set ::SzSnet::State(cctx_chan,800)  {$State(cctx_chan,800)}
       set ::SzSnet::State(vctx_chan,800)  {$State(vctx_chan,800)}

       set ::SzSnet::State(cctx_chan,900)  {$State(cctx_chan,900)}
       set ::SzSnet::State(vctx_chan,900)  {$State(vctx_chan,900)}

       set ::SzSnet::State(cctx_chan,UV)  {$State(cctx_chan,UV)}
       set ::SzSnet::State(ccrx_chan,UV)  {$State(ccrx_chan,UV)}
       set ::SzSnet::State(vctx_chan,UV)  {$State(vctx_chan,UV)}
       set ::SzSnet::State(vcrx_chan,UV)  {$State(vcrx_chan,UV)}

       set ::SzSnet::State(cctx_chan,USER)  {$State(cctx_chan,USER)}
       set ::SzSnet::State(vctx_chan,USER)  {$State(vctx_chan,USER)}

       set ::SzSnet::State(band)  {$State(band)}
       set ::SzSnet::State(band_text)  {$State(band_text)}

       SzSnet::setBand

       SzSnet cnxtone  "$conn_tones($State(conn_tone))"

       SzSnet sysid    "$State(sys_id)"
       SzSnet group    "$State(grp_id)"
       SzSnet unitid   "$State(unit_id)"

       SzSnet digital  "$State(call_type)"
   }
   
   append returnList  "\n"

   # Save the user defined TX and RX frequencies
   # for both Control and Voice Channels
   foreach { index } [array names State  *,USER,*] \
   {
     append returnList  "set ::SzSnet::State($index)  {$State($index)} \n"
   }

   #puts "$returnList"
   set returnList [subst -nocommands $returnList]
   if {$::SzSnet::State(sim_on_off) == 0} \
   {
      append returnList "::SavedStates::Finally ::SzSnet::Sim_turn_off\n"
   } \
   else\
   {
      append returnList "::SavedStates::Finally ::SzSnet::Sim_turn_on\n"
   }
   set rv \
   {
     if [IFR_options is_installed SzSnet] \
     {
        $returnList
     }
   }
   return [subst -nocommands $rv]
 }


#****************************************************************************
#        Name: SyncAll
#  Parameters:  none
# Description: Insures that TCL, the engine and hardware are in sync
#****************************************************************************
 proc SyncAll { } \
 {
   if ![IFR_options is_installed SzSnet] { return }
   
   set_szsnet_callbacks
 
   if { ![interp issafe] } \
   {
     variable path
     source [file join $path master.tcl]
   }
 }


#****************************************************************************
# Initialize the namespace and activate the callbacks
#****************************************************************************
   Load::on_system_done { ::SzSnet::SyncAll }



} ;# End of the SmartZone/SmartNet namespace (SzSnet)


#****************************************************************************
# Option Definition: Set up our HTML option info
#****************************************************************************
namespace eval ::SzSnet_900 \
{
  variable InfoString

  set SzSnet_version [string trimleft [SzSnet ver] "SzSnetEng"]
  set InfoString [subst \
  {
         <br>
         (using SmartZone / SmartNet version - $SzSnet_version)
         <br>
         This option provides a basic SmartZone / SmartNet Repeater
         simulation, allowing the 2975 to communicate in the 900 MHz frequency band.
  }]
}

#****************************************************************************
# Option Definition: Set up our HTML option info
#****************************************************************************
namespace eval ::SzSnet \
{
  variable InfoString

  set SzSnet_version [string trimleft [SzSnet ver] "SzSnetEng"]
  set InfoString [subst \
  {
         <br>
         (using SmartZone / SmartNet version - $SzSnet_version)
         <br>
         This option provides Motorola Smartnet/Smartzone testing for
mobile radios.
  }]
}

#****************************************************************************
# Option Definition: Define the namespace, and set up our HTML option info
#****************************************************************************
namespace eval ::Smartnet_Monitor \
{
  variable InfoString

  set SzSnet_version [string trimleft [SzSnet ver] "SzSnetEng"]
  set InfoString [subst \
  {
         <br>
         (using SmartZone / SmartNet version - $SzSnet_version)
         <br>
         This option monitors SmartZone / SmartNet communication.
  }]
}

#****************************************************************************
# Option Definition: Define the namespace, and set up our HTML option info
#****************************************************************************
namespace eval ::SzSnet_Reband \
{
  variable InfoString

  set SzSnet_version [string trimleft [SzSnet ver] "SzSnetEng"]
  set InfoString [subst \
  {
         <br>
         (using SmartZone / SmartNet version - $SzSnet_version)
         <br>
         This option enables messaging and channels for rebanded 800 MHz radios.
  }]
}

