##############################################################################
# IFR SYSTEMS, INC          COPYRIGHT (C)1998-2002        DESIGN ENGINEERING #
# -------------------------------------------------------------------------- #
# FILE        : load_it.tcl                                                  #
# PRODUCT     : FITS                                                         #
# AUTHOR      : Hieu Tran                                                    #
# DESCRIPTION : load the system code and firmware code, inform loading       #
#               status, and display the 'credits page'.                      # 
#                                                                            #
##############################################################################

package require Utils

namespace eval Load \
{
   variable State
   set State(VERBOSE_BOOT) [option get . verboseBoot {}]

   variable DSP_NUMBERS
   array set DSP_NUMBERS \
   {
      PARAMETRIC        1
      LAYER1            2
      MULTIFUNCTION     3
      SCOPE             4
      FPGA              7
      FPGA_LDR          3
   }
   
   set State(STATUS_FRAME) ""

# define the helper loader files
   variable DSP_HELPERS
   array set DSP_HELPERS \
   {
     PARAMETRIC        /system/DSP_code/mast_hlp.cld
     LAYER1            /system/DSP_code/slave_hlp.cld
     MULTIFUNCTION     /system/DSP_code/mast_hlp.cld
     SCOPE             /system/DSP_code/slave_hlp.cld
     FPGA              /system/DSP_code/multi_revb.rbf
     FPGA_LDR          /system/DSP_code/mast_hlp.cld
  }
# define the main files
# NOTE: the FPGA does not have a main file, it just has
#       a helper loader
  variable DSP_FILES
  array set DSP_FILES \
  {
     PARAMETRIC          /system/DSP_code/parmm.cld
     LAYER1              /system/DSP_code/layer1.cld
     MULTIFUNCTION       /system/DSP_code/multif.cld
     SCOPE               /system/DSP_code/scopeanl.cld
     FPGA_LDR            /system/DSP_code/fpga_ldr.cld
  }
  set State(on_system_done) ""
  set State(on_hwinit_done) ""
  set State(on_UI_done) ""
  set State(DSP_LOAD_STATE) START
  set State(DSP_ERR_STATE) START
  set State(DSP_LOAD_ERRS)  0
  set State(MAX_DSP_LOAD_ERRS) 10
  set State(DSP_TIMEOUT) "init"
  set State(DSP_PERCENT) 0
  set State(DSP_ERR_STATUS) ""
  set State(DSP_CALLBACKS_STATUS) 0

  set State(NEXT)             0
  set State(COUNT)            0
  set State(DSP_DONE)         0
  set State(ENGINES_DONE)     0
  set State(UI_DONE)          0
  set State(DONE)             0
  set State(HW_DONE)          0
  set State(MESSAGE)          0
  set State(UI_FLAG)          0
  set State(LD_ERROR)         0
  set State(LD_ADDRESS)       0
  set State(ARRAY_POS)        0
  set State(MAX_CREDIT)       0
  set State(NUM_CREDIT_LINES) 8
  variable CREDIT_ARRAY
  set CREDIT_ARRAY(0)  " "

  #
  # define the array of DSP names
  #
  variable DSP_LOAD
  array set DSP_LOAD \
  {
    0 PARAMETRIC
    1 LAYER1
    2 FPGA_LDR
    3 FPGA
    4 MULTIFUNCTION
    5 SCOPE
  }

  #
  # define the array of DSP status
  #
  variable DSP_STATUS
  array set DSP_STATUS \
  {
    0 OK
    1 "Write error"
    2 "Read error"
    3 "Insert error"
    4 "Null message"
    5 "Done"
    6 "Hardware error"
  }
  
  ################################################################################################################
  # Basic display routines

  proc loadbar_update_core { name } \
  {
   variable State
   set widget   $State(STATUS_FRAME).$name.bar
   set color    $State($name,color)
   set percent  $State($name,percent)
   
   after cancel $State($name,after)
   set State($name,after) ""

   # if no loadbar, then done...
   if { [winfo exists $widget] == 0 } {
      return
   }

   set height [winfo height $widget]
   set width  [winfo width $widget]
   set width  [expr ($width*$percent)/100]
   $widget delete all
   $widget create rect 0 0 $width $height -fill $color -tags all
  }

  ##############
  #
  # update loadbar  
  #   name    - name of widget
  #   count   - # of things loaded
  #   total   - total # of things to load
  #   color   - color of loadbar
  #
  proc loadbar_update { name count total {color blue } } \
  {
   variable State
   set State($name,percent) [expr ($count*100)/$total]
   set State($name,color) $color
   
   if {"$State($name,after)" == "" } \
   {
      set State($name,after) [after idle "Load::loadbar_update_core $name"]
   }
  }

  ################################################################################################################
  # Screen setup routines

  ##############
  # setup load messages and loadbars on screen
  #
  proc setup_load_bar { parent item } \
  {
   variable State
   array set State \
   [subst \
    {
       $item,color   ""
       $item,percent 0
       $item,after   ""
    }
   ]
   set name $parent.$item
   frame $name
   label $name.text
   canvas $name.bar -bg darkgrey -height 15
   pack $name.text $name.bar -side top -expand false -fill x
   pack $name -side top -fill x -expand false
   variable State
  }

 #############
 # define the procs for showing things
 proc load_message { which msg {color black} } \
 {
  variable State
  if [winfo exists $State(STATUS_FRAME)] \
  {
    if [info exists $msg] \
    {
       $State(STATUS_FRAME).$which.text config -textvariable $msg -fg $color -text ""
    } \
    else \
    {
       $State(STATUS_FRAME).$which.text config -text $msg -fg $color -textvariable ""
    }
  }
 }

 #############
 # Set up the load status frame
 proc setup_load_screen { name } \
 {
   variable State
   set State(STATUS_FRAME) $name

   setup_load_bar $name dsp
   setup_load_bar $name engines
   setup_load_bar $name ui
   setup_load_bar $name hardware

   load_message dsp     "Loading DSPs"
   load_message engines "Loading Engines"
   load_message ui      "Loading UI"
   load_message hardware "Hardware init"
 }

 #############
 # display loading message of UI code
 #    which - text message
 #
 proc load_ui_message { msg {color black} } \
 {
   load_message ui $msg $color
 }
 
 proc load_hardware_message { msg {color black} } \
 {
   load_message hardware $msg $color
 }

 #############
 # update loading of DSP loadbar and message
 #    percent - percent of load completed
 #
 proc update_dsp_loadbar { } \
 {
   variable State
   loadbar_update dsp $State(DSP_PERCENT) 100
 }

 proc update_hardware_loadbar { percent } \
 {
   variable State
   loadbar_update hardware $percent 100
 }
 
 #############
 #
 # setup dsp to handle DSP load callback message
 #
 proc setup_dsp_load {} \
 {
   variable State

   if { $State(DSP_CALLBACKS_STATUS) != 0 } \
   {
      return
   }

   set State(DSP_CALLBACKS_STATUS) [Dsploader -onchange filestatus { ::Load::handle_dsp_load_msg %V}]
   Dsploader -onchange loaddone { ::Load::handle_dsp_load_msg %V}
 }

 #############
 # handle load message callback from DSP, and call on the next DSP
 # to be loaded after one is completed
 #
 proc handle_dsp_load_msg { parmList } \
 {
   set stat    [lindex $parmList 0]
   set percent [lindex $parmList 1]
   set dsp     [lindex $parmList 2]
   variable State

   variable DSP_STATUS

   switch $stat \
   {
      0 { 
          set State(DSP_PERCENT) $percent
          update_dsp_loadbar
          start_dsp_timeout
          return
        }
     5 {
          cancel_dsp_timeout
          after idle Load::load_dsp_done
          return
      }
      default \
      {
         puts "Unknown DSP response msg $stat: $State(DSP_LOAD_STATE)"
      }
   }

   set State(DSP_ERR_STATUS) $DSP_STATUS($stat)
   after idle {
      Load::dsp_load_err $::Load::State(DSP_ERR_STATUS)
   }
 }

 #############
 # cancel the timeout for loading a DSP
 proc cancel_dsp_timeout {} {
   variable State
   # start our timeout variable
   after cancel Load::Check_dsp_timeout
   after cancel Load::dsp_timeout
   if { "$State(DSP_TIMEOUT)" != "" } {
     set $State(DSP_TIMEOUT) ""
   } \
   else \
   {
      puts "load done after timeout $State(DSP_LOAD_STATE)"
   }
 }

 proc Check_dsp_timeout { } \
 {
    variable State
    after idle Load::dsp_timeout
 }
 proc dsp_timeout { } \
 {
    variable State
    if {$State(DSP_TIMEOUT) == "" } \
    {
       puts "DSP timeout fired, but had been canceled. Ignoring"
       return
    }
    set State(DSP_TIMEOUT) ""
    after idle {Load::dsp_load_err TIMEOUT}
 }
 #############
 # start the timeout for loading a DSP
 proc start_dsp_timeout {} \
 {
   variable State
   after cancel Load::Check_dsp_timeout 
   # start our timeout variable
   set State(DSP_TIMEOUT) [after 15000 Load::Check_dsp_timeout ]
 }

 #############
 # send the message to the DSP engine to reboot a DSP and
 # load the helper loader
 #    dsp_ID - command code 1
 #    filename - file to be loaded
 #
 proc send_dsp_reboot_msg { dsp_ID filename } {
   # send the reboot command (command number 0) to the DSP engine
   Dsploader reboot $dsp_ID $filename -nowait
 }

 #############
 # send the message to the DSP engine to load the main code
 # (helper loader must be loaded)
 #    dsp_ID - which dsp 
 #    filename - file to be loaded
 #
 proc send_dsp_load_msg { dsp_ID filename } \
 {
   # send the load system code command (#1) to the DSP engine
   Dsploader loadfile $dsp_ID $filename -nowait
 }

 #############
 # reboot_dsp - reboot DSP and load standard helper
 proc reboot_dsp { which } \
 {
    variable DSP_NUMBERS
    variable DSP_HELPERS
    variable State
    set State(DSP_PERCENT) 0
    update_dsp_loadbar
    start_dsp_timeout

    Dsploader reboot $DSP_NUMBERS($which) $DSP_HELPERS($which) -nowait
 }

 #############
 # load_dsp - load a DSP with the standard file
 proc load_dsp { which } \
 {
    variable DSP_NUMBERS
    variable DSP_FILES
    variable State
    set State(DSP_PERCENT) 0
    update_dsp_loadbar
    start_dsp_timeout

    Dsploader loadfile $DSP_NUMBERS($which) $DSP_FILES($which) -nowait
 }

 #############
 # load_dsp_done - completes the load of one DSP and moves to next state
 proc load_dsp_done {} \
 {
   variable State

   set State(DSP_PERCENT) 100
   update_dsp_loadbar
   cancel_dsp_timeout
   after 100 ;# DDH BODGE
   
   # Now, move to the next state
   switch $State(DSP_LOAD_STATE) \
   {
      START {
          set State(DSP_LOAD_STATE) LOADING_FPGA_LDR_HELPER
          set DSP_LOAD_ERRS 0
      }
      # 
      # MF DSP stuff
      #
      LOADING_FPGA_LDR_HELPER \
      {
          set State(DSP_LOAD_STATE) LOADING_FPGA_LDR
      }
      LOADING_FPGA_LDR \
      {
          set State(DSP_LOAD_STATE) LOADING_FPGA
      }
      LOADING_FPGA \
      {
          set State(DSP_LOAD_STATE) LOADING_MF_LDR
      }
      LOADING_MF_LDR \
      {
          set State(DSP_LOAD_STATE) LOADING_MF
      }
      LOADING_MF \
      {
          set State(DSP_LOAD_STATE) LOADING_PARA_HELPER
      }
      LOADING_PARA_HELPER \
      {
          set State(DSP_LOAD_STATE) LOADING_PARA
      }
      LOADING_PARA \
      {
          set State(DSP_LOAD_STATE) LOADING_L1_HELPER
      }
      LOADING_L1_HELPER \
      {
          set State(DSP_LOAD_STATE) LOADING_LAYER1
      }
      LOADING_LAYER1 \
      {
          set State(DSP_LOAD_STATE) LOADING_SCOPE_LDR
      }
      LOADING_SCOPE_LDR \
      {
          set State(DSP_LOAD_STATE) LOADING_SCOPE
      }
      LOADING_SCOPE \
      {
          set State(DSP_LOAD_STATE) DONE
      }
      DONE \
      {
         return
      }
      default \
      {
         puts "load_dsp_done state is $State(DSP_LOAD_STATE) - going to START"
         set State(DSP_LOAD_STATE) START
      }
   }
   after idle Load::load_next_dsp
 }

 #############
 # load_next_dsp - loads the next DSP in the load sequence
 # uses global $State(DSP_LOAD_STATE) to move through the states
 proc load_next_dsp {} \
 {
   variable State
  
   switch $State(DSP_LOAD_STATE) \
   {
      LOADING_PARA_HELPER 
      {
          set State(DSP_ERR_STATE) LOADING_PARA_HELPER
          load_message dsp "Loading Parametric bootloader"
          reboot_dsp PARAMETRIC
      }
      LOADING_PARA 
      {
          set State(DSP_ERR_STATE) LOADING_PARA_HELPER
          load_message dsp "Loading Parametric"
          load_dsp PARAMETRIC
      }
      LOADING_L1_HELPER 
      {
          set State(DSP_ERR_STATE) LOADING_L1_HELPER
          load_message dsp "Loading Layer 1 bootloader"
          reboot_dsp LAYER1
      }
      LOADING_LAYER1 
      {
          set State(DSP_ERR_STATE) LOADING_L1_HELPER
          load_message dsp "Loading Layer 1"
          load_dsp LAYER1
      }
      LOADING_FPGA_LDR_HELPER 
      {
          set State(DSP_ERR_STATE) LOADING_FPGA_LDR_HELPER
          load_message dsp "Loading FPGA Bootloader"
          reboot_dsp FPGA_LDR
      }
      LOADING_FPGA_LDR 
      {
          set State(DSP_ERR_STATE) LOADING_FPGA_LDR_HELPER
          load_message dsp "Loading FPGA loader"
          load_dsp FPGA_LDR
      }
      LOADING_FPGA 
      {
          set State(DSP_ERR_STATE) LOADING_FPGA_LDR_HELPER
          load_message dsp "Loading FPGA"
          reboot_dsp FPGA
      }
      LOADING_MF_LDR 
      {
          set State(DSP_ERR_STATE) LOADING_MF_LDR
          load_message dsp "Loading Audio bootloader"
          reboot_dsp MULTIFUNCTION
      }
      LOADING_MF 
      {
          set State(DSP_ERR_STATE) LOADING_MF_LDR
          load_message dsp "Loading Audio DSP"
          load_dsp MULTIFUNCTION
      }
      LOADING_SCOPE_LDR 
      {
          set State(DSP_ERR_STATE) LOADING_SCOPE_LDR
          load_message dsp "Loading Scope bootloader"
          reboot_dsp SCOPE
      }
      LOADING_SCOPE 
      {
          set State(DSP_ERR_STATE) LOADING_SCOPE_LDR
          load_message dsp "Loading Scope DSP"
          load_dsp SCOPE
      }
      DONE 
      {
         load_done DSP_DONE
         load_message dsp "DSP loads done"
         return
      }
   }
 }

 #############
 # dsp_load_err - Handles loading errors on the DSPs
 # uses global $State(DSP_LOAD_STATE) to move through the states
 # uses global DSP_LOAD_ERRS to count the # of errors
 proc dsp_load_err { {err "Unknown"} } \
 {
   variable State

   if {"$State(DSP_LOAD_STATE)" == "DONE" } \
   {
     return
   }

   load_message dsp $err red
   loadbar_update dsp 100 100 red

   Debug::Puts Load 0 "DSP load error $err, state is $State(DSP_LOAD_STATE), restart is $State(DSP_ERR_STATE)"
   ErrLog::Log "DSP load error $err, state is $State(DSP_LOAD_STATE), restart is $State(DSP_ERR_STATE)"
   
   incr State(DSP_LOAD_ERRS)
   if { $State(DSP_LOAD_ERRS) > $State(MAX_DSP_LOAD_ERRS) } \
   {
      set State(LD_ERROR) 1
      return
   }

   set State(DSP_LOAD_STATE) $State(DSP_ERR_STATE)
   load_next_dsp
 }

 #############
 #
 # start the process of loading all DSPs
 #
 proc load_all_dsps {} \
 {
   variable State

   setup_dsp_load
   set State(DSP_LOAD_STATE) START
   load_dsp_done
 }

 #############
 #
 # handle load message callback from DSP, and call on the next DSP
 # to be loaded after one is completed
 #
 proc handle_module_load_msg { name stat} \
 {
     module_load_status $stat
 }
 
 proc module_load_status { stat } \
 {
     variable State
     if { "$State(ENGINES_TIMEOUT)" != ""} \
     {
         after cancel $State(ENGINES_TIMEOUT)
     }

     set State(LOAD_ADDRESS) $stat
     if {$State(LOAD_ADDRESS) != 0} \
     {
         incr State(ENGINES_LOADED)
         loadbar_update engines $State(ENGINES_LOADED) $State(ENGINES_COUNT) blue
     } \
     else \
     {
         set State(LD_ERROR) 1
         load_message engines "Error load: $State(engine_name)" red
         append_copyright "Error load: $State(engine_name)"
         loadbar_update engines 100 100 red
         ErrLog::Log "Error load: $State(engine_name)"
     }
     after idle { Load::load_an_engine }
 }

 #############
 #
 # load engine modules and update loadbar
 #
 proc load_engine_modules {} \
 {
   variable State

   # create our load callback
   Engine -onchange Load::handle_module_load_msg

   # count number of files
   set State(ENGINES)        [glob -nocomplain /system/engines/*.out]
   eval lappend  State(ENGINES)   [glob -nocomplain /system/engines/*.out,enc]
   set State(ENGINES)        [lsort $State(ENGINES)]
   set State(ENGINES_COUNT)  [llength $State(ENGINES)]
   set State(ENGINES_LOADED) 0
   set State(ENGINES_TIMEOUT) ""

   load_an_engine
 }
 
 proc load_an_engine {} \
 {
    variable State
    set State(engine_file) [lindex $State(ENGINES) 0]
    set State(ENGINES)     [lrange $State(ENGINES) 1 end]
    set State(engine_name) [file tail $State(engine_file)]
    
    if {"$State(engine_file)"==""} \
    {
      loadbar_update engines 100 100
      load_message engines "Engines done"
      Engine -onchange {}
      load_done ENGINES_DONE
      return
    }
    if { "$State(ENGINES_TIMEOUT)" != ""} \
    {
         after cancel $State(ENGINES_TIMEOUT)
    }
    set State(ENGINES_TIMEOUT) [after 30000 "Load::module_load_status 0" ]
    if {"$State(VERBOSE_BOOT)" != "" } {
       load_message engines "Loading $State(engine_name)"
    }
    Engine load $State(engine_file)
 }

 #############
 # Scroll the copyright messages
 #
 proc update_credits { name } \
 {
     variable State
     variable CREDIT_ARRAY

     if {[winfo exists $name] == 0} {
       return
     }

     set j $State(ARRAY_POS)
     for { set i $State(NUM_CREDIT_LINES)} { $i > 0 } {  } {
        incr i -1
        $name.text$i configure -textvariable Load::CREDIT_ARRAY($j)
        incr j
        if {$j >= $State(MAX_CREDIT) } {
          set j 0
        }
     }
     incr State(ARRAY_POS)
     if {$State(ARRAY_POS) >= $State(MAX_CREDIT)} {
         set State(ARRAY_POS) 0
     }
     if {$State(NUM_CREDIT_LINES) < $State(MAX_CREDIT) } {
        after 1500 "Load::update_credits $name"
     }
 }

 #############
 #
 # load copyright messages
 #
 proc append_copyright { value } \
 {
    variable State
    variable CREDIT_ARRAY
    set CREDIT_ARRAY($State(MAX_CREDIT)) $value
    incr State(MAX_CREDIT)
 }
 
 proc setup_credits { name } \
 {
   variable State
   variable CREDIT_ARRAY
   
   foreach copyright [lsort [glob -nocomplain /system/copyrights/*.txt]] \
   {
      if [catch {open $copyright r} fileID] \
      {
         Debug::Puts Credits 0 stderr "Cannot open $copyright"

      } \
      else \
      {
           if {[gets $fileID value] >= 0} \
           {
              append_copyright $value
           }
      }
      catch {close $fileID}
   }
   

   for { set i 0} { $i < $State(NUM_CREDIT_LINES) } { incr i } \
   {
     label $name.text$i 
     pack  $name.text$i -fill both -side bottom
   }

   update_credits $name
 }

 proc load_system {} \
 {
  # send Message bus messages to the DSP engine requesting the engine
  # begin loading the DSPs
  BGCommand Load::load_all_dsps
  BGCommand Load::load_engine_modules
  
  Debug::Puts Load 1 "loading of DSPs and engines started"
 }

 proc load_done_element { } \
 {
    variable State
    set cmd $State(on_system_done)
    set State(on_system_done) {}
    Debug::Puts Load 1 "On load done: execute $cmd"
    namespace eval :: $cmd
 }
 
 proc load_done { which } \
 {
    variable State
    set State($which) 1
    Debug::Puts Load 1 "$which load done"
    
    if {$State(DSP_DONE)     == 0 } \
    {
       Debug::Puts Load 1 "DSP not done"
       return 
    }
    if {$State(ENGINES_DONE) == 0 } \
    {
       Debug::Puts Load 1 "ENGINES not done"
       return
    }
    set State(DONE) 1
    Debug::Puts Load 1 "System load done"
    if {[llength $State(on_system_done)] > 0} \
    {
       after idle Load::load_done_element
    }
 }
 
 proc hwinit_done_element { } \
 {
    variable State
    set cmd $State(on_hwinit_done)
    set State(on_hwinit_doneon_hwinit_done) {}
    if {$cmd != ""} \
    {
       Debug::Puts Load 1 "On hwinit done: execute $cmd"
       eval $cmd
    }
 }
 
 proc hwinit_done { } \
 {
    variable State
    set State(HW_DONE) 1
    Debug::Puts Load 1 "HW init done"
    if {[llength $State(on_hwinit_done)] > 0} \
    {
       after idle Load::hwinit_done_element
    }
 }
 
 #############
 # wait_for_system - wait until engines are loaded & DSPs are up
 proc wait_for_system {} \
 {
  variable State

  while {"$State(DONE)" == 0 } \
  {
     Debug::Puts Load 1 "Wait for system done"
     vwait ::Load::State(DONE)
     Debug::Puts Load 1 "Wait exit: state $::Load::State(DONE)"
  }
 }
 
 proc on_system_done { cmd } \
 {
    variable State
    if {"$cmd"==""} { return}
    if {"$State(DONE)" == 0 } \
    {
      Debug::Puts Load 1 "on_system_done $cmd"
      append State(on_system_done) [concat $cmd]
      append State(on_system_done) "\n"
    } \
    else \
    {
       Debug::Puts Load 1 "on_system_done (DONE) $cmd"
       after idle $cmd
    }
 }
 
 proc on_hwinit_done { cmd } \
 {
    variable State
    if {"$cmd"==""} { return}
    if {"$State(HW_DONE)" == 0 } \
    {
      Debug::Puts Load 1 "on_hwinit_done $cmd"
      append State(on_hwinit_done) [concat $cmd]
      append State(on_hwinit_done) "\n"
    } \
    else \
    {
       Debug::Puts Load 1 "on_hwinit_done (DONE) $cmd"
       after idle $cmd
    }
 }

#****************************************************************************
# Name:        on_UI_done
# Parameters:  cmd	Process to be delayed
# Description: This function can be called with the name of a process to be
#              executed which must be delayed until all UI code is loaded
#****************************************************************************
 proc on_UI_done { cmd } \
 {
    variable State
    if {"$cmd"==""} { return}
    if {"$State(UI_DONE)" == 0 } \
    {
      Debug::Puts Load 1 "on_UI_done $cmd"
      append State(on_UI_done) [concat $cmd]
      append State(on_UI_done) "\n"
    } \
    else \
    {
       Debug::Puts Load 1 "on_UI_done (DONE) $cmd"
       after idle $cmd
    }
 }
 proc ui_done_element { } \
 {
    variable State
    set cmd $State(on_UI_done)
    set State(on_UI_done) {}
    if {$cmd == ""} { return }
    Debug::Puts Load 1 "On UI done: execute $cmd"
    eval $cmd
 }
#****************************************************************************
# Name:        load_UI_done
# Parameters:  none
# Description: This function is called when all of the UI TCL code has been
#              loaded, so that tasks that are waiting for a full program load
#              can get started.
#****************************************************************************
 proc load_UI_done { } \
 {
    variable State
    set State(UI_DONE) 1
    if {[llength $State(on_UI_done)] > 0} \
    {
       after idle Load::ui_done_element
    }
 }
}

