#****************************************************************************
# IFR SYSTEMS, INC        COPYRIGHT (C)1998,01      DESIGN ENGINEERING      *
#****************************************************************************
# FILE        : main.tcl                                                    *
# PRODUCT     : 2975-P25                                                    *
# DESCRIPTION : load and start the TCL application                          *
#****************************************************************************
package require IFR_options
package require Capture
# preset the HTTPD server vars, so that things can append to it correctly
namespace eval HTTPD \
{ 
   variable main_screen_items
   set main_screen_items(1) ""
   variable main_screen_readings
   set main_screen_readings(1) ""
}
namespace eval Recall_SetupsUI \
{
}

# add default help bindings for all widgets at lowest priority
option add *helpUrl file://localhost/sysdata/html/help/nohelp.shtml 0
option add *baseUrl file://localhost/sysdata/html/help/ 0

set ::sys(manufacturer) "IFR Systems, Inc."
set ::sys(model) "2975"

# unpost menus that are posted when their parent button is unmapped
bind Menubutton <Unmap> { set x [%W cget -menu]; if {[winfo exists $x] && [winfo ismapped $x]} {tkMenuUnpost $x}}

namespace eval Memory \
{
   variable Stats
   
   variable cleanup_list
   
   set cleanup_list {}
   
   variable Limits
   array set Limits \
   {
     view     7500000
     tile     1000000
     slave    1500000
     image    1000000
     help     12000000
     toplevel 1500000
   }
   
   proc Enough_for { type } \
   {
      variable Limits
      set check_proc Free
      
      switch $type \
      {
         "image" { set check_proc Free_main }
      }
      if ![info exists Limits($type)] { return 0}
      if {[$check_proc ] >= $Limits($type)}   { return 1}
      garbage_collect
      if {[$check_proc] >= $Limits($type)}   { return 1}
      return 0
   }
   proc Free_main { } \
   {
      set free_list [VxAlloc]
      set end [lindex $free_list end]
      return $end
   }
   proc Free { } \
   {
      set free_list [VxAlloc]
      set total 0
      foreach free $free_list \
      {
         incr total $free
      }
      return $total
   }
   proc Enough_for_view { } \
   {
      return [Enough_for view]
   }
   
   proc SetFree { id } \
   {
      variable Stats
      set Stats($id) [Free]
   }
   proc ListFree { } \
   {
      variable Stats
      foreach id [array names Memory::Stats] \
      {
          puts "$id $Stats($id)"
      }
   }
   
   # add a routine to be called when we are low on memory
   proc Add_cleanup_proc { cmd } \
   {
      variable cleanup_list
      lappend cleanup_list $cmd
   }
   
   # called when we are low on memory
   proc Cleanup { } \
   {
      variable cleanup_list
      foreach cmd $cleanup_list \
      {
         namespace eval :: $cmd
      }
   }
   
   
   set Stats(initial_free) [Free]

}

proc dummy { } { }

namespace eval Softkey {
  proc Press { widget } \
  {
    $widget configure -relief sunken
  }
  proc Release_after { widget } \
  {
     if [winfo exists $widget] \
     {
        $widget configure -relief raised
     }
  }
  proc Release { widget } \
  {
    if ![winfo exists $widget] { return }
    switch [winfo class $widget] \
    {
       Button     {$widget invoke; after idle "Softkey::Release_after $widget"}
       Menubutton {$widget configure -relief raised;tkMbPost $widget }
    }
  }
   bind Softkey <<Press>>         {Softkey::Press   %W; break}
   bind Softkey <<Release>>       {Softkey::Release %W; break}
   bind Softkey <ButtonRelease-1> {event generate %W <<Release>>; break}
   bind Softkey <Button-1>        {event generate %W <<Press>>; break}
}
xleds . 0 2
namespace eval Shift {
  variable state
  set state 0
  
  proc Mode_press { widget } \
  {
    variable state
    update idle;# let any shift status change occur
    if {[xleds $widget] & 1} \
    {
       event generate $widget <<OptionDown>>
    } \
    else \
    {
       event generate $widget <<ModeDown>>
    }
  }
  proc Mode_release { widget } \
  {
    variable state
    update idle;# let any shift status change occur
    if {[xleds $widget] & 1} \
    {
       event generate $widget <<Option>>
       if {[winfo screen $widget] == [winfo screen .] } {HW_control::ShiftLED OFF}
    } \
    else \
    {
       event generate $widget <<Mode>>
    }
  }
}

namespace eval Reflow {
  bind Reflow <Configure> { Reflow::configure %W}
  
  #********************************************************************
  #
  #       Name: reflow
  #
  # Parameters: widget
  #
  # Description: handles resizing a label widget
  #
  #********************************************************************
  proc configure { widget } \
  {  
     set width  [expr [winfo width  $widget] - 2*[$widget cget -borderwidth]-4]
     $widget config -wrap $width
  }
}

namespace eval SavedStates \
{
  variable screens   

  variable tiles
  
  variable engines
  
  variable current
  
  variable pre_restore_cmds
  
  variable post_restore_cmds
  
  variable system
  
  set pre_restore_cmds {}
  
  set post_restore_cmds {}
  
  variable finalize_cmds {}
  
  variable nowait -nowait
  
  variable extra_defaults
  
  set extra_defaults {}
  
  array set current \
  {
    label     ""
    date      ""
    serial    ""
    ver       ""
    password  ""
  }
  
  #*************************************************
  #       Name: Catch_errs
  # Parameters: Name of item being loaded, commands
  #Description: Execute a state recall, catching
  #             errors for later display
  #    Returns: none
  #*************************************************
  proc Catch_errs { thing cmd } \
  {
     variable errors
     if [catch $cmd err] \
     {
        append errors "Errors on $thing:\n$err\n$::errorInfo\n"
     }
     # Retire any pending events.
     update idle
  }
  
  #*************************************************
  #       Name: Label
  # Parameters: name of screen
  #Description: Sets the label of the current recalled
  #             state
  #    Returns: none
  #*************************************************
  proc Label { { label  "" } } \
  {
     variable current
     
     set current(label)  $label
     
     set ::Recall_SetupsUI::State(label)  $label
  }
  
  #*************************************************
  #       Name: Date
  # Parameters: date of creation
  #Description: sets the timestamp of the save
  #    Returns: none
  #*************************************************
  proc Date { { date  "" } } \
  {
     variable current
     
     set current(date)  $date
  }
  
  #*************************************************
  #       Name: SerialNum
  # Parameters: Serial #
  #Description: sets the serial # of the save
  #    Returns: none
  #*************************************************
  proc SerialNum { { num  "" } } \
  {
     variable current
     
     set current(serial)  $num
  }
  
  #*************************************************
  #       Name: Version
  # Parameters: Software version #
  #Description: sets the version # of the save
  #    Returns: none
  #*************************************************
  proc Version { { ver  "" } } \
  {
     variable current
     
     set current(ver)  $ver
  }
  
  #*************************************************
  #       Name: Password
  # Parameters: password
  #Description: sets the password of the save
  #    Returns: none
  #*************************************************
  proc Password { { password  "" } } \
  {
     variable current
     
     set current(password)  $password
  }
    
  #*************************************************
  #       Name: cleanup
  # Parameters: input saved state
  #Description: removed leading whitespace
  #    Returns: cleaned up state
  #*************************************************
  proc cleanup { s } \
  {
    regsub -all "\n\[ \\t\]+" $s "\n" s
    
    return  [string trimleft $s]
  }
  
  #*************************************************
  #       Name: Finalize
  # Parameters: none
  #Description: none
  #    Returns: Do any "final" things needed for save state
  #*************************************************
  proc Finalize { } \
  {
     update idle
     set cmd [namespace code Finalize_core]
     after cancel $cmd
     after idle $cmd
  }
  proc Finalize_core {} \
  {
     variable finalize_cmds
     foreach cmd $finalize_cmds \
     {
        after idle [subst {oldCatch "$cmd"}]
     }
     set finalize_cmds ""
  }
  proc Finally { cmd } \
  {
     variable finalize_cmds
     lappend finalize_cmds $cmd
  }
  
  #*************************************************
  #       Name: save_all
  # Parameters: none
  #Description: saves the state of the world
  #    Returns: a big list of commands to restore the
  #             engine states
  #*************************************************
  proc save_all { } \
  {
    append state  [save_info]
    append state  [save_engines]
    append state  [save_ui]
    append state "SavedStates::Finalize"
    return  $state
  }
  
  #*************************************************
  #       Name: save_info
  # Parameters: password - password for the setup
  #Description: saves the info about this saved state.
  #    Returns: a big list of commands to restore the
  #             engine states
  #*************************************************
  proc save_info { { password  "" } } \
  {
    variable current
    
    set ret_value \
    {
      # IFR 2975 saved setup file
      SavedStates::Label     {$current(label)}
      SavedStates::Date      {[clock format [clock seconds]]}
      SavedStates::SerialNum {[IFR_options ser]}
      SavedStates::Version   {[firmware_version]}
      SavedStates::Password  {$password}
    }
    
    set ret_value [subst $ret_value]
    
    set ret_value [subst -nocommands $ret_value]
    # append any commands we may need
    foreach cmd {Catch_errs Finalize Finalize_core Finally} \
    {
       set x ::SavedStates::$cmd
       set c \
       {
          if {\[info commands $x\] == ""} \
          {
             proc $x {[info args $x]} \
             {
                [info body $x]
             }
          }
       }
       append ret_value [subst $c]
    }
    
    set ret_value [cleanup $ret_value]
  }
  
  #*************************************************
  #       Name: read_info
  # Parameters: file name, name of array in parent to fill in
  #Description: reads a file & gets array data
  #    Returns: 1 if valid save file, else 0
  #*************************************************
  proc read_info { file  name } \
  {
    #puts "read_info:  $file,  $name"

    upvar $name  var
    
    set var(label) "<not used>"
    set var(label,valid) 0
    set var(ftime) ""

    if { ![file exist $file] } \
    {
      return  0
    }

    set var(ftime)  [clock format [file mtime $file]]
    
    # try to open the file for reading
    if [catch { set fid [open $file r] } err ] \
    {
       return 0
    }
    
    # scan not more than the first 10 lines of the file
    for { set i  0 } { $i < 10 } { incr i } \
    {
      if { [ gets $fid line ] < 0 } \
      {
        break
      }
       
      if [info complete $line] \
      {
        set cmd  [lindex $line 0]
        set data [lindex $line 1]
         
        # we DON'T want to change the current state, so we will
        # look for the data we want ourselves
        switch -- $cmd \
        {
          SavedStates::Label      { set var(label)        $data 
                                    set var(label,valid)  1     }
                                 
          SavedStates::Date       { set var(ftime)        $data }
           
          SavedStates::SerialNum  { set var(serial)       $data }
           
          SavedStates::Version    { set var(version)      $data }
           
          SavedStates::Password   { set var(password)     $data }
        }
      }
    }
    
    close $fid
    
    return
  }
  
  #*************************************************
  #       Name: order_list
  # Parameters: none
  #Description: Given the NAME of a list, sort it
  #             based upon the items given
  #    Returns: none - variable updated
  #*************************************************
  proc order_list { list_name } \
  {
     upvar $list_name list
     # algorithm: Build a new list by scanning the
     # current list. Look into each namespace given
     # for a SaveSortOrder item - if this exists, use
     # it to determine where to put the list:
     # FIRST - add to head of new list
     # LAST  - add to end of new list
     # BEFORE(namespace) - add before namespace, or first if not given
     # AFTER(namespace) - add after NAMESPACE, or at end if not given.
     
     set to_do_list $list
     set new_list {}
     
     while {$to_do_list != ""} \
     {
        set pass_list $to_do_list
        set to_do_list {}
        set changed 0
        foreach name $pass_list \
        {
           if ![info exists ${name}::SaveSortOrder] \
           {
              lappend new_list $name
              incr changed
              continue
           }
           upvar #0 ${name}::SaveSortOrder order
           switch -glob -- $order \
           {
              FIRST     \
              {
                 set new_list [linsert $new_list 0 $name]
                 incr changed
              }
              LAST      \
              {
                  lappend new_list $name
                  incr changed
              }
              BEFORE(*) \
              {
                 regexp {BEFORE\((.*)\)} $order foo where
                 set index [lsearch $new_list $where]
                 if {$index == -1} \
                 {
                     lappend to_do_list $name
                     continue
                 }
                 set new_list [linsert $new_list $index $name]
                 incr changed
              }
              AFTER(*) \
              {
                 regexp {AFTER\((.*)\)} $order foo where
                 set index [lsearch $new_list $where]
                 if {$index == -1} \
                 {
                     lappend to_do_list $name
                     continue
                 }
                 incr index 1
                 set new_list [linsert $new_list $index $name]
                 incr changed
              }
           }
        }
        # if no change this pass, software error
        if {$changed == 0} \
        {
            # force them onto the list
            foreach name $to_do_list \
            {
               lappend new_list $name
               upvar #0 ${name}::SaveSortOrder order
               append err "   $name: $order\n"
            }
            ErrLog::Log "Bad save state list, items are:\n$err"
        }
     }
     set list $new_list
  }
  
  #*************************************************
  #       Name: save_system
  # Parameters: none
  #Description: saves the state of all system level items
  #      NOTE:  These are NOT saved per-setup, but once
  #             for the whole system.
  #    Returns: a big list of commands to restore the
  #             engine states
  #*************************************************
  proc save_system { } \
  {
    variable system
     
    set ret_value  "\n#Begin global system state save\n\n"
    
    foreach item $system \
    {
      if [catch {set x [${item}::getSaveState] } err ] \
      {
        Debug::Puts Save_State 0 "Error saving state of $item: $err $::errorInfo"
      } \
      else \
      {
        append ret_value "\n# Save of $item\n"
        append ret_value "::SavedStates::Catch_errs $item \\\n{\n"
        append ret_value [cleanup $x]
        append ret_value "}\n"
      }
    }
    append ret_value "update idle\n"
    file mkdir /sysdata/current_state
    set fd [open /sysdata/current_state/system.tcl w]
    puts $fd $ret_value
    close $fd
    return $ret_value
  }
  #*************************************************
  #       Name: save_engines
  # Parameters: none
  #Description: saves the state of all engines.
  #    Returns: a big list of commands to restore the
  #             engine states
  #*************************************************
  proc save_engines { } \
  {
    variable engines
    variable pre_restore_cmds
    variable post_restore_cmds
    
    order_list engines
     
    set ret_value  "\n#Begin system state save\n\n"
     
    append ret_value  "# Pre-restore commands\n"
     
    foreach pre_cmd $pre_restore_cmds \
    {
      append ret_value "[subst $pre_cmd]\n"
    }
    append ret_value "update idle\n"
    
    foreach engine $engines \
    {
      if [catch {set x [${engine}::getSaveState] } err ] \
      {
        Debug::Puts Save_State 0 "Error saving state of $engine: $err $::errorInfo"
      } \
      else \
      {
        append ret_value "\n# Save of $engine\n"
        append ret_value "::SavedStates::Catch_errs $engine \\\n{\n"
        append ret_value [cleanup $x]
        append ret_value "}\n"
      }
    }
    append ret_value "# post restore commands \n"
    append ret_value "update idle\n"
    foreach post_cmd $post_restore_cmds \
    {
      append ret_value "[subst $post_cmd]\n"
    }
    append ret_value "update idle\n"
    return $ret_value
  }
  
  #*************************************************
  #       Name: save_ui
  # Parameters: recall_screen - forced screen to recall
  #Description: saves the state of the UI
  #    Returns: a big list of commands to restore the
  #             UI states
  #*************************************************
  proc save_ui { { recall_screen  "" } } \
  {
    #puts "save_ui:  $recall_screen"
    
    set ret_value "\n#Begin user interface save\n\n"

    append ret_value [save_tiles]
    append ret_value [save_screens]
    
    # If the 'recall_screen' argument is null (defualt), 
    # then save the screen as usual
    if { $recall_screen == "" } \
    {
      append ret_value "::SavedStates::Catch_errs Screens \\\n{\n"
      append ret_value [cleanup [Screens::getSaveState]]
      append ret_value "}\n"
    } \
    else \
    {
      # Else, save the screen as defined by the argument
      append ret_value "::SavedStates::Catch_errs Screens \\\n"
      append ret_value "{\n"
      append ret_value "set ::Screens::Local_screen  $recall_screen\n"
      append ret_value "\n"
      append ret_value "Screens::Show_local\n"
      append ret_value "}\n"
    }      

    #puts "  ret_value:  $ret_value"
    
    return  $ret_value
  }
  
  #*************************************************
  #       Name: save_tiles
  # Parameters: none
  #Description: saves the state of the tiles in the UI
  #    Returns: a big list of commands to restore the
  #             UI states
  #*************************************************
  proc save_tiles { } \
  {
    variable tiles
    
    set ret_value "\n#Begin tile save\n\n"
     
    if ![info exists tiles] { return }
    order_list tiles
         
    foreach tile $tiles \
    {
      if [catch {set x [${tile}::getSaveState] } err ] \
      {
        Debug::Puts Save_state 0 "Error saving state of $tile: $err $::errorInfo"
      } \
      else \
      {
        append ret_value "\n# Save of $tile\n"
        append ret_value "::SavedStates::Catch_errs $tile \\\n{\n"
        append ret_value [cleanup $x]
        append ret_value "}\n"
      }
    }
    
    return $ret_value
  }
  
  #*************************************************
  #       Name: save_screens
  # Parameters: none
  #Description: saves the state of the screens
  #    Returns: a big list of commands to restore the
  #             UI states
  #*************************************************
  proc save_screens { } \
  {
     variable screens
     set ret_value "\n#Begin screen save\n\n"
     if ![info exists screens] { return }
     order_list screens
     foreach screen $screens \
     {
        if [catch {set x [${screen}::getSaveState] } err ] \
        {
           Debug::Puts Save_state 0 "Error saving state of $screen: $err $::errorInfo"
        } \
        else \
        {
           append ret_value "\n# Save of $screen\n"
           append ret_value "::SavedStates::Catch_errs $screen \\\n{\n"
           append ret_value [cleanup $x]
           append ret_value "}\n"
        }
     }
     return $ret_value
  }
}

#*************************************************
#      Name: format_version
# Parameters: cvs_version, 
#Description: strips cvs label of engine name and \040
#*************************************************
proc format_version {cvs_version} {

  # OMIT ALL PRIOR TO AND INCLUDING "\040"
  regsub {^.*:\\040} $cvs_version {} next
  
  # OMIT ENDING "\040"
  regsub {\\040} $next {} rest

  return $rest
}
#*************************************************
#       Name: is_vxworks
# Parameters: none
#Description: detect hardware platform type
#*************************************************
proc is_vxworks { } {
  if { $::tcl_platform(os) == "VxWorks" } {
    return 1
  }

  return 0
}

#*************************************************
#       Name: isnt_vxworks
# Parameters: none
#Description: detect hardware platform type
#*************************************************
proc isnt_vxworks { } {
  if { $::tcl_platform(os) == "VxWorks" } {
    return 0
  }
  return 1
}


#*************************************************
#       Name: children
# Parameters: 
#Description: print a list of all child windows
#*************************************************
proc children { {name "."} {indent "" } } {
  foreach child [winfo children $name] {
    put "$indent $child"
    set z $indent
    append z "  "
    children $child $z
  }
}

#*************************************************
#       Name: 
# Parameters: 
#Description: create a list of fonts
#*************************************************
proc create_fonts {  main } {
  set flist [font names]
  if {$flist != "" } \
  {
     eval font delete [font names]
  }

  # first, try to create a font.
  font create header    -family "helvetica narrow" -size -19 -weight bold

  # now, see if we actually got that font.
  set family [font actual header -displayof $main -family]

  if {"$family" != "helvetica narrow"} {
    Debug::Puts Fonts 0"fonts check: $main helv narrow was $family"
    eval font delete [font names]
    font create Text      -family "times new roman" -size -15
    font create text21    -family "times new roman" -size -15 -weight bold
    font create big       -family "times new roman" -size -20
    font create large     -family "times new roman" -size -25
    font create huge      -family "times new roman" -size -33
    font create small     -family "times new roman" -size -12
    font create smallbold -family "times new roman" -size -12 -weight bold
    font create tiny      -family "times new roman" -size -8
    font create tinybold  -family "times new roman" -size -8 -weight bold
    font create dual_key    -family helvetica -size -8 -weight bold
    font create dual_label  -family helvetica -size -10 -weight bold
    font create analyzer_big   -family helvetica -size -13 -weight bold
    font create analyzer_med   -family helvetica -size -11  -weight bold
    font create analyzer_small -family helvetica -size -10 -weight bold
    font create analyzer_tiny  -family helvetica -size -8  -weight bold

    font create menu_small     -family helvetica -size -11  -weight bold

    font create rcvr_font -family helvetica -size -6
    font create lineplot_font -family times -size 11 

    font create fixed_1  -family fixed -size -12 -weight bold
    font create fixed_2  -family fixed -size -15 -weight bold

    font create Clock      -family "times new roman" -size -15
  } else {
    font create Text      -family "helvetica narrow" -size -19 -weight bold
    font create text21    -family "helvetica narrow" -size -21 -weight bold
    font create big       -family "helvetica narrow" -size -25 -weight bold
    font create large     -family "helvetica narrow" -size -31 -weight bold
    font create huge      -family "helvetica narrow" -size -42 -weight bold
    font create small     -family "helvetica narrow" -size -17
    font create smallbold -family "helvetica narrow" -size -17 -weight bold
    font create tiny      -family "helvetica narrow" -size -15
    font create tinybold  -family "helvetica narrow" -size -15 -weight bold
    font create rcvr_font -family helvetica -size -8
    font create dual_key    -family helvetica -size -10 -weight bold
    font create dual_label  -family helvetica -size -12 -weight bold
    font create analyzer_big    -family helvetica -size -15 -weight bold
    font create analyzer_med    -family helvetica -size -13 -weight bold
    font create analyzer_small  -family helvetica -size -12 -weight bold
    font create analyzer_tiny   -family helvetica -size -10 -weight bold

    font create menu_small      -family helvetica -size -13 -weight bold
    font create lineplot_font -family "helvetica narrow" -size 14
    
    font create fixed_1  -family fixed -size -12 -weight bold
    font create fixed_2  -family fixed -size -15 -weight bold

    font create Clock    -family "helvetica" -size -16 -weight bold
 }
}

namespace eval Mouse_cursor \
{
   variable State
   array set State \
   {
      current {}
      desired {}
      hiding  0
      hide_delay 3000
   }
   bind Mouse_cursor <Destroy> {Mouse_cursor::Destroy %W}

#*************************************************
#       Name: Destroy
# Parameters: toplevel this applies to
#Description: unhook ourselves from mouse cursor events
#*************************************************
   proc Destroy { widget } \
   {
      Debug::Puts Mouse_cursor 1 "Removing hook for $widget"
      set cmd "::Mouse_cursor::update_mouse $widget"
      trace vdelete ::Mouse_cursor::State(current) w $cmd
   }
#*************************************************
#       Name: Hook
# Parameters: toplevel this applies to
#Description: hook ourselves into mouse cursor events
#*************************************************
   proc Hook { widget } \
   {
      Debug::Puts Mouse_cursor 1 "Hooking up $widget"
      add_bindtag $widget Mouse_cursor
      set cmd "::Mouse_cursor::update_mouse $widget"
      trace vdelete  ::Mouse_cursor::State(current) w $cmd
      trace variable ::Mouse_cursor::State(current) w $cmd
      update_mouse $widget
   }
#*************************************************
#       Name: update_mouse
# Parameters: toplevel this applies to
#Description: update a toplevel's idea of what to
#             show as a mouse cursor
#*************************************************
  proc update_mouse { widget args} \
  {
     variable State
     Debug::Puts Mouse_cursor 2 "update_mouse $widget to $State(current)"
     if [winfo exists $widget] \
     {
        # sometimes the system barfs if we set the mouse to the same thing -
        # it forgets to tell X. So, force it to something completely different
        $widget configure -cursor $State(current)
     } \
     else \
     {
        Destroy $widget
        Debug::Puts Mouse_cursor 1 "update_mouse for dead widget $widget"
     }
  }
   
#*************************************************
#       Name: show_mouse
# Parameters: 
#Description: 
#*************************************************
  proc show_mouse { { cursor {left_ptr}} } \
  {
    variable State
    Debug::Puts Mouse_cursor 3 "show_mouse $cursor"
    set State(desired) $cursor
  
    unhide_mouse 
  }

#*************************************************
#       Name: hide_mouse
# Parameters: 
#Description: 
#*************************************************
  proc hide_mouse {} \
  {
    return
    variable State
    Debug::Puts Mouse_cursor 3 "hide_mouse"
#    set State(hiding) 1
#    set State(current) "@/system/rtx/icons/none white"
  
    bind all <Motion> Mouse_cursor::unhide_mouse
  }

#*************************************************
#       Name: unhide_mouse
# Parameters: 
#Description: 
#*************************************************
  proc unhide_mouse {} \
  {
    variable State
    Debug::Puts Mouse_cursor 3 "unhide_mouse"
    set State(hiding) 0
    set State(current) $State(desired)
    restart_hide_delay
  }

#*************************************************
#       Name: rebind_motion
# Parameters: 
#Description: 
#*************************************************
  proc rebind_motion {} \
  {
     Debug::Puts Mouse_cursor 4 "rebind_motion"
     bind all <Motion> Mouse_cursor::restart_hide_delay
  }
#*************************************************
#       Name: restart_hide_delay
# Parameters: 
#Description: 
#*************************************************
  proc restart_hide_delay {} \
  {
    variable State
    Debug::Puts Mouse_cursor 4 "restart_hide_delay"
    bind all <Motion> {}
    after 500 Mouse_cursor::rebind_motion
    
    after cancel Mouse_cursor::hide_mouse
    after $State(hide_delay) Mouse_cursor::hide_mouse
  }
}

#****************************************************************************
# Name:        Busy/NotBusy
# Parameters:  none
# Description: Indicate busy - do not allow new events
#****************************************************************************
variable busy_count 0
proc Busy {  } \
{
     Debug::Puts Busy 1 "Busy up"
     variable busy_count
     
     if {[incr busy_count] == 1} \
     {
        Debug::Puts Busy 1 "Now busy"
        if {[info command TraceWidget] != ""} { TraceWidget -disable}
        Mouse_cursor::show_mouse watch
     }
}
proc NotBusy {  } \
{
     Debug::Puts Busy 1 "Busy down"
     variable busy_count
     
     if {[incr busy_count -1] <= 0} \
     {
        set busy_count 0
        if {[info command TraceWidget] != ""} {TraceWidget -enable}
        Mouse_cursor::show_mouse
        Debug::Puts Busy 1 "Now not busy"
     }
}
set Debug::enable(Busy) 1
proc newUpdate { {arg ""} } \
{
   variable busy_count
   if {$busy_count > 0} \
   {
      if {$arg == "" } \
      {
        set rv "Tcl Call Trace\n"
        set indent "->"
        set level [info level ]
        for {set i 1 } { $i < $level } { incr i } \
        {
            append rv "$indent [info level $i]\n"
            set indent "-$indent"
        }
        ErrLog::Log "bare update called from \n$rv"
      }
      oldUpdate idle
      return
   }
   oldUpdate $arg
}
rename update oldUpdate
rename newUpdate update

#*************************************************
#       Name: title_mode
# Parameters: text
#Description: displays text in mode bar in title menu
#*************************************************
proc title_mode { text } {   
  # BODGE BODGE FIXME
  Debug::Puts DEPRECATED 0 "Deprecated function title_mode called"
  CallTrace
}

#*************************************************
#       Name: powerdown_message
# Parameters:
#Description: gives are you sure message
#*************************************************
proc powerdown_message { {widget .} } {
  if {$::sys(powerdown_allowed) != 1} \
  {
     return
  }
  if {$::sys(powerdown_sure)} \
  {
    set top [View::Find_Main_toplevel $widget]
    set msg [name_child $top powerdownsure]
    message_box $msg QUESTION
  } \
  else \
  {
    powerdown
  }
}


#*************************************************
#       Name: powerdown
# Parameters:
#Description: handles power down of box
#*************************************************
proc powerdown { {reboot_flag 0} } \
{
   after 200 "powerdown_core $reboot_flag"
}
proc powerdown_core { {reboot_flag 0} } \
{
   if {$::sys(powerdown_allowed) != 1} \
   {
      return
   }
   # schedule a fail-safe, do it damninit shutdown in 60 seconds
   shutdown $reboot_flag
   Debug::Puts Powerdown 1 "Calling at-exit functions"
   # call any registered "at exit" type functions
   foreach item $::sys(at_exit) \
   {
      Debug::Puts Powerdown 2 "Calling $item"
      catch {eval $item}
   }

   update idle
   
   # save the system state
   Debug::Puts Powerdown 1 "Get current state"
   # Turn off the trace widget updates
   TraceWidget -disable
   set state [SavedStates::save_all]
   SavedStates::save_system
   
  
   #Turn off message handling
   Debug::Puts Powerdown 1 "Turning off the message bus"
   Engmblink 0

   Debug::Puts Powerdown 1 "Save current state"
   if [catch \
       {
        file mkdir /sysdata/current_state/
        set f [open /sysdata/current_state/powerdown.tcl w]
        puts $f $state
        close $f
       } err
      ] \
   {
      Debug::Puts Powerdown 0 "Error saving new state - $err $::errorInfo"
   }
   Debug::Puts Powerdown 1 "Telling options system"
   IFR_options powerdown
   Debug::Puts Powerdown 1 "Buh-bye"
   after idle "real_exit $reboot_flag"
}
# move the exit command over
rename exit real_exit
interp alias {} exit {} powerdown

#*************************************************
#       Name: check_menu_focus
# Parameters: menu to check
#Description: If focus moves from this menu or one
#             of its children, unpost this menu
#*************************************************
proc check_menu_focus { widget } \
{
  #puts "check_menu_focus (main):  $widget"

  set new_focus  [focus]
  
  #puts "  new_focus:  $new_focus"

  if { [string  match  $widget*  $new_focus] } \
  { 
    #puts "  string match"

    return 
  }

  #puts "  NO string match, UNPOST:  $widget"

  tkMenuUnpost  $widget
  
  return
}

proc set_menu_spinner_mode { } \
{
   if ![info exists ::sys(menu_reverse)] \
   {
      set ::sys(menu_reverse) 0
   }
   # BIND SPINNER KNOB TO MENU CURSOR MOVEMENT
   if {$::sys(menu_reverse) == 0 } \
   {
     bind Menu <<Spin_down>> {event generate %W <Down>; break}
     bind Menu <<Spin_up>>   {event generate %W <Up>; break}
     set ::sys(menu_reverse,text) NORMAL
   } \
   else \
   {
     bind Menu <<Spin_down>> {event generate %W <Up>; break}
     bind Menu <<Spin_up>>   {event generate %W <Down>; break}
     set ::sys(menu_reverse,text) REVERSE
   }
}
proc toggle_reverse_menu_spinner {} \
{
   if {$::sys(menu_reverse) == 0 } \
   {
      set ::sys(menu_reverse) 1
   } \
   else \
   {
      set ::sys(menu_reverse) 0
   }
   set_menu_spinner_mode
}

#*************************************************
#       Name: bind_keys
# Parameters: base, widget name holding main
#Description: performs general binds
#*************************************************
proc bind_keys { } {
   upvar base base
   
   # NOTE WELL: This is the ONLY place you should be
   #            accessing the F2x and up keysyms.
   #            USE THE VIRTUAL EVENTS EVERY WHERE ELSE!
   # Add virtual bindings for key names
   # First, the bindings for our local keyboard
   event add <<Spin_down>> <F26>
   event add <<Spin_up>>   <F27>
   event add <<Vol_down>>  <F28>
   event add <<Vol_up>>    <F29>
   event add <<Squ_down>>  <F30>
   event add <<Squ_up>>    <F31>
   event add <<Help>>      <F1>
   event add <<Hold>>      <F10>
   
   # Now the bindings for a remote session
   event add <<Spin_down>> <Next>
   event add <<Spin_up>>   <Prior>
   event add <<Vol_down>>  <Shift-Down>
   event add <<Vol_up>>    <Shift-Up>
   event add <<Squ_down>>  <Control-Down>
   event add <<Squ_up>>    <Control-Up>
   
   # Common bindings
   
   bind all <Key-F9>        "Shift::Mode_press   %W"
   bind all <KeyRelease-F9> "Shift::Mode_release %W"
   event add <<Power>>      <KeyRelease-F11>
   event add <<PowerPress>> <KeyPress-F11>
   
  foreach {keypad key} {  KP_0 0 KP_1 1 KP_2 2 KP_3 3 KP_4 4 KP_5 5 KP_6 6 KP_7 7 
                          KP_8 8 KP_9 9
                          KP_Add plus KP_Decimal period KP_Subtract minus
                          KP_Multiply asterisk KP_Divide slash
                       } \
  {
     bind all <Key-$keypad>        "event generate %W <Key-$key>"
     bind all <KeyRelease-$keypad> "event generate %W <KeyRelease-$key>"
  }
   # BIND SPINNER KNOB TO MENU CURSOR MOVEMENT
   set_menu_spinner_mode

   # prevent cursor keys from going any furthur in menus
   bind Menu <Key-Down> {+break}
   bind Menu <Key-Up> {+break}
   bind Menu <Key-Right> {+break}
   bind Menu <Key-Left> {+break}
   bind Menu <KP_Down> {+break}
   bind Menu <KP_Up> {+break}
   bind Menu <KP_Right> {+break}
   bind Menu <KP_Left> {+break}

   # if we cted to an X server with a scroller, add it in too
   event add <<edit_up>>   <4>
   event add <<edit_down>> <5>
   # set it up so that a mouse click closes the current edit and
   # starts a new edit
   bind all <1> { 
        if {[winfo exists %W] && [%W cget -takefocus]} \
        { 
         focus %W
        } \
        else \
        {
           set cur_focus [focus -displayof %W]
           if {$cur_focus != ""} \
           {
              event generate $cur_focus <<EditClose>>
           }
        }
   }
         
   # POWER KEY, STORES LAST MODE BEFORE POWEROFF
   bind all <<Power>> {powerdown_message %W}


  # SUPPORTS ENTER KEY USE OF BUTTONS
  bind Button <KP_Enter>            {%W config -relief sunken}
  bind Button <KeyRelease-KP_Enter> {%W config -relief raised;after idle "if [winfo exists %W] {%W invoke}"}
  bind Button <Return>              {%W config -relief sunken}
  bind Button <KeyRelease-Return>   {%W config -relief raised;after idle "if [winfo exists %W] {%W invoke}"}
  
  # BIND SPINNER TO LEFT RIGHT KEYS TO PERFORM CURSOR MOVEMENT
  # EXISTS CHECK ADDED, SOMEHOW BINDING INVOKED AFTER WIDGET DESTROYED (ON TOPLEVEL)
  bind all <<Spin_down>> {
     if [winfo exists %W] {
        if {$::sys(use_spinner_edit)} {
          event generate %W <<edit_down>>
        } else {
          event generate %W <Left>
        }
     }
  }
  
  bind all <<Spin_up>>   {
     if [winfo exists %W] {
        if {$::sys(use_spinner_edit)} {
          event generate %W <<edit_up>>
        } else {
          event generate %W <Right>
        }
     }
  }
  

  # BIND UP, DOWN SO CASCADES DONT GET FOCUS AUTOMATICALLY
  bind Menu <Up>    {tkMenuNextEntry_Altered %W -1}
  bind Menu <Down>  {tkMenuNextEntry_Altered %W 1}



  # BIND HOLD TO "NOT AVAILABLE MESSAGE"
  bind all <<Hold>> {Grab_screen %W}
  # just for safety, create the capture dir if needed
  file mkdir /usr/pictures
}

#*************************************************
#       Name: Grab_screen
# Parameters: widget - item on screen to grab
#Description: save a screen/toplevel to a file
#*************************************************
proc Grab_screen { widget } \
{
   set toplevel [winfo toplevel $widget]
   set dir /usr/pictures
   file mkdir $dir
   image create photo snap
   Capture image snap $toplevel
   set time [clock format [clock seconds] -format "%Y%m%d%H%M%S"]
   set name [file join $dir cap$time.gif]
   set inc 0
   while [file exists $name] \
   {
      incr inc
      set name [file join $dir cap${time}-$inc.gif]
   }
   catch \
   {
      set fd [open $name w]
      fconfigure $fd -translation binary -buffering full -buffersize 160000
      snap write $fd -format gif
   }
   if [info exists fd] \
   {
      close $fd
   }
   image delete snap
}

#*************************************************
#       Name: OpenCalFile
# Parameters: filename - file name to use
#             attr     - attributes
#Description: open cal file, if not open then
#             find default file, copy into filename
#*************************************************
proc OpenCalFile { filename attr } \
{
  set fname [file join /system/calibration $filename]
  
  if ![file exists $fname] \
  {
    file copy -force $fname.default $fname
  }
  if [ catch \
       {
         set file [open /system/calibration/$filename $attr]
       } err 
     ] \
  {
     return -code error "Unable to open $filename: $err"
  } 

  return $file
}

#*************************************************
#       Name: start_main_screen
# Parameters: name of toplevel of frame to hold
#Description: draws, starts intro screen
#*************************************************
proc start_main_screen { } \
{
  puts "start_main_screen"
  trace variable Startup::loaded_count w update_ui_loadbar
  if {"$Load::State(VERBOSE_BOOT)"!="" } \
  {
     Load::load_ui_message Startup::file_name
  }

  Memory::SetFree startScreen
  
  Startup::CountDir /system/utilities
  Startup::CountDir /system/options
  Startup::CountDir /system/tiles
  Startup::CountDir /system/screens

  # SOURCE USER INTERFACE SCREENS, BUILD MODE MENU
  Startup::SourceDir /system/utilities
  Startup::SourceDir /system/options
  Startup::SourceDir /system/tiles
  Startup::SourceDir /system/screens

  Memory::SetFree UIscreensDone
  
  trace vdelete Startup::loaded_count w update_ui_loadbar
}   

#*************************************************
#       Name: start_main_engines
# Parameters: 
#Description: 
#*************************************************
proc start_main_engines {} \
{ 
    
  # in case the power term overload is tripped, reset it
  Power debug olreset -nowait
}

wm proto . WM_TAKE_FOCUS { focus -force [focus -lastfor .]; Debug::Puts verify_focus 1 "Took focus to [focus]"}
wm focus . active
#*************************************************
#       Name: verify_focus
# Parameters: 
#Description: if we lose focus on the main screen,
#             restore it
#*************************************************
proc verify_focus { } \
{
   variable disable_list
   variable focus_fix_retries
   
   Debug::Puts verify_focus 1 "Verifying focus"
   
   # if anybody has disabled focus checking, skip this
   if {"[array names disable_list]" != "" } { return }
   
   Debug::Puts verify_focus 1 "Verify focus is enabled"

   after cancel verify_focus
   after 1000 verify_focus
   set f [focus -displayof .]
   if {$f != "" } \
   {
      if [winfo ismapped $f] \
      {
         set focus_fix_retries 2
         return
      }
      Debug::Puts verify_focus 1 "Focus lost? focus $f isn't mapped"
      if {[incr focus_fix_retries -1] > 0}\
      {
         Debug::Puts verify_focus 1 "Retry is $focus_fix_retries"
         return 
      }
   }
   # focus either dead or not mapped. See if some other app has focus
   set pointerwin [winfo containing [winfo pointerx .] [winfo pointery .]]
   # if over one of our windows, force focus.
   if [winfo exists $pointerwin] \
   {
        set last [focus -lastfor $pointerwin]
        if {[winfo exists $last] && [winfo ismapped $last] && [$last cget -takefocus]} \
        {
           focus -force $last
        } \
        else \
        {
          set last_chance 0
          set candidate_list $pointerwin
          while [llength $candidate_list] \
          {
             Debug::Puts verify_focus "Considering $candidate_list"
             set possibles  $candidate_list
             set candidate_list {}
             foreach candidate $possibles \
             {
                if ![winfo ismapped $candidate] \
                {
                    Debug::Puts verify_focus "$candidate is not mapped"
                    continue
                }
                if [$candidate cget -takefocus] \
                {
                   focus -force $candidate
                   Debug::Puts verify_focus 1 "Forced focus to pointer $pointerwin $candidate - [focus]"
                   return
                } \
                else \
                {
                   Debug::Puts verify_focus "$candidate is not focusable - adding children"
                   set candidate_list [concat $candidate_list [winfo children $candidate]]
                }
             }
             if {![llength $candidate_list] && !$last_chance} \
             {
                incr last_chance
                set candidate_list [winfo toplevel $pointerwin]
                Debug::Puts verify_focus "Last chance for $pointerwin"
             }
          }
        }
        Debug::Puts verify_focus 1 "Forced focus to pointer $pointerwin - [focus]"
        return
   }
}
array set disable_list {}
set focus_fix_retries 2

#*************************************************
#       Name: disable_verify_focus
# Parameters: name of why we are disabling focus
#Description: Disable the verify_focus function.
#*************************************************
proc disable_verify_focus { name } \
{
   variable disable_list
   set disable_list($name) $name
   Debug::Puts disable_verify_focus 1 "Disabling focus because of $name"
}

#*************************************************
#       Name: enable_verify_focus
# Parameters: name of why we are enabling focus
#Description: counterpart to disable_verify_focus -
#             removes a disable
#*************************************************
proc enable_verify_focus { name } \
{
   variable disable_list
   if [info exists disable_list($name) ] \
   {
      unset disable_list($name)
   }
   Debug::Puts disable_verify_focus 1 "Enabling focus because of $name"
   verify_focus
}

#*************************************************
#       Name: yigcal_done
# Parameters: window for the recal message,
#             other args (ignored)
#Description: Update the unlock window as needed.
#*************************************************
proc yigcal_done { window args } {
   if {"$::RcvrYigCals(temp)" == "Running" } { return }
   destroy $window
   trace vdelete ::RcvrYigCals(temp) w "yigcal_done $window"
}

#*************************************************
#       Name: start_main
# Parameters: 
#Description: 
#*************************************************
proc start_main { } \
{
  puts "Create fonts"
  create_fonts .
  
  # add a "forget your normal state" function
  bind .auto <F1><F8> {file delete /sysdata/current_state/powerdown.tcl}
  focus .auto
  
  # clean up any trash left over from last run
  foreach filematch {/system/etc/ifcfg_eth0.bak /usr/cache/*} \
  {
     foreach file [glob -nocomplain $filematch] \
     {
        file delete $file
     }
  }


  Load::on_system_done \
  {
     after idle {
       Load::load_hardware_message "Starting system"
       start_main_engines
     }
     
     HW_control::OnYigDone \
     {
       Load::update_hardware_loadbar 66
       Load::load_hardware_message "Init Hardware"
       HW_control ref $::sys(10mhzref)
  
       Load::hwinit_done
  
       Load::update_hardware_loadbar 100
       Load::load_hardware_message "Done"

       # in case the power term overload is tripped, reset it
       Power debug olreset 
     }
  }
  Debug::Puts Startup 1 "Load system"
  BGCommand Load::load_system
  # Wait for engine and DSP loads done
  while {$::Load::State(DONE) != 1} \
  {
     vwait ::Load::State(DONE)
  }

  Debug::Puts Startup 1 "start screen"
  start_main_screen
  Debug::Puts Startup 1 "wait for system done"
  Load::on_system_done start_main_part2
}

proc start_main_part2 { } \
{
  Debug::Puts Startup 1 "start_main_part2 "
  # Release tasks that are blocked waiting for the UI to load
  Load::load_UI_done
   
  # to prevent us from ever completely losing keyboard focus,
  # set up a monitor to force focus back to the main screen.
  verify_focus
  add_bindtag . FocusFixer
  bind FocusFixer <FocusOut> verify_focus
  
  # set up the standard key connections
  bind_keys
  
  # set up the main screen on the local display
  View::CreateView  .
  # not ready to de-iconify just yet, so cancel the default de-iconify command
  after cancel "wm deiconify ."
  
  Load::load_ui_message "Waiting Yig Calibration"
  HW_control::WaitForYigDone
  
  Load::load_ui_message "Restoring instrument state"

  # put filesystem into "slow but safe" mode
  exec make_hda1_fast 0
  
  # attempt to recover the last system state
  if { [file exists /sysdata/current_state/powerdown.tcl] } \
  {
     set ::Recall_SetupsUI::State(recall_in_process) 1
     catch { source /sysdata/current_state/powerdown.tcl }
     set ::Recall_SetupsUI::State(recall_in_process) 0
  } \
  else \
  {
     set ::Recall_SetupsUI::State(recall_in_process) 1
     catch { source /system/default_state.tcl }
     set ::Recall_SetupsUI::State(recall_in_process) 0
  }
  Load::load_ui_message "Done"

  # Initialize the web server if needed
  HTTPD::Init

  Debug::Puts Startup 1 "Release startup busy lock" 
  NotBusy
  # wait 'til not busy
  while {$::busy_count > 0} \
  {
     Debug::Puts Startup 1 "busy lock is $::busy_count" 
     vwait ::busy_count
     update idle
  }
  Debug::Puts Startup 1 "Wait done" 
  # DESTROY WAIT SCREEN and show main screen
  Busy
  Wait_done
  Debug::Puts Startup 1 "Hide mouse" 
  after idle "Mouse_cursor::hide_mouse"
  after idle NotBusy
}

#************************************************* 
#       Name: update_ui_loadbar
# Parameters: none
#Description: update ui loading status
#*************************************************
proc update_ui_loadbar { args } \
{
  Load::loadbar_update ui $Startup::loaded_count $Startup::file_count blue
}

#*************************************************
#       Name: write_display
# Parameters: 
#Description: write display file
#*************************************************  
proc write_display {} {
  catch {
    set fileId [open /system/etc/master_setup w]
    set err ""
  } err

  if {$err != ""} {
    set ::sys(hdisplay) ":0"
    Debug::Puts Display 0 "couldn't open setup file: err was $err"
    return
  } 

  puts $fileId "hdisplay $::sys(hdisplay)"
  close $fileId
}

#*************************************************
#       Name: Exception
# Parameters: 
#Description: On a task fault, write the exception
#             to the log file, and exit
#*************************************************  
proc ::Exception { vector vec_name pc func task_name task} \
{
   set entry "Exception: $vector $vec_name 0x[format %08X $pc] $func $task_name 0x[format %08X $task]"
   ErrLog::Log $entry
   if ![info exists Debug::enable(NO_REBOOT)] \
   {
      powerdown 1
   } \
   else \
   {
      bgerror $entry
   }
}
proc main_entry { } \
{
set Debug::enable(Startup) 3

Busy
Memory::SetFree beforeALL
package require ALL
ALL

Memory::SetFree startMain
start_main
Memory::SetFree afterMain

# read system level setup
if [file exists /sysdata/current_state/system.tcl] \
{
   BGSource /sysdata/current_state/system.tcl
}
}
after idle main_entry
