#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C) 2001      DESIGN ENGINEERING        #
# --------------------------------------------------------------------------#
# FILE        : 2040.scope.tcl                                              #
# PRODUCT     : FITS                                                        #
# AUTHOR      : Bruce Butterfield                                           #
# DESCRIPTION : This namespace manages the Scope Screen                     #
#############################################################################
#                                                                           #
# Required Variables:                                                       #
#    LongName           Name of screen                                      #
#    ShortName          Name of screen for menus                            #
#    RecomendedTiles    List of tiles available for this screen. It is not  #
#                       necessary to put a tile on this list that is on the #
#                       RequiredTiles list                                  #
#    NotAllowedTiles    List of tiles NEVER on this screen                  #
#    RequiredTiles      List of tiles ALWAYS on this screen                 #
#    ActiveTiles        List of tiles currently displayed on this screen    #
#    MenuFlags          0 = Active Screen                                   #
#                       1 = Currently functioning as a submenu parent       #
#                                                                           #
# Required Variables for submenu "Screens":                                 #
#    SubmenuScreens     List of all tiles (or submenus) for droplist        #
#    SubDirPath         Path to subdirectory holding the screens for the    #
#                       submenu of this "screen"                            #
#                                                                           #
#############################################################################
#                                                                           #
# Dummy Functions:                                                          #
#    Setup              Called by the screen manager to create the screen.  #
#    Map                Called when screen is mapped on window              #
#    Unmap              Called when screen is removed from display          #
#    Destroy            Called when screen is destroyed                     #
#                                                                           #
#############################################################################

namespace eval Scope_Screen \
{
  variable LongName
  variable ShortName
  variable RecommendedTiles
  variable NotAllowedTiles
  variable RequiredTiles
  variable ActiveTiles
  variable MenuFlags
  variable SubmenuScreens
  variable SubDirPath

  set LongName scope_Screen
  set ShortName "Oscilloscope"
  
  set RequiredTiles [list FullScopeUI]

  set NotAllowedTiles { }
  set RecommendedTiles { }
  set ActiveTiles { }
  set MenuFlags 0
  set SubmenuScreens { }
  set SubDirPath { }


  #****************************************************************************
  # Name:        Setup
  # Parameters:  parent		parent of child window
  # Description: Dummy function
  # Returns:     Indicator that no tile is generated
  #****************************************************************************
  proc Setup { parent } \
  {
    # We will use the scope tile to pack into the parent window
    set topWin  [winfo toplevel  $parent]
    
    # We need to generate our options menu
    set dropMenu  [ScopeUI::set_scope_options_menu  $topWin]
    
    View::PackOptions  $topWin  $dropMenu

    Screens::ShowRequiredTiles  $topWin  Scope_Screen

    # When the scope tile is available, send this back to the main screen
    return  ""
  }


  #****************************************************************************
  # Name:        Map
  # Parameters:  parent		parent of child window
  # Description: Dummy function
  #****************************************************************************
  proc Map { name } \
  {
    
  }


  #****************************************************************************
  # Name:        Unmap
  # Parameters:  parent		parent of child window
  # Description: Dummy function
  #****************************************************************************
  proc Unmap { name } \
  {
  
  }


  #****************************************************************************
  # Name:        Destroy
  # Parameters:  parent		parent of child window
  # Description: Dummy function
  #****************************************************************************
  proc Destroy { name } \
  {
    
  }


  #****************************************************************************
  # Name:        getSaveState
  # Parameters:  none
  # Description: This function returns a list of TCL commands which can be used
  #              to set the Scope_Screen screen to its current state
  #****************************************************************************
  proc getSaveState { } \
  {
    set returnList \
    {
      
    }
  
    return [subst $returnList]
  }


  #****************************************************************************
  # Functions executed when the namespace is sourced
  #****************************************************************************
  # Add this on the menu
  ::View::RegisterScreen Scope_Screen


};  # End of the Scope_Screen



#****************************************************************************
# This registers the screens with the list for saving instrument states
#****************************************************************************
namespace eval ::SavedStates \
{
  variable screens   

  lappend screens Scope_Screen
}

