##########################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C) 2001      DESIGN ENGINEERING        #
# --------------------------------------------------------------------------#
# FILE        : 2065.Smartnet.tcl                                           #
# PRODUCT     : FITS                                                        #
# AUTHOR      : Preston McCormick                                           #
# DESCRIPTION : Script for the SmartNet / SmartZone screen.                 #
#############################################################################
#                                                                           #
# Required Variables:                                                       #
#    LongName           Name of screen                                      #
#    ShortName          Name of screen for menus                            #
#    RecomendedTiles    List of tiles available for this screen. It is not  #
#                       necessary to put a tile on this list that is on the #
#                       RequiredTiles list                                  #
#    NotAllowedTiles    List of tiles NEVER on this screen                  #
#    RequiredTiles      List of tiles ALWAYS on this screen                 #
#    ActiveTiles        List of tiles currently displayed on this screen    #
#    MenuFlags          0 = Active Screen                                   #
#                       1 = Currently functioning as a submenu parent       #
#                                                                           #
# Required Variables for submenu "Screens":                                 #
#    SubmenuScreens     List of all tiles (or submenus) for droplist        #
#    SubDirPath         Path to subdirectory holding the screens for the    #
#                       submenu of this "screen"                            #
#                                                                           #
#############################################################################
#                                                                           #
# Dummy Functions:                                                          #
#    Setup              Called by the screen manager to create the screen.  #
#    Map                Called when screen is mapped on window              #
#    Unmap              Called when screen is removed from display          #
#    Destroy            Called when screen is destroyed                     #
#                                                                           #
#############################################################################

namespace eval SmartNet_Screen \
{
  variable LongName
  variable ShortName
  variable RecommendedTiles
  variable NotAllowedTiles
  variable RequiredTiles
  variable ActiveTiles
  variable MenuFlags
  variable SubmenuScreens
  variable SubDirPath

  set LongName   "smartnet_Screen"
  set ShortName  "SmartNet / SmartZone"
  
  set RequiredTiles     { }
  
  #set RecommendedTiles  [ list SmartNet_Repeat_SimUI   SmartNet_Mob_SimUI     \
  #                             SmartNet_MonitorUI      SmartNet_Rec_MobileUI  \
  #                             SmartNet_Init_MobileUI  SmartNet_User_CmdUI    \
  #                             SmartNet_ScannerUI  ]

  set RecommendedTiles  [ list SmartNet_Repeat_SimUI   SmartNet_MonitorUI  \
                               SmartNet_ScannerUI  ]

  set NotAllowedTiles   { }
  set ActiveTiles       [ list SmartNet_ScannerUI  SmartNet_MonitorUI ]
  set SubmenuScreens    { }
  set SubDirPath        { }
  set MenuFlags         0


  #****************************************************************************
  # Name:        Setup
  # Parameters:  parent		parent of child window
  # Description: Dummy function
  # Returns:     Indicator that no tile is generated
  #****************************************************************************
  proc Setup { parent } \
  {
    # Use the Screen functions to manage our tiles
    set topWin  [winfo toplevel $parent]
     
    # Use the Screen functions to manage our options menu
    set dropMenu  [Screens::Build_std_options  $topWin  SmartNet_Screen]
     
    View::PackOptions  $topWin  $dropMenu

    View::PackKeyBar  $topWin

    Screens::ShowRequiredTiles  $topWin  SmartNet_Screen

    return  ""
  }
 
 
  #****************************************************************************
  # Name:        Map
  # Parameters:  parent		parent of child window
  # Description: Dummy function
  #****************************************************************************
  proc Map { name } \
  { 
    
  }


  #****************************************************************************
  # Name:        Unmap
  # Parameters:  parent		parent of child window
  # Description: Dummy function
  #****************************************************************************
  proc Unmap { name } \
  { 
    
  }

  
  #****************************************************************************
  # Name:        Destroy
  # Parameters:  parent		parent of child window
  # Description: Dummy function
  #****************************************************************************
  proc Destroy { name } \
  { 
    
  }


  #****************************************************************************
  # Name:        getSaveState
  # Parameters:  none
  # Description: This function returns a list of TCL commands which can be used
  #              to set the SmartNet_Screen screen to its current state
  #****************************************************************************
  proc getSaveState { } \
  {
    set returnList \
    {
      
    }
   
    return [subst $returnList]
  }

  
  #****************************************************************************
  # Functions executed when the namespace is sourced
  #****************************************************************************
  # Add this on the menu
  #::View::RegisterScreen SmartNet_Screen
  
  
};  # End of the SmartNet_Screen



#****************************************************************************
# This registers the screens with the list for saving instrument states
#****************************************************************************
namespace eval ::SavedStates \
{
  variable screens   

  lappend screens SmartNet_Screen
}

