#############################################################################
# IFR, An Aeroflex Co.       COPYRIGHT (C) 2003        DESIGN ENGINEERING   #
# --------------------------------------------------------------------------#
# FILE        : 2051.audio_anlz.tcl                                         #
# PRODUCT     : FITS                                                        #
# AUTHOR      : Preston McCormick                                           #
# DESCRIPTION : This namespace manages the Audio Analyzer Screen            #
#############################################################################
#                                                                           #
# Required Variables:                                                       #
#    LongName           Name of screen                                      #
#    ShortName          Name of screen for menus                            #
#    RecomendedTiles    List of tiles available for this screen. It is not  #
#                       necessary to put a tile on this list that is on the #
#                       RequiredTiles list                                  #
#    NotAllowedTiles    List of tiles NEVER on this screen                  #
#    RequiredTiles      List of tiles ALWAYS on this screen                 #
#    ActiveTiles        List of tiles currently displayed on this screen    #
#    MenuFlags          0 = Active Screen                                   #
#                       1 = Currently functioning as a submenu parent       #
#                                                                           #
# Required Variables for submenu "Screens":                                 #
#    SubmenuScreens     List of all tiles (or submenus) for droplist        #
#    SubDirPath         Path to subdirectory holding the screens for the    #
#                       submenu of this "screen"                            #
#                                                                           #
#############################################################################
#                                                                           #
# Dummy Functions:                                                          #
#    Setup              Called by the screen manager to create the screen.  #
#    Map                Called when screen is mapped on window              #
#    Unmap              Called when screen is removed from display          #
#    Destroy            Called when screen is destroyed                     #
#                                                                           #
#############################################################################

namespace eval Audio_Analyzer_Screen \
{
  variable LongName   audio_analyzer_Screen
  variable ShortName  "Audio Analyzer"

  variable RecommendedTiles  {}
  variable NotAllowedTiles   {}
  variable RequiredTiles     [list Audio_Analyzer_FullUI]
  variable ActiveTiles       {}

  variable MenuFlags       0
  variable SubmenuScreens  {}
  variable SubDirPath      {}

  variable Accel "8"



  #****************************************************************************
  # Name:        Setup
  # Parameters:  parent   parent of child window
  # Description: Dummy function
  # Returns:     Indicator that no tile is generated
  #****************************************************************************
  proc Setup { parent } \
  {
    # Use the Screen functions to manage our tiles
    set topWin  [winfo toplevel  $parent]
    
    set dropMenu  \
     [Audio_Analyzer_FullUI::set_options_menu  $topWin  Audio_Analyzer_Screen]

    View::PackOptions  $topWin  $dropMenu
    
    View::PackKeyBar  $topWin

    Screens::ShowRequiredTiles  $topWin  Audio_Analyzer_Screen
     
    # We are not generating a tile to pack into the main screen to return NULL
    return  ""
  }


  #****************************************************************************
  # Name:        Map
  # Parameters:  parent   parent of child window
  # Description: Dummy function
  #****************************************************************************
  proc Map { name } \
  {
    
  }


  #****************************************************************************
  # Name:        Unmap
  # Parameters:  parent   parent of child window
  # Description: Dummy function
  #****************************************************************************
  proc Unmap { name } \
  {
    
  }


  #****************************************************************************
  # Name:        Destroy
  # Parameters:  parent   parent of child window
  # Description: Dummy function
  #****************************************************************************
  proc Destroy { name } \
  {
    
  }


  #****************************************************************************
  # Name:        getSaveState
  # Parameters:  none
  # Description: This function returns a list of TCL commands which can be used
  #              to set the Audio_Analyzer_Screen screen to its current state
  #****************************************************************************
  proc getSaveState { } \
  {
    set returnList \
    {
      
    }
    
    return [subst $returnList]
  }


  #****************************************************************************
  # Functions executed when the namespace is sourced
  #****************************************************************************
  # Add this on the menu
  if { [IFR_options  is_installed  Audio_Analyzer] } \
  {
    ::View::RegisterScreen  Audio_Analyzer_Screen
  }


};  # End of the Audio_Analyzer_Screen



#****************************************************************************
# This registers the screens with the list for saving instrument states
#****************************************************************************
namespace eval ::SavedStates \
{
  variable screens   

  lappend screens Audio_Analyzer_Screen
}

