#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C) 2001      DESIGN ENGINEERING        #
# --------------------------------------------------------------------------#
# FILE        : 2080.Intro.tcl                                              #
# PRODUCT     : FITS                                                        #
# AUTHOR      : Preston F. McCormick                                        #
# DESCRIPTION : This namespace manages the Introduction Screen              #
#############################################################################
#                                                                           #
# Required Variables:                                                       #
#    LongName           Name of screen                                      #
#    ShortName          Name of screen for menus                            #
#    RecomendedTiles    List of tiles available for this screen. It is not  #
#                       necessary to put a tile on this list that is on the #
#                       RequiredTiles list                                  #
#    NotAllowedTiles    List of tiles NEVER on this screen                  #
#    RequiredTiles      List of tiles ALWAYS on this screen                 #
#    ActiveTiles        List of tiles currently displayed on this screen    #
#    MenuFlags          0 = Active Screen                                   #
#                       1 = Currently functioning as a submenu parent       #
#                                                                           #
# Required Variables for submenu "Screens":                                 #
#    SubmenuScreens     List of all tiles (or submenus) for droplist        #
#    SubDirPath         Path to subdirectory holding the screens for the    #
#                       submenu of this "screen"                            #
#                                                                           #
#############################################################################
#                                                                           #
# Dummy Functions:                                                          #
#    Setup              Called by the screen manager to create the screen.  #
#    Map                Called when screen is mapped on window              #
#    Unmap              Called when screen is removed from display          #
#    Destroy            Called when screen is destroyed                     #
#                                                                           #
#############################################################################

namespace eval Intro_Screen \
{
  variable LongName
  variable ShortName
  variable RecommendedTiles
  variable NotAllowedTiles
  variable RequiredTiles
  variable ActiveTiles
  variable MenuFlags
  variable SubmenuScreens
  variable SubDirPath

  set LongName  "intro_Screen"
  set ShortName "Intro"

  set RequiredTiles [ list IntroUI ]

  set RecommendedTiles { }
  set NotAllowedTiles  { }
  set ActiveTiles      { }
  set SubmenuScreens   { }
  set SubDirPath       { }
  
  set MenuFlags 0


  #****************************************************************************
  # Name:        Setup
  # Parameters:  parent		parent of child window
  # Description: Dummy function
  # Returns:     Indicator that no tile is generated
  #****************************************************************************
  proc Setup { parent } \
  {
    # Use the Screen functions to manage our tiles
    set topWin  [winfo toplevel $parent]
     
    View::PackKeyBar  $topWin

    Screens::ShowRequiredTiles  $topWin  Intro_Screen

    return  ""
  }


  #****************************************************************************
  # Name:        Map
  # Parameters:  parent		parent of child window
  # Description: Dummy function
  #****************************************************************************
  proc Map { name } \
  { 
  
  }


  #****************************************************************************
  # Name:        Unmap
  # Parameters:  parent		parent of child window
  # Description: Dummy function
  #****************************************************************************
  proc Unmap { name } \
  { 
    
  }


  #****************************************************************************
  # Name:        Destroy
  # Parameters:  parent		parent of child window
  # Description: Dummy function
  #****************************************************************************
  proc Destroy { name } \
  { 
    
  }


};  # End of the Intro_Screen


