#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C)1998,99      DESIGN ENGINEERING      #
# --------------------------------------------------------------------------#
# FILE        : 2290.config.tcl                                             #
# PRODUCT     : FITS                                                        #
# AUTHOR      : Preston McCormick                                           #
# DESCRIPTION : config screen                                               #
#                                                                           #
#############################################################################

package require DHCP

namespace eval config \
{
  namespace export start clear build_net build_sys start build label

  variable ip_address
  variable ip_gateway
  variable ip_netmask
  variable ip_display

  foreach octet {a b c d } \
  {
    set ip_address($octet) 0
    set ip_gateway($octet) 0
    set ip_netmask($octet) 0
  }
  
  set ip_display(host) {}
  set ip_display(display) 0
  set ip_display(screen) 0
  set ip_display(whole) :0

  variable label "Configuration"
  
  #*************************************************
  #       Name: set_10mhz_ref
  # Parameters: change {INITIAL, TOGGLE}
  #Description: sets 10 MHz reference
  #*************************************************
  proc set_10mhz_ref { change } \
  {
    # TOGGLE SELECTION
    if { $change == "TOGGLE" } \
    {
      if { $::sys(10mhzref) == "External" } \
      {
        set ::sys(10mhzref) Internal
      } \
      else \
      {
        set ::sys(10mhzref) External
      }
    }
    HW_control ref $::sys(10mhzref)
   }
    
  #*************************************************
  #       Name: reconfigure_network
  # Parameters: none
  #Description: change our network settings 
  #*************************************************
  proc reconfigure_network { } \
  {
    ::DHCP::Config
  }
  
  
  #*************************************************
  #       Name: write_ifcfg
  # Parameters: none
  #Description: write our ethernet config out 
  #*************************************************
  proc write_ifcfg { } \
  {
    ::DHCP::Config
    ::DHCP::Write_config
  }
  
  
  #*************************************************
  #       Name: validate_ip_name
  # Parameters: widget making request
  #Description: validate the name of the box
  #*************************************************
  proc validate_ip_name { widget } \
  {
    upvar #0 {::DHCP::State(Host Name)} name
     
    #allow pretty much anything, as long as not blank
    if {"$name" == ""} \
    {
      set name IFR2975-[IFR_options ser]
      # turn the widget yellow.
      $widget configure -fg yellow
      return 0
    }
    
    $widget configure -fg $::color(screenfg)
    
    return 1
  }

  
  #*************************************************
  #       Name: set_ip_name
  # Parameters: widget making request
  #Description: set the unit name if valid 
  #*************************************************
  proc set_ip_name { widget } \
  { 
    if [validate_ip_name $widget] \
    {
      write_ifcfg
      return 1
    }
    
    return 0
  }
  
  
  #*************************************************
  #       Name: validate_octet
  # Parameters: widget making request
  #Description: validate that an octet of an IP address
  #             is valid (i.e. between 0-255 inclusive)
  #*************************************************
  proc validate_octet { widget } \
  { 
    # get the variable it's modifying
    set vname [$widget cget -textvariable ]
    upvar #0 $vname octet
    
    if { $octet < 0 || $octet > 255 } \
    {
      $widget configure -fg yellow
      return 0
    }
   
    $widget configure -fg $::color(screenfg)
    
    return 1;
  }
  
  
  #*************************************************
  #       Name: validate_ip_addr
  # Parameters: widget frame making request
  #Description: validate the IP address
  #*************************************************
  proc validate_ip_addr { widget} \
  {
    upvar #0 ::DHCP::State(CLIENT_IP) addr
     
    # assumption: the widget frame was created
    # by the build_ip_addr proc later in this file
    # the individual widgets are getting the message,
    # so we will need to get the name of the frame.
    if {[winfo class $widget] != "Entry"} \
    {
      # must be the parent frame.
      set frame $widget
    } \
    else \
    {
      set frame [winfo parent $widget]
    }
   
    set invalid 0
    set temp {}
   
    foreach octet { a b c d } \
    {
      if {![validate_octet $frame.$octet ] } \
      {
        set invalid 1
      }
    }
   
    if { $invalid != 0 } \
    { 
      return 0 
    }
   
    # OK, this address is valid. Set the system IP address
    # variable from the parts.
    set addr [normalize_ip_address config::ip_address]
   
    return 1
  }
  
  
  #*************************************************
  #       Name: set_ip_addr
  # Parameters: widget making request
  #Description: set the unit address if valid 
  #*************************************************
  proc set_ip_addr { widget } \
  { 
    if [validate_ip_addr $widget] \
    {
      write_ifcfg
      return 1
    }
    
    return 0
  }
  #*************************************************
  #       Name: ip_addr_done
  # Parameters: widget making request
  #Description: finish an edit on the IP address 
  #*************************************************
  proc ip_addr_done {widget } \
  {
     if [set_ip_addr $widget] \
     {
        set frame [winfo parent $widget]
        set frame [winfo parent $frame]
        config::set_ip_gateway $frame.gateway
     }
  }
  #*************************************************
  #       Name: validate_ip_netmask
  # Parameters: widget making request
  #Description: validate the IP netmask
  #*************************************************
  proc validate_ip_netmask { widget } \
  {
    upvar #0 ::DHCP::State(Subnet) mask
     
    # assumption: the widget frame was created
    # by the build_ip_addr proc later in this file
    # the individual widgets are getting the message,
    # so we will need to get the name of the frame.
    if {[winfo class $widget] != "Entry"} \
    {
      # must be the parent frame.
      set frame $widget
    } \
    else \
    {
      set frame [winfo parent $widget]
    }
     
    set invalid 0
    foreach octet { a b c d } \
    {
      if {![validate_octet $frame.$octet ] } \
      {
        set invalid 1
      }
    }
    
    if {$invalid != 0 } \
    { 
      return 0
    }

    # now, check for an invalid netmask (e.g. all 0's, or
    # non-zero bits after a zero
    set bin [ip_address_to_bin ::config::ip_netmask]
    
    if {$bin == 0 } \
    {
      # netmasks cannot be 0!
      event generate $frame <<invalid>>
      return 0
    }
    
    # find the first 0 bit
    set bit 31
    while {[expr $bin & 0x80000000] != 0 } \
    {
      set bin [expr ($bin+$bin)&0xFFFFFFFF ]
      incr bit -1
    }
   
    # OK, damn well better be zero now
    if {$bin != 0 } \
    {
      # bit is the location of the first 0 bit.
      # we will highlight all fields
      if {$bit > 23 } { event generate $frame.a <<invalid>> }
      if {$bit > 15 } { event generate $frame.b <<invalid>> }
      if {$bit > 7  } { event generate $frame.c <<invalid>> }
      $frame.d configure -fg yellow
      return 0
    }

    event generate $frame <<valid>>
   
    set mask [normalize_ip_address ::config::ip_netmask]
   
    return 1
  }
  
  
  #*************************************************
  #       Name: set_ip_netmask
  # Parameters: widget making request
  #Description: set the unit net mask if valid 
  #*************************************************
  proc set_ip_netmask { widget } \
  { 
    if [validate_ip_netmask $widget] \
    {
      write_ifcfg
      return 1
    }
    
    return 0
  }
  #*************************************************
  #       Name: ip_netmask_done
  # Parameters: widget making request
  #Description: finish an edit on the IP netmask
  #*************************************************
  proc ip_netmask_done {widget } \
  {
     if [set_ip_netmask $widget] \
     {
        set frame [winfo parent $widget]
        set frame [winfo parent $frame]
        config::set_ip_gateway $frame.gateway
     }
  }
  
  #*************************************************
  #       Name: validate_ip_gateway
  # Parameters: widget making request
  #Description: validate the IP default gateway 
  #*************************************************
  proc validate_ip_gateway { widget} \
  {
    # assumption: the widget frame was created
    # by the build_ip_addr proc later in this file
    # the individual widgets are getting the message,
    # so we will need to get the name of the frame.
    if {[winfo class $widget] != "Entry"} \
    {
      # must be the parent frame.
      set frame $widget
    } \
    else \
    {
      set frame [winfo parent $widget]
    }
  
    set invalid 0
    
    foreach octet { a b c d } \
    {
      if {![validate_octet $frame.$octet ] } \
      {
        set invalid 1
      }
    }
     
    # bogus format address
    if {$invalid == 1 } \
    {
      return 0
    }
   
    # now, verify that the gateway is reachable
    set ip_bin   [ip_address_to_bin ::config::ip_address]
    set mask_bin [ip_address_to_bin ::config::ip_netmask]
    set gw_bin   [ip_address_to_bin ::config::ip_gateway]
   
    set net_bin [expr $ip_bin & $mask_bin]
    set gw_net  [expr $gw_bin & $mask_bin]
    set err_bin [expr $gw_net ^ $net_bin]
  
    if {$err_bin} \
    {
      # see what part is wrong...
      if [expr $err_bin & 0xFF000000 ] \
      {
        event generate $frame.a <<invalid>>
      }
      
      if [expr $err_bin & 0x00FF0000 ] \
      {
        event generate $frame.b <<invalid>>
      }
      
      if [expr $err_bin & 0x0000FF00 ] \
      {
        event generate $frame.c <<invalid>>
      }
      
      if [expr $err_bin & 0x000000FF ] \
      {
        event generate $frame.d <<invalid>>
      }
    
      return -1
    }
   
    event generate $frame <<valid>>
    set DHCP::State(Router) [normalize_ip_address ::config::ip_gateway]
    DHCP::Config
    
    return 1
  }
  
  
  #*************************************************
  #       Name: set_ip_gateway
  # Parameters: widget making request
  #Description: set the unit net mask if valid 
  #*************************************************
  proc set_ip_gateway { widget } \
  { 
    #puts "set_ip_gateway:  $widget"
    
    set i [validate_ip_gateway $widget]
    
    if {$i == 1}  \
    {
      write_ifcfg
      return 1
    }
    
    return 0
  }
  #*************************************************
  #       Name: ip_gateway_done
  # Parameters: widget making request
  #Description: finish an edit on the IP netmask
  #*************************************************
  proc ip_gateway_done {widget } \
  {
     set_ip_gateway $widget
  } 

  #*************************************************
  #       Name: validate_ip_display
  # Parameters: widget making request
  #Description: validate the remote display address
  #*************************************************
  proc validate_ip_display { widget} \
  {
    # assumption: the widget frame was created
    # by the build_ip_display proc later in this file
    # the individual widgets are getting the message,
    # so we will need to get the name of the frame.
    if {[winfo class $widget] != "Entry"} \
    {
      # must be the parent frame.
      set frame $widget
    } \
    else \
    {
      set frame [winfo parent $widget]
    }
     
    # first, try to get the host's IP address.
    if {"$::config::ip_display(name)" != "" } \
    { 
      set host [getHostByName $::config::ip_display(name)]
      if {"$host" == "" } \
      {
        # invalid host or unknown.
        event generate $frame.name <<invalid>>
        return 0
      }
    }
   
    # OK, compose a screen name from the parts
    if {"$::config::ip_display(screen)" == "" } \
    { 
      set ::config::ip_display(screen) 0
    }
   
    set ::config::ip_display(whole) "$::config::ip_display(name):$::config::ip_display(display).$::config::ip_display(screen)"
    
    event generate $frame <<valid>>
    return 1
  }


  #*************************************************
  #       Name: set_ip_display
  # Parameters: widget making request
  #Description: set the remote display address if valid 
  #*************************************************
  proc set_ip_display { widget } \
  { 
    set i [validate_ip_display $widget]
  
    if {$i == 1} \
    {
      set ::sys(hdisplay) $::config::ip_display(whole)
      write_display
      return
    }
    
  }
  trace variable ::DHCP::State(state) w "config::net_update"
  proc net_update { args } \
  {
     after cancel config::net_init
     after idle config::net_init
  }
  proc Net_Map { frame } \
  {
     trace variable ::DHCP::State(Mode) w "config::Net_monitor_DHCP $frame"
     after idle "config::Net_monitor_DHCP $frame"
     net_init $frame
  }
  proc Net_unmap { frame } \
  {
     Net_cleanup $frame
  }
  proc Net_destroy { frame } \
  {
     Net_cleanup $frame
  }
  proc Net_cleanup { frame } \
  {
     trace vdelete ::DHCP::State(Mode) w "config::Net_monitor_DHCP $frame"
  }
  proc Net_monitor_DHCP { frame args } \
  {
     set disable_list {name addr.a addr.b addr.c addr.d netmask.a netmask.b netmask.c netmask.d
                       gateway.a gateway.b gateway.c gateway.d
                      }
     if {$::DHCP::State(Mode) == "DHCP" } \
     {
        # DHCP mode - disable all edits
        foreach item $disable_list \
        {
          $frame.$item config -state disabled -fg darkgrey
        }
     } \
     else \
     {
        foreach item $disable_list \
        {
          $frame.$item config -state normal -fg black
        }
     }
  }
  

  #*************************************************
  #       Name: net_init
  # Parameters: do all needed preliminary setup
  #Description: do init for net config screen
  #*************************************************
  proc net_init { { frame {} } } \
  {
    
    # first, copy the system IP address, netmask, gateway
    # to our "broken up" vars
    validate_ip_address $::DHCP::State(CLIENT_IP) ::config::ip_address
    validate_ip_address $::DHCP::State(Subnet)    ::config::ip_netmask
    validate_ip_address $::DHCP::State(Router)    ::config::ip_gateway
     
    regexp (.*)\:(\[0-9\]+)(\.(\[0-9\]*))* $::sys(hdisplay)                \
                                             ::config::ip_display(whole)   \
                                             ::config::ip_display(name)    \
                                             ::config::ip_display(display) \
                                             junk                          \
                                             ::config::ip_display(screen)

    # now, validate the actual contents
    if {$frame != "" } \
    {
      validate_ip_addr    $frame.addr
      validate_ip_netmask $frame.netmask
      validate_ip_gateway $frame.gateway
      validate_ip_name    $frame.name
      validate_ip_display $frame.display
    }
  }
  
  
  #*************************************************
  #       Name: redirect_screen
  # Parameters: 
  #Description: move the screen, if the display host
  #             setting is different
  #*************************************************
  proc redirect_screen { widget } \
  {
    set new_display "$::config::ip_display(name):$::config::ip_display(display).$::config::ip_display(screen)"
    set current_display [winfo screen $widget]
    
    switch -- $::sys(hdisplay) \
    {
       ":0" -
       ":0.0" -
       "127.0.0.1:0" -
       "127.0.0.1:0.0" -
       "" -
       $current_display \
       {
          ::help::show $widget
          return
       }
    }
     
    if [catch {View::CreateView "" $new_display} err ] \
    {
      ::help::show $widget
      return
    }
    message_box [name_child $widget redirected] INFORM

  }
  
  #*************************************************
  #       Name: toggle_gen_power
  # Parameters: 
  #Description: toggle the state of the RF gen power block
  #*************************************************
  proc toggle_gen_power { } \
  {
    if {$::Generator::State(dcpower) == "ON"} \
    {
      Generator dcpower OFF

    } \
    else \
    {
      Generator dcpower ON
    }
  }
  
  #*************************************************
  #       Name: toggle_pwr_msg
  # Parameters: 
  #Description: toggle the state of the power off
  #             confirmation message
  #*************************************************
  proc toggle_pwr_msg { } \
  {
    if {$::sys(powerdown_sure) == "YES"} \
    {
      set ::sys(powerdown_sure) NO
    } \
    else \
    {
      set ::sys(powerdown_sure) YES
    }
  }
  

};  # End of config namespace




#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C) 2001      DESIGN ENGINEERING        #
# --------------------------------------------------------------------------#
# FILE        : 2093.config.tcl                                             #
# PRODUCT     : FITS                                                        #
# AUTHOR      : Bruce Butterfield                                           #
# DESCRIPTION : This namespace is a dummy "screen" that sources and holds   #
#             : the submenu namespace list for the "Systems" droplist       #
#############################################################################
#                                                                           #
# Required Variables:                                                       #
#    LongName		        Name of screen                              #
#    ShortName          Name of screen for menus                            #
#    RecomendedTiles    List of tiles available for this screen. It is not  #
#                       necessary to put a tile on this list that is on the #
#                       RequiredTiles list                                  #
#    NotAllowedTiles    List of tiles NEVER on this screen                  #
#    RequiredTiles      List of tiles ALWAYS on this screen                 #
#    ActiveTiles        List of tiles currently displayed on this screen    #
#    MenuFlags          0 = Active Screen                                   #
#                       1 = Currently functioning as a submenu parent       #
#                                                                           #
# Required Variables for submenu "Screens":                                 #
#    SubmenuScreens     List of all tiles (or submenus) for droplist        #
#    SubDirPath         Path to subdirectory holding the screens for the    #
#                       submenu of this "screen"                            #
#                                                                           #
#############################################################################
#                                                                           #
# Dummy Functions:                                                          #
#    Setup              Called by the screen manager to create the screen.  #
#    Map                Called when screen is mapped on window              #
#    Unmap              Called when screen is removed from display          #
#    Destroy            Called when screen is destroyed                     #
#                                                                           #
#############################################################################

namespace eval Config_Screen \
{
  variable LongName
  variable ShortName
  variable RecommendedTiles
  variable NotAllowedTiles
  variable RequiredTiles
  variable ActiveTiles
  variable MenuFlags
  variable SubmenuScreens
  variable SubDirPath
  variable restoringDefaults

  set LongName Config_Screen
  set ShortName "Configuration"
  set RequiredTiles [ list Config_Screen ]
  set NotAllowedTiles { }
  set RecommendedTiles { }
  set ActiveTiles { }
  set MenuFlags 0
  set SubmenuScreens { }
  set SubDirPath { }
  set restoringDefaults 0


  variable tile_size      "full"
  variable tile_class     "CONFIG"
  
  variable YIG_state "YIG CAL"
  
  OnChange hook sys "::Config_Screen::schedule_sys_save"
  
  #****************************************************************************
  # Name:        schedule_sys_save
  # Parameters:  none
  # Description: schedule the system setup to be saved in 1 second
  # Returns:     none
  #****************************************************************************
  proc schedule_sys_save {} \
  {
     after cancel ::Config_Screen::sys_save
     after 1000 ::Config_Screen::sys_save
  }
  #****************************************************************************
  # Name:        sys_save
  # Parameters:  none
  # Description: Save the system setup values that don't get saved elsewhere
  # Returns:     none
  #****************************************************************************
  proc sys_save {} \
  {
     catch \
     {
        set fd [open /sysdata/system.setup w]
        foreach index { vol_win_on_knob  10mhzref menu_reverse* powerdown* } \
        {
           puts $fd [array get ::sys $index]
        }
        close $fd
     }
  }
  #****************************************************************************
  # Name:        sys_load
  # Parameters:  none
  # Description: load the system setup data
  # Returns:     none
  #****************************************************************************
  proc sys_load {} \
  {
     catch \
     {
        if [file exists /sysdata/system.setup ] \
        {
           set fd [open /sysdata/system.setup r]
           set values [read $fd ]
           close $fd
           array set ::sys $values
        }
     }
  }
  after idle ::Config_Screen::sys_load

  #****************************************************************************
  # Name:        Setup
  # Parameters:  parent		parent of child window
  # Description: Create the screen
  # Returns:     Tile path to be packed into the parent frame
  #****************************************************************************
  proc Setup { parent } \
  {
    #puts "Config_Screen::Setup"
    #puts "parent:  $parent"
         
    set top_level [winfo toplevel $parent]
   
    set tile_path  [name_child $top_level config]

    set tile_class  $::Config_Screen::tile_class

    if { ![winfo exists $tile_path] } \
    {
      set tile_path  [::Screens::create_tile  $tile_path  config  $tile_class]
      $tile_path.net_cfg.mac configure -text [IFR_options MAC]
    }
       
    return $tile_path
  }

  #****************************************************************************
  # Name:        Save_Debug_info
  # Parameters:  none
  # Description: Save debug info (error logs and key strokes) to floppy
  #****************************************************************************
  proc Save_Debug_info { tile_path } \
  {

     if [catch { file copy /usr/logfiles/errorLog /fd0/errlog } err] \
     {
        if [string match "*S_dosFsLib_VOLUME_NOT_AVAILABLE*" $err] \
        {
           set widget ${tile_path}.prompt_no_floppy
           message_box $widget INFORM
           return
        }
     }

     catch \
     {
        file copy /usr/logfiles/errorLog.old /fd0/errlog.old
     }
     foreach keylog [glob -nocomplain /usr/logfiles/key_log*] \
     {
        set base [file tail $keylog]
        catch \
        {
          file copy $keylog /fd0/$base
        }
     }
     catch {
        set save [open /fd0/state w]
        catch {puts $save [SavedStates::save_all]}
        close $save
     }
  }
  
  #****************************************************************************
  # Name:        Map
  # Parameters:  name		parent of child window
  # Description: 
  #****************************************************************************
  proc Map { tile_path } \
  { 
    after cancel "Config_Screen::Cleanup_core $tile_path"
    variable YIG_state 
    set YIG_state "YIG CAL"

  
    ::View::ClearAllSoftkeys  $tile_path
    ::View::PackOptions $tile_path ""
    
    # ASSIGN 'FACTORY DEFAULT' SOFTKEY
    ::View::AssignSoftkey $tile_path  2  "FACTORY DEFAULT"  "::Config_Screen::Process_FactoryDefault $tile_path"

    ::View::AssignSoftkey $tile_path  3  ::Config_Screen::YIG_state "::Config_Screen::StartYigCal"

#    ::View::AssignSoftkey $tile_path  4  "IF TEMP" "::Config_Screen::if_temp_display $tile_path"

    View::AssignSoftkey  $tile_path  6  "Save debug info"  "::Config_Screen::Save_Debug_info $tile_path"
    View::AssignSoftkey  $tile_path  7  "$::KeyLog::State(file_state_text)"  \
                                          "KeyLog::toggle_logfile_state $tile_path"
    
    ::View::AssignReturn $tile_path  ""

    screen_done FORCE_COMPLETE
    
    return
  }
  proc StartYigCal { } \
  {
     variable YIG_state 
     set YIG_state "RUNNING"
     HW_control::OnYigDone {set ::Config_Screen::YIG_state "YIG CAL"}
     HW_control::StartYigCal
  }
  
  #****************************************************************************
  # Name:        Cleanup_core
  # Parameters:  parent object
  # Description: remove our tiles
  #****************************************************************************
  proc Cleanup_core { tile_path } \
  {
     catch \
     {
        if [winfo exists $tile_path] \
        {
           destroy $tile_path
        }
     }
  }
  
  #****************************************************************************
  # Name:        Cleanup
  # Parameters:  parent object
  # Description: Dummy function
  #****************************************************************************
  proc Cleanup { tile_path } \
  {
     after 1000 "Config_Screen::Cleanup_core $tile_path"
  }



  #****************************************************************************
  # Name:        Unmap
  # Parameters:  name		parent of child window
  # Description: Dummy function
  #****************************************************************************
  proc Unmap {tile_path } \
  { 
     Cleanup $tile_path
     return
  }

 
  #****************************************************************************
  # Name:        Destroy
  # Parameters:  name		parent of child window
  # Description: Dummy function
  #****************************************************************************
  proc Destroy { tile_path } \
  { 
     Cleanup $tile_path
  }
  #****************************************************************************
  # Name:        getSaveState
  # Parameters:  none
  # Description: This function returns a list of TCL commands which can be used
  #              to set the Config screen to its current state
  #****************************************************************************
  proc getSaveState { } \
  {
     if ![info exists ::sys(menu_reverse)] \
     {
        set ::sys(menu_reverse) 0
     }
     if ![info exists ::sys(vol_win_on_knob)] \
     {
        set ::sys(vol_win_on_knob) ON
     }
     set returnList \
     {
       set ::sys(useproxy)        {$::sys(useproxy)}
       set ::sys(proxyuser)       {$::sys(proxyuser)}
       set ::sys(proxyip)         {$::sys(proxyip)}
       set ::sys(proxypw)         {$::sys(proxypw)}
       set ::sys(proxyneedpw)     {$::sys(proxyneedpw)}
       set ::sys(powerdown_sure)  {$::sys(powerdown_sure)}
       set ::sys(menu_reverse)    {$::sys(menu_reverse)}
       set ::sys(vol_win_on_knob) {$::sys(vol_win_on_knob)}
       catch {set_menu_spinner_mode}
     }
     return [subst $returnList]
  }

  #****************************************************************************
  # Name:        Process_FactoryDefault
  # Parameters:  name		parent of child window
  # Description: Dummy function
  #****************************************************************************
  proc Process_FactoryDefault {tile_path } \
  { 
     variable restoringDefaults
 
     if { $restoringDefaults == 1 } \
     {
        return
     } 
     set restoringDefaults 1
     restore_factory_defaults
     ::Screens::Show_remote $tile_path
     set restoringDefaults 0
  }

  #*************************************************
  # Name: if_temp_display
  # Parameters: none
  # Description: display the current IF temperature
  #*************************************************
  proc if_temp_display { tile_path } \
  {
     set widget ${tile_path}.prompt_if_temp
     message_box $widget INFORM
  }

  #****************************************************************************
  # Functions executed when the namespace is sourced
  #****************************************************************************
  ::View::RegisterScreen Config_Screen	   ;# Add this on the menu


};  # End of the Config_Screen name space
#****************************************************************************
# This registers the screens with the list for saving instrument states
#****************************************************************************
namespace eval ::SavedStates {

   variable system   
   lappend system Config_Screen
}
