#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C) 2001      DESIGN ENGINEERING        #
# --------------------------------------------------------------------------#
# FILE        : 2095.timedate.tcl                                           #
# PRODUCT     : FITS                                                        #
# AUTHOR      : Bruce Butterfield                                           #
# DESCRIPTION : This namespace is a dummy "screen" that sources and holds   #
#             : the submenu namespace list for the "Systems" droplist       #
#############################################################################
#                                                                           #
# Required Variables:                                                       #
#    LongName		Name of screen                                      #
#    ShortName          Name of screen for menus                            #
#    RecomendedTiles    List of tiles available for this screen. It is not  #
#                       necessary to put a tile on this list that is on the #
#                       RequiredTiles list                                  #
#    NotAllowedTiles    List of tiles NEVER on this screen                  #
#    RequiredTiles      List of tiles ALWAYS on this screen                 #
#    ActiveTiles        List of tiles currently displayed on this screen    #
#    MenuFlags          0 = Active Screen                                   #
#                       1 = Currently functioning as a submenu parent       #
#                                                                           #
# Required Variables for submenu "Screens":                                 #
#    SubmenuScreens     List of all tiles (or submenus) for droplist        #
#    SubDirPath         Path to subdirectory holding the screens for the    #
#                       submenu of this "screen"                            #
#                                                                           #
#############################################################################
#                                                                           #
# Dummy Functions:                                                          #
#    Setup              Called by the screen manager to create the screen.  #
#    Map                Called when screen is mapped on window              #
#    Unmap              Called when screen is removed from display          #
#    Destroy            Called when screen is destroyed                     #
#                                                                           #
#############################################################################
package require HW_control

namespace eval TimeDate_Screen \
{
  variable LongName
  variable ShortName
  variable RecommendedTiles
  variable NotAllowedTiles
  variable RequiredTiles
  variable ActiveTiles
  variable MenuFlags
  variable SubmenuScreens
  variable SubDirPath

  set LongName TimeDate_Screen
  set ShortName "Time / Date"
  set RequiredTiles { }
  set NotAllowedTiles { }
  set RecommendedTiles { }
  set ActiveTiles { }
  set MenuFlags 0
  set SubmenuScreens { }
  set SubDirPath { }

  variable tile_path      ""
  variable tile_size      "full"
  variable tile_class     "TIME_DATE"

  variable timedate
  set timedate(current) [clock seconds]
  set timedate(noupdate) 0
  set timedate(time_format) "12 hour"
  
  variable days_per_month
  array set days_per_month \
  {
     JAN 31
     FEB *
     MAR 31
     APR 30
     MAY 31
     JUN 30
     JUL 31
     AUG 31
     SEP 30
     OCT 31
     NOV 30
     DEC 31
  }
  #*************************************************
  #       Name: get_days_per_month
  # Parameters: month, year (defaults to NOW)
  #Description: return the # of days in a month
  #*************************************************
  proc get_days_per_month { {month ""} {year ""} } \
  {
     variable days_per_month
     
     if {"$month" == "" } \
     {
        set month [clock format [clock seconds] -format %b]
     }
     
     set month [string toupper $month]
     
     # handle the easy stuff first
     if { "$days_per_month($month)" != "*" } \
     {
       return  $days_per_month($month)
     }
     
     # might be a leap year...
     if {"$year" == "" } \
     {
        set year [clock format [clock seconds] -format %Y]
     }
     if {[expr $year % 4] != 0} \
     {
        return 28
     }
     if {[expr $year % 400] == 0} \
     {
        return 29
     }
     if {[expr $year % 100] == 0} \
     {
        return 28
     }
     return 29
  }
  #*************************************************
  #       Name: clip_months
  # Parameters: 
  #Description: set the limits on the number of days
  #             in the month
  #*************************************************
  proc clip_months { widget args } \
  {
     variable timedate

     upvar #0 ::Entry::entry_data($widget,max) max
     set max [get_days_per_month $timedate(month) $timedate(year)]
  }
  
  #*************************************************
  #       Name: set_time
  # Parameters: 
  #Description: resets CPU time based on current values
  #*************************************************
  proc set_time {} \
  {
    variable timedate
    # make sure the day of month works for the current time
    set max_day [get_days_per_month $timedate(month) $timedate(year)]
    if {$timedate(day) > $max_day} \
    {
       set timedate(day) $max_day
    }
    catch \
    {
     if {"$timedate(time_format)" == "12 hour"} \
     {
       HW_control::Set_clock "$timedate(hours):$timedate(minutes):$timedate(seconds) $timedate(ampm) $timedate(day) $timedate(month) $timedate(year)"
       
     } \
     else \
     {
      HW_control::Set_clock "$timedate(hours24):$timedate(minutes):$timedate(seconds) $timedate(day) $timedate(month) $timedate(year)"
     }
    }
    # check for expired options.
    IFR_options clock
    after cancel ::TimeDate_Screen::Clock
    Clock
  }
  
  #*************************************************
  #       Name: clock_enable
  # Parameters: 
  #Description: sets AM or PM time value
  #*************************************************
  proc clock_enable {} \
  {
    variable timedate
    global ::View::KeyBarMode(.clock)
    
    if {$::View::KeyBarMode(.clock) != 1} {
      set ::View::KeyBarMode(.clock) 1
    } else {
     set ::View::KeyBarMode(.clock) 0
    }
  }
  #*************************************************
  #       Name: clock_monitor
  # Parameters: 
  #Description: sets the string based on the state of the clock
  #*************************************************
  proc clock_monitor { args } \
  {
    variable timedate
    global ::View::KeyBarMode(.clock)
    
    if {$::View::KeyBarMode(.clock) != 0} {
      set timedate(enable) ON
    } else {
      set timedate(enable) OFF
    }
  }
  
  #*************************************************
  #       Name: set_time_ampm
  # Parameters: 
  #Description: sets AM or PM time value
  #*************************************************
  proc set_time_ampm {} \
  {
    variable timedate
       
    if {$timedate(ampm) == "AM"} {
      set timedate(ampm) PM
    } else {
      set timedate(ampm) AM
    }
    set_time
  }
  
  #*************************************************
  #       Name: set_time_format
  # Parameters: none
  #Description: sets 12 or 24 hour time
  #*************************************************
  proc set_time_format {} \
  {  
    variable timedate
    if { $timedate(time_format) == "12 hour" } \
    {
      set timedate(time_format) "24 hour"
    } \
    else \
    {
      set timedate(time_format) "12 hour"
    }
  }
  #*************************************************
  #       Name: monitor_time_format
  # Parameters: widget making request
  #Description: shows the 24 or 12 hour fields as apropriate
  #*************************************************
  proc monitor_time_format { widget args } \
  {  
    variable timedate
    while {"[winfo class $widget]" != "TIME_DATE" } \
    {
       set widget [winfo parent $widget]
    }
    
    if { $timedate(time_format) == "12 hour" } \
    {
      grid forget $widget.hours24
      Screens::position_widget $widget.hours
      Screens::position_widget $widget.ampm
    } \
    else \
    {
      grid forget  $widget.hours
      grid forget  $widget.ampm
      Screens::position_widget $widget.hours24
    }
  }
  #*************************************************
  #       Name: Stop_update
  # Parameters:
  #Description: stop the update of the time variables
  #*************************************************
  proc Stop_update { } \
  {
     variable timedate
     set timedate(noupdate) 1
  }
  #*************************************************
  #       Name: Start_update
  # Parameters:
  #Description: start the update of the time variables
  #*************************************************
  proc Start_update { } \
  {
     variable timedate
     set timedate(noupdate) 0
  }  
  #*************************************************
  #       Name: Clock
  # Parameters:
  #Description: Real Time Clock divided into 3 parts
  #*************************************************
  proc Clock { } \
  {
    variable timedate
    after cancel ::TimeDate_Screen::Clock
    after 1000 ::TimeDate_Screen::Clock
    set timedate(current) [clock seconds]
    # update corner clock value
    
    if {"$timedate(time_format)" == "12 hour"} \
    {
       set ::View::ClockTime [clock format $timedate(current) -format "%I:%M:%S"]
    } \
    else \
    {
       set ::View::ClockTime [clock format $timedate(current) -format "%H:%M:%S"]
    }
    
    if {$timedate(noupdate) != 1 } \
    {
      array set timedate [clock format $timedate(current) -format \
        {
           seconds %S
           minutes %M
           hours24 %H
           hours   %I
           ampm    %p
           day     %d
           month   %b
           year    %Y
           weekday %A
        }
      ]
    }
  }
   
  
  #****************************************************************************
  # Name:        Setup
  # Parameters:  parent		parent of child window
  # Description: Dummy function
  # Returns:     Indicator that no tile is generated
  #****************************************************************************
  proc Setup { parent } \
  {
        
    set top_level [winfo toplevel $parent]
    set tile_path [name_child $top_level timedate]
    
    set tile_class  $::TimeDate_Screen::tile_class

    if { ![winfo exists $tile_path] } \
    {
      set tile_path  [::Screens::create_tile  $tile_path  timedate  $tile_class]
    }
    
    return $tile_path
  }

  
  #****************************************************************************
  # Name:        Map
  # Parameters:  parent		parent of child window
  # Description: Dummy function
  #****************************************************************************
  proc Map { name } \
  {
    after cancel "::TimeDate_Screen::Cleanup_core $name"
    Check_for_lease $name
    variable timedate
    trace variable timedate(time_format) w "::TimeDate_Screen::monitor_time_format $name"
    trace variable timedate(month) w "::TimeDate_Screen::clip_months $name.day"
    trace variable timedate(year)  w "::TimeDate_Screen::clip_months $name.day"
    monitor_time_format $name
    return
  }

  #****************************************************************************
  # Name:        Cleanup_core
  # Parameters:  parent		parent of child window
  # Description: 
  #****************************************************************************
  proc Cleanup_core { name } \
  { 
    catch {destroy $name}
  }

  #****************************************************************************
  # Name:        Cleanup
  # Parameters:  parent		parent of child window
  # Description: 
  #****************************************************************************
  proc Cleanup { name } \
  { 
    trace vdelete timedate(time_format) w "::TimeDate_Screen::monitor_time_format $name"
    trace vdelete timedate(month) w "::TimeDate_Screen::clip_months $name.day"
    trace vdelete timedate(year)  w "::TimeDate_Screen::clip_months $name.day"
    
    after 1000 "::TimeDate_Screen::Cleanup_core $name"
    return
  }

  #****************************************************************************
  # Name:        Unmap
  # Parameters:  parent		parent of child window
  # Description: Dummy function
  #****************************************************************************
  proc Unmap { name } \
  { 
    Cleanup $name
    return
  }

 
  #****************************************************************************
  # Name:        Destroy
  # Parameters:  parent		parent of child window
  # Description: Dummy function
  #****************************************************************************
  proc Destroy { name } \
  { 
    Cleanup $name
    return
  }
  
  #****************************************************************************
  # Name:        Check_for_lease
  # Parameters:  parent		parent of child window
  # Description: Check if we have any leased options - if so, show the
  #              "leased option" screen
  #****************************************************************************
  proc Check_for_lease { parent } \
  {
     set options [IFR_options list]
     foreach option $options \
     {
        if {[IFR_options state $option] == "LEASED"} \
        {
           set wind [name_child $parent warn]
           set options_url file://localhost/sysdata/html/help/Options
           ::HtmlFrame::make_toplevel $wind $options_url/clock.html
           return
        }
     }
  }

  #****************************************************************************
  # Name:        getSaveState
  # Parameters:  none
  # Description: This function returns a list of TCL commands which can be used
  #              to set the Config screen to its current state
  #****************************************************************************
  proc getSaveState { } \
  {
     set returnList \
     {
       set TimeDate_Screen::timedate(time_format) "$TimeDate_Screen::timedate(time_format)"
     }
     return [subst $returnList]
  }
  #****************************************************************************
  # Functions executed when the namespace is sourced
  #****************************************************************************
  ::View::RegisterScreen TimeDate_Screen;  # Add this on the menu

  # start the clock running
  Clock
  trace variable ::View::KeyBarMode(.clock) w "::TimeDate_Screen::clock_monitor"
  clock_monitor
  
};  # End of the TimeDate_Screen name space
#****************************************************************************
# This registers the screens with the list for saving instrument states
#****************************************************************************
namespace eval ::SavedStates {

   variable system   
   lappend system TimeDate_Screen
}
