#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C) 2001      DESIGN ENGINEERING        #
# --------------------------------------------------------------------------#
# FILE        : 2099.version.tcl                                            #
# PRODUCT     : FITS                                                        #
# AUTHOR      : Bruce Butterfield                                           #
# DESCRIPTION : This namespace is used to display the program version info  #
#############################################################################
#                                                                           #
# Required Variables:                                                       #
#    LongName		Name of screen                                      #
#    ShortName          Name of screen for menus                            #
#    RecomendedTiles    List of tiles available for this screen. It is not  #
#                       necessary to put a tile on this list that is on the #
#                       RequiredTiles list                                  #
#    NotAllowedTiles    List of tiles NEVER on this screen                  #
#    RequiredTiles      List of tiles ALWAYS on this screen                 #
#    ActiveTiles        List of tiles currently displayed on this screen    #
#    MenuFlags          0 = Active Screen                                   #
#                       1 = Currently functioning as a submenu parent       #
#                                                                           #
# Required Variables for submenu "Screens":                                 #
#    SubmenuScreens     List of all tiles (or submenus) for droplist        #
#    SubDirPath         Path to subdirectory holding the screens for the    #
#                       submenu of this "screen"                            #
#                                                                           #
#############################################################################
#                                                                           #
# Dummy Functions:                                                          #
#    Setup              Called by the screen manager to create the screen.  #
#    Map                Called when screen is mapped on window              #
#    Unmap              Called when screen is removed from display          #
#    Destroy            Called when screen is destroyed                     #
#                                                                           #
#############################################################################

namespace eval Version_Screen \
{
  variable LongName
  variable ShortName
  variable RecommendedTiles
  variable NotAllowedTiles
  variable RequiredTiles
  variable ActiveTiles
  variable MenuFlags
  variable SubmenuScreens
  variable SubDirPath

  set LongName Version_Screen
  set ShortName "Version"
  set RequiredTiles { }
  set NotAllowedTiles { }
  set RecommendedTiles { }
  set ActiveTiles { }
  set MenuFlags 0
  set SubmenuScreens { }
  set SubDirPath { }
  
  array set Serial \
  {
     ser1 ""
     ser2 ""
     msg ""
     match 0
  }


  variable tile_size      "full"
  variable tile_class     "VERSION"
  
  variable version
  variable names

  # this defines the items in the components window.
  # the indices must match the versions array assignments
  array set names {}
  set version(main) [firmware_version]
  set version(date) [firmware_date]

  # This is a depreciated function. Now each engine obtains its own version and updates the
  # version variable.
  proc query_engine_versions {} \
  {
    package require ALL
    variable version
  }
  
  #****************************************************************************
  # Name:        Setup
  # Parameters:  parent		parent of child window
  # Description: Create the screen
  # Returns:     Tile path to be packed into the parent frame
  #****************************************************************************
  proc Setup { parent } \
  {
    variable version
    variable names
    puts "Version_Screen::Setup"
    puts "parent:  $parent"
         
    set top_level [winfo toplevel $parent]
   
    set tile_path [name_child $top_level version]
    
    set tile_class  $::Version_Screen::tile_class

    if { ![winfo exists $tile_path] } \
    {
      set tile_path  [::Screens::create_tile  $tile_path  version  $tile_class]

      # tie the scroller into the canvas
      $tile_path.parts.viewframe.view    config -yscrollcommand "$tile_path.parts.yscroll set"
      $tile_path.parts.yscroll config -command        "$tile_path.parts.viewframe.view yview"
      raise $tile_path.parts.components
      
      set view $tile_path.parts.viewframe.view
      set subs $tile_path.parts.subs
      # tie frame containing the children into the canvas
      $view create window 0 0 -anchor nw -window $subs -tag subs
      grid propagate $subs true
      # set up the auto-resize logic
      bind $subs <Configure> {
         set parent [winfo parent %W]
         set width  [winfo width  %W]
         set height [winfo reqheight %W]
         $parent.viewframe.view config -scrollregion "0 0 $width $height"
      }
      bind $view <Configure> {
         set parent [winfo parent [winfo parent %W]]
         set members $parent.subs
         set width  [winfo width  %W]
         set height [winfo height %W]
         set mh     [winfo reqheight $members]
         if {$mh > $height} { set height $mh}
         %W itemconfigure subs -width $width -height $height
      }
      
      # now, create the items in the component display
      grid columnconfig $subs 0 -weight 1
      grid columnconfig $subs 1 -weight 1
      set i 0
      foreach item [lsort [array names names]] \
      {
         set l [label $subs.label$i -text $names($item)]
         grid $l -column 0 -row $i -sticky news
         set v [label $subs.version$i -textvariable ::Version_Screen::version($item)]
         grid $v -column 1 -row $i -sticky news
         grid rowconfig $subs $i -weight 1
         incr i
      }
    }
    grid rowconfig $tile_path 0 -weight 0
    grid rowconfig $tile_path 1 -weight 0
    grid rowconfig $tile_path 2 -weight 0
    grid rowconfig $tile_path.parts 0 -weight 0
     
    return $tile_path
  }
   
  #****************************************************************************
  # Name:        Show_options
  # Parameters:  parent		parent of child window
  # Description: Dummy function
  #****************************************************************************
  proc Show_options { options_win } \
  {
    set options_url file://localhost/sysdata/html/help/Options/
    if ![winfo exists $options_win] \
    {
       ::HtmlFrame::make_toplevel $options_win $options_url "Installed options"
    }
  }
  
  #****************************************************************************
  # Name:        Map
  # Parameters:  parent		parent of child window
  # Description: Dummy function
  #****************************************************************************
  proc Map { name } \
  {
    after cancel "::Version_Screen::Cleanup_core $name"
    
    set options_win [name_child $name options]
    query_engine_versions

    after idle  "safe_focus_force $name.parts.yscroll"
    
    ::View::ClearAllSoftkeys  $name
    
    ::View::AssignSoftkey  $name  2  "Installed Options" "::Version_Screen::Show_options $options_win"
    
    ::View::AssignReturn  $name  ""
    
    # check if the serial number is set. If not, pop up the serial # screen
    switch -- [IFR_options ser] \
    {
       0 -
       "" \
       {
          set_ser_window $name
       }
       default \
       {
       }
    }
    return
  }

  #****************************************************************************
  # Name:        Cleanup_core
  # Parameters:  parent object
  # Description: Dummy function
  #****************************************************************************
  proc Cleanup_core { tile_path } \
  {
     catch { destroy $tile_path }
  }


  #****************************************************************************
  # Name:        Cleanup
  # Parameters:  parent object
  # Description: Dummy function
  #****************************************************************************
  proc Cleanup { tile_path } \
  {
     after 1000 "::Version_Screen::Cleanup_core $tile_path"
  }


  #****************************************************************************
  # Name:        Unmap
  # Parameters:  parent		parent of child window
  # Description: Dummy function
  #****************************************************************************
  proc Unmap { name } \
  {
     Cleanup $name
    return
  }

 
  #****************************************************************************
  # Name:        Destroy
  # Parameters:  parent		parent of child window
  # Description: Dummy function
  #****************************************************************************
  proc Destroy { name } \
  { 
     Cleanup $name
    return
  }
  
  #****************************************************************************
  # Name:        
  # Parameters:  
  # Description:
  #****************************************************************************
  proc validate_ser { widget } \
  {
     set toplevel [winfo toplevel $widget]
     variable Serial
     
     if {"$Serial(ser1)" != "$Serial(ser2)"} \
     {
        set Serial(msg) "Serial numbers don't match."
        set Serial(match) 0
     } \
     else \
     {
        set Serial(msg) "Serial number match. Thank you."
        set Serial(match) 1
     }
     switch -glob -- $widget \
     {
        *.serial1 \
        {
            after 100 "safe_focus_force $toplevel.serial2 "
        }
        *.serial2 \
        {
            if {$Serial(match) == 1} \
            {
               after 100 "safe_focus_force $toplevel.accept"
            } \
            else \
            {
               after 100 "safe_focus_force $toplevel.serial1"
            }
        }
        default \
        {
        }
     }
  }
  #****************************************************************************
  # Name:        
  # Parameters:  
  # Description:
  #****************************************************************************
  proc accept_ser { widget } \
  {
     set toplevel [winfo toplevel $widget]
     variable Serial
     
     if {"$Serial(ser1)" != "$Serial(ser2)"} \
     {
        set Serial(msg) "Serial numbers don't match."
        set Serial(match) 0
        
        after idle  "safe_focus_force $toplevel.serial1"
        
        return
     }
     IFR_options ser [string trimright $Serial(ser1)]
     destroy $toplevel
  }
  
  #****************************************************************************
  # Name:        
  # Parameters:  
  # Description:
  #****************************************************************************
  proc destroy_ser_window { widget } \
  {
     set toplevel [winfo toplevel $widget]
     destroy $toplevel
  }
  #****************************************************************************
  # Name:        
  # Parameters:  
  # Description:
  #****************************************************************************
  proc set_ser_window { widget } \
  {
     set master [winfo toplevel $widget]
     set ser_win [name_child $master serial_number]
     
     Screens::create_toplevel_tile $ser_win SERIAL
  }
  
  #****************************************************************************
  # Functions executed when the namespace is sourced
  #****************************************************************************
  ::View::RegisterScreen Version_Screen;  # Add this on the menu


};  # End of the Version_Screen name space


  
