#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C)1998,99      DESIGN ENGINEERING      #
# --------------------------------------------------------------------------#
# FILE        : 2110.Pwr_Term.tcl					    #
# PRODUCT     : FITS                                                        #
# AUTHOR      : Steve Layton                                                #
# DESCRIPTION : Manual Power Meter Calibration Screen			    #
#                                                                           #
#############################################################################


#*************************************************
#       Name: clear_powerterm
# Parameters: next, next mode 
#Description: clears power term calibration screen
#*************************************************
proc clear_powerterm {next} \
{
  # DESTROY CALLBACK
  destroy_pwt_callback

  # CHANGE PWT ENGINE ACTIONS
  Power enable on
}



namespace eval powerterm \
{

   variable term

   set term(low_plot)       0          	;# name of lineplot for low power cal
   set term(high_plot)      0         	;# name of lineplot for high power cal
   set term(freq_plot)      0         	;# name of lineplot for frequency cal
   set term(low_data)       {}         	;# plot data 
   set term(high_data)      {}        	;# plot data 
   set term(freq_data)      {}        	;# plot data 
   set term(ref_freq_index) 0    	;# Reference frequency
   set term(freqs) {100 300 500 700 900 1100 1300 1500 1700 1900 2100 2300 2500 2700}
   set term(ref_freq) [lindex $term(freqs) $term(ref_freq_index)]
   set term(caldate) " "
   set term(caltime) " "
   set term(current) 0                	;# {low high freq} current term table shown
   set term(read_status) INITIAL_READ 	;# {INITIAL_READ, PERFORM_CAL} when set to PERFORM_CAL, 
                                         # receiving cal value updates cal status
   set term(base)      0              	;# outer frame of low power cal screen
   set term(dacvalues) 0
   set term(notsaved)  0              	;# current dac values recieved from engine

   set term(lownominal0) .0020
   set term(lownominal1) .0040 
   set term(lownominal2) .0080
   set term(lownominal3) .0160
   set term(lownominal4) .0312
   set term(lownominal5) .0625
   set term(lownominal6) .1250
   set term(lownominal7) .2500
   set term(lownominal8) .5000
   set term(lownominal9) 1.0000

   set term(highnominal0) .2
   set term(highnominal1) .4 
   set term(highnominal2) .8
   set term(highnominal3) 1.6
   set term(highnominal4) 3.12
   set term(highnominal5) 6.25
   set term(highnominal6) 12.5
   set term(highnominal7) 25
   set term(highnominal8) 50
   set term(highnominal9) 100

   for {set i 0} {$i < 10} {incr i} \
   {
      set term(lowactual$i)  0
      set term(lowcal$i)     0
      set term(lowstatus$i)  0
      set term(lowcolor$i)   $::color(screenfg)
      set term(highactual$i) 0
      set term(highcal$i)    0
      set term(highstatus$i) 0  
      set term(highcolor$i)  $::color(screenfg)  
   }
  
   for {set i 0} {$i < 14} {incr i} \
   {
      set term(freqcal$i)    1
      set term(freqactual$i) 0
      set term(freqOld$i)    0
      set term(freqstatus$i) 0  
      set term(freqcolor$i)  $::color(screenfg)  
   }
  


  #*************************************************
  #       Name: exit_mess_powerterm
  # Parameters: 
  #Description: gives "are you sure message" if needed
  #*************************************************
  proc exit_mess_powerterm { } \
  {
    variable term
    
    if {$term(notsaved)} {
      set ::temporary(exitmess_displayed) [message_box .pwtnotsaved CAUTION]
      return 1
    }
  
    # DONT EXIT MODE ROUTINE, NO MESSAGE
    return 0
  }
  
  
  #*************************************************
  #       Name: request_pwt_cals
  # Parameters: 
  #Description: request power term cal values from EEPROM
  #*************************************************
  proc request_pwt_cals { } \
  {
    # READ CAL TABLES FROM ENGINE
    Power debug readcal  0
    Power debug readcal  1
    Power debug readcal  2

    # GET CAL DATE
    catch {Power debug getcalinfo } err
  }
  Power -onchange debug diskcal  {::powerterm::request_pwt_cals }

  
  #*************************************************
  #       Name: save_pwt_cals
  # Parameters: 
  #Description: save power term cal values
  #*************************************************
  proc save_pwt_cals { } \
  {
    variable term
   
    # SAVE CALS TO EEPROM
    Power debug calwrite 0
  
    # SEND DATE AND TIME TO DRIVER
    # RETRIEVE DATE - TIME FROM TEST SET
    set day     [string trimleft [clock format [clock seconds] -format %d] "0"]
    set month   [string trimleft [clock format [clock seconds] -format %m] "0"]
    set year    [string trimleft [clock format [clock seconds] -format %Y] "0"]
    set hour    [string trimleft [clock format [clock seconds] -format %H] "0"]
    set minute  [string trimleft [clock format [clock seconds] -format %M] "0"]       
  
    Power debug calday    $day
    Power debug calmonth  $month
    Power debug calyear   $year
    Power debug calhour   $hour
    Power debug calmin    $minute
  
    # SAVE DATE, VERSIONS TO EEPROM 
    Power debug timewrite
  
    # UPDATE DATE AND TIME DISPLAYED ON SCREEN  
    assemble_pwt_date
    
    set term(notsaved) 0
  }
  
  
  #*************************************************
  #       Name: assemble_pwt_date
  # Parameters: 
  #Description: assembles power term cal date and time, 
  #*************************************************
  proc assemble_pwt_date { } \
  {
    variable term
    
    set date $::Power::State(month)/$::Power::State(day)/$::Power::State(year)
     
    if {$date != $term(caldate)} {
      set term(caldate) $date
    } 
  
    set caltime $::Power::State(hour):[format %02d $::Power::State(minute)]
     
    if {$caltime != $term(caltime)} {
      set term(caltime) $caltime
    }
  }
  
  
  #*************************************************
  #       Name: receive_pwt_cal
  # Parameters: type, cal type {0 1 2} 0 low, 1 high 2 freq
  #       : pwr_or_freq, array of cal data (power or freq data)
  #       : ad_or_factor, array of cal data (AD values or freq factor)
  #       : status, bit map, 1 calibrated
  #       : freq_index, index of reference freq
  #Description: receives power term cal value.  This procedure
  #       : updates cal status 
  #*************************************************
  proc receive_pwt_cal {type pwr_or_freq ad_or_factor status freq_index} \
  {
    variable term
    
    # BIG BODGE I hate doing this...
    set tile ""
    if [info exists term(tile)] { set tile $term(tile)}
    
    #puts "receive_pwt_cal $type \{$pwr_or_freq\} \{$ad_or_factor\} $status $freq_index"
    
    set base $term(base)
  
    # SET REFERENCE FREQ
    if {$term(ref_freq_index) != $freq_index} {
  
      if {$freq_index >= 0 && $freq_index <= [llength $term(freqs)]} {
        set term(ref_freq_index) [string trimleft [expr int($freq_index)]]
        set term(ref_freq) [lindex $term(freqs) $freq_index]
      }
    }
  
    # LOW POWER VALUE RECEIVED
    if {$type == 0} {
  
      # UPDATE POWER 
      set index 0
      foreach value $pwr_or_freq {
  #puts "receive_pwt_cal $type $index $value"
        set term(lowactual$index) [round_digit [string trimleft $value] 4]
        incr index
      }
  
      # UPDATE CAL VALUE AND CAL STATUS
      set index 0
      foreach value $ad_or_factor {
  #puts "receive_pwt_cal $type $index $value"
        set term(lowcal$index) [expr int($value)]
          set term(lowstatus$index) [ expr ( $status & (1<<$index) ) ? 1 : 0 ]
        if {$term(lowstatus$index)} {
          set term(lowcolor$index) $::color(screenfg)
        } else {
          set term(lowcolor$index) $::color(messagewarning)
        }   
        if {$term(current) == "low" && [winfo exists $tile.hold.frame2.table.$index.frame3.actual$index]} {
          $tile.hold.frame2.table.$index.frame3.actual$index config -fg $term(lowcolor$index)
        }
        incr index
      } 
  
      # UPDATE PLOT AND ONSCREEN COLOR
      set term(low_data) {}
      for {set index 0} {$index < 10} {incr index} {
        if {$term(lowcal$index) != "working"} {
          lappend term(low_data) $term(lowcal$index) $term(lowactual$index)
        }
      }
  
      if {[winfo exists $term(low_plot)]} {
        $term(low_plot) erase [$term(low_plot) cget -data]
        $term(low_plot) add $term(low_data)
      }
          
    # HIGH POWER VALUE RECEIVED
    } elseif {$type == 1} {
  
      # UPDATE POWER 
      set index 0
      foreach value $pwr_or_freq {
  #puts "receive_pwt_cal $type $index $value"
        set term(highactual$index) [round_digit [string trimleft $value] 4]
        incr index
      }
  
      # UPDATE CAL VALUE AND CAL STATUS
      set index 0
      foreach value $ad_or_factor {
  #puts "receive_pwt_cal $type $index $value"
        set term(highcal$index) [expr int($value)]
          set term(highstatus$index) [ expr ( $status & (1<<$index) ) ? 1 : 0 ]
  
        if {$term(highstatus$index)} {
          set term(highcolor$index) $::color(screenfg)
        } else {
          set term(highcolor$index) $::color(messagewarning)
        }   
        if {$term(current) == "high" && [winfo exists $tile.hold.frame2.table.$index.frame3.actual$index]} {
          $tile.hold.frame2.table.$index.frame3.actual$index config -fg $term(highcolor$index)
        }
        incr index
      } 
    
      # UPDATE PLOT AND ONSCREEN COLOR
      set term(high_data) {}
      for {set index 0} {$index < 10} {incr index} {
        if {$term(highcal$index) != "working"} {
          lappend term(high_data) $term(highcal$index) $term(highactual$index)
        }
      }
      if {[winfo exists $term(high_plot)]} {
        $term(high_plot) erase [$term(high_plot) cget -data]
        $term(high_plot) add $term(high_data)
      }
  
  
    # FREQUENCY VALUE RECEIVED
    } elseif {$type == 2} {
    
      # UPDATE CAL VALUE AND CAL STATUS
      set term(freq_data) {}
      set index 0
      foreach value $ad_or_factor {
  #puts "receive_pwt_cal $type $index $value"
        set term(freqcal$index) [round_digit [string trimleft $value] 4]
        if {$term(freqcal$index) != "working"} {
          lappend term(freq_data) [lindex $term(freqs) $index] $term(freqcal$index)
        }
          set term(freqstatus$index) [ expr ( $status & (1<<$index) ) ? 1 : 0 ]
        if {$term(freqstatus$index)} {
          set term(freqcolor$index) $::color(screenfg)
        } else {
          set term(freqcolor$index) $::color(messagewarning)
        }       
  
        if {$index < 7} {
          if {$term(current) == "freq" && [winfo exists $tile.hold.plot.$index.frame2.leftactual$index]} {
            $tile.hold.plot.$index.frame2.leftactual$index config -fg $term(freqcolor$index)
          }
        } else {
          if {$term(current) == "freq" && [winfo exists $tile.hold.plot.[expr $index-7].frame5.rightactual[expr $index-7]]} {
            $tile.hold.plot.[expr $index-7].frame5.rightactual[expr $index-7] config -fg $term(freqcolor$index)
          }
        }
  
        incr index
      } 
  
      # UPDATE PLOT AND ONSCREEN COLOR
      if {[winfo exists $term(freq_plot)]} {
        $term(freq_plot) erase [$term(freq_plot) cget -data]
        $term(freq_plot) add $term(freq_data)
      }
    }
  
  }
  
  
  #*************************************************
  #       Name: request_pwt_cal
  # Parameters: type, {low high freq}
  #       : point, cal point to calculate
  #       : plot_name, name of lineplot widget
  #Description: asks for cal point during calibration, this is to be
  #       : called as result of power level entry.
  #*************************************************
  proc request_pwt_cal { type point } \
  {
    variable term
    
    if {$type == "low"} {
        # ERASE OLD POINT FROM PLOT, GET OLD VALUE FROM ENTRY ROUTINE
      set old_index [lsearch -exact [$term(low_plot) cget -data] $term(freqOld$point)]
  
      if {$old_index != -1 && [lindex [$term(low_plot) cget -data] [expr $old_index-1]] != "working"} \
      {
        $term(low_plot) erase "[lindex [$term(low_plot) cget -data] [expr $old_index-1]] \
	                       [lindex [$term(low_plot) cget -data] $old_index]"
      }
  
      # PUT "WORKING" INTO CAL VALUE COLUMN
      set term(lowcal$point) working
      
      # SEND POWER VALUE, THEN REQUEST CAL VALUE
      update_pwt_cal_power 0 $point $term(lowactual$point)
      
      # MOVE CURSOR TO NEXT FIELD
      ::Cursor::move_cursor_down [focus]
      
      
    } elseif {$type == "high"} {
      # ERASE OLD POINT FROM PLOT, GET OLD VALUE FROM ENTRY ROUTINE
      set old_index [lsearch -exact [$term(high_plot) cget -data] $term(freqOld$point)]
      if {$old_index != -1 && [lindex [$term(high_plot) cget -data] [expr $old_index-1]] != "working"} \
      {
        $term(high_plot) erase "[lindex [$term(high_plot) cget -data] [expr $old_index-1]] \
	                        [lindex [$term(high_plot) cget -data] $old_index]"
      }
  
      # PUT "WORKING" INTO CAL VALUE COLUMN
      set term(highcal$point) working
    
      # SEND POWER VALUE, THEN REQUEST CAL VALUE
      update_pwt_cal_power 1 $point $term(highactual$point)
      
      # MOVE CURSOR TO NEXT FIELD
      ::Cursor::move_cursor_down [focus]
  
    
    } elseif {$type == "freq"} {
      # ERASE OLD POINT FROM PLOT
      set freq_point [lindex $term(freqs) $point]
      set old_index [lsearch -exact [$term(freq_plot) cget -data] $freq_point]    
  
      if {$old_index != -1 && [lindex [$term(freq_plot) cget -data] $old_index] != "working"} \
      {
        $term(freq_plot) erase "[lindex [$term(freq_plot) cget -data] $old_index] \
	                        [lindex [$term(freq_plot) cget -data] [expr 1+$old_index]]"
      }
  
      # PUT "WORKING" INTO CAL VALUE COLUMN
      set term(freqcal$point) working
  
      # SEND POWER VALUE, THEN REQUEST CAL VALUE
      update_pwt_cal_freq 1 $point $term(freqactual$point)
      
      # MOVE CURSOR TO NEXT FIELD
      if {$point != 6} {
        ::Cursor::move_cursor_down [focus]
      } else {

        after idle  "safe_focus_force $tile.hold.plot.0.frame5.rightactual0"
      }
    } 
    
    set term(notsaved) 1
  }
  

  #*************************************************
  #       Name: report_pwt_cal_status
  # Parameters: 
  #Description: for debug
  #*************************************************
  proc report_pwt_cal_status { } \
  {
    variable term
    
    for {set i 0} {$i < 10} {incr i} {
      put "STATUS: LOW $i $term(lowstatus$i), HIGH $term(highstatus$i)"
    }   
    
    for {set i 0} {$i < 14} {incr i 2} {  
      put "STATUS: FREQ $i $term(freqstatus$i), [expr $i+1] $term(freqstatus[expr $i+1])"
    }
  } 

  
  #*************************************************
  #       Name: reset_pwt_cals
  # Parameters: current_edit, index of value currently in edit, if any
  #Description: zero local copy of power term cals
  #*************************************************
  proc reset_pwt_cals { { current_edit -100 } } \
  {
    variable term

    set tile $::PowerTerm_Screen::tile_path
    
    # ZERO VALUES
    for {set i 0} {$i < 10} {incr i} {
      if {$i != $current_edit || $term(current) != "low"} {
        set term(lowactual$i) 0
      }
      set term(lowcal$i) 0
      set term(lowstatus$i) 0
      set term(lowcolor$i) $::color(messagewarning)
      if {$i != $current_edit || $term(current) != "high"} {    
        set term(highactual$i) 0
      }
      set term(highcal$i) 0
      set term(highstatus$i) 0
      set term(highcolor$i) $::color(messagewarning)
    }
    
    for {set i 0} {$i < 14} {incr i} {  
  
      if {$i != $current_edit || $term(current) != "freq"} {
        set term(freqactual$i) 0
        set term(freqcal$i) 1
        if {$i != $term(ref_freq_index)} {
          set term(freqstatus$i) 0
          set term(freqcolor$i) $::color(messagewarning)
        }
      }
    }
  
    # RECOLOR VALUES
    if {$term(current) == "low"} {
      for {set i 0} {$i < 10} {incr i} {
        if {$i != $current_edit} {
          $tile.hold.frame2.table.$i.frame3.actual$i config -fg $::color(messagewarning)
        }
      }
    } elseif {$term(current) == "high"} {
      for {set i 0} {$i < 10} {incr i} {
        if {$i != $current_edit} {
          $tile.hold.frame2.table.$i.frame3.actual$i config -fg $::color(messagewarning)
        }
      }
  
    } elseif {$term(current) == "freq"} {
      for {set i 0} {$i < 7} {incr i} {
        if {$i != $term(ref_freq_index) && $i != $current_edit} {
          $tile.hold.plot.$i.frame2.leftactual$i config -fg $::color(messagewarning)
        }
        if {[expr $i+7] != $term(ref_freq_index) && [expr $i+7] != $current_edit} {
          $tile.hold.plot.$i.frame5.rightactual$i config -fg $::color(messagewarning)
        }
      }
    }
    
  
    # REPLOT NO DATA
    if {$term(current) != "freq"} {
      $term($term(current)\_plot) erase [$term($term(current)\_plot) cget -data]
      set term($term(current)\_data) {0 0}
    } else {
      for {set i 0} {$i < 14} {incr i} {
        if {$term(freqcal$i) != "working"} {
          lappend term(freq_data) [lindex $term(freqs) $i]
          lappend term(freq_data) $term(freqcal$i)
        }
      }
    }
  }
  
  
  #*************************************************
  #       Name: set_pwt_referfreq
  # Parameters: 
  #Description: change reference frequency
  #*************************************************
  proc set_pwt_referfreq { } \
  {
    variable term
    
    # SET REFERENCE FREQUENCY INDEX
    set term(ref_freq_index) [lsearch -exact $term(freqs) $term(ref_freq)]
    
    reset_pwt_cals
    
    # SET REFERENCE FREQUENCY CAL VALUE
    set term(freqcal$term(ref_freq_index)) 1.0000
  
    set_pwt_factor_init $term(ref_freq_index)
    
  }
  
  
  #*************************************************
  #       Name: pwt_big_plot
  # Parameters: 
  #Description: displays big plot
  #*************************************************
  proc pwt_big_plot { } \
  {
    variable term
    
    # VERIFY CORRECT MODE IS STILL VALID
    if {$::sys(mode) != "powerterm"} {return}
  
    # DETERMINE TITLE
    set plot_text ""
    if {$term(current) == "low"} {
      set plot_text "LOW POWER CALIBRATION PLOT"
  
      if {![draw_toplevel .bigplot 400 430 $plot_text]} {
        return
      }
      draw_frame .bigplot.top 370 430
       draw_frame .bigplot.low 30 430 175 80 175
      draw_button .bigplot.low.frame2.exit "RETURN" "destroy .bigplot" 0 0 0 center c
  
      lineplot .bigplot.top.plot -width 430 -height 370 -autosizex false -autosizey false -maxx 4096 -maxy 2 -smooth false -unity W -data [$term($term(current)\_plot) cget -data] -smooth true -splinesteps 200
      
    } elseif {$term(current) == "high"} {
      if {![set plot_text "HIGH POWER CALIBRATION PLOT"]} {
        return
      }
  
      draw_toplevel .bigplot 400 430 $plot_text
      draw_frame .bigplot.top 370 430
       draw_frame .bigplot.low 30 430 175 80 175
      draw_button .bigplot.low.frame2.exit "RETURN" "destroy .bigplot" 0 0 0 center c
  
      lineplot .bigplot.top.plot -width 430 -height 370 -autosizex false -autosizey false -maxx 4096 -maxy 200 -smooth false -unity W -data [$term($term(current)\_plot) cget -data] -smooth true -splinesteps 200
  
    } elseif {$term(current) == "freq"} {
      set plot_text "FREQUENCY CALIBRATION PLOT"        
  
      if {![draw_toplevel .bigplot 330 500 $plot_text]} {
        return
      }
      draw_frame .bigplot.top 300 500
       draw_frame .bigplot.low 30 500 210 80 210
      draw_button .bigplot.low.frame2.exit "RETURN" "destroy .bigplot" 0 0 0 center c
  
      lineplot .bigplot.top.plot -ticcountx 14 -width 500 -height 300 -autosizex false -autosizey false -maxx 2700 -maxy 1.3 -miny 1 -smooth false -data [$term($term(current)\_plot) cget -data] -originplacement middle -ticcounty 10 -smooth true -splinesteps 200
    }
  
    # ESTABLISH FOCUS
      if {[winfo viewable .bigplot.low.frame2.exit]} {
      after idle  "safe_focus_force .bigplot.low.frame2.exit"
      grab -global .bigplot
    } else {
      bind .bigplot.low.frame2.exit <Visibility> "safe_focus_force .bigplot.low.frame2.exit;grab -global .bigplot; bind .bigplot.low.frame2.exit <Visibility> { }"
    }
    
  }
  
  
  #*************************************************
  #       Name: build_freq_power
  # Parameters: 
  #Description: builds freq power calibration screen
  #*************************************************
  proc build_freq_power { tile } \
  {
    variable term
  
    # BIG BODGE I hate doing this...
    
    set term(tile) $tile
      
    # SET POWER RANGE
    Power prange LOW
      
    # DONT REDRAW freq TABLE IF ALREADY SHOWN
    if {$term(current) == "freq"} {return}
        
    destroy $tile.hold
    
    #title_mode "POWER TERM FREQ POWER CALIBRATION"
    
    set term(current) freq
    
    draw_frame $tile.hold 440 555
  
  
    # DRAW FIELDS
    draw_frame $tile.hold.top 160 555 230 323 2
    
    # DATE, TIME  
    draw_frame $tile.hold.top.frame1.date $::line_height 230 80 55 95
    draw_label $tile.hold.top.frame1.date.frame1.label "LAST CAL" 8
    draw_label $tile.hold.top.frame1.date.frame2.text " " 5
    $tile.hold.top.frame1.date.frame2.text config -textvariable powerterm::term(caltime)
    draw_label $tile.hold.top.frame1.date.frame3.text " " 8
    $tile.hold.top.frame1.date.frame3.text config -textvariable powerterm::term(caldate)
    
    # REFERENCE FREQUENCY
    draw_frame $tile.hold.top.frame1.freq $::line_height 230 152 40 38
    draw_label $tile.hold.top.frame1.freq.frame1.label "REFERENCE FREQ" 15
    draw_label $tile.hold.top.frame1.freq.frame2.pop $term(ref_freq) 9
    $tile.hold.top.frame1.freq.frame2.pop config -justify right
    draw_label $tile.hold.top.frame1.freq.frame3.unit "MHz" 3
  
    # ZERO VALUE
    draw_frame $tile.hold.top.frame1.zero $::line_height 230 100 35 60 35 
    draw_label $tile.hold.top.frame1.zero.frame1.label "ZERO:LOW" 8
    draw_entry $tile.hold.top.frame1.zero.frame2.temp Power::State(low_zero) 7 
    $tile.hold.top.frame1.zero.frame2.temp config -state disabled -justify left
    draw_label $tile.hold.top.frame1.zero.frame3.label "HIGH" 4
    draw_entry $tile.hold.top.frame1.zero.frame4.temp Power::State(high_zero) 7 
    $tile.hold.top.frame1.zero.frame4.temp config -state disabled -justify left 
  
    #draw_frame $tile.hold.top.frame1.zerolow $::line_height 230 130 60 40
    #draw_label $tile.hold.top.frame1.zerolow.frame1.label "LOW ZERO" 15  
    #draw_entry $tile.hold.top.frame1.zerolow.frame2.temp Power::State(low_zero) 7 
    #$tile.hold.top.frame1.zerolow.frame2.temp config -state disabled -justify left
  
    #draw_frame $tile.hold.top.frame1.zerohigh $::line_height 230 130 60 40
    #draw_label $tile.hold.top.frame1.zerohigh.frame1.label "HIGH ZERO" 15  
    #draw_entry $tile.hold.top.frame1.zerohigh.frame2.temp Power::State(high_zero) 7 
    #$tile.hold.top.frame1.zerohigh.frame2.temp config -state disabled -justify left
  
    # DAC VALUES
    draw_frame $tile.hold.top.frame1.dacs $::line_height 230 145 60 25
    draw_label $tile.hold.top.frame1.dacs.frame1.label "DAC VALUES" 15  
    draw_label $tile.hold.top.frame1.dacs.frame2.dacvalues " " 4 
    $tile.hold.top.frame1.dacs.frame2.dacvalues config -textvariable Power::State(ad_value)
  
    # TEMPS
    draw_frame $tile.hold.top.frame1.temp1 $::line_height 230 145 60 25
    draw_label $tile.hold.top.frame1.temp1.frame1.label "POWER PAD TEMP" 15 
    draw_entry $tile.hold.top.frame1.temp1.frame2.temp Power::State(temp1) 7 
    $tile.hold.top.frame1.temp1.frame2.temp config -state disabled
    draw_label $tile.hold.top.frame1.temp1.frame3.label "C" 1 0 0 left w
      
    draw_frame $tile.hold.top.frame1.temp2 $::line_height 230 145 60 25
    draw_label $tile.hold.top.frame1.temp2.frame1.label "DETECTOR TEMP" 15  
    draw_entry $tile.hold.top.frame1.temp2.frame2.temp Power::State(temp2) 7 
    $tile.hold.top.frame1.temp2.frame2.temp config -state disabled
    draw_label $tile.hold.top.frame1.temp2.frame3.label "C" 1 0 0 left w  
  
    draw_frame $tile.hold.top.frame2.plot 160 321
    $tile.hold.top.frame2.plot config -bd 1 -relief groove
    #set freqpower_data {}
    #for {set i 0} {$i < 14} {incr i} {
    # if {$term(freqcal$i) != "working"} {
    #   lappend freqpower_data [lindex $term(freqs) $i]
    #   lappend freqpower_data $term(freqcal$i)
    # }
    #}
  
    lineplot $tile.hold.top.frame2.plot.freqpower -ticcountx 14 -width 323 -height 160 -autosizex false -autosizey false -maxx 2700 -maxy 1.3 -miny 1 -smooth false -data $term(freq_data) -originplacement middle -ticcounty 10 -smooth true -splinesteps 200
    set term(freq_plot) $tile.hold.top.frame2.plot.freqpower
  
    set row  20
    set linecolor darkgrey
    set linethickness 1
  
    # DRAW TABLE
     draw_frame $tile.hold.plot 280 555
      set table $tile.hold.plot
  
      # CREATE COLUMN HEADS
    draw_frame $table.head $row 555 80 85 110 80 85 110
    $table.head.frame1 config -bg $linecolor -bd $linethickness
    $table.head.frame2 config -bg $linecolor -bd $linethickness
    $table.head.frame3 config -bg $linecolor -bd $linethickness
    $table.head.frame4 config -bg $linecolor -bd $linethickness     
    $table.head.frame5 config -bg $linecolor -bd $linethickness
    $table.head.frame6 config -bg $linecolor -bd $linethickness     
  
    draw_label $table.head.frame1.leftpoint "FREQ" 6 0 0 center c
    draw_label $table.head.frame2.leftactual "INPUT W" 9 0 0 center c
    draw_label $table.head.frame3.leftcal "CAL VALUE" 7 0 0 center c
    draw_label $table.head.frame4.rightpoint "FREQ" 6 0 0 center c
    draw_label $table.head.frame5.rightactual "INPUT W" 9 0 0 center c
    draw_label $table.head.frame6.rightcal "CAL VALUE" 7 0 0 center c
  
   for {set i 0} {$i < 7} {incr i} \
   {
      draw_frame $table.$i $row 555 80 85 110 80 85 110   
      $table.$i.frame1 config -bg $linecolor -bd $linethickness
      $table.$i.frame2 config -bg $linecolor -bd $linethickness
      $table.$i.frame3 config -bg $linecolor -bd $linethickness
      $table.$i.frame4 config -bg $linecolor -bd $linethickness           
      $table.$i.frame5 config -bg $linecolor -bd $linethickness
      $table.$i.frame6 config -bg $linecolor -bd $linethickness           
         
      draw_label $table.$i.frame1.leftpoint$i [lindex $term(freqs) $i] 6 0 0 center c
      draw_entry   $table.$i.frame2.leftactual$i ::powerterm::term(freqactual$i) 9 $term(freqcolor$i) center
      bind_numeric $table.$i.frame2.leftactual$i 3 4 200 0 "::powerterm::request_pwt_cal freq $i"
      # arrange to save the old value on edit open.
      bind $table.$i.frame2.leftactual$i <<EditOpen>> "set term(freqOld$i) $term(freqactual$i)"
      draw_entry $table.$i.frame3.leftcal$i ::powerterm::term(freqcal$i) 7 0 center
      $table.$i.frame3.leftcal$i config -state disabled
            
      draw_label $table.$i.frame4.rightpoint$i [lindex $term(freqs) [expr $i+7]] 6 0 0 center c
      draw_entry $table.$i.frame5.rightactual$i ::powerterm::term(freqactual[expr $i+7]) 9 $term(freqcolor[expr $i+7]) center
      bind_numeric $table.$i.frame5.rightactual$i 3 4 200 0 "::powerterm::request_pwt_cal freq [expr $i+7]"
      draw_entry $table.$i.frame6.rightcal$i ::powerterm::term(freqcal[expr $i+7]) 7 0 center
      $table.$i.frame6.rightcal$i config -state disabled      
    }
  
    # LOCK OUT REFERENCE FREQUENCY ENTRY
    if {$term(ref_freq_index) < 7} {
      $table.$term(ref_freq_index).frame2.leftactual$term(ref_freq_index) config -state disabled -fg white -bg grey40
      $table.$term(ref_freq_index).frame3.leftcal$term(ref_freq_index) config -fg white -bg grey40
    } else {
      $table.[expr $term(ref_freq_index)-7].frame5.rightactual[expr $term(ref_freq_index)-7] config -state disabled -fg white -bg grey40
      $table.[expr $term(ref_freq_index)-7].frame6.rightcal[expr $term(ref_freq_index)-7] config -fg white -bg grey40
    }
  
  
    # SET FOCUS
    if {$term(ref_freq_index) == 0} {
      if {[winfo viewable $table.1.frame2.leftactual1]} {
        after idle  "safe_focus_force $table.1.frame2.leftactual1"
      } else {
        bind $table.1.frame2.leftactual1 <Visibility> "safe_focus_force $table.1.frame2.leftactual1;bind $table.1.frame2.leftactual1 <Visibility> { }"
      } 
    } else {
      if {[winfo viewable $table.0.frame2.leftactual0]} {
        after idle  "safe_focus_force $table.0.frame2.leftactual0"
      } else {
        bind $table.0.frame2.leftactual0 <Visibility> "safe_focus_force $table.0.frame2.leftactual0;bind $table.0.frame2.leftactual0 <Visibility> { }"
      } 
    }
  
  }
  
  
  #*************************************************
  #       Name: build_high_power
  # Parameters: 
  #Description: builds high power calibration screen
  #*************************************************
  proc build_high_power { tile } \
  {
    variable term
  
    # SET POWER RANGE
    Power prange HIGH
      
    # DONT REDRAW HIGH TABLE IF ALREADY SHOWN
    if {$term(current) == "high"} {return}
        
    destroy $tile.hold

    #title_mode "POWER TERM HIGH POWER CALIBRATION"

    set term(current) high
    draw_frame $tile.hold 440 555 230 325
  
  
    # DRAW FIELDS
    draw_frame $tile.hold.frame1.space0 5 230
              
    draw_frame $tile.hold.frame1.date $::line_height 230 80 55 95
    draw_label $tile.hold.frame1.date.frame1.label "LAST CAL" 8
    draw_label $tile.hold.frame1.date.frame2.text " " 5
    $tile.hold.frame1.date.frame2.text config -textvariable powerterm::term(caltime)
    draw_label $tile.hold.frame1.date.frame3.text " " 8
    $tile.hold.frame1.date.frame3.text config -textvariable powerterm::term(caldate)
    
    draw_frame $tile.hold.frame1.space02 $::line_height 230
    
    # REFERENCE FREQUENCY
    draw_frame $tile.hold.frame1.freq $::line_height 230 152 40 38
    draw_label $tile.hold.frame1.freq.frame1.label "REFERENCE FREQ" 15
    draw_label $tile.hold.frame1.freq.frame2.pop $term(ref_freq) 9
    $tile.hold.frame1.freq.frame2.pop config -justify right
    draw_label $tile.hold.frame1.freq.frame3.unit "MHz" 3
  
    draw_frame $tile.hold.frame1.space2 $::line_height 230
    
    draw_frame $tile.hold.frame1.zerov $::line_height 230 130 60 40
    draw_label $tile.hold.frame1.zerov.frame1.label "ZERO VALUE" 15 
    draw_entry $tile.hold.frame1.zerov.frame2.temp Power::State(high_zero) 7 
    $tile.hold.frame1.zerov.frame2.temp config -state disabled -justify left
  
    #draw_frame $tile.hold.frame1.space4 $::line_height 230
    
    draw_frame $tile.hold.frame1.temp1 $::line_height 230 145 60 25
    draw_label $tile.hold.frame1.temp1.frame1.label "POWER PAD TEMP" 15 
    draw_entry $tile.hold.frame1.temp1.frame2.temp Power::State(temp1) 7 
    $tile.hold.frame1.temp1.frame2.temp config -state disabled
    draw_label $tile.hold.frame1.temp1.frame3.label "C" 1 0 0 left w  
      
    draw_frame $tile.hold.frame1.temp2 $::line_height 230 145 60 25
    draw_label $tile.hold.frame1.temp2.frame1.label "DETECTOR TEMP" 15  
    draw_entry $tile.hold.frame1.temp2.frame2.temp Power::State(temp2) 7 
    $tile.hold.frame1.temp2.frame2.temp config -state disabled
    draw_label $tile.hold.frame1.temp2.frame3.label "C" 1 0 0 left w  
  
    # RAW DAC VALUES
    draw_frame $tile.hold.frame1.dacs $::line_height 230 145 60 25
    draw_label $tile.hold.frame1.dacs.frame1.label "DAC VALUES" 15  
    draw_label $tile.hold.frame1.dacs.frame2.dacvalues " " 4 
    $tile.hold.frame1.dacs.frame2.dacvalues config -textvariable Power::State(ad_value)
  
    draw_frame $tile.hold.frame1.space03 $::line_height 230
    
    draw_frame $tile.hold.frame1.plottitle $::line_height 230
    draw_label $tile.hold.frame1.plottitle.text "HIGH POWER PLOT" 14 0 0 center c
    
    draw_frame $tile.hold.frame1.plot [expr 9*$::line_height] 230
    $tile.hold.frame1.plot config -bd 1 -relief groove
    #set highpower_data {}
    #for {set i 0} {$i < 10} {incr i} {
    # if {$term(highcal$i) != "working"} {
    #   lappend highpower_data $term(highcal$i)
    #   lappend highpower_data $term(highactual$i)
    # }
    #}
  
    lineplot $tile.hold.frame1.plot.highpower -width 228 -height 200 -autosizex false -autosizey false -maxx 4096 -maxy 200 -smooth false -unity W -data $term(high_data) -smooth true -splinesteps 200
    set term(high_plot) $tile.hold.frame1.plot.highpower
  
  
    # DRAW TABLE
    draw_frame $tile.hold.frame2.table 430 325
  # draw_frame $tile.hold.frame2.space 6 325  
      
      set table $tile.hold.frame2.table
      
    set row 11 
    set linecolor darkgrey
    set linethickness 1
  
    draw_frame $table.head 30 325 60 90 100 75
    $table.head.frame1 config -bg $linecolor -bd $linethickness
    $table.head.frame2 config -bg $linecolor -bd $linethickness
    $table.head.frame3 config -bg $linecolor -bd $linethickness
    $table.head.frame4 config -bg $linecolor -bd $linethickness     
    draw_label $table.head.frame1.point "CAL\nPOINT" 6 0 0 center c
    draw_label $table.head.frame2.nominal "NOMINAL\nINPUT W" 9 0 0 center c
    draw_label $table.head.frame3.actual "ACTUAL\nINPUT W" 9 0 0 center c
    draw_label $table.head.frame4.cal "CAL\nVALUE" 7 0 0 center c
  
    for {set i 0} {$i < 10} {incr i} {
        
      draw_frame $table.$i $row 325 60 90 100 75
      $table.$i.frame1 config -bg $linecolor -bd $linethickness
      $table.$i.frame2 config -bg $linecolor -bd $linethickness
      $table.$i.frame3 config -bg $linecolor -bd $linethickness
      $table.$i.frame4 config -bg $linecolor -bd $linethickness           
      draw_label $table.$i.frame1.point$i [expr $i+1] 6 0 0 center c
      draw_label $table.$i.frame2.nominal$i $term(highnominal$i) 9 0 0 center c
      draw_entry $table.$i.frame3.actual$i ::powerterm::term(highactual$i) 9 $term(highcolor$i) center
      bind_numeric $table.$i.frame3.actual$i 3 4 250 .1 "::powerterm::request_pwt_cal high $i"
      draw_entry $table.$i.frame4.cal$i ::powerterm::term(highcal$i) 7 0 center
      $table.$i.frame4.cal$i config -state disabled     
    }       
  
    # SET FOCUS
    if {[winfo viewable $table.0.frame3.actual0]} {
      after idle  "safe_focus_force $table.0.frame3.actual0"
    } else {
      bind $table.0.frame3.actual0 <Visibility> "safe_focus_force $table.0.frame3.actual0;bind $table.0.frame3.actual0 <Visibility> { }"
    } 
  
  }
  
  
  #***************************************************************************
  #
  #       Name: build_low_power
  #
  # Parameters: tile
  #
  # Description: builds low power calibration screen
  #
  #***************************************************************************
  proc build_low_power { tile } \
  {
    variable term
  
    # SET POWER RANGE
    Power prange LOW
  
    # DONT REDRAW LOW TABLE IF ALREADY SHOWN
    if { $term(current) == "low" } { return }
  
    if { [winfo exists $tile.hold] } \
    {
      destroy $tile.hold
    }
  
    #title_mode "POWER TERM LOW POWER CALIBRATION"

    set term(current) low

    draw_frame $tile.hold 440 555 230 325
  
    # DRAW FIELDS
    draw_frame $tile.hold.frame1.space0 5 230
    
    draw_frame $tile.hold.frame1.date $::line_height 230 80 55 95
    draw_label $tile.hold.frame1.date.frame1.label "LAST CAL" 8
    draw_label $tile.hold.frame1.date.frame2.text " " 5

    $tile.hold.frame1.date.frame2.text config -textvariable powerterm::term(caltime)

    draw_label $tile.hold.frame1.date.frame3.text " " 8

    $tile.hold.frame1.date.frame3.text config -textvariable powerterm::term(caldate)
    
    draw_frame $tile.hold.frame1.space02 $::line_height 230
    
    # REFERENCE FREQUENCY
    draw_frame $tile.hold.frame1.freq [expr 2*$::line_height] 230 125 55 50
    draw_label $tile.hold.frame1.freq.frame1.label "REFERENCE\nFREQUENCY" 15

    draw_popdown $tile.hold.frame1.freq.frame2.pop ::powerterm::term(ref_freq) \
                                     "::powerterm::set_pwt_referfreq" [lrange $term(freqs) 0 5]

    $tile.hold.frame1.freq.frame2.pop config -font big -justify right 

    draw_label $tile.hold.frame1.freq.frame3.unit "MHz" 3

    $tile.hold.frame1.freq.frame3.unit config -font big -justify left -anchor w
  
    # draw_frame $tile.hold.frame1.space2 $::line_height 230
    
    # ZERO VALUE
    draw_frame $tile.hold.frame1.zerov $::line_height 230 130 60 40
    draw_label $tile.hold.frame1.zerov.frame1.label "ZERO VALUE" 15 
    draw_entry $tile.hold.frame1.zerov.frame2.temp Power::State(low_zero) 7 

    $tile.hold.frame1.zerov.frame2.temp config -state disabled -justify left
  
    # POWER PAD TEMP
    draw_frame $tile.hold.frame1.temp1 $::line_height 230 145 60 25
    draw_label $tile.hold.frame1.temp1.frame1.label "POWER PAD TEMP" 15 
    draw_entry $tile.hold.frame1.temp1.frame2.temp Power::State(temp1) 7 

    $tile.hold.frame1.temp1.frame2.temp config -state disabled

    draw_label $tile.hold.frame1.temp1.frame3.label "C" 1 0 0 left w
      
    # DETECTOR TEMP
    draw_frame $tile.hold.frame1.temp2 $::line_height 230 145 60 25
    draw_label $tile.hold.frame1.temp2.frame1.label "DETECTOR TEMP" 15  
    draw_entry $tile.hold.frame1.temp2.frame2.temp Power::State(temp2) 7 
    $tile.hold.frame1.temp2.frame2.temp config -state disabled
    draw_label $tile.hold.frame1.temp2.frame3.label "C" 1 0 0 left w  
  
    # RAW DAC VALUES
    draw_frame $tile.hold.frame1.dacs $::line_height 230 145 60 25
    draw_label $tile.hold.frame1.dacs.frame1.label "DAC VALUES" 15  
    draw_label $tile.hold.frame1.dacs.frame2.dacvalues " " 4 
    $tile.hold.frame1.dacs.frame2.dacvalues config -textvariable Power::State(ad_value)
    
  
    draw_frame $tile.hold.frame1.space03 $::line_height 230
    
    draw_frame $tile.hold.frame1.plottitle $::line_height 230
    draw_label $tile.hold.frame1.plottitle.text "LOW POWER PLOT" 14 0 0 center c
    
    draw_frame $tile.hold.frame1.plot [expr 9*$::line_height] 230
    $tile.hold.frame1.plot config -bd 1 -relief groove
  
    lineplot $tile.hold.frame1.plot.lowpower -width 228 -height 200 \
                                             -autosizex false       \
                                             -autosizey false       \
                                             -maxx 4096             \
                                             -maxy 2 -smooth false  \
                                             -unity W               \
                                             -data $term(low_data)  \
                                             -smooth true           \
                                             -splinesteps 200
                                             
    set term(low_plot) $tile.hold.frame1.plot.lowpower
  
  
    # DRAW TABLE
    draw_frame $tile.hold.frame2.table 430 325
    
    $tile.hold.frame2.table config -bg darkgrey
      
    set table $tile.hold.frame2.table
      
    set row  11
    set linecolor darkgrey
    set linethickness 1
  
    draw_frame $table.head 30 325 60 90 100 75
    
    $table.head.frame1 config -bg $linecolor -bd $linethickness
    $table.head.frame2 config -bg $linecolor -bd $linethickness
    $table.head.frame3 config -bg $linecolor -bd $linethickness
    $table.head.frame4 config -bg $linecolor -bd $linethickness     
    
    draw_label $table.head.frame1.point "CAL\nPOINT" 6 0 0 center c
    draw_label $table.head.frame2.nominal "NOMINAL\nINPUT W" 9 0 0 center c
    draw_label $table.head.frame3.actual "ACTUAL\nINPUT W" 9 0 0 center c
    draw_label $table.head.frame4.cal "CAL\nVALUE" 7 0 0 center c
  
    for {set i 0} {$i < 10} {incr i} \
    {
      draw_frame $table.$i $row 325 60 90 100 75
      $table.$i.frame1 config -bg $linecolor -bd $linethickness
      $table.$i.frame2 config -bg $linecolor -bd $linethickness
      $table.$i.frame3 config -bg $linecolor -bd $linethickness
      $table.$i.frame4 config -bg $linecolor -bd $linethickness           
      
      draw_label $table.$i.frame1.point$i [expr $i+1] 6 0 0 center c
      draw_label $table.$i.frame2.nominal$i $term(lownominal$i) 9 0 0 center c
      
      draw_entry $table.$i.frame3.actual$i powerterm::term(lowactual$i) 9 $term(lowcolor$i) center
      
      bind_numeric $table.$i.frame3.actual$i 1 4 3 0 "::powerterm::request_pwt_cal low $i"
      
      draw_entry $table.$i.frame4.cal$i ::powerterm::term(lowcal$i) 7 0 center
      
      $table.$i.frame4.cal$i config -state disabled     
    }       
  
    # SET FOCUS
    if { [winfo viewable $table.0.frame3.actual0] } \
    {
      after idle  "safe_focus_force $table.0.frame3.actual0"
    } \
    else \
    {
      bind $table.0.frame3.actual0 <Visibility> \
                      "safe_focus_force $table.0.frame3.actual0; bind $table.0.frame3.actual0 <Visibility> { }"
    } 
  
  }
  
  
  #***************************************************************************
  #
  #       Name: start_powerterm
  #
  # Parameters: tile
  #
  # Description: calls cal_password routine
  #
  #***************************************************************************
  proc start_powerterm { tile } \
  {
    ::Calibration_Screen::cal_password  ::powerterm::continue_powerterm $tile
    
    return
  }
  
  
  #***************************************************************************
  #
  #       Name: continue_powerterm
  #
  # Parameters: tile
  #
  # Description: starts power term calibration screen
  #
  #***************************************************************************
  proc continue_powerterm { } \
  {
    variable term
  
    set tile $::PowerTerm_Screen::tile_path
    
    # TURN AUDIO OUT ROUTING OFF
    ::BBAudio::default_audio_routes
  

    set term(current) 0
  
    # DRAW FIRST SCREEN
    ::powerterm::build_low_power  $tile
  
    # ASSIGN SOFTKEYS
    ::View::AssignSoftkey $tile  2  "CAL LOW POWER"  "::powerterm::build_low_power  $tile"
    ::View::AssignSoftkey $tile  3  "CAL HIGH POWER" "::powerterm::build_high_power  $tile"
    ::View::AssignSoftkey $tile  4  "CAL FREQ"       "::powerterm::build_freq_power  $tile"
    ::View::AssignSoftkey $tile  5  "SAVE"            "::powerterm::save_pwt_cals"

    # SOMTHING BROKE, THINK CAN NO LONGER HAVE 2 PLOTS ON SCREEN, 
    # SMALL PLOT ALREADY, ITS GOOD ENOUGH FOR NOW
    #::View::AssignSoftkey $tile  6  "BIG PLOT"   "::powerterm::pwt_big_plot"
    ::View::AssignSoftkey $tile  6  "" ""
    ::View::AssignSoftkey $tile  7  "ZERO METER" "Power zero"
    

    ## SOMTHING BROKE, THINK CAN NO LONGER HAVE 2 PLOTS ON SCREEN, 
    ## SMALL PLOT ALREADY, ITS GOOD ENOUGH FOR NOW
    ##assign_softkey 6 "BIG PLOT" "pwt_big_plot"
    #assign_softkey 6 "" ""
    #assign_softkey 7 "ZERO METER" "Power zero"
    
    after idle ::powerterm::setup_hardware
    
    return
  }
  

  proc give { index data } \
  {
    receive_pwt_cal 2 1 $index $data 1 0
  }
  
  proc setup_hardware { } \
  {
    # RETRIEVE CAL VALUES FROM EEPROM
    puts "Reading cals"
    if [ catch {Power debug eeprom } err ] \
    {
       puts "Error reading power term cals: $err"
    }
    request_pwt_cals
  }
  
};  # End of powerterm namespace



#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C) 2001      DESIGN ENGINEERING        #
# --------------------------------------------------------------------------#
# FILE        : 2110.powerterm.tcl                                          #
# PRODUCT     : FITS                                                        #
# AUTHOR      : Bruce Butterfield                                           #
# DESCRIPTION : This namespace is a dummy "screen" that sources and holds   #
#             : the submenu namespace list for the "Systems" droplist       #
#############################################################################
#                                                                           #
# Required Variables:                                                       #
#    LongName		        Name of screen                              #
#    ShortName          Name of screen for menus                            #
#    RecomendedTiles    List of tiles available for this screen. It is not  #
#                       necessary to put a tile on this list that is on the #
#                       RequiredTiles list                                  #
#    NotAllowedTiles    List of tiles NEVER on this screen                  #
#    RequiredTiles      List of tiles ALWAYS on this screen                 #
#    ActiveTiles        List of tiles currently displayed on this screen    #
#    MenuFlags          0 = Active Screen                                   #
#                       1 = Currently functioning as a submenu parent       #
#                                                                           #
# Required Variables for submenu "Screens":                                 #
#    SubmenuScreens     List of all tiles (or submenus) for droplist        #
#    SubDirPath         Path to subdirectory holding the screens for the    #
#                       submenu of this "screen"                            #
#                                                                           #
#############################################################################
#                                                                           #
# Dummy Functions:                                                          #
#    Setup              Called by the screen manager to create the screen.  #
#    Map                Called when screen is mapped on window              #
#    Unmap              Called when screen is removed from display          #
#    Destroy            Called when screen is destroyed                     #
#                                                                           #
#############################################################################

namespace eval PowerTerm_Screen \
{
  variable LongName
  variable ShortName
  variable RecommendedTiles
  variable NotAllowedTiles
  variable RequiredTiles
  variable ActiveTiles
  variable MenuFlags
  variable SubmenuScreens
  variable SubDirPath

  set LongName PowerTerm_Screen
  set ShortName "Power Term"
  set RequiredTiles { }
  set NotAllowedTiles { }
  set RecommendedTiles { }
  set ActiveTiles { }
  set MenuFlags 0
  set SubmenuScreens { }
  set SubDirPath { }

  variable tile_path      ""
  variable tile_size      "full"
  variable tile_class     "POWERTERM"

  #****************************************************************************
  # Name:        Setup
  # Parameters:  parent		parent of child window
  # Description: Create the screen
  # Returns:     Tile path to be packed into the parent frame
  #****************************************************************************
  proc Setup { parent } \
  {
    puts "PowerTerm_Screen::Setup"
    puts "parent:  $parent"
         
    set top_level [winfo toplevel $parent]
   
    set tile_path [name_child $top_level powerterm_cal]
    
    set tile_class  $::PowerTerm_Screen::tile_class

    if { ![winfo exists $tile_path] } \
    {
      set tile_path  [::Screens::create_tile  $tile_path  powerterm  $tile_class]
    }
    
    set ::PowerTerm_Screen::tile_path  $tile_path
    
    
    # Call start procedure here
    ::powerterm::start_powerterm $tile_path
    
    return $tile_path
  }

  
  #****************************************************************************
  # Name:        Map
  # Parameters:  parent		parent of child window
  # Description: Dummy function
  #****************************************************************************
  proc Map { name } \
  { 
    puts "PowerTerm_Screen::Map"
    
    # BODGE save the state of the power term
    puts "Saving power term state"
    variable term
    if ![info exists term(old_state)] {set term(old_state) ""}
    if {$term(old_state) == "" } \
    {
       set term(old_state) [Power::getSaveState]
    }
     
    return
  }


  #****************************************************************************
  # Name:        Unmap
  # Parameters:  parent		parent of child window
  # Description: Dummy function
  #****************************************************************************
  proc Unmap { name } \
  { 
    puts "PowerTerm_Screen::Unmap"
    # BODGE restore the power term state
    variable term
    
    if ![info exists term(old_state)] {set term(old_state) ""}
    if {$term(old_state) != "" } \
    {
       eval $term(old_state)
       set term(old_state) ""
    }
    
    
    return
  }

 
  #****************************************************************************
  # Name:        Destroy
  # Parameters:  parent		parent of child window
  # Description: Dummy function
  #****************************************************************************
  proc Destroy { name } \
  { 
    puts "PowerTerm_Screen::Destroy"
     
    return
  }


  #****************************************************************************
  # Functions executed when the namespace is sourced
  #****************************************************************************
  ::View::RegisterScreen PowerTerm_Screen	   ;# Add this on the menu


};  # End of the PowerTerm_Screen name space


