#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C)1998,99      DESIGN ENGINEERING      #
# --------------------------------------------------------------------------#
# FILE        : 2120.rfgencal.tcl                                           #
# PRODUCT     : FITS                                                        #
# AUTHOR      : Steve Layton                                                #
# DESCRIPTION : Manual RF Generator Calibration Screen                      #
#                                                                           #
#############################################################################


namespace eval rfgencal \
{

   # for compatibility with older software....
   namespace export gencal_send_vlevel \
                    rfgencal_figure_level start_rfgencal \
                    set_rfgencal_reference rfgencal_save
   
   variable line_height         22

   variable Old_state {}


  #***************************************************************************
  #
  #       Name: reset_notsaved_rfgencal
  #
  # Parameters: 
  #
  # Description: sets all rfgencal notsaved variables to zero
  #
  #***************************************************************************
  proc reset_notsaved_rfgencal { } \
  {
    global GENCAL
  
    set GENCAL(notsaved_level) 0
    set GENCAL(notsaved_tr) 0
    set GENCAL(notsaved_gen) 0
  }

  
  #***************************************************************************
  #
  #       Name: exit_mess_rfgencal
  #
  # Parameters: 
  #
  # Description: gives "are you sure message" if needed
  #
  #***************************************************************************
  proc exit_mess_rfgencal { } \
  {
    global GENCAL
    
    if { $GENCAL(notsaved_level) } \
    {
      message_box .gencalnotsaved CAUTION
     
      .gencalnotsaved.text config -text [subst [option get .gencalnotsaved leveler {}]]
     
      return 1
    }
  
    if { $GENCAL(notsaved_tr) } \
    {
      message_box .gencalnotsaved CAUTION
    
      .gencalnotsaved.text config -text [subst [option get .gencalnotsaved tr {}]]
     
      return 1
    }
  
    if { $GENCAL(notsaved_gen)} \
    {
      message_box .gencalnotsaved CAUTION
    
      .gencalnotsaved.text config -text [subst [option get .gencalnotsaved gen {}]]
      
      return 1
    }
    
    # DONT EXIT MODE ROUTINE, NO MESSAGE
    return 0
  }
  
  
  #***************************************************************************
  #
  #       Name: gencal_nextfocus
  #
  # Parameters: type {level tr0 tr10 tr20 gen0 gen10 gen20}
  #           : index, widget in table index
  #
  # Description: figures next focus widget, sets focus
  #
  #***************************************************************************
  proc gencal_nextfocus { tile type index } \
  {
    set left $tile.table.frame1
    set right $tile.table.frame2
  
    incr index
  
    if {$type == "level" && $index < 2} \
    {
      set consider $left.$index.frame1.1

    } elseif {$index < 8} {
      set consider $left.$index.frame2.2

    } else {
      set consider $right.$index.frame2.2
    }
 

    if {[winfo exists $consider] && [tkFocusOK $consider]} \
    {
      after idle  "safe_focus_force $consider"
  
    } elseif {$type == "level"} {
      after idle  "safe_focus_force $left.2.frame2.2"
      
    } else {
      after idle  "safe_focus_force $left.0.frame2.2"
    }
  
  }
  
  
  #***************************************************************************
  #
  #       Name: gencal_get_date
  #
  # Parameters: 
  #
  # Description: returns current date
  #
  #***************************************************************************
  proc gencal_get_date { } \
  {
    set day   [string trimleft [clock format [clock seconds] -format %d] "0"]
    set month [string trimleft [clock format [clock seconds] -format %m] "0"]
    set year  [string trimleft [clock format [clock seconds] -format %Y] "0"]
  
    return "$month/$day/$year"
  }
  
  
  #***************************************************************************
  #
  #       Name: gencal_get_time
  #
  # Parameters: 
  #
  # Description: returns current time
  #
  #***************************************************************************
  proc gencal_get_time { } \
  {
    set hour  [string trimleft [clock format [clock seconds] -format %H] "0"]
    set minute  [string trimleft [clock format [clock seconds] -format %M] "0"]       
  
    if {$minute < 10} \
    {
      set minute 0$minute
    }
  
    return "$hour:$minute"
  }
  
  
  #***************************************************************************
  #       Name: gencal_timestsmp
  # Parameters: type, {level tr gen} 
  #Description: gets date, time and sets GENCAL values
  #***************************************************************************
  proc gencal_timestamp { type } \
  {
    global GENCAL
    
    set GENCAL($type.caltime) [::rfgencal::gencal_get_time]
    set GENCAL($type.caldate) [::rfgencal::gencal_get_date]
  } 
  
  #***************************************************************************
  #       Name: gencal_send_vlevel
  # Parameters: index, 
  # Description: Sets up the generator level from the table
  #***************************************************************************
  proc gencal_send_vlevel { index } \
  {
    global GENCAL
  
    set alc $GENCAL(level,alc,$index)

    Debug::Puts RFGENCAL 2 "gencal_send_vlevel index $index, abs $GENCAL(level,abs,$index), alc $alc"
    if {$alc >= 0 && $alc < 4096} {
      Generator debug rawrflev $alc
    } 
  }
  
  #***************************************************************************
  #       Name: gencal_alc_padetect_capture
  # Parameters: index, index of entry
  #Description: updates padetect value
  #***************************************************************************
  proc gencal_alc_padetect_capture { index } \
  {
    global GENCAL
  
    set GENCAL(level,padetect,$index) $Generator::State(alc)
    set GENCAL(level,status,$index)   CAL
     
    set GENCAL(notsaved_level) 1
  }
  
  #***************************************************************************
  #
  #       Name: rfgencal_figure_level
  #
  # Parameters: tile
  #
  # Description: zeros leveler dac values, figures target output values
  #            : called when 0 or 4095 values entered
  #
  #***************************************************************************
  proc rfgencal_figure_level { tile } \
  {
    global GENCAL
    global RFGEN_ENGINE
    
    
    set top $GENCAL(level,abs,0)
    set bottom $GENCAL(level,abs,[expr $RFGEN_ENGINE(AlcMaxIndex)-1]) 
    
    if {$top == "-" || $bottom == "-"} \
    {
      return
    }
    
    set GENCAL(level,abs,1) [format_numeric [expr $top-.25] 2]
    set GENCAL(level,abs,2) [format_numeric [expr $top-.5] 1]
    set GENCAL(level,abs,3) [format_numeric [expr $top-1] 1]
    set GENCAL(level,abs,4) [format_numeric [expr $top-2] 1]
    set GENCAL(level,abs,5) [format_numeric [expr $top-3] 1]
    set GENCAL(level,abs,6) [format_numeric [expr $top-4] 1]
    set GENCAL(level,abs,7) [format_numeric [expr $top-5] 1]
    
    set GENCAL(level,abs,8) [format_numeric [expr ($top-($top-$bottom)/2)+5] 1]
    set GENCAL(level,abs,9) [format_numeric [expr $top-($top-$bottom)/2] 1]
    set GENCAL(level,abs,10) [format_numeric [expr ($top-($top-$bottom)/2)-5] 1]
  
    set GENCAL(level,abs,11) [format_numeric [expr $bottom+5] 1]
    set GENCAL(level,abs,12) [format_numeric [expr $bottom+3] 1]
    set GENCAL(level,abs,13) [format_numeric [expr $bottom+1] 1]
    set GENCAL(level,abs,14) [format_numeric [expr $bottom+.5] 1]
  
  
    set left $tile.table.frame1
    set right $tile.table.frame2
      
    # ZERO REMAINDER VALUES
    for {set i 1} {$i < [expr $RFGEN_ENGINE(AlcMaxIndex)-1]} {incr i} \
    {
  #   set GENCAL(level,alc,$i) 0
      set GENCAL(level,status,$i) UNCAL
    }
    
    # CHANGE TO UNCAL COLOR FOR ALL VALUES ON SCREEN
    # LEFT SIDE
  # $left.0.frame1.1 config -fg $::color(messagewarning)
  # $left.1.frame1.1 config -fg $::color(messagewarning)
      for {set i 2} {$i < ($GENCAL(pagemax)/2)} {incr i} {
      if {[$left.$i.frame2.2 cget -textvariable] != "dummyrfgencal"} {
      $left.$i.frame2.2 config -fg $::color(messagewarning)
    }
  }
  
    # RIGHT SIDE
      for {set i [expr $GENCAL(pagemax)/2]} {$i < $GENCAL(pagemax)} {incr i} {
  
      if {[$right.$i.frame1.1 cget -textvariable] != "dummyrfgencal"} {
        $right.$i.frame2.2 config -fg $::color(messagewarning)
      }
    }
  
  }
  
  #***************************************************************************
  #
  #       Name: set_rfgencal_reference
  #
  # Parameters: tile
  #
  # Description: changes reference frequency upon ref freq selection
  #
  #***************************************************************************
  proc set_rfgencal_reference { tile } \
  {
    global RFGEN_ENGINE
    global GENCAL
    
    
    Generator debug rawfreq $GENCAL(AlcFreq)
    
    # RESET ALL LEVELER CAL VALUES TO UNCAL
    for {set i 0} {$i < $RFGEN_ENGINE(AlcMaxIndex)} {incr i} \
    {
      set GENCAL(level,status,$i) UNCAL 
    }
  
    # RECOLOR CURRENT SCREEN
      set left $tile.table.frame1
      set right $tile.table.frame2
  
    # LEFT TABLE
      set GENCAL(level,abs,0) "-"
      $left.0.frame1.1 config -fg $::color(messagewarning)
      set GENCAL(level,abs,[expr $RFGEN_ENGINE(AlcMaxIndex)-1]) "-"
      $left.1.frame1.1 config -fg $::color(messagewarning)
      
      for {set i 2} {$i < [expr $GENCAL(pagemax)/2]} {incr i} {
      if {[$left.$i.frame2.2 cget -textvariable] != "dummyrfgencal"} {
        $left.$i.frame2.2 config -fg $::color(messagewarning)
      }
    }
  
    # RIGHT TABLE
      for {set i [expr $GENCAL(pagemax)/2]} {$i < $GENCAL(pagemax)} {incr i} {
      if {[$right.$i.frame2.2 cget -textvariable] != "dummyrfgencal"} {
      $right.$i.frame2.2 config -fg $::color(messagewarning)
      } 
    }
  
    # ZERO alc VALUES
      for {set i 1} {$i < [expr $RFGEN_ENGINE(AlcMaxIndex)-1]} {incr i} {
      set GENCAL(level,alc,$i) "-"
    }
  
  }
  
  
  #***************************************************************************
  #       Name: config_rfgencal_index
  # Parameters: index, 
  #Description: sends screen values to engine upon cursor movement
  #***************************************************************************
  proc config_rfgencal_index {index} \
  {
    global GENCAL
    
    Debug::Puts RFGENCAL 3 "config_rfgencal_index $index"
  
    if {$GENCAL(screen) == "level"} {
      gencal_send_vlevel $index
    } elseif {$GENCAL(screen) == "gen0"} {
      Generator debug rawfreq $GENCAL(freq,GEN,freq,$index)
      Generator debug rawdb 0.0
    } else {
      Generator debug rawfreq $GENCAL(freq,TR,freq,$index)
      Generator debug rawdb 0.0
    }
  } 
  
  #***************************************************************************
  #       Name: rfgencal_save
  # Parameters: kind, type of data to save
  #Description: sets normalize values, saves gencal entries
  #       : sends cals to engine
  #***************************************************************************
  proc rfgencal_save {kind} \
  {
    global RFGEN_ENGINE
    global GENCAL
  
    if {$kind == "ALC"} {
      gencal_timestamp level
  
      set RFGEN_ENGINE(Normalize.Genalc)        -10.00
      gencal_alc_save_cal rfgen_alc
      
      # save, with temp compensation factored in.
      gencal_set_engine_level $::HW_control::State(if_temp)
      
      set GENCAL(notsaved_level) 0
    
    } elseif {$kind == "TR"} {
      gencal_timestamp TR
    
      set RFGEN_ENGINE(Normalize.Genfreq.dB0)   -30.00
      set RFGEN_ENGINE(Normalize.Genfreq.dB10)  -40.00
      set RFGEN_ENGINE(Normalize.Genfreq.dB20)  -50.00
      gencal_freq_save_cal rfgen_freq_tr TR
    
      gencal_set_engine_freq TR
      
      set GENCAL(notsaved_tr) 0
          
    } elseif {$kind == "GEN"} {
      gencal_timestamp GEN
  
      set RFGEN_ENGINE(Normalize.Genfreq.dB0)    10.00
      set RFGEN_ENGINE(Normalize.Genfreq.dB10)    0.00
      set RFGEN_ENGINE(Normalize.Genfreq.dB20)  -10.00
      gencal_freq_save_cal rfgen_freq_gen GEN
  
      gencal_set_engine_freq GEN
      
      set GENCAL(notsaved_gen) 0
    }
  
  } 
  
  #***************************************************************************
  #       Name: rfgencal_plot
  # Parameters: 
  #Description: displays plot for rfgencal
  #***************************************************************************
  proc rfgencal_plot { widget } \
  {
    global RFGEN_ENGINE
    global GENCAL
    
    set top_level [winfo toplevel $widget]
    set tile_path [name_child $top_level bigplot]
    
    # VERIFY CORRECT MODE IS STILL VALID
   
    set plot_text ""
    set plot_data ""
  

    # DETERMINE TITLE
    if {$GENCAL(screen) == "level"} {
      set plot_text "LEVEL RF GEN CALIBRATION PLOT"
  
      # ASSEMBLE PLOT DATA
      for {set i 1} {$i < [expr $RFGEN_ENGINE(AlcMaxIndex)-1]} {incr i} {
        lappend plot_data $GENCAL(level,abs,$i) $GENCAL(level,alc,$i) 
      }
  
    } else {
      set plot_text "FREQ RF GEN CALIBRATION PLOT ("
      append plot_text [string toupper [string trimright $GENCAL(screen) "012"]] " " \
                       [string trimleft $GENCAL(screen) "gentr"] " dB)"
  
      # ASSEMBLE PLOT DATA
      for {set i 0} {$i < $RFGEN_ENGINE(FreqMaxIndex)} {incr i} {
        lappend plot_data $GENCAL(freq,[string toupper [string trimright $GENCAL(screen) "012"]],freq,$i) \
           $GENCAL(freq,[string toupper [string trimright $GENCAL(screen) "012"]],dB[string \
           trimleft $GENCAL(screen) "gentr"],abs,$i)
      }
  
    }
  
    
      # DRAW PLOT SURROUNDING WIDGETS 
      if {![draw_toplevel $tile_path 400 430 $plot_text]} {
        return
      }
      draw_frame $tile_path.top 370 430
      draw_frame $tile_path.low 30 430 175 80 175
      draw_button $tile_path.low.frame2.exit "RETURN" "destroy $tile_path" 0 0 $::color(newtopbg) center c
  
      # DRAW PLOT
      if {$GENCAL(screen) == "level"} \
      {
        lineplot $tile_path.top.plot -width 430 -height 370 -autosizex true -autosizey false \
                 -maxy 4095 -unitx dBm -smooth false -data $plot_data -smooth true -splinesteps 200

      } else {
      lineplot $tile_path.top.plot -width 430 -height 370 -autosizex false -autosizey true \
               -maxx 2700 -unitx MHz -unity dBm -smooth false -data $plot_data -smooth true -splinesteps 200
    }
  
  
    # ESTABLISH FOCUS
    focus_toplevel $tile_path.low.frame2.exit
  }
  
  #***************************************************************************
  #
  #       Name: build_rfgencal_screen
  #
  # Parameters: type, type of screen {tr0 tr10 tr20 gen0 gen10 gen20}
  #           : start, starting index
  #
  # Description: builds rf gen cal screen
  #
  #***************************************************************************
  proc build_rfgencal_screen { tile  type  start } \
  {
    global RFGEN_ENGINE
    global GENCAL

    variable line_height
    
    Debug::Puts RFGENCAL 3 "build_rfgencal_screen $tile $type $start" 
    
    
    if {$GENCAL(screen) == "level"} \
    {
      build_rfgencal_frame  $tile
    }

    # QUERY DESIRED Vlevel VALUE
    set GENCAL(querytype)    1
    set GENCAL(queryvalue)   0
    set GENCAL(querycommand) "Generator debug rawrflev "
    
    gencal_spline_info 1 0dB 0
  
    # SET CURRENT SCREEN TYPE
    set GENCAL(screen) $type
  
    set basic_type [string trimright $type "012"]
  
      
    # SET TITLE STATUS, SET ATTENUATION, SET PORT
    switch -exact -- $type \
    {
      tr0   {
             Debug::Puts RFGENCAL 3 "RF GENERATOR CAL - TR Screen, 0 dB Atten" 

             Generator debug rawrfoutput TR
             update idle
             update idle
             Generator debug rawatten 0
             Generator debug rawdb 0.0
            }

      tr10  {
             Debug::Puts RFGENCAL 3 "RF GENERATOR CAL - TR Screen, 10 dB Atten"
             Generator debug rawrfoutput TR
             update idle
             update idle
             update idle
             Generator debug rawatten 10
             Generator debug rawdb 0.0
            }

      tr20  {
             Debug::Puts RFGENCAL 3 "RF GENERATOR CAL - TR Screen, 20 dB Atten"
             Generator debug rawrfoutput TR
             update idle
             update idle
             Generator debug rawatten 20
             Generator debug rawdb 0.0
            }

      gen0  {Debug::Puts RFGENCAL 3 "RF GENERATOR CAL - Gen Screen, 0 dB Atten"
             Generator debug rawrfoutput GEN
             update idle
             update idle
             Generator debug rawatten 0
             Generator debug rawdb 0.0
            }

      gen10 {
             Debug::Puts RFGENCAL 3 "RF GENERATOR CAL - Gen Screen, 10 dB Atten"
             Generator debug rawrfoutput GEN
             update idle
             update idle
             Generator debug rawatten 10
             Generator debug rawdb 0.0
             }

      gen20 {
             Debug::Puts RFGENCAL 3 "RF GENERATOR CAL - Gen Screen, 20 dB Atten"
             Generator debug rawrfoutput GEN
             update idle
             update idle
             Generator debug rawatten 20
             Generator debug rawdb 0.0
            }
    }
  
    # CREATE/OMIT TOP BUTTONS
    if { $basic_type == "tr" } \
    {
      $tile.top.frame2.time config -textvariable GENCAL(TR.caltime)
      $tile.top.frame3.date config -textvariable GENCAL(TR.caldate)
    } \
    else \
    {
      $tile.top.frame2.time config -textvariable GENCAL(GEN.caltime)
      $tile.top.frame3.date config -textvariable GENCAL(GEN.caldate)
    }
    
    destroy $tile.top.frame5.ref

    $tile.top.frame4.label config -text ""
    $tile.top.frame6.label config -text ""

    if { $basic_type == "tr" } \
    {  
      $tile.top.frame8.save config -text "SAVE FREQ" -command "::rfgencal::rfgencal_save TR"
    } \
    else \
    {
      $tile.top.frame8.save config -text "SAVE FREQ" -command "::rfgencal::rfgencal_save GEN"
    }
  
  
    # ASSIGN SOFTKEYS
    # PREV AND NEXT KEYS
    if {$start > 0} \
    {
      if {$start < $GENCAL(pagemax)} \
      {
        ::View::AssignSoftkey $tile 2 "PREV VALUES" "::rfgencal::build_rfgencal_screen $tile $type 0"
        
      } \
      else \
      {
        ::View::AssignSoftkey $tile 2 "PREV VALUES" \
                        "::rfgencal::build_rfgencal_screen $tile $type [expr $start-$GENCAL(pagemax)]"
        
      }
    }
    
    if {[expr $start+$GENCAL(pagemax)] < $RFGEN_ENGINE(FreqMaxIndex)} \
    {
      ::View::AssignSoftkey $tile 3 "NEXT VALUES" \
                         "::rfgencal::build_rfgencal_screen $tile $type [expr $start+$GENCAL(pagemax)]"  
      
    }
  
    # OTHER MODE KEYS
    ::View::AssignSoftkey $tile 4 "LEVELER SCREEN" "::rfgencal::build_rfgencal_level $tile 0"
    
    
    if {[string first "tr" $type] == -1} \
    {
      ::View::AssignSoftkey $tile 5 "TR SCREEN" "::rfgencal::build_rfgencal_screen $tile tr0 0"
      
    } \
    else \
    {
      ::View::AssignSoftkey $tile 5 "GEN SCREEN" "::rfgencal::build_rfgencal_screen $tile gen0 0"
      
    }
    
    # ATTEN KEYS
    if {[string first "20" $type] != -1} \
    {
      set atten 20
      
      ::View::AssignSoftkey $tile 6 "[string toupper $basic_type] SCRN 0dB Attn" \
                       "::rfgencal::build_rfgencal_screen $tile [string trimright $type "012"]0 0"
      

      ::View::AssignSoftkey $tile 7 "[string toupper $basic_type] SCRN 10dB Attn" \
                      "::rfgencal::build_rfgencal_screen $tile [string trimright $type "012"]10 0"    
      
  
    } \
    elseif {[string first "10" $type] != -1} \
    {
      set atten 10
    
      ::View::AssignSoftkey $tile 6 "[string toupper $basic_type] SCRN 0dB Attn" \
                       "::rfgencal::build_rfgencal_screen $tile [string trimright $type "012"]0 0"
      
                       
      ::View::AssignSoftkey $tile 7 "[string toupper $basic_type] SCRN 20dB Attn" \
                      "::rfgencal::build_rfgencal_screen $tile [string trimright $type "012"]20 0"    
      
  
    } \
    else \
    {
      set atten 0
    
      ::View::AssignSoftkey $tile 6 "[string toupper $basic_type] SCRN 10dB Attn" \
                       "::rfgencal::build_rfgencal_screen $tile [string trimright $type "012"]10 0"
      
                       
      ::View::AssignSoftkey $tile 7 "[string toupper $basic_type] SCRN 20dB Attn" \
                       "::rfgencal::build_rfgencal_screen $tile [string trimright $type "012"]20 0"    
      
    }
    
    if { $basic_type == "tr" } \
    {
      set port TR
    } \
    else \
    {
      set port GEN
    } 
    
    set left $tile.table.frame1
    set right $tile.table.frame2
  
    # FILL HEADER
    $left.head.frame1.1 config -text FREQ

    $right.head.frame1.1 config -text FREQ

    $left.head.frame2.2 config -text "OUTPUT (dBm)"

    $right.head.frame2.2 config -text "OUTPUT (dBm)"
    
    set dummyrfgencal ""
    
    
    # FILL LEFT TABLE
    for { set i 0 } { $i < [expr $GENCAL(pagemax)/2] } { incr i } \
    {
      if { [expr $i+$start] < $RFGEN_ENGINE(FreqMaxIndex) } \
      {
        $left.$i.frame1.1 config -textvariable GENCAL(freq,$port,freq,[expr $start+$i]) \
                          -fg $::color(screenfg) -takefocus 0
        $left.$i.frame2.2 config -textvariable GENCAL(freq,$port,dB$atten,abs,[expr $start+$i]) -takefocus 1
        
        if { $GENCAL(freq,$port,dB$atten,status,[expr $start+$i]) == "CAL" } \
        {
          $left.$i.frame2.2 config -fg $::color(screenfg)
        } \
        else \
        {
          $left.$i.frame2.2 config -fg $::color(messagewarning)
        }
        
        bind_numeric $left.$i.frame2.2 2 2 20 -70 "$left.$i.frame2.2 config -fg $::color(screenfg); \
                                                   gencal_freq_capture [expr $i+$start];          \
                                                   ::rfgencal::gencal_nextfocus $tile $type $i"
                                                   
        bind $left.$i.frame2.2 <FocusIn> "::rfgencal::config_rfgencal_index [expr $start+$i]"
        
        $left.$i.frame3.3 config -textvariable dummyrfgencal
                   
      } \
      else \
      {
        # STOP IF OUT OF DATA
        $left.$i.frame1.1 config -textvariable dummyrfgencal
        $left.$i.frame2.2 config -textvariable dummyrfgencal -fg $::color(screenfg) -takefocus 0
        $left.$i.frame3.3 config -textvariable dummyrfgencal
      }
    }
  
    # FILL RIGHT TABLE
    for { set i [expr $GENCAL(pagemax)/2] } { $i < $GENCAL(pagemax) } { incr i } \
    {
      if { [expr $i+$start] < $RFGEN_ENGINE(FreqMaxIndex) } \
      {
        $right.$i.frame1.1 config -textvariable GENCAL(freq,$port,freq,[expr $start+$i]) -takefocus 0
        $right.$i.frame2.2 config -textvariable GENCAL(freq,$port,dB$atten,abs,[expr $start+$i]) -takefocus 1
      
        if { $GENCAL(freq,$port,dB$atten,status,[expr $start+$i]) == "CAL" } \
        {
          $right.$i.frame2.2 config -fg $::color(screenfg)
        } \
        else \
        {
          $right.$i.frame2.2 config -fg $::color(messagewarning)
        }
        
        bind_numeric $right.$i.frame2.2 2 2 20 -70 "$right.$i.frame2.2 config -fg $::color(screenfg); \
                                                    gencal_freq_capture [expr $i+$start];           \
                                                    ::rfgencal::gencal_nextfocus $tile $type $i"
                                                    
        bind $right.$i.frame2.2 <FocusIn> "::rfgencal::config_rfgencal_index [expr $start+$i]"
        
        $right.$i.frame3.3 config -textvariable dummyrfgencal
  
      } \
      else \
      {
        # BLANK IF OUT OF DATA
        $right.$i.frame1.1 config -textvariable dummyrfgencal
        $right.$i.frame2.2 config -textvariable dummyrfgencal -fg $::color(screenfg) -takefocus 0
        $right.$i.frame3.3 config -textvariable dummyrfgencal
      }
    }
  
  
    # BIND BOTTOM LEFT TO GO TO TOP RIGHT WHEN DOWN PRESSED
    bind $left.7.frame2.2 <<cursor_down>> "focus $right.8.frame2.2;break"
    bind $right.8.frame2.2 <<cursor_up>> "focus $left.7.frame2.2;break"
  

    set object $left.0.frame2.2
  
    # IF FOCUS DOESNT CHANGE ON NEW SCREEN, (LAST FOCUS WAS INDEX 0)
    # DO config_rfgencal_index 
    if { [focus] == $object } \
    {
      config_rfgencal_index  $start
    }
    
    # SET FOCUS
    if { [winfo viewable $object] } \
    {
      after idle  "safe_focus_force $object"
    } \
    else \
    {
      bind $object <Visibility> "safe_focus_force $object;bind $object <Visibility> { }"
    }

  }
  
  
  #***************************************************************************
  #
  #       Name: setup_level
  #
  # Parameters: tile
  #           : start, starting index
  #
  # Description: builds rf gen cal screen
  #
  #***************************************************************************
  proc setup_level { } \
  {
    global GENCAL
    Debug::Puts RFGENCAL 2 "build_rfgencal_level - set hardware"
    Generator debug rawrfoutput GEN
    # let this take effect first.
    update idle
    update idle
    Generator debug rawatten 20
    Generator debug rawfreq  $GENCAL(AlcFreq)
  }
  #***************************************************************************
  #
  #       Name: build_rfgencal_level
  #
  # Parameters: tile
  #           : start, starting index
  #
  # Description: builds rf gen cal screen
  #
  #***************************************************************************
  proc build_rfgencal_level { tile start } \
  {
    global GENCAL
    global RFGEN_ENGINE

    variable line_height
    
    Debug::Puts RFGENCAL 2 "build_rfgencal_level $tile $start"
    
    build_rfgencal_frame  $tile
    
    # SET CURRENT SCREEN TYPE
    set GENCAL(screen) level
     
    setup_level
    
    # CREATE/OMIT TOP BUTTONS
    $tile.top.frame2.time config -textvariable GENCAL(level.caltime)
    $tile.top.frame3.date config -textvariable GENCAL(level.caldate)

    destroy $tile.top.frame5.ref
    
    draw_popdown $tile.top.frame5.ref GENCAL(AlcFreq) "::rfgencal::set_rfgencal_reference $tile" $GENCAL(AlcFreqs)
    
    $tile.top.frame5.ref config -justify right -anchor e
    $tile.top.frame4.label config -text "REF FREQ"
    $tile.top.frame6.label config -text "MHz"  
  
    $tile.top.frame8.save config -text "SAVE ALC" -command "::rfgencal::rfgencal_save ALC"
    
    # ASSIGN SOFTKEYS
    # PREV AND NEXT KEYS
    if { $start > 0 } \
    {
      if { $start < $GENCAL(pagemax) } \
      {
        ::View::AssignSoftkey $tile 2 "PREV VALUES" "::rfgencal::build_rfgencal_level 0"
        
      } \
      else \
      {
        ::View::AssignSoftkey $tile 2 "PREV VALUES" \
                        "::rfgencal::build_rfgencal_level [expr $start-$GENCAL(pagemax)]"
        
      }
    }
    
    if { [expr $start+$GENCAL(pagemax)] < $RFGEN_ENGINE(AlcMaxIndex) } \
    {
      ::View::AssignSoftkey $tile 3 "NEXT VALUES" \
                        "::rfgencal::build_rfgencal_level [expr $start+$GENCAL(pagemax)]" 
      
    }

    # OTHER MODE KEYS
    ::View::AssignSoftkey $tile 4 "TR SCREEN" "::rfgencal::build_rfgencal_screen $tile tr0 0"
    
    
    ::View::AssignSoftkey $tile 5 "GEN SCREEN" "::rfgencal::build_rfgencal_screen $tile gen0 0"   
    
    
    
    set left $tile.table.frame1
    set right $tile.table.frame2
  
    # FILL HEADER
    $left.head.frame1.1 config -text "OUTPUT (dBm)"
    $right.head.frame1.1 config -text "OUTPUT (dBm)"
    $left.head.frame2.2 config -text "DAC VALUE"
    $right.head.frame2.2 config -text "DAC VALUE"
        
    set dummyrfgencal ""
      
    # FILL LEFT TABLE
    
    # FIRST SCREEN
    set loop_start 0
    
    if { $start == 0 } \
    {
      # INDEX 0 FOR FIRST SCREEN
      set GENCAL(level,alc,0) 4095
    
      $left.0.frame1.1 config -textvariable GENCAL(level,abs,0) -takefocus 1 -state normal
      $left.0.frame2.2 config -textvariable GENCAL(level,alc,0) -fg $::color(screenfg) -state disabled
    
      if { $GENCAL(level,status,0) == "CAL" } \
      {
        $left.0.frame1.1 config -fg $::color(screenfg)
      } \
      else \
      {
        $left.0.frame1.1 config -fg $::color(messagewarning)
      }
      
      bind_numeric $left.0.frame1.1 2 2 20 -60 "$left.0.frame1.1 config -fg $::color(screenfg); \
                                                ::rfgencal::rfgencal_figure_level $tile;      \
                                                ::rfgencal::gencal_alc_padetect_capture 0;    \
                                                ::rfgencal::gencal_nextfocus $tile level 0"
      
      bind $left.0.frame2.2 <FocusIn> {}
      bind $left.0.frame1.1 <FocusIn> "::rfgencal::config_rfgencal_index 0"
      
      $left.0.frame3.3 config -textvariable dummyrfgencal
  
      # INDEX 1 FOR FIRST SCREEN
      set GENCAL(level,alc,[expr $RFGEN_ENGINE(AlcMaxIndex)-1]) 0
      
      $left.1.frame1.1 config -textvariable GENCAL(level,abs,[expr $RFGEN_ENGINE(AlcMaxIndex)-1]) \
                              -takefocus 1 -state normal
                              
      $left.1.frame2.2 config -textvariable GENCAL(level,alc,[expr $RFGEN_ENGINE(AlcMaxIndex)-1]) \
                              -fg $::color(screenfg) -state disabled
      
      if { $GENCAL(level,status,[expr $RFGEN_ENGINE(AlcMaxIndex)-1]) == "CAL" } \
      {
        $left.1.frame1.1 config -fg $::color(screenfg)
      } \
      else \
      {
        $left.1.frame1.1 config -fg $::color(messagewarning)
      }
      
      bind_numeric $left.1.frame1.1 2 2 20 -60 "$left.1.frame1.1 config -fg $::color(screenfg); \
                                                ::rfgencal::rfgencal_figure_level $tile;      \
                                                ::rfgencal::gencal_alc_padetect_capture       \
                                                         [expr $RFGEN_ENGINE(AlcMaxIndex)-1]; \
                                                ::rfgencal::gencal_nextfocus $tile level 1"
      
      bind $left.1.frame2.2 <FocusIn> {}
      bind $left.1.frame1.1 <FocusIn> "::rfgencal::config_rfgencal_index [expr $RFGEN_ENGINE(AlcMaxIndex)-1]"
      bind $left.1.frame1.1 <<cursor_down>> "focus $left.2.frame2.2;break"
      
      $left.1.frame3.3 config -textvariable dummyrfgencal
      
      set loop_start 2
    }
    
  
    # FILL LEFT TABLE
    for { set i $loop_start } { $i < ($GENCAL(pagemax)/2) } { incr i } \
    {
      if { [expr $i+$start] < $RFGEN_ENGINE(AlcMaxIndex) } \
      {
        $left.$i.frame1.1 config -textvariable GENCAL(level,abs,[expr $start+$i-1]) -fg $::color(screenfg)
        $left.$i.frame2.2 config -textvariable GENCAL(level,alc,[expr $start+$i-1]) -takefocus 1
      
        if { $GENCAL(level,status,[expr $start+$i-1]) == "CAL" } \
        {
          $left.$i.frame2.2 config -fg $::color(screenfg)
        } \
        else \
        {
          $left.$i.frame2.2 config -fg $::color(messagewarning)
        }
        
        bind_numeric_spin $left.$i.frame2.2 4 0 4095 0 "$left.$i.frame2.2 config -fg $::color(screenfg);     \
                                                        ::rfgencal::gencal_send_vlevel [expr $start+$i-1]; \
                                                        ::rfgencal::gencal_alc_padetect_capture [expr $i-1]"
        
        bind $left.$i.frame2.2 <FocusIn> "::rfgencal::config_rfgencal_index [expr $start+$i-1]"
        
        $left.$i.frame3.3 config -textvariable dummyrfgencal
      } \
      else \
      {
        # STOP IF OUT OF DATA
        $left.$i.frame1.1 config -textvariable dummyrfgencal
        $left.$i.frame2.2 config -textvariable dummyrfgencal -fg $::color(screenfg) -takefocus 0
        $left.$i.frame3.3 config -textvariable dummyrfgencal
      }
    }
  
    # FILL RIGHT TABLE
    for { set i [expr $GENCAL(pagemax)/2] } { $i < $GENCAL(pagemax) } { incr i } \
    {
      if { [expr $i+$start] < [expr $RFGEN_ENGINE(AlcMaxIndex)] } \
      {
        $right.$i.frame1.1 config -textvariable GENCAL(level,abs,[expr $start+$i-1])
        $right.$i.frame2.2 config -textvariable GENCAL(level,alc,[expr $start+$i-1]) -takefocus 1
      
        if { $GENCAL(level,status,[expr $start+$i-1]) == "CAL" } \
        {
          $right.$i.frame2.2 config -fg $::color(screenfg)
        } \
        else \
        {
          $right.$i.frame2.2 config -fg $::color(messagewarning)
        }
        
        bind_numeric_spin $right.$i.frame2.2 4 0 4095 0 "$right.$i.frame2.2 config -fg $::color(screenfg);    \
                                                         ::rfgencal::gencal_send_vlevel [expr $start+$i-1]; \
                                                         ::rfgencal::gencal_alc_padetect_capture [expr $i-1]"
        
        bind $right.$i.frame2.2 <FocusIn> "::rfgencal::config_rfgencal_index [expr $start+$i-1]"
        
        $right.$i.frame3.3 config -textvariable dummyrfgencal
  
      } \
      else \
      {
        # BLANK IF OUT OF DATA
        $right.$i.frame1.1 config -textvariable dummyrfgencal
        $right.$i.frame2.2 config -textvariable dummyrfgencal -fg $::color(screenfg) -takefocus 0
        $right.$i.frame3.3 config -textvariable dummyrfgencal
      }
    }


    # BIND BOTTOM LEFT TO GO TO TOP RIGHT WHEN DOWN PRESSED
    bind $left.7.frame2.2 <<cursor_down>> "focus $right.8.frame2.2;break"
    bind $right.8.frame2.2 <<cursor_up>> "focus $left.7.frame2.2;break"
    
    # BIND 
    bind $left.2.frame2.2 <<cursor_up>> "focus $left.1.frame1.1;break"
  
    if { $start == 0 } \
    {
      set object $left.0.frame1.1
    } \
    else \
    {
      set object $left.0.frame2.2
    }

    # IF FOCUS DOESNT CHANGE ON NEW SCREEN, (LAST FOCUS WAS INDEX 0)
    # DO config_rfgencal_index 
    if { [focus] == $object } \
    {
      if { $start == 0 } \
      {
        config_rfgencal_index 0
      } \
      else \
      {
        config_rfgencal_index [expr $start-1]
      }
    }
    
    # SET FOCUS
    if { [winfo viewable $object] } \
    {
      after idle  "safe_focus_force $object"
    } \
    else \
    {
      bind $object <Visibility> "safe_focus_force $object;bind $object <Visibility> { }"
    }
  }
  
  
  #***************************************************************************
  #
  #       Name: build_rfgencal_frame
  #
  # Parameters: tile
  #
  # Description: builds rf gen cal outer screen
  #
  #***************************************************************************
  proc build_rfgencal_frame { tile } \
  {
    global GENCAL

    variable line_height
  
    
    draw_frame $tile.top [expr 2*$line_height] 555 85 45 85 95 45 45 60 95

    # TOP ROW CONTROLS
    draw_label $tile.top.frame1.label "LAST CAL" 8
    draw_label $tile.top.frame2.time "" 5
    draw_label $tile.top.frame3.date "" 8
    draw_label $tile.top.frame4.label "" 8 0 0 right e
    draw_label $tile.top.frame6.label "" 3
    
    draw_button $tile.top.frame7.plot PLOT "::rfgencal::rfgencal_plot $tile" 0 0 0 center c
    
    $tile.top.frame7.plot config -bd 1 -relief raised
    
    draw_button $tile.top.frame8.save SAVE "" 0 0 0 center c
    $tile.top.frame8.save config -bd 1 -relief raised
  
    draw_frame $tile.table [expr 440-$line_height] 555 267 267

    set row  20
    set linecolor darkgrey
    set linethickness 1
  
    # DRAW TABLE
    set left $tile.table.frame1
    set right $tile.table.frame2
  
    # CREATE COLUMN HEADS
    draw_frame $left.head $row 267 80 85 110
    draw_frame $right.head $row 267 80 85 110
    
    $left.head.frame1 config -bg $linecolor -bd $linethickness
    $left.head.frame2 config -bg $linecolor -bd $linethickness
    $left.head.frame3 config -bg $linecolor -bd $linethickness
    
    $right.head.frame1 config -bg $linecolor -bd $linethickness
    $right.head.frame2 config -bg $linecolor -bd $linethickness
    $right.head.frame3 config -bg $linecolor -bd $linethickness
  
    draw_label $left.head.frame1.1 "" 6 0 0 center c
    draw_label $left.head.frame2.2 "" 9 0 0 center c
    draw_label $left.head.frame3.3 "" 7 0 0 center c
    draw_label $right.head.frame1.1 "" 6 0 0 center c
    draw_label $right.head.frame2.2 "" 9 0 0 center c
    draw_label $right.head.frame3.3 "" 7 0 0 center c

 
    # CREATE LEFT TABLE
    for { set i 0 } { $i < [expr $GENCAL(pagemax)/2] } { incr i } \
    {
      draw_frame $left.$i $row 267 80 85 110
      draw_entry $left.$i.frame1.1 dummyrfgencal 0 0 center
    
      $left.$i.frame1.1 config -takefocus 0 -state disabled
    
      draw_entry $left.$i.frame2.2 dummyrfgencal 9 0 center
    
      draw_label $left.$i.frame3.3 "" 7 0 0 center c
    
      $left.$i.frame1 config -bg $linecolor -bd $linethickness
      $left.$i.frame2 config -bg $linecolor -bd $linethickness
      $left.$i.frame3 config -bg $linecolor -bd $linethickness
    }
  
    # CREATE RIGHT TABLE
    for { set i [expr $GENCAL(pagemax)/2] } { $i < $GENCAL(pagemax) } { incr i } \
    {
      draw_frame $right.$i $row 267 80 85 110
    
      draw_label $right.$i.frame1.1 "" 6 0 0 center c
    
      draw_entry $right.$i.frame2.2 dummyrfgencal 9 0 center
    
      draw_label $right.$i.frame3.3 "" 7 0 0 center c
    
      $right.$i.frame1 config -bg $linecolor -bd $linethickness
      $right.$i.frame2 config -bg $linecolor -bd $linethickness
      $right.$i.frame3 config -bg $linecolor -bd $linethickness
    }
  }
  
  
  #***************************************************************************
  #
  #       Name: start_rfgencal
  #
  # Parameters: tile
  #
  #Description: calls cal_password routine
  #
  #***************************************************************************
  proc start_rfgencal { tile } \
  {
    set cmd [list ::rfgencal::continue_rfgencal $tile]
    ::Calibration_Screen::cal_password  $cmd $tile
    
    return
  }
  
  
  #***************************************************************************
  #
  #       Name: continue_rfgencal
  #
  # Parameters: tile
  #
  # Description: starts RF Generator calibration screen
  #
  #***************************************************************************
  proc continue_rfgencal { tile } \
  {
    global GENCAL

    variable line_height
       
    setup_hardware
      
    set GENCAL(screen) 0 
    
    # START WITH RF GEN LEVEL CAL SCREEN
    build_rfgencal_level $tile 0
  }
  
  #***************************************************************************
  #
  #       Name: setup_hardware
  #
  # Parameters: tile
  #
  # Description: set up hardware for cal screen
  #
  #***************************************************************************
  proc setup_hardware { } \
  {
    Debug::Puts RFGENCAL 1 "setup_hardware - save gen state"

    set ::rfgencal::Old_state [Generator::getSaveState]
    
    Debug::Puts RFGENCAL 1 "setup_hardware - set hardware"
    
    # TURN AUDIO OUT ROUTING OFF
    ::BBAudio::default_audio_routes
    
        
    # SET GENERATOR TO DEFAULT CONDITIONS
    Generator rfout   GEN
    Generator freq    100.0 MHz
    Generator rflevel -30.0
    Generator ptt ON
    Generator dcpower ON
    Generator mod OFF
  
    # RECALL RF GEN CAL VALUES
    gencal_alc_read_cal  rfgen_alc
    gencal_freq_read_cal rfgen_freq_gen GEN
    gencal_freq_read_cal rfgen_freq_tr  TR
  
    
  }
  
  
  #*************************************************
  #       Name: test3
  # Parameters: 
  #Description: 
  #*************************************************
  proc test3 { } \
  {
    global GENCAL
  
    set GENCAL(querytype) 1
    set GENCAL(queryvalue) 0
    set GENCAL(querycommand) "Generator debug rawrflev "
  
    gencal_spline_info 1 0dB 0

    #handle_rfgen_spline 0 0 0 4 0 "  1" 0.0000 234 
  }

};  # End of rfgencal namespace



#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C) 2001      DESIGN ENGINEERING        #
# --------------------------------------------------------------------------#
# FILE        : 2120.rfgencal.tcl                                           #
# PRODUCT     : FITS                                                        #
# AUTHOR      : Bruce Butterfield                                           #
# DESCRIPTION : This namespace is a dummy "screen" that sources and holds   #
#             : the submenu namespace list for the "Systems" droplist       #
#############################################################################
#                                                                           #
# Required Variables:                                                       #
#    LongName                   Name of screen                              #
#    ShortName          Name of screen for menus                            #
#    RecomendedTiles    List of tiles available for this screen. It is not  #
#                       necessary to put a tile on this list that is on the #
#                       RequiredTiles list                                  #
#    NotAllowedTiles    List of tiles NEVER on this screen                  #
#    RequiredTiles      List of tiles ALWAYS on this screen                 #
#    ActiveTiles        List of tiles currently displayed on this screen    #
#    MenuFlags          0 = Active Screen                                   #
#                       1 = Currently functioning as a submenu parent       #
#                                                                           #
# Required Variables for submenu "Screens":                                 #
#    SubmenuScreens     List of all tiles (or submenus) for droplist        #
#    SubDirPath         Path to subdirectory holding the screens for the    #
#                       submenu of this "screen"                            #
#                                                                           #
#############################################################################
#                                                                           #
# Dummy Functions:                                                          #
#    Setup              Called by the screen manager to create the screen.  #
#    Map                Called when screen is mapped on window              #
#    Unmap              Called when screen is removed from display          #
#    Destroy            Called when screen is destroyed                     #
#                                                                           #
#############################################################################

namespace eval RFGenCal_Screen \
{
  variable LongName
  variable ShortName
  variable RecommendedTiles
  variable NotAllowedTiles
  variable RequiredTiles
  variable ActiveTiles
  variable MenuFlags
  variable SubmenuScreens
  variable SubDirPath

  set LongName RFGenCal_Screen
  set ShortName "RF Generator"
  set RequiredTiles { }
  set NotAllowedTiles { }
  set RecommendedTiles { }
  set ActiveTiles { }
  set MenuFlags 0
  set SubmenuScreens { }
  set SubDirPath { }

  variable tile_size      "full"
  variable tile_class     "RFGENCAL"
  
  namespace import ::rfgencal

  #****************************************************************************
  # Name:        Setup
  # Parameters:  parent         parent of child window
  # Description: Create the screen
  # Returns:     Tile path to be packed into the parent frame
  #****************************************************************************
  proc Setup { parent } \
  {
    variable tile_class
    set top_level [winfo toplevel $parent]
    set tile_path [name_child $top_level rfgencal]
    
    if { ![winfo exists $tile_path] } \
    {
      set tile_path  [::Screens::create_tile  $tile_path  rfgencal  $tile_class]
    }
    
    # Call start procedure here
    start_rfgencal $tile_path
    
    return $tile_path
  }

  #****************************************************************************
  # Name:        Cleanup
  # Parameters:  parent         parent of child window
  # Description: Clean up our state
  #****************************************************************************
  proc Cleanup { name } \
  {
    Debug::Puts RFGENCAL 2 "Cleanup $name - syncing gen"
    if {"$::rfgencal::Old_state" != "" }\
    {
       Debug::Puts RFGENCAL 2 "Restore old state"
       eval $::rfgencal::Old_state
       set ::rfgencal::Old_state {}
    } \
    else \
    {
       Debug::Puts RFGENCAL 2 "No old state - resyncing"
       Generator::Sync
    }
  }

  
  #****************************************************************************
  # Name:        Map
  # Parameters:  parent         parent of child window
  # Description: Dummy function
  #****************************************************************************
  proc Map { name } \
  { 
        
    return
  }


  #****************************************************************************
  # Name:        Unmap
  # Parameters:  parent         parent of child window
  # Description: Dummy function
  #****************************************************************************
  proc Unmap { name } \
  { 
    Cleanup $name
    ::View::ClearAllSoftkeys $name
     
    return
  }

 
  #****************************************************************************
  # Name:        Destroy
  # Parameters:  parent         parent of child window
  # Description: Dummy function
  #****************************************************************************
  proc Destroy { name } \
  { 
    Cleanup $name
    return
  }


  #****************************************************************************
  # Functions executed when the namespace is sourced
  #****************************************************************************
  ::View::RegisterScreen RFGenCal_Screen           ;# Add this on the menu


};  # End of the RFGenCal_Screen name space

proc build_rfgencal_screen { type  start  } \
{
   ::Calibration_Screen::force_cal_password
   ::Screens::Show . RFGenCal_Screen
   RFGenCal_Screen::Setup .
   return [rfgencal::build_rfgencal_screen .rfgencal $type $start]
}


   # for compatibility with older software....
   namespace eval cal_data { namespace import  ::rfgencal::* }
   namespace import  ::rfgencal::*
 
