#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C) 2001      DESIGN ENGINEERING        #
# --------------------------------------------------------------------------#
# FILE        : 2130.yigtcxo.tcl                                            #
# PRODUCT     : FITS                                                        #
# AUTHOR      : Bruce Butterfield                                           #
# DESCRIPTION : This namespace is a dummy "screen" that sources and holds   #
#             : the submenu namespace list for the "Systems" droplist       #
#############################################################################
#                                                                           #
# Required Variables:                                                       #
#    LongName		        Name of screen                              #
#    ShortName          Name of screen for menus                            #
#    RecomendedTiles    List of tiles available for this screen. It is not  #
#                       necessary to put a tile on this list that is on the #
#                       RequiredTiles list                                  #
#    NotAllowedTiles    List of tiles NEVER on this screen                  #
#    RequiredTiles      List of tiles ALWAYS on this screen                 #
#    ActiveTiles        List of tiles currently displayed on this screen    #
#    MenuFlags          0 = Active Screen                                   #
#                       1 = Currently functioning as a submenu parent       #
#                                                                           #
# Required Variables for submenu "Screens":                                 #
#    SubmenuScreens     List of all tiles (or submenus) for droplist        #
#    SubDirPath         Path to subdirectory holding the screens for the    #
#                       submenu of this "screen"                            #
#                                                                           #
#############################################################################
#                                                                           #
# Dummy Functions:                                                          #
#    Setup              Called by the screen manager to create the screen.  #
#    Map                Called when screen is mapped on window              #
#    Unmap              Called when screen is removed from display          #
#    Destroy            Called when screen is destroyed                     #
#                                                                           #
#############################################################################

namespace eval Yig_Screen \
{
  variable LongName
  variable ShortName
  variable RecommendedTiles
  variable NotAllowedTiles
  variable RequiredTiles
  variable ActiveTiles
  variable MenuFlags
  variable SubmenuScreens
  variable SubDirPath

  set LongName Yig_Screen
  set ShortName "Assorted Cals"
  set RequiredTiles { }
  set NotAllowedTiles { }
  set RecommendedTiles { }
  set ActiveTiles { }
  set MenuFlags 0
  set SubmenuScreens { }
  set SubDirPath { }

  variable tile_size      "full"
  variable tile_class     "YIGTCXO"
  
  variable StateUI
  
  array set StateUI \
  {
     c4FM_error 0.0
  }

  variable line_height			22
  variable PlaceWidget			""
  variable next_boot			""
  variable SET_TOP_OF_TABLE_RECOMPUTE	255

  variable  get_files   0
  variable  save_files  0


  #***************************************************************************
  #
  #        Name: get_tile
  #
  #  Parameters: any child widget of the tile
  #
  # Description: Get the name of the tile's parent frame
  #
  #***************************************************************************
  proc get_tile { widget } \
  {
    while { 1 } \
    {
      switch -- [winfo class $widget] \
      {
        "YIGTCXO"  { return $widget}
      }
        
      set widget [winfo parent $widget]
    }
  }


  #*************************************************
  #       Name: read_configui
  # Parameters: 
  #Description: read current state of config.ui
  #*************************************************
  proc read_configui { } \
  {
    variable next_boot
  
    if [catch {open /system/config.ui r} fileId] \
    {
      put "File Open failed. Could not open config.ui"
      set next_boot "Customer UI"
    }
    
    if {[gets $fileId firstline ] < 1} \
    {
      set next_boot "Customer UI"
    }
  
    if {[gets $fileId secondline ] < 1} \
    {
      set next_boot "Customer UI"
    }
  
    if {[regexp {^(ENGINEERING_APPLICATION)$} $firstline] && [regexp {^(HALT)$} $secondline]} \
    {
      set next_boot "No UI"
    } \
    elseif {[regexp {^(ENGINEERING_APPLICATION)$} $firstline] && [regexp {^(#HALT)$} $secondline]} \
    {
      set next_boot "Eng App"
    } \
    else \
    {
      set next_boot "Customer UI"
    } 
    
  }
  
  
  #*************************************************
  #       Name: make_configui
  # Parameters: boot, {"No UI" "ENG APP" "Customer UI"}
  #Description: change config.ui
  #*************************************************
  proc make_configui {boot} \
  {
    if {$boot == "Eng App"} \
    {
      if [catch {open /system/config.ui w} fileId] \
      {
        put "File Open failed. Could not open config.ui"
        return
      }
    
      puts $fileId "ENGINEERING_APPLICATION"
      puts $fileId "#HALT"
  
    } \
    elseif {$boot == "Customer UI"} \
    {
      if [catch {open /system/config.ui w} fileId] \
      {
        put "File Open failed. Could not open config.ui"
        return
      }
    
      puts $fileId "#ENGINEERING_APPLICATION"
      puts $fileId "#HALT"
  
    } \
    elseif {$boot == "No UI"} \
    {
      if [catch {open /system/config.ui w} fileId] \
      {
        put "File Open failed. Could not open config.ui"
        return
      }
  
      puts $fileId "ENGINEERING_APPLICATION"
      puts $fileId "HALT"
    }
  
    flush $fileId
    close $fileId
    
    # CHECK BY REREADING config.ui
    read_configui
  }
  
  
  #*************************************************
  #       Name: test_printout
  # Parameters:  
  #Description: prints test printout
  #*************************************************
  proc test_printout { } \
  {
    if [isnt_vxworks] {return}
    
    set printout [open /printer0 w]
    
    puts $printout "Print Test"
  
    close $printout
  }
  
  
  #*************************************************
  #       Name: set_cal_gen_state
  # Parameters: change, {off toggle}
  #Description: turns rf generator on/off
  #*************************************************
  proc set_cal_gen_state {change} \
  {
    if {$change == "toggle"} \
    {
      if {$::Generator::State(dcpower) == "ON"} \
      {
        Generator dcpower OFF -nowait
      } \
      else \
      {
        Generator dcpower ON -nowait
      }
    }
  }

  #*************************************************
  #       Name: do_relative_place
  # Parameters: path
  #             x
  #             y
  #Description: bind mouse middle and right click to return widget name
  #*************************************************
  proc do_relative_place { path x y } \
  {
    variable PlaceWidget
  
    set code [catch {
      set len [string last . $PlaceWidget]
      incr len -1
      set parent [string range $PlaceWidget 0 $len]
  
      set strx [$parent config -width]
      set stry [$parent config -height]
  
      scan $strx "-width width Width %d %d"    tempx maxx
      scan $stry "-height height Height %d %d" tempy maxy
    } err]
  
    if { $code } \
    {
      puts "::Yig_Screen::do_relative_place err: $PlaceWidget"
      return
    }
  
    set relx [expr double ( $x ) / double ( $maxx ) ]
    set rely [expr double ( $y ) / double ( $maxy ) ]
  
    set relx [format "%0.5f" $relx]
    set rely [format "%0.5f" $rely]
  
    place config $PlaceWidget -x 0 -y 0
    place config $PlaceWidget -relx $relx -rely $rely
  
    puts "%W relx:$relx rely:$rely (x:$x y:$y) (maxx:$maxx maxy:$maxy)"
  }
  
  
  #*************************************************
  #       Name: bind_screen_placement
  # Parameters: 
  #Description: bind mouse middle and right click to
  #                return widget name
  #*************************************************
  proc bind_screen_placement { } \
  {
    bind all <Button-3> {do_relative_place %W %x %y}
    bind all <Button-2> {set PlaceWidget %W; puts "%W"}
  }
  
  
  #*************************************************
  #       Name: bind_mouse_color
  # Parameters: 
  #Description: bind mouse middle and right click to color edit
  #*************************************************
  proc bind_mouse_color { } \
  {
  
    bind all <Button-2> { \
               if [catch {%W cget -foreground} dummy] \
               {
                 # add error message "no -foreground attribute for %W"
               } \
               else \
               {
                 AdjustColor -foreground %W
               }        }
                          
    bind all <Button-3> { AdjustColor -background %W }
  }
  
  
  #*************************************************
  #       Name: bind_mouse_widget
  # Parameters: 
  #Description: bind mouse middle and right click to return widget name
  #*************************************************
  proc bind_mouse_widget { } \
  {
    bind all <Button-2> {set dummy 0}
    bind all <Button-3> {puts "widget is %W"}
  }
  
  
  #*************************************************
  #       Name: set_mkt_color
  # Parameters: tile
  #Description: use marketing color set
  #*************************************************
  proc set_mkt_color { tile } \
  {
    set toplevel [winfo toplevel $tile]
    
    # DESTROY SCREENS, EXCEPT CURRENT
    foreach child [winfo children $toplevel] \
    {
      if {![winfo viewable $child]} \
      {
        destroy $child  
      }
    }
    
    # SOURCE COLOR FILE
    if ![file exists /system/utilities/1090.defaultcolors.tcl] \
    {
      put "cannot find default color file"
      return
    }

    source /system/utilities/1090.defaultcolors.tcl
  }
  
  
  #*************************************************
  #       Name: set_orig_color
  # Parameters: tile
  #Description: use marketing color set
  #*************************************************
  proc set_orig_color { tile } \
  {
    set toplevel [winfo toplevel $tile]
    
    # DESTROY SCREENS, EXCEPT CURRENT
    foreach child [winfo children $toplevel] \
    {
      if { ![winfo viewable $child] } \
      {
        destroy $child  
      }
    }
    
    # SOURCE COLOR FILE
    if ![file exists /system/utilities/1089.colors2.tcl] \
    {
      put "cannot find orig color file"
      return
    }

    source /system/utilities/1089.colors2.tcl
  }
  
  
  #*************************************************
  #       Name: save_ascii_to_fd0
  # Parameters: 
  #Description: 
  #*************************************************
  proc save_ascii_to_fd0 { path ext dialog_path {files *} } \
  {  
    cd $path
  
    set name /fd0/NAMES.$ext

    set j 1
    set files [lsort [ glob -nocomplain $path/$files ]]
    
    if [catch \
       {
          # for compatiability with the old system, write the path.
          # We ignore the path on reload
          
          set fout [ open "$name" a ]
          puts $fout "$path"
          close $fout

          foreach old_name $files \
          {     
            if [ file isfile $old_name ] \
            {
              if {[file extension $old_name] != ".default"} \
              {
                 set new_name [ format "FILE%04d.$ext" $j ]
                 $dialog_path insert end "$old_name"
                 $dialog_path yview end
                 set fout [ open "$name" a ]
                 puts $fout "$new_name [file tail $old_name]"
                 close $fout
                 file copy -force $old_name /fd0/$new_name
                 incr j
              }
            }\
            else \
            {
               puts "$old_name is not a file"
            }
         }
       } err ] \
    {
      #set str "$err $::errorInfo"
      set str "$err"
      
      puts $str

      $dialog_path insert end $str
        
      return  ERROR
    }
    
    return  OK
  }
  
  
  #*************************************************
  #       Name: do_etc (launch directory copy operations)
  # Parameters: dialog_path - listbox path
  #Description: copy /etc & /calibration files
  #*************************************************
  proc do_etc { dialog_path } \
  {
    if { [save_ascii_to_fd0  /system/etc  ETC  $dialog_path  ifcfg_eth0] == "ERROR" } \
    {
      return
    }
  
    $dialog_path insert end "Save Operation Complete"
    $dialog_path insert end "Remove Floppy Disk from Disk Drive"
    $dialog_path yview end
  }
  
  
  proc do_save_cal { dialog_path } \
  {
    #puts "do_save_cal:  $dialog_path"
    
    variable  save_files
    
    # Disable the buttons
    ${dialog_path}.ok    config  -state disabled
    ${dialog_path}.save  config  -state disabled
    
    update
    
    set listbox_path  ${dialog_path}.box.files
    
    if { $save_files == 0 } \
    {
      # Get the files
      save_ascii_to_fd0  /system/calibration  CAL  $listbox_path

      do_etc $listbox_path
      
      set save_files  1
    }

    # Enable the buttons
    ${dialog_path}.ok    config  -state normal
    ${dialog_path}.save  config  -state normal
    
    after idle  "safe_focus_force  ${dialog_path}.ok"

    update
    
    set save_files  0
    
    return
  }
  
  
  #*************************************************
  #       Name: save_cals_to_disk
  # Parameters: fr - parent supplied frame
  #Description: save /etc & /calibration directories
  #              to a floppy disk
  #*************************************************
  proc save_cals_to_disk { tg  parent  title } \
  {
    #puts "save_cals_to_disk:  $tg,  $parent,  $title"

    set fr  $tg.fr

    if { ![winfo exists  $tg] } \
    {
      toplevel $tg
      
      add_bindtag $tg Toplevel_cleanup
      
      Mouse_cursor::Hook $tg
      
      wm iconify $tg
      wm iconbitmap $tg [option get $tg windowIcon {}]
      wm transient $tg [winfo toplevel $parent]
      wm group   $tg [winfo toplevel $parent]
      wm title $tg $title
    
      frame $fr  -relief raised -bd 2
    
      button $fr.ok  -text "CLOSE"           \
                     -command "destroy  $tg" \
                     -default active
    
      pack $fr.ok  -side bottom -pady 2

      button $fr.save  -text "SAVE CALS"                       \
                       -command "Yig_Screen::do_save_cal  $fr" \
                       -default active
    
      pack $fr.save  -side bottom -pady 2

      pack $fr
    
      bind $tg <KeyRelease-F8> "destroy $tg; break"
      bind $tg <Key-F8> "break"
    
      # by iconifying the window, then deiconifying it,
      # we force the mouse point to go to the window.
      wm deiconify $tg

      # Display the status toplevel
      frame $fr.box
      
      listbox $fr.box.files  -relief groove                       \
                             -bg white                            \
                             -borderwidth 2                       \
                             -width 40                            \
                             -yscrollcommand "$fr.box.scroll set"
      
      $fr.box.files insert end "Insert a Floppy Disk into the Disk Drive"
      
      $fr.box.files insert end "then press SAVE CALS"

      pack $fr.box.files  -side left
      
      scrollbar $fr.box.scroll  -command "$fr.box.files yview"
      
      pack $fr.box.scroll  -side left -fill y
      
      pack $fr.box  -side top
    }

    after idle  "safe_focus_force  $fr.save"

    #frame $fr.box
    #
    #listbox $fr.box.files -relief raised -borderwidth 2 -width 35 \
    #                      -yscrollcommand "$fr.box.scroll set"
    #$fr.box.files insert end "Insert Floppy Disk into Disk Drive"
    #pack $fr.box.files    -side left
    #
    #scrollbar $fr.box.scroll -command "$fr.box.files yview"
    #pack $fr.box.scroll -side left -fill y
    #
    #pack $fr.box -side top
    #
    #after 1000 ::Yig_Screen::do_save_cal $fr.box.files
    
    return
  }
  
  
  #*************************************************
  #       Name: get_ascii_from_fd0
  # Parameters: fr - parent supplied frame
  #Description: restore /etc & /calibration directories
  #              from a floppy disk
  #*************************************************
  proc get_ascii_from_fd0 { path ext dialog_path { restore_files ""} } \
  {   
    #puts "get_ascii_from_fd0:  $path,  $ext,  $dialog_path,  $restore_files"

    cd $path
    
    if { [file exists "/fd0/names.$ext"] } \
    {
      if { [catch { set fin  [open "/fd0/names.$ext" r] } err] } \
      {
        #set str "Error reading /fd0/names.$ext: $err $::errorInfo"
        set str "Error reading /fd0/names.$ext"
        puts $str
        $dialog_path insert end $str

        return  ERROR
      }
    } \
    else \
    {
      set str "File /fd0/names.$ext not found"

      puts $str

      $dialog_path insert end $str

      return  ERROR
    }
    
    $dialog_path insert end $path
    
    # read the path that the old code wrote. We ignore it.
    set old_path [gets $fin]
    set files [read $fin]
    close $fin
    
    # see if this was written without the initial path...
    if {[llength $old_path] == 2} \
    {
       lappend files $old_path
    }
  
    foreach {old_name new_name} $files \
    {
      if {[llength $restore_files] > 0} \
      {
         if {[lsearch $restore_files $new_name] == -1 } \
         continue;
      }
      set new_name [file join $path $new_name]
      if [catch \
          {
             if { [ file extension $new_name ] != ".default" } \
             {
                $dialog_path insert end [file tail $new_name]
                $dialog_path yview end
                file copy -force /fd0/$old_name $new_name
             }
          } err ] \
       {
          set str "$err $::errorInfo"
          puts $str
          $dialog_path insert end $str
          
          return  ERROR
       }
    }
    
    return  OK
  }
  
  
  #*************************************************
  #       Name: do_get_etc (launch directory copy operations)
  # Parameters: dialog_path - listbox path
  #Description: copy /etc & /calibration files
  #*************************************************
  proc do_get_etc { dialog_path } \
  {
    if { [get_ascii_from_fd0 /system/etc ETC $dialog_path ifcfg_eth0] == "ERROR" } \
    {
      return
    }
  
    $dialog_path insert end "Restore Operation Complete"
    $dialog_path insert end "Remove Floppy Disk from Disk Drive"
    $dialog_path insert end "Reboot Test Set to Activate Calibrations"
    $dialog_path yview end
  }
  
  
  proc do_get_cal { dialog_path } \
  {
    #puts "do_get_cal:  $dialog_path"
    
    variable  get_files
    
    # Disable the buttons
    ${dialog_path}.ok   config  -state disabled
    ${dialog_path}.get  config  -state disabled
    
    update
    
    set listbox_path  ${dialog_path}.box.files
    
    if { $get_files == 0 } \
    {
      # Get the files
      get_ascii_from_fd0  /system/calibration  CAL  $listbox_path

      do_get_etc  $listbox_path
      
      set get_files  1
    }

    # Enable the buttons
    ${dialog_path}.ok   config  -state normal
    ${dialog_path}.get  config  -state normal
    
    after idle  "safe_focus_force  ${dialog_path}.ok"

    update
    
    set get_files  0
    
    return
  }
  
  
  #*************************************************
  #       Name: get_cals_from_disk
  # Parameters: fr - parent supplied frame
  #Description: restore /etc & /calibration directories
  #              from a floppy disk
  #*************************************************
  proc get_cals_from_disk { tg  parent  title } \
  {
    #puts "get_cals_from_disk:  $tg,  $parent,  $title"
    
    set fr  $tg.fr

    if { ![winfo exists  $tg] } \
    {
      toplevel $tg
      
      add_bindtag $tg Toplevel_cleanup
      
      Mouse_cursor::Hook $tg
      
      wm iconify $tg
      wm iconbitmap $tg [option get $tg windowIcon {}]
      wm transient $tg [winfo toplevel $parent]
      wm group   $tg [winfo toplevel $parent]
      wm title $tg $title
    
      frame $fr  -relief raised -bd 2
    
      button $fr.ok  -text "CLOSE"           \
                     -command "destroy  $tg" \
                     -default active
    
      pack $fr.ok  -side bottom -pady 2

      button $fr.get  -text "GET CALS"                       \
                      -command "Yig_Screen::do_get_cal  $fr" \
                      -default active
    
      pack $fr.get  -side bottom -pady 2

      pack $fr
    
      bind $tg <KeyRelease-F8> "destroy $tg; break"
      bind $tg <Key-F8> "break"
    
      # by iconifying the window, then deiconifying it,
      # we force the mouse point to go to the window.
      wm deiconify $tg

      # Display the status toplevel
      frame $fr.box
      
      listbox $fr.box.files  -relief groove                       \
                             -bg white                            \
                             -borderwidth 2                       \
                             -width 40                            \
                             -yscrollcommand "$fr.box.scroll set"
      
      $fr.box.files insert end "Insert a Floppy Disk into the Disk Drive"
      
      $fr.box.files insert end "then press GET CALS"

      pack $fr.box.files  -side left
      
      scrollbar $fr.box.scroll  -command "$fr.box.files yview"
      
      pack $fr.box.scroll  -side left -fill y
      
      pack $fr.box  -side top
    }

    after idle  "safe_focus_force  $fr.get"

    return
  }
  
  
  #*************************************************
  #       Name: build_yigtcxo_screen
  # Parameters: tile, outer frame for screen
  #Description: builds yig, tcxo cal screen
  #*************************************************
  proc build_yigtcxo_screen { tile } \
  {
  
    if [winfo exists $tile.frame1] { return }
    puts "build_yigtcxo_screen create $tile"
    
    variable line_height
    variable next_boot
    
  
    set leftline 0
    set text_width_left 150
    set left $tile.frame1
    set rightline 0
    set text_width_right 150
    set right $tile.frame3
    set middle $tile.frame2
  
    draw_frame $left 440 300
    draw_frame $middle 440 30
    draw_frame $right 440 255
    
    # READ config.ui
    read_configui
    
    # TCXO TITLE
    draw_frame $left.$leftline $line_height 250
    draw_label $left.$leftline.label "TCXO" 20 0 0 center c
    incr leftline
  
    # TCXO CAL
    draw_frame $left.$leftline $line_height 250 $text_width_left [expr 250-$text_width_left]
    draw_label $left.$leftline.frame1.label "TCXO Cal Value" 10
    draw_entry $left.$leftline.frame2.entry ::HW_control::State(tcxo) 2 $::color(readings)
    bind_numeric_spin $left.$leftline.frame2.entry 4 0 9999 0 \
                      {HW_control::Set_TCXO}
    incr leftline
    
    # RF Gen DC Power
    draw_frame $left.$leftline $line_height 250 $text_width_left [expr 250-$text_width_left]
    draw_label $left.$leftline.frame1.label "RF GEN DC Power Is:" 15 0 0 right e
    draw_button $left.$leftline.frame2.entry $::Generator::State(dcpower) \
                {::Yig_Screen::set_cal_gen_state toggle} $::color(readings)
    $left.$leftline.frame2.entry config -textvariable ::Generator::State(dcpower)
    incr leftline
  
    
    # RFGEN TEMP
    draw_frame $left.$leftline $line_height 250 $text_width_left [expr 250-$text_width_left]
    draw_label $left.$leftline.frame1.label "YIG Cal Status" 15
    draw_label $left.$leftline.frame2.entry "" 0
    $left.$leftline.frame2.entry config -textvar ::HW_control::State(yigres_gen)
    incr leftline
  
    # RFGEN YIG 0 
    draw_frame $left.$leftline $line_height 250 $text_width_left [expr 250-$text_width_left]
    draw_label $left.$leftline.frame1.label "RFGEN YIG 0 Hz" 15
    draw_entry $left.$leftline.frame2.entry ::HW_control::State(yig_gen0) 4 $::color(readings)
    bind_numeric $left.$leftline.frame2.entry 4 0 9999 0 \
                 "::HW_control::Set_GEN_YIG"
    incr leftline
  
    # RFGEN YIG 2800
    draw_frame $left.$leftline $line_height 250 $text_width_left [expr 250-$text_width_left]
    draw_label $left.$leftline.frame1.label "RFGEN YIG 2800 MHz" 15
    draw_entry $left.$leftline.frame2.entry ::HW_control::State(yig_gen2800) 4 $::color(readings)
    bind_numeric $left.$leftline.frame2.entry 4 0 9999 0 \
                 "::HW_control::Set_GEN_YIG"
    incr leftline
  
  
    # RCVR YIG 0
    draw_frame $left.$leftline $line_height 250 $text_width_left [expr 250-$text_width_left]
    draw_label $left.$leftline.frame1.label "RCVR YIG 0 Hz" 15
    draw_entry $left.$leftline.frame2.entry ::HW_control::State(yig_rec0) 4 $::color(readings)
    bind_numeric $left.$leftline.frame2.entry 4 0 9999 0 \
                 "::HW_control::Set_RX_YIG"
    incr leftline

    # RCVR YIG 2800
    draw_frame $left.$leftline $line_height 250 $text_width_left [expr 250-$text_width_left]
    draw_label $left.$leftline.frame1.label "RCVR YIG 2800 MHz" 15
    draw_entry $left.$leftline.frame2.entry ::HW_control::State(yig_rec2800) 4 $::color(readings)
    bind_numeric $left.$leftline.frame2.entry 4 0 9999 0 \
                 "::HW_control::Set_RX_YIG"
    incr leftline
  
    # Activate YIG Calibration
    draw_frame $left.$leftline $line_height 250 $text_width_left [expr 250-$text_width_left]
    draw_label $left.$leftline.frame1.label "Do YIG Calibration" 15 0 0 right e
    draw_button $left.$leftline.frame2.entry "RUN" {::HW_control::StartYigCal} $::color(readings)
    incr leftline

    # Display Current IF/Video Temperature 
    draw_frame $left.$leftline $line_height 250 $text_width_left [expr 250-$text_width_left]
    draw_label $left.$leftline.frame1.label "IF/Video Temp (Deg C)" 15
    label $left.$leftline.frame2.entry -textvariable ::HW_control::State(if_temp) \
                                        -width 20 \
                                        -bg $::color(screenbg) \
                                        -fg $::color(screenfg)
    pack $left.$leftline.frame2.entry -side right
    incr leftline
  
    
        # FILL SPACE WITH BLANK LINES
    for { } {$leftline < 12} {incr leftline} \
    {
      draw_frame $left.$leftline $line_height 250 $text_width_left [expr 277-$text_width_left]
    }
    
    # RIGHT SIDE
  
    # PRINTER TITLE
    draw_frame $right.$rightline $line_height 275
    draw_label $right.$rightline.label "PRINTER" 20 0 0 center c
    incr rightline

    # TEST PRINTER
    draw_frame $right.$rightline $line_height 275 
    draw_button $right.$rightline.label "Press for Test Printout" {::Yig_Screen::test_printout}
    $right.$rightline.label config -fg $::color(readings)
    incr rightline
  
    # CURRENT APPLICATION TITLE
    draw_frame $right.$rightline $line_height 275
    draw_label $right.$rightline.label "NEXT UI" 10 0 0 center c 
    incr rightline
  
    # INSTRUCTIONS 
    draw_frame $right.$rightline $line_height 275 
    draw_label $right.$rightline.label " " 14
    $right.$rightline.label config -font small -text "Upon next reboot, "
    label $right.$rightline.label2 -textvariable next_boot -font small -width 20 \
          -bg $::color(screenbg) -fg $::color(screenfg) -justify left -anchor w
    pack $right.$rightline.label2 -side left
    incr rightline
  
    # CHANGE TO CUSTOMER UI
    draw_frame $right.$rightline $line_height 275 
    draw_button $right.$rightline.label "Press for Customer UI" {::Yig_Screen::make_configui "Customer UI"}
    $right.$rightline.label config -fg $::color(readings)
    incr rightline

    # CHANGE TO ENGINEERING APP
    draw_frame $right.$rightline $line_height 275 
    draw_button $right.$rightline.label "Press for Eng App" {::Yig_Screen::make_configui "Eng App"}
    $right.$rightline.label config -fg $::color(readings)
    incr rightline
  
    # MOUSE ACTIONS
    draw_frame $right.$rightline $line_height 245
    draw_label $right.$rightline.label "MOUSE ACTIONS" 20 0 0 center c
    incr rightline
  
    # MOUSE COLOR EDIT
    draw_frame $right.$rightline $line_height 275 
    draw_button $right.$rightline.label "Add mouse color edit" {::Yig_Screen::bind_mouse_color}
    $right.$rightline.label config -fg $::color(readings)
    incr rightline
  
    # MOUSE WIDGET NAME
    draw_frame $right.$rightline $line_height 275 
    draw_button $right.$rightline.label \
                    "Widget Path, Screen Placement" {::Yig_Screen::bind_screen_placement}
    $right.$rightline.label config -fg $::color(readings)
    incr rightline
  
    # MOUSE WIDGET BINDINGS
    draw_frame $right.$rightline $line_height 275 
    draw_button $right.$rightline.label \
                    "Edit Widget Bindings" {::Bind::Path}
    $right.$rightline.label config -fg $::color(readings)
    incr rightline
  
    # C4FM Percentage Error input
    draw_frame $right.$rightline $line_height 275 $text_width_left [expr 275-$text_width_left]
    draw_label $right.$rightline.frame1.label "C4FM Error" 10
    draw_entry $right.$rightline.frame2.entry ::Yig_Screen::StateUI(c4FM_error) 5 $::color(readings)
    bind_numeric_spin $right.$rightline.frame2.entry 3 1 100 0 {set_P25EVM_level $::Yig_Screen::StateUI(c4FM_error)}
    incr rightline
  
      # FILL SPACE WITH BLANK LINES
    for { } {$rightline < 12} {incr rightline} \
    {
      draw_frame $right.$rightline $line_height 275 $text_width_right [expr 277-$text_width_right]
    }

    set_yigtcxo_softkeys $tile
    set_focus $left.1.frame2.entry
 
  }


  #*************************************************
  #       Name: set_yigtcxo_softkeys
  # Parameters: tile - tile path
  #Description: set yigtcxo softkeys 
  #************************************************* 
  proc set_yigtcxo_softkeys { tile } \
  {
    ::View::AssignSoftkey $tile 2 "MKT COLOR" "::Yig_Screen::set_orig_color $tile"

    ::View::AssignSoftkey $tile 3 "ORIG COLOR" "::Yig_Screen::set_mkt_color $tile"

    ::View::AssignSoftkey $tile 4 "PT Copy to HD" "Power debug diskcal 0"

    ::View::AssignSoftkey $tile 5 "PT Restore From HD" "Power debug diskcal 1 "
                                   
    ::View::AssignSoftkey $tile 6 "Save Cals Floppy" \
               "Yig_Screen::save_cals_to_disk  $tile.save_cal  $tile  {Save Calibrations to Floppy}"
               
    ::View::AssignSoftkey $tile 7 "Get Cals Floppy" "Yig_Screen::get_cals_from_disk \
                                     $tile.get_cal  $tile  {Restore Calibrations from Floppy}"
  }
 

  #*************************************************
  #       Name: start_yigtcxo
  # Parameters: tile - tile path
  #Description: calls cal_password routine
  #*************************************************
  proc start_yigtcxo { tile } \
  {
    set cmd [list ::Yig_Screen::continue_yigtcxo $tile]
    ::Calibration_Screen::cal_password  $cmd $tile
  }
  
  
  #*************************************************
  #       Name: continue_yigtcxo
  # Parameters: 
  #Description: builds tcxo yig cal screen
  #*************************************************
  proc continue_yigtcxo { tile } \
  {
    # TURN AUDIO OUT ROUTING OFF
    ::BBAudio::default_audio_routes
  
    build_yigtcxo_screen $tile
  }

  #****************************************************************************
  # Name:        Setup
  # Parameters:  parent		parent of child window
  # Description: Create the screen
  # Returns:     Tile path to be packed into the parent frame
  #****************************************************************************
  proc Setup { parent } \
  {
    variable tile_class

    puts "Yig_Screen::Setup"
         
    set top_level [winfo toplevel $parent]
  
    set tile_path [name_child $top_level yigtcxo]

    if { ![winfo exists $tile_path] } \
    {
      set tile_path  [::Screens::create_tile  $tile_path  yigtcxo  $tile_class]
    }
        
    # Call start procedure here
    start_yigtcxo $tile_path
    
    return $tile_path
  }

  
  #****************************************************************************
  # Name:        Map
  # Parameters:  parent		parent of child window
  # Description: Dummy function
  #****************************************************************************
  proc Map { tile } \
  { 
    puts "Yig_Screen::Map"
    set_yigtcxo_softkeys $tile

    return
  }


  #****************************************************************************
  # Name:        Unmap
  # Parameters:  parent		parent of child window
  # Description: Dummy function
  #****************************************************************************
  proc Unmap { name } \
  { 
    puts "Yig_Screen::Unmap"
    ::View::ClearAllSoftkeys $name
     
    return
  }

 
  #****************************************************************************
  # Name:        Destroy
  # Parameters:  parent		parent of child window
  # Description: Dummy function
  #****************************************************************************
  proc Destroy { name } \
  { 
    puts "Yig_Screen::Destroy"
     
    return
  }


  #****************************************************************************
  # Functions executed when the namespace is sourced
  #****************************************************************************
  ::View::RegisterScreen Yig_Screen	   ;# Add this on the menu


};  # End of the Yig_Screen name space
