#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C)1998,99      DESIGN ENGINEERING      #
# --------------------------------------------------------------------------#
# FILE        : 2140.anlzcal.tcl                                            #
# PRODUCT     : FITS                                                        #
# AUTHOR      : Steve Layton                                                #
# DESCRIPTION : spectrum analyzer bandwidth switching calibration           #
#                                                                           #
#############################################################################

#global ChAnlzState

#global RcvrState
#set RcvrState(ifbw)         300KHZ
#set RcvrState(input)          ANT
#set RcvrState(vbw)            BW_OFF 

#global AnlzrCalTableIndex
#set AnlzrCalTableIndex         2

#global AnlzrCalTableRoot
#set AnlzrCalTableRoot          1

#global AnlzrCalCfTableIndex
#set AnlzrCalCfTableIndex       1

#global AnlzrCalCfTableRoot
#set AnlzrCalCfTableRoot        1

#global AnlzrCalAntTableIndex
#set AnlzrCalAntTableIndex      2

#global AnlzrCalAntTableRoot
#set AnlzrCalAntTableRoot       1 

#global AnlzrCalTRTableIndex
#set AnlzrCalTRTableIndex       2

#global AnlzrCalTRTableRoot
#set AnlzrCalTRTableRoot        1



################################################################
#
#  Menu Dropdown Interface Procs
#
################################################################


namespace eval anlzcal \
{
   variable DSanlzcal

   variable AnlzrCalTableIndex
   variable AnlzrCalTableRoot
   variable AnlzrCalCfTableIndex
   variable AnlzrCalCfTableRoot
   variable AnlzrCalAntTableIndex
   variable AnlzrCalAntTableRoot
   variable AnlzrCalTRTableIndex
   variable AnlzrCalTRTableRoot


   set AnlzrCalCfTableIndex       1
   set AnlzrCalCfTableRoot        1
   set AnlzrCalTableIndex         2
   set AnlzrCalTableRoot          1

   set AnlzrCalAntTableIndex      2
   set AnlzrCalAntTableRoot       1 
   set AnlzrCalTRTableIndex       2
   set AnlzrCalTRTableRoot        1


  # Global variables
  variable  rec_freq
  
  set rec_freq  500
  
  variable  rec_input
  
  set rec_input "T/R"


  #***************************************************************************
  #
  #       Name: get_tile
  #
  # Parameters: any child widget of the tile
  #
  # Description: get the name of the tile's parent frame
  #
  #***************************************************************************
  proc get_tile { widget } \
  {
    while { 1 } \
    {
      switch -- [winfo class $widget] \
      {
        "ANLZCAL"  { return $widget}
      }
        
      set widget [winfo parent $widget]
    }
  }


  #*************************************************
  #       Name: start_anlzcal
  # Parameters: tile - tile path
  #Description: main entry for anlzr cal screen
  #*************************************************
  proc start_anlzcal { tile } \
  {
    ::Calibration_Screen::cal_password  ::anlzcal::continue_anlzcal  $tile
    
    return
  }

  
  #*************************************************
  #       Name: create_leftright_anlzcal
  # Parameters: none
  #Description: spinner selector movement list - use
  #               mouse 'right-click' for path names
  #*************************************************
  proc create_leftright_anlzcal { tile } \
  {
    set tile  [get_tile  $tile]

    # RIGHT SIDE CONTROLS
    set ::anlzcal::DSanlzcal(leftright)      .tos.entry
    lappend ::anlzcal::DSanlzcal(leftright)  .tos.entry2
    lappend ::anlzcal::DSanlzcal(leftright)  .tos.entry3
    lappend ::anlzcal::DSanlzcal(leftright)  .tos.entry4
    lappend ::anlzcal::DSanlzcal(leftright)  .tos.entry5
    lappend ::anlzcal::DSanlzcal(leftright)  .tos.entry6
    lappend ::anlzcal::DSanlzcal(leftright)  .left.input.ant
    lappend ::anlzcal::DSanlzcal(leftright)  .left.input.tr
    lappend ::anlzcal::DSanlzcal(leftright)  .left.freq.entry
    lappend ::anlzcal::DSanlzcal(leftright)  .left.span.entry
    lappend ::anlzcal::DSanlzcal(leftright)  .left.level.entry
    lappend ::anlzcal::DSanlzcal(leftright)  .chbw.bw1
    lappend ::anlzcal::DSanlzcal(leftright)  .chbw.bw2
    lappend ::anlzcal::DSanlzcal(leftright)  .chbw.bw3
    lappend ::anlzcal::DSanlzcal(leftright)  .mainbw.bw4
    lappend ::anlzcal::DSanlzcal(leftright)  .mainbw.bw5
    lappend ::anlzcal::DSanlzcal(leftright)  .mainbw.bw6
    lappend ::anlzcal::DSanlzcal(leftright)  .sweep.entry1
    lappend ::anlzcal::DSanlzcal(leftright)  .sweep.entry2
    lappend ::anlzcal::DSanlzcal(leftright)  .normsg.entry

    return  
  }
  
  
  #*************************************************
  #       Name: continue_anlzcal
  # Parameters: tile
  #Description: if password ok, then run cal applet
  #*************************************************
  proc continue_anlzcal { } \
  {
    set tile  $::AnalyzerCal_Screen::tile_path

    # TURN AUDIO OUT ROUTING OFF
    ::BBAudio::default_audio_routes
  
    build_anlzcal_screen $tile
    
    create_leftright_anlzcal $tile
    
    anlzcal_hw_start
    
    return
  }
  
  
  
  ################################################################
  #
  #  Analyzer Calibration Proc(s)
  #
  ################################################################
  
  #*************************************************
  #       Name: anlzcal_build_trace
  # Parameters: fr - parent frame
  #Description: build trace display
  #*************************************************
  proc anlzcal_build_trace { fr } \
  {
    TraceWidget $fr.wgt -width 547 -height 300 -fg gray -xgratdivs 10 -ygratdivs 8 -persist 0

    $fr.wgt trace create trace1 -sys 13        \
                                -subsys 16     \
                                -cfgsubsys 200 \
                                -lun 1         \
                                -flags 0x3D    \
                                -active 1      \
                                -fg white
    pack $fr.wgt
    
    return
  }
  
  
  #*************************************************
  #       Name: anlzcal_build_tos
  # Parameters: fr - parent frame
  #Description: build top-of-screen control
  #*************************************************
  proc anlzcal_build_tos { fr } \
  {
    label $fr.msg     -text "TOS (dBm)"
    
    entry $fr.entry   -textvariable  ::Analyzer::State(tos)  \
                      -width 5
                             
    label $fr.msg2    -text "Scale (dB/Div)"
    
    entry $fr.entry2  -textvariable  ::Analyzer::State(stat_db_step)  \
                      -width 5
                             
    label $fr.msg3    -text "Atten (dB)"
    
    entry $fr.entry3  -textvariable  ::Receiver::State(atten)  \
                      -width 3
                      
    entry $fr.entry4  -textvariable  ::Analyzer::State(stat_cal)  \
                      -width 6
                             
    label $fr.msg5    -text "IF Gain (dB)"
    
    entry $fr.entry5  -textvariable  ::Analyzer::State(stat_ifgain)  \
                      -width 5
  
    bind $fr.entry   <Return> { Analyzer tos    $::Analyzer::State(tos) -nowait }
    bind $fr.entry2  <Return> { Analyzer scale  $::Analyzer::State(stat_db_step) -nowait }
    bind $fr.entry3  <Return> { Receiver atten  $::Receiver::State(atten) -nowait }
  
    pack $fr.msg    -side left -padx 1
    pack $fr.entry  -side left -padx 1
    pack $fr.msg2   -side left -padx 1
    pack $fr.entry2 -side left -padx 1
    pack $fr.msg3   -side left -padx 1
    pack $fr.entry3 -side left -padx 1
    pack $fr.entry4 -side left -padx 1
    pack $fr.msg5   -side left -padx 1
    pack $fr.entry5 -side left -padx 1
    
    add_bindtag  $fr.entry   Cursor
    add_bindtag  $fr.entry2  Cursor
    add_bindtag  $fr.entry3  Cursor
    add_bindtag  $fr.entry5  Cursor
    
    return
  }
  
  
  #*************************************************
  #       Name: anlzcal_build_rffreq
  # Parameters: fr - parent frame
  #Description: 
  #*************************************************
  proc anlzcal_build_rffreq { fr } \
  {
    label $fr.msg   -text "Center (MHz)"  \
                    -padx  2
    
    entry $fr.entry -textvariable ::anlzcal::rec_freq  \
                    -width 8
  
    bind $fr.entry <Return> \
    {
      Receiver   freq  $::anlzcal::rec_freq  MHz  -nowait;
      Generator  freq  $::anlzcal::rec_freq  MHz  -nowait
    }
  
    pack $fr.msg    -side left
    pack $fr.entry  -side left
    
    add_bindtag  $fr.entry   Cursor

    return
  }
  
  
  #*************************************************
  #       Name: anlzcal_build_rflevel
  # Parameters: fr - parent frame
  #Description: 
  #*************************************************
  proc anlzcal_build_rflevel { fr } \
  {
    label $fr.msg   -text "RF Lvl (dBm)"  \
                    -padx  2

    
    entry $fr.entry -textvariable ::Generator::State(rflev)  \
                    -width 10
  
    bind $fr.entry <Return>  { Generator rflevel $::Generator::State(rflev) -nowait }
  
    pack $fr.msg    -side left
    pack $fr.entry  -side left
    
    add_bindtag  $fr.entry   Cursor

    return
  }
  
  
  #*************************************************
  #       Name: anlzcal_build_input
  # Parameters: fr - parent frame
  #Description: 
  #*************************************************
  proc anlzcal_build_input { fr } \
  {
    label $fr.msg       -text "RF Input"
    
    button $fr.input     -textvariable  ::Receiver::State(rfinput)  \
                         -width 5                                   \
                         -command "::anlzcal::toggle_input" 
  
    pack $fr.msg    -side left

    pack $fr.input    -side left

    add_bindtag  $fr.input   Cursor

    return
  }


  #*************************************************
  #       Name: toggle_input
  # Parameters: 
  #Description: 
  #*************************************************
  proc toggle_input { } \
  {
    if { $::Receiver::State(rfinput) == "ANT" } \
    {
      set ::Receiver::State(rfinput)  "T/R"
    } \
    else \
    {
      set ::Receiver::State(rfinput)  "ANT"
    }
    
    Receiver rfin $::Receiver::State(rfinput) -nowait

    return
  }
  
  
  #*************************************************
  #       Name: anlzcal_build_span
  # Parameters: fr - parent frame
  #Description: 
  #*************************************************
  proc anlzcal_build_span { fr } \
  {
    label $fr.msg -text "Span (MHz)"

    entry $fr.entry -textvariable  ::Analyzer::State(span,MHz)  \
                    -width 10
                    

    bind $fr.entry <Return> { Analyzer span $::Analyzer::State(span,MHz) -1.0 -nowait }
  
    pack $fr.msg    -side left -fill both -expand true
    
    pack $fr.entry
    
    add_bindtag  $fr.entry  Cursor
 
    return
  }
  
  
  #*************************************************
  #       Name: anlzcal_build_sweep
  # Parameters: fr - parent frame
  #Description: 
  #*************************************************
  proc anlzcal_build_sweep { fr } \
  {
    label $fr.msg1   -text "Sweep (uS/trace)"
    

    entry $fr.entry1 -textvariable  ::Analyzer::State(sweep_time)  \
                     -width 8 
    
    bind $fr.entry1 <Return>  { Analyzer sweep $::Analyzer::State(sweep_time) 2 -nowait }
  
    label $fr.msg6   -text "Center Amplitude"

    entry $fr.entry6 -textvariable  ::Analyzer::State(center_dBm) \
                     -width 9 
                     
    pack $fr.msg1      -side top
    pack $fr.entry1    -side top
  
    pack $fr.entry6    -side bottom
    pack $fr.msg6      -side bottom  
    
    add_bindtag  $fr.entry1  Cursor
    add_bindtag  $fr.entry6  Cursor

    return
  }


  #*************************************************
  #       Name: anlzcal_build_if_bw
  # Parameters: fr - parent frame
  #Description: 
  #*************************************************
  proc anlzcal_build_if_bw { fr } \
  {
    label $fr.msg2  -text "IF BW "
  

    menubutton $fr.rbw  -textvariable  ::Analyzer::State(rbw) \
                        -width 10                             \
                        -menu $fr.rbw.menu
    
    menu $fr.rbw.menu   -tearoff 0
    
    $fr.rbw.menu  add  radio  -label "300 Hz"                    \
                              -variable  ::Analyzer::State(rbw)  \
                              -command  { Analyzer rbw 300 -nowait;
                                          set ::anlzcal::AnlzrCalTableIndex 1;
                                          ::anlzcal::doUpdateChanlzrCalTable  }
                               
    $fr.rbw.menu  add  radio  -label "3 kHz"                     \
                              -variable  ::Analyzer::State(rbw)  \
                              -command  { Analyzer rbw 3000 -nowait;
                                          set ::anlzcal::AnlzrCalTableIndex 2;
                                          ::anlzcal::doUpdateChanlzrCalTable  }

    $fr.rbw.menu  add  radio  -label "30 kHz"                    \
                              -variable  ::Analyzer::State(rbw)  \
                              -command  { Analyzer rbw 30000 -nowait;
                                          set ::anlzcal::AnlzrCalTableIndex 4;
                                          ::anlzcal::doUpdateChanlzrCalTable  }

    $fr.rbw.menu  add  radio  -label "60 kHz"                    \
                              -variable  ::Analyzer::State(rbw)  \
                              -command  { Analyzer rbw 60000 -nowait;
                                          set ::anlzcal::AnlzrCalTableIndex 3;
                                          ::anlzcal::doUpdateChanlzrCalTable  }

    $fr.rbw.menu  add  radio  -label "300 kHz"                   \
                              -variable  ::Analyzer::State(rbw)  \
                              -command  { Analyzer rbw 300000 -nowait;
                                          set ::anlzcal::AnlzrCalTableIndex 5;
                                          ::anlzcal::doUpdateChanlzrCalTable  }

    $fr.rbw.menu  add  radio  -label "3 MHz"                     \
                              -variable  ::Analyzer::State(rbw)  \
                              -command  { Analyzer rbw 3000000 -nowait;
                                          set ::anlzcal::AnlzrCalTableIndex 6;
                                          ::anlzcal::doUpdateChanlzrCalTable  }
  
    pack $fr.msg2  -side top
  
    pack $fr.rbw   -side top

    add_bindtag  $fr.rbw  Cursor

    return
  }
  

  #*************************************************
  #       Name: anlzcal_build_video_bw
  # Parameters: fr - parent frame
  #Description: 
  #*************************************************
  proc anlzcal_build_video_bw { fr } \
  {
    label $fr.msg -text "Video BW"
  
  
    menubutton $fr.vbw  -textvariable  ::Analyzer::State(vbw) \
                        -width 10                             \
                        -menu $fr.vbw.menu
    
    menu $fr.vbw.menu   -tearoff 0
    
    $fr.vbw.menu  add  radio  -label "10 Hz"                     \
                              -variable  ::Analyzer::State(vbw)  \
                              -command  { Analyzer vbw 10 -nowait }

    $fr.vbw.menu  add  radio  -label "30 Hz"                     \
                              -variable  ::Analyzer::State(vbw)  \
                              -command  { Analyzer vbw 30 -nowait }

    $fr.vbw.menu  add  radio  -label "100 Hz"                    \
                              -variable  ::Analyzer::State(vbw)  \
                              -command  { Analyzer vbw 100 -nowait }

    $fr.vbw.menu  add  radio  -label "300 Hz"                    \
                              -variable  ::Analyzer::State(vbw)  \
                              -command  { Analyzer vbw 300 -nowait }

    $fr.vbw.menu  add  radio  -label "1 KHz"                     \
                              -variable  ::Analyzer::State(vbw)  \
                              -command  { Analyzer vbw 1000 -nowait }

    $fr.vbw.menu  add  radio  -label "3 KHz"                     \
                              -variable  ::Analyzer::State(vbw)  \
                              -command  { Analyzer vbw 3000 -nowait }

    $fr.vbw.menu  add  radio  -label "10 KHz"                    \
                              -variable  ::Analyzer::State(vbw)  \
                              -command  { Analyzer vbw 10000 -nowait }

    $fr.vbw.menu  add  radio  -label "30 KHz"                    \
                              -variable  ::Analyzer::State(vbw)  \
                              -command  { Analyzer vbw 30000 -nowait }

    $fr.vbw.menu  add  radio  -label "100 KHz"                   \
                              -variable  ::Analyzer::State(vbw)  \
                              -command  { Analyzer vbw 100000 -nowait }

    $fr.vbw.menu  add  radio  -label "300 KHz"                   \
                              -variable  ::Analyzer::State(vbw)  \
                              -command  { Analyzer vbw 300000 -nowait }

    $fr.vbw.menu  add  radio  -label "1 MHz"                     \
                              -variable  ::Analyzer::State(vbw)  \
                              -command  { Analyzer vbw 1000000 -nowait }

    $fr.vbw.menu  add  radio  -label "3 MHz"                     \
                              -variable  ::Analyzer::State(vbw)  \
                              -command  { Analyzer vbw 3000000 -nowait }

    $fr.vbw.menu  add  radio  -label "NONE"                      \
                              -variable  ::Analyzer::State(vbw)  \
                              -command  { Analyzer vbw 0 -nowait }

    
    pack $fr.msg  -side top
    pack $fr.vbw  -side top
    
    add_bindtag  $fr.vbw  Cursor

    return
  }
  


  ################################################################
  #
  #  Analyzer Calibration - do cal data table floating control
  #
  ################################################################
  
  #*************************************************
  #       Name: anlzcal_build_update_cal
  # Parameters: fr - parent frame
  #Description: 
  #*************************************************
  proc anlzcal_build_update_cal { } \
  {
    do_anlzr_cals bwswitch $::anlzcal::AnlzrCalTableIndex \
                  $::AnlzrCals($::anlzcal::AnlzrCalTableIndex.khz) \
                  $::AnlzrCals($::anlzcal::AnlzrCalTableIndex.norm) 0.0
  
    
    Analyzer span   $::Analyzer::State(span,MHz) -1.0 -nowait
    
    Analyzer tos    $::Analyzer::State(tos) -nowait
    
    return
  }



  #*************************************************
  #       Name: doUpdateChanlzrCalTable
  # Parameters: fr - parent frame
  #Description: 
  #*************************************************
  proc doUpdateChanlzrCalTable { } \
  {
    set code [ catch \
    {
      do_anlzr_cals bwswitch $::anlzcal::AnlzrCalTableIndex \
                    $::AnlzrCals($::anlzcal::AnlzrCalTableIndex.khz) \
                    $::AnlzrCals($::anlzcal::AnlzrCalTableIndex.norm) 0.0
  
      Analyzer span   $::Analyzer::State(span,MHz) -1.0 -nowait
    
      Analyzer tos    $::Analyzer::State(tos) -nowait
    
    } err ]
  
    #  will error out when cal table is not displayed
    if { $code } \
    {
      puts "doUpdateChanlzrCalTable err: $err"
    }
    
    return
  }
  
  
  #*************************************************
  #       Name: doChanlzrBWSwCalTable
  # Parameters: fr - parent frame
  #Description: 
  #*************************************************
  proc doChanlzrBWSwCalTable { fr } \
  {
    set ::anlzcal::AnlzrCalTableRoot $fr
  
    frame $fr.table
    frame $fr.save
    frame $fr.info
  
    frame $fr.table.index       -relief groove -borderwidth 2
    frame $fr.table.freq        -relief groove -borderwidth 2
  
    label $fr.table.index.msg   -text "IF BW Index (1-$::AnlzrCals(max))"

    entry $fr.table.index.entry -textvariable ::anlzcal::AnlzrCalTableIndex \
                                -width 3
                                
  
    label $fr.table.freq.msg    -text "Freq (kHz)"
    
    entry $fr.table.freq.entry  -textvariable  ::AnlzrCals($::anlzcal::AnlzrCalTableIndex.khz) \
                                -width 9
                                
    entry $fr.info.entry1       -textvariable  ::AnlzrCals(date) \
                                -width 12
                                
    entry $fr.info.entry2       -textvariable ::AnlzrCals(time) \
                                -width 12
  
    button $fr.save.put         -text  "Save to Cal File"               \
                                -command { put_anlzr_cal_data analyzer; \
                                           ::anlzcal::doUpdateChanlzrCalTable }
                                           
    button $fr.save.get         -text  "Retrieve Cal File"               \
                                -command { get_anlzr_cal_data analyzer }
  
    bind $::anlzcal::AnlzrCalTableRoot.table.index.entry <Return> \
    {
      $::anlzcal::AnlzrCalTableRoot.table.freq.entry  configure \
               -textvariable  ::AnlzrCals($::anlzcal::AnlzrCalTableIndex.khz)
    }
  
  
  #bind_numeric_spin .main.screen.anlzcal.cal_table.fr.table.freq.entry 4 3 1000 0 \
  #{ ::anlzcal::doUpdateChanlzrCalTable }
  
    bind $::anlzcal::AnlzrCalTableRoot.table.freq.entry <Enter> \
    {
      bind $::anlzcal::AnlzrCalTableRoot.table.freq.entry <<Spin_down>> \
      { 
        set ::AnlzrCals($::anlzcal::AnlzrCalTableIndex.khz) \
            [ expr $::AnlzrCals($::anlzcal::AnlzrCalTableIndex.khz) + 0.02];
            ::anlzcal::doUpdateChanlzrCalTable;
            break                   ;# cancels all further bindings
      }
      
      bind $::anlzcal::AnlzrCalTableRoot.table.freq.entry <<Spin_up>> \
      { 
        set ::AnlzrCals($::anlzcal::AnlzrCalTableIndex.khz) \
            [ expr $::AnlzrCals($::anlzcal::AnlzrCalTableIndex.khz) - 0.02];
            ::anlzcal::doUpdateChanlzrCalTable;
            break                   ;# cancels all further bindings
      }
    }


    bind $::anlzcal::AnlzrCalTableRoot.table.freq.entry <Return> \
    { 
      ::anlzcal::doUpdateChanlzrCalTable 
    }
  
    pack $fr.table.index.msg    -side top
    pack $fr.table.index.entry  -side top
  
    pack $fr.table.freq.msg     -side top
    pack $fr.table.freq.entry   -side top
  
    pack $fr.info.entry1        -side top
    pack $fr.info.entry2        -side top

    pack $fr.save.put           -side top
    pack $fr.save.get           -side top
  
    pack $fr.table.index        -side left
    pack $fr.table.freq         -side left
  
    pack $fr.table              -side top
    pack $fr.save               -side left
    pack $fr.info               -side left

    add_bindtag  $fr.table.index.entry  Cursor
    add_bindtag  $fr.table.freq.entry   Cursor
    add_bindtag  $fr.info.entry1        Cursor
    add_bindtag  $fr.info.entry2        Cursor
    add_bindtag  $fr.save.put           Cursor
    add_bindtag  $fr.save.get           Cursor
  
    after idle  "safe_focus_force  $fr.table.index.entry"
    
    return
  }
  
  
  #*************************************************
  #       Name: doAnlzrCfCalTable
  # Parameters: fr - parent frame
  #Description: 
  #*************************************************
  proc doAnlzrCfCalTable { fr } \
  {
    set ::anlzcal::AnlzrCalCfTableRoot $fr

    frame $fr.table
    frame $fr.save
    frame $fr.info
  
    frame $fr.table.index  -relief groove -borderwidth 2
    frame $fr.table.freq   -relief groove -borderwidth 2
    frame $fr.table.norm   -relief groove -borderwidth 2
    frame $fr.table.pt     -relief groove -borderwidth 2
  
    label $fr.table.index.msg   -text "Index (0-$::AnlzrCals(cfmax))"
    
    entry $fr.table.index.entry -textvariable ::anlzcal::AnlzrCalCfTableIndex \
                                -width 3

    label $fr.table.freq.msg    -text "Freq (MHz)"

    entry $fr.table.freq.entry  -textvariable  ::AnlzrCals($::anlzcal::AnlzrCalCfTableIndex.cf.mhz) \
                                -width 9
  
    label $fr.table.norm.msg    -text "CF Pwr (dBm)"

    entry $fr.table.norm.entry  -textvariable ::AnlzrCals($::anlzcal::AnlzrCalCfTableIndex.cf.dbm) \
                                -width 9
  
    label $fr.table.pt.msg      -text "Pwr Term (Watts)"

    entry $fr.table.pt.entry    -textvariable ::AnlzrCals($::anlzcal::AnlzrCalCfTableIndex.cf.watts) \
                                -width 9
  
    entry $fr.info.entry1       -textvariable  ::AnlzrCals(date) \
                                -width 12
                                
    entry $fr.info.entry2       -textvariable ::AnlzrCals(time) \
                                -width 12
                                
    button $fr.save.put         -text  "Save to Cal File"                \
                                -command { put_anlzr_cal_data analyzer }

    button $fr.save.get         -text  "Retrieve Cal File"               \
                                -command { get_anlzr_cal_data analyzer }
  
    bind $::anlzcal::AnlzrCalCfTableRoot.table.index.entry <Return> \
    {
      $::anlzcal::AnlzrCalCfTableRoot.table.freq.entry  configure \
             -textvariable  ::AnlzrCals($::anlzcal::AnlzrCalCfTableIndex.cf.mhz);
    
      $::anlzcal::AnlzrCalCfTableRoot.table.norm.entry  configure \
             -textvariable ::AnlzrCals($::anlzcal::AnlzrCalCfTableIndex.cf.dbm);
    
      $::anlzcal::AnlzrCalCfTableRoot.table.pt.entry  configure \
             -textvariable ::AnlzrCals($::anlzcal::AnlzrCalCfTableIndex.cf.watts);
    
      Receiver freq $::AnlzrCals($::anlzcal::AnlzrCalCfTableIndex.cf.mhz) MHz -nowait
    }
  
    pack $fr.table.index.msg    -side top
    pack $fr.table.index.entry  -side top
  
    pack $fr.table.freq.msg     -side top
    pack $fr.table.freq.entry   -side top
  
    pack $fr.table.norm.msg     -side top
    pack $fr.table.norm.entry   -side top
  
    pack $fr.table.pt.msg       -side top
    pack $fr.table.pt.entry     -side top
  
    pack $fr.info.entry1        -side top
    pack $fr.info.entry2        -side top
  
    pack $fr.save.put           -side top
    pack $fr.save.get           -side top
  
    pack $fr.table.index        -side left
    pack $fr.table.freq         -side left
    pack $fr.table.norm         -side left
    pack $fr.table.pt           -side left
  
    pack $fr.table              -side top
    pack $fr.save               -side left
    pack $fr.info               -side left

    add_bindtag  $fr.table.index.entry  Cursor
    add_bindtag  $fr.table.freq.entry   Cursor
    add_bindtag  $fr.table.norm.entry   Cursor
    add_bindtag  $fr.table.pt.entry     Cursor
    add_bindtag  $fr.info.entry1        Cursor
    add_bindtag  $fr.info.entry2        Cursor
    add_bindtag  $fr.save.put           Cursor
    add_bindtag  $fr.save.get           Cursor

    after idle  "safe_focus_force  $fr.table.index.entry"
  }


  #*************************************************
  #       Name: doAnlzrAntInputCalTable
  # Parameters: fr - parent frame
  #Description: 
  #*************************************************
  proc doAnlzrAntInputCalTable { fr } \
  {
    set ::anlzcal::AnlzrCalAntTableRoot $fr
  
    frame $fr.table
    frame $fr.save
    frame $fr.info
  
    frame $fr.table.index
    frame $fr.table.freq
    frame $fr.table.pwr
  
    label $fr.table.index.msg   -text "Index (0-$::AnlzrCals(antmax)-1)"

    entry $fr.table.index.entry -textvariable ::anlzcal::AnlzrCalAntTableIndex \
                                -width 3
  
    label $fr.table.freq.msg    -text "Freq (MHz)"

    entry $fr.table.freq.entry  -textvariable  ::AnlzrCals($::anlzcal::AnlzrCalAntTableIndex.ant.mhz) \
                                -width 9
  
    label $fr.table.pwr.msg     -text "Ant Correction (dB)"

    entry $fr.table.pwr.entry   -textvariable ::AnlzrCals($::anlzcal::AnlzrCalAntTableIndex.ant.db) \
                                -width 9 
  
    entry $fr.info.entry1       -textvariable  ::AnlzrCals(date) \
                                -width 12
                                
    entry $fr.info.entry2       -textvariable ::AnlzrCals(time) \
                                -width 12
                                
  
    button $fr.save.put         -text  "Save to Cal File"                \
                                -command { put_anlzr_cal_data analyzer }

    
    button $fr.save.get         -text  "Retrieve Cal File"               \
                                -command { get_anlzr_cal_data analyzer }
  
  
    bind $::anlzcal::AnlzrCalAntTableRoot.table.index.entry <Return> \
    {
      $::anlzcal::AnlzrCalAntTableRoot.table.freq.entry  configure \
             -textvariable  ::AnlzrCals($::anlzcal::AnlzrCalAntTableIndex.ant.mhz);
    
      $::anlzcal::AnlzrCalAntTableRoot.table.pwr.entry  configure \
             -textvariable ::AnlzrCals($::anlzcal::AnlzrCalAntTableIndex.ant.db);
    
      Receiver freq $::AnlzrCals($::anlzcal::AnlzrCalAntTableIndex.ant.mhz) MHz -nowait;
  
      set ::anlzcal::rec_freq \
              [ expr $[$::anlzcal::AnlzrCalAntTableRoot.table.freq.entry cget -textvariable] ]
    }
  
    bind $::anlzcal::AnlzrCalAntTableRoot.table.pwr.entry <Return> \
    {
      put_anlzr_cal_data analyzer;

      get_anlzr_cal_data analyzer;

      Analyzer tos  $::Analyzer::State(tos) -nowait
    }
  
    pack $fr.table.index.msg    -side top
    pack $fr.table.index.entry  -side top
  
    pack $fr.table.freq.msg     -side top
    pack $fr.table.freq.entry   -side top
  
    pack $fr.table.pwr.msg      -side top
    pack $fr.table.pwr.entry    -side top
  
    pack $fr.info.entry1        -side top
    pack $fr.info.entry2        -side top
  
    pack $fr.save.put           -side top
    pack $fr.save.get           -side top
  
    pack $fr.table.index        -side left
    pack $fr.table.freq         -side left
    pack $fr.table.pwr          -side left

    pack $fr.table              -side top
    pack $fr.save               -side left
    pack $fr.info               -side left

    add_bindtag  $fr.table.index.entry  Cursor
    add_bindtag  $fr.table.freq.entry   Cursor
    add_bindtag  $fr.table.pwr.entry    Cursor
    add_bindtag  $fr.info.entry1        Cursor
    add_bindtag  $fr.info.entry2        Cursor
    add_bindtag  $fr.save.put           Cursor
    add_bindtag  $fr.save.get           Cursor
  
    after idle  "safe_focus_force  $fr.table.index.entry"
    
    return
  }
  
  
  #*************************************************
  #       Name: doAnlzrTRInputCalTable
  # Parameters: fr - parent frame
  #Description: 
  #*************************************************
  proc doAnlzrTRInputCalTable { fr } \
  {
    set ::anlzcal::AnlzrCalTRTableRoot $fr
  
    frame $fr.table
    frame $fr.save
    frame $fr.info
  
    frame $fr.table.index
    frame $fr.table.freq
    frame $fr.table.pwr
  
    label $fr.table.index.msg   -text "Index (0-$::AnlzrCals(trmax)-1)"

    entry $fr.table.index.entry -textvariable ::anlzcal::AnlzrCalTRTableIndex \
                                -width 3
  
    label $fr.table.freq.msg    -text "Freq (MHz)"

    entry $fr.table.freq.entry  -textvariable  ::AnlzrCals($::anlzcal::AnlzrCalTRTableIndex.tr.mhz) \
                                -width 9
  
    label $fr.table.pwr.msg     -text "TR Correction (dB)"

    entry $fr.table.pwr.entry   -textvariable ::AnlzrCals($::anlzcal::AnlzrCalTRTableIndex.tr.db) \
                                -width 9
  
    entry $fr.info.entry1       -textvariable  ::AnlzrCals(date) \
                                -width 12 
                                
    entry $fr.info.entry2       -textvariable ::AnlzrCals(time) \
                                -width 12
                                
  
    button $fr.save.put         -text  "Save to Cal File"                \
                                -command { put_anlzr_cal_data analyzer }
                                
    button $fr.save.get         -text  "Retrieve Cal File"               \
                                -command { get_anlzr_cal_data analyzer }


    bind $::anlzcal::AnlzrCalTRTableRoot.table.index.entry <Return> \
    {
      $::anlzcal::AnlzrCalTRTableRoot.table.freq.entry  configure \
             -textvariable  ::AnlzrCals($::anlzcal::AnlzrCalTRTableIndex.tr.mhz);
    
      $::anlzcal::AnlzrCalTRTableRoot.table.pwr.entry  configure \
             -textvariable ::AnlzrCals($::anlzcal::AnlzrCalTRTableIndex.tr.db);
    
      Receiver freq $::AnlzrCals($::anlzcal::AnlzrCalTRTableIndex.tr.mhz) MHz -nowait;
    }
  
    bind $::anlzcal::AnlzrCalTRTableRoot.table.pwr.entry <Return> \
    {
      put_anlzr_cal_data analyzer;
      
      get_anlzr_cal_data analyzer;
      
      Analyzer tos  $::Analyzer::State(tos) -nowait
    }
  
    pack $fr.table.index.msg    -side top
    pack $fr.table.index.entry  -side top
  
    pack $fr.table.freq.msg     -side top
    pack $fr.table.freq.entry   -side top
  
    pack $fr.table.pwr.msg      -side top
    pack $fr.table.pwr.entry    -side top
  
    pack $fr.info.entry1        -side top
    pack $fr.info.entry2        -side top
  
    pack $fr.save.put           -side top
    pack $fr.save.get           -side top
  
    pack $fr.table.index        -side left
    pack $fr.table.freq         -side left
    pack $fr.table.pwr          -side left
  
    pack $fr.table              -side top
    pack $fr.save               -side left
    pack $fr.info               -side left

    add_bindtag  $fr.table.index.entry  Cursor
    add_bindtag  $fr.table.freq.entry   Cursor
    add_bindtag  $fr.table.pwr.entry    Cursor
    add_bindtag  $fr.info.entry1        Cursor
    add_bindtag  $fr.info.entry2        Cursor
    add_bindtag  $fr.save.put           Cursor
    add_bindtag  $fr.save.get           Cursor

    after idle  "safe_focus_force  $fr.table.index.entry"

    return
  }
  
  
  #*************************************************
  #       Name: anlzcal_build_normalize_msg
  # Parameters: fr - parent frame
  #Description: 
  #*************************************************
  proc anlzcal_build_normalize_msg { fr } \
  {
    button $fr.norm -text  "Do Normalize Cal"          \
                    -command { Analyzer normalize  1 }

    button $fr.cal  -text  "Do IF Gain Cal"            \
                    -command { Analyzer normalize  2 }

    button $fr.del  -text  "Default Gain Cal"            \
                    -command { reset_gain_calibrations }

    message $fr.entry -textvariable ::Analyzer::State(normalize) \
                      -width 5c -takefocus 0

    pack $fr.norm   -side top
    pack $fr.cal    -side top
    pack $fr.del    -side top
    pack $fr.entry  -side top -fill x
    
    add_bindtag  $fr.norm  Cursor
    add_bindtag  $fr.cal   Cursor
    add_bindtag  $fr.del   Cursor

    return
  }
  
  
  
  ################################################################
  #
  #  Analyzer Calibration - organize screen
  #
  ################################################################
  
  
  #*************************************************
  #       Name: anlzcal_build_left
  # Parameters: fr - parent frame
  #Description: 
  #*************************************************
  proc anlzcal_build_left { fr } \
  {
    frame $fr.input
    frame $fr.freq  -relief groove -borderwidth 2
    frame $fr.span
    frame $fr.level -relief groove -borderwidth 2
  
    anlzcal_build_input     $fr.input
    anlzcal_build_rffreq    $fr.freq
    anlzcal_build_span      $fr.span
    anlzcal_build_rflevel   $fr.level
  
    pack $fr.input -side top
    pack $fr.freq  -side top
    pack $fr.span  -side top
    pack $fr.level -side top
    
    return
  }
  
  
  #*************************************************
  #       Name: build_anlzcal_screen
  # Parameters: parent - a frame is passed to this widget
  #Description: anlzr cal screen - build anlzr cal applet
  #*************************************************
  proc build_anlzcal_screen { fr } \
  {
    if { [winfo exists $fr.tos] } \
    {
      destroy $fr.tos
      destroy $fr.grid
      destroy $fr.left
      destroy $fr.sweep
      destroy $fr.chbw
      destroy $fr.mainbw
      destroy $fr.normsg
    }
    

    frame $fr.tos    -relief ridge -borderwidth 4
    frame $fr.grid   -relief ridge -borderwidth 4
    frame $fr.left   -relief ridge -borderwidth 4
    frame $fr.sweep  -relief ridge -borderwidth 4
    frame $fr.chbw   -relief ridge -borderwidth 4
    frame $fr.mainbw -relief ridge -borderwidth 4
    frame $fr.normsg -relief ridge -borderwidth 4
  
    anlzcal_build_tos            $fr.tos
    anlzcal_build_trace          $fr.grid
    anlzcal_build_left           $fr.left
    anlzcal_build_sweep          $fr.sweep
    anlzcal_build_if_bw          $fr.chbw
    anlzcal_build_video_bw       $fr.chbw
    anlzcal_build_normalize_msg  $fr.normsg
  
    pack $fr.tos    -side top
    pack $fr.grid   -side top
    pack $fr.left   -side left -fill y
    pack $fr.chbw   -side left -fill y
    pack $fr.mainbw -side left -fill y
    pack $fr.sweep  -side left -fill y
    pack $fr.normsg -side left -fill y
  

    # softkey #1
    ::View::AssignSoftkey $fr 2 "BW SW\nTable" \
         "WidgetWindow ::anlzcal::doChanlzrBWSwCalTable $fr.cal_table $fr {Bandwidth Switching Table} "
                                                
    # softkey #2
    ::View::AssignSoftkey $fr 3 "PT C-F\nTable" \
            "WidgetWindow ::anlzcal::doAnlzrCfCalTable $fr.cal_table $fr {PowerTerm Cross-Feed Table} "

    # softkey #3
    ::View::AssignSoftkey $fr 4 "Ant Port\nTable" \
       "WidgetWindow ::anlzcal::doAnlzrAntInputCalTable $fr.cal_table $fr {PowerTerm Ant Input Table} "

    # softkey #4
    ::View::AssignSoftkey $fr 5 "T-R Port\nTable" \
        "WidgetWindow ::anlzcal::doAnlzrTRInputCalTable $fr.cal_table $fr {PowerTerm T-R Input Table} "

    # softkey #5
    ::View::AssignSoftkey $fr 6 "" ""

    # softkey #6
    ::View::AssignSoftkey $fr 7 "Cancel\nNormalize" "Analyzer normalize  0"

    return
  }
  
  
  ################################################################
  #
  #  Analyzer Calibration - start/stop hardware running
  #
  ################################################################
  
  
  #*************************************************
  #       Name: anlzcal_hw_start
  # Parameters: none
  #Description: 
  #*************************************************
  proc anlzcal_hw_start { } \
  {
    Generator rfout    GEN -nowait
    Generator freq     $::anlzcal::rec_freq  MHz -nowait
    Generator rflevel  -30 -nowait
    Generator p25mode  OFF -nowait
    Generator mod      OFF -nowait
    
    Generator ptt 1 -nowait
  
    Receiver freq $::anlzcal::rec_freq MHz -nowait
    Receiver rfin $::anlzcal::rec_input -nowait
    Receiver agcmode 0 -nowait

    
    set ::AnalyzerUI::StateUI(auto_man)  "MAN"

    set ::Analyzer::State(rbw)  "3 kHz";  ::AnalyzerUI::set_rbw

    Analyzer sweep  1000000  2  -nowait

    Analyzer span   0.1  -1.0  -nowait

    Analyzer scale  10 -nowait

    Analyzer tos    -10 -nowait

    set ::Analyzer::State(vbw)  "NONE";  ::AnalyzerUI::set_vbw
    
    ::Analyzer::setEnable "TOS SGLPT LIVE"
    
    Analyzer  run  2 -nowait
    
    catch {Analyzer autocpl 0}

    return
  }
  
  
  #*************************************************
  #       Name: anlzcal_hw_stop
  # Parameters: none
  #Description: 
  #*************************************************
  proc anlzcal_hw_stop { } \
  {
    Analyzer  run  3 -nowait

    ::Analyzer::setEnable "DISABLE"
  
    # Set HW AGC to slow
    Receiver hwagc 1 -nowait

    # Set SW AGC to on
    Receiver agcmode 1 -nowait
    
    Generator ptt 0

    return
  }


};  # End of anlzcal namespace



#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C) 2001      DESIGN ENGINEERING        #
# --------------------------------------------------------------------------#
# FILE        : 2140.anlzcal.tcl                                            #
# PRODUCT     : FITS                                                        #
# AUTHOR      : Bruce Butterfield                                           #
# DESCRIPTION : This namespace is a dummy "screen" that sources and holds   #
#             : the submenu namespace list for the "Systems" droplist       #
#############################################################################
#                                                                           #
# Required Variables:                                                       #
#    LongName           Name of screen                                      #
#    ShortName          Name of screen for menus                            #
#    RecomendedTiles    List of tiles available for this screen. It is not  #
#                       necessary to put a tile on this list that is on the #
#                       RequiredTiles list                                  #
#    NotAllowedTiles    List of tiles NEVER on this screen                  #
#    RequiredTiles      List of tiles ALWAYS on this screen                 #
#    ActiveTiles        List of tiles currently displayed on this screen    #
#    MenuFlags          0 = Active Screen                                   #
#                       1 = Currently functioning as a submenu parent       #
#                                                                           #
# Required Variables for submenu "Screens":                                 #
#    SubmenuScreens     List of all tiles (or submenus) for droplist        #
#    SubDirPath         Path to subdirectory holding the screens for the    #
#                       submenu of this "screen"                            #
#                                                                           #
#############################################################################
#                                                                           #
# Dummy Functions:                                                          #
#    Setup              Called by the screen manager to create the screen.  #
#    Map                Called when screen is mapped on window              #
#    Unmap              Called when screen is removed from display          #
#    Destroy            Called when screen is destroyed                     #
#                                                                           #
#############################################################################

namespace eval AnalyzerCal_Screen \
{
  variable LongName
  variable ShortName
  variable RecommendedTiles
  variable NotAllowedTiles
  variable RequiredTiles
  variable ActiveTiles
  variable MenuFlags
  variable SubmenuScreens
  variable SubDirPath

  set LongName         AnalyzerCal_Screen
  set ShortName        "Anlzr BW Switching"
  set RequiredTiles    { }
  set NotAllowedTiles  { }
  set RecommendedTiles { }
  set ActiveTiles      { }
  set SubmenuScreens   { }
  set SubDirPath       { }
  set MenuFlags 0
  
  variable tile_path      ""
  variable tile_size      "full"
  variable tile_class     "ANLZCAL"

  #****************************************************************************
  # Name:        Setup
  # Parameters:  parent   parent of child window
  # Description: Create the screen
  # Returns:     Tile path to be packed into the parent frame
  #****************************************************************************
  proc Setup { parent } \
  {
    variable tile_class

    puts "AnalyzerCal_Screen::Setup"
         
    set top_level [winfo toplevel $parent]
  
    set tile_path [name_child $top_level anlzcal]


    if { ![winfo exists $tile_path] } \
    {
      set tile_path  [::Screens::create_tile  $tile_path  anlzcal  $tile_class]
    }
    
    set ::AnalyzerCal_Screen::tile_path  $tile_path
    
    # Call start procedure here
    ::anlzcal::start_anlzcal $tile_path

    return $tile_path
  }

  
  #****************************************************************************
  # Name:        Map
  # Parameters:  parent   parent of child window
  # Description: Dummy function
  #****************************************************************************
  proc Map { name } \
  { 
    puts "AnalyzerCal_Screen::Map"
     
    return
  }


  #****************************************************************************
  # Name:        Unmap
  # Parameters:  parent   parent of child window
  # Description: Dummy function
  #****************************************************************************
  proc Unmap { name } \
  { 
    puts "AnalyzerCal_Screen::Unmap"

    ::View::ClearAllSoftkeys $name
    Analyzer run 1

    return
  }

 
  #****************************************************************************
  # Name:        Destroy
  # Parameters:  parent   parent of child window
  # Description: Dummy function
  #****************************************************************************
  proc Destroy { name } \
  { 
    puts "AnalyzerCal_Screen::Destroy"
     
    return
  }


  #****************************************************************************
  # Functions executed when the namespace is sourced
  #****************************************************************************
  ::View::RegisterScreen AnalyzerCal_Screen    ;# Add this on the menu


};  # End of the AnalyzerCal_Screen name space
