#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C) 2001      DESIGN ENGINEERING        #
# --------------------------------------------------------------------------#
# FILE        : 2100.calibrationMenu.tcl                                    #
# PRODUCT     : FITS                                                        #
# AUTHOR      : Bruce Butterfield                                           #
# DESCRIPTION : This namespace is a dummy "screen" that sources and holds   #
#             : the submenu namespace list for the "Calibrations" droplist  #
#############################################################################
#                                                                           #
# Required Variables:                                                       #
#    LongName		Name of screen                                      #
#    ShortName          Name of screen for menus                            #
#    RecomendedTiles    List of tiles available for this screen. It is not  #
#                       necessary to put a tile on this list that is on the #
#                       RequiredTiles list                                  #
#    NotAllowedTiles    List of tiles NEVER on this screen                  #
#    RequiredTiles      List of tiles ALWAYS on this screen                 #
#    ActiveTiles        List of tiles currently displayed on this screen    #
#    MenuFlags          0 = Active Screen                                   #
#                       1 = Currently functioning as a submenu parent       #
#                                                                           #
# Required Variables for submenu "Screens":                                 #
#    SubmenuScreens     List of all tiles (or submenus) for droplist        #
#    SubDirPath         Path to subdirectory holding the screens for the    #
#                       submenu of this "screen"                            #
#                                                                           #
#############################################################################
#                                                                           #
# Dummy Functions:                                                          #
#    Setup              Called by the screen manager to create the screen.  #
#    Map                Called when screen is mapped on window              #
#    Unmap              Called when screen is removed from display          #
#    Destroy            Called when screen is destroyed                     #
#                                                                           #
#############################################################################

namespace eval Calibration_Screen {
  variable LongName
  variable ShortName
  variable RecommendedTiles
  variable NotAllowedTiles
  variable RequiredTiles
  variable ActiveTiles
  variable MenuFlags
  variable SubmenuScreens
  variable SubDirPath

  set LongName Calibration_Screen
  set ShortName "Calibrations"
  set RequiredTiles { }
  set NotAllowedTiles { }
  set RecommendedTiles { }
  set ActiveTiles { }
  set MenuFlags 1
  #set SubmenuScreens [list PowerTerm_Screen RFGenCal_Screen Yig_Screen AnalyzerCal_Screen]
  set SubmenuScreens { }
  set SubDirPath /system/screens/2090.System/2100.Calibrations

  variable cal
  
  array set cal \
  {
     calscreenopen 0
     calpasstry ""
  }

#****************************************************************************
# Name:        Setup
# Parameters:  parent		parent of child window
# Description: Dummy function
# Returns:     Indicator that no tile is generated
#****************************************************************************
   proc Setup { parent } {
     # We are not generating a tile to pack into the main screen so return NULL
     return ""
  }

#****************************************************************************
# Name:        Map
# Parameters:  parent		parent of child window
# Description: Dummy function
#****************************************************************************
  proc Map { name } { 
  }

#****************************************************************************
# Name:        Unmap
# Parameters:  parent		parent of child window
# Description: Dummy function
#****************************************************************************
  proc Unmap { name } { }

#****************************************************************************
# Name:        Destroy
# Parameters:  parent		parent of child window
# Description: Dummy function
#****************************************************************************
  proc Destroy { name } { }
  
  
#*************************************************
#       Name: check_calpassword
# Parameters: comm, command to perform if password correct
#             tile, tile path
#Description: evaluates entered calibration screen password
#*************************************************
proc check_calpassword { comm tile } \
{

  variable cal

  set desired_password [option get $tile password {}]

  if {$cal(calpasstry) == $desired_password} \
  {
    destroy $tile 
    set cal(calscreenopen) 1
    set cal(calpasstry) ""
    eval $comm
  
  } \
  else \
  {
    set cal(calscreenopen) 0
    ${tile}.error config -text "Sorry, incorrect password."
  }

  set cal(calpasstry) ""
}

#*************************************************
#       Name: cal_password
# Parameters: comm, command to perform if password is correct
#             tile, tile path
#Description: calibration password box
#*************************************************
proc cal_password { comm  tile } \
{
   variable cal
   if {$cal(calscreenopen)} \
   {
      eval $comm

   } \
   else \
   {
      set tile_path [name_child $tile calpassword]
      message_box $tile_path QUESTION
      ${tile_path}.ok config -command "::Calibration_Screen::check_calpassword [list $comm] $tile_path"
   }
}

#*************************************************
#       Name: force_cal_password
# Parameters: none
#Description: forces the calibration open flag true
#             (bypasses cal password)
#*************************************************
proc force_cal_password { } \
{
   variable cal
   set cal(calscreenopen) 1
}

#****************************************************************************
# Functions executed when the namespace is sourced
#****************************************************************************
  ::View::RegisterScreen Calibration_Screen	   ;# Add this on the menu
  
  Startup::SourceScreenDir $::Calibration_Screen::SubDirPath ::Calibration_Screen::SubmenuScreens

}  ;# End of the Calibration_Screen


