# $Revision: 1.10 $
# $Date: 2004/05/20 18:25:04 $

namespace eval Memory_Screen {
    variable LongName
    variable ShortName
    variable RecommendedTiles
    variable NotAllowedTiles
    variable RequiredTiles
    variable ActiveTiles
    variable MenuFlags
    variable SubmenuScreens
    variable SubDirPath
    
    set LongName Memory_Screen
    set ShortName "Free Memory"
    set RequiredTiles { }
    set NotAllowedTiles { }
    set RecommendedTiles { }
    set ActiveTiles { }
    set MenuFlags 0
    set SubmenuScreens { }
    set SubDirPath { }
    
    variable tile_path      ""
    variable tile_size      "full"
    variable tile_class     "MEMORY"
    variable Pool
    
    #****************************************************************************
    # Name:        Setup
    # Parameters:  parent		parent of child window
    # Description: Dummy function
    # Returns:     Indicator that no tile is generated
    #****************************************************************************
    proc Setup { parent } \
    {
        variable Pool
        # variable elapsed
        
        # puts "Elapsed = [expr [clock seconds] - $elapsed]"
        set top_level [winfo toplevel $parent]
        set tile_path [name_child $top_level memory]
        set tile_class  $::Memory_Screen::tile_class
        if { ![winfo exists $tile_path] } \
        {
            set tile_path  [::Screens::create_tile $tile_path memory $tile_class]
        }
        set ::Memory_Screen::tile_path $tile_path
        
        
        return $tile_path
    }
    
    #****************************************************************************
    # Name:        Map
    # Parameters:  parent		parent of child window
    # Description: Dummy function
    #****************************************************************************
    proc Map { name } \
    {
        variable tile_path
        
        if {![winfo exists $tile_path.frame]} \
        {
            frame $tile_path.frame ;# -relief groove -borderwidth 2
            pack $tile_path.frame
        }
        ::View::ClearAllSoftkeys $tile_path
        ::View::PackOptions $tile_path ""
        
        # ASSIGN SOFTKEYs
        ::View::AssignSoftkey $tile_path 2 "Refresh" "::Memory_Screen::DisplayFree"
        # ::View::AssignSoftkey $tile_path 7 "Reload Me!" "source /system/screens/2090.System/2103.memory.tcl"
        if {[info exists ::Selftest_Screen::CallMemory]} \
        { ;# Only set up return key when this is called from selftest screen.
            ::View::AssignReturn $tile_path "::Selftest_Screen::Show Selftest_Screen"
        } \
        else \
        {
            ::View::AssignReturn $tile_path "" ;# disable Return key.
        }
        DisplayFree
        screen_done FORCE_COMPLETE
        
        return
    }
    
    #****************************************************************************
    # Name:        Unmap
    # Parameters:  parent		parent of child window
    # Description: Dummy function
    #****************************************************************************
    proc Unmap { name } \
    {
        variable Pool
        variable tile_path
        
        # We're leaving this screen, so lets store the current free ram
        set Pool(prev) $Pool(free)
        ::View::AssignReturn $tile_path "" ;# disable Return key.
        return
    }
    
    ################################################################################
    # Name:        Destroy
    # Parameters:  parent		parent of child window
    # Description: Dummy function
    ################################################################################
    proc Destroy { name } \
    {
        variable tile_path
        
        destroy $tile_path
        return
    }
    
    ##############
    #
    # MemoryBarUpdate
    #
    proc MemoryBarUpdate {name} \
    {
        variable State
        variable Pool
        
        # double is used here to force numbers to floating point
        set percentfree [expr double($Pool($name)) / $Pool(total) * 100.0]
        set widgetfree $State(STATUS_FRAME).$name.bar
        set color $Pool(color)
        set heightfree [winfo height $widgetfree]
        set widthfree [winfo width $widgetfree]
        set widthfree [expr double($widthfree) * $percentfree / 100.0]
        $widgetfree delete all
        $widgetfree create rect 0 0 $widthfree $heightfree -fill $color -tags all
    }
    proc LowMemoryAlert {} \
    {
        variable tile_path
        
        set ts .lowmem
        toplevel $ts ;# -height 440 -width 555
        wm withdraw $ts
        wm title $ts "Low Memory Alert!"
        wm resizable $ts 0 0
        label $ts.title -text "Warning: Free Memory is very low!\n\nA reboot is recommended to free up memory."
        frame $ts.bframe
        button $ts.bframe.reboot -text "Reboot" -width 12 -height 2 -command "powerdown 1; destroy $ts"
        button $ts.bframe.continue -text "Live\nDangerous" -width 12 -height 2 -command "destroy $ts"
        pack $ts.title $ts.bframe -side top -padx 20 -pady 10
        pack $ts.bframe.reboot $ts.bframe.continue -side left -padx 30
        update idle
        set winx [expr ([winfo width $tile_path] - [winfo reqwidth $ts]) / 2]
        set winy [expr ([winfo height $tile_path] - [winfo reqheight $ts]) / 2]
        wm geometry $ts +$winx+$winy
        wm deiconify $ts
        set_focus $ts.bframe.reboot
        tkwait window $ts
    }
    
    ################################################################################################################
    # Screen setup routines
    
    ##############
    # setup load messages and loadbars on screen
    #
    proc SetupMemoryBar { name } \
    {
        frame $name
        label $name.text ;# -font {courier 16 bold}
        canvas $name.bar -bg darkgrey -height 15
        pack $name.text $name.bar -side top -expand false -fill x
        pack $name -side top -fill x -expand false -pady 5
    }
    
    #############
    # define the procs for showing things
    proc MemoryMessage { which msg {color black} } \
    {
        variable State
        variable Pool
        
        if [winfo exists $State(STATUS_FRAME)] \
        {
            if [info exists $msg] \
            {
                $State(STATUS_FRAME).$which.text config -textvariable $msg -fg $color -text ""
            } \
            else \
            {
                $State(STATUS_FRAME).$which.text config -text $msg -fg $color -textvariable ""
            }
        }
    }
    
    #############
    # Set up the load status frame
    proc SetupMemoryScreen { name } \
    {
        variable State
        variable Pool
        
        set State(STATUS_FRAME) $name
        if {![winfo exists $name.free]} \
        {
            frame $name.totheap
            label $name.totheap.text -textvariable ::Memory_Screen::Pool(totaltext) ;# -font {courier 16 bold}
            pack $name.totheap.text -side top -expand false -fill x
            pack $name.totheap -side top -fill x -expand false -pady 5
            SetupMemoryBar $name.free
            SetupMemoryBar $name.prev
            frame $name.difffree
            label $name.difffree.text -textvariable ::Memory_Screen::Pool(difftext) ;# -font {courier 16 bold}
            pack $name.difffree.text -side top -expand false -fill x
            pack $name.difffree -side top -fill x -expand false -pady 5
        }
        MemoryMessage free $Pool(freetext)
        MemoryMessage prev $Pool(prevtext)
    }
    proc MemoryInit {} \
    {
        variable Pool
        # variable elapsed
        
        # set elapsed [clock seconds]
        set Pool(total) [Memory::Free]
        set Pool(free) $Pool(total)
        set Pool(prev) $Pool(total)
        set Pool(diff) 0
    }
    ################################################################################
    #       Name: DisplayFree
    # Parameters:
    #Description: displays a bar graph representing free memory.
    #
    ################################################################################
    proc DisplayFree {} \
    {
        variable Pool
        variable tile_path
        
        set Pool(free) [Memory::Free]
        if {![info exists Pool(total)]} {tkwait variable ::Memory_Screen::Pool(total)}
        set Pool(diff) [expr $Pool(free) - $Pool(prev)]
        set Pool(totaltext) "Total Heap: $::Memory_Screen::Pool(total)"
        set Pool(freetext) "Current Free: $::Memory_Screen::Pool(free)"
        set Pool(prevtext) "Previous Free: $::Memory_Screen::Pool(prev)"
        set Pool(difftext) [format "Difference: %+2d" $::Memory_Screen::Pool(diff)]
        SetupMemoryScreen $tile_path.frame
        if {$Pool(free) > [expr $Pool(total) * 0.5]} {set Pool(color) green} \
        elseif {$Pool(free) > [expr $Pool(total) * 0.25]} {set Pool(color) yellow} \
        else {set Pool(color) red}
        MemoryBarUpdate free
        set Pool(color) blue
        MemoryBarUpdate prev
        if {$Pool(free) < 1000000} \
        {
            LowMemoryAlert
        }
    }
    #****************************************************************************
    # Functions executed when the namespace is sourced
    #****************************************************************************
    # The following lines were remarked so that the memory screen will never be
    # added to the system menus, and will only be accessible through the self
    # test screen.
    # if {[option get . memoryTest {}] == "enabled" } \
    # {
    #    ::View::RegisterScreen Memory_Screen ;# Add this on the menu
    # }
    if {![info exists Pool(total)]} \
       {
           # Wait 12 seconds, and then get free memory
           after 12000 ::Memory_Screen::MemoryInit
       }
} ;# End of Memory_Screen namespace

