# $Revision: 1.16 $
# $Date: 2004/05/20 18:25:04 $

#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C) 2002        DESIGN ENGINEERING      #
# ------------------------------------------------------------------------- #
# FILE        : selftest.tcl                                                #
# PRODUCT     : FITS                                                        #
# AUTHOR      : Mark Buttram                                                #
# DESCRIPTION : FITS Built In Test main program code                        #
#############################################################################
#                                                                           #
# Test Window Design:                                                       #
#    o Test selection menu allows execution of individual or all tests      #
#    o Test Status - White=Running, Red=Fail, Green=Pass, Yellow=Incomplete #
#    o Text window - displays results of tests                              #
#                                                                           #
#############################################################################
#                                                                           #
# Required Variables:                                                       #
#    LongName           Name of screen                                      #
#    ShortName          Name of screen for menus                            #
#    RecomendedTiles    List of tiles available for this screen. It is not  #
#                       necessary to put a tile on this list that is on the #
#                       RequiredTiles list                                  #
#    NotAllowedTiles    List of tiles NEVER on this screen                  #
#    RequiredTiles      List of tiles ALWAYS on this screen                 #
#    ActiveTiles        List of tiles currently displayed on this screen    #
#    MenuFlags          0 = Active Screen                                   #
#                       1 = Currently functioning as a submenu parent       #
#                                                                           #
# Required Variables for submenu "Screens":                                 #
#    SubmenuScreens     List of all tiles (or submenus) for droplist        #
#    SubDirPath         Path to subdirectory holding the screens for the    #
#                       submenu of this "screen"                            #
#                                                                           #
#############################################################################
#                                                                           #
# Dummy Functions:                                                          #
#    Setup              Called by the screen manager to create the screen.  #
#    Map                Called when screen is mapped on window              #
#    Unmap              Called when screen is removed from display          #
#    Destroy            Called when screen is destroyed                     #
#                                                                           #
#############################################################################

package require HW_control
# To turn on debugging statements, create the file listed below.
if {[file exists "/system/screens/2090.System/set_debug.txt"]} \
{
    set debug_print 1
}

namespace eval Selftest_Screen {
    variable LongName
    variable ShortName
    variable RecommendedTiles
    variable NotAllowedTiles
    variable RequiredTiles
    variable ActiveTiles
    variable MenuFlags
    variable SubmenuScreens
    variable SubDirPath
    
    set LongName Selftest_Screen
    set ShortName "Self Test"
    set RequiredTiles { }
    set NotAllowedTiles { }
    set RecommendedTiles { }
    set ActiveTiles { }
    set MenuFlags 0
    set SubmenuScreens { }
    set SubDirPath { }
    
    variable tile_path
    variable tile_size      "full"
    variable tile_class     "SELFTEST"
    variable option_menu .selftest_Screen_options
    
    variable selftest     ;# My global selftest array.
    variable TestName     ;# Array holds the name of each test.
    variable TestStatus   ;# Array holds the status of each test (Pass, Fail, etc.).
    variable TestCheck    ;# Array holds the check state of the test buttons.
    variable NumTests     ;# Holds actual number of tests.
    variable Finished     ;# This flag is set whenever a test is finished.
    variable StopAllTests ;# This flag is set whenever the test should stop.
    variable ResultsId    ;# File id of results file.
    variable StopOnFail   ;# Tells whether to stop on failures or not.
    variable RunTest      ;# Array holds procedure name of each test.
    variable CallMemory   ;# Used to set the return key in the memory screen.
    
    # ************************************************************************
    # Name:        Setup
    # Parameters:  parent		parent of child window
    # Description: Dummy function
    # Returns:     Indicator that no tile is generated
    # ************************************************************************
    proc Setup { parent } \
    {
        variable selftest
        variable CallMemory
        
        if {$selftest(debug)} {puts "proc: ::Selftest_Screen::Setup"}
        set topWin [winfo toplevel $parent]
        set tile_path [name_child $topWin selftest]
        set tile_class  $::Selftest_Screen::tile_class
        if {![winfo exists $tile_path]} \
        {
            set tile_path [::Screens::create_tile $tile_path selftest $tile_class]
        }
        set ::Selftest_Screen::tile_path $tile_path
        
        if {[info exists CallMemory]} \
        { ;# Variable used on memory screen to determine if return is active.
            unset CallMemory
            puts "Returning from Memory_Screen"
        }
        if {[info exists ::Results_Screen::KillMe]} \
        { ;# Variable used to tell when returning from the results screen. If
            # so then destroy the results window (an attempt to conserve memory).
            unset ::Results_Screen::KillMe
            destroy .results
            puts "Results window destroyed"
        }
        
        return $tile_path
    }
    
    # ************************************************************************
    # Name:        Map
    # Parameters:  parent		parent of child window
    # Description: Dummy function
    # ************************************************************************
    proc Map { name } \
    {
        variable tile_path
        variable option_menu
        variable selftest
        
        if {$selftest(debug)} {puts "proc: ::Selftest_Screen::Map"}
        set selftest(active) 1
        # Turn Audio out Routing off
        ::BBAudio::default_audio_routes
        
        if {![winfo exists $tile_path.titleframe]} \
        {
            BuildSelftestScreen $tile_path
            BuildSelftestMenu
            set selftest(ididit) 0
        }
        
        ::View::PackOptions $tile_path $option_menu
        
        # ASSIGN SOFTKEYs
        if {$selftest(ididit)} {
            # The self test is running, so restore the Abort fkeys
            KeypadAbort
        } else {
            # The self test is not running so display the normal fkeys
            KeypadMain
        }
        
        screen_done FORCE_COMPLETE
        
        return
    }
    
    # ************************************************************************
    # Name:        Unmap
    # Parameters:  parent		parent of child window
    # Description: Dummy function
    # ************************************************************************
    proc Unmap { name } \
    {
        variable StopAllTests
        variable Finished
        variable selftest
        
        if {$selftest(debug)} \
        {
            puts "name: $name"
            puts "proc: ::Selftest_Screen::Unmap"
        }
        set selftest(active) 0
        if {[info exists StopAllTests]} \
        { ;# If Self Test is running
            set StopAllTests 1 ;# Stop the test if it is running.
            set Finished 1
        }
        # STORE LAST FOCUS
        # if {[string first ".main.title" [focus]] == -1} {
        # set selftest(lastfocus) [focus]
        # return
        # }
    }
    
    # ************************************************************************
    # Name:        Destroy
    # Parameters:  parent		parent of child window
    # Description: Dummy function
    # ************************************************************************
    proc Destroy { name } \
    {
        variable tile_path
        variable selftest
        
        if {$selftest(debug)} \
        {
            puts "name: $name"
            puts "proc: ::Selftest_Screen::Destroy"
        }
        destroy $tile_path
        return
    }
    
    #########################################################################
    #       Name: BuildSelftestMenu
    # Parameters:
    #Description: Creates self test options menu
    #
    ########################################################################
    proc BuildSelftestMenu { } \
    {
        global color
        variable selftest
        variable option_menu
        
        if {$selftest(debug)} {puts "BuildSelftestMenu:"}
        set menuName $option_menu.menu
        # BUILD MENU
        if {[winfo exists $menuName]} \
        {
            destroy $menuName
        } \
        else \
        {
            menubutton $option_menu -text  "Options"		\
            -menu $menuName	\
            -relief raised
            
            add_bindtag $option_menu Softkey
            
            menu $menuName -tearoff 0	\
            -postcommand ""
        }
        
        $menuName add cascade -label "Test Length" -menu $menuName.level
        menu $menuName.level
        $menuName.level add radiobutton -label "Short" \
        -command "set ::Selftest_Screen::selftest(level) 0" -value 0 -variable ::Selftest_Screen::selftest(level)
        
        $menuName.level add radiobutton -label "Long" \
        -command "set ::Selftest_Screen::selftest(level) 1" -value 1 -variable ::Selftest_Screen::selftest(level)
        
        $menuName add cascade -label "Audio to Speaker" -menu $menuName.spkr
        menu $menuName.spkr
        $menuName.spkr add radiobutton -label "Off" \
        -command "set ::Selftest_Screen::selftest(speaker) 0" -value 0 -variable ::Selftest_Screen::selftest(speaker)
        
        $menuName.spkr add radiobutton -label "On" \
        -command "set ::Selftest_Screen::selftest(speaker) 1" -value 1 -variable ::Selftest_Screen::selftest(speaker)
        
        $menuName add cascade -label "Audio to Audio Out 1" -menu $menuName.audout1
        menu $menuName.audout1
        $menuName.audout1 add radiobutton -label "Off" \
        -command "set ::Selftest_Screen::selftest(audout1) 0" -value 0 -variable ::Selftest_Screen::selftest(audout1)
        
        $menuName.audout1 add radiobutton -label "On" \
        -command "set ::Selftest_Screen::selftest(audout1) 1" -value 1 -variable ::Selftest_Screen::selftest(audout1)
        
        $menuName add cascade -label "Audio to Audio Out 2" -menu $menuName.audout2
        menu $menuName.audout2
        $menuName.audout2 add radiobutton -label "Off" \
        -command "set ::Selftest_Screen::selftest(audout2) 0" -value 0 -variable ::Selftest_Screen::selftest(audout2)
        
        $menuName.audout2 add radiobutton -label "On" \
        -command "set ::Selftest_Screen::selftest(audout2) 1" -value 1 -variable ::Selftest_Screen::selftest(audout2)
        
        # Bind button release so default bindings do not unpost the menu
        bind $menuName <ButtonRelease> {MenuInvokeRemain %W 1;break}
        bind $menuName <Return>        {MenuInvokeRemain %W 0;break}
        
        # Bind to the Return and Unmap
        # bind $menuName <KeyRelease-F8> "::ScopeUI::menu_return $tile"
        # bind $menuName <Unmap>         "::ScopeUI::set_last_menu_input %W $tile"
    }
    
    #########################################################################
    #       Name: BuildSelftestScreen
    # Parameters: tile, outer frame for screen
    #Description: builds screen containing selftest menu
    #
    #########################################################################
    proc BuildSelftestScreen {tile} \
    {
        variable selftest
        
        if {$selftest(debug)} {puts "BuildSelftestScreen:"}
        set title_frame $tile.titleframe
        set testname_frame $tile.testnameframe
        
        BuildTitle $title_frame
        BuildTestname $testname_frame
        
        pack $title_frame $testname_frame -side top -fill x
    }
    
    proc BuildTitle {frame} \
    {
        variable selftest
        # Create our main menu label
        if {$selftest(debug)} {puts "BuildTitle:"}
        
        frame $frame -height 30 -width 555
        label $frame.label -text "Self Test Menu"
        pack $frame.label -side top -fill y
    }
    
    proc BuildTestname {frame} \
    {
        variable NumTests
        variable tnf
        variable TestNum
        variable TestStatus
        variable selftest
        
        if {$selftest(debug)} {puts "BuildTestname:"}
        
        set tnf $frame
        
        # Create a frame to hold all the test check buttons
        
        frame $frame
        
        # Add the check buttons
        
        for {set TestNum 0} {$TestNum < $NumTests} {incr TestNum} \
        {
            checkbutton $frame.cb($TestNum) -justify left -variable ::Selftest_Screen::TestCheck($TestNum) \
            -textvariable ::Selftest_Screen::TestName($TestNum) -borderwidth 2 -highlightthickness 3 -takefocus 1
            label $frame.status($TestNum) -textvariable ::Selftest_Screen::TestStatus($TestNum)
            SetStatus [GetState $TestStatus($TestNum)]
            grid $frame.cb($TestNum) -sticky w -row $TestNum -column 0
            grid $frame.status($TestNum) -sticky ew -row $TestNum -column 1
        }
        grid columnconfigure $frame 0 -minsize 400
        grid columnconfigure $frame 1 -minsize 100
    }
    #########################################################################
    #       Name: ResetStatus
    # Parameters:
    #Description: resets the status fields to "Not Tested" on a gray
    #             background.
    #
    #########################################################################
    proc ResetStatus {} \
    {
        variable NumTests
        variable TestNum
        variable selftest
        
        if {$selftest(debug)} {puts "ResetStatus:"}
        
        for {set TestNum 0} {$TestNum < $NumTests} {incr TestNum} {
            SetStatus 5 ;# Set status to Not Tested
        }
    }
    
    #########################################################################
    #       Name: ResetAll
    # Parameters:
    #Description: Deselects all tests and resets the status fields
    #             to "Not Tested" on a gray background.
    #
    #########################################################################
    proc ResetAll {} \
    {
        variable NumTests
        variable TestCheck
        variable selftest
        
        if {$selftest(debug)} {puts "ResetAll:"}
        for {set x 0} {$x < $NumTests} {incr x} {
            set TestCheck($x) 0
        }
        ResetStatus
    }
    
    #########################################################################
    #       Name: SelectAll
    # Parameters:
    #Description: Selects all tests.
    #
    ########################################################################
    proc SelectAll {} \
    {
        variable NumTests
        variable TestCheck
        variable selftest
        
        if {$selftest(debug)} {puts "SelectAll:"}
        
        for {set x 0} {$x < $NumTests} {incr x} {
            set TestCheck($x) 1
        }
    }
    
    #########################################################################
    #       Name: RunSelected
    # Parameters:
    #Description: Opens the results file, runs all the selected tests, and
    #             then closes the file.
    #
    #########################################################################
    proc RunSelected {} \
    {
        variable Finished
        variable StopAllTests
        variable selftest
        variable TestCheck
        variable TestNum
        variable NumTests
        variable ResultsId
        variable tile_path
        variable RunTest
        variable Settings
        
        if {$selftest(debug)} {puts "RunSelected: Begin"}
        
        set ClkFmt "%I:%M:%S %p          %d %b %Y"
        set StopAllTests 0
        ResetStatus
        set selftest(lastfocus) [focus]
        if {! $selftest(remote)} \
        { ;# if we aren't running remotely
            if {$selftest(TXPort) != 0 || $selftest(RXPort) != 0} \
            {
                set MessageA "Connect a coax cable between the"
                if {$selftest(TXPort)} {set MessageB "GEN port, and the"} \
                else {set MessageB "T/R port, and the"}
                if {$selftest(RXPort)} {set MessageC "ANT port."} \
                else {set MessageC "T/R port."}
                
                set SFMessage "$MessageA $MessageB $MessageC"
            } \
            else \
            {
                set SFMessage "Remove all connections from the T/R port."
            }
            ConnectPrompt $SFMessage
        }
        # set_focus .
        set_focus $selftest(lastfocus)
        KeypadAbort
        # Save Settings
        if {$selftest(debug)} {puts "Saving settings"}
        set Settings ""
        append Settings [SavedStates::save_info]
        append Settings [SavedStates::save_engines]
        BBAudio clrmix L1         ;# Clear routing to Layer 1
        BBAudio clrmix spkr       ;# Clear routing to Speaker
        BBAudio clrmix fgen       ;# Clear routing to Audio out 1
        BBAudio clrmix demod      ;# Clear routing to Audio out 2
        # Analyzer run 3       ;# This shouldn't be necessary
        Receiver squelchlevel 900 -forgive ;# 900 is out of range.
        BBAudio speakervolume 2            ;# Set volume level (range 0-31).
        set ResultsId [open $::sys(system_path)/screens/2090.System/results.txt {WRONLY CREAT TRUNC}]
        puts $ResultsId [MakeHeader "2975 Self Test Results" 3]
        puts $ResultsId [MakeHeader [clock format [clock seconds] -format $ClkFmt] 3]
        switch $selftest(level) \
        {
            0 {puts $ResultsId [MakeHeader "Short test was run" 3]}
            1 {puts $ResultsId [MakeHeader "Long test was run" 3]}
        }
        puts $ResultsId ""
        for {set Tests 0} {$Tests < $NumTests} {incr Tests} \
        {
            if {$TestCheck($Tests)} \
            {
                set TestNum $Tests
                if {!$StopAllTests} \
                {
                    $RunTest($Tests)
                } \
                else \
                {
                    SetStatus 2
                    WriteStartEndHead 0
                    WriteStartEndHead 1
                }
            }
        }
        puts $ResultsId [MakeHeader [clock format [clock seconds] -format $ClkFmt] 3]
        puts $ResultsId [MakeHeader "End 2975 Self Test Results" 3]
        close $ResultsId
        # Restore Settings
        if {$selftest(debug)} {puts "Restoring settings"}
        eval $Settings
        unset Settings
        after 500
        KeypadMain
        unset StopAllTests ;# I only want this to exist if the test is running
        if {$selftest(debug)} {puts "RunSelected: End"}
    }
    
    #########################################################################
    #       Name: RunAll
    # Parameters:
    #Description: Selects all tests, and calls RunSelected.
    #
    #########################################################################
    proc RunAll {} \
    {
        variable NumTests
        variable TestCheck
        variable StopOnFail
        variable selftest
        
        if {$selftest(debug)} {puts "RunAll:"}
        set StopOnFail 0
        
        for {set x 0} {$x < $NumTests} {incr x} {
            set TestCheck($x) 1
        }
        RunSelected
        set StopOnFail 1
    }
    
    #########################################################################
    #       Name: Selftest
    # Parameters: Test
    #Description: Runs test number 'Test' and returns 1 for pass, or 0 for
    #             fail. This is used to run a test remotely without prompting.
    #
    #########################################################################
    proc Selftest {Test} \
    {
        variable NumTests
        variable TestCheck
        variable TestStatus
        variable StopOnFail
        variable selftest
        
        if {$selftest(debug)} {puts "Selftest:"}
        Screen Selftest
        ResetAll
        if {[string match [string toupper $Test] "ALL"]} \
        {
            # Run all tests
            set StopOnFail 0
            set selftest(remote) 1 ;# I'm running under remote control
            SelectAll
            RunSelected
            set selftest(remote) 0 ;# I'm done running under remote control
            set StopOnFail 1
            set TStatus 1 ;# Assume all tests passed
            for {set tests 0} {$tests<$NumTests} {incr tests} \
            {
                if {[string match $TestStatus($tests) "*Failed*"]} \
                {set TStatus 0 } ;# if any test fails TStatus will be 0
            }
            return $TStatus
        } \
        elseif {$Test < $NumTests} \
        {
            set StopOnFail 0
            set TestCheck($Test) 1
            set selftest(remote) 1 ;# I'm running under remote control
            RunSelected
            set selftest(remote) 0 ;# I'm done running under remote control
            set StopOnFail 1
            if {[string match $TestStatus($Test) "Passed"]} {return 1} else {return 0}
        }
        return 0
    }
    
    #########################################################################
    #       Name: GetMax, GetMin
    # Parameters: value1 value2
    #Description: GetMax: Returns the greater of value1, and value2
    #             GetMin: Returns the lesser of value1, and value2
    #
    #########################################################################
    proc GetMax {value1 value2} \
    { ;# Return the greater of value1, and value2
        if {$value1 > $value2} {return $value1} else {return $value2}
    }
    
    proc GetMin {value1 value2} \
    { ;# Return the lesser of value1, and value2
        if {$value1 < $value2} {return $value1} else {return $value2}
    }
    
    #########################################################################
    #       Name: GetState
    # Parameters:
    #Description: returns state of 'TestNum' Choices are:
    #             0 - *Failed*
    #             1 - Passed
    #             2 - Incomplete
    #             3 - Running...
    #             4 - ... (for testing only)
    #             5 - Not Tested (default)
    #             6 - Aborting...
    #
    #########################################################################
    proc GetState {Status} \
    {
        variable selftest
        
        if {$selftest(debug)} {puts "GetState: Status = $Status"}
        
        switch $Status \
        {
            "*Failed*" {format 0}
            "Passed" {format 1}
            "Incomplete" {format 2}
            "Running..." {format 3}
            "..." {format 4}
            "Not Tested" {format 5}
            "No Test" {format 6}
        }
    }
    
    #########################################################################
    #       Name: InitSelftest
    # Parameters:
    #Description: Sets the number of tests, and fills the TestName array with
    #             the test names, and the RunTest array with the test
    #             procedure names. Also sets some other variables.
    #
    #########################################################################
    proc InitSelftest {} \
    {
        variable NumTests
        variable TestName
        variable TestStatus
        variable TestCheck
        variable StopOnFail
        variable selftest
        variable tile_path
        variable RunTest
        
        # In the old UI, this function was run every time the selftest screen
        # was displayed. I only wanted to set these variables once, since they
        # reset the tests to defaults. So I created the selftest(load) variable
        # to keep track of whether the tests had been initialized. If the
        # variable exists then the tests have been initialized so don't waste
        # time doing it again (this also had the side effect of remembering the
        # test button states, and the test status whenever the self test screen
        # was displayed). In the new UI, this function is only run when the
        # file is sourced at start up, so there isn't a need for this check.
        
        if {! [info exists selftest(load)]} \
        { ;# load self test menu
            set selftest(debug) $::debug_print ;# Set debug printing to system setting
            if {$selftest(debug)} {puts "InitSelftest: This is the first time!"}
            set tile_path ""
            set selftest(lastfocus) 0
            set selftest(cmddly) 10 ;# Delay between groups of commands (10 mS)
            set selftest(TXPort) 0  ;# T/R Port for Generating by default
            set selftest(RXPort) 0  ;# T/R Port for Receiving by default
            set selftest(speaker) 1 ;# Audio to Speaker on by default
            set selftest(audout1) 0  ;# Audio to Audio out 1 off by default
            set selftest(audout2) 0  ;# Audio to Audio out 2 off by default
            set selftest(level) 1   ;# Used to set various levels of testing
            set selftest(remote) 0  ;# Used to run selftest remotely
            set StopOnFail 1
            
            set NumTests 10 ;# holds number of active tests
            array set TestName \
            { ;# holds name of the tests
                0 "Lock Test"
                1 "RF Error Meter Test"
                2 "Modulation Meter Test"
                3 "Audio Frequency Counter Test"
                4 "Sinad Meter Test"
                5 "Distortion Meter Test"
                6 "Receiver Sensitivity Test"
                7 "Analyzer Test"
                8 "Oscilloscope Test"
                9 "Project 25 Test"
            }
            array set RunTest \
            { ;# holds proc names of tests
                0 T00Lock
                1 T01RfError
                2 T02Modulation
                3 T03AFCounter
                4 T04Sinad
                5 T05Distortion
                6 T06RcvrSens
                7 T07Analyzer
                8 T08Scope
                9 T09P25
            }
            for {set x 0} {$x < $NumTests} {incr x} \
            { ;# Initialize main menu
                set TestStatus($x) "Not Tested"
                set TestCheck($x) 0
            }
            set selftest(load) 1 ;# the selftest menu has been loaded
        }
    }
    
    #########################################################################
    #       Name: ChgBit
    # Parameters: Value Bit State
    #Description: Sets or Clears a bit in 'Value'. 'Bit' is the bit to Set or
    #             Clear. If State = 0, the bit is cleared otherwise it is set.
    #
    #########################################################################
    proc ChgBit {Value Bit State} \
    {
        variable selftest
        
        if {$selftest(debug)} {puts "ChgBit: $Value $Bit $State"}
        
        if {$State != 1} {set newValue [expr $Value & ~ int(pow(2, $Bit))]} \
        else {set newValue [expr $Value | int(pow(2, $Bit))]}
        return $newValue ;# Returns the value of newValue
    }
    
    #########################################################################
    #       Name: SetStatus
    # Parameters: State
    #Description: Sets status field TestNum to the given State. Choices are:
    #             0 - *Failed* on a red background
    #             1 - Passed on a green background
    #             2 - Incomplete on a yellow background
    #             3 - Running... on a white background
    #             4 - ... on a white background (for testing only)
    #             X - Not Tested on a gray background (default)
    #
    #########################################################################
    proc SetStatus {State} \
    {
        variable TestStatus
        variable tnf
        variable TestNum
        variable selftest
        
        if {$selftest(debug)} {puts "SetStatus: State = $State"}
        
        switch $State \
        {
            0 { ;# Set Status to *Failed* on a red background
                $tnf.status($TestNum) configure -background red
                set TestStatus($TestNum) {*Failed*}
            }
            1 { ;# Set Status to Passed on a green background
                $tnf.status($TestNum) configure -background green
                set TestStatus($TestNum) {Passed}
            }
            2 { ;# Set Status to Incomplete on a yellow background
                $tnf.status($TestNum) configure -background yellow
                set TestStatus($TestNum) {Incomplete}
            }
            3 { ;# Set Status to Running... on a white background
                $tnf.status($TestNum) configure -background white
                set TestStatus($TestNum) {Running...}
            }
            4 { ;# Set Status to ... on a white background (for testing)
                $tnf.status($TestNum) configure -background white
                set TestStatus($TestNum) {...}
            }
            5 { ;# Set Status to Not Tested on a gray background
                $tnf.status($TestNum) configure -background gray
                set TestStatus($TestNum) {Not Tested}
            }
            6 { ;# Set Status to No Test on a blue background
                $tnf.status($TestNum) configure -background lightblue
                set TestStatus($TestNum) {No Test}
            }
            7 { ;# Set Status to Aborting... on an orange background
                $tnf.status($TestNum) configure -background orange
                set TestStatus($TestNum) {Aborting...}
            }
        }
    }
    
    #########################################################################
    #       Name: CheckLimits
    # Parameters: Measured, Lower, Upper, Units
    #Description: Compares 'Measured' to 'Lower', and 'Upper' and displays
    #             the Retry, Continue, or Abort dialog box.
    #             If 'Upper' is None then 'Measured' is checked to be > 'Lower'
    #             If 'Lower' is None then 'Measured' is checked to be < 'Upper'
    #
    #########################################################################
    proc CheckLimits {Measured Lower Upper Units} \
    {
        variable TestName
        variable StopAllTests
        variable Finished
        variable selftest
        variable TestNum
        variable StopOnFail
        variable retval
        
        if {$selftest(debug)} {puts "CheckLimits: $Measured $Lower $Upper"}
        
        set retval 1 ;# Default to continue... gets set to: 0 - retry, 1 - continue, or 3 - abort
        set PF 0     ;# Default to Fail
        if {[string match $Upper None]} \
        {
            if {$Measured <= $Lower} \
            {
                # Test Failed
                if {$StopOnFail} \
                {
                    set selftest(lastfocus) [focus]
                    set Msg [format "%s has failed.\nMeasured: %2.3f %s\nLimit: >%2.3f" \
                    $TestName($TestNum) $Measured $Units $Lower]
                    FailDial $Msg
                    set_focus .
                    set_focus $selftest(lastfocus)
                }
            } \
            else {set PF 1}
        } \
        elseif {[string match $Lower None]} \
        {
            if {$Measured >= $Upper} \
            {
                # Test Failed
                if {$StopOnFail} \
                {
                    set selftest(lastfocus) [focus]
                    set Msg [format "%s has failed.\nMeasured: %2.3f %s\nLimit: <%2.3f" \
                    $TestName($TestNum) $Measured $Units $Upper]
                    FailDial $Msg
                    set_focus .
                    set_focus $selftest(lastfocus)
                }
            } \
            else {set PF 1}
        } \
        else \
        {
            if {$Measured < $Lower || $Measured > $Upper} \
            {
                # Test Failed
                if {$StopOnFail} \
                {
                    set selftest(lastfocus) [focus]
                    set Msg [format "%s has failed.\nMeasured: %2.3f %s\nLimit: %2.3f to %2.3f" \
                    $TestName($TestNum) $Measured $Units $Lower $Upper]
                    FailDial $Msg
                    set_focus .
                    set_focus $selftest(lastfocus)
                }
            } \
            else {set PF 1}
        }
        # retval = 0 for Fail/Retry
        if {$retval == 1} {set Finished 1} ;# Pass or Fail/Continue
        if {$retval == 2} {AbortTest}      ;# Fail/Abort
        unset retval
        return $PF ;# Returns 0 for fail, or 1 for pass
    }
    
    #########################################################################
    #       Name: Screen
    # Parameters: ScreenName
    #Description: Switch to screen 'screen'.  Values for 'screen' can be...
    #             Analyzer, Selftest, Gen, Rec, Dup, Scope, AFGen, Config,
    #             TimeDate, Version, Reload.  Also "_Screen" may be appended.
    #
    #########################################################################
    proc Screen {ScreenName} \
    {
        variable TestNum      ;# This gets reset when screen = selftest
        variable Finished     ;# This gets set when we call another screen
        variable StopAllTests ;# This gets set when we call another screen
        variable tile_path
        variable selftest
        
        # did we append _Screen to the screen name?
        if {[string first "_Screen" $ScreenName] == -1} \
        {set ScreenName [join "$ScreenName Screen" "_"]}
        if {$selftest(debug)} {puts "Screen: Switching to screen: $ScreenName"}
        if {[info exists StopAllTests]} \
        { ;# Self Test must be running
            set restoreme 1
            set Finny $Finished   ;# Temporarily save the value of Finished
            set SAT $StopAllTests ;# Temporarily save the value of StopAllTests
            set TN $TestNum       ;# Temporarily save our current test number
        } \
        else {set restoreme 0}
        set Screens::Local_screen $ScreenName
        Screens::Show_local
        after 2000 ;# Give screen a couple of seconds to draw
        
        if {$restoreme} \
        {
            set Finished $Finny   ;# Recall Finished
            set StopAllTests $SAT ;# Recall StopAllTests
            set TestNum $TN       ;# Recall TestNum
            # This is handled elsewhere now.
            # if {[string match [string tolower $ScreenName] "selftest_screen"]} \
            # {
            #     KeypadAbort
            # }
        }
    }
    
    proc Show {ScreenName} {
        set Screens::Local_screen $ScreenName
        Screens::Show_local
    }
    
    #########################################################################
    #       Name: NormAn
    # Parameters:
    #Description: This routine normalizes the Analyzer. The unit must be in
    #             the full analyzer screen to work. Returns 1 for success,
    #             or 0 for failure.
    #
    #########################################################################
    proc NormAn {} \
    { ;# Normalize the Analyzer
        variable selftest
        
        if {$selftest(debug)} {puts "NormAn: Begin"}
        
        Analyzer norm 1
        after 2000
        set norm [Analyzer norm]
        while {$norm != 8 && $norm != 9} {
            set norm [Analyzer norm]
            after 1000
        }
        if {$selftest(debug)} {puts "NormAn: End"}
        if {$norm == 8} {return 1} else {return 0}
    }
    
    #########################################################################
    #       Name: Spaces
    # Parameters: num
    #Description: returns a string of 'num' spaces
    #
    #########################################################################
    proc Spaces {num} \
    {
        variable selftest
        
        set space " "
        set temp $space
        for  {set x 1} {$x < $num} {incr x} {append temp $space}
        return $temp
    }
    
    #########################################################################
    #       Name: KeypadAbort
    # Parameters:
    #Description: Creates the 'Abort' set of softkeys
    #
    #########################################################################
    proc KeypadAbort {} \
    { ;# Set up Abort softkey
        variable tile_path
        variable option_menu
        variable selftest
        
        if {$selftest(debug)} {puts "KeypadAbort:"}
        ::View::ClearAllSoftkeys $tile_path
        after 100
        
        set ::sys(powerdown_allowed) 0
        [View::GetMainMenuName $tile_path] configure -state disabled
        $option_menu configure -state disabled
        after 100
        ::View::AssignSoftkey $tile_path 2 "Abort\nTesting" "::Selftest_Screen::AbortTest"
    }
    
    #########################################################################
    #       Name: KeypadMain
    # Parameters:
    #Description: Creates the 'Main' set of softkeys
    #
    #########################################################################
    proc KeypadMain {} \
    { ;# Set up Main softkeys
        variable tile_path
        variable option_menu
        variable selftest
        
        if {$selftest(debug)} {puts "KeypadMain:"}
        if {$selftest(active)} \
        { ;# only draw softkeys when we are active
            ::View::ClearAllSoftkeys $tile_path
            after 100
            
            set ::sys(powerdown_allowed) 1
            [View::GetMainMenuName $tile_path] configure -state normal
            $option_menu configure -state normal
            after 100
            ::View::AssignSoftkey $tile_path 2 "Run All" "::Selftest_Screen::RunAll"
            ::View::AssignSoftkey $tile_path 3 "Run\nSelected" "::Selftest_Screen::RunSelected"
            ::View::AssignSoftkey $tile_path 4 "Select All" "::Selftest_Screen::SelectAll"
            ::View::AssignSoftkey $tile_path 5 "Reset" "::Selftest_Screen::ResetAll"
            ::View::AssignSoftkey $tile_path 6 "Display\nMemory" "set ::Selftest_Screen::CallMemory 1; ::Selftest_Screen::Show Memory_Screen"
            ::View::AssignSoftkey $tile_path 7 "Display\nResults" "::Selftest_Screen::Show Results_Screen"
            ::View::AssignReturn $tile_path "" ;# disable Return key.
        }
    }
    
    #########################################################################
    #       Name: MakeHeader
    # Parameters: header, type
    #Description: Takes the string passed to it, and adds 'pad' characters
    #             to it, and formats it for writing to the results file
    #
    #########################################################################
    proc MakeHeader {header type} \
    {
        variable selftest
        
        switch $type \
        {
            1 {
                set lead "+ + + + "
                set trail " + + + +"
                set final $lead[MakeString $header 50 center]$trail
            }
            2 {
                # set lead "- - "
                # set trail " - -"
                set final [MakeString $header 66 center]
            }
            3 {
                set lead "* * * * * * * * "
                set trail " * * * * * * * *"
                set final $lead[MakeString $header 34 center]$trail
            }
        }
        if {$selftest(debug)} {puts $final} ;# If debug is 1 then send out.
        return $final ;# returns the value of final
    }
    
    #########################################################################
    #        Name: MakeString
    #  Parameters: str, len, just
    # Description: returns a string that is 'len' characters long with 'str'
    #              justified center, left, or right, and padded with spaces.
    #
    #########################################################################
    proc MakeString {str len just} \
    {
        variable selftest
        
        # if {$selftest(debug)} {puts "MakeString:"}
        
        # Convert 'just' to one upper case character (Only the first character
        # is needed to determine centered, left, or right justification).
        set just [string toupper [string index $just 0]]
        set lengthy [string length $str]
        if {[expr {$len - $lengthy}] > 0} \
        {
            switch $just \
            {
                C { ;# Justify Center
                    set str1 [Spaces [expr int(($len - $lengthy) / 2.0)]]
                    if {[expr ($len - $lengthy) / 2.0] == [expr int(($len - $lengthy) / 2.0)]} \
                    {
                        # string length is even, so spaces are equal
                        set str2 $str1
                    } \
                    else \
                    {
                        # string length is odd, so add an extra space on the right
                        set str2 [Spaces [expr [string length $str1] + 1]]
                    }
                    set final $str1$str$str2
                }
                L { ;# Justify Left
                    set final $str[Spaces [expr $len - $lengthy]]
                }
                R { ;# Justify Right
                    set final [Spaces [expr $len - $lengthy]]$str
                }
                default { ;# Illegal justify parameter, return 'str' as is
                    set final $str
                }
            }
        } \
        else \
        {
            # If str is equal to, or longer than, the requested field length
            # return str without doing anything to it.
            set final $str
        }
        return $final ;# returns the value of final
    }
    
    #########################################################################
    #        Name: ShowLockStatus
    #  Parameters:
    # Description: Creates the dialog box for a failure during the Lock Test
    #
    #########################################################################
    proc ShowLockStatus {} \
    {
        variable Finished
        variable tile_path
        variable selftest
        variable LockState
        
        if {$selftest(debug)} {puts "ShowLockStatus:"}
        set ts $tile_path.test
        toplevel $ts
        wm withdraw $ts
        wm title $ts "Lock Status"
        wm resizable $ts 0 0
        label $ts.title -text "The Lock Status Test has Failed:"
        label $ts.null1 -text " "
        label $ts.null2 -text " "
        label $ts.gen_main -text "Generator Main"
        label $ts.gen_main_status -text $LockState(0)
        label $ts.gen_ref -text "Generator Reference"
        label $ts.gen_ref_status -text $LockState(1)
        label $ts.gen_clean -text "Generator Clean"
        label $ts.gen_clean_status -text $LockState(2)
        label $ts.gen_step -text "Generator Step"
        label $ts.gen_step_status -text $LockState(3)
        label $ts.rec_main -text "Receiver Main"
        label $ts.rec_main_status -text $LockState(4)
        label $ts.rec_ref -text "Receiver Reference"
        label $ts.rec_ref_status -text $LockState(5)
        label $ts.rec_clean -text "Receiver Clean"
        label $ts.rec_clean_status -text $LockState(6)
        label $ts.rec_step -text "Receiver Step"
        label $ts.rec_step_status -text $LockState(7)
        frame $ts.bframe
        button $ts.bframe.abort -text "Abort" -command "::Selftest_Screen::AbortTest; destroy $ts"
        button $ts.bframe.retry -text "Retry" -command "destroy $ts"
        pack $ts.bframe.retry $ts.bframe.abort -side left -pady 5 -padx 20
        grid $ts.title -columnspan 2
        grid $ts.null1 -columnspan 2
        grid $ts.gen_main $ts.gen_main_status -sticky w -padx 50
        grid $ts.gen_ref $ts.gen_ref_status -sticky w -padx 50
        grid $ts.gen_clean $ts.gen_clean_status -sticky w -padx 50
        grid $ts.gen_step $ts.gen_step_status -sticky w -padx 50
        grid $ts.rec_main $ts.rec_main_status -sticky w -padx 50
        grid $ts.rec_ref $ts.rec_ref_status -sticky w -padx 50
        grid $ts.rec_clean $ts.rec_clean_status -sticky w -padx 50
        grid $ts.rec_step $ts.rec_step_status -sticky w -padx 50
        grid $ts.null2 -columnspan 2
        grid $ts.bframe -columnspan 2
        update idle
        set winx [expr ([winfo width $tile_path] - [winfo reqwidth $ts]) / 2]
        set winy [expr ([winfo height $tile_path] - [winfo reqheight $ts]) / 2]
        wm geometry $ts +$winx+$winy
        wm deiconify $ts
        set_focus $ts.bframe.retry
        tkwait window $ts
    }
    
    #########################################################################
    #        Name: FailDial
    #  Parameters: Message
    # Description: Creates the 'Retry', 'Continue', 'Abort' dialog box
    #
    #########################################################################
    proc FailDial {Message} \
    {
        variable tile_path
        variable selftest
        
        if {$selftest(debug)} {puts "FailDial:"}
        set ts .faildial
        toplevel $ts
        wm withdraw $ts
        wm title $ts "Test Failed"
        wm resizable $ts 0 0
        label $ts.title -text $Message
        frame $ts.bframe
        button $ts.bframe.retry -text "Retry" -command "set ::Selftest_Screen::retval 0; destroy $ts"
        button $ts.bframe.continue -text "Continue" -command "set ::Selftest_Screen::retval 1; destroy $ts"
        button $ts.bframe.abort -text "Abort" -command "set ::Selftest_Screen::retval 2; destroy $ts"
        pack $ts.title $ts.bframe -side top -padx 20 -pady 10
        pack $ts.bframe.retry $ts.bframe.continue $ts.bframe.abort -side left -padx 30
        update idle
        set winx [expr ([winfo width $tile_path] - [winfo reqwidth $ts]) / 2]
        set winy [expr ([winfo height $tile_path] - [winfo reqheight $ts]) / 2]
        wm geometry $ts +$winx+$winy
        wm deiconify $ts
        set_focus $ts.bframe.retry
        tkwait window $ts
    }
    
    #########################################################################
    #        Name: AbortTest
    #  Parameters:
    # Description: Called during testing whenever the Abort softkey is pressed
    #
    #########################################################################
    proc AbortTest {} \
    {
        variable StopAllTests
        variable Finished
        variable selftest
        variable TestNum
        variable NumTests
        variable tile_path
        variable option_menu
        
        if {$selftest(debug)} {puts "AbortTest:"}
        
        set StopAllTests 1
        set Finished 1
        # only change the status if a test was run
        if {$TestNum != $NumTests} {SetStatus 7} ;# Change status to 'Aborting...'
        set ::sys(powerdown_allowed) 1
        [View::GetMainMenuName $tile_path] configure -state normal
        $option_menu configure -state normal
    }
    
    #########################################################################
    #        Name: ConnectPrompt
    #  Parameters:
    # Description: Creates the connection instruction dialog box
    #
    #########################################################################
    proc ConnectPrompt {Message} \
    {
        variable tile_path
        variable selftest
        
        if {$selftest(debug)} {puts "ConnectPrompt:"}
        set ts .connect
        toplevel $ts
        wm withdraw $ts
        wm title $ts "Connections"
        wm resizable $ts 0 0
        label $ts.title -text $Message
        frame $ts.bframe
        button $ts.bframe.abort -text "Abort" -command "set ::Selftest_Screen::StopAllTests 1; set ::Selftest_Screen::Finished 1; destroy $ts"
        button $ts.bframe.continue -text "Continue" -command "destroy $ts"
        pack $ts.title $ts.bframe -side top -padx 20 -pady 10
        pack $ts.bframe.continue $ts.bframe.abort -side left -padx 30
        update idle
        set winx [expr ([winfo width $tile_path] - [winfo reqwidth $ts]) / 2]
        set winy [expr ([winfo height $tile_path] - [winfo reqheight $ts]) / 2]
        wm geometry $ts +$winx+$winy
        wm deiconify $ts
        set_focus $ts.bframe.continue
        tkwait window $ts
    }
    
    #########################################################################
    #        Name: WriteStartEndHead
    #  Parameters: StartEnd
    # Description: Writes the Start, or End header to the results file
    #
    #########################################################################
    proc WriteStartEndHead {StartEnd} \
    { ;# Write header for start and end of test to file
        variable TestNum
        variable TestName
        variable TestStatus
        variable ResultsId
        variable selftest
        
        # if {$selftest(debug)} {puts "WriteStartEndHead:"}
        switch $StartEnd \
        {
            0 { ;# Write header for start of test to file
                puts $ResultsId [MakeHeader $TestName($TestNum) 1]
            }
            1 { ;# Write header for end of test to file
                puts $ResultsId ""
                puts $ResultsId [MakeHeader "$TestName($TestNum): $TestStatus($TestNum)" 1]
            }
        }
        puts $ResultsId ""
    }
    
    ########################################################################
    #       Name: TxxName
    # Parameters:
    #Description: Runs Test #xx
    ########################################################################
    proc T00Lock {} \
    { ;# Lock Status Test
        variable Finished
        variable StopAllTests
        variable TestNum
        variable ResultsId
        variable TestStatus
        variable TestName
        variable StopOnFail
        variable selftest
        variable LockState
        
        if {$selftest(debug)} {puts "T00Lock: Start"}
        # Test code goes here
        set Finished 0
        set MasterPF 1 ;# Default to Pass
        while {! $Finished} {
            # Display 'Running' Status
            SetStatus 3
            #
            # ***** Begin RCI test code. *****
            #
            set LockState(0) $Generator::State(lock,main)
            if {$LockState(0) != "LOCKED"} {set MasterPF 0}
            set LockState(1) $Generator::State(lock,ref)
            if {$LockState(1) != "LOCKED"} {set MasterPF 0}
            set LockState(2) $Generator::State(lock,clean)
            if {$LockState(2) != "LOCKED"} {set MasterPF 0}
            set LockState(3) $Generator::State(lock,step)
            if {$LockState(3) != "LOCKED"} {set MasterPF 0}
            set LockState(4) $::Receiver::State(lock,main)
            if {$LockState(4) != "LOCKED"} {set MasterPF 0}
            set LockState(5) $::Receiver::State(lock,ref)
            if {$LockState(5) != "LOCKED"} {set MasterPF 0}
            set LockState(6) $::Receiver::State(lock,clean)
            if {$LockState(6) != "LOCKED"} {set MasterPF 0}
            set LockState(7) $::Receiver::State(lock,step)
            if {$LockState(7) != "LOCKED"} {set MasterPF 0}
            #
            # ***** End RCI test code. *****
            #
            if {$MasterPF == 0} \
            {
                if {$StopOnFail} \
                {
                    set selftest(lastfocus) [focus]
                    ShowLockStatus
                    set_focus .
                    set_focus $selftest(lastfocus)
                } \
                else {set Finished 1}
                SetStatus 0
                set MasterPF 1 ;# Reset MasterPF for another try
            } \
            else \
            {
                SetStatus 1
                set Finished 1
            }
        }
        # Write results to results.txt
        WriteStartEndHead 0
        puts $ResultsId [MakeHeader "    Generator                  Receiver     " 2]
        puts $ResultsId [MakeHeader "-----------------          -----------------" 2]
        puts $ResultsId [MakeHeader "Main:      $LockState(0)          Main:      $LockState(4)" 2]
        puts $ResultsId [MakeHeader "Reference: $LockState(1)          Reference: $LockState(5)" 2]
        puts $ResultsId [MakeHeader "Clean:     $LockState(2)          Clean:     $LockState(6)" 2]
        puts $ResultsId [MakeHeader "Step:      $LockState(3)          Step:      $LockState(7)" 2]
        WriteStartEndHead 1
        if {$selftest(debug)} {puts "T00Lock: End"}
    }
    
    proc T01RfError {} \
    { ;# RF Error Meter Test
        variable Finished
        variable StopAllTests
        variable TestNum
        variable TestStatus
        variable TestName
        variable ResultsId
        variable selftest
        
        if {$selftest(debug)} {puts "T01RfError: Start"}
        # Test code goes here
        # This is a 3 part test... each pass/fail status is recorded in 1 bit of
        # MasterPF a pass is a 1, and a fail is a 0. If MasterPF = 7 at the end
        # then all 3 tests passed.
        set MasterPF 0 ;# set MasterPF to all fails at the start
        set Units {Hz}
        switch $selftest(level) \
        {
            0 {
                set loops 1
                set ExpectPF 1
                array set Expect {0 10000}
                array set RFreq {0 499.99}
                array set GFreq {0 500.00}
            }
            1 {
                set loops 3
                set ExpectPF 7
                # Fill array with expected frequency errors
                array set Expect {0 0 1 10000 2 -10000}
                # Fill array with Receive Frequencies
                array set RFreq {0 500.00 1 499.99 2 500.01}
                # Fill array with Generate Frequencies
                array set GFreq {0 500.00 1 500.00 2 500.00}
            }
        }
        for {set x 0} {$x < $loops} {incr x} {
            set Reading($x) "0"
            set Result($x) "Incomplete"
        }
        # Display 'Running' Status
        SetStatus 3
        Generator ptt on ;# Turn on Generator output
        #
        # RCI test code goes here.
        #
        # ***** Generator Gen port set up *****
        #
        Generator rflev -30
        after $selftest(cmddly)
        Generator rfout $selftest(TXPort)  ;# set port
        Generator mod off                  ;# gen modulation off
        after $selftest(cmddly)
        Receiver rfin $selftest(RXPort)
        Receiver atten 0
        after $selftest(cmddly)
        Receiver demod off                 ;# Rec demod Off
        Receiver ifbw 30 kHz               ;# Rec if rbw hardware
        after $selftest(cmddly)
        RFError en enable
        RFError run on
        after $selftest(cmddly)
        RFError gate tenth
        RFError avg 10
        after $selftest(cmddly)
        for {set loop 0} {$loop < $loops} {incr loop} \
        {
            if {$StopAllTests} {break}
            set Finished 0
            while {! $Finished} \
            {
                set Lower [expr $Expect($loop) - 10]
                set Upper [expr $Expect($loop) + 10]
                Generator freq $GFreq($loop)
                Receiver freq $RFreq($loop)
                after 5000
                if {$StopAllTests} {break}
                set Reading($loop) [RFError value]
                if {[CheckLimits $Reading($loop) $Lower $Upper $Units]} \
                {
                    set Result($loop) "Passed"
                    set MasterPF [ChgBit $MasterPF $loop 1] ;# Set bit for pass
                } \
                else \
                {
                    set Result($loop) "*Failed*"
                    set MasterPF [ChgBit $MasterPF $loop 0] ;# Clear bit for fail
                }
            }
        }
        RFError run off
        RFError en disable
        after $selftest(cmddly)
        Generator ptt off ;# Turn off Generator output
        if {! $StopAllTests} \
        {
            if {$MasterPF == $ExpectPF } {SetStatus 1} else {SetStatus 0}
        } \
        else {SetStatus 2}
        # Write results to results.txt
        WriteStartEndHead 0
        set head1 "  Gen       Recv      Expected     Actual        Test   "
        set head2 " Freq       Freq      RF Error    RF Error      Status  "
        set head3 " (MHz)      (MHz)       (Hz)        (Hz)         (P/F)  "
        set dashes "-------    -------    --------    --------    ----------"
        puts $ResultsId [MakeHeader $head1 2]
        puts $ResultsId [MakeHeader $head2 2]
        puts $ResultsId [MakeHeader $head3 2]
        puts $ResultsId [MakeHeader $dashes 2]
        for {set x 0} {$x < $loops} {incr x} \
        {
            set res [format {%7.3f    %7.3f    %7.0f     %7.0f     %s} \
            $GFreq($x) $RFreq($x) $Expect($x) $Reading($x) [MakeString $Result($x) 10 center]]
            puts $ResultsId [MakeHeader $res 2]
        }
        WriteStartEndHead 1
        if {$selftest(debug)} {puts "T01RfError: End"}
    }
    
    proc T02Modulation {} \
    { ;# Modulation Meter Test
        variable Finished
        variable StopAllTests
        variable TestNum
        variable TestName
        variable TestStatus
        variable ResultsId
        variable selftest
        
        if {$selftest(debug)} {puts "T02Modulation: Start"}
        # Test code goes here
        # This is a 3 part test... each pass/fail status is recorded in 1 bit of
        # MasterPF a pass is a 1, and a fail is a 0. If MasterPF = 7 at the end
        # then all 3 tests passed.
        set Units {kHz}
        switch $selftest(level) \
        {
            0 {
                set loops 1
                set ExpectPF 1
                array set Expect {0 25.00}
            }
            1 {
                set loops 6
                set ExpectPF 63
                # Fill array with our expected deviations
                array set Expect {0 5.00 1 10.00 2 25.00 3 10.00 4 20.00 5 50.00}
            }
        }
        for {set x 0} {$x < $loops} {incr x} \
        {
            set Reading($x) "0"
            set Result($x) "Incomplete"
        }
        set spec 2 ;# This is the FM spec
        Generator ptt on ;# Turn on Generator output
        # Display 'Running' Status
        SetStatus 3
        set MasterPF 0 ;# set MasterPF to all fails at the start
        #
        # RCI test code goes here.
        #
        # ***** Generator Setup *****
        Generator freq 700.00
        after $selftest(cmddly)
        Generator rflev -30
        Generator rfout $selftest(TXPort)   ;# set port
        after $selftest(cmddly)
        Generator mod fmn                   ;# gen FM
        FGen mod3 tone                      ;# FGen3 mode TONE
        after $selftest(cmddly)
        FGen freq3 1000
        FGen shape3 sine
        after $selftest(cmddly)
        BBAudio mix Fgen3 L1 on             ;# Fgen3 to layer 1
        #
        # ***** Receiver Setup *****
        Receiver freq 700.00
        after $selftest(cmddly)
        Receiver rfin $selftest(RXPort)
        Receiver atten 0
        after $selftest(cmddly)
        Receiver demod fmwideband           ;# Rec demod type/digital filter 2=20Khz
        Receiver ifbw 30 kHz                ;# Rec if rbw hardware
        after $selftest(cmddly)
        FMDev trigger continuous
        FMDev en enable
        after $selftest(cmddly)
        FMDev run on
        FMDev math average
        after $selftest(cmddly)
        FMDev avg 100
        if {$selftest(speaker)} {BBAudio mix L1 spkr on}
        after $selftest(cmddly)
        if {$selftest(audout1)} {BBAudio mix L1 fgen on}
        if {$selftest(audout2)} {BBAudio mix L1 demod on}
        after $selftest(cmddly)
        for {set loop 0} {$loop < $loops} {incr loop} \
        {
            if {$StopAllTests} {break}
            set Finished 0
            if {$loop == 3} \
            { ;# Switch to AM
                Generator mod am     ;# gen AM
                Receiver demod am    ;# demod AM
                set spec 4           ;# This is the AM spec
                set Units "%"
            }
            while {! $Finished} \
            {
                set Lower [expr $Expect($loop) - $spec]
                set Upper [expr $Expect($loop) + $spec]
                if {$loop < 3} {FGen dev3 $Expect($loop)} \
                else {FGen per3 $Expect($loop)}
                after 5000
                if {$StopAllTests} {break}
                set Reading($loop) [FMDev value]
                if {[CheckLimits $Reading($loop) $Lower $Upper $Units]} \
                {
                    set Result($loop) "Passed"
                    set MasterPF [ChgBit $MasterPF $loop 1] ;# Set bit for pass
                } \
                else \
                {
                    set Result($loop) "*Failed*"
                    set MasterPF [ChgBit $MasterPF $loop 0] ;# Clear bit for fail
                }
            }
        }
        FMDev en disable          ;# disable the deviation meter
        FMDev run off             ;# Stop taking deviation measurements
        after $selftest(cmddly)
        BBAudio mix L1 spkr off   ;# Clear Layer 1 to Speaker
        BBAudio mix L1 fgen off   ;# Clear Layer 1 to Audio out 1
        after $selftest(cmddly)
        BBAudio mix L1 demod off  ;# Clear Layer 1 to Audio out 2
        BBAudio mix Fgen3 L1 off  ;# Clear Fgen3 to layer 1
        after $selftest(cmddly)
        FGen mod3 off             ;# Turn off the FGen we used
        Receiver demod off        ;# Turn off demod
        after $selftest(cmddly)
        Generator ptt off         ;# Turn off the Generator output
        if {! $StopAllTests} \
        {
            if {$MasterPF == $ExpectPF } {SetStatus 1} else {SetStatus 0}
        } \
        else {SetStatus 2}
        # Write results to results.txt
        WriteStartEndHead 0
        set head01 "  RF                                            Test   "
        set head02 " Freq      Mod     Expected      Actual        Status  "
        set head03 " (MHz)     Type    Deviation    Deviation       (P/F)  "
        set dashes "-------    ----    ---------    ---------    ----------"
        puts $ResultsId [MakeHeader $head01 2]
        puts $ResultsId [MakeHeader $head02 2]
        puts $ResultsId [MakeHeader $head03 2]
        puts $ResultsId [MakeHeader $dashes 2]
        set Units "kHz"
        set ModType " FM "
        for {set x 0} {$x < $loops} {incr x} \
        {
            if {$x == 3} {
                set Units "%  "
                set ModType " AM "
            }
            set res [format { 700.00    %4s    %5.2f %3s    %5.2f %3s    %s} \
            $ModType $Expect($x) $Units $Reading($x) $Units [MakeString $Result($x) 10 center]]
            puts $ResultsId [MakeHeader $res 2]
        }
        WriteStartEndHead 1
        if {$selftest(debug)} {puts "T02Modulation: End"}
    }
    
    proc T03AFCounter {} \
    { ;# Audio Frequency Counter Test
        variable Finished
        variable StopAllTests
        variable TestNum
        variable TestName
        variable TestStatus
        variable ResultsId
        variable selftest
        
        if {$selftest(debug)} {puts "T03AFCounter: Start"}
        # Test code goes here
        # This is a 6 part test... each pass/fail status is recorded in 1 bit of
        # MasterPF a pass is a 1, and a fail is a 0. If MasterPF = 63 at the end
        # then all tests passed.
        set MasterPF 0 ;# set MasterPF to all fails at the start
        set Units {Hz}
        set NA "N/A"
        set RF "1000.00"
        switch $selftest(level) \
        {
            0 {
                set loops 2
                set halfloop [expr $loops / 2]
                set ExpectPF 3
                array set Expect {0 8775.00 1 8775.00}
                array set rez {0 %6.0f 1 %6.0f}
                array set RfFreq "0 $NA 1 $RF"
                array set Gate "0 tenth 1 tenth"
            }
            1 {
                set loops 6
                set halfloop [expr $loops / 2]
                set ExpectPF 63
                # Fill array with our expected frequencies
                array set Expect {0 1555.5 1 8775 2 16555 3 1555.5 4 8775 5 16555}
                # Fill array with our measurement resolutions
                array set rez {0 %6.1f 1 %6.0f 2 %6.0f 3 %6.1f 4 %6.0f 5 %6.0f}
                # Fill array with our RF Frequency (not used on first 3 tests)
                array set RfFreq "0 $NA 1 $NA 2 $NA 3 $RF 4 $RF 5 $RF"
                # Fill array with our counter gate times
                array set Gate "0 second 1 tenth 2 tenth 3 second 4 tenth 5 tenth"
            }
        }
        for {set x 0} {$x < $loops} {incr x} \
        {
            set Reading($x) "0"
            set Result($x) "Incomplete"
        }
        # Display 'Running' Status
        SetStatus 3
        # First route the FGen directly to the AF Counter
        FGen mod1 tone     ;# FGen1 mode TONE
        FGen shape1 sine
        after $selftest(cmddly)
        FGen lev1 5
        # Now route FGen1 to the AF Counter
        BBAudio route fgen1 afcounter
        after $selftest(cmddly)
        AFCounter trigger continuous
        AFCounter run on
        after $selftest(cmddly)
        AFCounter avg 5
        AFCounter en enable
        after $selftest(cmddly)
        if {$selftest(speaker)} {BBAudio mix Fgen1 spkr on}
        if {$selftest(audout1)} {BBAudio mix Fgen1 fgen on}
        after $selftest(cmddly)
        if {$selftest(audout2)} {BBAudio mix Fgen1 demod on}
        for {set loop 0} {$loop < $halfloop} {incr loop} \
        {
            if {$StopAllTests} {break}
            AFCounter gate $Gate($loop)
            set Finished 0
            while {! $Finished} \
            {
                set Lower [expr $Expect($loop) - 2]
                set Upper [expr $Expect($loop) + 2]
                FGen freq1 $Expect($loop)
                if {[string match $Gate($loop) "tenth"]} {after 4000} else {after 12000}
                if {$StopAllTests} {break}
                set Reading($loop) [AFCounter value]
                if {[CheckLimits $Reading($loop) $Lower $Upper $Units]} \
                {
                    set Result($loop) "Passed"
                    set MasterPF [ChgBit $MasterPF $loop 1] ;# Set bit for pass
                } \
                else \
                {
                    set Result($loop) "*Failed*"
                    set MasterPF [ChgBit $MasterPF $loop 0] ;# Clear bit for fail
                }
            }
        }
        FGen mod1 off                ;# Turn off Fgen 1
        BBAudio clrmix spkr          ;# Clear routing to Speaker
        after $selftest(cmddly)
        BBAudio clrmix fgen          ;# Clear routing to Audio out 1
        BBAudio clrmix demod         ;# Clear routing to Audio out 2
        after $selftest(cmddly)
        BBAudio route none afcounter ;# Remove all AF Counter inputs
        Generator ptt on             ;# Turn on Generator output
        after $selftest(cmddly)
        #
        # ***** Generator Setup *****
        Generator freq 1000.00
        Generator rflev -30
        after $selftest(cmddly)
        Generator rfout $selftest(TXPort)  ;# set port
        Generator mod fmn                  ;# gen FM
        after $selftest(cmddly)
        FGen mod3 tone                     ;# FGen3 mode TONE
        FGen shape3 sine
        after $selftest(cmddly)
        FGen dev3 25
        BBAudio mix Fgen3 L1 on            ;# Fgen3 to layer 1
        after $selftest(cmddly)
        #
        # ***** Receiver Setup *****
        Receiver freq 1000.00
        Receiver rfin $selftest(RXPort)
        after $selftest(cmddly)
        Receiver atten 0
        Receiver demod fmwideband   ;# Rec demod type/digital filter 2=20Khz
        after $selftest(cmddly)
        Receiver ifbw 30 kHz        ;# Rec if rbw hardware
        # Route Layer 1 to AF Counter
        BBAudio route L1 afcounter
        after $selftest(cmddly)
        if {$selftest(speaker)} {BBAudio mix Fgen3 spkr on}
        if {$selftest(audout1)} {BBAudio mix Fgen3 fgen on}
        after $selftest(cmddly)
        if {$selftest(audout2)} {BBAudio mix Fgen3 demod on}
        for {set loop $halfloop} {$loop < $loops} {incr loop} \
        {
            if {$StopAllTests} {break}
            AFCounter gate $Gate($loop)
            # Now use the fgen to modulate a carrier, and then measure the
            # frequency of the demodulated signal.
            set Finished 0
            while {! $Finished} \
            {
                set Lower [expr $Expect($loop) - 2]
                set Upper [expr $Expect($loop) + 2]
                FGen freq3 $Expect($loop)
                if {[string match $Gate($loop) "tenth"]} {after 4000} else {after 12000}
                if {$StopAllTests} {break}
                set Reading($loop) [AFCounter value]
                if {[CheckLimits $Reading($loop) $Lower $Upper $Units]} \
                {
                    set Result($loop) "Passed"
                    set MasterPF [ChgBit $MasterPF $loop 1] ;# Set bit for pass
                } \
                else \
                {
                    set Result($loop) "*Failed*"
                    set MasterPF [ChgBit $MasterPF $loop 0] ;# Clear bit for fail
                }
            }
        }
        AFCounter run off            ;# Stop the AF Counter
        AFCounter en disable         ;# Turn off AF Counter
        after $selftest(cmddly)
        FGen mod1 off                ;# Turn off Fgen1
        FGen mod3 off                ;# Turn off Fgen3
        after $selftest(cmddly)
        BBAudio clrmix L1            ;# Clear routing to Layer 1
        BBAudio clrmix spkr          ;# Clear routing to Speaker
        after $selftest(cmddly)
        BBAudio clrmix fgen          ;# Clear routing to Audio out 1
        BBAudio clrmix demod         ;# Clear routing to Audio out 2
        after $selftest(cmddly)
        BBAudio route none afcounter ;# Remove all AF Counter inputs
        Receiver demod off           ;# Turn off demod
        after $selftest(cmddly)
        Generator ptt off            ;# Turn off Generator output
        if {! $StopAllTests} \
        {
            if {$MasterPF == $ExpectPF } {SetStatus 1} else {SetStatus 0}
        } \
        else {SetStatus 2}
        # Write results to results.txt
        WriteStartEndHead 0
        set head01 "  RF       Expected      Actual         Test   "
        set head02 " Freq      Frequency    Frequency      Status  "
        set head03 " (MHz)       (Hz)         (Hz)          (P/F)  "
        set dashes "-------    ---------    ---------    ----------"
        puts $ResultsId [MakeHeader $head01 2]
        puts $ResultsId [MakeHeader $head02 2]
        puts $ResultsId [MakeHeader $head03 2]
        puts $ResultsId [MakeHeader $dashes 2]
        for {set x 0} {$x < $loops} {incr x} \
        {
            set res [format "%s     $rez($x)       $rez($x)      %s" \
            [MakeString $RfFreq($x) 7 center] $Expect($x) $Reading($x) [MakeString $Result($x) 10 center]]
            puts $ResultsId [MakeHeader $res 2]
        }
        WriteStartEndHead 1
        if {$selftest(debug)} {puts "T03AFCounter: End"}
    }
    
    proc T04Sinad {} \
    { ;# Sinad Meter Test
        variable Finished
        variable StopAllTests
        variable TestNum
        variable TestName
        variable TestStatus
        variable ResultsId
        variable selftest
        
        if {$selftest(debug)} {puts "T04Sinad: Start"}
        # This is a 3 part test... each pass/fail status is recorded in 1 bit of
        # MasterPF a pass is a 1, and a fail is a 0. If MasterPF = 7 at the end
        # then all tests passed.
        set Units {dB}
        switch $selftest(level) \
        {
            0 {
                set loops 1
                set ExpectPF 1
                array set Expect {0 12.3}
                array set Devi {0 10}
            }
            1 {
                set loops 3
                set ExpectPF 7
                # Fill array with our expected Sinad readings
                array set Expect {0 20.0 1 12.3 2 6.99}
                # Fill array with the fgen4 deviations
                array set Devi {0 4 1 10 2 20}
            }
        }
        for {set x 0} {$x < $loops} {incr x} \
        {
            set Reading($x) "0"
            set Result($x) "Incomplete"
        }
        # Display 'Running' Status
        SetStatus 3
        # We will use the fgens to modulate a carrier, and then measure the
        # Sinad of the demodulated signal.
        Generator ptt on ;# Turn on Generator output
        #
        # RCI test code goes here.
        #
        # ***** Generator Setup *****
        Generator freq 1800.0
        after $selftest(cmddly)
        Generator rflev -30
        Generator rfout $selftest(TXPort) ;# set port
        after $selftest(cmddly)
        Generator mod fmn                 ;# gen FM
        FGen mod3 tone                    ;# FGen3 mode TONE
        after $selftest(cmddly)
        FGen freq3 1000                   ;# FGen3 freq
        FGen shape3 sine                  ;# FGen3 shape
        after $selftest(cmddly)                         ;# Pause
        FGen dev3 40                      ;# FGen3 dev
        FGen mod4 tone                    ;# FGen4 mode TONE
        after $selftest(cmddly)
        FGen freq4 2000                   ;# FGen4 freq
        FGen shape4 sine                  ;# FGen4 shape
        after $selftest(cmddly)
        FGen dev4 4                       ;# FGen4 dev
        BBAudio mix Fgen3 L1 on           ;# Fgen3 to layer 1
        after $selftest(cmddly)                         ;# Pause
        BBAudio mix Fgen4 L1 on           ;# Fgen4 to layer 1
        #
        # ***** Receiver Setup *****
        Receiver freq 1800.0
        after $selftest(cmddly)
        Receiver rfin $selftest(RXPort)
        Receiver atten 0
        after $selftest(cmddly)
        Receiver demod fmwideband  ;# Rec demod type/digital filter 2=20Khz
        Receiver ifbw 30 kHz       ;# Rec if rbw hardware
        after $selftest(cmddly)
        BBAudio route L1 sinadmtr
        Sinad en enable
        after $selftest(cmddly)
        Sinad run on
        Sinad trigger continuous
        after $selftest(cmddly)
        Sinad math on
        Sinad Filter 5
        after $selftest(cmddly)
        Sinad Avg 5
        if {$selftest(speaker)} {BBAudio mix L1 spkr on}
        if {$selftest(audout1)} {BBAudio mix L1 fgen on}
        if {$selftest(audout2)} {BBAudio mix L1 demod on}
        set MasterPF 0 ;# set MasterPF to all fails at the start
        for {set loop 0} {$loop < $loops} {incr loop} \
        {
            if {$StopAllTests} {break}
            set Finished 0
            while {! $Finished} \
            {
                set Lower [expr $Expect($loop) - 2]
                set Upper [expr $Expect($loop) + 2]
                FGen dev4 $Devi($loop)
                after 5000
                if {$StopAllTests} {break}
                set Reading($loop) [Sinad value]
                if {[CheckLimits $Reading($loop) $Lower $Upper $Units]} \
                {
                    set Result($loop) "Passed"
                    set MasterPF [ChgBit $MasterPF $loop 1] ;# Set bit for pass
                } \
                else \
                {
                    set Result($loop) "*Failed*"
                    set MasterPF [ChgBit $MasterPF $loop 0] ;# Clear bit for fail
                }
            }
        }
        Sinad en disable            ;# Disable the Sinad meter
        Sinad run off               ;# Stop the Sinad meter
        after $selftest(cmddly)
        FGen mod3 off               ;# Turn off Fgen3
        FGen mod4 off               ;# Turn off Fgen4
        after $selftest(cmddly)
        BBAudio clrmix L1           ;# Clear routing to Layer 1
        BBAudio clrmix spkr         ;# Clear routing to Speaker
        after $selftest(cmddly)
        BBAudio clrmix fgen         ;# Clear routing to Audio out 1
        BBAudio clrmix demod        ;# Clear routing to Audio out 2
        after $selftest(cmddly)
        Receiver demod off          ;# Turn off demod
        Generator ptt off           ;# Turn off Generator output
        after $selftest(cmddly)
        if {! $StopAllTests} \
        {
            if {$MasterPF == $ExpectPF } {SetStatus 1} else {SetStatus 0}
        } \
        else {SetStatus 2}
        # Write results to results.txt
        WriteStartEndHead 0
        set head01 "  RF       Expected      Actual       Test   "
        set head02 " Freq       Sinad        Sinad       Status  "
        set head03 " (MHz)       (dB)        (dB)         (P/F)  "
        set dashes "-------    --------    --------    ----------"
        puts $ResultsId [MakeHeader $head01 2]
        puts $ResultsId [MakeHeader $head02 2]
        puts $ResultsId [MakeHeader $head03 2]
        puts $ResultsId [MakeHeader $dashes 2]
        for {set x 0} {$x < $loops} {incr x} \
        {
            set res [format "1800.00      %4.1f        %4.1f      %s" \
            $Expect($x) $Reading($x) [MakeString $Result($x) 10 center]]
            puts $ResultsId [MakeHeader $res 2]
        }
        WriteStartEndHead 1
        if {$selftest(debug)} {puts "T04Sinad: End"}
    }
    
    proc T05Distortion {} \
    { ;# Distortion Meter Test
        variable Finished
        variable StopAllTests
        variable TestNum
        variable TestName
        variable TestStatus
        variable ResultsId
        variable selftest
        
        if {$selftest(debug)} {puts "T05Distortion: Start"}
        # This is a 3 part test... each pass/fail status is recorded in 1 bit of
        # MasterPF a pass is a 1, and a fail is a 0. If MasterPF = 7 at the end
        # then all tests passed.
        set Units {%}
        switch $selftest(level) \
        {
            0 {
                set loops 1
                set ExpectPF 1
                array set Expect {0 25.0}
                array set Devi {0 10}
            }
            1 {
                set loops 3
                set ExpectPF 7
                # Fill array with our expected Sinad readings
                array set Expect {0 10.0 1 25.0 2 50.0}
                # Fill array with the fgen4 deviations
                array set Devi {0 4 1 10 2 20}
            }
        }
        for {set x 0} {$x < $loops} {incr x} \
        {
            set Reading($x) "0"
            set Result($x) "Incomplete"
        }
        # Display 'Running' Status
        SetStatus 3
        # We will use the fgens to modulate a carrier, and then measure the
        # Distortion of the demodulated signal.
        Generator ptt on ;# Turn on Generator output
        # ***** Generator Setup *****
        Generator freq 200.00
        after $selftest(cmddly)
        Generator rflev -30
        Generator rfout $selftest(TXPort) ;# set port
        after $selftest(cmddly)
        Generator mod fmn                 ;# gen FM
        FGen mod3 tone                    ;# FGen3 mode TONE
        after $selftest(cmddly)
        FGen freq3 1000
        FGen shape3 sine
        after $selftest(cmddly)
        FGen dev3 40
        FGen mod4 tone
        after $selftest(cmddly)
        FGen freq4 2000
        FGen shape4 sine
        after $selftest(cmddly)
        FGen dev4 4
        BBAudio mix Fgen3 L1 on ;# Fgen3 to layer 1
        after $selftest(cmddly)
        BBAudio mix Fgen4 L1 on ;# Fgen4 to layer 1
        #
        # ***** Receiver Setup *****
        Receiver freq 200.00
        after $selftest(cmddly)
        Receiver rfin $selftest(RXPort)
        Receiver atten 0
        after $selftest(cmddly)
        Receiver demod fmwideband  ;# Rec demod type/digital filter 2=20Khz
        Receiver ifbw 30 kHz       ;# Rec if rbw hardware
        after $selftest(cmddly)
        BBAudio route L1 sinadmtr
        Dist en enable
        after $selftest(cmddly)
        Dist run on
        Dist trigger continuous
        after $selftest(cmddly)
        Dist math on
        Dist Filter 5
        after $selftest(cmddly)
        Dist Avg 5
        if {$selftest(speaker)} {BBAudio mix L1 spkr on}
        after $selftest(cmddly)
        if {$selftest(audout1)} {BBAudio mix L1 fgen on}
        if {$selftest(audout2)} {BBAudio mix L1 demod on}
        after $selftest(cmddly)
        set MasterPF 0 ;# set MasterPF to all fails at the start
        for {set loop 0} {$loop < $loops} {incr loop} \
        {
            if {$StopAllTests} {break}
            set Finished 0
            while {! $Finished} \
            {
                set Lower [expr $Expect($loop) - 1]
                set Upper [expr $Expect($loop) + 1]
                FGen dev4 $Devi($loop)
                after 5000
                if {$StopAllTests} {break}
                set Reading($loop) [Dist value]
                if {[CheckLimits $Reading($loop) $Lower $Upper $Units]} \
                {
                    set Result($loop) "Passed"
                    set MasterPF [ChgBit $MasterPF $loop 1] ;# Set bit for pass
                } \
                else \
                {
                    set Result($loop) "*Failed*"
                    set MasterPF [ChgBit $MasterPF $loop 0] ;# Clear bit for fail
                }
            }
        }
        Dist en disable           ;# Disable the Distortion meter
        Dist run off              ;# Stop the Distortion meter
        after $selftest(cmddly)
        FGen mod3 off             ;# Turn off Fgen3
        FGen mod4 off             ;# Turn off Fgen4
        after $selftest(cmddly)
        BBAudio clrmix L1         ;# Clear routing to Layer 1
        BBAudio clrmix spkr       ;# Clear routing to Speaker
        after $selftest(cmddly)
        BBAudio clrmix fgen       ;# Clear routing to Audio out 1
        BBAudio clrmix demod      ;# Clear routing to Audio out 2
        after $selftest(cmddly)
        Receiver demod off        ;# Turn off demod
        Generator ptt off         ;# Turn off Generator output
        after $selftest(cmddly)
        if {! $StopAllTests} \
        {
            if {$MasterPF == $ExpectPF } {SetStatus 1} else {SetStatus 0}
        } \
        else {SetStatus 2}
        # Write results to results.txt
        WriteStartEndHead 0
        set head01 "  RF        Expected       Actual         Test   "
        set head02 " Freq      Distortion    Distortion      Status  "
        set head03 " (MHz)        (%)           (%)           (P/F)  "
        set dashes "-------    ----------    ----------    ----------"
        puts $ResultsId [MakeHeader $head01 2]
        puts $ResultsId [MakeHeader $head02 2]
        puts $ResultsId [MakeHeader $head03 2]
        puts $ResultsId [MakeHeader $dashes 2]
        for {set x 0} {$x < $loops} {incr x} \
        {
            set res [format " 200.00      %5.1f         %5.1f       %s" \
            $Expect($x) $Reading($x) [MakeString $Result($x) 10 center]]
            puts $ResultsId [MakeHeader $res 2]
        }
        WriteStartEndHead 1
        if {$selftest(debug)} {puts "T05Distortion: End"}
    }
    
    proc T06RcvrSens {} \
    { ;# Receiver Sensitivity Test
        variable Finished
        variable StopAllTests
        variable TestNum
        variable TestName
        variable TestStatus
        variable ResultsId
        variable selftest
        
        if {$selftest(debug)} {puts "T06RcvrSens: Start"}
        # This is a 3 part test... each pass/fail status is recorded in 1 bit of
        # MasterPF a pass is a 1, and a fail is a 0. If MasterPF = 7 at the end
        # then all tests passed.
        # Test code goes here
        set Units {dBm}
        switch $selftest(level) \
        {
            0 {
                set loops 1
                set ExpectPF 1
                array set RfFreq {0 1000.0}
            }
            1 {
                set loops 3
                set ExpectPF 7
                # Run test at 3 different frequencies
                array set RfFreq {0 501.1 1 1000.0 2 2000.0}
            }
        }
        if {$selftest(TXPort) == 0 && $selftest(RXPort) == 0} {set Spec -90.0} \
        elseif {$selftest(RXPort) == 1} {set Spec -80.0} \
        else {set Spec -40.0}
        for {set loop 0} {$loop < $loops} {incr loop} \
        {
            set Reading($loop) "-99"
            set Sinad($loop) "10"
            set Result($loop) "Incomplete"
        }
        # Display 'Running' Status
        SetStatus 3
        Analyzer tos 0 ;# This fixes the problem of the atten changing to 10
        Generator ptt on ;# Turn on Generator output
        after $selftest(cmddly)
        #
        # RCI test code goes here.
        #
        # ***** Generator Setup *****
        #
        Generator rflev -30
        Generator rfout $selftest(TXPort) ;# set port
        after $selftest(cmddly)
        Generator mod fmn                 ;# gen FM
        FGen mod3 tone                    ;# FGen3 mode TONE
        after $selftest(cmddly)
        FGen freq3 1000
        FGen shape3 sine
        after $selftest(cmddly)
        FGen dev3 3.3
        BBAudio mix Fgen3 L1 on           ;# Fgen3 to layer 1
        after $selftest(cmddly)
        # ***** Receiver Setup *****
        #
        Receiver rfin $selftest(RXPort)
        Receiver demod fmnarrow  ;# Rec demod type/digital filter 0=25khz
        after $selftest(cmddly)
        Receiver ifbw 30 kHz     ;# Rec if rbw hardware
        Receiver atten 0
        after $selftest(cmddly)
        set PrevAgc [Receiver agcmode]
        Receiver agcmode 1
        BBAudio route L1 sinadmtr ;# Layer 1 to Sinad meter
        after $selftest(cmddly)
        Sinad en enable
        Sinad run on
        after $selftest(cmddly)
        Sinad trigger continuous
        Sinad avg 1
        after $selftest(cmddly)
        Sinad math 0
        if {$selftest(speaker)} {BBAudio mix L1 spkr on}
        after $selftest(cmddly)
        if {$selftest(audout1)} {BBAudio mix L1 fgen on}
        if {$selftest(audout2)} {BBAudio mix L1 demod on}
        after $selftest(cmddly)
        set MasterPF 0 ;# set MasterPF to all fails at the start
        for {set loop 0} {$loop < $loops} {incr loop} \
        {
            if {$StopAllTests} {break}
            set Finished 0
            while {! $Finished} \
            {
                Generator freq $RfFreq($loop)
                Receiver freq $RfFreq($loop)
                after $selftest(cmddly)
                set level -20.0
                set stepper -10
                set Sinad($loop) 100
                Generator rflev -30.0
                after 3000 ;# Wait for initial level to stabilize
                if {$StopAllTests} {break}
                # if {$selftest(debug)} {puts "Atten After: [Receiver atten]"}
                while {$level > -100 && $Sinad($loop) > 10} \
                {
                    if {$Sinad($loop) > 35} {set stepper -10} \
                    elseif {$Sinad($loop) > 25} {set stepper -5} \
                    elseif {$Sinad($loop) > 15} {set stepper -2} \
                    else {set stepper -1}
                    # elseif {$Sinad($loop) > 15} {set stepper -1} \
                    # else {set stepper -0.5}
                    set level [expr $level + $stepper]
                    Generator rflev $level
                    # after 1000
                    after 2000
                    if {$StopAllTests} {break}
                    set Sinad($loop) [Sinad value]
                    if {$selftest(debug)} {puts "Level: $level -- Sinad: [format %5.2f $Sinad($loop)]"}
                }
                if {$StopAllTests} {break}
                set Reading($loop) $level
                if {[CheckLimits $Reading($loop) None $Spec $Units]} \
                {
                    set Result($loop) "Passed"
                    set MasterPF [ChgBit $MasterPF $loop 1] ;# Set bit for pass
                } \
                else \
                {
                    set Result($loop) "*Failed*"
                    set MasterPF [ChgBit $MasterPF $loop 0] ;# Clear bit for fail
                }
            }
        }
        Receiver agcmode $PrevAgc
        BBAudio clrmix L1     ;# Clear routing to Layer 1
        after $selftest(cmddly)
        BBAudio clrmix spkr   ;# Clear routing to Speaker
        BBAudio clrmix fgen   ;# Clear routing to Audio out 1
        after $selftest(cmddly)
        BBAudio clrmix demod  ;# Clear routing to Audio out 2
        Receiver demod off    ;# Demod off
        after $selftest(cmddly)
        Generator ptt off     ;# Turn off Generator output
        if {! $StopAllTests} \
        {
            if {$MasterPF == $ExpectPF } {SetStatus 1} else {SetStatus 0}
        } \
        else {SetStatus 2}
        # Write results to results.txt
        WriteStartEndHead 0
        set head01 "  RF        Expected       Actual      Measured       Test   "
        set head02 " Freq      10dB Sinad    10dB Sinad     Sinad        Status  "
        set head03 " (MHz)     Level(dBm)    Level(dBm)      (dB)         (P/F)  "
        set dashes "-------    ----------    ----------    --------    ----------"
        puts $ResultsId [MakeHeader $head01 2]
        puts $ResultsId [MakeHeader $head02 2]
        puts $ResultsId [MakeHeader $head03 2]
        puts $ResultsId [MakeHeader $dashes 2]
        for {set loop 0} {$loop < $loops} {incr loop} \
        {
            set res [format "%7.2f      < %5.1f       %6.1f        %4.1f      %s" \
            $RfFreq($loop) $Spec $Reading($loop) $Sinad($loop) [MakeString $Result($loop) 10 center]]
            puts $ResultsId [MakeHeader $res 2]
        }
        WriteStartEndHead 1
        if {$selftest(debug)} {puts "T06RcvrSens: End"}
    }
    
    proc T07Analyzer {} \
    { ;# Analyzer Test
        variable Finished
        variable StopAllTests
        variable TestNum
        variable TestName
        variable TestStatus
        variable ResultsId
        variable selftest
        
        if {$selftest(debug)} {puts "T07Analyzer: Start"}
        # Test code goes here
        # This is a 3 part test... each pass/fail status is recorded in 2 bits of
        # MasterPF a pass is a 1, and a fail is a 0. If MasterPF = 63 at the end
        # then all 3 tests passed.
        
        set Units {dBm}
        switch $selftest(level) \
        {
            0 {
                SetStatus 6
                return ;# No Short test yet
                # set loops 1
                # set ExpectPF 1
                # array set Expect {0 25.00}
            }
            1 {
                set loops 3
                set ExpectPF 127
                # Fill array with our expected RF Levels
                array set Expect {0 14.00 1 14.00 2 14.00}
                # Fill array with our RF Frequencies
                array set RfFreq {0 10.00 1 500.00 2 2000.00}
                # Fill array with our spans (in MHz)
                array set Span {0 1.0 1 1.0 2 1.0}
            }
        }
        for {set loop 0} {$loop < $loops} {incr loop} \
        {
            set maxvalue($loop) -999
            set maxfreq($loop) 0
            set LResult($loop) " Inc. "
            set FResult($loop) " Inc. "
        }
        set Finished 0 ;# So we don't get an error from the Screen proc
        # Display 'Running' Status
        SetStatus 3
        set MasterPF 0 ;# set MasterPF to all fails at the start
        Screen Analyzer ;# switch to Analyzer Screen
        # Normalize Analyzer
        if {! [NormAn]} \
        {
            # set StopAllTests 1
            set NormResult "Normalize: *Failed*"
            set MasterPF [ChgBit $MasterPF 6 0] ;# Clear bit for fail
        } \
        else \
        {
            set NormResult "Normalize: Passed"
            set MasterPF [ChgBit $MasterPF 6 1] ;# Set bit for pass
        }
        after 1000 ;# Give generator time to shut off, before turning it on again.
        Generator ptt on ;# Turn on Generator output
        #
        # RCI test code goes here.
        #
        # ***** Generator Setup *****
        Generator freq 10.00
        after $selftest(cmddly)
        Generator rflev -30.0
        Generator rfout $selftest(TXPort) ;# set port
        after $selftest(cmddly)
        Generator mod off                 ;# gen modulation off
        #
        # ***** Receiver Setup *****
        Receiver freq 10.00
        after $selftest(cmddly)
        Receiver rfin $selftest(RXPort)
        # Receiver atten 0
        Receiver demod off    ;# Rec demod Off
        after $selftest(cmddly)
        Receiver ifbw 30 kHz  ;# Rec if rbw hardware
        Analyzer tos 25.0
        after $selftest(cmddly)
        Analyzer rbw 60000
        Analyzer vbw 10000
        after $selftest(cmddly)
        Analyzer sweep 200000 2
        Analyzer enable 500 99 ;# This is how it is set in duplex mode.
        after $selftest(cmddly)
        Analyzer run 2         ;# 1 is channel, 2 is Main
        for {set loop 0} {$loop < $loops} {incr loop} \
        {
            if {$StopAllTests} {break}
            set Finished 0
            while {! $Finished} \
            {
                set LLower [expr $Expect($loop) - 10]
                set LUpper [expr $Expect($loop) + 10]
                set FLower [expr $RfFreq($loop) - .250] ;# CF - 250 kHz
                set FUpper [expr $RfFreq($loop) + .250] ;# CF + 250 kHz
                Generator freq $RfFreq($loop)
                after $selftest(cmddly)
                Receiver freq $RfFreq($loop)
                Analyzer span $Span($loop) -1 ;# Set Analyzer span
                after 4000
                if {$StopAllTests} {break}
                set maxvalue($loop) -999
                set maxfreq($loop) 0
                set mylist [Analyzer trace]
                set numpoints [lindex $mylist 0]
                set leftedge [expr $RfFreq($loop) - ($Span($loop) / 2)]
                set onepoint [expr $Span($loop) / $numpoints]
                for {set point 1} {$point < [llength $mylist]} {incr point} \
                {
                    set curr [lindex $mylist $point]
                    if {$curr > $maxvalue($loop)} \
                    {
                        set maxvalue($loop) $curr
                        set maxfreq($loop) [expr $leftedge + $onepoint * $point]
                    }
                }
                if {$selftest(debug)} {puts "Peak is $maxvalue($loop) dBm at [format "%.2f" $maxfreq($loop)] MHz."}
                if {$StopAllTests} {break}
                if {[CheckLimits $maxvalue($loop) $LLower $LUpper dBm]} \
                {
                    set LResult($loop) " Pass "
                    set MasterPF [ChgBit $MasterPF [expr $loop * 2] 1] ;# Set bit for pass
                } \
                else \
                {
                    set LResult($loop) "*Fail*"
                    set MasterPF [ChgBit $MasterPF [expr $loop * 2] 0] ;# Clear bit for fail
                }
                if {$StopAllTests} {break}
                if {[CheckLimits $maxfreq($loop) $FLower $FUpper MHz]} \
                {
                    set FResult($loop) " Pass "
                    set MasterPF [ChgBit $MasterPF [expr $loop * 2 + 1] 1] ;# Set bit for pass
                } \
                else \
                {
                    set FResult($loop) "*Fail*"
                    set MasterPF [ChgBit $MasterPF [expr $loop * 2 + 1] 0] ;# Clear bit for fail
                }
            }
        }
        Analyzer run 3
        Generator ptt off ;# Turn off the Generator output
        after $selftest(cmddly)
        if {! $StopAllTests} \
        {
            if {$MasterPF == $ExpectPF } {SetStatus 1} else {SetStatus 0}
        } \
        else {SetStatus 2}
        # SetStatus 6
        # Write results to results.txt
        WriteStartEndHead 0
        set head01 "Expected    Actual      Test     Expected    Actual     Test "
        set head02 " Freq        Freq      Status     Level      Level     Status"
        set head03 " (MHz)       (MHz)      (P/F)     (dBm)      (dBm)      (P/F)"
        set dashes "--------    -------    ------    --------    ------    ------"
        puts $ResultsId [MakeHeader $NormResult 2]
        puts $ResultsId ""
        puts $ResultsId [MakeHeader $head01 2]
        puts $ResultsId [MakeHeader $head02 2]
        puts $ResultsId [MakeHeader $head03 2]
        puts $ResultsId [MakeHeader $dashes 2]
        for {set loop 0} {$loop < $loops} {incr loop} \
        {
            set res [format { %7.2f    %7.2f    %6s     %5.1f      %6.1f    %6s} \
            $RfFreq($loop) $maxfreq($loop) $FResult($loop) \
            $Expect($loop) $maxvalue($loop) $LResult($loop)]
            puts $ResultsId [MakeHeader $res 2]
        }
        WriteStartEndHead 1
        set selftest(ididit) 1
        Screen Selftest ;# Switch back to self test screen
        set selftest(ididit) 0
        if {$selftest(debug)} {puts "T07Analyzer: End"}
    }
    
    proc T09P25 {} \
    { ;# P25 Test
        variable Finished
        variable StopAllTests
        variable TestNum
        variable TestName
        variable TestStatus
        variable ResultsId
        variable selftest
        
        if {$selftest(debug)} {puts "T08P25: Start"}
        # Test code goes here
        # This is a 3 part test... each pass/fail status is recorded in 1 bit of
        # MasterPF a pass is a 1, and a fail is a 0. If MasterPF = 7 at the end
        # then all 3 tests passed.
        switch $selftest(level) \
        {
            0 {
                SetStatus 6
                return ;# No short test yet
            }
            1 {
                array set C4Reading {0 "99" 1 "99"}
                array set AFReading {0 "   N/A   " 1 "0"}
                array set C4Expect {0 "  < 2.0 " 1 "  < 2.0 "}
                array set C4Result {0 " Inc. " 1 " Inc. "}
                array set AFResult {0 " N/A  " 1 " Inc. "}
            }
        }
        set ExpectPF 7
        set MasterPF 0 ;# set MasterPF to all fails at the start
        set RfFreq 501.1
        
        Generator ptt on ;# Turn on Generator output
        # Display 'Running' Status
        SetStatus 3
        set loop 0
        set Finished 0
        while {! $Finished} \
        {
            if {$StopAllTests} {break}
            set MasterPF [ChgBit $MasterPF $loop 1] ;# default to Pass
            #
            # ***** P25 Generator Setup *****
            Generator freq 501.1
            Generator rfout $selftest(TXPort)
            after $selftest(cmddly)
            Generator rflev -30
            Generator mod p25mis  ;# gen P25
            after $selftest(cmddly)
            Generator p25m 5      ;# P25 Speech mode
            #
            # ***** P25 Receiver Setup *****
            Receiver freq 501.1
            after $selftest(cmddly)
            Receiver rfin $selftest(RXPort)
            Receiver atten 0
            after $selftest(cmddly)
            Receiver demod p25   ;# Rec demod type/digital filter 5= P25 mode
            Receiver ifbw 30 khz ;# Rec if rbw hardware
            after $selftest(cmddly)
            C4fm trig continuous
            C4fm en enable
            after $selftest(cmddly)
            C4fm run on
            C4fm avg 200
            after $selftest(cmddly)
            if {$selftest(speaker)} {BBAudio mix L1 spkr on}
            if {$selftest(audout1)} {BBAudio mix L1 fgen on}
            after $selftest(cmddly)
            if {$selftest(audout2)} {BBAudio mix L1 demod on}
            after 10000
            if {$StopAllTests} {break}
            set C4Reading(0) [C4fm value]
            if {[CheckLimits $C4Reading(0) None 2.00 %]} \
            {
                set C4Result(0) " Pass "
            } \
            else \
            {
                set MasterPF [ChgBit $MasterPF $loop 0] ;# Failure
                set C4Result(0) "*Fail*"
            }
        }
        set loop 1
        set Finished 0
        while {! $Finished} \
        {
            if {$StopAllTests} {break}
            set MasterPF [ChgBit $MasterPF $loop 1] ;# default to Pass
            Generator P25m 6 ;# P25 to 1011 mode
            #	***** 1011 Mode Receiver setup *****
            AFCounter trigger continuous
            after $selftest(cmddly)
            AFCounter run on
            AFCounter gate tenth
            after $selftest(cmddly)
            AFCounter avg 5
            AFCounter en enable
            after $selftest(cmddly)
            BBAudio route L1 AFCounter ;# Set Layer 1 to AF Counter
            after 5000
            if {$StopAllTests} {break}
            set C4Reading($loop) [C4fm value]
            if {[CheckLimits $C4Reading($loop) None 2.00 %]} \
            {
                set C4Result($loop) " Pass "
            } \
            else \
            {
                set MasterPF [ChgBit $MasterPF $loop 0] ;# Failed
                set C4Result($loop) "*Fail*"
            }
        }
        set loop 2
        set Finished 0
        while {! $Finished} \
        {
            if {$StopAllTests} {break}
            set MasterPF [ChgBit $MasterPF $loop 1] ;# default to Pass
            set AFReading(1) [AFCounter value]
            if {$StopAllTests} {break}
            if {[CheckLimits $AFReading(1) 961 1061 Hz]} \
            {
                set AFResult(1) " Pass "
            } \
            else \
            {
                set MasterPF [ChgBit $MasterPF $loop 0] ;# Failure
                set AFResult(1) "*Fail*"
            }
        }
        C4fm en disable          ;# Turn off C4FM
        C4fm run off             ;# Turn off C4FM
        after $selftest(cmddly)
        AFCounter run off        ;# Turn off AFCounter
        AFCounter en disable     ;# Turn off AFCounter
        after $selftest(cmddly)
        BBAudio clrmix spkr      ;# Clear routing to Speaker
        after $selftest(cmddly)
        BBAudio clrmix fgen      ;# Clear routing to Audio out 1
        BBAudio clrmix demod     ;# Clear routing to Audio out 2
        after $selftest(cmddly)
        Generator mod off        ;# Turn off Generator modulation
        Receiver demod off       ;# Turn off demod
        after $selftest(cmddly)
        Generator ptt off        ;# Turn off Generator output
        if {! $StopAllTests} \
        {
            if {$MasterPF == $ExpectPF } {SetStatus 1} else {SetStatus 0}
        } \
        else {SetStatus 2}
        # Write results to results.txt
        WriteStartEndHead 0
        set res(0) [format {%7.3f    %s     %5.2f      %s    %s    %s} \
        $RfFreq $C4Expect(0) $C4Reading(0) $C4Result(0) $AFReading(0) $AFResult(0)]
        set res(1) [format {%7.3f    %s     %5.2f      %s     %7.2f     %s} \
        $RfFreq $C4Expect(1) $C4Reading(1) $C4Result(1) $AFReading(1) $AFResult(1)]
        set head1  "  RF       Expected     Actual      Test     AFCounter     Test "
        set head2  " Freq      C4FM Err    C4FM Err    Status     Reading     Status"
        set head3  " (MHz)       (%)         (%)        (P/F)       (Hz)       (P/F)"
        set dashes "-------    --------    --------    ------    ---------    ------"
        puts $ResultsId [MakeHeader $head1 2]
        puts $ResultsId [MakeHeader $head2 2]
        puts $ResultsId [MakeHeader $head3 2]
        puts $ResultsId [MakeHeader $dashes 2]
        for {set x 0} {$x < 2} {incr x} \
        {
            puts $ResultsId [MakeHeader $res($x) 2]
        }
        WriteStartEndHead 1
        if {$selftest(debug)} {puts "T08P25: End"}
    }
    
    proc T08Scope {} \
    { ;# Scope Test
        variable Finished
        variable StopAllTests
        variable TestNum
        variable TestName
        variable TestStatus
        variable ResultsId
        variable selftest
        
        if {$selftest(debug)} {puts "T09Scope: Start"}
        # Test code goes here
        # This is a 3 part test... each pass/fail status is recorded in 1 bit of
        # MasterPF a pass is a 1, and a fail is a 0. If MasterPF = 7 at the end
        # then all 3 tests passed.
        set Units {Vpp}
        switch $selftest(level) \
        {
            0 {
                SetStatus 6
                return ;# No Short test yet
                # set loops 1
                # set ExpectPF 1
                # array set Expect {0 25.00}
            }
            1 {
                set loops 9
                set ExpectPF 511
                # Fill array with our expected deviations
                array set freq {0 20 1 20 2 20 3 200 4 200 5 200 6 1000 7 1000 8 1000}
                array set Expect {0 4.00 1 8.00 2 20.00 3 4.00 4 8.00 5 20.00 6 4.00 7 8.00 8 20.00}
                array set Range {0 1 1 2 2 5 3 1 4 2 5 5 6 1 7 2 8 5}
                array set spec {0 0.5 1 1.0 2 2.5 3 0.5 4 1.0 5 2.5 6 0.5 7 1.0 8 2.5}
            }
        }
        for {set x 0} {$x < $loops} {incr x} \
        {
            set Reading($x) "0"
            set Result($x) "Incomplete"
        }
        Generator ptt on ;# Turn on Generator output
        # Display 'Running' Status
        SetStatus 3
        set Finished 0
        set MasterPF 0 ;# set MasterPF to all fails at the start
        InitScope
        Screen Scope
        #
        # RCI test code goes here.
        #
        # ********************** Generator Setup ***************
        Generator freq 700.00
        Generator rflev -30
        after $selftest(cmddly)
        Generator rfout $selftest(TXPort) ;# set port
        Generator mod fmn                 ;# gen FM
        after $selftest(cmddly)
        FGen mod3 tone                    ;# FGen3 mode TONE
        FGen freq3 20
        after $selftest(cmddly)
        FGen dev3 4
        BBAudio mix Fgen3 L1 on           ;# Fgen3 to layer 1
        after $selftest(cmddly)
        #
        # ********************** Receiver Setup ***************
        Receiver freq 700.00
        Receiver rfin $selftest(RXPort)
        after $selftest(cmddly)
        Receiver atten 0
        Receiver demod fmwideband  ;# Rec demod type/digital filter 2=20Khz
        after $selftest(cmddly)
        Receiver ifbw 30 kHz       ;# Rec if rbw hardware
        Scope run3 on
        after $selftest(cmddly)
        Scope math3 1
        Scope coupling3 dc
        after $selftest(cmddly)
        Scope sweep 5000
        Scope scale3 7
        after $selftest(cmddly)
        Scope trigtype3 normal
        Scope triglev3 0
        after $selftest(cmddly)
        BBAudio route L1 Scope
        if {$selftest(speaker)} {BBAudio mix L1 spkr on}
        after $selftest(cmddly)
        if {$selftest(audout1)} {BBAudio mix L1 fgen on}
        if {$selftest(audout2)} {BBAudio mix L1 demod on}
        after $selftest(cmddly)
        for {set loop 0} {$loop < $loops} {incr loop} \
        {
            if {! $StopAllTests} \
            {
                set Finished 0
                while {! $Finished} \
                {
                    set Lower [expr $Expect($loop) - $spec($loop)]
                    set Upper [expr $Expect($loop) + $spec($loop)]
                    Scope scale3 [expr ($loop % 3) + 7]
                    Scope sweep [expr 1000000.0 / $freq($loop)]
                    after $selftest(cmddly)
                    FGen dev3 $Expect($loop)
                    FGen freq3 $freq($loop)
                    after 5000
                    set maxvalue -1000
                    set minvalue 1000
                    set mylist [Scope trace3]
                    if {$selftest(debug)} {puts $mylist}
                    for {set point 1} {$point < [llength $mylist]} {incr point} \
                    {
                        set curr [lindex $mylist $point]
                        set maxvalue [GetMax $curr $maxvalue]
                        set minvalue [GetMin $curr $minvalue]
                    }
                    set Reading($loop) [expr ($maxvalue - $minvalue) / 819.2 * $Range($loop)]
                    # puts "The pk to pk amplitude is [format %4.2f $pktopk($loop)] kHz"
                    if {[CheckLimits $Reading($loop) $Lower $Upper $Units]} {
                        set Result($loop) "Passed"
                        set MasterPF [ChgBit $MasterPF $loop 1] ;# Set bit for pass
                    } else {
                        set Result($loop) "*Failed*"
                        set MasterPF [ChgBit $MasterPF $loop 0] ;# Clear bit for fail
                    }
                }
            }
        }
        Scope run3 off
        BBAudio route none Scope
        after $selftest(cmddly)
        BBAudio clrmix L1         ;# Clear routing to Layer 1
        BBAudio clrmix spkr       ;# Clear routing to Speaker
        after $selftest(cmddly)
        BBAudio clrmix fgen       ;# Clear routing to Audio out 1
        BBAudio clrmix demod      ;# Clear routing to Audio out 2
        after $selftest(cmddly)
        FGen mod3 off             ;# Turn off the FGen we used
        Receiver demod off        ;# Turn off demod
        after $selftest(cmddly)
        Generator ptt off         ;# Turn off the Generator output
        set selftest(ididit) 1
        Screen Selftest
        set selftest(ididit) 0
        InitScope
        if {! $StopAllTests} {
            if {$MasterPF == $ExpectPF } {SetStatus 1} else {SetStatus 0}
        } else {SetStatus 2}
        # Write results to results.txt
        WriteStartEndHead 0
        set head01 " Audio     Expected      Actual         Test   "
        set head02 " Freq      Deviation    Deviation      Status  "
        set head03 " (Hz)        (kHz)        (kHz)         (P/F)  "
        set dashes "-------    ---------    ---------    ----------"
        puts $ResultsId [MakeHeader $head01 2]
        puts $ResultsId [MakeHeader $head02 2]
        puts $ResultsId [MakeHeader $head03 2]
        puts $ResultsId [MakeHeader $dashes 2]
        for {set x 0} {$x < $loops} {incr x} \
        {
            set res [format {%7.2f      %5.2f        %5.2f      %s} \
            $freq($x) $Expect($x) $Reading($x) [MakeString $Result($x) 10 center]]
            puts $ResultsId [MakeHeader $res 2]
        }
        WriteStartEndHead 1
        if {$selftest(debug)} {puts "T09Scope: End"}
    }
    proc InitScope {} \
    { ;# We must set up the scope variables before we show the scope.
        variable OldState
        variable selftest
        array set index {0 ch1 1 ch2 2 mdyn 3 mect 4 aud 5 bal 6 demod \
            7 demod,triglev 8 demod,horiz 9 demod,vpos}
        array set NewState {0 OFF 1 OFF 2 OFF 3 OFF 4 OFF 5 OFF 6 ON \
            7 0.000 8 0.0 9 0}
        if {[info exists OldState]} \
        { ;# OldState exists so restore Scope state, and delete OldState
            if {$selftest(debug)} {puts "Restoring Scope state"}
            for {set element 0} {$element < 10} {incr element} \
            {
                set ScopeUI::StateUI($index($element)) $OldState($element)
            }
            unset OldState ;# Wipe it out, so it won't exist if called again
        } \
        else \
        { ;# OldState doesn't exist so save Scope State
            if {$selftest(debug)} {puts "Saving Scope state"}
            for {set element 0} {$element < 10} {incr element} \
            {
                set OldState($element) $ScopeUI::StateUI($index($element))
                set ScopeUI::StateUI($index($element)) $NewState($element)
            }
        }
    }
    # ***********************************************************************
    # Functions executed when the namespace is sourced
    # ***********************************************************************
    if {[option get . selfTest {}] == "enabled" } \
    {
        ::View::RegisterScreen Selftest_Screen;  # Add this on the menu
        InitSelftest
    }
    
};  # End of the Selftest_Screen name space

namespace eval Results_Screen {
    variable LongName
    variable ShortName
    variable RecommendedTiles
    variable NotAllowedTiles
    variable RequiredTiles
    variable ActiveTiles
    variable MenuFlags
    variable SubmenuScreens
    variable SubDirPath
    
    set LongName Results_Screen
    set ShortName "Results"
    set RequiredTiles { }
    set NotAllowedTiles { }
    set RecommendedTiles { }
    set ActiveTiles { }
    set MenuFlags 0
    set SubmenuScreens { }
    set SubDirPath { }
    
    variable tile_path      ""
    variable tile_size      "full"
    variable tile_class     "RESULTS"
    
    variable resultsbox .results.myframe.box
    variable resultscroll .results.myframe.yscroll
    
    # ***********************************************************************
    # Name:        Setup
    # Parameters:  parent		parent of child window
    # Description: Dummy function
    # Returns:     Indicator that no tile is generated
    # ***********************************************************************
    proc Setup { parent } \
    {
        if {$::Selftest_Screen::selftest(debug)} {puts "proc: ::Results_Screen::Setup"}
        set top_level [winfo toplevel $parent]
        set tile_path [name_child $top_level results]
        set tile_class  $::Results_Screen::tile_class
        if { ![winfo exists $tile_path] } \
        {
            set tile_path  [::Screens::create_tile  $tile_path  results  $tile_class]
        }
        set ::Results_Screen::tile_path $tile_path
        
        return $tile_path
    }
    
    # ***********************************************************************
    # Name:        Map
    # Parameters:  parent		parent of child window
    # Description: Dummy function
    # ***********************************************************************
    proc Map { name } \
    {
        variable tile_path
        variable resultsbox
        variable resultscroll
        
        if {$::Selftest_Screen::selftest(debug)} {puts "proc: ::Results_Screen::Map"}
        
        ::View::ClearAllSoftkeys  $tile_path
        ::View::PackOptions $tile_path ""
        
        # ASSIGN SOFTKEYs
        ::View::AssignSoftkey $tile_path 2 "Page\nUp" "::Results_Screen::UpDown $resultsbox scroll -1 pages"
        ::View::AssignSoftkey $tile_path 3 "Scroll\nUp" "::Results_Screen::UpDown $resultsbox scroll -1 units"
        ::View::AssignSoftkey $tile_path 4 "Scroll\nDown" "::Results_Screen::UpDown $resultsbox scroll 1 units"
        ::View::AssignSoftkey $tile_path 5 "Page\nDown" "::Results_Screen::UpDown $resultsbox scroll 1 pages"
        ::View::AssignReturn $tile_path "::Selftest_Screen::Show Selftest_Screen" ;# Set up Return key.
        DisplayResults
        screen_done FORCE_COMPLETE
        
        return
    }
    
    # ***********************************************************************
    # Name:        Unmap
    # Parameters:  parent		parent of child window
    # Description: Dummy function
    # ***********************************************************************
    proc Unmap { name } \
    {
        variable KillMe
        
        if {$::Selftest_Screen::selftest(debug)} \
        {
            puts "name: $name"
            puts "proc: ::Results_Screen::Unmap"
        }
        set KillMe 1
        return
    }
    
    #########################################################################
    # Name:        Destroy
    # Parameters:  parent		parent of child window
    # Description: Dummy function
    #########################################################################
    proc Destroy { name } \
    {
        variable tile_path
        
        if {$::Selftest_Screen::selftest(debug)} \
        {
            puts "name: $name"
            puts "proc: ::Results_Screen::Destroy"
        }
        destroy $tile_path
        return
    }
    
    #########################################################################
    #       Name: DisplayResults
    # Parameters:
    #Description: opens results.txt, and displays the contents in a listbox.
    #
    #########################################################################
    proc DisplayResults {} \
    {
        variable resultsbox
        variable resultscroll
        
        $resultsbox delete 0 end ;# Clear the resultsbox before we start.
        set fileid [open $::sys(system_path)/screens/2090.System/results.txt {RDONLY CREAT}]
        while {! [eof $fileid]} \
        {
            gets $fileid line
            $resultsbox insert end $line
        }
        close $fileid
        update idle
        set thelist [$resultsbox yview]
        $resultscroll set [lindex $thelist 0] [lindex $thelist 1]
    }
    
    #########################################################################
    #       Name: UpDown
    # Parameters: box, args
    #Description: called whenever the scrollbar is manipulated in
    #             the results listbox
    #
    #########################################################################
    proc UpDown {box args} \
    {
        variable resultscroll
        
        if {[llength $args] == 2} \
        {
            $box yview [lindex $args 0] [lindex $args 1]
            #		puts "The parameters passed were: [lindex $args 0] [lindex $args 1]"
        } \
        else \
        {
            $box yview [lindex $args 0] [lindex $args 1] [lindex $args 2]
            #		puts "The parameters passed were: [lindex $args 0] [lindex $args 1] [lindex $args 2]"
        }
        
        set thelist [$box yview]
        $resultscroll set [lindex $thelist 0] [lindex $thelist 1]
    }
}
