#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C) 2001      DESIGN ENGINEERING        #
# --------------------------------------------------------------------------#
# FILE        : 2086.recall_setup.tcl                                       #
# PRODUCT     : FITS                                                        #
# AUTHOR      : Preston McCormick                                           #
# DESCRIPTION : This namespace is a dummy "screen" that sources and holds   #
#             : the menu command button to display the Recall Setup         #
#             : toplevel                                                    #
#############################################################################
#                                                                           #
# Required Variables:                                                       #
#    LongName           Name of screen                                      #
#    ShortName          Name of screen for menus                            #
#    RecomendedTiles    List of tiles available for this screen. It is not  #
#                       necessary to put a tile on this list that is on the #
#                       RequiredTiles list                                  #
#    NotAllowedTiles    List of tiles NEVER on this screen                  #
#    RequiredTiles      List of tiles ALWAYS on this screen                 #
#    ActiveTiles        List of tiles currently displayed on this screen    #
#    MenuFlags          0 = Active Screen                                   #
#                       1 = Currently functioning as a submenu parent       #
#                                                                           #
# Required Variables for submenu "Screens":                                 #
#    SubmenuScreens     List of all tiles (or submenus) for droplist        #
#    SubDirPath         Path to subdirectory holding the screens for the    #
#                       submenu of this "screen"                            #
#                                                                           #
#############################################################################
#                                                                           #
# Dummy Functions:                                                          #
#    Setup              Called by the screen manager to create the screen.  #
#    Map                Called when screen is mapped on window              #
#    Unmap              Called when screen is removed from display          #
#    Destroy            Called when screen is destroyed                     #
#                                                                           #
#############################################################################

namespace eval Recall_Setup_Screen \
{
  variable  LongName   recall_Setup_Screen
  #variable  ShortName  "Recall Setups"
  variable  ShortName  "Save / Recall Setups"
  
  variable  ActiveTiles       { }
  variable  NotAllowedTiles   { }
  variable  RecommendedTiles  { }
  variable  RequiredTiles     [list  Recall_SetupsUI]
  variable  SubmenuScreens    { }
            
  variable  MenuFlags   0
  variable  SubDirPath  { }


  #****************************************************************************
  # Name:        Setup
  # Parameters:  parent - parent of child window
  # Description: Dummy function
  # Returns:     Indicator that no tile is generated
  #****************************************************************************
  proc Setup { parent } \
  {
    #puts "Recall_Setup_Screen::Setup:  $parent"
  
    # Use the Screen functions to manage our tiles
    set topWin  [winfo toplevel $parent]

    View::PackOptions  $topWin  ""

    View::PackKeyBar  $topWin

    Screens::ShowRequiredTiles  $topWin  Recall_Setup_Screen

    return  ""
  }


  #****************************************************************************
  # Name:        Map
  # Parameters:  parent	- parent of child window
  # Description: Dummy function
  #****************************************************************************
  proc Map { name } \
  {
    
  }


  #****************************************************************************
  # Name:        Unmap
  # Parameters:  parent	- parent of child window
  # Description: Dummy function
  #****************************************************************************
  proc Unmap { name } \
  { 
    
  }


  #****************************************************************************
  # Name:        Destroy
  # Parameters:  parent	- parent of child window
  # Description: Dummy function
  #****************************************************************************
  proc Destroy { name } \
  { 
    
  }


  #****************************************************************************
  # Name:        getSaveState
  # Parameters:  none
  # Description: This function returns a list of TCL commands which can be used
  #              to set the Recall_Setup_Screen screen to its current state
  #****************************************************************************
  proc getSaveState { } \
  {
    set returnList \
    {
      
    }
    
    return [subst $returnList]
  }


  #****************************************************************************
  # Functions executed when the namespace is sourced
  #****************************************************************************
  # Add this on the menu
  View::RegisterScreen  Recall_Setup_Screen
  

};  # End of the Recall_Setup_Screen


