#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C) 2001      DESIGN ENGINEERING        #
# --------------------------------------------------------------------------#
# FILE        : 3002.auto_test.tcl                                          #
# PRODUCT     : FITS                                                        #
# AUTHOR      : Preston McCormick                                           #
# DESCRIPTION : This namespace manages the Auto Test                        #
#############################################################################
#                                                                           #
# Required Variables:                                                       #
#    LongName           Name of screen                                      #
#    ShortName          Name of screen for menus                            #
#    RecomendedTiles    List of tiles available for this screen. It is not  #
#                       necessary to put a tile on this list that is on the #
#                       RequiredTiles list                                  #
#    NotAllowedTiles    List of tiles NEVER on this screen                  #
#    RequiredTiles      List of tiles ALWAYS on this screen                 #
#    ActiveTiles        List of tiles currently displayed on this screen    #
#    MenuFlags          0 = Active Screen                                   #
#                       1 = Currently functioning as a submenu parent       #
#                                                                           #
# Required Variables for submenu "Screens":                                 #
#    SubmenuScreens     List of all tiles (or submenus) for droplist        #
#    SubDirPath         Path to subdirectory holding the screens for the    #
#                       submenu of this "screen"                            #
#                                                                           #
#############################################################################
#                                                                           #
# Utility Functions:                                                        #
#    Setup              Called by the screen manager to create the screen.  #
#    Map                Called when screen is mapped on window              #
#    Unmap              Called when screen is removed from display          #
#    Destroy            Called when screen is destroyed                     #
#                                                                           #
#############################################################################

namespace eval Auto_Test \
{
  variable  LongName   "auto_test"
  variable  ShortName  "Auto Test"
  variable  Size       Full
            

  variable  RequiredTiles    { }
  variable  NotAllowedTiles  { }
  variable  RecommendedTiles { }
  variable  ActiveTiles      { }
  variable  SubmenuScreens   { }

  variable  MenuFlags 0

  variable  SubDirPath { }

  variable  last_focus  ""

  # Test information variables
  variable  test_num     0
  variable  subtest_num  0
  
  variable  test_start_time  ""
  variable  test_stop_time   ""
  variable  test_total_time  ""
  
  variable  test_date       "Nov. 24, 2002"
  variable  test_time       "11:00:00"
  variable  test_unit_id    "1234"
  variable  test_tech_name  "John Q."

  variable  prev_radio_type       ""
  variable  test_radio_type       "ASTRO SABER"
  variable  new_radio_type        ""  
  variable  del_radio_type        ""  
  variable  test_radio_type_menu  { "ASTRO SABER"  "XTS-3000" }

  variable  prev_test_type  ""
  variable  new_test_type   ""  
  variable  del_test_type   ""  
  variable  test_type       "INCOMING RADIO"
  variable  test_type_menu  { "INCOMING RADIO"  "OUTGOING RADIO"  "RADIO STATUS CHANGE" }
  
  variable  config_description  ""

  variable  pass_fail_label  
  variable  pass_fail_max  
  variable  pass_fail_min  
  variable  pass_fail_units  
    
  variable  results         ""
  
  variable  save_flag   0
  variable  abort_flag  0

  variable  test_enable
  
  set  test_enable(1)  1
  set  test_enable(2)  1
  set  test_enable(3)  0
  set  test_enable(4)  0
  set  test_enable(5)  0

  variable  test_title
  
  set test_title(1)  ""
  set test_title(2)  ""
  set test_title(3)  ""
  set test_title(4)  ""
  set test_title(5)  ""

  variable  chan_info_type  "NONE"
  

  variable  printer_dir   "printer0"

  # Test variables
  variable  test_data
  variable  test_data_save  ""
  
  # Results file variables
  variable  file_ext      ".TXT"
  variable  file_label    "Results"
  variable  file_prefix   "FILE"
  variable  working_dir   "/usr/apps/"

  # Test data file variables
  variable  data_file_name     "TEST.DAT"
  variable  data_working_dir   "/usr/apps/test_data/"


  # Operator prompt dialog variables
  variable  return_value
  

  #***************************************************************************
  #
  # Name: Setup
  #
  # Parameters: parent - parent of child window
  #
  # Description: Creates the main tile
  #
  #***************************************************************************
  proc Setup { parent } \
  {
    #puts "Auto_Test::Setup:  $parent"

    variable  LongName
    variable  tile_class

    set top_level  [winfo toplevel $parent]
   
    set tile_path  [name_child  $top_level  $LongName]

    if { ![winfo exists $tile_path] } \
    {
      set tile_path  [::Screens::create_tile  $tile_path  $LongName  AUTO_TEST]
    }  
     
    return $tile_path
  }


  #***************************************************************************
  #
  # Name: Map
  #
  # Parameters: 
  #
  # Description: Initializes the tile
  #
  #***************************************************************************
  proc Map { tile_path } \
  {
    #puts "::Auto_Test::Map:  $tile_path"
    
    set tile_path  [get_tile  $tile_path]    
    
    ::View::PackOptions  $tile_path  ""

    trace variable ::Auto_Test::results  w  \
                               "::Auto_Test::test_results_output  $tile_path"
                                   
    # Zero the Power meter
    after idle "Power zero"
        
    # Verify that the test data file exists
    verify_test_data

    test_info_display  $tile_path
    
    
    return
  }


  #***************************************************************************
  #
  # Name: Cleanup
  #
  # Parameters: 
  #
  # Description: 
  #
  #***************************************************************************
  proc Cleanup { tile_path } \
  {
    #puts "::Auto_Test::Cleanup:  $tile_path"

    if ![winfo exists $tile_path] { return }
    set tile_path  [get_tile  $tile_path]    
    
    trace vdelete ::Auto_Test::results  w  \
                               "::Auto_Test::test_results_output  $tile_path"

    return
  }


  #***************************************************************************
  #
  # Name: Unmap
  #
  # Parameters: 
  #
  # Description: 
  #
  #***************************************************************************
  proc Unmap { tile_path } \
  {
    #puts "::Auto_Test::Unmap:  $tile_path"

    if ![winfo exists $tile_path] { return }
    set tile_path  [get_tile  $tile_path]    

    Cleanup  $tile_path
    
    return
  }


  #***************************************************************************
  #
  # Name: Destroy
  #
  # Parameters: 
  #
  # Description: 
  #
  #***************************************************************************
  proc Destroy { tile_path } \
  {
    #puts "::Auto_Test::Destroy:  $tile_path"

    if ![winfo exists $tile_path] { return }
    set tile_path  [get_tile  $tile_path]    

    Cleanup  $tile_path

    return
  }


  #***************************************************************************
  #
  # Name: Register
  #
  # Parameters: 
  #
  # Description: 
  #
  #***************************************************************************
  proc Register { } \
  {
    #puts "::Auto_Test::Register:  $tile_path"

    set tile_path  [get_tile  $tile_path]    

    return
  }
  

  #***************************************************************************
  #
  # Name: get_tile
  #
  # Parameters: any child widget of the tile
  #
  # Description: Get the name of the tile's parent frame
  #
  #***************************************************************************
  proc get_tile { widget } \
  {
    while { 1 } \
    {
      switch -- [winfo class $widget] \
      {
        "AUTO_TEST"  { return $widget }
      }
        
      set widget [winfo parent $widget]
    }


    return     
  }


  #***************************************************************************
  #
  # Name: verify_test_data
  #
  # Parameters:
  #
  # Description: Verifies that the test data exists
  #
  #***************************************************************************
  proc verify_test_data { } \
  {
    #puts "verify_test_data"
    
    variable  data_file_name
    variable  data_working_dir
  
    set file_name  [file join  $data_working_dir  $data_file_name]

    #puts "file_name:  $file_name"
    
    # Determine if the test data working directory exists,
    # If not, create it
    if { ![file exists $data_working_dir] } \
    {
      #puts "directory does not exist"
      
      file mkdir $data_working_dir

      #puts "directory: $data_working_dir created"
    }
    
    # Determine if the test data file exists,
    # If not, create it
    if { ![file exists $file_name] } \
    {
      #puts "file does not exist"

      # Create the file
      set file_id  [open  $file_name  w]
      
      close  $file_id

      #puts "file: $file_name created"

      # Create the default test data
      create_default_test_data
    }
        
    return
  }


  #***************************************************************************
  #
  # Name: create_default_test_data
  #
  # Parameters:
  #
  # Description: Creates the default test data
  #
  #***************************************************************************
  proc create_default_test_data { } \
  {
    #puts "create_default_test_data"
    
    variable  test_data

    # Configure the variables for the new radio
    set  test_data(1,test_title)  "Analog Receiver Test"

    set  test_data(1,chan_info_types)  "CTCSS  DCS"
    set  test_data(1,chan_info_type)   ""

    set  test_data(1,chan,1)  ""
    set  test_data(1,freq,1)  ""
    set  test_data(1,info,1)  ""
    set  test_data(1,chan,2)  ""
    set  test_data(1,freq,2)  ""
    set  test_data(1,info,2)  ""
    set  test_data(1,chan,3)  ""
    set  test_data(1,freq,3)  ""
    set  test_data(1,info,3)  ""
    set  test_data(1,chan,4)  ""
    set  test_data(1,freq,4)  ""
    set  test_data(1,info,4)  ""
    set  test_data(1,chan,5)  ""
    set  test_data(1,freq,5)  ""
    set  test_data(1,info,5)  ""
    set  test_data(1,chan,6)  ""
    set  test_data(1,freq,6)  ""
    set  test_data(1,info,6)  ""
    
    set  test_data(1,setup,1,label)   "CTCSS/DCS Deviation"
    set  test_data(1,setup,1,value)   "0.5"
    set  test_data(1,setup,1,units)   "kHz"

    set  test_data(1,p_f,1,label)   "Distortion"
    set  test_data(1,p_f,1,max)     "10.00"
    set  test_data(1,p_f,1,min)     "0.00"
    set  test_data(1,p_f,1,units)   "%"

    set  test_data(1,p_f,2,label)   "RF Threshold"
    set  test_data(1,p_f,2,max)     "-116.0"
    set  test_data(1,p_f,2,min)     ""
    set  test_data(1,p_f,2,units)   "dBm"
    
    set  test_data(1,p_f,3,label)   "SINAD"
    set  test_data(1,p_f,3,max)     ""
    set  test_data(1,p_f,3,min)     "12.0"
    set  test_data(1,p_f,3,units)   "dB"
    
    set  test_data(1,p_f,4,label)   ""
    set  test_data(1,p_f,4,max)     ""
    set  test_data(1,p_f,4,min)     ""
    set  test_data(1,p_f,4,units)   ""
    
    set  test_data(1,p_f,5,label)   ""
    set  test_data(1,p_f,5,max)     ""
    set  test_data(1,p_f,5,min)     ""
    set  test_data(1,p_f,5,units)   ""

    set  test_data(2,test_title)  "Analog Transmitter Test"

    set  test_data(2,chan_info_types)  ""
    set  test_data(2,chan_info_type)   ""

    set  test_data(2,chan,1)  ""
    set  test_data(2,freq,1)  ""
    set  test_data(2,info,1)  ""
    set  test_data(2,chan,2)  ""
    set  test_data(2,freq,2)  ""
    set  test_data(2,info,2)  ""
    set  test_data(2,chan,3)  ""
    set  test_data(2,freq,3)  ""
    set  test_data(2,info,3)  ""
    set  test_data(2,chan,4)  ""
    set  test_data(2,freq,4)  ""
    set  test_data(2,info,4)  ""
    set  test_data(2,chan,5)  ""
    set  test_data(2,freq,5)  ""
    set  test_data(2,info,5)  ""
    set  test_data(2,chan,6)  ""
    set  test_data(2,freq,6)  ""
    set  test_data(2,info,6)  ""
    
    set  test_data(2,setup,1,label)   "Audio inj. level"
    set  test_data(2,setup,1,value)   "1.00"
    set  test_data(2,setup,1,units)   "V p-p"

    set  test_data(2,p_f,1,label)   "Power"
    set  test_data(2,p_f,1,max)     ""
    set  test_data(2,p_f,1,min)     ""
    set  test_data(2,p_f,1,units)   "W"
    
    set  test_data(2,p_f,2,label)   "Frequency Error"
    set  test_data(2,p_f,2,max)     ""
    set  test_data(2,p_f,2,min)     ""
    set  test_data(2,p_f,2,units)   "Hz"
    
    set  test_data(2,p_f,3,label)   "FM Deviation"
    set  test_data(2,p_f,3,max)     ""
    set  test_data(2,p_f,3,min)     ""
    set  test_data(2,p_f,3,units)   "kHz"

    set  test_data(2,p_f,4,label)   "Distortion"
    set  test_data(2,p_f,4,max)     "10"
    set  test_data(2,p_f,4,min)     "0"
    set  test_data(2,p_f,4,units)   "%"
    
    set  test_data(2,p_f,5,label)   ""
    set  test_data(2,p_f,5,max)     ""
    set  test_data(2,p_f,5,min)     ""
    set  test_data(2,p_f,5,units)   ""
    
    set  test_data(3,test_title)  "Digital Receiver Test"

    set  test_data(3,chan_info_types)  "NAC"
    set  test_data(3,chan_info_type)   ""

    set  test_data(3,chan,1)  ""
    set  test_data(3,freq,1)  ""
    set  test_data(3,info,1)  ""
    set  test_data(3,chan,2)  ""
    set  test_data(3,freq,2)  ""
    set  test_data(3,info,2)  ""
    set  test_data(3,chan,3)  ""
    set  test_data(3,freq,3)  ""
    set  test_data(3,info,3)  ""
    set  test_data(3,chan,4)  ""
    set  test_data(3,freq,4)  ""
    set  test_data(3,info,4)  ""
    set  test_data(3,chan,5)  ""
    set  test_data(3,freq,5)  ""
    set  test_data(3,info,5)  ""
    set  test_data(3,chan,6)  ""
    set  test_data(3,freq,6)  ""
    set  test_data(3,info,6)  ""
    
    set  test_data(3,setup,1,label)   "SID"
    set  test_data(3,setup,1,value)   "000001"
    set  test_data(3,setup,1,units)   ""

    set  test_data(3,setup,2,label)   "TGID"
    set  test_data(3,setup,2,value)   "0001"
    set  test_data(3,setup,2,units)   ""

    set  test_data(3,p_f,1,label)   "RF Threshold"
    set  test_data(3,p_f,1,max)     "-118.0"
    set  test_data(3,p_f,1,min)     ""
    set  test_data(3,p_f,1,units)   "dBm"
    
    set  test_data(3,p_f,2,label)   ""
    set  test_data(3,p_f,2,max)     ""
    set  test_data(3,p_f,2,min)     ""
    set  test_data(3,p_f,2,units)   ""
    
    set  test_data(3,p_f,3,label)   ""
    set  test_data(3,p_f,3,max)     ""
    set  test_data(3,p_f,3,min)     ""
    set  test_data(3,p_f,3,units)   ""
    
    set  test_data(3,p_f,4,label)   ""
    set  test_data(3,p_f,4,max)     ""
    set  test_data(3,p_f,4,min)     ""
    set  test_data(3,p_f,4,units)   ""

    set  test_data(3,p_f,5,label)   ""
    set  test_data(3,p_f,5,max)     ""
    set  test_data(3,p_f,5,min)     ""
    set  test_data(3,p_f,5,units)   ""

    set  test_data(4,test_title)  "Digital Transmitter Test"

    set  test_data(4,chan_info_types)  "NAC"
    set  test_data(4,chan_info_type)   ""

    set  test_data(4,chan,1)  ""
    set  test_data(4,freq,1)  ""
    set  test_data(4,info,1)  ""
    set  test_data(4,chan,2)  ""
    set  test_data(4,freq,2)  ""
    set  test_data(4,info,2)  ""
    set  test_data(4,chan,3)  ""
    set  test_data(4,freq,3)  ""
    set  test_data(4,info,3)  ""
    set  test_data(4,chan,4)  ""
    set  test_data(4,freq,4)  ""
    set  test_data(4,info,4)  ""
    set  test_data(4,chan,5)  ""
    set  test_data(4,freq,5)  ""
    set  test_data(4,info,5)  ""
    set  test_data(4,chan,6)  ""
    set  test_data(4,freq,6)  ""
    set  test_data(4,info,6)  ""
    
    set  test_data(4,p_f,1,label)   "Power"
    set  test_data(4,p_f,1,max)     ""
    set  test_data(4,p_f,1,min)     ""
    set  test_data(4,p_f,1,units)   "W"
    
    set  test_data(4,p_f,2,label)   "Frequency Error"
    set  test_data(4,p_f,2,max)     ""
    set  test_data(4,p_f,2,min)     ""
    set  test_data(4,p_f,2,units)   "Hz"
    
    set  test_data(4,p_f,3,label)   "C4FM"
    set  test_data(4,p_f,3,max)     ""
    set  test_data(4,p_f,3,min)     ""
    set  test_data(4,p_f,3,units)   "%"
    
    set  test_data(4,p_f,4,label)   ""
    set  test_data(4,p_f,4,max)     ""
    set  test_data(4,p_f,4,min)     ""
    set  test_data(4,p_f,4,units)   ""
    
    set  test_data(4,p_f,5,label)   ""
    set  test_data(4,p_f,5,max)     ""
    set  test_data(4,p_f,5,min)     ""
    set  test_data(4,p_f,5,units)   ""
    
    set  test_data(4,p_f,6,label)   ""
    set  test_data(4,p_f,6,max)     ""
    set  test_data(4,p_f,6,min)     ""
    set  test_data(4,p_f,6,units)   ""
    
    set  test_data(5,test_title)  "MDC-1200 Emergency Test"

    set  test_data(5,chan_info_types)  ""
    set  test_data(5,chan_info_type)   ""

    set  test_data(5,chan,1)  ""
    set  test_data(5,freq,1)  ""
    set  test_data(5,info,1)  ""
    set  test_data(5,chan,2)  ""
    set  test_data(5,freq,2)  ""
    set  test_data(5,info,2)  ""
    set  test_data(5,chan,3)  ""
    set  test_data(5,freq,3)  ""
    set  test_data(5,info,3)  ""
    set  test_data(5,chan,4)  ""
    set  test_data(5,freq,4)  ""
    set  test_data(5,info,4)  ""
    set  test_data(5,chan,5)  ""
    set  test_data(5,freq,5)  ""
    set  test_data(5,info,5)  ""
    set  test_data(5,chan,6)  ""
    set  test_data(5,freq,6)  ""
    set  test_data(5,info,6)  ""
    
    set  test_data(5,p_f,1,label)   "Power"
    set  test_data(5,p_f,1,max)     "10.0"
    set  test_data(5,p_f,1,min)     "0.0"
    set  test_data(5,p_f,1,units)   "W"

    set  test_data(5,p_f,2,label)   ""
    set  test_data(5,p_f,2,max)     ""
    set  test_data(5,p_f,2,min)     ""
    set  test_data(5,p_f,2,units)   ""

    set  test_data(5,p_f,3,label)   ""
    set  test_data(5,p_f,3,max)     ""
    set  test_data(5,p_f,3,min)     ""
    set  test_data(5,p_f,3,units)   ""

    set  test_data(5,p_f,4,label)   ""
    set  test_data(5,p_f,4,max)     ""
    set  test_data(5,p_f,4,min)     ""
    set  test_data(5,p_f,4,units)   ""

    set  test_data(5,p_f,5,label)   ""
    set  test_data(5,p_f,5,max)     ""
    set  test_data(5,p_f,5,min)     ""
    set  test_data(5,p_f,5,units)   ""

    set  test_data(5,p_f,6,label)   ""
    set  test_data(5,p_f,6,max)     ""
    set  test_data(5,p_f,6,min)     ""
    set  test_data(5,p_f,6,units)   ""
    
    return
  }
  

  #***************************************************************************
  #
  # Test Information procedures
  #
  #***************************************************************************
  #***************************************************************************
  #
  # Name: test_info_display
  #
  # Parameters: widget - invoking widget tile path
  #
  # Description: Displays the Test Information tile
  #
  #***************************************************************************
  proc test_info_display { widget } \
  {
    #puts "test_info_display:  $widget"

    set tile_path  [get_tile  $widget]
    
    # Display the tile
    grid remove ${tile_path}.frm_test_select
    grid remove ${tile_path}.frm_test_config
    grid remove ${tile_path}.frm_test_results
    grid remove ${tile_path}.frm_test_connect_drawing

    Screens::position_widget  ${tile_path}.frm_test_info
    
    # Remove the New and Delete Radio entry options from the screen
    grid remove  ${tile_path}.frm_test_info.lbl_new_radio_type
    grid remove  ${tile_path}.frm_test_info.ent_new_radio_type
    grid remove  ${tile_path}.frm_test_info.lbl_del_radio_type
    grid remove  ${tile_path}.frm_test_info.ent_del_radio_type

    # Remove the New and Delete Test type entry options from the screen
    grid remove  ${tile_path}.frm_test_info.lbl_new_test_type
    grid remove  ${tile_path}.frm_test_info.ent_new_test_type
    grid remove  ${tile_path}.frm_test_info.lbl_del_test_type
    grid remove  ${tile_path}.frm_test_info.ent_del_test_type

    # Remove the New and Delete Test type Cancel button
    grid remove  ${tile_path}.frm_test_info.btn_add_delete_cancel


    # Update the DATE and TIME
    update_date_time
    
    # Update the radio type, and load the correct test data
    update_radio_type
    
    # Set the softkeys
    # Softkey #2:  Start
    ::View::AssignSoftkey  $tile_path  2  "CONTINUE"  \
                                "::Auto_Test::test_select_display  $tile_path"

    # Softkey #3:  Add radio type
    ::View::AssignSoftkey  $tile_path  3  "ADD\nRADIO"  \
                             "::Auto_Test::display_add_radio_type  $tile_path"

    # Softkey #4:  Delete radio type
    ::View::AssignSoftkey  $tile_path  4  "DELETE\nRADIO"  \
                          "::Auto_Test::display_delete_radio_type  $tile_path"

    # Softkey #5:  Add test type
    ::View::AssignSoftkey  $tile_path  5  "ADD\nTEST TYPE"  \
                             "::Auto_Test::display_add_test_type  $tile_path"

    # Softkey #6:  Delete test type
    ::View::AssignSoftkey  $tile_path  6  "DELETE\nTEST TYPE"  \
                          "::Auto_Test::display_delete_test_type  $tile_path"

    # Softkey #7:
    ::View::AssignSoftkey  $tile_path  7  "CONNECT\nDRAWING"  \
                      "::Auto_Test::test_connect_drawing_display  $tile_path"

    # RETURN Softkey
    ::View::AssignReturn  $tile_path  ""
    
    return
  }
  
  
  #***************************************************************************
  #
  # Name: update_radio_type
  #
  # Parameters:
  #
  # Description: Loads the test data for the new radio type
  #
  #***************************************************************************
  proc update_radio_type { } \
  {
    #puts "update_radio_type"

    variable  prev_radio_type
    variable  test_radio_type
    
    recall_test_data
    
    # Save this value for the next pass through
    set prev_radio_type  $test_radio_type
    
    return
  }
  

  #***************************************************************************
  #
  # Name: save_test_data
  #
  # Parameters: 
  #
  # Description: Saves the present test data
  #
  #***************************************************************************
  proc save_test_data { } \
  {
    #puts "save_test_data"

    variable  data_file_name
    variable  data_working_dir

    variable  prev_radio_type
    variable  test_radio_type
    
    variable  test_data
    
    variable  test_data_save
    
    set test_data_save  "Saving present config..."
    
    update idle
      
    # Open the present test data file and write all but the radio type
    # data over to a new test data file.
    # This effectively deletes the present radio type data from the file
    # in order to add the new data
    
    # Open the present data file
    set old_data_file  [file join  $data_working_dir  $data_file_name]
    
    #puts "old_data_file:  $old_data_file"
    
    set old_data_file_id  [open  $old_data_file  r]
          
    # Open the new data file
    set new_data_file  [file join  $data_working_dir  ${data_file_name}.new]
    
    #puts "new_data_file:  $new_data_file"
    
    set new_data_file_id  [open  $new_data_file  w+]
    
    set found      0
    set data_read  0
    
    set end_test_radio_type  "END $prev_radio_type"


    # Find the radio type section
    while { $data_read != -1 } \
    {
      set data_read  [gets  $old_data_file_id  file_line]
      
      set file_line  [string trim  $file_line]
      
      #puts "file_line:  $file_line"
      
      # If the radio type is found, the data will not be copied over
      if { [string compare  $file_line  $prev_radio_type] == 0 } \
      {
        set found  1
        
        #puts "found:  $found"
      }
    
      if { $found == 0 } \
      {
        puts $new_data_file_id  "$file_line"
      }

      # If the end of the radio type section is found, 
      # copy over the other lines in the file
      if { [string compare  $file_line  $end_test_radio_type] == 0 } \
      {
        set found  0
        
        #puts "found:  $found"
      }
    }
    
    
    # Add the new radio type data
    # Save the lines to the new file
    puts $new_data_file_id  "$prev_radio_type"

    set index1  1
    
    # Write all available data the test data file
    while { [info exists  test_data($index1,test_title)] } \
    {
      if { $index1 > 1 } \
      {
        puts $new_data_file_id ""
      }
      
      puts $new_data_file_id  \
       "set test_data($index1,test_title)  \"$test_data($index1,test_title)\""

      puts $new_data_file_id ""
      
      puts $new_data_file_id  \
       "set test_data($index1,chan_info_types)  \"$test_data($index1,chan_info_types)\""

      puts $new_data_file_id  \
       "set test_data($index1,chan_info_type)  \"$test_data($index1,chan_info_type)\""

      puts $new_data_file_id ""

      set index2  1
      
      # Write all available channel and frequency values the test data file
      while { [info exists  test_data($index1,chan,$index2)] } \
      {
        puts $new_data_file_id  \
          "set test_data($index1,chan,$index2)  \"$test_data($index1,chan,$index2)\""

        puts $new_data_file_id  \
          "set test_data($index1,freq,$index2)  \"$test_data($index1,freq,$index2)\""
        
        puts $new_data_file_id  \
          "set test_data($index1,info,$index2)  \"$test_data($index1,info,$index2)\""

        incr index2  1
      }
      

      set index2  1
      
      # Write all available setup values the test data file
      while { [info exists  test_data($index1,setup,$index2,label)] } \
      {
        puts $new_data_file_id  \
          "set test_data($index1,setup,$index2,label)  \"$test_data($index1,setup,$index2,label)\""
                                
        puts $new_data_file_id  \
          "set test_data($index1,setup,$index2,value)  \"$test_data($index1,setup,$index2,value)\""
                                  
        puts $new_data_file_id  \
          "set test_data($index1,setup,$index2,units)  \"$test_data($index1,setup,$index2,units)\""
      
        incr index2  1
      }

      set index2  1

      # Write all available pass/fail values the test data file
      while { [info exists  test_data($index1,p_f,$index2,label)] } \
      {
        puts $new_data_file_id  \
          "set test_data($index1,p_f,$index2,label)  \"$test_data($index1,p_f,$index2,label)\""
                                
        puts $new_data_file_id  \
          "set test_data($index1,p_f,$index2,max)  \"$test_data($index1,p_f,$index2,max)\""
                                  
        puts $new_data_file_id  \
          "set test_data($index1,p_f,$index2,min)  \"$test_data($index1,p_f,$index2,min)\""
                                  
        puts $new_data_file_id  \
          "set test_data($index1,p_f,$index2,units)  \"$test_data($index1,p_f,$index2,units)\""
      
        incr index2  1
      }
      

      incr index1  1
    }
            
    puts $new_data_file_id "END $prev_radio_type"
    
    set test_data_save  ""
    
    update idle

    # Close the files
    close  $old_data_file_id
    close  $new_data_file_id
    
    # Delete the old file
    file delete  $old_data_file
    
    # Rename the new file
    file rename  $new_data_file  $old_data_file
    

    return
  }


  #***************************************************************************
  #
  # Name: recall_test_data
  #
  # Parameters: 
  #
  # Description: Recalls the test data from the file
  #
  #***************************************************************************
  proc recall_test_data { } \
  {
    #puts "recall_test_data"

    variable  data_file_name
    variable  data_working_dir

    variable  prev_radio_type
    variable  test_radio_type
    
    variable  test_data
    
    variable  test_data_save
    
    set test_data_save  "Recalling config..."
    
    update idle

    # Open the test data file and get the values for this radio type
    # Open the file
    set data_file  [file join  $data_working_dir  $data_file_name]
      
    set create_defaults  1;  # Assume the test data is not defined in the file

    if { [file exists  $data_file] } \
    {
      # If the file exists, read the data from it
      set data_file_id  [open  $data_file  r]
    
      # If the end of the radio type section is found, 
      # the file line evaluations will stop.
      set end_test_radio_type  "END $test_radio_type"
      
      set found            0
      set data_read        0
        
      # Find the radio type section
      while { $data_read != -1 } \
      {
        # Read the lines in and 'eval' them one at a time
        set data_read  [gets  $data_file_id  file_line]
          
        set file_line  [string trim  "$file_line"]
        
        #puts "file_line:  $file_line"
          
        # If the end of the radio type data is found, 
        # stop the file line evaluations
        if { [string compare  "$file_line"  "$end_test_radio_type"] == 0 } \
        {
          set found  0
          
          #puts "found:  $found"
      
          # Stop the loop
          set data_read  -1
        }
      
        if { $found == 1 } \
        {
          #puts "evaluate the file line"
          #puts "$file_line"
          
          eval "$file_line"
        }
      
        # If the radio type is found, the file lines will be evaluated to
        # set the 'test_data' variables
        if { [string compare  "$file_line"  "$test_radio_type"] == 0 } \
        {
          set found  1
      
          # If the radio type was found in the file do not create the defaults
          set create_defaults  0
      
          #puts "found:  $found"
        }
      }
    
      # Close the file
      close  $data_file_id
    } \
    else \
    {
      # If the file does not exist, create it
      set data_file_id  [open  $data_file  w+]
    }
    
    if { $create_defaults == 1 } \
    {
      # If the file does not exist or if the radio type was not found 
      # in the file create the default test_data
      create_default_test_data
    }
    
    set test_data_save  ""
    
    update idle

    
    return
  }
  

  #***************************************************************************
  #
  # Name: update_date_time
  #
  # Parameters:
  #
  # Description: Updates the date and time of the the test to system time
  #              when the START softkey is pressed
  #
  #***************************************************************************
  proc update_date_time { } \
  {
    #puts "update_date_time"

    variable  test_date
    variable  test_time

    set date_time  [clock format [clock seconds]]

    # Example:  Wed Nov 13 18:36:05  2002
    
    foreach { weekday  month  month_day  time year } $date_time \
    {
      set test_date  "$month.  $month_day,  $year"
      set test_time  "$time"
    }

    return
  }


  #***************************************************************************
  #
  # Name: display_add_radio_type
  #
  # Parameters: widget - invoking widget tile path
  #
  # Description: Displays an entry field to enter a new radio type
  #
  #***************************************************************************
  proc display_add_radio_type { widget } \
  {
    #puts "display_add_radio_type:  $widget"

    variable  new_radio_type
    variable  test_radio_type_menu
     
    set tile_path  [get_tile  $widget]
    
    set new_radio_type  ""

    # Remove any fields that are possibly displayed in this area
    grid remove  ${tile_path}.frm_test_info.lbl_new_radio_type
    grid remove  ${tile_path}.frm_test_info.ent_new_radio_type
    grid remove  ${tile_path}.frm_test_info.lbl_del_radio_type
    grid remove  ${tile_path}.frm_test_info.ent_del_radio_type
    grid remove  ${tile_path}.frm_test_info.lbl_new_test_type
    grid remove  ${tile_path}.frm_test_info.ent_new_test_type
    grid remove  ${tile_path}.frm_test_info.lbl_del_test_type
    grid remove  ${tile_path}.frm_test_info.ent_del_test_type
    grid remove  ${tile_path}.frm_test_info.btn_add_delete_cancel

    # Get the name of the new radio type
    Screens::position_widget  ${tile_path}.frm_test_info.lbl_new_radio_type
    Screens::position_widget  ${tile_path}.frm_test_info.ent_new_radio_type
    Screens::position_widget  ${tile_path}.frm_test_info.btn_add_delete_cancel
    
    ## Softkey #7:  Cancel
    #::View::AssignSoftkey  $widget  7  "CANCEL"  \
    #                                "::Auto_Test::test_info_display  $widget"

    return
  }


  #***************************************************************************
  #
  # Name: add_radio_type
  #
  # Parameters: widget - invoking widget tile path
  #
  # Description: Adds a new radio type to the menu
  #
  #***************************************************************************
  proc add_radio_type { widget } \
  {
    #puts "add_radio_type:  $widget"

    variable  test_data
 
    variable  new_radio_type
    variable  test_radio_type_menu
     
    set tile_path  [get_tile  $widget]

    grid remove  ${tile_path}.frm_test_info.lbl_new_radio_type
    grid remove  ${tile_path}.frm_test_info.ent_new_radio_type
    grid remove  ${tile_path}.frm_test_info.btn_add_delete_cancel

    if { $new_radio_type != "" } \
    {
      # Save the present test data
      save_test_data

      # Add the new radio type to the menu
      set menu_name  ${tile_path}.frm_test_info.mnu_test_radio_type.menu
      
      set last_index  [$menu_name  index  end]
      
      incr last_index  1
      
      $menu_name add radiobutton                              \
                    -accel     $last_index                    \
                    -label     $new_radio_type                \
                    -value     $new_radio_type                \
                    -variable  ::Auto_Test::test_radio_type   \
                    -command   "::Auto_Test::update_radio_type"

      # Add the new radio type to the menu list variable
      lappend test_radio_type_menu  "$new_radio_type"
    
      create_default_test_data
      
      # Display the new selection
      $menu_name  invoke  end
    }

    return
  }


  #***************************************************************************
  #
  # Name: display_delete_radio_type
  #
  # Parameters: widget - invoking widget tile path
  #
  # Description: Displays an entry field to enter a radio type to delete
  #
  #***************************************************************************
  proc display_delete_radio_type { widget } \
  {
    #puts "display_delete_radio_type:  $widget"

    variable  del_radio_type
    variable  test_radio_type_menu
     
    set tile_path  [get_tile  $widget]
    
    set del_radio_type  ""

    # Remove any fields that are possibly displayed in this area
    grid remove  ${tile_path}.frm_test_info.lbl_new_radio_type
    grid remove  ${tile_path}.frm_test_info.ent_new_radio_type
    grid remove  ${tile_path}.frm_test_info.lbl_del_radio_type
    grid remove  ${tile_path}.frm_test_info.ent_del_radio_type
    grid remove  ${tile_path}.frm_test_info.lbl_new_test_type
    grid remove  ${tile_path}.frm_test_info.ent_new_test_type
    grid remove  ${tile_path}.frm_test_info.lbl_del_test_type
    grid remove  ${tile_path}.frm_test_info.ent_del_test_type
    grid remove  ${tile_path}.frm_test_info.btn_add_delete_cancel

    # Get the name of the new radio type
    Screens::position_widget  ${tile_path}.frm_test_info.lbl_del_radio_type
    Screens::position_widget  ${tile_path}.frm_test_info.ent_del_radio_type
    Screens::position_widget  ${tile_path}.frm_test_info.btn_add_delete_cancel
    

    ## Softkey #7:  Cancel
    #::View::AssignSoftkey  $widget  7  "CANCEL"  \
    #                                "::Auto_Test::test_info_display  $widget"

    return
  }


  #***************************************************************************
  #
  # Name: delete_radio_type
  #
  # Parameters: widget - invoking widget tile path
  #
  # Description: Deletes a radio type from the menu
  #
  #***************************************************************************
  proc delete_radio_type { widget } \
  {
    #puts "delete_radio_type:  $widget"

    variable  test_data
 
    variable  del_radio_type
    variable  test_radio_type_menu
     
    set tile_path  [get_tile  $widget]

    grid remove  ${tile_path}.frm_test_info.lbl_del_radio_type
    grid remove  ${tile_path}.frm_test_info.ent_del_radio_type
    grid remove  ${tile_path}.frm_test_info.btn_add_delete_cancel

    if { $del_radio_type != "" } \
    {
      # Delete the radio type from the menu
      set menu_name  ${tile_path}.frm_test_info.mnu_test_radio_type.menu
      
      set index  [lsearch -exact $test_radio_type_menu "$del_radio_type"]
        
      #puts "index:  $index"
      
      if { $index >= 0 } \
      {
        set test_radio_type_menu  [lreplace $test_radio_type_menu $index $index]
        
        #puts "test_radio_type_menu:  $test_radio_type_menu"
        
        # Delete the radio type from the menu
        $menu_name delete $index

        $menu_name invoke 0
      }
    }
    
    return
  }


  #***************************************************************************
  #
  # Name: display_add_test_type
  #
  # Parameters: widget - invoking widget tile path
  #
  # Description: Displays an entry field to enter a new test type
  #
  #***************************************************************************
  proc display_add_test_type { widget } \
  {
    #puts "display_add_test_type  $widget"

    variable  new_test_type
    variable  test_type_menu
     
    set tile_path  [get_tile  $widget]
    
    set new_test_type  ""

    # Remove any fields that are possibly displayed in this area
    grid remove  ${tile_path}.frm_test_info.lbl_new_radio_type
    grid remove  ${tile_path}.frm_test_info.ent_new_radio_type
    grid remove  ${tile_path}.frm_test_info.lbl_del_radio_type
    grid remove  ${tile_path}.frm_test_info.ent_del_radio_type
    grid remove  ${tile_path}.frm_test_info.lbl_new_test_type
    grid remove  ${tile_path}.frm_test_info.ent_new_test_type
    grid remove  ${tile_path}.frm_test_info.lbl_del_test_type
    grid remove  ${tile_path}.frm_test_info.ent_del_test_type
    grid remove  ${tile_path}.frm_test_info.btn_add_delete_cancel

    # Get the name of the new radio type
    Screens::position_widget  ${tile_path}.frm_test_info.lbl_new_test_type
    Screens::position_widget  ${tile_path}.frm_test_info.ent_new_test_type
    Screens::position_widget  ${tile_path}.frm_test_info.btn_add_delete_cancel
    

    ## Softkey #7:  Cancel
    #::View::AssignSoftkey  $widget  7  "CANCEL"  \
    #                                "::Auto_Test::test_info_display  $widget"

    return
  }


  #***************************************************************************
  #
  # Name: add_test_type
  #
  # Parameters: widget - invoking widget tile path
  #
  # Description: Adds a new test type to the menu
  #
  #***************************************************************************
  proc add_test_type { widget } \
  {
    #puts "add_test_type:  $widget"

    variable  new_test_type
    variable  test_type_menu
     
    set tile_path  [get_tile  $widget]

    grid remove  ${tile_path}.frm_test_info.lbl_new_test_type
    grid remove  ${tile_path}.frm_test_info.ent_new_test_type
    grid remove  ${tile_path}.frm_test_info.btn_add_delete_cancel

    if { $new_test_type != "" } \
    {
      # Add the new radio type to the menu
      set menu_name  ${tile_path}.frm_test_info.mnu_test_type.menu
      
      set last_index  [$menu_name  index  end]
      
      incr last_index  1
      
      $menu_name add radiobutton  -accel     $last_index            \
                                  -label     $new_test_type         \
                                  -value     $new_test_type         \
                                  -variable  ::Auto_Test::test_type

      # Add the new radio type to the menu list variable
      lappend test_type_menu  "$new_test_type"
    
      # Display the new selection
      $menu_name  invoke  end
    }

    return
  }


  #***************************************************************************
  #
  # Name: display_delete_test_type
  #
  # Parameters: widget - invoking widget tile path
  #
  # Description: Displays an entry field to enter a test type to delete
  #
  #***************************************************************************
  proc display_delete_test_type { widget } \
  {
    #puts "display_delete_test_type:  $widget"

    variable  del_test_type
    variable  test_type_menu
     
    set tile_path  [get_tile  $widget]
    
    set del_test_type  ""

    # Remove any fields that are possibly displayed in this area
    grid remove  ${tile_path}.frm_test_info.lbl_new_radio_type
    grid remove  ${tile_path}.frm_test_info.ent_new_radio_type
    grid remove  ${tile_path}.frm_test_info.lbl_del_radio_type
    grid remove  ${tile_path}.frm_test_info.ent_del_radio_type
    grid remove  ${tile_path}.frm_test_info.lbl_new_test_type
    grid remove  ${tile_path}.frm_test_info.ent_new_test_type
    grid remove  ${tile_path}.frm_test_info.lbl_del_test_type
    grid remove  ${tile_path}.frm_test_info.ent_del_test_type
    grid remove  ${tile_path}.frm_test_info.btn_add_delete_cancel

    # Get the name of the new radio type
    Screens::position_widget  ${tile_path}.frm_test_info.lbl_del_test_type
    Screens::position_widget  ${tile_path}.frm_test_info.ent_del_test_type
    Screens::position_widget  ${tile_path}.frm_test_info.btn_add_delete_cancel
    

    ## Softkey #7:  Cancel
    #::View::AssignSoftkey  $widget  7  "CANCEL"  \
    #                                "::Auto_Test::test_info_display  $widget"

    return
  }


  #***************************************************************************
  #
  # Name: delete_test_type
  #
  # Parameters: widget - invoking widget tile path
  #
  # Description: Deletes a test type from the menu
  #
  #***************************************************************************
  proc delete_test_type { widget } \
  {
    #puts "delete_test_type:  $widget"

    variable  del_test_type
    variable  test_type_menu
     
    set tile_path  [get_tile  $widget]

    grid remove  ${tile_path}.frm_test_info.lbl_del_test_type
    grid remove  ${tile_path}.frm_test_info.ent_del_test_type
    grid remove  ${tile_path}.frm_test_info.btn_add_delete_cancel

    if { $del_test_type != "" } \
    {
      # Delete the radio type from the menu
      set menu_name  ${tile_path}.frm_test_info.mnu_test_type.menu
      
      set index  [lsearch -exact $test_type_menu "$del_test_type"]
        
      #puts "index:  $index"
      
      if { $index >= 0 } \
      {
        set test_type_menu  [lreplace $test_type_menu $index $index]
        
        # Delete the radio type from the menu
        $menu_name delete $index

        $menu_name invoke 0
      }
    }
    
    return
  }



  #***************************************************************************
  #
  # Test Select procedures
  #
  #***************************************************************************
  #***************************************************************************
  #
  # Name: test_select_display
  #
  # Parameters: widget - invoking widget tile path
  #
  # Description: Displays the Test Select tile
  #
  #***************************************************************************
  proc test_select_display { widget } \
  {
    #puts "test_select_display:  $widget"

    variable  test_data
    variable  test_title
    
    set tile_path  [get_tile  $widget]

    # Display the tile
    grid remove ${tile_path}.frm_test_info
    grid remove ${tile_path}.frm_test_config
    grid remove ${tile_path}.frm_test_results
    grid remove ${tile_path}.frm_test_connect_drawing

    Screens::position_widget  ${tile_path}.frm_test_select
    
    # Set the softkeys
    # Softkey #2:  Continue
    ::View::AssignSoftkey  $tile_path  2  "CONTINUE"  \
                               "::Auto_Test::test_results_display  $tile_path"

    # Softkey #3:
    ::View::AssignSoftkey  $tile_path  3  ""  ""

    # Softkey #4:
    ::View::AssignSoftkey  $tile_path  4  ""  ""

    # Softkey #5:
    ::View::AssignSoftkey  $tile_path  5  ""  ""

    # Softkey #6:
    ::View::AssignSoftkey  $tile_path  6  ""  ""

    # Softkey #7:
    ::View::AssignSoftkey  $tile_path  7  ""  ""

    # Softkey #8:  RETURN
    ::View::AssignReturn  $tile_path  \
                                  "::Auto_Test::test_info_display  $tile_path"

    
    # Display the test options
    test_select_options  $tile_path
    
    return
  }


  #***************************************************************************
  #
  # Name: test_select_options
  #
  # Parameters: widget - invoking widget tile path
  #
  # Description: Displays the Test Select options
  #
  #***************************************************************************
  proc test_select_options { widget } \
  {
    #puts "test_select_options:  $widget"

    variable  test_data
    variable  test_title
    
    set tile_path  [get_tile  $widget]

    # Loop through test options and remove all that exist
    set index  1
    
    # If the option widgets exist, remove them from the display
    while { [winfo exists ${tile_path}.frm_test_select.chk_test${index}] } \
    {
      grid remove  ${tile_path}.frm_test_select.chk_test${index}
      grid remove  ${tile_path}.frm_test_select.btn_test${index}
      
      incr index  1
    }


    # Display the valid test options
    set index  1
    
    # If the title variable exists, display the test option
    while { [info exists  test_data($index,test_title)] } \
    {
      Screens::position_widget  ${tile_path}.frm_test_select.chk_test${index}
      Screens::position_widget  ${tile_path}.frm_test_select.btn_test${index}
      
      set test_title($index)  $test_data($index,test_title)
    
      incr index  1
    }
    
    return
  }



  #***************************************************************************
  #
  # Test Configuration procedures
  #
  #***************************************************************************
  #***************************************************************************
  #
  # Name: test_config_display
  #
  # Parameters: widget - invoking widget tile path
  #             num    - number of the configuration
  #
  # Description: Displays the Test Configuration tile
  #
  #***************************************************************************
  proc test_config_display { widget  test_number } \
  {
    #puts "test_config_display:  $widget,  $test_number"

    variable  test_num
    
    variable  test_data
    
    variable  chan_info_type
    variable  config_description
    
    set tile_path  [get_tile  $widget]

    # Set the global variable
    set test_num  $test_number
    
    # Display the tile
    grid remove ${tile_path}.frm_test_info
    grid remove ${tile_path}.frm_test_select
    grid remove ${tile_path}.frm_test_results
    grid remove ${tile_path}.frm_test_connect_drawing

    Screens::position_widget  ${tile_path}.frm_test_config
    
    # Set the softkeys
    # Softkey #2:
    ::View::AssignSoftkey  $tile_path  2  ""  ""

    # Softkey #3:
    ::View::AssignSoftkey  $tile_path  3  ""  ""

    # Softkey #4:
    ::View::AssignSoftkey  $tile_path  4  ""  ""

    # Softkey #5:
    ::View::AssignSoftkey  $tile_path  5  ""  ""

    # Softkey #6:
    ::View::AssignSoftkey  $tile_path  6  ""  ""

    # Softkey #7:
    ::View::AssignSoftkey  $tile_path  7  ""  ""

    # Softkey #8:  RETURN
    ::View::AssignReturn  $tile_path  \
                         "::Auto_Test::test_config_display_return  $tile_path"


    # Configure the test description
    set config_description  $test_data($test_number,test_title)
    
    # Configure the 'Misc. Info' options
    set_chan_info_options  $tile_path  $test_number
    
    # Configure the 'Misc. Info' field to the test data value
    if { ![info exists  test_data($test_number,chan_info_type)] } \
    {
      set test_data($test_number,chan_info_type)  ""
    }
      
    if { $test_data($test_number,chan_info_type) == "" } \
    {
      set chan_info_type  "NONE"
    } \
    else \
    {
      set chan_info_type  $test_data($test_number,chan_info_type)
    }
    

    # Configure the channel fields
    test_config_channel_fields  $tile_path  $test_number

    # Configure the setup fields
    test_config_setup_fields  $tile_path  $test_number

    # Display and configure the pass / fail fields
    test_config_pass_fail_fields  $tile_path  $test_number
    
    
    return
  }


  #***************************************************************************
  #
  # Name: test_config_channel_fields
  #
  # Parameters: tile_path - the invoking tile path
  #             test_num  - test number of the configuration
  #
  # Description: Configures the text variable of the channel fields.
  #
  #***************************************************************************
  proc test_config_channel_fields { tile_path  test_num } \
  {
    #puts "test_config_channel_fields:  $tile_path,  $test_num"

    variable  test_data

    set tile_path  [get_tile  $tile_path]

    # Display the proper fields for each test
    set frame_name  ${tile_path}.frm_test_config.frm_channels

    # Configure all fields with a textvariable
    for { set index  1 } { $index <= 6 } { incr index  1 } \
    { 
      ${frame_name}.ent_chan_index${index} config -textvariable \
                                 ::Auto_Test::test_data($test_num,chan,$index)
       
      ${frame_name}.ent_chan_freq${index} config -textvariable \
                                 ::Auto_Test::test_data($test_num,freq,$index)
       
      ${frame_name}.ent_chan_info${index} config -textvariable \
                                 ::Auto_Test::test_data($test_num,info,$index)
    }

    return
  }


  #***************************************************************************
  #
  # Name: test_config_display_return
  #
  # Parameters: tile_path - the invoking tile path
  #
  # Description: Performs procedure calls after the RETURN button is pressed
  #
  #***************************************************************************
  proc test_config_display_return { tile_path } \
  {
    #puts "test_config_display_return:  $tile_path"
  
    # Save the present data
    save_test_data
    
    # Display the Select screen
    ::Auto_Test::test_select_display  $tile_path    
      
    return
  }
  

  #***************************************************************************
  #
  # Name: test_config_setup_fields
  #
  # Parameters: tile_path - the invoking tile path
  #             test_num  - test number of the configuration
  #
  # Description: Configures the text variable of the setup entry fields.
  #
  #***************************************************************************
  proc test_config_setup_fields { tile_path  test_num } \
  {
    #puts "test_config_setup_fields:  $tile_path,  $test_num"

    variable  test_data

    set tile_path  [get_tile  $tile_path]

    # Display the proper fields for each test
    set frame_name  ${tile_path}.frm_test_config.frm_setup

    # Remove all of the possible setup fields
    set index  1
    
    # Loop through the setup fields
    # If the fields exist, remove them from the display
    while { [winfo exists ${frame_name}.lbl_setup${index}] } \
    {
      grid remove  ${frame_name}.lbl_setup${index}
      grid remove  ${frame_name}.ent_setup${index}
      grid remove  ${frame_name}.lbl_setup_units${index}

      incr index  1
    }

    set index  1
    set num_setups  0
    
    while { [info exists  test_data($test_num,setup,$index,label)] } \
    {
      # Determine if the Chan has a valid value
      # If it does, display the setup information
      if { $test_data($test_num,setup,$index,label) != "" } \
      {
        Screens::position_widget  ${frame_name}.lbl_setup${index}

        ${frame_name}.lbl_setup${index} config -textvariable \
                          ::Auto_Test::test_data($test_num,setup,$index,label)
    

        Screens::position_widget  ${frame_name}.ent_setup${index}

        ${frame_name}.ent_setup${index} config -textvariable \
                          ::Auto_Test::test_data($test_num,setup,$index,value)


        Screens::position_widget  ${frame_name}.lbl_setup_units${index}

        ${frame_name}.lbl_setup_units${index} config -textvariable \
                          ::Auto_Test::test_data($test_num,setup,$index,units)
      }
                                 
      incr index  1
    }
    
    return
  }


  #***************************************************************************
  #
  # Name: test_config_pass_fail_fields
  #
  # Parameters: tile_path - the invoking tile path
  #             test_num  - test number of the configuration
  #
  # Description: Configures the text variables of the pass / fail label, max,
  #              min, and units fields.
  #
  #***************************************************************************
  proc test_config_pass_fail_fields { tile_path  test_num } \
  {
    #puts "test_config_pass_fail_fields:  $tile_path,  $test_num"

    variable  test_data

    set tile_path  [get_tile  $tile_path]

    # Display the proper fields for each test
    set frame_name  ${tile_path}.frm_test_config.frm_pass_fail
    
    # Remove all of the possible pass/fail fields
    set index  1
    
    # Loop through the pass/fail fields
    # If the fields exist, remove them from the display
    while { [winfo exists ${frame_name}.lbl_pass_fail${index}] } \
    {
      grid remove  ${frame_name}.lbl_pass_fail${index}
      grid remove  ${frame_name}.lbl_pass_fail_max${index}
      grid remove  ${frame_name}.ent_pass_fail_max${index}
      grid remove  ${frame_name}.lbl_pass_fail_min${index}
      grid remove  ${frame_name}.ent_pass_fail_min${index}
      grid remove  ${frame_name}.lbl_pass_fail_units${index}

      incr index  1
    }

    set index  1
    set num_pass_fails  0
    
    # Loop through the pass/fail options
    # Determine if the Label variable exists.
    while { [info exists  test_data($test_num,p_f,$index,label)] } \
    {
      # Determine if the Label has a valid value
      # If it does, display the pass/fail information
      if { $test_data($test_num,p_f,$index,label) != "" } \
      {
        # Display the label, and set the textvariable of the label widget
        Screens::position_widget  ${frame_name}.lbl_pass_fail${index}

        ${frame_name}.lbl_pass_fail${index} config -textvariable \
                            ::Auto_Test::test_data($test_num,p_f,$index,label)
       
       
        # Display the 'max' label and the entry widget, and
        # set the textvariable of the entry widget
        Screens::position_widget  ${frame_name}.lbl_pass_fail_max${index}
        Screens::position_widget  ${frame_name}.ent_pass_fail_max${index}
       
        ${frame_name}.ent_pass_fail_max${index} config -textvariable \
                              ::Auto_Test::test_data($test_num,p_f,$index,max)

       
        # Display the 'min' label and the entry widget, and
        # set the textvariable of the entry widget
        Screens::position_widget  ${frame_name}.lbl_pass_fail_min${index}
        Screens::position_widget  ${frame_name}.ent_pass_fail_min${index}
       
        ${frame_name}.ent_pass_fail_min${index} config -textvariable \
                              ::Auto_Test::test_data($test_num,p_f,$index,min)


        # Display the 'units' label and set the textvariable 
        # of the label
        Screens::position_widget  ${frame_name}.lbl_pass_fail_units${index}
       
        ${frame_name}.lbl_pass_fail_units${index} config -textvariable \
                            ::Auto_Test::test_data($test_num,p_f,$index,units)
      }
      
      incr index  1
    }

    return
  }


  #***************************************************************************
  #
  # Name: set_chan_index
  #
  # Parameters: num - number of the index
  #
  # Description: Nulls out the index if it is set to '0'
  #
  #***************************************************************************
  proc set_chan_index { num } \
  {
    #puts "set_chan_index:  $num"
    
    variable  test_num
    variable  test_data
    
    if { $test_data($test_num,chan,$num) == "0" } \
    {
      set test_data($test_num,chan,$num)  ""
    }

    return
  }


  #***************************************************************************
  #
  # Name: set_chan_freq
  #
  # Parameters: num - number of the index
  #
  # Description: Nulls out the frequency if it is set to '0'
  #
  #***************************************************************************
  proc set_chan_freq { num } \
  {
    #puts "set_chan_freq:  $num"

    variable  test_num
    variable  test_data

    if { $test_data($test_num,freq,$num) == "0" } \
    {
      set test_data($test_num,freq,$num)  ""
    }

    return
  }


  #***************************************************************************
  #
  # Name: set_chan_info_type
  #
  # Parameters:
  #
  # Description: Sets the Channel 'Misc. Info' value
  #
  #***************************************************************************
  proc set_chan_info_type { } \
  {
    #puts "set_chan_info_type"

    variable  test_num
    variable  test_data
    variable  chan_info_type
    
    set test_data($test_num,chan_info_type)  $chan_info_type

    return
  }


  #***************************************************************************
  #
  # Name: set_chan_info
  #
  # Parameters: num - number of the index
  #
  # Description: Nulls out the info if it is set to '0'
  #
  #***************************************************************************
  proc set_chan_info { num } \
  {
    #puts "set_chan_info:  $num"

    variable  test_num
    variable  test_data

    if { $test_data($test_num,info,$num) == "0" } \
    {
      set test_data($test_num,info,$num)  ""
    }

    return
  }


  #***************************************************************************
  #
  # Name: set_pass_fail_max
  #
  # Parameters: num - number of the index
  #
  # Description: Nulls out the pass / fail max value if it is set to '-1'
  #
  #***************************************************************************
  proc set_pass_fail_max { num } \
  {
    #puts "set_pass_fail_max:  $num"

    variable  test_num
    variable  test_data

    if { $test_data($test_num,p_f,$num,max) == "-1" } \
    {
      set test_data($test_num,p_f,$num,max)  ""
    }

    return
  }


  #***************************************************************************
  #
  # Name: set_pass_fail_min
  #
  # Parameters: num - number of the index
  #
  # Description: Nulls out the pass / fail min value if it is set to '-1'
  #
  #***************************************************************************
  proc set_pass_fail_min { num } \
  {
    #puts "set_pass_fail_min:  $num"

    variable  test_num
    variable  test_data

    if { $test_data($test_num,p_f,$num,min) == "-1" } \
    {
      set test_data($test_num,p_f,$num,min)  ""
    }

    return
  }


  #***************************************************************************
  #
  # Name: set_chan_info_options
  #
  # Parameters: tile_path - tile path of the invoking widget
  #
  # Description: Sets the Channel 'Misc. Info' menu options
  #
  #***************************************************************************
  proc set_chan_info_options { tile_path  test_number } \
  {
    #puts "set_chan_info_options:  $tile_path,  $test_number"

    variable  test_data
    
    set tile_path  [get_tile  $tile_path]

    set frame_name  ${tile_path}.frm_test_config.frm_channels
    
    for { set index  1 } { $index <= 6 } { incr index  1 } \
    {
      set menu  ${frame_name}.mnu_chan_info${index}.menu

      # Destroy the existing menu
      destroy $menu
      
      # Create a new menu
      menu  $menu
      
      # Add the 'NONE' option to the menu
      $menu  add  radio  -label  "NONE"                         \
                         -value  "NONE"                         \
                         -variable  ::Auto_Test::chan_info_type \
                         -command   ::Auto_Test::set_chan_info_type

      # Get the options for the 'Misc. Info' menu
      foreach  option  $test_data($test_number,chan_info_types) \
      {
        if { $option != "" } \
        {
          # Add the options to the menu
          $menu  add  radio  -label  $option                        \
                             -value  $option                        \
                             -variable  ::Auto_Test::chan_info_type \
                             -command   ::Auto_Test::set_chan_info_type
        }
      }
    }
    
    return
  }



  #***************************************************************************
  #
  # Test Results procedures
  #
  #***************************************************************************
  #***************************************************************************
  #
  # Name: test_results_display
  #
  # Parameters: widget - invoking widget tile path
  #
  # Description: Displays the Test Results tile
  #
  #***************************************************************************
  proc test_results_display { widget } \
  {
    #puts "test_results_display:  $widget"

    set tile_path  [get_tile  $widget]

    # Display the tile
    grid remove ${tile_path}.frm_test_info
    grid remove ${tile_path}.frm_test_select
    grid remove ${tile_path}.frm_test_config
    grid remove ${tile_path}.frm_test_connect_drawing

    Screens::position_widget  ${tile_path}.frm_test_results
    

    ${tile_path}.frm_test_results.scrl_results config  \
                    -command "${tile_path}.frm_test_results.txt_results yview"
    
    ${tile_path}.frm_test_results.txt_results config  \
              -yscrollcommand "${tile_path}.frm_test_results.scrl_results set"

    
    # Set the softkeys
    # Softkey #2:  Start
    ::View::AssignSoftkey  $tile_path  2  "START"  \
                                         "::Auto_Test::test_start  $tile_path"

    # Softkey #3:  Abort
    ::View::AssignSoftkey  $tile_path  3  "ABORT"  \
                                         "::Auto_Test::test_abort  $tile_path"

    # Softkey #4:  Save
    ::View::AssignSoftkey  $tile_path  4  "SAVE"  \
                                  "::Auto_Test::test_results_save  $tile_path"

    # Softkey #5:  Recall
    ::View::AssignSoftkey  $tile_path  5  "RECALL"  \
                                "::Auto_Test::test_results_recall  $tile_path"

    # Softkey #6:  Clear
    ::View::AssignSoftkey  $tile_path  6  "CLEAR"  \
                                 "::Auto_Test::test_results_clear  $tile_path"

    # Softkey #7:  Print
    ::View::AssignSoftkey  $tile_path  7  "PRINT"  \
                                 "::Auto_Test::test_results_print  $tile_path"

    # Softkey #8:  RETURN
    ::View::AssignReturn  $tile_path \
                                "::Auto_Test::test_select_display  $tile_path"

    return
  }


  #***************************************************************************
  #
  # Name: test_start
  #
  # Parameters: widget - invoking widget tile path
  #
  # Description: Starts the tests
  #
  #***************************************************************************
  proc test_start { widget } \
  {
    #puts "test_start:  $widget"

    variable  results

    variable  abort_flag

    variable  test_data

    variable  test_date
    variable  test_time
    variable  test_unit_id
    variable  test_tech_name
    variable  test_radio_type
    variable  test_type
    
    variable  test_enable

    variable  test_start_time
    variable  test_stop_time
    variable  test_total_time

    set tile_path  [get_tile  $widget]

    set abort_flag  0
      
    # Get the reference for the time of the test
    set test_start_time  [expr int([clock seconds])]
    
    # Display the test info in the results box
    set results  ""
    set results  ""
    set results  "Date:        $test_date,  $test_time"
    set results  "ID Number:   $test_unit_id"
    set results  "Tech Name:   $test_tech_name"
    set results  "Radio Type:  $test_radio_type"
    set results  "Test Type:   $test_type"
    set results  ""
    
    # Display the test comment in the results box
    set comment  [${tile_path}.frm_test_info.txt_comment get  1.0  end]
    
    set results  "$comment"
    set results  ""

    set test_num  1
    
    # Loop as long as a valid test title is specified
    while { [info exists  test_data($test_num,test_title)] } \
    {
      #puts "test_num:  $test_num"
    
      if { $abort_flag == 1 } \
      {
        break
      }
      
      if { $test_enable($test_num) == 1 } \
      {
        # Create the test name
        set test_name  "::Test${test_num}::test"
        
        eval "$test_name  $tile_path"
      }
      
      incr test_num  1
    }


    if { $abort_flag == 0 } \
    {
      set results  ""
      set results  "-------> TEST COMPLETED <-------"
      set results  ""
    } \
    else \
    {
      set results  ""
      set results  "******** TEST ABORTED ********"
      set results  ""
    }

    # Determine the test time
    set test_stop_time  [expr int([clock seconds])]

    set test_total_time  [expr $test_stop_time - $test_start_time]

    set test_total_time_min  [expr $test_total_time / 60]

    set test_total_time_sec  [expr $test_total_time - ($test_total_time_min * 60)]

    set results  "Test Time:  $test_total_time_min  min   $test_total_time_sec  sec"

    return
  }


  #***************************************************************************
  #
  # Name: test_abort
  #
  # Parameters: widget - invoking widget tile path
  #
  # Description: Aborts the tests
  #
  #***************************************************************************
  proc test_abort { widget } \
  {
    #puts "test_abort:  $widget"

    variable  abort_flag

    set tile_path  [get_tile  $widget]

    set abort_flag  1
    
    return
  }


  #***************************************************************************
  #
  # Name: test_results_save
  #
  # Parameters: widget - invoking widget tile path
  #
  # Description: Saves the results
  #
  #***************************************************************************
  proc test_results_save { widget } \
  {
    #puts "test_results_save:  $widget"

    variable  file_ext
    variable  file_label
    variable  file_prefix
    variable  working_dir
    
    variable  save_flag

    set tile_path  [get_tile  $widget]
    
    set text_box  ${tile_path}.frm_test_results.txt_results
    
    if { [winfo exists $text_box] } \
    {
      $text_box config -state normal
    
      set file_buffer  [$text_box get  1.0  end]
    }
    
    # If no data exists, return
    if { $file_buffer == "" } \
    {
      return
    }
    
    # Produce a comma delimited version of the data
    regsub -all ":" $file_buffer "," file_buffer_csv

    set save_flag  [File_SaveUI::display_file_save  $tile_path       \
                                                    $working_dir     \
                                                    $file_buffer     \
                                                    $file_prefix     \
                                                    "$file_label"    \
                                                    $file_ext        \
                                                    NO_ENCRYPT       \
                                                    $file_buffer_csv \
                                                    MODES            \
                                                    NO_OPT_EXT]
    
    #puts "save_flag:  $save_flag"
                                                               
    # Get the file label after the file has been saved
    set file_label  $::File_SaveUI::file_label

    return
  }


  #***************************************************************************
  #
  # Name: test_results_recall
  #
  # Parameters: widget - invoking widget tile path
  #
  # Description: Recalls results saved to a file
  #
  #***************************************************************************
  proc test_results_recall { widget } \
  {
    #puts "test_results_recall:  $widget"
    
    variable  file_ext
    variable  file_prefix
    variable  working_dir
    
    variable  results

    set tile_path  [get_tile  $widget]
    
    set file_buffer  [::File_RecallUI::display_file_recall  $tile_path    \
                                                            $working_dir  \
                                                            $file_prefix  \
                                                            $file_ext     ]
                                                               
    #puts "file_buffer:  $file_buffer"

    # If the recalled results are valid, display them
    if { $file_buffer != "" } \
    {
      set results  "$file_buffer"
    }
    
    return
  }


  #***************************************************************************
  #
  # Name: test_results_clear
  #
  # Parameters: widget - invoking widget tile path
  #
  # Description: Clears the results textbox
  #
  #***************************************************************************
  proc test_results_clear { widget  args } \
  {
    #puts "test_results_clear:  $widget,  $args"
    
    variable  save_flag

    set tile_path  [get_tile  $widget]

    # Prompt the operator if the present data is not saved
    if { $save_flag == 0 } \
    {
      set title  "Warning"
      set prompt  "The results data has not been saved,\n\nSave them now?"
      
      set window_name  ${tile_path}.warning
      
      set result  [tk_dialog  $window_name  $title  $prompt  info  0  "YES"  "NO"]
    
      switch -- $result \
      {
        0 { set save_flag  -1;  test_results_save  $tile_path }
      
        1 { set save_flag  1 }
      }
    }    

    # Wait for the 'save_flag' to become valid
    while { $save_flag == -1 } \
    {
      vwait Auto_Test::save_flag
    }
    
    # Retest the 'save_flag' to verify
    if { $save_flag == 1 } \
    {
      set text_box  ${tile_path}.frm_test_results.txt_results
    
      if { [winfo exists $text_box] } \
      {
        $text_box config -state normal

        $text_box delete  1.0  end

        $text_box config -state disabled
      }
    }
    
    return
  }


  #***************************************************************************
  #
  # Name: test_results_print
  #
  # Parameters: widget - invoking widget tile path
  #
  # Description: Prints the results of the test
  #
  #***************************************************************************
  proc test_results_print { widget } \
  {
    #puts "test_results_print:  $widget"

    variable  results
    
    variable  printer_dir
    
    set tile_path  [get_tile  $widget]
    
    set printer_path  [file join  /  $printer_dir]
    
    set text_box  ${tile_path}.frm_test_results.txt_results
    
    $text_box config -state normal
    
    set last_index  [$text_box  index  end]
    
    set last_index  [expr int($last_index)]
    
    if { [winfo exists $text_box] } \
    {
      # Open the printer
      set error  [catch { set output_id  [open  $printer_path  w] } result]
      
      #puts "error:  $error"
      
      if { $error != 0 } \
      {
        # Error opening the printer
        return
      }
      
      # Set the buffering to 'none'
      fconfigure  $output_id  -buffering  none
      
      for { set index 1 } { $index <= $last_index } { incr index 1 } \
      {
        set print_buffer  [$text_box  get  ${index}.0  ${index}.end]
        
        #puts "${index}: $print_buffer"

        # Write the data to the file (printer)
        catch { puts  -nonewline  $output_id  "$print_buffer \r" } result
    
        ## Flush the printer queue
        #catch { flush  $output_id } result
      }

      # Write a 'form feed' to the file (printer)
      catch { puts  -nonewline  $output_id  "\f" } result

      ## Flush the printer queue
      #catch { flush  $output_id } result

      # Close the printer
      catch { close  $output_id } result

      $text_box config -state disabled
    }

    return
  }


  #***************************************************************************
  #
  # Name: test_results_output
  #
  # Parameters: widget - invoking widget tile path
  #
  # Description: Prints the results to the RESULTS text box
  #
  #***************************************************************************
  proc test_results_output { widget  args } \
  {
    #puts "test_results_output:  $widget,  $args"
    
    variable  results
    variable  save_flag
    
    set tile_path  [get_tile  $widget]
    
    set text_box  ${tile_path}.frm_test_results.txt_results
    
    if { [winfo exists $text_box] } \
    {
      $text_box config -state normal

      $text_box insert end "$results\n"

      $text_box see end

      $text_box config -state disabled
      
      set save_flag  0
    }
    
    return
  }


  #***************************************************************************
  #
  # Test Connect Drawing procedures
  #
  #***************************************************************************
  #***************************************************************************
  #
  # Name: test_connect_drawing_display
  #
  # Parameters: widget - invoking widget tile path
  #
  # Description: Displays the Test Connect Drawing
  #
  #***************************************************************************
  proc test_connect_drawing_display { widget } \
  {
    #puts "test_connect_drawing_display:  $widget"

    variable  test_data
    variable  test_title
    
    set tile_path  [get_tile  $widget]

    # Display the tile
    grid remove ${tile_path}.frm_test_info
    grid remove ${tile_path}.frm_test_config
    grid remove ${tile_path}.frm_test_results
    grid remove ${tile_path}.frm_test_connect_drawing

    Screens::position_widget  ${tile_path}.frm_test_connect_drawing
    
    # Set the softkeys
    # Softkey #2:  Continue
    ::View::AssignSoftkey  $tile_path  2  ""  ""

    # Softkey #3:
    ::View::AssignSoftkey  $tile_path  3  ""  ""

    # Softkey #4:
    ::View::AssignSoftkey  $tile_path  4  ""  ""

    # Softkey #5:
    ::View::AssignSoftkey  $tile_path  5  ""  ""

    # Softkey #6:
    ::View::AssignSoftkey  $tile_path  6  ""  ""

    # Softkey #7:
    ::View::AssignSoftkey  $tile_path  7  ""  ""

    # Softkey #8:  RETURN
    ::View::AssignReturn  $tile_path  \
                                  "::Auto_Test::test_info_display  $tile_path"

    set canvas_name  ${tile_path}.frm_test_connect_drawing.drawing
    
    # 2975
    ${canvas_name} create rect 70 20 440 180 -fill cornsilk2
    ${canvas_name} create rect 90 30 300 120 -fill white
    
    ${canvas_name} create text 195 50 -text "IFR 2975"  \
                                      -anchor c         \
                                      -font {-weight bold}
   
    # T/R Port
    ${canvas_name} create oval 370 150 390 170 -fill white
    
    ${canvas_name} create text 379 140 -text T/R
   
    # Audio I/O
    ${canvas_name} create rect 280 150 300 170 -fill white
    
    ${canvas_name} create text 307 140 -text "Aud I/O"
   
    # MIC
    ${canvas_name} create rect 255 150 275 170 -fill white
    
    ${canvas_name} create text 260 140 -text MIC
   
   
    # DVM
    ${canvas_name} create oval 200 150 220 170 -fill white
    
    ${canvas_name} create text 209 140 -text "DVM"
   
   
    # Radio
    ${canvas_name} create rect 340 250 440 400
    
    ${canvas_name} create text 380 265 -text "Antenna" -anchor c
    ${canvas_name} create text 350 290 -text "Speaker Out" -anchor w
    ${canvas_name} create text 350 320 -text "MIC In" -anchor w
    
    ${canvas_name} create text 390 365 -text "Radio\nunder test"  \
                                       -anchor c                  \
                                       -justify center            \
                                       -font {-weight bold}
   
   
    # MIC attachment
    ${canvas_name} create poly  80 250           \
                                220 250          \
                                220 330          \
                                200 380          \
                                100 380          \
                                 80 330          \
                                -fill cornsilk2  \
                                -outline black
    
    ${canvas_name} create text 150 340 -text "AC25007"  \
                                       -anchor c        \
                                       -font {-weight bold}
                                                           
    # MIC attachment - PTT button
    ${canvas_name} create rect 70 300  80 320 -fill black
   
   
    # MIC attachment - Audio In port
    ${canvas_name} create rect  90 230 120 250
    
    ${canvas_name} create text 105 260 -text "Aud\nIN"  \
                                       -justify center  \
                                       -anchor n
   
   
    # MIC attachment - Audio Out #1 port
    ${canvas_name} create rect 135 230 165 250
    
    ${canvas_name} create text 150 260 -text "Aud\nOut\n#1"  \
                                       -justify center       \
                                       -anchor n
   
   
    # MIC attachment - Audio Out #2 port
    ${canvas_name} create rect 180 230 210 250
    
    ${canvas_name} create text 195 260 -text "Aud\nOut\n#2"  \
                                       -justify center       \
                                       -anchor n
   
   
    # Cable (flat ribbon) - MIC attachment to 2975
    ${canvas_name} create line 160 380 160 400 -width 3 -fill gray
    ${canvas_name} create line 160 400 290 400 -width 3 -fill gray
    ${canvas_name} create line 290 400 290 160 -width 3 -fill gray
   
    ${canvas_name} create line 140 380 140 410 -width 3 -fill gray
    ${canvas_name} create line 140 410 265 410 -width 3 -fill gray
    ${canvas_name} create line 265 410 265 160 -width 3 -fill gray
   
   
    # Cable - T/R Port to Radio
    ${canvas_name} create line 380 160 380 250 -width 3
   
    # Cable - MIC attachment Audio Out #1 port to Radio MIC input
    ${canvas_name} create line 150 230 150 215 -width 2 -fill brown
    ${canvas_name} create line 150 215 310 215 -width 2 -fill brown
    ${canvas_name} create line 310 215 310 320 -width 2 -fill brown
    ${canvas_name} create line 310 320 340 320 -width 2 -fill brown
   
    # Cable - MIC attachment Audio In port to Radio Speaker output
    ${canvas_name} create line 105 230 105 200 -width 2 -fill brown
    ${canvas_name} create line 105 200 325 200 -width 2 -fill brown
    ${canvas_name} create line 325 200 325 290 -width 2 -fill brown
    ${canvas_name} create line 325 290 340 290 -width 2 -fill brown
   
    # Cable - MIC attachment Audio In port to 
    # DVM input ( through a 'T' connector )
    ${canvas_name} create line 210 160 210 200 -width 2 -fill brown
   
    # 'T' connector
    ${canvas_name} create poly 195 195      \
                               205 195      \
                               205 185      \
                               215 185      \
                               215 195      \
                               225 195      \
                               225 205      \
                               195 205      \
                               -fill white  \
                               -outline black

    return
  }



  #***************************************************************************
  #
  # Name: Format
  #
  # Parameters: args - list of values to display
  #
  # Description: Formats the results for display
  #
  #***************************************************************************
  proc Format { args } \
  {
    #puts "Format:  $args"

    variable  results

    foreach { label  value  value_format  units  pass_fail_label } $args \
    {
      set value  [format "$value_format" $value]
      
      set results  [format "%-30s %-15s %-10s %10s" "$label" "$value" "$units" "$pass_fail_label"]
    }
    
    return
  }


  #***************************************************************************
  #
  # Name: tk_adjust_dialog
  #
  # Parameters: tile_path    - invoking widget tile path
  #             title_text   - text to display as the dialog title
  #             prompt_text  - test to display as the dialog prompt
  #             adjust_label - test describing the adjustment
  #             adjust_value - the variable to monitor during the adjustment
  #             adjust_units - the units of the adjustment
  #
  # Description: Displays a toplevel used as a user input dialog box
  #
  #***************************************************************************
  proc tk_adjust_dialog { tile_path  title_text  prompt_text  \
                                  adjust_label  adjust_value  adjust_units } \
  {
    #puts "tk_adjust_dialog:  $tile_path"
    
    variable  return_value
    
    set return_value  -1
    
    set top_level  [winfo toplevel $tile_path]
    
    set tile_path  [name_child  $top_level  prompt_dialog]
    
    if { ![winfo exists $tile_path] } \
    {
      set tile_path  \
                  [::Screens::create_toplevel_tile  $tile_path  PROMPT_DIALOG]
    }
        
    wm title $tile_path  $title_text
    
    set ::Auto_Test::operator_prompt  $prompt_text
    
    set ::Auto_Test::adjust_label  $adjust_label
    
    ${tile_path}.lbl_adjust_value config -textvariable  $adjust_value
    
    set ::Auto_Test::adjust_units  $adjust_units
    
    # Bind the F8 key for a return
    bind $tile_path <KeyRelease-F8>  \
                        "::Auto_Test::prompt_dialog_input  $tile_path  CANCEL"

    after idle  "safe_focus_force  ${tile_path}.btn_ok"
    
    while { $return_value == -1 } { vwait Auto_Test::return_value }

    destroy  [winfo toplevel $tile_path]

    return  $return_value
  }


  #***************************************************************************
  #
  # Name: prompt_dialog_input
  #
  # Parameters: input - OK or CANCEL
  #
  # Description: Accepts the prompt dialog input as either OK or CANCEL
  #
  #***************************************************************************
  proc prompt_dialog_input { input } \
  {
    #puts "prompt_dialog_input:  $input"
    
    variable  return_value
    
    switch -- $input \
    {
      "OK"      { set return_value  0 }
      
      "CANCEL"  { set return_value  1 }
      
      default   { set return_value  1 }
    }
    
    return
  }


  #****************************************************************************
  #
  # Name:        getSaveState
  #
  # Parameters:  none
  #
  # Description: This function returns a list of TCL commands which can be used
  #              to setup the Auto Test screen to its current state
  #
  #****************************************************************************
  proc getSaveState { } \
  {
    #puts "::Auto_Test::getSaveState"
    
    variable  test_enable
    variable  test_radio_type_menu
    variable  test_type_menu

    set returnList \
    {
      set ::Auto_Test::test_radio_type_menu  {$test_radio_type_menu}
      set ::Auto_Test::test_type_menu        {$test_type_menu}
    }

    foreach { index } [array names ::Auto_Test::test_enable] \
    {
      append returnList  "set ::Auto_Test::test_enable($index) {$test_enable($index)}\n"
    }

    #puts "returnList:  $returnList"
    
    return [subst $returnList]
  }


  #****************************************************************************
  # Functions executed when the namespace is sourced
  #****************************************************************************
  # If the 'Auto Test' option is installed, register the screen
  if { [IFR_options  is_installed  Auto_Test] } \
  {
    #::View::RegisterScreen  Auto_Test
  }


};  # End of the Auto_Test


#****************************************************************************
# This registers the screens with the list for saving instrument states
#****************************************************************************
#namespace eval ::SavedStates \
#{
#  variable screens
#
#  lappend screens  Auto_Test
#}

