#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C) 2001      DESIGN ENGINEERING        #
# --------------------------------------------------------------------------#
# FILE        : 3003.auto_test_2.tcl                                        #
# PRODUCT     : FITS                                                        #
# AUTHOR      : Preston McCormick                                           #
# DESCRIPTION : This namespace manages the improved Auto Test               #
#############################################################################
#                                                                           #
# Required Variables:                                                       #
#    LongName           Name of screen                                      #
#    ShortName          Name of screen for menus                            #
#    RecomendedTiles    List of tiles available for this screen. It is not  #
#                       necessary to put a tile on this list that is on the #
#                       RequiredTiles list                                  #
#    NotAllowedTiles    List of tiles NEVER on this screen                  #
#    RequiredTiles      List of tiles ALWAYS on this screen                 #
#    ActiveTiles        List of tiles currently displayed on this screen    #
#    MenuFlags          0 = Active Screen                                   #
#                       1 = Currently functioning as a submenu parent       #
#                                                                           #
# Required Variables for submenu "Screens":                                 #
#    SubmenuScreens     List of all tiles (or submenus) for droplist        #
#    SubDirPath         Path to subdirectory holding the screens for the    #
#                       submenu of this "screen"                            #
#                                                                           #
#############################################################################
#                                                                           #
# Utility Functions:                                                        #
#    Setup              Called by the screen manager to create the screen.  #
#    Map                Called when screen is mapped on window              #
#    Unmap              Called when screen is removed from display          #
#    Destroy            Called when screen is destroyed                     #
#                                                                           #
#############################################################################

namespace eval Auto_Test_2 \
{
  variable  LongName    "auto_test_2"
  variable  ShortName   "Auto Test"
  variable  Size        Full
  
  variable  tile_class  "AUTO_TEST_2"
            

  variable  RequiredTiles    { }
  variable  NotAllowedTiles  { }
  variable  RecommendedTiles { }
  variable  ActiveTiles      { }
  variable  SubmenuScreens   { }

  variable  MenuFlags 0
  
  variable test_running ""

  variable  SubDirPath { }

  variable  last_focus  ""
  
  variable  config_file_label   "Test_Config"
  variable  results_file_label  "Results"

  # Test information variables
  variable  test_start_time  ""
  variable  test_stop_time   ""
  variable  test_total_time  ""
  
  variable  test_date       "Nov. 24, 2002"
  variable  test_time       "11:00:00"
  variable  test_unit_id    ""
  variable  test_tech_name  "John Q."
  variable  test_comment    ""

  variable  prev_radio_type       ""
  variable  test_radio_type       "ASTRO SABER"
  variable  new_radio_type        ""  
  variable  del_radio_type        ""  
  variable  test_radio_type_menu  [list "ASTRO SABER"  "XTS-3000"]

  variable  prev_test_type  ""
  variable  new_test_type   ""  
  variable  del_test_type   ""  
  variable  test_type       "INCOMING RADIO"
  variable  test_type_menu  [list "INCOMING RADIO"  "OUTGOING RADIO"  "RADIO STATUS CHANGE"]
  
  variable  config_description  ""

  variable  pass_fail_label  
  variable  pass_fail_max  
  variable  pass_fail_min  
  variable  pass_fail_units  
    
  variable  results         ""
  variable  results_csv
  variable  results_line
  
  variable  save_flag   0
  variable  abort_flag  0

  variable  test_namespace_list  {}

  variable  test_enable
  
  set  test_enable(Test_Analog_Rx)      1
  set  test_enable(Test_Analog_Tx)      1
  set  test_enable(Test_Digital_Rx)     0
  set  test_enable(Test_Digital_Tx)     0
  set  test_enable(Test_MDC_1200)       0
  set  test_enable(Test_Squelch)        0
  set  test_enable(Test_Siren_PA)       0


  variable  printer_dir   "printer0"

  # Test variables

  
  # Results and Configuration file variables
  variable  working_dir   "/usr/apps/auto_test/"

  # Scroll bar enabled variables
  variable  summary_scroll_on  0
  variable  results_scroll_on  0

  # Operator prompt dialog variables

  variable  return_value
  variable  power_threshold  0.05

  namespace export Append get_tile Print Format wait_dialog_remove wait_dialog Abort settle_reading adjust_dialog \
                   wait_for_power wait_for_no_power

  proc Abort { {newstate ""} } \
  {
     variable abort_flag
     if {$newstate != ""} \
     {
        set abort_flag $newstate 
     }
     return $abort_flag
  }
  
  #***************************************************************************
  #
  #        Name: Setup
  #
  #  Parameters: parent - parent of child window
  #
  # Description: Creates the main tile
  #
  #***************************************************************************
  proc Setup { parent } \
  {
    Debug::Puts AutoTest 2 "Auto_Test_2::Setup:  $parent"

    variable  LongName
    variable  tile_class
    variable  test_namespace_list

    set top_level  [winfo toplevel $parent]
   
    set tile_path  [name_child  $top_level  $LongName]

    if { ![winfo exists $tile_path] } \
    {
      set tile_path  [Screens::create_tile  $tile_path  $LongName  $tile_class]
      # build these menus on post, rather than staticly
      set menu $tile_path.frm_info.mnu_test_radio_type.menu
      $menu config -post "::Auto_Test_2::post_radio_menu $menu"
      set menu $tile_path.frm_info.mnu_test_type.menu
      $menu config -post "::Auto_Test_2::post_test_menu $menu"

    }
    
    # Sort the present test namespace list
    set test_namespace_list  [lsort -ascii $test_namespace_list]
     
    return $tile_path
  }


  #***************************************************************************
  #
  #        Name: Map
  #
  #  Parameters: 
  #
  # Description: Initializes the tile
  #
  #***************************************************************************
  proc Map { tile_path } \
  {
    Debug::Puts AutoTest 2 "Auto_Test_2::Map:  $tile_path"
    
    set tile_path  [get_tile  $tile_path]    
    
    ::View::PackOptions  $tile_path  ""

    trace variable ::Auto_Test_2::results_line  w  \
                                "Auto_Test_2::test_results_output  $tile_path"
                                   
    # Zero the Power meter
    set repeat  1
    
    while { $repeat == 1 } \
    {
      catch { Power zero } err
      
      if { $err != "" } \
      {
        # Prompt the user to verify that there is no signal at the T/R port
        set title  "Operator Instruction"
        
        set prompt_text  "Unable to zero the Power meter \n\n"

        append prompt_text  "Remove the input to the T/R port and press OK"
      
        set window_name  ${tile_path}.instruction
        
        # Dialog:  no bitmap displayed, OK button only
        tk_dialog  $window_name  $title        \
                                 $prompt_text  \
                                 {}            \
                                 0             \
                                 "OK"
      } \
      else \
      {
        # End the loop
        set repeat  0
      }
    }
        
    # Display the first screen
    test_info_display  $tile_path
    
    
    return
  }


  #***************************************************************************
  #
  #        Name: Cleanup
  #
  #  Parameters: 
  #
  # Description: 
  #
  #***************************************************************************
  proc Cleanup { tile_path } \
  {
    Debug::Puts AutoTest 2 "Auto_Test_2::Cleanup:  $tile_path"

    set tile_path  [get_tile  $tile_path]
    
    trace vdelete ::Auto_Test_2::results_line  w  \
                                "Auto_Test_2::test_results_output  $tile_path"
    if ![winfo exists $tile_path] { return }

    # Clear the softkeys
    # Softkey #2:  Start
    ::View::AssignSoftkey  $tile_path  2  ""  ""

    # Softkey #3:  Add radio type
    ::View::AssignSoftkey  $tile_path  3  ""  ""

    # Softkey #4:  Delete radio type
    ::View::AssignSoftkey  $tile_path  4  ""  ""

    # Softkey #5:  Add test type
    ::View::AssignSoftkey  $tile_path  5  ""  ""

    # Softkey #6:  Delete test type
    ::View::AssignSoftkey  $tile_path  6  ""  ""

    # Softkey #7:
    ::View::AssignSoftkey  $tile_path  7  ""  ""

    # RETURN Softkey
    ::View::AssignReturn  $tile_path  ""

    return
  }


  #***************************************************************************
  #
  #        Name: Unmap
  #
  #  Parameters: 
  #
  # Description: 
  #
  #***************************************************************************
  proc Unmap { tile_path } \
  {
    Debug::Puts AutoTest 2 "Auto_Test_2::Unmap:  $tile_path"

    set tile_path  [get_tile  $tile_path]    

    Cleanup  $tile_path
    
    return
  }


  #***************************************************************************
  #
  #        Name: Destroy
  #
  #  Parameters: 
  #
  # Description: 
  #
  #***************************************************************************
  proc Destroy { tile_path } \
  {
    Debug::Puts AutoTest 2 "Auto_Test_2::Destroy:  $tile_path"

    set tile_path  [get_tile  $tile_path]    

    Cleanup  $tile_path

    return
  }


  #***************************************************************************
  #
  #        Name: Register
  #
  #  Parameters: 
  #
  # Description: 
  #
  #***************************************************************************
  proc Register { } \
  {
    Debug::Puts AutoTest 2 "Auto_Test_2::Register:  $tile_path"

    set tile_path  [get_tile  $tile_path]    

    return
  }
  

  #***************************************************************************
  #
  #        Name: get_tile
  #
  #  Parameters: any child widget of the tile
  #
  # Description: Get the name of the tile's parent frame
  #
  #***************************************************************************
  proc get_tile { widget } \
  {
    while { 1 } \
    {
      switch -- [winfo class $widget] \
      {
        "AUTO_TEST_2"  { return $widget }
      }
        
      set widget [winfo parent $widget]
    }


    return     
  }
  

  #***************************************************************************
  #
  # Test Information procedures
  #
  #***************************************************************************
  #***************************************************************************
  #
  #        Name: test_info_display
  #
  #  Parameters: widget - invoking widget tile path
  #
  # Description: Displays the Test Information tile
  #
  #***************************************************************************
  proc test_info_display { widget } \
  {
    Debug::Puts AutoTest 2 "test_info_display:  $widget"

    set tile_path  [get_tile  $widget]
    
    # Display the tile
    grid remove ${tile_path}.frm_select
    grid remove ${tile_path}.frm_config
    grid remove ${tile_path}.frm_config_summary
    grid remove ${tile_path}.frm_results

    Screens::position_widget  ${tile_path}.frm_info
    
    # Remove the New and Delete Radio entry options from the screen
    grid remove  ${tile_path}.frm_info.lbl_new_radio_type
    grid remove  ${tile_path}.frm_info.ent_new_radio_type

    # Remove the New and Delete Test type entry options from the screen
    grid remove  ${tile_path}.frm_info.lbl_new_test_type
    grid remove  ${tile_path}.frm_info.ent_new_test_type

    # Remove the New and Delete Test type Cancel button
    grid remove  ${tile_path}.frm_info.btn_add_delete_cancel

    
    # Set the softkeys
    # Softkey #2:  Start
    ::View::AssignSoftkey  $tile_path  2  "CONTINUE"  \
                              "Auto_Test_2::test_select_display  $tile_path"

    # Softkey #3:  Add radio type
    ::View::AssignSoftkey  $tile_path  3  "ADD RADIO"  \
                           "Auto_Test_2::display_add_radio_type  $tile_path"

    # Softkey #4:  Delete radio type
    ::View::AssignSoftkey  $tile_path  4  "DELETE RADIO"  \
                        "Auto_Test_2::display_delete_radio_type  $tile_path"

    # Softkey #5:  Add test type
    ::View::AssignSoftkey  $tile_path  5  "ADD TEST TYPE"  \
                            "Auto_Test_2::display_add_test_type  $tile_path"

    # Softkey #6:  Delete test type
    ::View::AssignSoftkey  $tile_path  6  "DELETE TEST TYPE"  \
                         "Auto_Test_2::display_delete_test_type  $tile_path"

    # Softkey #7:
    ::View::AssignSoftkey  $tile_path  7  "CONNECT DRAWING"  \
                     "Auto_Test_2::test_connect_drawing_display  $tile_path"

    # RETURN Softkey
    ::View::AssignReturn  $tile_path  ""
    
    return
  }
  
  
  #***************************************************************************
  #
  #        Name: update_date_time
  #
  #  Parameters:
  #
  # Description: Updates the date and time of the the test to system time
  #              when the START softkey is pressed
  #
  #***************************************************************************
  proc update_date_time { } \
  {
    Debug::Puts AutoTest 2 "update_date_time"

    variable  test_date
    variable  test_time

    set date_time  [clock format [clock seconds]]

    # Example:  Wed Nov 13 18:36:05  2002
    
    foreach { weekday  month  month_day  time year } $date_time \
    {
      set test_date  "$month.  $month_day,  $year"
      set test_time  "$time"
    }

    return
  }


  #***************************************************************************
  #
  #        Name: display_add_radio_type
  #
  #  Parameters: widget - invoking widget tile path
  #
  # Description: Displays an entry field to enter a new radio type
  #
  #***************************************************************************
  proc display_add_radio_type { widget } \
  {
    Debug::Puts AutoTest 2 "display_add_radio_type:  $widget"

    variable  new_radio_type
    variable  test_radio_type_menu
     
    set tile_path  [get_tile  $widget]
    
    set new_radio_type  ""

    # Remove any fields that are possibly displayed in this area
    grid remove  ${tile_path}.frm_info.lbl_new_radio_type
    grid remove  ${tile_path}.frm_info.ent_new_radio_type
    grid remove  ${tile_path}.frm_info.lbl_new_test_type
    grid remove  ${tile_path}.frm_info.ent_new_test_type
    grid remove  ${tile_path}.frm_info.btn_add_delete_cancel

    # Get the name of the new radio type
    Screens::position_widget  ${tile_path}.frm_info.lbl_new_radio_type
    Screens::position_widget  ${tile_path}.frm_info.ent_new_radio_type
    Screens::position_widget  ${tile_path}.frm_info.btn_add_delete_cancel
    safe_focus_force ${tile_path}.frm_info.ent_new_radio_type
    return
  }

  #***************************************************************************
  #
  #        Name: post_radio_menu
  #
  #  Parameters: widget - invoking widget tile path
  #
  # Description: build the menu for the radio type field
  #
  #***************************************************************************
  proc post_radio_menu { widget } \
  {
     variable test_radio_type_menu
     $widget delete 0 end
     set accel 1
     set items [llength $test_radio_type_menu]
     set digits [expr int(log10($items))]
     set format %0${digits}d
     
     foreach item $test_radio_type_menu \
     {
         $widget add radiobutton                             \
                    -accel     [format $format $accel]        \
                    -label     $item                            \
                    -value     $item                            \
                    -variable  ::Auto_Test_2::test_radio_type
         incr accel
     }
  }
  #***************************************************************************
  #
  #        Name: post_test_menu
  #
  #  Parameters: widget - invoking widget tile path
  #
  # Description: build the menu for the radio type field
  #
  #***************************************************************************
  proc post_test_menu { widget } \
  {
     variable test_type_menu
     $widget delete 0 end
     set accel 1
     set items [llength $test_type_menu]
     set digits [expr int(log10($items))]
     set format %0${digits}d
     
     foreach item $test_type_menu \
     {
         $widget add radiobutton \
                    -accel     [format $format $accel] \
                    -label     $item  \
                    -value     $item  \
                    -variable  ::Auto_Test_2::test_type
         incr accel
     }
  }
  #***************************************************************************
  #
  #        Name: add_radio_type
  #
  #  Parameters: widget - invoking widget tile path
  #
  # Description: Adds a new radio type to the menu
  #
  #***************************************************************************
  proc add_radio_type { widget } \
  {
    Debug::Puts AutoTest 2 "add_radio_type:  $widget"

    variable  new_radio_type
    variable  test_radio_type_menu
    variable  test_radio_type
    set tile_path  [get_tile  $widget]

    grid remove  ${tile_path}.frm_info.lbl_new_radio_type
    grid remove  ${tile_path}.frm_info.ent_new_radio_type
    grid remove  ${tile_path}.frm_info.btn_add_delete_cancel

    if { $new_radio_type != "" } \
    {
      # Add the new radio type to the menu list variable
      if {[lsearch -exact $test_radio_type_menu $new_radio_type] == -1} \
      {
         lappend test_radio_type_menu  $new_radio_type
      }
      set test_radio_type $new_radio_type
    }

    return
  }


  #***************************************************************************
  #
  #        Name: display_delete_radio_type
  #
  #  Parameters: widget - invoking widget tile path
  #
  # Description: Displays an entry field to enter a radio type to delete
  #
  #***************************************************************************
  proc display_delete_radio_type { widget } \
  {
    Debug::Puts AutoTest 2 "display_delete_radio_type:  $widget"

    set msg [name_child $widget delete_radio]
    message_box $msg QUESTION
  }


  #***************************************************************************
  #
  #        Name: delete_radio_type
  #
  #  Parameters: 
  #
  # Description: Deletes a radio type from the menu
  #
  #***************************************************************************
  proc delete_radio_type { } \
  {
    Debug::Puts AutoTest 2 "delete_radio_type"

    variable  test_radio_type_menu
    variable  test_radio_type
    
    while {[set index  [lsearch -exact $test_radio_type_menu $test_radio_type]] != -1} \
    {
       Debug::Puts AutoTest 2 "index:  $index"
       set test_radio_type_menu  [lreplace $test_radio_type_menu $index $index]
    }
    set test_radio_type [lindex $test_radio_type_menu 0]
    return
  }


  #***************************************************************************
  #
  #        Name: display_add_test_type
  #
  #  Parameters: widget - invoking widget tile path
  #
  # Description: Displays an entry field to enter a new test type
  #
  #***************************************************************************
  proc display_add_test_type { widget } \
  {
    Debug::Puts AutoTest 2 "display_add_test_type  $widget"

    variable  new_test_type
    variable  test_type_menu
     
    set tile_path  [get_tile  $widget]
    
    set new_test_type  ""

    # Remove any fields that are possibly displayed in this area
    grid remove  ${tile_path}.frm_info.lbl_new_radio_type
    grid remove  ${tile_path}.frm_info.ent_new_radio_type
    grid remove  ${tile_path}.frm_info.lbl_new_test_type
    grid remove  ${tile_path}.frm_info.ent_new_test_type
    grid remove  ${tile_path}.frm_info.btn_add_delete_cancel

    # Get the name of the new radio type
    Screens::position_widget  ${tile_path}.frm_info.lbl_new_test_type
    Screens::position_widget  ${tile_path}.frm_info.ent_new_test_type
    Screens::position_widget  ${tile_path}.frm_info.btn_add_delete_cancel
    
    return
  }


  #***************************************************************************
  #
  #        Name: add_test_type
  #
  #  Parameters: widget - invoking widget tile path
  #
  # Description: Adds a new test type to the menu
  #
  #***************************************************************************
  proc add_test_type { widget } \
  {
    Debug::Puts AutoTest 2 "add_test_type:  $widget"

    variable  new_test_type
    variable  test_type_menu
    variable test_type
     
    set tile_path  [get_tile  $widget]

    grid remove  ${tile_path}.frm_info.lbl_new_test_type
    grid remove  ${tile_path}.frm_info.ent_new_test_type
    grid remove  ${tile_path}.frm_info.btn_add_delete_cancel

    if { $new_test_type != "" } \
    {
      if {[lsearch $test_type_menu $new_test_type] == -1} \
      {
         # Add the new radio type to the menu list variable
         lappend test_type_menu  $new_test_type
      }
      set test_type $new_test_type
    }

    return
  }


  #***************************************************************************
  #
  #        Name: display_delete_test_type
  #
  #  Parameters: widget - invoking widget tile path
  #
  # Description: Displays an entry field to enter a test type to delete
  #
  #***************************************************************************
  proc display_delete_test_type { widget } \
  {
    Debug::Puts AutoTest 2 "display_delete_test_type:  $widget"

    set msg [name_child $widget delete_test]
    message_box $msg QUESTION

  }


  #***************************************************************************
  #
  #        Name: delete_test_type
  #
  #  Parameters: 
  # Description: Deletes a test type from the menu
  #
  #***************************************************************************
  proc delete_test_type { } \
  {
    Debug::Puts AutoTest 2 "delete_test_type"

    variable  test_type_menu
    variable  test_type 
    while {[set index  [lsearch -exact $test_type_menu $test_type]] != -1} \
    {
       Debug::Puts AutoTest 2 "index:  $index"
       set test_type_menu [lreplace $test_type_menu $index $index]
    }
    set test_type [lindex $test_type_menu 0]

    return
  }



  #***************************************************************************
  #
  # Test Select procedures
  #
  #***************************************************************************
  #***************************************************************************
  #
  #        Name: test_select_display
  #
  #  Parameters: widget - invoking widget tile path
  #
  # Description: Displays the Test Select tile
  #
  #***************************************************************************
  proc test_select_display { widget } \
  {
    Debug::Puts AutoTest 2 "test_select_display:  $widget"
    
    variable  test_namespace_list

    set tile_path  [get_tile  $widget]

    # Display the tile
    grid remove ${tile_path}.frm_info
    grid remove ${tile_path}.frm_config
    grid remove ${tile_path}.frm_config_summary
    grid remove ${tile_path}.frm_results

    Screens::position_widget  ${tile_path}.frm_select
    
    # Set the softkeys
    # Softkey #2:  Continue
    ::View::AssignSoftkey  $tile_path  2  "CONTINUE"  \
                             "Auto_Test_2::test_results_display  $tile_path"

    # Softkey #3: Save Configuration
    ::View::AssignSoftkey  $tile_path  3  "SAVE CONFIG"  \
                                      "Auto_Test_2::save_config  $tile_path"

    # Softkey #4: Recall Configuration
    ::View::AssignSoftkey  $tile_path  4  "RECALL CONFIG"  \
                                    "Auto_Test_2::recall_config  $tile_path"

    # Softkey #5:
    ::View::AssignSoftkey  $tile_path  5  "CONFIG SUMMARY"  \
                                   "Auto_Test_2::config_summary  $tile_path"

    # Softkey #6:
    ::View::AssignSoftkey  $tile_path  6  "CLEAR ALL"  "Auto_Test_2::Clear_all"

    # Softkey #7:
    ::View::AssignSoftkey  $tile_path  7  ""  ""

    # Softkey #8:  RETURN
    ::View::AssignReturn  $tile_path  \
                                "Auto_Test_2::test_info_display  $tile_path"

    # Remove all chackbuttons and CONFIG buttons
    set frame_name  ${tile_path}.frm_select
    
    foreach { chk_btn_index } { 1  2  3  4  5  6  7  8  9  10 } \
    {
      grid remove  ${frame_name}.chk_test${chk_btn_index}
      grid remove  ${frame_name}.btn_test${chk_btn_index}_config
    }
    
    # Display the present test selections
    set chk_btn_index  1

    foreach { test_namespace } $test_namespace_list \
    {
      # Display and configure the checkbutton
      Screens::position_widget  ${frame_name}.chk_test${chk_btn_index}
      
      ${frame_name}.chk_test${chk_btn_index} config  \
                         -textvariable ::${test_namespace}::long_name  \
                         -variable ::Auto_Test_2::test_enable($test_namespace)

      # Display and configure the CONFIG button
      Screens::position_widget  ${frame_name}.btn_test${chk_btn_index}_config

      ${frame_name}.btn_test${chk_btn_index}_config config  -command  \
             "Auto_Test_2::test_config_display  $frame_name  $test_namespace"
              
      incr chk_btn_index  1
    }

    return
  }
  
  proc Clear_all { } \
  {
     variable  test_namespace_list
     foreach { test_namespace } $test_namespace_list \
     {
        upvar #0 ${test_namespace}::test_data test_data
        foreach chan test_data(chan_list) \
        {
           foreach item [array names test_data "$chan,*"] \
           {
              unset test_data($item)
           }
        }
        set test_data(chan_list) ""
        set test_data(channel) 1
     }
  }


  #***************************************************************************
  #
  #        Name: save_config
  #
  #  Parameters: tile_path - tile path of the invoking widget
  #
  # Description: Saves the configuration data
  #
  #***************************************************************************
  proc save_config { tile_path } \
  {
    Debug::Puts AutoTest 2 "save_config:  $tile_path"

    variable  config_file_label
    variable  save_flag
    variable  test_enable
    variable  test_namespace_list
    variable  working_dir
    
    set tile_path  [get_tile  $tile_path]

    set data_buffer  ""

    # Get the test selection data
    foreach { index } [array names ::Auto_Test_2::test_enable] \
    {
      append data_buffer  "set ::Auto_Test_2::test_enable($index) {$test_enable($index)}\n"
    }

    # Add some space
    append data_buffer  "\n\n"

    # Get the test data from each namespace
    foreach { test_namespace } $test_namespace_list \
    {
      append data_buffer  [${test_namespace}::get_config_data]
    }
    
    Debug::Puts AutoTest 2 "data_buffer:  $data_buffer"
    
    # Save the data to a file
    set save_flag  [File_SaveUI::display_file_save  $tile_path            \
                                                    $working_dir          \
                                                    $data_buffer          \
                                                    "TEST_CONFIG"         \
                                                    "$config_file_label"  \
                                                    ".TSTCFG"             \
                                                    NO_ENCRYPT            \
                                                    NO_CSV                \
                                                    NO_MODES              \
                                                    NO_OPT_EXT]
    
    # 1.7.0 File Save
    #set save_flag  [File_SaveUI::display_file_save  $tile_path     \
    #                                                $working_dir   \
    #                                                $data_buffer   \
    #                                                "TEST_CONFIG"  \
    #                                                ".TSTCFG"      \
    #                                                NO_ENCRYPT     \
    #                                                NO_CSV         \
    #                                                NO_MODES       \
    #                                                NO_OPT_EXT]
    
    # Get the file label after the file has been saved
    set config_file_label  $::File_SaveUI::file_label

    return
  }
  
  
  #***************************************************************************
  #
  #        Name: recall_config
  #
  #  Parameters: tile_path - tile path of the invoking widget
  #
  # Description: Recalls previously saved configuration data
  #
  #***************************************************************************
  proc recall_config { tile_path } \
  {
    Debug::Puts AutoTest 2 "recall_config:  $tile_path"

    variable  working_dir

    set tile_path  [get_tile  $tile_path]
    
    set data_buffer  [File_RecallUI::display_file_recall  $tile_path     \
                                                          $working_dir   \
                                                          "TEST_CONFIG"  \
                                                          ".TSTCFG"      \
                                                          NO_ENCRYPT]
                                                               
    Debug::Puts AutoTest 2 "data_buffer:  $data_buffer"
    
    # Source the test data to set the variables
    eval "$data_buffer"

    # Handle variable initialization needed by current code which is not set
    # up with old recall files
    Test_Analog_Rx::default_all_channel_data
    Test_Analog_Tx::default_all_channel_data


    return
  }


  #***************************************************************************
  #
  #        Name: config_summary
  #
  #  Parameters: tile_path - tile path of the invoking widget
  #
  # Description: Displays a summary of the present configuration
  #
  #***************************************************************************
  proc config_summary { tile_path {show_list ""} { return_to ""} } \
  {
    Debug::Puts AutoTest 2 "config_summary:  $tile_path"

    variable  test_namespace_list
    
    set tile_path  [get_tile  $tile_path]
    
    if {$show_list == ""} { set show_list $test_namespace_list }
    if {$return_to == ""} { set return_to "Auto_Test_2::test_select_display $tile_path" }
        
    # Display the tile
    grid remove ${tile_path}.frm_info
    grid remove ${tile_path}.frm_config
    grid remove ${tile_path}.frm_config_summary
    grid remove ${tile_path}.frm_results


    Screens::position_widget  ${tile_path}.frm_config_summary
    
    ${tile_path}.frm_config_summary.scrl_summary config  \
                 -command "${tile_path}.frm_config_summary.txt_summary yview"
    
    ${tile_path}.frm_config_summary.txt_summary config  \
           -yscrollcommand "${tile_path}.frm_config_summary.scrl_summary set"

    # Set the softkeys
    # Softkey #2:
    ::View::AssignSoftkey  $tile_path  2  ""  ""

    # Softkey #3:
    ::View::AssignSoftkey  $tile_path  3  ""  ""

    # Softkey #4:
    ::View::AssignSoftkey  $tile_path  4  ""  ""

    # Softkey #5:
    ::View::AssignSoftkey  $tile_path  5  ""  ""

    # Softkey #6:
    ::View::AssignSoftkey  $tile_path  6  ""  ""

    # Softkey #7:
    ::View::AssignSoftkey  $tile_path  7  ""  ""

    # Softkey #8:  RETURN to the Test Select screen
    ::View::AssignReturn  $tile_path  $return_to

    # Enable the configuration summary textbox
    ${tile_path}.frm_config_summary.txt_summary  config  -state normal
    
    # Clear the configuration summary textbox
    ${tile_path}.frm_config_summary.txt_summary  delete  1.0  end
    
    # Display the present configuration in the textbox
    foreach { test_namespace } $show_list \
    {
      set summary  [${test_namespace}::get_config_summary]
      
      ${tile_path}.frm_config_summary.txt_summary  insert  end  $summary
    }
    
    # Disable the configuration summary textbox
    ${tile_path}.frm_config_summary.txt_summary  config  -state normal

    return
  }


  #***************************************************************************
  #
  # Test Configuration procedures
  #
  #***************************************************************************
  #***************************************************************************
  #
  #        Name: test_config_display
  #
  #  Parameters: widget - invoking widget tile path
  #              test   - description of the test
  #
  # Description: Displays the Test Configuration tile
  #
  #***************************************************************************
  proc test_config_display { widget  test } \
  {
    Debug::Puts AutoTest 2 "test_config_display:  $widget,  $test"

    set tile_path  [get_tile  $widget]

    # Display the tile
    grid remove ${tile_path}.frm_info
    grid remove ${tile_path}.frm_select
    grid remove ${tile_path}.frm_config_summary
    grid remove ${tile_path}.frm_results

    Screens::position_widget  ${tile_path}.frm_config

    # Remove the present config tiles from the display
    if { [winfo children ${tile_path}.frm_config] != "" } \
    {
      foreach { child_tile } [winfo children ${tile_path}.frm_config] \
      {
        grid remove  $child_tile
      }
    }
    
    # Display the correct configuration sub tile
    set config_tile [show_test_config_tile $tile_path $test]
    # Set the softkeys
    # Softkey #2
    View::AssignSoftkey  $tile_path  2  "Delete Channel"  "Auto_Test_2::Delete_channel $tile_path $test $config_tile"

    # Softkey #3
    View::AssignSoftkey  $tile_path  3  "Show Config"  "Auto_Test_2::Show_test_config $tile_path $test"

    # Softkey #4:
    View::AssignSoftkey  $tile_path  4  ""  ""

    # Softkey #5:
    View::AssignSoftkey  $tile_path  5  ""  ""

    # Softkey #6:
    View::AssignSoftkey  $tile_path  6  ""  ""

    # Softkey #7:
    View::AssignSoftkey  $tile_path  7  ""  ""

    # Softkey #8:  RETURN
    View::AssignReturn  $tile_path  \
                        "Auto_Test_2::test_config_display_return  $tile_path"


    return
  }
  proc show_test_config_tile { tile_path  test } \
  {
    set config_tile  [${test}::Create  ${tile_path}.frm_config]
    grid $config_tile  -row 5 -column 5 -rowspan 425 -columnspan 530
    return $config_tile
  }
  
  #***************************************************************************
  #
  #        Name: Delete_channel
  #
  #  Parameters: the invoking widget path, the test namespace, config tile
  #
  # Description: Delete the current channel from the specified test
  #
  #***************************************************************************
  proc Delete_channel { widget  test config_tile} \
  {
     #puts "Delete channel $widget $test"
     # remove the tile, so we can destroy variables
     destroy $config_tile
     update idle
     
     upvar #0 ${test}::test_data test_data
     set chan  $test_data(channel)
     # remove channel # from channel list
     set index [lsearch $test_data(chan_list) $chan]
     if {$index == -1} { return }
     # remove this channel # from the list
     set test_data(chan_list) [lreplace $test_data(chan_list) $index $index]

     # find all this guys's data
     set vlist [array names test_data "$chan,*"]
     #puts "found $vlist"
     foreach item $vlist \
     {
        #puts "unset test_data($item)"
        unset test_data($item)
     }
     #puts "now find [array names test_data "$chan,*"]"

     # OK, go to different channel.
     incr index -1
     #puts "new index $index"
     if {$index < 0} \
     {
        set index 0
     }
     set chan [lindex $test_data(chan_list) $index]
     if {$chan != ""} \
     {
        set test_data(channel) $chan
     }
     show_test_config_tile $widget $test
  }
  
  #***************************************************************************
  #
  #        Name: Show_test_config
  #
  #  Parameters: the invoking widget path, the test namespace
  #
  # Description: Show the configuration for just this test
  #
  #***************************************************************************
  proc Show_test_config { widget  test } \
  {
     #puts "Show config $widget $test"
     config_summary $widget $test "Auto_Test_2::test_config_display $widget $test"
  }
  
  #***************************************************************************
  #
  #        Name: test_config_display_return
  #
  #  Parameters: tile_path - the invoking tile path
  #
  # Description: Performs procedure calls after the RETURN button is pressed
  #
  #***************************************************************************
  proc test_config_display_return { tile_path } \
  {
    Debug::Puts AutoTest 2 "test_config_display_return:  $tile_path"
  
    # Display the Select screen
    Auto_Test_2::test_select_display  $tile_path
      
    return
  }
  

  #***************************************************************************
  #
  # Test Results procedures
  #
  #***************************************************************************
  #***************************************************************************
  #
  #        Name: test_results_display
  #
  #  Parameters: widget - invoking widget tile path
  #
  # Description: Displays the Test Results tile
  #
  #***************************************************************************
  proc test_results_display { widget } \
  {
    Debug::Puts AutoTest 2 "test_results_display:  $widget"

    set tile_path  [get_tile  $widget]

    # Display the tile
    grid remove ${tile_path}.frm_info
    grid remove ${tile_path}.frm_select
    grid remove ${tile_path}.frm_config
    grid remove ${tile_path}.frm_config_summary

    Screens::position_widget  ${tile_path}.frm_results
    

    ${tile_path}.frm_results.scrl_results config  \
                        -command "${tile_path}.frm_results.txt_results yview"
    
    ${tile_path}.frm_results.txt_results config  \
                  -yscrollcommand "${tile_path}.frm_results.scrl_results set"

    
    # Set the softkeys
    # Softkey #2:  Start
    ::View::AssignSoftkey  $tile_path  2  "START"  \
                                        "after idle \"Auto_Test_2::test_start  $tile_path\""

    # Softkey #3:  Abort
    ::View::AssignSoftkey  $tile_path  3  "ABORT"  \
                                        "Auto_Test_2::Abort 1"

    # Softkey #4:  Save
    ::View::AssignSoftkey  $tile_path  4  "SAVE"  \
                                 "Auto_Test_2::test_results_save  $tile_path"

    # Softkey #5:  Recall
    ::View::AssignSoftkey  $tile_path  5  "RECALL"  \
                               "Auto_Test_2::test_results_recall  $tile_path"

    # Softkey #6:  Clear
    ::View::AssignSoftkey  $tile_path  6  "CLEAR"  \
                                "Auto_Test_2::test_results_clear  $tile_path"

    # Softkey #7:  Print
    ::View::AssignSoftkey  $tile_path  7  "PRINT"  \
                                "Auto_Test_2::test_results_print  $tile_path"

    # Softkey #8:  RETURN
    ::View::AssignReturn  $tile_path \
                               "Auto_Test_2::test_select_display  $tile_path"

    return
  }


  #***************************************************************************
  #
  #        Name: test_start
  #
  #  Parameters: widget - invoking widget tile path
  #
  # Description: Starts the tests
  #
  #***************************************************************************
  proc test_start { widget } \
  {
    variable  test_running
    
    set tile_path  [get_tile  $widget]
    
    if {$test_running != ""} \
    {
       # is this me? if so, skip it.
       if {$test_running != "${tile_path}.frm_results.txt_results"} \
       {
	       # a test is already running - silly human.
	       if [winfo exists $test_running] \
	       {
		       set text_box  ${tile_path}.frm_results.txt_results
		       if { [winfo exists $text_box] } \
		       {
		         $text_box config -state normal
		         $text_box delete 1.0 end

		         $text_box insert end [$test_running get 1.0 end]
		         $text_box insert end "<Test already running>"

		         $text_box see end

		         $text_box config -state disabled
		      
		         return
		       }
		   }
	   }
    }
    set test_running  ${tile_path}.frm_results.txt_results
    
    Debug::Puts AutoTest 2 "test_start:  $widget"

    variable  results

    variable  results_csv

    variable  abort_flag

    variable  test_date
    variable  test_namespace_list
    variable  test_time
    variable  test_unit_id
    variable  test_tech_name
    variable  test_radio_type
    variable  test_type
    variable  test_comment
    
    variable  test_enable

    variable  test_start_time
    variable  test_stop_time
    variable  test_total_time

    set abort_flag  0
    
    # Update the DATE and TIME
    update_date_time

    #puts "About to simulate a factory default"
    catch { source /system/default_autotest.tcl }

    set results ""
    set results_csv ""
    
    Append  Date $test_date $test_time
    Append "Tech Name"  $test_tech_name
    Append "Radio Type" $test_radio_type
    Append "Test Type"  $test_type
    
    # Prompt the enter the serial number
    set test_unit_id  ""

    # Loop until a valid serial number is entered
    while { $test_unit_id == "" } \
    {
      set title  "Operator Instruction"

      set prompt_text  "Enter a Serial Number"
    
      set input_value  "::Auto_Test_2::test_unit_id"
    
      set prompt_value  [input_dialog  $tile_path $title  $prompt_text  $input_value]

      switch -- $prompt_value \
      {
        0 { }
    
        1 { Abort 1;  break }
      }
    }
    
    Append "Serial No." $test_unit_id
    
    # Display the test comment in the results box
    Append "Comment" $test_comment
    
    # Get the reference for the time of the test
    set test_start_time  [expr int([clock seconds])]
    
    # Loop for each test listed
    foreach { test_namespace } $test_namespace_list \
    {
      Debug::Puts AutoTest 2 "test_namespace:  $test_namespace"
    
      if { $abort_flag == 1 } \
      {
        break
      }
      
      set test_name  "::${test_namespace}::test"

      Debug::Puts AutoTest 2 "test_name:  $test_name"
      
      if { $test_enable($test_namespace) == 1 } \
      {
        if [catch "$test_name  $tile_path" err] \
        {
           Append "Error" "$err\n"
#           Append "Error" "$err\n$errorInfo"
           Abort 1
        }
      }
    }


    if { $abort_flag == 0 } \
    {
      Append Status COMPLETED
    } \
    else \
    {
      Append Status ABORTED
    }

    # Determine the test time
    set test_stop_time  [expr int([clock seconds])]

    set test_total_time  [expr $test_stop_time - $test_start_time]

    Append "Test Time" $test_total_time
    return
  }


  #***************************************************************************
  #
  #        Name: test_abort
  #
  #  Parameters: widget - invoking widget tile path
  #
  # Description: Aborts the tests
  #
  #***************************************************************************
  proc test_abort { widget } \
  {
    Abort 1
    
    return
  }


  #***************************************************************************
  #
  #        Name: test_results_save
  #
  #  Parameters: widget - invoking widget tile path
  #
  # Description: Saves the results
  #
  #***************************************************************************
  proc test_results_save { widget } \
  {
    Debug::Puts AutoTest 2 "test_results_save:  $widget"

    variable  results_file_label
    variable  save_flag
    variable  working_dir
    variable  results_csv
    variable  results
    variable  test_date
    variable  test_time
    
    if ![info exists results_csv] \
    {
      update_date_time
      Append  Date $test_date $test_time
    }

    set tile_path  [get_tile  $widget]
    
    # If no data exists, return
    if { $results_csv == "" } \
    {
      return
    }

    File_SaveUI::display_file_save  $tile_path             \
                                                    $working_dir           \
                                                    $results               \
                                                    "RESULTS"              \
                                                    "$results_file_label"  \
                                                    ".TXT"                 \
                                                    NO_ENCRYPT             \
                                                    $results_csv           \
                                                    ""               \
                                                    CSV
                                                                   
    # Get the file label after the file has been saved
    Print "Results saved to $::File_SaveUI::file_name"
    set save_flag 1
  }


  #***************************************************************************
  #
  #        Name: test_results_recall
  #
  #  Parameters: widget - invoking widget tile path
  #
  # Description: Recalls results saved to a file
  #
  #***************************************************************************
  proc test_results_recall { widget } \
  {
    Debug::Puts AutoTest 2 "test_results_recall:  $widget"
    
    variable  results
    variable  results_csv
    variable  working_dir
    variable  save_flag
    variable  results_line
    
    set tile_path  [get_tile  $widget]
    
    set file_buffer  [::File_RecallUI::display_file_recall  $tile_path    \
                                                            $working_dir  \
                                                            "RESULTS"     \
                                                            ".TXT" ]
                                                               
    # If the recalled results are valid, display them
    if { $file_buffer != "" } \
    {
       set tile_path  [get_tile  $widget]
       set text_box  ${tile_path}.frm_results.txt_results
       if { [winfo exists $text_box] } \
       {
             $text_box config -state normal
             $text_box delete 1.0 end
             $text_box see end
             $text_box config -state disabled
       }
       set results_line $file_buffer
    }
    set save_flag 1
  }


  #***************************************************************************
  #
  #        Name: test_results_clear
  #
  #  Parameters: widget - invoking widget tile path
  #
  # Description: Clears the results textbox
  #
  #***************************************************************************
  proc test_results_clear { widget  args } \
  {
    Debug::Puts AutoTest 2 "test_results_clear:  $widget,  $args"
    
    variable  save_flag

    set tile_path  [get_tile  $widget]

    # Prompt the operator if the present data is not saved
    if { $save_flag == 0 } \
    {
      set title  "Warning"
      set prompt  "The results data has not been saved!\n\nSave it now?"
      
      set window_name  ${tile_path}.warning
      
      set result  [tk_dialog  $window_name  $title  $prompt  info  0  "YES"  "NO"]
    
      switch -- $result \
      {
        0 { set save_flag  -1;  test_results_save  $tile_path }
      
        1 { set save_flag  1 }
      }
    }    

    # Wait for the 'save_flag' to become valid
    while { $save_flag == -1 } \
    {
       after 100
    }
    
    # Retest the 'save_flag' to verify
    if { $save_flag == 1 } \
    {
      set text_box  ${tile_path}.frm_results.txt_results
    
      if { [winfo exists $text_box] } \
      {
        $text_box config -state normal

        $text_box delete  1.0  end

        $text_box config -state disabled
      }
    }
    
    return
  }


  #***************************************************************************
  #
  #        Name: test_results_print
  #
  #  Parameters: widget - invoking widget tile path
  #
  # Description: Prints the results of the test
  #
  #***************************************************************************
  proc test_results_print { widget } \
  {
    Debug::Puts AutoTest 2 "test_results_print:  $widget"

    variable  results
    
    variable  printer_dir
    
    set tile_path  [get_tile  $widget]
    
    set printer_path  [file join  /  $printer_dir]
    
    set text_box  ${tile_path}.frm_results.txt_results
    
    $text_box config -state normal
    
    set last_index  [$text_box  index  end]
    
    set last_index  [expr int($last_index)]
    
    if { [winfo exists $text_box] } \
    {
      # Open the printer
      set error  [catch { set output_id  [open  $printer_path  w] } result]
      
      Debug::Puts AutoTest 2 "error:  $error"
      
      if { $error != 0 } \
      {
        # Error opening the printer
        return
      }
      
      # Set the buffering to 'none'
      fconfigure  $output_id  -buffering  none -eofchar { "" "\f"}
      
      for { set index 1 } { $index <= $last_index } { incr index 1 } \
      {
        set print_buffer  [$text_box  get  ${index}.0  ${index}.end]
        
        Debug::Puts AutoTest 2 "${index}: $print_buffer"

        # Write the data to the file (printer)
        catch { puts  -nonewline  $output_id  "$print_buffer \r" } result
      }

      # Close the printer
      catch { close  $output_id } result

      $text_box config -state disabled
    }

    return
  }


  #***************************************************************************
  #
  #        Name: test_results_output
  #
  #  Parameters: widget - invoking widget tile path
  #
  # Description: Prints the results to the RESULTS text box
  #
  #***************************************************************************
  proc test_results_output { widget  args } \
  {
    Debug::Puts AutoTest 2 "test_results_output:  $widget,  $args"
    
    variable  results_line
    variable  save_flag
    
    set tile_path  [get_tile  $widget]
    
    set text_box  ${tile_path}.frm_results.txt_results
    
    if { [winfo exists $text_box] } \
    {
      $text_box config -state normal

      $text_box insert end "$results_line\n"

      $text_box see end

      $text_box config -state disabled
      
      set save_flag  0
    }
    
    return
  }


  #***************************************************************************
  #
  # Test Connect Drawing procedures
  #
  #***************************************************************************
  #***************************************************************************
  #
  #        Name: test_connect_drawing_display
  #
  #  Parameters: widget - invoking widget tile path
  #
  # Description: Displays the Test Connect Drawing
  #
  #***************************************************************************
  proc test_connect_drawing_display { widget } \
  {
    ::help::show $widget connect.shtml "Connection diagrams"
    return
  }

  #***************************************************************************
  #
  #        Name: Append
  #
  #  Parameters: args - list of values to display
  #
  # Description: Append values onto the results object
  #
  #***************************************************************************
  proc Append { args } \
  {
     variable results_csv
     set rv ""
     foreach arg $args \
     {
        if {$rv != ""} { append rv ,}
        # escape any nasty special chars
        set arg [list $arg]
        append rv $arg
     }
     append results_csv "$rv\n"
     Display $rv
  }
  
  #***************************************************************************
  #
  #        Name: Print
  #
  #  Parameters: line to display
  #
  # Description: Print a line of text to the window
  #
  #***************************************************************************
  proc Print { line args } \
  {
     variable results
     variable results_line
     set x [eval format {$line} $args  ]
     append results "$x\n"
     set results_line $x
  }
  
  #***************************************************************************
  #
  #        Name: Display
  #
  #  Parameters: line of CSV to display
  #
  # Description: Convert CSV into display format
  #
  #***************************************************************************
  proc Display { line } \
  {
     # first, bust into a list
     set v [CSVsplit $line]
     
     # parse the first element, and format the line
     switch [lindex $v 0] \
     {
       "Date" \
       {
          Print "Date:        %s, %s" [lindex $v 1] [lindex $v 2]
       }
       "Tech Name" \
       {
          Print  "Tech Name:   %s" [lindex $v 1]
       }
       "Radio Type" \
       {
           Print  "Radio Type:  %s" [lindex $v 1]
       }
       "Test Type" \
       {
          Print  "Test Type:   %s" [lindex $v 1]
       }
       "Serial No." \
       {
          Print  "Serial No.:  %s" [lindex $v 1]
       }
       "Comment" \
       {
          Print  "%s" [lindex $v 1]
          Print  ""
       }
       "Status" \
       {
          if {[lindex $v 1] == "COMPLETED"} \
          {
             Print  ""
             Print  "-------> TEST COMPLETED <-------"
             Print  ""
          } \
          else \
          {
             Print  ""
             Print  "******** TEST ABORTED ********"
             Print  ""
          }
       }
       "Test Time" \
       {
          set time [lindex $v 1]
          Print  "Test Time:  %02d:%02d:%02d" [expr int($time/3600)] [expr int($time/60)%60] [expr int($time)%60]
       }
       "Result" \
       {
          Print  "%-30s: %-15s %-10s %10s" [lindex $v 1] [lindex $v 2] [lindex $v 3] [lindex $v 4]
       }
       "Test" \
       {
          Print ""
          Print "Test #%s: %s" [lindex $v 1] [lindex $v 2]
          Print ""
       }
       "Error" \
       {
          Print "(An internal software error has occurred)"
          Print $line
       }
       default \
       {
          return ""
       }
     }
     return $line
  }
  
  #***************************************************************************
  #
  #        Name: Format
  #
  #  Parameters: args - list of values to display
  #
  # Description: Formats the results for display
  #
  #***************************************************************************
  proc Format { args } \
  {
    Debug::Puts AutoTest 2 "Format:  $args"

    variable  results
    variable  results_csv

    foreach { label  value  value_format  units  pass_fail_label } $args \
    {
      set value  [format "$value_format" $value]
            
      Append Result $label $value $units $pass_fail_label
    }
    
    return
  }


  #***************************************************************************
  #
  #        Name: settle_reading
  #
  #  Parameters: system     - measurement system
  #              subsystem  - measurement sub-system
  #              init_delay - initial delay before taking readings
  #              delay      - time delay between measurements
  #
  # Description: Takes readings until the readings settle to with 5% of
  #              each other, or a 3 second timeout expires.
  #
  #***************************************************************************
  proc settle_reading { system  subsystem  { init_delay  0 } { delay  500 } } \
  {
    Debug::Puts AutoTest 2 "settle_reading:  $system,  $subsystem"

    set reading_diff  100
    set prev_reading  0
    set timeout       0
    
    after  $init_delay
    
    set initial_time  [clock seconds]
    
    # Five percent tolerance
    # Three second timeout
    while { $reading_diff > 0.05 && $timeout < 3 } \
    {
      set reading  ["$system"  "$subsystem"]
      
      if { $reading == 0.0 } \
      {
        set reading  0.001
      }
      
      Debug::Puts AutoTest 2 "  reading:  $reading"
      
      set reading_diff  [expr  ($reading - $prev_reading) / $reading]

      set reading_diff  [expr  abs($reading_diff)]

      set prev_reading  $reading
      
      set timeout  [expr  [clock seconds] - $initial_time]
      
      after $delay
    }
    
    return  $reading
  }
  
  
  #***************************************************************************
  #
  #        Name: adjust_dialog
  #
  #  Parameters: tile_path        - invoking widget tile path
  #              title_text       - text to display as the dialog title
  #              prompt_text      - text to display as the dialog prompt
  #              adjust_label     - text describing the adjustment
  #              adjust_value     - the variable to monitor during the 
  #                                 adjustment
  #              adjust_units     - the units of the adjustment
  #              adjust_target    - adjustment target of the value
  #              adjust_tolerance - adjustment tolerance
  #
  # Description: Displays a toplevel used as a user adjustment dialog box.
  #              The dialog will be removed from the display when the 
  #              adjustment reaches the desired level.
  #
  #***************************************************************************
  proc adjust_dialog { tile_path  title_text  prompt_text  adjust_label  \
      adjust_value  adjust_units  adjust_target  { adjust_tolerance  0.1 } } \
  {
    Debug::Puts AutoTest 2 "adjust_dialog:  $tile_path"
    
    variable  return_value
    
    set return_value  -1
    
    set top_level  [winfo toplevel $tile_path]
    
    set tile_path  [name_child  $top_level  prompt_dialog]
    
    if { ![winfo exists $tile_path] } \
    {
      set tile_path  [Screens::create_toplevel_tile  $tile_path  ADJUST_DIALOG]
    }
        
    wm  title  $tile_path  $title_text
    
    set ::Auto_Test_2::operator_prompt  $prompt_text
    
    set ::Auto_Test_2::adjust_label  $adjust_label
    
    ${tile_path}.lbl_adjust_value config -textvariable  $adjust_value
    
    set ::Auto_Test_2::adjust_units  $adjust_units
    
    # Bind the F8 key for a return
    bind $tile_path <KeyRelease-F8>  \
                       "Auto_Test_2::adjust_dialog_input  $tile_path  CANCEL"

    safe_focus_force  ${tile_path}.btn_abort
    
    # Set the max and min tolerances for the measurement to +/- 10%
    set adjust_target_max  [expr $adjust_target + $adjust_tolerance]
    set adjust_target_min  [expr $adjust_target - $adjust_tolerance]

    while { $return_value == -1 } \
    { 
      # Read the value of the variable
      set value  [subst $$adjust_value]
      
      if { $value < $adjust_target_min } \
      {
        ${tile_path}.lbl_adjust_value config -foreground blue
      } \
      elseif { $value > $adjust_target_max } \
      {
        ${tile_path}.lbl_adjust_value config -foreground red
      } \
      else \
      {
        ${tile_path}.lbl_adjust_value config -foreground black
      }
    
      # If the value is within the tolerances, set the 'return_value'
      # to '0' (OK) to remove the dialog and continue
      if { $value >= $adjust_target_min && $value <= $adjust_target_max } \
      {
        after 1000
        
        # Read the value of the variable a second time
        set value  [subst $$adjust_value]

        if { $value > $adjust_target_min && $value < $adjust_target_max } \
        {
          set return_value  0
        }
      }
      
      after 100
    }

    destroy  [winfo toplevel $tile_path]

    return  $return_value
  }


  #***************************************************************************
  #
  #        Name: adjust_dialog_input
  #
  #  Parameters: input - OK or CANCEL
  #
  # Description: Accepts the prompt dialog input as either OK or CANCEL
  #
  #***************************************************************************
  proc adjust_dialog_input { input } \
  {
    Debug::Puts AutoTest 2 "adjust_dialog_input:  $input"
    
    variable  return_value
    
    switch -- $input \
    {
      "OK"      { set return_value  0 }
      
      "ABORT"   -
      "CANCEL"  { set return_value  1 }
      
      default   { set return_value  1 }
    }
    
    return
  }


  #***************************************************************************
  #
  #        Name: input_dialog
  #
  #  Parameters: tile_path    - invoking widget tile path
  #              title_text   - text to display as the dialog title
  #              prompt_text  - text to display as the dialog prompt
  #              input_value  - the variable of the input
  #
  # Description: Displays a toplevel used as a user input dialog box
  #
  #***************************************************************************
  proc input_dialog { tile_path  title_text  prompt_text  input_value } \
  {
    Debug::Puts AutoTest 2 "input_dialog:  $tile_path"
    
    variable  return_value
    
    set return_value  -1
    
    set top_level  [winfo toplevel $tile_path]
    
    set tile_path  [name_child  $top_level  input_dialog]
    
    if { ![winfo exists $tile_path] } \
    {
      set tile_path  [Screens::create_toplevel_tile  $tile_path  INPUT_DIALOG]
    }
        
    wm title $tile_path  $title_text
    
    set ::Auto_Test_2::operator_prompt  $prompt_text
    
    ${tile_path}.ent_input_value config -textvariable  $input_value
    
    # Bind the Return key for a return
    bind $tile_path <<EditClose>>  \
                                "Auto_Test_2::adjust_dialog_input  OK; break"

    after idle  "safe_focus_force  ${tile_path}.ent_input_value"
    
    while { $return_value == -1 } { after 100 }

    # Delay before destroying the toplevel so that the key presses and 
    # key releases propagate correctly.
    after 500
    
    destroy  $tile_path
    
    # Allow the display to update to finish removing the dialog box
    after 500
    
    return  $return_value
  }


  #***************************************************************************
  #
  #        Name: wait_dialog
  #
  #  Parameters: tile_path   - invoking widget tile path
  #              title_text  - text to display as the dialog title
  #              prompt_text - text to display as the dialog prompt
  #              abort_text  - text to display on the ABORT button
  #
  # Description: Displays a toplevel used to prompt the user then waits 
  #              to be removed from the display.
  #              Provides an ABORT button.
  #
  #***************************************************************************
  proc wait_dialog { tile_path  title_text  prompt_text  { abort_text  "ABORT" } { continue_text ""} } \
  {
    Debug::Puts AutoTest 2 "wait_dialog:  $tile_path"

    set top_level  [winfo toplevel $tile_path]
    
    set tile_path  [name_child  $top_level  wait_dialog]
    
    if { ![winfo exists $tile_path] } \
    {
      set tile_path  [Screens::create_toplevel_tile  $tile_path  WAIT_DIALOG]
    }
        
    wm title  $tile_path  $title_text
    grid rowconfig $tile_path 1 -weight 0
    
    set ::Auto_Test_2::operator_prompt  $prompt_text
    
    ${tile_path}.btn_abort config -text  $abort_text
    
    if {$continue_text != ""} \
    {
       Screens::position_widget ${tile_path}.btn_continue
       $tile_path.btn_continue configure -text $continue_text
       after 100
       after idle "safe_focus_force  ${tile_path}.btn_continue"
    } else \
    {
       after idle "safe_focus_force  ${tile_path}.btn_abort"
    }
    
    return  $tile_path
  }

  #***************************************************************************
  #
  #        Name: wait_continue
  #
  #  Parameters: tile_path - tile path of the invoking widget
  #
  # Description: Called when the CONTINUE button on the WAIT dialog is pressed
  #
  #***************************************************************************
  proc wait_continue { tile_path } \
  {
    Debug::Puts AutoTest 2 "wait_continue:  $tile_path"
    
    variable  abort_flag

    set abort_flag  2
    
    wait_dialog_remove  $tile_path

  }
  
  #***************************************************************************
  #
  #        Name: wait_abort
  #
  #  Parameters: tile_path - tile path of the invoking widget
  #
  # Description: Called when the ABORT button on the WAIT dialog is pressed
  #
  #***************************************************************************
  proc wait_abort { tile_path } \
  {
    Debug::Puts AutoTest 2 "wait_abort:  $tile_path"
    
    variable  abort_flag

    set abort_flag  1
    
    # Remove the dialog from the display
    wait_dialog_remove  $tile_path
    
    return
  }

  #***************************************************************************
  #
  #        Name: wait_for_power
  #
  #  Parameters: tile_path - tile path of the invoking widget
  #
  # Description: Wait until power is detected or CONTINUE is pressed
  #
  #***************************************************************************
  proc wait_for_power { tile_path {prompt ""}} \
  {
      variable  power_threshold
      set title  "Operator Instruction"
      
      if {$prompt == ""} {set prompt  "Key the radio"}

      # Display a NO WAIT dialog then wait for the Power level to exceed 0.1 W
      set wait_tile_path  [wait_dialog  $tile_path  $title  $prompt  "ABORT" "CONTINUE"]

      # Measure the Power level
   
      # Loop until the Power level exceeds the threshold value,
      # or the user presses the ABORT button
      while { ($::Power::State(reading)  < $power_threshold) && ![Abort]} \
      {
         after 100
      }
      
      if {[Abort] == "1"} {return -code break}
      
      Abort 0
      
      # Remove the NO WAIT dialog from the display
      # if the Power level exceeded the threshold
      wait_dialog_remove  $wait_tile_path
  }
  #***************************************************************************
  #
  #        Name: wait_for_no_power
  #
  #  Parameters: tile_path - tile path of the invoking widget
  #
  # Description: Wait until no power is detected or CONTINUE is pressed
  #
  #***************************************************************************
  proc wait_for_no_power { tile_path } \
  {
      variable  power_threshold
      set title  "Operator Instruction"
      set prompt_text  "Unkey the radio"

      # Display a NO WAIT dialog then wait for the Power level to fall below 0.1 W
      set wait_tile_path  [wait_dialog  $tile_path  $title  $prompt_text  "ABORT" "CONTINUE" ]
      
      # Measure the Power level
          
      # Loop until the Power level exceeds the threshold value,
      # or the user presses the ABORT button
      while { ($::Power::State(reading) > $power_threshold) && ![Abort] } \
      {
         after 100
      }
      
      if {[Abort] == "1"} {return -code break}
      # Remove the NO WAIT dialog from the display
      # if the Power level exceeded the threshold

      wait_dialog_remove  $wait_tile_path
  }

  #***************************************************************************
  #
  #        Name: wait_dialog_remove
  #
  #  Parameters: tile_path - tile path of the invoking widget
  #
  # Description: Removes the WAIT dialog from the display
  #
  #***************************************************************************
  proc wait_dialog_remove { tile_path } \
  {
    Debug::Puts AutoTest 2 "wait_dialog_remove:  $tile_path"
    
    if { [winfo  exists  $tile_path] } \
    {
      set tile_path  [winfo  toplevel  $tile_path]
    
      # Delay before destroying the toplevel so that the key presses and 
      # key releases propagate correctly.
      after 750 "destroy  $tile_path"
    }
        
    return
  }


  #***************************************************************************
  #
  #        Name: toggle_scroll_on
  #
  #  Parameters: tile_path      - tile path of the invoking widget
  #              scrollbar_name - name of the invoking scrollbar
  #
  # Description: Toggles the 'scroll_on' variable to allow the spinner
  #              knob to control the scrollbar
  #
  #***************************************************************************
  proc toggle_scroll_on { tile_path  scrollbar_name } \
  {
    Debug::Puts AutoTest 2 "toggle_scroll_on:  $tile_path,  $scrollbar_name"
    
    variable  summary_scroll_on
    variable  results_scroll_on
    
    set tile_path  [get_tile  $tile_path]
    
    if { $scrollbar_name == "scrl_summary" } \
    {
      set scrollbar_path  ${tile_path}.frm_config_summary.scrl_summary
    
      if { $summary_scroll_on == 0 } \
      {
        # Change the color of the scrollbar to indicate it is active
        $scrollbar_path config  -bg white
      
        set summary_scroll_on  1
        
        # For each event, call the procedure to move the scrollbar,
        # then break to keep the focus from moving in response to the 
        # event
        # Bind the <Right> event  (includes <<Spin_up>>)
        bind $scrollbar_path  <Right>  "Auto_Test_2::move_textbox  $tile_path  txt_summary  UP; break"
                                     
        # Bind the <Left> event  (includes <<Spin_down>>)
        bind $scrollbar_path  <Left>  "Auto_Test_2::move_textbox  $tile_path  txt_summary  DOWN; break"
                                     
        bind $scrollbar_path  <Up>  "Auto_Test_2::move_textbox  $tile_path  txt_summary  UP; break"
                                     
        bind $scrollbar_path  <Down>  "Auto_Test_2::move_textbox  $tile_path  txt_summary  DOWN; break"
      } \
      else \
      {
        # Change the color of the scrollbar to indicate it is NOT active
        $scrollbar_path config  -bg gray
      
        set summary_scroll_on  0
      
        # Unbind the <Right> event  (includes <<Spin_up>>)
        bind $scrollbar_path  <Right>  ""

        # Unbind the <Left> event  (includes <<Spin_down>>)
        bind $scrollbar_path  <Left>  ""
        
        # For the 'Up' of 'Down' event, generate the 'cursor_up' or 'cursor_down'
        # event, then break to keep the scrollbar from moving in response to 
        # either event
        bind $scrollbar_path  <Up>    "event generate %W <<cursor_up>>; break"
        bind $scrollbar_path  <Down>  "event generate %W <<cursor_down>>; break"
      }
    }

    if { $scrollbar_name == "scrl_results" } \
    {
      set scrollbar_path  ${tile_path}.frm_results.scrl_results
    
      if { $results_scroll_on == 0 } \
      {
        # Change the color of the scrollbar to indicate it is active
        $scrollbar_path config  -bg white
      
        set results_scroll_on  1
        
        # For each event, call the procedure to move the scrollbar,
        # then break to keep the focus from moving in response to the 
        # event
        # Bind the <Right> event  (includes <<Spin_up>>)
        bind $scrollbar_path  <Right>  "Auto_Test_2::move_textbox  $tile_path  txt_results  UP; break"
                                     
        # Bind the <Left> event  (includes <<Spin_down>>)
        bind $scrollbar_path  <Left>  "Auto_Test_2::move_textbox  $tile_path  txt_results  DOWN; break"
                                     
        bind $scrollbar_path  <Up>  "Auto_Test_2::move_textbox  $tile_path  txt_results  UP; break"
                                     
        bind $scrollbar_path  <Down>  "Auto_Test_2::move_textbox  $tile_path  txt_results  DOWN; break"
      } \
      else \
      {
        # Change the color of the scrollbar to indicate it is NOT active
        $scrollbar_path config  -bg gray
      
        set results_scroll_on  0
      
        # Unbind the <Right> event  (includes <<Spin_up>>)
        bind $scrollbar_path  <Right>  ""

        # Unbind the <Left> event  (includes <<Spin_down>>)
        bind $scrollbar_path  <Left>  ""
        
        # For the 'Up' of 'Down' event, generate the 'cursor_up' or 'cursor_down'
        # event, then break to keep the scrollbar from moving in response to 
        # either event
        bind $scrollbar_path  <Up>    "event generate %W <<cursor_up>>; break"
        bind $scrollbar_path  <Down>  "event generate %W <<cursor_down>>; break"
      }
    }

    
    return
  }
      

  #***************************************************************************
  #
  #        Name: move_textbox
  #
  #  Parameters: tile_path    - tile path of the invoking widget
  #              textbox_name - name of the textbox to move
  #              direction    - direction to move the scrollbar
  #
  # Description: Moves the textbox display for changes on the spinner knob 
  #              or Up and Down arrows.
  #
  #***************************************************************************
  proc move_textbox { tile_path  textbox_name  direction } \
  {
    Debug::Puts AutoTest 2 "move_textbox:  $tile_path,  $textbox_name,  $direction"
    
    variable  summary_scroll_on
    variable  results_scroll_on
    
    set tile_path  [get_tile  $tile_path]
    
    if { $textbox_name == "txt_summary" } \
    {
      if { $summary_scroll_on == 0 } \
      {
        return
      }

      set textbox_path  ${tile_path}.frm_config_summary.txt_summary

      # Move the canvas display
      switch -- $direction \
      {
        "UP"    { $textbox_path  yview  scroll  -1  unit }
    
        "DOWN"  { $textbox_path  yview  scroll  1  unit }
      }
    }
    
    if { $textbox_name == "txt_results" } \
    {
      if { $results_scroll_on == 0 } \
      {
        return
      }

      set textbox_path  ${tile_path}.frm_results.txt_results

      # Move the canvas display
      switch -- $direction \
      {
        "UP"    { $textbox_path  yview  scroll  -1  unit }
    
        "DOWN"  { $textbox_path  yview  scroll  1  unit }
      }
    }

    return
  }


  #****************************************************************************
  #
  #        Name:        getSaveState
  #
  #  Parameters:  none
  #
  # Description: This function returns a list of TCL commands which can be used
  #              to setup the Auto Test screen to its current state
  #
  #****************************************************************************
  proc getSaveState { } \
  {
    Debug::Puts AutoTest 2 "Auto_Test_2::getSaveState"
    
    variable  test_enable
    variable  test_radio_type
    variable  test_radio_type_menu
    variable  test_type
    variable  test_type_menu

    set returnList \
    {
      set ::Auto_Test_2::test_radio_type_menu  {$test_radio_type_menu}
      set ::Auto_Test_2::test_radio_type       {$test_radio_type}
      set ::Auto_Test_2::test_type_menu        {$test_type_menu}
      set ::Auto_Test_2::test_type             {$test_type}
      set ::Auto_Test_2::abort_flag 1
    }

    foreach { index } [array names ::Auto_Test_2::test_enable] \
    {
      append returnList  "set ::Auto_Test_2::test_enable($index) {$test_enable($index)}\n"
    }

    Debug::Puts AutoTest 2 "returnList:  $returnList"
    
    return [subst $returnList]
  }


  #****************************************************************************
  #
  #        Name: Get_directory_entry_test_config
  #
  #  Parameters: file name being examined
  #
  #  Upvar's values:
  #    type  - the string to show for the file type
  #    extra - any extra HTML to go in the last table column
  #    url   - the URL to use if the user clicks on this file
  #    name  - the name string for the file
  #    size  - file size in bytes
  #
  # Description: Return any file specific info back to the web server
  #
  #****************************************************************************
  proc Get_directory_entry_test_config { f } \
  {
    upvar type  type
    upvar extra  extra

    # Set the type of file
    set type  "Auto Test config"

    array set setup_info  { }

    # Parse the file to get the 'FILE_LABEL'
    read_info  $f  setup_info
        
    if { ($setup_info(label,valid) == 1) && ($setup_info(label) != "") } \
    {
      append type  ":<br><b>$setup_info(label)</b>"
    }

    set extra  [subst {<td><a href="/help/Applications/Auto_Test/show_setup.shtml?file=[HTTPD::encode $f]">View as HTML</a></td>}]
    
    return
  }
  
  
  #****************************************************************************
  #
  #        Name: Get_directory_entry_test_results    
  #
  #  Parameters: file name being examined
  #
  #  Upvar's values:
  #    type  - the string to show for the file type
  #    extra - any extra HTML to go in the last table column
  #    url   - The URL to use if the user clicks on this file
  #    name  - the name string for the file
  #    size  - file size in bytes
  #
  # Description: Return any file specific info back to the web server
  #
  #****************************************************************************
  proc Get_directory_entry_test_results { f } \
  {
    upvar type  type
    upvar extra  extra

    # Set the type of file
    set type  "Auto Test results"

    array set setup_info  { }

    # Parse the file to get the 'FILE_LABEL'
    read_info  $f  setup_info
        
    if { ($setup_info(label,valid) == 1) && ($setup_info(label) != "") } \
    {
      append type  ":<br><b>$setup_info(label)</b>"
    }

    # NO 'extra' which will display the file data as a structured web page
    #set extra  ""
  }
    
    
  #****************************************************************************
  #
  #        Name: read_info
  #
  #  Parameters: file name, name of array in parent to fill in
  #
  # Description: Reads a file & gets array data
  #
  #     Returns: 1 if valid save file, else 0
  #
  #****************************************************************************
  proc read_info { file  name } \
  {
    Debug::Puts AutoTest 2 "read_info:  $file,  $name"

    upvar $name  var
    
    set var(label) "<not used>"
    set var(label,valid) 0
    set var(ftime) ""

    if { ![file exist $file] } \
    {
      return  0
    }

    set var(ftime)  [clock format [file mtime $file]]
    
    # try to open the file for reading
    if [catch {set fid [open $file r] } err ] \
    {
      return  0
    }
    
    # scan not more than the first 10 lines of the file
    for { set i 0 } { $i < 10 } { incr i } \
    {
      if { [ gets $fid line ] < 0 } \
      {
        break
      }
       
      if { [info complete $line] } \
      {
        if {[regexp {FILE_LABEL[ ]*(.*)} $line x data] == 1} \
        {
           set var(label)        $data 
           set var(label,valid)  1
        }
        if {[regexp {FILE_DATE[ ]*(.*)} $line x data] == 1} \
        {
            set var(ftime)        $data
        }
      }
    }
    
    close  $fid
    
    return
  }


  #****************************************************************************
  #
  #        Name: Config_to_HTML    
  #
  #  Parameters: none
  #
  #  Upvar's values:
  #    this(CGI,file) - the file being examined
  #
  # Description: Convert a setup file into HTML.
  #              Called from the 'show_setup.shtml' file
  #
  #****************************************************************************
  proc Config_to_HTML {  } \
  {
    variable  test_namespace_list
    
    upvar this  this

    # Localize the file name
    set file  $this(CGI,file)

    # Open the file for reading
    set fid  [open  $file  r]

    # Read the file data
    set d  [read  $fid]

    # Close the file
    close  $fid

    # Create the web page display of the data in the file
    # Get rid of the first two lines of the file data, i.e FILE_LABEL and FILE_DATE
      # Find out where the first 'set' command is located
      set start_index  [string  first  "set"  $d]

      # Get the data from this index to the end
      set d  [string  range  $d  $start_index  end]    

    # Substitute the 'test_data' variable for 'test_data_html',
    # in order to allow the data to be sourced as Tcl commands without 
    # affecting the present test data
    regsub  -all  "test_data"  $d  "test_data_html"  html_data
    
    # Source the data to set the values
    eval  "$html_data"
    
    set html_display  ""
    
    # Get the HTML data from each namespace
    foreach { test_namespace } $test_namespace_list \
    {
      append html_display  [${test_namespace}::get_html_config]
    }
    
    puts "html_display:  $html_display"
     
    return  $html_display
  }
  
  
  #****************************************************************************
  # Functions executed when the namespace is sourced
  #****************************************************************************
  # If the 'Auto Test' option is installed, register the screen
  if { [IFR_options  is_installed  Auto_Test] } \
  {
    ::View::RegisterScreen  Auto_Test_2;  # Add this on the menu
  }


};  # End of the Auto_Test_2


#****************************************************************************
# This registers the screens with the list for saving instrument states
#****************************************************************************
namespace eval ::SavedStates \
{
  variable screens   

  lappend screens  Auto_Test_2
}


#****************************************************************************
# Tell webserver about my file formats
#****************************************************************************
namespace eval ::HTTPD \
{
  array set file_handlers \
  {
    .tstcfg      ::Auto_Test_2::Get_directory_entry_test_config
    .csv  ::Auto_Test_2::Get_directory_entry_test_results
  }
}

#****************************************************************************
# Option Definition: Define the namespace, and set up our HTML option info
#****************************************************************************
namespace eval ::Auto_Test \
{
  variable InfoString

  set temp_date [string trimleft [firmware_date] "Date:"]
  set end_loc [string first "Time:" $temp_date]
  set end_loc [expr $end_loc - 2]
  set temp_date [string range $temp_date 0 $end_loc]
  set InfoString [subst \
  {
         <br>
         (using Auto Test version - $temp_date, [firmware_version])
         <br>
         This option allows the user to setup and run automated radio tests.
  }]
}
