#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C) 2001      DESIGN ENGINEERING        #
# --------------------------------------------------------------------------#
# FILE        : 3005.manual_test.tcl                                        #
# PRODUCT     : FITS                                                        #
# AUTHOR      : Preston McCormick                                           #
# DESCRIPTION : This namespace manages the Manual Test                      #
#############################################################################
#                                                                           #
# Required Variables:                                                       #
#    LongName           Name of screen                                      #
#    ShortName          Name of screen for menus                            #
#    RecomendedTiles    List of tiles available for this screen. It is not  #
#                       necessary to put a tile on this list that is on the #
#                       RequiredTiles list                                  #
#    NotAllowedTiles    List of tiles NEVER on this screen                  #
#    RequiredTiles      List of tiles ALWAYS on this screen                 #
#    ActiveTiles        List of tiles currently displayed on this screen    #
#    MenuFlags          0 = Active Screen                                   #
#                       1 = Currently functioning as a submenu parent       #
#                                                                           #
# Required Variables for submenu "Screens":                                 #
#    SubmenuScreens     List of all tiles (or submenus) for droplist        #
#    SubDirPath         Path to subdirectory holding the screens for the    #
#                       submenu of this "screen"                            #
#                                                                           #
#############################################################################
#                                                                           #
# Utility Functions:                                                        #
#    Setup              Called by the screen manager to create the screen.  #
#    Map                Called when screen is mapped on window              #
#    Unmap              Called when screen is removed from display          #
#    Destroy            Called when screen is destroyed                     #
#                                                                           #
#############################################################################

namespace eval Manual_Test \
{
  variable  LongName   "manual_test"
  variable  ShortName  "Manual Test"
  variable  Size       Full
            

  variable  RequiredTiles    { }
  variable  NotAllowedTiles  { }
  variable  RecommendedTiles { }
  variable  ActiveTiles      { }
  variable  SubmenuScreens   { }

  variable  MenuFlags 0

  variable  SubDirPath { }

  variable  last_focus  ""

  # Test information variables
  
  variable  test_start_time  ""
  variable  test_stop_time   ""
  variable  test_total_time  ""
  
  variable  test_date       "Nov. 24, 2002"
  variable  test_time       "11:00:00"
  variable  test_unit_id    "1234"
  variable  test_tech_name  "John Q."

  variable  test_name       "GENERIC"
  variable  test_name_menu  { "GENERIC"  "SPECIFIC" }

  variable  test_label       "FIRST TIME"
  variable  test_label_menu  { "FIRST TIME"  "SECOND TIME"  "THIRD TIME" }
  
  
  variable  setup_menu

  set  setup_menu(command)   { "SET"  "GET"  "PASS/FAIL" }
  
  set  setup_menu(system)    { "Generator"           \
                               "Receiver"            \
                               "Spectrum Analyzer"   \
                               "Function Generator"  \
                               "Oscilloscope" }

  set  setup_menu(subsystem,gen)     { "Freq" "Level" "RF" "Output" "Mod Type" }
  set  setup_menu(gen,freq_units)    { "MHz" }
  set  setup_menu(gen,level_units)   { "dBm" "mV" "uV" }
  set  setup_menu(gen,rf_state)      { "OFF" "ON" }
  set  setup_menu(gen,output)        { "ANT" "T/R" }
  set  setup_menu(gen,mod_type)      { "OFF" "P25" "FM" "AM" }
  
  set  setup_menu(subsystem,rec)     { "Freq" "Input" "Atten" "Demod" "IF BW" "Filter" }
  set  setup_menu(rec,freq_units)    { "MHz" }
  set  setup_menu(rec,input)         { "ANT" "T/R" }
  set  setup_menu(rec,atten)         { "0" "10" "20" "30" "40" }
  set  setup_menu(rec,demod)         { "P25" "FM" "AM" }
  set  setup_menu(rec,if_bw)         { "12.5kHz" "25kHz" "60kHz" "200kHz" }
  set  setup_menu(rec,filter)        { "NONE" "300Hz BP" "300Hz LP" "4kHz B LP" "15kHz LP" \
                                       "20kHz LP" "4kHz HP" "300-4kHz BP" "80Hz N BP" "3kHz N BP" }

  set  setup_menu(subsystem,anlz)    { "Freq" "Span" "RBW" "VBW" "Sweep" "TOS" "Scale" }
  set  setup_menu(anlz,freq_units)   { "MHz" }
  set  setup_menu(anlz,span)         { "0" "1kHz" "2kHz" "5kHz" "10kHz" "20kHz" "50kHz" \
                                       "100kHz" "200kHz" "500kHz" "1MHz" "2MHz" "5MHz"  \
                                       "10MHz" "20MHz" "50MHz" "100MHz" "200MHz" "500MHz" \
                                       "1GHz" "2GHz" "3GHz" }
  set  setup_menu(anlz,rbw)          { "300Hz" "3kHz" "30kHz" "60kHz" "300kHz" "6MHz" }
  set  setup_menu(anlz,vbw)          { "10Hz" "30Hz" "100Hz" "300Hz" "1kHz" "3kHz" "10kHz" \
                                       "30kHz" "300kHz" "1MHz" "3MHz" "NONE" }
  set  setup_menu(anlz,sweep)        { "200ms" "500ms" "750ms" "1s" "2s" "3s" "4s" "5s" "6s" }
  set  setup_menu(anlz,tos)          { "60dBm" "50dBm" "40dBm" "30dBm" "20dBm" "10dBm" "0dBm" \
                                       "-10dBm" "-20dBm" "-30dBm" "-40dBm" }
  set  setup_menu(anlz,scale)        { "10" "5" "2" }

  set  setup_menu(subsystem,fgen)    { "TONE" "TREM" "DTMF" "CTCSS" "DCS" "TSIG" }
 
  set  setup_menu(subsystem,oscope)    { "Source" "Sweep" "Scale" "HPos" "Trig Type" \
                                         "Trig Source" "Trig Level" "Trig Slope" \
                                         "Coupling" }
  set  setup_menu(oscope,source)       { "CH1" "CH2" "MIC Dyn" "MIC Elec" "Aud Unbal" \
                                         "Aud Bal" "DEMOD" }
  set  setup_menu(oscope,sweep)        { "1us" "2us" "5us" "10us" "20us" "50us" "100us" "200us" \
                                         "500us" "1ms" "2ms" "5ms" "10ms" "20ms" "50ms" "100ms" \
                                         "200ms" "500ms" "1s" }
  set  setup_menu(oscope,scale)        { ".02" ".05" ".1" ".2" ".5" "1" "2" "5" "10" "20" "50" }
  set  setup_menu(oscope,hpos_units)   { "div" }
  set  setup_menu(oscope,trig_type)    { "AUTO" "NORM" "ONE" }
  set  setup_menu(oscope,trig_source)  { "CH1" "CH2" "Ext" }
  set  setup_menu(oscope,level_units)  { "V" }
  set  setup_menu(oscope,trig_slope)   { "FALL" "RISE" }
  set  setup_menu(oscope,coupling)     { "AC" "DC" "GND" }

  variable  config_description  ""

  variable  results         ""
  
  variable  save_flag   0
  variable  abort_flag  0

  variable  test_enable
  
  set  test_enable(1)  1

  variable  test_title
  
  set test_title(1)  ""

  variable  test_num

  variable  num_setups      0
  variable  num_setups_max  10

  variable  printer_dir   "printer0"

  # Test variables
  variable  test_data

  set test_data(1,test_title)  "Example Test"


  variable  test_data_save  ""

  # Results file variables
  variable  file_ext      ".TXT"
  variable  file_label    "Results"
  variable  file_prefix   "FILE"
  variable  working_dir   "/usr/apps/"

  # Test data file variables
  variable  data_file_name     "TEST.DAT"
  variable  data_working_dir   "/usr/apps/test_data/"


  # Operator prompt dialog variables
  variable  return_value
  

  #***************************************************************************
  #
  # Name: Setup
  #
  # Parameters: parent - parent of child window
  #
  # Description: Creates the main tile
  #
  #***************************************************************************
  proc Setup { parent } \
  {
    #puts "Manual_Test::Setup:  $parent"

    variable  LongName
    variable  tile_class

    set top_level  [winfo toplevel $parent]
   
    set tile_path  [name_child  $top_level  $LongName]

    if { ![winfo exists $tile_path] } \
    {
      set tile_path  [::Screens::create_tile  $tile_path  $LongName  MANUAL_TEST]
    }  
     
    return $tile_path
  }


  #***************************************************************************
  #
  # Name: Map
  #
  # Parameters: 
  #
  # Description: Initializes the tile
  #
  #***************************************************************************
  proc Map { tile_path } \
  {
    #puts "::Manual_Test::Map:  $tile_path"
    
    set tile_path  [get_tile  $tile_path]    
    
    ::View::PackOptions  $tile_path  ""

    trace variable ::Manual_Test::results  w  \
                             "::Manual_Test::test_results_display  $tile_path"
                                   
    test_info_display  $tile_path
    
    
    return
  }


  #***************************************************************************
  #
  # Name: Cleanup
  #
  # Parameters: 
  #
  # Description: 
  #
  #***************************************************************************
  proc Cleanup { tile_path } \
  {
    #puts "::Manual_Test::Cleanup:  $tile_path"

    if ![winfo exists $tile_path] { return }
    set tile_path  [get_tile  $tile_path]    
    
    trace vdelete ::Manual_Test::results  w  \
                             "::Manual_Test::test_results_display  $tile_path"

    return
  }


  #***************************************************************************
  #
  # Name: Unmap
  #
  # Parameters: 
  #
  # Description: 
  #
  #***************************************************************************
  proc Unmap { tile_path } \
  {
    #puts "::Manual_Test::Unmap:  $tile_path"
    if ![winfo exists $tile_path] { return }
    set tile_path  [get_tile  $tile_path]    

    Cleanup  $tile_path
    
    return
  }


  #***************************************************************************
  #
  # Name: Destroy
  #
  # Parameters: 
  #
  # Description: 
  #
  #***************************************************************************
  proc Destroy { tile_path } \
  {
    #puts "::Manual_Test::Destroy:  $tile_path"

    if ![winfo exists $tile_path] { return }
    set tile_path  [get_tile  $tile_path]    

    Cleanup  $tile_path

    return
  }


  #***************************************************************************
  #
  # Name: Register
  #
  # Parameters: 
  #
  # Description: 
  #
  #***************************************************************************
  proc Register { } \
  {
    #puts "::Manual_Test::Register:  $tile_path"

    set tile_path  [get_tile  $tile_path]    

    return
  }
  

  #***************************************************************************
  #
  # Name: get_tile
  #
  # Parameters: any child widget of the tile
  #
  # Description: Get the name of the tile's parent frame
  #
  #***************************************************************************
  proc get_tile { widget } \
  {
    while { 1 } \
    {
      switch -- [winfo class $widget] \
      {
        "MANUAL_TEST"  { return $widget }
      }
        
      set widget [winfo parent $widget]
    }


    return     
  }



  #***************************************************************************
  #
  # Test Information procedures
  #
  #***************************************************************************
  #***************************************************************************
  #
  # Name: test_info_display
  #
  # Parameters: widget - invoking widget tile path
  #
  # Description: Displays the Test Information tile
  #
  #***************************************************************************
  proc test_info_display { widget } \
  {
    #puts "test_info_display:  $widget"

    set tile_path  [get_tile  $widget]
    
    # Display the tile
    grid remove ${tile_path}.frm_test_select
    grid remove ${tile_path}.frm_test_config
    grid remove ${tile_path}.frm_test_results

    Screens::position_widget  ${tile_path}.frm_test_info
    
    # Remove the New and Delete Test name entry options from the screen
    grid remove  ${tile_path}.frm_test_info.lbl_new_test_name
    grid remove  ${tile_path}.frm_test_info.ent_new_test_name
    grid remove  ${tile_path}.frm_test_info.lbl_del_test_name
    grid remove  ${tile_path}.frm_test_info.ent_del_test_name

    # Remove the New and Delete Test label entry options from the screen
    grid remove  ${tile_path}.frm_test_info.lbl_new_test_label
    grid remove  ${tile_path}.frm_test_info.ent_new_test_label
    grid remove  ${tile_path}.frm_test_info.lbl_del_test_label
    grid remove  ${tile_path}.frm_test_info.ent_del_test_label

    # Update the DATE and TIME
    update_date_time
    
    # Set the softkeys
    # Softkey #2:  Start
    ::View::AssignSoftkey  $tile_path  2  "CONTINUE"  \
                              "::Manual_Test::test_select_display  $tile_path"

    # Softkey #3:  Add radio type
    ::View::AssignSoftkey  $tile_path  3  "ADD\nNAME"  \
                            "::Manual_Test::display_add_test_name  $tile_path"

    # Softkey #4:  Delete radio type
    ::View::AssignSoftkey  $tile_path  4  "DELETE\nNAME"  \
                         "::Manual_Test::display_delete_test_name  $tile_path"

    # Softkey #5:  Add test type
    ::View::AssignSoftkey  $tile_path  5  "ADD\nLABEL"  \
                           "::Manual_Test::display_add_test_label  $tile_path"

    # Softkey #6:  Delete test type
    ::View::AssignSoftkey  $tile_path  6  "DELETE\nLABEL"  \
                        "::Manual_Test::display_delete_test_label  $tile_path"

    # Softkey #7:
    ::View::AssignSoftkey $tile_path  7  ""  ""

    return
  }
  
  
  #***************************************************************************
  #
  # Name: update_date_time
  #
  # Parameters:
  #
  # Description: Updates the date and time of the the test to system time
  #              when the START softkey is pressed
  #
  #***************************************************************************
  proc update_date_time { } \
  {
    #puts "update_date_time"

    variable  test_date
    variable  test_time

    set date_time  [clock format [clock seconds]]

    # Example:  Wed Nov 13 18:36:05  2002
    
    foreach { weekday  month  month_day  time year } $date_time \
    {
      set test_date  "$month.  $month_day,  $year"
      set test_time  "$time"
    }

    return
  }


  #***************************************************************************
  #
  # Name: display_add_test_name
  #
  # Parameters: widget - invoking widget tile path
  #
  # Description: Displays an entry field to enter a new test name
  #
  #***************************************************************************
  proc display_add_test_name { widget } \
  {
    #puts "display_add_test_name:  $widget"

    variable  new_test_name
    variable  test_name_menu
     
    set tile_path  [get_tile  $widget]
    
    set new_test_name  ""

    # Get the name of the new radio type
    Screens::position_widget  ${tile_path}.frm_test_info.lbl_new_test_name
    Screens::position_widget  ${tile_path}.frm_test_info.ent_new_test_name
    

    # Softkey #7:  Cancel
    ::View::AssignSoftkey  $widget  7  "CANCEL"  \
                                   "::Manual_Test::test_info_display  $widget"

    return
  }


  #***************************************************************************
  #
  # Name: add_test_name
  #
  # Parameters: widget - invoking widget tile path
  #
  # Description: Adds a new test name to the menu
  #
  #***************************************************************************
  proc add_test_name { widget } \
  {
    #puts "add_test_name:  $widget"

    variable  test_data
 
    variable  new_test_name
    variable  test_name_menu
     
    set tile_path  [get_tile  $widget]

    grid remove  ${tile_path}.frm_test_info.lbl_new_test_name
    grid remove  ${tile_path}.frm_test_info.ent_new_test_name

    if { $new_test_name != "" } \
    {
      # Add the new radio type to the menu
      set menu_name  ${tile_path}.frm_test_info.mnu_test_name.menu
      
      set last_index  [$menu_name  index  end]
      
      incr last_index  1
      
      $menu_name add radiobutton                          \
                    -accel     $last_index                \
                    -label     $new_test_name             \
                    -value     $new_test_name             \
                    -variable  ::Manual_Test::test_name

      # Add the new radio type to the menu list variable
      lappend test_name_menu  "$new_test_name"
    
      # Display the new selection
      $menu_name  invoke  end
    }

    return
  }


  #***************************************************************************
  #
  # Name: display_delete_test_name
  #
  # Parameters: widget - invoking widget tile path
  #
  # Description: Displays an entry field to enter a test name to delete
  #
  #***************************************************************************
  proc display_delete_test_name { widget } \
  {
    #puts "display_delete_test_name:  $widget"

    variable  del_test_name
    variable  test_name_menu
     
    set tile_path  [get_tile  $widget]
    
    set del_test_name  ""

    # Get the name of the new radio type
    Screens::position_widget  ${tile_path}.frm_test_info.lbl_del_test_name
    Screens::position_widget  ${tile_path}.frm_test_info.ent_del_test_name
    

    # Softkey #7:  Cancel
    ::View::AssignSoftkey  $widget  7  "CANCEL"  \
                                   "::Manual_Test::test_info_display  $widget"

    return
  }


  #***************************************************************************
  #
  # Name: delete_test_name
  #
  # Parameters: widget - invoking widget tile path
  #
  # Description: Deletes a radio type from the menu
  #
  #***************************************************************************
  proc delete_test_name { widget } \
  {
    #puts "delete_test_name:  $widget"

    variable  test_data
 
    variable  del_test_name
    variable  test_name_menu
     
    set tile_path  [get_tile  $widget]

    grid remove  ${tile_path}.frm_test_info.lbl_del_test_name
    grid remove  ${tile_path}.frm_test_info.ent_del_test_name

    if { $del_test_name != "" } \
    {
      # Delete the radio type from the menu
      set menu_name  ${tile_path}.frm_test_info.mnu_test_name.menu
      
      set index  [lsearch -exact $test_name_menu "$del_test_name"]
        
      #puts "index:  $index"
      
      if { $index >= 0 } \
      {
        set test_name_menu  [lreplace $test_name_menu $index $index]
        
        #puts "test_name_menu:  $test_name_menu"
        
        # Delete the radio type from the menu
        $menu_name delete $index

        $menu_name invoke 0
      }
    }
    
    return
  }


  #***************************************************************************
  #
  # Name: display_add_test_label
  #
  # Parameters: widget - invoking widget tile path
  #
  # Description: Displays an entry field to enter a new test type
  #
  #***************************************************************************
  proc display_add_test_label { widget } \
  {
    puts "display_add_test_label  $widget"

    variable  new_test_label
    variable  test_label_menu
     
    set tile_path  [get_tile  $widget]
    
    set new_test_label  ""

    # Get the name of the new radio type
    Screens::position_widget  ${tile_path}.frm_test_info.lbl_new_test_label
    Screens::position_widget  ${tile_path}.frm_test_info.ent_new_test_label
    

    # Softkey #7:  Cancel
    ::View::AssignSoftkey  $widget  7  "CANCEL"  \
                                   "::Manual_Test::test_info_display  $widget"

    return
  }


  #***************************************************************************
  #
  # Name: add_test_label
  #
  # Parameters: widget - invoking widget tile path
  #
  # Description: Adds a new test type to the menu
  #
  #***************************************************************************
  proc add_test_label { widget } \
  {
    puts "add_test_label:  $widget"

    variable  new_test_label
    variable  test_label_menu
     
    set tile_path  [get_tile  $widget]

    grid remove  ${tile_path}.frm_test_info.lbl_new_test_label
    grid remove  ${tile_path}.frm_test_info.ent_new_test_label

    if { $new_test_label != "" } \
    {
      # Add the new radio type to the menu
      set menu_name  ${tile_path}.frm_test_info.mnu_test_label.menu
      
      set last_index  [$menu_name  index  end]
      
      incr last_index  1
      
      $menu_name add radiobutton  -accel     $last_index            \
                                  -label     $new_test_label         \
                                  -value     $new_test_label         \
                                  -variable  ::Manual_Test::test_label

      # Add the new radio type to the menu list variable
      lappend test_label_menu  "$new_test_label"
    
      # Display the new selection
      $menu_name  invoke  end
    }

    return
  }


  #***************************************************************************
  #
  # Name: display_delete_test_label
  #
  # Parameters: widget - invoking widget tile path
  #
  # Description: Displays an entry field to enter a test type to delete
  #
  #***************************************************************************
  proc display_delete_test_label { widget } \
  {
    puts "display_delete_test_label:  $widget"

    variable  del_test_label
    variable  test_label_menu
     
    set tile_path  [get_tile  $widget]
    
    set del_test_label  ""

    # Get the name of the new radio type
    Screens::position_widget  ${tile_path}.frm_test_info.lbl_del_test_label
    Screens::position_widget  ${tile_path}.frm_test_info.ent_del_test_label
    

    # Softkey #7:  Cancel
    ::View::AssignSoftkey  $widget  7  "CANCEL"  \
                                   "::Manual_Test::test_info_display  $widget"

    return
  }


  #***************************************************************************
  #
  # Name: delete_test_label
  #
  # Parameters: widget - invoking widget tile path
  #
  # Description: Deletes a test type from the menu
  #
  #***************************************************************************
  proc delete_test_label { widget } \
  {
    puts "delete_test_label:  $widget"

    variable  del_test_label
    variable  test_label_menu
     
    set tile_path  [get_tile  $widget]

    grid remove  ${tile_path}.frm_test_info.lbl_del_test_label
    grid remove  ${tile_path}.frm_test_info.ent_del_test_label

    if { $del_test_label != "" } \
    {
      # Delete the radio type from the menu
      set menu_name  ${tile_path}.frm_test_info.mnu_test_label.menu
      
      set index  [lsearch -exact $test_label_menu "$del_test_label"]
        
      #puts "index:  $index"
      
      if { $index >= 0 } \
      {
        set test_label_menu  [lreplace $test_label_menu $index $index]
        
        puts "test_label_menu:  $test_label_menu"
        
        # Delete the radio type from the menu
        $menu_name delete $index

        $menu_name invoke 0
      }
    }
    
    return
  }



  #***************************************************************************
  #
  # Test Select procedures
  #
  #***************************************************************************
  #***************************************************************************
  #
  # Name: test_select_display
  #
  # Parameters: widget - invoking widget tile path
  #
  # Description: Displays the Test Select tile
  #
  #***************************************************************************
  proc test_select_display { widget } \
  {
    #puts "test_select_display:  $widget"

    variable  test_data
    variable  test_title
    
    set tile_path  [get_tile  $widget]

    # Display the tile
    grid remove ${tile_path}.frm_test_info
    grid remove ${tile_path}.frm_test_config
    grid remove ${tile_path}.frm_test_results

    Screens::position_widget  ${tile_path}.frm_test_select
    
    # Remove the New and Delete Test entry options from the screen
    grid remove  ${tile_path}.frm_test_select.lbl_new_test
    grid remove  ${tile_path}.frm_test_select.ent_new_test
    grid remove  ${tile_path}.frm_test_select.lbl_del_test
    grid remove  ${tile_path}.frm_test_select.ent_del_test

    # Set the softkeys
    # Softkey #2:  Continue
    ::View::AssignSoftkey $tile_path  2  "CONTINUE"  \
                             "::Manual_Test::test_results_display  $tile_path"

    # Softkey #3:
    ::View::AssignSoftkey $tile_path  3  "ADD\nTEST"  \
                     "::Manual_Test::test_select_display_add_test  $tile_path"

    # Softkey #4:
    ::View::AssignSoftkey $tile_path  4  "REMOVE\nTEST"  \
                  "::Manual_Test::test_select_display_remove_test  $tile_path"

    # Softkey #5:
    ::View::AssignSoftkey $tile_path  5  ""  ""

    # Softkey #6:
    ::View::AssignSoftkey $tile_path  6  ""  ""

    # Softkey #7:
    ::View::AssignSoftkey $tile_path  7  ""  ""

    # Softkey #8:  RETURN
    ::View::AssignReturn $tile_path \
                                "::Manual_Test::test_info_display  $tile_path"

    
    # Loop through test options and remove all that exist
    set index  1
    
    # If the option widgets exist, remove them from the display
    while { [winfo exists ${tile_path}.frm_test_select.chk_test${index}] } \
    {
      grid remove  ${tile_path}.frm_test_select.chk_test${index}
      grid remove  ${tile_path}.frm_test_select.btn_test${index}
      
      incr index  1
    }


    # Display the valid test options
    set index  1
    
    # If the title variable exists, display the test option
    while { [info exists  test_data($index,test_title)] } \
    {
      Screens::position_widget  ${tile_path}.frm_test_select.chk_test${index}
      Screens::position_widget  ${tile_path}.frm_test_select.btn_test${index}
      
      set test_title($index)  $test_data($index,test_title)
    
      incr index  1
    }
    
    return
  }


  #***************************************************************************
  #
  # Name: test_select_display_add_test
  #
  # Parameters: widget - the invoking tile path
  #
  # Description: Displays an alphanumentry widget to allow a test to 
  #              be added to the select options
  #
  #***************************************************************************
  proc test_select_display_add_test { widget } \
  {
    puts "test_select_display_add_test:  $widget"

    variable  new_test
    
    set tile_path  [get_tile  $widget]
    
    set new_test  ""

    # Get the name of the new radio type
    Screens::position_widget  ${tile_path}.frm_test_select.lbl_new_test
    Screens::position_widget  ${tile_path}.frm_test_select.ent_new_test
    

    # Softkey #7:  Cancel
    ::View::AssignSoftkey  $widget  7  "CANCEL"  \
                                 "::Manual_Test::test_select_display  $widget"

    after idle  "safe_focus_force ${tile_path}.frm_test_select.ent_new_test"

    return
  }


  #***************************************************************************
  #
  # Name: test_select_add_test
  #
  # Parameters: widget - the invoking tile path
  #
  # Description: Adds a test to the select options
  #
  #***************************************************************************
  proc test_select_add_test { widget } \
  {
    puts "test_select_add_test:  $widget"

    variable  new_test

    variable  test_data
    variable  test_title
    variable  test_enable

    set tile_path  [get_tile  $widget]
    
    set frame_name  ${tile_path}.frm_test_select
    
    # Configure the checkbutton and config button
    # Determine the index of the last checkbutton displayed,
    # then configure the next checkbutton and config button
    # for the added test
    for { set index  1 } { $index <= 10 } { incr index  1 } \
    {
      if { ![winfo ismapped ${frame_name}.chk_test${index}] } \
      {
        Screens::position_widget  ${tile_path}.frm_test_select.chk_test${index}
        Screens::position_widget  ${tile_path}.frm_test_select.btn_test${index}

        set test_data($index,test_title)  "$new_test Test"

        set test_title($index)  "$new_test Test"
        set test_enable($index)  0
        
        break
      }
    }
    
    grid remove  ${tile_path}.frm_test_select.lbl_new_test
    grid remove  ${tile_path}.frm_test_select.ent_new_test

    return
  }


  #***************************************************************************
  #
  # Name: test_select_display_remove_test
  #
  # Parameters: widget - the invoking tile path
  #
  # Description: Displays an alphanumentry widget to allow a test to 
  #              be removed from the select options
  #
  #***************************************************************************
  proc test_select_display_remove_test { widget } \
  {
    puts "test_select_display_remove_test:  $widget"

    variable  del_test

    set tile_path  [get_tile  $widget]
    
    set del_test  ""

    # Get the name of the new radio type
    Screens::position_widget  ${tile_path}.frm_test_select.lbl_del_test
    Screens::position_widget  ${tile_path}.frm_test_select.ent_del_test
    

    # Softkey #7:  Cancel
    ::View::AssignSoftkey  $widget  7  "CANCEL"  \
                                 "::Manual_Test::test_select_display  $widget"

    after idle  "safe_focus_force ${tile_path}.frm_test_select.ent_del_test"

    return
  }


  #***************************************************************************
  #
  # Name: test_select_remove_test
  #
  # Parameters: widget - the invoking tile path
  #
  # Description: Removes a test from the select options
  #
  #***************************************************************************
  proc test_select_remove_test { widget } \
  {
    puts "test_select_remove_test:  $widget"

    variable  del_test

    set tile_path  [get_tile  $widget]
    
    set frame_name  ${tile_path}.frm_test_select
    
    grid remove  ${frame_name}.lbl_del_test
    grid remove  ${frame_name}.ent_del_test

    # Loop through the labels, 
    # find the one that matches the test to be removed,
    # remove it,
    # then readjust the other checkbuttons and config buttons
    for { set index  1 } { $index <= 10 } { incr index  1 } \
    {
      puts "index:  $index"
      
      if { $test_title($index) == "$del_test" } \
      {
        puts "Test found at index:  $index"
        
        set test_data($index,test_title)  ""

        set test_title($index)   ""
        set test_enable($index)  ""
        
        break
      }
    }
    
    # Only loop up to 9
    for { set index1  $index } { $index1 <= 9 } { incr index1  1 } \
    {
      puts "index1:  $index1"
      
      set index2  [incr index1  1]
      
      puts "index2:  $index2"
      
      # If the next checkbutton and config button are not mapped
      # break out of the loop
      if { ![winfo ismapped ${frame_name}.chk_test${index2}] } \
      {
        break
      }

      # Move all of the data value up by one
      if { [info exists test_data($index2,test_title)] } \
      {
        set test_data($index1,test_title)  "$test_data($index2,test_title)"
      }

      if { [info exists test_title($index2)] } \
      {
        set test_title($index1)   "$test_title($index2)"
      }
  
      if { [info exists test_enable($index2)] } \
      {
        set test_enable($index1)  $test_enable($index2)
      }
    }

    # Delete the last checkbutton and config button
    grid remove  ${frame_name}.chk_test${index1}
    grid remove  ${frame_name}.btn_test${index1}

    return
  }



  #***************************************************************************
  #
  # Test Configuration procedures
  #
  #***************************************************************************
  #***************************************************************************
  #
  # Name: test_config_display
  #
  # Parameters: widget - invoking widget tile path
  #             num    - number of the configuration
  #
  # Description: Displays the Test Configuration tile
  #
  #***************************************************************************
  proc test_config_display { widget  test_number } \
  {
    #puts "test_config_display:  $widget,  $test_number"

    variable  test_num
    
    variable  test_data
    
    variable  chan_info_type
    variable  config_description
    
    set tile_path  [get_tile  $widget]

    # Set the global variable
    set test_num  $test_number
    
    # Display the tile
    grid remove ${tile_path}.frm_test_info
    grid remove ${tile_path}.frm_test_select
    grid remove ${tile_path}.frm_test_results

    Screens::position_widget  ${tile_path}.frm_test_config
    
    # Set the softkeys
    # Softkey #2:
    ::View::AssignSoftkey $tile_path  2  ""  ""

    # Softkey #3:
    ::View::AssignSoftkey $tile_path  3  ""  ""

    # Softkey #4:
    ::View::AssignSoftkey $tile_path  4  ""  ""

    # Softkey #5:
    ::View::AssignSoftkey $tile_path  5  ""  ""

    # Softkey #6:
    ::View::AssignSoftkey $tile_path  6  ""  ""

    # Softkey #7:
    ::View::AssignSoftkey $tile_path  7  ""  ""

    # Softkey #8:  RETURN
    ::View::AssignReturn $tile_path  \
                      "::Manual_Test::test_config_display_return  $tile_path"

    # Configure the test description
    set config_description  $test_data($test_number,test_title)


    return
  }


  #***************************************************************************
  #
  # Name: test_config_display_return
  #
  # Parameters: tile_path - the invoking tile path
  #
  # Description: Performs procedure calls after the RETURN button is pressed
  #
  #***************************************************************************
  proc test_config_display_return { tile_path } \
  {
    #puts "test_config_display_return:  $tile_path"
  
    # Display the Select screen
    ::Manual_Test::test_select_display  $tile_path    
      
    return
  }
  

  #***************************************************************************
  #
  # Name: add_setup
  #
  # Parameters: widget - tile path of the invoking widget
  #
  # Description: Adds a set of setup information to the screen
  #
  #***************************************************************************
  proc add_setup { widget } \
  {
    puts "add_setup:  $widget"
    
    variable  num_setups
    variable  num_setups_max
        
    variable  test_num
    variable  test_data
    
    variable  setup_menu
    
    set tile_path  [get_tile  $widget]

    incr num_setups  1
    
    set frame_name  ${tile_path}.frm_test_config.frm_setup${num_setups}

    # Determine if the maximum number of channels that can be displayed
    # has been exceeded
    if { $num_setups > $num_setups_max } \
    {
      # If it has, decrement the number and return
      incr num_setups  -1
      
      return
    }
    

    # Initialize the command
    set test_data($test_num,system,$num_setups)  ""


    # Display the setup frame
    Screens::position_widget  $frame_name
    
    # Configure the menubutton 
    ${frame_name}.mnu_command config  \
       -textvariable  ::Manual_Test::test_data($test_num,command,$num_setups)

    # Delete the existing entries
    ${frame_name}.mnu_command.menu  delete  0  end

    # Configure the menu entries
    foreach { command } $setup_menu(command)  \
    {
      ${frame_name}.mnu_command.menu  add  radio  \
         -label  $command                     \
         -value  $command                     \
         -variable  ::Manual_Test::test_data($test_num,command,$num_setups)  \
         -command   "::Manual_Test::set_setup_command  $tile_path  $num_setups"
    }    

    # Set the initial state of the textvariable
    set test_data($test_num,command,$num_setups)  "SET"
    
    set_setup_command  $tile_path  $num_setups
    
    return
  }


  #***************************************************************************
  #
  # Name: remove_setup
  #
  # Parameters: widget - tile path of the invoking widget
  #
  # Description: Removes a set of setup information from the screen
  #
  #***************************************************************************
  proc remove_setup { widget } \
  {
    puts "remove_setup:  $widget"
    
    variable  num_setups
    
    variable  test_num
    variable  test_data
    
    set tile_path  [get_tile  $widget]

    if { $num_setups == 0 } \
    {
      return
    }
    
    set frame_name  ${tile_path}.frm_test_config.frm_setup${num_setups}

    grid remove  $frame_name

    incr num_setups  -1
    
    return
  }
  
  
  #***************************************************************************
  #
  # Name: set_setup_command
  #
  # Parameters: widget - tile path of the invoking widget
  #             num    - setup number
  #
  # Description: Sets the command for the setup
  #
  #***************************************************************************
  proc set_setup_command { widget  num } \
  {
    puts "set_setup_command:  $widget,  $num"
    
    variable  test_num
    variable  test_data
    
    variable  setup_menu
    
    set tile_path  [get_tile  $widget]

    set frame_name  ${tile_path}.frm_test_config.frm_setup${num}

    grid remove  ${frame_name}.mnu_system
    grid remove  ${frame_name}.lbl_max
    grid remove  ${frame_name}.ent_max
    grid remove  ${frame_name}.lbl_min
    grid remove  ${frame_name}.ent_min


    # Initialize the system
    set test_data($test_num,system,$num)  ""


    puts "test_data($test_num,command,$num):  $test_data($test_num,command,$num)"
    
    switch -- $test_data($test_num,command,$num) \
    {
      "SET"        { Screens::position_widget  ${frame_name}.mnu_system
      
                     ${frame_name}.mnu_system  config  \
                           -textvariable  ::Manual_Test::test_data($test_num,system,$num)
                                
                     # Delete the existing entries
                     ${frame_name}.mnu_system.menu  delete  0  end

                     # Configure the System menu
                     foreach { system } $::Manual_Test::setup_menu(system) \
                     {
                       ${frame_name}.mnu_system.menu  add  radio  \
                           -label  $system                        \
                           -value  $system                        \
                           -variable  ::Manual_Test::test_data($test_num,system,$num) \
                           -command   "::Manual_Test::set_setup_system  $tile_path  $num"
                     }

                     ${frame_name}.mnu_system.menu  invoke  0 }
      
      "GET"        { Screens::position_widget  ${frame_name}.mnu_system

                     ${frame_name}.mnu_system  config  \
                           -textvariable  ::Manual_Test::test_data($test_num,system,$num)
                                
                     # Delete the existing entries
                     ${frame_name}.mnu_system.menu  delete  0  end

                     # Configure the System menu
                     foreach { system } $::Manual_Test::setup_menu(system) \
                     {
                       ${frame_name}.mnu_system.menu  add  radio  \
                           -label  $system                        \
                           -value  $system                        \
                           -variable  ::Manual_Test::test_data($test_num,system,$num) \
                           -command   "::Manual_Test::set_setup_system  $tile_path  $num"
                     }

                     ${frame_name}.mnu_system.menu  invoke  0 }

      "PASS/FAIL"  { Screens::position_widget  ${frame_name}.lbl_max
                     Screens::position_widget  ${frame_name}.ent_max

                     Screens::position_widget  ${frame_name}.lbl_min
                     Screens::position_widget  ${frame_name}.ent_min 
                     
                     set_setup_system  $widget  $num
                     set_setup_subsystem  $widget  $num }
    }
    
    return
  }


  #***************************************************************************
  #
  # Name: set_setup_system
  #
  # Parameters: widget - tile path of the invoking widget
  #
  # Description: Sets the system for the setup
  #
  #***************************************************************************
  proc set_setup_system { widget  num } \
  {
    puts "set_setup_system:  $widget,  $num"

    variable  test_num
    variable  test_data

    variable  setup_menu

    set tile_path  [get_tile  $widget]

    set frame_name  ${tile_path}.frm_test_config.frm_setup${num}

    grid remove  ${frame_name}.mnu_subsystem_gen
    grid remove  ${frame_name}.mnu_subsystem_rec
    grid remove  ${frame_name}.mnu_subsystem_anlz
    grid remove  ${frame_name}.mnu_subsystem_fgen
    grid remove  ${frame_name}.mnu_subsystem_oscope

    if { $test_data($test_num,command,$num) == "PASS/FAIL" } \
    {
      return
    }

    # Initialize the subsystem
    set test_data($test_num,subsystem,$num)  ""

    puts "test_data($test_num,system,$num):  $test_data($test_num,system,$num)"
    
    switch -- $test_data($test_num,system,$num) \
    {
      "Generator"           { Screens::position_widget  ${frame_name}.mnu_subsystem_gen    
                          
                              ${frame_name}.mnu_subsystem_gen  config  \
                                    -textvariable  ::Manual_Test::test_data($test_num,subsystem,$num)
                                          
                              # Delete the existing entries
                              ${frame_name}.mnu_subsystem_gen.menu  delete  0  end

                              # Configure the Sub-system menu
                              foreach { subsystem } $::Manual_Test::setup_menu(subsystem,gen) \
                              {
                                ${frame_name}.mnu_subsystem_gen.menu  add  radio \
                                    -label  $subsystem                           \
                                    -value  $subsystem                           \
                                    -variable  ::Manual_Test::test_data($test_num,subsystem,$num) \
                                    -command   "::Manual_Test::set_setup_subsystem  $tile_path  $num"
                              }

                              ${frame_name}.mnu_subsystem_gen.menu  invoke  0 }

      "Receiver"            { Screens::position_widget  ${frame_name}.mnu_subsystem_rec

                              ${frame_name}.mnu_subsystem_rec  config  \
                                    -textvariable  ::Manual_Test::test_data($test_num,subsystem,$num)
                                         
                              # Delete the existing entries
                              ${frame_name}.mnu_subsystem_rec.menu  delete  0  end

                              # Configure the Sub-system menu
                              foreach { subsystem } $::Manual_Test::setup_menu(subsystem,rec) \
                              {
                                ${frame_name}.mnu_subsystem_rec.menu  add  radio \
                                    -label  $subsystem                           \
                                    -value  $subsystem                           \
                                    -variable  ::Manual_Test::test_data($test_num,subsystem,$num) \
                                    -command   "::Manual_Test::set_setup_subsystem  $tile_path  $num"
                              }

                              ${frame_name}.mnu_subsystem_rec.menu  invoke  0 }

      "Spectrum Analyzer"   { Screens::position_widget  ${frame_name}.mnu_subsystem_anlz

                              ${frame_name}.mnu_subsystem_anlz  config  \
                                    -textvariable  ::Manual_Test::test_data($test_num,subsystem,$num)
                                         
                              # Delete the existing entries
                              ${frame_name}.mnu_subsystem_anlz.menu  delete  0  end

                              # Configure the Sub-system menu
                              foreach { subsystem } $::Manual_Test::setup_menu(subsystem,anlz) \
                              {
                                ${frame_name}.mnu_subsystem_anlz.menu  add  radio \
                                    -label  $subsystem                            \
                                    -value  $subsystem                            \
                                    -variable  ::Manual_Test::test_data($test_num,subsystem,$num) \
                                    -command   "::Manual_Test::set_setup_subsystem  $tile_path  $num"
                              }

                              ${frame_name}.mnu_subsystem_anlz.menu  invoke  0 }

      "Function Generator"  { Screens::position_widget  ${frame_name}.mnu_subsystem_fgen

                              ${frame_name}.mnu_subsystem_fgen  config  \
                                    -textvariable  ::Manual_Test::test_data($test_num,subsystem,$num)
                                         
                              # Delete the existing entries
                              ${frame_name}.mnu_subsystem_fgen.menu  delete  0  end

                              # Configure the Sub-system menu
                              foreach { subsystem } $::Manual_Test::setup_menu(subsystem,fgen) \
                              {
                                ${frame_name}.mnu_subsystem_fgen.menu  add  radio \
                                    -label  $subsystem                            \
                                    -value  $subsystem                            \
                                    -variable  ::Manual_Test::test_data($test_num,subsystem,$num) \
                                    -command   "::Manual_Test::set_setup_subsystem  $tile_path  $num"
                              }

                              ${frame_name}.mnu_subsystem_fgen.menu  invoke  0 }

      "Oscilloscope"        { Screens::position_widget  ${frame_name}.mnu_subsystem_oscope

                              ${frame_name}.mnu_subsystem_oscope  config  \
                                    -textvariable  ::Manual_Test::test_data($test_num,subsystem,$num)
                                         
                              # Delete the existing entries
                              ${frame_name}.mnu_subsystem_oscope.menu  delete  0  end

                              # Configure the Sub-system menu
                              foreach { subsystem } $::Manual_Test::setup_menu(subsystem,oscope) \
                              {
                                ${frame_name}.mnu_subsystem_oscope.menu  add  radio \
                                    -label  $subsystem                              \
                                    -value  $subsystem                              \
                                    -variable  ::Manual_Test::test_data($test_num,subsystem,$num) \
                                    -command   "::Manual_Test::set_setup_subsystem  $tile_path  $num"
                              }

                              ${frame_name}.mnu_subsystem_oscope.menu  invoke  0 }
    }      
    
    return
  }

  
  #***************************************************************************
  #
  # Name: set_setup_subsystem
  #
  # Parameters: widget - tile path of the invoking widget
  #
  # Description: Sets the subsystem for the setup
  #
  #***************************************************************************
  proc set_setup_subsystem { widget  num } \
  {
    puts "set_setup_subsystem:  $widget,  $num"
    
    variable  test_num
    variable  test_data

    variable  setup_menu

    set tile_path  [get_tile  $widget]

    set frame_name  ${tile_path}.frm_test_config.frm_setup${num}

    grid remove  ${frame_name}.ent_value
    grid remove  ${frame_name}.mnu_value
    grid remove  ${frame_name}.mnu_units

    if { $test_data($test_num,command,$num) == "PASS/FAIL" || \
                                $test_data($test_num,command,$num) == "GET" } \
    {
      return
    }
    
    # Initialize the value and units
    set test_data($test_num,value,$num)  ""
    set test_data($test_num,units,$num)  ""

    puts "test_data($test_num,subsystem,$num):  $test_data($test_num,subsystem,$num)"

    if { $test_data($test_num,system,$num) == "Generator" } \
    {
      switch -- $test_data($test_num,subsystem,$num) \
      {
        "Freq"      { Screens::position_widget  ${frame_name}.ent_value
                      Screens::position_widget  ${frame_name}.mnu_units
        
                      ${frame_name}.ent_value  config  \
                            -textvariable  ::Manual_Test::test_data($test_num,value,$num)
                                 
                      ${frame_name}.mnu_units  config  \
                            -textvariable  ::Manual_Test::test_data($test_num,units,$num)
    
                      # Delete the existing entries
                      ${frame_name}.mnu_units.menu  delete  0  end
    
                      # Configure the Sub-system menu
                      foreach { units } $::Manual_Test::setup_menu(gen,freq_units) \
                      {
                        ${frame_name}.mnu_units.menu  add  radio \
                            -label  $units                       \
                            -value  $units                       \
                            -variable  ::Manual_Test::test_data($test_num,units,$num) \
                            -command   "::Manual_Test::set_setup_units  $tile_path  $num"
                      }
    
                      ${frame_name}.mnu_units.menu  invoke  0 }
                      
        "Level"     { Screens::position_widget  ${frame_name}.ent_value
                      Screens::position_widget  ${frame_name}.mnu_units
        
                      ${frame_name}.ent_value  config  \
                            -textvariable  ::Manual_Test::test_data($test_num,value,$num)
                                 
                      ${frame_name}.mnu_units  config  \
                            -textvariable  ::Manual_Test::test_data($test_num,units,$num)
    
                      # Delete the existing entries
                      ${frame_name}.mnu_units.menu  delete  0  end
    
                      # Configure the Sub-system menu
                      foreach { units } $::Manual_Test::setup_menu(gen,level_units) \
                      {
                        ${frame_name}.mnu_units.menu  add  radio \
                            -label  $units                       \
                            -value  $units                       \
                            -variable  ::Manual_Test::test_data($test_num,units,$num) \
                            -command   "::Manual_Test::set_setup_units  $tile_path  $num"
                      }
    
                      ${frame_name}.mnu_units.menu  invoke  0 }
        
        "RF"        { Screens::position_widget  ${frame_name}.mnu_value
        
                      ${frame_name}.mnu_value  config  \
                            -textvariable  ::Manual_Test::test_data($test_num,value,$num)
    
                      # Delete the existing entries
                      ${frame_name}.mnu_value.menu  delete  0  end
    
                      # Configure the Sub-system menu
                      foreach { value } $::Manual_Test::setup_menu(gen,rf_state) \
                      {
                        ${frame_name}.mnu_value.menu  add  radio \
                            -label  $value                       \
                            -value  $value                       \
                            -variable  ::Manual_Test::test_data($test_num,value,$num) \
                            -command   "::Manual_Test::set_setup_value  $tile_path  $num"
                      }
    
                      ${frame_name}.mnu_value.menu  invoke  0 }
        
        "Output"    { Screens::position_widget  ${frame_name}.mnu_value
        
                      ${frame_name}.mnu_value  config  \
                            -textvariable  ::Manual_Test::test_data($test_num,value,$num)
    
                      # Delete the existing entries
                      ${frame_name}.mnu_value.menu  delete  0  end
    
                      # Configure the Sub-system menu
                      foreach { value } $::Manual_Test::setup_menu(gen,output) \
                      {
                        ${frame_name}.mnu_value.menu  add  radio \
                            -label  $value                       \
                            -value  $value                       \
                            -variable  ::Manual_Test::test_data($test_num,value,$num) \
                            -command   "::Manual_Test::set_setup_value  $tile_path  $num"
                      }
    
                      ${frame_name}.mnu_value.menu  invoke  0 }
        
        "Mod Type"  { Screens::position_widget  ${frame_name}.mnu_value
        
                      ${frame_name}.mnu_value  config  \
                            -textvariable  ::Manual_Test::test_data($test_num,value,$num)
    
                      # Delete the existing entries
                      ${frame_name}.mnu_value.menu  delete  0  end
    
                      # Configure the Sub-system menu
                      foreach { value } $::Manual_Test::setup_menu(gen,mod_type) \
                      {
                        ${frame_name}.mnu_value.menu  add  radio \
                            -label  $value                       \
                            -value  $value                       \
                            -variable  ::Manual_Test::test_data($test_num,value,$num) \
                            -command   "::Manual_Test::set_setup_value  $tile_path  $num"
                      }
    
                      ${frame_name}.mnu_value.menu  invoke  0 }
      }
    }
             
    if { $test_data($test_num,system,$num) == "Receiver" } \
    {
      switch -- $test_data($test_num,subsystem,$num) \
      {
        "Freq"      { Screens::position_widget  ${frame_name}.ent_value
                      Screens::position_widget  ${frame_name}.mnu_units
        
                      ${frame_name}.ent_value  config  \
                            -textvariable  ::Manual_Test::test_data($test_num,value,$num)
                                 
                      ${frame_name}.mnu_units  config  \
                            -textvariable  ::Manual_Test::test_data($test_num,units,$num)
    
                      # Delete the existing entries
                      ${frame_name}.mnu_units.menu  delete  0  end
    
                      # Configure the Sub-system menu
                      foreach { units } $::Manual_Test::setup_menu(rec,freq_units) \
                      {
                        ${frame_name}.mnu_units.menu  add  radio \
                            -label  $units                       \
                            -value  $units                       \
                            -variable  ::Manual_Test::test_data($test_num,units,$num) \
                            -command   "::Manual_Test::set_setup_units  $tile_path  $num"
                      }
    
                      ${frame_name}.mnu_units.menu  invoke  0 }
                      
        "Input"     { Screens::position_widget  ${frame_name}.mnu_value
        
                      ${frame_name}.mnu_value  config  \
                            -textvariable  ::Manual_Test::test_data($test_num,value,$num)
    
                      # Delete the existing entries
                      ${frame_name}.mnu_value.menu  delete  0  end
    
                      # Configure the Sub-system menu
                      foreach { value } $::Manual_Test::setup_menu(rec,input) \
                      {
                        ${frame_name}.mnu_value.menu  add  radio \
                            -label  $value                       \
                            -value  $value                       \
                            -variable  ::Manual_Test::test_data($test_num,value,$num) \
                            -command   "::Manual_Test::set_setup_value  $tile_path  $num"
                      }
    
                      ${frame_name}.mnu_value.menu  invoke  0 }
        
        "Atten"     { Screens::position_widget  ${frame_name}.mnu_value
        
                      ${frame_name}.mnu_value  config  \
                            -textvariable  ::Manual_Test::test_data($test_num,value,$num)
    
                      # Delete the existing entries
                      ${frame_name}.mnu_value.menu  delete  0  end
    
                      # Configure the Sub-system menu
                      foreach { value } $::Manual_Test::setup_menu(rec,atten) \
                      {
                        ${frame_name}.mnu_value.menu  add  radio \
                            -label  $value                       \
                            -value  $value                       \
                            -variable  ::Manual_Test::test_data($test_num,value,$num) \
                            -command   "::Manual_Test::set_setup_value  $tile_path  $num"
                      }
    
                      ${frame_name}.mnu_value.menu  invoke  0 }
        
        "Demod"     { Screens::position_widget  ${frame_name}.mnu_value
        
                      ${frame_name}.mnu_value  config  \
                            -textvariable  ::Manual_Test::test_data($test_num,value,$num)
    
                      # Delete the existing entries
                      ${frame_name}.mnu_value.menu  delete  0  end
    
                      # Configure the Sub-system menu
                      foreach { value } $::Manual_Test::setup_menu(rec,demod) \
                      {
                        ${frame_name}.mnu_value.menu  add  radio \
                            -label  $value                       \
                            -value  $value                       \
                            -variable  ::Manual_Test::test_data($test_num,value,$num) \
                            -command   "::Manual_Test::set_setup_value  $tile_path  $num"
                      }
    
                      ${frame_name}.mnu_value.menu  invoke  0 }
    
        "IF BW"     { Screens::position_widget  ${frame_name}.mnu_value
        
                      ${frame_name}.mnu_value  config  \
                            -textvariable  ::Manual_Test::test_data($test_num,value,$num)
    
                      # Delete the existing entries
                      ${frame_name}.mnu_value.menu  delete  0  end
    
                      # Configure the Sub-system menu
                      foreach { value } $::Manual_Test::setup_menu(rec,if_bw) \
                      {
                        ${frame_name}.mnu_value.menu  add  radio \
                            -label  $value                       \
                            -value  $value                       \
                            -variable  ::Manual_Test::test_data($test_num,value,$num) \
                            -command   "::Manual_Test::set_setup_value  $tile_path  $num"
                      }
    
                      ${frame_name}.mnu_value.menu  invoke  0 }
    
        "Filter"    { Screens::position_widget  ${frame_name}.mnu_value
        
                      ${frame_name}.mnu_value  config  \
                            -textvariable  ::Manual_Test::test_data($test_num,value,$num)
    
                      # Delete the existing entries
                      ${frame_name}.mnu_value.menu  delete  0  end
    
                      # Configure the Sub-system menu
                      foreach { value } $::Manual_Test::setup_menu(rec,filter) \
                      {
                        ${frame_name}.mnu_value.menu  add  radio \
                            -label  $value                       \
                            -value  $value                       \
                            -variable  ::Manual_Test::test_data($test_num,value,$num) \
                            -command   "::Manual_Test::set_setup_value  $tile_path  $num"
                      }
    
                      ${frame_name}.mnu_value.menu  invoke  0 }
      }
    }
    
    if { $test_data($test_num,system,$num) == "Spectrum Analyzer" } \
    {
      switch -- $test_data($test_num,subsystem,$num) \
      {
        "Freq"   { Screens::position_widget  ${frame_name}.ent_value
                   Screens::position_widget  ${frame_name}.mnu_units
        
                   ${frame_name}.ent_value  config  \
                         -textvariable  ::Manual_Test::test_data($test_num,value,$num)
                              
                   ${frame_name}.mnu_units  config  \
                         -textvariable  ::Manual_Test::test_data($test_num,units,$num)
    
                   # Delete the existing entries
                   ${frame_name}.mnu_units.menu  delete  0  end
    
                   # Configure the Sub-system menu
                   foreach { units } $::Manual_Test::setup_menu(anlz,freq_units) \
                   {
                     ${frame_name}.mnu_units.menu  add  radio \
                         -label  $units                       \
                         -value  $units                       \
                         -variable  ::Manual_Test::test_data($test_num,units,$num) \
                         -command   "::Manual_Test::set_setup_units  $tile_path  $num"
                   }
    
                   ${frame_name}.mnu_units.menu  invoke  0 }
                   
        "Span"   { Screens::position_widget  ${frame_name}.mnu_value
        
                   ${frame_name}.mnu_value  config  \
                         -textvariable  ::Manual_Test::test_data($test_num,value,$num)
    
                   # Delete the existing entries
                   ${frame_name}.mnu_value.menu  delete  0  end
    
                   # Configure the Sub-system menu
                   foreach { value } $::Manual_Test::setup_menu(anlz,span) \
                   {
                     ${frame_name}.mnu_value.menu  add  radio \
                         -label  $value                       \
                         -value  $value                       \
                         -variable  ::Manual_Test::test_data($test_num,value,$num) \
                         -command   "::Manual_Test::set_setup_value  $tile_path  $num"
                   }
    
                   ${frame_name}.mnu_value.menu  invoke  0 }

        "RBW"    { Screens::position_widget  ${frame_name}.mnu_value
        
                   ${frame_name}.mnu_value  config  \
                         -textvariable  ::Manual_Test::test_data($test_num,value,$num)
    
                   # Delete the existing entries
                   ${frame_name}.mnu_value.menu  delete  0  end
    
                   # Configure the Sub-system menu
                   foreach { value } $::Manual_Test::setup_menu(anlz,rbw) \
                   {
                     ${frame_name}.mnu_value.menu  add  radio \
                         -label  $value                       \
                         -value  $value                       \
                         -variable  ::Manual_Test::test_data($test_num,value,$num) \
                         -command   "::Manual_Test::set_setup_value  $tile_path  $num"
                   }
    
                   ${frame_name}.mnu_value.menu  invoke  0 }

        "VBW"    { Screens::position_widget  ${frame_name}.mnu_value
        
                   ${frame_name}.mnu_value  config  \
                         -textvariable  ::Manual_Test::test_data($test_num,value,$num)
    
                   # Delete the existing entries
                   ${frame_name}.mnu_value.menu  delete  0  end
    
                   # Configure the Sub-system menu
                   foreach { value } $::Manual_Test::setup_menu(anlz,vbw) \
                   {
                     ${frame_name}.mnu_value.menu  add  radio \
                         -label  $value                       \
                         -value  $value                       \
                         -variable  ::Manual_Test::test_data($test_num,value,$num) \
                         -command   "::Manual_Test::set_setup_value  $tile_path  $num"
                   }
    
                   ${frame_name}.mnu_value.menu  invoke  0 }

        "Sweep"  { Screens::position_widget  ${frame_name}.mnu_value
        
                   ${frame_name}.mnu_value  config  \
                         -textvariable  ::Manual_Test::test_data($test_num,value,$num)
    
                   # Delete the existing entries
                   ${frame_name}.mnu_value.menu  delete  0  end
    
                   # Configure the Sub-system menu
                   foreach { value } $::Manual_Test::setup_menu(anlz,sweep) \
                   {
                     ${frame_name}.mnu_value.menu  add  radio \
                         -label  $value                       \
                         -value  $value                       \
                         -variable  ::Manual_Test::test_data($test_num,value,$num) \
                         -command   "::Manual_Test::set_setup_value  $tile_path  $num"
                   }
    
                   ${frame_name}.mnu_value.menu  invoke  0 }

        "TOS"    { Screens::position_widget  ${frame_name}.mnu_value
        
                   ${frame_name}.mnu_value  config  \
                         -textvariable  ::Manual_Test::test_data($test_num,value,$num)
    
                   # Delete the existing entries
                   ${frame_name}.mnu_value.menu  delete  0  end
    
                   # Configure the Sub-system menu
                   foreach { value } $::Manual_Test::setup_menu(anlz,tos) \
                   {
                     ${frame_name}.mnu_value.menu  add  radio \
                         -label  $value                       \
                         -value  $value                       \
                         -variable  ::Manual_Test::test_data($test_num,value,$num) \
                         -command   "::Manual_Test::set_setup_value  $tile_path  $num"
                   }
    
                   ${frame_name}.mnu_value.menu  invoke  0 }

        "Scale"  { Screens::position_widget  ${frame_name}.mnu_value
        
                   ${frame_name}.mnu_value  config  \
                         -textvariable  ::Manual_Test::test_data($test_num,value,$num)
    
                   # Delete the existing entries
                   ${frame_name}.mnu_value.menu  delete  0  end
    
                   # Configure the Sub-system menu
                   foreach { value } $::Manual_Test::setup_menu(anlz,scale) \
                   {
                     ${frame_name}.mnu_value.menu  add  radio \
                         -label  $value                       \
                         -value  $value                       \
                         -variable  ::Manual_Test::test_data($test_num,value,$num) \
                         -command   "::Manual_Test::set_setup_value  $tile_path  $num"
                   }
    
                   ${frame_name}.mnu_value.menu  invoke  0 }
      }
    }
    
    if { $test_data($test_num,system,$num) == "Function Generator" } \
    {
      #switch -- $test_data($test_num,subsystem,$num) \
      #{
      #
      #}
    }

    if { $test_data($test_num,system,$num) == "Oscilloscope" } \
    {
      switch -- $test_data($test_num,subsystem,$num) \
      {
        "Source"        { Screens::position_widget  ${frame_name}.mnu_value
                       
                          ${frame_name}.mnu_value  config  \
                                -textvariable  ::Manual_Test::test_data($test_num,value,$num)
                       
                          # Delete the existing entries
                          ${frame_name}.mnu_value.menu  delete  0  end
                       
                          # Configure the Sub-system menu
                          foreach { value } $::Manual_Test::setup_menu(oscope,source) \
                          {
                            ${frame_name}.mnu_value.menu  add  radio \
                                -label  $value                       \
                                -value  $value                       \
                                -variable  ::Manual_Test::test_data($test_num,value,$num) \
                                -command   "::Manual_Test::set_setup_value  $tile_path  $num"
                          }
                       
                          ${frame_name}.mnu_value.menu  invoke  0 }
                       
        "Sweep"         { Screens::position_widget  ${frame_name}.mnu_value
                       
                          ${frame_name}.mnu_value  config  \
                                -textvariable  ::Manual_Test::test_data($test_num,value,$num)
                       
                          # Delete the existing entries
                          ${frame_name}.mnu_value.menu  delete  0  end
                       
                          # Configure the Sub-system menu
                          foreach { value } $::Manual_Test::setup_menu(oscope,sweep) \
                          {
                            ${frame_name}.mnu_value.menu  add  radio \
                                -label  $value                       \
                                -value  $value                       \
                                -variable  ::Manual_Test::test_data($test_num,value,$num) \
                                -command   "::Manual_Test::set_setup_value  $tile_path  $num"
                          }
                       
                          ${frame_name}.mnu_value.menu  invoke  0 }
                       
        "Scale"         { Screens::position_widget  ${frame_name}.mnu_value
                       
                          ${frame_name}.mnu_value  config  \
                                -textvariable  ::Manual_Test::test_data($test_num,value,$num)
                       
                          # Delete the existing entries
                          ${frame_name}.mnu_value.menu  delete  0  end
                       
                          # Configure the Sub-system menu
                          foreach { value } $::Manual_Test::setup_menu(oscope,scale) \
                          {
                            ${frame_name}.mnu_value.menu  add  radio \
                                -label  $value                       \
                                -value  $value                       \
                                -variable  ::Manual_Test::test_data($test_num,value,$num) \
                                -command   "::Manual_Test::set_setup_value  $tile_path  $num"
                          }
                       
                          ${frame_name}.mnu_value.menu  invoke  0 }
                       
        "HPos"          { Screens::position_widget  ${frame_name}.ent_value
                          Screens::position_widget  ${frame_name}.mnu_units
                       
                          ${frame_name}.ent_value  config  \
                                -textvariable  ::Manual_Test::test_data($test_num,value,$num)
                                     
                          ${frame_name}.mnu_units  config  \
                                -textvariable  ::Manual_Test::test_data($test_num,units,$num)
                       
                          # Delete the existing entries
                          ${frame_name}.mnu_units.menu  delete  0  end
                       
                          # Configure the Sub-system menu
                          foreach { units } $::Manual_Test::setup_menu(oscope,hpos_units) \
                          {
                            ${frame_name}.mnu_units.menu  add  radio \
                                -label  $units                       \
                                -value  $units                       \
                                -variable  ::Manual_Test::test_data($test_num,units,$num) \
                                -command   "::Manual_Test::set_setup_units  $tile_path  $num"
                          }
                       
                          ${frame_name}.mnu_units.menu  invoke  0 }
                       
        "Trig Type"     { Screens::position_widget  ${frame_name}.mnu_value
                       
                          ${frame_name}.mnu_value  config  \
                                -textvariable  ::Manual_Test::test_data($test_num,value,$num)
                       
                          # Delete the existing entries
                          ${frame_name}.mnu_value.menu  delete  0  end
                       
                          # Configure the Sub-system menu
                          foreach { value } $::Manual_Test::setup_menu(oscope,trig_type) \
                          {
                            ${frame_name}.mnu_value.menu  add  radio \
                                -label  $value                       \
                                -value  $value                       \
                                -variable  ::Manual_Test::test_data($test_num,value,$num) \
                                -command   "::Manual_Test::set_setup_value  $tile_path  $num"
                          }
                       
                          ${frame_name}.mnu_value.menu  invoke  0 }
                       
        "Trig Source"   { Screens::position_widget  ${frame_name}.mnu_value
                       
                          ${frame_name}.mnu_value  config  \
                                -textvariable  ::Manual_Test::test_data($test_num,value,$num)
                       
                          # Delete the existing entries
                          ${frame_name}.mnu_value.menu  delete  0  end
                       
                          # Configure the Sub-system menu
                          foreach { value } $::Manual_Test::setup_menu(oscope,trig_source) \
                          {
                            ${frame_name}.mnu_value.menu  add  radio \
                                -label  $value                       \
                                -value  $value                       \
                                -variable  ::Manual_Test::test_data($test_num,value,$num) \
                                -command   "::Manual_Test::set_setup_value  $tile_path  $num"
                          }
                       
                          ${frame_name}.mnu_value.menu  invoke  0 }
                       
        "Trig Level"    { Screens::position_widget  ${frame_name}.ent_value
                          Screens::position_widget  ${frame_name}.mnu_units
                       
                          ${frame_name}.ent_value  config  \
                                -textvariable  ::Manual_Test::test_data($test_num,value,$num)
                                     
                          ${frame_name}.mnu_units  config  \
                                -textvariable  ::Manual_Test::test_data($test_num,units,$num)
                       
                          # Delete the existing entries
                          ${frame_name}.mnu_units.menu  delete  0  end
                       
                          # Configure the Sub-system menu
                          foreach { units } $::Manual_Test::setup_menu(oscope,level_units) \
                          {
                            ${frame_name}.mnu_units.menu  add  radio \
                                -label  $units                       \
                                -value  $units                       \
                                -variable  ::Manual_Test::test_data($test_num,units,$num) \
                                -command   "::Manual_Test::set_setup_units  $tile_path  $num"
                          }
                       
                          ${frame_name}.mnu_units.menu  invoke  0 }
                       
        "Trig Slope"    { Screens::position_widget  ${frame_name}.mnu_value
                       
                          ${frame_name}.mnu_value  config  \
                                -textvariable  ::Manual_Test::test_data($test_num,value,$num)
                       
                          # Delete the existing entries
                          ${frame_name}.mnu_value.menu  delete  0  end
                       
                          # Configure the Sub-system menu
                          foreach { value } $::Manual_Test::setup_menu(oscope,trig_slope) \
                          {
                            ${frame_name}.mnu_value.menu  add  radio \
                                -label  $value                       \
                                -value  $value                       \
                                -variable  ::Manual_Test::test_data($test_num,value,$num) \
                                -command   "::Manual_Test::set_setup_value  $tile_path  $num"
                          }
                       
                          ${frame_name}.mnu_value.menu  invoke  0 }
                       
        "Coupling"      { Screens::position_widget  ${frame_name}.mnu_value
                       
                          ${frame_name}.mnu_value  config  \
                                -textvariable  ::Manual_Test::test_data($test_num,value,$num)
                       
                          # Delete the existing entries
                          ${frame_name}.mnu_value.menu  delete  0  end
                       
                          # Configure the Sub-system menu
                          foreach { value } $::Manual_Test::setup_menu(oscope,coupling) \
                          {
                            ${frame_name}.mnu_value.menu  add  radio \
                                -label  $value                       \
                                -value  $value                       \
                                -variable  ::Manual_Test::test_data($test_num,value,$num) \
                                -command   "::Manual_Test::set_setup_value  $tile_path  $num"
                          }
                       
                          ${frame_name}.mnu_value.menu  invoke  0 }
      }
    }
    
    return
  }

  
  #***************************************************************************
  #
  # Name: set_setup_value
  #
  # Parameters: widget - tile path of the invoking widget
  #
  # Description: Sets the value for the setup
  #
  #***************************************************************************
  proc set_setup_value { widget  num } \
  {
    puts "set_setup_value:  $widget,  $num"
  
    variable  test_num
    variable  test_data

    set tile_path  [get_tile  $widget]

    set frame_name  ${tile_path}.frm_test_config.frm_setup${num}

    if { $test_data($test_num,command,$num) == "PASS/FAIL" || \
                                $test_data($test_num,command,$num) == "GET" } \
    {
      return
    }

    puts "test_data($test_num,value,$num):  $test_data($test_num,value,$num)"
    
    #switch -- $test_data(1,value,1) \
    #{
    #
    #}

    return
  }


  #***************************************************************************
  #
  # Name: set_setup_units
  #
  # Parameters: widget - tile path of the invoking widget
  #
  # Description: Sets the units for the setup
  #
  #***************************************************************************
  proc set_setup_units { widget  num } \
  {
    puts "set_setup_units:  $widget,  $num"
  
    variable  test_num
    variable  test_data

    set tile_path  [get_tile  $widget]

    set frame_name  ${tile_path}.frm_test_config.frm_setup${num}

    if { $test_data($test_num,command,$num) == "PASS/FAIL" || \
                                $test_data($test_num,command,$num) == "GET" } \
    {
      return
    }

    puts "test_data($test_num,units,$num):  $test_data($test_num,units,$num)"
    
    #switch -- $test_data(1,units,1) \
    #{
    #
    #}
  
    return
  }



  #***************************************************************************
  #
  # Test Results procedures
  #
  #***************************************************************************
  #***************************************************************************
  #
  # Name: test_results_display
  #
  # Parameters: widget - invoking widget tile path
  #
  # Description: Displays the Test Results tile
  #
  #***************************************************************************
  proc test_results_display { widget } \
  {
    #puts "test_results_display:  $widget"

    set tile_path  [get_tile  $widget]

    # Display the tile
    grid remove ${tile_path}.frm_test_info
    grid remove ${tile_path}.frm_test_select
    grid remove ${tile_path}.frm_test_config

    Screens::position_widget  ${tile_path}.frm_test_results
    

    ${tile_path}.frm_test_results.scrl_results config  \
                    -command "${tile_path}.frm_test_results.txt_results yview"
    
    ${tile_path}.frm_test_results.txt_results config  \
              -yscrollcommand "${tile_path}.frm_test_results.scrl_results set"

    
    # Set the softkeys
    # Softkey #2:  Start
    ::View::AssignSoftkey $tile_path  2  "START"  \
                                       "::Manual_Test::test_start  $tile_path"

    # Softkey #3:  Abort
    ::View::AssignSoftkey $tile_path  3  "ABORT"  \
                                       "::Manual_Test::test_abort  $tile_path"

    # Softkey #4:  Save
    ::View::AssignSoftkey $tile_path  4  "SAVE"  \
                                "::Manual_Test::test_results_save  $tile_path"

    # Softkey #5:  Recall
    ::View::AssignSoftkey $tile_path  5  "RECALL"  \
                              "::Manual_Test::test_results_recall  $tile_path"

    # Softkey #6:  Clear
    ::View::AssignSoftkey $tile_path  6  "CLEAR"  \
                               "::Manual_Test::test_results_clear  $tile_path"

    # Softkey #7:  Print
    ::View::AssignSoftkey $tile_path  7  "PRINT"  \
                               "::Manual_Test::test_results_print  $tile_path"

    # Softkey #8:  RETURN
    ::View::AssignReturn $tile_path \
                              "::Manual_Test::test_select_display  $tile_path"

    return
  }


  #***************************************************************************
  #
  # Name: test_start
  #
  # Parameters: widget - invoking widget tile path
  #
  # Description: Starts the tests
  #
  #***************************************************************************
  proc test_start { widget } \
  {
    #puts "test_start:  $widget"

    variable  results

    variable  abort_flag

    variable  test_data

    variable  test_date
    variable  test_time
    variable  test_unit_id
    variable  test_tech_name
    variable  test_name
    variable  test_label
    
    variable  test_start_time
    variable  test_stop_time
    variable  test_total_time

    set tile_path  [get_tile  $widget]

    set abort_flag  0
      
    # Get the reference for the time of the test
    set test_start_time  [expr int([clock seconds])]
    
    # Display the test info in the results box
    set results  ""
    set results  ""
    set results  "Date:        $test_date,  $test_time"
    set results  "ID Number:   $test_unit_id"
    set results  "Tech Name:   $test_tech_name"
    set results  "Radio Type:  $test_name"
    set results  "Test Type:   $test_label"
    set results  ""
    

    set test_num  1
    
    # Loop as long as a valid test title is specified
    while { [info exists  test_data($test_num,test_title)] } \
    {
      #puts "test_num:  $test_num"
    
      if { $abort_flag == 1 } \
      {
        break
      }
      
      if { $test_enable($test_num) == 1 } \
      {
        set test_name  test_${test_num}
        
        puts "test_name:  $test_name"
        
        #eval "$test_name  $tile_path"
      }
      
      incr test_num  1
    }


    if { $abort_flag == 0 } \
    {
      set results  ""
      set results  "-------> TEST COMPLETED <-------"
      set results  ""
    } \
    else \
    {
      set results  ""
      set results  "******** TEST ABORTED ********"
      set results  ""
    }

    # Determine the test time
    set test_stop_time  [expr int([clock seconds])]

    set test_total_time  [expr $test_stop_time - $test_start_time]

    set test_total_time_min  [expr $test_total_time / 60]

    set test_total_time_sec  [expr $test_total_time - ($test_total_time_min * 60)]

    set results  "Test Time:  $test_total_time_min  min   $test_total_time_sec  sec"

    return
  }


  #***************************************************************************
  #
  # Name: test_abort
  #
  # Parameters: widget - invoking widget tile path
  #
  # Description: Aborts the tests
  #
  #***************************************************************************
  proc test_abort { widget } \
  {
    #puts "test_abort:  $widget"

    variable  abort_flag

    set tile_path  [get_tile  $widget]

    set abort_flag  1
    
    return
  }


  #***************************************************************************
  #
  # Name: test_results_save
  #
  # Parameters: widget - invoking widget tile path
  #
  # Description: Saves the results
  #
  #***************************************************************************
  proc test_results_save { widget } \
  {
    #puts "test_results_save:  $widget"

    variable  file_ext
    variable  file_label
    variable  file_prefix
    variable  working_dir
    
    variable  save_flag

    set tile_path  [get_tile  $widget]
    
    set text_box  ${tile_path}.frm_test_results.txt_results
    
    if { [winfo exists $text_box] } \
    {
      $text_box config -state normal
    
      set file_buffer  [$text_box get  1.0  end]
    }
    
    # If no data exists, return
    if { $file_buffer == "" } \
    {
      return
    }
    
    # Produce a comma delimited version of the data
    regsub -all ":" $file_buffer "," file_buffer_csv

    set save_flag  [File_SaveUI::display_file_save  $tile_path        \
                                                    $working_dir      \
                                                    $file_buffer      \
                                                    $file_prefix      \
                                                    "$file_label"     \
                                                    $file_ext         \
                                                    NO_ENCRYPT        \
                                                    $file_buffer_csv  \
                                                    NO_MODES          \
                                                    NO_OPT_EXT]
    
    #puts "save_flag:  $save_flag"
                                                               
    # Get the file label after the file has been saved
    set file_label  $::File_SaveUI::file_label

    return
  }


  #***************************************************************************
  #
  # Name: test_results_recall
  #
  # Parameters: widget - invoking widget tile path
  #
  # Description: Recalls results saved to a file
  #
  #***************************************************************************
  proc test_results_recall { widget } \
  {
    #puts "test_results_recall:  $widget"
    
    variable  file_ext
    variable  file_prefix
    variable  working_dir
    
    variable  results

    set tile_path  [get_tile  $widget]
    
    set file_buffer  [::File_RecallUI::display_file_recall  $tile_path    \
                                                            $working_dir  \
                                                            $file_prefix  \
                                                            $file_ext     ]
                                                               
    #puts "file_buffer:  $file_buffer"

    # If the recalled results are valid, display them
    if { $file_buffer != "" } \
    {
      set results  "$file_buffer"
    }
    
    return
  }


  #***************************************************************************
  #
  # Name: test_results_clear
  #
  # Parameters: widget - invoking widget tile path
  #
  # Description: Clears the results textbox
  #
  #***************************************************************************
  proc test_results_clear { widget  args } \
  {
    #puts "test_results_clear:  $widget,  $args"
    
    variable  save_flag

    set tile_path  [get_tile  $widget]

    # Prompt the operator if the present data is not saved
    if { $save_flag == 0 } \
    {
      set title  "Warning"
      set prompt  "The results data has not been saved,\n\nSave them now?"
      
      set window_name  ${tile_path}.warning
      
      set result  [tk_dialog  $window_name  $title  $prompt  info  0  "YES"  "NO"]
    
      switch -- $result \
      {
        0 { set save_flag  -1;  test_results_save  $tile_path }
      
        1 { set save_flag  1 }
      }
    }    

    # Wait for the 'save_flag' to become valid
    while { $save_flag == -1 } \
    {
      vwait Manual_Test::save_flag
    }
    
    # Retest the 'save_flag' to verify
    if { $save_flag == 1 } \
    {
      set text_box  ${tile_path}.frm_test_results.txt_results
    
      if { [winfo exists $text_box] } \
      {
        $text_box config -state normal

        $text_box delete  1.0  end

        $text_box config -state disabled
      }
    }
    
    return
  }


  #***************************************************************************
  #
  # Name: test_results_print
  #
  # Parameters: widget - invoking widget tile path
  #
  # Description: Prints the results of the test
  #
  #***************************************************************************
  proc test_results_print { widget } \
  {
    puts "test_results_print:  $widget"

    variable  results
    
    variable  printer_dir
    
    set tile_path  [get_tile  $widget]
    
    set printer_path  [file join  /  $printer_dir]
    
    set text_box  ${tile_path}.frm_test_results.txt_results
    
    $text_box config -state normal
    
    set last_index  [$text_box  index  end]
    
    set last_index  [expr int($last_index)]
    
    if { [winfo exists $text_box] } \
    {
      # Open the printer
      set error  [catch { set output_id  [open  $printer_path  w] } result]
      
      puts "error:  $error"
      
      if { $error != 0 } \
      {
        # Error opening the printer
        return
      }
      
      # Set the buffering to 'none'
      fconfigure  $output_id  -buffering  none
      
      for { set index 1 } { $index <= $last_index } { incr index 1 } \
      {
        set print_buffer  [$text_box  get  ${index}.0  ${index}.end]
        
        puts "${index}: $print_buffer"

        # Write the data to the file (printer)
        catch { puts  -nonewline  $output_id  "$print_buffer \r" } result
    
        ## Flush the printer queue
        #catch { flush  $output_id } result
      }

      # Write a 'form feed' to the file (printer)
      catch { puts  -nonewline  $output_id  "\f" } result

      ## Flush the printer queue
      #catch { flush  $output_id } result

      # Close the printer
      catch { close  $output_id } result

      $text_box config -state disabled
    }

    return
  }


  #***************************************************************************
  #
  # Name: test_results_display
  #
  # Parameters: widget - invoking widget tile path
  #
  # Description: Prints the results to the RESULTS text box
  #
  #***************************************************************************
  proc test_results_display { widget  args } \
  {
    #puts "test_results_display:  $widget,  $args"
    
    variable  results
    variable  save_flag
    
    set tile_path  [get_tile  $widget]
    
    set text_box  ${tile_path}.frm_test_results.txt_results
    
    if { [winfo exists $text_box] } \
    {
      $text_box config -state normal

      $text_box insert end "$results\n"

      $text_box see end

      $text_box config -state disabled
      
      set save_flag  0
    }
    
    return
  }


  #****************************************************************************
  #
  # Name:        getSaveState
  #
  # Parameters:  none
  #
  # Description: This function returns a list of TCL commands which can be used
  #              to setup the Manual Test screen to its current state
  #
  #****************************************************************************
  proc getSaveState { } \
  {
    #puts "::Manual_Test::getSaveState"
    
    variable  test_name_menu

    set returnList \
    {
      set ::Manual_Test::test_name_menu  {$test_name_menu}
    }


    return [subst $returnList]
  }


  #****************************************************************************
  # Functions executed when the namespace is sourced
  #****************************************************************************
  # If the 'Manual Test' option is installed, register the screen
  if { [IFR_options  is_installed  Manual_Test] } \
  {
    #::View::RegisterScreen  Manual_Test;  # Add this on the menu
  }

};  # End of the Manual_Test


#****************************************************************************
# This registers the screens with the list for saving instrument states
#****************************************************************************
namespace eval ::SavedStates \
{
  variable screens   

  lappend screens  Manual_Test
}




