#***************************************************************************
#
# Test procedure:  Test #1:  Analog Receiver Test
#
#***************************************************************************


namespace eval Test_Analog_Rx \
{
  variable  short_name  "analog_rx"
  
  variable  long_name   "Analog Receiver Test"
  
  variable  tile_class  "TEST_ANALOG_RX"
  
  variable  test_data
  
  set test_data(chan_list)  {}
  
  set test_data(channel)    1

  set rflevel  -130

  namespace import ::Auto_Test_2::*
  
  #***************************************************************************
  #
  #        Name: Create
  #
  #  Parameters: parent - parent of child window
  #
  # Description: Creates the tile
  #
  #***************************************************************************
  proc Create { parent } \
  {
    #puts "Test_Analog_Rx::Create:  $parent"

    variable  short_name
    variable  tile_class

    set tile_path  [name_child  $parent  $short_name]
    
    if { ![winfo exists $tile_path] } \
    {
      set tile_path  [Screens::create_tile  $tile_path  $short_name  $tile_class]
    }  
     
    return  $tile_path
  }


  #***************************************************************************
  #
  #        Name: Map
  #
  #  Parameters: 
  #
  # Description: Initializes the tile
  #
  #***************************************************************************
  proc Map { tile_path } \
  {
    #puts "::Test_Analog_Rx::Map:  $tile_path"
    
    #set tile_path  [get_tile  $tile_path]
    
    # Set the configuration data
    set_channel_data  $tile_path
    
    # Set the Mode
    set_channel_mode  $tile_path
    
    # Set the Subaudio units
    set_subaudio_units  $tile_path
    
    # Set the 20 dB Quieting Test ENABLE / DISABLE
    set_quiet_enable  $tile_path

    return
  }


  #***************************************************************************
  #
  #        Name: Cleanup
  #
  #  Parameters: 
  #
  # Description: 
  #
  #***************************************************************************
  proc Cleanup { tile_path } \
  {
    #puts "::Test_Analog_Rx::Cleanup:  $tile_path"

    #set tile_path  [get_tile  $tile_path]
    
    return
  }


  #***************************************************************************
  #
  #        Name: Unmap
  #
  #  Parameters: 
  #
  # Description: 
  #
  #***************************************************************************
  proc Unmap { tile_path } \
  {
    #puts "::Test_Analog_Rx::Unmap:  $tile_path"

    #set tile_path  [get_tile  $tile_path]

    Cleanup  $tile_path
    
    return
  }


  #***************************************************************************
  #
  #        Name: Destroy
  #
  #  Parameters: 
  #
  # Description: 
  #
  #***************************************************************************
  proc Destroy { tile_path } \
  {
    #puts "::Test_Analog_Rx::Destroy:  $tile_path"

    #set tile_path  [get_tile  $tile_path]

    Cleanup  $tile_path

    return
  }

  #***************************************************************************
  #  Name: default_all_channel_data
  #  Parameters:   None
  #  Description:  This function does a validation on the test_data array to
  #     insure that array data exists for all defined channel values
  #***************************************************************************
  proc default_all_channel_data { } \
  {
     #puts "default_all_channel_data"
     variable  test_data

     foreach { chan } $test_data(chan_list) \
     {
        set_default_channel_data $chan
     }
  }
  #***************************************************************************
  #  Name: set_default_channel_data
  #  Parameters:   chan         Channel number to set up
  #  Description:  This function does a validation on the test_data array to
  #     insure that array data exists for the requested channel value
  #***************************************************************************
  proc set_default_channel_data { chan } \
  {
     #puts "default_channel_data"
     variable  test_data

     # Setup a default frequency
     if { ![info exists test_data($chan,freq)] } \
     {
        set test_data($chan,freq)  "0.000000"
     }

     # Setup a default channel mode
     if { ![info exists test_data($chan,mode)] } \
     {
        set test_data($chan,mode)  "DUPLEX"
     }

     # Setup a default Volume Output
     if { ![info exists test_data($chan,vol_output)] } \
     {
        set test_data($chan,vol_output)  2.00
     }

     # Setup a default Audio deviation level
     if { ![info exists test_data($chan,audio_dev)] } \
     {
        set test_data($chan,audio_dev)  "3.0"
     }

     # Setup a default Subaudio squelch type
     if { ![info exists test_data($chan,subaudio_type)] } \
     {
        set test_data($chan,subaudio_type)  "NONE"
     }

     # Setup a default DCS squelch code
     if { ![info exists test_data($chan,dcs_code)] } \
     {
        set test_data($chan,dcs_code)  "023"
     }

     # Setup a default CTCSS squelch code
     if { ![info exists test_data($chan,ctcss_code)] } \
     {
        set test_data($chan,ctcss_code)  "XZ"
     }
 
     # Setup a default CTCSS code / frequency string
     if { ![info exists test_data($chan,ctcss_code_and_freq)] } \
     {
        set test_data($chan,ctcss_code_and_freq)  "XZ - 67.0"
     }

     # Setup a default Subaudio units label
     if { ![info exists test_data($chan,subaudio_units)] } \
     {
        set test_data($chan,subaudio_units)  ""
     }

     # Setup a default Subaudio Deviation level
     if { ![info exists test_data($chan,subaudio_dev)] } \
     {
        set test_data($chan,subaudio_dev)  "0.00"
     }

     # Default to enable the 20dB Quieting Test
     if { ![info exists test_data($chan,quiet_enable)] } \
     {
        set test_data($chan,quiet_enable)  "ENABLED"
     }

     # Setup a default DVM Load value
     if { ![info exists test_data($chan,dvm_load)] } \
     {
        set test_data($chan,dvm_load)  "8.00"
     }

     # Setup a default max Distortion value
     if { ![info exists test_data($chan,dist_max)] } \
     {
        set test_data($chan,dist_max)  "0.00"
     }

     # Setup a default max SINAD RF level
     if { ![info exists test_data($chan,sinad_rf_lvl_max)] } \
     {
        set test_data($chan,sinad_rf_lvl_max)  "0.00"
     }

     # Setup a default min SINAD value
     if { ![info exists test_data($chan,sinad_min)] } \
     {
        set test_data($chan,sinad_min)  "0.00"
     }

    # Configure the 20 dB Quieting max entry
      if { ![info exists test_data($chan,quiet_max)] } \
      {
        set test_data($chan,quiet_max)  "0.00"
      }

     # Configure the 20 dB Quieting max entry
     if { ![info exists test_data($chan,quiet_max)] } \
     {
        set test_data($chan,quiet_max)  "0.00"
     }
  }
  #***************************************************************************
  #
  #        Name: set_channel_data
  #
  #  Parameters: tile_path  - tile path of the invoking widget
  #
  # Description: Sets the text variables of the configuration display 
  #              depending on the channel number
  #
  #***************************************************************************
  proc set_channel_data { tile_path } \
  {
     #puts "set_channel_data:  $tile_path"
  
     variable  test_data

     set tile_path  [get_tile  $tile_path]
    
     set tile_path  ${tile_path}.frm_config.analog_rx

     set chan  $test_data(channel)
    
     # Determine if the present channel number is in the channel list.
     # If it is not, append the new value to the list, initialize the array 
     # variables, and then sort the channel list in increasing order.
     if { [lsearch $test_data(chan_list) $chan] == -1 } \
     {
        lappend test_data(chan_list)  $chan
        set_default_channel_data $chan
        if [oldCatch { set tempList  [lsort -increasing -integer $test_data(chan_list)]} err] \
        {
           set tempList  [lsort -increasing $test_data(chan_list)]
        }
        set test_data(chan_list) $tempList
      }
      
      # Configure the Frequency entry
      ${tile_path}.ent_freq  config  \
                         -textvariable ::Test_Analog_Rx::test_data($chan,freq)

      # Configure the Mode button
      ${tile_path}.btn_mode  config  \
                         -textvariable ::Test_Analog_Rx::test_data($chan,mode)

      # Configure the Volume Output entry
      ${tile_path}.ent_vol_output  config  \
                   -textvariable ::Test_Analog_Rx::test_data($chan,vol_output)

      # Configure the Audio menubutton
      ${tile_path}.mnu_audio_dev  config  \
                -textvariable ::Test_Analog_Rx::test_data($chan,audio_dev)
    
      # Configure the Audio menu
      set max_index  [${tile_path}.mnu_audio_dev.menu  index  end]
      for { set i  0 } { $i <= $max_index } { incr i  1 } \
      {
         ${tile_path}.mnu_audio_dev.menu  entryconfig  $i  \
                    -variable ::Test_Analog_Rx::test_data($chan,audio_dev)
      }
    
      # Configure the Subaudio menubutton
      ${tile_path}.mnu_subaudio_type  config  \
                -textvariable ::Test_Analog_Rx::test_data($chan,subaudio_type)
    
      # Configure the Subaudio menu
      set max_index  [${tile_path}.mnu_subaudio_type.menu  index  end]
      for { set i  0 } { $i <= $max_index } { incr i  1 } \
      {
         ${tile_path}.mnu_subaudio_type.menu  entryconfig  $i  \
                    -variable ::Test_Analog_Rx::test_data($chan,subaudio_type)
      }
    
      # Configure the DCS code menubutton
      ${tile_path}.mnu_dcs_codes  config  \
                     -textvariable ::Test_Analog_Rx::test_data($chan,dcs_code)

      # Configure the DCS code menu
      set max_index  [${tile_path}.mnu_dcs_codes.menu  index  end]
      for { set i  0 } { $i <= $max_index } { incr i  1 } \
      {
         ${tile_path}.mnu_dcs_codes.menu  entryconfig  $i  \
                         -variable ::Test_Analog_Rx::test_data($chan,dcs_code)
      }

      # Configure the CTCSS code menubutton
      ${tile_path}.mnu_ctcss_codes  config  \
                   -textvariable ::Test_Analog_Rx::test_data($chan,ctcss_code)

      # Configure the CTCSS code menu
      set max_index  [${tile_path}.mnu_ctcss_codes.menu  index  end]
      for { set i  0 } { $i <= $max_index } { incr i  1 } \
      {
         ${tile_path}.mnu_ctcss_codes.menu  entryconfig  $i  \
              -variable ::Test_Analog_Rx::test_data($chan,ctcss_code_and_freq)
      }

      # Configure the Subaudio units label
      ${tile_path}.lbl_subaudio_units  config  \
               -textvariable ::Test_Analog_Rx::test_data($chan,subaudio_units)

      # Configure the Subaudio Deviation entry
      ${tile_path}.ent_subaudio_dev  config  \
                 -textvariable ::Test_Analog_Rx::test_data($chan,subaudio_dev)
    
      # Configure the 20dB Quieting Test Enable / Disable button
      ${tile_path}.btn_quiet_enable  config  \
                 -textvariable ::Test_Analog_Rx::test_data($chan,quiet_enable)

      # Configure the DVM Load entry
      ${tile_path}.ent_dvm_load  config  \
                     -textvariable ::Test_Analog_Rx::test_data($chan,dvm_load)

      # Configure the Distortion max entry
      ${tile_path}.ent_dist_max  config  \
                     -textvariable ::Test_Analog_Rx::test_data($chan,dist_max)
             
      # Configure the SINAD RF level max entry
      ${tile_path}.ent_sinad_rf_lvl_max  config  \
             -textvariable ::Test_Analog_Rx::test_data($chan,sinad_rf_lvl_max)
                 
      # Configure the SINAD min entry
      ${tile_path}.ent_sinad_min  config  \
                    -textvariable ::Test_Analog_Rx::test_data($chan,sinad_min)
    
      # Configure the 20 dB Quieting max entry
      ${tile_path}.ent_quiet_max  config  \
                    -textvariable ::Test_Analog_Rx::test_data($chan,quiet_max)
            
    return
  }


  #***************************************************************************
  #
  #        Name: toggle_channel_mode
  #
  #  Parameters: tile_path - tile path of the invoking widget
  #
  # Description: Toggles the mode for the present channel
  #
  #***************************************************************************
  proc toggle_channel_mode { tile_path } \
  {
    #puts "toggle_channel_mode"
    
    variable  test_data
  
    set tile_path  [get_tile  $tile_path]
    
    set tile_path  ${tile_path}.frm_config.analog_rx

    # Get the present channel
    set chan  $test_data(channel)
                
    # Toggle the mode
    if { $test_data($chan,mode) == "SIMPLEX" } \
    {
      set test_data($chan,mode)  "DUPLEX"
    } \
    else \
    {
      set test_data($chan,mode)  "SIMPLEX"
    }
    
    # Update the display
    set_channel_mode  $tile_path

    return
  }


  #***************************************************************************
  #
  #        Name: set_channel_mode
  #
  #  Parameters: tile_path - tile path of the invoking widget
  #
  # Description: Sets the mode for the present channel
  #
  #***************************************************************************
  proc set_channel_mode { tile_path } \
  {
    #puts "set_channel_mode"
    
    variable  test_data
  
    set tile_path  [get_tile  $tile_path]
    
    set tile_path  ${tile_path}.frm_config.analog_rx

    # Get the present channel
    set chan  $test_data(channel)
                
    # Set the mode
    switch -- $test_data($chan,mode) \
    { 
      "SIMPLEX"  { # Configure the Frequency entry to the 
                   # Analog Transmitter value
                   ${tile_path}.ent_freq  config  \
                         -textvariable ::Test_Analog_Tx::test_data($chan,freq)
                         
                   # Synchronize the values
                   set test_data($chan,freq)  $::Test_Analog_Tx::test_data($chan,freq)
                 }

      "DUPLEX"   { # Configure the Frequency entry to the 
                   # Analog Receiver value
                   ${tile_path}.ent_freq  config  \
                         -textvariable ::Test_Analog_Rx::test_data($chan,freq)
                 }
    }
    
    return
  }


  #***************************************************************************
  #
  #        Name: set_receive_freq
  #
  #  Parameters: tile_path - tile path of the invoking widget
  #
  # Description: Sets the Receiver freq to the Transmit freq when in SIMPLEX
  #              mode
  #
  #***************************************************************************
  proc set_receive_freq { tile_path } \
  {
    #puts "set_receive_freq"
    
    variable  test_data
  
    set tile_path  [get_tile  $tile_path]
    
    set tile_path  ${tile_path}.frm_config.analog_rx

    # Get the present channel
    set chan  $test_data(channel)
                
    # Set the mode
    if { $test_data($chan,mode) == "SIMPLEX" } \
    { 
      # Synchronize the values
      set test_data($chan,freq)  $::Test_Analog_Tx::test_data($chan,freq)
    }
    
    return
  }

 

  #***************************************************************************
  #
  #        Name: set_subaudio_units
  #
  #  Parameters: tile_path - tile path of the invoking widget
  #
  # Description: Sets the units of the Subaudio
  #
  #***************************************************************************
  proc set_subaudio_units { tile_path } \
  {
    #puts "set_subaudio_units:  $tile_path"
    
    variable  test_data
  
    set tile_path  [get_tile  $tile_path]
    
    set tile_path  ${tile_path}.frm_config.analog_rx

    # Get the present channel
    set chan  $test_data(channel)
                
    switch -- $test_data($chan,subaudio_type) \
    {
      "NONE"   { set test_data($chan,subaudio)  ""
                 set test_data($chan,subaudio_units)  ""
                 
                 grid remove  ${tile_path}.mnu_dcs_codes
                 grid remove  ${tile_path}.mnu_ctcss_codes
                 
                 grid remove  ${tile_path}.lbl_subaudio_dev
                 grid remove  ${tile_path}.ent_subaudio_dev
                 grid remove  ${tile_path}.lbl_subaudio_dev_units
               }
               
      "CTCSS"  { grid remove  ${tile_path}.mnu_dcs_codes

                 Screens::position_widget  ${tile_path}.mnu_ctcss_codes
                 
                 Screens::position_widget  ${tile_path}.lbl_subaudio_dev
                 Screens::position_widget  ${tile_path}.ent_subaudio_dev
                 Screens::position_widget  ${tile_path}.lbl_subaudio_dev_units
                 
                 # Set the CTCSS code and frequency
                 set_ctcss_code  $tile_path
               }
               
      "DCS"    { grid remove  ${tile_path}.mnu_ctcss_codes

                 Screens::position_widget  ${tile_path}.mnu_dcs_codes
                 
                 Screens::position_widget  ${tile_path}.lbl_subaudio_dev
                 Screens::position_widget  ${tile_path}.ent_subaudio_dev
                 Screens::position_widget  ${tile_path}.lbl_subaudio_dev_units
                 
                 # Set the DCS code
                 set_dcs_code  $tile_path
               }
    }
  
    return
  }


  #***************************************************************************
  #
  #        Name: set_ctcss_code
  #
  #  Parameters: tile_path - tile path of the invoking widget
  #
  # Description: Sets the CTCSS code for the Subaudio tone
  #
  #***************************************************************************
  proc set_ctcss_code { tile_path } \
  {
    #puts "set_ctcss_code:  $tile_path"
    
    variable  test_data

    set tile_path  [get_tile  $tile_path]
    
    set tile_path  ${tile_path}.frm_config.analog_rx

    # Get the present channel
    set chan  $test_data(channel)

    foreach { code  dash  freq } $test_data($chan,ctcss_code_and_freq) { }
    
    # Get the code, and display it on the menubutton
    set test_data($chan,ctcss_code)  $code
    
    # Get the code frequency, and display it in the 'freq' field,
    set test_data($chan,subaudio_units)  "freq:  $freq"

    # Set the 'test_data' variable to the frequency
    set test_data($chan,subaudio)  $freq
    
    return
  }
  
  
  #***************************************************************************
  #
  #        Name: set_dcs_code
  #
  #  Parameters: tile_path - tile path of the invoking widget
  #
  # Description: Sets the DCS code for the Subaudio tone
  #
  #***************************************************************************
  proc set_dcs_code { tile_path } \
  {
    #puts "set_dcs_code:  $tile_path"
    
    variable  test_data

    set tile_path  [get_tile  $tile_path]
    
    set tile_path  ${tile_path}.frm_config.analog_rx

    # Get the present channel
    set chan  $test_data(channel)

    # Display the octal representation
    set test_data($chan,subaudio_units)  "(oct.)"

    # Set the 'test_data' variable to the DCS code
    set test_data($chan,subaudio)  $test_data($chan,dcs_code)
    
    return
  }


  #***************************************************************************
  #
  #        Name: set_quiet_enable
  #
  #  Parameters: tile_path - tile path of the invoking widget
  #
  # Description: Sets the state of the 20 dB Quieting Test between
  #              ENABLE and DISABLE
  #
  #***************************************************************************
  proc set_quiet_enable { tile_path } \
  {
    #puts "set_quiet_enable:  $tile_path"
    
    variable  test_data
  
    set tile_path  [get_tile  $tile_path]
    
    set tile_path  ${tile_path}.frm_config.analog_rx

    set chan  $test_data(channel)
    
    if { $test_data($chan,quiet_enable) == "ENABLED" } \
    {
      Screens::position_widget  ${tile_path}.lbl_dvm_load
      Screens::position_widget  ${tile_path}.ent_dvm_load
      Screens::position_widget  ${tile_path}.lbl_dvm_load_units

      Screens::position_widget  ${tile_path}.lbl_quiet
      Screens::position_widget  ${tile_path}.lbl_max4
      Screens::position_widget  ${tile_path}.ent_quiet_max
      Screens::position_widget  ${tile_path}.lbl_quiet_units
    } \
    else \
    {
      grid remove  ${tile_path}.lbl_dvm_load
      grid remove  ${tile_path}.ent_dvm_load
      grid remove  ${tile_path}.lbl_dvm_load_units

      grid remove  ${tile_path}.lbl_quiet
      grid remove  ${tile_path}.lbl_max4
      grid remove  ${tile_path}.ent_quiet_max
      grid remove  ${tile_path}.lbl_quiet_units
    }

    return
  }


  #***************************************************************************
  #
  #        Name: toggle_quiet_enable
  #
  #  Parameters: tile_path - tile path of the invoking widget
  #
  # Description: Toggles the state of the 20 dB Quieting Test between
  #              ENABLE and DISABLE
  #
  #***************************************************************************
  proc toggle_quiet_enable { tile_path } \
  {
    #puts "toggle_quiet_enable:  $tile_path"
    
    variable  test_data
  
    set tile_path  [get_tile  $tile_path]
    
    set tile_path  ${tile_path}.frm_config.analog_rx

    set chan  $test_data(channel)
    
    if { $test_data($chan,quiet_enable) == "ENABLED" } \
    {
      set test_data($chan,quiet_enable)  "DISABLED"
      
      grid remove  ${tile_path}.lbl_dvm_load
      grid remove  ${tile_path}.ent_dvm_load
      grid remove  ${tile_path}.lbl_dvm_load_units

      grid remove  ${tile_path}.lbl_quiet
      grid remove  ${tile_path}.lbl_max4
      grid remove  ${tile_path}.ent_quiet_max
      grid remove  ${tile_path}.lbl_quiet_units
    } \
    else \
    {
      set test_data($chan,quiet_enable)  "ENABLED"

      Screens::position_widget  ${tile_path}.lbl_dvm_load
      Screens::position_widget  ${tile_path}.ent_dvm_load
      Screens::position_widget  ${tile_path}.lbl_dvm_load_units

      Screens::position_widget  ${tile_path}.lbl_quiet
      Screens::position_widget  ${tile_path}.lbl_max4
      Screens::position_widget  ${tile_path}.ent_quiet_max
      Screens::position_widget  ${tile_path}.lbl_quiet_units
    }
          
    return
  }


  #***************************************************************************
  #
  #        Name: test
  #
  #  Parameters: tile_path - tile path of the invoking widget
  #
  # Description: Runs the test
  #
  #***************************************************************************
  proc test { tile_path } \
  {
    #puts "Test_Analog_Rx - test:  $tile_path"
    
    variable  test_data
        
    set tile_path  [get_tile  $tile_path]

    Append Test 1 "Analog Receiver Test"

    if [Abort] \
    {
      return
    }
    
    
    # Setup for Test #1
    Print "Setting up for Test #1 ..."

    # set_default_channel_data
    setup
    
    after 2000

    # Set the Generator frequency
    foreach { chan } $test_data(chan_list) \
    {
      if [Abort] \
      {
        break
      }
    
      # Prompt the operator to set the radio to the test channel
      set title  "Operator Instruction"

      set prompt_text  "Set the radio to Channel $chan\n\n"
      
      if { $test_data($chan,mode) == "SIMPLEX" } \
      {
        # Use the Analog Transmitter frequency
        append prompt_text  "\[Frequency:  $::Test_Analog_Tx::test_data($chan,freq)\]"
      } \
      else \
      {
        # Use the Analog Receiver frequency
        append prompt_text  "\[Frequency:  $::Test_Analog_Rx::test_data($chan,freq)\]"
      }

      set window_name  ${tile_path}.instruction
      
      # Dialog:  no bitmap displayed, OK and ABORT buttons
      set prompt_value  [tk_dialog  $window_name  $title        \
                                                  $prompt_text  \
                                                  {}            \
                                                  0             \
                                                  "OK"          \
                                                  "ABORT"]
        
      switch -- $prompt_value \
      {
        0 { }
      
        1 { Abort 1;  break }
      }
      
      if { $test_data($chan,mode) == "SIMPLEX" } \
      {
        # Use the Analog Transmitter frequency
        Generator freq  $::Test_Analog_Tx::test_data($chan,freq) -nowait
      } \
      else \
      {
        # Use the Analog Receiver frequency
        Generator freq  $::Test_Analog_Rx::test_data($chan,freq) -nowait
      }

      after 1000

      # Setup for subtest #1
      Generator rflevel  -47.0  -nowait

      after 1000

      # Setup for subtest #1;  CTCSS or DCS mode and deviation
      if { $test_data($chan,subaudio_type) == "CTCSS" } \
      {
        FGen::setMode  4  CTCSS
        
        FGen::setDeviation  4  $test_data($chan,subaudio_dev)
          
        FGen::setFreq  4  $test_data($chan,subaudio)
      } \
      elseif { $test_data($chan,subaudio_type) == "DCS" } \
      {
        set ::FGen::State(4,DCS,state)   "ON"
        set ::FGen::State(4,DCS,invert)  "NORMAL"
        set ::FGen::State(4,DCS,code)    $test_data($chan,subaudio)
        
        FGen::setMode  4  DCS
        
        FGen::setDeviation  4  $test_data($chan,subaudio_dev)
      }
        
      after 2000

      # Prompt the operator to adjust the audio output level
      set title  "Operator Instruction"

      set prompt_text  "Verify that the audio output of the radio \n"
      
      append prompt_text  "is connected to the DVM input port,\n"
      
      append prompt_text  "and adjust the radio volume for approx. "
      
      append prompt_text  "$test_data($chan,vol_output) volts rms."
      
      set adjust_label  "DVM Reading:"
      
      set adjust_value  "::Dvm::State(reading,V)"
      
      set adjust_units  "V-RMS"
      
      set adjust_target  $test_data($chan,vol_output)
      
      set adjust_tolerance  0.1;  # +/- 0.1 VRMS

      set prompt_value  [adjust_dialog  $tile_path      \
                                                     $title          \
                                                     $prompt_text    \
                                                     $adjust_label   \
                                                     $adjust_value   \
                                                     $adjust_units   \
                                                     $adjust_target  \
                                                     $adjust_tolerance]

      switch -- $prompt_value \
      {
        0 { }
      
        1 { Abort 1;  break }
      }


      Format  "Transmit Channel"  "$chan"  "%s"
      
      if { $test_data($chan,mode) == "SIMPLEX" } \
      {
        # Use the Analog Transmitter frequency
        Format  "Transmit Frequency"  \
                     "$::Test_Analog_Tx::test_data($chan,freq)"  "%.6f"  "MHz"
      } \
      else \
      {
        # Use the Analog Receiver frequency
        Format  "Transmit Frequency"  \
                     "$::Test_Analog_Rx::test_data($chan,freq)"  "%.6f"  "MHz"
      }


      set rflevel  [Generator  rflevel]

      if { $test_data($chan,audio_dev) == "1.5" } \
      {
          FGen::setDeviation  3  1.50
      } else \
      {
          FGen::setDeviation  3  3.00
      }
      FGen::setFreq       3  1000.0
      FGen::setShape      3  SINE
      FGen::setMode  3  TONE

      Format  "Transmit Level"  "$rflevel"  "%.2f"  "dBm"

      # Format  "Audible Tone Frequency"  "$::FGen::State(3,TONE,freq)"  "%.2f"  "Hz"
      # Format  "Audible Tone Shape"  "$::FGen::State(3,TONE,shape)"  "%s"  ""
      Format  "Audible Tone Deviation"  "$::FGen::State(3,TONE,dev)"  "%.2f"  "kHz"
      

      #after 2000
      
      # Measure Distortion
      # Get the reading
      set result  [Auto_Test_2::settle_reading  "Dist" "value"]
      
      set max  $test_data($chan,dist_max)
      
      # Check the result
      if { $result <= $max } \
      {
        Format  "Receiver Distortion"  "$result"  "%.2f"  "%"  "PASS"
      } \
      else \
      {
        Format  "Receiver Distortion"  "$result"  "%.2f"  "%"  "FAIL"
      }
    
      # Setup for subtest #2
      Generator rflevel  $test_data($chan,sinad_rf_lvl_max)  -nowait

      after 1000

      # Display the RF Generator level
      set rflevel  [Generator  rflevel]

      
      #Format  "Generator Level"  "$rflevel"  "%.2f"  "dBm"

      Format  "Sensitivity Level"  "$test_data($chan,sinad_rf_lvl_max)"  "%.2f"  "dBm"


      # Measure SINAD
      # Get the reading
      set result  [Auto_Test_2::settle_reading  "Sinad" "value"]
      
      set min  $test_data($chan,sinad_min)
      
      # Check the result
      if { $result >= $min } \
      {
        Format  "SINAD"  "$result"  "%.2f"  "dB"  "PASS"
      } \
      else \
      {
        Format  "SINAD"  "$result"  "%.2f"  "dB"  "FAIL"
      }

      # Measure 20dB Quieting
      if { $test_data($chan,quiet_enable) == "ENABLED" } \
      {
        # Turn off the Modulator 3
        FGen::setMode  3  OFF

        after 2000

        # Set up an initial generator level. This used to use -137, but the test was too
        # long. Start with the level the user wanted to run the Sinad sensitivity test at
        set initial_rflevel  $test_data($chan,sinad_rf_lvl_max)
        
        set rflevel [Generator rflevel $initial_rflevel -forgive]
      
        after 2000
        
        # Setup the DVM
        Dvm  avg  10  -nowait
        Dvm  mode  AC  -nowait
        Dvm  impedance  0  -nowait
        
        ::Dvm::setResistance  $test_data($chan,dvm_load)
        
        set ::Dvm::State(units)  "dBm"
        
        after 2000
        
        # Get the initial DVM reading
        set initial_reading  $::Dvm::State(reading,any)
        
        # Initialize the variables
        set next_reading   0.00
        set reading_delta  0.00

        while { $reading_delta < 20.0 } \
        {
          if [Abort] \
          {
            set reading_delta  21.0
            
            break
          }
          
          # Get the next reading
          set next_reading  $::Dvm::State(reading,any)

          # Calculate the delta
          set reading_delta  [expr abs($initial_reading - $next_reading)]

          # Increase the Generator level by 2 dBm, then reset to what the generator accepts
          # Exit if the RF level gets unreasonably high
          if { $rflevel > -40 }\
          {
             set reading_delta  21.0
             break
          }
          set rflevel [expr $rflevel + 2.0]
          set rflevel [Generator rflevel  $rflevel -forgive]
          
          after 1000
        }
        
        # Get the RF Level
        set result  [Generator rflevel]
        
        set max  $test_data($chan,quiet_max)
      
        # Check the result
        if { $result <= $max } \
        {
          Format  "20dB Quieting"  "$result"  "%.2f"  "dB"  "PASS"
        } \
        else \
        {
          Format  "20dB Quieting"  "$result"  "%.2f"  "dB"  "FAIL"
        }
      }
    }
      
    # Shut off the Generator
    Generator ptt OFF  -nowait

    return
  }


  #***************************************************************************
  #
  #        Name: setup
  #
  #  Parameters:
  #
  # Description: Sets up for the ANALOG RX test
  #
  #***************************************************************************
  proc setup { } \
  {
    #puts "setup"
    
    # Setup Generator
    Generator dcpower   ON          -nowait
    Generator ptt       ON          -nowait
    Generator enable    11          -nowait
    Generator freq      170.000000  -nowait
    Generator rfoutput  T/R         -nowait
    Generator rflevel   -40.0       -nowait
    Generator hwalc     154         -nowait
    Generator hwbw      NARROW      -nowait
    Generator mod       1           -nowait
    Generator p25mode   1           -nowait
    Generator yigspeed  1           -nowait

    # Setup BBAudio
    BBAudio  micbias      ELEC   -nowait
    BBAudio  sinadbal     BALAN  -nowait
    BBAudio  demodfilter  TONE   -nowait
    BBAudio  fgenfilter   TONE   -nowait
    BBAudio  fgenbalance  BALAN  -nowait
    BBAudio  fgenlevel    1      -nowait

    BBAudio  clearmixer  SPKR  -nowait
    BBAudio::set_bbaudio_route  SINAD  SPKR    1;  # Audio In to Speaker
    BBAudio::update_mixers [BBAudio getmixer SPKR]

    BBAudio  clearmixer  DEMOD  -nowait
    BBAudio::update_mixers [BBAudio getmixer DEMOD]

    BBAudio  clearmixer  FGEN  -nowait
    BBAudio::update_mixers [BBAudio getmixer FGEN]

    BBAudio  clearmixer  L1  -nowait
    BBAudio::set_bbaudio_route  FGEN3  L1  1
    BBAudio::set_bbaudio_route  FGEN4  L1  1
    BBAudio::update_mixers [BBAudio getmixer L1]
    
    BBAudio  clearmixer  USER0  -nowait
    BBAudio::update_mixers [BBAudio getmixer USER0]

    BBAudio  clearmixer  USER1  -nowait
    BBAudio::update_mixers [BBAudio getmixer USER1]


    # Setup Dvm
    Dvm run        1    -nowait
    Dvm avg        10   -nowait
    Dvm en         3    -nowait
    Dvm peak       0    -nowait
    Dvm alerts     0    -nowait
    Dvm ulim       0.0  -nowait
    Dvm llim       0.0  -nowait
    Dvm impedance  0    -nowait
    Dvm mode       AC   -nowait
    Dvm hwrange    0    -nowait
    Dvm::setResistance 600
    
    set ::Dvm::State(BellEnabled)   {0}
    set ::Dvm::State(units)         {V}
    
    
    # Setup Sinad
    Sinad run     1       -nowait
    Sinad math    1       -nowait
    Sinad trig    2       -nowait
    Sinad avg     1       -nowait
    Sinad filter  10      -nowait
    Sinad en      3       -nowait
    Sinad peak    0       -nowait
    Sinad alerts  0       -nowait
    Sinad ulim    0.00    -nowait
    Sinad llim    0.00    -nowait
    Sinad encwt   OFF     -nowait
    Sinad rate    4       -nowait
    Sinad minx    0       -nowait
    Sinad maxx    20000   -nowait
    Sinad miny    -80.0   -nowait
    Sinad maxy    0.0     -nowait
    Sinad size    2048    -nowait
    Sinad alpha   2.6     -nowait
    Sinad window  1       -nowait
    Sinad lnoise  300.0   -nowait
    Sinad hnoise  4000.0  -nowait
    Sinad lnotch  900.0   -nowait
    Sinad hnotch  1100.0  -nowait
    
    catch { Sinad log 0 -nowait }
    
    Dist peak    0     -nowait
    Dist alerts  0     -nowait
    Dist ulim    0.00  -nowait
    Dist llim    0.00  -nowait
    
    set ::Sinad::State(Sinad.BellEnabled)   {0}
    set ::Sinad::State(Sinad.resolution)    {1.0}
    set ::Sinad::State(Dist.BellEnabled)    {0}
    set ::Sinad::State(Dist.resolution)     {1.0}
    
    Sinad::Input {Aud Bal}
    
    
    # Setup Function Generators
    FGen::setMode  1  OFF
    FGen::setMode  2  OFF

    FGen::setMode  3  TONE

    FGen::setDeviation  3  3.00
    FGen::setFreq       3  1000.0
    FGen::setShape      3  SINE

    # Turn Function Generator #4 (Modulator #2) OFF for now
    FGen::setMode  4  OFF

    ## Version 1.6.0 compatibility
    #catch { set GeneratorUI::fgen_mode(3)   "TONE" } result
    #catch { set GeneratorUI::fgen_mode(4)   "OFF"  } result
    #

    return
  }


  #***************************************************************************
  #
  #        Name: get_config_summary
  #
  #  Parameters:
  #
  # Description: Returns the a formatted summary of the present 
  #              configuration
  #
  #***************************************************************************
  proc get_config_summary { } \
  {
    #puts "get_config_summary"
    
    variable  test_data
    
    # set_default_channel_data
    set return_list  {}

    append return_list  "**** Analog Receiver Test Configuration **** \n\n"
      
    if { $test_data(chan_list) == "" } \
    {
      append return_list  "  NO CONFIGURATION DATA \n"
    }

    foreach { chan } $test_data(chan_list) \
    {
      append return_list  "  SETUP   Channel:             $chan \n"
      append return_list  "          Freq:                $test_data($chan,freq) \n"
      append return_list  "          Mode:                $test_data($chan,mode) \n"
      append return_list  "          Volume Output:       $test_data($chan,vol_output) \n"
      append return_list  "          Audio deviation:     $test_data($chan,audio_dev) \n"
      append return_list  "          Subaudio type:       $test_data($chan,subaudio_type) \n"
      
      if { $test_data($chan,subaudio_type) != "NONE" } \
      {
        append return_list  "          Subaudio:            $test_data($chan,subaudio) \n"
        append return_list  "          Subaudio deviation:  $test_data($chan,subaudio_dev) \n"
      }
      
      append return_list  "          20dB Quieting Test:  $test_data($chan,quiet_enable) \n"

      if { $test_data($chan,quiet_enable) == "ENABLED" } \
      {
        append return_list  "          Dvm Load:            $test_data($chan,dvm_load) \n"
      }

      append return_list  "\n"

      append return_list  "  PASS/FAIL   Distortion  \n"
      append return_list  "                 max:  $test_data($chan,dist_max) \n"
      append return_list  "              RF Threshold  \n"
      append return_list  "                 max:  $test_data($chan,sinad_rf_lvl_max) \n"
      append return_list  "              SINAD  \n"
      append return_list  "                 min:  $test_data($chan,sinad_min) \n"

      if { $test_data($chan,quiet_enable) == "ENABLED" } \
      {
        append return_list  "              20 dB Quieting  \n"
        append return_list  "                 max:  $test_data($chan,quiet_max) \n"
      }
    }

    append return_list  " \n\n"

    return  [subst -nocommands $return_list]
  }


  #***************************************************************************
  #
  #        Name: get_html_config
  #
  #  Parameters:
  #
  # Description: Returns the an HTML formatted configuration display
  #
  #***************************************************************************
  proc get_html_config { } \
  {
    #puts "get_html_config"
    
    variable  test_data_html
    
    # Initialize the return data
    set return_list  {}

    # Set up the HTML page section header
    append return_list  "<B>ANALOG RECEIVER TEST CONFIGURATION</B> <BR><BR>"

    if { $test_data_html(chan_list) == "" } \
    {
      append return_list  "  NO CONFIGURATION DATA <BR>"
    }

    foreach { chan } $test_data_html(chan_list) \
    {
      append return_list  "  SETUP   Channel:             $chan <BR>"
      append return_list  "          Freq:                $test_data_html($chan,freq) <BR>"
      append return_list  "          Mode:                $test_data_html($chan,mode) <BR>"
      append return_list  "          Volume Output:       $test_data_html($chan,vol_output) <BR>"
      append return_list  "          Subaudio type:       $test_data_html($chan,subaudio_type) <BR>"
      
      if { $test_data_html($chan,subaudio_type) != "NONE" } \
      {
        append return_list  "          Subaudio:            $test_data_html($chan,subaudio) <BR>"
        append return_list  "          Subaudio deviation:  $test_data_html($chan,subaudio_dev) <BR>"
      }
      
      append return_list  "          20dB Quieting Test:  $test_data_html($chan,quiet_enable) <BR>"

      if { $test_data_html($chan,quiet_enable) == "ENABLED" } \
      {
        append return_list  "          Dvm Load:            $test_data_html($chan,dvm_load) <BR>"
      }

      append return_list  "<BR>"

      append return_list  "  PASS/FAIL   Distortion  <BR>"
      append return_list  "                 max:  $test_data_html($chan,dist_max) <BR>"
      append return_list  "              RF Threshold  <BR>"
      append return_list  "                 max:  $test_data_html($chan,sinad_rf_lvl_max) <BR>"
      append return_list  "              SINAD  <BR>"
      append return_list  "                 min:  $test_data_html($chan,sinad_min) <BR>"

      if { $test_data_html($chan,quiet_enable) == "ENABLED" } \
      {
        append return_list  "              20 dB Quieting  <BR>"
        append return_list  "                 max:  $test_data_html($chan,quiet_max) <BR>"
      }
    }

    append return_list  "<BR><BR>"

    return  $return_list
  }


  #***************************************************************************
  #
  #        Name: get_config_data
  #
  #  Parameters:
  #
  # Description: Returns the present configuration
  #
  #***************************************************************************
  proc get_config_data { } \
  {
    #puts "get_config_data"
    
    variable  test_data
    
    set return_list  {}
    
    foreach { array_index } [array names  test_data] \
    {
      append return_list  "set ::Test_Analog_Rx::test_data($array_index)  {$test_data($array_index)} \n"
    }

    return  [subst -nocommands $return_list]
  }


  #***************************************************************************
  #
  # Register this test by adding the namespace to the Auto Test list
  #
  #***************************************************************************
  if { [IFR_options  is_installed  Auto_Test] } \
  {
    lappend  ::Auto_Test_2::test_namespace_list  "Test_Analog_Rx"
  }


};  # End of namespace Test_Analog_Rx

