#***************************************************************************
#
# Test procedure:  Test #3:  Digital Receiver Test
#
#***************************************************************************


namespace eval Test_Digital_Rx \
{
  variable  short_name  "digital_rx"
  
  variable  long_name   "Digital Receiver Test"
  
  variable  tile_class  "TEST_DIGITAL_RX"
  
  variable  test_data
  
  set test_data(chan_list)  {}

  set test_data(channel)  1
  
  namespace import ::Auto_Test_2::*
  
  
  #***************************************************************************
  #
  #        Name: Create
  #
  #  Parameters: parent - parent of child window
  #
  # Description: Creates the tile
  #
  #***************************************************************************
  proc Create { parent } \
  {
    Debug::Puts Autotest 2 "Test_Digital_Rx::Create:  $parent"

    variable  short_name
    variable  tile_class

    set tile_path  [name_child  $parent  $short_name]
    
    if { ![winfo exists $tile_path] } \
    {
      set tile_path  \
                 [Screens::create_tile  $tile_path  $short_name  $tile_class]
    }  
     
    return  $tile_path
  }


  #***************************************************************************
  #
  #        Name: Map
  #
  #  Parameters: 
  #
  # Description: Initializes the tile
  #
  #***************************************************************************
  proc Map { tile_path } \
  {
    Debug::Puts Autotest 2 "::Test_Digital_Rx::Map:  $tile_path"
    
    #set tile_path  [get_tile  $tile_path]
    
    # Set the configuration data
    set_channel_data  $tile_path
    
    # Set the Mode
    set_channel_mode  $tile_path

    # Set the Encryption options on the menu
    set_encryption_options  $tile_path

    # Set the Encryption display
    set_encryption  $tile_path
    
    trace variable ::Key_LoaderUI::keyloader_done_flag  w  \
                                    "Test_Digital_Rx::set_keyid_status"

    return
  }


  #***************************************************************************
  #
  #        Name: Cleanup
  #
  #  Parameters: 
  #
  # Description: 
  #
  #***************************************************************************
  proc Cleanup { tile_path } \
  {
    Debug::Puts Autotest 2 "::Test_Digital_Rx::Cleanup:  $tile_path"

    #set tile_path  [get_tile  $tile_path]
    
    return
  }


  #***************************************************************************
  #
  #        Name: Unmap
  #
  #  Parameters: 
  #
  # Description: 
  #
  #***************************************************************************
  proc Unmap { tile_path } \
  {
    Debug::Puts Autotest 2 "::Test_Digital_Rx::Unmap:  $tile_path"

    #set tile_path  [get_tile  $tile_path]

    Cleanup  $tile_path

    trace vdelete ::Key_LoaderUI::keyloader_done_flag  w  \
                                    "Test_Digital_Rx::set_keyid_status"
    
    return
  }


  #***************************************************************************
  #
  #        Name: Destroy
  #
  #  Parameters: 
  #
  # Description: 
  #
  #***************************************************************************
  proc Destroy { tile_path } \
  {
    Debug::Puts Autotest 2 "::Test_Digital_Rx::Destroy:  $tile_path"

    #set tile_path  [get_tile  $tile_path]

    Cleanup  $tile_path

    return
  }


  #***************************************************************************
  #
  #        Name: set_channel_data
  #
  #  Parameters: tile_path  - tile path of the invoking widget
  #
  # Description: Sets the text variables of the configuration display 
  #              depending on the channel number
  #
  #***************************************************************************
  proc set_channel_data { tile_path } \
  {
    Debug::Puts Autotest 2 "set_channel_data:  $tile_path"
  
    variable  test_data

    set tile_path  [get_tile  $tile_path]
    
    set tile_path  ${tile_path}.frm_config.digital_rx

    set chan  $test_data(channel)
    
    Debug::Puts Autotest 2 "set_channel_data:  channel is $chan"
    # Determine if the present channel number is in the channel list.
    # If it is not, append the new value and then sort the list in
    # increasing order.
    if { [lsearch $test_data(chan_list) $chan] == -1 } \
    {
      Debug::Puts Autotest 2 "set_channel_data:  adding $chan"
      lappend test_data(chan_list)  $chan
      
      if [oldCatch { set tempList  [lsort -increasing -integer $test_data(chan_list)]} err] \
      {
         set tempList  [lsort -increasing $test_data(chan_list)]
      }
      set test_data(chan_list) $tempList


    }
      
    # Configure the Frequency entry
      if { ![info exists test_data($chan,freq)] } \
      {
        Debug::Puts Autotest 2 "set_channel_data:  adding $chan freq "
        set test_data($chan,freq)  "0.000000"
      }
    
      ${tile_path}.ent_freq  config  -state normal \
                       -textvariable ::Test_Digital_Rx::test_data($chan,freq)

    # Configure the Mode button
      if { ![info exists test_data($chan,mode)] } \
      {
        Debug::Puts Autotest 2 "set_channel_data:  adding $chan mode "
        set test_data($chan,mode)  "DUPLEX"
      }

      ${tile_path}.btn_mode  config  -state normal \
                       -textvariable ::Test_Digital_Rx::test_data($chan,mode)
    
    # Configure the NAC entry
      if { ![info exists test_data($chan,nac)] } \
      {
        Debug::Puts Autotest 2 "set_channel_data:  adding $chan nac "
        set test_data($chan,nac)  "000"
      }

      ${tile_path}.ent_nac  config  -state normal \
                        -textvariable ::Test_Digital_Rx::test_data($chan,nac)
    
    # Configure the SID entry
      if { ![info exists test_data($chan,sid)] } \
      {
        Debug::Puts Autotest 2 "set_channel_data:  adding $chan sid "
        set test_data($chan,sid)  "00000000"
      }

      ${tile_path}.ent_sid  config  -state normal \
                        -textvariable ::Test_Digital_Rx::test_data($chan,sid)
    
    # Configure the TGID entry
      if { ![info exists test_data($chan,tgid)] } \
      {
        Debug::Puts Autotest 2 "set_channel_data:  adding $chan tgid "
        set test_data($chan,tgid)  "0000"
      }

      ${tile_path}.ent_tgid  config   -state normal\
                       -textvariable ::Test_Digital_Rx::test_data($chan,tgid)
    
    # Configure the Encryption algorithm
      if { ![info exists test_data($chan,algid)] } \
      {
        set test_data($chan,algid)  "80"
      }

    # Configure the Encryption menubutton
      if { ![info exists test_data($chan,encrypt)] } \
      {
        Debug::Puts Autotest 2 "set_channel_data:  adding $chan encrypt "
        set test_data($chan,encrypt)  "CLEAR"
        set test_data($chan,algid)  "80"
      }

      ${tile_path}.mnu_encrypt config  -state normal \
                    -textvariable ::Test_Digital_Rx::test_data($chan,encrypt)

    # Configure the Encryption menu
      set max_index  [${tile_path}.mnu_encrypt.menu  index  end]
    
      for { set i  0 } { $i <= $max_index } { incr i  1 } \
      {
        ${tile_path}.mnu_encrypt.menu  entryconfig  $i  \
                        -variable ::Test_Digital_Rx::test_data($chan,encrypt)
      }
    
    # Configure the KEYID entry
      if { ![info exists test_data($chan,keyid)] } \
      {
         Debug::Puts Autotest 2 "set_channel_data:  adding $chan keyid "
         set test_data($chan,keyid)  "0000"
      }

      ${tile_path}.ent_keyid  config  \
                      -textvariable ::Test_Digital_Rx::test_data($chan,keyid)

    # Configure the KEYID Status label
      if { ![info exists test_data($chan,keyid_status)] } \
      {
         Debug::Puts Autotest 2 "set_channel_data:  adding $chan keyid_status "
         set test_data($chan,keyid_status)  ""
      }

      ${tile_path}.lbl_keyid_status  config  \
               -textvariable ::Test_Digital_Rx::test_data($chan,keyid_status)

    # Configure the RF Threshold min entry
      if { ![info exists test_data($chan,rf_thrshld_min)] } \
      {
         Debug::Puts Autotest 2 "set_channel_data:  adding $chan rf_thrshld_min "
         set test_data($chan,rf_thrshld_min)  "0.00"
      }

      ${tile_path}.ent_rf_thrshld_min  config  \
             -textvariable ::Test_Digital_Rx::test_data($chan,rf_thrshld_min)

    return
  }


  #***************************************************************************
  #
  #        Name: toggle_channel_mode
  #
  #  Parameters: tile_path - tile path of the invoking widget
  #
  # Description: Toggles the mode for the present channel
  #
  #***************************************************************************
  proc toggle_channel_mode { tile_path } \
  {
    Debug::Puts Autotest 2 "toggle_channel_mode"
    
    variable  test_data
  
    set tile_path  [get_tile  $tile_path]
    
    set tile_path  ${tile_path}.frm_config.digital_rx

    # Get the present channel
    set chan  $test_data(channel)
                
    # Toggle the mode
    if { $test_data($chan,mode) == "SIMPLEX" } \
    {
      set test_data($chan,mode)  "DUPLEX"
    } \
    else \
    {
      set test_data($chan,mode)  "SIMPLEX"
    }
    
    # Update the display
    set_channel_mode  $tile_path

    return
  }


  #***************************************************************************
  #
  #        Name: set_channel_mode
  #
  #  Parameters: tile_path - tile path of the invoking widget
  #
  # Description: Sets the mode for the present channel
  #
  #***************************************************************************
  proc set_channel_mode { tile_path } \
  {
    Debug::Puts Autotest 2 "set_channel_mode"
    
    variable  test_data
  
    set tile_path  [get_tile  $tile_path]
    
    set tile_path  ${tile_path}.frm_config.digital_rx

    # Get the present channel
    set chan  $test_data(channel)
                
    # Set the mode
    switch -- $test_data($chan,mode) \
    {
      "SIMPLEX"  { # Configure the Frequency entry to the 
                   # Digital Transmitter value
                   ${tile_path}.ent_freq  config  \
                       -textvariable ::Test_Digital_Tx::test_data($chan,freq)
                       
                   # Synchronize the values
                   set test_data($chan,freq)  $::Test_Digital_Tx::test_data($chan,freq)
                 }

      "DUPLEX"   { # Configure the Frequency entry to the 
                   # Digital Receiver value
                   ${tile_path}.ent_freq  config  \
                       -textvariable ::Test_Digital_Rx::test_data($chan,freq)
                 }
    }
    
    return
  }


  #***************************************************************************
  #
  #        Name: set_receive_freq
  #
  #  Parameters: tile_path - tile path of the invoking widget
  #
  # Description: Sets the Receiver freq to the Transmit freq when in SIMPLEX
  #              mode
  #
  #***************************************************************************
  proc set_receive_freq { tile_path } \
  {
    Debug::Puts Autotest 2 "set_receive_freq"
    
    variable  test_data
  
    set tile_path  [get_tile  $tile_path]
    
    set tile_path  ${tile_path}.frm_config.digital_rx

    # Get the present channel
    set chan  $test_data(channel)
                
    # Set the mode
    if { $test_data($chan,mode) == "SIMPLEX" } \
    { 
      # Synchronize the values
      set test_data($chan,freq)  $::Test_Digital_Tx::test_data($chan,freq)
    }
    
    return
  }


  #***************************************************************************
  #
  #        Name: set_encryption_options
  #
  #  Parameters: tile_path - tile path of the invoking widget
  #
  # Description: Sets the encryption options by verifying the KVL and AES
  #              options are installed
  #
  #***************************************************************************
  proc set_encryption_options { tile_path } \
  {
    Debug::Puts Autotest 2 "set_encryption_options"
    
    variable  test_data
  
    set tile_path  [get_tile  $tile_path]
    
    set tile_path  ${tile_path}.frm_config.digital_rx

    # Get the present channel
    set chan  $test_data(channel)
                
    # Verify that both the KVL and AES options are installed
    if { [IFR_options  is_installed  AES] && \
                                     [IFR_options  is_installed  KEY_MGT] } \
    {
      # If they are, enable the 'AES' menu option
      ${tile_path}.mnu_encrypt.menu  entryconfig  "AES"  -state normal
    } \
    else \
    {
      # If not, disable the 'AES' menu option
      ${tile_path}.mnu_encrypt.menu  entryconfig  "AES"  -state disabled
      
      # Invoke the CLEAR menu entry
      ${tile_path}.mnu_encrypt.menu  invoke  "CLEAR"
    }

    return
  }


  #***************************************************************************
  #
  #        Name: set_encryption
  #
  #  Parameters: tile_path - tile path of the invoking widget
  #
  # Description: Sets the encryption mode
  #
  #***************************************************************************
  proc set_encryption { tile_path } \
  {
    Debug::Puts Autotest 2 "set_encryption"
    
    variable  test_data
  
    set tile_path  [get_tile  $tile_path]
    
    set tile_path  ${tile_path}.frm_config.digital_rx

    # Get the present channel
    set chan  $test_data(channel)
                
    # Set the algorithm ID
    switch -- $test_data($chan,encrypt) \
    {
      "CLEAR"  { set test_data($chan,algid)  "80" }
      "DES"    { set test_data($chan,algid)  "81" }
      "AES"    { set test_data($chan,algid)  "84" }
    }

    # Toggle the mode
    switch -- $test_data($chan,encrypt) \
    {
      "CLEAR"  { # Remove the KEYID fields
                 grid remove  ${tile_path}.lbl_keyid
                 grid remove  ${tile_path}.ent_keyid
                 grid remove  ${tile_path}.lbl_keyid_units
                 grid remove  ${tile_path}.lbl_keyid_status
                 grid remove  ${tile_path}.btn_key_loader
               }
      "DES" -
      "AES"    { # Display the KEYID fields
                 Screens::position_widget  ${tile_path}.lbl_keyid
                 Screens::position_widget  ${tile_path}.ent_keyid
                 Screens::position_widget  ${tile_path}.lbl_keyid_units
                 Screens::position_widget  ${tile_path}.lbl_keyid_status
                 Screens::position_widget  ${tile_path}.btn_key_loader
                
                 # Set the KEYID Status
                 set_keyid_status_core
               }
    }
    
    return
  }

  #***************************************************************************
  #
  #        Name: set_keyid_status
  #
  #  Parameters:
  #
  # Description: Sets the status of the KEYID by determining if the KEYID
  #              is in the database
  #
  #***************************************************************************
  proc set_keyid_status { args } \
  {
     set_keyid_status_core
  }

  proc set_keyid_status_core { } \
  {
    Debug::Puts Autotest 2 "set_keyid_status"
    
    variable  test_data
  
    # Get the present channel
    set chan  $test_data(channel)
                
    # Set the status to INVALID to start
    set test_data($chan,keyid_status)  "INVALID"
    
    # Determine if the KEYID is in the database
    set keyset_list  [Keys get]
    
    foreach { data_set }  $keyset_list \
    {
      Debug::Puts Autotest 2 "data_set:  $data_set"
        
      foreach { data_subset } $data_set \
      {
        Debug::Puts Autotest 2 "data_subset:  $data_subset"
          
        # Get the values of interest
        set keyid  [lindex  $data_subset  1]
        set algid  [lindex  $data_subset  2]
 
        Debug::Puts Autotest 2 "keyid:  $keyid"
        
        # If the present KEYID is in the database, 
        # display the VALID status
        # Else, the INVALID status is displayed
        set temp1 [expr $keyid]
        set temp2 "0x$test_data($chan,keyid)"
        set temp2 [expr $temp2]
        if { $temp1 == $temp2 } \
        {
          set temp1 [expr $algid]
          set temp2 "0x$test_data($chan,algid)"
          set temp2 [expr $temp2]
          if { $temp1 == $temp2 } \
          {
             set test_data($chan,keyid_status)  "VALID"
          } else \
          {
 #            puts "Hit keyid match, but missed algid = $algid, $test_data($chan,algid)"
          }
        }
      }
    }

    return
  }


  #***************************************************************************
  #
  #        Name: test
  #
  #  Parameters: tile_path - tile path of the invoking widget
  #
  # Description: Runs the test
  #
  #***************************************************************************
  proc test { tile_path } \
  {
    Debug::Puts Autotest 2 "test:  $tile_path"

    variable  test_data

    set tile_path  [Auto_Test_2::get_tile  $tile_path]

    Append Test 3 "Digital Receiver Test"

    if [Abort] \
    {
      return
    }
    
    # Setup for Test #3
    Print "Setting up for Test #3 ..."

    setup
    
    after 2000

    # Set the Generator frequency
    foreach { chan } $test_data(chan_list) \
    {
      if [Abort] \
      {
        break
      }
    
      set title  "Operator Instruction"

      set prompt_text  "Set the radio to Channel $chan\n\n"
      
      if { $test_data($chan,mode) == "SIMPLEX" } \
      {
        # Use the Digital Transmitter frequency
        append prompt_text  "\[Frequency:  $::Test_Digital_Tx::test_data($chan,freq)\]"
      } \
      else \
      {
        # Use the Digital Receiver frequency
        append prompt_text  "\[Frequency:  $::Test_Digital_Rx::test_data($chan,freq)\]"
      }

      set window_name  ${tile_path}.instruction
      
      set  prompt  [tk_dialog  $window_name  $title        \
                                             $prompt_text  \
                                             {}            \
                                             0             \
                                             "OK"          \
                                             "ABORT"]
    
      switch -- $prompt \
      {
        0 { }
      
        1 { Abort 1;  break }
      }
      
      if { $test_data($chan,mode) == "SIMPLEX" } \
      {
        # Use the Digital Transmitter frequency
        after idle "Generator freq  $::Test_Digital_Tx::test_data($chan,freq) -nowait"
      } \
      else \
      {
        # Use the Digital Receiver frequency
        after idle "Generator freq  $::Test_Digital_Rx::test_data($chan,freq) -nowait"
      }
      
      after 2000


      Format  "Transmit Channel"  "$chan"  "%s"
      
      if { $test_data($chan,mode) == "SIMPLEX" } \
      {
        # Use the Digital Transmitter frequency
        Format  "Transmit Frequency"  \
                    "$::Test_Digital_Tx::test_data($chan,freq)"  "%.6f"  "MHz"
      } \
      else \
      {
        # Use the Digital Receiver frequency
        Format  "Transmit Frequency"  \
                    "$::Test_Digital_Rx::test_data($chan,freq)"  "%.6f"  "MHz"
      }


      # Setup for subtest #1
      # Set the NAC
      Debug::Puts Autotest 2 "P25 encode field  nac  0x$test_data($chan,nac)  -nowait"

      after idle "P25 encode field  nac  0x$test_data($chan,nac)  -nowait"
      
      after 2000
      
      # Set the SID and TGID values in the LCO
      Debug::Puts Autotest 2 "P25 encode lcomsg 0 0 0 0x00 0 0x00 0x00 0x$test_data($chan,tgid) 0x$test_data($chan,sid)  -nowait"

      after idle "P25 encode lcomsg 0 0                        \
                                      0                        \
                                      0x00                     \
                                      0                        \
                                      0x00                     \
                                      0x00                     \
                                      $test_data($chan,tgid)  \
                                      0x$test_data($chan,sid)  -nowait"
      
      set temp 0x$test_data($chan,algid)
      set temp [expr $temp]
      if { ($temp == 0x81) || ($temp == 0x83) } \
      {
         # Force usage of the keys
         ::Key_LoaderUI::download_keyset
      } 

      P25 encode field  KEYID  0x$test_data($chan,keyid)  -nowait
      P25 encode field  ALGID  0x$test_data($chan,algid)  -nowait

      after 2000

      # Set the initial RF level to the user input value plus 2.0 dBm
      set rflevel  [expr $test_data($chan,rf_thrshld_min) + 2.0]
      
      Debug::Puts Autotest 2 "rflevel:  $rflevel"
      
      after idle "Generator rflevel $rflevel  -nowait"
      after idle "Generator ptt ON -nowait"

      after 2000

      # Measure Distortion
      # Start the loop to decrease the RF Level
      # Subtest #1
      set result  0.0
      
      Debug::Puts Autotest 2 "result:  $result"

      # Loop until the Distortion goes above 100%
      while { $result <= 100.0 } \
      {
        Debug::Puts Autotest 2 "result:  $result"

        set rflevel  [Generator rflevel]
        
        set rflevel  [expr $rflevel - 0.2]
        
        Debug::Puts Autotest 2 "rflevel:  $rflevel"

        after idle "Generator rflevel  $rflevel  -nowait"
        
        after 2000

        set result  [Dist value]
        
        if [Abort] \
        {
          set result  101.0
          
          break
        }
      }
      
      
      # Get the Generator level at which the Distortion increased above 100
      set result  [Generator rflevel]
      
      Debug::Puts Autotest 2 "result:  $result"

      set min  $test_data($chan,rf_thrshld_min)

      # Check the result
      if { $result <= $min } \
      {
        Format  "Sensitivity"  "$result"  "%.2f"  "dBm"  "PASS"
      } \
      else \
      {
        Format  "Sensitivity"  "$result"  "%.2f"  "dBm"  "FAIL"
      }
    }
      
    # Shut off the Generator
    after idle "Generator ptt OFF -nowait"

    return
  }


  #***************************************************************************
  #
  #        Name: setup
  #
  #  Parameters:
  #
  # Description: Sets up for the DIGITAL RX test
  #
  #***************************************************************************
  proc setup { } \
  {
    Debug::Puts Autotest 2 "setup"
    
    # Setup BBAudio
    Generator dcpower   ON          -nowait
    Generator ptt       ON          -nowait
    Generator enable    11          -nowait
    Generator freq      170.000000  -nowait
    Generator rfoutput  T/R         -nowait
    Generator rflevel   -40.0       -nowait
    Generator hwalc     869         -nowait
    Generator hwbw      NARROW      -nowait
    Generator mod       8           -nowait
    Generator p25mode   4           -nowait
    Generator yigspeed  1           -nowait
   
   
    # Setup BBAudio
    BBAudio micbias      ELEC   -nowait
    BBAudio sinadbal     BALAN  -nowait
    BBAudio demodfilter  TONE   -nowait
    BBAudio fgenfilter   TONE   -nowait
    BBAudio fgenbalance  BALAN  -nowait
    BBAudio fgenlevel    1      -nowait

    BBAudio  clearmixer  SPKR  -nowait
    BBAudio::set_bbaudio_route  SINAD  SPKR    1;  # Audio In to Speaker
    BBAudio::update_mixers [BBAudio getmixer SPKR]

    BBAudio  clearmixer  DEMOD  -nowait
    BBAudio::update_mixers [BBAudio getmixer DEMOD]

    BBAudio  clearmixer  FGEN  -nowait
    BBAudio::update_mixers [BBAudio getmixer FGEN]

    BBAudio  clearmixer  L1  -nowait
    BBAudio::update_mixers [BBAudio getmixer L1]
    
    BBAudio  clearmixer  USER0  -nowait
    BBAudio::update_mixers [BBAudio getmixer USER0]

    BBAudio  clearmixer  USER1  -nowait
    BBAudio::update_mixers [BBAudio getmixer USER1]


    # Setup Sinad
    Sinad run     1        -nowait
    Sinad math    0        -nowait
    Sinad trig    2        -nowait
    Sinad avg     10       -nowait
    Sinad filter  1        -nowait
    Sinad en      3        -nowait
    Sinad peak    0        -nowait
    Sinad alerts  0        -nowait
    Sinad ulim    0.00     -nowait
    Sinad llim    0.00     -nowait
    Sinad encwt   OFF      -nowait
    Sinad rate    4        -nowait
    Sinad minx    0        -nowait
    Sinad maxx    20000    -nowait
    Sinad miny    -80.0    -nowait
    Sinad maxy    0.0      -nowait
    Sinad size    2048     -nowait
    Sinad alpha   2.6      -nowait
    Sinad window  1        -nowait
    Sinad lnotch  900.0    -nowait
    Sinad hnotch  1100.0   -nowait
    Sinad lnoise  100.0    -nowait
    Sinad hnoise  10000.0  -nowait
    
    catch { Sinad  log  0  -nowait }
    
    Dist peak    0     -nowait
    Dist alerts  0     -nowait
    Dist ulim    0.00  -nowait
    Dist llim    0.00  -nowait
    
    set ::Sinad::State(Sinad.BellEnabled)   {0}
    set ::Sinad::State(Sinad.resolution)    {1.0}
    set ::Sinad::State(Dist.BellEnabled)    {0}
    set ::Sinad::State(Dist.resolution)     {1.0}
    
    Sinad::Input {Aud Bal}
    
    
    # Setup Function Generator
    FGen::setMode  1  OFF
    FGen::setMode  2  OFF
    FGen::setMode  3  OFF
    FGen::setMode  4  OFF

    # Version 1.6.0 compatibility
    catch { set GeneratorUI::fgen_mode(3)   "OFF" } result
    catch { set GeneratorUI::fgen_mode(4)   "OFF"  } result
    
    
    # Setup P25ns
    # Restore P25 mode variables
    P25 decode samplerate  64   -nowait
    P25 decode enable      3    -nowait
    P25 decode run         0    -nowait
    P25 encode encrypt     0 0  -nowait
    
    if { [IFR_options is_installed P25_trunk] || [IFR_options is_installed P25_OBT] } \
    {
      after idle "::P25ns::EnableTrunking  Disable"
    }
    
    set ::P25ns::TrunkModeCurrent  800
    
    # Restore variables used for both P25 and P25 trunking
    P25 encode field rfsubid    0x01
    P25 encode field siteid     0x01
    P25 encode field lra        0x01
    P25 encode field sclass     0x60
    
    # Restore settings for P25 trunking variables
    P25 encode field WACNID      0x00001
    P25 encode field WUID        0x000001
    P25 encode field WGID        0x0001
    P25 encode field CHANID_C    0x0
    P25 encode field CHANNUM_C   0x000
    P25 encode field CHANID_V    0x0
    P25 encode field CHANNUM_V   0x032
    P25 encode field ANNGRPADR   0x0000
    
    P25 encode field T_PROT      0
    P25 encode field T_ANET      1
    P25 encode field T_LG        0
    P25 encode field T_GAV       0
    P25 encode field T_RV        0
    
    # Set up the LCO message saves
    # These selected LCO messages save all of the user enterable fields
    P25 encode lcomsg  0  0x0  0x0  0x00  0x0  0x00  0x0  0x0001  0x000000           -nowait
    P25 encode lcomsg  2  0x0  0x0  0x0  0x000  0x0000  0x0  0x000  0x0000           -nowait
    P25 encode lcomsg  3  0x0  0x0  0x00  0x0  0x00  0x000000  0x000000              -nowait
    P25 encode lcomsg  4  0x0  0x0  0x0  0x0  0x00  0x0001  0x0  0x000  0x0  0x000   -nowait
    P25 encode lcomsg  6  0x0  0x0  0x0  0x0  0x00  0x0  0x0000  0x000000            -nowait
    P25 encode lcomsg  17  0x0  0x0  0x00000  0x001  0x000000  0x0                   -nowait
    P25 encode lcomsg  20  0x0  0x0  0x0000  0x000000  0x000000                      -nowait
    P25 encode lcomsg  21  0x0  0x0  0x0000  0x000000  0x000000                      -nowait
    P25 encode lcomsg  23  0x0  0x0  0x0000  0x000000  0x000000                      -nowait
    P25 encode lcomsg  24  0x0  0x0  0x0  0x000  0x000  0x000  0x00000000            -nowait
    P25 encode lcomsg  32  0x0  0x0  0x0  0x0  0x0  0x000000  0x000000               -nowait
    P25 encode lcomsg  33  0x0  0x0  0x01  0x01  0x0  0x000  0x00  0x0  0x000  0x00  -nowait
    P25 encode lcomsg  34  0x0  0x0  0x01  0x0  0x001  0x01  0x01  0x0  0x000  0x60  -nowait
    P25 encode lcomsg  37  0x0  0x0  0x0  0x00  0x0000  0x000000                     -nowait
    
    # Give them time to take effect
    update idle
    
    # Restore P25 header information 
    # using the temporary values saved above for P, SF, and LCO
    P25 encode sendarray 0x000  0x00  0x0000  0x0000  0x0000  0x00  0x000000  0x000000  \
                                                           0x0  0x0  0x0  0x0000  0x0 -nowait
    
    P25 encode digits  0000000000 -nowait

    P25 encode field  ALGID  0x0  -nowait
    
    return
  }


  #***************************************************************************
  #
  #        Name: get_config_summary
  #
  #  Parameters:
  #
  # Description: Returns the a formatted summary of the present 
  #              configuration
  #
  #***************************************************************************
  proc get_config_summary { } \
  {
    Debug::Puts Autotest 2 "get_config_summary"
    
    variable  test_data
    
    set return_list  {}

    append return_list  "**** Digital Receiver Test Configuration **** \n\n"
      
    if { $test_data(chan_list) == "" } \
    {
      append return_list  "  NO CONFIGURATION DATA \n"
    }

    foreach { chan } $test_data(chan_list) \
    {
      append return_list  "  Setup   Channel:     $chan \n"
      append return_list  "          Freq:        $test_data($chan,freq)    \n"
      append return_list  "          Mode:        $test_data($chan,mode)    \n"
      append return_list  "          NAC:         $test_data($chan,nac)     \n"
      append return_list  "          SID:         $test_data($chan,sid)     \n"
      append return_list  "          TGID:        $test_data($chan,tgid)    \n"
      append return_list  "          Encryption:  $test_data($chan,encrypt) \n"

      switch -- $test_data($chan,encrypt) \
      {
         "DES" -
         "AES"\
         {
           append return_list  "          KEYID:       $test_data($chan,keyid)   \n"
         }
      }
      
      append return_list  "\n"

      append return_list  "  Pass/Fail   RF Threshold \n"
      append return_list  "                min:  $test_data($chan,rf_thrshld_min) \n"
    }

    append return_list  " \n\n"

    return  [subst -nocommands $return_list]
  }


  #***************************************************************************
  #
  #        Name: get_html_config
  #
  #  Parameters:
  #
  # Description: Returns the an HTML formatted configuration display
  #
  #***************************************************************************
  proc get_html_config { } \
  {
    Debug::Puts Autotest 2 "get_html_config"
    
    variable  test_data_html
    
    set return_list  {}

    append return_list  "<B>DIGITAL RECEIVER TEST CONFIGURATION</B> <BR><BR>"
    
    if { $test_data_html(chan_list) == "" } \
    {
      append return_list  "  NO CONFIGURATION DATA <BR>"
    }

    foreach { chan } $test_data_html(chan_list) \
    {
      append return_list  "  Setup   Channel:     $chan <BR>"
      append return_list  "          Freq:        $test_data_html($chan,freq)    <BR>"
      append return_list  "          Mode:        $test_data_html($chan,mode)    <BR>"
      append return_list  "          NAC:         $test_data_html($chan,nac)     <BR>"
      append return_list  "          SID:         $test_data_html($chan,sid)     <BR>"
      append return_list  "          TGID:        $test_data_html($chan,tgid)    <BR>"
      append return_list  "          Encryption:  $test_data_html($chan,encrypt) <BR>"

      swtich $test_data_html($chan,encrypt) \
      {
         "DES" -
         "AES" \
         {
            append return_list  "          KEYID:       $test_data_html($chan,keyid)   <BR>"
         }
      }
      
      append return_list  "<BR>"

      append return_list  "  Pass/Fail   RF Threshold <BR>"
      append return_list  "                min:  $test_data_html($chan,rf_thrshld_min) <BR>"
    }

    append return_list  "<BR><BR>"

    return  $return_list
  }


  #***************************************************************************
  #
  #        Name: get_config_data
  #
  #  Parameters:
  #
  # Description: Returns the present configuration
  #
  #***************************************************************************
  proc get_config_data { } \
  {
    Debug::Puts Autotest 2 "get_config_data"
    
    variable  test_data
    
    set return_list  {}
    
    foreach { array_index } [array names  test_data] \
    {
      append return_list  "set ::Test_Digital_Rx::test_data($array_index)  {$test_data($array_index)} \n"
    }

    return  [subst -nocommands $return_list]
  }


  #***************************************************************************
  #
  # Register this test by adding the namespace to the Auto Test list
  #
  #***************************************************************************
  if { [IFR_options  is_installed  Auto_Test] } \
  {
    lappend  ::Auto_Test_2::test_namespace_list  "Test_Digital_Rx"
  }


};  # End of namespace Test_Digital_Rx
