#***************************************************************************
#
# Test procedure:  Test #5:  MDC-1200 Emergency Test
#
#***************************************************************************


namespace eval Test_MDC_1200 \
{
  variable  short_name  "mdc_1200"
  
  variable  long_name   "MDC-1200 Emergency Test"
  
  variable  tile_class  "TEST_MDC_1200"

  variable  test_data
  
  set test_data(chan_list)  {}

  set test_data(channel)  1
  
  namespace import ::Auto_Test_2::*
  
  
  #***************************************************************************
  #
  #        Name: Create
  #
  #  Parameters: parent - parent of child window
  #
  # Description: Creates the tile
  #
  #***************************************************************************
  proc Create { parent } \
  {
    #puts "Test_MDC_1200::Create:  $parent"

    variable  short_name
    variable  tile_class

    set tile_path  [name_child  $parent  $short_name]
    
    if { ![winfo exists $tile_path] } \
    {
      set tile_path  [Screens::create_tile  $tile_path  $short_name  $tile_class]
    }  
     
    return  $tile_path
  }


  #***************************************************************************
  #
  #        Name: Map
  #
  #  Parameters: 
  #
  # Description: Initializes the tile
  #
  #***************************************************************************
  proc Map { tile_path } \
  {
    #puts "::Test_MDC_1200::Map:  $tile_path"
    
    #set tile_path  [get_tile  $tile_path]
    
    # Set the configuration data
    set_channel_data  $tile_path
    
    return
  }


  #***************************************************************************
  #
  #        Name: Cleanup
  #
  #  Parameters: 
  #
  # Description: 
  #
  #***************************************************************************
  proc Cleanup { tile_path } \
  {
    #puts "::Test_MDC_1200::Cleanup:  $tile_path"

    #set tile_path  [get_tile  $tile_path]
    
    return
  }


  #***************************************************************************
  #
  #        Name: Unmap
  #
  #  Parameters: 
  #
  # Description: 
  #
  #***************************************************************************
  proc Unmap { tile_path } \
  {
    #puts "::Test_MDC_1200::Unmap:  $tile_path"

    #set tile_path  [get_tile  $tile_path]

    Cleanup  $tile_path
    
    return
  }


  #***************************************************************************
  #
  #        Name: Destroy
  #
  #  Parameters: 
  #
  # Description: 
  #
  #***************************************************************************
  proc Destroy { tile_path } \
  {
    #puts "::Test_MDC_1200::Destroy:  $tile_path"

    #set tile_path  [get_tile  $tile_path]

    Cleanup  $tile_path

    return
  }

  #***************************************************************************
  #
  #        Name: set_channel_data
  #
  #  Parameters: tile_path  - tile path of the invoking widget
  #
  # Description: Sets the text variables of the configuration display 
  #              depending on the channel number
  #
  #***************************************************************************
  proc set_channel_data { tile_path } \
  {
    #puts "set_channel_data:  $tile_path"
  
    variable  test_data

    set tile_path  [get_tile  $tile_path]
    
    set tile_path  ${tile_path}.frm_config.mdc_1200

    set chan  $test_data(channel)
      
    # Determine if the present channel number is in the channel list.
    # If it is not, append the new value and then sort the list in
    # increasing order.
    if { [lsearch $test_data(chan_list) $chan] == -1 } \
    {
      lappend test_data(chan_list)  $chan
      
      if [oldCatch { set tempList  [lsort -increasing -integer $test_data(chan_list)]} err] \
      {
         set tempList  [lsort -increasing $test_data(chan_list)]
      }
      set test_data(chan_list) $tempList
    }
      
    # Configure the Frequency entry
      if { ![info exists test_data($chan,freq)] } \
      {
        set test_data($chan,freq)  "0.000000"
      }
    
      ${tile_path}.ent_freq  config  \
                          -textvariable ::Test_MDC_1200::test_data($chan,freq)

    # Configure the Power max and min entries
      if { ![info exists test_data($chan,power_max)] } \
      {
        set test_data($chan,power_max)  "0.00"
      }
    
      ${tile_path}.ent_power_max  config  \
                     -textvariable ::Test_MDC_1200::test_data($chan,power_max)
                 
      if { ![info exists test_data($chan,power_min)] } \
      {
        set test_data($chan,power_min)  "0.00"
      }
    
      ${tile_path}.ent_power_min  config  \
                     -textvariable ::Test_MDC_1200::test_data($chan,power_min)

    return
  }


  #***************************************************************************
  #
  #        Name: toggle_channel_mode
  #
  #  Parameters: 
  #
  # Description: Toggles the mode for the present channel
  #
  #***************************************************************************
  proc toggle_channel_mode { } \
  {
    #puts "toggle_channel_mode"
    
    variable  test_data
  
    # Get the present channel
    set channel  $test_data(channel)
                
    # Toggle the mode
    if { $test_data($channel,mode) == "SIMPLEX" } \
    {
      set test_data($channel,mode)  "DUPLEX"
    } \
    else \
    {
      set test_data($channel,mode)  "SIMPLEX"
    }
          
    return
  }


  #***************************************************************************
  #
  #        Name: test
  #
  #  Parameters: tile_path - tile path of the invoking widget
  #
  # Description: Runs the test
  #
  #***************************************************************************
  proc test { tile_path } \
  {
    #puts "test:  $tile_path"
    
    variable  test_data
    
    set tile_path  [::Auto_Test_2::get_tile  $tile_path]

    Append Test 5 "MDC-1200 Emergency Test"

    if [Abort] \
    {
      return
    }
    
    # Setup for Test #5
    setup
    
    after 2000

    # Set the Generator frequency
    foreach { chan } $test_data(chan_list) \
    {
      if [Abort] \
      {
        return
      }
    
      ## Zero the Power meter
      #after idle "Power zero"
      #
      #after 2000
      
      set title  "Operator Instruction"

      set prompt_text  "Press the Emergency button then press OK"
    
      set window_name  ${tile_path}.instruction
    
      # Dialog:  no bitmap displayed, OK and ABORT buttons
      set  prompt  [tk_dialog  $window_name  $title        \
                                             $prompt_text  \
                                             {}            \
                                             0             \
                                             "OK"          \
                                             "ABORT"]
    
      switch -- $prompt \
      {
        0 { }
    
        1 { Abort 1;  return }
      }
      
      after idle "Receiver freq  $test_data($chan,freq)  -forgive"

      after 2000
      

      Format  "Monitor Channel"  "$chan"  "%s"

      Format  "Monitor Frequency"  "$test_data($chan,freq)"  "%.6f"  "MHz"


      # Setup for subtest #1
      # Measure Power

      # Subtest #1
      set result  [Power value]
      
      set max  $test_data($chan,power_max)
      set min  $test_data($chan,power_min)
      
      # Check the result
      if { $result >= $min && $result <= $max } \
      {
        Format  "Power Level"  "$result"  "%.2f"  "W"  "PASS"
      } \
      else \
      {
        Format  "Power Level"  "$result"  "%.2f"  "W"  "FAIL"
      }

    }

    return
  }


  #***************************************************************************
  #
  #        Name: setup
  #
  #  Parameters:
  #
  # Description: Sets up for the MDC 1200 test
  #
  #***************************************************************************
  proc setup { } \
  {
    #puts "setup"
    
    # Setup Receiver
    Receiver freq          170.000000  -nowait
    Receiver rfinput       0           -nowait
    Receiver atten         10          -nowait
    Receiver ifbw          30000       -nowait
    Receiver demod         10          -nowait
    Receiver range         WIDE        -nowait
    Receiver hwagc         1           -nowait
    Receiver agcmode       1           -nowait
    Receiver filter        7           -nowait
    Receiver squelchmode   1           -nowait
    Receiver squelchlevel  216         -nowait
    Receiver lock          1           -nowait
    
    Receiver::Config_agc   300  -10.5  -9.5  4
    
    
    # Setup Power
    Power enable    255   -forgive
    Power peakhold  0     -forgive
    Power alerts    0     -forgive
    Power ulimit    0.0   -forgive
    Power llimit    0.0   -forgive
    Power cable     0.00  -forgive
    Power prange    0     -forgive
    
    set ::Power::State(BellEnabled) {0}
    
   
    # Setup BBAudio
    BBAudio micbias      ELEC   -forgive
    BBAudio sinadbal     BALAN  -forgive
    BBAudio demodfilter  TONE   -forgive
    BBAudio fgenfilter   TONE   -forgive
    BBAudio fgenbalance  BALAN  -forgive
    BBAudio fgenlevel    0      -forgive

    BBAudio  clearmixer  SPKR  -forgive
    BBAudio::update_mixers [BBAudio  getmixer  SPKR]

    BBAudio  clearmixer  DEMOD  -forgive
    BBAudio::update_mixers [BBAudio  getmixer  DEMOD]

    BBAudio  clearmixer  FGEN  -forgive
    BBAudio::update_mixers [BBAudio  getmixer  FGEN]

    BBAudio  clearmixer  L1  -forgive
    BBAudio::update_mixers [BBAudio  getmixer  L1]
    
    BBAudio  clearmixer  USER0  -forgive
    BBAudio::update_mixers [BBAudio  getmixer  USER0]

    BBAudio  clearmixer  USER1  -forgive
    BBAudio::update_mixers [BBAudio  getmixer  USER1]


    # Setup RFError
    RFError run     1     -forgive
    RFError gate    0     -forgive
    RFError avg     20    -forgive
    RFError en      3     -forgive
    RFError peak    0     -forgive
    RFError alerts  0     -forgive
    RFError ulim    0.00  -forgive
    RFError llim    0.00  -forgive
    
    set ::RFError::State(BellEnabled)   {0}
    set ::RFError::State(resolution)    {1.0}
    set ::RFError::State(units)         {Hz}
    
    
    # Setup FGen
    FGen::setMode  1  OFF
    FGen::setMode  2  OFF
    FGen::setMode  3  OFF
    FGen::setMode  4  OFF

    return
  }


  #***************************************************************************
  #
  #        Name: get_config_summary
  #
  #  Parameters:
  #
  # Description: Returns the a formatted summary of the present 
  #              configuration
  #
  #***************************************************************************
  proc get_config_summary { } \
  {
    #puts "get_config_summary"
    
    variable  test_data
    
    set return_list  {}

    append return_list  "**** MDC-1200 Emergency Test Configuration **** \n\n"
      
    if { $test_data(chan_list) == "" } \
    {
      append return_list  "  NO CONFIGURATION DATA \n"
    }

    foreach { chan } $test_data(chan_list) \
    {
      append return_list  "  Setup   Channel:  $chan \n"
      append return_list  "          Freq:     $test_data($chan,freq) \n"
      
      append return_list  "\n"
      
      append return_list  "  Pass/Fail   Power \n"
      append return_list  "                max:  $test_data($chan,power_max) \n"
      append return_list  "                min:  $test_data($chan,power_min) \n"
    }

    append return_list  " \n\n"

    return  [subst -nocommands $return_list]
  }


  #***************************************************************************
  #
  #        Name: get_html_config
  #
  #  Parameters:
  #
  # Description: Returns the an HTML formatted configuration display
  #
  #***************************************************************************
  proc get_html_config { } \
  {
    #puts "get_html_config"
    
    variable  test_data_html
    
    set return_list  {}

    append return_list  "<B>MDC-1200 EMERGENCY TEST CONFIGURATION</B> <BR><BR>"

    if { $test_data_html(chan_list) == "" } \
    {
      append return_list  "  NO CONFIGURATION DATA <BR>"
    }

    foreach { chan } $test_data_html(chan_list) \
    {
      append return_list  "  Setup   Channel:  $chan <BR>"
      append return_list  "          Freq:     $test_data_html($chan,freq) <BR>"
      
      append return_list  "<BR>"
      
      append return_list  "  Pass/Fail   Power <BR>"
      append return_list  "                max:  $test_data_html($chan,power_max) <BR>"
      append return_list  "                min:  $test_data_html($chan,power_min) <BR>"
    }

    append return_list  "<BR><BR>"

    return  $return_list
  }


  #***************************************************************************
  #
  #        Name: get_config_data
  #
  #  Parameters:
  #
  # Description: Returns the present configuration
  #
  #***************************************************************************
  proc get_config_data { } \
  {
    #puts "get_config_data"
    
    variable  test_data
    
    set return_list  {}
    
    foreach { array_index } [array names  test_data] \
    {
      append return_list  "set ::Test_MDC_1200::test_data($array_index)  {$test_data($array_index)} \n"
    }

    return  [subst -nocommands $return_list]
  }


  #***************************************************************************
  #
  # Register this test by adding the namespace to the Auto Test list
  #
  #***************************************************************************
  if { [IFR_options  is_installed  Auto_Test] } \
  {
    lappend  ::Auto_Test_2::test_namespace_list  "Test_MDC_1200"
  }


};  # End of namespace Test_MDC_1200
