#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C) 2001      DESIGN ENGINEERING        #
# --------------------------------------------------------------------------#
# FILE        : xxxx.SampleScreens.tcl                                      #
# PRODUCT     : FITS                                                        #
# AUTHOR      : Bruce Butterfield                                           #
# DESCRIPTION :                                                             #
#############################################################################
#                                                                           #
# CRITICAL ISSUES:                                                          #
#    o Do not configure hardware when this file is sourced. We would like   #
#      to source screens while the DSPs are loading.                        #
#                                                                           #
#############################################################################
#                                                                           #
# Required Variables:                                                       #
#    LongName		Name of screen                                      #
#    ShortName          Name of screen for menus                            #
#    RecomendedTiles    List of tiles available for this screen. It is not  #
#                       necessary to put a tile on this list that is on the #
#                       RequiredTiles list                                  #
#    NotAllowedTiles    List of tiles NEVER on this screen                  #
#    RequiredTiles      List of tiles ALWAYS on this screen                 #
#    ActiveTiles        List of tiles currently displayed on this screen    #
#    MenuFlags          0 = Active Screen                                   #
#                       1 = Currently functioning as a submenu parent       #
#                                                                           #
# Required Variables for submenu "Screens":                                 #
#    SubmenuTiles       List of all tiles (or submenus) for droplist        #
#    SubDirPath         Path to subdirectory holding the screens for the    #
#                       submenu of this "screen"                            #
#                                                                           #
#############################################################################
#                                                                           #
# Required Functions:                                                       #
#    Setup              Called by the screen manager to create the screen.  #
#    Map                Called when screen is mapped on window              #
#    Unmap              Called when screen is removed from display          #
#    Destroy            Called when screen is destroyed                     #
#                                                                           #
#############################################################################

namespace eval TestA_Screen {
  variable LongName
  variable ShortName
  variable RecommendedTiles
  variable NotAllowedTiles
  variable RequiredTiles
  variable ActiveTiles
  variable MenuFlags
  variable SubmenuTiles
  variable SubDirPath

  set LongName testA_Screen
  set ShortName Scrn_A
  set RequiredTiles [list Quarter_A Eighth_AV Eighth_CV ]
  set NotAllowedTiles { }
  set RecommendedTiles [list Quarter_B Eighth_AH Eighth_BH Eighth_CH Eighth_BV Counter_small_tile ]
  set ActiveTiles { }
  set MenuFlags 0
  set SubmenuTiles { }
  set SubDirPath /system/screens/2090.System


#****************************************************************************
# Name:        Setup
# Parameters:  parent		parent of child window
# Description: Called by the screen manager to create and display a screen
# Returns:     Name of the screen tile to pack into the instrument area
#****************************************************************************
# Design Concepts:
#    o Screens made up of a collection of subtiles can use the ::Screens::
#      utility functions to manage the tile display and options menu. They
#      should return an empty string
#    o Screens which which to personally manage the screen should create a 
#      frame which can be packed into the main instrument area, fill that frame
#      and return the name of the frame.
#    o If hardware setup is required for a screen, put the setup in a "proc"
#      and activate the proc in an "after idle" call from the "Map" function
#      to allow a fast screen change
#****************************************************************************
# CRITICAL DON'Ts:
#    o Do not "configure" the "parent" window
#    o The parameter "parent" should ONLY be used to determine the toplevel.
#      Do NOT create frames or widgets as children of "parent"
#****************************************************************************
   proc Setup { parent } {

     # Use the Screen functions to manage our tiles
     set topWin [winfo toplevel $parent]
     ::Screens::ShowRequiredTiles $topWin TestA_Screen

     # Use the Screen functions to manage our options menu
     set dropMenu [::Screens::Build_std_options $topWin TestA_Screen]
     ::View::PackOptions $topWin $dropMenu

     # We are not generating a tile to pack into the main screen to return NULL
     return ""
  }

#****************************************************************************
# Name:        Map
# Parameters:  parent		parent of child window
# Description: Called by the screen manager when the screen is displayed. Can
#              be used to trigger hardware setup using "after idle"
#****************************************************************************
  proc Map { name } { 
  }

#****************************************************************************
# Name:        Unmap
# Parameters:  parent		parent of child window
# Description: Called by the screen manager when the screen is hidden. Might 
#              be used to throttle down unused hardware, but this could cause
#              an interaction with the Mapped display
#****************************************************************************
  proc Unmap { name } { }

#****************************************************************************
# Name:        Destroy
# Parameters:  parent		parent of child window
# Description: Called when the screen is destroyed. Handle any object and
#              hardware cleanup.
#****************************************************************************
  proc Destroy { name } { }

#****************************************************************************
# Source Loading Functions:
#****************************************************************************
  # Commented out so that the screen does not register
  #::View::RegisterScreen TestA_Screen

}  ;# End of the TestA_Screen


