#############################################################################
# COPYRIGHT (C) 2001, 2003, 2005  Aeroflex, Inc.                            #
# --------------------------------------------------------------------------#
# FILE        : 1001.test_Afctr.tcl                                         #
# PRODUCT     : FITS                                                        #
# AUTHOR      : Bruce Butterfield, Jon Butterfield                          #
# DESCRIPTION : Remote Command Line Test: AF Counter                        #
# --------------------------------------------------------------------------#
# Update      : Resynch and engine cleanup for 1.9.1.4                      #
#############################################################################

package require AFCounter
package require BIT

#****************************************************************************
# Test variables
#****************************************************************************
namespace eval ::cmdline_AFCounter {
    variable testCaseList		;# List of test cases
    variable shortDesc			;# Test description - menu size
    variable longDesc			;# Test description - used by reports
    variable testTime			;# Test duration (Seconds - approximation)

    # Define the list of AF Counter tests
    set testCaseList [list \
           testAfctrList1 testAfctrList2 testAfctrList3 testAfctrList4 \
           testAfctrList5 testAfctrList6 testAfctrList7 testAfctrList8 \
           testAfctrList9 testAfctrList10]

    # Declare each test as a namespace variable
    foreach testName $testCaseList {
       variable $testName
    }

    set shortDesc "AFCTR cmds"
    set longDesc "AFCounter command line test"
    set testTime 6

#****************************************************************************
# Name:        setup
# Parameters:  none
# Description: Sets up a list of AF Counter tests
#****************************************************************************
 proc setup { } {
    variable testCaseList

    # Declare each test as a namespace variable
    foreach testName $testCaseList {
       variable $testName
    }

    #
    # Define test cases
    #

#Test alerts
    set testAfctrList1 [list                     \
      {"AFCounter al -1 -forgive" 0 "INT" 0}     \
      {"AFCounter al 0" 0 "INT" 0}               \
      {"AFCounter al 1" 1 "INT" 0}               \
      {"AFCounter al 2" 2 "INT" 0}               \
      {"AFCounter al 3" 3 "INT" 0}               \
      {"AFCounter al 7" 7 "INT" 0}               \
      {"AFCounter al 8 -forgive" 0 "INT" 0}      \
      {"AFCounter alerts -1 -forgive" 0 "INT" 0} \
      {"AFCounter alerts 1" 1 "INT" 0}           \
      {"AFCounter alerts 0" 0 "INT" 0}           \
      {"AFCounter alerts 2" 2 "INT" 0}           \
      {"AFCounter alerts 3" 3 "INT" 0}           \
      {"AFCounter alerts 7" 7 "INT" 0}           \
      {"AFCounter alerts off" 0 "INT" 0}         \
      {"AFCounter alerts enablelo" 1 "INT" 0}    \
      {"AFCounter alerts enablehi" 2 "INT" 0}    \
      {"AFCounter alerts enableboth" 3 "INT" 0}  \
      {"AFCounter alerts 8 -forgive" 0 "INT" 0}]

#Test average
    set testAfctrList2 [list                         \
      {"AFCounter avg 0 -forgive" 1 "INT" 0}         \
      {"AFCounter avg 10" 10 "INT" 0}                \
      {"AFCounter avg 99" 99 "INT" 0}                \
      {"AFCounter avg 100" 100 "INT" 0}              \
      {"AFCounter avg 101 -forgive" 101 "INT" 0}     \
      {"AFCounter avg 199" 199 "INT" 0}              \
      {"AFCounter avg 200" 200 "INT" 0}              \
      {"AFCounter avg 201 -forgive" 200 "INT" 0}     \
      {"AFCounter average 0 -forgive" 1 "INT" 0}     \
      {"AFCounter average 10" 10 "INT" 0}            \
      {"AFCounter average 99" 99 "INT" 0}            \
      {"AFCounter average 100" 100 "INT" 0}          \
      {"AFCounter average 101 -forgive" 101 "INT" 0} \
      {"AFCounter average 199" 199 "INT" 0}          \
      {"AFCounter average 200" 200 "INT" 0}          \
      {"AFCounter average 201 -forgive" 200 "INT" 0}]

#Test enable
    set testAfctrList3 [list                      \
      {"AFCounter en -1 -forgive" 3 "INT" 0}      \
      {"AFCounter en 0" 1 "INT" 0}                \
      {"AFCounter en 1" 1 "INT" 0}                \
      {"AFCounter en 2" 3 "INT" 0}                \
      {"AFCounter en 3" 3 "INT" 0}                \
      {"AFCounter en 4 -forgive" 1 "INT" 0}       \
      {"AFCounter en 5 -forgive" 1 "INT" 0}       \
      {"AFCounter en 6 -forgive" 3 "INT" 0}       \
      {"AFCounter en 7 -forgive" 3 "INT" 0}       \
      {"AFCounter en 8 -forgive" 1 "INT" 0}       \
      {"AFCounter enable -1 -forgive" 3 "INT" 0}  \
      {"AFCounter enable 0" 1 "INT" 0}            \
      {"AFCounter enable 1" 1 "INT" 0}            \
      {"AFCounter enable 2" 3 "INT" 0}            \
      {"AFCounter enable 3" 3 "INT" 0}            \
      {"AFCounter enable disable" 1 "INT" 0}      \
      {"AFCounter enable enable" 3 "INT" 0}       \
      {"AFCounter enable off" 1 "INT" 0}          \
      {"AFCounter enable on" 3 "INT" 0}           \
      {"AFCounter enable 4 -forgive" 1 "INT" 0}   \
      {"AFCounter enable 5 -forgive" 1 "INT" 0}   \
      {"AFCounter enable 6 -forgive" 3 "INT" 0}   \
      {"AFCounter enable 7 -forgive" 3 "INT" 0}   \
      {"AFCounter enable 8 -forgive" 1 "INT" 0}   \
      {"AFCounter en2 -1 -forgive" 5 "INT" 0}     \
      {"AFCounter en2 0" 1 "INT" 0}               \
      {"AFCounter en2 1" 1 "INT" 0}               \
      {"AFCounter en2 2 -forgive" 1 "INT" 0}      \
      {"AFCounter en2 3 -forgive" 1 "INT" 0}      \
      {"AFCounter en2 4" 5 "INT" 0}               \
      {"AFCounter en2 5" 5 "INT" 0}               \
      {"AFCounter en2 6 -forgive" 5 "INT" 0}      \
      {"AFCounter en2 7 -forgive" 5 "INT" 0}      \
      {"AFCounter en2 8 -forgive" 1 "INT" 0}      \
      {"AFCounter enable2 -1 -forgive" 5 "INT" 0} \
      {"AFCounter enable2 0" 1 "INT" 0}           \
      {"AFCounter enable2 1" 1 "INT" 0}           \
      {"AFCounter enable2 2 -forgive" 1 "INT" 0}  \
      {"AFCounter enable2 3 -forgive" 1 "INT" 0}  \
      {"AFCounter enable2 4" 5 "INT" 0}           \
      {"AFCounter enable2 5" 5 "INT" 0}           \
      {"AFCounter enable2 6 -forgive" 5 "INT" 0}  \
      {"AFCounter enable2 7 -forgive" 5 "INT" 0}  \
      {"AFCounter enable2 8 -forgive" 1 "INT" 0}]

#Test gate
    set testAfctrList4 [list                   \
      {"AFCounter gate -1 -forgive" 0 "INT" 0} \
      {"AFCounter gate 1" 1 "INT" 0}           \
      {"AFCounter gate 0" 0 "INT" 0}           \
      {"AFCounter gate 1" 1 "INT" 0}           \
      {"AFCounter gate tenth" 0 "INT" 0}       \
      {"AFCounter gate second" 1 "INT" 0}      \
      {"AFCounter gate 2 -forgive" 0 "INT" 0} ]

#Test lower limit
    set testAfctrList5 [list                                    \
      {"AFCounter llim -1 -forgive" 0.0 "DBL" 0.001}            \
      {"AFCounter llim 1234.5" 1234.5 "DBL" 0.001}              \
      {"AFCounter llim 9876.5" 9876.5 "DBL" 0.001}              \
      {"AFCounter llim 20000.0" 20000.0 "DBL" 0.001}            \
      {"AFCounter llim 39999.9" 39999.9 "DBL" 0.001}            \
      {"AFCounter llim 40000.0" 40000.0 "DBL" 0.001}            \
      {"AFCounter llim 0.0" 0.0 "DBL" 0.001}                    \
      {"AFCounter llim 40000.5 -forgive" 40000.0 "DBL" 0.001}   \
      {"AFCounter llim 9876.5 -forgive" 9876.5 "DBL" 0.001}     \
      {"AFCounter llimit -1 -forgive" 0.0 "DBL" 0.001}          \
      {"AFCounter llimit 1234.5" 1234.5 "DBL" 0.001}            \
      {"AFCounter llimit 9876.5" 9876.5 "DBL" 0.001}            \
      {"AFCounter llimit 0.0200000 mhz" 20000.0 "DBL" 0.001}    \
      {"AFCounter llimit 39.9999 khz" 39999.9 "DBL" 0.001}      \
      {"AFCounter llimit 40000.0" 40000.0 "DBL" 0.001}          \
      {"AFCounter llimit 0.0" 0.0 "DBL" 0.001}                  \
      {"AFCounter llimit 40000.5 -forgive" 40000.0 "DBL" 0.001} \
      {"AFCounter llimit 9876.5 -forgive" 9876.5 "DBL" 0.001}]

#Test peak hold
    set testAfctrList6 [list                       \
      {"AFCounter peak -1 -forgive" 0 "INT" 0}     \
      {"AFCounter peak 1" 1 "INT" 0}               \
      {"AFCounter peak 0" 0 "INT" 0}               \
      {"AFCounter peak 1" 1 "INT" 0}               \
      {"AFCounter peak off" 0 "INT" 0}             \
      {"AFCounter peak on" 1 "INT" 0}              \
      {"AFCounter peak clr" 0 "INT" 0}             \
      {"AFCounter peak set" 1 "INT" 0}             \
      {"AFCounter peak 2 -forgive" 1 "INT" 0}      \
      {"AFCounter peakhold -1 -forgive" 0 "INT" 0} \
      {"AFCounter peakhold 1" 1 "INT" 0}           \
      {"AFCounter peakhold 0" 0 "INT" 0}           \
      {"AFCounter peakhold 1" 1 "INT" 0}           \
      {"AFCounter peakhold 2 -forgive" 1 "INT" 0}]

#Test run
    set testAfctrList7 [list                   \
      {"AFCounter run -1 -forgive" 0 "INT" 0}  \
      {"AFCounter run 1" 1 "INT" 0}            \
      {"AFCounter run 0" 0 "INT" 0}            \
      {"AFCounter run on" 1 "INT" 0}           \
      {"AFCounter run off" 0 "INT" 0}          \
      {"AFCounter run 4 -forgive" 1 "INT" 0}   \
      {"AFCounter run1 -1 -forgive" 0 "INT" 0} \
      {"AFCounter run1 1" 1 "INT" 0}           \
      {"AFCounter run1 0" 0 "INT" 0}           \
      {"AFCounter run1 on" 1 "INT" 0}          \
      {"AFCounter run1 off" 0 "INT" 0}         \
      {"AFCounter run1 4 -forgive" 1 "INT" 0}  \
      {"AFCounter run2 -1 -forgive" 0 "INT" 0} \
      {"AFCounter run2 1" 1 "INT" 0}           \
      {"AFCounter run2 0" 0 "INT" 0}           \
      {"AFCounter run2 on" 1 "INT" 0}          \
      {"AFCounter run2 off" 0 "INT" 0}         \
      {"AFCounter run2 4 -forgive" 1 "INT" 0}]

#Test trigger
    set testAfctrList8 [list                      \
      {"AFCounter trig 0  -forgive" 1 "INT" 0}    \
      {"AFCounter trig 1" 1 "INT" 0}              \
      {"AFCounter trig 2" 2 "INT" 0}              \
      {"AFCounter trig 3 -forgive" 1 "INT" 0}     \
      {"AFCounter trigger 0  -forgive" 1 "INT" 0} \
      {"AFCounter trigger 1" 1 "INT" 0}           \
      {"AFCounter trigger 2" 2 "INT" 0}           \
      {"AFCounter trigger continuous" 1 "INT" 0}  \
      {"AFCounter trigger oneshot" 2 "INT" 0}     \
      {"AFCounter trigger 3 -forgive" 1 "INT" 0}]

#Test upper limit
    set testAfctrList9 [list                                    \
      {"AFCounter ulim -1 -forgive" 0.0 "DBL" 0.001}            \
      {"AFCounter ulim 1234.5" 1234.5 "DBL" 0.001}              \
      {"AFCounter ulim 9876.5" 9876.5 "DBL" 0.001}              \
      {"AFCounter ulim 20000.0" 20000.0 "DBL" 0.001}            \
      {"AFCounter ulim 39999.9" 39999.9 "DBL" 0.001}            \
      {"AFCounter ulim 40000.0" 40000.0 "DBL" 0.001}            \
      {"AFCounter ulim 0.0" 0.0 "DBL" 0.001}                    \
      {"AFCounter ulim 40000.5 -forgive" 40000.0 "DBL" 0.001}   \
      {"AFCounter ulim 9876.5 -forgive" 9876.5 "DBL" 0.001}     \
      {"AFCounter ulimit -1 -forgive" 0.0 "DBL" 0.001}          \
      {"AFCounter ulimit 1234.5" 1234.5 "DBL" 0.001}            \
      {"AFCounter ulimit 9876.5" 9876.5 "DBL" 0.001}            \
      {"AFCounter ulimit 20.0000 khz" 20000.0 "DBL" 0.001}      \
      {"AFCounter ulimit 0.0399999 mhz" 39999.9 "DBL" 0.001}    \
      {"AFCounter ulimit 40000.0" 40000.0 "DBL" 0.001}          \
      {"AFCounter ulimit 0.0" 0.0 "DBL" 0.001}                  \
      {"AFCounter ulimit 40000.5 -forgive" 40000.0 "DBL" 0.001} \
      {"AFCounter ulimit 9876.5 -forgive" 9876.5 "DBL" 0.001}]

#Test READING, VALUE and Averaging
    set testAfctrList10 {
      {"Fgen sh1 sine" 0 "INT" 0}
      {"Fgen lev1 1.0" {1.0 10.0 5.0} "compareDbls" 0.001}
      {"Fgen mod1 1" 1 "INT" 0} 
      {"BBAudio route fgen1 afcounter" {5 15} "compareInts" 0}

      {"AFCounter trigger continuous" 1 "INT" 0}
      {"AFCounter gate tenth" 0 "INT" 0}
      {"AFCounter enable on" 3 "INT" 0}
      {"AFCounter run on" 1 "INT" 0}

      {"AFCounter avg 1" 1 "INT" 0}
      {"Fgen freq1 123.456" 123.456 "DBL" 0.001}
      {"after 2000" 0 "DELAY" 0}
      {"AFCounter value" 123.5 "DBL" 0.1} 
      {"AFCounter reading" {123.5 -1 0 -1} "compareDbls" 0.1} 

      {"AFCounter avg 11" 11 "INT" 0}
      {"Fgen freq1 234.567" 234.567 "DBL" 0.001} 
      {"after 2000" 0 "DELAY" 0}
      {"AFCounter value" 234.567 "DBL" 0.1} 
      {"AFCounter reading" {234.567 0 0 0} "compareDbls" 0.1} 

      {"AFCounter avg 101" 101 "INT" 0}
      {"Fgen freq1 345.678" 345.678 "DBL" 0.001} 
      {"after 2000" 0 "DELAY" 0}
      {"AFCounter value" 345.678 "DBL" 0.1} 
      {"AFCounter reading" {345.678 1 0 1} "compareDbls" 0.1} }

}

#****************************************************************************
# Name:        autoTest
# Parameters:  resultsLevel		Indicates the results display level
# Description: Executes automatic test on AFCounter commands
#****************************************************************************
 proc autoTest { resultsLevel } {
    variable testCaseList

    # Declare each test as a namespace variable
    foreach testName $testCaseList {
       variable $testName
    }

    setup 

    # Execute the test lists, and then destroy the test lists
    ::BIT::resetTestStats "cmdline_AFCounter"
    foreach testName $testCaseList {
       ::BIT::runCmdLineTests "cmdline_AFCounter" $testName
       unset $testName
    }

    if { ($resultsLevel == "Full") || ($resultsLevel == "Manual") } \
    {
       # Test commands with no data - requires visual inspection
       ::BIT::putsTest [AFCounter phrst]
       ::BIT::putsTest [AFCounter phreset]

       # Test string functions - requires visual inspection
        foreach cmd {alerts average enable gate llimit peakhold phreset reading readpeak res run trigger ulimit value version } \
       {
          set temp [AFCounter $cmd -range]
          ::BIT::putsTest "The range of <$cmd> is:  $temp"
       }

       ::BIT::putsTest [AFCounter res]
       ::BIT::putsTest [AFCounter resources]
       ::BIT::putsTest [AFCounter version]
       ::BIT::putsTest [AFCounter ver 0]
       ::BIT::putsTest [AFCounter ver 1]
    }

    # Reset AF Counter to typical values after completion of the test
    AFCounter en 3
    AFCounter run 0
    AFCounter avg 10
    AFCounter trig 1
 }

}

# Add this test to the list of tests
::BIT::addTest cmdline_AFCounter

