#############################################################################
# COPYRIGHT (C) 2001, 2003, 2005  Aeroflex, Inc.                            #
# --------------------------------------------------------------------------#
# FILE        : test_C4FM.tcl                                               #
# PRODUCT     : FITS                                                        #
# AUTHOR      : Bruce Butterfield                                           #
# DESCRIPTION : Remote Command Line Test: C4FM                              #
# --------------------------------------------------------------------------#
# Update      : Resynch and engine cleanup for 1.9.1.4                      #
#############################################################################

package require C4fm
package require BIT

#****************************************************************************
# Test variables
#****************************************************************************
namespace eval ::cmdline_C4fm {
    variable testCaseList		;# List of test cases
    variable shortDesc			;# Test description - menu size
    variable longDesc			;# Test description - used by reports
    variable testTime			;# Test duration (Seconds - approximation)

    # Define the list of C4fm tests
    set testCaseList [list \
           testC4FMList1 testC4FMList1a testC4FMList1b \
           testC4FMList2 testC4FMList3 \
           testC4FMList4 testC4FMList4a testC4FMList4b \
           testC4FMList5 testC4FMList5a testC4FMList5b \
           testC4FMList6 testC4FMList7 \
           testC4FMList8 testC4FMList8a testC4FMList8b ]

    # Declare each test as a namespace variable
    foreach testName $testCaseList {
       variable $testName
    }

    set shortDesc "C4fm cmds"
    set longDesc "C4fm command line test"
    set testTime 0.50

#****************************************************************************
# Name:        setup
# Parameters:  none
# Description: Sets up a list of C4FM tests
#****************************************************************************
 proc setup { } {
    variable testCaseList

    # Declare each test as a namespace variable
    foreach testName $testCaseList {
       variable $testName
    }

    #
    # Define test cases
    #

    set testC4FMList1 [list \
      {"C4fm al -1 -forgive" 0 "INT" 0} \
      {"C4fm al 1" 1 "INT" 0} \
      {"C4fm al 0" 0 "INT" 0} \
      {"C4fm al 2" 2 "INT" 0} \
      {"C4fm al 3" 3 "INT" 0} \
      {"C4fm al 7" 7 "INT" 0} \
      {"C4fm al 8 -forgive" 0 "INT" 0} \
      {"C4fm alerts -1 -forgive" 0 "INT" 0} \
      {"C4fm alerts 1" 1 "INT" 0} \
      {"C4fm alerts 0" 0 "INT" 0} \
      {"C4fm alerts 2" 2 "INT" 0} \
      {"C4fm alerts 3" 3 "INT" 0} \
      {"C4fm alerts 7" 7 "INT" 0} \
      {"C4fm alerts off" 0 "INT" 0} \
      {"C4fm alerts enablelo" 1 "INT" 0} \
      {"C4fm alerts enablehi" 2 "INT" 0} \
      {"C4fm alerts enableboth" 3 "INT" 0} \
      {"C4fm alerts 8 -forgive" 0 "INT" 0} ]

    set testC4FMList1a [list \
      {"C4fm dal -1 -forgive" 0 "INT" 0} \
      {"C4fm dal 1" 1 "INT" 0} \
      {"C4fm dal 0" 0 "INT" 0} \
      {"C4fm dal 2" 2 "INT" 0} \
      {"C4fm dal 3" 3 "INT" 0} \
      {"C4fm dal 7" 7 "INT" 0} \
      {"C4fm dal 8 -forgive" 0 "INT" 0} \
      {"C4fm dev_alerts -1 -forgive" 0 "INT" 0} \
      {"C4fm dev_alerts 1" 1 "INT" 0} \
      {"C4fm dev_alerts 0" 0 "INT" 0} \
      {"C4fm dev_alerts 2" 2 "INT" 0} \
      {"C4fm dev_alerts 3" 3 "INT" 0} \
      {"C4fm dev_alerts 7" 7 "INT" 0} \
      {"C4fm dev_alerts off" 0 "INT" 0} \
      {"C4fm dev_alerts enablelo" 1 "INT" 0} \
      {"C4fm dev_alerts enablehi" 2 "INT" 0} \
      {"C4fm dev_alerts enableboth" 3 "INT" 0} \
      {"C4fm dev_alerts 8 -forgive" 0 "INT" 0} ]

    set testC4FMList1b [list \
      {"C4fm fal -1 -forgive" 0 "INT" 0} \
      {"C4fm fal 1" 1 "INT" 0} \
      {"C4fm fal 0" 0 "INT" 0} \
      {"C4fm fal 2" 2 "INT" 0} \
      {"C4fm fal 3" 3 "INT" 0} \
      {"C4fm fal 7" 7 "INT" 0} \
      {"C4fm fal 8 -forgive" 0 "INT" 0} \
      {"C4fm ferr_alerts -1 -forgive" 0 "INT" 0} \
      {"C4fm ferr_alerts 1" 1 "INT" 0} \
      {"C4fm ferr_alerts 0" 0 "INT" 0} \
      {"C4fm ferr_alerts 2" 2 "INT" 0} \
      {"C4fm ferr_alerts 3" 3 "INT" 0} \
      {"C4fm ferr_alerts 7" 7 "INT" 0} \
      {"C4fm ferr_alerts off" 0 "INT" 0} \
      {"C4fm ferr_alerts enablelo" 1 "INT" 0} \
      {"C4fm ferr_alerts enablehi" 2 "INT" 0} \
      {"C4fm ferr_alerts enableboth" 3 "INT" 0} \
      {"C4fm ferr_alerts 8 -forgive" 0 "INT" 0} ]

    set testC4FMList2 [list \
      {"C4fm avg 199 -forgive" 200 "INT" 0} \
      {"C4fm avg 400" 400 "INT" 0} \
      {"C4fm avg 200" 200 "INT" 0} \
      {"C4fm avg 10000" 10000 "INT" 0} \
      {"C4fm avg 65535" 65535 "INT" 0} \
      {"C4fm avg 65536 -forgive" 65535 "INT" 0} \
      {"C4fm average 199 -forgive" 200 "INT" 0} \
      {"C4fm average 400" 400 "INT" 0} \
      {"C4fm average 200" 200 "INT" 0} \
      {"C4fm average 10000" 10000 "INT" 0} \
      {"C4fm average 65535" 65535 "INT" 0} \
      {"C4fm average 65536 -forgive" 65535 "INT" 0} ]

    set testC4FMList3 [list \
      {"C4fm en -1 -forgive" 3 "INT" 0} \
      {"C4fm en 0" 1 "INT" 0} \
      {"C4fm en 1" 1 "INT" 0} \
      {"C4fm en 2" 3 "INT" 0} \
      {"C4fm en 3" 3 "INT" 0} \
      {"C4fm en 4 -forgive" 1 "INT" 0} \
      {"C4fm enable -1 -forgive" 3 "INT" 0} \
      {"C4fm enable 0" 1 "INT" 0} \
      {"C4fm enable 1" 1 "INT" 0} \
      {"C4fm enable 2" 3 "INT" 0} \
      {"C4fm enable 3" 3 "INT" 0} \
      {"C4fm enable disable" 1 "INT" 0} \
      {"C4fm enable enable" 3 "INT" 0} \
      {"C4fm enable off" 1 "INT" 0} \
      {"C4fm enable on" 3 "INT" 0} \
      {"C4fm enable 4 -forgive" 1 "INT" 0} ]

    set testC4FMList4 [list \
      {"C4fm llim -1 -forgive" 0.0 "DBL" 0.001} \
      {"C4fm llim 12.3" 12.3 "DBL" 0.001} \
      {"C4fm llim 49.9" 49.9 "DBL" 0.001} \
      {"C4fm llim 50.0" 50.0 "DBL" 0.001} \
      {"C4fm llim 0.0" 0.0 "DBL" 0.001} \
      {"C4fm llim 50.1 -forgive" 50.0 "DBL" 0.001} \
      {"C4fm llimit -1 -forgive" 0.0 "DBL" 0.001} \
      {"C4fm llimit 12.3" 12.3 "DBL" 0.001} \
      {"C4fm llimit 49.9" 49.9 "DBL" 0.001} \
      {"C4fm llimit 50.0" 50.0 "DBL" 0.001} \
      {"C4fm llimit 50.1 -forgive" 50.0 "DBL" 0.001} \
      {"C4fm llimit 0.0" 0.0 "DBL" 0.001} ]

    set testC4FMList4a [list \
      {"C4fm dllim -1 -forgive" 0.0 "DBL" 0.001} \
      {"C4fm dllim 12.3" 12.3 "DBL" 0.001} \
      {"C4fm dllim 49.9" 49.9 "DBL" 0.001} \
      {"C4fm dllim 2000.0" 2000.0 "DBL" 0.001} \
      {"C4fm dllim 2399.99" 2399.99 "DBL" 0.001} \
      {"C4fm dllim 2400.0" 2400.0 "DBL" 0.001} \
      {"C4fm dllim 2400.1 -forgive" 2400.0 "DBL" 0.001} \
      {"C4fm dev_llimit -1 -forgive" 0.0 "DBL" 0.001} \
      {"C4fm dev_llimit 12.3" 12.3 "DBL" 0.001} \
      {"C4fm dev_llimit 49.9" 49.9 "DBL" 0.001} \
      {"C4fm dev_llimit 2000.0" 2000.0 "DBL" 0.001} \
      {"C4fm dev_llimit 2399.99" 2399.99 "DBL" 0.001} \
      {"C4fm dev_llimit 2400.0" 2400.0 "DBL" 0.001} \
      {"C4fm dev_llimit 2400.1 -forgive" 2400.0 "DBL" 0.001} \
      {"C4fm dev_llimit 0.0" 0.0 "DBL" 0.001} ]

    set testC4FMList4b [list \
      {"C4fm fllim 0.0" 0.0 "DBL" 0.001} \
      {"C4fm fllim 1.0" 1.0 "DBL" 0.001} \
      {"C4fm fllim 10.0" 10.0 "DBL" 0.001} \
      {"C4fm fllim 100.0" 100.0 "DBL" 0.001} \
      {"C4fm fllim 1000.0" 1000.0 "DBL" 0.001} \
      {"C4fm fllim 9999.9" 9999.9 "DBL" 0.001} \
      {"C4fm fllim 10000.0" 10000.0 "DBL" 0.001} \
      {"C4fm fllim 10000.1 -forgive" 10000.0 "DBL" 0.001} \
      {"C4fm fllim -1.0" -1.0 "DBL" 0.001} \
      {"C4fm fllim -10.0" -10.0 "DBL" 0.001} \
      {"C4fm fllim -100.0" -100.0 "DBL" 0.001} \
      {"C4fm fllim -1000.0" -1000.0 "DBL" 0.001} \
      {"C4fm fllim -9999.9" -9999.9 "DBL" 0.001} \
      {"C4fm fllim -10000.0" -10000.0 "DBL" 0.001} \
      {"C4fm fllim -10000.1 -forgive" -10000.0 "DBL" 0.001} \
      {"C4fm ferr_llimit 0.0" 0.0 "DBL" 0.001} \
      {"C4fm ferr_llimit 1.0" 1.0 "DBL" 0.001} \
      {"C4fm ferr_llimit 10.0" 10.0 "DBL" 0.001} \
      {"C4fm ferr_llimit 100.0" 100.0 "DBL" 0.001} \
      {"C4fm ferr_llimit 1000.0" 1000.0 "DBL" 0.001} \
      {"C4fm ferr_llimit 9999.9" 9999.9 "DBL" 0.001} \
      {"C4fm ferr_llimit 10000.0" 10000.0 "DBL" 0.001} \
      {"C4fm ferr_llimit 10000.1 -forgive" 10000.0 "DBL" 0.001} \
      {"C4fm ferr_llimit -1.0" -1.0 "DBL" 0.001} \
      {"C4fm ferr_llimit -10.0" -10.0 "DBL" 0.001} \
      {"C4fm ferr_llimit -100.0" -100.0 "DBL" 0.001} \
      {"C4fm ferr_llimit -1000.0" -1000.0 "DBL" 0.001} \
      {"C4fm ferr_llimit -9999.9" -9999.9 "DBL" 0.001} \
      {"C4fm ferr_llimit -10000.0" -10000.0 "DBL" 0.001} \
      {"C4fm ferr_llimit -10000.1 -forgive" -10000.0 "DBL" 0.001} ]

    set testC4FMList5 [list \
      {"C4fm peak -1 -forgive" 0 "INT" 0} \
      {"C4fm peak 1" 1 "INT" 0} \
      {"C4fm peak 0" 0 "INT" 0} \
      {"C4fm peak 1" 1 "INT" 0} \
      {"C4fm peak 2 -forgive" 1 "INT" 0} \
      {"C4fm peakhold -1 -forgive" 0 "INT" 0} \
      {"C4fm peakhold 1" 1 "INT" 0} \
      {"C4fm peakhold 0" 0 "INT" 0} \
      {"C4fm peakhold 1" 1 "INT" 0} \
      {"C4fm peakhold off" 0 "INT" 0} \
      {"C4fm peakhold on" 1 "INT" 0} \
      {"C4fm peakhold clr" 0 "INT" 0} \
      {"C4fm peakhold set" 1 "INT" 0} \
      {"C4fm peakhold 2 -forgive" 1 "INT" 0} ]

    set testC4FMList5a [list \
      {"C4fm dpeak -1 -forgive" 0 "INT" 0} \
      {"C4fm dpeak 1" 1 "INT" 0} \
      {"C4fm dpeak 0" 0 "INT" 0} \
      {"C4fm dpeak 1" 1 "INT" 0} \
      {"C4fm dpeak 2 -forgive" 1 "INT" 0} \
      {"C4fm dev_peakhold -1 -forgive" 0 "INT" 0} \
      {"C4fm dev_peakhold 1" 1 "INT" 0} \
      {"C4fm dev_peakhold 0" 0 "INT" 0} \
      {"C4fm dev_peakhold 1" 1 "INT" 0} \
      {"C4fm dev_peakhold off" 0 "INT" 0} \
      {"C4fm dev_peakhold on" 1 "INT" 0} \
      {"C4fm dev_peakhold clr" 0 "INT" 0} \
      {"C4fm dev_peakhold set" 1 "INT" 0} \
      {"C4fm dev_peakhold 2 -forgive" 1 "INT" 0} ]

    set testC4FMList5b [list \
      {"C4fm fpeak -1 -forgive" 0 "INT" 0} \
      {"C4fm fpeak 1" 1 "INT" 0} \
      {"C4fm fpeak 0" 0 "INT" 0} \
      {"C4fm fpeak 1" 1 "INT" 0} \
      {"C4fm fpeak 2 -forgive" 1 "INT" 0} \
      {"C4fm ferr_peakhold -1 -forgive" 0 "INT" 0} \
      {"C4fm ferr_peakhold 1" 1 "INT" 0} \
      {"C4fm ferr_peakhold 0" 0 "INT" 0} \
      {"C4fm ferr_peakhold 1" 1 "INT" 0} \
      {"C4fm ferr_peakhold off" 0 "INT" 0} \
      {"C4fm ferr_peakhold on" 1 "INT" 0} \
      {"C4fm ferr_peakhold clr" 0 "INT" 0} \
      {"C4fm ferr_peakhold set" 1 "INT" 0} \
      {"C4fm ferr_peakhold 2 -forgive" 1 "INT" 0} ]

    set testC4FMList6 [list \
      {"C4fm run -1 -forgive" 0 "INT" 0} \
      {"C4fm run 1" 1 "INT" 0} \
      {"C4fm run 0" 0 "INT" 0} \
      {"C4fm run on" 1 "INT" 0} \
      {"C4fm run off" 0 "INT" 0} \
      {"C4fm run 4 -forgive" 1 "INT" 0} ]

    set testC4FMList7 [list \
      {"C4fm trig 0  -forgive" 1 "INT" 0} \
      {"C4fm trig 1" 1 "INT" 0} \
      {"C4fm trig 2" 2 "INT" 0} \
      {"C4fm trig 3 -forgive" 1 "INT" 0} \
      {"C4fm trigger 0  -forgive" 1 "INT" 0} \
      {"C4fm trigger 1" 1 "INT" 0} \
      {"C4fm trigger 2" 2 "INT" 0} \
      {"C4fm trigger continuous" 1 "INT" 0} \
      {"C4fm trigger oneshot" 2 "INT" 0} \
      {"C4fm trigger 3 -forgive" 1 "INT" 0} ]

    set testC4FMList8 [list \
      {"C4fm ulim -1 -forgive" 0.0 "DBL" 0.001} \
      {"C4fm ulim 12.3" 12.3 "DBL" 0.001} \
      {"C4fm ulim 49.9" 49.9 "DBL" 0.001} \
      {"C4fm ulim 50.0" 50.0 "DBL" 0.001} \
      {"C4fm ulim 0.0" 0.0 "DBL" 0.001} \
      {"C4fm ulim 50.1 -forgive" 50.0 "DBL" 0.001} \
      {"C4fm ulimit -1 -forgive" 0.0 "DBL" 0.001} \
      {"C4fm ulimit 12.3" 12.3 "DBL" 0.001} \
      {"C4fm ulimit 49.9" 49.9 "DBL" 0.001} \
      {"C4fm ulimit 50.0" 50.0 "DBL" 0.001} \
      {"C4fm ulimit 50.1 -forgive" 50.0 "DBL" 0.001} \
      {"C4fm ulimit 0.0" 0.0 "DBL" 0.001} ]

    set testC4FMList8a [list \
      {"C4fm dulim -1 -forgive" 0.0 "DBL" 0.001} \
      {"C4fm dulim 12.3" 12.3 "DBL" 0.001} \
      {"C4fm dulim 49.9" 49.9 "DBL" 0.001} \
      {"C4fm dulim 2000.0" 2000.0 "DBL" 0.001} \
      {"C4fm dulim 2399.99" 2399.99 "DBL" 0.001} \
      {"C4fm dulim 2400.0" 2400.0 "DBL" 0.001} \
      {"C4fm dulim 2400.1 -forgive" 2400.0 "DBL" 0.001} \
      {"C4fm dev_ulimit -1 -forgive" 0.0 "DBL" 0.001} \
      {"C4fm dev_ulimit 12.3" 12.3 "DBL" 0.001} \
      {"C4fm dev_ulimit 49.9" 49.9 "DBL" 0.001} \
      {"C4fm dev_ulimit 2000.0" 2000.0 "DBL" 0.001} \
      {"C4fm dev_ulimit 2399.99" 2399.99 "DBL" 0.001} \
      {"C4fm dev_ulimit 2400.0" 2400.0 "DBL" 0.001} \
      {"C4fm dev_ulimit 2400.1 -forgive" 2400.0 "DBL" 0.001} \
      {"C4fm dev_ulimit 0.0" 0.0 "DBL" 0.001} ]

    set testC4FMList8b [list \
      {"C4fm fulim 0.0" 0.0 "DBL" 0.001} \
      {"C4fm fulim 1.0" 1.0 "DBL" 0.001} \
      {"C4fm fulim 10.0" 10.0 "DBL" 0.001} \
      {"C4fm fulim 100.0" 100.0 "DBL" 0.001} \
      {"C4fm fulim 1000.0" 1000.0 "DBL" 0.001} \
      {"C4fm fulim 9999.9" 9999.9 "DBL" 0.001} \
      {"C4fm fulim 10000.0" 10000.0 "DBL" 0.001} \
      {"C4fm fulim 10000.1 -forgive" 10000.0 "DBL" 0.001} \
      {"C4fm fulim -1.0" -1.0 "DBL" 0.001} \
      {"C4fm fulim -10.0" -10.0 "DBL" 0.001} \
      {"C4fm fulim -100.0" -100.0 "DBL" 0.001} \
      {"C4fm fulim -1000.0" -1000.0 "DBL" 0.001} \
      {"C4fm fulim -9999.9" -9999.9 "DBL" 0.001} \
      {"C4fm fulim -10000.0" -10000.0 "DBL" 0.001} \
      {"C4fm fulim -10000.1 -forgive" -10000.0 "DBL" 0.001} \
      {"C4fm ferr_ulimit 0.0" 0.0 "DBL" 0.001} \
      {"C4fm ferr_ulimit 1.0" 1.0 "DBL" 0.001} \
      {"C4fm ferr_ulimit 10.0" 10.0 "DBL" 0.001} \
      {"C4fm ferr_ulimit 100.0" 100.0 "DBL" 0.001} \
      {"C4fm ferr_ulimit 1000.0" 1000.0 "DBL" 0.001} \
      {"C4fm ferr_ulimit 9999.9" 9999.9 "DBL" 0.001} \
      {"C4fm ferr_ulimit 10000.0" 10000.0 "DBL" 0.001} \
      {"C4fm ferr_ulimit 10000.1 -forgive" 10000.0 "DBL" 0.001} \
      {"C4fm ferr_ulimit -1.0" -1.0 "DBL" 0.001} \
      {"C4fm ferr_ulimit -10.0" -10.0 "DBL" 0.001} \
      {"C4fm ferr_ulimit -100.0" -100.0 "DBL" 0.001} \
      {"C4fm ferr_ulimit -1000.0" -1000.0 "DBL" 0.001} \
      {"C4fm ferr_ulimit -9999.9" -9999.9 "DBL" 0.001} \
      {"C4fm ferr_ulimit -10000.0" -10000.0 "DBL" 0.001} \
      {"C4fm ferr_ulimit -10000.1 -forgive" -10000.0 "DBL" 0.001} ]

 }

#****************************************************************************
# Name:        autoTest
# Parameters:  resultsLevel		Indicates the results display level
# Description: Executes automatic test on AFCounter commands
#****************************************************************************
 proc autoTest { resultsLevel } {
    variable testCaseList

    # Declare each test as a namespace variable
    foreach testName $testCaseList {
       variable $testName
    }

    setup 

    # Execute the test lists, and then destroy the test lists
    ::BIT::resetTestStats "cmdline_C4fm"
    foreach testName $testCaseList {
       ::BIT::runCmdLineTests "cmdline_C4fm" $testName
       unset $testName
    }

    if { ($resultsLevel == "Full") || ($resultsLevel == "Manual") } \
    {
       # Test commands with no data - requires visual inspection
       ::BIT::putsTest [C4fm phrst]
       ::BIT::putsTest [C4fm phreset]
       ::BIT::putsTest [C4fm dphrst]
       ::BIT::putsTest [C4fm dev_phreset]
       ::BIT::putsTest [C4fm fphrst]
       ::BIT::putsTest [C4fm ferr_phreset]

       ::BIT::putsTest [C4fm readpeak]
       ::BIT::putsTest [C4fm readpeakhold]
       ::BIT::putsTest [C4fm freadpeak]
       ::BIT::putsTest [C4fm ferr_readpeakhold]
       ::BIT::putsTest [C4fm dreadpeak]
       ::BIT::putsTest [C4fm dev_readpeakhold]

       ::BIT::putsTest [C4fm reading]
       ::BIT::putsTest [C4fm freading]
       ::BIT::putsTest [C4fm ferr_reading]
       ::BIT::putsTest [C4fm dreading]
       ::BIT::putsTest [C4fm dev_reading]

       # Test string functions - requires visual inspection
       foreach cmd {alerts average enable llimit peakhold phreset reading \
                    readpeakhold resources run trigger ulimit value version \
                    dev_alerts dev_llimit dev_peakhold dev_phreset dev_ulimit \
                    dev_value dev_reading dev_readpeakhold \
                    ferr_alerts ferr_llimit ferr_peakhold ferr_phreset \
                    ferr_ulimit ferr_value ferr_reading ferr_readpeakhold } \
       {
          set temp [C4fm $cmd -range]
          ::BIT::putsTest "The range of <$cmd> is:  $temp"
       }

       ::BIT::putsTest [C4fm res]
       ::BIT::putsTest [C4fm resources]
       ::BIT::putsTest [C4fm ver 0]
       ::BIT::putsTest [C4fm version 1]
    }

    # Reset C4FM to typical values after completion of the test
    C4fm en 3
    C4fm run 0
    C4fm avg 200
    C4fm trig 1
 }

}

# Add this test to the list of tests
::BIT::addTest cmdline_C4fm


