#############################################################################
# COPYRIGHT (C) 2001, 2003, 2005  Aeroflex, Inc.                            #
# --------------------------------------------------------------------------#
# FILE        : test_Fmdev.tcl                                              #
# PRODUCT     : FITS                                                        #
# AUTHOR      : Bruce Butterfield, Jon Butterfield                          #
# DESCRIPTION : Remote Command Line Test: FM Deviation                      #
# --------------------------------------------------------------------------#
# Update      : Resynch and engine cleanup for 1.9.1.4                      #
#############################################################################

package require FMDev
package require BIT

#****************************************************************************
# Test variables
#****************************************************************************
namespace eval ::cmdline_FMDev {
    variable testCaseList		;# List of test cases
    variable shortDesc			;# Test description - menu size
    variable longDesc			;# Test description - used by reports
    variable testTime			;# Test duration (Seconds - approximation)

    # Define the list of FMDev tests
    set testCaseList [list \
           testFmdevList1 testFmdevList2  testFmdevList3  testFmdevList4 \
           testFmdevList5 testFmdevList6  testFmdevList7  testFmdevList8 \
           testFmdevList9 testFmdevList10 testFmdevList11 testFmdevList12]

    # Declare each test as a namespace variable
    foreach testName $testCaseList {
       variable $testName
    }

    set shortDesc "FMDev cmds"
    set longDesc "FM Deviation command line test"
    set testTime 24

#****************************************************************************
# Name:        setup
# Parameters:  none
# Description: Sets up a list of FMDev tests
#****************************************************************************
 proc setup { } {
    variable testCaseList

    # Declare each test as a namespace variable
    foreach testName $testCaseList {
       variable $testName
    }

    #
    # Define test cases
    #

    set testFmdevList1 [list \
      {"FMDev al -1 -forgive" 0 "INT" 0} \
      {"FMDev al 1" 1 "INT" 0} \
      {"FMDev al 0" 0 "INT" 0} \
      {"FMDev al 2" 2 "INT" 0} \
      {"FMDev al 3" 3 "INT" 0} \
      {"FMDev al 7" 7 "INT" 0} \
      {"FMDev al 8 -forgive" 0 "INT" 0} \
      {"FMDev alerts -1 -forgive" 0 "INT" 0} \
      {"FMDev alerts 1" 1 "INT" 0} \
      {"FMDev alerts 0" 0 "INT" 0} \
      {"FMDev alerts 2" 2 "INT" 0} \
      {"FMDev alerts 3" 3 "INT" 0} \
      {"FMDev alerts 7" 7 "INT" 0} \
      {"FMDev alerts off" 0 "INT" 0} \
      {"FMDev alerts enablelo" 1 "INT" 0} \
      {"FMDev alerts enablehi" 2 "INT" 0} \
      {"FMDev alerts enableboth" 3 "INT" 0} \
      {"FMDev alerts 8 -forgive" 0 "INT" 0} ]

    set testFmdevList2 [list \
      {"FMDev avg 0 -forgive" 50 "INT" 0} \
      {"FMDev avg 49 -forgive" 50 "INT" 0} \
      {"FMDev avg 50" 50 "INT" 0} \
      {"FMDev avg 100" 100 "INT" 0} \
      {"FMDev avg 10000" 10000 "INT" 0} \
      {"FMDev avg 10001 -forgive" 10000 "INT" 0} \
      {"FMDev avg 8000000 -forgive" 10000 "INT" 0} \
      {"FMDev average 0 -forgive" 50 "INT" 0} \
      {"FMDev average 49 -forgive" 50 "INT" 0} \
      {"FMDev average 50" 50 "INT" 0} \
      {"FMDev average 100" 100 "INT" 0} \
      {"FMDev average 10000" 10000 "INT" 0} \
      {"FMDev average 10001 -forgive" 10000 "INT" 0} \
      {"FMDev average 1000 -forgive" 1000 "INT" 0} ]

    set testFmdevList3 [list \
      {"FMDev en -1 -forgive" 3 "INT" 0} \
      {"FMDev en 0" 1 "INT" 0} \
      {"FMDev en 1" 1 "INT" 0} \
      {"FMDev en 2" 3 "INT" 0} \
      {"FMDev en 3" 3 "INT" 0} \
      {"FMDev en 4 -forgive" 1 "INT" 0} \
      {"FMDev enable -1 -forgive" 3 "INT" 0} \
      {"FMDev enable 0" 1 "INT" 0} \
      {"FMDev enable 1" 1 "INT" 0} \
      {"FMDev enable 2" 3 "INT" 0} \
      {"FMDev enable 3" 3 "INT" 0} \
      {"FMDev enable off" 1 "INT" 0} \
      {"FMDev enable on" 3 "INT" 0} \
      {"FMDev enable disable" 1 "INT" 0} \
      {"FMDev enable enable" 3 "INT" 0} \
      {"FMDev enable 4 -forgive" 1 "INT" 0} ]

    set testFmdevList4 [list \
      {"FMDev llim -1 -forgive" 0.0 "DBL" 0.001} \
      {"FMDev llim 12.3" 12.3 "DBL" 0.001} \
      {"FMDev llim 67.8" 67.8 "DBL" 0.001} \
      {"FMDev llim 99.999" 99.999 "DBL" 0.001} \
      {"FMDev llim 100.0" 100.0 "DBL" 0.001} \
      {"FMDev llim 100.1 -forgive" 100.0 "DBL" 0.001} \
      {"FMDev llim 9999.1 -forgive" 100.0 "DBL" 0.001} \
      {"FMDev llimit -1 -forgive" 0.0 "DBL" 0.001} \
      {"FMDev llimit 12.3" 12.3 "DBL" 0.001} \
      {"FMDev llimit 67.8" 67.8 "DBL" 0.001} \
      {"FMDev llimit 99.999" 99.999 "DBL" 0.001} \
      {"FMDev llimit 100.0" 100.0 "DBL" 0.001} \
      {"FMDev llimit 100.1 -forgive" 100.0 "DBL" 0.001} \
      {"FMDev llimit 9999.1 -forgive" 100.0 "DBL" 0.001} ]

    set testFmdevList5 [list \
      {"FMDev math -1 -forgive" 0 "INT" 0} \
      {"FMDev math 1" 1 "INT" 0} \
      {"FMDev math 0" 0 "INT" 0} \
      {"FMDev math 2" 2 "INT" 0} \
      {"FMDev math 3" 3 "INT" 0} \
      {"FMDev math average" 1 "INT" 0} \
      {"FMDev math peak" 0 "INT" 0} \
      {"FMDev math avg" 1 "INT" 0} \
      {"FMDev math plus" 2 "INT" 0} \
      {"FMDev math minus" 3 "INT" 0} \
      {"FMDev math 4 -forgive" 0 "INT" 0} ]

    set testFmdevList6 [list \
      {"FMDev peak -1 -forgive" 0 "INT" 0} \
      {"FMDev peak 1" 1 "INT" 0} \
      {"FMDev peak 0" 0 "INT" 0} \
      {"FMDev peak 1" 1 "INT" 0} \
      {"FMDev peak 2 -forgive" 1 "INT" 0} \
      {"FMDev peakhold -1 -forgive" 0 "INT" 0} \
      {"FMDev peakhold 1" 1 "INT" 0} \
      {"FMDev peakhold 0" 0 "INT" 0} \
      {"FMDev peakhold 1" 1 "INT" 0} \
      {"FMDev peakhold off" 0 "INT" 0} \
      {"FMDev peakhold on" 1 "INT" 0} \
      {"FMDev peakhold 2 -forgive" 1 "INT" 0} ]

    set testFmdevList7 [list \
      {"FMDev run -1 -forgive" 0 "INT" 0} \
      {"FMDev run 1" 1 "INT" 0} \
      {"FMDev run 0" 0 "INT" 0} \
      {"FMDev run on" 1 "INT" 0} \
      {"FMDev run off" 0 "INT" 0} \
      {"FMDev run 4 -forgive" 1 "INT" 0} ]

    set testFmdevList8 [list \
      {"FMDev squ -1 -forgive" 0 "INT" 0} \
      {"FMDev squ 1" 1 "INT" 0} \
      {"FMDev squ 0" 0 "INT" 0} \
      {"FMDev squ on" 1 "INT" 0} \
      {"FMDev squ off" 0 "INT" 0} \
      {"FMDev squ 4 -forgive" 1 "INT" 0} \
      {"FMDev squelch -1 -forgive" 0 "INT" 0} \
      {"FMDev squelch 1" 1 "INT" 0} \
      {"FMDev squelch 0" 0 "INT" 0} \
      {"FMDev squelch on" 1 "INT" 0} \
      {"FMDev squelch off" 0 "INT" 0} \
      {"FMDev squelch 4 -forgive" 1 "INT" 0} ]

    set testFmdevList9 [list \
      {"FMDev trig 0  -forgive" 1 "INT" 0} \
      {"FMDev trig 1" 1 "INT" 0} \
      {"FMDev trig 2" 2 "INT" 0} \
      {"FMDev trig 3 -forgive" 1 "INT" 0} \
      {"FMDev trigger 0  -forgive" 1 "INT" 0} \
      {"FMDev trigger 1" 1 "INT" 0} \
      {"FMDev trigger 2" 2 "INT" 0} \
      {"FMDev trigger continuous" 1 "INT" 0} \
      {"FMDev trigger oneshot" 2 "INT" 0} \
      {"FMDev trigger 3 -forgive" 1 "INT" 0} ]

    set testFmdevList10 [list \
      {"FMDev ulim -1 -forgive" 0.0 "DBL" 0.001} \
      {"FMDev ulim 12.3" 12.3 "DBL" 0.001} \
      {"FMDev ulim 67.8" 67.8 "DBL" 0.001} \
      {"FMDev ulim 99.999" 99.999 "DBL" 0.001} \
      {"FMDev ulim 100.0" 100.0 "DBL" 0.001} \
      {"FMDev ulim 100.1 -forgive" 100.0 "DBL" 0.001} \
      {"FMDev ulim 9999.1 -forgive" 100.0 "DBL" 0.001} \
      {"FMDev ulimit -1 -forgive" 0.0 "DBL" 0.001} \
      {"FMDev ulimit 12.3" 12.3 "DBL" 0.001} \
      {"FMDev ulimit 67.8" 67.8 "DBL" 0.001} \
      {"FMDev ulimit 99.999" 99.999 "DBL" 0.001} \
      {"FMDev ulimit 100.0" 100.0 "DBL" 0.001} \
      {"FMDev ulimit 100.1 -forgive" 100.0 "DBL" 0.001} \
      {"FMDev ulimit 9999.1 -forgive" 100.0 "DBL" 0.001} ]

# Partial system test: testing "value" and "reading"
    set testFmdevList11 {
      {"Generator freq 345.6 mhz" 345.6 "DBL" 0.001} 
      {"Generator rflevel -50.0" -50.0 "DBL" 0.001} 
      {"Generator rfoutput t/r" 0 "INT" 0} 
      {"Generator modulation fmn" 1 "INT" 0} 
      {"Generator enable 11" 11 "INT" 0} 
      {"Generator ptt on" 1 "INT" 0} 

      {"Receiver freq 345.6 mhz" 345.6 "DBL" 0.001}
      {"Receiver rfinput t/r" 0 "INT" 0}
      {"Receiver atten 0" 0 "INT" 0}
      {"Receiver ifbandwidth 300000" 300000 "INT" 0}
      {"Receiver demod 3" 3 "INT" 0}

      {"FMDev alerts off" 0 "INT" 0}
      {"FMDev average 1000" 1000 "INT" 0}
      {"FMDev enable on" 3 "INT" 0}
      {"FMDev math peak" 0 "INT" 0}
      {"FMDev peakhold off" 0 "INT" 0}
      {"FMDev run on" 1 "INT" 0}
      {"FMDev trigger continuous" 1 "INT" 0}
      {"FMDev squelch off" 0 "INT" 0}

      {"Fgen mod4 0" 0 "INT" 0}
      {"Fgen mod3 1" 1 "INT" 0}
      {"Fgen freq3 1000.0" 1000.0 "DBL" 0.001}

      {"Fgen dev3 5.0" {1.0 10.0 5.0} "compareDbls" 0.001}
      {"after 2000" 0 "DELAY" 0}
      {"FMDev value" 5.0 "DBL" 0.2}
      {"FMDev reading" {5.0 2.0 0.0} "compareDbls" 0.2}

      {"Fgen dev3 4.0" {0.8 8.0 4.0} "compareDbls" 0.001}
      {"after 2000" 0 "DELAY" 0}
      {"FMDev value" 4.0 "DBL" 0.2}
      {"FMDev reading" {4.0 2.0 0.0} "compareDbls" 0.2}

      {"Fgen dev3 3.0" {0.6 6.0 3.0} "compareDbls" 0.001}
      {"after 2000" 0 "DELAY" 0}
      {"FMDev value" 3.0 "DBL" 0.2}
      {"FMDev reading" {3.0 2.0 0.0} "compareDbls" 0.2}

      {"Fgen dev3 2.0" {0.4 4.0 2.0} "compareDbls" 0.001}
      {"after 2000" 0 "DELAY" 0}
      {"FMDev value" 2.0 "DBL" 0.2}
      {"FMDev reading" {2.0 2.0 0.0} "compareDbls" 0.2}

      {"Fgen dev3 1.0" {0.2 2.0 1.0} "compareDbls" 0.001}
      {"after 2000" 0 "DELAY" 0}
      {"FMDev value" 1.0 "DBL" 0.2}
      {"FMDev reading" {1.0 2.0 0.0} "compareDbls" 0.2} 
      }

# Partial system test: testing "phreset" and "readpeakhold"
    set testFmdevList12 {
      {"Generator freq 345.6 mhz" 345.6 "DBL" 0.001} 
      {"Generator rflevel -50.0" -50.0 "DBL" 0.001} 
      {"Generator rfoutput t/r" 0 "INT" 0} 
      {"Generator modulation fmn" 1 "INT" 0} 
      {"Generator enable 11" 11 "INT" 0} 
      {"Generator ptt on" 1 "INT" 0} 

      {"Receiver freq 345.6 mhz" 345.6 "DBL" 0.001}
      {"Receiver rfinput t/r" 0 "INT" 0}
      {"Receiver atten 0" 0 "INT" 0}
      {"Receiver ifbandwidth 300000" 300000 "INT" 0}
      {"Receiver demod 3" 3 "INT" 0}

      {"FMDev alerts off" 0 "INT" 0}
      {"FMDev average 1000" 1000 "INT" 0}
      {"FMDev enable on" 3 "INT" 0}
      {"FMDev math peak" 0 "INT" 0}
      {"FMDev peakhold off" 0 "INT" 0}
      {"FMDev run on" 1 "INT" 0}
      {"FMDev trigger continuous" 1 "INT" 0}
      {"FMDev squelch off" 0 "INT" 0}

      {"Fgen mod4 0" 0 "INT" 0}
      {"Fgen mod3 1" 1 "INT" 0}
      {"Fgen freq3 1000.0" 1000.0 "DBL" 0.001}

      {"Fgen dev3 5.0" {1.0 10.0 5.0} "compareDbls" 0.001}
      {"after 2000" 0 "DELAY" 0}
      {"FMDev value" 5.0 "DBL" 0.2}
      {"FMDev reading" {5.0 2.0 0.0} "compareDbls" 0.2}
      {"FMDev phreset" 0 "NOCMP" 0}
      {"after 1000" 0 "DELAY" 0}
      {"FMDev readpeakhold" {5.0 0.0 5.0 0.0 0} "compareDbls" 0.2}

      {"FMDev peakhold on" 1 "INT" 0} \
      {"Fgen dev3 4.0" {0.8 8.0 4.0} "compareDbls" 0.001}
      {"after 2000" 0 "DELAY" 0}
      {"FMDev value" 4.0 "DBL" 0.2}
      {"FMDev reading" {4.0 2.0 0.0} "compareDbls" 0.2}
      {"FMDev readpeakhold" {5.0 0.0 4.0 0.0 1} "compareDbls" 0.2}

      {"Fgen dev3 3.0" {0.6 6.0 3.0} "compareDbls" 0.001}
      {"after 2000" 0 "DELAY" 0}
      {"FMDev value" 3.0 "DBL" 0.2}
      {"FMDev reading" {3.0 2.0 0.0} "compareDbls" 0.2}
      {"FMDev readpeak" {5.0 0.0 3.0 0.0 1} "compareDbls" 0.2}
      {"FMDev phrst" 0 "NOCMP" 0}
      {"after 1000" 0 "DELAY" 0}
      {"FMDev readpeak" {3.0 0.0 3.0 0.0 1} "compareDbls" 0.2}

      {"Fgen dev3 2.0" {0.4 4.0 2.0} "compareDbls" 0.001}
      {"after 2000" 0 "DELAY" 0}
      {"FMDev value" 2.0 "DBL" 0.2}
      {"FMDev reading" {2.0 2.0 0.0} "compareDbls" 0.2}
      {"FMDev readpeak" {3.0 0.0 2.0 0.0 1} "compareDbls" 0.2}

      {"Fgen dev3 1.0" {0.2 2.0 1.0} "compareDbls" 0.001}
      {"after 2000" 0 "DELAY" 0}
      {"FMDev value" 1.0 "DBL" 0.2}
      {"FMDev reading" {1.0 2.0 0.0} "compareDbls" 0.2} 
      {"FMDev readpeak" {3.0 0.0 1.0 0.0 1} "compareDbls" 0.2}
      {"FMDev phrst" 0 "NOCMP" 0}
      {"after 1000" 0 "DELAY" 0}
      {"FMDev readpeak" {1.0 0.0 1.0 0.0 1} "compareDbls" 0.2}
      }


 }

#****************************************************************************
# Name:        autoTest
# Parameters:  resultsLevel		Indicates the results display level
# Description: Executes automatic test on FMDev commands
#****************************************************************************
 proc autoTest { resultsLevel } {
    variable testCaseList

    # Declare each test as a namespace variable
    foreach testName $testCaseList {
       variable $testName
    }

    setup 

    # Execute the test lists, and then destroy the test lists
    ::BIT::resetTestStats "cmdline_FMDev"
    foreach testName $testCaseList {
       ::BIT::runCmdLineTests "cmdline_FMDev" $testName
       unset $testName
    }

    if { ($resultsLevel == "Full") || ($resultsLevel == "Manual") } \
    {
       # Test commands with no data - requires visual inspection
       ::BIT::putsTest [FMDev phrst]
       ::BIT::putsTest [FMDev phreset]

       # Test string functions - requires visual inspection
       foreach cmd {alerts average enable llimit math peakhold phreset reading readpeakhold resources run squelch trigger ulimit value version} \
       {
          set temp [FMDev $cmd -range]
          ::BIT::putsTest "The range of <$cmd> is:  $temp"
       }

       ::BIT::putsTest [FMDev res]
       ::BIT::putsTest [FMDev resources]
       ::BIT::putsTest [FMDev version]
       ::BIT::putsTest [FMDev ver 0]
       ::BIT::putsTest [FMDev ver 1]
    }

    # Reset FM Deviation to typical values after completion of the test
    FMDev enable on
    FMDev run on
    FMDev avg 1000
    FMDev trigger continuous
    FMDev alerts off
    FMDev math peak
    FMDev peakhold off
    FMDev squelch off
 }

}

# Add this test to the list of tests
::BIT::addTest cmdline_FMDev



