#############################################################################
# COPYRIGHT (C) 2001, 2003, 2005  Aeroflex, Inc.                            #
# --------------------------------------------------------------------------#
# FILE        : 1004.test_Dcs.tcl                                           #
# PRODUCT     : FITS                                                        #
# AUTHOR      : Bruce Butterfield                                           #
# DESCRIPTION : Remote Command Line Test:  Digital Coded Squelch (DCS)      #
# --------------------------------------------------------------------------#
# Update      : Resynch and engine cleanup for 1.9.1.4                      #
#############################################################################

package require Dcs
package require BIT

#****************************************************************************
# Test variables
#****************************************************************************
namespace eval ::cmdline_Dcs {
    variable testCaseList		;# List of test cases
    variable shortDesc			;# Test description - menu size
    variable longDesc			;# Test description - used by reports
    variable testTime			;# Test duration (Seconds - approximation)

    # Define the list of Digital Coded Squelch tests
    set testCaseList [list \
           testDcsList1 testDcsList2 ]


    # Declare each test as a namespace variable
    foreach testName $testCaseList {
       variable $testName
    }

    set shortDesc "Dcs cmds"
    set longDesc "Digital Coded Squelch command line test"
    set testTime 0.50

#****************************************************************************
# Name:        setup
# Parameters:  none
# Description: Sets up a list of Digital Coded Squelch tests
#****************************************************************************
 proc setup { } {
    variable testCaseList

    # Declare each test as a namespace variable
    foreach testName $testCaseList {
       variable $testName
    }

    #
    # Define test cases
    #
    set testDcsList1 [list \
      {"Dcs en -1 -forgive" 3 "INT" 0} \
      {"Dcs en 0" 1 "INT" 0} \
      {"Dcs en 1" 1 "INT" 0} \
      {"Dcs en 2" 3 "INT" 0} \
      {"Dcs en 3" 3 "INT" 0} \
      {"Dcs en 4 -forgive" 1 "INT" 0} \
      {"Dcs enable -1 -forgive" 3 "INT" 0} \
      {"Dcs enable 0" 1 "INT" 0} \
      {"Dcs enable 1" 1 "INT" 0} \
      {"Dcs enable 2" 3 "INT" 0} \
      {"Dcs enable 3" 3 "INT" 0} \
      {"Dcs enable disable" 1 "INT" 0} \
      {"Dcs enable enable" 3 "INT" 0} \
      {"Dcs enable off" 1 "INT" 0} \
      {"Dcs enable on" 3 "INT" 0} \
      {"Dcs enable 4 -forgive" 1 "INT" 0} ]

    set testDcsList2 [list \
      {"Dcs run -1 -forgive" 0 "INT" 0} \
      {"Dcs run 1" 1 "INT" 0} \
      {"Dcs run 0" 0 "INT" 0} \
      {"Dcs run on" 1 "INT" 0} \
      {"Dcs run off" 0 "INT" 0} \
      {"Dcs run 4 -forgive" 0 "INT" 0} ]

    # At the moment this test is useless. There is no obvious way
    # to predict what the binary value received in the READING will
    # be, and without that value, or a way of masking that value,
    # the compare test fails
    set testDcsList10 {
      {"Generator freq 123.4 " 123.4 "DBL" 0.001} 
      {"Generator rflevel -50.0" -50.0 "DBL" 0.001} 
      {"Generator rfoutput t/r" 0 "INT" 0} 
      {"Generator dcpower on" 1 "INT" 0} 
      {"Generator enable 11" 11 "INT" 0} 
      {"Generator mod 1" 1 "INT" 0} 
      {"Generator ptt ON" 1 "INT" 0} 
      {"Receiver freq 123.4" 123.4 "DBL" 0.001} 
      {"Receiver rfinput t/r" 0 "INT" 0} 
      {"Receiver atten 0" 0 "DBL" 0.001} 
      {"Receiver ifbw 30000" 30000 "INT" 0} 
      {"Receiver demod 0" 0 "INT" 0} 
      {"Receiver filter 0" 0 "INT" 0} 
      {"Receiver squelchmode 1" 1 "INT" 0} 
      {"Fgen mod3 1" 1 "INT" 0}
      {"Fgen freq3 1000.0" 1000.0 "DBL" 0.001}
      {"Fgen dev3 5.0" {1.0 10.0 5.0} "compareDbls" 0.001}
      {"Fgen mod4 0" 0 "INT" 0}
      {"FGen::setMode 1 0" 0 "NOCMP" 0}
      {"::FGen::setMode 3 DCS" 0 "NOCMP" 0}
      {"BBAudio mixer L1 DEMOD 1" {0 1} "compareInts" 0}
      {"BBAudio clearmixer L1" 3 "INT" 0}
      {"BBAudio mixer FGEN3 L1 1" {7 3} "compareInts" 0}
      {"Dcs enable on" 3 "INT" 0} \
      {"Dcs run on" 1 "INT" 0} \

      {"::DCS_UI::dcs_stop 3" 0 "NOCMP" 0}
      {"set ::FGen::State(3,DCS,code) 431" 0 "NOCMP" 0}
      {"after 3000" 0 "DELAY" 0}
      {"::DCS_UI::dcs_start 3" 0 "NOCMP" 0}
      {"after 10000" 0 "DELAY" 0}
      {"Dcs reading" {5839320 5839320 0} "compareInts" 0}

      {"::DCS_UI::dcs_stop 3" 0 "NOCMP" 0}
      {"set ::FGen::State(3,DCS,code) 143" 0 "NOCMP" 0}
      {"after 3000" 0 "DELAY" 0}
      {"::DCS_UI::dcs_start 3" 0 "NOCMP" 0}
      {"after 10000" 0 "DELAY" 0}
      {"Dcs reading" {6953179 6953179 0} "compareInts" 0}

      {"::DCS_UI::dcs_stop 3" 0 "NOCMP" 0}
      }

 }

#****************************************************************************
# Name:        autoTest
# Parameters:  resultsLevel		Indicates the results display level
# Description: Executes automatic test on Digital Coded Squelch commands
#****************************************************************************
 proc autoTest { resultsLevel } {
    variable testCaseList

    # Declare each test as a namespace variable
    foreach testName $testCaseList {
       variable $testName
    }

    setup 

    # Execute the test lists, and then destroy the test lists
    ::BIT::resetTestStats "cmdline_Dcs"
    foreach testName $testCaseList {
       ::BIT::runCmdLineTests "cmdline_Dcs" $testName
       unset $testName
    }

    if { ($resultsLevel == "Full") || ($resultsLevel == "Manual") } \
    {
       # Test commands with no data - requires visual inspection

       # Test string functions - requires visual inspection
        foreach cmd {enable reading res run version} \
       {
          set temp [AFCounter $cmd -range]
          ::BIT::putsTest "The range of <$cmd> is:  $temp"
       }

       ::BIT::putsTest [Dcs res]
       ::BIT::putsTest [Dcs resources]

       ::BIT::putsTest [Dcs ver 0]
       ::BIT::putsTest [Dcs ver 1]
       ::BIT::putsTest [Dcs version]

       ::BIT::putsTest [Dcs reading]
   }

   # Reset Digital Coded Squelch to typical values after completion of the test
   Dcs en 3
   Dcs run 0
 }

}

# Add this test to the list of tests
::BIT::addTest cmdline_Dcs


