#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C) 2001, 2003  DESIGN ENGINEERING      #
# --------------------------------------------------------------------------#
# FILE        : test_Generator.tcl                                          #
# PRODUCT     : FITS                                                        #
# AUTHOR      : Bruce Butterfield                                           #
# DESCRIPTION : Remote Command Line Test: RF Generator                      #
#############################################################################

package require Generator
package require BIT

#****************************************************************************
# Test variables
#****************************************************************************
namespace eval ::cmdline_Generator {
    variable testCaseList		;# List of test cases
    variable shortDesc			;# Test description - menu size
    variable longDesc			;# Test description - used by reports
    variable testTime			;# Test duration (Seconds - approximation)

    # Define the list of Generator tests
    set testCaseList [list \
           testGenList1 testGenList2 testGenList3 testGenList4 testGenList5 \
           testGenList6 testGenList7 testGenList8a testGenList8b testGenList9 testGenList10 \
           testGenList11 testGenList12 testGenList13 testGenList14 testGenList15 ]

    # Declare each test as a namespace variable
    foreach testName $testCaseList {
       variable $testName
    }

    set shortDesc "Generator cmds"
    set longDesc "Generator command line test"
    set testTime 26

#****************************************************************************
# Name:        setup
# Parameters:  none
# Description: Sets up a list of AF Counter tests
#****************************************************************************
 proc setup { } {
    variable testCaseList

    # Declare each test as a namespace variable
    foreach testName $testCaseList {
       variable $testName
    }

    #
    # Define test cases
    #
# Engine driver fault: Engine defines "alarm" which is not supported in driver
#      {"Generator en 4" 5 "INT" 0} \
#      {"Generator enable 4" 5 "INT" 0} \
# Engine issue: FORGIVE is ignored for this feature
#      {"Generator en -1 -forgive" 1 "INT" 0} \
#      {"Generator en 32 -forgive" 1 "INT" 0} \
#      {"Generator enable -1 -forgive" 1 "INT" 0} \
#      {"Generator enable 32 -forgive" 1 "INT" 0} ]

    set testGenList1 [list \
      {"Generator en 0" 1 "INT" 0} \
      {"Generator en 1" 1 "INT" 0} \
      {"Generator en 2" 3 "INT" 0} \
      {"Generator en 8" 9 "INT" 0} \
      {"Generator en 16" 17 "INT" 0} \
      {"Generator en 27" 27 "INT" 0} \
      {"Generator enable 0" 1 "INT" 0} \
      {"Generator enable 1" 1 "INT" 0} \
      {"Generator enable 2" 3 "INT" 0} \
      {"Generator enable 8" 9 "INT" 0} \
      {"Generator enable 16" 17 "INT" 0} \
      {"Generator enable 27" 27 "INT" 0} ]

    set testGenList2 [list \
      {"Generator debug rawFreq -1 -forgive" 0.0 "DBL" 0.001} \
      {"Generator debug rawFreq 0 " 0.0 "DBL" 0.001} \
      {"Generator debug rawFreq 10 " 10.0 "DBL" 0.001} \
      {"Generator debug rawFreq 100 " 100.0 "DBL" 0.001} \
      {"Generator debug rawFreq 1000 " 1000.0 "DBL" 0.001} \
      {"Generator debug rawFreq 2799.9 " 2799.9 "DBL" 0.001} \
      {"Generator debug rawFreq 2800.3  -forgive" 2800.0 "DBL" 0.001} \
      {"Generator debug rawFreq 55 mhz" 55.0 "DBL" 0.001} \
      {"Generator debug rawFreq 555 mhz" 555.0 "DBL" 0.001} \
      {"Generator debug rawFreq 555 khz" 0.555 "DBL" 0.001} \
      {"Generator debug rawFreq 0.555 ghz" 555.0 "DBL" 0.001} \
      {"Generator debug rawFrequency -1 -forgive" 0.0 "DBL" 0.001} \
      {"Generator debug rawFrequency 0 " 0.0 "DBL" 0.001} \
      {"Generator debug rawFrequency 10 " 10.0 "DBL" 0.001} \
      {"Generator debug rawFrequency 100 " 100.0 "DBL" 0.001} \
      {"Generator debug rawFrequency 1000 " 1000.0 "DBL" 0.001} \
      {"Generator debug rawFrequency 2799.9 " 2799.9 "DBL" 0.001} \
      {"Generator debug rawFrequency 3000.0  -forgive" 2800.0 "DBL" 0.001} \
      {"Generator debug rawFrequency 55 mhz" 55.0 "DBL" 0.001} \
      {"Generator debug rawFrequency 555 mhz" 555.0 "DBL" 0.001} \
      {"Generator debug rawFrequency 555 khz" 0.555 "DBL" 0.001} \
      {"Generator debug rawFrequency 0.555 ghz" 555.0 "DBL" 0.001} ]

# Engine issue: FORGIVE is ignored for this feature until a valid setting for an
# out of range condition is defined.
#      {"Generator rawrflev -1 -forgive" 0 "INT" 0} \
#      {"Generator rawrflev 4096 -forgive" 1 "INT" 0} \
#      {"Generator RawRFLevel -1 -forgive" 0 "INT" 0} \
#      {"Generator RawRFLevel 4096 -forgive" 1 "INT" 0} ]

    set testGenList3 [list \
      {"Generator debug rawrflev 0" 0 "INT" 0} \
      {"Generator debug rawrflev 1" 1 "INT" 0} \
      {"Generator debug rawrflev 10" 10 "INT" 0} \
      {"Generator debug rawrflev 100" 100 "INT" 0} \
      {"Generator debug rawrflev 1000" 1000 "INT" 0} \
      {"Generator debug rawrflev 4095" 4095 "INT" 0} \
      {"Generator debug rawrflev" 4095 "INT" 0} \
      {"Generator debug RawRFLevel 0" 0 "INT" 0} \
      {"Generator debug RawRFLevel 1" 1 "INT" 0} \
      {"Generator debug RawRFLevel 10" 10 "INT" 0} \
      {"Generator debug RawRFLevel 100" 100 "INT" 0} \
      {"Generator debug RawRFLevel 1000" 1000 "INT" 0} \
      {"Generator debug RawRFLevel 4095" 4095 "INT" 0} ]

    set testGenList4 [list \
      {"Generator debug rawrfout -1 -forgive" 0 "INT" 0} \
      {"Generator debug rawrfout 0" 0 "INT" 0} \
      {"Generator debug rawrfout 1" 1 "INT" 0} \
      {"Generator debug rawrfout 2 -forgive" 1 "INT" 0} \
      {"Generator debug RawRFOutput -1 -forgive" 0 "INT" 0} \
      {"Generator debug RawRFOutput 0" 0 "INT" 0} \
      {"Generator debug RawRFOutput 1" 1 "INT" 0} \
      {"Generator debug RawRFOutput 2 -forgive" 1 "INT" 0} ]

    set testGenList5 [list \
      {"Generator debug rawatten -1 -forgive" 0 "INT" 0} \
      {"Generator debug rawatten 0" 0 "INT" 0} \
      {"Generator debug rawatten 1" 1 "INT" 0} \
      {"Generator debug rawatten 10" 10 "INT" 0} \
      {"Generator debug rawatten 100" 100 "INT" 0} \
      {"Generator debug rawatten 110" 110 "INT" 0} \
      {"Generator debug rawatten 111 -forgive" 110 "INT" 0} \
      {"Generator debug rawatten" 110 "INT" 0} \
      {"Generator debug rawattenuator -1 -forgive" 0 "INT" 0} \
      {"Generator debug rawattenuator 0" 0 "INT" 0} \
      {"Generator debug rawattenuator 1" 1 "INT" 0} \
      {"Generator debug rawattenuator 10" 10 "INT" 0} \
      {"Generator debug rawattenuator 100" 100 "INT" 0} \
      {"Generator debug rawattenuator 110" 110 "INT" 0} \
      {"Generator debug rawattenuator 111 -forgive" 110 "INT" 0} \
      {"Generator debug rawattenuator" 110 "INT" 0} ]

    set testGenList6 [list \
      {"Generator Freq -1 -forgive" 0.0 "DBL" 0.001} \
      {"Generator Freq 0 " 0.0 "DBL" 0.001} \
      {"Generator Freq 10 " 10.0 "DBL" 0.001} \
      {"Generator Freq 100 " 100.0 "DBL" 0.001} \
      {"Generator Freq 1000 " 1000.0 "DBL" 0.001} \
      {"Generator Freq 2799.9 " 2799.9 "DBL" 0.001} \
      {"Generator Freq 2800.3  -forgive" 2800.0 "DBL" 0.001} \
      {"Generator Freq 55 mhz" 55.0 "DBL" 0.001} \
      {"Generator Freq 555 mhz" 555.0 "DBL" 0.001} \
      {"Generator Freq 555 khz" 0.555 "DBL" 0.001} \
      {"Generator Freq 0.555 ghz" 555.0 "DBL" 0.001} \
      {"Generator Frequency -1 -forgive" 0.0 "DBL" 0.001} \
      {"Generator Frequency 0 " 0.0 "DBL" 0.001} \
      {"Generator Frequency 10 " 10.0 "DBL" 0.001} \
      {"Generator Frequency 100 " 100.0 "DBL" 0.001} \
      {"Generator Frequency 1000 " 1000.0 "DBL" 0.001} \
      {"Generator Frequency 2799.9 " 2799.9 "DBL" 0.001} \
      {"Generator Frequency 3000.0  -forgive" 2800.0 "DBL" 0.001} \
      {"Generator Frequency 55 mhz" 55.0 "DBL" 0.001} \
      {"Generator Frequency 555 mhz" 555.0 "DBL" 0.001} \
      {"Generator Frequency 555 khz" 0.555 "DBL" 0.001} \
      {"Generator Frequency 0.555 ghz" 555.0 "DBL" 0.001} ]

    set testGenList7 [list \
      {"Generator rfout 1" 1 "INT" 0} \
      {"Generator rfout -1 -forgive" 0 "INT" 0} \
      {"Generator rfout 1" 1 "INT" 0} \
      {"Generator rfout 0" 0 "INT" 0} \
      {"Generator rfout gen" 1 "INT" 0} \
      {"Generator rfout tr" 0 "INT" 0} \
      {"Generator rfout t/r" 0 "INT" 0} \
      {"Generator rfout 2 -forgive" 1 "INT" 0} \
      {"Generator RFOutput -1 -forgive" 0 "INT" 0} \
      {"Generator RFOutput 1" 1 "INT" 0} \
      {"Generator RFOutput 0" 0 "INT" 0} \
      {"Generator RFOutput 2 -forgive" 1 "INT" 0} ]

    # This test is for the T/R port
    set testGenList8a [list \
      {"Generator rfout t/r" 0 "INT" 0} \
      {"Generator rflev -138 -forgive" -137.0 "DBL" 0.001} \
      {"Generator rflev -137.0" -137.0 "DBL" 0.001} \
      {"Generator rflev -100.0" -100.0 "DBL" 0.001} \
      {"Generator rflev -50.0" -50.0 "DBL" 0.001} \
      {"Generator rflev -30.0" -30.0 "DBL" 0.001} \
      {"Generator rflev -29.00 -forgive" -30.0 "DBL" 0.001} \
      {"Generator RFLevel -138 -forgive" -137.0 "DBL" 0.001} \
      {"Generator RFLevel -137.0" -137.0 "DBL" 0.001} \
      {"Generator RFLevel -100.0" -100.0 "DBL" 0.001} \
      {"Generator RFLevel -50.0" -50.0 "DBL" 0.001} \
      {"Generator RFLevel -30.0" -30.0 "DBL" 0.001} \
      {"Generator RFLevel -29.00 -forgive" -30.0 "DBL" 0.001} ]

    # This test is for the GEN port
    set testGenList8b [list \
      {"Generator rfout gen" 1 "INT" 0} \
      {"Generator rflev -111 -forgive" -110.0 "DBL" 0.001} \
      {"Generator rflev -110.0" -110.0 "DBL" 0.001} \
      {"Generator rflev -100.0" -100.0 "DBL" 0.001} \
      {"Generator rflev 0.0" 0.0 "DBL" 0.001} \
      {"Generator rflev 10.0" 10.0 "DBL" 0.001} \
      {"Generator rflev 11.00 -forgive" 10.0 "DBL" 0.001} \
      {"Generator RFLevel -111 -forgive" -110.0 "DBL" 0.001} \
      {"Generator RFLevel -110.0" -110.0 "DBL" 0.001} \
      {"Generator RFLevel -100.0" -100.0 "DBL" 0.001} \
      {"Generator RFLevel 0.0" 0.0 "DBL" 0.001} \
      {"Generator RFLevel 10.0" 10.0 "DBL" 0.001} \
      {"Generator RFLevel 11.00 -forgive" 10.0 "DBL" 0.001} \
      {"Generator RFLevel -50.00" -50.0 "DBL" 0.001} ]

    set testGenList9 [list \
      {"Generator alc 1" 1 "INT" 0} \
      {"Generator alc -1 -forgive" 0 "INT" 0} \
      {"Generator alc" 0 "INT" 0} \
      {"Generator alc 0" 0 "INT" 0} \
      {"Generator alc 1" 1 "INT" 0} \
      {"Generator alc 2" 2 "INT" 0} \
      {"Generator alc slow" 0 "INT" 0} \
      {"Generator alc fast" 1 "INT" 0} \
      {"Generator alc digital" 2 "INT" 0} \
      {"Generator alc 3 -forgive" 0 "INT" 0} \
      {"Generator alc" 0 "INT" 0} \
      {"Generator hwalc 1" 1 "INT" 0} \
      {"Generator hwalc -1 -forgive" 0 "INT" 0} \
      {"Generator hwalc" 0 "INT" 0} \
      {"Generator hwalc 0" 0 "INT" 0} \
      {"Generator hwalc 1" 1 "INT" 0} \
      {"Generator hwalc 2" 2 "INT" 0} \
      {"Generator hwalc 3 -forgive" 0 "INT" 0} \
      {"Generator hwalc" 0 "INT" 0} ]

    set testGenList10 [list \
      {"Generator bw 1" 1 "INT" 0} \
      {"Generator bw -1 -forgive" 0 "INT" 0} \
      {"Generator bw" 0 "INT" 0} \
      {"Generator bw 0" 0 "INT" 0} \
      {"Generator bw 1" 1 "INT" 0} \
      {"Generator bw 2 -forgive" 0 "INT" 0} \
      {"Generator bw" 0 "INT" 0} \
      {"Generator hwbw 1" 1 "INT" 0} \
      {"Generator hwbw -1 -forgive" 0 "INT" 0} \
      {"Generator hwbw" 0 "INT" 0} \
      {"Generator hwbw 0" 0 "INT" 0} \
      {"Generator hwbw 1" 1 "INT" 0} \
      {"Generator hwbw 2 -forgive" 0 "INT" 0} \
      {"Generator hwbw" 0 "INT" 0} ]

    set testGenList11 [list \
      {"Generator dcpwr 1" 1 "INT" 0} \
      {"Generator dcpwr -1 -forgive" 0 "INT" 0} \
      {"Generator dcpwr" 0 "INT" 0} \
      {"Generator dcpwr 0" 0 "INT" 0} \
      {"Generator dcpwr 1" 1 "INT" 0} \
      {"Generator dcpwr 2 -forgive" 1 "INT" 0} \
      {"Generator dcpwr" 1 "INT" 0} \
      {"Generator dcpower 1" 1 "INT" 0} \
      {"Generator dcpower -1 -forgive" 0 "INT" 0} \
      {"Generator dcpower" 0 "INT" 0} \
      {"Generator dcpower 0" 0 "INT" 0} \
      {"Generator dcpower 1" 1 "INT" 0} \
      {"Generator dcpower 2 -forgive" 1 "INT" 0} \
      {"Generator dcpower" 1 "INT" 0} ]

    set testGenList12 [list \
      {"Generator yigspeed 1" 1 "INT" 0} \
      {"Generator yigspeed -1 -forgive" 0 "INT" 0} \
      {"Generator yigspeed" 0 "INT" 0} \
      {"Generator yigspeed 0" 0 "INT" 0} \
      {"Generator yigspeed 1" 1 "INT" 0} \
      {"Generator yigspeed 2 -forgive" 1 "INT" 0} \
      {"Generator yigspeed" 1 "INT" 0} ]

      # Disable tests for obsolete modulation types
      # Mod type 11 is disabled because it requires the load of an option
#      {"Generator mod 3" 3 "INT" 0} \
#      {"Generator mod 5" 5 "INT" 0} \
#      {"Generator mod 6" 6 "INT" 0} \
#      {"Generator mod 7" 7 "INT" 0} \
#      {"Generator mod 11" 11 "INT" 0} \
#      {"Generator modulation 3" 3 "INT" 0} \
#      {"Generator modulation 5" 5 "INT" 0} \
#      {"Generator modulation 6" 6 "INT" 0} \
#      {"Generator modulation 7" 7 "INT" 0} \
#      {"Generator modulation 11" 11 "INT" 0} \


    set testGenList13 [list \
      {"Generator mod 1" 1 "INT" 0} \
      {"Generator mod -1 -forgive" 0 "INT" 0} \
      {"Generator mod" 0 "INT" 0} \
      {"Generator mod 0" 0 "INT" 0} \
      {"Generator mod 1" 1 "INT" 0} \
      {"Generator mod 2" 2 "INT" 0} \
      {"Generator mod 4" 4 "INT" 0} \
      {"Generator mod 8" 8 "INT" 0} \
      {"Generator mod 9" 9 "INT" 0} \
      {"Generator mod 10" 10 "INT" 0} \
      {"Generator mod" 10 "INT" 0} \
      {"Generator modulation 1" 1 "INT" 0} \
      {"Generator modulation -1 -forgive" 0 "INT" 0} \
      {"Generator modulation" 0 "INT" 0} \
      {"Generator modulation 0" 0 "INT" 0} \
      {"Generator modulation 1" 1 "INT" 0} \
      {"Generator modulation 2" 2 "INT" 0} \
      {"Generator modulation 4" 4 "INT" 0} \
      {"Generator modulation 8" 8 "INT" 0} \
      {"Generator modulation 9" 9 "INT" 0} \
      {"Generator modulation 10" 10 "INT" 0} \
      {"Generator modulation" 10 "INT" 0} ]

    set testGenList14 [list \
      {"Generator p25m 1" 1 "INT" 0} \
      {"Generator p25m -1 -forgive" 0 "INT" 0} \
      {"Generator p25m" 0 "INT" 0} \
      {"Generator p25m 0" 0 "INT" 0} \
      {"Generator p25m 1" 1 "INT" 0} \
      {"Generator p25m 2" 2 "INT" 0} \
      {"Generator p25m 3" 3 "INT" 0} \
      {"Generator p25m 4" 4 "INT" 0} \
      {"Generator p25m 5" 5 "INT" 0} \
      {"Generator p25m 6" 6 "INT" 0} \
      {"Generator p25m 7" 7 "INT" 0} \
      {"Generator p25m 8" 8 "INT" 0} \
      {"Generator p25m 9" 9 "INT" 0} \
      {"Generator p25m 10" 10 "INT" 0} \
      {"Generator p25m 11" 11 "INT" 0} \
      {"Generator p25m 12" 12 "INT" 0} \
      {"Generator p25m 13" 13 "INT" 0} \
      {"Generator p25m 14" 14 "INT" 0} \
      {"Generator p25m 15" 15 "INT" 0} \
      {"Generator p25m 16" 16 "INT" 0} \
      {"Generator p25m 17" 17 "INT" 0} \
      {"Generator p25m 18" 18 "INT" 0} \
      {"Generator p25m 19" 19 "INT" 0} \
      {"Generator p25m 20" 20 "INT" 0} \
      {"Generator p25m 21 -forgive" 0 "INT" 0} \
      {"Generator p25m" 0 "INT" 0} \
      {"Generator p25mode 1" 1 "INT" 0} \
      {"Generator p25mode -1 -forgive" 0 "INT" 0} \
      {"Generator p25mode" 0 "INT" 0} \
      {"Generator p25mode 0" 0 "INT" 0} \
      {"Generator p25mode 5" 5 "INT" 0} \
      {"Generator p25mode 12" 12 "INT" 0} \
      {"Generator p25mode 16" 16 "INT" 0} \
      {"Generator p25mode 19" 19 "INT" 0} \
      {"Generator p25mode 20" 20 "INT" 0} \
      {"Generator p25mode 21 -forgive" 0 "INT" 0} \
      {"Generator p25mode" 0 "INT" 0} ]

    set testGenList15 [list \
      {"Generator ptt 1" 1 "INT" 0} \
      {"Generator ptt -1 -forgive" 0 "INT" 0} \
      {"Generator ptt" 0 "INT" 0} \
      {"Generator ptt 0" 0 "INT" 0} \
      {"Generator ptt 1" 1 "INT" 0} \
      {"Generator ptt 2" 2 "INT" 0} \
      {"Generator ptt 3 -forgive" 2 "INT" 0} \
      {"Generator ptt" 2 "INT" 0} \
      {"Generator pushtotalk 1" 1 "INT" 0} \
      {"Generator pushtotalk -1 -forgive" 0 "INT" 0} \
      {"Generator pushtotalk" 0 "INT" 0} \
      {"Generator pushtotalk 0" 0 "INT" 0} \
      {"Generator pushtotalk 1" 1 "INT" 0} \
      {"Generator pushtotalk 2" 2 "INT" 0} \
      {"Generator pushtotalk 3 -forgive" 2 "INT" 0} \
      {"Generator pushtotalk" 2 "INT" 0} ]


}

#****************************************************************************
# Name:        autoTest
# Parameters:  resultsLevel		Indicates the results display level
# Description: Executes automatic test on Generator commands
#****************************************************************************
 proc autoTest { resultsLevel } {
    variable testCaseList

    # Declare each test as a namespace variable
    foreach testName $testCaseList {
       variable $testName
    }

    setup 

    # Execute the test lists, and then destroy the test lists
    ::BIT::resetTestStats "cmdline_Generator"
    foreach testName $testCaseList {
       ::BIT::runCmdLineTests "cmdline_Generator" $testName
       unset $testName
    }

    if { ($resultsLevel == "Full") || ($resultsLevel == "Manual") } \
    {
       # Test commands with no data - requires visual inspection

       # Test string functions - requires visual inspection
       ::BIT::putsTest [Generator ver]
       ::BIT::putsTest [Generator version 1]
    }

   # Reset Generator to typical values after completion of the test
   Generator enable 1
   Generator dcpower 0
 }

}

# Add this test to the list of tests
::BIT::addTest cmdline_Generator


