#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C) 2001, 2003  DESIGN ENGINEERING      #
# --------------------------------------------------------------------------#
# FILE        : test_Receiver.tcl                                           #
# PRODUCT     : FITS                                                        #
# AUTHOR      : Bruce Butterfield                                           #
# DESCRIPTION : Remote Command Line Test: Receiver                          #
#############################################################################

package require Receiver
package require BIT

#****************************************************************************
# Test variables
#****************************************************************************
namespace eval ::cmdline_Receiver {
    variable testCaseList		;# List of test cases
    variable shortDesc			;# Test description - menu size
    variable longDesc			;# Test description - used by reports
    variable testTime			;# Test duration (Seconds - approximation)

    # Define the list of Receiver tests
    set testCaseList [list \
         testReceiverList1 testReceiverList2 testReceiverList3 \
	   testReceiverList4 testReceiverList5 testReceiverList6 \
	   testReceiverList7 testReceiverList8 testReceiverList9 \
	   testReceiverList10 testReceiverList11 testReceiverList12 \
	   testReceiverList13 testReceiverList14 ]

    # Declare each test as a namespace variable
    foreach testName $testCaseList {
       variable $testName
    }

    set shortDesc "Receiver cmds"
    set longDesc "Receiver command line test"
    set testTime 0.50

#****************************************************************************
# Name:        setup
# Parameters:  none
# Description: Sets up a list of AF Counter tests
#****************************************************************************
 proc setup { } {
    variable testCaseList

    # Declare each test as a namespace variable
    foreach testName $testCaseList {
       variable $testName
    }

    #
    # Define test cases
    #
    set testReceiverList1 [list \
      {"Receiver enable 0" 1 "INT" 0} \
      {"Receiver enable -1 -forgive" 1 "INT" 0} \
      {"Receiver enable" 1 "INT" 0} \
      {"Receiver enable 1" 1 "INT" 0} \
      {"Receiver enable 2" 3 "INT" 0} \
      {"Receiver enable 8" 9 "INT" 0} \
      {"Receiver enable 16" 17 "INT" 0} \
      {"Receiver enable 32" 33 "INT" 0} \
      {"Receiver enable 64 -forgive" 1 "INT" 0} \
      {"Receiver enable" 1 "INT" 0} \
      {"Receiver MessageEnable 0" 1 "INT" 0} \
      {"Receiver MessageEnable -1 -forgive" 1 "INT" 0} \
      {"Receiver MessageEnable" 1 "INT" 0} \
      {"Receiver MessageEnable 1" 1 "INT" 0} \
      {"Receiver MessageEnable 2" 3 "INT" 0} \
      {"Receiver MessageEnable 8" 9 "INT" 0} \
      {"Receiver MessageEnable 16" 17 "INT" 0} \
      {"Receiver MessageEnable 32" 33 "INT" 0} \
      {"Receiver MessageEnable 64 -forgive" 1 "INT" 0} \
      {"Receiver MessageEnable" 1 "INT" 0} ] 


#{"Receiver freq -1 -forgive" 0.0 "DBL" 0.001}
#{"Receiver freq 2800.3  -forgive" 2800.0 "DBL" 0.001} 
#{"Receiver frequency -1 -forgive" 0.0 "DBL" 0.001} 
#{"Receiver frequency 2800.3  -forgive" 2800.0 "DBL" 0.001} 

    set testReceiverList2 [list \
      {"Receiver freq 0 " 0.0 "DBL" 0.001} \
      {"Receiver freq 10 " 10.0 "DBL" 0.001} \
      {"Receiver freq 100 " 100.0 "DBL" 0.001} \
      {"Receiver freq 1000 " 1000.0 "DBL" 0.001} \
      {"Receiver freq 2799.9 " 2799.9 "DBL" 0.001} \
      {"Receiver freq 55 mhz" 55.0 "DBL" 0.001} \
      {"Receiver freq 555 mhz" 555.0 "DBL" 0.001} \
      {"Receiver freq 555 khz" 0.555 "DBL" 0.001} \
      {"Receiver freq 0.555 ghz" 555.0 "DBL" 0.001} \
      {"Receiver frequency 0 " 0.0 "DBL" 0.001} \
      {"Receiver frequency 10 " 10.0 "DBL" 0.001} \
      {"Receiver frequency 100 " 100.0 "DBL" 0.001} \
      {"Receiver frequency 1000 " 1000.0 "DBL" 0.001} \
      {"Receiver frequency 2799.9 " 2799.9 "DBL" 0.001} \
      {"Receiver frequency 55 mhz" 55.0 "DBL" 0.001} \
      {"Receiver frequency 555 mhz" 555.0 "DBL" 0.001} \
      {"Receiver frequency 555 khz" 0.555 "DBL" 0.001} \
      {"Receiver frequency 0.555 ghz" 555.0 "DBL" 0.001} ]

    set testReceiverList3 [list \
      {"Receiver rfin 0" 0 "INT" 0} \
      {"Receiver rfin -1 -forgive" 0 "INT" 0} \
      {"Receiver rfin" 0 "INT" 0} \
      {"Receiver rfin 1" 1 "INT" 0} \
      {"Receiver rfin 2 -forgive" 0 "INT" 0} \
      {"Receiver rfin" 0 "INT" 0} \
      {"Receiver rfin tr" 0 "INT" 0} \
      {"Receiver rfin ant" 1 "INT" 0} \
      {"Receiver rfin t/r" 0 "INT" 0} \
      {"Receiver rfinput 0" 0 "INT" 0} \
      {"Receiver rfinput -1 -forgive" 0 "INT" 0} \
      {"Receiver rfinput" 0 "INT" 0} \
      {"Receiver rfinput 1" 1 "INT" 0} \
      {"Receiver rfinput 2 -forgive" 0 "INT" 0} \
      {"Receiver rfinput" 0 "INT" 0} \
      {"Receiver rfinput tr" 0 "INT" 0} \
      {"Receiver rfinput ant" 1 "INT" 0} \
      {"Receiver rfinput t/r" 0 "INT" 0} ]

    set testReceiverList4 [list \
      {"Receiver atten -50" 50 "INT" 0} \
      {"Receiver atten -51 -forgive" 0 "INT" 0} \
      {"Receiver atten" 50 "INT" 0} \
      {"Receiver atten 30" 30 "INT" 0} \
      {"Receiver atten 50" 50 "INT" 0} \
      {"Receiver atten 51 -forgive" 0 "INT" 0} \
      {"Receiver atten" 50 "INT" 0} \
      {"Receiver attenuation -50" 50 "INT" 0} \
      {"Receiver attenuation -51 -forgive" 0 "INT" 0} \
      {"Receiver attenuation" 50 "INT" 0} \
      {"Receiver attenuation 30" 30 "INT" 0} \
      {"Receiver attenuation 50" 50 "INT" 0} \
      {"Receiver attenuation 51 -forgive" 0 "INT" 0} \
      {"Receiver attenuation" 50 "INT" 0} ]

    set testReceiverList5 [list \
      {"Receiver ifbw 30 khz" 30000 "INT" 0} \
      {"Receiver ifbw 300 khz" 300000 "INT" 0} \
      {"Receiver ifbw 6 mhz" 6000000 "INT" 0} \
      {"Receiver ifbw 1 khz -forgive" 30000 "INT" 0} \
      {"Receiver ifbw 29 khz -forgive" 30000 "INT" 0} \
      {"Receiver ifbw 0.0449 mhz -forgive" 30000 "INT" 0} \
      {"Receiver ifbw 0.0451 mhz -forgive" 300000 "INT" 0} \
      {"Receiver ifbw 100 khz -forgive" 300000 "INT" 0} \
      {"Receiver ifbw 500 khz -forgive" 300000 "INT" 0} \
      {"Receiver ifbw 999.99 khz -forgive" 300000 "INT" 0} \
      {"Receiver ifbw 1.0001 mhz -forgive" 6000000 "INT" 0} \
      {"Receiver ifbw 10000 khz -forgive" 6000000 "INT" 0} \
      {"Receiver ifbw 100 mhz -forgive" 6000000 "INT" 0} \
      {"Receiver ifbw 1.0 ghz -forgive" 6000000 "INT" 0} \
      {"Receiver ifbw 2.0 ghz -forgive" 6000000 "INT" 0} \
      {"Receiver ifbandwidth 30000" 30000 "INT" 0} \
      {"Receiver ifbandwidth 300000" 300000 "INT" 0} \
      {"Receiver ifbandwidth 6000000" 6000000 "INT" 0} \
      {"Receiver ifbandwidth 1 khz -forgive" 30000 "INT" 0} \
      {"Receiver ifbandwidth 29 khz -forgive" 30000 "INT" 0} \
      {"Receiver ifbandwidth 0.0449 mhz -forgive" 30000 "INT" 0} \
      {"Receiver ifbandwidth 0.0451 mhz -forgive" 300000 "INT" 0} \
      {"Receiver ifbandwidth 100 khz -forgive" 300000 "INT" 0} \
      {"Receiver ifbandwidth 500 khz -forgive" 300000 "INT" 0} \
      {"Receiver ifbandwidth 999.99 khz -forgive" 300000 "INT" 0} \
      {"Receiver ifbandwidth 1.0001 mhz -forgive" 6000000 "INT" 0} \
      {"Receiver ifbandwidth 10000 khz -forgive" 6000000 "INT" 0} \
      {"Receiver ifbandwidth 100 mhz -forgive" 6000000 "INT" 0} \
      {"Receiver ifbandwidth 1.0 ghz -forgive" 6000000 "INT" 0} \
      {"Receiver ifbandwidth 2.0 ghz -forgive" 6000000 "INT" 0} ]

    set testReceiverList6 [list \
      {"Receiver range 0" 0 "INT" 0} \
      {"Receiver range -1 -forgive" 1 "INT" 0} \
      {"Receiver range" 1 "INT" 0} \
      {"Receiver range 1" 1 "INT" 0} \
      {"Receiver range 2 -forgive" 1 "INT" 0} \
      {"Receiver range" 1 "INT" 0} \
      {"Receiver capturerange 0" 0 "INT" 0} \
      {"Receiver capturerange -1 -forgive" 1 "INT" 0} \
      {"Receiver capturerange" 1 "INT" 0} \
      {"Receiver capturerange 1" 1 "INT" 0} \
      {"Receiver capturerange 2 -forgive" 1 "INT" 0} \
      {"Receiver capturerange" 1 "INT" 0} ]

    set testReceiverList7 [list \
      {"Receiver lock 0" 0 "INT" 0} \
      {"Receiver lock -1 -forgive" 1 "INT" 0} \
      {"Receiver lock" 1 "INT" 0} \
      {"Receiver lock 1" 1 "INT" 0} \
      {"Receiver lock 2 -forgive" 1 "INT" 0} \
      {"Receiver lock" 1 "INT" 0} \
      {"Receiver sweeplock 0" 0 "INT" 0} \
      {"Receiver sweeplock -1 -forgive" 1 "INT" 0} \
      {"Receiver sweeplock" 1 "INT" 0} \
      {"Receiver sweeplock 1" 1 "INT" 0} \
      {"Receiver sweeplock 2 -forgive" 1 "INT" 0} \
      {"Receiver sweeplock" 1 "INT" 0} ]

    set testReceiverList8 [list \
      {"Receiver hwagc 0" 0 "INT" 0} \
      {"Receiver hwagc -1 -forgive" 0 "INT" 0} \
      {"Receiver hwagc" 0 "INT" 0} \
      {"Receiver hwagc 1" 1 "INT" 0} \
      {"Receiver hwagc 2 -forgive" 0 "INT" 0} \
      {"Receiver hwagc" 0 "INT" 0} ]

    set testReceiverList9 [list \
      {"Receiver agcmode 0" 0 "INT" 0} \
      {"Receiver agcmode -1 -forgive" 0 "INT" 0} \
      {"Receiver agcmode" 0 "INT" 0} \
      {"Receiver agcmode 1" 1 "INT" 0} \
      {"Receiver agcmode 2 -forgive" 0 "INT" 0} \
      {"Receiver agcmode" 1 "INT" 0} ]

# Avoid testing values which may not be available
# Codes 6-9 have not been added at this time
# Codes 12-13 are only available if an option has been selected
# "Forgive" on the upper range is unpredictible, since it must assume state of option installation
#      {"Receiver demod 6" 6 "INT" 0} \
#      {"Receiver demod 7" 7 "INT" 0} \
#      {"Receiver demod 8" 8 "INT" 0} \
#      {"Receiver demod 9" 9  "INT" 0} \
#      {"Receiver demod 12" 12 "INT" 0} \
#      {"Receiver demod 13" 13 "INT" 0} \
#      {"Receiver demodulation 6" 6 "INT" 0} \
#      {"Receiver demodulation 7" 7 "INT" 0} \
#      {"Receiver demodulation 8" 8 "INT" 0} \
#      {"Receiver demodulation 9" 9  "INT" 0} \
#      {"Receiver demodulation 12" 12 "INT" 0} \
#      {"Receiver demodulation 13" 13 "INT" 0} \

    set testReceiverList10 [list \
      {"Receiver demod 0" 0 "INT" 0} \
      {"Receiver demod -1 -forgive" 0 "INT" 0} \
      {"Receiver demod 1" 1 "INT" 0} \
      {"Receiver demod 2" 2 "INT" 0} \
      {"Receiver demod 3" 3 "INT" 0} \
      {"Receiver demod 4" 4 "INT" 0} \
      {"Receiver demod 5" 5 "INT" 0} \
      {"Receiver demod 10" 10 "INT" 0} \
      {"Receiver demod 11" 11 "INT" 0} \
      {"Receiver demodulation 0" 0 "INT" 0} \
      {"Receiver demodulation -1 -forgive" 0 "INT" 0} \
      {"Receiver demodulation 1" 1 "INT" 0} \
      {"Receiver demodulation 2" 2 "INT" 0} \
      {"Receiver demodulation 3" 3 "INT" 0} \
      {"Receiver demodulation 4" 4 "INT" 0} \
      {"Receiver demodulation 5" 5 "INT" 0} \
      {"Receiver demodulation 10" 10 "INT" 0} \
      {"Receiver demodulation 11" 11 "INT" 0} ]

    set testReceiverList11 [list \
      {"Receiver hop 0" 0 "INT" 0} \
      {"Receiver hop 31" 31 "INT" 0} \
      {"Receiver synthesizerhop 0" 0 "INT" 0} \
      {"Receiver synthesizerhop 31" 31 "INT" 0} ]


#{"Receiver ifgain 83.0 -forgive" 1 "INT" 0}
#{"Receiver ifgain -15.0 -forgive" 1 "INT" 0}

    set testReceiverList12 [list \
      {"Receiver ifgain 82.0" 82.0 "DBL" 0.001} \
      {"Receiver ifgain -14.0" -14.0 "DBL" 0.001} \
      {"Receiver ifgain 0.0" 0.0 "DBL" 0.001} ]

    set testReceiverList13 [list \
      {"Receiver squelchlev -100 " -100 "INT" 0} \
      {"Receiver squelchlev -141 -forgive" -140 "INT" 0} \
      {"Receiver squelchlev -50 " -50 "INT" 0} \
      {"Receiver squelchlev 0 " 0 "INT" 0} \
      {"Receiver squelchlev 100 " 100 "INT" 0} \
      {"Receiver squelchlev 500 " 500 "INT" 0} \
      {"Receiver squelchlev 800 " 800 "INT" 0} \
      {"Receiver squelchlev 822 -forgive " 821 "INT" 0} \
      {"Receiver squelchlev 300 " 300 "INT" 0} \
      {"Receiver squelchlevel -100 " -100 "INT" 0} \
      {"Receiver squelchlevel -141 -forgive" -140 "INT" 0} \
      {"Receiver squelchlevel -50 " -50 "INT" 0} \
      {"Receiver squelchlevel 0 " 0 "INT" 0} \
      {"Receiver squelchlevel 100 " 100 "INT" 0} \
      {"Receiver squelchlevel 500 " 500 "INT" 0} \
      {"Receiver squelchlevel 800 " 800 "INT" 0} \
      {"Receiver squelchlevel 822 -forgive " 821 "INT" 0} \
      {"Receiver squelchlevel 300 " 300 "INT" 0} ]

    set testReceiverList14 [list \
      {"Receiver squelchmod 0 -forgive" 1 "INT" 0} \
      {"Receiver squelchmod 1 " 1 "INT" 0} \
      {"Receiver squelchmod 2 " 2 "INT" 0} \
      {"Receiver squelchmod 3 " 3 "INT" 0} \
      {"Receiver squelchmod 4 " 3 "INT" 0} \
      {"Receiver squelchmod 5 " 3 "INT" 0} \
      {"Receiver squelchmod 6 " 3 "INT" 0} \
      {"Receiver squelchmod 7 -forgive" 3 "INT" 0} \
      {"Receiver squelchmod 2 " 2 "INT" 0} \
      {"Receiver squelchmod 4 " 2 "INT" 0} \
      {"Receiver squelchmod 5 " 2 "INT" 0} \
      {"Receiver squelchmod 6 " 2 "INT" 0} \
      {"Receiver squelchmod 7 -forgive" 2 "INT" 0} \
      {"Receiver squelchmod 1 " 1 "INT" 0} \
      {"Receiver squelchmod 4 " 1 "INT" 0} \
      {"Receiver squelchmod 5 " 1 "INT" 0} \
      {"Receiver squelchmod 6 " 1 "INT" 0} \
      {"Receiver squelchmod 7 -forgive" 1 "INT" 0} \
      {"Receiver squelchmode 0 -forgive" 1 "INT" 0} \
      {"Receiver squelchmode 1 " 1 "INT" 0} \
      {"Receiver squelchmode 2 " 2 "INT" 0} \
      {"Receiver squelchmode 3 " 3 "INT" 0} \
      {"Receiver squelchmode 4 " 3 "INT" 0} \
      {"Receiver squelchmode 5 " 3 "INT" 0} \
      {"Receiver squelchmode 6 " 3 "INT" 0} \
      {"Receiver squelchmode 7 -forgive" 3 "INT" 0} \
      {"Receiver squelchmode 2 " 2 "INT" 0} \
      {"Receiver squelchmode 4 " 2 "INT" 0} \
      {"Receiver squelchmode 5 " 2 "INT" 0} \
      {"Receiver squelchmode 6 " 2 "INT" 0} \
      {"Receiver squelchmode 7 -forgive" 2 "INT" 0} \
      {"Receiver squelchmode 1 " 1 "INT" 0} \
      {"Receiver squelchmode 4 " 1 "INT" 0} \
      {"Receiver squelchmode 5 " 1 "INT" 0} \
      {"Receiver squelchmode 6 " 1 "INT" 0} \
      {"Receiver squelchmode 7 -forgive" 1 "INT" 0} ]
 }

#****************************************************************************
# Name:        autoTest
# Parameters:  resultsLevel		Indicates the results display level
# Description: Executes automatic test on Receiver commands
#****************************************************************************
 proc autoTest { resultsLevel } {
    variable testCaseList

    # Declare each test as a namespace variable
    foreach testName $testCaseList {
       variable $testName
    }

    setup 

    # Execute the test lists, and then destroy the test lists
    ::BIT::resetTestStats "cmdline_Receiver"
    foreach testName $testCaseList {
       ::BIT::runCmdLineTests "cmdline_Receiver" $testName
       unset $testName
    }

    if { ($resultsLevel == "Full") || ($resultsLevel == "Manual") } \
    {
       # Test commands with no data - requires visual inspection

       # Test string functions - requires visual inspection
    }

    # Reset Receiver to typical values after completion of the test
    Receiver enable 1
    Receiver freq 170
 }

}

# Add this test to the list of tests
::BIT::addTest cmdline_Receiver
