#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C) 2001        DESIGN ENGINEERING      #
# --------------------------------------------------------------------------#
# FILE        : test_Scope.tcl                                              #
# PRODUCT     : FITS                                                        #
# AUTHOR      : Bruce Butterfield                                           #
# DESCRIPTION : Test basic Scope commands                                   #
#############################################################################

package require Scope

#############################################################################
# FUNCTION    : setupScopeTests                                             #
# ARGUMENTS   : none                                                        #
# RETURNS     : none                                                        #
# PURPOSE     : Sets up a list of Scope tests                               #
#############################################################################
proc setupScopeTests { } {
   global scopeTestList 

   # Define the list of Scope tests
   # testScopeList6 - Currently causes C++ interface errors which terminate the test

   set scopeTestList [list \
           testScopeList1 testScopeList2 testScopeList3 testScopeList4 \
           testScopeList5 testScopeList7 testScopeList8 \
           testScopeList9 testScopeList10 testScopeList11 \
           testScopeList12 testScopeList13 testScopeList14 ]

   # Declare each test as a global variable
   foreach testName $scopeTestList {
      global $testName
   }

   #
   # Define test cases
   #
# Disable limit conditions for software scope, because the sweep
# control is not handled right at this time and the engine ignores
# the LUN differences until things are fixed.
#      {"Scope sweep3 99 -forgive" 100 "INT" 0} \
#      {"Scope sweep3 1000001 -forgive" 1000000 "INT" 0} \

   set testScopeList1 [list \
      {"Scope sweep 0 -forgive" 1 "INT" 0} \
      {"Scope sweep 1" 1 "INT" 0} \
      {"Scope sweep 100" 100 "INT" 0} \
      {"Scope sweep 10000" 10000 "INT" 0} \
      {"Scope sweep 1000000" 1000000 "INT" 0} \
      {"Scope sweep 9999999" 9999999 "INT" 0} \
      {"Scope sweep 10000001 -forgive" 10000000 "INT" 0} \
      {"Scope sweep1 0 -forgive" 1 "INT" 0} \
      {"Scope sweep1 1" 1 "INT" 0} \
      {"Scope sweep1 100" 100 "INT" 0} \
      {"Scope sweep1 10000" 10000 "INT" 0} \
      {"Scope sweep1 1000000" 1000000 "INT" 0} \
      {"Scope sweep1 9999999" 9999999 "INT" 0} \
      {"Scope sweep1 10000001 -forgive" 10000000 "INT" 0} \
      {"Scope sweep2 0 -forgive" 1 "INT" 0} \
      {"Scope sweep2 1" 1 "INT" 0} \
      {"Scope sweep2 100" 100 "INT" 0} \
      {"Scope sweep2 10000" 10000 "INT" 0} \
      {"Scope sweep2 1000000" 1000000 "INT" 0} \
      {"Scope sweep2 9999999" 9999999 "INT" 0} \
      {"Scope sweep2 10000001 -forgive" 10000000 "INT" 0} \
      {"Scope sweep3 100" 100 "INT" 0} \
      {"Scope sweep3 10000" 10000 "INT" 0} \
      {"Scope sweep3 100000" 100000 "INT" 0} \
      {"Scope sweep3 999999" 999999 "INT" 0} \
      {"Scope horizontalsweep 0 -forgive" 1 "INT" 0} \
      {"Scope horizontalsweep 1" 1 "INT" 0} \
      {"Scope horizontalsweep 100" 100 "INT" 0} \
      {"Scope horizontalsweep 10000" 10000 "INT" 0} \
      {"Scope horizontalsweep 1000000" 1000000 "INT" 0} \
      {"Scope horizontalsweep 9999999" 9999999 "INT" 0} \
      {"Scope horizontalsweep 10000001 -forgive" 10000000 "INT" 0} ]

   set testScopeList2 [list \
      {"Scope scale -1 -forgive" 1 "INT" 0} \
      {"after 100" 0 "DELAY" 0} \
      {"Scope scale 0 -forgive" 1 "INT" 0} \
      {"after 100" 0 "DELAY" 0} \
      {"Scope scale 1" 1 "INT" 0} \
      {"after 100" 0 "DELAY" 0} \
      {"Scope scale 11" 11 "INT" 0} \
      {"after 100" 0 "DELAY" 0} \
      {"Scope scale 12" 12 "INT" 0} \
      {"after 100" 0 "DELAY" 0} \
      {"Scope scale 13 -forgive" 12 "INT" 0} \
      {"after 100" 0 "DELAY" 0} \
      {"Scope scale1 -1 -forgive" 1 "INT" 0} \
      {"after 100" 0 "DELAY" 0} \
      {"Scope scale1 0 -forgive" 1 "INT" 0} \
      {"after 100" 0 "DELAY" 0} \
      {"Scope scale1 1" 1 "INT" 0} \
      {"after 100" 0 "DELAY" 0} \
      {"Scope scale1 11" 11 "INT" 0} \
      {"after 100" 0 "DELAY" 0} \
      {"Scope scale1 12" 12 "INT" 0} \
      {"after 100" 0 "DELAY" 0} \
      {"Scope scale1 13 -forgive" 12 "INT" 0} \
      {"after 100" 0 "DELAY" 0} \
      {"Scope scale2 -1 -forgive" 1 "INT" 0} \
      {"after 100" 0 "DELAY" 0} \
      {"Scope scale2 0 -forgive" 1 "INT" 0} \
      {"after 100" 0 "DELAY" 0} \
      {"Scope scale2 1" 1 "INT" 0} \
      {"after 100" 0 "DELAY" 0} \
      {"Scope scale2 11" 11 "INT" 0} \
      {"after 100" 0 "DELAY" 0} \
      {"Scope scale2 12" 12 "INT" 0} \
      {"after 100" 0 "DELAY" 0} \
      {"Scope scale2 13 -forgive" 12 "INT" 0} \
      {"after 100" 0 "DELAY" 0} \
      {"Scope scale3 -1 -forgive" 0 "INT" 0} \
      {"after 100" 0 "DELAY" 0} \
      {"Scope scale3 0" 0 "INT" 0} \
      {"after 100" 0 "DELAY" 0} \
      {"Scope scale3 1" 1 "INT" 0} \
      {"after 100" 0 "DELAY" 0} \
      {"Scope scale3 11" 11 "INT" 0} \
      {"after 100" 0 "DELAY" 0} \
      {"Scope scale3 12" 12 "INT" 0} \
      {"after 100" 0 "DELAY" 0} \
      {"Scope scale3 13 -forgive" 12 "INT" 0} \
      {"after 100" 0 "DELAY" 0} \
      {"Scope verticalscale -1 -forgive" 1 "INT" 0} \
      {"after 100" 0 "DELAY" 0} \
      {"Scope verticalscale 0 -forgive" 1 "INT" 0} \
      {"after 100" 0 "DELAY" 0} \
      {"Scope verticalscale 1" 1 "INT" 0} \
      {"after 100" 0 "DELAY" 0} \
      {"Scope verticalscale 11" 11 "INT" 0} \
      {"after 100" 0 "DELAY" 0} \
      {"Scope verticalscale 12" 12 "INT" 0} \
      {"after 100" 0 "DELAY" 0} \
      {"Scope verticalscale 13 -forgive" 12 "INT" 0} \
      {"after 100" 0 "DELAY" 0} \
      {"Scope scale3 ground" 0 "INT" 0} \
      {"after 100" 0 "DELAY" 0} \
      {"Scope scale3 gnd" 0 "INT" 0} \
      {"after 100" 0 "DELAY" 0} \
      {"Scope scale 10" 1 "INT" 0} \
      {"after 100" 0 "DELAY" 0} \
      {"Scope scale 10mv" 1 "INT" 0} \
      {"after 100" 0 "DELAY" 0} \
      {"Scope scale 20" 2 "INT" 0} \
      {"after 100" 0 "DELAY" 0} \
      {"Scope scale 20mv" 2 "INT" 0} \
      {"after 100" 0 "DELAY" 0} \
      {"Scope scale 50" 3 "INT" 0} \
      {"after 100" 0 "DELAY" 0} \
      {"Scope scale 50mv" 3 "INT" 0} \
      {"after 100" 0 "DELAY" 0} \
      {"Scope scale 100" 4 "INT" 0} \
      {"after 100" 0 "DELAY" 0} \
      {"Scope scale 100mv" 4 "INT" 0} \
      {"after 100" 0 "DELAY" 0} \
      {"Scope scale 200" 5 "INT" 0} \
      {"after 100" 0 "DELAY" 0} \
      {"Scope scale 200mv" 5 "INT" 0} \
      {"after 100" 0 "DELAY" 0} \
      {"Scope scale 500" 6 "INT" 0} \
      {"after 100" 0 "DELAY" 0} \
      {"Scope scale 500mv" 6 "INT" 0} \
      {"after 100" 0 "DELAY" 0} \
      {"Scope scale 1000" 7 "INT" 0} \
      {"after 100" 0 "DELAY" 0} \
      {"Scope scale 1v" 7 "INT" 0} \
      {"after 100" 0 "DELAY" 0} \
      {"Scope scale 2000" 8 "INT" 0} \
      {"after 100" 0 "DELAY" 0} \
      {"Scope scale 2v" 8 "INT" 0} \
      {"after 100" 0 "DELAY" 0} \
      {"Scope scale 5000" 9 "INT" 0} \
      {"after 100" 0 "DELAY" 0} \
      {"Scope scale 5v" 9 "INT" 0} \
      {"after 100" 0 "DELAY" 0} \
      {"Scope scale 10000" 10 "INT" 0} \
      {"after 100" 0 "DELAY" 0} \
      {"Scope scale 10v" 10 "INT" 0} \
      {"after 100" 0 "DELAY" 0} \
      {"Scope scale 20000" 11 "INT" 0} \
      {"after 100" 0 "DELAY" 0} \
      {"Scope scale 20v" 11 "INT" 0} \
      {"after 100" 0 "DELAY" 0} \
      {"Scope scale 50000" 12 "INT" 0} \
      {"after 100" 0 "DELAY" 0} \
      {"Scope scale 50v" 12 "INT" 0} ]

   set testScopeList3 [list \
      {"Scope position -101 -forgive" -100 "INT" 0} \
      {"Scope position -100" -100 "INT" 0} \
      {"Scope position -50" -50 "INT" 0} \
      {"Scope position -1" -1 "INT" 0} \
      {"Scope position 0" 0 "INT" 0} \
      {"Scope position 1" 1 "INT" 0} \
      {"Scope position 50" 50 "INT" 0} \
      {"Scope position 100" 100 "INT" 0} \
      {"Scope position 101 -forgive" 100 "INT" 0} \
      {"Scope position1 -101 -forgive" -100 "INT" 0} \
      {"Scope position1 -100" -100 "INT" 0} \
      {"Scope position1 -50" -50 "INT" 0} \
      {"Scope position1 -1" -1 "INT" 0} \
      {"Scope position1 0" 0 "INT" 0} \
      {"Scope position1 1" 1 "INT" 0} \
      {"Scope position1 50" 50 "INT" 0} \
      {"Scope position1 100" 100 "INT" 0} \
      {"Scope position1 101 -forgive" 100 "INT" 0} \
      {"Scope position2 -101 -forgive" -100 "INT" 0} \
      {"Scope position2 -100" -100 "INT" 0} \
      {"Scope position2 -50" -50 "INT" 0} \
      {"Scope position2 -1" -1 "INT" 0} \
      {"Scope position2 0" 0 "INT" 0} \
      {"Scope position2 1" 1 "INT" 0} \
      {"Scope position2 50" 50 "INT" 0} \
      {"Scope position2 100" 100 "INT" 0} \
      {"Scope position2 101 -forgive" 100 "INT" 0} \
      {"Scope position3 -101 -forgive" -100 "INT" 0} \
      {"Scope position3 -100" -100 "INT" 0} \
      {"Scope position3 -50" -50 "INT" 0} \
      {"Scope position3 -1" -1 "INT" 0} \
      {"Scope position3 0" 0 "INT" 0} \
      {"Scope position3 1" 1 "INT" 0} \
      {"Scope position3 50" 50 "INT" 0} \
      {"Scope position3 100" 100 "INT" 0} \
      {"Scope position3 101 -forgive" 100 "INT" 0} \
      {"Scope horizontalposition -101 -forgive" -100 "INT" 0} \
      {"Scope horizontalposition -100" -100 "INT" 0} \
      {"Scope horizontalposition -50" -50 "INT" 0} \
      {"Scope horizontalposition -1" -1 "INT" 0} \
      {"Scope horizontalposition 0" 0 "INT" 0} \
      {"Scope horizontalposition 1" 1 "INT" 0} \
      {"Scope horizontalposition 50" 50 "INT" 0} \
      {"Scope horizontalposition 100" 100 "INT" 0} \
      {"Scope horizontalposition 50" 50 "INT" 0} \
      {"Scope horizontalposition 101 -forgive" 100 "INT" 0} \
      {"Scope horizontalposition" 100 "INT" 0} ]

   set testScopeList4 [list \
      {"Scope trigtype 1" 1 "INT" 0} \
      {"Scope trigtype -1 -forgive" 0 "INT" 0} \
      {"Scope trigtype" 0 "INT" 0} \
      {"Scope trigtype 0" 0 "INT" 0} \
      {"Scope trigtype 1" 1 "INT" 0} \
      {"Scope trigtype 2" 2 "INT" 0} \
      {"Scope trigtype auto" 0 "INT" 0} \
      {"Scope trigtype normal" 1 "INT" 0} \
      {"Scope trigtype 1shot" 2 "INT" 0} \
      {"Scope trigtype 3 -forgive" 0 "INT" 0} \
      {"Scope trigtype1 1" 1 "INT" 0} \
      {"Scope trigtype1 -1 -forgive" 0 "INT" 0} \
      {"Scope trigtype1" 0 "INT" 0} \
      {"Scope trigtype1 0" 0 "INT" 0} \
      {"Scope trigtype1 1" 1 "INT" 0} \
      {"Scope trigtype1 2" 2 "INT" 0} \
      {"Scope trigtype1 3 -forgive" 0 "INT" 0} \
      {"Scope trigtype2 1" 1 "INT" 0} \
      {"Scope trigtype2 -1 -forgive" 0 "INT" 0} \
      {"Scope trigtype2" 0 "INT" 0} \
      {"Scope trigtype2 0" 0 "INT" 0} \
      {"Scope trigtype2 1" 1 "INT" 0} \
      {"Scope trigtype2 2" 2 "INT" 0} \
      {"Scope trigtype2 3 -forgive" 0 "INT" 0} \
      {"Scope trigtype3 1" 1 "INT" 0} \
      {"Scope trigtype3 -1 -forgive" 0 "INT" 0} \
      {"Scope trigtype3" 0 "INT" 0} \
      {"Scope trigtype3 0" 0 "INT" 0} \
      {"Scope trigtype3 1" 1 "INT" 0} \
      {"Scope trigtype3 2" 2 "INT" 0} \
      {"Scope trigtype3 3 -forgive" 0 "INT" 0} \
      {"Scope triggertype 1" 1 "INT" 0} \
      {"Scope triggertype -1 -forgive" 0 "INT" 0} \
      {"Scope triggertype" 0 "INT" 0} \
      {"Scope triggertype 0" 0 "INT" 0} \
      {"Scope triggertype 1" 1 "INT" 0} \
      {"Scope triggertype 2" 2 "INT" 0} \
      {"Scope triggertype 3 -forgive" 0 "INT" 0} ]

   set testScopeList5 [list \
      {"Scope trigsource 1" 1 "INT" 0} \
      {"Scope trigsource -1 -forgive" 0 "INT" 0} \
      {"Scope trigsource" 0 "INT" 0} \
      {"Scope trigsource 0" 0 "INT" 0} \
      {"Scope trigsource 1" 1 "INT" 0} \
      {"Scope trigsource 2" 2 "INT" 0} \
      {"Scope trigsource ext" 0 "INT" 0} \
      {"Scope trigsource ch1" 1 "INT" 0} \
      {"Scope trigsource ch2" 2 "INT" 0} \
      {"Scope trigsource 3 -forgive" 2 "INT" 0} \
      {"Scope trigsource1 1" 1 "INT" 0} \
      {"Scope trigsource1 -1 -forgive" 0 "INT" 0} \
      {"Scope trigsource1" 0 "INT" 0} \
      {"Scope trigsource1 0" 0 "INT" 0} \
      {"Scope trigsource1 1" 1 "INT" 0} \
      {"Scope trigsource1 2" 2 "INT" 0} \
      {"Scope trigsource1 3 -forgive" 2 "INT" 0} \
      {"Scope trigsource2 1" 1 "INT" 0} \
      {"Scope trigsource2 -1 -forgive" 0 "INT" 0} \
      {"Scope trigsource2" 0 "INT" 0} \
      {"Scope trigsource2 0" 0 "INT" 0} \
      {"Scope trigsource2 1" 1 "INT" 0} \
      {"Scope trigsource2 2" 2 "INT" 0} \
      {"Scope trigsource2 3 -forgive" 2 "INT" 0} \
      {"Scope trigsource3 1" 1 "INT" 0} \
      {"Scope trigsource3 -1 -forgive" 0 "INT" 0} \
      {"Scope trigsource3" 0 "INT" 0} \
      {"Scope trigsource3 0" 0 "INT" 0} \
      {"Scope trigsource3 1" 1 "INT" 0} \
      {"Scope trigsource3 2" 2 "INT" 0} \
      {"Scope trigsource3 3 -forgive" 2 "INT" 0} \
      {"Scope triggersource 1" 1 "INT" 0} \
      {"Scope triggersource -1 -forgive" 0 "INT" 0} \
      {"Scope triggersource" 0 "INT" 0} \
      {"Scope triggersource 0" 0 "INT" 0} \
      {"Scope triggersource 1" 1 "INT" 0} \
      {"Scope triggersource 2" 2 "INT" 0} \
      {"Scope triggersource ext" 0 "INT" 0} \
      {"Scope triggersource ch1" 1 "INT" 0} \
      {"Scope triggersource ch2" 2 "INT" 0} \
      {"Scope triggersource 3 -forgive" 2 "INT" 0} ]

   set testScopeList6 [list \
      {"Scope triglev -501.0 -forgive" -500.0 "DBL" 0.001} \
      {"Scope triglev -500.0 " -500.0 "DBL" 0.001} \
      {"Scope triglev -250.0 " -250.0 "DBL" 0.001} \
      {"Scope triglev -100.0 " -100.0 "DBL" 0.001} \
      {"Scope triglev -1.0 " -1.0 "DBL" 0.001} \
      {"Scope triglev 0.0 " 0.0 "DBL" 0.001} \
      {"Scope triglev 1.0 " 1.0 "DBL" 0.001} \
      {"Scope triglev 100.0 " 100.0 "DBL" 0.001} \
      {"Scope triglev 250.0 " 250.0 "DBL" 0.001} \
      {"Scope triglev 499.3 " 499.3 "DBL" 0.001} \
      {"Scope triglev 500.0 " 500.0 "DBL" 0.001} \
      {"Scope triglev 501.0 -forgive" 500.0 "DBL" 0.001} \
      {"Scope triglev1 -501.0 -forgive" -500.0 "DBL" 0.001} \
      {"Scope triglev1 -500.0 " -500.0 "DBL" 0.001} \
      {"Scope triglev1 -250.0 " -250.0 "DBL" 0.001} \
      {"Scope triglev1 -100.0 " -100.0 "DBL" 0.001} \
      {"Scope triglev1 -1.0 " -1.0 "DBL" 0.001} \
      {"Scope triglev1 0.0 " 0.0 "DBL" 0.001} \
      {"Scope triglev1 1.0 " 1.0 "DBL" 0.001} \
      {"Scope triglev1 100.0 " 100.0 "DBL" 0.001} \
      {"Scope triglev1 250.0 " 250.0 "DBL" 0.001} \
      {"Scope triglev1 499.3 " 499.3 "DBL" 0.001} \
      {"Scope triglev1 500.0 " 500.0 "DBL" 0.001} \
      {"Scope triglev1 501.0 -forgive" 500.0 "DBL" 0.001} \
      {"Scope triglev2 -501.0 -forgive" -500.0 "DBL" 0.001} \
      {"Scope triglev2 -500.0 " -500.0 "DBL" 0.001} \
      {"Scope triglev2 -250.0 " -250.0 "DBL" 0.001} \
      {"Scope triglev2 -100.0 " -100.0 "DBL" 0.001} \
      {"Scope triglev2 -1.0 " -1.0 "DBL" 0.001} \
      {"Scope triglev2 0.0 " 0.0 "DBL" 0.001} \
      {"Scope triglev2 1.0 " 1.0 "DBL" 0.001} \
      {"Scope triglev2 100.0 " 100.0 "DBL" 0.001} \
      {"Scope triglev2 250.0 " 250.0 "DBL" 0.001} \
      {"Scope triglev2 499.3 " 499.3 "DBL" 0.001} \
      {"Scope triglev2 500.0 " 500.0 "DBL" 0.001} \
      {"Scope triglev2 501.0 -forgive" 500.0 "DBL" 0.001} \
      {"Scope triglev3 -501.0 -forgive" -500.0 "DBL" 0.001} \
      {"Scope triglev3 -500.0 " -500.0 "DBL" 0.001} \
      {"Scope triglev3 -250.0 " -250.0 "DBL" 0.001} \
      {"Scope triglev3 -100.0 " -100.0 "DBL" 0.001} \
      {"Scope triglev3 -1.0 " -1.0 "DBL" 0.001} \
      {"Scope triglev3 0.0 " 0.0 "DBL" 0.001} \
      {"Scope triglev3 1.0 " 1.0 "DBL" 0.001} \
      {"Scope triglev3 100.0 " 100.0 "DBL" 0.001} \
      {"Scope triglev3 250.0 " 250.0 "DBL" 0.001} \
      {"Scope triglev3 499.3 " 499.3 "DBL" 0.001} \
      {"Scope triglev3 500.0 " 500.0 "DBL" 0.001} \
      {"Scope triglev3 501.0 -forgive" 500.0 "DBL" 0.001} \
      {"Scope triggerlevel -501.0 -forgive" -500.0 "DBL" 0.001} \
      {"Scope triggerlevel -500.0 " -500.0 "DBL" 0.001} \
      {"Scope triggerlevel -250.0 " -250.0 "DBL" 0.001} \
      {"Scope triggerlevel -100.0 " -100.0 "DBL" 0.001} \
      {"Scope triggerlevel -1.0 " -1.0 "DBL" 0.001} \
      {"Scope triggerlevel 0.0 " 0.0 "DBL" 0.001} \
      {"Scope triggerlevel 1.0 " 1.0 "DBL" 0.001} \
      {"Scope triggerlevel 100.0 " 100.0 "DBL" 0.001} \
      {"Scope triggerlevel 250.0 " 250.0 "DBL" 0.001} \
      {"Scope triggerlevel 499.3 " 499.3 "DBL" 0.001} \
      {"Scope triggerlevel 500.0 " 500.0 "DBL" 0.001} \
      {"Scope triggerlevel 501.0 -forgive" 500.0 "DBL" 0.001} ]

   set testScopeList7 [list \
      {"Scope peakreset 2" 2 "INT" 0} \
      {"Scope peakreset -1 -forgive" 1 "INT" 0} \
      {"Scope peakreset 2" 2 "INT" 0} \
      {"Scope peakreset 4" 4 "INT" 0} \
      {"Scope peakreset 8" 8 "INT" 0} \
      {"Scope peakreset 16" 16 "INT" 0} \
      {"Scope peakreset 32" 32 "INT" 0} \
      {"Scope peakreset 64" 64 "INT" 0} \
      {"Scope peakreset 127" 127 "INT" 0} \
      {"Scope peakreset 128 -forgive" 1 "INT" 0} \
      {"Scope peakreset1 2" 2 "INT" 0} \
      {"Scope peakreset1 -1 -forgive" 1 "INT" 0} \
      {"Scope peakreset1 2" 2 "INT" 0} \
      {"Scope peakreset1 4" 4 "INT" 0} \
      {"Scope peakreset1 8" 8 "INT" 0} \
      {"Scope peakreset1 16" 16 "INT" 0} \
      {"Scope peakreset1 32" 32 "INT" 0} \
      {"Scope peakreset1 64" 64 "INT" 0} \
      {"Scope peakreset1 127" 127 "INT" 0} \
      {"Scope peakreset1 128 -forgive" 1 "INT" 0} \
      {"Scope peakreset2 2" 2 "INT" 0} \
      {"Scope peakreset2 -1 -forgive" 1 "INT" 0} \
      {"Scope peakreset2 2" 2 "INT" 0} \
      {"Scope peakreset2 4" 4 "INT" 0} \
      {"Scope peakreset2 8" 8 "INT" 0} \
      {"Scope peakreset2 16" 16 "INT" 0} \
      {"Scope peakreset2 32" 32 "INT" 0} \
      {"Scope peakreset2 64" 64 "INT" 0} \
      {"Scope peakreset2 127" 127 "INT" 0} \
      {"Scope peakreset2 128 -forgive" 1 "INT" 0} \
      {"Scope peakreset3 2" 2 "INT" 0} \
      {"Scope peakreset3 -1 -forgive" 1 "INT" 0} \
      {"Scope peakreset3 2" 2 "INT" 0} \
      {"Scope peakreset3 4" 4 "INT" 0} \
      {"Scope peakreset3 8" 8 "INT" 0} \
      {"Scope peakreset3 16" 16 "INT" 0} \
      {"Scope peakreset3 32" 32 "INT" 0} \
      {"Scope peakreset3 64" 64 "INT" 0} \
      {"Scope peakreset3 127" 127 "INT" 0} \
      {"Scope peakreset3 128 -forgive" 1 "INT" 0} \
      {"Scope peakholdreset 2" 2 "INT" 0} \
      {"Scope peakholdreset -1 -forgive" 1 "INT" 0} \
      {"Scope peakholdreset 2" 2 "INT" 0} \
      {"Scope peakholdreset 4" 4 "INT" 0} \
      {"Scope peakholdreset 8" 8 "INT" 0} \
      {"Scope peakholdreset 16" 16 "INT" 0} \
      {"Scope peakholdreset 32" 32 "INT" 0} \
      {"Scope peakholdreset 64" 64 "INT" 0} \
      {"Scope peakholdreset 127" 127 "INT" 0} \
      {"Scope peakholdreset 128 -forgive" 1 "INT" 0} ]

   set testScopeList8 [list \
      {"Scope avg -1 -forgive" 2 "INT" 0} \
      {"Scope avg 0 -forgive" 2 "INT" 0} \
      {"Scope avg 1 -forgive" 2 "INT" 0} \
      {"Scope avg 2" 2 "INT" 0} \
      {"Scope avg 3" 3 "INT" 0} \
      {"Scope avg 100" 100 "INT" 0} \
      {"Scope avg 254" 254 "INT" 0} \
      {"Scope avg 255" 255 "INT" 0} \
      {"Scope avg 256 -forgive" 255 "INT" 0} \
      {"Scope avg1 -1 -forgive" 2 "INT" 0} \
      {"Scope avg1 0 -forgive" 2 "INT" 0} \
      {"Scope avg1 1 -forgive" 2 "INT" 0} \
      {"Scope avg1 2" 2 "INT" 0} \
      {"Scope avg1 3" 3 "INT" 0} \
      {"Scope avg1 100" 100 "INT" 0} \
      {"Scope avg1 254" 254 "INT" 0} \
      {"Scope avg1 255" 255 "INT" 0} \
      {"Scope avg1 256 -forgive" 255 "INT" 0} \
      {"Scope avg2 -1 -forgive" 2 "INT" 0} \
      {"Scope avg2 0 -forgive" 2 "INT" 0} \
      {"Scope avg2 1 -forgive" 2 "INT" 0} \
      {"Scope avg2 2" 2 "INT" 0} \
      {"Scope avg2 3" 3 "INT" 0} \
      {"Scope avg2 100" 100 "INT" 0} \
      {"Scope avg2 254" 254 "INT" 0} \
      {"Scope avg2 255" 255 "INT" 0} \
      {"Scope avg2 256 -forgive" 255 "INT" 0} \
      {"Scope avg3 -1 -forgive" 2 "INT" 0} \
      {"Scope avg3 0 -forgive" 2 "INT" 0} \
      {"Scope avg3 1 -forgive" 2 "INT" 0} \
      {"Scope avg3 2" 2 "INT" 0} \
      {"Scope avg3 3" 3 "INT" 0} \
      {"Scope avg3 100" 100 "INT" 0} \
      {"Scope avg3 254" 254 "INT" 0} \
      {"Scope avg3 255" 255 "INT" 0} \
      {"Scope avg3 256 -forgive" 255 "INT" 0} \
      {"Scope average -1 -forgive" 2 "INT" 0} \
      {"Scope average 0 -forgive" 2 "INT" 0} \
      {"Scope average 1 -forgive" 2 "INT" 0} \
      {"Scope average 2" 2 "INT" 0} \
      {"Scope average 3" 3 "INT" 0} \
      {"Scope average 100" 100 "INT" 0} \
      {"Scope average 254" 254 "INT" 0} \
      {"Scope average 255" 255 "INT" 0} \
      {"Scope average 256 -forgive" 255 "INT" 0} ]

   set testScopeList9 [list \
      {"Scope coupling 1" 1 "INT" 0} \
      {"after 100" 0 "DELAY" 0} \
      {"Scope coupling -1 -forgive" 0 "INT" 0} \
      {"after 100" 0 "DELAY" 0} \
      {"Scope coupling" 0 "INT" 0} \
      {"after 100" 0 "DELAY" 0} \
      {"Scope coupling 0" 0 "INT" 0} \
      {"after 100" 0 "DELAY" 0} \
      {"Scope coupling 1" 1 "INT" 0} \
      {"after 100" 0 "DELAY" 0} \
      {"Scope coupling ac" 0 "INT" 0} \
      {"after 100" 0 "DELAY" 0} \
      {"Scope coupling dc" 1 "INT" 0} \
      {"after 100" 0 "DELAY" 0} \
      {"Scope coupling ac_t" 0 "INT" 0} \
      {"after 100" 0 "DELAY" 0} \
      {"Scope coupling dc_t" 1 "INT" 0} \
      {"after 100" 0 "DELAY" 0} \
      {"Scope coupling 2 -forgive" 1 "INT" 0} \
      {"after 100" 0 "DELAY" 0} \
      {"Scope coupling1 1" 1 "INT" 0} \
      {"after 100" 0 "DELAY" 0} \
      {"Scope coupling1 -1 -forgive" 0 "INT" 0} \
      {"after 100" 0 "DELAY" 0} \
      {"Scope coupling1" 0 "INT" 0} \
      {"after 100" 0 "DELAY" 0} \
      {"Scope coupling1 0" 0 "INT" 0} \
      {"after 100" 0 "DELAY" 0} \
      {"Scope coupling1 1" 1 "INT" 0} \
      {"after 100" 0 "DELAY" 0} \
      {"Scope coupling1 2 -forgive" 1 "INT" 0} \
      {"after 100" 0 "DELAY" 0} \
      {"Scope coupling2 1" 1 "INT" 0} \
      {"after 100" 0 "DELAY" 0} \
      {"Scope coupling2 -1 -forgive" 0 "INT" 0} \
      {"after 100" 0 "DELAY" 0} \
      {"Scope coupling2" 0 "INT" 0} \
      {"after 100" 0 "DELAY" 0} \
      {"Scope coupling2 0" 0 "INT" 0} \
      {"after 100" 0 "DELAY" 0} \
      {"Scope coupling2 1" 1 "INT" 0} \
      {"after 100" 0 "DELAY" 0} \
      {"Scope coupling2 2 -forgive" 1 "INT" 0} \
      {"after 100" 0 "DELAY" 0} \
      {"Scope coupling3 1" 1 "INT" 0} \
      {"after 100" 0 "DELAY" 0} \
      {"Scope coupling3 -1 -forgive" 0 "INT" 0} \
      {"after 100" 0 "DELAY" 0} \
      {"Scope coupling3" 0 "INT" 0} \
      {"after 100" 0 "DELAY" 0} \
      {"Scope coupling3 0" 0 "INT" 0} \
      {"after 100" 0 "DELAY" 0} \
      {"Scope coupling3 1" 1 "INT" 0} \
      {"after 100" 0 "DELAY" 0} \
      {"Scope coupling3 2 -forgive" 1 "INT" 0} \
      {"after 100" 0 "DELAY" 0} \
      {"Scope inputcoupling 1" 1 "INT" 0} \
      {"after 100" 0 "DELAY" 0} \
      {"Scope inputcoupling -1 -forgive" 0 "INT" 0} \
      {"after 100" 0 "DELAY" 0} \
      {"Scope inputcoupling" 0 "INT" 0} \
      {"after 100" 0 "DELAY" 0} \
      {"Scope inputcoupling 0" 0 "INT" 0} \
      {"after 100" 0 "DELAY" 0} \
      {"Scope inputcoupling 1" 1 "INT" 0} \
      {"after 100" 0 "DELAY" 0} \
      {"Scope inputcoupling ac" 0 "INT" 0} \
      {"after 100" 0 "DELAY" 0} \
      {"Scope inputcoupling dc" 1 "INT" 0} \
      {"after 100" 0 "DELAY" 0} \
      {"Scope inputcoupling ac_t" 0 "INT" 0} \
      {"after 100" 0 "DELAY" 0} \
      {"Scope inputcoupling dc_t" 1 "INT" 0} \
      {"after 100" 0 "DELAY" 0} \
      {"Scope inputcoupling 2 -forgive" 1 "INT" 0} ]

   set testScopeList10 [list \
      {"Scope math -1 -forgive" 1 "INT" 0} \
      {"Scope math 0 -forgive" 1 "INT" 0} \
      {"Scope math 1" 1 "INT" 0} \
      {"Scope math 2" 2 "INT" 0} \
      {"Scope math 3" 3 "INT" 0} \
      {"Scope math 4" 4 "INT" 0} \
      {"Scope math 5 -forgive" 4 "INT" 0} \
      {"Scope math1 -1 -forgive" 1 "INT" 0} \
      {"Scope math1 0 -forgive" 1 "INT" 0} \
      {"Scope math1 1" 1 "INT" 0} \
      {"Scope math1 2" 2 "INT" 0} \
      {"Scope math1 3" 3 "INT" 0} \
      {"Scope math1 4" 4 "INT" 0} \
      {"Scope math1 5 -forgive" 4 "INT" 0} \
      {"Scope math2 -1 -forgive" 1 "INT" 0} \
      {"Scope math2 0 -forgive" 1 "INT" 0} \
      {"Scope math2 1" 1 "INT" 0} \
      {"Scope math2 2" 2 "INT" 0} \
      {"Scope math2 3" 3 "INT" 0} \
      {"Scope math2 4" 4 "INT" 0} \
      {"Scope math2 5 -forgive" 4 "INT" 0} \
      {"Scope math3 -1 -forgive" 1 "INT" 0} \
      {"Scope math3 0 -forgive" 1 "INT" 0} \
      {"Scope math3 1" 1 "INT" 0} \
      {"Scope math3 2" 2 "INT" 0} \
      {"Scope math3 3" 3 "INT" 0} \
      {"Scope math3 4" 4 "INT" 0} \
      {"Scope math3 5 -forgive" 4 "INT" 0} \
      {"Scope tracemath -1 -forgive" 1 "INT" 0} \
      {"Scope tracemath 0 -forgive" 1 "INT" 0} \
      {"Scope tracemath 1" 1 "INT" 0} \
      {"Scope tracemath 2" 2 "INT" 0} \
      {"Scope tracemath 3" 3 "INT" 0} \
      {"Scope tracemath 4" 4 "INT" 0} \
      {"Scope tracemath 5 -forgive" 4 "INT" 0} ]

   set testScopeList11 [list \
      {"Scope enable -1 -forgive" 1 "INT" 0} \
      {"Scope enable 0" 1 "INT" 0} \
      {"Scope enable 1" 1 "INT" 0} \
      {"Scope enable 2" 3 "INT" 0} \
      {"Scope enable 4" 5 "INT" 0} \
      {"Scope enable 7" 7 "INT" 0} \
      {"Scope enable 8 -forgive" 1 "INT" 0} \
      {"Scope enable1 -1 -forgive" 1 "INT" 0} \
      {"Scope enable1 0" 1 "INT" 0} \
      {"Scope enable1 1" 1 "INT" 0} \
      {"Scope enable1 2" 3 "INT" 0} \
      {"Scope enable1 4" 5 "INT" 0} \
      {"Scope enable1 7" 7 "INT" 0} \
      {"Scope enable1 8 -forgive" 1 "INT" 0} \
      {"Scope enable2 -1 -forgive" 1 "INT" 0} \
      {"Scope enable2 0" 1 "INT" 0} \
      {"Scope enable2 1" 1 "INT" 0} \
      {"Scope enable2 2" 3 "INT" 0} \
      {"Scope enable2 4" 5 "INT" 0} \
      {"Scope enable2 7" 7 "INT" 0} \
      {"Scope enable2 8 -forgive" 1 "INT" 0} \
      {"Scope enable3 -1 -forgive" 1 "INT" 0} \
      {"Scope enable3 0" 1 "INT" 0} \
      {"Scope enable3 1" 1 "INT" 0} \
      {"Scope enable3 2" 3 "INT" 0} \
      {"Scope enable3 4" 5 "INT" 0} \
      {"Scope enable3 7" 7 "INT" 0} \
      {"Scope enable3 8 -forgive" 1 "INT" 0} \
      {"Scope messageenable -1 -forgive" 1 "INT" 0} \
      {"Scope messageenable 0" 1 "INT" 0} \
      {"Scope messageenable 1" 1 "INT" 0} \
      {"Scope messageenable 2" 3 "INT" 0} \
      {"Scope messageenable 4" 5 "INT" 0} \
      {"Scope messageenable 7" 7 "INT" 0} \
      {"Scope messageenable 8 -forgive" 1 "INT" 0} ]

   set testScopeList12 [list \
      {"Scope run 1" 1 "INT" 0} \
      {"Scope run -1 -forgive" 0 "INT" 0} \
      {"Scope run" 0 "INT" 0} \
      {"Scope run 0" 0 "INT" 0} \
      {"Scope run 1" 1 "INT" 0} \
      {"Scope run 2 -forgive" 0 "INT" 0} \
      {"Scope run" 0 "INT" 0} \
      {"Scope run1 1" 1 "INT" 0} \
      {"Scope run1 -1 -forgive" 0 "INT" 0} \
      {"Scope run1" 0 "INT" 0} \
      {"Scope run1 0" 0 "INT" 0} \
      {"Scope run1 1" 1 "INT" 0} \
      {"Scope run1 2 -forgive" 0 "INT" 0} \
      {"Scope run1" 0 "INT" 0} \
      {"Scope run2 1" 1 "INT" 0} \
      {"Scope run2 -1 -forgive" 0 "INT" 0} \
      {"Scope run2" 0 "INT" 0} \
      {"Scope run2 0" 0 "INT" 0} \
      {"Scope run2 1" 1 "INT" 0} \
      {"Scope run2 2 -forgive" 0 "INT" 0} \
      {"Scope run2" 0 "INT" 0} \
      {"Scope run3 1" 1 "INT" 0} \
      {"Scope run3 -1 -forgive" 0 "INT" 0} \
      {"Scope run3" 0 "INT" 0} \
      {"Scope run3 0" 0 "INT" 0} \
      {"Scope run3 1" 1 "INT" 0} \
      {"Scope run3 2 -forgive" 0 "INT" 0} \
      {"Scope run3" 0 "INT" 0} ]

   # Set up the scale to avoid an error in a function called when the trigger polarity is changed
   set testScopeList13 [list \
      {"Scope scale1 10mv" 1 "INT" 0} \
      {"Scope scale2 10mv" 1 "INT" 0} \
      {"Scope scale3 10mv" 1 "INT" 0} \
      {"Scope trigpol 1" 1 "INT" 0} \
      {"Scope trigpol -1 -forgive" 0 "INT" 0} \
      {"Scope trigpol" 0 "INT" 0} \
      {"Scope trigpol 0" 0 "INT" 0} \
      {"Scope trigpol 1" 1 "INT" 0} \
      {"Scope trigpol neg" 0 "INT" 0} \
      {"Scope trigpol pos" 1 "INT" 0} \
      {"Scope trigpol 2 -forgive" 1 "INT" 0} \
      {"Scope trigpol" 1 "INT" 0} \
      {"Scope trigpol1 1" 1 "INT" 0} \
      {"Scope trigpol1 -1 -forgive" 0 "INT" 0} \
      {"Scope trigpol1" 0 "INT" 0} \
      {"Scope trigpol1 0" 0 "INT" 0} \
      {"Scope trigpol1 1" 1 "INT" 0} \
      {"Scope trigpol1 2 -forgive" 1 "INT" 0} \
      {"Scope trigpol1" 1 "INT" 0} \
      {"Scope trigpol2 1" 1 "INT" 0} \
      {"Scope trigpol2 -1 -forgive" 0 "INT" 0} \
      {"Scope trigpol2" 0 "INT" 0} \
      {"Scope trigpol2 0" 0 "INT" 0} \
      {"Scope trigpol2 1" 1 "INT" 0} \
      {"Scope trigpol2 2 -forgive" 1 "INT" 0} \
      {"Scope trigpol2" 1 "INT" 0} \
      {"Scope trigpol3 1" 1 "INT" 0} \
      {"Scope trigpol3 -1 -forgive" 0 "INT" 0} \
      {"Scope trigpol3" 0 "INT" 0} \
      {"Scope trigpol3 0" 0 "INT" 0} \
      {"Scope trigpol3 1" 1 "INT" 0} \
      {"Scope trigpol3 2 -forgive" 1 "INT" 0} \
      {"Scope trigpol3" 1 "INT" 0} \
      {"Scope triggerpolarity 1" 1 "INT" 0} \
      {"Scope triggerpolarity -1 -forgive" 0 "INT" 0} \
      {"Scope triggerpolarity" 0 "INT" 0} \
      {"Scope triggerpolarity 0" 0 "INT" 0} \
      {"Scope triggerpolarity 1" 1 "INT" 0} \
      {"Scope triggerpolarity neg" 0 "INT" 0} \
      {"Scope triggerpolarity pos" 1 "INT" 0} \
      {"Scope triggerpolarity 2 -forgive" 1 "INT" 0} \
      {"Scope triggerpolarity" 1 "INT" 0} ]

   set testScopeList14 [list \
      {"Scope zerooffset 2" 2 "INT" 0} \
      {"Scope zerooffset -1 -forgive" 1 "INT" 0} \
      {"Scope zerooffset 0 -forgive" 1 "INT" 0} \
      {"Scope zerooffset" 1 "INT" 0} \
      {"Scope zerooffset 1" 1 "INT" 0} \
      {"Scope zerooffset 2" 2 "INT" 0} \
      {"Scope zerooffset 3 -forgive" 1 "INT" 0} \
      {"Scope zerooffset" 1 "INT" 0} \
      {"Scope zerooffset1 2" 2 "INT" 0} \
      {"Scope zerooffset1 -1 -forgive" 1 "INT" 0} \
      {"Scope zerooffset1 0 -forgive" 1 "INT" 0} \
      {"Scope zerooffset1" 1 "INT" 0} \
      {"Scope zerooffset1 1" 1 "INT" 0} \
      {"Scope zerooffset1 2" 2 "INT" 0} \
      {"Scope zerooffset1 3 -forgive" 1 "INT" 0} \
      {"Scope zerooffset1" 1 "INT" 0} \
      {"Scope zerooffset2 2" 2 "INT" 0} \
      {"Scope zerooffset2 -1 -forgive" 1 "INT" 0} \
      {"Scope zerooffset2 0 -forgive" 1 "INT" 0} \
      {"Scope zerooffset2" 1 "INT" 0} \
      {"Scope zerooffset2 1" 1 "INT" 0} \
      {"Scope zerooffset2 2" 2 "INT" 0} \
      {"Scope zerooffset2 3 -forgive" 1 "INT" 0} \
      {"Scope zerooffset2" 1 "INT" 0} \
      {"Scope zerooffset3 2" 2 "INT" 0} \
      {"Scope zerooffset3 -1 -forgive" 1 "INT" 0} \
      {"Scope zerooffset3 0 -forgive" 1 "INT" 0} \
      {"Scope zerooffset3" 1 "INT" 0} \
      {"Scope zerooffset3 1" 1 "INT" 0} \
      {"Scope zerooffset3 2" 2 "INT" 0} \
      {"Scope zerooffset3 3 -forgive" 1 "INT" 0} \
      {"Scope zerooffset3" 1 "INT" 0} \
      {"Scope setzerooffset 2" 2 "INT" 0} \
      {"Scope setzerooffset -1 -forgive" 1 "INT" 0} \
      {"Scope setzerooffset 0 -forgive" 1 "INT" 0} \
      {"Scope setzerooffset" 1 "INT" 0} \
      {"Scope setzerooffset 1" 1 "INT" 0} \
      {"Scope setzerooffset 2" 2 "INT" 0} \
      {"Scope setzerooffset 3 -forgive" 1 "INT" 0} \
      {"Scope setzerooffset" 1 "INT" 0} ]




#    {"calibrationgainoffset", "caloffset", MTR_CAL_X5},	// Structure - SCOPEMESSAGECAL 
#    {"calibrategain",   "calgain",    MTR_CAL_GAIN},	// Structure - convert to 2 INTS, no range checks









}

#############################################################################
# FUNCTION    : autoTestScope                                               #
# ARGUMENTS   : none                                                        #
# RETURNS     : none                                                        #
# PURPOSE     : Executes automatic test on RF Scope commands                #
#############################################################################
proc autoTestScope { } {
   global scopeTestList 

   package require Scope
   setupScopeTests 

   # Execute the test lists, and then destroy the test lists
   foreach testName $scopeTestList {
      testCompare $testName
      global $testName
      unset $testName
   }
   unset scopeTestList

   # Test commands with no data - requires visual inspection
   Scope trigarm
   Scope trigarm1
   Scope trigarm2
   Scope trigarm3
   Scope triggerarm
   Scope triggerarm1
   Scope triggerarm2
   Scope triggerarm3

   # Test string functions - requires visual inspection

   Scope ver 0 -forgive
   Scope ver 1
   Scope ver 2
   Scope ver 3
   Scope ver 4
   Scope ver 5 -forgive
   Scope version 1
   Scope version 2
   Scope version 3
   Scope version 4

   # Reset Scope to typical values after completion of the test
   Scope enable 1
}


