#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C) 2001        DESIGN ENGINEERING      #
# --------------------------------------------------------------------------#
# FILE        : test_BBAudio.tcl                                            #
# PRODUCT     : FITS                                                        #
# AUTHOR      : Bruce Butterfield                                           #
# DESCRIPTION : Test Baseband Audio engine commands                         #
#############################################################################

package require BBAudio

#############################################################################
# FUNCTION    : setupBBAudioTests                                           #
# ARGUMENTS   : none                                                        #
# RETURNS     : none                                                        #
# PURPOSE     : Sets up a list of BBAudio tests                             #
#############################################################################
proc setupBBAudioTests { } {
   global bbaudioTestList 

   # Define the list of BBAudio tests
   set bbaudioTestList [list \
           testBBAudioList1 testBBAudioList2 testBBAudioList3 testBBAudioList4 \
           testBBAudioList5 testBBAudioList6 testBBAudioList7 testBBAudioList8 \
           testBBAudioList9 testBBAudioList10 testBBAudioList11 testBBAudioList12 ]

   # Declare each test as a global variable
   foreach testName $bbaudioTestList {
      global $testName
   }

   #
   # Define test cases
   #
   set testBBAudioList1 [list \
      {"BBAudio demodfilt 1" 1 "INT" 0} \
      {"BBAudio demodfilt -1 -forgive" 0 "INT" 0} \
      {"BBAudio demodfilt" 0 "INT" 0} \
      {"BBAudio demodfilt 0" 0 "INT" 0} \
      {"BBAudio demodfilt 2 -forgive" 1 "INT" 0} \
      {"BBAudio demodfilt" 1 "INT" 0} \
      {"BBAudio demodfilt data" 0 "INT" 0} \
      {"BBAudio demodfilt sine" 1 "INT" 0} \
      {"BBAudio demodfilt square" 0 "INT" 0} \
      {"BBAudio demodfilt tone" 1 "INT" 0} \
      {"BBAudio demodfilt data" 0 "INT" 0} \
      {"BBAudio demodfilter 1" 1 "INT" 0} \
      {"BBAudio demodfilter -1 -forgive" 0 "INT" 0} \
      {"BBAudio demodfilter" 0 "INT" 0} \
      {"BBAudio demodfilter 0" 0 "INT" 0} \
      {"BBAudio demodfilter 2 -forgive" 1 "INT" 0} \
      {"BBAudio demodfilter" 1 "INT" 0} ]

   set testBBAudioList2 [list \
      {"BBAudio fgenfilt 1 " 1 "INT" 0} \
      {"BBAudio fgenfilt -1 -forgive" 0 "INT" 0} \
      {"BBAudio fgenfilt" 0 "INT" 0} \
      {"BBAudio fgenfilt 0" 0 "INT" 0} \
      {"BBAudio fgenfilt 2 -forgive" 1 "INT" 0} \
      {"BBAudio fgenfilt" 1 "INT" 0} \
      {"BBAudio fgenfilt data" 0 "INT" 0} \
      {"BBAudio fgenfilt sine" 1 "INT" 0} \
      {"BBAudio fgenfilt square" 0 "INT" 0} \
      {"BBAudio fgenfilt tone" 1 "INT" 0} \
      {"BBAudio fgenfilt data" 0 "INT" 0} \
      {"BBAudio fgenfilter 1" 1 "INT" 0} \
      {"BBAudio fgenfilter -1 -forgive" 0 "INT" 0} \
      {"BBAudio fgenfilter" 0 "INT" 0} \
      {"BBAudio fgenfilter 0 " 0 "INT" 0} \
      {"BBAudio fgenfilter 2 -forgive" 1 "INT" 0} \
      {"BBAudio fgenfilter" 1 "INT" 0} ]

   set testBBAudioList3 [list \
      {"BBAudio fgenbal 1 " 1 "INT" 0} \
      {"BBAudio fgenbal -1 -forgive" 0 "INT" 0} \
      {"BBAudio fgenbal" 0 "INT" 0} \
      {"BBAudio fgenbal 0 " 0 "INT" 0} \
      {"BBAudio fgenbal balanced" 1 "INT" 0} \
      {"BBAudio fgenbal unbalanced" 0 "INT" 0} \
      {"BBAudio fgenbal 2 -forgive" 1 "INT" 0} \
      {"BBAudio fgenbal" 1 "INT" 0} \
      {"BBAudio fgenbalance 1 " 1 "INT" 0} \
      {"BBAudio fgenbalance -1 -forgive" 0 "INT" 0} \
      {"BBAudio fgenbalance" 0 "INT" 0} \
      {"BBAudio fgenbalance 0 " 0 "INT" 0} \
      {"BBAudio fgenbalance 2 -forgive" 1 "INT" 0} \
      {"BBAudio fgenbalance" 1 "INT" 0} ]

   set testBBAudioList4 [list \
      {"BBAudio fgenlev 1 " 1 "INT" 0} \
      {"BBAudio fgenlev -1 -forgive" 0 "INT" 0} \
      {"BBAudio fgenlev" 0 "INT" 0} \
      {"BBAudio fgenlev 0 " 0 "INT" 0} \
      {"BBAudio fgenlev tenth" 1 "INT" 0} \
      {"BBAudio fgenlev normal" 0 "INT" 0} \
      {"BBAudio fgenlev 2 -forgive" 1 "INT" 0} \
      {"BBAudio fgenlev" 1 "INT" 0} \
      {"BBAudio fgenlevel 1 " 1 "INT" 0} \
      {"BBAudio fgenlevel -1 -forgive" 0 "INT" 0} \
      {"BBAudio fgenlevel" 0 "INT" 0} \
      {"BBAudio fgenlevel 0 " 0 "INT" 0} \
      {"BBAudio fgenlevel 2 -forgive" 1 "INT" 0} \
      {"BBAudio fgenlevel" 1 "INT" 0} ]

   set testBBAudioList5 [list \
      {"BBAudio sinadbal 1 " 1 "INT" 0} \
      {"BBAudio sinadbal -1 -forgive" 0 "INT" 0} \
      {"BBAudio sinadbal" 0 "INT" 0} \
      {"BBAudio sinadbal 0" 0 "INT" 0} \
      {"BBAudio sinadbal unbalanced" 1 "INT" 0} \
      {"BBAudio sinadbal balanced" 0 "INT" 0} \
      {"BBAudio sinadbal 2 -forgive" 1 "INT" 0} \
      {"BBAudio sinadbal" 1 "INT" 0} \
      {"BBAudio sinadbalance 1 " 1 "INT" 0} \
      {"BBAudio sinadbalance -1 -forgive" 0 "INT" 0} \
      {"BBAudio sinadbalance" 0 "INT" 0} \
      {"BBAudio sinadbalance 0 " 0 "INT" 0} \
      {"BBAudio sinadbalance 2 -forgive" 1 "INT" 0} \
      {"BBAudio sinadbalance" 1 "INT" 0} ]

   set testBBAudioList6 [list \
      {"BBAudio micbias 1 " 1 "INT" 0} \
      {"BBAudio micbias -1 -forgive" 0 "INT" 0} \
      {"BBAudio micbias" 0 "INT" 0} \
      {"BBAudio micbias 0" 0 "INT" 0} \
      {"BBAudio micbias powered" 1 "INT" 0} \
      {"BBAudio micbias normal" 0 "INT" 0} \
      {"BBAudio micbias 2 -forgive" 1 "INT" 0} \
      {"BBAudio micbias" 1 "INT" 0} \
      {"BBAudio microphonebias 1 " 1 "INT" 0} \
      {"BBAudio microphonebias -1 -forgive" 0 "INT" 0} \
      {"BBAudio microphonebias" 0 "INT" 0} \
      {"BBAudio microphonebias 0 " 0 "INT" 0} \
      {"BBAudio microphonebias 2 -forgive" 1 "INT" 0} \
      {"BBAudio microphonebias" 1 "INT" 0} ]

   set testBBAudioList7 [list \
      {"BBAudio speakervolrel -32 -forgive" 0 "INT" 0} \
      {"BBAudio speakervolrel 5 " 5 "INT" 0} \
      {"BBAudio speakervolrel 10 " 15 "INT" 0} \
      {"BBAudio speakervolrel -31" 0 "INT" 0} \
      {"BBAudio speakervolrel 25 " 25 "INT" 0} \
      {"BBAudio speakervolrel -31" 0 "INT" 0} \
      {"BBAudio speakervolrel 31 " 31 "INT" 0} \
      {"BBAudio speakervolrel -31" 0 "INT" 0} \
      {"BBAudio speakervolrel 32 -forgive" 31 "INT" 0} \
      {"BBAudio speakervolrel -31" 0 "INT" 0} \
      {"BBAudio speakervolumerel -32 -forgive" 0 "INT" 0} \
      {"BBAudio speakervolumerel 5 " 5 "INT" 0} \
      {"BBAudio speakervolumerel 10 " 15 "INT" 0} \
      {"BBAudio speakervolumerel -31" 0 "INT" 0} \
      {"BBAudio speakervolumerel 25 " 25 "INT" 0} \
      {"BBAudio speakervolumerel -13" 12 "INT" 0} \
      {"BBAudio speakervolumerel 31 " 31 "INT" 0} \
      {"BBAudio speakervolumerel -31" 0 "INT" 0} \
      {"BBAudio speakervolumerel 32 -forgive" 31 "INT" 0} \
      {"BBAudio speakervolumerel -15" 16 "INT" 0} ]

   set testBBAudioList8 [list \
      {"BBAudio speakervol 5 " 5 "INT" 0} \
      {"BBAudio speakervol -1 -forgive" 0 "INT" 0} \
      {"BBAudio speakervol" 0 "INT" 0} \
      {"BBAudio speakervol 10 " 10 "INT" 0} \
      {"BBAudio speakervol 25 " 25 "INT" 0} \
      {"BBAudio speakervol 31 " 31 "INT" 0} \
      {"BBAudio speakervol 32 -forgive" 31 "INT" 0} \
      {"BBAudio speakervol" 31 "INT" 0} \
      {"BBAudio speakervol 0" 0 "INT" 0} \
      {"BBAudio speakervolume 5 " 5 "INT" 0} \
      {"BBAudio speakervolume -1 -forgive" 0 "INT" 0} \
      {"BBAudio speakervolume" 0 "INT" 0} \
      {"BBAudio speakervolume 10 " 10 "INT" 0} \
      {"BBAudio speakervolume 25 " 25 "INT" 0} \
      {"BBAudio speakervolume 31 " 31 "INT" 0} \
      {"BBAudio speakervolume 32 -forgive" 31 "INT" 0} \
      {"BBAudio speakervolume" 31 "INT" 0} \
      {"BBAudio speakervolume 0" 0 "INT" 0} ]

   set testBBAudioList9 [list \
      {"BBAudio spkrvolpercent 5.0" 5.0 "DBL" 0.001} \
      {"BBAudio spkrvolpercent -1.0 -forgive" 0.0 "DBL" 0.001} \
      {"BBAudio spkrvolpercent" 0.0 "DBL" 0.001} \
      {"BBAudio spkrvolpercent 10.0" 10.0 "DBL" 0.001} \
      {"BBAudio spkrvolpercent 50.0" 50.0 "DBL" 0.001} \
      {"BBAudio spkrvolpercent 95.0" 95.0 "DBL" 0.001} \
      {"BBAudio spkrvolpercent 101.0 -forgive" 100.0 "DBL" 0.001} \
      {"BBAudio spkrvolpercent" 100.0 "DBL" 0.001} \
      {"BBAudio spkrvolpercent 0.0" 0.0 "DBL" 0.001} \
      {"BBAudio speakervolpercent 5.0" 5.0 "DBL" 0.001} \
      {"BBAudio speakervolpercent -1.0 -forgive" 0.0 "DBL" 0.001} \
      {"BBAudio speakervolpercent" 0.0 "DBL" 0.001} \
      {"BBAudio speakervolpercent 10.0" 10.0 "DBL" 0.001} \
      {"BBAudio speakervolpercent 50.0" 50.0 "DBL" 0.001} \
      {"BBAudio speakervolpercent 95.0" 95.0 "DBL" 0.001} \
      {"BBAudio speakervolpercent 101.0 -forgive" 100.0 "DBL" 0.001} \
      {"BBAudio speakervolpercent" 100.0 "DBL" 0.001} \
      {"BBAudio speakervolpercent 0.0" 0.0 "DBL" 0.001} ]

   set testBBAudioList10 [list \
      {"BBAudio spkrvol_temp 5.0" 5.0 "DBL" 0.001} \
      {"BBAudio spkrvol_temp -1.0 -forgive" 0.0 "DBL" 0.001} \
      {"BBAudio spkrvol_temp" 0.0 "DBL" 0.001} \
      {"BBAudio spkrvol_temp 10.0" 10.0 "DBL" 0.001} \
      {"BBAudio spkrvol_temp 50.0" 50.0 "DBL" 0.001} \
      {"BBAudio spkrvol_temp 95.0" 95.0 "DBL" 0.001} \
      {"BBAudio spkrvol_temp 101.0 -forgive" 100.0 "DBL" 0.001} \
      {"BBAudio spkrvol_temp" 100.0 "DBL" 0.001} \
      {"BBAudio spkrvol_temp 0.0" 0.0 "DBL" 0.001} \
      {"BBAudio speakervol_temp 5.0" 5.0 "DBL" 0.001} \
      {"BBAudio speakervol_temp -1.0 -forgive" 0.0 "DBL" 0.001} \
      {"BBAudio speakervol_temp" 0.0 "DBL" 0.001} \
      {"BBAudio speakervol_temp 10.0" 10.0 "DBL" 0.001} \
      {"BBAudio speakervol_temp 50.0" 50.0 "DBL" 0.001} \
      {"BBAudio speakervol_temp 95.0" 95.0 "DBL" 0.001} \
      {"BBAudio speakervol_temp 101.0 -forgive" 100.0 "DBL" 0.001} \
      {"BBAudio speakervol_temp" 100.0 "DBL" 0.001} \
      {"BBAudio speakervol_temp 0.0" 0.0 "DBL" 0.001} ]

   set testBBAudioList11 [list \
      {"BBAudio mix l1 speaker 0"      "0 0" "STR" 0} \
      {"BBAudio mix mic speaker 1"     "1 0" "STR" 0} \
      {"BBAudio mix sinad speaker 0"   "2 0" "STR" 0} \
      {"BBAudio mix file speaker 1"    "3 0" "STR" 0} \
      {"BBAudio mix fgen0 speaker off" "4 0" "STR" 0} \
      {"BBAudio mix fgen1 speaker on"  "5 0" "STR" 0} \
      {"BBAudio mix fgen2 speaker on"  "6 0" "STR" 0} \
      {"BBAudio mix m1 speaker 0"      "7 0" "STR" 0} \
      {"BBAudio mix fgen3 speaker 1"   "7 0" "STR" 0} \
      {"BBAudio mix m2 speaker 0"      "8 0" "STR" 0} \
      {"BBAudio mix fgen4 speaker 1"   "8 0" "STR" 0} \
      {"BBAudio mix user0 speaker off" "9 0" "STR" 0} \
      {"BBAudio mix user1 speaker on"  "10 0" "STR" 0} \
      {"BBAudio mix user2 speaker off" "11 0" "STR" 0} \
      {"BBAudio mix user3 speaker on"  "12 0" "STR" 0} \
      {"BBAudio mix user4 speaker off" "13 0" "STR" 0} \
      {"BBAudio mix user5 speaker on"  "14 0" "STR" 0} \
      {"BBAudio mixer l1 demod 0"      "0 1" "STR" 0} \
      {"BBAudio mixer mic demod 1"     "1 1" "STR" 0} \
      {"BBAudio mixer sinad demod 0"   "2 1" "STR" 0} \
      {"BBAudio mixer file fgen 1"     "3 2" "STR" 0} \
      {"BBAudio mixer fgen0 fgen off"  "4 2" "STR" 0} \
      {"BBAudio mixer fgen1 fgen on"   "5 2" "STR" 0} \
      {"BBAudio mixer fgen2 l1 on"     "6 3" "STR" 0} \
      {"BBAudio mixer m1 l1 0"         "7 3" "STR" 0} \
      {"BBAudio mixer fgen3 l1 1"      "7 3" "STR" 0} \
      {"BBAudio mixer m2 user0 0"      "8 4" "STR" 0} \
      {"BBAudio mixer fgen4 user0 1"   "8 4" "STR" 0} \
      {"BBAudio mixer user0 user0 off" "9 4" "STR" 0} \
      {"BBAudio mixer user1 demod on"  "10 1" "STR" 0} \
      {"BBAudio mixer user2 fgen off"  "11 2" "STR" 0} \
      {"BBAudio mixer user3 l1 on"     "12 3" "STR" 0} \
      {"BBAudio mixer user4 user0 off" "13 4" "STR" 0} \
      {"BBAudio mixer user5 user1 on"  "14 5" "STR" 0} \
      {"BBAudio mix l1 l1 0"           "0 3"  "STR" 0} \
      {"BBAudio mix l1 l1 1"           "0 3"  "STR" 0} \
      {"BBAudio mixer user0 user0 on"  "9 4"  "STR" 0} \
      {"BBAudio mixer user0 user0 off" "9 4"  "STR" 0} \
      {"BBAudio mixer user1 user1 on"  "10 5" "STR" 0} \
      {"BBAudio mixer user1 user1 off" "10 5" "STR" 0} ]

   set testBBAudioList12 [list \
      {"BBAudio route none l1"    "-1 0" "STR" 0} \
      {"BBAudio route l1 l1"      "0 0" "STR" 0} \
      {"BBAudio route mic l1"     "1 0" "STR" 0} \
      {"BBAudio route sinad l1"   "2 0" "STR" 0} \
      {"BBAudio route file l1"    "3 0" "STR" 0} \
      {"BBAudio route fgen0 l1"   "4 0" "STR" 0} \
      {"BBAudio route fgen1 l1"   "5 0" "STR" 0} \
      {"BBAudio route fgen2 l1"   "6 0" "STR" 0} \
      {"BBAudio route m1 l1"      "7 0" "STR" 0} \
      {"BBAudio route fgen3 l1"   "7 0" "STR" 0} \
      {"BBAudio route m2 l1"      "8 0" "STR" 0} \
      {"BBAudio route fgen4 l1"   "8 0" "STR" 0} \
      {"BBAudio route speaker l1" "9 0" "STR" 0} \
      {"BBAudio route demod l1"   "10 0" "STR" 0} \
      {"BBAudio route fgen l1"    "11 0" "STR" 0} \
      {"BBAudio route mixl1 l1"   "12 0" "STR" 0} \
      {"BBAudio route user0 l1"   "13 0" "STR" 0} \
      {"BBAudio route user1 l1"   "14 0" "STR" 0} \
      {"BBAudio route afcounter l1" "15 0" "STR" 0} \
      {"BBAudio route sinadmtr l1"  "16 0" "STR" 0} \
      {"BBAudio route scope l1"   "17 0" "STR" 0} \
      {"BBAudio route scalar l1"  "18 0" "STR" 0} \
      {"BBAudio route m_f4 l1"    "19 0" "STR" 0} \
      {"BBAudio route m_f5 l1"    "20 0" "STR" 0} \
      {"BBAudio route m_f6 l1"    "21 0" "STR" 0} \
      {"BBAudio route m_f7 l1"    "22 0" "STR" 0} \
      {"BBAudio route m_f8 l1"    "23 0" "STR" 0} \
      {"BBAudio route none m_f8"    "-1 23" "STR" 0} \
      {"BBAudio route l1 l1"         "0 0" "STR" 0} \
      {"BBAudio route mic mic"       "1 1" "STR" 0} \
      {"BBAudio route sinad sinad"   "2 2" "STR" 0} \
      {"BBAudio route file file"     "3 3" "STR" 0} \
      {"BBAudio route fgen0 fgen0"   "4 4" "STR" 0} \
      {"BBAudio route fgen1 fgen1"   "5 5" "STR" 0} \
      {"BBAudio route fgen2 fgen2"   "6 6" "STR" 0} \
      {"BBAudio route m1 fgen3"      "7 7" "STR" 0} \
      {"BBAudio route fgen3 m1"      "7 7" "STR" 0} \
      {"BBAudio route m2 fgen4"      "8 8" "STR" 0} \
      {"BBAudio route fgen4 m2"      "8 8" "STR" 0} \
      {"BBAudio route speaker user0" "9 9" "STR" 0} \
      {"BBAudio route demod user1"   "10 10" "STR" 0} \
      {"BBAudio route fgen user2"    "11 11" "STR" 0} \
      {"BBAudio route mixl1 user3"   "12 12" "STR" 0} \
      {"BBAudio route user0 user4"   "13 13" "STR" 0} \
      {"BBAudio route user1 user5"   "14 14" "STR" 0} \
      {"BBAudio route afcounter afcounter" "15 15" "STR" 0} \
      {"BBAudio route sinadmtr sinadmtr"   "16 16" "STR" 0} \
      {"BBAudio route scope scope"   "17 17" "STR" 0} \
      {"BBAudio route scalar scalar" "18 18" "STR" 0} \
      {"BBAudio route m_f4 m_f4"     "19 19" "STR" 0} \
      {"BBAudio route m_f5 m_f5"     "20 20" "STR" 0} \
      {"BBAudio route m_f6 m_f6"     "21 21" "STR" 0} \
      {"BBAudio route m_f7 m_f7"     "22 22" "STR" 0} \
      {"BBAudio route m_f8 m_f8"     "23 23" "STR" 0} \
      {"BBAudio audioroute none m_f8"     "-1 23" "STR" 0} \
      {"BBAudio audioroute l1 m_f7"        "0 22" "STR" 0} \
      {"BBAudio audioroute mic m_f6"       "1 21" "STR" 0} \
      {"BBAudio audioroute sinad m_f5"     "2 20" "STR" 0} \
      {"BBAudio audioroute file m_f4"      "3 19" "STR" 0} \
      {"BBAudio audioroute fgen0 scalar"   "4 18" "STR" 0} \
      {"BBAudio audioroute fgen1 scope"    "5 17" "STR" 0} \
      {"BBAudio audioroute fgen2 sinadmtr" "6 16" "STR" 0} \
      {"BBAudio audioroute m1 afcounter"   "7 15" "STR" 0} \
      {"BBAudio audioroute fgen3 user5"    "7 14" "STR" 0} \
      {"BBAudio audioroute m2 user4"       "8 13" "STR" 0} \
      {"BBAudio audioroute fgen4 user3"    "8 12" "STR" 0} \
      {"BBAudio audioroute speaker user2"  "9 11" "STR" 0} \
      {"BBAudio audioroute demod user1"    "10 10" "STR" 0} \
      {"BBAudio audioroute fgen user0"     "11 9" "STR" 0} \
      {"BBAudio audioroute mixl1 fgen4"    "12 8" "STR" 0} \
      {"BBAudio audioroute user0 m2"       "13 8" "STR" 0} \
      {"BBAudio audioroute user1 fgen3"    "14 7" "STR" 0} \
      {"BBAudio audioroute afcounter m1"   "15 7" "STR" 0} \
      {"BBAudio audioroute sinadmtr fgen2" "16 6" "STR" 0} \
      {"BBAudio audioroute scope fgen1"    "17 5" "STR" 0} \
      {"BBAudio audioroute scalar fgen0"   "18 4" "STR" 0} \
      {"BBAudio audioroute m_f4 file"      "19 3" "STR" 0} \
      {"BBAudio audioroute m_f5 sinad"     "20 2" "STR" 0} \
      {"BBAudio audioroute m_f6 mic"       "21 1" "STR" 0} \
      {"BBAudio audioroute m_f7 l1"        "22 0" "STR" 0} \
      {"BBAudio audioroute m_f8 l1"        "23 0" "STR" 0} ]

   set testBBAudioList13 [list \
      {"BBAudio enmeter 3 0"        "3 0" "STR" 0} \
      {"BBAudio enmeter 3 1"        "3 1" "STR" 0} \
      {"BBAudio enmeter 3 off"      "3 0" "STR" 0} \
      {"BBAudio enmeter 3 on"       "3 1" "STR" 0} \
      {"BBAudio enmeter 4 0"        "3 0" "STR" 0} \
      {"BBAudio enmeter 4 1"        "3 1" "STR" 0} \
      {"BBAudio enmeter 4 off"      "3 0" "STR" 0} \
      {"BBAudio enmeter 4 on"       "3 1" "STR" 0} \
      {"BBAudio enmeter 5 0"        "3 0" "STR" 0} \
      {"BBAudio enmeter 5 1"        "3 1" "STR" 0} \
      {"BBAudio enmeter 5 off"      "3 0" "STR" 0} \
      {"BBAudio enmeter 5 on"       "3 1" "STR" 0} \
      {"BBAudio enmeter 6 0"        "3 0" "STR" 0} \
      {"BBAudio enmeter 6 1"        "3 1" "STR" 0} \
      {"BBAudio enmeter 6 off"      "3 0" "STR" 0} \
      {"BBAudio enmeter 6 on"       "3 1" "STR" 0} \
      {"BBAudio enmeter 7 0"        "3 0" "STR" 0} \
      {"BBAudio enmeter 7 1"        "3 1" "STR" 0} \
      {"BBAudio enmeter 7 off"      "3 0" "STR" 0} \
      {"BBAudio enmeter 7 on"       "3 1" "STR" 0} \
      {"BBAudio enmeter 8 0"        "3 0" "STR" 0} \
      {"BBAudio enmeter 8 1"        "3 1" "STR" 0} \
      {"BBAudio enmeter 8 off"      "3 0" "STR" 0} \
      {"BBAudio enmeter 8 on"       "3 1" "STR" 0} ]



}

#############################################################################
# FUNCTION    : autoTestScope                                               #
# ARGUMENTS   : none                                                        #
# RETURNS     : none                                                        #
# PURPOSE     : Executes automatic test on RF Scope commands                #
#############################################################################
proc autoTestBBAudio { } {
   global bbaudioTestList 

   package require BBAudio
   setupBBAudioTests 

   # Execute the test lists, and then destroy the test lists
   foreach testName $bbaudioTestList {
      testCompare $testName
      global $testName
      unset $testName
   }
   unset bbaudioTestList

   # Test commands with no data - requires visual inspection
   BBAudio ClrRoute
   BBAudio ClearRoute
#   BBAudio SpkrVolReset
#   BBAudio SpeakerVolReset

   # Test string functions - requires visual inspection

   # Reset hardware to typical values after completion of the test
   BBAudio speakervol 0



}


