#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C) 2001, 2003  DESIGN ENGINEERING      #
# --------------------------------------------------------------------------#
# FILE        : 1010.test_RFError.tcl                                       #
# PRODUCT     : FITS                                                        #
# AUTHOR      : Bruce Butterfield                                           #
# DESCRIPTION : Remote Command Line Test: RF Error                          #
#############################################################################

package require RFError
package require BIT

#****************************************************************************
# Test variables
#****************************************************************************
namespace eval ::cmdline_RFError {
    variable testCaseList		;# List of test cases
    variable shortDesc			;# Test description - menu size
    variable longDesc			;# Test description - used by reports
    variable testTime			;# Test duration (Seconds - approximation)

    # Define the list of RF Error tests
    set testCaseList [list \
           testRferrorList1 testRferrorList2 testRferrorList3 testRferrorList4 \
           testRferrorList5 testRferrorList6 testRferrorList7 testRferrorList8 ]

    # Declare each test as a namespace variable
    foreach testName $testCaseList {
       variable $testName
    }

    set shortDesc "RFError cmds"
    set longDesc "RF Error command line test"
    set testTime 0.50

#****************************************************************************
# Name:        setup
# Parameters:  none
# Description: Sets up a list of RF Error tests
#****************************************************************************
 proc setup { } {
    variable testCaseList

    # Declare each test as a namespace variable
    foreach testName $testCaseList {
       variable $testName
    }

    #
    # Define test cases
    #
    set testRferrorList1 [list \
      {"RFError avg 0 -forgive" 1 "INT" 0} \
      {"RFError avg 1" 1 "INT" 0} \
      {"RFError avg 100" 100 "INT" 0} \
      {"RFError avg 101 -forgive" 100 "INT" 0} \
      {"RFError average 0 -forgive" 1 "INT" 0} \
      {"RFError average 1" 1 "INT" 0} \
      {"RFError average 100" 100 "INT" 0} \
      {"RFError average 101 -forgive" 100 "INT" 0} ]

    set testRferrorList2 [list \
      {"RFError gate -1 -forgive" 0 "INT" 0} \
      {"RFError gate 1" 1 "INT" 0} \
      {"RFError gate 0" 0 "INT" 0} \
      {"RFError gate second" 1 "INT" 0} \
      {"RFError gate tenth" 0 "INT" 0} \
      {"RFError gate onesec" 1 "INT" 0} \
      {"RFError gate tenthsec" 0 "INT" 0} \
      {"RFError gate 2 -forgive" 1 "INT" 0} ]

    set testRferrorList3 [list \
      {"RFError en -1 -forgive" 3 "INT" 0} \
      {"RFError en 0" 1 "INT" 0} \
      {"RFError en 1" 1 "INT" 0} \
      {"RFError en 2" 3 "INT" 0} \
      {"RFError en 3" 3 "INT" 0} \
      {"RFError en disable" 1 "INT" 0} \
      {"RFError en enable" 3 "INT" 0} \
      {"RFError en off" 1 "INT" 0} \
      {"RFError en on" 3 "INT" 0} \
      {"RFError en 4 -forgive" 1 "INT" 0} \
      {"RFError enable -1 -forgive" 3 "INT" 0} \
      {"RFError enable 0" 1 "INT" 0} \
      {"RFError enable 1" 1 "INT" 0} \
      {"RFError enable 2" 3 "INT" 0} \
      {"RFError enable 3" 3 "INT" 0} \
      {"RFError enable 4 -forgive" 1 "INT" 0} ]

    set testRferrorList4 [list \
      {"RFError run -1 -forgive" 0 "INT" 0} \
      {"RFError run 1" 1 "INT" 0} \
      {"RFError run 0" 0 "INT" 0} \
      {"RFError run on" 1 "INT" 0} \
      {"RFError run off" 0 "INT" 0} \
      {"RFError run 4 -forgive" 1 "INT" 0} ]

    set testRferrorList5 [list \
      {"RFError ulim -3000000000.1 -forgive" -3000000000.0 "DBL" 0.001} \
      {"RFError ulim -2987654321.0" -2987654321.0 "DBL" 0.001} \
      {"RFError ulim -3000000.0" -3000000.0 "DBL" 0.001} \
      {"RFError ulim -10000.0 hz" -10000.0 "DBL" 0.001} \
      {"RFError ulim -1234.5" -1234.5 "DBL" 0.001} \
      {"RFError ulim 0.1 " 0.1 "DBL" 0.001} \
      {"RFError ulim 12.3" 12.3 "DBL" 0.001} \
      {"RFError ulim 1234.5" 1234.5 "DBL" 0.001} \
      {"RFError ulim 9999.9" 9999.9 "DBL" 0.001} \
      {"RFError ulim 10.0001 khz" 10000.1 "DBL" 0.001} \
      {"RFError ulim 3.0000000 mhz" 3000000.0 "DBL" 0.001} \
      {"RFError ulim 2987654321.0" 2987654321.0 "DBL" 0.001} \
      {"RFError ulim 3.0000000001 ghz -forgive" 3000000000.0 "DBL" 0.001} \
      {"RFError ulimit -3000000000.1 -forgive" -3000000000.0 "DBL" 0.001} \
      {"RFError ulimit -2987654321.0" -2987654321.0 "DBL" 0.001} \
      {"RFError ulimit -3000000.0" -3000000.0 "DBL" 0.001} \
      {"RFError ulimit -10000.0" -10000.0 "DBL" 0.001} \
      {"RFError ulimit -1234.5" -1234.5 "DBL" 0.001} \
      {"RFError ulimit 0.1 " 0.1 "DBL" 0.001} \
      {"RFError ulimit 12.3" 12.3 "DBL" 0.001} \
      {"RFError ulimit 1234.5" 1234.5 "DBL" 0.001} \
      {"RFError ulimit 9999.9" 9999.9 "DBL" 0.001} \
      {"RFError ulimit 10000.1" 10000.1 "DBL" 0.001} \
      {"RFError ulimit 3000000.0" 3000000.0 "DBL" 0.001} \
      {"RFError ulimit 2987654321.0" 2987654321.0 "DBL" 0.001} \
      {"RFError ulimit 3000000000.1 -forgive" 3000000000.0 "DBL" 0.001} ]

    set testRferrorList6 [list \
      {"RFError llim -3000000000.1 -forgive" -3000000000.0 "DBL" 0.001} \
      {"RFError llim -2987654321.0" -2987654321.0 "DBL" 0.001} \
      {"RFError llim -3000000.0" -3000000.0 "DBL" 0.001} \
      {"RFError llim -10000.0 hz" -10000.0 "DBL" 0.001} \
      {"RFError llim -1234.5" -1234.5 "DBL" 0.001} \
      {"RFError llim 0.1 " 0.1 "DBL" 0.001} \
      {"RFError llim 12.3" 12.3 "DBL" 0.001} \
      {"RFError llim 1234.5" 1234.5 "DBL" 0.001} \
      {"RFError llim 9.9999 khz" 9999.9 "DBL" 0.001} \
      {"RFError llim 10000.1" 10000.1 "DBL" 0.001} \
      {"RFError llim 3000000.0" 3000000.0 "DBL" 0.001} \
      {"RFError llim 2.9876543210 ghz" 2987654321.0 "DBL" 0.001} \
      {"RFError llim 3000000000.1 -forgive" 3000000000.0 "DBL" 0.001} \
      {"RFError llimit -3000000000.1 -forgive" -3000000000.0 "DBL" 0.001} \
      {"RFError llimit -2987.6543210 mhz" -2987654321.0 "DBL" 0.001} \
      {"RFError llimit -3000000.0" -3000000.0 "DBL" 0.001} \
      {"RFError llimit -10000.0" -10000.0 "DBL" 0.001} \
      {"RFError llimit -1234.5" -1234.5 "DBL" 0.001} \
      {"RFError llimit 0.1 " 0.1 "DBL" 0.001} \
      {"RFError llimit 12.3" 12.3 "DBL" 0.001} \
      {"RFError llimit 1234.5" 1234.5 "DBL" 0.001} \
      {"RFError llimit 9999.9" 9999.9 "DBL" 0.001} \
      {"RFError llimit 10000.1" 10000.1 "DBL" 0.001} \
      {"RFError llimit 3000000.0" 3000000.0 "DBL" 0.001} \
      {"RFError llimit 2987654321.0" 2987654321.0 "DBL" 0.001} \
      {"RFError llimit 3000000000.1 -forgive" 3000000000.0 "DBL" 0.001} ]

    set testRferrorList7 [list \
      {"RFError al -1 -forgive" 0 "INT" 0} \
      {"RFError al 1" 1 "INT" 0} \
      {"RFError al 0" 0 "INT" 0} \
      {"RFError al 2" 2 "INT" 0} \
      {"RFError al 3" 3 "INT" 0} \
      {"RFError al off" 0 "INT" 0} \
      {"RFError al enablelo" 1 "INT" 0} \
      {"RFError al enablehi" 2 "INT" 0} \
      {"RFError al enableboth" 3 "INT" 0} \
      {"RFError al 4 -forgive" 0 "INT" 0} \
      {"RFError alerts -1 -forgive" 0 "INT" 0} \
      {"RFError alerts 1" 1 "INT" 0} \
      {"RFError alerts 0" 0 "INT" 0} \
      {"RFError alerts 2" 2 "INT" 0} \
      {"RFError alerts 3" 3 "INT" 0} \
      {"RFError alerts 4 -forgive" 0 "INT" 0} ]

    set testRferrorList8 [list \
      {"RFError peak -1 -forgive" 0 "INT" 0} \
      {"RFError peak 1" 1 "INT" 0} \
      {"RFError peak 0" 0 "INT" 0} \
      {"RFError peak 1" 1 "INT" 0} \
      {"RFError peak off" 0 "INT" 0} \
      {"RFError peak on" 1 "INT" 0} \
      {"RFError peak 2 -forgive" 1 "INT" 0} \
      {"RFError peakhold -1 -forgive" 0 "INT" 0} \
      {"RFError peakhold 1" 1 "INT" 0} \
      {"RFError peakhold 0" 0 "INT" 0} \
      {"RFError peakhold 1" 1 "INT" 0} \
      {"RFError peakhold 2 -forgive" 1 "INT" 0} ]

 }

#****************************************************************************
# Name:        autoTest
# Parameters:  resultsLevel		Indicates the results display level
# Description: Executes automatic test on AFCounter commands
#****************************************************************************
 proc autoTest { resultsLevel } {
    variable testCaseList

    # Declare each test as a namespace variable
    foreach testName $testCaseList {
       variable $testName
    }

    setup 

    # Execute the test lists, and then destroy the test lists
    ::BIT::resetTestStats "cmdline_RFError"
    foreach testName $testCaseList {
       ::BIT::runCmdLineTests "cmdline_RFError" $testName
       unset $testName
    }

    if { ($resultsLevel == "Full") || ($resultsLevel == "Manual") } \
    {
       # Test commands with no data - requires visual inspection
       ::BIT::putsTest [RFError phrst]
       ::BIT::putsTest [RFError phreset]

       # Test range functions - requires visual inspection
        foreach cmd {average alerts gate llimit peakhold run ulimit version } \
       {
          set temp [AFCounter $cmd -range]
          ::BIT::putsTest "The range of <$cmd> is:  $temp"
       }

       # Test string functions - requires visual inspection
       ::BIT::putsTest [RFError res]
       ::BIT::putsTest [RFError resources]
       ::BIT::putsTest [RFError ver 0]
       ::BIT::putsTest [RFError version 1]
    }

    # Reset RF Error engine to typical values after completion of the test
    RFError en 3
    RFError run 0
    RFError avg 10
    RFError gate 0
 }

}

# Add this test to the list of tests
::BIT::addTest cmdline_RFError

