#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C) 2001, 2003  DESIGN ENGINEERING      #
# --------------------------------------------------------------------------#
# FILE        : 1011.test_Dvm.tcl                                           #
# PRODUCT     : FITS                                                        #
# AUTHOR      : Bruce Butterfield                                           #
# DESCRIPTION : Remote Command Line Test:  Dvm                              #
#############################################################################

package require Dvm
package require BIT

#****************************************************************************
# Test variables
#****************************************************************************
namespace eval ::cmdline_Dvm {
    variable testCaseList		;# List of test cases
    variable shortDesc			;# Test description - menu size
    variable longDesc			;# Test description - used by reports
    variable testTime			;# Test duration (Seconds - approximation)

    # Define the list of Dvm tests
    set testCaseList [list \
           testDvmList1 testDvmList2 testDvmList3 testDvmList4 \
           testDvmList5 testDvmList7 testDvmList8 \
           testDvmList9 testDvmList10 testDvmList11 testDvmList12 \
           testDvmList13 testDvmList14 testDvmList15 ]

    # Declare each test as a namespace variable
    foreach testName $testCaseList {
       variable $testName
    }

    set shortDesc "Dvm cmds"
    set longDesc "Dvm command line test"
    set testTime 0.50

#****************************************************************************
# Name:        setup
# Parameters:  none
# Description: Sets up a list of Dvm tests
#****************************************************************************
 proc setup { } {
    variable testCaseList

    # Declare each test as a namespace variable
    foreach testName $testCaseList {
       variable $testName
    }

    #
    # Define test cases
    #
    set testDvmList1 [list \
      {"Dvm avg 1 -forgive" 2 "INT" 0} \
      {"Dvm avg 2" 2 "INT" 0} \
      {"Dvm avg 100" 100 "INT" 0} \
      {"Dvm avg 255" 255 "INT" 0} \
      {"Dvm avg 256 -forgive" 255 "INT" 0} \
      {"Dvm average 1 -forgive" 2 "INT" 0} \
      {"Dvm average 2" 2 "INT" 0} \
      {"Dvm average 100" 100 "INT" 0} \
      {"Dvm average 255" 255 "INT" 0} \
      {"Dvm average 256 -forgive" 255 "INT" 0} ]

    set testDvmList2 [list \
      {"Dvm en -1 -forgive" 3 "INT" 0} \
      {"Dvm en 0" 1 "INT" 0} \
      {"Dvm en 1" 1 "INT" 0} \
      {"Dvm en 2" 3 "INT" 0} \
      {"Dvm en 3" 3 "INT" 0} \
      {"Dvm en disable" 1 "INT" 0} \
      {"Dvm en enable" 3 "INT" 0} \
      {"Dvm en off" 1 "INT" 0} \
      {"Dvm en on" 3 "INT" 0} \
      {"Dvm en 4 -forgive" 1 "INT" 0} \
      {"Dvm enable -1 -forgive" 3 "INT" 0} \
      {"Dvm enable 0" 1 "INT" 0} \
      {"Dvm enable 1" 1 "INT" 0} \
      {"Dvm enable 2" 3 "INT" 0} \
      {"Dvm enable 3" 3 "INT" 0} \
      {"Dvm enable 4 -forgive" 1 "INT" 0} ]

    set testDvmList3 [list \
      {"Dvm run -1 -forgive" 0 "INT" 0} \
      {"Dvm run 1" 1 "INT" 0} \
      {"Dvm run 0" 0 "INT" 0} \
      {"Dvm run on" 1 "INT" 0} \
      {"Dvm run off" 0 "INT" 0} \
      {"Dvm run 4 -forgive" 1 "INT" 0} ]

    set testDvmList4 [list \
      {"Dvm ulim -100.1 -forgive" -100.0 "DBL" 0.001} \
      {"Dvm ulim -99.9" -99.9 "DBL" 0.001} \
      {"Dvm ulim -12.34" -12.34 "DBL" 0.001} \
      {"Dvm ulim 0.12 " 0.12 "DBL" 0.001} \
      {"Dvm ulim 12.3" 12.3 "DBL" 0.001} \
      {"Dvm ulim 99.9" 99.9 "DBL" 0.001} \
      {"Dvm ulim 100.1 -forgive" 100.0 "DBL" 0.001} \
      {"Dvm ulimit -100.1 -forgive" -100.0 "DBL" 0.001} \
      {"Dvm ulimit -99.9" -99.9 "DBL" 0.001} \
      {"Dvm ulimit -12.34" -12.34 "DBL" 0.001} \
      {"Dvm ulimit 0.12 " 0.12 "DBL" 0.001} \
      {"Dvm ulimit 12.3" 12.3 "DBL" 0.001} \
      {"Dvm ulimit 99.9" 99.9 "DBL" 0.001} \
      {"Dvm ulimit 100.1 -forgive" 100.0 "DBL" 0.001} ]

    set testDvmList5 [list \
      {"Dvm llim -100.1 -forgive" -100.0 "DBL" 0.001} \
      {"Dvm llim -99.9" -99.9 "DBL" 0.001} \
      {"Dvm llim -12.34" -12.34 "DBL" 0.001} \
      {"Dvm llim 0.12 " 0.12 "DBL" 0.001} \
      {"Dvm llim 12.3" 12.3 "DBL" 0.001} \
      {"Dvm llim 99.9" 99.9 "DBL" 0.001} \
      {"Dvm llim 100.1 -forgive" 100.0 "DBL" 0.001} \
      {"Dvm llimit -100.1 -forgive" -100.0 "DBL" 0.001} \
      {"Dvm llimit -99.9" -99.9 "DBL" 0.001} \
      {"Dvm llimit -12.34" -12.34 "DBL" 0.001} \
      {"Dvm llimit 0.12 " 0.12 "DBL" 0.001} \
      {"Dvm llimit 12.3" 12.3 "DBL" 0.001} \
      {"Dvm llimit 99.9" 99.9 "DBL" 0.001} \
      {"Dvm llimit 100.1 -forgive" 100.0 "DBL" 0.001} ]

    set testDvmList7 [list \
      {"Dvm al -1 -forgive" 0 "INT" 0} \
      {"Dvm al 1" 1 "INT" 0} \
      {"Dvm al 0" 0 "INT" 0} \
      {"Dvm al 2" 2 "INT" 0} \
      {"Dvm al 3" 3 "INT" 0} \
      {"Dvm al off" 0 "INT" 0} \
      {"Dvm al enablelo" 1 "INT" 0} \
      {"Dvm al enablehi" 2 "INT" 0} \
      {"Dvm al enableboth" 3 "INT" 0} \
      {"Dvm al 4 -forgive" 0 "INT" 0} \
      {"Dvm alerts -1 -forgive" 0 "INT" 0} \
      {"Dvm alerts 1" 1 "INT" 0} \
      {"Dvm alerts 0" 0 "INT" 0} \
      {"Dvm alerts 2" 2 "INT" 0} \
      {"Dvm alerts 3" 3 "INT" 0} \
      {"Dvm alerts 4 -forgive" 0 "INT" 0} ]

    set testDvmList8 [list \
      {"Dvm peak -1 -forgive" 0 "INT" 0} \
      {"Dvm peak 1" 1 "INT" 0} \
      {"Dvm peak 0" 0 "INT" 0} \
      {"Dvm peak 1" 1 "INT" 0} \
      {"Dvm peak off" 0 "INT" 0} \
      {"Dvm peak on" 1 "INT" 0} \
      {"Dvm peak 2 -forgive" 1 "INT" 0} \
      {"Dvm peakhold -1 -forgive" 0 "INT" 0} \
      {"Dvm peakhold 1" 1 "INT" 0} \
      {"Dvm peakhold 0" 0 "INT" 0} \
      {"Dvm peakhold 1" 1 "INT" 0} \
      {"Dvm peakhold 2 -forgive" 1 "INT" 0} ]

    set testDvmList9 [list \
      {"Dvm mod 0 -forgive" 1 "INT" 0} \
      {"Dvm mod 1" 1 "INT" 0} \
      {"Dvm mod 2" 2 "INT" 0} \
      {"Dvm mod ac" 1 "INT" 0} \
      {"Dvm mod dc" 2 "INT" 0} \
      {"Dvm mod 3 -forgive" 1 "INT" 0} \
      {"Dvm mode 0 -forgive" 1 "INT" 0} \
      {"Dvm mode 1" 1 "INT" 0} \
      {"Dvm mode 2" 2 "INT" 0} \
      {"Dvm mode 3 -forgive" 1 "INT" 0} ]

    set testDvmList10 [list \
      {"Dvm zero -1 -forgive" 0 "INT" 0} \
      {"Dvm zero 1" 1 "INT" 0} \
      {"Dvm zero 0" 0 "INT" 0} \
      {"Dvm zero 2 -forgive" 1 "INT" 0} ]

    set testDvmList11 [list \
      {"Dvm load 0" 0 "INT" 0} \
      {"Dvm load 150" 150 "INT" 0} \
      {"Dvm load 600" 600 "INT" 0} \
      {"Dvm load 1000000" 1000000 "INT" 0} \
      {"Dvm load -1 -forgive" 0 "INT" 0} \
      {"Dvm load 74 -forgive" 0 "INT" 0} \
      {"Dvm load 75 -forgive" 150 "INT" 0} \
      {"Dvm load 76 -forgive" 150 "INT" 0} \
      {"Dvm load 374 -forgive" 150 "INT" 0} \
      {"Dvm load 375 -forgive" 600 "INT" 0} \
      {"Dvm load 376 -forgive" 600 "INT" 0} \
      {"Dvm load 1000 -forgive" 600 "INT" 0} \
      {"Dvm load 1001 -forgive" 1000000 "INT" 0} \
      {"Dvm load 10000 -forgive" 1000000 "INT" 0} \
      {"Dvm load 100000 -forgive" 1000000 "INT" 0} \
      {"Dvm load 2000000 -forgive" 1000000 "INT" 0} \
      {"Dvm impedance 0" 0 "INT" 0} \
      {"Dvm impedance 150" 150 "INT" 0} \
      {"Dvm impedance 600" 600 "INT" 0} \
      {"Dvm impedance 1000000" 1000000 "INT" 0} \
      {"Dvm impedance -1 -forgive" 0 "INT" 0} \
      {"Dvm impedance 74 -forgive" 0 "INT" 0} \
      {"Dvm impedance 75 -forgive" 150 "INT" 0} \
      {"Dvm impedance 76 -forgive" 150 "INT" 0} \
      {"Dvm impedance 374 -forgive" 150 "INT" 0} \
      {"Dvm impedance 375 -forgive" 600 "INT" 0} \
      {"Dvm impedance 376 -forgive" 600 "INT" 0} \
      {"Dvm impedance 1000 -forgive" 600 "INT" 0} \
      {"Dvm impedance 1001 -forgive" 1000000 "INT" 0} \
      {"Dvm impedance 10000 -forgive" 1000000 "INT" 0} \
      {"Dvm impedance 100000 -forgive" 1000000 "INT" 0} \
      {"Dvm impedance 2000000 -forgive" 1000000 "INT" 0} ]

    set testDvmList12 [list \
      {"Dvm hold -1 -forgive" 0 "INT" 0} \
      {"Dvm hold 1" 1 "INT" 0} \
      {"Dvm hold 0" 0 "INT" 0} \
      {"Dvm hold on" 1 "INT" 0} \
      {"Dvm hold off" 0 "INT" 0} \
      {"Dvm hold 1" 1 "INT" 0} \
      {"Dvm hold 2 -forgive" 0 "INT" 0} ]

    set testDvmList13 [list \
      {"Dvm linefil -1 -forgive" 60 "INT" 0} \
      {"Dvm linefil 0 -forgive" 60 "INT" 0} \
      {"Dvm linefil 49 -forgive" 60 "INT" 0} \
      {"Dvm linefil 50" 50 "INT" 0} \
      {"Dvm linefil 51 -forgive" 60 "INT" 0} \
      {"Dvm linefil 55 -forgive" 60 "INT" 0} \
      {"Dvm linefil 59 -forgive" 60 "INT" 0} \
      {"Dvm linefil 60" 60 "INT" 0} \
      {"Dvm linefil 61 -forgive" 60 "INT" 0} \
      {"Dvm linefil 100 -forgive" 60 "INT" 0} \
      {"Dvm linefilter -1 -forgive" 60 "INT" 0} \
      {"Dvm linefilter 0 -forgive" 60 "INT" 0} \
      {"Dvm linefilter 49 -forgive" 60 "INT" 0} \
      {"Dvm linefilter 50" 50 "INT" 0} \
      {"Dvm linefilter 51 -forgive" 60 "INT" 0} \
      {"Dvm linefilter 55 -forgive" 60 "INT" 0} \
      {"Dvm linefilter 59 -forgive" 60 "INT" 0} \
      {"Dvm linefilter 60" 60 "INT" 0} \
      {"Dvm linefilter 61 -forgive" 60 "INT" 0} \
      {"Dvm linefilter 100 -forgive" 60 "INT" 0} ]

    set testDvmList14 [list \
      {"Dvm hwran -401 -forgive" 0 "INT" 0} \
      {"Dvm hwran -400" -400 "INT" 0} \
      {"Dvm hwran -399 -forgive" 0 "INT" 0} \
      {"Dvm hwran -1 -forgive" 0 "INT" 0} \
      {"Dvm hwran 0" 0 "INT" 0} \
      {"Dvm hwran 1 -forgive" 0 "INT" 0} \
      {"Dvm hwran 3 -forgive" 0 "INT" 0} \
      {"Dvm hwran 4" 4 "INT" 0} \
      {"Dvm hwran 5 -forgive" 0 "INT" 0} \
      {"Dvm hwran 39 -forgive" 0 "INT" 0} \
      {"Dvm hwran 40" 40 "INT" 0} \
      {"Dvm hwran 41 -forgive" 0 "INT" 0} \
      {"Dvm hwran 399 -forgive" 0 "INT" 0} \
      {"Dvm hwran 400" 400 "INT" 0} \
      {"Dvm hwran 401 -forgive" 0 "INT" 0} \
      {"Dvm hwrange -401 -forgive" 0 "INT" 0} \
      {"Dvm hwrange -400" -400 "INT" 0} \
      {"Dvm hwrange -399 -forgive" 0 "INT" 0} \
      {"Dvm hwrange -1 -forgive" 0 "INT" 0} \
      {"Dvm hwrange 0" 0 "INT" 0} \
      {"Dvm hwrange 1 -forgive" 0 "INT" 0} \
      {"Dvm hwrange 3 -forgive" 0 "INT" 0} \
      {"Dvm hwrange 4" 4 "INT" 0} \
      {"Dvm hwrange 5 -forgive" 0 "INT" 0} \
      {"Dvm hwrange 39 -forgive" 0 "INT" 0} \
      {"Dvm hwrange 40" 40 "INT" 0} \
      {"Dvm hwrange 41 -forgive" 0 "INT" 0} \
      {"Dvm hwrange 399 -forgive" 0 "INT" 0} \
      {"Dvm hwrange 400" 400 "INT" 0} \
      {"Dvm hwrange 401 -forgive" 0 "INT" 0} ]

    set testDvmList15 [list \
      {"Dvm sour -1 -forgive" 0 "INT" 0} \
      {"Dvm sour 1" 1 "INT" 0} \
      {"Dvm sour 0" 0 "INT" 0} \
      {"Dvm sour 1" 1 "INT" 0} \
      {"Dvm sour 2" 2 "INT" 0} \
      {"Dvm sour dvm" 0 "INT" 0} \
      {"Dvm sour scope1" 1 "INT" 0} \
      {"Dvm sour scope2" 2 "INT" 0} \
      {"Dvm sour 3 -forgive" 0 "INT" 0} \
      {"Dvm source -1 -forgive" 0 "INT" 0} \
      {"Dvm source 1" 1 "INT" 0} \
      {"Dvm source 0" 0 "INT" 0} \
      {"Dvm source 1" 1 "INT" 0} \
      {"Dvm source 2" 2 "INT" 0} \
      {"Dvm source 3 -forgive" 0 "INT" 0} ]

}

#****************************************************************************
# Name:        autoTest
# Parameters:  resultsLevel		Indicates the results display level
# Description: Executes automatic test on Dvm commands
#****************************************************************************
 proc autoTest { resultsLevel } {
    variable testCaseList

    # Declare each test as a namespace variable
    foreach testName $testCaseList {
       variable $testName
    }

    setup 

    # Execute the test lists, and then destroy the test lists
    ::BIT::resetTestStats "cmdline_Dvm"
    foreach testName $testCaseList {
       ::BIT::runCmdLineTests "cmdline_Dvm" $testName
       unset $testName
    }

    if { ($resultsLevel == "Full") || ($resultsLevel == "Manual") } \
    {
       # Test commands with no data - requires visual inspection
       ::BIT::putsTest [Dvm phrst]
       ::BIT::putsTest [Dvm phreset]

       # Test range functions - requires visual inspection
        foreach cmd {alerts average llimit peakhold run ulimit version} \
       {
          set temp [Dvm $cmd -range]
          ::BIT::putsTest "The range of <$cmd> is:  $temp"
       }

       # Test string functions - requires visual inspection

       ::BIT::putsTest [Dvm res]
       ::BIT::putsTest [Dvm resources]
       ::BIT::putsTest [Dvm ver 0]
       ::BIT::putsTest [Dvm version 1]
    }

    # Reset Dvm engine to typical values after completion of the test
    Dvm enable 1
    Dvm run 0
    Dvm average 10
    Dvm hwrange 0
    Dvm source 0
    Dvm linefilter 60
    Dvm hold 0
    Dvm impedance 0
    Dvm mode 1
    Dvm zero 0
 }

}

# Add this test to the list of tests
::BIT::addTest cmdline_Dvm

