#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C) 2001, 2003  DESIGN ENGINEERING      #
# --------------------------------------------------------------------------#
# FILE        : 1012.test_Ber.tcl                                           #
# PRODUCT     : FITS                                                        #
# AUTHOR      : Bruce Butterfield                                           #
# DESCRIPTION : Remote Command Line Test: Bit Error Rate (BER) meter        #
#############################################################################

package require Ber
package require BIT

#****************************************************************************
# Test variables
#****************************************************************************
namespace eval ::cmdline_Ber {
    variable testCaseList		;# List of test cases
    variable shortDesc			;# Test description - menu size
    variable longDesc			;# Test description - used by reports
    variable testTime			;# Test duration (Seconds - approximation)

    # Define the list of Ber tests
    set testCaseList [list \
           testBerList1 testBerList2 testBerList3 testBerList4 \
           testBerList5 testBerList6 testBerList7 ]

    # Declare each test as a namespace variable
    foreach testName $testCaseList {
       variable $testName
    }

    set shortDesc "Ber cmds"
    set longDesc "Ber command line test"
    set testTime 0.50

#****************************************************************************
# Name:        setup
# Parameters:  none
# Description: Sets up a list of Ber tests
#****************************************************************************
 proc setup { } {
    variable testCaseList

    # Declare each test as a namespace variable
    foreach testName $testCaseList {
       variable $testName
    }

    #
    # Define test cases
    #
    set testBerList1 [list \
      {"Ber frames 0 0 -forgive" {1 0} "compareInts" 0} \
      {"Ber frames 1 0" {1 0} "compareInts" 0} \
      {"Ber frames 100 0" {100 0} "compareInts" 0} \
      {"Ber frames 199 0" {199 0} "compareInts" 0} \
      {"Ber frames 200 0" {200 0} "compareInts" 0} \
      {"Ber frames 201 0 -forgive" {200 0} "compareInts" 0} \
      {"Ber frames 100 -1 -forgive" {100 0} "compareInts" 0} \
      {"Ber frames 100 0" {100 0} "compareInts" 0} \
      {"Ber frames 100 1" {100 1} "compareInts" 0} \
      {"Ber frames 100 2" {100 2} "compareInts" 0} \
      {"Ber frames 100 3" {100 3} "compareInts" 0} \
      {"Ber frames 100 4" {100 4} "compareInts" 0} \
      {"Ber frames 100 5" {100 5} "compareInts" 0} \
      {"Ber frames 100 6 -forgive" {100 5} "compareInts" 0} \
      {"Ber framedepth 0 0 -forgive" {1 0} "compareInts" 0} \
      {"Ber framedepth 1 0" {1 0} "compareInts" 0} \
      {"Ber framedepth 100 0" {100 0} "compareInts" 0} \
      {"Ber framedepth 199 0" {199 0} "compareInts" 0} \
      {"Ber framedepth 200 0" {200 0} "compareInts" 0} \
      {"Ber framedepth 201 0 -forgive" {200 0} "compareInts" 0} \
      {"Ber framedepth 100 -1 -forgive" {100 0} "compareInts" 0} \
      {"Ber framedepth 100 0" {100 0} "compareInts" 0} \
      {"Ber framedepth 100 1" {100 1} "compareInts" 0} \
      {"Ber framedepth 100 2" {100 2} "compareInts" 0} \
      {"Ber framedepth 100 3" {100 3} "compareInts" 0} \
      {"Ber framedepth 100 4" {100 4} "compareInts" 0} \
      {"Ber framedepth 100 5" {100 5} "compareInts" 0} \
      {"Ber framedepth 100 6 -forgive" {100 5} "compareInts" 0} ]

    set testBerList2 [list \
      {"Ber en -1 -forgive" 3 "INT" 0} \
      {"Ber en 0" 1 "INT" 0} \
      {"Ber en 1" 1 "INT" 0} \
      {"Ber en 2" 3 "INT" 0} \
      {"Ber en 3" 3 "INT" 0} \
      {"Ber en 4 -forgive" 1 "INT" 0} \
      {"Ber enable -1 -forgive" 3 "INT" 0} \
      {"Ber enable 0" 1 "INT" 0} \
      {"Ber enable 1" 1 "INT" 0} \
      {"Ber enable 2" 3 "INT" 0} \
      {"Ber enable 3" 3 "INT" 0} \
      {"Ber enable 4 -forgive" 1 "INT" 0} ]

    set testBerList3 [list \
      {"Ber run -1 -forgive" 0 "INT" 0} \
      {"Ber run 1" 1 "INT" 0} \
      {"Ber run 0" 0 "INT" 0} \
      {"Ber run 4 -forgive" 1 "INT" 0} ]

    set testBerList4 [list \
      {"Ber ulim -1.0 -forgive" 0.0 "DBL" 0.001} \
      {"Ber ulim 0.001" 0.001 "DBL" 0.001} \
      {"Ber ulim 0.123" 0.123 "DBL" 0.001} \
      {"Ber ulim 0.456" 0.456 "DBL" 0.001} \
      {"Ber ulim 0.789" 0.789 "DBL" 0.001} \
      {"Ber ulim 0.999" 0.999 "DBL" 0.001} \
      {"Ber ulim 1.000" 1.000 "DBL" 0.001} \
      {"Ber ulim 1.001 -forgive" 1.000 "DBL" 0.001} \
      {"Ber ulimit -1.0 -forgive" 0.0 "DBL" 0.001} \
      {"Ber ulimit 0.001" 0.001 "DBL" 0.001} \
      {"Ber ulimit 0.123" 0.123 "DBL" 0.001} \
      {"Ber ulimit 0.456" 0.456 "DBL" 0.001} \
      {"Ber ulimit 0.789" 0.789 "DBL" 0.001} \
      {"Ber ulimit 0.999" 0.999 "DBL" 0.001} \
      {"Ber ulimit 1.000" 1.000 "DBL" 0.001} \
      {"Ber ulimit 1.001 -forgive" 1.000 "DBL" 0.001} ]

    set testBerList5 [list \
      {"Ber llim -1.0 -forgive" 0.0 "DBL" 0.001} \
      {"Ber llim 0.001" 0.001 "DBL" 0.001} \
      {"Ber llim 0.123" 0.123 "DBL" 0.001} \
      {"Ber llim 0.456" 0.456 "DBL" 0.001} \
      {"Ber llim 0.789" 0.789 "DBL" 0.001} \
      {"Ber llim 0.999" 0.999 "DBL" 0.001} \
      {"Ber llim 1.000" 1.000 "DBL" 0.001} \
      {"Ber llim 1.001 -forgive" 1.000 "DBL" 0.001} \
      {"Ber llimit -1.0 -forgive" 0.0 "DBL" 0.001} \
      {"Ber llimit 0.001" 0.001 "DBL" 0.001} \
      {"Ber llimit 0.123" 0.123 "DBL" 0.001} \
      {"Ber llimit 0.456" 0.456 "DBL" 0.001} \
      {"Ber llimit 0.789" 0.789 "DBL" 0.001} \
      {"Ber llimit 0.999" 0.999 "DBL" 0.001} \
      {"Ber llimit 1.000" 1.000 "DBL" 0.001} \
      {"Ber llimit 1.001 -forgive" 1.000 "DBL" 0.001} ]

    set testBerList6 [list \
      {"Ber al -1 -forgive" 0 "INT" 0} \
      {"Ber al 1" 1 "INT" 0} \
      {"Ber al 0" 0 "INT" 0} \
      {"Ber al 2" 2 "INT" 0} \
      {"Ber al 3" 3 "INT" 0} \
      {"Ber al 4 -forgive" 0 "INT" 0} \
      {"Ber alerts -1 -forgive" 0 "INT" 0} \
      {"Ber alerts 1" 1 "INT" 0} \
      {"Ber alerts 0" 0 "INT" 0} \
      {"Ber alerts 2" 2 "INT" 0} \
      {"Ber alerts 3" 3 "INT" 0} \
      {"Ber alerts 4 -forgive" 0 "INT" 0} ]

    set testBerList7 [list \
      {"Ber peak -1 -forgive" 0 "INT" 0} \
      {"Ber peak 1" 1 "INT" 0} \
      {"Ber peak 0" 0 "INT" 0} \
      {"Ber peak 1" 1 "INT" 0} \
      {"Ber peak 2 -forgive" 1 "INT" 0} \
      {"Ber peakhold -1 -forgive" 0 "INT" 0} \
      {"Ber peakhold 1" 1 "INT" 0} \
      {"Ber peakhold 0" 0 "INT" 0} \
      {"Ber peakhold 1" 1 "INT" 0} \
      {"Ber peakhold 2 -forgive" 1 "INT" 0} ]
 }

#****************************************************************************
# Name:        autoTest
# Parameters:  resultsLevel		Indicates the results display level
# Description: Executes automatic test on Ber commands
#****************************************************************************
 proc autoTest { resultsLevel } {
    variable testCaseList

    # Declare each test as a namespace variable
    foreach testName $testCaseList {
       variable $testName
    }

    setup 

    # Execute the test lists, and then destroy the test lists
    ::BIT::resetTestStats "cmdline_Ber"
    foreach testName $testCaseList {
       ::BIT::runCmdLineTests "cmdline_Ber" $testName
       unset $testName
    }

    if { ($resultsLevel == "Full") || ($resultsLevel == "Manual") } \
    {
       # Test commands with no data - requires visual inspection
       ::BIT::putsTest [Ber phrst]
       ::BIT::putsTest [Ber phreset]

       # Test range functions - requires visual inspection
        foreach cmd {alerts framedepth llimit peakhold phrst reseterrors run ulimit version} \
       {
          set temp [Ber $cmd -range]
          ::BIT::putsTest "The range of <$cmd> is:  $temp"
       }

       # Test string functions - requires visual inspection
       ::BIT::putsTest [Ber res]
       ::BIT::putsTest [Ber resources]
       ::BIT::putsTest [Ber ver 0]
       ::BIT::putsTest [Ber version 1]

       # Reset Ber engine to typical values after completion of the test
       Ber enable 3
       Ber run 0
       Ber frames 10 0
    }
 }

}

# Add this test to the list of tests
::BIT::addTest cmdline_Ber

