#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C) 2001        DESIGN ENGINEERING      #
# --------------------------------------------------------------------------#
# FILE        : test_P25.tcl                                                #
# PRODUCT     : FITS                                                        #
# AUTHOR      : Bruce Butterfield                                           #
# DESCRIPTION : Test basic P25 commands                                     #
#############################################################################

package require P25

#############################################################################
# FUNCTION    : setupP25Tests                                               #
# ARGUMENTS   : none                                                        #
# RETURNS     : none                                                        #
# PURPOSE     : Sets up a list of P25 tests                                 #
#############################################################################
proc setupP25Tests { } {
   global p25TestList 

   # Define the list of P25 tests
   set p25TestList [list \
           testP25List1 testP25List2 testP25List3 testP25List4 \
	   testP25List6 testP25List7 testP25List8 testP25List9 ]

   # Declare each test as a global variable
   foreach testName $p25TestList {
      global $testName
   }

   #
   # Define test cases
   #
   set testP25List1 [list \
      {"P25 decode en -1 -forgive" 3 "INT" 0} \
      {"P25 decode en 1" 1 "INT" 0} \
      {"P25 decode en 0" 1 "INT" 0} \
      {"P25 decode en on" 3 "INT" 0} \
      {"P25 decode en off" 1 "INT" 0} \
      {"P25 decode en 2 -forgive" 3 "INT" 0} \
      {"P25 decode en 3" 3 "INT" 0} \
      {"P25 decode enable -1 -forgive" 3 "INT" 0} \
      {"P25 decode enable 1" 1 "INT" 0} \
      {"P25 decode enable 0" 1 "INT" 0} \
      {"P25 decode enable on" 3 "INT" 0} \
      {"P25 decode enable off" 1 "INT" 0} \
      {"P25 decode enable 2 -forgive" 3 "INT" 0} \
      {"P25 decode enable 3" 3 "INT" 0} ]

   set testP25List2 [list \
      {"P25 decode run -1 -forgive" 0 "INT" 0} \
      {"P25 decode run 1" 1 "INT" 0} \
      {"P25 decode run 0" 0 "INT" 0} \
      {"P25 decode run on" 1 "INT" 0} \
      {"P25 decode run off" 0 "INT" 0} \
      {"P25 decode run 4 -forgive" 1 "INT" 0}  ]

   set testP25List3 [list \
      {"P25 decode rate -1 -forgive" 64 "INT" 0} \
      {"P25 decode rate 0 -forgive" 64 "INT" 0} \
      {"P25 decode rate 1" 1 "INT" 0} \
      {"P25 decode rate 2" 2 "INT" 0} \
      {"P25 decode rate 3 -forgive" 64 "INT" 0} \
      {"P25 decode rate 4" 4 "INT" 0} \
      {"P25 decode rate 5 -forgive" 64 "INT" 0} \
      {"P25 decode rate 7 -forgive" 64 "INT" 0} \
      {"P25 decode rate 8" 8 "INT" 0} \
      {"P25 decode rate 9 -forgive" 64 "INT" 0} \
      {"P25 decode rate 15 -forgive" 64 "INT" 0} \
      {"P25 decode rate 16" 16 "INT" 0} \
      {"P25 decode rate 17 -forgive" 64 "INT" 0} \
      {"P25 decode rate 31 -forgive" 64 "INT" 0} \
      {"P25 decode rate 32" 32 "INT" 0} \
      {"P25 decode rate 33 -forgive" 64 "INT" 0} \
      {"P25 decode rate 63 -forgive" 64 "INT" 0} \
      {"P25 decode rate 64" 64 "INT" 0} \
      {"P25 decode rate 65 -forgive" 64 "INT" 0} \
      {"P25 decode rate 127 -forgive" 64 "INT" 0} \
      {"P25 decode rate 128 -forgive" 64 "INT" 0} \
      {"P25 decode rate 129 -forgive" 64 "INT" 0} \
      {"P25 decode samplerate -1 -forgive" 64 "INT" 0} \
      {"P25 decode samplerate 0 -forgive" 64 "INT" 0} \
      {"P25 decode samplerate 1" 1 "INT" 0} \
      {"P25 decode samplerate 2" 2 "INT" 0} \
      {"P25 decode samplerate 3 -forgive" 64 "INT" 0} \
      {"P25 decode samplerate 4" 4 "INT" 0} \
      {"P25 decode samplerate 5 -forgive" 64 "INT" 0} \
      {"P25 decode samplerate 7 -forgive" 64 "INT" 0} \
      {"P25 decode samplerate 8" 8 "INT" 0} \
      {"P25 decode samplerate 9 -forgive" 64 "INT" 0} \
      {"P25 decode samplerate 15 -forgive" 64 "INT" 0} \
      {"P25 decode samplerate 16" 16 "INT" 0} \
      {"P25 decode samplerate 17 -forgive" 64 "INT" 0} \
      {"P25 decode samplerate 31 -forgive" 64 "INT" 0} \
      {"P25 decode samplerate 32" 32 "INT" 0} \
      {"P25 decode samplerate 33 -forgive" 64 "INT" 0} \
      {"P25 decode samplerate 63 -forgive" 64 "INT" 0} \
      {"P25 decode samplerate 64" 64 "INT" 0} \
      {"P25 decode samplerate 65 -forgive" 64 "INT" 0} \
      {"P25 decode samplerate 127 -forgive" 64 "INT" 0} \
      {"P25 decode samplerate 128 -forgive" 64 "INT" 0} \
      {"P25 decode samplerate 129 -forgive" 64 "INT" 0} ]


   set testP25List4 [list \
      {"P25 encode encrypt -1 -forgive" 0 "INT" 0} \
      {"P25 encode encrypt 1" 1 "INT" 0} \
      {"P25 encode encrypt 0" 0 "INT" 0} \
      {"P25 encode encrypt on" 1 "INT" 0} \
      {"P25 encode encrypt off" 0 "INT" 0} \
      {"P25 encode encrypt 2 -forgive" 1 "INT" 0} \
      {"P25 encode encryptenable -1 -forgive" 0 "INT" 0} \
      {"P25 encode encryptenable 1" 1 "INT" 0} \
      {"P25 encode encryptenable 0" 0 "INT" 0} \
      {"P25 encode encryptenable on" 1 "INT" 0} \
      {"P25 encode encryptenable off" 0 "INT" 0} \
      {"P25 encode encryptenable 2 -forgive" 1 "INT" 0} ]

   set testP25List5 [list \
      {"P25 encode sigtype -1 -forgive" 0 "INT" 0} \
      {"P25 encode sigtype 1" 1 "INT" 0} \
      {"P25 encode sigtype 0" 0 "INT" 0} \
      {"P25 encode sigtype 2" 2 "INT" 0} \
      {"P25 encode sigtype 3" 3 "INT" 0} \
      {"P25 encode sigtype 4" 4 "INT" 0} \
      {"P25 encode sigtype 5" 5 "INT" 0} \
      {"P25 encode sigtype off" 0 "INT" 0} \
      {"P25 encode sigtype audio" 1 "INT" 0} \
      {"P25 encode sigtype silence" 2 "INT" 0} \
      {"P25 encode sigtype error" 3 "INT" 0} \
      {"P25 encode sigtype sine" 4 "INT" 0} \
      {"P25 encode sigtype file" 5 "INT" 0} \
      {"P25 encode sigtype 6 -forgive" 5 "INT" 0} \
      {"P25 encode signaltype -1 -forgive" 0 "INT" 0} \
      {"P25 encode signaltype 1" 1 "INT" 0} \
      {"P25 encode signaltype 0" 0 "INT" 0} \
      {"P25 encode signaltype 2" 2 "INT" 0} \
      {"P25 encode signaltype 3" 3 "INT" 0} \
      {"P25 encode signaltype 4" 4 "INT" 0} \
      {"P25 encode signaltype 5" 5 "INT" 0} \
      {"P25 encode signaltype 6 -forgive" 5 "INT" 0} ]

   set testP25List6 [list \
      {"P25 encode c4fm -1 -forgive" 0.0 "DBL" 0.001} \
      {"P25 encode c4fm 0.0" 0.0 "DBL" 0.001} \
      {"P25 encode c4fm 14.987" 14.987 "DBL" 0.001} \
      {"P25 encode c4fm 99.999" 99.999 "DBL" 0.001} \
      {"P25 encode c4fm 100.0" 100.0 "DBL" 0.001} \
      {"P25 encode c4fm 100.1234 -forgive" 100.0 "DBL" 0.001} \
      {"P25 encode c4fmpercent -1 -forgive" 0.0 "DBL" 0.001} \
      {"P25 encode c4fmpercent 0.0" 0.0 "DBL" 0.001} \
      {"P25 encode c4fmpercent 14.987" 14.987 "DBL" 0.001} \
      {"P25 encode c4fmpercent 99.999" 99.999 "DBL" 0.001} \
      {"P25 encode c4fmpercent 100.0" 100.0 "DBL" 0.001} \
      {"P25 encode c4fmpercent 100.1234 -forgive" 100.0 "DBL" 0.001} ]

   set testP25List7 [list \
      {"P25 encode nac -1 -forgive" 0 "INT" 0} \
      {"P25 encode nac 0" 0 "INT" 0} \
      {"P25 encode nac 100" 100 "INT" 0} \
      {"P25 encode nac 1000" 1000 "INT" 0} \
      {"P25 encode nac 4095" 4095 "INT" 0} \
      {"P25 encode nac 4096 -forgive" 4095 "INT" 0} \
      {"P25 encode mfid -1 -forgive" 0 "INT" 0} \
      {"P25 encode mfid 0" 0 "INT" 0} \
      {"P25 encode mfid 100" 100 "INT" 0} \
      {"P25 encode mfid 255" 255 "INT" 0} \
      {"P25 encode mfid 256 -forgive" 255 "INT" 0} \
      {"P25 encode tgid -1 -forgive" 0 "INT" 0} \
      {"P25 encode tgid 0" 0 "INT" 0} \
      {"P25 encode tgid 100" 100 "INT" 0} \
      {"P25 encode tgid 65535" 65535 "INT" 0} \
      {"P25 encode tgid 65536 -forgive" 65535 "INT" 0} \
      {"P25 encode lsda -1 -forgive" 0 "INT" 0} \
      {"P25 encode lsda 0" 0 "INT" 0} \
      {"P25 encode lsda 100" 100 "INT" 0} \
      {"P25 encode lsda 1000" 1000 "INT" 0} \
      {"P25 encode lsda 10000" 10000 "INT" 0} \
      {"P25 encode lsda 65535" 65535 "INT" 0} \
      {"P25 encode lsda 65536 -forgive" 65535 "INT" 0} \
      {"P25 encode lsdb -1 -forgive" 0 "INT" 0} \
      {"P25 encode lsdb 0" 0 "INT" 0} \
      {"P25 encode lsdb 100" 100 "INT" 0} \
      {"P25 encode lsdb 1000" 1000 "INT" 0} \
      {"P25 encode lsdb 10000" 10000 "INT" 0} \
      {"P25 encode lsdb 65535" 65535 "INT" 0} \
      {"P25 encode lsdb 65536 -forgive" 65535 "INT" 0} \
      {"P25 encode lco -1 -forgive" 0 "INT" 0} \
      {"P25 encode lco 0" 0 "INT" 0} \
      {"P25 encode lco 63" 63 "INT" 0} \
      {"P25 encode lco 64 -forgive" 63 "INT" 0} \
      {"P25 encode did -1 -forgive" 0 "INT" 0} \
      {"P25 encode did 0" 0 "INT" 0} \
      {"P25 encode did 100" 100 "INT" 0} \
      {"P25 encode did 1000" 1000 "INT" 0} \
      {"P25 encode did 10000" 10000 "INT" 0} \
      {"P25 encode did 100000" 100000 "INT" 0} \
      {"P25 encode did 16777215" 16777215 "INT" 0} \
      {"P25 encode did 16777216 -forgive" 16777215 "INT" 0} \
      {"P25 encode sid -1 -forgive" 0 "INT" 0} \
      {"P25 encode sid 0" 0 "INT" 0} \
      {"P25 encode sid 100" 100 "INT" 0} \
      {"P25 encode sid 1000" 1000 "INT" 0} \
      {"P25 encode sid 10000" 10000 "INT" 0} \
      {"P25 encode sid 100000" 100000 "INT" 0} \
      {"P25 encode sid 16777215" 16777215 "INT" 0} \
      {"P25 encode sid 16777216 -forgive" 16777215 "INT" 0} \
      {"P25 encode emg -1 -forgive" 0 "INT" 0} \
      {"P25 encode emg 0" 0 "INT" 0} \
      {"P25 encode emg 1" 1 "INT" 0} \
      {"P25 encode emg 2 -forgive" 1 "INT" 0} \
      {"P25 encode sf -1 -forgive" 0 "INT" 0} \
      {"P25 encode sf 0" 0 "INT" 0} \
      {"P25 encode sf 1" 1 "INT" 0} \
      {"P25 encode sf 2 -forgive" 1 "INT" 0} \
      {"P25 encode p -1 -forgive" 0 "INT" 0} \
      {"P25 encode p 0" 0 "INT" 0} \
      {"P25 encode p 1" 1 "INT" 0} \
      {"P25 encode p 2 -forgive" 1 "INT" 0} \
      {"P25 encode kid -1 -forgive" 0 "INT" 0} \
      {"P25 encode kid 0" 0 "INT" 0} \
      {"P25 encode kid 100" 100 "INT" 0} \
      {"P25 encode kid 1000" 1000 "INT" 0} \
      {"P25 encode kid 65535" 65535 "INT" 0} \
      {"P25 encode kid 65536 -forgive" 65535 "INT" 0} \
      {"P25 encode status -1 -forgive" 0 "INT" 0} \
      {"P25 encode status 0" 0 "INT" 0} \
      {"P25 encode status 1" 1 "INT" 0} \
      {"P25 encode status 2" 2 "INT" 0} \
      {"P25 encode status 3" 3 "INT" 0} \
      {"P25 encode status 4 -forgive" 3 "INT" 0} \
      {"P25 encode st -1 -forgive" 0 "INT" 0} \
      {"P25 encode st 0" 0 "INT" 0} \
      {"P25 encode st 1" 1 "INT" 0} \
      {"P25 encode st 2" 2 "INT" 0} \
      {"P25 encode st 3" 3 "INT" 0} \
      {"P25 encode st 4 -forgive" 3 "INT" 0} ]




   #	       Min      Max		Default
   #  NAC:  	0	0xfff		0x239		Network Access Code (12 bits)
   #  MFID:	0	0xff		0		Manufacturers ID (8 bits)
   #  TGID:	0	0xffff		1		Talk Group ID (16 bits)
   #  LSDa:	0	0xffff		0x4040		LSD Info (MSW) (16 bits)
   #  LSDb:	0	0xffff		0x4142		LSD Info (LSW) (16 bits)
   #  LCO:	0	0x3f		0		Link Control Opcode (6 bits)
   #  DID:	0	0xffffff	0		Destination ID (24 bits)
   #  SID:	0	0xffffff	0		Source ID (24 bits)
   #  EMG:	0	1		0		Emergency bit (1 bit)
   #  SF	0	1		0		Standard Format bit (1 bit)
   #  P		0	1		0		Priority bit (1 bit)
   #  KID:	0	0xffff		0		Key ID (16 bits)
   #  STATUS:	0	3		0		Status symbol (2 bits)
   #
   #
   set testP25List8 [list \
      {"P25 encode nac 0x239" 569 "INT" 0} \
      {"P25 encode mfid 0" 0 "INT" 0} \
      {"P25 encode tgid 1" 1 "INT" 0} \
      {"P25 encode lsda 0x4040" 16448 "INT" 0} \
      {"P25 encode lsdb 0x4142" 16706 "INT" 0} \
      {"P25 encode lco 0" 0 "INT" 0} \
      {"P25 encode did 0" 0 "INT" 0} \
      {"P25 encode sid 0" 0 "INT" 0} \
      {"P25 encode emg 0" 0 "INT" 0} \
      {"P25 encode sf 0" 0 "INT" 0} \
      {"P25 encode p 0" 0 "INT" 0} \
      {"P25 encode kid 0" 0 "INT" 0} \
      {"P25 encode status 0" 0 "INT" 0} \
      {"P25 encode sync" "569 0 1 16448 16706 0 0 0 0 0 0 0 0" "STR" 0} \
      {"P25 encode nac 0x123" 291 "INT" 0} \
      {"P25 encode sync" "291 0 1 16448 16706 0 0 0 0 0 0 0 0" "STR" 0} \
      {"P25 encode mfid 0xfe" 254 "INT" 0} \
      {"P25 encode sync" "291 254 1 16448 16706 0 0 0 0 0 0 0 0" "STR" 0} \
      {"P25 encode tgid 0x76" 118 "INT" 0} \
      {"P25 encode sync" "291 254 118 16448 16706 0 0 0 0 0 0 0 0" "STR" 0} \
      {"P25 encode lsda 0x1234" 4660 "INT" 0} \
      {"P25 encode sync" "291 254 118 4660 16706 0 0 0 0 0 0 0 0" "STR" 0} \
      {"P25 encode lsdb 0x4321" 17185 "INT" 0} \
      {"P25 encode sync" "291 254 118 4660 17185 0 0 0 0 0 0 0 0" "STR" 0} \
      {"P25 encode lco 0x3f" 63 "INT" 0} \
      {"P25 encode sync" "291 254 118 4660 17185 63 0 0 0 0 0 0 0" "STR" 0} \
      {"P25 encode did 0x987654" 9991764 "INT" 0} \
      {"P25 encode sync" "291 254 118 4660 17185 63 9991764 0 0 0 0 0 0" "STR" 0} \
      {"P25 encode sid 0x123456" 1193046 "INT" 0} \
      {"P25 encode sync" "291 254 118 4660 17185 63 9991764 1193046 0 0 0 0 0" "STR" 0} \
      {"P25 encode emg 1" 1 "INT" 0} \
      {"P25 encode sync" "291 254 118 4660 17185 63 9991764 1193046 1 0 0 0 0" "STR" 0} \
      {"P25 encode sf 1" 1 "INT" 0} \
      {"P25 encode sync" "291 254 118 4660 17185 63 9991764 1193046 1 1 0 0 0" "STR" 0} \
      {"P25 encode p 1" 1 "INT" 0} \
      {"P25 encode sync" "291 254 118 4660 17185 63 9991764 1193046 1 1 1 0 0" "STR" 0} \
      {"P25 encode kid 0x5a5a" 23130 "INT" 0} \
      {"P25 encode sync" "291 254 118 4660 17185 63 9991764 1193046 1 1 1 23130 0" "STR" 0} \
      {"P25 encode status 3" 3 "INT" 0} \
      {"P25 encode sync" "291 254 118 4660 17185 63 9991764 1193046 1 1 1 23130 3" "STR" 0} \
      ]

   set testP25List9 [list \
      {"P25 encode array 0 0 0 0 0 0 0 0 0 0 0 0 0" "0 0 0 0 0 0 0 0 0 0 0 0 0" "STR" 0} \
      {"P25 encode array -1 0 0 0 0 0 0 0 0 0 0 0 0 -forgive" "0 0 0 0 0 0 0 0 0 0 0 0 0" "STR" 0} \
      {"P25 encode array 0 -1 0 0 0 0 0 0 0 0 0 0 0 -forgive" "0 0 0 0 0 0 0 0 0 0 0 0 0" "STR" 0} \
      {"P25 encode array 0 0 -1 0 0 0 0 0 0 0 0 0 0 -forgive" "0 0 0 0 0 0 0 0 0 0 0 0 0" "STR" 0} \
      {"P25 encode array 0 0 0 -1 0 0 0 0 0 0 0 0 0 -forgive" "0 0 0 0 0 0 0 0 0 0 0 0 0" "STR" 0} \
      {"P25 encode array 0 0 0 0 -1 0 0 0 0 0 0 0 0 -forgive" "0 0 0 0 0 0 0 0 0 0 0 0 0" "STR" 0} \
      {"P25 encode array 0 0 0 0 0 -1 0 0 0 0 0 0 0 -forgive" "0 0 0 0 0 0 0 0 0 0 0 0 0" "STR" 0} \
      {"P25 encode array 0 0 0 0 0 0 -1 0 0 0 0 0 0 -forgive" "0 0 0 0 0 0 0 0 0 0 0 0 0" "STR" 0} \
      {"P25 encode array 0 0 0 0 0 0 0 -1 0 0 0 0 0 -forgive" "0 0 0 0 0 0 0 0 0 0 0 0 0" "STR" 0} \
      {"P25 encode array 0 0 0 0 0 0 0 0 -1 0 0 0 0 -forgive" "0 0 0 0 0 0 0 0 0 0 0 0 0" "STR" 0} \
      {"P25 encode array 0 0 0 0 0 0 0 0 0 -1 0 0 0 -forgive" "0 0 0 0 0 0 0 0 0 0 0 0 0" "STR" 0} \
      {"P25 encode array 0 0 0 0 0 0 0 0 0 0 -1 0 0 -forgive" "0 0 0 0 0 0 0 0 0 0 0 0 0" "STR" 0} \
      {"P25 encode array 0 0 0 0 0 0 0 0 0 0 0 -1 0 -forgive" "0 0 0 0 0 0 0 0 0 0 0 0 0" "STR" 0} \
      {"P25 encode array 0 0 0 0 0 0 0 0 0 0 0 0 -1 -forgive" "0 0 0 0 0 0 0 0 0 0 0 0 0" "STR" 0} \
      {"P25 encode array 4096 0 0 0 0 0 0 0 0 0 0 0 0 -forgive" "4095 0 0 0 0 0 0 0 0 0 0 0 0" "STR" 0} \
      {"P25 encode array 0 256 0 0 0 0 0 0 0 0 0 0 0 -forgive" "0 255 0 0 0 0 0 0 0 0 0 0 0" "STR" 0} \
      {"P25 encode array 0 0 65536 0 0 0 0 0 0 0 0 0 0 -forgive" "0 0 65535 0 0 0 0 0 0 0 0 0 0" "STR" 0} \
      {"P25 encode array 0 0 0 65536 0 0 0 0 0 0 0 0 0 -forgive" "0 0 0 65535 0 0 0 0 0 0 0 0 0" "STR" 0} \
      {"P25 encode array 0 0 0 0 65536 0 0 0 0 0 0 0 0 -forgive" "0 0 0 0 65535 0 0 0 0 0 0 0 0" "STR" 0} \
      {"P25 encode array 0 0 0 0 0 64 0 0 0 0 0 0 0 -forgive" "0 0 0 0 0 63 0 0 0 0 0 0 0" "STR" 0} \
      {"P25 encode array 0 0 0 0 0 0 16777216 0 0 0 0 0 0 -forgive" "0 0 0 0 0 0 16777215 0 0 0 0 0 0" "STR" 0} \
      {"P25 encode array 0 0 0 0 0 0 0 16777216 0 0 0 0 0 -forgive" "0 0 0 0 0 0 0 16777215 0 0 0 0 0" "STR" 0} \
      {"P25 encode array 0 0 0 0 0 0 0 0 2 0 0 0 0 -forgive" "0 0 0 0 0 0 0 0 1 0 0 0 0" "STR" 0} \
      {"P25 encode array 0 0 0 0 0 0 0 0 0 2 0 0 0 -forgive" "0 0 0 0 0 0 0 0 0 1 0 0 0" "STR" 0} \
      {"P25 encode array 0 0 0 0 0 0 0 0 0 0 2 0 0 -forgive" "0 0 0 0 0 0 0 0 0 0 1 0 0" "STR" 0} \
      {"P25 encode array 0 0 0 0 0 0 0 0 0 0 0 65536 0 -forgive" "0 0 0 0 0 0 0 0 0 0 0 65535 0" "STR" 0} \
      {"P25 encode array 0 0 0 0 0 0 0 0 0 0 0 0 4 -forgive" "0 0 0 0 0 0 0 0 0 0 0 0 3" "STR" 0} \
      ]

}

#############################################################################
# FUNCTION    : autoTestP25                                                 #
# ARGUMENTS   : none                                                        #
# RETURNS     : none                                                        #
# PURPOSE     : Executes automatic test on P25 commands                     #
#############################################################################
proc autoTestP25 { } {
   global p25TestList 

   setupP25Tests 

   # Execute the test lists, and then destroy the test lists
   foreach testName $p25TestList {
      testCompare $testName 
      global $testName
      unset $testName
   }
   unset p25TestList

   # Test commands with no data - requires visual inspection
   P25 decode encrypt
   P25 decode encryption
#   P25 decode getreport
#   P25 decode rep

#   P25 encode info
#   P25 encode information


   # Test string functions - requires visual inspection

   # Reset P25 engines to typical values after completion of the test
}


