#############################################################################
# COPYRIGHT (C) 2001, 2005  Aeroflex, Inc.                                  #
# --------------------------------------------------------------------------#
# FILE        : 1014.test_Power.tcl                                         #
# PRODUCT     : FITS                                                        #
# AUTHOR      : Bruce Butterfield, Jon Butterfield                          #
# DESCRIPTION : Remote Command Line Test: Power                             #
# --------------------------------------------------------------------------#
# Update      : Converted for use with BIT, engine cleanup for 1.9.1.4      #
#############################################################################

package require Power
package require BIT

#****************************************************************************
# Test variables
#****************************************************************************
namespace eval ::cmdline_Power {
    variable testCaseList		;# List of test cases
    variable shortDesc			;# Test description - menu size
    variable longDesc			;# Test description - used by reports
    variable testTime			;# Test duration (Seconds - approximation)

    # Define the list of Power tests
    set testCaseList [list \
           testPowerList1 testPowerList2 testPowerList3 testPowerList4 \
           testPowerList5 testPowerList6 testPowerList7 testPowerList8 \
           testPowerList9 testPowerList10 testPowerList11]

    # Declare each test as a namespace variable
    foreach testName $testCaseList {
       variable $testName
    }

    set shortDesc "Power cmds"
    set longDesc "Powerterm command line test"
    set testTime 3.0

#****************************************************************************
# Name:        setup
# Parameters:  none
# Description: Sets up a list of Power tests
#****************************************************************************
 proc setup { } {
    variable testCaseList

    # Declare each test as a namespace variable
    foreach testName $testCaseList {
       variable $testName
    }

   #
   # Define test cases
   #

# Test power alerts 
   set testPowerList1 [list                   \
      {"Power al  0" 0 "INT" 0}               \
      {"Power al  1" 1 "INT" 0}               \
      {"Power al  2" 2 "INT" 0}               \
      {"Power al  3" 3 "INT" 0}               \
      {"Power al  7" 7 "INT" 0}               \
      {"Power al -1 -forgive" 0 "INT" 0}      \
      {"Power al  8 -forgive" 0 "INT" 0}      \
      {"Power al off"        0 "INT" 0}       \
      {"Power al enablelo"  1 "INT" 0}        \
      {"Power al enablehi" 2 "INT" 0}         \
      {"Power al enableboth" 3 "INT" 0}       \
      {"Power al Off"        0 "INT" 0}       \
      {"Power al EnableLo"  1 "INT" 0}        \
      {"Power al EnableHi" 2 "INT" 0}         \
      {"Power al EnableBoth" 3 "INT" 0}       \
      {"Power alerts  0" 0 "INT" 0}           \
      {"Power alerts  1" 1 "INT" 0}           \
      {"Power alerts  2" 2 "INT" 0}           \
      {"Power alerts  3" 3 "INT" 0}           \
      {"Power alerts  7" 7 "INT" 0}           \
      {"Power alerts -1 -forgive" 0 "INT" 0}  \
      {"Power alerts  8 -forgive" 0 "INT" 0}  \
      {"Power alerts off"        0 "INT" 0}   \
      {"Power alerts enablelo"  1 "INT" 0}    \
      {"Power alerts enablehi" 2 "INT" 0}     \
      {"Power alerts enableboth" 3 "INT" 0}   \
      {"Power alerts Off"        0 "INT" 0}   \
      {"Power alerts EnableLo"  1 "INT" 0}    \
      {"Power alerts EnableHi" 2 "INT" 0}     \
      {"Power alerts EnableBoth" 3 "INT" 0}]

# Test cable loss
  set testPowerList2 [list                              \
      {"Power cable   0.0" 0.0 "DBL" 0.001}             \
      {"Power cable -60.0" -60.0 "DBL" 0.001}           \
      {"Power cable -60.01 -forgive" -60.0 "DBL" 0.001} \
      {"Power cable 60.0" 60.0 "DBL" 0.001}             \
      {"Power cable 60.0 -forgive" 60.0 "DBL" 0.001}    \
      {"Power cable 60.01 -forgive" 60.0 "DBL" 0.001}   \
      {"Power cableloss   0.0" 0.0 "DBL" 0.001}         \
      {"Power cableloss -60.0" -60.0 "DBL" 0.001}       \
      {"Power cableloss 60.0" 60.0 "DBL" 0.001}]

#Test detector
   set testPowerList3 [list                    \
      {"Power dect -1 -forgive" 0 "INT" 0}     \
      {"Power dect 0" 0 "INT" 0}               \
      {"Power dect 1" 1 "INT" 0}               \
      {"Power dect fast" 0 "INT" 0}            \
      {"Power dect slow" 1 "INT" 0}            \
      {"Power dect 2 -forgive" 1 "INT" 0}      \
      {"Power detector -1 -forgive" 0 "INT" 0} \
      {"Power detector 0" 0 "INT" 0}           \
      {"Power detector 1" 1 "INT" 0}           \
      {"Power detector fast" 0 "INT" 0}        \
      {"Power detector slow" 1 "INT" 0}        \
      {"Power detector 2 -forgive" 1 "INT" 0}]

# Test Llimit
  set testPowerList4 [list                                 \
      {"Power llim  0.0"      0.0   "DBL" 0.001}           \
      {"Power llim  12.823"  12.823 "DBL" 0.001}           \
      {"Power llim  72.23"   72.23  "DBL" 0.001}           \
      {"Power llim  144.99" 144.99  "DBL" 0.001}           \
      {"Power llim  -12.2  -forgive" 0.0   "DBL" 0.001}    \
      {"Power llim  151.99 -forgive" 150.00 "DBL" 0.001}   \
      {"Power llimit  0.0"       0.0   "DBL" 0.001}        \
      {"Power llimit  15.98"   15.98   "DBL" 0.001}        \
      {"Power llimit  51.29"   51.29   "DBL" 0.001}        \
      {"Power llimit  147.28" 147.28   "DBL" 0.001}        \
      {"Power llimit  -10.8  -forgive" 0.0   "DBL" 0.001}  \
      {"Power llimit  152.50 -forgive" 150.00 "DBL" 0.001}]

# Test Message enable
  set testPowerList5 [list                        \
      {"Power enable on" "On" "STR" 0}            \
      {"Power enable off" "Off" "STR" 0}          \
      {"Power messageenable on" "On" "STR" 0}     \
      {"Power messageenable off" "Off" "STR" 0}]

# Test Peakhold
  set testPowerList6 [list                     \
      {"Power peak  0" 0 "INT" 0}              \
      {"Power peak  1" 1 "INT" 0}              \
      {"Power peak -1 -forgive" 0 "INT" 0}     \
      {"Power peak  2 -forgive" 1 "INT" 0}     \
      {"Power peak off" 0 "INT" 0}             \
      {"Power peak on"  1 "INT" 0}             \
      {"Power peak Off" 0 "INT" 0}             \
      {"Power peak On"  1 "INT" 0}             \
      {"Power peakhold  0" 0 "INT" 0}          \
      {"Power peakhold  1" 1 "INT" 0}          \
      {"Power peakhold -1 -forgive" 0 "INT" 0} \
      {"Power peakhold  2 -forgive" 1 "INT" 0} \
      {"Power peakhold off" 0 "INT" 0}         \
      {"Power peakhold on"  1 "INT" 0}         \
      {"Power peakhold Off" 0 "INT" 0}         \
      {"Power peakhold On"  1 "INT" 0}]

# Test power reading range
   set testPowerList7 [list                       \
      {"Power prange  0" 0 "INT" 0}               \
      {"Power prange  1" 1 "INT" 0}               \
      {"Power prange  2" 2 "INT" 0}               \
      {"Power prange -1 -forgive" 0 "INT" 0}      \
      {"Power prange  3 -forgive" 2 "INT" 0}      \
      {"Power prange auto" 0 "INT" 0}             \
      {"Power prange low"  1 "INT" 0}             \
      {"Power prange high" 2 "INT" 0}             \
      {"Power prange AUTO" 0 "INT" 0}             \
      {"Power prange LOW"  1 "INT" 0}             \
      {"Power prange HIGH" 2 "INT" 0}             \
      {"Power powerrange  0" 0 "INT" 0}           \
      {"Power powerrange  1" 1 "INT" 0}           \
      {"Power powerrange  2" 2 "INT" 0}           \
      {"Power powerrange -1 -forgive" 0 "INT" 0}  \
      {"Power powerrange  3 -forgive" 2 "INT" 0}  \
      {"Power powerrange AUTO" 0 "INT" 0}         \
      {"Power powerrange LOW"  1 "INT" 0}         \
      {"Power powerrange HIGH" 2 "INT" 0}]

# Test raw enable
  set testPowerList8 [list                         \
      {"Power rawen 0" 1 "INT" 0}                  \
      {"Power rawen 1" 1 "INT" 0}                  \
      {"Power rawen 2" 3 "INT" 0}                  \
      {"Power rawen 4 -forgive" 1 "INT" 0}         \
      {"Power rawen 11" 11 "INT" 0}                \
      {"Power rawen 27" 27 "INT" 0}                \
      {"Power rawen 59" 59 "INT" 0}                \
      {"Power rawen 123" 123 "INT" 0}              \
      {"Power rawen 251" 251 "INT" 0}              \
      {"Power rawenable 0" 1 "INT" 0}              \
      {"Power rawenable -15 -forgive" 241 "INT" 0} \
      {"Power rawenable 0xfb" 251 "INT" 0}]

# Test read rate
  set testPowerList9 [list                            \
      {"Power rate 0" 0 "INT" 0}                      \
      {"Power rate -1 -forgive" 0 "INT" 0}            \
      {"Power rate 200" 200 "INT" 0}                  \
      {"Power rate 10000" 10000 "INT" 0}              \
      {"Power rate 10001 -forgive" 10000 "INT" 0}     \
      {"Power readrate 0" 0 "INT" 0}                  \
      {"Power readrate -1 -forgive" 0 "INT" 0}        \
      {"Power readrate 200" 200 "INT" 0}              \
      {"Power readrate 10000" 10000 "INT" 0}          \
      {"Power readrate 10001 -forgive" 10000 "INT" 0}]

# Test sample rate
  set testPowerList10 [list                             \
      {"Power srate 1200 -forgive" 2500 "INT" 0}        \
      {"Power srate 2500" 2500 "INT" 0}                 \
      {"Power srate 4000" 4000 "INT" 0}                 \
      {"Power srate 10000" 10000 "INT" 0}               \
      {"Power srate 10001 -forgive" 10000 "INT" 0}      \
      {"Power samplerate 1200 -forgive" 2500 "INT" 0}   \
      {"Power samplerate 2500" 2500 "INT" 0}            \
      {"Power samplerate 4000" 4000 "INT" 0}            \
      {"Power samplerate 10000" 10000 "INT" 0}          \
      {"Power samplerate 10001 -forgive" 10000 "INT" 0}]

# Test Ulimit
  set testPowerList11 [list                                \
      {"Power ulim  0.0"      0.0   "DBL" 0.001}           \
      {"Power ulim  12.823"  12.823 "DBL" 0.001}           \
      {"Power ulim  72.23"   72.23  "DBL" 0.001}           \
      {"Power ulim  144.99" 144.99  "DBL" 0.001}           \
      {"Power ulim  -12.2  -forgive" 0.0   "DBL" 0.001}    \
      {"Power ulim  151.99 -forgive" 150.00 "DBL" 0.001}   \
      {"Power ulimit  0.0"       0.0   "DBL" 0.001}        \
      {"Power ulimit  15.98"   15.98   "DBL" 0.001}        \
      {"Power ulimit  51.29"   51.29   "DBL" 0.001}        \
      {"Power ulimit  147.28" 147.28   "DBL" 0.001}        \
      {"Power ulimit  -10.8  -forgive" 0.0   "DBL" 0.001}  \
      {"Power ulimit  152.50 -forgive" 150.00 "DBL" 0.001}]

}


#****************************************************************************
# Name:        autoTest
# Parameters:  resultsLevel		Indicates the results display level
# Description: Executes automatic test on Power commands
#****************************************************************************
 proc autoTest { resultsLevel } {
    variable testCaseList

    # Declare each test as a namespace variable
    foreach testName $testCaseList {
       variable $testName
    }

    setup 

    # Execute the test lists, and then destroy the test lists
    ::BIT::resetTestStats "cmdline_Power"
    foreach testName $testCaseList {
       ::BIT::runCmdLineTests "cmdline_Power" $testName
       unset $testName
    }

    if { ($resultsLevel == "Full") || ($resultsLevel == "Manual") } \
    {
       # Test commands with no data - requires visual inspection
       ::BIT::putsTest [Power barread]
       ::BIT::putsTest [Power barreading]
       ::BIT::putsTest [Power phrst]
       ::BIT::putsTest [Power phreset]
       ::BIT::putsTest [Power read]
       ::BIT::putsTest [Power reading]
       ::BIT::putsTest [Power readpeak]
       ::BIT::putsTest [Power readpeakhold]
       ::BIT::putsTest [Power temp]
       ::BIT::putsTest [Power temperature]
       ::BIT::putsTest [Power val]
       ::BIT::putsTest [Power value]

       # Test string functions - requires visual inspection
        foreach cmd {al cable dect llim peak prange rate srate ulim ver} \
       {
          set temp [Power $cmd -range]
          ::BIT::putsTest "The range of <$cmd> is:  $temp"
       }

       ::BIT::putsTest [Power res]
       ::BIT::putsTest [Power resources]
       ::BIT::putsTest [Power version]
       ::BIT::putsTest [Power ver 0]
       ::BIT::putsTest [Power ver 1]
       ::BIT::putsTest [Power ver 2]
    }

    # Reset Power engines to typical values after completion of the test
    Power enable on
    Power dect 0
    Power prange 0
    Power zero
 }

}

# Add this test to the list of tests
::BIT::addTest cmdline_Power

