#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C) 2001, 2003  DESIGN ENGINEERING      #
# --------------------------------------------------------------------------#
# FILE        : 1016.test_RLmtr.tcl                                         #
# PRODUCT     : FITS                                                        #
# AUTHOR      : Nghia Tran, Bruce Butterfield                               #
# DESCRIPTION : Remote Command Line Test:  Receiver Level meter             #
#############################################################################

package require RLmeter
package require BIT

#****************************************************************************
# Test variables
#****************************************************************************
namespace eval ::cmdline_RLmeter {
    variable testCaseList		;# List of test cases
    variable shortDesc			;# Test description - menu size
    variable longDesc			;# Test description - used by reports
    variable testTime			;# Test duration (Seconds - approximation)

    # Define the list of Digital Coded Squelch tests
    set testCaseList [list \
            testRLmtrList1 testRLmtrList2 testRLmtrList3 testRLmtrList4 \
            testRLmtrList7 testRLmtrList8 \
            testRLmtrList9 testRLmtrList10]


    # Declare each test as a namespace variable
    foreach testName $testCaseList {
       variable $testName
    }

    set shortDesc "RLmeter cmds"
    set longDesc "Receiver Level meter command line test"
    set testTime 0.50

#****************************************************************************
# Name:        setup
# Parameters:  none
# Description: Sets up a list of Digital Coded Squelch tests
#****************************************************************************
 proc setup { } {
    variable testCaseList

    # Declare each test as a namespace variable
    foreach testName $testCaseList {
       variable $testName
    }

    #
    # Define test cases
    #
    set testRLmtrList1 [list \
      {"RLmeter rat 0" 0 "INT" 0} \
      {"RLmeter rat 100" 100 "INT" 0} \
      {"RLmeter rat 999" 999 "INT" 0} \
      {"RLmeter rat 1001 -forgive" 1000 "INT" 0} \
      {"RLmeter rat -1 -forgive" 0 "INT" 0} \
      {"RLmeter rate 1" 1 "INT" 0} \
      {"RLmeter rate 121" 121 "INT" 0} \
      {"RLmeter rate 980" 980 "INT" 0} \
      {"RLmeter rate 1020 -forgive" 1000 "INT" 0} ]

    set testRLmtrList2 [list \
      {"RLmeter ref 20.345" 20.345 "DBL" 0.001} \
      {"RLmeter ref 2500.876" 2500.876 "DBL" 0.001} \
      {"RLmeter ref -20.00 -forgive" 1.00 "DBL" 0.001} \
      {"RLmeter ref 2900.123 -forgive" 2700.00 "DBL" 0.001} \
      {"RLmeter reference 38.345" 38.345 "DBL" 0.001} \
      {"RLmeter reference 2500.876" 2500.876 "DBL" 0.001} \
      {"RLmeter reference -40.123 -forgive" 1.00 "DBL" 0.001} \
      {"RLmeter reference 2800.123 -forgive" 2700.00 "DBL" 0.001}]
      
    set testRLmtrList3 [list \
      {"RLmeter cable -12.345" -12.345 "DBL" 0.001} \
      {"RLmeter cable  24.125"  24.125 "DBL" 0.001} \
      {"RLmeter cable -20.275" -20.275 "DBL" 0.001} \
      {"RLmeter cable 57.465"   57.465 "DBL" 0.001} \
      {"RLmeter cable -61.275 -forgive" -60.000 "DBL" 0.001} \
      {"RLmeter cable  87.123 -forgive" 60.000 "DBL" 0.001} \
      {"RLmeter cableloss -32.345" -32.345 "DBL" 0.001} \
      {"RLmeter cableloss  44.125"  44.125 "DBL" 0.001} \
      {"RLmeter cableloss -28.275" -28.275 "DBL" 0.001} \
      {"RLmeter cableloss 27.465"   27.465 "DBL" 0.001} \
      {"RLmeter cableloss -71.275 -forgive" -60.000 "DBL" 0.001} \
      {"RLmeter cableloss  97.123 -forgive" 60.000 "DBL" 0.001}] 

    set testRLmtrList4 [list \
      {"RLmeter en on" 3 "INT" 0} \
      {"RLmeter en off" 1 "INT" 0} \
      {"RLmeter en 3" 3 "INT" 0} \
      {"RLmeter en 1" 1 "INT" 0} \
      {"RLmeter enable on" 3 "INT" 0} \
      {"RLmeter enable off" 1 "INT" 0} \
      {"RLmeter enable 3" 3 "INT" 0} \
      {"RLmeter enable 1" 1 "INT" 0} ]      

   set testRLmtrList7 [list                     \
      {"RLmeter peak  0" 0 "INT" 0}              \
      {"RLmeter peak  1" 1 "INT" 0}              \
      {"RLmeter peak -1 -forgive" 0 "INT" 0}     \
      {"RLmeter peak  2 -forgive" 1 "INT" 0}     \
      {"RLmeter peak off" 0 "INT" 0}             \
      {"RLmeter peak on"  1 "INT" 0}             \
      {"RLmeter peak Off" 0 "INT" 0}             \
      {"RLmeter peak On"  1 "INT" 0}             \
      {"RLmeter peakhold  0" 0 "INT" 0}          \
      {"RLmeter peakhold  1" 1 "INT" 0}          \
      {"RLmeter peakhold -1 -forgive" 0 "INT" 0} \
      {"RLmeter peakhold  2 -forgive" 1 "INT" 0} \
      {"RLmeter peakhold off" 0 "INT" 0}         \
      {"RLmeter peakhold on"  1 "INT" 0}         \
      {"RLmeter peakhold Off" 0 "INT" 0}         \
      {"RLmeter peakhold On"  1 "INT" 0}]

   set testRLmtrList8 [list                                   \
      {"RLmeter ulim  0.0" 0.0   "DBL" 0.001}           \
      {"RLmeter ulim  12.823"  12.823 "DBL" 0.001}           \
      {"RLmeter ulim  -22.23"  -22.23  "DBL" 0.001}           \
      {"RLmeter ulim  24.99"   24.99  "DBL" 0.001}           \
      {"RLmeter ulim  -120.2  -forgive" -110.00   "DBL" 0.001}    \
      {"RLmeter ulim   53.123 -forgive" 53.00 "DBL" 0.001}   \
      {"RLmeter ulimit  1.0"       1.0   "DBL" 0.001}        \
      {"RLmeter ulimit  15.98"   15.98   "DBL" 0.001}        \
      {"RLmeter ulimit  -51.29"   -51.29   "DBL" 0.001}        \
      {"RLmeter ulimit  -107.28"  -107.28   "DBL" 0.001}        \
      {"RLmeter ulimit  -120.8  -forgive" -110.0   "DBL" 0.001}  \
      {"RLmeter ulimit  52.99"   52.99   "DBL" 0.001}        \
      {"RLmeter ulimit  55.50 -forgive" 53.00 "DBL" 0.001} ]

 
# Test Llimit
   set testRLmtrList9 [list                                   \
      {"RLmeter llim  2.123"   2.123  "DBL" 0.001}           \
      {"RLmeter llim  12.823"  12.823 "DBL" 0.001}           \
      {"RLmeter llim  -72.23"   -72.23  "DBL" 0.001}           \
      {"RLmeter llim   14.99"  14.99  "DBL" 0.001}           \
      {"RLmeter llim  -132.2  -forgive" -110.0   "DBL" 0.001}    \
      {"RLmeter llim  52.99" 52.99 "DBL" 0.001}   \
      {"RLmeter llim  55.99 -forgive" 53.00 "DBL" 0.001}   \
      {"RLmeter llimit  4.123"   4.123   "DBL" 0.001}        \
      {"RLmeter llimit 15.98"   15.98   "DBL" 0.001}        \
      {"RLmeter llimit  21.29"   21.29   "DBL" 0.001}        \
      {"RLmeter llimit  -107.28" -107.28   "DBL" 0.001}        \
      {"RLmeter llimit  -121.8  -forgive" -110.0   "DBL" 0.001}  \
      {"RLmeter llimit  152.50 -forgive" 53.00 "DBL" 0.001} ]

    set testRLmtrList10 [list                    \
      {"RLmeter al  0" 0 "INT" 0}               \
      {"RLmeter al  1" 1 "INT" 0}               \
      {"RLmeter al  2" 2 "INT" 0}               \
      {"RLmeter al  3" 3 "INT" 0}               \
      {"RLmeter al -1 -forgive" 0 "INT" 0}      \
      {"RLmeter al  4 -forgive" 0 "INT" 0}      \
      {"RLmeter al off"        0 "INT" 0}       \
      {"RLmeter al enablelo"  1 "INT" 0}        \
      {"RLmeter al enablehi" 2 "INT" 0}         \
      {"RLmeter al enableboth" 3 "INT" 0}       \
      {"RLmeter al Off"        0 "INT" 0}       \
      {"RLmeter al EnableLo"  1 "INT" 0}        \
      {"RLmeter al EnableHi" 2 "INT" 0}         \
      {"RLmeter al EnableBoth" 3 "INT" 0}       \
      {"RLmeter alerts  0" 0 "INT" 0}           \
      {"RLmeter alerts  1" 1 "INT" 0}           \
      {"RLmeter alerts  2" 2 "INT" 0}           \
      {"RLmeter alerts  3" 3 "INT" 0}           \
      {"RLmeter alerts -1 -forgive" 0 "INT" 0}  \
      {"RLmeter alerts  4 -forgive" 0 "INT" 0}  \
      {"RLmeter alerts off"        0 "INT" 0}   \
      {"RLmeter alerts enablelo"  1 "INT" 0}    \
      {"RLmeter alerts enablehi" 2 "INT" 0}     \
      {"RLmeter alerts enableboth" 3 "INT" 0}   \
      {"RLmeter alerts Off"        0 "INT" 0}   \
      {"RLmeter alerts EnableLo"  1 "INT" 0}    \
      {"RLmeter alerts EnableHi" 2 "INT" 0}     \
      {"RLmeter alerts EnableBoth" 3 "INT" 0} ]
 }

#****************************************************************************
# Name:        autoTest
# Parameters:  resultsLevel		Indicates the results display level
# Description: Executes automatic test on Digital Coded Squelch commands
#****************************************************************************
 proc autoTest { resultsLevel } {
    variable testCaseList

    # Declare each test as a namespace variable
    foreach testName $testCaseList {
       variable $testName
    }

    setup 

    # Execute the test lists, and then destroy the test lists
    ::BIT::resetTestStats "cmdline_RLmeter"
    foreach testName $testCaseList {
       ::BIT::runCmdLineTests "cmdline_RLmeter" $testName
       unset $testName
    }

    if { ($resultsLevel == "Full") || ($resultsLevel == "Manual") } \
    {
       # Test commands with no data - requires visual inspection
       ::BIT::putsTest [RLmeter phrst]
       ::BIT::putsTest [RLmeter phreset]

       # Test range functions - requires visual inspection
        foreach cmd {alerts cableloss enable llimit peakhold rate reference run ulimit version} \
       {
          set temp [RLmeter $cmd -range]
          ::BIT::putsTest "The range of <$cmd> is:  $temp"
       }

       # Test string functions - requires visual inspection
       ::BIT::putsTest [RLmeter res]
       ::BIT::putsTest [RLmeter resources]

       ::BIT::putsTest [RLmeter ver 0]
       ::BIT::putsTest [RLmeter ver 1]
       ::BIT::putsTest [RLmeter version]
    }

    # Reset RLmeter engines to typical values after completion of the test
 }

}

# Add this test to the list of tests
::BIT::addTest cmdline_RLmeter
