#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C) 2001, 2003  DESIGN ENGINEERING      #
# --------------------------------------------------------------------------#
# FILE        : 1017.test_Hw_utility.tcl                                    #
# PRODUCT     : FITS                                                        #
# AUTHOR      : Nghia Tran                                                  #
# DESCRIPTION : Remote Command Line Test: Hardware Utilities                #
#############################################################################

package require HW_control
package require BIT

#****************************************************************************
# Test variables
#****************************************************************************
namespace eval ::cmdline_HW_control {
    variable testCaseList		;# List of test cases
    variable shortDesc			;# Test description - menu size
    variable longDesc			;# Test description - used by reports
    variable testTime			;# Test duration (Seconds - approximation)

    # Define the list of HW control tests
    set testCaseList [list \
           testhwutilityList1 testhwutilityList2 testhwutilityList3 \
           testhwutilityList4 testhwutilityList5 testhwutilityList6]

    # Declare each test as a namespace variable
    foreach testName $testCaseList {
       variable $testName
    }

    set shortDesc "HWctl cmds"
    set longDesc "Hardware Control command line test"
    set testTime 1.00

#****************************************************************************
# Name:        setup
# Parameters:  none
# Description: Sets up a list of HW control tests
#****************************************************************************
 proc setup { } {
    variable testCaseList

    # Declare each test as a namespace variable
    foreach testName $testCaseList {
       variable $testName
    }

    #
    # Define test cases
    #
    set testhwutilityList1 [list \
       {"HW_control bklgt -1 -forgive" 2048 "INT" 0} \
       {"HW_control bklgt 0" 0 "INT" 0} \
       {"HW_control bklgt 100" 100 "INT" 0} \
       {"HW_control bklgt 999" 999 "INT" 0} \
       {"HW_control bklgt 4095" 4095 "INT" 0} \
       {"HW_control bklgt 4096 -forgive" 2048 "INT" 0} \
       {"HW_control bklgt" 2048 "INT" 0} \
       {"HW_control backlight 1" 1 "INT" 0} \
       {"HW_control backlight 121" 121 "INT" 0} \
       {"HW_control backlight 980" 980 "INT" 0} \
       {"HW_control backlight 4098 -forgive" 2048 "INT" 0} ]

    set testhwutilityList2 [list \
       {"HW_control ref -1 -forgive" 0 "INT" 0} \
       {"HW_control ref 0" 0 "INT" 0} \
       {"HW_control ref" 0 "INT" 0} \
       {"HW_control ref 1" 1 "INT" 0} \
       {"HW_control ref 2 -forgive" 1 "INT" 0} \
       {"HW_control ref" 1 "INT" 0} \
       {"HW_control ref_10mhz -1 -forgive" 0 "INT" 0} \
       {"HW_control ref_10mhz 0" 0 "INT" 0} \
       {"HW_control ref_10mhz" 0 "INT" 0} \
       {"HW_control ref_10mhz 1" 1 "INT" 0} \
       {"HW_control ref_10mhz 2 -forgive" 1 "INT" 0} \
       {"HW_control ref_10mhz" 1 "INT" 0} ]
     
    set testhwutilityList3 [list \
       {"HW_control tcxo -1 -forgive" 2048 "INT" 0} \
       {"HW_control tcxo" 2048 "INT" 0} \
       {"HW_control tcxo 0" 0 "INT" 0} \
       {"HW_control tcxo 100" 100 "INT" 0} \
       {"HW_control tcxo 999" 999 "INT" 0} \
       {"HW_control tcxo" 999 "INT" 0} \
       {"HW_control tcxo 4095" 4095 "INT" 0} \
       {"HW_control tcxo 4096 -forgive" 2048 "INT" 0} \
       {"HW_control tcxo 10000 -forgive" 2048 "INT" 0} \
       {"HW_control tcxo" 2048 "INT" 0} ]
              
    set testhwutilityList4 [list \
       {"HW_control yig -1000 -forgive" 0 "INT" 0} \
       {"HW_control yig -1 -forgive" 0 "INT" 0} \
       {"HW_control yig" 0 "INT" 0} \
       {"HW_control yig 0" 0 "INT" 0} \
       {"HW_control yig 1" 1 "INT" 0} \
       {"HW_control yig" 1 "INT" 0} \
       {"HW_control yig 2 -forgive" 1 "INT" 0} \
       {"HW_control yig 1000 -forgive" 1 "INT" 0} \
       {"HW_control yig_lock -1000 -forgive" 0 "INT" 0} \
       {"HW_control yig_lock -1 -forgive" 0 "INT" 0} \
       {"HW_control yig_lock" 0 "INT" 0} \
       {"HW_control yig_lock 0" 0 "INT" 0} \
       {"HW_control yig_lock 1" 1 "INT" 0} \
       {"HW_control yig_lock" 1 "INT" 0} \
       {"HW_control yig_lock 2 -forgive" 1 "INT" 0} \
       {"HW_control yig_lock 1000 -forgive" 1 "INT" 0} ]
      
    set testhwutilityList5 [list \
       {"HW_control leds 0x00 0x07" 0x00 "INT" 0} \
       {"HW_control leds " 0x00 "INT" 0} \
       {"HW_control leds 0x00 0x00" 0x00 "INT" 0} \
       {"HW_control leds 0x01 0x00" 0x00 "INT" 0} \
       {"HW_control leds 0x02 0x00" 0x00 "INT" 0} \
       {"HW_control leds 0x04 0x00" 0x00 "INT" 0} \
       {"HW_control leds 0x07 0x00" 0x00 "INT" 0} \
       {"HW_control leds 0x08 0x00 -forgive" 0x00 "INT" 0} \
       {"HW_control leds 0xFF 0x00 -forgive" 0x00 "INT" 0} \
       {"HW_control leds 0x00 0x01" 0x00 "INT" 0} \
       {"HW_control leds 0x01 0x01" 0x01 "INT" 0} \
       {"HW_control leds 0x02 0x01" 0x00 "INT" 0} \
       {"HW_control leds 0x04 0x01" 0x00 "INT" 0} \
       {"HW_control leds 0x07 0x01" 0x01 "INT" 0} \
       {"HW_control leds 0x08 0x01 -forgive" 0x00 "INT" 0} \
       {"HW_control leds" 0x00 "INT" 0} \
       {"HW_control leds 0xFF 0x01 -forgive" 0x01 "INT" 0} \
       {"HW_control leds 0x00 0x07" 0x00 "INT" 0} \
       {"HW_control leds 0x00 0x02" 0x00 "INT" 0} \
       {"HW_control leds 0x01 0x02" 0x00 "INT" 0} \
       {"HW_control leds 0x02 0x02" 0x02 "INT" 0} \
       {"HW_control leds 0x04 0x02" 0x00 "INT" 0} \
       {"HW_control leds 0x07 0x02" 0x02 "INT" 0} \
       {"HW_control leds 0x08 0x02 -forgive" 0x00 "INT" 0} \
       {"HW_control leds" 0x00 "INT" 0} \
       {"HW_control leds 0xFF 0x02 -forgive" 0x02 "INT" 0} \
       {"HW_control leds 0x00 0x07" 0x00 "INT" 0} \
       {"HW_control leds 0x00 0x04" 0x00 "INT" 0} \
       {"HW_control leds 0x01 0x04" 0x00 "INT" 0} \
       {"HW_control leds 0x02 0x04" 0x00 "INT" 0} \
       {"HW_control leds 0x04 0x04" 0x04 "INT" 0} \
       {"HW_control leds 0x07 0x04" 0x04 "INT" 0} \
       {"HW_control leds 0x08 0x04 -forgive" 0x00 "INT" 0} \
       {"HW_control leds" 0x00 "INT" 0} \
       {"HW_control leds 0xFF 0x04 -forgive" 0x04 "INT" 0} \
       {"HW_control leds 0x00 0x07" 0x00 "INT" 0} \
       {"HW_control leds 0x00 0x07" 0x00 "INT" 0} \
       {"HW_control leds 0x01 0x07" 0x01 "INT" 0} \
       {"HW_control leds 0x02 0x07" 0x02 "INT" 0} \
       {"HW_control leds 0x04 0x07" 0x04 "INT" 0} \
       {"HW_control leds 0x07 0x07" 0x07 "INT" 0} \
       {"HW_control leds 0x08 0x07 -forgive" 0x00 "INT" 0} \
       {"HW_control leds" 0x00 "INT" 0} \
       {"HW_control leds 0xFF 0x07 -forgive" 0x07 "INT" 0} \
       {"HW_control leds 0x00 0x07" 0x00 "INT" 0} \
       {"HW_control leds 0x00 0xFF -forgive" 0x00 "INT" 0} \
       {"HW_control leds 0x01 0xFF -forgive" 0x01 "INT" 0} \
       {"HW_control leds 0x02 0xFF -forgive" 0x02 "INT" 0} \
       {"HW_control leds 0x04 0xFF -forgive" 0x04 "INT" 0} \
       {"HW_control leds 0x07 0xFF -forgive" 0x07 "INT" 0} \
       {"HW_control leds 0x08 0xFF -forgive" 0x00 "INT" 0} \
       {"HW_control leds" 0x00 "INT" 0} \
       {"HW_control leds 0xFF 0xFF -forgive" 0x07 "INT" 0} \
       {"HW_control leds 0x00 0x07" 0x00 "INT" 0} \
       {"HW_control leds 0x00 0x08 -forgive" 0x00 "INT" 0} \
       {"HW_control leds 0x01 0x08 -forgive" 0x00 "INT" 0} \
       {"HW_control leds 0x02 0x08 -forgive" 0x00 "INT" 0} \
       {"HW_control leds 0x04 0x08 -forgive" 0x00 "INT" 0} \
       {"HW_control leds 0x07 0x08 -forgive" 0x00 "INT" 0} \
       {"HW_control leds 0x08 0x08 -forgive" 0x00 "INT" 0} \
       {"HW_control leds 0xFF 0x08 -forgive" 0x00 "INT" 0} \
       {"HW_control leds 0x07 0x07" 0x07 "INT" 0} \
       {"HW_control leds 0x06 0x01" 0x06 "INT" 0} \
       {"HW_control leds 0x07 0x07" 0x07 "INT" 0} \
       {"HW_control leds 0x05 0x01" 0x07 "INT" 0} \
       {"HW_control leds 0x03 0x01" 0x07 "INT" 0} \
       {"HW_control leds 0x07 0x07" 0x07 "INT" 0} \
       {"HW_control leds 0x06 0x02" 0x07 "INT" 0} \
       {"HW_control leds 0x05 0x02" 0x05 "INT" 0} \
       {"HW_control leds 0x07 0x07" 0x07 "INT" 0} \
       {"HW_control leds 0x03 0x02" 0x07 "INT" 0} \
       {"HW_control leds 0x07 0x07" 0x07 "INT" 0} \
       {"HW_control leds 0x06 0x04" 0x07 "INT" 0} \
       {"HW_control leds 0x05 0x04" 0x07 "INT" 0} \
       {"HW_control leds 0x03 0x04" 0x03 "INT" 0} \
       {"HW_control fp_leds 0 7" 0 "INT" 0} \
       {"HW_control fp_leds 1 7" 1 "INT" 0} \
       {"HW_control fp_leds 2 7" 2 "INT" 0} \
       {"HW_control fp_leds 3 7" 3 "INT" 0} \
       {"HW_control fp_leds 4 7" 4 "INT" 0} \
       {"HW_control fp_leds 5 7" 5 "INT" 0} \
       {"HW_control fp_leds 6 7" 6 "INT" 0} \
       {"HW_control fp_leds 7 7" 7 "INT" 0} ]

    set testhwutilityList6 [list \
       {"HW_control ref -1 -forgive" 0 "INT" 0} \
       {"HW_control ref 0" 0 "INT" 0} \
       {"HW_control ref 1" 1 "INT" 0} \
       {"HW_control ref 2 -forgive" 1 "INT" 0} \
       {"HW_control ref_10mhz -1 -forgive" 0 "INT" 0} \
       {"HW_control ref_10mhz 0" 0 "INT" 0} \
       {"HW_control ref_10mhz 1" 1 "INT" 0} \
       {"HW_control ref_10mhz 2 -forgive" 1 "INT" 0} ]
 }

#****************************************************************************
# Name:        autoTest
# Parameters:  resultsLevel		Indicates the results display level
# Description: Executes automatic test on HW control commands
#****************************************************************************
 proc autoTest { resultsLevel } {
    variable testCaseList

    # Declare each test as a namespace variable
    foreach testName $testCaseList {
       variable $testName
    }

    setup 

    # Execute the test lists, and then destroy the test lists
    ::BIT::resetTestStats "cmdline_HW_control"
    foreach testName $testCaseList {
       ::BIT::runCmdLineTests "cmdline_HW_control" $testName
       unset $testName
    }

    if { ($resultsLevel == "Full") || ($resultsLevel == "Manual") } \
    {
       # Test commands with no data - requires visual inspection
       ::BIT::putsTest [HW_control start_yig_cal]
       ::BIT::putsTest [HW_control yig_cal]

       # Test string functions - requires visual inspection
        foreach cmd {bklgt ref tcxo yig leds yig_cal} \
       {
          set temp [HW_control $cmd -range]
          ::BIT::putsTest "The range of <$cmd> is:  $temp"
       }

       # Test string functions - requires visual inspection
       ::BIT::putsTest [HW_control res]
       ::BIT::putsTest [HW_control resources]
       ::BIT::putsTest [HW_control version -1]
       ::BIT::putsTest [HW_control version 0]
       ::BIT::putsTest [HW_control version 1]
       ::BIT::putsTest [HW_control version 2]
   }
   # Reset HW_control engines to typical values after completion of the test

 }

}

# Add this test to the list of tests
::BIT::addTest cmdline_HW_control

