#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C) 2001        DESIGN ENGINEERING      #
# --------------------------------------------------------------------------#
# FILE        : test_Dtmf.tcl                                               #
# PRODUCT     : FITS                                                        #
# AUTHOR      : Bruce Butterfield                                           #
# DESCRIPTION : Test basic DTMF commands                                    #
#############################################################################

package require Dtmf

#############################################################################
# FUNCTION    : setupDtmfTests                                              #
# ARGUMENTS   : none                                                        #
# RETURNS     : none                                                        #
# PURPOSE     : Sets up a list of DTMF                                      #
#############################################################################
proc setupDtmfTests { } \
{
   global dtmfTestList 

   # Define the list of Digital Coded Squelch tests
   set dtmfTestList [list \
           testDtmfList1 testDtmfList2 ]

   # Declare each test as a global variable
   foreach testName $dtmfTestList {
      global $testName
   }

   #
   # Define test cases
   #
   set testDtmfList1 [list \
      {"Dtmf en -1 -forgive" 3 "INT" 0} \
      {"Dtmf en 0" 1 "INT" 0} \
      {"Dtmf en 1" 1 "INT" 0} \
      {"Dtmf en 2" 3 "INT" 0} \
      {"Dtmf en 3" 3 "INT" 0} \
      {"Dtmf en 4 -forgive" 1 "INT" 0} \
      {"Dtmf enable -1 -forgive" 3 "INT" 0} \
      {"Dtmf enable 0" 1 "INT" 0} \
      {"Dtmf enable 1" 1 "INT" 0} \
      {"Dtmf enable 2" 3 "INT" 0} \
      {"Dtmf enable 3" 3 "INT" 0} \
      {"Dtmf enable disable" 1 "INT" 0} \
      {"Dtmf enable enable" 3 "INT" 0} \
      {"Dtmf enable off" 1 "INT" 0} \
      {"Dtmf enable on" 3 "INT" 0} \
      {"Dtmf enable 4 -forgive" 1 "INT" 0} ]

   set testDtmfList2 [list \
      {"Dtmf run -1 -forgive" 0 "INT" 0} \
      {"Dtmf run 1" 1 "INT" 0} \
      {"Dtmf run 0" 0 "INT" 0} \
      {"Dtmf run on" 1 "INT" 0} \
      {"Dtmf run off" 0 "INT" 0} \
      {"Dtmf run 4 -forgive" 0 "INT" 0} ]

}

#############################################################################
# FUNCTION    : autoTestDtmf                                                #
# ARGUMENTS   : none                                                        #
# RETURNS     : none                                                        #
# PURPOSE     : Executes automatic test on Digital Coded Squelch commands   #
#############################################################################
proc autoTestDtmf { } \
{
   global dtmfTestList 

   package require Dtmf
   setupDtmfTests 

   # Execute the test lists, and then destroy the test lists
   foreach testName $dtmfTestList {
      testCompare $testName
      global $testName
      unset $testName
   }
   unset dtmfTestList

   # Test commands with no data - requires visual inspection

   # Test string functions - requires visual inspection
   set tempResults [Dtmf en -range]
   testReportData $tempResults
   set tempResults [Dtmf run -range]
   testReportData $tempResults
   set tempResults [Dtmf ver -range]
   testReportData $tempResults

   set tempResults [Dtmf res]
   testReportData $tempResults
   set tempResults [Dtmf resources]
   testReportData $tempResults

   set tempResults [Dtmf ver 0]
   testReportData $tempResults
   set tempResults [Dtmf ver 1]
   testReportData $tempResults
   set tempResults [Dtmf version]
   testReportData $tempResults

   # Reset Digital Coded Squelch to typical values after completion of the test
   Dtmf en 3
   Dtmf run 0
}


