#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C) 2003  DESIGN ENGINEERING            #
# --------------------------------------------------------------------------#
# FILE        : 1019.test_LTR.tcl                                           #
# PRODUCT     : FITS                                                        #
# AUTHOR      : Bruce Butterfield                                           #
# DESCRIPTION : Remote Command Line Test:  LTR (Logic Trunked Radio)        #
#############################################################################

package require Ltr
package require BIT

#****************************************************************************
# Test variables
#****************************************************************************
namespace eval ::cmdline_Ltr {
    variable testCaseList		;# List of test cases
    variable shortDesc			;# Test description - menu size
    variable longDesc			;# Test description - used by reports
    variable testTime			;# Test duration (Seconds - approximation)

    # Define the list of Sinad/Distortion tests
    set testCaseList [list \
           testLtrList1  testLtrList2  testLtrList3  testLtrList4  testLtrList5 \
           testLtrList6  testLtrList7  testLtrList8  testLtrList9  testLtrList10 \
           testLtrList11 testLtrList12 testLtrList13 testLtrList14 testLtrList15 ]

    # Declare each test as a namespace variable
    foreach testName $testCaseList {
       variable $testName
    }

    set shortDesc "LTR cmds"
    set longDesc "LTR command line test"
    set testTime 0.50

#****************************************************************************
# Name:        setup
# Parameters:  none
# Description: Sets up a list of LTR tests
#****************************************************************************
 proc setup { } {
    variable testCaseList

    # Declare each test as a namespace variable
    foreach testName $testCaseList {
       variable $testName
    }

    #
    # Define test cases
    #
    set testLtrList1 [list \
       {"Ltr area -1 -forgive" 1 "INT" 0} \
       {"Ltr area 0x12345 -forgive" 1 "INT" 0} \
       {"Ltr area 0x1234E -forgive" 0 "INT" 0} \
       {"Ltr area 0" 0 "INT" 0} \
       {"Ltr area 1" 1 "INT" 0} \
       {"Ltr area" 1 "INT" 0} \
       {"Ltr area 2 -forgive" 0 "INT" 0} \
       {"Ltr area off" 0 "INT" 0} \
       {"Ltr area on" 1 "INT" 0} ]

    set testLtrList2 [list \
       {"Ltr band -1 -forgive" 0 "INT" 0} \
       {"Ltr band -1000 -forgive" 0 "INT" 0} \
       {"Ltr band 0" 0 "INT" 0} \
       {"Ltr band 1" 1 "INT" 0} \
       {"Ltr band" 1 "INT" 0} \
       {"Ltr band 2" 2 "INT" 0} \
       {"Ltr band 3 -forgive" 0 "INT" 0} \
       {"Ltr band" 0 "INT" 0} \
       {"Ltr band 3000 -forgive" 0 "INT" 0} ]

    set testLtrList3 [list \
       {"Ltr border -1000 -forgive" 0 "INT" 0} \
       {"Ltr border -3 -forgive" 1 "INT" 0} \
       {"Ltr border -2 -forgive" 0 "INT" 0} \
       {"Ltr border -1 -forgive" 1 "INT" 0} \
       {"Ltr border 0" 0 "INT" 0} \
       {"Ltr border 1" 1 "INT" 0} \
       {"Ltr border" 1 "INT" 0} \
       {"Ltr border 2 -forgive" 0 "INT" 0} \
       {"Ltr border 3 -forgive" 1 "INT" 0} \
       {"Ltr border 4 -forgive" 0 "INT" 0} \
       {"Ltr border 1000 -forgive" 0 "INT" 0} \
       {"Ltr borderoffset -1000 -forgive" 0 "INT" 0} \
       {"Ltr borderoffset -1 -forgive" 1 "INT" 0} \
       {"Ltr borderoffset 0" 0 "INT" 0} \
       {"Ltr borderoffset 1" 1 "INT" 0} \
       {"Ltr borderoffset" 1 "INT" 0} \
       {"Ltr borderoffset 2 -forgive" 0 "INT" 0} \
       {"Ltr borderoffset 3 -forgive" 1 "INT" 0} \
       {"Ltr borderoffset 1000 -forgive" 0 "INT" 0} ]

    set testLtrList4 [list \
       {"Ltr en -1 -forgive" 7 "INT" 0} \
       {"Ltr en 0" 1 "INT" 0} \
       {"Ltr en 1" 1 "INT" 0} \
       {"Ltr en" 1 "INT" 0} \
       {"Ltr en 2" 3 "INT" 0} \
       {"Ltr en 3" 3 "INT" 0} \
       {"Ltr en" 3 "INT" 0} \
       {"Ltr en 4" 5 "INT" 0} \
       {"Ltr en 5" 5 "INT" 0} \
       {"Ltr en 6" 7 "INT" 0} \
       {"Ltr en 7" 7 "INT" 0} \
       {"Ltr en 8 -forgive" 1 "INT" 0} \
       {"Ltr enable -1 -forgive" 7 "INT" 0} \
       {"Ltr enable 0" 1 "INT" 0} \
       {"Ltr enable 1" 1 "INT" 0} \
       {"Ltr enable" 1 "INT" 0} \
       {"Ltr enable 2" 3 "INT" 0} \
       {"Ltr enable 3" 3 "INT" 0} \
       {"Ltr enable" 3 "INT" 0} \
       {"Ltr enable 4" 5 "INT" 0} \
       {"Ltr enable 5" 5 "INT" 0} \
       {"Ltr enable 6" 7 "INT" 0} \
       {"Ltr enable 7" 7 "INT" 0} \
       {"Ltr enable 8 -forgive" 1 "INT" 0} ]

    # 800 band - rxfreq   = 806.0125 + channel * 0.025
    # 900 band - rxfreq   = 896.0125 + channel * 0.125
    set testLtrList5 [list \
       {"Ltr band 0" 0 "INT" 0} \
       {"Ltr borderoffset 0" 0 "INT" 0} \
       {"Ltr fccrx -1000 -forgive" 806.0125 "DBL" 0.001} \
       {"Ltr fccrx -1 -forgive" 806.0125 "DBL" 0.001} \
       {"Ltr fccrx 0 -forgive" 806.0125 "DBL" 0.001} \
       {"Ltr fccrx 1" 806.0125 "DBL" 0.001} \
       {"Ltr fccrx 2" 806.0375 "DBL" 0.001} \
       {"Ltr fccrx 3" 806.0625 "DBL" 0.001} \
       {"Ltr fccrx 11" 806.2625 "DBL" 0.001} \
       {"Ltr fccrx 101" 808.5125 "DBL" 0.001} \
       {"Ltr fccrx 1001" 831.0125 "DBL" 0.001} \
       {"Ltr fccrx 6000" 955.9875 "DBL" 0.001} \
       {"Ltr fccrx 6001 -forgive" 955.9875 "DBL" 0.001} \
       {"Ltr fccrx 10000 -forgive" 955.9875 "DBL" 0.001} \
       {"Ltr borderoffset 1" 1 "INT" 0} \
       {"Ltr fccrx 11" 806.2500 "DBL" 0.001} \
       {"Ltr fccrx 101" 808.5000 "DBL" 0.001} \
       {"Ltr band 1" 1 "INT" 0} \
       {"Ltr borderoffset 0" 0 "INT" 0} \
       {"Ltr fccrx -1000 -forgive" 896.0125 "DBL" 0.001} \
       {"Ltr fccrx -1 -forgive" 896.0125 "DBL" 0.001} \
       {"Ltr fccrx 0 -forgive" 896.0125 "DBL" 0.001} \
       {"Ltr fccrx 1" 896.0125 "DBL" 0.001} \
       {"Ltr fccrx 2" 896.0250 "DBL" 0.001} \
       {"Ltr fccrx 3" 896.0375 "DBL" 0.001} \
       {"Ltr fccrx 11" 896.1375 "DBL" 0.001} \
       {"Ltr fccrx 101" 897.2625 "DBL" 0.001} \
       {"Ltr fccrx 1001" 908.5125 "DBL" 0.001} \
       {"Ltr fccrx 6000" 971.0000 "DBL" 0.001} \
       {"Ltr fccrx 6001 -forgive" 971.0000 "DBL" 0.001} \
       {"Ltr fccrx 10000 -forgive" 971.0000 "DBL" 0.001} \
       {"Ltr borderoffset 1" 1 "INT" 0} \
       {"Ltr fccrx 11" 896.1250 "DBL" 0.001} \
       {"Ltr fccrx 101" 897.2500 "DBL" 0.001} ]

    # 800 band - rxfreq   = 806.0125 + channel * 0.025 + 45
    # 900 band - rxfreq   = 896.0125 + channel * 0.125 + 39
    set testLtrList6 [list \
       {"Ltr band 0" 0 "INT" 0} \
       {"Ltr borderoffset 0" 0 "INT" 0} \
       {"Ltr fcctx -1000 -forgive" 851.0125 "DBL" 0.001} \
       {"Ltr fcctx -1 -forgive" 851.0125 "DBL" 0.001} \
       {"Ltr fcctx 0 -forgive" 851.0125 "DBL" 0.001} \
       {"Ltr fcctx 1" 851.0125 "DBL" 0.001} \
       {"Ltr fcctx 2" 851.0375 "DBL" 0.001} \
       {"Ltr fcctx 3" 851.0625 "DBL" 0.001} \
       {"Ltr fcctx 11" 851.2625 "DBL" 0.001} \
       {"Ltr fcctx 101" 853.5125 "DBL" 0.001} \
       {"Ltr fcctx 1001" 876.0125 "DBL" 0.001} \
       {"Ltr fcctx 6000" 1000.9875 "DBL" 0.001} \
       {"Ltr fcctx 6001 -forgive" 1000.9875 "DBL" 0.001} \
       {"Ltr fcctx 10000 -forgive" 1000.9875 "DBL" 0.001} \
       {"Ltr borderoffset 1" 1 "INT" 0} \
       {"Ltr fcctx 11" 851.2500 "DBL" 0.001} \
       {"Ltr fcctx 101" 853.5000 "DBL" 0.001} \
       {"Ltr band 1" 1 "INT" 0} \
       {"Ltr borderoffset 0" 0 "INT" 0} \
       {"Ltr fcctx -1000 -forgive" 935.0125 "DBL" 0.001} \
       {"Ltr fcctx -1 -forgive" 935.0125 "DBL" 0.001} \
       {"Ltr fcctx 0 -forgive" 935.0125 "DBL" 0.001} \
       {"Ltr fcctx 1" 935.0125 "DBL" 0.001} \
       {"Ltr fcctx 2" 935.0250 "DBL" 0.001} \
       {"Ltr fcctx 3" 935.0375 "DBL" 0.001} \
       {"Ltr fcctx 11" 935.1375 "DBL" 0.001} \
       {"Ltr fcctx 101" 936.2625 "DBL" 0.001} \
       {"Ltr fcctx 1001" 947.5125 "DBL" 0.001} \
       {"Ltr fcctx 6000" 1010.0000 "DBL" 0.001} \
       {"Ltr fcctx 6001 -forgive" 1010.0000 "DBL" 0.001} \
       {"Ltr fcctx 10000 -forgive" 1010.0000 "DBL" 0.001} \
       {"Ltr borderoffset 1" 1 "INT" 0} \
       {"Ltr fcctx 11" 935.1250 "DBL" 0.001} \
       {"Ltr fcctx 101" 936.2500 "DBL" 0.001} ]

    set testLtrList7 [list \
       {"Ltr free -1 -forgive" 0x1F "INT" 0} \
       {"Ltr free 0x00" 0x00 "INT" 0} \
       {"Ltr free 0x01" 0x01 "INT" 0} \
       {"Ltr free 0x02" 0x02 "INT" 0} \
       {"Ltr free 0x04" 0x04 "INT" 0} \
       {"Ltr free 0x08" 0x08 "INT" 0} \
       {"Ltr free 0x10" 0x10 "INT" 0} \
       {"Ltr free 0x1F" 0x1F "INT" 0} \
       {"Ltr free 0x20 -forgive" 0x00 "INT" 0} \
       {"Ltr free 0x12345 -forgive" 0x05 "INT" 0} \
       {"Ltr free 0xABCDE -forgive" 0x1E "INT" 0} ]

     # Test mode before GO because it's setting is used in the GO command
     set testLtrList8 [list \
       {"Ltr mode -1000 -forgive" 0 "INT" 0} \
       {"Ltr mode" 0 "INT" 0} \
       {"Ltr mode 1" 1 "INT" 0} \
       {"Ltr mode -1 -forgive" 0 "INT" 0} \
       {"Ltr mode 0" 0 "INT" 0} \
       {"Ltr mode 1" 1 "INT" 0} \
       {"Ltr mode 2" 2 "INT" 0} \
       {"Ltr mode" 2 "INT" 0} \
       {"Ltr mode 3" 3 "INT" 0} \
       {"Ltr mode 4" 4 "INT" 0} \
       {"Ltr mode 5" 5 "INT" 0} \
       {"Ltr mode 6 -forgive" 5 "INT" 0} \
       {"Ltr mode 1000 -forgive" 5 "INT" 0} \
       {"Ltr mode" 5 "INT" 0} ]

    # NOTE: Since the simulation is running, the RxChannel and TxChannel might
    # be modified by the simulation after the GOTO has been issued. The query
    # on these SHOULD pass most of the time, but there may be occational errors
    # which will not be repeatable.  Any repeatable error should be investigated.
    set testLtrList9 [list \
       {"Ltr mode 4" 4 "INT" 0} \
       {"Ltr go -1 -forgive" 0x1F "INT" 0} \
       {"Ltr go 0x00" 0x00 "INT" 0} \
       {"Ltr go 0x01" 0x01 "INT" 0} \
       {"Ltr rxchan" 1 "INT" 0} \
       {"Ltr txchan" 1 "INT" 0} \
       {"Ltr go 0x02" 0x02 "INT" 0} \
       {"Ltr go 0x04" 0x04 "INT" 0} \
       {"Ltr go 0x08" 0x08 "INT" 0} \
       {"Ltr rxchan" 8 "INT" 0} \
       {"Ltr txchan" 8 "INT" 0} \
       {"Ltr go 0x10" 0x10 "INT" 0} \
       {"Ltr rxchan" 16 "INT" 0} \
       {"Ltr txchan" 16 "INT" 0} \
       {"Ltr go 0x1F" 0x1F "INT" 0} \
       {"Ltr rxchan" 31 "INT" 0} \
       {"Ltr txchan" 31 "INT" 0} \
       {"Ltr go 0x40 -forgive" 0x00 "INT" 0} \
       {"Ltr go 0x12345 -forgive" 0x05 "INT" 0} \
       {"Ltr go 0xABCDE -forgive" 0x1E "INT" 0} \
       {"Ltr goto -1 -forgive" 0x1F "INT" 0} \
       {"Ltr goto 0x00" 0x00 "INT" 0} \
       {"Ltr goto 0x01" 0x01 "INT" 0} \
       {"Ltr goto 0x02" 0x02 "INT" 0} \
       {"Ltr goto 0x04" 0x04 "INT" 0} \
       {"Ltr goto 0x08" 0x08 "INT" 0} \
       {"Ltr goto 0x10" 0x10 "INT" 0} \
       {"Ltr goto 0x1F" 0x1F "INT" 0} \
       {"Ltr goto 0x20 -forgive" 0x00 "INT" 0} \
       {"Ltr goto 0x12345 -forgive" 0x05 "INT" 0} \
       {"Ltr goto 0xABCDE -forgive" 0x1E "INT" 0} \
       {"Ltr mode 4" 4 "INT" 0} \
       {"Ltr go 13" 13 "INT" 0} \
       {"Ltr rxchan" 13 "INT" 0} \
       {"Ltr txchan" 13 "INT" 0} \
       {"Ltr mode 0" 0 "INT" 0} \
       {"Ltr go 5" 5 "INT" 0} \
       {"Ltr rxchan" 13 "INT" 0} \
       {"Ltr txchan" 13 "INT" 0} \
       {"Ltr mode 1" 1 "INT" 0} \
       {"Ltr go 6" 6 "INT" 0} \
       {"Ltr rxchan" 13 "INT" 0} \
       {"Ltr txchan" 13 "INT" 0} \
       {"Ltr mode 2" 2 "INT" 0} \
       {"Ltr go 7" 7 "INT" 0} \
       {"Ltr rxchan" 13 "INT" 0} \
       {"Ltr txchan" 13 "INT" 0} \
       {"Ltr mode 3" 3 "INT" 0} \
       {"Ltr go 8" 8 "INT" 0} \
       {"Ltr rxchan" 13 "INT" 0} \
       {"Ltr txchan" 13 "INT" 0} \
       {"Ltr mode 5" 5 "INT" 0} \
       {"Ltr go 9" 9 "INT" 0} \
       {"Ltr rxchan" 13 "INT" 0} \
       {"Ltr txchan" 13 "INT" 0} ]

    set testLtrList10 [list \
       {"Ltr grp -1 -forgive" 255 "INT" 0} \
       {"Ltr grp 0xFFFF -forgive" 0xFF "INT" 0} \
       {"Ltr grp 0x1234 -forgive" 0x34 "INT" 0} \
       {"Ltr grp 0x5678 -forgive" 0x78 "INT" 0} \
       {"Ltr grp 0x9ABC -forgive" 0xBC "INT" 0} \
       {"Ltr grp 0" 0 "INT" 0} \
       {"Ltr grp 1" 1 "INT" 0} \
       {"Ltr grp 10" 10 "INT" 0} \
       {"Ltr grp" 10 "INT" 0} \
       {"Ltr grp 100" 100 "INT" 0} \
       {"Ltr grp 255" 255 "INT" 0} \
       {"Ltr grp 256 -forgive" 0 "INT" 0} \
       {"Ltr group -1 -forgive" 255 "INT" 0} \
       {"Ltr group 0" 0 "INT" 0} \
       {"Ltr group 1" 1 "INT" 0} \
       {"Ltr group 10" 10 "INT" 0} \
       {"Ltr group" 10 "INT" 0} \
       {"Ltr group 100" 100 "INT" 0} \
       {"Ltr group 255" 255 "INT" 0} \
       {"Ltr group 256 -forgive" 0 "INT" 0} ]

     # Test MobileState before HOME because HOME uses it's settings 
     set testLtrList11 [list \
       {"Ltr mstate -1000 -forgive" 0 "INT" 0} \
       {"Ltr mstate" 0 "INT" 0} \
       {"Ltr mstate -1 -forgive" 0 "INT" 0} \
       {"Ltr mstate 0" 0 "INT" 0} \
       {"Ltr mstate 1" 1 "INT" 0} \
       {"Ltr mstate 2 -forgive" 1 "INT" 0} \
       {"Ltr mstate 3 -forgive" 1 "INT" 0} \
       {"Ltr mstate" 1 "INT" 0} \
       {"Ltr mstate 1000 -forgive" 1 "INT" 0} \
       {"Ltr mobilestate -1000 -forgive" 0 "INT" 0} \
       {"Ltr mobilestate" 0 "INT" 0} \
       {"Ltr mobilestate -1 -forgive" 0 "INT" 0} \
       {"Ltr mobilestate 0" 0 "INT" 0} \
       {"Ltr mobilestate 1" 1 "INT" 0} \
       {"Ltr mobilestate 2 -forgive" 1 "INT" 0} \
       {"Ltr mobilestate 3 -forgive" 1 "INT" 0} \
       {"Ltr mobilestate" 1 "INT" 0} \
       {"Ltr mobilestate 1000 -forgive" 1 "INT" 0} ]

    set testLtrList12 [list \
       {"Ltr mode 5" 5 "INT" 0} \
       {"Ltr mobilestate 0" 0 "INT" 0} \
       {"Ltr home -1 -forgive" 0x1F "INT" 0} \
       {"Ltr home 0x00" 0x00 "INT" 0} \
       {"Ltr home 0x01" 0x01 "INT" 0} \
       {"Ltr rxchan" 1 "INT" 0} \
       {"Ltr txchan" 1 "INT" 0} \
       {"Ltr home 0x02" 0x02 "INT" 0} \
       {"Ltr home 0x04" 0x04 "INT" 0} \
       {"Ltr home 0x08" 0x08 "INT" 0} \
       {"Ltr rxchan" 8 "INT" 0} \
       {"Ltr txchan" 8 "INT" 0} \
       {"Ltr home 0x10" 0x10 "INT" 0} \
       {"Ltr rxchan" 16 "INT" 0} \
       {"Ltr txchan" 16 "INT" 0} \
       {"Ltr home 0x1F" 0x1F "INT" 0} \
       {"Ltr rxchan" 31 "INT" 0} \
       {"Ltr txchan" 31 "INT" 0} \
       {"Ltr home 0x20 -forgive" 0x00 "INT" 0} \
       {"Ltr home 0x12345 -forgive" 0x05 "INT" 0} \
       {"Ltr home 0xABCDE -forgive" 0x1E "INT" 0} \
       {"Ltr home 15" 15 "INT" 0} \
       {"Ltr rxchan" 15 "INT" 0} \
       {"Ltr txchan" 15 "INT" 0} \
       {"Ltr mobilestate 1" 1 "INT" 0} \
       {"Ltr home 7" 7 "INT" 0} \
       {"Ltr rxchan" 15 "INT" 0} \
       {"Ltr txchan" 15 "INT" 0} \
       {"Ltr mobilestate 0" 0 "INT" 0} \
       {"Ltr home 7" 7 "INT" 0} \
       {"Ltr rxchan" 7 "INT" 0} \
       {"Ltr txchan" 7 "INT" 0} \
       {"Ltr mode 0" 0 "INT" 0} \
       {"Ltr home 2" 2 "INT" 0} \
       {"Ltr rxchan" 7 "INT" 0} \
       {"Ltr txchan" 7 "INT" 0} \
       {"Ltr mode 1" 1 "INT" 0} \
       {"Ltr home 3" 3 "INT" 0} \
       {"Ltr rxchan" 7 "INT" 0} \
       {"Ltr txchan" 7 "INT" 0} \
       {"Ltr mode 2" 2 "INT" 0} \
       {"Ltr home 4" 4 "INT" 0} \
       {"Ltr rxchan" 7 "INT" 0} \
       {"Ltr txchan" 7 "INT" 0} \
       {"Ltr mode 3" 3 "INT" 0} \
       {"Ltr home 5" 5 "INT" 0} \
       {"Ltr rxchan" 7 "INT" 0} \
       {"Ltr txchan" 7 "INT" 0} \
       {"Ltr mode 4" 4 "INT" 0} \
       {"Ltr home 6" 6 "INT" 0} \
       {"Ltr rxchan" 7 "INT" 0} \
       {"Ltr txchan" 7 "INT" 0} \
       {"Ltr mode 5" 5 "INT" 0} \
       {"Ltr home 5" 5 "INT" 0} \
       {"Ltr rxchan" 5 "INT" 0} \
       {"Ltr txchan" 5 "INT" 0} ]

    set testLtrList13 [list \
       {"Ltr inuse -1 -forgive" 0x1F "INT" 0} \
       {"Ltr inuse 0x00" 0x00 "INT" 0} \
       {"Ltr inuse 0x01" 0x01 "INT" 0} \
       {"Ltr inuse 0x02" 0x02 "INT" 0} \
       {"Ltr inuse 0x04" 0x04 "INT" 0} \
       {"Ltr inuse 0x08" 0x08 "INT" 0} \
       {"Ltr inuse 0x10" 0x10 "INT" 0} \
       {"Ltr inuse 0x1F" 0x1F "INT" 0} \
       {"Ltr inuse 0x20 -forgive" 0x00 "INT" 0} \
       {"Ltr inuse 0x12345 -forgive" 0x05 "INT" 0} \
       {"Ltr inuse 0xABCDE -forgive" 0x1E "INT" 0} ]

    set testLtrList14 [list \
       {"Ltr mtries -1000 -forgive" 1 "INT" 0} \
       {"Ltr mtries -1 -forgive" 1 "INT" 0} \
       {"Ltr mtries 0 -forgive" 1 "INT" 0} \
       {"Ltr mtries 1" 1 "INT" 0} \
       {"Ltr mtries" 1 "INT" 0} \
       {"Ltr mtries 10" 10 "INT" 0} \
       {"Ltr mtries 99" 99 "INT" 0} \
       {"Ltr mtries 100" 100 "INT" 0} \
       {"Ltr mtries 101 -forgive" 100 "INT" 0} \
       {"Ltr mtries 1000 -forgive" 100 "INT" 0} \
       {"Ltr mobiletries -1000 -forgive" 1 "INT" 0} \
       {"Ltr mobiletries -1 -forgive" 1 "INT" 0} \
       {"Ltr mobiletries 0 -forgive" 1 "INT" 0} \
       {"Ltr mobiletries 1" 1 "INT" 0} \
       {"Ltr mobiletries" 1 "INT" 0} \
       {"Ltr mobiletries 10" 10 "INT" 0} \
       {"Ltr mobiletries 99" 99 "INT" 0} \
       {"Ltr mobiletries 100" 100 "INT" 0} \
       {"Ltr mobiletries 101 -forgive" 100 "INT" 0} \
       {"Ltr mobiletries 1000 -forgive" 100 "INT" 0} ]

    set testLtrList15 [list \
       {"Ltr mon -1 -forgive" 1 "INT" 0} \
       {"Ltr mon 0" 0 "INT" 0} \
       {"Ltr mon 1" 1 "INT" 0} \
       {"Ltr mon" 1 "INT" 0} \
       {"Ltr mon 2 -forgive" 1 "INT" 0} \
       {"Ltr mon off" 0 "INT" 0} \
       {"Ltr mon on" 1 "INT" 0} \
       {"Ltr monitor -1 -forgive" 1 "INT" 0} \
       {"Ltr monitor 0" 0 "INT" 0} \
       {"Ltr monitor 1" 1 "INT" 0} \
       {"Ltr monitor" 1 "INT" 0} \
       {"Ltr monitor 2 -forgive" 1 "INT" 0} \
       {"Ltr monitor off" 0 "INT" 0} \
       {"Ltr monitor on" 1 "INT" 0} ]








    set testLtrList22 [list \
       {"Ltr rxchan -1 -forgive" 1 "INT" 0} \
       {"Ltr rxchan 0 -forgive" 1 "INT" 0} \
       {"Ltr rxchan 1" 1 "INT" 0} \
       {"Ltr rxchan 5" 5 "INT" 0} \
       {"Ltr rxchan 13" 13 "INT" 0} \
       {"Ltr rxchan" 13 "INT" 0} \
       {"Ltr rxchan 31" 31 "INT" 0} \
       {"Ltr rxchan 32 -forgive" 31 "INT" 0} \
       {"Ltr rxchannel -1 -forgive" 1 "INT" 0} \
       {"Ltr rxchannel 0 -forgive" 1 "INT" 0} \
       {"Ltr rxchannel 1" 1 "INT" 0} \
       {"Ltr rxchannel 5" 5 "INT" 0} \
       {"Ltr rxchannel" 5 "INT" 0} \
       {"Ltr rxchannel 13" 13 "INT" 0} \
       {"Ltr rxchannel 31" 31 "INT" 0} \
       {"Ltr rxchannel 32 -forgive" 31 "INT" 0} ]

    set testLtrList23 [list \
       {"Ltr txchan -1 -forgive" 1 "INT" 0} \
       {"Ltr txchan 0 -forgive" 1 "INT" 0} \
       {"Ltr txchan 1" 1 "INT" 0} \
       {"Ltr txchan 5" 5 "INT" 0} \
       {"Ltr txchan 13" 13 "INT" 0} \
       {"Ltr txchan" 13 "INT" 0} \
       {"Ltr txchan 31" 31 "INT" 0} \
       {"Ltr txchan 32 -forgive" 31 "INT" 0} \
       {"Ltr txchannel -1 -forgive" 1 "INT" 0} \
       {"Ltr txchannel 0 -forgive" 1 "INT" 0} \
       {"Ltr txchannel 1" 1 "INT" 0} \
       {"Ltr txchannel 5" 5 "INT" 0} \
       {"Ltr txchannel" 5 "INT" 0} \
       {"Ltr txchannel 13" 13 "INT" 0} \
       {"Ltr txchannel 31" 31 "INT" 0} \
       {"Ltr txchannel 32 -forgive" 31 "INT" 0} ]

    set testLtrList25 [list \
       {"Ltr ptt -1 -forgive" 1 "INT" 0} \
       {"Ltr ptt 0" 0 "INT" 0} \
       {"Ltr ptt 1" 1 "INT" 0} \
       {"Ltr ptt" 1 "INT" 0} \
       {"Ltr ptt 2 -forgive" 1 "INT" 0} \
       {"Ltr ptt off" 0 "INT" 0} \
       {"Ltr ptt on" 1 "INT" 0} ]


    set testLtrList28 [list \
       {"Ltr rxfreq 1 1234.5 -forgive" 1234.5 "DBL" 0.001} \
       {"Ltr rxfreq 1" 1234.5 "DBL" 0.001} \
       {"Ltr rxfreq -1 333.1 -forgive" 1234.5 "DBL" 0.001} \
       {"Ltr rxfreq 0 333.1 -forgive" 1234.5 "DBL" 0.001} \
       {"Ltr rxfreq 1" 1234.5 "DBL" 0.001} \
       {"Ltr rxfreq 1 333.1" 333.1 "DBL" 0.001} \
       {"Ltr rxfreq 1" 333.1 "DBL" 0.001} \
       {"Ltr rxfreq 10 444.4" 444.4 "DBL" 0.001} \
       {"Ltr rxfreq 30 555.5" 555.5 "DBL" 0.001} \
       {"Ltr rxfreq 31 666.6" 666.6 "DBL" 0.001} \
       {"Ltr rxfreq 32 777.7 -forgive" 666.6 "DBL" 0.001} \
       {"Ltr rxfreq 30" 555.5 "DBL" 0.001} \
       {"Ltr rxfreq 31" 666.6 "DBL" 0.001} \
       {"Ltr rxfreq 10000 555.1 -forgive" 666.6 "DBL" 0.001} \
       {"Ltr rxfreq 1 11.11" 11.11 "DBL" 0.001} \
       {"Ltr rxfreq 2 22.22" 22.22 "DBL" 0.001} \
       {"Ltr rxfreq 3 33.33" 33.33 "DBL" 0.001} \
       {"Ltr rxfreq 4 44.44" 44.44 "DBL" 0.001} \
       {"Ltr rxfreq 2 0.001" 0.001 "DBL" 0.001} \
       {"Ltr rxfreq 3 0.0" 33.33 "DBL" 0.001} \
       {"Ltr rxfreq 2 -1.5 -forgive" 0.001 "DBL" 0.001} \
       {"Ltr rxfreq 3 -2000.0 -forgive" 33.33 "DBL" 0.001} \
       {"Ltr rxfreq 2 2799.99" 2799.99 "DBL" 0.001} \
       {"Ltr rxfreq 3 2800.0" 2800.0 "DBL" 0.001} \
       {"Ltr rxfreq 2 22.22" 22.22 "DBL" 0.001} \
       {"Ltr rxfreq 3 33.33" 33.33 "DBL" 0.001} \
       {"Ltr rxfreq 2 2800.1 -forgive" 2800.0 "DBL" 0.001} \
       {"Ltr rxfreq 3 3000.0 -forgive" 2800.0 "DBL" 0.001} \
       {"Ltr rxfreq 1" 11.11 "DBL" 0.001} \
       {"Ltr rxfreq 2" 2800.0 "DBL" 0.001} \
       {"Ltr rxfreq 3" 2800.0 "DBL" 0.001} \
       {"Ltr rxfreq 4" 44.44 "DBL" 0.001} \
       {"Ltr rxfrequency 1 111.111" 111.111 "DBL" 0.001} \
       {"Ltr rxfrequency 2 222.222" 222.222 "DBL" 0.001} \
       {"Ltr rxfrequency 13 345.678" 345.678 "DBL" 0.001} \
       {"Ltr rxfrequency 1" 111.111 "DBL" 0.001} \
       {"Ltr rxfrequency 2" 222.222 "DBL" 0.001} \
       {"Ltr rxfrequency 3" 2800.0 "DBL" 0.001} \
       {"Ltr rxfrequency 4" 44.44 "DBL" 0.001} ]

     set testLtrList30 [list \
       {"Ltr debug rx -1 -forgive" 3 "INT" 0} \
       {"Ltr debug rx 0" 0 "INT" 0} \
       {"Ltr debug rx 1" 1 "INT" 0} \
       {"Ltr debug rx" 1 "INT" 0} \
       {"Ltr debug rx 2 -forgive" 1 "INT" 0} \
       {"Ltr debug rx off" 0 "INT" 0} \
       {"Ltr debug rx on" 1 "INT" 0} \
       {"Ltr debug rcvltr -1 -forgive" 3 "INT" 0} \
       {"Ltr debug rcvltr 0" 0 "INT" 0} \
       {"Ltr debug rcvltr 1" 1 "INT" 0} \
       {"Ltr debug rcvltr" 1 "INT" 0} \
       {"Ltr debug rcvltr 2 -forgive" 1 "INT" 0} \
       {"Ltr debug rcvltr off" 0 "INT" 0} \
       {"Ltr debug rcvltr on" 1 "INT" 0} ]
 }

#****************************************************************************
# Name:        autoTest
# Parameters:  resultsLevel		Indicates the results display level
# Description: Executes automatic test on AFCounter commands
#****************************************************************************
 proc autoTest { resultsLevel } {
    variable testCaseList

    # Declare each test as a namespace variable
    foreach testName $testCaseList {
       variable $testName
    }

    setup 

    # Execute the test lists, and then destroy the test lists
    ::BIT::resetTestStats "cmdline_Ltr"
    foreach testName $testCaseList {
       ::BIT::runCmdLineTests "cmdline_Ltr" $testName
       unset $testName
    }

    if { ($resultsLevel == "Full") || ($resultsLevel == "Manual") } \
    {
       # Test commands with no data - requires visual inspection

       # Test string functions - requires visual inspection
#       ::BIT::putsTest [Ltr en -range]
       foreach cmd {rxchannel txchannel monitor ptt group } \
       {
          set temp [Ltr $cmd -range]
          ::BIT::putsTest "The range of <$cmd> is:  $temp"
       }

       # Test whether debug commands are enabled for user
#       ::BIT::putsTest [Ltr rcvltr 1]
#       ::BIT::putsTest [Ltr rx 1]
#       ::BIT::putsTest [Ltr xmtltr 1 2 3]
#       ::BIT::putsTest [Ltr xmt 1 2 3]

       ::BIT::putsTest [Ltr res]
       ::BIT::putsTest [Ltr resources]

       ::BIT::putsTest [Ltr ver 0]
       ::BIT::putsTest [Ltr version 1]
    }

    # Reset Ltr to typical values after completion of the test
 }

}

# Add this test to the list of tests
::BIT::addTest cmdline_Ltr

