#############################################################################
# COPYRIGHT (C) 2001, 2005  Aeroflex, Inc.                                  #
# --------------------------------------------------------------------------#
# FILE        : sys_Sinad.tcl                                               #
# PRODUCT     : FITS                                                        #
# AUTHOR      : Bruce Butterfield                                           #
# DESCRIPTION : System Test of Sinad / Distortion                           #
# --------------------------------------------------------------------------#
# Update      : Converted to BIT format for 1.9.1.4                         #
#############################################################################

package require Sinad
package require BBAudio
package require Generator
package require Fgen
package require BIT

#****************************************************************************
# Test variables
#****************************************************************************
namespace eval ::sys_SinadDist {
    variable testCaseList		;# List of test cases
    variable shortDesc			;# Test description - menu size
    variable longDesc			;# Test description - used by reports
    variable testTime			;# Test duration (Seconds - approximation)

    # Define the list of Sinad / Distortion tests
    set testCaseList [list sysSinadList1]

    # Declare each test as a namespace variable
    foreach testName $testCaseList {
       variable $testName
    }

    set shortDesc "Sinad system"
    set longDesc "Sinad/Dist system test"
    set testTime 8

 #****************************************************************************
 # Name:        setup
 # Parameters:  none
 # Description: Sets up a list of AF Counter tests
 #****************************************************************************
 proc setup { } {
    variable testCaseList

    # Declare each test as a namespace variable
    foreach testName $testCaseList {
       variable $testName
    }

    #
    # Define test cases
    #

    # Initial setup:  Freq = 154.6, RF = -50 db, Port = T/R
    #                 Modulation = Narrowband FM, M1 = 1000 khz @ 5.0 khz deviation
    #                 M2 = 1800 khz, with changing deviation
    #                 Sinad and Distortion are measured after each deviation change in M2
    #
    set sysSinadList1 {
      {"Generator freq 154.6 " 154.6 "DBL" 0.001} 
      {"Generator rflevel -50.0" -50.0 "DBL" 0.001} 
      {"Generator rfoutput t/r" 0 "INT" 0} 
      {"BBAudio mixer m1 user0 1" {7 4} "compareInts" 0}
      {"BBAudio mixer m2 user0 1" {8 4} "compareInts" 0}
      {"BBAudio route user0 sinadmtr" {13 16} "compareInts" 0}
      {"Sinad avg 1" 1 "INT" 0}
      {"Sinad trig continuous" 1 "INT" 0}
      {"Sinad en on" 3 "INT" 0}
      {"Sinad run on" 1 "INT" 0}
      {"Sinad filter 3" 3 "INT" 0}
      {"Sinad math on" 1 "INT" 0}
      {"Fgen mod3 1" 1 "INT" 0}
      {"Fgen freq3 1000.0" 1000.0 "DBL" 0.001}
      {"Fgen dev3 5.0" {1.0 10.0 5.0} "compareDbls" 0.001}
      {"Fgen mod4 1" 1 "INT" 0}
      {"Fgen freq4 1800.0" 1800.0 "DBL" 0.001}

      {"Fgen dev4 1.0" {0.2 2.0 1.0} "compareDbls" 0.001}
      {"after 1000" 0 "DELAY" 0}
      {"Sinad reading" {14.1 2.0 0.0} "compareDbls" 0.3}
      {"Dist reading" {20.0 2.0 0.0} "compareDbls" 0.3}

      {"Fgen dev4 2.0" {0.4 4.0 2.0} "compareDbls" 0.001}
      {"after 1000" 0 "DELAY" 0}
      {"Sinad reading" {8.6 2.0 0.0} "compareDbls" 0.3}
      {"Dist reading" {40.0 2.0 0.0} "compareDbls" 0.3}

      {"Fgen dev4 3.0" {0.6 6.0 3.0} "compareDbls" 0.001}
      {"after 1000" 0 "DELAY" 0}
      {"Sinad reading" {5.8 2.0 0.0} "compareDbls" 0.3}
      {"Dist reading" {60.0 2.0 0.0} "compareDbls" 0.3}

      {"Fgen dev4 4.0" {0.8 8.0 4.0} "compareDbls" 0.001}
      {"after 1000" 0 "DELAY" 0}
      {"Sinad reading" {4.2 2.0 0.0} "compareDbls" 0.3}
      {"Dist reading" {80.0 2.0 0.0} "compareDbls" 0.3}

      {"Fgen dev4 4.9" {0.98 9.8 4.9} "compareDbls" 0.001}
      {"after 1000" 0 "DELAY" 0}
      {"Sinad reading" {3.0 2.0 0.0} "compareDbls" 0.3}
      {"Dist reading" {98.0 2.0 0.0} "compareDbls" 0.3}

      {"Fgen dev4 5.1" {1.02 10.2 5.1} "compareDbls" 0.001}
      {"after 1000" 0 "DELAY" 0}
      {"Sinad reading" {3.0 2.0 0.0} "compareDbls" 0.3}
      {"Dist reading" {102.0 2.0 0.0} "compareDbls" 0.3}

      {"BBAudio mixer m1 user0 0" {7 4} "compareInts" 0}
      {"BBAudio mixer m2 user0 0" {8 4} "compareInts" 0}
      {"BBAudio route none sinadmtr" {-1 16} "compareInts" 0}
      }
 }

 #****************************************************************************
 # Name:        autoTest
 # Parameters:  resultsLevel		Indicates the results display level
 # Description: Executes automatic test on the Sinad /Distortion subsystem
 #****************************************************************************
 proc autoTest { resultsLevel } {
    variable testCaseList

    # Declare each test as a namespace variable
    foreach testName $testCaseList {
       variable $testName
    }

    setup 

    # Execute the test lists, and then destroy the test lists
    ::BIT::resetTestStats "sys_SinadDist"
    foreach testName $testCaseList {
       ::BIT::runCmdLineTests "sys_SinadDist" $testName
       unset $testName
    }

    # Reset Sinad to typical values after completion of the test
    Sinad en 3
    Sinad run 0
    Sinad filter 1
    Sinad avg 15
    Sinad trig 1
 }

}

# Add this test to the list of tests
::BIT::addTest sys_SinadDist

