#############################################################################
# COPYRIGHT (C) 2002, 2005  Aeroflex, Inc.                                  #
# --------------------------------------------------------------------------#
# FILE        : sys_AFCtr.tcl                                               #
# PRODUCT     : FITS                                                        #
# AUTHOR      : Bruce Butterfield                                           #
# DESCRIPTION : System Test of AF Counter                                   #
# --------------------------------------------------------------------------#
# Update      : Converted to BIT format for 1.9.1.4                         #
#############################################################################

package require AFCounter
package require BBAudio
package require Fgen
package require BIT

#****************************************************************************
# Test variables
#****************************************************************************
namespace eval ::sys_AFCtr {
    variable testCaseList		;# List of test cases
    variable shortDesc			;# Test description - menu size
    variable longDesc			;# Test description - used by reports
    variable testTime			;# Test duration (Seconds - approximation)

    # Define the list of AF Counter tests
    set testCaseList [list sysAFctrList1 sysAFctrList10 ]

    # Declare each test as a namespace variable
    foreach testName $testCaseList {
       variable $testName
    }

    set shortDesc "AFCtr system"
    set longDesc "AF Counter system test"
    set testTime 8

 #****************************************************************************
 # Name:        setup
 # Parameters:  none
 # Description: Sets up a list of AF Counter tests
 #****************************************************************************
 proc setup { } {
    variable testCaseList

    # Declare each test as a namespace variable
    foreach testName $testCaseList {
       variable $testName
    }

    #
    # Define test cases
    #

    # Initial setup:  AF Counter = 0.1 gate, Avg = 10
    #                 Modulation comes directly from FGENx  (1-4)
    #
    set sysAFctrList1 {
      {"Fgen sh1 sine" 0 "INT" 0} 
      {"Fgen lev1 1.0" {1.0 10.0 5.0} "compareDbls" 0.001} 
      {"Fgen mod1 1" 1 "INT" 0}
      {"BBAudio route fgen1 afcounter" {5 15} "compareInts" 0} 

      {"AFCounter trigger continuous" 1 "INT" 0} 
      {"AFCounter gate tenth" 0 "INT" 0} 
      {"AFCounter enable on" 3 "INT" 0} 
      {"AFCounter run on" 1 "INT" 0} 
      {"AFCounter avg 10" 10 "INT" 0}

      {"Fgen freq1 40.345" 40.345 "DBL" 0.001} 
      {"after 2000" 0 "DELAY" 0}
      {"AFCounter value" 40.345 "DBL" 0.1} 

      {"Fgen freq1 123.456" 123.456 "DBL" 0.001} 
      {"after 2000" 0 "DELAY" 0}
      {"AFCounter value" 123.5 "DBL" 0.1} 

      {"Fgen freq1 234.567" 234.567 "DBL" 0.001} 
      {"after 2000" 0 "DELAY" 0}
      {"AFCounter value" 234.6 "DBL" 0.1} 

      {"Fgen freq1 345.678" 345.678 "DBL" 0.001} 
      {"after 2000" 0 "DELAY" 0}
      {"AFCounter value" 345.7 "DBL" 0.1} 

      {"Fgen freq1 500.00" 500.00 "DBL" 0.001} 
      {"after 2000" 0 "DELAY" 0}
      {"AFCounter value" 500.00 "DBL" 0.1} 

      {"Fgen freq1 1000.00" 1000.00 "DBL" 0.001} 
      {"after 2000" 0 "DELAY" 0}
      {"AFCounter value" 1000.00 "DBL" 0.1} 

      {"Fgen freq1 2000.00" 2000.00 "DBL" 0.001} 
      {"after 2000" 0 "DELAY" 0}
      {"AFCounter value" 2000.00 "DBL" 0.1} 

      {"Fgen freq1 4000.00" 4000.00 "DBL" 0.001} 
      {"after 2000" 0 "DELAY" 0}
      {"AFCounter value" 4000.00 "DBL" 0.1} 

      {"Fgen freq1 8765.432" 8765.432 "DBL" 0.001} 
      {"after 2000" 0 "DELAY" 0}
      {"AFCounter value" 8765.4 "DBL" 1.0} 

      {"Fgen freq1 16000.0" 16000.0 "DBL" 0.001} 
      {"after 2000" 0 "DELAY" 0}
      {"AFCounter value" 16000.0 "DBL" 1.0} 

      {"Fgen freq1 19876.543" 19876.543 "DBL" 0.001} 
      {"after 2000" 0 "DELAY" 0}
      {"AFCounter value" 19876.543 "DBL" 1.0} 

      {"Fgen freq1 20000.0" 20000.0 "DBL" 0.001} 
      {"after 2000" 0 "DELAY" 0}
      {"AFCounter value" 20000.0 "DBL" 1.0} 

      {"BBAudio route none afcounter" {-1 15} "compareInts" 0} 
      {"Fgen mod1 0" 0 "INT" 0}
      }


   # Drift test - Verify a high accuracy that does not glitch over time
   #    - Use a 1 second gate time
   #    - Use a 100 max average (after allowing time for settling and sync)
   # NOTE: At least some of the delays in the setup before the first AFCounter
   # command are required to keep the system happy. This may need to be checked
   # out later.
   set sysAFctrList10 {
      {"BBAudio route fgen1 afcounter" {5 15} "compareInts" 0} 
      {"after 2000" 0 "DELAY" 0}
      {"Fgen freq1 20000.0" 20000.0 "DBL" 0.001} 
      {"after 2000" 0 "DELAY" 0}
      {"Fgen sh1 sine" 0 "INT" 0} 
      {"after 2000" 0 "DELAY" 0}
      {"Fgen lev1 1.0" {1.0 10.0 5.0} "compareDbls" 0.001} 
      {"after 2000" 0 "DELAY" 0}
      {"Fgen mod1 1" 1 "INT" 0}
      {"after 2000" 0 "DELAY" 0}

      {"AFCounter trigger continuous" 1 "INT" 0} 
      {"AFCounter gate second" 1 "INT" 0} 
      {"AFCounter enable on" 3 "INT" 0} 
      {"AFCounter run on" 1 "INT" 0} 
      {"AFCounter avg 1" 1 "INT" 0}
      {"AFCounter avg 100" 100 "INT" 0}
      {"after 2000" 0 "DELAY" 0}

      {"AFCounter value" 20000.0 "DBL" 1.0} 
      {"after 2000" 0 "DELAY" 0}
      {"AFCounter value" 20000.0 "DBL" 1.0} 
      {"after 2000" 0 "DELAY" 0}
      {"AFCounter value" 20000.0 "DBL" 1.0} 
      {"after 2000" 0 "DELAY" 0}
      {"AFCounter value" 20000.0 "DBL" 1.0} 
      {"after 2000" 0 "DELAY" 0}
      {"AFCounter value" 20000.0 "DBL" 1.0} 
      {"after 2000" 0 "DELAY" 0}
      {"AFCounter value" 20000.0 "DBL" 0.1} 
      {"after 2000" 0 "DELAY" 0}
      {"AFCounter value" 20000.0 "DBL" 0.1} 
      {"after 4000" 0 "DELAY" 0}
      {"AFCounter value" 20000.0 "DBL" 0.1} 
      {"after 4000" 0 "DELAY" 0}
      {"AFCounter value" 20000.0 "DBL" 0.1} 
      {"after 4000" 0 "DELAY" 0}
      {"AFCounter value" 20000.0 "DBL" 0.1} 
      {"after 4000" 0 "DELAY" 0}
      {"AFCounter value" 20000.0 "DBL" 0.1} 
      {"after 4000" 0 "DELAY" 0}
      {"AFCounter value" 20000.0 "DBL" 0.1} 
      {"after 4000" 0 "DELAY" 0}
      {"AFCounter value" 20000.0 "DBL" 0.1} 
      {"after 4000" 0 "DELAY" 0}
      {"AFCounter value" 20000.0 "DBL" 0.1} 
      {"after 4000" 0 "DELAY" 0}
      {"AFCounter value" 20000.0 "DBL" 0.1} 
      {"after 4000" 0 "DELAY" 0}
      {"AFCounter value" 20000.0 "DBL" 0.1} 
      {"after 4000" 0 "DELAY" 0}
      {"AFCounter value" 20000.0 "DBL" 0.1} 
      {"after 4000" 0 "DELAY" 0}
      {"AFCounter value" 20000.0 "DBL" 0.1} 
      {"after 4000" 0 "DELAY" 0}
      {"AFCounter value" 20000.0 "DBL" 0.1} 
      {"after 4000" 0 "DELAY" 0}
      {"AFCounter value" 20000.0 "DBL" 0.1} 
      {"after 4000" 0 "DELAY" 0}
      {"AFCounter value" 20000.0 "DBL" 0.1} 
      {"after 4000" 0 "DELAY" 0}
      {"AFCounter value" 20000.0 "DBL" 0.1} 
      {"after 4000" 0 "DELAY" 0}
      {"AFCounter value" 20000.0 "DBL" 0.1} 
      {"after 4000" 0 "DELAY" 0}
      {"AFCounter value" 20000.0 "DBL" 0.1} 
      {"after 4000" 0 "DELAY" 0}
      {"AFCounter value" 20000.0 "DBL" 0.1} 
      {"after 4000" 0 "DELAY" 0}
      {"AFCounter value" 20000.0 "DBL" 0.1} 
      {"after 4000" 0 "DELAY" 0}
      {"AFCounter value" 20000.0 "DBL" 0.1} 
      {"after 4000" 0 "DELAY" 0}
      {"AFCounter value" 20000.0 "DBL" 0.1} 
      {"after 4000" 0 "DELAY" 0}
      {"AFCounter value" 20000.0 "DBL" 0.1} 
      {"after 4000" 0 "DELAY" 0}
      {"AFCounter value" 20000.0 "DBL" 0.1} 
      {"after 4000" 0 "DELAY" 0}
      {"AFCounter value" 20000.0 "DBL" 0.1} 
      {"after 4000" 0 "DELAY" 0}
      {"AFCounter value" 20000.0 "DBL" 0.1} 
      {"after 4000" 0 "DELAY" 0}
      {"AFCounter value" 20000.0 "DBL" 0.1} 
      {"after 4000" 0 "DELAY" 0}
      {"AFCounter value" 20000.0 "DBL" 0.1} 
      {"after 4000" 0 "DELAY" 0}
      {"AFCounter value" 20000.0 "DBL" 0.1} 
      {"after 4000" 0 "DELAY" 0}
      {"AFCounter value" 20000.0 "DBL" 0.1} 
      {"after 4000" 0 "DELAY" 0}
      {"AFCounter value" 20000.0 "DBL" 0.1} 
      {"after 4000" 0 "DELAY" 0}
      {"AFCounter value" 20000.0 "DBL" 0.1} 
      {"after 4000" 0 "DELAY" 0}
      {"AFCounter value" 20000.0 "DBL" 0.1} 
      {"after 4000" 0 "DELAY" 0}
      {"AFCounter value" 20000.0 "DBL" 0.1} 
      {"after 4000" 0 "DELAY" 0}
      {"AFCounter value" 20000.0 "DBL" 0.1} 
      {"after 4000" 0 "DELAY" 0}
      {"AFCounter value" 20000.0 "DBL" 0.1} 
      {"after 4000" 0 "DELAY" 0}
      {"AFCounter value" 20000.0 "DBL" 0.1} 
      {"after 4000" 0 "DELAY" 0}
      {"AFCounter value" 20000.0 "DBL" 0.1} 
      {"after 4000" 0 "DELAY" 0}
      {"AFCounter value" 20000.0 "DBL" 0.1} 
      {"after 4000" 0 "DELAY" 0}
      {"AFCounter value" 20000.0 "DBL" 0.1} 
      {"after 4000" 0 "DELAY" 0}
      {"AFCounter value" 20000.0 "DBL" 0.1} 
      {"after 4000" 0 "DELAY" 0}
      {"AFCounter value" 20000.0 "DBL" 0.1} 
      {"after 4000" 0 "DELAY" 0}
      {"AFCounter value" 20000.0 "DBL" 0.1} 
      {"after 4000" 0 "DELAY" 0}
      {"AFCounter value" 20000.0 "DBL" 0.1} 
      {"after 4000" 0 "DELAY" 0}
      {"AFCounter value" 20000.0 "DBL" 0.1} 
      {"after 4000" 0 "DELAY" 0}
      {"AFCounter value" 20000.0 "DBL" 0.1} 
      {"after 4000" 0 "DELAY" 0}
      {"AFCounter value" 20000.0 "DBL" 0.1} 
      {"after 4000" 0 "DELAY" 0}
      {"AFCounter value" 20000.0 "DBL" 0.1} 
      {"after 4000" 0 "DELAY" 0}
      {"AFCounter value" 20000.0 "DBL" 0.1} 
      {"after 4000" 0 "DELAY" 0}
      {"AFCounter value" 20000.0 "DBL" 0.1} 
      {"after 4000" 0 "DELAY" 0}
      {"AFCounter value" 20000.0 "DBL" 0.1} 
      {"after 4000" 0 "DELAY" 0}
      {"AFCounter value" 20000.0 "DBL" 0.1} 
      {"after 4000" 0 "DELAY" 0}
      {"AFCounter value" 20000.0 "DBL" 0.1} 
      {"after 4000" 0 "DELAY" 0}
      {"AFCounter value" 20000.0 "DBL" 0.01} 
      {"after 4000" 0 "DELAY" 0}
      {"AFCounter value" 20000.0 "DBL" 0.01} 
      {"after 4000" 0 "DELAY" 0}
      {"AFCounter value" 20000.0 "DBL" 0.01} 
      {"after 4000" 0 "DELAY" 0}
      {"AFCounter value" 20000.0 "DBL" 0.01} 
      {"after 4000" 0 "DELAY" 0}
      {"AFCounter value" 20000.0 "DBL" 0.01} 
      {"after 4000" 0 "DELAY" 0}
      {"AFCounter value" 20000.0 "DBL" 0.01} 

      {"BBAudio route none afcounter" {-1 15} "compareInts" 0} 
      {"Fgen mod1 0" 0 "INT" 0}
      }
 }

 #****************************************************************************
 # Name:        autoTest
 # Parameters:  resultsLevel		Indicates the results display level
 # Description: Executes automatic test on the AF Counter subsystem
 #****************************************************************************
 proc autoTest { resultsLevel } {
    variable testCaseList

    # Declare each test as a namespace variable
    foreach testName $testCaseList {
       variable $testName
    }

    setup 

    # Execute the test lists, and then destroy the test lists
    ::BIT::resetTestStats "sys_AFCtr"
    foreach testName $testCaseList {
       ::BIT::runCmdLineTests "sys_AFCtr" $testName
       unset $testName
    }

    # Reset AF Counter to typical values after completion of the test
 }

}

# Add this test to the list of tests
::BIT::addTest sys_AFCtr

