#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C) 2001        DESIGN ENGINEERING      #
# --------------------------------------------------------------------------#
# FILE        : testVarInit.tcl                                             #
# PRODUCT     : FITS                                                        #
# AUTHOR      : Bruce Butterfield                                           #
# DESCRIPTION : Global variables used in FITS autotests                     #
#############################################################################

proc initTestVar { } {
   global      t_color
   global      testStatus 
   global      autoTestList 
   global      testMode                      

   #
   # Define standard test colors
   #
   set t_color(bkgnd) gray
   set t_color(keys)  cornsilk
   set t_color(menu)  cornsilk
   set t_color(title) green
   set t_color(text)  black

   #
   # Define test status variables
   #
   set testStatus(totalCases) 0
   set testStatus(moduleCases) 0
   set testStatus(totalErrors) 0
   set testStatus(moduleErrors) 0
   set testStatus(moduleName) "No Test"
   set testStatus(reportLevel) 2
   set testStatus(device) 1
   set testStatus(visual) 1

   #
   # Define automatic tests for the menu. The fields in the list are:
   #    - The menu index:  0 = Cascading menu, 1 = Test selection information
   #    - The test name on the menu
   #    - The test proc name
   #    - The TCL file which contains the test proc
   #
   set autoTestList [list \
      {0 "Meters"} \
      {1 "Afctr" "autoTestAfctr" "/system/self_test/1001.test_Afctr.tcl" } \
      {1 "C4fm" "autoTestC4FM" "/system/self_test/1002.test_Civ_fm.tcl" } \
      {1 "FMDev" "autoTestFmdev" "/system/self_test/1003.test_Fmdev.tcl" } \
      {1 "Sinad/Dist" "autoTestSinad" "/system/self_test/1009.test_Sinad.tcl" } \
      {1 "RFError" "autoTestRferror" "/system/self_test/1010.test_RFError.tcl" } \
      {1 "Dvm" "autoTestDvm" "/system/self_test/1011.test_Dvm.tcl" } \
      {1 {Power (wideband)} "autoTestPower" "/system/self_test/1014.test_Power.tcl" } \
      {1 "Rlmtr" "autoTestRLmtr" "/system/self_test/1016.test_RLmtr.tcl" } \
      {0 "Engines"} \
      {1 "Fgen"  "autoTestFgen"  "/system/self_test/1000.test_Fgen.tcl" } \
      {1 "Scope" "autoTestScope" "/system/self_test/1007.test_Scope.tcl" } \
      {1 "Generator" "autoTestGenerator" "/system/self_test/1005.test_Generator.tcl" } \
      {1 "Receiver" "autoTestReceiver" "/system/self_test/1006.test_Receiver.tcl" } \
      {1 {BB_Audio} "autoTestBBAudio" "/system/self_test/1008.test_BBAudio.tcl" } \
      {1 "P25" "autoTestP25" "/system/self_test/1013.test_P25.tcl" } \
      {1 "Analyzer" "autoTestAnalyzer" "/system/self_test/1015.test_Analyzer.tcl" } \
      {0 "New"} \
      {1 "BER" "autoTestBer" "/system/self_test/1012.test_Ber.tcl" } \
      {1 "DCS" "autoTestDcs" "/system/self_test/1004.test_Dcs.tcl" } \
      {0 "System"} \
      {1 "Sinad" "autoTestSysSinad" "/system/self_test/2000.sys_Sinad.tcl" } \
      {1 "AFcounter" "autoTestSysAFctr" "/system/self_test/2001.sys_AFctr.tcl" } ]

   #
   # Define test mode menu variables
   #
   set testMode(current) AUTO           ;# Current test mode
   set testMode(avail) {AUTO MAN}       ;# Available test modes

}

#
# Define test menus
#
#set tileTestList [list \
#   {"FGEN1" "pack_tile_fgen_small_1" "$utilDirPath/3000.fgen_tile.tcl"} \
#   {"FGEN2" "pack_tile_fgen_small_2" "$utilDirPath/3000.fgen_tile.tcl"} ]



