#############################################################################
#                                                                           #
# IFR SYSTEMS, INC        COPYRIGHT (C)1998-2001    DESIGN ENGINEERING      #
#                                                                           #
# --------------------------------------------------------------------------#
#                                                                           #
# FILE        : 2099.IntroUI.tcl                                            #
#                                                                           #
# PRODUCT     : FITS                                                        #
#                                                                           #
# AUTHOR      : Preston McCormick                                           #
#                                                                           #
# DESCRIPTION : Introduction User Interface                                 #
#                                                                           #
#############################################################################

namespace eval IntroUI \
{
  # Global variables
  variable LongName       "intro"
  variable ShortName      "Introduction"
  variable Size           Full

  variable tile_class     "INTRO"

   

  #********************************************************************
  #
  #       Name: Create  
  #
  # Parameters:
  #
  # Desciption: 
  #
  #********************************************************************
  proc Create { tile_path } \
  {
    Debug::Puts IntroUI 2 "IntroUI::Create:  $tile_path"

    variable  LongName
    variable  tile_class
    
    set top_level [winfo toplevel $tile_path]
 
    set tile_path  [name_child  $top_level $LongName]

    if { ![winfo exists $tile_path] } \
    {
      image create photo image_large_logo -file "/system/logo/logo.gif"
      
      set tile_path  [::Screens::create_tile  $tile_path  intro  $tile_class]
    }  

    return $tile_path
  }


  #******************************************************************** 
  #
  #       Name: Map
  #
  # Parameters: 
  #
  # Desciption: 
  #
  #********************************************************************
  proc Map { tile_path } \
  {
    after cancel "::IntroUI::Cleanup_core $tile_path"

    Debug::Puts IntroUI 2 "IntroUI::Map:  $tile_path"

    set tile_path  [get_tile  $tile_path]

    
    # SOFTKEY 2
    ::View::AssignSoftkey $tile_path  2  "Generator"  "::Screens::Show $tile_path Gen_Screen"

    # SOFTKEY 3
    ::View::AssignSoftkey $tile_path  3  "Receiver"  "::Screens::Show $tile_path Rec_Screen"

    # SOFTKEY 4
    ::View::AssignSoftkey $tile_path  4  "Duplex"  "::Screens::Show $tile_path Duplex_Screen"

    # SOFTKEY 5
    ::View::AssignSoftkey $tile_path  5  "Function Generator" "::Screens::Show $tile_path AFGen_Screen"
    
    # SOFTKEY 6
    ::View::AssignSoftkey $tile_path  6  "Scope"  "::Screens::Show $tile_path Scope_Screen"

    # SOFTKEY 7
    ::View::AssignSoftkey $tile_path  7  "Spectrum Analyzer" "::Screens::Show $tile_path Analyzer_Screen"

    raise  $tile_path
    
    
    return
  }


  #********************************************************************
  #
  #       Name: Cleanup
  #
  # Parameters: 
  #
  # Desciption: 
  #
  #********************************************************************
  proc Delete_image {} \
  {
     if {[lsearch [image names] image_large_logo] != -1 } \
     {
        image delete image_large_log
     }
  }
  proc Cleanup { tile_path } \
  {
    Debug::Puts IntroUI 2 "IntroUI::Cleanup:  $tile_path"

    # this is a pig of a tile, so we destroy it every time
    after 1000 "::IntroUI::Cleanup_core $tile_path"
    
    return
    
  }
  proc Cleanup_core { tile_path } \
  {
    if [winfo exists $tile_path] \
    {
       catch {destroy $tile_path}
    }
    if {[lsearch [image names] image_large_logo] != -1 } \
    {
        catch {image delete image_large_logo}
    }  
  }
  
  #********************************************************************
  #
  #       Name: Unmap
  #
  # Parameters: 
  #
  # Desciption: 
  #
  #********************************************************************
  proc Unmap { tile_path } \
  {
    Debug::Puts IntroUI 2 "IntroUI::Unmap:  $tile_path"

    Cleanup  $tile_path
      
    return
  }


  #********************************************************************
  #
  #       Name: Destroy
  #
  # Parameters: 
  #
  # Desciption: 
  #
  #********************************************************************
  proc Destroy { tile_path } \
  {
    Debug::Puts IntroUI 2 "IntroUI::Destroy: $tile_path"

    Cleanup  $tile_path
    
    return
  }


  #********************************************************************
  #
  #       Name: Register
  #
  # Parameters: 
  #
  # Desciption: 
  #
  #********************************************************************
  proc Register { tile_path } \
  {
    Debug::Puts IntroUI 2 "IntroUI::Register:  $tile_path"


    return
  }


  ################################################################  
  #
  #  User Interface support procedures
  #
  ################################################################

  #***************************************************************************
  #
  #       Name: get_tile
  #
  # Parameters: any child widget of the tile
  #
  # Description: get the name of the tile's parent frame
  #
  #***************************************************************************
  proc get_tile { widget } \
  {
     while { 1 } \
     {
       switch -- [winfo class $widget] \
       {
         "INTRO" { return $widget}
       }
        
       set widget [winfo parent $widget]
     }
     
  }
  
  

};  # End of IntroUI namespace

