#############################################################################
#                                                                           #
# IFR SYSTEMS, INC        COPYRIGHT (C) 2002      DESIGN ENGINEERING        #
# --------------------------------------------------------------------------#
#                                                                           #
# FILE        : fgen_UI.tcl                                                 #
#                                                                           #
# PRODUCT     : FITS                                                        #
#                                                                           #
# AUTHOR      : Bruce Butterfield                                           #
#                                                                           #
# DESCRIPTION : Code for the FGEN control tiles                             #
#                                                                           #
#############################################################################

package require FGen

package require Tone_Signaling

package require ToneRemote

package require DTMF

package require DCS

package require CTCSS


namespace eval FGen_SmallUI \
{
  variable  LongName      "fgen"
  variable  ShortName     "Function Generators"
  variable  Size          Eighth_v
  
  variable  tile_class    SMALL_FGEN
  variable  tile_name     ""

  
  #***************************************************************************
  #
  #        Name: Create  
  #
  #  Parameters: parent_path - the tile path of the parent tile
  #
  # Description: Creates the tile
  #
  #***************************************************************************
  proc Create { parent_path } \
  {
    #puts "FGen_SmallUI::Create:  $parent_path"

    variable  LongName
    variable  tile_class

    set top_level [winfo toplevel $parent_path]
   
    set tile_path  [name_child  $top_level  $LongName]

    
    if { ![winfo exists $tile_path] } \
    {
      set tile_path  [Screens::create_tile  $tile_path  $LongName  $tile_class]
    }
    
    return $tile_path
  }

};  # End of FGen_SmallUI namespace



namespace eval FGen_QtrUI \
{
  # This data is used for state information in the FGEN tiles
  variable  LongName      "qtr_fgen"
  variable  ShortName     "Function Generator"
  variable  Size          Quarter
  
  variable  tile_class    QTR_FGEN
  variable  tile_name     ""


  #***************************************************************************
  #
  #        Name: Create
  #
  #  Parameters: parent_path - the tile path of the parent tile
  #
  # Description: Creates the tile
  #
  #***************************************************************************
  proc Create { parent_path } \
  {
    #puts "FGen_QtrUI::Create:  $parent_path"

    variable  LongName
    variable  tile_class

    set top_level [winfo toplevel $parent_path]
   
    set tile_path  [name_child  $top_level  $LongName]

    if { ![winfo exists $tile_path] } \
    {
      set tile_path  [Screens::create_tile  $tile_path  $LongName  $tile_class]
    }
    
    return $tile_path
  }

  
};  # End of the FGen_QtrUI namespace



namespace eval FGenUI \
{
  # This data is used for state information in the FGEN tiles
  variable  Softkeys
  
  variable  last_focus    ""
  variable  temp_string   ""
  variable  please_wait   ""
  
  variable  fgen_mode
  
  set fgen_mode(1)  "OFF"
  set fgen_mode(2)  "OFF"
  
  variable  StateUI
  
  set  StateUI(dtmf_mode_menu)   { "CONT"  "BURST"  "KEY PAD" }

  set  StateUI(tsig_mode_menu)   { "CONT"  "BURST" }

  
  #***************************************************************************
  #
  #        Name: Map
  #
  #  Parameters: lun - Logical Unit Number for the function generator
  #
  # Description: Handles the <Map> event for the tile
  #
  #***************************************************************************
  proc Map { tile_path  lun } \
  { 
    #puts "FGenUI::Map:  $tile_path,  $lun"
    
    set tile_path  [get_tile  $tile_path]


    trace variable  ::FGen::State($lun,fgen_mode)  w  \
                             "FGenUI::update_fgen_mode  $tile_path  $lun  ON"

    trace variable  ::FGen::State($lun,shape)  w  \
                             "FGenUI::reset_tone_level_units $tile_path $lun"

    if { $lun == 1 || $lun == 2 } \
    {
       trace variable  ::FGen::State(level_units,$lun)  w  \
                             "FGenUI::update_tone_level  $tile_path  $lun"
    }

    # Update the tile to the current setting
    update_fgen_mode_core  $tile_path  $lun  ON
    update_tone_level $tile_path $lun

  }
  
  
  #***************************************************************************
  #
  #        Name: Cleanup
  #
  #  Parameters: lun - Logical Unit Number for the function generator
  #
  # Description: Cleans up the tile setup before the tile is destroyed
  #
  #***************************************************************************
  proc Cleanup { tile_path  lun } \
  { 
    #puts "FGenUI::Cleanup:  $tile_path,  $lun"

    set tile_path  [get_tile  $tile_path]
    

    trace vdelete  ::FGen::State($lun,fgen_mode)  w  \
                             "FGenUI::update_fgen_mode  $tile_path  $lun  ON"

    trace vdelete  ::FGen::State($lun,shape)  w  \
                             "FGenUI::reset_tone_level_units $tile_path $lun"

    if { $lun == 1 || $lun == 2 } \
    {
       trace vdelete  ::FGen::State(level_units,$lun)  w  \
                             "FGenUI::update_tone_level  $tile_path  $lun"
    }

    set class [winfo class $tile_path]

    if { $class == "QTR_FGEN" } \
    {
      if { $lun == 1 } \
      {
        View::ClearSoftkey $tile_path 2
        View::ClearSoftkey $tile_path 3
        View::ClearSoftkey $tile_path 4
      }


      if { $lun == 2 } \
      {
        View::ClearSoftkey $tile_path 5
        View::ClearSoftkey $tile_path 6
        View::ClearSoftkey $tile_path 7
      }
    }
    
    return
  }


  #***************************************************************************
  #
  #        Name: Unmap
  #
  #  Parameters: tile_path - tile path of the invoking widget
  #              lun       - Logical Unit Number for the function generator
  #
  # Description: Handles the <Unmap> event for the tile
  #
  #***************************************************************************
  proc Unmap { tile_path  lun } \
  { 
    #puts "FGenUI::Unmap:  $tile_path,  $lun"

    Cleanup  $tile_path  $lun
     
    return
  }
  

  #***************************************************************************
  #
  #        Name: Destroy
  #
  #  Parameters: lun - Logical Unit Number for the function generator
  #
  # Description: Handles the <Destroy> event for the tile
  #
  #***************************************************************************
  proc Destroy { tile_path  lun } \
  { 
    #puts "FGenUI::Destroy:  $tile_path,  $lun"

    Cleanup  $tile_path  $lun

    return
  }


  #***************************************************************************
  #
  #        Name: Register
  #
  #  Parameters:
  #
  # Description: 
  #
  #***************************************************************************
  proc Register { } \
  { 
    #puts "FGenUI::Register"
  }


  #***************************************************************************
  #
  #        Name: get_tile
  #
  #  Parameters: widget - any child widget of the tile
  #
  # Description: Get the name of the tile's parent frame
  #
  #***************************************************************************
  proc get_tile { widget } \
  {
    while { 1 } \
    {
      switch -- [winfo class $widget] \
      {
        "SMALL_FGEN"    -
        "QTR_FGEN"      { return $widget }
      }
       
      set widget [winfo parent $widget]
    }
    
    return
  }


  #**************************************************************************
  #
  #        Name: set_fgen_mode
  #
  #  Parameters: lun - Logical Unit Number for the function generator
  #
  # Description: Sets the mode of the Function Generator
  #
  #**************************************************************************
  proc set_fgen_mode { lun } \
  {
    #puts "set_fgen_mode:  $lun"
    
    variable  fgen_mode

    switch -- $fgen_mode($lun) \
    {
      "OFF"    { after idle "FGen::setMode  $lun  0"  }
       
      "TONE"   { after idle "FGen::setMode  $lun  1"  }
    
      "TREM"   { after idle "FGen::setMode  $lun  3"  }
    
      "DTMF"   { after idle "FGen::setMode  $lun  4"  }
    
      "CTCSS"  { after idle "FGen::setMode  $lun  5"  }
    
      "DCS"    { after idle "FGen::setMode  $lun  6"  }
    
      "TSIG"   { after idle "FGen::setMode  $lun  7"  }

      "SINC"   { after idle "FGen::setMode  $lun  9"  }

      "ARB"    { after idle "FGen::setMode  $lun  10" }
      
      "SWEEP"  { after idle "FGen::setMode  $lun  11" }
    }

    # Procedure 'FGenUI::tone_send_seq'     called in 'FGen::setMode' 
    # Procedure 'CTCSS_UI::ctcss_send_seq'  called in 'FGen::setMode' 
    # Procedure 'DCS_UI::dcs_start'         called in 'FGen::setMode' 
    # Procedure 'SincUI::send_sinc_seq'     called in 'FGen::setMode' 
    # Procedure 'ArbitraryUI::send_arb_seq' called in 'FGen::setMode' 
    # Procedure 'GeneratorUI::p25_send_seq' called in 'FGen::setMode' 

    return
  }


  #****************************************************************************
  #
  #        Name: update_fgen_mode
  #
  #  Parameters: tile_path    - tile path of the invoking widget
  #              lun          - Logical Unit Number for the function generator
  #              display_mode - display mode: ON or OFF
  #
  # Description: Updates the display to the function generator mode
  #
  #****************************************************************************
  proc update_fgen_mode { tile_path  lun  display_mode  args } \
  {
     set cmd [namespace code "update_fgen_mode_core $tile_path $lun $display_mode"]
     after cancel $cmd
     after idle $cmd
  }
  proc update_fgen_mode_core { tile_path  lun  display_mode } \
  {
    #puts "update_fgen_mode:  $tile_path,  $lun,  $display_mode"
    
    variable  StateUI
    variable  fgen_mode
    
    set tile_path  [get_tile  $tile_path]
      
    set class  [winfo class $tile_path]
   
    foreach subframe [grid slaves ${tile_path}.fgen$lun] \
    {
       if [string match ${tile_path}.fgen$lun.frm* $subframe] \
       {
          grid remove $subframe
       }
    }
  
    
    set fgen_mode($lun)  $::FGen::State($lun,fgen_mode)
    
    
    switch -- $fgen_mode($lun) \
    {
      "OFF"   { if { $lun == 3 || $lun == 4 } \
                {
                  puts "lun:  $lun reference in 'FGenUI::update_fgen_mode'"
                }
                
                if { $class == "QTR_FGEN" } \
                {
                  if { $lun == 1 } { set softkey  2 }
                  if { $lun == 2 } { set softkey  3 }
                 
                  # SOFTKEY 2/3
                  View::AssignSoftkey  $tile_path  $softkey  "AFGEN $lun\nOFF"  \
                         "${tile_path}.fgen${lun}.mnu_mode.menu  invoke  TONE"
               
                  # SOFTKEY 4
                  View::AssignSoftkey  $tile_path  4  ""  ""
                 
                  # SOFTKEY 5
                  View::AssignSoftkey  $tile_path  5  ""  ""
                 
                  # SOFTKEY 6
                  View::AssignSoftkey  $tile_path  6  ""  ""
                 
                  # SOFTKEY 7
                  View::AssignSoftkey  $tile_path  7  ""  ""
                }
              }
    
      "TONE"  { if { $lun == 3 || $lun == 4 } \
                {
                  puts "lun:  $lun reference in 'FGenUI::update_fgen_mode'"
                }
               
                Screens::position_widget ${tile_path}.fgen$lun.frm_tone
                update_tone_level $tile_path $lun
                
                # Procedure 'FGenUI::tone_send_seq' called in 'FGen::setMode'
               
                if { $class == "QTR_FGEN" } \
                {
                  if { $lun == 1 } { set softkey  2 }
                  if { $lun == 2 } { set softkey  3 }
               
                  # SOFTKEY 2/3
                  View::AssignSoftkey  $tile_path  $softkey  "AFGEN $lun\nTONE"  \
                         "${tile_path}.fgen${lun}.mnu_mode.menu  invoke  TREM"
               
                  # SOFTKEY 4
                  View::AssignSoftkey  $tile_path  4  ""  ""
               
                  # SOFTKEY 5
                  View::AssignSoftkey  $tile_path  5  ""  ""
               
                  # SOFTKEY 6
                  View::AssignSoftkey  $tile_path  6  ""  ""
               
                  # SOFTKEY 7
                  View::AssignSoftkey  $tile_path  7  ""  ""
                }
              }
  
     "TREM"   { if { $lun == 3 || $lun == 4 } \
                {
                  puts "lun:  $lun reference in 'FGenUI::update_fgen_mode'"
                }
               
                Screens::position_widget ${tile_path}.fgen$lun.frm_tone_rem
                
                if { $class == "QTR_FGEN" } \
                {
                  if { $lun == 1 } { set softkey  2 }
                  if { $lun == 2 } { set softkey  3 }
               
                  # SOFTKEY 2/3
                  View::AssignSoftkey  $tile_path  $softkey  "AFGEN $lun\nTONE REM"  \
                         "${tile_path}.fgen${lun}.mnu_mode.menu  invoke  DTMF"
               
                  # SOFTKEY 4
                  View::AssignSoftkey  $tile_path  4  ""  ""
               
                  # SOFTKEY 5
                  View::AssignSoftkey  $tile_path  5  ""  ""
               
                  # SOFTKEY 6
                  View::AssignSoftkey  $tile_path  6  ""  ""
               
                  # SOFTKEY 7
                  View::AssignSoftkey  $tile_path  7  ""  ""
                }
              }

     "DTMF"   { if { $lun == 3 || $lun == 4 } \
                {
                  puts "lun:  $lun reference in 'FGenUI::update_fgen_mode'"
                }
               
                Screens::position_widget ${tile_path}.fgen$lun.frm_dtmf
                
                if { $class == "QTR_FGEN" } \
                {
                  if { $lun == 1 } { set softkey  2 }
                  if { $lun == 2 } { set softkey  3 }
               
                  # SOFTKEY 2/3
                  View::AssignSoftkey  $tile_path  $softkey  "AFGEN $lun\nDTMF"  \
                         "${tile_path}.fgen${lun}.mnu_mode.menu  invoke  CTCSS"
               
                  # SOFTKEY 4 (Letter A)
                  View::AssignSoftkey  $tile_path  4  "A"  ""
               
                  # SOFTKEY 5 (Letter B)
                  View::AssignSoftkey  $tile_path  5  "B"  ""
               
                  # SOFTKEY 6 (Letter C)
                  View::AssignSoftkey  $tile_path  6  "C"  ""
               
                  # SOFTKEY 7 (Letter D)
                  View::AssignSoftkey  $tile_path  7  "D"  ""
                }
              }

     "CTCSS"  { if { $lun == 3 || $lun == 4 } \
                {
                  puts "lun:  $lun reference in 'FGenUI::update_fgen_mode'"
                }
               
                Screens::position_widget ${tile_path}.fgen$lun.frm_ctcss
               
                # Procedure 'CTCSS_UI::ctcss_send_seq' called in 'FGen::setMode' 

                if { $class == "QTR_FGEN" } \
                {
                  if { $lun == 1 } { set softkey  2 }
                  if { $lun == 2 } { set softkey  3 }
               
                  # SOFTKEY 2/3
                  View::AssignSoftkey  $tile_path  $softkey  "AFGEN $lun\nCTCSS"  \
                           "${tile_path}.fgen${lun}.mnu_mode.menu  invoke  DCS"
               
                  # SOFTKEY 4
                  View::AssignSoftkey  $tile_path  4  ""  ""
               
                  # SOFTKEY 5
                  View::AssignSoftkey  $tile_path  5  ""  ""
               
                  # SOFTKEY 6
                  View::AssignSoftkey  $tile_path  6  ""  ""
               
                  # SOFTKEY 7
                  View::AssignSoftkey  $tile_path  7  ""  ""
                }

                # Update the CTCSS code frequency
                CTCSS_UI::ctcss_set_reed_code  $lun
              }

     "DCS"    { if { $lun == 3 || $lun == 4 } \
                {
                  puts "lun:  $lun reference in 'FGenUI::update_fgen_mode'"
                }
               
                Screens::position_widget ${tile_path}.fgen$lun.frm_dcs
                
                # Procedure 'DCS_UI::dcs_start' called in 'FGen::setMode' 

                if { $class == "QTR_FGEN" } \
                {
                  if { $lun == 1 } { set softkey  2 }
                  if { $lun == 2 } { set softkey  3 }
               
                  # SOFTKEY 2/3
                  View::AssignSoftkey  $tile_path  $softkey  "AFGEN $lun\nDCS"  \
                          "${tile_path}.fgen${lun}.mnu_mode.menu  invoke  TSIG"
               
                  # SOFTKEY 4
                  View::AssignSoftkey  $tile_path  4  ""  ""
               
                  # SOFTKEY 5
                  View::AssignSoftkey  $tile_path  5  ""  ""
               
                  # SOFTKEY 6
                  View::AssignSoftkey  $tile_path  6  ""  ""
               
                  # SOFTKEY 7
                  View::AssignSoftkey  $tile_path  7  ""  ""
                }
              }

     "TSIG"   { if { $lun == 3 || $lun == 4 } \
                {
                  puts "lun:  $lun reference in 'FGenUI::update_fgen_mode'"
                }
               
                Screens::position_widget ${tile_path}.fgen$lun.frm_tone_sig
                
                if { $class == "QTR_FGEN" } \
                {
                  if { $lun == 1 } { set softkey  2 }
                  if { $lun == 2 } { set softkey  3 }
               
                  # SOFTKEY 2/3
                  View::AssignSoftkey  $tile_path  $softkey  "AFGEN $lun\nTONE SIG"  \
                      "${tile_path}.fgen${lun}.mnu_mode.menu  invoke  \"SINC\""
               
                  # SOFTKEY 4
                  View::AssignSoftkey  $tile_path  4  "G"  ""
               
                  # SOFTKEY 5
                  View::AssignSoftkey  $tile_path  5  "R"  ""
               
                  # SOFTKEY 6
                  View::AssignSoftkey  $tile_path  6  "A"  ""
               
                  # SOFTKEY 7
                  View::AssignSoftkey  $tile_path  7  ""  ""
                }
              }

     "P25"    { # Special case
                return 
              }

     "SINC"   { if { $lun == 3 || $lun == 4 } \
                {
                  puts "lun:  $lun reference in 'FGenUI::update_fgen_mode'"
                }
               
                Screens::position_widget ${tile_path}.fgen$lun.frm_sinc
                
                # Procedure 'SincUI::setup_sinc_waveform' called in 'FGen::setMode'
                
                if { $class == "QTR_FGEN" } \
                {
                  if { $lun == 1 } { set softkey  2 }
                  if { $lun == 2 } { set softkey  3 }
               
                  # SOFTKEY 2/3
                  View::AssignSoftkey  $tile_path  $softkey  "AFGEN $lun\nSINC"  \
                      "${tile_path}.fgen${lun}.mnu_mode.menu  invoke  \"ARB\""
               
                  # SOFTKEY 4
                  View::AssignSoftkey  $tile_path  4  ""  ""
               
                  # SOFTKEY 5
                  View::AssignSoftkey  $tile_path  5  ""  ""
               
                  # SOFTKEY 6
                  View::AssignSoftkey  $tile_path  6  ""  ""
               
                  # SOFTKEY 7
                  View::AssignSoftkey  $tile_path  7  ""  ""
                }
              }

     "ARB"    { if { $lun == 3 || $lun == 4 } \
                {
                  puts "lun:  $lun reference in 'FGenUI::update_fgen_mode'"
                }
               
                Screens::position_widget ${tile_path}.fgen$lun.frm_arb
                set menu ${tile_path}.fgen$lun.frm_arb.mnu_arb_shape.menu
                if [winfo exists $menu] \
                {
                   $menu config -postcommand "ArbitraryUI::get_arb_file_names  $menu  $lun"
                }
                
                # Procedure 'ArbitraryUI::setup_arb_waveform' called in 'FGen::setMode'

                if { $class == "QTR_FGEN" } \
                {
                 
                  if { $lun == 1 } { set softkey  2 }
                  if { $lun == 2 } { set softkey  3 }
               
                                
                  View::AssignSoftkey  $tile_path  $softkey  "AFGEN $lun\nARBIT"  \
                      "${tile_path}.fgen${lun}.mnu_mode.menu  invoke  \"OFF\""

                  # SOFTKEY 4
                  View::AssignSoftkey  $tile_path  4  ""  ""
               
                  # SOFTKEY 5
                  View::AssignSoftkey  $tile_path  5  ""  ""
               
                  # SOFTKEY 6
                  View::AssignSoftkey  $tile_path  6  ""  ""
               
                  # SOFTKEY 7
                  View::AssignSoftkey  $tile_path  7  ""  ""
                }
              }

     "SWEEP"  { if { $lun == 3 || $lun == 4 } \
                {
                  puts "lun:  $lun reference in 'FGenUI::update_fgen_mode'"
                }
               
                Screens::position_widget ${tile_path}.fgen$lun.frm_sweep
                
                if { $class == "QTR_FGEN" } \
                {
                  if { $lun == 1 } { set softkey  2 }
                  if { $lun == 2 } { set softkey  3 }
               
                  # SOFTKEY 2/3
                  View::AssignSoftkey  $tile_path  $softkey  "AFGEN $lun\nSWEEP"  \
                      "${tile_path}.fgen${lun}.mnu_mode.menu  invoke  \"OFF\""
               
                  # SOFTKEY 4
                  View::AssignSoftkey  $tile_path  4  ""  ""
               
                  # SOFTKEY 5
                  View::AssignSoftkey  $tile_path  5  ""  ""
               
                  # SOFTKEY 6
                  View::AssignSoftkey  $tile_path  6  ""  ""
               
                  # SOFTKEY 7
                  View::AssignSoftkey  $tile_path  7  ""  ""
                }
              }
    }
    
    return
  }
  
  #****************************************************************************
  #
  #  Name: update_tone_level
  #
  #  Parameters: tile_path    - tile path of the invoking widget
  #              lun          - Logical Unit Number for the function generator
  #
  # Description: Determines which level entry is displayed in the TONE mode
  #
  #****************************************************************************
  proc update_tone_level { tile_path  lun  args } \
  {
     # Ignore if not FGEN1 or FGEN2
     if { ($lun != 1) && ($lun != 2) } \
     {
        return
     }

     set cmd [namespace code "update_tone_level_core $tile_path $lun"]
     after cancel $cmd
     after idle $cmd
  }
  proc update_tone_level_core { tile_path  lun } \
  {
     #puts "update_tone_level_core:  $tile_path,  $lun"

     # Ignore if not displaying the SINE waveform on the TONE mode
     set mode  $::FGen::State($lun,fgen_mode)
     if { $mode != "TONE" } \
     {
        return
     }

     set tile_path  [get_tile  $tile_path]
      
     set tone_used 0
     foreach subframe [grid slaves ${tile_path}.fgen$lun] \
     {
        if [string match ${tile_path}.fgen$lun.frm_tone $subframe] \
        {
           set tone_used 1
        }
     }
  
     # Do not play mapping games if the subframe is not mapped
     if { $tone_used == 0 } \
     {
        return
     }
     grid remove ${tile_path}.fgen$lun.frm_tone.ent_level
     grid remove ${tile_path}.fgen$lun.frm_tone.ent_level_rms
     if { $::FGen::State(level_units,$lun) == "Vrms" } \
     {
        Screens::position_widget ${tile_path}.fgen$lun.frm_tone.ent_level_rms
     } else \
     {
        Screens::position_widget ${tile_path}.fgen$lun.frm_tone.ent_level
     }
  }
  #**************************************************************************
  #
  #        Name: set_fgen_freq
  #
  #  Parameters: lun - Logical Unit Number for the function generator
  #
  # Description: Sets the frequency of the Function Generator
  #
  #**************************************************************************
  proc set_fgen_freq { lun } \
  {
    #puts "set_fgen_freq:  $lun"
     
    set mode  $::FGen::State($lun,fgen_mode)
    
    if { [info exists  ::FGen::State($lun,$mode,freq)] } \
    {
      FGen::setFreq  $lun  $::FGen::State($lun,$mode,freq)
    }

    # Procedure 'FGenUI::tone_send_seq'     called in 'FGen::setFreq' 
    # Procedure 'CTCSS_UI::ctcss_send_seq'  called in 'FGen::setFreq' 
    # Procedure 'DCS_UI::dcs_start'         called in 'FGen::setFreq' 

    return
  }


  #**************************************************************************
  #
  #        Name: set_fgen_level
  #
  #  Parameters: lun - Logical Unit Number for the function generator
  #
  # Description: Sets the level of the Function Generator
  #
  #**************************************************************************
  proc set_fgen_level { lun } \
  {
     
    set mode  $::FGen::State($lun,fgen_mode)
    #puts "set_fgen_level:  $lun $mode"

    # Handle the special case of TONE RMS
    if { ($lun == 1) || ($lun == 2) } \
    {
      if { $mode == "TONE" } \
      {
        if {$::FGen::State($lun,shape) == "SINE" } \
        {
          if {$::FGen::State(level_units,$lun) == "Vrms" } \
          {
             if { [info exists  ::FGen::State($lun,$mode,level_rms)] } \
             {
                # Multiply RMS * 2 * (2 ** 0.5)
                # We are setting the Vpp variable here because of the crazy interaction
                # between the "C" interface and the UI code
                set Vpp [expr $::FGen::State($lun,$mode,level_rms) * 2.828427]
                set ::FGen::State($lun,$mode,level) $Vpp
             }
          }
        }
      }
    }
    
    if { [info exists  ::FGen::State($lun,$mode,level)] } \
    {
      FGen::setLevel  $lun  $::FGen::State($lun,$mode,level)
    }

    # Procedure 'FGenUI::tone_send_seq'     called in 'FGen::setLevel' 
    # Procedure 'CTCSS_UI::ctcss_send_seq'  called in 'FGen::setLevel' 
    # Procedure 'DCS_UI::dcs_start'         called in 'FGen::setLevel' 

    return
  }

  #**************************************************************************
  #
  #  Name: toggle_tone_level_units
  #
  #  Parameters: lun - Logical Unit Number for the function generator
  #
  # Description: This toggles the type of units used when setting the level 
  #    of the sine wave output of the TONE mode of the Function Generator
  #
  #**************************************************************************
  proc toggle_tone_level_units { lun } \
  {
    # Function is illegal for anything other than FGEN1 and FGEN2
    if { ($lun != 1) && ($lun != 2) } \
    {
       return
    }

    # Function is illegal for anything other than TONE mode
    set mode  $::FGen::State($lun,fgen_mode)
    if { $mode != "TONE" } \
    {
       return
    }

    # For all waveshapes other than SINE, force Vpp rather than try to
    # determine an equation for RMS for each shape
    if {$::FGen::State($lun,TONE,shape) != "SINE" } \
    {
       set ::FGen::State(level_units,$lun)  "Vpp"
       return
    }   

    if { $::FGen::State(level_units,$lun) == "Vpp" } \
    {
       set ::FGen::State(level_units,$lun)  "Vrms"
    } \
    else \
    {
       set ::FGen::State(level_units,$lun)  "Vpp"
    }
  }

  #**************************************************************************
  #
  #  Name: reset_tone_level_units
  #
  #  Parameters: none
  #
  # Description: This forces the level unit type to be Vpp if the shape for
  #    the TONE mode of FGEN1 or FGEN2 is not SINE
  #
  #**************************************************************************
  proc reset_tone_level_units { tile_path lun  args } \
  {
    # Function is illegal for anything other than FGEN1 and FGEN2
    if { ($lun != 1) && ($lun != 2) } \
    {
       return
    }

    # Function is illegal for anything other than TONE mode
    set mode  $::FGen::State($lun,fgen_mode)
    if { $mode != "TONE" } \
    {
       return
    }

    # For all waveshapes other than SINE, force Vpp rather than try to
    # determine an equation for RMS for each shape
    if {$::FGen::State($lun,shape) != "SINE" } \
    {
       set ::FGen::State(level_units,$lun)  "Vpp"
    }   
    update_tone_level $tile_path $lun
  }



  #**************************************************************************
  #
  #        Name: set_fgen_dev
  #
  #  Parameters: lun - Logical Unit Number for the function generator
  #
  # Description: Sets the deviation of the Function Generator
  #
  #**************************************************************************
  proc set_fgen_dev { lun } \
  {
    #puts "set_fgen_dev:  $lun"
     
    set mode  $::FGen::State($lun,fgen_mode)
    
    if { [info exists  ::FGen::State($lun,$mode,dev)] } \
    {
      FGen::setDeviation  $lun  $::FGen::State($lun,$mode,dev)
    }

    # Procedure 'FGenUI::tone_send_seq'     called in 'FGen::setDeviation' 
    # Procedure 'CTCSS_UI::ctcss_send_seq'  called in 'FGen::setDeviation' 
    # Procedure 'DCS_UI::dcs_start'         called in 'FGen::setDeviation' 

    return
  }


  #**************************************************************************
  #
  #        Name: set_fgen_percent
  #
  #  Parameters: lun - Logical Unit Number for the function generator
  #
  # Description: Sets the percent deviation of the Function Generator
  #
  #**************************************************************************
  proc set_fgen_percent { lun } \
  {
    #puts "set_fgen_percent:  $lun"
     
    set mode  $::FGen::State($lun,fgen_mode)
    
    if { [info exists  ::FGen::State($lun,$mode,percent)] } \
    {
      FGen::setPercent  $lun  $::FGen::State($lun,$mode,percent)
    }

    # Procedure 'FGenUI::tone_send_seq'     called in 'FGen::setPercent' 
    # Procedure 'CTCSS_UI::ctcss_send_seq'  called in 'FGen::setPercent' 
    # Procedure 'DCS_UI::dcs_start'         called in 'FGen::setPercent' 

    return
  }


  #**************************************************************************
  #
  #        Name: set_fgen_shape
  #
  #  Parameters: lun - Logical Unit Number for the function generator
  #
  # Description: Sets the shape of the Function Generator
  #
  #**************************************************************************
  proc set_fgen_shape { lun } \
  {
    #puts "set_fgen_shape:  $lun"
     
    set mode  $::FGen::State($lun,fgen_mode)
    
    if { [info exists  ::FGen::State($lun,$mode,shape)] } \
    {
      FGen::setShape  $lun  $::FGen::State($lun,$mode,shape)
    }

    # Procedure 'FGenUI::tone_send_seq'     called in 'FGen::setShape' 
    # Procedure 'CTCSS_UI::ctcss_send_seq'  called in 'FGen::setShape' 
    # Procedure 'DCS_UI::dcs_start'         called in 'FGen::setShape' 

    return
  }


  #***************************************************************************
  #
  #        Name: tone_send_seq
  #
  #  Parameters: lun - Logical Unit Number of the Function Generator
  #
  # Description: Sends out a sequence for the TONE mode
  #
  #***************************************************************************
  proc tone_send_seq { lun } \
  {
    #puts "tone_send_seq:  $lun"
    
    switch -- $lun \
    {
      1  -
      2  -
      3  -
      4  { set freq   $::FGen::State($lun,TONE,freq)
           set level  $::FGen::State($lun,TONE,level)
           set shape  $::FGen::State($lun,TONE,shape) 
           
           set tone_seq  "$freq  0  $shape  0  $level"
    
           #puts "tone_seq:  $lun,  $tone_seq"
    
           FGen::Send  $lun  $tone_seq }
           
      default {  }
    }
    
    return
  }


  #**************************************************************************
  #
  #        Name: getSaveState
  #
  #  Parameters: none
  #
  # Description: This function returns a list of TCL commands which can be used
  #              to set the FGenUI tiles to its current state
  #
  #**************************************************************************
  proc getSaveState { } \
  {
    variable  fgen_mode
    
    set returnList { }
    

    return [subst -nocommands $returnList]
  }


  #**************************************************************************
  #
  #        Name: Sync
  #
  #  Parameters: none
  #
  # Description: Write state variables to FGenUI
  #
  #****************************************************************************
  proc Sync { } \
  {
    eval [getSaveState]
  }

  # If we are the master, route M1 and M2 to L1 at startup
  if ![interp issafe {}] \
  {
    Load::on_system_done \
    {
      BBAudio mixer FGEN3 L1 ON -nowait
      BBAudio mixer FGEN4 L1 ON -nowait
    }
  }

};  # End of the FGenUI namespace




#****************************************************************************
# This registers the tiles with the list for saving instrument states
#****************************************************************************
namespace eval ::SavedStates {

   variable tiles
   lappend tiles  FGenUI
}

namespace eval View \
{
  lappend PreCreate  FGen_SmallUI
}


