############################################################################
#                                                                           #
# IFR SYSTEMS, INC        COPYRIGHT (C) 2002      DESIGN ENGINEERING        #
# --------------------------------------------------------------------------#
#                                                                           #
# FILE        : routing.tcl                                                 #
#                                                                           #
# PRODUCT     : FITS                                                        #
#                                                                           #
# AUTHOR      : David Hagood                                                #
#                                                                           #
# DESCRIPTION : Code for the audio routing tiles                            #
#                                                                           #
#############################################################################


#****************************************************************************
#
# Set up a namespace for routing
#
#****************************************************************************
namespace eval routing_UI \
{

  #********************************************************************
  #
  #        Name: Create  
  #
  #  Parameters:
  #
  # Description: 
  #
  #********************************************************************
  proc Create { parent_path } \
  {
    set top_level [winfo toplevel $parent_path]
   
    set tile_path [name_child $top_level routing]
    
    if { ![winfo exists $tile_path] } \
    {   
      set tile_path  [Screens::create_tile  $tile_path  $tile_name  $tile_class]

      bind $tile_path <KeyRelease-F8> "${tile_path}.return invoke"
    }
    
    return $tile_path
  }

  
  #********************************************************************
  #
  #        Name: Map
  #
  #  Parameters:
  #
  # Description: 
  #
  #********************************************************************
  proc Map { tile_path } \
  { 
    safe_focus_force ${tile_path}.return
  }
  
  
  #********************************************************************
  #
  #        Name: Unmap
  #
  #  Parameters:
  #
  # Description: 
  #
  #********************************************************************
  proc Unmap { tile_path } \
  { 
    
  }
  

  #********************************************************************
  #
  #        Name: Destroy
  #
  #  Parameters:
  #
  # Description: 
  #
  #********************************************************************
  proc Destroy { tile_path } \
  { 
    
  }

  
  #********************************************************************
  #
  #        Name: Register
  #
  #  Parameters:
  #
  # Description: 
  #
  #********************************************************************
  proc Register { } \
  { 

  }


  #********************************************************************
  #
  #        Name: Show_route
  #
  #  Parameters: widget generating request
  #
  # Description: Show the "generic" routing table
  #
  #********************************************************************
  proc Show_route { {widget {}} {name route} } \
  {
     if ![winfo exists $widget] { set widget . }
     set parent [winfo toplevel $widget]
     set toplevel [name_child $parent $name]
     
     if { ![winfo exists $toplevel] } \
     {
       Screens::create_toplevel_tile $toplevel ROUTING
     }
     
     bind $toplevel <KeyRelease-F8> "$toplevel.return invoke"
  }

  
  #********************************************************************
  #
  #        Name: Show_rx_route
  #
  #  Parameters: widget generating request
  #
  # Description: Show the "receiver" routing table
  #
  #********************************************************************
  proc Show_rx_route { {widget {}} } \
  { 
     Show_route $widget rcv_route
  }
  
  
  #********************************************************************
  #
  #        Name: Show_gen_route
  #
  #  Parameters: widget generating request
  #
  # Description: Show the "generator" routing table
  #
  #********************************************************************
  proc Show_gen_route { {widget {}} } \
  { 
     Show_route $widget gen_route
  }
  
  
  #********************************************************************
  #
  #        Name: audio_atten_map
  #
  #  Parameters: widget generating request
  #
  # Description: Start watching the state of the audio out balance.
  # NOTE #2: BODGE this will need to change when this is properly hooked up.
  #********************************************************************
  proc audio_atten_map { widget } \
  {
     trace variable ::audio(audiobal) w "routing_UI::audio_bal_change $widget"
  }

  
  #********************************************************************
  #
  #        Name: audio_bal_change
  #
  #  Parameters: widget generating request
  #
  # Description: monitor the state of audio output balance. If balance
  #              is switched to unbalanced, disable this field.
  # NOTE: the engine really ought to set the atten setting to x1 in
  #       this case!
  # NOTE #2: BODGE this will need to change when this is properly hooked up.
  #********************************************************************
  proc audio_bal_change { widget args } \
  {
     set cmd "routing_UI::audio_bal_change_core $widget"
     after cancel $cmd
     after idle $cmd
  }
  proc audio_bal_change_core { widget } \
  {
     global audio
  
     if {$audio(audiobal) == "BALAN"} {
       $widget config -state normal
     } else {
       $widget config -state disabled    
     }
  }


  #********************************************************************
  #
  #        Name: audio_atten_destroy
  #
  #  Parameters: widget generating request
  #
  # Description: stop monitoring the state of the balance.
  #
  #********************************************************************
  proc audio_atten_destroy { widget } \
  {
     trace vdelete ::audio(audiobal) w "routing_UI::audio_bal_change $widget"
  }


  #********************************************************************
  #
  #        Name: set_demod_to_mod
  #
  #  Parameters: 
  #
  # Description: Sets the DEMOD AUDIO to MODULATOR connection
  #
  #********************************************************************
  proc set_demod_to_mod { } \
  {
    #puts "set_demod_to_mod"
    
    BBAudio mixer  SQUELCHED  L1  OFF -nowait
    BBAudio mixer  L1         L1  OFF -nowait
    
    switch -- $::BBAudio::State(demod_to_mod) \
    {
      "DEMOD"    { BBAudio mixer  L1         L1  ON  -nowait }
      
      "SQUELCH"  { BBAudio mixer  SQUELCHED  L1  ON  -nowait }

      "OFF"      -
      default    {}
    }
  }


  #********************************************************************
  #
  #        Name: factory_default
  #
  #  Parameters: 
  #
  # Description: Restores the Factory Defaults for the Audio Routing
  #
  #********************************************************************
  proc factory_default { } \
  {
    ::BBAudio::Factory_defaults
    
    return
  }


};  # End of routing_UI namespace


