#############################################################################
#                                                                           #
# IFR SYSTEMS, INC        COPYRIGHT (C)1998-2004      DESIGN ENGINEERING    #
# ------------------------------------------------------------------------- #
# FILE        : 3002.File_Save_RecallUI.tcl                                 #
# PRODUCT     : FITS                                                        #
# AUTHOR      :                                                             #
# DESCRIPTION : User interface for simple file Save / Recall operations     #
#                                                                           #
#############################################################################

package require IFR_options


namespace eval File_SaveUI \
{
  # File variables
  variable  save_flag    -1

  variable  disk_drive        "INTERNAL"
  variable  disk_drive_menu   { "INTERNAL"  "FLOPPY" }

  variable  file_dir      ""
  variable  file_name     ""
  variable  file_prefix   "FILE"

  variable  file_write_flag    0

  variable  file_number  1
  variable  file_date    ""

  variable  file_label  "File1"

  variable  file_append_overwrite  "APPEND"

  variable  file_format  "USER"
  variable  file_format_menu  { "DEFAULT"  "TEXT"  "USER"  "COMMA DEL" }

  variable  file_extension  ""
  variable  file_ext_user   ""
  
  variable  file_data      ""
  variable  file_data_csv  ""

  variable  file_encrypt_mode

  variable  file_source  "Save file to:"
 


  #***************************************************************************
  #
  #  Name: get_tile
  #
  #  Parameters: any child widget of the tile
  #
  #  Description: Returns the widget or parent which has the class of FILE_SAVE
  #               Returns -1 if there is an error
  #
  #***************************************************************************
  proc get_tile { widget } \
  {

     if { [winfo class $widget] == "FILE_SAVE" } { return $widget }

     # initialize value to force one pass through the loop
     set top_level ""
   
     while { $top_level != $widget } \
     {
        set widget [winfo parent $widget]
        switch -- [winfo class $widget] \
        {
           "FILE_SAVE"  { return $widget }
        }
        set top_level  [winfo toplevel $widget]
     }

     return -1     
  }


  #***************************************************************************
  #
  #        Name: display_file_save
  #
  #  Parameters: widget           - invoking widget tile path
  #              dir              - directory for the file
  #              data             - data to be saved to the file
  #              file_name_prefix - The first part of the file name
  #              file_label_user  - file label to use
  #              file_ext         - file extension to use
  #              file_encryption  - specify if the file should be encrypted
  #              data_csv         - Comma delimited data
  #              file_modes       - allows APPEND option
  #              file_ext_options - file extension options to be displayed
  #
  # Description: Saves the data to a file
  #
  #***************************************************************************
  proc display_file_save { widget  dir  data                             \
                                    { file_name_prefix  "FILE"       }   \
                                    { file_label_user   "File"       }   \
                                    { file_ext          ".TXT"       }   \
                                    { file_encryption   "NO_ENCRYPT" }   \
                                    { data_csv          "NO_CSV"     }   \
                                    { file_modes        "NO_MODES"   }   \
                                    { file_ext_options  "NO_OPT_EXT" } } \
  {
     variable  save_flag

     set save_flag  -1
     display_file_save_core $widget $dir $data $file_name_prefix        \
                            $file_label_user $file_ext $file_encryption \
                            $data_csv $file_modes $file_ext_options

    # Wait for user input:  SAVE or CANCEL
    while { $save_flag == -1 } { vwait File_SaveUI::save_flag }
    return  $save_flag
  }  

  proc display_file_save_core { widget  dir  data                        \
                                    { file_name_prefix  "FILE"       }   \
                                    { file_label_user   "File"       }   \
                                    { file_ext          ".TXT"       }   \
                                    { file_encryption   "NO_ENCRYPT" }   \
                                    { data_csv          "NO_CSV"     }   \
                                    { file_modes        "NO_MODES"   }   \
                                    { file_ext_options  "NO_OPT_EXT" } } \
  {
    #puts "display_file_save:  $widget,  $dir,  $file_ext,  $file_ext,"
    #puts "                    $file_modes,  $file_ext_options"

    variable  last_focus
    variable  save_flag
    
    variable  file_append_overwrite
    variable  file_data
    variable  file_data_csv
    variable  file_dir
    variable  file_encrypt_mode
    variable  file_extension
    variable  file_ext_user
    variable  file_label
    variable  file_number
    variable  file_prefix

    
    set last_focus  $widget
    
    set file_dir           $dir
    set file_data          $data
    set file_data_csv      $data_csv
    set file_label         $file_label_user
    set file_prefix        $file_name_prefix
    set file_encrypt_mode  $file_encryption
    
    set top_level  [winfo toplevel  $widget]
    
    set tile_path  [name_child  $top_level  file_save]
    
    if { ![winfo exists $tile_path] } \
    {
      set tile_path  [Screens::create_toplevel_tile  $tile_path  FILE_SAVE]
    }
        
    # Bind the F8 key for a return
    bind $tile_path  <KeyRelease-F8>  "File_SaveUI::file_save_cancel"

    # If there is no comma delimited data, 
    # delete the option from the menu, and invoke the 'USER' option
    if { $file_data_csv == "NO_CSV" } \
    {
      # Catch the error in case the menu item does not exist.
      oldCatch { ${tile_path}.mnu_file_format.menu  delete  "COMMA DEL" } err
      
      ${tile_path}.mnu_file_format.menu  invoke  2
    }

    # If the file_modes is not 'NO_MODES'
    # remove the file mode button from the display
    if { $file_modes == "NO_MODES" } \
    {
      grid remove  ${tile_path}.lbl_append_overwrt
      grid remove  ${tile_path}.btn_append_overwrt

      set file_append_overwrite  "OVERWRITE"
    }
  
    # If the file_ext_options is not 'NO_OPT_EXT'
    # remove the file extension menubutton from the display
    switch ($file_ext_options) \
    {
	    "NO_OPT_EXT"  \
	    {
	      grid remove  ${tile_path}.lbl_file_ext
	      grid remove  ${tile_path}.mnu_file_format
	      
	      Screens::position_widget  ${tile_path}.lbl_file_ext_2
	      
	      set file_extension  $file_ext
	      set file_format "USER"
	    }
	    "CSV" \
	    {
	      grid remove  ${tile_path}.lbl_file_ext
	      grid remove  ${tile_path}.mnu_file_format
	      
	      Screens::position_widget  ${tile_path}.lbl_file_ext_2
	      
	      set file_extension  ".CSV"
	      set file_format "COMMA DEL"

	    }
	    default \
	    {
	      grid remove  ${tile_path}.lbl_file_ext_2
	      Screens::position_widget  ${tile_path}.lbl_file_ext
	      Screens::position_widget  ${tile_path}.mnu_file_format

	      set file_ext_user  $file_ext
	  
	      set_file_extension
	    }
    }
    safe_focus_force ${tile_path}.btn_file_save
  }


  #**************************************************************************
  #  Name: file_save_cancel
  #
  #  Parameters:  widget        The button pressed to cancel the SAVE
  #  Description: Indicate that the user is canceling the operation and 
  #               terminate the parent tile
  #**************************************************************************
  proc file_save_cancel { widget } \
  {
    #puts "file_save_cancel"
    
    variable  save_flag
    set save_flag  0

    destroy_prompt $widget
  }

  #**************************************************************************
  #  Name: destroy_prompt
  #
  #  Parameters:  widget        The button pressed which initiates the exit
  #  Description: Terminate the parent tile
  #**************************************************************************
  proc destroy_prompt { widget } \
  {
    #puts "destroy_prompt"
    set tile_path [get_tile $widget]
    if { [winfo exists $tile_path] } \
    {
       destroy  $tile_path
    }
  }


  #**************************************************************************
  #
  #        Name: file_save
  #
  #  Parameters: 
  #
  # Description: Save the results to a file
  #
  #**************************************************************************
  proc file_save { tile_path } \
  {
     variable  file_name
     variable  file_append_overwrite

     #puts "file_save:  $tile_path"
     set tile_path  [get_tile  $tile_path]
     if { ![winfo exists $tile_path] } \
     {
        return
     }

     # Insure that the file name is up to date
     set_file_name
    
     # Determine if the directory for the file exists
     set dir_path  [file dirname  $file_name]
    
     # Make the directory
     # An error is not generated if the directory exists
     file mkdir  $dir_path
    
     # Determine if the file exists
     if { $file_append_overwrite == "OVERWRITE" } \
     {
        if { [file exists $file_name] } \
        {    
           # Display the over-write option
           grid remove ${tile_path}.btn_file_save
           grid remove ${tile_path}.btn_file_cancel
        
           Screens::position_widget ${tile_path}.lbl_file_exists
           Screens::position_widget ${tile_path}.lbl_file_write
           Screens::position_widget ${tile_path}.btn_yes
           Screens::position_widget ${tile_path}.btn_no
        
           safe_focus_force ${tile_path}.btn_no

           # Exit - User has to respond before overwriting file
           return
        }
     }
    
     write_file
     destroy  $tile_path
  }

  #**************************************************************************
  #  Name: restore_SaveCancel
  #
  #  Parameters:  widget        The button pressed to cancel the OVERWRITE
  #  Description: This function is called when the user refuses to overwrite
  #               an existing file, so the control is reset to the original
  #               buttons.
  #**************************************************************************
  proc restore_SaveCancel { widget } \
  {
     set tile_path  [get_tile  $widget]
     if { ![winfo exists $tile_path] } \
     {
        return
     }

     # Remove the overwrite option
     grid remove ${tile_path}.lbl_file_exists
     grid remove ${tile_path}.lbl_file_write
     grid remove ${tile_path}.btn_yes
     grid remove ${tile_path}.btn_no

     # Redisplay the SAVE / CANCEL options
     Screens::position_widget ${tile_path}.btn_file_save
     Screens::position_widget ${tile_path}.btn_file_cancel

     safe_focus_force ${tile_path}.btn_file_cancel
  }

  #**************************************************************************
  #  Name: write_file
  #
  #  Parameters:  widget        The button pressed to cancel the SAVE
  #  Description: Indicate that the user is canceling the operation and 
  #               terminate the parent tile
  #**************************************************************************
  proc write_file { } \
  {
     variable  file_append_overwrite
     variable  file_data
     variable  file_data_csv
     variable  file_date
     variable  file_encrypt_mode
     variable  file_format
     variable  file_label
     variable  file_name
     variable  status_text
     variable  save_flag

     # Insure that the file name is up to date
     set_file_name

     # Determine the write mode
     switch -- $file_append_overwrite \
     {
        "OVERWRITE"  { set file_id  [open $file_name w]  }
        "APPEND"     { set file_id  [open $file_name a] }
        default      { set file_id  [open $file_name a] }
     }
        
     set status_text  "SAVING..."
     update idle
      
     # Save the file label and date
     set file_date  [clock format [clock seconds]]
      
     # Determine the data to write to the file,
     # comma delimited or as sent by the user
     switch -- $file_format \
     {
        "COMMA DEL" \
        {
           puts  $file_id  "FILE_LABEL,$file_label"
           puts  $file_id  "FILE_DATE,$file_date"

           # Write the comma delimited data to the file
           puts  $file_id  "$file_data_csv"
        }
        
        default \
        {
           # Write the file
           puts  $file_id  "  FILE_LABEL   $file_label"
           puts  $file_id  "  FILE_DATE    $file_date"
           puts  $file_id  ""

           # Write the data to the file
           puts  $file_id  "$file_data"
        }
     }

     set status_text  "Save file to:"
       
     # Close the file
     close $file_id
      
     # If the file needs to be encrypted, call the procedure
     if { $file_encrypt_mode == "ENCRYPT" } \
     {
        file_encrypt
     }
     set save_flag  1
  }

  #**************************************************************************
  #
  #        Name: set_file_overwrite
  #
  #  Parameters: 
  #
  # Description: Sets the 'file_write_flag' flag
  #
  #**************************************************************************
  proc set_file_overwrite { value } \
  {
    #puts "set_file_overwrite:  $value"
    
    variable  file_write_flag
    
    set file_write_flag  $value
  
    return
  }

  #***************************************************************************
  #  Name: set_file_extension
  #
  #  Parameters:  None
  #  Description: Sets the file extension based on the file format selected
  #***************************************************************************
  proc set_file_extension { } \
  {
    #puts "set_file_extension"
    
    variable  file_format
    variable  file_extension
    variable  file_ext_user
    
    switch -- $file_format \
    {
      "DEFAULT"    { set file_extension  ".OUT"         }
      "COMMA DEL"  { set file_extension  ".CSV"         }
      "TEXT"       { set file_extension  ".TXT"         }
      "USER"       { set file_extension  $file_ext_user }
      default      { set file_extension  $file_ext_user }
    }

    set_file_name
  }

  #**************************************************************************
  #  Name: set_file_name
  #
  #  Parameters:  None
  #  Description: Sets the file name for changes in the disk drive, and
  #               the output number
  #**************************************************************************
  proc set_file_name { } \
  {
    #puts "set_file_name"
    
    variable  disk_drive
    variable  file_dir
    variable  file_name
    variable  file_prefix
    variable  file_number
    variable  file_extension
    variable  file_label

    #puts "disk_drive:  $disk_drive"

    # Make the subdirectory so that it exists.
    switch -- $disk_drive \
    {
      "INTERNAL"  { set dir_path  "$file_dir"       }
      "FLOPPY"    { set dir_path  [file join / fd0] }
    }

    #puts "dir_path:  $dir_path"
    
    # No error if the directory exists
    file mkdir  $dir_path

    # Limit the filename to 8 characters plus a 3 character extension
    set file_name  ${file_prefix}${file_number}${file_extension}
    
    set file_name  [file join $dir_path $file_name]
  }


  #**************************************************************************
  #
  #        Name: file_encrypt
  #
  #  Parameters: 
  #
  # Description: Creates an encrypted file then deletes the unencrypted file
  #
  #**************************************************************************
  proc file_encrypt { } \
  {
    #puts "file_encrypt"
    
    variable  disk_drive
    variable  file_dir
    variable  file_name

    #puts "disk_drive:  $disk_drive"

    # Create the file path
    switch -- $disk_drive \
    {
      "INTERNAL"  { set dir_path  "$file_dir"       }
      "FLOPPY"    { set dir_path  [file join / fd0] }
    }

    #puts "dir_path:  $dir_path"
    
    set temp_file_name  [file join  $dir_path  "TEMP.ENC"]
    
    # Create an encrypted file
    IFR_options  encrypt  $file_name  $temp_file_name
    
    after 500

    # Delete the unencrypted file
    file delete  $file_name
    
    # Rename the encrypted file to the original file name
    file rename  $temp_file_name  $file_name

    return
  }


  #***************************************************************************
  #
  #        Name: toggle_file_append_overwrite
  #
  #  Parameters:
  #
  # Description: Toggles the save mode between APPEND and OVERWRITE
  #
  #***************************************************************************
  proc toggle_file_append_overwrite { } \
  {
    #puts "toggle_file_append_overwrite"
    
    variable  file_append_overwrite
    
    if { $file_append_overwrite == "APPEND" } \
    {
      set file_append_overwrite  "OVERWRITE"
    } \
    else \
    {
      set file_append_overwrite  "APPEND"
    }

    return
  }


};  # End of namespace  File_SaveUI




namespace eval File_RecallUI \
{
  # File variables
  variable  max_num_save  100

  variable  recall_flag  -1

  variable  enable_recall  0

  variable  disk_drive        "INTERNAL"
  variable  disk_drive_menu   { "INTERNAL"  "FLOPPY" }

  variable  file_dir      ""
  variable  tempfile_dir  "/sysdata/"
  variable  file_name     ""
  variable  file_prefix   "FILE"

  variable  file_number  1
  variable  file_date    ""

  variable  file_label  "File1"

  variable  file_format  "USER"
  variable  file_format_menu  { "DEFAULT"  "TEXT"  "USER"  "COMMA DEL" }

  variable  file_extension  ""
  variable  file_ext_user   ""
  
  variable  file_data      ""
  variable  file_data_csv  ""

  variable  file_encrypt_mode

  variable  file_recall_opt

  variable  file_source  "Recall from:"
  
  variable  trace_option  "STORED"
  variable  filedata_semiphore  0
  variable  block_filenum_change  0


  #***************************************************************************
  #
  #  Name: get_tile
  #
  #  Parameters: any child widget of the tile
  #
  #  Description: Returns the widget or parent which has the class of FILE_RECALL
  #               Returns -1 if there is an error
  #
  #***************************************************************************
  proc get_tile { widget } \
  {
     if { [winfo class $widget] == "FILE_RECALL" } { return $widget }

     # initialize value to force one pass through the loop
     set top_level ""
   
     while { $top_level != $widget } \
     {
        set widget [winfo parent $widget]
        switch -- [winfo class $widget] \
        {
           "FILE_RECALL"  { return $widget }
        }
        set top_level  [winfo toplevel $widget]
     }

     return -1     
  }

  #***************************************************************************
  #
  #        Name: display_file_recall
  #
  #  Parameters: widget           - invoking widget tile path
  #              dir              - directory for the file
  #              file_name_prefix - the first part of the file name
  #              file_ext         - file extension to use
  #              file_encryption  - Specify whether the file should be
  #                                 excrypted
  #              recall_options   - allows recall options to be specified
  #                                 used for Audio Analyzer traces
  #
  # Description: Recalls the data from a file
  #
  #***************************************************************************
  proc display_file_recall { widget  dir  \
                               { file_name_prefix  "FILE"          }  \
                               { file_ext          ".TXT"          }  \
                               { file_encryption   "NO_ENCRYPT"    }  \
                               { recall_options    "NO_RECALL_OPT" } } \
  {
    #puts "display_file_recall:  $widget,  $dir,  $file_name_prefix, "
    #puts "                      $file_ext,  $file_encryption"

    variable  last_focus
    variable  file_dir
    variable  file_data
    variable  file_prefix
    variable  file_extension
    variable  file_encrypt_mode
    variable  file_recall_opt

    variable  recall_flag
    
    set last_focus  $widget
    
    set file_dir           $dir
    set file_prefix        $file_name_prefix
    set file_extension     $file_ext
    set file_encrypt_mode  $file_encryption
    set file_recall_opt    $recall_options
    
    set file_data  ""

    set recall_flag  -1
    
    set top_level  [winfo toplevel $widget]
    
    set tile_path  [name_child  $top_level  file_recall]
    
    if { ![winfo exists $tile_path] } \
    {
      set tile_path  [Screens::create_toplevel_tile  $tile_path  FILE_RECALL]
    }
        
    # Setup the display based for recall options
    grid remove  ${tile_path}.lbl_trace_options
    grid remove  ${tile_path}.mnu_trace_options
    
    if { $file_recall_opt == "NO_RECALL_OPT" } \
    {
      # No action
    } \
    elseif { $file_recall_opt == "TRACE_OPTIONS" } \
    {
      Screens::position_widget  ${tile_path}.lbl_trace_options
      Screens::position_widget  ${tile_path}.mnu_trace_options
    }
    
    # Bind the F8 key for a return
    bind $tile_path <KeyRelease-F8> "File_RecallUI::file_recall_cancel"

    bind $tile_path <Destroy>  \
                    "File_RecallUI::destroy_file_recall  $tile_path  $widget"

    trace variable  ::File_RecallUI::enable_recall  w \
                                   "File_RecallUI::update_recall  $tile_path"

    trace variable  ::File_RecallUI::block_filenum_change  w \
                                   "File_RecallUI::update_filenum_entry  $tile_path"

    trace variable  ::File_RecallUI::file_number  w \
                                   "File_RecallUI::get_file_info"

    # Update the present file number data: date, time
    get_file_info
    
    # Focus on the RECALL Button
    safe_focus_force ${tile_path}.btn_file_recall

    # Wait for user input:  RECALL or CANCEL
    while { $recall_flag == -1 } { vwait File_RecallUI::recall_flag }

    destroy  $tile_path

    safe_focus_force $last_focus

    return  $file_data
  }


  #**************************************************************************
  #
  #        Name: destroy_file_recall
  #
  #  Parameters: tile_path  - tile path of the invoking widget
  #              last_focus - widget that displayed the toplevel
  #
  # Description: Handles the <Destroy> bind for the Recall toplevel
  #
  #**************************************************************************
  proc destroy_file_recall { tile_path  last_focus } \
  {
    #puts "destroy_file_recall:  $tile_path,  $last_focus"
    
    trace vdelete  ::File_RecallUI::enable_recall  w \
                                   "File_RecallUI::update_recall  $tile_path"

    trace vdelete  ::File_RecallUI::block_filenum_change  w \
                                   "File_RecallUI::update_filenum_entry  $tile_path"

    trace vdelete  ::File_RecallUI::file_number  w \
                                   "File_RecallUI::get_file_info"
    
    return
  }


  #**************************************************************************
  #
  #        Name: file_recall_cancel
  #
  #  Parameters: 
  #
  # Description: Cancel the recall operation
  #
  #**************************************************************************
  proc file_recall_cancel { } \
  {
    #puts "file_recall_cancel"
    
    variable  file_data
    variable  recall_flag

    set file_data  ""
    
    set recall_flag  0
    
    return
  }


  #**************************************************************************
  #
  #        Name: file_recall
  #
  #  Parameters: 
  #
  # Description: Recall the results from a file
  #
  #**************************************************************************
  proc file_recall { tile_path } \
  {
    #puts "file_recall:  $tile_path"
    
    variable  last_focus
    variable  file_name
    variable  file_number
    variable  file_data
    variable  recall_flag
    variable  file_encrypt_mode
    variable  tempfile_dir
    variable  block_filenum_change

    set tile_path  [get_tile  $tile_path]
    if { ![winfo exists $tile_path] } \
    {
      return
    }
    
    set_file_name

    # Determine if the file exists
    if { [file exists $file_name] } \
    {
      set block_filenum_change 1
      if { $file_encrypt_mode == "ENCRYPT" } \
      {
        set temp_file_name  [file join  $tempfile_dir  "TEMP.DEC"]
        IFR_options  decrypt  $file_name  $temp_file_name
        set local_file_name $temp_file_name
      } else \
      {
        set local_file_name $file_name
      }

      # Open file
      set file_id  [open $local_file_name r]
      
      # Get the file label
      gets  $file_id  file_line
      regexp {FILE_LABEL[ ,]*(.*)[ ]*} $file_line foo file_label

      # Get the file date
      gets  $file_id  file_line
      regexp {FILE_DATE[ ,]*(.*)[ ]*} $file_line foo file_date

      # Get the blank space
      gets  $file_id  file_line

      ###########################
      # Get data from the file
      ###########################
      set file_data  ""
      set data_read  0
      
      # Read data until the EOF is encountered
      while { $data_read != -1 } \
      {
        # Read the data from the file
        set data_read  [gets  $file_id  file_line]
        
        append file_data  "$file_line\n"
      }

      # Close the file
      close $file_id
      
      # If we decrypted the file, delete it
      if { $file_encrypt_mode == "ENCRYPT" } \
      {
        file delete  $local_file_name
      }

      set block_filenum_change 0
      set recall_flag  1
    } \
    else \
    {
      set recall_flag  0
    }
    
    return
  }

  #**************************************************************************
  #  Name: incr_file_number
  #
  #  Parameters: None
  #  Description: Increments the number of the file being displayed
  #**************************************************************************
  proc incr_file_number { } \
  {
    #puts "incr_file_number:"
    
    variable  max_num_save
    variable  file_number
    variable  filedata_semiphore

    # Skip processing if we are still getting the data about the previous file
    if {$filedata_semiphore == 1} \
    {
      return
    }

    incr file_number  1
    if { $file_number > $max_num_save } \
    {
      set file_number  $max_num_save
    }
  }


  #**************************************************************************
  #  Name: decr_file_number
  #
  #  Parameters: None
  #  Description: Decrements the number of the file being displayed
  #**************************************************************************
  proc decr_file_number { } \
  {
    #puts "decr_file_number"
    
    variable  file_number
    variable  filedata_semiphore
    
    # Skip processing if we are still getting the data about the previous file
    if {$filedata_semiphore == 1} \
    {
      return
    }

    incr file_number  -1
    if { $file_number < 1 } \
    {
      set file_number  1
    }
  }


  #**************************************************************************
  #  Name: get_file_info
  #
  #  Parameters: None
  #  Description: Retrieves file information: label and date
  #**************************************************************************
  proc get_file_info { args } \
  {
    #puts "get_file_info"
    
    variable  file_name
    variable  file_number
    variable  file_label
    variable  file_date
    variable  file_encrypt_mode
    variable  tempfile_dir
    variable  enable_recall
    variable  filedata_semiphore
    variable  block_filenum_change

    # Check our "semiphore" that we are using to lock the file data access
    # Exit if not done with previous call
    if {$filedata_semiphore == 1} \
    {
       return
    } else \
    {
       set filedata_semiphore 1
    }

    set file_label  ""
    set file_date   ""

    set_file_name
    set recall_avail  0

    # Determine if the file exists
    if { [file exists $file_name] } \
    {    
       set block_filenum_change 1
       # Decrypt the file to read the info
       if { $file_encrypt_mode == "ENCRYPT" } \
       {
          set temp_file_name  [file join  $tempfile_dir  "TEMP.DEC"]
          IFR_options  decrypt  $file_name  $temp_file_name
          set local_file_name $temp_file_name
       } else \
       {
          set local_file_name $file_name
       }

       # Open file
       set file_id  [open $local_file_name r]
       catch \
       {
          # Get the file label
	  gets  $file_id  file_line
	  regexp {FILE_LABEL[ ,]*(.*)[ ]*} $file_line foo file_label
	      
	  # Get the file date
	  gets  $file_id  file_line
	  regexp {FILE_DATE[ ,]*(.*)[ ]*} $file_line foo file_date
       }
       # Close the file
       close $file_id

       # If we decrypted the file, delete it
       if { $file_encrypt_mode == "ENCRYPT" } \
       {
          file delete  $local_file_name
       }
       set recall_avail  1
       set block_filenum_change 0
    }

    set filedata_semiphore 0
    set enable_recall  $recall_avail
  }


  #**************************************************************************
  #  Name: update_recall
  #
  #  Parameters:  tile_path     path of the invoking widget
  #               args          arguments provided from a TRACE call
  #  Description: Updates the state of the RECALL button to changes in the 
  #              'enable_recall' flag.
  #**************************************************************************
  proc update_recall { tile_path  args } \
  {
    #puts "update_recall:  $tile_path,  $args"
    
    variable  enable_recall

    set tile_path  [get_tile  $tile_path]
    if { ![winfo exists $tile_path] } \
    {
      return
    }

    switch -- $enable_recall \
    {
      0  { ${tile_path}.btn_file_recall config -state disabled }
      1  { ${tile_path}.btn_file_recall config -state normal }
    }
  }


  #**************************************************************************
  #  Name: update_filenum_entry
  #
  #  Parameters:  tile_path     path of the invoking widget
  #               args          arguments provided from a TRACE call
  #  Description: Updates the state of the RECALL button to changes in the 
  #              'enable_recall' flag.
  #**************************************************************************
  proc update_filenum_entry { tile_path  args } \
  {
    #puts "update_filenum_entry:  $tile_path,  $args"
    
    variable  block_filenum_change

    set tile_path  [get_tile  $tile_path]
    if { ![winfo exists $tile_path] } \
    {
      return
    }

    switch -- $block_filenum_change \
    {
      1  { ${tile_path}.ent_file_number config -state disabled }
      0  { ${tile_path}.ent_file_number config -state normal }
    }
  }


  #**************************************************************************
  #
  #        Name: set_file_name
  #
  #  Parameters: 
  #
  # Description: Sets the file name for changes in the disk drive, and
  #            : the output number
  #
  #**************************************************************************
  proc set_file_name { } \
  {
    #puts "set_file_name"
    
    variable  disk_drive
    variable  file_dir
    variable  file_name
    variable  file_prefix
    
    variable  file_number

    variable  file_extension
    
    variable  file_label

    #puts "disk_drive:  $disk_drive"

    # Make the subdirectory so that it exists.
    switch -- $disk_drive \
    {
      "INTERNAL"  { set dir_path  "$file_dir"       }
      
      "FLOPPY"    { set dir_path  [file join / fd0] }
    }

    #puts "dir_path:  $dir_path"
    
    # No error if the directory exists
    file mkdir  $dir_path

    # Limit the filename to 8 characters plus a 3 character extension
    set file_name  ${file_prefix}${file_number}${file_extension}
    
    set file_name  [file join $dir_path $file_name]
    
    return
  }


  #**************************************************************************
  #  Name: file_delete
  #
  #  Parameters:  tile_path     The path of the widget requesting a file delete
  #  Description: This function determines the name of the file to be deleted
  #               and it modifies the prompt to get a user verification of
  #               the delete operation.
  #**************************************************************************
  proc file_delete { tile_path } \
  {
    #puts "file_delete:  $tile_path"
    
    variable  file_name
    variable  block_filenum_change

    set tile_path  [get_tile  $tile_path]

    if { ![winfo exists $tile_path] } \
    {
      return
    }

    set_file_name

    # Determine if the file exists
    if { [file exists $file_name] } \
    {    
      set block_filenum_change 1
      # Prompt the user to verify before deleting the file
      grid remove ${tile_path}.btn_file_delete
      grid remove ${tile_path}.btn_file_recall
      grid remove ${tile_path}.btn_file_cancel
        
      Screens::position_widget ${tile_path}.lbl_delete
      Screens::position_widget ${tile_path}.btn_yes
      Screens::position_widget ${tile_path}.btn_no
      
      safe_focus_force ${tile_path}.btn_no
    }
  }

  #**************************************************************************
  #  Name: process_delete_selection
  #
  #  Parameters:  value         0 = cancel delete, 1 = delete file
  #               widget        File recall window
  #  Description: Retrieves file information: label and date
  #**************************************************************************
  proc process_delete_selection { value widget } \
  {
     variable  filedata_semiphore
     variable  file_name
     variable  file_label
     variable  file_date
     variable  enable_recall
     variable  block_filenum_change

     if { $value == 1 } \
     {
        # If we are currently changing the file data, we have to ignore the
        # delete to insure that we are deleting the right file
        if {$filedata_semiphore == 0} \
        {
           set filedata_semiphore 1
           file delete  $file_name
           set file_label  ""
           set file_date   ""
           set filedata_semiphore 0
           set enable_recall  0
        }
     }
     # Attempt to get the main tile path from the button
     set tile_path  [get_tile  $widget]

     if { [winfo exists $tile_path] } \
     {
        grid remove ${tile_path}.lbl_delete
        grid remove ${tile_path}.btn_yes
        grid remove ${tile_path}.btn_no

        Screens::position_widget ${tile_path}.btn_file_delete 
        Screens::position_widget ${tile_path}.btn_file_recall
        Screens::position_widget ${tile_path}.btn_file_cancel
      
        safe_focus_force ${tile_path}.btn_file_recall
     }
     set block_filenum_change 0
  }

  #***************************************************************************
  #
  #        Name: set_file_extension
  #
  #  Parameters:
  #
  # Description: Sets the file extension based on the file format selected
  #
  #***************************************************************************
  proc set_file_extension { } \
  {
    #puts "set_file_extension"
    
    variable  file_format
    variable  file_extension
    variable  file_ext_user
    
    switch -- $file_format \
    {
      "DEFAULT"    { set file_extension  ".OUT"         }
      
      "COMMA DEL"  { set file_extension  ".CSV"         }
      
      "TEXT"       { set file_extension  ".TXT"         }

      "USER"       { set file_extension  $file_ext_user }
      
      default      { set file_extension  $file_ext_user }
    }

    set_file_name
    
    return
  }


  #**************************************************************************
  #
  #        Name: file_encrypt
  #
  #  Parameters: 
  #
  # Description: Creates an encrypted file then deletes the unencrypted file
  #
  #**************************************************************************
  proc file_encrypt { } \
  {
    #puts "file_encrypt"
    
    variable  disk_drive
    variable  file_dir
    variable  file_name

    #puts "disk_drive:  $disk_drive"

    # Create the file path
    switch -- $disk_drive \
    {
      "INTERNAL"  { set dir_path  "$file_dir"       }
      
      "FLOPPY"    { set dir_path  [file join / fd0] }
    }

    set temp_file_name  [file join  $dir_path  "TEMP.ENC"]
    
    # Create an encrypted file
    IFR_options  encrypt  $file_name  $temp_file_name
    
    after 500
    
    # Delete the unencrypted file
    file delete  $file_name
    
    # Rename the encrypted file to the original file name
    file rename  $temp_file_name  $file_name

    return
  }

};  # End of namespace  File_RecallUI

