#############################################################################
#                                                                           #
# IFR SYSTEMS, INC        COPYRIGHT (C) 2003      DESIGN ENGINEERING        #
# --------------------------------------------------------------------------#
#                                                                           #
# FILE        : 3003.SweepUI.rc                                             #
#                                                                           #
# PRODUCT     : FITS                                                        #
#                                                                           #
# AUTHOR      : Preston McCormick                                           #
#                                                                           #
# DESCRIPTION : Code for the Function Generator Sweep mode                  #
#                                                                           #
#############################################################################

package require FGen


namespace eval SweepUI \
{
  variable  last_focus
  variable  please_wait  ""
  
  variable  StateUI
  
  set StateUI(sweep_mode_menu)  { "BURST"  "CONT" }
  

  #**************************************************************************
  #
  #        Name: set_sweep_level
  #
  #  Parameters: lun - Logical Unit Number for the function generator
  #
  # Description: Sets the level of the Sweep mode
  #
  #**************************************************************************
  proc set_sweep_level { lun } \
  {
    #puts "set_sweep_level:  $lun"
     

    return
  }


  #**************************************************************************
  #
  #        Name: set_sweep_start_freq
  #
  #  Parameters: lun - Logical Unit Number for the function generator
  #
  # Description: Sets the start frequency of the Sweep mode
  #
  #**************************************************************************
  proc set_sweep_start_freq { lun } \
  {
    #puts "set_sweep_start_freq:  $lun"
     

    return
  }


  #**************************************************************************
  #
  #        Name: set_sweep_stop_freq
  #
  #  Parameters: lun - Logical Unit Number for the function generator
  #
  # Description: Sets the stop frequency of the Sweep mode
  #
  #**************************************************************************
  proc set_sweep_stop_freq { lun } \
  {
    #puts "set_sweep_stop_freq:  $lun"
     

    return
  }


  #**************************************************************************
  #
  #        Name: set_sweep_duration
  #
  #  Parameters: lun - Logical Unit Number for the function generator
  #
  # Description: Sets the duration of the Sweep mode
  #
  #**************************************************************************
  proc set_sweep_duration { lun } \
  {
    #puts "set_sweep_duration:  $lun"
     

    return
  }


  #**************************************************************************
  #
  #        Name: set_sweep_mode
  #
  #  Parameters: lun - Logical Unit Number for the function generator
  #
  # Description: Sets the mode of the Sweep mode
  #
  #**************************************************************************
  proc set_sweep_mode { lun } \
  {
    #puts "set_sweep_mode:  $lun"
     

    return
  }


  #**************************************************************************
  #
  #        Name: display_sweep_config
  # 
  #  Parameters: tile_path - tile path of the invoking widget
  #              lun       - Logical Unit Number for the function generator
  #
  # Description: Displays the TONE SWEEP mode configuration toplevel 
  #
  #**************************************************************************
  proc display_sweep_config { tile_path  lun } \
  {
    #puts "display_sweep_config:  $tile_path,  $lun"
    
    variable  last_focus
    
    set last_focus  $tile_path

    set top_level  [winfo toplevel $tile_path]

    set tile_path  [name_child  $top_level  sweep_${lun}]

    if { ![winfo exists $tile_path] } \
    {
      set tile_path  [Screens::create_toplevel_tile  $tile_path  SWEEP_CONFIG]
    }
    
    # Configure the title to reflect the LUN number
    wm title  $tile_path  "TONE SWEEP #$lun"

    # Bind the F8 key for a return
    bind $tile_path <KeyRelease-F8> "destroy  [winfo toplevel $tile_path]"
    
    # Bind the <Destroy> event for cleanup
    bind $tile_path  <<Destroy>>  "SweepUI::destroy_sweep_config  $tile_path  $lun"

    after idle  "focus -force  ${tile_path}.btn_return"
    
    return
  }


  #**************************************************************************
  #
  #        Name: destroy_sweep_config
  # 
  #  Parameters: tile_path - tile path of the invoking widget
  #              lun       - Logical Unit Number for the function generator
  #
  # Description: Handles the <Destroy> event for the TONE SWEEP 
  #              configuration toplevel 
  #
  #**************************************************************************
  proc destroy_sweep_config { tile_path  lun } \
  {
    #puts "destroy_sweep_config:  $tile_path,  $lun"
     
    variable  last_focus
    variable  please_wait
    
    after idle  "focus -force $last_focus"

    return
  }


  #**************************************************************************
  #
  #        Name: getSaveState
  #
  #  Parameters: none
  #
  # Description: This function returns a list of TCL commands which can be 
  #              used to set the TONE SWEEP mode to its current state
  #
  #**************************************************************************
  proc getSaveState { } \
  {
    set returnList { }
    
    #append returnList  "set ::SweepUI::fgen_mode(2)  {$fgen_mode(2)} \n"

    return [subst -nocommands $returnList]
  }


  #**************************************************************************
  #
  #        Name: Sync
  #
  #  Parameters: none
  #
  # Description: Write state variables to SweepUI
  #
  #****************************************************************************
  proc Sync { } \
  {
    eval [getSaveState]
  }

};  # End of the SweepUI namespace


#****************************************************************************
# This registers the tiles with the list for saving instrument states
#****************************************************************************
namespace eval ::SavedStates \
{
  variable tiles   

  lappend tiles  SweepUI
}
