#############################################################################
#                                                                           #
# IFR SYSTEMS, INC        COPYRIGHT (C) 2003      DESIGN ENGINEERING        #
# --------------------------------------------------------------------------#
#                                                                           #
# FILE        : 3005.CtcssUI.tcl                                            #
#                                                                           #
# PRODUCT     : FITS                                                        #
#                                                                           #
# AUTHOR      : Preston McCormick                                           #
#                                                                           #
# DESCRIPTION : Code for the CTCSS Tone Signaling mode                      #
#                                                                           #
#############################################################################


namespace eval CTCSS_UI \
{
  variable  last_focus   ""
  variable  please_wait  ""


  #***************************************************************************
  #
  #        Name: get_tile
  #
  #  Parameters: widget - any child widget of the tile
  #
  # Description: Get the name of the tile's parent frame
  #
  #***************************************************************************
  proc get_tile { widget } \
  {
     
     while { 1 } \
     {
       switch -- [winfo class $widget] \
       {
         "GEN"          -
         "SMALL_FGEN"   -
         "QTR_FGEN"     -
         "CTCSS_CONFIG" { return $widget}
       }
        
       set widget [winfo parent $widget]
     }
     
  }


  #***************************************************************************
  #
  #        Name: ctcss_set_reed_code
  #
  #  Parameters: lun - Logical Unit Number of the function generator
  #
  # Description: Sets the CTCSS code and frequency
  #
  #***************************************************************************
  proc ctcss_set_reed_code { lun  args } \
  {
    #puts "ctcss_set_reed_code:  $lun,  $args"
    
    # Get the CTCSS code string from the menu variable
    set temp_code  $::FGen::State($lun,CTCSS,temp_code)

    # If the code string is NULL, set it to the first code string in the list
    if { $temp_code == "" } \
    {
      set temp_code  [lindex $::CTCSS::codes_and_freqs  0]
    }
    
    # Set the code variable to the two digit code value in the menu variable
    set ::FGen::State($lun,CTCSS,code)  [string range $temp_code  0  1]
    
    # Get the string length of the menu variable
    set str_length  [string length $temp_code]
    
    # Get the frequency value, from the 5th character to the end
    set ::FGen::State($lun,CTCSS,freq)  \
                                     [string range $temp_code  5  $str_length]
    
    ctcss_send_seq  $lun

    return
  }


  #***************************************************************************
  #
  #        Name: ctcss_send_seq
  #
  #  Parameters: lun - Logical Unit Number of the Function Generator
  #
  # Description: Sends out a sequence for the CTCSS mode
  #
  #***************************************************************************
  proc ctcss_send_seq { lun } \
  {
    #puts "ctcss_send_seq:  $lun"
    
    switch -- $lun \
    {
      1  -
      2  -
      3  -
      4  { set freq   $::FGen::State($lun,CTCSS,freq)
           set level  $::FGen::State($lun,CTCSS,level)
    
           set ctcss_seq  "$freq  0  SINE  0  $level"
    
           #puts "ctcss_seq:  $ctcss_seq"
    
           FGen::Send  $lun  $ctcss_seq }

      default {  }
    }
        
    return
  }


  #***************************************************************************
  #
  #        Name: ctcss_config
  #
  #  Parameters: tile_path - tile path of the invoking widget
  #              lun       - Logical Unit Number of the function generator
  #
  # Description: Displays the CTCSS configuration toplevel
  #
  #***************************************************************************
  proc ctcss_config { tile_path  lun } \
  {
    #puts "ctcss_config:  $tile_path,  $lun"
    
    variable  last_focus
      
    set last_focus  $tile_path

    set top_level [winfo toplevel $tile_path]

    set tile_path   [name_child  $top_level  ctcss_$lun]

    if { ![winfo exists $tile_path] } \
    {
      set tile_path  [Screens::create_toplevel_tile  $tile_path  CTCSS_CONFIG]
    }
    
    # Configure the title to reflect the LUN number
    wm title  $tile_path  "CTCSS Config #$lun"

    # Bind the F8 key for a return
    bind $tile_path <KeyRelease-F8>  "destroy [winfo toplevel $tile_path]"

    bind $tile_path <<Destroy>>  "CTCSS_UI::ctcss_config_return  $tile_path  $lun"

    safe_focus_force ${tile_path}.btn_return

    return
  }


  #***************************************************************************
  #
  #        Name: ctcss_config_return
  #
  #  Parameters: tile_path - tile path of the invoking widget
  #              lun       - Logical Unit Number of the function generator
  #
  # Description: Handles the <destroy> event of the CTCSS configuration
  #              toplevel
  #
  #***************************************************************************
  proc ctcss_config_return { tile_path  lun } \
  {
    #puts "ctcss_config_return:  $tile_path,  $lun"

    variable  last_focus
    variable  please_wait
    
    set please_wait  "Please Wait ..."
  
    # Stop the trigger event
    FGen::setResponse  $lun  1  {}

    set please_wait  ""

    safe_focus_force $last_focus
  }


};  # End of CTCSS_UI namespace
