#############################################################################
#                                                                           #
# IFR SYSTEMS, INC        COPYRIGHT (C)1998,99      DESIGN ENGINEERING      #
# ------------------------------------------------------------------------- #
#                                                                           #
# FILE        : 3005.Freq_ListUI.tcl                                        #
#                                                                           #
# PRODUCT     : FITS                                                        #
#                                                                           #
# AUTHOR      : P. McCormick                                                #
#                                                                           #
# DESCRIPTION : Frequency List User Interface                               #
#                                                                           #
#############################################################################

namespace eval Freq_ListUI \
{
  variable SaveSortOrder LAST
  # Global variables
  variable  last_focus

  variable  list_done  0

  variable  list_max_index    300

  variable  max_num_save      100
    
  variable  list_freq_rec     0.0000
  variable  list_freq_gen     0.0000
  variable  list_freq_offset  0.0000
  variable  list_freq_label   ""
  variable  list_freq_index   0
  variable  list_freq_incr    0.0000
  
  variable  pres_list_label   ""

  variable  narrative_text    "from Index"
  
  variable  ListFreqArray

  # File variables
  variable  working_dir  "/usr/freq_lists/"
  variable  file_label   "Freq_list"
  variable  file_date ""
  variable  file_prefix  "FLIST"
  variable  file_ext     ".FRQ"
  variable  save_flag    0
  
  variable current_file_name ""


  #***************************************************************************
  #
  #        Name: get_tile
  #
  #  Parameters: widget - any child widget of the tile
  #
  # Description: Get the name of the tile's parent frame
  #
  #***************************************************************************
  proc get_tile { widget } \
  {
    #set class  $::FGenUI::tile_class
     
    while { 1 } \
    {
      switch -- [winfo class $widget] \
      {
        "FREQ_LIST"  { return $widget }
      }
        
      set widget [winfo parent $widget]
    }
  }


  #**************************************************************************
  #
  #        Name: display_freq_list
  #
  #  Parameters: tile_path - tile path of the invoking widget
  #
  # Description: Displays toplevel for editing frequency list values
  #
  #**************************************************************************
  proc display_freq_list { tile_path } \
  {
    #puts "display_freq_list:  $tile_path"
    
    variable  last_focus
    variable  list_done
    variable  save_flag

    set list_done  0
    set save_flag  0

    set last_focus  $tile_path
    
    set top_level  [winfo toplevel $tile_path]
   
    set tile_path  [name_child  $top_level  freq_list]

    if { ![winfo exists $tile_path] } \
    {
      set tile_path  [Screens::create_toplevel_tile  $tile_path  FREQ_LIST]
    }
    load_listbox  $tile_path
    
    update_list_index  $tile_path
  }


  #**************************************************************************
  #
  #        Name: init_array
  #
  #  Parameters: 
  #
  # Description: Initializes the frequency list array
  #
  #**************************************************************************
  proc init_array { } \
  {
    #puts "init_array"
    
    variable  list_max_index

    variable  ListFreqArray
    
    # If the array does not exist, load the defaults
    if { ![info exists ListFreqArray(index,0)] } \
    {
      for { set i 0 } { $i <= $list_max_index } { incr i } \
      {
        set ListFreqArray(index,$i)        $i
        set ListFreqArray(rx_freq,$i)      0.0000
        set ListFreqArray(tx_freq,$i)      0.0000
        set ListFreqArray(offset_freq,$i)  0.0000
        set ListFreqArray(rx_mode,$i)      "XXX"
        set ListFreqArray(tx_mode,$i)      "XXX"
        set ListFreqArray(label,$i)        "chan$i"
      }
    }
    
    return
  }


  #**************************************************************************
  #
  #        Name: fill_array
  #
  #  Parameters: base_index - index to start loading
  #
  # Description: Loads the array starting at the specified index
  #
  #**************************************************************************
  proc fill_array { { base_index 0 } } \
  {
    #puts "fill_array:  $base_index"
    
    variable  list_max_index

    variable  list_freq_rec
    variable  list_freq_offset
    variable  list_freq_incr

    variable  ListFreqArray
    
    # Load the array based on the user input Receiver base freq, Offset, 
    # and Increment
    for { set i $base_index } { $i <= $list_max_index } { incr i } \
    {
      set rel_index  [expr $i - $base_index]
      
      set ListFreqArray(index,$i)  $i
        
      set rec_freq  [expr $list_freq_rec + [expr $list_freq_incr * $rel_index]]
      
      if { $rec_freq > 2800.0 } \
      {
        set rec_freq  2800.0
      }
        
      set ListFreqArray(rx_freq,$i)  [format "%.4f" $rec_freq]
                           
      set gen_freq  [expr $list_freq_rec + $list_freq_offset + [expr $list_freq_incr * $rel_index]]
        
      if { $gen_freq > 2800.0 } \
      {
        set gen_freq  2800.0
      }

      set ListFreqArray(tx_freq,$i)  [format "%.4f" $gen_freq]
              
      set ListFreqArray(offset_freq,$i)  [format "%.4f" $list_freq_offset]
        
      #set ListFreqArray(rx_mode,$i)  ""

      #set ListFreqArray(tx_mode,$i)  ""

      set ListFreqArray(label,$i)  "chan$i"
    }
    
    return
  }


  #**************************************************************************
  #
  #        Name: load_listbox
  #
  #  Parameters: tile_path - tile path of the invoking widget
  #
  # Description: Loads the list values into the listbox
  #
  #**************************************************************************
  proc load_listbox { tile_path } \
  {
    #puts "load_listbox:  $tile_path"
    
    variable  list_max_index
    
    variable  ListFreqArray
    
    set tile_path  [get_tile  $tile_path]
    
    if { ![winfo exists $tile_path] } \
    {
      return
    }

    for { set i 0 } { $i <= $list_max_index } { incr i } \
    {
      if { [info exists ListFreqArray(index,$i)] } \
      {
        # Load the listboxes
        ${tile_path}.lst_list_index   insert  $i  $ListFreqArray(index,$i)
        ${tile_path}.lst_list_index   delete  [expr $i + 1]

        ${tile_path}.lst_list_rec     insert  $i  $ListFreqArray(rx_freq,$i)
        ${tile_path}.lst_list_rec     delete  [expr $i + 1]

        ${tile_path}.lst_list_gen     insert  $i  $ListFreqArray(tx_freq,$i)
        ${tile_path}.lst_list_gen     delete  [expr $i + 1]

        ${tile_path}.lst_list_offset  insert  $i  $ListFreqArray(offset_freq,$i)
        ${tile_path}.lst_list_offset  delete  [expr $i + 1]

        #${tile_path}.lst_list_rx_mode  insert  $i  $ListFreqArray(rx_mode,$i)
        #${tile_path}.lst_list_rx_mode  delete  [expr $i + 1]

        #${tile_path}.lst_list_tx_mode  insert  $i  $ListFreqArray(tx_mode,$i)
        #${tile_path}.lst_list_tx_mode  delete  [expr $i + 1]

        ${tile_path}.lst_list_label   insert  $i  $ListFreqArray(label,$i)
        ${tile_path}.lst_list_label   delete  [expr $i + 1]
      }
    }
    
    return
  }


  #**************************************************************************
  #
  #        Name: update_list_index
  #
  #  Parameters: tile_path - tile path of the invoking widget
  #
  # Description: Updates the display values while the index is changing
  #
  #**************************************************************************
  proc update_list_index { tile_path } \
  {
    #puts "update_list_index:  $tile_path"
    
    variable  list_freq_index
    variable  list_freq_rec
    variable  list_freq_gen
    variable  list_freq_offset
    variable  list_freq_label
    
    variable  narrative_text
    
    variable  ListFreqArray

    set tile_path  [get_tile  $tile_path]
    
    if { ![winfo exists $tile_path] } \
    {
      return
    }

    if { $list_freq_index == "" } \
    {
      set list_freq_index  0
    }
    
    if { [info exists ListFreqArray(index,$list_freq_index)] } \
    {
      set list_freq_rec     $ListFreqArray(rx_freq,$list_freq_index)
      set list_freq_gen     $ListFreqArray(tx_freq,$list_freq_index)
      set list_freq_offset  $ListFreqArray(offset_freq,$list_freq_index)
      #set list_freq_rx_mode   $ListFreqArray(rx_mode,$list_freq_index)
      #set list_freq_tx_mode   $ListFreqArray(tx_mode,$list_freq_index)
      set list_freq_label   $ListFreqArray(label,$list_freq_index)

      ${tile_path}.lst_list_index    see  $list_freq_index
      ${tile_path}.lst_list_rec      see  $list_freq_index
      ${tile_path}.lst_list_gen      see  $list_freq_index
      ${tile_path}.lst_list_offset   see  $list_freq_index
      #${tile_path}.lst_list_rx_mode  see  $list_freq_index
      #${tile_path}.lst_list_tx_mode  see  $list_freq_index
      ${tile_path}.lst_list_label    see  $list_freq_index
    }
    
    set narrative_text  [format "with data starting at Index %4s" $list_freq_index]
    
    return
  }


  #**************************************************************************
  #
  #        Name: set_list_freq_rec
  #
  #  Parameters: tile_path - tile path of the invoking widget
  #
  # Description: Set the Generator freq to changes in the Receiver freq,
  #              and replace the value in the Receiver listbox
  #
  #**************************************************************************
  proc set_list_freq_rec { tile_path } \
  {
    #puts "set_list_freq_rec:  $tile_path"
    
    variable  list_freq_index
    variable  list_freq_rec
    variable  list_freq_gen
    variable  list_freq_offset

    variable  ListFreqArray

    set tile_path  [get_tile  $tile_path]

    if { ![winfo exists $tile_path] } \
    {
      return
    }

    set list_freq_rec  [format "%.4f" $list_freq_rec]

    # Replace the value in the listbox with the new value
    set list_freq_index  [expr int($list_freq_index)]
    
    ${tile_path}.lst_list_rec  insert  $list_freq_index  $list_freq_rec
    
    ${tile_path}.lst_list_rec  delete  [expr $list_freq_index + 1]
    
    set ListFreqArray(rx_freq,$list_freq_index)  $list_freq_rec


    set list_freq_gen  [expr $list_freq_rec + $list_freq_offset]
    
    if { $list_freq_gen < 0 } \
    {
      set list_freq_gen  0

      # Change the Offset value
      # Since the Generator freq is 0, the Offset is the Receiver freq
      set list_freq_offset  $list_freq_rec

      set list_freq_offset  [format "%.4f" $list_freq_offset]

      # Replace the value in the listbox with the new value
      ${tile_path}.lst_list_offset  insert  $list_freq_index  $list_freq_offset

      ${tile_path}.lst_list_offset  delete  [expr $list_freq_index + 1]
    
      set ListFreqArray(offset_freq,$list_freq_index)  $list_freq_offset
    }

    set list_freq_gen  [format "%.4f" $list_freq_gen]
    
    # Replace the value in the listbox with the new value
    ${tile_path}.lst_list_gen  insert  $list_freq_index  $list_freq_gen

    ${tile_path}.lst_list_gen  delete  [expr $list_freq_index + 1]
    
    set ListFreqArray(tx_freq,$list_freq_index)  $list_freq_gen

    return
  }


  #**************************************************************************
  #
  #        Name: set_list_freq_gen
  #
  #  Parameters: tile_path - tile path of the invoking widget
  #
  # Description: Set the Receiver freq to changes in the Generator freq,
  #              and replace the value in the Generator listbox
  #
  #**************************************************************************
  proc set_list_freq_gen { tile_path } \
  {
    #puts "set_list_freq_gen:  $tile_path"
    
    variable  list_freq_index
    variable  list_freq_rec
    variable  list_freq_gen
    variable  list_freq_offset

    variable  ListFreqArray

    set tile_path  [get_tile  $tile_path]

    if { ![winfo exists $tile_path] } \
    {
      return
    }

    set list_freq_gen  [format "%.4f" $list_freq_gen]

    # Replace the value in the listbox with the new value
    set list_freq_index  [expr int($list_freq_index)]

    ${tile_path}.lst_list_gen  insert  $list_freq_index  $list_freq_gen

    ${tile_path}.lst_list_gen  delete  [expr $list_freq_index + 1]
    
    set ListFreqArray(tx_freq,$list_freq_index)  $list_freq_gen


    # Change the Offset value
    set list_freq_offset  [expr $list_freq_gen - $list_freq_rec]
    
    set list_freq_offset  [format "%.4f" $list_freq_offset]

    # Replace the value in the listbox with the new value
    ${tile_path}.lst_list_offset  insert  $list_freq_index  $list_freq_offset

    ${tile_path}.lst_list_offset  delete  [expr $list_freq_index + 1]
    
    set ListFreqArray(offset_freq,$list_freq_index)  $list_freq_offset

    return
  }


  #**************************************************************************
  #
  #        Name: set_list_freq_offset
  #
  #  Parameters: tile_path - tile path of the invoking widget
  #
  # Description: Set the Generator and Receiver freq's to changes in the 
  #              Offset freq, and replace the value in the Offset listbox
  #
  #**************************************************************************
  proc set_list_freq_offset { tile_path } \
  {
    #puts "set_list_freq_offset:  $tile_path"
    
    variable  list_freq_index
    variable  list_freq_rec
    variable  list_freq_gen
    variable  list_freq_offset

    variable  ListFreqArray

    set tile_path  [get_tile  $tile_path]

    if { ![winfo exists $tile_path] } \
    {
      return
    }

    set list_freq_offset  [format "%.4f" $list_freq_offset]

    # Replace the value in the listbox with the new value
    set list_freq_index  [expr int($list_freq_index)]

    ${tile_path}.lst_list_offset  insert  $list_freq_index  $list_freq_offset

    ${tile_path}.lst_list_offset  delete  [expr $list_freq_index + 1]
    
    set ListFreqArray(offset_freq,$list_freq_index)  $list_freq_offset


    set list_freq_gen  [expr $list_freq_rec + $list_freq_offset]

    if { $list_freq_gen < 0 } \
    {
      set list_freq_gen  0

      # Change the Offset value
      # Since the Generator freq is 0, the Offset is the Receiver freq
      set list_freq_offset  $list_freq_rec

      set list_freq_offset  [format "%.4f" $list_freq_offset]

      # Replace the value in the listbox with the new value
      ${tile_path}.lst_list_offset  insert  $list_freq_index  $list_freq_offset

      ${tile_path}.lst_list_offset  delete  [expr $list_freq_index + 1]
    
      set ListFreqArray(offset_freq,$list_freq_index)  $list_freq_offset
    }

    set list_freq_gen  [format "%.4f" $list_freq_gen]

    # Replace the value in the listbox with the new value
    ${tile_path}.lst_list_gen  insert  $list_freq_index  $list_freq_gen

    ${tile_path}.lst_list_gen  delete  [expr $list_freq_index + 1]
    
    set ListFreqArray(tx_freq,$list_freq_index)  $list_freq_gen


    return
  }


  #**************************************************************************
  #
  #        Name: set_list_freq_label
  #
  #  Parameters: tile_path - tile path of the invoking widget
  #
  # Description: Replace the value in the Label listbox
  #
  #**************************************************************************
  proc set_list_freq_label { tile_path } \
  {
    #puts "set_list_freq_label:  $tile_path"
    
    variable  list_freq_index
    variable  list_freq_label

    variable  ListFreqArray

    set tile_path  [get_tile  $tile_path]

    if { ![winfo exists $tile_path] } \
    {
      return
    }

    # Limit the string to 8 characters
    set list_freq_label  [string range "$list_freq_label"  0  7]
    
    # Replace the value in the listbox with the new value
    set list_freq_index  [expr int($list_freq_index)]

    ${tile_path}.lst_list_label  insert  $list_freq_index  "$list_freq_label"

    ${tile_path}.lst_list_label  delete  [expr $list_freq_index + 1]
    
    set ListFreqArray(label,$list_freq_index)  "$list_freq_label"


    return
  }



 
  #**************************************************************************
  #
  #        Name: config_freq_list
  #
  #  Parameters: tile_path - tile path of the invoking widget
  #
  # Description: Configures the freq list from the Index value indicated
  #              then adjusts the list box display to see the Index values
  #
  #**************************************************************************
  proc config_freq_list { tile_path } \
  {
    #puts "config_freq_list:  $tile_path"
    
    variable  list_freq_index
    
    set tile_path  [get_tile  $tile_path]

    if { ![winfo exists $tile_path] } \
    {
      return
    }

    ${tile_path}.lst_list_index   see  $list_freq_index
    ${tile_path}.lst_list_rec     see  $list_freq_index
    ${tile_path}.lst_list_gen     see  $list_freq_index
    ${tile_path}.lst_list_offset  see  $list_freq_index
    ${tile_path}.lst_list_label   see  $list_freq_index
    
    fill_array  $list_freq_index
    
    load_listbox  $tile_path
  }


  #**************************************************************************
  #
  #        Name: freq_list_save
  #
  #  Parameters: widget - tile path of the invoking widget
  #
  # Description: Display the Save Freq List toplevel
  #
  #**************************************************************************
  proc freq_list_save { widget } \
  {
    #puts "freq_list_save:  $widget"
    
    variable  file_label
    variable  file_prefix
    variable  file_ext
    variable  save_flag
    variable  working_dir
    variable  current_file_name

    set file_buffer  [create_file_buffer]
    
    set save_flag  [File_SaveUI::display_file_save  $widget        \
                                                    $working_dir   \
                                                    $file_buffer   \
                                                    $file_prefix   \
                                                    "$file_label"  \
                                                    $file_ext      \
                                                    NO_ENCRYPT     \
                                                    NO_CSV         \
                                                    NO_MODES       \
                                                    NO_OPT_EXT]
                                                      
    # Save mem - unset buffer
    set file_buffer  {}
    
    # Get the file label after the file has been saved
    set file_label  $::File_SaveUI::file_label
    if {$save_flag} \
    {
       set current_file_name $File_SaveUI::file_name
    }
  }
  

  #**************************************************************************
  #
  #        Name: create_file_buffer
  #
  #  Parameters: 
  #
  # Description: Creates the file data buffer to save to the file
  #
  #**************************************************************************
  proc create_file_buffer { } \
  {
    #puts "create_file_buffer"
    
    variable  freq_list_label
    variable  freq_list_date
    
    variable  ListFreqArray
    
    variable  list_max_index

    set file_buffer  ""
    
    if { [array exists ListFreqArray] } \
    {
      # If the array exists, save it to the file
      if { [info exists ListFreqArray(index,0)] } \
      {
        for { set i  0 } { $i <= $list_max_index } { incr i } \
        {
          append file_buffer  "INDEX  $ListFreqArray(index,$i)  "
          append file_buffer  "RX_FREQ  $ListFreqArray(rx_freq,$i)  "
          append file_buffer  "TX_FREQ  $ListFreqArray(tx_freq,$i)  "
          append file_buffer  "OFFSET_FREQ  $ListFreqArray(offset_freq,$i)  "
          append file_buffer  "RX_MODE  $ListFreqArray(rx_mode,$i)  "
          append file_buffer  "TX_MODE  $ListFreqArray(tx_mode,$i)  "
          append file_buffer  "LABEL  \"$ListFreqArray(label,$i)\" \n"
        }
      }
    }
    
    #puts "file_buffer:  $file_buffer"
          
    return  $file_buffer
  }


  #**************************************************************************
  #
  #        Name: freq_list_recall
  #
  #  Parameters: widget - tile path of the invoking widget
  #
  # Description: Recall the frequency list
  #
  #**************************************************************************
  proc freq_list_recall { widget } \
  {
    #puts "freq_list_recall:  $widget"
    
    variable  file_prefix
    variable  file_ext
    variable  list_max_index
    variable  working_dir
    variable  freq_list_label
    variable  freq_list_date
    variable  file_label

    set tile_path  [get_tile  $widget]
    
    set file_buffer  [File_RecallUI::display_file_recall  $widget       \
                                                          $working_dir  \
                                                          $file_prefix  \
                                                          $file_ext     ]
    if {$File_RecallUI::recall_flag == 1} \
    {
        variable current_file_name
        set current_file_name  $File_RecallUI::file_name
    }
    set freq_list_label $::File_RecallUI::file_label
    set freq_list_date  $::File_RecallUI::file_date
    set file_label $freq_list_label
    parse_freq_list  $file_buffer

    # Load the new values
    load_listbox  $tile_path

    # Update the values displayed for the given index
    update_list_index  $tile_path
  }
  

  #**************************************************************************
  #
  #        Name: parse_freq_list
  #
  #  Parameters: file_buffer
  #
  # Description: 
  #
  #**************************************************************************
  proc parse_freq_list { file_buffer } \
  {

    variable  ListFreqArray
                         
    # If the recalled results are valid, display them
    if { $file_buffer == "" } \
    {
      variable current_file_name
      set current_file_name ""
      return
    }
    
    # locate the file label and data, and set those variables
    variable  file_label
    variable  file_date
    
    set lines [split $file_buffer \n]
        
    foreach line $lines \
    {
       switch [lindex $line 0] \
       {
          "FILE_LABEL" {regexp {FILE_LABEL[ ]*(.*)} $line foo file_label}
          "FILE_DATE"  {regexp {FILE_DATE[ ]*(.*)} $line foo file_date}
          "INDEX" \
          {
            foreach  { type  value } $line \
            {
		      switch -- $type \
		      {
		        "INDEX" \
		        {
		            set index  $value
		            set ListFreqArray(index,$index)        $value
		        }
		        "RX_FREQ" \
		        {
		            set ListFreqArray(rx_freq,$index)      $value
		        }
		        "TX_FREQ" \
		        {
		           set ListFreqArray(tx_freq,$index)      $value
		        }
		        "OFFSET_FREQ" \
		        {
		           set ListFreqArray(offset_freq,$index)  $value
		        }
		        "RX_MODE" \
		        {
		           set ListFreqArray(rx_mode,$index)      $value
		        }
		        "TX_MODE" \
		        {
		           set ListFreqArray(tx_mode,$index)      $value
		        }
		        "LABEL" \
		        {
		            set ListFreqArray(label,$index)        "$value"
		        }
		      }
		    }
          }
       }
    }
  }


  #**************************************************************************
  #
  #        Name: reload_freq_list
  #
  #  Parameters: file_name
  #
  # Description: 
  #
  #**************************************************************************
  proc reload_freq_list { file_name } \
  {
    variable current_file_name
    #puts "reload_freq_list:  $file_name"
    
    if {$current_file_name == $file_name} { return }
    
    if { ![file exists   $file_name] } \
    { 
      return 
    }
    
    if { ![file isfile   $file_name] } \
    { 
      return 
    }
    
    if { ![file readable $file_name] } \
    { 
      return 
    }
    
    if { [catch { set fd [open $file_name r] } err] } \
    {
      return
    }
    
    if { [catch { set file_buffer [read $fd] } err] } \
    {
      catch { close $fd }

      return
    }      
    
    catch { close $fd }
    
    # save the name of the active list
    set current_file_name  $file_name
    
    parse_freq_list  $file_buffer
  }


  #**************************************************************************
  #
  #        Name: save_direct
  #
  #  Parameters: file_name  - file name including path
  #              file_label - label of the file
  #              file_data  - data to be saved in the file
  #
  # Description: Saves the Freq List directly without user intervention
  #
  #**************************************************************************
  proc save_direct { file_name  file_label  file_data } \
  {
    #puts "save_direct:  $file_name,  $file_label"
    
    # Determine if the directory for the file exists
    set dir_path  [file dirname  $file_name]
    
    # Make the directory
    # An error is not generated if the directory exists
    file mkdir  $dir_path
    
    # Determine if the file exists
    set file_id  [open  $file_name  w]
        
    # Save the file label and date
    set file_date  [clock format [clock seconds]]
      
    # Write the file
    set value   "  FILE_LABEL   {$file_label}"
    set value  [subst $value]
    puts  $file_id  "$value"
    
    set value   "  FILE_DATE   {$file_date}"
    set value  [subst $value]
    puts  $file_id  "$value"

    puts  $file_id  ""
      
    # Write the data to the file
    puts  $file_id  "$file_data"

    # Close the file
    close $file_id
    
    # save the name of the active list
    variable current_file_name
    set current_file_name  $file_name

  }
  
  
  #**************************************************************************
  #
  #        Name: getSaveState
  #
  #  Parameters: none
  #
  # Description: This function returns a list of TCL commands which can be 
  #              used to set the tile state
  #
  #**************************************************************************
  proc getSaveState { } \
  {
    variable current_file_name
    
    set returnList  [namespace code {reload_freq_list {$current_file_name}}]
   
    return [subst -nocommands $returnList]
  }
 
  #**************************************************************************
  #
  #        Name: Sync
  #
  #  Parameters: none
  #
  # Description: Get the saved state
  #
  #**************************************************************************
  proc Sync { } \
  {
    eval [getSaveState]
  }


  #**************************************************************************
  #
  #        Name: SyncAll
  #
  #  Parameters: none
  #
  # Description: Insures that TCL, the engine and hardware are in sync
  #
  #**************************************************************************
  proc SyncAll { } \
  {
    if { ![interp issafe] } \
    {
      Sync
    }
  }


  #**************************************************************************
  #
  # Initialize the namespace and activate the callbacks
  #
  #**************************************************************************
  Load::on_system_done { Freq_ListUI::SyncAll }


  #**************************************************************************
  #
  # Initialize the frequency list
  #
  #**************************************************************************
  init_array
  

};  # End of Freq_ListUI namespace


#****************************************************************************
# This registers the engine with the list for saving instrument states
# NOTE: it may look strange that we are registering as an ENGINE when we
#       look like a tile, but since we provide services to the engines, we
#       need to restore with the engines.
#****************************************************************************
namespace eval ::SavedStates \
{
  variable engines   

  lappend engines  Freq_ListUI
}

