namespace eval Tone_SignalingUI \
{
  variable  last_focus   ""
  variable  please_wait  ""
  
  proc validate { lun key } \
  {
     set format $::FGen::State($lun,TSIG,format)
     switch $format \
     {
        "EEA" \
        {
           if {[string first $key "0123456789AGR"] != -1} \
           {
              return 1
           }
        }
        "USER" \
        {
           if {[string first $key "0123456789AGR"] != -1} \
           {
              return 1
           }
        }
        
        "EUROSIG" \
        {
           if {[string first $key "0123456789AR"] != -1} \
           {
              return 1
           }
        }
        "MODAT" \
        {
           if {[string first $key "0123456789AR"] != -1} \
           {
              return 1
           }
        }
        default
        {
           if {[string first $key "0123456789GR"] != -1} \
           {
              return 1
           }
        }
     }
     return 0
  }
   
  #***************************************************************************
  #
  # Name:        get_tile
  #
  # Parameters:  widget - any child widget of the tile
  #
  # Description: get the name of the tile's parent frame
  #
  #***************************************************************************
  proc get_tile { widget } \
  {
    while { 1 } \
    {
      switch -- [winfo class $widget] \
      {
        "GEN"          -
        "SMALL_FGEN"   -
        "QTR_FGEN"     -
        "TSIG_CONFIG"  { return $widget}
      }
       
      set widget [winfo parent $widget]
    }
    
  }


  #***************************************************************************
  #
  # Name:        tsig_update_format
  #
  # Parameters:  tile_path - tile path of the invoking widget
  #              lun       - Logical Unit Number of the function generator
  #
  # Description: Updates the display to the Tone Signaling format
  #
  #***************************************************************************
  proc tsig_update_format { tile_path  lun  args } \
  {
    #puts "tsig_update_format:  $tile_path,  $lun,  $args"
    
    set tile_path  [get_tile  $tile_path]
    
    # Stop the trigger event
    FGen::setResponse  $lun  1  {}

    set class  [winfo class $tile_path]
    
    
    set t_sig_format  $::FGen::State($lun,TSIG,format)

    #puts "t_sig_format:  $t_sig_format"
    
    
    if { $t_sig_format == "User_TS" } \
    {
      set ::FGen::State($lun,TSIG,format)  "USER"
      
      # The above change to the variable will retrigger the variable trace
      return
    }      

    if { $t_sig_format == "USER" } \
    {
      # Set the digit freq values to the USER digit freq values - Config toplevel
      if { [winfo exists  ${tile_path}.ent_freq0] } \
      {
        foreach { index } { 0 1 2 3 4 5 6 7 8 9 0 R G A } \
        {
          ${tile_path}.ent_freq${index}  config -textvariable ::User_TS::tones($index)
          ${tile_path}.ent_mark${index}  config -textvariable ::User_TS::mark_timing($index)
          ${tile_path}.ent_space${index} config -textvariable ::User_TS::space_timing($index)
        }

        ${tile_path}.ent_begin_timing config  \
                                         -textvariable ::User_TS::timing(BEGIN)
      }
    } \
    else \
    {
      # Set the digit freq values to the format freq values - Config toplevel
      if { [winfo exists  ${tile_path}.ent_freq0] } \
      {
        foreach { index } { 0 1 2 3 4 5 6 7 8 9 0 R G A } \
        {
          # Set the digit freq values to the format digit freq values
          ${tile_path}.ent_freq${index}  config -textvariable ::${t_sig_format}::tones($index)
          ${tile_path}.ent_mark${index}  config -textvariable ::${t_sig_format}::mark_timing($index)
          ${tile_path}.ent_space${index} config -textvariable ::${t_sig_format}::space_timing($index)
        }

        ${tile_path}.ent_begin_timing config  \
                                  -textvariable ::${t_sig_format}::timing(BEGIN)
      }
    }          
  }


  #***************************************************************************
  #
  # Name:        tsig_set_mode
  #
  # Parameters:  lun - Logical Unit Number of the function generator
  #
  # Description: Sets the Tone Signaling mode
  #
  #***************************************************************************
  proc tsig_set_mode { lun  args } \
  {
    #puts "tsig_set_mode:  $lun,  $args"
    
    set t_sig_mode  $::FGen::State($lun,TSIG,mode)
    
    #puts "t_sig_mode:  $t_sig_mode"
    
    
    if { $t_sig_mode == "CONT" } \
    {

    }
    
    if { $t_sig_mode == "BURST" } \
    {
      # Stop the trigger event
      FGen::setResponse  $lun  1  {}
    }

    return
  }


  #***************************************************************************
  #
  # Name:        tsig_key_return
  #
  # Parameters:  tile_path - tile path of the invoking widget
  #              lun       - Logical Unit Number of the function generator
  #
  # Description: Handles the <Return> key press
  #
  #***************************************************************************
  proc tsig_key_return { lun } \
  {
    tsig_send  $lun
  }

  #***************************************************************************
  #
  # Name:        tsig_send
  #
  # Parameters:  lun - Logical Unit Number of the function generator
  #
  # Description: Send the Tone Signaling code
  #
  #***************************************************************************
  proc tsig_send { lun } \
  {
    #puts "tsig_send:  $lun"
    
    set t_sig_code  $::FGen::State($lun,TSIG,code)

    if { $t_sig_code == "" } \
    {
      return
    }
    
    #puts "t_sig_code:  $t_sig_code"
    
    # Clear the present sequence value list
    set FGen::State($lun,TSIG,seq)  ""


    set t_sig_format  $::FGen::State($lun,TSIG,format)

    if { $t_sig_format == "USER" || $t_sig_format == "User_TS" } \
    {
      # Compose the Tone list
      set ::FGen::State($lun,TSIG,seq)  [User_TS::Compose  $t_sig_code]
    } \
    else \
    {
      # Compose the Tone list for all other formats
      set ::FGen::State($lun,TSIG,seq)  [${t_sig_format}::Compose  $t_sig_code]
    }
    

    # Append the Event item to the list
    lappend  ::FGen::State($lun,TSIG,seq)  [FGen::Event 1]


    set t_sig_mode  $::FGen::State($lun,TSIG,mode)
      
    if { $t_sig_mode == "CONT" } \
    {
      # Setup the trigger event trigger
      FGen::setResponse  $lun  1  "::Tone_SignalingUI::tsig_burst  $lun"
    }
      
    if { $t_sig_mode == "BURST" } \
    {
      # Stop the trigger event
      FGen::setResponse  $lun  1  {}
    }
    
    # Send the burst
    tsig_burst  $lun
  }
  

  #***************************************************************************
  #
  # Name:        tsig_burst
  #
  # Parameters:  lun - Logical Unit Number of the function generator
  #
  # Description: Send a Tone Signaling burst
  #
  #***************************************************************************
  proc tsig_burst { lun } \
  {
    #puts "tsig_burst:  $lun"
    
    #puts "t_sig_seq:  $::FGen::State($lun,TSIG,seq)"
    
    # Send the Tone items
    FGen::Send  $lun  $::FGen::State($lun,TSIG,seq)
  }


  #***************************************************************************
  #
  # Name:        tsig_config
  #
  # Parameters:  tile_path - tile path of the invoking widget
  #              lun       - Logical Unit Number of the function generator
  #
  # Description: Displays the Tone Signaling configuration toplevel
  #
  #***************************************************************************
  proc tsig_config { tile_path  lun } \
  {
    #puts "tsig_config:  $tile_path,  $lun"
    
    variable  last_focus
      
    set last_focus  $tile_path

    set top_level [winfo toplevel $tile_path]

    set tile_path   [name_child  $top_level  t_sig_$lun]

    if { ![winfo exists $tile_path] } \
    {
      set tile_path  [Screens::create_toplevel_tile  $tile_path  TSIG_CONFIG]
    }
    
    # Configure the title to reflect the LUN number
    wm title  $tile_path  "Tone Signaling Config #$lun"

    # Bind the F8 key for a return
    bind $tile_path <KeyRelease-F8> "destroy [winfo toplevel $tile_path]"

    bind $tile_path <<Destroy>> \
                       "Tone_SignalingUI::tsig_config_return  $tile_path  $lun"

    trace variable  ::FGen::State($lun,TSIG,format)  w  \
                       "Tone_SignalingUI::tsig_update_format  $tile_path  $lun"

    tsig_update_format  $tile_path  $lun

    safe_focus_force ${tile_path}.btn_return
  }


  #***************************************************************************
  #
  # Name:        tsig_config_return
  #
  # Parameters:  tile_path - tile path of the invoking widget
  #              lun       - Logical Unit Number of the function generator
  #
  # Description: Handles the <Destroy> event for the Tone Signaling 
  #              configuration toplevel
  #
  #***************************************************************************
  proc tsig_config_return { tile_path  lun } \
  {
    #puts "tsig_config_return:  $tile_path,  $lun"

    variable  last_focus
    variable  please_wait
    
    set please_wait  "Please Wait ..."
    
    trace vdelete  ::FGen::State($lun,TSIG,format)  w  \
                       "Tone_SignalingUI::tsig_update_format  $tile_path  $lun"

    # Stop the trigger event
    FGen::setResponse  $lun  1  {}

    set please_wait  ""
    safe_focus_force $last_focus
  }


  #***************************************************************************
  #
  # Name:        tsig_restore_def
  #
  # Parameters:  lun - Logical Unit Number of the function generator
  #
  # Description: Restore the Tone Signaling configuration defaults
  #
  #***************************************************************************
  proc tsig_restore_def { lun } \
  {
    #puts "tsig_restore_def:  $lun"

    # Get the format to know which defaults to restore
    set t_sig_format  $::FGen::State($lun,TSIG,format)

    if { $t_sig_format == "USER" } \
    {
      foreach { index } { 0 1 2 3 4 5 6 7 8 9 0 R G A } \
      {
        set ::User_TS::tones($index)  $::User_TS::tones(default_${index})
        
        set ::User_TS::mark_timing($index)  \
                                     $::User_TS::mark_timing(default_${index})
                                     
        set ::User_TS::space_timing($index)  \
                                    $::User_TS::space_timing(default_${index})
      }

      set ::User_TS::timing(BEGIN)    $::User_TS::timing(BEGIN_DEF)
    } \
    else \
    {
      foreach { index } { 0 1 2 3 4 5 6 7 8 9 0 R G A } \
      {
        # All other formats
        set ::${t_sig_format}::tones($index)  \
                         [subst $\::${t_sig_format}\::tones(default_${index})]
                         
        set ::${t_sig_format}::mark_timing($index)  \
                   [subst $\::${t_sig_format}\::mark_timing(default_${index})]
                   
        set ::${t_sig_format}::space_timing($index)  \
                  [subst $\::${t_sig_format}\::space_timing(default_${index})]
      }

      set ::${t_sig_format}::timing(BEGIN)  \
                               [subst $\::${t_sig_format}\::timing(BEGIN_DEF)]
    }

  }


};  # End of Tone_SignalingUI namespace

