#############################################################################
#                                                                           #
# IFR SYSTEMS, INC        COPYRIGHT (C)1998,99      DESIGN ENGINEERING      #
# ------------------------------------------------------------------------- #
#                                                                           #
# FILE        : 3050.DvmUI.tcl                                              #
#                                                                           #
# PRODUCT     : FITS                                                        #
#                                                                           #
# AUTHOR      : P. McCormick                                                #
#                                                                           #
# DESCRIPTION : DVM Meter User Interface                                    #
#                                                                           #
#############################################################################

package require Dvm


namespace eval DvmUI \
{

  # Global variables
  variable  LongName     "dvm"
  variable  ShortName    "DVM"
  variable  Size         Eighth_v
            
  variable  StateUI
            
  set StateUI(reading)          0
  set StateUI(reading,color)   green
  set StateUI(range_selection) 0
  set StateUI(range_selection,V)  0
  set StateUI(range_selection,dBm)  0
  set StateUI(auto_range)           "AR"
  set StateUI(next_range)           0
  set StateUI(current_range)        1
  
  set StateUI(hi_limit,on) 0
  set StateUI(lo_limit,on) 0

  set  StateUI(alarm,state)      0
   
  set  StateUI(ref_mode)         "OFF"
  set  StateUI(ref_indicate)     ""
  set  StateUI(ref_reading,dBm)  0.0
  set  StateUI(ref_reading,V)    0.0
  set  StateUI(peak_reading)     0.0

  # The following maintains compatibility with the present 'CanvasWidget'
  set  StateUI(peak_hold)  "OFF"

  set  StateUI(underrange_count) 0

  set  StateUI(load_menu) {150 600 ">1 Meg" User}

  array set ranges \
  {
     V,max   8
     V,0     {100  50    0  25}
     V,1     {100  50    0  25}
     V,2     { 40  20    0  10}
     V,3     { 20  10    0  7}
     V,4     { 10   5    0  2.5}
     V,5     {  4   2    0  1}
     V,6     {  2   1    0  .5}
     V,7     {  1   0.5  0  .25}
     V,8     {  0.4 0.2  0  0}
     
     dBm,max   4
     dBm,0     {40  -10 -60 -15}
     dBm,1     {40  -10 -60 -15}
     dBm,2     {32  -14 -60 -15}
     dBm,3     {12  -24 -60 -35}
     dBm,4     {-8  -34 -60 -60}
  }
  
  
  set  StateUI(reading,max)          100
  set  StateUI(mid_range)            50
  set  StateUI(reading,min)          0
  
            
  # and the vars for the menus
  variable ranges_V
  variable ranges_dBm

  variable ranges_HW
  
  set ranges_V { AR 100V 40V 20V 10V 4V 2V 1V 0.4V }

  set ranges_dBm { AR 40dBm 32dBm 12dBm -8dBm }
  
  set ranges_HW { -400 0 400 40 4 }
  
  
  #********************************************************************
  #
  #       Name: Create  
  #
  # Parameters: size - meter size 
  #
  # Decsription: 
  #
  #********************************************************************
  proc Create { tile_path  { meter_size  small } } \
  {
    #puts "DvmUI::Create:  $tile_path,  $meter_size"

    variable LongName
    
    set top_level [winfo toplevel $tile_path]
 
    set meter_class  ${meter_size}_DVM
    
    set meter_class  [string toupper $meter_class]
    
    if { $meter_size == "small" } \
    {
      set tile_path  [name_child  $top_level $LongName]
      
      if { ![winfo exists $tile_path] } \
      {
        set tile_path  [::Screens::create_tile  $tile_path  $LongName  $meter_class]
      }  
    }
     
    if { $meter_size == "big" } \
    {
      set tile_path   [name_child  $top_level big$LongName]
   
      if { ![winfo exists $tile_path] } \
      {
        set tile_path  [::Screens::create_toplevel_tile  $tile_path  $meter_class]
      }

      # Bind the 'big meter' to the 'F8' key
      bind $tile_path <KeyRelease-F8>  "destroy $tile_path"
    }

    return $tile_path
  }


  #******************************************************************** 
  #
  #       Name: Map
  #
  # Parameters: 
  #
  # Decsription: 
  #
  #********************************************************************
  proc Map { tile_path } \
  {
    #puts "DvmUI::Map:  $tile_path"

    variable  StateUI
    
    trace variable ::Dvm::State(reading)  w  "::DvmUI::Update_reading  $tile_path"
    
    trace variable ::Dvm::State(units) w "::DvmUI::update_meter_units $tile_path"


     
    Update_reading_core  $tile_path
    
    update_meter_units_core $tile_path

        
    if { $::Dvm::State(units) == "V" } \
    {
      set_meter_range_V
    }
     
    if { $::Dvm::State(units) == "dBm" } \
    {
      set_meter_range_dBm
    }
    
          
    set_ref_mode  $StateUI(ref_mode)

    Dvm en  3  -nowait
    Dvm run 1  -nowait
  
    if { [winfo class $tile_path] == "SMALL_DVM" } \
    {
      trace variable ::Dvm::State(peakEnable)  w  "DvmUI::update_peak_hold_small $tile_path"

      update_peak_hold_small_core $tile_path
      
    } \
    else \
    {
      trace variable ::Dvm::State(load)  w "::DvmUI::update_meter_load $tile_path"

      update_meter_load_core $tile_path
    }
  }


  #********************************************************************
  #
  #       Name: Cleanup
  #
  # Parameters: 
  #
  # Description: 
  #
  #********************************************************************
  proc Cleanup { tile_path } \
  {
    trace vdelete ::Dvm::State(reading)  w  "::DvmUI::Update_reading  $tile_path"

    trace vdelete ::Dvm::State(load) w "::DvmUI::update_meter_load $tile_path"

    trace vdelete ::Dvm::State(units) w "::DvmUI::update_meter_units $tile_path"

    if { [winfo class $tile_path] == "SMALL_DVM" } \
    {
      trace vdelete ::Dvm::State(peakEnable)  w  "DvmUI::update_peak_hold_small $tile_path"
    }
  }


  #********************************************************************
  #
  #       Name: Unmap
  #
  # Parameters: 
  #
  # Decsription: 
  #
  #********************************************************************
  proc Unmap { tile_path } \
  {
    #puts "DvmUI::Unmap"
    
    Cleanup $tile_path
  }


  #********************************************************************
  #
  #       Name: Destroy
  #
  # Parameters: 
  #
  # Decsription: 
  #
  #********************************************************************
  proc Destroy { tile_path } \
  {
    #puts "DvmUI::Destroy"
    
    Cleanup $tile_path
  }


  #********************************************************************
  #
  #       Name: Register
  #
  # Parameters: 
  #
  # Decsription: 
  #
  #********************************************************************
  proc Register { } \
  {
    #puts "DvmUI::Register"

      
    return
  }


  #********************************************************************
  #
  #       Name: Update_reading
  #
  # Parameters: 
  #
  # Description: Updates the variables for a reading change
  #
  #********************************************************************
  proc Update_reading { tile_path  args } \
  {
     set cmd "DvmUI::Update_reading_core  $tile_path  $args"
     
     after cancel $cmd
     after idle $cmd
  }
  
  proc Update_reading_core { tile_path  args } \
  {
    variable StateUI
    
    if ![winfo exists $tile_path] { return }
    
    set tile_path  [get_tile  $tile_path]
    
    if { ![winfo exists $tile_path] } \
    {
      return
    }

    if { ![winfo ismapped $tile_path] } \
    {
      #puts "Dvm is not mapped"
      
      return
    }

    # check for alarm states & update as needed
    if { $Dvm::State(status) & 0x200 } \
    {
      set StateUI(reading,color)  red
    } \
    elseif { $Dvm::State(status) & 0x800 } \
    {
      set StateUI(reading,color)  blue
    } \
    elseif { $Dvm::State(status) & 0x101D} \
    {
      set StateUI(reading,color)  yellow
    } \
    else \
    {
      set StateUI(reading,color)  green
    }
    
    if { $::Dvm::State(BellEnabled_text) == "ON"} \
    {
      # turn on alert tone
      set StateUI(alarm,state)  1
    } \
    else \
    {
      # turn off alert tone
      set StateUI(alarm,state)  0
    }
 
    # grab the appropriate reading value & set up for meter bar
    set r  $Dvm::State(reading,$::Dvm::State(units))
    
    if { "$r" != "---" } \
    {
      if { $StateUI(ref_mode) == "ON" } \
      {
        # Adjust the reading by subtracting the reference reading
        set StateUI(reading)  [expr $r - $StateUI(ref_reading,$Dvm::State(units))]
      } \
      else \
      {
        set StateUI(reading)  $r
      }
    } \
    else \
    {
      set StateUI(reading)        $StateUI(reading,min)
      set StateUI(reading,color)  red
    }

    set StateUI(peak_reading) $Dvm::State(peakHi,$::Dvm::State(units))
    
    if { $::Dvm::State(peakEnable) != 0 } \
    {
      set StateUI(reading,displayed) $Dvm::State(peak,$Dvm::State(units))
    } \
    else \
    {
      if { $StateUI(ref_mode) == "ON" } \
      {
        # Adjust the reading by subtracting the reference reading
        if { "$r" != "---" } \
        {
           set StateUI(reading,displayed)  \
                   [format %.2f  [expr $Dvm::State(reading,$Dvm::State(units)) - \
                                                    $StateUI(ref_reading,$Dvm::State(units))]]
        } else \
        {
           set StateUI(reading,displayed) $r
        }
                                          
        # Set the color of the reading to dark blue
        ${tile_path}.lbl_reading  config  -fg "dark blue"
        
        if { [winfo class $tile_path] == "SMALL_DVM" } \
        {
          # Show the Ref Mode indicator on the Small Meter
          # Set the color of the Ref Mode indicator to blue background and gray foreground
          ${tile_path}.lbl_ref_mode_ind  config  -bg "dark blue"  -fg gray
        }
      } \
      else \
      {
        if { "$r" != "---" } \
        {
           set StateUI(reading,displayed)  [format %.2f  $Dvm::State(reading,$Dvm::State(units))]
        } else \
        {
           set StateUI(reading,displayed) $r
        }

        # Set the color of the reading to black
        ${tile_path}.lbl_reading  config  -fg black

        if { [winfo class $tile_path] == "SMALL_DVM" } \
        {
          # Hide the Ref Mode indicator on the Small Meter
          # Set the color of the Ref Mode indicator to gray background and gray foreground
          ${tile_path}.lbl_ref_mode_ind  config  -bg gray  -fg gray
        }
      }
    }
    
    auto_range
  }
  
 
  #********************************************************************
  #
  #       Name: Update_limits
  #
  # Parameters: 
  #
  # Description: Conditions the 1/0 value from the engine to a 
  #            : "ON"/"OFF" and
  #            : "ON"/"OFF" 
  #            : string for the user interface
  #
  #********************************************************************
  proc Update_limits { args } \
  {
    #puts "DvmUI::Update_limits:  $args"
    
    set cmd  "DvmUI::Update_limits_core  $args"
    
    after cancel  $cmd
    
    after idle  $cmd
  }
  

  #********************************************************************
  #
  #       Name: Update_limits_core
  #
  # Parameters: 
  #
  # Description: Conditions the 1/0 value from the engine to a 
  #            : "ON"/"OFF" and
  #            : "ON"/"OFF" 
  #            : string for the user interface
  #
  #********************************************************************
  proc Update_limits_core { args } \
  {
    #puts "DvmUI::Update_limits_core:  $args"

    variable StateUI

    set status  $::Dvm::State(alerts)

    # High Limit
    if { $status & 0x2 } \
    {
      set ::Dvm::State(alertHi_text)  "ON"
      set StateUI(hi_limit,on) 1
    } \
    else \
    {
      set ::Dvm::State(alertHi_text)  "OFF"
      set StateUI(hi_limit,on) 0
    }
   
    # Low limit
    if { $status & 0x1 } \
    {
      set ::Dvm::State(alertLo_text)  "ON"
      set StateUI(lo_limit,on) 1
    } \
    else \
    {
      set ::Dvm::State(alertLo_text)  "OFF"
      set StateUI(lo_limit,on) 0
    }    
  }

  #********************************************************************
  #
  #       Name: get_tile                   
  #
  # Parameters: a widget
  #
  # Description: given widget within a tile, get the name of the tile
  #
  #********************************************************************
  proc get_tile { widget } \
  {
    while {1} \
    {
      if { [catch { set class  [winfo class $widget] } result] } \
      {
        return ""
      }
         
      switch -- $class \
      {
        "SMALL_DVM"  -
        "BIG_DVM"    { return $widget }
      }

      set widget [winfo parent $widget]
    }
    
    return ""
  }
 

  #********************************************************************
  #
  #       Name: update_hi_limit
  #
  # Parameters: 
  #
  # Description: updates the limits of the meter
  #
  #********************************************************************
  proc update_hi_limits { args } \
  {
     set cmd DvmUI::update_hi_limits_core
     after cancel $cmd
     after idle $cmd
  }
  proc update_hi_limits_core { } \
  {
    variable StateUI
    set StateUI(hi_limit,value) $Dvm::State(alertHi,$::Dvm::State(units))
    return
  }
  proc set_hi_limit { } \
  {
     set cmd DvmUI::set_hi_limit_core
     after cancel $cmd
     after idle $cmd
  }
  proc set_hi_limit_core { } \
  {
     variable StateUI

     Dvm::ulimit $StateUI(hi_limit,value) $::Dvm::State(units)
  }
  
  
  #********************************************************************
  #
  #       Name: update_lo_limit
  #
  # Parameters: 
  #
  # Description: updates the limits of the meter
  #
  #********************************************************************
  proc update_lo_limits { args } \
  {
     set cmd DvmUI::update_lo_limits_core
     after cancel $cmd
     after idle $cmd
  }
  proc update_lo_limits_core { } \
  {
    variable StateUI
    set StateUI(lo_limit,value) $Dvm::State(alertLo,$::Dvm::State(units))
    return
  }
  proc set_lo_limit { } \
  {
     set cmd DvmUI::set_lo_limit_core
     after cancel $cmd
     after idle $cmd
  }
  proc set_lo_limit_core { } \
  {
     variable StateUI

     Dvm::llimit $StateUI(lo_limit,value) $::Dvm::State(units)
  }  

  #********************************************************************
  #
  #       Name: toggle_meter_peakhold
  #
  # Parameters:
  # Decsription: Displays PH and sets peak hold variable
  #
  #********************************************************************
  proc toggle_meter_peakhold {  } \
  {
     if {$::Dvm::State(peakEnable) == 1 } \
     {
        Dvm peak 0 -nowait
     } \
     else \
     {
        Dvm peak 1 -nowait
        Dvm phreset  -nowait
     }
  }
  
  
  #********************************************************************
  #
  #       Name: update_mid_range                     
  #
  # Parameters: 
  #
  # Description: sets the meter midrange value
  #
  #********************************************************************
  proc update_mid_range { args } \
  {
    #puts "update_mid_range"

    variable StateUI
    
    set HighRange  $StateUI(reading,max)
    set LowRange   $StateUI(reading,min)

    set modifier_1  ""
    set modifier_2  ""
    
    regexp {([-.0-9]+)}   $HighRange  high_range
    regexp {([a-zA-Z]$)}  $HighRange  modifier_1
    
    regexp {([-.0-9]+)}   $LowRange  low_range
    regexp {([a-zA-Z]$)}  $LowRange  modifier_2

    set mid_range  [format %g [expr [expr $high_range + $low_range] / 2.0]]
    
    set StateUI(mid_range)  ${mid_range}${modifier_1}
  }


  #********************************************************************
  #
  #       Name: set_meter_range                     
  #
  # Parameters: new_value   - requested new value 
  #                         -1 - no new value requested
  #
  # Description: sets meter range, displays AR and sets scale variable
  #
  #********************************************************************
  proc set_meter_range_V { } \
  {
    variable StateUI
    if {$StateUI(range_selection,V) == 0 } \
    {
      set StateUI(auto_range)  "AR"
      set StateUI(current_range) 1
    } \
    else \
    {
      set StateUI(auto_range)  ""
      set StateUI(current_range) $StateUI(range_selection,V)
    }
    update_range
  }
  
  proc set_meter_range_dBm { } \
  {
    variable StateUI
    if {$StateUI(range_selection,dBm) == 0 } \
    {
      set StateUI(auto_range)  "AR"
      set StateUI(current_range) 1
    } \
    else \
    {
      set StateUI(auto_range)  ""
      set StateUI(current_range) $StateUI(range_selection,dBm)
    }
    update_range
  }
  
  proc update_range { } \
  {
    variable StateUI
    variable ranges
    variable ranges_HW
  
    if {$StateUI(current_range) < 1 } \
    {
       set StateUI(current_range) 1
    }
    if ![info exists ranges($::Dvm::State(units),$StateUI(current_range))] \
    {
       set StateUI(current_range) 1
    }
    
    upvar #0 DvmUI::ranges($::Dvm::State(units),$StateUI(current_range)) range
    
    set StateUI(reading,max) [lindex $range 0]
    #set StateUI(mid_range)   [lindex $range 1]
    set StateUI(reading,min) [lindex $range 2]
    set StateUI(next_range)  [lindex $range 3]
    
    if {$::Dvm::State(units) == "V"} {
      if {$StateUI(current_range) <= 1} {
              set hwrange [lindex $ranges_HW 2]   
      } elseif {$StateUI(current_range) <= 4} {
          set hwrange [lindex $ranges_HW 3]
      } elseif {$StateUI(current_range) <= 7} {
          set hwrange [lindex $ranges_HW 4]
      } else {
          set hwrange [lindex $ranges_HW 0]
      }  
     
    } elseif {$::Dvm::State(units) == "dBm" } {
       if {$StateUI(current_range) == 0} {
              set hwrange [lindex $ranges_HW 1]   
       } elseif {$StateUI(current_range) == 1} {
          set hwrange [lindex $ranges_HW 2]
       } elseif {$StateUI(current_range) == 2} {
          set hwrange [lindex $ranges_HW 3]
       } elseif {$StateUI(current_range) == 4} {
          set hwrange [lindex $ranges_HW 4]
       } else {
          set hwrange [lindex $ranges_HW 0]
       }  
    }
    
    # Commented out 7/15/02 for further discussion.
    #Dvm hwrange $hwrange -nowait 
     
  }
  
  proc auto_range { } \
  {
     variable StateUI
     variable ranges
     if {$StateUI(range_selection,$::Dvm::State(units)) != 0 } { return }

     # use absolute value to check for the range
     set absReading [expr abs($StateUI(reading))]
     # handle overranges
     
     while {$absReading > $StateUI(reading,max) } \
     {
        if {$StateUI(current_range) <= 1 } \
        {
           return
        }
        incr StateUI(current_range) -1
        update_range
     }
     
     # handle underranges
     if {$absReading > $StateUI(next_range) } \
     {
        set StateUI(underrange_count) 0
        return
     }
     incr StateUI(underrange_count)
     if {$StateUI(underrange_count) < 10 } \
     {
        return
     }
     if {$StateUI(current_range) < $ranges($::Dvm::State(units),max) } \
     {
        incr StateUI(current_range)
        update_range
     }
  }
  
  
  #********************************************************************
  #
  #       Name: set_meter_load
  #
  # Parameters: new_value   - requested new value 
  #                           -1 - no new value requested
  #
  # Decsription: sets meter load
  #
  #********************************************************************
  proc set_meter_load {} \
  {
    variable StateUI
     
    switch -- $StateUI(load) \
    {
      0       { Dvm load 150 }
      1       { Dvm load 600 }
      2       { Dvm load 1000000 }
       
      default { Dvm load 0 }
    }
  }
  
  
  proc update_load { args } \
  {
     #puts "update_load:  $args"

     set cmd "DvmUI::update_load_core"
     after cancel $cmd
     after idle $cmd
  }
  
  proc update_load_core { } \
  {
    variable StateUI
  
    switch -- $Dvm::State(load) \
    {
      150     { set StateUI(load) 0 }
      600     { set StateUI(load) 1 }
      1000000 { set StateUI(load) 2 }
      0       { set StateUI(load) 3 }
    }
  
    set StateUI(load,text) [lindex $StateUI(load_menu) $StateUI(load)]
  }
  
  
  proc update_meter_load { widget args } \
  {
    #puts "update_meter_load:  $widget,  $args"
  
    set cmd "DvmUI::update_meter_load_core $widget"
     
    after cancel $cmd
     
    after idle $cmd
  }
  
  proc update_meter_load_core { tile_path } \
  {
  
    variable StateUI

    set tile_path   [get_tile  $tile_path]
    

    switch -- $Dvm::State(load) \
    {
      0  { ::Screens::position_widget ${tile_path}.ent_user_load
           ::Screens::position_widget ${tile_path}.lbl_user_load }
          
      default { grid forget ${tile_path}.ent_user_load
                grid forget ${tile_path}.lbl_user_load }
    }
  }
  
  
  #********************************************************************
  #
  #       Name: Update_limit_values
  #
  # Parameters: 
  #
  # Decsription: updates the meter limits when the resistance changes
  #
  #********************************************************************
  proc Update_limit_values { args } \
  {
     set cmd "DvmUI::Update_limit_values_core"
     after cancel $cmd
     after idle $cmd
  }
  proc Update_limit_values_core { } \
  {
  
     variable StateUI
     if {"$::Dvm::State(units)" != "dBm" } { return }
     # OK, resend the limits
     set_hi_limit_core
     set_lo_limit_core
  }


  #********************************************************************
  #
  #        Name: update_peak_hold_small
  #
  #  Parameters:
  #
  # Description: Updates the Peak Hold button on the small meter 
  #              to changes in the 'peakEnable' state variable
  #
  #********************************************************************
  proc update_peak_hold_small { tile_path  args } \
  {
    set cmd  "::DvmUI::update_peak_hold_small_core  $tile_path"
    
    after cancel  $cmd
    
    after idle  $cmd
  }
  
  proc update_peak_hold_small_core { tile_path } \
  {
    #puts "update_peak_hold_small_core:  $tile_path"

    #variable StateUI
    
    set tile_path  [get_tile  $tile_path]
    
    if { ![winfo exists $tile_path] } \
    { 
      return 
    }

    set btn_peak_hold  ${tile_path}.btn_peak_hold
 
    if { ![winfo exists $btn_peak_hold] } \
    { 
      return 
    }

    switch -- $::Dvm::State(peakEnable) \
    {
      1  { $btn_peak_hold  config  -relief sunken }
         
      0  { $btn_peak_hold  config  -relief raised }
    }
  }


  #********************************************************************
  #
  #       Name: set_meter_units
  #
  # Parameters: new_value   - requested new value 
  #                           -1 - no new value requested
  #
  # Decsription: sets meter units
  #
  #********************************************************************
  proc set_meter_units { } \
  {
    variable StateUI
    
    if {"$::Dvm::State(units)" == "V" } \
    {
       set ::Dvm::State(units) dBm
    } \
    else \
    {
       set ::Dvm::State(units) V
    }
  }
  
  proc update_meter_units { tile_path args } \
  {
     set cmd "::DvmUI::update_meter_units_core $tile_path"
     after cancel $cmd
     after idle $cmd
  }
  
  proc update_meter_units_core { tile_path } \
  {
     variable StateUI
     $tile_path.mnu_range config -menu $tile_path.mnu_range.menu_$::Dvm::State(units)

    update_range
    set_meter_range_$::Dvm::State(units)
    update_lo_limits_core
    update_hi_limits_core
    
    Update_reading_core  $tile_path
  }
  

  #********************************************************************
  #
  #       Name: set_ref_mode
  #
  # Parameters: 
  #
  # Description: sets the Reference Mode of the meter
  #
  #********************************************************************
  proc set_ref_mode { { new_value "" } } \
  {
    #puts "set_ref_mode:  $new_value"
    
    variable  StateUI
    
    if { $new_value == "" } \
    {
      # Toggle the reference mode
      if { $StateUI(ref_mode) == "OFF" } \
      {
        set StateUI(ref_mode)  "ON"
      } \
      else \
      {
        set StateUI(ref_mode)  "OFF"
      }
    } \
    else \
    {
      # Set the reference mode
      set StateUI(ref_mode)  $new_value
    }       
    

    if { $StateUI(ref_mode) == "ON" } \
    {
      set StateUI(ref_indicate)  "REF"
      
      # Set the ref_reading value if the present value is 0.0,
      # which is the initialization value
      if { $StateUI(ref_reading,dBm) == 0.0 } \
      {
        set StateUI(ref_reading,dBm)   $::Dvm::State(reading,dBm)
      }

      if { $StateUI(ref_reading,V) == 0.0 } \
      {
        set StateUI(ref_reading,V)   $::Dvm::State(reading,V)
      }
    } \
    else \
    {
      set StateUI(ref_indicate)     ""

      set StateUI(ref_reading,dBm)  0.0
      set StateUI(ref_reading,V)    0.0
    }
  
    return
  }


  trace variable Dvm::State(load) w DvmUI::update_load

  trace variable Dvm::State(alertHi) w DvmUI::update_hi_limits
  trace variable Dvm::State(alertLo) w DvmUI::update_lo_limits

  trace variable ::Dvm::State(alerts)   w  ::DvmUI::Update_limits
  trace variable ::Dvm::State(current_ohms)  w  ::DvmUI::Update_limit_values

  Load::on_system_done \
  {
    namespace eval DvmUI \
    {
      update_load
      Update_limits
      update_hi_limits
      update_lo_limits
    }
  }



#****************************************************************************
# Name:        getSaveState
# Parameters:  none
# Description: This function returns a list of TCL commands which can be used
#              to set the DistortionUI tiles to its current state
#****************************************************************************
 proc getSaveState { } {
   
   variable StateUI
 
   set returnList \
   {
     set ::DvmUI::StateUI(hi_limit,on)  {$StateUI(hi_limit,on)}
     set ::DvmUI::StateUI(lo_limit,on)  {$StateUI(lo_limit,on)}
     set ::DvmUI::StateUI(range_selection,V)  {$StateUI(range_selection,V)}
     set ::DvmUI::StateUI(range_selection,dBm)  {$StateUI(range_selection,dBm)}
     set ::DvmUI::StateUI(ref_mode) {$StateUI(ref_mode)}
   }
       
   return [subst -nocommands $returnList]
 }

#****************************************************************************
# Name:        Sync
# Parameters:  none
# Description: Write state variables to DvmUI
#****************************************************************************
 proc Sync { } {
   eval [getSaveState]
 }
 
 trace variable ::DvmUI::StateUI(reading,max)  w  "::DvmUI::update_mid_range"
 update_mid_range

};  # End of DvmUI namespace

#****************************************************************************
# This registers the tiles with the list for saving instrument states
#****************************************************************************
namespace eval ::SavedStates {

   variable tiles   
   lappend tiles DvmUI
}



namespace eval View \
{
   lappend PreCreate DvmUI
}





