#############################################################################
#                                                                           #
# IFR SYSTEMS, INC        COPYRIGHT (C)1998,99      DESIGN ENGINEERING      #
# ------------------------------------------------------------------------- #
#                                                                           #
# FILE        : 3060.PowerUI.tcl                                            #
#                                                                           #
# PRODUCT     : FITS                                                        #
#                                                                           #
# AUTHOR      : P. McCormick                                                #
#                                                                           #
# DESCRIPTION : Power Meter User Interface                                  #
#                                                                           #
#############################################################################

package require Power

namespace eval PowerUI \
{

  # Global variables
  variable  LongName     "power"
  variable  ShortName    "Power"
  variable  Size         Eighth_v
            

  variable  StateUI

  set  StateUI(textreading)          0
  set  StateUI(reading,max)          200
  set  StateUI(reading,next_range)          0
  set  StateUI(reading,middle)          0
  set  StateUI(reading,min)          0
  set  StateUI(reading)          0
  set  StateUI(reading,count)          0
  set  StateUI(reading,color)   red
  set  StateUI(peak_reading)     0
  set  StateUI(hi_limit,on) 0
  set  StateUI(hi_limit,value) 0
  set  StateUI(lo_limit,on) 0
  set  StateUI(lo_limit,value) 0
  set  StateUI(alarm,enable) 0
  set  StateUI(alarm,state) 0
  set  StateUI(range,auto) 0
  set  StateUI(range,W) 0
  set  StateUI(range,dBm) 0
  set  StateUI(alertLo,dBm) 0
  set  StateUI(alertHi,dBm) 0
  set  StateUI(alertLo,W) 0
  set  StateUI(alertHi,W) 0
  set  StateUI(range_selection,W) 0
  set  StateUI(range_selection,dBm) 0
  set  StateUI(peak_hold)        "OFF"
  set  StateUI(units)            "W"
            
  set  StateUI(ref_mode)         "OFF"
  set  StateUI(ref_indicate)     ""
  set  StateUI(ref_reading)      0.0
  set  StateUI(high_limit_set)   "OFF"
  set  StateUI(low_limit_set)    "OFF"
  set  StateUI(average)          ""
  set  StateUI(cable_loss)       0.0

  array set ranges \
  {
     W,max   11
     W,0     {200 100    0  75}
     W,1     {200 100    0  75}
     W,2     {100  50    0  37.5}
     W,3     { 50  25    0  15}
     W,4     { 20  10    0  7}
     W,5     { 10   5    0  3.75}
     W,6     {  5   2.5  0  1.5}
     W,7     {  2   1    0  .75}
     W,8     {  1   0.5  0  .375}
     W,9     {  0.5 0.25 0  .15}
     W,10    {  0.2 0.1  0  .075}
     W,11    {  0.1 0.05 0  0}
     
     dBm,max   2
     dBm,0     {50  25 0 20}
     dBm,1     {50  25 0 20}
     dBm,2     {30  15 0 0 }
  }
  # and the vars for the menus
  variable ranges_W
  variable ranges_dBm
  set ranges_W \
  {
     AR 200 100 50 20 10 5 2 1 0.5 0.2 0.1
  }
   set ranges_dBm \
  {
     AR +50 +30
  }
  
  #********************************************************************
  #
  #       Name: get_tile                   
  #
  # Parameters: a widget
  #
  # Description: given widget within a tile, get the name of the tile
  #
  #********************************************************************
  proc get_tile { widget } \
  {
    while {1} \
    {
      if { [catch { set class  [winfo class $widget] } result] } \
      {
        return ""
      }
         
      switch -- $class \
      {
        "SMALL_POWER" -
        "BIG_POWER"   { return $widget }
      }

      set widget [winfo parent $widget]
    }
    
    return ""
  }

 
  #********************************************************************
  #
  #       Name: set_meter_cable_loss
  #
  # Parameters: new_value   - requested new value 
  #                           -1 - no new value requested
  #
  # Decsription: sets meter cable loss
  #
  #********************************************************************
  proc set_meter_cable_loss {  } \
  {
    # Set the meter engine
    #Power  cableloss  $StateUI(cable_loss)  -nowait

    return
  }

   
  #********************************************************************
  #
  #       Name: Create  
  #
  # Parameters: size - meter size 
  #
  # Decsription: 
  #
  #********************************************************************
  proc Create { tile_path  { meter_size  small } } \
  {
    #puts "PowerUI::Create"

    variable LongName
  
    # Call the function 'create_tile' to generate the meter tile
    # from the resource database
    set meter_class  ${meter_size}_POWER
    
    set meter_class  [string toupper $meter_class]
    set top_level [winfo toplevel $tile_path]
    
    switch -- $meter_class \
    {
      "SMALL_POWER" \
      {
        set tile_path  [name_child  $top_level $LongName]
        if { ![winfo exists $tile_path] } \
        {
          set tile_path  [::Screens::create_tile  $tile_path  $LongName  $meter_class]
        }
      }
      "BIG_POWER" \
      {
        set tile_path   [name_child  $top_level big$LongName]
      
        if { ![winfo exists $tile_path] } \
        {
          set tile_path  [::Screens::create_toplevel_tile  $tile_path  $meter_class]
        }

        # Bind the 'big meter' to the 'F8' key
        bind $tile_path <KeyRelease-F8>  "destroy $tile_path"
      }
    }
    
    return  $tile_path
  
  }

  #******************************************************************** 
  #
  #       Name: Map_small
  #
  # Parameters: 
  #
  # Decsription: 
  #
  #********************************************************************
  proc Map_small { tile_path } \
  {
    #puts "PowerUI::Map_small:  $tile_path"

    variable StateUI
    
    switch -- $StateUI(units) \
    {
       W    { set_meter_range_W }
       
       dBm  { set_meter_range_dBm }
    }

    
    trace variable ::PowerUI::StateUI(units)  w  "::PowerUI::Update_units  $tile_path"

    trace variable ::PowerUI::StateUI(units)  w  "PowerUI::update_meter_units_small $tile_path"
                   
    update_meter_units_small_core  $tile_path
    
    trace variable ::Power::State(peakEnable)  w  "PowerUI::update_peak_hold_small $tile_path"

    update_peak_hold_small  $tile_path
    
    Update_units_core  $tile_path

    set_ref_mode  $StateUI(ref_mode)

    #::Power::DefaultSetup
    #::Power::QuickStart

    # BODGE force meter to auto-range
    Power prange AUTO -nowait
  }


  #******************************************************************** 
  #
  #       Name: Map_large
  #
  # Parameters: 
  #
  # Decsription: 
  #
  #********************************************************************
  proc Map_large { tile_path } \
  {
    #puts "PowerUI::Map_large:  $tile_path"

    variable StateUI
    

    switch -- $StateUI(units) \
    {
       W    { set_meter_range_W }
       
       dBm  { set_meter_range_dBm }
    }

    trace variable ::PowerUI::StateUI(units)  w  \
                               "::PowerUI::Update_units  $tile_path"

    trace variable ::PowerUI::StateUI(units)  w  \
                                 "PowerUI::update_meter_units_large $tile_path"
                    
    update_meter_units_large_core  $tile_path
    
    Update_units_core  $tile_path

    set_ref_mode  $StateUI(ref_mode)
    

    #::Power::DefaultSetup
    #::Power::QuickStart

    # BODGE force meter to auto-range
    Power prange AUTO -nowait
  }
  #********************************************************************
  #
  #       Name: Cleanup
  #
  # Parameters: 
  #
  # Decsription: 
  #
  #********************************************************************
  proc Cleanup { tile_path  size } \
  {
    set temp  [get_tile  $tile_path]

    trace vdelete ::PowerUI::StateUI(units)  w  \
                               "::PowerUI::Update_units  $tile_path"

    if { $size == "small" } \
    {
      trace vdelete ::PowerUI::StateUI(units)  w  \
                                 "PowerUI::update_meter_units_small $temp"

      trace vdelete ::Power::State(peakEnable)  w  \
                                 "PowerUI::update_peak_hold_small $tile_path"
    } \
    else \
    {
      trace vdelete ::PowerUI::StateUI(units)  w  \
                                 "PowerUI::update_meter_units_large $temp"
    }

    return
  }

  #********************************************************************
  #
  #       Name: Unmap
  #
  # Parameters: 
  #
  # Decsription: 
  #
  #********************************************************************
  proc Unmap { tile_path size } \
  {
    #puts "PowerUI::Unmap:  $tile_path,  $size"

    Cleanup $tile_path $size
  }


  #********************************************************************
  #
  #       Name: Destroy
  #
  # Parameters: 
  #
  # Decsription: 
  #
  #********************************************************************
  proc Destroy { tile_path size } \
  {
    #puts "PowerUI::Destroy:  $tile_path,  $size"

    Cleanup $tile_path $size
  }

  #********************************************************************
  #
  #       Name: Register
  #
  # Parameters: 
  #
  # Decsription: 
  #
  #********************************************************************
  proc Register { } \
  {
    #puts "PowerUI::Register"

      
    return
  }

  #********************************************************************
  #
  #       Name: Check_alarm                     
  #
  # Parameters: 
  #
  # Description: updates the alarm operation
  #
  #********************************************************************
  proc Check_alarm {  } \
  {
    variable StateUI

    set temp [expr $::Power::State(alertStatus) & 0x200]
    if { $temp != 0 } \
    {
       # High alert was exceeded
       set StateUI(reading,color) red
    } else \
    {
       set temp [expr $::Power::State(alertStatus) & 0x800]
       if { $temp != 0 } \
       {
          # Low alert was exceeded
          set StateUI(reading,color) blue
       } else \
       {
          set StateUI(reading,color) green
       }
    }
  }

  #********************************************************************
  # Name:        UpdateAlertStatus
  # Parameters:  None
  # Description: Updates an alarm bell status value
  #              0 = Alerts are
  #              0x02 = Hi alert ON/OFF
  #********************************************************************
  proc UpdateAlertStatus { args } \
  {
     after cancel PowerUI::UpdateAlertStatus_core
     after idle   PowerUI::UpdateAlertStatus_core
  }
  proc UpdateAlertStatus_core { } \
  {
    variable StateUI

    if { $::Power::State(BellEnabled) == 1 } \
    {
       set StateUI(alarm,state) 1
    } else \
    {
       set StateUI(alarm,state) 0
    }
  }

  #********************************************************************
  # Name:        UpdateAlertStrings
  # Parameters:  None
  # Description: Updates the alert text strings when alert enable changes
  #              0x01 = Low alert ON/OFF
  #              0x02 = Hi alert ON/OFF
  #********************************************************************
  proc UpdateAlertStrings { args } \
  {
    variable StateUI
 
    set status  $::Power::State(alerts)
    
    # High Limit
    if { $status & 0x2 } \
    {
       set StateUI(high_limit_set)  "ON"
       set StateUI(hi_limit,on) 1
    } \
    else \
    {
       set StateUI(high_limit_set)  "OFF"
       set StateUI(hi_limit,on) 0
    }

    # Low limit
    if { $status & 0x1 } \
    {
       set StateUI(low_limit_set)  "ON"
       set StateUI(lo_limit,on) 1
    } \
    else \
    {
       set StateUI(low_limit_set)  "OFF"
       set StateUI(lo_limit,on) 0
    }    
  }

  
  #********************************************************************
  #
  #       Name: toggle_meter_peakhold
  #
  # Parameters: new_value   - requested new value 
  #                           -1 - no new value requested
  #
  # Decsription: Displays PH and sets peak hold variable
  #
  #********************************************************************
  proc toggle_meter_peakhold {  } \
  {
     if {$::Power::State(peakEnable) == 0 } \
     {
        Power peakhold 1
     } \
     else \
     {
        Power peakhold 0
     }
  }
 
  
  #********************************************************************
  #
  #       Name: set_meter_range_W                    
  #
  # Parameters: 
  # Description: sets meter range
  #
  #********************************************************************
  proc set_meter_range_W { } \
  {
    variable StateUI
   
    if {$StateUI(range_selection,W) == 0 } \
    {
       # autorange selected
       set StateUI(range,auto) 1
       set StateUI(range,auto_txt) "AR"
    } \
    else \
    {
       set StateUI(range,auto) 0
       set StateUI(range,auto_txt) ""
       set StateUI(range,W) $StateUI(range_selection,W)
    }
    update_range
    return
  }
  #********************************************************************
  #
  #       Name: set_meter_range_dBm                    
  #
  # Parameters: 
  # Description: sets meter range
  #
  #********************************************************************
  proc set_meter_range_dBm { } \
  {
    variable StateUI
    
    if {$StateUI(range_selection,dBm) == 0 } \
    {
       # autorange selected
       set StateUI(range,auto) 1
       set StateUI(range,auto_txt) "AR"
    } \
    else \
    {
       set StateUI(range,auto) 0
       set StateUI(range,auto_txt) ""
       set StateUI(range,dBm) $StateUI(range_selection,dBm)
    }
    update_range
    return
  }  

  #********************************************************************
  #
  #       Name: set_meter_units
  #
  # Parameters: 
  # Decsription: 
  #
  #********************************************************************
  proc set_meter_units { } \
  {
    variable StateUI
    #puts "set_meter_units:  $meter_type"
   
    switch -- $StateUI(units) \
    {
        "W"     { set StateUI(units) "dBm"}
        default { set StateUI(units) "W" }
    }
  } 
  #********************************************************************
  #
  #       Name: Update_units
  #
  # Parameters: 
  #
  # Decsription: recomputes our setup based on units setup
  #
  #********************************************************************
  proc Update_units { tile_path  args } \
  {
     after cancel "PowerUI::Update_units_core  $tile_path"
     after idle   "PowerUI::Update_units_core  $tile_path"
  }
  
  proc Update_units_core { tile_path } \
  {
     Update_readings_core
  }
  proc Update_units_var { args } \
  {
     set cmd [namespace code "Update_units_var_core"]
     after cancel $cmd
     after idle $cmd
  }
  proc Update_units_var_core { } \
  {
     Update_limits_core
     Update_peakhold_core
     update_range
  }
  
  #********************************************************************
  #
  #       Name: update_meter_units
  #
  # Parameters: tile to update
  #
  # Decsription: updates display for new units
  #
  #********************************************************************
  proc update_meter_units_small { tile_path args } \
  {
     after cancel "PowerUI::update_meter_units_small_core $tile_path"
     after idle   "PowerUI::update_meter_units_small_core $tile_path"
  }
  proc update_meter_units_small_core { tile_path } \
  {
     variable StateUI
     # swap the menus to the approprate menus
     switch -- $StateUI(units) \
     {
        W   \
        {
           $tile_path.mnu_range config -menu $tile_path.mnu_range.menu_W
        }
        dBm \
        { 
          $tile_path.mnu_range config -menu $tile_path.mnu_range.menu_dBm
        }
     }
  }
  #********************************************************************
  #
  #       Name: update_meter_units
  #
  # Parameters: tile to update
  #
  # Decsription: updates display for new units
  #
  #********************************************************************
  proc update_meter_units_large { tile_path args } \
  {
     after cancel "PowerUI::update_meter_units_large_core $tile_path"
     after idle   "PowerUI::update_meter_units_large_core $tile_path"
  }
  proc update_meter_units_large_core { tile_path } \
  {
     variable StateUI
     # swap the menus to the approprate menus
     switch -- $StateUI(units) \
     {
        W   \
        {
            $tile_path.mnu_range config -menu $tile_path.mnu_range.menu_W
            grid forget $tile_path.ent_high_limit_dBm
            grid forget $tile_path.ent_low_limit_dBm
            ::Screens::position_widget $tile_path.ent_high_limit_W
            ::Screens::position_widget $tile_path.ent_low_limit_W
        }
        dBm \
        {
            $tile_path.mnu_range config -menu $tile_path.mnu_range.menu_dBm
            grid forget $tile_path.ent_high_limit_W
            grid forget $tile_path.ent_low_limit_W
            ::Screens::position_widget $tile_path.ent_high_limit_dBm
            ::Screens::position_widget $tile_path.ent_low_limit_dBm
        }
     }
  }
    
  #********************************************************************
  #
  #       Name: do_auto_range
  #
  # Parameters: 
  #
  # Description: Adjusts the displayed ranges
  #********************************************************************
  proc do_auto_range { } \
  {
     variable StateUI
     variable ranges
  
     if { $StateUI(range,auto) == 0} { return } 
     if {$StateUI(reading) == "" } { return }
         
     switch -- $StateUI(units) \
     {
        W \
        {
          upvar 0 StateUI(range,W) range
          upvar 0 StateUI(range_selection,W) range_selection
        }
        dBm \
        {
          upvar 0 StateUI(range,dBm) range
          upvar 0 StateUI(range_selection,dBm) range_selection
        }
     }
     
     set new_range $range
      
     if { $StateUI(reading) > $StateUI(reading,max) } \
     {
        incr new_range -1 ;# ranges are ordered backwards
     } \
     elseif { $StateUI(reading) < $StateUI(reading,next_range) } \
     {
        # have we seen enough readings to change?
        incr StateUI(reading,count)
        if {$StateUI(reading,count) > 5 } \
        {
           incr new_range
        }
     } \
     else \
     {
        # reading is in range
        set StateUI(reading,count) 0
        return
     }
     # Range might need to change - clip it & recompute
     if {$new_range < 1 } \
     {
        set new_range 1
     } \
     elseif {$new_range > $ranges($StateUI(units),max) } \
     {
        set new_range $ranges($StateUI(units),max)
     }
     if {$range == $new_range } { return }
     
     # OK, the range really changed
     set range $new_range
     update_range
 }
  #********************************************************************
  #
  #       Name: update_range
  #
  # Parameters: 
  #
  # Description: Updates the state variables for meter bars 
  #
  #********************************************************************
  proc update_range { } \
  {
   variable StateUI
   variable ranges
   variable ranges_dBm
   variable ranges_W
     
   # locate the range string for this range
   switch -- $StateUI(units) \
   {
      W \
      {
         upvar 0 StateUI(range_selection,W) range_selection
         upvar 0 StateUI(range,W) range
         upvar 0 ranges_W ranges_units
      }

      dBm \
      {
         upvar 0 StateUI(range_selection,dBm) range_selection
         upvar 0 StateUI(range,dBm) range
         upvar 0 ranges_dBm ranges_units
      }

      default { return }
   }
   # if the range is out of limits, force it in limits
   if {$range < 1} { set range 1 }

   set StateUI(reading,range) [lindex $ranges_units $range]
   
   if { $StateUI(range,auto) == 1 } \
   {
      set StateUI(auto_btn) "AR"
   } \
   else \
   {
      set StateUI(auto_btn) $StateUI(reading,range)
   }
   
   # set the new min and max display values for this range
   set StateUI(reading,max)        [lindex $ranges($StateUI(units),$range) 0]
   set StateUI(reading,middle)     [lindex $ranges($StateUI(units),$range) 1]
   set StateUI(reading,min)        [lindex $ranges($StateUI(units),$range) 2]
   set StateUI(reading,next_range) [lindex $ranges($StateUI(units),$range) 3]
  }
  
  #********************************************************************
  #
  #       Name: Update_readings
  #
  # Parameters: 
  #
  # Description: Updates the state variables for meter bars 
  #            : engine 'State(reading)' variable
  #
  #********************************************************************
  proc Update_readings { args } \
  {
     after cancel "PowerUI::Update_readings_core"
     after idle   "PowerUI::Update_readings_core"
  }
  
  proc Update_readings_core { } \
  {
     variable StateUI
     
     if ![info exists ::Power::State(reading)] { return }

     switch -- $StateUI(units) \
     {
        "W" \
        {
          set reading $::Power::State(reading)     
          set peak    $::Power::State(peakHi)
        }
      
        "dBm" \
        {
          set reading  $::Power::State(reading,dBm)  
          set peak     $::Power::State(peakHi,dBm)  
        }
        
        default { return }
     }
     if {$reading == "" } { return }
     
     if {$StateUI(reading) != $reading } { set StateUI(reading) $reading }               
     if {$StateUI(peak_reading) != $peak }  { set StateUI(peak_reading) $peak }

     if {$::Power::State(peakEnable) == 1} \
     {
	set StateUI(textreading)	$peak
     } \
     else \
     {
	set StateUI(textreading)	$reading
     }


     # re-range the UI if needed
     do_auto_range

     # service any alarms
     Check_alarm
  }

  #********************************************************************
  #
  #       Name: Update_limits
  #
  # Parameters: 
  #
  # Description: Conditions the 1/0 value from the engine to a 
  #            : "ON"/"OFF" and
  #            : "ON"/"OFF" 
  #            : string for the user interface
  #
  #********************************************************************
  proc Update_limits { args } \
  {
     after cancel PowerUI::Update_limits_core
     after idle   PowerUI::Update_limits_core
  }
  
  proc Update_limits_core { } \
  {
    variable StateUI
 
     switch -- $StateUI(units) \
     {
        "dBm" \
        {
          set StateUI(lo_limit,value) [format %.2f $::Power::State(alertLo,dBm)]
          set StateUI(hi_limit,value) [format %.2f $::Power::State(alertHi,dBm)]
        }

        "W" \
        {
          set StateUI(lo_limit,value) [format %.2f $::Power::State(alertLo,W)]
          set StateUI(hi_limit,value) [format %.2f $::Power::State(alertHi,W)]
        }
        
        default { return }
     }
   
    return
  }


  #********************************************************************
  #
  #       Name: Update_peakhold
  #
  # Parameters: 
  #
  # Description: Conditions the 1/0 value from the engine
  #            : to a "ON"/"OFF" string for the user interface
  #
  #********************************************************************
  proc Update_peakhold { args } \
  {
     after cancel PowerUI::Update_peakhold_core
     after idle   PowerUI::Update_peakhold_core
  }
  
  proc Update_peakhold_core { } \
  {
    variable StateUI
 
    set status  $::Power::State(peakEnable)
    
    if { $status == 1 } \
    {     
      set StateUI(peak_hold)  "ON"
    } \
    else \
    {
      set StateUI(peak_hold)  "OFF"
    }
    
    return
  }
  
  
  #********************************************************************
  #
  #        Name: update_peak_hold_small
  #
  #  Parameters:
  #
  # Description: Updates the Peak Hold button on the small meter 
  #              to changes in the 'peakEnable' state variable
  #
  #********************************************************************
  proc update_peak_hold_small { tile_path  args } \
  {
    set cmd  "::PowerUI::update_peak_hold_small_core  $tile_path"
    
    after cancel  $cmd
    
    after idle  $cmd
    
    return
  }
  
  proc update_peak_hold_small_core { tile_path } \
  {
    #puts "update_peak_hold_small_core:  $tile_path"

    #variable StateUI
    
    set tile_path  [get_tile  $tile_path]
    
    if { ![winfo exists $tile_path] } \
    { 
      return 
    }

    set btn_peak_hold  ${tile_path}.btn_peak_hold
 
    if { ![winfo exists $btn_peak_hold] } \
    { 
      return 
    }

    switch -- $::Power::State(peakEnable) \
    {
      1  { $btn_peak_hold  config  -relief sunken }
         
      0  { $btn_peak_hold  config  -relief raised }
    }
    
    return
  }


  #********************************************************************
  #
  #       Name: set_meter_average
  #
  # Parameters: new_value   - requested new value 
  #                           -1 - no new value requested
  #
  # Description: set the meter average
  #
  #********************************************************************
  proc set_meter_average { { new_value "" } } \
  {
    #puts "set_meter_average:  $new_value"
    
    if { "$new_value" == "" } \
    {
      set new_value  $::Power::State(average)
    } 
      
    # Set the meter engine
    after idle "Power  average  $new_value  -nowait"
      
    return
  }


  #********************************************************************
  #
  #       Name: set_ref_mode
  #
  # Parameters: 
  #
  # Description: sets the Reference Mode of the meter
  #
  #********************************************************************
  proc set_ref_mode { { new_value "" } } \
  {
    #puts "set_ref_mode:  $new_value"
    
    variable  StateUI
    
    if { $new_value == "" } \
    {
      set ref_mode  $StateUI(ref_mode)
      
      if { $ref_mode == "ON" } \
      {
        set ref_mode  "OFF"
      } \
      else \
      {
        set ref_mode  "ON"
      }
    } \
    else \
    {
      set ref_mode  $new_value
    }       
    
    set StateUI(ref_mode)  $ref_mode
    

    if { $ref_mode == "ON" } \
    {
      set StateUI(ref_indicate)  "REF"
      
      # Set the ref_reading value if the present value is 0.0,
      # which is the initialization value
      if { $StateUI(ref_reading) == 0.0 } \
      {
        set StateUI(ref_reading)   $StateUI(reading)
      }
    }
  
    if { $ref_mode == "OFF" } \
    {
      set StateUI(ref_indicate)  ""
      set StateUI(ref_reading)   0.0
    }
  
    return
  }


#****************************************************************************
# Name:        getSaveState
# Parameters:  none
# Description: This function returns a list of TCL commands which can be used
#              to set the PowerUI tiles to its current state
#****************************************************************************
 proc getSaveState { } {
   
   variable StateUI
 
   set returnList \
   {
      set ::PowerUI::StateUI(range,auto_txt) {$StateUI(range,auto_txt)}
      set ::PowerUI::StateUI(range,auto)     {$StateUI(range,auto)}
      set ::PowerUI::StateUI(units)          {$StateUI(units)}
      set ::PowerUI::StateUI(cable_loss)     {$StateUI(cable_loss)}

      set ::PowerUI::StateUI(range,W)             {$StateUI(range,W)}
      set ::PowerUI::StateUI(range,dBm)           {$StateUI(range,dBm)}
      set ::PowerUI::StateUI(range_selection,W)   {$StateUI(range_selection,W)}
      set ::PowerUI::StateUI(range_selection,dBm) {$StateUI(range_selection,dBm)}
   }
                     
   return [subst -nocommands $returnList]
 }


#****************************************************************************
# Name:        Sync
# Parameters:  none
# Description: Write state variables to PowerUI
#****************************************************************************
 proc Sync { } {
   eval [getSaveState]
 }

#****************************************************************************
# Name:        Zero
# Parameters:  none
# Description: Attempt to do a power term zero
#****************************************************************************
 proc Zero { widget } \
 {    
    if [catch {Power zero} err] \
    {
       set toplevel [winfo toplevel $widget]
       set tile_path [name_child $toplevel pwt_err]
       ::Screens::create_toplevel_tile  $tile_path  PWT_ZERO_ERR
    }
 }  
 

  trace variable ::Power::State(reading)    w  "::PowerUI::Update_readings"
  trace variable ::Power::State(peakEnable) w  "after idle ::PowerUI::Update_peakhold"
  trace variable ::Power::State(alerts)     w  "after idle ::PowerUI::UpdateAlertStrings"
  trace variable ::Power::State(alertLo)    w  "after idle ::PowerUI::Update_limits"
  trace variable ::Power::State(alertHi)    w  "after idle ::PowerUI::Update_limits"
  trace variable ::Power::State(BellEnabled) w "after idle ::PowerUI::UpdateAlertStatus"
  trace variable ::Power::State(alertStatus) w "after idle ::PowerUI::UpdateAlertStatus"
  trace variable ::PowerUI::StateUI(reading,max)  w  "::PowerUI::Update_limits"
  trace variable ::PowerUI::StateUI(units)  w  "::PowerUI::Update_units_var"
  Update_units_var
  Update_limits
  
  ::Load::on_system_done {
    namespace eval PowerUI {
      # sync our vars to the engine
      Update_peakhold
      Update_limits
      Power enable on -nowait
      UpdateAlertStatus
      UpdateAlertStrings
    }
  }
};  # End of PowerUI namespace

#****************************************************************************
# This registers the tiles with the list for saving instrument states
#****************************************************************************
namespace eval ::SavedStates {

   variable tiles   
   lappend tiles PowerUI
}

namespace eval View \
{
   lappend PreCreate PowerUI
}




