#############################################################################
#                                                                           #
# IFR SYSTEMS, INC        COPYRIGHT (C)1998,99      DESIGN ENGINEERING      #
# ------------------------------------------------------------------------- #
#                                                                           #
# FILE        : 3070.RferrUI.tcl                                            #
#                                                                           #
# PRODUCT     : FITS                                                        #
#                                                                           #
# AUTHOR      : P. McCormick                                                #
#                                                                           #
# DESCRIPTION : Rferr Meter User Interface                                  #
#                                                                           #
#############################################################################

package require RFError


namespace eval RferrUI \
{

  # Global variables
  variable  LongName     "rferr"
  variable  ShortName    "RF Error"
  variable  Size         Eighth_v
           
  variable  StateUI
  
  set  StateUI(reading)          0

  set  StateUI(range_selection)  "AR"
  set  StateUI(high_range)       ""
  set  StateUI(auto_range)       ""
  set  StateUI(mid_range)        "0"
  set  StateUI(low_range)        ""
           

  set  StateUI(ref_mode)         "OFF"
  set  StateUI(ref_indicate)     ""
  set  StateUI(ref_reading)      0.0


  set  StateUI(ranges)  { "AR"  "100K"  "50K"  "20K"  "10K"  "5000"  "1000"  "500"  "200"  "100" }
    
  set  StateUI(max_range) [llength $StateUI(ranges)]
  
  incr StateUI(max_range) -1


  #********************************************************************
  #
  #       Name: Create  
  #
  # Parameters: size - meter size 
  #
  # Decsription: 
  #
  #********************************************************************
  proc Create { tile_path  { meter_size  small } } \
  {
    #puts "RferrUI::Create:  $tile_path,  $meter_size"

    variable LongName
    
    set top_level [winfo toplevel $tile_path]
 
    set meter_class  ${meter_size}_RFERR
    
    set meter_class  [string toupper $meter_class]
    
    if { $meter_size == "small" } \
    {
      set tile_path  [name_child  $top_level $LongName]
      
      if { ![winfo exists $tile_path] } \
      {
        set tile_path  [::Screens::create_tile  $tile_path  $LongName  $meter_class]
      }  
    }
     
    if { $meter_size == "big" } \
    {
      set tile_path   [name_child  $top_level big$LongName]
  
      if { ![winfo exists $tile_path] } \
      {
        set tile_path  [::Screens::create_toplevel_tile  $tile_path  $meter_class]
      }

      # Bind the 'big meter' to the 'F8' key
      bind $tile_path <KeyRelease-F8>  "destroy  $tile_path"
    }
    
    return $tile_path
  }


  #******************************************************************** 
  #
  #       Name: Map
  #
  # Parameters: 
  #
  # Decsription: 
  #
  #********************************************************************
  proc Map { tile_path  meter_size } \
  {
    #puts "RferrUI::Map:  $tile_path,  $meter_size"

    variable StateUI
    
    set tile_path  [get_tile  $tile_path]


    # Set various meter conditions
      # Initialize the range
      set_meter_range  $StateUI(range_selection)
               
    
    set_ref_mode  $StateUI(ref_mode)
   
   
    trace variable ::RFError::State(reading_raw)  w  \
                                           "RferrUI::Update_display $tile_path"
     

    trace variable ::RFError::State(alerts)  w  \
                                          "::RferrUI::update_limits $tile_path"
                                          
    trace variable ::RFError::State(alertHi)  w  \
                                          "::RferrUI::update_limits $tile_path"
                                          
    trace variable ::RFError::State(alertLo)  w  \
                                          "::RferrUI::update_limits $tile_path"
                                          
    trace variable ::RferrUI::StateUI(high_range)  w  \
                                          "::RferrUI::update_limits $tile_path"
    

    update_limits_core $tile_path
    Update_display_core $tile_path

    if { $meter_size == "small" } \
    {
      trace variable ::RFError::State(BellEnabled)  w  \
                                      "RferrUI::update_alarm_small  $tile_path"
 
      update_alarm_small_core  $tile_path
       
      trace variable ::RFError::State(peakEnable)  w  \
                                   "RferrUI::update_peak_hold_small $tile_path"

      update_peak_hold_small_core  $tile_path

      ::RFError::DefaultSetup
      ::RFError::QuickStart
    } \
    else \
    {
      trace variable ::RFError::State(BellEnabled)  w  \
                                      "RferrUI::update_alarm_large  $tile_path"

      update_alarm_large_core  $tile_path
    }

    update_stale $tile_path

    return
  }


  #********************************************************************
  #
  #       Name: Cleanup
  #
  # Parameters: 
  #
  # Description: 
  #
  #********************************************************************
  proc Cleanup { tile_path  meter_size } \
  {
    #puts "RferrUI::Cleanup:  $tile_path,  $meter_size"
    
    trace vdelete ::RFError::State(reading_raw)   w  \
                                           "RferrUI::Update_display $tile_path"

    trace vdelete ::RFError::State(alerts)  w  \
                                          "::RferrUI::update_limits $tile_path"
                                          
    trace vdelete ::RFError::State(alertHi)  w  \
                                          "::RferrUI::update_limits $tile_path"
                                          
    trace vdelete ::RFError::State(alertLo)  w  \
                                          "::RferrUI::update_limits $tile_path"
                                          
    trace vdelete ::RferrUI::StateUI(high_range)  w  \
                                          "::RferrUI::update_limits $tile_path"

    if { $meter_size == "small" } \
    {
      trace vdelete ::RFError::State(BellEnabled)  w  \
                                      "RferrUI::update_alarm_small  $tile_path"

      trace vdelete ::RFError::State(peakEnable)  w  \
                                   "RferrUI::update_peak_hold_small $tile_path"
    } \
    else \
    {
      trace vdelete ::RFError::State(BellEnabled)  w  \
                                      "RferrUI::update_alarm_large  $tile_path"
    }
    
    after cancel "RferrUI::Map_AfterIdle $tile_path $meter_size"
    after cancel "RferrUI::Map_AfterDelay $tile_path $meter_size"

    return
  }


  #********************************************************************
  #
  #       Name: Unmap
  #
  # Parameters: 
  #
  # Decsription: 
  #
  #********************************************************************
  proc Unmap { tile_path  meter_size } \
  {
    #puts "RferrUI::Unmap:  $tile_path,  $meter_size"

    Cleanup  $tile_path  $meter_size
  }


  #********************************************************************
  #
  #       Name: Destroy
  #
  # Parameters: 
  #
  # Decsription: 
  #
  #********************************************************************
  proc Destroy { tile_path  meter_size } \
  {
    #puts "RferrUI::Destroy:  $tile_path,  $meter_size"

    Cleanup  $tile_path  $meter_size
  }


  #********************************************************************
  #
  #       Name: Register
  #
  # Parameters: 
  #
  # Decsription: 
  #
  #********************************************************************
  proc Register { } \
  {
    #puts "RferrUI::Register"

    return
  }


  #********************************************************************
  #
  #       Name: Update_display
  #
  # Parameters: 
  #
  # Description: Updates the display for a change in the 
  #            : engine 'State(reading)' variable
  #
  #********************************************************************
  proc Update_display { tile_path  args } \
  {
     set cmd "RferrUI::Update_display_core $tile_path"
     after cancel $cmd
     after idle $cmd
  }
  
  proc Update_display_core { tile_path } \
  {
    #puts "RferrUI::Update_display:  $tile_path,  $args"

    variable StateUI
        
    if { ![winfo exists $tile_path] } \
    { 
      return 
    }
    
    set tile_path  [get_tile  $tile_path]

    if { ![winfo ismapped $tile_path] } \
    { 
      #puts "Rferr is not mapped"
      
      return 
    }

    # Update the peak hold depending on whether the peak hold is set
      set peak_enable  $::RFError::State(peakEnable)

      if { $peak_enable == 1 } \
      {
        # If the reading is negative, get the low peak value
        if { $::RFError::State(reading_raw) < 0 } \
        {
          # Update the reading to the low peak value
          set reading_raw   $::RFError::State(reading_raw)
          set reading_cond  $::RFError::State(peakLo)
          #set reading_peak  $::RFError::State(peakLo)
        } \
        else \
        {
          # Update the reading to the high peak value
          set reading_raw   $::RFError::State(reading_raw)
          set reading_cond  $::RFError::State(peakHi)
          #set reading_peak  $::RFError::State(peakHi)
        }
      }
    
      if { $peak_enable == 0 } \
      {
        # Update the reading to the conditioned reading value
        set reading_raw   $::RFError::State(reading_raw)
        set reading_cond  $::RFError::State(reading_cond_Hz)
        #set reading_peak  $::RFError::State(reading_raw)
      }


      if { $StateUI(ref_mode) == "OFF" } \
      {
        set StateUI(reading)  $reading_cond
      }

      if { $StateUI(ref_mode) == "ON" } \
      {
        set reading_comp  [format "%.1f" $reading_raw]

        set reading_ref   [format "%.1f" $StateUI(ref_reading)]

        set StateUI(reading)  [expr $reading_comp - $reading_ref]
      }


    # ######################################################################
    # # Changed for the meterbar
    #   return
    # # Changed for the meterbar
    # ######################################################################
      

    # Update the bargraph, range, limits, and alarm to the raw reading
    # Update the bargraph
    update_bargraph  $tile_path  $reading_raw

    # Update the range
    update_range  $reading_raw

    # Update the alarm status
    update_alarm  $tile_path  $reading_raw
    
    # Update the peak hold
    update_peakhold  $tile_path

    update_stale  $tile_path
    
    return
  }
  

  #********************************************************************
  #
  #       Name: get_tile                   
  #
  # Parameters: a widget
  #
  # Description: given widget within a tile, get the name of the tile
  #
  #********************************************************************
  proc get_tile { widget } \
  {
    while {1} \
    {
      if { [catch { set class  [winfo class $widget] } result] } \
      {
        return ""
      }
         
      switch -- $class \
      {
        "SMALL_RFERR"  -
        "BIG_RFERR"    { return $widget }
      }

      set widget [winfo parent $widget]
    }
    
    return ""
  }


  #********************************************************************
  #
  #       Name: update_mid_range                     
  #
  # Parameters: 
  #
  # Description: sets the meter midrange value
  #
  #********************************************************************
  proc update_mid_range { args } \
  {
     set cmd [namespace code "update_mid_range_core"]
     after cancel $cmd
     after idle $cmd
  }
  proc update_mid_range_core { } \
  {

    #puts "update_mid_range"

    variable StateUI
    
    set Range  $StateUI(high_range)

    set temp ""
    set modifier ""
    
    if [regexp {([-.0-9]+)}  $Range MidRange] \
    {
      regexp {([a-zA-Z]$)} $Range modifier
      set StateUI(low_range)  "-${MidRange}${modifier}"
    }
  }
  
  
  #********************************************************************
  #
  #       Name: get_mult                     
  #
  # Parameters: 
  #
  # Description: Gets the value of the muliplier
  #
  #********************************************************************
  proc get_mult { modifier } \
  {
    switch -- $modifier \
    {
      "K" -
      "k" { return 1000 }
    
      "M" -
      "m" { return 1000000 }
  
      "G" -
      "g" { return 1000000000 }
    }
    
    return 1
  }

  #********************************************************************
  #
  #       Name: set_hi_limit                     
  #
  # Parameters: 
  #
  # Description: set high limit
  #
  #********************************************************************
  proc set_hi_limit { } \
  {
     set cmd "::RFError::setLimits high"
     after cancel $cmd
     after idle $cmd
  }
  
  #********************************************************************
  #
  #       Name: set_lo_limit                     
  #
  # Parameters: 
  #
  # Description: set high limit
  #
  #********************************************************************
  proc set_lo_limit { } \
  {
     set cmd "::RFError::setLimits low"
     after cancel $cmd
     after idle $cmd
  }
  
  #********************************************************************
  #
  #       Name: get_high_range                     
  #
  # Parameters: 
  #
  # Description: gets meter range
  #
  #********************************************************************
  proc get_high_range { } \
  {
    #puts "get_high_range"

    variable StateUI
    
    set range  $StateUI(high_range)
    
    #puts "range:  $range"
    
    set modifier    ""
    set multiplier  1
    
    regexp {([-.0-9]+)}  $range  range_value
    regexp {([a-zA-Z]$)} $range  modifier
    
    set multiplier  [get_mult  $modifier]

    set high_range  [expr $range_value * $multiplier]
    
    return $high_range
  }
  
  
  #********************************************************************
  #
  #       Name: get_highest_range
  #
  # Parameters: 
  #
  # Description: gets the meters highest range
  #
  #********************************************************************
  proc get_highest_range { } \
  {
    #puts "get_highest_range"

    variable  StateUI
    
    # Get the range value of index '1' of the menu
    set high_range  [lindex  $StateUI(ranges)  1]

    return $high_range
  }

  
  #********************************************************************
  #
  #       Name: update_stale
  #
  # Parameters: tile to update
  #
  # Description: update the meter for stale/not stale
  #
  #********************************************************************
  proc update_stale { tile_path } \
  {
    variable StateUI
    set tile_path  [get_tile  $tile_path]
    
    if { ![winfo exists $tile_path] } \
    { 
      return 
    }

    set canvas_name  ${tile_path}.cnvs_bargraph
    set reading      ${tile_path}.lbl_reading
    
    if {$::RFError::State(status) & 2} \
    {
       $canvas_name itemconfigure bargraph       -stipple gray50
       $canvas_name itemconfigure bargraph_frame -fill DimGrey
    } \
    else \
    {
       $canvas_name itemconfigure bargraph       -stipple {}
       $canvas_name itemconfigure bargraph_frame -fill {}
    }
  }
  
  #********************************************************************
  #
  #       Name: update_bargraph                     
  #
  # Parameters: reading - the new reading
  #
  # Description: updates the bargraph widget of the meter
  #
  #********************************************************************
  proc update_bargraph { tile_path  reading } \
  {
    #puts "update_bargraph:  $tile_path,  $reading"
    
    if { $reading == "" } \
    {
      return
    }

    set tile_path  [get_tile  $tile_path]
    
    if { ![winfo exists $tile_path] } \
    { 
      return 
    }

    set canvas_name  ${tile_path}.cnvs_bargraph
    
    if { ![winfo exists $canvas_name] } \
    { 
      return 
    }

    # Adjust for negative readings
    set reading_abs [expr abs($reading)]
    
    # Determine the value of the high range
    set range [get_high_range]
    
    # Convert the range to the absolute value
    set range [expr abs($range)]

    if { $reading_abs > $range } \
    {
      if { $reading > 0 } \
      {
        set reading  $range
      }

      if { $reading < 0 } \
      {
        set reading  [expr $range * -1.0]
      }
    }

    # Determine the present dimensions of the bargraph frame
    set bargraph_coords [$canvas_name coords bargraph_frame]

    foreach { x1 y1 x2 y2 } $bargraph_coords {}

    set meterbar_height  [expr [expr [expr $y2 - $y1] - 1] / 2.0]

    set meterbar_zero  [expr $meterbar_height + $y1]
    
    # The value 'meterbar_height' is the max height of the bargraph (in pixels)
    set ratio [expr $meterbar_height / $range]
    
    set increase [expr $reading * $ratio]

    # Scale the bargraph by the correct amount
    # Determine the present dimensions
    set bargraph_coords [$canvas_name coords bargraph]

    foreach { x1 y1 x2 y2 } $bargraph_coords {}

    set new_height  [expr $meterbar_zero - $increase]
    
    # Display the bargraph for the present reading
    $canvas_name coords bargraph $x1 $meterbar_zero $x2 $new_height
 
    return
  }
  
  
  #********************************************************************
  #
  #       Name: update_range                     
  #
  # Parameters: reading - the new reading
  #
  # Description: updates the reading widget of the meter
  #
  #********************************************************************
  proc update_range { reading } \
  {
    #puts "update_range:  $reading"

    variable  StateUI

    if { $reading == "" } \
    {
      return
    }

    # Adjust for negative readings
    set reading  [expr abs($reading)]

    # Determine the value of the high range
    set range  [get_high_range]
    
    # Convert the range to the absolute value
    set range  [expr abs($range)]
    
    set quarter_range  [expr $range / 4.0]
    
    set auto_range  $StateUI(auto_range)
  
    set pres_range  $StateUI(high_range)
  
    # Get the present active index
    set pres_index  0

    foreach label_range $StateUI(ranges) \
    {
      if { $label_range == $pres_range } \
      {
        break
      }
      
      incr pres_index
    }
    
    if { $auto_range == "AR" } \
    {
      if { $reading > $range } \
      {
        incr pres_index -1
      } \
      elseif { $reading < $quarter_range } \
      {
        incr pres_index
      } \
      else \
      {
        # No range change. all done
         return
      }
      
      if { $pres_index < 1 } \
      {
        set new_index 1
      } \
      elseif { $pres_index > $StateUI(max_range) } \
      {
        set new_index  $StateUI(max_range)
      } \
      else \
      {
        set new_index  $pres_index
      }
      
      set new_value  [lindex $StateUI(ranges) $new_index]

      # Don't update the range if it is the same        
      if { $StateUI(high_range) != $new_value } \
      {
        set StateUI(high_range)  $new_value
      }

      set StateUI(auto_range)  "AR"
    }

    return
  }
  
  
  #********************************************************************
  #
  #       Name: update_peakhold
  #
  # Parameters:
  #
  # Description: updates the bargraph peak hold indicator
  #
  #********************************************************************
  proc update_peakhold { tile_path  } \
  {
    #puts "update_peakhold:  $tile_path" 

    variable StateUI

    set tile_path  [get_tile  $tile_path]
    
    if { ![winfo exists $tile_path] } \
    { 
      return 
    }

    set canvas_name  ${tile_path}.cnvs_bargraph
    
    if { ![winfo exists $canvas_name] } \
    { 
      return 
    }

    set marker_x1 [option get $tile_path marker_x1 {}]
    set marker_x2 [option get $tile_path marker_x2 {}]

    # Get both the upper and lower peak values
    set peak_high  $::RFError::State(peakHi)
    set peak_low   $::RFError::State(peakLo)
    

    # Delete the old peak hold marker
    $canvas_name delete peak_marker_high
    $canvas_name delete peak_marker_low

    # Determine peak hold condition
    if { $::RFError::State(peakEnable) == 1 } \
    {
      # Determine the value of the high range
      set range [get_high_range]
  
      set range [expr abs($range)]
      
      # Determine the present dimensions of the bargraph frame
      set bargraph_coords [$canvas_name coords bargraph_frame]
      
      foreach { x1 y1 x2 y2 } $bargraph_coords {}
    
      set meterbar_height  [expr [expr [expr $y2 - $y1] - 1] / 2.0]

      set meterbar_zero  [expr $meterbar_height + $y1]

      # Determine the ratio of the reading to the scale
      set ratio [expr $meterbar_height / $range]
        
      # Determine the present dimensions
      set bargraph_coords  [$canvas_name coords bargraph]
      
      foreach { x1 y1 x2 y2 } $bargraph_coords {}
      
      set new_peak_high_val [expr $peak_high * $ratio]
      set new_peak_low_val  [expr $peak_low  * $ratio]
  
      # If the reading exceeds the scale of the bargraph,
      # limit the peak hold marker to the upper edge of the bargraph
      if { $peak_high > $range } \
      {
        set new_peak_high_val  $meterbar_height
      }
      
      if { $peak_low < [expr $range * -1.0] } \
      {
        set new_peak_low_val  [expr $meterbar_height * -1.0]
      }

      set new_peak1_high  [expr $meterbar_zero - $new_peak_high_val]
    
      set new_peak2_high  [expr $new_peak1_high + 2]
     
      $canvas_name create rect $marker_x1 $new_peak1_high   \
                               $marker_x2 $new_peak2_high   \
                               -fill white                  \
                               -tags peak_marker_high


      set new_peak1_low  [expr $meterbar_zero - $new_peak_low_val]
    
      set new_peak2_low  [expr $new_peak1_low + 2]
     
      $canvas_name create rect $marker_x1 $new_peak1_low    \
                               $marker_x2 $new_peak2_low    \
                               -fill white                  \
                               -tags peak_marker_low
    } 

    return
  }
  
  
  #********************************************************************
  #
  #       Name: update_alarm                     
  #
  # Parameters: new_reading - the new reading
  #
  # Description: updates the alarm operation
  #
  #********************************************************************
  proc update_alarm { tile_path  new_reading } \
  {
    #puts "update_alarm:  $tile_path,  $new_reading"
 
    variable StateUI

    if { $new_reading == "" } \
    {
      return
    }

    set tile_path  [get_tile  $tile_path]
    
    if { ![winfo exists $tile_path] } \
    { 
      return 
    }

    set canvas_name  ${tile_path}.cnvs_bargraph
    
    if { ![winfo exists $canvas_name] } \
    { 
      return 
    }
    
    # Determine if Alarm is set
    set text_color black
    set bar_color green
    set alarm off
      
    set alertHi_text  $::RFError::State(alertHi_text)

    set high_limit    $::RFError::State(alertHi)
      
    set alertLo_text  $::RFError::State(alertLo_text)

    set low_limit     $::RFError::State(alertLo)
      
     
    if {$::RFError::State(status) & 0x200 } \
    { 
      set bar_color red

      # Turn on the audio alarm
      set alarm hi
    } \
    elseif {$::RFError::State(status) & 0x800  } \
    { 
      set bar_color blue

      # Turn on the audio alarm
      set alarm lo
    } \
    elseif {$::RFError::State(status) & 0x101D  } \
    {
        set bar_color yellow
        set text_color yellow
    }

    $canvas_name itemconfigure bargraph -fill $bar_color
    ${tile_path}.lbl_reading config -fg $text_color
  
    if { $::RFError::State(BellEnabled) == 0 } \
    {
      set alarm off
    }

    switch -- $alarm \
    {
       hi {}
       lo {}
       off -
       default {}
    }

    return
  }


  #********************************************************************
  #
  #       Name: update_limits
  #
  # Parameters:
  #
  # Description: updates the limits of the meter
  #
  #********************************************************************
  proc update_limits { tile_path  args } \
  {
     set cmd  "RferrUI::update_limits_core $tile_path"

     after cancel $cmd
     
     after 100 $cmd
  }
  proc update_limits_core { tile_path } \
  {
     update_limits_hi  $tile_path
     update_limits_lo  $tile_path
  }

  #********************************************************************
  #
  #       Name: update_limits_hi
  #
  # Parameters:
  #
  # Description: updates the limits of the meter
  #
  #********************************************************************
  proc update_limits_hi { tile_path } \
  {
    #puts "update_limits_hi:  $tile_path"
    
    set tile_path  [get_tile  $tile_path]
    
    if { ![winfo exists $tile_path] } \
    { 
      return 
    }

    set meter_class   [winfo class $tile_path]

    set canvas_name  ${tile_path}.cnvs_bargraph
    
    if { ![winfo exists $canvas_name] } \
    { 
      return 
    }

    set line_width  [option get $tile_path line_width {}]


    # Determine the present dimensions of the bargraph frame
    set bargraph_coords [$canvas_name coords bargraph_frame]
    
    foreach { x1 y1 x2 y2 } $bargraph_coords {}
    
    set meterbar_height  [expr [expr [expr $y2 - $y1] - 1] / 2.0]
    
    set meterbar_zero  [expr $meterbar_height + $y1]

    # Delete the upper limit marker from the screen
    $canvas_name delete high_marker

    $canvas_name delete high_value

    if { $::RFError::State(alertHi_text) == "ON" } \
    {
      # Get the value of the limit
      set limit_value  $::RFError::State(alertHi)

      if { $limit_value == "" } \
      {
        set limit_value  0.00
      }

      set limit_value  [format "%.1f" $limit_value]

      # Determine the value of the high range
      set range [get_high_range]
    
      # If the value exceeds the range, 
      # return without displaying the marker
      if { $limit_value > $range || $limit_value < [expr $range * -1.0] } \
      {
        return
      }
    
      # The value 'meterbar_height' is the max height of the bargraph (in pixels)
      set ratio [expr $meterbar_height / $range]

      # Use the value of 'limit_value' to set the marker
      set marker_pos [expr $limit_value * $ratio]
  
      # Scale the bargraph by the correct amount
      # Determine the present dimensions
      set bargraph_coords [$canvas_name coords bargraph]
    
      set marker_height  [expr $meterbar_zero - $marker_pos]
    
      # Display the bargraph for the present reading
      $canvas_name create line 0  $marker_height              \
                                  $line_width $marker_height  \
                                  -width 2                    \
                                  -fill red                   \
                                  -tags high_marker

      if { $meter_class == "BIG_RFERR"} \
      {
        # Display the value of the limit next to the marker on the big meter
        $canvas_name create text 0  [expr $marker_height - 8]   \
                                         -text $limit_value     \
                                         -font {-size 12}       \
                                         -anchor w              \
                                         -fill red              \
                                         -tags high_value
      }
    
    }

    return
  }
  

  #********************************************************************
  #
  #       Name: update_limits_lo
  #
  # Parameters:
  #
  # Description: updates the limits of the meter
  #
  #********************************************************************
  proc update_limits_lo { tile_path } \
  {
    #puts "update_limits_lo:  $tile_path"
    
    set tile_path  [get_tile  $tile_path]
    
    if { ![winfo exists $tile_path] } \
    { 
      return 
    }

    set meter_class   [winfo class $tile_path]

    set canvas_name  ${tile_path}.cnvs_bargraph
    
    if { ![winfo exists $canvas_name] } \
    { 
      return 
    }

    set line_width  [option get $tile_path line_width {}]


    # Determine the present dimensions of the bargraph frame
    set bargraph_coords [$canvas_name coords bargraph_frame]
    
    foreach { x1 y1 x2 y2 } $bargraph_coords {}
    
    set meterbar_height  [expr [expr [expr $y2 - $y1] - 1] / 2.0]
    
    set meterbar_zero  [expr $meterbar_height + $y1]
    
    # Delete the lower limit marker from the screen
    $canvas_name delete low_marker
    
    $canvas_name delete low_value

    if { $::RFError::State(alertLo_text) == "ON" } \
    {
      # Get the value of the limit
      set limit_value  $::RFError::State(alertLo)
         
      if { $limit_value == "" } \
      {
        set limit_value  0.00
      }

      set limit_value  [format "%.1f" $limit_value]
         
      # Determine the value of the high range
      set range [get_high_range]
    
      # If the value exceeds the range, 
      # return without displaying the marker
      if { $limit_value > $range || $limit_value < [expr $range * -1.0] } \
      {
        return
      }
  
      # The value 'meterbar_height' is the max height of the bargraph (in pixels)
      set ratio [expr $meterbar_height / $range]

      # Use the value of 'limit_value' to set the marker
      set marker_pos [expr $limit_value * $ratio]
  
      # Scale the bargraph by the correct amount
      # Determine the present dimensions
      set bargraph_coords [$canvas_name coords bargraph]
      
      foreach { x1 y1 x2 y2 } $bargraph_coords {}

      set marker_height  [expr $meterbar_zero - $marker_pos]
    
      # Display the bargraph for the present reading
      $canvas_name create line 0  $marker_height              \
                                  $line_width $marker_height  \
                                  -width 2                    \
                                  -fill blue                  \
                                  -tags low_marker

      if { $meter_class == "BIG_RFERR"} \
      {
        # Display the value of the limit next to the marker on the big meter
        $canvas_name create text 0  [expr $marker_height + 7]   \
                                         -text $limit_value     \
                                         -font {-size 12}       \
                                         -anchor w              \
                                         -fill blue             \
                                         -tags low_value
      }
    }

    return
  }


  #********************************************************************
  #
  #       Name: update_alarm_small
  #
  # Parameters: 
  #
  # Decsription:
  #
  #********************************************************************
  proc update_alarm_small { tile_path  args } \
  {
    #puts "update_alarm_small:  $tile_path,  $args"
    
    set cmd  "RferrUI::update_alarm_small_core  $tile_path  $args"
    
    after cancel  $cmd
    
    after idle  $cmd
    
    return
  }

  proc update_alarm_small_core { tile_path  args } \
  {
    #puts "update_alarm_small_core:  $tile_path,  $args"

    variable StateUI
    
    set tile_path  [get_tile  $tile_path]
    
    if { ![winfo exists $tile_path] } \
    { 
      return 
    }

    set btn_alarm  ${tile_path}.btn_alarm
    
    if { ![winfo exists $btn_alarm] } \
    { 
      return 
    }
    
    switch -- $::RFError::State(BellEnabled) \
    {
      1  { $btn_alarm configure -image image_smallbellon  }
      0  { $btn_alarm configure -image image_smallbelloff }
    }

    return    
  }
  

  #********************************************************************
  #
  #       Name: update_alarm_large
  #
  # Parameters: 
  #
  # Decsription:
  #
  #********************************************************************
  proc update_alarm_large { tile_path  args } \
  {
    #puts "update_alarm_large:  $tile_path,  $args"
    
    set cmd  "RferrUI::update_alarm_large_core  $tile_path  $args"
    
    after cancel  $cmd
    
    after idle  $cmd
    
    return
  }
  
  proc update_alarm_large_core { tile_path  args } \
  {
    #puts "update_alarm_large_core:  $tile_path,  $args"

    variable StateUI

    set tile_path  [get_tile  $tile_path]
    
    if { ![winfo exists $tile_path] } \
    { 
      return 
    }

    set btn_alarm  ${tile_path}.btn_alarm_bell
    
    if { ![winfo exists $btn_alarm] } \
    { 
      return 
    }

    switch -- $::RFError::State(BellEnabled) \
    {
      1  { $btn_alarm configure -image image_bigbellon  }
      0  { $btn_alarm configure -image image_bigbelloff }
    }
   
    return
  }
 

  #********************************************************************
  #
  #        Name: update_peak_hold_small
  #
  #  Parameters:
  #
  # Description: Updates the Peak Hold button on the small meter 
  #              to changes in the 'peakEnable' state variable
  #
  #********************************************************************
  proc update_peak_hold_small { tile_path  args } \
  {
    set cmd  "::RferrUI::update_peak_hold_small_core  $tile_path"
    
    after cancel  $cmd
    
    after idle  $cmd
    
    return
  }
  
  proc update_peak_hold_small_core { tile_path } \
  {
    #puts "update_peak_hold_small_core:  $tile_path"

    #variable StateUI
    
    set tile_path  [get_tile  $tile_path]
    
    if { ![winfo exists $tile_path] } \
    { 
      return 
    }

    set btn_peak_hold  ${tile_path}.btn_peak_hold
 
    if { ![winfo exists $btn_peak_hold] } \
    { 
      return 
    }

    switch -- $::RFError::State(peakEnable) \
    {
      1  { $btn_peak_hold  config  -relief sunken }
         
      0  { $btn_peak_hold  config  -relief raised }
    }
    
    return
  }


  #********************************************************************
  #
  #       Name: set_meter_range                     
  #
  # Parameters: new_value   - requested new value 
  #                           -1 - no new value requested
  #
  # Description: sets meter range, displays AR and sets scale variable
  #
  #********************************************************************
  proc set_meter_range { { new_value "" } } \
  {
    #puts "set_meter_range:  $new_value"

    variable StateUI


    if { $new_value != "" } \
    {
      set StateUI(range_selection)  $new_value
    }
    

    # If the variable has no value, set the value to 'AR'
    if { $StateUI(range_selection) == "" } \
    {
      set StateUI(range_selection)  "AR"
    }


    if { $StateUI(range_selection) == "AR" } \
    { 
      # Set the namespace variable 
      set StateUI(auto_range)  "AR"
    
      # Set the meter to the highest range initially
      set range_value  [get_highest_range]

      set StateUI(high_range)  $range_value
    } \
    else \
    { 
      # Set the namespace variable 
      set StateUI(auto_range)  ""
  
      set range_value  $StateUI(range_selection)
       
      # Set the button test to indicate the range selected
      set StateUI(high_range)  $range_value
    } 
    
  
    return
  }
  
  
  #********************************************************************
  #
  #       Name: set_ref_mode
  #
  # Parameters: 
  #
  # Description: sets the Reference Mode of the meter
  #
  #********************************************************************
  proc set_ref_mode { { new_value "" } } \
  {
    #puts "set_ref_mode:  $new_value"
    
    variable  StateUI
    
    if { $new_value == "" } \
    {
      if { $StateUI(ref_mode) == "ON" } \
      {
        set StateUI(ref_mode)  "OFF"
      } \
      else \
      {
        set StateUI(ref_mode)  "ON"
      }
    } \
    else \
    {
      set StateUI(ref_mode)  $new_value
    }       
    

    if { $StateUI(ref_mode) == "ON" } \
    {
      set StateUI(ref_indicate)  "REF"
      
      # Set the ref_reading value if the present value is 0.0,
      # which is the initialization value
      if { $StateUI(ref_reading) == 0.0 } \
      {
        set StateUI(ref_reading)   $StateUI(reading)
      }
    }
  
    if { $StateUI(ref_mode) == "OFF" } \
    {
      set StateUI(ref_indicate)  ""
      set StateUI(ref_reading)   0.0
    }
  
    return
  }


  #****************************************************************************
  # Name:        getSaveState
  # Parameters:  none
  # Description: This function returns a list of TCL commands which can be used
  #              to set the RferrUI tiles to its current state
  #****************************************************************************
  proc getSaveState { } \
  {
    variable StateUI
 
    set returnList \
    {
      set ::RferrUI::StateUI(range_selection) {$StateUI(range_selection)}
      set ::RferrUI::StateUI(ref_mode)        {$StateUI(ref_mode)}
    }
       
    return [subst -nocommands $returnList]
  }


  #****************************************************************************
  # Name:        Sync
  # Parameters:  none
  # Description: Write state variables to RferrUI
  #****************************************************************************
  proc Sync { } \
  {
    eval [getSaveState]
  }
  trace variable ::RferrUI::StateUI(high_range)  w  "::RferrUI::update_mid_range"
  update_mid_range

};  # End of RferrUI namespace



#****************************************************************************
# This registers the tiles with the list for saving instrument states
#****************************************************************************
namespace eval ::SavedStates {

   variable tiles   
   lappend tiles RferrUI
}

namespace eval View \
{
   lappend PreCreate RferrUI
}
