#############################################################################
#                                                                           #
# IFR SYSTEMS, INC        COPYRIGHT (C)1998,99      DESIGN ENGINEERING      #
# ------------------------------------------------------------------------- #
#                                                                           #
# FILE        : 3080.SinadUI.tcl                                            #
#                                                                           #
# PRODUCT     : FITS                                                        #
#                                                                           #
# AUTHOR      : P. McCormick                                                #
#                                                                           #
# DESCRIPTION : Sinad Meter User Interface                                  #
#                                                                           #
#############################################################################

package require Sinad


namespace eval SinadUI \
{

  # Global variables
  variable  LongName     "sinad"
  variable  ShortName    "SINAD"
  variable  Size         Eighth_v
           
  variable  meter_type        sinad
  variable  meter_size        ""
  variable  meter_class       ""
  variable  meter_engine      Sinad
  
  variable  StateUI
  
  array set StateUI \
  {
    reading          0

    range_selection  "AR"
    high_range       60
    auto_range       ""
    mid_range        ""
    low_range        "0"
           
    auto_search      "OFF"
    auto_search_lvl  0.0

    ref_mode         "OFF"
    ref_indicate     ""
    ref_reading      0.0
                                           
    ranges { "AR" "60" "20" }
    
  }
  
  set  StateUI(max_range) [llength $StateUI(ranges)]
  
  incr StateUI(max_range) -1

  
  #********************************************************************
  #
  #        Name: Create  
  #
  #  Parameters: size - meter size 
  #
  # Description: 
  #
  #********************************************************************
  proc Create { tile_path  { meter_size  small } } \
  {
    #puts "SinadUI::Create:  $tile_path,  $meter_size"
    
    variable  LongName
    
    set top_level [winfo toplevel $tile_path]
   
    set tile_path  [name_child  $top_level  $LongName]

    #puts "tile_path:  $tile_path"


    # Call the function 'create_tile' to generate the meter tile
    # from the resource database
    set meter_class  ${meter_size}_SINAD
    
    set meter_class  [string toupper $meter_class]
    
    if { $meter_size == "small" } \
    {
      if { ![winfo exists $tile_path] } \
      {
        set tile_path  [::Screens::create_tile  $tile_path  $LongName  $meter_class]
      }
    }
  
      
    if { $meter_size == "big" } \
    {
      set tile_path   [name_child  $top_level big$LongName]
   
      if { ![winfo exists $tile_path] } \
      {
        set tile_path  [::Screens::create_toplevel_tile  $tile_path  $meter_class]
      }

      # Bind the 'big meter' to the 'F8' key
      bind $tile_path <KeyRelease-F8>  "destroy  $tile_path"
    }
  
    return  $tile_path
  }


  #******************************************************************** 
  #
  #        Name: Map
  #
  #  Parameters: 
  #
  # Description: 
  #
  #********************************************************************
  proc Map { tile_path  meter_size } \
  {
    #puts "SinadUI::Map:  $tile_path,  $meter_size"

    variable  StateUI
    
    
    # Set various meter conditions

    # Initialize the range
    set_meter_range  $StateUI(range_selection)

    set_ref_mode     $StateUI(ref_mode)

    trace variable ::Sinad::State(sinad)  w  \
                                         "::SinadUI::Update_display $tile_path"

    trace variable ::Sinad::State(Sinad.alerts)  w  \
                                          "::SinadUI::update_limits $tile_path"
                                          
    trace variable ::Sinad::State(Sinad.alertHi)  w  \
                                          "::SinadUI::update_limits $tile_path"
                                          
    trace variable ::Sinad::State(Sinad.alertLo)  w  \
                                          "::SinadUI::update_limits $tile_path"
                                          
    trace variable ::SinadUI::StateUI(high_range)  w  \
                                          "::SinadUI::update_limits $tile_path"
    
    update_limits_core $tile_path
    
    update_stale $tile_path
    Update_display_core $tile_path
    Sinad::Track_input


    if { $meter_size == "small" } \
    {
      trace variable ::Sinad::State(Sinad.BellEnabled)  w  \
                                     "::SinadUI::update_alarm_small $tile_path"

      update_alarm_small_core  $tile_path

      trace variable ::Sinad::State(Sinad.peakEnable)  w  \
                                 "::SinadUI::update_peak_hold_small $tile_path"

      update_peak_hold_small_core  $tile_path
    } \
    else \
    {
      trace variable ::Sinad::State(Sinad.BellEnabled)  w  \
                                     "::SinadUI::update_alarm_large $tile_path"
                    
      update_alarm_large_core  $tile_path

      set_auto_search  $StateUI(auto_search)
    }
  }


  #********************************************************************
  #
  #        Name: Cleanup
  #
  #  Parameters: 
  #
  # Description: 
  #
  #********************************************************************
  proc Cleanup { tile_path  meter_size } \
  {
    #puts "SinadUI::Cleanup:  $tile_path,  $meter_size"
    
    trace vdelete ::Sinad::State(sinad)  w  \
                                         "::SinadUI::Update_display $tile_path"

    trace vdelete ::Sinad::State(Sinad.alerts)  w  \
                                          "::SinadUI::update_limits $tile_path"
                                          
    trace vdelete ::Sinad::State(Sinad.alertHi)  w  \
                                          "::SinadUI::update_limits $tile_path"
    
    trace vdelete ::Sinad::State(Sinad.alertLo)  w  \
                                          "::SinadUI::update_limits $tile_path"
                                          
    trace vdelete ::SinadUI::StateUI(high_range)  w  \
                                          "::SinadUI::update_limits $tile_path"

    trace vdelete ::Sinad::State(Sinad.BellEnabled)  w  \
                                     "::SinadUI::update_alarm_small $tile_path"

    trace vdelete ::Sinad::State(Sinad.BellEnabled)  w  \
                                     "::SinadUI::update_alarm_large $tile_path"  

    trace vdelete ::Sinad::State(Sinad.peakEnable)  w  \
                                 "::SinadUI::update_peak_hold_small $tile_path"
  }
  

  #********************************************************************
  #
  #        Name: Unmap
  #
  #  Parameters: 
  #
  # Description: 
  #
  #********************************************************************
  proc Unmap { widget  meter_size } \
  {
    #puts "SinadUI::Unmap:  $widget,  $meter_size"
 
    Cleanup  $widget  $meter_size
  }


  #********************************************************************
  #
  #        Name: Destroy
  #
  #  Parameters: 
  #
  # Description: 
  #
  #********************************************************************
  proc Destroy { widget  meter_size } \
  {
    #puts "SinadUI::Destroy:  $widget,  $meter_size"
     
    Cleanup  $widget  $meter_size
  }


  #********************************************************************
  #
  #        Name: Register
  #
  #  Parameters: 
  #
  # Description: 
  #
  #********************************************************************
  proc Register { } \
  {
    #puts "SinadUI::Register"
  }


  #********************************************************************
  #
  #        Name: Update_display
  #
  #  Parameters: 
  #
  # Description: 
  #
  #********************************************************************
  proc Update_display { tile_path args } \
  {
    set cmd  "::SinadUI::Update_display_core  $tile_path"
    
    after cancel  $cmd
    
    after idle    $cmd
  }
  
  
  #********************************************************************
  #
  #        Name: Update_display_core
  #
  #  Parameters: 
  #
  # Description: Updates the display for a change in the 
  #            : engine 'State(sinad)' variable
  #
  #********************************************************************
  proc Update_display_core { tile_path } \
  {
    #puts "SinadUI::Update_display_core:  $tile_path,  $args"
    
    variable  StateUI
    
    if { ![winfo exists $tile_path] } \
    { 
      return 
    }
    
    set tile_path  [get_tile  $tile_path]
    
    if { ![winfo ismapped $tile_path] } \
    { 
      #puts "Sinad is not mapped"
      
      return 
    }

    # Update the peak hold depending on whether the peak hold is set
      set peak_enable  $::Sinad::State(Sinad.peakEnable)

      if { $peak_enable == 1 } \
      {
        # Update the reading to the high peak value
        set reading_raw   $::Sinad::State(sinad)
        set reading_cond  $::Sinad::State(Sinad.peakHi)
        set reading_peak  $::Sinad::State(Sinad.peakHi)
      } \
      else  \
      {
        # Update the reading to the conditioned reading value
        set reading_raw   $::Sinad::State(sinad)
        set reading_cond  $::Sinad::State(sinad)
        set reading_peak  $::Sinad::State(sinad)
      }


      if { $StateUI(ref_mode) == "OFF" } \
      {
        set StateUI(reading)  $reading_cond
      }\
      else \
      {
        set reading_comp  $reading_raw

        set reading_ref   $StateUI(ref_reading)

        set StateUI(reading)  [expr $reading_comp - $reading_ref]
      }


    # ######################################################################
    # # Changed for the meterbar
    #   return
    # # Changed for the meterbar
    # ######################################################################


    # Update the bargraph, range, limits and alarm to changes in the reading
    # Update the bargraph
    update_bargraph  $tile_path $reading_raw
    
    update_stale $tile_path

    # Update the range
    update_range  $reading_raw

    # Update the alarm status
    update_alarm  $tile_path  $reading_raw

    # Update the peak hold
    update_peakhold  $tile_path  $reading_peak
            
    return
  }
  #********************************************************************
  #
  #        Name: update_stale
  #
  #  Parameters: tile to update
  #
  # Description: update the meter for stale/not stale
  #
  #********************************************************************
  proc update_stale { tile_path } \
  {
    variable StateUI
    set tile_path  [get_tile  $tile_path]
    
    if { ![winfo exists $tile_path] } \
    { 
      return 
    }

    set canvas_name  ${tile_path}.cnvs_bargraph
    set reading      ${tile_path}.lbl_reading
    
    if {$::Sinad::State(Sinad.alertStatus) & 2} \
    {
       $canvas_name itemconfigure bargraph       -stipple gray50
       $canvas_name itemconfigure bargraph_frame -fill DimGrey
    } \
    else \
    {
       $canvas_name itemconfigure bargraph       -stipple {}
       $canvas_name itemconfigure bargraph_frame -fill {}
    }
  }
  

  #********************************************************************
  #
  #        Name: get_tile                   
  #
  #  Parameters: a widget
  #
  # Description: given widget within a tile, get the name of the tile
  #
  #********************************************************************
  proc get_tile { widget } \
  {
    while {1} \
    {
      if { [catch { set class  [winfo class $widget] } result] } \
      {
        return ""
      }
         
      switch -- $class \
      {
        "SMALL_SINAD"  -
        "BIG_SINAD"    { return $widget }
      }

      set widget [winfo parent $widget]
    }
    
    return ""
  }


  #********************************************************************
  #
  #        Name: update_mid_range                     
  #
  #  Parameters: 
  #
  # Description: sets the meter midrange value
  #
  #********************************************************************
  proc update_mid_range { args } \
  {
     set cmd [namespace code "update_mid_range_core"]
     after cancel $cmd
     after idle $cmd
  }
  proc update_mid_range_core { } \
  {
    #puts "update_mid_range"
    
    variable  StateUI
    
    
    if { $StateUI(high_range) == "" } \
    { 
      return 
    }
    
    set StateUI(mid_range) [format "%g" [expr $StateUI(high_range) / 2.0]]
    
    return
  }
  
  
  #********************************************************************
  #
  #        Name: get_high_range                     
  #
  #  Parameters: 
  #
  # Description: gets meter range
  #
  #********************************************************************
  proc get_high_range { } \
  {
    #puts "get_high_range"
    
    variable  StateUI
    
    if { $StateUI(high_range) == "" } \
    {
      set StateUI(high_range) [get_highest_range]
    }
    
    return $StateUI(high_range)
  }
  
  
  #********************************************************************
  #
  #        Name: get_highest_range
  #
  #  Parameters: 
  #
  # Description: gets the meters highest range
  #
  #********************************************************************
  proc get_highest_range { } \
  {
    #puts "get_highest_range"

    variable  StateUI
    
    # Get the range value of index '1' of the menu
    set high_range  [lindex  $StateUI(ranges)  1]

    return $high_range
  }


  #********************************************************************
  #
  #        Name: update_bargraph                     
  #
  #  Parameters: reading - the new reading
  #
  # Description: updates the bargraph widget of the meter
  #
  #********************************************************************
  proc update_bargraph { tile_path  reading } \
  {
    #puts "update_bargraph:  $reading"

    if { "$reading" == "" } \
    {
      return
    }

    set tile_path  [get_tile  $tile_path]

    if { ![winfo exists $tile_path] } \
    { 
      return 
    }

    set canvas_name  ${tile_path}.cnvs_bargraph

    if { ![winfo exists $canvas_name] } \
    { 
      return 
    }
    
    # Adjust for negative readings
    set reading [expr abs($reading)]
    
    # Determine the value of the high range
    set range [get_high_range]
    
    # Convert the range to the absolute value
    set range [expr abs($range)]

    if { $reading > $range } \
    {
      set reading  $range
    }

    # Determine the present dimensions of the bargraph frame
    set bargraph_coords [$canvas_name coords bargraph_frame]
    
    foreach { x1 y1 x2 y2 } $bargraph_coords {}
  
    set meterbar_height  [expr [expr $y2 - $y1] - 1]
    
    # The value 'meterbar_height' is the max height of the bargraph (in pixels)
    set ratio [expr $meterbar_height / $range]

    set increase [expr $reading * $ratio]

    # Scale the bargraph by the correct amount
    # Determine the present dimensions
    set bargraph_coords [$canvas_name coords bargraph]

    # Initialize the coordinates
    foreach { x1 y1 x2 y2 } $bargraph_coords {}
    
    set new_height  [expr $y2 - $increase]
    
    # Display the bargraph for the present reading
    $canvas_name coords bargraph $x1 $new_height $x2 $y2
    
    return
  }
  
  
  #********************************************************************
  #
  #        Name: update_range                     
  #
  #  Parameters: reading - the new reading
  #
  # Description: updates the reading widget of the meter
  #
  #********************************************************************
  proc update_range { reading } \
  {
    #puts "update_range:  $reading"

    variable  StateUI

    if { "$reading" == "" } \
    {
      return
    }

    # Adjust for negative readings
    set reading [expr abs($reading)]

    # Determine the value of the high range
    set range [get_high_range]
    
    # Convert the range to the absolute value
    set range [expr abs($range)]
  
    set quarter_range  [expr $range / 4.0]
    
    set auto_range  $StateUI(auto_range)
  
    set pres_range  $StateUI(high_range)
  
    # Get the present active index
    set pres_index  0

    foreach label_range $StateUI(ranges) \
    {
      if { $label_range == $pres_range } \
      {
        break
      }
      
      incr pres_index
    }
    
    if { $auto_range == "AR" } \
    {
      if { $reading > $range } \
      {
        incr pres_index -1
      } \
      elseif { $reading < $quarter_range } \
      {
        incr pres_index
      } \
      else \
      {
        # No range change. all done
         return
      }
      
      if { $pres_index < 1 } \
      {
        set new_index 1
      } \
      elseif { $pres_index > $StateUI(max_range) } \
      {
        set new_index $StateUI(max_range)
      } \
      else \
      {
        set new_index $pres_index
      }

      set new_value  [lindex $StateUI(ranges) $new_index]
        
      # Don't update the range if it is the same        
      if { $StateUI(high_range) != $new_value } \
      {
        set StateUI(high_range)  $new_value
      }

      set StateUI(auto_range)  "AR"
    }

    return
  }
  
  
  #********************************************************************
  #
  #        Name: update_peakhold                     
  #
  #  Parameters: peak_reading - the peak reading
  #
  # Description: updates the bargraph peak hold indicator
  #
  #********************************************************************
  proc update_peakhold { tile_path  peak_reading } \
  {
    #puts "update_peakhold:  $peak_reading" 
    
    variable  StateUI
    
    if { "$peak_reading" == "" } \
    {
      return
    }

    set tile_path  [get_tile  $tile_path]

    if { ![winfo exists $tile_path] } \
    { 
      return 
    }

    set canvas_name   ${tile_path}.cnvs_bargraph
    
    if { ![winfo exists $canvas_name] } \
    { 
      return 
    }
    
    set marker_x1 [option get $tile_path marker_x1 {}]
    set marker_x2 [option get $tile_path marker_x2 {}]

    # Adjust for negative readings
    set peak_reading [expr abs($peak_reading)]

    # Delete the old peak hold marker
    $canvas_name delete peak_marker

    # Determine peak hold condition
    if { $::Sinad::State(Sinad.peakEnable) == 1 } \
    { 
      # Determine the value of the high range
      set range [get_high_range]
  
      set range [expr abs($range)]
      
      # Determine the present dimensions of the bargraph frame
      set bargraph_coords [$canvas_name coords bargraph_frame]
    
      foreach { x1 y1 x2 y2 } $bargraph_coords {}

      set marker_y  [expr [expr $y2 - $y1] - 1]
    
      # Determine the ratio of the reading to the scale
      set ratio [expr $marker_y / $range]
        
      # Determine the present dimensions
      set bargraph_coords [$canvas_name coords bargraph]
      
      foreach { x1 y1 x2 y2 } $bargraph_coords {}
  
      set new_peak_val [expr $peak_reading * $ratio]
  
      # If the reading exceeds the scale of the bargraph,
      # limit the peak hold marker to the upper edge of the bargraph
      if { $peak_reading > $range } \
      {
        set new_peak_val $marker_y
      }
      
      set new_peak [expr $y2 - $new_peak_val]
    
      set new_peak1  $new_peak
      
      set new_peak2  [expr $new_peak + 2]
     
      $canvas_name create rect $marker_x1 $new_peak1        \
                               $marker_x2 $new_peak2        \
                               -fill white                  \
                               -tags peak_marker
    }
   
    return
  }
  
  
  #********************************************************************
  #
  #        Name: update_alarm                     
  #
  #  Parameters: new_reading - the new reading
  #
  # Description: updates the alarm operation
  #
  #********************************************************************
  proc update_alarm { tile_path  new_reading } \
  {
    #puts "update_alarm:  $tile_path,  $new_reading"
 
    variable StateUI

    set tile_path  [get_tile  $tile_path]

    if { ![winfo exists $tile_path] } \
    { 
      return 
    }

    set canvas_name  ${tile_path}.cnvs_bargraph
    
    if { ![winfo exists $canvas_name] } \
    { 
      return 
    }
    
    if { $new_reading == "" } \
    {
      return
    }
    
    set text_color black
    set bar_color green
    set alarm off
    
    if {$::Sinad::State(Sinad.alertStatus) & 0x200 } \
    { 
      set bar_color red
    } \
    elseif {$::Sinad::State(Sinad.alertStatus) & 0x800  } \
    { 
      set bar_color blue
    } \
    else \
    { 
      if {$::Sinad::State(Sinad.alertStatus) & 0x101D  } \
      {
        set bar_color yellow
        set text_color yellow
      }
    }
    $canvas_name itemconfigure bargraph -fill $bar_color
    ${tile_path}.lbl_reading config -fg $text_color
       
    return
  }


  #********************************************************************
  #
  #        Name: update_limits
  #
  #  Parameters:
  #
  # Description: updates the limits of the meter
  #
  #********************************************************************
  proc update_limits { tile_path  args } \
  {
     set cmd  "SinadUI::update_limits_core  $tile_path"

     after cancel $cmd
     
     after idle $cmd
  }
  proc update_limits_core { tile_path  } \
  {
     update_limits_hi  $tile_path
     update_limits_lo  $tile_path
  }
  
  proc update_limits_hi { tile_path } \
  {
    #variable  StateUI
    
    if { ![winfo exists $tile_path] } \
    { 
      return 
    }

    set meter_class  [winfo class $tile_path]
    
    set canvas_name  ${tile_path}.cnvs_bargraph
    
    if { ![winfo exists $canvas_name] } \
    { 
      return 
    }

    set line_width  [option get $tile_path line_width {}]

    # Determine the present dimensions of the bargraph frame
    set bargraph_coords [$canvas_name coords bargraph_frame]
    
    foreach { x1 y1 x2 y2 } $bargraph_coords {}

    set meterbar_height  [expr [expr $y2 - $y1] - 1]
    
    # Delete the upper limit marker from the screen
    $canvas_name delete high_marker

    $canvas_name delete high_value

    if { $::Sinad::State(Sinad.alertHi_text) == "ON" } \
    {
      # Get the value of the limit
      set limit_value  $::Sinad::State(Sinad.alertHi)

      if { $limit_value == "" } \
      {
        set limit_value  0.00
      }

      set limit_value  [format "%.1f" $limit_value]
         
      # Determine the value of the high range
      set range [get_high_range]
    
      # If the value exceeds the range, 
      # return without displaying the marker
      if { $limit_value > $range } \
      {
        return
      }
      
      if { $limit_value < 0.0 } \
      {
        return
      }

      # The value 'meterbar_height' is the max height of the bargraph (in pixels)
      set ratio [expr $meterbar_height / $range]

      # Use the value of 'limit_value' to set the marker
      set marker_pos [expr $limit_value * $ratio]
  
      # Scale the bargraph by the correct amount
        # Determine the present dimensions
      set bargraph_coords [$canvas_name coords bargraph]
      
      #puts "bargraph_coords:  $bargraph_coords"
      
      # Initialize the coordinates
      foreach {x1 y1 x2 y2 } $bargraph_coords {}

      set marker_height  [expr $y2 - $marker_pos]
    
      # Display the bargraph for the present reading
      $canvas_name create line 0  $marker_height           \
                                       $line_width $marker_height  \
                                       -width 2                    \
                                       -fill red                   \
                                       -tags high_marker

      if { $meter_class == "BIG_SINAD" } \
      {
        # Display the value of the limit next to the marker on the big meter
        $canvas_name create text 0  [expr $marker_height - 8]   \
                                         -text $limit_value             \
                                         -font {-size 12}               \
                                         -anchor w                      \
                                         -fill red                      \
                                         -tags high_value
      }
      
    }
      
   
    return
  }
  
  proc update_limits_lo { tile_path } \
  {
    #puts "update_limits_lo:  $tile_path"

    #variable  StateUI
    
    if { ![winfo exists $tile_path] } \
    { 
      return 
    }

    set meter_class  [winfo class $tile_path]
    
    set canvas_name  ${tile_path}.cnvs_bargraph
    
    if { ![winfo exists $canvas_name] } \
    { 
      return 
    }

    set line_width  [option get $tile_path line_width {}]

    # Determine the present dimensions of the bargraph frame
    set bargraph_coords [$canvas_name coords bargraph_frame]
    
    foreach { x1 y1 x2 y2 } $bargraph_coords {}

    set meterbar_height  [expr [expr $y2 - $y1] - 1]
    
    # Delete the lower limit marker from the screen
    $canvas_name delete low_marker
      
    $canvas_name delete low_value

    if { $::Sinad::State(Sinad.alertLo_text) == "ON" } \
    {
      # Get the value of the limit
      set limit_value  $::Sinad::State(Sinad.alertLo)
         
      if { $limit_value == "" } \
      {
        set limit_value  0.00
      }

      set limit_value  [format "%.1f" $limit_value]
         
      # Determine the value of the high range
      set range [get_high_range]
    
      # If the value exceeds the range, 
      # return without displaying the marker
      if { $limit_value > $range } \
      {
        return
      }
  
      if { $limit_value < 0.0 } \
      {
        return
      }

      # The value 'meterbar_height' is the max height of the bargraph (in pixels)
      set ratio [expr $meterbar_height / $range]

      # Use the value of 'limit_value' to set the marker
      set marker_pos [expr $limit_value * $ratio]
  
      # Scale the bargraph by the correct amount
        # Determine the present dimensions
        set bargraph_coords [$canvas_name coords bargraph]
        foreach { x1 y1 x2 y2 } $bargraph_coords {}

        set marker_height  [expr $y2 - $marker_pos]
      
      # Display the bargraph for the present reading
      $canvas_name create line 0  $marker_height           \
                                  $line_width $marker_height  \
                                  -width 2                    \
                                  -fill blue                  \
                                  -tags low_marker

      if { $meter_class == "BIG_SINAD" } \
      {
        # Display the value of the limit next to the marker on the big meter
        $canvas_name create text 0  [expr $marker_height + 7]   \
                                         -text $limit_value             \
                                         -font {-size 12}               \
                                         -anchor w                      \
                                         -fill blue                     \
                                         -tags low_value
      }
       
    }  
  
   
    return
  }
  
  
  #********************************************************************
  #
  #        Name: update_alarm_small
  #
  #  Parameters: 
  #
  # Description: 
  #
  #********************************************************************
  proc update_alarm_small { tile_path  args } \
  {
    #puts "update_alarm_small:  $tile_path"

    set cmd  "::SinadUI::update_alarm_small_core  $tile_path "
    
    after cancel  $cmd
    
    after idle    $cmd
    
    return
  }

  proc update_alarm_small_core { tile_path  } \
  {
    #puts "update_alarm_small_core:  $tile_path"

    variable  StateUI

    set tile_path  [get_tile  $tile_path]

    if { ![winfo exists $tile_path] } \
    { 
      return 
    }
     
    switch -- $::Sinad::State(Sinad.BellEnabled) \
    {
      0  { ${tile_path}.btn_alarm configure -image image_smallbelloff }
      1  { ${tile_path}.btn_alarm configure -image image_smallbellon  }
    }
     
    return
  }
  

  #********************************************************************
  #
  #        Name: update_alarm_large
  #
  #  Parameters: 
  #
  # Description: 
  #
  #********************************************************************
  proc update_alarm_large { tile_path  args } \
  {
    #puts "update_alarm_large:  $tile_path,  $args"

    set cmd  "::SinadUI::update_alarm_large_core  $tile_path"
    
    after cancel  $cmd
    
    after idle    $cmd
    
    return
  }

  proc update_alarm_large_core { tile_path } \
  {
    #puts "update_alarm_large_core:  $tile_path"

    variable  StateUI

    set tile_path  [get_tile  $tile_path]

    if { ![winfo exists $tile_path] } \
    { 
      return 
    }

    switch -- $::Sinad::State(Sinad.BellEnabled) \
    {
      0  { ${tile_path}.btn_alarm_bell configure -image image_bigbelloff }
      1  { ${tile_path}.btn_alarm_bell configure -image image_bigbellon  }
    }
    
    return
  }


  #********************************************************************
  #
  #        Name: update_peak_hold_small
  #
  #  Parameters:
  #
  # Description: Updates the Peak Hold button on the small meter 
  #              to changes in the 'peakEnable' state variable
  #
  #********************************************************************
  proc update_peak_hold_small { tile_path  args } \
  {
    set cmd  "::SinadUI::update_peak_hold_small_core  $tile_path"
    
    after cancel  $cmd
    
    after idle  $cmd
    
    return
  }
  
  proc update_peak_hold_small_core { tile_path } \
  {
    #puts "update_peak_hold_small_core:  $tile_path"

    #variable StateUI
    
    set tile_path  [get_tile  $tile_path]
    
    if { ![winfo exists $tile_path] } \
    { 
      return 
    }

    set btn_peak_hold  ${tile_path}.btn_peak_hold
 
    if { ![winfo exists $btn_peak_hold] } \
    { 
      return 
    }

    switch -- $::Sinad::State(Sinad.peakEnable) \
    {
      1  { $btn_peak_hold  config  -relief sunken  -bd 3 }
         
      0  { $btn_peak_hold  config  -relief raised  -bd 1  }
    }
    
    return
  }


  #********************************************************************
  #
  #        Name: set_meter_range                     
  #
  #  Parameters: new_value   - requested new value 
  #                           -1 - no new value requested
  #
  # Description: sets meter range, displays AR and sets scale variable
  #
  #********************************************************************
  proc set_meter_range { { new_value "" } } \
  {
    #puts "set_meter_range:  $new_value"

    variable  StateUI
    
    
    if { $new_value != "" } \
    {
      set StateUI(range_selection)  $new_value
    }
    

    # If the variable has no value, set the value to 'AR'
    if { $StateUI(range_selection) == "" } \
    {
      set StateUI(range_selection)  "AR"
    }


    if { $StateUI(range_selection) == "AR" } \
    { 
      # Set the namespace variable 
      set StateUI(auto_range)  "AR"
    
      # Set the meter to the highest range initially
      set range_value  [get_highest_range]

      set StateUI(high_range)  $range_value
    } \
    else \
    { 
      # Set the namespace variable 
      set StateUI(auto_range)  ""
  
      set range_value  $StateUI(range_selection)
       
      # Set the button text to indicate the range selected
      set StateUI(high_range)  $range_value
    } 
    
  
    return
  }
  
  
  #********************************************************************
  #
  #        Name: set_ref_mode
  #
  #  Parameters: 
  #
  # Description: sets the Reference Mode of the meter
  #
  #********************************************************************
  proc set_ref_mode { { new_value "" } } \
  {
    #puts "set_ref_mode:  $new_value"
    
    variable  StateUI
    
    if { $new_value == "" } \
    {
      if { $StateUI(ref_mode) == "ON" } \
      {
        set StateUI(ref_mode)  "OFF"
      } \
      else \
      {
        set StateUI(ref_mode)  "ON"
      }
    } \
    else \
    {
      set StateUI(ref_mode)  $new_value
    }       
    

    if { $StateUI(ref_mode) == "ON" } \
    {
      set StateUI(ref_indicate)  "REF"
      
      # Set the ref_reading value if the present value is 0.0,
      # which is the initialization value
      if { $StateUI(ref_reading) == 0.0 } \
      {
        set StateUI(ref_reading)   $StateUI(reading)
      }
    }
  
    if { $StateUI(ref_mode) == "OFF" } \
    {
      set StateUI(ref_indicate)  ""
      set StateUI(ref_reading)   0.0
    }
  
    return
  }


  #********************************************************************
  #
  #        Name: set_auto_search
  #
  #  Parameters: 
  #
  # Description: sets the Auto Search mode of the meter
  #
  #********************************************************************
  proc set_auto_search { { new_value "" } } \
  {
    #puts "set_auto_search:  $new_value"
    
    variable  StateUI
    
    if { $new_value == "" } \
    {
      if { $StateUI(auto_search) == "ON" } \
      {
        set StateUI(auto_search)  "OFF"
      } \
      else \
      {
        set StateUI(auto_search)  "ON"
      }
    } \
    else \
    {
      set StateUI(auto_search)  $new_value
    }       
    

    if { $StateUI(auto_search) == "ON" } \
    {

    }
  
    if { $StateUI(auto_search) == "OFF" } \
    {

    }
  
    return
  }


  #***************************************************************************
  #
  #        Name: config_rf_level_entry
  # 
  #  Parameters: tile_path - tile path 
  #
  # Description: Configures the SINAD RF Gen level entry widget, according to
  #            : the RF Gen configuration when the Big SINAD is mapped.
  #
  #***************************************************************************
  proc config_rf_level_entry { tile_path } \
  {
    #puts "config_rf_level_entry:  $tile_path"
    
    set tile_path [get_tile  $tile_path]
      
    set level_widget ${tile_path}.ent_rf_level

    
    if { $::Generator::State(rfout) == "GEN" } \
    {
      if { $::GeneratorUI::levelunit == "dBm" } \
      {
        bind_numeric_spin $level_widget 3 1 10 -110 ::GeneratorUI::enter_gen_level
      }
      
      if { $::GeneratorUI::levelunit == "mV" } \
      {
        bind_numeric_spin $level_widget 3 4 707.1068 .0007 ::GeneratorUI::enter_gen_level
      }
      
      if { $::GeneratorUI::levelunit == "uV" } \
      {
        bind_numeric_spin $level_widget 6 2 707106.81 .0007 ::GeneratorUI::enter_gen_level
      }
    } \
    else \
    {
      if { $::GeneratorUI::levelunit == "dBm" } \
      {
        bind_numeric_spin $level_widget 3 1 -30 -137 ::GeneratorUI::enter_gen_level
      }
      
      if { $::GeneratorUI::levelunit == "mV" } \
      {
        bind_numeric_spin $level_widget 3 4 7.0711 0 ::GeneratorUI::enter_gen_level
      }
      
      if { $::GeneratorUI::levelunit == "uV" } \
      {
        bind_numeric_spin $level_widget 6 2 7071.10 0 ::GeneratorUI::enter_gen_level
      }
    }
    
    return
  }
  #***************************************************************************
  #
  #        Name: Show_adv
  #  Parameters: widget 
  #
  # Description: Show the advanced setup tile
  #
  #***************************************************************************
  proc Show_adv { tile_path } \
  {
    set top_level [winfo toplevel $tile_path]
   
    set tile_path  [name_child  $top_level sinad_adv]

    return [::Screens::create_toplevel_tile  $tile_path SINAD_ADV]
  }
  
  #***************************************************************************
  #
  #        Name: set_lnotch/set_lnoise/set_hnotch/set_hnoise
  #  Parameters: none 
  #
  # Description: set the notch and noise values
  #
  #***************************************************************************
  proc set_lnotch {  } \
  {
    Sinad lnotch $::Sinad::State(lnotch) -nowait
  }
   proc set_lnoise {  } \
  {
    Sinad lnoise $::Sinad::State(lnoise) -nowait
  }

  proc set_hnotch {  } \
  {
    Sinad hnotch $::Sinad::State(hnotch) -nowait
  }

  proc set_hnoise {  } \
  {
    Sinad hnoise $::Sinad::State(hnoise) -nowait
  }
  proc Restore_default {  } \
  {
    Sinad lnoise    100.0 -nowait
    Sinad hnoise  10000.0 -nowait
    Sinad lnotch    900.0 -nowait
    Sinad hnotch   1100.0 -nowait
                
  }


  #****************************************************************************
  # Name:        getSaveState
  #  Parameters:  none
  # Description: This function returns a list of TCL commands which can be used
  #              to set the SinadUI tiles to its current state
  #****************************************************************************
  proc getSaveState { } \
  {
    variable  StateUI
 
    set returnList \
    {
      set ::SinadUI::StateUI(range_selection)  {$StateUI(range_selection)}
      set ::SinadUI::StateUI(ref_mode)         {$StateUI(ref_mode)}
    }

    return [subst -nocommands $returnList]
  }


  #****************************************************************************
  # Name:        Sync
  #  Parameters:  none
  # Description: Write state variables to SinadUI
  #****************************************************************************
  proc Sync { } \
  {
    eval [getSaveState]
  }
 
  trace variable ::SinadUI::StateUI(high_range)  w  \
                                                  "::SinadUI::update_mid_range"


  update_mid_range

};  # End of SinadUI namespace


#****************************************************************************
# This registers the tiles with the list for saving instrument states
#****************************************************************************
namespace eval ::SavedStates {

   variable tiles   
   lappend tiles SinadUI
}


namespace eval View \
{
   lappend PreCreate SinadUI
}



