#############################################################################
#                                                                           #
# IFR SYSTEMS, INC        COPYRIGHT (C)1998,99      DESIGN ENGINEERING      #
# ------------------------------------------------------------------------- #
#                                                                           #
# FILE        : 3090.RssiUI.tcl                                             #
#                                                                           #
# PRODUCT     : FITS                                                        #
#                                                                           #
# AUTHOR      : P. McCormick                                                #
#                                                                           #
# DESCRIPTION : RSSI Meter User Interface                                   #
#                                                                           #
#############################################################################

package require RLmeter


namespace eval RssiUI \
{

  # Global variables
  variable  LongName     "rssi"
  variable  ShortName    "RSSI"
  variable  Size         Eighth_v
                     
  variable  StateUI
  
  set  StateUI(reading,max)          200
  set  StateUI(reading,next_range)          0
  set  StateUI(reading,middle)          0
  set  StateUI(reading,min)          0
  set  StateUI(reading)          0
  set  StateUI(reading,count)          0
  set  StateUI(reading,color)   red
  set  StateUI(reading,textcolor)   black
  set  StateUI(peak_reading)     0
  set  StateUI(hi_limit,on) 0
  set  StateUI(hi_limit,value) 0
  set  StateUI(lo_limit,on) 0
  set  StateUI(lo_limit,value) 0
  set  StateUI(alarm,enable) 0
  set  StateUI(alarm,state) 0

  set  StateUI(range,W)    0
  set  StateUI(range,dBm)    0
  set  StateUI(range_selection,W)    0
  set  StateUI(range_selection,dBm)  0
  set  StateUI(range_selection,text)  "AR"
  
  set  StateUI(range,auto)  1
  set  StateUI(range,auto_txt) ""
            
  set  StateUI(uncal)            ""

  set  StateUI(alarm)            "OFF"
  set  StateUI(peak_hold)        "OFF"
  set  StateUI(units)            "dB"
           

  set  StateUI(average)          ""
  set  StateUI(cable_loss)       0.0
  set  StateUI(input)            ""
  
  set  StateUI(ref_mode)         "OFF"
  set  StateUI(ref_indicate)     ""
  set  StateUI(ref_reading)      0.0


  array set ranges \
  {
     W,max   11
     W,0     {200 100    0  75}
     W,1     {200 100    0  75}
     W,2     {100  50    0  37.5}
     W,3     { 50  25    0  15}
     W,4     { 20  10    0  7}
     W,5     { 10   5    0  3.75}
     W,6     {  5   2    0  1.5}
     W,7     {  2   1    0  .75}
     W,8     {  1   0.5  0  3.75}
     W,9     {  0.5 0.2  0  .15}
     W,10    {  0.2 0.1  0  .075}
     W,11    {  0.1 0.05 0  0}
     
     dBm,max   2
     dBm,0     {53  -33 -120 -15}
     dBm,1     {53  -33 -120 -15}
     dBm,2     {10  -55 -120 -35}
     dBm,3     {-30 -75 -120 -120}
  }
  # and the vars for the menus
  variable ranges_W
  variable ranges_dBm
  set ranges_W \
  {
     AR 200W 100W 50W 20W 10W 5W 2W 1W 0.5W 0.2W 0.1W
  }
   set ranges_dBm \
  {
     AR +53dBm +10dBm -30dBm
  }
   
  #********************************************************************
  #
  #       Name: get_tile                   
  #
  # Parameters: a widget
  #
  # Description: given widget within a tile, get the name of the tile
  #
  #********************************************************************
  proc get_tile { widget } \
  {
    while {1} \
    {
      if { [catch { set class  [winfo class $widget] } result] } \
      {
        return ""
      }
         
      switch -- $class \
      {
        "SMALL_RSSI" -
        "BIG_RSSI"   { return $widget }
      }

      set widget [winfo parent $widget]
    }
    
    return ""
  }
  
  #********************************************************************
  #
  #       Name: Check_alarm                     
  #
  # Parameters: 
  #
  # Description: updates the alarm operation
  #
  #********************************************************************
  proc Check_alarm {  } \
  {
    variable StateUI

    if { $RLmeter::State(status) == 0 } \
    {
       set StateUI(reading,textcolor) black
    } \
    else \
    {
       # Per bug 828, we will NOT show yellow if:
       # The selected port is the T/R prot
       # the reading is above the spline range
       if { ($::Receiver::State(rfinput_raw) == 0) && ($RLmeter::State(status) == -11) } \
       {
          set StateUI(reading,textcolor) black

          # Clear the UNCAL label
          set StateUI(uncal)  ""
       } \
       else \
       {
         set StateUI(reading,textcolor) yellow
         
         # Display the UNCAL label
         set StateUI(uncal)  "UNCAL"
       }
    }

    set temp [expr $::RLmeter::State(alertStatus) & 0x200]
    
    if { $temp != 0 } \
    {
       # High alert was exceeded
       set StateUI(reading,color) red
    } \
    else \
    {
       set temp [expr $::RLmeter::State(alertStatus) & 0x800]
       
       if { $temp != 0 } \
       {
          # Low alert was exceeded
          set StateUI(reading,color) blue
       } \
       else \
       {
          if { $::RLmeter::State(status) == 0 } \
          {
             set StateUI(reading,color) green

             # Clear the UNCAL label
             set StateUI(uncal)  ""
          } \
          else \
          {
             set StateUI(reading,color) yellow

             # Display the UNCAL label
             set StateUI(uncal)  "UNCAL"
          }
       }
    }
  }

  
  #********************************************************************
  #
  #       Name: toggle_meter_peakhold
  #
  # Parameters: new_value   - requested new value 
  #                           -1 - no new value requested
  #
  # Decsription: Displays PH and sets peak hold variable
  #
  #********************************************************************
  proc toggle_meter_peakhold {  } \
  {
     if {$::RLmeter::State(peakEnable) == 0 } \
     {
        RLmeter peakhold 1
        RLmeter phreset

     } \
     else \
     {
        RLmeter peakhold 0
     }
  }
 
  
  #********************************************************************
  #
  #       Name: set_meter_range_W                    
  #
  # Parameters: 
  # Description: sets meter range
  #
  #********************************************************************
  proc set_meter_range_W { } \
  {
    variable StateUI
   
    if {$StateUI(range_selection,W) == 0 } \
    {
       # autorange selected
       set StateUI(range,auto) 1
       set StateUI(range,auto_txt) "AR"
    } \
    else \
    {
       set StateUI(range,auto) 0
       set StateUI(range,auto_txt) ""
       set StateUI(range,W) $StateUI(range_selection,W)
    }
    update_range
    return
  }
  #********************************************************************
  #
  #       Name: set_meter_range_dBm                    
  #
  # Parameters: 
  # Description: sets meter range
  #
  #********************************************************************
  proc set_meter_range_dBm { } \
  {
    variable StateUI
    
    if {$StateUI(range_selection,dBm) == 0 } \
    {
       # autorange selected
       set StateUI(range,auto) 1
       set StateUI(range,auto_txt) "AR"
    } \
    else \
    {
       set StateUI(range,auto) 0
       set StateUI(range,auto_txt) ""
       set StateUI(range,dBm) $StateUI(range_selection,dBm)
    }
    update_range
    return
  }  
  #********************************************************************
  #
  #       Name: set_meter_cable_loss
  #
  # Parameters: new_value   - requested new value 
  #                           -1 - no new value requested
  #
  # Decsription: sets meter cable loss
  #
  #********************************************************************
  proc set_meter_cable_loss { { new_value "" } } \
  {
    variable StateUI
    #puts "set_meter_cable_loss:  $meter_type"

    if { $new_value == "" } \
    {
      set new_value  $::RssiUI::StateUI(cable_loss)
    } 
    
    if { $new_value < 0 } \
    {
      set new_value  0.0
    }
    
    # Set the namespace variable 
    set StateUI(cable_loss)  $new_value
  
    # Set the meter engine
    #Rssi  cableloss  $new_value  -nowait

    return
  }


  #********************************************************************
  #
  #       Name: set_meter_units
  #
  # Parameters: 
  # Decsription: 
  #
  #********************************************************************
  proc set_meter_units { } \
  {
    variable StateUI
    #puts "set_meter_units:  $meter_type"
    
    # if we aren't cal'ed, force units to dB.
    
    if {$RLmeter::State(status) == -2 } \
    {
       set StateUI(units) dB
       return
    } \
    elseif {$RLmeter::State(status) == -1 } \
    {
       set StateUI(units) dB
       return
    }
   
    switch -- $StateUI(units) \
    {
        "W"     { set StateUI(units) "dBm"}
        default { set StateUI(units) "W" }
    }
  } 
  #********************************************************************
  #
  #       Name: Update_units
  #
  # Parameters: 
  #
  # Decsription: recomputes our setup based on units setup
  #
  #********************************************************************
  proc Update_units { args } \
  {
     after cancel "RssiUI::Update_units_core"
     after idle   "RssiUI::Update_units_core"
  }
  proc Update_units_core {  } \
  {
     Update_readings_core

     Update_limits_core

     Update_peakhold_core
     update_range
  }
  
  #********************************************************************
  #
  #       Name: update_meter_units
  #
  # Parameters: tile to update
  #
  # Decsription: updates display for new units
  #
  #********************************************************************
  proc update_meter_units_small { tile_path args } \
  {
     set cmd "RssiUI::update_meter_units_small_core $tile_path"
     after cancel $cmd
     after idle   $cmd
  }
  proc update_meter_units_small_core { tile_path } \
  {
     variable StateUI
     # swap the menus to the approprate menus

     switch $StateUI(units) \
     {
        W   \
        {
            $tile_path.mnu_range config -menu $tile_path.mnu_range.menu_W
        }
        dB  -
        dBm \
        {
            $tile_path.mnu_range config -menu $tile_path.mnu_range.menu_dBm
        }
     }
  }
  proc update_meter_units_large { tile_path args } \
  {
     set cmd "RssiUI::update_meter_units_large_core $tile_path"
     after cancel $cmd
     after idle   $cmd
  }
  proc update_meter_units_large_core { tile_path } \
  {
     variable StateUI
     # swap the menus to the approprate menus
     switch $StateUI(units) \
     {
        W   \
        {
            $tile_path.mnu_range config -menu $tile_path.mnu_range.menu_W
            grid forget $tile_path.ent_high_limit_dBm
            grid forget $tile_path.ent_low_limit_dBm
            ::Screens::position_widget $tile_path.ent_high_limit_W
            ::Screens::position_widget $tile_path.ent_low_limit_W
        }
        dB  -
        dBm \
        {
            $tile_path.mnu_range config -menu $tile_path.mnu_range.menu_dBm
            grid forget $tile_path.ent_high_limit_W
            grid forget $tile_path.ent_low_limit_W
            ::Screens::position_widget $tile_path.ent_high_limit_dBm
            ::Screens::position_widget $tile_path.ent_low_limit_dBm
        }
     }
  }
  

  #********************************************************************
  #
  #        Name: update_peak_hold_small
  #
  #  Parameters:
  #
  # Description: Updates the Peak Hold button on the small meter 
  #              to changes in the 'peakEnable' state variable
  #
  #********************************************************************
  proc update_peak_hold_small { tile_path  args } \
  {
    set cmd  "::RssiUI::update_peak_hold_small_core  $tile_path"
    
    after cancel  $cmd
    
    after idle  $cmd
    
    return
  }
  
  proc update_peak_hold_small_core { tile_path } \
  {
    #puts "update_peak_hold_small_core:  $tile_path"

    #variable StateUI
    
    set tile_path  [get_tile  $tile_path]
    
    if { ![winfo exists $tile_path] } \
    { 
      return 
    }

    set btn_peak_hold  ${tile_path}.btn_peak_hold
 
    if { ![winfo exists $btn_peak_hold] } \
    { 
      return 
    }

    switch -- $::RLmeter::State(peakEnable) \
    {
      1  { $btn_peak_hold  config  -relief sunken }
         
      0  { $btn_peak_hold  config  -relief raised }
    }
    
    return
  }


  #********************************************************************
  #
  #       Name: set_meter_cal
  #
  # Parameters: 
  #
  # Decsription: sets meter calibration procedure
  #
  #********************************************************************
  proc set_meter_cal { widget } \
  {
    set tile_path [get_tile $widget]
    set toplevel [winfo toplevel $tile_path]
    set prompt [name_child $toplevel cal_rssi_ant]
    
    if { ![winfo exists  $prompt] } \
    {
      set prompt  [Screens::create_toplevel_tile  $prompt  RSSI_CAL]
    }
    
    if {$::Receiver::State(rfinput) == "T/R"} \
    {
       ::Screens::position_widget $prompt.prompt_tr
    } \
    else \
    {
       ::Screens::position_widget $prompt.prompt_ant
    }
    return
  }
  #********************************************************************
  #
  #       Name: 
  #
  # Parameters: 
  #
  # Decsription: 
  #
  #********************************************************************
  proc  Cal_start { widget } \
  {
    variable StateUI
    set StateUI(cal,color) blue

    set top_level [winfo toplevel $widget]
    $top_level.ok     config -state disabled   
    RLmeter::Cal_start
    ::Screens::position_widget $top_level.progress
    
    trace variable RLmeter::State(cal_done) w "after idle RssiUI::Cal_state $top_level"
    return
  }
  #********************************************************************
  #
  #       Name: 
  #
  # Parameters: 
  #
  # Decsription: 
  #
  #********************************************************************
  proc  Cal_OK { widget } \
  {
     set top_level [winfo toplevel $widget]
     $top_level.ok invoke
  }
  proc Cal_cancel { widget } \
  {
     set top_level [winfo toplevel $widget]
     $top_level.cancel invoke
  }
  
  #********************************************************************
  #
  #       Name: 
  #
  # Parameters: 
  #
  # Decsription: 
  #
  #********************************************************************
  proc  Cal_cleanup { top_level } \
  {
    trace vdelete RLmeter::State(cal_done) w "after idle RssiUI::Cal_state $top_level"
    return
  }
  #********************************************************************
  #
  #       Name: 
  #
  # Parameters: 
  #
  # Decsription: 
  #
  #********************************************************************
  proc  Cal_state { top_level args } \
  {
     set cmd "RssiUI::Cal_state_core $top_level"
     after cancel $cmd
     after idle $cmd
  }
  proc  Cal_state_core { top_level } \
  {
  
     variable StateUI
     if ![winfo exists $top_level] { CallTrace; return }
     if {$RLmeter::State(cal_done) == 0 } \
     {
        set StateUI(cal,color) blue
        # cal started - disable the OK button.
        $top_level.ok config -state disabled
        return
     }
     # cal finished - what was the result?
     $top_level.ok config -state normal
     switch -- $RLmeter::State(cal,status) \
     {
       5 -
       8 \
       {
          # successful test - destroy our window
          destroy $top_level
          return
       }
     }
     
     set StateUI(cal,color) red

  }
  
  
  #********************************************************************
  #
  #       Name: 
  #
  # Parameters: 
  #
  # Decsription: 
  #
  #********************************************************************
  proc  Cal_abort { widget } \
  {
    if {$RLmeter::State(cal_done) } \
    {
       set top_level [winfo toplevel $widget]
       destroy $top_level
       return
    }
    RLmeter::Cal_abort
    return
  }

  #********************************************************************
  #
  #       Name: Create  
  #
  # Parameters: size - meter size 
  #
  # Decsription: 
  #
  #********************************************************************
  proc Create { tile_path  { meter_size  small } } \
  {
    variable LongName
    #puts "RssiUI::Create"
  
    #puts "tile_path:  $tile_path"

    # Call the function 'create_tile' to generate the meter tile
    # from the resource database
    set meter_class  ${meter_size}_RSSI
    
    set meter_class  [string toupper $meter_class]
    set top_level [winfo toplevel $tile_path]
    
    switch -- $meter_class \
    {
      "SMALL_RSSI" \
      {
        set tile_path  [name_child  $top_level $LongName]
        if { ![winfo exists $tile_path] } \
        {
          set tile_path  [::Screens::create_tile  $tile_path  $LongName  $meter_class]
        }
      }
      "BIG_RSSI" \
      {
        set tile_path   [name_child  $top_level  big$LongName]
      
        if { ![winfo exists $tile_path] } \
        {
          set tile_path  [::Screens::create_toplevel_tile  $tile_path  $meter_class]
        }

        # Bind the 'big meter' to the 'F8' key
        bind $tile_path <KeyRelease-F8>  "destroy  $tile_path"
      }
    }
    
    return  $tile_path
  
  }


  #******************************************************************** 
  #
  #       Name: Close
  #
  # Parameters: 
  #
  # Decsription: 
  #
  #********************************************************************
  proc Close { widget } \
  {
     set top_level [get_tile $widget]
     
     #destroy $widget
     destroy $top_level
  }

  
  #******************************************************************** 
  #
  #       Name: Map_small
  #
  # Parameters: 
  #
  # Decsription: 
  #
  #********************************************************************
  proc Map_small { tile_path } \
  {
    #puts "RssiUI::Map_small:  $tile_path"

    variable StateUI
    

    trace variable ::RssiUI::StateUI(units)  w  \
                                  "RssiUI::update_meter_units_small $tile_path"

    update_meter_units_small_core $tile_path

    trace variable ::RLmeter::State(peakEnable)  w  \
                                    "RssiUI::update_peak_hold_small $tile_path"

    update_peak_hold_small_core  $tile_path
    
    trace variable ::RssiUI::StateUI(reading,textcolor)  w  \
                                         "RssiUI::update_text_color $tile_path"


    update_text_color_core $tile_path
    
    set_ref_mode  $StateUI(ref_mode)
        
    RLmeter run ON -nowait
  }

  
  #******************************************************************** 
  #
  #       Name: Map_large
  #
  # Parameters: 
  #
  # Decsription: 
  #
  #********************************************************************
  proc Map_large { tile_path } \
  {
    #puts "RssiUI::Map_large:  $tile_path"

    variable StateUI
    
    trace variable ::RssiUI::StateUI(units)  w  \
                                  "RssiUI::update_meter_units_large $tile_path"
                     
    update_meter_units_large_core $tile_path

    trace variable ::RssiUI::StateUI(reading,textcolor)  w  \
                                         "RssiUI::update_text_color $tile_path"
    

    update_text_color_core $tile_path
    
    set_ref_mode  $StateUI(ref_mode)
    
    RLmeter run ON -nowait
  }


  #********************************************************************
  #
  #       Name: Cleanup_small
  #
  # Parameters: 
  #
  # Decsription: 
  #
  #********************************************************************
  proc Cleanup_small { tile_path  } \
  {

    trace vdelete ::RssiUI::StateUI(units)  w  \
                                  "RssiUI::update_meter_units_small $tile_path"

    trace vdelete ::RLmeter::State(peakEnable)  w  \
                                    "RssiUI::update_peak_hold_small $tile_path"

    
    trace vdelete ::RssiUI::StateUI(reading,textcolor)  w  \
                                         "RssiUI::update_text_color $tile_path"
    

    after cancel "RssiUI::Map_smallAfterIdle $tile_path"
    after cancel "RssiUI::Map_smallAfterDelay $tile_path"

    return
  }


  #********************************************************************
  #
  #       Name: Cleanup_large
  #
  # Parameters: 
  #
  # Decsription: 
  #
  #********************************************************************
  proc Cleanup_large { tile_path  } \
  {
    trace vdelete ::RssiUI::StateUI(units)  w  \
                                           "::RssiUI::Update_units  $tile_path"

    trace vdelete ::RssiUI::StateUI(units)  w  \
                                  "RssiUI::update_meter_units_large $tile_path"
                     
    trace vdelete ::RssiUI::StateUI(reading,textcolor)  w  \
                                         "RssiUI::update_text_color $tile_path"
    

    trace vdelete ::RssiUI::StateUI(reading,max)  w  "::RssiUI::Update_limits"


    after cancel "RssiUI::Map_largeAfterIdle $tile_path"
    after cancel "RssiUI::Map_largeAfterDelay $tile_path"

    return
  }


  #********************************************************************
  #
  #       Name: Unmap_small
  #
  # Parameters: 
  #
  # Decsription: 
  #
  #********************************************************************
  proc Unmap_small { tile_path } \
  {
    #puts "RssiUI::Unmap:  $tile_path"
    
    Cleanup_small $tile_path
  }


  #********************************************************************
  #
  #       Name: Unmap_large
  #
  # Parameters: 
  #
  # Decsription: 
  #
  #********************************************************************
  proc Unmap_large { tile_path } \
  {
    #puts "RssiUI::Unmap_large:  $tile_path"
    
    Cleanup_large $tile_path
  }


  #********************************************************************
  #
  #       Name: Destroy_small
  #
  # Parameters: 
  #
  # Decsription: 
  #
  #********************************************************************
  proc Destroy_small { tile_path } \
  {
    #puts "RssiUI::Destroy:  $tile_path"

    Cleanup_small $tile_path
  }


  #********************************************************************
  #
  #       Name: Destroy_large
  #
  # Parameters: 
  #
  # Decsription: 
  #
  #********************************************************************
  proc Destroy_large { tile_path } \
  {
    #puts "RssiUI::Destroy_large:  $tile_path"

    Cleanup_large $tile_path
  }


  #********************************************************************
  #
  #       Name: Register
  #
  # Parameters: 
  #
  # Decsription: 
  #
  #********************************************************************
  proc Register { } \
  {
    #puts "RssiUI::Register"

      
    return
  }


  #********************************************************************
  #
  #       Name: do_auto_range
  #
  # Parameters: 
  #
  # Description: Adjusts the displayed ranges
  #********************************************************************
  proc do_auto_range { } \
  {
     variable StateUI
     variable ranges
  
     if { $StateUI(range,auto) == 0} { return } 
     if {$StateUI(reading) == "" } { return }
         
     switch -- $StateUI(units) \
     {
        W \
        {
          upvar 0 StateUI(range,W) range
          upvar 0 StateUI(range_selection,W) range_selection
          upvar 0 ranges(W,max)              max_range
        }
        dB -
        dBm \
        {
          upvar 0 StateUI(range,dBm) range
          upvar 0 StateUI(range_selection,dBm) range_selection
          upvar 0 ranges(dBm,max)              max_range
        }
     }
     
     set new_range $range
      
     if { $StateUI(reading) > $StateUI(reading,max) } \
     {
        incr new_range -1 ;# ranges are ordered backwards
     } \
     elseif { $StateUI(reading) < $StateUI(reading,next_range) } \
     {
        # have we seen enough readings to change?
        incr StateUI(reading,count)
        if {$StateUI(reading,count) > 5 } \
        {
           incr new_range
        }
     } \
     else \
     {
        # reading is in range
        set StateUI(reading,count) 0
        return
     }
     # Range might need to change - clip it & recompute
     if {$new_range < 1 } \
     {
        set new_range 1
     } \
     elseif {$new_range > $max_range } \
     {
        set new_range $max_range
     }
     if {$range == $new_range } { return }
     
     # OK, the range really changed
     set range $new_range
     update_range
  }


  #********************************************************************
  #
  #       Name: update_range
  #
  # Parameters: 
  #
  # Description: Updates the state variables for meter bars 
  #
  #********************************************************************
  proc update_range { } \
  {
   variable StateUI
   variable ranges
   variable ranges_dBm
   variable ranges_W
     
   # locate the range string for this range
   switch -- $StateUI(units) \
   {
      W \
      {
         upvar 0 StateUI(range_selection,W) range_selection
         upvar 0 StateUI(range,W) range
         upvar 0 ranges_W ranges_units
         set units W
      }
      dB -
      dBm \
      {
         upvar 0 StateUI(range_selection,dBm) range_selection
         upvar 0 StateUI(range,dBm) range
         upvar 0 ranges_dBm ranges_units
         set units dBm
      }
   }
   # if the range is out of limits, force it in limits
   if {$range < 1} { set range 1 }

   set StateUI(reading,range) [lindex $ranges_units $range]
   
   if {$StateUI(range,auto) == 1 } \
   {
      set StateUI(auto_btn) "AR"
   } \
   else \
   {
      set StateUI(auto_btn) $StateUI(reading,range)
   }
   
   # set the new min and max display values for this range
   set StateUI(reading,max)        [lindex $ranges($units,$range) 0]
   set StateUI(reading,middle)     [lindex $ranges($units,$range) 1]
   set StateUI(reading,min)        [lindex $ranges($units,$range) 2]
   set StateUI(reading,next_range) [lindex $ranges($units,$range) 3]
  }
  
 
  #********************************************************************
  #
  #       Name: Update_readings
  #
  # Parameters: 
  #
  # Description: Updates the state variables for meter bars 
  #            : engine 'State(reading)' variable
  #
  #********************************************************************
  proc Update_readings { args } \
  {
     #puts "RssiUI::Update_readings:  $args"
   
     after cancel "RssiUI::Update_readings_core"
     after idle   "RssiUI::Update_readings_core"
  }
  
  proc Update_readings_core { } \
  {
     #puts "RssiUI::Update_readings_core"

     variable StateUI
     
     
     if ![info exists ::RLmeter::State(reading)] { return }

     
     set reading $::RLmeter::State(reading)     
     set peak    $::RLmeter::State(peakHi)
        
     if {$reading == "" } { return }
     
     if {$RLmeter::State(status) == -2 } \
     {
       if {$StateUI(units) != "dB" } \
       {
         #puts "set StateUI(units) dB"
         
         set StateUI(units) dB
       }
     } \
     elseif {$RLmeter::State(status) == -1 } \
     {
       if {$StateUI(units) != "dB" } \
       {
         #puts "set StateUI(units) dB"

         set StateUI(units) dB
       }
     } \
     elseif {$StateUI(units) == "dB" }\
     {
       #puts "set StateUI(units) dBm"

       set StateUI(units) dBm
     }
       
     switch -- $StateUI(units) \
     {
        "W" \
        {
          set reading  $::RLmeter::State(W,reading)
          set peak     $::RLmeter::State(W,peakHi)
        }
      
        "dB" -
        "dBm" \
        {
          set reading  $::RLmeter::State(dBm,reading)
          set peak     $::RLmeter::State(dBm,peakHi)
        }
        
        default { return }
     }
     
          
     if {$StateUI(reading)      != $reading } { set StateUI(reading)      $reading }
     if {$StateUI(peak_reading) != $peak    } { set StateUI(peak_reading) $peak    }

     # re-range the UI if needed
     do_auto_range

     # service any alarms
     Check_alarm
  }

 
  #********************************************************************
  # Name:        UpdateAlertStatus
  # Parameters:  None
  # Description: Updates an alarm bell status value
  #              0 = Alerts are
  #              0x02 = Hi alert ON/OFF
  #********************************************************************
  proc UpdateAlertStatus { args } \
  {
     after cancel RssiUI::UpdateAlertStatus_core
     after idle   RssiUI::UpdateAlertStatus_core
  }
  proc UpdateAlertStatus_core { } \
  {
    variable StateUI

    if { $::RLmeter::State(BellEnabled) == 1 } \
    {
       set ::RssiUI::StateUI(alarm) "ON"
       set StateUI(alarm,state) 1
    } else \
    {
       set ::RssiUI::StateUI(alarm) "OFF"
       set StateUI(alarm,state) 0
    }
  }

 
  #********************************************************************
  # Name:        UpdateAlertStrings
  # Parameters:  None
  # Description: Updates the alert text strings when alert enable changes
  #              0x01 = Low alert ON/OFF
  #              0x02 = Hi alert ON/OFF
  #********************************************************************
  proc UpdateAlertStrings { args } \
  {
     set cmd "::RssiUI::UpdateAlertStrings_core"
     after cancel $cmd
     after idle $cmd
  }
  proc UpdateAlertStrings_core { } \
  {
    variable StateUI
 
    set status  $::RLmeter::State(alerts)
    
    # High Limit
    if { $status & 0x2 } \
    {
       set StateUI(hi_limit,on) 1
    } \
    else \
    {
       set StateUI(hi_limit,on) 0
    }

    # Low limit
    if { $status & 0x1 } \
    {
       set StateUI(lo_limit,on) 1
    } \
    else \
    {
       set StateUI(lo_limit,on) 0
    }    
  }


  #********************************************************************
  #
  #       Name: Update_limits
  #
  # Parameters: 
  #
  # Description: Conditions the 1/0 value from the engine to a 
  #            : "ON"/"OFF" and
  #            : "ON"/"OFF" 
  #            : string for the user interface
  #
  #********************************************************************
  proc Update_limits { args } \
  {
     after cancel RssiUI::Update_limits_core
     after idle   RssiUI::Update_limits_core
  }
  
  proc Update_limits_core { } \
  {
     variable StateUI
 
     switch -- $StateUI(units) \
     {
        "W" \
        {
          set StateUI(lo_limit,value) [format %.2f [::Units_convert::dBm_to_W $::RLmeter::State(alertLo)]]
          set StateUI(hi_limit,value) [format %.2f [::Units_convert::dBm_to_W $::RLmeter::State(alertHi)]]
        }
      
        "dB" -
        "dBm" \
        {
          set StateUI(lo_limit,value) [format %.2f $::RLmeter::State(alertLo) ]
          set StateUI(hi_limit,value) [format %.2f $::RLmeter::State(alertHi) ]
        }
        
        default { return }
     }
   
    return
  }

 
  #********************************************************************
  #
  #       Name: Update_peakhold
  #
  # Parameters: 
  #
  # Description: Conditions the 1/0 value from the engine
  #            : to a "ON"/"OFF" string for the user interface
  #
  #********************************************************************
  proc Update_peakhold { args } \
  {
     after cancel RssiUI::Update_peakhold_core
     after idle   RssiUI::Update_peakhold_core
  }
  
  proc Update_peakhold_core { } \
  {
    variable StateUI
 
    set status  $::RLmeter::State(peakEnable)
    
    if { $status == 1 } \
    {     
      set StateUI(peak_hold)  "ON"
    } \
    else \
    {
      set StateUI(peak_hold)  "OFF"
    }
    
    return
  }
 
 
  #********************************************************************
  #
  #       Name: update_text_color
  #
  # Parameters: 
  #
  # Description: change the color of the meter text
  #
  #********************************************************************
  proc update_text_color { tile_path args } \
  {
    set cmd "::RssiUI::update_text_color_core $tile_path"
    after cancel $cmd
    after idle $cmd
  }
  
  proc update_text_color_core { tile_path } \
  {
     variable StateUI
     
     if { ![winfo exists $tile_path.lbl_reading] } { return }
     
     $tile_path.lbl_reading config -fg $StateUI(reading,textcolor)
  }
  
 
  #********************************************************************
  #
  #       Name: set_meter_average
  #
  # Parameters: new_value   - requested new value 
  #                           -1 - no new value requested
  #
  # Description: set the meter average
  #
  #********************************************************************
  proc set_meter_average { { new_value "" } } \
  {
    #puts "set_meter_average:  $new_value"
    
    if { "$new_value" == "" } \
    {
      set new_value  $::RLmeter::State(average)
    } 
      
    # Set the meter engine
    after idle "RLmeter  average  $new_value  -nowait"
      
    return
  }


  #********************************************************************
  #
  #       Name: set_ref_mode
  #
  # Parameters: 
  #
  # Description: sets the Reference Mode of the meter
  #
  #********************************************************************
  proc set_ref_mode { { new_value "" } } \
  {
    #puts "set_ref_mode:  $new_value"
    
    variable  StateUI
    
    if { $new_value == "" } \
    {
      set ref_mode  $StateUI(ref_mode)
      
      if { $ref_mode == "ON" } \
      {
        set ref_mode  "OFF"
      } \
      else \
      {
        set ref_mode  "ON"
      }
    } \
    else \
    {
      set ref_mode  $new_value
    }       
    
    set StateUI(ref_mode)  $ref_mode
    

    if { $ref_mode == "ON" } \
    {
      set StateUI(ref_indicate)  "REF"
      
      # Set the ref_reading value if the present value is 0.0,
      # which is the initialization value
      if { $StateUI(ref_reading) == 0.0 } \
      {
        set StateUI(ref_reading)   $StateUI(reading)
      }
    }
  
    if { $ref_mode == "OFF" } \
    {
      set StateUI(ref_indicate)  ""
      set StateUI(ref_reading)   0.0
    }
  
    return
  }


#****************************************************************************
# Name:        getSaveState
# Parameters:  none
# Description: This function returns a list of TCL commands which can be used
#              to set the RssiUI tiles to its current state
#****************************************************************************
 proc getSaveState { } {
   
   variable StateUI
 
   set returnList \
   {
      set ::RssiUI::StateUI(range,auto_txt) {$StateUI(range,auto_txt)}
      set ::RssiUI::StateUI(range,auto)     {$StateUI(range,auto)}
      set ::RssiUI::StateUI(units)          {$StateUI(units)}
      set ::RssiUI::StateUI(cable_loss)     {$StateUI(cable_loss)}
   }
       
   return [subst -nocommands $returnList]
 }


#****************************************************************************
# Name:        Sync
# Parameters:  none
# Description: Write state variables to RssiUI
#****************************************************************************
 proc Sync { } {
   eval [getSaveState]
 }
  trace variable ::RLmeter::State(reading)     w "::RssiUI::Update_readings"
  trace variable ::RLmeter::State(status)      w "::RssiUI::Update_readings"
  trace variable ::RLmeter::State(peakEnable)  w "::RssiUI::Update_peakhold"
  trace variable ::RLmeter::State(alerts)      w "::RssiUI::UpdateAlertStrings"
  trace variable ::RLmeter::State(alertLo)     w "::RssiUI::Update_limits"
  trace variable ::RLmeter::State(alertHi)     w "::RssiUI::Update_limits"
  trace variable ::RLmeter::State(BellEnabled) w "::RssiUI::UpdateAlertStatus"
  trace variable ::RLmeter::State(alertStatus) w "::RssiUI::UpdateAlertStatus"
  trace variable ::RssiUI::StateUI(units)      w "::RssiUI::Update_units"
  trace variable ::RssiUI::StateUI(reading,max)  w  "::RssiUI::Update_limits"
  
  # sync our vars to the engine
  Update_readings
  Update_peakhold
  Update_limits
  Update_units


};  # End of RssiUI namespace


#****************************************************************************
# This registers the tiles with the list for saving instrument states
#****************************************************************************
namespace eval ::SavedStates {

   variable tiles   
   lappend tiles RssiUI
}

# add us to the list of tiles to pre-create
namespace eval View \
{
   lappend PreCreate RssiUI
}


