#############################################################################
#                                                                           #
# IFR SYSTEMS, INC        COPYRIGHT (C) 1998,99, 2001  DESIGN ENGINEERING   #
# ------------------------------------------------------------------------- #
#                                                                           #
# FILE        : 3100.MultimeterUI.tcl                                       #
#                                                                           #
# PRODUCT     : FITS                                                        #
#                                                                           #
# AUTHOR      : P. McCormick                                                #
#                                                                           #
# DESCRIPTION : Counter Meter User Interface                                #
#                                                                           #
#############################################################################

package require AFCounter
package require C4fm
package require AFCounter
package require FMDev
package require Sinad
package require Dvm
package require RLmeter
package require RFError
    


namespace eval MultimeterUI \
{
  # Global variables
  variable  LongName     "multimeter"
  variable  ShortName    "Meter Panel"
  variable  Size         Eighth_v
            
  variable  StateUI
    

  #********************************************************************
  #
  #       Name: Create  
  #
  # Parameters: size - meter size 
  #
  # Description: 
  #
  #********************************************************************
  proc Create { tile_path { meter_size  small } } \
  {
    variable LongName
    
    #puts "MultimeterUI::Create"

    set top_level [winfo toplevel $tile_path]
 
    # Call the function 'create_tile' to generate the meter tile
    # from the resource database
    set meter_class  MULTI_METER
    
    set tile_path  [name_child  $top_level $LongName]

    if { ![winfo exists $tile_path] } \
    {
      set tile_path  [::Screens::create_tile  $tile_path  $LongName  $meter_class]
    }  


    return $tile_path
  }


  #******************************************************************** 
  #
  #       Name: Map
  #
  # Parameters: 
  #
  # Description: Initializes the meter tile
  #
  #********************************************************************
  proc Map { tile_path } \
  {
     after idle "MultimeterUI::Map_AfterIdle $tile_path"
  }

  proc Map_AfterIdle { tile_path } \
  {
     after 1000 "MultimeterUI::Map_AfterDelay $tile_path"
  }

  proc Map_AfterDelay { tile_path } \
  {
    #puts "MultimeterUI::Map $tile_path"

    # Start the Counter Meter
    ::AFCounter::QuickStart

    # Start the C4FM Meter
    ::C4fm::QuickStart

    # Start the Deviation Meter
    ::FMDev::QuickStart

    # Start the Sinad and Distortion Meters
    Sinad en  [expr [Sinad en] | 3]  -nowait
    Sinad run 1  -nowait

    Sinad trig 2  -nowait

    # Start the DVM
    Dvm en  [expr [Dvm en] | 3]  -nowait
    Dvm run 1  -nowait

    # Start the Power Meter
    Power enable on -nowait
    Power prange AUTO -nowait
    
    # Start the RSSI Meter
    RLmeter run ON -nowait
    
    ::RFError::QuickStart  
   

    trace variable ::AFCounter::State(reading_raw)  w  \
                          "::MultimeterUI::Update_counter_display  $tile_path"
                
    trace variable ::C4fm::State(modacc)  w  \
                             "::MultimeterUI::Update_c4fm_display  $tile_path"

    trace variable ::FMDev::State(avg_value)  w  \
                              "::MultimeterUI::Update_dev_display  $tile_path"

    trace variable ::Sinad::State(distortion)  w  \
                             "::MultimeterUI::Update_dist_display  $tile_path"

    trace variable ::Sinad::State(sinad)  w  \
                            "::MultimeterUI::Update_sinad_display  $tile_path"

    trace variable ::Dvm::State(units)  w  \
                     "after idle ::MultimeterUI::Update_dvm_units  $tile_path"

    trace variable ::Dvm::State(reading)  w  \
                              "::MultimeterUI::Update_dvm_display  $tile_path"

    trace variable ::PowerUI::StateUI(reading)  w  \
                            "::MultimeterUI::Update_power_display  $tile_path"

    trace variable ::RLmeter::State(reading)  w  \
                             "::MultimeterUI::Update_rssi_display  $tile_path"

    trace variable ::RFError::State(reading_cond_Hz)  w  \
                            "::MultimeterUI::Update_rferr_display  $tile_path"


    Update_counter_display  $tile_path
    Update_c4fm_display     $tile_path
    Update_dev_display      $tile_path
    Update_dist_display     $tile_path
    Update_sinad_display    $tile_path
    Update_dvm_units        $tile_path
    Update_dvm_display      $tile_path
    Update_power_display    $tile_path
    Update_rssi_display     $tile_path
    Update_rferr_display    $tile_path


    return
  }


  #********************************************************************
  #
  #       Name: Cleanup
  #
  # Parameters: 
  #
  # Description: 
  #
  #********************************************************************
  proc Cleanup { tile_path } \
  {
    #puts "Cleanup:  $tile_path"

    trace vdelete ::AFCounter::State(reading_raw)  w  \
                          "::MultimeterUI::Update_counter_display  $tile_path"
                
    trace vdelete ::C4fm::State(reading)  w  \
                             "::MultimeterUI::Update_c4fm_display  $tile_path"

    trace vdelete ::FMDev::State(avg_value)  w  \
                              "::MultimeterUI::Update_dev_display  $tile_path"

    trace vdelete ::Sinad::State(distortion)  w  \
                             "::MultimeterUI::Update_dist_display  $tile_path"

    trace vdelete ::Sinad::State(sinad)  w  \
                            "::MultimeterUI::Update_sinad_display  $tile_path"

    trace vdelete ::Dvm::State(units)  w  \
                     "after idle ::MultimeterUI::Update_dvm_units  $tile_path"

    trace vdelete ::Dvm::State(reading)  w  \
                              "::MultimeterUI::Update_dvm_display  $tile_path"

    trace vdelete ::PowerUI::StateUI(reading)  w  \
                            "::MultimeterUI::Update_power_display  $tile_path"

    trace vdelete ::RLmeter::State(reading)  w  \
                             "::MultimeterUI::Update_rssi_display  $tile_path"

    trace vdelete ::RFError::State(reading_cond_Hz)  w  \
                            "::MultimeterUI::Update_rferr_display  $tile_path"

    after cancel "MultimeterUI::Map_AfterIdle $tile_path"
    after cancel "MultimeterUI::Map_AfterDelay $tile_path"
    
    return
  }
  
  
  #********************************************************************
  #
  #       Name: Unmap
  #
  # Parameters: 
  #
  # Description: 
  #
  #********************************************************************
  proc Unmap { tile_path } \
  {
    #puts "MultimeterUI::Unmap $tile_path"
    
    Cleanup  $tile_path

    return
  }


  #********************************************************************
  #
  #       Name: Destroy
  #
  # Parameters: 
  #
  # Description: 
  #
  #********************************************************************
  proc Destroy { tile_path } \
  {
    #puts "MultimeterUI::Destroy $tile_path"

    Cleanup  $tile_path

    return
  }


  #********************************************************************
  #
  #       Name: Register
  #
  # Parameters: 
  #
  # Description: 
  #
  #********************************************************************
  proc Register { } \
  {
    #puts "MultimeterUI::Register"

    return
  }
  #********************************************************************
  #
  #       Name:  Update_display_core
  #
  # Parameters: tile path, widget to update, namespace of meter
  #
  # Description:  Updates the color of the meter reading to
  #            :  reflect the status condition of the limits
  #
  #********************************************************************
  proc Update_display_core { tile_path  widget  Status } \
  {
    if ![winfo exists $tile_path.$widget] { return } 
    
    upvar #0 $Status status
     
    if {$status & 2} \
    {
       set color DimGray
       set bg DarkGrey
    } \
    else \
    {
       set color black
       set bg [$tile_path cget -bg]
    }
    if { $status & 0x200 } \
    { 
      set color red
    } \
    elseif { $status & 0x800 } \
    { 
      set color blue
    } \
    elseif { $status & 0x101D } \
    {
      set color yellow
    }
    
    $tile_path.$widget config -fg $color -bg $bg
    
    return 
  }


  #********************************************************************
  #
  #       Name: get_tile
  #
  # Parameters: a widget within the tile
  #
  #Description: get the name of the tile frame
  #
  #********************************************************************
  proc get_tile { widget } \
  {
    while { 1 } \
    {
      if { [catch { set class  [winfo class $widget] } result] } \
      {
        return ""
      }
         
      switch -- $class \
      {
        "MULTI_METER"    { return $widget }
      }

      set widget [winfo parent $widget]
    }
    
    return ""
  }


  #********************************************************************
  #
  #       Name:  Update_counter_display
  #
  # Parameters: 
  #
  # Description:  Updates the color of the meter reading to
  #            :  reflect the status condition of the limits
  #
  #********************************************************************
  proc Update_counter_display { tile_path  args } \
  {
    set cmd "::MultimeterUI::Update_display_core  \
                              $tile_path ent_count ::AFCounter::State(status)"
    
    after cancel $cmd
    after idle   $cmd
  }
  
  #********************************************************************
  #
  #       Name:  Update_c4fm_display
  #
  # Parameters: 
  #
  # Description:  Updates the color of the meter reading to
  #            :  reflect the status condition of the limits
  #
  #********************************************************************
  proc Update_c4fm_display { tile_path  args } \
  {
    set cmd "::MultimeterUI::Update_display_core  \
                                    $tile_path ent_c4fm ::C4fm::State(modacc,status)"
    
    after cancel $cmd
    after idle   $cmd
  }
  
  #********************************************************************
  #
  #       Name:  Update_dev_display
  #
  # Parameters: 
  #
  # Description:  Updates the color of the meter reading to
  #            :  reflect the status condition of the limits
  #
  #********************************************************************
  proc Update_dev_display { tile_path  args } \
  {
    set cmd "::MultimeterUI::Update_display_core  \
                                    $tile_path ent_dev ::FMDev::State(status)"
    
    after cancel $cmd
    after idle   $cmd
  }
  
  #********************************************************************
  #
  #       Name:  Update_dist_display
  #
  # Parameters: 
  #
  # Description:  Updates the color of the meter reading to
  #            :  reflect the status condition of the limits
  #
  #********************************************************************
  proc Update_dist_display { tile_path  args } \
  {
    set cmd "::MultimeterUI::Update_display_core  \
                         $tile_path ent_dist ::Sinad::State(Dist.alertStatus)"
    
    after cancel $cmd
    after idle   $cmd
  }
  
  #********************************************************************
  #
  #       Name:  Update_sinad_display
  #
  # Parameters: 
  #
  # Description:  Updates the color of the meter reading to
  #            :  reflect the status condition of the limits
  #
  #********************************************************************
  proc Update_sinad_display { tile_path  args } \
  {
    set cmd "::MultimeterUI::Update_display_core  \
                       $tile_path ent_sinad ::Sinad::State(Sinad.alertStatus)"
    
    after cancel $cmd
    after idle   $cmd
  }
  
  #********************************************************************
  #
  #       Name:  Update_dvm_units
  #
  # Parameters: 
  #
  # Description:  Updates the meter reading to the current units
  #
  #********************************************************************
  proc Update_dvm_units { tile_path  args } \
  {
    #puts "Update_dvm_units:  $tile_path,  $args"

    variable StateUI
    
    set tile_path  [get_tile  $tile_path]
    
    switch -- $::Dvm::State(units) \
    {
      "dBm"  { ${tile_path}.ent_dvm config -textvariable ::Dvm::State(reading,dBm) }

      "V"    { ${tile_path}.ent_dvm config -textvariable ::Dvm::State(reading,V) }
    }

    return
  }
  

  #********************************************************************
  #
  #       Name:  Update_dvm_display
  #
  # Parameters: 
  #
  # Description:  Updates the color of the meter reading to
  #            :  reflect the status condition of the limits
  #
  #********************************************************************
  proc Update_dvm_display { tile_path  args } \
  {
    set cmd "::MultimeterUI::Update_display_core  \
                                      $tile_path ent_dvm ::Dvm::State(status)"
    
    after cancel $cmd
    after idle   $cmd
  }
  
  #********************************************************************
  #
  #       Name:  Update_power_display
  #
  # Parameters: 
  #
  # Description:  Updates the color of the meter reading to
  #            :  reflect the status condition of the limits
  #
  #********************************************************************
  proc Update_power_display { tile_path  args } \
  {
    set cmd "::MultimeterUI::Update_display_core  \
                            $tile_path ent_power ::Power::State(alertStatus) "
    
    after cancel $cmd
    after idle   $cmd
  }
  
  #********************************************************************
  #
  #       Name:  Update_rssi_display
  #
  # Parameters: 
  #
  # Description:  Updates the color of the meter reading to
  #            :  reflect the status condition of the limits
  #
  #********************************************************************
  proc Update_rssi_display { tile_path  args } \
  {
    set cmd "::MultimeterUI::Update_display_core  \
                            $tile_path ent_rssi ::RLmeter::State(alertStatus)"
    
    after cancel $cmd
    after idle   $cmd
  }
  
  #********************************************************************
  #
  #       Name:  Update_rferr_display
  #
  # Parameters: 
  #
  # Description:  Updates the color of the meter reading to
  #            :  reflect the status condition of the limits
  #
  #********************************************************************
  proc Update_rferr_display { tile_path  args } \
  {
    set cmd "::MultimeterUI::Update_display_core  \
                                $tile_path ent_rferr ::RFError::State(status)"
    
    after cancel $cmd
    after idle   $cmd
  }
  

};  # End of MultimeterUI namespace



namespace eval View \
{
   lappend PreCreate MultimeterUI
}



