#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C)1998-2002      DESIGN ENGINEERING    #
# --------------------------------------------------------------------------#
# FILE        : Generator.tcl                                               #
# PRODUCT     : FITS                                                        #
# AUTHOR      : Hieu Tran                                                   #
# DESCRIPTION : RF Generator User Interface                                 #
#                                                                           #
#############################################################################

package require FGen
package require Generator

# this is cheesy, but - we need this var, but it may not exist when we set up
# if we don't do this...
namespace eval ReceiverUI \
{
  set StateUI(list_freq_index)  0
}

#############################################################################
# Generator tile
#############################################################################

namespace eval GeneratorUI \
{
   variable  LongName    gen
   variable  ShortName   "Generator (RX Test)"
   variable  Size        Quarter
   variable  Position    upper_right

   variable  tile_class  "GEN"

   variable  level           0
   variable  offset          0.000000
   variable  sweepon         0
   variable  sweeppause      0
   variable  key             7070
   variable  levelunit       dBm

   variable  fgen_mode

   # NOTE: The use of fgen_mode(3) and fgen_mode(4) have several problems.
   #   #1) The only reason that they exist is that the FGEN mode does not correspond
   #       to the settings used for M1 and M2, which means that the display cannot 
   #       be directly driven from the engine.
   #   #2) Since the display cannot be driven from the engine, keeping the engine
   #       and tile in sync is a problem. To partially solve this problem, the tile
   #       drives its settings to the engine when it is mapped
   #   #3) Because the engine has a simple interface, there is a need to preserve a 
   #       matrix of settings for each FGEN and mode. This is needed so that when the
   #       user changes the mode, the last settings for that mode are restored. The
   #       problem is capturing this information. When a new mode is SELECTED, the 
   #       old settings must be sent to the engine. When the settings are CHANGED for
   #       a mode, the settings must be captured with the mode. This capture / setting
   #       process causes RECALL problems.
   #   #4) The generator settings are saved in the setup. The FGEN also saves its modes
   #       If the generator tile is mapped during a RECALL, FGEN sets up the mode, and
   #       then the GeneratorUI tries to set it up. This is a problem of who really OWNS
   #       the mode for M1 or M2
   #   #5) Bacause the GeneratorUI mode settings for M1 and M2 have to change when RCI
   #       commands affect them, there are feedback problems during a RECALL. Depending
   #       on what the original settings were, and what the new M1/M2 modes are, 
   #       fgen_mode has been observed to toggle states several times during a RECALL,
   #       trigger a new setup of the FGEN engine mode. Logic was installed to minimize
   #       this feedback during a RECALL.
   # IT IS HIGHLY DESIRABLE TO FIND A WAY OF ELIMINATING THESE VARIABLES, ALLOWING FGEN
   # TO HAVE OWNERSHIP OF THE MODE, AND HAVE THE GENERATOR TILE REFLECT THE STATE OF THE
   # ENGINE (OR C++ INTERFACE OBJECT). If this can be done, kill all of the setup control
   # which this UI uses.
   set fgen_mode(3)  "OFF"
   set fgen_mode(4)  "OFF"

   variable  mic_state          "OFF"
   variable  aud_state          "OFF"
   variable  aud_state_display  "OFF"
   
   variable  StateUI

   set StateUI(chanmode)	"FREQ"
   set StateUI(voice_source)    "M1"

   set StateUI(sweepstart)      200.000000
   set StateUI(sweepstop)       500.000000
   set StateUI(sweeprate)       0.50
   set StateUI(sweepincr)       45.000000
   set StateUI(incrfreq)        45.000000
   
   set StateUI(mode,text) 	""
   
   set StateUI(pres_list_label)  ""
  
   set StateUI(sweep_menu)    { "FREQ"  "INCR"  "SWEEP"  "LIST" }
   
   set StateUI(units_menu)    { "dBm"  "mV"  "uV" }
   
   set StateUI(key_menu)      { "7070" }
   
   set StateUI(p25mode_menu)  { "1011"  "CALIBRATION"  "SILENCE"  "SPEECH"  "VOICE" \
	                        "STANDARD" }

   set StateUI(p25mode_submenu) { "STD 1011" "STD CAL" "STD SILENCE" "STD INTFRNC" \
	                          "STD BUSY" "STD IDLE" "STD LDU1TRG" "STD NOTRIG" \
	                          "STD LDU2TRG" "STD 511" "STD SYMRATE" "STD LOWDEV" \
	                          "STD FIDPAT" "STD FIDSPECT" }
   
   set StateUI(voice_source_menu)  { "OFF"  "M1"  "M2"  "MIC"  "AUDIO"  "DEMOD" }
   
  
   #*************************************************
   #        Name: Create
   #  Parameters: tile path of parent
   # Description: create Generator tile
   #*************************************************
   proc Create { parent } \
   {
     variable LongName
     variable tile_class

     set top_level [winfo toplevel $parent]

     set tile_path [name_child $top_level $LongName]

     if { ![winfo exists $tile_path] } \
     {
       set tile_path  [Screens::create_tile $tile_path $LongName $tile_class]
     }

     return $tile_path
   }
        

   #*************************************************
   #        Name: Map 
   #  Parameters: widget - tile path
   # Description: called when the screen is displayed
   #*************************************************
   proc Map { tile } \
   {
     Debug::Puts GenUI 2 "Map:  $tile"
     Debug::Puts GenUI 2 "Mapping Generator:  $tile"

     set tile [get_tile $tile]


     trace variable ::Generator::State(lockstat) w  \
                                     "::GeneratorUI::rfgen_lock_status  $tile"

    
     trace variable ::GeneratorUI::levelunit  w  \
                                       "::GeneratorUI::update_gen_unit  $tile"

     trace variable ::Generator::State(mod)  w  \
                                       "::GeneratorUI::update_mod_type  $tile"

     trace variable ::Generator::State(p25mode)  w  \
                              "::GeneratorUI::update_p25_pattern_frame  $tile"
      
     trace variable ::FGen::State(3,fgen_mode)  w  \
                                  "::GeneratorUI::update_fgen_frame  $tile  3"
     
     trace variable ::FGen::State(4,fgen_mode)  w  \
                                  "::GeneratorUI::update_fgen_frame  $tile  4"

     trace variable ::GeneratorUI::sweepon  w  \
                                      "::GeneratorUI::set_gen_softkeys  $tile"

     trace variable ::GeneratorUI::sweeppause  w  \
                                      "::GeneratorUI::set_gen_softkeys  $tile"

    
     trace variable ::BBAudio::State(demod_to_mod)  w  \
                               "::GeneratorUI::update_p25_voice_source  $tile"

     trace variable ::GeneratorUI::StateUI(chanmode)  w "::GeneratorUI::update_mode $tile"
     update_gen_unit_core    $tile 
 
     update_mod_type_core    $tile
     
     update_fgen_frame_core   $tile  3
     update_fgen_frame_core   $tile  4

     rfgen_lock_status_core  $tile

     set_gen_softkeys_core $tile

     set_chanmode_frame $tile
   }


   #*************************************************
   #        Name: Cleanup
   #  Parameters: none 
   # Description:  called when the screen is hidden
   #*************************************************
   proc Cleanup { widget } \
   {
     Debug::Puts GenUI 2 "GeneratorUI::Cleanup:  $widget"
     
     variable  sweepon
     
     set tile  [get_tile  $widget]
      

     trace vdelete ::Generator::State(lockstat) w  \
                                     "::GeneratorUI::rfgen_lock_status  $tile"

     trace vdelete ::GeneratorUI::levelunit  w  \
                                       "::GeneratorUI::update_gen_unit  $tile"

     trace vdelete ::Generator::State(mod)  w  \
                                       "::GeneratorUI::update_mod_type  $tile"

     trace vdelete ::Generator::State(p25mode)  w  \
                              "::GeneratorUI::update_p25_pattern_frame  $tile"
      
     trace vdelete ::FGen::State(3,fgen_mode)  w  \
                                  "::GeneratorUI::update_fgen_frame  $tile  3"
     
     trace vdelete ::FGen::State(4,fgen_mode)  w  \
                                  "::GeneratorUI::update_fgen_frame  $tile  4"

     trace vdelete ::GeneratorUI::sweepon  w  \
                                      "::GeneratorUI::set_gen_softkeys  $tile"

     trace vdelete ::GeneratorUI::sweeppause  w  \
                                      "::GeneratorUI::set_gen_softkeys  $tile"

     trace vdelete ::BBAudio::State(demod_to_mod)  w  \
                               "::GeneratorUI::update_p25_voice_source  $tile"

     trace vdelete ::GeneratorUI::StateUI(chanmode)  w "::GeneratorUI::update_mode $tile"
   }


   #*************************************************
   #        Name: Unmap 
   #  Parameters: none 
   # Description:  called when the screen is hidden
   #*************************************************
   proc Unmap { widget } \
   {
     Debug::Puts GenUI 2 "GeneratorUI::Unmap:  $widget"
     Debug::Puts GenUI 2 "UnMapping Generator:  $widget"
     
     Cleanup $widget
   }


   #*************************************************
   #        Name: Destroy
   #  Parameters: none
   # Description: called when the screen is destroyed
   #*************************************************
   proc Destroy { widget } \
   {
     Debug::Puts GenUI 2 "GeneratorUI::Destroy:  $widget"
     
     Cleanup $widget
   }


   #*************************************************
   #       Name: get_tile
   # Parameters: a widget within the tile
   #Description: get the name of the tile frame
   #*************************************************
   proc get_tile { widget } \
   {
     while { 1 } \
     {
       switch -- [winfo class $widget] \
       {
         "GEN" { return $widget } 
       }

       set widget [winfo parent $widget]
     }
   }


   #*************************************************
   #        Name: rfgen_lock_status
   #  Parameters: widget, args (ignored)
   # Description: setup after idle to update rfgen lock status
   #*************************************************
   proc rfgen_lock_status { widget  args } \
   {
     set cmd "GeneratorUI::rfgen_lock_status_core  $widget"

     after cancel $cmd
     after idle $cmd
   }

   proc rfgen_lock_status_core { widget } \
   {
     if ![winfo exists $widget] { return }
     set widget [get_tile  $widget]

     set widget  ${widget}.frm_gen.gen.ent_freq

     if { $::Generator::State(lockstat) } \
     {
       $widget configure -fg $::color(messagecaution)
     } \
     else \
     {
       $widget configure -fg $::color(menufg)
     }
   }


   #*************************************************
   #        Name: set_dup_offset
   #  Parameters: none
   # Description: set RF Gen. frequency offset in duplex mode
   #*************************************************
   proc set_dup_offset { } \
   {
     variable offset

     Generator freq [expr $::Receiver::State(freq) - $offset] MHz -nowait
   }


   #*************************************************
   #       Name: udpate_offset
   # Parameters: none
   #Description: update the offset field if Rcv or Gen
   #             freqs change
   #*************************************************
   proc update_offset { args } \
   {
     set cmd "::GeneratorUI::update_offset_core"

     after cancel $cmd
     after idle $cmd
   }
   
   proc update_offset_core { } \
   {
     variable offset
   
     catch \
     {
       set new_offset [expr $::Receiver::State(freq) - $::Generator::State(freq)]
       
       set new_offset [format %0.6f $new_offset]
   
       if { $offset != $new_offset } \
       {
         set offset $new_offset
       }
     }
   }
   

   #*************************************************
   #        Name: enter_gen_level
   #  Parameters: none
   # Description: sets level (dBm) once gen level is entered
   #*************************************************
   proc enter_gen_level { } \
   {
     Debug::Puts GenUI 2 "enter_gen_level"
     
     variable level
     variable levelunit

     set rf_level  0
     
     switch -- $levelunit \
     {
       "dBm"  { set rf_level  $level }
       
       "mV"   { set rf_level  [convert_V_dBm  [expr $level / 1000.0]]    }
              
       "uV"   { set rf_level  [convert_V_dBm  [expr $level / 1000000.0]] }
     }

     Generator rflevel  $rf_level  -nowait
   }


   #*************************************************
   #       Name: update_rflevel
   # Parameters: none
   #Description: update the RF level if it changes
   #*************************************************
   proc update_rflevel { args } \
   {
     set cmd "::GeneratorUI::update_rflevel_core"

     after cancel $cmd
     after idle $cmd
   }
   
   proc update_rflevel_core { } \
   {
     Debug::Puts GenUI 2 "update_rflevel_core"
     
     variable level
     variable levelunit

     switch -- $levelunit \
     {
       "dBm"  { set level  $::Generator::State(rflev,dBm) }

       "mV"   { set level  $::Generator::State(rflev,mV)  }
             
       "uV"   { set level  $::Generator::State(rflev,uV)  }
     }
   }
   
   #*************************************************
   #       Name: set_gen_rf
   # Parameters: none
   #Description: set the mode of the generator
   #*************************************************
   proc set_gen_rf { } \
   {
     variable StateUI
     
     after idle "Generator dcpower 1 -nowait"
     switch -- $StateUI(mode,text) \
     {
        "XXX"  -
        "ON"   {after idle "Generator ptt ON  -nowait"}
        "OFF"  {after idle "Generator ptt OFF -nowait"}
        "PTT"  {after idle "Generator ptt PTT -nowait"}
     }
   }


   #*************************************************
   #       Name: update_rf
   # Parameters: none
   #Description: watch the state of the generator
   #             PTT and dcpower and update the screen
   #*************************************************
   proc update_rf { args } \
   {
     set cmd "GeneratorUI::update_rf_core"

     after cancel $cmd
     after idle $cmd
   }

   proc update_rf_core { } \
   {
     variable StateUI

     if { $::Generator::State(dcpower) == "OFF" } \
     {
       set StateUI(mode,text)  "XXX"
       return
     }
     set StateUI(mode,text) $::Generator::State(ptt)
   }
   

   #*************************************************
   #        Name: update_gen_unit
   #  Parameters: widget - tile path 
   # Description: updates gen units, converts gen output level
   #*************************************************
   proc update_gen_unit { widget  args } \
   {
     set cmd "GeneratorUI::update_gen_unit_core $widget"

     after cancel $cmd
     after idle $cmd
   }
   
   proc update_gen_unit_core { widget } \
   {
   
     variable level
     variable levelunit
     variable StateUI
     
     set StateUI(levelunit,shadow) $levelunit

     set widget [get_tile $widget]
      
     set level_widget  ${widget}.frm_gen.lvl.ent_rflevel
      
     switch -- $levelunit \
     {
       "dBm"  { if { $::Generator::State(rfout) == "GEN" } \
                {
                  bind_numeric_spin $level_widget 3 1 10 -110 ::GeneratorUI::enter_gen_level
                } \
                else \
                {
                  bind_numeric_spin $level_widget 3 1 -30 -137 ::GeneratorUI::enter_gen_level
                }
              }

       "mV"   { if { $::Generator::State(rfout) == "GEN" } \
                {
                  bind_numeric_spin $level_widget 3 4 707.1068 .0007 ::GeneratorUI::enter_gen_level
                } \
                else \
                {
                  bind_numeric_spin $level_widget 3 4 7.0711 0 ::GeneratorUI::enter_gen_level
                }
              }
              
       "uV"   { if { $::Generator::State(rfout) == "GEN" } \
                {
                  bind_numeric_spin $level_widget 6 2 707106.81 .0007 ::GeneratorUI::enter_gen_level
                } \
                else \
                {
                  bind_numeric_spin $level_widget 6 2 7071.10 0 ::GeneratorUI::enter_gen_level
                }
              }
     }

     update_rflevel
   }


   #*************************************************
   #        Name: toggle_gen_out
   #  Parameters: widget - tile path
   # Description: sets generator output port & adjusts level 
   #              limit depending on T/R or GEN port
   #*************************************************
   proc toggle_gen_out { widget } \
   {
     variable level
     variable levelunit

     set widget  [get_tile $widget]

     set rflevel_tile  ${widget}.frm_gen.lvl.ent_rflevel

     if { $::Generator::State(rfout) == "T/R" } \
     {
       # SWITCHING FROM T/R TO GEN OUT
       after idle "Generator rfout GEN"

       switch -- $levelunit \
       {
         "dBm"  { bind_numeric_spin $rflevel_tile 3 1 10 -110 ::GeneratorUI::enter_gen_level         }
                
         "mV"   { bind_numeric_spin $rflevel_tile 3 4 707.1068 .0007 ::GeneratorUI::enter_gen_level  }
                
         "uV"   { bind_numeric_spin $rflevel_tile 6 2 707106.81 .0007 ::GeneratorUI::enter_gen_level }
       }
     } \
     else \
     {
       # SWITCHING FROM GEN TO T/R OUT
       after idle "Generator rfout T/R"

       switch -- $levelunit \
       {
         "dBm"  { bind_numeric_spin $rflevel_tile 3 1 -30 -137 ::GeneratorUI::enter_gen_level  }
                
         "mV"   { bind_numeric_spin $rflevel_tile 3 4 7.0711 0 ::GeneratorUI::enter_gen_level  }
                
         "uV"   { bind_numeric_spin $rflevel_tile 6 2 7071.10 0 ::GeneratorUI::enter_gen_level }
       }
     }
   }


   #*************************************************
   #        Name: set_chanmode_frame
   #  Parameters: widget - tile path
   # Description: display channel mode frame 
   #*************************************************
   proc update_mode { widget args} \
   {
      set cmd [namespace code "set_chanmode_frame $widget"]
      after cancel $cmd
      after idle $cmd
   }
   proc set_chanmode_frame { widget } \
   {   
     Debug::Puts GenUI 2 "set_chanmode_frame:  $widget"
     
     variable  StateUI

     set tile  [get_tile $widget]
     
     Debug::Puts GenUI 2 "tile:  $tile"
     
     foreach subtile {incr sweep list offset} \
     {
        if [winfo exists  ${tile}.frm_gen.$subtile] \
        {
           grid remove  ${tile}.frm_gen.$subtile
        }
     }

     # Allow the display of the Audio Route button on all subframes.
     # Verify that each exists prior to positioning
     foreach subtile {mod_off mod_p25 mod_fm_am} \
     {
        if [winfo exists  ${tile}.frm_gen.$subtile.aud_route] \
        { 
          Screens::position_widget  ${tile}.frm_gen.$subtile.aud_route
        }
     }

     # TURN OFF FREQ SWEEP
     gen_freq_sweep  "OFF"

     switch -- $StateUI(chanmode) \
     {
       "FREQ"  {
                  Screens::position_widget  ${tile}.frm_gen.offset 
               }

       "INCR"  {
                 Screens::position_widget  ${tile}.frm_gen.incr }
                 
       "SWEEP" {
                 Screens::position_widget  ${tile}.frm_gen.sweep }
                 
       "LIST"  { 
                 Screens::position_widget  ${tile}.frm_gen.list
               }
     }

     set_gen_softkeys_core $tile
   }

   #*************************************************
   #        Name: setup_gen_freqsweep
   #  Parameters: tile - tile path
   # Description: displays toplevel for editing freq sweep values
   #*************************************************
   proc setup_gen_freqsweep { widget { name sweep } } \
   {
     if ![winfo exists $widget] { set widget . }
     
     set parent [winfo toplevel $widget]
     
     set toplevel [name_child $parent $name]
    
     if {[winfo exists $toplevel]} \
     {
       wm deiconify $toplevel
       
       safe_focus_force ${toplevel}.entry_start_freq
     } \
     else \
     {
       Screens::create_toplevel_tile  $toplevel  GENSWEEP
        
       safe_focus_force ${toplevel}.entry_start_freq
        
       bind $toplevel <KeyRelease-F8> "::GeneratorUI::gensweep_return $toplevel"
     }
   }


   #*************************************************
   #        Name: gensweep_return 
   #  Parameters: widget - tile path 
   # Description: close generator sweep toplevel 
   #*************************************************
   proc gensweep_return { widget } \
   {  
     set tile [winfo toplevel $widget]

     wm withdraw $tile
   }


   #*************************************************
   #        Name: gen_freq_sweep
   #  Parameters: status - OFF or TOGGLE 
   # Description: starts or stops sweep of frequencies
   #*************************************************
   proc gen_freq_sweep { status } \
   {
     variable  StateUI
     variable  sweepon
     variable  sweeppause

     switch -- $status \
     {
       "TOGGLE" { if { $sweepon } \
                  {
                    set sweepon  0
                  } \
                  else \
                  {
                    set sweepon  1
                  }
                }
                
       "OFF"    { set sweepon  0 }
       
       "ON"     { set sweepon  1 }
     }

     if { $sweepon } \
     {
       # Stop the sweep
       set sweeppause  0

       # Reset the frequency to the start freq
       set ::Generator::State(freq) [format_numeric $StateUI(sweepstart) 6]

       after idle "Generator freq $::Generator::State(freq) MHz -nowait"

       set delay  [expr int($StateUI(sweeprate)*1000)]
       if {$delay < 100} { set delay 100 }
       
       after $delay "::GeneratorUI::incr_gen_sweep"
     } \
     else \
     {
       # Stop the sweep
       set sweeppause  1

       after cancel "::GeneratorUI::incr_gen_sweep"
     }
   }


   #*************************************************
   #        Name: gen_sweep_pause
   #  Parameters: 
   # Description: toggles sweep pause state
   #*************************************************
   proc gen_sweep_pause { } \
   {
     variable sweepon
     variable sweeppause

     if { $sweepon } \
     {
       if { $sweeppause } \
       {
         set sweeppause  0

         after idle "::GeneratorUI::incr_gen_sweep"
       } \
       else \
       {
         set sweeppause  1
       
         after cancel "::GeneratorUI::incr_gen_sweep"
       }
     }
   }


   #*************************************************
   #        Name: incr_gen_sweep
   #  Parameters: none
   # Description: updates sweep freq
   #*************************************************
   proc incr_gen_sweep { } \
   {
     variable StateUI
     variable sweepon
     variable sweeppause

     if { $sweeppause == 0 } \
     {
       # Exit if either the Receiver or Generator becomes unlocked.
       if { $::Receiver::State(lockstat,raw) != 0 } \
       {
         puts "Receiver unlock, stop scan mode"
         
         # Turn off the freq sweep
         gen_freq_sweep  "OFF"
       }

       if { $::Generator::State(lockstat,raw) != 0 } \
       {
         puts "Generator unlock, stop scan mode"

         # Turn off the freq sweep
         gen_freq_sweep  "OFF"
       }

       if { $::Generator::State(freq) >= $StateUI(sweepstop) } \
       {
         set ::Generator::State(freq) [format_numeric $StateUI(sweepstart) 6]
       } \
	   else \
	   {
         set ::Generator::State(freq) [format_numeric [limit_value \
                                      [expr $::Generator::State(freq) + \
                                            $StateUI(sweepincr)] \
                                            $StateUI(sweepstop) 0] 6]
       }

       after idle "Generator freq $::Generator::State(freq) MHz -nowait"
       set delay [expr int($StateUI(sweeprate)*1000)]
       if {$delay < 100} { set delay 100 } 
       after $delay "::GeneratorUI::incr_gen_sweep"
     }
   }


   #*************************************************
   #        Name: incr_gen_freq
   #  Parameters: PLUS or MINUS
   # Description: increments gen freq in response to + or - key
   #*************************************************
   proc incr_gen_freq { direction } \
   {
     variable StateUI 

     if { $direction == "PLUS" } \
     {
       set add [expr $::Generator::State(freq) + $StateUI(incrfreq)]
   
     } \
     else \
     {
       set add [expr $::Generator::State(freq) - $StateUI(incrfreq)]

     }

     after idle "Generator freq $add MHz -nowait"
   }


   #**************************************************************************
   #
   # Name: display_freq_list
   #
   # Parameters: widget - invoking widget tile path
   #
   # Description: Displays toplevel for editing freq list values
   #
   #**************************************************************************
   proc display_freq_list { widget } \
   {
     Debug::Puts GenUI 2 "display_freq_list"
     
     ::Freq_ListUI::display_freq_list  $widget
     
     get_list_freq
   }
  
  
   #**************************************************************************
   #
   # Name: get_list_freq
   #
   # Parameters:
   #
   # Description: Displays the values of the freq list for the specified index
   #
   #**************************************************************************
   proc get_list_freq { {no_crosslink 0 }} \
   {
     Debug::Puts GenUI 2 "get_list_freq"
     
     variable  StateUI
     
     if {$StateUI(chanmode) != "LIST"} { return }
     
     set list_freq_index  $::ReceiverUI::StateUI(list_freq_index)
     
     # Return if the array value does not exist
     if { ![info exists ::Freq_ListUI::ListFreqArray(index,$list_freq_index)] } \
     {
       return
     }
     
     # Get the frequency list values
     set tx_freq      $::Freq_ListUI::ListFreqArray(tx_freq,$list_freq_index)
     set label        $::Freq_ListUI::ListFreqArray(label,$list_freq_index)
  
     # Display the label
     set StateUI(pres_list_label)  $label
     
     # Set the Generator frequency
     Generator  freq  $tx_freq -nowait
     
     # if we HAVEN'T been told not to, ask the receiver if he wants to update
     if {$no_crosslink == 0} \
     {
        ::ReceiverUI::get_list_freq 1
     }
   }


   #*************************************************
   #        Name: set_gen_softkeys
   #  Parameters: widget - tile path
   # Description: sets generator softkeys
   #*************************************************
   proc set_gen_softkeys { widget  args } \
   {
       set cmd "GeneratorUI::set_gen_softkeys_core $widget"
       after cancel $cmd
       after idle $cmd
   }
   proc set_gen_softkeys_core { widget } \
   {
     Debug::Puts GenUI 2 "set_gen_softkeys:  $widget"
     set_softkey_2 $widget
     set_softkey_3 $widget
     set_softkey_4 $widget
     set_softkey_5 $widget
     set_softkey_6 $widget
     set_softkey_7 $widget
   }


   #*************************************************
   #        Name: set_softkey_2
   #  Parameters: widget - tile path
   # Description: set generator softkey two
   #*************************************************
   proc set_softkey_2 { widget } \
   {
     Debug::Puts GenUI 2 "set_softkey_2:  $widget"

     set tile   ${widget}.frm_gen.gen
     
     set entry  ${tile}.ent_freq

     # SOFTKEY 2 (RF FREQ)
     View::AssignSoftkey $widget 2 "RF FREQ" \
       "safe_focus_force $entry; after idle \"event generate $entry <Return>\""
   }


   #*************************************************
   #        Name: set_softkey_3
   #  Parameters: widget - tile path
   # Description: set generator softkey three
   #*************************************************
   proc set_softkey_3 { widget } \
   {
     Debug::Puts GenUI 2 "set_softkey_3:  $widget"

     set tile   ${widget}.frm_gen
     
     set entry  ${tile}.lvl.ent_rflevel

     # SOFTKEY 3 (GEN FREQ)
     View::AssignSoftkey $widget 3 "RF LEVEL" \
       "safe_focus_force $entry; after idle \"event generate $entry <Return>\""
   }


   #*************************************************
   #        Name: set_softkey_4
   #  Parameters: widget - tile path
   # Description: set generator softkey four
   #*************************************************
   proc set_softkey_4 { widget } \
   {
     Debug::Puts GenUI 2 "set_softkey_4:  $widget"
  
     set tile ${widget}.frm_gen
     set entry ${tile}.mod.mnu_mod_type

     View::AssignSoftkey $widget 4 "MOD TYPE" \
       "safe_focus_force $entry; after idle \"tkMbPost $entry\""
   }


   #*************************************************
   #        Name: set_softkey_5
   #  Parameters: widget - tile path
   # Description: set generator softkey five
   #*************************************************
   proc set_softkey_5 { widget } \
   {
     Debug::Puts GenUI 2 "set_softkey_5:  $widget"
   
     set tile  ${widget}.frm_gen
     
   
     # SOFTKEY 5 (P25 SIGNAL TYPE/ FM M1 ON-OFF)
     if {$::Generator::State(is_P25_mode)} \
     {
        View::AssignSoftkey $widget 5 "P25 PATTERN" \
                             "post_menu ${tile}.mod_p25.mnu_pattern"
     } \
     elseif {$::Generator::State(mod_raw) == 0} \
     {
        View::ClearSoftkey $widget 5 
     } \
     else \
     {
        set_tone_softkey $widget 3 
     }
   }
   

   #*************************************************
   #        Name: set_softkey_6
   #  Parameters: widget  - tile path
   # Description: set generator softkey six
   #*************************************************
   proc set_softkey_6 { widget } \
   {
     Debug::Puts GenUI 2 "set_softkey_6:  $widget"
   
     set tile ${widget}.frm_gen
   
     # SOFTKEY 6 (P25 VOICE SOURCE, FM M1 DEV)
     if { ![set_chan_softkeys  $widget] } \
     {
       if {$::Generator::State(is_P25_mode)} \
       {
           if { $::Generator::State(p25mode_raw) == 1 } \
           {
              View::AssignSoftkey $widget 6 "P25 VOICE SOURCE" \
                                 "post_menu ${tile}.mod_p25.mnu_source"
           } \
           else \
           {
              View::ClearSoftkey $widget 6
           }
       } \
       elseif {$::Generator::State(mod_raw) == 0} \
       {
          View::ClearSoftkey $widget 6 
       } \
       else \
       {
          set_tone_softkey $widget 4
       }
     }
   }


   #*************************************************
   #        Name: set_softkey_7
   #  Parameters: widget  - tile path
   # Description: set generator softkey seven 
   #*************************************************
   proc set_softkey_7 { widget } \
   {
     Debug::Puts GenUI 2 "set_softkey_7:  $widget"

     variable  StateUI 

     set tile    ${widget}.frm_gen
     
     set entry1  ${tile}.mod_p25.voice_m1.ent_lev1
     set entry2  ${tile}.mod_p25.voice_m2.ent_lev2


     # SOFTKEY 7 (P25 VOICE M1-M2 DEV, FM M1 AF)
     if { ![set_chan_softkeys  $widget] } \
     {
       if {$::Generator::State(is_P25_mode)} \
       {
         if { $::Generator::State(p25mode_raw) == 1 } \
         {
           switch -- $StateUI(voice_source) \
           {
             "M1"    { View::AssignSoftkey $widget 7 "P25 M1 DEV" \
                             "unpost_last_menu; focus $entry1; update; \
                                             event generate $entry1 <Return>"
                       return
                     }

             "M2"    { View::AssignSoftkey $widget 7 "P25 M2 DEV" \
                            "unpost_last_menu; focus $entry2; update; \
                                             event generate $entry2 <Return>"
                       return
                     }
           }
         }
       }
     }        
   }


   #*************************************************
   #        Name: set_tone_softkey
   #  Parameters: tile - tile path 
   #              lun  - 3 is M1, 4 is M2
   # Description: set generator tone signal softkey
   #*************************************************
   proc set_tone_softkey { tile  lun } \
   {
     Debug::Puts GenUI 2 "set_tone_softkey:  $tile,  $lun"
   
     variable StateUI
   
     if { $lun == "3" } \
     {
       set num  1
       set id  5
     } \
     else \
     {
       set num  2
       set id  6
     }
   
     
     # Clear the softkey
     View::AssignSoftkey  $tile  $id  ""  ""
   
     switch -- $::FGen::State($lun,fgen_mode) \
     {
       "OFF"   { View::AssignSoftkey  $tile  $id  "M$num OFF"    "FGen::setMode  $lun  1"  }
   
       "TONE"  { View::AssignSoftkey  $tile  $id  "M$num TONE"   "FGen::setMode  $lun  3"  }
 
       "TREM"  { View::AssignSoftkey  $tile  $id  "M$num TREM"   "FGen::setMode  $lun  4"  }
   
       "DTMF"  { View::AssignSoftkey  $tile  $id  "M$num DTMF"   "FGen::setMode  $lun  5"  }
   
       "CTCSS" { View::AssignSoftkey  $tile  $id  "M$num CTCSS"  "FGen::setMode  $lun  6"  }
    
       "DCS"   { View::AssignSoftkey  $tile  $id  "M$num DCS"    "FGen::setMode  $lun  7"  }
   
       "TSIG"  { View::AssignSoftkey  $tile  $id  "M$num TSIG"   "FGen::setMode  $lun  9"  }

       "SINC"  { View::AssignSoftkey  $tile  $id  "M$num SINC"    "FGen::setMode  $lun  10" }

       "ARB"   { View::AssignSoftkey  $tile  $id  "M$num ARB"     "FGen::setMode  $lun  0"  }

       "SWEEP" { View::AssignSoftkey  $tile  $id  "M$num SWEEP"   "FGen::setMode  $lun  0"  }
     }
   }


   #*************************************************
   #        Name: set_chan_softkeys
   #  Parameters: widget - tile path
   # Description: sets channel mode softkeys
   #*************************************************
   proc set_chan_softkeys { widget } \
   {
     Debug::Puts GenUI 2 "set_chan_softkeys:  $widget"

     variable  StateUI
     variable  sweeppause
     variable  sweepon

     set tile  [get_tile  $widget]
     

     # SOFTKEY 6 (INCR UP, START SWEEP)
     switch -- $StateUI(chanmode) \
     {
       "INCR"   { View::AssignSoftkey $widget 6  "INCR UP"  \
                                         "::GeneratorUI::incr_gen_freq  PLUS"
       
                  View::AssignSoftkey $widget 7  "INCR DOWN"  \
                                        "::GeneratorUI::incr_gen_freq  MINUS"
        
                  return 1
                }
              
       "SWEEP"  { if { $sweepon } \
                  {
                    View::AssignSoftkey $widget 6  "STOP SWEEP"  \
                                      "::GeneratorUI::gen_freq_sweep  TOGGLE"
                  
                    if { $sweeppause } \
                    {
                      View::AssignSoftkey $widget 7  "CONT SWEEP"  \
                                             "::GeneratorUI::gen_sweep_pause"
                    } \
                    else \
                    {
                      View::AssignSoftkey $widget 7  "PAUSE SWEEP"  \
                                             "::GeneratorUI::gen_sweep_pause"
                    }
                  } \
                  else \
                  {
                    View::AssignSoftkey $widget 6  "START SWEEP"  \
                                      "::GeneratorUI::gen_freq_sweep  TOGGLE"
                   
                    View::ClearSoftkey $widget 7
                  }
                  
                  return 1
                }
                
       "LIST"   { View::AssignSoftkey $widget 6  "DISPLAY LIST"  \
                                        "${tile}.frm_gen.list.btn_list invoke"
       
                  View::ClearSoftkey $widget 7
    
                  return 1
                }
        default {
                  View::ClearSoftkey $widget 6
                  View::ClearSoftkey $widget 7
                }
     }

     return 0
   }


   #*************************************************
   #        Name: gen_post_menu
   #  Parameters: widget - tile path
   # Description: post frequency menu button 
   #*************************************************
   proc gen_post_menu { widget } \
   {
     set tile [winfo toplevel $widget]
     
     set tile [name_child $tile gen.frm_gen.gen.mnu_freq]
      
     post_menu $tile
   }


   #*************************************************
   #        Name: set_mod_type
   #  Parameters: none
   # Description: set to modulation type (AM, FM, P25, P25 Encr, or OFF)
   #*************************************************
   proc set_mod_type { } \
   {
     Debug::Puts GenUI 2 "set_mod_type"
     foreach format [array names ::Generator::modtype_map] \
     {
        if {$::Generator::modtype_map($format) == $::Generator::State(mod)} \
        {
            after idle "Generator mod $format"
            return
        }
     }
   }


   #*************************************************
   #        Name: update_mod_type
   #  Parameters: none
   # Description: updates to modulation type (AM, FM, P25 or OFF)
   #*************************************************
   proc update_mod_type { widget  args } \
   {
      variable StateUI

      update_fgen_mode
      set cmd "GeneratorUI::update_mod_type_core $widget"
      after cancel $cmd
      after idle $cmd
   }
   proc update_mod_type_core { widget } \
   {
     variable StateUI
     set StateUI(mod,shadow) $::Generator::State(mod)
     Debug::Puts GenUI 2 "update_mod_type_core:  $widget"
     
     variable  fgen_mode

     Debug::Puts GenUI 2 "::Generator::State(mod):  $::Generator::State(mod)"

     set tile  [get_tile  $widget]


     # Remove all applicable frames
     grid remove ${tile}.frm_gen.mod_off
     grid remove ${tile}.frm_gen.mod_p25
     grid remove ${tile}.frm_gen.mod_fm_am


     if { $::Generator::State(is_P25_mode) } \
     {
        Screens::position_widget ${tile}.frm_gen.mod_p25
       
        set ::FGen::State(dev_units)  "%"
        
        update_p25_pattern_frame  $tile
     } \
     else \
     {
      switch -- $::Generator::State(mod_raw) \
      {
        0     { Screens::position_widget ${tile}.frm_gen.mod_off }
        
        1     { # FM
                Screens::position_widget ${tile}.frm_gen.mod_fm_am
                 
                set ::FGen::State(dev_units)  "kHz"
                
                # Update the Modulator entry widgets
                set frame_name  ${tile}.frm_gen.mod_fm_am

                if { [winfo exists ${frame_name}.m1.tone.dev_entry] } \
                {
                  ${frame_name}.m1.tone.dev_entry  config  \
                                      -textvariable  ::FGen::State(3,TONE,dev)
                }
                
                if { [winfo exists ${frame_name}.m2.tone.dev_entry] } \
                {
                  ${frame_name}.m2.tone.dev_entry  config  \
                                      -textvariable  ::FGen::State(4,TONE,dev)
                }

                if { [winfo exists ${frame_name}.m1.sinc.ent_dev] } \
                {
                  ${frame_name}.m1.sinc.ent_dev  config  \
                                      -textvariable  ::FGen::State(3,SINC,dev)
                }

                if { [winfo exists ${frame_name}.m2.sinc.ent_dev] } \
                {
                  ${frame_name}.m2.sinc.ent_dev  config  \
                                      -textvariable  ::FGen::State(4,SINC,dev)
                }

                if { [winfo exists ${frame_name}.m1.arb.ent_dev] } \
                {
                  ${frame_name}.m1.arb.ent_dev  config  \
                                       -textvariable  ::FGen::State(3,ARB,dev)
                }

                if { [winfo exists ${frame_name}.m2.arb.ent_dev] } \
                {
                  ${frame_name}.m2.arb.ent_dev  config  \
                                       -textvariable  ::FGen::State(4,ARB,dev)
                }

                if { [winfo exists ${frame_name}.m1.sweep.ent_dev] } \
                {
                  ${frame_name}.m1.sweep.ent_dev  config  \
                                     -textvariable  ::FGen::State(3,SWEEP,dev)
                }

                if { [winfo exists ${frame_name}.m2.sweep.ent_dev] } \
                {
                  ${frame_name}.m2.sweep.ent_dev  config  \
                                     -textvariable  ::FGen::State(4,SWEEP,dev)
                }

                # Setup the Function Generator routing
                BBAudio mixer FGEN3  L1  ON -nowait
                BBAudio mixer FGEN4  L1  ON -nowait
                
                update_mic_state
                update_aud_state }
                
       4      { # AM
                Screens::position_widget ${tile}.frm_gen.mod_fm_am
                
                set ::FGen::State(dev_units)  "%"
                
                # Update the Modulator entry widgets
                # Update the Modulator entry widgets
                set frame_name  ${tile}.frm_gen.mod_fm_am

                if { [winfo exists ${frame_name}.m1.tone.dev_entry] } \
                {
                  ${frame_name}.m1.tone.dev_entry  config  \
                                  -textvariable  ::FGen::State(3,TONE,percent)
                }
                
                if { [winfo exists ${frame_name}.m2.tone.dev_entry] } \
                {
                  ${frame_name}.m2.tone.dev_entry  config  \
                                  -textvariable  ::FGen::State(4,TONE,percent)
                }

                if { [winfo exists ${frame_name}.m1.sinc.ent_dev] } \
                {
                  ${frame_name}.m1.sinc.ent_dev  config  \
                                  -textvariable  ::FGen::State(3,SINC,percent)
                }

                if { [winfo exists ${frame_name}.m2.sinc.ent_dev] } \
                {
                  ${frame_name}.m2.sinc.ent_dev  config  \
                                  -textvariable  ::FGen::State(4,SINC,percent)
                }

                if { [winfo exists ${frame_name}.m1.arb.ent_dev] } \
                {
                  ${frame_name}.m1.arb.ent_dev  config  \
                                   -textvariable  ::FGen::State(3,ARB,percent)
                }

                if { [winfo exists ${frame_name}.m2.arb.ent_dev] } \
                {
                  ${frame_name}.m2.arb.ent_dev  config  \
                                   -textvariable  ::FGen::State(4,ARB,percent)
                }

                if { [winfo exists ${frame_name}.m1.sweep.ent_dev] } \
                {
                  ${frame_name}.m1.sweep.ent_dev  config  \
                                 -textvariable  ::FGen::State(3,SWEEP,percent)
                }

                if { [winfo exists ${frame_name}.m2.sweep.ent_dev] } \
                {
                  ${frame_name}.m2.sweep.ent_dev  config  \
                                   -textvariable  ::FGen::State(4,SWEEP,percent)
                }

                # Setup the Function Generator routing
                BBAudio mixer FGEN3  L1  ON -nowait
                BBAudio mixer FGEN4  L1  ON -nowait

                update_mic_state
                update_aud_state }
      }
     }

     set_gen_softkeys  $tile

     set_softkey_5  $tile
     set_softkey_6  $tile
     set_softkey_7  $tile
   }

   #*************************************************************************
   #  Name:       update_fgen_mode
   #  Parameters: none
   #  Description: This function updates the FGEN modes when the generator 
   #      mode changes
   #*************************************************************************
   proc update_fgen_mode { } \
   {
      variable  fgen_mode

      if { $::Generator::State(is_P25_mode) } \
      {
          FGen::setMode  3  "P25"
          FGen::setMode  4  "P25"
      } \
      else \
      {
          switch -- $::Generator::State(mod_raw) \
          {
              1   - 
              4   { # 1 = FM, 4 = AM
                    # Setup the Function Generator routing
                    BBAudio mixer FGEN3  L1  ON -nowait
                    BBAudio mixer FGEN4  L1  ON -nowait
                
                    FGen::setMode  3  "$fgen_mode(3)"
                    FGen::setMode  4  "$fgen_mode(4)"
                  }

              default { }
          }
      }
   }



   #*************************************************
   #        Name: set_p25_pattern
   #  Parameters: none 
   # Description: set P25 mode
   #*************************************************
   proc set_p25_pattern { } \
   {
     Debug::Puts GenUI 2 "set_p25_pattern"

     # Determine if in P25 mode
     switch -- $::Generator::State(p25mode) \
     {
       "OFF"          { after idle "Generator p25mode OFF          -nowait" }

       "1011"         { after idle "Generator p25mode HZ1011       -nowait" }

       "CALIBRATION"  { after idle "Generator p25mode ERR5PERCENT  -nowait" }

       "SILENCE"      { after idle "Generator p25mode SILENCE      -nowait" }

       "VOICE"        { after idle "Generator p25mode SPEECH       -nowait" }    

       "SPEECH"       { after idle "Generator p25mode FILE         -nowait" }

       "TRUNKING"     { after idle "Generator p25mode TRUNKING     -nowait" }

       "STD 1011"     { after idle "Generator p25mode STDTONE      -nowait" }

       "STD SILENCE"  { after idle "Generator p25mode STDSILENCE   -nowait" }

       "STD INTFRNC"  { after idle "Generator p25mode INTERFERENCE -nowait" }

       "STD BUSY"     { after idle "Generator p25mode BUSY         -nowait" }

       "STD IDLE"     { after idle "Generator p25mode IDLE         -nowait" }

       "STD CAL"      { after idle "Generator p25mode CALIBRATION  -nowait" }

       "STD LDU1TRG"  { after idle "Generator p25mode LUD1TRIG     -nowait" }

       "STD NOTRIG"   { after idle "Generator p25mode NOTRIG       -nowait" }

       "STD LDU2TRG"  { after idle "Generator p25mode LUD2TRIG     -nowait" }

       "STD 511"      { after idle "Generator p25mode STD511       -nowait" }

       "STD SYMRATE"  { after idle "Generator p25mode SYMRATE      -nowait" }

       "STD LOWDEV"   { after idle "Generator p25mode LOWDEVIATION -nowait" }

       "STD FIDPAT"   { after idle "Generator p25mode FIDPATTERN   -nowait" }

       "STD FIDSPECT" { after idle "Generator p25mode FIDSPECTRUM  -nowait" }
     }
   }


   #*************************************************
   #       Name: update_p25_pattern_frame
   # Parameters: none
   #Description: update P25 pattern display
   #*************************************************
   proc update_p25_pattern_frame { widget  args } \
   {
     Debug::Puts GenUI 2 "update_p25_pattern_frame:  $widget,  $args"
  
     variable  StateUI
     
     set tile  [get_tile  $widget]

     # Remove the source button
     grid remove ${tile}.frm_gen.mod_p25.mnu_source

     # Determine if in P25 mode and the P25 PATTERN is set to VOICE
     if {$::Generator::State(is_P25_mode)} \
     {
       if { $::Generator::State(p25mode) == "VOICE" } \
       {
         # Display the P25 SOURCE menubutton
         Screens::position_widget ${tile}.frm_gen.mod_p25.mnu_source
       }
       # Set the sources to the present setting, or all OFF
       set_p25_source  $tile
     }
   }


   #*************************************************
   #       Name: set_p25_source
   # Parameters: tile
   #Description: sets P25 source
   #*************************************************
   proc set_p25_source { tile } \
   {
     Debug::Puts GenUI 2 "set_p25_source:  $tile"

     variable  StateUI
     
     # Turn OFF all Audio Routing
     BBAudio mixer  FGEN3  L1  OFF  -nowait
     BBAudio mixer  FGEN4  L1  OFF  -nowait
     BBAudio mixer  MIC    L1  OFF  -nowait
     BBAudio mixer  SINAD  L1  OFF  -nowait
     BBAudio mixer  L1     L1  OFF  -nowait

     # Determine if in P25 mode and the P25 PATTERN is set to VOICE
     if { $::Generator::State(is_P25_mode) } \
     {
       if { $::Generator::State(p25mode) == "VOICE" } \
       {
         # Set the Audio Routing
         switch -- $StateUI(voice_source) \
         {
           "OFF"    {}
           
           "M1"     { BBAudio mixer  FGEN3  L1  ON  -nowait }
                      
           "M2"     { BBAudio mixer  FGEN4  L1  ON  -nowait }
       
           "MIC"    { BBAudio mixer  MIC    L1  ON  -nowait }
       
           "AUDIO"  { BBAudio mixer  SINAD  L1  ON  -nowait }
       
           "DEMOD"  { BBAudio mixer  L1     L1  ON  -nowait }
         }
       }
     }

     # Update the display
     update_p25_source_frame  $tile
   }


   #*************************************************
   #       Name: update_p25_source_frame
   # Parameters: none
   #Description: update P25 source display
   #*************************************************
   proc update_p25_source_frame { widget  args } \
   {
     Debug::Puts GenUI 2 "update_p25_source_frame:  $widget,  $args"
  
     variable  StateUI

     set tile  [get_tile  $widget]

     set frame_path  ${tile}.frm_gen.mod_p25

     # Remove all possible frames
     grid remove ${frame_path}.voice_m1
     grid remove ${frame_path}.voice_m2
     grid remove ${frame_path}.voice_mic
     grid remove ${frame_path}.voice_aud

     # Determine if in P25 mode and the P25 PATTERN is set to VOICE
     if { $::Generator::State(is_P25_mode) } \
     {
       if { $::Generator::State(p25mode) == "VOICE" } \
       {
         # Display the correct fields
         switch -- $StateUI(voice_source) \
         {
           "OFF"    {}
           
           "M1"     { Screens::position_widget ${frame_path}.voice_m1  }
       
           "M2"     { Screens::position_widget ${frame_path}.voice_m2  }
        
           "MIC"    { Screens::position_widget ${frame_path}.voice_mic }
       
           "AUDIO"  { Screens::position_widget ${frame_path}.voice_aud }

           "DEMOD"  {}
         }
       }
       
       set_gen_softkeys  $tile
     }
   }


   #******************************************************
   #       Name: update_p25_voice_source
   # Parameters:
   #Description: update P25 voice display to changes
   #             in the Audio Route Modulator setting
   #******************************************************
   proc update_p25_voice_source { tile  args } \
   {
     Debug::Puts GenUI 2 "update_p25_voice_source:  $tile,  $args"
  
     variable  StateUI

     set tile  [get_tile  $tile]
     
     # Synchronize the P25 SOURCE with the Audio Routing Modulator setting
     # Determine if in P25 mode and the P25 PATTERN is set to VOICE
     if { $::Generator::State(is_P25_mode) } \
     {
       if { $::Generator::State(p25mode) == "VOICE" } \
       {
         Debug::Puts GenUI 2 "  ::BBAudio::State(demod_to_mod):  $::BBAudio::State(demod_to_mod)"
         
         # If the Audio Routing Modulator setting is DEMOD
         # set the P25 SOURCE to DEMOD
         # If the Audio Routing Modulator setting is OFF
         # set the P25 SOURCE to OFF
         switch -- $::BBAudio::State(demod_to_mod) \
         {
           "OFF"      { if { $StateUI(voice_source) != "OFF" } \
                        {
                          set StateUI(voice_source)  "OFF"
                        
                          # Turn OFF all Audio Routing
                          BBAudio mixer  FGEN3  L1  OFF  -nowait
                          BBAudio mixer  FGEN4  L1  OFF  -nowait
                          BBAudio mixer  MIC    L1  OFF  -nowait
                          BBAudio mixer  SINAD  L1  OFF  -nowait
                        
                          # Update the display
                          update_p25_source_frame  $tile
                        }
                      }

           "DEMOD"    { if { $StateUI(voice_source) != "DEMOD" } \
                        {
                          set StateUI(voice_source)  "DEMOD"
                        
                          # Turn OFF all Audio Routing but DEMOD
                          BBAudio mixer  FGEN3  L1  OFF  -nowait
                          BBAudio mixer  FGEN4  L1  OFF  -nowait
                          BBAudio mixer  MIC    L1  OFF  -nowait
                          BBAudio mixer  SINAD  L1  OFF  -nowait
                        
                          # Update the display
                          update_p25_source_frame  $tile
                        }
                      }
                    
           "SQUELCH"  {}
         }
       }
     }
   }
   
   
   #*************************************************
   #        Name: set_fgen_mode
   #  Parameters: lun - LUN value of the function generator
   # Description: sets the mode of the Function Generator
   #*************************************************
   proc set_fgen_mode { lun } \
   {
     Debug::Puts GenUI 2 "set_fgen_mode:  $lun"

     variable  fgen_mode
      
     # If the Generator is in the P25 mode, return
     if { $::Generator::State(is_P25_mode) } \
     {
       return
     }

     Debug::Puts GenUI 2 "fgen_mode($lun):  $fgen_mode($lun)"
     
     FGen::setMode  $lun  $fgen_mode($lun)
     
     # Notes:
     # Procedure 'FGenUI::tone_send_seq'           called in 'FGen::setMode' 
     # Procedure 'CTCSS_UI::ctcss_send_seq'        called in 'FGen::setMode' 
     # Procedure 'DCS_UI::dcs_start'               called in 'FGen::setMode' 
     # Procedure 'SincUI::setup_sinc_waveform'     called in 'FGen::setMode' 
     # Procedure 'ArbitraryUI::setup_arb_waveform' called in 'FGen::setMode' 
     # Procedure 'GeneratorUI::p25_send_seq'       called in 'FGen::setMode' 
   }


   #*************************************************
   #        Name: set_fgen_dev
   #  Parameters: lun - LUN value of the function generator
   # Description: sets the deviation of the Function Generator
   #*************************************************
   proc set_fgen_dev { lun } \
   {
     Debug::Puts GenUI 2 "set_fgen_dev:  $lun"

     set mode  $::FGen::State($lun,fgen_mode)

     if { $::Generator::State(is_P25_mode) } \
     {
       # P25
       # Displayed user field allows setting the Percent Deviation
       FGen::setPercent  $lun  $::FGen::State($lun,$mode,percent)
     } \
     else \
     {
       switch -- $::Generator::State(mod_raw) \
       {
         0   { }
         
         1   { # FM
               # Displayed user field allows setting the Deviation
               FGen::setDeviation  $lun  $::FGen::State($lun,$mode,dev) 
         
               # Notes:
               # Procedure 'FGenUI::tone_send_seq'     called in 'FGen::setDeviation' 
               # Procedure 'CTCSS_UI::ctcss_send_seq'  called in 'FGen::setDeviation' 
               # Procedure 'DCS_UI::dcs_start'         called in 'FGen::setDeviation' 
               # Procedure 'SincUI::send_sinc_seq'     called in 'FGen::setDeviation' 
               # Procedure 'ArbitraryUI::send_arb_seq' called in 'FGen::setDeviation' 
               # Procedure 'GeneratorUI::p25_send_seq' called in 'FGen::setDeviation' 
             }
         
         4   { # AM
               # Displayed user field allows setting the Percent Deviation
               FGen::setPercent  $lun  $::FGen::State($lun,$mode,percent) 
         
               # Notes:
               # Procedure 'FGenUI::tone_send_seq'     called in 'FGen::setPercent' 
               # Procedure 'CTCSS_UI::ctcss_send_seq'  called in 'FGen::setPercent' 
               # Procedure 'DCS_UI::dcs_start'         called in 'FGen::setPercent' 
               # Procedure 'SincUI::send_sinc_seq'     called in 'FGen::setPercent' 
               # Procedure 'ArbitraryUI::send_arb_seq' called in 'FGen::setPercent' 
               # Procedure 'GeneratorUI::p25_send_seq' called in 'FGen::setPercent' 
             }
       }
     }
   }


   #*************************************************
   #        Name: set_fgen_freq
   #  Parameters: lun - LUN value of the function generator
   # Description: sets the frequency of the Function Generator
   #*************************************************
   proc set_fgen_freq { lun } \
   {
     Debug::Puts GenUI 2 "set_fgen_freq:  $lun"

     set mode  $::FGen::State($lun,fgen_mode)

     FGen::setFreq  $lun  $::FGen::State($lun,$mode,freq)

     # Notes:
     # Procedure 'FGenUI::tone_send_seq'     called in 'FGen::setFreq' 
     # Procedure 'CTCSS_UI::ctcss_send_seq'  called in 'FGen::setFreq' 
     # Procedure 'DCS_UI::dcs_start'         called in 'FGen::setFreq' 
     # Procedure 'SincUI::send_sinc_seq'     called in 'FGen::setFreq' 
     # Procedure 'ArbitraryUI::send_arb_seq' called in 'FGen::setFreq' 
     # Procedure 'GeneratorUI::p25_send_seq' called in 'FGen::setFreq' 
   }


   #*************************************************
   #        Name: set_fgen_shape
   #  Parameters: lun - LUN value of the function generator
   # Description: sets the shape of the Function Generator
   #*************************************************
   proc set_fgen_shape { lun } \
   {
     Debug::Puts GenUI 2 "set_fgen_shape:  $lun"

     set mode  $::FGen::State($lun,fgen_mode)

     FGen::setShape  $lun  $::FGen::State($lun,$mode,shape)

     # Notes:
     # Procedure 'FGenUI::tone_send_seq'     called in 'FGen::setShape' 
     # Procedure 'GeneratorUI::p25_send_seq' called in 'FGen::setShape' 
   }


   #*************************************************
   #        Name: update_fgen_frame
   #  Parameters: widget - tile path
   # Description: display/remove FM modulation frames
   #*************************************************
   proc update_fgen_frame { widget  lun  args } \
   {
     Debug::Puts GenUI 2 "update_fgen_frame:  $widget,  $lun,  $args"
     variable StateUI
     
     set cmd "GeneratorUI::update_fgen_frame_core $widget $lun"
     after cancel $cmd
     after idle $cmd
   }
   proc update_fgen_frame_core { widget  lun } \
   {
     variable  fgen_mode
     variable StateUI
     set StateUI($lun,fgen_mode) $::FGen::State($lun,fgen_mode)
     
     # Do not update any variables if the Generator is in the P25 mode     
     if { $::Generator::State(is_P25_mode) } \
     {
       return
     }


     set tile  [get_tile  $widget]
 
     set mod_num  [expr $lun - 2]

     
     # Remove all applicable frames
     grid remove ${tile}.frm_gen.mod_fm_am.m${mod_num}.off
     grid remove ${tile}.frm_gen.mod_fm_am.m${mod_num}.tone
     grid remove ${tile}.frm_gen.mod_fm_am.m${mod_num}.trem
     grid remove ${tile}.frm_gen.mod_fm_am.m${mod_num}.dtmf
     grid remove ${tile}.frm_gen.mod_fm_am.m${mod_num}.ctcss
     grid remove ${tile}.frm_gen.mod_fm_am.m${mod_num}.dcs
     grid remove ${tile}.frm_gen.mod_fm_am.m${mod_num}.tsig
     grid remove ${tile}.frm_gen.mod_fm_am.m${mod_num}.sinc
     grid remove ${tile}.frm_gen.mod_fm_am.m${mod_num}.arb
     grid remove ${tile}.frm_gen.mod_fm_am.m${mod_num}.sweep


     if { $::FGen::State($lun,fgen_mode) != "P25" } \
     {
        # We want to capture what the mode was for the LUN if it was not
        # P25 so that it can be restored. But we do NOT want to capture it
        # during a setup recall, because we are getting feedback issues
        if { [info exists ::Recall_SetupsUI::State(recall_in_process)] &&  \
             $::Recall_SetupsUI::State(recall_in_process) == 0 } \
        {
           set fgen_mode($lun)  $::FGen::State($lun,fgen_mode)
        }
     }
     
     if { $::Generator::State(mod) == "OFF" } \
     {
       # Display should be turned off
       # All frames have been unmapped
       return
     }

     Debug::Puts GenUI 2 "fgen_mode($lun):  $fgen_mode($lun)"

     switch -- $::FGen::State($lun,fgen_mode) \
     {
       "OFF"     { Screens::position_widget ${tile}.frm_gen.mod_fm_am.m${mod_num}.off   }
                   
       "TONE"    { Screens::position_widget ${tile}.frm_gen.mod_fm_am.m${mod_num}.tone
                   
                   # Procedure 'FGenUI::tone_send_seq' called in 'FGen::setMode' 
                 }
                   
       "TREM"    { Screens::position_widget ${tile}.frm_gen.mod_fm_am.m${mod_num}.trem  }
                    
       "DTMF"    { Screens::position_widget ${tile}.frm_gen.mod_fm_am.m${mod_num}.dtmf  }
  
       "CTCSS"   { Screens::position_widget ${tile}.frm_gen.mod_fm_am.m${mod_num}.ctcss
                   
                   # Procedure 'CTCSS_UI::ctcss_send_seq' called in 'FGen::setMode' 
                 }
                   
       "DCS"     { Screens::position_widget ${tile}.frm_gen.mod_fm_am.m${mod_num}.dcs   
       
                   # Procedure 'DCS_UI::dcs_start' called in 'FGen::setMode' 
                 }
                   
       "TSIG"    { Screens::position_widget ${tile}.frm_gen.mod_fm_am.m${mod_num}.tsig  }

       "SINC"    { Screens::position_widget ${tile}.frm_gen.mod_fm_am.m${mod_num}.sinc  
       
                   # Procedure 'SincUI::send_sinc_seq' called in 'FGen::setMode' 
                 }

       "ARB"     { Screens::position_widget ${tile}.frm_gen.mod_fm_am.m${mod_num}.arb   
       
                   # Procedure 'ArbitraryUI::send_arb_seq' called in 'FGen::setMode' 
                 }

       "SWEEP"   { Screens::position_widget ${tile}.frm_gen.mod_fm_am.m${mod_num}.sweep }   
     }

     set_tone_softkey  $widget  $lun
   }

   #***************************************************************************
   #
   # Name:        p25_send_seq
   #
   # Parameters:  lun - Logical Unit Number of the Function Generator
   #
   # Description: Sends out a sequence for the P25 mode
   #
   #***************************************************************************
   proc p25_send_seq { lun } \
   {
     Debug::Puts GenUI 2 "p25_send_seq:  $lun"
     
     switch -- $lun \
     {
       3  -
       4  { set freq   $::FGen::State($lun,P25,freq)
            set level  $::FGen::State($lun,P25,level)
            set shape  $::FGen::State($lun,P25,shape) 
            
            set tone_seq  "$freq  0  $shape  0  $level"
     
            Debug::Puts GenUI 2 "tone_seq:  $lun,  $tone_seq"
     
            FGen::Send  $lun  $tone_seq }
            
       default {  }
     }
   }


   #*************************************************
   #        Name: set_mic_state
   #  Parameters: 
   # Description: sets the MIC input state
   #*************************************************
   proc set_mic_state { } \
   {
     Debug::Puts GenUI 2 "set_mic_state"
     
     variable  mic_state
     
     switch -- $mic_state \
     {
       "OFF"       { after idle "BBAudio mixer  MIC  L1  OFF -nowait" }
       
       "ELECTRET"  { after idle "BBAudio mixer  MIC  L1  ON -nowait"
                     after idle "BBAudio micbias 1 -nowait" }
       
       "DYNAMIC"   { after idle "BBAudio mixer  MIC  L1  ON -nowait"
                     after idle "BBAudio micbias 0 -nowait" }
     }
   }


   #*************************************************
   #        Name: update_mic_state
   #  Parameters: 
   # Description: updates the display to the MIC 
   #              input state
   #*************************************************
   proc update_mic_state { args } \
   {
     Debug::Puts GenUI 2 "update_mic_state:  $args"
     
     variable  mic_state
     
     if { $::BBAudio::Mixers(L1.MIC) == "OFF" } \
     { 
       set mic_state  "OFF"
       
       return
     }


     switch -- $::BBAudio::State(micbias_sname) \
     {
       "ELEC"  { set mic_state  "ELECTRET" }
       
       "DYN"   { set mic_state  "DYNAMIC"  }
     }
   }


   #*************************************************
   #        Name: toggle_mic_bias
   #  Parameters: 
   # Description: set input MIC bias
   #*************************************************
   proc toggle_mic_bias { } \
   {
     # TOGGLE ON/OFF MIC BIAS
     if { $::BBAudio::State(micbias_sname) == "ELEC" } \
     {
       after idle "BBAudio micbias 0 -nowait"
     } \
     else \
     {
       after idle "BBAudio micbias 1 -nowait"
     }
   }

   
   #*************************************************
   #        Name: set_aud_state
   #  Parameters: 
   # Description: sets the AUDIO input state
   #*************************************************
   proc set_aud_state { } \
   {
     Debug::Puts GenUI 2 "set_aud_state"
     
     variable  aud_state
     
     switch -- $aud_state \
     {
       "OFF"         { after idle "BBAudio mixer  SINAD  L1  OFF -nowait" }
       
       "UNBALANCED"  { after idle "BBAudio mixer  SINAD  L1  ON -nowait"
                       after idle "BBAudio sinadbal 1 -nowait" }
       
       "BALANCED"    { after idle "BBAudio mixer  SINAD  L1  ON -nowait"
                       after idle "BBAudio sinadbal 0 -nowait" }
     }
   }


   #*************************************************
   #        Name: update_aud_state
   #  Parameters: 
   # Description: updates the display to the AUDIO
   #              input state
   #*************************************************
   proc update_aud_state { args } \
   {
     Debug::Puts GenUI 2 "update_aud_state:  $args"
     
     variable  aud_state_display
     
     if { $::BBAudio::Mixers(L1.SINAD) == "OFF" } \
     { 
       set aud_state_display  "OFF"
       
       return
     }


     switch -- $::BBAudio::State(sinadbal_sname) \
     {
       "BALAN"  { set aud_state_display  "BALAN" }
       
       "UNBAL"  { set aud_state_display  "UNBAL"  }
     }
   }


   #*************************************************
   #        Name: toggle_aud_balance
   #  Parameters: 
   # Description: sets rf generator audio in balance for analog modulation 
   #*************************************************
   proc toggle_aud_balance { } \
   {
     # TOGGLE ON/OFF AUDIO BALANCE
     if { $::BBAudio::State(sinadbal_sname) == "BALAN" } \
     {
       after idle "BBAudio sinadbal 1 -nowait"
     } \
     else \
     {
       after idle "BBAudio sinadbal 0 -nowait"
     }
   }


   #****************************************************************************
   # Name:        getSaveState
   # Parameters:  none
   # Description: This function returns a list of TCL commands which can be used
   #              to set the GeneratorUI tiles to its current state
   #****************************************************************************
   proc getSaveState { } \
   {
     variable  StateUI
     variable  levelunit
     variable  fgen_mode
 
     set returnList \
     {
       set ::GeneratorUI::levelunit               {$levelunit}
       set ::GeneratorUI::StateUI(chanmode)	      {$StateUI(chanmode)}
       set ::ReceiverUI::StateUI(list_freq_index) {$::ReceiverUI::StateUI(list_freq_index)}
       set ::GeneratorUI::StateUI(sweepstart)     {$StateUI(sweepstart)}
       set ::GeneratorUI::StateUI(sweepstop)      {$StateUI(sweepstop)}
       set ::GeneratorUI::StateUI(sweeprate)      {$StateUI(sweeprate)}
       set ::GeneratorUI::StateUI(sweepincr)      {$StateUI(sweepincr)}
       set ::GeneratorUI::StateUI(incrfreq)       {$StateUI(incrfreq)}
       set ::GeneratorUI::StateUI(voice_source)   {$StateUI(voice_source)}

       set ::GeneratorUI::fgen_mode(3)   {$fgen_mode(3)}
       set ::GeneratorUI::fgen_mode(4)   {$fgen_mode(4)}
     }
     set returnList [subst -nocommands $returnList]
     append returnList [namespace code \
     {
       if {$StateUI(chanmode) == "LIST"} \
       {
          get_list_freq
       }
     }]
     return $returnList
   }


   #****************************************************************************
   # Name:        Sync
   # Parameters:  none
   # Description: Write state variables to GeneratorUI
   #****************************************************************************
   proc Sync { } \
   {
     eval [getSaveState]
   }
   
   #****************************************************************************
   # Name:        Hook_unlock
   # Parameters:  toplevel window for this view
   # Description: Hook up to handle generator unlock messages for a view
   #****************************************************************************  
   proc Hook_unlock { toplevel} \
   {
      variable StateUI
      
      # trace the unlock state
      trace variable ::Generator::State(lockstat) w "GeneratorUI::Unlock $toplevel"
      
      set StateUI(unlock_dismiss,$toplevel) 0

   }
   
   #****************************************************************************
   # Name:        Unhook_unlock
   # Parameters:  toplevel window for this view
   # Description: Remove hook, as the screen is being destroyed
   #****************************************************************************
   proc Unhook_unlock { toplevel } \
   {
      variable StateUI
      trace vdelete ::Generator::State(lockstat) w "GeneratorUI::Unlock $toplevel"
      
      unset StateUI(unlock_dismiss,$toplevel)
   }
   #****************************************************************************
   # Name:        Unlock
   # Parameters:  toplevel window for this view, trace var args (ignored)
   # Description: Update the unlock window as needed
   #****************************************************************************
   proc Unlock { toplevel args } \
   {
      # if the lock status is changing, take no action....
      if {$::Generator::State(lock_unstable) != 0 } { return }
      
      set cmd "GeneratorUI::Unlock_core $toplevel"
      after cancel $cmd
      after idle $cmd
   }
 
   proc Unlock_core { toplevel } \
   {
     variable  StateUI
     set alert [name_child $toplevel genunlock]
     upvar #0 ::Generator::State(lockstat) lockstat
     
     if {$lockstat } \
     {
        if {$StateUI(unlock_dismiss,$toplevel) == 1 } { return }
        
        # unlocked: Show the window if it does not already exists
        if ![winfo exists $alert] \
        {
           message_box $alert CAUTION
           bind $alert <<Destroy>> "GeneratorUI::Unlock_Cleanup $alert"
           trace variable ::HW_control::State(yigres_gen) w "GeneratorUI::Unlock_update $alert"
           Unlock_update $alert
        }
     } \
     else \
     {
        # locked - destroy the alert window if it exists
        if [winfo exists $alert] \
        {
           destroy $alert
        }
        set StateUI(unlock_dismiss,$toplevel) 0
     }
   }
   #****************************************************************************
   # Name:        Unlock_cleanup
   # Parameters:  alert box
   # Description: clean up when destroyed
   #****************************************************************************
   proc Unlock_Cleanup { alert } \
   {
      trace vdelete ::HW_control::State(yigres_gen) w "GeneratorUI::Unlock_update $alert"
   }
   #****************************************************************************
   # Name:        Unlock_continue
   # Parameters:  the name of the unlock window
   # Description: dismiss window even though we are still unlocked...
   #****************************************************************************
   proc Unlock_continue { alert } \
   {
      variable StateUI
      set toplevel [winfo parent $alert]
      set StateUI(unlock_dismiss,$toplevel) 1

      if [winfo exists $alert] \
      {
         destroy $alert
      }
   }

   #****************************************************************************
   # Name:        Unlock_update
   # Parameters:  the name of the unlock window
   # Description: 
   #****************************************************************************
   proc Unlock_update { alert args } \
   {
      switch -- $::HW_control::State(yigres_gen) \
      {
         "Complete" \
         {
            if [winfo exists $alert] { destroy $alert }
         }
         "Running"  \
         {
            # lock out the button while cal is running
            $alert.cancel config -state disabled \
                                 -text [subst [option get $alert.cancel altText {}]]
         }
         default    \
         {
            # Cal failed. Allow cancel
            $alert.cancel config -state normal \
                         -text [option get $alert.cancel text {}]
            $alert.text config -text [subst [option get $alert.text altText {}]]

         }
      }
   }
   # now, do all the tile independant traces
   trace variable ::Generator::State(dcpower)      w ::GeneratorUI::update_rf
   trace variable ::Generator::State(ptt)          w ::GeneratorUI::update_rf
   trace variable ::Generator::State(freq)         w ::GeneratorUI::update_offset                                        
   trace variable ::Receiver::State(freq)          w ::GeneratorUI::update_offset
   trace variable ::Generator::State(rflev,raw)    w ::GeneratorUI::update_rflevel
   trace variable ::BBAudio::Mixers(L1.MIC)        w ::GeneratorUI::update_mic_state
   trace variable ::BBAudio::Mixers(L1.SINAD)      w ::GeneratorUI::update_aud_state
   trace variable ::BBAudio::State(micbias_sname)  w ::GeneratorUI::update_mic_state
   trace variable ::BBAudio::State(sinadbal_sname) w ::GeneratorUI::update_aud_state
   update_rflevel
   update_offset
   update_rf
   
  proc trace_mode_vars { trace } \
  {
     variable StateUI
     if {$trace != 0} \
     {
        if {$StateUI(update_mode_vars_trace) == 0} \
        {
            trace variable ::Freq_ListUI::ListFreqArray           w $StateUI(update_mode_vars_cmd)
            incr StateUI(update_mode_vars_trace)
        }
     } \
     else \
     {
        if {$StateUI(update_mode_vars_trace) != 0} \
        {
          trace vdelete ::Freq_ListUI::ListFreqArray           w $StateUI(update_mode_vars_cmd)
          set StateUI(update_mode_vars_trace) 0
        }
     }
  }
  proc update_mode_vars { args } \
  {
     variable StateUI
     Debug::Puts GenUI 2 "update_mode_vars"
     set cmd [namespace code "update_mode_vars_core"]
     # optimization: we saw the change, don't keep retriggering!
     trace_mode_vars 0

     after cancel $cmd
     after 500    $cmd
  }
  proc update_mode_vars_core {  } \
  {
    variable StateUI
    Debug::Puts GenUI 2 "update_mode_vars_core"
    set cmd [namespace code update_freq_list]
    if {$StateUI(chanmode) == "LIST"} \
    {
       # watch for changes on the frequency list vars, and update as needed
       trace_mode_vars 1
       get_list_freq
    } \
    else \
    {
       trace_mode_vars 0
    }
  }
  proc update_freq_list { args } \
  {
     Debug::Puts GenUI 2 "update_freq_list"
     trace_mode_vars 1
     set cmd [namespace code "get_list_freq"]
     after cancel $cmd
     after 500 $cmd
  }
  set StateUI(update_mode_vars_trace) 0
  set StateUI(update_mode_vars_cmd) [namespace code update_freq_list]
  trace variable ::GeneratorUI::StateUI(chanmode)  w "::GeneratorUI::update_mode_vars"
  update_mode_vars_core
}


#****************************************************************************
# This registers the tiles with the list for saving instrument states
#****************************************************************************
namespace eval ::SavedStates \
{
  variable tiles   
  lappend tiles GeneratorUI
}



namespace eval View \
{
   lappend PreCreate GeneratorUI
   lappend Alert_routines GeneratorUI::Hook_unlock
   lappend Clean_ups GeneratorUI::Unhook_unlock

}




