#############################################################################
#                                                                           #
# IFR SYSTEMS, INC        COPYRIGHT (C)1998-2002      DESIGN ENGINEERING    #
# ------------------------------------------------------------------------- #
# FILE        : 3510.P25uplink_UI.tcl                                       #
# PRODUCT     : FITS                                                        #
# AUTHOR      : Hazmar Feroze, Hieu Tran                                    #
# DESCRIPTION : P25 User Interface	                                        #
#                                                                           #
#############################################################################



namespace eval UplinkUI \
{
   # Global variables
   variable  LongName 		p25uplink
   variable  ShortName  	"P 25 Uplink Data"	
   variable  Size      		Quarter
   variable  tile_class 	P25INPUT_QUARTER
   variable  top_tile		""

   variable  toplevel		""

   variable  StateUI
  

   #********************************************************************
   #        Name: Create  
   #  Parameters: tile path of parent
   # Description: create P 25 uplink data tile
   #********************************************************************
   proc Create { parent } \
   {
      variable LongName
      variable tile_class

      set top_level [winfo toplevel $parent]
      
      set tile_path [name_child $top_level $LongName]

      if {![winfo exists $tile_path]} \
      {
         set tile [Screens::create_tile $tile_path $LongName $tile_class]
      }

      return $tile_path
   }


   #******************************************************************** 
   #        Name: Map
   #  Parameters: widget - tile path
   # Description: called when the screen is displayed 
   #********************************************************************
   proc Map { widget } \
   {
      # RESET DECODE COUNT
      set ::P25ns::DecodeCount 0

      set_p25in_lco_core $widget
      set_p25in_receive

      trace variable ::P25ns::DecodeParm(LCO) w "::UplinkUI::set_p25in_lco $widget"
   }
  

   #********************************************************************
   #        Name: Unmap
   #  Parameters: widget - tile path
   # Description: called when the screen is hidden
   #********************************************************************
   proc Unmap { widget } \
   {
      trace vdelete ::P25ns::DecodeParm(LCO) w "::UplinkUI::set_p25in_lco $widget"
   }


   #********************************************************************
   #        Name: Destroy
   # Parameters : widget - tile path
   # Description: called when the screen is destroyed 
   #********************************************************************
   proc Destroy { widget } \
   {
      return
   }


   #***************************************************************************
   #
   #       Name: get_tile
   #
   # Parameters: any child widget of the tile
   #
   # Description: Get the name of the tile's parent frame
   #
   #***************************************************************************
   proc get_tile { widget } \
   {
     while { 1 } \
     {
       switch -- [winfo class $widget] \
       {
         "P25INPUT_QUARTER"  -
         "P25INPUT_TOP"      -
         "P25_LOG_SETUP"     -
         "P25_LOG_VIEW"      { return $widget }
       }
        
       set widget [winfo parent $widget]
     }

     return     
   }


   #*************************************************
   #        Name: display_toplevel
   #  Parameters: widget - tile path
   # Description: display P25 uplink data toplevel
   #*************************************************
   proc display_toplevel { widget { name uplink } } \
   {
      variable  toplevel

      if { ![winfo exists $widget] } { set widget  . }

      set parent  [winfo toplevel  $widget]
      
      set toplevel  [name_child  $parent  $name]

      if { ![winfo exists  $toplevel] } \
      {
        Screens::create_toplevel_tile  $toplevel  P25INPUT_TOP
       
        bind  $toplevel  <KeyRelease-F8>  "destroy $toplevel; break"
      }

      draw_p25in_lco  ${toplevel}.frm_voice.ctrl
      
      safe_focus_force  ${toplevel}.frm_voice.ctrl
   }


   #*************************************************
   #        Name: remove_lco_frames
   #  Parameters: tile - tile path
   # Description: remove all LCO frames
   #*************************************************
   proc remove_lco_frames { tile } \
   {
      grid remove ${tile}.frm_lco00
      grid remove ${tile}.frm_lco02
      grid remove ${tile}.frm_lco03
      grid remove ${tile}.frm_lco04
      grid remove ${tile}.frm_lco05
      grid remove ${tile}.frm_lco06
      grid remove ${tile}.frm_lco07
      grid remove ${tile}.frm_lco15
      grid remove ${tile}.frm_lco16
      grid remove ${tile}.frm_lco17
      grid remove ${tile}.frm_lco32
      grid remove ${tile}.frm_lco33
      grid remove ${tile}.frm_lco34
      grid remove ${tile}.frm_lco36
      grid remove ${tile}.frm_lco20
      grid remove ${tile}.frm_lco21
      grid remove ${tile}.frm_lco23
      grid remove ${tile}.frm_lco24
      grid remove ${tile}.frm_lco37
   }


   #*************************************************
   #        Name: map_lco_frame
   #  Parameters: lco_path - tile path for all lco frames
   #              lco - lco frame to be mapped
   # Description: map lco frame
   #*************************************************
   proc map_lco_frame { lco_path lco } \
   {
      if {[winfo exists $lco] && ![winfo ismapped $lco]} \
      {
         remove_lco_frames $lco_path
         ::Screens::position_widget $lco
      } 
   }


   #*************************************************
   #        Name: draw_p25in_lco
   #  Parameters: tile - tile path
   # Description: display/remove frames depending on LCO value
   #*************************************************
   proc draw_p25in_lco { tile } \
   {
      variable StateUI
      
      set tile [get_tile $tile]
      if ![winfo exists ${tile}.frm_lco00] \
      {
         set tile $tile.frm_voice.ctrl
      }

      switch -- $::P25ns::DecodeParm(LCO) \
      {
         "00" { map_lco_frame $tile ${tile}.frm_lco00 }

         "02" { map_lco_frame $tile ${tile}.frm_lco02 }

	 "03" { map_lco_frame $tile ${tile}.frm_lco03 }

	 "04" { map_lco_frame $tile ${tile}.frm_lco04 }

	 "05" { map_lco_frame $tile ${tile}.frm_lco05 }

	 "06" { map_lco_frame $tile ${tile}.frm_lco06 }

	 "07" { map_lco_frame $tile ${tile}.frm_lco07 }

         "0F" { map_lco_frame $tile ${tile}.frm_lco15 }

	 "10" -
	 "13" -
         "16" { map_lco_frame $tile ${tile}.frm_lco16 }

	 "11" -
	 "12" { map_lco_frame $tile ${tile}.frm_lco17 }

	 "14" { map_lco_frame $tile ${tile}.frm_lco20 }

	 "15" { map_lco_frame $tile ${tile}.frm_lco21 }
       
	 "17" { map_lco_frame $tile ${tile}.frm_lco23 }

	 "18" { map_lco_frame $tile ${tile}.frm_lco24 }

	 "20" { map_lco_frame $tile ${tile}.frm_lco32 }

	 "21" { map_lco_frame $tile ${tile}.frm_lco33 }

	 "22" -
	 "23" { map_lco_frame $tile ${tile}.frm_lco34 }

	 "24" { map_lco_frame $tile ${tile}.frm_lco36 }

	 "25" { map_lco_frame $tile ${tile}.frm_lco37 }

	 default { remove_lco_frames $tile }
      }

   }


   #*************************************************
   #        Name: set_p25in_receive
   #  Parameters: none 
   # Description: setup to receive P25 decode data
   #*************************************************
   proc set_p25in_receive { } \
   {
      # SET HARDWARE AGC
      if {$::Receiver::State(hwagc) != "SLOW"} \
      {
         Receiver hwagc SLOW
      }

      # SET SW AGC MODE
      if {$::Receiver::State(agcmode) != 1} \
      {
         Receiver agcmode ON
      }

      # ENABLE P25 DECODE DATA
      if {$::P25ns::State(d_enable) != 3} \
      {
         P25 decode enable 3
      }

      # START DECODE P25 DATA
      if {$::P25ns::State(d_run) != 1} \
      {
         P25 decode run on
      }
   }


   #*************************************************
   #        Name: set_p25in_lco
   #  Parameters: widget - tile path
   # Description: updating LCO frame on LCO's state variable change
   #*************************************************
   proc set_p25in_lco { widget args } \
   {
      set cmd "::UplinkUI::set_p25in_lco_core $widget"
      after cancel $cmd
      after idle $cmd
   }
   proc set_p25in_lco_core { widget } \
   {
      variable toplevel

      set tile_path ${widget}

      set top_path ${toplevel}.frm_voice.ctrl

      if {[winfo exists $tile_path]} \
      {
         draw_p25in_lco $tile_path
      }

      if {[winfo exists $top_path]} \
      {
         draw_p25in_lco $top_path
      }
   }


}



namespace eval View \
{
   lappend PreCreate UplinkUI
}

