#############################################################################
#                                                                           #
# IFR SYSTEMS, INC        COPYRIGHT (C)1998 - 2002      DESIGN ENGINEERING  #
# ------------------------------------------------------------------------- #
#                                                                           #
# FILE        : 3520.P25downlinkUI.tcl                                      #
#                                                                           #
# PRODUCT     : FITS                                                        #
#                                                                           #
# AUTHOR      : Hazmar Feroze, Hieu Tran                                    #
#                                                                           #
# DESCRIPTION : P25 Downlink User Interface                                 #
#                                                                           #
#############################################################################

package require Keys


namespace eval DownlinkUI \
{
  # Global variables
  variable  LongName 		p25downlink
  variable  ShortName		"P 25 Downlink Data"
  variable  Size         	Quarter
  variable  tile_class		P25OUTPUT_QUARTER
  
  #variable  show_key_id_menu  0

  variable  file_label   "Key_Set"

  variable  Encode
  
  set Encode(raw1)	        00
  set Encode(raw2)		00
  set Encode(raw3)		00
  set Encode(raw4)		00
  set Encode(raw5)		00
  set Encode(raw6)		00
  set Encode(raw7)		00
  set Encode(raw8)		00
  set Encode(raw9)		00
  set Encode(StatusText)	""
  set Encode(StatusShortText)	""
  set Encode(LCO_text)		""
  set Encode(LCO_short_text)	""

  variable  StateUI

  for { set i 1 } { $i < 11 } { incr i } \
  {
     set StateUI(DGT$i)  0
  }
 
  set  StateUI(kid_menu)    { }
  set  StateUI(algid_menu)  { "CLEAR"  "DES"  "AES" }


  variable  algid_list

  set algid_list(0x80)  ""
  set algid_list(0x81)  ""
  set algid_list(0x83)  ""
  
  variable  keyid_list  {}


  #***************************************************************************
  #
  #        Name: Create  
  #
  #  Parameters: tile path of parent
  #
  # Description: Create P 25 downlink data tile
  #
  #***************************************************************************
  proc Create { parent } \
  {
    variable LongName
    variable tile_class

    set top_level  [winfo toplevel $parent]
     
    set tile_path  [name_child $top_level $LongName]

    if { ![winfo exists $tile_path] } \
    {
      set tile [Screens::create_tile $tile_path $LongName $tile_class]
    }

    return $tile_path
  }


  #***************************************************************************
  #
  #        Name: Map
  #
  #  Parameters: tile_path - tile path
  #
  # Description: Called when the screen is displayed
  #
  #***************************************************************************
  proc Map { tile_path } \
  {
    #puts "Map:  $tile_path"

    trace variable ::P25ns::EncodeParm(LCO)  w  \
                               "::DownlinkUI::display_lco_fields  $tile_path"

    trace variable ::Generator::State(p25mode) w  \
                                     "::DownlinkUI::Watch_pattern $tile_path"
      

    display_lco_fields_core  $tile_path
      
    update_lcotext
    
    Watch_pattern_core  $tile_path
    
    return
  }
  

  #***************************************************************************
  #
  #        Name: Cleanup
  #
  #  Parameters: tile_path - tile path
  #
  # Description: Called when the screen is hidden
  #
  #***************************************************************************
  proc Cleanup { tile_path } \
  {
    trace vdelete ::P25ns::EncodeParm(LCO) w  \
                              "::DownlinkUI::display_lco_fields  $tile_path"
    
    trace vdelete ::Generator::State(p25mode) w  \
                                    "::DownlinkUI::Watch_pattern $tile_path"


    return
  }


  #***************************************************************************
  #
  #        Name: Unmap
  #
  #  Parameters: tile_path - tile path
  #
  # Description: Called when the screen is hidden
  #
  #***************************************************************************
  proc Unmap { tile_path } \
  {
    Cleanup  $tile_path
  }


  #***************************************************************************
  #
  #        Name: Destroy
  #
  #  Parameters: tile_path - tile path
  #
  # Description: called when the screen is destroyed
  #
  #***************************************************************************
  proc Destroy { tile_path } \
  {
    Cleanup  $tile_path
  }

  #***************************************************************************
  #  Name: get_tile
  #
  #  Parameters: any child widget of the tile
  #  Description: Returns the widget or parent which has the class of
  #               P25OUTPUT_QUARTER  or  P25OUTPUT_TOP
  #               Returns "" if there is an error
  #***************************************************************************
  proc get_tile { widget } \
  {
     switch -- [winfo class $widget] \
     {
        "P25OUTPUT_QUARTER"  -
        "P25OUTPUT_TOP"      { return $widget }
     }

     # initialize value to force one pass through the loop
     set top_level ""
   
     while { $top_level != $widget } \
     {
        set widget [winfo parent $widget]
        switch -- [winfo class $widget] \
        {
           "P25OUTPUT_QUARTER"  -
           "P25OUTPUT_TOP"      { return $widget }
        }
        set top_level  [winfo toplevel $widget]
     }
     return ""     
  }



  #***************************************************************************
  #
  #        Name: display_toplevel
  #
  #  Parameters: widget - tile path
  #
  # Description: Display P25 downlink data toplevel
  #
  #***************************************************************************
  proc display_toplevel { widget  { name  downlink } } \
  {
    #puts "display_toplevel:  $widget,  $name"
    
    if { ![winfo exists $widget] } { set widget . }

    set parent [winfo toplevel $widget]
    
    set toplevel [name_child $parent $name]
  
    # TAKE MORE THAN 20 SEC TO DESTROY P25 ENCODE ZOOM TILE
    # WITH ALL THE NEW ADDED LCO FIELDS
    # SO UNMAP THE TILE INSTEAD OF DESTROYING IT 
    if { ![winfo exists $toplevel] } \
    {
      Screens::create_toplevel_tile $toplevel P25OUTPUT_TOP

      wm protocol $toplevel WM_DELETE_WINDOW "wm withdraw $toplevel"

      bind $toplevel <KeyRelease-F8> "wm withdraw $toplevel; break"
    } \
    else \
    {
      wm deiconify $toplevel
    }
    
    if { [winfo exists ${toplevel}.btn_key_loader] } \
    {
      if { [IFR_options  is_installed  KEY_MGT] } \
      {
        ${toplevel}.btn_key_loader config  -state normal
      } \
      else \
      {
        ${toplevel}.btn_key_loader config  -state disabled
      }
    }
 
    after idle  "focus  ${toplevel}.btn_send_header"
    
    return
  }


  #***************************************************************************
  #
  #        Name: set_value
  #
  #  Parameters: none
  #
  # Description: Sets the value sent as a parameter using the subsystem
  #            : call sent as a paramter
  #
  #***************************************************************************
  proc set_value { sub_system  array_variable } \
  {
    #puts "set_value:  $sub_system,  $array_variable"
    
    if { $::P25ns::EncodeParm($array_variable) == "" } \
    {
      set ::P25ns::EncodeParm($array_variable)  0
    }

    P25 encode field  $sub_system  \
                           0x$::P25ns::EncodeParm($array_variable)  -nowait

    return
  }


  #***************************************************************************
  #  
  #        Name: remove_lco_fields
  #
  #  Parameters: tile_path - tile path
  #
  # Description: Remove all LCO frames
  #
  #***************************************************************************
  proc remove_lco_fields { tile_path } \
  {
    variable StateUI

    set tile_path  [get_tile  $tile_path]

    grid remove ${tile_path}.lbl_mfid_1         
    grid remove ${tile_path}.ent_mfid_1         
    grid remove ${tile_path}.lbl_srv_r_1        
    grid remove ${tile_path}.ent_srv_r_1        
    grid remove ${tile_path}.lbl_srv_l_1        
    grid remove ${tile_path}.ent_srv_l_1        
    grid remove ${tile_path}.lbl_tgid_1         
    grid remove ${tile_path}.ent_tgid_1         
    grid remove ${tile_path}.lbl_did_1          
    grid remove ${tile_path}.ent_did_1          
    grid remove ${tile_path}.lbl_sid_1          
    grid remove ${tile_path}.ent_sid_1          
    grid remove ${tile_path}.lbl_chan_a_1       
    grid remove ${tile_path}.ent_chanid_a_1     
    grid remove ${tile_path}.ent_channum_a_1    
    grid remove ${tile_path}.lbl_grp_a_1        
    grid remove ${tile_path}.ent_grp_a_1        
    grid remove ${tile_path}.lbl_net_id_1       
    grid remove ${tile_path}.ent_net_id_1       
    grid remove ${tile_path}.lbl_stat_1         
    grid remove ${tile_path}.ent_stat_1         
    grid remove ${tile_path}.lbl_msg_1          
    grid remove ${tile_path}.ent_msg_1          
    grid remove ${tile_path}.lbl_func_1         
    grid remove ${tile_path}.ent_func_a_1       
    grid remove ${tile_path}.ent_func_b_1       
    grid remove ${tile_path}.lbl_chid_1         
    grid remove ${tile_path}.ent_chid_1         
    grid remove ${tile_path}.lbl_bw_1           
    grid remove ${tile_path}.ent_bw_1           
    grid remove ${tile_path}.lbl_prilev_1       
    grid remove ${tile_path}.ent_prilev_1       
    grid remove ${tile_path}.lbl_rfss_id_1      
    grid remove ${tile_path}.ent_rfss_id_1      
    grid remove ${tile_path}.lbl_site_id_1      
    grid remove ${tile_path}.ent_site_id_1      
    grid remove ${tile_path}.lbl_lra_1          
    grid remove ${tile_path}.ent_lra_1          
    grid remove ${tile_path}.lbl_sys_id_1       
    grid remove ${tile_path}.ent_sys_id_1       
    grid remove ${tile_path}.lbl_algid_1        
    grid remove ${tile_path}.ent_algid_1        
    grid remove ${tile_path}.mnu_algid_1
    grid remove ${tile_path}.lbl_digits
    grid remove ${tile_path}.ent_digits
    grid remove ${tile_path}.lbl_tgid_2         
    grid remove ${tile_path}.ent_tgid_2         
    grid remove ${tile_path}.lbl_sid_2          
    grid remove ${tile_path}.ent_sid_2          
    grid remove ${tile_path}.lbl_chan_b_2       
    grid remove ${tile_path}.ent_chanid_b_2     
    grid remove ${tile_path}.ent_channum_b_2    
    grid remove ${tile_path}.lbl_grp_b_2        
    grid remove ${tile_path}.ent_grp_b_2        
    grid remove ${tile_path}.lbl_chan_t_2       
    grid remove ${tile_path}.ent_chanid_t_2     
    grid remove ${tile_path}.ent_channum_t_2    
    grid remove ${tile_path}.lbl_chan_r_2       
    grid remove ${tile_path}.ent_chanid_r_2     
    grid remove ${tile_path}.ent_channum_r_2    
    grid remove ${tile_path}.lbl_did_2          
    grid remove ${tile_path}.ent_did_2          
    grid remove ${tile_path}.lbl_ctime_2        
    grid remove ${tile_path}.ent_ctime_2        
    grid remove ${tile_path}.lbl_txoffset_2     
    grid remove ${tile_path}.ent_txoffset_2     
    grid remove ${tile_path}.lbl_chan_space_2   
    grid remove ${tile_path}.ent_chan_space_2   
    grid remove ${tile_path}.lbl_avail_2        
    grid remove ${tile_path}.ent_avail_2        
    grid remove ${tile_path}.lbl_chan_a_2       
    grid remove ${tile_path}.ent_chanid_a_2     
    grid remove ${tile_path}.ent_channum_a_2    
    grid remove ${tile_path}.lbl_class_a_2      
    grid remove ${tile_path}.ent_class_a_2      
    grid remove ${tile_path}.lbl_chan_2
    grid remove ${tile_path}.ent_chanid_2
    grid remove ${tile_path}.ent_channum_2
    grid remove ${tile_path}.lbl_class_2
    grid remove ${tile_path}.ent_class_2
    grid remove ${tile_path}.lbl_rfss_id_2      
    grid remove ${tile_path}.ent_rfss_id_2      
    grid remove ${tile_path}.lbl_site_id_2      
    grid remove ${tile_path}.ent_site_id_2      
    grid remove ${tile_path}.lbl_src_id_2
    grid remove ${tile_path}.ent_src_id_2
    grid remove ${tile_path}.lbl_kid_2          
    grid remove ${tile_path}.ent_kid_2          
    grid remove ${tile_path}.mnu_kid_2
    grid remove ${tile_path}.lbl_did_3          
    grid remove ${tile_path}.ent_did_3          
    grid remove ${tile_path}.lbl_sid_3          
    grid remove ${tile_path}.ent_sid_3          
    grid remove ${tile_path}.lbl_basefreq_3     
    grid remove ${tile_path}.ent_basefreq_3     
    grid remove ${tile_path}.lbl_chan_b_3       
    grid remove ${tile_path}.ent_chanid_b_3     
    grid remove ${tile_path}.ent_channum_b_3    
    grid remove ${tile_path}.lbl_class_b_3      
    grid remove ${tile_path}.ent_class_b_3      
    grid remove ${tile_path}.lbl_chan_3         
    grid remove ${tile_path}.ent_chanid_3       
    grid remove ${tile_path}.ent_channum_3      
    grid remove ${tile_path}.lbl_class_3        
    grid remove ${tile_path}.ent_class_3        
    grid remove ${tile_path}.lbl_supp_3         
    grid remove ${tile_path}.ent_supp_3         
    
    return
  }


  #***************************************************************************
  #
  #        Name: display_lco_fields
  #
  #  Parameters: tile_path - tile path for all lco frames
  #
  # Description: Display the LCO fields for the selected LCO
  #
  #***************************************************************************
  proc display_lco_fields { tile_path  args } \
  {
    set cmd [namespace code "display_lco_fields_core $tile_path"]
    after cancel $cmd
    after idle $cmd
  }
  proc display_lco_fields_core { tile_path  } \
  {
    variable  Encode
    
    set tile_path  [get_tile  $tile_path]
    
    remove_lco_fields  $tile_path
    
    switch -- $::P25ns::EncodeParm(LCO) \
    {
      "0" -
      "00" { ::Screens::position_widget ${tile_path}.lbl_mfid_1
             ::Screens::position_widget ${tile_path}.ent_mfid_1
             ::Screens::position_widget ${tile_path}.lbl_srv_r_1
             ::Screens::position_widget ${tile_path}.ent_srv_r_1
             ::Screens::position_widget ${tile_path}.lbl_tgid_2
             ::Screens::position_widget ${tile_path}.ent_tgid_2
             ::Screens::position_widget ${tile_path}.lbl_sid_2
             ::Screens::position_widget ${tile_path}.ent_sid_2  }

      "2" -
      "02" { ::Screens::position_widget ${tile_path}.lbl_chan_a_1
             ::Screens::position_widget ${tile_path}.ent_chanid_a_1
             ::Screens::position_widget ${tile_path}.ent_channum_a_1
             ::Screens::position_widget ${tile_path}.lbl_grp_a_1
             ::Screens::position_widget ${tile_path}.ent_grp_a_1
             ::Screens::position_widget ${tile_path}.lbl_chan_b_2
             ::Screens::position_widget ${tile_path}.ent_chanid_b_2
             ::Screens::position_widget ${tile_path}.ent_channum_b_2
             ::Screens::position_widget ${tile_path}.lbl_grp_b_2
             ::Screens::position_widget ${tile_path}.ent_grp_b_2  }

      "3" -
      "03" { ::Screens::position_widget ${tile_path}.lbl_mfid_1
             ::Screens::position_widget ${tile_path}.ent_mfid_1
             ::Screens::position_widget ${tile_path}.lbl_srv_r_1
             ::Screens::position_widget ${tile_path}.ent_srv_r_1
             ::Screens::position_widget ${tile_path}.lbl_did_2
             ::Screens::position_widget ${tile_path}.ent_did_2
             ::Screens::position_widget ${tile_path}.lbl_sid_2
             ::Screens::position_widget ${tile_path}.ent_sid_2  }

      "4" -
      "04" { ::Screens::position_widget ${tile_path}.lbl_srv_l_1
             ::Screens::position_widget ${tile_path}.ent_srv_l_1
             ::Screens::position_widget ${tile_path}.lbl_tgid_1
             ::Screens::position_widget ${tile_path}.ent_tgid_1
             ::Screens::position_widget ${tile_path}.lbl_chan_t_2
             ::Screens::position_widget ${tile_path}.ent_chanid_t_2
             ::Screens::position_widget ${tile_path}.ent_channum_t_2
             ::Screens::position_widget ${tile_path}.lbl_chan_r_2
             ::Screens::position_widget ${tile_path}.ent_chanid_r_2
             ::Screens::position_widget ${tile_path}.ent_channum_r_2  }

      "5" -
      "05" { ::Screens::position_widget ${tile_path}.lbl_srv_l_1
             ::Screens::position_widget ${tile_path}.ent_srv_l_1
             ::Screens::position_widget ${tile_path}.lbl_did_2
             ::Screens::position_widget ${tile_path}.ent_did_2
             ::Screens::position_widget ${tile_path}.lbl_sid_2
             ::Screens::position_widget ${tile_path}.ent_sid_2  }

      "6" -
      "06" { ::Screens::position_widget ${tile_path}.lbl_srv_l_1
             ::Screens::position_widget ${tile_path}.ent_srv_l_1
             ::Screens::position_widget ${tile_path}.lbl_ctime_2
             ::Screens::position_widget ${tile_path}.ent_ctime_2
             ::Screens::position_widget ${tile_path}.lbl_did_3
             ::Screens::position_widget ${tile_path}.ent_did_3 }

      "7" -
      "07" { 
             ::Screens::position_widget ${tile_path}.lbl_digits
	       ::Screens::position_widget ${tile_path}.ent_digits
	       ::Screens::position_widget ${tile_path}.lbl_did_2
	       ::Screens::position_widget ${tile_path}.ent_did_2 
           }

      "F" -
      "0F" { ::Screens::position_widget ${tile_path}.lbl_did_1
             ::Screens::position_widget ${tile_path}.ent_did_1
             ::Screens::position_widget ${tile_path}.lbl_sid_1
             ::Screens::position_widget ${tile_path}.ent_sid_1 }

      "10" -
      "13" -
      "16" { ::Screens::position_widget ${tile_path}.lbl_did_1
             ::Screens::position_widget ${tile_path}.ent_did_1
             ::Screens::position_widget ${tile_path}.lbl_sid_1
             ::Screens::position_widget ${tile_path}.ent_sid_1  }

      "11" -
      "12" { ::Screens::position_widget ${tile_path}.lbl_net_id_1
             ::Screens::position_widget ${tile_path}.ent_net_id_1
             ::Screens::position_widget ${tile_path}.lbl_sys_id_1
             ::Screens::position_widget ${tile_path}.ent_sys_id_1
             ::Screens::position_widget ${tile_path}.lbl_src_id_2
             ::Screens::position_widget ${tile_path}.ent_src_id_2 }

      "14" { ::Screens::position_widget ${tile_path}.lbl_stat_1
             ::Screens::position_widget ${tile_path}.ent_stat_1
             ::Screens::position_widget ${tile_path}.lbl_did_2
             ::Screens::position_widget ${tile_path}.ent_did_2
             ::Screens::position_widget ${tile_path}.lbl_sid_2
             ::Screens::position_widget ${tile_path}.ent_sid_2  }

      "15" { ::Screens::position_widget ${tile_path}.lbl_msg_1
             ::Screens::position_widget ${tile_path}.ent_msg_1
             ::Screens::position_widget ${tile_path}.lbl_did_2
             ::Screens::position_widget ${tile_path}.ent_did_2
             ::Screens::position_widget ${tile_path}.lbl_sid_2
             ::Screens::position_widget ${tile_path}.ent_sid_2  }

      "17" { ::Screens::position_widget ${tile_path}.lbl_func_1
             ::Screens::position_widget ${tile_path}.ent_func_a_1
             ::Screens::position_widget ${tile_path}.ent_func_b_1
             ::Screens::position_widget ${tile_path}.lbl_did_2
             ::Screens::position_widget ${tile_path}.ent_did_2  }

      "18" { ::Screens::position_widget ${tile_path}.lbl_chid_1
             ::Screens::position_widget ${tile_path}.ent_chid_1
             ::Screens::position_widget ${tile_path}.lbl_bw_1
             ::Screens::position_widget ${tile_path}.ent_bw_1
             ::Screens::position_widget ${tile_path}.lbl_txoffset_2
             ::Screens::position_widget ${tile_path}.ent_txoffset_2
             ::Screens::position_widget ${tile_path}.lbl_chan_space_2
             ::Screens::position_widget ${tile_path}.ent_chan_space_2
             ::Screens::position_widget ${tile_path}.lbl_basefreq_3
             ::Screens::position_widget ${tile_path}.ent_basefreq_3  }

      "20" { ::Screens::position_widget ${tile_path}.lbl_prilev_1
             ::Screens::position_widget ${tile_path}.ent_prilev_1
             ::Screens::position_widget ${tile_path}.lbl_avail_2
             ::Screens::position_widget ${tile_path}.ent_avail_2
             ::Screens::position_widget ${tile_path}.lbl_supp_3
             ::Screens::position_widget ${tile_path}.ent_supp_3  }

      "21" { ::Screens::position_widget ${tile_path}.lbl_rfss_id_1
             ::Screens::position_widget ${tile_path}.ent_rfss_id_1
             ::Screens::position_widget ${tile_path}.lbl_site_id_1
             ::Screens::position_widget ${tile_path}.ent_site_id_1
             ::Screens::position_widget ${tile_path}.lbl_chan_a_2
             ::Screens::position_widget ${tile_path}.ent_chanid_a_2
             ::Screens::position_widget ${tile_path}.ent_channum_a_2
             ::Screens::position_widget ${tile_path}.lbl_class_a_2
             ::Screens::position_widget ${tile_path}.ent_class_a_2
             ::Screens::position_widget ${tile_path}.lbl_chan_b_3
             ::Screens::position_widget ${tile_path}.ent_chanid_b_3
             ::Screens::position_widget ${tile_path}.ent_channum_b_3
             ::Screens::position_widget ${tile_path}.lbl_class_b_3
             ::Screens::position_widget ${tile_path}.ent_class_b_3  }

      "22" -
      "23" { ::Screens::position_widget ${tile_path}.lbl_lra_1
 	     ::Screens::position_widget ${tile_path}.ent_lra_1
	     ::Screens::position_widget ${tile_path}.lbl_sys_id_1
	     ::Screens::position_widget ${tile_path}.ent_sys_id_1
	     ::Screens::position_widget ${tile_path}.lbl_rfss_id_2
	     ::Screens::position_widget ${tile_path}.ent_rfss_id_2
	     ::Screens::position_widget ${tile_path}.lbl_site_id_2
	     ::Screens::position_widget ${tile_path}.ent_site_id_2
	     ::Screens::position_widget ${tile_path}.lbl_chan_3
	     ::Screens::position_widget ${tile_path}.ent_chanid_3
	     ::Screens::position_widget ${tile_path}.ent_channum_3
	     ::Screens::position_widget ${tile_path}.lbl_class_3 
	     ::Screens::position_widget ${tile_path}.ent_class_3  }

      "24" { ::Screens::position_widget ${tile_path}.lbl_net_id_1
             ::Screens::position_widget ${tile_path}.ent_net_id_1
             ::Screens::position_widget ${tile_path}.lbl_sys_id_1
             ::Screens::position_widget ${tile_path}.ent_sys_id_1
             ::Screens::position_widget ${tile_path}.lbl_chan_2
             ::Screens::position_widget ${tile_path}.ent_chanid_2
             ::Screens::position_widget ${tile_path}.ent_channum_2
             ::Screens::position_widget ${tile_path}.lbl_class_2
             ::Screens::position_widget ${tile_path}.ent_class_2  }

      "25" { ::Screens::position_widget ${tile_path}.lbl_algid_1
             ::Screens::position_widget ${tile_path}.ent_algid_1
             ::Screens::position_widget ${tile_path}.mnu_algid_1
             ::Screens::position_widget ${tile_path}.lbl_kid_2
             ::Screens::position_widget ${tile_path}.ent_kid_2
             ::Screens::position_widget ${tile_path}.mnu_kid_2
             ::Screens::position_widget ${tile_path}.lbl_did_3
             ::Screens::position_widget ${tile_path}.ent_did_3  }
    }
       
    return
  }


  #***************************************************************************
  #
  #        Name: update_lcotext
  #
  #  Parameters: none
  #
  # Description: updates LCO text and short LCO text
  #
  #***************************************************************************
  proc update_lcotext { args } \
  {
    set cmd [namespace code "update_lcotext_core"]
    after cancel $cmd
    after idle $cmd
  }
  proc update_lcotext_core { } \
  {
    variable Encode

    # TOPLEVEL TILE LCO TEXT
    if { [info exists ::P25ns::LCO_strings($::P25ns::EncodeParm(LCO))] } \
    {
      set Encode(LCO_text)  $::P25ns::LCO_strings($::P25ns::EncodeParm(LCO))
    } \
    else \
    {
      set Encode(LCO_text) "<reserved>"
    }

    # QUARTER TILE LCO TEXT
    if { [info exists ::P25ns::LCO_short_strings($::P25ns::EncodeParm(LCO))] } \
    {
      set Encode(LCO_short_text)  $::P25ns::LCO_short_strings($::P25ns::EncodeParm(LCO))
    } \
    else \
    {
      set Encode(LCO_short_text) "<reserved>"
    }
  }


  #***************************************************************************
  #
  #        Name: set_status_text
  #
  #  Parameters: none
  #
  # Description: Sets the status text to be displayed
  #
  #***************************************************************************
  proc set_status_text { } \
  {
    variable Encode

    if { $::P25ns::EncodeParm(HDR_SSYM) == "" } \
    {
      set ::P25ns::EncodeParm(HDR_SSYM)  0
    }
     
    P25 encode field status 0x$::P25ns::EncodeParm(HDR_SSYM)
    
    return
  }


  #***************************************************************************
  #
  #        Name: update_service_options
  #
  #  Parameters: none
  #
  # Description: set emergency service bit in service options
  #
  #***************************************************************************
  proc update_service_options { } \
  {
    if { $::P25ns::EncodeParm(EMG) } \
    {
      set ::P25ns::EncodeParm(SERVICEOP) 80
    } \
    else \
    {
      set ::P25ns::EncodeParm(SERVICEOP) 00
    }
  }


  #***************************************************************************
  #
  #        Name: validate_values
  #
  #  Parameters: none
  #
  # Description: Validates the Encode parameters if set to a null value
  #
  #***************************************************************************
  proc validate_values { } \
  {
    if { $::P25ns::EncodeParm(LCO) == "" } \
    {
      set ::P25ns::EncodeParm(LCO)  0
    }

    if { $::P25ns::EncodeParm(P) == "" } \
    {
      set ::P25ns::EncodeParm(P)  0
    }

    if { $::P25ns::EncodeParm(SF) == "" } \
    {
      set ::P25ns::EncodeParm(SF)  0
    }

    if { $::P25ns::EncodeParm(LCO_MFID) == "" } \
    {
      set ::P25ns::EncodeParm(LCO_MFID)  0
    }

    if { $::P25ns::EncodeParm(SERVICEOP) == "" } \
    {
      set ::P25ns::EncodeParm(SERVICEOP)  0
    }

    if { $::P25ns::EncodeParm(RFSUBID) == "" } \
    {
      set ::P25ns::EncodeParm(RFSUBID)  0
    }

    if { $::P25ns::EncodeParm(SITEID) == "" } \
    {
      set ::P25ns::EncodeParm(SITEID)  0
    }

    if { $::P25ns::EncodeParm(SERVCLASS_A) == "" } \
    {
      set ::P25ns::EncodeParm(SERVCLASS_A)  0
    }

    if { $::P25ns::EncodeParm(SERVCLASS_B) == "" } \
    {
      set ::P25ns::EncodeParm(SERVCLASS_B)  0
    }

    if { $::P25ns::EncodeParm(LOCREGAREA) == "" } \
    {
      set ::P25ns::EncodeParm(LOCREGAREA)  0
    }

    if { $::P25ns::EncodeParm(SERVCLASS) == "" } \
    {
      set ::P25ns::EncodeParm(SERVCLASS)  0
    }

    if { $::P25ns::EncodeParm(CHANNUM) == "" } \
    {
      set ::P25ns::EncodeParm(CHANNUM)  0
    }

    if { $::P25ns::EncodeParm(GRPADDR) == "" } \
    {
      set ::P25ns::EncodeParm(GRPADDR)  0
    }

    if { $::P25ns::EncodeParm(TARGETADDR) == "" } \
    {
      set ::P25ns::EncodeParm(TARGETADDR)  0
    }

    if { $::P25ns::EncodeParm(SOURCEADDR) == "" } \
    {
      set ::P25ns::EncodeParm(SOURCEADDR)  0
    }

    if { $::P25ns::EncodeParm(GRPADDR) == "" } \
    {
      set ::P25ns::EncodeParm(GRPADDR)  0
    }

    if { $::P25ns::EncodeParm(CHANID_A) == "" } \
    {
      set ::P25ns::EncodeParm(CHANID_A)  0
    }

    if { $::P25ns::EncodeParm(CHANNUM_A) == "" } \
    {
      set ::P25ns::EncodeParm(CHANNUM_A)  0
    }

    if { $::P25ns::EncodeParm(GRPADDR_A) == "" } \
    {
      set ::P25ns::EncodeParm(GRPADDR_A)  0
    }

    if { $::P25ns::EncodeParm(CHANID_B) == "" } \
    {
      set ::P25ns::EncodeParm(CHANID_B)  0
    }

    if { $::P25ns::EncodeParm(CHANNUM_B) == "" } \
    {
      set ::P25ns::EncodeParm(CHANNUM_B)  0
    }

    if { $::P25ns::EncodeParm(GRPADDR_B) == "" } \
    {
      set ::P25ns::EncodeParm(GRPADDR_B)  0
    }

    if { $::P25ns::EncodeParm(CHANID_T) == "" } \
    {
      set ::P25ns::EncodeParm(CHANID_T)  0
    }

    if { $::P25ns::EncodeParm(CHANNUM_T) == "" } \
    {
      set ::P25ns::EncodeParm(CHANNUM_T)  0
    }

    if { $::P25ns::EncodeParm(CHANID_R) == "" } \
    {
      set ::P25ns::EncodeParm(CHANID_R)  0
    }

    if { $::P25ns::EncodeParm(CHANNUM_R) == "" } \
    {
      set ::P25ns::EncodeParm(CHANNUM_R)  0
    }

    if { $::P25ns::EncodeParm(CTIMER) == "" } \
    {
      set ::P25ns::EncodeParm(CTIMER)  0
    }

    if { $::P25ns::EncodeParm(NETID) == "" } \
    {
      set ::P25ns::EncodeParm(NETID)  0
    }

    if { $::P25ns::EncodeParm(SYSID) == "" } \
    {
      set ::P25ns::EncodeParm(SYSID)  0
    }

    if { $::P25ns::EncodeParm(SRCID) == "" } \
    {
      set ::P25ns::EncodeParm(SRCID)  0
    }

    if { $::P25ns::EncodeParm(STSUPDATE) == "" } \
    {
      set ::P25ns::EncodeParm(STSUPDATE)  0
    }

    if { $::P25ns::EncodeParm(MSG) == "" } \
    {
      set ::P25ns::EncodeParm(MSG)  0
    }

    if { $::P25ns::EncodeParm(EXTFUNCT_A) == "" } \
    {
      set ::P25ns::EncodeParm(EXTFUNCT_A)  0
    }

    if { $::P25ns::EncodeParm(EXTFUNCT_B) == "" } \
    {
      set ::P25ns::EncodeParm(EXTFUNCT_B)  0
    }

    if { $::P25ns::EncodeParm(BW) == "" } \
    {
      set ::P25ns::EncodeParm(BW)  0
    }

    if { $::P25ns::EncodeParm(XMITOFF) == "" } \
    {
      set ::P25ns::EncodeParm(XMITOFF)  0
    }

    if { $::P25ns::EncodeParm(CHANSPACE) == "" } \
    {
      set ::P25ns::EncodeParm(CHANSPACE)  0
    }

    if { $::P25ns::EncodeParm(BASEFREQ) == "" } \
    {
      set ::P25ns::EncodeParm(BASEFREQ)  0
    }

    if { $::P25ns::EncodeParm(PRIREQ) == "" } \
    {
      set ::P25ns::EncodeParm(PRIREQ)  0
    }

    if { $::P25ns::EncodeParm(SSAVAIL) == "" } \
    {
      set ::P25ns::EncodeParm(SSAVAIL)  0
    }

    if { $::P25ns::EncodeParm(SSSUPPORT) == "" } \
    {
      set ::P25ns::EncodeParm(SSSUPPORT)  0
    }

    if { $::P25ns::EncodeParm(LCO_KID) == "" } \
    {
      set ::P25ns::EncodeParm(LCO_KID)  0
    }

    if { $::P25ns::EncodeParm(DIG0_4) == "" } \
    {
      set ::P25ns::EncodeParm(DIG0_4)  0
    }

    if { $::P25ns::EncodeParm(DIG5_9) == "" } \
    {
      set ::P25ns::EncodeParm(DIG5_9)  0
    }
    
    return
  }


  #****************************************************************************
  #        Name: Watch_pattern
  #  Parameters: tile base, args from trace (ignored)
  # Description: Change what we show in the small downlink tile based upon the
  #              generated pattern
  #****************************************************************************
  proc Watch_pattern { tile args } \
  {
     set cmd "DownlinkUI::Watch_pattern_core $tile"
  
     after cancel $cmd
     after idle $cmd
  }
  
  
  #****************************************************************************
  #        Name: Watch_pattern_core
  #  Parameters: tile base
  # Description: Change what we show in the small downlink tile based upon the
  #              generated pattern
  #****************************************************************************
  proc Watch_pattern_core { tile } \
  {
     set frames  [option get $tile pattern_frames {}]
     
     switch -- $::Generator::State(p25mode) \
     {
        "OFF"          -
        "VOICE"        -
        "SILENCE"      -
        "CALIBRATION"  -
        "1011"         -
        "SPEECH"       { 
                         grid forget $tile.frm_not_applicable
                         
                         foreach f $frames \
                         {
                           ::Screens::position_widget $tile.$f
                         }
                         .p25downlink.btn_downlink config -state normal
                       }
        
        "STD 1011"     -
        "STD SILENCE"  -
        "STD INTFRNC"  -
        "STD BUSY"     -
        "STD IDLE"     -
        "STD CAL"      -
        "STD LDU1TRG"  -
        "STD NOTRIG"   -
        "STD LDU2TRG"  -
        "STD 511"      -
        "STD SYMRATE"  -
        "STD LOWDEV"   -
        "STD FIDPAT"   -
        "STD FIDSPECT" -
        "TRUNKING"     -
        default        {  
                         foreach f $frames \
                         {
                           grid forget $tile.$f
                         }
                          
                         ::Screens::position_widget $tile.frm_not_applicable
                         .p25downlink.btn_downlink config -state disabled
                       }
     }
  }


  #****************************************************************************
  #        Name: set_algid
  #  Parameters:
  # Description: Sets the ALGID value from either the menu or the entry widget
  #****************************************************************************
  proc set_algid { widget_type } \
  {
    #puts "set_algid:  $widget_type"
    
    variable  StateUI
    
    switch -- $widget_type \
    {
      "menu"     { # Get the menu selection and send down as a number 
                 
                   switch -- $::P25ns::EncodeParm(HDR_ALGID_txt) \
                   {
                     "CLEAR"  { # Send '0x80' 
                                P25 encode field algid "0x80"   }
                   
                     "DES"    { # Send '0x81'
                                P25 encode field algid "0x81"   }
                   
                     "AES"    { # Send '0x84'
                                P25 encode field algid "0x84"   }
                   
                     default  {  }
                   }
                 }
      
      "entry"    { # Send down the number
                   P25 encode field algid "0x$::P25ns::EncodeParm(HDR_ALGID)" }
      
      default    {  }
    }
    
    
    return
  }

  #****************************************************************************
  #        Name: set_lco_algid
  #  Parameters:
  # Description: Sets the LCO ALGID value from the menu
  #****************************************************************************
  proc set_lco_algid { } \
  {
     switch -- $::P25ns::EncodeParm(LCO_ALGID_txt) \
     {
        "CLEAR"  { set ::P25ns::EncodeParm(LCO_ALGID) "80" }
        "DES"    { set ::P25ns::EncodeParm(LCO_ALGID) "81" }
        "AES"    { set ::P25ns::EncodeParm(LCO_ALGID) "84" }
        default  {  }
     }
  }

  #****************************************************************************
  #        Name: set_lco_algid_txt
  #  Parameters:
  # Description: Sets the LCO ALGID value from the menu
  #****************************************************************************
  proc set_lco_algid_txt { } \
  {
    switch -- $::P25ns::EncodeParm(LCO_ALGID) \
    {
      "80"  { set ::P25ns::EncodeParm(LCO_ALGID_txt) "CLEAR" }
      "81"    { set ::P25ns::EncodeParm(LCO_ALGID_txt) "DES" }
      "84"    { set ::P25ns::EncodeParm(LCO_ALGID_txt) "AES" }

      default  { set ::P25ns::EncodeParm(LCO_ALGID_txt) "Unknown" }
    }
     
    return
  }


  #****************************************************************************
  #        Name: getSaveState
  #  Parameters: none
  # Description: This function returns a list of TCL commands which can be used
  #              to set the SzSnet_Repeat_SimUI tiles to its current state
  #****************************************************************************
  proc getSaveState { } \
  {
    variable  StateUI

    set returnList \
    {
    
    }

    return  [subst -nocommands $returnList]
  }


  #****************************************************************************
  #        Name: Sync
  #  Parameters: none
  # Description: Write state variables to SzSnet_Repeat_SimUI
  #****************************************************************************
  proc Sync { } \
  {
    eval [getSaveState]
  }
  
  trace variable ::Generator::State(mod) w  "::P25ns::send_lco_0"
  
}; # End of DownlinkUI namespace



#****************************************************************************
# This registers the tiles with the list for saving instrument states
#****************************************************************************
namespace eval ::SavedStates \
{
  variable tiles   

  lappend tiles DownlinkUI
}


############################################################################
# P25 downlink data tile
############################################################################

namespace eval View \
{
   lappend PreCreate DownlinkUI
}



namespace eval Key_LoaderUI \
{
  # Key Loader variables
  variable  keyset_list  {}
  
  variable  saved_flag       1;  # Initialized to indicate the data has been saved
  variable  temp_saved_flag  1
  variable  last_focus
  
  variable  StateUI

  set StateUI(keyloader_mode)       "MANUAL LOAD"
  set StateUI(keyloader_mode_menu)  { "MANUAL LOAD"  "AUTO LOAD" }
  set StateUI(keyid_manual)         0000
  set StateUI(keyid_manual_value)   0
  set StateUI(algid_manual)         "DES"
  set StateUI(algid_manual_value)   0x80
  set StateUI(algid_manual_menu)    { "DES"  "AES" }
  set StateUI(format_manual)        "TEK"
  set StateUI(format_menu)          { "KEK"  "TEK" }
  set StateUI(size_manual)          0
  set StateUI(sln_manual_kek)       61440
  set StateUI(sln_manual_tek)       10
  set StateUI(sln_manual_value)     10
  set StateUI(keyset_manual_value)  0
  set StateUI(key_manual_value)     0

  set StateUI(key_manual_des)       0
  set StateUI(key_manual_aes_1)     0
  set StateUI(key_manual_aes_2)     0
  set StateUI(key_manual_des,def)   "****************"
  set StateUI(key_manual_aes_1,def) "********************************"
  set StateUI(key_manual_aes_2,def) "********************************"


  set StateUI(name_manual)          ""
  
  set StateUI(cancel_or_use_keys)  0;  # Flag to determine if the user has 
                                       # pressed the USE KEYS or CANCEL button
                                       # to close the window.
                                       # will be 0 if the user has clicked on 
                                       # the window 'X' to close the window

  set StateUI(onchange_handle)     0

  # Change the variable 'temp_new_key' when the engine variable is defined
  variable  temp_new_key  ""

  variable  scroll_on  0
  variable  sln        -1
  variable  usrResponse ""
  variable  keyloader_done_flag

  #***************************************************************************
  #
  #  Key Loader procedures
  #
  #***************************************************************************
  #***************************************************************************
  #  Name: get_tile
  #
  #  Parameters: any child widget of the tile
  #  Description: Returns the widget or parent which has the class of KEY_LOADER
  #               Returns "" if there is an error
  #***************************************************************************
  proc get_tile { widget } \
  {
     if { [winfo class $widget] == "KEY_LOADER" } { return $widget }

     # initialize value to force one pass through the loop
     set top_level ""
   
     while { $top_level != $widget } \
     {
        set widget [winfo parent $widget]
        switch -- [winfo class $widget] \
        {
           "KEY_LOADER"  { return $widget }
        }
        set top_level  [winfo toplevel $widget]
     }
     return ""     
  }

  #****************************************************************************
  #
  #        Name:  display_key_loader
  #
  #  Parameters: widget - invoking widget
  #
  # Description:  Displays the key loader toplevel
  #
  #****************************************************************************
  proc display_key_loader { widget } \
  {
    #puts "display_key_loader:  $widget"

    variable  StateUI
    variable  keyset_list
    variable  last_focus
    variable  saved_flag
    variable  temp_saved_flag
    
    if { ![IFR_options  is_installed  KEY_MGT] } \
    {
      return
    }
    
    set last_focus  $widget
   
    set top_level  [winfo toplevel  $widget]
       
    set tile_path  [name_child  $top_level  key_loader]

    if { ![winfo exists $tile_path] } \
    {
      set tile_path  [Screens::create_toplevel_tile  $tile_path  KEY_LOADER]
    }
        
    # Bind the <<Destroy>> event
    bind $tile_path  <<Destroy>>  \
                      "Key_LoaderUI::destroy_key_loader  $tile_path  $widget"
        
    # Bind the textbox to the scrollbar and vice versa
    ${tile_path}.scrl_keylist  config  \
                                    -command "${tile_path}.txt_keylist yview"
                       
    ${tile_path}.txt_keylist  config  \
                              -yscrollcommand "${tile_path}.scrl_keylist set"

    trace variable ::Key_LoaderUI::StateUI(keyloader_mode)  w  \
                                      "Key_LoaderUI::update_mode  $tile_path"

    trace variable ::Key_LoaderUI::keyset_list  w  \
                                    "Key_LoaderUI::update_keyset  $tile_path"

    # Update the display to the current mode
    update_mode_core  $tile_path
    
    # Get the present keys
    set keyset_list  [Keys get]
    
    
    # Restore the 'saved_flag' state
    set saved_flag  $temp_saved_flag
              
    # Set up the -onchange
    set StateUI(onchange_handle)  [Keys -onchange  "Key_LoaderUI::auto_add $tile_path"]
                                    
    safe_focus_force ${tile_path}.btn_download
  }


  #**************************************************************************
  #
  #        Name: destroy_key_loader
  #
  #  Parameters: tile_path - tile path of the invoking widget
  #              old_focus - ????????????????
  #
  # Description: Handles the <<Destroy>> event of the Key Loader toplevel
  #
  #**************************************************************************
  proc destroy_key_loader { tile_path  old_focus } \
  {
    #puts "destroy_key_loader:  $tile_path  $old_focus"
       
    variable  StateUI
    variable  saved_flag
    variable  temp_saved_flag
    variable  keyloader_done_flag
    
    set tile_path  [get_tile  $tile_path]
    
    set parent_path  [winfo parent  $tile_path]
    
    trace vdelete ::Key_LoaderUI::StateUI(keyloader_mode)  w  \
                                      "Key_LoaderUI::update_mode  $tile_path"

    trace vdelete ::Key_LoaderUI::keyset_list  w  \
                                    "Key_LoaderUI::update_keyset  $tile_path"

    # Disable the UUT Test Port
    Keys uut  0

    # Unbind the -onchange
    if { $StateUI(onchange_handle) > 0 } \
    {
      # Catch the -unbind command for the case where the command 
      # has been unbound
      Keys -unbind  $StateUI(onchange_handle)
      
      set StateUI(onchange_handle)  0
    }
    
    # If the user has clicked on the window 'X' to close the window,
    # prompt the user to ask if they want to use the keys

    if { $StateUI(cancel_or_use_keys) == 0 } \
    {
      set title  "Use keys?"

      set prompt_text  "Use the displayed keys?"
    
      if { $parent_path == "." } \
      {
        set window_name  ".request"
      } \
      else \
      {
        set window_name  ${parent_path}.request
      }
    
      set prompt_return  [tk_dialog  $window_name  $title  \
                                      $prompt_text  question  1  "YES"  "NO"]
    
      switch -- $prompt_return \
      {
        0 { # YES selected, download the keyset
            download_keyset }
    
        1 { }
      }
    } \
    else \
    {
      # Clear the close method flag
      set StateUI(cancel_or_use_keys)  0
    }

    # Clear the KEY field to default
    set StateUI(key_manual_des)    $StateUI(key_manual_des,def)
    set StateUI(key_manual_aes_1)  $StateUI(key_manual_aes_1,def)
    set StateUI(key_manual_aes_2)  $StateUI(key_manual_aes_2,def)

    # Store the 'saved_flag' state
    set temp_saved_flag  $saved_flag

    # This flag exists to signal other functions that the keyloader display
    # has been removed. For example, the autotest needs to update display info
    # when keys have changed, and there is no nice way to know when to retrieve
    # the keys for a status check
    set keyloader_done_flag 1
    
    after 100  "safe_focus_force  $old_focus"
   
    return
  }


  #**************************************************************************
  #
  #        Name: update_mode
  #
  #  Parameters: widget - tile_path of the invoking widget
  #
  # Description: Updates the display to the mode of the key loader
  #
  #**************************************************************************
  proc update_mode { widget  args } \
  {
     set cmd [namespace code "update_mode_core $widget"]
     after cancel $cmd
     after idle $cmd
  }
  proc update_mode_core { widget } \
  {
    #puts "update_mode:  $widget"
    
    variable  StateUI
    
    set tile_path  [get_tile  $widget]
    
    switch -- $StateUI(keyloader_mode) \
    {
      "AUTO LOAD"    { # Remove the manual entry fields
                       grid remove  ${tile_path}.lbl_keyid
                       grid remove  ${tile_path}.ent_keyid
                       grid remove  ${tile_path}.lbl_keyid_desc
                       grid remove  ${tile_path}.lbl_algid
                       grid remove  ${tile_path}.mnu_algid
                       grid remove  ${tile_path}.lbl_algid_desc
                       grid remove  ${tile_path}.lbl_format
                       grid remove  ${tile_path}.mnu_format
                       grid remove  ${tile_path}.lbl_size
                       grid remove  ${tile_path}.ent_size
                       grid remove  ${tile_path}.lbl_size_desc
                       grid remove  ${tile_path}.lbl_sln
                       grid remove  ${tile_path}.ent_sln_kek
                       grid remove  ${tile_path}.ent_sln_tek
                       grid remove  ${tile_path}.lbl_key_des
                       grid remove  ${tile_path}.ent_key_des
                       grid remove  ${tile_path}.lbl_key_des_desc
                       grid remove  ${tile_path}.lbl_key_aes_1
                       grid remove  ${tile_path}.ent_key_aes_1
                       grid remove  ${tile_path}.lbl_key_aes_desc_1
                       grid remove  ${tile_path}.lbl_key_aes_2
                       grid remove  ${tile_path}.ent_key_aes_2
                       grid remove  ${tile_path}.lbl_key_aes_desc_2
                       grid remove  ${tile_path}.lbl_name
                       grid remove  ${tile_path}.ent_name
                       grid remove  ${tile_path}.lbl_name_desc 
                       grid remove  ${tile_path}.btn_add
                       grid remove  ${tile_path}.btn_remove
                       
                       # Increase the height of the textbox and scrollbar
                       grid config ${tile_path}.txt_keylist -row 30 -rowspan 250
                       grid config ${tile_path}.scrl_keylist -row 30 -rowspan 250 
                      
                       
                       # Enable the UUT Test Port
                       Keys uut  1 
                     }
                       
      
      "MANUAL LOAD"  { # Display the manual entry fields
                       Screens::position_widget  ${tile_path}.lbl_keyid
                       Screens::position_widget  ${tile_path}.ent_keyid
                       Screens::position_widget  ${tile_path}.lbl_keyid_desc
                       Screens::position_widget  ${tile_path}.lbl_algid
                       Screens::position_widget  ${tile_path}.mnu_algid
                       Screens::position_widget  ${tile_path}.lbl_algid_desc
                       Screens::position_widget  ${tile_path}.lbl_format
                       Screens::position_widget  ${tile_path}.mnu_format
                       Screens::position_widget  ${tile_path}.lbl_size
                       Screens::position_widget  ${tile_path}.ent_size
                       Screens::position_widget  ${tile_path}.lbl_size_desc
                       Screens::position_widget  ${tile_path}.lbl_sln
                       Screens::position_widget  ${tile_path}.lbl_name
                       Screens::position_widget  ${tile_path}.ent_name
                       Screens::position_widget  ${tile_path}.lbl_name_desc 

                       update_key_fields  $tile_path 
                       update_sln_field   $tile_path
                       
                       # Disable the UUT Test Port
                       Keys uut 0
                     }
                       
    }
                           
    return
  }
  
  
  #**************************************************************************
  #
  #        Name: update_key_fields
  #
  #  Parameters: widget - tile_path of the invoking widget
  #
  # Description: Updates the display of the Key entry field(s) to changes in
  #              the ALGID
  #
  #**************************************************************************
  proc update_key_fields { widget } \
  {
    #puts "update_key_fields:  $widget"
    
    variable  StateUI

    set tile_path  [get_tile  $widget]
    
    grid remove  ${tile_path}.lbl_key_des
    grid remove  ${tile_path}.ent_key_des
    grid remove  ${tile_path}.lbl_key_des_desc
    
    grid remove  ${tile_path}.lbl_key_aes_1
    grid remove  ${tile_path}.ent_key_aes_1
    grid remove  ${tile_path}.lbl_key_aes_desc_1
    grid remove  ${tile_path}.lbl_key_aes_2
    grid remove  ${tile_path}.ent_key_aes_2
    grid remove  ${tile_path}.lbl_key_aes_desc_2

    switch -- $StateUI(algid_manual) \
    {
      "DES"  { # Reduce the room for the extra KEY field
               grid config ${tile_path}.txt_keylist -row 180 -rowspan 100
               grid config ${tile_path}.scrl_keylist -row 180 -rowspan 100
               
               grid config ${tile_path}.btn_add -row 150
               grid config ${tile_path}.btn_remove -row 150
               
               # Configure the KEY entry field for 16 characters 
               Screens::position_widget  ${tile_path}.lbl_key_des
               Screens::position_widget  ${tile_path}.ent_key_des
               Screens::position_widget  ${tile_path}.lbl_key_des_desc 
              
               # Set the default value (all asterisks)
               set StateUI(key_manual_des)  $StateUI(key_manual_des,def)

               set StateUI(size_manual)  64 }
      
      "AES"  { # Make room for the extra KEY field
               grid config ${tile_path}.txt_keylist -row 210 -rowspan 70
               grid config ${tile_path}.scrl_keylist -row 210 -rowspan 70
               
               grid config ${tile_path}.btn_add -row 180
               grid config ${tile_path}.btn_remove -row 180
               
               # Configure the KEY entry field for 64 characters each
               Screens::position_widget  ${tile_path}.lbl_key_aes_1
               Screens::position_widget  ${tile_path}.ent_key_aes_1
               Screens::position_widget  ${tile_path}.lbl_key_aes_desc_1
               Screens::position_widget  ${tile_path}.lbl_key_aes_2
               Screens::position_widget  ${tile_path}.ent_key_aes_2
               Screens::position_widget  ${tile_path}.lbl_key_aes_desc_2
              
               # Set the default value (all asterisks)
               set StateUI(key_manual_aes_1)  $StateUI(key_manual_aes_1,def)
               set StateUI(key_manual_aes_2)  $StateUI(key_manual_aes_2,def)

               set StateUI(size_manual)  256 }
    }
    
    return
  }


  #**************************************************************************
  #
  #        Name: update_sln_field
  #
  #  Parameters: widget - tile_path of the invoking widget
  #
  # Description: Updates the display of the SLN field(s) to changes in
  #              the FORMAT
  #
  #**************************************************************************
  proc update_sln_field { widget } \
  {
    #puts "update_sln_field:  $widget"
    
    variable  StateUI

    set tile_path  [get_tile  $widget]
    
    grid remove  ${tile_path}.ent_sln_kek
    grid remove  ${tile_path}.ent_sln_tek

    switch -- $StateUI(format_manual) \
    {
      "KEK"  { # Configure the KEY entry field for KEK SLN
               Screens::position_widget  ${tile_path}.ent_sln_kek 
               
               # Disable the ALGID menu, only DES is allowed
               ${tile_path}.mnu_algid.menu  invoke  "DES"
               
               ${tile_path}.mnu_algid  config -state disabled }
      
      "TEK"  { # Configure the KEY entry field for TEK SLN
               Screens::position_widget  ${tile_path}.ent_sln_tek

               # Enable the ALGID menu
               ${tile_path}.mnu_algid  config -state normal }
    }

    return
  }


  #**************************************************************************
  #
  #        Name: auto_add
  #
  #  Parameters:
  #
  # Description: Adds an automatically entered key to the Keyset
  #
  #**************************************************************************
  proc auto_add { tile_path } \
  {
    # did we get a protocol error?
    set err [Keys err]
    if {$err != ""} \
    {
        set title "Communications error"
        set prompt_text  "Key fill error:\n$err"
        if [string match "ASN protocol option not installed" $err] \
        {
           append prompt_text "\n\nYou need to:\nInstall Option #20\nSwitch your KFD to APCO-25 mode\nor use a new KFD"
        }
    
        set window_name  ${tile_path}.instruction
    
        tk_dialog  $window_name  $title $prompt_text  info  0  "DISMISS"
        return
    }
   
    variable  keyset_list

    set keyset_list  [Keys get]
  }

  #**************************************************************************
  #
  #        Name: manual_add
  #
  #  Parameters: widget - tile path of the invoking widget
  #
  # Description: Adds a manually entered key to the Keyset
  #
  #**************************************************************************
  proc manual_add { widget } \
  {
    set cmd [namespace code "manual_add_core $widget"]
    after cancel $cmd
    after idle $cmd
  }
  proc manual_add_core { widget } \
  {
    #puts "manual_add_core:  $widget"
    
    variable  StateUI
    variable  keyset_list
    
    # Do some sanity checks - block trying to ADD an unentered key
    switch -- $StateUI(algid_manual) \
    {
      "DES"  { 
                if { $StateUI(key_manual_des) == "****************" } \
                {
                   return 
                }
             }
      
      "AES"  { 
                if { $StateUI(key_manual_aes_1) == "********************************" } \
                {
                   return 
                }
             }
    }

    # Determine if the new key KEYID and ALGID exists in the Keyset
    check_for_key $widget
  }

  #**************************************************************************
  #  Name: cvt_user_values
  #
  #  Parameters:  None
  #  Description: This converts the user entered values into variable data
  #               formatted for the engine
  #  Return:      None
  #**************************************************************************
  proc cvt_user_values { } \
  {
     variable  StateUI
     variable  keyset_list

     # Convert HEX values
     set StateUI(keyid_manual_value) "0x$StateUI(keyid_manual)"

     # Determine various values from the format string
     switch -- $StateUI(format_manual) \
     {
        "KEK"  { set StateUI(format_manual_value)  "0x80" 
                 set StateUI(sln_manual_value)  $StateUI(sln_manual_kek) 
                 switch -- $StateUI(algid_manual) \
                 {
                    "DES"  { set StateUI(keyset_manual_value)  255 
                             set StateUI(algid_manual_value)  "0x81" 
                             set StateUI(key_manual_value)  [format "0x%s" $StateUI(key_manual_des)]
                           }
      
                    "AES"  { # Not allowed 
                             return
                           }
                  }
               }
      
        "TEK"  { set StateUI(format_manual_value)  "0x00"
                 set StateUI(sln_manual_value)  $StateUI(sln_manual_tek)
                 switch -- $StateUI(algid_manual) \
                 {
                    "DES"  { set StateUI(keyset_manual_value)  1
                             set StateUI(algid_manual_value)  "0x81"
                             set StateUI(key_manual_value)  [format "0x%s" $StateUI(key_manual_des)]
                           }

                    "AES"  { set StateUI(keyset_manual_value)  2 
                             set StateUI(algid_manual_value)  "0x84" 
                             set StateUI(key_manual_value)  [format "0x%s%s" \
                                      $StateUI(key_manual_aes_1) $StateUI(key_manual_aes_2)]  
                           }
                  }
               }               
     }

     # Limit the length of the key name to 31 characters
     set str_length  [string length "$StateUI(name_manual)"]
     if { $str_length > 31 } \
     {
        set name_manual  [string range "$StateUI(name_manual)" 0  30]
     } \
     else \
     {
        set name_manual  "$StateUI(name_manual)"
     }
     set StateUI(name_manual) $name_manual

     # Add the length of the name string to the format value
     # The "format" variable is a combination of data fields: see "keyset.h"
     #   0 - 0x1F        Length of name field
     #   0x20            Delete / Add mask
     #   0x40            Checksum include flag
     #   0x80            Key type: 0 = TEK, 1 = KEK
     set str_length  [string length "$StateUI(name_manual)"]
     set StateUI(format_manual_value)  [expr $StateUI(format_manual_value) + $str_length]
  }

  #**************************************************************************
  #  Name: add_replace_key
  #
  #  Parameters:  operation     Code for what needs to be done
  #                             SKIP = Do nothing - user canceled operation
  #                             1 = Add a new key
  #                             2 = Replace the key in the specified slot
  #                             3 = Replace a key in a different slot
  #                                 o Delete key in slot "old_sln"
  #                                 o Add key in a new slot
  #               old_sln       SLN of the old key
  #  Description: This function adds or replaces keys. It uses the information
  #               set up by the user to get the Algid, Keyid, Name and SLN.
  #               It uses information provided to the routine when it needs to
  #               replace a key in a different slot.
  #  Return:      None
  #**************************************************************************
  proc add_replace_key { operation old_sln } \
  {
     variable  StateUI
     variable  keyset_list

     #puts "add_replace_key: $operation $old_sln"
     if { $operation == "REPLACE_OLD_SLN" } \
     {
        # Delete the old key from the database
        set format_temp [expr $StateUI(format_manual_value) | 0x20]
        Keys  modify  $StateUI(keyset_manual_value)         \
                      $StateUI(keyid_manual_value)          \
                      $StateUI(algid_manual_value)          \
                      $format_temp                          \
                      $StateUI(size_manual)                 \
                      $StateUI(key_manual_value)            \
                      $StateUI(name_manual)                 \
                      $old_sln
     }

     if { ($operation == "ADD") || ($operation == "REPLACE_SAME_SLN") || \
          ($operation == "REPLACE_OLD_SLN") } \
     {
        # Add or Replace the key in a specific slot
        Keys  modify  $StateUI(keyset_manual_value)         \
                      $StateUI(keyid_manual_value)          \
                      $StateUI(algid_manual_value)          \
                      $StateUI(format_manual_value)         \
                      $StateUI(size_manual)                 \
                      $StateUI(key_manual_value)            \
                      $StateUI(name_manual)                 \
                      $StateUI(sln_manual_value)
     }

     # Clear the KEY field to default
     set StateUI(key_manual_des)    $StateUI(key_manual_des,def)
     set StateUI(key_manual_aes_1)  $StateUI(key_manual_aes_1,def)
     set StateUI(key_manual_aes_2)  $StateUI(key_manual_aes_2,def)
  }

  #**************************************************************************
  #  Name: check_for_key
  #
  #  Parameters:  widget - tile_path of the invoking widget
  #  Description: This checks to see if the new key matches an existing key
  #               o Matching KeyID/AlgID are not allowed because the 2975 does
  #                 not have a way of selecting which one to use
  #               o Matching AlgID/SLN are not allowed because the DSP only
  #                 allows one key per algorithm in a slot
  #               o The format is checked to insure that a KEK does not block
  #                 the addition of a TEK, since KEKs will be discarded when
  #                 the set of keys is sent to the DSP
  #  Return:      -1 = Cancel the request to add the key
  #               -2 = No conflict
  #               -3 = Need to replace key in the specified slot
  #               +xxx = SLN of the slot where there is an AlgID/KeyID match
  #                      This key must be deleted before the insert
  #**************************************************************************
  proc check_for_key { widget } \
  {
     variable  keyset_list
     variable  StateUI

     set tile_path  [get_tile  $widget]
     if { ![winfo exists $tile_path] } \
     {
        return
     }

     # Insure that the converted user values are current
     cvt_user_values

     foreach { data_set } $keyset_list \
     {
        foreach { data_subset } $data_set \
        {
           foreach  { grpid  keyid  algid  format  size  key  name  sln }  $data_subset \
           {
              # Since KEKs will never be sent to the DSP, ignore matches if one key is
              # a KEK and the other is a TEK
              set user_type [expr $StateUI(format_manual_value) & 0x80]
              set database_type [expr $format & 0x80]
              if { $user_type != $database_type} \
              {
                 continue
              }

              if { "$sln" == "$StateUI(sln_manual_value)" && \
                   "$algid" == "$StateUI(algid_manual_value)" } \
              {
                 # We are trying to put a second key in the same slot
                 set widget ${tile_path}.prompt_dup_key
                 message_box $widget QUESTION
                 ${widget}.text config -text \
                     "SLN $StateUI(sln_manual_value) has a key for this AlgID\nReplace the existing key?"
                 ${widget}.yes config -command \
                     "::Key_LoaderUI::add_replace_key REPLACE_SAME_SLN $StateUI(sln_manual_value) ;destroy $widget"
                 return
              }

              if { "$keyid" == "$StateUI(keyid_manual_value)" && \
                   "$algid" == "$StateUI(algid_manual_value)" } \
              {
                 # Matching key identity found in a slot
                 set widget ${tile_path}.prompt_dup_key
                 message_box $widget QUESTION
                 ${widget}.text config -text \
                     "This KEYID and ALGID already exist in SLN $sln\nReplace the existing key?"
                 if { "$sln" == "$StateUI(sln_manual_value)" } \
                 {
                    ${widget}.yes config -command \
                       "::Key_LoaderUI::add_replace_key REPLACE_SAME_SLN $sln ;destroy $widget"
                 } else \
                 {
                    ${widget}.yes config -command \
                       "::Key_LoaderUI::add_replace_key REPLACE_OLD_SLN $sln ;destroy $widget"
                 }
                 return
              }
           }
        }
     }

     # If we did not have a conflict, simply add the key
     add_replace_key ADD $StateUI(sln_manual_value)"
  }

  #**************************************************************************
  #
  #        Name: set_key_to_zeros
  #
  #  Parameters: key_type - key type, either DES or AES
  #
  # Description: Sets the present KEY value to all zeros
  #
  #**************************************************************************
  proc set_key_to_zeros { key_type } \
  {
    #puts "set_key_to_zeros:  $key_type"
    
    variable  StateUI

    switch -- $key_type \
    {
      "DES"  { 
                if { $StateUI(key_manual_des) == "****************" } \
                {
                   set StateUI(key_manual_des) "0000000000000000" 
                }
             }
      
      "AES"  { 
                if { $StateUI(key_manual_aes_1) == "********************************" } \
                {
                   set   StateUI(key_manual_aes_1)  "00000000000000000000000000000000" 
                   set StateUI(key_manual_aes_2)  "00000000000000000000000000000000" 
                }
             }
     }
  }
  
  #**************************************************************************
  #
  #        Name: manual_remove
  #
  #  Parameters: widget - tile path of the invoking widget
  #
  # Description: Removes a manually entered key from the Keyset
  #
  #**************************************************************************
  proc manual_remove { widget } \
  {
     #puts "manual_remove:  $widget"
    
     variable  keyset_list
     variable  StateUI
    
     set tile_path  [get_tile  $widget]
    
     set keyid_manual  "0x$StateUI(keyid_manual)"
    
     switch -- $StateUI(algid_manual) \
     {
        "DES"   { set algid_manual  "0x81" }
        "AES"   { set algid_manual  "0x84" }
        default { return }
     }

     foreach { data_set } $keyset_list \
     {
        foreach { data_subset } $data_set \
        {
           foreach  { grpid  keyid  algid  format  size  key  name  sln }  $data_subset \
           {
              if { "$keyid" == "$keyid_manual" && \
                   "$algid" == "$algid_manual" } \
              {
                 # Verify that the user wants to delete the key
                 set widget ${tile_path}.prompt_delete_key
                 message_box $widget QUESTION
                 ${widget}.text config -text \
                     "Delete key for KeyID ($keyid_manual)\n and Algorithm ($StateUI(algid_manual))?"
                 ${widget}.yes config -command \
                     "Keys  delete  $keyid_manual  $algid_manual; destroy $widget"
                 return
              }
           }
        }
     }
  }

  #**************************************************************************
  #
  #        Name: update_keyset
  #
  #  Parameters: widget - tile_path of the invoking widget
  #
  # Description: Displays the current Keyset in the textbox
  #
  #**************************************************************************
  proc update_keyset { widget  args } \
  {
    #puts ""
    #puts ""
    #puts "update_keyset:  $widget,  $args"
    
    variable  keyset_list
    variable  saved_flag

    set tile_path  [get_tile  $widget]
    
    #puts "keyset_list:  $keyset_list"

    if { $keyset_list == "{}" } \
    {
      set saved_flag  1
    } \
    else \
    {
      # The keyset has been modified
      # Clear the 'saved_flag' so the data can be saved
      set saved_flag  0
    }
    
    if { ![winfo exists $tile_path] } \
    {
       return
    }


    # Enable the textbox for updating
    ${tile_path}.txt_keylist  config  -state  normal
    
    # Update the display
    update_display  $tile_path
    
    # Delete the existing values in the textbox
    ${tile_path}.txt_keylist  delete  1.0  end
    
    # Display the header
    set text_line  [format "%-12s%-12s%-8s%-15s%-5s"  \
                         "KEYID(hex)"  "ALGID(hex)"  "FORMAT"  "NAME"  "SLN"]
    
    ${tile_path}.txt_keylist  insert  end  "$text_line\n"
    
    # Display the seperator
    set text_line  "--------------------------------------------------------"
    
    ${tile_path}.txt_keylist  insert  end  "$text_line\n"
    
    # Delete the existing values in the KEYID list
    set ::DownlinkUI::keyid_list  {}
    
    # Delete the existing values in the ALGID list
    foreach { names } [array names  ::DownlinkUI::algid_list] \
    {
      set ::DownlinkUI::algid_list($names)  {}
    }

    foreach { data_set }  $keyset_list \
    {
      #puts "data_set:  $data_set"
      
      foreach { data_subset } $data_set \
      {
        #puts "data_subset:  $data_subset"

        # Get the values of interest
        set keyid  [lindex $data_subset  1]

        if { $keyid == "0x0000" } \
        {
          set keyid  "0000"
        } \
        else \
        {
          set keyid  [string trimleft  $keyid  "0x"]
          set keyid  [format "%04X" 0x$keyid]
        }

        set algid  [lindex $data_subset  2]

        if { $algid == "0x00" } \
        {
          set algid  "00"
        } \
        else \
        {
          set algid  [string trimleft  $algid  "0x"]
          set algid  [format "%02X" 0x$algid]
        }

        switch -- $algid \
        {
          "81"  { append algid  " (DES)" }
          
          "84"  { append algid  " (AES)" }
        }

        set format  [lindex $data_subset  3]
        
        if { [expr $format & 0x80] } \
        {
          set format  "KEK"
        } \
        else \
        {
          set format  "TEK"
        }
        
        set name    [lindex $data_subset  6]
        set sln     [lindex $data_subset  7]

        # Limit the displayed name to 12 characters to fit in the field space on the screen 
        set name_length  [string length "$name"]
        if { $name_length > 12 } \
        {
           set name  [string range "$name" 0  11]
        }
     
        #puts ""
        #puts "keyid:  $keyid"
        #puts "algid:  $algid"
        #puts "format:  $format"
        #puts "name:  $name"
        #puts "sln:  $sln"
        #puts ""
      
        # Format the display
        set text_line  [format "%6s%15s%7s%4s%-15s%-5s"  \
                         "$keyid"  "$algid"  "$format"  " "  "$name"  "$sln"]
    
        # Display the new values in the textbox
        ${tile_path}.txt_keylist  insert  end  "$text_line\n"
      
        # Update the KEYID list
        lappend  ::DownlinkUI::keyid_list  $keyid
      
        # Update the ALGID list
        lappend  ::DownlinkUI::algid_list($algid)  $keyid
      }
    }

    # Disable the textbox after updating
    ${tile_path}.txt_keylist  config  -state  disabled
    

    return
  }


  #**************************************************************************
  #
  #        Name: update_display
  #
  #  Parameters: widget - tile_path of the invoking widget
  #
  # Description: Updates the display if a Keyset exists
  #
  #**************************************************************************
  proc update_display { widget  args } \
  {
    #puts "update_display:  $widget,  $args"
    
    variable  keyset_list

    set tile_path  [get_tile  $widget]
    
    #puts "keyset_list:  $keyset_list"

    # Update the display
    if { $keyset_list == "{}" } \
    {
      ${tile_path}.btn_zero    config  -state disabled
      ${tile_path}.btn_save    config  -state disabled
      ${tile_path}.btn_remove  config  -state disabled
    } \
    else \
    {
      ${tile_path}.btn_zero    config  -state normal
      ${tile_path}.btn_save    config  -state normal
      ${tile_path}.btn_remove  config  -state normal
    }
      
    return
  }
    

  #**************************************************************************
  #
  #        Name: zero_keyset
  #
  #  Parameters: widget - tile_path of the invoking widget
  #
  # Description: Clears the present keyset
  #
  #**************************************************************************
  proc zero_keyset { widget } \
  {
    #puts "zero_keyset:  $widget"
    
    variable  keyset_list

    set tile_path  [get_tile  $widget]

    # Prompt the user if the present keyset needs to be saved
    prompt_keyset_not_saved  $widget

    # Zero the keyset
    Keys zeroize
    
    # Zero the Keyset in the DSP
    P25 encode zerokeyid
   
    # Set the keyset_list to null, 
    # display will update on a variable trace
    set keyset_list  [list {}]
    
    # Set the keyid_list to null, 
    set ::DownlinkUI::keyid_list   [list {}]
    
    return
  }


  #**************************************************************************
  #
  #        Name: use_keyset
  #
  #  Parameters: widget - tile_path of the invoking widget
  #
  # Description: Done with the Key Loader, use the keyset and destroy the 
  #              Key Loader toplevel.
  #
  #**************************************************************************
  proc use_keyset { widget } \
  {
    #puts "use_keyset:  $widget"
    
    variable  StateUI
    
    set tile_path  [get_tile  $widget]
    
    ${tile_path}.btn_download  config  -state disabled
    
    download_keyset
 
    prompt_keyset_not_saved  $widget
    
    # Indicate that the user has pressed the USE KEYS button
    set StateUI(cancel_or_use_keys)  1
    
    after 1000
    
    destroy  $tile_path
    
    return
  }


  #**************************************************************************
  #
  #        Name: download_keyset
  #
  #  Parameters:
  #
  # Description: Download the keyset.
  #
  #**************************************************************************
  proc download_keyset { } \
  {
    #puts "download_keyset"
    
    variable  keyset_list

    if { $keyset_list != "{}" } \
    {
      # Zero the Keyset in the DSP before loading
      P25 encode zerokeyid

      foreach { data_set }  $keyset_list \
      {
        #puts "data_set:  $data_set"
        
        foreach { data_subset } $data_set \
        {
          #puts "data_subset:  $data_subset"
          
          # Get the values of interest
          set keyid  [lindex  $data_subset  1]
          set algid  [lindex  $data_subset  2]
          set size   [lindex  $data_subset  4]
          set key    [lindex  $data_subset  5]
          
          # Remove the '0x' designator from the key string
          set key    [string trimleft  $key  "0x"]
     
          #puts "P25 encode setkey  $keyid  $algid  $size  $key  -nowait"

          # Send the key down to the P25 engine
          after idle "P25 encode setkey  $keyid  $algid  $size  $key  -nowait"
        }
      }
    }
    
    # Set the ALGID/KID value to synchronize the ALGID/KID to a 
    # key in the keyset
    DownlinkUI::set_algid  entry
    DownlinkUI::set_value  keyid  HDR_KID

    # Invoke 'Send Header' button to synchronize the ALGID/KID
    #P25ns::Send_header
    
    return
  }


  #**************************************************************************
  #
  #        Name: cancel_keyset
  #
  #  Parameters: widget - tile_path of the invoking widget
  #
  # Description: Cancel the Key Loader, destroy the Key Loader toplevel
  #
  #**************************************************************************
  proc cancel_keyset { widget } \
  {
    #puts "cancel_keyset:  $widget"
    
    variable  StateUI

    set tile_path  [get_tile  $widget]
    
    ${tile_path}.btn_cancel  config  -state disabled

    prompt_keyset_not_saved  $widget

    # Indicate that the user has pressed the CANCEL button
    set StateUI(cancel_or_use_keys)  1

    after 1000
    
    destroy  $tile_path
    
    return
  }


  #**************************************************************************
  #
  #        Name: prompt_keyset_not_saved
  #
  #  Parameters: widget - tile path of the invoking widget
  #
  # Description: Prompts the user if the Keyset hasn't been saved.
  #
  #**************************************************************************
  proc prompt_keyset_not_saved { widget } \
  {
    #puts "prompt_keyset_not_saved:  $widget"
    
    variable  keyset_list
    variable  saved_flag
    
    set tile_path  [get_tile  $widget]
    if { ($keyset_list == "{}") || ![winfo exists $tile_path] } \
    {
       return
    }
    
    if { $saved_flag == 0 } \
    {
      # If the Keyset has not been saved, prompt the user to save it
      set title  "Save keyset?"

      set prompt_text  "The Key Set has not been saved.\nSave it now?"
    
      set window_name  ${tile_path}.instruction
    
      set prompt_return  \
        [tk_dialog  $window_name  $title  $prompt_text  info  0  "YES"  "NO"]
    
      switch -- $prompt_return \
      {
        0 { # Save the key list
            save_keyset  $widget }
    
        1 { }
      }
    }
    
    return
  }


  #**************************************************************************
  #
  #        Name: save_keyset
  #
  #  Parameters: widget - tile_path of the invoking widget
  #
  # Description: Saves the present keyset to a file
  #
  #**************************************************************************
  proc save_keyset { widget } \
  {
    #puts "save_keyset:  $widget"
    
    variable  file_label
    variable  keyset_list
    variable  saved_flag

    set tile_path  [get_tile  $widget]
    if { ($keyset_list == "{}") || ![winfo exists $tile_path] } \
    {
       return
    }

    # Prepare the keyset list by creating a comma delimited string 
    # from the data
    set keyset_list_comma  ""

    foreach { data_set } $keyset_list \
    {
      #puts "data_set:  $data_set"
      
      foreach { line } $data_set \
      {
        #puts "line:  $line"
        
        # Replace the spaces with commas
        regsub -all " " $line "," temp
      
        append  keyset_list_comma  "$temp\n"
      
        #puts "keyset_list_comma:  $keyset_list_comma"
      }
    }
    
    # Prepare the file data
    set file_buffer  ""
    
    # Create the file_header
    append  file_buffer  "IFR KEYSET 1.0\n"
    append  file_buffer  "KEY FILE\n"
    append  file_buffer  "\n"
    append  file_buffer  "\n"
    append  file_buffer  "\n"
    append  file_buffer  "BEGIN\n"
    
    # Append the keyset
    append  file_buffer  "$keyset_list_comma"
    
    # Append the last line to the file data
    append  file_buffer  "END\n"

    #puts "file_buffer:  $file_buffer"
    
    # Write the data to a file
    set working_dir  "/usr/keysets/"
    set file_prefix  "KEYSET"
    set file_ext     ".KST"
    
    set saved_flag  [File_SaveUI::display_file_save  $widget        \
                                                     $working_dir   \
                                                     $file_buffer   \
                                                     $file_prefix   \
                                                     "$::DownlinkUI::file_label"  \
                                                     $file_ext      \
                                                     ENCRYPT        \
                                                     NO_CSV         \
                                                     NO_MODES       \
                                                     NO_OPT_EXT]
    
    # Get the file label after the file has been saved
    set file_label  $::File_SaveUI::file_label

    return
  }


  #**************************************************************************
  #
  #        Name: recall_keyset
  #
  #  Parameters: widget - tile_path of the invoking widget
  #
  # Description: Recalls a keyset from a file
  #
  #**************************************************************************
  proc recall_keyset { widget } \
  {
    #puts "recall_keyset:  $widget"
    
    variable  keyset_list
    variable  saved_flag

    set tile_path  [get_tile  $widget]

    # Read the data from the file
    set working_dir  "/usr/keysets"
    set file_prefix  "KEYSET"
    set file_ext     ".KST"

    set file_buffer  [File_RecallUI::display_file_recall  $widget       \
                                                          $working_dir  \
                                                          $file_prefix  \
                                                          $file_ext     \
                                                          ENCRYPT]
    
    #puts "recall_keyset: file_buffer = $file_buffer"
    
    # Parse the file data
    # Find the 'BEGIN' and 'END' tags in the file data
    if { $file_buffer != "" } \
    {
      # Clear the present keyset
      zero_keyset  $tile_path

      set start_index  [string first  "BEGIN"  $file_buffer]
      set stop_index   [string first  "END"  $file_buffer]
      
      set temp  [string range $file_buffer \
                              [expr $start_index + 6] [expr $stop_index - 1]]
                                
      # Get each line of the keyset data, and display the data
      set new_key  {}
        
      foreach { text_line } $temp \
      {
        #puts "text_line:  $text_line"
        
        # Replace the commas with spaces
        regsub -all "," $text_line " " text_line
        
        lappend new_key  $text_line
        
        #puts "new_key:  $new_key"
      }

      # Add the key to the keyset list
      set keyset_list  [list $new_key]
    
      #puts "recall_keyset: - keyset_list =  $keyset_list"
    }
    
    # Set the 'saved_flag' after setting the 'keyset_list' to recalled values
    # Recalled values do not need to be saved

    set saved_flag  1

    return
  }


  #**************************************************************************
  #
  #        Name: default_keyset
  #
  #  Parameters: widget - tile_path of the invoking widget
  #
  # Description: Restores the default Keyset
  #
  #**************************************************************************
  proc default_keyset { widget } \
  {
    #puts "default_keyset:  $widget"
    
    variable  keyset_list
    variable  saved_flag
    
    prompt_keyset_not_saved  $widget
    
    Keys zeroize
    
    Keys defaults
    
    set keyset_list  [Keys get]
    
    # Set the 'saved_flag' after setting the 'keyset_list' to defaults
    # Default values do not need to be saved
    set saved_flag  1
    
    return
  }


  #***************************************************************************
  #
  #        Name: toggle_scroll_on
  #
  #  Parameters: tile_path - tile path of the invoking widget
  #
  # Description: Toggles the 'scroll_on' variable to allow the spinner
  #              knob to control the scrollbar
  #
  #***************************************************************************
  proc toggle_scroll_on { tile_path } \
  {
    #puts "toggle_scroll_on:  $tile_path"
    
    variable  scroll_on
    
    set tile_path  [get_tile  $tile_path]
    
    set scrollbar_path  ${tile_path}.scrl_keylist
    
    if { $scroll_on == 0 } \
    {
      # Change the color of the scrollbar to indicate it is active
      $scrollbar_path config  -bg white
      
      set scroll_on  1
      
      # For each event, call the procedure to move the scrollbar,
      # then break to keep the focus from moving in response to the 
      # event
      # Bind the <Right> event  (includes <<Spin_up>>)
      bind $scrollbar_path  <Right>  \
                          "Key_LoaderUI::move_textbox  $tile_path  UP; break"
                                   
      # Bind the <Left> event  (includes <<Spin_down>>)
      bind $scrollbar_path  <Left>  \
                        "Key_LoaderUI::move_textbox  $tile_path  DOWN; break"
                                   
      bind $scrollbar_path  <Up>  \
                          "Key_LoaderUI::move_textbox  $tile_path  UP; break"
                                   
      bind $scrollbar_path  <Down>  \
                        "Key_LoaderUI::move_textbox  $tile_path  DOWN; break"
    } \
    else \
    {
      # Change the color of the scrollbar to indicate it is NOT active
      $scrollbar_path config  -bg gray

      set scroll_on  0

      # Unbind the <Right> event (includes <<Spin_up>>)
      bind $scrollbar_path  <Right>  ""

      # Unbind the <Left> event (includes <<Spin_down>>)
      bind $scrollbar_path  <Left>  ""
      
      # For the 'Up' of 'Down' event, generate the 'cursor_up' or 'cursor_down'
      # event, then break to keep the scrollbar from moving in response to 
      # either event
      bind $scrollbar_path  <Up>    "event generate %W <<cursor_up>>; break"
      bind $scrollbar_path  <Down>  "event generate %W <<cursor_down>>; break"
    }
    
    return
  }
      

  #***************************************************************************
  #
  #        Name: move_textbox
  #
  #  Parameters: tile_path - tile path of the invoking widget
  #              direction - direction to move the scrollbar
  #
  # Description: Moves the textbox display for changes on the spinner knob 
  #              or Up and Down arrows.
  #
  #***************************************************************************
  proc move_textbox { tile_path  direction } \
  {
    #puts "move_textbox:  $tile_path,  $direction"
    
    variable  scroll_on
    
    if { $scroll_on == 0 } \
    {
      return
    }

    set tile_path  [get_tile  $tile_path]
    
    set textbox_path  ${tile_path}.txt_keylist

    # Move the canvas display
    switch -- $direction \
    {
      "UP"    { $textbox_path  yview  scroll  -1  unit }
    
      "DOWN"  { $textbox_path  yview  scroll  1  unit }
    }
    
    return
  }


};  # End of namespace Key_LoaderUI




