#############################################################################
#                                                                           #
# IFR SYSTEMS, INC        COPYRIGHT (C)1998-2002      DESIGN ENGINEERING    #
# ------------------------------------------------------------------------- #
# FILE        : 3510.P25uplink_UI.tcl                                       #
# PRODUCT     : FITS                                                        #
# AUTHOR      : Hazmar Feroze, Hieu Tran                                    #
# DESCRIPTION : P25 User Interface	                                    #
#                                                                           #
#############################################################################



namespace eval P25_LoggerUI \
{
  # Global variables
  variable  LongName    "p25_log_setup"
  variable  ShortName   "P 25 Logger"	
  variable  Size        Quarter
  variable  tile_class  P25_LOG_SETUP
  
  variable  scroll_on  0

  variable  StateUI
  
  # Control Channel Logging
  set  StateUI(log_file_id_1)    0
  set  StateUI(log_file_id_2)    0

  set  StateUI(log_mode)       "START"
  set  StateUI(log_filename)   "LOG1.TXT"
  set  StateUI(log_filepath)   "/usr/p25_log"
  set  StateUI(log_file_mode)  "APPEND"
  
  set  StateUI(get_next)      0
  set  StateUI(get_previous)  0
  
  set  StateUI(log_file_start_index)  0
  set  StateUI(log_file_stop_index)   1000
  
  set  StateUI(log_file_top)     1
  set  StateUI(log_file_bottom)  0


  #***************************************************************************
  #
  #        Name: Create
  #
  #  Parameters: tile path of parent
  #
  # Description: create P 25 Logger tile
  #
  #***************************************************************************
  proc Create { parent } \
  {
    variable LongName
    variable tile_class

    set top_level [winfo toplevel  $parent]
     
    set tile_path [name_child  $top_level  $LongName]

    if {![winfo exists $tile_path]} \
    {
      set tile  [Screens::create_tile  $tile_path  $LongName  $tile_class]
    }

    return  $tile_path
  }


  #***************************************************************************
  #
  #        Name: Map
  #
  #  Parameters: widget - tile path
  #
  # Description: called when the screen is displayed 
  #
  #***************************************************************************
  proc Map { widget } \
  {
    # If the P25 Control Channel Logging option is not installed, 
    # remove the tile and return
    if { ![IFR_options  is_installed  P25_CTRL_LOG] } \
    {
      grid remove  $widget
      
      return
    }
           
    P25 decode enable 3 -nowait
    P25 decode run on -nowait

    trace variable ::P25_LoggerUI::StateUI(log_mode)  w  \
                                    "::P25_LoggerUI::update_log_mode  $widget"

    update_log_mode_core  $widget
    
    return
  }
  

  #***************************************************************************
  #
  #        Name: Cleanup
  #
  #  Parameters: widget - tile path
  #
  # Description: Called when the screen is hidden
  #
  #***************************************************************************
  proc Cleanup { widget } \
  {
    trace vdelete ::P25_LoggerUI::StateUI(log_mode)  w  \
                                     "::P25_LoggerUI::update_log_mode  $widget"

    return
  }
  
  
  #***************************************************************************
  #
  #        Name: Unmap
  #
  #  Parameters: widget - tile path
  #
  # Description: Called when the screen is hidden
  #
  #***************************************************************************
  proc Unmap { widget } \
  {
    Cleanup  $widget

    return
  }


  #***************************************************************************
  #
  #        Name: Destroy
  #
  # Parameters : widget - tile path
  #
  # Description: Called when the screen is destroyed 
  #
  #***************************************************************************
  proc Destroy { widget } \
  {
    Cleanup  $widget

    return
  }


  #***************************************************************************
  #
  #        Name: get_tile
  #
  #  Parameters: any child widget of the tile
  #
  # Description: Get the name of the tile's parent frame
  #
  #***************************************************************************
  proc get_tile { widget } \
  {
    while { 1 } \
    {
      switch -- [winfo class $widget] \
      {
        "P25_LOG_SETUP"     -
        "P25_LOG_VIEW"      { return $widget }
      }
       
      set widget [winfo parent $widget]
    }

    return     
  }


  #**************************************************************************
  #
  #        Name: toggle_log_mode
  #
  #  Parameters: 
  #
  # Description: Toggles the Logging between START and STOP
  #
  #**************************************************************************
  proc toggle_log_mode { } \
  {
    #puts "toggle_log_mode"
    
    variable  StateUI
    variable  P25_log_ctrl_fd
    
    if { $StateUI(log_mode) == "START" } \
    {
      set StateUI(log_mode)  "STOP"
      
      # Set the filename
      set filename  [file join $StateUI(log_filepath) $StateUI(log_filename)]
      
      # Determine if the directory exists
      # If it does not, create it
      if { ![file exists $StateUI(log_filepath)] } \
      {
        file mkdir $StateUI(log_filepath)
      }
      
      # Determine the file mode: OVERWRITE or APPEND
      switch -- $StateUI(log_file_mode) \
      {
        "OVERWRITE"  { # Delete the file before beginning

                       #puts "file_deleted"
                       # Insure that the logger knows the file name, and
                       # then indicate that it should be deleted
                       P25 log filename "$filename"
                       P25 log delete
                       }
        
        "APPEND"     { # No action, the engine already appends the file 
                       #puts "file_append" }
      }
            
      # Start logging
      set StateUI(log_file_id_1)  [P25 log open "$filename" $::P25ns::Log(mask)]
      
      # Cancel the pending call
      after cancel "::P25_LoggerUI::toggle_log_mode"
    } \
    else \
    {
      set StateUI(log_mode)  "START"

      # Stop the logging if it is in progress
      P25 log close $StateUI(log_file_id_1)

      # Cancel the pending call
      after cancel "::P25_LoggerUI::toggle_log_mode"
    }
    
    return
  }


  #**************************************************************************
  #
  #        Name: update_log_mode
  #
  #  Parameters: widget - tile path
  #
  # Description: Updates the display to changes in the Logging mode
  #
  #**************************************************************************
  proc update_log_mode { widget  args } \
  {
     set cmd [namespace code "update_log_mode_core $widget"]
     after cancel $cmd
     after idle $cmd
  }
  proc update_log_mode_core { widget } \
  {
    #puts "update_log_mode:  $widget "
    variable  StateUI
    
    set tile_path  [get_tile  $widget]
    
    if { [winfo exists  ${tile_path}.btn_view_log] } \
    {
      if { $StateUI(log_mode) == "START" } \
      {
        ## Enable the VIEW LOG button
        #${tile_path}.btn_view_log config -state normal
      }

      if { $StateUI(log_mode) == "STOP" } \
      {
        ## Disable the VIEW LOG button
        #${tile_path}.btn_view_log config -state disabled
      }
    }
  }


  #**************************************************************************
  #
  #        Name: toggle_log_file_mode
  #
  #  Parameters: 
  #
  # Description: Toggles the Logging file mode between OVERWRITE and APPEND
  #
  #**************************************************************************
  proc toggle_log_file_mode { } \
  {
    #puts "toggle_log_file_mode"
    
    variable  StateUI
    
    if { $StateUI(log_file_mode) == "OVERWRITE" } \
    {
      set StateUI(log_file_mode)  "APPEND"
    } \
    else \
    {
      set StateUI(log_file_mode)  "OVERWRITE"
    }
    
    return
  }

  #***************************************************************************
  #
  #        Name:  display_log_view
  #
  #  Parameters:
  #
  # Description:  Displays the Logging view
  #
  #***************************************************************************
  proc display_log_view { tile_path } \
  {
    #puts "display_log_view:  $tile_path"
    
    variable  last_focus
    
    variable  StateUI
    
    set last_focus  $tile_path
    
    set tile_path  [get_tile  $tile_path]
    
    set top_level  [winfo toplevel $tile_path]
   
    set tile_path  [name_child  $top_level  p25_log_view]

    if { ![winfo exists $tile_path] } \
    {
      set tile_path  [::Screens::create_toplevel_tile  $tile_path  P25_LOG_VIEW]
      
      # Hook up the scroll bar
      $tile_path.txt_log_view config -yscrollcommand "$tile_path.scrl_log_view set"
      $tile_path.scrl_log_view config -command        "$tile_path.txt_log_view yview"
      
      # Bind the button so we can scroll
      bind $tile_path.btn_return <Left>   "tkScrollByUnits $tile_path.scrl_log_view v 1; break"
      bind $tile_path.btn_return <Right>  "tkScrollByUnits $tile_path.scrl_log_view v -1; break"
    }
        
    # Bind the F8 key for a return
    bind $tile_path <KeyRelease-F8> "destroy [winfo toplevel $tile_path]"

    bind $tile_path <<Destroy>> "P25_LoggerUI::destroy_log_view"

    ${tile_path}.scrl_log_view config  \
                              -command "P25_LoggerUI::update_log_view  $tile_path"

    ${tile_path}.txt_log_view config  \
                              -yscrollcommand "${tile_path}.scrl_log_view set"


    # Get the first section of data from the file
    # and display it in the text box
    set filename  [file join $StateUI(log_filepath) $StateUI(log_filename)]
      
    #puts "filename:  $filename"

    set  StateUI(log_file_start_index)  0
    set  StateUI(log_file_stop_index)   1000
      
    get_file_data  $tile_path  $filename

    # Display the first line of the textbox
    ${tile_path}.txt_log_view  see  1.0

    
    safe_focus_force ${tile_path}.btn_return
    
    return
  }


  #***************************************************************************
  #
  #        Name:  destroy_log_view
  #
  #  Parameters:
  #
  # Description:  Destroys the Logging view
  #
  #***************************************************************************
  proc destroy_log_view { } \
  {
    #puts "destroy_log_view"
    
    variable  StateUI
    
    return
  }


  #***************************************************************************
  #
  #        Name:  update_log_view
  #
  #  Parameters:  tile_path - invoking widget tile path
  #
  # Description:  Updates the log display to changes in the vertical scrollbar
  #
  #***************************************************************************
  proc update_log_view { tile_path  args } \
  {
    #puts "update_log_view:  $tile_path,  $args"
    
    variable  StateUI
    
    set tile_path  [get_tile  $tile_path]
    
    # Get the first section of data from the file
    # and display it in the text box
    set filename  [file join $StateUI(log_filepath) $StateUI(log_filename)]
      
    #puts "filename:  $filename"

    foreach { cmd  amount  units } $args { }
    
    if { $cmd == "moveto" } \
    {
      if { $amount > 1.0 } \
      {
        set amount  1.0
      }
      
      if { $amount < 0.0 } \
      {
        set amount  0.0
      }

      ${tile_path}.txt_log_view  yview  moveto  $amount
    }
    
    if { $cmd == "scroll" } \
    {
      ${tile_path}.txt_log_view  yview  scroll  $amount  $units
    }

    # Get the position of the scrollbar relative to the textbox
    set position  [${tile_path}.txt_log_view  yview]
    
    #puts "position:  $position"
    
    #puts "StateUI(log_file_start_index):  $StateUI(log_file_start_index)"
    #puts "StateUI(log_file_stop_index):  $StateUI(log_file_stop_index)"
    
    foreach { pos1 pos2 } $position { }
    
    if { $pos1 == 0.0 && $StateUI(log_file_start_index) != 0 } \
    {
      # If the position of the scrollbar is at the top
      # Reset the file indices
      # Set the stop index to the present start index
      set StateUI(log_file_stop_index)  $StateUI(log_file_start_index)

      # Set the start index to the stop index minus 1000
      set StateUI(log_file_start_index)  [expr $StateUI(log_file_stop_index) - 1000]
      
      if { $StateUI(log_file_start_index) < 0 } \
      {
        set StateUI(log_file_start_index)  0

        # Readjust the stop index
        set StateUI(log_file_stop_index)  [expr $StateUI(log_file_start_index) + 1000]
      }
      
      # Get the new file data
      get_file_data  $tile_path  $filename

      # Display the last line of the textbox
      ${tile_path}.txt_log_view  see  end
    } \
    elseif { $pos2 == 1.0 && $StateUI(log_file_bottom) == 0 } \
    {
      # If the position of the scrollbar is at the bottom
      # Reset the file indices
      # Set the start index to the present stop index
      set StateUI(log_file_start_index)  $StateUI(log_file_stop_index)

      # Set the stop index to the start index plus 1000
      set StateUI(log_file_stop_index)  [expr $StateUI(log_file_start_index) + 1000]
        
      # Get the new file data
      get_file_data  $tile_path  $filename
        
      # Display the first line of the textbox
      ${tile_path}.txt_log_view  see  1.0
    }    

    return
  }



  #***************************************************************************
  #
  #        Name:  get_file_data
  #
  #  Parameters:  tile_path  - path of the invoking tile
  #              filename   - name of the data file
  #
  # Description:  Gets data from the file and load it into the textbox
  #
  #***************************************************************************
  proc get_file_data { tile_path  filename } \
  {
    #puts "get_file_data:  $tile_path,  $filename"
    
    # allow changes to the text object
    ${tile_path}.txt_log_view config -state normal

    variable  StateUI
    
    set tile_path  [get_tile  $tile_path]
    
    if { [file exists $filename] } \
    {
      #puts "file exists"
      
      #puts "get lines $StateUI(log_file_start_index) to $StateUI(log_file_stop_index)"
      # Delete the present contents of the textbox
      ${tile_path}.txt_log_view  delete  1.0  end
      
      # Open the file
      set StateUI(log_file_id_2)  [open $filename  r]
        
      set data_read   0
      set file_index  1
      set view_index  1

      set StateUI(log_file_bottom)  0
        
      # Read the lines up to the start index
      while { $file_index < $StateUI(log_file_start_index) } \
      {
        #puts "file_index:  $file_index   read lines - no action"
        
        # Read the data from the file
        set data_read  [gets  $StateUI(log_file_id_2)  file_line]
        
        incr file_index  1
      }
      

      # Read in lines from the start index to the stop index 
      # and put them in the textbox
      while { $file_index >= $StateUI(log_file_start_index) && \
              $file_index <= $StateUI(log_file_stop_index) && \
              $data_read != -1 } \
      {
        #puts "file_index:  $file_index   read lines and insert"

        # Read the data from the file
        set data_read  [gets  $StateUI(log_file_id_2)  file_line]
        
        #puts "data_read:  $data_read"
        #puts "file_line:  $file_line"
         
        incr file_index  1

        if { $data_read != -1 } \
        {
          # Display the lines in the text box
          ${tile_path}.txt_log_view  insert  end  "$file_line\n"
        } \
        else \
        {
          set StateUI(log_file_bottom)  1
        }
      }
      
      if { $StateUI(log_file_bottom) == 1 } \
      {
        # Set the stop increment if the end of the file has been reached
        set StateUI(log_file_stop_index)  $file_index
      }

      close  $StateUI(log_file_id_2)
    } \
    else \
    {
      #puts "file does not exist"
    }

    # now, lock the object so we cannot edit it
    ${tile_path}.txt_log_view config -state disabled
    
    return
  }


  #***************************************************************************
  #
  #        Name: toggle_scroll_on
  #
  #  Parameters: tile_path - tile path of the invoking widget
  #
  # Description: Toggles the 'scroll_on' variable to allow the spinner
  #              knob to control the scrollbar
  #
  #***************************************************************************
  proc toggle_scroll_on { tile_path } \
  {
    #puts "toggle_scroll_on:  $tile_path"
    
    variable  scroll_on
    
    set tile_path  [get_tile  $tile_path]
    
    set scrollbar_path  ${tile_path}.scrl_log_view
    
    if { $scroll_on == 0 } \
    {
      # Change the color of the scrollbar to indicate it is active
      $scrollbar_path config  -bg white
      
      set scroll_on  1
      
      # For each event, call the procedure to move the scrollbar,
      # then break to keep the focus from moving in response to the 
      # event
      # Bind the <Right> event  (includes <<Spin_up>>)
      bind $scrollbar_path  <Right>  "::P25_LoggerUI::move_textbox  $tile_path  UP; break"
                                   
      # Bind the <Left> event  (includes <<Spin_down>>)
      bind $scrollbar_path  <Left>  "::P25_LoggerUI::move_textbox  $tile_path  DOWN; break"
                                   
      bind $scrollbar_path  <Up>  "::P25_LoggerUI::move_textbox  $tile_path  UP; break"
                                   
      bind $scrollbar_path  <Down>  "::P25_LoggerUI::move_textbox  $tile_path  DOWN; break"
    } \
    else \
    {
      # Change the color of the scrollbar to indicate it is NOT active
      $scrollbar_path config  -bg gray

      set scroll_on  0

      # Unbind the <Right> event  (includes <<Spin_up>>)
      bind $scrollbar_path  <Right>  ""

      # Unbind the <Left> event  (includes <<Spin_down>>)
      bind $scrollbar_path  <Left>  ""
      
      # For the 'Up' of 'Down' event, generate the 'cursor_up' or 'cursor_down'
      # event, then break to keep the scrollbar from moving in response to 
      # either event
      bind $scrollbar_path  <Up>    "event generate %W <<cursor_up>>; break"
      bind $scrollbar_path  <Down>  "event generate %W <<cursor_down>>; break"
    }
    
    return
  }
      

  #***************************************************************************
  #
  #        Name: move_textbox
  #
  #  Parameters: tile_path - tile path of the invoking widget
  #              direction - direction to move the scrollbar
  #
  # Description: Moves the textbox display for changes on the spinner knob 
  #              or Up and Down arrows.
  #
  #***************************************************************************
  proc move_textbox { tile_path  direction } \
  {
    #puts "move_textbox:  $tile_path,  $direction"
    
    variable  scroll_on
    
    if { $scroll_on == 0 } \
    {
      return
    }

    set tile_path  [get_tile  $tile_path]
    
    set textbox_path  ${tile_path}.txt_log_view

    # Move the canvas display
    switch -- $direction \
    {
      "UP"    { $textbox_path  yview  scroll  -1  unit }
    
      "DOWN"  { $textbox_path  yview  scroll  1  unit }
    }
    
    return
  }

  #****************************************************************************
  #
  #        Name: getSaveState
  #
  # Parameters:  none
  #
  # Description: This function returns a list of TCL commands which can be used
  #              to set the P25_LoggerUI tile to its current state
  #
  #****************************************************************************
  proc getSaveState { } \
  {
    variable  StateUI
    variable  marker_list

    set returnList  \
    {
      # set  ::P25_LoggerUI::StateUI(log_filename) $StateUI(log_filename)
      # set  ::P25_LoggerUI::StateUI(log_file_mode) $StateUI(log_file_mode)
    }
    
    return [subst $returnList]
  }

};  # End of P25_LoggerUI namespace

#****************************************************************************
# This registers the tiles with the list for saving instrument states
#****************************************************************************
namespace eval ::SavedStates \
{
  variable tiles   

  lappend tiles  P25_LoggerUI
}

