#############################################################################
#                                                                           #
# AEROFLEX, INC    COPYRIGHT (C) 2002  DESIGN ENGINEERING                   #
# ------------------------------------------------------------------------- #
#                                                                           #
# FILE        : 3522.P25_Repeat_SimUI.tcl                                   #
#                                                                           #
# PRODUCT     : FITS                                                        #
#                                                                           #
# AUTHOR      : P. McCormick                                                #
#                                                                           #
# DESCRIPTION : P25 Trunking Repeater Simulator User Interface              #
#                                                                           #
#############################################################################


package require P25

namespace eval P25_Repeat_SimUI \
{
  # Global variables
  variable  LongName      "p25_repeat_sim"
  variable  ShortName     "P 25 Repeater Sim"
  variable  Size          Quarter
            
  variable  tile_class    "P25_REPEAT_SIM"

  variable  last_focus    ""
  
  variable  StateUI
  variable  ChanConfigUI
  variable  scroll_on         0
  
  set  StateUI(cctx_label)    "CC"
  set  StateUI(ccrx_label)    ""
  set  StateUI(vctx_label)    "VC"
  set  StateUI(vcrx_label)    ""

  set  StateUI(call_type)            1
  set  StateUI(call_type_text)       "DIGITAL"

  set  StateUI(ptt_on_off)           0
  set  StateUI(ptt_on_off_text)      "PTT OFF"


  set  StateUI(cctx_chan,800)        2
  set  StateUI(cctx_freq,800)        0
  set  StateUI(vctx_chan,800)        50
  set  StateUI(vctx_freq,800)        0
  

  set  StateUI(num_format)           "HEX"

  set  StateUI(band_short)  "800"

  set  StateUI(band_menu)  { "700"  "800"  "UHF/VHF" }

  set  StateUI(state_menu)  { "OFF"  "ON" }

  set  StateUI(ptt_menu)  { "Off" "Group Call" "Unit to Unit" "Dialing"}
  set  StateUI(ptt_menu_var) "PTT OFF"
  set  StateUI(ptt_menu_txt) "PTT OFF"

  set  StateUI(term_menu)  { "Expand with Nulls (DEFAULT)" "Expand with FS + NID" }
  set  StateUI(obtctrl) "Expand with Nulls (DEFAULT)"

  #P5 Channel Information variables
  set  StateUI(p25_info_id)            "0"
  set  StateUI(p25_info_bw_other)      "12.500"
  set  StateUI(p25_info_bw_UV)         "12.5"
  set  StateUI(p25_info_offset_other)  "45.000"
  set  StateUI(p25_info_offset_UV)     "45"
  set  StateUI(p25_info_spacing)       "6.250"
  set  StateUI(p25_info_base_freq)     "100.000000"

  set  StateUI(p25_info_bw_menu)  { "6.250"  "12.500" }
  


  #***************************************************************************
  #
  #        Name: Create  
  #
  #  Parameters:
  #
  # Description: 
  #
  #***************************************************************************
  proc Create { tile_path } \
  {
    #puts "P25_Repeat_SimUI::Create:  $tile_path"
    
    variable  LongName
    variable  tile_class
    
    set top_level  [winfo toplevel $tile_path]
   
    set tile_path  [name_child  $top_level  $LongName]

    #puts "tile_path:  $tile_path"

    if { ![winfo exists $tile_path] } \
    {
      set tile_path  [Screens::create_tile  $tile_path  $LongName  $tile_class]
    }  
     
    return $tile_path
  }


  #***************************************************************************
  #
  #        Name: Map
  #
  #  Parameters: 
  #
  # Description: Initializes the tile
  #
  #***************************************************************************
  proc Map { tile_path } \
  {
    #puts "P25_Repeat_SimUI::Map: $tile_path"
    
    # Unmap ourselves if we are disabled
    if { [expr ![IFR_options is_installed P25_trunk] &&    \
               ![IFR_options is_installed P25_OBT] &&      \
               ![IFR_options is_installed P25_Explicit]] } \
    {
      grid forget $tile_path

      return
    }

    # Force a valid mode if only one P25 option is loaded and a save file is loaded with 
    # another option set
    if { ![IFR_options is_installed P25_trunk] } \
    {
      if {$::P25ns::TrunkModeCurrent == "800"} \
      {
        set ::P25ns::TrunkModeCurrent "UHF/VHF"
 
        ::P25ns::setTrunkModeIndex "UHF/VHF"
      }
    } 

    if { ![IFR_options is_installed P25_OBT] } \
    {
      if {$::P25ns::TrunkModeCurrent != "800"} \
      {
        set ::P25ns::TrunkModeCurrent "800"
        
        ::P25ns::setTrunkModeIndex "800"
      }
    }
    
    # Set local variables
    set tile_path  [get_tile  $tile_path]    


    trace variable  ::P25ns::State(trunk_on_off)  w  \
                           "P25_Repeat_SimUI::update_trunk_on_off  $tile_path"

    trace variable  ::P25ns::State(trunk_ptt_rcv)  w  \
                                "P25_Repeat_SimUI::update_ptt_rcv  $tile_path"

    trace variable  ::P25ns::State(trunk_reg_rcv)  w  \
                                "P25_Repeat_SimUI::update_reg_rcv  $tile_path"

    trace variable  ::P25ns::State(trunk_aff_rcv)  w  \
                                "P25_Repeat_SimUI::update_aff_rcv  $tile_path"

    trace variable  ::P25ns::State(trunk_signal_rcv)  w  \
                                "P25_Repeat_SimUI::update_signal_rcv  $tile_path"

    trace variable  ::P25ns::State(trunk_state)  w  \
                            "P25_Repeat_SimUI::update_trunk_state  $tile_path"
                            

    trace variable  ::P25ns::TrunkModeCurrent  w  \
                                   "P25_Repeat_SimUI::update_msg_type  $tile_path"

    trace variable  ::P25ns::TrunkModeCurrent  w  \
                                   "P25_Repeat_SimUI::update_band  $tile_path"

    trace variable  ::P25ns::State(expmsg_txt)  w  \
                                   "P25_Repeat_SimUI::update_msg_type  $tile_path"

    trace variable  ::P25ns::State(expmsg_txt)  w  \
                                   "P25_Repeat_SimUI::update_band  $tile_path"

    trace variable  ::P25_Repeat_SimUI::StateUI(num_format)  w  \
                             "P25_Repeat_SimUI::update_num_format  $tile_path"

    trace variable  ::P25ns::State(obtctrl)  w  \
                                            "P25_Repeat_SimUI::updateOBT_Ctrl"

    trace variable  ::P25ns::State(trunk_ptt_state)  w  \
                                            "P25_Repeat_SimUI::updatePttMode"

    update_band_core  $tile_path
    updateOBT_Ctrl
    update_num_format_core  $tile_path

    edit_fields  $tile_path  ENABLE
    
    # Determine if the Other Band Trunking option is installed, 
    # and set the menu options accordingly
    if { [IFR_options  is_installed  P25_OBT] } \
    {
      # No action
      # The menu option is on the list
    } \
    else \
    {
      # Remove the option from the menu
      if { [winfo exists ${tile_path}.frame1.mnu_band.menu] } \
      {
        catch { ${tile_path}.frame1.mnu_band.menu  delete  "700*" } result
        catch { ${tile_path}.frame1.mnu_band.menu  delete  "UHF/VHF*" } result
      }
    }

    if { [IFR_options is_installed P25_Explicit] && [IFR_options is_installed P25_UU_PSTN]} \
    {
       grid remove ${tile_path}.frame2.btn_ptt_on_off 
    } else \
    {
       grid remove ${tile_path}.frame2.mnu_ptt_uu_pstn 
    }       


    # Determine if the 800 MHz option is installed, 
    # and set the menu options accordingly
    if { [IFR_options  is_installed  P25_trunk] } \
    {
      # No action
      # The menu option is on the list
    } \
    else \
    {
      # Remove the option from the menu
      if { [winfo exists ${tile_path}.frame1.mnu_band.menu] } \
      {
        catch { ${tile_path}.frame1.mnu_band.menu  delete  "800*" } result
      }
    }
    

    # Bypass if doing a recall of a setup
    if { [info exists ::Recall_SetupsUI::State(recall_in_process)] } \
    {
       if { $::Recall_SetupsUI::State(recall_in_process) == 0 } \
       {
          ::P25ns::setTrunkModeIndex
       }
    }
    ::P25ns::set_channel_config_parms
  }


  #***************************************************************************
  #
  #        Name: Cleanup
  #
  #  Parameters: 
  #
  # Description: 
  #
  #***************************************************************************
  proc Cleanup { tile_path } \
  {
    #puts "P25_Repeat_SimUI::Cleanup:  $tile_path"
    
    # Set local variables
    set tile_path  [get_tile  $tile_path]    
    

    trace vdelete  ::P25ns::State(trunk_on_off)  w  \
                           "P25_Repeat_SimUI::update_trunk_on_off  $tile_path"

    trace vdelete  ::P25ns::State(trunk_ptt_rcv)  w  \
                                "P25_Repeat_SimUI::update_ptt_rcv  $tile_path"

    trace vdelete  ::P25ns::State(trunk_reg_rcv)  w  \
                                "P25_Repeat_SimUI::update_reg_rcv  $tile_path"

    trace vdelete  ::P25ns::State(trunk_aff_rcv)  w  \
                                "P25_Repeat_SimUI::update_aff_rcv  $tile_path"

    trace vdelete  ::P25ns::State(trunk_signal_rcv)  w  \
                                "P25_Repeat_SimUI::update_signal_rcv  $tile_path"

    trace vdelete  ::P25ns::State(trunk_state)  w  \
                            "P25_Repeat_SimUI::update_trunk_state  $tile_path"

    trace vdelete  ::P25ns::TrunkModeCurrent  w  \
                                   "P25_Repeat_SimUI::update_msg_type  $tile_path"

    trace vdelete  ::P25ns::TrunkModeCurrent  w  \
                                   "P25_Repeat_SimUI::update_band  $tile_path"

    trace vdelete  ::P25ns::State(expmsg_txt)  w  \
                                   "P25_Repeat_SimUI::update_msg_type  $tile_path"

    trace vdelete  ::P25ns::State(expmsg_txt)  w  \
                                   "P25_Repeat_SimUI::update_band  $tile_path"

    trace vdelete  ::P25_Repeat_SimUI::StateUI(num_format)  w  \
                             "P25_Repeat_SimUI::update_num_format  $tile_path"

    trace vdelete  ::BBAudio::State(PTT)  w  \
                                            "P25_Repeat_SimUI::set_ptt_on_off"

    trace vdelete  ::P25ns::State(obtctrl)  w  \
                                            "P25_Repeat_SimUI::updateOBT_Ctrl"

    trace vdelete  ::P25ns::State(trunk_ptt_state)  w  \
                                            "P25_Repeat_SimUI::updatePttMode"

  }


  #***************************************************************************
  #
  #        Name: Unmap
  #
  #  Parameters: 
  #
  # Description: 
  #
  #***************************************************************************
  proc Unmap { tile_path } \
  {
    #puts "P25_Repeat_SimUI::Unmap:  $tile_path"
    
    # Set local variables
    set tile_path  [get_tile  $tile_path]    
    
    Cleanup  $tile_path
    
    return
  }


  #***************************************************************************
  #
  #        Name: Destroy
  #
  #  Parameters: 
  #
  # Description: 
  #
  #***************************************************************************
  proc Destroy { tile_path } \
  {
    #puts "P25_Repeat_SimUI::Destroy:  $tile_path"
    
    # Set local variables
    set tile_path  [get_tile  $tile_path]    

    Cleanup  $tile_path

    return
  }


  #***************************************************************************
  #
  #        Name: Register
  #
  #  Parameters: 
  #
  # Description: 
  #
  #***************************************************************************
  proc Register { } \
  {
    #puts "P25_Repeat_SimUI::Register"

    # Set local variables
    set tile_path  [get_tile  $tile_path]    

    return
  }
  

  #***************************************************************************
  #
  #        Name: get_tile
  #
  #  Parameters: any child widget of the tile
  #
  # Description: Get the name of the tile's parent frame
  #
  #***************************************************************************
  proc get_tile { widget } \
  {
    while { 1 } \
    {
      switch -- [winfo class $widget] \
      {
        "P25_REPEAT_SIM"                -
        "P25_REPEAT_SIM_DWNLNK_CONFIG"  -
        "P25_CHANNEL_CONFIG"            -
        "P25_REPEAT_SIM_UPLNK_DATA"     { return $widget }
      }
        
      set widget [winfo parent $widget]
    }

    return     
  }


  #***************************************************************************
  #
  #        Name:  update_ptt_rcv
  #
  #  Parameters:
  #
  # Description:  Displays the PTT REC indicator
  #
  #***************************************************************************
  proc update_ptt_rcv { tile_path  args } \
  {
     set cmd [namespace code "update_ptt_rcv_core $tile_path"]
     after cancel $cmd
     after idle $cmd
  }
  proc update_ptt_rcv_core { tile_path  } \
  {
    #puts "update_ptt_rcv:  $tile_path"
    
    variable  StateUI
    
    if { $::P25ns::State(trunk_ptt_rcv) == 1 } \
    {
      ${tile_path}.frame2.lbl_ptt_rec config -text "PTT REC" -bg green  \
                                             -fg black

      ${tile_path}.frame2.btn_ptt_on_off config -state disabled
    } \
    else \
    {
      ${tile_path}.frame2.lbl_ptt_rec config -text "PTT REC" -bg gray  \
                                             -fg gray60

      set StateUI(ptt_menu_var) "Off"

      ${tile_path}.frame2.btn_ptt_on_off config -state normal
    }
  }


  #***************************************************************************
  #
  #        Name:  update_reg_rcv
  #
  #  Parameters:
  #
  # Description:  Displays the REG REC indicator
  #
  #***************************************************************************
  proc update_reg_rcv { tile_path  args } \
  {
     set cmd [namespace code "update_reg_rcv_core $tile_path"]
     after cancel $cmd
     after idle $cmd
  }

  proc update_reg_rcv_core { tile_path } \
  {
    #puts "update_reg_rcv:  $tile_path "
    
    variable  StateUI
    
    if { $::P25ns::State(trunk_reg_rcv) == 1 } \
    {
      ${tile_path}.frame2.lbl_reg_aff_rec config -text "REG REC" \
                                                 -bg green       \
                                                 -fg black
      set cmd [namespace code "update_rcv_core2 $tile_path"]
      after cancel $cmd
      after 2000 $cmd
    }
  }
  proc update_rcv_core2 { tile_path } \
  {
      ${tile_path}.frame2.lbl_reg_aff_rec config -text "REG/AFF" \
                                                 -bg gray        \
                                                 -fg gray60
  }

  #***************************************************************************
  #
  #        Name:  update_aff_rcv
  #
  #  Parameters:
  #
  # Description:  Displays the AFF REC indicator
  #
  #***************************************************************************
  proc update_aff_rcv { tile_path  args } \
  {
     set cmd [namespace code "update_aff_rcv_core $tile_path"]
     after cancel $cmd
     after idle $cmd
  }
  proc update_aff_rcv_core { tile_path } \
  {

    #puts "update_aff_rcv:  $tile_path,  $args"
    
    variable  StateUI
    
    if { $::P25ns::State(trunk_aff_rcv) == 1 } \
    {
      ${tile_path}.frame2.lbl_reg_aff_rec config -text "AFF REC" \
                                                 -bg green       \
                                                 -fg black

      set cmd [namespace code "update_rcv_core2 $tile_path"]
      after cancel $cmd
      after 5000 $cmd
    }
  }

  #***************************************************************************
  #
  #        Name:  update_signal_rcv
  #
  #  Parameters:
  #
  # Description:  Displays the UU REC indicator
  #
  #***************************************************************************
  proc update_signal_rcv { tile_path  args } \
  {
    variable  StateUI
    
    set cmd "${tile_path}.frame2.lbl_ptt_rec config -bg gray -fg gray60"

#    puts "$cmd"
    if { $::P25ns::State(trunk_signal_rcv) == 0 } \
    {
       after cancel $cmd
       $cmd
       return
    } \
    elseif { $::P25ns::State(trunk_signal_rcv) == 1 } \
    {
       ${tile_path}.frame2.lbl_ptt_rec config -text "UU REC" -bg green -fg black
    } \
    elseif { $::P25ns::State(trunk_signal_rcv) == 2 } \
    {
       ${tile_path}.frame2.lbl_ptt_rec config -text "DIAL REC" -bg green -fg black
    } \
    elseif { $::P25ns::State(trunk_signal_rcv) == 3 } \
    {
       ${tile_path}.frame2.lbl_ptt_rec config -text "PSTN REC" -bg green -fg black
    } \
    elseif { $::P25ns::State(trunk_signal_rcv) == 4 } \
    {
       ${tile_path}.frame2.lbl_ptt_rec config -text "R_WAIT" -bg green -fg black
    } \
    elseif { $::P25ns::State(trunk_signal_rcv) == 5 } \
    {
       ${tile_path}.frame2.lbl_ptt_rec config -text "R_GO" -bg green -fg black
     } \
    elseif { $::P25ns::State(trunk_signal_rcv) == 6 } \
    {
       ${tile_path}.frame2.lbl_ptt_rec config -text "R_DENY" -bg green -fg black
    }
   
    after cancel $cmd
    after 5000 $cmd
    return
  }

  #***************************************************************************
  #
  #        Name:  update_dial_rcv
  #
  #  Parameters:
  #
  # Description:  Displays the DIAL REC indicator
  #
  #***************************************************************************
  proc update_dial_rcv { tile_path  args } \
  {
    #puts "update_dial_rcv:  $tile_path,  $args"
    
    variable  StateUI
    
    if { $::P25ns::State(trunk_dial_rcv) == 1 } \
    {
      ${tile_path}.frame2.lbl_ptt_rec config -text "DIAL REC" \
                                                 -bg green       \
                                                 -fg black
      after 5000
      
      ${tile_path}.frame2.lbl_ptt_rec config -text "DIAL REC" \
                                                 -bg gray        \
                                                 -fg gray60
    }
    
    return
  }

  #***************************************************************************
  #
  #        Name:  update_pstn_rcv
  #
  #  Parameters:
  #
  # Description:  Displays the PSTN REC indicator
  #
  #***************************************************************************
  proc update_pstn_rcv { tile_path  args } \
  {
    #puts "update_pstn_rcv:  $tile_path,  $args"
    
    variable  StateUI
    
    if { $::P25ns::State(trunk_pstnl_rcv) == 1 } \
    {
      ${tile_path}.frame2.lbl_ptt_rec config -text "PSTN REC" \
                                                 -bg green       \
                                                 -fg black
      after 5000
      
      ${tile_path}.frame2.lbl_ptt_rec config -text "PSTN REC" \
                                                 -bg gray        \
                                                 -fg gray60
    }
  }


  #*************************************************************************** 
  #
  #        Name: set_band
  #
  #  Parameters:
  # 
  # Description: Sets the frequency band of the simulator
  #
  #***************************************************************************
  proc set_band { } \
  {
    # Turn off the trunking simulation, and then set up the new band
    P25ns::EnableTrunking "Disable"
    P25ns::setTrunkModeIndex
  }


  #*************************************************************************** 
  #
  #        Name: update_msg_type
  #
  #  Parameters: tile_path - tile path of the invoking widget
  # 
  # Description: Updates the screens when Implicit/Explicit mode changes 
  #
  #***************************************************************************
  proc update_msg_type { tile_path  args } \
  {
     set cmd [namespace code "update_msg_type_core $tile_path"]
     after cancel $cmd
     after idle $cmd
  }
  proc update_msg_type_core { tile_path } \
  {
    #puts "update_msg_type:  $tile_path"
  
    variable  StateUI

    set inputTilePath $tile_path

    if { ![winfo exists $tile_path] }  {return}
    set top_level  [winfo toplevel  $tile_path]

    if { $top_level == "." } \
    {
      set tile_path  .p25_repeat_sim_dwnlnk_config
    } \
    else \
    {
      set tile_path  [name_child  $top_level  p25_repeat_sim_dwnlnk_config]
    }

    if { ![winfo exists $tile_path] }  {return}

    # Note: Things needed to setup the downlink configuration screen
    #   1) We need to remove unused frames
    #   2) Configure data and size of SCCB frame
    #   3) Configure data and size of horizontal autocopy frame (if used)
    #   4) Configure the button frame
    #   5) Determine which frames are displayed
    #   6) Determine the location of the frames and display them
    #   7) Adjust the size of the configuration frame

    set frame_adjust 0
    set border  [option get ${tile_path} borderWidth {}]
    set ext_frame_row  [option get ${tile_path}.ext_frame row {}]
    set sccb_frame_row [option get ${tile_path}.sccb_frame row {}]
    set adj_frame_row  [option get ${tile_path}.adj_frame row {}]
    set temp3 [option get ${tile_path}.button_frame row {}]
    set expert_frame_h_rowspan [option get ${tile_path}.expert_frame_h rowspan {}]
    set adj_frame_rowspan [option get ${tile_path}.adj_frame rowspan {}]

    set ext_delta [expr $sccb_frame_row - $ext_frame_row]
    set sccb_delta [expr $adj_frame_row - $sccb_frame_row]
    set expert_delta [expr $adj_frame_rowspan - $expert_frame_h_rowspan]

    # Initially remove all optional frames
    grid remove ${tile_path}.ext_frame       
    grid remove ${tile_path}.adj_frame       
    grid remove ${tile_path}.expert_frame_v  
    grid remove ${tile_path}.expert_frame_h  
    grid remove ${tile_path}.sccb_frame.sccb_subframe1       
    grid remove ${tile_path}.sccb_frame.sccb_subframe3       
    grid remove ${tile_path}.expert_frame_h.chk_dial_h         
    grid remove ${tile_path}.expert_frame_h.chk_unit_h         
    if {$P25ns::State(expmsg_txt) == "EXPLICIT"} \
    {
       # Display data if the extended messaging option is installed, and on the UHF/VHF band
       if { ([IFR_options is_installed P25_UU_PSTN]) && ($::P25ns::TrunkModeCurrent == "UHF/VHF") } \
       {
          grid config ${tile_path}.ext_frame -row $ext_frame_row
       } else \
       {
          set frame_adjust $ext_delta
       }

       if { [IFR_options is_installed P25_SCCB] } \
       {
          # Adjust the position of the SCCB frame if needed
          set temp [option get ${tile_path}.sccb_frame.sccb_subframe1 row {}]
          grid config ${tile_path}.sccb_frame.sccb_subframe1 -row $temp
          set temp [option get ${tile_path}.sccb_frame.sccb_subframe2 row {}]
          grid config ${tile_path}.sccb_frame.sccb_subframe2 -row $temp
          set temp [option get ${tile_path}.sccb_frame.sccb_subframe3 row {}]
          grid config ${tile_path}.sccb_frame.sccb_subframe3 -row $temp
          set temp [expr $sccb_frame_row - $frame_adjust]
          set row_span [option get ${tile_path}.sccb_frame rowspan {}]
          grid config ${tile_path}.sccb_frame -row $temp -rowspan $row_span
       } else \
       {
          grid remove ${tile_path}.sccb_frame
          set frame_adjust [expr $frame_adjust + $sccb_delta]
       }

       # Check for display of the Adj site and Auto Copy frames
       # If displaying Adj site, the autocopy is displayed beside it
       # If just displaying the autocopy, it is displayed horizontal
       set temp [expr $adj_frame_row - $frame_adjust]
       if { [IFR_options is_installed P25_Adj] } \
       {
          grid config ${tile_path}.adj_frame -row $temp
          grid config ${tile_path}.expert_frame_v -row $temp
       } else \
       {
          set row_span [option get ${tile_path}.expert_frame_h rowspan {}]
          grid config ${tile_path}.expert_frame_h -row $temp -rowspan $row_span
          Screens::position_widget  ${tile_path}.expert_frame_h.chk_dial_h
          Screens::position_widget  ${tile_path}.expert_frame_h.chk_unit_h
          set frame_adjust [expr $frame_adjust + $expert_delta]
       }
    } else \
    {
       if { [IFR_options is_installed P25_SCCB] } \
       {
          # Adjust the position of the SCCB frame to where the extended frame was
          set temp1 [option get ${tile_path}.sccb_frame.sccb_subframe1 row {}]
          set temp2 [option get ${tile_path}.sccb_frame.sccb_subframe2 row {}]
          set temp [expr ($temp2 - $temp1) * 2]
          grid config ${tile_path}.sccb_frame.sccb_subframe2 -row $temp1
          set row_span [option get ${tile_path}.sccb_frame rowspan {}]
          set row_span [expr $row_span - $temp]
          grid config ${tile_path}.sccb_frame -row $ext_frame_row -rowspan $row_span
          set frame_adjust [expr $ext_delta + $temp]
       } else \
       {
          grid remove ${tile_path}.sccb_frame
          set frame_adjust [expr $frame_adjust + $sccb_delta + $ext_delta]
       }

       # Adjust the position and size of the Autocopy frame
       set size_delta [option get ${tile_path}.expert_frame_h.chk_dial_h rowspan {}]
       set row_span [option get ${tile_path}.expert_frame_h rowspan {}]
       set row_span [expr $row_span - $size_delta]
       set temp [expr $adj_frame_row - $frame_adjust]
       grid config ${tile_path}.expert_frame_h -row $temp -rowspan $row_span
       set frame_adjust [expr $frame_adjust + $expert_delta + $size_delta]
    }

    if {$P25ns::State(expmsg_txt) != "EXPLICIT"} \
    {
       grid remove ${tile_path}.button_frame.btn_msgtype 
       grid remove ${tile_path}.button_frame.mnu_terminator   
       if { (![IFR_options is_installed P25_Explicit]) || ($::P25ns::TrunkModeCurrent != "UHF/VHF") } \
       {
          set temp1 [option get ${tile_path}.button_frame.btn_hex_dec column {}]
          set temp2 [option get ${tile_path}.button_frame.btn_hex_dec columnspan {}]
       } else \
       {
          set temp1 [option get ${tile_path}.button_frame.btn_msgtype column {}]
          set temp2 [option get ${tile_path}.button_frame.btn_msgtype columnspan {}]
       }
           
       set temp3 [option get ${tile_path}.button_frame columnspan {}]
       set delta [expr ($temp3 - ($temp1 + $temp2)) / 2]

       set temp1 [option get ${tile_path}.button_frame.btn_return column {}]
       set temp2 [expr $temp1 + $delta]
       grid config ${tile_path}.button_frame.btn_return -column $temp2

       set temp1 [option get ${tile_path}.button_frame.btn_defaults column {}]
       set temp2 [expr $temp1 + $delta]
       grid config ${tile_path}.button_frame.btn_defaults -column $temp2

       set temp1 [option get ${tile_path}.button_frame.btn_hex_dec column {}]
       set temp2 [expr $temp1 + $delta]
       grid config ${tile_path}.button_frame.btn_hex_dec -column $temp2

       if { ([IFR_options is_installed P25_Explicit]) && ($::P25ns::TrunkModeCurrent == "UHF/VHF") } \
       {
          set temp1 [option get ${tile_path}.button_frame.btn_msgtype column {}]
          set temp2 [expr $temp1 + $delta]
          grid config ${tile_path}.button_frame.btn_msgtype -column $temp2
       }
    } else \
    {
       Screens::position_widget  ${tile_path}.button_frame.btn_return
       Screens::position_widget  ${tile_path}.button_frame.btn_defaults
       Screens::position_widget  ${tile_path}.button_frame.btn_hex_dec
       Screens::position_widget  ${tile_path}.button_frame.btn_msgtype
       Screens::position_widget  ${tile_path}.button_frame.mnu_terminator
    }


    # Move position of button frame based on options
    if {$frame_adjust >= 0} \
    {
       # Move position of button frame based on options
       set temp [option get ${tile_path}.button_frame row {}]
       set temp [expr $temp - $frame_adjust]
       grid config ${tile_path}.button_frame -row $temp

       # Change height of main frame based on options
       set height [option get ${tile_path} height {}]
       set width [option get ${tile_path} width {}]
       set height [expr $height + (2 * $border) - $frame_adjust]
       after idle "wm geo ${tile_path} ${width}x${height}"
    }
  }
  #*************************************************************************** 
  #
  #        Name: update_band
  #
  #  Parameters: tile_path - tile path of the invoking widget
  # 
  # Description: Updates the frequency band of the simulator
  #
  #***************************************************************************
  proc update_band { tile_path  args } \
  {
     set cmd [namespace code "update_band_core $tile_path"]
     after cancel $cmd
     after idle $cmd
  }
  proc update_band_core { tile_path } \
  {
    #puts "update_band:  $tile_path"
  
    variable  StateUI
    
    if { ![winfo exists $tile_path] }  {return}
    set top_level  [winfo toplevel  $tile_path]

    set top_level  [winfo toplevel  $tile_path]
    set classpath  [winfo class $tile_path]
    set classtop   [winfo class $top_level]

    if { $top_level == "." } \
    {
      set tile_path  .p25_repeat_sim
    } \
    else \
    {
       if { $classtop == "IFR-2975" } \
       {
          set tile_path [name_child $top_level p25_repeat_sim]
       } else \
       {
          set tile_path  $top_level
       }
    }

    if { [winfo class $tile_path] != "P25_REPEAT_SIM" } \
    {
       puts "Hit update_band_core with converted path of $tile_path"
       return
    }

    set frame_path  ${tile_path}.frame2
    
    grid remove  ${frame_path}.lbl_cctx
    grid remove  ${frame_path}.ent_cctx_800
    grid remove  ${frame_path}.ent_cctx_freq_800
    grid remove  ${frame_path}.lbl_vctx
    grid remove  ${frame_path}.ent_vctx_800
    grid remove  ${frame_path}.ent_vctx_freq_800
    grid remove  ${frame_path}.lbl_cctx
    grid remove  ${frame_path}.ent_cctx_700
    grid remove  ${frame_path}.ent_cctx_freq_700
    grid remove  ${frame_path}.lbl_vctx
    grid remove  ${frame_path}.ent_vctx_700
    grid remove  ${frame_path}.ent_vctx_freq_700
    grid remove  ${frame_path}.lbl_cctx_UV
    grid remove  ${frame_path}.lbl_cctx_UVi
    grid remove  ${frame_path}.ent_cctxid_UV
    grid remove  ${frame_path}.ent_cctx_UV
    grid remove  ${frame_path}.ent_cctx_UVi
    grid remove  ${frame_path}.ent_cctx_freq_UV
    grid remove  ${frame_path}.ent_cctx_freq_UVi
    grid remove  ${frame_path}.lbl_ccrx_UV
    grid remove  ${frame_path}.ent_ccrxid_UV
    grid remove  ${frame_path}.ent_ccrx_UV
    grid remove  ${frame_path}.ent_ccrx_freq_UV
    grid remove  ${frame_path}.lbl_vctx_UV
    grid remove  ${frame_path}.lbl_vctx_UVi
    grid remove  ${frame_path}.ent_vctxid_UV
    grid remove  ${frame_path}.ent_vctx_UV
    grid remove  ${frame_path}.ent_vctx_UVi
    grid remove  ${frame_path}.ent_vctx_freq_UV
    grid remove  ${frame_path}.ent_vctx_freq_UVi
    grid remove  ${frame_path}.lbl_vcrx_UV
    grid remove  ${frame_path}.ent_vcrxid_UV
    grid remove  ${frame_path}.ent_vcrx_UV
    grid remove  ${frame_path}.ent_vcrx_freq_UV
    grid remove  ${frame_path}.btn_config_freq

    grid remove  ${frame_path}.lbl_cctx_units
    grid remove  ${frame_path}.lbl_ccrx_units
    grid remove  ${frame_path}.lbl_vctx_units
    grid remove  ${frame_path}.lbl_vcrx_units

    switch -- $::P25ns::TrunkModeCurrent \
    {
      "800"      { set StateUI(band_short)  "800" 

                   grid remove ${tile_path}.frame2.mnu_ptt_uu_pstn 
                   Screens::position_widget  ${frame_path}.btn_ptt_on_off

                   Screens::position_widget  ${frame_path}.lbl_cctx
                   Screens::position_widget  ${frame_path}.ent_cctx_800
                   Screens::position_widget  ${frame_path}.ent_cctx_freq_800
                   Screens::position_widget  ${frame_path}.lbl_cctx_units

                   Screens::position_widget  ${frame_path}.lbl_vctx
                   Screens::position_widget  ${frame_path}.ent_vctx_800
                   Screens::position_widget  ${frame_path}.ent_vctx_freq_800
                   Screens::position_widget  ${frame_path}.lbl_vctx_units

                   set StateUI(cctx_label)  "CC"
                   set StateUI(ccrx_label)  ""
                   set StateUI(vctx_label)  "VC"
                   set StateUI(vcrx_label)  "" }
      
      "700"      { set StateUI(band_short)  "700"

                   grid remove ${tile_path}.frame2.mnu_ptt_uu_pstn 
                   Screens::position_widget  ${frame_path}.btn_ptt_on_off

                   Screens::position_widget  ${frame_path}.lbl_cctx
                   Screens::position_widget  ${frame_path}.ent_cctx_700
                   Screens::position_widget  ${frame_path}.ent_cctx_freq_700
                   Screens::position_widget  ${frame_path}.lbl_cctx_units

                   Screens::position_widget  ${frame_path}.lbl_vctx
                   Screens::position_widget  ${frame_path}.ent_vctx_700
                   Screens::position_widget  ${frame_path}.ent_vctx_freq_700
                   Screens::position_widget  ${frame_path}.lbl_vctx_units

                   Screens::position_widget  ${frame_path}.btn_config_freq

                   set StateUI(cctx_label)  "CC"
                   set StateUI(ccrx_label)  ""
                   set StateUI(vctx_label)  "VC"
                   set StateUI(vcrx_label)  "" }

      "UHF/VHF"  { set StateUI(band_short)  "U/V"

                   if {$P25ns::State(expmsg_txt) == "EXPLICIT"} \
                   {
                      if { [IFR_options is_installed P25_UU_PSTN] } \
                      {
                         grid remove ${tile_path}.frame2.btn_ptt_on_off 
                         Screens::position_widget  ${frame_path}.mnu_ptt_uu_pstn
                      } else \
                      {
                         grid remove ${tile_path}.frame2.mnu_ptt_uu_pstn 
                         Screens::position_widget  ${frame_path}.btn_ptt_on_off
                      }

                      Screens::position_widget  ${frame_path}.lbl_cctx_UV
                      Screens::position_widget  ${frame_path}.ent_cctxid_UV
                      Screens::position_widget  ${frame_path}.ent_cctx_UV
                      Screens::position_widget  ${frame_path}.ent_cctx_freq_UV

                      Screens::position_widget  ${frame_path}.lbl_ccrx_UV
                      Screens::position_widget  ${frame_path}.ent_ccrxid_UV
                      Screens::position_widget  ${frame_path}.ent_ccrx_UV
                      Screens::position_widget  ${frame_path}.ent_ccrx_freq_UV

                      Screens::position_widget  ${frame_path}.lbl_vctx_UV
                      Screens::position_widget  ${frame_path}.ent_vctxid_UV
                      Screens::position_widget  ${frame_path}.ent_vctx_UV
                      Screens::position_widget  ${frame_path}.ent_vctx_freq_UV

                      Screens::position_widget  ${frame_path}.lbl_vcrx_UV
                      Screens::position_widget  ${frame_path}.ent_vcrxid_UV
                      Screens::position_widget  ${frame_path}.ent_vcrx_UV
                      Screens::position_widget  ${frame_path}.ent_vcrx_freq_UV
                   } else \
                   {

                      grid remove ${tile_path}.frame2.mnu_ptt_uu_pstn 
                      Screens::position_widget  ${frame_path}.btn_ptt_on_off

                      Screens::position_widget  ${frame_path}.lbl_cctx_UVi
                      Screens::position_widget  ${frame_path}.ent_cctx_UVi
                      Screens::position_widget  ${frame_path}.ent_cctx_freq_UVi
                      Screens::position_widget  ${frame_path}.lbl_cctx_units

                      Screens::position_widget  ${frame_path}.lbl_vctx_UVi
                      Screens::position_widget  ${frame_path}.ent_vctx_UVi
                      Screens::position_widget  ${frame_path}.ent_vctx_freq_UVi
                      Screens::position_widget  ${frame_path}.lbl_vctx_units
                   }

                   Screens::position_widget  ${frame_path}.btn_config_freq
                   
                   # Disable full settings until Motorola adds multiblock messages
                   #set StateUI(cctx_label)  "CCTx"
                   #set StateUI(ccrx_label)  "CCRx"
                   #set StateUI(vctx_label)  "VCTx"
                   #set StateUI(vcrx_label)  "VCRx" 

                   set StateUI(cctx_label)  "CC"
                   set StateUI(ccrx_label)  ""
                   set StateUI(vctx_label)  "VC"
                   set StateUI(vcrx_label)  "" }

      "USER"     { set StateUI(band_short)  "USER" }
    }

    ::P25ns::SyncFreq


    return
  }

  #*************************************************************************** 
  #
  #        Name: display_band_config
  #
  #  Parameters: tile_path - tile path of the invoking widget
  # 
  # Description: Displays the configuration toplevel for the channel schedule
  #
  #***************************************************************************
  proc display_band_config { tile_path } \
  {
    #puts "display_band_config:  $tile_path"
  
    variable  StateUI
    
    variable  last_focus_2
    
    set last_focus_2 $tile_path

    set top_level  [winfo toplevel $tile_path]
   
    #puts "top_level:  $top_level"
    
    set tile_path  [name_child  $top_level  p25_channel_config]

    #puts "tile_path:  $tile_path"

    if { ![winfo exists $tile_path] } \
    {
      set tile_path  [::Screens::create_toplevel_tile  $tile_path  P25_CHANNEL_CONFIG]
      # Load the canvas
      load_canvas  $tile_path
    } else { \
      return
    }
    
    #puts "tile_path:  $tile_path"

    bind $tile_path <KeyRelease-F8> "destroy [winfo toplevel $tile_path]"

    bind ${tile_path}.btn_return <<Destroy>> \
                        "::P25_Repeat_SimUI::destroy_band_config  $tile_path" 

    ${tile_path}.cnvs_config config  \
                              -yscrollcommand "${tile_path}.scrl_config  set"

    ${tile_path}.scrl_config config  -command "${tile_path}.cnvs_config  yview"
    

    # Display the correct widgets
    grid remove ${tile_path}.ent_bw_other
    grid remove ${tile_path}.mnu_bw_UV
    grid remove ${tile_path}.ent_offset_other
    grid remove ${tile_path}.ent_offset_VU
    
    switch -- $::P25ns::TrunkModeCurrent \
    {
      "700"      -
      "800"      { Screens::position_widget  ${tile_path}.ent_bw_other 
                   Screens::position_widget  ${tile_path}.ent_offset_other} 

      "UHF/VHF"  { Screens::position_widget  ${tile_path}.mnu_bw_UV
                   Screens::position_widget  ${tile_path}.ent_offset_VU} 
    }
    
    safe_focus_force ${tile_path}.btn_return
    ::P25ns::set_channel_config_parms

    return
  }
  

  #***************************************************************************
  #
  #        Name: load_canvas
  #
  #  Parameters: tile_path - tile path of the invoking widget
  #
  # Description: Loads the canvas with the labels to display the channel 
  #              configuration summary.
  #
  #***************************************************************************
  proc load_canvas { tile_path } \
  {
    puts "P25 load_canvas:  $tile_path"

    variable  max_chan_index
    variable  StateUI
    
    puts "About to call get_tile with $tile_path"
    set tile_path  [get_tile  $tile_path]
    puts "After call to get_tile, path is now $tile_path"
    
    set canvas_path  ${tile_path}.cnvs_config
    
    set y_index  0
    
    for { set i  0 } { $i <= 15 } { incr i  1 } \
    {
      # Create the Channel ID 
      label  ${canvas_path}.lbl_chanid_${i} -text $i              \
                                            -font analyzer_small  \
                                            -anchor e
        
      # Place the Channel ID
      $canvas_path  create  window  26  $y_index                        \
                                -width   20                             \
                                -height  20                             \
                                -window  ${canvas_path}.lbl_chanid_${i} \
                                -anchor  nw

      # Create base frequency 
      label  ${canvas_path}.lbl_freq_base_${i}                          \
                       -textvariable  ::P25ns::ChannelConfig($i,FREQ)   \
                       -font analyzer_small                             \
                       -anchor e
            
      # Place the base frequency
      $canvas_path  create  window  55  $y_index                         \
                              -width   120                               \
                              -height  20                                \
                              -window  ${canvas_path}.lbl_freq_base_${i} \
                              -anchor  nw


      # Create bandwidth
      label  ${canvas_path}.lbl_bw_${i}                                  \
                       -textvariable  ::P25ns::ChannelConfig($i,BW)      \
                       -font analyzer_small                              \
                       -anchor e
            
      # Place the bandwidth
      $canvas_path  create  window  195  $y_index                      \
                              -width   80                              \
                              -height  20                              \
                              -window  ${canvas_path}.lbl_bw_${i}      \
                              -anchor  nw


      # Create transmit offset
      label  ${canvas_path}.lbl_offset_${i}                             \
                       -textvariable  ::P25ns::ChannelConfig($i,OFFSET) \
                       -font analyzer_small                             \
                       -anchor e
            
      # Place the transmit offset
      $canvas_path  create  window  309  $y_index                      \
                              -width   80                              \
                              -height  20                              \
                              -window  ${canvas_path}.lbl_offset_${i}  \
                              -anchor  nw


      # Create channel spacing
      label  ${canvas_path}.lbl_spacing_${i}                             \
                       -textvariable  ::P25ns::ChannelConfig($i,SPACING) \
                       -font analyzer_small                              \
                       -anchor e
            
      # Place the channel spacing
      $canvas_path  create  window  414  $y_index                      \
                              -width   80                              \
                              -height  20                              \
                              -window  ${canvas_path}.lbl_spacing_${i} \
                              -anchor  nw



      # Increment the Y index
      incr y_index  20
    }
  }

  #***************************************************************************
  #
  #        Name: toggle_scroll_on
  #
  #  Parameters: tile_path - tile path of the invoking widget
  #
  # Description: Toggles the 'scroll_on' variable to allow the spinner
  #              knob to control the scrollbar
  #
  #***************************************************************************
  proc toggle_scroll_on { tile_path } \
  {
    #puts "toggle_scroll_on:  $tile_path"
    
    variable  scroll_on
    
    set tile_path  [get_tile  $tile_path]
    
    set scrollbar_path  ${tile_path}.scrl_config
    
    if { $scroll_on == 0 } \
    {
      # Change the color of the scrollbar to indicate it is active
      $scrollbar_path config  -bg white
      
      set scroll_on  1
      
      # For each event, call the procedure to move the scrollbar,
      # then break to keep the focus from moving in response to the 
      # event
      # Bind the <Right> event  (includes <<Spin_up>>)
      bind $scrollbar_path  <Right>  "::P25_Repeat_SimUI::move_canvas  $tile_path  UP; break"
                                   
      # Bind the <Left> event  (includes <<Spin_down>>)
      bind $scrollbar_path  <Left>  "::P25_Repeat_SimUI::move_canvas  $tile_path  DOWN; break"
                                   
      bind $scrollbar_path  <Up>  "::P25_Repeat_SimUI::move_canvas  $tile_path  UP; break"
                                   
      bind $scrollbar_path  <Down>  "::P25_Repeat_SimUI::move_canvas  $tile_path  DOWN; break"
    } \
    else \
    {
      # Change the color of the scrollbar to indicate it is NOT active
      $scrollbar_path config  -bg gray

      set scroll_on  0

      # Unbind the <Right> and <Left> events
      bind $scrollbar_path  <Right>  ""
      bind $scrollbar_path  <Left>   ""
      
      # For the 'Up' of 'Down' event, generate the 'cursor_up' or 'cursor_down'
      # event, then break to keep the scrollbar from moving in response to 
      # either event
      bind $scrollbar_path  <Up>    "event generate %W <<cursor_up>>; break"
      bind $scrollbar_path  <Down>  "event generate %W <<cursor_down>>; break"
    }
  }
      
  #***************************************************************************
  #
  #        Name: move_canvas
  #
  #  Parameters: tile_path - tile path of the invoking widget
  #              direction - direction to move the scrollbar
  #
  # Description: Moves the canvas display for changes on the spinner knob 
  #              or Up and Down arrows.
  #
  #***************************************************************************
  proc move_canvas { tile_path  direction } \
  {
    #puts "move_canvas:  $tile_path,  $direction"
    
    variable  scroll_on
    
    if { $scroll_on == 0 } \
    {
      return
    }

    set tile_path  [get_tile  $tile_path]
    
    set canvas_path  ${tile_path}.cnvs_config

    # Move the canvas display
    switch -- $direction \
    {
      "UP"    { $canvas_path  yview  scroll  -1  unit }
    
      "DOWN"  { $canvas_path  yview  scroll  1  unit }
    }
  }





  #*************************************************************************** 
  #
  #        Name: display_band_config
  #
  #  Parameters: tile_path - tile path of the invoking widget
  # 
  # Description: Displays the configuration toplevel for the channel schedule
  #
  #***************************************************************************
  proc display_band_config_old { tile_path } \
  {
    #puts "display_band_config:  $tile_path"
  
    variable  StateUI
    
    variable  last_focus_2
    
    set last_focus_2 $tile_path

    set top_level  [winfo toplevel $tile_path]
   
    #puts "top_level:  $top_level"
    
    set tile_path  [name_child  $top_level  p25_chan_config]

    #puts "tile_path:  $tile_path"

    if { ![winfo exists $tile_path] } \
    {
      set tile_path  [::Screens::create_toplevel_tile  $tile_path  P25_CHAN_CONFIG]
    }
    
    #puts "tile_path:  $tile_path"

    bind $tile_path <KeyRelease-F8> "destroy [winfo toplevel $tile_path]"

    bind ${tile_path}.btn_return <<Destroy>> \
                        "::P25_Repeat_SimUI::destroy_band_config  $tile_path" 


    # Display the correct widgets
    grid remove ${tile_path}.ent_bw_other
    grid remove ${tile_path}.mnu_bw_UV
    grid remove ${tile_path}.ent_offset_other
    grid remove ${tile_path}.ent_offset_VU
    grid remove ${tile_path}.lbl_offset_units
    
    switch -- $::P25ns::TrunkModeCurrent \
    {
      "700"      -
      "800"      { Screens::position_widget  ${tile_path}.ent_bw_other 
                   Screens::position_widget  ${tile_path}.ent_offset_other 
                   
                   Screens::position_widget  ${tile_path}.lbl_offset_units }

      "UHF/VHF"  { Screens::position_widget  ${tile_path}.mnu_bw_UV
                   Screens::position_widget  ${tile_path}.ent_offset_VU 
                   Screens::position_widget  ${tile_path}.lbl_offset_units }
                   
       "USER"     { }
    }
    
    
    safe_focus_force ${tile_path}.btn_return

    return
  }
  
  
  #*************************************************************************** 
  #
  #        Name: destroy_band_config
  #
  #  Parameters: tile_path - tile path of the invoking widget
  # 
  # Description: Handles the <Destroy> event for the configuration toplevel
  #
  #***************************************************************************
  proc destroy_band_config { tile_path } \
  {
    #puts "destroy_band_config:  $tile_path"
  
    variable  StateUI

    variable  last_focus_2

    # Setup per the config values
       
    
    
    safe_focus_force  $last_focus_2

    return
  }


  #*************************************************************************** 
  #
  #        Name: set_trunk_state
  #
  #  Parameters:
  # 
  # Description: Sets the state of the simulator
  #
  #***************************************************************************
  proc set_trunk_state { } \
  {
    #puts "set_trunk_state"
  
    variable  StateUI
    
    switch -- $::P25ns::State(trunk_state) \
    {
      "ON"       { ::P25ns::EnableTrunking  Enable  }
      
      "OFF"      { ::P25ns::EnableTrunking  Disable }
               
      default    { }
    }

    return
  }


  #*************************************************************************** 
  #
  #        Name: toggle_trunk_state
  #
  #  Parameters:
  # 
  # Description: Toggles the state of the simulator between ON and OFF
  #
  #***************************************************************************
  proc toggle_trunk_state { } \
  {
    #puts "toggle_trunk_state"
  
    variable  StateUI
    
    switch -- $::P25ns::State(trunk_state) \
    {
      "OFF"    { ::P25ns::EnableTrunking  Enable  }
               
      default  { ::P25ns::EnableTrunking  Disable }
    }

    return
  }


  #*************************************************************************** 
  #
  #        Name: update_trunk_state
  #
  #  Parameters: tile_path - tile path of the invoking widget
  # 
  # Description: Updates the display to the state of the simulator
  #
  #***************************************************************************
  proc update_trunk_state { tile_path  args } \
  {
    #puts "update_trunk_state:  $tile_path,  $args"
  
    variable  StateUI
    
    set tile_path  [get_tile  $tile_path]    
    
    switch -- $::P25ns::State(trunk_state) \
    {
      "ON"       { }
      
      "OFF"      { }
               
      "MONITOR"  { # Display the Log toplevel when Monitor is selected
                   ::UplinkUI::display_log_setup  $tile_path  MONITOR }
                   
      default    { }
    }

    return
  }


  #*************************************************************************** 
  #
  #        Name: update_trunk_on_off
  #
  #  Parameters:
  # 
  # Description: Updates the display to changes in the ON / OFF status
  #
  #***************************************************************************
  proc update_trunk_on_off { tile_path  args } \
  {
    #puts "update_trunk_on_off:  $tile_path,  $args"
  
    variable  StateUI
    
    if { $::P25ns::State(trunk_on_off) == "ON" } \
    {
      ${tile_path}.frame2.btn_ptt_on_off config -state normal
    }
    
    if { $::P25ns::State(trunk_on_off) == "OFF" } \
    {
      ${tile_path}.frame2.btn_ptt_on_off config -state disabled
    }
  
    return
  }


  #*************************************************************************** 
  #
  #        Name: toggle_call_type
  #
  #  Parameters:
  # 
  # Description: Toggles the call type between ANALOG and DIGITAL
  #
  #***************************************************************************
  proc toggle_call_type { } \
  {
    #puts "toggle_call_type"
  
    variable  StateUI
    
    return
  }


  #***************************************************************************
  #
  #        Name: update_call_type
  #
  #  Parameters:
  #
  # Description: Updates the display to the call type
  #
  #***************************************************************************
  proc update_call_type { tile_path  args } \
  {
    #puts "update_call_type:  $tile_path,  $args"
    
    variable  StateUI
    
    set tile_path  [get_tile  $tile_path]
    
    return
  }


  #***************************************************************************
  #
  #        Name:  set_ptt_on_off
  #
  #  Parameters:
  #
  # Description:  Toggles the PTT ON/OFF display - This is for the simple
  #     mode where the PTT only activates a voice call.
  #
  #***************************************************************************
  proc set_ptt_on_off { args } \
  {
    #puts "set_ptt_on_off"
    
    variable  StateUI
    
    switch -- $::BBAudio::State(PTT) \
    {
      1        { after idle "P25 encode trunkpttenable  ON -nowait"  }

      default  { after idle "P25 encode trunkpttenable  OFF -nowait" }
    }

    return
  }


  #***************************************************************************
  #
  #        Name:  toggle_ptt
  #
  #  Parameters:
  #
  # Description:  Toggles the PTT ON/OFF display - This is for the simple 
  #    mode where the PTT only activates a voice call
  #
  #***************************************************************************
  proc toggle_ptt { } \
  {
    #puts "toggle_ptt"
    
    variable  StateUI
    
    if { $::P25ns::State(trunk_ptt_on_off) == "PTT OFF" } \
    {
      P25 encode trunkpttenable  ON -nowait
    } \
    else \
    {
      P25 encode trunkpttenable  OFF -nowait
    }

    return
  }

  #*************************************************************************** 
  #
  #        Name: setPttType
  #
  #  Parameters: None
  # 
  # Description: Used to simulate the pressing of the PTT on a remote radio, 
  #     which is used to activate a "simulated" group, unit to unit, or PSTN
  #     call.
  #
  #***************************************************************************
  proc setPttType { } \
  {
     variable  StateUI

     if { $::P25ns::State(trunk_on_off) == "OFF" } \
     {
        set StateUI(ptt_menu_var) "Off"
        return
     }

     if {$StateUI(ptt_menu_var) == "Off"} \
     {
        P25 encode trunkpttenable 0
     } elseif {$StateUI(ptt_menu_var) == "Group Call"} \
     {
        P25 encode trunkpttenable 1
     } elseif {$StateUI(ptt_menu_var) == "Unit to Unit"} \
     {
        P25 encode trunkpttenable 2
     } elseif {$StateUI(ptt_menu_var) == "Dialing"} \
     {
        P25 encode trunkpttenable 3
     } elseif {$StateUI(ptt_menu_var) == "PSTN"} \
     {
         P25 encode trunkpttenable 4
     }
  }

  #*************************************************************************** 
  #
  #        Name: updatePttMode
  #
  #  Parameters: None
  # 
  # Description: Used to simulate the pressing of the PTT on a remote radio, 
  #     which is used to activate a "simulated" group, unit to unit, or PSTN
  #     call.
  #
  #***************************************************************************
  proc updatePttMode { args } \
  {
     variable  StateUI

     if {$::P25ns::State(trunk_ptt_state) == 0} \
     {
        set StateUI(ptt_menu_txt) "PTT OFF"
     } elseif {$::P25ns::State(trunk_ptt_state) == 1} \
     {
        set StateUI(ptt_menu_txt) "PTT ON"
     } elseif {$::P25ns::State(trunk_ptt_state) == 2} \
     {
        set StateUI(ptt_menu_txt) "UU ON"
     } elseif {$::P25ns::State(trunk_ptt_state) == 3} \
     {
        set StateUI(ptt_menu_txt) "DIAL"
     } elseif {$::P25ns::State(trunk_ptt_state) == 4} \
     {
        set StateUI(ptt_menu_txt) "PSTN"
     }
  }



  #***************************************************************************
  #
  #        Name:  set_channels
  #
  #  Parameters:
  #
  # Description:  Sets the channels numbers when entered
  #
  #***************************************************************************
  proc set_channels { } \
  {
    #puts "set_channels"
    
    variable  StateUI
    
    P25 encode field channum_c  $::P25ns::EncodeParm(CHANNUM_C_dec)
    
    P25 encode field channum_v  $::P25ns::EncodeParm(CHANNUM_V_dec)

    return
  }


  #***************************************************************************
  #
  #        Name:  update_800_cctx_freq
  #
  #  Parameters:
  #
  # Description:  Updates the frequency entry for changes in the channel
  #
  #***************************************************************************
  proc update_800_cctx_freq { args } \
  {
    #puts "update_800_cctx_freq:  $args"
    
    variable  StateUI
    
    return
  }


  #***************************************************************************
  #
  #        Name:  update_800_vctx_freq
  #
  #  Parameters:
  #
  # Description:  Updates the frequency entry for changes in the channel
  #
  #***************************************************************************
  proc update_800_vctx_freq { args } \
  {
    #puts "update_800_vctx_freq:  $args"
    
    variable  StateUI
    
    return
  }

  #****************************************************************************
  # Name:        setOBT_Ctrl
  # Parameters:  none
  # Description: This is called by the menu to set the desired method used to
  #     expand 2 block trunking packets to 3 block sized packets
  #****************************************************************************
   proc setOBT_Ctrl {  } \
   {
      variable  StateUI

    if { $StateUI(obtctrl) == "Expand with Nulls (DEFAULT)" } \
    {
       P25 encode obtctrl 2
    } elseif { $StateUI(obtctrl) == "Expand with FS + NID" } \
    {
       P25 encode obtctrl 6
    } else \
    {
       set StateUI(obtctrl) 0
       P25 encode obtctrl 2
    }
  }

  #****************************************************************************
  # Name:        updateOBT_Ctrl
  # Parameters:  none   (args exists to remove trace data info)
  # Description: This updates the menu variable whenever the engine changes
  #     the core information.
  #****************************************************************************
  proc updateOBT_Ctrl { args } \
  {
     variable StateUI

     if { $::P25ns::State(obtctrl) == 6 } \
     {
        set  StateUI(obtctrl) "Expand with FS + NID"
     } else \
     {
        set  StateUI(obtctrl) "Expand with Nulls (DEFAULT)"
     }
  }

  #***************************************************************************
  #
  #        Name:  set_encode_parm
  #
  #  Parameters:  array_variable - array variable of the EncodeParm array
  #           :  subsystem      - subsystem call
  #
  # Description:  Sets the Encode parameter value
  #
  #***************************************************************************
  proc set_encode_parm { array_variable  subsystem } \
  {
    #puts "set_encode_parm:  $array_variable,  $subsystem"
    
    variable  StateUI
    
    #if { $::P25ns::EncodeParm($array_variable) == "" } \
    #{
    #  set ::P25ns::EncodeParm($array_variable)  0
    #}
    
    #if { $::P25ns::EncodeParm(${array_variable}_dec) == "" } \
    #{
    #  set ::P25ns::EncodeParm(${array_variable}_dec)  0
    #}

    if { $StateUI(num_format) == "HEX" } \
    {
      P25 encode field  $subsystem  0x$::P25ns::EncodeParm($array_variable) -nowait
    }
   
    if { $StateUI(num_format) == "DEC" } \
    {
      P25 encode field  $subsystem  $::P25ns::EncodeParm(${array_variable}_dec) -nowait
    }
    
    return
  }

  #***************************************************************************
  #
  #        Name:  set_encode_parm_fixed
  #
  #  Parameters:  array_variable - array variable of the EncodeParm array
  #           :  subsystem      - subsystem call
  #           :  format         - DEC or HEX
  #
  # Description:  Sets the Encode parameter value
  #
  #***************************************************************************
  proc set_encode_parm_fixed { array_variable  subsystem {format DEC } } \
  {
    if { $format == "HEX" } \
    {
      P25 encode field  $subsystem  0x$::P25ns::EncodeParm($array_variable) -nowait
    }
   
    if { $format == "DEC" } \
    {
      P25 encode field  $subsystem  $::P25ns::EncodeParm(${array_variable}_dec) -nowait
    }
  }


  #***************************************************************************
  #
  #        Name:  toggle_num_format
  #
  #  Parameters:
  #
  # Description:  Toggles the entries to display either HEX of DECIMAL values
  #
  #***************************************************************************
  proc toggle_num_format { } \
  {
    #puts "toggle_num_format"
    
    variable  StateUI
    
    if { $StateUI(num_format) == "HEX" } \
    {
      set StateUI(num_format)  "DEC"
    } \
    else \
    {
      set StateUI(num_format)  "HEX"
    }
    
    return
  }


  #***************************************************************************
  #
  #        Name: update_num_format
  #
  #  Parameters:
  #
  # Description: Updates the display for the format selected
  #
  #***************************************************************************
  proc update_num_format { tile_path  args } \
  {
   set cmd [namespace code "update_num_format_core $tile_path"]
   after cancel $cmd
   after idle $cmd
  }
  proc update_num_format_core { tile_path } \
  {
    #puts "update_num_format:  $tile_path"

    variable  StateUI
    
    if { ![winfo exists $tile_path] }  {return}
    set top_level  [winfo toplevel  $tile_path]

    set tile_path  [get_tile  $tile_path]

    grid remove ${tile_path}.frame4_1.lbl_g_id_val
    grid remove ${tile_path}.frame4_1.lbl_g_id_val_hex

    grid remove ${tile_path}.frame4_1.lbl_u_id_val
    grid remove ${tile_path}.frame4_1.lbl_u_id_val_hex

    grid remove ${tile_path}.frame5.ent_wacn_id
    grid remove ${tile_path}.frame5.ent_wacn_id_hex

    grid remove ${tile_path}.frame5.ent_sys_id
    grid remove ${tile_path}.frame5.ent_sys_id_hex

    grid remove ${tile_path}.frame5.ent_nac
    grid remove ${tile_path}.frame5.ent_nac_hex

    grid remove ${tile_path}.frame5.ent_mfid
    grid remove ${tile_path}.frame5.ent_mfid_hex


    if { $StateUI(num_format) == "HEX" } \
    {
      Screens::position_widget ${tile_path}.frame4_1.lbl_g_id_val_hex
      Screens::position_widget ${tile_path}.frame4_1.lbl_u_id_val_hex

      Screens::position_widget ${tile_path}.frame5.ent_wacn_id_hex
      Screens::position_widget ${tile_path}.frame5.ent_sys_id_hex
      Screens::position_widget ${tile_path}.frame5.ent_nac_hex
      Screens::position_widget ${tile_path}.frame5.ent_mfid_hex
    }

    if { $StateUI(num_format) == "DEC" } \
    {
      Screens::position_widget ${tile_path}.frame4_1.lbl_g_id_val
      Screens::position_widget ${tile_path}.frame4_1.lbl_u_id_val

      Screens::position_widget ${tile_path}.frame5.ent_wacn_id
      Screens::position_widget ${tile_path}.frame5.ent_sys_id
      Screens::position_widget ${tile_path}.frame5.ent_nac
      Screens::position_widget ${tile_path}.frame5.ent_mfid
    }
    
    # Change the entry widgets on the configuration toplevel
    # if it is mapped
    if { $top_level == "." } \
    {
      if { [winfo exists .p25_repeat_sim_dwnlnk_config] } \
      {
        update_num_format_downlink_config  $tile_path
      }
    } \
    else \
    {
      # The case of a remote screen
      if { [winfo exists ${top_level}.p25_repeat_sim_dwnlnk_config] } \
      {
        update_num_format_downlink_config  $tile_path
      }
    }
    
    if { $top_level == "." } \
    {
      if { [winfo exists .p25_repeat_sim_uplnk_config] } \
      {
        update_num_format_uplink_config  $tile_path
      }
    } \
    else \
    {
      # The case of a remote screen
      if { [winfo exists ${top_level}.p25_repeat_sim_uplnk_config] } \
      {
        update_num_format_uplink_config  $tile_path
      }
    }

    return
  }


  #***************************************************************************
  #
  #        Name: update_num_format_downlink_config
  #
  #  Parameters:
  #
  # Description: Updates the configuration toplevel for the format selected
  #
  #***************************************************************************
  proc update_num_format_downlink_config { tile_path  args } \
  {
   set cmd [namespace code "update_num_format_downlink_config_core $tile_path"]
   after cancel $cmd
   after idle $cmd
  }
  proc update_num_format_downlink_config_core { tile_path } \
  {

    #puts "update_num_format_downlink_config:  $tile_path"

    variable  StateUI
    
    if { ![winfo exists $tile_path] }  {return}
    set top_level  [winfo toplevel  $tile_path]
    set classpath  [winfo class $tile_path]
    set classtop   [winfo class $top_level]

    if { $top_level == "." } \
    {
      set tile_path  .p25_repeat_sim_dwnlnk_config
    } \
    else \
    {
       if { $classtop == "IFR-2975" } \
       {
          set tile_path [name_child $top_level p25_repeat_sim_dwnlnk_config]
       } else \
       {
          set tile_path  $top_level
       }
    }
    
    grid remove ${tile_path}.ent_rfss_id         
    grid remove ${tile_path}.ent_rfss_id_hex     
    grid remove ${tile_path}.ent_site_id         
    grid remove ${tile_path}.ent_site_id_hex     
    grid remove ${tile_path}.ent_wg_id           
    grid remove ${tile_path}.ent_wg_id_hex       
    grid remove ${tile_path}.ent_wu_id           
    grid remove ${tile_path}.ent_wu_id_hex       
    grid remove ${tile_path}.ent_lra             
    grid remove ${tile_path}.ent_lra_hex         
    grid remove ${tile_path}.ent_p               
    grid remove ${tile_path}.ent_p_hex           
    grid remove ${tile_path}.ent_a               
    grid remove ${tile_path}.ent_a_hex           
    grid remove ${tile_path}.ent_lg              
    grid remove ${tile_path}.ent_lg_hex          
    grid remove ${tile_path}.ent_gav             
    grid remove ${tile_path}.ent_gav_hex         
    grid remove ${tile_path}.ent_rv              
    grid remove ${tile_path}.ent_rv_hex          
    grid remove ${tile_path}.ent_svc_class       
    grid remove ${tile_path}.ent_svc_class_hex   
    grid remove ${tile_path}.ent_svc_opt         
    grid remove ${tile_path}.ent_svc_opt_hex     
    grid remove ${tile_path}.ent_aga             
    grid remove ${tile_path}.ent_aga_hex         
    grid remove ${tile_path}.ext_frame.ent_source_addr     
    grid remove ${tile_path}.ext_frame.ent_source_addr_hex 
    grid remove ${tile_path}.ext_frame.ent_source_id       
    grid remove ${tile_path}.ext_frame.ent_source_id_hex   
    grid remove ${tile_path}.ext_frame.ent_target_addr     
    grid remove ${tile_path}.ext_frame.ent_target_addr_hex 
    grid remove ${tile_path}.ext_frame.ent_timer           
    grid remove ${tile_path}.ext_frame.ent_timer_hex       
    grid remove ${tile_path}.adj_frame.ent_adjmfid         
    grid remove ${tile_path}.adj_frame.ent_adjmfid_hex     
    grid remove ${tile_path}.adj_frame.ent_adjrfss_id      
    grid remove ${tile_path}.adj_frame.ent_adjrfss_id_hex  
    grid remove ${tile_path}.adj_frame.ent_adjsys_id       
    grid remove ${tile_path}.adj_frame.ent_adjsys_id_hex   
    grid remove ${tile_path}.adj_frame.ent_adjsite_id      
    grid remove ${tile_path}.adj_frame.ent_adjsite_id_hex  
    grid remove ${tile_path}.adj_frame.ent_adjlra          
    grid remove ${tile_path}.adj_frame.ent_adjlra_hex      
    grid remove ${tile_path}.adj_frame.ent_adjsvc_class    
    grid remove ${tile_path}.adj_frame.ent_adjsvc_class_hex
    grid remove ${tile_path}.sccb_frame.ent_sccb1_svc_class
    grid remove ${tile_path}.sccb_frame.ent_sccb1_svc_class_hex
    grid remove ${tile_path}.sccb_frame.sccb_subframe2.ent_sccb2_svc_class
    grid remove ${tile_path}.sccb_frame.sccb_subframe2.ent_sccb2_svc_class_hex

    if { $StateUI(num_format) == "HEX" } \
    {
      Screens::position_widget ${tile_path}.ent_rfss_id_hex     
      Screens::position_widget ${tile_path}.ent_site_id_hex     
      Screens::position_widget ${tile_path}.ent_wg_id_hex       
      Screens::position_widget ${tile_path}.ent_wu_id_hex       
      Screens::position_widget ${tile_path}.ent_lra_hex         
      Screens::position_widget ${tile_path}.ent_p_hex           
      Screens::position_widget ${tile_path}.ent_a_hex           
      Screens::position_widget ${tile_path}.ent_lg_hex          
      Screens::position_widget ${tile_path}.ent_gav_hex         
      Screens::position_widget ${tile_path}.ent_rv_hex          
      Screens::position_widget ${tile_path}.ent_svc_class_hex   
      Screens::position_widget ${tile_path}.ent_svc_opt_hex     
      Screens::position_widget ${tile_path}.ent_aga_hex         
      Screens::position_widget ${tile_path}.ext_frame.ent_source_id_hex   
      Screens::position_widget ${tile_path}.ext_frame.ent_source_addr_hex 
      Screens::position_widget ${tile_path}.ext_frame.ent_target_addr_hex 
      Screens::position_widget ${tile_path}.ext_frame.ent_timer_hex       
      Screens::position_widget ${tile_path}.adj_frame.ent_adjmfid_hex     
      Screens::position_widget ${tile_path}.adj_frame.ent_adjrfss_id_hex  
      Screens::position_widget ${tile_path}.adj_frame.ent_adjsys_id_hex   
      Screens::position_widget ${tile_path}.adj_frame.ent_adjsite_id_hex  
      Screens::position_widget ${tile_path}.adj_frame.ent_adjlra_hex      
      Screens::position_widget ${tile_path}.adj_frame.ent_adjsvc_class_hex
      Screens::position_widget ${tile_path}.sccb_frame.ent_sccb1_svc_class_hex
      Screens::position_widget ${tile_path}.sccb_frame.sccb_subframe2.ent_sccb2_svc_class_hex
    } \
    elseif { $StateUI(num_format) == "DEC" } \
    {
      Screens::position_widget ${tile_path}.ent_rfss_id         
      Screens::position_widget ${tile_path}.ent_site_id         
      Screens::position_widget ${tile_path}.ent_wg_id           
      Screens::position_widget ${tile_path}.ent_wu_id           
      Screens::position_widget ${tile_path}.ent_lra             
      Screens::position_widget ${tile_path}.ent_p               
      Screens::position_widget ${tile_path}.ent_a               
      Screens::position_widget ${tile_path}.ent_lg              
      Screens::position_widget ${tile_path}.ent_gav             
      Screens::position_widget ${tile_path}.ent_rv              
      Screens::position_widget ${tile_path}.ent_svc_class       
      Screens::position_widget ${tile_path}.ent_svc_opt         
      Screens::position_widget ${tile_path}.ent_aga             
      Screens::position_widget ${tile_path}.ext_frame.ent_source_id       
      Screens::position_widget ${tile_path}.ext_frame.ent_source_addr     
      Screens::position_widget ${tile_path}.ext_frame.ent_target_addr     
      Screens::position_widget ${tile_path}.ext_frame.ent_timer           
      Screens::position_widget ${tile_path}.adj_frame.ent_adjmfid         
      Screens::position_widget ${tile_path}.adj_frame.ent_adjrfss_id      
      Screens::position_widget ${tile_path}.adj_frame.ent_adjsys_id       
      Screens::position_widget ${tile_path}.adj_frame.ent_adjsite_id      
      Screens::position_widget ${tile_path}.adj_frame.ent_adjlra          
      Screens::position_widget ${tile_path}.adj_frame.ent_adjsvc_class    
      Screens::position_widget ${tile_path}.sccb_frame.ent_sccb1_svc_class
      Screens::position_widget ${tile_path}.sccb_frame.sccb_subframe2.ent_sccb2_svc_class
    }
  }


  #***************************************************************************
  #
  #        Name: update_num_format_uplink_config
  #
  #  Parameters:
  #
  # Description: Updates the configuration toplevel for the format selected
  #
  #***************************************************************************
  proc update_num_format_uplink_config { tile_path  args } \
  {
   set cmd [namespace code "update_num_format_uplink_config_core $tile_path"]
   after cancel $cmd
   after idle $cmd
  }
  proc update_num_format_uplink_config_core { tile_path } \
  {
    #puts "update_num_format_uplink_config:  $tile_path"

    variable  StateUI
    
    if { ![winfo exists $tile_path] }  {return}
    set top_level  [winfo toplevel  $tile_path]
    set classpath  [winfo class $tile_path]
    set classtop   [winfo class $top_level]

    if { $top_level == "." } \
    {
      set tile_path  .p25_repeat_sim_uplnk_config
    } \
    else \
    {
      set tile_path [name_child $top_level p25_repeat_sim_uplnk_config]
    }
    
    grid remove ${tile_path}.lbl_mfid_val
    grid remove ${tile_path}.lbl_mfid_val_hex
    
    grid remove ${tile_path}.lbl_sys_id_val
    grid remove ${tile_path}.lbl_sys_id_val_hex
    
    grid remove ${tile_path}.lbl_wacn_id_val
    grid remove ${tile_path}.lbl_wacn_id_val_hex
    
    grid remove ${tile_path}.lbl_op_code_val
    grid remove ${tile_path}.lbl_op_code_val_hex
    
    grid remove ${tile_path}.lbl_g_id_val
    grid remove ${tile_path}.lbl_g_id_val_hex
    
    grid remove ${tile_path}.lbl_u_id_val
    grid remove ${tile_path}.lbl_u_id_val_hex
    
    grid remove ${tile_path}.lbl_source_addr_val
    grid remove ${tile_path}.lbl_source_addr_val_hex
    
    grid remove ${tile_path}.lbl_source_val
    grid remove ${tile_path}.lbl_source_val_hex
    
    grid remove ${tile_path}.lbl_svc_opt_val
    grid remove ${tile_path}.lbl_svc_opt_val_hex
    
    grid remove ${tile_path}.lbl_grp_addr_val
    grid remove ${tile_path}.lbl_grp_addr_val_hex


    if { $StateUI(num_format) == "HEX" } \
    {
      Screens::position_widget ${tile_path}.lbl_mfid_val_hex
      Screens::position_widget ${tile_path}.lbl_sys_id_val_hex
      Screens::position_widget ${tile_path}.lbl_wacn_id_val_hex
      Screens::position_widget ${tile_path}.lbl_op_code_val_hex
      Screens::position_widget ${tile_path}.lbl_g_id_val_hex
      Screens::position_widget ${tile_path}.lbl_u_id_val_hex
      Screens::position_widget ${tile_path}.lbl_source_addr_val_hex
      Screens::position_widget ${tile_path}.lbl_source_val_hex
      Screens::position_widget ${tile_path}.lbl_svc_opt_val_hex
      Screens::position_widget ${tile_path}.lbl_grp_addr_val_hex
    }

    if { $StateUI(num_format) == "DEC" } \
    {
      Screens::position_widget ${tile_path}.lbl_mfid_val
      Screens::position_widget ${tile_path}.lbl_sys_id_val
      Screens::position_widget ${tile_path}.lbl_wacn_id_val
      Screens::position_widget ${tile_path}.lbl_op_code_val
      Screens::position_widget ${tile_path}.lbl_g_id_val
      Screens::position_widget ${tile_path}.lbl_u_id_val
      Screens::position_widget ${tile_path}.lbl_source_addr_val
      Screens::position_widget ${tile_path}.lbl_source_val
      Screens::position_widget ${tile_path}.lbl_svc_opt_val
      Screens::position_widget ${tile_path}.lbl_grp_addr_val
    }
    
    
    return
  }


  #***************************************************************************
  #
  #        Name:  uplink_config
  #
  #  Parameters:
  #
  # Description:  Displays the Uplink Config toplevel
  #
  #***************************************************************************
  proc uplink_config { tile_path } \
  {
    #puts "uplink_config:  $tile_path"
    
    variable  last_focus
    
    variable  StateUI
    
    set last_focus  $tile_path
    
    set top_level  [winfo toplevel $tile_path]
   
    set input_path $tile_path
    set tile_path  [name_child  $top_level  p25_repeat_sim_uplnk_config]

    if { ![winfo exists $tile_path] } \
    {
      set tile_path  [::Screens::create_toplevel_tile  $tile_path  P25_REPEAT_SIM_UPLNK_DATA]
    }
        
    wm title $tile_path  "P25 TRUNKING UPLINK DATA"
    
    # Bind the F8 key for a return
    bind $tile_path <KeyRelease-F8> "destroy [winfo toplevel $tile_path]"

    bind ${tile_path}.btn_return <Destroy> \
                     "::P25_Repeat_SimUI::destroy_uplink_config  $tile_path"

    update_num_format_uplink_config  $input_path

    safe_focus_force ${tile_path}.btn_return
    
    return
  }


  #**************************************************************************
  #
  #        Name: destroy_uplink_config
  #
  #  Parameters: 
  #
  #Description: Destroys toplevel for Uplink Config
  #
  #**************************************************************************
  proc destroy_uplink_config { tile_path } \
  {
    #puts "destroy_uplink_config:  $tile_path"
    
    variable  last_focus

    safe_focus_force $last_focus
  }


  #***************************************************************************
  #
  #        Name:  downlink_config
  #
  #  Parameters:
  #
  # Description:  Displays the Downlink Config toplevel
  #
  #***************************************************************************
  proc downlink_config { tile_path } \
  {
    #puts "downlink_config:  $tile_path"
    
    variable  last_focus
    
    variable  StateUI
    
    set last_focus  $tile_path
    set inputpath $tile_path
    
    set top_level  [winfo toplevel $tile_path]
   
    set tile_path  [name_child  $top_level  p25_repeat_sim_dwnlnk_config]

    if { ![winfo exists $tile_path] } \
    {
      set tile_path  [::Screens::create_toplevel_tile  $tile_path  P25_REPEAT_SIM_DWNLNK_CONFIG]
    }
        
    wm title $tile_path  "P25 TRUNKING DOWNLINK CONFIGURATION"
    
    # Bind the F8 key for a return
    bind $tile_path <KeyRelease-F8> "destroy [winfo toplevel $tile_path]"

    bind ${tile_path}.button_frame.btn_return <Destroy> \
                     "::P25_Repeat_SimUI::destroy_downlink_config  $tile_path"

    update_num_format_downlink_config  $inputpath
    update_msg_type $inputpath

    safe_focus_force ${tile_path}.button_frame.btn_return
    
    return
  }


  #**************************************************************************
  #
  #        Name: destroy_downlink_config
  #
  #  Parameters: 
  #
  #Description: Destroys toplevel for Downlink Config
  #
  #**************************************************************************
  proc destroy_downlink_config { tile_path } \
  {
    #puts "destroy_downlink_config:  $tile_path"
    
    variable  last_focus
    ::P25ns::SyncFreq

    safe_focus_force $last_focus
  }


  #**************************************************************************
  #
  #        Name: set_downlink_defaults
  #
  #  Parameters: 
  #
  #Description: Sets the Downlink Config values back to defaults
  #
  #**************************************************************************
  proc set_downlink_defaults { } \
  {
    #puts "set_downlink_defaults"
    
    ::P25ns::SetTrunkingDefault
    
    return
  }

  #**************************************************************************
  #
  #        Name: set_trunk_mfid
  #
  #  Parameters: 
  #
  #Description: Sets the trunking MFID - it also must force an LCO update
  #   so that the P25 LCO data is correct when switching to a voice channel
  #
  #**************************************************************************
  proc set_trunk_mfid { } \
  {
     # Force setup of the MFID in the P25 header (used for trunking MFID)
     set_encode_parm  HDR_MFID  mfid    
    
     # Force setup of the MFID and WGID in LCO = 0
     P25 encode lcomsg  0  0x$::P25ns::EncodeParm(P)         \
                           0x$::P25ns::EncodeParm(SF)        \
                           0x$::P25ns::EncodeParm(HDR_MFID)  \
                           0x$::P25ns::EncodeParm(EMG)       \
                           0x$::P25ns::EncodeParm(SERVICEOP) \
                           0x0                               \
                           0x$::P25ns::EncodeParm(WGID)      \
                           0x$::P25ns::EncodeParm(SOURCEADDR) -nowait
  }

  #**************************************************************************
  #
  #        Name: set_trunk_wgid
  #
  #  Parameters: 
  #
  #Description: Sets the trunking WGID - it also must force an LCO update
  #   so that the P25 LCO data is correct when switching to a voice channel
  #
  #**************************************************************************
  proc set_trunk_wgid { } \
  {
     # Force setup of the WGID in the engine
     set_encode_parm  WGID  wgid    
    
     # Force setup of the MFID and WGID in LCO = 0
     P25 encode lcomsg  0  0x$::P25ns::EncodeParm(P)         \
                           0x$::P25ns::EncodeParm(SF)        \
                           0x$::P25ns::EncodeParm(HDR_MFID)  \
                           0x$::P25ns::EncodeParm(EMG)       \
                           0x$::P25ns::EncodeParm(SERVICEOP) \
                           0x0                               \
                           0x$::P25ns::EncodeParm(WGID)      \
                           0x$::P25ns::EncodeParm(SOURCEADDR) -nowait
  }



  #***************************************************************************
  #
  #        Name: edit_fields
  #
  #  Parameters:
  #
  # Description: Enables and Disables the edit fields of the tile depending on
  #            : the Options status
  #
  #***************************************************************************
  proc edit_fields { tile_path  { mode  DISABLE } } \
  {
    #puts "edit_fields:  $tile_path,  $mode"
    
    if { ![winfo exists $tile_path] }  {return}

    set tile_path  [get_tile  $tile_path]
    
    
    switch -- $mode \
    {
      "ENABLE"   { # List of edit fields 
                   ${tile_path}.frame2.ent_cctx_800        config -state normal
                   ${tile_path}.frame2.ent_vctx_800        config -state normal

                   ${tile_path}.frame2.ent_cctx_700        config -state normal
                   ${tile_path}.frame2.ent_vctx_700        config -state normal

                   ${tile_path}.frame2.ent_cctx_UV         config -state normal
                   ${tile_path}.frame2.ent_ccrx_UV         config -state normal
                   ${tile_path}.frame2.ent_vctx_UV         config -state normal
                   ${tile_path}.frame2.ent_vcrx_UV         config -state normal

                   ${tile_path}.frame4_1.btn_uplink_data   config -state normal

                   ${tile_path}.frame4_2.btn_hex_dec       config -state normal

                   ${tile_path}.frame5.btn_downlink_data   config -state normal

                   ${tile_path}.frame5.ent_wacn_id         config -state normal
                   ${tile_path}.frame5.ent_wacn_id_hex     config -state normal
                                                      
                   ${tile_path}.frame5.ent_sys_id          config -state normal
                   ${tile_path}.frame5.ent_sys_id_hex      config -state normal

                   ${tile_path}.frame5.ent_nac             config -state normal
                   ${tile_path}.frame5.ent_nac_hex         config -state normal
                                                      
                   ${tile_path}.frame5.ent_mfid            config -state normal
                   ${tile_path}.frame5.ent_mfid_hex        config -state normal
                 } 
                   
      "DISABLE"  { # List of edit fields 
                   ${tile_path}.frame2.ent_cctx_800        config -state disabled
                   ${tile_path}.frame2.ent_vctx_800        config -state disabled

                   ${tile_path}.frame2.ent_cctx_700        config -state disabled
                   ${tile_path}.frame2.ent_vctx_700        config -state disabled

                   ${tile_path}.frame2.ent_cctx_UV         config -state disabled
                   ${tile_path}.frame2.ent_ccrx_UV         config -state disabled
                   ${tile_path}.frame2.ent_vctx_UV         config -state disabled
                   ${tile_path}.frame2.ent_vcrx_UV         config -state disabled

                   ${tile_path}.frame4_1.btn_uplink_data   config -state disabled

                   ${tile_path}.frame4_2.btn_hex_dec       config -state disabled

                   ${tile_path}.frame5.btn_downlink_data   config -state disabled

                   ${tile_path}.frame5.ent_wacn_id         config -state disabled
                   ${tile_path}.frame5.ent_wacn_id_hex     config -state disabled
                                                      
                   ${tile_path}.frame5.ent_sys_id          config -state disabled
                   ${tile_path}.frame5.ent_sys_id_hex      config -state disabled

                   ${tile_path}.frame5.ent_nac             config -state disabled
                   ${tile_path}.frame5.ent_nac_hex         config -state disabled
                                                      
                   ${tile_path}.frame5.ent_mfid            config -state disabled
                   ${tile_path}.frame5.ent_mfid_hex        config -state disabled
                 } 
                   
      default    {  }
    }              
                   
    return         
  }                
                   

  #****************************************************************************
  # Name:        getSaveState
  #  Parameters:  none
  # Description: This function returns a list of TCL commands which can be used
  #              to set the P25_Repeat_SimUI tiles to its current state
  #****************************************************************************
  proc getSaveState { } \
  {
    variable  StateUI

    set returnList \
    {

    }
      
    return [subst -nocommands $returnList]
  }


  #****************************************************************************
  # Name:        Sync
  #  Parameters:  none
  # Description: Write state variables to P25_Repeat_SimUI
  #****************************************************************************
  proc Sync { } \
  {
    eval [getSaveState]
  }
  trace variable  ::BBAudio::State(PTT)  w  "P25_Repeat_SimUI::set_ptt_on_off"
  set_ptt_on_off

};  # End of P25_Repeat_SimUI namespace


#****************************************************************************
# This registers the tiles with the list for saving instrument states
#****************************************************************************
namespace eval ::SavedStates \
{
  variable tiles
  
  lappend tiles P25_Repeat_SimUI
}



