#############################################################################
#                                                                           #
# IFR SYSTEMS, INC        COPYRIGHT (C)1998,99      DESIGN ENGINEERING      #
# ------------------------------------------------------------------------- #
#                                                                           #
# FILE        : 3530.ReceiverUI.tcl                                         #
#                                                                           #
# PRODUCT     : FITS                                                        #
#                                                                           #
# AUTHOR      : P. McCormick                                                #
#                                                                           #
# DESCRIPTION : Receiver User Interface                                     #
#                                                                           #
#############################################################################

namespace eval ReceiverUI \
{
  # Global variables
  variable  LongName   "rec"
  variable  ShortName  "Receiver (TX Test)"
  variable  Size       Quarter
  variable  Position   upper_left

  variable  tile_class   "REC"

  variable  last_focus
  
  variable  StateUI
  
  set StateUI(incrfreq)         45.0000   
  set StateUI(mode)             "FREQ"
  set StateUI(scanincr)         45.0000
  set StateUI(scanrate)         0.50      
  set StateUI(scanstart)        100.0000
  set StateUI(scanstop)         500.0000
  set StateUI(scanmode)         "START"
  set StateUI(scanon)           0
  set StateUI(scanpause)        0

  set StateUI(encrypt)          ""  
  
  set StateUI(decode_mode)      "NONE"
  set StateUI(decode_received)  ""

  set StateUI(list_freq_index)  0
  set StateUI(pres_list_label)  ""
     
  set StateUI(mode_menu)   { "FREQ"  "INCR"  "SCAN"  "LIST"  "Find"  "Find Next"  "Fine Tune" }

  set StateUI(atten_menu)  { "0"  "10"  "20"  "30"  "40" }
  
  set StateUI(if_menu)     { "12.5kHz"  "25kHz"  "60kHz"  "200kHz" }
  
  set StateUI(if_menu_am)  { "6kHz" "12.5kHz"  "25kHz"  "60kHz"  "200kHz" }
  
  set StateUI(if_menu_p25) { "12.5kHz"  "25kHz" }
  set StateUI(if_menu_p25p2) { "6.25kHz" "12.5kHz"}
  set StateUI(if_menu_p25wp) { "25kHz" "50kHz" }

  set StateUI(demod_menu)  {"FM"  "AM"  "P25"}
  if [IFR_options is_installed P25_LSM] \
  {
     lappend StateUI(demod_menu)  "P25 LSM"  "P25 P2"
  }
  if [IFR_options is_installed LTD] \
  {
     lappend StateUI(demod_menu)  "LTD"
  }
  if [IFR_options is_installed P25_WIDEPULSE] \
  {
     lappend StateUI(demod_menu)  "P25 WP"
  }
                                             
  image create bitmap ::ReceiverUI::squelch_image
  
  trace variable ::Receiver::State(squelchmode) w {ReceiverUI::update_squelch_icon}
  trace variable ::Receiver::State(squelchstate) w {ReceiverUI::update_squelch_icon}
  
  proc Force_to_freq {args } \
  {
     after idle "set ReceiverUI::StateUI(mode) \"FREQ\""
  }
  
  trace variable ::Receiver::State(fine_tune) w ReceiverUI::Force_to_freq
  
  #****************************************************************************
  # Name:        update_squelch_icon
  #  Parameters:  none
  # Description: Updates the squelch icon
  #****************************************************************************
  proc update_squelch_icon { args} \
  {
     set cmd "ReceiverUI::update_squelch_icon_core"
     after cancel $cmd
     after idle $cmd
  }
  proc update_squelch_icon_core { } \
  {
    if {$::Receiver::State(squelchmode) == 2 } {
      ::ReceiverUI::squelch_image configure -file /system/logo/squelch.xbm -foreground black
    } elseif {$::Receiver::State(squelchmode) != 1 } {
      # not in auto mode, so clear the squelch icon
      ::ReceiverUI::squelch_image configure -file /system/logo/squelchnone.xbm -foreground red
    } else {
       # in auto mode. Show the squelch
       if {$::Receiver::State(squelchstate) == 0} {
          ::ReceiverUI::squelch_image configure -file /system/logo/unsquelch.xbm -foreground ForestGreen
       } elseif {$::Receiver::State(squelchstate) == 1} {
          ::ReceiverUI::squelch_image configure -file /system/logo/squelch.xbm -foreground black
       }
    }
  }

  proc analyzer_mode { tile_path args } \
  {
     set cmd [namespace code "analyzer_mode_core $tile_path"]
     after cancel $cmd
     after idle $cmd
  }
  proc analyzer_mode_core { tile_path args } \
  {
     if {$Analyzer::State(Engine_run) == 2} \
     {
        $tile_path.frame4.mnubtn_atten config -state disabled
     }\
     else \
     {
         $tile_path.frame4.mnubtn_atten config -state normal
     }
  }
  
  #**************************************************************************
  #
  #        Name: Create  
  #
  #  Parameters:
  #
  # Decsription: 
  #
  #**************************************************************************
  proc Create { tile_path  } \
  {
    variable  LongName
    variable  tile_class

    set top_level  [winfo toplevel $tile_path]
   
    set tile_path  [name_child $top_level $LongName]

    if { ![winfo exists $tile_path] } \
    {
      # Call the function 'create_tile' to generate the Receiver tile
      # from the resource database
      set tile_path  [::Screens::create_tile  $tile_path  $LongName  $tile_class]
    }
    trace variable ::Analyzer::State(Engine_run) w [namespace code "analyzer_mode $tile_path"]

    return $tile_path
  }


  #**************************************************************************
  #
  #        Name: Map
  #
  #  Parameters: 
  #
  # Decsription: 
  #
  #**************************************************************************
  proc Map { tile_path } \
  {

    variable  StateUI
    
    trace variable ::ReceiverUI::StateUI(mode)  w  \
                                        "::ReceiverUI::update_mode $tile_path"

    trace variable ::Receiver::State(demod_raw)  w  \
                                  "::ReceiverUI::Update_tile_demod $tile_path"

    trace variable ::Receiver::State(lockstat)  w  \
                                    "::ReceiverUI::rcv_lock_status $tile_path"

    trace variable ::RFError::State(status)  w  \
                                       "::ReceiverUI::rferr_status $tile_path"

    trace variable ::Receiver::State(maxatten)  w  \
                                "::ReceiverUI::update_atten_widget $tile_path"

    trace variable ::ReceiverUI::StateUI(scanon)  w  \
                                       "::ReceiverUI::set_softkeys $tile_path"

    trace variable ::ReceiverUI::StateUI(scanpause)  w  \
                                       "::ReceiverUI::set_softkeys $tile_path"

    update_mode_core        $tile_path
    
    Update_tile_demod_core  $tile_path

    rcv_lock_status_core    $tile_path
    
    # if the analyzer is in full mode, force it to channel
    if {$::Analyzer::State(Engine_run) == 2} { Analyzer run 1 -nowait}

  }
  
  
  #**************************************************************************
  #
  #        Name: Cleanup
  #
  #  Parameters: 
  #
  # Decsription: 
  #
  #**************************************************************************
  proc Cleanup { tile_path } \
  {
    Debug::Puts ReceiverUI 2 "Cleanup:  $tile_path"

    trace vdelete ::ReceiverUI::StateUI(mode)  w  \
                                        "::ReceiverUI::update_mode $tile_path"

    trace vdelete ::Receiver::State(demod_raw)  w  \
                                  "::ReceiverUI::Update_tile_demod $tile_path"

    trace vdelete ::Receiver::State(lockstat)  w  \
                                    "::ReceiverUI::rcv_lock_status $tile_path"

    trace vdelete ::RFError::State(status)  w  \
                                       "::ReceiverUI::rferr_status $tile_path"

    trace vdelete ::Receiver::State(maxatten)  w  \
                                "::ReceiverUI::update_atten_widget $tile_path"

    trace vdelete ::ReceiverUI::StateUI(scanon)  w  \
                                       "::ReceiverUI::set_softkeys $tile_path"

    trace vdelete ::ReceiverUI::StateUI(scanpause)  w  \
                                       "::ReceiverUI::set_softkeys $tile_path"
    trace vdelete ::Analyzer::State(Engine_run) w [namespace code "analyzer_mode $tile_path"]
  }


  #**************************************************************************
  #
  #        Name: Unmap
  #
  #  Parameters: 
  #
  # Decsription: 
  #
  #**************************************************************************
  proc Unmap { tile_path } \
  {
    Debug::Puts ReceiverUI 2 "ReceiverUI::Unmap"
    
    Cleanup $tile_path
    trace vdelete ::Analyzer::State(Engine_run) w [namespace code "analyzer_mode $tile_path"]
    return
  }


  #**************************************************************************
  #
  #        Name: Destroy
  #
  #  Parameters: 
  #
  # Decsription: 
  #
  #**************************************************************************
  proc Destroy { tile_path } \
  {
    Debug::Puts ReceiverUI 2 "ReceiverUI::Destroy"
    
    Cleanup $tile_path
    
    return
  }


  #**************************************************************************
  #
  #        Name: Register
  #
  #  Parameters: 
  #
  # Decsription: 
  #
  #**************************************************************************
  proc Register { tile_path } \
  {
    Debug::Puts ReceiverUI 2 "ReceiverUI::Register"     
    
    return
  }


  #**************************************************************************
  #
  #        Name: Update_encrypt
  #
  #  Parameters: 
  #
  # Decsription: Updates the display of the Encryption indicator
  #
  #**************************************************************************
  proc Update_encrypt {  args } \
  {
     set cmd "::ReceiverUI::Update_encrypt_core"

     after cancel $cmd
     after idle   $cmd
  }

  proc Update_encrypt_core { } \
  {
    variable StateUI
    
    if { $::P25ns::State(d_encryptState) == 1 } \
    {
      set StateUI(encrypt) "ENC"
    } \
    else \
    {
      set StateUI(encrypt) ""
    }

    return
  }




  #############################################################################
  # 
  #   Softkey Support
  # 
  #############################################################################
  
  #***************************************************************************
  #
  #        Name: get_tile
  #
  #  Parameters: any child widget of the tile
  #
  # Description: Get the name of the tile's parent frame
  #
  #***************************************************************************
  proc get_tile { widget } \
  {
     #set class  $::FGenUI::tile_class
     
     while { 1 } \
     {
       switch -- [winfo class $widget] \
       {
         "REC"        -
         "FREQ_SCAN"  { return $widget }
       }
        
       set widget [winfo parent $widget]
     }
     
  }


  #**************************************************************************
  #
  #        Name: softkey_goto_freq
  #
  #  Parameters: none
  #
  # Description: Move cursor to RF frequency entry
  #
  #**************************************************************************
  proc softkey_goto_freq { tile_path } \
  {
    Debug::Puts ReceiverUI 2 "softkey_goto_freq"
    
    set tile_path  [get_tile  $tile_path]

    
    set path  ${tile_path}.frame1.ent_freq

    unpost_last_menu
  
    safe_focus_force $path
  
    event generate $path <Return>
  
  }
  

  #**************************************************************************
  #
  #        Name: softkey_goto_atten
  #
  #  Parameters: none
  #
  # Description: Move cursor to RF frequency entry
  #
  #**************************************************************************
  proc softkey_goto_atten { tile_path } \
  {
    Debug::Puts ReceiverUI 2 "softkey_goto_atten"
    
    set tile_path  [get_tile  $tile_path]

    
    post_menu ${tile_path}.frame4.mnubtn_atten
  
  }
  

  #**************************************************************************
  #
  #        Name: softkey_goto_ifbw
  #
  #  Parameters: none
  #
  # Description: Move cursor to RF frequency entry
  #
  #**************************************************************************
  proc softkey_goto_ifbw { tile_path } \
  {
    Debug::Puts ReceiverUI 2 "softkey_goto_ifbw"
    
    set tile_path  [get_tile  $tile_path]
    foreach menu {frame5.mnubtn_ifbw frame5.mnubtn_ifbw_am} \
    {
       set menu ${tile_path}.$menu
       if [winfo ismapped $menu] \
       {
          post_menu $menu
       }
    }
    
  
  }
  

  #**************************************************************************
  #
  #        Name: softkey_goto_demod
  #
  #  Parameters: none
  #
  # Description: Move cursor to RF frequency entry
  #
  #**************************************************************************
  proc softkey_goto_demod { tile_path } \
  {
    Debug::Puts ReceiverUI 2 "softkey_goto_demod"
     
    post_menu ${tile_path}.frame5.mnubtn_demod
  
  }
  

  #**************************************************************************
  #
  #        Name: softkey_goto_rfport
  #
  #  Parameters: none
  #
  # Description: Move cursor to RF frequency entry
  #
  #**************************************************************************
  proc softkey_goto_rfport { tile_path } \
  {
    Debug::Puts ReceiverUI 2 "softkey_goto_rfport"
       
    ${tile_path}.frame4.btn_input invoke
  
  }


  #**************************************************************************
  #
  #        Name: set_filter
  #
  #  Parameters: none
  #
  # Description: Set the receiver post detection filter
  #
  #**************************************************************************
  proc set_filter { } \
  {
      Receiver filter $::Receiver::State(iirFilter) -nowait
  } 


  #**************************************************************************
  #
  #        Name: set_softkeys
  #
  #  Parameters: cause reason proc called {initial chanmode demod}
  #
  # Description: Sets receiver softkeys
  #
  #**************************************************************************
  proc set_softkeys { tile_path  args } \
  {
     set cmd [namespace code "set_softkeys_core $tile_path"]
     after cancel $cmd
     after idle $cmd
  }
  proc set_softkeys_core { tile_path } \
  {
    variable  StateUI
    
    # SOFTKEY 2 (FREQ)
    ::View::AssignSoftkey $tile_path  2  "RF FREQ"  \
                                 "::ReceiverUI::softkey_goto_freq  $tile_path"
      
    # SOFTKEY 3 (ATTEN)
    ::View::AssignSoftkey $tile_path  3  "INPUT ATTEN"  \
                                "::ReceiverUI::softkey_goto_atten  $tile_path"
      
    # SOFTKEY 4 (IF FILTER)
      
    switch --  $Receiver::State(demod_raw) \
    {
       0 -
       1 -
       2 -
       3 -
       4 -
       15 -
       16 -
       17 -
       18 -
       12 - 
       13 -
       21 -
       22 -
       10  \
       {
          ::View::AssignSoftkey $tile_path  4  "IF FILTER"  \
                                 "::ReceiverUI::softkey_goto_ifbw  $tile_path"
       } \
       default \
       {
         ::View::ClearSoftkey $tile_path  4 
       }
    }
    
    # SOFTKEY 5 (DEMOD)
    ::View::AssignSoftkey $tile_path  5  "DEMOD TYPE"  \
                                "::ReceiverUI::softkey_goto_demod  $tile_path"
      
    # SOFTKEY 6 (INCR UP, START SWEEP, INPUT)
    if { $StateUI(mode) == "INCR" } \
    {
      ::View::AssignSoftkey $tile_path  6  "INCR UP" "::ReceiverUI::freq_incr"
    } \
    elseif { $StateUI(mode) == "SCAN" } \
    {
      if { $StateUI(scanon) } \
      {
        ::View::AssignSoftkey $tile_path  6  "STOP SCAN"  \
                                                "::ReceiverUI::set_scan  STOP"
      } \
      else \
      {
        ::View::AssignSoftkey $tile_path  6  "START SCAN"  \
                                               "::ReceiverUI::set_scan  START"
      }
    } \
    else \
    {
      ::View::AssignSoftkey $tile_path  6  "INPUT PORT"  \
                                        "::ReceiverUI::toggle_input $tile_path"
    }
     
    # SOFTKEY 7 (INCR DOWN, PAUSE SWEEP)
    if { $StateUI(mode) == "INCR" } \
    {
      ::View::AssignSoftkey $tile_path  7  "INCR DOWN" "::ReceiverUI::freq_decr"
    } \
    elseif { $StateUI(mode) == "SCAN" } \
    {
      if { $StateUI(scanon) } \
      {
        if { $StateUI(scanpause) } \
        {
          ::View::AssignSoftkey $tile_path  7  "CONT SCAN"  \
                                                 "::ReceiverUI::scan_pause  0"
        } \
        else \
        {
          ::View::AssignSoftkey $tile_path  7  "PAUSE SCAN"  \
                                                 "::ReceiverUI::scan_pause  1"
        }
      } \
      else \
      {
        ::View::ClearSoftkey $tile_path  7  
      }
    } \
    else \
    {
      ::View::ClearSoftkey $tile_path  7
    }
    
  }
  
  
  #############################################################################
  # 
  #   Receiver Tile Support Functions
  # 
  #############################################################################
  
  
  #**************************************************************************
  #
  #        Name: set_freq
  #
  #  Parameters: none
  #
  # Description: Sets the Receiver frequency
  #
  #**************************************************************************
  proc set_freq { } \
  {
    Receiver  freq  $::Receiver::State(freq_raw) MHz -nowait      
  } 


  #**************************************************************************
  #
  #        Name: set_demod
  #
  #  Parameters:
  #
  # Description: Sets Receiver demod
  #
  #**************************************************************************
  proc set_demod { } \
  {
    switch -- $Receiver::State(demod,text) \
    {
       "P25 P2"  { 
                   switch -- $Receiver::State(ifbw,bw) \
                   {
                      "6.25kHz" {Receiver demod p25_p2 -nowait}
                      "12.5kHz" {Receiver demod p25_p2_wide -nowait}
                      default   {Receiver demod p25_p2 -nowait}
                   }

                   set ::DeviationUI::StateUI(title)  "DEV"
                   set ::DeviationUI::StateUI(units)  "kHz" }
                   
       "P25 LSM" {
                    switch -- $Receiver::State(ifbw,bw) \
                   {
                      "12.5Hz"  {Receiver demod p25qpsk -nowait}
                      "25kHz"   {Receiver demod p25qpsk_wide -nowait}
                      default   {Receiver demod p25qpsk -nowait}
                   }

                   set ::DeviationUI::StateUI(title)  "DEV"
                   set ::DeviationUI::StateUI(units)  "kHz" }
       
       "P25"     { switch -- $Receiver::State(ifbw,bw) \
                   {
                     "12.5kHz"  { Receiver  demod  p25       -nowait }
                     "25kHz"    { Receiver  demod  p25_wide  -nowait }
                     
                     default    { Receiver  demod  p25       -nowait }
                   }
                   
                   set ::DeviationUI::StateUI(title)  "DEV"
                   set ::DeviationUI::StateUI(units)  "kHz" }
       "LTD"     { switch -- $Receiver::State(ifbw,bw) \
                   {
                     "12.5kHz"  { Receiver  demod  ltd       -nowait }
                     "25kHz"    { Receiver  demod  ltd_wide  -nowait }
                     
                     default    { Receiver  demod  ltd       -nowait }
                   }
                   
                   set ::DeviationUI::StateUI(title)  "DEV"
                   set ::DeviationUI::StateUI(units)  "kHz" }
       "P25 WP"   { switch -- $Receiver::State(ifbw,bw) \
                   {
                     "25kHz"  { Receiver  demod  p25wp_narrow -nowait }
                     "50kHz"  { Receiver  demod  p25wp_wide   -nowait }
                     
                     default    { Receiver  demod  p25wp_narrow -nowait }
                   }
                   
                   set ::DeviationUI::StateUI(title)  "DEV"
                   set ::DeviationUI::StateUI(units)  "kHz" }
    
       "FM"      { switch -- $Receiver::State(ifbw,bw) \
                   {
                     "12.5kHz"  { Receiver  demod  fmwide      -nowait }
                     "25kHz"    { Receiver  demod  fmmedium    -nowait }
                     "200kHz"   { Receiver  demod  fmwideband  -nowait }
                     "60kHz"    { Receiver  demod  fmtv        -nowait }
                     
                     default    { Receiver  demod  fmwide      -nowait }
                   }

                   set ::DeviationUI::StateUI(title)  "DEV"
                   set ::DeviationUI::StateUI(units)  "kHz" }
    
       "AM"      { switch -- $Receiver::State(ifbw,bw) \
                   {
                     "6kHz"     { Receiver  demod  am          -nowait }
                     "12.5kHz"  { Receiver  demod  am12        -nowait }
                     "25kHz"    { Receiver  demod  am25        -nowait }
                     "60kHz"    { Receiver  demod  am60        -nowait }
                     "200kHz"   { Receiver  demod  am200       -nowait }
                     
                     default    { Receiver  demod  am          -nowait }
                   }

                   set ::DeviationUI::StateUI(title)  "MOD"
                   set ::DeviationUI::StateUI(units)  "%" }
    }
  }


  #**************************************************************************
  #
  #        Name: Update_tile_demod
  #
  #  Parameters: tile_path - tile path of the invoking widget
  #
  # Description: Updates the display to the Receiver demod
  #
  #**************************************************************************
  proc Update_tile_demod { tile_path  args } \
  {
     set cmd "::ReceiverUI::Update_tile_demod_core $tile_path"
     
     after cancel $cmd
     after idle $cmd
  }

  proc Update_tile_demod_core { tile_path } \
  {
    if ![winfo exists $tile_path] { return }
     
    # Remove optional widgets
      # Encryption indicator label
      grid remove ${tile_path}.frame3.lbl_encrypt

      # IF Bandwidth label and P25 and standard menus
      grid remove ${tile_path}.frame5.mnubtn_ifbw
      grid remove ${tile_path}.frame5.mnubtn_ifbw_p25
      grid remove ${tile_path}.frame5.mnubtn_ifbw_p25p2
      grid remove ${tile_path}.frame5.mnubtn_ifbw_am
      grid remove ${tile_path}.frame5.mnubtn_ifbw_p25wp

    # Remove all subframes
    grid remove ${tile_path}.frame6_1
    grid remove ${tile_path}.frame6_2

    switch -- $Receiver::State(demod_raw) \
    {
      5   -
      14  -
      19  -
      20  { # P25 modes (including LTD)
            # Display the 'KEY' label and menubutton
            Screens::position_widget ${tile_path}.frame6_2
            Screens::position_widget ${tile_path}.frame3.lbl_encrypt 
         
            # Display the P25 IF bandwidth menubutton
            Screens::position_widget  ${tile_path}.frame5.mnubtn_ifbw_p25 }

      13 -
      21 { # P25 Phase II
            # Display the 'KEY' label and menubutton
            Screens::position_widget ${tile_path}.frame6_2
            Screens::position_widget ${tile_path}.frame3.lbl_encrypt
    
            # Display the P25 IF bandwidth menubutton
            Screens::position_widget  ${tile_path}.frame5.mnubtn_ifbw_p25p2
          }

      22 -
      12  { # P25 LSM 
            # Display the 'KEY' label and menubutton
            Screens::position_widget ${tile_path}.frame6_2
            Screens::position_widget ${tile_path}.frame3.lbl_encrypt
          
            # Display the P25 IF bandwidth menubutton
            Screens::position_widget  ${tile_path}.frame5.mnubtn_ifbw_p25
          }

      23 - 
      24 { # P25 Widepulse
            # Display the 'KEY' label and menubutton
            Screens::position_widget ${tile_path}.frame6_2
            Screens::position_widget ${tile_path}.frame3.lbl_encrypt
          
            # Display the P25 IF bandwidth menubutton
            Screens::position_widget  ${tile_path}.frame5.mnubtn_ifbw_p25wp
         }
      0   -
      1   -
      2   -
      3   -
      10  { # FM modes
            # Display the 'AUDIO WB' menubutton
            Screens::position_widget ${tile_path}.frame6_1

            # Display the standard IF bandwidth menubutton
            Screens::position_widget  ${tile_path}.frame5.mnubtn_ifbw
            
            # enable the menubutton
            ${tile_path}.frame5.mnubtn_ifbw  config  -state normal }

      4 -
      15 -
      16 -
      17 -
      18   { # AM mode
            # Display the 'AUDIO WB' menubutton
            Screens::position_widget ${tile_path}.frame6_1

            # Display the standard IF bandwidth menubutton,
            Screens::position_widget  ${tile_path}.frame5.mnubtn_ifbw_am}
    } 

    set_softkeys  $tile_path
  
    return

  }



  #**************************************************************************
  #
  #        Name: update_atten_widget
  #
  #  Parameters: tile_path
  #             args
  #
  # Description: Changes active entries based on current port (ANT or T/R)
  #
  #**************************************************************************
  proc update_atten_widget { tile_path args } {

    # Obtain the current menu
    set tile_path  [get_tile $tile_path]
    Debug::Puts ReceiverUI 2 "toggle_input $tile_path"
    set menu  ${tile_path}.frame4.mnubtn_atten.menu

    # this will interate through available attens defined in 3530.ReceiverUI.tcl
    catch {
      set index 0
      for { set db 0 } { $db <= 40 } { incr db 10 } {
        if { $db > $::Receiver::State(maxatten) } {
          $menu entryconfig $index -state disabled
        } else {
          $menu entryconfig $index -state normal
        }

        incr index
      }
    } err
    
    Debug::Puts ReceiverUI 2 "ReceiverUI::update_atten_widget $err"

    return
  }


  #**************************************************************************
  #
  #        Name: toggle_input
  #
  #  Parameters: tile_path
  #
  # Description: Sets receive input to ANT or T/R, also update receiver
  #           : attenuation choices
  #
  #**************************************************************************
  proc toggle_input { tile_path } \
  {
    if { $::Receiver::State(rfinput) == "ANT" } \
    {
      Receiver rfin "T/R" -nowait
    } \
    else \
    {
      Receiver rfin "ANT" -nowait
    }

    return    
  }
  
  #**************************************************************************
  #
  #        Name: update_mode
  #
  #  Parameters: 
  #
  # Description: Sets Receiver mode
  #
  #**************************************************************************
  proc update_mode { tile_path args } \
  {
     set cmd "::ReceiverUI::update_mode_core $tile_path"
     after cancel $cmd
     after idle $cmd
  }
  
  proc update_mode_core { tile_path } \
  {
    Debug::Puts ReceiverUI 2 "update_mode_core:  $tile_path"

    variable  StateUI
    
    set tile_path  [get_tile  $tile_path]

    set_softkeys  $tile_path

    grid remove ${tile_path}.frame2_1
    grid remove ${tile_path}.frame2_2
    grid remove ${tile_path}.frame2_3
  
    # Stop any present SCAN
    set_scan  "STOP"
    
    switch -- $StateUI(mode) \
    {
      "FREQ" { }

      "INCR" { Screens::position_widget ${tile_path}.frame2_1 }
    
      "SCAN" { Screens::position_widget ${tile_path}.frame2_2 }
               
      "LIST" { Screens::position_widget ${tile_path}.frame2_3

             }
    }
  }


  #**************************************************************************
  #
  #        Name: set_scan
  #
  #  Parameters: new_value
  #
  # Description: Sets freq_lock variable, sets highlight color on button
  #
  #**************************************************************************
  proc set_scan { { new_value  "TOGGLE" } } \
  {
    Debug::Puts ReceiverUI 2 "set_scan:  $new_value"
    
    variable  StateUI
    
    switch -- $new_value \
    {
      "TOGGLE"  { if { $StateUI(scanon) } \
                  {
                    set StateUI(scanon)  0
                  } \
                  else \
                  {
                    set StateUI(scanon)  1
                  }
                }
                
      "STOP"    { set StateUI(scanon)  0 }
      
      "START"   { set StateUI(scanon)  1 }
    }


    if { $StateUI(scanon) } \
    {
      set StateUI(scanmode)  "STOP"

      after idle "ReceiverUI::freq_scan"
    } \
    else \
    {
      set StateUI(scanmode)  "START"

      after cancel "ReceiverUI::freq_scan"
    }

    return
  }


  #**************************************************************************
  #
  #        Name: scan_pause
  #
  #  Parameters: pauses scan
  #
  #**************************************************************************
  proc scan_pause { { new_value  "TOGGLE" } } \
  {
    Debug::Puts ReceiverUI 2 "scan_pause:  $new_value"

    variable  StateUI
    
    if { $new_value == "TOGGLE" } \
    {
      if { $StateUI(scanpause) } \
      {
        set StateUI(scanpause)  0
      } \
      else \
      {
        set StateUI(scanpause)  1
      }
    } \
    else \
    {
      set StateUI(scanpause)  $new_value
    }

    return        
  }
  

  #**************************************************************************
  #
  #        Name: setup_freq_scan
  #
  #  Parameters: 
  #
  # Description: Displays toplevel for editing freq scan values
  #
  #**************************************************************************
  proc setup_freq_scan { tile_path } \
  {
    Debug::Puts ReceiverUI 2 "setup_freq_scan"
    
    set top_level  [winfo toplevel $tile_path]
   
    set tile_path  [name_child  $top_level  freq_scan]

    if { ![winfo exists $tile_path] } \
    {
      set tile_path  [::Screens::create_toplevel_tile  $tile_path  FREQ_SCAN]
    }
  }


  #**************************************************************************
  #
  #        Name: destroy_freq_scan
  #
  #  Parameters: 
  #
  # Description: Destroys toplevel for editing freq scan values
  #
  #**************************************************************************
  proc destroy_freq_scan { tile_path } \
  {
    Debug::Puts ReceiverUI 2 "destroy_freq_scan:  $tile_path"

    destroy [winfo toplevel $tile_path]
  }

  
  #**************************************************************************
  #
  # Name: freq_scan
  #
  #  Parameters: 
  #
  # Description: Starts or stops scan of frequencies
  #
  #**************************************************************************
  proc freq_scan { } \
  {
    Debug::Puts ReceiverUI 2 "freq_scan"
    
    variable  StateUI
    
    # Start the freq scan
    # Set the freq to the start freq
    set rec_freq  $StateUI(scanstart)

    while { $StateUI(scanon) == 1 } \
    {
      # Exit if either the Receiver or Generator becomes unlocked.
      if { $::Receiver::State(lockstat,raw) != 0 } \
      {
        puts "Receiver unlock, stop scan mode"
        
        set_scan  "STOP"
      }

      if { $::Generator::State(lockstat,raw) != 0 } \
      {
        puts "Generator unlock, stop scan mode"

        set_scan  "STOP"
      }

      if { $StateUI(scanstop) > $StateUI(scanstart) } \
      {
        # If the STOP is greater than the START
        # follow the incrementing procedure
        Receiver freq  $rec_freq  -nowait

        # Wait the duration or the rate delay
        set delay [expr int($StateUI(scanrate) * 1000.0)]
        if {$delay < 100} { set delay 100 }
        after $delay
      
        if { $rec_freq < $StateUI(scanstop) } \
        {
          # If the freq is less than the STOP freq,
          # increment and check against the STOP freq again
          set rec_freq  [expr $rec_freq + $StateUI(scanincr)]
        
          # If the freq is now greater than the STOP freq,
          # set it to the STOP freq
          if { $rec_freq > $StateUI(scanstop) } \
          {
            set rec_freq  $StateUI(scanstop)
          }
        } \
        elseif { $rec_freq == $StateUI(scanstop) } \
        {
          # If the freq is equal to the STOP freq
          # set the freq to the START freq
          set rec_freq  $StateUI(scanstart)
        }
      } \
      elseif { $StateUI(scanstop) < $StateUI(scanstart) } \
      {
        # If the STOP is less than the START
        # follow the decrementing procedure
        Receiver freq  $rec_freq  -nowait

        # Wait the duration or the rate delay
        set delay [expr int($StateUI(scanrate) * 1000.0)]
        if {$delay < 100 } { set delay 100 }
        after $delay
      
        if { $rec_freq > $StateUI(scanstop) } \
        {
          # If the freq is greater than the STOP freq,
          # decrement and check against the STOP freq again
          set rec_freq  [expr $rec_freq - $StateUI(scanincr)]
        
          # If the freq is now less than the STOP freq,
          # set it to the STOP freq
          if { $rec_freq < $StateUI(scanstop) } \
          {
            set rec_freq  $StateUI(scanstop)
          }
        } \
        elseif { $rec_freq == $StateUI(scanstop) } \
        {
          # If the freq is equal to the STOP freq
          # set the freq to the START freq
          set rec_freq  $StateUI(scanstart)
        }
      } \
      else \
      {
        # If the STOP is equal to the START
        # set the freq to the STOP frequency
        Receiver freq  $StateUI(scanstop)  -nowait
        
        # Wait the duration or the rate delay
        set delay [expr int($StateUI(scanrate) * 1000.0)]
        if {$delay < 100 } { set delay 100 }
        after $delay

      }
        
      while { $StateUI(scanpause) == 1 } \
      {
        if { $StateUI(scanon) == 0 } \
        {
          set StateUI(scanpause)  0
        }
        
        update idle
      }
      
      update idle
    }
    
    return  
  }
  
  
  #**************************************************************************
  #
  #        Name: freq_decr
  #
  #  Parameters: none
  #
  # Description: Decrements rec freq 
  #
  #**************************************************************************
  proc freq_decr { } \
  {
    variable  StateUI

    set add [expr $::Receiver::State(freq_raw) - $StateUI(incrfreq)]

    Receiver freq  $add -nowait
  }


  #**************************************************************************
  #
  #        Name: freq_incr
  #
  #  Parameters: none
  #
  # Description: Increments rec freq
  #
  #**************************************************************************
  proc freq_incr { } \
  {
    variable  StateUI

    set add [expr $::Receiver::State(freq_raw) + $StateUI(incrfreq)]

    Receiver freq  $add -nowait
  }
  
  
  #**************************************************************************
  #
  # Name: display_freq_list
  #
  #  Parameters: widget - invoking widget tile path
  #
  # Description: Displays toplevel for editing freq list values
  #
  #**************************************************************************
  proc display_freq_list { widget } \
  {
    Debug::Puts ReceiverUI 2 "display_freq_list:  $widget"
    
    #set tile_path  [get_tile  $widget]

    ::Freq_ListUI::display_freq_list  $widget
    
    get_list_freq
    
    return
  }


  #**************************************************************************
  #
  # Name: get_list_freq
  #
  #  Parameters:
  #
  # Description: Displays the values of the freq list for the specified index
  #
  #**************************************************************************
  proc get_list_freq { {no_crosslink 0 } } \
  {
    Debug::Puts ReceiverUI 2 "get_list_freq"
    
    variable  StateUI
    
    if {$StateUI(mode) != "LIST"} { return }
    set list_freq_index  $StateUI(list_freq_index)
    
    # Return if the array value does not exist
    if { ![info exists ::Freq_ListUI::ListFreqArray(index,$list_freq_index)] } \
    {
      return
    }
    
    # Get the frequency list values
    set rx_freq      $::Freq_ListUI::ListFreqArray(rx_freq,$list_freq_index)
    set label        $::Freq_ListUI::ListFreqArray(label,$list_freq_index)

    # Display the label
    set StateUI(pres_list_label)  $label
    
    # Set the Receiver frequency
    Receiver  freq  $rx_freq -nowait
    
    # if we HAVEN'T been told not to, ask the generator if he wants to update
    if {$no_crosslink == 0} \
    {
       ::GeneratorUI::get_list_freq 1
    }

  }
  #**************************************************************************
  #
  #        Name: rferr_status
  #
  #  Parameters: widget, args (ignored)
  #
  # Description: Change the color of a RX error freq widget
  #             based on the reading status
  #
  #**************************************************************************
  proc rferr_status { tile_path  args } \
  {
     set cmd "ReceiverUI::rferr_status_core $tile_path"
     after cancel $cmd
     after idle $cmd
  }
  
  proc rferr_status_core { tile_path } \
  {
     if ![winfo exists $tile_path.frame3.lbl_recd_val] { return }
     set text_color black
     set bg [$tile_path cget -bg]

     if {$::RFError::State(status) & 0x101D  } \
     {
        set text_color yellow
     } \
     else \
     {
       if {$::RFError::State(status) & 2} \
       {
          set color DimGray
          set bg DarkGrey
       }
     }
     ${tile_path}.frame3.lbl_recd_val configure -fg $text_color -bg $bg
  }
  

  #**************************************************************************
  #
  #        Name: rcv_lock_status
  #
  #  Parameters: widget, args (ignored)
  #
  # Description: Change the color of a RX freq widget
  #             based on the lock status
  #
  #**************************************************************************
  proc rcv_lock_status { tile_path  args } \
  {
     set cmd "ReceiverUI::rcv_lock_status_core $tile_path"
     after cancel $cmd
     after idle $cmd
  }
  
  proc rcv_lock_status_core { tile_path } \
  {
    ###Debug::Puts ReceiverUI 2 "rcv_lock_status:  $widget  $args"
     
    variable  StateUI
    
    set entry  ${tile_path}.frame1.ent_freq
    
    
    if { $::Receiver::State(lockstat) } \
    {
      $entry configure -fg $::color(messagecaution)
    } \
    else \
    {
      $entry configure -fg $::color(menufg)
    }

  }
  
  #****************************************************************************
  # Name:        SetRcvrAtten
  #  Parameters:  none
  # Description: Set the Receiver Attenuation.
  #     This function is called by the drop menu which sets the attenuation.
  #     It cannot be used directly on the command line in the .rc file because
  #     the Receiver variable gets evaluated before the menu command is set up,
  #     instead of being evaluated when the menu selection is finished.
  #****************************************************************************
  proc SetRcvrAtten { } \
  {
     Receiver atten $::Receiver::State(atten) -nowait
  }

  #****************************************************************************
  # Name:        getSaveState
  #  Parameters:  none
  # Description: This function returns a list of TCL commands which can be used
  #              to set the ReceiverUI tiles to its current state
  #****************************************************************************
  proc getSaveState { } \
  {
    variable  StateUI
    
    set returnList \
    {
      set ::ReceiverUI::StateUI(mode)      {$StateUI(mode)}
      set ::ReceiverUI::StateUI(incrfreq)  {$StateUI(incrfreq)}
      set ::ReceiverUI::StateUI(scanincr)  {$StateUI(scanincr)}
      set ::ReceiverUI::StateUI(scanrate)  {$StateUI(scanrate)}
      set ::ReceiverUI::StateUI(scanstart) {$StateUI(scanstart)}
      set ::ReceiverUI::StateUI(scanstop)  {$StateUI(scanstop)}
      set ::ReceiverUI::StateUI(scanmode)  {$StateUI(scanmode)}
      set ::ReceiverUI::StateUI(scanon)    {$StateUI(scanon)}
      set ::ReceiverUI::StateUI(scanpause) {$StateUI(scanpause)}
    }

    Debug::Puts ReceiverUI 2 "returnList:  $returnList"
    set returnList [subst -nocommands $returnList]
    append returnList [namespace code \
    {
       if {$StateUI(mode) == "LIST"} \
       {
          get_list_freq
       }
    }]
    return $returnList

  }


  #****************************************************************************
  # Name:        Sync
  #  Parameters:  none
  # Description: Write state variables to ReceiverUI
  #****************************************************************************
  proc Sync { } \
  {
    eval [getSaveState]
  }

  trace variable ::P25ns::State(d_encryptState) w "::ReceiverUI::Update_encrypt"
  
  Update_encrypt
  
   #****************************************************************************
   # Name:        Hook_unlock
   #  Parameters:  toplevel window for this view
   # Description: Hook up to handle generator unlock messages for a view
   #****************************************************************************  
   proc Hook_unlock { toplevel} \
   {
      variable StateUI
      
      # trace the unlock state
      trace variable ::Receiver::State(lockstat) w "ReceiverUI::Unlock $toplevel"
      
      set StateUI(unlock_dismiss,$toplevel) 0
      set StateUI(ref_unlock_dismiss,$toplevel) 0
   }
   
   #****************************************************************************
   # Name:        Unhook_unlock
   #  Parameters:  toplevel window for this view
   # Description: Remove hook, as the screen is being destroyed
   #****************************************************************************
   proc Unhook_unlock { toplevel } \
   {
      variable StateUI
      trace vdelete ::Receiver::State(lockstat) w "ReceiverUI::Unlock $toplevel"
      
      unset StateUI(unlock_dismiss,$toplevel)
      unset StateUI(ref_unlock_dismiss,$toplevel)
   }
   #****************************************************************************
   # Name:        Unlock
   #  Parameters:  toplevel window for this view, trace var args (ignored)
   # Description: Update the unlock window as needed
   #****************************************************************************
   proc Unlock { toplevel args } \
   {
      # if the lock status is changing, take no action....
      if {$::Receiver::State(lock_unstable) != 0 } { return }
      
      set cmd "ReceiverUI::Unlock_core $toplevel"
      after cancel $cmd
      after idle $cmd
   }
 
   #****************************************************************************
   # Name:        Unlock_core
   #  Parameters:  toplevel window for this view
   # Description: Update the unlock window as needed
   #****************************************************************************
   proc Unlock_core { toplevel } \
   {
     variable  StateUI
     upvar #0 ::Receiver::State(lockstat) lockstat
     
     # if ref is unlocked, just show its dialog
     if {($lockstat & 8) != 0 } \
     {
       # if we've already dismissed this message, do nothing.
       if {$StateUI(ref_unlock_dismiss,$toplevel) == 1 } { return }

       set alert [name_child $toplevel refunlock]
       if ![winfo exists $alert] \
       {
           message_box $alert CAUTION
       }
     } \
     elseif {$lockstat } \
     {
        # receiver is unlocked - show its dialog
        
        # if we've already dismissed this message, do nothing.
        if {$StateUI(unlock_dismiss,$toplevel) == 1 } { return }
        
        set alert [name_child $toplevel rcvrunlock]

        if ![winfo exists $alert] \
        {
           message_box $alert CAUTION
           bind $alert <<Destroy>> "ReceiverUI::Unlock_Cleanup $alert"
           trace variable ::HW_control::State(yigres_rec) w "ReceiverUI::Unlock_update $alert"
           Unlock_update $alert
        }
     } \
     else \
     {
        # locked - destroy the alert windows if they exists
        destroy [name_child $toplevel rcvrunlock]
        destroy [name_child $toplevel refunlock]
        set StateUI(unlock_dismiss,$toplevel) 0
        set StateUI(ref_unlock_dismiss,$toplevel) 0
     }
   }
   #****************************************************************************
   # Name:        Unlock_cleanup
   #  Parameters:  alert box
   # Description: Clean up when destroyed
   #****************************************************************************
   proc Unlock_Cleanup { alert } \
   {
      trace vdelete ::HW_control::State(yigres_rec) w "ReceiverUI::Unlock_update $alert"
   }
   #****************************************************************************
   # Name:        Unlock_continue
   #  Parameters:  the name of the unlock window
   # Description: Dismiss window even though we are still unlocked...
   #****************************************************************************
   proc Unlock_continue { alert } \
   {
      variable StateUI
      set toplevel [winfo parent $alert]
      set StateUI(unlock_dismiss,$toplevel) 1
      destroy $alert
   }
   #****************************************************************************
   # Name:        Ref_unlock_continue
   #  Parameters:  the name of the unlock window
   # Description: Dismiss window even though we are still unlocked...
   #****************************************************************************
   proc Ref_unlock_continue { alert } \
   {
      variable StateUI
      set toplevel [winfo parent $alert]
      set StateUI(ref_unlock_dismiss,$toplevel) 1
      destroy $alert
   }


   #****************************************************************************
   # Name:        Unlock_update
   #  Parameters:  the name of the unlock window
   # Description: 
   #****************************************************************************
   proc Unlock_update { alert args } \
   {
      switch -- $::HW_control::State(yigres_rec) \
      {
         "Complete" \
         {
            if [winfo exists $alert] { destroy $alert }
         }
         "Running"  \
         {
            # lock out the button while cal is running
            $alert.cancel config -state disabled \
                                 -text [subst [option get $alert.cancel altText {}]]
         }
         default    \
         {
            # Cal failed. Allow cancel
            $alert.cancel config -state normal \
                         -text [option get $alert.cancel text {}]
            $alert.text config -text [subst [option get $alert.text altText {}]]

         }
      }
   }

  set_demod
  
  proc trace_mode_vars { trace } \
  {
     variable StateUI
     if {$trace != 0} \
     {
        if {$StateUI(update_mode_vars_trace) == 0} \
        {
            trace variable ::Freq_ListUI::ListFreqArray           w $StateUI(update_mode_vars_cmd)
            incr StateUI(update_mode_vars_trace)
        }
     } \
     else \
     {
        if {$StateUI(update_mode_vars_trace) != 0} \
        {
          trace vdelete ::Freq_ListUI::ListFreqArray           w $StateUI(update_mode_vars_cmd)
          set StateUI(update_mode_vars_trace) 0
        }
     }
  }
  proc update_mode_vars { args } \
  {
     Debug::Puts ReceiverUI 2 "update_mode_vars"
     variable StateUI
     set cmd [namespace code "update_mode_vars_core"]
     # optimization: we saw the change, don't keep retriggering!
     trace_mode_vars 0
     after cancel $StateUI(update_mode_vars_cmd)
     after 500 $StateUI(update_mode_vars_cmd)
  }
  proc update_mode_vars_core {  } \
  {
    Debug::Puts ReceiverUI 2 "update_mode_vars_core"
    variable StateUI

    if {$StateUI(mode) == "LIST"} \
    {
       # watch for changes on the frequency list vars, and update as needed
       trace_mode_vars 1
       get_list_freq
    } \
    else \
    {
       trace_mode_vars 0
    }
  }
  proc update_freq_list { args } \
  {
     Debug::Puts ReceiverUI 2 "update_freq_list"
     # optimization: Re-arm the trace
     trace_mode_vars 1
     set cmd [namespace code "get_list_freq"]
     after cancel $cmd
     after 500    $cmd
  }
  set StateUI(update_mode_vars_trace) 0
  trace variable ::ReceiverUI::StateUI(mode)  w "::ReceiverUI::update_mode_vars"
  set StateUI(update_mode_vars_cmd) [namespace code update_freq_list]
  update_mode_vars_core
  
};  # End of ReceiverUI namespace


#****************************************************************************
# This registers the tiles with the list for saving instrument states
#****************************************************************************
namespace eval ::SavedStates {

   variable tiles   
   lappend tiles ReceiverUI
}

namespace eval View \
{
   lappend PreCreate ReceiverUI
   lappend Alert_routines ReceiverUI::Hook_unlock
   lappend Clean_ups ReceiverUI::Unhook_unlock
}


