#############################################################################
#                                                                           #
# IFR SYSTEMS, INC        COPYRIGHT (C)1998,99      DESIGN ENGINEERING      #
# ------------------------------------------------------------------------- #
#                                                                           #
# FILE        : 3532.Dev_C4fmUI.tcl                                         #
#                                                                           #
# PRODUCT     : FITS                                                        #
#                                                                           #
# AUTHOR      : P. McCormick                                                #
#                                                                           #
# DESCRIPTION : Generic User Interface for both the Deviation and C4 FM     #
#               meters                                                      #
#                                                                           #
#############################################################################

namespace eval Dev_C4fmUI \
{

  # Global variables
  variable LongName     "dev_c4fm"
  variable ShortName    "Modulation Meter"
  variable Size         Eighth_v
  variable OldRcvDemod	""

 
  #********************************************************************
  #
  #       Name: Create  
  #
  # Parameters:
  #
  # Decsription: 
  #
  #********************************************************************
  proc Create { tile_path } \
  {
    variable LongName
    
    set top_level [winfo toplevel $tile_path]
   
    set tile_path [name_child $top_level $LongName]
    
    if { ![winfo exists $tile_path] } \
    {
      set tile_path  [::Screens::create_tile  $tile_path  $::Dev_C4fmUI::LongName  DEV_C4FM]
    }
    
    # make sure of our children's existance
    set dev_path [::DeviationUI::Create $top_level]
    set c4_path  [::C4fmUI::Create $top_level]

    return $tile_path
  }


  #******************************************************************** 
  #
  #       Name: Map
  #
  # Parameters: 
  #
  # Decsription: 
  #
  #********************************************************************
  proc Map { widget } \
  {
    #puts "Dev_C4fmUI::Map $widget"

    trace variable ::Receiver::State(demod_raw) w "::Dev_C4fmUI::Track_demod $widget"
    
    # Call core procedure to make sure the correct tile gets displayed
    Track_demod_core  $widget
  }

  #******************************************************************** 
  #
  #       Name: Cleanup
  #
  # Parameters: 
  #
  # Decsription: 
  #
  #********************************************************************
  proc Cleanup { widget } \
  {
    variable OldRcvDemod
    
    set OldRcvDemod ""

    trace vdelete ::Receiver::State(demod_raw) w "::Dev_C4fmUI::Track_demod $widget"

    after cancel "Dev_C4fmUI::Map_AfterIdle $widget"
    after cancel "Dev_C4fmUI::Map_AfterDelay $widget"

    return
  }
  #********************************************************************
  #
  #       Name: Unmap
  #
  # Parameters: 
  #
  # Decsription: 
  #
  #********************************************************************
  proc Unmap { widget } \
  {
    #puts "Dev_C4fmUI::Unmap"
    
    Cleanup $widget
    
    return
  }


  #********************************************************************
  #
  #       Name: Destroy
  #
  # Parameters: 
  #
  # Decsription: 
  #
  #********************************************************************
  proc Destroy { widget } \
  {
    #puts "Dev_C4fmUI::Destroy"
    
    Cleanup $widget
    
    return
  }


  #********************************************************************
  #
  #       Name: Update
  #
  # Parameters: 
  #
  # Decsription: 
  #
  #********************************************************************
  proc Update { } \
  {
    #puts "Dev_C4fmUI::Update"
  }
    
  #********************************************************************
  #
  #       Name: Track_demod
  #
  # Parameters: 
  #
  # Decsription: 
  #
  #********************************************************************
  proc Track_demod { widget args } \
  {
    # Track the state of the demod so we do not switch unless the value changes
    variable OldRcvDemod

    if { $OldRcvDemod != $::Receiver::State(demod_raw) } \
    {
      set cmd "Dev_C4fmUI::Track_demod_core $widget"

      after cancel $cmd
      after idle $cmd
    }
  }
  #********************************************************************
  #
  #       Name: Track_demod_core
  #
  # Parameters: 
  #
  # Decsription: 
  #
  #********************************************************************
  proc Track_demod_core { widget } \
  {
    #puts "Track_demod_core:  $widget"
    variable OldRcvDemod
    set OldRcvDemod $::Receiver::State(demod_raw)
    set top_level [winfo toplevel $widget]
   
    set dev_path  [name_child $top_level $::DeviationUI::LongName]
    set c4_path   [name_child $top_level $::C4fmUI::LongName]

     pack forget [pack slaves $widget]
     switch -- $Receiver::State(demod_raw) \
     {
        12 -
        13 -
        14 -
        19 -
        20 -
        21 -
        22 -
        23 - 
        24 -
        5  { pack $c4_path -in $widget -fill both -expand true
             raise $c4_path
           }
                 
        4 -
        15 -
        16 -
        17 -
        18  { pack $dev_path -in $widget -fill both -expand true
             raise $dev_path
           }
                                                  
        default { pack $dev_path -in $widget -fill both -expand true
                  raise $dev_path
                }
     }
  }
  

  #********************************************************************
  #
  #       Name: Register
  #
  # Parameters: 
  #
  # Decsription: 
  #
  #********************************************************************
  proc Register { } \
  {
    #puts "Dev_C4fmUI::Register"

    return
  }
  

};  # End of Dev_C4fmUI namespace


namespace eval View \
{
   lappend PreCreate Dev_C4fmUI
}



