#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C)1998-2002      DESIGN ENGINEERING    #
# ------------------------------------------------------------------------- #
# FILE        : ScopeUI.tcl                                                 #
# PRODUCT     : FITS                                                        #
# AUTHOR      : Hieu Tran                                                   #
# DESCRIPTION : Scope User Interface                                        #
#############################################################################



###########################################################################
# Quarter size of scope tile
###########################################################################

namespace eval QtrScopeUI \
{
   # Global variables
   variable LongName    qtrscope
   variable ShortName   Oscilloscope
   variable Size        Quarter
   variable tile        ""
   variable tile_class  QUARTER_SCOPE


   #********************************************************************************************
   #        Name: Create  
   #  Parameters: tile path of parent
   # Decsription: create quarter Scope tile 
   #********************************************************************************************
   proc Create { parent } \
   {
      variable LongName
      variable tile_class

      set top_level [winfo toplevel $parent]
      set tile_path [name_child $top_level $LongName]

      if { ![winfo exists $tile_path] } \
      {
         set tile [::Screens::create_tile  $tile_path $LongName $tile_class]
      }

      return $tile_path
   }


   #********************************************************************************************
   #       Name: Map
   # Parameters: none
   #Decsription: called when the screen is displayed
   #********************************************************************************************
   proc Map { widget } \
   {
     ScopeUI::Map $widget QUARTER_SCOPE
     OnChange hook ::ScopeUI::StateUI(source) "ScopeUI::set_source_frame $widget"
   }


   #********************************************************************************************
   #       Name: Cleanup
   # Parameters: none 
   #Decsription: called when the screen is hidden
   #********************************************************************************************
   proc Cleanup { widget } \
   {
     ScopeUI::Cleanup $widget QUARTER_SCOPE
     OnChange unhook ::ScopeUI::StateUI(source) "ScopeUI::set_source_frame $widget"
   }


   #********************************************************************************************
   #       Name: Unmap
   # Parameters: none 
   #Decsription: called when the screen is hidden
   #********************************************************************************************
   proc Unmap { widget } \
   {
     Cleanup  $widget
   }


   #********************************************************************************************
   #       Name: Destroy
   # Parameters: none
   #Decsription: called when the screen is destroyed
   #********************************************************************************************
   proc Destroy { widget } \
   {
     Cleanup  $widget
   }


};  # End of QtrScopeUI namespace


namespace eval View \
{
   lappend PreCreate QtrScopeUI
}



##########################################################################
# Full size of scope tile
##########################################################################


namespace eval FullScopeUI \
{

   # Global variables
   variable LongName    Scope_tile
   variable ShortName   scope
   variable Size        Full
   variable tile_class  FULL_SCOPE


   #********************************************************************************************
   #        Name: Create  
   #  Parameters: tile path of parent
   # Decsription: create Scope tile 
   #********************************************************************************************
   proc Create { parent } \
   {
      variable ShortName
      variable tile_class
      
      set top_level [winfo toplevel $parent]
      set tile_path [name_child $top_level $::FullScopeUI::ShortName]
    
      if { ![winfo exists $tile_path] } \
      {
         set tile [::Screens::create_tile  $tile_path  $ShortName  $tile_class]
      }
      bind $tile_path.lev_pointer <Configure> "ScopeUI::set_trig_level_indicator $tile_path"

      return $tile_path
   }


   #******************************************************************************************** 
   #       Name: Map
   # Parameters: none
   #Decsription: called when the screen is displayed
   #********************************************************************************************
   proc Map { widget } \
   {
     OnChange hook ::ScopeUI::StateUI(trig_rel,main) "ScopeUI::set_trig_level_indicator $widget"
     OnChange hook ::ScopeUI::StateUI(trig_rel,baseband) "ScopeUI::set_trig_level_indicator $widget"          
     ScopeUI::set_active_trace $widget FULL_SCOPE
     ScopeUI::Map $widget FULL_SCOPE
     ScopeUI::set_scope_softkeys $widget
     
     OnChange hook ::ScopeUI::StateUI(baseband) "ScopeUI::set_scope_softkeys $widget"
     OnChange hook ::ScopeUI::StateUI(ch1) "ScopeUI::set_scope_softkeys $widget"
     OnChange hook ::ScopeUI::StateUI(ch1) "ScopeUI::set_scope_softkeys $widget"
     OnChange hook ::ScopeUI::active "ScopeUI::set_scope_softkeys $widget"
   }


   #********************************************************************************************
   #       Name: Cleanup
   # Parameters: none
   #Decsription: called when the screen is hidden
   #********************************************************************************************
   proc Cleanup { widget } \
   {
     OnChange unhook ::ScopeUI::StateUI(trig_rel,main) "ScopeUI::set_trig_level_indicator $widget"
     OnChange unhook ::ScopeUI::StateUI(trig_rel,baseband) "ScopeUI::set_trig_level_indicator $widget"
     OnChange unhook ::ScopeUI::StateUI(baseband) "ScopeUI::set_scope_softkeys $widget"
     OnChange unhook ::ScopeUI::StateUI(ch1) "ScopeUI::set_scope_softkeys $widget"
     OnChange unhook ::ScopeUI::StateUI(ch1) "ScopeUI::set_scope_softkeys $widget"
     OnChange unhook ::ScopeUI::active "ScopeUI::set_scope_softkeys $widget"

     ScopeUI::Cleanup $widget FULL_SCOPE
   }


   #********************************************************************************************
   #       Name: Unmap
   # Parameters: none
   #Decsription: called when the screen is hidden
   #********************************************************************************************
   proc Unmap { widget } \
   {
     ::View::AssignReturn [winfo parent $widget] ""

     Cleanup  $widget

   }


   #********************************************************************************************
   #       Name: Destroy
   # Parameters: none
   #Decsription: called when the screen is destroyed
   #********************************************************************************************
   proc Destroy { widget } \
   {
     Cleanup  $widget

   }

}



namespace eval ScopeUI \
{
   variable trace
   variable StateUI
   variable source              CH1
   variable active              ch2
   variable srctext
   variable trigULExt           2
   variable trigLLExt           0

   variable sourcetext          ""
   variable trig_state		"True"
   variable horizpos            0.0
   variable StateUI(triglevel,1)           0.000
   variable trace_height        388
   variable demod_unit		""
   variable level_unit		""
   variable triglighttime	50
   variable lun
   variable menu_label
   
   set menu_label(ch1)          "CH1"
   set menu_label(ch2)          "CH2"
   set menu_label(mdyn)         "MIC (Dynamic)"
   set menu_label(mect)	   	"MIC (Electret)"
   set menu_label(aud)		"AUDIO"
   set menu_label(bal)		"Audio Balance"
   set menu_label(demod)	"DEMOD"
   set menu_label(squelched)	"SQLCH"

   set trace(ch1)               ch1
   set trace(ch2)               ch2
   set trace(mdyn)            	aud
   set trace(mect)		aud
   set trace(aud)               aud
   set trace(bal)               aud
   set trace(demod)             aud
   set trace(squelched)         aud

   set lun(ch1) 		1
   set lun(ch2) 		2
   set lun(aud) 		3

   set StateUI(source)          OFF

   set StateUI(peristance)      OFF
   set StateUI(ch1)             OFF
   set StateUI(ch2)             OFF
   set StateUI(mdyn)          	OFF
   set StateUI(mect)		    OFF
   set StateUI(aud)             OFF
   set StateUI(bal)             OFF
   set StateUI(demod)           OFF
   set StateUI(squelched)       OFF

   set StateUI(ch1,triglev)        0.000
   set StateUI(ch2,triglev)        0.000
   set StateUI(ext,triglev)        0.000
   set StateUI(mdyn,triglev)   	   0.000
   set StateUI(mect,triglev)	   0.000
   set StateUI(aud,triglev)        0.000
   set StateUI(bal,triglev)        0.000
   set StateUI(demod,triglev)      0.000
   set StateUI(squelched,triglev)  0.000

   set StateUI(ch1,horiz)        0.0
   set StateUI(ch2,horiz)        0.0
   set StateUI(mdyn,horiz)     	 0.0
   set StateUI(mect,horiz)	 0.0
   set StateUI(aud,horiz)        0.0
   set StateUI(bal,horiz)        0.0
   set StateUI(demod,horiz)      0.0
   set StateUI(squelched,horiz)  0.0

   set StateUI(ch1,vpos)        0
   set StateUI(ch2,vpos)        0
   set StateUI(mdyn,vpos)       0
   set StateUI(mect,vpos)	0
   set StateUI(aud,vpos)        0
   set StateUI(bal,vpos)        0
   set StateUI(demod,vpos)      0
   set StateUI(squelched,vpos)  0

   set StateUI(ch1,scale)        1
   set StateUI(ch2,scale)        1
   set StateUI(ext,scale)        .25
   set StateUI(mdyn,scale)       1
   set StateUI(mect,scale)	 1
   set StateUI(aud,scale)        1
   set StateUI(bal,scale)        1
   set StateUI(demod,scale)      1
   set StateUI(squelched,scale)  1
   set StateUI(baseband,vpos) 0
   set srctext(mdyn)       "MIC dyn"
   set srctext(mect)	   "MIC elec"
   set srctext(aud)        "AUDIO"
   set srctext(bal)        "Aud Bal"
   set srctext(demod)      "DEMOD"
   set srctext(squelched)  "SQLCH"
   set srctext(ch1)  "CH 1"
   set srctext(ch2)  "CH 2"
      
   set StateUI(run1)      OFF
   set StateUI(run3)      OFF
   set StateUI(baseband)  OFF
   set StateUI(baseband,raw)  aud
   set StateUI(baseband,scale) 10
      
   set  StateUI(marker_units)  "V"

   set StateUI(marker,accel) 66666
   foreach marker { 1 2 3 4 } \
   {
      array set StateUI \
      "
         marker,$marker,state  OFF
         marker,$marker,x      0.0
         marker,$marker,y      0.0
         marker,$marker,time   0.0
         marker,$marker,time,formatted 0.0
         marker,$marker,time,units Sec
         marker,$marker,mag    0.0
         marker,$marker,mag,formatted    0.0
         marker,$marker,orient VERT
         marker,$marker,spintime 0
         marker,$marker,spin_count 0
      "
   }
   foreach diff { 1 2 } \
   {
      set  StateUI(marker,diff$diff,1)       1
      set  StateUI(marker,diff$diff,2)       1
      set  StateUI(marker,diff$diff,mag)  0.00
      set  StateUI(marker,diff$diff,time)  0.00
      set  StateUI(marker,diff$diff,time_units)  Sec
   }
   
   proc Get_source_list { } \
   {
      variable srctext
      foreach item {ch1 ch2 mdyn mect aud bal demod squelched} \
      {
         lappend rv $srctext($item)
      }
      return $rv
   }
   
   #********************************************************************************************
   #       Name: Map
   # Parameters: none
   #Decsription: called when the screen is displayed
   #********************************************************************************************
   proc Map { widget class } \
   {

     OnChange hook ::Scope::State(trigtype,1)        "ScopeUI::update_trig_mode $widget"
     OnChange hook ::Scope::State(trigtype,3)        "ScopeUI::update_trig_mode $widget"
     OnChange hook ::ScopeUI::StateUI(peristance)    "ScopeUI::update_persist $widget"
     OnChange hook ::ScopeUI::StateUI(ch1,vpos)      "ScopeUI::update_oscope_vpos $widget"
     OnChange hook ::ScopeUI::StateUI(ch2,vpos)      "ScopeUI::update_oscope_vpos $widget"
     OnChange hook ::ScopeUI::StateUI(baseband,vpos) "ScopeUI::update_oscope_vpos $widget"
     OnChange hook ::ScopeUI::StateUI(baseband)      "ScopeUI::set_active_trace $widget $class"
     OnChange hook ::ScopeUI::StateUI(ch1)           "ScopeUI::set_active_trace $widget $class"
     OnChange hook ::ScopeUI::StateUI(ch2)           "ScopeUI::set_active_trace $widget $class" 
            
     # set up any markers
     foreach num { 1 2 3 4 } \
     {
        OnChange hook ::ScopeUI::StateUI(marker,$num,state)  "::ScopeUI::update_marker_on_off $widget $num"
        OnChange hook ::ScopeUI::StateUI(marker,$num,orient) "::ScopeUI::update_marker_on_off $widget $num"
     }


   }


   #********************************************************************************************
   #       Name: Cleanup
   # Parameters: none 
   #Decsription: called when the screen is hidden
   #********************************************************************************************
   proc Cleanup { widget class} \
   {
     OnChange unhook ::Scope::State(trigtype,1) "ScopeUI::update_trig_mode $widget"
     OnChange unhook ::Scope::State(trigtype,3) "ScopeUI::update_trig_mode $widget"
     OnChange unhook ::ScopeUI::StateUI(peristance) "ScopeUI::update_persist $widget"
     OnChange unhook ::ScopeUI::StateUI(ch1,vpos) "ScopeUI::update_oscope_vpos $widget"
     OnChange unhook ::ScopeUI::StateUI(ch2,vpos) "ScopeUI::update_oscope_vpos $widget"
     OnChange unhook ::ScopeUI::StateUI(baseband,vpos) "ScopeUI::update_oscope_vpos $widget"
     OnChange unhook ::ScopeUI::StateUI(baseband)      "ScopeUI::set_active_trace $widget $class"
     OnChange unhook ::ScopeUI::StateUI(ch1)           "ScopeUI::set_active_trace $widget $class"
     OnChange unhook ::ScopeUI::StateUI(ch2)          "ScopeUI::set_active_trace $widget $class" 

     # set up any markers
     foreach num { 1 2 3 4 } \
     {
        OnChange unhook ::ScopeUI::StateUI(marker,$num,state)  "::ScopeUI::update_marker_on_off $widget $num"
        OnChange unhook ::ScopeUI::StateUI(marker,$num,orient) "::ScopeUI::update_marker_on_off $widget $num"
     }
   }

   
   #*************************************************************************
   #       Name: get_input_source 
   # Parameters: none
   #Description: get input source from quarter scope 
   #             menubutton selection & convert it into searchable format
   #*************************************************************************
   proc get_input_source { {source ""} } \
   {
      variable StateUI
      if {$source == ""} { set source $StateUI(source)}
      variable srctext
      # CONVERT TO SEARCHABLE VARIABLE 
      foreach src [array names srctext] \
      {
         if [string match $srctext($src) $source] \
         {
            return $src
         }
      }
      return $source
   }


   #*************************************************************************
   #       Name: update_bb_input 
   # Parameters: none
   #Description: respond to a change of baseband input status
   #*************************************************************************
   proc update_bb_input { } \
   {
		variable StateUI
		variable srctext
		
		set src ""
		switch -- $::BBAudio::State(Route_17) \
		{
			0  { set src demod}
			23 { set src squelched}
			2  \
			{
			    switch -- $::BBAudio::State(sinadbal) \
			    {
			        0 { set src bal }
			        1 { set src aud }
			    }
			}
			1 \
			{
			    switch -- $::BBAudio::State(micbias) \
			    {
			        1 { set src mect }
			        0 { set src mdyn }
			    }
			}
		}
		set StateUI(baseband,raw) $src
		
		# if bb scope not running, done
		if {$::Scope::State(run,3) == 0 } \
		{
			set StateUI(baseband) "OFF"
		    return
		}
		if {$StateUI(baseband,raw) == ""} \
		{
		   return
		}
		set StateUI(baseband) $StateUI(baseband,raw)
		
     # Check all markers, and switch any that are bound to an inactive source
	  foreach num { 1 2 3 4} \
	  {
	     switch $StateUI(marker,$num,state) \
	     {
	        CH1 { if {$StateUI(ch1)      == "OFF"} { Update_marker_channel $num} }
	        CH2 { if {$StateUI(ch2)      == "OFF"} { Update_marker_channel $num} }
	        ON  { if {$StateUI(baseband) == "OFF"} { Update_marker_channel $num} }
	     }
	  }
        variable sourcetext
        set sourcetext $srctext($src)
		# update the UI, as needed
		set StateUI(source) $srctext($src)
	    # GET LAST SAVE HORIZONTAL POSITION
        variable horizpos
        set horizpos $StateUI($src,horiz)
        set_oscope_horiz
        # update vpos
        set StateUI(baseband,vpos) $StateUI($src,vpos)
        # reset trigger level.
        Scope triglev3 $StateUI($src,triglev) -nowait
        # update scale
        set_scope_scale aud $StateUI($src,scale)
        # update units
        update_demod_units
   }
   #*************************************************************************
   #       Name: Update_marker_channel
   # Parameters: marker to check
   #Description: insure the marker is bound to an active trace
   #             we assume the marker is on, but not valid
   #*************************************************************************
   proc Update_marker_channel { num } \
   {
      variable StateUI
      if {$StateUI(ch1) != "OFF"} \
      {
         set StateUI(marker,$num,state) CH1
         return
      }
      if {$StateUI(ch2) != "OFF"} \
      {
         set StateUI(marker,$num,state) CH2
         return
      }
      if {$StateUI(baseband) != "OFF"} \
      {
         set StateUI(marker,$num,state) ON
         return
      }
      set StateUI(marker,$num,state) OFF
   }
   #*************************************************************************
   #       Name: update_main_input 
   # Parameters: none
   #Description: respond to a change of baseband input status
   #*************************************************************************
   proc update_main_input { } \
   {
   	  variable StateUI
   	  variable srctext
   	  
   	  if { ($::Scope::State(run,1) == 0) || (($::Scope::State(enable,1)&2) == 0) } \
	  {
         set StateUI(ch1) "OFF"
	  } \
	  else \
	  {
	     set StateUI(ch1) "ON"
	  }
	  if { ($::Scope::State(run,2) == 0) || (($::Scope::State(enable,2)&2) == 0)} \
	  {
         set StateUI(ch2) "OFF"
      } \
      else \
	  {
	     set StateUI(ch2) "ON"
	  }
	  # update small scope display var if needed
	  if { ($StateUI(source) != $srctext(ch1)) && ($StateUI(source) != $srctext(ch2))} \
	  {
	     if     {$StateUI(ch1) == "ON"} { set StateUI(source) $srctext(ch1)} \
	     elseif {$StateUI(ch2) == "ON"} { set StateUI(source) $srctext(ch2)}
	  }
	  
	  # Check all markers, and switch any that are bound to an inactive source
	  foreach num { 1 2 3 4} \
	  {
	     switch $StateUI(marker,$num,state) \
	     {
	        CH1 { if {$StateUI(ch1)      == "OFF"} { Update_marker_channel $num} }
	        CH2 { if {$StateUI(ch2)      == "OFF"} { Update_marker_channel $num} }
	        ON  { if {$StateUI(baseband) == "OFF"} { Update_marker_channel $num} }
	     }
	  }
	  
	  
	  
	  # GET LAST SAVE HORIZONTAL POSITION
      variable horizpos
      set horizpos $StateUI(ch1,horiz)
      set_oscope_horiz
      set ::ScopeUI::level_unit "LVL V"
      update_demod_units
      Scope triglev1 $StateUI(ch1,triglev) -nowait
   }
   
   #*************************************************************************
   #       Name: set_scope_input
   # Parameters: none
   #Description: Set the input source - activate hw
   #*************************************************************************
   proc set_scope_input { {source ""} } \
   {
        variable StateUI
        variable srctext
        
        if {$source == ""} { set source $StateUI(source)}
        
        set source [get_input_source $source]
        
        switch -- $source \
        {
            ch1 \
            {
               Scope run3 0 -nowait
               Scope run1 1 -nowait
               Scope enable1 7 -nowait
               set StateUI(source) $srctext($source)
               set StateUI(marker_units) "V"
            }
            ch2 \
            {
               Scope run3 0 -nowait
               Scope run2 1 -nowait
               Scope enable2 7 -nowait
               set StateUI(source) $srctext($source)
               set StateUI(marker_units) "V"
            }
            mdyn \
            {
               Scope run1 0 -nowait
               Scope run2 0 -nowait
               Scope run3 1 -nowait
               BBAudio route MIC SCOPE -nowait
    	         BBAudio micbias NORMAL -nowait
            }

    	    mect \
    	    {
               Scope run1 0 -nowait
               Scope run2 0 -nowait
               Scope enable1 5 -nowait
               Scope enable2 5 -nowait
               Scope run3 1 -nowait
    	       BBAudio route MIC SCOPE -nowait
    		   BBAudio micbias POWERED -nowait
    	    }	

            aud \
            {
               Scope run1 0 -nowait
               Scope run2 0 -nowait
               Scope enable1 5 -nowait
               Scope enable2 5 -nowait
               Scope run3 1 -nowait
               BBAudio sinadbal UNBALANCED -nowait
               BBAudio route SINAD SCOPE -nowait
            }

            bal \
            {
               Scope run1 0 -nowait
               Scope run2 0 -nowait
               Scope enable1 5 -nowait
               Scope enable2 5 -nowait
               Scope run3 1 -nowait
               BBAudio sinadbal BALANCED -nowait
               BBAudio route SINAD SCOPE -nowait
            }

            demod \
            {
               Scope run1 0 -nowait
               Scope run2 0 -nowait
               Scope enable1 5 -nowait
               Scope enable2 5 -nowait
               Scope run3 1 -nowait
               BBAudio route L1 SCOPE -nowait
            }

            squelched \
            {
               Scope run1 0 -nowait
               Scope run2 0 -nowait
               Scope enable1 5 -nowait
               Scope enable2 5 -nowait
               Scope run3 1 -nowait
               BBAudio route SQUELCHED SCOPE -nowait
            }

            fgen \
            {
               Scope run1 0 -nowait
               Scope run2 0 -nowait
               Scope run3 1 -nowait
               Scope enable1 5 -nowait
               Scope enable2 5 -nowait
               BBAudio route FGEN1 USER0 -nowait
               BBAudio route FGEN2 USER0 -nowait
               BBAudio route USER0 SCOPE -nowait
            }
        }
   }
   
   #*************************************************************************
   #       Name: update_scope_horiz
   # Parameters: lun - 1, 2, or 3 logical unit number
   #Description: update scope horizontal offset display
   #*************************************************************************
   proc update_scope_horiz { lun } \
   {
      variable StateUI
      variable horizpos

      # UPDATE UI HORIZ. POSITION VALUE
      switch -- $lun \
      {
         "1" \
         {
            if {$StateUI(ch1) == "ON"} \
            {
               set hpos [expr $::Scope::State(hposition,1) / 10.0]
               set StateUI(ch1,horiz) $hpos
               set horizpos $StateUI(ch1,horiz)
            }
         }

         "2" \
         {
	        if {$StateUI(ch1) != "ON" && $StateUI(ch2) == "ON"} \
           {
               set hpos [expr $::Scope::State(hposition,2) / 10.0]
               set StateUI(ch2,horiz) $hpos
               set horizpos $StateUI(ch2,horiz)
           }
         }

         "3" \
         {
            if {$StateUI(ch1) != "ON" && $StateUI(ch2) != "ON"} \
            {
               set active $StateUI(baseband)
               set hpos [expr $::Scope::State(hposition,3) / 10.0]
               set StateUI($active,horiz) $hpos
               set horizpos $StateUI($active,horiz)
            }
         }
      }
   }


   #*************************************************************************
   #       Name: update_horizpos
   # Parameters: none
   #Description: update scope horizontal offset display
   #*************************************************************************
   proc update_horizpos { } \
   {
      variable StateUI

      if {$StateUI(run1) == "ON"} \
      {
         if {$StateUI(ch1) == "ON"} \
	 {
            update_scope_horiz 1

	 } \
	 else \
	 {
	    update_scope_horiz 2
	 }
	    

      } \
      else \
      {
         update_scope_horiz 3
      }
   }


   #*************************************************************************
   #       Name: set_oscope_horiz
   # Parameters: none
   #Description: set scope horizontal offset
   #*************************************************************************
   proc set_oscope_horiz { } \
   {
      variable horizpos
      
      # CONVERT DIV TO PIXELS
      set horiz_setting [expr int(10 * $horizpos)]
      
      Scope position0 $horiz_setting -nowait
   }


   #*************************************************************************
   #       Name: btn_vpos_bind
   # Parameters: keypad - keypad
   #             src    - input source
   #             widget - tile path
   #Description: set binding for vertical trace adjustment 
   #*************************************************************************
   proc btn_vpos_bind { widget keypad src } \
   {
      if {$keypad == "1"} \
      {
         safe_focus_force $widget
         
         activate_vpos toggle $widget $src

      } \
      elseif {$keypad == "escape"} \
      {
         activate_vpos stop $widget $src
      }
   }


   #*************************************************************************
   #       Name: get_tile
   # Parameters: a widget within the tile
   #Description: get the name of the tile frame
   #*************************************************************************
   proc get_tile { widget } \
   {
      while { 1 } \
      {
         switch -- [winfo class $widget] \
         {
           "QUARTER_SCOPE" -
           "FULL_SCOPE" { return $widget } 
         }
         set widget [winfo parent $widget]
      }
   }


   #*************************************************************************
   #       Name: set_oscope_vpos
   # Parameters: source - trace source, {ch1 ch2 mic aud audbal demod squelched}
   #             direc  - direction { up or down }
   #             widget - tile path
   #Description: sets scope vertical position
   #*************************************************************************
   proc set_oscope_vpos { source direc } \
   {
      variable StateUI

      if {$direc == "up"} \
      {
	     set vpos [expr $StateUI($source,vpos) + .0025]
      } \
      else \
      {
	     set vpos [expr $StateUI($source,vpos) - .0025]
      }

      set StateUI($source,vpos) [limit_value $vpos .5 -.5]
      if {$StateUI(baseband) == $source } { set StateUI(baseband,vpos) $StateUI($source,vpos) }
   }
   #*************************************************************************
   #       Name: update_oscope_vpos 
   # Parameters: none
   #Description: respond to a change of vertical position
   #*************************************************************************
   proc update_oscope_vpos { widget } \
   {
      variable StateUI
      set trace_widget $widget.scope
      
      $trace_widget.ch1 config -voffs $StateUI(ch1,vpos)
      $trace_widget.ch2 config -voffs $StateUI(ch2,vpos)
      $trace_widget.aud config -voffs $StateUI(baseband,vpos)
      
      set_trig_level_indicator $widget
   }


   #*************************************************************************
   #       Name: activate_vpos
   # Parameters: action - {stop toggle}
   #             name   - VPOS widget name
   #             source - trace source
   #Description: sets scope vertical position
   #*************************************************************************
   proc activate_vpos { action name source } \
   {
      set fgcolor [option get $name foreground {}]

      if {($::sys(use_spinner_edit) && $action == "toggle") || $action == "stop"} \
      {
         # RESTORE BG COLOR, TOGGLE editon
         set ::sys(use_spinner_edit) 0
         $name config -bg $::color(screenbg) \
                      -fg $fgcolor \
                      -highlightbackground $::color(screenbg) \
                      -activebackground $::color(screenbg) \
                      -activeforeground $fgcolor
         bind $name <FocusOut> { }

      } \
      elseif {!$::sys(use_spinner_edit) && $action == "toggle"} \
      {
         # CHANGE BG COLOR, TOGGLE editon
         $name config -bg $::color(editbg) \
                      -fg $::color(screenfg) \
                      -highlightbackground $::color(editbg) \
                      -activebackground $::color(editbg) \
                      -activeforeground $::color(screenfg)
         set ::sys(use_spinner_edit) 1
         bind $name <FocusOut> "::ScopeUI::activate_vpos stop %W $source"
      }
   }

   #*************************************************************************
   #       Name: set_source_frame
   # Parameters: widget - tile path
   #Description: select quarter scope input source frame 
   #             (ch1, ch2, mic dyn, mic elec, aud, audbal, demod,squelched)
   #*************************************************************************
   proc set_source_frame { widget  } \
   {
      variable StateUI
      variable srctext
      
      set tile [get_tile $widget]
      set src  [get_input_source]
      
      

      # SET QUARTER SCOPE MENU BUTTON COLOR
      place forget ${tile}.frm_rate.mnubtn_sweep2
      place forget ${tile}.frm_rate.mnubtn_sweep1

      foreach item [array names ::ScopeUI::trace] \
      {
          place forget ${tile}.frm_$item
      }
      if {$::ScopeUI::StateUI(source) == $srctext(ch1)} \
      {
         Screens::position_widget ${tile}.frm_ch1
         Screens::position_widget ${tile}.frm_rate.mnubtn_sweep1
#         ${tile}.mnubtn_src configure -foreground [${tile}.scope.ch1 cget -color]
         ${tile}.mnubtn_src configure -foreground black
      } \
      elseif {$::ScopeUI::StateUI(source) == $srctext(ch2)} \
      {
         Screens::position_widget ${tile}.frm_ch2
         Screens::position_widget ${tile}.frm_rate.mnubtn_sweep1
#         ${tile}.mnubtn_src configure -foreground [${tile}.scope.ch2 cget -color]
         ${tile}.mnubtn_src configure -foreground black
      } \
      else \
      {
         set src $StateUI(baseband)
         Screens::position_widget ${tile}.frm_$src
         Screens::position_widget ${tile}.frm_rate.mnubtn_sweep2
#         ${tile}.mnubtn_src configure -foreground [${tile}.scope.aud cget -color]
         ${tile}.mnubtn_src configure -foreground black
      }
   }
         
   #*************************************************************************
   #       Name: set_active_trace
   # Parameters: tile  - active tile
   #             class - tile class
   #Description: Update display based upon active trace changing
   #*************************************************************************
   proc set_active_trace { tile class } \
   {
      variable StateUI

      if { $class == "QUARTER_SCOPE"} \
      {
         set_source_frame $tile
      } \
      else \
      {
          # CH1 AND CH2
          foreach source { ch1 ch2 } \
          {
             if {$StateUI($source) == "ON"} \
    	     {
                Screens::position_widget ${tile}.frm_$source

             } \
             else \
             {
                place forget ${tile}.frm_$source
             }
          }

          # BASEBAND
          foreach source { aud mdyn mect bal demod squelched} \
          {
             if {$StateUI(baseband) == $source} \
             {
                Screens::position_widget ${tile}.frm_$source

             } \
             else \
             {
                place forget ${tile}.frm_$source
             }
          }

          set_trigsource_frame $tile
      }
      set_scope_trace $tile
   }

   #*************************************************************************
   #       Name: set_trigsource_frame
   # Parameters: tile  - tile path
   #Description: select source label to display for full scope
   #*************************************************************************
   proc set_trigsource_frame { tile } \
   {
      variable  StateUI
      
      if {$StateUI(baseband) != "OFF"} \
      {
         place forget ${tile}.frm_trig.mnubtn_src
	     Screens::position_widget ${tile}.frm_trig.lbl_src
         ${tile}.frm_trig.btn_trigger config -textvar ::Scope::State(trig_state,3) -command "Scope trigarm3 -nowait"
      } \
      else \
      {
          place forget ${tile}.frm_trig.lbl_src
          Screens::position_widget ${tile}.frm_trig.mnubtn_src
          ${tile}.frm_trig.btn_trigger config -textvar ::Scope::State(trig_state,1) -command "Scope trigarm1 -nowait"
      }  
   }


   #*************************************************************************
   #       Name: get_trigger_source
   # Parameters: none 
   #Description: get active trigger source 
   #*************************************************************************
   proc get_trigger_source { } \
   {
      variable StateUI
      variable active
      variable srctext

      
      if {$StateUI(baseband) != "OFF"} \
      {
         set source $StateUI(baseband)
         set sourcetext $srctext($source)

      } \
      else \
      {
         # NO TRIGGER SOURCE, SET TO CH1
         if {$::Scope::State(trigsource,1) == 0} \
	     {
	        Scope trigsource1 CH1 -nowait
	        set source ch1
	     } \
	     else \
	     {
            set source [string tolower $::Scope::State(trigsource,1)]
         }
      }

      return $source
   }

   #*************************************************************************
   #       Name: set_option_toggle
   # Parameters: var - variable to toggle
   #Description: toggle a var from ON to OFF or OFF to ON 
   #************************************************************************* 
   proc set_option_toggle { var } \
   {
      variable StateUI
   
      if {$StateUI($var) == "OFF"} \
      {
         set StateUI($var) "ON"
      } \
      else \
      {
         set StateUI($var) "OFF"
      }
      check_input $var
   }

   #*************************************************************************
   #       Name: check_input
   # Parameters: input - input source
   #Description: set up the selected input to ON or OFF
   #************************************************************************* 
   proc check_input { input } \
   {
      variable StateUI
   
      if {$StateUI($input) == "ON"} \
      {
         set_scope_input $input
      } \
      else \
      {
         switch -- $input \
         {
            "ch1" { Scope enable1 [expr $::Scope::State(enable,1) & ~2] -nowait}
            "ch2" { Scope enable2 [expr $::Scope::State(enable,2) & ~2] -nowait}
         }
      }
   }
  

   #*************************************************************************
   #       Name: set_scope_options_menu
   # Parameters: tile - tile path
   #Description: draw Scope options menu
   #************************************************************************* 
   proc set_scope_options_menu { tile } \
   {
      variable menu_label

      set accelNum   1

      set topWin  [winfo toplevel $tile]
      
      set scope_tile_path  [name_child  $topWin  $::FullScopeUI::ShortName]
      
      # Get the Options menubutton path
      set buttonName  [View::GetOptionMenuName  $topWin]
    
      option add  *$buttonName*helpUrl  index.html  widgetDefault
    
      # Get the Options menu path
      set menuName  ${buttonName}.menu
    
      # Destroy the existing Options menu
      destroy  $menuName
    
      # Create a new Options menu
      menu $menuName  -tearoff 0  -postcommand ""

      $menuName insert 0 command                                     \
                            -label    "Accept Options"               \
                            -accel    RETURN                         \
                            -command  "tkMenuUnpost %W"


      foreach { key } { ch1  ch2 } \
      {
         $menuName add check -accel    $accelNum			\
                             -label    $menu_label($key)   		\
	 		                 -onvalue  ON				\
	 		                 -offvalue OFF				\
	 		                 -command  "ScopeUI::check_input $key"	\
                             -variable ::ScopeUI::StateUI($key)

         bind $menuName $accelNum "ScopeUI::set_option_toggle $key"
 
         incr accelNum
      }
      $menuName add separator
      
      foreach { key } { mdyn  mect  aud  bal  demod squelched} \
      {
         $menuName add radio -accel    $accelNum			\
                             -label    $menu_label($key)   		\
	 		                 -value  $key				\
	 		                 -command  "ScopeUI::set_scope_input $key;tkMenuUnpost $menuName"	\
                             -variable ::ScopeUI::StateUI(baseband)

         bind $menuName $accelNum "ScopeUI::set_scope_input $key;tkMenuUnpost $menuName"
 
         incr accelNum
      }
      
      incr accelNum

      # Bind button release so default bindings do not unpost the menu
      bind $menuName <ButtonRelease-1> { MenuInvokeRemain %W 1; break }  
      bind $menuName <KeyRelease-Return> { MenuInvokeRemain %W 0; break }

      # Bind to the Return and Unmap
      bind $menuName <KeyRelease-F8> "tkMenuUnpost %W"
      
      return  $buttonName
   }

   #*************************************************************************
   #       Name: clear_persis
   # Parameters: widget - tile path 
   #Description: Resets the persistance buffer
   #*************************************************************************
   proc clear_persist { widget } \
   {
      set tile [get_tile $widget]
      $tile.scope clear
   }
  

   #*************************************************************************
   #       Name: set_persis
   # Parameters: widget - tile path
   #Description: set trace widget persistance
   #*************************************************************************
   proc set_persist { } \
   {
      variable StateUI
     
      if {$StateUI(peristance) == "OFF"} \
      {
         set StateUI(peristance) ON
      } \
      else \
      {
         set StateUI(peristance) OFF
      }
   }
   #*************************************************************************
   #       Name: set_persis
   # Parameters: widget - tile path
   #Description: set trace widget persistance
   #*************************************************************************
   proc update_persist { widget } \
   {
      variable StateUI

      set tile [get_tile $widget]
      
      if {$StateUI(peristance) == "ON"} \
      {
         ${tile}.scope config -persist 100
      } \
      else \
      {
         ${tile}.scope config -persist 0
      }

      ${tile}.scope clear
   }

   #*************************************************************************
   #       Name: set_trig_level_indicator
   # Parameters: widget - tile path
   #Description: draws trigger level position indicator
   #*************************************************************************
   proc set_trig_level_indicator { widget } \
   {
      variable StateUI

      if {![winfo exists $widget]} { return }

      set tile [get_tile $widget]
     
      # NOT IN FULL SCOPE, DON'T DRAW LEVEL INDICATOR
      if {[winfo class $tile] != "FULL_SCOPE"} \
      {
         return
      }

      set scale $tile.lev_pointer
      if ![winfo exists $scale] { return }
      if ![winfo ismapped $scale] { return }
      
      set height [winfo height $scale]
      set width  [winfo width $scale]
      set half [expr $width/2]
      
      # remove all old items
      $scale delete baseband_gnd ch1_gnd ch2_gnd bb_trig main_trig
           
      # update baseband scope ground point and show trigger
      if {$StateUI(baseband) != "OFF"} \
      {
         set color [${tile}.scope.aud cget -foreground]
         set vp    [expr int( (0.5 - $StateUI(baseband,vpos)) * $height  +.5 )]
         set trig  [expr int(  (0.5 + $StateUI(trig_rel,baseband)) * $height + .5)]
         $scale create line 0 $vp $half $vp -tag baseband_gnd -fill $color -width 3
         $scale create line $half $trig $width $trig -tag bb_trig      -fill $color -arrow last -arrowshape  {5 5 5}
      }
      
      if {$StateUI(ch1) != "OFF" } \
      {
         set color [${tile}.scope.ch1 cget -foreground]
         set vp [expr int( (.5 - $StateUI(ch1,vpos)) * $height + .5)]
         $scale create line 0 $vp $half $vp -tag ch1_gnd -fill $color -width 3
         if { ($::Scope::State(trigsource,1) == "CH1") && ($StateUI(trig_rel,main) != "") } \
         {
             set trig  [expr int(  (0.5 + $StateUI(trig_rel,main)) * $height + .5)]
             $scale create line $half $trig $width $trig -tag main_trig  -fill $color -arrow last -arrowshape  {5 5 5}
         }
      }
      if {$StateUI(ch2) != "OFF" } \
      {
         set color [${tile}.scope.ch2 cget -foreground]
         set vp [expr int( (.5 - $StateUI(ch2,vpos)) * $height + .5)]
         $scale create line 0 $vp $half $vp -tag ch2_gnd -fill $color -width 3
         if {($::Scope::State(trigsource,1) == "CH2") && ($StateUI(trig_rel,main) != "") } \
         {
             set trig  [expr int(  (0.5 + $StateUI(trig_rel,main)) * $height + .5)]
             $scale create line $half $trig $width $trig -tag main_trig  -fill $color -arrow last -arrowshape  {5 5 5}
         }
      }
   }


   #*************************************************************************
   #       Name: set_trig_mode
   # Parameters: none 
   #Description: set trigger mode
   #*************************************************************************
   proc set_trig_mode { } \
   {
      switch -- $::Scope::State(trigtype) \
      {
         "AUTO" { Scope trigtype0  0  -nowait}
	     "NORM" { Scope trigtype0  1  -nowait}
	     "ONE"  { Scope trigtype0  2  -nowait}
      }
   }


   #*************************************************************************
   #       Name: update_trig_mode
   # Parameters: widget - tile path
   #Description: update trigger button state and label
   #*************************************************************************
   proc update_trig_mode { widget } \
   {
      set tile [get_tile $widget]

      switch -- $::Scope::State(trigtype,1) \
      {
        1  { # AUTO TRIGGER MODE
                 
                  ${tile}.frm_trig.btn_trigger config  -state disabled
                  
	              if {[winfo class $tile] == "FULL_SCOPE"} \
                  {
	                 Screens::position_widget ${tile}.frm_persist
	             
                     View::ClearSoftkey $tile 7
                  }
                }
                
        1  { # NORM TRIGGER MODE
                  
                  ${tile}.frm_trig.btn_trigger config  -state disabled
                  
    	          if {[winfo class $tile] == "FULL_SCOPE"} \
    	          {
    	             Screens::position_widget ${tile}.frm_persist
    	             
    	             View::ClearSoftkey $tile 7
    	          }
                }
                
        2   { # ONE SHOT TRIGGER MODE
                  
                  ${tile}.frm_trig.btn_trigger config  -state normal
                  
    	          if {[winfo class $tile] == "FULL_SCOPE"} \
    	          {
    	             Screens::position_widget ${tile}.frm_persist
                      
    	             ::View::AssignSoftkey $tile 7 "ARM SCOPE" "Scope trigarm0 -nowait"
    	          }
                }
      }
   }

   #*************************************************************************
   #       Name: update_trig_level
   # Parameters: tile - tile path
   #Description: update trigger indicator and display value
   #*************************************************************************
   proc update_trig_level { lun } \
   {
      variable StateUI
      set trig_raw $::Scope::State(triglev,$lun)
      set StateUI(triglevel) [format %.3f $trig_raw]
      
      switch -- $lun \
      {
        1 -
        2 \
        {
           switch -- $::Scope::State(trigsource,$lun) \
           {
              "CH1" \
              {
                 set StateUI(ch1,triglev) $trig_raw
                 set StateUI(trig_rel,main) [expr -$trig_raw / (10 * $StateUI(ch1,scale)) - $StateUI(ch1,vpos)]
              }
              "CH2" \
              {
                 set StateUI(ch2,triglev) $trig_raw
                 set StateUI(trig_rel,main) [expr -$trig_raw / (10 * $StateUI(ch2,scale)) - $StateUI(ch2,vpos)]
              }
              default \
              {
                 set StateUI(ext,triglev) $trig_raw
                 set StateUI(trig_rel,main) [expr -$trig_raw / (10 * $StateUI(ch1,scale)) - $StateUI(ch1,vpos)]
              }
           }
        }
        3 \
        {
           set StateUI(baseband,triglev) $trig_raw
           set StateUI($StateUI(baseband),triglev) $trig_raw
           set StateUI(trig_rel,baseband) [expr -$trig_raw / (10* $StateUI(baseband,scale)) - $StateUI(baseband,vpos)]
        }
      }
   }


   #*************************************************************************
   #       Name: set_trig_source
   # Parameters: tile - tile path
   #Description: set trigger source, level
   #*************************************************************************
   proc set_trig_source { }\
   {
      variable StateUI
      Scope trigsource1 $::Scope::State(trigsource,1) -nowait
   }
   proc update_trig_source { } \
   {
      variable StateUI
      
      switch -- $::Scope::State(trigsource,1) \
      {
         "CH1"   {  Scope triglev1 $StateUI(ch1,triglev) -nowait }
         "CH2"   {  Scope triglev1 $StateUI(ch2,triglev) -nowait }
         default {  Scope triglev1 $StateUI(ext,triglev) -nowait }
      }
   }


   #*************************************************************************
   #       Name: set_scope_trace
   # Parameters: source - scope source {ch1 ch2 mic aud audbal demod squelched}
   #             tile   - scope trace widget path name
   #Description: set scope trace active and trace height
   #*************************************************************************
   proc set_scope_trace { widget } \
   {
      variable StateUI

      set tile [get_tile $widget]
      
      if ![winfo exists $tile] { return }

      set scope ${tile}.scope

      if {$StateUI(ch1) == "ON" } \
      {
         $scope.ch1 config -active 1
      } \
      else \
      {
         $scope.ch1 config -active 0
      }
      if {$StateUI(ch2) == "ON" } \
      {
         $scope.ch2 config -active 1
      } \
      else \
      {
         $scope.ch2 config -active 0
      }
      if {$StateUI(baseband) != "OFF" } \
      {
         $scope.aud config -active 1
      } \
      else \
      {
         $scope.aud config -active 0
      }
   }

   proc Set_coupling { lun } \
   {
      Scope coupling$lun $::Scope::State(coupling,$lun) -nowait
   }
   #*************************************************************************
   #       Name: update_ch1_coupling
   # Parameters: none
   #Description: update based upon a change of coupling
   #*************************************************************************
   proc update_ch1_coupling { } \
   {
      variable StateUI
      if {$::Scope::State(coupling,1) != "GND"} \
      {
    	 set_scope_scale ch1 $StateUI(ch1,scale)
      }

   }
   proc update_ch2_coupling { } \
   {
      variable StateUI
      if {$::Scope::State(coupling,2) != "GND"} \
      {
    	 set_scope_scale ch2 $StateUI(ch2,scale)
      }
   }

   #*************************************************************************
   #        Name: set_scope_trigslope
   #  Parameters: none
   # Description: toggle Scope trigger slope polarity 
   #*************************************************************************
   proc set_scope_trigslope { } \
   {
      if {$::Scope::State(trigpol,1) == "RISE"} \
      {
         Scope trigpol0 0 -nowait

      } \
      else \
      {
         Scope trigpol0 1 -nowait
      }
   }


   #*************************************************************************
   #        Name: set_scope_scale
   #  Parameters: lun  - 1, 2, or 3
   #              vdiv - Volts/Div
   # Description: set Scope vertical scale 
   #*************************************************************************
   proc set_scope_scale { chan vdiv } \
   {
      variable lun

      # Bypass the update if being called due to a recall of a setup
      if { ![info exists ::Recall_SetupsUI::State(recall_in_process)] } \
      {
          return
      }
      if { $::Recall_SetupsUI::State(recall_in_process) == 1 } \
      {
          return
      }
      set chan_lun $lun($chan)
      set scale 100
      
      switch -- $vdiv \
      {
         .01 {set scale 1}
         .02 {set scale 2}
         .05 {set scale 3}
         .1  {set scale 4}
         .2  {set scale 5}
         .5  {set scale 6}
         1   {set scale 7}
         2   {set scale 8}
         5   {set scale 9}
         10  {set scale 10}
         20  {set scale 11}
         50  {set scale 12}
      }

      Scope scale$chan_lun $scale -nowait
   }


   #*************************************************************************
   #        Name: update_scope_scale
   #  Parameters: lun - 1, 2, or 3 logical unit number
   #              tile - tile path
   # Description: update Scope vertical scale label
   #*************************************************************************
   proc update_scope_scale { lun } \
   {
      variable  StateUI

      switch -- $lun \
      {
        1 \
        {
           update_scale_label ch1 $::Scope::State(scale,1)
        }
        2 \
        {
           update_scale_label ch2 $::Scope::State(scale,2)
        }
        3 \
        {
          update_scale_label baseband $::Scope::State(scale,3)
          if {$StateUI(baseband,raw) != ""} \
          {
            set StateUI($StateUI(baseband,raw),scale) $StateUI(baseband,scale)
          }
        }
      }
      update_trig_level $lun
   }


   #*************************************************************************
   #        Name: update_scope_label
   #  Parameters: source - input source
   #              scale - V/Div, kHz/Div, or %/Div
   # Description: update Scope vertical scale label
   #*************************************************************************
   proc update_scale_label { source vscale } \
   {
      variable StateUI

      switch -- $vscale \
      {
         1   { set StateUI($source,scale)  ".01" }
         2   { set StateUI($source,scale)  ".02" }
         3   { set StateUI($source,scale)  ".05" }
         4   { set StateUI($source,scale)  ".1"  }
         5   { set StateUI($source,scale)  ".2"  }
         6   { set StateUI($source,scale)  ".5"  }
         7   { set StateUI($source,scale)  "1"   }
         8   { set StateUI($source,scale)  "2"   }
         9   { set StateUI($source,scale)  "5"   }
         10  { set StateUI($source,scale)  "10"  }
         11  { set StateUI($source,scale)  "20"  }
         12  { set StateUI($source,scale)  "50"  }
      }
   }


   #*************************************************************************
   #       Name: set_scope_sweep
   # Parameters: none 
   #Description: set scope sweep
   #*************************************************************************
   proc set_scope_sweep { lun } \
   {
      set rate $::Scope::State(sweep,$lun)
      if ![info exists ::Scope::sweep_map($rate)] { return }
      Scope sweep$lun $::Scope::sweep_map($rate) -nowait
   }

  
   #*************************************************************************
   #       Name: set_oscope_scale
   # Parameters: source - input source
   #Description: set scope scale
   #*************************************************************************
   proc set_oscope_scale { widget source } \
   {
      variable trace
      variable StateUI

      if {$source == "ch1"} \
      {
         if {$::Scope::State(coupling,1) != "GND"} \
         {
            set_scope_scale $trace(ch1) $StateUI(ch1,scale)
         }

      } \
      elseif {$source == "ch2"} \
      {
         if {$::Scope::State(coupling,2) != "GND"} \
         {
            set_scope_scale $trace(ch2) $StateUI(ch2,scale)
	 }
      
      } \
      else \
      {
         set_scope_scale $trace($source) $StateUI($source,scale)
      }

   }

   #*************************************************************************
   #        Name: update_demod_units
   #  Parameters: none 
   # Description: update demod unit depending on scope input
   #*************************************************************************
   proc update_demod_units { } \
   {
      variable level_unit
      variable demod_unit
      variable StateUI


      if {($StateUI(baseband) == "demod") || ($StateUI(baseband) == "squelched")} \
      {
          switch -- $::Receiver::State(demod_raw) \
          {
             0 -
    	     1 -
    	     2 -
    	     3 -
    	     10 {
    		       set demod_unit "kHz/div"
    	           set level_unit "LVL kHz"
                   set StateUI(marker_units) "kHz"
    	        }

             4 -
             5 -
             12 -
             13 -
             15 -
             16 -
             17 -
             18 -
             19 -
             20 -
             21 -
             22 -
             23 -
             24 {
                   set demod_unit "%/div"
                   set level_unit "LVL %"
                   set StateUI(marker_units) "%"
    	        }
          }
          update_scope_scale 3
      } \
      else \
      {
         set demod_unit "V"
         set level_unit "LVL V"
         set StateUI(marker_units) "V"
      }
   }

   #*************************************************************************
   #        Name: zoom_scope_screen
   #  Parameters: none 
   # Description: zoom/switch to full scope screen 
   #*************************************************************************
   proc zoom_scope_screen { widget } \
   {
      set tile [winfo toplevel $widget]
      View::AssignReturn $widget "::Screens::Show $tile $::Screens::CurrentNamespace"
      
      ::Screens::Show $widget Scope_Screen
   }


   #*************************************************************************
   #        Name: set_scope_softkeys
   #  Parameters: tile - tile path
   # Description: set scope softkeys
   #*************************************************************************
   proc set_scope_softkeys { tile } \
   {
      variable active
      variable StateUI
      variable srctext

      set src $active

      # CLEAR SOFTKEYS
      ::View::ClearAllSoftkeys $tile

      # SOFTKEY 2 (SWEEP RATE)
      if {$StateUI(baseband) != "OFF"} \
      {
         set frm_sweep ${tile}.frm_rate.mnubtn_sweep2
         place forget ${tile}.frm_rate.mnubtn_sweep1

         
      } \
      else \
      {
         set frm_sweep ${tile}.frm_rate.mnubtn_sweep1
         place forget ${tile}.frm_rate.mnubtn_sweep2
      }
      Screens::position_widget $frm_sweep
      
      ::View::AssignSoftkey $tile 2 "MARKERS" "::ScopeUI::config_marker $tile"

      # SOFTKEY 3, 4, 5, & 6
      if {$StateUI(ch1) == "ON"} \
      {
         ::View::AssignSoftkey $tile 3 "CH1 SCALE" \
            "post_menu ${tile}.frm_ch1.mnubtn_vdiv1"

         ::View::AssignSoftkey $tile 5 "CH1 VPOS" \
            "focus ${tile}.frm_ch1.btn_vpch1; ${tile}.frm_ch1.btn_vpch1 invoke"

         if {$::ScopeUI::StateUI(ch2) == "ON"} \
         {
            ::View::AssignSoftkey $tile 4 "CH2 SCALE" \
               "post_menu ${tile}.frm_ch2.mnubtn_vdiv2"

            ::View::AssignSoftkey $tile 6 "CH2 VPOS" \
               "focus ${tile}.frm_ch2.btn_vpch2; ${tile}.frm_ch2.btn_vpch2 invoke"
         }
         
         
         # SOFTKEY 7 (ARM SCOPE)
         if {$::Scope::State(trigtype,1) == "ONE"} \
         {
           ::View::AssignSoftkey $tile 7 "ARM SCOPE" "Scope  trigarm1 -nowait"
         }
      } \
      elseif {$StateUI(ch2) == "ON"} \
      {
         ::View::AssignSoftkey $tile 4 "CH2 SCALE" \
            "post_menu ${tile}.frm_ch2.mnubtn_vdiv2"

         ::View::AssignSoftkey $tile 6 "CH2 VPOS" \
            "focus ${tile}.frm_ch2.btn_vpch2; ${tile}.frm_ch2.btn_vpch2 invoke"
            
         # SOFTKEY 7 (ARM SCOPE)
         if {$::Scope::State(trigtype,2) == "ONE"} \
         {
           ::View::AssignSoftkey $tile 7 "ARM SCOPE" "Scope  trigarm2 -nowait"
         }

      } \
      elseif {$StateUI($src) == "ON"} \
      {
         set keytext [string trim $srctext($src)]
       
         ::View::AssignSoftkey $tile 5 "$keytext VPOS" \
            "focus ${tile}.frm_${src}.btn_vp${src}; ${tile}.frm_${src}.btn_vp${src} invoke"

         ::View::AssignSoftkey $tile 3 "$keytext SCALE" \
            "post_menu ${tile}.frm_${src}.mnubtn_vd${src}"
         
         if {$::Scope::State(trigtype,3) == "ONE"} \
         {
           ::View::AssignSoftkey $tile 7 "ARM SCOPE" "Scope  trigarm3 -nowait"
         }
      }

   }
 
  #***************************************************************************************************
  #
  # Name: update_vert_marker
  #
  # Parameters: num    - marker number
  #
  # Description: update the text on a vertical marker
  #
  #***************************************************************************************************
  proc update_vert_marker { num } \
  {
     variable StateUI
     set engine_num [expr $num -1 ]
     
     set time_units "sec"
     set time $StateUI(marker,$num,time)
     set mag  $StateUI(marker,$num,mag)
     set mag_units $StateUI(marker_units)
     if {$time < 1.0} \
     {
        set time_units "mSec"
        set time [expr $time*1000]
     }
     set StateUI(marker,$num,mag,formatted) [format "%1.3f" $mag]
     set format "% 7.1f %s\n%8s %s"
     switch $StateUI(marker,$num,state) \
     {
       "CH1" \
       {
         upvar #0 ::ScopeUI::StateUI(ch1,vpos) vpos
         set format "CH1 $format"
         set mag_units V
       }
       "CH2" \
       {
         upvar #0 ::ScopeUI::StateUI(ch2,vpos) vpos
         set format "CH2 $format"
         set mag_units V
       }
     }
     set StateUI(marker,$num,text) [format $format $time $time_units $StateUI(marker,$num,mag,formatted) $mag_units]
  }
  #***************************************************************************************************
  #
  # Name: update_horz_marker
  #
  # Parameters: num    - marker number
  #
  # Description: update the text on a horizontal marker
  #
  #***************************************************************************************************
  proc update_horz_marker { num } \
  {
     variable StateUI
     
     set format "%5.1f %s"

     switch $StateUI(marker,$num,state) \
     {
       "CH1" \
       {
         upvar #0 ::ScopeUI::StateUI(ch1,vpos) vpos
         set mag_units V
         set scale $StateUI(ch1,scale)
         set format "CH1 $format"
       }
       "CH2" \
       {
         upvar #0 ::ScopeUI::StateUI(ch2,vpos) vpos
         set mag_units V
         set scale $StateUI(ch2,scale)
         set format "CH2 $format"
       }
       "ON" \
       {
         upvar #0 ::ScopeUI::StateUI(baseband,vpos) vpos
         set mag_units $StateUI(marker_units)
         set scale $StateUI(baseband,scale)
       }
       default \
       {
          return
       }
     }
     set mag  $StateUI(marker,$num,mag)
     set StateUI(marker,$num,mag,formatted) [format "%1.3f" $mag]

     set StateUI(marker,$num,text) [format $format $mag $mag_units]
     set StateUI(marker,$num,y) [expr .5-$StateUI(marker,$num,mag)/($scale*10)-$vpos]
  }
  #***************************************************************************************************
  #
  # Name: setup_marker
  #
  # Parameters: num    - marker number
  #
  # Description: does the non-display related setup for a marker - binding it to the appropriate
  #              engine level marker & setting up the update routines
  #
  #***************************************************************************************************
  proc setup_marker { num } \
  {
    variable StateUI
    # remove any current variable traces & disable the low-level messages
    set mask [expr 1<<(3+$num)]
    foreach lun {1 2 3 } \
    {
       OnChange unhook ::Scope::State(marker,$lun,$num,live) "::ScopeUI::update_marker_data $num $lun"
       OnChange unhook ::ScopeUI::StateUI(marker_units)      "::ScopeUI::update_vert_marker $num $lun"
       Scope enable$lun [expr $Scope::State(enable,$lun) & ~$mask] -nowait
    }
    OnChange unhook ::ScopeUI::StateUI(marker,$num,mag) "::ScopeUI::update_horz_marker $num"
    OnChange unhook ::ScopeUI::StateUI(marker,$num,mag) "::ScopeUI::update_vert_marker $num"
    OnChange unhook ::ScopeUI::StateUI(marker_units)    "::ScopeUI::update_horz_marker $num"
    OnChange unhook ::ScopeUI::StateUI(ch1,vpos)        "::ScopeUI::update_horz_marker $num"
    OnChange unhook ::ScopeUI::StateUI(ch2,vpos)        "::ScopeUI::update_horz_marker $num"
    OnChange unhook ::ScopeUI::StateUI(baseband,vpos)   "::ScopeUI::update_horz_marker $num"
    OnChange unhook ::ScopeUI::StateUI(baseband,scale)  "::ScopeUI::update_horz_marker $num"
    OnChange unhook ::ScopeUI::StateUI(ch1,scale)       "::ScopeUI::update_horz_marker $num"
    OnChange unhook ::ScopeUI::StateUI(ch1,scale)       "::ScopeUI::update_horz_marker $num"
    update idle
    # disable any messages from these markers
    if {$StateUI(marker,$num,orient) == "VERT" } \
    {
       setup_vert_marker $num
    } \
    else \
    {
       setup_horiz_marker $num
    }
  }
  #***************************************************************************************************
  #
  # Name: setup_vert_marker
  #
  # Parameters: num    - marker number
  #
  # Description: does the non-display related setup for a marker - binding it to the appropriate
  #              engine level marker & setting up the update routines
  #
  #***************************************************************************************************
 proc setup_vert_marker { num } \
 {
    variable StateUI
    set mask [expr 1<<(3+$num)]
    switch $StateUI(marker,$num,state) \
    {
       "CH1" \
       {
         set lun 1
       }
       "CH2" \
       {
         set lun 2
       }
       "ON" \
       {
         set lun 3
       }
       default { return }
    }
    OnChange hook ::Scope::State(marker,$lun,$num,live) "::ScopeUI::update_marker_data $num $lun"
    Scope enable$lun [expr $Scope::State(enable,$lun) | $mask] -nowait
    OnChange hook ::ScopeUI::StateUI(marker,$num,mag) "::ScopeUI::update_vert_marker $num"
    OnChange hook ::ScopeUI::StateUI(marker_units)    "::ScopeUI::update_vert_marker $num"
  }

  #***************************************************************************************************
  #
  # Name: setup_horiz_marker
  #
  # Parameters: num    - marker number
  #
  # Description: does the non-display related setup for a marker - binding it to the appropriate
  #              engine level marker & setting up the update routines
  #
  #***************************************************************************************************
  proc setup_horiz_marker { num } \
  {
    variable StateUI
    switch $StateUI(marker,$num,state) \
    {
       "CH1" \
       {
         set vpos ::ScopeUI::StateUI(ch1,vpos)
         set scale ::ScopeUI::StateUI(ch1,scale)
       }
       "CH2" \
       {
         set vpos ::ScopeUI::StateUI(ch2,vpos)
         set scale ::ScopeUI::StateUI(ch2,scale)
       }
       "ON" \
       {
         set vpos ::ScopeUI::StateUI(baseband,vpos)
         set scale ::ScopeUI::StateUI(baseband,scale)
       }
       default { return }
    }
    OnChange hook ::ScopeUI::StateUI(marker,$num,mag) "::ScopeUI::update_horz_marker $num"
    OnChange hook ::ScopeUI::StateUI(marker_units)    "::ScopeUI::update_horz_marker $num"
    OnChange hook $vpos                               "::ScopeUI::update_horz_marker $num"
    OnChange hook $scale                              "::ScopeUI::update_horz_marker $num"
 }
  #***************************************************************************************************
  #
  # Name: update_marker_diff
  #
  # Parameters: difference # to set up
  #
  # Description: update the difference info for a marker difference
  #
  #***************************************************************************************************
 proc update_marker_diff { diff } \
 {
    variable StateUI
    set num1 $StateUI(marker,diff$diff,1)
    set num2 $StateUI(marker,diff$diff,2)
    set mag  [expr $::ScopeUI::StateUI(marker,$num1,mag)  - $::ScopeUI::StateUI(marker,$num2,mag)]
    set time [expr $::ScopeUI::StateUI(marker,$num1,time) - $::ScopeUI::StateUI(marker,$num2,time)]
    set a [expr abs($time)]
    set StateUI(marker,diff$diff,mag) [format "% 3.3f" $mag]
    if {$a < .001} \
    {
       set StateUI(marker,diff$diff,time) [format "% 4.2f" [expr 1E6*$time]]
       set StateUI(marker,diff$diff,time_units) uSec
    } \
    elseif { $a < 1} \
    {
       set StateUI(marker,diff$diff,time) [format "% 4.2f" [expr 1E3*$time]]
       set StateUI(marker,diff$diff,time_units) mSec
    } \
    else \
    {
       set StateUI(marker,diff$diff,time) [format "% 4.2f" $time]
       set StateUI(marker,diff$diff,time_units) Sec
    }
 }
  #***************************************************************************************************
  #
  # Name: setup_marker_diff
  #
  # Parameters: difference # to set up
  #
  # Description: set up the trace on the variables to update a marker difference
  #
  #***************************************************************************************************
 proc setup_marker_diff { diff } \
 {
    # remove any old traces
    variable StateUI
    set cmd "ScopeUI::update_marker_diff $diff"
    foreach num { 1 2 3 4 } \
    {
       OnChange unhook ::ScopeUI::StateUI(marker,$num,time) $cmd
       OnChange unhook ::ScopeUI::StateUI(marker,$num,mag)  $cmd
    }
    # hook up to the 2 we are listening to
    set num1 $StateUI(marker,diff$diff,1)
    set num2 $StateUI(marker,diff$diff,2)
    OnChange hook ::ScopeUI::StateUI(marker,$num1,time) $cmd
    OnChange hook ::ScopeUI::StateUI(marker,$num1,time) $cmd
    OnChange hook ::ScopeUI::StateUI(marker,$num1,mag)  $cmd
    OnChange hook ::ScopeUI::StateUI(marker,$num2,mag)  $cmd
    update_marker_diff $diff
 }
 foreach diff { 1 2 } \
 {
    setup_marker_diff $diff
    OnChange hook ::ScopeUI::StateUI(marker,diff$diff,1) "ScopeUI::setup_marker_diff $diff"
    OnChange hook ::ScopeUI::StateUI(marker,diff$diff,2) "ScopeUI::setup_marker_diff $diff"
 }
  #***************************************************************************************************
  #
  # Name: update_marker_data
  #
  # Parameters: num    - marker number, lun - engine LUN we are connected to
  #
  # Description: copies the data from the engine to the display variables.
  #
  #***************************************************************************************************
 proc update_marker_data { num lun } \
 {
    variable StateUI
    set StateUI(marker,$num,mag)  $::Scope::State(marker,$lun,$num,live)
    if {$StateUI(marker,$num,time) != $::Scope::State(marker,$lun,$num,time)} \
    {
       set StateUI(marker,$num,time) $::Scope::State(marker,$lun,$num,time)
       update_marker_time_formatted $num
    }
    if {$StateUI(marker,$num,x) != $::Scope::State(marker,$lun,$num,pos)} \
    {
       set StateUI(marker,$num,x)    $::Scope::State(marker,$lun,$num,pos)
    }
 }
 proc update_marker_time_formatted { num } \
 {
   variable StateUI
   switch $StateUI(marker,$num,time,units)\
   {
      "Sec"   {set StateUI(marker,$num,time,formatted) [format "%3.2f" $StateUI(marker,$num,time)]}
      "mSec"  {set StateUI(marker,$num,time,formatted) [format "%3.2f" [expr 1E3*$StateUI(marker,$num,time)]]}
      "uSec"  {set StateUI(marker,$num,time,formatted) [format "%3.2f" [expr 1E6*$StateUI(marker,$num,time)]]}
   }
 }
  #***************************************************************************************************
  #
  # Name: update_marker_on_off
  #
  # Parameters: widget - invoking widget path
  #             num    - marker number
  #
  # Description: Displays a marker on the Scope widget, or removes it from 
  #              the display
  #
  #***************************************************************************************************
  proc update_marker_on_off {widget num} \
  {
    
    variable  StateUI
    
    set tile_path  [get_tile  $widget]
    set scope $tile_path.scope
    if { ![winfo exists $scope] } \
    {
      return
    }

    set block ${scope}.block$num
    set line  ${scope}.line$num
    set text  ${scope}.text$num

    destroy $block
    destroy $line
    destroy $text
    
    if {$StateUI(marker,$num,state) == "OFF" } {return }
    
    set bg           [$scope cget -bg]
    set trace_width  [winfo reqwidth $scope]
    set trace_height [winfo reqheight $scope]
       
    button $block -text $num -font analyzer_tiny -just center \
                  -highlightthickness 2 \
                  -activebackground white -activeforeground black \
                  -command "ScopeUI::Marker_toggle $block $num"
    set fg [$block cget -bg]
    add_bindtag $block Cursor ScopeMarker$num
    
    # Marker text
    label $text -anchor e -just right -bg $bg -font analyzer_tiny \
                  -textvar ::ScopeUI::StateUI(marker,$num,text) \
                  -fg $fg -width 11
    add_bindtag $text ScopeMarker$num
    switch $StateUI(marker,$num,orient) \
    {
       "HORIZ" \
       {
          canvas $line -height 3 -width $trace_width -bg $fg
          set cmd "ScopeUI::position_horz_marker $widget $num"
          set var ::ScopeUI::StateUI(marker,$num,y)
       }
       "VERT" \
       {
          canvas $line -height $trace_height -width 3 -bg $fg
          set cmd "ScopeUI::position_vert_marker $widget $num"
          set var ::ScopeUI::StateUI(marker,$num,x)
       }
       default \
       {
          set StateUI(marker,$num,orient) VERT
          return
       }
    }
    add_bindtag $line ScopeMarker$num
    # update our position on a change in the mag variable
    OnChange hook $var $cmd
    bind $block <Destroy> "OnChange unhook $var \"$cmd\""
  }
  #***************************************************************************************************
  #
  # Name: Marker_toggle,Marker_activate,Marker_deactivate
  #
  # Parameters: widget - invoking widget path
  #             num    - marker number
  #
  # Description: Toggle/set/clear a marker from normal to active (spinner controlled)
  #
  #***************************************************************************************************
  proc Marker_toggle {scope num} \
  {
     switch -- [$scope cget -state] \
     {
        "normal" { Marker_activate $scope $num}
        "active" { Marker_deactivate $scope $num}
     }
  }
  proc Marker_activate {scope num} \
  {
     variable StateUI
     change_marker $scope $num 1
     set ::sys(use_spinner_edit) 1
  }
  proc Marker_deactivate {scope num} \
  {
     variable StateUI
     change_marker $scope $num 0
     set ::sys(use_spinner_edit) 0
  }

  #***************************************************************************************************
  #
  # Name: Drag_marker
  #
  # Parameters: widget - invoking widget path
  #             num    - marker number
  #             X and Y of pointer
  #
  # Description: Drag a marker with the mouse
  #
  #***************************************************************************************************
  proc Drag_marker { marker num x y} \
  {
     variable StateUI
     if [info exists StateUI(marker,$num,after)] \
     {
        after cancel $StateUI(marker,$num,after)
     }
     set cmd "ScopeUI::Drag_marker_update $marker $num $x $y"
     set StateUI(marker,$num,after) [after idle $cmd]
  }
  proc Drag_marker_update { marker num x y} \
  {
     variable StateUI
     if [info exists StateUI(marker,$num,after)] \
     {
        unset StateUI(marker,$num,after)
     }
     switch $StateUI(marker,$num,state) \
     {
         "CH1" \
         {
           set vscale $StateUI(ch1,scale)
         }
         "CH2" \
         {
            set vscale $StateUI(ch2,scale)
         }
         "ON" \
         {
            set vscale $StateUI(baseband,scale)
         }
         default \
         {
            return
         }
      }
      set parent [winfo parent $marker]
      switch $StateUI(marker,$num,orient) \
      {
       "VERT" \
       {
           set parent_x [winfo rootx $parent]
           set x [expr $x-$parent_x]
           set width [winfo reqwidth $parent]
           set x [expr double($x)/$width]
           Scope marker0 $num 0 $x -nowait
       }
       "HORIZ" \
       {
           set parent_y [winfo rooty $parent]
           set y [expr $parent_y-$y]
           set height [winfo reqheight $parent]
           set y [expr (double($y)/($height)+0.5)*$vscale*10]
           set StateUI(marker,$num,mag) $y
       }
       default {return}
      }
  }
  #***************************************************************************************************
  #
  # Name: Click_marker/UnClick_marker
  #
  # Parameters: widget - invoking widget path
  #             num    - marker number
  #             X and Y of pointer
  #
  # Description: activate/deactivate dragging of a marker
  #
  #***************************************************************************************************
  proc Click_marker { marker num x y} \
  {
     variable StateUI
     change_marker $marker $num 1
  }
  
  proc UnClick_marker { marker num x y} \
  {
     variable StateUI
     change_marker $marker $num 0
  }
  #***************************************************************************************************
  #
  # Name: Marker_spin
  #
  # Parameters: num    - marker number
  #             direction
  #
  # Description: Move a marker with the spinner/arrow keys
  #
  #***************************************************************************************************
  proc Marker_spin { num dir} \
  {
     variable StateUI
     set accel 1
     set time [clock clicks]
     set delta [expr abs($StateUI(marker,$num,spintime)-$time)]
     set StateUI(marker,$num,spintime) $time
     if {$delta < $StateUI(marker,accel)} { set accel 20}
     incr StateUI(marker,$num,spin_count) [expr $dir*$accel]
  }

  proc Marker_spin_core { num } \
  {
     variable StateUI
     while {$StateUI(marker,$num,spin_count) != 0} \
     {
         set delta $StateUI(marker,$num,spin_count)
         set StateUI(marker,$num,spin_count) 0
         switch -- $StateUI(marker,$num,orient) \
         {
            "VERT" \
            {
               set x [expr $StateUI(marker,$num,x) + $delta*.0002]
               Scope marker0 $num 0 $x
            }
            "HORIZ" \
            {
               switch $StateUI(marker,$num,state) \
               {
                   "CH1" \
                   {
                     set vscale $StateUI(ch1,scale)
                   }
                   "CH2" \
                   {
                      set vscale $StateUI(ch2,scale)
                   }
                   "ON" \
                   {
                      set vscale $StateUI(baseband,scale)
                   }
                   default \
                   {
                      return
                   }
                }
                puts "scale $vscale"
                set y [expr $StateUI(marker,$num,mag) + $delta*$vscale/50.0]
                puts "y is $y"
                set StateUI(marker,$num,mag) $y
            }
         }
     }
  }
  #***************************************************************************************************
  #
  # Name: Marker_toggle_orient
  #
  # Parameters: num    - marker number
  #
  # Description: change a marker from horizontal to vertical or vis-versa
  #
  #***************************************************************************************************
  proc Marker_toggle_orient { num } \
  {
     variable StateUI
     switch -- $StateUI(marker,$num,orient) \
     {
        "VERT"  { set StateUI(marker,$num,orient) "HORIZ"}
        "HORIZ" { set StateUI(marker,$num,orient) "VERT"}
     }
  }

  foreach num { 1 2 3 4 } \
  {
    bind ScopeMarker$num <B1-Motion> "ScopeUI::Drag_marker %W $num %X %Y"
    bind ScopeMarker$num <1> "ScopeUI::Click_marker %W $num %X %Y"
    bind ScopeMarker$num <ButtonRelease-1> "ScopeUI::UnClick_marker %W $num %X %Y"
    bind ScopeMarker$num <FocusOut>     "ScopeUI::Marker_deactivate %W $num"
    bind ScopeMarker$num <Key-Escape>   "ScopeUI::Marker_deactivate %W $num"
    bind ScopeMarker$num <<edit_up>>    "ScopeUI::Marker_spin $num 1"
    bind ScopeMarker$num <<edit_right>> "ScopeUI::Marker_spin $num 1"
    bind ScopeMarker$num <<edit_down>>  "ScopeUI::Marker_spin $num -1"
    bind ScopeMarker$num <<edit_left>>  "ScopeUI::Marker_spin $num -1"
    bind ScopeMarker$num <Double-1>     "ScopeUI::Marker_toggle_orient $num"
    bind ScopeMarker$num <Button-4> "ScopeUI::Marker_spin $num 1"
    bind ScopeMarker$num <Button-5> "ScopeUI::Marker_spin $num -1"
  }
  
  #***************************************************************************************************
  #
  # Name: position_vert_marker
  #
  # Parameters: widget - invoking widget path
  #             num    - marker number
  #
  # Description: move a vertical marker
  #
  #***************************************************************************************************
  proc position_vert_marker { tile_path num  } \
  {
    set scope $tile_path.scope
    variable StateUI
    set block ${scope}.block$num
    set line  ${scope}.line$num
    set text  ${scope}.text$num
    set block_center [expr [winfo reqwidth $block]/2]
    set block_height [winfo reqheight $block]
    set text_height [winfo reqheight $text]
    set x $StateUI(marker,$num,x)
    set y [expr $block_height+$text_height*($num-1)]
    
    if {$x > 1.0} { set x 1.0}
    if {$x < 0.0} { set x 0.0}

    place $line  -relx $x -x -1  -y 0
    place $block -relx $x -x -$block_center -y 0
    if {$x > .5} \
    {
       set tx [expr -3 - [winfo reqwidth $text]]
       place $text  -relx $x -x $tx -y $y
    } \
    else \
    {
       place $text  -relx $x -x 3 -y $y
    }
    raise $block
    Cursor::clear_relations $scope
  }
  #***************************************************************************************************
  #
  # Name: position_horz_marker
  #
  # Parameters: widget - invoking widget path
  #             num    - marker number
  #
  # Description: move a horizontal marker
  #
  #***************************************************************************************************
  proc position_horz_marker { tile_path num } \
  {
    set scope $tile_path.scope
    variable StateUI
    set block ${scope}.block$num
    set line  ${scope}.line$num
    set text  ${scope}.text$num
    set block_center [expr [winfo reqheight $block]/2]
    set block_width [winfo reqheight $block]
    set text_width [winfo reqwidth $text]
    set y $StateUI(marker,$num,y)
    set x [expr $block_width+$text_width*($num-1)]

    if {$y > 1.0} { set y 1.0}
    if {$y < 0.0} { set y 0.0}

    place $line  -rely $y -y -1 -x 0
    place $block -rely $y -y -$block_center -x 0
    place $text  -rely $y -y 3 -y $y
    if {$y > .5} \
    {
       set ty [expr -3 - [winfo reqheight $text]]
       place $text  -rely $y -y $ty -x $x
    } \
    else \
    {
       place $text  -rely $y -y 3 -x $x
    }
    raise $block
    Cursor::clear_relations $scope
  }

  #***************************************************************************************************
  #
  # Name: change_marker
  #
  # Parameters: widget  - tile path of the widget
  #             num     - marker number
  #
  # Description: Changes the color of the marker to indicate it has been 
  #              clicked on.
  #
  #***************************************************************************************************
  proc change_marker { widget  num  selected} \
  {
    #puts "change_marker:  $widget,  $num"
    
    variable  StateUI

    set tile_path  [get_tile  $widget]
    
    set scope $tile_path.scope
    set block ${scope}.block$num
    set line  ${scope}.line$num
    set text  ${scope}.text$num
    
    if { ![winfo exists $block] } \
    {
      return
    }

    # Change the marker only if it is ON
    if { $selected } \
    {
      # Change the color of the marker to indicate it has been clicked on
      $block config -state active
      $line  config -bg white
      $text  config -fg white
    } \
    else \
    {
      $block config -state normal
      set bg [$block cget -bg]
      $line  config -bg $bg
      $text  config -fg $bg
    }
  }
  #***************************************************************************************************
  #
  # Name: config_marker
  #
  # Parameters: widget  - tile path of the widget
  #
  # Description: Displays the configuration toplevel for the markers.
  #
  #***************************************************************************************************
  proc config_marker { widget } \
  {

    set tile_path  [get_tile  $widget]
    
    set top_level  [winfo toplevel $tile_path]
   
    set tile_path  [name_child  $top_level  scope_marker_config]

    if { [winfo exists $tile_path] } { return }
    
    ::Screens::create_toplevel_tile  $tile_path  SCOPE_MARKER_CONFIG
    grid propagate $tile_path true
    # hook up the vars
    foreach num { 1 2 3 4 } \
    {
       $tile_path.marker$num.name       config -text "Marker $num"
       $tile_path.marker$num.state      config -textvar ::ScopeUI::StateUI(marker,$num,state)
       $tile_path.marker$num.state.menu config -post "ScopeUI::post_marker_state_menu $tile_path.marker$num.state.menu $num"
       $tile_path.marker$num.orient     config -textvar ::ScopeUI::StateUI(marker,$num,orient) \
                                               -command "ScopeUI::Marker_toggle_orient $num"
       $tile_path.marker$num.time       config -textvar ::ScopeUI::StateUI(marker,$num,time,formatted)
       $tile_path.marker$num.time_units config -textvar ::ScopeUI::StateUI(marker,$num,time,units)
       $tile_path.marker$num.mag        config -textvar ::ScopeUI::StateUI(marker,$num,mag,formatted)
       set cmd "ScopeUI::update_marker_orient $tile_path $num"
       OnChange hook ::ScopeUI::StateUI(marker,$num,orient) $cmd
       bind $tile_path.marker$num.orient <Destroy> [list OnChange unhook ::ScopeUI::StateUI(marker,$num,orient) $cmd]
    }
  }
  proc update_marker_orient { tile_path num } \
  {
     variable StateUI
     switch  $::ScopeUI::StateUI(marker,$num,orient) \
     {
        HORIZ  \
        {
           $tile_path.marker$num.time config -state disabled -relief flat -takefocus 0
           $tile_path.marker$num.mag config  -state normal -relief solid -takefocus 1
        }
        VERT \
        {
           $tile_path.marker$num.time config -state normal -relief solid -takefocus 1
           $tile_path.marker$num.mag config  -state disabled -relief flat -takefocus 0
        }
     }
     Cursor::clear_relations $tile_path
  }
  proc set_marker_mag  { num } \
  {
     variable StateUI
     set StateUI(marker,$num,mag) $StateUI(marker,$num,mag,formatted)
  }
  proc set_marker_time { num } \
  {
     variable StateUI
     switch -- $StateUI(marker,$num,time,units) \
     {
        "Sec"  { Scope marker0 $num [expr $StateUI(marker,$num,time,formatted)     ] -nowait}
        "mSec" { Scope marker0 $num [expr $StateUI(marker,$num,time,formatted)*1E-3] -nowait}
        "uSec" { Scope marker0 $num [expr $StateUI(marker,$num,time,formatted)*1E-6] -nowait}
     }
  }
  
  proc post_marker_state_menu { menu num } \
  {
     variable StateUI
     $menu delete 0 end
     set accel 0
     $menu add command -label "OFF" -command "set ScopeUI::StateUI(marker,$num,state) OFF" -accel $accel
     bind $menu <KeyRelease-$accel> "do_menu_key %W $accel"
     incr accel
     if {$StateUI(ch1) != "OFF"} \
     {
        $menu add command -label "CH1" -command "set ScopeUI::StateUI(marker,$num,state) CH1" -accel $accel
        bind $menu <KeyRelease-$accel> "do_menu_key %W $accel"
        incr accel
     }
     if {$StateUI(ch2) != "OFF"} \
     {
        $menu add command -label "CH2" -command "set ScopeUI::StateUI(marker,$num,state) CH2" -accel $accel
        bind $menu <KeyRelease-$accel> "do_menu_key %W $accel"
        incr accel
     }
     if {$StateUI(baseband) != "OFF"} \
     {
        $menu add command -label "ON" -command "set ScopeUI::StateUI(marker,$num,state) ON" -accel $accel
        bind $menu <KeyRelease-$accel> "do_menu_key %W $accel"
        incr accel
     }
  }

 #****************************************************************************************************
 # Name:        getSaveState
 # Parameters:  none
 # Description: This function returns a list of TCL commands which can be used
 #              to set the ScopeUI tiles to its current state
 #****************************************************************************************************
 proc getSaveState { } {
   
   variable StateUI
 
   set returnList \
   {
      namespace eval ::ScopeUI \
      {
         array set StateUI \
         {
            [array get StateUI *,triglev]
            [array get StateUI *,horiz]
            [array get StateUI *,vpos]
            [array get StateUI *,scale]
            [array get StateUI marker,*,orient]
            [array get StateUI marker,*,state]
            [array get StateUI marker,diff*,1]
            [array get StateUI marker,diff*,2]
            [array get StateUI marker,*,mag]
            [array get StateUI persistance]
            [array get StateUI source]
         }
      }
   }
   
       
   return [subst $returnList]
 }

#****************************************************************************************************
# Name:        Sync
# Parameters:  none
# Description: Write state variables to ScopeUI
#****************************************************************************************************
 proc Sync { } {
   eval [getSaveState]
 }
 
   OnChange hook ::Scope::State(scale,1)           "ScopeUI::update_scope_scale 1"
   OnChange hook ::Scope::State(scale,2)           "ScopeUI::update_scope_scale 2"
   OnChange hook ::Scope::State(scale,3)           "ScopeUI::update_scope_scale 3"
   OnChange hook ::Receiver::State(demod_raw)    "ScopeUI::update_demod_units"
   OnChange hook ::BBAudio::State(Route_17)  [namespace code update_bb_input]
   OnChange hook ::BBAudio::State(sinadbal)  [namespace code update_bb_input]
   OnChange hook ::BBAudio::State(micbias)   [namespace code update_bb_input]
   OnChange hook ::Scope::State(hposition,1) "ScopeUI::update_horizpos"
   OnChange hook ::Scope::State(hposition,3) "ScopeUI::update_horizpos"
   OnChange hook ::Scope::State(coupling,1)  [namespace code "update_ch1_coupling"]
   OnChange hook ::Scope::State(coupling,2)  [namespace code "update_ch2_coupling"]
   OnChange hook ::Scope::State(run,3)       [namespace code update_bb_input]
   OnChange hook ::Scope::State(run,2)       [namespace code update_main_input]
   OnChange hook ::Scope::State(run,1)       [namespace code update_main_input]
   OnChange hook ::Scope::State(enable,1)    [namespace code update_main_input]
   OnChange hook ::Scope::State(enable,2)    [namespace code update_main_input]
   OnChange hook ::Scope::State(enable,3)    [namespace code update_bb_input]
   OnChange hook ::Scope::State(triglev,1)   "ScopeUI::update_trig_level 1"
   OnChange hook ::Scope::State(triglev,2)   "ScopeUI::update_trig_level 2"
   OnChange hook ::Scope::State(triglev,3)   "ScopeUI::update_trig_level 3"
   OnChange hook ::Scope::State(trigsource,1) "ScopeUI::update_trig_source"
   OnChange hook ::ScopeUI::StateUI(ch1,vpos)  "ScopeUI::update_trig_level 1"
   OnChange hook ::ScopeUI::StateUI(ch2,vpos) "ScopeUI::update_trig_level 2"
   OnChange hook ::ScopeUI::StateUI(baseband,vpos) "ScopeUI::update_trig_level 3"
         
            
    # hook into the variable
    foreach num { 1 2 3 4 } \
    {
        OnChange hook ::ScopeUI::StateUI(marker,$num,state) "::ScopeUI::setup_marker $num"
        OnChange hook ::ScopeUI::StateUI(marker,$num,orient) "::ScopeUI::setup_marker $num"
        OnChange hook ::ScopeUI::StateUI(marker,$num,time,units) "::ScopeUI::update_marker_time_formatted $num"
        update_marker_time_formatted $num
    }
   foreach marker { 1 2 3 4 } \
   {
      OnChange hook ::ScopeUI::StateUI(marker,$marker,spin_count)  [namespace code "Marker_spin_core $marker"]
   }
}
#End of ScopeUI namespace


#****************************************************************************************************
# This registers the tiles with the list for saving instrument states
#****************************************************************************************************
namespace eval ::SavedStates \
{
   variable tiles   

   lappend tiles ScopeUI
}


