#############################################################################
#                                                                           #
# IFR SYSTEMS, INC    COPYRIGHT (C) 1998,99,2001,2002  DESIGN ENGINEERING   #
# ------------------------------------------------------------------------- #
#                                                                           #
# FILE        : 3625.SzSnet_MonitorUI.tcl                                   #
#                                                                           #
# PRODUCT     : FITS                                                        #
#                                                                           #
# AUTHOR      : P. McCormick                                                #
#                                                                           #
# DESCRIPTION : SmartNet Monitor User Interface                             #
#                                                                           #
#############################################################################

package require SzSnet
package require IFR_options

namespace eval SzSnet_MonitorUI \
{

  # Global variables
  variable  LongName     "monitor"
  variable  ShortName    "SN/SZ Monitor"
  variable  Size         Quarter
            
  variable  tile_path    ""
  variable  tile_class   "MONITOR"

  variable  scroll_on    0

  variable  StateUI
  
  set StateUI(monitor_mode)       "OFF"
  set StateUI(monitor_mode_menu)  { "OFF"  "OSW"  "LSD" }

  set StateUI(chan)               100
  set StateUI(capture)            "CAPTURE OFF"
  set StateUI(data_log)           "LOG DATA OFF"
  set StateUI(emergency)          ""
  
  set StateUI(word,0)           0
  set StateUI(parity,0)         0
  set StateUI(corrections,0)    0
  set StateUI(t_bits_binary,0)  "0000"
  set StateUI(timestamp,0)      0
  set StateUI(dc_ave,0)         0
  set StateUI(standard_dev,0)   0
  set StateUI(symbolrate,0)     0



  #***************************************************************************
  #
  #        Name: Create  
  #
  #  Parameters: 
  #
  # Description: 
  #
  #***************************************************************************
  proc Create { tile_path } \
  {
    #puts "SzSnet_MonitorUI::Create"
    
    variable  LongName
    variable  tile_class


    set top_level [winfo toplevel $tile_path]
   
    set tile_path  [name_child  $top_level  $LongName]

    if { ![winfo exists $tile_path] } \
    {
      set tile_path  [Screens::create_tile  $tile_path  $LongName  $tile_class]
    }  
     

    return $tile_path
  
  }


  #***************************************************************************
  #
  #        Name: Map
  #
  #  Parameters: 
  #
  # Description: 
  #
  #***************************************************************************
  proc Map { tile_path } \
  {
    #puts "SzSnet_MonitorUI::Map:  $tile_path"

    if { ![IFR_options is_installed SzSnet] } \
    {
      grid forget $tile_path

      return
    }
    
    # Call the procedure to check for the SmartNet Monitor option
    if { ![SzSnet_MonitorUI::check_for_option] } \
    {
      grid forget $tile_path
      
      return
    }

    trace variable ::SzSnet_MonitorUI::StateUI(monitor_mode)  w  \
                           "SzSnet_MonitorUI::update_monitor_mode  $tile_path"
                           
    update_monitor_mode_core  $tile_path

    trace variable ::SzSnet::State(timestamp)  w  \
                                  "SzSnet_MonitorUI::Update_array  $tile_path"

    # Trace on the T-BITS and display the emergency field if the value is 
    # binary 010
    trace variable ::SzSnet::State(t_bits)  w  \
                              "SzSnet_MonitorUI::update_emergency  $tile_path"


    ${tile_path}.frame4.lst_read config  \
                           -yscrollcommand "${tile_path}.frame4.scrl_read set"

    ${tile_path}.frame4.scrl_read config  \
                                 -command "${tile_path}.frame4.lst_read yview"
    

    return
  }


  #***************************************************************************
  #
  #        Name: Cleanup
  #
  #  Parameters: 
  #
  # Description: 
  #
  #***************************************************************************
  proc Cleanup { tile_path } \
  {
    #puts "SzSnet_MonitorUI::Cleanup:  $tile_path"
    

    trace vdelete ::SzSnet_MonitorUI::StateUI(monitor_mode)  w  \
                            "SzSnet_MonitorUI::update_monitor_mode  $tile_path"

    trace vdelete ::SzSnet::State(timestamp)  w  \
                                   "SzSnet_MonitorUI::Update_array  $tile_path"

    trace vdelete ::SzSnet::State(t_bits)  w  \
                               "SzSnet_MonitorUI::update_emergency  $tile_path"

    return
  }
  

  #***************************************************************************
  #
  #        Name: Unmap
  #
  #  Parameters: 
  #
  # Description: 
  #
  #***************************************************************************
  proc Unmap { tile_path } \
  {
    #puts "SzSnet_MonitorUI::Unmap"
    
    Cleanup  $tile_path
    
    return
  }


  #***************************************************************************
  #
  #        Name: Destroy
  #
  #  Parameters: 
  #
  # Description: 
  #
  #***************************************************************************
  proc Destroy { tile_path } \
  {
    #puts "SzSnet_MonitorUI::Destroy"
    
    Cleanup  $tile_path
    
    return
  }


  #***************************************************************************
  #
  #        Name: Register
  #
  #  Parameters: 
  #
  # Description: 
  #
  #***************************************************************************
  proc Register { } \
  {
    #puts "SzSnet_MonitorUI::Register"

    return
  }
  

  #***************************************************************************
  #
  #        Name: Update_array
  #
  #  Parameters: 
  #
  # Description: 
  #
  #***************************************************************************
  proc Update_array { tile_path  args } \
  {
    #puts "SzSnet_MonitorUI::Update_array"

    variable  StateUI
    
    if { $StateUI(monitor_mode) == "OFF" } \
    { 
      return
    }

    # Set local variables
    set tile_path  [get_tile  $tile_path]

    if { $StateUI(capture) == "CAPTURE ON" } \
    {
      set word  $::SzSnet::State(mon_word)
    
      set index  [${tile_path}.frame4.lst_read  index  end]
    
      #puts "index:  $index"

      ${tile_path}.frame4.lst_read  insert  end  "$index:  $word"
    
      ${tile_path}.frame4.lst_read  see  end

      set StateUI(word,$index)           $::SzSnet::State(mon_word)

      set StateUI(parity,$index)         $::SzSnet::State(mon_parity)
      set StateUI(corrections,$index)    $::SzSnet::State(mon_corrections)
      set StateUI(t_bits_binary,$index)  $::SzSnet::State(t_bits_binary)
      set StateUI(timestamp,$index)      $::SzSnet::State(mon_timestamp)
      set StateUI(dc_ave,$index)         $::SzSnet::State(mon_dc_ave)
      set StateUI(standard_dev,$index)   $::SzSnet::State(mon_standard_dev)
      set StateUI(symbolrate,$index)     $::SzSnet::State(mon_symbolrate)
    }

    if { $StateUI(data_log) == "LOG DATA ON" } \
    {
      # Log the data to a file
    }
    
    return
  }


  #***************************************************************************
  #
  #        Name: get_tile
  #
  #  Parameters: any child widget of the tile
  #
  # Description: Get the name of the tile's parent frame
  #
  #***************************************************************************
  proc get_tile { widget } \
  {
    while { 1 } \
    {
      switch -- [winfo class $widget] \
      {
        "MONITOR"   { return $widget }
      }
        
      set widget [winfo parent $widget]
    }
  }


  #***************************************************************************
  #
  #        Name: set_monitor_mode
  #
  #  Parameters: new_value -
  #
  # Description: Sets the Monitor mode
  #
  #***************************************************************************
  proc set_monitor_mode { } \
  {
    #puts "set_monitor_mode"
    
    variable  StateUI
    
    SzSnet  rcvosw  0
    SzSnet  rcvisw  0
    SzSnet  rcvlsd  0
    
    switch -- $StateUI(monitor_mode) \
    {
      "OFF"  { }

      "OSW"  { after idle "SzSnet  rcvosw  1" }
               
      "ISW"  { }
               
      "LSD"  { after idle "SzSnet  rcvlsd  1" }
    }
    
    return
  }


  #***************************************************************************
  #
  #        Name: update_monitor_mode
  #
  #  Parameters: tile_path - tile path of the invoking widget
  #
  # Description: Updates the display to the Monitor mode
  #
  #***************************************************************************
  proc update_monitor_mode { tile_path  args } \
  {
     set cmd [namespace code "update_monitor_mode_core $tile_path"]
     after cancel $cmd
     after idle $cmd
  }
  proc update_monitor_mode_core { tile_path } \
  {
    #puts "update_monitor_mode:  $tile_path"
    
    variable  StateUI

    if { $StateUI(monitor_mode) == "" } \
    {
      return
    }
    
    set tile_path  [get_tile  $tile_path]
    
    # Turn off the capture
    set StateUI(capture)  "CAPTURE OFF"
    
    # Zero out all fields,
    # when the mode is changed
    set ::SzSnet::State(mon_word)          0
    set ::SzSnet::State(mon_parity)        0
    set ::SzSnet::State(mon_corrections)   0
    set ::SzSnet::State(t_bits_binary)     "0000"
    set ::SzSnet::State(mon_timestamp)     0
    set ::SzSnet::State(mon_dc_ave)        0
    set ::SzSnet::State(mon_standard_dev)  0
    set ::SzSnet::State(mon_symbolrate)    0

    ${tile_path}.frame5.btn_capture  config  -state normal
 
    switch -- $StateUI(monitor_mode) \
    {
      "OFF"  { ${tile_path}.frame5.btn_capture  config  -state disabled }
      
      "OSW"  { # Track only the OSW data
               set ::SzSnet::State(monitor_type)  2 }
    
      "ISW"  { # Track only the ISW data
               set ::SzSnet::State(monitor_type)  1 }

      "LSD"  { # Track only the LSD data
               set ::SzSnet::State(monitor_type)  3 }
    }

    return
  }


  #***************************************************************************
  #
  #        Name: set_chan
  #
  #  Parameters: new_value -
  #
  # Description: Sets Monitor channel
  #
  #***************************************************************************
  proc set_chan { { new_value "" } } \
  {
    #puts "set_chan"
    
    catch { SzSnet control $::SzSnet::State(control_chan) -nowait } result
    
    return
  }


  #***************************************************************************
  #
  #        Name: set_selection
  #
  #  Parameters: new_value -
  #
  # Description: Sets selection in the list box
  #
  #***************************************************************************
  proc set_selection { tile_path { new_value "" } } \
  {
    #puts "set_selection"
    
    variable  StateUI

    set tile_path  [get_tile  $tile_path]

    set index  [${tile_path}.frame4.lst_read  curselection]
    
    if { $index == "" } \
    {
      return
    }

    #puts "index:  $index"
    
    set ::SzSnet::State(mon_word)          $StateUI(word,$index)
    set ::SzSnet::State(mon_parity)        $StateUI(parity,$index)
    set ::SzSnet::State(mon_corrections)   $StateUI(corrections,$index)
    set ::SzSnet::State(t_bits_binary)     $StateUI(t_bits_binary,$index)
    set ::SzSnet::State(mon_timestamp)     $StateUI(timestamp,$index)
    set ::SzSnet::State(mon_dc_ave)        $StateUI(dc_ave,$index)
    set ::SzSnet::State(mon_standard_dev)  $StateUI(standard_dev,$index)
    set ::SzSnet::State(mon_symbolrate)    $StateUI(symbolrate,$index)
  }


  #***************************************************************************
  #
  #        Name: toggle_capture
  #
  #  Parameters:
  #
  # Description: Sets the capture parameters
  #
  #***************************************************************************
  proc toggle_capture { } \
  {
    #puts "toggle_capture:"
    
    variable  StateUI

    if { $StateUI(capture) == "CAPTURE OFF" } \
    {
      set StateUI(capture)  "CAPTURE ON"
    } \
    else \
    {
      set StateUI(capture)  "CAPTURE OFF"
    }    
    
    return
  }
    

  #***************************************************************************
  #
  #        Name: toggle_data_log
  #
  #  Parameters: new_value -
  #
  # Description: sets the data logging
  #
  #***************************************************************************
  proc toggle_data_log { tile_path { new_value "" } } \
  {
    #puts "toggle_data_log"

    variable  StateUI
    
    if { $StateUI(data_log) == "LOG DATA OFF" } \
    {
      set StateUI(data_log)  "LOG DATA ON"
      
      # Open the file for data logging
    } \
    else \
    {
      set StateUI(data_log)  "LOG DATA OFF"

      # Close the file for data logging
    }
    
  }


  #***************************************************************************
  #
  #        Name: update_emergency
  #
  #  Parameters: new_value -
  #
  # Description: Update the display to T BITS value
  #
  #***************************************************************************
  proc update_emergency { tile_path  args } \
  {
    #puts "update_emergency"
    
    variable  StateUI
    
    set tile_path  [get_tile  $tile_path]

    set temp  [expr $::SzSnet::State(t_bits) & 0x7]
    
    if { $temp == 2 } \
    {
      set StateUI(emergency)  "EMERG"
      
      ${tile_path}.frame3.lbl_emergency  config  -bg red
    } \
    else \
    {
      set StateUI(emergency)  ""
      
      ${tile_path}.frame3.lbl_emergency  config  -bg gray
    }

    return
  }


  #***************************************************************************
  #
  #        Name: clear_textbox
  #
  #  Parameters: tile_path - tile path of the invoking widget
  #
  # Description: Clears the text box
  #
  #***************************************************************************
  proc clear_textbox { tile_path } \
  {
    #puts "clear_textbox"
    
    variable  StateUI
    
    set tile_path  [get_tile  $tile_path]
    
    # Clear the textbox
    ${tile_path}.frame4.lst_read  delete  0  end
        
    return
  }


  #***************************************************************************
  #
  #        Name: toggle_scroll_on
  #
  #  Parameters: tile_path - tile path of the invoking widget
  #
  # Description: Toggles the 'scroll_on' variable to allow the spinner
  #              knob to control the scrollbar
  #
  #***************************************************************************
  proc toggle_scroll_on { tile_path } \
  {
    #puts "toggle_scroll_on:  $tile_path"
    
    variable  scroll_on
    
    set tile_path  [get_tile  $tile_path]
    
    set scrollbar_path  ${tile_path}.frame4.scrl_read
    
    if { $scroll_on == 0 } \
    {
      # Change the color of the scrollbar to indicate it is active
      $scrollbar_path config  -bg white
      
      set scroll_on  1
      
      # For each event, call the procedure to move the scrollbar,
      # then break to keep the focus from moving in response to the 
      # event
      # Bind the <Right> event  (includes <<Spin_up>>)
      bind $scrollbar_path  <Right>  "::SzSnet_MonitorUI::move_listbox  $tile_path  UP; break"
                                   
      # Bind the <Left> event  (includes <<Spin_down>>)
      bind $scrollbar_path  <Left>  "::SzSnet_MonitorUI::move_listbox  $tile_path  DOWN; break"
                                   
      bind $scrollbar_path  <Up>  "::SzSnet_MonitorUI::move_listbox  $tile_path  UP; break"
                                   
      bind $scrollbar_path  <Down>  "::SzSnet_MonitorUI::move_listbox  $tile_path  DOWN; break"
    } \
    else \
    {
      # Change the color of the scrollbar to indicate it is NOT active
      $scrollbar_path config  -bg gray

      set scroll_on  0

      # Unbind the <Right> event  (includes <<Spin_up>>)
      bind $scrollbar_path  <Right>    ""

      # Unbind the <Left> event  (includes <<Spin_down>>)
      bind $scrollbar_path  <Left>  ""
      
      # For the 'Up' of 'Down' event, generate the 'cursor_up' or 'cursor_down'
      # event, then break to keep the scrollbar from moving in response to 
      # either event
      bind $scrollbar_path  <Up>    "event generate %W <<cursor_up>>; break"
      bind $scrollbar_path  <Down>  "event generate %W <<cursor_down>>; break"
    }
    
    return
  }
      

  #***************************************************************************
  #
  #        Name: move_listbox
  #
  #  Parameters: tile_path - tile path of the invoking widget
  #              direction - direction to move the scrollbar
  #
  # Description: Moves the listbox display for changes on the spinner knob 
  #              or Up and Down arrows.
  #
  #***************************************************************************
  proc move_listbox { tile_path  direction } \
  {
    #puts "move_listbox:  $tile_path,  $direction"
    
    variable  scroll_on
    
    if { $scroll_on == 0 } \
    {
      return
    }

    set tile_path  [get_tile  $tile_path]
    
    set listbox_path  ${tile_path}.frame4.lst_read

    # Move the canvas display
    switch -- $direction \
    {
      "UP"    { $listbox_path  yview  scroll  -1  unit }
    
      "DOWN"  { $listbox_path  yview  scroll  1  unit }
    }
    
    return
  }


  #***************************************************************************
  #
  #        Name: check_for_option
  #
  #  Parameters: 
  #
  # Description: Checks for the SmartNet Monitor option
  #
  #***************************************************************************
  proc check_for_option { } \
  {
    return [IFR_options is_installed Smartnet_Monitor]

  }
  
  
};  # End of SzSnet_MonitorUI namespace


