#############################################################################
#                                                                           #
# IFR SYSTEMS, INC    COPYRIGHT (C) 1998,99,2001,2002  DESIGN ENGINEERING   #
# ------------------------------------------------------------------------- #
#                                                                           #
# FILE        : 3660.SzSnet_Repeat_SimUI.tcl                                #
#                                                                           #
# PRODUCT     : FITS                                                        #
#                                                                           #
# AUTHOR      : P. McCormick                                                #
#                                                                           #
# DESCRIPTION : SmartNet Repeater Simulator User Interface                  #
#                                                                           #
#############################################################################


package require SzSnet
package require P25
package require IFR_options

namespace eval SzSnet_Repeat_SimUI \
{
  # Global variables
  variable  LongName      "repeat_sim"
  variable  ShortName     "SN/SZ Repeater Sim"
  variable  Size          Quarter
  
            
  variable  tile_class    "REPEAT_SIM"

  variable  last_focus
  
  variable  StateUI
  
  set StateUI(num_format)  "HEX"
  
  set StateUI(NAC_dec)         0
  set StateUI(NAC_hex)         000

  set StateUI(MFID_dec)        0
  set StateUI(MFID_hex)        00
  
  set StateUI(update_nac)      1
  set StateUI(update_mfid)     1
  
  set StateUI(xmit_dev)        3200
  set StateUI(xmit_dev_desc)   "WIDE"
  
  namespace export *
  

  #***************************************************************************
  #
  #        Name: Create  
  #
  #  Parameters:
  #
  # Description: 
  #
  #***************************************************************************
  proc Create { tile_path } \
  {
    #puts "SzSnet_Repeat_SimUI::Create:  $tile_path"
    
    variable  LongName
    variable  tile_class
    
    set top_level  [winfo toplevel $tile_path]
   
    set tile_path  [name_child  $top_level  $LongName]

    #puts "tile_path:  $tile_path"

    if { ![winfo exists $tile_path] } \
    {
      set tile_path  [Screens::create_tile  $tile_path  $LongName  $tile_class]
    }  
     
    # Create the Transmit Deviation menu as a list of command buttons
    # to set the value to the default or to escape the menu
    ${tile_path}.frame4.mnu_xmit_dev.menu  delete  0  end
    
    ${tile_path}.frame4.mnu_xmit_dev.menu  add  command  \
                              -label  "DEFAULT"          \
                              -command  SzSnet_Repeat_SimUI::set_xmit_dev_def
    
    ${tile_path}.frame4.mnu_xmit_dev.menu  add  command  -label  "Escape"

    return $tile_path
  }


  #***************************************************************************
  #
  #        Name: Map
  #
  #  Parameters: 
  #
  # Description: Initializes the tile
  #
  #***************************************************************************
  proc Map { tile_path } \
  {
    #puts "SzSnet_Repeat_SimUI::Map: $tile_path"

    if { ![IFR_options is_installed SzSnet] } \
    {
      grid forget $tile_path

      return
    }
    
    # Set local variables
    set tile_path  [get_tile  $tile_path]    
    
    if { ![winfo exists $tile_path] } \
    {
      return
    }

    # Make a call to the SzSnet namespace to load the namespace
    SzSnet::Init
    
    # Set the AF Counter for fast readings
    AFCounter avg2  1 -nowait
    
    
    # Synchronize the configured channel and frequency values
    set_channels

    trace variable  ::SzSnet::State(uutptt)  w  \
                             "SzSnet_Repeat_SimUI::update_ptt_rec  $tile_path"

    trace variable  ::SzSnet::State(uutaff)  w  \
                           "SzSnet_Repeat_SimUI::update_affil_rec  $tile_path"

    trace variable  ::SzSnet::State(affil)  w  \
                               "SzSnet_Repeat_SimUI::update_affil  $tile_path"

    trace variable  ::SzSnet::State(call_type)  w  \
                           "SzSnet_Repeat_SimUI::update_call_type  $tile_path"

    trace variable  ::SzSnet::State(sim_on_off)  w  \
                          "SzSnet_Repeat_SimUI::update_sim_on_off  $tile_path"

    trace variable  ::SzSnet::State(ptt_on_off)  w  \
                          "SzSnet_Repeat_SimUI::update_ptt_on_off  $tile_path"
                           
    trace variable  ::SzSnet::State(grp_id)  w  \
                         "SzSnet_Repeat_SimUI::update_tgid_encode  $tile_path"
    

    trace variable  ::SzSnet_Repeat_SimUI::StateUI(num_format)  w  \
                          "SzSnet_Repeat_SimUI::update_num_format  $tile_path"

    trace variable  ::SzSnet_Repeat_SimUI::StateUI(xmit_dev)  w  \
                            "SzSnet_Repeat_SimUI::update_xmit_dev  $tile_path"

    trace variable  ::SzSnet::State(band)  w  \
                          "SzSnet_Repeat_SimUI::update_band  $tile_path"


    update_sim_on_off_core  $tile_path

    update_band_core        $tile_path

    update_affil_core       $tile_path

    update_call_type_core   $tile_path
    
    update_num_format_core  $tile_path
    
    # Set the Transmit Deviation on the mapping
    set_xmit_dev
        
    update_xmit_dev_core    $tile_path


    # Determine if the 900 MHz option is installed, 
    # and set the menu options accordingly
    if { [IFR_options  is_installed  SzSnet_900] } \
    {
      # No action
      # The menu option is on the list
    } \
    else \
    {
      # Remove the option from the menu
      oldCatch { ${tile_path}.frame2.mnu_band.menu  delete  "900*" } result
    }

    # Determine if the 800 MHz Reband option is installed, 
    # and set the menu options accordingly
    if { [IFR_options  is_installed  SzSnet_Reband] } \
    {
      # No action
      # The menu option is on the list
    } \
    else \
    {
      # Remove the option from the menu
      oldCatch { ${tile_path}.frame2.mnu_band.menu  delete  "*REBND" } result
    }


    return
  }

  #***************************************************************************
  #
  #        Name: Cleanup
  #
  #  Parameters: 
  #
  # Description: 
  #
  #***************************************************************************
  proc Cleanup { tile_path } \
  {
    #puts "SzSnet_Repeat_SimUI::Cleanup:  $tile_path"
    
    # Set local variables
    set tile_path  [get_tile  $tile_path]    
    

    trace vdelete  ::SzSnet::State(uutptt)  w  \
                             "SzSnet_Repeat_SimUI::update_ptt_rec  $tile_path"

    trace vdelete  ::SzSnet::State(uutaff)  w  \
                           "SzSnet_Repeat_SimUI::update_affil_rec  $tile_path"

    trace vdelete  ::SzSnet::State(affil)  w  \
                               "SzSnet_Repeat_SimUI::update_affil  $tile_path"

    trace vdelete  ::SzSnet::State(call_type)  w  \
                           "SzSnet_Repeat_SimUI::update_call_type  $tile_path"

    trace vdelete  ::SzSnet::State(sim_on_off)  w  \
                          "SzSnet_Repeat_SimUI::update_sim_on_off  $tile_path"

    trace vdelete  ::SzSnet::State(ptt_on_off)  w  \
                          "SzSnet_Repeat_SimUI::update_ptt_on_off  $tile_path"
                           
    trace vdelete  ::SzSnet::State(grp_id)  w  \
                         "SzSnet_Repeat_SimUI::update_tgid_encode  $tile_path"
    

    trace vdelete  ::SzSnet_Repeat_SimUI::StateUI(num_format)  w  \
                          "SzSnet_Repeat_SimUI::update_num_format  $tile_path"

    trace vdelete  ::SzSnet_Repeat_SimUI::StateUI(xmit_dev)  w  \
                            "SzSnet_Repeat_SimUI::update_xmit_dev  $tile_path"

    trace vdelete  ::SzSnet::State(band)  w  \
                          "SzSnet_Repeat_SimUI::update_band  $tile_path"

      
    ## Restore the previous Receiver squelch mode
    #after idle "Receiver squelchmode  AUTO"

    return
  }


  #***************************************************************************
  #
  #        Name: Unmap
  #
  #  Parameters: 
  #
  # Description: 
  #
  #***************************************************************************
  proc Unmap { tile_path } \
  {
    #puts "SzSnet_Repeat_SimUI::Unmap:  $tile_path"
    
    # Set local variables
    set tile_path  [get_tile  $tile_path]    
    
    Cleanup  $tile_path
    
    return
  }


  #***************************************************************************
  #
  #        Name: Destroy
  #
  #  Parameters: 
  #
  # Description: 
  #
  #***************************************************************************
  proc Destroy { tile_path } \
  {
    #puts "SzSnet_Repeat_SimUI::Destroy:  $tile_path"
    
    # Set local variables
    set tile_path  [get_tile  $tile_path]    

    Cleanup  $tile_path

    return
  }


  #***************************************************************************
  #
  #        Name: Register
  #
  #  Parameters: 
  #
  # Description: 
  #
  #***************************************************************************
  proc Register { } \
  {
    #puts "SzSnet_Repeat_SimUI::Register"

    # Set local variables
    set tile_path  [get_tile  $tile_path]    

    return
  }
  

  #***************************************************************************
  #
  #        Name: get_tile
  #
  #  Parameters: any child widget of the tile
  #
  # Description: Get the name of the tile's parent frame
  #
  #***************************************************************************
  proc get_tile { widget } \
  {
    while { 1 } \
    {
      switch -- [winfo class $widget] \
      {
        "REPEAT_SIM"   { return $widget }
      }
        
      set widget [winfo parent $widget]
    }

    return     
  }


  #***************************************************************************
  #
  #        Name: update_sim_on_off
  #
  #  Parameters:
  #
  # Description: Sets the indicator on the user interface to reflect changes
  #              in the Simulator ON / OFF state
  #
  #***************************************************************************
  proc update_sim_on_off { tile_path  args } \
  {
     set cmd [namespace code "update_sim_on_off_core $tile_path"]
     after cancel $cmd
     after idle $cmd
  }
  proc update_sim_on_off_core { tile_path } \
  {
    #puts "update_sim_on_off:  $tile_path"
    
    set tile_path  [get_tile  $tile_path]
    
    if { ![winfo exists $tile_path] } \
    {
      return
    }

    #puts "::SzSnet::State(sim_on_off):  $::SzSnet::State(sim_on_off)"

    switch -- $::SzSnet::State(sim_on_off) \
    {
      0  { ${tile_path}.frame2.btn_ptt_on_off config -state disabled }
      
      1  { ${tile_path}.frame2.btn_ptt_on_off config -state normal   }
    }
    
    return
  }


  #***************************************************************************
  #
  #        Name: update_ptt_on_off
  #
  #  Parameters:
  #
  # Description: Sends the Downlink data when the PTT is set to ON
  #
  #***************************************************************************
  proc update_ptt_on_off { tile_path  args } \
  {
     set cmd [namespace code "update_ptt_on_off_core $tile_path"]
     after cancel $cmd
     after idle $cmd
  }
  proc update_ptt_on_off_core { tile_path  args } \
  {
    #puts "update_ptt_on_off:  $tile_path"
    
    variable  StateUI
    
    set tile_path  [get_tile  $tile_path]
    
    if { $::SzSnet::State(unit_id_hex) == "" } \
    {
      set ::SzSnet::State(unit_id_hex)  0
    }

    if { $::SzSnet::State(grp_id_hex) == "" } \
    {
      set ::SzSnet::State(grp_id_hex)  0
    }
    
    
    switch -- $::SzSnet::State(ptt_on_off) \
    {
      0  { }
      
      1  { if { $::P25ns::EncodeParm(HDR_SID) != $::SzSnet::State(unit_id_hex) } \
           {
             after idle "P25 encode field sid  0x$::SzSnet::State(unit_id_hex) -nowait"
           }
           
           if { $::P25ns::EncodeParm(HDR_TGID) != $::SzSnet::State(grp_id_hex) } \
           {
             after idle "P25 encode field tgid  0x$::SzSnet::State(grp_id_hex) -nowait"
           }
         }
    }
    
    return
  }


  #***************************************************************************
  #
  #        Name: update_ptt_rec
  #
  #  Parameters:
  #
  # Description: Sets the indicator on the user interface to reflect changes
  #              in the Radio Unit initiated PTT 
  #
  #***************************************************************************
  proc update_ptt_rec { tile_path  args } \
  {
     set cmd [namespace code "update_ptt_rec_core $tile_path"]
     after cancel $cmd
     after idle $cmd
  }
  proc update_ptt_rec_core { tile_path } \
  {
    #puts "update_ptt_rec:  $tile_path,  $args"
    
    variable  StateUI
    
    set tile_path  [get_tile  $tile_path]
    
    if { ![winfo exists $tile_path] } \
    {
      return
    }

    if { $::SzSnet::State(unit_id_hex) == "" } \
    {
      set ::SzSnet::State(unit_id_hex)  0
    }

    if { $::SzSnet::State(grp_id_hex) == "" } \
    {
      set ::SzSnet::State(grp_id_hex)  0
    }


    switch -- $::SzSnet::State(uutptt) \
    {
      0   { if { $::SzSnet::State(sim_on_off) == 1 } \
            {
              ${tile_path}.frame2.btn_ptt_on_off config -state normal
            }
           
            ${tile_path}.frame3.lbl_ptt_rec config -fg gray60 -bg gray
          }

      1   { ${tile_path}.frame2.btn_ptt_on_off config -state disabled
           
            ${tile_path}.frame3.lbl_ptt_rec config -fg black -bg green
           
            if { $::SzSnet::State(call_type) == 0 } \
            {
              # Analog call
            } \
            elseif { $::SzSnet::State(call_type) == 1 } \
            {
              # Digital call
              if { $::P25ns::EncodeParm(HDR_SID) != $::SzSnet::State(unit_id_hex) } \
              {
                after idle "P25 encode field sid  0x$::SzSnet::State(unit_id_hex) -nowait"
              }
              
              if { $::P25ns::EncodeParm(HDR_TGID) != $::SzSnet::State(grp_id_hex) } \
              {
                after idle "P25 encode field tgid  0x$::SzSnet::State(grp_id_hex) -nowait"
              }
            }
          }

    }
    
    return
  }
  

  #***************************************************************************
  #
  #        Name: update_affil_rec
  #
  #  Parameters:
  #
  # Description: Sets the indicator on the user interface to reflect changes
  #              in the Radio Unit initiated Affiliation 
  #
  #***************************************************************************
  proc update_affil_rec { tile_path  args } \
  {
     set cmd [namespace code "update_affil_rec_core $tile_path"]
     after cancel $cmd
     after idle $cmd
  }
  proc update_affil_rec_core { tile_path  } \
  {
    #puts "update_affil_rec:  $tile_path"
    
    variable  StateUI
    
    set tile_path  [get_tile  $tile_path]
    
    if { ![winfo exists $tile_path] } \
    {
      return
    }

    if { $::SzSnet::State(unit_id_hex) == "" } \
    {
      set ::SzSnet::State(unit_id_hex)  0
    }

    if { $::SzSnet::State(grp_id_hex) == "" } \
    {
      set ::SzSnet::State(grp_id_hex)  0
    }


    switch -- $::SzSnet::State(uutaff) \
    {
      0  { ${tile_path}.frame3.lbl_affil_rec config -fg gray60 -bg gray }
    
      1  { ${tile_path}.frame3.lbl_affil_rec config -fg black -bg green
      
           if { $::SzSnet::State(call_type) == 0 } \
           {
             # Analog call
           } \
           elseif { $::SzSnet::State(call_type) == 1 } \
           {
             # Digital call
             if { $::P25ns::EncodeParm(HDR_SID) != $::SzSnet::State(unit_id_hex) } \
             {
               after idle "P25 encode field sid  0x$::SzSnet::State(unit_id_hex) -nowait"
             }
             
             if { $::P25ns::EncodeParm(HDR_TGID) != $::SzSnet::State(grp_id_hex) } \
             {
               after idle "P25 encode field tgid  0x$::SzSnet::State(grp_id_hex) -nowait"
             }
           }
         }

    }
    
    return
  }


  #***************************************************************************
  #
  #        Name: update_affil
  #
  #  Parameters:
  #
  # Description: Sets the indicator on the user interface to reflect changes
  #              in the Affiliation state
  #
  #***************************************************************************
  proc update_affil { tile_path  args } \
  {
     set cmd [namespace code "update_affil_core $tile_path"]
     after cancel $cmd
     after idle $cmd
  }
  proc update_affil_core { tile_path } \
  {
    #puts "update_affil:  $tile_path"
    
    set tile_path  [get_tile  $tile_path]
    
    if { ![winfo exists $tile_path] } \
    {
      return
    }

    switch -- $::SzSnet::State(affil) \
    {
      0  { ${tile_path}.frame4.btn_affil config -bg gray -activebackground gray   }
    
      1  { ${tile_path}.frame4.btn_affil config -bg green -activebackground green }
    }
    
    return
  }


  #***************************************************************************
  #
  #        Name: set_channels
  #
  #  Parameters:
  #
  # Description: Sets the Control Channel and Voice Channel Transmit 
  #              channel numbers
  #
  #***************************************************************************
  proc set_channels { } \
  {
    #puts "SzSnet_Repeat_SimUI::set_channels"
    
    switch -- $::SzSnet::State(band_text) \
    {
      "800"        -
      "800 REBND"  -
      "800 SPLNT"  { # Set control channel and voice channel
                     SzSnet control  $::SzSnet::State(cctx_chan,800) -nowait
                     SzSnet voice    $::SzSnet::State(vctx_chan,800) -nowait
                   }

      "VHF/UHF"    { # Set control channels and voice channels
                     SzSnet uhfcctx  $::SzSnet::State(cctx_chan,UV) -nowait
                     SzSnet uhfccrx  $::SzSnet::State(ccrx_chan,UV) -nowait
                     SzSnet uhfvctx  $::SzSnet::State(vctx_chan,UV) -nowait
                     SzSnet uhfvcrx  $::SzSnet::State(vcrx_chan,UV) -nowait
                   }

      "900"        { # Set control channel and voice channel
                     SzSnet control  $::SzSnet::State(cctx_chan,900) -nowait
                     SzSnet voice    $::SzSnet::State(vctx_chan,900) -nowait
                   }

      "USER"       { # Set control channels and voice channels
                     SzSnet control  $::SzSnet::State(cctx_chan,USER) -nowait
                     SzSnet voice    $::SzSnet::State(vctx_chan,USER) -nowait
                     
                     # Set the TX and RX system frequencies
                     # after a delay to allow the above calls to take affect
                     after 500
                     
                     ::SzSnet::set_user_freq
                   }
      default  { }
    }
 
 
    if { $::SzSnet::State(sim_on_off) == 1 } \
    {
      after idle "::SzSnet::cycleSimOnOff"
    }
 
    return
  }



  #***************************************************************************
  #
  #        Name: update_call_type
  #
  #  Parameters:
  #
  # Description: Updates the display to changes in the call type
  #
  #***************************************************************************
  proc update_call_type { tile_path  args } \
  {
     set cmd [namespace code "update_call_type_core $tile_path"]
     after cancel $cmd
     after idle $cmd
  }
  proc update_call_type_core { tile_path } \
  {
    #puts "update_call_type:  $tile_path"
    
    variable  StateUI
    
    set tile_path  [get_tile  $tile_path]
    
    if { ![winfo exists $tile_path] } \
    {
      return
    }

    grid remove ${tile_path}.frame4.lbl_conn_tone
    grid remove ${tile_path}.frame4.mnu_conn_tone
    grid remove ${tile_path}.frame4.ent_conn_tone

    grid remove ${tile_path}.frame4.mnu_xmit_dev
    grid remove ${tile_path}.frame4.ent_xmit_dev
    grid remove ${tile_path}.frame4.lbl_xmit_dev_units
    grid remove ${tile_path}.frame4.lbl_xmit_dev_desc

    grid remove ${tile_path}.frame4.lbl_nac
    grid remove ${tile_path}.frame4.lbl_nac_val_hex
    grid remove ${tile_path}.frame4.lbl_nac_val_dec

    grid remove ${tile_path}.frame4.lbl_mfid
    grid remove ${tile_path}.frame4.lbl_mfid_val_hex
    grid remove ${tile_path}.frame4.lbl_mfid_val_dec

    grid remove ${tile_path}.frame4.btn_set_downlink

    switch -- $::SzSnet::State(call_type) \
    {
      0  { # Analog call type
           Screens::position_widget ${tile_path}.frame4.lbl_conn_tone
           Screens::position_widget ${tile_path}.frame4.mnu_conn_tone
           Screens::position_widget ${tile_path}.frame4.ent_conn_tone
          
           Screens::position_widget ${tile_path}.frame4.mnu_xmit_dev
           Screens::position_widget ${tile_path}.frame4.ent_xmit_dev
           Screens::position_widget ${tile_path}.frame4.lbl_xmit_dev_units
           Screens::position_widget ${tile_path}.frame4.lbl_xmit_dev_desc
          
           after idle "P25 decode enable 0 -nowait"
           
           after idle "P25 decode run 0 -nowait"
         }
    
      1  { # Digital call type
           Screens::position_widget ${tile_path}.frame4.lbl_nac
           Screens::position_widget ${tile_path}.frame4.lbl_mfid
          
           switch -- $StateUI(num_format) \
           {
             "HEX"  { Screens::position_widget ${tile_path}.frame4.lbl_nac_val_hex
                      Screens::position_widget ${tile_path}.frame4.lbl_mfid_val_hex }
                      
             "DEC"  { Screens::position_widget ${tile_path}.frame4.lbl_nac_val_dec
                      Screens::position_widget ${tile_path}.frame4.lbl_mfid_val_dec }
           }
          
           Screens::position_widget ${tile_path}.frame4.btn_set_downlink
          
           # Enable P25 decode data
           after idle "P25 decode enable 3 -nowait"
           
           # Start decode P25 data
           after idle "P25 decode run on -nowait" 
         }
    }
    
    return
  }


  #***************************************************************************
  #
  #        Name: update_band
  #
  #  Parameters:
  #
  # Description: Updates the display to the new OSW band
  #
  #***************************************************************************
  proc update_band { tile_path  args } \
  {
     set cmd [namespace code "update_band_core $tile_path"]
     after cancel $cmd
     after idle $cmd
  }
  proc update_band_core { tile_path } \
  {
    #puts "update_band:  $tile_path"
    
    variable  cctx_label
    variable  ccrx_label
    variable  vctx_label
    variable  vcrx_label

    #puts "::SzSnet::State(band_text):  $::SzSnet::State(band_text)"

    set tile_path  [get_tile  $tile_path]

    if { ![winfo exists $tile_path] } \
    {
      return
    }

    grid remove ${tile_path}.frame2.btn_band_config

    grid remove ${tile_path}.frame3.ent_cctx_800
    grid remove ${tile_path}.frame3.ent_cctx_freq_800

    grid remove ${tile_path}.frame3.ent_cctx_uv
    grid remove ${tile_path}.frame3.ent_cctx_freq_uv

    grid remove ${tile_path}.frame3.ent_cctx_900
    grid remove ${tile_path}.frame3.ent_cctx_freq_900

    grid remove ${tile_path}.frame3.ent_cctx_user
    grid remove ${tile_path}.frame3.ent_cctx_freq_user

    grid remove ${tile_path}.frame3.lbl_ccrx

    grid remove ${tile_path}.frame3.ent_ccrx_uv
    grid remove ${tile_path}.frame3.ent_ccrx_freq_uv

    grid remove ${tile_path}.frame3.ent_ccrx_user
    grid remove ${tile_path}.frame3.ent_ccrx_freq_user

    grid remove ${tile_path}.frame3.lbl_ccrx_freq_units

    grid remove ${tile_path}.frame3.ent_vctx_800
    grid remove ${tile_path}.frame3.ent_vctx_freq_800

    grid remove ${tile_path}.frame3.ent_vctx_uv
    grid remove ${tile_path}.frame3.ent_vctx_freq_uv

    grid remove ${tile_path}.frame3.ent_vctx_900
    grid remove ${tile_path}.frame3.ent_vctx_freq_900

    grid remove ${tile_path}.frame3.ent_vctx_user
    grid remove ${tile_path}.frame3.ent_vctx_freq_user

    grid remove ${tile_path}.frame3.lbl_vcrx

    grid remove ${tile_path}.frame3.ent_vcrx_uv
    grid remove ${tile_path}.frame3.ent_vcrx_freq_uv
    
    grid remove ${tile_path}.frame3.ent_vcrx_user
    grid remove ${tile_path}.frame3.ent_vcrx_freq_user

    grid remove ${tile_path}.frame3.lbl_vcrx_freq_units

    switch -- $::SzSnet::State(band_text) \
    {
      "800"        -
      "800 REBND"  -
      "800 SPLNT"  { # Display 800 Splinter information
                     set cctx_label  "CC"
                     set vctx_label  "VC"
                    
                     Screens::position_widget ${tile_path}.frame3.ent_cctx_800
                     Screens::position_widget ${tile_path}.frame3.ent_cctx_freq_800
                    
                     Screens::position_widget ${tile_path}.frame3.ent_vctx_800
                     Screens::position_widget ${tile_path}.frame3.ent_vctx_freq_800
                   }

      "900"        { # Display 900 information
                     set cctx_label  "CC"
                     set vctx_label  "VC"
                    
                     Screens::position_widget ${tile_path}.frame3.ent_cctx_900
                     Screens::position_widget ${tile_path}.frame3.ent_cctx_freq_900
                    
                     Screens::position_widget ${tile_path}.frame3.ent_vctx_900
                     Screens::position_widget ${tile_path}.frame3.ent_vctx_freq_900
                   }

      "VHF/UHF"    { # Display UHF / VHF information
                     set cctx_label  "CCTx"
                     set ccrx_label  "CCRx"
                     set vctx_label  "VCTx"
                     set vcrx_label  "VCRx"
                    
                     Screens::position_widget ${tile_path}.frame2.btn_band_config
                    
                     Screens::position_widget ${tile_path}.frame3.ent_cctx_uv
                     Screens::position_widget ${tile_path}.frame3.ent_cctx_freq_uv
                    
                     Screens::position_widget ${tile_path}.frame3.lbl_ccrx
                     Screens::position_widget ${tile_path}.frame3.ent_ccrx_uv
                     Screens::position_widget ${tile_path}.frame3.ent_ccrx_freq_uv
                    
                     Screens::position_widget ${tile_path}.frame3.lbl_ccrx_freq_units
                    
                     Screens::position_widget ${tile_path}.frame3.ent_vctx_uv
                     Screens::position_widget ${tile_path}.frame3.ent_vctx_freq_uv
                    
                     Screens::position_widget ${tile_path}.frame3.lbl_vcrx
                     Screens::position_widget ${tile_path}.frame3.ent_vcrx_uv
                     Screens::position_widget ${tile_path}.frame3.ent_vcrx_freq_uv
                     
                     Screens::position_widget ${tile_path}.frame3.lbl_vcrx_freq_units
                    
                     # Setup the channel table
                     SzSnet_ConfigUI::set_present_values
                   }

      "USER"       { # Display UHF / VHF information
                     set cctx_label  "CCTx"
                     set ccrx_label  "CCRx"
                     set vctx_label  "VCTx"
                     set vcrx_label  "VCRx"
                    
                     Screens::position_widget ${tile_path}.frame3.ent_cctx_user
                     Screens::position_widget ${tile_path}.frame3.ent_cctx_freq_user
                    
                     Screens::position_widget ${tile_path}.frame3.lbl_ccrx
                     Screens::position_widget ${tile_path}.frame3.ent_ccrx_user
                     Screens::position_widget ${tile_path}.frame3.ent_ccrx_freq_user
                    
                     Screens::position_widget ${tile_path}.frame3.lbl_ccrx_freq_units
                    
                     Screens::position_widget ${tile_path}.frame3.ent_vctx_user
                     Screens::position_widget ${tile_path}.frame3.ent_vctx_freq_user
                    
                     Screens::position_widget ${tile_path}.frame3.lbl_vcrx
                     Screens::position_widget ${tile_path}.frame3.ent_vcrx_user
                     Screens::position_widget ${tile_path}.frame3.ent_vcrx_freq_user
                    
                     Screens::position_widget ${tile_path}.frame3.lbl_vcrx_freq_units
                    
                     # Update the frequency displays to the present channels
                     SzSnet::update_CCTx_freq  $::SzSnet::State(cctx_chan,USER)
                     SzSnet::update_VCTx_freq  $::SzSnet::State(vctx_chan,USER)
                   }
    }

    return
  }  
  

  #***************************************************************************
  #
  #        Name: set_ptt_on_off
  #
  #  Parameters:
  #
  # Description: Sets the PTT state based on input from the MIC attachment
  #
  #***************************************************************************
  proc set_ptt_on_off { args } \
  {
     set cmd [namespace code "set_ptt_on_off_core"]
     after cancel $cmd
     after idle $cmd
  }

  proc set_ptt_on_off_core { } \
  {
    #puts "set_ptt_on_off"
    
    variable  StateUI

    if { !$::SzSnet::State(sim_on_off) } \
    { 
      return 
    }

    switch -- $::BBAudio::State(PTT) \
    {
      1        { after idle "SzSnet ptt 1 -nowait" }

      default  { SzSnet ptt 0 -nowait              }
    }

    return
  }
  
  
  #***************************************************************************
  #
  #        Name: toggle_num_format
  #
  #  Parameters:
  #
  # Description: Toggles the format of the numbers displayed
  #
  #***************************************************************************
  proc toggle_num_format { } \
  {
    #puts "toggle_num_format"

    variable  StateUI
    
    if { $StateUI(num_format) == "HEX" } \
    {
      set StateUI(num_format)  "DEC"
    } \
    else \
    {
      set StateUI(num_format)  "HEX"
    }
    
    
    return
  }


  #***************************************************************************
  #
  #        Name: update_num_format
  #
  #  Parameters:
  #
  # Description: Updates the display for the format selected
  #
  #***************************************************************************
  proc update_num_format { tile_path  args } \
  {
     set cmd [namespace code "update_num_format_core $tile_path"]
     after cancel $cmd
     after idle $cmd
  }
  proc update_num_format_core { tile_path  } \
  {
    #puts "update_num_format:  $tile_path"

    variable  StateUI
    
    set tile_path  [get_tile  $tile_path]

    if { ![winfo exists $tile_path] } \
    {
      return
    }

    grid remove ${tile_path}.frame4.lbl_nac_val_hex
    grid remove ${tile_path}.frame4.lbl_nac_val_dec
    grid remove ${tile_path}.frame4.lbl_mfid_val_hex
    grid remove ${tile_path}.frame4.lbl_mfid_val_dec

    grid remove ${tile_path}.frame5.ent_sys_id_hex
    grid remove ${tile_path}.frame5.ent_sys_id
    
    grid remove ${tile_path}.frame5.ent_grp_id_hex
    grid remove ${tile_path}.frame5.ent_grp_id
    
    grid remove ${tile_path}.frame5.ent_unit_id_hex
    grid remove ${tile_path}.frame5.ent_unit_id

    switch -- $StateUI(num_format) \
    {
      "HEX"  { if { $::SzSnet::State(call_type) == 1 } \
               {
                 # Digital call type
                 Screens::position_widget ${tile_path}.frame4.lbl_nac_val_hex
                 Screens::position_widget ${tile_path}.frame4.lbl_mfid_val_hex
               }
              
               Screens::position_widget ${tile_path}.frame5.ent_sys_id_hex
               Screens::position_widget ${tile_path}.frame5.ent_grp_id_hex
               Screens::position_widget ${tile_path}.frame5.ent_unit_id_hex 
             }

      "DEC"  { if { $::SzSnet::State(call_type) == 1 } \
               {
                 # Digital call type
                 Screens::position_widget ${tile_path}.frame4.lbl_nac_val_dec
                 Screens::position_widget ${tile_path}.frame4.lbl_mfid_val_dec
               }
              
               Screens::position_widget ${tile_path}.frame5.ent_sys_id
               Screens::position_widget ${tile_path}.frame5.ent_grp_id
               Screens::position_widget ${tile_path}.frame5.ent_unit_id
             }
    }
    
    
    return
  }

  
  #***************************************************************************
  #
  #        Name: update_tgid_encode
  #
  #  Parameters:
  #
  # Description: Updates the Encode value of the TGID to the value of the 
  #              Group ID from the Smart Net Control Channel
  #
  #***************************************************************************
  proc update_tgid_encode { args } \
  {
     set cmd [namespace code "update_tgid_encode_core"]
     after cancel $cmd
     after idle $cmd
  }
  proc update_tgid_encode_core { } \
  {
    #puts "update_tgid_encode: "

    variable StateUI

    if { $::SzSnet::State(grp_id) == "" } \
    {
      set ::SzSnet::State(grp_id)  0
    }
    
    if { $StateUI(MFID_hex) == "" } \
    {
      set StateUI(MFID_hex)  0
    }

    after idle "P25 encode field tgid  $::SzSnet::State(grp_id) -nowait"
    
    after idle "P25 encode field grpaddr  $::SzSnet::State(grp_id) -nowait"
    
    # Make a call to set the stream data
    after idle "P25 encode lcomsg  0                         \
                                   0                         \
                                   0                         \
                                   0x$StateUI(MFID_hex)      \
                                   0                         \
                                   0                         \
                                   0                         \
                                   $::SzSnet::State(grp_id)  \
                                   0 -nowait"
    
    return
  }


  #*************************************************************************** 
  #
  #  The following are wrappers to read/write the correct values for
  #  the entry fields on the Repeat Sim tile to the Uplink/Downlink 
  #  fields, respectively
  #
  #*************************************************************************** 
  #***************************************************************************
  #
  #        Name: set_nac
  #
  #  Parameters:
  #
  # Description: Sets the NAC value for Encode
  #
  #***************************************************************************
  proc set_nac { } \
  {
    #puts "set_nac"

    variable  StateUI
    
    if { $StateUI(NAC_hex) == "" } \
    {
      set StateUI(NAC_hex)  0
    }
      
    if { $StateUI(NAC_dec) == "" } \
    {
      set StateUI(NAC_dec)  0
    }


    switch -- $StateUI(num_format) \
    {
      "HEX"  { set StateUI(NAC_dec)  [format "%d" 0x$StateUI(NAC_hex)]

               after idle "P25 encode field nac  0x$StateUI(NAC_hex) -nowait" 
             }
    
      "DEC"  { set StateUI(NAC_hex)  [format "%X" $StateUI(NAC_dec)]

               after idle "P25 encode field nac  $StateUI(NAC_dec) -nowait"   
             }
    }
   
    set StateUI(update_nac)  1
     
    return
  }
    

  #***************************************************************************
  #
  #        Name: update_nac
  #
  #  Parameters:
  #
  # Description: Updates the StateUI(NAC_hex) and StateUI(NAC_dec) values
  #              to changes in the DecodeParm(HDR_NAC) value.
  #
  #***************************************************************************
  proc update_nac { args } \
  {
     set cmd [namespace code "update_nac_core"]
     after cancel $cmd
     after idle $cmd
  }
  proc update_nac_core { } \
  {
    #puts "update_nac: "

    variable StateUI

    if { $StateUI(update_nac) == 0 } \
    {
      return
    }

    if { $::P25ns::DecodeParm(HDR_NAC) != "---" } \
    {
      if { $::P25ns::DecodeParm(HDR_NAC) == "" } \
      {
        return
      }
      
      set StateUI(NAC_hex)  [format "%X" 0x$::P25ns::DecodeParm(HDR_NAC)]
    
      set StateUI(NAC_dec)  [format "%d" 0x$::P25ns::DecodeParm(HDR_NAC)]
    }
    
    return
  }


  #***************************************************************************
  #
  #        Name: set_mfid
  #
  #  Parameters:
  #
  # Description: Sets the MFID value for Encode
  #
  #***************************************************************************
  proc set_mfid { } \
  {
    #puts "set_mfid"

    variable  StateUI
    
    if { $::SzSnet::State(grp_id) == "" } \
    {
      set ::SzSnet::State(grp_id)  0
    }

    if { $StateUI(MFID_hex) == "" } \
    {
      set StateUI(MFID_hex)  0
    }

    if { $StateUI(MFID_dec) == "" } \
    {
      set StateUI(MFID_dec)  0
    }


    switch -- $StateUI(num_format) \
    {
      "HEX"  { # Value entered while in the HEX mode
               # Update the DEC variable
               set StateUI(MFID_dec)  [format "%d" 0x$StateUI(MFID_hex)]
              
               after idle "P25 encode field lco_mfid  0x$StateUI(MFID_hex) -nowait"
              
               # Make a call to set the stream data
               after idle "P25 encode lcomsg  0                         \
                                              0                         \
                                              0                         \
                                              0x$StateUI(MFID_hex)      \
                                              0                         \
                                              0                         \
                                              0                         \
                                              $::SzSnet::State(grp_id)  \
                                              0 -nowait"
             }

      "DEC"  { # Value entered while in the DEC mode
               # Update the HEX variable
               set StateUI(MFID_hex)  [format "%X" $StateUI(MFID_dec)]
              
               after idle "P25 encode field lco_mfid  $StateUI(MFID_dec) -nowait"
              
               # Make a call to set the stream data
               after idle "P25 encode lcomsg  0                         \
                                              0                         \
                                              0                         \
                                              $StateUI(MFID_dec)        \
                                              0                         \
                                              0                         \
                                              0                         \
                                              $::SzSnet::State(grp_id)  \
                                              0 -nowait"
             }
    }

    set StateUI(update_mfid)  1

    return
  }
    

  #***************************************************************************
  #
  #        Name: update_mfid
  #
  #  Parameters:
  #
  # Description: Updates the StateUI(MFID_hex) and StateUI(MFID_dec) values
  #              to changes in the DecodeParm(MFID) value.
  #
  #***************************************************************************
  proc update_mfid { args } \
  {
     set cmd [namespace code "update_mfid_core"]
     after cancel $cmd
     after idle $cmd
  }
  proc update_mfid_core { } \
  {
    #puts "update_mfid: "

    variable StateUI

    if { $StateUI(update_mfid) == 0 } \
    {
      return
    }

    if { $::P25ns::DecodeParm(MFID) != "--" } \
    {
      if { $::P25ns::DecodeParm(MFID) == "" } \
      {
        return
      }

      set StateUI(MFID_hex)  [format "%X" 0x$::P25ns::DecodeParm(MFID)]
    
      set StateUI(MFID_dec)  [format "%d" 0x$::P25ns::DecodeParm(MFID)]
    }
    
    return
  }


  #***************************************************************************
  #
  #        Name: set_downlink_data
  #
  #  Parameters:
  #
  # Description: Sets the Downlink Data to the values entered on the 
  #              Repeater SIm. tile.
  #
  #***************************************************************************
  proc set_downlink_data { } \
  {
    #puts "set_downlink_data"
    
    set_nac
    
    set_mfid
    
    after 100 "::P25ns::send_lco_0"
    
    return
  }


  #***************************************************************************
  #
  #        Name: set_xmit_dev
  #
  #  Parameters:
  #
  # Description: Sets the Transmit Deviation
  #
  #***************************************************************************
  proc set_xmit_dev { } \
  {
    #puts "set_xmit_dev"
    
    variable  StateUI
    
    after idle  "SzSnet  params  $StateUI(xmit_dev)  0"

    return
  }


  #***************************************************************************
  #
  #        Name: set_xmit_dev_def
  #
  #  Parameters:
  #
  # Description: Sets the Transmit Deviation default for the selected 
  #              band and channel
  #
  #***************************************************************************
  proc set_xmit_dev_def { } \
  {
    #puts "set_xmit_dev_def"
    
    variable  StateUI
    
    switch -- $::SzSnet::State(band_text) \
    {
      "800"        -
      "800 REBND"  -
      "800 SPLNT"  { # Set the default Transmit Deviation
                     if { $::SzSnet::State(cctx_chan,800) <= 599 } \
                     {
                       set StateUI(xmit_dev)  3125
                     } \
                     else \
                     {
                       set StateUI(xmit_dev)  2400
                     }
                   }

      "VHF/UHF"    { # Set the default Transmit Deviation
                     set StateUI(xmit_dev)  3125
                   }

      "900"        { # Set the default Transmit Deviation
                     set StateUI(xmit_dev)  1200
                   }

      "USER"       { # Set the default Transmit Deviation
                     set StateUI(xmit_dev)  3125
                   }
    }
    
    # Set the Transmit Deviation
    set_xmit_dev

    return
  }


  #***************************************************************************
  #
  #        Name: update_xmit_dev
  #
  #  Parameters:
  #
  # Description: Updates the display to changes in the Transmit Deviation
  #
  #***************************************************************************
  proc update_xmit_dev { tile_path  args } \
  {
     set cmd [namespace code "update_xmit_dev_core $tile_path"]
     after cancel $cmd
     after idle $cmd
  }
  proc update_xmit_dev_core { tile_path } \
  {

    #puts "update_xmit_dev:  $tile_path"
    
    variable  StateUI
    
    if { $StateUI(xmit_dev) <= 3500 && $StateUI(xmit_dev) >= 2750 } \
    {
      # WIDE
      ${tile_path}.frame4.lbl_xmit_dev_desc  config  -bg gray70
      
      set StateUI(xmit_dev_desc)  "WIDE"
    } \
    elseif { $StateUI(xmit_dev) <= 2700 && $StateUI(xmit_dev) >= 2100 } \
    {
      # MEDIUM
      ${tile_path}.frame4.lbl_xmit_dev_desc  config  -bg gray70

      set StateUI(xmit_dev_desc)  "MEDIUM"
    } \
    elseif { $StateUI(xmit_dev) <= 1500 && $StateUI(xmit_dev) >= 900 } \
    {
      # NARROW
      ${tile_path}.frame4.lbl_xmit_dev_desc  config  -bg gray70

      set StateUI(xmit_dev_desc)  "NARROW"
    } \
    else \
    {
      # Not valid
      ${tile_path}.frame4.lbl_xmit_dev_desc  config  -bg gray

      set StateUI(xmit_dev_desc)  ""
    }
    
    
    return
  }


  #***************************************************************************
  #
  #        Name: band_config
  #
  #  Parameters:
  #
  # Description: Displays the Band Configuration tile
  #
  #***************************************************************************
  proc band_config { tile_path } \
  {
    #puts "SzSnet_Repeat_SimUI::band_config:  $tile_path"
    
    SzSnet_ConfigUI::band_config  $tile_path  SzSnet_Repeat_SimUI
    
    return
  }


  #***************************************************************************
  #
  #        Name: band_config_return
  #
  #  Parameters: new_value -
  #
  # Description: Destroys the Band Configuration tile
  #
  #***************************************************************************
  proc band_config_return { tile_path } \
  {
    #puts "SzSnet_Repeat_SimUI::band_config_return:  $tile_path"
    
    # Synchronize the configured channel and frequency values
    set_channels

    return
  } 
    

  #****************************************************************************
  #
  #        Name: getSaveState
  #
  #  Parameters:
  #
  # Description: This function returns a list of TCL commands which can be used
  #              to set the SzSnet_Repeat_SimUI tiles to its current state
  #
  #****************************************************************************
  proc getSaveState { } \
  {
    variable  StateUI

    set returnList \
    {
      set ::SzSnet_Repeat_SimUI::StateUI(num_format)     {$StateUI(num_format)}
      set ::SzSnet_Repeat_SimUI::StateUI(xmit_dev)       {$StateUI(xmit_dev)}
      set ::SzSnet_Repeat_SimUI::StateUI(xmit_dev_desc)  {$StateUI(xmit_dev_desc)}
      set ::SzSnet_Repeat_SimUI::StateUI(NAC_hex)        {$StateUI(NAC_hex)}
      set ::SzSnet_Repeat_SimUI::StateUI(MFID_hex)       {$StateUI(MFID_hex)}
    }
    
    #puts "$returnList"
    
    return [subst -nocommands $returnList]
  }


  #****************************************************************************
  #        Name:        Sync
  #  Parameters:  none
  # Description: Write state variables to SzSnet_Repeat_SimUI
  #****************************************************************************
  proc Sync { } \
  {
    eval [getSaveState]
  }

  trace variable  ::P25ns::DecodeParm(HDR_NAC)  w  \
                                             "SzSnet_Repeat_SimUI::update_nac"

  trace variable  ::P25ns::DecodeParm(MFID)  w  \
                                            "SzSnet_Repeat_SimUI::update_mfid"

  trace variable  ::BBAudio::State(PTT)  w  \
                                         "SzSnet_Repeat_SimUI::set_ptt_on_off"

  update_nac
    
  update_mfid
        


};  # End of SzSnet_Repeat_SimUI namespace


#****************************************************************************
# This registers the tiles with the list for saving instrument states
#****************************************************************************
namespace eval ::SavedStates \
{
  variable tiles
  
  lappend tiles  SzSnet_Repeat_SimUI
}


