#############################################################################
#                                                                           #
# IFR SYSTEMS, INC    COPYRIGHT (C) 1998,99,2001,2002  DESIGN ENGINEERING   #
# ------------------------------------------------------------------------- #
#                                                                           #
# FILE        : 3690.SmartNet_ScannerUI.tcl                                 #
#                                                                           #
# PRODUCT     : FITS                                                        #
#                                                                           #
# AUTHOR      : P. McCormick                                                #
#                                                                           #
# DESCRIPTION : SmartNet Scanner User Interface                             #
#                                                                           #
#############################################################################


package require SzSnet
package require IFR_options

namespace eval SzSnet_ScannerUI \
{
  # Global variables
  variable  LongName     "scanner"
  variable  ShortName    "SN/SZ Scanner"
  variable  Size         Quarter
            
  variable  tile_class   "SCANNER"
  variable  last_focus
  
  variable  StateUI
  
  set StateUI(find_mode)     "FIND"
  set StateUI(start)         300
  set StateUI(stop)          400


  variable  control_chan_list
  variable  group_id_list
  variable  unit_id_list
  
  variable  on_off   "OFF"
  
  namespace import ::SzSnet_Repeat_SimUI::band_config_return


  #********************************************************************
  #
  #       Name: Create  
  #
  # Parameters: size - meter size 
  #
  # Description: 
  #
  #********************************************************************
  proc Create { tile_path } \
  {
    #puts "SzSnet_ScannerUI::Create:  $tile_path"
    
    variable  LongName
    variable  tile_class
    
    set top_level  [winfo toplevel $tile_path]
   
    set tile_path  [name_child  $top_level  $LongName]

    #puts "tile_path:  $tile_path"

    if { ![winfo exists $tile_path] } \
    {
      set tile_path  [Screens::create_tile  $tile_path  $LongName  $tile_class]
    }  
     
    return $tile_path
  }


  #******************************************************************** 
  #
  #       Name: Map
  #
  # Parameters: 
  #
  # Description: Initializes the tile
  #
  #********************************************************************
  proc Map { tile_path } \
  {
    #puts "SzSnet_ScannerUI::Map:  $tile_path"

    if { ![IFR_options is_installed SzSnet] } \
    {
      grid forget $tile_path

      return
    }
    
    variable  StateUI
    
    # Set local variables
    set tile_path  [get_tile  $tile_path]

    # Make a call to the SzSnet namespace to load the namespace
    #SzSnet ptt

    # Trace engine variables
    trace variable ::SzSnet::State(found)  w   \
                                     "SzSnet_ScannerUI::update_add  $tile_path"
    
    trace variable ::SzSnet::State(onvx)  w   \
                                    "SzSnet_ScannerUI::update_onvx  $tile_path"

    # Trace interface variables
    trace variable ::SzSnet::State(follow)  w   \
                                  "SzSnet_ScannerUI::update_follow  $tile_path"

    trace variable ::SzSnet::State(scan_on_off)  w   \
                               "SzSnet_ScannerUI::update_find_mode  $tile_path"


    # Setup the tile
    update_follow_core     $tile_path
    update_band_core       $tile_path
    update_onvx_core       $tile_path

    # Determine if the option is installed, and set the edit fields accordingly
    if { [IFR_options  is_installed  SzSnet] } \
    {
      edit_fields  $tile_path  ENABLE
    } \
    else \
    {
      edit_fields  $tile_path  DISABLE
    }

    
    # Determine if the 900 MHz option is installed, 
    # and set the menu options accordingly
    if { [IFR_options  is_installed  SzSnet_900] } \
    {
      # No action
      # The menu option is on the list
    } \
    else \
    {
      # Remove the option from the menu
      oldCatch { ${tile_path}.frame2.mnu_band.menu  delete  "900*" } result
    }

    # Determine if the 800 MHz Reband option is installed, 
    # and set the menu options accordingly
    if { [IFR_options  is_installed  SzSnet_Reband] } \
    {
      # No action
      # The menu option is on the list
    } \
    else \
    {
      # Remove the option from the menu
      oldCatch { ${tile_path}.frame2.mnu_band.menu  delete  "*REBND" } result
    }
  }


  #********************************************************************
  #
  #       Name: Cleanup
  #
  # Parameters: 
  #
  # Description: 
  #
  #********************************************************************
  proc Cleanup { tile_path } \
  {
    #puts "SzSnet_ScannerUI::Cleanup:  $tile_path"
    
    variable  StateUI

    # Delete variable traces
    trace vdelete ::SzSnet::State(found)  w   \
                                     "SzSnet_ScannerUI::update_add  $tile_path"
    
    trace vdelete ::SzSnet::State(onvx)  w   \
                                    "SzSnet_ScannerUI::update_onvx  $tile_path"

    trace vdelete ::SzSnet::State(follow)  w   \
                                  "SzSnet_ScannerUI::update_follow  $tile_path"

    trace vdelete ::SzSnet::State(scan_on_off)  w   \
                               "SzSnet_ScannerUI::update_find_mode  $tile_path"

    # Restore the previous Receiver squelch mode
    after idle "Receiver squelchmode  AUTO"
  }


  #********************************************************************
  #
  #       Name: Unmap
  #
  # Parameters: 
  #
  # Description: 
  #
  #********************************************************************
  proc Unmap { tile_path } \
  {
    #puts "SzSnet_ScannerUI::Unmap:  $tile_path"
    
    Cleanup  $tile_path
  }


  #********************************************************************
  #
  #       Name: Destroy
  #
  # Parameters: 
  #
  # Description: 
  #
  #********************************************************************
  proc Destroy { tile_path } \
  {
    #puts "SzSnet_ScannerUI::Destroy:  $tile_path"
    
    Cleanup  $tile_path
  }


  #********************************************************************
  #
  #       Name: Register
  #
  # Parameters: 
  #
  # Description: 
  #
  #********************************************************************
  proc Register { tile_path } \
  {
    #puts "SzSnet_ScannerUI::Register:  $tile_path"

    variable  StateUI
    
    set tile_path  [get_tile  $tile_path]
  }
  

  #***************************************************************************
  #
  #       Name: get_tile
  #
  # Parameters: any child widget of the tile
  #
  # Description: get the name of the tile's parent frame
  #
  #***************************************************************************
  proc get_tile { widget } \
  {
    while { 1 } \
    {
      switch -- [winfo class $widget] \
      {
        "SCANNER"   { return $widget }
      }
        
      set widget [winfo parent $widget]
    }
  }


  #********************************************************************
  #
  #       Name: update_add
  #
  # Parameters: 
  #
  # Description: Condition the engine variable trace for the 
  #            : 'list_add' procedure
  #
  #********************************************************************
  proc update_add { tile_path  args } \
  {
    #puts "SzSnet_ScannerUI::update_add:  $tile_path,  $args"

    variable  StateUI
    
    set tile_path  [get_tile  $tile_path]


    list_add  $tile_path  $::SzSnet::State(found)
  }


  #********************************************************************
  #
  #       Name: update_onvx
  #
  # Parameters: 
  #
  # Description: Condition the engine variable trace for the 
  #            : 'voice_chan_active' procedure
  #
  #********************************************************************
  proc update_onvx { tile_path  args } \
  {
     set cmd [namespace code "update_onvx_core $tile_path"]
     after cancel $cmd
     after idle $cmd
  }
  proc update_onvx_core { tile_path } \
  {
    #puts "SzSnet_ScannerUI::update_onvx:  $tile_path"

    set tile_path  [get_tile  $tile_path]


    if { $::SzSnet::State(onvx) == 1 } \
    {
      ${tile_path}.frame5.lbl_vc_active config -bg green
    }
    
    if { $::SzSnet::State(onvx) == 0 } \
    {
      ${tile_path}.frame5.lbl_vc_active config -bg gray
    }
  }


  #***************************************************************************
  #
  #       Name: update_follow
  #
  # Parameters: new_value -
  #
  # Description: Updates the display to the Scanner Follow mode
  #
  #***************************************************************************
  proc update_follow { tile_path  args } \
  {
     set cmd [namespace code "update_follow_core $tile_path"]
     after cancel $cmd
     after idle $cmd
  }
  proc update_follow_core { tile_path } \
  {
    #puts "update_follow:  $tile_path"
    
    variable  StateUI
    variable  on_off
    
    set tile_path  [get_tile  $tile_path]
    
    # Remove all subtiles
    grid remove ${tile_path}.frame3_1
    grid remove ${tile_path}.frame3_2
    grid remove ${tile_path}.frame3_3

    grid remove ${tile_path}.frame4_1
    grid remove ${tile_path}.frame4_2
    grid remove ${tile_path}.frame4_3


    # Determine the present menu selection,
    # then invoke the next one and set the third softkey
    # to the appropriate title
    if { $::SzSnet::State(follow) == "CONTROL CHANNEL" } \
    {
      Screens::position_widget ${tile_path}.frame3_1
      Screens::position_widget ${tile_path}.frame4_1
    }
    
    if { $::SzSnet::State(follow) == "GROUP ID" } \
    {
      Screens::position_widget ${tile_path}.frame3_2
      Screens::position_widget ${tile_path}.frame4_2
    }

    if { $::SzSnet::State(follow) == "UNIT ID" } \
    {
      Screens::position_widget ${tile_path}.frame3_3
      Screens::position_widget ${tile_path}.frame4_3
    }
  }


  #***************************************************************************
  #
  #       Name: update_band
  #
  # Parameters: new_value -
  #
  # Description: Updates the display to the new OSW band
  #
  #***************************************************************************
  proc update_band { tile_path  args } \
  {
     set cmd [namespace code "update_band_core $tile_path"]
     after cancel $cmd
     after idle $cmd
  }
  proc update_band_core { tile_path } \
  {
    #puts "update_band:  $tile_path"
    
    variable  StateUI
    
    set tile_path  [get_tile  $tile_path]
    
    
    grid remove ${tile_path}.frame2.btn_band_config


    if { $::SzSnet::State(band_text) == "800" } \
    {
      set ::SzSnet::State(max_chan)  1023
      set ::SzSnet::State(min_chan)  0
    }

    if { $::SzSnet::State(band_text) == "800 SPLNT" } \
    {
      set ::SzSnet::State(max_chan)  1023
      set ::SzSnet::State(min_chan)  0
    }
    
    if { $::SzSnet::State(band_text) == "VHF/UHF" } \
    {
      Screens::position_widget ${tile_path}.frame2.btn_band_config

      set ::SzSnet::State(max_chan)  759
      set ::SzSnet::State(min_chan)  380

      # Setup the channel table
      SzSnet_ConfigUI::set_present_values
    }
        
    if { $::SzSnet::State(band_text) == "900" } \
    {
      set ::SzSnet::State(max_chan)  1023
      set ::SzSnet::State(min_chan)  0
    }

    if { $::SzSnet::State(band_text) == "USER" } \
    {
      set ::SzSnet::State(max_chan)  1023
      set ::SzSnet::State(min_chan)  0
    }

    # Call the 'set_scan_control_chan' procedure to update the Scanner channel
    set_scan_control_chan
  }
    

  #***************************************************************************
  #
  #       Name: set_scan_follow_mode
  #
  # Parameters:
  #
  # Description: Sets the Scanner follow mode
  #
  #***************************************************************************
  proc set_scan_follow_mode { } \
  {
    #puts "set_scan_follow_mode"
    
    variable  StateUI
  
    # Cancel the Group ID and Unit ID follow
    SzSnet followid     0  -nowait
    SzSnet followgroup  0  -nowait

    if { $::SzSnet::State(follow) == "CONTROL CHANNEL" } \
    {

    }
    
    if { $::SzSnet::State(follow) == "GROUP ID" } \
    {
      if { $::SzSnet::State(grp_id) == "" } \
      {
        #puts "return"
        
        return
      }
   
      # Set the Group ID
      SzSnet followgroup  $::SzSnet::State(grp_id)  -nowait
    }

    if { $::SzSnet::State(follow) == "UNIT ID" } \
    {
      if { $::SzSnet::State(unit_id) == "" } \
      {
        #puts "return"
        
        return
      }
    
    
      # Set the Unit ID
      SzSnet followid  $::SzSnet::State(unit_id)  -nowait
    }

    # If the Scanner is on, cycle the Scanner OFF / ON
    if { $::SzSnet::State(scan_on_off) == 1 } \
    {
      after idle "SzSnet::cycleScanOnOff"
    }
  }


  #***************************************************************************
  #
  #       Name: set_scan_control_chan
  #
  # Parameters:
  #
  # Description: Sets the Scanner Control Channel Transmit channel number
  #
  #***************************************************************************
  proc set_scan_control_chan { } \
  {
    #puts "set_scan_control_chan"
    
    if { $::SzSnet::State(control_chan) == "" } \
    {
      return
    }
     
    if { $::SzSnet::State(control_chan) == "NONE" } \
    {
      set ::SzSnet::State(control_chan)  ""
      
      return
    }
  
    set ::SzSnet::State(control_chan)  [string trim $::SzSnet::State(control_chan)]
  
    if { $::SzSnet::State(control_chan) > $::SzSnet::State(max_chan) } \
    {
      set ::SzSnet::State(control_chan)  $::SzSnet::State(max_chan)
    }
    
    if { $::SzSnet::State(control_chan) < $::SzSnet::State(min_chan) } \
    {
      set ::SzSnet::State(control_chan)  $::SzSnet::State(min_chan)
    }
    
    # Set the Control Channel
    SzSnet control $::SzSnet::State(control_chan) -nowait
    
    # Reset the System ID
    set ::SzSnet::State(found_sys_id)  ""
  

    # If the Scanner is on, cycle the Scanner OFF / ON
    if { $::SzSnet::State(scan_on_off) == 1 } \
    {
      after idle "SzSnet::cycleScanOnOff"
    }
  }


  #***************************************************************************
  #
  #       Name: set_band
  #
  # Parameters:
  #
  # Description: Sets the OSW band
  #
  #***************************************************************************
  proc set_band { } \
  {
    #puts "set_band"
    
    ::SzSnet::setBand
   
    # If the Scanner is on, cycle the Scanner OFF/ ON
    if { $::SzSnet::State(scan_on_off) == 1 } \
    {
      after idle "SzSnet::cycleScanOnOff"
    }
  }  


  #***************************************************************************
  #
  #       Name: set_find_mode
  #
  # Parameters: new_value -
  #
  # Description: Sets the Scanner Find mode: FIND/STOP
  #
  #***************************************************************************
  proc set_find_mode { { new_value "" } } \
  {
    #puts "set_find_mode:  $new_value"
 
    variable  StateUI
    

    if { $new_value != "" } \
    {
      set StateUI(find_mode)  $new_value
    }
    

    if { $StateUI(find_mode) == "FIND" } \
    {
      set StateUI(find_mode)  "STOP"

      find_control_chan
    } \
    else \
    {
      set StateUI(find_mode)  "FIND"
    }
  }


  #***************************************************************************
  #
  #       Name: update_find_mode
  #
  # Parameters:
  #
  # Description: Updates the Scanner Find mode
  #
  #***************************************************************************
  proc update_find_mode { args } \
  {
    if { $::SzSnet::State(scan_on_off) == 1 } \
    {
      after idle [namespace code "set_find_mode STOP"]
    }
  }

  #***************************************************************************
  #
  #       Name: find_control_chan
  #
  # Parameters: new_value -
  #
  # Description: Starts Scanner Find 
  #
  #***************************************************************************
  proc find_control_chan { { new_value "" } } \
  {
    #puts "find_control_chan"
    
    variable  StateUI
    
    set start  $StateUI(start)
    set stop   $StateUI(stop)


    if { $start == "" || $stop == "" } \
    {
      return
    }


    if { $start > $stop } \
    {
      set temp   $stop
      set stop   $start
      set start  $temp
    }
    
    # Get the present ON/OFF state
    set on_off  $::SzSnet::State(scan_on_off_text)

    # Turn off the Scanner
    SzSnet follow  0  -nowait
     
    # Clear the 'ACTIVE' indicator
    set State(onvx)  0    
    
    # set generator and receiver state
    Generator ptt 0 -nowait
    
    # Find the Control Channels
    SzSnet find  $start  $stop
      
    while { [SzSnet find] == 1 } \
    {
      # Check to see if the Find Mode has changed
      if { $StateUI(find_mode) == "FIND" } \
      {
        SzSnet find 0
      }

      # Exit if either the Receiver or Generator becomes unlocked.
      if { $Receiver::State(lockstat,raw) != 0 } \
      {
        puts "Receiver unlock, stop find mode"
        
        SzSnet find 0
      }

      if { $Generator::State(lockstat,raw) != 0 } \
      {
        puts "Generator unlock, stop find mode"

        SzSnet find 0
      }
      
      after 500
    }

    # Set the Find mode to STOP
    set_find_mode  "STOP"
    
    # Restore the hardware state
    if { $on_off == "OFF" } \
    {
      after idle "SzSnet follow  0  -nowait"
    } \
    else \
    {
      after idle "SzSnet follow  1  -nowait"
    }
  }



  #***************************************************************************
  #
  #       Name: list_add
  #
  # Parameters: new_value -
  #
  # Description: Adds the user value to the Scanner list 
  #
  #***************************************************************************
  proc list_add { tile_path { new_value "" } } \
  {
    #puts "list_add:  $tile_path,  $new_value"
    
    variable  StateUI
    variable  control_chan_list
    variable  group_id_list
    variable  unit_id_list
    
    set tile_path  [get_tile  $tile_path]
    

    if { $::SzSnet::State(follow) == "" } \
    {
      return
    }
    
    
    if { $::SzSnet::State(follow) == "CONTROL CHANNEL" } \
    {
      if { $new_value == "" } \
      {
        set control_chan  $::SzSnet::State(control_chan)
      } \
      else \
      {
        set control_chan  $new_value
      }

      if { $control_chan == "" } \
      {
        # If the value is invalid
        # then nothing else needs to be done
        return
      }
      
      set exists  [array exists control_chan_list]
      
      if { $exists != 1 } \
      {
        set size  0
      } \
      else \
      {
        set size  [array size control_chan_list]
      }
      
      # Search through the array to determine if the value is already
      # in the array
      for { set i 0 } { $i < $size } { incr i } \
      {
        set value  $control_chan_list($i)
        
        if { $value == $control_chan } \
        { 
          # If the value was found to be in the array already,
          # then nothing else needs to be done
          return
        }
      }
      
      # Add the value to the array
      set control_chan_list($size)  $control_chan
      

      # Setup the menu from the array
      set menubtn  ${tile_path}.frame3_1.mnu_list_chan
      set menu     ${tile_path}.frame3_1.mnu_list_chan.menu

      # If the menu exists, 
      # delete all items in the menu, and create a new menu
      if { [winfo exists $menu] } \
      {
        $menu  delete  0  last
      } \
      else \
      {
        $menubtn config -menu $menu
      
        menu  $menu -tearoff 0
      }
      
      set size  [array size control_chan_list]

      for { set i 0 } { $i < $size } { incr i } \
      {
        $menu add radio -label $control_chan_list($i)  \
                        -value $control_chan_list($i)  \
                        -command "::SzSnet_ScannerUI::set_scan_control_chan" \
                        -variable ::SzSnet::State(control_chan)
      }
    
      # Show the new value on the menubutton
      set ::SzSnet::State(control_chan)  $control_chan
    }
  }


  #***************************************************************************
  #
  #       Name: list_clear
  #
  # Parameters: new_value -
  #
  # Description: Clears the list
  #
  #***************************************************************************
  proc list_clear { tile_path { new_value "" } } \
  {
    #puts "list_clear"

    variable  StateUI
    variable  control_chan_list
    variable  group_id_list
    variable  unit_id_list
    
    set tile_path  [get_tile  $tile_path]

    set follow  $::SzSnet::State(follow)
    

    if { $follow == "" } \
    {
      return
    }


    if { $follow == "CONTROL CHANNEL" } \
    {
      # Delete all items in the menu
      set menu  ${tile_path}.frame3_1.mnu_list_chan.menu

      if { [winfo exists $menu] } \
      {
        $menu  delete  0  last
        
        #destroy $menu
        $menu add radio -label NONE  \
                        -value NONE
      }

      if { [info exists control_chan_list] } \
      {
        unset control_chan_list
      }
    }
  }


  #***************************************************************************
  #
  #       Name: band_config
  #
  # Parameters: new_value -
  #
  # Description: Displays the Band Configuration tile
  #
  #***************************************************************************
  proc band_config { tile_path } \
  {
    #puts "ScannerUI::band_config:  $tile_path"
    
    SzSnet_ConfigUI::band_config  $tile_path  SzSnet_ScannerUI
  }

  #***************************************************************************
  #
  #       Name: edit_fields
  #
  # Parameters:
  #
  # Description: Enables and Disables the edit fields of the tile depending on
  #            : the Options status
  #
  #***************************************************************************
  proc edit_fields { tile_path  { mode  DISABLE } } \
  {
    #puts "edit_fields:  $tile_path,  $mode"
    
    set tile_path  [get_tile  $tile_path]
    
    
    switch -- $mode \
    {
      "ENABLE"   { # List of edit fields
                   ${tile_path}.frame0.btn_on_off       config -state normal
                   
                   ${tile_path}.frame2.btn_band_config  config -state normal
                   
                   ${tile_path}.frame3_1.ent_chan       config -state normal
                   ${tile_path}.frame3_1.ent_sys_id     config -state normal
                   
                   ${tile_path}.frame3_2.ent_group_id   config -state normal
                   
                   ${tile_path}.frame3_3.ent_unit_id    config -state normal
                   
                   ${tile_path}.frame4_1.ent_start      config -state normal
                   ${tile_path}.frame4_1.ent_stop       config -state normal
                 }

      "DISABLE"  { # List of edit fields 
                   ${tile_path}.frame0.btn_on_off       config -state disabled
                   
                   ${tile_path}.frame2.btn_band_config  config -state normal

                   ${tile_path}.frame3_1.ent_chan       config -state disabled
                   ${tile_path}.frame3_1.ent_sys_id     config -state disabled
                   
                   ${tile_path}.frame3_2.ent_group_id   config -state disabled
                   
                   ${tile_path}.frame3_3.ent_unit_id    config -state disabled
                   
                   ${tile_path}.frame4_1.ent_start      config -state disabled
                   ${tile_path}.frame4_1.ent_stop       config -state disabled
                 }
      
      default    {  }
    } 
  }


  #****************************************************************************
  # Name:        getSaveState
  # Parameters:  none
  # Description: This function returns a list of TCL commands which can be used
  #              to set the SzSnet_Repeat_SimUI tiles to its current state
  #****************************************************************************
  proc getSaveState { } \
  {
    variable  StateUI

    set returnList { }

    set returnList \
    {
      set ::SzSnet_ScannerUI::StateUI(start)  {$StateUI(start)}
      set ::SzSnet_ScannerUI::StateUI(stop)   {$StateUI(stop)}
    }
    
    return [subst -nocommands $returnList]
  }


  #****************************************************************************
  # Name:        Sync
  # Parameters:  none
  # Description: Write state variables to SzSnet_Repeat_SimUI
  #****************************************************************************
  proc Sync { } \
  {
    eval [getSaveState]
  }


};  # End of SzSnet_ScannerUI namespace


#****************************************************************************
# This registers the tiles with the list for saving instrument states
#****************************************************************************
namespace eval ::SavedStates \
{
  variable tiles
  
  lappend tiles SzSnet_ScannerUI
}

