#############################################################################
#                                                                           #
# IFR SYSTEMS, INC    COPYRIGHT (C) 1998,99,2001,2002  DESIGN ENGINEERING   #
# ------------------------------------------------------------------------- #
#                                                                           #
# FILE        : 3700.SmartNet_ConfigUI.tcl                                  #
#                                                                           #
# PRODUCT     : FITS                                                        #
#                                                                           #
# AUTHOR      : P. McCormick                                                #
#                                                                           #
# DESCRIPTION : SmartNet Repeater Simulator User Interface                  #
#                                                                           #
#############################################################################

package require IFR_options

namespace eval SzSnet_ConfigUI \
{
  # Global variables
  variable  LongName     "config"
  variable  ShortName    "CONFIG"
  variable  Size         Half_v
            
  variable  tile_class   "CONFIG"

  variable  last_focus
  variable  ret_namespace
  variable  please_wait   ""
  
  variable StateUI
  

  ###################################
  #
  # VHF/UHF Configuration variables
  #
  ###################################
  set StateUI(tx_start,0)  ""
  set StateUI(tx_stop,0)   ""
  set StateUI(tx_space,0)  ""

  set StateUI(rx_start,0)  ""
  set StateUI(rx_stop,0)   ""
  set StateUI(rx_space,0)  ""
  
  set StateUI(tx_start,1)  ""
  set StateUI(tx_stop,1)   ""
  set StateUI(tx_space,1)  ""

  set StateUI(rx_start,1)  ""
  set StateUI(rx_stop,1)   ""
  set StateUI(rx_space,1)  ""
  
  set StateUI(tx_start,2)  ""
  set StateUI(tx_stop,2)   ""
  set StateUI(tx_space,2)  ""

  set StateUI(rx_start,2)  ""
  set StateUI(rx_stop,2)   ""
  set StateUI(rx_space,2)  ""
  

  ###########################################
  #
  # VHF/UHF Default Configuration variables
  #
  ###########################################
  set StateUI(tx_start_def,0)  404.000000
  set StateUI(tx_stop_def,0)   407.150000
  set StateUI(tx_space_def,0)  25.000000

  set StateUI(tx_start_def,1)  407.175000
  set StateUI(tx_stop_def,1)   410.325000
  set StateUI(tx_space_def,1)  25.000000

  set StateUI(tx_start_def,2)  410.350000
  set StateUI(tx_stop_def,2)   413.475000
  set StateUI(tx_space_def,2)  25.000000

  set StateUI(rx_start_def,0)  403.000000
  set StateUI(rx_stop_def,0)   406.150000
  set StateUI(rx_space_def,0)  25.000000
  
  set StateUI(rx_start_def,1)  406.175000
  set StateUI(rx_stop_def,1)   409.325000
  set StateUI(rx_space_def,1)  25.000000
  
  set StateUI(rx_start_def,2)  409.350000
  set StateUI(rx_stop_def,2)   412.475000
  set StateUI(rx_space_def,2)  25.000000


  ########################################
  #
  # User Defined Configuration variables
  #
  ########################################
  set StateUI(user_chan_list)  { 0  1  2  3  4  5  6 }
  set StateUI(user_chan_no)    0
  set StateUI(user_band)       "TEST 1"


  ################################################
  #
  # User Defined Default Configuration variables
  #
  ################################################
  set StateUI(user_tx_freq,0)  937.000000
  set StateUI(user_rx_freq,0)  898.000000

  set StateUI(user_tx_freq,1)  937.500000
  set StateUI(user_rx_freq,1)  898.500000

  set StateUI(user_tx_freq,2)  935.437500
  set StateUI(user_rx_freq,2)  896.437500

  set StateUI(user_tx_freq,3)  935.425000
  set StateUI(user_rx_freq,3)  896.425000

  set StateUI(user_tx_freq,4)  935.412500
  set StateUI(user_rx_freq,4)  896.412500

  set StateUI(user_tx_freq,5)  935.400000
  set StateUI(user_rx_freq,5)  896.400000

  set StateUI(user_tx_freq,6)  935.387500
  set StateUI(user_rx_freq,6)  896.387500




  #***************************************************************************
  #
  #        Name: get_tile
  #
  #  Parameters: a widget within the tile
  #
  #Description: get the name of the tile frame
  #
  #***************************************************************************
  proc get_tile { widget } \
  {
    while { 1 } \
    {
      switch -- [winfo class $widget] \
      {
        "VHF_UHF_CONFIG"  { return $widget } 
      }

      set widget [winfo parent $widget]
    }
  }


  #***************************************************************************
  #
  #        Name: band_config
  #
  #  Parameters:
  #
  # Description: Displays the Band Configuration tile
  #
  #***************************************************************************
  proc band_config { tile_path  name_space } \
  {
    #puts "ConfigUI::band_config:  $tile_path,  $name_space"
    
    variable  StateUI
    variable  last_focus 
    variable  ret_namespace
    
    set last_focus     $tile_path
    set ret_namespace  $name_space
     
    #puts "last_focus:  $last_focus"
    #puts "ret_namespace:  $ret_namespace"

    if { ![IFR_options is_installed SzSnet] } \
    {
      return
    }
    
    set top_level  [winfo toplevel $tile_path]
   
    #puts "top_level:  $top_level"

    if { $::SzSnet::State(band_text) == "VHF/UHF" } \
    {    
      get_present_values

      set tile_path  [name_child  $top_level  vhf_uhf]

      #puts "tile_path:  $tile_path"

      if { ![winfo exists $tile_path] } \
      {
        set tile_path  [Screens::create_toplevel_tile  $tile_path  VHF_UHF_CONFIG]
      }
    
      #puts "tile_path:  $tile_path"
    }

    if { $::SzSnet::State(band_text) == "USER" } \
    {
      #set tile_path  [name_child  $top_level  user_config]
      #
      ##puts "tile_path:  $tile_path"
      #
      #if { ![winfo exists $tile_path] } \
      #{
      #  set tile_path  [Screens::create_toplevel_tile  $tile_path  USER_CONFIG]
      #}
      #
      ##puts "tile_path:  $tile_path"
      # 
      #set_user_chan_number  $tile_path
    }

    bind $tile_path <KeyRelease-F8> "destroy [winfo toplevel $tile_path]"

    bind ${tile_path}.btn_return <Destroy> \
                           "::SzSnet_ConfigUI::band_config_return  $tile_path" 

    safe_focus_force "${tile_path}.btn_return"

    return    
  }


  #***************************************************************************
  #
  #        Name: band_config_return
  #
  #  Parameters:
  #
  # Description: Destroys the Band Configuration tile
  #
  #***************************************************************************
  proc band_config_return { tile_path } \
  {
    #puts "ConfigUI::band_config_return:  $tile_path"
    
    variable  StateUI
    variable  last_focus 
    variable  ret_namespace

    variable  please_wait
    
    set please_wait  "Please Wait ..."
    
    update idle
    

    # Set up the channel values
    set_present_values
    
    safe_focus_force $last_focus

    set please_wait  ""
    
    # Call the return procedure for the calling namespace
    ${ret_namespace}\::band_config_return  $last_focus
    
    return    
  }


  #***************************************************************************
  #
  #        Name: set_present_values
  #
  #  Parameters:
  #
  # Description: Sets the values as entered into the configuration toplevel.
  #
  #***************************************************************************
  proc set_present_values { } \
  {
    #puts "set_present_values"

    variable  StateUI
    
    #puts "::SzSnet::State(band_text):  $::SzSnet::State(band_text)"
    
    if { $::SzSnet::State(band_text) == "VHF/UHF" } \
    {    
      for { set index 0 } { $index < 3 } { incr index } \
      {
        # Check to see if the present values are valid,
        # If they are not, set them to the respective default value
        if { $StateUI(tx_start,$index) == "" } \
        {
          set StateUI(tx_start,$index)  [format "%.4f" $StateUI(tx_start_def,$index)]
        }
        
        if { $StateUI(tx_stop,$index) == "" } \
        {
          set StateUI(tx_stop,$index)  [format "%.4f" $StateUI(tx_stop_def,$index)]
        }
     
        if { $StateUI(tx_space,$index) == "" } \
        {
          set StateUI(tx_space,$index)  [format "%.4f" $StateUI(tx_space_def,$index)]
        }
     
        # If the Start freq is greater than the Stop freq,
        # swap the values
        if { $StateUI(tx_start,$index) > $StateUI(tx_stop,$index) } \
        {
          set temp  $StateUI(tx_stop,$index)
          
          set StateUI(tx_stop,$index)  [format "%.4f" $StateUI(tx_start,$index)]
          
          set StateUI(tx_start,$index)  [format "%.4f" $temp]
        }
        
        # Transmit blocks
        # UUT Tx is the 2975 Rx
        SzSnet uhfrx $index $StateUI(tx_start,$index) \
                            $StateUI(tx_stop,$index)  \
                            $StateUI(tx_space,$index)
      
     
        # Check to see if the present values are valid,
        # If they are not, set them to the respective default value
        if { $StateUI(rx_start,$index) == "" } \
        {
          set StateUI(rx_start,$index)  [format "%.4f" $StateUI(rx_start_def,$index)]
        }
        
        if { $StateUI(rx_stop,$index) == "" } \
        {
          set StateUI(rx_stop,$index)  [format "%.4f" $StateUI(rx_stop_def,$index)]
        }
     
        if { $StateUI(rx_space,$index) == "" } \
        {
          set StateUI(rx_space,$index)  [format "%.4f" $StateUI(rx_space_def,$index)]
        }
     
        # If the Start freq is greater than the Stop freq,
        # swap the values
        if { $StateUI(rx_start,$index) > $StateUI(rx_stop,$index) } \
        {
          set temp  $StateUI(rx_stop,$index)
          
          set StateUI(rx_stop,$index)  [format "%.4f" $StateUI(rx_start,$index)]
          
          set StateUI(rx_start,$index)  [format "%.4f" $temp]
        }
     
        # Receive blocks
        # UUT Rx is the 2975 Tx
        SzSnet uhftx $index $StateUI(rx_start,$index) \
                            $StateUI(rx_stop,$index)  \
                            $StateUI(rx_space,$index)
      }
    }
    
    if { $::SzSnet::State(band_text) == "USER" } \
    {
      ## Set the frequency of each channel
      #foreach { channel } $StateUI(user_chan_list) \
      #{
      #  #puts "channel:  $channel"
      #  
      #  SzSnet  txfreq  $channel  $StateUI(user_tx_freq,$channel)  -nowait
      #           
      #  SzSnet  rxfreq  $channel  $StateUI(user_rx_freq,$channel)  -nowait
      #}
    }
  
    return
  }


  #***************************************************************************
  #
  #        Name: get_present_values
  #
  #  Parameters:
  #
  # Description: Gets the values from the engine, if the present values
  #              are null.
  #
  #***************************************************************************
  proc get_present_values { } \
  {
    #puts "get_present_values"

    variable  StateUI
    
    for { set index 0 } { $index < 3 } { incr index } \
    {
      foreach { start  stop  space } [SzSnet uhfrx $index] \
      {
        if { $StateUI(rx_start,$index) == "" } \
        {
          set StateUI(rx_start,$index)  [format "%.4f" $start]
        }
        
        if { $StateUI(rx_stop,$index) == "" } \
        {
          set StateUI(rx_stop,$index)   [format "%.4f" $stop]
        }

        if { $StateUI(rx_space,$index) == "" } \
        {
          set StateUI(rx_space,$index)  [format "%.4f" $space]
        }
      }

      foreach { start  stop  space } [SzSnet uhftx $index] \
      {
        if { $StateUI(tx_start,$index) == "" } \
        {
          set StateUI(tx_start,$index)  [format "%.4f" $start]
        }
        
        if { $StateUI(tx_stop,$index) == "" } \
        {
          set StateUI(tx_stop,$index)   [format "%.4f" $stop]
        }

        if { $StateUI(tx_space,$index) == "" } \
        {
          set StateUI(tx_space,$index)  [format "%.4f" $space]
        }
      }
    }
    
    return
  }


  #***************************************************************************
  #
  #        Name: restore_defaults
  #
  #  Parameters:
  #
  # Description: Displays the Band Configuration tile
  #
  #***************************************************************************
  proc restore_defaults { } \
  {
    #puts "ConfigUI::restore_defaults"
    
    variable  StateUI

    if { $::SzSnet::State(band_text) == "VHF/UHF" } \
    {    
      # Set the UHF/VHF freq blocks
      for { set index 0 } { $index < 3 } { incr index } \
      {
        set StateUI(tx_start,$index)  $StateUI(tx_start_def,$index)
        set StateUI(tx_stop,$index)   $StateUI(tx_stop_def,$index)
        set StateUI(tx_space,$index)  $StateUI(tx_space_def,$index)
     
        set StateUI(rx_start,$index)  $StateUI(rx_start_def,$index)
        set StateUI(rx_stop,$index)   $StateUI(rx_stop_def,$index) 
        set StateUI(rx_space,$index)  $StateUI(rx_space_def,$index)
     
        # Transmit blocks
        # UUT Tx is the 2975 Rx
        SzSnet uhfrx $index $StateUI(tx_start,$index) \
                            $StateUI(tx_stop,$index)  \
                            $StateUI(tx_space,$index)
      
        # Receive blocks
        # UUT Rx is the 2975 Tx
        SzSnet uhftx $index $StateUI(rx_start,$index) \
                            $StateUI(rx_stop,$index)  \
                            $StateUI(rx_space,$index)
      }
    }
      
    return
  }


  #***************************************************************************
  #
  #        Name: set_user_chan_number
  #
  #  Parameters: tile_path - tile path of the invoking widget
  #
  # Description: Sets the User Defined channel number and configures the
  #              textvariables of the TX FREQ and RX FREQ entry fields.
  #
  #***************************************************************************
  proc set_user_chan_number { tile_path } \
  {
    #puts "set_user_chan_number:  $tile_path"
    
    variable  StateUI
    
    set tile_path  [get_tile  $tile_path]
    
    set chan  $StateUI(user_chan_no)
    
    # Determine if the present channel number is in the channel list.
    # If it is not, append the new value and then sort the list in
    # increasing order.
    if { [lsearch $StateUI(user_chan_list) $chan] == -1 } \
    {
      lappend StateUI(user_chan_list)  $chan
      
      set StateUI(user_chan_list)  [lsort -increasing $StateUI(user_chan_list)]
    }

    if { ![info exists  StateUI(user_tx_freq,$chan)] } \
    {
      set StateUI(user_tx_freq,$chan)  937.500000
    }
    
    ${tile_path}.ent_tx_freq  config  \
                 -textvariable  ::SzSnet_ConfigUI::StateUI(user_tx_freq,$chan)
                 
    if { ![info exists  StateUI(user_rx_freq,$chan)] } \
    {
      set StateUI(user_rx_freq,$chan)  898.500000
    }

    ${tile_path}.ent_rx_freq  config  \
                 -textvariable  ::SzSnet_ConfigUI::StateUI(user_rx_freq,$chan)
    
    return
  }
  
  
  #****************************************************************************
  # Name:        getSaveState
  #  Parameters:  none
  # Description: This function returns a list of TCL commands which can be used
  #              to set the SzSnet_Repeat_SimUI tiles to its current state
  #****************************************************************************
  proc getSaveState { } \
  {
    variable  StateUI

    set returnList { }

    # Save the UHF/VHF values
    foreach index { 0 1 2 } \
    {
      append returnList  "set ::SzSnet_ConfigUI::StateUI(tx_start,$index)  \
                                               {$StateUI(tx_start,$index)}\n"
                                                   
      append returnList  "set ::SzSnet_ConfigUI::StateUI(tx_stop,$index)  \
                                                {$StateUI(tx_stop,$index)}\n"
                                                   
      append returnList  "set ::SzSnet_ConfigUI::StateUI(tx_space,$index)  \
                                               {$StateUI(tx_space,$index)}\n"

                 
      append returnList  "set ::SzSnet_ConfigUI::StateUI(rx_start,$index)  \
                                               {$StateUI(rx_start,$index)}\n"
                                                   
      append returnList  "set ::SzSnet_ConfigUI::StateUI(rx_stop,$index)  \
                                                {$StateUI(rx_stop,$index)}\n"
                                                   
      append returnList  "set ::SzSnet_ConfigUI::StateUI(rx_space,$index)  \
                                               {$StateUI(rx_space,$index)}\n"
    }

    # Save the User Defined values
    append returnList  "set ::SzSnet_ConfigUI::StateUI(user_band)  \
                                                     {$StateUI(user_band)}\n"
                                                     
    append returnList  "set ::SzSnet_ConfigUI::StateUI(user_chan_list)  \
                                                {$StateUI(user_chan_list)}\n"

    append returnList  "set ::SzSnet_ConfigUI::StateUI(user_chan_no)  \
                                                  {$StateUI(user_chan_no)}\n"

    foreach { index } $StateUI(user_chan_list) \
    {
      if { [info exists  StateUI(user_tx_freq,$index)] } \
      {
        append returnList  \
                 "set ::SzSnet_ConfigUI::StateUI(user_tx_freq,$index)  \
                                           {$StateUI(user_tx_freq,$index)}\n"
      }

      if { [info exists  StateUI(user_rx_freq,$index)] } \
      {
        append returnList  \
                 "set ::SzSnet_ConfigUI::StateUI(user_rx_freq,$index)  \
                                           {$StateUI(user_rx_freq,$index)}\n"
      }
    }

    #puts "$returnList"
    
    return [subst -nocommands $returnList]
  }


  #****************************************************************************
  # Name:        Sync
  #  Parameters:  none
  # Description: Write state variables to SzSnet_Repeat_SimUI
  #****************************************************************************
  proc Sync { } \
  {
    eval [getSaveState]
  }


};  # End of SzSnet_ConfigUI namespace


#****************************************************************************
# This registers the tiles with the list for saving instrument states
#****************************************************************************
namespace eval ::SavedStates \
{
  variable tiles
  
  lappend tiles SzSnet_ConfigUI
}

