#############################################################################
#                                                                           #
# IFR SYSTEMS, INC    COPYRIGHT (C) 1998,99,2001,2002  DESIGN ENGINEERING   #
# ------------------------------------------------------------------------- #
#                                                                           #
# FILE        : 4000.Save_setup.tcl                                         #
#                                                                           #
# PRODUCT     : FITS                                                        #
#                                                                           #
# AUTHOR      : P. McCormick                                                #
#                                                                           #
# DESCRIPTION : Instrument Setup - Save User Interface                      #
#                                                                           #
#############################################################################



namespace eval Save_setup \
{
  # Global variables
  variable  LongName      "save_setup"
  variable  RecallScreen  ""
  variable  ShortName     "Save Setup"
  variable  Size          Quarter
            
  variable  tile_class   "SAVE_SETUP"

  variable  State
  
  set State(setup_num)           ""
  set State(save_inst_state)     1
  set State(save_tile_layout)    1
  
  set State(file_number)         1
  set State(destination)         "INTERNAL"
  set State(save_to)             { "INTERNAL"  "FLOPPY"  "STANDARD" }
  set State(focus_delay)         1000
  set State(user_password)       ""
  set State(save_running)        0

  variable overwrite

  array set overwrite {}

  variable  save_flag  0


  #***************************************************************************
  #
  #        Name: get_tile
  #
  #  Parameters: any child widget of the tile
  #
  # Description: Get the name of the tile's parent frame
  #
  #***************************************************************************
  proc get_tile { widget } \
  {
    while { 1 } \
    {
      switch -- [winfo class $widget] \
      {
        "SAVE_SETUP"   { return $widget }
      }
        
      set widget [winfo parent $widget]
    }

    return     
  }
 
  
  #***************************************************************************
  #
  #        Name: do_set_focus  
  #
  #  Parameters: tile_path
  #
  # Description: Set focus to Save_setup tile
  #
  #***************************************************************************
  proc do_set_focus { tile_path } \
  {
    variable State
    
    set current_focus  [focus]
    
    if { $current_focus != "${tile_path}.ent_file_number" } \
    {
      if { [catch {focus ${tile_path}.ent_file_number} ] } \
      {
        after $State(focus_delay) [list ::Save_setup::do_set_focus $tile_path]
      }
    } 
    
    return
  }


  #***************************************************************************
  #
  #        Name: start_set_focus  
  #
  #  Parameters: tile_path
  #
  # Description: Start to set focus to Save_setup tile
  #
  #***************************************************************************
  proc start_set_focus { tile_path } \
  {
    do_set_focus $tile_path
  }


  #***************************************************************************
  #
  #        Name: display_save  
  #
  #  Parameters:
  #
  # Description: 
  #
  #***************************************************************************
  proc display_save { tile_path  { recall_screen  "" } } \
  {
    #puts "display_save:  $tile_path,  $recall_screen"
    
    variable  LongName
    variable  RecallScreen
    variable  tile_class
    variable  save_flag
    variable  State
    
    if { $save_flag == -1 } \
    {
       return  $save_flag
    }

    set RecallScreen  $recall_screen
    
    # Initialize the 'save_flag'
    set save_flag  -1
    
    set top_level  [winfo toplevel $tile_path]
    
    set tile_path  [name_child  $top_level  $LongName]

    if { ![winfo exists $tile_path] } \
    {
      set tile_path  [Screens::create_toplevel_tile  $tile_path  SAVE_SETUP]
    }  
    
    # Bind the F8 key for a return
    bind $tile_path  <KeyRelease-F8>  "destroy $tile_path"
    bind $tile_path  <<Destroy>>      "Save_setup::save_cancel  $tile_path"

    # Clear the password field
    set State(user_password)  ""

    # Synchronize the Setup and Recall file numbers
    set State(file_number)  $::Recall_SetupsUI::State(file_number)
    
    set_mode  $tile_path

    after idle  "::Save_setup::forceFocus ${tile_path}.ent_file_number"

    # Wait for user input:  OK or CANCEL
    while { $save_flag == -1 } { vwait Save_setup::save_flag }

    #puts "save_flag:  $save_flag"
    
    destroy  $tile_path

    # Clear the password field
    set State(user_password)  ""

    return  $save_flag
  }

  #***************************************************************************
  #
  #  Name: forceFocus
  #
  #  Parameters: 
  #
  # Description: 
  #
  #***************************************************************************
  proc forceFocus { tile_path } \
  {
     if { ![winfo exists $tile_path] } \
     {
        return
     }
     focus -force $tile_path
  }

  #***************************************************************************
  #
  #        Name: save_cancel
  #
  #  Parameters: 
  #
  # Description: 
  #
  #***************************************************************************
  proc save_cancel { tile_path } \
  {
    variable  save_flag
    
    set save_flag  0

    return
  }


  #***************************************************************************
  #
  #        Name: save
  #
  #  Parameters:
  #
  # Description: Save the setup
  #
  #***************************************************************************
  proc save { widget } \
  {
     variable  State
    
     if {$State(save_running) == 1} {return}
     set State(save_running) 1
     after 500 "::Save_setup::save_core $widget; set ::Save_setup::State(save_running) 0"
  }

  proc save_core { widget } \
  {
    variable  overwrite
    variable  State
    
    set tile_path  [get_tile  $widget]

    switch -- $State(destination) \
    {
      "STANDARD"  { set save_dir      $::Recall_SetupsUI::State(std_save_dir)  
                    set save_sub_dir  $::Recall_SetupsUI::State(std_save_sub_dir)
               
                    # If the Save setup password is null, 
                    # prompt user to enter a password
                    # then return from the save process to try again
                    if { $State(user_password) == "" } \
                    {
                      display_password_msg  $widget  "PASSWORD REQUIRED"
        
                      return
                    }
                  }

      "FLOPPY"    { set save_dir      $::Recall_SetupsUI::State(ext_save_dir)
                    set save_sub_dir  $::Recall_SetupsUI::State(ext_save_sub_dir) }

      "INTERNAL"  -
      default     { set save_dir      $::Recall_SetupsUI::State(int_save_dir)  
                    set save_sub_dir  $::Recall_SetupsUI::State(int_save_sub_dir) }
    }

    set State(fname)  "SAVE$State(file_number).SAV"

    # Compose full file name
    set full_name  [file join  "$save_dir"  "$save_sub_dir"  "$State(fname)"]
    
    if { [file  exist  $full_name] } \
    {
      # Read the setup information
      SavedStates::read_info  $full_name  overwrite

      if { $State(destination) == "STANDARD" } \
      {
        # If the setup file password is not null,
        # compare the setup file password to the user entered password
        # If they do not match, return
        if { $overwrite(password) != "" } \
        {
          if { $overwrite(password) != $State(user_password) } \
          {
            display_password_msg  $widget  "PASSWORD INCORRECT"
        
            # Clear the present password
            set State(user_password)  ""
        
            return
          }
        }
      }

      # Set up the overwrite prompt box
      set prompt  [name_child  $widget  overwrite]

      if { [winfo exists $prompt] } \
      {
         #puts "Exit because window $prompt already exists"
         return
      }

      Screens::create_toplevel_tile  $prompt  SAVE_OVERWRITE

      return
    }

    # Write the setup to the file
    do_save $widget
    
    return
  }

  
  #***************************************************************************
  #
  #        Name: do_save
  #
  #  Parameters:
  #
  # Description: Save the setup (does NOT check for overwrite!)
  #
  #***************************************************************************
  proc do_save { widget } \
  {
    variable  RecallScreen
    variable  State
    variable  save_flag

    set tile_path  [get_tile $widget]

    switch -- $State(destination) \
    {
      "STANDARD"  { set save_dir      $::Recall_SetupsUI::State(std_save_dir)
                    set save_sub_dir  $::Recall_SetupsUI::State(std_save_sub_dir) }

      "FLOPPY"    { set save_dir      $::Recall_SetupsUI::State(ext_save_dir)
                    set save_sub_dir  $::Recall_SetupsUI::State(ext_save_sub_dir) }

      "INTERNAL"  -
      default     { set save_dir      $::Recall_SetupsUI::State(int_save_dir)
                    set save_sub_dir  $::Recall_SetupsUI::State(int_save_sub_dir) }
    }

    set save_dir   [file join  "$save_dir"  "$save_sub_dir"]
    
    set full_name  [file join  "$save_dir"  "$State(fname)"]
    
    # Set the file information
    # and the password on the setup file
    append list  [SavedStates::save_info  $State(user_password)] 
    
    if { $State(save_tile_layout) } \
    {
        append state "\nScreens::Clear .\n"
    }
    if { $State(save_inst_state) } \
    {
      append state  [SavedStates::save_engines] 
    }
    
    if { $State(save_tile_layout) } \
    { 
      append state  [SavedStates::save_ui  $RecallScreen]
    }
    
    if [info exists state] \
    {
        # older versions of firmware didn't support "-nowait"  - so escape it...
        regsub -all -- "-nowait" $state {$::SavedStates::nowait} state
        
        append list {if ![info exists ::SavedStates::nowait] {set ::SavedStates::nowait -forgive}}
        
        append list $state
    }
    
    # Insure our save location exists.
    file  mkdir  $save_dir
 
    set fid  [open $full_name w]

    puts  $fid  $list

    close  $fid
    
    set save_flag  1

    return
  }


  #***************************************************************************
  #
  #        Name: abort_overwrite
  #
  #  Parameters: button widget
  #
  # Description: Cancel overwrite
  #
  #***************************************************************************
  proc abort_overwrite { widget } \
  {
    after idle "destroy [winfo toplevel $widget]"
  }  


  #***************************************************************************
  #
  #        Name: set_mode
  #
  #  Parameters: tile_path - tile path of the invoking widget
  #
  # Description: Displays the password entry if the STANDARD mode is 
  #              selected
  #
  #***************************************************************************
  proc set_mode { tile_path } \
  {
    variable  State
    
    set tile_path  [get_tile  $tile_path]
    
    # Remove the password entry from the display
    grid remove  ${tile_path}.lbl_password
    grid remove  ${tile_path}.ent_password

    grid remove  ${tile_path}.lbl_dir
    grid remove  ${tile_path}.ent_ext_sub_dir
    grid remove  ${tile_path}.ent_int_sub_dir
    
    # Set the appropriate directory to the user entered directory
    switch -- $State(destination) \
    {
      "STANDARD"  { # Display the password entry
                    Screens::position_widget  ${tile_path}.lbl_password
                    Screens::position_widget  ${tile_path}.ent_password }

      "FLOPPY"    { Screens::position_widget  ${tile_path}.lbl_dir
                    Screens::position_widget  ${tile_path}.ent_ext_sub_dir }
                  
      "INTERNAL"  -
      default     { Screens::position_widget  ${tile_path}.lbl_dir
                    Screens::position_widget  ${tile_path}.ent_int_sub_dir }
    }

    # Display the change on the menubutton
    update idle

    return
  }  


  #***************************************************************************
  #
  #        Name: display_password_msg
  #
  #  Parameters: widget  - tile path of the invoking widget
  #              message - message to display on the toplevel
  #
  # Description: Displays the password message toplevel
  #
  #***************************************************************************
  proc display_password_msg { widget  message } \
  {
    variable  State
    variable  last_focus
    
    set last_focus  $widget    

    set tile_path  [get_tile  $widget]
    
    set top_level  [winfo toplevel $tile_path]
   
    set tile_path  [name_child  $top_level  save_password_msg]

    if { ![winfo exists $tile_path] } \
    {
      set tile_path  [Screens::create_toplevel_tile  $tile_path  SAVE_PASSWORD_MSG]
    }  
    
    ${tile_path}.lbl_message  config  -text "$message"
    
    bind  $tile_path  <<Destroy>>  "Save_setup::destroy_password_msg  $tile_path"
    
    after idle  "focus  -force  ${tile_path}.btn_ok"
    
    return
  }  


  #***************************************************************************
  #
  #        Name: destroy_password_msg
  #
  #  Parameters: tile_path - tile path of the invoking widget
  #
  # Description: Handles the <<Destroy>> event for the password message 
  #              toplevel
  #
  #***************************************************************************
  proc destroy_password_msg { tile_path } \
  {
    variable  State
    variable  last_focus
    
    after idle  "focus  -force  $last_focus"
    
    return
  }


  #***************************************************************************
  #
  #        Name: format_directory
  #
  #  Parameters: 
  #
  # Description: Formats the user entered directory by removing leading and
  #              trailing space and replaces any spaces in the name with
  #              underscores
  #
  #***************************************************************************
  proc format_directory { } \
  {
    #puts "format_directory"
    
    variable  State
    
    # Set the appropriate directory to the user entered directory
    switch -- $State(destination) \
    {
      "STANDARD"  { # Display the 'standard' directory name,
                    # but do not change it
                    set ::Recall_SetupsUI::State(std_save_sub_dir)  "standard" }

      "FLOPPY"    { # Do not allow the "/..", "/.", "." or ".." directory entry
                    if { [string match  "*.*"  $::Recall_SetupsUI::State(ext_save_sub_dir)] } \
                    {
                      set ::Recall_SetupsUI::State(ext_save_sub_dir)  ""
                      
                      return
                    }
                    
                    # Trim off whitespace
                    set ::Recall_SetupsUI::State(ext_save_sub_dir)  \
                           [string trim  $::Recall_SetupsUI::State(ext_save_sub_dir)]
                    
                    # Trim off '/'
                    set ::Recall_SetupsUI::State(ext_save_sub_dir)  \
                      [string trim  $::Recall_SetupsUI::State(ext_save_sub_dir)  "/"]
                  }


      "INTERNAL"  -
      default     { # Do not allow the "/..", "/.", "." or ".." directory entry
                    if { [string match  "*.*"  $::Recall_SetupsUI::State(int_save_sub_dir)] } \
                    {
                      set ::Recall_SetupsUI::State(int_save_sub_dir)  ""
                      
                      return
                    }
                    
                    # Trim off whitespace
                    set ::Recall_SetupsUI::State(int_save_sub_dir)  \
                            [string trim  $::Recall_SetupsUI::State(int_save_sub_dir)]
                    
                    # Trim off '/'
                    set ::Recall_SetupsUI::State(int_save_sub_dir)  \
                       [string trim  $::Recall_SetupsUI::State(int_save_sub_dir)  "/"]

                  }
    }

    return
  }
  

};  # End of Save_setup namespace


