#############################################################################
#                                                                           #
# IFR SYSTEMS, INC    COPYRIGHT (C) 1998,99,2001,2002  DESIGN ENGINEERING   #
# ------------------------------------------------------------------------- #
#                                                                           #
# FILE        : 4011.Recall_setups.tcl                                      #
#                                                                           #
# PRODUCT     : FITS                                                        #
#                                                                           #
# AUTHOR      : P. McCormick                                                #
#                                                                           #
# DESCRIPTION : Instrument Setup - Save and Recall User Interface,          #
#                                  Full tile                                #
#                                                                           #
#############################################################################


namespace eval Recall_SetupsUI \
{
  # Global variables
  variable  LongName      "recall_setups"
  variable  RecallScreen  ""
  variable  ShortName     "Save/Recall Setups"
  variable  Size          Full
            
  variable  tile_class   "RECALL_SETUPS"

  variable  save_dir      ""
  variable  save_sub_dir  ""
  variable  save_gen_freq
  variable  save_rec_freq
  variable  save_m2   { }

  variable  State
  
  set State(int_save_dir)  "/usr/setups/"
  set State(ext_save_dir)  "/fd0"
  set State(std_save_dir)  "/usr/setups/standard/"
  
  set State(int_save_sub_dir)  ""
  set State(ext_save_sub_dir)  ""
  set State(std_save_sub_dir)  ""
  
  set State(file_number)        1
  set State(source)             "INTERNAL"
  set State(restore_from)       { "INTERNAL"  "FLOPPY"  "STANDARD" }  

  # Local State variables
  set State(num_files)     0
  set State(curr_select)   0
  set State(scroll_on)     0
  
  set State(no_change_freq)  0
  set State(no_change_m2)    0
  
  set State(delete_msg)  "DELETE FILE?"

  set State(quick_file_number)  1

  set State(label)        "";  # Label for the Quick Recall
  set State(label,valid)  black
  
  set State(password)         ""
  set State(user_password)    ""
  set State(password_verify)  0
  set State(recall_in_process) 0
  


  #***************************************************************************
  #
  #        Name: Create  
  #
  #  Parameters:
  #
  # Description: 
  #
  #***************************************************************************
  proc Create { tile_path } \
  {
    Debug::Puts RECALL 10 "Recall_SetupsUI::Create:  $tile_path"
    
    variable  LongName
    variable  RecallScreen
    variable  tile_class
    
    # Get the current screen before displaying the Recall Setups screen
    set RecallScreen  $::Screens::Local_screen

    set top_level  [winfo toplevel  $tile_path]
   
    set tile_path  [name_child  $top_level  $LongName]

    if { ![winfo exists $tile_path] } \
    {
      set tile_path  [Screens::create_tile  $tile_path  $LongName  $tile_class]
    }  
     
    return  $tile_path
  }


  #***************************************************************************
  #
  #        Name: Map
  #
  #  Parameters: 
  #
  # Description: Initializes the tile
  #
  #***************************************************************************
  proc Map { tile_path } \
  {
    Debug::Puts RECALL 10 "Recall_SetupsUI::Map:  $tile_path"
    
    variable  State
    
    set tile_path  [get_tile  $tile_path]
    
    set State(scroll_on)       0
#    set State(no_change_freq)  0
    
    ${tile_path}.lst_setups config  \
                               -yscrollcommand "${tile_path}.scrl_setups  set"

    ${tile_path}.scrl_setups config  -command "${tile_path}.lst_setups  yview"

    # Softkey #2:  RECALL
    View::AssignSoftkey  $tile_path  2  "RECALL\nSETUP"  \
                                "Recall_SetupsUI::listbox_recall  $tile_path"

    # Softkey #3:  SAVE SETUP
    View::AssignSoftkey  $tile_path  3  "SAVE\nSETUP"  \
                                    "Recall_SetupsUI::save_setup  $tile_path"

    # Softkey #4:  BACKUP
    View::AssignSoftkey  $tile_path  4  "BACKUP\nSETUPS"  \
                                  "Recall_SetupsUI::backup_setup  $tile_path"

    # Softkey #6:  DELETE
    View::AssignSoftkey  $tile_path  6  "DELETE"  \
                                   "Recall_SetupsUI::delete_file  $tile_path"

    # Softkey #5:  CHANGE DIR (set in procedure 'set_file_source_dir')

    # Softkey #5:  PREV DIR (set in procedure 'set_file_source_dir')

    # Softkey #6:  DELETE (set in procedure 'set_file_source')
      
    # Get the initial file list
    set_file_source  $tile_path


    trace variable ::Recall_SetupsUI::State(no_change_freq) w  \
                            "Recall_SetupsUI::update_change_freq  $tile_path"
    
    update_change_freq  $tile_path



    trace variable ::Recall_SetupsUI::State(scroll_on) w  \
                                 "Recall_SetupsUI::update_scroll  $tile_path"
                                  
    update_scroll  $tile_path
    
    # Indicate the current active selection on the listbox
    ${tile_path}.lst_setups  activate  $State(curr_select)
    
    return
  }


  #***************************************************************************
  #
  #        Name: Cleanup
  #
  #  Parameters: 
  #
  # Description: 
  #
  #***************************************************************************
  proc Cleanup { tile_path } \
  {
    variable  State
    
    trace vdelete ::Recall_SetupsUI::State(no_change_freq) w  \
                            "Recall_SetupsUI::update_change_freq  $tile_path"

    trace vdelete ::Recall_SetupsUI::State(scroll_on) w  \
                                 "Recall_SetupsUI::update_scroll  $tile_path"

    # Set the source to INTERNAL so that the user does not have to 
    # wait for a floppy read when the tile is redisplayed
    set State(source)  "INTERNAL"
    
    return
  }


  #***************************************************************************
  #
  #        Name: Unmap
  #
  #  Parameters: 
  #
  # Description: 
  #
  #***************************************************************************
  proc Unmap { tile_path } \
  {
    Cleanup  $tile_path
    
    return
  }


  #***************************************************************************
  #
  #        Name: Destroy
  #
  #  Parameters: 
  #
  # Description: 
  #
  #***************************************************************************
  proc Destroy { tile_path } \
  {
    Cleanup  $tile_path

    return
  }


  #***************************************************************************
  #
  #        Name: get_tile
  #
  #  Parameters: any child widget of the tile
  #
  # Description: Get the name of the tile's parent frame
  #
  #***************************************************************************
  proc get_tile { widget } \
  {
    while { 1 } \
    {
      switch -- [winfo class $widget] \
      {
        "RECALL_SETUPS"   { return $widget }
        "IFR-2975"        { return {}      }
      }
        
      set widget [winfo parent $widget]
    }

    return     
  }


  #***************************************************************************
  #
  #        Name: set_file_source
  #
  #  Parameters: tile_path   - tile path of the invoking widget
  #
  # Description: Set the drive source of the files
  #
  #***************************************************************************
  proc set_file_source { tile_path } \
  {
    Debug::Puts RECALL 10 "set_file_source:  $tile_path"
    
    variable  State
  
    set tile_path  [get_tile  $tile_path]
    
    # Remove the DELETE verify fields
    grid remove  ${tile_path}.lbl_delete
    grid remove  ${tile_path}.btn_yes
    grid remove  ${tile_path}.btn_no
    
    # Remove the PASSWORD verify fields
    grid remove  ${tile_path}.lbl_password
    grid remove  ${tile_path}.ent_password
    grid remove  ${tile_path}.btn_password_cancel

    # Remove the sub-directory fields
    grid remove  ${tile_path}.lbl_dir
    grid remove  ${tile_path}.ent_std_sub_dir
    grid remove  ${tile_path}.ent_int_sub_dir
    grid remove  ${tile_path}.ent_ext_sub_dir
    
    # Set the appropriate directory to the user entered directory
    switch -- $State(source) \
    {
      "STANDARD"  { Screens::position_widget  ${tile_path}.lbl_dir
                    Screens::position_widget  ${tile_path}.ent_std_sub_dir }

      "FLOPPY"    { Screens::position_widget  ${tile_path}.lbl_dir
                    Screens::position_widget  ${tile_path}.ent_ext_sub_dir }
                  
      "INTERNAL"  -
      default     { Screens::position_widget  ${tile_path}.lbl_dir
                    Screens::position_widget  ${tile_path}.ent_int_sub_dir }
    }
    
    # Display the change on the menubutton
    update idle

    set_file_source_dir  $tile_path
    
    get_file_list  $tile_path

    # Initially display the files listed in order by the filename
    list_by_filenumber  $tile_path
    
    return
  }
  
  
  #***************************************************************************
  #
  #        Name: set_file_source_dir
  #
  #  Parameters: tile_path   - tile path of the invoking widget
  #
  # Description: Set the source directory of the files
  #
  #***************************************************************************
  proc set_file_source_dir { tile_path } \
  {
    Debug::Puts RECALL 10 "set_file_source_dir:  $tile_path"
    
    variable  State
  
    set tile_path  [get_tile  $tile_path]
    
    # Softkey #5:  Clear CHANGE DIR / PREV DIR Softkey
    View::AssignSoftkey  $tile_path  5  ""  ""

    # Set the appropriate directory to the user entered directory
    switch -- $State(source) \
    {
      "STANDARD"  { # Do not allow the "/..", "/.", "." or ".." directory entry
                    if { [string match  "*.*"  $State(std_save_sub_dir)] } \
                    {
                      set State(std_save_sub_dir)  ""
                      
                      return
                    }
                    
                    # Trim off whitespace
                    set State(std_save_sub_dir)  [string trim  $State(std_save_sub_dir)]
                    
                    # Trim off '/'
                    set State(std_save_sub_dir)  [string trim  $State(std_save_sub_dir)  "/"]

                    # Do not allow the "/" in the directory entry
                    if { [string match  "*/*"  $State(std_save_sub_dir)] } \
                    {
                      set State(std_save_sub_dir)  ""
                      
                      return
                    }

                    if { $State(std_save_sub_dir) == "" } \
                    {
                      # In the '/usr/setups/standard/' directory
                      # Enable changing to a sub-directory
                      # Disable changing to the previous directory
                      # Softkey #5:  CHANGE DIR
                      View::AssignSoftkey  $tile_path  5  "CHANGE DIR"  \
                                    "Recall_SetupsUI::change_dir  $tile_path"
                    } \
                    else \
                    {
                      # In the '/usr/setups/standard/<sub-directory>/' directory
                      # Disable changing to a sub-directory
                      # Enable changing to the previous directory
                      # Softkey #5:  PREVIOUS DIR
                      View::AssignSoftkey  $tile_path  5  "PREV\nDIR"  \
                                    "Recall_SetupsUI::return_dir  $tile_path"
                    }
                  }

      "FLOPPY"    { # Do not allow the "/..", "/.", "." or ".." directory entry
                    if { [string match  "*.*"  $State(ext_save_sub_dir)] } \
                    {
                      set State(ext_save_sub_dir)  ""
                      
                      return
                    }
                    
                    # Trim off whitespace
                    set State(ext_save_sub_dir)  [string trim  $State(ext_save_sub_dir)]
                    
                    # Trim off '/'
                    set State(ext_save_sub_dir)  [string trim  $State(ext_save_sub_dir)  "/"]

                    # Do not allow the "/" in the directory entry
                    if { [string match  "*/*"  $State(ext_save_sub_dir)] } \
                    {
                      set State(ext_save_sub_dir)  ""
                      
                      return
                    }

                    if { $State(ext_save_sub_dir) == "" } \
                    {
                      # In the '/fd0/' directory
                      # Enable changing to a sub-directory
                      # Disable changing to the previous directory
                      # Softkey #5:  CHANGE DIR
                      View::AssignSoftkey  $tile_path  5  "CHANGE DIR"  \
                                    "Recall_SetupsUI::change_dir  $tile_path"
                    } \
                    else \
                    {
                      # In the '/fd0/<sub-directory>/' directory
                      # Disable changing to a sub-directory
                      # Enable changing to the previous directory
                      # Softkey #5:  PREVIOUS DIR
                      View::AssignSoftkey  $tile_path  5  "PREV\nDIR"  \
                                    "Recall_SetupsUI::return_dir  $tile_path"
                    }
                  }


      "INTERNAL"  -
      default     { # Do not allow the "/..", "/.", "." or ".." directory entry
                    if { [string match  "*.*"  $State(int_save_sub_dir)] } \
                    {
                      set State(int_save_sub_dir)  ""
                      
                      return
                    }
                    
                    # Trim off whitespace
                    set State(int_save_sub_dir)  [string trim  $State(int_save_sub_dir)]
                    
                    # Trim off '/'
                    set State(int_save_sub_dir)  [string trim  $State(int_save_sub_dir)  "/"]

                    # Do not allow the "/" in the directory entry
                    if { [string match  "*/*"  $State(int_save_sub_dir)] } \
                    {
                      set State(int_save_sub_dir)  ""
                      
                      return
                    }

                    if { $State(int_save_sub_dir) == "" } \
                    {
                      # In the '/usr/setups/' directory
                      # Enable changing to a sub-directory
                      # Disable changing to the previous directory
                      # Softkey #5:  CHANGE DIR
                      View::AssignSoftkey  $tile_path  5  "CHANGE DIR"  \
                                    "Recall_SetupsUI::change_dir  $tile_path"
                    } \
                    else \
                    {
                      # In the '/usr/setups/<sub-directory>/' directory
                      # Disable changing to a sub-directory
                      # Enable changing to the previous directory
                      # Softkey #5:  PREVIOUS DIR
                      View::AssignSoftkey  $tile_path  5  "PREV DIR"  \
                                    "Recall_SetupsUI::return_dir  $tile_path"
                    }
                  }
    }

    return
  }


  #***************************************************************************
  #
  #        Name: get_file_list
  #
  #  Parameters: tile_path   - tile path of the invoking widget
  #
  # Description: Get a list of file names and file labels
  #
  #***************************************************************************
  proc get_file_list { tile_path } \
  {
    Debug::Puts RECALL 10 "get_file_list:  $tile_path"
    
    variable  save_dir
    variable  save_sub_dir
    variable  State
  
    set tile_path  [get_tile  $tile_path]

    set list_box  ${tile_path}.lst_setups
        
    # Save the initial directory
    set pwd  [pwd]
    
    # Clear the listbox
    $list_box  delete  0  end
    
    # Set the appropriate directory to the user entered directory
    switch -- $State(source) \
    {
      "STANDARD"  { set save_dir      $State(std_save_dir)
                    set save_sub_dir  $State(std_save_sub_dir) }

      "FLOPPY"    { set save_dir      $State(ext_save_dir)
                    set save_sub_dir  $State(ext_save_sub_dir) }

      "INTERNAL"  -
      default     { set save_dir      $State(int_save_dir)
                    set save_sub_dir  $State(int_save_sub_dir) }
    }

    # Setup the save directory
    set save_dir  [file join  "$save_dir"  "$save_sub_dir"]
    
    # Change to the save directory, catch the error
    if { [catch { cd  $save_dir } err] } \
    {
      return
    }
    
    set index  0
    
    # Get the file name of each file in the directory
    foreach { filename } [glob -nocomplain *] \
    {
      # If the file is a directory, display as a directory
      if { [file isdirectory  $filename] } \
      {
        # Skip displaying the STANDARD directory
        if { $filename == "standard" || $filename == "STANDARD" } \
        {
          continue
        }
        
        set State($index,f_name)  $filename
        
        set State($index,f_number)  -1
        
        set State($index,f_path)  [file join  $save_dir  $filename]
        
        set State($index,f_label)  "<DIR>"
        
        set State($index,f_date)  ""

        set State($index,f_password)  ""
      } \
      else \
      {      
        # Display only '.SAV' files
        if ![string match  "*.SAV"  $filename] \
        {
           # don't know what it is, don't care, not going to show it
           continue
        }
        # The file is a Setup file, parse it
        set State($index,f_name)  $filename
          
        # Get the number of the file
        set i  [expr [string first  "SAVE"  $filename] + 4]
        set j  [expr [string first  ".SAV"  $filename] - 1]
          
        set State($index,f_number)  [string  range  $filename  $i  $j]
         
        # Set full path for this file
        set file_path  [file join  $save_dir  $filename]
          
        # Store the file path in an array
        set State($index,f_path)  $file_path
          
        # Get the label and time for each file
        SavedStates::read_info  $file_path  temp
          
        if { $temp(label) == "" } \
        {
          set temp(label)  "<no label>"
        }
          
        # Limit the display label to 23 characters
        if { [string length  $temp(label)] > 23 } \
        {
          set temp(label)  [string range  $temp(label)  0  22]
        }
          
        # Store the file label in an array
        set State($index,f_label)  $temp(label)
          
        # Get the date only from the file time
        foreach  { weekday  month  day  time  year }  $temp(ftime)  {}
          
        set temp(date)  "$month $day, $year"
          
        # Store the file date in an array
        set State($index,f_date)  $temp(date)

        # Store the file password in an array
        # Check for a password for older setups
        if { ![info exists  temp(password)] } \
        {
            set temp(password)  ""
        }
          
        set State($index,f_password)  $temp(password)
      }

      set temp_string  [format  "%-15s %-24s %-15s"  \
              "$filename"  "$State($index,f_label)"  "$State($index,f_date)"]
      
      # Place the file information into the listbox
      $list_box  insert  end  $temp_string

      incr index
    }
    
    set State(num_files)  $index
    
    # Return to the initial directory
    cd  $pwd

    return
  }


  #***************************************************************************
  #
  #        Name: listbox_recall
  #
  #  Parameters: widget - tile path of the invoking widget
  #
  # Description: Recall of a setup selected in the listbox
  #
  #***************************************************************************
  proc listbox_recall { widget } \
  {
    Debug::Puts RECALL 10 "listbox_recall:  $widget"

    variable  State
    
    set tile_path  [get_tile  $widget]

    set list_box  ${tile_path}.lst_setups

    # Get the active selection
    set index  [$list_box  index  active]
    
    if ![info exists State($index,f_path)] { return }
    
    # Remove the tile to avoid multiple setup recalls
    grid remove  $tile_path
    
    # Determine if the file path variable is valid
    if { ![info exists  State($index,f_path)] } \
    {
      return
    }
    
    # Determine if the file path is valid
    if { $State($index,f_path) == "" } \
    {
      return
    }
    # make sure this "number" really is a number
    regexp {[0-9]+} $State($index,f_number) n
    # Update the Save Setup / Recall Setup file number
    set State(file_number)  $n
    
    # Update the Quick Recall number
    set State(quick_file_number)  $n

    # Update the Quick Recall label
    set State(label)  $State($index,f_label)

    set file_path  $State($index,f_path)
    
    if [file isfile $file_path] \
    {
    
       recall  $tile_path  $file_path
    
    }
  }


  #***************************************************************************
  #
  #        Name: quick_recall
  #
  #  Parameters: widget - tile path of the invoking widget
  #
  # Description: Recall the setup selected in the main screen title bar
  #
  #***************************************************************************
  proc quick_recall { widget  { force_drive  {} } } \
  {
    Debug::Puts RECALL 10 "quick_recall:  $widget"

    variable  save_dir
    variable  save_sub_dir
    variable  State
    
    if { $force_drive != "" } \
    {
      set save_dir  $State(int_save_dir)
    } \
    else \
    {
      switch -- $State(source) \
      {
        "STANDARD"  { set save_dir      $State(std_save_dir)
                      set save_sub_dir  $State(std_save_sub_dir) }

        "FLOPPY"    { set save_dir      $State(ext_save_dir)
                      set save_sub_dir  $State(ext_save_sub_dir) }

        "INTERNAL"  -
        default     { set save_dir      $State(int_save_dir)
                      set save_sub_dir  $State(int_save_sub_dir) }
      }
    }
    
    set file_name  "SAVE$State(quick_file_number).SAV"
    
    # Set full path for this file
    set file_path  [file join  "$save_dir"  "$save_sub_dir"  "$file_name"]
    
    recall  $widget  $file_path
    
    return
  }


  #***************************************************************************
  #
  #        Name: get_file_info
  #
  #  Parameters:
  #
  # Description: Get the information from the given file number
  #
  #***************************************************************************
  proc get_file_info { { force_drive  {} } } \
  {
    Debug::Puts RECALL 10 "get_file_info:  $force_drive"

    variable  save_dir
    variable  save_sub_dir
    variable  State
  
    if { $force_drive != "" } \
    {
      set save_dir  $State(int_save_dir)       
    } \
    else \
    {
      switch -- $State(source) \
      {
        "STANDARD"  { set save_dir      $State(std_save_dir)
                      set save_sub_dir  $State(std_save_sub_dir) }

        "FLOPPY"    { set save_dir      $State(ext_save_dir)
                      set save_sub_dir  $State(ext_save_sub_dir) }

        "INTERNAL"  -
        default     { set save_dir      $State(int_save_dir)
                      set save_sub_dir  $State(int_save_sub_dir) }
      }
    }
    
    set file_name  "SAVE$State(quick_file_number).SAV"
    
    # Set full path for this file
    set file_path  [file join  "$save_dir"  "$save_sub_dir"  "$file_name"]
    
    # Set the Quick Recall label
    SavedStates::read_info  $file_path  ::Recall_SetupsUI::State
    
    return
  }
  # make the value update if the quick recall value is updated
  
  OnChange hook ::Recall_SetupsUI::State(quick_file_number) "Recall_SetupsUI::get_file_info  1"
  
  #***************************************************************************
  #
  #        Name: recall
  #
  #  Parameters: widget    - tile path of the invoking widget
  #              file_path - path of the setup file
  #
  # Description: Recall the setup selected in the listbox
  #
  #***************************************************************************
  proc recall { widget  file_path } \
  {
    variable  save_gen_freq
    variable  save_rec_freq
    variable  save_m2
    variable  State
    
    Debug::Puts RECALL 10 "recall:  $widget, $file_path"
    if {$::busy_count > 0 } { return }

    if { $State(recall_in_process) == 1 } { return }

    # Capture any data which is needed to simulate the effect of not
    # reloading part of the script file
    #    Save current Receiver frequency and Generator frequency
    if { $State(no_change_freq) } \
    {
      set save_gen_freq  $::Generator::State(freq)
      set save_rec_freq  $::Receiver::State(freq_raw)
      
      Debug::Puts RECALL 10 "  gen_freq:  $save_gen_freq"
      Debug::Puts RECALL 10 "  rec_freq:  $save_rec_freq"
    }

    #    Save current M2 information
    if { $State(no_change_m2) } \
    {
#       set save_m2 [eval capture_m2_data]
    } else \
    {
       set save_m2 { }
    }

    # start loading the state in the background - won't execute till we idle
    if ![file exists $file_path] { return }
    set State(recall_in_process) 1
    set ::Slave::local_lockout RECALL
    BGSource $file_path
    Busy
    Engmblink 2

    set cmd [namespace code "recall_finish $widget"]
    #after state has executed, finish up
    BGCommand "after 1000 $cmd"
  }

  proc recall_finish { widget } \
  {
    variable  save_gen_freq
    variable  save_rec_freq
    variable  save_m2
    variable  State

    Debug::Puts RECALL 10 "recall_finish:  $widget"
    SavedStates::Finalize
    # If this is a remote screen, then force a redisplay on it
    set main_screen  [Screens::Get_main_screen  $widget]
    
    if { $main_screen != "." } \
    {
      BGCommand "Screens::Show  $main_screen  $::Screens::Local_screen"
    }
    
    # Restore current Receiver frequency and Generator frequency
    if { $State(no_change_freq) } \
    {
      Debug::Puts RECALL 10 "  gen_freq:  $save_gen_freq"
      Debug::Puts RECALL 10 "  rec_freq:  $save_rec_freq"

      BGCommand "set ::ReceiverUI::StateUI(mode) FREQ"
      BGCommand "Generator freq  $save_gen_freq  MHz  -nowait"
      BGCommand "Receiver  freq  $save_rec_freq  MHz  -nowait"
    }
    if { $State(no_change_m2) } \
    {
#       BGCommand $save_m2
    } 

    BGCommand "set ::Slave::local_lockout {}"

    BGCommand "NotBusy"
    BGCommand "Engmblink 0"
    BGCommand "safe_focus_force $widget"
    BGCommand "set ::Recall_SetupsUI::State(recall_in_process) 0"
  }

  #***************************************************************************
  #  Name:        capture_m2_data
  #  Parameters:  none
  #  Description: This generates a TCL list which captures the state of FGEN 4
  #     allowing it to be restored after a reload
  #***************************************************************************
  proc capture_m2_data { } \
  {
     set returnList {}

     # Save the array
     foreach { mode } { TONE TREM DTMF CTCSS DCS TSIG P25 SINC ARB } \
     {
        foreach { item } [array names ::FGen::State 4,$mode,*] \
        {
           append returnList "set ::FGen::State($item) {$::FGen::State($item)}\n"
        }
     }
     append returnList  "FGen::setMode  4  $::FGen::State(4,fgen_mode_value)"
    
     return [subst -nocommands $returnList]
  }

  #***************************************************************************
  #
  #        Name: delete_file
  #
  #  Parameters: widget - tile path of the invoking widget
  #
  # Description: Delete the setup file
  #
  #***************************************************************************
  proc delete_file { widget } \
  {
    Debug::Puts RECALL 10 "delete_file:  $widget"
    
    variable  State
    
    set tile_path  [get_tile  $widget]
    
    set list_box  ${tile_path}.lst_setups

    # Get the active selection
    set index  [$list_box  index  active]
    
    if ![info exists State($index,f_label)] { return }
    
    # Determine the verification prompt depending on whether
    # the file is a directory or a file
    if { $State($index,f_label) == "<DIR>" } \
    {
      set State(delete_msg)  "DELETE DIRECTORY?"
    } \
    else \
    {
      set State(delete_msg)  "DELETE FILE?"

      # Request a password for the deleting files from the STANDARD directory
      # or sub-directories
      if { $State(source) == "STANDARD" } \
      {
        # Prompt the user for a password
        Screens::position_widget  ${tile_path}.lbl_password
        Screens::position_widget  ${tile_path}.ent_password
        Screens::position_widget  ${tile_path}.btn_password_cancel
      
        # Refocus on the password field
        after idle  "safe_focus_force  ${tile_path}.ent_password"
      
        # Reset the password field
        set State(user_password)  ""
      
        # Reset the password verify flag
        set State(password_verify)  0
        
        # Wait for password verification
        # Correct password:  1
        # Cancel password:  -1
        while { $State(password_verify) == 0 } \
        {
          vwait Recall_SetupsUI::State(password_verify)
        }
        
        # Remove the password fields
        grid remove  ${tile_path}.lbl_password
        grid remove  ${tile_path}.ent_password
        grid remove  ${tile_path}.btn_password_cancel
      
        if { $State(password_verify) == -1 } \
        {
          # Password request cancelled
          # Do not continue
          return
        }
      
        # Delay to let the Enter key press to propagate
        after 1000
      }
    }
    
    if { [file  exists  $State($index,f_path)] } \
    {
      # Display the DELETE verify fields
      Screens::position_widget  ${tile_path}.lbl_delete
      Screens::position_widget  ${tile_path}.btn_yes
      Screens::position_widget  ${tile_path}.btn_no
    }
    
    # Refocus on the NO button
    after idle  "safe_focus_force  ${tile_path}.btn_no"
    
    return
  }  
  

  #***************************************************************************
  #
  #        Name: delete_file_yes
  #
  #  Parameters: widget - tile path of the invoking widget
  #
  # Description: Delete the file
  #
  #***************************************************************************
  proc delete_file_yes { widget } \
  {
    Debug::Puts RECALL 10 "delete_file_yes:  $widget"
    
    variable  State
    
    set tile_path  [get_tile  $widget]
    
    set list_box  ${tile_path}.lst_setups

    # Get the active selection
    set index  [$list_box  index  active]
    
    if ![info exists State($index,f_label) ] { return }
    
    # If the file is a directory, then handle the delete different
    # If there are files in the directory
    if { $State($index,f_label) == "<DIR>" } \
    {
      if [catch {file delete $State($index,f_path)} err] \
      {
        # Notify the user that the directory cannot be deleted
        set msg  "The directory contains files and cannot be deleted"
        
        tk_messageBox  -icon warning                    \
                       -message $msg                    \
                       -parent [winfo toplevel $widget] \
                       -title "WARNING"                 \
                       -type ok
      }
    } \
    else \
    {
      catch {file delete $State($index,f_path)}
    }
    
    # Remove the DELETE verify fields
    grid remove  ${tile_path}.lbl_delete
    grid remove  ${tile_path}.btn_yes
    grid remove  ${tile_path}.btn_no

    # Get the present available setup files
    set_file_source  $tile_path
  }
   
   
  #***************************************************************************
  #
  #        Name: delete_file_no
  #
  #  Parameters: widget - tile_path of the invoking widget
  #
  # Description: no delete file
  #
  #***************************************************************************
  proc delete_file_no { widget } \
  {
    variable  State
    
    set tile_path  [get_tile  $widget]
    
    # Remove the DELETE verify fields
    grid remove  ${tile_path}.lbl_delete
    grid remove  ${tile_path}.btn_yes
    grid remove  ${tile_path}.btn_no

    return
  }


  #***************************************************************************
  #
  #        Name: toggle_change_freq
  #
  #  Parameters: 
  #
  # Description: Toggles the Change frequency checkbutton
  #
  #***************************************************************************
  proc toggle_change_freq { } \
  {
    Debug::Puts RECALL 10 "toggle_change_freq"

    variable  State
    
    if { $State(no_change_freq) == 0 } \
    {
      set State(no_change_freq)  1
    } \
    else \
    {
      set State(no_change_freq)  0
    }
      
    return
  }


  #***************************************************************************
  #
  #        Name: update_change_freq
  #
  #  Parameters: widget - tile_path of the invoking widget
  #
  # Description: Updates the display to changes in the Change frequency 
  #              checkbutton
  #
  #***************************************************************************
  proc update_change_freq { widget  args } \
  {
    Debug::Puts RECALL 10 "update_change_freq:  $widget,  $args"

    variable  State
    
    set tile_path  [get_tile  $widget]
    
    if { $State(no_change_freq) == 0 } \
    {
      # Softkey #7:  No Change Frequency
      View::AssignSoftkey  $tile_path   7  "NO\nCHANGE"  \
                                        "Recall_SetupsUI::toggle_change_freq"
    } \
    else \
    {
      # Softkey #7:  Change Frequency
      View::AssignSoftkey  $tile_path   7  "CHANGE\nFREQ"  \
                                        "Recall_SetupsUI::toggle_change_freq"
    }
      
    return
  }


  #***************************************************************************
  #
  #        Name: toggle_scroll_on
  #
  #  Parameters:
  #
  # Description: Toggles the 'scroll_on' variable
  #
  #***************************************************************************
  proc toggle_scroll_on { } \
  {
    Debug::Puts RECALL 10 "toggle_scroll_on"
    
    variable  State
    
    if { $State(scroll_on) == 0 } \
    {
      set State(scroll_on)  1
    } \
    else \
    {
      set State(scroll_on)  0
    }

    return
  }


  #***************************************************************************
  #
  #        Name: set_scroll_off
  #
  #  Parameters:
  #
  # Description: Sets the 'scroll_on' variable to OFF
  #              Handles the <Escape> event for the listbox
  #
  #***************************************************************************
  proc set_scroll_off { } \
  {
    Debug::Puts RECALL 10 "set_scroll_off"
    
    variable  State
    
    if { $State(scroll_on) == 1 } \
    {
      set State(scroll_on)  0
    }

    return
  }


  #***************************************************************************
  #
  #        Name: update_scroll
  #
  #  Parameters: tile_path - tile path of the invoking widget
  #
  # Description: Update the display to changes in the 'scroll_on' variable 
  #              to allow the spinner knob to control the scrollbar
  #
  #***************************************************************************
  proc update_scroll { tile_path  args } \
  {
    Debug::Puts RECALL 10 "update_scroll:  $tile_path,  $args"
    
    variable  State
    
    set tile_path  [get_tile  $tile_path]
    
    set list_box    ${tile_path}.lst_setups
    set scroll_bar  ${tile_path}.scrl_setups
    
    if { $State(scroll_on) == 1 } \
    {
      # Change the color of the scrollbar to indicate it is active
      $scroll_bar config  -bg white
      
      # For each event, call the procedure to move the scrollbar,
      # then break to keep the focus from moving in response to the event
      bind $scroll_bar  <Right>  \
                  "Recall_SetupsUI::move_listbox_view  $tile_path  UP; break"
                                   
      bind $scroll_bar  <Left>  \
                "Recall_SetupsUI::move_listbox_view  $tile_path  DOWN; break"
                                   
      bind $scroll_bar  <Up>  \
                  "Recall_SetupsUI::move_listbox_view  $tile_path  UP; break"
                                   
      bind $scroll_bar  <Down>  \
                "Recall_SetupsUI::move_listbox_view  $tile_path  DOWN; break"

      # Change the color of the listbox to indicate it is active
      $list_box config  -bg white

      # For each event, call the procedure to move the listbox selection
      # then break to keep the focus from moving in response to the event
      bind $list_box  <Right>  \
             "Recall_SetupsUI::move_listbox_selection  $tile_path  UP; break"
                                   
      bind $list_box  <Left>  \
           "Recall_SetupsUI::move_listbox_selection  $tile_path  DOWN; break"
                                   
      bind $list_box  <Up>  \
             "Recall_SetupsUI::move_listbox_selection  $tile_path  UP; break"
                                   
      bind $list_box  <Down>  \
           "Recall_SetupsUI::move_listbox_selection  $tile_path  DOWN; break"
    } \
    else \
    {
      # Change the color of the scrollbar to indicate it is NOT active
      $scroll_bar config  -bg gray

      # For the 'Right' of 'Left' event, generate the 'cursor_right' or 
      # 'cursor_left' event, then break to keep the scrollbar from moving 
      # in response to either event
      bind $scroll_bar  <Right>  "event generate %W <<cursor_right>>; break"
      bind $scroll_bar  <Left>   "event generate %W <<cursor_left>>; break"
      
      # For the 'Up' of 'Down' event, generate the 'cursor_up' or 'cursor_down'
      # event, then break to keep the scrollbar from moving in response to 
      # either event
      bind $scroll_bar  <Up>    "event generate %W <<cursor_up>>; break"
      bind $scroll_bar  <Down>  "event generate %W <<cursor_down>>; break"

      
      # Change the color of the listbox to indicate it is NOT active
      $list_box config  -bg gray

      # For the 'Right' of 'Left' event, generate the 'cursor_right' or 
      # 'cursor_left' event, then break to keep the listbox selection from 
      # moving in response to either event
      bind $list_box  <Right>  "event generate %W <<cursor_right>>; break"
      bind $list_box  <Left>   "event generate %W <<cursor_left>>; break"
      
      # For the 'Up' of 'Down' event, generate the 'cursor_up' or 'cursor_down'
      # event, then break to keep the listbox selection from moving in response
      # to either event
      bind $list_box  <Up>    "event generate %W <<cursor_up>>; break"
      bind $list_box  <Down>  "event generate %W <<cursor_down>>; break"
    }
    
    return
  }
      

  #***************************************************************************
  #
  #        Name: move_listbox_view
  #
  #  Parameters: tile_path - tile path of the invoking widget
  #              direction - direction to move the scrollbar
  #
  # Description: Moves the scrollbar for changes on the spinner knob 
  #              or Up and Down arrows.
  #
  #***************************************************************************
  proc move_listbox_view { tile_path  direction } \
  {
    Debug::Puts RECALL 10 "move_listbox_view:  $tile_path,  $direction"
    
    variable  State
    
    if { $State(scroll_on) == 0 } \
    {
      return
    }

    set tile_path  [get_tile  $tile_path]
    
    set list_box  ${tile_path}.lst_setups

    # Move the canvas display
    switch -- $direction \
    {
      "UP"    { $list_box  yview  scroll  -1  unit }
    
      "DOWN"  { $list_box  yview  scroll   1  unit }
    }
    
    return
  }


  #***************************************************************************
  #
  #        Name: move_listbox_selection
  #
  #  Parameters: tile_path - tile path of the invoking widget
  #              direction - direction to move the scrollbar
  #
  # Description: Moves the listbox display for changes on the spinner knob 
  #              or Up and Down arrows.
  #
  #***************************************************************************
  proc move_listbox_selection { tile_path  direction } \
  {
    Debug::Puts RECALL 10 "move_listbox_selection:  $tile_path,  $direction"
    
    variable  State
    
    if { $State(scroll_on) == 0 } \
    {
      return
    }

    set tile_path  [get_tile  $tile_path]
    
    set list_box  ${tile_path}.lst_setups

    # Get the maximum number of list items
    set max_index  [expr [$list_box  index  end] - 1]
    
    # Move the canvas display
    switch -- $direction \
    {
      "UP"    { incr  State(curr_select)  -1 }
    
      "DOWN"  { incr  State(curr_select)   1 }
    }
    
    # Limit the current selection index
    if { $State(curr_select) < 0 } \
    {
      set State(curr_select)  0
    } \
    elseif { $State(curr_select) > $max_index } \
    {
      set State(curr_select)  $max_index
    }
    
    # Clear the current selection(s)
    $list_box  selection  clear  0  end
    
    # Activate the new current index
    $list_box  activate  $State(curr_select)

    # Set the current selection to the new current index
    $list_box  selection  set  $State(curr_select)
    
    # Set the view to see the new current index
    $list_box  see  $State(curr_select)

    return
  }


  #***************************************************************************
  #
  #        Name: click_listbox
  #
  #  Parameters: widget - tile_path of the invoking widget
  #
  # Description: Handles the <Button-1> event for the listbox.
  #
  #***************************************************************************
  proc click_listbox { widget } \
  {
    Debug::Puts RECALL 10 "click_listbox:  $widget"

    variable  State
     
    set tile_path  [get_tile  $widget]

    set list_box  ${tile_path}.lst_setups
    
    # Set the 'scroll_on' variable to 1
    set State(scroll_on)  1
    
    # Delay to let the display catch up to the selection
    after 100
    
    # Get the active selection
    set index  [$list_box  index  active]
    
    if ![info exists State($index,f_label)] { return }
    
    if { $State($index,f_label) == "<DIR>" } \
    {
      
    }

    return
  }


  #***************************************************************************
  #
  #        Name: double_click_listbox
  #
  #  Parameters: widget - tile_path of the invoking widget
  #
  # Description: Handles the <Double-1> (double click) event for the listbox.
  #
  #***************************************************************************
  proc double_click_listbox { widget } \
  {
    Debug::Puts RECALL 10 "double_click_listbox:  $widget"

    variable  State
      
    set tile_path  [get_tile  $widget]

    set list_box  ${tile_path}.lst_setups
    
    # Set the 'scroll_on' variable to 1
    set State(scroll_on)  1

    # Delay to let the display catch up to the selection
    after 100

    # Get the active selection
    set index  [$list_box  index  active]

    if ![info exists State($index,f_label)] { return }
    
    if { $State($index,f_label) == "<DIR>" } \
    {
      # Change the directory
      # Place in an 'after' call so that the listbox
      # changes back to the gray (inactive) background
      # before the change
      after 500  "Recall_SetupsUI::change_dir  $widget"
    } \
    else \
    {
      # Recall the setup
      # Place in an 'after' call so that the listbox
      # changes back to the gray (inactive) background
      # before the recall
      after 500  "Recall_SetupsUI::listbox_recall  $tile_path"
    }
    
    # Set the 'scroll_on' variable to 0
    set State(scroll_on)  0
        
    return
  }


  #***************************************************************************
  #
  #        Name: right_click_listbox
  #
  #  Parameters: widget - tile_path of the invoking widget
  #
  # Description: Handles the <Button-2> (right button) event for the listbox.
  #
  #***************************************************************************
  proc right_click_listbox { widget  x_coord  y_coord } \
  {
    Debug::Puts RECALL 10 "right_click_listbox:  $widget,  $x_coord,  $y_coord"

    variable  State
    
    # Procedure not used
    return
    # Procedure not used
    
    set tile_path  [get_tile  $widget]

    set list_box  ${tile_path}.lst_setups
    
    destroy  ${list_box}.menu
    
    set popup_menu  [menu  ${list_box}.menu]
    
    $popup_menu add command  -label "Help"  -command "help::show  $widget"
    
    $popup_menu add separator

    $popup_menu add command  -label "Generator"  \
                             -command "Screens::Show  $tile_path  Gen_Screen"
      
    $popup_menu add command  -label "Receiver"  \
                             -command "Screens::Show  $tile_path  Rec_Screen"
      
    $popup_menu add command  -label "Duplex"  \
                          -command "Screens::Show  $tile_path  Duplex_Screen"
      
    $popup_menu add command  -label "Function Generator"  \
                           -command "Screens::Show  $tile_path  AFGen_Screen"
      
    $popup_menu add command  -label "Oscilliscope"  \
                           -command "Screens::Show  $tile_path  Scope_Screen"
      
    $popup_menu add command  -label "Spectrum Analyzer"  \
                        -command "Screens::Show  $tile_path  Analyzer_Screen"

    tk_popup  $popup_menu  $x_coord  $y_coord
    
    return
  }


  #***************************************************************************
  #
  #        Name: list_by_filenumber
  #
  #  Parameters: widget - tile_path of the invoking widget
  #
  # Description: Lists the setups by File number
  #
  #***************************************************************************
  proc list_by_filenumber { widget } \
  {
    Debug::Puts RECALL 10 "list_by_filenumber:  $widget"

    variable  State
    
    set tile_path  [get_tile  $widget]
    
    set list_box  ${tile_path}.lst_setups

    # Clear the listbox
    $list_box  delete  0  end
    
    # Sort the array by file number:  State($index,f_number)
    set start  [expr $State(num_files) - 1]
    
    for { set sort_index  $start } { $sort_index >= 1 } { incr sort_index  -1 } \
    {
      for { set index  0 } { $index < $sort_index } { incr index  1 } \
      {
        set next_index  [expr $index + 1]
        
        if { $State($index,f_label) == "<DIR>" } \
        {
          # Directories are designated by a file label of '<DIR>',
          # and are listed first
        } \
        elseif { $State($index,f_number) > $State($next_index,f_number) } \
        {
          # Switch the index entries
          set temp1  $State($index,f_name)
          set temp2  $State($index,f_number)
          set temp3  $State($index,f_path)
          set temp4  $State($index,f_label)
          set temp5  $State($index,f_date)
          set temp6  $State($index,f_password)

          set State($index,f_name)      $State($next_index,f_name) 
          set State($index,f_number)    $State($next_index,f_number) 
          set State($index,f_path)      $State($next_index,f_path) 
          set State($index,f_label)     $State($next_index,f_label)
          set State($index,f_date)      $State($next_index,f_date) 
          set State($index,f_password)  $State($next_index,f_password) 

          set State($next_index,f_name)      $temp1
          set State($next_index,f_number)    $temp2
          set State($next_index,f_path)      $temp3
          set State($next_index,f_label)     $temp4
          set State($next_index,f_date)      $temp5
          set State($next_index,f_password)  $temp6
        }
      }
    }

    # Display the new list
    for { set index  0 } { $index < $State(num_files) } { incr index  1 } \
    {    
      set temp_string  [format  "%-15s %-24s %-15s"  "$State($index,f_name)"   \
                                                     "$State($index,f_label)"  \
                                                     "$State($index,f_date)"]

      $list_box  insert  end  $temp_string
    }
    
    return
  }


  #***************************************************************************
  #
  #        Name: list_by_label
  #
  #  Parameters: widget - tile_path of the invoking widget
  #
  # Description: Lists the setups by Label
  #
  #***************************************************************************
  proc list_by_label { widget } \
  {
    Debug::Puts RECALL 10 "list_by_label:  $widget"

    variable  State
    
    set tile_path  [get_tile  $widget]
    
    set list_box  ${tile_path}.lst_setups

    # Clear the listbox
    $list_box  delete  0  end
    
    # Sort the array by file label:  State($index,f_label)
    set start  [expr $State(num_files) - 1]
    
    for { set sort_index  $start } { $sort_index >= 1 } { incr sort_index  -1 } \
    {
      for { set index  0 } { $index < $sort_index } { incr index  1 } \
      {
        set next_index  [expr $index + 1]
        
        if { $State($index,f_label) == "<DIR>" } \
        {
          # Directories are designated by a file number of '<DIR>',
          # and are listed first
        } \
        elseif { $State($index,f_label) > $State($next_index,f_label) } \
        {
          # Switch the index entries
          set temp1  $State($index,f_name)
          set temp2  $State($index,f_number)
          set temp3  $State($index,f_path)
          set temp4  $State($index,f_label)
          set temp5  $State($index,f_date)
          set temp6  $State($index,f_password)

          set State($index,f_name)      $State($next_index,f_name) 
          set State($index,f_number)    $State($next_index,f_number) 
          set State($index,f_path)      $State($next_index,f_path) 
          set State($index,f_label)     $State($next_index,f_label)
          set State($index,f_date)      $State($next_index,f_date) 
          set State($index,f_password)  $State($next_index,f_password) 

          set State($next_index,f_name)      $temp1
          set State($next_index,f_number)    $temp2
          set State($next_index,f_path)      $temp3
          set State($next_index,f_label)     $temp4
          set State($next_index,f_date)      $temp5
          set State($next_index,f_password)  $temp6
        }
      }
    }

    # Display the new list
    for { set index  0 } { $index < $State(num_files) } { incr index  1 } \
    {    
      set temp_string  [format  "%-15s %-24s %-15s"  "$State($index,f_name)"   \
                                                     "$State($index,f_label)"  \
                                                     "$State($index,f_date)"]

      $list_box  insert  end  $temp_string
    }

    return
  }


  #***************************************************************************
  #
  #        Name: list_by_date
  #
  #  Parameters: widget - tile_path of the invoking widget
  #
  # Description: Lists the setups by Date
  #
  #***************************************************************************
  proc list_by_date { widget } \
  {
    Debug::Puts RECALL 10 "list_by_date:  $widget"

    variable  State
    
    set tile_path  [get_tile  $widget]

    set list_box  ${tile_path}.lst_setups

    # Clear the listbox
    $list_box  delete  0  end
    
    # Sort the array by file date:  State($index,f_date)
    set start  [expr $State(num_files) - 1]
    
    for { set sort_index  $start } { $sort_index >= 1 } { incr sort_index  -1 } \
    {
      for { set index  0 } { $index < $sort_index } { incr index  1 } \
      {
        set next_index  [expr $index + 1]
        
        if { $State($index,f_label) == "<DIR>" } \
        {
          # Directories are designated by a file number of '<DIR>',
          # and are listed first
        } \
        elseif { $State($index,f_date) > $State($next_index,f_date) } \
        {
          # Switch the index entries
          set temp1  $State($index,f_name)
          set temp2  $State($index,f_number)
          set temp3  $State($index,f_path)
          set temp4  $State($index,f_label)
          set temp5  $State($index,f_date)
          set temp6  $State($index,f_password)

          set State($index,f_name)      $State($next_index,f_name) 
          set State($index,f_number)    $State($next_index,f_number) 
          set State($index,f_path)      $State($next_index,f_path) 
          set State($index,f_label)     $State($next_index,f_label)
          set State($index,f_date)      $State($next_index,f_date) 
          set State($index,f_password)  $State($next_index,f_password) 

          set State($next_index,f_name)      $temp1
          set State($next_index,f_number)    $temp2
          set State($next_index,f_path)      $temp3
          set State($next_index,f_label)     $temp4
          set State($next_index,f_date)      $temp5
          set State($next_index,f_password)  $temp6
        }
      }
    }

    # Display the new list
    for { set index  0 } { $index < $State(num_files) } { incr index  1 } \
    {    
      set temp_string  [format  "%-15s %-24s %-15s"  "$State($index,f_name)"   \
                                                     "$State($index,f_label)"  \
                                                     "$State($index,f_date)"]

      $list_box  insert  end  $temp_string
    }

    return
  }


  #***************************************************************************
  #
  #        Name: save_setup
  #
  #  Parameters: widget - tile_path of the invoking widget
  #
  # Description: Displays fields to save the present setup
  #
  #***************************************************************************
  proc save_setup { widget } \
  {
    Debug::Puts RECALL 10 "save_setup:  $widget"

    variable  RecallScreen
    variable  State
    
    Save_setup::display_save  $widget  $RecallScreen
    
    set_file_source  $widget

    return
  }


  #***************************************************************************
  #
  #        Name: backup_setup
  #
  #  Parameters: widget - tile_path of the invoking widget
  #
  # Description: Displays the Backup toplevel
  #
  #***************************************************************************
  proc backup_setup { widget } \
  {
    Debug::Puts RECALL 10 "backup_setup:  $widget"

    set tile_path  [get_tile  $widget]
    
    set backup_tile_path  [Backup_setupUI::Create  $tile_path]
    
    bind $backup_tile_path  <<Destroy>>  \
                             "Recall_SetupsUI::destroy_backup_setup  $widget"

    return
  }


  #***************************************************************************
  #
  #        Name: destroy_backup_setup
  #
  #  Parameters: widget - tile_path of the invoking widget
  #
  # Description: Handles the <<Destroy>> bind for the Backup toplevel
  #
  #***************************************************************************
  proc destroy_backup_setup { widget } \
  {
    Debug::Puts RECALL 10 "destroy_backup_setup:  $widget"

    # Display the setup files
    set_file_source  $widget

    return
  }


  #***************************************************************************
  #
  #        Name: verify_dir
  #
  #  Parameters: widget - tile_path of the invoking widget
  #
  # Description: Verifies that the user entered directory exists
  #
  #***************************************************************************
  proc verify_dir { widget } \
  {
    Debug::Puts RECALL 10 "verify_dir:  $widget"

    variable  State
    
    set tile_path  [get_tile  $widget]

    set list_box  ${tile_path}.lst_setups

    # Determine if the user entered directory exists
    # If it does, change to that directory
    switch -- $State(source) \
    {
      "STANDARD"  { set temp_dir  $State(std_save_sub_dir) }
      
      "FLOPPY"    { set temp_dir  $State(ext_save_sub_dir) }
      
      "INTERNAL"  -
      default     { set temp_dir  $State(int_save_sub_dir) }
    }

    if { $temp_dir == "" } \
    {
      # A null user entered sub-directory returns to the previous directory
      set_file_source  $widget
      
      return
    }
    
    set max_index  [$list_box  index  end]
    
    for { set i  0 } { $i < $max_index } { incr i  1 } \
    {
      # Check only directory listbox items
      if { $State($i,f_label) == "<DIR>" } \
      {
        # Compare the user entered value to each array directory name
        if { $temp_dir == $State($i,f_name) } \
        {
          # Directory found
          set_file_source  $widget
                          
          break
        }                      
      }
    }
    
    return
  }


  #***************************************************************************
  #
  #        Name: change_dir
  #
  #  Parameters: widget - tile_path of the invoking widget
  #
  # Description: Changes the directory to the selected listbox directory
  #
  #***************************************************************************
  proc change_dir { widget } \
  {
    Debug::Puts RECALL 10 "change_dir:  $widget"

    variable  State
    
    set tile_path  [get_tile  $widget]

    set list_box    ${tile_path}.lst_setups
    set scroll_bar  ${tile_path}.scrl_setups

    # Get the active selection
    set index  [$list_box  index  active]
    
    # Determine if the file is a directory
    # If it is, change to that directory
    if ![info exists State($index,f_label)] { return}
    
    if { $State($index,f_label) == "<DIR>" } \
    {
      # Set the appropriate directory to the user entered directory
      switch -- $State(source) \
      {
        "STANDARD"  { set State(std_save_sub_dir)  $State($index,f_name) }
      
        "FLOPPY"    { set State(ext_save_sub_dir)  $State($index,f_name) }
      
        "INTERNAL"  -
        default     { set State(int_save_sub_dir)  $State($index,f_name) }
      }

      set_file_source  $widget
    }

    # Set the 'scroll_on' variable to 0
    set State(scroll_on)  0

    return
  }


  #***************************************************************************
  #
  #        Name: return_dir
  #
  #  Parameters: widget - tile_path of the invoking widget
  #
  # Description: Returns to the previous directory
  #
  #***************************************************************************
  proc return_dir { widget } \
  {
    Debug::Puts RECALL 10 "return_dir:  $widget"

    variable  State
    
    set tile_path  [get_tile  $widget]

    set list_box    ${tile_path}.lst_setups
    set scroll_bar  ${tile_path}.scrl_setups

    # Set the appropriate directory to the user entered directory
    switch -- $State(source) \
    {
      "STANDARD"  { set State(std_save_sub_dir)  "" }
    
      "FLOPPY"    { set State(ext_save_sub_dir)  "" }
    
      "INTERNAL"  -
      default     { set State(int_save_sub_dir)  "" }
    }

    set_file_source  $widget

    # Set the 'scroll_on' variable to 0
    set State(scroll_on)  0

    return
  }


  #***************************************************************************
  #
  #        Name: verify_password
  #
  #  Parameters: widget - tile_path of the invoking widget
  #
  # Description: Verifies the password for deleting files from the
  #              STANDARD directory.
  #
  #***************************************************************************
  proc verify_password { widget } \
  {
    Debug::Puts RECALL 10 "verify_password:  $widget"

    variable  State
    
    set tile_path  [get_tile  $widget]

    # Verify the password
    if { $State(source) == "STANDARD" } \
    {
      set list_box  ${tile_path}.lst_setups
  
      # Get the active selection
      set index  [$list_box  index  active]
      
      # Compare the user entered password to the file password
      if { $State(user_password) == $State($index,f_password) } \
      {
        set State(password_verify)  1
      } \
      else \
      {
        set State(password_verify)  0
      }
    }

    return
  }


  #***************************************************************************
  #
  #        Name: cancel_password
  #
  #  Parameters: widget - tile_path of the invoking widget
  #
  # Description: Cancels the password request fro deleting from the 
  #              STANDARD directory
  #
  #***************************************************************************
  proc cancel_password { widget } \
  {
    Debug::Puts RECALL 10 "cancel_password:  $widget"

    variable  State
    
    set tile_path  [get_tile  $widget]

    # Cancel the password request
    if { $State(source) == "STANDARD" } \
    {
      set State(password_verify)  -1
    }

    return
  }


};  # End of Recall_SetupsUI namespace


