#############################################################################
#                                                                           #
# IFR SYSTEMS, INC    COPYRIGHT (C) 1998,99,2001,2002  DESIGN ENGINEERING   #
# ------------------------------------------------------------------------- #
#                                                                           #
# FILE        : 4020.Backup_setupUI.tcl                                     #
#                                                                           #
# PRODUCT     : FITS                                                        #
#                                                                           #
# AUTHOR      : N. Tran                                                     #
#                                                                           #
# DESCRIPTION : Instrument Setup - Backup Setup file                        #
#                                                                           #
#############################################################################

namespace eval Backup_setupUI \
{
  # Global variables
  variable  LongName     "backup_setup"
  variable  ShortName    "Backup Setup UI"
  variable  Size         Quarter
            
  variable  tile_class   "BACKUP_SETUP"

  variable  last_focus
  
  variable  StateUI
  
  set StateUI(act_menu) { "Backup to:"  "Restore from:" }
  set StateUI(act)      "Backup to:"
  
  set StateUI(int_saved_dir)  "/usr/setups"
  set StateUI(ext_saved_dir)  "/fd0"
  
  #********************************************************************
  #
  #       Name: Create  
  #
  # Parameters:
  #
  # Description: 
  #
  #********************************************************************
  proc Create { tile_path } \
  {
    #puts "Create:  $tile_path"

    variable LongName
    variable tile_class
    variable last_focus
    
    # Get the last focus 
    set last_focus   $tile_path
    
    set top_level  [winfo toplevel $tile_path]

    set tile_path  [name_child  $top_level  $LongName]

    if { ![winfo exists $tile_path] } \
    {
      set tile_path  [Screens::create_toplevel_tile  $tile_path  BACKUP_SETUP]
    }  
    
    # Bind the F8 key for a return
    bind $tile_path <KeyRelease-F8>  "destroy [winfo toplevel $tile_path]"

    #bind $tile_path <<Destroy>>  "Backup_setupUI::destroy_backup  $tile_path"

    return  $tile_path
  }


  #***************************************************************************
  #
  #       Name: get_tile
  #
  # Parameters: any child widget of the tile
  #
  # Description: get the name of the tile's parent frame
  #
  #***************************************************************************
  proc get_tile { widget } \
  {
    while { 1 } \
    {
      switch -- [winfo class $widget] \
      {
        "BACKUP_SETUP"   { return $widget }
      }
        
      set widget [winfo parent $widget]
    }

    return     
  }

  #***************************************************************************
  #
  #       Name: backup
  #
  # Parameters: None
  #
  # Description: backup the setup-files
  #
  #***************************************************************************
   proc backup { tile_path } \
   {
     variable StateUI
     variable last_focus
     
     if { $StateUI(act) == "Backup to:" } \
     {
       # Backup file from the current directory to the floppy
       # Setup the Source directory
       set source_dir  [file join $StateUI(int_saved_dir) \
                                 $::Recall_SetupsUI::State(int_save_sub_dir)]
       
       # Setup the Destination directory
       set dest_dir  [file join $StateUI(ext_saved_dir) \
                                 $::Recall_SetupsUI::State(int_save_sub_dir)]

       # Create the directory on the floppy
       file mkdir  $dest_dir
       
       # Define the file names to be copied
       set f  [file join  $source_dir  *.SAV]
       
       foreach { file } [glob -nocomplain  $f] \
       {
         #puts "  Copying $file to $dest_dir"

         # Catch errors in the case the disk gets full
         catch { file copy -force  $file  $dest_dir } error
         
         #puts "  error:  $error"
         
         if { [string match  "*DISK_FULL*"  $error] } \
         {
           # If the disk is full,
           # delete the last copied file from the disk
           set file_path  [file split  $file]
         
           set file_name  [lindex  $file_path  end]
         
           set file_name  [file join  $dest_dir  $file_name]
         
           #puts "    file_name:  $file_name"
           
           file delete  -force  $file_name

           # Notify the user that the disk is full
           set title  "Warning"
           
           set prompt  "  The disk is full  "
           
           set window_name  ${tile_path}.warning
           
           # Display the Warning dialog
           tk_dialog  $window_name  $title  $prompt  warning  0  "OK"
           
           # Stop the file copy
           break
         }
       }
     } \
     else \
     {
       # Restore from floppy
       # Catch mistake from previous versions - if the setup location
       # is a file, not a directory (from a bug in previous code),
       # wipe out the file before trying to create the directory
       if { ![file isdir  $StateUI(int_saved_dir)] } \
       {
         file delete -force  $StateUI(int_saved_dir)
       }
     
       file mkdir  $StateUI(int_saved_dir)
     
       set f  [file join  $StateUI(ext_saved_dir) *]
     
       foreach  f  [glob -nocomplain  $f] \
       {
         #puts "  f:  $f"
         
         # If the file is a directory, make the directory locally
         if { [file isdirectory  $f] } \
         {
           # Get the directory path components
           set dir_components  [file split  $f]
           
           # Use the last component as the directory name
           set dir_name  [lindex  $dir_components  end]
           
           set sub_dir_dest  [file join  $StateUI(int_saved_dir)  $dir_name]

           # Make the directory locally
           file mkdir  $sub_dir_dest
           
           # Get the files in the sub-directory
           set sub_dir_files  [file join  $f  *]
           
           foreach { sub_dir_file } [glob -nocomplain  $sub_dir_files] \
           {
             # Copy over '.SAV' files
             if { [string match "*.SAV"  $sub_dir_file] } \
             {
               #puts "  Copying $sub_dir_file to $sub_dir_dest"
     
               file copy -force  $sub_dir_file  $sub_dir_dest
             }
           }
         }
         
         # Copy over '.SAV' files
         if { [string match  "*.SAV"  $f] } \
         {
           #puts "  Copying $f to $StateUI(int_saved_dir)"
  
           file copy -force  $f  $StateUI(int_saved_dir)
         }
       }
     }
     
     # Move focus back to Save_setup tile
     after idle  "safe_focus_force  $last_focus"
     
     # We have done our job, close the tile
     destroy  [get_tile  $tile_path]

   }
  
  #***************************************************************************
  #
  #       Name: cancel
  #
  # Parameters: None
  #
  # Description: cancel backup 
  #
  #***************************************************************************
  proc cancel { tile_path } \
  {
    variable last_focus
  
    # Move focus back to Save_setup tile
    after idle  "safe_focus_force $last_focus"
    
    destroy  [get_tile  $tile_path]
  }
  
};  # End of Backup_setupUI namespace

