#############################################################################
#                                                                           #
# IFR SYSTEMS, INC        COPYRIGHT (C)1998-2001    DESIGN ENGINEERING      #
#                                                                           #
# --------------------------------------------------------------------------#
#                                                                           #
# FILE        : 4050.EVM_DataUI.tcl                                         #
#                                                                           #
# PRODUCT     : FITS                                                        #
#                                                                           #
# AUTHOR      : Preston McCormick                                           #
#                                                                           #
# DESCRIPTION : Error Vector Magnitude Data Tile                            #
#                                                                           #
#############################################################################

package require Modacc

namespace eval EVM_DataUI \
{
  # Global variables
  variable  LongName       "small_evm_data"
  variable  ShortName      "EVM Data"
  variable  Size           Eighth_v

  variable  tile_class     SMALL_EVM_DATA

  variable  last_focus     ""
  
  variable  StateUI
  
  # EVM reading variables
  set  StateUI(evm,title)             "Error Vector Magnitude"
  set  StateUI(evm,btn_text)          "EVM"
  set  StateUI(evm,units)             "%"

  # Peak EVM reading variables
  set  StateUI(peak_evm,title)             "Peak Error Vector Magnitude"
  set  StateUI(peak_evm,btn_text)          "PEAK\nEVM"
  set  StateUI(peak_evm,units)             "%"

  # Magnitude Error reading variables
  set  StateUI(mag,title)             "Magnitude Error"
  set  StateUI(mag,btn_text)          "MAG\nERROR"
  set  StateUI(mag,units)             "%"

  # Peak Magnitude Error reading variables
  set  StateUI(peak_mag,title)             "Peak Magnitude Error"
  set  StateUI(peak_mag,btn_text)          "PEAK\nMAG ERR"
  set  StateUI(peak_mag,units)             "%"

  # Phase Error reading variables
  set  StateUI(ph,title)             "Phase Error"
  set  StateUI(ph,btn_text)          "PHASE\nERROR"
  set  StateUI(ph,units)             "deg"

  # Peak Phase Error reading variables
  set  StateUI(peak_ph,title)             "Peak Phase Error"
  set  StateUI(peak_ph,btn_text)          "PEAK\nPHS ERR"
  set  StateUI(peak_ph,units)             "deg"

  # Carrier Error reading variables
  set  StateUI(or,title)             "Origin Offset"
  set  StateUI(or,btn_text)          "ORIGIN\nOFFSET"
  set  StateUI(or,units)             "dB"

  # Frequency Error reading variables
  set  StateUI(freq,title)             "Frequency Error"
  set  StateUI(freq,btn_text)          "FREQ\nERROR"
  set  StateUI(freq,units)             "Hz"


  #********************************************************************
  #
  # Name: Create
  #
  # Parameters:
  #
  # Desciption: 
  #
  #********************************************************************
  proc Create { tile_path } \
  {
    #puts "Create:  $tile_path"

    variable LongName
    variable tile_class

    set top_level  [winfo toplevel  $tile_path]
   
    set tile_path  [name_child  $top_level  $LongName]
  
    if { ![winfo exists $tile_path] } \
    {
      set tile_path  [Screens::create_tile  $tile_path  $LongName  $tile_class]
    }  

    return  $tile_path
  }


  #******************************************************************** 
  #
  # Name: Map
  #
  # Parameters: 
  #
  # Desciption: 
  #
  #********************************************************************
  proc Map { tile_path } \
  {
    #puts "Map:  $tile_path"

    variable  StateUI
    
    # Check to see if the option is enabled
    if { ![IFR_options is_installed EVM] } \
    {
      grid forget $tile_path
    
      return
    }

    # Trace on the reading of each measurement
    OnChange hook ::Modacc::State(evm,value) "EVM_DataUI::update_display  $tile_path  evm  frame_qpsk.lbl_evm_val"

    OnChange hook ::Modacc::State(mag,value) "EVM_DataUI::update_display  $tile_path  mag  frame_qpsk.lbl_mag_err_val"
    
    OnChange hook ::Modacc::State(ph,value) "EVM_DataUI::update_display  $tile_path  ph  frame_qpsk.lbl_phase_err_val"
    
    OnChange hook ::Modacc::State(or,value) "EVM_DataUI::update_display  $tile_path  or  frame_qpsk.lbl_origin_offset_val"
    
    OnChange hook ::Modacc::State(freq,value) "EVM_DataUI::update_display  $tile_path  freq  frame_qpsk.lbl_freq_err_val"

    OnChange hook ::C4fm::State(freq_err,status) "meter_text_colors $tile_path.frame_fm.carr_err_value ::C4fm::State(freq_err,status)"
    
    OnChange hook ::C4fm::State(deviation,status) "meter_text_colors $tile_path.frame_fm.sym_dev_value ::C4fm::State(deviation,status)"

    OnChange hook ::C4fm::State(modacc,status) "meter_text_colors $tile_path.frame_fm.moderr_value ::C4fm::State(modacc,status)"

    OnChange hook ::Modacc::State(alpha) "EVM_DataUI::update_mode $tile_path"
    
    # Increment the ON count
    Modacc run 1 -nowait
    
  }


  #********************************************************************
  #
  # Name: Cleanup
  #
  # Parameters:  tile_path
  #
  # Desciption: 
  #
  #********************************************************************
  proc Cleanup { tile_path } \
  {
    #puts "Cleanup:  $tile_path"

    variable  StateUI

    OnChange unhook ::Modacc::State(evm,value) "EVM_DataUI::update_display  $tile_path  evm  frame_qpsk.lbl_evm_val"

    OnChange unhook ::Modacc::State(mag,value) "EVM_DataUI::update_display  $tile_path  mag  frame_qpsk.lbl_mag_err_val"
    
    OnChange unhook ::Modacc::State(ph,value) "EVM_DataUI::update_display  $tile_path  ph  frame_qpsk.lbl_phase_err_val"
    
    OnChange unhook ::Modacc::State(or,value) "EVM_DataUI::update_display  $tile_path  or  frame_qpsk.lbl_origin_offset_val"
    
    OnChange unhook ::Modacc::State(freq,value) "EVM_DataUI::update_display  $tile_path  freq  frame_qpsk.lbl_freq_err_val"

    OnChange unhook ::C4fm::State(freq_err,status) "meter_text_colors $tile_path.frame_fm.carr_err_value ::C4fm::State(freq_err,status)"
    
    OnChange unhook ::C4fm::State(deviation,status) "meter_text_colors $tile_path.frame_fm.sym_dev_value ::C4fm::State(deviation,status)"

    OnChange unhook ::C4fm::State(modacc,status) "meter_text_colors $tile_path.frame_fm.moderr_value ::C4fm::State(modacc,status)"

    OnChange unhook ::Modacc::State(alpha) "EVM_DataUI::update_mode $tile_path"
    
    # Decrement the ON count
    Modacc run -1 -nowait
  }
  
  
  #********************************************************************
  #
  # Name: Unmap
  #
  # Parameters:  tile_path
  #
  # Desciption: 
  #
  #********************************************************************
  proc Unmap { tile_path } \
  {
    #puts "Unmap:  $tile_path"
    
    Cleanup $tile_path
  }


  #********************************************************************
  #
  # Name: Destroy
  #
  # Parameters: 
  #
  # Desciption: 
  #
  #********************************************************************
  proc Destroy { tile_path } \
  {
    #puts "Destroy:  $tile_path"
    
    Cleanup $tile_path
  }


  #********************************************************************
  #
  # Name: Register
  #
  # Parameters: 
  #
  # Desciption: 
  #
  #********************************************************************
  proc Register { tile_path } \
  {
    #puts "Register:  $tile_path"
  }

  
  #***************************************************************************
  #
  # Name: get_tile
  #
  # Parameters: any child widget of the tile
  #
  # Description: get the name of the tile's parent frame
  #
  #***************************************************************************
  proc get_tile { widget } \
  {
    while { 1 } \
    {
      switch -- [winfo class $widget] \
      {
        "SMALL_EVM_DATA"   -
        "FULL_EVM_PLOT"    -
        "EVM_DATA_CONFIG"  { return $widget}
      }
        
      set widget [winfo parent $widget]
    }
  }

  #***************************************************************************
  #
  # Name: update_mode
  #
  # Parameters:  tile_path - tile path of the invoking widget
  #
  # Description: Update the display based upon the mode the EVM system is in
  #              (QPSK or FM)
  #
  #***************************************************************************
  proc update_mode { tile_path } \
  {
      Debug::Puts EVMUI 1 "update_mode:  $tile_path"
      if ![winfo exists ${tile_path}] { return }
      foreach slave [pack slaves $tile_path] { pack forget $slave }
      switch -- $::Modacc::State(alpha) \
      {
         0.0 \
         {
            Screens::position_widget $tile_path.frame_fm
            C4fm run 1 -nowait
            C4fm en 3 -nowait
            
         }
         default \
         {
            Screens::position_widget $tile_path.frame_qpsk
         } 
      }
  }

  #***************************************************************************
  #
  # Name: update_display
  #
  # Parameters: tile_path - tile path of the invoking widget
  #             option    - EVM option
  #
  # Desciption: Updates the display to changes status of the engine 
  #             readings
  #
  #***************************************************************************
  proc update_display { tile_path  option  reading_widget } \
  {
    #puts "update_display:  $tile_path,  $option,  $reading_widget"
    if ![winfo exists $tile_path]   { return }
    if ![winfo ismapped $tile_path] { return }
    set tile_path  [get_tile  $tile_path]
    
    set text_color  black
    set bg_color    grey
        
    # Check for stale readings
    if { $::Modacc::State($option,state) & 0x0002 } \
    {
      set bg_color DarkGrey
    } 

    # Check status
    if { $::Modacc::State($option,state) & 0x0200 } \
    { 
      # Above High Limit, color the reading red
      set text_color  red

      # Turn on the audio alarm
      set alarm  "hi"
    } \
    elseif { $::Modacc::State($option,state) & 0x0800  } \
    { 
      # Below Low limit, color the reading blue
      set text_color  blue
    } \
    else \
    { 
      if { $::Modacc::State($option,state) & 0x101C  } \
      {
        # Suspect reading
        set text_color  yellow
      }
    }

    ${tile_path}.${reading_widget}  config  -fg $text_color -bg $bg_color
  }


  #********************************************************************
  #
  # Name: display_config
  #
  # Parameters: tile_path - tile path of the invoking widget
  #             option    - EVM option to configure
  #
  # Desciption: Displays the configuration toplevel for the EVM fields
  #
  #********************************************************************
  proc display_config { tile_path  option } \
  {
    #puts "display_config:  $tile_path,  $option"
    
    variable  StateUI
    variable  last_focus
    
    set last_focus  $tile_path
    
    set top_level  [winfo toplevel $tile_path]
   
    set tile_path  [name_child  $top_level  evm_data_config]
    
    destroy $tile_path

    if { ![winfo exists $tile_path] } \
    {
      set tile_path  [::Screens::create_toplevel_tile  $tile_path  EVM_DATA_CONFIG]
    }
        
    # Bind the F8 key for a return
    bind $tile_path <KeyRelease-F8>  "destroy [winfo toplevel $tile_path]"

    bind  $tile_path  <<Destroy>>  \
                             "EVM_DataUI::destroy_config  $tile_path  $option"


    wm title  $tile_path  "Configure:  $StateUI($option,title)"
    
    setup_config_fields  $tile_path  $option

    # Trace on the value for this particular reading
    OnChange hook ::Modacc::State($option,value) "EVM_DataUI::update_display  $tile_path  $option  lbl_reading_val"


    safe_focus_force ${tile_path}.btn_return
  }


  #********************************************************************
  #
  # Name: destroy_config
  #
  # Parameters: tile_path - tile path of the invoking widget
  #             option    - EVM option to configure
  #
  # Desciption: Handles the <<Destroy>> event for the EVM configuration 
  #             toplevel
  #
  #********************************************************************
  proc destroy_config { tile_path  option } \
  {
    #puts "destroy_config:  $tile_path,  $option"
    
    variable  last_focus

    # Delete the trace on the value for this particular reading
    OnChange unhook ::Modacc::State($option,value) "EVM_DataUI::update_display  $tile_path  $option  lbl_reading_val"


    safe_focus_force $last_focus

    return
  }
  
  
  #********************************************************************
  #
  # Name: setup_config_fields
  #
  # Parameters: tile_path - tile path of the invoking widget
  #             option    - EVM option to configure
  #
  # Desciption: Ties the Configuration fields to the correct variables
  #             and commands
  #
  #********************************************************************
  proc setup_config_fields { tile_path  option } \
  {
    #puts "setup_config_fields:  $tile_path,  $option"
    
    ${tile_path}.lbl_reading_val  config  \
                                 -textvariable  ::Modacc::State($option,value)
  
    ${tile_path}.btn_high_limit  config  \
                       -textvariable  ::Modacc::State($option,hi_limit,text) \
                       -command  "EVM_DataUI::toggle_high_limit  $option"
  
    ${tile_path}.ent_high_limit  config  \
                        -textvariable  ::Modacc::State($option,hi_limit,value)
  
    bind_numeric_spin  ${tile_path}.ent_high_limit  {} {} {} {}  \
                                     "EVM_DataUI::set_high_limit_val  $option"

    ${tile_path}.btn_low_limit  config  \
                       -textvariable  ::Modacc::State($option,lo_limit,text) \
                       -command  "EVM_DataUI::toggle_low_limit  $option"
  
    ${tile_path}.ent_low_limit  config  \
                        -textvariable  ::Modacc::State($option,lo_limit,value)

    bind_numeric_spin  ${tile_path}.ent_low_limit  {} {} {} {}  \
                                      "EVM_DataUI::set_low_limit_val  $option"
  
    # Not displayed
    #${tile_path}.btn_peak_hold  config  \
    #            -textvariable  ::EVM_DataUI::StateUI($option,peak_hold_state) \
    #            -command  "EVM_DataUI::toggle_peak_hold  $option"
    
    ${tile_path}.ent_averaging  config  \
                               -textvariable  ::Modacc::State($option,average)

    bind_numeric_spin  ${tile_path}.ent_averaging {} {} {} {}  \
                                            "EVM_DataUI::set_average  $option"
  }


  #********************************************************************
  #
  # Name: toggle_high_limit
  #
  # Parameters: option - EVM option to configure
  #
  # Desciption: Toggles the High Limit ON and OFF
  #
  #********************************************************************
  proc toggle_high_limit { option } \
  {
    #puts "toggle_high_limit:  $option"
    
    set alert_state  [expr $::Modacc::State($option,limit,flags) ^ 0x1]
    
    Modacc  $option  alerts  $alert_state -nowait
  }
  

  #********************************************************************
  #
  # Name: toggle_low_limit
  #
  # Parameters: option - EVM option to configure
  #
  # Desciption: Toggles the Low Limit ON and OFF
  #
  #********************************************************************
  proc toggle_low_limit { option } \
  {
    #puts "toggle_low_limit:  $option"
    
    set alert_state  [expr $::Modacc::State($option,limit,flags) ^ 0x2]
    
    Modacc  $option  alerts  $alert_state -nowait
  }


  #********************************************************************
  #
  # Name: set_high_limit_val
  #
  # Parameters: option - EVM option to configure
  #
  # Desciption: Sets the High Limit value
  #
  #********************************************************************
  proc set_high_limit_val { option } \
  {
    #puts "set_high_limit_val:  $option"
    
    Modacc  $option  ulimit  $::Modacc::State($option,hi_limit,value) -nowait
  }
  

  #********************************************************************
  #
  # Name: set_low_limit_val
  #
  # Parameters: option - EVM option to configure
  #
  # Desciption: Sets the Low Limit value
  #
  #********************************************************************
  proc set_low_limit_val { option } \
  {
    #puts "set_low_limit_val:  $option"
    
    Modacc  $option  llimit  $::Modacc::State($option,lo_limit,value) -nowait
  }


  #********************************************************************
  #
  # Name: toggle_peak_hold
  #
  # Parameters: option - EVM option to configure
  #
  # Desciption: Toggles the Peak Hold ON and OFF
  #
  #********************************************************************
  proc toggle_peak_hold { option } \
  {
    #puts "toggle_peak_hold:  $option"
    
    # ::Modacc::State($option,peak,flags)

    #Modacc  $option  peak_hold  $peak_hold_state
  }


  #********************************************************************
  #
  # Name: set_average
  #
  # Parameters: option - EVM option to configure
  #
  # Desciption: Set the Averaging value for the reading
  #
  #********************************************************************
  proc set_average { option } \
  {
    #puts "set_average:  $option"
    
    Modacc  $option  average  $::Modacc::State($option,average) -nowait
  }


  #****************************************************************************
  #
  # Name: getSaveState
  #
  # Parameters:
  #
  # Description: This function returns a list of TCL commands which can be 
  #              used to set the EVM_DataUI tile to its current state
  #
  #****************************************************************************
  proc getSaveState { } \
  {
    variable StateUI
 
    set returnList \
    {

    }
       
    return [subst $returnList]
  }


  #****************************************************************************
  #
  # Name: Sync
  #
  # Parameters:
  #
  # Description: Write state variables to EVM_DataUI
  #
  #****************************************************************************
  proc Sync { } \
  {
    eval [getSaveState]
  }


};  # End of EVM_DataUI namespace



#****************************************************************************
# This registers the tiles with the list for saving instrument states
#****************************************************************************
namespace eval ::SavedStates \
{
  variable tiles   

  lappend tiles EVM_DataUI
}

