#############################################################################
#                                                                           #
# IFR SYSTEMS, INC        COPYRIGHT (C)1998-2001    DESIGN ENGINEERING      #
#                                                                           #
# --------------------------------------------------------------------------#
#                                                                           #
# FILE        : 4060.EVM_PlotUI.tcl                                         #
#                                                                           #
# PRODUCT     : FITS                                                        #
#                                                                           #
# AUTHOR      : Preston McCormick                                           #
#                                                                           #
# DESCRIPTION : Error Vector Magnitude Plot Tile                            #
#                                                                           #
#############################################################################

package require Modacc

namespace eval EVM_Plot_QtrUI \
{
  # Global variables
  variable  LongName       "qtr_evm_plot"
  variable  ShortName      "Modulation Plot"
  variable  Size           Quarter

  variable  tile_class     QTR_EVM_PLOT

  #***************************************************************************
  #
  # Name: Create
  #
  # Parameters:
  #
  # Desciption: 
  #
  #***************************************************************************
  proc Create { tile_path } \
  {
    variable LongName
    variable tile_class

    Debug::Puts EVMUI 1 "EVM_Plot_QtrUI::Create $tile_path"
    
    set top_level  [winfo toplevel  $tile_path]

    set tile_path  [name_child  $top_level  $LongName]

    ::EVM_PlotUI::Create $tile_path $LongName $tile_class

    return  $tile_path
  }


  #***************************************************************************
  #
  # Name: Map
  #
  # Parameters: 
  #
  # Desciption: 
  #
  #***************************************************************************
  proc Map { tile_path } \
  {
    Debug::Puts EVMUI 1 "EVM_Plot_QtrUI::Map $tile_path"

    # Check to see if the option is enabled
    if { ![IFR_options is_installed EVM] } \
    {
      grid forget $tile_path
    
      return
    }
    
    ::EVM_PlotUI::Map $tile_path
  }


  #***************************************************************************
  #
  # Name: Cleanup
  #
  # Parameters:  tile_path
  #
  # Desciption: 
  #
  #***************************************************************************
  proc Cleanup { tile_path } \
  {
    Debug::Puts EVMUI 1 "EVM_Plot_QtrUI::Cleanup $tile_path"

    ::EVM_PlotUI::Cleanup $tile_path
  }
  
  
  #***************************************************************************
  #
  # Name: Unmap
  #
  # Parameters:  tile_path
  #
  # Desciption: 
  #
  #***************************************************************************
  proc Unmap { tile_path } \
  {
    Debug::Puts EVMUI 1 "EVM_Plot_QtrUI::Unmap $tile_path"
    
    Cleanup $tile_path
  }


  #***************************************************************************
  #
  # Name: Destroy
  #
  # Parameters: 
  #
  # Desciption: 
  #
  #***************************************************************************
  proc Destroy { tile_path } \
  {
    Debug::Puts EVMUI 1 "EVM_Plot_QtrUI::Destroy $tile_path"
    
    Cleanup $tile_path
  }


  #***************************************************************************
  #
  # Name: Register
  #
  # Parameters: 
  #
  # Desciption: 
  #
  #***************************************************************************
  proc Register { tile_path } \
  {
    Debug::Puts EVMUI 1 "EVM_Plot_QtrUI::Register $tile_path"
  }
  
};  # End of EVM_Plot_QtrUI namespace



namespace eval EVM_Plot_FullUI \
{
  # Global variables
  variable  LongName       "full_evm_plot"
  variable  ShortName      "Modulation Plot"
  variable  Size           Full

  variable  tile_class     FULL_EVM_PLOT



  #***************************************************************************
  #
  # Name: Create
  #
  # Parameters:
  #
  # Desciption: 
  #
  #***************************************************************************
  proc Create { tile_path } \
  {
    variable LongName
    variable tile_class
    
    Debug::Puts EVMUI 1 "EVM_Plot_FullUI::Create $tile_path"
    
    set top_level  [winfo toplevel  $tile_path]

    set tile_path  [name_child  $top_level  $LongName]

    ::EVM_PlotUI::Create $tile_path $LongName $tile_class
    
    ::Screens::adjust_grid  $tile_path.evm_frame
    ::Screens::adjust_grid  $tile_path.c4fm_frame
        
    return  $tile_path
  }


  #*************************************************************************** 
  #
  # Name: Map
  #
  # Parameters: 
  #
  # Desciption: 
  #
  #***************************************************************************
  proc Map { tile_path } \
  {
    Debug::Puts EVMUI 1 "EVM_Plot_FullUI::Map $tile_path"

    variable  StateUI

    # Check to see if the option is enabled
    if { ![IFR_options is_installed EVM] } \
    {
      grid forget $tile_path
    
      return
    }
        
    # Trace on the reading of each measurement
    OnChange hook ::Modacc::State(evm,value) "EVM_DataUI::update_display  $tile_path  evm  lbl_evm_val"

    OnChange hook ::Modacc::State(mag,value) "EVM_DataUI::update_display  $tile_path  mag  lbl_mag_err_val"
    
    OnChange hook ::Modacc::State(ph,value) "EVM_DataUI::update_display  $tile_path  ph  lbl_phase_err_val"
    
    OnChange hook ::Modacc::State(or,value) "EVM_DataUI::update_display  $tile_path  or  lbl_origin_offset_val"
    
    OnChange hook ::Modacc::State(freq,value) "EVM_DataUI::update_display  $tile_path  freq  lbl_freq_err_val"

    OnChange hook ::Modacc::State(alpha) "EVM_Plot_FullUI::update_mode  $tile_path"
    
    OnChange hook ::C4fm::State(freq_err,status) "meter_text_colors $tile_path.c4fm_freq_err_value ::C4fm::State(freq_err,status)"
    
    OnChange hook ::C4fm::State(deviation,status) "meter_text_colors $tile_path.c4fm_dev_value ::C4fm::State(deviation,status)"

    OnChange hook ::C4fm::State(modacc,status) "meter_text_colors $tile_path.c4fm_percent_value ::C4fm::State(modacc,status)"

    ::EVM_PlotUI::Map $tile_path
  }


  #***************************************************************************
  #
  # Name: Cleanup
  #
  # Parameters:  tile_path
  #
  # Desciption: 
  #
  #***************************************************************************
  proc Cleanup { tile_path } \
  {
    Debug::Puts EVMUI 1 "EVM_Plot_FullUI::Cleanup:  $tile_path"

    OnChange unhook ::Modacc::State(evm,value) "EVM_DataUI::update_display  $tile_path  evm  lbl_evm_val"

    OnChange unhook ::Modacc::State(mag,value) "EVM_DataUI::update_display  $tile_path  mag  lbl_mag_err_val"
    
    OnChange unhook ::Modacc::State(ph,value) "EVM_DataUI::update_display  $tile_path  ph  lbl_phase_err_val"
    
    OnChange unhook ::Modacc::State(or,value) "EVM_DataUI::update_display  $tile_path  or  lbl_origin_offset_val"
    
    OnChange unhook ::Modacc::State(freq,value) "EVM_DataUI::update_display  $tile_path  freq  lbl_freq_err_val"

    OnChange unhook ::Modacc::State(alpha) "EVM_Plot_FullUI::update_mode  $tile_path"
    
    OnChange unhook ::C4fm::State(freq_err,status) "meter_text_colors $tile_path.c4fm_freq_err_value ::C4fm::State(freq_err,status)"
    
    OnChange unhook ::C4fm::State(deviation,status) "meter_text_colors $tile_path.c4fm_dev_value ::C4fm::State(deviation,status)"

    OnChange unhook ::C4fm::State(modacc,status) "meter_text_colors $tile_path.c4fm_percent_value ::C4fm::State(modacc,status)"

    ::EVM_PlotUI::Cleanup $tile_path
  }
  
  
  #***************************************************************************
  #
  # Name: Unmap
  #
  # Parameters:  tile_path
  #
  # Desciption: 
  #
  #***************************************************************************
  proc Unmap { tile_path } \
  {
    Debug::Puts EVMUI 1 "Unmap:  $tile_path"
    # disable RETURN softkey
    ::View::AssignReturn [winfo parent $tile_path] ""
    
    Cleanup $tile_path
  }


  #***************************************************************************
  #
  # Name: Destroy
  #
  # Parameters: 
  #
  # Desciption: 
  #
  #***************************************************************************
  proc Destroy { tile_path } \
  {
    Debug::Puts EVMUI 1 "Destroy:  $tile_path"
    
    Cleanup $tile_path
   
  }


  #***************************************************************************
  #
  # Name: Register
  #
  # Parameters: 
  #
  # Desciption: 
  #
  #***************************************************************************
  proc Register { tile_path } \
  {
    Debug::Puts EVMUI 1 "Register:  $tile_path"
  }
  
  #***************************************************************************
  #
  # Name: update_mode
  #
  # Parameters:  tile_path - tile path of the invoking widget
  #
  # Description: Update the display based upon the mode the EVM system is in
  #              (QPSK or FM)
  #
  #***************************************************************************
  proc update_mode { tile_path } \
  {
      Debug::Puts EVMUI 1 "update_mode:  $tile_path"
      if ![winfo exists ${tile_path}] { return }
      switch -- $::Modacc::State(alpha) \
      {
         0.0 \
         {
            $tile_path.lbl_q_chan config -text Freq
            $tile_path.lbl_i_chan config -text Time
            grid forget $tile_path.evm_frame
            Screens::position_widget $tile_path.c4fm_frame
            C4fm run 1 -nowait
            C4fm en 3 -nowait
            
         }
         default \
         {
            $tile_path.lbl_q_chan config -text Q
            $tile_path.lbl_i_chan config -text I
            grid forget $tile_path.c4fm_frame
            Screens::position_widget $tile_path.evm_frame
         } 
      }
  }

};  # End of EVM_Plot_FullUI namespace

  
namespace eval EVM_PlotUI \
{
  # Global variables
  variable  LongName       "evm_plot"
  variable  ShortName      "Modulation Plot"
  variable  Size           ""

  variable  tile_class     ""

  variable  prev_screen    ""
  variable  StateUI
  set StateUI(const_on_off)      "ON"
  set StateUI(traj_on_off)       "ON"

  set StateUI(persistence)  "OFF"
  set StateUI(persistence,traj)  "OFF"


  #***************************************************************************
  #
  # Name: Create
  #
  # Parameters:
  #
  # Desciption: 
  #
  #***************************************************************************
  proc Create { tile_path LongName tile_class} \
  {
    Debug::Puts EVMUI 1 "EVM_PlotUI::Create $tile_path"

    if { ![winfo exists $tile_path] } \
    {
      set tile_path  [Screens::create_tile  $tile_path  $LongName  $tile_class]
      
     # Configure the Constellation Trace
     ${tile_path}.trce_plot trace create  const                               \
                                         -sys        [Modacc -system]         \
                                         -subsys     [Modacc -subsys const]   \
                                         -cfgsubsys  [Modacc -subsys cconfig] \
                                         -color      white                    \
                                         -refresh    5000

     # Configure the Trajectory Trace
     ${tile_path}.trce_plot trace create  traj                                \
                                         -sys        [Modacc -system]         \
                                         -subsys     [Modacc -subsys traj]    \
                                         -cfgsubsys  [Modacc -subsys tconfig] \
                                         -color      DarkOrange2              \
                                         -refresh    5000

    }  

    return $tile_path
  }

  #*************************************************************************** 
  #
  # Name: Map
  #
  # Parameters: 
  #
  # Desciption: 
  #
  #***************************************************************************
  proc Map { tile_path } \
  {
    Debug::Puts EVMUI 1 "EVM_PlotUI::Map $tile_path"
    # Check to see if the option is enabled
    if { ![IFR_options is_installed EVM] } \
    {
      grid forget $tile_path
    
      return
    }

    OnChange hook ::EVM_PlotUI::StateUI(const_on_off) "EVM_PlotUI::update_const_on_off  $tile_path"

    OnChange hook ::EVM_PlotUI::StateUI(traj_on_off) "EVM_PlotUI::update_traj_on_off  $tile_path"

    OnChange hook ::EVM_PlotUI::StateUI(persistence) "EVM_PlotUI::update_persist  $tile_path"
    OnChange hook ::EVM_PlotUI::StateUI(persistence,traj) "EVM_PlotUI::update_persist_traj  $tile_path"

    # Increment the ON count
    Modacc run 1 -nowait
  }

  #***************************************************************************
  #
  # Name: Cleanup
  #
  # Parameters:  tile_path
  #
  # Desciption: 
  #
  #***************************************************************************
  proc Cleanup { tile_path } \
  {
    Debug::Puts EVMUI 1 "EVM_PlotUI::Cleanup $tile_path"

    OnChange unhook ::EVM_PlotUI::StateUI(const_on_off) "EVM_PlotUI::update_const_on_off  $tile_path"

    OnChange unhook ::EVM_PlotUI::StateUI(traj_on_off) "EVM_PlotUI::update_traj_on_off  $tile_path"
    OnChange unhook ::EVM_PlotUI::StateUI(persistence) "EVM_PlotUI::update_persist  $tile_path"
    OnChange unhook ::EVM_PlotUI::StateUI(persistence,traj) "EVM_PlotUI::update_persist_traj  $tile_path"
    
    # decrement the ON count
    Modacc run -1 -nowait
  }
  

  #***************************************************************************
  #
  # Name: get_tile
  #
  # Parameters: any child widget of the tile
  #
  # Description: get the name of the tile's parent frame
  #
  #***************************************************************************
  proc get_tile { widget } \
  {
    while { 1 } \
    {
      switch -- [winfo class $widget] \
      {
        "QTR_EVM_PLOT"   -
        "FULL_EVM_PLOT"  { return $widget }
      }
        
      set widget [winfo parent $widget]
    }
  }


  #***************************************************************************
  #
  # Name: display_full_plot
  #
  # Parameters: tile_path - tile path of the invoking widget
  #
  # Description: Displays the full tile EVM plot
  #
  #***************************************************************************
  proc display_full_plot { tile_path } \
  {
    Debug::Puts EVMUI 1 "display_full_plot:  $tile_path"

    variable prev_screen

    set prev_screen  $::Screens::CurrentNamespace
    
    Debug::Puts EVMUI 1 "prev_screen:  $prev_screen"

    Screens::Show  $tile_path  EVM_Plot_Screen
    
    # Set up the return key
    set widget  [winfo toplevel $tile_path]
        
    View::AssignReturn  $tile_path  \
                        "::Screens::Show  $widget  $::EVM_PlotUI::prev_screen"

    return
  }


  #***************************************************************************
  #
  #       Name: set_options_menu
  #
  # Parameters: 
  #
  # Description: Assigns the EVM Plot options menu
  #
  #***************************************************************************
  proc set_options_menu { widget  ScreenNamespace } \
  {
    Debug::Puts EVMUI 1 "set_options_menu:  $widget,  $ScreenNamespace"

    set topWin  [winfo toplevel $widget]
    
    # Get the Options menubutton path
    set buttonName  [View::GetOptionMenuName  $topWin]
    
    option add  *$buttonName*helpUrl  index.html  widgetDefault
    
    # Get the Options menu path
    set menuName  ${buttonName}.menu
    
    # Destroy the existing Options menu
    destroy  $menuName
    
    # Create a new Options menu
    menu $menuName -tearoff 0  
    
    # Add ENTER key options
    $menuName add command -accel RETURN            \
                          -label "Accept Options"  \
                          -com "tkMenuUnpost $menuName"
    

    #$menuName add radio  -label     "Live"  \
    #                     -accel     1       \
    #                     -value     LIVE    \
    #                     -variable  ::AnalyzerUI::StateUI(mode)
    #                     
    #bind $menuName <KeyRelease-1> { set ::AnalyzerUI::StateUI(mode) LIVE }
                               

    $menuName add separator


    $menuName add check  -label     "Constellation"              \
                         -accel     1                            \
                         -onvalue   ON                           \
                         -offvalue  OFF                          \
                         -variable  ::EVM_PlotUI::StateUI(const_on_off)
    
    bind $menuName <KeyRelease-1> { ::EVM_PlotUI::toggle_const_on_off }

    $menuName add check  -label     "Trajectory"                \
                         -accel     2                           \
                         -onvalue   ON                          \
                         -offvalue  OFF                         \
                         -variable  ::EVM_PlotUI::StateUI(traj_on_off)
    
    bind $menuName <KeyRelease-2> { ::EVM_PlotUI::toggle_traj_on_off }


    $menuName add separator
    

    # Add ESCAPE functionality
    $menuName add command -accel ESC              \
                          -label "Escape"         \
                          -com  "tkMenuUnpost $menuName"


    # Bind button release so default bindings do not unpost the menu
    # bind to the Return and Escape hardkeys
    bind $menuName <KeyRelease-F8>     "tkMenuUnpost $menuName; break"

    bind $menuName <KeyRelease-Escape> "tkMenuUnpost $menuName; break"

    bind $menuName <KeyRelease-Return> {MenuInvokeRemain %W 0; break}

    bind $menuName <ButtonRelease-1> {MenuInvokeRemain %W 1; break}

    return $buttonName
  } 

  #***************************************************************************
  #
  # Name: toggle_const_on_off
  #
  # Parameters:
  #
  # Description: Toggles the Constellation plot ON and OFF
  #
  #***************************************************************************
  proc toggle_const_on_off { } \
  {
    Debug::Puts EVMUI 1 "toggle_const_on_off"
    
    variable  StateUI
    
    if { $StateUI(const_on_off) == "ON" } \
    {
      set StateUI(const_on_off)  "OFF"
    } \
    else \
    {
      set StateUI(const_on_off)  "ON"
    }
    
    return
  }
  
  

  
  #***************************************************************************
  #
  # Name: update_const_on_off
  #
  # Parameters:  tile_path - tile path of the invoking widget
  #
  # Description: Updates the display to changes in the Constellation 
  #              plot state
  #
  #***************************************************************************
  proc update_const_on_off { tile_path } \
  {

    Debug::Puts EVMUI 1 "update_const_on_off:  $tile_path"
    
    variable  StateUI
    if ![winfo exists ${tile_path}.trce_plot] { return }
    switch -- $StateUI(const_on_off) \
    {
      "ON"   { ${tile_path}.trce_plot.const  config  -active 1 } 
      
      "OFF"  { ${tile_path}.trce_plot.const  config  -active 0 }
    }
  }


  #***************************************************************************
  #
  # Name: toggle_traj_on_off
  #
  # Parameters:
  #
  # Description: Toggles the Trajectory plot ON and OFF
  #
  #***************************************************************************
  proc toggle_traj_on_off { } \
  {
    Debug::Puts EVMUI 1 "toggle_traj_on_off"
    
    variable StateUI

    if { $StateUI(traj_on_off) == "ON" } \
    {
      set StateUI(traj_on_off)  "OFF"
    } \
    else \
    {
      set StateUI(traj_on_off)  "ON"
    }

    return
  }


  #***************************************************************************
  #
  # Name: update_traj_on_off
  #
  # Parameters:  tile_path - tile path of the invoking widget
  #
  # Description: Updates the display to changes in the Trajectory 
  #              plot state
  #
  #***************************************************************************
  proc update_traj_on_off { tile_path } \
  {
    Debug::Puts EVMUI 1 "update_traj_on_off:  $tile_path"
    
    variable  StateUI
    if ![winfo exists ${tile_path}.trce_plot] { return }
    switch -- $StateUI(traj_on_off) \
    {
      "ON"   { ${tile_path}.trce_plot.traj  config  -active 1 } 
      
      "OFF"  { ${tile_path}.trce_plot.traj  config  -active 0 }
    }
  }


  #***************************************************************************
  #
  # Name: toggle_persistence
  #
  # Parameters:
  #
  # Description: Toggles the persistence of the trace widget
  #
  #***************************************************************************
  proc toggle_persistence { } \
  {
    variable  StateUI
  
    switch -- $StateUI(persistence) \
    {
      "OFF"    { set StateUI(persistence)  "ON"  }

      "ON"     -
      default  { set StateUI(persistence)  "OFF" }
    }
    
    return
  }
  
  #***************************************************************************
  #
  # Name: toggle_persistence_traj
  #
  # Parameters:
  #
  # Description: Toggles the persistence of the trajectory trace widget
  #
  #***************************************************************************
  proc toggle_persistence_traj { } \
  {
    variable  StateUI
  
    switch -- $StateUI(persistence,traj) \
    {
      "OFF"    { set StateUI(persistence,traj)  "ON"  }

      "ON"     -
      default  { set StateUI(persistence,traj)  "OFF" }
    }
    
    return
  }
  #****************************************************************************
  #
  # Name: update_persist
  #
  # Parameters:
  #
  # Description: Update the persistance on the constellation trace
  #****************************************************************************
  proc update_persist {tile_path } \
  {
     variable StateUI
     if ![winfo exists ${tile_path}.trce_plot] { return }
     if {$StateUI(persistence) == "ON"} \
     {
        ${tile_path}.trce_plot.const  config  -persist 1
     } \
     else \
     {
        ${tile_path}.trce_plot.const  config  -persist 0
     }
  }
  
  #****************************************************************************
  #
  # Name: update_persist_traj
  #
  # Parameters:
  #
  # Description: Update the persistance on the trajectory trace
  #****************************************************************************
  proc update_persist_traj {tile_path } \
  {
     variable StateUI
     if ![winfo exists ${tile_path}.trce_plot] { return }
     if {$StateUI(persistence,traj) == "ON"} \
     {
        ${tile_path}.trce_plot.traj  config  -persist 1
     } \
     else \
     {
        ${tile_path}.trce_plot.traj  config  -persist 0
     }
  }
  

  #****************************************************************************
  #
  # Name: getSaveState
  #
  # Parameters:
  #
  # Description: This function returns a list of TCL commands which can be 
  #              used to set the EVM_PlotUI tile to its current state
  #
  #****************************************************************************
  proc getSaveState { } \
  {
    variable  StateUI
 
    set returnList \
    {
         set ::EVM_PlotUI::StateUI(persistence) {$StateUI(persistence)}
         set ::EVM_PlotUI::StateUI(persistence,traj) {$StateUI(persistence,traj)}
         set ::EVM_PlotUI::StateUI(traj_on_off) {$StateUI(traj_on_off)}
         set ::EVM_PlotUI::StateUI(const_on_off) {$StateUI(const_on_off)}
    }
       
    return [subst $returnList]
  }


  #****************************************************************************
  #
  # Name: Sync
  #
  # Parameters:
  #
  # Description: Write state variables to EVM_PlotUI
  #
  #****************************************************************************
  proc Sync { } \
  {
    eval [getSaveState]
  }


};  # End of EVM_PlotUI namespace



#****************************************************************************
# This registers the tiles with the list for saving instrument states
#****************************************************************************
namespace eval ::SavedStates \
{
  variable tiles   

  lappend tiles EVM_PlotUI
}

