#############################################################################
#                                                                           #
# IFR SYSTEMS, INC        COPYRIGHT (C)1998,99      DESIGN ENGINEERING      #
# ------------------------------------------------------------------------- #
#                                                                           #
# FILE        : 3530.Tone_Signal_DecodeUI.tcl                               #
#                                                                           #
# PRODUCT     : FITS                                                        #
#                                                                           #
# AUTHOR      : P. McCormick                                                #
#                                                                           #
# DESCRIPTION : Tone Signaling Decode User Interface                        #
#                                                                           #
#############################################################################

namespace eval Tone_Signal_DecodeUI \
{
  # Global variables
  variable  LongName   "ts_decode"
  variable  ShortName  "Tone Signal Decode"
  variable  Size       Quarter

  variable  tile_class   "TS_DECODE"

  variable  last_focus
  
  variable  StateUI
  
  set StateUI(mode)  "OFF"

  set StateUI(dev_units)  ""

  set StateUI(mode_menu)  { "OFF"  "DCS"  "DTMF"  "CTCSS" }
  set StateUI(mode,old) ""

  #**************************************************************************
  #
  #        Name: Create  
  #
  #  Parameters:
  #
  # Decsription: 
  #
  #**************************************************************************
  proc Create { tile_path  } \
  {
    variable  LongName
    variable  tile_class

    set top_level  [winfo toplevel $tile_path]
   
    set tile_path  [name_child  $top_level  $LongName]


    if { ![winfo exists $tile_path] } \
    {
      set tile_path  [Screens::create_tile  $tile_path  $LongName  $tile_class]
    }
        
    return $tile_path
  }


  #**************************************************************************
  #
  #        Name: Map
  #
  #  Parameters: 
  #
  # Decsription: 
  #
  #**************************************************************************
  proc Map { tile_path } \
  {
    #puts "Tone_Signal_DecodeUI::Map:  $tile_path"

    variable  StateUI
    
    trace variable ::Tone_Signal_DecodeUI::StateUI(mode)  w  \
                             "Tone_Signal_DecodeUI::update_mode  $tile_path"

    update_mode  $tile_path


    trace variable ::Dcs::State(status)  w  \
                     "Tone_Signal_DecodeUI::update_decode_color  $tile_path"

    trace variable ::AFCounter::State(enabled2)  w  \
                     "Tone_Signal_DecodeUI::update_decode_state  $tile_path"

    return
  }
  
  
  #**************************************************************************
  #
  #        Name: Cleanup
  #
  #  Parameters: 
  #
  # Decsription: 
  #
  #**************************************************************************
  proc Cleanup { tile_path } \
  {
    #puts "Tone_Signal_DecodeUI::Cleanup:  $tile_path"

    trace vdelete ::Tone_Signal_DecodeUI::StateUI(mode)  w  \
                             "Tone_Signal_DecodeUI::update_mode  $tile_path"

    trace vdelete ::Dcs::State(status)  w  \
                     "Tone_Signal_DecodeUI::update_decode_color  $tile_path"

    trace vdelete ::AFCounter::State(enabled2)  w  \
                     "Tone_Signal_DecodeUI::update_decode_state  $tile_path"

    return
  }


  #**************************************************************************
  #
  #        Name: Unmap
  #
  #  Parameters: 
  #
  # Decsription: 
  #
  #**************************************************************************
  proc Unmap { tile_path } \
  {
    #puts "Tone_Signal_DecodeUI::Unmap"
    
    Cleanup $tile_path
    
    return
  }


  #**************************************************************************
  #
  #        Name: Destroy
  #
  #  Parameters: 
  #
  # Decsription: 
  #
  #**************************************************************************
  proc Destroy { tile_path } \
  {
    #puts "Tone_Signal_DecodeUI::Destroy"
    
    Cleanup $tile_path
    
    return
  }


  #**************************************************************************
  #
  #        Name: Register
  #
  #  Parameters: 
  #
  # Decsription: 
  #
  #**************************************************************************
  proc Register { tile_path } \
  {
    #puts "Tone_Signal_DecodeUI::Register"     
    
    return
  }


  #***************************************************************************
  #
  #        Name: get_tile
  #
  #  Parameters: any child widget of the tile
  #
  # Description: Get the name of the tile's parent frame
  #
  #***************************************************************************
  proc get_tile { widget } \
  {
     while { 1 } \
     {
       switch -- [winfo class $widget] \
       {
         "TS_DECODE"  { return $widget }
       }
        
       set widget [winfo parent $widget]
     }
     
  }


  #**************************************************************************
  #
  #        Name: set_mode
  #
  #  Parameters:
  #
  # Description: Sets the decode mode of the Reciever
  #
  #**************************************************************************
  proc set_mode { } \
  {
    #puts "set_mode"
    
    variable  StateUI
    
    if {$StateUI(mode,old) == $StateUI(mode)} \
    {
       return;
    }
    
    Dcs  run  0
    Dcs  en   3
      
    Dtmf run  0
    Dtmf en   3
    
    AFCounter::QuickStop2

    switch -- $StateUI(mode) \
    {
      "OFF"    { }
      
      "DCS"    { after idle  "Dcs run 1"              }
      
      "DTMF"   { after idle  "Dtmf run 1"             }
      
      "CTCSS"  { after idle  "AFCounter::QuickStart2" }
      
      default  { }
    }
    
    set StateUI(decode_received)  ""
    set StateUI(mode,old) $StateUI(mode)
    return
  }
  
    
  #**************************************************************************
  #
  #        Name: update_mode
  #
  #  Parameters: tile_path - tile path of the invoking widget
  #
  # Description: Updates the display to the decode mode
  #
  #**************************************************************************
  proc update_mode { tile_path  args } \
  {
    #puts "update_mode:  $tile_path"
    
    variable  StateUI
    
    set tile_path  [get_tile  $tile_path]
    
    grid remove  ${tile_path}.frm_off
    grid remove  ${tile_path}.frm_dtmf
    grid remove  ${tile_path}.frm_dcs
    grid remove  ${tile_path}.frm_ctcss
    
    switch -- $StateUI(mode) \
    {
      "OFF"    { Screens::position_widget  ${tile_path}.frm_off   }
      
      "DCS"    { Screens::position_widget  ${tile_path}.frm_dcs   }
      
      "DTMF"   { Screens::position_widget  ${tile_path}.frm_dtmf  }
      
      "CTCSS"  { Screens::position_widget  ${tile_path}.frm_ctcss }

      default  { }
    }
    set_mode
    return
  }
  

  #**************************************************************************
  #
  #        Name: update_decode_color
  #
  #  Parameters: tile_path - tile path of the invoking widget
  #
  # Description: Updates the foreground color of the DCS code widget to
  #              changes in the decode engine status
  #
  #**************************************************************************
  proc update_decode_color { tile_path  args } \
  {
    #puts "update_decode_color:  $tile_path,  $args"
    
    variable  StateUI
    
    set tile_path  [get_tile  $tile_path]
    
    switch -- $StateUI(mode) \
    {
      "DCS"  { if { $::Dcs::State(status) == 0 } \
               {
                 ${tile_path}.frm_dcs.ent_received     config  -fg black
                 ${tile_path}.frm_dcs.ent_code         config  -fg black
                 ${tile_path}.frm_dcs.ent_invert_code  config  -fg black
               } \
               else \
               {
                 ${tile_path}.frm_dcs.ent_received     config  -fg yellow
                 ${tile_path}.frm_dcs.ent_code         config  -fg yellow
                 ${tile_path}.frm_dcs.ent_invert_code  config  -fg yellow
               }
             }
    }

    return
  }


  #**************************************************************************
  #
  #        Name: update_decode_state
  #
  #  Parameters: tile_path - tile path of the invoking widget
  #
  # Description: Updates the state of the tile if the CTCSS mode is
  #              selected and the then the CTCSS counter (AF Counter lun #2)
  #              is turned off.
  #
  #**************************************************************************
  proc update_decode_state { tile_path  args } \
  {
    #puts "update_decode_state:  $tile_path,  $args"
    
    variable  StateUI
    
    set tile_path  [get_tile  $tile_path]

    if { $StateUI(mode) == "CTCSS" } \
    {
      if { $::AFCounter::State(enabled2) == "Disabled" } \
      {
        after 1000 { set Tone_Signal_DecodeUI::StateUI(mode)  "OFF" }
      } \
      else \
      {
         after cancel { set Tone_Signal_DecodeUI::StateUI(mode)  "OFF" }
      }
    }
    return
  }
  
  
  #****************************************************************************
  #
  #        Name: getSaveState
  #
  #  Parameters: none
  #
  # Description: This function returns a list of TCL commands which can be 
  #              used to set the Tone_Signal_DecodeUI tiles to its current 
  #              state
  #
  #****************************************************************************
  proc getSaveState { } \
  {
    variable  StateUI
    
    set returnList \
    {
      set ::Tone_Signal_DecodeUI::StateUI(mode)  {$StateUI(mode)}
    }

    #puts "returnList:  $returnList"
    
    return [subst -nocommands $returnList]
  }


};  # End of Tone_Signal_DecodeUI namespace


#****************************************************************************
# This registers the tiles with the list for saving instrument states
#****************************************************************************
namespace eval ::SavedStates \
{
   variable tiles   

   lappend tiles  Tone_Signal_DecodeUI
}


