#############################################################################
#                                                                           #
# IFR, An Aeroflex Co.                                                      #
#                                                                           #
# COPYRIGHT (C) 2003                                                        #
#                                                                           #
# DESIGN ENGINEERING                                                        #
# ------------------------------------------------------------------------- #
#                                                                           #
# FILE        : 4080.LTR_Repeat_SimUI.tcl                                   #
#                                                                           #
# PRODUCT     : FITS                                                        #
#                                                                           #
# AUTHOR      : P. McCormick                                                #
#                                                                           #
# DESCRIPTION : LTR Repeater Simulator User Interface                       #
#                                                                           #
#############################################################################

package require Ltr

namespace eval LTR_Repeat_SimUI \
{
  # Global variables
  variable  LongName      "ltr_repeat_sim"
  variable  ShortName     "LTR Repeater Sim"
  variable  Size          Quarter
  
            
  variable  tile_class    "LTR_REPEAT_SIM"

  variable  last_focus
  
  variable  StateUI

  set StateUI(display)  "TRANSMIT SETUP"
  
  set StateUI(display_menu)  { "TRANSMIT SETUP"  "RECEIVED DATA" }
  
    
  #***************************************************************************
  #
  #        Name: Create  
  #
  #  Parameters: tile_path - tile path of the invoking widget
  #
  # Description: Create the tile from the resource database
  #
  #***************************************************************************
  proc Create { tile_path } \
  {
    #puts "LTR_Repeat_SimUI::Create:  $tile_path"
    
    variable  LongName
    variable  tile_class
    
    set top_level  [winfo toplevel $tile_path]
   
    set tile_path  [name_child  $top_level  $LongName]

    #puts "tile_path:  $tile_path"

    if { ![winfo exists $tile_path] } \
    {
      set tile_path  [Screens::create_tile  $tile_path  $LongName  $tile_class]
    }  
     
    return $tile_path
  }


  #***************************************************************************
  #
  #        Name: Map
  #
  #  Parameters: 
  #
  # Description: Initializes the tile
  #
  #***************************************************************************
  proc Map { tile_path } \
  {
    #puts "LTR_Repeat_SimUI::Map: $tile_path"

    if { ![IFR_options  is_installed  LTR] } \
    {
      grid forget $tile_path
     
      return
    }
    
    #trace variable  ::Ltr::State(uutptt)  w  \
    #                         "LTR_Repeat_SimUI::update_ptt_rec  $tile_path"
    #
    #update_ptt_rec  $tile_path

    trace variable  ::Ltr::State(tx_rstate)  w  \
                             "LTR_Repeat_SimUI::update_state_button  $tile_path"
    set_display $tile_path
    LTR_Sim_ConfigUI::set_sim_freq_list
  }


  #***************************************************************************
  #
  #        Name: Cleanup
  #
  #  Parameters: 
  #
  # Description: 
  #
  #***************************************************************************
  proc Cleanup { tile_path } \
  {
    #puts "LTR_Repeat_SimUI::Cleanup:  $tile_path"
    
    #trace vdelete  ::Ltr::State(uutptt)  w  \
    #                         "LTR_Repeat_SimUI::update_ptt_rec  $tile_path"

    trace vdelete  ::Ltr::State(tx_rstate)  w  \
                             "LTR_Repeat_SimUI::update_state_button  $tile_path"
  }


  #***************************************************************************
  #
  #        Name: Unmap
  #
  #  Parameters: 
  #
  # Description: 
  #
  #***************************************************************************
  proc Unmap { tile_path } \
  {
    #puts "LTR_Repeat_SimUI::Unmap:  $tile_path"
    
    Cleanup  $tile_path
  }


  #***************************************************************************
  #
  #        Name: Destroy
  #
  #  Parameters: 
  #
  # Description: 
  #
  #***************************************************************************
  proc Destroy { tile_path } \
  {
    #puts "LTR_Repeat_SimUI::Destroy:  $tile_path"
    
    Cleanup  $tile_path
  }


  #***************************************************************************
  #
  #        Name: Register
  #
  #  Parameters: 
  #
  # Description: 
  #
  #***************************************************************************
  proc Register { } \
  {
    #puts "LTR_Repeat_SimUI::Register"
  }
  

  #***************************************************************************
  #
  #        Name: get_tile
  #
  #  Parameters: any child widget of the tile
  #
  # Description: Get the name of the tile's parent frame
  #
  #***************************************************************************
  proc get_tile { widget } \
  {
    while { 1 } \
    {
      switch -- [winfo class $widget] \
      {
        "LTR_REPEAT_SIM"   { return $widget }
      }
        
      set widget [winfo parent $widget]
    }
  }


  #***************************************************************************
  #
  #        Name: toggle_display
  #
  #  Parameters: tile_path - tile path of the invoking widget
  #
  # Description: Toggles display of the Simulator
  #
  #***************************************************************************
  proc toggle_display { tile_path } \
  {
    #puts "toggle_display:  $tile_path"
    
    variable  StateUI
    
    set tile_path  [get_tile  $tile_path]

    grid remove  ${tile_path}.frm_transmit
    grid remove  ${tile_path}.frm_receive
    
    if { $StateUI(display) == "TRANSMIT SETUP" } \
    {
      set StateUI(display)  "RECEIVED DATA"

      Screens::position_widget  ${tile_path}.frm_receive
    } \
    else \
    {
      set StateUI(display)  "TRANSMIT SETUP"

      Screens::position_widget  ${tile_path}.frm_transmit
    }
  }


  #***************************************************************************
  #
  #        Name: set_display
  #
  #  Parameters: tile_path - tile path of the invoking widget
  #
  # Description: Sets display of the Simulator
  #
  #***************************************************************************
  proc set_display { tile_path } \
  {
    #puts "set_display:  $tile_path"
    
    variable  StateUI
    
    set tile_path  [get_tile  $tile_path]

    grid remove  ${tile_path}.frm_transmit
    grid remove  ${tile_path}.frm_receive
    
    switch -- $StateUI(display) \
    {
      "TRANSMIT SETUP"  { Screens::position_widget  ${tile_path}.frm_transmit }

      "RECEIVED DATA"   { Screens::position_widget  ${tile_path}.frm_receive  }
    }
  }


  #***************************************************************************
  #
  #        Name: toggle_sim_on_off
  #
  #  Parameters:
  #
  # Description: Toggles the Simulator ON or OFF
  #
  #***************************************************************************
  proc toggle_sim_on_off { } \
  {
    #puts "toggle_sim_on_off"
    
    variable  StateUI

    if { $::Ltr::State(repeat_sim_on_off) == 0 } \
    {
      # Send down the present values
      Ltr area     $::Ltr::State(tx_area) -nowait
      Ltr goto     $::Ltr::State(tx_goto) -nowait
      Ltr home     $::Ltr::State(tx_home) -nowait
      Ltr group    $::Ltr::State(tx_group) -nowait
      Ltr riccode  $::Ltr::State(tx_riccode) -nowait
      Ltr free     $::Ltr::State(tx_free) -nowait
      Ltr rstate   0 -nowait
                 
      # Turn ON the Repeater Simulator
      Ltr::repeaterSimTurnOn
    } \
    else \
    {
      # Turn OFF the Repeater Simulator
      Ltr::ltrTurnOff
    }
  }


  #***************************************************************************
  #
  #        Name: set_transmit_area
  #
  #  Parameters:
  #
  # Description: Sets the AREA value of the Transmit message
  #
  #***************************************************************************
  proc set_transmit_area { } \
  {
    #puts "set_transmit_area
    
    after idle "Ltr area $::Ltr::State(tx_area) -nowait"
  }


  #***************************************************************************
  #
  #        Name: set_transmit_goto
  #
  #  Parameters:
  #
  # Description: Sets the GOTO value of the Transmit message
  #
  #***************************************************************************
  proc set_transmit_goto { } \
  {
    #puts "set_transmit_goto
    
    after idle "Ltr goto $::Ltr::State(tx_goto) -nowait"
  }


  #***************************************************************************
  #
  #        Name: set_transmit_home
  #
  #  Parameters:
  #
  # Description: Sets the HOME value of the Transmit message
  #
  #***************************************************************************
  proc set_transmit_home { } \
  {
    #puts "set_transmit_home
    
    after idle "Ltr home $::Ltr::State(tx_home) -nowait"
  }


  #***************************************************************************
  #
  #        Name: set_transmit_group
  #
  #  Parameters:
  #
  # Description: Sets the GROUP value of the Transmit message
  #
  #***************************************************************************
  proc set_transmit_group { } \
  {
    #puts "set_transmit_group
    
    after idle "Ltr group $::Ltr::State(tx_group) -nowait"
  }


  #***************************************************************************
  #
  #        Name: set_transmit_free
  #
  #  Parameters:
  #
  # Description: Sets the FREE value of the Transmit message
  #
  #***************************************************************************
  proc set_transmit_free { } \
  {
    #puts "set_transmit_free
    
    after idle "Ltr free $::Ltr::State(tx_free) -nowait"
  }


  #***************************************************************************
  #
  #        Name: set_transmit_riccode
  #
  #  Parameters:
  #
  # Description: Sets the RIC Code value of the Transmit message
  #
  #***************************************************************************
  proc set_transmit_riccode { } \
  {
    #puts "set_transmit_riccode
    
    after idle "Ltr riccode $::Ltr::State(tx_riccode) -nowait"
  }


  #***************************************************************************
  #
  #        Name: set_transmit_state
  #
  #  Parameters:
  #
  # Description: Sets the Status of the Transmit message
  #
  #***************************************************************************
  proc set_transmit_state { } \
  {
    #puts "set_transmit_state
    
    switch -- $::Ltr::State(tx_rstate) \
    {
      "IDLE"    -
      "DISCON"   { after idle "Ltr rstate  0 -nowait" }
      "SYS BUSY" { after idle "Ltr rstate  1 -nowait" }
      "REP BUSY" { after idle "Ltr rstate  2 -nowait" }
      "CONT"     { after idle "Ltr rstate  3 -nowait" }
    }
  }


  #***************************************************************************
  #
  #        Name: set_transmit_invert
  #
  #  Parameters:
  #
  # Description: Sets the Invert Mode
  #
  #***************************************************************************
  proc set_transmit_invert { } \
  {
    #puts "RSIM:set_transmit_invert
    
    Ltr  params  $::Ltr::State(tx_deviation)  $::Ltr::State(tx_invert)  0x$::Ltr::State(tx_sync,hex) -nowait
  }


  #***************************************************************************
  #
  #        Name: set_transmit_deviation
  #
  #  Parameters:
  #
  # Description: Sets the transmit deviation
  #
  #***************************************************************************
  proc set_transmit_deviation { } \
  {
    #puts "RSIM:set_transmit_deviation
    
    Ltr  params  $::Ltr::State(tx_deviation)  $::Ltr::State(tx_invert)  0x$::Ltr::State(tx_sync,hex) -nowait
  }

  #***************************************************************************
  #
  #        Name: update_state_button
  #
  #  Parameters:
  #
  # Description: Sets the transmit deviation
  #
  #***************************************************************************
  proc update_state_button { widget args } \
  {
    #puts "RSIM:update_state_button
    
    set tile_path  [get_tile $widget]
 
    if {$::Ltr::State(tx_rstate) == "WAIT..."} \
    {
        ${tile_path}.frm_transmit.mnu_state config -fg gray90 -bg gray60
    } else \
    {
        ${tile_path}.frm_transmit.mnu_state config -fg black -bg gray
    }
  }



  #****************************************************************************
  #
  #        Name: getSaveState
  #
  #  Parameters: none
  #
  # Description: This function returns a list of TCL commands which can be used
  #              to set the LTR_Repeat_SimUI tiles to its current state
  #
  #****************************************************************************
  proc getSaveState { } \
  {
    variable  StateUI

    set returnList \
    {
      array set ::LTR_Repeat_SimUI::StateUI \
      {
         [array get StateUI]
      }
    }
    
    return [subst $returnList]
  }


  #****************************************************************************
  #
  #        Name: Sync
  #
  #  Parameters: none
  #
  # Description: Write state variables to LTR_Repeat_SimUI
  #
  #****************************************************************************
  proc Sync { } \
  {
    eval [getSaveState]
  }


};  # End of LTR_Repeat_SimUI namespace


#****************************************************************************
# This registers the tiles with the list for saving instrument states
#****************************************************************************
namespace eval ::SavedStates \
{
  variable tiles
  
  lappend tiles  LTR_Repeat_SimUI
}


