#############################################################################
#                                                                           #
# IFR, An Aeroflex Co.                                                      #
#                                                                           #
# COPYRIGHT (C) 2003                                                        #
#                                                                           #
# DESIGN ENGINEERING                                                        #
# ------------------------------------------------------------------------- #
#                                                                           #
# FILE        : 4090.LTR_Radio_SimUI.tcl                                    #
#                                                                           #
# PRODUCT     : FITS                                                        #
#                                                                           #
# AUTHOR      : P. McCormick                                                #
#                                                                           #
# DESCRIPTION : LTR Radio Simulator User Interface                          #
#                                                                           #
#############################################################################

package require Ltr

namespace eval LTR_Radio_SimUI \
{
  # Global variables
  variable  LongName      "ltr_radio_sim"
  variable  ShortName     "LTR Radio Sim"
  variable  Size          Quarter
  
            
  variable  tile_class    "LTR_RADIO_SIM"

  variable  last_focus
  
  variable  StateUI

  set StateUI(display)  "TRANSMIT SETUP"

  set StateUI(display_menu)  { "TRANSMIT SETUP"  "RECEIVED DATA" }


    
  #***************************************************************************
  #
  #        Name: Create  
  #
  #  Parameters: tile_path - tile path of the invoking widget
  #
  # Description: Create the tile from the resource database
  #
  #***************************************************************************
  proc Create { tile_path } \
  {
    #puts "LTR_Radio_SimUI::Create:  $tile_path"
    
    variable  LongName
    variable  tile_class
    
    set top_level  [winfo toplevel $tile_path]
   
    set tile_path  [name_child  $top_level  $LongName]

    #puts "tile_path:  $tile_path"

    if { ![winfo exists $tile_path] } \
    {
      set tile_path  [Screens::create_tile  $tile_path  $LongName  $tile_class]
    }  
     
    return $tile_path
  }


  #***************************************************************************
  #
  #        Name: Map
  #
  #  Parameters: 
  #
  # Description: Initializes the tile
  #
  #***************************************************************************
  proc Map { tile_path } \
  {
    #puts "LTR_Radio_SimUI::Map: $tile_path"

    if { ![IFR_options  is_installed  LTR] } \
    {
      grid forget $tile_path
     
      return
    }
    
    #trace variable  ::LTR::State(uutptt)  w  \
    #                         "LTR_Radio_SimUI::update_ptt_rec  $tile_path"
    #
    #update_ptt_rec  $tile_path
    set_display $tile_path
    LTR_Sim_ConfigUI::set_sim_freq_list
  }


  #***************************************************************************
  #
  #        Name: Cleanup
  #
  #  Parameters: 
  #
  # Description: 
  #
  #***************************************************************************
  proc Cleanup { tile_path } \
  {
    #puts "LTR_Radio_SimUI::Cleanup:  $tile_path"
    
    #trace vdelete  ::LTR::State(uutptt)  w  \
    #                         "LTR_Radio_SimUI::update_ptt_rec  $tile_path"

    return
  }


  #***************************************************************************
  #
  #        Name: Unmap
  #
  #  Parameters: 
  #
  # Description: 
  #
  #***************************************************************************
  proc Unmap { tile_path } \
  {
    #puts "LTR_Radio_SimUI::Unmap:  $tile_path"
    
    Cleanup  $tile_path
  }


  #***************************************************************************
  #
  #        Name: Destroy
  #
  #  Parameters: 
  #
  # Description: 
  #
  #***************************************************************************
  proc Destroy { tile_path } \
  {
    #puts "LTR_Radio_SimUI::Destroy:  $tile_path"
    
    Cleanup  $tile_path
  }


  #***************************************************************************
  #
  #        Name: Register
  #
  #  Parameters: 
  #
  # Description: 
  #
  #***************************************************************************
  proc Register { } \
  {
    #puts "LTR_Radio_SimUI::Register"
  }
  

  #***************************************************************************
  #
  #        Name: get_tile
  #
  #  Parameters: any child widget of the tile
  #
  # Description: Get the name of the tile's parent frame
  #
  #***************************************************************************
  proc get_tile { widget } \
  {
    while { 1 } \
    {
      switch -- [winfo class $widget] \
      {
        "LTR_RADIO_SIM"   { return $widget }
      }
        
      set widget [winfo parent $widget]
    }
  }


  #***************************************************************************
  #
  #        Name: toggle_display
  #
  #  Parameters: tile_path - tile path of the invoking widget
  #
  # Description: Toggles display of the Simulator
  #
  #***************************************************************************
  proc toggle_display { tile_path } \
  {
    #puts "toggle_display:  $tile_path"
    
    variable  StateUI
    
    set tile_path  [get_tile  $tile_path]

    grid remove  ${tile_path}.frm_transmit
    grid remove  ${tile_path}.frm_receive
    
    if { $StateUI(display) == "TRANSMIT SETUP" } \
    {
      set StateUI(display)  "RECEIVED DATA"

      Screens::position_widget  ${tile_path}.frm_receive
    } \
    else \
    {
      set StateUI(display)  "TRANSMIT SETUP"

      Screens::position_widget  ${tile_path}.frm_transmit
    }
  }


  #***************************************************************************
  #
  #        Name: set_display
  #
  #  Parameters: tile_path - tile path of the invoking widget
  #
  # Description: Sets display of the Simulator
  #
  #***************************************************************************
  proc set_display { tile_path } \
  {
    #puts "set_display:  $tile_path"
    
    variable  StateUI
    
    set tile_path  [get_tile  $tile_path]

    grid remove  ${tile_path}.frm_transmit
    grid remove  ${tile_path}.frm_receive
    
    switch -- $StateUI(display) \
    {
      "TRANSMIT SETUP"  { Screens::position_widget  ${tile_path}.frm_transmit }

      "RECEIVED DATA"   { Screens::position_widget  ${tile_path}.frm_receive  }
    }
  }


  #***************************************************************************
  #
  #        Name: toggle_radio_sim_on_off
  #
  #  Parameters:
  #
  # Description: Toggles the Simulator ON or OFF
  #
  #***************************************************************************
  proc toggle_radio_sim_on_off { } \
  {
    #puts "toggle_radio_sim_on_off"
    
    if { $::Ltr::State(radio_sim_on_off) == 0 } \
    { 
      # Send down the present values
      Ltr area     $::Ltr::State(tx_area)
      Ltr home     $::Ltr::State(tx_home)
      Ltr group    $::Ltr::State(tx_group)
      Ltr free     $::Ltr::State(tx_free)
      Ltr mstate   0

      # Turn ON the Radio Simulator
      Ltr::radioSimTurnOn
    } \
    else \
    {
      # Turn OFF the Radio Simulator
      Ltr::ltrTurnOff
    }
  }


  #***************************************************************************
  #
  #        Name: set_transmit_area
  #
  #  Parameters:
  #
  # Description: Sets the AREA value of the Transmit message
  #
  #***************************************************************************
  proc set_transmit_area { } \
  {
    #puts "set_transmit_area
    
    after idle "Ltr area $::Ltr::State(tx_area) -nowait"
  }


  #***************************************************************************
  #
  #        Name: set_transmit_in_use
  #
  #  Parameters:
  #
  # Description: Sets the IN USE value of the Transmit message
  #
  #***************************************************************************
  proc set_transmit_in_use { } \
  {
    #puts "set_transmit_in_use
    
    after idle "Ltr inuse $::Ltr::State(tx_in_use) -nowait"
  }


  #***************************************************************************
  #
  #        Name: set_transmit_home
  #
  #  Parameters:
  #
  # Description: Sets the HOME value of the Transmit message
  #
  #***************************************************************************
  proc set_transmit_home { } \
  {
    #puts "set_transmit_home
    
    after idle "Ltr home $::Ltr::State(tx_home) -nowait"
  }


  #***************************************************************************
  #
  #        Name: set_transmit_group
  #
  #  Parameters:
  #
  # Description: Sets the GROUP value of the Transmit message
  #
  #***************************************************************************
  proc set_transmit_group { } \
  {
    #puts "set_transmit_group
    
    after idle "Ltr group $::Ltr::State(tx_group) -nowait"
  }


  #***************************************************************************
  #
  #        Name: set_transmit_free
  #
  #  Parameters:
  #
  # Description: Sets the FREE value of the Transmit message
  #
  #***************************************************************************
  proc set_transmit_free { } \
  {
    #puts "set_transmit_free
    
    after idle "Ltr free $::Ltr::State(tx_free) -nowait"
  }


  #***************************************************************************
  #
  #        Name: set_transmit_state
  #
  #  Parameters:
  #
  # Description: Sets the State of the Radio Simulator
  #
  #***************************************************************************
  proc set_transmit_state { } \
  {
    #puts "set_transmit_state
    
    switch -- $::Ltr::State(tx_mstate) \
    {
      "IDLE"  { after idle "Ltr mstate  0 -nowait" }
           
      "PTT"   { after idle "Ltr mstate  1 -nowait" }
    }

    return
  }


  #***************************************************************************
  #
  #        Name: set_transmit_invert
  #
  #  Parameters:
  #
  # Description: Sets the Invert Mode
  #
  #***************************************************************************
  proc set_transmit_invert { } \
  {
    #puts "RSIM:set_transmit_invert
    
    Ltr  params  $::Ltr::State(tx_deviation)  $::Ltr::State(tx_invert)  0x$::Ltr::State(tx_sync,hex) -nowait
  }


  #***************************************************************************
  #
  #        Name: set_transmit_deviation
  #
  #  Parameters:
  #
  # Description: Sets the transmit deviation
  #
  #***************************************************************************
  proc set_transmit_deviation { } \
  {
    #puts "RSIM:set_transmit_deviation
    
    Ltr  params  $::Ltr::State(tx_deviation)  $::Ltr::State(tx_invert)  0x$::Ltr::State(tx_sync,hex) -nowait
  }


  #****************************************************************************
  #
  #        Name: getSaveState
  #
  #  Parameters: none
  #
  # Description: This function returns a list of TCL commands which can be used
  #              to set the LTR_Radio_SimUI tiles to its current state
  #
  #****************************************************************************
  proc getSaveState { } \
  {
    variable  StateUI

    set returnList \
    {
      array set ::LTR_Radio_SimUI::StateUI \
      {
         [array get StateUI]
      }
    }
        
    return [subst $returnList]
  }


  #****************************************************************************
  #
  #        Name: Sync
  #
  #  Parameters: none
  #
  # Description: Write state variables to LTR_Radio_SimUI
  #
  #****************************************************************************
  proc Sync { } \
  {
    eval [getSaveState]
  }


};  # End of LTR_Radio_SimUI namespace


#****************************************************************************
# This registers the tiles with the list for saving instrument states
#****************************************************************************
namespace eval ::SavedStates \
{
  variable tiles
  
  lappend tiles LTR_Radio_SimUI
}


