#############################################################################
#                                                                           #
# IFR, An Aeroflex Co.   COPYRIGHT (C) 2003            DESIGN ENGINEERING   #
# ------------------------------------------------------------------------- #
#                                                                           #
# FILE        : 4100.LTR_Sim_ConfigUI.tcl                                   #
#                                                                           #
# PRODUCT     : FITS                                                        #
#                                                                           #
# AUTHOR      : P. McCormick                                                #
#                                                                           #
# DESCRIPTION : LTR Simulator Configuration User Interface                  #
#                                                                           #
#############################################################################

package require IFR_options

namespace eval LTR_Sim_ConfigUI \
{
  # Global variables
  variable  LongName     "ltr_config"
  variable  ShortName    "LTR_CONFIG"
  variable  Size         Half_v
            
  variable  tile_class   "LTR_CONFIG"

  variable  last_focus
  variable  ret_namespace
  variable  please_wait   ""
  
  variable  file_label  "LTR_Config"

  variable  config_dir  "/usr/ltr_data/"

  variable  max_chan_index  20
  
  variable  prev_chan  1
  variable  scroll_on  0

  variable  StateUI
  
  set StateUI(chan)  1

  for { set index  1 } { $index <= $max_chan_index } { incr index  1 } \
  {
    set StateUI($index,band)            "800"
    set StateUI($index,band,def)        "800"

    set StateUI($index,freq_tx)         [format  "%.6f"  [Ltr  fcctx  $index]]
    set StateUI($index,freq_tx,def)     [format  "%.6f"  [Ltr  fcctx  $index]]

    set StateUI($index,freq_rx)         [format  "%.6f"  [Ltr  fccrx  $index]]
    set StateUI($index,freq_rx,def)     [format  "%.6f"  [Ltr  fccrx  $index]]

    set StateUI($index,fcc)             $index
    set StateUI($index,fcc,def)         $index

    set StateUI($index,border_bit)      0
    set StateUI($index,border_bit,def)  0
  }
  
    # Setup values for the in house test radios
    # Standard brand radio
      # Set the band to '800' to retrieve the right values 
      # from the call to 'fcctx' and 'fccrx'
      Ltr band 0
      
      set StateUI(8,band,def)        "800"
      set StateUI(8,freq_tx,def)     [format  "%.6f"  [Ltr  fcctx  521]]
      set StateUI(8,freq_rx,def)     [format  "%.6f"  [Ltr  fccrx  521]]
      set StateUI(8,fcc,def)         521
      set StateUI(8,border_bit,def)  0
    
      set StateUI(4,band,def)        "800"
      set StateUI(4,freq_tx,def)     [format  "%.6f"  [Ltr  fcctx  561]]
      set StateUI(4,freq_rx,def)     [format  "%.6f"  [Ltr  fccrx  561]]
      set StateUI(4,fcc,def)         561
      set StateUI(4,border_bit,def)  0
    
    # King radio
      set StateUI(5,band,def)        "800"
      set StateUI(5,freq_tx,def)     [format  "%.6f"  [Ltr  fcctx  258]]
      set StateUI(5,freq_rx,def)     [format  "%.6f"  [Ltr  fccrx  258]]
      set StateUI(5,fcc,def)         258
      set StateUI(5,border_bit,def)  0
    
      set StateUI(9,band,def)        "800"
      set StateUI(9,freq_tx,def)     [format  "%.6f"  [Ltr  fcctx  298]]
      set StateUI(9,freq_rx,def)     [format  "%.6f"  [Ltr  fccrx  298]]
      set StateUI(9,fcc,def)         298
      set StateUI(9,border_bit,def)  0
    
    # EF Johnson Radio
      # East Parameters
      set StateUI(1,band,def)        "USER"
      set StateUI(1,freq_tx,def)     461.412500
      set StateUI(1,freq_rx,def)     466.412500
      set StateUI(1,fcc,def)         ""
      set StateUI(1,border_bit,def)  ""
    
      set StateUI(11,band,def)        "USER"
      set StateUI(11,freq_tx,def)     462.062500
      set StateUI(11,freq_rx,def)     467.062500
      set StateUI(11,fcc,def)         ""
      set StateUI(11,border_bit,def)  ""
    
      # Special Ed Parameters
      set StateUI(3,band,def)        "USER"
      set StateUI(3,freq_tx,def)     463.487500
      set StateUI(3,freq_rx,def)     468.487500
      set StateUI(3,fcc,def)         ""
      set StateUI(3,border_bit,def)  ""

      set StateUI(13,band,def)        "USER"
      set StateUI(13,freq_tx,def)     463.787500
      set StateUI(13,freq_rx,def)     468.787500
      set StateUI(13,fcc,def)         ""
      set StateUI(13,border_bit,def)  ""

      set StateUI(18,band,def)        "USER"
      set StateUI(18,freq_tx,def)     463.937500
      set StateUI(18,freq_rx,def)     468.937500
      set StateUI(18,fcc,def)         ""
      set StateUI(18,border_bit,def)  ""

    # EF Johnson Repeater
      # Set the band to '800' to retrieve the right values 
      # from the call to 'fcctx' and 'fccrx'
      Ltr band 1

      set StateUI(1,band,def)        "900"
      set StateUI(1,freq_tx,def)     [format  "%.6f"  [Ltr  fcctx  248]]
      set StateUI(1,freq_rx,def)     [format  "%.6f"  [Ltr  fccrx  248]]
      set StateUI(1,fcc,def)         248
      set StateUI(1,border_bit,def)  0
  

  set StateUI(band_menu)  { "800"  "900"  "USER" }
  


  #***************************************************************************
  #
  #        Name: get_tile
  #
  #  Parameters: a widget within the tile
  #
  # Description: get the name of the tile frame
  #
  #***************************************************************************
  proc get_tile { widget } \
  {
    while { 1 } \
    {
      switch -- [winfo class $widget] \
      {
        "LTR_CONFIG" { return $widget } 
      }

      set widget [winfo parent $widget]
    }
  }


  #***************************************************************************
  #
  #        Name: band_config
  #
  #  Parameters:
  #
  # Description: Displays the Band Configuration tile
  #
  #***************************************************************************
  proc band_config { tile_path } \
  {
    #puts "band_config:  $tile_path"
    
    variable  StateUI
    variable  last_focus 
    
    set last_focus     $tile_path
     
    set top_level  [winfo toplevel $tile_path]
   
    set tile_path  [name_child  $top_level  ltr_config]

    if { ![winfo exists $tile_path] } \
    {
      set tile_path  [Screens::create_toplevel_tile  $tile_path  LTR_CONFIG]

      # Load the canvas
      load_canvas  $tile_path
    }
    
    bind $tile_path <KeyRelease-F8> "destroy [winfo toplevel $tile_path]"

    bind ${tile_path}.btn_return <Destroy>  \
                           "LTR_Sim_ConfigUI::band_config_return  $tile_path"

    ${tile_path}.cnvs_config config  \
                              -yscrollcommand "${tile_path}.scrl_config  set"

    ${tile_path}.scrl_config config  -command "${tile_path}.cnvs_config  yview"
    
    # Initialize the CHANNEL setup data
    set_config_channel  $tile_path

    # Focus on the first BAND button
    after idle  "focus -force ${tile_path}.btn_return"

    return    
  }


  #***************************************************************************
  #
  #        Name: band_config_return
  #
  #  Parameters:
  #
  # Description: Destroys the Band Configuration tile
  #
  #***************************************************************************
  proc band_config_return { tile_path } \
  {
    #puts "band_config_return:  $tile_path"
    
    variable  StateUI
    variable  last_focus 

    variable  please_wait
    
    set please_wait  "Please Wait ..."
    
    update idle
    
    # Set the frequencies of the Simulator
    # to the values displayed on the configuration dialog
    set_sim_freq_list
    
    # Set the Ltr engine values
    Ltr  home  $::Ltr::State(tx_home) -nowait
    Ltr  free  $::Ltr::State(tx_free) -nowait
    Ltr  goto  $::Ltr::State(tx_goto) -nowait

    
    safe_focus_force $last_focus

    set please_wait  ""
    
    return    
  }


  #***************************************************************************
  #
  #        Name: load_canvas
  #
  #  Parameters: tile_path - tile path of the invoking widget
  #
  # Description: Loads the canvas with the labels to display the channel 
  #              configuration summary.
  #
  #***************************************************************************
  proc load_canvas { tile_path } \
  {
    #puts "load_canvas:  $tile_path"

    variable  max_chan_index
    variable  StateUI
    
    set tile_path  [get_tile  $tile_path]
    
    set canvas_path  ${tile_path}.cnvs_config
    
    set y_index  0
    
    for { set i  1 } { $i <= $max_chan_index } { incr i  1 } \
    {
      # Create the CHAN label
      label  ${canvas_path}.lbl_chan_${i} -text $i              \
                                          -font analyzer_small  \
                                          -anchor center
        
      # Place the CHAN label
      $canvas_path  create  window  0  $y_index                       \
                                -width   55                           \
                                -height  25                           \
                                -window  ${canvas_path}.lbl_chan_${i} \
                                -anchor  nw

      # Create the BAND label
      label  ${canvas_path}.lbl_band_${i}                                     \
                          -textvariable  ::LTR_Sim_ConfigUI::StateUI($i,band) \
                          -font analyzer_small                                \
                          -anchor center
        
      # Place the BAND label
      $canvas_path  create  window  55  $y_index                      \
                                -width   60                           \
                                -height  25                           \
                                -window  ${canvas_path}.lbl_band_${i} \
                                -anchor  nw
                                                     
      # Create Tx FREQ label
      label  ${canvas_path}.lbl_freq_tx_${i}                                  \
                       -textvariable  ::LTR_Sim_ConfigUI::StateUI($i,freq_tx) \
                       -font analyzer_small                                   \
                       -anchor center
            
      # Place the Tx FREQ label
      $canvas_path  create  window  115  $y_index                      \
                              -width   120                             \
                              -height  25                              \
                              -window  ${canvas_path}.lbl_freq_tx_${i} \
                              -anchor  nw

      # Create Rx FREQ label
      label  ${canvas_path}.lbl_freq_rx_${i}                                  \
                       -textvariable  ::LTR_Sim_ConfigUI::StateUI($i,freq_rx) \
                       -font analyzer_small                                   \
                       -anchor center

      # Place the FREQ label
      $canvas_path  create  window  235  $y_index                      \
                              -width   100                             \
                              -height  25                              \
                              -window  ${canvas_path}.lbl_freq_rx_${i} \
                              -anchor  nw

      # Create FCC CH# label
      label  ${canvas_path}.lbl_fcc_${i}                                      \
                           -textvariable  ::LTR_Sim_ConfigUI::StateUI($i,fcc) \
                           -font analyzer_small                               \
                           -anchor center
             
      # Place the FCC CH# label for '800' or '900' band
      if { $StateUI($i,band) == "800" || $StateUI($i,band) == "900" } \
      {
        $canvas_path  create  window  335  $y_index                   \
                              -width   70                             \
                              -height  25                             \
                              -window  ${canvas_path}.lbl_fcc_${i}    \
                              -anchor  nw                             \
                              -tags    lbl_fcc_${i}
      }
                                
      # Create a space for the FCC CH# label
      label  ${canvas_path}.lbl_space_1_${i}  -font analyzer_small  \
                                              -anchor center
             
      # Place the space for the FCC CH# label for the 'USER' band
      if { $StateUI($i,band) == "USER" } \
      {
        $canvas_path  create  window  335  $y_index                     \
                              -width   70                               \
                              -height  25                               \
                              -window  ${canvas_path}.lbl_space_1_${i}  \
                              -anchor  nw                               \
                              -tags    lbl_space_1_${i}
      }

      # Create BORDER BIT label
      label  ${canvas_path}.lbl_border_bit_${i}                               \
                     -textvariable ::LTR_Sim_ConfigUI::StateUI($i,border_bit) \
                     -font analyzer_small                                     \
                     -anchor center
             
      # Place the BORDER BIT label for '800'
      if { $StateUI($i,band) == "800" || $StateUI($i,band) == "900" } \
      {
        $canvas_path  create  window  405  $y_index                    \
                          -width   100                                 \
                          -height  25                                  \
                          -window  ${canvas_path}.lbl_border_bit_${i}  \
                          -anchor  nw                                  \
                          -tags    lbl_border_bit_${i}
      }
      
      # Create a space for the BORDER BIT label
      label  ${canvas_path}.lbl_space_2_${i}  -font analyzer_small  \
                                              -anchor center
             
      # Place the space for the BORDER BIT label for '900' or 'USER' band
      if { $StateUI($i,band) == "USER" } \
      {
        $canvas_path  create  window  405  $y_index                 \
                          -width   100                              \
                          -height  25                               \
                          -window  ${canvas_path}.lbl_space_2_${i}  \
                          -anchor  nw                               \
                          -tags    lbl_space_2_${i}
      }

      # Increment the Y index
      incr y_index  25
    }
  }


  #***************************************************************************
  #
  #        Name: set_config_channel
  #
  #  Parameters: tile_path - tile path of the invoking widget
  #
  # Description: Sets the channel to be configured and highlights the 
  #              labels in the canvas to show the user which values are 
  #              being editted.
  #
  #***************************************************************************
  proc set_config_channel { tile_path } \
  {
    #puts "set_config_channel:  $tile_path"

    variable  prev_chan
    variable  StateUI
    
    set tile_path  [get_tile  $tile_path]
    
    set canvas_path  ${tile_path}.cnvs_config
    
    # Return the previous channel canvas labels to a background of 'gray'
    ${canvas_path}.lbl_chan_${prev_chan}        config  -bg gray
    ${canvas_path}.lbl_band_${prev_chan}        config  -bg gray
    ${canvas_path}.lbl_freq_tx_${prev_chan}     config  -bg gray
    ${canvas_path}.lbl_freq_rx_${prev_chan}     config  -bg gray
    ${canvas_path}.lbl_fcc_${prev_chan}         config  -bg gray
    ${canvas_path}.lbl_space_1_${prev_chan}     config  -bg gray
    ${canvas_path}.lbl_border_bit_${prev_chan}  config  -bg gray
    ${canvas_path}.lbl_space_2_${prev_chan}     config  -bg gray

    set chan  $StateUI(chan)
    
    # Highlight the present channel canvas labels by changing thier 
    # background to 'gray70'
    ${canvas_path}.lbl_chan_${chan}        config  -bg gray70
    ${canvas_path}.lbl_band_${chan}        config  -bg gray70
    ${canvas_path}.lbl_freq_tx_${chan}     config  -bg gray70
    ${canvas_path}.lbl_freq_rx_${chan}     config  -bg gray70
    ${canvas_path}.lbl_fcc_${chan}         config  -bg gray70
    ${canvas_path}.lbl_space_1_${chan}     config  -bg gray70
    ${canvas_path}.lbl_border_bit_${chan}  config  -bg gray70
    ${canvas_path}.lbl_space_2_${chan}     config  -bg gray70

    # Adjust the canvas to display the present highlighted labels
    if { $chan <= 8 } \
    {
      $canvas_path  yview  moveto  0.0
    } \
    elseif { $chan <= 16 } \
    {
      $canvas_path  yview  moveto  0.4
    } \
    else \
    {
      $canvas_path  yview  moveto  1.0
    }

    # Configure the BAND menubutton
      if { ![info exists  StateUI($chan,band)] } \
      {
        set StateUI($chan,band)  $StateUI($chan,band,def)
      }
    
      ${tile_path}.mnu_band  config  -textvariable ::LTR_Sim_ConfigUI::StateUI($chan,band)

    # Set the BAND value
      set_band  $tile_path  $chan
    
    # Configure the BAND menu
      set max_index  [${tile_path}.mnu_band.menu  index  end]
    
      for { set i  0 } { $i <= $max_index } { incr i  1 } \
      {
        ${tile_path}.mnu_band.menu  entryconfig  $i  \
                -variable ::LTR_Sim_ConfigUI::StateUI($chan,band)  \
                -command  "LTR_Sim_ConfigUI::set_band  $tile_path  $chan"
      }

    # Configure the Tx FREQ entry
      if { ![info exists  StateUI($chan,freq_tx)] } \
      {
        set StateUI($chan,freq_tx)  $StateUI($chan,freq_tx,def)
      }
    
      ${tile_path}.ent_freq_tx  config  \
                     -textvariable ::LTR_Sim_ConfigUI::StateUI($chan,freq_tx)
                
      bind_numeric_spin  ${tile_path}.ent_freq_tx  4 6 2800.000000 0.000000  \
                         "LTR_Sim_ConfigUI::set_freq  $tile_path  TX  $chan"
                         
    # Configure the Rx FREQ entry
      if { ![info exists  StateUI($chan,freq_rx)] } \
      {
        set StateUI($chan,freq_rx)  $StateUI($chan,freq_rx,def)
      }
    
      ${tile_path}.ent_freq_rx  config  \
                     -textvariable ::LTR_Sim_ConfigUI::StateUI($chan,freq_rx)

      bind_numeric_spin  ${tile_path}.ent_freq_rx  4 6 2800.000000 0.000000  \
                         "LTR_Sim_ConfigUI::set_freq  $tile_path  RX  $chan"

    # Configure the FCC entry
      if { ![info exists  StateUI($chan,fcc)] } \
      {
        set StateUI($chan,fcc)  $StateUI($chan,fcc,def)
      }
    
      ${tile_path}.ent_fcc  config  \
                         -textvariable ::LTR_Sim_ConfigUI::StateUI($chan,fcc)

      bind_numeric_spin  ${tile_path}.ent_fcc  4 0 1023 1 \
                              "LTR_Sim_ConfigUI::set_fcc  $tile_path  $chan"

    # Configure the BORDER BIT entry
      if { ![info exists  StateUI($chan,border_bit)] } \
      {
        set StateUI($chan,border_bit)  $StateUI($chan,border_bit,def)
      }
    
      ${tile_path}.ent_border_bit  config  \
                  -textvariable ::LTR_Sim_ConfigUI::StateUI($chan,border_bit)

      bind_numeric_spin  ${tile_path}.ent_border_bit  1 0 1 0 \
                       "LTR_Sim_ConfigUI::set_border_bit  $tile_path  $chan"

    # Set the 'prev_chan' value to the present channel
    set prev_chan  $chan
  }


  #***************************************************************************
  #
  #        Name: set_band
  #
  #  Parameters: tile_path - tile path of the invoking widget
  #              index     - index number of the fields
  #
  # Description: Set the BAND and other fields accordingly
  #
  #***************************************************************************
  proc set_band { tile_path  index } \
  {
    #puts "set_band:  $tile_path,  $index"

    variable  StateUI
    
    set tile_path  [get_tile  $tile_path]
    
    set canvas_path  ${tile_path}.cnvs_config
    
    # Remove the FCC CH# field
    grid remove ${tile_path}.lbl_fcc
    grid remove ${tile_path}.ent_fcc

    # Remove the BORDER BIT field
    grid remove ${tile_path}.lbl_border_bit
    grid remove ${tile_path}.ent_border_bit

    # Remove the FCC CH# label from the canvas
    $canvas_path  delete  lbl_fcc_${index}

    # Remove the space for the FCC CH# label from the canvas
    $canvas_path  delete  lbl_space_1_${index}

    # Remove the space for the BORDER BIT label from the canvas
    $canvas_path  delete  lbl_space_2_${index}

    switch -- $StateUI($index,band) \
    {
      "800"    { Ltr band  0
                 
                 # Display the FCC CH# field
                 Screens::position_widget  ${tile_path}.lbl_fcc
                 Screens::position_widget  ${tile_path}.ent_fcc

                 Screens::position_widget  ${tile_path}.lbl_border_bit
                 Screens::position_widget  ${tile_path}.ent_border_bit

                 set y_index  [expr ($index - 1) * 25]

                 # Display the FCC CH# label on the canvas
                 $canvas_path  create  window  335  $y_index              \
                              -width   70                                 \
                              -height  25                                 \
                              -window  ${canvas_path}.lbl_fcc_${index}    \
                              -anchor  nw                                 \
                              -tags    lbl_fcc_${index}

                 # Display the BORDER BIT label on the canvas
                 $canvas_path  create  window  405  $y_index                  \
                              -width   100                                    \
                              -height  25                                     \
                              -window  ${canvas_path}.lbl_border_bit_${index} \
                              -anchor  nw                                     \
                              -tags    lbl_border_bit_${index}

                 # Disable the Rx entry fields
                 # Enable the BORDER BIT entry fields
                 ${tile_path}.ent_freq_rx  config  -state disabled  \
                                                   -takefocus  0
                                                       
                 # Rebuild the cursor focus map
                 Cursor::rebuildControl  $tile_path  MAP

                 set_fcc         $tile_path  $index
                 set_border_bit  $tile_path  $index
               }
               
      "900"    { Ltr band  1
      
                 # Display the FCC CH# field
                 Screens::position_widget  ${tile_path}.lbl_fcc
                 Screens::position_widget  ${tile_path}.ent_fcc

                 Screens::position_widget  ${tile_path}.lbl_border_bit
                 Screens::position_widget  ${tile_path}.ent_border_bit

                 set y_index  [expr ($index - 1) * 25]

                 # Display the FCC CH# label on the canvas
                 $canvas_path  create  window  335  $y_index              \
                              -width   70                                 \
                              -height  25                                 \
                              -window  ${canvas_path}.lbl_fcc_${index}    \
                              -anchor  nw                                 \
                              -tags    lbl_fcc_${index}

                 # Display the BORDER BIT label on the canvas
                 $canvas_path  create  window  405  $y_index                  \
                              -width   100                                    \
                              -height  25                                     \
                              -window  ${canvas_path}.lbl_border_bit_${index} \
                              -anchor  nw                                     \
                              -tags    lbl_border_bit_${index}

                 # Disable the Rx entry fields
                 # Enable the BORDER BIT entry fields
                 ${tile_path}.ent_freq_rx  config  -state disabled  \
                                                   -takefocus  0
                                           
                 # Rebuild the cursor focus map
                 Cursor::rebuildControl  $tile_path  MAP

                 set_fcc         $tile_path  $index
                 set_border_bit  $tile_path  $index
               }
                 
      "USER"   -
      default  { Ltr band  2
      
                 set y_index  [expr ($index - 1) * 25]
                 
                 # Display the space for the FCC CH# label on the canvas
                 $canvas_path  create  window  335  $y_index                \
                              -width   70                                   \
                              -height  25                                   \
                              -window  ${canvas_path}.lbl_space_1_${index}  \
                              -anchor  nw                                   \
                              -tags    lbl_space_1_${index}

                 # Display the space for the BORDER BIT label on the canvas
                 $canvas_path  create  window  405  $y_index                \
                              -width   100                                  \
                              -height  25                                   \
                              -window  ${canvas_path}.lbl_space_2_${index}  \
                              -anchor  nw                                   \
                              -tags    lbl_space_2_${index}


                 # Enable the Rx entry fields
                 # Disable the BORDER BIT entry fields
                 ${tile_path}.ent_freq_rx  config  -state normal  \
                                                   -takefocus  1
                                           
                 # Rebuild the cursor focus map
                 Cursor::rebuildControl  $tile_path  MAP

                 set_freq  $tile_path  TX  $index
                 set_freq  $tile_path  RX  $index
               }
    }
  }


  #***************************************************************************
  #
  #        Name: set_freq
  #
  #  Parameters: tile_path - tile path of the invoking widget
  #              tx_rx     - TX or RX indicator
  #              index     - index number of the fields
  #
  # Description: Set the Frequency value
  #
  #***************************************************************************
  proc set_freq { tile_path  tx_rx  index } \
  {
    puts "set_freq:  $tile_path,  $tx_rx,  $index"

    variable  StateUI
    
    set tile_path  [get_tile  $tile_path]
    
    # If the Frequency is being set, then clear the FCC Channel # and 
    # Border Bit
    set StateUI($index,fcc)         ""
    set StateUI($index,border_bit)  ""

    if { $tx_rx == "TX" } \
    {
      switch -- $StateUI($index,band) \
      {
        "800"   { # Set the Tx frequency
                  Ltr  txfreq  $index  $StateUI($index,freq_tx)
                  
                  # Calculate the Rx frequency
                  set StateUI($index,freq_rx)  \
                    [format "%.6f" [expr $StateUI($index,freq_tx) - 45.00000]]
                  
                  # Set the Rx frequency
                  Ltr  rxfreq  $index  $StateUI($index,freq_rx)
                  
                  # Get the nearest FCC # to the user entered frequency
                  set StateUI($index,fcc)  \
                                [Ltr  freqtochan  0  $StateUI($index,freq_tx)]
                   
                  # Set frequency to this FCC #
                  set_fcc  $tile_path  $index
                }
                
        "900"   { # Set the Tx frequency
                  Ltr  txfreq  $index  $StateUI($index,freq_tx)
                  
                  # Calculate the Rx frequency
                  set StateUI($index,freq_rx)  \
                   [format "%.6f" [expr $StateUI($index,freq_tx) - 39.00000]]
                  
                  # Set the Rx frequency
                  Ltr  rxfreq  $index  $StateUI($index,freq_rx)
                  
                  # Get the nearest FCC # to the user entered frequency
                  set StateUI($index,fcc)  \
                               [Ltr  freqtochan  1  $StateUI($index,freq_tx)]
                  
                  # Set frequency to this FCC #
                  set_fcc  $tile_path  $index
                }
      
        "USER"  { # Set the Tx frequency
                  Ltr  txfreq  $index  $StateUI($index,freq_tx)
                }
      }
    }
    
    if { $tx_rx == "RX" } \
    {
      switch -- $StateUI($index,band) \
      {
        "800"   { # Cannot set Rx in '800' band }

        "900"   { # Cannot set Rx in '900' band }

        "USER"  { # Set the Rx frequency
                  Ltr  rxfreq  $index  $StateUI($index,freq_rx)
                }
      }
    }
  }


  #***************************************************************************
  #
  #        Name: set_fcc
  #
  #  Parameters: tile_path - tile path of the invoking widget
  #              index     - index number of the fields
  #
  # Description: Set the FCC Channel value
  #
  #***************************************************************************
  proc set_fcc { tile_path  index } \
  {
    puts "set_fcc:  $tile_path  $index"

    variable  StateUI
    
    set tile_path  [get_tile  $tile_path]
    
    if { $StateUI($index,fcc) == "" } \
    {
      return
    }
                
    switch -- $StateUI($index,band) \
    {
      "800"   -
      "900"   { # Determine the Tx Frequency from the FCC Channel #
                set StateUI($index,freq_tx)  \
                           [format "%.6f" [Ltr  fcctx  $StateUI($index,fcc)]]
                  
                # Set the Tx frequency
                Ltr  txfreq  $index  $StateUI($index,freq_tx)
                  
                # Determine the Rx Frequency from the FCC Channel #
                set StateUI($index,freq_rx)  \
                           [format "%.6f" [Ltr  fccrx  $StateUI($index,fcc)]]
                  
                # Set the Rx frequency
                Ltr  rxfreq  $index  $StateUI($index,freq_rx)
              }

      "USER"  { # Not valid }
    }
  }


  #***************************************************************************
  #
  #        Name: set_border_bit
  #
  #  Parameters: tile_path - tile path of the invoking widget
  #              index     - index number of the fields
  #
  # Description: Set the Border Bit value
  #
  #***************************************************************************
  proc set_border_bit { tile_path  index } \
  {
    #puts "set_border_bit:  $tile_path,  $index"

    variable  StateUI
    
    set tile_path  [get_tile  $tile_path]
    
    if { $StateUI($index,border_bit) == "" } \
    {
      return
    }
    
    switch -- $StateUI($index,band) \
    {
      "800"   -
      "900"   { # Set the Border Bit
                Ltr  border  $StateUI($index,border_bit)
                
                # Call to redisplay the Tx Frequency
                set_fcc  $tile_path  $index
              }

      "USER"  { # Not valid }
    }
  }


  #***************************************************************************
  #
  #        Name: restore_defaults
  #
  #  Parameters:
  #
  # Description: Displays the Band Configuration tile
  #
  #***************************************************************************
  proc restore_defaults { } \
  {
    #puts "restore_defaults"
    
    variable  max_chan_index
    variable  StateUI

    for { set i  1 } { $i <= $max_chan_index } { incr i  1 } \
    {
      set StateUI($i,band)     $StateUI($i,band,def)
      set StateUI($i,freq_tx)  $StateUI($i,freq_tx,def)
      set StateUI($i,freq_rx)  $StateUI($i,freq_rx,def)
      set StateUI($i,fcc)      $StateUI($i,fcc,def)
      
      set StateUI($i,border_bit)  $StateUI($i,border_bit,def)
    }
  }


  #***************************************************************************
  #
  #        Name: set_sim_freq_list
  #
  #  Parameters:
  #
  # Description: Sets the Tx and Rx frequencies of the Simulator
  #
  #***************************************************************************
  proc set_sim_freq_list { } \
  {
    #puts "set_sim_freq_list"
    
    variable  max_chan_index
    variable  StateUI

    for { set i  1 } { $i <= $max_chan_index } { incr i  1 } \
    {
      # Call to set the frequencies directly
      Ltr  txfreq  $i  $StateUI($i,freq_tx)
      Ltr  rxfreq  $i  $StateUI($i,freq_rx)
    }
  }


  #***************************************************************************
  #
  #        Name: save_config
  #
  #  Parameters: tile_path - tile path of the invoking widget
  #
  # Description: Saves the present configuration
  #
  #***************************************************************************
  proc save_config { tile_path } \
  {
    #puts "save_config:  $tile_path"
    
    variable  config_dir
    variable  file_label
    
    set data_buffer  [getSaveState]
    
    #puts "data_buffer:  $data_buffer"

    # Recall the configuration data
    set save_flag  [File_SaveUI::display_file_save  $tile_path     \
                                                    $config_dir    \
                                                    $data_buffer   \
                                                    "LTR_CONFIG"   \
                                                    "$file_label"  \
                                                    ".DAT"         \
                                                    NO_ENCRYPT     \
                                                    NO_CSV         \
                                                    NO_MODES       \
                                                    NO_OPT_EXT]
    
    # Get the file label after the file has been saved
    set file_label  $::File_SaveUI::file_label
  }


  #***************************************************************************
  #
  #        Name: recall_config
  #
  #  Parameters: tile_path - tile path of the invoking widget
  #
  # Description: Recalls a configuration
  #
  #***************************************************************************
  proc recall_config { tile_path } \
  {
    #puts "recall_config:  $tile_path"
    
    variable  config_dir

    # Recall the configuration data
    set data_buffer  [File_RecallUI::display_file_recall  $tile_path    \
                                                          $config_dir   \
                                                          "LTR_CONFIG"  \
                                                          ".DAT"        \
                                                          NO_ENCRYPT]
    #puts "data_buffer:  $data_buffer"
    
    # Source the configuration data that was recalled
    eval  "$data_buffer"
  }


  #***************************************************************************
  #
  #        Name: toggle_scroll_on
  #
  #  Parameters: tile_path - tile path of the invoking widget
  #
  # Description: Toggles the 'scroll_on' variable to allow the spinner
  #              knob to control the scrollbar
  #
  #***************************************************************************
  proc toggle_scroll_on { tile_path } \
  {
    #puts "toggle_scroll_on:  $tile_path"
    
    variable  scroll_on
    
    set tile_path  [get_tile  $tile_path]
    
    set scrollbar_path  ${tile_path}.scrl_config
    
    if { $scroll_on == 0 } \
    {
      # Change the color of the scrollbar to indicate it is active
      $scrollbar_path config  -bg white
      
      set scroll_on  1
      
      # For each event, call the procedure to move the scrollbar,
      # then break to keep the focus from moving in response to the 
      # event
      # Bind the <Right> event  (includes <<Spin_up>>)
      bind $scrollbar_path  <Right>  "LTR_Sim_ConfigUI::move_canvas  $tile_path  UP; break"
                                   
      # Bind the <Left> event  (includes <<Spin_down>>)
      bind $scrollbar_path  <Left>  "LTR_Sim_ConfigUI::move_canvas  $tile_path  DOWN; break"
                                   
      bind $scrollbar_path  <Up>  "LTR_Sim_ConfigUI::move_canvas  $tile_path  UP; break"
                                   
      bind $scrollbar_path  <Down>  "LTR_Sim_ConfigUI::move_canvas  $tile_path  DOWN; break"
    } \
    else \
    {
      # Change the color of the scrollbar to indicate it is NOT active
      $scrollbar_path config  -bg gray

      set scroll_on  0

      # Unbind the <Right> and <Left> events
      bind $scrollbar_path  <Right>  ""
      bind $scrollbar_path  <Left>   ""
      
      # For the 'Up' of 'Down' event, generate the 'cursor_up' or 'cursor_down'
      # event, then break to keep the scrollbar from moving in response to 
      # either event
      bind $scrollbar_path  <Up>    "event generate %W <<cursor_up>>; break"
      bind $scrollbar_path  <Down>  "event generate %W <<cursor_down>>; break"
    }
  }
      

  #***************************************************************************
  #
  #        Name: move_canvas
  #
  #  Parameters: tile_path - tile path of the invoking widget
  #              direction - direction to move the scrollbar
  #
  # Description: Moves the canvas display for changes on the spinner knob 
  #              or Up and Down arrows.
  #
  #***************************************************************************
  proc move_canvas { tile_path  direction } \
  {
    #puts "move_canvas:  $tile_path,  $direction"
    
    variable  scroll_on
    
    if { $scroll_on == 0 } \
    {
      return
    }

    set tile_path  [get_tile  $tile_path]
    
    set canvas_path  ${tile_path}.cnvs_config

    # Move the canvas display
    switch -- $direction \
    {
      "UP"    { $canvas_path  yview  scroll  -1  unit }
    
      "DOWN"  { $canvas_path  yview  scroll  1  unit }
    }
  }


  #****************************************************************************
  #
  #        Name: getSaveState
  #
  #  Parameters: none
  #
  # Description: This function returns a list of TCL commands which can be 
  #              used to set the LTR_Sim_ConfigUI tiles to its current state
  #
  #****************************************************************************
  proc getSaveState { } \
  {
    variable  max_chan_index
    variable  StateUI

    set returnList { }

    for { set i  1 } { $i <= $max_chan_index } { incr i  1 } \
    {
      append returnList  "set ::LTR_Sim_ConfigUI::StateUI($i,band)  \
                                                        {$StateUI($i,band)}\n"

      append returnList  "set ::LTR_Sim_ConfigUI::StateUI($i,freq_tx)  \
                                                     {$StateUI($i,freq_tx)}\n"

      append returnList  "set ::LTR_Sim_ConfigUI::StateUI($i,freq_rx)  \
                                                     {$StateUI($i,freq_rx)}\n"

      append returnList  "set ::LTR_Sim_ConfigUI::StateUI($i,fcc)  \
                                                         {$StateUI($i,fcc)}\n"

      append returnList  "set ::LTR_Sim_ConfigUI::StateUI($i,border_bit)  \
                                                  {$StateUI($i,border_bit)}\n"
    }

    #puts "$returnList"
    
    return [subst -nocommands $returnList]
  }


  #****************************************************************************
  #
  #        Name: Sync
  #
  #  Parameters: none
  #
  # Description: Write state variables to LTR_Sim_ConfigUI
  #
  #****************************************************************************
  proc Sync { } \
  {
    eval [getSaveState]
  }


};  # End of LTR_Sim_ConfigUI namespace


#****************************************************************************
# This registers the tiles with the list for saving instrument states
#****************************************************************************
namespace eval ::SavedStates \
{
  variable tiles
  
  lappend tiles  LTR_Sim_ConfigUI
}

