#############################################################################
#                                                                           #
# IFR, An Aeroflex Co.       COPYRIGHT (C) 2003        DESIGN ENGINEERING   #
# ------------------------------------------------------------------------- #
#                                                                           #
# FILE        : 4105.LTR_MonitorUI.tcl                                      #
#                                                                           #
# PRODUCT     : FITS                                                        #
#                                                                           #
# AUTHOR      : P. McCormick                                                #
#                                                                           #
# DESCRIPTION : LTR Monitor User Interface                                  #
#                                                                           #
#############################################################################

package require Ltr
package require IFR_options

namespace eval LTR_MonitorUI \
{

  # Global variables
  variable  LongName     "ltr_monitor"
  variable  ShortName    "LTR Monitor"
  variable  Size         Quarter
            
  variable  tile_path    ""
  variable  tile_class   "LTR_MONITOR"

  variable  scroll_on  0

  variable  StateUI
  
  set StateUI(capture)   "CAPTURE OFF"
  set StateUI(data_log)  "LOG DATA OFF"
  
  set StateUI(monitor_mode_menu)  { "OFF"  "REPEATER"  "RADIO" }

  set StateUI(timestamp_ref)  -1

  set StateUI(word,0)       000000
  set StateUI(sync,0)       000
  set StateUI(checksum,0)   00
  set StateUI(timestamp,0)  0
  set StateUI(inverted,0)   0



  #***************************************************************************
  #
  #        Name: Create  
  #
  #  Parameters: 
  #
  # Description: 
  #
  #***************************************************************************
  proc Create { tile_path } \
  {
    #puts "LTR_MonitorUI::Create"
    
    variable  LongName
    variable  tile_class

    set top_level [winfo toplevel $tile_path]
   
    set tile_path  [name_child  $top_level  $LongName]

    if { ![winfo exists $tile_path] } \
    {
      set tile_path  [Screens::create_tile  $tile_path  $LongName  $tile_class]
    }  
     
    return $tile_path
  }


  #***************************************************************************
  #
  #        Name: Map
  #
  #  Parameters: 
  #
  # Description: 
  #
  #***************************************************************************
  proc Map { tile_path } \
  {
    #puts "LTR_MonitorUI::Map:  $tile_path"

    if { ![IFR_options is_installed LTR] } \
    {
      grid forget $tile_path

      return
    }
    

    trace variable ::Ltr::State(timestamp)  w  \
                                     "LTR_MonitorUI::Update_array  $tile_path"

    trace variable ::Ltr::State(monitor_mode_text)  w  \
                              "LTR_MonitorUI::update_monitor_mode  $tile_path"
                                     
    update_monitor_mode_core  $tile_path


    ${tile_path}.frame4.lst_read config  \
                           -yscrollcommand "${tile_path}.frame4.scrl_read set"

    ${tile_path}.frame4.scrl_read config  \
                                 -command "${tile_path}.frame4.lst_read yview"
    
    # Set the current channel of the Monitor
    set_chan
  }


  #***************************************************************************
  #
  #        Name: Cleanup
  #
  #  Parameters: 
  #
  # Description: 
  #
  #***************************************************************************
  proc Cleanup { tile_path } \
  {
    #puts "LTR_MonitorUI::Cleanup:  $tile_path"
    

    trace vdelete ::Ltr::State(timestamp)  w  \
                                     "LTR_MonitorUI::Update_array  $tile_path"

    trace vdelete ::Ltr::State(monitor_mode_text)  w  \
                              "LTR_MonitorUI::update_monitor_mode  $tile_path"

    set StateUI(capture)  "CAPTURE OFF"
  }
  

  #***************************************************************************
  #
  #        Name: Unmap
  #
  #  Parameters: 
  #
  # Description: 
  #
  #***************************************************************************
  proc Unmap { tile_path } \
  {
    #puts "LTR_MonitorUI::Unmap"
    
    Cleanup  $tile_path
  }


  #***************************************************************************
  #
  #        Name: Destroy
  #
  #  Parameters: 
  #
  # Description: 
  #
  #***************************************************************************
  proc Destroy { tile_path } \
  {
    #puts "LTR_MonitorUI::Destroy"
    
    Cleanup  $tile_path
  }


  #***************************************************************************
  #
  #        Name: Register
  #
  #  Parameters: 
  #
  # Description: 
  #
  #***************************************************************************
  proc Register { } \
  {
    #puts "LTR_MonitorUI::Register"
  }
  

  #***************************************************************************
  #
  #        Name: Update_array
  #
  #  Parameters: 
  #
  # Description: 
  #
  #***************************************************************************
  proc Update_array { tile_path  args } \
  {
    #puts "LTR_MonitorUI::Update_array"

    variable  StateUI
    
    # Return if the Monitor is OFF
    if { $::Ltr::State(monitor_mode_text) == "OFF" } \
    {
      return
    }
    
    set tile_path  [get_tile  $tile_path]

    if { $StateUI(capture) == "CAPTURE ON" } \
    {
      # Set the timestamp reference
      if { $StateUI(timestamp_ref) == -1 } \
      {
        set StateUI(timestamp_ref)  $::Ltr::State(timestamp)
      }
      
      set index  [${tile_path}.frame4.lst_read  index  end]
    
      #puts "index:  $index"

      set StateUI(sync,$index)       $::Ltr::State(sync)
      set StateUI(word,$index)       $::Ltr::State(word)
      set StateUI(checksum,$index)   $::Ltr::State(checksum)
      
      # Mask large timestamp values to eliminate negative values
      if { $::Ltr::State(timestamp) != "" } \
      {
        set StateUI(timestamp,$index)  [expr $::Ltr::State(timestamp) & 0x7FFFFFFF]
      }
      
      set StateUI(inverted,$index)   $::Ltr::State(inverted)

      set display  "$index:  "
      
      append display  "$::Ltr::State(sync)   "
      append display  "$::Ltr::State(word)   "
      append display  "$::Ltr::State(checksum)   "
      append display  "$::Ltr::State(inverted)"
      
      ${tile_path}.frame4.lst_read  insert  end  "$display"
    
      ${tile_path}.frame4.lst_read  see  end
    }

    if { $StateUI(data_log) == "LOG DATA ON" } \
    {
      # Log the data to a file
    }
  }


  #***************************************************************************
  #
  #        Name: get_tile
  #
  #  Parameters: any child widget of the tile
  #
  # Description: Get the name of the tile's parent frame
  #
  #***************************************************************************
  proc get_tile { widget } \
  {
    while { 1 } \
    {
      switch -- [winfo class $widget] \
      {
        "LTR_MONITOR"   { return $widget }
      }
        
      set widget [winfo parent $widget]
    }
  }


  #***************************************************************************
  #
  #        Name: set_monitor_mode
  #
  #  Parameters:
  #
  # Description: Sets Monitor mode
  #
  #***************************************************************************
  proc set_monitor_mode { } \
  {
    #puts "set_monitor_mode"
    
    variable  StateUI
    
    # Turn off the CAPTURE
    set StateUI(capture)  "CAPTURE OFF"

    # Clear the display fields
    set ::Ltr::State(word)       ""
    set ::Ltr::State(sync)       ""
    set ::Ltr::State(checksum)   ""
    set ::Ltr::State(timestamp)  ""
    set ::Ltr::State(inverted)   ""
    
    # Set the Monitor mode
    Ltr::monitorTurnOn
  }


  #***************************************************************************
  #
  #        Name: update_monitor_mode
  #
  #  Parameters: tile_path - tile path of the invoking widget
  #
  # Description: Updates the display to changes in the Monitor mode
  #
  #***************************************************************************
  proc update_monitor_mode { tile_path  args } \
  {
     set cmd [namespace code "update_monitor_mode_core $tile_path"]
     after cancel $cmd
     after idle $cmd
  }
  proc update_monitor_mode_core { tile_path } \
  {

    #puts "update_monitor_mode:  $tile_path"
    
    variable  StateUI
    
    set tile_path  [get_tile  $tile_path]
    
    switch -- $::Ltr::State(monitor_mode_text) \
    {
      "OFF"       { ${tile_path}.frame5.btn_capture  config  -state disabled }

      "REPEATER"  -
      "RADIO"     { ${tile_path}.frame5.btn_capture  config  -state normal   }
    }
  }


  #***************************************************************************
  #
  #        Name: set_chan
  #
  #  Parameters:
  #
  # Description: Sets Monitor channel
  #
  #***************************************************************************
  proc set_chan { } \
  {
    #puts "set_chan"
    
    after idle "Ltr  goto  $::Ltr::State(tx_goto) -nowait"
  }


  #***************************************************************************
  #
  #        Name: set_selection
  #
  #  Parameters: tile_path - tile path of the invoking widget
  #
  # Description: Sets selection in the list box
  #
  #***************************************************************************
  proc set_selection { tile_path } \
  {
    #puts "set_selection"
    
    variable  StateUI

    set tile_path  [get_tile  $tile_path]

    set index  [${tile_path}.frame4.lst_read  curselection]
    
    if { $index == "" } \
    {
      return
    }

    #puts "index:  $index"
    
    set ::Ltr::State(word)       $StateUI(word,$index)
    set ::Ltr::State(sync)       $StateUI(sync,$index)
    set ::Ltr::State(checksum)   $StateUI(checksum,$index)
    set ::Ltr::State(timestamp)  $StateUI(timestamp,$index)
    set ::Ltr::State(inverted)   $StateUI(inverted,$index)
  }


  #***************************************************************************
  #
  #        Name: toggle_capture
  #
  #  Parameters:
  #
  # Description: Sets the capture parameters
  #
  #***************************************************************************
  proc toggle_capture { } \
  {
    #puts "toggle_capture:"
    
    variable  StateUI

    if { $StateUI(capture) == "CAPTURE OFF" } \
    {
      set StateUI(capture)  "CAPTURE ON"
    } \
    else \
    {
      set StateUI(capture)  "CAPTURE OFF"
    }    
  }
    

  #***************************************************************************
  #
  #        Name: toggle_data_log
  #
  #  Parameters: new_value -
  #
  # Description: sets the data logging
  #
  #***************************************************************************
  proc toggle_data_log { tile_path { new_value "" } } \
  {
    #puts "toggle_data_log"

    variable  StateUI
    
    if { $StateUI(data_log) == "LOG DATA OFF" } \
    {
      set StateUI(data_log)  "LOG DATA ON"
      
      # Open the file for data logging
    } \
    else \
    {
      set StateUI(data_log)  "LOG DATA OFF"

      # Close the file for data logging
    }
  }


  #***************************************************************************
  #
  #        Name: clear_textbox
  #
  #  Parameters: tile_path - tile path of the invoking widget
  #
  # Description: Clears the text box
  #
  #***************************************************************************
  proc clear_textbox { tile_path } \
  {
    #puts "clear_textbox"
    
    variable  StateUI
    
    set tile_path  [get_tile  $tile_path]
    
    # Clear the textbox
    ${tile_path}.frame4.lst_read  delete  0  end
    
    # Clear the timestamp reference
    set StateUI(timestamp_ref)  -1
  }


  #***************************************************************************
  #
  #        Name: toggle_scroll_on
  #
  #  Parameters: tile_path - tile path of the invoking widget
  #
  # Description: Toggles the 'scroll_on' variable to allow the spinner
  #              knob to control the scrollbar
  #
  #***************************************************************************
  proc toggle_scroll_on { tile_path } \
  {
    #puts "toggle_scroll_on:  $tile_path"
    
    variable  scroll_on
    
    set tile_path  [get_tile  $tile_path]
    
    set scrollbar_path  ${tile_path}.frame4.scrl_read
    
    if { $scroll_on == 0 } \
    {
      # Change the color of the scrollbar to indicate it is active
      $scrollbar_path config  -bg white
      
      set scroll_on  1
      
      # For each event, call the procedure to move the scrollbar,
      # then break to keep the focus from moving in response to the 
      # event
      # Bind the <Right> event  (includes <<Spin_up>>)
      bind $scrollbar_path  <Right>  "::LTR_MonitorUI::move_listbox  $tile_path  UP; break"
                                   
      # Bind the <Left> event  (includes <<Spin_down>>)
      bind $scrollbar_path  <Left>  "::LTR_MonitorUI::move_listbox  $tile_path  DOWN; break"
                                   
      bind $scrollbar_path  <Up>  "::LTR_MonitorUI::move_listbox  $tile_path  UP; break"
                                   
      bind $scrollbar_path  <Down>  "::LTR_MonitorUI::move_listbox  $tile_path  DOWN; break"
    } \
    else \
    {
      # Change the color of the scrollbar to indicate it is NOT active
      $scrollbar_path config  -bg gray

      set scroll_on  0

      # Unbind the <Right> and <Left> events
      bind $scrollbar_path  <Right>  ""
      bind $scrollbar_path  <Left>   ""
      
      # For the 'Up' of 'Down' event, generate the 'cursor_up' or 'cursor_down'
      # event, then break to keep the scrollbar from moving in response to 
      # either event
      bind $scrollbar_path  <Up>    "event generate %W <<cursor_up>>; break"
      bind $scrollbar_path  <Down>  "event generate %W <<cursor_down>>; break"
    }
  }
      

  #***************************************************************************
  #
  #        Name: move_listbox
  #
  #  Parameters: tile_path - tile path of the invoking widget
  #              direction - direction to move the scrollbar
  #
  # Description: Moves the listbox display for changes on the spinner knob 
  #              or Up and Down arrows.
  #
  #***************************************************************************
  proc move_listbox { tile_path  direction } \
  {
    #puts "move_listbox:  $tile_path,  $direction"
    
    variable  scroll_on
    
    if { $scroll_on == 0 } \
    {
      return
    }

    set tile_path  [get_tile  $tile_path]
    
    set listbox_path  ${tile_path}.frame4.lst_read

    # Move the canvas display
    switch -- $direction \
    {
      "UP"    { $listbox_path  yview  scroll  -1  unit }
    
      "DOWN"  { $listbox_path  yview  scroll  1  unit }
    }
  }


};  # End of LTR_MonitorUI namespace


