#############################################################################
#                                                                           #
# IFR, An Aeroflex Co.     COPYRIGHT (C)1998-2003       DESIGN ENGINEERING  #
#                                                                           #
# --------------------------------------------------------------------------#
#                                                                           #
# FILE        : 4110.Audio_AnalyzerUI.tcl                                   #
#                                                                           #
# PRODUCT     : FITS                                                        #
#                                                                           #
# AUTHOR      : Preston McCormick                                           #
#                                                                           #
# DESCRIPTION : Audio Spectrum Analyzer User Interface                      #
#                                                                           #
#############################################################################

package require Sinad

namespace eval Audio_Analyzer_FullUI \
{
  # Global variables
  variable LongName       "full_audio_anlz"
  variable ShortName      "Audio Analyzer"
  variable Size           Full

  variable tile_class     FULL_AUDIO_ANLZ

  variable State
  set State(ul_from) LIVE
  set State(ll_from) LIVE
  set State(ul_offset) 0.0
  set State(ll_offset) 0.0
  set State(limits_from) {LIVE AVG REF PEAK+ PEAK- LIMIT+ LIMIT- MATH}
  
  #**************************************************************************
  #
  #         Name: Audio_Analyzer_FullUI::Create
  #
  #   Parameters:
  #
  #  Description: 
  #
  #**************************************************************************
  proc Create { tile_path } \
  {
    #puts "Audio_Analyzer_FullUI::Create"

    variable LongName
    variable tile_class

    set top_level  [View::Find_Main_toplevel $tile_path]
   
    set tile_path  [name_child  $top_level  $LongName]
  
    if { ![winfo exists $tile_path] } \
    {
      set tile_path  [Screens::create_tile  $tile_path  $LongName  $tile_class]

      # Configure the Audio Analyzer traces
      ${tile_path}.anlz trace create  tlive  -sys        [Sinad -system]       \
                                             -subsys     [Sinad -subsys tlive] \
                                             -cfgsubsys  [Sinad -subsys tcfg]  \
                                             -color      white                 \
                                             -active 1

      ${tile_path}.anlz trace create  tref   -sys        [Sinad -system]       \
                                             -subsys     [Sinad -subsys tref]  \
                                             -cfgsubsys  [Sinad -subsys tcfg]  \
                                             -color      yellow                \
                                             -active 0

      ${tile_path}.anlz trace create  tulim  -sys        [Sinad -system]       \
                                             -subsys     [Sinad -subsys tulim] \
                                             -cfgsubsys  [Sinad -subsys tcfg]  \
                                             -color      green                 \
                                             -active 0

      ${tile_path}.anlz trace create  tllim  -sys        [Sinad -system]       \
                                             -subsys     [Sinad -subsys tllim] \
                                             -cfgsubsys  [Sinad -subsys tcfg]  \
                                             -color      cyan                  \
                                             -active 0

      ${tile_path}.anlz trace create  tavg   -sys        [Sinad -system]       \
                                             -subsys     [Sinad -subsys tavg]  \
                                             -cfgsubsys  [Sinad -subsys tcfg]  \
                                             -color      orange                \
                                             -active 0
      ${tile_path}.anlz trace create  pkhi   -sys        [Sinad -system]       \
                                             -subsys     [Sinad -subsys tpkh]  \
                                             -cfgsubsys  [Sinad -subsys tcfg]  \
                                             -color      grey                \
                                             -active 0

      ${tile_path}.anlz trace create  pklo   -sys        [Sinad -system]       \
                                             -subsys     [Sinad -subsys tpkl]  \
                                             -cfgsubsys  [Sinad -subsys tcfg]  \
                                             -color      grey                \
                                             -active 0
      ${tile_path}.anlz trace create  math   -sys        [Sinad -system]       \
                                             -subsys     [Sinad -subsys tmath]  \
                                             -cfgsubsys  [Sinad -subsys tmcfg]  \
                                             -color      white                \
                                             -active 0

    }  

    return $tile_path
  }


  #**************************************************************************
  #
  #        Name: Audio_Analyzer_FullUI::Map
  #
  #  Parameters: tile_path - tile path of the invoking widget
  #
  # Description: This function is called when the full screen audio analyzer 
  #              needs to be mapped on the screen.
  #
  #**************************************************************************
  proc Map { tile_path } \
  {
    #puts "Audio_Analyzer_FullUI::Map:  $tile_path"
    
    if { ![IFR_options  is_installed  Audio_Analyzer] } \
    {
      grid forget  $tile_path
     
      return
    }
    # SOFTKEY 8 ( RETURN )
    View::AssignReturn  $tile_path  ""

    # Handle generic setup of the analyzer
    Audio_AnalyzerUI::Map  $tile_path
    
    # update sofkeys on trace state changes
    OnChange hook ::Sinad::State(Engine_enabled) [namespace code "Update_softkeys $tile_path"]
    OnChange hook ::Sinad::State(tlimit,state)   [namespace code "Update_alert $tile_path"]
    OnChange hook ::Sinad::State(tlimit,alert)   [namespace code "Update_alert $tile_path"]
  }
  
  #**************************************************************************
  #
  #        Name: Audio_Analyzer_FullUI::Unmap
  #
  #  Parameters: tile_path - tile path of the invoking widget
  #
  # Description: This function is called when the full screen audio analyzer 
  #              is unmapped from the screen.
  #
  #**************************************************************************
  proc Unmap { tile_path } \
  {
    Audio_AnalyzerUI::Unmap  $tile_path
    Cleanup $tile_path
  }
  proc Cleanup { tile_path } \
  {
    # disable RETURN softkey
    ::View::AssignReturn [winfo parent $tile_path] ""

    OnChange unhook ::Sinad::State(tlimit,state)  [namespace code "Update_alert $tile_path"]
    OnChange unhook ::Sinad::State(tlimit,alert)  [namespace code "Update_alert $tile_path"]
    OnChange unhook ::Sinad::State(Engine_enable) [namespace code "Update_softkeys $tile_path"]
  }
  

  #**************************************************************************
  #
  #        Name: Audio_Analyzer_FullUI::Destroy
  #
  #  Parameters: tile_path - tile path of the invoking widget
  #
  # Description: This function is called when the full screen audio analyzer 
  #              is destroyed.
  #
  #**************************************************************************
  proc Destroy { tile_path } \
  {
    Audio_AnalyzerUI::Destroy  $tile_path
    Cleanup $tile_path
  }
  
  #***************************************************************************
  #
  #        Name: Map_tg
  #
  #  Parameters: tile
  #
  # Description: map the track gen config tile
  #
  #***************************************************************************
  proc Map_tg {tile_path } \
  {
     set tile_path [Audio_AnalyzerUI::get_tile $tile_path]
     OnChange hook ::Sinad::State(tracking_gen) "Audio_Analyzer_FullUI::update_tg_source $tile_path"
     OnChange hook ::FGen::State(dev_units)     "Audio_Analyzer_FullUI::update_tg_source $tile_path"

  }
  #***************************************************************************
  #
  #        Name: Cleanup_tg
  #
  #  Parameters: tile
  #
  # Description: clean up track gen config tile
  #
  #***************************************************************************
  proc Cleanup_tg { tile_path } \
  {
     OnChange unhook  ::Sinad::State(tracking_gen) "Audio_Analyzer_FullUI::update_tg_source $tile_path"
     OnChange unhook ::FGen::State(dev_units)      "Audio_Analyzer_FullUI::update_tg_source $tile_path"
  }
  #***************************************************************************
  #
  #        Name: Unmap_tg
  #
  #  Parameters: tile
  #
  # Description: unmap track gen config tile
  #
  #***************************************************************************
  proc Unmap_tg { tile_path } \
  {
     set tile_path [Audio_AnalyzerUI::get_tile $tile_path]
     Cleanup_tg $tile_path 
  }
  #***************************************************************************
  #
  #        Name: Destroy_tg
  #
  #  Parameters: tile
  #
  # Description: 
  #
  #***************************************************************************
  proc Destroy_tg { tile_path } \
  {
     set tile_path [Audio_AnalyzerUI::get_tile $tile_path]
     Cleanup_tg $tile_path 
  }

  #**************************************************************************
  #
  #        Name: Audio_Analyzer_FullUI::Update_alert
  #
  #  Parameters: tile_path - 
  #
  # Description: update the state of the alert button
  #
  #**************************************************************************
  proc Update_alert { tile_path args } \
  {
     if {$::Sinad::State(tlimit,state) & 0x2000} \
     {
       $tile_path.alerts config -image image_bigbellalert
     } \
     elseif {$::Sinad::State(tlimit,alert) == "ON"} \
     {
        $tile_path.alerts config -image image_bigbellon
     } \
     else \
     {
        $tile_path.alerts config -image image_bigbelloff
     }

     switch -- [expr $::Sinad::State(tlimit,state) & 0xA00] \
     {
       0 { $tile_path.alerts config -bg grey}
       2048 { $tile_path.alerts config -bg blue}
       512  { $tile_path.alerts config -bg red}
       2560 { $tile_path.alerts config -bg red}
     }
  }
  
  #**************************************************************************
  #
  #        Name: update_tg_source
  # 
  #  Parameters: tile
  #
  # Description: Update display based upon changes to track source
  #
  #**************************************************************************
  proc update_tg_source { tile_path } \
  {
     if ![winfo exists $tile_path] { return }
     set level $tile_path.ent_sinc_level
     set units $tile_path.lbl_sinc_level_units
     set lun $::Sinad::State(tracking_gen)
     switch -- $lun \
     {
        0 \
        {
           grid remove $level
           grid remove $units
           return
        }
        1 -
        2 \
        {
           $units config -textvar {} -text V
           $level config -textvar FGen::State($lun,SINC,level)
        }
        3 -
        4 \
        {
            $units config -textvar ::FGen::State(dev_units) -text {}
            switch -- $::FGen::State(dev_units) \
            {
               "kHz" {$level config -textvar FGen::State($lun,SINC,dev)}
               "%"   {$level config -textvar FGen::State($lun,SINC,percent)}
            }
        }
     }

     Screens::position_widget $level
     Screens::position_widget $units
  }

  #**************************************************************************
  #
  #        Name: set_sinc_level
  # 
  #  Parameters:
  #
  # Description: Sets the level of the SINC waveform
  #
  #**************************************************************************
  proc set_sinc_level { } \
  {
    #puts "set_sinc_level"
    
    variable  StateUI
    set lun $::Sinad::State(tracking_gen)
    switch -- $lun \
     {
        0 \
        {
           grid remove $level
           grid remove $units
           return
        }
        1 -
        2 \
        {
           FGen::setLevel  $lun  $::FGen::State($lun,SINC,level)
        }
        3 -
        4 \
        {
            switch -- $::FGen::State(dev_units) \
            {
               "kHz" {FGen::setDeviation $lun $FGen::State($lun,SINC,dev)}
               "%"   {FGen::setPercent   $lun $FGen::State($lun,SINC,percent)}
            }
        }
     }
  }
  #***************************************************************************
  #
  #        Name: display_config
  #
  #  Parameters: tile_path - tile path of the invoking widget
  #
  # Description: Displays the Simulcast Application configuration toplevel
  #
  #***************************************************************************
  proc display_config { tile_path } \
  {
    #puts "display_config:  $tile_path"
    
    set top_level  [winfo toplevel $tile_path]

    if { [IFR_options  is_installed  Analog_Simulcast] && ($::Analog_Simul_App::StateUI(state) == "ON")  } \
    {
       set class ANALOG_SIMUL_APP_CFG
       set name analog_app_cfg
    } \
    else \
    {
       set class TRACK_CFG
       set name aud_anlz_cfg
    }
 
    set tile_path  [name_child  $top_level $name]

    if { ![winfo exists $tile_path] } \
    {
      set tile_path  [Screens::create_toplevel_tile  $tile_path  $class]
    }
        
    # Bind the F8 key for a return
    bind $tile_path <KeyRelease-F8>  "destroy $tile_path"

    safe_focus_force  ${tile_path}.btn_return
  }


  #**************************************************************************
  #
  #        Name: Audio_Analyzer_FullUI::Update_softkeys 
  #
  #  Parameters: tile_path - tile path of the invoking widget
  #
  # Description: Update the softkeys as needed
  #
  #**************************************************************************
  proc Update_softkeys { tile_path } \
  {
    if ![winfo exists $tile_path] { return }
    if ![winfo ismapped $tile_path] { return }
    
    # SOFTKEY 2:  Store Trace
    View::AssignSoftkey  $tile_path  2  "STORE TRACE"  \
                                   "Audio_AnalyzerUI::store_trace  $tile_path"
      
    View::AssignSoftkey  $tile_path  3  "SAVE TRACE"  \
                                    "Audio_AnalyzerUI::save_trace  $tile_path"
    # SOFTKEY 4:  Recall trace
    View::AssignSoftkey  $tile_path  4  "RECALL TRACE"  \
                                  "Audio_AnalyzerUI::recall_trace  $tile_path"
    
    # SOFTKEY 5:  Reset peak traces
    set en $::Sinad::State(Engine_enabled)
    if { $en & 0x30000} \
    {
       View::AssignSoftkey  $tile_path  5  "RESET PEAK"  \
                                  "Sinad trace_peak_rst -nowait"
    } \
    else \
    {
       View::AssignSoftkey  $tile_path  5  ""
    }
    
    if {$en & 8} \
    {
       View::AssignSoftkey $tile_path  6  "CLEAR AVG"  "Sinad tavrst -nowait"
    } \
    else \
    {
       View::AssignSoftkey $tile_path  6  ""
    }
    View::AssignSoftkey $tile_path  7 "MARKERS" "::Audio_Anlz_MarkersUI::config_marker  $tile_path"
    
    # show/remove the Math setup items
    if {$en & 0x40000} \
    {
       Screens::position_widget $tile_path.btn_ref_mode
       Screens::position_widget $tile_path.ent_ref_offset
       Screens::position_widget $tile_path.lbl_ref_offset_units 
    } \
    else \
    {
       grid forget $tile_path.btn_ref_mode
       grid forget $tile_path.ent_ref_offset
       grid forget $tile_path.lbl_ref_offset_units 
    }
  }


  ################################################################  
  #
  #  Menu Dropdown Interface Support
  #
  ################################################################

  #***************************************************************************
  #
  #        Name: set_options_menu
  #
  #  Parameters: 
  #
  # Description: Assigns Analyzer options menu
  #
  #***************************************************************************
  proc set_options_menu { widget  ScreenNamespace } \
  {
    #puts "set_options_menu:  $widget,  $ScreenNamespace"

    variable StateUI

    set topWin  [winfo toplevel $widget]
    
    # Get the Options menubutton path
    set buttonName  [View::GetOptionMenuName  $topWin]
    
    option add  *$buttonName*helpUrl  index.html  widgetDefault
    
    # Get the Options menu path
    set menuName  ${buttonName}.menu
    
    # Destroy the existing Options menu
    destroy  $menuName
    
    # Create a new Options menu
    menu $menuName -tearoff 0  
    
    # Add ENTER key options
    $menuName add command -accel  RETURN            \
                          -label  "Accept Options"  \
                          -com    "tkMenuUnpost $menuName"
                          
    $menuName add separator

    $menuName add check  -label     "Live Trace"                      \
                         -accel     1                                 \
                         -onvalue   "ON"                              \
                         -offvalue  "OFF"                             \
                         -variable  ::Sinad::State(trace_state,live)  \
                         -command   "Audio_AnalyzerUI::set_trace_state  live"
                         
    bind $menuName <KeyRelease-1>  "%W invoke [$menuName index end]"
                               
    $menuName add check  -label     "Stored Trace"                      \
                         -accel     2                                   \
                         -onvalue   "ON"                                \
                         -offvalue  "OFF"                               \
                         -variable  ::Sinad::State(trace_state,stored)  \
                         -command   "Audio_AnalyzerUI::set_trace_state  stored"
                         
    bind $menuName <KeyRelease-2>  "%W invoke [$menuName index end]"

    $menuName add check  -label     "Average"                        \
                         -accel     3                                \
                         -onvalue   "ON"                             \
                         -offvalue  "OFF"                            \
                         -variable  ::Sinad::State(trace_state,avg)  \
                         -command   "Audio_AnalyzerUI::set_trace_state  avg"
                         
    bind $menuName <KeyRelease-3>  "%W invoke [$menuName index end]"

    $menuName add check  -label     "Upper Limit Trace"               \
                         -accel     4                                 \
                         -onvalue   "ON"                              \
                         -offvalue  "OFF"                             \
                         -variable  ::Sinad::State(trace_state,ulim)  \
                         -command   "Audio_AnalyzerUI::set_trace_state  ulim"
                         
    bind $menuName <KeyRelease-4>  "%W invoke [$menuName index end]"

    $menuName add check  -label     "Lower Limit Trace"               \
                         -accel     5                                 \
                         -onvalue   "ON"                              \
                         -offvalue  "OFF"                             \
                         -variable  ::Sinad::State(trace_state,llim)  \
                         -command   "Audio_AnalyzerUI::set_trace_state  llim"
                         
    bind $menuName <KeyRelease-5>  "%W invoke [$menuName index end]"

    $menuName add check  -label     "Peak High Trace"               \
                         -accel     6                                 \
                         -onvalue   "ON"                              \
                         -offvalue  "OFF"                             \
                         -variable  ::Sinad::State(trace_state,pkhi)  \
                         -command   "Audio_AnalyzerUI::set_trace_state  pkhi"
                         
    bind $menuName <KeyRelease-6>  "%W invoke [$menuName index end]"
    $menuName add check  -label     "Peak Low Trace"               \
                         -accel     7                                 \
                         -onvalue   "ON"                              \
                         -offvalue  "OFF"                             \
                         -variable  ::Sinad::State(trace_state,pklo)  \
                         -command   "Audio_AnalyzerUI::set_trace_state  pklo"
                         
    bind $menuName <KeyRelease-7>  "%W invoke [$menuName index end]"

    $menuName add check   -label    "Math Trace"                       \
                          -accel    8                                  \
                          -onvalue   "ON"                              \
                          -offvalue  "OFF"                             \
                          -variable  ::Sinad::State(trace_state,math)  \
                          -command   "Audio_AnalyzerUI::set_trace_state math"
                                     
    bind $menuName <KeyRelease-8>  "%W invoke [$menuName index end]"

    $menuName add separator

    # Motorola Simulcast Measurement Application
    if { [IFR_options  is_installed  Analog_Simulcast] } \
    {
      $menuName add check  -label     "Motorola Simulcast App"           \
                           -accel     9                                  \
                           -onvalue   "ON"                               \
                           -offvalue  "OFF"                              \
                           -variable  ::Analog_Simul_App::StateUI(state) \
                           -command   Analog_Simul_App::start
                         
      bind $menuName <KeyRelease-9>  "%W invoke [$menuName index end]"

      $menuName add separator
    }

    # Add ESCAPE functionality
    $menuName add command -accel ESC       \
                          -label "Escape"  \
                          -com   "tkMenuUnpost $menuName"

    # Bind button release so default bindings do not unpost the menu
    # bind to the Return and Escape hardkeys
    bind $menuName <KeyRelease-F8>     "tkMenuUnpost $menuName; break"
    
    bind $menuName <KeyRelease-Return> { MenuInvokeRemain %W 0; break }
    bind $menuName <ButtonRelease-1>   { MenuInvokeRemain %W 1; break }

    return $buttonName
  } 

};  # End of Audio_Analyzer_FullUI namespace



namespace eval Audio_Analyzer_QtrUI \
{
  # Global variables
  variable LongName       "qtr_audio_anlz"
  variable ShortName      "Audio Analyzer"
  variable Size           Quarter

  variable tile_path      ""
  variable tile_class     QTR_AUDIO_ANLZ


  #**************************************************************************
  #
  #         Name: Audio_Analyzer_QtrUI::Create
  #
  #   Parameters:
  #
  #  Description: 
  #
  #**************************************************************************
  proc Create { tile_path } \
  {
    #puts "Audio_Analyzer_QtrUI::Create"
    
    variable LongName
    variable tile_class

    set top_level  [winfo toplevel $tile_path]

    set tile_path  [name_child  $top_level  $LongName]
 
    #puts "tile_path:  $tile_path"
    #puts "tile_class:  $tile_class"
    
    if { ![winfo exists $tile_path] } \
    {
      set tile_path  [Screens::create_tile  $tile_path  $LongName  $tile_class]
      
      # Configure the Audio Analyzer traces
      ${tile_path}.anlz trace create  tlive  -sys        [Sinad -system]       \
                                             -subsys     [Sinad -subsys tlive] \
                                             -cfgsubsys  [Sinad -subsys tcfg]  \
                                             -color      white                 \
                                             -active 1

      ${tile_path}.anlz trace create  tref   -sys        [Sinad -system]       \
                                             -subsys     [Sinad -subsys tref]  \
                                             -cfgsubsys  [Sinad -subsys tcfg]  \
                                             -color      yellow                \
                                             -active 0

      ${tile_path}.anlz trace create  tulim  -sys        [Sinad -system]       \
                                             -subsys     [Sinad -subsys tulim] \
                                             -cfgsubsys  [Sinad -subsys tcfg]  \
                                             -color      green                \
                                             -active 0

      ${tile_path}.anlz trace create  tllim  -sys        [Sinad -system]       \
                                             -subsys     [Sinad -subsys tllim] \
                                             -cfgsubsys  [Sinad -subsys tcfg]  \
                                             -color      cyan                \
                                             -active 0

      ${tile_path}.anlz trace create  tavg   -sys        [Sinad -system]       \
                                             -subsys     [Sinad -subsys tavg]  \
                                             -cfgsubsys  [Sinad -subsys tcfg]  \
                                             -color      orange                \
                                             -active 0
      ${tile_path}.anlz trace create  pkhi   -sys        [Sinad -system]       \
                                             -subsys     [Sinad -subsys tpkh]  \
                                             -cfgsubsys  [Sinad -subsys tcfg]  \
                                             -color      grey                  \
                                             -active 0

      ${tile_path}.anlz trace create  pklo   -sys        [Sinad -system]       \
                                             -subsys     [Sinad -subsys tpkl]  \
                                             -cfgsubsys  [Sinad -subsys tcfg]  \
                                             -color      grey                  \
                                             -active 0
      ${tile_path}.anlz trace create  math   -sys        [Sinad -system]       \
                                             -subsys     [Sinad -subsys tmath]  \
                                             -cfgsubsys  [Sinad -subsys tmcfg]  \
                                             -color      white                  \
                                             -active 0

    }
    
    return $tile_path
  }


  #**************************************************************************
  #
  #        Name: Audio_Analyzer_QtrUI::Map
  #
  #  Parameters: tile_path  Analyzer widget path
  #
  # Description: This function is called when the quarter screen audio 
  #              analyzer needs to be mapped on the screen.
  #
  #**************************************************************************
  proc Map { tile_path } \
  {
    #puts "Audio_Analyzer_QtrUI::Map:  $tile_path"

    if { ![IFR_options  is_installed  Audio_Analyzer] } \
    {
      grid forget  $tile_path
     
      return
    }

    # Handle generic setup of the analyzer
    Audio_AnalyzerUI::Map  $tile_path

    return
  }


  #**************************************************************************
  #
  #        Name: Audio_Analyzer_QtrUI::Unmap
  #
  #  Parameters: tile_path  - Analyzer widget path
  #
  # Description: This function is called when the quarter screen audio 
  #              analyzer needs to be removed from the screen.
  #
  #**************************************************************************
  proc Unmap { tile_path } \
  {
    Audio_AnalyzerUI::Unmap  $tile_path

    return
  }
 
  
  #**************************************************************************
  #
  #        Name: Audio_Analyzer_QtrUI::Destroy
  #
  #  Parameters: tile_path - Analyzer widget path
  #
  # Description: 
  #
  #**************************************************************************
  proc Destroy { tile_path } \
  {
    Audio_AnalyzerUI::Destroy $tile_path
  }


  #***************************************************************************
  #
  #        Name: zoom_analyzer_screen
  #
  #  Parameters: widget - tile path of the invoking widget
  #
  # Description: Zoom to the full Audio Analyzer screen 
  #
  #***************************************************************************
  proc zoom_analyzer_screen { widget } \
  {
     variable StateUI

     set tile_path  [winfo toplevel $widget]
     
     # Save the current screen to assign to the RETURN Softkey
     set current_screen  $::Screens::CurrentNamespace

     # Show the full Audio Analyzer
     Screens::Show  $widget  Audio_Analyzer_Screen

     # Delay to allow the screen to setup and display
     after 1000
     
     # Assign the RETURN softkey to return to the current screen
     View::AssignReturn  $tile_path  "Screens::Show  $widget  $current_screen"

     return
  }

};  # End of Audio_Analyzer_QtrUI namespace


namespace eval Audio_AnalyzerUI \
{
  # Global variables
  variable  LongName       "audio_anlz"
  variable  ShortName      "Audio Analyzer"
  variable  Size           ""

  variable  tile_path      ""
  variable  tile_class     ""
  
  variable  last_focus     ""
  
  variable  file_label  "Trace_data"

  variable  StateUI
  
  set StateUI(units)         "dB"

  set StateUI(triggering)    "NONE"
  
  
  set StateUI(live_data)     ""
  set StateUI(trace_data)    ""

  set StateUI(triggering_menu)  { "NONE"  "KAISER"  "KAISER TRIG" }
  
  set StateUI(temp_status)  ""

  variable Traces
  variable Modes
  foreach {ui name} { "LIVE" tlive
                      "REFERENCE" tref
                      "UPPER LIMIT" tulim
                      "LOWER LIMIT" tllim
                      "AVERAGE" tavg
                      "PEAK HI" tpkh
                      "PEAK LO" tpkl
                  } \
  {
     set Traces([Sinad -subsystem $name]) $ui
  }
  foreach {ui value} { OFF    0
                      PLUS   1
                      MINUS  2
                      INVERT 3  
                    } \
  {
     set Modes($value) $ui
  }
  proc update_math_mode { } \
  {
     variable StateUI
     variable Modes
     variable Traces
     if [info exists Traces($::Sinad::State(tmath,trace_a))] \
     {
        set StateUI(tmath,trace_a) $Traces($::Sinad::State(tmath,trace_a))
     }
     if [info exists Traces($::Sinad::State(tmath,trace_b))] \
     {
        set StateUI(tmath,trace_b) $Traces($::Sinad::State(tmath,trace_b))
     }
     if [info exists Modes($::Sinad::State(tmath,mode,raw))] \
     {
        set StateUI(tmath,mode) $Modes($::Sinad::State(tmath,mode,raw))
     }
  }
  OnChange hook ::Sinad::State(tmath,mode,raw) [namespace code "update_math_mode"]
  OnChange hook ::Sinad::State(tmath,trace_a)  [namespace code "update_math_mode"]
  OnChange hook ::Sinad::State(tmath,trace_b)  [namespace code "update_math_mode"]

  
  proc Update_tmath_vpos {tile_path} \
  {
     if ![winfo exists $tile_path] { return }
     if {![info exists ::Audio_AnalyzerUI::StateUI(tmath,offset)] || ($::Audio_AnalyzerUI::StateUI(tmath,offset) == "")} \
     {
        set ::Audio_AnalyzerUI::StateUI(tmath,offset) 0.0
     }
     
     $tile_path.anlz.math config -voffs $::Audio_AnalyzerUI::StateUI(tmath,offset)
  }
  
  #**************************************************************************
  #
  #        Name: Map
  #
  #  Parameters: 
  #
  # Description: 
  #
  #**************************************************************************
  proc Map { tile_path } \
  {
    #puts "Audio_AnalyzerUI::Map:  $tile_path"

    variable StateUI

    if { ![IFR_options  is_installed  Audio_Analyzer] } \
    {
      grid forget  $tile_path
     
      return
    }
    OnChange hook ::Audio_AnalyzerUI::StateUI(tmath,offset) "::Audio_AnalyzerUI::Update_tmath_vpos $tile_path"
    OnChange hook ::Sinad::State(Engine_enabled) \
                            "Audio_AnalyzerUI::update_trace_state  $tile_path"

    # Motorola Simulcast Measurement Application
    OnChange hook ::Analog_Simul_App::StateUI(state) \
                                  "Analog_Simul_App::update_start  $tile_path"
    
    # Synchronize the Function Generator SINC variables
    OnChange hook ::FGen::State(1,SINC,level)  "Analog_Simul_App::sync_vars"

    OnChange hook ::FGen::State(1,SINC,rate)   "Analog_Simul_App::sync_vars"

    OnChange hook ::FGen::State(1,SINC,omega)  "Analog_Simul_App::sync_vars"


    # Set the display to the present scale
    Sinad::setScale
    
    
    # Markers
    foreach { num } $::Audio_Anlz_MarkersUI::marker_list \
    {
      OnChange hook ::Sinad::State(marker,$num,state)  \
                "Audio_Anlz_MarkersUI::update_marker_on_off  $tile_path  $num"
                  
      OnChange hook ::Audio_Anlz_MarkersUI::StateUI(marker,$num,orient)  \
                "Audio_Anlz_MarkersUI::update_marker_on_off  $tile_path  $num"

      # Update the vertical marker position for changes in frequency
      OnChange hook ::Sinad::State(marker,$num,freq)  \
              "Audio_Anlz_MarkersUI::update_marker_position  $tile_path  $num"

      OnChange hook ::Sinad::State(span) \
              "Audio_Anlz_MarkersUI::update_marker_position  $tile_path  $num"

      OnChange hook ::Sinad::State(center_freq) \
              "Audio_Anlz_MarkersUI::update_marker_position  $tile_path  $num"

    }
    return
  }


  #**************************************************************************
  #
  #        Name: Cleanup
  #
  #  Parameters: tile_path
  #
  # Description: 
  #
  #**************************************************************************
  proc Cleanup { tile_path } \
  {
    #puts "Audio_AnalyzerUI::Cleanup:  $tile_path"

    OnChange unhook ::Sinad::State(Engine_enabled) \
                            "Audio_AnalyzerUI::update_trace_state  $tile_path"

    # Motorola Simulcast Measurement Application
    OnChange unhook ::Analog_Simul_App::StateUI(state) \
                                  "Analog_Simul_App::update_start  $tile_path"
    
    # Synchronize the Function Generator SINC variables
    OnChange unhook ::FGen::State(1,SINC,level) "Analog_Simul_App::sync_vars"

    OnChange unhook ::FGen::State(1,SINC,rate)  "Analog_Simul_App::sync_vars"

    OnChange unhook ::FGen::State(1,SINC,omega)  "Analog_Simul_App::sync_vars"
    OnChange unhook ::Audio_AnalyzerUI::StateUI(tmath,offset) "::Audio_AnalyzerUI::Update_tmath_vpos $tile_path"

    # Markers
    foreach { num } $::Audio_Anlz_MarkersUI::marker_list \
    {
      OnChange unhook ::Sinad::State(marker,$num,state) \
                "Audio_Anlz_MarkersUI::update_marker_on_off  $tile_path  $num"

      OnChange unhook ::Audio_Anlz_MarkersUI::StateUI(marker,$num,orient) \
                "Audio_Anlz_MarkersUI::update_marker_on_off  $tile_path  $num"

      OnChange unhook ::Sinad::State(marker,$num,freq) \
              "Audio_Anlz_MarkersUI::update_marker_position  $tile_path  $num"

      OnChange unhook ::Sinad::State(span) \
              "Audio_Anlz_MarkersUI::update_marker_position  $tile_path  $num"

      OnChange unhook ::Sinad::State(center_freq) \
              "Audio_Anlz_MarkersUI::update_marker_position  $tile_path  $num"
    }

    return
  }
  
  
  #**************************************************************************
  #
  #        Name: Unmap
  #
  #  Parameters: 
  #
  # Description: 
  #
  #**************************************************************************
  proc Unmap { tile_path } \
  {
    Cleanup $tile_path
    
    return
  }


  #**************************************************************************
  #
  #        Name: Destroy
  #
  #  Parameters: 
  #
  # Description: 
  #
  #**************************************************************************
  proc Destroy { tile_path } \
  {
    Cleanup $tile_path

    return
  }
  #**************************************************************************
  #
  #        Name: Update_math_mode
  #
  #  Parameters: 
  #
  # Description: 
  #
  #**************************************************************************
  proc Update_math_mode { tile_path args} \
  {
     set tile_path [winfo toplevel $tile_path]
     if ![winfo exists $tile_path] \
     {
        Cleanup_math $tile_path
        return
     }
     switch -- $::Sinad::State(tmath,mode,raw) \
     {
        0 \
        {
            $tile_path.btn_a config -state disabled
            $tile_path.btn_b config -state disabled
        }
        3 \
        {
            $tile_path.btn_a config -state normal
            $tile_path.btn_b config -state disabled
        }
        default \
        {
            $tile_path.btn_a config -state normal
            $tile_path.btn_b config -state normal
        }
     }
  }
  #**************************************************************************
  #
  #        Name: Map_math
  #
  #  Parameters: 
  #
  # Description: 
  #
  #**************************************************************************
  proc Map_math { tile_path } \
  {
      set cmd [namespace code "Update_math_mode $tile_path"]
      OnChange hook ::Sinad::State(tmath,mode,raw) $cmd
  }
  #**************************************************************************
  #
  #        Name: Cleanup_math
  #
  #  Parameters: 
  #
  # Description: 
  #
  #**************************************************************************
  proc Cleanup_math { tile_path } \
  {
      set cmd [namespace code "Update_math_mode $tile_path"]
      OnChange unhook ::Sinad::State(tmath,mode,raw) $cmd
  }
  #**************************************************************************
  #
  #        Name: Unmap_math
  #
  #  Parameters: 
  #
  # Description: 
  #
  #**************************************************************************
  proc Unmap_math { tile_path } \
  {
    Cleanup_math $tile_path
  }
  #**************************************************************************
  #
  #        Name: Destroy_math
  #
  #  Parameters: 
  #
  # Description: 
  #
  #**************************************************************************
  proc Destroy_math { tile_path } \
  {
    Cleanup_math $tile_path
  }
  #**************************************************************************
  #
  #        Name: Register
  #
  #  Parameters: 
  #
  # Description: 
  #
  #**************************************************************************
  proc Register { tile_path } \
  {
    #puts "Audio_AnalyzerUI::Register"

    return
  }

  
  ################################################################  
  #
  #  User Interface support procedures
  #
  ################################################################

  #***************************************************************************
  #
  #        Name: get_tile
  #
  #  Parameters: any child widget of the tile
  #
  # Description: Get the name of the tile's parent frame
  #
  #***************************************************************************
  proc get_tile { widget } \
  {
    while { 1 } \
    {
      switch -- [winfo class $widget] \
      {
        "QTR_AUDIO_ANLZ"   -
        "TRACK_CFG" -
        "ANALOG_SIMUL_APP_CFG" -
        "FULL_AUDIO_ANLZ"  { return $widget}
      }
        
      set widget [winfo parent $widget]
    }
  }
  
  
  #***************************************************************************
  #
  #        Name: set_start_freq
  #
  #  Parameters:
  #
  # Description: Sets the Start frequency.
  #
  #***************************************************************************
  proc set_start_freq { } \
  {
    #puts "set_start_freq"
    
    after idle "Sinad  minx  $::Sinad::State(minx)  -nowait"

    return
  }


  #***************************************************************************
  #
  #        Name: set_stop_freq
  #
  #  Parameters:
  #
  # Description: Sets the Stop frequency
  #
  #***************************************************************************
  proc set_stop_freq { } \
  {
    #puts "set_stop_freq"
    
    after idle "Sinad  maxx  $::Sinad::State(maxx)  -nowait"
    
    return
  }

  #***************************************************************************
  #
  #        Name: set_center_freq
  #
  #  Parameters:
  #
  # Description: Sets the center frequency
  # Purpose:  This function is needed because the UI overwrites the center
  #    frequency, and because of engine clipping, we need to know what type
  #    of change is being made to the center frequency.
  #***************************************************************************
  proc set_center_freq { } \
  {
    #puts "set_center_freq"

    set oldCenter [expr $::Sinad::State(minx) + \
             (($::Sinad::State(maxx) - $::Sinad::State(minx)) / 2.0) ]

    ::Sinad::setCenterFreq $::Sinad::State(center_freq) $oldCenter     
  }


  #***************************************************************************
  #
  #        Name: set_tos
  #
  #  Parameters:
  #
  # Description: sets Analyzer top of screen
  #
  #***************************************************************************
  proc set_tos { } \
  {
    Sinad maxy $::Sinad::State(maxy)  -nowait
  }


  #***************************************************************************
  #
  #        Name: set_average
  #
  #  Parameters: 
  #
  # Description: Sets Averaging.
  #
  #***************************************************************************
  proc set_average { } \
  {
    #puts "set_average"

    after idle "Sinad  tavg_factor  $::Sinad::State(tavg_factor)  -nowait"

    return
  }


  #***************************************************************************
  #
  #        Name: set_trace_state
  #
  #  Parameters: trace - Audio Analyzer trace
  #
  # Description: Sets the state of the Audio Analyzer traces: LIVE, 
  #              AVERAGE, STORED, ULIM, or LLIM.
  #
  #***************************************************************************
  proc set_trace_state { trace } \
  {
    #puts "set_trace_state:  $trace"

    variable StateUI
    
    switch -- $trace \
    {
      "live"    { set trace_enable  4  }

      "avg"     { set trace_enable  8  }

      "stored"  { set trace_enable  16 }

      "ulim"    { set trace_enable  32 }

      "llim"    { set trace_enable  64 }

      "pkhi"    { set trace_enable  0x10000 }
      
      "pklo"    { set trace_enable  0x20000 }
      
      "math"    { set trace_enable  0x40000 }
    }
    
    # Get the present Sinad enable value
    set sinad_enable  [Sinad  en]
    
    # Determine if the trace should be turned ON or OFF
    # The variable has been set in the menu checkbutton
    if { $::Sinad::State(trace_state,${trace}) == "OFF" } \
    {
      # Turn OFF the trace
      set sinad_enable  [expr  $sinad_enable & ~$trace_enable]
    } \
    else \
    {
      # Turn ON the trace
      set sinad_enable  [expr  $sinad_enable | $trace_enable]
    }
    
    # Set the Sinad enable value
    Sinad  en  $sinad_enable -nowait
  }


  #***************************************************************************
  #
  #        Name: update_trace_state
  #
  #  Parameters: tile_path - tile path of the invoking widget
  #
  # Description: Updates the display to the mode of the Audio Analyzer
  #
  #***************************************************************************
  proc update_trace_state { tile_path } \
  {
    #puts "update_trace_state:  $tile_path,  $args"

    variable StateUI
    
    set tile_path  [get_tile  $tile_path]

    # For the Full Audio Analyzer,
    # remove the AVERAGE entry field and clear the Softkey
    set en $::Sinad::State(Engine_enabled)
    if { [winfo class $tile_path] == "FULL_AUDIO_ANLZ" } \
    {
      if {$en & 8} \
      {
         Screens::position_widget ${tile_path}.lbl_avg
         Screens::position_widget ${tile_path}.ent_avg
      } \
      else \
      {
         grid remove  ${tile_path}.lbl_avg
         grid remove  ${tile_path}.ent_avg
      }
    }
    

    # stupidity check - if no traces active, turn on live
    if {($en & 0x7007c) == 0} \
    {
       $tile_path.anlz.tlive config -active 1
    }
    # turn on the traces
    ${tile_path}.anlz.tlive  config  -active [expr $en & 4]
    ${tile_path}.anlz.tavg   config  -active [expr $en & 8]
    ${tile_path}.anlz.tref   config  -active [expr $en & 16]
    ${tile_path}.anlz.tulim  config  -active [expr $en & 32]
    ${tile_path}.anlz.tllim  config  -active [expr $en & 64]
    ${tile_path}.anlz.pkhi   config  -active [expr $en & 0x10000]
    ${tile_path}.anlz.pklo   config  -active [expr $en & 0x20000]
    ${tile_path}.anlz.math   config  -active [expr $en & 0x40000]
    
  }


  #***************************************************************************
  #
  #        Name: toggle_triggering
  #
  #  Parameters: 
  #
  # Description: Sets the triggering of the Sinad meter
  #
  #***************************************************************************
  proc toggle_triggering { } \
  {
    if { $::Sinad::State(anlz_trig,raw) == 0 } \
    {
      Sinad anlz_trig  on
    } \
    else \
    {
      Sinad anlz_trig  off
    }

    return
  }
  
  

  #############################################################################
  # 
  #   Analyzer Trace Store/Save/Recall Routines
  # 
  #############################################################################
  
  #***************************************************************************
  #
  #        Name: store_trace
  #
  #  Parameters: tile_path - tile path of the invoking widget
  #
  # Description: Stores the present LIVE trace to the REFERENCE trace
  #
  #***************************************************************************
  proc store_trace { tile_path } \
  {
    #puts "store_trace:  $tile_path"
    
    variable  StateUI
    
    set tile_path  [get_tile  $tile_path]

    # If the AVERAGE trace is the only trace displayed, store it,
    # else store the LIVE trace
    if { [expr  $::Sinad::State(Engine_enabled) & 0x7C] == 8 } \
    {
      set StateUI(live_data)  [Sinad  trace  tavg]
    } \
    else \
    {
      # Get the current LIVE trace data
      set StateUI(live_data)  [Sinad  trace  tlive]
    }
    
    # Set the REFERENCE trace data to the LIVE trace data
    Sinad  trace  tref  ::Audio_AnalyzerUI::StateUI(live_data)
  }


  #***************************************************************************
  #
  #        Name: save_trace
  #
  #  Parameters: widget - tile path of the invoking widget
  #
  # Description: Saves the previously stored trace data to a file
  #
  #***************************************************************************
  proc save_trace { widget } \
  {
    #puts "save_trace:  $widget"
    
    variable  file_label
    variable  StateUI
    
    set tile_path  [get_tile  $widget]

    set trace_data_dir  "/usr/audio_analyzer/trace_data/"

    # Inititalize the stored trace data
    
    append data_buffer "input $::Sinad::State(input)\n"
    
    # Start the stored trace data with the Sinad minx value
    append  data_buffer  "minx  {[Sinad  minx]} \n"

    # Next, add the Sinad maxx value
    append  data_buffer  "maxx  {[Sinad  maxx]} \n"
    
    # Add the source of the data
    append  data_buffer  [subst "source  {$::Sinad::State(input)} \n"]

    # Get the stored trace data
    foreach trace { tlive tref tulim tavg tpkh tpkl tmath } \
    {
       append  data_buffer  "$trace  {[Sinad  trace  $trace]} \n"
    }

    #puts "data_buffer:  $data_buffer"
    
    # Save the data to a file
    set save_flag  [File_SaveUI::display_file_save  $widget          \
                                                    $trace_data_dir  \
                                                    $data_buffer     \
                                                    "TRACE_DATA"     \
                                                    "$file_label"    \
                                                    ".DAT"           \
                                                    NO_ENCRYPT       \
                                                    NO_CSV           \
                                                    NO_MODES         \
                                                    NO_OPT_EXT]

    #puts "save_flag:  $save_flag"

    # Get the file label after the file has been saved
    set file_label  $::File_SaveUI::file_label
    unset data_buffer
  }


  #***************************************************************************
  #
  #        Name: recall_trace
  #
  #  Parameters: tile_path - tile path of the invoking widget
  #
  # Description: Recalls previously saved trace data from a file to 
  #              either the UPPER LIMIT, LOWER LIMIT, or REFERENCE
  #              trace.
  #
  #***************************************************************************
  proc recall_trace { tile_path } \
  {
    #puts "recall_trace:  $tile_path"
    
    variable  StateUI

    set tile_path  [get_tile  $tile_path]

    set trace_data_dir  "/usr/audio_analyzer/trace_data"
    
    set data_buffer  [::File_RecallUI::display_file_recall  $tile_path       \
                                                            $trace_data_dir  \
                                                            "TRACE_DATA"     \
                                                            ".DAT"           \
                                                            NO_ENCRYPT       \
                                                            TRACE_OPTIONS    ]
                                                               
    #puts "data_buffer(0-100):  [string range $data_buffer  0  100]"
    #puts "data_buffer:  $data_buffer"
    
    # Return if no data was recalled, i.e. CANCEL was pressed
    if { $data_buffer == "" } \
    {
      return
    }
    
    # Set up an array from the recalled data
    array set  temp  $data_buffer
    
    if [info exists temp(data)] \
    {
       set ul data
       set ll data
       set st data
    } \
    else \
    {
       set ul tlive
       set ll tlive
       set st tlive
    }
    if [info exists temp(tulim)] { set ul tulim}
    if [info exists temp(tllim)] { set ll tllim}
    if [info exists temp(tref) ] { set ul tref}
    
    #puts "StateUI(trace_data)(0-100):  [lrange $StateUI(trace_data)  0  30]"
    #puts "StateUI(trace_data):  $StateUI(trace_data)"
    
    # Set either the UPPER LIMIT, LOWER LIMIT, or REFERENCE trace to the 
    # recalled trace data
    switch -- $::File_RecallUI::trace_option \
    {
      "UPPER LIMIT"  { if [info exists temp($ul)] {Sinad  trace  tulim  temp($ul)} } 
      
      "LOWER LIMIT"  { if [info exists temp($ll)] {Sinad  trace  tllim  temp($ll)} }
      
      "STORED"       { if [info exists temp($st)] {Sinad  trace  tref   temp($st)} }
    }
    
    return
  }


  #############################################################################
  # 
  #   Audio Analyzer Subsystem Control/Interface Routines
  # 
  #############################################################################
  
  #***************************************************************************
  #
  #        Name: set_freq_mouse
  #
  #  Parameters: widget  - invoking widget path
  #              x_coord - present x coordinate of the mouse relative to the 
  #                        Analyzer trace widget
  #
  # Description: Sets the Audio Analyzer center frequency to the frequency 
  #              represented by the position of the mouse on the Audio 
  #              Analyzer trace widget when double clicked.
  #
  #***************************************************************************
  proc set_freq_mouse { widget  x_coord } \
  {
    #puts "set_freq_mouse:  $widget,  $x_coord"
    
    set tile_path  [get_tile  $widget]
    
    # Format the x-coordinate as a real number
    set x_coord  [expr double($x_coord)]
    
    # Get the width of the Audio Analyzer trace widget
    set width  [${tile_path}.anlz cget -width]
    
    # Format the width as a real number
    set width  [expr double($width)]
    
    # Determine the center of the Audio Analyzer trace widget
    set center  [expr $width / 2.0]
    
    # Determine the x offset of the mouse position
    set offset  [expr $x_coord - $center]
    
    # Get the Audio Analyzer span
    set span  $::Sinad::State(span)
    
    # Set the offset frequency to the ratio of the x offset value
    set offset_freq  [expr ($offset / $center) * ($span / 2.0)]

    # Get the center frequency
    set center_freq  $::Sinad::State(center_freq)
    
    # Set the new center frequency to the present center freq plus the offset
    set new_freq  [expr $center_freq + $offset_freq]
    
    # Set the Audio Analyzer to the new center frequency 
#    set ::Sinad::State(center_freq)  $new_freq
 
    after idle  "Sinad::setCenterFreq $new_freq $::Sinad::State(center_freq)" 
  }
  
 
  #***************************************************************************
  #
  #        Name: set_span_mouse
  #
  #  Parameters: widget  - invoking widget path
  #              x_coord - present x coordinate of the mouse relative to the 
  #                        Analyzer trace widget
  #              mode    - a flag to determine which maouse button action
  #                        was received to capture the start and stop
  #                        mouse coordinates.
  #
  # Description: Sets the Audio Analyzer span to a span represented by the 
  #              movement of the mouse on the Audio Analyzer trace widget 
  #              when the left mouse button is held down while in motion.
  #
  #***************************************************************************
  proc set_span_mouse { widget  x_coord  mode } \
  {
    #puts "set_span_mouse:  $widget,  $x_coord,  $mode"
    
    variable  StateUI
    
    set tile_path  [get_tile  $widget]
    
    if { $mode == "START" } \
    {
      set StateUI(x_start)  [expr double($x_coord)]
    }
    
    if { $mode == "STOP" } \
    {
      set StateUI(x_stop)  [expr double($x_coord)]
      
      set StateUI(x_span)  [expr abs($StateUI(x_stop) - $StateUI(x_start))]
      
      set StateUI(x_center)  [expr ($StateUI(x_stop) + $StateUI(x_start)) / 2.0]
      
      set StateUI(x_center)  [expr int($StateUI(x_center))]

      # Reset the center frequency to the center of the selected area
      set_freq_mouse  $tile_path  $StateUI(x_center)

      set trace_width   [${tile_path}.anlz cget -width]
  
      set trace_width   [expr double($trace_width)]

      set span  $::Sinad::State(span)
    
      set span_selected  [expr ($StateUI(x_span) / $trace_width) * $span]
      
      # Round off the span
      set ::Sinad::State(span)  [format  "%.0f"  $span_selected]

      after idle "Sinad::setSpan"
    }

    return
  }


  #**************************************************************************
  #
  #        Name: getSaveState
  #
  #  Parameters: none
  #
  # Description: This function returns a list of TCL commands which can be 
  #              used to set the Audio_AnalyzerUI tiles to its current state
  #
  #**************************************************************************
  proc getSaveState { } \
  {
    variable StateUI
 
    set returnList \
    {

    }
        
    return [subst $returnList]
  }


  #**************************************************************************
  # set_math_offset - set the offset of the math trace
  #**************************************************************************
  proc set_math_offset { } \
  {
     Sinad tmmode $::Sinad::State(tmath,mode,raw) $::Sinad::State(tmath,offset) -nowait
  }
  #**************************************************************************
  # setup_math - configure the math trace settings
  #**************************************************************************
  proc setup_math { widget } \
  {
    set top_level  [View::Find_Main_toplevel $widget]
    set tile_path  [name_child  $top_level  audio_tmath]
    set tile_path  [Screens::create_toplevel_tile  $tile_path  AUDIO_MATH]
  }
  #**************************************************************************
  # Config_limits - set the limit alarm modes
  #**************************************************************************
  proc Config_limits { widget } \
  {
     set top_level  [View::Find_Main_toplevel $widget]
     set tile_path  [name_child  $top_level  audio_limits]
     set tile_path  [Screens::create_toplevel_tile  $tile_path  AUDIO_LIMITS]
     # add in the controls for the trace on/off
     set enables [name_child $tile_path enables]
     set row 1
     label $enables.title_l -text "Trace"
     label $enables.title_lower -text "Low limit"
     label $enables.title_upper -text "Hi limit"
     label $enables.title_alert -text "Alert"
     grid $enables.title_l $enables.title_lower $enables.title_upper \
                 $enables.title_alert
     
     foreach {trace name} { tlive "Live" tavg "Average" tref "Reference"
                            tpkh "Peak high" tpkl "Peak low" tmath "Math"} \
     {
        label $enables.${trace}_l -text $name
        button $enables.${trace}_lower -textvar $::Sinad::State(tlimit,$trace,low) \
                                       -command [namespace code "Toggle_item $trace 1"]
        button $enables.${trace}_upper -textvar $::Sinad::State(tlimit,$trace,hi) \
                                       -command [namespace code "Toggle_item $trace 2"]
        button $enables.${trace}_alert -textvar $::Sinad::State(tlimit,$trace,alarm) \
                                       -command [namespace code "Toggle_item $trace 4"]
        grid $enables.${trace}_l $enables.${trace}_lower \
             $enables.${trace}_upper $enables.${trace}_alert
     }
  }
  proc Toggle_item { trace bit } \
  {
     set bit [expr $::Sinad::State(tlimit,$trace,enable) ^ $bit]
     Sinad::set_trace_enable
  }
  proc Map_limit { tile_path } \
  {
  }
  proc Cleanup_limit { tile_path } \
  {
  }
  proc Unmap_limit { tile_path } \
  {
     Cleanup_limit $tile_path
  }
  proc Destroy_limit { tile_path } \
  {
     Cleanup_limit $tile_path
  }
  proc set_upper_limit { } \
  {
  }
  proc set_lower_limit { } \
  {
  }
  #**************************************************************************
  #
  #        Name: Sync
  #
  #  Parameters: none
  #
  # Description: Write state variables to Audio_AnalyzerUI
  #
  #**************************************************************************
  proc Sync { } \
  {
    eval [getSaveState]
  }


};  # End of Audio_AnalyzerUI namespace


#****************************************************************************
# This registers the tiles with the list for saving instrument states
#****************************************************************************
namespace eval ::SavedStates  \
{
   variable tiles   

   lappend tiles Audio_AnalyzerUI
}

namespace eval Audio_Anlz_MarkersUI \
{
  # Global variables
  variable  marker_list  { 0  1  2  3  4  5 }; # List of logical Marker numbers

  variable  marker_nums  { 1  2  3  4  5  6 }; # Used for Marker names and
                                               # Difference and Cable Fault
                                               # menus

  array set marker_color_list  { 0  "gold"   \
                                 1  "green"  \
                                 2  "cyan"   \
                                 3  "tan"    \
                                 4  "red"    \
                                 5  "gray70" }
                                 
  set Marker_orient { HORIZ LIVE AVG REF "PK HI" "PK LO" "LIM HI" "LIM LO" "MATH" }
                                 
  variable  StateUI
  
  # Configure all Marker defaults
  foreach { num } $marker_list \
  {
    set StateUI(marker,$num,color)       $marker_color_list($num)
    set StateUI(marker,$num,orient)      "LIVE"
    set StateUI(marker,$num,mag)         0.00
    set StateUI(marker,$num,temp_state)  "OFF"
    set StateUI(marker,$num,spin_x)      0
    set StateUI(marker,$num,mouse_x)     0
    set StateUI(marker,$num,spin_y)      0
    set StateUI(marker,$num,mouse_y)     0
    set StateUI(marker,$num,var) ::Sinad::State(marker,0,live)
    set StateUI(marker,$num,units) dB
  }

  set StateUI(marker,diff1,1)      1
  set StateUI(marker,diff1,2)      1
  set StateUI(marker,diff1,db)     0.00
  set StateUI(marker,diff1,freq)   0

  set StateUI(marker,diff2,1)      1
  set StateUI(marker,diff2,2)      1
  set StateUI(marker,diff2,db)     0.00
  set StateUI(marker,diff2,freq)   0

  set StateUI(diff_units,1)  "dB"
  set StateUI(diff_units,2)  "dB"
  


  #***************************************************************************
  #
  #  Marker procedures
  #
  #***************************************************************************
  #***************************************************************************
  #
  #        Name: config_marker
  #
  #  Parameters: tile_path - tile path of the widget
  #
  # Description: Displays the configuration toplevel for the markers.
  #
  #***************************************************************************
  proc config_marker { tile_path } \
  {
    #puts "config_marker:  $tile_path"
    
    variable  marker_list
    variable  marker_color_list
    
    if { ![IFR_options  is_installed  Audio_Analyzer] } \
    {
      return
    }

    set top_level  [winfo toplevel $tile_path]
   
    set tile_path  [name_child  $top_level  audio_anlz_marker_cfg]

    if { ![winfo exists $tile_path] } \
    {
      set tile_path  [Screens::create_toplevel_tile  $tile_path  AUDIO_ANLZ_MARKER_CFG]
    }
    
    # Bind to the marker configuration toplevel
    bind $tile_path <KeyRelease-F8>  "destroy [winfo toplevel $tile_path]"
    
    bind $tile_path <<Destroy>>  "::Audio_Anlz_MarkersUI::destroy_marker_config  $tile_path"


    # Set up the various marker configuration toplevel fields
    foreach { num } $marker_list \
    {
      # Configure the color of each marker label
      ${tile_path}.lbl_mark${num}  config  -bg $marker_color_list($num)
      
      # Bind each marker frequency entry to disable then re-enable the
      # marker when the frequency is being set
      bind  ${tile_path}.ent_mark${num}_freq  <<EditOpen>>   \
                         "::Audio_Anlz_MarkersUI::set_marker_state_off  $num"

      bind  ${tile_path}.ent_mark${num}_freq  <<EditClose>>  \
                         "::Audio_Anlz_MarkersUI::restore_marker_state  $num"
                         
      OnChange hook ::Audio_Anlz_MarkersUI::StateUI(marker,$num,orient) \
                     "Audio_Anlz_MarkersUI::update_config_fields  $tile_path  $num"
      OnChange hook ::Audio_Anlz_MarkersUI::StateUI(marker,$num,var) \
                     "Audio_Anlz_MarkersUI::update_config_fields  $tile_path  $num"
                     
      update_config_fields  $tile_path  $num
    }
    
    OnChange hook ::Audio_Anlz_MarkersUI::StateUI(diff_units,1) \
                     "Audio_Anlz_MarkersUI::update_diff_units  $tile_path  1"

    OnChange hook ::Audio_Anlz_MarkersUI::StateUI(diff_units,2)  \
                     "Audio_Anlz_MarkersUI::update_diff_units  $tile_path  2"

    after idle  "safe_focus_force ${tile_path}.btn_return"
    
    return
  }


  #***************************************************************************
  #
  #        Name: destroy_marker_config
  #
  #  Parameters: tile_path - tile path of the invoking widget 
  #
  # Description: Performs a clean up when the marker config toplevel is 
  #              destroyed
  #
  #***************************************************************************
  proc destroy_marker_config { tile_path } \
  {
    #puts "destroy_marker_config:  $tile_path"
    
    variable  marker_list
    
    foreach { num } $marker_list \
    {
      OnChange unhook ::Audio_Anlz_MarkersUI::StateUI(marker,$num,orient)  \
                     "Audio_Anlz_MarkersUI::update_config_fields  $tile_path  $num"
      OnChange unhook ::Audio_Anlz_MarkersUI::StateUI(marker,$num,var) \
                     "Audio_Anlz_MarkersUI::update_config_fields  $tile_path  $num"
    }

    OnChange unhook ::Audio_Anlz_MarkersUI::StateUI(diff_units,1)  \
                     "Audio_Anlz_MarkersUI::update_diff_units  $tile_path  1"

    OnChange unhook ::Audio_Anlz_MarkersUI::StateUI(diff_units,2)  \
                     "Audio_Anlz_MarkersUI::update_diff_units  $tile_path  2"
    
    return
  }


  #***************************************************************************
  #
  #        Name: get_tile
  #
  #  Parameters: any child widget of the tile
  #
  # Description: Get the name of the tile's parent frame
  #
  #***************************************************************************
  proc get_tile { widget } \
  {
    while { 1 } \
    {
      switch -- [winfo class $widget] \
      {
        "QTR_AUDIO_ANLZ"         -
        "FULL_AUDIO_ANLZ"        -
        "AUDIO_ANLZ_MARKER_CFG"  { return  $widget }
      }
        
      set widget [winfo parent $widget]
    }
  }
  
  
  #***************************************************************************
  #
  #        Name: set_marker_state_off
  #
  #  Parameters: num   - marker number
  #
  # Description: Sets the state of the marker to OFF to allow editting of the
  #              frequency value
  #
  #***************************************************************************
  proc set_marker_state_off { num } \
  {
    #puts "set_marker_state_off:  $num"
    
    variable  StateUI
    
    # Save the present state of the marker
    set StateUI(marker,$num,temp_state)  $::Sinad::State(marker,$num,state)
    
    # If the marker is ON, turn it OFF for editing
    if { $::Sinad::State(marker,$num,state) == "ON" } \
    {
      toggle_marker_state  $num
    }
    
    return
  }


  #***************************************************************************
  #
  #        Name: restore_marker_state
  #
  #  Parameters: num   - marker number
  #
  # Description: Restores the state of the marker after editting of the
  #              frequency value
  #
  #***************************************************************************
  proc restore_marker_state { num } \
  {
    #puts "restore_marker_state:  $num"
    
    variable  StateUI
    
    # If the previous state was ON, toggle the marker state back to ON
    if { $StateUI(marker,$num,temp_state) == "ON" } \
    {
      toggle_marker_state  $num
    }
      
    return
  }


  #***************************************************************************
  #
  #        Name: toggle_marker_state
  #
  #  Parameters: num - marker number
  #
  # Description: Toggles the state of the marker ON and OFF
  #
  #***************************************************************************
  proc toggle_marker_state { num } \
  {
    #puts "toggle_marker_state:  $num"
    
    variable  StateUI
    
    # Get the marker bit pattern
    switch -- $num \
    {
      0   { set enable  128   }
      1   { set enable  256   }
      2   { set enable  512   }
      3   { set enable  1024  }
      4   { set enable  2048  }
      5   { set enable  4096  }
      6   { set enable  8192  }
      7   { set enable  16384 }
      8   { set enable  32768 }
      9   { set enable  65536 }
    }
            
    # Get the present Sinad engine enable state
    set sinad_enable  [Sinad  en]
      
    if { $::Sinad::State(marker,$num,state) == "ON" } \
    {
      # Turn OFF the selected marker 
      set enable  [expr $sinad_enable & ~$enable]
    } \
    else \
    {
      # Turn ON the selected marker 
      set enable  [expr $sinad_enable | $enable]
    }
    
    # Set the the Sinad engine enable state
    Sinad  en  $enable

    return
  }


  #***************************************************************************
  #
  #        Name: set_marker_freq
  #
  #  Parameters: num - marker number
  #
  # Description: Sets the frequency of the marker (vertical orientation)
  #
  #***************************************************************************
  proc set_marker_freq { num } \
  {
    #puts "set_marker_freq:  $num"
    
    variable  StateUI
    
    if { $StateUI(marker,$num,orient) != "HORIZ" } \
    { 
      Sinad  marker  $num  $::Sinad::State(marker,$num,freq)
    }
  
    return
  }
  
  #***************************************************************************
  #
  #        Name: update_config_fields
  #
  #  Parameters: tile_path - tile path of the invoking widget
  #              num       - marker number
  #
  # Description: Updates the display for changes in the Marker orientation.
  #              For updating the fields on the Marker configuration 
  #              toplevel.
  #
  #***************************************************************************
  proc update_config_fields { tile_path  num  args } \
  {
    #puts "update_config_fields:  $tile_path,  $num,  $args"
    
    variable  StateUI
    
    set tile_path  [get_tile  $tile_path]
    
    # Update fields for the Marker configuration toplevel only
    if { [winfo class  $tile_path] != "AUDIO_ANLZ_MARKER_CFG" } \
    {
      return
    }
    
    # Remove the Frequency entry and label fields and the Magnitude
    # entry and label fields
    grid remove  ${tile_path}.ent_mark${num}_freq
    grid remove  ${tile_path}.lbl_mark${num}_freq_val
    grid remove  ${tile_path}.ent_mark${num}_mag
    grid remove  ${tile_path}.lbl_mark${num}_mag_val
    
    switch -- $StateUI(marker,$num,orient) \
    {
     
      
      "HORIZ"  { # Display the Frequency label field and the Magnitude entry field
                 Screens::position_widget  ${tile_path}.lbl_mark${num}_freq_val
                 Screens::position_widget  ${tile_path}.ent_mark${num}_mag }
      default   { # Display the Frequency entry field and the Magnitude label field
                 Screens::position_widget  ${tile_path}.ent_mark${num}_freq
                 Screens::position_widget  ${tile_path}.lbl_mark${num}_mag_val }
    }
    ${tile_path}.lbl_mark${num}_mag_val config -textvar $StateUI(marker,$num,var)
    
    return
  }


  #***************************************************************************
  #
  #        Name: update_marker_on_off
  #
  #  Parameters: widget - invoking widget path
  #              num    - marker number
  #
  # Description: Displays a marker on the Analyzer widget, or removes it from 
  #              the display
  #
  #***************************************************************************
  proc update_marker_on_off { widget  num } \
  {
    #puts "update_marker_on_off:  $widget,  $num"
    
    variable  StateUI
    
    set tile_path  [get_tile  $widget]
    
    if { ![winfo exists ${tile_path}.anlz] } \
    {
      return
    }
    
    set trace_height  [${tile_path}.anlz cget -height]
    set trace_width   [${tile_path}.anlz cget -width]
    
    switch -- $StateUI(marker,$num,orient) \
    {
       "HORIZ" \
       {
          # Horizontally oriented markers
          set value  ::Audio_Anlz_MarkersUI::StateUI(marker,$num,mag)
          set units  "dB"
       }
       "LIVE" \
       {
          set value  ::Sinad::State(marker,$num,live)
          set units  "dB"
       }
       "AVG" \
       {
          set value  ::Sinad::State(marker,$num,avg)
          set units  "dB"
       }
       "REF" \
       {
          set value  ::Sinad::State(marker,$num,ref)
          set units  "dB"
       }
       "PK HI" \
       {
          set value  ::Sinad::State(marker,$num,ph)
          set units  "dB"
       }
       "PK LO" \
       {
          set value  ::Sinad::State(marker,$num,pl)
          set units  "dB"
       }
       "LIM HI" \
       {
          set value  ::Sinad::State(marker,$num,ul)
          set units  "dB"
       }
       "LIM LO" \
       {
          set value  ::Sinad::State(marker,$num,ll)
          set units  "dB"
       }
       "MATH" \
       {
          set value  ::Sinad::State(marker,$num,math)
          set units  "dB"
       }
       
       default \
       {
          set StateUI(marker,$num,orient) "LIVE"
          set value  ::Sinad::State(marker,$num,live)
          set units  "dB"
       }
    }
    set StateUI(marker,$num,units) $units
    set StateUI(marker,$num,var) $value


    set fg     $StateUI(marker,$num,color)
    set bg    [${tile_path}.anlz cget -bg]
    set anlz   ${tile_path}.anlz
    set marker $anlz.marker$num
    set blk    ${marker}blk
    set info   ${marker}info
    set ref_num  [expr  $num + 1]
    destroy $marker
    destroy $blk
    destroy $info
    
    canvas $marker  -bg $fg
    canvas $blk     -bg $fg -takefocus  1 -highlightthickness  3 -highlightbackground  blue
    frame $info     -bg $bg -borderwidth 3
    label $info.mag    -justify right -fg $fg -bg $bg -font analyzer_tiny -width 0 -textvar $value
    label $info.munits -fg $fg -bg $bg -font analyzer_tiny -width 0 -textvar ::Audio_Anlz_MarkersUI::StateUI(marker,$num,units)
    label $info.freq   -justify right -fg $fg -bg $bg -font analyzer_tiny -width 0 -textvar ::Sinad::State(marker,$num,freq)
    label $info.funits -fg $fg -bg $bg -font analyzer_tiny -width 0 -text Hz
    label $info.name   -fg $fg -bg $bg -font analyzer_tiny -width 0 
      
    grid columnconfigure $info 0 -weight 1
    grid columnconfigure $info 1 -weight 1
    # Vertically oriented markers
    if { $StateUI(marker,$num,orient) != "HORIZ" } \
    { 
      # Marker line
      $marker config -height $trace_height -width 3
    
      # Create bindings
      bind $marker <ButtonPress-1> "Audio_Anlz_MarkersUI::set_marker_color  %W  $num  white"
    
      bind $marker <ButtonRelease-1>                    \
                  "Audio_Anlz_MarkersUI::set_marker_color  %W  $num  normal; \
                   Audio_Anlz_MarkersUI::set_marker_x  %W  $num  %X"

      # Marker block - to help the user click on the marker for cursor movement
      $blk configure -height 10 -width  23
      
      # Create bindings
      bind $blk <ButtonPress-1> "Audio_Anlz_MarkersUI::set_marker_color  %W  $num  white"
    
      bind $blk <ButtonRelease-1>                \
                  "Audio_Anlz_MarkersUI::set_marker_color  %W  $num  normal; \
                   Audio_Anlz_MarkersUI::set_marker_x  %W  $num  %X"
                              
      bind $blk <Return> "Audio_Anlz_MarkersUI::set_marker_spin_move  %W  $num"

      add_bindtag  $blk  Cursor

      # Marker info
      $info.name configure -text "MRKR${ref_num} - $StateUI(marker,$num,orient)"
      grid $info.name -columnspan 2 -sticky news
      grid $info.freq $info.funits -sticky news
      grid $info.mag  $info.munits -sticky news
      
      set cmd "::Audio_Anlz_MarkersUI::move_marker_x_core $widget $num"
      set var ::Audio_Anlz_MarkersUI::StateUI(marker,$num,spin_x)
      OnChange hook $var $cmd
      bind $marker <Destroy> [list OnChange unhook $var $cmd]
    } \
    else \
    { 
      # Marker line
      $marker config -height 3 -width $trace_width
    
      # Create bindings
      bind ${tile_path}.anlz.marker$num <ButtonPress-1>  \
                     "Audio_Anlz_MarkersUI::set_marker_color  %W  $num  white"
    
      bind ${tile_path}.anlz.marker$num <ButtonRelease-1>                    \
                  "Audio_Anlz_MarkersUI::set_marker_color  %W  $num  normal; \
                   Audio_Anlz_MarkersUI::set_marker_y  %W  $num  %Y"

      # Marker block - to help the user click on the marker for cursor movement
      $blk config -height 23 -width 10
    
      # Create bindings
      bind $blk <ButtonPress-1> "Audio_Anlz_MarkersUI::set_marker_color  %W  $num  white"
    
      bind $blk <ButtonRelease-1>                \
                  "Audio_Anlz_MarkersUI::set_marker_color  %W  $num  normal; \
                   Audio_Anlz_MarkersUI::set_marker_y  %W  $num  %Y"
                              
      bind $blk <Return>  \
                        "Audio_Anlz_MarkersUI::set_marker_spin_move  %W  $num"

      add_bindtag  $blk  Cursor

      # Marker name
      $info.name configure -fg $fg -bg $bg -font analyzer_tiny -width 0 -text "MRKR${ref_num}"
      grid $info.name  -columnspan 2 -sticky news
      grid $info.mag  $info.munits -sticky news

    }

    # Call update to 'place' or 'place forget' the marker
    update_marker_position  $tile_path  $num
  }


  #***************************************************************************
  #
  #        Name: set_marker_x
  #
  #  Parameters: widget   - tile path of the widget
  #              num      - marker number
  #              x_coord  - x coordinate of the cursor
  #              x_offset - offset used to compensate for the marker block 
  #                         area
  #
  # Description: Called when the mouse is used to move the marker or
  #              the spinner knob is turned while the marker has focus.
  #              Adjusts the marker position for the new mouse cursor 
  #              position relative to the current marker position.
  #
  #***************************************************************************
  proc move_marker_x { widget  num  x_offset } \
  {
    variable  StateUI
    
    incr StateUI(marker,$num,spin_x)  $x_offset
  }
  
  proc move_marker_x_core { widget  num  } \
  {
    variable  StateUI
  
    set x  $StateUI(marker,$num,spin_x)
  
    set StateUI(marker,$num,spin_x)  0
  
    set_marker_x $widget  $num  {}  $x
  }

  
  proc update_marker_x { widget  num  x_coord } \
  {
    variable  StateUI
    
    set StateUI(marker,$num,mouse_x) $x_coord
    
    set cmd "::Audio_Anlz_MarkersUI::update_marker_x_core $widget $num"
    
    after cancel  $cmd
    after idle    $cmd
  }
  
  proc update_marker_x_core { widget  num  } \
  {
    variable  StateUI
    
    set_marker_x $widget $num $StateUI(marker,$num,mouse_x)
  }
  
  proc set_marker_x { widget  num  x_coord  { x_offset  0 }} \
  {
   
    variable  StateUI
    
    set tile_path  [get_tile  $widget]

    if { ![winfo exists ${tile_path}.anlz] } \
    {
      return
    }
    
    set root_x  [winfo rootx ${tile_path}.anlz]
 
    # Get the dimensions of the trace widget
    set trace_width   [${tile_path}.anlz cget -width]

    if { $x_coord == "" } \
    {
      if { $x_offset == 0 } \
      {
        return
      }

      set x_coord  [expr int($trace_width * $::Sinad::State(marker,$num,pos) + $root_x + 0.5)]
    }

    # Calculate the X position delta
    set x_m  [expr $x_coord + $x_offset - $root_x] 
    
    # Check the limit of the additional value
    if { $x_m < 0 } \
    {
      set x_m  0
    }
    
    if { $x_m > $trace_width } \
    {
      set x_m  $trace_width
    }
    
    set x_m  [expr double($x_m) / $trace_width]
    
    # Reposition marker
    Sinad marker $num  0  $x_m -nowait
  }


  #***************************************************************************
  #
  #        Name: set_marker_y
  #
  #  Parameters: widget   - tile path of the widget
  #              num      - marker number
  #              y_coord  - y coordinate of the cursor
  #              y_offset - offset used to compensate for the marker block 
  #                         area
  #
  # Description: Adjusts the marker position for the new mouse cursor 
  #              position relative to the current marker position
  #
  #***************************************************************************
  proc set_marker_y { widget  num  y_coord  { y_offset  0 } } \
  {
    #puts "set_marker_y:  $widget,  $num,  $y_coord,  $y_offset"
    
    variable  StateUI
    
    set tile_path  [get_tile  $widget]

    if { ![winfo exists ${tile_path}.anlz] } \
    {
      return
    }
    
    set root_y  [winfo rooty ${tile_path}.anlz]

    # Get the dimensions of the trace widget
    set trace_height  [${tile_path}.anlz cget -height]

    if {$y_coord == "" } \
    {
       set y_coord  [Sinad::db_to_Y  ${tile_path}.anlz  $StateUI(marker,$num,mag)]
       
       set y_coord [expr  int($y_coord + $root_y)]
    }

    # Calculate the Y position delta
    set y_m  [expr $y_coord + $y_offset - $root_y]
    
    # Check the limit of the additional value
    if { $y_m < 0 } \
    {
      set y_m  0
    }
    
    if { $y_m > $trace_height } \
    {
      set y_m  $trace_height
    }

    # Get the level of the Y position
    set db_m  [Sinad::Y_to_db  ${tile_path}.anlz  $y_m]

    # Set the marker to the new level
    set StateUI(marker,$num,mag)  [format  "%.2f"  $db_m]
    
    update_marker_position  $widget  $num
  }


  #***************************************************************************
  #
  #        Name: update_marker_position
  #
  #  Parameters: widget  - tile path of the widget
  #              num     - marker number
  #
  # Description: Moves the marker
  #
  #***************************************************************************
  proc update_marker_position { widget  num } \
  {
    #puts "update_marker_position:  $widget,  $num"
    
    variable  StateUI
    
    set tile_path  [get_tile  $widget]

    if { ![winfo exists ${tile_path}.anlz.marker$num] } \
    {
      return
    }
    set anlz   ${tile_path}.anlz
    set marker $anlz.marker$num
    set blk    ${marker}blk
    set info   ${marker}info
    
    if { $::Sinad::State(marker,$num,state) == "OFF" } \
    {
      place forget $marker
      place forget $blk
      place forget $info
      return
    }
    # Get the dimensions of the trace widget
    set trace_height  [${tile_path}.anlz cget -height]
    set trace_width   [${tile_path}.anlz cget -width]
    # get the width of the info frame
    set info_width   [winfo reqwidth $info]
    set info_height  [winfo reqheight $info]
    set block_width  [winfo reqwidth $blk]
    set block_height [winfo reqheight $blk]
    set info_rel_width  [expr $info_width/$trace_width]
    set info_rel_height [expr $info_height/$trace_height]
    # Vertically oriented markers
    if { $StateUI(marker,$num,orient) != "HORIZ" } \
    {
        set pos  $::Sinad::State(marker,$num,pos)

        # position the name block on the left if there is not enough space on right
        if { $pos >= .5} \
        {
          set marker_name_offset  [expr -$info_width-3]
        } \
        else \
        {
          set marker_name_offset 3
        }
        set y [expr $num * $info_height + $block_height]
        # keep it on display
        if {$y > $trace_height} \
        {
           set y [expr $trace_height - $info_height]
        }
       
        # Place the marker line
        place $marker  -relx $pos -x -1 -y 0
       
        # Place the marker block
        place $blk  -relx $pos -x [expr -$block_width/2] -y 0
       
        # Place the marker info
        place $info -relx $pos -x $marker_name_offset -y $y
      } \
      else \
      {
        # Get the y position of the present marker magnitude
        set db_m  $StateUI(marker,$num,mag)
        set y [expr ($Sinad::State(maxy)-$db_m)/$Sinad::State(rangey)]
               
        # Move the 'name' below the marker line if the marker is
        # closer than 30 pixels to the top of the Audio Analyzer 
        # widget, else move it above the marker line
        if { $y <= .5} \
        {
          set marker_name_offset [expr $info_height+3]
        } \
        else \
        {
          set marker_name_offset [expr -$info_height-3]
        }
        set x [expr $num * $info_width + $block_width]
        # keep it on display
        if {$x > $trace_width} \
        {
           set x [expr $trace_width - $info_width]
        }
        
        # Place the marker components
        place $marker  -x 0 -rely $y -y -1
       
        place $blk  -x 0 -rely $y -y [expr -$block_height/2]
       
        place $info -x $x -rely $y -y $marker_name_offset
       }
  }

  #***************************************************************************
  #
  #        Name: set_marker_spin_move
  #
  #  Parameters: widget - tile path of the Marker block area
  #              num    - marker number
  #
  # Description: Changes the Marker block color, and takes Spin knob events
  #              to move the marker
  #
  #***************************************************************************
  proc set_marker_spin_move { widget  num } \
  {
    #puts "set_marker_spin_move:  $widget,  $num,  $orient"
    
    variable  StateUI
    
    set tile_path  [get_tile  $widget]
    set marker ${tile_path}.anlz.marker$num
    set marker_blk  ${marker}blk

    set present_color  [$marker_blk cget  -bg]
    
    if { $present_color == $StateUI(marker,$num,color) } \
    {
      set_marker_color  $tile_path  $num  white
      
      # Vertically oriented markers
      if { $StateUI(marker,$num,orient) != "HORIZ" } \
      {
        # Bind the <Right> event to increment the Marker X position by 2  (includes <<Spin_up>>)
        bind  $marker_blk  <Right>  "Audio_Anlz_MarkersUI::move_marker_x  $marker_blk  $num 2; break"
                        
        # Bind the <Left> event to decrement the Marker X position by 2  (includes <<Spin_down>>)
        bind  $marker_blk  <Left>  "Audio_Anlz_MarkersUI::move_marker_x  $marker_blk  $num -2; break"
      } \
      else \
      {
        # Horizontally oriented markers
        # Bind the <Right> event to decrement the Marker Y position by 2  (includes <<Spin_up>>)
        bind  $marker_blk  <Right>  "Audio_Anlz_MarkersUI::set_marker_y  $marker_blk  $num {} -2; break"
                        
        # Bind the <Left> event to increment the Marker Y position by 2  (includes <<Spin_down>>)
        bind  $marker_blk  <Left>  "Audio_Anlz_MarkersUI::set_marker_y  $marker_blk  $num {} 2; break"
      }
    } \
    else \
    {
      set_marker_color  $tile_path  $num  normal

      # Unbind the <Right> and <Left> events from the Marker block
      bind  $marker_blk  <Right>  {}
      bind  $marker_blk  <Left>   {}
    }
        
    return
  }
  
  
  #***************************************************************************
  #
  #        Name: set_marker_color
  #
  #  Parameters: widget - tile path of the Marker block area
  #              num    - marker number
  #              mode   - mode of the color change
  #                       white
  #                       normal - StateUI color value
  #
  # Description: Changes the Marker color
  #
  #***************************************************************************
  proc set_marker_color { widget  num  mode } \
  {
    #puts "set_marker_color:  $widget,  $num,  $mode"
    
    variable  StateUI

    set tile_path  [get_tile  $widget]
    
    if { ![winfo exists ${tile_path}.anlz.marker$num] } \
    {
      return
    }
    
    set marker       ${tile_path}.anlz.marker$num
    set marker_blk   ${marker}blk
    # Change the marker only if it is ON
    if { $::Sinad::State(marker,$num,state) == "ON" } \
    {
      # Vertically oriented markers
      if { $StateUI(marker,$num,orient) != "HORIZ" } \
      {
        switch -- $mode \
        {
          "white"    { $marker      config  -bg  white
                       $marker_blk  config  -bg  white
                       
                       focus $marker_blk
                       
                       bind $marker_blk  <Motion>  "Audio_Anlz_MarkersUI::update_marker_x  %W  $num  %X"
                       bind $marker      <Motion>  "Audio_Anlz_MarkersUI::update_marker_x  %W  $num  %X"
                     }
                     
          "normal"   { $marker      config  -bg  $StateUI(marker,$num,color)
                       $marker_blk  config  -bg  $StateUI(marker,$num,color)
                                                 
                       # Call the 'rebuildCursor' procedure to remap the
                       # cursor focus.
                       # This is done in case one Marker is mapped directly
                       # over another.
                       # When the covering Marker is moved, the Marker below
                       # will be able to get focus.
                       Cursor::rebuildControl  $tile_path  MAP
                       
                       bind $marker_blk  <Motion>  {}
                       bind $marker      <Motion>  {}
                     }
        }
      } \
      else \
      {
        switch -- $mode \
        {
          "white"    { $marker      config  -bg  white
                       $marker_blk  config  -bg  white
     
                       focus $marker_blk
                       
                       bind $marker_blk  <Motion>  "Audio_Anlz_MarkersUI::set_marker_y  %W  $num  %Y"
                       bind $marker      <Motion>  "Audio_Anlz_MarkersUI::set_marker_y  %W  $num  %Y"
                     }
                     
          "normal"   { $marker      config  -bg  $StateUI(marker,$num,color)
                       $marker_blk  config  -bg  $StateUI(marker,$num,color)
                                                 
                       # Call the 'rebuildCursor' procedure to remap the
                       # cursor focus.
                       # This is done in case one Marker is mapped directly
                       # over another.
                       # When the covering Marker is moved, the Marker below
                       # will be able to get focus.
                       Cursor::rebuildControl  $tile_path  MAP
                       
                       bind $marker_blk  <Motion>  {}
                       bind $marker      <Motion>  {}
                     }
        }
      }
    }

    return
  }


  #***************************************************************************
  #
  #        Name: toggle_diff_units
  #
  #  Parameters: diff - index of difference field
  #
  # Description: Toggles the units of the difference readings.
  #
  #***************************************************************************
  proc toggle_diff_units { diff } \
  {
    #puts "toggle_diff_units:  $diff"
    
    variable  StateUI
    
    if { $StateUI(diff_units,$diff) == "dB" } \
    {
      set StateUI(diff_units,$diff)  "Hz"
    } \
    else \
    {
      set StateUI(diff_units,$diff)  "dB"
    }
    
    return
  }


  #***************************************************************************
  #
  #        Name: update_diff_units
  #
  #  Parameters: widget - tile path of the invoking widget
  #              diff   - index of difference field
  #
  # Description: Update the display to the present difference units
  #
  #***************************************************************************
  proc update_diff_units { widget  diff } \
  {

    #puts "update_diff_units:  $widget,  $diff"
    
    variable  StateUI
    
    set tile_path  [get_tile  $widget]
    
    switch -- $diff \
    {
      1  { grid remove  ${tile_path}.lbl_diff1_db
           grid remove  ${tile_path}.lbl_diff1_freq }
    
      2  { grid remove  ${tile_path}.lbl_diff2_db
           grid remove  ${tile_path}.lbl_diff2_freq }
    }
    
    if { $StateUI(diff_units,$diff) == "dB" } \
    {
      switch -- $diff \
      {
        1  { Screens::position_widget  ${tile_path}.lbl_diff1_db }
      
        2  { Screens::position_widget  ${tile_path}.lbl_diff2_db }
      }
    } \
    elseif { $StateUI(diff_units,$diff) == "Hz" } \
    {
      switch -- $diff \
      {
        1  { Screens::position_widget  ${tile_path}.lbl_diff1_freq }
      
        2  { Screens::position_widget  ${tile_path}.lbl_diff2_freq }
      }
    }
    
    return
  }


  #***************************************************************************
  #
  #        Name: update_marker_diff
  #
  #  Parameters: 
  #
  # Description: Displays between two markers that are selected
  #
  #***************************************************************************
  proc update_marker_diff {  } \
  {
     set cmd [namespace code update_marker_diff_core]
     after cancel $cmd
     after idle $cmd
  }
  proc update_marker_diff_core { } \
  {
    #puts "update_marker_diff"
    
    variable  StateUI

    # First difference
    # Subtract 1 to get the actual marker number from the menu selection
    set num_1  [expr  $StateUI(marker,diff1,1) - 1]
    set num_2  [expr  $StateUI(marker,diff1,2) - 1]
    set num_3  [expr  $StateUI(marker,diff2,1) - 1]
    set num_4  [expr  $StateUI(marker,diff2,2) - 1]

    upvar 0 $StateUI(marker,$num_1,var) v1
    upvar 0 $StateUI(marker,$num_2,var) v2
    upvar 0 $StateUI(marker,$num_3,var) v3
    upvar 0 $StateUI(marker,$num_4,var) v4

    set result_db  [format "%.2f" [expr $v1 - $v2]]
    set result_freq  [expr $::Sinad::State(marker,$num_1,freq) - $::Sinad::State(marker,$num_2,freq)]

    set StateUI(marker,diff1,db)    $result_db
    set StateUI(marker,diff1,freq)  [format "%.3f"  $result_freq]


    # Second difference
    set result_db  [format "%.2f" [expr $v3 - $v4]]
    set result_freq  [expr $::Sinad::State(marker,$num_1,freq) - $::Sinad::State(marker,$num_2,freq)]

    set StateUI(marker,diff2,db)    $result_db
    set StateUI(marker,diff2,freq)  [format "%.3f"  $result_freq]
  }
  Sinad -onchange marker [namespace code update_marker_diff]

  #**************************************************************************
  #
  #        Name: getSaveState
  #
  #  Parameters: none
  #
  # Description: This function returns a list of TCL commands which can be 
  #              used to set the Audio_Anlz_MarkersUI tiles to its current 
  #              state
  #
  #**************************************************************************
  proc getSaveState { } \
  {
    variable  marker_list
    variable  StateUI
 
    set returnList  {}

    # Save the Marker data
    foreach { num } $marker_list \
    {
      append returnList "set ::Audio_Anlz_MarkersUI::StateUI(marker,$num,orient) {$StateUI(marker,$num,orient)}\n"
      append returnList "set ::Audio_Anlz_MarkersUI::StateUI(marker,$num,mag) {$StateUI(marker,$num,mag)}\n"
    }
      
    #puts "returnList:  $returnList"
            
    return [subst $returnList]
  }


  #**************************************************************************
  #
  #        Name: Sync
  #
  #  Parameters: none
  #
  # Description: Write state variables to Audio_Anlz_MarkersUI
  #
  #**************************************************************************
  proc Sync { } \
  {
    eval [getSaveState]
  }

  
};  # End of Audio_Anlz_MarkersUI namespace


#****************************************************************************
# This registers the tiles with the list for saving instrument states
#****************************************************************************
namespace eval ::SavedStates  \
{
   variable tiles   

   lappend tiles Audio_Anlz_MarkersUI
}



namespace eval  Analog_Simul_App \
{
  # Motorola Simulcast Measurement Application
  set StateUI(state)  "OFF"

  set StateUI(trace_data)      ""
  set StateUI(trace_data,max)  ""
  set StateUI(trace_data,min)  ""
  
  set StateUI(upper_limit)  "0.3"
  set StateUI(lower_limit)  "-0.3"

  set StateUI(sinad_enable_present)  ""
  
  
  #############################################################################
  # 
  #   Audio Analyzer Analog Simulcast Application
  # 
  #############################################################################
  
  #***************************************************************************
  #
  #        Name: Map
  #
  #  Parameters: tile
  #
  # Description: 
  #
  #***************************************************************************
  proc Map { tile_path } \
  {
     ::Audio_Analyzer_FullUI::Map_tg $tile_path
  }
  #***************************************************************************
  #
  #        Name: Cleanup
  #
  #  Parameters: tile
  #
  # Description: 
  #
  #***************************************************************************
  proc Cleanup { tile_path } \
  {
     ::Audio_Analyzer_FullUI::Cleanup_tg $tile_path
  }
  #***************************************************************************
  #
  #        Name: Unmap
  #
  #  Parameters: tile
  #
  # Description: 
  #
  #***************************************************************************
  proc Unmap { tile_path } \
  {
      Cleanup $tile_path
  }
  #***************************************************************************
  #
  #        Name: Destroy
  #
  #  Parameters: tile
  #
  # Description: 
  #
  #***************************************************************************
  proc Destroy { tile_path } \
  {
     Cleanup $tile_path
  }
  
  #***************************************************************************
  #
  #        Name: get_tile
  #
  #  Parameters: any child widget of the tile
  #
  # Description: Get the name of the tile's parent frame
  #
  #***************************************************************************
  proc get_tile { widget } \
  {
    while { 1 } \
    {
      switch -- [winfo class $widget] \
      {
        "QTR_AUDIO_ANLZ"        -
        "FULL_AUDIO_ANLZ"       -
        "ANALOG_SIMUL_APP_CFG"  { return $widget}
      }
        
      set widget [winfo parent $widget]
    }
  }


  #***************************************************************************
  #
  #        Name: start
  #
  #  Parameters:
  #
  # Description: Special application designed for Motorola for Analog 
  #              Simulcast measurements
  #
  #***************************************************************************
  proc start { } \
  {
    #puts "start"
    
    variable  StateUI
    
    # Update the display
    update
      
    if { $StateUI(state) == "OFF" } \
    {
      
      # Restore the Sinad Enable state
      if { $StateUI(sinad_enable_present) != "" } \
      {
        Sinad  en  $StateUI(sinad_enable_present)
      }
    } \
    else \
    {
       # Get the present Sinad enable value
      set sinad_enable  [Sinad  en]
      
      # Save the Enable state so that it can be restored
      set StateUI(sinad_enable_present)  $sinad_enable
    
      # Turn OFF the Stored Trace
      set sinad_enable  [expr  $sinad_enable & ~16]
    
      # Turn ON the UPPER LIMIT and LOWER LIMIT traces
      set sinad_enable  [expr  $sinad_enable | 96]
      
      # Set the Sinad enable value
      after idle "Sinad  en  $sinad_enable -nowait"
      
      # Setup Receiver
        # Set the IF Bandwidth to 12.5 kHz
        Receiver  demod  fmwide
      
      # Setup Audio Analyzer
        # Set Input to DEMOD
        Sinad::Input  "DEMOD"
        
        # Set Start freq to 0
        set ::Sinad::State(minx)  0
        
        Audio_AnalyzerUI::set_start_freq
        
        # Set Stop freq to 120
        set ::Sinad::State(maxx)  120
        
        Audio_AnalyzerUI::set_stop_freq
        
        # Turn on the triggering
        Sinad  anlz_trig  on
    }
      
   
    return
  }


  #***************************************************************************
  #
  #        Name: update_start
  #
  #  Parameters: widget - invoking widget tile path
  #
  # Description: Updates the display to changes in the Special application 
  #              designed for Motorola for Analog Simulcast measurements
  #
  #***************************************************************************
  proc update_start { widget } \
  {
    #puts "update_start_core:  $widget"
    
    variable  StateUI
    
    set tile_path  [get_tile  $widget]
    
    if { ![winfo  ismapped  $tile_path] } \
    {
      return
    }
    
    # If the option is disabled, set the state to OFF
    if { ![IFR_options  is_installed  Analog_Simulcast] } \
    {
      if {$StateUI(state) != "OFF" } \
      {
         set StateUI(state)  "OFF"
      }
    }
        
    if { $StateUI(state) == "OFF" } \
    {
      if { [winfo class $tile_path] == "FULL_AUDIO_ANLZ" } \
      {
        set topWin  [winfo toplevel $widget]
        
        # Get the Options menubutton path
        set buttonName  [View::GetOptionMenuName  $topWin]
    
        # Get the Options menu path
        set menuName  ${buttonName}.menu
      
        # Remove the application fields
        grid remove  ${tile_path}.frm_moto_simul_app
      }

      # Update the traces and the status
      Audio_AnalyzerUI::update_trace_state  $tile_path
    } \
    else \
    {
      if { [winfo class $tile_path] == "FULL_AUDIO_ANLZ" } \
      {
        set topWin  [winfo toplevel $widget]
    
        # Get the Options menubutton path
        set buttonName  [View::GetOptionMenuName  $topWin]
    
        # Get the Options menu path
        set menuName  ${buttonName}.menu

        # Display the application fields
        Screens::position_widget  ${tile_path}.frm_moto_simul_app
      }
    }

    return
  }
  

  #***************************************************************************
  #
  #        Name: grab_trace
  #
  #  Parameters: widget - invoking widget tile path
  #
  # Description: Grabs the present trace, and displays the UPPER LIMIT and
  #              LOWER LIMIT traces
  #
  #***************************************************************************
  proc grab_trace { widget } \
  {
    #puts "grab_trace:  $widget"
    
    variable  StateUI
    
    set tile_path  [get_tile  $widget]
    
    # If the AVERAGE trace is the only trace displayed, use it,
    # else use the LIVE trace
    if { [expr  $::Sinad::State(Engine_enabled) & 8] == 8 } \
    {
      set StateUI(trace_data)  [Sinad  trace  tavg]
    } \
    else \
    {
      # Get the current LIVE trace data
      set StateUI(trace_data)  [Sinad  trace  tlive]
    }
    
    # Initialize the max trace data variable
    set StateUI(trace_data,max)  ""
    set StateUI(trace_data,min)  ""
    
    # Add the LIMIT value to the trace data and set the UPPER LIMIT trace
    foreach { value } $StateUI(trace_data) \
    {
      lappend StateUI(trace_data,max)  [expr $value + $StateUI(upper_limit)]
      lappend StateUI(trace_data,min)  [expr $value + $StateUI(lower_limit)]
    }
    
    Sinad  trace  tulim StateUI(trace_data,max)
    Sinad  trace  tllim StateUI(trace_data,min)

    return
  }


  #***************************************************************************
  #
  #        Name: sync_vars
  #
  #  Parameters:
  #
  # Description: Synchronizes the Simulcast App variables to changes in the 
  #              Function Generator variables
  #
  #***************************************************************************
  proc sync_vars { } \
  {
    #puts "sync_vars"
    
    variable  StateUI

    if { $StateUI(state) == "ON" } \
    {
      set StateUI(fgen,level)  $::FGen::State(1,SINC,level)

      set StateUI(fgen,rate)   $::FGen::State(1,SINC,rate)

      set StateUI(fgen,omega)  $::FGen::State(1,SINC,omega)
    }
    
    return
  }

  #**************************************************************************
  #
  #        Name: getSaveState
  #
  #  Parameters: none
  #
  # Description: This function returns a list of TCL commands which can be 
  #              used to set the Analog_Simul_App tiles to its current state
  #
  #**************************************************************************
  proc getSaveState { } \
  {
    variable StateUI
 
    set returnList \
    {
      # Motorola Simulcast Measurement Application
      set ::Analog_Simul_App::StateUI(state)  {$StateUI(state)}

      set ::Analog_Simul_App::StateUI(upper_limit)  {$StateUI(upper_limit)}
      set ::Analog_Simul_App::StateUI(lower_limit)  {$StateUI(lower_limit)}

      set ::Analog_Simul_App::StateUI(fgen,level)  {$StateUI(fgen,level)}
      set ::Analog_Simul_App::StateUI(fgen,rate)   {$StateUI(fgen,rate)}
      set ::Analog_Simul_App::StateUI(fgen,omega)  {$StateUI(fgen,omega)}
    }
        
    return [subst $returnList]
  }


  #**************************************************************************
  #
  #        Name: Sync
  #
  #  Parameters: none
  #
  # Description: Write state variables to Analog_Simul_App
  #
  #**************************************************************************
  proc Sync { } \
  {
    eval [getSaveState]
  }
};  # End of namespace 'Analog_Simul_App'



#****************************************************************************
# This registers the tiles with the list for saving instrument states
#****************************************************************************
namespace eval ::SavedStates  \
{
   variable tiles   

   lappend tiles  Analog_Simul_App
}

#****************************************************************************
# Option Definition: Define the namespace, and set up our HTML option info
#****************************************************************************
namespace eval ::Analog_Simulcast \
{
  variable InfoString

  set temp_date [string trimleft [firmware_date] "Date:"]
  set end_loc [string first "Time:" $temp_date]
  set end_loc [expr $end_loc - 2]
  set temp_date [string range $temp_date 0 $end_loc]
  set InfoString [subst \
  {
         <br>
         (using Audio Analyzer version - $temp_date, [firmware_version])
         <br>
         This option extends the audio analyzer to allow it to check a 
         Simulcast repeater subaudible path.
  }]
}
