#############################################################################
#                                                                           #
# IFR, An Aeroflex Co.       COPYRIGHT (C) 2003        DESIGN ENGINEERING   #
# ------------------------------------------------------------------------- #
#                                                                           #
# FILE        : 4110.BerMtrUI.tcl                                           #
#                                                                           #
# PRODUCT     : FITS                                                        #
#                                                                           #
# AUTHOR      : P. McCormick                                                #
#                                                                           #
# DESCRIPTION : Bit Error Rate Meter User Interface                         #
#                                                                           #
#############################################################################

package require Ber


namespace eval BerMtrUI \
{
  # Global variables
  variable  LongName     "ber_mtr"
  variable  ShortName    "BER Meter"
  variable  Size         Eighth_v
            
  variable  last_focus

  variable  StateUI
  
  set StateUI(reading)           0

  set StateUI(range_selection)   "AR"
  set StateUI(high_range)        ""
  set StateUI(auto_range)        ""
  set StateUI(mid_range)         ""
  set StateUI(low_range)         "0"
  
  set StateUI(ref_mode)          "OFF"
  set StateUI(ref_indicate)      ""
  set StateUI(ref_reading)       0.0
  
  set StateUI(ranges,percent)  { "AR"  "100"  "80"  "60"  "50"  "40"  "20" }
  set StateUI(ranges,ppm)      { "AR"  "100K"  "10K"  "1000"  "100"  "50"  "10" }
  
  set StateUI(max_range,percent)  [llength $StateUI(ranges,percent)]
  
  incr StateUI(max_range,percent)  -1

  set StateUI(max_range,ppm)  [llength $StateUI(ranges,ppm)]
  
  incr StateUI(max_range,ppm)  -1
   

  #***************************************************************************
  #
  #        Name: Create  
  #
  #  Parameters: size - meter size 
  #
  # Description: 
  #
  #***************************************************************************
  proc Create { tile_path  { meter_size  small } } \
  {
    variable LongName
    
    #puts "BerMtrUI::Create:  $tile_path,  $meter_size"

    set top_level  [winfo toplevel $tile_path]
 
    # Call the function 'create_tile' to generate the meter tile
    # from the resource database
    set meter_class  ${meter_size}_BER_MTR
    
    set meter_class  [string toupper $meter_class]
    
    #puts "meter_class:  $meter_class"
    
    if { $meter_size == "small" } \
    {
      set tile_path  [name_child  $top_level  $LongName]

      if { ![winfo exists $tile_path] } \
      {
        set tile_path  [Screens::create_tile  $tile_path  $LongName  $meter_class]
      }  
    }
     
    if { $meter_size == "big" } \
    {
      set tile_path  [name_child  $top_level  big$LongName]
   
      if { ![winfo exists $tile_path] } \
      {
        set tile_path  [Screens::create_toplevel_tile  $tile_path  $meter_class]
      }

      # Bind the 'big meter' to the 'F8' key
      bind $tile_path <KeyRelease-F8>  "destroy $tile_path"
    }

    return $tile_path
  
  }

  #***************************************************************************
  #
  #        Name: Map
  #
  #  Parameters: 
  #
  # Description: Initializes the meter tile
  #
  #***************************************************************************
  proc Map { tile_path  meter_size } \
  {
    #puts "BerMtrUI::Map:  $tile_path,  $meter_size"

    variable StateUI
    
    # Initialize the meter
      # Set the input range
      set_meter_range  $StateUI(range_selection)
      
      set_ref_mode     $StateUI(ref_mode)
      

    # Set up to change the input button/menu when the hardware changes

    trace variable ::Ber::State(ber_raw)  w  "BerMtrUI::Update_display $tile_path"
     

    trace variable ::Ber::State(alerts)  w  "BerMtrUI::update_limits $tile_path"
                                          
    trace variable ::Ber::State(alertHi,per)  w  "BerMtrUI::update_limits $tile_path"
                                          
    trace variable ::Ber::State(alertLo,per)  w  "BerMtrUI::update_limits $tile_path"
                                          
    trace variable ::BerMtrUI::StateUI(high_range)  w  "BerMtrUI::update_limits $tile_path"
    

    update_limits_core $tile_path
    
    update_stale $tile_path
    Update_display_core $tile_path
    
    if { $meter_size == "small" } \
    {
      trace variable ::Ber::State(BellEnabled)  w  "BerMtrUI::update_alarm_small $tile_path"
               
      update_alarm_small_core  $tile_path
      
      trace variable ::Ber::State(pattern_match)  w  "BerMtrUI::update_pattern_small $tile_path"
      
      update_pattern_small_core $tile_path

      trace variable ::Ber::State(units)  w  "BerMtrUI::update_units_small  $tile_path"
      
      update_units_small_core  $tile_path

      trace variable ::Ber::State(peakEnable)  w  "BerMtrUI::update_peak_hold_small $tile_path"

      update_peak_hold_small_core $tile_path

      Ber::QuickStart
    } \
    else \
    {
      trace variable ::Ber::State(BellEnabled)  w  "BerMtrUI::update_alarm_large $tile_path"
               
      update_alarm_large_core  $tile_path
      
      trace variable ::Ber::State(pattern_match)  w  "BerMtrUI::update_pattern_large $tile_path"
      
      update_pattern_large_core $tile_path

      trace variable ::Ber::State(units)  w  "BerMtrUI::update_units_large  $tile_path"
      
      update_units_large_core  $tile_path
      
      # Enable the Pattern Source menubutton
      ${tile_path}.mnu_pattern_source  config  -state normal

      # Set the Pattern Source '-postcommand'
      # Configure the option here so that the 'tile_path' can be included 
      # in the call
      ${tile_path}.mnu_pattern_source.menu  config  \
            -postcommand "BerMtrUI::create_pattern_source_menu  $tile_path"
              
      # Create the menu
      create_pattern_source_menu  $tile_path
    }
  }


  #***************************************************************************
  #
  #        Name: Cleanup
  #
  #  Parameters: 
  #
  # Description: 
  #
  #***************************************************************************
  proc Cleanup { tile_path  meter_size } \
  {
    #puts "BerMtrUI::Cleanup:  $tile_path,  $meter_size"
    
    trace vdelete ::Ber::State(ber_raw)   w  "BerMtrUI::Update_display $tile_path"


    trace vdelete ::Ber::State(alerts)  w  "BerMtrUI::update_limits $tile_path"
                                          
    trace vdelete ::Ber::State(alertHi,per)  w  "BerMtrUI::update_limits $tile_path"
                                          
    trace vdelete ::Ber::State(alertLo,per)  w  "BerMtrUI::update_limits $tile_path"
                                          
    trace vdelete ::BerMtrUI::StateUI(high_range)  w  "BerMtrUI::update_limits $tile_path"



    if { $meter_size == "small" } \
    {
      trace vdelete ::Ber::State(BellEnabled)  w  "BerMtrUI::update_alarm_small $tile_path"

      trace vdelete ::Ber::State(pattern_match)  w  "BerMtrUI::update_pattern_small $tile_path"

      trace vdelete ::Ber::State(units)  w  "BerMtrUI::update_units_small  $tile_path"

      trace vdelete ::Ber::State(peakEnable)  w  "BerMtrUI::update_peak_hold_small $tile_path"
    } \
    else \
    {
      trace vdelete ::Ber::State(BellEnabled)  w  "BerMtrUI::update_alarm_large $tile_path"

      trace vdelete ::Ber::State(pattern_match)  w  "BerMtrUI::update_pattern_large $tile_path"

      trace vdelete ::Ber::State(units)  w  "BerMtrUI::update_units_large  $tile_path"
    }
    
    after cancel "BerMtrUI::Map_AfterIdle $tile_path $meter_size"
    
    after cancel "BerMtrUI::Map_AfterDelay $tile_path $meter_size"

    return
  }
  
  
  #***************************************************************************
  #
  #        Name: Unmap
  #
  #  Parameters: 
  #
  # Description: 
  #
  #***************************************************************************
  proc Unmap { tile_path  meter_size } \
  {
    #puts "BerMtrUI::Unmap:  $tile_path,  $meter_size"
  
    Cleanup  $tile_path  $meter_size
  }


  #***************************************************************************
  #
  #        Name: Destroy
  #
  #  Parameters: 
  #
  # Description: 
  #
  #***************************************************************************
  proc Destroy { tile_path  meter_size } \
  {
    #puts "BerMtrUI::Destroy:  $tile_path,  $meter_size"

    Cleanup  $tile_path  $meter_size
  }


  #***************************************************************************
  #
  #        Name: Register
  #
  #  Parameters: 
  #
  # Description: 
  #
  #***************************************************************************
  proc Register { } \
  {
    #puts "BerMtrUI::Register"
  }
  

  #***************************************************************************
  #
  #        Name: Update_display
  #
  #  Parameters: 
  #
  # Description: Updates the display for a change in the 
  #            : engine 'State(reading)' variable
  #
  #***************************************************************************
  proc Update_display { tile_path  args } \
  {
    #puts "BerMtrUI::Update_display:  $tile_path,  $args"

    set cmd  "::BerMtrUI::Update_display_core  $tile_path"
    
    after cancel  $cmd
    
    after idle  $cmd
  }
  
  
  #***************************************************************************
  #
  #        Name: Update_display_core
  #
  #  Parameters: 
  #
  # Description: Updates the display for a change in the engine 
  #              'State(ber_percent)' or 'State(ber_ppm)' variable
  #
  #***************************************************************************
  proc Update_display_core { tile_path } \
  {
    #puts "BerMtrUI::Update_display_core:  $tile_path"
  
    variable StateUI
       
    if { ![winfo exists $tile_path] } \
    { 
      return 
    }

    if { ![winfo ismapped $tile_path] } \
    { 
      #puts "BER Meter is not mapped"
      
      return 
    }

    # Update the peak hold depending on whether the peak hold is set
      set peak_enable  $::Ber::State(peakEnable)

      if { $peak_enable == 1 } \
      {
        # Update the reading to the peak value
        switch -- $::Ber::State(units) \
        {
          "%"    { set reading_raw   $::Ber::State(ber_percent)

                   set reading_cond  $::Ber::State(peakHi,per)
                   set reading_peak  $::Ber::State(peakHi,per)
                 }

          "ppm"  { set reading_raw   $::Ber::State(ber_ppm)

                   set reading_cond  $::Ber::State(peakHi,ppm)
                   set reading_peak  $::Ber::State(peakHi,ppm)
                 }
        }
        
      }
    
      if { $peak_enable == 0 } \
      {
        # Update the reading to the conditioned reading value
        switch -- $::Ber::State(units) \
        {
          "%"    { set reading_raw   $::Ber::State(ber_percent)
                   set reading_cond  $::Ber::State(ber_percent)
                   set reading_peak  $::Ber::State(ber_percent)
                 }

          "ppm"  { set reading_raw   $::Ber::State(ber_ppm)
                   set reading_cond  $::Ber::State(ber_ppm)
                   set reading_peak  $::Ber::State(ber_ppm)
                 }
        }
      }


      if { $StateUI(ref_mode) == "OFF" } \
      {
        set StateUI(reading)  $reading_cond
      }

      if { $StateUI(ref_mode) == "ON" } \
      {
        set reading_comp  [format "%.2f" $reading_raw]

        set reading_ref   [format "%.2f" $StateUI(ref_reading)]

        set StateUI(reading)  [expr $reading_comp - $reading_ref]
      }
      
      
    # ######################################################################
    # # Changed for the meterbar
    #   return
    # # Changed for the meterbar
    # ######################################################################
      

    # Update the bargraph, range, limits, alarm, and peakhold to the raw reading
      # Update the bargraph
      update_bargraph  $tile_path  $reading_raw

      # Update the range
      update_range  $reading_raw
      
      update_stale $tile_path

      # Update the alarm status
      update_alarm  $tile_path

      # Update the peak hold
      update_peakhold  $tile_path  $reading_peak
  }
  

  #***************************************************************************
  #
  #        Name: get_tile                   
  #
  #  Parameters: a widget
  #
  # Description: given widget within a tile, get the name of the tile
  #
  #***************************************************************************
  proc get_tile { widget } \
  {
    while {1} \
    {
      if { [catch { set class  [winfo class $widget] } result] } \
      {
        return ""
      }
         
      switch -- $class \
      {
        "SMALL_BER_MTR"     -
        "BIG_BER_MTR"       -
        "BER_MTR_ADVANCED"  { return $widget }
      }

      set widget [winfo parent $widget]
    }
    
    return ""
  }


  #***************************************************************************
  #
  #        Name: update_mid_range                     
  #
  #  Parameters: 
  #
  # Description: sets the meter midrange value
  #
  #***************************************************************************
  proc update_mid_range { args } \
  {
     set cmd [namespace code "update_mid_range_core"]
     after cancel $cmd
     after idle $cmd
  }
  proc update_mid_range_core {  } \
  {

    #puts "update_mid_range"

    variable StateUI
    
    set Range  $StateUI(high_range)

    set temp      ""
    set modifier  ""
    set MidRange  0
    
    regexp {([-.0-9]+)}   $Range  MidRange
    regexp {([a-zA-Z]$)}  $Range  modifier
    
     
    set MidRange  [format  "%g"  [expr $MidRange / 2.0]]
    
    set StateUI(mid_range)  ${MidRange}${modifier}
  }
  
  
  #***************************************************************************
  #
  #        Name: get_mult                     
  #
  #  Parameters: 
  #
  # Description: Gets the value of the muliplier
  #
  #***************************************************************************
  proc get_mult { modifier } \
  {
    switch -- $modifier \
    {
      "K" -
      "k" { return 1000 }
    
      "M" -
      "m" { return 1000000 }
  
      "G" -
      "g" { return 1000000000 }
    }
    
    return  1
  }


  #***************************************************************************
  #
  #        Name: get_high_range                     
  #
  #  Parameters: 
  #
  # Description: gets meter range
  #
  #***************************************************************************
  proc get_high_range { } \
  {
    #puts "get_high_range"

    variable StateUI
    
    set range  $StateUI(high_range)
    
    set modifier ""
    set multiplier 1
    
    regexp {([-.0-9]+)}   $range  range_value
    regexp {([a-zA-Z]$)}  $range  modifier
    
    set multiplier  [get_mult  $modifier]

    set high_range  [expr $range_value * $multiplier]
    
    return $high_range
  }
  
  
  #***************************************************************************
  #
  #        Name: get_highest_range
  #
  #  Parameters: 
  #
  # Description: gets the meters highest range string
  #
  #***************************************************************************
  proc get_highest_range { } \
  {
    #puts "get_highest_range"

    variable  StateUI
    
    # Get the range value of index '1' of the menu
    switch -- $::Ber::State(units) \
    {
      "%"    { set high_range  [lindex  $StateUI(ranges,percent)  1] }

      "ppm"  { set high_range  [lindex  $StateUI(ranges,ppm)  1]     }
    }

    return  $high_range
  }


  #***************************************************************************
  #
  #        Name: update_bargraph
  #
  #  Parameters: reading - the new reading
  #
  # Description: updates the bargraph widget of the meter
  #
  #***************************************************************************
  proc update_bargraph { tile_path  reading } \
  {
    #puts "update_bargraph:  $tile_path,  $reading"

    #variable StateUI

    if { $reading == "" } \
    {
      return
    }

    set tile_path  [get_tile  $tile_path]
    
    if { ![winfo exists $tile_path] } \
    { 
      return 
    }

    set canvas_name  ${tile_path}.cnvs_bargraph

    if { ![winfo exists $canvas_name] } \
    { 
      return 
    }
    
    
    # Adjust for negative readings
    set reading  [expr abs($reading)]
    
    # Determine the value of the high range
    set range  [get_high_range]
    
    # Convert the range to the absolute value
    set range  [expr abs($range)]

    if { $reading > $range } \
    {
      set reading  $range
    }

    # Determine the present dimensions of the bargraph frame
    set bargraph_coords  [$canvas_name coords bargraph_frame]
    
    foreach { x1 y1 x2 y2 } $bargraph_coords {}

    set meterbar_height  [expr [expr $y2 - $y1] - 1]
    
    # The value 'meterbar_height' is the max height of the bargraph (in pixels)
    set ratio  [expr $meterbar_height / $range]

    set increase  [expr $reading * $ratio]
    
    # Scale the bargraph by the correct amount
    # Determine the present dimensions
    set bargraph_coords  [$canvas_name coords bargraph]
    
    foreach { x1 y1 x2 y2 } $bargraph_coords {}

    set new_height  [expr $y2 - $increase]
    
    # Display the bargraph for the present reading
    $canvas_name coords bargraph $x1 $new_height $x2 $y2

    return
  }
  
  
  #***************************************************************************
  #
  #        Name: update_range                     
  #
  #  Parameters: reading - the new reading
  #
  # Description: updates the reading widget of the meter
  #
  #***************************************************************************
  proc update_range { reading } \
  {
    #puts "update_range:  $reading"

    variable  StateUI

    if { $reading == "" } \
    {
      return
    }

    # Adjust for negative readings
    set reading  [expr abs($reading)]

    # Determine the value of the high range
    set range  [get_high_range]
    
    # Convert the range to the absolute value
    set range  [expr abs($range)]
    
    set tenth_range  [expr $range / 10.0]
    
    set auto_range  $StateUI(auto_range)
  
    set pres_range  $StateUI(high_range)
  
    # Get the present active index
    set pres_index  0

    switch -- $::Ber::State(units) \
    {
      "%"    { foreach { label_range } $StateUI(ranges,percent) \
               {
                 if { $label_range == $pres_range } \
                 {
                   break
                 }
    
                 incr pres_index
               }
             }

      "ppm"  { foreach { label_range } $StateUI(ranges,ppm) \
               {
                 if { $label_range == $pres_range } \
                 {
                   break
                 }
    
                 incr pres_index
               }
             }
    }
    
    if { $auto_range == "AR" } \
    {
      if { $reading > $range } \
      {
        incr pres_index -1
      } \
      elseif { $reading < $tenth_range } \
      {
        incr pres_index
      } \
      else \
      {
        # No range change. all done
         return
      }
      
      switch -- $::Ber::State(units) \
      {
        "%"    { if { $pres_index < 1 } \
                 {
                   set new_index 1
                 } \
                 elseif { $pres_index > $StateUI(max_range,percent) } \
                 {
                   set new_index  $StateUI(max_range,percent)
                 } \
                 else \
                 {
                   set new_index  $pres_index
                 }
                 
                 set new_value  [lindex $StateUI(ranges,percent)  $new_index]
               }

        "ppm"  { if { $pres_index < 1 } \
                 {
                   set new_index 1
                 } \
                 elseif { $pres_index > $StateUI(max_range,ppm) } \
                 {
                   set new_index  $StateUI(max_range,ppm)
                 } \
                 else \
                 {
                   set new_index  $pres_index
                 }
                 
                 set new_value  [lindex $StateUI(ranges,ppm)  $new_index]
               }
      }
      
      # Don't update the range if it is the same        
      if { $StateUI(high_range) != $new_value } \
      {
        set StateUI(high_range)  $new_value
      }

      set StateUI(auto_range)  "AR"
    }
  }
  
  
  #***************************************************************************
  #
  #        Name: update_peakhold                     
  #
  #  Parameters: peak_reading - the peak reading
  #
  # Description: updates the bargraph peak hold indicator
  #
  #***************************************************************************
  proc update_peakhold { tile_path  peak_reading } \
  {
    #puts "update_peakhold:  $peak_reading" 

    #variable StateUI

    if { $peak_reading == "" } \
    {
      return
    }

    set tile_path  [get_tile  $tile_path]
    
    if { ![winfo exists $tile_path] } \
    { 
      return 
    }

    set canvas_name   ${tile_path}.cnvs_bargraph

    if { ![winfo exists $canvas_name] } \
    {
      return
    }


    set marker_x1  [option get $tile_path marker_x1 {}]
    set marker_x2  [option get $tile_path marker_x2 {}]

    # Adjust for negative readings
    set peak_reading  [expr abs($peak_reading)]
    

    # Delete the old peak hold marker
    $canvas_name  delete  peak_marker 

    # Determine peak hold condition
    if { $::Ber::State(peakEnable) == 1 } \
    {
      # Determine the value of the high range
      set range  [get_high_range]
  
      set range  [expr abs($range)]
      
      # Determine the present dimensions of the bargraph frame
      set bargraph_coords [$canvas_name  coords  bargraph_frame]

      foreach { x1 y1 x2 y2 } $bargraph_coords {}
    
      set marker_y  [expr [expr $y2 - $y1] - 1]
    
      # Determine the ratio of the reading to the scale
      set ratio [expr $marker_y / $range]
        
      # Determine the present dimensions
      set bargraph_coords [$canvas_name coords bargraph]

      foreach { x1 y1 x2 y2 } $bargraph_coords {}
      
      set new_peak_val [expr $peak_reading * $ratio]
  
      # If the reading exceeds the scale of the bargraph,
      # limit the peak hold marker to the upper edge of the bargraph
      if { $peak_reading > $range } \
      {
        set new_peak_val $marker_y
      }
      
      set new_peak  [expr $y2 - $new_peak_val]
    
      set new_peak1  $new_peak
      
      set new_peak2  [expr $new_peak + 2]
     
      $canvas_name create rect $marker_x1 $new_peak1        \
                               $marker_x2 $new_peak2        \
                               -fill white                  \
                               -tags peak_marker
    } 
  }
  
  #***************************************************************************
  #
  #        Name: update_stale
  #
  #  Parameters: tile to update
  #
  # Description: update the meter for stale/not stale
  #
  #***************************************************************************
  proc update_stale { tile_path } \
  {
    variable StateUI
    
    set tile_path  [get_tile  $tile_path]
    
    if { ![winfo exists $tile_path] } \
    { 
      return 
    }

    set canvas_name  ${tile_path}.cnvs_bargraph
    set reading      ${tile_path}.lbl_reading
    
    if {$::Ber::State(status) & 2} \
    {
       $canvas_name itemconfigure bargraph       -stipple gray50
       $canvas_name itemconfigure bargraph_frame -fill DimGrey
    } \
    else \
    {
       $canvas_name itemconfigure bargraph       -stipple {}
       $canvas_name itemconfigure bargraph_frame -fill {}
    }
  }
  
  
  #***************************************************************************
  #
  #        Name: update_alarm                     
  #
  #  Parameters: new_reading - the new reading
  #
  # Description: updates the alarm operation
  #
  #***************************************************************************
  proc update_alarm { tile_path } \
  {
    #variable StateUI

    set tile_path  [get_tile  $tile_path]
    
    if { ![winfo exists $tile_path] } \
    { 
      return 
    }

    set canvas_name  ${tile_path}.cnvs_bargraph
    
    if { ![winfo exists $canvas_name] } \
    { 
      return 
    }
    
    # Determine if Alarm is set
    set alarm  ""

    if { $::Ber::State(status) & 0x200} \
    { 
      set color  red

      # Turn on the audio alarm
      set alarm  hi
    } \
    elseif { $::Ber::State(status) & 0x800 } \
    { 
      set color  blue

      # Turn on the audio alarm
      set alarm  lo
    } \
    else \
    { 
      set color  green

      # Turn off the audio alarm
      set alarm  off
    }
    
    $canvas_name itemconfigure bargraph -fill $color

    if { $::Ber::State(BellEnabled) == 0 } \
    {
      set alarm  off
    }

    switch -- $alarm \
    {
      hi      { }
      
      lo      { }
      
      off     -
      default { }
    }
  }


  #***************************************************************************
  #
  #        Name: update_limits
  #
  #  Parameters:
  #
  # Description: updates the limits of the meter
  #
  #***************************************************************************
  proc update_limits { tile_path  args } \
  {
    set cmd  "BerMtrUI::update_limits_core $tile_path"
    after cancel  $cmd
    after idle  $cmd
  }
  proc update_limits_core { tile_path } \
  {
     update_limits_hi  $tile_path
     update_limits_lo  $tile_path
  }
  
  #***************************************************************************
  #
  #        Name: update_limits_hi
  #
  #  Parameters:
  #
  # Description: updates the upper limits of the meter
  #
  #***************************************************************************
  proc update_limits_hi { tile_path } \
  {
    #puts "update_limits_hi:  $tile_path"

    #variable StateUI
 
    set tile_path  [get_tile  $tile_path]
    
    if { ![winfo exists $tile_path] } \
    { 
      return 
    }

    set meter_class  [winfo class $tile_path]
    
    set canvas_name  ${tile_path}.cnvs_bargraph

    if { ![winfo exists $canvas_name] } \
    { 
      return 
    }

    set line_width  [option get $tile_path line_width {}]


    # Determine the present dimensions of the bargraph frame
    set bargraph_coords  [$canvas_name coords bargraph_frame]
    
    foreach { x1 y1 x2 y2 } $bargraph_coords {}
    
    set meterbar_height  [expr [expr $y2 - $y1] - 1]
    
    # Delete the upper limit marker from the screen
    $canvas_name  delete  high_marker

    $canvas_name  delete  high_value
    

    if { $::Ber::State(alertHi_text) == "ON" } \
    {
      # Get the value of the limit
      switch -- $::Ber::State(units) \
      {
        "%"    { set limit_value  $::Ber::State(alertHi,per)

                 if { $limit_value == "" } \
                 {
                   set limit_value  0.0
                 }
                 
                 set limit_value  [format "%.*f" [expr $::Ber::State(precision)-2] $limit_value]
               }

        "ppm"  { set limit_value  $::Ber::State(alertHi,ppm)

                 if { $limit_value == "" } \
                 {
                   set limit_value  0.0
                 }
                 
                 set limit_value  [format "%.*f" [expr $::Ber::State(precision)-6] $limit_value]
               }
      }
      

      # Determine the value of the high range
      set range  [get_high_range]
    
      # If the value exceeds the range, 
      # return without displaying the marker
      if { $limit_value > $range } \
      {
        return
      }
    
      if { $limit_value < 0.0 } \
      {
        return
      }

      # The value 'meterbar_height' is the max height of the bargraph (in pixels)
      set ratio  [expr $meterbar_height / $range]

      # Use the value of 'limit_value' to set the marker
      set marker_pos  [expr $limit_value * $ratio]
  
      # Scale the bargraph by the correct amount
      # Determine the present dimensions
      set bargraph_coords  [$canvas_name coords bargraph]
    
      set marker_height  [expr $y2 - $marker_pos]
    
      # Display the bargraph for the present reading
      $canvas_name create line 0  $marker_height              \
                                  $line_width $marker_height  \
                                  -width 2                    \
                                  -fill red                   \
                                  -tags high_marker

      if { $meter_class == "BIG_BER_MTR"} \
      {
        # Display the value of the limit next to the marker on the big meter
        $canvas_name create text 0  [expr $marker_height - 8]  \
                                    -text $limit_value         \
                                    -font {-size 12}           \
                                    -anchor w                  \
                                    -fill red                  \
                                    -tags high_value
      }
    
    }
  }
  

  #***************************************************************************
  #
  #        Name: update_limits_lo
  #
  #  Parameters:
  #
  # Description: updates the lower limits of the meter
  #
  #***************************************************************************
  proc update_limits_lo { tile_path } \
  {
    #puts "update_limits_lo:  $tile_path"

    #variable StateUI

    set tile_path  [get_tile  $tile_path]
    
    if { ![winfo exists $tile_path] } \
    { 
      return 
    }

    set meter_class  [winfo class $tile_path]
    
    set canvas_name  ${tile_path}.cnvs_bargraph

    if { ![winfo exists $canvas_name] } \
    { 
      return 
    }

    set line_width  [option get $tile_path line_width {}]


    # Determine the present dimensions of the bargraph frame
    set bargraph_coords  [$canvas_name coords bargraph_frame]
    
    foreach { x1 y1 x2 y2 } $bargraph_coords {}
    
    set meterbar_height  [expr [expr $y2 - $y1] - 1]
    
    # Delete the lower limit marker from the screen
    $canvas_name  delete  low_marker
    
    $canvas_name  delete  low_value

    if { $::Ber::State(alertLo_text) == "ON" } \
    {
      # Get the value of the limit
      switch -- $::Ber::State(units) \
      {
        "%"    { set limit_value  $::Ber::State(alertLo,per)

                 if { $limit_value == "" } \
                 {
                   set limit_value  0.0
                 }
                 
                 set limit_value  [format "%.*f" [expr $::Ber::State(precision)-2] $limit_value]
               }

        "ppm"  { set limit_value  $::Ber::State(alertLo,ppm)

                 if { $limit_value == "" } \
                 {
                   set limit_value  0.0
                 }
                 
                 set limit_value  [format "%.*f" [expr $::Ber::State(precision)-6] $limit_value]
               }
      }
         
         
      # Determine the value of the high range
      set range  [get_high_range]
    
      # If the value exceeds the range, 
      # return without displaying the marker
      if { $limit_value > $range } \
      {
        return
      }
  
      if { $limit_value < 0.0 } \
      {
        return
      }

      # The value 'meterbar_height' is the max height of the bargraph (in pixels)
      set ratio  [expr $meterbar_height / $range]

      # Use the value of 'limit_value' to set the marker
      set marker_pos  [expr $limit_value * $ratio]
  
      # Scale the bargraph by the correct amount
      # Determine the present dimensions
      set bargraph_coords  [$canvas_name coords bargraph]
      
      foreach { x1 y1 x2 y2 } $bargraph_coords {}

      set marker_height  [expr $y2 - $marker_pos]
    
      # Display the bargraph for the present reading
      $canvas_name create line 0  $marker_height              \
                                  $line_width $marker_height  \
                                  -width 2                    \
                                  -fill blue                  \
                                  -tags low_marker

      if { $meter_class == "BIG_BER_MTR"} \
      {
        # Display the value of the limit next to the marker on the big meter
        $canvas_name create text 0  [expr $marker_height + 7]  \
                                    -text $limit_value         \
                                    -font {-size 12}           \
                                    -anchor w                  \
                                    -fill blue                 \
                                    -tags low_value
      }
    }
  }


  #***************************************************************************
  #
  #        Name: update_alarm_small
  #
  #  Parameters:
  #
  # Description:
  #
  #***************************************************************************
  proc update_alarm_small { tile_path  args } \
  {
    set cmd  "::BerMtrUI::update_alarm_small_core  $tile_path"
    after cancel  $cmd
    after idle  $cmd
  }
  
  proc update_alarm_small_core { tile_path } \
  {
    #puts "update_alarm_small_core:  $tile_path"

    #variable StateUI
    
    set tile_path  [get_tile  $tile_path]
    
    if { ![winfo exists $tile_path] } \
    { 
      return 
    }

    set btn_alarm  ${tile_path}.btn_alarm
 
    if { ![winfo exists $btn_alarm] } \
    { 
      return 
    }

    switch -- $::Ber::State(BellEnabled) \
    {
      1  { $btn_alarm configure -image image_smallbellon  }
         
      0  { $btn_alarm configure -image image_smallbelloff }
    }
  }
  

  #***************************************************************************
  #
  #        Name: update_alarm_large
  #
  #  Parameters:
  #
  # Description:
  #
  #***************************************************************************
  proc update_alarm_large { tile_path  args } \
  {
    set cmd  "::BerMtrUI::update_alarm_large_core  $tile_path"
    after cancel  $cmd
    after idle  $cmd
  }
  
  proc update_alarm_large_core { tile_path } \
  {
    #puts "update_alarm_large_core:  $tile_path"

    #variable StateUI

    set tile_path  [get_tile  $tile_path]
    
    if { ![winfo exists $tile_path] } \
    { 
      return 
    }

    set btn_alarm  ${tile_path}.btn_alarm_bell
 
    if { ![winfo exists $btn_alarm] } \
    { 
      return 
    }

    switch -- $::Ber::State(BellEnabled) \
    {
      1  { $btn_alarm configure -image image_bigbellon  }
         
      0  { $btn_alarm configure -image image_bigbelloff }
    }
  }
 

  #********************************************************************
  #
  #        Name: update_peak_hold_small
  #
  #  Parameters:
  #
  # Description: Updates the Peak Hold button on the small meter 
  #              to changes in the 'peakEnable' state variable
  #
  #********************************************************************
  proc update_peak_hold_small { tile_path  args } \
  {
    set cmd  "::BerMtrUI::update_peak_hold_small_core  $tile_path"
    after cancel  $cmd
    after idle  $cmd
  }
  
  proc update_peak_hold_small_core { tile_path } \
  {
    #puts "update_peak_hold_small_core:  $tile_path"

    #variable StateUI
    
    set tile_path  [get_tile  $tile_path]
    
    if { ![winfo exists $tile_path] } \
    { 
      return 
    }

    set btn_peak_hold  ${tile_path}.btn_peak_hold
 
    if { ![winfo exists $btn_peak_hold] } \
    { 
      return 
    }

    switch -- $::Ber::State(peakEnable) \
    {
      1  { $btn_peak_hold  config  -relief sunken }
         
      0  { $btn_peak_hold  config  -relief raised }
    }
  }


  #***************************************************************************
  #
  #        Name: update_pattern_small
  #
  #  Parameters: tile_path - tile path of the invoking widget
  #
  # Description: Updates the display to changes in the Generator Modulation 
  #              or P25 pattern
  #
  #***************************************************************************
  proc update_pattern_small { tile_path  args } \
  {
     set cmd [namespace code "update_pattern_small_core $tile_path"]
     after cancel $cmd
     after idle $cmd
  }
  proc update_pattern_small_core { tile_path } \
  {

    #puts "update_to_gen_pattern_small:  $tile_path"

    variable StateUI

    set tile_path  [get_tile  $tile_path]
    
    if {$::Ber::State(pattern_match)} \
    {
    
       ${tile_path}.mnu_pattern  config  -bg  gray                \
                                        -activebackground  gray  \
                                        -activeforeground  white
    } \
    else \
    {
      ${tile_path}.mnu_pattern  config  -bg  yellow                \
                                        -activebackground  yellow  \
                                        -activeforeground  gray70
    }
  }


  #***************************************************************************
  #
  #        Name: update_pattern_large
  #
  #  Parameters: tile_path - tile path of the invoking widget
  #
  # Description: Updates the display to changes in the Generator Modulation 
  #              or P25 pattern
  #
  #***************************************************************************
  proc update_pattern_large { tile_path  args } \
  {
     set cmd [namespace code "update_pattern_large_core $tile_path"]
     after cancel $cmd
     after idle $cmd
  }
  
  proc update_pattern_large_core { tile_path  } \
  {
    #puts "update_to_gen_pattern_large:  $tile_path"

    variable StateUI

    set tile_path  [get_tile  $tile_path]
    
    if {$::Ber::State(pattern_match)} \
    {
       # Turn the BER Meter pattern menubutton to gray
        ${tile_path}.mnu_test_pattern  config  -bg  gray                \
                                               -activebackground  gray  \
                                               -activeforeground  white
    } \
    else \
    {
       # turn the BER Meter pattern menubutton to yellow
       ${tile_path}.mnu_test_pattern  config  -bg  yellow                \
                                             -activebackground  yellow  \
                                             -activeforeground  gray70
    }    
  }

  #***************************************************************************
  #
  #        Name: update_units_small
  #
  #  Parameters: tile_path - tile path of the invoking widget
  #
  # Description: Updates the display to changes in the BER Meter units
  #
  #***************************************************************************
  proc update_units_small { tile_path  args } \
  {
     set cmd [namespace code "update_units_small_core $tile_path"]
     after cancel $cmd
     after idle $cmd
  }
  proc update_units_small_core { tile_path } \
  {
    #puts "update_units_small:  $tile_path"

    variable StateUI

    set tile_path  [get_tile  $tile_path]
    
    # Display the correct high range menubutton
    grid remove  ${tile_path}.mnu_range_percent
    grid remove  ${tile_path}.mnu_range_ppm
    
    switch -- $::Ber::State(units) \
    {
      "%"    { Screens::position_widget  ${tile_path}.mnu_range_percent
    
               # Invoke the Auto Range menu item
               ${tile_path}.mnu_range_percent.menu  invoke  0
             }

      "ppm"  { Screens::position_widget  ${tile_path}.mnu_range_ppm

               # Invoke the Auto Range menu item
               ${tile_path}.mnu_range_ppm.menu  invoke  0
             }
    }

    return
  }


  #***************************************************************************
  #
  #        Name: update_units_large
  #
  #  Parameters: tile_path - tile path of the invoking widget
  #
  # Description: Updates the display to changes in the BER Meter units
  #
  #***************************************************************************
  proc update_units_large { tile_path  args } \
  {
     set cmd [namespace code "update_units_large_core $tile_path"]
     after cancel $cmd
     after idle $cmd
  }
  proc update_units_large_core { tile_path } \
  {
    #puts "update_units_large:  $tile_path"

    variable StateUI

    set tile_path  [get_tile  $tile_path]
    
    # Display the correct upper limit entry
    grid remove  ${tile_path}.ent_high_limit_per
    grid remove  ${tile_path}.ent_high_limit_ppm

    # Display the correct lower limit entry
    grid remove  ${tile_path}.ent_low_limit_per
    grid remove  ${tile_path}.ent_low_limit_ppm

    # Display the correct high range menubutton
    grid remove  ${tile_path}.mnu_range_percent
    grid remove  ${tile_path}.mnu_range_ppm
    
    switch -- $::Ber::State(units) \
    {
      "%"    { Screens::position_widget  ${tile_path}.ent_high_limit_per
               Screens::position_widget  ${tile_path}.ent_low_limit_per
               
               Screens::position_widget  ${tile_path}.mnu_range_percent 
    
               # Invoke the Auto Range menu item
               ${tile_path}.mnu_range_percent.menu  invoke  0
             }

      "ppm"  { Screens::position_widget  ${tile_path}.ent_high_limit_ppm
               Screens::position_widget  ${tile_path}.ent_low_limit_ppm
               
               Screens::position_widget  ${tile_path}.mnu_range_ppm

               # Invoke the Auto Range menu item
               ${tile_path}.mnu_range_ppm.menu  invoke  0
             }
    }
  }


  #***************************************************************************
  #
  #        Name: set_meter_range                     
  #
  #  Parameters: new_value   - requested new value 
  #                           -1 - no new value requested
  #
  # Description: sets meter range, displays AR and sets scale variable
  #
  #***************************************************************************
  proc set_meter_range { { new_value "" } } \
  {
    #puts "set_meter_range:  $new_value"

    variable StateUI


    if { $new_value != "" } \
    {
      set StateUI(range_selection)  $new_value
    }
    

    # If the variable has no value, set the value to 'AR'
    if { $StateUI(range_selection) == "" } \
    {
      set StateUI(range_selection)  "AR"
    }


    if { $StateUI(range_selection) == "AR" } \
    { 
      # Set the namespace variable 
      set StateUI(auto_range)  "AR"
    
      # Set the meter to the highest range initially
      set range_value  [get_highest_range]

      set StateUI(high_range)  $range_value
    } \
    else \
    { 
      # Set the namespace variable 
      set StateUI(auto_range)  ""
  
      set range_value  $StateUI(range_selection)
       
      # Set the button test to indicate the range selected
      set StateUI(high_range)  $range_value
    } 
    
  
    return
  }
  

  #***************************************************************************
  #
  #        Name: set_ref_mode
  #
  #  Parameters: 
  #
  # Description: sets the Reference Mode of the meter
  #
  #***************************************************************************
  proc set_ref_mode { { new_value "" } } \
  {
    #puts "set_ref_mode:  $new_value"
    
    variable  StateUI
    
    if { $new_value == "" } \
    {
      if { $StateUI(ref_mode) == "ON" } \
      {
        set StateUI(ref_mode)  "OFF"
      } \
      else \
      {
        set StateUI(ref_mode)  "ON"
      }
    } \
    else \
    {
      set StateUI(ref_mode)  $new_value
    }       
    

    if { $StateUI(ref_mode) == "ON" } \
    {
      set StateUI(ref_indicate)  "REF"
      
      # Set the ref_reading value if the present value is 0.0,
      # which is the initialization value
      if { $StateUI(ref_reading) == 0.0 } \
      {
        set StateUI(ref_reading)   $StateUI(reading)
      }
    }
  
    if { $StateUI(ref_mode) == "OFF" } \
    {
      set StateUI(ref_indicate)  ""
      set StateUI(ref_reading)   0.0
    }
  
    return
  }


  #***************************************************************************
  #
  #        Name: config_rf_level_entry
  # 
  #  Parameters: tile_path - tile path 
  #
  # Description: Configures the BER meter RF Gen level entry widget, 
  #              according to the RF Gen configuration when the Big BER meter
  #              is mapped.
  #
  #***************************************************************************
  proc config_rf_level_entry { tile_path } \
  {
    #puts "config_rf_level_entry:  $tile_path"
    
    set tile_path  [get_tile  $tile_path]
      
    set level_widget  ${tile_path}.ent_rf_level

    
    if { $::Generator::State(rfout) == "GEN" } \
    {
      if { $::GeneratorUI::levelunit == "dBm" } \
      {
        bind_numeric_spin $level_widget 3 1 10 -110 ::GeneratorUI::enter_gen_level
      }
      
      if { $::GeneratorUI::levelunit == "mV" } \
      {
        bind_numeric_spin $level_widget 3 4 707.1068 .0007 ::GeneratorUI::enter_gen_level
      }
      
      if { $::GeneratorUI::levelunit == "uV" } \
      {
        bind_numeric_spin $level_widget 6 2 707106.81 .0007 ::GeneratorUI::enter_gen_level
      }
    } \
    else \
    {
      if { $::GeneratorUI::levelunit == "dBm" } \
      {
        bind_numeric_spin $level_widget 3 1 -30 -137 ::GeneratorUI::enter_gen_level
      }
      
      if { $::GeneratorUI::levelunit == "mV" } \
      {
        bind_numeric_spin $level_widget 3 4 7.0711 0 ::GeneratorUI::enter_gen_level
      }
      
      if { $::GeneratorUI::levelunit == "uV" } \
      {
        bind_numeric_spin $level_widget 6 2 7071.10 0 ::GeneratorUI::enter_gen_level
      }
    }
    
    return
  }
  

  #***************************************************************************
  #
  #        Name: create_pattern_source_menu
  # 
  #  Parameters: tile_path - tile path of the invoking widget
  #
  # Description: Menu 'postcommand'.
  #              Sets the items in the Pattern Source menu.
  #
  #***************************************************************************
  proc create_pattern_source_menu { tile_path } \
  {
    #puts "create_pattern_source_menu:  $tile_path"
    
    set tile_path  [get_tile  $tile_path]
    
    set menu_path  ${tile_path}.mnu_pattern_source.menu
 
    # Clear the present menu items
    $menu_path  delete  0  end
    
    foreach  { name_space }  $::Ber::Sources \
    {
      set name  [subst $\::${name_space}\::Name]
      if [${name_space}::Enabled] \
      {
          # Add each item as a command button
          $menu_path  add  command  -label    "$name"  \
                                -command  "Ber::setPatternSource  $name_space"
      }
    }
  }


  #***************************************************************************
  #
  #        Name: getSaveState
  #
  #  Parameters: none
  #
  # Description: This function returns a list of TCL commands which can be 
  #              used to set the BER Meter tiles to its current state
  #
  #***************************************************************************
  proc getSaveState { } \
  {
    variable StateUI
 
    set returnList \
    {
      set ::BerMtrUI::StateUI(range_selection)  {$StateUI(range_selection)}
      set ::BerMtrUI::StateUI(ref_mode)         {$StateUI(ref_mode)}
    }
       
    return [subst -nocommands $returnList]
  }
  

  #***************************************************************************
  #
  #        Name: Sync
  #
  #  Parameters: none
  #
  # Description: Write state variables to BerMtrUI
  #
  #***************************************************************************
  proc Sync { } \
  {
    eval [getSaveState]
  }
 
  trace variable ::BerMtrUI::StateUI(high_range)  w  \
                                                "::BerMtrUI::update_mid_range"

  update_mid_range


};  # End of BerMtrUI namespace


#****************************************************************************
# This registers the tiles with the list for saving instrument states
#****************************************************************************
namespace eval ::SavedStates \
{
  variable tiles   

  lappend tiles  BerMtrUI
}


