
# -------------------------------------------------------------------
# Routines for encoding and decoding base64
# encoding from Time Janes, 
# decoding from Pascual Alonso,
# namespace'ing and bugs from Parand Tony Darugar 
# (tdarugar@binevolve.com)
# 
# $Id: base64.tcl,v 1.2 2000/10/20 21:35:56 wowbaggr Exp $
# -------------------------------------------------------------------

package provide base64 1.0

namespace eval base64 {
  variable charset "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/"

  namespace export encode decode
  # ----------------------------------------
  # encode the given text
}

proc base64::encode {text} {
    variable charset
    set encoded ""
    set y 0
    for {set i 0} {$i < [string length $text] } {incr i} {
      binary scan [string index $text $i] c x
      if { $x < 0 } {
        set x [expr $x + 256 ]
      }
      set y [expr ( $y << 8 ) + $x]
      if { [expr $i % 3 ] == 2}  {
        append  encoded [string index $charset [expr ( $y & 0xfc0000 ) >> 18 ]]
	append  encoded [string index $charset [expr ( $y & 0x03f000 ) >> 12 ]]
	append  encoded [string index $charset [expr ( $y & 0x000fc0 ) >> 6 ]]
	append  encoded [string index $charset [expr ( $y & 0x00003f ) ]]
	set y 0
      }
    }
    if { [expr $i % 3 ] == 1 } {
      set y [ expr $y << 4 ]
      append encoded [string index $charset [ expr ( $y & 0x000fc0 ) >> 6]]
      append encoded [string index $charset [ expr ( $y & 0x00003f ) ]]
      append encoded "=="
    }
    if { [expr $i % 3 ] == 2 } {
      set y [ expr $y << 2 ] 
      append  encoded [string index $charset [expr ( $y & 0x03f000 ) >> 12 ]]
      append  encoded [string index $charset [expr ( $y & 0x000fc0 ) >> 6 ]]
      append  encoded [string index $charset [expr ( $y & 0x00003f ) ]]
      append encoded "="
    }
    return $encoded
  }

  # ----------------------------------------
  # decode the given text
  # Generously contributed by Pascual Alonso
  proc base64::decode {text} {
    variable charset
    set decoded ""
    set y 0
    if {[string first = $text] == -1} {
      set lenx [string length $text]
    } else {
      set lenx [string first = $text]
    }
    for {set i 0} {$i < $lenx } {incr i} {
      set x [string first [string index $text $i] $charset]
      set y [expr ( $y << 6 ) + $x]
      if { [expr $i % 4 ] == 3}  {
        append decoded \
	  [binary format c [expr $y >> 16 ]]
	append decoded \
	  [binary format c [expr ( $y & 0x00ff00 ) >> 8 ]]
	append decoded \
	  [binary format c [expr ( $y & 0x0000ff ) ]]
	set y 0
      }
    }
    if { [expr $i % 4 ] == 3 } {
      set y [ expr $y >> 2 ] 
	append decoded \
	  [binary format c [expr ( $y & 0x00ff00 ) >> 8 ]]
	append decoded \
	  [binary format c [expr ( $y & 0x0000ff ) ]]
    }
    if { [expr $i % 4 ] == 2 } {
      set y [ expr $y >> 4 ] 
	append decoded \
	  [binary format c [expr ( $y & 0x0000ff ) ]]
    }
    return $decoded
  }

