#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C)2000      DESIGN ENGINEERING      
# --------------------------------------------------------------------------
# FILE        : 1105.altered_tk
# PRODUCT     : FITS, IFR 2975                                                      
# AUTHOR      : Steve Layton                                              
# DESCRIPTION : Contains altered tk library files that needed to be
#               changed.
#############################################################################

#****************************************************************************
#        Name: tkMenuNextEntry_Altered
#  Parameters: menu, menu widget name
#              count, 1 for down, -1 for up
# Description: copy of tcl's tkMenuNextEntry only
#              "tkMenuFirstEntry $cascade" is commented out.
#              stops up,down from transferring focus to cascades   
#****************************************************************************
proc tkMenuNextEntry_Altered { menu  count } \
{
  global tkPriv

  if { [$menu index last] == "none" } \
  {
    return
  }
  
  set length  [expr {[$menu index last] + 1}]
  set quitAfter  $length
  set active  [$menu index active]

  if { $active == "none" } \
  {
    set i  0
  } \
  else \
  {
    set i  [expr {$active + $count}]
  }
  
  while { 1 } \
  {
    if { $quitAfter <= 0 } \
    {
      # We've tried every entry in the menu.  Either there are
      # none, or they're all disabled.  Just give up.
      return
    }
    
    while { $i < 0 } \
    {
      incr i $length
    }
  
    while { $i >= $length } \
    {
      incr i -$length
    }
    
    switch [$menu type $i] \
    {
      radiobutton  -
      cascade      -
      checkbutton  -
      command      { if { [catch {$menu entrycget $i -state} state] == 0 } \
                     {
                       if { $state != "disabled" } \
                       {
                         break
                       }
                     }
                   }
     
      default      { }
    }
    
    if { $i == $active } \
    {
      return
    }
  
    incr i  $count
    incr quitAfter  -1
  }
   
  $menu activate  $i
   
  tkGenerateMenuSelect  $menu
   
  if { [$menu type $i] == "cascade" } \
  {
    set cascade  [$menu entrycget $i -menu]
     
    if { [string compare $cascade ""] != 0 } \
    {
      #$menu postcascade $i
      #tkMenuFirstEntry $cascade
    }
  }
   
  return
}

#****************************************************************************
#        Name: MenuInvokeRemain
#  Parameters: w, menu widget name
# Description: copy of tkMenuInvoke (tk binding routine)
#              without unposting of menu
#****************************************************************************
proc MenuInvokeRemain { w  buttonRelease} \
{
  global tkPriv

  if {$buttonRelease && ($tkPriv(window) == "")} \
  {
    # Mouse was pressed over a menu without a menu button, then
    # dragged off the menu (possibly with a cascade posted) and
    # released.  Unpost everything and quit.

    $w postcascade none
    $w activate none
  
    event generate $w <<MenuSelect>>
    
    #tkMenuUnpost $w
    return
  }
   
  if { [$w type active] == "cascade" } \
  {
    $w postcascade active
    set menu  [$w entrycget active -menu]
    tkMenuFirstEntry $menu
  } \
  elseif { [$w type active] == "tearoff" } \
  {
    tkMenuUnpost $w
    tkTearOffMenu $w
  } \
  elseif { [$w cget -type] == "menubar" } \
  {
    $w postcascade none
    $w activate none
    event generate $w <<MenuSelect>>
    tkMenuUnpost $w
  } \
  else \
  {
    #tkMenuUnpost $w
    uplevel #0 [list $w invoke active]
  }
  
  return
}

#****************************************************************************
#        Name: tkMenuNextMenu
#  Parameters: 
# Description: 
#****************************************************************************
proc tkMenuNextMenu { menu  direction } \
{
  global tkPriv

  # First handle traversals into and out of cascaded menus.

  if { $direction == "right" } \
  {
    set count 1
    set parent [winfo parent $menu]
    set class [winfo class $parent]
    
    if { [$menu type active] == "cascade" } \
    {
      $menu postcascade active
      set m2 [$menu entrycget active -menu]
  
      if { $m2 != "" } \
      {
        tkMenuFirstEntry $m2
      }
	
      return
    } \
    else \
    {
      set parent [winfo parent $menu]
      
      while { $parent != "." } \
      {
        if {([winfo class $parent] == "Menu") && ([$parent cget -type] == "menubar")} \
        {
          tk_menuSetFocus $parent
          tkMenuNextEntry $parent 1
          return
        }
       
        set parent [winfo parent $parent]
      }
    }
  } \
  else \
  {
    set count -1
    set m2 [winfo parent $menu]
    
    if { [winfo class $m2] == "Menu" } \
    {
      if { [$m2 cget -type] != "menubar" } \
      {
        $menu activate none
        tkGenerateMenuSelect $menu
        tk_menuSetFocus $m2
        
        # This code unposts any posted submenu in the parent.
       

        $m2 postcascade none
       
        return
      }
    }
  }

  # Can't traverse into or out of a cascaded menu.  Go to the next
  # or previous menubutton, if that makes sense.

  set m2 [winfo parent $menu]
  
  if {[winfo class $m2] == "Menu"} \
  {
    if {[$m2 cget -type] == "menubar"} \
    {
      tk_menuSetFocus $m2
      tkMenuNextEntry $m2 -1
 
      return
    }
  }
  # hand the event upstairs, and let him worry about it
  tkMenuUnpost $menu
  switch -- [string tolower $direction] \
  {
       "left"  { event generate $m2 <Left> }
       "right" { event generate $m2 <Right> }
  }
}


#****************************************************************************
#        Name: tkMenuInvoke
#  Parameters: 
# Description: 
#****************************************************************************
proc tkMenuInvoke { w  buttonRelease } \
{
  global tkPriv

  Debug::Puts Menu 2 "tkMenuInvoke {$w $buttonRelease } [$w type active]"

  switch -- [$w type active] \
  {
    "cascade"  \
    {
      $w postcascade active
      set menu [$w entrycget active -menu]
      tkMenuFirstEntry $menu
      return
    }

    "tearoff"  \
    {
      tkMenuUnpost $w
      tkTearOffMenu $w
      return
    }
  }
  
  if {[$w cget -type] == "menubar"} \
  {
    $w postcascade none
    $w activate none
    event generate $w <<MenuSelect>>
    tkMenuUnpost $w
  } \
  else \
  {
    uplevel #0 [list $w invoke active]
    Debug::Puts Menu 10 "unposting due to default - widget is $w type is [$w cget -type]"
    tkMenuUnpost $w
  }
  return
}

# unpost one level of a menu
proc unpost_one_level { item } \
{
   if ![winfo exists $item] { return }
   set parent [winfo parent $item]
   if {[winfo class $parent] == "Menu"} \
   {
      $parent postcascade none
   } \
   else \
   {
      tkMenuUnpost $item
   }
   safe_focus_force $parent
}

proc unfocus_menu { item } \
{
   if ![winfo exists $item] { return }
   if ![winfo ismapped $item] {return }
   set parent [winfo parent $item]
   set focus [focus -displayof $item]
   Debug::Puts Menu 2 "unfocus menu: I'm $item, focus is $focus, parent is $parent"
   # if nobody focused, ignore this
   if {$focus == ""} { return }
   # first check - if focus has gone to one of my children, do nothing
   if [string match $item* $focus] \
   {
      Debug::Puts Menu 2 "unfocus menu: child got focus"
      return
   }
   # OK, not one of my children. Unpost menus all the way up to focus,
   # or until we leave this menu hierarchy
   while {[winfo class $parent] == "Menu"} \
   {
      Debug::Puts Menu 2 "unfocus menu: unpost $item"
      $parent postcascade none
      set item $parent
      set parent [winfo parent $parent]
   }
   # final check - unpost last menu if needed
   if {$focus != $item} \
   {
      tkMenuUnpost $item
      if {$focus != ""} \
      {
         after idle "safe_focus_force $focus"
      }
   }
}
bind Menu <ButtonPress-1> { break;}

bind Menu <ButtonRelease-1> { tkMenuInvoke %W 1}

bind Menubutton <Return> \
{
 %W configure -relief sunken
}
bind Menubutton <KeyRelease-Return> \
{ 
    %W configure -relief raised
    tkMbPost %W
    tkMenuFirstEntry [%W cget -menu]
} 
# rebind the behavior of menu buttons...
bind Menubutton <Button-1> \
{
 %W configure -relief sunken
}
bind Menubutton <ButtonRelease-1> \
{
    %W configure -relief raised
    tkMbPost $tkPriv(inMenubutton) %X %Y
    tkMenuFirstEntry [%W cget -menu]
}

bind Menu <KeyRelease-Escape> {unpost_one_level %W}
bind Menu <Escape> {}
bind Menu <Return> {}
bind Menu <Button> {}
bind Menu <ButtonRelease> {}
bind Menu <KeyRelease-Return> { tkMenuInvoke %W 0; break }
bind Menu <FocusOut>  {unfocus_menu %W}
bind Menu <<EditClose>> { tkMenuUnpost %W}
