#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C)2001             DESIGN ENGINEERING  #
# --------------------------------------------------------------------------#
# FILE        : 1110.DevelopmentTools.tcl                                   #
# PRODUCT     : FITS                                                        #
# AUTHOR      : Steve Johnson                                               #
# DESCRIPTION : various things for debug and development                    #
#                                                                           #
#############################################################################




###########################################################################
#
#    Routines from "Practical Programming in Tck and Tk", Brent B. Welch
#
###########################################################################
#*************************************************
#       Name: ConfigFont
# Parameters: widget - widget to configure the font of
#Description: create a popup allowing the font to be
#             changed
#*************************************************
proc ConfigFont { {widget ""} } \
{
  if {"$widget" == "" } { set widget [focus]}

  set ::dbg_font(widget) $widget
  set ::dbg_font(title) "Set font of $::dbg_font(widget)"
  set ::dbg_font(font) [$widget cget -font]
  set tile [name_child $widget cfg_font]
  Screens::create_toplevel_tile $tile FONT_WINDOW
  
  set menu $tile.font.menu
  set max [$menu index end]
  for {set i 0 } { $i < $max } { incr i } \
  {
     $menu entryconfig $i -font [$menu entrycget $i -label]
  }
}
proc dbg_font_close { widget } \
{
   destroy [winfo parent $widget]
}
proc dbg_set_font { } \
{
   $::dbg_font(widget) config -font $::dbg_font(font)
   set ::dbg_font(title) "Set font of $::dbg_font(widget)"
}

#*************************************************
#       Name: CallTrace
# Parameters: file - file handle to use (optional)
#Description: will do a stack dump of function 
#               calls, for debugging use
#*************************************************
proc CallTrace { {file stdout} } \
{
  set rv "Tcl Call Trace\n"
  set indent "->"
  set level [info level ]
  for {set i 1 } { $i < $level } { incr i } {
     append rv "$indent [info level $i]\n"
     set indent "-$indent"
  }
  
  if {$file == ""} { return $rv}
  puts $file $rv
}


#*************************************************
#       Name: Proc_Show
# Parameters: 
#Description: Printing a procedure definition.
#             Example 13-2
#*************************************************
proc Proc_Show {{namepat *} {file stdout}} {
  foreach proc [info procs $namepat] {
    set space ""
    puts -nonewline $file "proc $proc {"
    foreach arg [info args $proc] {
      if [info default $proc $arg value] {
        puts -nonewline $file "$space{$arg $value}"
      } else {
        puts -nonewline $file $space$arg
      }
      set space " "
    }

    # No newline needed because info body may return a
    # value that starts with a newline

    puts -nonewline $file "} {"
    puts -nonewline $file [info body $proc]
    puts $file "}"
  }
}


#*************************************************
#       Name: Command_Info
# Parameters: 
#Description: Finding built-in commands.
#             Example 13-4
#
#             puts [Command_Info "t*"] 
#
#*************************************************
proc Command_Info {{pattern *}} {
  # Create a table of procedures for quick lookup

  foreach p [info procs $pattern] {
    set isproc($p) 1
  }

  # Look for command not in the procedure table
  set result {}
  foreach c [info commands $pattern] {
    if {![info exists isproc($c)]} {
      lappend result $c
    }
  }
  return [lsort $result]
}

#*************************************************
#       Name: show_array
# Parameters: name of array
#Description: builds a toplevel showing the array
#             and all of its elements
#
#*************************************************
set ::arrayshow_count 0
proc show_array { name {screen ""} } {
   upvar #0 $name array
   
   # create our toplevel
   if {$screen == "" } \
   {
      set screen [option get . debugScreen {} ]
   }
   
   set frame [toplevel .array$::arrayshow_count -screen $screen -width 100 -height 100 -bd 0]
   incr ::arrayshow_count
   wm title $frame "members of $name"
   label $frame.title -text "members of $name" -font Helvetica
   grid  $frame.title -row 0 -column 0 -sticky news -columnspan 2
   
   scrollbar $frame.xscroll -orient horiz \
                            -command "$frame.view xview"
   grid  $frame.xscroll -row 3 -column 0 -sticky news
   
   scrollbar $frame.yscroll -orient vert \
                            -command "$frame.view yview"
   grid $frame.yscroll -row 1 -column 1 -sticky news -rowspan 3
   
   frame $frame.f
   
   canvas $frame.view -width 0 -height 0 \
                         -xscrollcommand "$frame.xscroll set" \
                         -yscrollcommand "$frame.yscroll set" -bg yellow
   grid $frame.view -row 1 -column 0 -sticky news
   lower $frame.view
   
   button $frame.refresh -text "Refresh view" -command "draw_array_members $name $frame"
   grid $frame.refresh -row 2 -column 0 -columnspan 2
   
   grid rowconfigure $frame 0 -weight 0
   grid rowconfigure $frame 1 -weight 1
   grid rowconfigure $frame 2 -weight 0
   grid rowconfigure $frame 3 -weight 0

   grid columnconfigure $frame 0 -weight 1
   grid columnconfigure $frame 1 -weight 0
   
   frame $frame.members
   $frame.view create window 0 0 -anchor nw -window $frame.members -tag members
   raise $frame.members $frame.view

   # now we have our drawing surface. Create the items
   grid propagate $frame.members true
   draw_array_members $name $frame
   
   grid columnconfig $frame.members 0 -weight 0
   grid columnconfig $frame.members 1 -weight 1
   bind $frame.members <Configure> {
      set parent [winfo parent %W]
      set width  [winfo reqwidth  %W]
      set height [winfo reqheight %W]
      $parent.view config -scrollregion "0 0 $width $height"
   }
   bind $frame.view <Configure> {
      set parent [winfo parent %W]
      set members $parent.members
      set width  [winfo width  %W]
      set height [winfo height %W]
      set mw     [winfo reqwidth $members]
      set mh     [winfo reqheight $members]
      if {$mw > $width} { set width $mw}
      if {$mh > $height} { set height $mh}
      %W itemconfigure members -width $width -height $height
   }
}
proc draw_array_members { name frame } \
{
   upvar #0 $name array
   set row 0
   set names [lsort [array names array]]
   foreach member $names \
   {
     set v $name\($member)
     set l $frame.members.name$row
     set d $frame.members.value$row
     
     destroy $l
     destroy $d
     
     label $l -text         $member -font Helvetica -relief ridge -bd 1
     label $d -textvariable $v      -font Helvetica -relief ridge -bd 1
     grid $l -in $frame.members -row $row -col 0 -sticky news
     grid $d -in $frame.members -row $row -col 1 -sticky news
     incr row
   }
}

#############################################################################
# FUNCTION    : showPackageVar                                              #
# ARGUMENTS   : none                                                        #
# RETURNS     : none                                                        #
# PURPOSE     : Displays the data in a package interface                    #
#             :                                                             #
# Issues      : If this routine is activated before namespace variables are #
#             : set to a value, it will not detect their existance. This is #
#             : especially important for array variables.  If this happens, #
#             : close the window and reactivate after the variables have    #
#             : been set to some value.                                     #
#############################################################################
proc showPackageVar { name } {

    # Get the list of qualified variables in the namespace
    set front "::$name\::"
    set vars [info vars $name\::* ]

    # Setup a window to hold the variables
    set testParent ".testMainWin"
    if {[winfo exists $testParent]} {
        destroy $testParent
    }
    toplevel $testParent -width 400 -height 370 -bd 0
    grid propagate $testParent 0

    # Break the screen into a canvas area and two scroll bars
    scrollbar $testParent.xscroll -orient horiz \
                            -command "$testParent.view xview"
   
    scrollbar $testParent.yscroll -orient vert \
                            -command "$testParent.view yview"

    canvas $testParent.view -width 0 -height 0 \
                         -xscrollcommand "$testParent.xscroll set" \
                         -yscrollcommand "$testParent.yscroll set"

    grid $testParent.view    -row 0 -column 0 -sticky news
    grid $testParent.yscroll -row 0 -column 1 -sticky news -rowspan 2
    grid $testParent.xscroll -row 1 -column 0 -sticky news
   
    grid rowconfigure $testParent 0 -weight 1
    grid rowconfigure $testParent 1 -weight 0

    grid columnconfigure $testParent 0 -weight 1
    grid columnconfigure $testParent 1 -weight 0
    grid propagate $testParent.view 1

    # Create a frame to hold all of the labels and data
    set cWin $testParent.members
    frame $cWin
    $testParent.view create window 0 0 -anchor nw -window $cWin 

    # NOTE:  IT IS CRITICAL TO RAISE THE SCROLL BARS ABOVE THE WINDOW WHICH 
    # HAS ALL OF THE DATA, BECAUSE OF A BUG IN THE CANVAS OBJECT.  The canvas 
    # object is being used to display and scroll through a data window.  In theory,
    # it should confine the scroll space to the size of the canvas object.  In 
    # reality, it assumes that the entire parent window is available for scrolling,
    # which means that the canvas will scroll the window over the scrollbars, or
    # if the data window is bigger than the parent object, the scrollbars are hidden
    # under the data window, blocking all scrolling
    raise $testParent.xscroll $cWin
    raise $testParent.yscroll $cWin

    grid propagate $cWin 1

    set winIndex 0
    foreach tvar $vars {
        # Get the unqualified variable name
        set idx [string last ":" $tvar]
        incr idx 1
        set shortName [string range $tvar $idx end]
        if {[array exists $tvar]} {
            # Process array variables
            set tnames [array names $tvar]
            foreach aName [lsort $tnames] {
                set fullName $front$shortName\($aName\)
                label $cWin.label_win$winIndex -text $shortName\($aName\) -fg black  
                entry $cWin.state_win$winIndex -text $fullName -fg green 
                incr winIndex 1
            }
        } else {
            # Process variables which are not arrays
            label $cWin.label_win$winIndex -text $shortName -fg black  
            entry $cWin.state_win$winIndex -text $tvar -fg green 
            incr winIndex 1
        }
    }

    # Make a simple two column grid of the data
    for {set i 0} {$i < $winIndex} {incr i 1} {
        grid $cWin.label_win$i -in $cWin -row $i -column 0 -sticky news 
        grid $cWin.state_win$i -in $cWin -row $i -column 2 -sticky news
    }

    # The center column is simply a spacer of fixed width
    grid columnconfig $cWin 0 -weight 1
    grid columnconfig $cWin 1 -minsize 25
    grid columnconfig $cWin 2 -weight 1

    # This is used to configure the scroll size of the canvas after all of the
    # data is entered into the window
    bind $cWin <Configure> {
        set parent [winfo parent %W]
        set width  [winfo reqwidth  %W]
        set height [winfo reqheight %W]
        set vwidth  [winfo width  $parent.view]
        set vheight [winfo height $parent.view]
        if { $vwidth > $width   }   { set width $vwidth }
        if { $vheight > $height } { set height $vheight}
        $parent.view config -scrollregion "0 0 $width $height"
    }
}


###########################################################################
#
#    Hard Disk Routines
#
###########################################################################


#*************************************************
#       Name: ClearCurrentState
# Parameters: 
#Description: clears all current_state files, 
#                used during development
#*************************************************
proc ClearCurrentState { } {
  set path $::sys(current_save)

  cd $path
  foreach name {gen rec scope anlz afgen dup p25 sys} {
    append name s00.fds
    put "deleting $name"
    catch {file delete $name}
  }
  
  cd /system
}


###############################################################################
#  Test_help - check all widgets for having proper help files
#  This proc will iterate over all children of ., identifying those widgets
#  which can take focus. It will then query the help URL for them, and verify
#  that the URL in question exists. If it does not, the widget's name, the
#  RC file that defined it, the class of the frame enclosing it, and the URL
#  will be written to a file in the /usr directory that may then be viewed to
#  see what is broken.
################################################################################
namespace eval Test_help \
{
   variable defined_urls
   variable refd_urls
   variable no_help [option get . helpUrl {}]
   array set defined_urls {}
   array set refd_urls {}
   
   proc cross_check_file { fname } \
   {
       variable defined_urls
       variable refd_urls

       set defined_urls(file://localhost$fname) $fname
       
       set base [file tail $fname]
       set is_index [lsearch $::HtmlFrame::indexes $base]
       if {$is_index != -1} \
       {
          set dir [file dirname $fname]
          set defined_urls(file://localhost$dir/) $fname
       }
       
       set fd [open $fname r]
       set filedata [read $fd]
       close $fd
       
       # locate all href tags for validity
       set start 0
       set search1 {<([^>]*)>}
       set search2 {href=["]?([^ "]*)["]?}
       set search3 {name=["]?([^ "]*)["]?}
       while [regexp -nocase -indices $search1 [string range $filedata $start end] x i] \
       {
          # found anchor - is it a ref?
          set s [lindex $x 0]
          set e [lindex $x 1]
          incr s $start
          incr e $start
          set start $e
          set target [string range $filedata $s $e]
          
          if [regexp -nocase $search2 $target x url] \
          {
            # skip non-"file://" links
            if ![string match "file://*" $url] { continue}
            # skip TCL expansions...
            if [string match {*[\[$]*} $url] { continue}
            # deal with .. URLs
            set here $fname
            while [regexp {^\.\.[/]?(.*)} $url foo bar] \
            {
               if [string match */ $here] \
               {
                  set here [file dirname $here]
               }
               set here [file dirname $here]
               set url $bar
            }
            if [regexp {^\.[/]?(.*)} $url foo bar] \
            {
               set url $bar
            }
            set full_url [::HtmlFrame::merge_urls file://localhost$here $url]
            lappend refd_urls($full_url) $fname
          }
          if [regexp -nocase $search3 $target x url] \
          {
            set full_url "file://localhost$fname#$url"
            set defined_urls($full_url) $fname
            # Fill in any alternate versions, too
            if {$is_index != -1} \
            {
                set full_url "file://localhost$dir/#$url"
                set defined_urls($full_url) $fname
            }

          }
          update idle
       }
   }
   proc cross_check_dir { dirname } \
   {
      variable defined_urls
      
      set defined_urls(file://localhost$dirname) $dirname
      
      foreach f [glob -nocomplain $dirname/*] \
      {
         if [file isdir $f] \
         {
            set defined_urls(file://localhost$f/) $f
            cross_check_dir $f
         } \
         else \
         {
            switch -- [string tolower [file extension $f]] \
            {
               ".html" -
               ".shtml" { cross_check_file $f }
               default \
               {
                 set defined_urls(file://localhost$f) $f
               } 
            }
         }
      }
   }
   
    # Internal proc to analyze a widget and report
   proc do_a_widget { widget } \
   {
     variable no_help
     variable defined_urls
     
     upvar container_class container_class
     set fname ""
     
     set error ""
     
     set url [option get $widget helpUrl {}]
     # get any base help URL and add it in
     set base_url [option get $widget baseUrl {}]
     set full_url [::HtmlFrame::merge_urls $base_url $url]
     set rc_file [option get $widget rc_file {}]

     # defined?
     if ![info exists defined_urls($full_url)] \
     {
        set error "Not found"
     }
     
     # is this NOT the default? if so, check for existance
     if { $full_url == $no_help} \
     {
        set error "NO HELP ASSIGNED"
     }
     
     if {$error == ""} { return ""}
     
     return [subst \
     {
        <tr>
          <td>$container_class</td>
          <td>$rc_file</td>
          <td>$widget</td>
          <td>$base_url</td>
          <td>$url</td>
          <td>$full_url</td>
          <td><b>$error</b></td>
        </tr>
     }]
   }
   
   proc do_a_level { widget fd } \
   {
     upvar no_help no_help
     upvar container_class prev_container
     upvar filedata filedata

     set  container_class [winfo class $widget]
     switch -- $container_class \
     {
        "Frame" -
        "TraceWidget" \
        {
           set container_class $prev_container
        }
     }
          
     foreach child [winfo children $widget] \
     {
        if [$child cget -takefocus] \
        {
           set f [do_a_widget $child]
           if {$f != ""} { puts $fd $f}
        }
        do_a_level $child $fd
        update idle
     }
   }
   
   proc crosscheck_widgets { fd } \
   {
     puts $fd {<h1>Errors in help system</h1>
<table>
<tr>
  <td>Container class</td>
  <td>RC file</td>
  <td>Widget</td>
  <td>Base URL</td>
  <td>URL</td>
  <td>Full URL</td>
  <td>Error</td>
</tr>
}
      if [catch {do_a_level . $fd } err] { puts "error: $err"}
   
      puts $fd {</table>}
   }
   
   proc crosscheck_urls { fd } \
   {
      variable defined_urls
      variable refd_urls
      
      # check all the urls given therein
      puts $fd {<table><tr><td>URL</td><td>referenced in </td></tr>}
      foreach url [lsort [array names refd_urls]] \
      {
         if ![info exists defined_urls($url) ] \
         {
            
            puts $fd "<tr><td>$url</td><td>$refd_urls($url)</td></tr>"
         }
      }
      puts $fd {</table>}
   }
   
   proc Main { } \
   {
      variable defined_urls
      variable refd_urls
      
      if [info exists defined_urls] {unset defined_urls}
      if [info exists refd_urls]    {unset refd_urls}

      set defined_urls(file://localhost/)       "/sysdata/html/index.shtml"
      set defined_urls(file://localhost/fonts/) ""
      set defined_urls(file://localhost/other/) ""
      set defined_urls(file://localhost/usr/) ""
      set defined_urls(file://localhost/usr/setups/) ""
      set defined_urls(file://localhost/help/) "/sysdata/html/help/"
      set defined_urls(file://localhost/help/Options/) ""
      set defined_urls(file://localhost/help/Options/version.shtml) ""
      set defined_urls(file://localhost/help/AppNotes/) ""
      set defined_urls(file://localhost/REDIRECT/) ""
      set defined_urls(file://localhost/sysdata/) ""
            
      set container_class ""

      # Build list of valid and referenced URLs
      cross_check_dir /sysdata/html
   
      set fd [open /usr/help_errors.html w]
   
      puts $fd {<html><head><title>Errors in help system</title></head>}
      puts $fd {<body><h1>Errors in help system</h1>}
      
      crosscheck_urls $fd
      crosscheck_widgets $fd
     
      puts $fd {</body></html>}
      close $fd
   
    }
}



###########################################################################
#
#    Test Routines
#
###########################################################################
namespace eval ::TestTools \
{
    variable  next_setup
    set next_setup "102"


###########################################################################
#  Function:  toggle_setups
#  Input:     None
#  Purpose:   This routine provides an example of how to rapidly cycle 
#     between a number of setup files to do automatic stress testing of
#     the 2975 setup Recall process. This function needs to be activated
#     from a Master telnet session.
###########################################################################
 proc toggle_setups { } \
 {
    variable  next_setup

    if { $next_setup == "102"} \
    {
        set next_setup 106
        BGSource "/usr/setups/phoenix/SAVE102.SAV"
        BGCommand "after 5000 ::TestTools::toggle_setups"
    }  elseif { $next_setup == "106" } \
    {
        set next_setup "102"
        BGSource "/usr/setups/phoenix/SAVE106.SAV"
        BGCommand "after 5000 ::TestTools::toggle_setups"
    }  else \
    {
        set next_setup "done"
    }
 }

} 


