#*************************************************
#       Name: screen_done
# Parameters: mode:  RETRY wait for screen focus
#                    DONE  screen is valid now
#Description: checks focus to be valid
#*************************************************
global screen_done_data
package require Units

set screen_done_data(modemenu_allow)   0
set screen_done_data(screen_done_trys) 0

proc screen_done { {mode RETRY} } {
  global screen_done_data

  if {$mode == "FORCE_COMPLETE" } {
    set screen_done_data(modemenu_allow) 1
    set screen_done_data(screen_done_trys) 50
    return 
  }

  if {$mode == "INIT" } {
    set screen_done_data(modemenu_allow) 0
    set screen_done_data(screen_done_trys) 0
    return 
  }

  # CHECK FOCUS TO BE SET AND VALID
  set len [string length [focus]] 

#puts "screen_done ($len) [focus]"

  if {[string length [focus]] > 1} {
    set screen_done_data(modemenu_allow) 1
    set screen_done_data(screen_done_trys) 50
    return
  }

  incr screen_done_data(screen_done_trys)
  
  if {$screen_done_data(screen_done_trys) < 50} {
    after 150 "screen_done"
  } else {
    set screen_done_data(modemenu_allow) 1
  }
}

#*************************************************
#       Name: is_screen_done
# Parameters: none
#Description: 1 screen ready, 0 not ready
#*************************************************
proc is_screen_done {} {
  global screen_done_data
  return $screen_done_data(modemenu_allow)
}


#*************************************************
#       Name: set_focus
# Parameters: last, perferred widget to set focus 
#              to (usually last widget to have focus)
#           : default, default widget to set focus 
#              to if last not exist
#Description: sets focus to widget, used after new 
#               screen drawn
#       : DONT WORK ON TOPLEVELS
#*************************************************
proc set_focus {last {default 0} } {
 
  # GIVE FOCUS TO LAST WIDGET TO HAVE FOCUS   
  if {[winfo exists $last]} \
  {
     after idle  "safe_focus_force $last"
  }
} 

#*************************************************
#       Name: serial_number
# Parameters: 
#Description: returns serial number of test set
#*************************************************
proc serial_number { } \
{
  puts "serial_number DEPRECTATED: use IFR_options ser"
  return [IFR_options ser]
}



#*************************************************
#       Name: save_settings
# Parameters: type, {CURRENTSAVE SCREENSAVE TRACESAVE STATESAVE}
#       : name, file name
#       : array_list, list of arrays to save
#       : variable_list, list of variables to save
#Description: saves state of screens or traces or state of box
#*************************************************
proc save_settings { type name array_list {variable_list {}} } {
  global sys
  global tcl_platform

  if {$type == "CURRENTSAVE"} {
    set path $sys(current_save)
  } else {
    set path $sys(user_screen_save)
  }

  
  set combined [file join $path $name]
  if [file isdirectory $combined] {
    put "save failed, combined is $combined"
    return 1
  }
  file mkdir [file dirname $combined]

  # IF FILE EXISTS, DELETE
  file delete $combined

  # OPEN NEW FILE
  set fileid [open $combined w 0666]
  
  # ADD TIME OF STORE
  set store_time [clock format [clock seconds] -format "%H:%M:%S"]
  set store_date [clock format [clock seconds] -format "%b:%d:%Y"]
    
  # CREATE LIST
  # HEADER PART
    
  puts $fileid "#$type 2975 [IFR_options ser] $store_time $store_date {[firmware_version]}"
  # INDIVIDUAL VARIABLE PART
  if {$variable_list != 0 && $variable_list != "NULL"} {
    foreach var $variable_list {
      upvar $var x
      puts $fileid "  set ::$var {$x}"
    }
  }

  # ARRAY PART
  if {$array_list != 0 && $array_list != "NULL"} {
    foreach value $array_list {
      upvar #0 $value x
      if [info exists x(save_list)] {
        set indexes $x(save_list)
      } else {
        set indexes [array names x]
      }
      set indexes [lsort $indexes]
      puts $fileid "# saving:"
      puts $fileid "# $indexes"
      puts $fileid " set ::$value\(foo) 1"
      puts $fileid " array set ::$value {"
      foreach index $indexes {
        if [info exists x($index)] {
          puts $fileid "  $index {$x($index)}"
        }
      }
      puts $fileid " }"
    }
  }
     
  close $fileid
  return 0

}

#*************************************************
#       Name: recall_settings
# Parameters: name, file name
#       : type, {CURRENTSAVE (current screen saves) SCREENSAVE (user screen saves)} 
#Description: recalls arrays and variables stored in file
#*************************************************
proc recall_settings { name {type SCREENSAVE} } {
  global sys
  global tcl_platform

  if {$type == "CURRENTSAVE"} {
    set path $sys(current_save)
  } else {
    set path $sys(user_screen_save)
  }

  set combined [file join $path $name]
  if {![file exists $combined]} {
put "recall failed: desired file $name failed to open"  
    return 1
  }

  set fileid [open $combined r]

  # try to read the header
  gets $fileid line
  
  # verify the header
  if {[scan $line "#%s 2975 %d %s %s \{%\[^\}\]\}" type serial time date ver] != 5 } {
      puts "recall failed: $name file corrupt, could not read file"
      close $fileid
      return 1
  }
    
  # CHECK VERSION OF SAVED FILE
  if {$ver != [firmware_version]} {
      puts "recall failed: recalled file is version $ver, current version is [firmware_version]"
      close $fileid
      return 1
  }
  close $fileid

  # OK, source the file now
  if [catch {source $combined} err ] { puts "$err while restoring"}
  
#put "FILE $name RECALLED"
#put ""
    
    return 0
}

#*************************************************
#       Name: source_defaults
# Parameters: 
#Description: restores all screen parameters
#*************************************************
proc source_defaults { } {
   set ::Recall_SetupsUI::State(recall_in_process) 1
   catch { source /system/default_state.tcl }
   set ::Recall_SetupsUI::State(recall_in_process) 0
}

#*************************************************
#       Name: restore_factory_defaults
# Parameters: 
#Description: restores all screen parameters, destroys current screens
#         : does not destroy current screen
#*************************************************
proc restore_factory_defaults { } {
  global sys

  # DESTROY SAVED CURRENT STATE FILES
  foreach file {/sysdata/current_state/powerdown.tcl} \
  {
     file delete -force $file
  }
   
  # SOURCE DEFAULT FILE
  source_defaults
}

#*************************************************
#       Name: backlight_brighter
# Parameters:
#Description: 
#*************************************************
proc backlight_brighter {} {
   exec backlight_brighter
}

#*************************************************
#       Name: backlight_dimmer
# Parameters:
#Description: 
#*************************************************
proc backlight_dimmer {} {
   exec backlight_dimmer
}    

#*************************************************
#       Name: round_digit
# Parameters: value, value to round
#       : digit, number of digits to round to
#Description: returns entered value rounded 
#*************************************************
proc round_digit { value digit } {
  if {$digit < 0} {return $value}

   set rounded [expr pow(10,$digit)]
   set rounded [expr $value*$rounded]
   set rounded [expr round($rounded)]
   set rounded [expr $rounded/(pow(10,$digit))]
   if {$digit == 0} {
    if {$rounded == 0} {
      return 0
    } else {
      return [string trimright $rounded ".0"]
    }
   } 
   return $rounded
}

#*************************************************
#       Name: validate_ip_address
# Parameters: ip address checked
#Description: checks address for validitity,
#       : returns list of the octets if valid, else null list
#*************************************************
proc validate_ip_address {value {arrayname {}} } {
  # create a regexp for parsing IP octets
  set match "(\[0-9\]*)\[ \.\](\[0-9\]*)\[ \.\](\[0-9\]*)\[ \.\](\[0-9\]*)"
 
  # bust the IP address up into pieces
  if {![regexp $match $value addr a b c d]} { return {}}
  # validate each part of the IP address
  foreach octet "$a $b $c $d" {
    if {$octet < 0}    { return {} }
    if {$octet > 255 } { return {} }
  }
  if {"$arrayname" != "" } {
     upvar $arrayname ip
     array set ip "a $a b $b c $c d $d"
  }
  return "$a $b $c $d"
}
#*************************************************
#       Name: normalize_ip_address
# Parameters: ip address 
#Description: given an IP address of the following
#             forms:
#             xx.xx.xx.xx
#             xx xx xx xx
#             {xx xx xx xx}
#             return the IP address in the form
#             xx.xx.xx.xx
#*************************************************
proc normalize_ip_address {args } {
   # check for xx.xx.xx.xx adddresses and convert
   set l $args
   set match "(\[0-9\]*)\[ \.\](\[0-9\]*)\[ \.\](\[0-9\]*)\[ \.\](\[0-9\]*)"
   if {[regexp $match $l addr a b c d]} { 
      set l "$a.$b.$c.$d"
   } else {
      # not a valid IP address. Maybe it's an array name?
      upvar $args var
      if {![info exists var]} { 
         error "Bad IP address $args"
      }
      set l "$var(a).$var(b).$var(c).$var(d)"
   }
   return $l
}

#*************************************************
#       Name: ip_address_to_bin
# Parameters: ip address 
#Description: converts a list of {a b c d} to
#             a binary value
#*************************************************
proc ip_address_to_bin { args } {
   # check for xx.xx.xx.xx adddresses and convert
   set l $args
   set match "(\[0-9\]*)\[ \.\](\[0-9\]*)\[ \.\](\[0-9\]*)\[ \.\](\[0-9\]*)"
   if {[regexp $match $l addr a b c d]} { 
      set l { $a $b $c $d }
   } else {
      # not a valid IP address. Maybe it's an array name?
      upvar $args var
      if {![info exists var]} { 
         error "Bad IP address $args"
      }
      set l "$var(a) $var(b) $var(c) $var(d)"
   }

   set bin 0
   foreach octet $l {
      set bin [expr $bin*256 + $octet]
   }
   return $bin
}


#*************************************************
#       Name: bin_to_ip_address
# Parameters: ip address 
#Description: converts binary IPv4 address into
#             4 octets
#*************************************************
proc bin_to_ip_address { b } {
   set octet [expr $b & 0xff]
   set b [expr $b >> 8]
   set l $octet
   for {set i 1} { $i < 4 } { incr i } {
      set octet [expr $b & 0xff]
      set b [expr $b >> 8]
      set l $l.$octet
   }
   return $l
}
#*************************************************
#       Name: getHostByName
# Parameters: host name to get 
#Description: get the IP address of a host my name
#*************************************************
proc getHostByName { name } {
   if {! [is_vxworks]} { return }
   set b [exec hostGetByName $name]
   if {$b == -1 } {
     # bad name or it's already an IP address
     if {"[validate_ip_address $name]" == "" } {
        # unknown host
        return {}
     }
     return [normalize_ip_address $name]
   }
   return [bin_to_ip_address $b]
}

#*************************************************
#       Name: format_numeric
# Parameters: value, value to pad
#       : precision, number of places to be filled
#       : max, maximum value allowed
#       : min, minimum value allowed
#Description: pad value with zeros if needed
#*************************************************
proc format_numeric { value  precision  {max NONE} {min NONE} {format {}} } \
{
  #puts "format_numeric:  $value,  $precision,  $max,  $min NONE"

  if {$format == ""} { set format "%1.*f"}
  
  # LIMIT VALUE IF NEEDED
  if { $max != "NONE" && $min != "NONE" } \
  {
    set value  [limit_value  $value  $max  $min]
    
    #puts "  value:  $value"
  }

  # FORMAT VALUE
  if [catch {set value [format $format $precision $value]} err] \
  {
    set value 0.0
    set value [format $format $precision $value]
  }
  
  return $value
}

#*************************************************
#       Name: limit_value
# Parameters: value to limit, max allowed, min allowed
#Description: returns value once checked for limits
#*************************************************
proc limit_value { value  max  min } \
{
  #puts "limit_value:  $value,  $max,  $min"

  if { $value == "." } \
  {
    set value 0
  } 

  if { $value <= $min } \
  {
    return $min
  } \
  elseif { $value > $max } \
  {
    return $max
  } \
  else \
  {
    return $value
  }
}

