#****************************************************************************
# IFR SYSTEMS, INC        COPYRIGHT (C)1998,01      DESIGN ENGINEERING      *
#****************************************************************************
# FILE        : 1122.SoftKey.tcl                                            *
# PRODUCT     : 2975-P25                                                    *
# DESCRIPTION : softkey support routines                                    *
#****************************************************************************


#*************************************************
#       Name: assign_softkey
# Parameters: key, number of softkey {2 to 7}
#       : label, text displayed on key {same (text) }
#       : comm, command assigned to key {same (command) }
#       : hue, background color assigned to key {same default (color)}
#       : name, array name used to hold softkey values, cuurent $sys(mode) default
#Description: assigns softkey values, stores values in softkey array
#*************************************************
proc assign_softkey { key label comm {hue default} {name 0}} {
   global color
   global softkey
   global sys

   upvar base base

   if {$hue == "default"} {
    set hue $color(softkeybg)
   }
   if {$name == 0} {
    set name $sys(mode)
   }

  # CHANGE WIDGET CONFIGURE AND UPDATE SOFTKEY ARRAY
  if {$label != "same"} {
    $softkey(name$key) configure -text $label -font smallbold
    set softkey($name\_text$key) $label
  }
  if {$comm != "same"} {
    $softkey(name$key) configure -com $comm
    set softkey($name\_comm$key) $comm
  }
  if {$hue != "same"} {
     $softkey(name$key) configure -bg $hue -activebackground $hue
     set softkey($name\_hue$key) $hue   
  }
}

#*************************************************
#       Name: assign_softkey_special
# Parameters: key, number of softkey {2 to 7}
#       : label, text displayed on key {same (text) }
#       : comm, command assigned to key {same (command) }
#       : hue, background color assigned to key {same default (color)}
#       : name, array name used to hold softkey values, cuurent $sys(mode) default
#Description: assigns softkey values, stores values in softkey array
#*************************************************
proc assign_softkey_special { key label comm {hue default} {name 0}} {
   global color
   global softkey
   global sys

   upvar base base

   if {$hue == "default"} {
    set hue $color(softkeybg)
   }
   if {$name == 0} {
    set name $sys(mode)
   }

  # CHANGE WIDGET CONFIGURE AND UPDATE SOFTKEY ARRAY
  if {$label != "same"} {
    $softkey(name$key) configure -text $label -font tinybold
    set softkey($name\_text$key) $label
  }
  if {$comm != "same"} {
    $softkey(name$key) configure -com $comm
    set softkey($name\_comm$key) $comm
  }
  if {$hue != "same"} {
     $softkey(name$key) configure -bg $hue -activebackground $hue
     set softkey($name\_hue$key) $hue   
  }
}

#*************************************************
#       Name: restore_softkeys
# Parameters: name, array name {optional name defaults to $sys(mode)}
#Description: sets softkey widget configs to softkey array values
#*************************************************
proc restore_softkeys {{name 0}} {
  global softkey
  global sys
  global color
  
  if {$name == 0} {
    set name $sys(mode)
  }
  global $name
  
  for {set key 2} {$key < 8} {incr key} {
    if {[info exists softkey($name\_text$key)]} {
      $softkey(name$key) config -text $softkey($name\_text$key)
    } else {
      $softkey(name$key) config -text " "
    }
    if {[info exists softkey($name\_comm$key)]} {
      $softkey(name$key) config -command $softkey($name\_comm$key)
    } else {
      $softkey(name$key) config -command {set dummy 0}
    }
    if {[info exists softkey($name\_hue$key)]} {
      $softkey(name$key) config -bg $softkey($name\_hue$key) -activebackground $softkey($name\_hue$key)
    } else {
      $softkey(name$key) config -bg $color(softkeybg) -activebackground $color(softkeybg)
    }
  }
}

#*************************************************
#       Name: store_softkeys
# Parameters: name, array name {optional name defaults to $sys(mode)}
#Description: stores current softkey widget configs to an array
#*************************************************
proc store_softkeys {name} {
  global softkey
  
  for {set key 2} {$key < 8} {incr key} {
    set softkey($name\_text$key) [$softkey(name$key) cget -text]
    set softkey($name\_comm$key) [$softkey(name$key) cget -command]
    set softkey($name\_hue$key) [$softkey(name$key) cget -bg]
  }
}

#*************************************************
#       Name: temp_softkey
# Parameters: key, number of softkey {2 to 7}
#       : label, text displayed on key {same (text) }
#       : comm, command assigned to key {same (command) }
#       : hue, background color assigned to key {same default (color)}
#Description: assigns softkey values, does not store them
#*************************************************
proc temp_softkey { key {label {}} {comm {}} {hue {}} } {
   global color
   global softkey
   global sys
   
   upvar base base
   
   if {"$hue" == "default" || $hue == ""} {
    set hue $color(softkeybg)
   }

  # CHANGE WIDGET CONFIGURE AND UPDATE SOFTKEY ARRAY
  if {"$label" != "same" && "$label" != ""} {
    $softkey(name$key) configure -text $label
  }
  if {"$comm" != "same" && "$comm" != ""} {
    $softkey(name$key) configure -com $comm
  }
  if {"$hue" != "same" && "$hue" != "" } {
     $softkey(name$key) configure -bg $hue -activebackground $hue
  }
}

#*************************************************
#       Name: clear_softkeys
# Parameters: 
#Description: clears softkeys
#
#    does not change array values for softkeys
#
#*************************************************
proc clear_softkeys {} {
  global softkey
  global color

  set len  [string first softkey $softkey(name2)]
  incr len -2
  
  for { set key 2 } { $key < 8 } { incr key } {
    set path [string range $softkey(name$key) 0 $len]
    UnPack $path

    pack $softkey(name$key) -expand 1 -fill both

    $softkey(name$key) configure -text " " \
                                 -comm {set dummy 0} \
                                 -bg $color(softkeybg) \
                                 -activebackground $color(softkeybg)

    bind . <F$key> {}
  }
}
