# bindings to allow numeric keys to activate menu buttons.
proc menubutton_keys { widget key } \
{
    if {$::busy_count > 0} { return }
    Busy
    if { [$widget cget -state] == "disabled" } {return}
    set menu [$widget cget -menu]
    tkMbPost $widget
    tkwait visibility $menu
    event generate $menu <Key-BackSpace>
    event generate $menu $key
    NotBusy
}

foreach key { 0 1 2 3 4 5 6 7 8 9 A B C D E F } {
  bind Menubutton <Key-$key>        {menubutton_keys %W %K; break}
}

bind Toplevel_cleanup <Destroy> { Toplevel_cleanup::Destroy %W}
bind Toplevel_cleanup <Map>     { Toplevel_cleanup::Map %W}
bind Toplevel_cleanup <Unmap>   { Toplevel_cleanup::Unmap %W}

proc safe_focus_force { {widget ""}} \
{
   if {$widget == ""} \
   {
       set x [focus -lastfor .]
       Debug::Puts safe_focus_force 1 "safe_focus_force {} use $x"
       focus -force $x
   }
   Debug::Puts safe_focus_force 1 "safe_focus_force $widget"
   after idle "safe_focus_force_core $widget"
}
proc safe_focus_force_core { {widget ""} } \
{
   Debug::Puts safe_focus_force 1 "safe_focus_force_core $widget"
   if [winfo exists $widget] \
   {
      if [$widget cget -takefocus]\
      {
         focus -force $widget
         return
      }
      Debug::Puts safe_focus_force 1 "$widget not focusable - finding best candidate"

      # OK, this won't take focus - does it have any children that will?
      set children [winfo children $widget]
      while {$children != ""} \
      {
         Debug::Puts safe_focus_force 2 "Children = $children"
         # scan this bunch of children, possible adding more to the list
         set bus $children
         set children ""
         foreach seat $bus \
         {
            if ![winfo exists $seat] { continue }
            Debug::Puts  safe_focus_force 2 "$seat"
            if [winfo ismapped $seat] \
            {
               if [$seat cget -takefocus]\
               {
                  focus -force $seat
                  Debug::Puts  safe_focus_force 1 "Focused $seat"
                  return
               }
               Debug::Puts safe_focus_force 2 "adding children of $seat"
               set children [concat $children [winfo children $seat]]
            }
         }
      }
   }
   Debug::Puts safe_focus_force 1 "no focusable objects - using ."
   after idle "safe_focus_force ."
}

namespace eval Toplevel_cleanup \
{
  variable mouse_detected 0
  
  bind . <Motion> { Toplevel_cleanup::Mouse_detect }
  
  proc Mouse_detect {} \
  {
     variable mouse_detected
     set mouse_detected 1
     bind . <Motion> {}
  }
  
  proc Destroy { widget } \
  {
     # unbind all SpinEntries under us...
     SpinEntry::Unbind $widget
     # generate the <<Destroy>> event to be caught by the widget
     # since we cannot just generate the event (we ARE beging destroyed)
     # we have to fake this
     set cmd [bind $widget <<Destroy>>]
     regsub %W $cmd $widget cmd
     eval $cmd
     # restore focus
     set cmd [bind $widget <<RESTORE_FOCUS>>]
     regsub %W $cmd $widget cmd
     if {$cmd != ""} \
     {
        eval $cmd
     } \
     else \
     {
        set parent [winfo parent $widget]
        if ![winfo exists $parent] { return }
        set focus [focus -lastfor $parent]
        if ![winfo exists $focus] { return }
        after idle  "safe_focus_force $focus"
     }
  }
  proc Map { widget } \
  {
     # generate the <<Map>> event to be caught by the widget
     event generate $widget <<Map>>
     # focus him
     after idle  "safe_focus_force $widget"
  }
  proc Unmap { widget } \
  {
    # generate the <<Unmap>> event to be caught by the widget
     event generate $widget <<Unmap>>
  }
  # If we lose focus, remove ourselves
  proc FocusLoss { widget } \
  {
     set cmd [namespace code "FocusLoss_core $widget"]
     after cancel $cmd
     after idle $cmd
  }
  proc FocusLoss_core { widget } \
  {
     variable mouse_detected
     if ![winfo exists $widget] { return }
     if {$mouse_detected} { return }
     set top [winfo toplevel $widget]
     set focus [focus -displayof $widget]
     # OK, is this a child of my toplevel?
     if [string match $top* $focus] \
     {
        return
     }
     destroy $top
     safe_focus_force $focus
  }
}

#*************************************************
#       Name: smart_config
# Parameters: widget to config, field to config
#             value
#Description: configure an aspect of a widget
#             if the value isn't {} or 0
#*************************************************
proc smart_config { widget field value } {
   if {"$value" == "" } { return }
   if {"$value" == "0"} { return }
   $widget configure $field $value
}

#*************************************************
#       Name: destroy_message_box
# Parameters: name, name of toplevel
#Description: detroy message, return focus
#*************************************************
proc destroy_message_box {name} {
   destroy $name
}

#*************************************************
#       Name: generate_event
# Parameters: widget to generate for
#           : event to generate
# if the widget exists, generate the event
#*************************************************
proc generate_event { widget event } {
   if [winfo exists $widget] {
      event generate $widget $event
   }
}
 
#*************************************************
#       Name: message_box
# Parameters: name of toplevel
#           : class of toplevel
# The layout of message boxes is now handled via
# option files.
#*************************************************
proc message_box {name {class MESSAGE}} {

  # create our toplevel. We do this first so we can
  # access the options data for this toplevel
  Busy
  Engmblink 2
  destroy $name
  toplevel $name -class $class
  wm withdraw $name
  add_bindtag $name Toplevel_cleanup
  Mouse_cursor::Hook $name
  
  # we will start the item out iconified, then expand it when done.
  # this will force TWM to warp the mouse to the window.
  wm iconbitmap $name [option get $name windowIcon {}]
  wm group      $name [winfo parent $name]
   
  # Get the basic info for the window from the
  # options
  set title_text [option get $name titleText  {}]
  set geometry   [option get $name geometry   {}]
  set blist      [option get $name buttonList {}]
  set icon       [option get $name icon       {}]
 
  wm geometry $name $geometry
  wm title    $name $title_text

  # get any desired bindings for the window
  set binds [option get $name binds {}]
  foreach { event cmd } $binds {
     bind $name $event $cmd
  }
  
  # Set up the frames
  frame $name.message
  frame $name.extras
  frame $name.buttons
  grid $name.message   -row 0 -sticky news
  grid $name.extras    -row 1 -sticky news
  grid $name.buttons   -row 2 -sticky news
  grid rowconfigure    $name 0 -weight 1
  grid rowconfigure    $name 1 -weight 1
  grid rowconfigure    $name 0 -weight 0
  grid columnconfigure $name 0 -weight 1
  
  label $name.text
  # since the option may contain newlines, we have to re-eval
  # it to make it expand them
  $name.text config -text [subst [$name.text cget -text]]
  # set the text field up to configure auto-wordwrap when it
  # changes size
  bind $name.text <Configure> {
     set width [winfo width %W]
     %W configure -wraplength $width
  }
  
  if {"$icon" != "" } {
     label $name.icon -bitmap $icon
     pack $name.icon -in $name.message -side left -fill none -expand false
  }
  pack $name.text -in $name.message -side left -fill both -expand true
  
  # create any buttons we need
  set column 0
  foreach button $blist {
      set bname $name.$button
      # the button text and command will be in the option file
      button $bname
      
      # since the option may contain newlines, we have to re-eval
      # it to make it expand them
      $bname config -text [subst [$bname cget -text]]
      
      # set the button up to configure auto-wordwrap when it
      # changes size
      bind $bname <Configure> {
         set width [winfo width %W]
         %W configure -wraplength $width
      }
      # buttons are both Cursors and Softkeys
      add_bindtag $bname Cursor Softkey
      
      # get any keyboard bindings desired for the button
      set keybinds [option get $bname keybinds {}]
      foreach key $keybinds {
         bind $name <Key-$key>        "generate_event $bname <<Press>>;   break"
         bind $name <KeyRelease-$key> "generate_event $bname <<Release>> ;break"
      }
      # get any other desired bindings for the button
      set binds [option get $bname binds {}]
      foreach { event cmd } $binds {
         bind $bname $event [subst $cmd]
      }
      # force substitution to happen on the commands
      $bname config -command [subst [$bname cget -command]]
      # if this button is tagged as automatically closing the
      # window when activated, add that command.
      set autoclose [string tolower [option get $bname autoClose {}]]
      switch -- $autoclose {
        true -
        True -
        1 { $bname config -command "[$bname cget -command] ;destroy $name"}     
      }

      grid $bname -in $name.buttons -column $column -row 0 -sticky news
      grid columnconfigure $name.buttons $column -weight 1
      incr column
  }
  
  # create any extra items
  set extras [option get $name extraItems {} ]
  foreach {item type} $extras {
     set iname $name.$item
     $type $iname
     if {"$type" == "entry" } {$iname select adjust 0}
     set cmd       [option get $iname placementCmd {}]
     set placement [option get $iname placement    {}]
     eval $cmd $iname -in $name.extras $placement
 
      # get any other desired bindings for the object
      set binds [option get $iname binds {}]
      foreach { event cmd } $binds {
         bind $iname $event [subst $cmd]
      }
      # get any extra commands for the object
      set cmds [option get $iname extraCmds {}]
      foreach cmd $cmds {
         eval $cmd
      }
    
  }
  set extra_config [option get $name extraCmd {} ]
  if {"$extra_config" != "" } {
     eval $extra_config
  }
  # now, open the window, thus warping mouse focus to it.
  wm deiconify $name
  wm geometry $name $geometry

  # force our button focus
  set focus [option get $name defaultFocus {} ]
  after idle "safe_focus_force $name.$focus;after 500 \"NotBusy;Engmblink 0\""
  
  return $name
}

#*************************************************
#       Name: focus_toplevel
# Parameters: name, widget targeted for focus
#         : keys, list of keys allowed {softkeys}
#Description: set focus to toplevel, performs grab
#*************************************************
proc focus_toplevel {name {keys 0}} {
   if ![winfo exists $name] \
   {
      after idle "safe_focus_force [focus -lastfor .]"
      return
   }
   set last [focus -lastfor $name]
   if [winfo exists $last] {
     safe_focus_force $last
   } else {
     safe_focus_force $name
   }
}     

#*************************************************
# DEPRECATED FUNCTION
#       Name: draw_toplevel
# Parameters: name, toplevel name
#           : height, this value + title height is height of toplevel
#       : width, width of toplevel
#       : title_text, title text (optional, default "")
#       : title_height, height of title (optional, default 30)
#       : placement, (optional, default c) {c n s w e ne nw se sw}
#       : allow_closeX, (optional, default close) close allows top right X,
#       :     none does not.
#       : mask_bus, {mask dontmask} mask to call mask_msgbus_streams/
#       :     unmask_msgbus_streams, dontmask to make no such calls
#Description: draws toplevel with title, centers top level, does binds
#       : returns 0 if draw fails
#*************************************************
proc draw_toplevel {name {height {}} {width {}} {title_text {}}     \
                                      {title_height 30}    \
                                      {placement c}        \
                                      {allow_closeX close} \
                                      {mask_bus mask}      \
                                      {class {} }} {
  global sys
  global color
  global constant
  global temporary
  global tkPriv

  # VERIFY $name DOES NOT ALREADY EXISTS
  if {[winfo exists $name]} {return 0}
  
  # IF SHIFT ON, TURN SHIFT OFF, OTHERWISE OPTION MENU COULD BE ACCESSED
  # AND FOCUS POSSIBLE LOST
  Shift::Clear
  
  # IF MENU IS POSTED, UNPOST IT
  if {[winfo exists $tkPriv(postedMb)] && [winfo viewable $tkPriv(postedMb)]} {
    tkMenuUnpost $tkPriv(postedMb)
  }
   
  if {"$class" != "" } {
     lappend parms -class $class
  }
  if { $width != 0 } {
     lappend parms -width $width
  }
  if { $height != 0 } {
     lappend parms -height $height
  }
 
  eval toplevel $name $parms
  add_bindtag $name Toplevel_cleanup
  Mouse_cursor::Hook $name
  
  # get any parameters that were not suppled from the options database
  if {"$title_text" == ""} {
     set title_text [option get $name title_text {}]
  }

   wm title $name "IFR-2975: $title_text"
   wm iconify $name
   wm iconbitmap $name [option get $name windowIcon {}]
   wm group $name [winfo toplevel $name]
   if { [winfo screen $name] == [winfo screen .] } {
     # OK, this is on our screen. We will set placement
     
     wm geometry $name [option get $name geometry {} ]
   }

  # BIND F8 IF NO TITLE BAR OR NOT
  bind $name <KeyRelease-F8> "destroy $name;break"
  bind $name <Key-F8> "break"
  
  # by iconifying the window, then de-iconifying it, we force the mouse
  # pointer to warp to the window.
  after 100 [subst -nocommands {if [winfo exists $name] {wm deiconify $name; focus_toplevel $name}}]
  return 1
}

#*************************************************
# DEPRECATED FUNCTION
#       Name: draw_button
# Parameters: name, button path name
#           : text, button text label
#           : comm, command to link to button
#       : text_color, foreground color (optional with default color(screenfg))
#       : bgcolor, background color (optional with default color(screenbg))
#       : surround_color, background color of surrounding widgets
#       : just, justification (optional with default left)
#       : anch, anchor (optional with default w)
#Description: draws button 
#*************************************************
proc draw_button { name text comm {text_color {}    } \
                                  { bgcolor {}      } \
                                  {surround_color {}} \
                                  {just {}          } \
                                  {anch {}          } } {

  button $name -text $text -command $comm -takefocus 1
               
  smart_config $name -bg $bgcolor
  smart_config $name -fg $text_color
  smart_config $name -justify $just
  smart_config $name -anchor $anch
  smart_config $name -highlightbackground $surround_color
 
  pack $name -expand true -fill both
  add_bindtag $name Cursor Reflow
}


#*************************************************
# DEPRECATED FUNCTION
#       Name: draw_textvariable
# Parameters: name,
#       : text variable,
#       : width,
#       : text_color (optional, default color(screenfg)
#       : justification (optional)
#       : bgcolor (optional)
#Description: draws entry 
#*************************************************
proc draw_textvariable { name var len {text_color {}    } \
                                      {just       right } \
                                      {bgcolor    {}    } } {

#  label $name -textvariable $var -width $len  
  label $name -textvariable $var  
  smart_config $name -bg $bgcolor
  smart_config $name -fg $text_color
  smart_config $name -justify $just
  
  pack $name -side left -expand true -fill both
  add_bindtag $name Reflow
}

#*************************************************
# DEPRECATED FUNCTION
#       Name: draw_entry
# Parameters: name,
#       : text variable,
#       : width,
#       : text_color (optional, default color(screenfg)
#       : justification (optional)
#       : bgcolor (optional)
#Description: draws entry 
#*************************************************
proc draw_entry { name var len {text_color {}} \
                               {just       {}} \
                               {bgcolor    {}} } {

  entry $name -textvariable $var -width $len -takefocus 1
             
  smart_config $name -bg $bgcolor
  smart_config $name -fg $text_color
  smart_config $name -justify $just
  
  pack $name -side left -expand true -fill both
  add_bindtag $name Cursor
}

#*************************************************
# DEPRECATED FUNCTION
#       Name: draw_frame
# Parameters: name - path name 
#             high - frame height
#             len1 - length of outer frame packed -side top
#             len2 - inner frames packed -side left
#          .. len16- inner frames packed -side left
#Description: draws frame, sets propagate false, packs
#             generates additional frame names from $name.frame$count
#             always a top window and below a number of child windows (pack left)
#       Note: uses global color structure
#*************************************************
proc draw_frame { name high len1 args } { 
   destroy $name
   # do title across the top
   frame $name -height $high -width $len1
   pack propagate $name false
   pack $name -expand true -fill both

   set count 1

   # ok, beneath do a list of child windows
   foreach value $args {
     if {"$value" == "" } {
        frame $name.frame$count -height $high 
        pack propagate $name.frame$count true
     } else {   
        frame $name.frame$count -height $high -width $value
        pack propagate $name.frame$count false
     }
     pack $name.frame$count -side left -expand true -fill both
     incr count
   }
}

#*************************************************
# DEPRECATED FUNCTION
#       Name: draw_label
# Parameters: name, label widget name
#       : text, label text
#       : width, label width in chars
#       : text_color, foreground color (optional default color(screenfg))
#       : bgcolor, background color (optional default color(screenbg))
#       : just, justification (optional default left)
#   `   : anch, anchor (optional default w)
#Description: draws label, packs
#*************************************************
proc draw_label { name text width {text_color {}}    \
                                  {bgcolor    {}}    \
                                  {just       {}}    \
                                  {anch       {}} } {

#  label $name -text $text -width $width
  label $name -text $text
  smart_config $name -bg $bgcolor
  smart_config $name -fg $text_color
  smart_config $name -justify $just
  smart_config $name -anchor $anch

  pack $name -side left -expand true -fill both
  add_bindtag $name Reflow
}

#*************************************************
#       Name: post_menu
# Parameters: name, menubutton path name
#Description: displays popdown, gives it focus
#*************************************************
proc post_menu { name } {
  global tkPriv

  tkMbPost $name [winfo rootx $name] [winfo rooty $name]

  # SEE NOTE IN MAIN.TCL ABOUT tkPriv(postedMb)
  #set tkPriv(postedMb) ""
}

#*************************************************
#       Name: unpost_last_menu
# Parameters: 
#Description: unposts last menu posted
#*************************************************
proc unpost_last_menu { } {
  global tkPriv

  tkMenuUnpost $tkPriv(postedMb)
}

#*************************************************
#       Name: check_menu_room
# Parameters: button_name, menubutton path name
#Description: checks room for posting menu, changes -direction
#       : if necessary
#*************************************************
proc check_menu_room {button_name} {
  set menu [$button_name cget -menu]
  if ![winfo exists $menu] { return }
  
  # IF MENU WONT FIT BELOW, POST RIGHT
  if {[winfo reqheight $menu] > \
     ([winfo screenheight $button_name] - \
      [winfo rooty $button_name] - \
      [winfo height $button_name]) } {

      $button_name config -direction right  

  } else {
      return
  }
  
  # IF MENU WONT FIT RIGHT, POST LEFT
  if {[winfo reqwidth $menu] > \
     ([winfo screenwidth $button_name] - \
      [winfo rootx $button_name] - \
      [winfo width $button_name])} {  

      $button_name config -direction left
  }
}

#*************************************************
#       Name: activate_current
# Parameters: name, menubutton path name
#Description: activates current menu selection when mwnu is posted
#*************************************************
# Search this menu and it's children (if any) to find the active element.
# this is a recursive algorithm.
proc activate_submenu { name value } \
{
    # get the list of sub menus to post in local scope

    upvar postlist postlist
    upvar focus focus
    set end [$name index end]
    for {set i 0} { $i <= $end} { incr i} \
    {
       set type [$name type $i]
       switch $type \
       {
          "cascade" \
          {
            set submenu [$name entrycget $i -menu]
            if [activate_submenu $submenu $value] \
            {
                set postlist [concat "$name activate $i;$name postcascade active;update idle;" $postlist]
                return 1
            }
          }
          "separator" \
          {
          }
          "command" \
          {
          }
          "tearoff" \
          {
          }
          default \
          {
             set label [$name entrycget $i -value]
             if {$label == $value} \
             {
                set postlist [concat "$name activate $i; after idle \"focus $name\";" $postlist]
                set focus $name
                return 1
             }
          }
       }
    }
    return 0
}
  
proc tkMenuFirstEntry { name } \
{
  # GET MENU NAME
  if ![winfo exists $name] \
  {
     Debug::Puts Menu 2 "tkMenuFirstEntry: $name does not exist"
     return
  }

  if ![winfo viewable $name] \
  {
     Debug::Puts Menu 2 "tkMenuFirstEntry: $name is not viewable"
     return
  }

  switch -- [$name type 0] {
    "checkbutton" -
    "radiobutton" {
      if [oldCatch {set variable [$name entrycget 0 -variable]}] \
      {
        Debug::Puts Menu 2 "tkMenuFirstEntry: $name has no variable assigned"
        $name activate 0
        after idle "focus $name"
        return
      }
    }
    default \
    {
      Debug::Puts Menu 2 "tkMenuFirstEntry: $name's first entry is type [$name type 0]"
      $name activate 0
      after idle "focus $name"
      return
    }
  }

  # IF NO variable, RETURN
  if {"$variable"==""} \
  {
     Debug::Puts Menu 2 "tkMenuFirstEntry: $name's variable is null?"
     $name activate 0
     after idle "focus $name"
     return
  }
  
  if ![info exists $variable ] \
  {
     Debug::Puts Menu 2 "tkMenuFirstEntry: $name's variable $variable does not exist"
     $name activate 0
     after idle "focus $name"
     return
  }
    
  upvar 0 $variable x

  set postlist ""
  if {[activate_submenu $name $x] != 0} \
  {
     eval $postlist
  } \
  else \
  {
     # no entry found - might as well activate the default
     $name activate 0
     after idle "focus $name"
  }
}

#*************************************************
#       Name: do_menu_key
# Parameters: widget
#           : index
#Description: handle selecting a button
#*************************************************
proc do_menu_key { widget key } {
  $widget activate $key
  update idle
  tkMenuInvoke $widget 0
}

#*************************************************
#       Name: do_menu_post
# Parameters: widget
#Description: handle showing a menu
#*************************************************
proc do_menu_post { widget} {
   check_menu_room $widget
}

#*************************************************
# DEPRECATED FUNCTION
#       Name: make_menu
# Parameters: menu name
#           : variable name
#           : command sequence to execute 
#           : popdown menu selections, as list or separate selections
#Description: draws menu button and sets up menu
#*************************************************
proc make_menu { menu {variable_name {}} {comm {}} args } {

  if ![winfo exists $menu] { menu $menu} 
  
  # if the args weren't passed on the command line, get them from
  # the options database
  if {"$variable_name" == "" } {
    set variable_name [option get $menu textVariable {}]
  }
 
  if {"$comm" == "" } {
    set comm [option get $menu command {}]
  }
 
  set args [join $args]
  
  if {"$args" == "" } {
    set args [subst [option get $menu entries {}]]
  }
  if {"$args" == "" } {
    set args [subst [option get $menu menu_items {}]]
  }

  # FOR 9 OR LESS MENU SELECTIONS
  set len [llength $args]
  # compute where to break the list
  set breakpoint [expr $len /2]
  # don't let the list be more than 16 items tall...
  if {$breakpoint > 16} { 
     set breakpoint 16
  } elseif {$breakpoint < 5 } {
     # or less than 5 items tall
     set breakpoint 0
  }
  # break the list on an even count
  if {$breakpoint & 2 } {
     incr breakpoint
  }
  
  set base [option get $menu base {}]
  if {"$base" != "" } {
     set count [expr $base]
  } else {
     set count 0
  }
  set format %01d
  if {$len >= 10} {
     set format %02d
  }
  if {$len >= 100} {
     set format %03d
  }
    
  set index 0
  set breakcount 0
  
  set use_index [option get $menu use_index {}]
    
  foreach value $args {
  
 
     # if $value is a list, assume it is of the form { label value } or {label value command }
     
     switch [llength $ ] \
     {
        1 { 
            set label $value
            # if we are using the raw index, pre-set it
            if {$use_index != "" } { set value $index}
            set cmd $comm
          }
        2 { set label [lindex $value 0]; set value [lindex $value 1]; set cmd $comm }
        3 { set label [lindex $value 0]; set value [lindex $value 1]; set cmd [lindex $value 2]}
     }
  
     if { ($breakpoint > 0) && ($breakcount >= $breakpoint) } {
        set breakcount 0
        set colbreak 1
     } else {
        set colbreak 0
        incr breakcount
     }
     set accel [format $format $count]
              
     $menu add radio -accel $accel \
                     -label $label \
                     -variable $variable_name \
                     -value $value \
                     -command $cmd \
                     -columnbreak $colbreak

     bind $menu $accel "do_menu_key %W $index"
     incr count
     incr index
  }
  # force the focus back to our parent when unmapped
  bind $menu <Unmap> { focus [winfo parent %W]}
}

#*************************************************
# DEPRECATED FUNCTION
#       Name: make_popdown
# Parameters: button path name
#           : label variable for button
#           : variable name
#           : command sequence to execute 
#           : popdown menu selections, as list or separate selections
#Description: draws menu button and sets up menu
#*************************************************
proc make_popdown { button_name {labelvar {}} {variable_name {}} {comm {}} args } {

  menubutton   $button_name -menu $button_name.menu -takefocus 1
  smart_config $button_name -textvariable $labelvar
  
  if {"$variable_name" == "" } {
    set variable_name [$button_name cget -textvariable]
  }
  
  if {"$comm" == "" } {
    set comm [option get $button_name command {}]
  }
  
  if {"$args" == "" } {
    set args [subst [option get $button_name entries {}]]
  }
 
  make_menu $button_name.menu $variable_name $comm [join $args]
  
  $button_name.menu config -postcommand "do_menu_post $button_name"

  add_bindtag $button_name Cursor
}

#*************************************************
# DEPRECATED FUNCTION
#       Name: draw_popdown
# Parameters: button path name
#           : variable name
#           : command sequence to execute 
#           : popdown menu selections, as list or separate selections
#Description: draws menu button and sets up menu
#*************************************************
proc draw_popdown { button_name variable_name comm args } {
   make_popdown $button_name $variable_name $variable_name $comm [join $args]
   pack $button_name -side left -expand true -fill both
}


#*************************************************
# DEPRECATED FUNCTION
#       Name: draw_popdown_labeled
# Parameters: button_name, button path name
#           : text_variable_name, button text variable name
#           : menu_variable_name, menu items varaiable name
#           : comm, command sequence to execute 
#           : opt1 - opt12, popdown menu selections (opt1 can be list if opt2 - opt12 not used)
#Description: draws menu button and sets up menu except label is linked to
#           : one variable, menu items to another variable
#*************************************************
proc draw_popdown_labeled { button_name text_variable_name menu_variable_name comm args} {
   make_popdown $button_name $text_variable_name $menu_variable_name $comm [join $args]
   pack $button_name -side left -expand true -fill both
}

namespace eval Progress \
{
  bind PROGRESS <Configure> "Progress::Configure %W"
  bind PROGRESS <Destroy>   "Progress::Destroy %W"
  
#*************************************************
#       Name: progress_bar
# Parameters: name of widget
#Description: draws progress bar
#*************************************************
 proc Create { widget } \
 {
  frame  $widget -class PROGRESS
  add_bindtag $widget PROGRESS
  
  label  $widget.text
  canvas $widget.bar -relief solid -bd 1 -height 10
  $widget.bar create rect 0 0 0 0 -tags box
  $widget.bar create rect 0 0 0 0 -tags progress -fill blue
  
  set color   [option get $widget color   {}]
  set percent [option get $widget percent {}]
  set current [option get $widget current {}]
  set max     [option get $widget max     {}]
 
  if [info exists $color] \
  {
     trace variable $color w "Progress::Color $widget"
  }
  if [info exists $percent] \
  {
     trace variable $percent w "Progress::Percent $widget"
  }
  if [info exists $current] \
  {
     trace variable $current w "Progress::Update $widget"
  }
  if [info exists $max] \
  {
     trace variable $max w "Progress::Update $widget"
  }
  
  add_bindtag $widget.text Reflow
  
  grid $widget.text -row 0 -column 0 -sticky news
  grid $widget.bar  -row 1 -column 0 -sticky news
  grid rowconfigure $widget 0 -weight 1 
  grid rowconfigure $widget 1 -weight 0
  grid columnconfigure $widget 0 -weight 1 

  bind $widget <Destroy> "Progress::Destroy %W"
 }
 proc Destroy { widget } \
 {
   set color   [option get $widget color   {}]
   set percent [option get $widget percent {}]
   set current [option get $widget current {}]
   set max     [option get $widget max     {}]
   trace vdelete $color   w "Progress::Color $widget"
   trace vdelete $percent w "Progress::Percent $widget"
   trace vdelete $current w "Progress::Update $widget"
   trace vdelete $max     w "Progress::Update $widget"
 }
 
#*************************************************
#       Name: Update_old
# Parameters: type, identifier for progress bar
#       : percent, percentage of progress bar being fully shaded
#Description: changes percent full of progress bar
#*************************************************
 proc Update_old { widget percent {color ""} } \
 {
  if ![winfo exists $widget] { return }

  set high [winfo height $widget.bar]
  set wide [winfo width  $widget.bar]
  set fill [expr int(double($percent*$wide)/100.0)]
  if {"$color"==""} \
  {
     set color [option get $widget color {}]
  }
  $widget.bar coords box      0 0 $wide $high
  $widget.bar coords progress 0 0 $fill $high
  $widget.bar itemconfigure progress -fill $color
 }
#*************************************************
#       Name: Color
# Parameters: widget to update, args
#Description: changes color of bar
#*************************************************
 proc Color { widget args } \
 {
    if ![winfo exists $widget] { return }
    set color   [option get $widget color   {}]
    if ![info exists $color] { return }
    upvar #0 $color c
    $widget.bar itemconfigure progress -fill $c
 }
 
 
#*************************************************
#       Name: Update
# Parameters: widget to update, args
#Description: changes percent full of progress bar
#*************************************************
 proc Update { widget args } \
 {
    after cancel "Progress::Update_core $widget"
    after idle "Progress::Update_core $widget"
 }
 proc Update_core { widget } \
 {
  if ![winfo exists $widget] { return }
  
  set current [option get $widget current {}]
  set total   [option get $widget max     {}]
  
  if ![info exists $current] { return }
  if ![info exists $total]   { return }
  
  upvar #0 $current cur
  upvar #0 $total max
  
  if {$max == 0 } { return }
  
  set high [winfo height $widget.bar]
  set wide [winfo width  $widget.bar]
  set fill [expr int(double($cur)*$wide/$max)]

  $widget.bar coords progress 0 0 $fill $high
  $widget.bar coords box      0 0 $wide $high
 }
 
#*************************************************
#       Name: Percent
# Parameters: widget to update, args
#Description: changes percent full of progress bar
#*************************************************
 proc Percent { widget args } \
 {
    after cancel "Progress::Percent_core $widget"
    after idle   "Progress::Percent_core $widget"
 }
 proc Percent_core  { widget } \
 {
  if ![winfo exists $widget] { return }

  set percent [option get $widget percent {}]
 
  if ![info exists $percent] { return }
  
  upvar #0 $percent per
  
  set high [winfo height $widget.bar]
  set wide [winfo width  $widget.bar]
  set fill [expr int(double($per*$wide)/100.0)]

  $widget.bar coords progress 0 0 $fill $high
  $widget.bar coords box      0 0 $wide $high
 }
#*************************************************
#       Name: Configure
# Parameters: widget to update
#Description: 
#*************************************************
 proc Configure { widget } \
 {
  Color $widget
  Percent $widget
  Update $widget
 }
}

proc progress_bar        {widget } { Progress::Create $widget }
proc update_progress_bar {widget percent {color ""}} { Progress::Update_old $widget $percent $color }

#*************************************************
# DEPRECATED FUNCTION
#       Name: WidgetWindow
# Parameters: widget
#             tg
#             parent
#Description: build a floating top level - needs to be moved
#                          to a common utility file...
#*************************************************
proc WidgetWindow { widget tg parent title }  {
    toplevel $tg
    add_bindtag $tg Toplevel_cleanup
    Mouse_cursor::Hook $tg
    wm iconify $tg
    wm iconbitmap $tg [option get $tg windowIcon {}]
    wm transient $tg [winfo toplevel $parent]
    wm group   $tg [winfo toplevel $parent]
    wm title $tg $title

    set fr $tg.fr
    frame $fr     -relief raised -bd 2

    set cmd "$widget $fr"
    eval $cmd

    button $fr.ok -text RETURN -command "destroy $tg" \
                  -default active

    pack $fr.ok   -side bottom -pady 2
    pack $fr

    bind $tg <KeyRelease-F8> "destroy $tg;break"
    bind $tg <Key-F8> "break"
    
    # by iconifying the window, then deiconifying it,
    # we force the mouse point to go to the window.
    wm deiconify $tg
    
    after idle  "safe_focus_force $fr.ok"
}

#*************************************************
#       Name: name_child
# Parameters: widget to make a child name of
#             name for child
#Description: Given a widget W, create a valid
#             Tk name W.child for a child of W.
#             You cannot just concatenate the
#             widget name and the child, as in
#             the case of ".", you'd get ..child,
#             which is invalid.
#*************************************************
proc name_child { widget child } {
   set kid $widget.$child
   regsub  -all \[\.\]\[\.\] $kid \. kid
   return $kid
}

#*************************************************
#       Name: add_bindtag
# Parameters: widget
#             new bind tag
#Description: Given a widget W, add a bind tag to
#             the widget. This bindtag will be added
#             after the widget name, but before its
#             toplevel and all. Any other existing
#             tags in that list will be preserved.
#             
#*************************************************
proc add_bindtag { widget args } {
   set toplevel [winfo toplevel $widget]
   # now, extract the bindtag list
   set tags [bindtags $widget]
   
   # build a list of tags to add
   set newtags {}
   foreach tag $args {
      if {[lsearch $tags $tag] == -1 } { 
         lappend newtags $tag
      }
   }
   
   # remove the widget name, it's parent, and all.
   
   if {$widget == $toplevel } \
   {
       set cur_tags [concat $newtags [lrange $tags 1 [expr [llength $tags]-2]]]
       bindtags $widget "$widget $cur_tags all"
   } \
   else \
   {
     set cur_tags [concat $newtags [lrange $tags 1 [expr [llength $tags]-3]]]
     bindtags $widget "$widget $cur_tags $toplevel all"
   }
}

#*************************************************
#       Name: del_bindtags
# Parameters: widget
#             bind tags to remove
#Description: Give a widget W, remove the specified
#             bindtags if the already exist.
#*************************************************
proc del_bindtag { widget args } {
   set toplevel [winfo toplevel $widget]
   # now, extract the bindtag list
   set tags [bindtags $widget]
   
   # build a list of tags to remove
   set newtags {}
   foreach tag $tags {
      # if this tag is not listed in the remove tags,
      # keep it
      if {[lsearch $args $tag] == -1 } { 
         lappend newtags $tag
      }
   }
   
   bindtags $widget $newtags
}

#*************************************************
#       Name: meter_text_colors
# Parameters: widget, meter status variable
#Description: Update a meter text field based upon
#             the meter reading status flags
#*************************************************
proc meter_text_colors { widget status_name } \
{
    upvar #0 $status_name status
    set tile_path [winfo parent $widget]
    if { ![winfo exists $tile_path] } \
    { 
      return 
    }

    if { ![winfo exists $widget] } \
    { 
      return 
    }

    if { $status & 2 } \
    {
      set color  DimGray
      set bg     DarkGrey
    } \
    else \
    {
      set color  black
      set bg     [$tile_path cget -bg]
    }
    
    if { $status & 0x200 } \
    { 
      set color  red
    } \
    elseif { $status & 0x800 } \
    { 
      set color  blue
    } \
    elseif { $status & 0x101D } \
    {
      set color  yellow
    }
    
    $widget  config  -fg $color -bg $bg
}
