#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C)2001     DESIGN ENGINEERING          #
# --------------------------------------------------------------------------#
# FILE        : 1130.help.tcl                                               #
# PRODUCT     : FITS                                                        #
# AUTHOR      : David Hagood                                                #
# DESCRIPTION : Utilities for doing help operations                         #
#                                                                           #
#############################################################################

namespace eval ::help \
{
   bind all <<Help>> {::help::show %W}
   bind Menu <<Help>> { tkMenuUnpost %W; after idle "::help::show %W";break}

   # array that tracks what help windows are open on what screens
   variable windows
   # allows us to create unique names for the help windows
   variable win_count 0
   
   variable lock 0
  
   #*************************************************
   #       Name: help::lockout_end
   # Parameters: none
   #Description: unlock help 1/2 second after starting
   #             a help
   #*************************************************
   proc lockout_end { } \
   {
      variable lock
      set lock 0
   }
   
   proc Get_url { {widget {}} } \
   {
      if {$widget == "" } { set widget [focus]}
      set url [option get $widget helpUrl {}]
      set base_url [option get $widget baseUrl {}]
      set url [::HtmlFrame::merge_urls $base_url $url]
      return $url
   }
   
   #*************************************************
   #       Name: help::show
   # Parameters: widget generating request
   #             URL to show
   #             Window title
   #Description: Show a help window for network config
   #*************************************************
   proc show { {widget {}} {url {}} {title {Help}} } \
   {
     #puts "show:  $widget,  $url,  $title"
     variable lock

     # first, see if we are already trying to bring up a help. If so
     # then ignore this one.
     set cmd "::help::lockout_end"
     after cancel $cmd
     after 500   $cmd
     if {$lock} { return }
     set lock 1
     
     variable windows
     variable win_count
     variable skey_bindings
     global softkey
     
     set top [View::Find_Main_toplevel $widget]
     
     set name [name_child $top help]
     
     # see if we have a help URL defined.
     if {"$url" == "" } {
        # no URL passed in. See if one if defined for this widget
        set url [option get $widget helpUrl {}]
     }
     # get any base help URL and add it in
     set base_url [option get $widget baseUrl {}]
     
     set url [::HtmlFrame::merge_urls $base_url $url]

     if ![winfo exists $name] \
     {
        set top_url file://localhost/sysdata/html/help/
        ::HtmlFrame::make_toplevel $name $url $title "IFR-2975 Help"
        ::HtmlFrame::do_history $name $top_url
        # it might have gone away while we did this...
        if [winfo exists $name.b5] \
        {
           $name.b5 configure -text "MAIN HELP" -command "::HtmlFrame::do_url $name $top_url"
        }
     } \
     else \
     {
        set old_focus [focus -displayof $name]
        bind $name <<RESTORE_FOCUS>> "safe_focus_force $old_focus"
        ::HtmlFrame::do_url $name $url
     }
     if [winfo exists $name] \
     {
        bind $name <<Help>> "::help::help $name; break"
     }
     return $name
   }
   
   #*************************************************
   #       Name: help::help
   # Parameters: widget generating request
   #Description: Show the help on help
   #*************************************************
   proc help { name } \
   {
     variable lock

     # first, see if we are already trying to bring up a help. If so
     # then ignore this one.
     set cmd "::help::lockout_end"
     after cancel $cmd
     after 500   $cmd
     if {$lock} { return }
     set lock 1
     ::HtmlFrame::do_url $name file://localhost/sysdata/html/help/basics.shtml
   }

};  # End of namespace help
