package require Tracewidget

# Include the Alarm button images
image create photo image_smallbellalert  -file \
                                       "/system/logo/smallbellalert.gif"

image create photo image_smallbellon  -file \
                                       "/system/logo/smallbellon.gif"

image create photo image_smallbelloff -file \
                                      "/system/logo/smallbelloff.gif"

image create photo image_bigbellalert    -file \
                                         "/system/logo/bigbellalert.gif"

image create photo image_bigbellon    -file \
                                         "/system/logo/bigbellon.gif"

image create photo image_bigbelloff   -file \
                                        "/system/logo/bigbelloff.gif"

image create photo image_bigclose     -file \
                                          "/system/logo/bigclose.gif"


# Include the menu button arrow image
image create photo image_menuarrow  -file \
                                       "/system/logo/menuarrow.gif"



namespace eval Screens \
{

  variable  MAX_COLUMNS  2000
  variable  MAX_ROWS     2000
  variable  default_percent_subs
  
 
  #********************************************************************
  #
  #       Name: release_unmapped
  #
  # Parameters: none
  #
  # Returns: none
  #
  # Destroy any tile object that is not currently mapped, in order to free
  # up memory
  #********************************************************************
  proc release_unmapped {  } \
  {
     set widget_list .
     # non-recursively traverse the set of all objects, freeing them
     
     while {$widget_list != ""} \
     {
        set to_do $widget_list
        set widget_list {}
        foreach item $to_do \
        {
           if ![winfo exists $item] { continue }
           if [string match "*TILE_CLEANUP*" [bindtags $item]] \
           {
              if ![winfo ismapped $item] \
              {
                 Debug::Puts CLEANUP 1 "released_ummaped_core : destroy $item"
                 destroy $item
                 continue
              }
           }
           foreach child [winfo children $item] \
           {
              lappend widget_list $child
           }
        }
     }
  }
  Memory::Add_cleanup_proc Screens::release_unmapped
  
  #********************************************************************
  #
  #       Name: subs_string
  #
  # Parameters: string to subsitute
  #
  # Returns: string with any non-backslash escaped %c sequences expanded
  #
  #********************************************************************
  proc subs_string { str {array {::Screens::default_percent_subs}} } \
  {
     # Expand the command - specificly replace %W with the button name, so commands
     # can be defined like events
     set re ((^)|(\[^\\]))%(.)
     # the above regexp will parse are any non-escaped % constructs, leaving
     # escaped constructs alone
  
     regsub -all $re $str {\1$widget_name} str
        
     return $str
  }
  #********************************************************************
  #
  #       Name: get_opt
  #
  # Parameters: widget to query, option name
  #
  # Returns: option from the widget's resources, with substitution
  #          as per calling environment's vars
  #
  #********************************************************************
  proc get_opt { widget_name parm {default_value {}} } \
  {
     set str [option get $widget_name $parm {}]
     set str [uplevel subst [list $str]]
     set str [string trim $str]
      
     if {"$str" == ""} \
     {
        set str $default_value
     }
     return $str
  }
  #********************************************************************
  #
  #       Name: place_widget
  #
  # Parameters: 
  #
  #********************************************************************
  proc place_widget { tile_path  widget_name } \
  {
      set item_relx    [get_opt $widget_name place_relx   0.5]
      set item_rely    [get_opt $widget_name place_rely   0.5]
      set item_height  [get_opt $widget_name place_height {}]
      set item_width   [get_opt $widget_name place_width  {}]
      set item_anchor  [get_opt $widget_name place_anchor center]
      set item_in      [get_opt $widget_name in      $tile_path]
    
      if ![winfo exists $item_in] {
         # perhaps this is a "relative" path to this dialog?
         set item_in [name_child $tile_path $item_in]
      }

              
      place  $widget_name  -relx    $item_relx     \
                           -rely    $item_rely     \
                           -height  $item_height   \
                           -width   $item_width    \
                           -anchor  $item_anchor   \
                           -in      $item_in
      if {$tile_path != $item_in } {raise $widget_name }
    }

  #********************************************************************
  #
  #       Name: pack_widget
  #
  # Parameters: 
  #
  #********************************************************************
  proc pack_widget { tile_path  widget_name } \
  {
      set item_side        [get_opt $widget_name side    top]
      set item_expand      [get_opt $widget_name expand  true]
      set item_fill        [get_opt $widget_name fill    both]
      set item_in          [get_opt $widget_name in      $tile_path]
    
      if ![winfo exists $item_in] {
         # perhaps this is a "relative" path to this dialog?
         set item_in [name_child $tile_path $item_in]
      }
      pack $widget_name -side       $item_side    \
                        -expand     $item_expand  \
                        -fill       $item_fill    \
                        -in         $item_in
      if {$tile_path != $item_in } {raise $widget_name }
  }

  #********************************************************************
  #
  #       Name: grid_widget
  #
  # Parameters: 
  #
  #********************************************************************
  proc grid_widget { tile_path  widget_name } \
  {
      set item_row         [get_opt $widget_name row        0]
      set item_column      [get_opt $widget_name column     0]
      set item_rowspan     [get_opt $widget_name rowspan    1]
      set item_columnspan  [get_opt $widget_name columnspan 1]
      set item_sticky      [get_opt $widget_name sticky     news]
      set item_in      [get_opt $widget_name in      $tile_path]
    
      if ![winfo exists $item_in] {
         # perhaps this is a "relative" path to this dialog?
         set item_in [name_child $tile_path $item_in]
      }
                             
      grid $widget_name -row        $item_row          \
                        -column     $item_column       \
                        -rowspan    $item_rowspan      \
                        -columnspan $item_columnspan   \
                        -sticky     $item_sticky       \
                        -in         $item_in
      if {$tile_path != $item_in } {raise $widget_name }
  }
  #********************************************************************
  #
  #       Name: position_widget
  #
  # Parameters: 
  #
  #********************************************************************
  proc position_widget {tile_path  {widget_name ""} { placement "" } } \
  {
    if ![winfo exists $tile_path] { return }
    
    # allow for just the widget to be positioned to be given
    if {$widget_name == "" } \
    {
       set widget_name $tile_path
       set tile_path [winfo parent $widget_name]
    }
    if ![winfo exists $widget_name] { return }
    if { "$placement" == "auto" } \
    {
       # autoplacing widget: see if this widget has the "noAutoPlace" attribute set
       set noauto [get_opt $widget_name noAutoPlace]
       if {"$noauto" != "" } \
       {
          return
       }
       set placement ""
    }
    if { "$placement" == "" } \
    {
      # Determine the placement command
      set placement [get_opt $tile_path placementCmd]
    }
    if ![winfo exists $widget_name] \
    {
       ErrLog::Log "Trying to place $widget_name but it does not exist"
       return
    }

    switch -- "$placement" \
    {
       place { place_widget $tile_path $widget_name }
       pack  { pack_widget  $tile_path $widget_name }
       grid  { grid_widget  $tile_path $widget_name }
    }
  }
  
  
  #********************************************************************
  #
  #       Name: del_bindtags_widget
  #
  # Parameters: name of widget
  #
  #********************************************************************
  proc del_bindtags_widget { widget_name } \
  {
    Debug::Puts Tile 1 "del_bindtags_widget:  $widget_name"
  
    set item_del_bindtags  [get_opt  $widget_name  del_bindtags]
    
    foreach { bind_tag } $item_del_bindtags \
    {
      del_bindtag  $widget_name  $bind_tag
    }

    return $widget_name
  }


  #********************************************************************
  #
  #       Name: bind_widget
  #
  # Parameters: name of widget
  #
  #********************************************************************
  proc bind_widget { widget_name } \
  {
    Debug::Puts Tile 1 "bind_widget:  $widget_name"

    set item_binds  [get_opt $widget_name binds]
    
    Debug::Puts Tile 2 "item_binds:  $item_binds"
    
    foreach { key_event  event_action } $item_binds \
    {
      bind  $widget_name  $key_event  [subst $event_action]
    }

    set item_bindtags  [get_opt $widget_name bindtags]
   
    eval add_bindtag $widget_name $item_bindtags
   
    return $widget_name
  }

  
  #********************************************************************
  #
  #       Name: create_widget
  #
  # Parameters: name of widget
  #
  #********************************************************************
  proc create_widget { item_type  tile_path  widget_name  { placement "" } { extra_args {} } } \
  {
    Debug::Puts Tile 1 "create_widget:  $item_type,  $tile_path,  $widget_name,  $placement,  $extra_args"

    set widget_name  [name_child  $tile_path  $widget_name]
      
    destroy $widget_name

    eval $item_type  $widget_name  $extra_args
    
    if {"$extra_args" == "" } \
    {
       # now that the widget exists, we can see if it needs any extra args from the database
       set extra_args [get_opt $widget_name extra_args]
       
       if {"$extra_args" != "" } \
       {
          eval $widget_name config $extra_args
       }
    }
    
    position_widget $tile_path $widget_name $placement

    bind_widget $widget_name

    del_bindtags_widget $widget_name

    return $widget_name

  }


  #********************************************************************
  #
  #       Name: create_canvasmeterbar
  #
  # Parameters: name of Canvas Meter Bar
  #
  #********************************************************************
  proc create_canvasmeterbar { tile_path  widget_name  { placement "" } } \
  {
    #set canvasmeterbar_name  \
    #     [create_widget  CanvasMeterbar::Create  $tile_path  $placement]

    return
  }


  #********************************************************************
  #
  #       Name: create_p25digitentry
  #
  # Parameters: name of P25 DIGIT Entry widget
  #
  #********************************************************************
  proc create_p25digitentry { tile_path  widget_name  { placement "" } } \
  {
    set p25digitentry_name  \
         [create_widget  P25DigitEntry::Create  $tile_path  $widget_name  $placement]

    # Add the Cursor bindtag
    add_bindtag  $p25digitentry_name  Cursor
    
    # Add the LogKeyStrokes bindtag
    add_bindtag  $p25digitentry_name  LogKeyStrokes

    return
  }


  #********************************************************************
  #
  #       Name: create_dtmfentry
  #
  # Parameters: name of DTMF Entry widget
  #
  #********************************************************************
  proc create_dtmfentry { tile_path  widget_name  { placement "" } } \
  {
    set dtmfentry_name  \
         [create_widget  DTMFEntry::Create  $tile_path  $widget_name  $placement]

    # Add the Cursor bindtag
    add_bindtag  $dtmfentry_name  Cursor
    
    # Add the LogKeyStrokes bindtag
    add_bindtag  $dtmfentry_name  LogKeyStrokes
  }
  #********************************************************************
  #
  #       Name: create_tsigentry
  #
  # Parameters: name of Tsig Entry widget
  #
  #********************************************************************
  proc create_tsigentry { tile_path  widget_name  { placement "" } } \
  {
    set tsigentry_name  \
         [create_widget  TsigEntry::Create  $tile_path  $widget_name  $placement]

    # Add the Cursor bindtag
    add_bindtag  $tsigentry_name  Cursor
    
    # Add the LogKeyStrokes bindtag
    add_bindtag  $tsigentry_name  LogKeyStrokes
  }


  #********************************************************************
  #
  #       Name: create_alphanumentry
  #
  # Parameters: name of AlphaNumeric Entry widget
  #
  #********************************************************************
  proc create_alphanumentry { tile_path  widget_name  { placement "" } } \
  {
    set alphanumentry_name  \
         [create_widget  AlphaNumEntry::Create  $tile_path  $widget_name  $placement]

    # Add the Cursor bindtag
    add_bindtag  $alphanumentry_name  Cursor
    
    # Add the LogKeyStrokes bindtag
    add_bindtag  $alphanumentry_name  LogKeyStrokes

    return
  }


  #********************************************************************
  #
  #       Name: create_hexstringentry
  #
  # Parameters: name of Hex String Entry widget
  #
  #********************************************************************
  proc create_hexstringentry { tile_path  widget_name  { placement "" } } \
  {
    set hexstringentry  [create_widget  HexStringEntry::Create  $tile_path  $widget_name  $placement]
    
    # Add the Cursor bindtag
    add_bindtag  $hexstringentry  Cursor
    
    # Add the LogKeyStrokes bindtag
    add_bindtag  $hexstringentry  LogKeyStrokes

    return
  }


  #********************************************************************
  #
  #       Name: create_hexentry
  #
  # Parameters: name of Hex Entry widget
  #
  #********************************************************************
  proc create_hexentry { tile_path  widget_name  { placement "" } } \
  {
    set hexentry_name  [create_widget  HexEntry::Create  $tile_path  $widget_name  $placement]
    
    # Add the Cursor bindtag
    add_bindtag  $hexentry_name  Cursor
    
    # Add the LogKeyStrokes bindtag
    add_bindtag  $hexentry_name  LogKeyStrokes

    return
  }


  #********************************************************************
  #
  #       Name: create_spinentry
  #
  # Parameters: name of Spin Entry widget
  #
  #********************************************************************
  proc create_spinentry { tile_path  widget_name  { placement "" } } \
  {
    set spinentry_name  [create_widget  SpinEntry::Create  $tile_path  $widget_name  $placement]
    
    # Add the Cursor bindtag
    add_bindtag  $spinentry_name  Cursor
    
    # Add the LogKeyStrokes bindtag
    add_bindtag  $spinentry_name  LogKeyStrokes

    return
  }


  #********************************************************************
  #
  #       Name: create_scrollbar
  #
  # Parameters: name of scrollbar
  #
  #********************************************************************
  proc create_scrollbar { tile_path  widget_name  { placement "" } } \
  {
    set scrollbar_name [create_widget scrollbar $tile_path $widget_name $placement]

    # Add the Cursor bindtag
    add_bindtag  $scrollbar_name  Cursor
    
    # Add the LogKeyStrokes bindtag
    add_bindtag  $scrollbar_name  LogKeyStrokes

    return
  }


  #********************************************************************
  #
  #       Name: create_listbox
  #
  # Parameters: name of listbox widget
  #
  #********************************************************************
  proc create_listbox { tile_path  widget_name  { placement "" } } \
  {
    set listbox_name [create_widget listbox $tile_path $widget_name $placement]

    # Add the Cursor bindtag
    add_bindtag  $listbox_name  Cursor
    
    # Add the LogKeyStrokes bindtag
    add_bindtag  $listbox_name  LogKeyStrokes

    return
  }      

  #********************************************************************
  #
  #       Name: create_text
  #
  # Parameters: name of text widget
  #
  #********************************************************************
  proc create_text { tile_path  widget_name  { placement "" } } \
  {
    set text_name [create_widget text  $tile_path  $widget_name  $placement]
    
    # Add the Cursor bindtag
    add_bindtag  $text_name  Cursor
    
    # Add the LogKeyStrokes bindtag
    add_bindtag  $text_name  LogKeyStrokes
    
    return
  }      

  #********************************************************************
  #
  #       Name: create_canvas
  #
  # Parameters: name of canvas
  #
  #********************************************************************
  proc create_canvas { tile_path widget_name  { placement "" } } \
  {
    set canvas_path [create_widget canvas $tile_path $widget_name $placement]

    set frame_place [get_opt $canvas_path placementCmd]

    make_subitems $canvas_path $frame_place
 
    set canvas_obj [get_opt $canvas_path objects]

    foreach { object left top right bottom object_config } $canvas_obj \
    {
      eval $canvas_path create $object $left $top $right $bottom $object_config
    }
    
    return
  }

  #********************************************************************
  #
  #       Name: create_alt_menu
  #
  # Parameters: name of menubutton
  #
  #********************************************************************
  proc create_alt_menu { tile_path widget_name  { placement "" } } \
  {
     return [create_menu $tile_path $widget_name $placement alt_menu_items]
  }


  #********************************************************************
  #
  #       Name: create_menu
  #
  # Parameters: name of menubutton
  #
  #********************************************************************
  proc create_menu { tile_path widget_name  { placement "" } { resource menu_items }} \
  {
    set widget_name [create_widget menubutton $tile_path $widget_name $placement]

    # Add the Cursor bindtag
    add_bindtag  $widget_name  Cursor
    
    # Add the LogKeyStrokes bindtag
    add_bindtag  $widget_name  LogKeyStrokes

    set menus        [get_opt $widget_name menus menu]
    set default_menu [get_opt $widget_name default_menu menu]
    
    foreach  menu  $menus \
    {
      set item_menu ${widget_name}.${menu}
       
      create_menu_only ${widget_name} ${item_menu} $resource
    }

    set def_menu ${widget_name}.${default_menu}

    if [winfo exists $def_menu ] \
    {
      $widget_name configure -menu $def_menu
    }
  }
 

  #********************************************************************
  #
  #       Name: create_menu_only
  #
  # Parameters: name of menu, resources to use
  #
  #********************************************************************
  proc create_menu_only { widget_name item_menu  { resource menu_items }} \
  {
    menu $item_menu
    
    Debug::Puts Tile 1 "create_menu_only item_menu:  $item_menu"
    
    set menu_items  [get_opt $widget_name $resource]
    
    if { "$menu_items" == "" } \
    {
      # see if we have menu items of our own
      set menu_items [get_opt $item_menu menu_items]
    }
    
    if { $menu_items != "" } \
    {
      set menu_variable  [get_opt $widget_name menu_var]

      if {$menu_variable == "" } \
      {
         set menu_variable  [get_opt $item_menu menu_var]
      }

      if {$menu_variable == "" } \
      {
         set menu_variable [$widget_name cget -textvariable]
      }

      set menu_command [get_opt $widget_name command]

      if {"$menu_command" == "" } \
      {
         set menu_command [get_opt $item_menu command]
      }

      # expand any % options
      set menu_command [subs_string $menu_command]

      # now replace any variables
      set menu_command [subst -nocommands -nobackslashes $menu_command]

      if {[resource_menu  $item_menu  $menu_variable  $menu_command  $menu_items] == 1} \
      {
         set cmd "Screens::resource_menu  $item_menu $menu_variable \"$menu_command\" \"$menu_items\";do_menu_post $widget_name"
         $item_menu configure -postcommand $cmd
      } \
      else \
      {
          $item_menu configure -postcommand "do_menu_post  $widget_name"
      }

      # Add the LogKeyStrokes bindtag
      add_bindtag  $item_menu  LogKeyStrokes
    }

    return
  }


  #********************************************************************
  #
  #       Name: create_entry
  #
  # Parameters: name of entry widget
  #
  #********************************************************************
  proc create_entry { tile_path widget_name  { placement "" } } \
  {
    set entry_name [create_widget entry $tile_path $widget_name $placement]

    # Add the Cursor bindtag
    add_bindtag  $entry_name  Cursor
    
    # Add the LogKeyStrokes bindtag
    add_bindtag  $entry_name  LogKeyStrokes

    return
  }      


  #********************************************************************
  #
  #       Name: create_label
  #
  # Parameters: name of label
  #
  #********************************************************************
  proc create_label { tile_path  widget_name  { placement "" } } \
  {
    create_widget label $tile_path $widget_name $placement
  }      
  
  #********************************************************************
  #
  #       Name: create_TraceWidget
  #
  # Parameters: name of label
  #
  #********************************************************************
  proc create_TraceWidget { tile_path  widget_name  { placement "" } } \
  {
    set widget [create_widget TraceWidget $tile_path $widget_name $placement]
    # create any traces desired
    set trace_list [get_opt $widget traces]
    set sys [get_opt $widget sysid]
    set cfg [get_opt $widget cfgid]
    foreach { trace subsys lun color } $trace_list \
    {
       $widget trace create $trace -cfgsubsys $cfg -sys $sys -subsys $subsys -lun $lun -color $color
    }
  }
    
  #********************************************************************
  #
  #       Name: create_class
  #
  # Parameters: 
  #
  #********************************************************************
  proc create_class { tile_path  widget_name  { placement "" } classstring } \
  {
    # extract the class from the class type
    regexp {class\((.*)\)} $classstring junk class
    # if that didn't match, look for tile()
    if {"$class" == ""} \
    {
       regexp {tile\((.*)\)} $classstring junk class
    }
    
    set frame_path  [create_widget frame $tile_path $widget_name auto "-class $class"]
    set frame_place [get_opt $frame_path placementCmd]

    # BODGE if the frame doesn't specify a placement command, then use the placement
    #       command of it's parent, but don't place this item
    #       I'd prefer we start using the .noAutoPlace attribute in the resource file
    if {"$frame_place" ==""} \
    {
       set parent [winfo parent $frame_path]
       set frame_place [get_opt $parent placementCmd]
       puts "$frame_path has no placement specified, using $frame_place from $parent"
       puts "if your intention was to avoid auto-placing the frame, please set the "
       puts ".noAutoPlace attribute on this item"
       pack forget  $frame_path
       grid forget  $frame_path
       place forget $frame_path
    }
    make_subitems $frame_path $frame_place
        
    if { $frame_place == "grid" } \
    {
       adjust_grid $frame_path
    }
    
  }
  #********************************************************************
  #
  #       Name: create_scale
  #
  # Parameters: name of scale
  #
  #********************************************************************
  proc create_scale { tile_path  tile_class  widget_name  { placement "" } } \
  {
    set widget_name [create_widget scale $tile_path $tile_class $widget_name $placement]
    
    # get the command and expand any % options
    set cmd [subs_string [$widget_name cget -command]]
    # now replace any variables
    set cmd [subst -nocommands -nobackslashes $cmd]
    
    $widget_name configure -command $cmd
    return
  }        
  #********************************************************************
  #
  #       Name: create_button
  #
  # Parameters: name of button
  #
  #********************************************************************
  proc create_button { tile_path  tile_class  widget_name  { placement "" } } \
  {
    set widget_name [create_widget button $tile_path $tile_class $widget_name $placement]
    
    # get the command and expand any % options
    set cmd [subs_string [$widget_name cget -command]]
    
    # now replace any variables
    set cmd [subst -nocommands -nobackslashes $cmd]
    
    $widget_name configure -command $cmd
    
    # Add the Cursor bindtag
    add_bindtag  $widget_name  Cursor
    
    # Add the LogKeyStrokes bindtag
    add_bindtag  $widget_name  LogKeyStrokes

    return
  }  
  #********************************************************************
  #
  #       Name: adjust_grid
  #
  # Parameters: frame
  #
  #********************************************************************
  proc adjust_grid { tile_path } \
  {
      set tile_height  [get_opt $tile_path rows    ]
      set tile_width   [get_opt $tile_path columns ]

      if {($tile_height == "") || ($tile_width == "")} \
      {
         # this is a kind of tricky way to set the two vars at the same time...
         foreach { tile_width tile_height } [grid size $tile_path] \
         {
         }
      }
      
      # Configure each column and row of the grid
      for { set icolumn 0 } { $icolumn < $tile_width } { incr icolumn 1 } \
      { 
        grid columnconfigure $tile_path $icolumn -weight 1 -minsize 1
      }

      for { set irow 0 } { $irow < $tile_height } { incr irow 1 } \
      {
        grid rowconfigure $tile_path $irow -weight 1 -minsize 1
      }
  }
    
  #********************************************************************
  #
  #       Name: make_subitems
  #
  # Parameters: frame
  #
  #********************************************************************
  proc make_subitems {frame_path frame_place } \
  {
    Debug::Puts Tile 1 "make_subitems:  $frame_path,  $frame_place"

    set frame_items  [get_opt $frame_path items ]
    
    if { $frame_items != "" } \
    {
      foreach { frame_item_name frame_item_type } $frame_items \
      { 
        Debug::Puts Tile 2 "$frame_item_name $frame_item_type"
        switch -glob $frame_item_type \
        {
          frame           { create_frame      $frame_path  $frame_item_name  $frame_place }
                        
          scale           { create_scale      $frame_path  $frame_item_name  $frame_place }
                        
          button          { create_button     $frame_path  $frame_item_name  $frame_place }
                        
          label           { create_label      $frame_path  $frame_item_name  $frame_place }
                        
          entry           { create_entry      $frame_path  $frame_item_name  $frame_place }
                        
          menubutton      { create_menu       $frame_path  $frame_item_name  $frame_place }

          menu            { create_menu_only  $frame_path  $frame_item_name  }

          listbox         { create_listbox    $frame_path  $frame_item_name  $frame_place }

          scrollbar       { create_scrollbar  $frame_path  $frame_item_name  $frame_place }

          canvas          { create_canvas     $frame_path  $frame_item_name  $frame_place }
        
          text            { create_text       $frame_path  $frame_item_name  $frame_place }

          TraceWidget     { create_TraceWidget $frame_path  $frame_item_name  $frame_place }
          
          class(*)        { create_class      $frame_path  $frame_item_name  $frame_place  $frame_item_type}

          tile(*)         { create_class      $frame_path  $frame_item_name  $frame_place  $frame_item_type}

          spinentry       { create_spinentry  $frame_path  $frame_item_name  $frame_place }

          hexentry        { create_hexentry   $frame_path  $frame_item_name  $frame_place }
          
          hexstringentry  { create_hexstringentry   $frame_path  $frame_item_name  $frame_place }

          alphanumentry   { create_alphanumentry   $frame_path  $frame_item_name  $frame_place }
          
          dtmfentry       { create_dtmfentry  $frame_path  $frame_item_name  $frame_place }
          
          tsigentry       { create_tsigentry  $frame_path  $frame_item_name  $frame_place }

          p25digitentry   { create_p25digitentry  $frame_path  $frame_item_name  $frame_place }

          canvasmeterbar  { create_canvasmeterbar  $frame_path  $frame_item_name  $frame_place }

          default         { create_widget     $frame_item_type  $frame_path  $frame_item_name  $frame_place }
        }
      }
     }
   }
  #********************************************************************
  #
  #       Name: create_frame
  #
  # Parameters: name of frame
  #
  #********************************************************************
  proc create_frame { tile_path  widget_name  { placement "" } } \
  {
    Debug::Puts Tile 1 "create_frame $tile_path $widget_name $placement"
    
    set frame_path  [create_widget frame $tile_path $widget_name auto]
    set frame_place [get_opt $frame_path placementCmd]

    # BODGE if the frame doesn't specify a placement command, then use the placement
    #       command of it's parent, but don't place this item
    #       I'd prefer we start using the .noAutoPlace attribute in the resource file
    if {"$frame_place" ==""} \
    {
       set parent [winfo parent $frame_path]
       set frame_place [get_opt $parent placementCmd]
       puts "$frame_path has no placement specified, using $frame_place from $parent"
       puts "if your intention was to avoid auto-placing the frame, please set the "
       puts ".noAutoPlace attribute on this item"
       pack forget  $frame_path
       grid forget  $frame_path
       place forget $frame_path
    }
    make_subitems $frame_path $frame_place
        
    if { $frame_place == "grid" } \
    {
       adjust_grid $frame_path
    }
    
    return
  }
  
  
  #****************************************************************************
  #
  #       Name: create_tile_common  
  #
  # Parameters: tile_path - the frame into which the tile will be placed
  #
  #           : tile_name   - the final part of the name hierarchy of the tile 
  #
  #           : tile_class  - the class type of the tile, optional
  #                           if not supplied, the class will be set to IFR-2975
  #
  # Description: Creates a tile filled with widgets that are applicable to 
  #            : the class type sent as a parameter.
  #            : Accesses the options database to determine the widgets and
  #            : widget options for the class type
  #
  #  create_tile { tile_path  tile_name  tile_class } 
  #
  #****************************************************************************
  proc create_tile_common { tile_path } \
  {
    Debug::Puts Tile 1 "create_tile_common $tile_path"
    
    grid propagate $tile_path 0
    pack propagate $tile_path 0
    
    add_bindtag $tile_path TILE_CLEANUP
    
    # Frame configuration determined from the resource database after the
    # the frame is specified above
    
    # Determine the placement command
    set placement [get_opt $tile_path placementCmd]
    
    make_subitems $tile_path auto

    if { $placement == "grid" } \
    {
      Screens::adjust_grid $tile_path
    }
    
    Screens::bind_widget $tile_path
    return $tile_path
  }
  #****************************************************************************
  #
  #       Name: create_tile  
  #
  # Parameters: tile_path - the frame into which the tile will be placed
  #
  #           : tile_name   - the final part of the name hierarchy of the tile 
  #
  #           : tile_class  - the class type of the tile, optional
  #                           if not supplied, the class will be set to IFR-2975
  #
  # Description: Creates a tile filled with widgets that are applicable to 
  #            : the class type sent as a parameter.
  #            : Accesses the options database to determine the widgets and
  #            : widget options for the class type
  #
  #  create_tile { tile_path  tile_name  tile_class } 
  #
  #****************************************************************************
  proc create_tile { tile_path  tile_name  { tile_class  IFR-2975 } } \
  {
    Debug::Puts Tile 1 "create_tile $tile_path $tile_name $tile_class"
    frame $tile_path -class $tile_class
    if ![Memory::Enough_for tile] \
    {
       label $tile_path.no_memory -text "Not enough memory for this tile"
       pack $tile_path.no_memory -fill both -expand true
       return $tile_path
    }
    return [create_tile_common $tile_path]
  }
  #****************************************************************************
  #
  #       Name: create_toplevel_tile  
  #
  # Parameters: tile_path - the frame into which the tile will be placed
  #
  #           : tile_name   - the final part of the name hierarchy of the tile 
  #
  #           : tile_class  - the class type of the tile, optional
  #                           if not supplied, the class will be set to IFR-2975
  #
  # Description: Creates a tile filled with widgets that are applicable to 
  #            : the class type sent as a parameter.
  #            : Accesses the options database to determine the widgets and
  #            : widget options for the class type
  #
  #  create_tile { tile_path  tile_name  tile_class } 
  #
  #****************************************************************************
  proc create_toplevel_tile_hidden { tile_path { tile_class  IFR-2975 } } \
  {
    Debug::Puts Tile 1 "create_toplevel_tile_hidden $tile_path $tile_class"
    # Configure the frame of the tile
    toplevel $tile_path -class $tile_class
    if ![Memory::Enough_for toplevel] \
    {
       set top [winfo parent $tile_path]
       set msg [name_child $top lowmem]
       message_box $msg CAUTION
       destroy $tile_path
       return -code return
    }

    # IF this window is on the local display, then bind on focus out to make it go away
    set local [winfo screen .]
    set his   [winfo screen $tile_path]
    if {$his == $local} \
    {
       bind $tile_path <FocusOut> "Toplevel_cleanup::FocusLoss %W"
       set cmd [subst -nocommand {focus -force [focus -lastfor $tile_path]; Debug::Puts verify_focus 1 "Took focus to [focus]"}]
       wm focus $tile_path active
       wm proto $tile_path WM_TAKE_FOCUS $cmd
       Debug::Puts Tile 1 "$tile_path WM_TAKE_FOCUS $cmd"
    }
    # Save current focus
    set old_focus [focus -displayof $tile_path]
    bind $tile_path <<RESTORE_FOCUS>> "safe_focus_force $old_focus"
    wm withdraw $tile_path
    add_bindtag $tile_path Toplevel_cleanup
    Mouse_cursor::Hook $tile_path

    wm title    $tile_path "IFR-2975: [get_opt $tile_path title]"
    wm geometry $tile_path [get_opt $tile_path geometry]
    wm iconbitmap  $tile_path @/system/logo/logo.xbm
    create_tile_common $tile_path
    
    bind $tile_path <KeyRelease-F8> "destroy $tile_path;break"
    bind $tile_path <Key-F8> "break"
    return $tile_path
  }
  proc create_toplevel_tile { tile_path { tile_class  IFR-2975 } } \
  {
    Busy
    Engmblink 2
    create_toplevel_tile_hidden $tile_path $tile_class
    after idle "Screens::render_state $tile_path icon"
    return $tile_path
  }
  proc render_state { tile_path state } \
  {
     if ![winfo exists $tile_path] \
     {
         set state unbusy
     }
     switch $state \
     {
        "icon"       { wm deiconify $tile_path ; set state geo}
        "geo"        { wm geometry $tile_path [::Screens::get_opt $tile_path geometry]; set state done_wait}
        "done_wait"  { set state done_wait2 }
        "done_wait2" { set state unbusy }
        "unbusy"     { after 500 "Screens::render_state $tile_path done" ; return}
        "done" -
        "default"    { NotBusy;  Engmblink 0; return}
     }
     after idle "Screens::render_state $tile_path $state"
  }
  

  #****************************************************************************
  #
  #       Name : mnu_breakpoint
  #
  # Parameters : len - number of menu items
  #
  # Description: set breakpoint for menu selections
  #
  #****************************************************************************
  proc mnu_breakpoint { len } \
  {
    # compute where to break the list
    set breakpoint [expr $len /2]

    # don't let the list be more than 16 items tall...
    if {$breakpoint > 16} \
    { 
      set breakpoint 16

    } \
    elseif {$breakpoint < 5 } \
    {
      # or less than 5 items tall
      set breakpoint 0
    }

    # break the list on an even count
    if {$breakpoint & 2 } \
    {
     incr breakpoint
    }

    return $breakpoint
  }


  #****************************************************************************
  #
  #       Name : mnu_base
  #
  # Parameters : base - start base number
  #
  # Description: set count for menu selections
  #
  #****************************************************************************
  proc mnu_base { base } \
  {
    if {"$base" != ""} \
    {
      set count [expr $base]

    } \
    else \
    {
      set count 0
    }

    return $count
  }


  #****************************************************************************
  #
  #       Name : mnu_format
  #
  # Parameters : len - number of menu item
  #
  # Description: set format for menu selections
  #
  #****************************************************************************
  proc mnu_format { len {base 0}} \
  {
    set format %01d
    
    if {$base == 0} { incr len -1}

    if {$len >= 10} \
    {
      set format %02d
    }

    if {$len >= 100} \
    {
      set format %03d
    }

    return $format
  }


  #****************************************************************************
  #
  #       Name : resource_menu
  #
  # Parameters : menu          - menu name
  #              variable_name - variable name
  #              comm          - command sequence to execute
  #              args          - menu items
  #
  # Description: draws menu and submenu
  #
  #****************************************************************************
  proc resource_menu { menu {varname {}} {comm {}} args } \
  {
    set args [join $args]
    set len  [llength $args]
    set base [option get $menu base {}]

    set breakpoint [mnu_breakpoint $len]
    set count      [mnu_base $base]
    set format     [mnu_format $len $count]

    set index 0
    set breakcount 0
    
    $menu delete 0 end

    set use_index [option get $menu use_index {}]

    # if we are a submenu, jam an Escape entry in
    set parent [winfo parent $menu]
    if {[winfo class $parent] == "Menu"} \
    {
        $menu add command -accel ESC \
	                      -label "<previous menu>" \
	                      -command "$parent postcascade none; return -code return"
	    incr breakcount
	    incr index
        # no need to bind this - the defaults will handle it
    }
    
    # if args is a single item that is an array name, set args to be a list of {label,value}
    # pairs based upon the menu's content and names
    set has_value 0
    if {([llength $args] == 1) && [array exists $args]} \
    {
       upvar #0 $args items
       unset args
       set names [array names items]
       set len [llength $names]
       if     {$len < 10}  { set format  %01d } \
       elseif {$len < 100} {  set format %02d } \
       else                {  set format %03d }
       
       if [oldCatch { set names [lsort -integer $names]} err] \
       {
          set names [lsort [array names items]]
       }
       foreach item $names \
       {
          lappend args [list $items($item) $item]
       }
       set has_value 1
       
    }
    
    foreach value $args \
    {
      # if $value is a list, assume it is of the form { label value }
      # or {label value command }

      if {$has_value}  \
      {
           set label [lindex $value 0]
           set value [lindex $value 1]
      } \
      else \
      {
        set label $value
      }

      # if we are using the raw index, pre-set it
      if {$use_index != "" } { set value $index}
  
      if { ($breakpoint > 0) && ($breakcount >= $breakpoint) } \
      {
        set breakcount 0
        set colbreak 1

      } \
      else \
      {
        set colbreak 0
        incr breakcount
      }

      set accel [format $format $count]
      set type  [option get $menu item_type$accel {}]
      set cmd   [option get $menu item_cmd$accel {}]
      if {$cmd == ""} \
      {
         set cmd $comm
      }

      # set default item type to radio button
      # if item type is not specified
      if {$type == ""} \
      {
         set type radio
      }


      switch -- $type \
      {
       check \
	   {
	      $menu add checkbutton -accel $accel \
	                           -label $label \
	                           -variable $varname \
	                           -command $cmd \
	                           -columnbreak $colbreak \
	                           -onvalue $value
       }

	   radio \
	   {
	          $menu add radiobutton -accel $accel \
	                                -label $label \
	                                -variable $varname \
	                                -value $value \
	                                -command $cmd \
	                                -columnbreak $colbreak \
	                                -value $value
	   }

	   separator \
	   {
	     $menu add separator
	   }

	   cascade \
	   {
	     set sub sub$accel
	     set submenu [menu $menu.$sub -tearoff 0]

	          $menu add cascade -accel $accel \
	                       -label $label \
	                       -command $cmd \
	                       -columnbreak $colbreak \
	                       -menu $submenu

	           set args [subst [option get $submenu menu_items {}]]
	           resource_menu $submenu $varname $comm $args
	    }

	    command \
	    {
	      $menu add command -accel $accel \
	                        -label $label \
	                        -command $cmd \
	                        -columnbreak $colbreak
	    }
      }


      bind $menu $accel "do_menu_key %W $index"
      incr count
      incr index
    }
    
    # force the focus back to our parent when unmapped
    bind $menu <Unmap> { focus [winfo parent %W]}
    return $has_value
  }


};   # End of namespace Screens



namespace eval CanvasMeterbar \
{
   bind CanvasMeterbar <Map>     "CanvasMeterbar::Map   %W"
   bind CanvasMeterbar <Unmap>   "CanvasMeterbar::Unmap %W"
   bind CanvasMeterbar <Destroy> "CanvasMeterbar::Destroy %W"
   bind CanvasMeterbar <Configure> "CanvasMeterbar::Configure %W"
   
   variable items
   variable coords
   
   array set items \
   {
      bargraph_frame { rect }
      bargraph       { rect }
      top_line       { line }
      middle_line    { line }
      bottom_line    { line }
   }
   
   # Adjusted the coordinate values in order for all of the
   # bargraphs to look similar.
   array set coords \
   {
      bargraph_frame { 0.21  0.94  0.84  0.04 }
      bargraph       { 0.25  0.94  0.77  0.04 }
      top_line       { 0.84  0.04  0.98  0.04 }
      middle_line    { 0.84  0.50  0.98  0.50 }
      bottom_line    { 0.84  0.94  0.98  0.94 }
   }
   
   #*************************************************
   #       Name: Create
   # Parameters: widget
   #Description: set up a Canvas based meter bar
   #*************************************************
   proc Create { widget  { placement "" } } \
   {
      variable coords
      variable items
      
      # create the basic widget
      frame $widget
      pack propagate $widget false
      
      # create the canvas
      set canvas [canvas $widget.bar]
      pack $canvas -in $widget -expand true -fill both
      
      # create the standard items in the canvas
      foreach item [array names items] \
      {
        eval $canvas create $items($item) [eval Scale_values $canvas $coords($item)] -tags $item
      }
      
      add_bindtag $widget CanvasMeterbar
      Redraw $widget
   }
   
   #*************************************************
   #       Name: Configure
   # Parameters: widget
   #Description: handle resizing a CanvasMeterbar
   #*************************************************
   proc Configure { widget_name } \
   {

      # redraw the fixed items
      after cancel "CanvasMeterbar::Redraw $widget_name"
      after idle "CanvasMeterbar::Redraw $widget_name"
      

   }
   
   #*************************************************
   #       Name: Cleanup
   # Parameters: widget
   #Description: clean up after a CanvasMeterbar is unmapped or destroyed
   #*************************************************
   proc Cleanup { widget_name } \
   {
      set namespace [option get $widget_name namespace {}]
      
      trace vdelete ${namespace}::StateUI(reading)        w " CanvasMeterbar::Update_reading $widget_name"
      trace vdelete ${namespace}::StateUI(reading,color)  w " CanvasMeterbar::Update_reading_color $widget_name"
      trace vdelete ${namespace}::StateUI(reading,max)    w " CanvasMeterbar::Update_scale $widget_name"
      trace vdelete ${namespace}::StateUI(reading,min)    w " CanvasMeterbar::Update_scale $widget_name"
      trace vdelete ${namespace}::StateUI(peak_reading)   w " CanvasMeterbar::Update_peak $widget_name"
      trace vdelete ${namespace}::StateUI(peak_hold)      w " CanvasMeterbar::Update_peak $widget_name"
      trace vdelete ${namespace}::StateUI(hi_limit,on)    w " CanvasMeterbar::Update_hi_limit $widget_name"
      trace vdelete ${namespace}::StateUI(hi_limit,value) w " CanvasMeterbar::Update_hi_limit $widget_name"
      trace vdelete ${namespace}::StateUI(lo_limit,on)    w " CanvasMeterbar::Update_lo_limit $widget_name"
      trace vdelete ${namespace}::StateUI(lo_limit,value) w " CanvasMeterbar::Update_lo_limit $widget_name"
   }
   
   #*************************************************
   #       Name: Destroy
   # Parameters: widget
   #Description: clean up after a CanvasMeterbar is destroyed
   #*************************************************
   proc Destroy { widget_name } \
   {
      Cleanup $widget_name
   }
     
   #*************************************************
   #       Name: Unmap
   # Parameters: widget
   #Description: clean up after a CanvasMeterbar is unmapped
   #*************************************************
   proc Unmap { widget_name } \
   {
      Cleanup $widget_name
   }
     
   #*************************************************
   #       Name: Map
   # Parameters: widget
   #Description: Map a widget on screen
   #*************************************************
   proc Map { widget_name } \
   {
      Redraw $widget_name
      set namespace [option get $widget_name namespace {}]
      
      trace variable ${namespace}::StateUI(reading)        w " CanvasMeterbar::Update_reading $widget_name"
      trace variable ${namespace}::StateUI(reading,color)  w " CanvasMeterbar::Update_reading_color $widget_name"
      trace variable ${namespace}::StateUI(reading,max)    w " CanvasMeterbar::Update_scale $widget_name"
      trace variable ${namespace}::StateUI(reading,min)    w " CanvasMeterbar::Update_scale $widget_name"
      trace variable ${namespace}::StateUI(peak_reading)   w " CanvasMeterbar::Update_peak $widget_name"
      trace variable ${namespace}::StateUI(peak_hold)      w " CanvasMeterbar::Update_peak $widget_name"
      trace variable ${namespace}::StateUI(hi_limit,on)    w " CanvasMeterbar::Update_hi_limit $widget_name"
      trace variable ${namespace}::StateUI(hi_limit,value) w " CanvasMeterbar::Update_hi_limit $widget_name"
      trace variable ${namespace}::StateUI(lo_limit,on)    w " CanvasMeterbar::Update_lo_limit $widget_name"
      trace variable ${namespace}::StateUI(lo_limit,value) w " CanvasMeterbar::Update_lo_limit $widget_name"
   }

   #*************************************************
   #       Name: Scale_values
   # Parameters: widget, value to scale
   #Description: Given a value, return a Y co-ord in
   #             the meter for display
   #*************************************************
   proc Scale_values { canvas  args } \
   {
      # get the size of the canvas itself
      set height [$canvas cget -height]
      set width  [$canvas cget -width]
      
      set ret_value {}
      
      foreach { x y } $args \
      {
         set x [expr int($x * $width)]
         set y [expr int($y * $height)]
         
         lappend ret_value $x $y
      }
      return $ret_value
   }
   
   #*************************************************
   #       Name: Redraw
   # Parameters: widget
   #Description: Redraws the initial objects
   #*************************************************
   proc Redraw { widget_name } \
   {
      if { ![winfo exists $widget_name] } { return }
      
      # get the size of the frame itself
      set height [winfo height $widget_name]
      set width  [winfo width  $widget_name]
      
      if { ![winfo exists ${widget_name}.bar] } { return }

      # make the canvas that big
      $widget_name.bar config -width $width -height $height
      
      variable coords 
      set canvas $widget_name.bar
      
      foreach item [array names coords] \
      {
         set x [eval Scale_values $canvas $coords($item)]
         eval $canvas coords $item $x
      }
      # update the display in the canvas
      Update_reading        $widget_name
      Update_reading_color  $widget_name
      Update_peak           $widget_name
      Update_hi_limit       $widget_name
      Update_lo_limit       $widget_name
   }
   #*************************************************
   #       Name: Update_scale
   # Parameters: widget
   #Description: handle changes of reading scales
   #*************************************************
   proc Update_scale { widget_name  args } \
   {
      set cmd "CanvasMeterbar::Update_scale_core $widget_name"
      after cancel $cmd
      after idle $cmd
   }
   proc Update_scale_core { widget_name } \
   {
   
      Update_reading  $widget_name
      Update_peak     $widget_name
      Update_hi_limit $widget_name
      Update_lo_limit $widget_name
   }
   
   #*************************************************
   #       Name: Update_reading
   # Parameters: widget
   #Description: Update a meter bar on screen
   #*************************************************
   proc Update_reading { widget_name  args } \
   {
      set cmd "CanvasMeterbar::Update_reading_core $widget_name"
      after cancel $cmd
      after idle $cmd
   }
   proc Update_reading_core { widget_name } \
   {
   
      if ![winfo exists $widget_name] { return }
      set canvas $widget_name.bar
     
      # get the namespace vars
      set namespace [option get $widget_name namespace {}]

      upvar #0 ${namespace}::StateUI StateUI
      set reading     $StateUI(reading)
      set max_reading $StateUI(reading,max)
      set min_reading $StateUI(reading,min)
      
      # get the dimensions of the bargraph's enclosing box
      foreach { x2 y2 x1 y1 } [$canvas coords bargraph_frame] {}
      
      # convert the reading into a value from 0 to 1
      if [catch \
          {
            set new_y2 [expr double($reading-$min_reading)/($max_reading-$min_reading)]
          } err
         ] \
      {
         # error on values - bail out
         return
      }
      
      # sanity check: clip reading to the range 0..1
      if {$new_y2 < 0.0 } \
      { 
        set new_y2 0.0 
      } \
      elseif {$new_y2 > 1.0 } \
      { 
        set new_y2 1.0 
      }
      
      set new_y2 [expr int($new_y2*($y2-$y1)+$y1)]
           
      # get the dimensions of the bargraph
      foreach { bar_x2  bar_y2  bar_x1  bar_y1 } [$canvas coords bargraph] {}

      $canvas coords bargraph $bar_x1 $y1 $bar_x2 $new_y2

      return
   }
   
   #*************************************************
   #       Name: Update_reading_color
   # Parameters: widget
   #Description: Update a meter bar on screen
   #*************************************************
   proc Update_reading_color { widget_name  args } \
   {
      set cmd "CanvasMeterbar::Update_reading_color_core $widget_name"
      after cancel $cmd
      after idle $cmd
   }
   proc Update_reading_color_core {widget_name } \
   {
      if ![winfo exists $widget_name] { return }
      set canvas $widget_name.bar
     
      # get the namespace vars
      set namespace [option get $widget_name namespace {}]
      upvar #0 ${namespace}::StateUI StateUI
      $canvas itemconfigure bargraph -fill $StateUI(reading,color)
   }
   #*************************************************
   #       Name: Update_peak
   # Parameters: widget
   #Description: Update a meter bar on screen
   #*************************************************
   proc Update_peak { widget_name  args } \
   {
      set cmd "CanvasMeterbar::Update_peak_core $widget_name"
      after cancel $cmd
      after idle $cmd
   }
   proc Update_peak_core { widget_name } \
   {
      if ![winfo exists $widget_name] { return }
      set canvas $widget_name.bar
     
      # get the namespace vars
      set namespace [option get $widget_name namespace {}]

      upvar #0 ${namespace}::StateUI StateUI
      set reading     $StateUI(peak_reading)
      set max_reading $StateUI(reading,max)
      set min_reading $StateUI(reading,min)
           
      $canvas delete peak

      if {$StateUI(peak_hold) != "ON"} \
      {
         return
      }
     
      # get the dimensions of the bargraph's enclosing box
      foreach { x2 y2 x1 y1 } [$canvas coords bargraph_frame] {}
      
      # convert the reading into a value from 0 to 1
      if [catch \
          {
            set new_y2 [expr double($reading-$min_reading)/($max_reading-$min_reading)]
          } err
         ] \
      {
         # error on values - bail out
         return
      }
      
      # sanity check: clip reading to the range 0..1
      if {$new_y2 < 0.0 } \
      { 
        set new_y2 0.0 
      } \
      elseif {$new_y2 > 1.0 } \
      { 
        set new_y2 1.0 
      }
      
      set new_y2 [expr int($new_y2*($y2-$y1)+$y1)]
      set new_y1 [expr $new_y2-2]
      set new_x1 [expr $x1+5]
           
      $canvas create rect $new_x1 $new_y1 $x2 $new_y2 -tags peak -fill white

      return
   }
   
   #*************************************************
   #       Name: Update_hi_limit
   # Parameters: widget
   #Description: Update the high limit marker
   #*************************************************
   proc Update_hi_limit { widget_name  args } \
   {
      set cmd "CanvasMeterbar::Update_hi_limit_core $widget_name"
      after cancel $cmd
      after idle $cmd
   }
   proc Update_hi_limit_core {widget_name } \
   {
      if ![winfo exists $widget_name] { return }
      set canvas $widget_name.bar
     
      # get the namespace vars
      set namespace [option get $widget_name namespace {}]
      upvar #0 ${namespace}::StateUI StateUI
      
      $canvas delete hi_marker
      $canvas delete hi_value

      if {$StateUI(hi_limit,on) == 0} \
      {
         return
      }
      
      set reading     $StateUI(hi_limit,value)
      set max_reading $StateUI(reading,max)
      set min_reading $StateUI(reading,min)

      # get the dimensions of the bargraph's enclosing box
      foreach {x2 y2 x1 y1} [$canvas coords bargraph_frame] {}
      
      # convert the reading into a value from 0 to 1
      if [catch \
          {
            set new_y2 [expr double($reading-$min_reading)/($max_reading-$min_reading)]
          } err
         ] \
      {
         # error on values - bail out
         return
      }
      
      # sanity check: clip reading to the range 0..1
      if {$new_y2 < 0.0 } \
      { 
        set new_y2 0.0 
      } \
      elseif {$new_y2 > 1.0 } \
      { 
        set new_y2 1.0 
      }
      
      set new_y2 [expr int($new_y2*($y2-$y1)+$y1)]
      set new_y1 [expr int($new_y2-2)]
      set new_x2 [expr int($x2+5)]
           
      if {"[option get $widget_name size {}]" == "big"} \
      {
         $canvas create rect 0 $new_y1 $new_x2 $new_y2 -tags hi_marker -fill red
         incr new_y2 -7
         $canvas create text 0 $new_y2 -fill red -tag hi_value -text $reading -anchor w
      } else \
      {
         $canvas create rect 6 $new_y1 $new_x2 $new_y2 -tags hi_marker -fill red
      }

      return
   }
   #*************************************************
   #       Name: Update_lo_limit
   # Parameters: widget
   #Description: Update the low limit marker
   #*************************************************
   proc Update_lo_limit { widget_name  args } \
   {
      set cmd "CanvasMeterbar::Update_lo_limit_core $widget_name"
      after cancel $cmd
      after idle $cmd
   }
   proc Update_lo_limit_core {widget_name } \
   {
      if ![winfo exists $widget_name] { return }
      set canvas $widget_name.bar
     
      # get the namespace vars
      set namespace [option get $widget_name namespace {}]
      upvar #0 ${namespace}::StateUI StateUI
      
      $canvas delete lo_marker
      $canvas delete lo_value

      if {$StateUI(lo_limit,on) == 0} \
      {
         return
      }
      
      set reading     $StateUI(lo_limit,value)
      set max_reading $StateUI(reading,max)
      set min_reading $StateUI(reading,min)

      # get the dimensions of the bargraph's enclosing box
      foreach { x2 y2 x1 y1 } [$canvas coords bargraph_frame] {}
      
      # convert the reading into a value from 0 to 1
      if [catch \
          {
            set new_y2 [expr double($reading-$min_reading)/($max_reading-$min_reading)]
          } err
         ] \
      {
         # error on values - bail out
         return
      }
      
      # sanity check: clip reading to the range 0..1
      if {$new_y2 < 0.0 } \
      { 
        set new_y2 0.0 
      } \
      elseif {$new_y2 > 1.0 } \
      { 
        set new_y2 1.0 
      }
      
      set new_y2 [expr int($new_y2*($y2-$y1)+$y1)]
      set new_y1 [expr int($new_y2-2)]
      set new_x2 [expr int($x2+5)]
           
      if {"[option get $widget_name size {}]" == "big"} \
      {
         $canvas create rect 0 $new_y1 $new_x2 $new_y2 -tags lo_marker -fill blue
         incr new_y2 +7
         $canvas create text 0 $new_y2 -fill blue -tag lo_value -text $reading -anchor w
      } else \
      {
         $canvas create rect 6 $new_y1 $new_x2 $new_y2 -tags lo_marker -fill blue
      }

      return
   }
}

namespace eval AlertBell \
{
   bind AlertBell <Map>     "AlertBell::Map   %W"
   bind AlertBell <Unmap>   "AlertBell::Unmap %W"
   bind AlertBell <Destroy> "AlertBell::Destroy %W"

   #*************************************************
   #       Name: Map
   # Parameters: widget
   #Description: hook up to auto-update
   #*************************************************
   proc Map { widget_name } \
   {
      set namespace [option get $widget_name namespace {}]
      
      trace variable ${namespace}::StateUI(alarm,state) w "AlertBell::Update $widget_name"
      Update $widget_name
   }
   #*************************************************
   #       Name: Cleanup
   # Parameters: widget
   #Description: cleanup
   #*************************************************
   proc Cleanup { widget_name } \
   {
      set namespace [option get $widget_name namespace {}]
      
      trace vdelete ${namespace}::StateUI(alarm,state) w "AlertBell::Update $widget_name"
      
      after cancel "AlertBell::Update_core $widget_name"
   }
   #*************************************************
   #       Name: Unmap
   # Parameters: widget
   #Description: unhook
   #*************************************************
   proc Unmap { widget_name } \
   {
      Cleanup $widget_name
   }
   #*************************************************
   #       Name: Destroy
   # Parameters: widget
   #Description: unhook
   #*************************************************
   proc Destroy { widget_name } \
   {
      Cleanup $widget_name
   }
   #*************************************************
   #       Name: Update
   # Parameters: widget
   #Description: Schedule an update
   #*************************************************
   proc Update { widget_name  args } \
   {
      after cancel "AlertBell::Update_core $widget_name"
      after idle   "AlertBell::Update_core $widget_name"
   }
   proc Update_core { widget_name } \
   {
      set namespace [option get $widget_name namespace {}]
            
      upvar #0 ${namespace}::StateUI StateUI
      
      if {"[option get $widget_name size {}]" == "big"} \
      {
         switch -- $StateUI(alarm,state) \
         {
            0 {$widget_name configure -image image_bigbelloff -bg gray}
            1 {$widget_name configure -image image_bigbellon -bg grey }
         }
      } \
      else \
      {
         switch -- $StateUI(alarm,state) \
         {
            0 {$widget_name configure -image image_smallbelloff -bg gray }
            1 {$widget_name configure -image image_smallbellon -bg grey }
         }
      }
   }
}

namespace eval StickyButton \
{
   bind StickyButton <Map>     "StickyButton::Map   %W"
   bind StickyButton <Unmap>   "StickyButton::Unmap %W"
   bind StickyButton <Destroy> "StickyButton::Destroy %W"
   
   
   proc Map { widget } \
   {
      set var [option get $widget state_variable {}]
      trace variable $var w "StickyButton::State $widget"
      State $widget
   }
   
   proc Cleanup { widget } \
   {
      set var [option get $widget state_variable {}]
      trace vdelete $var w "StickyButton::State $widget"
   }
   
   proc Unmap { widget } \
   {
      Cleanup $widget
   }
   
   proc Destroy { widget } \
   {
      Cleanup $widget
   }
   
   proc State { widget args } \
   {
      after cancel "StickyButton::State_core $widget"
      after idle "StickyButton::State_core $widget"
   }
   
   proc State_core { widget } \
   {
      set var [option get $widget state_variable {} ]
      upvar #0 $var v
      
      switch -- $v \
      {
         ON -
         on -
         1 -
         TRUE -
         true    { $widget config -relief sunken }
         
         default { $widget config -relief raised }
      }
   }
   
}

namespace eval ColorTrack \
{
   bind ColorTrack <Map>     { ColorTrack::Map %W}
   bind ColorTrack <Unmap>   { ColorTrack::Unmap %W}
   bind ColorTrack <Destroy> { ColorTrack::Destroy %W}
 
   proc Map { widget } \
   {
      set v [option get $widget color_var {}]
      if ![info exists $v] { return }
      trace variable $v w "ColorTrack::Update $widget"
      
      ColorTrack::Update $widget
   }
   
   proc Cleanup { widget } \
   {
      set v [option get $widget color_var {}]
      if ![info exists $v] { return }
      trace vdelete $v w "ColorTrack::Update $widget"  
   }
   
   proc Unmap { widget } \
   {
     Cleanup $widget
   }
   
   proc Destroy { widget } \
   {
     Cleanup $widget
   }
   
   proc Update { widget  args } \
   {
      set cmd "ColorTrack::Update_core $widget"
      after cancel $cmd
      after idle $cmd
   }
   proc Update_core { widget } \
   {
      if ![winfo exists $widget] { return }
      # look if edit open
      upvar #0 ::Entry::entry_data($widget,editon)  editon
      if {[info exists editon] && $editon != 0} { return }
      
      set v [option get $widget color_var {}]
      if ![info exists $v] { return }
      upvar #0 $v value
      
      if ![winfo exists $widget] { return }
      set color [option get $widget color_fg$value {}]
      if {$color == "" } \
      {
         if ![winfo exists $widget] { return }
         set color [option get $widget color_fg_default {}]
      }
      if {$color != "" } \
      {
         if ![winfo exists $widget] { return }
         $widget config -fg $color
      }

      if ![winfo exists $widget] { return }
      set color [option get $widget color_bg$value {}]
      if {$color == "" } \
      {
         if ![winfo exists $widget] { return }
         set color [option get $widget color_bg_default {}]
      }
      if {$color != "" } \
      {
         if ![winfo exists $widget] { return }
         $widget config -bg $color
      }
   }
}
