#****************************************************************************
# IFR SYSTEMS, INC        COPYRIGHT (C)2003      DESIGN ENGINEERING         *
#****************************************************************************
# FILE        : 1152.tSMTPMail.tcl                                          *
# PRODUCT     : 2975-P25                                                    *
# DESCRIPTION : SMTP mail sender, example at end of file                    *
#                                                                           *
#****************************************************************************

namespace eval SMTP \
{
# Scans for Options. opvar is a array with possible options.
#     It may contain defaults. Takes options from args lists
 proc ScanOpts {optsvar} {
  #puts "Got var: $optsvar"
  #set arglist [split [string trim $args {\{\} }]]
  #set args [list $args]

  upvar args args

  if {[llength $args]==1} {
    eval set args $args
  }

  #puts "Got args: $args [lindex $args 3]"

  set i 0
  set count 0

  while { $i < [llength $args]} {
    set token [lindex $args $i]
    #puts "Scan: $token"
    if {[string index $token 0] == "-"} {
      set argname [string range $token 1 end]
      incr i
      set argval [lindex $args $i]
      uplevel set ${optsvar}($argname) \"$argval\"
      #puts "set ${optsvar}($argname) to \"$argval\""
      incr i
      incr count;
      continue
    }
    break
    # no more args!
  }

  # tell me if you understand this...
  return $count
 }


# Private. Build full address.
 proc GetAdr { fulladr } {
    return "<[lindex $fulladr 1]>"
 }

# Private. Returns first part.
 proc GetName { fulladr } {
    return "[lindex $fulladr 0]"
 }

# Private. Builds a full address.
 proc GetFullAdr { fulladr } {
    return "[lindex $fulladr 0] <[lindex $fulladr 1]>"
 }
    
 proc TalkToServer { what should } \
 { 
      upvar sock sock
      upvar op op
      upvar rtime rtime
      upvar tout tout

      puts $sock $what

      flush $sock
      Debug::Puts SMTP 5 "Send: <$what>"

      if {$should != 0} \
      {
        if [catch {
          CheckMailResponse $should
        } e_info] {
          error "Response Err When sending -- $what -- : $e_info"
        }
      }
 }
    # looks for returns within timeout. fblocking might show up strange
    # on some platforms...
  proc CheckMailResponse { should } \
  {
      upvar  op op
      upvar sock sock
      upvar rtime rtime
      upvar tout tout
      set res ""
      set ttime 0
      while 1 { 
          set ttt [time {set res [gets $sock]}]
          if { $res != "" } break

          incr ttime 100
          after 100

          if {$ttime > $tout } {
            error "tSMTPmail: Err When Sending: Exceeded Timeout \
                      of $op(Timeout) Sec."
            break
          }
      }

      incr rtime [expr [lindex $ttt 0]/1000 + $ttime]
      Debug::Puts SMTP 1 "Response: <$res>"
      Debug::Puts SMTP 1 "Response time: $rtime ms"

      if { [lindex $res 0] != $should } {
          Debug::Puts SMTP 1 "Wrong Response: Should be $should, \
                             is $res"
          puts  "Wrong Response: Should be $should, is $res\n"
          error "Wrong Response: Should be $should, is [lindex $res 0]\n"
          return 1
      }

      return 0
 }

# Note how the (one element) list is build. Using list is probably most
#      straighforeward.
#
# NOTE: You may use any allowed header as command line arg, such as 
#       Reply-To etc. But the string must conform exactly to the 
#       standard, means not reply-TO etc. There is not explicit check 
#       for this.
#
# NOTE: The name-address Syntax is fitting to most sendmail setups, in
#       strange cases it might fail. You'll have to dig into the code 
#       then.
#
# option -smtphost Optional. Uses localhost otherwise. IP Addr. or name.
#        -authuser HELO Connection user. Sometimes rebuild as "Sender".
#        -toList,ccList,bccList Lists of Names/Addresses. See Examples.
#        -port Sendmail port
#        -debug 1|0 overrides global ::Debug::enable(SMTP) (if exists). 
#               1 Means verbose debug output on.
#        -Timeout Defaults to 5 Seconds. This is sendmails allowed 
#               response delay. An Error will be thrown otherwise.
#        -blocking 0|1 On windows systems the fblocking call is not 
#               working well; you may disable the timeout check by setting
#               this to 1. Default is 0.
#
# returns -1 on Error , Total Response time if all goes ok.
#
# error   
#       Throws tcl errors if something goes wrong (Unexpected replies 
#       from sendmail or Timeout).
# 
#
# How to Test:
#        Check if the first line of this file points to tclsh8.0, make 
#        it executable and start it. A dialog with sendmail will appear.
#        The code for this test is at the end of the file. Remove it if 
#        you start using it for production!


 proc Send { args } \
 {
    # handles a complete smtp transaction

    #
    # Main Procedure.
    #
    if [catch {
        set rtime 0

        # read options
        set op(smtphost) localhost
        set op(authuser) tSMTPmail      ;# dosen't seem necessary
        set op(from)     ""
        set op(toList)   ""
        set op(debug)    0
        set op(ccList)   ""
        set op(bccList)  ""
        set op(port)     25
        set op(body)     "test message"
        set op(Timeout)  5
        set op(blocking) 0
        ScanOpts     op
        set tout [expr $op(Timeout) * 1000]

        if [info exists ::Debug::enable(SMTP)] \
        {
           set olddebug $::Debug::enable(SMTP)
        } \
        else \
        {
           set olddebug 0
        }
        if {$op(debug) != -1} {
            set ::Debug::enable(SMTP) $op(debug)
        }

        # exclude list for header fields
        set donotsend "Timeout smtphost body authuser port \
                           debug from toList ccList bccList blocking"
        #
        Debug::Puts SMTP 1 "Read Options:"
        Debug::Puts SMTP 1 "[parray op]"
        Debug::Puts SMTP 1 " ---end Options array---"

        set sock [socket $op(smtphost) $op(port)]
        fconfigure $sock -blocking $op(blocking)
        Debug::Puts SMTP 1 "Connected to $op(smtphost):$op(port)"
        CheckMailResponse 220

        # authenticate..
        TalkToServer "HELO $op(authuser)" 250
        TalkToServer "MAIL FROM: <[GetName $op(from)]>" 250

        # to's
        Debug::Puts SMTP 1 "SMTP To's..."
        foreach t $op(toList) {
            TalkToServer "RCPT TO: [GetAdr $t]" 250
        }

        # cc's
        Debug::Puts SMTP 1 "SMPT CC's..."
        foreach t $op(ccList) {
            TalkToServer "RCPT TO: [GetAdr $t]" 250
        }

        # bcc's
        Debug::Puts SMTP 1 "SMPT BCC's..."
        foreach t $op(bccList) {
            TalkToServer "RCPT TO: [GetAdr $t]" 250
        }

        # data
        TalkToServer "DATA" 354
        # header
        Debug::Puts SMTP 1 "Writing Header..."   
        # to's
        Debug::Puts SMTP 1 "Header To's..."
        foreach t $op(toList) {
            TalkToServer "To: [GetFullAdr $t]" 0
        }

        # cc's
        Debug::Puts SMTP 1 "Header cc's..."
        foreach t $op(ccList) {
            TalkToServer "Cc: [GetFullAdr $t]" 0
        }

        # bcc's NOT
        # other fields
        foreach f [array name op] {
            #puts "$donotsend -- [lsearch -exact $donotsend $f] --"
            if {[lsearch -exact $donotsend $f] > -1} {
              continue
            }

            Debug::Puts SMTP 1 "Header $f"
            TalkToServer "$f: $op($f)" 0
        }

        # text
        Debug::Puts SMTP 1 "Text Body..."

        # escape leading .
        TalkToServer "$op(body)" 0

        # close
        TalkToServer "." 250
        TalkToServer "QUIT" 221
        catch {close $sock}

        # restore debug
        set ::Debug::enable(SMTP) $olddebug

        #return 0
      } e_info] {

        catch {close $sock}
        catch {set ::Debug::enable(SMTP) $olddebug}
        error "tSMTPmail: $e_info"
        return -1
      }

      return [expr $rtime / 1000.0]
}
package require DNS

# SMTP::Send_err "tcl error" $bgerrordata(info)
 proc Send_err { subject msg } \
 {
  global sys

  set fr [option get . mailFrom {} ]
  set to [option get . mailTo   {} ]
  
  # Try to find a mail server - First, look for an MX for eng.ifrsys.com.
  # Failing that, use any SMTP server assigned from DHCP.
  # Failing that, look for the MX for ifrsys.com and aeroflex.com.
  # Failing that - return.
  set ip [DNS::Lookup_MX eng.ifrsys.com]
  if {$ip == ""} \
  {
     if [info exists {::DHCP::State(SMTP server)}] \
     {
        set ip "$::DHCP::State(SMTP server)"
        if [info exists "DHCP::State(Domain name)"] \
        {
           set fr "ifr2975@$DHCP::State(Domain name)"
        }
     }
  }
  if {$ip == ""} \
  {
    set ip [DNS::Lookup_MX ifrsys.com]
  }
  if {$ip == ""} \
  {
    set ip [DNS::Lookup_MX aeroflex.com]
  }
  if  {$ip == ""} \
  {
     return
  }
  
  set len [string length $msg]
  set message {}

  #replace " with ' to satisfy tSMTPmail
  #        $ with \$ to prevent variable substitution
  for {set i 0} {$i < $len} {incr i} {
    if { [string match "\"" [string index $msg $i] ] } {
      set message [format "$message'"]
    } elseif { [string match "\$" [string index $msg $i] ] } {
      set message [format "$message\\$"]
    } else {
      set message [format "$message%s" [string index $msg $i]]
    }
  }

  # this turned out to be really important, leading-tailing CR/LF
  set message [format "\n$message\n"]

  # OK, try to send. Since $ip could be a single IP address or a list of MX's, iterate
  foreach server $ip \
  {
     if {[llength $server] == 3} \
     {
        set server [lindex $server 2]
     }
     if [oldCatch {Send -smtphost $server -from $fr -toList $to -Subject $subject -body  $message} err] \
     {
        puts "Error sending to $server: $err"
     } \
     else \
     {
        break
     }
  }
 }
}

