# bgerror.tcl --
# FITS\Software\tcl\tk8.0.4\library\bgerror.tcl
# A modified version to support sending an EMAIL of error.

# Copyright (c) 1992-1994 The Regents of the University of California.
# Copyright (c) 1994-1996 Sun Microsystems, Inc.
#
# See the file "license.terms" for information on usage and redistribution
# of this file, and for a DISCLAIMER OF ALL WARRANTIES.


# bgerror --
# It tries to execute tkerror, if that fails it posts a dialog box containing
# the error message and gives the user a chance to ask to see a stack
# trace.
# Arguments:
# err -        The error message.


global bgerrordata
set ::tkPriv(button) -2

bind Dialog <Map> \
{

   set screen [winfo screen %W]
   set dot [winfo screen .]
   if {$dot == $screen} \
   {
      grab release .
      oldCatch {grab -global %W}
   } \
   else \
   {
      oldCatch {grab %W}
   }
   safe_focus_force %W
}
bind Dialog <Unmap> \
{
   grab release %W
}

# replace Tk's normal dialog function with one better bound for our interface.
proc tk_dialog {w title text bitmap default args} {
    global tkPriv tcl_platform
    
    set old_button ""
    if [info exists tkPriv(button)] \
    {
       set old_button $tkPriv(button)
    }

    # 1. Create the top-level window and divide it into top
    # and bottom parts.

    catch {destroy $w}
    toplevel $w -class Dialog
    
    set parent [winfo parent $w]
    set old_focus [focus -displayof $parent]

    wm iconbitmap  $w @/system/logo/logo.xbm
    wm title $w $title
    wm iconname $w Dialog

    # The following command means that the dialog won't be posted if
    # [winfo parent $w] is iconified, but it's really needed;  otherwise
    # the dialog can become obscured by other windows in the application,
    # even though its grab keeps the rest of the application from being used.

    wm transient $w [winfo toplevel [winfo parent $w]]

    frame $w.bot
    frame $w.top
    $w.bot configure -relief raised -bd 1
    $w.top configure -relief raised -bd 1
    pack $w.bot -side bottom -fill both
    pack $w.top -side top -fill both -expand 1

    # 2. Fill the top part with bitmap and message (use the option
    # database for -wraplength so that it can be overridden by
    # the caller).

    option add *Dialog.msg.wrapLength 3i widgetDefault
    label $w.msg -justify left -text $text
    $w.msg configure -font {Times 18}
    pack $w.msg -in $w.top -side right -expand 1 -fill both -padx 3m -pady 3m
    if {$bitmap != ""} \
    {
	  label $w.bitmap -bitmap $bitmap
	  pack $w.bitmap -in $w.top -side left -padx 3m -pady 3m
    }

    # 3. Create a row of buttons at the bottom of the dialog.
    set ::tkPriv(button) -2
    set i 0
    foreach but $args \
    {
	  button $w.button$i -text $but -command "set ::tkPriv(button) $i"
	  add_bindtag $w.button$i Cursor Softkey
	  if {$i == $default} \
	  {
	    $w.button$i configure -default active
	  } \
	  else \
	  {
	    $w.button$i configure -default normal
	  }
	  grid $w.button$i -in $w.bot -column $i -row 0 -sticky ew -padx 10
	  grid columnconfigure $w.bot $i
	  set prev_button $w.button$i
	  incr i
    }
    # 4. there is no step 4

    # 5. Create a <Destroy> binding for the window that sets the
    # button variable to -1;  this is needed in case something happens
    # that destroys the window, such as its parent window being destroyed.

    bind $w <Destroy> {set ::tkPriv(button) -1}
    bind $w <KeyRelease-F8> "destroy $w"
    
    # 6. Withdraw the window, then update all the geometry information
    # so we know how big it wants to be, then center the window in the
    # display and de-iconify it.

    wm withdraw $w

    set x [expr {[winfo screenwidth $w]/2 - [winfo reqwidth $w]/2 \
	    - [winfo vrootx [winfo parent $w]]}]
    set y [expr {[winfo screenheight $w]/2 - [winfo reqheight $w]/2 \
	    - [winfo vrooty [winfo parent $w]]}]
    wm geom $w +$x+$y
    wm deiconify $w

    # 7. Set a grab and claim the focus too.
    set default_focus $w
    if {$default >= 0} \
    {
	  set default_focus $w.button$default
    }

    # 8. Wait for the user to respond, then restore the focus and
    # return the index of the selected button.  Restore the focus
    # before deleting the window, since otherwise the window manager
    # may take the focus away so we can't redirect it.  Finally,
    # restore any grab that was in effect.
    update idle
    set ::tkPriv(button) -2
    while {$::tkPriv(button) == -2 } \
    {
       if ![winfo exists $w] \
       {
          break
       }
       # force focus IF AND ONLY IF this is the local display
       set local [winfo screen .]
       set me [winfo screen $w]
       if {$local == $me } \
       {
         set x [focus -displayof $w]
       
         if ![string match $w* $x] \
         {
            focus $default_focus
         }
       }
       after 100
    }
    puts "button is $::tkPriv(button)"
    set ret_val $::tkPriv(button)

    catch {
	# It's possible that the window has already been destroyed,
	# hence this "catch".  Delete the Destroy handler so that
	# tkPriv(button) doesn't get reset by it.
	    if [winfo exists $w] \
	    {
		   bind $w <Destroy> {}
		   destroy $w
		}
    }
    set tkPriv(button) $old_button
    safe_focus_force $old_focus
    return $ret_val
}
proc send_err_email {} \
{
   set header "Tcl Error from [IFR_options man] [IFR_options model] #[IFR_options ser] $::sys(ip_name)"
   set msg "$header\n Error was:\n$::bgerrordata(info)"
   ::SMTP::Send_err $header $msg
   return
}

proc bgerror err {
   global errorInfo tcl_platform
   global bgerrordata
   global sys

   set screen [option get . debugScreen {} ]
   set auto_trace [option get . debugAutoTrace {} ]

   # save errorInfo which would be erased in the catch below otherwise.
   set bgerrordata(info) $errorInfo
   set info $errorInfo;

   # For backward compatibility :
   # Let's try to execute "tkerror" (using catch {tkerror ...} 
   # instead of searching it with info procs so the application gets
   # a chance to auto load it using its favorite "unknown" mecanism.
   # (we do the default dialog only if we get a TCL_ERROR (=1) return
   #  code from the tkerror trial, other ret codes are passed back
   #  to our caller (tcl background error handler) so the called "tkerror"
   #  can still use  return -code break, to skip remaining messages
   #  in the error queue for instance)  -- dl
   if {"[info procs ::tkerror]" != "" } \
   {
     set ret [catch {::tkerror $err} msg];
     if {$ret != 1} {
        return -code $ret $msg
     }
   }

   set bgerrordata(info) $info


   # Write the 'info' messsage to the error log
   ErrLog::Log  ""
   ErrLog::Log  "################  TCL STACK TRACE  ################"
   ErrLog::Log  "$bgerrordata(info)"
   ErrLog::Log  ""
   
   
   # Ok the application's tkerror either failed or was not found
   # we use the default dialog then :

   if {"$auto_trace" == "" } {
   
      set count 0
      while [winfo exists .bgerrorDialog$count] \
      {
         incr count
      }
      set button [tk_dialog .bgerrorDialog$count "Error in Tcl Script" \
               "Error: $err" error 0 OK "Skip Messages" "Stack Trace"]

     if {$button == 0} {
        return
     } elseif {$button == 1} {
        return -code break
     } elseif {$button == -1} {
        return -code break
     } elseif {$button != 2} { return }
   } elseif {"$auto_trace" == "mail" } \
   {
      send_err_email
      return
   }

   set count 0
   while [winfo exists .bgerrorTrace$count] \
   {
      incr count
   }
   set w .bgerrorTrace$count
   catch {destroy $w}

   toplevel $w -class ErrorTrace -screen $screen
   wm minsize $w 1 1
   wm title $w "Stack Trace for Error"
   wm iconname $w "Stack Trace"
   wm iconbitmap  $w @/system/logo/logo.xbm


   frame  $w.bottom
   button $w.bottom.ok    -text OK    \
                          -command "destroy $w"  \
                          -default active
   button $w.bottom.email -text EMAIL \
                          -command {send_err_email} \
                          -default active

   text $w.text -relief sunken -bd 2 -yscrollcommand "$w.scroll set" \
                   -setgrid true -width 60 -height 15 -takefocus 0


   scrollbar $w.scroll -relief sunken -command "$w.text yview"

   pack $w.bottom.ok    -side left -padx 3m -pady 2m
   pack $w.bottom.email -side left -padx 3m -pady 2m
   pack $w.bottom       -side bottom

   pack $w.scroll -side right -fill y
   pack $w.text   -side left  -expand yes -fill both

   $w.text insert 0.0 $info
   $w.text mark set insert 0.0

   bind $w <Return> "destroy $w"
   bind $w.text <Return> "destroy $w; break"

   # Center the window on the screen.

   wm withdraw $w


   set x [expr {[winfo screenwidth $w]/2 - [winfo reqwidth $w]/2 \
                            - [winfo vrootx [winfo parent $w]]}]
   set y [expr {[winfo screenheight $w]/2 - [winfo reqheight $w]/2 \
                            - [winfo vrooty [winfo parent $w]]}]

   #wm geom $w +$x+$y
   wm geom $w +30+20
   wm deiconify $w
   safe_focus_force $w.bottom.ok 

}
