#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C)1998,99      DESIGN ENGINEERING      #
# --------------------------------------------------------------------------#
# FILE        : to_rfgen.tcl                                                #
# PRODUCT     : FITS                                                        #
# AUTHOR      : Steve Johnson                                               #
# DESCRIPTION : Interface to RF Gen via Message Bus                         #
#                                                                           #
#############################################################################
package require Tclmsgbus
#
#  RF Gen System Number
#
set RFGEN_ENGINE(SYSTEM_ID)           10

#
#  RF Gen Sub-Systems (general operation)
#
set RFGEN_ENGINE(SUB_UPDATE_YIG_CAL)  10

#
#  RF Gen Sub-Systems (calibration)
#


set RFGEN_ENGINE(SUB_SPLINEINFO)      33
    

#
#  RF Generator Call-back Data
#

set RFGEN_STATUS(yig_hold)            -1
set RFGEN_STATUS(yig_dac)             -1
set RFGEN_STATUS(yig_padetect)        -1
set RFGEN_STATUS(spline)              -1

set Rfgen_Handler(spline)                0

#
#  Calibration Globals
#
set RFGEN_ENGINE(Filename)                 junk

set RFGEN_ENGINE(Port)                     GEN
set RFGEN_ENGINE(Step)                     0
set RFGEN_ENGINE(AlcDac)                   0

set RFGEN_ENGINE(Normalize.Genalc)        -10.00
set RFGEN_ENGINE(Normalize.Genfreq.dB0)   -30.00
set RFGEN_ENGINE(Normalize.Genfreq.dB10)  -40.00
set RFGEN_ENGINE(Normalize.Genfreq.dB20)  -50.00

set RFGEN_ENGINE(AlcIndex)                 0
set RFGEN_ENGINE(AlcMaxIndex)              16

set RFGEN_ENGINE(FreqIndex)                0
set RFGEN_ENGINE(FreqMaxIndex)             59


for {set i 0} {$i < $RFGEN_ENGINE(AlcMaxIndex)} {incr i} {
  set GENCAL(level,abs,$i) 0
  set GENCAL(level,normalize,$i) 0  
  set GENCAL(level,alc,$i) 0
  set GENCAL(level,status,$i) UNCAL
  set GENCAL(level,padetect,$i) 0
}
  
for {set i 0} {$i < $RFGEN_ENGINE(FreqMaxIndex)} {incr i} {
  foreach port {TR GEN} {
    foreach atten {dB0 dB10 dB20} {
      set GENCAL(freq,$port,$atten,abs,$i) 0
      set GENCAL(freq,$port,$atten,normalize,$i) 0      
      set GENCAL(freq,$port,$atten,status,$i) UNCAL
      set GENCAL(freq,$port,$atten,padetect,$i) 0
    }
  }
}
  
  

set SET_TOP_OF_TABLE_RECOMPUTE 255

#
#  RF Gen Font Declaration
#
font create rfgen_font -family helvetica -size 8

#
#  RF Gen Yig Caligration Data
#
set RfGenYigCals(temp)                "Initial"
set RfGenYigCals(0)                    100
set RfGenYigCals(2800)                 2651


#############################################################################
#                                                                           #
#                       Message Bus Interfaces                              #
#                                                                           #
#############################################################################

#
#  Message bus callback - fill in supplied structure
#
#       type - 1 ALC Correction Curve
#              2 T/R Freq Correction Curve
#              3 GEN Freq Correction Curve
#         with atten included:
#           atten - for FREQ curve, use atten ( 0=0db, 16=10db, 32=20db )
#
#       thing1 - value sent to engine, freq or dB
#              for ALC curve, dB
#              for FREQ curve, Freq in MHz
#
#       point - returned dac value
#

proc handle_rfgen_spline {sys sub lun flags uid type thing point } {
  global RFGEN_STATUS
  global GENCAL

  # only look at status message
  if { $flags & 4 } {
    set RFGEN_STATUS(spline)     $point

   # IF QUERY, PERFORM QUERY COMMAND
    if {$GENCAL(querytype) == $type && $GENCAL(queryvalue) == $thing} {

    set point [expr int($point)]
      set command $GENCAL(querycommand)
      append command $point

      eval $command
      set GENCAL(querytype) none
      set GENCAL(queryvalue) none
    
    } else {

      if {$GENCAL(querytype) == $type} {
        puts "types equaled"
      } else {
        puts "types not match: querytype is $GENCAL(querytype), type is $type"
    }    
    if {$GENCAL(queryvalue) == $thing} {
      puts "thing matched value"
    } else {
      puts "values not matched: queryvalue is $GENCAL(queryvalue), value is $thing"
    }
   }
  }
}


#############################################################################
#                                                                           #
#                       Engine Interfaces                                   #
#                                                                           #
#############################################################################

#############################################################################
#                                                                           #
#                       Calibration Interfaces                              #
#                                                                           #
#############################################################################

#############################################################################
#                                                                           #
#                       Auxilary Files                                      #
#                                                                           #
#############################################################################

source /system/utilities/aux_files/to_gencal.tcl



#############################################################################
#                                                                           #
#                       Callback                                            #
#                                                                           #
#############################################################################

#
#  Message bus interfaces for messages being generated FROM rfgen engine
#
proc create_rfgen_callback { } {
  global RFGEN_ENGINE
  global Rfgen_Handler


  if {$Rfgen_Handler(spline) == 0} {
    set Rfgen_Handler(spline) [tclMsgbus rcv handle \
                             -sys $RFGEN_ENGINE(SYSTEM_ID) \
                             -sub $RFGEN_ENGINE(SUB_SPLINEINFO) \
                             -lun 1 -cmd "handle_rfgen_spline" -mask 16]
  }
}



#
#  Message bus interfaces - removed
#
proc destroy_rfgen_callback { } {
  global Rfgen_Handler

  if {$Rfgen_Handler(spline) != 0 } {
    tclMsgbus $Rfgen_Handler(spline) destroy
  }

  set Rfgen_Handler(spline)    0
}


#############################################################################
#                                                                           #
#               Init Module                                                 #
#                                                                           #
#############################################################################
proc set_rfgen_level_format { dbm } {
   global rfgen

   # OK, if the value we get back from the engine matches our setting, don't
   # recompute the uV and mV values. This prevents the user from entering
   # 20.00 uV and getting back 19.93
   if {$dbm != $rfgen(leveldbm)} {
     set rfgen(leveldbm) $dbm
     set rfgen(leveluV)  [convert_dBm_uV $dbm]
     set rfgen(levelmV)  [convert_dBm_mV $dbm]
     set rfgen(levelmW)  [convert_dBm_mW $dbm]
   }
   
   switch -- $rfgen(levelunit) {
      "dBm"    {
               set rfgen(rflevel)  [format "%3.1f" $rfgen(leveldbm)]
               }
      "uV"     {
                set rfgen(rflevel) [format "%1.2f" $rfgen(leveluV)]
               }
      "mV"     {
                set rfgen(rflevel) [format "%1.4 $rfgen(levelmV)]
               }
      "mWatts" {
                set rfgen(rflevel) [format "%1.1 $rfgen(levelmW)]
               }
   }
}

 proc start_rfgen_module { } {
   global GENCAL
   gencal_alc_read_cal    rfgen_alc
   
   # if this is a cal for new hardware, do not do temp compensation
   set temp_comp_flag 1
   switch -- $GENCAL(hwversion) \
   {
      0 {set temp_comp_flag 0}
      1 {set temp_comp_flag 1}
      2 {set temp_comp_flag 0}
      default {set temp_comp_flag 0}
   }

   gencal_set_engine_level $::GENCAL(level.caltemp) $temp_comp_flag

   gencal_freq_read_cal   rfgen_freq_tr  TR
   gencal_set_engine_freq TR

   gencal_freq_read_cal   rfgen_freq_gen GEN
   gencal_set_engine_freq GEN
   get_mixer_router_cal_data bb_audio_lvl

   create_rfgen_callback

 }
 Load::on_system_done \
{
   start_rfgen_module
}

