#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C)1998,99      DESIGN ENGINEERING      #
# --------------------------------------------------------------------------#
# FILE        : 1380.to_anlzr.tcl                                           #
# PRODUCT     : FITS                                                        #
# AUTHOR      : Steve Johnson                                               #
# DESCRIPTION : Interface to Channel Analyzer via Message Bus               #
#                                                                           #
#############################################################################
package require Analyzer
Analyzer ver 0
#############################################################################
#                                                                           #
#                       Engine Interfaces                                   #
#                                                                           #
#############################################################################

#
#  Set Anlzr Calibration Data
#
#           type  - cal data type to send to engine
#           index - 1 to 6
#                   1 to 3 is channel bw ( 300, 3k, 60k )
#                   4 to 6 is main bw ( 30k, 300k, 3M )
#           khz   - is amount of freq to change the YIG or DDS
#                      to center the trace because of filter
#           db    - move screen up/down this number of dB (+/- 0.1)
#           watts - power term reading in watts
#
proc do_anlzr_cals { type index khz db watts } {


  switch $type {
    header   { set itype 1  }
    bwswitch { set itype 2  }
    cf       { set itype 3  }
    tr       { set itype 4  }
    ant      { set itype 5  }
    cfsize   { set itype 12 }
    trsize   { set itype 13 }
    antsize  { set itype 14 }
    mainifgain { set itype 51 }
    chifgain { set itype 52 }
    delnorm  { set itype 100 }
    default  { puts "do_anlzr_cals err: $type"; return }
  }

  Analyzer cal $itype $index $khz $db $watts
}

#
#  Set IF/Video Gain scaling
#
proc do_ifvideo_cals { type db dac } {
  puts "do_ifvideo_cal DEPRECATED - use Analyzer cal"
  CallTrace
  do_anlzr_cals $type $dac 0.0 $db 0.0
}


#
#  Delete Normalize Calibration File
#
proc del_normalize_calibrations { } {
  puts "del_normalize_calibrations DEPRECATED - use Analyzer cal"
  CallTrace

  do_anlzr_cals delnorm 0 0.0 0.0 0.0

  set code [ catch {
    file delete -force /usr/analyzer/AnalyzerTweeks
  } err ]

  if { $code } {
    puts "open_event_log failure: $err"
    return
  }
}


#
#  Reset Gain Calibration File
#
proc reset_gain_calibrations { } {

  set code [ catch {
    file delete -force  /system/calibration/ifvideo
    get_anlzr_ifgain_data ifvideo
  } err ]

  if { $code } {
    puts "open_event_log failure: $err"
    return
  }
}


#
#  Set Anlzr Save/Recall Data
#
#           type      - SAVE, RECALL
#           operation - SETREF, CLEARREF
#                  
proc set_anlzr_save_recall { type operation } {

  set code [file isdirectory "/usr/analyzer"]

  if { $code == 0 } {
    set code [ catch {
       file mkdir /usr/analyzer
    } err ]

    if { $code } {
      puts "set_anlzr_save_recall (mkdir) $err"
      return
    }
  }

  switch $type {
    SAVE     { set itype 1  }
    RECALL   -
    DONTCARE { set itype 0  }
    default  { return }
  }

  switch $operation {
    SETREF   { set ioperation 1  }
    CLEARREF { set ioperation 2  }
    USEREF   { set ioperation 3  }
    default  { return }
  }

  Analyzer save $itype $ioperation
}


#############################################################################
#                                                                           #
#                       Calibration Interfaces                              #
#                                                                           #
#############################################################################

set AnlzrCals(cfmax)   0
set AnlzrCals(max)     0
set AnlzrCals(date)   -1
set AnlzrCals(time)   -1

set AnlzrCals(1.khz)  -1
set AnlzrCals(1.norm) -1
set AnlzrCals(2.khz)  -1
set AnlzrCals(2.norm) -1
set AnlzrCals(3.khz)  -1
set AnlzrCals(3.norm) -1
set AnlzrCals(4.khz)  -1
set AnlzrCals(4.norm) -1
set AnlzrCals(5.khz)  -1
set AnlzrCals(5.norm) -1
set AnlzrCals(6.khz)  -1
set AnlzrCals(6.norm) -1

#
#   Parse Header Info
#
#
proc parse_anlz_header { line} {
  global AnlzrCals
 
  set AnlzrCals(date)   [ lindex $line 2 ]
  set AnlzrCals(time)   [ lindex $line 4 ]

  do_anlzr_cals header 0 0.0 0.0 0.0
}

#
#  Parse Switching Cal Data entry
#
#
proc parse_anlz_cal { line } {
  global AnlzrCals
 
  set index [ lindex $line 2 ]
  set khz   [ lindex $line 4 ]
  set norm  [ lindex $line 6 ]

  set AnlzrCals($index.khz)   $khz
  set AnlzrCals($index.norm)  $norm

  do_anlzr_cals bwswitch $index $khz $norm 0.0
}

#
#  Parse Cross-Feed Cal Data entry
#
#
proc parse_anlz_cf_cal { line } {
  global AnlzrCals
 
  set index [ lindex $line 2 ]
  set mhz   [ lindex $line 4 ]
  set dbm   [ lindex $line 6 ]
  set watts [ lindex $line 8 ]

  set AnlzrCals($index.cf.mhz)   $mhz
  set AnlzrCals($index.cf.dbm)   $dbm
  set AnlzrCals($index.cf.watts) $watts

  do_anlzr_cals cf $index $mhz $dbm $watts
}

#
#  Parse Input Signal Path Loss entry
#
#
proc parse_anlz_input_cal { type line } {
  global AnlzrCals
 
  set index [ lindex $line 2 ]
  set mhz   [ lindex $line 4 ]
  set db    [ lindex $line 6 ]

  set AnlzrCals($index.$type.mhz)   $mhz
  set AnlzrCals($index.$type.db)    $db

  do_anlzr_cals $type $index $mhz $db 0.0
}


#
# update tcl anlzr cal data
#
proc get_anlzr_cal_data { filename } {
  global AnlzrCals

  set file [ OpenCalFile $filename r ]

  while { [ gets $file line ] >= 0 } {
# puts $line
    if { [ lindex $line 0 ] == "ANALYZER_CHARACTERISTICS" } {
      parse_anlz_header $line
    }
    if { [ lindex $line 0 ] == "ANLZR_BW_SW_CAL_MAX" } {
      set AnlzrCals(max)    [ lindex $line 1 ]
    }
    if { [ lindex $line 0 ] == "ANLZR_BW_SW_CAL" } {
      parse_anlz_cal $line
    }
    if { [ lindex $line 0 ] == "ANLZR_FREQ_CF_CAL_MAX" } {
      set AnlzrCals(cfmax)  [ lindex $line 1 ]
      do_anlzr_cals cfsize $AnlzrCals(cfmax) 0.0 0.0 0.0
    }
    if { [ lindex $line 0 ] == "ANLZR_FREQ_CF_CAL" } {
      parse_anlz_cf_cal $line
    }
    if { [ lindex $line 0 ] == "ANLZR_FREQ_TR2REC_CAL_MAX" } {
      set AnlzrCals(trmax)  [ lindex $line 1 ]
      do_anlzr_cals trsize $AnlzrCals(trmax) 0.0 0.0 0.0
    }
    if { [ lindex $line 0 ] == "ANLZR_FREQ_TR2REC_CAL" } {
      parse_anlz_input_cal tr $line
    }
    if { [ lindex $line 0 ] == "ANLZR_FREQ_ANT2REC_CAL_MAX" } {
      set AnlzrCals(antmax)  [ lindex $line 1 ]
      do_anlzr_cals antsize $AnlzrCals(antmax) 0.0 0.0 0.0
    }
    if { [ lindex $line 0 ] == "ANLZR_FREQ_ANT2REC_CAL" } {
      parse_anlz_input_cal ant $line
    }
  }

  close $file
}


#
# update tcl if/video cal data
#
proc get_anlzr_ifgain_data { filename } {
  global AnlzrCals

  set file [ OpenCalFile $filename r ]

  while { [ gets $file line ] >= 0 } {
# puts $line
    if { [ lindex $line 0 ] == "IFVIDEO_CHARACTERISTICS" } {
    }
    if { [ lindex $line 0 ] == "IFVIDEO_CH_GAIN_CAL_MAX" } {
      set AnlzrCals(ifvideo.chmax)    [ lindex $line 1 ]
    }
    if { [ lindex $line 0 ] == "IFVIDEO_CH_GAIN_CAL" } {
      set index [ lindex $line 2 ]
      set AnlzrCals(ifvideo.ch$index.db)   [ lindex $line 4 ]
      set AnlzrCals(ifvideo.ch$index.dac)  [ lindex $line 6 ]
      do_ifvideo_cals chifgain [ lindex $line 4 ] [ lindex $line 6 ]
    }
    if { [ lindex $line 0 ] == "IFVIDEO_MAIN_GAIN_CAL_MAX" } {
      set AnlzrCals(ifvideo.mainmax)  [ lindex $line 1 ]
    }
    if { [ lindex $line 0 ] == "IFVIDEO_MAIN_GAIN_CAL" } {
      set index [ lindex $line 2 ]
      set AnlzrCals(ifvideo.main$index.db)   [ lindex $line 4 ]
      set AnlzrCals(ifvideo.main$index.dac)  [ lindex $line 6 ]
      do_ifvideo_cals mainifgain [ lindex $line 4 ] [ lindex $line 6 ]
    }
  }

  close $file
}



#
# update tcl anlzr cal data - write current data
#
proc put_anlzr_cal_data { filename } {
  global AnlzrCals

  set file [ OpenCalFile $filename w ]

  set AnlzrCals(date) [clock format [clock seconds] -format %m/%d/%Y]
  set AnlzrCals(time) [clock format [clock seconds] -format %H:%M]
  puts $file "#"
  puts $file "#  Spectrum Analyzer Calibration Table"
  puts $file "#====================================================================="
  set msg [ format "ANALYZER_CHARACTERISTICS   DATE %10s TIME %10s" \
                    $AnlzrCals(date) $AnlzrCals(time) ]
  puts $file $msg
  puts $file " "

  # normalize cal data - normalization occures using the middle filter
  set temp $AnlzrCals(2.norm)
  set AnlzrCals(1.norm) [ expr $AnlzrCals(1.norm) - $temp ]
  set AnlzrCals(2.norm) 0.0
  set AnlzrCals(3.norm) [ expr $AnlzrCals(3.norm) - $temp ]

  set AnlzrCals(4.norm) [ expr $AnlzrCals(4.norm) - $temp ]
  set AnlzrCals(5.norm) [ expr $AnlzrCals(5.norm) - $temp ]
  set AnlzrCals(6.norm) [ expr $AnlzrCals(6.norm) - $temp ]

  set msg [ format "ANLZR_BW_SW_CAL_MAX    %3d" $AnlzrCals(max) ]
  puts $file $msg
  for {set i 1 } { $i <= $AnlzrCals(max) } { incr i } { 
    set msg [ format "ANLZR_BW_SW_CAL  INDEX %2d  FREQADJ %7.3f  NORMALIZE %7.2f" \
               $i $AnlzrCals($i.khz) $AnlzrCals($i.norm) ]
    puts $file $msg
  }

  puts $file " "

  set msg [ format "ANLZR_FREQ_CF_CAL_MAX    %3d" $AnlzrCals(cfmax) ]
  puts $file $msg
  set i [ expr $AnlzrCals(cfmax) -1 ]
  for { } { $i >= 0 } { incr i -1 } { 
    set msg [ format "ANLZR_FREQ_CF_CAL INDEX %2d  FREQ %8.3f  RCVRPWR %6.2f  PT %7.4f" \
     $i $AnlzrCals($i.cf.mhz) $AnlzrCals($i.cf.dbm) $AnlzrCals($i.cf.watts)]
    puts $file $msg
  }

  puts $file " "

  set msg [ format "ANLZR_FREQ_TR2REC_CAL_MAX    %3d" $AnlzrCals(trmax) ]
  puts $file $msg
  set i [ expr $AnlzrCals(trmax) -1 ]
  for { } { $i >= 0 } { incr i -1 } { 
    set msg [ format "ANLZR_FREQ_TR2REC_CAL  INDEX %2d  FREQ %8.3f  LOSS %7.2f" \
     $i $AnlzrCals($i.tr.mhz) $AnlzrCals($i.tr.db)]
    puts $file $msg
  }

  puts $file " "

  set msg [ format "ANLZR_FREQ_ANT2REC_CAL_MAX    %3d" $AnlzrCals(antmax) ]
  puts $file $msg
  set i [ expr $AnlzrCals(antmax) -1 ]
  for { } { $i >= 0 } { incr i -1 } { 
    set msg [ format "ANLZR_FREQ_ANT2REC_CAL INDEX %3d  FREQ %8.3f  LOSS %7.2f" \
     $i $AnlzrCals($i.ant.mhz) $AnlzrCals($i.ant.db)]
    puts $file $msg
  }


  flush $file
  close $file
}



#*************************************************
#       Name: start_anlz_module
# Parameters: 
#Description: starts rec
#*************************************************
global start_anlz_module_state
set start_anlz_module_state 0
proc start_anlz_module { } \
{
   global start_anlz_module_state

   #only do this once
   if { $start_anlz_module_state } { 
     return
   } else {
     set start_anlz_module_state 1
   }
   

}


Load::on_system_done \
{
  get_anlzr_cal_data analyzer
}
