#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C)1998-2001      DESIGN ENGINEERING    #
# --------------------------------------------------------------------------#
# FILE        : download_html.tcl                                           #
# PRODUCT     : FITS                                                        #
# AUTHOR      : David Hagood/Hieu Tran                                      #
# DESCRIPTION : Setup of web browser for download reload file               #
#                                                                           #
#############################################################################
package require DNS

if [is_vxworks] {
package require Tkhtml
package require http
package require base64
}
bind HtmlClip <Motion>         "::HtmlFrame::mouseover_url %W %x %y"
bind HtmlClip <1>              "::HtmlFrame::do_hyperlink  %W %x %y "
bind HtmlFrame <Right>         "::HtmlFrame::next_link     %W  1; break"
bind HtmlFrame <Left>          "::HtmlFrame::next_link     %W -1; break"
bind HtmlFrame <Down>          "::HtmlFrame::yview         %W  1 unit; break"
bind HtmlFrame <Up>            "::HtmlFrame::yview         %W -1 unit; break"
bind HtmlFrame <Return>        "::HtmlFrame::pick          %W; break"
bind HtmlFrame <Key-F8>        "break"
bind HtmlFrame <KeyRelease-F8> "::HtmlFrame::Close         %W; break"
bind HtmlFrame_MAIN <Destroy>       "::HtmlFrame::Destroy       %W"
bind HtmlFrame <<ModeDown>>   "break;"
bind HtmlFrame <<Mode>>       "break;"
bind HtmlFrame <<OptionDown>> "break;"
bind HtmlFrame <<Option>>     "break;"
bind HtmlWindow <Map> \
{
     if {[winfo screen .] == [winfo screen %W]} \
     {
        after idle "wm geo %W 640x480+0+0"
     }
     HtmlFrame::Focus %W
}
bind HtmlFrame <FocusIn> \
{
   HtmlFrame::Focus %W
}

foreach { key button } { F2 b1 F3 b2 F4 b3 F5 b4 F6 b5 F7 b6 } {
  bind HtmlFrame <Key-$key>        " 
     event generate \[::HtmlFrame::frame_of %W\].$button <<Press>>
     break
  "
  bind HtmlFrame <KeyRelease-$key> "
     event generate \[::HtmlFrame::frame_of %W\].$button <<Release>>
     break 
  "
}
  
  #################################################################
  # set our user agent and accept strings...
  #################################################################
  http::config -useragent "IFR 2975" -proxyfilter ::HtmlFrame::proxy \
  -accept {text/*,image/gif,/image/x-MS-bmp,application/x-tar,application/x-gzip}
  
namespace eval ::HtmlFrame {

  set indexes {index.tcl index.shtml index.html Index.html index.htm Index.htm index.txt}
  namespace export encode decode split_url unsplit_url do_dir
  
  variable default_page "file://localhost/sysdata/html/help/"
  variable notfound_page "file://localhost/sysdata/html/notfound.html"
  variable html_base file://localhost/sysdata/html/
  variable err_base  $html_base/errors
  
  variable htmlscreen
 
  variable http_image_count 0
  variable html_frame_count 0
  
  namespace export split_url
  
  set local_hosts [list dumas1 localhost patch ]
  
  variable error_in_progress 0
  
  # create an unmapping table for de-encoding URLs
  for {set i 0} { $i < 256} { incr i } {
   set c [format %c $i]
   set x %[format %.2x $i]
   set unmap($x) $c
   set map($c) $x
  }
  
  set htmlscreen(start_html) \
  {<html><head><title>Please wait</title></head>
     <body>
     <center><font size="+6"><b>Please wait...</b></font></center>
     </body>
     </html>
  }
  
  set htmlscreen(finding) \
  {<html><head><title>Please wait</title></head>
     <body>
     <center><font size="+6"><b>Please wait...</b></font></center>
     <font size="+6"><b>Attempting to retrieve $safer_url</b></font>
     </body>
     </html>
  }
  
  #################################################################
  # find_title - get the <title> block of a file
  #################################################################
  proc find_title { fd } \
  {
   upvar dont_index_flag dont_index_flag
   set title {no title given}
   if [catch \
   {
     # read the first K of the file. If the title ain't there, screw it!
     set l [read $fd 1024]
     # look for title string
     set pattern {.*<head.*>.*<title.*>[ ]*(\[IFR_options long\] Help:)*[ ]*(.*)</title.*>.*</head.*>}
     regexp -nocase $pattern $l v x title
     # look for files we are not to index
     if [info exists dont_index_flag] \
     {
        if {[regexp -nocase {<!-- DO NOT INDEX -->} $l] != 0} \
        {
           set dont_index_flag 1
        }
     }
   } err ] \
   {
      return ""
   }
   return $title
  }
  #################################################################
  # find_file_title - get the <title> block of a file
  #################################################################
  proc find_file_title { file } \
  {
   upvar dont_index_flag dont_index_flag
   if ![file isfile $file] { return ""}
   if [catch {set fd [open $file ]} err ] \
   {
      return ""
   }
   set title [find_title $fd]
   close $fd
   return $title
  }

  #################################################################
  # do_dir - look for app notes and such within a directory
  #################################################################
  proc do_dir { title dir } \
  {
     variable indexes
  set dirs [glob -nocomplain $dir/*]

  if {$dirs == "" } { return }

  set y ""
  foreach app $dirs \
  {
    set app_encoded [encode $app]
    if [file isdir $app] \
    {
       set index       [find_index_file $app]
       set app_encoded [encode $index]
       set index_title [find_file_title $index]
       if {$index_title == ""} \
       {
          set t [file join $app title.html]
          if [file exists $t] \
          {
             catch \
             {
                set f [open $t r]
                catch {set index_title [string trimright [read $f]]}
                close $f
             }
          }
       }
       if {$index_title != ""} \
       {
          append y [subst {  <li><a href="$app_encoded">$index_title</a></li>\n}]
       }
    } \
    elseif [file isfile $app] \
    {
       set ext [string tolower [file extension $app]]
       if {[lsearch {.htm .html .shtml} $ext] == -1} \
       {
          # not HTML file - skip it
          continue
       }
       set f [file tail $app]
       if {[lsearch $indexes $f] == -1} \
       {
          set dont_index_flag 0
          set index_title [find_file_title $app]
          if {$dont_index_flag != 0} { continue }
          if {$index_title == ""} { set title "User application note (untitled)"}
          append y [subst {  <li><a href="$app_encoded">$index_title</a></li>\n}]
       }
    }
  }
  if {$y == ""} { return }

  if {$title == ""} \
  {
     return [subst $y]
  }
  append x {<li><a href="[encode $dir]/">$title</a>:\n <ol type="a">\n}
  append x $y
  append x " </ol>\n</li>\n"

  return [subst $x]
  }
  
  #################################################################
  # decode - undo the standard URL encodeing
  #################################################################
  proc decode { url } {
    variable unmap
    regsub -all \[+\] $url " " url
    regsub -all %\[0-9a-fA-F\]\[0-9a-fA-F\] $url {$unmap(&)} url
    return [subst $url]
  }
  #################################################################
  # encode - undo the standard URL encodeing
  #################################################################
  proc encode {string } {
    variable map
    set alphanumeric a-zA-Z0-9./_
    regsub -all \[^$alphanumeric\] $string {$map(&)} string
    regsub -all \n $string {\\n} string
    regsub -all \t $string {\\t} string
    regsub -all {[][{})\\]\)} $string {\\&} string
    return [subst $string]
  }
  
  #################################################################
  # split_url - break a URL into it's component parts.
  #################################################################
  proc split_url { url array } { 
     upvar $array URL
     array set URL { user {} password {} proto {} server {} \
                      portNsep {} port {} path {} anchorNsep {} anchor {} \
                      cgi {} parms {} filename {} match {} \
                      user_password {} match3 {}
                   }
     #define the match strings for the regexp command for readability
     set proto    {[^:]+}
     set user     {[^@:/]+}
     set password {[^@]+}
     set user_password ($user)(:($password))?@
     set server   {[^:/]+}
     set port     {[0-9]+}
     set where    (($proto)://($user_password)?($server)(:($port))?\[/\]?)?
     set file     {[^?#]*}
     set anchorNsep #(.*)
     set cgi        {\?([^#]*)}
     
     regexp ($where)?(($file)?($cgi)?($anchorNsep)?)    \
             $url URL(match) URL(where1) URL(where)                 \
             URL(proto) URL(user_password) URL(user) URL(match3) URL(password) URL(server) URL(portNsep) \
             URL(port) URL(path) URL(filename) URL(cgi) URL(parms) URL(anchorNsep) URL(anchor)
     set URL(proto)  [string tolower $URL(proto)]
     set URL(server) [string tolower $URL(server)]
     if {"$URL(where)" != "" } {
       # need to add back a leading / to the filename
       if {"$URL(filename)" != "" } {
          if {"[string index $URL(filename) 0]" != "~"} {
             set URL(filename) "/$URL(filename)"
          }
       }
     }
  }
  
  #################################################################
  # unsplit_url - given a URL array from split_url, make a string
  #################################################################
  proc unsplit_url { array } {
     upvar $array URL
     
     set url {}
     if {"$URL(proto)" != "" } {
       append url $URL(proto) : / /
     }
     if {"$URL(user)" != "" } {
       append url $URL(user)
       if {"$URL(password)" != "" } {
          append url : $URL(password)
       }
       append url @
     }
     if {"$URL(server)" != "" } {
       append url $URL(server) /
     }
     if {"$URL(filename)" != "" } {
       append url [string trimleft $URL(filename) /]
     }
     if {"$URL(parms)" != "" } {
       append url ? $URL(parms)
     }
     if {"$URL(anchor)" != "" } {
       append url # $URL(anchor)
     }
     return $url
  }

  #################################################################
  # merge_urls - combine 2 URLs into 1.
  #################################################################
  proc merge_urls { master slave} {
     split_url $master Master
     split_url $slave Slave
     # now the tricky bit - joining the URLs. We make the following
     # assumptions:
     # 1) If the slave specifies a protocol, server, user, or port,
     #    use the slave as is.
     # 2) we ignore the anchor and CGI parts of the master.
     # 3) If the master filename ends in a /, it is a directory, and
     #    we join it as is with the slave.
     # 4) If the master filename DOESN'T end in a /, it is a file.
     #    If the slave supplies a file, remove the master file and
     #    replace with the slaves.
     # 5) If the slave supplies an anchor or CGI portion, use them.
     
     foreach part { proto user password server port } {
        if {"$Slave($part)" != "" } {
           return $slave
        }
     }
     # if no slave filename, do nothing
     if {$Slave(filename) != "" } \
     {
      if [string match "*/" $Master(filename)] \
      {
        set Master(filename) [file join $Master(filename) $Slave(filename)]
      } \
      else \
      {
        set Master(filename) [file join [file dirname $Master(filename)] $Slave(filename)]
      }
     }
     # if slave ends in a /, then join will trim that off. put it back.
     if [string match "*/" $Slave(filename)] { 
        set Master(filename) $Master(filename)/
     } 
     set Master(anchor) $Slave(anchor)
     set Master(parms)  $Slave(parms)

     return [unsplit_url Master]
  }

  #################################################################
  #     Name: check_if_proxy_needed
  # Parameters: host
  #Description: sees if we need to use a proxy
  #################################################################
  proc proxy { host } {
     variable local_hosts 
     if {[lsearch -exact $local_hosts $host ] != -1 } {
        return {}
     }
     return [list [http::config -proxyhost] [http::config -proxyport]]
  }
  #################################################################
  # frame_of - return the HTML frame of the widget
  #################################################################
  proc frame_of { widget } {
   if ![winfo exists $widget] { return -code return }
   if {"[winfo class $widget]" == "HtmlClip" } {
     set widget [winfo parent $widget]
   }
   if {"[winfo class $widget]" == "Html" } {
      set widget [winfo parent $widget]
   }
   if {"[winfo class $widget]" == "Scrollbar" } {
      set widget [winfo parent $widget]
   }
   
   return $widget
  }
 
  #################################################################
  # yview - scroll in y
  #################################################################
  proc yview { widget dir {units unit}} {
   set frame [frame_of $widget]
   $frame.html yview scroll $dir $units
  }
  
  #################################################################
  # safe_url
  # url to show
  #################################################################
  proc safe_url { url } {
    # rebuild the URL without the username or password
    split_url $url URL
    set URL(user) {}
    set URL(password) {}
    set new_url [ unsplit_url URL]
    return $new_url
  }
 
  
  #################################################################
  # show_url
  # widget - widget to show
  # url to show
  #################################################################
  proc show_url { frame url } {
    $frame.title configure -text [safe_url $url]
  }
 
  #################################################################
  # mouseover_url - show the URL the mouse is over
  # widget - widget to show
  # x, y - where the mouse is
  #################################################################
  proc mouseover_url { widget x y } {
    set frame [frame_of $widget]
    set url [$widget href $x $y]
    show_url $frame $url
  }

  #################################################################
  # do_hyperlink - activate a hyperlink
  # widget - name of HTML frame
  # x, y - location of click within the widget
  #################################################################
  proc do_hyperlink { widget x y } {
    set frame [frame_of $widget]
 
    set url [$frame.html href $x $y]
    do_url $frame $url
  }
  
  #################################################################
  # anchor - show a specified anchor & move cursor highlight near it
  # widget - name of HTML frame
  # anchor - name of anchor
  #################################################################
  proc anchor { widget anchor} \
  {
     if {$anchor == "" } { return }
     variable htmlscreen
     set frame [frame_of $widget]
     set html $frame.html
     upvar 0 htmlscreen($frame,links) links
     upvar $htmlscreen($frame,names) names
     
     if ![winfo ismapped $html] \
     {
        after 100 [namespace code "anchor $widget $anchor"]
        return
     }

     if {$anchor != ""} \
     {
         update idle
         $html yview $anchor
         if [info exists names($anchor)] \
         {
            Debug::Puts HTML 3 "anchor $frame $anchor found"
            set index 0
            foreach item $links \
            {
               foreach { start end url } $item \
               {
                  Debug::Puts HTML 4 "start $start end $end url $url anchor $names($anchor)"
                  if {$start > $names($anchor) } \
                  {
                     Debug::Puts HTML 4 "Setting selection to $start $end"
                     $html selection set $start $end
                     set htmlscreen($frame,current_link) [$html resolve $url]
                     show_url $frame $htmlscreen($frame,current_link)
                     set htmlscreen($frame,current_index) $index
                     return
                  }
               }
               incr index
            }
         }
     }
  }
  #################################################################
  # Focus - focus a part of this window
  # widget - HTML frame containing the data
  #################################################################
  proc Focus { widget } \
  {
     set frame [frame_of $widget]
     if {[winfo parent $widget] == $frame} { return }
     
     variable htmlscreen
     upvar 0 htmlscreen($frame,links) links
     Debug::Puts HTML 1 "Focus $frame"
     focus -force $frame.html
  }
  
  #################################################################
  # next_link - scan for the "next" href and returns its index.
  # widget - HTML frame containing the data
  # dir - direction of search: 1 for next, -1 for prev
  #################################################################
  proc next_link { widget {dir 1} } {
    variable htmlscreen
    Debug::Puts HTML 3 "next_link $widget $dir"
    
    set frame [frame_of $widget]
    set html $frame.html
    
    upvar 0 htmlscreen($frame,links) links
    set current_index $htmlscreen($frame,current_index)
    if [catch \
        {
           incr current_index $dir
           set max [llength $links]
           if {$current_index >= $max } \
           {
              set current_index 0
           }
           if {$current_index < 0 } \
           {
              set current_index $max
              incr current_index -1
           }
           set info [lindex $links $current_index]
           foreach {start end url} $info \
           {
              $html selection set $start $end
              foreach { top bottom } [$html yview] {}
              foreach { left right} [$html xview] {}
              set height [expr ($bottom-$top)]
              set bottom [expr $bottom - .1*$height]
              set width [expr ($right - $left)]
              set right [expr $right - .1*$width]
              foreach { x y } [$html selection pos] \
              {
                 if {$y == "NaN" } { continue }
                 Debug::Puts HTML 10 "y $y, top $top, bottom $bottom middle $height"
                 if { ($top > $y ) || ($y > $bottom) } \
                 {
                    $html yview moveto [expr $y - $height/2.0]
                 }
                 if {$x == "NaN" } { continue }
                 Debug::Puts HTML 10 "x $x, left $left, right $right middle $width"
                 if { ($left > $x ) || ($x > $right) } \
                 {
                    $html xview moveto [expr $x - $width/2.0]
                 }
              }
              set htmlscreen($frame,current_link) [$html resolve $url]
              show_url $frame $htmlscreen($frame,current_link)
              set htmlscreen($frame,current_index) $current_index
           }
         } err 
       ] \
    {
        Debug::Puts HTML 1 "next_link: $err"
    }
  }

  #################################################################
  # NoConnect - show an error message box if we cannot connect
  #################################################################
  proc NoConnect { widget url err} {
     variable err_base
     variable error_in_progress
     if {$error_in_progress } { return }
     if ![winfo exists $widget] { return }
     set error_in_progress 1
     set errs [::http::formatQuery url $url err $err]
     catch { do_url $widget $err_base/no_connect.tcl?$errs }
     set error_in_progress 0
  }
  
  #################################################################
  # make - create an HTML frame
  #################################################################
  proc make { name {url ""} } \
  {
     Busy
     Debug::Puts HTML 3 "make:  $name,  $url"
     
     if ![Memory::Enough_for help] \
     {
        label $name.lowmem -text "Too little memory for this operation"
        pack $name.lowmem -fill both -expand true
        NotBusy
        return
     }
     
     variable htmlscreen
     
     variable html_frame_count
     
     
     set htmlscreen($name,names) ::HtmlFrame::names_$html_frame_count  
     upvar #0 $htmlscreen($name,names) names
     array set names {}

     incr html_frame_count
 
     # set up the proxy data as needed.
     if {"$::sys(useproxy)"=="YES"} {
        set port $::sys(proxyport)
        if {"$port"==""} { set port 80}
        http::config -proxyhost $::sys(proxyip) -proxyport $port
     } else {
        http::config -proxyhost {} -proxyport {}
     }

     set htmlscreen($name,history)             {}
     set htmlscreen($name,current_link)        {}
     set htmlscreen($name,current_index)       0  
     set htmlscreen($name,download_filehandle) 0
     set htmlscreen($name,images)              {}

     if {$url == "" } {
       global default_page
       set url $default_page
     }

     frame $name.body  -bg white -class HtmlFrame
     label $name.title
     foreach { bar orient cmd } { yscroll vert yview xscroll horiz xview } \
     {
       scrollbar $name.$bar -orient $orient -command "$name.html $cmd"  -takefocus 1 -borderwidth 2 \
                 -activebackground white -highlightthickness 2
     }
     bind $name.yscroll <<Spin_up>>   "tkScrollByUnits %W vert -1; break"
     bind $name.yscroll <<Spin_down>> "tkScrollByUnits %W vert 1; break"
     bind $name.xscroll <<Spin_up>>   "tkScrollByUnits %W horiz 1; break"
     bind $name.xscroll <<Spin_down>> "tkScrollByUnits %W horiz -1; break"
     bind $name.xscroll <FocusIn>     "::HtmlFrame::Do_scroll_focus $name.xscroll $name.b3 in"
     bind $name.xscroll <FocusOut>    "::HtmlFrame::Do_scroll_focus $name.xscroll $name.b3 out"
     bind $name.yscroll <FocusIn>     "::HtmlFrame::Do_scroll_focus $name.yscroll $name.b2 in"
     bind $name.yscroll <FocusOut>    "::HtmlFrame::Do_scroll_focus $name.yscroll $name.b2 out"
          
     html      $name.html    -yscroll "$name.yscroll set " -selectioncolor yellow -takefocus 1 -bg white  \
                             -xscroll "$name.xscroll set "
     $name.html config -imagecommand "::HtmlFrame::html_image $name.html " \
                       -appletcommand ::HtmlFrame::do_applet
                       
     $name.html parse $htmlscreen(start_html)
     
     add_bindtag $name.html    HtmlFrame
     add_bindtag $name.html.x  HtmlFrame
     add_bindtag $name.yscroll HtmlFrame
     add_bindtag $name.xscroll HtmlFrame
     add_bindtag $name.body    HtmlFrame
     add_bindtag $name HtmlFrame_MAIN
     frame $name.buttons
     grid propagate $name.buttons false
     foreach button { 1 2 3 4 5 6 } \
     {
        button $name.b$button  \
               -bg $::color(softkeybg) \
               -fg $::color(softkeyfg) \
               -relief raised \
               -takefocus 0 \
               -highlightbackground $::color(softkeybg) \
               -activebackground    $::color(softkeybg) \
               -bd 4 \
               -height 2 \
               -padx 2
        
        grid $name.b$button -in $name.buttons -sticky news -column 0 -row $button
        
        add_bindtag $name.b$button Softkey HtmlFrame Reflow
        
        raise $name.b$button $name.buttons
     }
     
     button $name.return \
             -bg $::color(softkeybg) \
             -fg $::color(softkeyfg) \
             -relief flat \
             -takefocus 0 \
             -highlightbackground $::color(softkeybg) \
             -activebackground    $::color(softkeybg) \
             -bd 0 \
             -padx 2
             
     grid $name.return -in $name.buttons -sticky news -column 0 -row 7
     
     add_bindtag $name.return  Softkey  HtmlFrame Reflow
     
     raise $name.return $name.buttons
          
     for { set i 0 } { $i <= 7 } { incr i } {
        grid rowconfigure $name.buttons $i -weight 1 -minsize 10
     }
     
     $name.b2 configure -text "UP/\nDOWN"   -command "::HtmlFrame::Do_skey_scroll_v $name"
     $name.b3 configure -text "LEFT/\nRIGHT" -command "::HtmlFrame::Do_skey_scroll_h $name"
     $name.b6 configure -text "BACK"         -command "::HtmlFrame::do_back $name" -state disabled
     $name.return configure -text "RETURN"   -command "::HtmlFrame::Close $name"
 
     pack $name.buttons -in $name -side right    -fill y    -expand false
     pack $name.yscroll -in $name -side right    -fill y    -expand false
     pack $name.body    -in $name -side right    -fill both -expand true

     grid propagate $name.buttons false
     grid propagate $name.body false
     grid $name.title   -in $name.body -row 0 -sticky news
     grid $name.html    -in $name.body -row 1 -sticky news
     grid $name.xscroll -in $name.body -row 2 -sticky news
     grid rowconfig $name.body 1 -weight 1
     grid columnconfig $name.body 0 -weight 1
     if {[winfo toplevel $name] == $name }\
     {
        wm deiconify $name
        update idle
     }
     after idle  "::HtmlFrame::Focus  $name"
     after idle "::HtmlFrame::do_url $name $url"
     
     NotBusy
  }
  proc Do_scroll_focus { widget button which} \
  {
     if {$which == "in"} \
     {
        $button config -bg white
        $widget config -bg white
     } \
     else \
     {
        $button config -bg grey
        $widget config -bg grey
     }
  }
  #################################################################
  # Do_skey_scroll_h - activate/deactive horizontal scroller
  #################################################################
  proc Do_skey_scroll_h { widget } \
  {
      set frame [frame_of $widget]
      if {[$frame.b3 cget -bg] != "white"} \
      {
         focus -force $frame.xscroll
      } \
      else \
      {
         focus -force $frame.html
      }
  }
  #################################################################
  # Do_skey_scroll_v - activate/deactive vertical scroller
  #################################################################
  proc Do_skey_scroll_v { widget } \
  {
      set frame [frame_of $widget]
      if {[$frame.b2 cget -bg] != "white"} \
      {
         focus -force $frame.yscroll
      } \
      else \
      {
         focus -force $frame.html
      }
  }
  
  #################################################################
  # Parse - parse a file, update links, and deal with scrollbars
  # inputs: widget within window
  #################################################################
  proc Parse { widget data anchor } \
  {
    set frame [frame_of $widget]
    $frame.html clear
    $frame.html parse $data
    locate_links $frame
    anchor $frame $anchor
    set desired_width [winfo reqwidth $frame.html]
    set width [winfo width $frame.html]
  }
  
  #################################################################
  # make_toplevel - create a frame containing an HTML widget, a scroller
  #                   a back button, and a URL display
  # inputs: name - name of new top level to create
  #         url  - URL to open
  #         where - either a screen designation, or a widget name
  #                 used to find what display to use
  #################################################################
  proc make_toplevel { name {url ""} {title "IFR-2975 Browser"} {class "IFR-2975"}} \
  {
     Busy
     Debug::Puts HTML 1 "make_toplevel $name $url $title $class" 
     destroy $name

     toplevel $name -class $class -width 640 -height 480 -takefocus 1
     if ![Memory::Enough_for help] \
     {
        set parent [winfo parent $name]
        set msg [name_child $parent lowmem]
        message_box $msg CAUTION
        destroy $name
        NotBusy
        return $msg
     }

     set old_focus [focus -displayof $name]
     bind $name <<RESTORE_FOCUS>> "safe_focus_force $old_focus"
     if {[winfo screen .] == [winfo screen $name]} \
     {
        wm geo $name 640x480+0+0
     }
     
     wm protocol $name  WM_DELETE_WINDOW  "::HtmlFrame::Close $name"
     wm withdraw $name
     add_bindtag $name HtmlFrame HtmlWindow Mouse_cursor FocusFixer Toplevel_cleanup
     wm title    $name $title
     pack propagate $name false
     Mouse_cursor::Hook $name
     make $name $url
     bind $name.html <Destroy> "destroy $name"
     wm deiconify $name 
     update idle
     NotBusy
  }
  
  #################################################################
  #  - destroy the HTML frame
  #################################################################
  proc Close { widget } \
  {
    Debug::Puts HTML 1 "close $widget" 

    set frame  [frame_of $widget]
    
    Debug::Puts HTML 3 "Close:  $frame"
  
    Destroy $widget

    destroy $frame  
     
    return
  }
  
  
  #################################################################
  # Clean - clean up after a frame
  #################################################################
  proc Destroy { widget } {
     Debug::Puts HTML 1 "Destroy $widget" 
     after idle "HtmlFrame::clear_cache $widget"
     Debug::Puts HTML 1 "idle scheduled" 
     variable htmlscreen
     if ![winfo exists $widget] { return }
     set frame [frame_of $widget]
     bind $frame.html <Destroy> {}
  }

 
  #################################################################
  # pick - activate a hyperlink
  #################################################################
  proc pick { widget } {
     set frame [frame_of $widget]
     do_url $widget $::HtmlFrame::htmlscreen($frame,current_link)
  }

  #################################################################
  # do_url - render a link in an HTML frame
  # widget - name of frame to render in
  # url    - url to render
  #################################################################
  proc do_url { widget url } {
    Busy
    catch \
    {
	    variable htmlscreen
	    Debug::Puts HTML 3 "do_url $widget $url"
	    if ![winfo exists $widget] { NotBusy; return }
	    set frame [frame_of $widget]
	    if {$url == ""} { return }
	    set url [$frame.html resolve $url]
	    set htmlscreen($frame,history) [linsert $htmlscreen($frame,history) 0 $url ]
	    set htmlscreen($frame,current_index) 0
	    # if same except for anchor, just do the anchor part
	    split_url $url New_URL
	    split_url [$frame.html cget -base] Curr_URL
	    if { ($New_URL(where) == $Curr_URL(where)) && ($New_URL(filename) == $Curr_URL(filename))} \
	    {
	        anchor $widget $New_URL(anchor)
	        return
	    }
	    set safer_url [safe_url $url]
	    $frame.title configure -text "Finding $safer_url"
	    $frame.html clear
	    clear_image_cache $frame
	    $frame.html parse [subst $htmlscreen(finding)]
	    after idle "DNS::Do_A_query $New_URL(server)"
	    after idle [namespace code "do_url_cb $widget $url"]
     }
     NotBusy
  }
  proc do_url_cb { widget url }\
  {
    set frame [frame_of $widget]
    if [catch {parse_url $frame $url} err] {
      catch {NoConnect $frame $url $err } err
      NotBusy
      return
    }
    $frame.title configure -text [safe_url $url]
    NotBusy
  }

  #################################################################
  # do_back - move to previous link in history
  #################################################################
  proc do_back { widget } {
     variable htmlscreen
     set frame [frame_of $widget]
     set old_url [ lindex $::HtmlFrame::htmlscreen($frame,history) 1]
     set ::HtmlFrame::htmlscreen($frame,history) [lrange $::HtmlFrame::htmlscreen($frame,history) 1 end]
     set ::HtmlFrame::htmlscreen($frame,current_index) 0
     parse_url $widget $old_url
  }
  #################################################################
  # clear_image_cache - destroy all image objects not in use
  #################################################################
  proc clear_image_cache { frame } {
     variable htmlscreen
     Debug::Puts HTML 3 "clear_image_cache $frame"
     upvar 0 htmlscreen($frame,images) images
     set new_images {}
     if [info exists images] \
     {
       foreach image $images \
       {
          if {[info commands $image] != ""} \
          {
             if [$image inuse] \
             {
                Debug::Puts HTML 3 "image $image in use"
                lappend new_images $image
             } \
             else \
             {
                Debug::Puts HTML 3 "image $image NOT in use"
                catch {image delete $image}
             }
          }
       }
       set images $new_images
     }
  }
  #################################################################
  # clear_cache - destroy all image objects and files for this item
  #################################################################
  proc clear_cache { frame } {
     variable htmlscreen
     Debug::Puts HTML 3 "clear_cache $frame"
     clear_image_cache $frame
     if [info exists htmlscreen($frame,names)] \
     {
        if [info exists $htmlscreen($frame,names)] \
        {
           unset $htmlscreen($frame,names)
        }
     }
     foreach item [array names htmlscreen $frame,*] \
     {
        unset htmlscreen($item)
     }
  }
  #################################################################
  # html_image - get an image for display
  # widget - name of HTML widget we are being created for
  # url - URL of image
  # width, height - width and height spec'ed in the HTML IMG tag
  # args - any left over parms (ignored)
  #################################################################
  proc html_image { widget url width height args } {
     Debug::Puts HTML 3 "html_image $widget $url $width $height $args"
     variable htmlscreen
     set frame [frame_of $widget]
     upvar 0 htmlscreen($frame,images) images
     # see if we already have this image
     set image_name $url$width$height
     if { [lsearch -exact [image names] $image_name] >= 0} {
        return $image_name
     }

     # create the image with a broken link icon
     image create photo $image_name -file /sysdata/html/broken.gif 
     scale_image $image_name $width $height
     
     # now, find out what we have to do to get the real image

     array set URL {}
     split_url $url URL
     # handle file://, ftp://, and http:// urls

     variable pending_images
     switch $URL(proto) {
         http { lappend pending_images [list http_image $image_name $url        $width $height]}
         file { lappend pending_images [list file_image $image_name /$URL(path) $width $height]}
         ftp  { lappend pending_images [list ftp_image  $image_name $url        $width $height]}
     }
     set cmd [namespace code load_images]
     after cancel $cmd
     after idle $cmd
     lappend images $image_name
     return $image_name
  }
  proc load_images { } \
  {
     variable pending_images
     set next [lindex $pending_images 0]
     set pending_images [lrange $pending_images 1 end]
     if {$next != ""} \
     {
        eval $next
     }
     if {[llength $pending_images] != 0} \
     {
        after idle [namespace code load_images]
     }
  }

  #################################################################
  # scale_image - resize an image for display
  # image name, desired width, desired height
  #################################################################
  proc scale_image { image width height } {
   Debug::Puts HTML 3 "scale_image $width $height"
   if {$width != "" }  {
      if {[regexp .*% $width] == 0} {
        $image config -width $width
      }
   }
   if {$height != "" } {
      if {[regexp .*% $height] == 0} {
        $image config -height $height
      }
   }
  }
  #################################################################
  # render_image - draw an image once it's loaded
  #################################################################
  proc render_image { image_name file width height } {
    Debug::Puts HTML 3 "render_image $image_name $file $width $height"
    if [catch {
      if ![Memory::Enough_for image] \
      {
         error "Not enough RAM for image"
      }
      $image_name blank
      $image_name read $file
      scale_image $image_name $width $height
    } err ] { 
       Debug::Puts HTML 1 "render_image: $image_name $err"
    }
  }
  #################################################################
  # render_http_image - draw an image once it's loaded
  #################################################################
  proc render_http_image { url image_name filehandle file width height http } {
   Debug::Puts HTML 3 "render_http_image $url $image_name $filehandle $file $width $height $http"
   upvar $http http_data
   set http_status [split $http_data(http)]
   set http_status_code [lindex $http_status 1]
   switch $http_status_code {
    401 { 
           seek $filehandle 0 start
           return
        }
    100 {}
    200 {}
    default {
      close $filehandle
      return
    }
   }

   close $filehandle
   render_image $image_name $file $width $height
  }
  #################################################################
  # render_ftp_image - draw an image once it's loaded
  #################################################################
  proc render_ftp_image { url image_name file width height token } {
     Debug::Puts HTML 3 "render_ftp_image $url $image_name $filehandle $file $width $height $token"
     render_image $image_name $file $width $height
     ::ftp::done $token
  }

  #################################################################
  # get_image_filename - create a file name for an image
  # since the image command has to work with FILES, not file handles
  # we have to download the image into a file. use /usr/cache as
  # the file location, and generate a unique name with the counter
  #################################################################
  proc get_image_filename { } {
    variable http_image_count

    set workdir /usr/cache
    # make sure the cache directory exists
    file mkdir $workdir
    set workfile $workdir/foo$http_image_count
    incr http_image_count
    
    return $workfile
  }

  #################################################################
  # http_image - get an image via HTTP
  #################################################################
  proc http_image { image_name url width height } {
    Debug::Puts HTML 3 "http_image $image_name $url $width $height"
    set workfile [get_image_filename]
    set f [open $workfile w]
    if [catch {
      geturl_with_auth $url -channel $f -timeout 30000 \
         -command "::HtmlFrame::render_http_image $url $image_name \
                   $f $workfile \{$width\} \{$height\}"
    } err ] {
       Debug::Puts HTML 1 "http_image: $err"
    }
    return $image_name
  }

  #################################################################
  # ftp_image - display image via ftp 
  # url - URL of image 
  # image_name - URL of image + width + height
  # array - array component of URL 
  # width - widget width
  # height - widget height
  #################################################################
  proc ftp_image { image_name url width height } {
    Debug::Puts HTML 3 "ftp_image $image_name $url $width $height"
    set file [get_image_filename]
    set widget ""

    if [catch {
      set ftp [::ftp::create $url -filename $file]
      ::ftp::configure $ftp -command \
         "::HtmlFrame::render_ftp_image $url $image_name $file \{$width\} \{$height\}"
      ::ftp::Open $ftp
      ::ftp::get  $ftp
      return
    } err ] {
      Debug::Puts HTML 1 "ftp_image: $err"
    }
  }

  #################################################################
  # file_image - render image from local file
  #################################################################
  proc file_image { image_name file width height } {
      render_image $image_name $file $width $height
  }



  #################################################################
  # encode_http_passwd - convert a username:passwd combo to
  #                      base64 encoding
  # widget - HTML frame that neede the password
  # url - URL that the password applies to
  # index - computed index in password table to save to
  #################################################################
  proc encode_http_passwd { widget url index } {
    set base $widget.password
    set name [$base.name.r get]
    set passwd [$base.pass.r get]
    destroy $base
    set code $name:$passwd
    set code [base64::encode $code]
    global http_passwords
    set http_passwords($index) $code
    parse_url_core $widget $url
  }

  #################################################################
  # get_http_password - query user for password for site
  # widget - HTML frame that needs the password
  # url - URL that needs the password
  # index - computed index into password table to save to
  #################################################################
  proc get_http_password { widget url index} {
    upvar meta meta
    set base $widget.password
    set old_focus [focus]
    toplevel $base -screen [winfo screen $widget]
    bind <Destroy> $base "safe_focus_force $old_focus"
    set title "Password required for $url"
    wm title $base $title 
    label $base.title -text $title
    frame $base.main
    frame $base.buttons
    pack  $base.buttons -side right -fill y    -expand false
    pack  $base.main    -side right -fill both -expand true
    frame $base.name
    frame $base.pass
    pack $base.title $base.name $base.pass -in $base.main -side top -fill both -expand true
    label $base.name.l -text "User name:"
    entry $base.name.r 
    
    add_bindtag $base.name.r AlphaNumEntry
    
    pack $base.name.l -side left -fill y
    pack $base.name.r -side left -fill both -expand true
  
    label $base.pass.l -text "Password"
    entry $base.pass.r -show *
    
    add_bindtag $base.pass.r AlphaNumEntry
    
    pack $base.pass.l -side left -fill y
    pack $base.pass.r -side left -fill both -expand true
 
    foreach button { 1 2 3 4 5 6 } {
        button $base.b$button  \
               -bg $::color(softkeybg) \
               -fg $::color(softkeyfg) \
               -relief raised \
               -takefocus 0 \
               -highlightbackground $::color(softkeybg) \
               -activebackground    $::color(softkeybg) \
               -bd 4 \
               -height 2 \
               -wrap 100
        grid $base.b$button -in $base.buttons -sticky news -column 0 -row $button
        add_bindtag $base.b$button Softkey HtmlFrame
        raise $base.b$button $base.buttons
        
        set key [expr $button+1]
        bind $base <Key-F$key>        { event generate $base.b$button <<Press>> }
        bind $base <KeyRelease-F$key> { event generate $base.b$button <<Release>> }
     }

    $base.b1 config -text "OK"     \
       -command "::HtmlFrame::encode_http_passwd $widget $url {$index}"
    $base.b6 config -text "Cancel" \
       -command "destroy $base"
    bind $base.name.r <Return> "focus $base.pass.r"
    bind $base.name.r <Tab>    "focus $base.pass.r"
    bind $base.pass.r <Return> "::HtmlFrame::encode_http_passwd $widget $url {$index}"
    bind $base.pass.r <Tab>    "::HtmlFrame::encode_http_passwd $widget $url {$index}"
    after idle  "safe_focus_force $base.name.r"
  }

  #################################################################
  # http_401 - handle authentication request (show password box)
  #################################################################
  proc http_401 { } {
     upvar index index
     upvar sys sys
     upvar header headers
     upvar url_body url_body
     upvar url_data url_data
     upvar args args
     ::http::wait $url_body
     
     Debug::Puts HTTP 1 "HtmlFrame::http_401"
     
     array set meta $url_data(meta)
     if [info exists meta(WWW-Authenticate)] {
        set index $meta(WWW-Authenticate)
     }
     # some stupid web server *cough*IIS*cough* don't use the correct
     # HTTP header. You would think this spec was closed
     if [info exists meta(Www-Authenticate)] {
        set index $meta(Www-Authenticate)
     }
     if {$index != "" } {
        # see if we have this already
        global http_passwords
        if [info exists http_passwords($index)] {
          # get the encoded password & try again
          lappend headers Authorization "Basic $http_passwords($index)"
          set url_body [eval http::geturl $args -headers {$headers} ]
          return $url_body
        }
      }
      return ""
  }

  #################################################################
  # http_30x - handle "Object moved" messages (follow redirects)
  #################################################################
  proc http_30x { } {
     upvar index index
     upvar sys sys
     upvar header headers
     upvar url_body url_body
     upvar url_data url_data
     upvar args args
   
     Debug::Puts HTTP 1 "HtmlFrame::http_30x"
     
     http::wait $url_body
     array set meta $url_data(meta)
     if [info exists meta(Location)] {
       set new_url [concat $meta(Location)]
       if {[string match *://* $new_url] == 0 } {
          set new_url $url_data(url)/$new_url
       }
       set args [concat $new_url [lrange $args 1 end]]
       return [geturl_with_auth $new_url ]
     }
     return ""
  }
  
  #################################################################
  # geturl_with_auth - get a URL, handling any HTTP authentication
  # as needed
  # args - whatever args need to be passed to the core 
  # http::geturl routine
  #################################################################
  proc geturl_with_auth { args } {
    upvar index index
    set headers {}
    # get the URL, and see if a username/password was given
    set url [lindex $args 0]
    set other_args [lrange $args 1 end]
    split_url $url URL
    if {"$URL(user)"!=""} {
       set code [::base64::encode $URL(user):$URL(password)]
       lappend headers Authorization "Basic $code"
       Debug::Puts HTML 3 "adding password header $headers"
    }
   
    # see if we need to use a proxy authorization
    if {"$::sys(useproxy)"=="YES"} {
      set proxy_pw $::sys(proxyuser):$::sys(proxypw)
      # see if we have a proxy password, and give that a shot
      if {"$proxy_pw"!=":"} {
          set code [::base64::encode $proxy_pw]
          lappend headers Proxy-Authorization "Basic $code"
      }
    }
    Debug::Puts HTTP 1 "HtmlFrame::geturl_with_auth $args - getting"

    set url_body [eval ::http::geturl $url $other_args -headers {$headers} ]
    Debug::Puts HTTP 1 "HtmlFrame::geturl_with_auth $args - got data: $url_body"
    return [finish_geturl $url_body]
  }
  proc finish_geturl { url_body } \
  {
    upvar index index
    upvar args args
    upvar #0 $url_body url_data
    
    Debug::Puts HTTP 1 "HtmlFrame::finish_geturl $url_body"
    
    # see if we got a 401 auth request. If so, do a pop-up
    while { (![info exists url_data(http)]) || ($url_data(http) == "")} \
    {
      Debug::Puts HTTP 1 "HtmlFrame::geturl_with_auth $args - not done yet, waiting..."
      if { [array exists url_data] } \
      {
        if { [info exists url_data(-channel)] } \
        {
          # If a file has been defined, erase any previous contents 
          seek $url_data(-channel) 0 start
        }
      }
      set cmd "after 5000 \"HtmlFrame::force_timeout $url_body\""
      after cancel $cmd
      after 10000 $cmd
      vwait $url_body\(http)
      after cancel $cmd
      Debug::Puts HTTP 1 "HtmlFrame::geturl_with_auth $args - wait over"
    }
    set http_status [split $url_data(http)]
    set http_status_code [lindex $http_status 1]
    switch $http_status_code {
      401 { set url_body [http_401]; return [finish_geturl $url_body] }
      301 -
      302 { set url_body [http_30x]; return [finish_geturl $url_body] }
      TIMEOUT { error "HTTP transfer timeout"}
    }
    return $url_body
  }
  #################################################################
  # force_timeout
  # HTTP object for force
  #################################################################
  proc force_timeout {url_body} \
  {
     Debug::Puts HTTP 1 "HtmlFrame::force_timeout $url_body"
     upvar #0 $url_body url_data
     set url_data(http) TIMEOUT
  }
  
  #################################################################
  # parse_http_post_url - Show a URL in an HTML widget
  # widget - TKhtml widget
  # url - URL to show
  #################################################################
  proc parse_http_post_url { widget url postdata {anchor ""}} {
    set index ""
    if {$url == "" } { return }
    set q [http::formatQuery $postdata]
    set url_body [geturl_with_auth $url -timeout 30000 -query $q]
    if {$url_body == ""} {
      get_http_password $widget $url $index
      return
    }
    upvar #0 $url_body url_data
    $widget clear
    $widget configure -base $url_data(url)
    Parse $widget $url_data(body) $anchor
  }
  #################################################################
  # parse_http_url - Show a URL in an HTML widget
  # widget - TKhtml widget
  # url - URL to show
  #################################################################
  proc parse_http_url { widget url {anchor ""}} {
    set index ""
    if {$url == "" } { return }
    Debug::Puts HTTP 1 "HtmlFrame::parse_http_url $widget $url $anchor"
    if ![winfo exists $widget] { return }
    set url_body [geturl_with_auth $url -timeout 30000]
    if ![winfo exists $widget] { return }
    if {$url_body == ""} {
      Debug::Puts HTTP 1 "HtmlFrame::parse_http_url $widget $url $anchor - need password"
      get_http_password $widget $url $index
      return
    }
    Debug::Puts HTTP 1 "HtmlFrame::parse_http_url $widget $url $anchor - got data, parsing"
    upvar #0 $url_body url_data
    if ![winfo exists $widget] { return }
    $widget configure -base $url_data(url)
    Parse $widget $url_data(body) $anchor
    Debug::Puts HTTP 1 "HtmlFrame::parse_http_url $widget $url $anchor - rendered"
  }
  
  #################################################################
  # get_file_contents - read a file & make ready to display
  # Returns: contents of file ready to display
  # Actions: for html files, just read the file
  #          for .txt files, wrap file contents in <pre></pre>
  #          for .tcl files, source the script and return it's output
  #################################################################
  proc get_file_contents { filename {parms {}}} {
     # remove any extra parms from the URL
     Debug::Puts HTML 3 "get_file_contents $filename $parms"
     if ![file isfile $filename] \
     {
        set value "<html><head><title>$filename not found</title><body><h1>$filename not found</h1></body></html>"
     }
     switch -- [string tolower [file extension $filename]] {
        ".htm" -
        ".html" \
        {
          Debug::Puts HTML 3 "doing HTML file"
          set chan [open $filename r]
          set value [read $chan]
          close $chan
        }
        ".shtml" \
        {
          Debug::Puts HTML 3 "doing SHTML file"
          # set up vars for server parsed HTML
          set this(args) $parms
          set this(url) file://localhost/$filename
          HTTPD::Parse_CGI
          set chan [open $filename r]
          set value [read $chan]
          close $chan
          set pd [pwd]
          catch \
          {
             cd [file dirname $filename]
             set value [subst $value]
          }
          cd $pd
        }
        ".txt"     \
        {
          Debug::Puts HTML 3 "doing txt file"
          set chan [open $filename r]
          append value "<html>" "<head>"          \
                       "<title>$filename</title>" \
                       </head>                    \
                       "<body><pre>"              \
                       "[read $chan]"             \
                       "</pre></body>"            \
                       "</html>"
          close $chan
        }
        ".tcl"    \
        {
         Debug::Puts HTML 3 "doing TCL file"
         # bust the CGI parms into a useable form
         set parmlist [split $parms &=]
         array set CGI_PARMS(raw) $parmlist
         Debug::Puts HTML 4 "parmlist is $parmlist"
         set old_pwd [pwd]
         Debug::Puts HTML 4 "Current dir $old_pwd"
         cd [file dirname $filename]
         Debug::Puts HTML 4 "Now in [pwd]"
         catch {
           foreach { var value } $parmlist {
              set var [decode $var]
              set value [decode $value]
              set CGI_PARMS($var) $value
           }
         }
         Debug::Puts HTML 4 "Sourcing $filename"
         catch { set value [source $filename] }
         cd $old_pwd
         Debug::Puts HTML 4 "back to [pwd]"
        }
        ".gif" -
        ".jpg" -
        ".bmp" -
        ".jpeg" \
        {
           set value [subst {<html><head><title>$filename</title></head><body><img width="640" height="480" src="file://localhost/$filename"></body></html>}]
           
        }
        default \
        {
           set value [subst {<html><head><title>$filename - unknown type</title></head><body><h1>$filename is an unknown type</h1></body></html>}]
        }
     }
     return $value
  }

  proc find_index_file { dir } \
  {
      variable indexes
      foreach file $indexes \
      {
         set trial [file join $dir $file]
         if [file exists $trial] \
         {
           return $trial
         } 
      }
      return ""
  }
  
  #################################################################
  # parse_file - show a file on local system
  #################################################################
  proc parse_file {widget url {anchor ""}}  {
    Debug::Puts HTTP 1 "parse_file $widget $url $anchor"
    if ![winfo exists $widget] { return }
    # parse the URL into pieces
    array set URL {}
    split_url $url URL
    # get just the base name of the file, minus any anchor or
    # parameters. 
    set fname [file join / [decode $URL(filename)]]
    
    # if this is the "top" level HTML directory, re-direct it to the help directory
    # as the top level help won't work for us.
    Debug::Puts HTTP 1 "fname is $fname"
    if {$fname == "/sysdata/html"} \
    {
       set fname "/sysdata/html/help";
    }

    if [file isdir $fname] \
    {
      set fname [find_index_file $fname]
    }
    if {![file exists $fname]} {
       parse_url_core $widget $::HtmlFrame::notfound_page 
       return
    }
    set text [get_file_contents $fname $URL(parms)]
    $widget configure -base file://localhost$fname
    Parse $widget $text $anchor
  }

  #################################################################
  # parse_ftp_url - Show a URL in an HTML widget
  # widget - TKhtml widget
  # url - URL to show
  #################################################################
  proc parse_ftp_url { widget url {anchor ""} } {
     Debug::Puts HTTP 1 "parse_ftp_url $widget $url $anchor"
     if ![winfo exists $widget] { return }
     set ftp [::ftp::create $url]
     
     upvar #0 $ftp ftp_data
     ::ftp::Open $ftp
     ::ftp::get $ftp

     $widget configure -base $url
     Parse $widget $ftp_data(data) $anchor
     ::ftp::done $ftp
     unset $ftp
  }

  #################################################################
  # parse_url_core - Show a URL in an HTML widget
  # widget - TKhtml widget
  # url - URL to show
  #################################################################
  proc parse_url_core { widget url } {
    Debug::Puts HTTP 1 "parse_url_core $widget $url"
    if ![winfo exists $widget] { return }
    if {$url == ""} { return } 

    # split the URL up into parts
    variable notfound_page
    array set URL {}
    split_url $url URL

    switch $URL(proto) {
       http { parse_http_url $widget $url  $URL(anchor) }
       file { parse_file     $widget $url  $URL(anchor) }
       ftp  { parse_ftp_url  $widget $url  $URL(anchor) } 
       default { 
          if {$url != $notfound_page } {
            parse_file $widget $notfound_page
          }
       }
    }

  }

  #################################################################
  # parse_url - Show a URL in an HTML frame
  # widget - HTML frame, not the html widget
  # url - URL to show
  # also handles softkeys
  #################################################################
  proc parse_url { widget url} {
    Debug::Puts HTTP 1 "parse_url $widget $url"
    variable htmlscreen
    if ![winfo exists $widget] { return }
    
    set frame [frame_of $widget]
    set html $frame.html
    do_history $widget
    parse_url_core $html $url
  }
  #################################################################
  # do_history - possibly add item to history, and update back button
  # widget - HTML frame, not the html widget
  # url - URL to show
  # also handles softkeys
  #################################################################
  proc do_history { widget {url ""} } \
  {
    Debug::Puts HTML 3 "do_history $widget $url"
    variable htmlscreen
    set frame [frame_of $widget]
    if {$url != ""} \
    {
      set htmlscreen($widget,history) [linsert $htmlscreen($frame,history) 1 $url ]
    }
    # DRAW BACK SOFTKEY IF NEEDED
    if {[llength $htmlscreen($frame,history)] > 1} {
      $widget.b6 configure -state normal
    } else {
      $widget.b6 configure -state disabled
    }
  }

  #################################################################
  # do_applet - handle <embed> and <applet> tags in the code
  # Since this is a security risk (not to mention the fact
  # that most applets are in Java, which we don't support)
  # we will restrict applets to only a small set of applet
  # URLs that are on the local system. Those applets are
  # in the sysdata/html/applets directory.
  # The basic game plan is that we will source the applet
  # file. The applet file will have the args from the
  # tag available to it in the array APPLET_ARGS
  #################################################################
  proc do_applet { widget applet_args } { 
    if [catch {
       array set APPLET_ARGS $applet_args
       # now, we parse the URL. If no URL is given, bail
       if {[info exists APPLET_ARGS(src)] == 0 } {
           return
       }
       array set URL {}
       split_url $APPLET_ARGS(src) URL
       if {"$URL(proto)" != "file"} {
         return;
       }
       if {"$URL(server)" != "localhost" } {
         return;
       }

       set full_path /sysdata/html/applets/$URL(path)
   
       if {[file exists $full_path] == 0 } {
         return
       }

       # OK, the file exists. Create a frame for it and source the file
       destroy $widget
       frame $widget
       pack propagate $widget false
       if [info exists APPLET_ARGS(width)] {
         $widget configure -width $APPLET_ARGS(width)
       }
       if [info exists APPLET_ARGS(height)] {
         $widget configure -height $APPLET_ARGS(height)
       }
       set parent [winfo parent $widget]
       $widget configure -bg [$parent cget -bg]
       catch {
         source $full_path
       } err
       if {$err != "" } { 
         Debug::Puts HTML 1 "do_applet (while sourcing $full_path): $err\n"
       }
     } err ] {
        error $err
     }
  }

  #################################################################
  # default_URL - set the members of a array if they aren't already set
  # The intended purpose of this proc it to set the members of a
  # URL() array that weren't set by the actual HTML.
  # It takes the name of the array to pre-set, and a list of
  # name/value pairs to default it to
  #################################################################
  proc default_URL { array defaults } {
     # first, create an array of defaults
     if [ catch {
         array set DEFAULTS $defaults
        } err ] {
       error "default_URL: $err"
       return
     }
     upvar $array URL
     foreach name [array names DEFAULTS] {
        if {[info exists URL($name)] == 0} {
           set URL($name) $DEFAULTS($name)
        }
     }
   }
   
  #################################################################
  # locate_links - identify the <a> tags in a file and record them
  #################################################################
  proc locate_links { widget} \
  {
     Debug::Puts HTML 3 "locate_links $widget"
     variable htmlscreen
     set frame [frame_of $widget]
     set html $frame.html
     upvar $htmlscreen($frame,names) names
     set entries [$html token list 1 end ]
     set links ""
     if [info exists names] {unset names}
     set index 0
     set href_begin ""
     set href_url ""
     foreach item $entries \
     {
        incr index
        if {[lindex $item 0] == "Markup" }\
        {
           array set m $item
           switch -- $m(Markup) \
           {
              "a"  \
              {
                 if [info exists m(name)] \
                 {
                    set names($m(name)) $index
                 }
                 if [info exists m(href)] \
                 {
                    set href_begin $index
                    set href_url $m(href)
                 }
              }
              "/a" \
              { 
                 if {$href_begin != ""} \
                 {
                    lappend links [list $href_begin $index $href_url]
                    set href_begin ""
                 }
              }
           }
           unset m
        }

     }
     Debug::Puts HTML 4 "links are $links"
     set htmlscreen($frame,links) $links

  }
  
   
};  # End of namespace HtmlFrame

# Initial setup and cleanup
 if [is_vxworks] {
    catch {
      file mkdir /usr/cache
      foreach f [glob -nocomplain /usr/cache/*] {
        Debug::Puts HTML 3 "Removing $f"
        file delete $f
      }
    }


}
