##############################################################################
# IFR SYSTEMS, INC          COPYRIGHT (C)1998-2001        DESIGN ENGINEERING #
# -------------------------------------------------------------------------- #
# FILE        : remote_cal.tcl                                               #
# PRODUCT     : FITS                                                         #
# AUTHOR      : Hieu Tran                                                    #
# DESCRIPTION : set up for remote operation on calibration data              #
#                                                                            #
##############################################################################

# Create calibration data namespace
namespace eval cal_data {

   namespace export c4fm_err tcxo_cal rfgen_cal_lvl rfgen_cal anlz_cal anlz \
                    rfgen_cal_lvl rfgencal_figure_level  \
                    rfgen_cal anlz_cal
                    
   #*************************************************
   # Name        : write_anlz_cal 
   # Parameters  : arg1 - analyzer characteristic 
   #               arg2 - index
   #               arg3 - frequency or frequency adjustment
   #               arg4 - receive power, normalize, or loss
   #               arg5 - power term 
   # Description : write analyzer calibration data 
   #*************************************************
   proc write_anlz_cal { arg1 arg2 arg3 arg4 arg5 } \
   {
      if { [ check_index_range $arg1 $arg2 $arg2 ] } \
      {
         error "$arg2: no such index in array"
         return
      }

      switch $arg1 \
      {
         bw_sw \
         {
            if { $arg4 != "" } \
            {
               set ::AnlzrCals($arg2.khz) [ ::format_numeric $arg3 3 ]
               set ::AnlzrCals($arg2.norm) [ ::format_numeric $arg4 2 ]
               do_anlzr_cals bwswitch $arg2 $::AnlzrCals($arg2.khz) $::AnlzrCals($arg2.norm) 0.0
               eval Analyzer span  $::Analyzer::State(span,raw)
               Analyzer topofscreen $::Analyzer::State(tos) -nowait 
            }
         }

         freq_cf \
         {
            if { $arg4 != "" } \
            {
               if { $arg5 != "" } \
               {
                  set ::AnlzrCals($arg2.cf.mhz) [ ::format_numeric $arg3 3 ]
                  set ::AnlzrCals($arg2.cf.dbm) [ ::format_numeric $arg4 2 ]
                  set ::AnlzrCals($arg2.cf.watts) [ ::format_numeric $arg5 4 ]
                  Receiver freq $::AnlzrCals($arg2.cf.mhz) MHz
               }
            }
         }

         tr2rec \
         {
            if { $arg4 != "" } \
            {
               set ::AnlzrCals($arg2.tr.mhz) [ ::format_numeric $arg3 3 ]
               set ::AnlzrCals($arg2.tr.db) [ ::format_numeric $arg4 2 ]
               put_anlzr_cal_data analyzer
               get_anlzr_cal_data analyzer
               Analyzer topofscreen $::Analyzer::State(tos) -nowait 
            }
         }

         ant2rec \
         {
            if { $arg4 != "" } \
            {
               set ::AnlzrCals($arg2.ant.mhz) [ ::format_numeric $arg3 3 ]
               set ::AnlzrCals($arg2.ant.db) [ ::format_numeric $arg4 2 ]
               put_anlzr_cal_data analyzer
               get_anlzr_cal_data analyzer
               Analyzer topofscreen $::Analyzer::State(tos) -nowait 
            }
         }
      }
   }


   #*************************************************
   # Name        : write_anlz_array 
   # Parameters  : arg - analyzer chars.
   #               index - array index
   #               arg1 - freq. or freq. adjustment
   #               arg2 - receive power, or loss
   #               arg3 - array index
   #               arr - rest of input in list
   # Description : write array of analyzer calibration data
   #*************************************************
   proc write_anlz_array { arg index arg1 arg2 arg3 arr } \
   {
      write_anlz_cal $arg $index $arg1 $arg2 

      set arr [ linsert $arr 0 $arg3 ]
      set j 0
      set max [ expr [ llength $arr ] / 3 ]

      # go through arr list and write cal. data 
      for { set i 0 } { $i < $max } { incr i } \
      {
         set index [ lindex $arr [ expr 0 + $j ] ]
         set arg1  [ lindex $arr [ expr 1 + $j ] ]
         set arg2  [ lindex $arr [ expr 2 + $j ] ]

         write_anlz_cal $arg $index $arg1 $arg2
         set j [ expr $j + 3 ]
      }
   }


   #*************************************************
   # Name        : write_anlz_array2 
   # Parameters  : arg - analyzer chars.
   #               index - array index
   #               arg1 - frequency
   #               arg2 - receive power
   #               arg3 - power term
   #               arr - rest of input in list
   # Description : write array of analyzer calibration data 
   #*************************************************
   proc write_anlz_array2 { arg index arg1 arg2 arg3 arr } \
   {
      write_anlz_cal $arg $index $arg1 $arg2 $arg3 

      set j 0
      set max [ expr [ llength $arr ] / 4 ]

      # go through arr list and write cal. data
      for { set i 0 } { $i < $max } { incr i } \
      {
         set index [ lindex $arr [ expr 0 + $j ] ]
         set arg1  [ lindex $arr [ expr 1 + $j ] ]
         set arg2  [ lindex $arr [ expr 2 + $j ] ]
         set arg3  [ lindex $arr [ expr 3 + $j ] ]

         write_anlz_cal $arg $index $arg1 $arg2 $arg3
         set j [ expr $j + 4 ]
      }
   }


   #*************************************************
   # Name        : check_index_range 
   # Parameters  : arg - analyzer chars.
   #               start - start index
   #               end - end index 
   # Description : check for valid index range for anlz. cal. 
   #*************************************************
   proc check_index_range { arg start end } \
   {
      switch $arg \
      {
         bw_sw \
         {
           if { $start < 1 || $start > $::AnlzrCals(max) || $end > [ expr $::AnlzrCals(max) + 1 ] } \
           {  
              return 1 
           } 
         }

         freq_cf \
         {
            if { $start < 0 || $start > $::AnlzrCals(cfmax) || $end > $::AnlzrCals(cfmax) } \
            { 
               return 1 
            } 
         }

         tr2rec \
         {
            if { $start < 0 || $start > $::AnlzrCals(trmax) || $end > $::AnlzrCals(trmax) } \
            { 
               return 1 
            }
         }

         ant2rec \
         {
            if { $start < 0 || $start > $::AnlzrCals(antmax) || $end > $::AnlzrCals(antmax) } \
            { 
               return 1
            }
         }
      }
   
      return 0
   }


   #*************************************************
   # Name        : save_anlzr_cal_data 
   # Parameters  : none 
   # Description : save analyzer calibration data 
   #*************************************************
   proc save_anlzr_cal_data { } \
   {
      set ::AnlzrCals(time) [ rfgencal::gencal_get_time ]
      set ::AnlzrCals(date) [ rfgencal::gencal_get_date ]
      put_anlzr_cal_data analyzer
   }


   #*************************************************
   # Name        : anlz_cal 
   # Parameters  : arg - caltime/caldate, analyzer chars., save, or screen 
   #               arg1 - index, readarray, or writearray
   #               arg2 - freq., freq. adj., or index
   #               arg3 - recv. pwr, normalize, or loss, freq.,
   #                      freq. adj., or index
   #               arg4 - power term, recv. pwr., normalize, or loss 
   #               arg5 - power term, or index
   #               args - rest of input in list
   # Description : read/write Spectrum Analyzer calibration
   #*************************************************
   proc anlz_cal { arg { arg1 "" } { arg2 "" } { arg3 "" } { arg4 "" } { arg5 "" } args } \
   {
      # read the array of calibration data
      if { [ string compare $arg1 "readarray" ] == 0 } \
      {
         set end [ expr $arg3 + 1 ]
         read_anlz_array $arg $arg2 $end 
         return
      }

      # write array of calibration data
      if { [ string compare $arg1 "writearray" ] == 0 } \
      {

         if { [ string compare $arg "freq_cf" ] == 0 } \
         {
            write_anlz_array2 $arg $arg2 $arg3 $arg4 $arg5 $args

         } \
         else \
         {
            write_anlz_array $arg $arg2 $arg3 $arg4 $arg5 $args
         }
         save_anlzr_cal_data
         return
      }

      # write calibration data
      if { $arg2 != "" } \
      {
         write_anlz_cal $arg $arg1 $arg2 $arg3 $arg4
         save_anlzr_cal_data
         return
      }

      # read calibration data, time, date, save cal. data, display ANLZ screen,
      # set RF input port, normalize, span, IFCHBW, IFBW, or sweep
      switch $arg \
      {
         normalize \
         {
           set_anlzr_normalize 1
         }

         time \
         {
            return $::AnlzrCals(time)
         }

         date \
         {
            return $::AnlzrCals(date)
         }

         save \
         {
            save_anlzr_cal_data
         }

         span \
         {
            if { $arg1 != "" } \
            {
               Analyzer span  $arg1 -1.0
            } \
            else \
            {
               puts $::Analyzer::State(span)
            }
         }

         tos \
         {
            if { $arg1 != "" } \
            { 
               Analyzer topofscreen $arg1
            } \
            else \
            {
               return $::Analyzer::State(tos)
            }
         }

         port \
         {
            if { $arg1 != "" } \
            {
               Receiver rfin $arg1
               set ::RcvrState(input) $arg1
            } \
            else \
            {
               return $::RcvrState(input) 
            }
         }

         ifchbw \
         {
            if { $arg1 != "" } \
            {
               set_anlzr_ifchbw $arg1
               set ::ChAnlzState(ifbw) $arg1
               anlz_table_index $arg1 
               doUpdateChanlzrCalTable
            } \
            else \
            {
               return $::ChAnlzState(ifbw)
            }
         }

         ifbw \
         {
            if { $arg1 != "" } \
            {
               set_rcvr_ifbw $arg1
               set ::RcvrState(ifbw) $arg1
               anlz_table_index $arg1
               doUpdateChanlzrCalTable
            } \
            else \
            {
               return $::RcvrState(ifbw)
            }
         }

         sweep \
         {
            if { $arg1 != "" } \
            {
               Analyzer sweep $arg1 2
               set ::Analyzer::State(sweep_time) $arg1
            } \
            else \
            {
               return $::Analyzer::State(sweep_time)
            }
         }

         bw_sw \
         {
            if { $arg1 != "" } \
            { 
               if { ![info exists ::AnlzrCals($arg1.khz)] || ![info exists ::AnlzrCals($arg1.norm)] } \ 
               { 
                  error "$arg1: no such index in array"
               }
               return "$arg1 $::AnlzrCals($arg1.khz) $::AnlzrCals($arg1.norm)" 
            } \
            else \
            {
                set max [ expr $::AnlzrCals(max) + 1 ]
                read_anlz_array $arg 1 $max
            }
         }

         freq_cf \
         {
            if { $arg1 != "" } \
            {
               if { ![info exists ::AnlzrCals($arg1.cf.mhz)] ||
                    ![info exists ::AnlzrCals($arg1.cf.dbm)] ||
                    ![info exists ::AnlzrCals($arg1.cf.watts)]
                  } \
               { 
                  error "$arg1: no such index in array"
               }
               return "$arg1 $::AnlzrCals($arg1.cf.mhz) $::AnlzrCals($arg1.cf.dbm) $::AnlzrCals($arg1.cf.watts)"
            } \
            else \
            {
               read_anlz_array $arg 0 $::AnlzrCals(cfmax)
            }
         }

         tr2rec \
         {
            if { $arg1 != "" } \
            {
              if { ![info exists ::AnlzrCals($arg1.tr.mhz)] || ![info exists ::AnlzrCals($arg1.tr.db)] } \ 
              { 
                error "$arg1: no such index in array"
              }
              return "$arg1 $::AnlzrCals($arg1.tr.mhz) $::AnlzrCals($arg1.tr.db)" 
            } \
            else \
            {
               read_anlz_array $arg 0 $::AnlzrCals(trmax)
            }
         }

         ant2rec \
         {
           if { $arg1 != "" } \
           {
              if { ![info exists ::AnlzrCals($arg1.ant.mhz)] || ![info exists ::AnlzrCals($arg1.ant.db)] } \
              { 
                 error "$arg1: no such index in array"
              }
              return "$arg1 $::AnlzrCals($arg1.ant.mhz) $::AnlzrCals($arg1.ant.db)"
           } \
           else \
           {
              read_anlz_array $arg 0 $::AnlzrCals(antmax)
           }
        }

        default \
        {
           error "$arg: no such command option"
        }
      }
   }


   #*************************************************
   # Name        : anlz 
   # Parameters  : see above 
   # Description : alias command for anlz_cal  
   #*************************************************
   proc anlz { arg { arg1 "" } { arg2 "" } { arg3 "" } { arg4 "" } { arg5 "" } args } \
   {
      anlz_cal $arg $arg1 $arg2 $arg3 $arg4 $arg5 $args
   }


   #*************************************************
   # Name        : anlz_table_index 
   # Parameters  : arg - bandwidth value 
   # Description : set index for specify bandwidth 
   #*************************************************
   proc anlz_table_index { arg } \
   {
      switch $arg \
      {
         300HZ \
         {
            set ::AnlzrCalTableIndex 1
         }

         3KHZ \
         {
            set ::AnlzrCalTableIndex 2
         }

         60KHZ \
         {
            set ::AnlzrCalTableIndex 3
         }

         30KHZ \
         {
            set ::AnlzrCalTableIndex 4
         }

         300KHZ \
         {
            set ::AnlzrCalTableIndex 5
         }

         3MHZ \
         {
           set ::AnlzrCalTableIndex 6
         }
      }
   }

      
   #*************************************************
   # Name        : read_anlz_array 
   # Parameters  : arg - analyzer characteristic
   #               start - start array index
   #               end - end array index
   # Description : read array of analyzer calibration
   #*************************************************
   proc read_anlz_array { arg start end } \
   { 
      if { [ check_index_range $arg $start $end ] } \
      {
         error "Index out of range"
         return
      }

      switch $arg \
      {
         bw_sw \
         { 
            for { set i $start } { $i < $end } { incr i } \
            {
               append retval "$i $::AnlzrCals($i.khz) $::AnlzrCals($i.norm)" 
            }
            return $retval
         }

         freq_cf \
         {
            for { set i $start } { $i < $end } { incr i } \
            {
               append retval "$i $::AnlzrCals($i.cf.mhz) $::AnlzrCals($i.cf.dbm) $::AnlzrCals($i.cf.watts)" 
            }
            return $retval
         }

         tr2rec \
         {
            for { set i $start } { $i < $end } { incr i } \
            {
               append retval "$i $::AnlzrCals($i.tr.mhz) $::AnlzrCals($i.tr.db)"
            }
            return $retval
         }

         ant2rec \
         {
            for { set i $start } { $i < $end } { incr i } \
            {
               append retval "$i $::AnlzrCals($i.ant.mhz) $::AnlzrCals($i.ant.db)"
            }
            return $retval
         }
      }
   }


   #*************************************************
   # Name        : read_rfgen_array 
   # Parameters  : rfport - TR or GEN
   #               db - dB atten.
   #               start - start array index
   #               end - end array index
   # Description : read array of RF GEN freq. level calibration
   #*************************************************
   proc read_rfgen_array { rfport db start end } \
   {
      if { $end > $::RFGEN_ENGINE(FreqMaxIndex) } \
      {
         error "$end: ending index out of range"

      }
      if { $start < 0 } \
      {
         error "$start: starting index out of range"

      }
      for { set i $start } { $i < $end } { incr i } \
      {
            append retval "$i $::GENCAL(freq,$rfport,freq,$i) $::GENCAL(freq,$rfport,$db,abs,$i)" 
      }
      return $retval
   }


   #*************************************************
   # Name        : write_rfgen_cal 
   # Parameters  : rfport - TR or GEN
   #               db - dB atten.
   #               arg1 - index
   #               arg2 - frequency
   #               arg3 - level
   # Description : write RF GEN freq. level calibration
   #*************************************************
   proc write_rfgen_cal { rfport db arg1 { arg2 "" } { arg3 "" } } \
   { 
      if { $arg1 > $::RFGEN_ENGINE(FreqMaxIndex) || $arg1 < 0 } \
      {
         error "$arg1: no such index in array"
      }

      if { $arg2 != "" } \
      {
         if { $arg3 != "" } \
         {
            set ::GENCAL(freq,$rfport,freq,$arg1) [ ::format_numeric $arg2 0 ]
            set ::GENCAL(freq,$rfport,$db,abs,$arg1) [ ::format_numeric $arg3 2 ]
            return 0
         }
      }
      return 1
   }


   #*************************************************
   # Name        : write_rfgen_array 
   # Parameters  : rfport - TR or GEN
   #               arg - dB atten.
   #               index - array index
   #               arg1 - frequency
   #               arg2 - level
   #               arr - rest of input in list
   # Description : write an array of value for RF Gen freq. 
   #               level calibration 
   #*************************************************
   proc write_rfgen_array { rfport arg index arg1 arg2 arr } \
   {
      write_rfgen_cal $rfport $arg $index $arg1 $arg2

      set j 0
      set max [ expr [ llength $arr ] / 3 ]

      # go through arr list and write cal. data
      for { set i 0 } { $i < $max } { incr i } \
      {
         set index [ lindex $arr [ expr 0 + $j ] ]
         set arg1  [ lindex $arr [ expr 1 + $j ] ]
         set arg2  [ lindex $arr [ expr 2 + $j ] ]

         write_rfgen_cal $rfport $arg $index $arg1 $arg2
         set j [ expr $j + 3 ]
      }
   }


   #*************************************************
   # Name        : rfgen_cal 
   # Parameters  : rfport - TR or GEN
   #               arg - caltime/caldate, save, or dB atten.  
   #               arg1 - index, readarray, or writearray
   #               arg2 - frequency, or index
   #               arg3 - level, or frequency
   #               arg4 - level
   #               args - rest of input in list
   # Description : read/write RF GEN freq. level calibration
   #*************************************************
   proc rfgen_cal { rfport arg { arg1 "" } { arg2 "" } { arg3 "" } { arg4 "" } args } \
   {
      # read cal. date, time, or save cal. data
      switch $arg \
      {
         date \
         {
            return $::GENCAL($rfport.caldate)  
         }

         time \
         {
            return $::GENCAL($rfport.caltime)
         }

         save \
         {
           ::rfgencal_save $rfport
           return
         }
      }

      # read array of cal. data
      if { [ string compare $arg1 "readarray" ] == 0 } \
      {
         read_rfgen_array $rfport $arg $arg2 [ expr $arg3 + 1 ]
         return
      }

      # write array of cal. data
      if { [ string compare $arg1 "writearray" ] == 0 } \
      {
         write_rfgen_array $rfport $arg $arg2 $arg3 $arg4 $args
         ::rfgencal_save $rfport
         return
      }

      # list all calibration data
      if { $arg1 == "" } \
      {
         read_rfgen_array $rfport $arg 0 $::RFGEN_ENGINE(FreqMaxIndex)
         return

      # read/write calibration data
      } \
      else \
      {
         if { [ write_rfgen_cal $rfport $arg $arg1 $arg2 $arg3 ] == 1 } \
         {
            if { ![info exists ::GENCAL(freq,$rfport,freq,$arg1)] ||
                 ![info exists ::GENCAL(freq,$rfport,$arg,abs,$arg1)] 
               } \
            {
               error "$arg1: no such index in array"
            }
            return "$arg1 $::GENCAL(freq,$rfport,freq,$arg1) $::GENCAL(freq,$rfport,$arg,abs,$arg1)"
         } \
         else \
         {
            ::rfgencal_save $rfport
         }
      }
   }


   #*************************************************
   # Name        : write_cal_lvl_array 
   # Parameters  : index - leveler index
   #               arg1 - ALC level
   #               arg2 - level out
   #               arr - rest of input data in list
   # Description : write array of RF GEN ALC leveler calibration 
   #*************************************************
   proc write_cal_lvl_array { index arg1 arg2 arr } \
   {
      # write calibration data if index is valid
      if { $index >= $::RFGEN_ENGINE(AlcMaxIndex) || $index < 0 } \
      {
         error "$index: no such index in array"

      } \
      else \
      {
         set ::GENCAL(level,alc,$index) [ ::format_numeric $arg1 0 ]
         set ::GENCAL(level,abs,$index) [ ::format_numeric $arg2 2 ]
      }

      set j 0
      set max [ expr [ llength $arr ] / 3 ]

      # go through arr list and write calibration data
      for { set i 0 } { $i < $max } { incr i } \
      {
         set index [ lindex $arr [ expr 0 + $j ] ]
         set arg1  [ lindex $arr [ expr 1 + $j ] ]
         set arg2  [ lindex $arr [ expr 2 + $j ] ]

         if { $index >= $::RFGEN_ENGINE(AlcMaxIndex) || $index < 0 } \
         {
            error "$index: no such index in array"

         } \
         else \
         {
            set ::GENCAL(level,alc,$index) [ ::format_numeric $arg1 0 ]
            set ::GENCAL(level,abs,$index) [ ::format_numeric $arg2 2 ]
         }

         set j [ expr $j + 3 ]
      }
   }


   #*************************************************
   # Name        : read_cal_lvl_array 
   # Parameters  : arg1 - start index
   #               arg2 - end index
   # Description : read array of RF GEN ALC level calibration 
   #*************************************************
   proc read_cal_lvl_array { arg1 arg2 } \
   {
      if { $arg2 >= $::RFGEN_ENGINE(AlcMaxIndex) } \
      {
         error "$arg2: end index out of range"
   
      } \
      elseif { $arg1 < 0 } \
      {
         error "$arg1: start index out of range"
      }

      for { set i $arg1 } { $i <= $arg2 } { incr i } \
      {
         append retval "$i $::GENCAL(level,alc,$i) $::GENCAL(level,abs,$i)" 
      }
      return $retval
   }


   #*************************************************
   # Name        : rfgen_lvl_option 
   # Parameters  : arg1 - option switch 
   #               arg2 - index  
   #               arg3 - index, or ALC level 
   #               arg4 - level out
   #               arr - rest of input in list
   # Description : execute the choice selection of RF GEN 
   #               ALC leveler calibration 
   #*************************************************
   proc rfgen_lvl_option { arg1 { arg2 "" } { arg3 "" } { arg4 "" } arr } \
   {
      switch $arg1 \
      {
         save \
         {
            rfgencal_save ALC  
         }

         time \
         {
            return $::GENCAL(level.caltime)
         }

         date \
         {
            return $::GENCAL(level.caldate)
         }

         readarray \
         {
            return [read_cal_lvl_array $arg2 $arg3]
         }

         writearray \
         {
            write_cal_lvl_array $arg2 $arg3 $arg4 $arr
         }

         default \
         {
            return 1
         }
      }

      return 0
   }


   #*************************************************
   # Name        : rfgen_cal_lvl 
   # Parameters  : arg1 - caltime/caldate, index, or save
   #               arg2 - ALC level or index
   #               arg3 - level out or ALC level
   #               arg4 - level out
   #               args - rest of input data in list 
   # Description : read/write RF GEN ALC leveler calibration 
   #*************************************************
   proc rfgen_cal_lvl { { arg1 "" } { arg2 "" } { arg3 "" } { arg4 "" } args } \
   {
      # list all calibration values
      if { $arg1 == "" } \
      {
         for { set i 0 } { $i < $::RFGEN_ENGINE(AlcMaxIndex) } { incr i }\
         {
            append retval "$i $::GENCAL(level,alc,$i) $::GENCAL(level,abs,$i)" 
         }
         return $retval

      }
      
      if { [ rfgen_lvl_option $arg1 $arg2 $arg3 $arg4 $args ] == 0 } \
      {
         return
      }

      # write calibration values
      if { $arg2 != "" } \
      {
         if { $arg3 != "" } \
         {
            set ::GENCAL(level,alc,$arg1) [ ::format_numeric $arg2 0 ]
            set ::GENCAL(level,abs,$arg1) [ ::format_numeric $arg3 2 ]
         }

         return
      }

      if { ![info exists ::GENCAL(level,alc,$arg1)] || ![info exists ::GENCAL(level,abs,$arg1)]} \
      {
         error "$arg1: no such index in array"
      }
      return "$arg1 $::GENCAL(level,alc,$arg1) $::GENCAL(level,abs,$arg1)" 
   }


   #*************************************************
   # Name        : tcxo_cal  
   # Parameters  : TCXO value 
   # Description : read/write TCXO value 
   #*************************************************
   proc tcxo_cal { { arg1 "" } } \
   {
      if { $arg1 == "" } \
      {
         ::read_yigtcxo_cals
         return $::cal(tcxo)

      } 
      set ::cal(tcxo) [ ::format_numeric $arg1 0 ]
      ::write_yigtcxo_cals tcxo
      ::set_hwutil_tcxo $::cal(tcxo) 
   }
     

   #*************************************************
   # Name        : c4fm_err  
   # Parameters  : percent of error 
   # Description : read/write C4FM error 
   #*************************************************
   proc c4fm_err { { arg1 "" } } \
   {
      if { $arg1 == "" } \
      {
         return $::c4FM_error

      } 
      set ::c4FM_error [ ::format_numeric $arg1 1 ]
      ::set_P25EVM_level $::c4FM_error 
   }
   proc gencal_freq { freq } {Generator debug rawfreq $freq}
}

# import procedures from cal_data namespace
namespace import cal_data::*
