#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C) 2002      DESIGN ENGINEERING        #
# --------------------------------------------------------------------------#
# FILE        : 2002.VolSquelch.tcl                                         #
# PRODUCT     : FITS                                                        #
# AUTHOR      : David Hagood                                                #
# DESCRIPTION : Implements the Vol/Squelch window, which shows and allows   #
#               changing of the Volume and squelch settings                 #
#############################################################################
namespace eval VolSquelch \
{
   if ![winfo exists ::sys(vol_win_on_knob)] {set ::sys(vol_win_on_knob) ON}
   switch -- $::sys(vol_win_on_knob) \
   {
     ON {}
     OFF {}
     default { set ::sys(vol_win_on_knob) ON}
   }
   
   # BIND VOLUME KNOB TO VOLUME CONTROL
   bind all <<Vol_down>> {::VolSquelch::vol_down; ::VolSquelch::Show %W 1}
   bind all <<Vol_up>>   {::VolSquelch::vol_up; ::VolSquelch::Show %W 1}
 
   # BIND SQUELCH KNOB TO VOLUME CONTROL
   bind all <<Squ_down>> {::Receiver::update_squelch_level down; ::VolSquelch::Show %W 1}
   bind all <<Squ_up>>   {::Receiver::update_squelch_level up; ::VolSquelch::Show %W 1}


  bind <Unmap> { event generate %W <<Unmap>>}
  
  bind VOL_SQUELCH <<Vol_down>> {::VolSquelch::vol_down; VolSquelch::restart_delay %W; break}
  bind VOL_SQUELCH <<Vol_up>>   {::VolSquelch::vol_up; VolSquelch::restart_delay %W; break}
  bind VOL_SQUELCH <<Squ_down>> {::Receiver::update_squelch_level down; VolSquelch::restart_delay %W; break}
  bind VOL_SQUELCH <<Squ_up>>   {::Receiver::update_squelch_level up; VolSquelch::restart_delay %W; break}
  bind VOL_SQUELCH <<Help>>     {::VolSquelch::Show_help %W; break}

  proc vol_up { } \
  {
     variable State
     incr ::BBAudio::State(volume_raw)
     after cancel VolSquelch::vol_core
     after idle   VolSquelch::vol_core
  }

  proc vol_down { } \
  {
     variable State
     incr ::BBAudio::State(volume_raw) -1
     after cancel VolSquelch::vol_core
     after idle   VolSquelch::vol_core
  }

  proc vol_core {} \
  {
     BBAudio speakervol $::BBAudio::State(volume_raw) -nowait
  }
   
  #****************************************************************************
  #
  # Name:        Show_help
  #
  # Parameters:  widget - item help requested for
  #
  # Description: Show help for us - since we hide, we have to hide BEFORE
  #              the help comes up, or we screw the focus
  #
  #****************************************************************************
  proc Show_help { widget } \
  {
    set toplevel [winfo toplevel $widget]
    wm withdraw $toplevel
    set cmd "::help::show $widget"
    after cancel $cmd
    after idle $cmd
  }
  
  #****************************************************************************
  #
  # Name:        Create_tile
  #
  # Parameters:  tile_path - where to create our widget
  #
  # Description: Create and bind the tile
  #
  #****************************************************************************
  proc Create_tile { tile_path } \
  {
     if ![winfo exists $tile_path] \
     {
        ::Screens::create_toplevel_tile_hidden  $tile_path  VOL_SQUELCH 
        bind $tile_path <KeyRelease-F8>     "wm withdraw $tile_path"
        bind $tile_path <KeyRelease-Return> "wm withdraw $tile_path"
        bind $tile_path <Destroy> "after cancel \"wm withdraw $tile_path\""
     }
  }
  
  #****************************************************************************
  #
  # Name:        Create
  #
  # Parameters:  toplevel - where to create our widget
  #
  # Description: Create (but not show) the volume/Squelch window
  #
  #****************************************************************************
  proc Create { toplevel } \
  {
     set top [View::Find_Main_toplevel $toplevel]
     set tile_path [name_child $top vol_squelch]
     Create_tile $tile_path
  }
  #****************************************************************************
  #
  # Name:        Toggle_window
  #
  # Parameters:  none
  #
  # Description: Toggle the state of the "show vol knob"
  #
  #****************************************************************************
  proc Toggle_window {} \
  {
     if { $::sys(vol_win_on_knob) != "ON" } \
     {
        set ::sys(vol_win_on_knob) "ON"
     } \
     else \
     {
        set ::sys(vol_win_on_knob) "OFF"
     }
  }
  
  #****************************************************************************
  #
  # Name:        Show
  #
  # Parameters:  widget - widget that activated us
  #
  # Description: Show the volume/Squelch window
  #
  #****************************************************************************
  proc Show { widget {knob ""} } \
  {
     # if this is from a knob, and we have that turned off, don't do it
     if {$knob != ""} \
     {
        if { $::sys(vol_win_on_knob) != "ON" } { return }
     }
     set toplevel [winfo toplevel $widget]
     # work upwards to the master level
     set screen_of_me [winfo screen $toplevel]
     while {1 } \
     {
        if {$toplevel == "."} { break }
        set next [winfo parent $toplevel]
        if {[winfo screen $next] != $screen_of_me} { break }
        set toplevel $next
     }

     set tile_path [name_child $toplevel vol_squelch]
     
     if ![winfo exists $tile_path] \
     {
        Create_tile $tile_path
     }
     bind $tile_path <<Unmap>> "VolSquelch::Cleanup $tile_path $widget"
     restart_delay $tile_path
  }
  #****************************************************************************
  #
  # Name:        Cleanup
  #
  # Parameters:  Toplevel of window, who to focus
  #
  # Description: Clean up when we hide
  #
  #****************************************************************************
  proc Cleanup { toplevel focus } \
  {
     bind $toplevel <<Unmap>> {}
     
     after idle  "safe_focus_force $focus"
  }
  
  #****************************************************************************
  #
  # Name:        restart_delay
  #
  # Parameters:  widget within window
  #
  # Description: Restart our delay before we auto-destruct
  #
  #****************************************************************************
  proc restart_delay { widget } \
  {
     set toplevel [winfo toplevel $widget]
     set cmd "wm withdraw $toplevel"
     wm deiconify $toplevel
     after cancel $cmd
     after 3000 $cmd
  }
  #****************************************************************************
  #
  # Name:        adjust_volume
  #
  # Parameters:  new volume value
  #
  # Description: 
  #
  #****************************************************************************
  proc adjust_volume { widget volume } \
  {
     restart_delay $widget
     BBAudio speakervolpercent $volume
  }
  #****************************************************************************
  #
  # Name:        adjust_squelch
  #
  # Parameters:  new squelch value
  #
  # Description: 
  #
  #****************************************************************************
  proc adjust_squelch { widget squelch } \
  {
     restart_delay $widget
     Receiver squelchmode auto
     Receiver squelchlevel $squelch -nowait
  }
}
