##############################################################################
# IFR SYSTEMS, INC          COPYRIGHT (C)1998-2001        DESIGN ENGINEERING #
# -------------------------------------------------------------------------- #
# FILE        : 9000_httpd.tcl                                               #
# PRODUCT     : FITS                                                         #
# AUTHOR      : David Hagood                                                 #
# DESCRIPTION : a really stupid HTTPD server                                 #
#                                                                            #
##############################################################################
package require Capture
namespace eval HTTPD \
{
   variable counter 0
   
   namespace import ::HtmlFrame::decode ::HtmlFrame::split_url ::HtmlFrame::encode 
   namespace import ::HtmlFrame::find_index_file ::HtmlFrame::do_dir
   variable State
   
   array set State \
   {
      enable 1
      enable,text ENABLED
      socket ""
   }
   
   variable main_screen_items
   variable main_screen_readings
   variable indexes
   
   set main_screen_items(3) HTTPD
   
   set indexes $::HtmlFrame::indexes
   
   variable hidden_files 
   set hidden_files { /usr/cache /usr/cache/* */CVS* /usr/analyzer /usr/analzyer/* }
   
   variable GET_handlers
   array set GET_handlers \
   {
      /help/*   Get_help
      /         Get_main
      /other/*  Get_other
      /usr/*    Get_usr
      /fonts/*  Get_fonts
      /icons/*  Get_icons
      /REDIRECT/ Get_redirect
      /REDIRECT  Get_redirect
      /SNAPSHOT/ Get_snapshot
      /SNAPSHOT  Get_snapshot
      /screen.gif Get_screen
   }
   array set POST_handlers \
   {
      /UPLOAD/   Post_upload
      /DELETE/   Post_delete
      /FIRMWARE/ Post_reload
      /MKDIR/    Post_mkdir
   }
   variable file_handlers
   array set file_handlers \
   {
     .sav        Get_directory_entry_sav
     .gif        Get_directory_entry_img
     .jpg        Get_directory_entry_img
     .jpeg       Get_directory_entry_img
     .bmp        Get_directory_entry_img
     .arb        Get_directory_entry_waveform
   }
   
   variable verboten_files_and_paths \
   {
      /usr/reload
   }
   
   variable http_errs
   array set http_errs \
   {
       200 OK
       404 {Not found}
       100 {Continue}
       101 {Switching Protocols}
       200 {OK}
       201 {Created}
       202 {Accepted}
       203 {Non-Authoritative Information}
       204 {No Content}
       205 {Reset Content}
       206 {Partial Content}
       300 {Multiple Choices}
       301 {Moved Permanently}
       302 {Found}
       303 {See Other}
       304 {Not Modified}
       305 {Use Proxy}
       307 {Temporary Redirect}
       400 {Bad Request}
       401 {Unauthorized}
       402 {Payment Required}
       403 {Forbidden}
       404 {Not Found}
       405 {Method Not Allowed}
       406 {Not Acceptable}
       407 {Proxy Authentication Required}
       408 {Request Time-out}
       409 {Conflict}
       410 {Gone}
       411 {Length Required}
       412 {Precondition Failed}
       413 {Request Entity Too Large}
       414 {Request-URI Too Large}
       415 {Unsupported Media Type}
       416 {Requested range not satisfiable}
       417 {Expectation Failed}
       500 {Internal Server Error}
       501 {Not Implemented}
       502 {Bad Gateway}
       503 {Service Unavailable}
       504 {Gateway Time-out}
       505 {HTTP Version not supported}
   }
   
   variable mime_types
   array set mime_types \
   {
      *.gif {image/gif}
      *.htm {text/html}
      *.csv {text/x-comma-separated-values}
      *.html {text/html}
      *.shtml {text/html}
      *.txt  {text/plain}
      *.log  {text/plain}
      *.tcl  {text/html}
      *.mpg  {video/mpeg}
      *.mpeg {video/mpeg}
      *.mp3  {audio/mpeg}
      *.pdf   {application/pdf}
      /usr/errorlog* {text/plain}
      /usr/logfiles/* {text/plain}
      /usr/setups* {text/plain}
      *.frq {text/plain}
   }
   proc stop_server { } \
   {
      variable State
      if {"$State(socket)" != ""} \
      {
          close $State(socket); 
          set State(socket) ""
      }
   }
   proc server { {port 80} } \
   {
      variable State
      stop_server
      set State(socket) [socket -server ::HTTPD::Request $port]
      Debug::Puts HTTPD 1  "listening on $port"
   }
   
   proc Toggle_enable { } \
   {
      variable State
      if {$State(enable)} \
      {
         set State(enable) 0
      } \
      else \
      {
         set State(enable) 1
      }
      Update_enable
      catch \
      {
         set f [open /system/etc/httpd.conf w]
         set x \
         {
            namespace eval ::HTTPD \
            {
               set State(enable) $State(enable)
               Update_enable
            }
         }
         puts $f [subst $x]
         close $f
      }
   }
   
   proc Update_enable { } \
   {
      variable State
      if {$State(enable)} \
      {
         set State(enable,text) "ENABLED"
         server 80
      } \
      else \
      {
         set State(enable,text) "DISABLED"
         stop_server
      }
   }
   
   proc Request { fd addr port } \
   {
      variable counter
      set name ::HTTPD::$counter
      incr counter
      Debug::Puts HTTPD 1  "Request $fd $addr $port - $name"
      upvar #0 $name this
      
      array set this \
      "
         fd        $fd
         requester $addr
         port      $port
         name      $name
         exec_allowed 1
         upload_allowed 0
         delete_allowed 0
         index_allowed  0
      "
      fconfigure $this(fd) -buffering full -buffersize 65536
      set this(KeepAlive) 0
      # Read the headers from the caller
      Get_headers 
      
      # evaluate and respond
      Respond
   }
   
   proc Respond { } \
   {
       upvar this this
       Debug::Puts HTTPD 1  "Respond $this(name)"
       if [catch \
       {
          if     [info exists this(GET)]  { Get} \
          elseif [info exists this(HEAD)] { set this(GET) $this(HEAD); Get} \
          elseif [info exists this(POST)] { Post} \
          else {Error 500}
       } err] \
       {
          Error 500
       }
       Delete $this(name)
   }

   proc Redirect { url } \
   {
       upvar this this
       puts $this(fd) "HTTP/1.1 301 Moved Permanently"
       puts $this(fd) "Location: $url"
       puts $this(fd) "Content-type: text/html\n"
       puts $this(fd) [subst \
       {<html><head><title>HTTP/1.1 301 Moved Permanently</title></head><body>Moved to <a href="$url">$url</a></body></html>}
       ]
   }
   
   proc Redirect { url } \
   {
       upvar this this
       puts $this(fd) "HTTP/1.1 301 Moved Permanently"
       puts $this(fd) "Location: $url"
       puts $this(fd) "Content-type: text/html\n"
       puts $this(fd) [subst \
       {<html><head><title>HTTP/1.1 301 Moved Permanently</title></head><body>Moved to <a href="$url">$url</a></body></html>}
       ]
   }
   
   proc Post { } \
   {
       upvar this this
       if [regexp {([^ ?]*)\??([^ ]*)[ ]*(.*)} $this(POST) match this(url) this(args) this(proto)] \
       {
          catch {Parse_CGI; Post_URL }
       } \
       else \
       {
          Error 500
       }
   }
   proc Post_URL { } \
   {
       upvar this this
       variable POST_handlers
       
       if [string match *../* $this(url)] \
       {
          return [Error 404]
       }
       
       foreach handler [array names POST_handlers] \
       {
          Debug::Puts HTTPD 1  "Considering $handler for $this(url)"
          
          if [string match $handler $this(url)] \
          {
             return [$POST_handlers($handler)]
          }
       }
       Error 404
   }
   proc Post_reload { } \
   {
       upvar this this
       set this(upload_allowed) 1
       Post_core
       set fname $this(form-data,TheFile)
       set t [file tail $fname]
       if {$t != "options.new"} \
       {
          set dest /usr/reload/reload.tar.gz
          file delete $dest
          file rename $fname $dest
       }
       Debug::Puts HTTPD 1  "Read done"
       set this(next_url) /help/Options/version.shtml
       Get_file /sysdata/html/uploaded.shtml
       Debug::Puts HTTPD 1  "Sent reply"
   }   
   proc Post_upload { } \
   {
       upvar this this
       set this(upload_allowed) 1
       Post_core
       Debug::Puts HTTPD 1  "Read done"
       set this(next_url) "$this(form-data,dir)/"
       Get_file /sysdata/html/uploaded.shtml
       Debug::Puts HTTPD 1  "Sent reply"
   }
   proc Post_mkdir { } \
   {
       upvar this this
       set this(upload_allowed) 1
       Post_core
       Debug::Puts HTTPD 1  "Read done. dir $this(form-data,dir) name $this(form-data,newdir)"
       if {$this(form-data,newdir) == ""} \
       {
          Redirect $this(form-data,dir)/
          return
       }
       set dir [file join $this(form-data,dir) $this(form-data,newdir)]
       Debug::Puts HTTPD 1  "new dir $dir"

       # look for relative paths, and stomp on them
       if [string match *../* $dir] \
       {
          return [Error 404]
       }
       # insure in user dir
       if ![string match /usr/* $dir] \
       {
          return [Error 404]
       }
       file mkdir $dir
       Redirect $dir
       Debug::Puts HTTPD 1  "Sent reply"
   }
   proc Post_delete { } \
   {
       upvar this this
       set this(upload_allowed) 0
       set this(delete_allowed) 1
       Post_core
       # if no files given, just redirect and exit
       if ![info exists this(form-data,WalkingDead)] \
       {
          Redirect "$this(form-data,dir)/"
          return
       }
       # if they haven't answered the question...
       if ![info exists this(form-data,IsThatYourFinalAnswer)] \
       {
          Debug::Puts HTTPD 1  "Prompting"
          Get_file /sysdata/html/delete_confirm.shtml
          return
       }
       Debug::Puts HTTPD 1  "Is that your final answer: $this(form-data,IsThatYourFinalAnswer)"
       # OK, they answered, one way or the other:
       if {$this(form-data,IsThatYourFinalAnswer) == "DELETE"} \
       {
           Debug::Puts HTTPD 1  "Deleting files"
           set dir $this(form-data,dir)
           foreach file $this(form-data,WalkingDead) \
           {
              set fn [file join $dir $file]
              catch {file delete $fn}
           }

       }
       Redirect "$this(form-data,dir)/"
       return
   }   
   
   proc Post_core { } \
   {
       upvar this this
       if ![regexp {multipart/form-data;.*boundary=(.*)} $this(Content-Type) foo this(boundary)] \
       {
          Error 500
          return
       }
       Debug::Puts HTTPD 5 "boundary is $this(boundary)"
       
       package require multipart_form
       set this(multipart_form) \
       [
         multipart_form $this(fd) -fd $this(fd) -len $this(Content-Length) \
                       -bound $this(boundary)
       ]
       
       Debug::Puts HTTPD 1  "POST $this(name) $this(Content-Length) bytes handler $this(multipart_form)"
       while ![$this(multipart_form) eof] \
       {
          if [catch Post_process err ] \
          {
              Debug::Puts HTTPD 1  "Error in Post_core: $err"
              return -code return 
          }
       }
   }
   
   proc Post_process { } \
   {
       upvar this this
       upvar 0 this(multipart_form) form
       Debug::Puts HTTPD 5 "Parsing section"
       if [$form eof] \
       {
          Debug::Puts HTTPD 2 "EOF on form"
          return
       }
       # read headers until empty line
       set line [$form get]
       Debug::Puts HTTPD 10 "Got $line"
       while {$line != ""} \
       {
          if [$form was_boundary] \
          {
             Debug::Puts HTTPD 2 "Boundary found"
             return
          }
          if {$line == $this(boundary)} \
          {
             Debug::Puts HTTPD 2 "Boundary found by TCL"
             return
          }
          # break the header down into its parts
          if ![regexp {([^ :]*):?[ ]?([^ ;]*)[; ]?(.*)} $line foo header parm data] \
          {
             Debug::Puts HTTPD 2 "Header error $line"
             return
          }
          set this($header) $parm
          Debug::Puts HTTPD 5 "Header $header $parm $data"
          if {$data != "" } \
          {
            # got some data - so chew it up too
            regsub -all {[;=]} $data " " data
            foreach { index value} $data \
            {
               set this($parm,$index) $value
               Debug::Puts HTTPD 5 "subheader $parm,$index $value"
            }
          }
          if [$form eof] \
          {
             Debug::Puts HTTPD 1 "EOF reading headers?"
             return
          }
          set line [$form get]
       }
       # OK, read the headers. Now read the data. 2 possiblities:
       # 1) this is just data, in which we read into the structure
       # 2) This is a file, in which case we save to disk
       # We assume if we were passed a filename it's a file
       if [info exists this(form-data,filename)] \
       {
          if {!$this(upload_allowed)} \
          {
             Error 403
             error "File upload not allowed"
          }
          # strip off certain stupid browsers' idea of path-ing...
          regsub -all {\\} $this(form-data,filename) "/" filename_and_path
          set filename [file tail $filename_and_path]
          set filename_and_path [file join $this(form-data,dir) $filename]
          Debug::Puts HTTPD 2 "filename_and_path $filename_and_path"
          Debug::Puts HTTPD 2 "filename $filename"
          if {$filename == "" } \
          {
             set this(next_url) $this(form-data,dir)/
             Get_file /sysdata/html/no_upload.shtml
             error "no file given"
          }
          # make sure it's not a reserved name
          variable indexes
          variable verboten_files_and_paths
          if {[lsearch -exact $verboten_files_and_paths $filename_and_path] != -1} \
          {
             Error 403
             return [error "Not allowed to upload special files"]
          }
          if {[lsearch -exact $indexes $filename] != -1} \
          {
             # only one special case - IF file is index.html, and
             # IF the directory is under /usr/AppNotes, then
             # we will allow this...
             if {($filename != "index.html") || 
                 ![string match /usr/AppNotes* $this(form-data,dir)]
                } \
             {
               Error 403
               return [error "Not allowed to upload special files"]
             }
          }
          set filename [file join $this(form-data,dir) $filename]
          set name "form-data,$this(form-data,name)"
          set this($name) $filename
          Debug::Puts HTTPD 1 "$name - Writing $filename"
          set start [clock clicks]
          set start_bytes [$form count]
          $form save $filename
          set stop [clock clicks]
          set stop_bytes [$form count]
          set time [expr ($stop - $start)/1000000.0]
          set len  [expr $stop_bytes - $start_bytes]
          Debug::Puts HTTPD 1 "Time $time seconds, $len bytes"
          Debug::Puts HTTPD 1 "boundary found = [$form was]"
          if {$time > 0} \
          {
             Debug::Puts HTTPD 1 "Throughput [expr $len/$time] bytes/sec"
          }
          Debug::Puts HTTPD 1 "done Writing $filename"
       } \
       else \
       {
         # OK, plain old stuff. Store into the structure
         set name "form-data,$this(form-data,name)"
         Debug::Puts HTTPD 2 "Saving data to $name"
         set d [$form get]
         while {![$form eof] && ![$form was_bound]} \
         {
            if {[info exists this($name)] && ($this($name)!="")}\
            {
               append this($name) \n "$d"
            } \
            else \
            {
               set this($name) "$d"
            }
            set d [$form get]
         }
         Debug::Puts HTTPD 2 "Done saving data to $name - $this($name)"
       }
   }
   
   proc Error { code } \
   {
       upvar this this
       set this(exec_allowed) 1
       Debug::Puts HTTPD 1  "Sending an error $code"
       set file /sysdata/html/errors/$code.html
       if [file exists $file] \
       {
          return [Get_file $file $code]
       }
       set file /sysdata/html/errors/$code.shtml
       if [file exists $file] \
       {
          return [Get_file $file $code]
       }
       return [Get_file /sysdata/html/errors/unknown_err.shtml $code]
   }
   
   proc Delete {name } \
   {
      upvar this this
      Debug::Puts HTTPD 1  "Delete $name"
      if [info exists this(multipart_form)] { $this(multipart_form) done}
      catch { close $this(fd) }
      unset $this(name)
   }
   
   proc Get_headers {  } \
   {
      upvar this this
      upvar 0 this(fd) fd
      
      while {"[set line [gets $fd]]" != ""} \
      {
         Debug::Puts HTTPD 11  "Got header $line"
         if [regexp {([^: ]*):?[ ]*(.*)} $line match header value] \
         {
            set this($header) $value
            Debug::Puts HTTPD 10  "Header $header: $value"
         }
      }
      Debug::Puts HTTPD 1  "End of headers"
   }
   proc Parse_CGI { } \
   {
       upvar this this
       set arglist [split $this(args) &=]
       foreach { arg value } $arglist \
       {
          set this(CGI,$arg) [decode $value]
          Debug::Puts HTTPD 5 "CGI parm $arg $value"
       }
       set this(url) [decode $this(url)]
   }
   
   proc Get {  } \
   {
       upvar this this
       Debug::Puts HTTPD 1  "GET $this(GET)"
       
       if [regexp {([^ ?]*)\??([^ ]*)[ ]*(.*)} $this(GET) match this(url) this(args) this(proto)] \
       {
          catch { Parse_CGI; Get_URL }
       } \
       else \
       {
          Error 500
       }
   }
   
   proc Get_URL {  } \
   {
       upvar this this
       variable GET_handlers
       
       if [string match *../* $this(url)] \
       {
          return [Error 404]
       }
       
       foreach handler [array names GET_handlers] \
       {
          Debug::Puts HTTPD 1  "Considering $handler for $this(url)"
          
          if [string match $handler $this(url)] \
          {
             return [$GET_handlers($handler)]
          }
       }
       Error 404
   }
 
   proc Get_help {} \
   {
       upvar this this
       if ![regexp {/help/(.*)} $this(url) match location]\
       {
          Debug::Puts HTTPD 1  "malformed HELP url $this(url)"
          Error 404
          return
       }
       
       set file [file join /sysdata/html/help/ $location]
       Get_file $file
   }
   proc Get_usr {} \
   {
       upvar this this
       if ![regexp {/usr/(.*)} $this(url) match location]\
       {
          Debug::Puts HTTPD 1  "malformed USR url $this(url)"
          Error 404
          return
       }
       set this(exec_allowed) 0
       set this(upload_allowed) 1
       set this(delete_allowed) 1
       set this(index_allowed) 2 ;# prefer our indexes
       Debug::Puts HTTPD 1  "exec disallowed now"

       set file [file join /usr $location]
       Debug::Puts HTTPD 1  "Get_usr file is $file"

       Get_file $file
   }
   proc Get_fonts {} \
   {
       upvar this this
       if ![regexp {/fonts/(.*)} $this(url) match location]\
       {
          Debug::Puts HTTPD 1  "malformed USR url $this(url)"
          Error 404
          return
       }
       set this(exec_allowed) 1
       set this(upload_allowed) 0
       set this(delete_allowed) 0
       set this(index_allowed) 1
       Debug::Puts HTTPD 1  "exec disallowed now"

       set file [file join /system/rtx/fonts/urw $location]
       Debug::Puts HTTPD 1  "Get_fonts file is $file"

       Get_file $file
   }
   proc Get_directory { file } \
   {
      upvar this this
      variable indexes

      # check the URL, and put a / at the end if needed
      if ![string match */ $this(url)] \
      {
         Debug::Puts HTTPD 5 "301 $this(url)/"
         Redirect "$this(url)/"
         return
      }
      # if force index set, don't look for index files
      if {$this(index_allowed) != 2} \
      {
        set trial [::HtmlFrame::find_index_file $file]
        if [file exists $trial] \
        {
          Debug::Puts HTTPD 5 "File is $file"
          return [Get_file $trial]
        }
      }
      if {$this(index_allowed) == 0} \
      {
         Error 403
         return
      }
      set this(exec_allowed) 1
      set this(dir) $file
      return [Get_file /sysdata/html/dir.shtml]
   }
   proc Get_directory_contents { } \
   {
      upvar this this
      variable hidden_files
      set file $this(dir)
      set result ""
      set dir_result ""
      
      foreach f [lsort [glob -nocomplain $file/*]] \
      {
         Debug::Puts HTTPD 5 "Get_directory_contents file is $f"
         set hidden 0
         foreach hide $hidden_files \
         {
            if [string match $hide $f] \
            {
               set hidden 1
               Debug::Puts HTTPD 5 "Hiding $f because of $hide"
               break
            }
         }
         if {$hidden == 1} { continue }

         if [file isdirectory $f] \
         {
            set url  [file join  $this(url) $f]
            set name [file tail $f]
            Debug::Puts HTTPD 5 "url $url name $name"
            append dir_result [subst $this(dir_line)]
         } \
         else \
         {
            append result [Get_directory_entry "$f"]
         }
      }
      append dir_result $result
      return $dir_result
   }
   
   proc Get_directory_entry_img { f } \
   {
      upvar this this
      upvar type type
      set type "Image:<br><a href=\"$f\"><img width=\"160\" height=\"120\" src=\"$f\"></a>"
   }
   proc Get_directory_entry_waveform { f } \
   {
      upvar this this
      upvar type type
      if [catch \
          {
             set fd [open $f r]
             set d [read $fd]
             close $fd
             unset fd
             array set narf $d
             set type "Arb waveform: $narf(NAME)<br>$narf(DESCRIPTION)"
             if [info exists narf(RATE)] \
             {
                set type "Arb waveform: $narf(NAME)<br>$narf(RATE) samples/second<br>$narf(DESCRIPTION)"
             } \
             else \
             {
                set type "Arb waveform: $narf(NAME)<br>$narf(DESCRIPTION)"
             }
          } err
         ] \
      {
         if [info exists fd] \
         {
            close $fd
         }
         return
      }
   }
   proc Get_directory_entry_sav { f } \
   {
      upvar this this
      upvar type type
      set type "System setup"
      array set setup_info { }
      SavedStates::read_info $f setup_info
      if {($setup_info(label,valid) == 1) && ($setup_info(label) != "") } \
      {
        append type ":<br><b>$setup_info(label)</b>"
      }
   }
   
   proc Get_directory_entry { f } \
   {
      upvar this this
      variable file_handlers
      # Determine the type of file
      set type FILE
      set extra ""
      set url  [file join  $this(url) $f]
      set name [file tail $f]
      set size [file size $f]
      set time [clock format [file atime $f]]
      set ext [string tolower [file extension $f]]
      if [info exists file_handlers($ext)] \
      {
         eval $file_handlers($ext) [list $f]
      }
      return [subst $this(file_line)]
   }

   proc Mime_type { file } \
   {
      variable mime_types
      set file [string tolower $file]

      foreach match [array names mime_types] \
      {
         if [string match $match $file] \
         {
            return $mime_types($match)
         }
      }
      return "application/octet-stream"
   }
   
   proc Get_file { file {http_code 200}} \
   {
       upvar this this
       variable http_errs
       variable mime_types
       
       if ![file exists $file] \
       {
          Error 404
          return
       }
       if [file isdirectory $file] \
       {
          return [Get_directory $file]
       }
       
       Debug::Puts HTTPD 5 "File is $file"
       # now, handle TCL files and txt files
       set ext [string tolower [file extension $file]]
       Debug::Puts HTTPD 5  "Ext is $ext"
       set mime [Mime_type $file]
       
       #fconfigure $this(fd) -blocking 0
       
       switch -- $ext \
       {
       
          ".tcl" \
          {
             if {$this(exec_allowed) == 0} \
             {
                Error 403
                return
             }
             Debug::Puts HTTPD 1  "Sourcing"
             set dir [pwd]             
             if [catch \
             {
                 cd [file dirname $file]
                 set result [source $file]
             } err] \
             {
                cd $dir
                Error 500
                return
             }
             cd $dir
             # expand LF -> CR/LF for an accurate byte count
             regsub -all "\n" $result "\r\n" result
             puts $this(fd) "HTTP/1.1 $http_code $http_errs($http_code)"
             puts $this(fd) "Content-Length: [string length $result]"
             puts $this(fd) "Content-type: $mime\n"
             fconfigure $this(fd) -translation binary
             if ![info exists this(HEAD)] \
             {
                puts -nonewline $this(fd) $result
             }
          }
          
          ".shtml" \
          {
             if {$this(exec_allowed) == 0} \
             {
                Error 403
                return
             }
             
             if [catch \
             {
                Debug::Puts HTTPD 1  "SHTML parse"
                set f [open $file r]
                set d [read $f]
                close $f
                set pd [pwd]
                catch \
                {
                  cd [file dirname $file]
                  set d [subst $d]
                }
                cd $pd
             } err ] \
             {
                Error 500
                return
             }

             # expand LF -> CR/LF for an accurate byte count
             regsub -all "\n" $d "\r\n" d
             puts $this(fd) "HTTP/1.1 $http_code $http_errs($http_code)"
             puts $this(fd) "Content-Length: [string length $d]"
             puts $this(fd) "Content-type: text/html\n"
             fconfigure $this(fd) -translation binary
             if ![info exists this(HEAD)] \
             {
                puts -nonewline $this(fd) $d
             }
             Debug::Puts HTTPD 1  "SHTML done"
          }
          
          default \
          {
             puts $this(fd) "HTTP/1.1 $http_code $http_errs($http_code)"
             puts $this(fd) "Content-Length: [file size $file]"
             puts $this(fd) "Content-type: $mime\n"
             if ![info exists this(HEAD)] \
             {
	             set f [open $file r]
	             fconfigure $f -translation binary
	             fconfigure $this(fd) -translation binary
	             fcopy $f $this(fd)
	             close $f
	         }
          }
       }
   }
   proc Get_main_items { {base_url "/"} } \
   {
       variable main_screen_items
       set main_items ""
       # compose the individual strings
       foreach priority [lsort [array names main_screen_items] ]\
       {
          foreach item [lsort $main_screen_items($priority)] \
          {
             append main_items "<div>[${item}::Get_HTML $base_url]</div>\n"
          }
       }
       return $main_items
   }
   
   proc Get_main_readings { {base_url "/"} } \
   {
       variable main_screen_readings
       set main_readings ""
       foreach priority [lsort [array names main_screen_readings] ]\
       {
          foreach item [lsort $main_screen_readings($priority)] \
          {
             append main_readings "[${item}::Get_HTML_reading /]"
          }
       }
       return $main_readings
   }
   
   proc Get_main {} \
   {
       upvar this this
       Get_file /sysdata/html/
   }
   proc Get_redirect {} \
   {
       upvar this this
       Get_file /sysdata/html/redirect.shtml
   }
   proc Get_snapshot {} \
   {
       upvar this this
       Get_file /sysdata/html/main_screen.shtml
   }
   proc Get_screen {} \
   {
       upvar this this
       variable http_errs
       # Generate the capture
       if [catch \
       {
           set image [image create photo]
           Capture image $image .
           Debug::Puts HTTPD 1 "writing screen capture"
           set http_code 200
           puts $this(fd) "HTTP/1.1 $http_code $http_errs($http_code)"
           puts $this(fd) "Pragma: No-cache"
           puts $this(fd) "Content-type: image/gif\n"
           fconfigure $this(fd) -translation binary -buffering full -buffersize 160000
           $image write $this(fd) -format gif
           Debug::Puts HTTPD 1 "Deleting image"
           image delete $image
           Debug::Puts HTTPD 1 "done with screen capture"
       } err ] \
       {
          Error 500
       }
   }

   proc Get_other {} \
   {
       upvar this this
       if ![regexp {/other/(.*)} $this(url) match location]\
       {
          Debug::Puts HTTPD 1  "malformed other url $this(url)"
          Error 404
          return
       }
       set file [file join /sysdata/html/other/ $location]
       Get_file $file
   }
   proc Get_icons {} \
   {
       upvar this this
       if ![regexp {/icons/(.*)} $this(url) match location]\
       {
          Debug::Puts HTTPD 1  "malformed icons url $this(url)"
          Error 404
          return
       }
       set this(exec_allowed) 0
       set file [file join /system/logo/ $location]
       Get_file $file
   }
   
   # now, a Get_HTML routine to create the meter table.
   proc Get_HTML { {base_url ""} } \
   {
      set x \
{<table align="left" border=1>
<tr><td nowrap colspan=2><center><h2>Meters</h2></center></td></tr>
[Get_main_readings]
</table>}
      return [subst $x]
   }

   proc Init {} \
   {
     if [file exists /system/etc/httpd.conf] \
     {
        catch { source /system/etc/httpd.conf}
     } \
     else \
     {
        Update_enable
     }
   }
}


