/*****************************************************************************
*  Copyright(c)2002-03 : IFR Systems Inc : Wichita,KS : Design Engineering  
******************************************************************************
* IFR 2975 P25 Radio Test Set                                               
*= IFR 2975 ==================================================================
* LabWindows/CVI 6.0 VXIpnp Instrument Driver                               
* Original Release: February 2003                                           
* By: Jeff Hand                                                             
*****************************************************************************/

#include <visa.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <ansi_c.h>
#include "ifr2975.h"

/****************************************************************************** 
* Instrument driver revision 
******************************************************************************/
#ifndef VI_INTF_TCPIP
   #define VI_INTF_TCPIP  3
#endif

#define ifr2975_REVISION_DRIVER         "Rev 1.0, 02/2003, CVI 6.0"
#define ifr2975_REVISION_UNIT           "UNKNOWN" 

/****************************************************************************** 
* File I/O buffer size
******************************************************************************/
#define MAX_BUFFER_SIZE 10000
#define BUFFER_SIZE 10000

/****************************************************************************** 
* Default Timeout value in ms (10 sec)
******************************************************************************/
#define ifr2975_DEFAULT_TIMEOUT		10

/****************************************************************************** 
* AND mask for ESR checking 
******************************************************************************/
#define IEEE_ERROR_BITS          0x3C

/*= Useful Macros =============================================================
******************************************************************************* 
* Error checking macro
******************************************************************************/
#define CHECKERR(fCal) if (ifr2975_status = (fCal), ifr2975_status < VI_SUCCESS) \
    return ifr2975_status; else

/****************************************************************************** 
* Error checking macro with clean up (ifr2975_init only)
******************************************************************************/
#define CHECKERR_CU(fCal) if (ifr2975_status = (fCal), ifr2975_status < VI_SUCCESS) \
    return ifr2975_initCleanUp (rmSession, instrumentHandle, ifr2975_status); else

/******************************************************************************
== INSTRUMENT-DEPENDENT COMMAND ARRAYS ========================================
*******************************************************************************
*                                                                           
* static ViString primFunc[] =                                              
* {                                                                         
*      "A1","B2","C3","D4","E5",VI_NULL                                     
* };                                                                        
* static ViString secFunc[]  =                                              
* {                                                                         
*      "AAA","BBB","CCC","DDD","EEE",VI_NULL                                
* };                                                                        
*                                                                           
*  When formatting strings to send to your instrument, these arrays can be  
*  used to simplify your code, for example:                                 
*                                                                           
*  CHECKERR (viPrintf (instrumentHandle, ":FUNC %s,%s", primFunc[func1],        
*                      secFunc[func2]));                                    
*                                                                           
*  The variables func1 and func2 are being used as indexing parameters that 
*  are passed into the function.                                            
*                                                                           
******************************************************************************/



/******************************************************************************
=== INSTRUMENT-DEPENDENT STATUS/RANGE STRUCTURE  ==============================
*******************************************************************************
* ifr2975_stringValPair is used in the ifr2975_error_message function       
*                                                                           
* ifr2975_statusDataRanges is used to track session dependent status        
* & ranges                                                                  
******************************************************************************/
typedef struct  ifr2975_stringValPair
{
  ViStatus stringVal;
  ViString stringName;
} ifr2975_tStringValPair;
 
struct ifr2975_statusDataRanges 
{
  ViInt16 model;
  ViReal64 freqHigh;
  ViReal64 spanHigh;
  ViChar instrDriverRevision[256];
};

typedef struct ifr2975_statusDataRanges *ifr2975_instrRange;

/******************************************************************************
=== UTILITY ROUTINE DECLARATIONS (Non-Exportable Functions) ===================
******************************************************************************/
ViBoolean ifr2975_invalidViBooleanRange (ViBoolean val);

ViBoolean ifr2975_invalidViInt16Range (ViInt16 val, ViInt16 min, ViInt16 max);

ViBoolean ifr2975_invalidViInt32Range (ViInt32 val, ViInt32 min, ViInt32 max);

ViBoolean ifr2975_invalidViUInt8Range (ViUInt8 val, ViUInt8 min, ViUInt8 max);

ViBoolean ifr2975_invalidViUInt16Range (ViUInt16 val, ViUInt16 min, ViUInt16 max);

ViBoolean ifr2975_invalidViUInt32Range (ViUInt32 val, ViUInt32 min, ViUInt32 max);

ViBoolean ifr2975_invalidViReal32Range (ViReal32 val, ViReal32 min, ViReal32 max);

ViBoolean ifr2975_invalidViReal64Range (ViReal64 val, ViReal64 min, ViReal64 max);

ViStatus ifr2975_initCleanUp (ViSession openRMSession, ViPSession openinstrumentHandle,
                                ViStatus currentStatus);

ViStatus ifr2975_readToFile (ViSession instrumentHandle, ViString filename,
                                ViUInt32 readBytes, ViPUInt32 retCount);

ViStatus ifr2975_writeFromFile (ViSession instrumentHandle, ViString filename,
                                ViUInt32 writeBytes, ViPUInt32 retCount);

ViStatus ifr2975_resetInternal (ViSession instrumentHandle, ViInt16 model);

ViStatus ifr2975_defaultInstrSetup (ViSession openinstrumentHandle);

void ifr2975_delay (ViReal64 numberOfSeconds);

// Get/Set Commands 
ViStatus ifr2975_setViString (ViSession instrumentHandle, ViString cmd, 
                              ViString value);

ViStatus ifr2975_getViString (ViSession instrumentHandle, ViString cmd, 
                              ViString value, ViInt32 string_length);

ViStatus ifr2975_sysStatus (ViSession instrumentHandle);                              

ViStatus ifr2975_setViReal64 (ViSession instrumentHandle, ViString cmd, 
                             ViReal64 value, ViReal64 min, ViReal64 max, ViString unit);

ViStatus ifr2975_getViReal64 (ViSession instrumentHandle, ViString cmd, 
                             ViReal64 *value);

ViStatus ifr2975_getViBoolean (ViSession instrumentHandle, ViString cmd,
                             ViBoolean *value);

ViStatus ifr2975_setViInt32 (ViSession instrumentHandle, ViString cmd, 
                             ViInt32 value, ViInt32 min, ViInt32 max);

ViStatus ifr2975_getViInt32 (ViSession instrumentHandle, ViString cmd, 
                             ViInt32 *value);

ViStatus ifr2975_setViInt16 (ViSession instrumentHandle, ViString cmd, 
                             ViInt16 value, ViInt16 min, ViInt16 max);

ViStatus ifr2975_getViInt16 (ViSession instrumentHandle, ViString cmd, 
                             ViInt16 *value);


/******************************************************************************
*       INSERT INSTRUMENT-DEPENDENT UTILITY ROUTINE DECLARATIONS HERE       
******************************************************************************/

ViStatus ifr2975_checkStatus (ViSession instrumentHandle);

/******************************************************************************
======== USER-CALLABLE FUNCTIONS (Exportable Functions) =======================
******************************************************************************/

/******************************************************************************
* Function: ifr2975_init                 
* Purpose:  This function opens the instrument, queries the instrument      
*           for its ID, and initializes the instrument to a known state.    
******************************************************************************/
ViStatus _VI_FUNC ifr2975_init (ViRsrc resourceName, ViBoolean IDQuery,
                               ViBoolean resetDevice, ViSession *instrumentHandle)
{
  // Standard function variables
  ViStatus ifr2975_status = VI_SUCCESS;
  ViSession rmSession = 0;
  ViByte rdbuf[BUFFER_SIZE];
  ViUInt16 interface = VI_INTF_GPIB;
    
  ifr2975_instrRange  instrPtr;   

  // Check input parameter ranges 
  if (ifr2975_invalidViBooleanRange (IDQuery))
  {
    return VI_ERROR_PARAMETER2;
  }
        
  if (ifr2975_invalidViBooleanRange (resetDevice))
  {
    return VI_ERROR_PARAMETER3;
  }

  // Open instrument session 
  CHECKERR (((viOpenDefaultRM (&rmSession)) < 0));

  if ((ifr2975_status = viOpen (rmSession, resourceName, VI_NULL, VI_NULL, instrumentHandle)) < 0) 
  {
    viClose (rmSession);
    
    return ifr2975_status;
  }

  // Determine if the structure has been initialized for the current VISA
  // Session and malloc if it has not.                                     
  CHECKERR_CU( viGetAttribute (*instrumentHandle, VI_ATTR_USER_DATA, &instrPtr));

  if (instrPtr == NULL)
  {
    instrPtr = malloc (sizeof (struct ifr2975_statusDataRanges));
  }

  CHECKERR_CU(viSetAttribute (*instrumentHandle, VI_ATTR_USER_DATA, (ViUInt32)instrPtr));

  /**************************************************************************
  * Configure VISA Formatted I/O
  **************************************************************************/
  // Set instrument Timeout
  CHECKERR (((viSetAttribute (*instrumentHandle, VI_ATTR_TMO_VALUE, 10000)) < 0));
  
  // Set buffer sizes for this instruments I/O
  CHECKERR (((viSetBuf (*instrumentHandle, VI_READ_BUF|VI_WRITE_BUF, 10000)) < 0));
        
  // Set write buffer to flush on instrument access
  CHECKERR (((viSetAttribute (*instrumentHandle, VI_ATTR_WR_BUF_OPER_MODE,VI_FLUSH_ON_ACCESS)) < 0));
        
  // Set read buffer to flush on instrument access
  CHECKERR (((viSetAttribute (*instrumentHandle, VI_ATTR_RD_BUF_OPER_MODE,VI_FLUSH_ON_ACCESS)) < 0));    
        
  // Serial interface support 
  if ((ifr2975_status = viGetAttribute (*instrumentHandle, VI_ATTR_INTF_TYPE, &interface)) < 0)
  {
    return ifr2975_initCleanUp (rmSession, instrumentHandle, ifr2975_status);
  }

  if (interface == VI_INTF_ASRL) 
  {
    // Serial Interface initialized to 8 databits, 1 stopbit, no parity,
    // baud rate = 9600, flow control = XonXoff and timeout = 20000 ms  
	CHECKERR_CU(viSetAttribute (*instrumentHandle, VI_ATTR_ASRL_DATA_BITS, 8));

	CHECKERR_CU(viSetAttribute (*instrumentHandle, VI_ATTR_ASRL_STOP_BITS,VI_ASRL_STOP_ONE));
		    
	CHECKERR_CU(viSetAttribute (*instrumentHandle, VI_ATTR_ASRL_PARITY, VI_ASRL_PAR_NONE));
		    
	CHECKERR_CU(viSetAttribute (*instrumentHandle, VI_ATTR_ASRL_BAUD, 115200));
		    
	CHECKERR_CU(viSetAttribute (*instrumentHandle, VI_ATTR_ASRL_FLOW_CNTRL,VI_ASRL_FLOW_XON_XOFF));
		    
	CHECKERR_CU(viSetAttribute (*instrumentHandle, VI_ATTR_TMO_VALUE, 20000));
		    
	CHECKERR_CU(viSetBuf (*instrumentHandle, VI_ASRL_IN_BUF|VI_ASRL_OUT_BUF, 512));
		    
	CHECKERR_CU(viSetBuf (*instrumentHandle, VI_READ_BUF|VI_WRITE_BUF, 4000));
		    
	CHECKERR_CU(viSetAttribute (*instrumentHandle, VI_ATTR_WR_BUF_OPER_MODE,VI_FLUSH_ON_ACCESS));
		    
	CHECKERR_CU(viSetAttribute (*instrumentHandle, VI_ATTR_RD_BUF_OPER_MODE,VI_FLUSH_ON_ACCESS));
		    
    CHECKERR_CU(viSetAttribute (*instrumentHandle, VI_ATTR_TERMCHAR_EN, VI_TRUE));
		    
	CHECKERR_CU(viSetAttribute (*instrumentHandle, VI_ATTR_TERMCHAR, '\n'));
		    
	CHECKERR_CU(viSetAttribute (*instrumentHandle, VI_ATTR_ASRL_END_IN,VI_ASRL_END_TERMCHAR));
		    
	CHECKERR_CU(viSetAttribute (*instrumentHandle, VI_ATTR_ASRL_END_OUT,VI_ASRL_END_NONE));
		    
	CHECKERR (viWrite(*instrumentHandle, (ViBuf) "*idn? \n" ,strlen ("*idn? \n"), 0));

	CHECKERR (viRead (*instrumentHandle,(ViBuf) rdbuf, 10000, 0));
  }

#ifndef VI_INTF_TCPIP
  if (interface == 6) 
  {
#else
  if (interface == VI_INTF_TCPIP) 
  {
#endif
	CHECKERR_CU(viSetAttribute (*instrumentHandle, VI_ATTR_TERMCHAR, '\n'));
		    
	CHECKERR_CU(viSetAttribute (*instrumentHandle, VI_ATTR_TERMCHAR_EN, VI_TRUE));

	CHECKERR_CU(viSetAttribute (*instrumentHandle, VI_ATTR_TMO_VALUE, 20000));

    CHECKERR_CU(viSetAttribute (*instrumentHandle, VI_ATTR_WR_BUF_OPER_MODE,VI_FLUSH_ON_ACCESS));
		    
	CHECKERR_CU(viSetAttribute (*instrumentHandle, VI_ATTR_RD_BUF_OPER_MODE,VI_FLUSH_ON_ACCESS));
		    
  	// Added to clear output buffer so commands can be issued 
	CHECKERR (viRead (*instrumentHandle,(ViBuf) rdbuf, 10000, 0));
		
	// Send *IDN?* to instrument
	CHECKERR (viWrite(*instrumentHandle, (ViBuf) "echo off \n",strlen ("echo off \n"),0));

	ifr2975_delay(2);
		
	CHECKERR (viRead (*instrumentHandle,(ViBuf) rdbuf, 10000, 0));		
		
	ifr2975_delay(.2);
	
	CHECKERR (ifr2975_getViString (*instrumentHandle, "*IDN?", (ViString) rdbuf, 100));
		
	ifr2975_delay(.2);
  }

  // Send *IDN?* to instrument
  CHECKERR (ifr2975_getViString (*instrumentHandle, "*IDN? ", (ViString) rdbuf, 100));
	
  // Identification Query
  if ( IDQuery )
  {
	// Checks to make sure that the 2975 is responding correctly 
	if(strncmp ("IFR Systems, Inc. 2975 TCL\n\n", (const char *)rdbuf,10)!=0) 
	{
	  return 0xBFFF005F;
    }
  }
    
  // Reset instrument
  if ( resetDevice ) 
  {
   	// This resets 2975 to the default state 
	CHECKERR (ifr2975_Reset (*instrumentHandle));
  }
   
  return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Set_Avg_Count_AFC    
* Purpose:  This function sets the number of sample to average over for the 
*           audio frequency counter.                                       
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_Avg_Count_AFC (ViSession instrumentHandle,
                                             ViInt16 samplestoAverageOver) 
{
    ViStatus    ifr2975_status = VI_SUCCESS;
    
    //This writes command to instrument 
    CHECKERR (ifr2975_setViInt16 (instrumentHandle, "AFCounter AVG ", samplestoAverageOver, 1, 100));

    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Set_Trig_Mode_AFC    
* Purpose:  This function sets the trigger mode on the audio frequency      
*           counter.                                                       
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_Trig_Mode_AFC (ViSession instrumentHandle,
                                            ViInt16 triggerMode)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    ViChar rdbuf[BUFFER_SIZE];
    
    // Verify Triggermode is in range     
    // 1 = Continuous                     
    // 2 = OneShot                        
    if (ifr2975_invalidViInt16Range (triggerMode, 1, 2))
    {
      return VI_ERROR_PARAMETER2;
    }
    
    // Formats Data According to Input Data 
    switch (triggerMode)
    {
      case 1:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "AFCounter TRIG Continuous ");
        break;
      case 2:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "AFCounter TRIG OneShot ");
        break;
      default:
        return VI_ERROR_PARAMETER2;
    }
    
    // This writes command to instrument 
    CHECKERR (ifr2975_setViString (instrumentHandle, wrtbuf, 0));
    
    return ifr2975_status;
}

/******************************************************************************
* Function: ifr2975_Set_Pk_Hold_State_AFC
* Purpose:  This function enables/disables peak hold mode on the audio      
*           frequency counter.           
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_Pk_Hold_State_AFC (ViSession instrumentHandle,
                                          ViInt16 peakHoldMode)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    
    // Verify peakHoldMode is in range
    // 0 = Disable
    // 1 = Enable                     
    if (ifr2975_invalidViInt16Range (peakHoldMode, 0, 1))
    {
      return VI_ERROR_PARAMETER2;
    }
    
    // Formats Data According to Input Data 
    switch (peakHoldMode)
    {
      case 0:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "AFCounter peakhold off ");
        break;
      case 1:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "AFCounter peakhold on ");
        break;
      default:
        return VI_ERROR_PARAMETER2;
    }

    // This writes command to instrument 
    CHECKERR (ifr2975_setViString (instrumentHandle, wrtbuf, 0));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Reset_Pk_Hold_AFC    
* Purpose:  This function resets the peak hold mode  value on the audio     
*           frequency counter.           
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Reset_Pk_Hold_AFC (ViSession instrumentHandle)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    
    // Formats string to be transmitted 
    sprintf ((char*) wrtbuf, "AFCounter PHRESET ");

    // This writes command to instrument 
    CHECKERR (ifr2975_setViString (instrumentHandle, wrtbuf, "2"));
    
    return ifr2975_status;
}

/******************************************************************************
* Function: ifr2975_Set_Alerts_AFC       
* Purpose:  This function sets the alert mode on the audio frequency        
*           counter.                     
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_Alerts_AFC (ViSession instrumentHandle,
                                         ViInt16 alertMode)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    
    // Verify alertMode is in range       
    // 0 = Off                            
    // 1 = Low On                         
    // 2 = Hi On                          
    // 3 = Low and Hi On                  
    if (ifr2975_invalidViInt16Range (alertMode, 0, 3))
    {
      return VI_ERROR_PARAMETER2;
    }
    
    // Formats Data According to Input Data 
    switch (alertMode)
    {
      case 0:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "AFCounter alerts off ");
        break;
      case 1:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "AFCounter alerts enablelo ");
        break;
      case 2:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "AFCounter alerts enablehi ");
        break;
      case 3:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "AFCounter alerts enableboth ");
        break;
      default:
        return VI_ERROR_PARAMETER2;
    }

    // This writes command to instrument 
    CHECKERR (ifr2975_setViString (instrumentHandle, wrtbuf, 0));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Set_UL_AFC           
* Purpose:  This function sets upper alert limit on the audio frequency     
*           counter.                     
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_UL_AFC (ViSession instrumentHandle,
                                     ViReal64 upperAlertLimit)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_setViReal64 (instrumentHandle, "AFCounter ulimit", upperAlertLimit, 0, 20e3, "Hz"));
    
    return ifr2975_status;
}

/******************************************************************************
* Function: ifr2975_Set_LL_AFC           
* Purpose:  This function sets lower alert limit on the audio frequency     
*           counter.                     
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_LL_AFC (ViSession instrumentHandle,
                                     ViReal64 lowerAlertLimit)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_setViReal64 (instrumentHandle, "AFCounter llimit", lowerAlertLimit, 0, 20e3, "Hz"));
    
    return ifr2975_status;
}

/******************************************************************************
* Function: ifr2975_Set_Enable_State_AFC 
* Purpose:  This function enables/disables data mode on the audio frequency 
*           counter.                     
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_Enable_State_AFC (ViSession instrumentHandle,
                                         ViInt16 enableMode)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    
    // Verify enableMode is in range
    // 1 = Disabled
    // 3 = Enabled
    if (ifr2975_invalidViInt16Range (enableMode, 1, 1) && ifr2975_invalidViInt16Range (enableMode, 3, 3))
    {
      return VI_ERROR_PARAMETER2;
    }
    
    // Formats Data According to Input Data 
    switch (enableMode)
    {
      case 1:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "AFCounter enable disable ");
        break;
      case 3:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "AFCounter enable enable ");
        break;
      default:
        return VI_ERROR_PARAMETER2;
    }

    // This writes command to instrument 
    CHECKERR (ifr2975_setViString (instrumentHandle, wrtbuf, 0));
    
    return ifr2975_status;
}

/******************************************************************************
* Function: ifr2975_Set_Run_State_AFC    
* Purpose:  This function enables/disables run mode on  the audio frequency 
*           counter.                     
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_Run_State_AFC (ViSession instrumentHandle,
                                           ViInt16 runMode)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    
    // Verify runMode is in range
    // 0 = off
    // 1 = on
    if (ifr2975_invalidViInt16Range (runMode, 0, 1))
    {
      return VI_ERROR_PARAMETER2;
    }
    
    // Formats Data According to Input Data 
    switch (runMode)
    {
      case 0:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "AFCounter run off ");
        break;
      case 1:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "AFCounter run on ");
        break;
      default:
        return VI_ERROR_PARAMETER2;
    }
    
    // This writes command to instrument 
    CHECKERR (ifr2975_setViString (instrumentHandle, wrtbuf, 0));
    
    return ifr2975_status;
}

/******************************************************************************
* Function: ifr2975_Set_Gate_State_AFC    
* Purpose:  This function set the gate time  the audio frequency            
*           counter.                     
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_Gate_State_AFC (ViSession instrumentHandle,
                                            ViInt16 gateTime)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];

    // Verify gateTime is in range  
    // 1 = 1.0 secs                   
    // 0 = 0.1 secs                   
    if (ifr2975_invalidViInt16Range (gateTime, 0, 1))
    {
      return VI_ERROR_PARAMETER2;
    }
    
    // Formats Data According to Input Data 
    switch (gateTime)
    {
      case 0:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "AFCounter gate tenth ");
        break;
      case 1:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "AFCounter gate second ");
        break;
      default:
        return VI_ERROR_PARAMETER2;
    }

    // This writes command to instrument 
    CHECKERR (ifr2975_setViString (instrumentHandle, wrtbuf, 0));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Set_Avg_Count_FMDM   
* Purpose:  This function set samples to average count on the FM deviation  
*           meter.                       
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_Avg_Count_FMDM (ViSession instrumentHandle,
                                             ViInt16 samplestoAverageOver)
{
    ViStatus     ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_setViInt16 (instrumentHandle, "FMDev AVG ", samplestoAverageOver, 200, 10000));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Set_Avg_Math_State_FMDM                                 
* Purpose:  This function enables/disables averaging mode on the FM         
*           deviation meter.             
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_Avg_Math_State_FMDM (ViSession instrumentHandle,
                                            ViInt16 averagingMode)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    
    // Verify averagingMode is in range   
    // 0 = Disable Averaging              
    // 1 = Enable Averaging               
    if (ifr2975_invalidViInt16Range (averagingMode, 0, 1))
    {
      return VI_ERROR_PARAMETER2;
    }
    
    // Formats Data According to Input Data 
    switch (averagingMode)
    {
      case 0:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "FMDev MATH peak ");
        break;
      case 1:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "FMDev MATH average ");
        break;
      default:
        return VI_ERROR_PARAMETER2;
    }
        
    // This writes command to instrument 
    CHECKERR (ifr2975_setViString (instrumentHandle, wrtbuf, 0));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Set_Trig_Mode_FMDM   
* Purpose:  This function sets the trigger mode on the FM                   
*           deviation meter.             
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_Trig_Mode_FMDM (ViSession instrumentHandle,
                                             ViInt16 triggerMode)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    
    // Verify triggerMode is in range     
    // 0 = Continuous                     
    // 1 = OneShot                        
    if (ifr2975_invalidViInt16Range (triggerMode, 1, 2))
    {
      return VI_ERROR_PARAMETER2;
    }
    
    // Formats Data According to Input Data 
    switch (triggerMode)
    {
      case 1:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "FMDev trigger Continuous ");
        break;
      case 2:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "FMDev trigger OneShot ");
        break;
      default:
        return VI_ERROR_PARAMETER2;
    }
    
    // This writes command to instrument 
    CHECKERR (ifr2975_setViString (instrumentHandle, wrtbuf, 0));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Set_Pk_Hold_State_FMDM                                  
* Purpose:  This function enables/disable peak hold mode on the FM          
*           deviation meter.             
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_Pk_Hold_State_FMDM (ViSession instrumentHandle,
                                           ViInt16 peakHoldMode)
{    
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    
    // Verify peakHoldMode is in range    
    // 0 = Disable peak hold              
    // 1 = Enable peak hold               
    if (ifr2975_invalidViInt16Range (peakHoldMode, 0, 1))
    {
      return VI_ERROR_PARAMETER2;
    }
    
    // Formats Data According to Input Data 
    switch (peakHoldMode)
    {
      case 0:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "FMDev PEAK off ");
        break;
      case 1:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "FMDev PEAK on ");
        break;
      default:
        return VI_ERROR_PARAMETER2;
    }

    // This writes command to instrument 
    CHECKERR (ifr2975_setViString (instrumentHandle, wrtbuf, 0));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Reset_Pk_Hold_FMDM   
* Purpose:  This function resets the peak hold on the FM                    
*           deviation meter.             
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Reset_Pk_Hold_FMDM (ViSession instrumentHandle)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    ViChar rdbuf[BUFFER_SIZE];
    
    // Formats string to be transmitted 
    sprintf ((char*) wrtbuf, "FMDev PHRST ");

    // This writes command to instrument 
    CHECKERR (ifr2975_setViString (instrumentHandle, wrtbuf, "2"));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Set_Alerts_FMDM      
* Purpose:  This function set the alert mode on the FM                      
*           deviation meter.             
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_Alerts_FMDM (ViSession instrumentHandle,
                                          ViInt16 alertMode)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    
    // Verify alertMode is in range       
    // 0 = Off                            
    // 1 = Low On                         
    // 2 = Hi On                          
    // 3 = Low and Hi On                  
    if (ifr2975_invalidViInt16Range (alertMode, 0, 3))
    {
      return VI_ERROR_PARAMETER2;
    }
    
    // Formats Data According to Input Data 
    switch (alertMode)
    {
      case 0:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "FMDev alerts off ");
        break;
      case 1:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "FMDev alerts enablelo ");
        break;
      case 2:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "FMDev alerts enablehi ");
        break;
      case 3:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "FMDev alerts enableboth ");
        break;
      default:
        return VI_ERROR_PARAMETER2;
    }

    // This writes command to instrument 
    CHECKERR (ifr2975_setViString (instrumentHandle, wrtbuf, 0));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Set_UL_FMDM          
* Purpose:  This function sets the upper alert limit on the FM              
*           deviation meter.             
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_UL_FMDM (ViSession instrumentHandle,
                                      ViReal64 upperAlertLimit)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_setViReal64 (instrumentHandle, "FMDev ulimit", upperAlertLimit, 0, 100, 0));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Set_LL_FMDM          
* Purpose:  This function sets the lower alert limit on the FM              
*           deviation meter.             
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_LL_FMDM (ViSession instrumentHandle,
                                      ViReal64 lowerAlertLimit)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // Verify lowerAlertLimit is in range
    if (ifr2975_invalidViReal64Range (lowerAlertLimit, 0, 100))
    {
      return VI_ERROR_PARAMETER2;
    }
        
    // This writes command to instrument 
    CHECKERR (ifr2975_setViReal64 (instrumentHandle, "FMDev llimit", lowerAlertLimit, 0, 100, 0));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Set_Enable_State_FMDM
* Purpose:  This function enables/disables data mode on the FM              
*           deviation meter.             
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_Enable_State_FMDM(ViSession instrumentHandle,
                                               ViInt16 enableMode)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    
    // Verify enableMode is in range      
    // 1 = Disabled                          
    // 3 = Enabled                          
    if (ifr2975_invalidViInt16Range (enableMode, 1, 1)  && ifr2975_invalidViInt16Range (enableMode, 3, 3))
    {
      return VI_ERROR_PARAMETER2;
    }
    
    // Formats Data According to Input Data 
    switch (enableMode)
    {
      case 1:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "FMDev enable off ");
        break;
      case 3:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "FMDev enable on ");
        break;
      default:
        return VI_ERROR_PARAMETER2;
    }

    // This writes command to instrument 
    CHECKERR (ifr2975_setViString (instrumentHandle, wrtbuf, 0));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Set_Run_State_FMDM   
* Purpose:  This function enables/disables run mode on the FM               
*           deviation meter.             
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_Run_State_FMDM (ViSession instrumentHandle,
                                            ViInt16 runMode)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    
    // Verify runMode is in range         
    // 0 = Disabled                          
    // 1 = Enabled                          
    if (ifr2975_invalidViInt16Range (runMode, 0, 1))
    {
      return VI_ERROR_PARAMETER2;
    }
    
    // Formats Data According to Input Data 
    switch (runMode)
    {
      case 0:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "FMDev run off ");
        break;
      case 1:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "FMDev run on ");
        break;
      default:
        return VI_ERROR_PARAMETER2;
    }

    // This writes command to instrument 
    CHECKERR (ifr2975_setViString (instrumentHandle, wrtbuf, 0));
    
    return ifr2975_status;
}



/******************************************************************************
* Function: ifr2975_Set_Filter_State_SINAD                                  
* Purpose:  This function enables/disables SINAD meter filtering.           
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_Filter_State_SINAD (ViSession instrumentHandle,
                                         ViInt16 filterSetting)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    
    // Verify filterSetting is in range   
    // 0 = No filtering                   
    // 1 = Enable filtering                  
    if (ifr2975_invalidViInt16Range (filterSetting, 0, 1))
    {
      return VI_ERROR_PARAMETER2;
    }
    
    // Formats Data According to Input Data 
    switch (filterSetting)
    {
      case 0:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Sinad math off ");
        break;
      case 1:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Sinad math on ");
        break;
      default:
        return VI_ERROR_PARAMETER2;
    }

    // This writes command to instrument 
    CHECKERR (ifr2975_setViString (instrumentHandle, wrtbuf, 0));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Set_Avg_SINAD        
* Purpose:  This function sets the number of samples to average over for the
*           SINAD meter.                 
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_Avg_SINAD (ViSession instrumentHandle,
                                        ViInt16 samplestoAverageOver)
{
    ViStatus     ifr2975_status = VI_SUCCESS; 
    ViChar         wrtbuf[BUFFER_SIZE];    
    
    // This writes command to instrument 
    CHECKERR (ifr2975_setViInt16 (instrumentHandle, "Sinad FILTer", samplestoAverageOver, 1, 100));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Set_Avg_DSP_SINAD    
* Purpose:  This function sets the number of DSP samples to average over for
*           the SINAD meter.             
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_Avg_DSP_SINAD (ViSession instrumentHandle,
                                            ViInt16 samplestoAverageOver)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_setViInt16 (instrumentHandle, "Sinad AVG", samplestoAverageOver, 1, 2000));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Set_Trig_Mode_SINAD  
* Purpose:  This function sets the trigger mode for                         
*           the SINAD meter.             
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_Trig_Mode_SINAD (ViSession instrumentHandle,
                                              ViInt16 triggerMode)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    
    // Verify triggerMode is in range   
    // 0 = Continuous                   
    // 1 = OneShot                     
    if (ifr2975_invalidViInt16Range (triggerMode, 1, 2))
    {
      return VI_ERROR_PARAMETER2;
    }
    
    // Formats Data According to Input Data 
    switch (triggerMode)
    {
      case 1:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Sinad TRIG Continuous ");
        break;
      case 2:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Sinad TRIG OneShot ");
        break;
      default:
        return VI_ERROR_PARAMETER2;
    }
        
    // This writes command to instrument 
    CHECKERR (ifr2975_setViString (instrumentHandle, wrtbuf, 0));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Set_Pk_Hold_State_SINAD                                 
* Purpose:  This function enables/disable peak hold mode on the SINAD       
*           meter.                       
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_Pk_Hold_State_SINAD (ViSession instrumentHandle,
                                            ViInt16 peakHoldMode)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    
    // Verify peakHoldMode is in range   
    // 0 = Disable peak hold             
    // 1 = Enable peak hold              
    if (ifr2975_invalidViInt16Range (peakHoldMode, 0, 1))
    {
      return VI_ERROR_PARAMETER2;
    }
    
    // Formats Data According to Input Data 
    switch (peakHoldMode)
    {
      case 0:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Sinad PEAK off ");
        break;
      case 1:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Sinad PEAK on ");
        break;
      default:
        return VI_ERROR_PARAMETER2;
    }

    // This writes command to instrument 
    CHECKERR (ifr2975_setViString (instrumentHandle, wrtbuf, 0));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Reset_Pk_Hold_SINAD  
* Purpose:  This function resets peak hold mode on the SINAD                
*           meter.                       
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Reset_Pk_Hold_SINAD (ViSession instrumentHandle)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_setViString (instrumentHandle, "Sinad PHRST", "2"));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Set_Alerts_SINAD     
* Purpose:  This function sets the alert enable status on the SINAD         
*           meter.                       
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_Alerts_SINAD (ViSession instrumentHandle,
                                           ViInt16 alertMode)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    
    // Verify alertMode is in range
    // 0 = Off                      
    // 1 = Low On                  
    // 2 = Hi On                      
    // 3 = Low and Hi On           
    if (ifr2975_invalidViInt16Range (alertMode, 0, 3))
    {
      return VI_ERROR_PARAMETER2;
    }
    
    // Formats Data According to Input Data 
    switch (alertMode)
    {
      case 0:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Sinad alerts off ");
        break;
      case 1:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Sinad alerts enablelo ");
        break;
      case 2:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Sinad alerts enablehi");
        break;
      case 3:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Sinad alerts enableboth ");
        break;
      default:
        return VI_ERROR_PARAMETER2;
    }
    
    // This writes command to instrument 
    CHECKERR (ifr2975_setViString (instrumentHandle, wrtbuf, 0));
    
    return ifr2975_status;
}

/******************************************************************************
* Function: ifr2975_Set_UL_SINAD         
* Purpose:  This function sets the upper alert limit on the SINAD           
*           meter.                       
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_UL_SINAD (ViSession instrumentHandle,
                                       ViReal64 upperAlertLimit)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_setViReal64 (instrumentHandle, "Sinad ulimit", upperAlertLimit, 0, 40, 0));
    
    return ifr2975_status;
}

/******************************************************************************
* Function: ifr2975_Set_LL_SINAD         
* Purpose:  This function sets the lower alert limit on the SINAD           
*           meter.                       
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_LL_SINAD (ViSession instrumentHandle,
                                       ViReal64 lowerAlertLimit)
{
    ViStatus ifr2975_status = VI_SUCCESS; 

    // This writes command to instrument 
    CHECKERR (ifr2975_setViReal64 (instrumentHandle, "Sinad llimit", lowerAlertLimit, 0, 40, 0));
    
    return ifr2975_status;
}

/******************************************************************************
* Function: ifr2975_Set_Enable_State_SINAD
* Purpose:  This function enables/disables data mode on the SINAD           
*           meter.                       
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_Enable_State_SINAD (ViSession instrumentHandle,
                                                ViInt16 enableMode)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    
    // Verify enableMode is in range
    // 1 = Disabled
    // 3 = Enabled
    if (ifr2975_invalidViInt16Range (enableMode, 1, 1) && ifr2975_invalidViInt16Range (enableMode, 3, 3))
    {
      return VI_ERROR_PARAMETER2;
    }
    
    // Formats Data According to Input Data 
    switch (enableMode)
    {
      case 1:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Sinad enable disable ");
        break;
      case 3:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Sinad enable enable ");
        break;
      default:
        return VI_ERROR_PARAMETER2;
    }

    // This writes command to instrument 
    CHECKERR (ifr2975_setViString (instrumentHandle, wrtbuf, 0));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Set_Run_State_SINAD  
* Purpose:  This function enables/disables run mode on the SINAD            
*           meter.                       
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_Run_State_SINAD (ViSession instrumentHandle,
                                             ViInt16 runMode)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    
    // Verify runMode is in range
    // 0 = Disabled                   
    // 1 = Enabled                   
    if (ifr2975_invalidViInt16Range (runMode, 0, 1))
    {
      return VI_ERROR_PARAMETER2;
    }
    
    // Formats Data According to Input Data 
    switch (runMode)
    {
      case 0:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Sinad run off ");
        break;
      case 1:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Sinad run on ");
        break;
      default:
        return VI_ERROR_PARAMETER2;
    }

    // This writes command to instrument 
    CHECKERR (ifr2975_setViString (instrumentHandle, wrtbuf, 0));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Set_Filter_State_DIST
* Purpose:  This function enables/disables distortion meter filtering.      
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_Filter_State_DIST (ViSession instrumentHandle,
                                        ViInt16 filterSetting)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    
    // Verify filterSetting is in range 
    // 0 = No filtering                  
    // 1 = Enable filtering                
    if (ifr2975_invalidViInt16Range (filterSetting, 0, 1))
    {
      return VI_ERROR_PARAMETER2;
    }
    
    // Formats Data According to Input Data 
    switch (filterSetting)
    {
      case 0:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Dist math off ");
        break;
      case 1:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Dist math on ");
        break;
      default:
        return VI_ERROR_PARAMETER2;
    }

    // This writes command to instrument 
    CHECKERR (ifr2975_setViString (instrumentHandle, wrtbuf, 0));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Set_Avg_DIST         
* Purpose:  This function sets the number of samples to average over for the
*           SINAD meter.                 
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_Avg_DIST (ViSession instrumentHandle,
                                       ViInt16 samplestoAverageOver)
{
    ViStatus ifr2975_status = VI_SUCCESS; 

    // This writes command to instrument 
    CHECKERR (ifr2975_setViInt16 (instrumentHandle, "Dist FILTer", samplestoAverageOver, 1, 100));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Set_Avg_DSP_DIST     
* Purpose:  This function sets the number of DSP samples to average over for
*           the distortion meter.        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_Avg_DSP_DIST (ViSession instrumentHandle,
                                           ViInt16 samplestoAverageOver)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_setViInt16 (instrumentHandle, "Dist AVG", samplestoAverageOver, 1, 2000));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Set_Trig_Mode_DIST   
* Purpose:  This function sets the trigger mode for                         
*           the distortion meter.        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_Trig_Mode_DIST (ViSession instrumentHandle,
                                             ViInt16 triggerMode)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    
    // Verify triggerMode is in range   
    // 0 = Continuous                   
    // 1 = OneShot                     
    if (ifr2975_invalidViInt16Range (triggerMode, 1, 2))
    {
      return VI_ERROR_PARAMETER2;
    }
    
    // Formats Data According to Input Data 
    switch (triggerMode)
    {
      case 1:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Dist trigger Continuous ");
        break;
      case 2:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Dist trigger OneShot ");
        break;
      default:
        return VI_ERROR_PARAMETER2;
    }

    // This writes command to instrument 
    CHECKERR (ifr2975_setViString (instrumentHandle, wrtbuf, 0));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Set_Pk_Hold_State_DIST                                  
* Purpose:  This function enables/disable peak hold mode on the distortion  
*           meter.                       
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_Pk_Hold_State_DIST (ViSession instrumentHandle,
                                           ViInt16 peakHoldMode)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    
    // Verify peakHoldMode is in range   
    // 0 = Disable peak hold             
    // 1 = Enable peak hold              
    if (ifr2975_invalidViInt16Range (peakHoldMode, 0, 1))
    {
      return VI_ERROR_PARAMETER2;
    }
    
    // Formats Data According to Input Data 
    switch (peakHoldMode)
    {
      case 0:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Dist peak off ");
        break;
      case 1:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Dist peak on ");
        break;
      default:
        return VI_ERROR_PARAMETER2;
    }

    // This writes command to instrument 
    CHECKERR (ifr2975_setViString (instrumentHandle, wrtbuf, 0));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Reset_Pk_Hold_DIST   
* Purpose:  This function resets peak hold mode on the distortion           
*           meter.                       
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Reset_Pk_Hold_DIST (ViSession instrumentHandle)
{
    ViStatus ifr2975_status = VI_SUCCESS; 

    // This writes command to instrument 
    CHECKERR (ifr2975_setViString (instrumentHandle, "Dist PHRST ", "2"));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Set_Alerts_DIST      
* Purpose:  This function sets the alert enable status on the distortion    
*           meter.                       
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_Alerts_DIST (ViSession instrumentHandle,
                                          ViInt16 alertMode)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    
    // Verify alertMode is in range
    // 0 = Off                      
    // 1 = Low On                  
    // 2 = Hi On                      
    // 3 = Low and Hi On           
    if (ifr2975_invalidViInt16Range (alertMode, 0, 3))
    {
      return VI_ERROR_PARAMETER2;
    }
    
    // Formats Data According to Input Data 
    switch (alertMode)
    {
      case 0:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Dist alerts off ");
        break;
      case 1:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Dist alerts enablelo ");
        break;
      case 2:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Dist alerts enablehi ");
        break;
      case 3:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Dist alerts enableboth ");
        break;
      default:
        return VI_ERROR_PARAMETER2;
    }
    
    // This writes command to instrument 
    CHECKERR (ifr2975_setViString (instrumentHandle, wrtbuf, 0));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Set_UL_DIST          
* Purpose:  This function sets the upper alert limit on the distortion      
*           meter.                       
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_UL_DIST (ViSession instrumentHandle,
                                      ViReal64 upperAlertLimit)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_setViReal64 (instrumentHandle, "Dist ulimit", upperAlertLimit, 0, 100, 0));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Set_LL_DIST          
* Purpose:  This function sets the lower alert limit on the distortion      
*           meter.                       
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_LL_DIST (ViSession instrumentHandle,
                                      ViReal64 lowerAlertLimit)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_setViReal64 (instrumentHandle, "Dist llimit", lowerAlertLimit, 0, 100, 0));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Set_Enable_State_DIST
* Purpose:  This function enables/disables data mode on the distortion      
*           meter.                       
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_Enable_State_DIST (ViSession instrumentHandle,
                                               ViInt16 enableMode)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    
    // Verify enableMode is in range
    // 1 = Disabled
    // 3 = Enabled
    if (ifr2975_invalidViInt16Range (enableMode, 1, 1)  && ifr2975_invalidViInt16Range (enableMode, 3, 3))
    {
      return VI_ERROR_PARAMETER2;
    }
    
    // Formats Data According to Input Data 
    switch (enableMode)
    {
      case 1:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Dist enable off ");
        break;
      case 3:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Dist enable on ");
        break;
      default:
        return VI_ERROR_PARAMETER2;
    }
    
    // This writes command to instrument 
    CHECKERR (ifr2975_setViString (instrumentHandle, wrtbuf, 0));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Set_Run_State_DIST   
* Purpose:  This function enables/disables run mode on the distortion       
*           meter.                       
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_Run_State_DIST (ViSession instrumentHandle,
                                            ViInt16 runMode)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    
    // Verify runMode is in range
    // 0 = Disabled                   
    // 1 = Enabled                   
    if (ifr2975_invalidViInt16Range (runMode, 0, 1))
    {
      return VI_ERROR_PARAMETER2;
    }
    
    // Formats Data According to Input Data 
    switch (runMode)
    {
      case 0:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Dist run off ");
        break;
      case 1:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Dist run on ");
        break;
      default:
        return VI_ERROR_PARAMETER2;
    }

    // This writes command to instrument 
    CHECKERR (ifr2975_setViString (instrumentHandle, wrtbuf, 0));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Set_Freq_FGEN        
* Purpose:  This function sets the frequency of the selected function       
*           generator.                   
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_Freq_FGEN (ViSession instrumentHandle,
                                        ViInt16 output, ViReal64 frequency)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    
    // Verify output is in range
    // 1 = FGEN1
    // 2 = FGEN2
    // 3 = M1
    // 4 = M2
    if (ifr2975_invalidViInt16Range (output, 1, 4))
    {
      return VI_ERROR_PARAMETER2;
    }
        
    // Verify frequency is in range
    if (ifr2975_invalidViReal64Range (frequency, 0, 20e3))
    {
      return VI_ERROR_PARAMETER2;
    }
        
    // Formats string to be transmitted 
    sprintf ((char*) wrtbuf, "FGen FREQuency%i %f ", output, frequency);

    // This writes command to instrument 
    CHECKERR (ifr2975_setViString (instrumentHandle, wrtbuf, 0));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Set_Lev_FGEN         
* Purpose:  This function sets the level of the selected function           
*           generator.                   
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_Lev_FGEN (ViSession instrumentHandle,
                                       ViInt16 output, ViReal64 level)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    
    // Verify output is in range
    // 1 = FGEN1
    // 2 = FGEN2
    if (ifr2975_invalidViInt16Range (output, 1, 2))
    {
      return VI_ERROR_PARAMETER2;
    }

    // Verify frequency is in range
    if (ifr2975_invalidViReal64Range (level, 0, 20))
    {
      return VI_ERROR_PARAMETER2;
    }
        
    // Formats string to be transmitted 
    sprintf ((char*) wrtbuf, "FGen LEVel%i %f ", output, level);

    // This writes command to instrument 
    CHECKERR (ifr2975_setViString (instrumentHandle, wrtbuf, 0));
    
    return ifr2975_status;
}

/******************************************************************************
* Function: ifr2975_Set_Dev_FGEN         
* Purpose:  This function sets the deviation of the selected function       
*           generator.                   
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_Dev_FGEN (ViSession instrumentHandle,
                                       ViInt16 output, ViReal64 deviation)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    
    // Verify output is in range
    // 3 = M1
    // 4 = M2
    if (ifr2975_invalidViInt16Range (output, 3, 4))
    {
      return VI_ERROR_PARAMETER2;
    }
        
    // Verify frequency is in range
    // 0-40 kHz
    if (ifr2975_invalidViReal64Range (deviation, 0, 40))
    {
      return VI_ERROR_PARAMETER2;
    }
        
    // Formats string to be transmitted 
    sprintf ((char*) wrtbuf, "FGen DEViation%i %f ", output, deviation);

    // This writes command to instrument 
    CHECKERR (ifr2975_setViString (instrumentHandle, wrtbuf, 0));
    
    return ifr2975_status;
}

/******************************************************************************
* Function: ifr2975_Set_Per_FGEN         
* Purpose:  This function sets the deviation of the selected function       
*           generator.                   
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_Per_FGEN (ViSession instrumentHandle,
                                       ViInt16 output, ViReal64 percent)
{
    
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    
    // Verify output is in range
    // 3 = M1
    // 4 = M2
    if (ifr2975_invalidViInt16Range (output, 3, 4))
    {
      return VI_ERROR_PARAMETER2;
    }
        
    // Verify frequency is in range
    if (ifr2975_invalidViReal64Range (percent, 0, 100))
    {
      return VI_ERROR_PARAMETER2;
    }
        
    // Formats string to be transmitted 
    sprintf ((char*) wrtbuf, "FGen per%i %f ", output, percent);

    // This writes command to instrument 
    CHECKERR (ifr2975_setViString (instrumentHandle, wrtbuf, 0));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Set_Shape_FGEN       
* Purpose:  This function sets the waveshape of the selected function       
*           generator.                   
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_Shape_FGEN (ViSession instrumentHandle,
                                         ViInt16 output, ViInt16 waveshape)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    
    // Verify output is in range
    // 1 = FGEN1
    // 2 = FGEN2
    // 3 = M1
    // 4 = M2
    if (ifr2975_invalidViInt16Range (output, 1, 4))
    {
      return VI_ERROR_PARAMETER2;
    }
        
    // Verify waveshape is in range
    if (ifr2975_invalidViInt16Range (waveshape, 0, 3))
    {
      return VI_ERROR_PARAMETER3;
    }
    
    // Formats Data According to Input Data 
    switch (waveshape)
    {
      case 0:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "FGen SHape%i sine", output);
        break;
      case 1:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "FGen SHape%i triangle", output);
        break;
      case 2:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "FGen SHape%i square", output);
        break;
      case 3:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "FGen SHape%i ramp", output);
        break;
      default:
        return VI_ERROR_PARAMETER3;
    }
    
    // This writes command to instrument 
    CHECKERR (ifr2975_setViString (instrumentHandle, wrtbuf, 0));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Set_Dur_FGEN         
* Purpose:  This function sets the duration of the selected function        
*           generator.                   
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_Dur_FGEN (ViSession instrumentHandle,
                                       ViInt16 output, ViInt32 duration)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    
    // Verify output is in range
    // 1 = FGEN1
    // 2 = FGEN2
    // 3 = M1
    // 4 = M2
    if (ifr2975_invalidViInt16Range (output, 1, 4))
    {
      return VI_ERROR_PARAMETER2;
    }
        
    // Verify duration is in range
    if (ifr2975_invalidViInt32Range (duration, 1, 100000))
    {
      return VI_ERROR_PARAMETER3;
    }
        
    // Formats string to be transmitted 
    sprintf ((char*) wrtbuf, "FGen DURation%i %i ", output, duration);

    // This writes command to instrument 
    CHECKERR (ifr2975_setViString (instrumentHandle, wrtbuf, 0));
    
    return ifr2975_status;
}

/******************************************************************************
* Function: ifr2975_Set_Mode_FGEN        
* Purpose:  This function sets the mode of the selected function            
*           generator.                   
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_Mode_FGEN (ViSession instrumentHandle,
                                        ViInt16 output, ViInt16 mode)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    
    // Verify output is in range
    // 1 = FGEN1
    // 2 = FGEN2
    // 3 = M1
    // 4 = M2
    if (ifr2975_invalidViInt16Range (output, 1, 4))
    {
      return VI_ERROR_PARAMETER2;
    }
        
    // Verify mode is in range
    // 0 = Sine               
    // 1 = Triangle           
    // 2 = Square             
    // 3 = Ramp               
    if (ifr2975_invalidViInt16Range (mode, 0, 1)&&ifr2975_invalidViInt16Range (mode, 3, 7))
    {
      return VI_ERROR_PARAMETER3;
    }
        
    // Formats string to be transmitted 
    sprintf ((char*) wrtbuf, "FGen MODe%i %i ", output, mode);

    // This writes command to instrument 
    CHECKERR (ifr2975_setViString (instrumentHandle, wrtbuf, 0));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Synchro_FGEN         
* Purpose:  This function synchronizes the phase of the function generators 
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Synchro_FGEN (ViSession instrumentHandle)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_setViString (instrumentHandle, "FGen SYNC", "2"));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Set_Demod_Filt_BBA   
* Purpose:  This function sets the demodulation filter.                     
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_Demod_Filt_BBA (ViSession instrumentHandle,
                                             ViInt16 filterSetting)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    
    // Verify filter setting is in range
    // 0 = Square                   
    // 1 = Sine                      
    if (ifr2975_invalidViInt16Range (filterSetting, 0, 1))
    {
      return VI_ERROR_PARAMETER2;
    }
    
    // Formats Data According to Input Data 
    switch (filterSetting)
    {
      case 0:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "BBAudio DEMODFILter square ");
        break;
      case 1:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "BBAudio DEMODFILter sine ");
        break;
      default:
        return VI_ERROR_PARAMETER2;
    }
    
    // This writes command to instrument 
    CHECKERR (ifr2975_setViString (instrumentHandle, wrtbuf, 0));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Set_FGen_Filt_BBA    
* Purpose:  This function sets the FGEN filter.                             
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_FGen_Filt_BBA (ViSession instrumentHandle,
                                            ViInt16 filterSetting)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    
    // Verify filter setting is in range
    // 0 = Square                   
    // 1 = Sine                      
    if (ifr2975_invalidViInt16Range (filterSetting, 0, 1))
    {
      return VI_ERROR_PARAMETER2;
    }
    
    // Formats Data According to Input Data 
    switch (filterSetting)
    {
      case 0:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "BBAudio FGENFILTer square ");
        break;
      case 1:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "BBAudio FGENFILTer sine ");
        break;
      default:
        return VI_ERROR_PARAMETER2;
    }
    
    // This writes command to instrument 
    CHECKERR (ifr2975_setViString (instrumentHandle, wrtbuf, 0));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Set_FGen_Bal_BBA     
* Purpose:  This function sets the FGEN balance.                            
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_FGen_Bal_BBA (ViSession instrumentHandle,
                                           ViInt16 balanceSetting)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    
    // Verify balanceSetting setting is in range
    // 0 = Unbalanced                  
    // 1 = Balanced                      
    if (ifr2975_invalidViInt16Range (balanceSetting, 0, 1))
    {
      return VI_ERROR_PARAMETER2;
    }
    
    // Formats Data According to Input Data 
    switch (balanceSetting)
    {
      case 0:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "BBAudio fgenbalance unbalanced ");
        break;
      case 1:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "BBAudio fgenbalance balanced ");
        break;
      default:
        return VI_ERROR_PARAMETER2;
    }

    // This writes command to instrument 
    CHECKERR (ifr2975_setViString (instrumentHandle, wrtbuf, 0));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Set_FGen_Lev_BBA     
* Purpose:  This function sets the FGEN level (x1 or x.1).                  
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_FGen_Lev_BBA (ViSession instrumentHandle,
                                           ViInt16 level)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    
    // Verify level setting is in range
    // 0 = Normal                   
    // 1 = Tenth                     
    if (ifr2975_invalidViInt16Range (level, 0, 1))
    {
      return VI_ERROR_PARAMETER2;
    }
    
    // Formats Data According to Input Data 
    switch (level)
    {
      case 0:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "BBAudio FGENLEVel normal ");
        break;
      case 1:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "BBAudio FGENLEVel tenth ");
        break;
      default:
        return VI_ERROR_PARAMETER2;
    }
    
    // This writes command to instrument 
    CHECKERR (ifr2975_setViString (instrumentHandle, wrtbuf, 0));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Set_SINAD_Bal_BBA    
* Purpose:  This function sets the SINAD balance.                           
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_SINAD_Bal_BBA (ViSession instrumentHandle,
                                            ViInt16 balanceSetting)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    
    // Verify balanceSetting setting is in range
    // 0 = Balanced                   
    // 1 = Unbalanced               
    if (ifr2975_invalidViInt16Range (balanceSetting, 0, 1))
    {
      return VI_ERROR_PARAMETER2;
    }
    
    // Formats Data According to Input Data 
    switch (balanceSetting)
    {
      case 0:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "BBAudio SINADBALance balanced ");
        break;
      case 1:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "BBAudio SINADBALance unbalanced ");
        break;
      default:
        return VI_ERROR_PARAMETER2;
    }
    
    // This writes command to instrument 
    CHECKERR (ifr2975_setViString (instrumentHandle, wrtbuf, 0));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Set_Mic_Bias_BBA     
* Purpose:  This function sets the microphone bias.                         
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_Mic_Bias_BBA (ViSession instrumentHandle,
                                           ViInt16 microphoneBias)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];

    // Verify microphoneBias setting is in range
    // 0 = Normal                   
    // 1 = Powered                  
    if (ifr2975_invalidViInt16Range (microphoneBias, 0, 1))
    {
      return VI_ERROR_PARAMETER2;
    }
    
    // Formats Data According to Input Data 
    switch (microphoneBias)
    {
      case 0:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "BBAudio micbias normal ");
        break;
      case 1:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "BBAudio micbias powered ");
        break;
      default:
        return VI_ERROR_PARAMETER2;
    }

    // This writes command to instrument 
    CHECKERR (ifr2975_setViString (instrumentHandle, wrtbuf, 0));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Set_Aud_Mix_BBA      
* Purpose:  This function connects or disconnects signal inputs to a mixer. 
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_Aud_Mix_BBA (ViSession instrumentHandle,
                                          ViInt16 source, ViInt16 mixer,
                                          ViInt16 state)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    ViChar state_string[10];
    ViChar source_string[10];
    ViChar mixer_string[10];
    
    // Verify source setting is in range
    // 1 = L1
    // 2 = Mic
    // 4 = Sinad
    // 8 = File
    // 16 = Fgen0
    // 32 = Fgen1
    // 64 = Fgen2
    // 128 = Fgen3
    // 256 = Fgen4
    // 512 = User0
    // 1024 = User1
    // 2048 = User2
    // 4096 = User3
    // 8192 = User4
    // 16384 = User5
    if (ifr2975_invalidViInt16Range (source, 1, 2) && ifr2975_invalidViInt16Range (source, 4, 4)
                                                   && ifr2975_invalidViInt16Range (source, 8, 8)
                                                   && ifr2975_invalidViInt16Range (source, 16, 16)
                                                   && ifr2975_invalidViInt16Range (source, 32, 32)
                                                   && ifr2975_invalidViInt16Range (source, 64, 64)
                                                   && ifr2975_invalidViInt16Range (source, 128, 128)
                                                   && ifr2975_invalidViInt16Range (source, 256, 256)
                                                   && ifr2975_invalidViInt16Range (source, 512, 512)
                                                   && ifr2975_invalidViInt16Range (source, 1024, 1024)
                                                   && ifr2975_invalidViInt16Range (source, 2048, 2048)
                                                   && ifr2975_invalidViInt16Range (source, 4096, 4096)
                                                   && ifr2975_invalidViInt16Range (source, 8192, 8192)
                                                   && ifr2975_invalidViInt16Range (source, 16384, 16384))
    {
      return VI_ERROR_PARAMETER2;
    }
    
    // Formats Data According to Input Data 
    switch (source)
    {
      case 1:
        strcpy (source_string, "L1");
        break;
      case 2:
        strcpy (source_string, "Mic");
        break;
      case 4:
        strcpy (source_string, "Sinad");
        break;
      case 8:
        strcpy (source_string, "File");
        break;
      case 16:
        strcpy (source_string, "Fgen0");
        break;
      case 32:
        strcpy (source_string, "Fgen1");
        break;
      case 64:
        strcpy (source_string, "Fgen2");
        break;
      case 128:
        strcpy (source_string, "Fgen3");
        break;
      case 256:
        strcpy (source_string, "Fgen4");
        break;
      case 512:
        strcpy (source_string, "User0");
        break;
      case 1024:
        strcpy (source_string, "User1");
        break;
      case 2048:
        strcpy (source_string, "User2");
        break;
      case 4096:
        strcpy (source_string, "User3");
        break;
      case 8192:
        strcpy (source_string, "User4");
        break;
      case 16384:
        strcpy (source_string, "User5");
        break;
      default:
        return VI_ERROR_PARAMETER2;
    }
    
    // Verify mixer is in range
    // 1 = Speaker
    // 2 = Demod
    // 4 = Fgen
    // 8 = L1
    // 16 = User0
    // 32 = User1
    if (ifr2975_invalidViInt16Range (mixer, 1, 2) && ifr2975_invalidViInt16Range (mixer, 4, 4)
                                                  && ifr2975_invalidViInt16Range (mixer, 8, 8)
                                                  && ifr2975_invalidViInt16Range (mixer, 16, 16)
                                                  && ifr2975_invalidViInt16Range (mixer, 32, 32))
    {
      return VI_ERROR_PARAMETER3;
    }
    
    // Formats Data According to Input Data 
    switch (mixer)
    {
      case 1:
        strcpy (mixer_string , "Speaker");
        break;
      case 2:
        strcpy (mixer_string , "Demod");
        break;
      case 4:
        strcpy (mixer_string , "Fgen");
        break;
      case 8:
        strcpy (mixer_string , "L1");
        break;
      case 16:
        strcpy (mixer_string , "User0");
        break;
      case 32:
        strcpy (mixer_string , "User1");
        break;
      default:
        return VI_ERROR_PARAMETER2;
    }
        
    // Verify state is in range
    // 0 = Off                
    // 1 = On                
    if (ifr2975_invalidViInt16Range (state, 0, 1))
    {
      return VI_ERROR_PARAMETER3;
    }
    
    // Formats Data According to Input Data 
    switch (state)
    {
      case 0:
        strcpy (state_string, "off");
        break;
      case 1:
        strcpy (state_string, "on");
        break;
      default:
        return VI_ERROR_PARAMETER2;
    }
    
    // Formats string to be transmitted 
    sprintf ((char*) wrtbuf, "BBAudio MIXer %s %s %s ",source_string, mixer_string, state_string);

    // This writes command to instrument 
    CHECKERR (ifr2975_setViString (instrumentHandle, wrtbuf, 0));
    
    return ifr2975_status;
}

/******************************************************************************
* Function: ifr2975_Clear_Aud_Mix_BBA    
* Purpose:  This function clears all input data connections to all mixers.  
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Clear_Aud_Mix_BBA (ViSession instrumentHandle, ViInt16 mixer)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    ViChar mixer_string[10];
    
    if (ifr2975_invalidViInt16Range (mixer, 0, 5))
    {
      return VI_ERROR_PARAMETER2;
    }
        
    // Formats Data According to Input Data 
    switch (mixer)
    {
      case 0:
        strcpy (mixer_string , "Speaker");
        break;
      case 1:
        strcpy (mixer_string , "Demod");
        break;
      case 2:
        strcpy (mixer_string , "Fgen");
        break;
      case 3:
        strcpy (mixer_string , "L1");
        break;
      case 4:
        strcpy (mixer_string , "User0");
        break;
      case 5:
        strcpy (mixer_string , "User1");
        break;
      default:
        return VI_ERROR_PARAMETER2;
    }
    
    // Formats string to be transmitted 
    sprintf ((char*) wrtbuf, "BBAudio clearmixer %s", mixer_string);

    // This writes command to instrument 
    CHECKERR (ifr2975_setViString (instrumentHandle, wrtbuf, 0));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Set_Aud_Route_BBA    
* Purpose:  This function connects a data output source to a data consumer. 
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_Aud_Route_BBA (ViSession instrumentHandle,
                                            ViInt16 source, ViInt16 destination)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    ViChar source_string[10];
    ViChar destination_string[10];
    
    // Verify source range
    if (ifr2975_invalidViInt16Range (source, -1, 18))
    {
      return VI_ERROR_PARAMETER2;
    }
    
    // Formats Data According to Input Data 
    switch (source)
    {
      case -1:
        strcpy (source_string , "None");
        break;
      case 0:
        strcpy (source_string , "L1");
        break;
      case 1:
        strcpy (source_string , "Mic");
        break;
      case 2:
        strcpy (source_string , "Sinad");
        break;
      case 3:
        strcpy (source_string , "File");
        break;
      case 4:
        strcpy (source_string , "Fgen0");
        break;
      case 5:
        strcpy (source_string , "Fgen1");
        break;
      case 6:
        strcpy (source_string , "Fgen2");
        break;
      case 7:
        strcpy (source_string , "Fgen3");
        break;
      case 8:
        strcpy (source_string , "Fgen4");
        break;
      case 9:
        strcpy (source_string , "Speaker");
        break;
      case 10:
        strcpy (source_string , "Demod");
        break;
      case 11:
        strcpy (source_string , "Fgen");
        break;
      case 12:
        //strcpy (source_string , "None");
        strcpy (source_string , "MixL1");
        break;
      case 13:
        //strcpy (source_string , "User1");
        strcpy (source_string , "User0");
        break;
      case 14:
        //strcpy (source_string , "User2");
        strcpy (source_string , "User1");
        break;
      case 15:
        strcpy (source_string , "afcounter");
        break;
      case 16:
        strcpy (source_string , "Sinadmtr");
        break;
      case 17:
        strcpy (source_string , "Scope");
        break;
      case 18:
        strcpy (source_string , "Scalar");
        break;
      default:
        return VI_ERROR_PARAMETER2;
    }
    
    if (ifr2975_invalidViInt16Range (destination, 9, 18))
    {
      return VI_ERROR_PARAMETER2;
    }
    
    // Formats Data According to Input Data 
    switch (destination)
    {
      case 9:
        strcpy (destination_string, "User0");
        break;
      case 10:
        strcpy (destination_string, "User1");
        break;
      case 11:
        strcpy (destination_string, "User2");
        break;
      case 12:
        strcpy (destination_string, "User3");
        break;
      case 13:
        strcpy (destination_string, "User4");
        break;
      case 14:
        strcpy (destination_string, "User5");
        break;
      case 15:
        strcpy (destination_string, "AFCounter");
        break;
      case 16:
        strcpy (destination_string, "Sinadmtr");
        break;
      case 17:
        strcpy (destination_string, "Scope");
        break;
      case 18:
        strcpy (destination_string, "Scalar");
        break;
      default:
        return VI_ERROR_PARAMETER2;
    }
    
    // Formats string to be transmitted 
    sprintf ((char*) wrtbuf, "BBAudio audioROUTE %s %s ", source_string, destination_string);

    // This writes command to instrument 
    CHECKERR (ifr2975_setViString (instrumentHandle, wrtbuf, 0));
                 
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Set_Avg_RFERR        
* Purpose:  This function sets the samples to average the RF Error Meter    
*           measurement over.            
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_Avg_RFERR (ViSession instrumentHandle,
                                        ViInt16 samplestoAverageOver)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
        
    // This writes command to instrument 
    CHECKERR (ifr2975_setViInt16 (instrumentHandle, "RFError AVG", samplestoAverageOver, 1, 100));

    return ifr2975_status;
}

/******************************************************************************
* Function: ifr2975_Set_Pk_State_RFERR    
* Purpose:  This function sets the Peak Hold  for the RF Error Meter.       
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_Pk_State_RFERR (ViSession instrumentHandle,
                                            ViInt16 peakHoldMode)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    
    // Verify peakHoldMode is in range   
    // 0 = Disable peak hold             
    // 1 = Enable peak hold              
    if (ifr2975_invalidViInt16Range (peakHoldMode, 0, 1))
    {
      return VI_ERROR_PARAMETER2;
    }
    
    // Formats Data According to Input Data 
    switch (peakHoldMode)
    {
      case 0:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "RFError peakhold off ");
        break;
      case 1:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "RFError peakhold on ");
        break;
      default:
        return VI_ERROR_PARAMETER2;
    }

    // This writes command to instrument 
    CHECKERR (ifr2975_setViString (instrumentHandle, wrtbuf, 0));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Reset_Pk_Hold_RFERR  
* Purpose:  This function resets the RF Error Meter peak hold.              
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Reset_Pk_Hold_RFERR (ViSession instrumentHandle)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];

    // This writes command to instrument 
    CHECKERR (ifr2975_setViString (instrumentHandle, "RFError PHRST ", "2"));
    
    return ifr2975_status;
}

/******************************************************************************
* Function: ifr2975_Set_Alerts_RFERR     
* Purpose:  This function sets the Alert mode the RF Error Meter.           
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_Alerts_RFERR (ViSession instrumentHandle,
                                           ViInt16 alertMode)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    
    // Verify alertMode is in range
    // 0 = Off                      
    // 1 = Low On                  
    // 2 = Hi On                      
    // 3 = Low and Hi On           
    if (ifr2975_invalidViInt16Range (alertMode, 0, 3))
    {
      return VI_ERROR_PARAMETER2;
    }
    
    // Formats Data According to Input Data 
    switch (alertMode)
    {
      case 0:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "RFError alerts off ");
        break;
      case 1:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "RFError alerts enablelo ");
        break;
      case 2:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "RFError alerts enablehi ");
        break;
      case 3:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "RFError alerts enableboth ");
        break;
      default:
        return VI_ERROR_PARAMETER2;
    }
        
    // This writes command to instrument 
    CHECKERR (ifr2975_setViString (instrumentHandle, wrtbuf, 0));
    
    return ifr2975_status;
}

/******************************************************************************
* Function: ifr2975_Set_UL_RFERR         
* Purpose:  This function sets the Upper Alert Limit of the RF Error Meter. 
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_UL_RFERR (ViSession instrumentHandle,
                                       ViReal64 upperAlertLimit)
{
    ViStatus ifr2975_status = VI_SUCCESS; 

    // This writes command to instrument 
    CHECKERR (ifr2975_setViReal64 (instrumentHandle, "RFError ulimit", upperAlertLimit, -100, 100, 0));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Set_LL_RFERR         
* Purpose:  This function sets the Lower Alert Limit of the RF Error Meter. 
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_LL_RFERR (ViSession instrumentHandle,
                                       ViReal64 lowerAlertLimit)
{
    ViStatus ifr2975_status = VI_SUCCESS; 

    // This writes command to instrument 
    CHECKERR (ifr2975_setViReal64 (instrumentHandle, "RFError llimit", lowerAlertLimit, -100, 100, 0));
    
    return ifr2975_status;
}

/******************************************************************************
* Function: ifr2975_Set_Enable_State_RFERR                                  
* Purpose:  This function sets the RF Error Meter enabled data mode.        
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_Enable_State_RFERR (ViSession instrumentHandle,
                                           ViInt16 enableMode)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    
    // Verify enableMode is in range
    if (ifr2975_invalidViInt16Range (enableMode, 1, 1) && ifr2975_invalidViInt16Range (enableMode, 3, 3))
    {
      return VI_ERROR_PARAMETER2;
    }
    
    // Formats Data According to Input Data 
    switch (enableMode)
    {
      case 1:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "RFError enable disable ");
        break;
      case 3:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "RFError enable enable ");
        break;
      default:
        return VI_ERROR_PARAMETER2;
    }
        
    // This writes command to instrument 
    CHECKERR (ifr2975_setViString (instrumentHandle, wrtbuf, 0));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Set_Run_State_RFERR  
* Purpose:  This function sets the RF Error Meter run state.                
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_Run_State_RFERR (ViSession instrumentHandle,
                                        ViInt16 runMode)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    ViChar rdbuf[BUFFER_SIZE];
    
    // Verify runMode is in range
    if (ifr2975_invalidViInt16Range (runMode, 0, 1))
    {
      return VI_ERROR_PARAMETER2;
    }
    
    // Formats Data According to Input Data 
    switch (runMode)
    {
      case 0:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "RFError run off");
        break;
      case 1:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "RFError run on");
        break;
      default:
        return VI_ERROR_PARAMETER2;
    }
        
    // This writes command to instrument 
    CHECKERR (ifr2975_setViString (instrumentHandle, wrtbuf, 0));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Set_Gate_State_RFERR  
* Purpose:  This function sets the RF Error Meter enabled data mode.        
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_Gate_State_RFERR (ViSession instrumentHandle,
                                         ViInt16 gateTime)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    
    // Verify gateTime is in range
    if (ifr2975_invalidViInt16Range (gateTime, 0, 1))
    {
      return VI_ERROR_PARAMETER2;
    }
    
    // Formats Data According to Input Data 
    switch (gateTime)
    {
      case 0:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "RFError gate tenth ");
        break;
      case 1:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "RFError gate second ");
        break;
      default:
        return VI_ERROR_PARAMETER2;
    }
        
    // This writes command to instrument 
    CHECKERR (ifr2975_setViString (instrumentHandle, wrtbuf, 0));
    
    return ifr2975_status;
}

/******************************************************************************
* Function: ifr2975_Set_Avg_DVM          
* Purpose:  This function sets the number of samples to average over for the
*           DVM.                         
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_Avg_DVM (ViSession instrumentHandle,
                                      ViInt16 samplestoAverageOver)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_setViInt16 (instrumentHandle, "Dvm AVG", samplestoAverageOver, 2, 255));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Set_Pk_Hold_State_DVM
* Purpose:  This function enables/disable peak hold mode on the DVM         
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_Pk_Hold_State_DVM (ViSession instrumentHandle,
                                          ViInt16 peakHoldMode)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    
    // Verify peakHoldMode is in range   
    // 0 = Disable peak hold             
    // 1 = Enable peak hold              
    if (ifr2975_invalidViInt16Range (peakHoldMode, 0, 1))
    {
      return VI_ERROR_PARAMETER2;
    }
    
    // Formats Data According to Input Data 
    switch (peakHoldMode)
    {
      case 0:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Dvm peakhold off ");
        break;
      case 1:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Dvm peakhold on ");
        break;
      default:
        return VI_ERROR_PARAMETER2;
    }

    // This writes command to instrument 
    CHECKERR (ifr2975_setViString (instrumentHandle, wrtbuf, 0));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Reset_Pk_Hold_DVM    
* Purpose:  This function resets peak hold mode on the DVM                  
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Reset_Pk_Hold_DVM (ViSession instrumentHandle)
{
    ViStatus ifr2975_status = VI_SUCCESS; 

    // This writes command to instrument 
    CHECKERR (ifr2975_setViString (instrumentHandle, "Dvm PHRST ", "2"));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Set_Alerts_DVM       
* Purpose:  This function sets the alert enable status on the DVM           
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_Alerts_DVM (ViSession instrumentHandle,
                                         ViInt16 alertMode)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    
    // Verify alertMode is in range
    // 0 = Off                      
    // 1 = Low On                  
    // 2 = Hi On                      
    // 3 = Low and Hi On           
    if (ifr2975_invalidViInt16Range (alertMode, 0, 3))
    {
      return VI_ERROR_PARAMETER2;
    }
    
    // Formats Data According to Input Data 
    switch (alertMode)
    {
      case 0:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Dvm alerts off ");
        break;
      case 1:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Dvm alerts enablelo ");
        break;
      case 2:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Dvm alerts enablehi ");
        break;
      case 3:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Dvm alerts enableboth ");
        break;
      default:
        return VI_ERROR_PARAMETER2;
    }
        
    // This writes command to instrument 
    CHECKERR (ifr2975_setViString (instrumentHandle, wrtbuf, 0));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Set_UL_DVM           
* Purpose:  This function sets the upper alert limit on the DVM             
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_UL_DVM (ViSession instrumentHandle,
                                     ViReal64 upperAlertLimit)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_setViReal64 (instrumentHandle, "Dvm ulimit", upperAlertLimit, -100, 100, 0));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Set_LL_DVM           
* Purpose:  This function sets the lower alert limit on the DVM             
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_LL_DVM (ViSession instrumentHandle,
                                     ViReal64 lowerAlertLimit)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_setViReal64 (instrumentHandle, "Dvm llimit", lowerAlertLimit, -100, 100, 0));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Set_Enable_DVM       
* Purpose:  This function enables/disables data mode on the DVM             
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_Enable_DVM (ViSession instrumentHandle,
                                         ViInt16 enableMode)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    
    // Verify enableMode is in range
    // 1 = Disabled
    // 3 = Enabled
    if (ifr2975_invalidViInt16Range (enableMode, 1, 1) && ifr2975_invalidViInt16Range (enableMode, 3, 3))
    {
      return VI_ERROR_PARAMETER2;
    }
    
    // Formats Data According to Input Data 
    switch (enableMode)
    {
      case 1:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Dvm enable off ");
        break;
      case 3:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Dvm enable on ");
        break;
      default:
        return VI_ERROR_PARAMETER2;
    }

    // This writes command to instrument 
    CHECKERR (ifr2975_setViString (instrumentHandle, wrtbuf, 0));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Set_Run_State_DVM    
* Purpose:  This function enables/disables run mode on the DVM              
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_Run_State_DVM (ViSession instrumentHandle,
                                      ViInt16 runMode)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    
    // Verify runMode is in range
    // 0 = Disabled                   
    // 1 = Enabled                   
    if (ifr2975_invalidViInt16Range (runMode, 0, 1))
    {
      return VI_ERROR_PARAMETER2;
    }
    
    // Formats Data According to Input Data 
    switch (runMode)
    {
      case 0:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Dvm run off ");
        break;
      case 1:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Dvm run on ");
        break;
      default:
        return VI_ERROR_PARAMETER2;
    }

    // This writes command to instrument 
    CHECKERR (ifr2975_setViString (instrumentHandle, wrtbuf, 0));
    
    return ifr2975_status;
}

/******************************************************************************
* Function: ifr2975_Set_Math_Mode_DVM    
* Purpose:  This function sets the math mode on the DVM                     
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_Math_Mode_DVM (ViSession instrumentHandle,
                                            ViInt16 mathMode)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    
    // Verify mathMode is in range
    // 1 = None
    // 2 = dB
    // 3 = dBm
    // 4 = Average
    if (ifr2975_invalidViInt16Range (mathMode, 1, 4))
    {
      return VI_ERROR_PARAMETER2;
    }
    
    // Formats Data According to Input Data 
    switch (mathMode)
    {
      case 1:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Dvm MATH none ");
        break;
      case 2:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Dvm MATH dB ");
        break;
      case 3:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Dvm MATH dBm ");
        break;
      case 4:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Dvm MATH average ");
        break;
      default:
        return VI_ERROR_PARAMETER2;
    }
    
    // This writes command to instrument 
    CHECKERR (ifr2975_setViString (instrumentHandle, wrtbuf, 0));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Set_Imp_DVM          
* Purpose:  This function sets input impedance on the DVM                   
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_Imp_DVM (ViSession instrumentHandle,
                                      ViInt16 inputImpedance)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    
    // Verify inputImpedance is in range
    if (ifr2975_invalidViInt16Range (inputImpedance, 0, 0) && 
        ifr2975_invalidViInt16Range (inputImpedance, 150, 150) &&
        ifr2975_invalidViInt16Range (inputImpedance, 600, 600))
    {
      return VI_ERROR_PARAMETER2;
    }
        
    // Formats string to be transmitted 
    sprintf ((char*) wrtbuf, "Dvm impedance %i ", inputImpedance);

    // This writes command to instrument 
    CHECKERR (ifr2975_setViString (instrumentHandle, wrtbuf, 0));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Set_Hold_DVM         
* Purpose:  This function sets the hold state on the DVM                    
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_Hold_DVM (ViSession instrumentHandle,
                                       ViInt16 holdStatus)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    
    // Verify holdStatus is in range
    // 0 = Normal                   
    // 1 = Hold                     
    if (ifr2975_invalidViInt16Range (holdStatus, 0, 1))
    {
      return VI_ERROR_PARAMETER2;
    }
    
    // Formats Data According to Input Data 
    switch (holdStatus)
    {
      case 0:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Dvm HOLD off ");
        break;
      case 1:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Dvm HOLD on ");
        break;
      default:
        return VI_ERROR_PARAMETER2;
    }
    
    // This writes command to instrument 
    CHECKERR (ifr2975_setViString (instrumentHandle, wrtbuf, 0));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Set_Line_Filter_DVM  
* Purpose:  This function sets line filter on the DVM                       
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_Line_Filter_DVM (ViSession instrumentHandle,
                                              ViInt16 lineFilterSetting)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // Verify lineFilterSetting is in range
    if (ifr2975_invalidViInt16Range (lineFilterSetting, 50, 50) &&
        ifr2975_invalidViInt16Range (lineFilterSetting, 60, 60))
    {
      return VI_ERROR_PARAMETER2;
    }
        
    // This writes command to instrument 
    CHECKERR (ifr2975_setViInt16 (instrumentHandle, "Dvm LINEFILter", lineFilterSetting, 0, 0));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Set_Range_DVM        
* Purpose:  This function sets the hardware range on the DVM                
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_Range_DVM (ViSession instrumentHandle,
                                        ViInt16 range)
{
    ViStatus ifr2975_status = VI_SUCCESS; 

    // Verify range is in range
    if (ifr2975_invalidViInt16Range (range, -400, -400) && ifr2975_invalidViInt16Range (range, 0, 0)
                                                        && ifr2975_invalidViInt16Range (range, 4, 4)
                                                        && ifr2975_invalidViInt16Range (range, 40, 40)
                                                        && ifr2975_invalidViInt16Range (range, 400, 400))
    {
      return VI_ERROR_PARAMETER2;
    }
        
    // This writes command to instrument 
    CHECKERR (ifr2975_setViInt16 (instrumentHandle, "Dvm HWRANge", range, 0, 0));
    
    return ifr2975_status;
}



/******************************************************************************
* Function: ifr2975_Set_Source_DVM       
* Purpose:  This function sets the input source on the DVM                  
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_Source_DVM (ViSession instrumentHandle,
                                         ViInt16 inputSource)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    
    // Verify inputSource is in range
    // 0 = DVM                       
    // 1 = Scope 1                   
    // 2 = Scope 2                   
    if (ifr2975_invalidViInt16Range (inputSource, 0,2))
    {
      return VI_ERROR_PARAMETER2;
    }
    
    // Formats Data According to Input Data 
    switch (inputSource)
    {
      case 0:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Dvm source dvm ");
        break;
      case 1:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Dvm source scope1 ");
        break;
      case 2:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Dvm source scope2 ");
        break;
      default:
        return VI_ERROR_PARAMETER2;
    }

    // This writes command to instrument 
    CHECKERR (ifr2975_setViString (instrumentHandle, wrtbuf, 0));
    
    return ifr2975_status;
}

/******************************************************************************
* Function: ifr2975_Set_Enable_State_DVM 
* Purpose:  This function sets the mode on the DVM                          
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_Enable_State_DVM (ViSession instrumentHandle,
                                         ViInt16 enablemode)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    
    // Verify Mode is in range
    // 0 = AC                 
    // 1 = DC                 
    if (ifr2975_invalidViInt16Range (enablemode, 1,2))
    {
      return VI_ERROR_PARAMETER2;
    }
    
    // Formats Data According to Input Data 
    switch (enablemode)
    {
      case 1:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Dvm mode ac ");
        break;
      case 2:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Dvm mode dc ");
        break;
      default:
        return VI_ERROR_PARAMETER2;
    }

    // This writes command to instrument 
    CHECKERR (ifr2975_setViString (instrumentHandle, wrtbuf, 0));
    
    return ifr2975_status;
}

/******************************************************************************
* Function: ifr2975_Set_Avg_C4FM         
* Purpose:  This function sets the number of samples to average over for the
*           C4FM Meter.                  
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_Avg_C4FM (ViSession instrumentHandle,
                                       ViInt32 samplestoAverageOver)
{
    ViStatus ifr2975_status = VI_SUCCESS; 

    // This writes command to instrument 
    CHECKERR (ifr2975_setViInt32 (instrumentHandle, "C4fm AVG", samplestoAverageOver, 200, 65535));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Set_Trig_Mode_C4FM   
* Purpose:  This function sets the trigger mode for the                     
*           C4FM Meter.                  
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_Trig_Mode_C4FM (ViSession instrumentHandle,
                                             ViInt16 triggerMode)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    
    // Verify triggermode is in range
    // 1 = Continuous
    // 2 = One Shot
    if (ifr2975_invalidViInt16Range (triggerMode, 1, 2))
    {
      return VI_ERROR_PARAMETER2;
    }
    
    // Formats Data According to Input Data 
    switch (triggerMode)
    {
      case 1:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "C4fm trigger continuous ");
        break;
      case 2:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "C4fm trigger oneshot ");
        break;
      default:
        return VI_ERROR_PARAMETER2;
    }
    
    // This writes command to instrument 
    CHECKERR (ifr2975_setViString (instrumentHandle, wrtbuf, 0));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Set_Pk_Hold_State_C4FM                                  
* Purpose:  This function sets the peak hold mode for the                   
*           C4FM Meter.                  
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_Pk_Hold_State_C4FM (ViSession instrumentHandle,
                                           ViInt16 peakHoldMode)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    
    // Verify peakHoldMode is in range
    // 0 = Disable
    // 1 = Enable                     
    if (ifr2975_invalidViInt16Range (peakHoldMode, 0, 1))
    {
      return VI_ERROR_PARAMETER2;
    }
    
    // Formats Data According to Input Data 
    switch (peakHoldMode)
    {
      case 0:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "C4fm peakhold off ");
        break;
      case 1:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "C4fm peakhold on ");
        break;
      default:
        return VI_ERROR_PARAMETER2;
    }
    
    // This writes command to instrument 
    CHECKERR (ifr2975_setViString (instrumentHandle, wrtbuf, 0));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Reset_Pk_Hold_C4FM   
* Purpose:  This function reset the peak hold on the C4FM                   
*           meter.                       
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Reset_Pk_Hold_C4FM (ViSession instrumentHandle)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_setViString (instrumentHandle, "C4fm PHRST", "2"));
    
    return ifr2975_status;
}

/******************************************************************************
* Function: ifr2975_Set_Alerts_C4FM      
* Purpose:  This function sets the alert mode on the C4FM                   
*           meter.                       
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_Alerts_C4FM (ViSession instrumentHandle,
                                          ViInt16 alertMode)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    
    // Verify alertMode is in range
    // 0 = Off                      
    // 1 = Low On                  
    // 2 = Hi On                      
    // 3 = Low and Hi On           
    if (ifr2975_invalidViInt16Range (alertMode, 0, 3))
    {
      return VI_ERROR_PARAMETER2;
    }
    
    // Formats Data According to Input Data 
    switch (alertMode)
    {
      case 0:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "C4fm alerts off ");
        break;
      case 1:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "C4fm alerts enablelo ");
        break;
      case 2:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "C4fm alerts enablehi ");
        break;
      case 3:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "C4fm alerts enableboth ");
        break;
      default:
        return VI_ERROR_PARAMETER2;
    }

    // This writes command to instrument 
    CHECKERR (ifr2975_setViString (instrumentHandle, wrtbuf, 0));
    
    return ifr2975_status;
}

/******************************************************************************
* Function: ifr2975_Set_UL_C4FM          
* Purpose:  This function sets upper alert limit on the C4FM                
*           meter.                       
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_UL_C4FM (ViSession instrumentHandle,
                                      ViReal64 upperAlertLimit)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_setViReal64 (instrumentHandle, "C4fm ulimit", upperAlertLimit, 0, 50, 0));
    
    return ifr2975_status;
}

/******************************************************************************
* Function: ifr2975_Set_LL_C4FM          
* Purpose:  This function sets lower alert limit on the C4FM                
*           meter.                       
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_LL_C4FM (ViSession instrumentHandle,
                                      ViReal64 lowerAlertLimit)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_setViReal64 (instrumentHandle, "C4fm llimit", lowerAlertLimit, 0, 50, 0));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Set_Enable_State_C4FM
* Purpose:  This function enables/disables data mode on the C4FM            
*           meter.                       
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_Enable_State_C4FM (ViSession instrumentHandle,
                                          ViInt16 enableMode)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    
    // Verify enableMode is in range
    // 1 = Disabled
    // 3 = Enabled
    if (ifr2975_invalidViInt16Range (enableMode, 1, 1) && ifr2975_invalidViInt16Range (enableMode, 3, 3))
    {
      return VI_ERROR_PARAMETER2;
    }
    
    // Formats Data According to Input Data 
    switch (enableMode)
    {
      case 1:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "C4fm enable disable ");
        break;
      case 3:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "C4fm enable enable ");
        break;
      default:
        return VI_ERROR_PARAMETER2;
    }
        
    // This writes command to instrument 
    CHECKERR (ifr2975_setViString (instrumentHandle, wrtbuf, 0));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Set_Run_State_C4FM   
* Purpose:  This function enables/disables run mode on  the C4FM            
*           meter.                       
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_Run_State_C4FM (ViSession instrumentHandle,
                                       ViInt16 runMode)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    ViChar rdbuf[BUFFER_SIZE];
    
    // Verify runMode is in range
    // 0 = Disabled                   
    // 1 = Enabled                   
    if (ifr2975_invalidViInt16Range (runMode, 0, 1))
    {
      return VI_ERROR_PARAMETER2;
    }
    
    // Formats Data According to Input Data 
    switch (runMode)
    {
      case 0:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "C4fm run off ");
        break;
      case 1:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "C4fm run on ");
        break;
      default:
        return VI_ERROR_PARAMETER2;
    }

    // This writes command to instrument 
    CHECKERR (ifr2975_setViString (instrumentHandle, wrtbuf, 0));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Set_Sweep_OSCOPE     
* Purpose:  This function sets the sweep rate on the Oscillscope.           
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_Sweep_OSCOPE (ViSession instrumentHandle,
                                           ViReal64 sweepRate)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    
    // Verify sweepRate is in range
    if (ifr2975_invalidViReal64Range (sweepRate, 10, 10) && ifr2975_invalidViReal64Range (sweepRate, 20, 20)
                                                         && ifr2975_invalidViReal64Range (sweepRate, 50, 50)
                                                         && ifr2975_invalidViReal64Range (sweepRate, 100, 100)
                                                         && ifr2975_invalidViReal64Range (sweepRate, 200, 200)
                                                         && ifr2975_invalidViReal64Range (sweepRate, 500, 500)
                                                         && ifr2975_invalidViReal64Range (sweepRate, 1000, 1000)
                                                         && ifr2975_invalidViReal64Range (sweepRate, 2000, 2000)
                                                         && ifr2975_invalidViReal64Range (sweepRate, 5000, 5000)
                                                         && ifr2975_invalidViReal64Range (sweepRate, 10000, 10000)
                                                         && ifr2975_invalidViReal64Range (sweepRate, 20000, 20000)
                                                         && ifr2975_invalidViReal64Range (sweepRate, 50000, 50000)
                                                         && ifr2975_invalidViReal64Range (sweepRate, 100000, 100000)
                                                         && ifr2975_invalidViReal64Range (sweepRate, 200000, 200000)
                                                         && ifr2975_invalidViReal64Range (sweepRate, 500000, 500000)
                                                         && ifr2975_invalidViReal64Range (sweepRate, 1000000, 1000000)
                                                         && ifr2975_invalidViReal64Range (sweepRate, 2000000, 2000000)
                                                         && ifr2975_invalidViReal64Range (sweepRate, 5000000, 5000000)
                                                         && ifr2975_invalidViReal64Range (sweepRate, 10000000, 10000000))
    {
      return VI_ERROR_PARAMETER2;
    }
    
    if (sweepRate == 10) 
    {
      sprintf ((char*) wrtbuf, "Scope horizontalsweep 10 ");
    } 
    else if (sweepRate == 20) 
    {
      sprintf ((char*) wrtbuf, "Scope horizontalsweep 20 ");
    } 
    else if (sweepRate == 50) 
    {
      sprintf ((char*) wrtbuf, "Scope horizontalsweep 50 ");
    } 
    else if (sweepRate == 100) 
    {
      sprintf ((char*) wrtbuf, "Scope horizontalsweep 100 ");
    } 
    else if (sweepRate == 200) 
    {
      sprintf ((char*) wrtbuf, "Scope horizontalsweep 200 ");
    } 
    else if (sweepRate == 500) 
    {
      sprintf ((char*) wrtbuf, "Scope horizontalsweep 500 ");
    } 
    else if (sweepRate == 1000) 
    {
      sprintf ((char*) wrtbuf, "Scope horizontalsweep 1000 ");
    } 
    else if (sweepRate == 2000) 
    {
      sprintf ((char*) wrtbuf, "Scope horizontalsweep 2000 ");
    } 
    else if (sweepRate == 5000) 
    {
      sprintf ((char*) wrtbuf, "Scope horizontalsweep 5000 ");
    } 
    else if (sweepRate == 10000) 
    {
      sprintf ((char*) wrtbuf, "Scope horizontalsweep 10000 ");
    } 
    else if (sweepRate == 20000) 
    {
      sprintf ((char*) wrtbuf, "Scope horizontalsweep 20000 ");
    } 
    else if (sweepRate == 50000) 
    {
      sprintf ((char*) wrtbuf, "Scope horizontalsweep 50000 ");
    } 
    else if (sweepRate == 100000) 
    {
      sprintf ((char*) wrtbuf, "Scope horizontalsweep 100000 ");
    } 
    else if (sweepRate == 200000) 
    {
      sprintf ((char*) wrtbuf, "Scope horizontalsweep 200000 ");
    } 
    else if (sweepRate == 500000) 
    {
      sprintf ((char*) wrtbuf, "Scope horizontalsweep 500000 ");
    } 
    else if (sweepRate == 1000000) 
    {
      sprintf ((char*) wrtbuf, "Scope horizontalsweep 1000000 ");
    } 
    else if (sweepRate == 2000000) 
    {
      sprintf ((char*) wrtbuf, "Scope horizontalsweep 2000000 ");
    } 
    else if (sweepRate == 5000000) 
    {
      sprintf ((char*) wrtbuf, "Scope horizontalsweep 5000000 ");
    } 
    else if (sweepRate == 10000000) 
    {
      sprintf ((char*) wrtbuf, "Scope horizontalsweep 10000000 ");
    } 
    else 
    {
      return VI_ERROR_PARAMETER2;
    }

    // This writes command to instrument 
    CHECKERR (ifr2975_setViString (instrumentHandle, wrtbuf, 0));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Set_Vert_Scale_OSCOPE
* Purpose:  This function sets vertical scales on the Oscilloscope.         
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_Vert_Scale_OSCOPE (ViSession instrumentHandle,
                                                ViInt16 channel,
                                                ViInt16 verticalScale)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    
    // Verify channel is in range
    if (ifr2975_invalidViInt16Range (channel, 1, 2))
    {
      return VI_ERROR_PARAMETER2;
    }
    
    // Verify verticalScale is in range
    // 1=20mV/div
    // 2=50mV/div
    // 3=100mV/div
    // 4=200mV/div
    // 5=500mV/div
    // 6=1V/div
    // 7=2V/div
    // 8=5V/div
    // 9=10V/div
    // 10=20V/div
    // 11=50V/div
    if (ifr2975_invalidViInt16Range (verticalScale, 2, 12))
    {
      return VI_ERROR_PARAMETER3;
    }
    
    // Formats Data According to Input Data 
    switch (verticalScale)
    {
      case 2:
        sprintf ((char*) wrtbuf, "Scope scale%i 20mV ", channel);
        break;
      case 3:
        sprintf ((char*) wrtbuf, "Scope scale%i 50mV ", channel);
        break;
      case 4:
        sprintf ((char*) wrtbuf, "Scope scale%i 100mV ", channel);
        break;
      case 5:
        sprintf ((char*) wrtbuf, "Scope scale%i 200mV ", channel);
        break;
      case 6:
        sprintf ((char*) wrtbuf, "Scope scale%i 500mV ", channel);
        break;
      case 7:
        sprintf ((char*) wrtbuf, "Scope scale%i 1V ", channel);
        break;
      case 8:
        sprintf ((char*) wrtbuf, "Scope scale%i 2V ", channel);
        break;
      case 9:
        sprintf ((char*) wrtbuf, "Scope scale%i 5V ", channel);
        break;
      case 10:
        sprintf ((char*) wrtbuf, "Scope scale%i 10V ", channel);
        break;
      case 11:
        sprintf ((char*) wrtbuf, "Scope scale%i 20V ", channel);
        break;
      case 12:
        sprintf ((char*) wrtbuf, "Scope scale%i 50V ", channel);
        break;
      default:
        return VI_ERROR_PARAMETER2;
    }

    // This writes command to instrument 
    CHECKERR (ifr2975_setViString (instrumentHandle, wrtbuf, 0));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Set_Horiz_Pos_OSCOPE 
* Purpose:  This function sets the horizontal offset postion on the         
*           Oscilloscope.                
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_Horiz_Pos_OSCOPE (ViSession instrumentHandle,
                                               ViReal64 offset)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    offset = offset*10;
    
    // This writes command to instrument 
    CHECKERR (ifr2975_setViReal64 (instrumentHandle, "Scope horizontalposition", offset, -100, 100, 0));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Set_Trig_Type_OSCOPE 
* Purpose:  This function sets the trigger type on the                      
*           Oscilloscope.                
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_Trig_Type_OSCOPE (ViSession instrumentHandle,
                                               ViInt16 triggerType)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    
    // Verify triggerType is in range
    // 0=Auto    
    // 1=Normal  
    // 2=One Shot
    if (ifr2975_invalidViInt16Range (triggerType, 0, 2))
    {
      return VI_ERROR_PARAMETER2;
    }
    
    // Formats Data According to Input Data 
    switch (triggerType)
    {
      case 0:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Scope trigtype auto ");
        break;
      case 1:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Scope trigtype normal ");
        break;
      case 2:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Scope trigtype 2 ");
        break;
      default:
        return VI_ERROR_PARAMETER2;
    }
    
    // This writes command to instrument 
    CHECKERR (ifr2975_setViString (instrumentHandle, wrtbuf, 0));
    
    return ifr2975_status;
}




/******************************************************************************
* Function: ifr2975_Set_Trig_Source_OSCOPE                                  
* Purpose:  This function sets the trigger source on the                    
*           Oscilloscope.                
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_Trig_Source_OSCOPE (ViSession instrumentHandle,
                                                 ViInt16 triggerSource)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    
    // Verify triggerSource is in range
    // 0=External
    // 1=Ch. 1   
    // 2=Ch. 2   
    if (ifr2975_invalidViInt16Range (triggerSource, 0, 2))
    {
      return VI_ERROR_PARAMETER2;
    }
    
    // Formats Data According to Input Data 
    switch (triggerSource)
    {
      case 0:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Scope trigsource ext");
        break;
      case 1:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Scope trigsource ch1");
        break;
      case 2:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Scope trigsource ch2");
        break;
      default:
        return VI_ERROR_PARAMETER2;
    }
    
    // This writes command to instrument 
    CHECKERR (ifr2975_setViString (instrumentHandle, wrtbuf, 0));
    
    return ifr2975_status;
}

/******************************************************************************
* Function: ifr2975_Set_Trig_Lev_OSCOPE  
* Purpose:  This function sets the trigger level on the                     
*           Oscilloscope.                
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_Trig_Lev_OSCOPE (ViSession instrumentHandle,
                                              ViReal64 triggerLevel)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_setViReal64 (instrumentHandle, "Scope triglev", triggerLevel, -500, 500, 0));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Set_Trig_Pol_OSCOPE  
* Purpose:  This function sets the trigger polarity on the                  
*           Oscilloscope.                
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_Trig_Pol_OSCOPE (ViSession instrumentHandle,
                                              ViInt16 triggerPolarity)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    
    // Verify triggerPolarity is in range
    // 0=
    // 1=+
    if (ifr2975_invalidViInt16Range (triggerPolarity, 0, 1))
    {
      return VI_ERROR_PARAMETER2;
    }
    
    // Formats Data According to Input Data 
    switch (triggerPolarity)
    {
      case 0:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Scope trigpol neg ");
        break;
      case 1:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Scope trigpol pos ");
        break;
      default:
        return VI_ERROR_PARAMETER2;
    }
    
    // This writes command to instrument 
    CHECKERR (ifr2975_setViString (instrumentHandle, wrtbuf, 0));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Set_Pk_Hold_Reset_Flags_OSCOPE                          
* Purpose:  This function sets the peak hold reset flags on the             
*           Oscilloscope.                
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_Pk_Hold_Reset_Flags_OSCOPE (ViSession instrumentHandle,
                                                         ViInt16 flagBits)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_setViInt16 (instrumentHandle, "Scope peakreset", flagBits, 1, 31));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Set_Avg_OSCOPE       
* Purpose:  This function sets the samples to average over on the           
*           Oscilloscope.                
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_Avg_OSCOPE (ViSession instrumentHandle,
                                         ViInt16 samplestoAverageOver)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    ViChar rdbuf[BUFFER_SIZE];
    
    // This writes command to instrument 
    CHECKERR (ifr2975_setViInt16 (instrumentHandle, "Scope avg", samplestoAverageOver, 2, 255));
    
    return ifr2975_status;
}

/******************************************************************************
* Function: ifr2975_Set_Input_Coupling_OSCOPE                               
* Purpose:  This function sets the input coupling on the                    
*           Oscilloscope.                
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_Input_Coupling_OSCOPE (ViSession instrumentHandle,
                                                    ViInt16 channel,
                                                    ViInt16 inputCoupling)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    
    // Verify channel is in range
    if (ifr2975_invalidViInt16Range (channel, 1, 2))
    {
      return VI_ERROR_PARAMETER2;
    }
    
    // Verify inputCoupling is in range
    if (ifr2975_invalidViInt16Range (inputCoupling, 0, 1))
    {
      return VI_ERROR_PARAMETER3;
    }
    
    // Formats Data According to Input Data 
    switch (inputCoupling)
    {
      case 0:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Scope coupling%i ac ", channel);
        break;
      case 1:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Scope coupling%i dc ", channel);
        break;
      default:
        return VI_ERROR_PARAMETER3;
    }
    
    // This writes command to instrument 
    CHECKERR (ifr2975_setViString (instrumentHandle, wrtbuf, 0));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Set_Trace_Math_Mode_OSCOPE                              
* Purpose:  This function sets the trace math mode on the                   
*           Oscilloscope.                
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_Trace_Math_Mode_OSCOPE (ViSession instrumentHandle,
                                                     ViInt16 mathType)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    ViChar rdbuf[BUFFER_SIZE];
    
    // Verify channel is in range
    // 0=None    
    // 1=live-ref
    // 2=ref-live
    // 3=live+ref
    if (ifr2975_invalidViInt16Range (mathType, 1, 4))
    {
      return VI_ERROR_PARAMETER2;
    }
    
    // Formats Data According to Input Data 
    switch (mathType)
    {
      case 1:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Scope tracemath 1 ");
        break;
      case 2:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Scope tracemath 2 ");
        break;
      case 3:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Scope tracemath 3 ");
        break;
      case 4:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Scope tracemath 4 ");
        break;
      default:
        return VI_ERROR_PARAMETER2;
    }
    
    // This writes command to instrument 
    CHECKERR (ifr2975_setViString (instrumentHandle, wrtbuf, 0));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Set_Run_State_OSCOPE 
* Purpose:  This function turns a given trace on/off on the                 
*           Oscilloscope.                
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_Run_State_OSCOPE (ViSession instrumentHandle, ViInt16 control, ViInt16 runMode)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    
    if (ifr2975_invalidViInt16Range (control, 1, 2))
    {
      return VI_ERROR_PARAMETER3;
    }
    
    // Verify runMode is in range
    // 0=disable
    // 1=enable
    if (ifr2975_invalidViInt16Range (runMode, 0, 1))
    {
      return VI_ERROR_PARAMETER3;
    }
    
    // Formats Data According to Input Data 
    switch (runMode)
    {
      case 0:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Scope RUN%i off ",control);
        break;
      case 1:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Scope RUN%i on ",control);
        break;
      default:
        return VI_ERROR_PARAMETER2;
    }
        
    // This writes command to instrument 
    CHECKERR (ifr2975_setViString (instrumentHandle, wrtbuf, 0));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Set_Freq_REC         
* Purpose:  This function sets the receiver frequency.                      
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_Freq_REC (ViSession instrumentHandle,
                                       ViReal64 frequency)
{
    ViStatus ifr2975_status = VI_SUCCESS; 

    // This writes command to instrument 
    CHECKERR (ifr2975_setViReal64 (instrumentHandle, "Receiver FREQ ", frequency, 0, 2.8e9, 0));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Set_RFIn_REC         
* Purpose:  This function sets the receiver input port.                     
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_RFIn_REC (ViSession instrumentHandle, ViInt16 input)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    
    // Verify input is in range
    // 0 = T/R
    // 1 = ANT
    if (ifr2975_invalidViInt16Range (input, 0, 1)) 
    {
      return VI_ERROR_PARAMETER2;
    }

    switch (input)
    {
      case 0:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Receiver RFIN tr ");
        break;
      case 1:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Receiver RFIN ant ");
        break;
      default:
        return VI_ERROR_PARAMETER2;
    }

    // This writes command to instrument 
    CHECKERR (ifr2975_setViString (instrumentHandle, wrtbuf, 0));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Set_Attn_REC         
* Purpose:  This function sets the receiver attenuation.                    
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_Attn_REC (ViSession instrumentHandle,
                                       ViInt16 attenuation)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // Verify attenuation is in range
    if (ifr2975_invalidViInt16Range (attenuation, 0, 0) && ifr2975_invalidViInt16Range (attenuation, 10, 10) 
                                                        && ifr2975_invalidViInt16Range (attenuation, 20, 20) 
                                                        && ifr2975_invalidViInt16Range (attenuation, 30, 30) 
                                                        && ifr2975_invalidViInt16Range (attenuation, 40, 40))
    {
      return VI_ERROR_PARAMETER2;
    }
        
    // This writes command to instrument 
    CHECKERR (ifr2975_setViInt16 (instrumentHandle, "Receiver ATTEN", attenuation, 0, 0));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Set_Filter_REC       
* Purpose:  This function sets the receiver filter.                         
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_Filter_REC (ViSession instrumentHandle,
                                         ViInt16 filter)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_setViInt16 (instrumentHandle, "Receiver filter", filter, 0, 10));
    
    return ifr2975_status;    
}

/******************************************************************************
* Function: ifr2975_Set_Demod_REC        
* Purpose:  This function sets the receiver demodulation format.            
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_Demod_Rec (ViSession instrumentHandle,
                                        ViInt16 demodFormat)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    
    // Verify demodFormat is in range
    // 0=FMWide
    // 1=FMNarrow
    // 2=FMWideband
    // 3=sprintfV
    // 4=AM
    // 5=P25
    // 10=FMMedium
    // 11=Off
    if (ifr2975_invalidViInt16Range (demodFormat, 0, 5) && ifr2975_invalidViInt16Range (demodFormat, 10, 11))
    {
      return VI_ERROR_PARAMETER2;
    }
    
    
    // Formats Data According to Input Data 
    switch (demodFormat)
    {
      case 0:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Receiver demod fmwide ");
        break;
      case 1:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Receiver demod fmnarrow");
        break;
      case 2:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Receiver demod fmwideband");
        break;
      case 3:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Receiver demod fmtv");
        break;
      case 4:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Receiver demod am");
        break;
      case 5:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Receiver demod p25");
        break;
      case 10:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Receiver demod fmmedium");
        break;
      case 11:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Receiver demod off");
        break;
      default:
        return VI_ERROR_PARAMETER2;
    }
        
    // This writes command to instrument 
    CHECKERR (ifr2975_setViString (instrumentHandle, wrtbuf, 0));
    
    return ifr2975_status;
}

/******************************************************************************
* Function: ifr2975_Set_Range_REC        
* Purpose:  This function sets the receiver range.                          
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_Range_REC (ViSession instrumentHandle,
                                        ViInt16 range)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    
    // Verify range is in range
    // 0=Wide
    // 1=Narrow
    if (ifr2975_invalidViInt16Range (range, 0, 1))
    {
      return VI_ERROR_PARAMETER2;
    }
    
    // Formats Data According to Input Data 
    switch (range)
    {
      case 0:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Receiver RANGE wide ");
        break;
      case 1:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Receiver RANGE narrow ");
        break;
      default:
        return VI_ERROR_PARAMETER2;
    }
        
    // This writes command to instrument 
    CHECKERR (ifr2975_setViString (instrumentHandle, wrtbuf, 0));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Set_IFBW_REC         
* Purpose:  This function sets the receiver IF bandwidth.                   
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_IFBW_REC (ViSession instrumentHandle,
                                       ViReal64 IFBandwidth)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    ViChar rdbuf[BUFFER_SIZE];
    ViChar IFBW[10];
    
    // Verify IFBandwidth is in range
    // 0=30 kHz
    // 1=300 kHz
    // 2=3 MHz
    if (ifr2975_invalidViReal64Range (IFBandwidth, 30000, 30000) 
            && ifr2975_invalidViReal64Range (IFBandwidth, 300000, 300000)
            && ifr2975_invalidViReal64Range (IFBandwidth, 6000000, 6000000))
    {
      return VI_ERROR_PARAMETER2;
    }
    
    if ( IFBandwidth == 30000 ) 
    {
        strcpy(IFBW,"30 KHZ ");
    } 
    else if ( IFBandwidth == 300000 ) 
    {
        strcpy(IFBW,"300 KHZ ");
    } 
    else if ( IFBandwidth == 6000000 ) 
    {
        strcpy(IFBW,"6 MHZ ");
    } 
    else 
    {
        return VI_ERROR_PARAMETER2;
    }
        
    // Formats string to be transmitted 
    sprintf ((char*) wrtbuf, "Receiver ifbw %s ", IFBW);
    
    // This writes command to instrument 
    CHECKERR (ifr2975_setViString (instrumentHandle, wrtbuf, 0));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Set_IFGain_REC       
* Purpose:  This function sets the receiver IF Gain.                        
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_IFGain_REC (ViSession instrumentHandle,
                                         ViReal64 IFGain)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_setViReal64 (instrumentHandle, "Receiver IFGAIN", IFGain, -14, 82, 0));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Set_HWAGC_REC        
* Purpose:  This function sets the receiver hardware AGC speed.            
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_HWAGC_REC (ViSession instrumentHandle,
                                        ViInt16 AGCSpeed)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    
    // Verify AGCSpeed is in range
    // 0=Fast
    // 1=Slow
    if (ifr2975_invalidViInt16Range (AGCSpeed, 0, 1))
    {
      return VI_ERROR_PARAMETER2;
    }
    
    // Formats Data According to Input Data 
    switch (AGCSpeed)
    {
      case 0:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Receiver HWAGC fast ");
        break;
      case 1:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Receiver HWAGC slow ");
        break;
      default:
        return VI_ERROR_PARAMETER2;
    }
    
    // This writes command to instrument 
    CHECKERR (ifr2975_setViString (instrumentHandle, wrtbuf, 0));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Set_SWAGC_State_REC  
* Purpose:  This function sets the software AGC mode.                      
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_SWAGC_State_REC (ViSession instrumentHandle,
                                        ViInt16 AGCMode)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    
    // Verify AGCMode is in range
    // 0 = Fast					
    // 1 = Slow
    if (ifr2975_invalidViInt16Range (AGCMode, 0, 1))
    {
      return VI_ERROR_PARAMETER2;
    }
    
    // Formats Data According to Input Data 
    switch ( AGCMode )
    {
      case 0:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Receiver AGCMODE off ");
        break;
      case 1:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Receiver AGCMODE on ");
        break;
      default:
        return VI_ERROR_PARAMETER2;
    }
    
    // This writes command to instrument 
    CHECKERR (ifr2975_setViString (instrumentHandle, wrtbuf, 0));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Set_Sweep_Lock_State_REC                                
* Purpose:  This function sets the receiver lock setting.                   
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_Sweep_Lock_State_REC (ViSession instrumentHandle,
                                             ViInt16 setting)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    
    // Verify setting is in range
    // 0=Sweep
    // 1=Lock
    if (ifr2975_invalidViInt16Range (setting, 0, 1))
    {
      return VI_ERROR_PARAMETER2;
    }
    
    // Formats Data According to Input Data 
    switch (setting)
    {
      case 0:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Receiver LOCK sweep ");
        break;
      case 1:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Receiver LOCK lock ");
        break;
      default:
        return VI_ERROR_PARAMETER2;
    }
    
    // This writes command to instrument 
    CHECKERR (ifr2975_setViString (instrumentHandle, wrtbuf, 0));
    
    return ifr2975_status;
}

/******************************************************************************
* Function: ifr2975_Set_Freq_GEN         
* Purpose:  This function sets the Generator frequency.                     
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_Freq_GEN (ViSession instrumentHandle,
                                       ViReal64 frequency)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_setViReal64 (instrumentHandle, "Generator FREQ ", frequency, 0, 2.8e9, 0));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Set_Level_GEN        
* Purpose:  This function sets the Generator level.                         
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_Level_GEN (ViSession instrumentHandle,
                                        ViReal64 level)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_setViReal64 (instrumentHandle, "Generator RFLEV", level, -137, 10.0, 0));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Set_RFOut_GEN        
* Purpose:  This function sets the Generator output port.                  
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_RFOut_GEN (ViSession instrumentHandle,
                                              ViInt16 outputPort)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    
    // Verify outputPort is in range
    // 0=T/R
    // 1=Gen
    if (ifr2975_invalidViInt16Range (outputPort,0, 1))
    {
      return VI_ERROR_PARAMETER2;
    }
    
    // Formats Data According to Input Data 
    switch (outputPort)
    {
      case 0:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Generator RFOUT tr ");
        break;
      case 1:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Generator RFOUT gen ");
        break;
      default:
        return VI_ERROR_PARAMETER2;
    }

    // This writes command to instrument 
    CHECKERR (ifr2975_setViString (instrumentHandle, wrtbuf, 0));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Set_HWALC_GEN        
* Purpose:  This function sets the Generator hardware ALC speed.            
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_HWALC_GEN (ViSession instrumentHandle,
                                        ViInt16 ALCSpeed)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    
    // Verify ALCSpeed is in range
    // 0=Slow   
    // 1=Fast   
    // 2=Digital
    if (ifr2975_invalidViInt16Range (ALCSpeed,0, 2))
    {
      return VI_ERROR_PARAMETER2;
    }
    
    // Formats Data According to Input Data 
    switch (ALCSpeed)
    {
      case 0:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Generator ALC slow ");
        break;
      case 1:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Generator ALC fast ");
        break;
      case 2:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Generator ALC digital ");
        break;
      default:
        return VI_ERROR_PARAMETER2;
    }
    
    // This writes command to instrument 
    CHECKERR (ifr2975_setViString (instrumentHandle, wrtbuf, 0));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Set_BW_GEN           
* Purpose:  This function sets the Generator bandwidth.                     
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_BW_GEN (ViSession instrumentHandle,
                                     ViInt16 bandwidth)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    
    // Verify bandwidth is in range
    // 0=Narrow
    // 1=Wide
    if (ifr2975_invalidViInt16Range (bandwidth,0, 1))
    {
      return VI_ERROR_PARAMETER2;
    }
    
    // Formats Data According to Input Data 
    switch (bandwidth)
    {
      case 0:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Generator BW narrow ");
        break;
      case 1:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Generator BW wide ");
        break;
      default:
        return VI_ERROR_PARAMETER2;
    }
    
    // This writes command to instrument 
    CHECKERR (ifr2975_setViString (instrumentHandle, wrtbuf, 0));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Set_Gen_State_GEN    
* Purpose:  This function sets the Generator DC power On/Off.               
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_Gen_State_GEN (ViSession instrumentHandle,
                                         ViBoolean output)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    
    // Verify output is in range
    // 0=Off
    // 1=On 
    if (ifr2975_invalidViBooleanRange (output))
    {
      return VI_ERROR_PARAMETER2;
    }
    
    // Formats Data According to Input Data 
    switch (output)
    {
      case 0:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Generator dcpower off ");
        break;
      case 1:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Generator dcpower on ");
        break;
      default:
        return VI_ERROR_PARAMETER2;
    }
    
    // This writes command to instrument 
    CHECKERR (ifr2975_setViString (instrumentHandle, wrtbuf, 0));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Set_Yig_Speed_GEN    
* Purpose:  This function sets the Generator YIG speed.                    
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_Yig_Speed_GEN (ViSession instrumentHandle,
                                            ViInt16 speed)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    ViChar rdbuf[BUFFER_SIZE];
    
    // Verify speed is in range
    // 0=Fast
    // 1=Slow
    if (ifr2975_invalidViInt16Range (speed,0,1))
    {
      return VI_ERROR_PARAMETER2;
    }
    
    // Formats Data According to Input Data 
    switch (speed)
    {
      case 0:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Generator YIGSPEED fast ");
        break;
      case 1:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Generator YIGSPEED slow ");
        break;
      default:
        return VI_ERROR_PARAMETER2;
    }

    // This writes command to instrument 
    CHECKERR (ifr2975_setViString (instrumentHandle, wrtbuf, 0));
    
    return ifr2975_status;
}

/******************************************************************************
* Function: ifr2975_Set_Mod_GEN          
* Purpose:  This function sets the Generator modulation type.               
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_Mod_GEN (ViSession instrumentHandle,
                                      ViInt16 modulationMode)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    
    // Verify modulationMode is in range
    // 0=Off   
    // 1=FMN   
    // 2=FMW   
    // 3=PM    
    // 4=AM    
    // 5=PSK   
    // 6=USB   
    // 7=LSB   
    // 8=P25MIS
    // 9=P25MCC
    if (ifr2975_invalidViInt16Range (modulationMode,0,9))
    {
      return VI_ERROR_PARAMETER2;
    }
    
    // Formats Data According to Input Data 
    switch (modulationMode )
    {
      case 0:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Generator MOD off ");
        break;
      case 1:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Generator MOD FMN");
        break;
      case 2:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Generator MOD FMW");
        break;
      case 3:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Generator MOD PM");
        break;
      case 4:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Generator MOD AM");
        break;
      case 5:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Generator MOD PSK");
        break;
      case 6:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Generator MOD USB");
        break;
      case 7:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Generator MOD LSB");
        break;
      case 8:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Generator MOD P25MIS");
        break;
      case 9:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Generator MOD P25MCC");
        break;
      default:
        return VI_ERROR_PARAMETER2;
    }
    
    // This writes command to instrument 
    CHECKERR (ifr2975_setViString (instrumentHandle, wrtbuf, 0));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Set_P25_GEN          
* Purpose:  This function sets the Generator P25 mode.                      
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_P25_GEN (ViSession instrumentHandle, ViInt16 mode)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    
    // Verify mode is in range
    // 0=Off   
    // 1=Speech   
    // 5=File
    // 6=Hz1011
    // 7=Silence
    // 8=ViInt32erference
    // 9=Busy
    // 10=Idle
    // 11=Err5percent
    // 12=Lud1Trig
    // 13=NoTrig
    // 14=Lud2Trig
    // 15=Std511
    // 16=SymRate
    // 17=Deviation
    // 18=FidPattern
    // 19=FidSpectrum
    if ((ifr2975_invalidViInt16Range (mode,0,5)) && (ifr2975_invalidViInt16Range (mode,8,19)))
    {
      return VI_ERROR_PARAMETER2;
    }
        
    // Formats Data According to Input Data 
    switch (mode)
    {
      case 0:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Generator P25M off");
        break;
      case 1:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Generator P25M Speech");
        break;
      case 2:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Generator P25M Silence");
        break;
      case 3:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Generator P25M err5percent");
        break;
      case 4:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Generator P25M Hz1011");
        break;
      case 5:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Generator P25M File");
        break;
      case 8:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Generator P25M interference");
        break;
      case 9:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Generator P25M Busy");
        break;
      case 10:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Generator P25M idle"); 
        break;
      case 12:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Generator P25M lud1trig");
        break;
      case 13:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Generator P25M notrig");
        break;
      case 14:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Generator P25M lud2trig");
        break;
      case 15:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Generator P25M std511");
        break;
      case 16:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Generator P25M symrate");
        break;
      case 17:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Generator P25M lowdeviation");
        break;
      case 18:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Generator P25M fidpattern");
        break;
      case 19:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Generator P25M fidspectrum");
        break;
      default:
        return VI_ERROR_PARAMETER2;
    }
        
    // This writes command to instrument 
    CHECKERR (ifr2975_setViString (instrumentHandle, wrtbuf, 0));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Set_PTT_State_GEN    
* Purpose:  This function sets the Generator Push To Talk state.            
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_PTT_State_GEN (ViSession instrumentHandle, ViBoolean PTT)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    
    // Verify PTT is in range
    if (ifr2975_invalidViBooleanRange (PTT))
    {
      return VI_ERROR_PARAMETER2;
    }
    
    // Formats Data According to Input Data 
    switch (PTT)
    {
      case 0:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Generator ptt off ");
        break;
      case 1:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Generator ptt on ");
        break;
      default:
        return VI_ERROR_PARAMETER2;
    }
        
    // This writes command to instrument 
    CHECKERR (ifr2975_setViString (instrumentHandle, wrtbuf, 0));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Set_Decode_State_APCO
* Purpose:  This function enables/disables decoding of received P25 data.   
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_Decode_State_APCO (ViSession instrumentHandle,
                                             ViBoolean decode)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    
    // Verify decode is in range
    if (ifr2975_invalidViBooleanRange (decode))
    {
      return VI_ERROR_PARAMETER2;
    }
    
    // Formats Data According to Input Data 
    switch (decode)
    {
      case 0:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "P25 decode enable off ");
        break;
      case 1:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "P25 decode enable on ");
        break;
      default:
        return VI_ERROR_PARAMETER2;
    }
        
    // This writes command to instrument 
    CHECKERR (ifr2975_setViString (instrumentHandle, wrtbuf, 0));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Set_Dec_Sample_Rate_APCO                                
* Purpose:  This function sets the sample rate for received P25 data.       
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_Dec_Sample_Rate_APCO (ViSession instrumentHandle,
                                                   ViInt16 sampleRate)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    
    // Verify sampleRate is in range
    if (ifr2975_invalidViInt16Range (sampleRate,1,2) && ifr2975_invalidViInt16Range (sampleRate,4,4)
                                                     && ifr2975_invalidViInt16Range (sampleRate,8,8)
                                                     && ifr2975_invalidViInt16Range (sampleRate,16,16)
                                                     && ifr2975_invalidViInt16Range (sampleRate,32,32)
                                                     && ifr2975_invalidViInt16Range (sampleRate,64,64)
                                                     && ifr2975_invalidViInt16Range (sampleRate,128,128))
    {
      return VI_ERROR_PARAMETER2;
    }
        
    // This writes command to instrument 
    CHECKERR (ifr2975_setViInt16 (instrumentHandle, "P25 decode samplerate", sampleRate, 0, 0));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Set_Encrypt_State_APCO                                  
* Purpose:  This function enables/disables encryption for transmitted P25   
*           data.                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_Encrypt_State_APCO (ViSession instrumentHandle, ViInt16 Encrypt_State, ViInt16 ALGID)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    
    if (ifr2975_invalidViUInt16Range (ALGID, 0, 255))
    {
      return VI_ERROR_PARAMETER2;
    }
        
    sprintf ((char*) wrtbuf, "P25 encode encryptenable  %i  %i ", Encrypt_State, ALGID);
 
    // This writes command to instrument 
    CHECKERR (ifr2975_setViString (instrumentHandle, wrtbuf, 0));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Set_Encode_Bits_APCO 
* Purpose:  This function sets the P25 encoding for differenct key IDs.     
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_Encode_Bits_APCO (ViSession instrumentHandle,
                                            ViInt16 encode, ViInt32 value)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    
    // Formats Data According to Input Data 
    switch (encode)
    {
      case 1:
        if (ifr2975_invalidViReal64Range (value ,0,4095))
        {
          return VI_ERROR_PARAMETER3;
        }

        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "P25 encode field NAC %i ", value);
        break;
      case 2:
        if (ifr2975_invalidViReal64Range (value ,0,255))
        {
          return VI_ERROR_PARAMETER3;
        }
        
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "P25 encode field MFID %i ", value);
        break;
      case 3:
        if (ifr2975_invalidViReal64Range (value ,0,65535))
        {
          return VI_ERROR_PARAMETER3;
        }
        
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "P25 encode field TGID %i ", value);
        break;
      case 4:
        if (ifr2975_invalidViReal64Range (value ,0,65535))
        {
          return VI_ERROR_PARAMETER3;
        }
        
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "P25 encode field LSD1 %i ", value);
        break;
      case 5:
        if (ifr2975_invalidViReal64Range (value ,0,65535))
        {
          return VI_ERROR_PARAMETER3;
        }
        
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "P25 encode field LSD2 %i ", value);
        break;
      case 6:
        if (ifr2975_invalidViReal64Range (value ,0,63))
        {
          return VI_ERROR_PARAMETER3;
        }
          
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "P25 encode field LCO %i ", value);
        break;
      case 7:
        if (ifr2975_invalidViReal64Range (value ,0,16777215))
        {
          return VI_ERROR_PARAMETER3;
        }
        
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "P25 encode field DID %i ", value);
        break;
      case 8:
        if (ifr2975_invalidViReal64Range (value ,0,16777215))
        {
          return VI_ERROR_PARAMETER3;
        }
        
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "P25 encode field SID %i ", value);
        break;
      case 9:
        if (ifr2975_invalidViReal64Range (value ,0,1))
        {
          return VI_ERROR_PARAMETER3;
        }
        
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "P25 encode field EMG %i ", value);
        break;
      case 10:
        if (ifr2975_invalidViReal64Range (value ,0,1))
        {
          return VI_ERROR_PARAMETER3;
        }

        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "P25 encode field SF %i ", value);
        break;
      case 11:
        if (ifr2975_invalidViReal64Range (value ,0,1))
        {
          return VI_ERROR_PARAMETER3;
        }
        
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "P25 encode field P %i ", value);
        break;
      case 12:
        if (ifr2975_invalidViReal64Range (value ,0,65535))
        {
          return VI_ERROR_PARAMETER3;
        }
        
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "P25 encode field KEYID %i ", value);
        break;
      case 13:
        if (ifr2975_invalidViReal64Range (value ,0,3))
        {
          return VI_ERROR_PARAMETER3;
        }

        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "P25 encode field STATUS %i ", value);
        break;
      default:
        return VI_ERROR_PARAMETER2;
    }
    
    // This writes command to instrument 
    CHECKERR (ifr2975_setViString (instrumentHandle, wrtbuf, 0));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Set_C4FM_Mod_APCO    
* Purpose:  This function sets the C4FM modulation percentage.             
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_C4FM_Mod_APCO (ViSession instrumentHandle,
                                            ViReal64 modulation)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_setViReal64 (instrumentHandle, "P25 encode C4FM", modulation, 0, 100, 0));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Set_Enable_State_WBPWR                                  
* Purpose:  This function sets the enabled data mode(s) on the power meter.
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_Enable_State_WBPWR (ViSession instrumentHandle,
                                                ViInt16 enableMode)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    
    // Verify enableMode is in range
    if (ifr2975_invalidViInt16Range (enableMode,0,1))
    {
      return VI_ERROR_PARAMETER2;
    }
    
    // Formats Data According to Input Data 
    switch (enableMode)
    {
      case 0:
        sprintf ((char*) wrtbuf, "Power enable off");
        break;
      case 1:
        sprintf ((char*) wrtbuf, "Power enable on");
        break;
      default:
        return VI_ERROR_PARAMETER2;
    }
            
    // This writes command to instrument 
    CHECKERR (ifr2975_setViString (instrumentHandle, wrtbuf, 0));
    
    return ifr2975_status;
}



/******************************************************************************
* Function: ifr2975_Zero_WBPWR           
* Purpose:  This function zeroes the power meter.                          
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Zero_WBPWR (ViSession instrumentHandle)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_setViString (instrumentHandle, "Power ZERO ", 0));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Set_Range_WBPWR      
* Purpose:  This function sets the power meter range.                       
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_Range_WBPWR (ViSession instrumentHandle,
                                          ViInt16 range)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    
    // Verify range is in range
    // 0 = Auto 
    // 1 = Low  
    // 2 = High 
    if (ifr2975_invalidViInt16Range (range,0,2))
    {
      return VI_ERROR_PARAMETER2;
    }
    
    // Formats Data According to Input Data 
    switch (range)
    {
      case 0:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Power prange auto ");
        break;
      case 1:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Power prange low ");
        break;
      case 2:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Power prange high ");
        break;
      default:
        return VI_ERROR_PARAMETER2;
    }
            
    // This writes command to instrument 
    CHECKERR (ifr2975_setViString (instrumentHandle, wrtbuf, 0));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Set_Det_WBPWR        
* Purpose:  This function sets the power meter detector speed.             
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_Det_WBPWR (ViSession instrumentHandle,
                                        ViInt16 speed)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    
    // Verify speed is in range
    // 0 = Fast 
    // 1 = Slow 
    if (ifr2975_invalidViInt16Range (speed,0,1))
    {
      return VI_ERROR_PARAMETER2;
    }
    
    // Formats Data According to Input Data 
    switch (speed)
    {
      case 0:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Power dect fast ");
        break;
      case 1:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "Power dect slow ");
        break;
      default:
        return VI_ERROR_PARAMETER2;
    }
            
    // This writes command to instrument 
    CHECKERR (ifr2975_setViString (instrumentHandle, wrtbuf, 0));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Get_Enable_State_WBPWR                                  
* Purpose:  This function returns the power meter enabled mode(s).          
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_Enable_State_WBPWR (ViSession instrumentHandle,
                                           ViInt16 *enabledModes)
{
    ViStatus     ifr2975_status = VI_SUCCESS; 
    ViChar        rdbuf[100];
    int temp;
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViString (instrumentHandle, "Power enable", rdbuf, 100));
    
    if(0 == strncmp("Off",rdbuf,3))
    {
      *enabledModes = 0;
    }
    else if(0 == strncmp ("On", rdbuf, 2))
    {
      *enabledModes = 1;
    }
    else
    {
      return VI_ERROR_PARAMETER2;
    }
    
    return ifr2975_status;
}



/******************************************************************************
* Function: ifr2975_Get_Enable_State_WBPWR                                  
* Purpose:  This function returns the power meter range.                    
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_Range_WBPWR (ViSession instrumentHandle,
                                          ViInt16 *range)
{
    ViStatus ifr2975_status = VI_SUCCESS; 

    // This writes command to instrument 
    CHECKERR (ifr2975_getViInt16 (instrumentHandle, "Power prange ", range));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Get_Enable_State_WBPWR                                  
* Purpose:  This function returns the power meter range.                    
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_Det_WBPWR (ViSession instrumentHandle,
                                        ViInt16 *detectorSetting)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViInt16 (instrumentHandle, "Power dect", detectorSetting));
    
    return ifr2975_status;
}



/******************************************************************************
* Function: ifr2975_Set_Enable_State_RLMeter                                
* Purpose:  This function sets the enabled data mode(s) on the Receive      
*           Level Meter.                 
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_Enable_State_RLMeter (ViSession instrumentHandle,
                                                  ViInt16 enableMode)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    
    // Verify enableMode is in range
    // 0 = Off 
    // 1 = On 
    if (ifr2975_invalidViInt16Range (enableMode,1,1) && ifr2975_invalidViInt16Range (enableMode,3,3))
    {
      return VI_ERROR_PARAMETER2;
    }
    
    // Formats Data According to Input Data 
    switch (enableMode)
    {
      case 1:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "RLmeter enable Off ");
        break;
      case 3:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "RLmeter enable On ");
        break;
      default:
        return VI_ERROR_PARAMETER2;
    }
    
    // This writes command to instrument 
    CHECKERR (ifr2975_setViString (instrumentHandle, wrtbuf, 0));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Set_Update_Rate_RLMeter                                 
* Purpose:  This function sets the update rate on the Receive               
*           Level Meter.                 
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_Update_Rate_RLMeter (ViSession instrumentHandle,
                                                  ViInt16 updateRate)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_setViInt16 (instrumentHandle, "RLmeter RATe", updateRate, 20, 10000));
    
    return ifr2975_status;
}

/******************************************************************************
* Function: ifr2975_Set_Cable_Loss_RLMeter                                  
* Purpose:  This function sets the cable loss on the Receive                
*           Level Meter.                 
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_Cable_Loss_RLMeter (ViSession instrumentHandle,
                                                 ViReal64 cableLoss)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_setViReal64 (instrumentHandle, "RLmeter CABLEloss", cableLoss, -60, 60, 0));
    
    return ifr2975_status;
}



/******************************************************************************
* Function: ifr2975_Set_Vert_Scale_SA    
* Purpose:  This function sets the spectrum analyzer vertical scale         
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_Vert_Scale_SA (ViSession instrumentHandle,
                                            ViInt16 scale)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // Verify scale is in range
    if (ifr2975_invalidViInt16Range (scale,10,10) && ifr2975_invalidViInt16Range (scale,5,5)
                                                  && ifr2975_invalidViInt16Range (scale,2,2))
    {
      return VI_ERROR_PARAMETER2;
    }
        
    // This writes command to instrument 
    CHECKERR (ifr2975_setViInt16 (instrumentHandle, "Analyzer verticalSCALE", scale, 0, 0));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Set_TOS_SA           
* Purpose:  This function sets the spectrum analyzer top of screen          
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_TOS_SA (ViSession instrumentHandle,
                                     ViReal64 topofScreen)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_setViReal64 (instrumentHandle, "Analyzer topofscreen", topofScreen, -99.9, 80.0, 0));
    
    return ifr2975_status;
}

/******************************************************************************
* Function: ifr2975_Set_Span_SA          
* Purpose:  This function sets the spectrum analyzer span                   
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_Span_SA (ViSession instrumentHandle,
                                      ViReal64 startFrequency,
                                      ViReal64 stopFrequency)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    
    // Verify startFrequency is in range
    if (ifr2975_invalidViReal64Range (startFrequency,0,3000))
    {
      return VI_ERROR_PARAMETER2;
    }
        
    // Verify stopFrequency is in range
    if (ifr2975_invalidViReal64Range (stopFrequency,-1,3000))
    {
      return VI_ERROR_PARAMETER3;
    }
        
    // Formats string to be transmitted 
    sprintf ((char*) wrtbuf, "Analyzer SPAN %f %f ",startFrequency, stopFrequency);
    
    // This writes command to instrument 
    CHECKERR (ifr2975_setViString (instrumentHandle, wrtbuf, 0));
    
    return ifr2975_status;
}

/******************************************************************************
* Function: ifr2975_Set_RBW_SA           
* Purpose:  This function sets the spectrum analyzer resolution bandwidth  
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_RBW_SA (ViSession instrumentHandle,
                                     ViReal64 resolutionBandwidth)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    ViChar rdbuf[BUFFER_SIZE];
    
    // Verify resolutionBandwidth is in range
    if (ifr2975_invalidViReal64Range (resolutionBandwidth,300,300) 
        && ifr2975_invalidViReal64Range (resolutionBandwidth,3e3,3e3)
        && ifr2975_invalidViReal64Range (resolutionBandwidth,30e3,30e3)
        && ifr2975_invalidViReal64Range (resolutionBandwidth,60e3,60e3)
        && ifr2975_invalidViReal64Range (resolutionBandwidth,300e3,300e3)
        && ifr2975_invalidViReal64Range (resolutionBandwidth,6e6,6e6))
    {
      return VI_ERROR_PARAMETER2;
    }
        
    // This writes command to instrument 
    CHECKERR (ifr2975_setViReal64 (instrumentHandle, "Analyzer resolutionbandwidth", resolutionBandwidth, 0, 0, "Hz"));
    
    return ifr2975_status;
}

/******************************************************************************
* Function: ifr2975_Set_VBW_SA           
* Purpose:  This function sets the spectrum analyzer video bandwidth        
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_VBW_SA (ViSession instrumentHandle,
                                     ViReal64 videoBandwidth)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // Verify videoBandwidth is in range
    if (ifr2975_invalidViReal64Range (videoBandwidth,0,0) 
        && ifr2975_invalidViReal64Range (videoBandwidth,10,10)
        && ifr2975_invalidViReal64Range (videoBandwidth,30,30)
        && ifr2975_invalidViReal64Range (videoBandwidth,100,100)
        && ifr2975_invalidViReal64Range (videoBandwidth,300,300)
        && ifr2975_invalidViReal64Range (videoBandwidth,1e3,1e3)
        && ifr2975_invalidViReal64Range (videoBandwidth,3e3,3e3)
        && ifr2975_invalidViReal64Range (videoBandwidth,10e3,10e3)
        && ifr2975_invalidViReal64Range (videoBandwidth,30e3,30e3)
        && ifr2975_invalidViReal64Range (videoBandwidth,100e3,100e3)
        && ifr2975_invalidViReal64Range (videoBandwidth,300e3,300e3)
        && ifr2975_invalidViReal64Range (videoBandwidth,1e6,1e6)
        && ifr2975_invalidViReal64Range (videoBandwidth,3e6,3e6))
    {
      return VI_ERROR_PARAMETER2;
    }
    
    // This writes command to instrument 
    CHECKERR (ifr2975_setViReal64 (instrumentHandle, "Analyzer videobandwidth", videoBandwidth, 0, 0, "Hz"));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Set_IFGain_SA        
* Purpose:  This function sets the spectrum analyzer IF Gain                
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_IFGain_SA (ViSession instrumentHandle,
                                        ViReal64 IFGain)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_setViReal64 (instrumentHandle, "Analyzer IFGAIN", IFGain, -14.0, 82.0, 0));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Set_Run_State_SA     
* Purpose:  This function sets the spectrum analyzer run mode.              
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_Run_State_SA (ViSession instrumentHandle,
                                          ViInt16 runState)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_setViInt16 (instrumentHandle, "Analyzer RUNanalyzer", runState, 1, 3));
    
    return ifr2975_status;
}

/******************************************************************************
* Function: ifr2975_Set_Norm_State_SA    
* Purpose:  This function start/stops the spectrum analyzer normalize state.
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_Norm_State_SA (ViSession instrumentHandle,
                                             ViInt16 normalizeState)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_setViInt16 (instrumentHandle, "Analyzer normalize", normalizeState, 0, 1));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Set_Sweep_SA         
* Purpose:  This function sets the spectrum analyzer sweep speed.           
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_Sweep_SA (ViSession instrumentHandle,
                                       ViReal64 speed)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // Verify speed) is in range
    if (ifr2975_invalidViReal64Range (speed,20000,20000)
        && ifr2975_invalidViReal64Range (speed,50000,50000)
        && ifr2975_invalidViReal64Range (speed,100000,100000)
        && ifr2975_invalidViReal64Range (speed,200000,200000)
        && ifr2975_invalidViReal64Range (speed,500000,500000)
        && ifr2975_invalidViReal64Range (speed,1000000,1000000)
        && ifr2975_invalidViReal64Range (speed,2000000,2000000)
        && ifr2975_invalidViReal64Range (speed,5000000,5000000)
        && ifr2975_invalidViReal64Range (speed,10000000,10000000))
    {
      return VI_ERROR_PARAMETER2;
    }
        
    // This writes command to instrument 
    CHECKERR (ifr2975_setViReal64 (instrumentHandle, "Analyzer sweepspeed", speed, 0, 0, "2"));

    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Set_Tracking_Gen_State_SA                               
* Purpose:  This function sets the spectrum analyzer run mode.              
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_Tracking_Gen_State_SA (ViSession instrumentHandle,
                                          ViInt16 tracking)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_setViInt16 (instrumentHandle, "Analyzer tgstate", tracking, 0, 1));
    
    return ifr2975_status;
}

/******************************************************************************
* Function: ifr2975_Set_Screen_Mode      
* Purpose:  This function sets the screen mode                              
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_Screen_Mode (ViSession instrumentHandle,
                                          ViInt16 screenMode)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    ViChar rdbuf[BUFFER_SIZE];
    
    // Verify screenMode is in range
    if (ifr2975_invalidViInt16Range (screenMode,1,12))
    {
      return VI_ERROR_PARAMETER2;
    }
    
    // Formats Data According to Input Data 
    switch (screenMode)
    {
      case 1:
        sprintf ((char*) wrtbuf, "::Screens::Show . Gen_Screen ");
        break;
      case 2:
        sprintf ((char*) wrtbuf, "::Screens::Show . Rec_Screen ");
        break;
      case 3:
        sprintf ((char*) wrtbuf, "::Screens::Show . Duplex_Screen ");
        break;
      case 4:
        sprintf ((char*) wrtbuf, "::Screens::Show . Scope_Screen ");
        break;
      case 5:
        sprintf ((char*) wrtbuf, "::Screens::Show . Analyzer_Screen ");
        break;
      case 6:
        sprintf ((char*) wrtbuf, "::Screens::Show . Analyzer_Channel_Screen ");
        break;
      case 7:
        sprintf ((char*) wrtbuf, "::Screens::Show . AFGen_Screen ");
        break;
      case 8:
        sprintf ((char*) wrtbuf, "::Screens::Show . Config_Screen ");
        break;
      case 9:
        sprintf ((char*) wrtbuf, "::Screens::Show . TimeDate_Screen ");
        break;
      case 10:
        sprintf ((char*) wrtbuf, "::Screens::Show . Version_Screen ");
        break;
      default:
        return VI_ERROR_PARAMETER2;
    }
    
    // This writes command to instrument 
    CHECKERR (ifr2975_setViString (instrumentHandle, wrtbuf, 0));

    ifr2975_delay(1.5);
    
    return ifr2975_status;
}

/******************************************************************************
* Function: ifr2975_Set_Time_Date_MISC   
* Purpose:  This function sets the system time                              
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_Time_Date_MISC (ViSession instrumentHandle,
                                             ViInt16 month, ViInt16 day,
                                             ViInt16 year, ViInt16 hour,
                                             ViInt16 minute)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    
    // Verify month is in range
    if (ifr2975_invalidViInt16Range (month,1,12))
    {
      return VI_ERROR_PARAMETER2;
    }
        
    // Verify day is in range
    if (ifr2975_invalidViInt16Range (day,1,31))
    {
      return VI_ERROR_PARAMETER3;
    }
        
    // Verify year is in range
    if (ifr2975_invalidViInt16Range (year,2002,2037))
    {
      return VI_ERROR_PARAMETER4;
    }
        
    // Verify hour is in range
    if (ifr2975_invalidViInt16Range (hour,0,23))
    {
      return VI_ERROR_PARAMETER5;
    }
        
    // Verify minute is in range
    if (ifr2975_invalidViInt16Range (minute,0,59))
    {
      return VI_ERROR_PARAMETER6;
    }
        
    // Formats string to be transmitted 
    sprintf ((char*) wrtbuf, "HW_control::Set_clock \"%d:%d:01 %d/%d/%d\" ",hour,minute,month,day,year);
    
    // This writes command to instrument 
    CHECKERR (ifr2975_setViString (instrumentHandle, wrtbuf, 0));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Set_Ref_MISC         
* Purpose:  This function sets the system reference source input            
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Set_Ref_MISC (ViSession instrumentHandle,
                                       ViBoolean referenceSource)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    
    // Verify referenceSource is in range
    if (ifr2975_invalidViInt16Range (referenceSource,0,1))
    {
      return VI_ERROR_PARAMETER2;
    }
    
    // Formats Data According to Input Data 
    switch (referenceSource)
    {
      case 0:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "HW_control ref internal ");
        break;
      case 1:
        // Formats string to be transmitted 
        sprintf ((char*) wrtbuf, "HW_control ref external ");
        break;
      default:
        return VI_ERROR_PARAMETER2;
    }
    
    // This writes command to instrument 
    CHECKERR (ifr2975_setViString (instrumentHandle, wrtbuf, 0));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Get_Avg_Count_AFC    
* Purpose:  This function returns the current number of samples being used  
*           to average the AF Counter reading.                              
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_Avg_Count_AFC (ViSession instrumentHandle,
                                      ViInt16 *averagingCount)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViInt16 (instrumentHandle, "AFCounter AVG ", averagingCount));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Get_Trig_Mode_AFC    
* Purpose:  This function returns the current trigger mode being used       
*           by the AF Counter.           
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_Trig_Mode_AFC (ViSession instrumentHandle,
                                       ViInt16 *triggerMode)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViInt16 (instrumentHandle, "AFCounter TRIG ", triggerMode));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Get_Pk_Hold_State_AFC
* Purpose:  This function returns the peak hold mode being used             
*           by the AF Counter.           
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_Pk_Hold_State_AFC (ViSession instrumentHandle,
                                             ViInt16 *peakHoldMode)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViInt16 (instrumentHandle, "AFCounter PEAK ", peakHoldMode));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Get_Alert_Status_AFC 
* Purpose:  This function returns the alert enable status being used        
*           by the AF Counter.           
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_Alert_Status_AFC (ViSession instrumentHandle,
                                               ViInt16 *alertEnableStatus)
{
    ViStatus ifr2975_status = VI_SUCCESS; 

    // This writes command to instrument 
    CHECKERR (ifr2975_getViInt16 (instrumentHandle, "AFCounter AL ", alertEnableStatus));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Get_UL_AFC           
* Purpose:  This function returns the upper alert limit being used          
*           by the AF Counter.           
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_UL_AFC (ViSession instrumentHandle,
                                     ViReal64 *upperAlertLimit)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViReal64 (instrumentHandle, "AFCounter ulimit ", upperAlertLimit));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Get_LL_AFC           
* Purpose:  This function returns the lower alert limit being used          
*           by the AF Counter.           
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_LL_AFC (ViSession instrumentHandle,
                                     ViReal64 *lowerAlertLimit)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViReal64 (instrumentHandle, "AFCounter llimit ", lowerAlertLimit));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Get_Enable_State_AFC 
* Purpose:  This function returns the current enable mode being used        
*           by the AF Counter.           
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_Enable_State_AFC (ViSession instrumentHandle,
                                              ViInt16 *enableMode)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViInt16 (instrumentHandle, "AFCounter EN ", enableMode));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Get_Run_State_AFC    
* Purpose:  This function returns the current run state being used          
*           by the AF Counter.           
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_Run_State_AFC (ViSession instrumentHandle,
                                            ViInt16 *runState)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViInt16 (instrumentHandle, "AFCounter RUN ", runState));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Get_Gate_State_AFC    
* Purpose:  This function returns the current gate setting being used       
*           by the AF Counter.           
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_Gate_State_AFC (ViSession instrumentHandle,
                                       ViInt16 *gateStatus)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViInt16 (instrumentHandle, "AFCounter GATE ", gateStatus));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Get_Avg_Count_FMDM   
* Purpose:  This function returns the current number of samples being used  
*           to average the FM Deviation meter reading.                      
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_Avg_Count_FMDM (ViSession instrumentHandle,
                                             ViInt16 *averagingCount)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViInt16 (instrumentHandle, "FMDev AVG ", averagingCount));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Get_Avg_Math_State_FMDM                                 
* Purpose:  This function returns the current averaging mode being used     
*           by the FM Deviation meter.   
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_Avg_Math_State_FMDM (ViSession instrumentHandle,
                                            ViInt16 *averagingMode)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViInt16 (instrumentHandle, "FMDev MATH ", averagingMode));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Get_Trig_Mode_FMDM   
* Purpose:  This function returns the current trigger mode being used       
*           by the FM Deviation meter.   
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_Trig_Mode_FMDM (ViSession instrumentHandle,
                                             ViInt16 *triggerMode)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViInt16 (instrumentHandle, "FMDev TRIG ", triggerMode));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Get_Pk_Hold_State_FMDM                                  
* Purpose:  This function returns the current peak hold mode being used     
*           by the FM Deviation meter.   
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_Pk_Hold_State_FMDM (ViSession instrumentHandle,
                                           ViInt16 *peakHoldMode)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViInt16 (instrumentHandle, "FMDev PEAK ", peakHoldMode));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Get_Alert_Status_FMDM
* Purpose:  This function returns the current alert enable status being used
*           by the FM Deviation meter.   
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_Alert_Status_FMDM (ViSession instrumentHandle,
                                                ViInt16 *alertEnableStatus)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViInt16 (instrumentHandle, "FMDev AL ", alertEnableStatus));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Get_UL_FMDM          
* Purpose:  This function returns the current upper alert limit being used  
*           by the FM Deviation meter.   
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_UL_FMDM (ViSession instrumentHandle,
                                      ViReal64 *upperAlertLimit)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViReal64 (instrumentHandle, "FMDev ulimit ", upperAlertLimit));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Get_LL_FMDM          
* Purpose:  This function returns the current lower alert limit being used  
*           by the FM Deviation meter.   
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_LL_FMDM (ViSession instrumentHandle,
                                      ViReal64 *lowerAlertLimit)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViReal64 (instrumentHandle, "FMDev llimit ", lowerAlertLimit));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Get_Enable_State_FMDM
* Purpose:  This function returns the current enable mode being used        
*           by the FM Deviation meter.   
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_Enable_State_FMDM(ViSession instrumentHandle,
                                               ViInt16 *enableMode)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViInt16 (instrumentHandle, "FMDev EN", enableMode));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Get_Run_State_FMDM   
* Purpose:  This function returns the run state being used                  
*           by the FM Deviation meter.   
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_Run_State_FMDM (ViSession instrumentHandle,
                                             ViInt16 *runState)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViInt16 (instrumentHandle, "FMDev RUN", runState));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Get_Filter_State_SINAD                                  
* Purpose:  This function returns the current status of filtering           
*           used by the SINAD meter.     
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_Filter_State_SINAD (ViSession instrumentHandle,
                                                ViInt16 *filteringStatus)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViInt16 (instrumentHandle, "Sinad MATH", filteringStatus));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Get_Avg_SINAD        
* Purpose:  This function returns the current number of samples being used  
*           for averaging by the SINAD meter.                               
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_Avg_SINAD (ViSession instrumentHandle,
                                        ViInt16 *averagingCount)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViInt16 (instrumentHandle, "Sinad FILTer", averagingCount));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Get_Avg_DSP_SINAD    
* Purpose:  This function returns the current number of samples being used  
*           by the DSP for the SINAD meter.                                 
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_Avg_DSP_SINAD (ViSession instrumentHandle,
                                            ViInt16 *averagingCount)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViInt16 (instrumentHandle, "Sinad AVG", averagingCount));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Get_Trig_Mode_SINAD  
* Purpose:  This function returns the current trigger mode being used       
*           by the SINAD meter.          
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_Trig_Mode_SINAD (ViSession instrumentHandle,
                                              ViInt16 *triggerMode)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViInt16 (instrumentHandle, "Sinad TRIG", triggerMode));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Get_Pk_Hold_State_SINAD                                 
* Purpose:  This function returns the current peak hold mode being used     
*           by the SINAD meter.          
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_Pk_Hold_State_SINAD (ViSession instrumentHandle,
                                            ViInt16 *peakHoldMode)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViInt16 (instrumentHandle, "Sinad PEAK ", peakHoldMode));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Get_Alert_Status_SINAD                                  
* Purpose:  This function returns the alert enable status of the            
*           SINAD meter.                 
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_Alert_Status_SINAD (ViSession instrumentHandle,
                                                 ViInt16 *alertEnableStatus)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViInt16 (instrumentHandle, "Sinad AL ", alertEnableStatus));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Get_UL_SINAD         
* Purpose:  This function returns the upper alert limit of the              
*           SINAD meter.                 
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_UL_SINAD (ViSession instrumentHandle,
                                       ViReal64 *upperAlertLimit)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViReal64 (instrumentHandle, "Sinad ulimit ", upperAlertLimit));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Get_LL_SINAD         
* Purpose:  This function returns the lower alert limit of the              
*           SINAD meter.                 
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_LL_SINAD (ViSession instrumentHandle,
                                       ViReal64 *lowerAlertLimit)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViReal64 (instrumentHandle, "Sinad llimit ", lowerAlertLimit));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Get_Enable_State_SINAD                                  
* Purpose:  This function returns the current enabled modes of the          
*           SINAD meter.                 
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_Enable_State_SINAD (ViSession instrumentHandle,
                                                ViInt16 *enableMode)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViInt16 (instrumentHandle, "Sinad EN ", enableMode));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Get_Run_State_SINAD  
* Purpose:  This function returns the current run state of the              
*           SINAD meter.                 
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_Run_State_SINAD (ViSession instrumentHandle,
                                              ViInt16 *runState)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViInt16 (instrumentHandle, "Sinad RUN", runState));
    
    return ifr2975_status;
}



/******************************************************************************
* Function: ifr2975_Get_Filter_State_DIST
* Purpose:  This function returns the current status of filtering used by   
*           the distortion meter.        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_Filter_State_DIST (ViSession instrumentHandle,
                                               ViInt16 *filteringStatus)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViInt16 (instrumentHandle, "Dist MATH ", filteringStatus));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Get_Avg_DIST         
* Purpose:  This function returns the current number of samples used for    
*           averaging by the distortion meter.                              
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_Avg_DIST (ViSession instrumentHandle,
                                       ViInt16 *averagingCount)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViInt16 (instrumentHandle, "Dist FILTer", averagingCount));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Get_Avg_DSP_DIST     
* Purpose:  This function returns the current number of samples used for    
*           averaging by the DSP for the distortion meter.                  
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_Avg_DSP_DIST (ViSession instrumentHandle,
                                           ViInt16 *averagingCount)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViInt16 (instrumentHandle, "Dist AVG ", averagingCount));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Get_Trig_Mode_DIST   
* Purpose:  This function returns the current trigger mode used by the     
*           distortion meter.            
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_Trig_Mode_DIST (ViSession instrumentHandle,
                                             ViInt16 *triggerMode)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViInt16 (instrumentHandle, "Dist trigger ", triggerMode));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Get_Pk_Hold_State_DIST                                  
* Purpose:  This function returns the current peak hold mode used by the    
*           distortion meter.            
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_Pk_Hold_State_DIST (ViSession instrumentHandle,
                                                ViInt16 *peakHoldMode)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViInt16 (instrumentHandle, "Dist PEAK ", peakHoldMode));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Get_Alert_Status_DIST
* Purpose:  This function returns the current alert enable status used by  
*           the distortion meter.        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_Alert_Status_DIST (ViSession instrumentHandle,
                                                ViInt16 *alertEnableStatus)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViInt16 (instrumentHandle, "Dist AL ", alertEnableStatus));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Get_UL_DIST          
* Purpose:  This function returns the upper alert limit used by             
*           the distortion meter.        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_UL_DIST (ViSession instrumentHandle,
                                      ViReal64 *upperAlertLimit)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViReal64 (instrumentHandle, "Dist ulimit ", upperAlertLimit));
    
    return ifr2975_status;
}

/******************************************************************************
* Function: ifr2975_Get_LL_DIST          
* Purpose:  This function returns the lower alert limit used by             
*           the distortion meter.        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_LL_DIST (ViSession instrumentHandle,
                                      ViReal64 *lowerAlertLimit)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViReal64 (instrumentHandle, "Dist llimit ", lowerAlertLimit));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Get_Enable_State_DIST
* Purpose:  This function returns the current enable mode of                
*           the distortion meter.        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_Enable_State_DIST (ViSession instrumentHandle,
                                               ViInt16 *enableMode)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViInt16 (instrumentHandle, "Dist EN", enableMode));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Get_Run_State_DIST   
* Purpose:  This function returns the run state of                          
*           the distortion meter.        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_Run_State_DIST (ViSession instrumentHandle,
                                             ViInt16 *runState)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViInt16 (instrumentHandle, "Dist RUN", runState));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Get_Freq_FGEN        
* Purpose:  This function returns the selected function generator frequency 
*           setting.                     
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_Freq_FGEN (ViSession instrumentHandle,
                                        ViInt16 source, ViReal64 *frequency)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    ViChar rdbuf[BUFFER_SIZE];
    
    // Verify source range
    // 1=FGEN1
    // 2=FGEN2
    // 3=M1
    // 4=M2
    if (ifr2975_invalidViInt16Range (source, 1, 4))
    {
      return VI_ERROR_PARAMETER2;
    }
    
    // Formats string to be transmitted 
    sprintf ((char*) wrtbuf, "FGen FREQuency%i ",source);
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViReal64 (instrumentHandle, wrtbuf, frequency));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Get_Lev_FGEN         
* Purpose:  This function returns the selected function generator level    
*           setting.                     
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_Lev_FGEN (ViSession instrumentHandle,
                                       ViInt16 source, ViReal64 *level)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    ViInt16 verticalScale;

    // Verify source range
    // 1=FGEN1
    // 2=FGEN2
    if (ifr2975_invalidViInt16Range (source, 1, 2))
    {
      return VI_ERROR_PARAMETER2;
    }
    
    // Formats string to be transmitted 
    sprintf ((char*) wrtbuf, "FGen LEVel%i ",source);
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViReal64 (instrumentHandle, wrtbuf, level));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Get_Dev_FGEN         
* Purpose:  This function returns the selected function generator deviation 
*           setting.                     
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_Dev_FGEN (ViSession instrumentHandle,
                                       ViInt16 source, ViReal64 *deviation)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    ViChar rdbuf[BUFFER_SIZE];
    ViReal64 temp1;
    ViReal64 temp2;

    // Verify source range
    // 3=M1
    // 4=M2
    if (ifr2975_invalidViInt16Range (source, 3, 4))
    {
      return VI_ERROR_PARAMETER2;
    }
    
    // Formats string to be transmitted 
    sprintf ((char*) wrtbuf, "FGen DEViation%i ",source);

    // This writes command to instrument 
    CHECKERR (ifr2975_getViString (instrumentHandle, wrtbuf, rdbuf, 100));
    
    // Gets the parameter that is needed
    sscanf(rdbuf, "%lf %lf %lf", &temp1, &temp2, deviation);
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Get_Per_FGEN         
* Purpose:  This function returns the selected function generator deviation 
*           setting.                     
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_Per_FGEN (ViSession instrumentHandle,
                                       ViInt16 source, ViReal64 *percent)
{
    ViStatus     ifr2975_status = VI_SUCCESS; 
    ViChar         wrtbuf[BUFFER_SIZE];
    ViChar         rdbuf[BUFFER_SIZE];
    ViReal64    temp1;
    ViReal64    temp2;

    // Verify source range
    // 3=M1
    // 4=M2
    if (ifr2975_invalidViInt16Range (source, 3, 4))
    {
      return VI_ERROR_PARAMETER2;
    }
    
    // Formats string to be transmitted 
    sprintf ((char*) wrtbuf, "FGen per%i ",source);
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViString (instrumentHandle, wrtbuf, rdbuf, 100));
    
    // Gets the parameter that is needed
    sscanf(rdbuf, "%lf %lf %lf", &temp1, percent, &temp2);
    
    return ifr2975_status;
}



/******************************************************************************
* Function: ifr2975_Get_Shape_FGEN       
* Purpose:  This function returns the selected function generator audio     
*           wave shape setting.          
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_Shape_FGEN (ViSession instrumentHandle,
                                         ViInt16 source, ViInt16 *shape)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    
    // Verify source range
    // 1=FGEN1
    // 2=FGEN2
    // 3=M1
    // 4=M2
    if (ifr2975_invalidViInt16Range (source, 1, 4))
    {
      return VI_ERROR_PARAMETER2;
    }
    
    // Formats string to be transmitted 
    sprintf ((char*) wrtbuf, "FGen SHape%i ",source);
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViInt16 (instrumentHandle, wrtbuf, shape));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Get_Dur_FGEN         
* Purpose:  This function returns the selected function generator current   
*           tone duration.               
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_Dur_FGEN (ViSession instrumentHandle,
                                       ViInt16 source, ViReal64 *duration)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    
    // Verify source range
    // 1=FGEN1
    // 2=FGEN2
    // 3=M1
    // 4=M2
    if (ifr2975_invalidViInt16Range (source, 1, 4))
    {
      return VI_ERROR_PARAMETER2;
    }
    
    // Formats string to be transmitted 
    sprintf ((char*) wrtbuf, "FGen DURation%i ",source);
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViReal64 (instrumentHandle, wrtbuf, duration));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Get_Mode_FGEN        
* Purpose:  This function returns the selected function generators' mode    
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_Mode_FGEN (ViSession instrumentHandle,
                                        ViInt16 source, ViInt16 *mode)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    
    // Verify source range
    // 1=FGEN1
    // 2=FGEN2
    // 3=M1
    // 4=M2
    if (ifr2975_invalidViInt16Range (source, 1, 4))
    {
      return VI_ERROR_PARAMETER2;
    }
    
    // Formats string to be transmitted 
    sprintf ((char*) wrtbuf, "Fgen mode%i ",source);
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViInt16 (instrumentHandle, wrtbuf, mode));
    
    return ifr2975_status;
}



/******************************************************************************
* Function: ifr2975_Get_Demod_Filt_BBA   
* Purpose:  This function returns the demod filter setting.                
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_Demod_Filt_BBA (ViSession instrumentHandle,
                                             ViInt16 *filter)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViInt16 (instrumentHandle, "BBAudio DEMODFILter", filter));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Get_FGen_Filt_BBA    
* Purpose:  This function returns the fgen filter setting.                 
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_FGen_Filt_BBA (ViSession instrumentHandle,
                                            ViInt16 *filter)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];

    // Formats string to be transmitted 
    sprintf ((char*) wrtbuf, "BBAudio FGENFILter ");
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViInt16 (instrumentHandle, "BBAudio FGENFILter", filter));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Get_FGen_Filt_BBA    
* Purpose:  This function returns the fgen balance setting.                 
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_FGen_Bal_BBA (ViSession instrumentHandle,
                                           ViInt16 *balance)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
        
    // This writes command to instrument 
    CHECKERR (ifr2975_getViInt16 (instrumentHandle, "BBAudio FGENBALance", balance));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Get_FGen_Level_BBA   
* Purpose:  This function returns the fgen level.                           
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_FGen_Level_BBA (ViSession instrumentHandle,
                                             ViInt16 *level)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViInt16 (instrumentHandle, "BBAudio FGENLEVel", level));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Get_SINAD_Bal_BBA    
* Purpose:  This function returns the SINAD balance setting.               
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_SINAD_Bal_BBA (ViSession instrumentHandle,
                                            ViInt16 *balance)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViInt16 (instrumentHandle, "BBAudio SINADBALance", balance));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Get_Mic_Bias_BBA     
* Purpose:  This function returns the MIC bias setting.                     
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_Mic_Bias_BBA (ViSession instrumentHandle,
                                           ViInt16 *bias)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViInt16 (instrumentHandle, "BBAudio micbias", bias));
    
    return ifr2975_status;
}

/******************************************************************************
* Function: ifr2975_Get_Avg_RFERR        
* Purpose:  This function returns the RF Error Meter averaging sample count.
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_Avg_RFERR (ViSession instrumentHandle,
                                        ViInt16 *averagingCount)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViInt16 (instrumentHandle, "RFError AVG", averagingCount));
    
    return ifr2975_status;
}

/******************************************************************************
* Function: ifr2975_Get_Alerts_RFERR
* Purpose:  This function returns the RF Error Meter alert status.
*                                                                           
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_Alerts_RFERR (ViSession instrumentHandle,
                                                 ViInt16 *alertEnableStatus)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViInt16 (instrumentHandle, "RFError alerts", alertEnableStatus));
    
    return ifr2975_status;
}

/******************************************************************************
* Function: ifr2975_Get_UL_RFERR         
* Purpose:  This function returns the Upper Alert Limit of the RF Error    
*           Meter.                       
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_UL_RFERR (ViSession instrumentHandle,
                                       ViReal64 *upperAlertLimit)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViReal64 (instrumentHandle, "RFError ulimit ", upperAlertLimit));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Get_LL_RFERR         
* Purpose:  This function returns the Lower Alert Limit of the RF Error    
*           Meter.                       
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_LL_RFERR (ViSession instrumentHandle,
                                       ViReal64 *lowerAlertLimit)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViReal64 (instrumentHandle, "RFError llimit ", lowerAlertLimit));
    
    return ifr2975_status;
}

/******************************************************************************
* Function: ifr2975_Get_Avg_RFERR        
* Purpose:  This function returns the RF Error Meter enabled modes.         
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_Enable_State_RFERR (ViSession instrumentHandle,
                                                ViInt16 *enableMode)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViInt16 (instrumentHandle, "RFError EN", enableMode));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Get_Run_State_RFERR  
* Purpose:  This function returns the RF Error Meter run state.            
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_Run_State_RFERR (ViSession instrumentHandle,
                                              ViInt16 *runState)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViInt16 (instrumentHandle, "RFError RUN", runState));
    
    return ifr2975_status;
}


/******************************************************************************
			_tmp47 = ifr2975_Get_Run_State_DVM (_tmp1, &_tmp48);
* Function: ifr2975_Get_Gate_State_RFERR 
* Purpose:  This function returns the RF Error Meter gate state.           
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_Gate_State_RFERR (ViSession instrumentHandle,
                                         ViInt16 *gateStatus)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViInt16 (instrumentHandle, "RFError GATE ", gateStatus));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Get_Avg_DVM          
* Purpose:  This function returns the number of samples being used to      
*           average the DVM reading      
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_Avg_DVM (ViSession instrumentHandle,
                                      ViInt16 *averagingCount)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViInt16 (instrumentHandle, "Dvm AVG ", averagingCount));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Get_Pk_Hold_State_DVM
* Purpose:  This function returns the current peak hold mode of the DVM.    
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_Pk_Hold_State_DVM (ViSession instrumentHandle,
                                               ViInt16 *peakHoldMode)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViInt16 (instrumentHandle, "Dvm peakhold ", peakHoldMode));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Get_Alert_Status_DVM 
* Purpose:  This function returns the current alert enable status of the DVM
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_Alert_Status_DVM (ViSession instrumentHandle,
                                               ViInt16 *alertEnableStatus)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViInt16 (instrumentHandle, "Dvm AL ", alertEnableStatus));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Get_UL_DVM           
* Purpose:  This function returns the upper alert limit of the DVM.         
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_UL_DVM (ViSession instrumentHandle,
                                     ViReal64 *upperAlertLimit)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViReal64 (instrumentHandle, "Dvm ulimit ", upperAlertLimit));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Get_LL_DVM           
* Purpose:  This function returns the lower alert limit of the DVM.         
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_LL_DVM (ViSession instrumentHandle,
                                     ViReal64 *lowerAlertLimit)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViReal64 (instrumentHandle, "Dvm llimit ", lowerAlertLimit));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Get_Enable_DVM
* Purpose:  This function returns the current enable mode of the DVM.
*
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_Enable_DVM (ViSession instrumentHandle,
                                              ViInt16 *enableMode)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViInt16 (instrumentHandle, "Dvm EN ", enableMode));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Get_Run_State_DVM
* Purpose:  This function returns the current run state of the DVM.
*
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_Run_State_DVM (ViSession instrumentHandle,
                                            ViInt16 *runState)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViInt16 (instrumentHandle, "Dvm RUN ", runState));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Get_Math_Mode_DVM    
* Purpose:  This function returns the current math mode of the DVM.         
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_Math_Mode_DVM (ViSession instrumentHandle,
                                            ViInt16 *mathMode)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViInt16 (instrumentHandle, "Dvm MATH ", mathMode));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Get_Imp_DVM          
* Purpose:  This function returns the current impedance setting of the DVM. 
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_Imp_DVM (ViSession instrumentHandle,
                                      ViInt16 *impedance)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViInt16 (instrumentHandle, "Dvm impedance ", impedance));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Get_Imp_DVM          
* Purpose:  This function returns the current hold state of the DVM.       
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_Hold_DVM (ViSession instrumentHandle,
                                       ViInt16 *holdState)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViInt16 (instrumentHandle, "Dvm HOLD ", holdState));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Get_Line_Filter_DVM  
* Purpose:  This function returns the current line filter setting of the DVM
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_Line_Filter_DVM (ViSession instrumentHandle,
                                              ViInt16 *lineFilterSetting)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViInt16 (instrumentHandle, "Dvm LINEFIL", lineFilterSetting));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Get_Range_DVM        
* Purpose:  This function returns the current hardware range of the DVM.   
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_Range_DVM (ViSession instrumentHandle,
                                        ViInt16 *range)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViInt16 (instrumentHandle, "Dvm HWRAN ", range));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Get_Source_DVM       
* Purpose:  This function returns the current input source of the DVM.     
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_Source_DVM (ViSession instrumentHandle,
                                         ViInt16 *source)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViInt16 (instrumentHandle, "Dvm SOUR", source));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Get_Enable_State_DVM         
* Purpose:  This function gets the mode on the DVM                         
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_Enable_State_DVM (ViSession instrumentHandle,
                                               ViInt16 *inputSource)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViInt16 (instrumentHandle, "Dvm mode ", inputSource));
                                                                    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Get_Avg_C4FM         
* Purpose:  This function returns the current number of symbols being used  
*           to average the reading in the C4FM meter.                       
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_Avg_C4FM (ViSession instrumentHandle,
                                       ViInt32 *averagingCount)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViInt32 (instrumentHandle, "C4fm AVG", averagingCount));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Get_Avg_C4FM         
* Purpose:  This function returns the current trigger mode of the C4FM meter
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_Trig_Mode_C4FM (ViSession instrumentHandle,
                                             ViInt16 *triggerMode)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViInt16 (instrumentHandle, "C4fm trigger ", triggerMode));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Get_Pk_Hold_State_C4FM                                  
* Purpose:  This function returns the current peak hold mode of the C4FM    
*           meter.                       
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_Pk_Hold_State_C4FM (ViSession instrumentHandle,
                                                ViInt16 *peakHoldMode)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViInt16 (instrumentHandle, "C4fm peakhold", peakHoldMode));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Get_Alert_Status_C4FM
* Purpose:  This function returns the current alert enable status of the    
*           C4FM meter.                  
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_Alert_Status_C4FM (ViSession instrumentHandle,
                                                ViInt16 *alertEnableStatus)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViInt16 (instrumentHandle, "C4fm alerts ", alertEnableStatus));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Get_Pk_Hold_State_RFERR                                 
* Purpose:  This function returns the peak hold mode of the RF Error Meter. 
*           C4FM meter.                  
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_Pk_Hold_State_RFERR (ViSession instrumentHandle,
                                                 ViInt16 *peakHoldMode)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViInt16 (instrumentHandle, "RFError PEAK  ", peakHoldMode));
    
    return ifr2975_status;
}

/******************************************************************************
* Function: ifr2975_Get_UL_C4FM          
* Purpose:  This function returns the current upper alert limit of the      
*           C4FM meter.                  
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_UL_C4FM (ViSession instrumentHandle,
                                      ViReal64 *upperAlertLimit)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViReal64 (instrumentHandle, "C4fm ulimit ", upperAlertLimit));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Get_LL_C4FM          
* Purpose:  This function returns the current lower alert limit of the      
*           C4FM meter.                  
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_LL_C4FM (ViSession instrumentHandle,
                                      ViReal64 *lowerAlertLimit)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViReal64 (instrumentHandle, "C4fm llimit ", lowerAlertLimit));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Get_Enable_State_C4FM
* Purpose:  This function returns the current enable mode of the            
*           C4FM meter.                  
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_Enable_State_C4FM (ViSession instrumentHandle,
                                               ViInt16 *enableMode)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViInt16 (instrumentHandle, "C4fm enable ", enableMode));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Get_Run_State_OSCOPE 
* Purpose:  This function returns the current run state of the             
*           Scope.
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_Run_State_OSCOPE (ViSession instrumentHandle,
                                               ViInt16 scope, ViInt16 *runState)
{
    ViStatus     ifr2975_status = VI_SUCCESS; 
    ViChar        wrtbuf[100];

    if (ifr2975_invalidViInt16Range (scope, 1, 2))
    {
      return VI_ERROR_PARAMETER3;
    }
    
    // Formats string to be transmitted 
    sprintf ((char*) wrtbuf, "Scope RUN%i",scope);
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViInt16 (instrumentHandle, wrtbuf, runState));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Get_Run_State_C4FM                              
* Purpose:  This function returns the current run state of the              
*           C4FM Meter.                  
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_Run_State_C4FM (ViSession instrumentHandle,
                                                     ViInt16 *runState)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViInt16 (instrumentHandle, "C4fm RUN ", runState));
    
    return ifr2975_status;
}

/******************************************************************************
* Function: ifr2975_Get_Sweep_OSCOPE     
* Purpose:  This function returns the horizontal sweep rate of the          
*           oscilloscope.                
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_Sweep_OSCOPE (ViSession instrumentHandle,
                                           ViReal64 *sweepSpeed)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViReal64 (instrumentHandle, "Scope horizontalsweep ", sweepSpeed));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Get_Vert_Scale_OSCOPE
* Purpose:  This function returns the vertical scale of the                 
*           oscilloscope.                
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_Vert_Scale_OSCOPE (ViSession instrumentHandle,
                                                ViInt16 channel,
                                                ViInt16 *verticalScale)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    ViChar rdbuf[BUFFER_SIZE];
    
    // Verify channel range
    // 1=CH1
    // 2=CH2
    if (ifr2975_invalidViInt16Range (channel, 1, 2))
    {
      return VI_ERROR_PARAMETER2;
    }
    
    // Formats string to be transmitted 
    sprintf ((char*) wrtbuf, "Scope scale%i ",channel);
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViInt16 (instrumentHandle, wrtbuf, verticalScale));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Get_Horiz_Pos_OSCOPE 
* Purpose:  This function returns the horizontal position offset of the     
*           oscilloscope.                
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_Horiz_Pos_OSCOPE (ViSession instrumentHandle,
                                               ViReal64 *horizontalPosition)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViReal64 (instrumentHandle, "Scope horizontalposition ", horizontalPosition));
    
    *horizontalPosition = *horizontalPosition/10;
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Get_Trig_Type_OSCOPE 
* Purpose:  This function returns the current trigger types of the          
*           oscilloscope.                
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_Trig_Type_OSCOPE (ViSession instrumentHandle,
                                               ViInt16 *triggerType)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViInt16 (instrumentHandle, "Scope trigtype ", triggerType));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Get_Trig_Source_OSCOPE                                  
* Purpose:  This function returns the current trigger source of the         
*           oscilloscope.                
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_Trig_Source_OSCOPE (ViSession instrumentHandle,
                                                 ViInt16 *triggerSource)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViInt16 (instrumentHandle, "Scope trigsource ", triggerSource));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Get_Trig_Lev_OSCOPE  
* Purpose:  This function returns the current trigger level of the          
*           oscilloscope.                
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_Trig_Lev_OSCOPE (ViSession instrumentHandle,
                                              ViReal64 *triggerLevel)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViReal64 (instrumentHandle, "Scope triglev ", triggerLevel));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Get_Trig_Pol_OSCOPE  
* Purpose:  This function returns the current trigger polarity of the       
*           oscilloscope.                
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_Trig_Pol_OSCOPE (ViSession instrumentHandle,
                                              ViInt16 *triggerPolarity)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViInt16 (instrumentHandle, "Scope trigpol ", triggerPolarity));
    
    return ifr2975_status;
}


/******************************************************************************
			_tmp97 = ifr2975_Get_Avg_OSCOPE (_tmp93, &_tmp98);
* Function: ifr2975_Get_Pk_Hold_Reset_Flags_OSCOPE                         
* Purpose:  This function returns the peak hold flags of the               
*           oscilloscope.                
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_Pk_Hold_Reset_Flags_OSCOPE (ViSession instrumentHandle,
                                                         ViInt16 *flagBits)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViInt16 (instrumentHandle, "Scope peakreset ", flagBits));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Get_Avg_OSCOPE       
* Purpose:  This function returns current average setting of the            
*           oscilloscope.                
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_Avg_OSCOPE (ViSession instrumentHandle,
                                         ViInt16 *averagingCount)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViInt16 (instrumentHandle, "Scope avg ", averagingCount));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Get_Input_Coupling_OSCOPE                               
* Purpose:  This function returns current input coupling setting of the     
*           oscilloscope.                
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_Input_Coupling_OSCOPE (ViSession instrumentHandle,
                                                    ViInt16 Channel,ViInt16 *inputCoupling)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    
    // Formats string to be transmitted 
    sprintf ((char*) wrtbuf, "Scope coupling%i ", Channel);
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViInt16 (instrumentHandle, wrtbuf, inputCoupling));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Get_Trace_Math_Mode_OSCOPE                              
* Purpose:  This function returns current math mode of the                  
*           oscilloscope.                
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_Trace_Math_Mode_OSCOPE (ViSession instrumentHandle,
                                                     ViInt16 *mathMode)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViInt16 (instrumentHandle, "Scope math ", mathMode));
    
    return ifr2975_status;
}

/******************************************************************************
* Function: ifr2975_Get_Trace_OSCOPE     
* Purpose:  This function returns the current execution state of the        
*           oscilloscope.                
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_Trace_OSCOPE (ViSession instrumentHandle, ViInt16 control,
                        ViInt32 *length_array, ViReal64 *TEMP_DATA)
{
    // Variables
    ViStatus ifr2975_status = VI_SUCCESS;
    ViByte rdbuf[10000];
    ViByte wrtbuf[10000];
    ViByte TEMP_BUF[10000];
    
    ViInt16 i;
    char *End_Pointer;
    ViInt32 verticalScale;
    
    if (ifr2975_invalidViInt16Range (control, 1, 3))
    {
      return VI_ERROR_PARAMETER3;
    }

    // Formats string to be transmitted 
    sprintf ((char*) wrtbuf, "Scope trace%i", control);

    // This writes command to instrument 
    CHECKERR (ifr2975_getViString (instrumentHandle, (ViString) wrtbuf, (ViString) rdbuf, 10000));
    
    sscanf((const char *)rdbuf, "%d", length_array);
    
    strtod ((const char *) rdbuf, &End_Pointer);
    
    // Process Oscilloscope Trace Data
    if((*length_array==50)||(*length_array==100)||(*length_array==250)||(*length_array==500)||(*length_array==1000))
    {
      for (i = 0; i <=(*length_array-1); i++) 
      {
        sscanf(End_Pointer, "%lf", &TEMP_DATA[i]);
        strtod (End_Pointer, &End_Pointer);
      };
    } 
    else 
    {
      *length_array = 0;
    }

    // Formats string to be transmitted 
    sprintf ((char*) wrtbuf, "Scope scale%i",control);

    // This writes command to instrument 
    CHECKERR (ifr2975_getViInt32 (instrumentHandle, (ViString) wrtbuf, &verticalScale));
       
    // 2 .02 V/div
    // 3 .05 V/div
    // 4 .1 V/div
    // 5 .2 V/div
    // 6 .5 V/div
    // 7 1 V/div
    // 8 2 V/div
    // 9 5 V/div
    // 10 10 V/div
    // 11 20 V/div
    // 12 50 V/div
    switch (verticalScale) 
    {
      case 2:
        for (i = 0; i <=(*length_array-1); i++) 
        {
          // .02 // 10/4096
          TEMP_DATA[i] = TEMP_DATA[i];
        }
        break;
      case 3:
        for (i = 0; i <=(*length_array-1); i++) 
        {
          // .05 // 10/4096
          TEMP_DATA[i] = TEMP_DATA[i];
        }
        break;
      case 4:
        for (i = 0; i <=(*length_array-1); i++) 
        {
          // .1 // 10/4096
          TEMP_DATA[i] = TEMP_DATA[i];
        }
        break;
      case 5:
        for (i = 0; i <=(*length_array-1); i++) 
        {
          // .2 // 10/4096
          TEMP_DATA[i] = TEMP_DATA[i];
        }
        break;
      case 6:
        for (i = 0; i <=(*length_array-1); i++) 
        {
          // .5 // 10/4096
          TEMP_DATA[i] = TEMP_DATA[i];
        }
        break;
      case 7:
        for (i = 0; i <=(*length_array-1); i++) 
        {
          // 1 // 10/4096
          TEMP_DATA[i] = TEMP_DATA[i];
        }
        break;
      case 8:
        for (i = 0; i <=(*length_array-1); i++) 
        {
          // 2 // 10/4096
          TEMP_DATA[i] = TEMP_DATA[i];
        }
        break;
      case 9:
        for (i = 0; i <=(*length_array-1); i++) 
        {
          // 5 // 10/4096
          TEMP_DATA[i] = TEMP_DATA[i];
        }
        break;
      case 10:
        for (i = 0; i <=(*length_array-1); i++) 
        {
          // 10 // 10/4096
          TEMP_DATA[i] = TEMP_DATA[i];
        }
        break;
      case 11:
        for (i = 0; i <=(*length_array-1); i++) 
        {
          // 20 // 10/4096
          TEMP_DATA[i] = TEMP_DATA[i];
        }
        break;
      case 12:
        for (i = 0; i <=(*length_array-1); i++) 
        {
          // 50 // 10/4096
          TEMP_DATA[i] = TEMP_DATA[i];
        }
        break;
      default:
        *length_array = 0;
        break;
    }
        
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Get_Freq_REC         
* Purpose:  This function returns the receiver frequency.                   
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_Freq_REC (ViSession instrumentHandle,
                                       ViReal64 *frequency)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViReal64 (instrumentHandle, "Receiver FREQ ", frequency));
    
    return ifr2975_status;
}

/******************************************************************************
* Function: ifr2975_Get_RFIn_REC         
* Purpose:  This function returns the receiver input port.                  
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_RFIn_REC (ViSession instrumentHandle,
                                       ViInt16 *RFInput)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViInt16 (instrumentHandle, "Receiver RFIN ", RFInput));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Get_Attn_REC         
* Purpose:  This function returns the receiver attenuation setting.         
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_Attn_REC (ViSession instrumentHandle,
                                       ViInt16 *attenuation)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViInt16 (instrumentHandle, "Receiver ATTEN ", attenuation));
    
    return ifr2975_status;
}

/******************************************************************************
* Function: ifr2975_Get_Filter_REC       
* Purpose:  This function returns the receiver filter.                      
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_Filter_REC (ViSession instrumentHandle,
                                         ViInt16 *filter)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViInt16 (instrumentHandle, "Receiver filter ", filter));
    
    return ifr2975_status;
}

/******************************************************************************
* Function: ifr2975_Get_Demod_REC        
* Purpose:  This function returns the receiver demodulation setting.        
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_Demod_REC (ViSession instrumentHandle,
                                        ViInt16 *demodulation)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    ViChar rdbuf[BUFFER_SIZE];
    
    // Formats string to be transmitted 
    sprintf ((char*) wrtbuf, "Receiver DEMOD ");
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViInt16 (instrumentHandle, "Receiver DEMOD ", demodulation));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Get_Range_REC        
* Purpose:  This function returns the receiver range setting.              
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_Range_REC (ViSession instrumentHandle,
                                        ViInt16 *range)
{
    ViStatus ifr2975_status = VI_SUCCESS; 

    // This writes command to instrument 
    CHECKERR (ifr2975_getViInt16 (instrumentHandle, "Receiver RANGE ", range));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Get_IFBW_REC         
* Purpose:  This function returns the receiver IF bandwidth setting.        
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_IFBW_REC (ViSession instrumentHandle,
                                       ViReal64 *IFBandwidth)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViReal64 (instrumentHandle, "Receiver ifbw ", IFBandwidth));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Get_IFGain_REC       
* Purpose:  This function returns the receiver IF gain setting.             
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_IFGain_REC (ViSession instrumentHandle,
                                         ViReal64 *IFGain)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViReal64 (instrumentHandle, "Receiver IFGAIN ", IFGain));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Get_HWAGC_REC        
* Purpose:  This function returns the receiver hardware AGC setting.        
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_HWAGC_REC (ViSession instrumentHandle,
                                        ViInt16 *HWAGCSpeed)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    ViChar rdbuf[BUFFER_SIZE];
    
    // Formats string to be transmitted 
    sprintf ((char*) wrtbuf, "Receiver HWAGC ");
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViInt16 (instrumentHandle, "Receiver HWAGC ", HWAGCSpeed));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Get_SWAGC_State_REC  
* Purpose:  This function returns the receiver software AGC mode.           
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_SWAGC_State_REC (ViSession instrumentHandle,
                                        ViInt16 *softwareAGCMode)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViInt16 (instrumentHandle, "Receiver AGCMODE ", softwareAGCMode));
    
    return ifr2975_status;
}



/******************************************************************************
* Function: ifr2975_Get_Sweep_Lock_State_REC                                
* Purpose:  This function returns the receiver frequency lock setting.      
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_Sweep_Lock_State_REC (ViSession instrumentHandle,
                                             ViInt16 *setting)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViInt16 (instrumentHandle, "Receiver LOCK ", setting));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Get_Freq_GEN         
* Purpose:  This function returns the Generator frequency.                 
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_Freq_GEN (ViSession instrumentHandle,
                                       ViReal64 *frequency)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViReal64 (instrumentHandle, "Generator FREQ ", frequency));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Get_Level_GEN         
* Purpose:  This function returns the Generator level.                     
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_Level_GEN (ViSession instrumentHandle,
                                        ViReal64 *level)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViReal64 (instrumentHandle, "Generator RFLEV ", level));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Get_RFOut_GEN        
* Purpose:  This function returns the Generator output port.                
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_RFOut_GEN (ViSession instrumentHandle,
                                              ViInt16 *outputPort)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViInt16 (instrumentHandle, "Generator RFOUT ", outputPort));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Get_HWALC_GEN        
* Purpose:  This function returns the Generator hardware ALC setting.      
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_HWALC_GEN (ViSession instrumentHandle,
                                        ViInt16 *ALCSetting)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViInt16 (instrumentHandle, "Generator ALC ", ALCSetting));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Get_BW_GEN           
* Purpose:  This function returns the Generator bandwidth setting.         
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_BW_GEN (ViSession instrumentHandle,
                                     ViInt16 *bandwidth)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViInt16 (instrumentHandle, "Generator BW ", bandwidth));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Get_Gen_State_GEN    
* Purpose:  This function returns the Generator on/off setting.            
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_Gen_State_GEN (ViSession instrumentHandle,
                                         ViInt16 *onOff)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViInt16 (instrumentHandle, "Generator dcpower ", onOff));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Get_Yig_Speed_GEN    
* Purpose:  This function returns the Generator YIG speed.                 
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_Yig_Speed_GEN (ViSession instrumentHandle,
                                            ViInt16 *yigSpeed)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViInt16 (instrumentHandle, "Generator YIGSPEED ", yigSpeed));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Get_Mod_GEN          
* Purpose:  This function returns the Generator modulation mode.            
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_Mod_GEN (ViSession instrumentHandle,
                                      ViInt16 *modulationMode)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViInt16 (instrumentHandle, "Generator MOD ", modulationMode));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Get_P25_GEN          
* Purpose:  This function returns the Generator P25 mode.                  
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_P25_GEN (ViSession instrumentHandle,
                                      ViInt16 *p25Mode)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViInt16 (instrumentHandle, "Generator P25M ", p25Mode));
    
    return ifr2975_status;
}



/******************************************************************************
* Function: ifr2975_Get_PTT_State_GEN    
* Purpose:  This function returns the Generator PTT state.                 
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_PTT_State_GEN (ViSession instrumentHandle,
                                      ViInt16 *PTTState)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViInt16 (instrumentHandle, "Generator ptt ", PTTState));
    
    return ifr2975_status;
}

/******************************************************************************
* Function: ifr2975_Get_Decode_State_APCO
* Purpose:  This function returns P25 decode status.                       
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_Decode_State_APCO (ViSession instrumentHandle,
                                                 ViInt16 *decodeEnableStatus)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViInt16 (instrumentHandle, "P25 decode EN ", decodeEnableStatus));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Get_Decode_State_APCO
* Purpose:  This function returns P25 decode sample rate.                   
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_Dec_Sample_Rate_APCO (ViSession instrumentHandle,
                                                   ViInt16 *decodeSampleRate)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViInt16 (instrumentHandle, "P25 decode RATE ", decodeSampleRate));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Get_Encrypt_State_APCO                                  
* Purpose:  This function returns encryption status.                        
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_Encrypt_State_APCO (ViSession instrumentHandle,
                                                     ViChar encryptionStatus[],
                                                      ViChar ALGID[])
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    ViChar rdbuf[BUFFER_SIZE];

    ViChar charEncryptionStatus[BUFFER_SIZE];
    ViChar charALGID[BUFFER_SIZE];
    
    // Formats string to be transmitted 
    sprintf ((char*) wrtbuf, "P25 encode ENCRYPT ");

    // This writes command to instrument 
    CHECKERR (ifr2975_getViString (instrumentHandle, wrtbuf, rdbuf, 100));
    
    // Get the parameter(s) that is needed
    sscanf(rdbuf, "%s %s", charEncryptionStatus, charALGID);
    
    // Compare the Encryption Status to the known values
    // Display "Unknown" if there is no match
    if( strncmp("128",charEncryptionStatus,3) == 0 )
    {
      sprintf ((char*) encryptionStatus, "Clear");
    }
    else if( strncmp("129",charEncryptionStatus,3) == 0 )
    {
      sprintf ((char*) encryptionStatus, "DES");
    }    
    else if( strncmp("132",charEncryptionStatus,3) == 0 )
    {
      sprintf ((char*) encryptionStatus, "AES");
    }    
    else
    {
      sprintf ((char*) encryptionStatus, "Unknown");
    }
    
	// Determine if the ALGID value returned is a number
	// If it is, return that number
	if ( isdigit(charALGID[0]) != 0 )
	{
	  sscanf(charALGID, "%s", ALGID); 
	}
	
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Get_C4FM_Mod_APCO    
* Purpose:  This function returns C4FM Modulation percentage.              
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_C4FM_Mod_APCO (ViSession instrumentHandle,
                                                   ViReal64 *C4FMModulation)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViReal64 (instrumentHandle, "P25 encode C4FM ", C4FMModulation));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Get_Enable_State_RLMeter                                
* Purpose:  This function returns the enable mode status of the receive     
*           level meter.                 
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_Enable_State_RLMeter (ViSession instrumentHandle,
                                                  ViInt16 *enableMode)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    ViChar rdbuf[BUFFER_SIZE];
    
    // Formats string to be transmitted 
    sprintf ((char*) wrtbuf, "RLmeter enable ");
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViInt16 (instrumentHandle, "RLmeter enable ", enableMode));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Get_Update_Rate_RLMeter                                 
* Purpose:  This function returns the update rate of the receive            
*           level meter.                 
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_Update_Rate_RLMeter (ViSession instrumentHandle,
                                                  ViInt16 *updateRate)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViInt16 (instrumentHandle, "RLmeter RATe ", updateRate));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Get_Cable_Loss_RLMeter                                  
* Purpose:  This function returns the cable loss figure being used with the 
*           receive level meter.         
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_Cable_Loss_RLMeter (ViSession instrumentHandle,
                                                 ViReal64 *cableLoss)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViReal64 (instrumentHandle, "RLmeter CABLEloss ", cableLoss));
    
    return ifr2975_status;
}

/******************************************************************************
* Function: ifr2975_Get_Vert_Scale_SA    
* Purpose:  This function returns the spectrum analyzer vertical scale      
*           setting                      
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_Vert_Scale_SA (ViSession instrumentHandle,
                                            ViInt16 *verticalScale)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViInt16 (instrumentHandle, "Analyzer verticalSCALE ", verticalScale));
    
    return ifr2975_status;
}

/******************************************************************************
* Function: ifr2975_Get_TOS_SA           
* Purpose:  This function returns the spectrum analyzer top of screen      
*           (reference level) setting    
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_TOS_SA (ViSession instrumentHandle,
                                     ViReal64 *topofScreen)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViReal64 (instrumentHandle, "Analyzer topofscreen ", topofScreen));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Get_Span_SA          
* Purpose:  This function returns the spectrum analyzer start and start    
*           frequency setting            
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_Span_SA (ViSession instrumentHandle,
                                              ViReal64 *startFreq,
                                              ViReal64 *stopFreq)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    ViChar rdbuf[BUFFER_SIZE];
    
    char *End_Pointer;

    // Formats string to be transmitted 
    sprintf ((char*) wrtbuf, "Analyzer SPAN ");
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViString (instrumentHandle, wrtbuf, rdbuf, 100));
    
    sscanf((const char *)rdbuf, "%lf", startFreq);
    
    strtod ((const char *) rdbuf, &End_Pointer);
    
    sscanf(End_Pointer, "%lf", stopFreq);
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Get_RBW_SA           
* Purpose:  This function returns the spectrum analyzer resolution bandwidth
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_RBW_SA (ViSession instrumentHandle,
                                     ViReal64 *resolutionBandwidth)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViReal64 (instrumentHandle, "Analyzer resolutionbandwidth ", resolutionBandwidth));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Get_VBW_SA           
* Purpose:  This function returns the spectrum analyzer video  bandwidth   
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_VBW_SA (ViSession instrumentHandle,
                                     ViReal64 *videoBandwidth)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViReal64 (instrumentHandle, "Analyzer videobandwidth ", videoBandwidth));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Get_IFGain_SA        
* Purpose:  This function returns the spectrum analyzer IF Gain             
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_IFGain_SA (ViSession instrumentHandle,
                                         ViReal64 *IFGain)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViReal64 (instrumentHandle, "Analyzer IFGAIN ", IFGain));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Get_Run_State_SA     
* Purpose:  This function returns the spectrum analyzer runstate           
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_Run_State_SA (ViSession instrumentHandle,
                                           ViInt16 *runState)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViInt16 (instrumentHandle, "Analyzer RUNanalyzer ", runState));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Get_Norm_State_SA    
* Purpose:  This function returns the spectrum analyzer normalize state     
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_Norm_State_SA (ViSession instrumentHandle,
                                         ViInt16 *normalizeState)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViInt16 (instrumentHandle, "Analyzer normalize ", normalizeState));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Get_Sweep_SA         
* Purpose:  This function returns the spectrum analyzer sweep speed        
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_Sweep_SA (ViSession instrumentHandle,
                                       ViReal64 *sweepSpeed)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViReal64 (instrumentHandle, "Analyzer SWEEPspeed ", sweepSpeed));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Query_Ver_SA         
* Purpose:  This function returns the spectrum analyzer driver/engine      
*           version                      
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Query_Ver_SA (ViSession instrumentHandle,
                                       ViChar version[], ViInt16 ID)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    ViChar rdbuf[BUFFER_SIZE];
    
    // Verify ID range
    // 1=Driver
    // 2=Engine
    if (ifr2975_invalidViInt16Range (ID, 1, 2))
    {
      return VI_ERROR_PARAMETER2;
    }
    
    // Formats string to be transmitted 
    sprintf ((char*) wrtbuf, "Analyzer VER %i ",ID);
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViString (instrumentHandle, wrtbuf, version, 100));
    
    return ifr2975_status;
}



/******************************************************************************
* Function: ifr2975_Get_Tracking_Gen_State_SA  
* Purpose:  This function returns the spectrum analyzer runstate           
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_Tracking_Gen_State_SA (ViSession instrumentHandle,
                                           ViInt16 *tracking)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViInt16 (instrumentHandle, "Analyzer tgstate ", tracking));
    
    return ifr2975_status;
}



/******************************************************************************
* Function: ifr2975_Meas_Freq_AFC        
* Purpose:  This function returns the current AF Counter reading.          
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Meas_Freq_AFC (ViSession instrumentHandle,
                                        ViReal64 *frequency)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViReal64 (instrumentHandle, "AFCounter value ", frequency));
    
    return ifr2975_status;
}
    

/******************************************************************************
* Function: ifr2975_Query_Ver_AFC        
* Purpose:  This function queries the AF Counter for driver/engine version. 
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Query_Ver_AFC (ViSession instrumentHandle,
                                        ViInt16 system, ViChar version[])
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    
    // Verify system range
    // 1=Driver
    // 2=Engine
    if (ifr2975_invalidViInt16Range (system, 0, 1))
    {
      return VI_ERROR_PARAMETER2;
    }
    
    // Formats string to be transmitted 
    sprintf ((char*) wrtbuf, "AFCounter VER %i ",system);
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViString (instrumentHandle, wrtbuf, version, 100));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Meas_Dev_FMDM        
* Purpose:  This function returns the current FM Deviation meter reading. 
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Meas_Dev_FMDM (ViSession instrumentHandle,
                                        ViReal64 *deviation)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViReal64 (instrumentHandle, "FMDev value ", deviation));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Query_Ver_FMDM       
* Purpose:  This function queries and returns driver/engine version info    
*           for the FM deviation meter.  
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Query_Ver_FMDM (ViSession instrumentHandle,
                                         ViInt16 system, ViChar version[])
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    
    // Verify system range
    // 1=Driver
    // 2=Engine
    if (ifr2975_invalidViInt16Range (system, 0, 1))
    {
      return VI_ERROR_PARAMETER2;
    }
    
    // Formats string to be transmitted 
    sprintf ((char*) wrtbuf, "FMDev VER %i ",system);
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViString (instrumentHandle, wrtbuf, version, 100));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Meas_SINAD_SINAD     
* Purpose:  This function returns the current SINAD meter reading.          
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Meas_SINAD_SINAD (ViSession instrumentHandle,
                                           ViReal64 *SINAD)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViReal64 (instrumentHandle, "Sinad value ", SINAD));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Query_Ver_SINAD      
* Purpose:  This function queries and returns driver/engine version info    
*           for the SINAD meter.         
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Query_Ver_SINAD (ViSession instrumentHandle,
                                          ViInt16 system, ViChar version[])
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    ViChar rdbuf[BUFFER_SIZE];
    
    // Verify system range
    // 1=Driver
    // 2=Engine
    if (ifr2975_invalidViInt16Range (system, 0, 1))
    {
      return VI_ERROR_PARAMETER2;
    }
    
    // Formats string to be transmitted 
    sprintf ((char*) wrtbuf, "Sinad VER %i ",system);
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViString (instrumentHandle, wrtbuf, version, 100));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Meas_Dist_DIST       
* Purpose:  This function  returns the latest distortion meter reading.    
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Meas_Dist_DIST (ViSession instrumentHandle,
                                         ViReal64 *distortion)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViReal64 (instrumentHandle, "Dist Value ", distortion));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Query_Ver_DIST       
* Purpose:  This function queries and returns driver/engine version info   
*           for the Distortion meter.    
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Query_Ver_DIST (ViSession instrumentHandle,
                                         ViInt16 system, ViChar version[])
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    ViChar rdbuf[BUFFER_SIZE];
    
    // Verify system range
    // 1=Driver
    // 2=Engine
    if (ifr2975_invalidViInt16Range (system, 0, 1))
    {
      return VI_ERROR_PARAMETER2;
    }
    
    // Formats string to be transmitted 
    sprintf ((char*) wrtbuf, "Dist VER %i ",system);
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViString (instrumentHandle, wrtbuf, version, 100));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Query_Ver_FGEN       
* Purpose:  This function queries and returns driver/engine version info    
*           for the function generator.  
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Query_Ver_FGEN (ViSession instrumentHandle,
                                         ViInt16 system, ViChar version[])
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    
    // Verify system range
    // 1=Driver
    // 2=Engine
    if (ifr2975_invalidViInt16Range (system, 0, 1))
    {
      return VI_ERROR_PARAMETER2;
    }
    
    // Formats string to be transmitted 
    sprintf ((char*) wrtbuf, "FGen VER %i ",system);
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViString (instrumentHandle, wrtbuf, version, 100));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Query_Ver_BBA  
* Purpose:  This function queries the 2975 for the version.      
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Query_Ver_BBA (ViSession instrumentHandle,
                                        ViInt16 system, ViChar version[])
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    
    // Verify system range
    // 1=Driver
    // 2=Engine
    if (ifr2975_invalidViInt16Range (system, 0, 1))
    {
      return VI_ERROR_PARAMETER2;
    }
    
    // Formats string to be transmitted 
    sprintf ((char*) wrtbuf, "BBAudio VER %i ",system);
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViString (instrumentHandle, wrtbuf, version, 100));
    
    return ifr2975_status;
}

/******************************************************************************
* Function: ifr2975_Meas_Freq_Err_RFERR  
* Purpose:  This function  returns the latest RF Error meter reading.      
*                                        
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Meas_Freq_Err_RFERR (ViSession instrumentHandle,
                                              ViReal64 *frequencyError)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViReal64 (instrumentHandle, "RFError Value ", frequencyError));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Query_Ver_RFERR      
* Purpose:  This function returns RF Error Meter Driver/Enginer version    
*           information.                 
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Query_Ver_RFERR (ViSession instrumentHandle,
                                          ViInt16 system, ViChar version[])
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    ViChar rdbuf[BUFFER_SIZE];
    
    // Verify system range
    // 1=Driver
    // 2=Engine
    if (ifr2975_invalidViInt16Range (system, 0, 1))
    {
      return VI_ERROR_PARAMETER2;
    }
    
    // Formats string to be transmitted 
    sprintf ((char*) wrtbuf, "RFError VER %i ", system);
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViString (instrumentHandle, wrtbuf, version, 100));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Meas_Volt_DVM                                          
* Purpose:  This function  returns the latest voltage reading from the DVM.
*                                                                          
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Meas_Volt_DVM (ViSession instrumentHandle,
                                        ViReal64 *voltage)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViReal64 (instrumentHandle, "Dvm value ", voltage));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Query_Ver_DVM                       
* Purpose:  This function queries and returns driver/engine version info 
*           for the DVM.                                                 
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Query_Ver_DVM (ViSession instrumentHandle,
                                        ViInt16 system, ViChar version[])
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    ViChar rdbuf[BUFFER_SIZE];
    
    // Verify system range
    // 1=Driver
    // 2=Engine
    if (ifr2975_invalidViInt16Range (system, 0, 1))
    {
      return VI_ERROR_PARAMETER2;
    }
    
    // Formats string to be transmitted 
    sprintf ((char*) wrtbuf, "Dvm VER %i ", system);
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViString (instrumentHandle, wrtbuf, version, 100));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Meas_Error_C4FM                                         
* Purpose:  This function  returns the latest voltage reading from the C4FM
*           meter.                                                         
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Meas_Error_C4FM (ViSession instrumentHandle,
                                          ViReal64 *error)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViReal64 (instrumentHandle, "C4fm value ", error));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Query_Ver_C4FM                                         
* Purpose:  This function queries and returns driver/engine version info   
*           for the DVM.                                                   
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Query_Ver_C4FM (ViSession instrumentHandle,
                                         ViInt16 system, ViChar version[])
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    
    // Verify system range
    // 1=Driver
    // 2=Engine
    if (ifr2975_invalidViInt16Range (system, 0, 1))
    {
      return VI_ERROR_PARAMETER2;
    }
    
    // Formats string to be transmitted 
    sprintf ((char*) wrtbuf, "C4fm VER %i ",system);
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViString (instrumentHandle, wrtbuf, version, 100));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Trig_Arm_OSCOPE                                        
* Purpose:  This function arms a one-shot trigger mode on the oscilloscope.
*                                                                          
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Trig_Arm_OSCOPE (ViSession instrumentHandle)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_setViString (instrumentHandle, "Scope trigarm ", 0));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Query_Ver_OSCOPE                                        
* Purpose:  This function queries the 2975 for the version.
*                                                                          
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Query_Ver_OSCOPE (ViSession instrumentHandle,
                                           ViInt16 system, ViChar version[])
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    
    // Verify system range
    // 1=Driver
    // 2=Engine
    if (ifr2975_invalidViInt16Range (system, 1, 2))
    {
      return VI_ERROR_PARAMETER2;
    }
    
    // Formats string to be transmitted 
    sprintf ((char*) wrtbuf, "Scope VER %i ",system);
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViString (instrumentHandle, wrtbuf, version, 100));
    
    return ifr2975_status;
}

/******************************************************************************
* Function: ifr2975_Query_Ver_REC                                        
* Purpose:  This function queries the 2975 for the version.
*                                                                          
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Query_Ver_REC (ViSession instrumentHandle,
                                           ViInt16 system, ViChar version[])
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    
    // Verify system range
    // 1=Driver
    // 0=Engine
    if (ifr2975_invalidViInt16Range (system, 0, 1))
    {
      return VI_ERROR_PARAMETER2;
    }
    
    // Formats string to be transmitted 
    sprintf ((char*) wrtbuf, "Receiver VER %i ",system);
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViString (instrumentHandle, wrtbuf, version, 100));
    
    return ifr2975_status;
}

/******************************************************************************
* Function: ifr2975_Query_Ver_GEN                                        
* Purpose:  This function queries the 2975 for the version.
*                                                                          
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Query_Ver_GEN (ViSession instrumentHandle,
                                           ViInt16 system, ViChar version[])
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    
    // Verify system range
    // 1=Driver
    // 0=Engine
    if (ifr2975_invalidViInt16Range (system, 0, 1))
    {
      return VI_ERROR_PARAMETER2;
    }
    
    // Formats string to be transmitted 
    sprintf ((char*) wrtbuf, "Generator VER %i ",system);
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViString (instrumentHandle, wrtbuf, version, 100));
    
    return ifr2975_status;
}

/******************************************************************************
* Function: ifr2975_Query_Ver_APCO                                        
* Purpose:  This function queries the 2975 for the version.
*                                                                          
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Query_Ver_APCO (ViSession instrumentHandle,
                                         ViInt16 system, ViChar version[])
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    
    // Verify system range
    // 1=Driver
    // 2=Engine
    if (ifr2975_invalidViInt16Range (system, 0, 1))
    {
      return VI_ERROR_PARAMETER2;
    }
    
    // Formats string to be transmitted 
    sprintf ((char*) wrtbuf, "P25 encode VER %i ",system);
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViString (instrumentHandle, wrtbuf, version, 100));
    
    return ifr2975_status;
}

/******************************************************************************
* Function: ifr2975_Meas_Power_WBPWR                                       
* Purpose:  This function returns the latest power meter reading.          
*
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Meas_Power_WBPWR (ViSession instrumentHandle,
                                           ViReal64 *reading)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViReal64 (instrumentHandle, "Power value ", reading));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: ifr2975_Meas_RLMeter                                           
* Purpose:  This function returns the latest receive level meter reading.  
*                                                                          
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Meas_RLMeter (ViSession instrumentHandle,
                                       ViReal64 *receivedLevel)
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViReal64 (instrumentHandle, "RLmeter value ", receivedLevel));
    
    return ifr2975_status;
}

/******************************************************************************
* Function: ifr2975_Reset      
* Purpose:  This function resets the 2975 to a default state.              
*
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Reset (ViSession instrumentHandle)
{
    ViStatus ifr2975_status = VI_SUCCESS;

    /*=CHANGE:=============================================================*/
    /* The following lines are used to Reset the instrument to a known     */
    /* state.  Replace the string "*RST" with the reset string you wish to */
    /* send to the instrument.                                             */
    /*=====================================================================*/

    /*  Initialize the instrument to a known state.  */
    CHECKERR (ifr2975_defaultInstrSetup (instrumentHandle));
        
    /*=CHANGE: ==============================================================*/
    /* If the Instrument Does Not support a Reset, delete the above section  */
    /* that sends the Reset command  to the instrument and use the following */
    /* line of code to return the proper warning status code.                */
    /*                                                                       */
    /* ifr2975_status = VI_WARN_NSUP_RESET;                                   */ 
    /*                                                                       */
    /* If a Reset is supported delete this section of comment and code.      */
    /*=======================================================================*/
       
    return ifr2975_status;
}

/******************************************************************************
* Function: Error Query                                                     
* Purpose:  This function queries the instrument error queue, and returns   
*           the result. If the error query function is not supported by the 
*           instrument, this function returns the warning                   
*           VI_WARN_NSUP_ERROR_QUERY.                                       
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Error_Query (ViSession instrumentHandle,
                                      ViInt32 *errorCode, ViChar errorMessage[])
{
    ViStatus ifr2975_status = VI_SUCCESS;

    /*=CHANGE:=============================================================*/
    /* Change the :SYST:ERR? command to the Error Query command for your   */
    /* device.                                                             */
    /*=====================================================================*/

    //CHECKERR (viPrintf (instrSession, ":SYST:ERR?"));

    /*=CHANGE:=============================================================*/
    /* Parse the error message and return the Error Code and Error String. */
    /* This code makes the following assumption: The String returned from  */
    /* the instrument is in the format set forth in the SCPI 1994 Command  */
    /* Reference which is <Error/Event Number,"Error/Event Descriptor">,   */
    /* for example  0,"No Error" or -410,"Query Interrupted".              */
    /*=====================================================================*/

    //CHECKERR (viScanf (instrSession, "%ld,\"%[^\"]", errorCode, errorMessage));

    /*=CHANGE: ==============================================================*/
    /* If the Instrument Does Not support an Error Query, delete the above   */
    /* section that sends the Error Query command to the instrument and      */
    /* parses the response and use the following line of code to return the  */ 
    /* proper warning status code.                                           */
    /*                                                                       */
	
	ifr2975_status = VI_WARN_NSUP_ERROR_QUERY;                            
    
    /*                                                                       */ 
    /* If a Error Query is supported delete this section of comment and code.*/                                          
    /*=======================================================================*/

    return ifr2975_status;
}

/******************************************************************************
* Function: Error Message                                                   
* Purpose:  This function translates the error return value from the        
*           instrument driver into a user-readable string.                  
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Error_Message (ViSession instrumentHandle,
                                        ViStatus statusCode, ViChar errMessage[])
{
    ViStatus ifr2975_status = VI_SUCCESS;
    ViInt16 i;

    static ifr2975_tStringValPair statusDescArray[] = {
        {VI_WARN_NSUP_ID_QUERY,     "WARNING: ID Query not supported"},
        {VI_WARN_NSUP_RESET,        "WARNING: Reset not supported"},
        {VI_WARN_NSUP_SELF_TEST,    "WARNING: Self-test not supported"},
        {VI_WARN_NSUP_ERROR_QUERY,  "WARNING: Error Query not supported"},     
        {VI_WARN_NSUP_REV_QUERY,    "WARNING: Revision Query not supported"},
        {VI_ERROR_PARAMETER1,   "ERROR: Parameter 1 out of range"},
        {VI_ERROR_PARAMETER2,   "ERROR: Parameter 2 out of range"},
        {VI_ERROR_PARAMETER3,   "ERROR: Parameter 3 out of range"},
        {VI_ERROR_PARAMETER4,   "ERROR: Parameter 4 out of range"},
        {VI_ERROR_PARAMETER5,   "ERROR: Parameter 5 out of range"},
        {VI_ERROR_PARAMETER6,   "ERROR: Parameter 6 out of range"},
        {VI_ERROR_PARAMETER7,   "ERROR: Parameter 7 out of range"},
        {VI_ERROR_PARAMETER8,   "ERROR: Parameter 8 out of range"},
        {VI_ERROR_FAIL_ID_QUERY,"ERROR: Identification query failed"},
        {VI_ERROR_INV_RESPONSE, "ERROR: Interpreting instrument response"},
        {VI_ERROR_INSTR_FILE_OPEN,    "ERROR: Opening the specified file"},
        {VI_ERROR_INSTR_FILE_WRITE,   "ERROR: Writing to the specified file"},
        {VI_ERROR_INSTR_INTERPRETING_RESPONSE, "ERROR: Interpreting the instrument's response"},
        {VI_ERROR_INSTR_PARAMETER9 ,  "ERROR: Parameter 9 out of range"},
        {VI_ERROR_INSTR_PARAMETER10,  "ERROR: Parameter 10 out of range"},
        {VI_ERROR_INSTR_PARAMETER11,  "ERROR: Parameter 11 out of range"},
        {VI_ERROR_INSTR_PARAMETER12,  "ERROR: Parameter 12 out of range"},
        {VI_ERROR_INSTR_PARAMETER13,  "ERROR: Parameter 13 out of range"},
        {VI_ERROR_INSTR_PARAMETER14,  "ERROR: Parameter 14 out of range"},
        {VI_ERROR_INSTR_PARAMETER15,  "ERROR: Parameter 15 out of range"},

        /************************************************************************
        // Insert instrument-specific error codes here.  Example:              
        ************************************************************************/

        {ifr2975_ERROR_INSTRUMENT_ERROR,         "ERROR: Instrument status error"},
        {ifr2975_ERROR_INVALID_CONFIGURATION,    "ERROR: Instrument configuration error"},
        {ifr2975_ERROR_INSTRUMENT_OPTION,        "ERROR: Option not installed"},
        {ifr2975_ERROR_INSTRUMENT_NSUP_MODEL,    "ERROR: Required Instrument model is not available."},
        {ifr2975_ERROR_SETTINGS_CONFLICT,        "ERROR: Instrument settings conflict."},
        {VI_NULL, VI_NULL}
    };

    ifr2975_status = viStatusDesc (instrumentHandle, statusCode, errMessage);
    
    if (ifr2975_status == VI_WARN_UNKNOWN_STATUS)
    {
      for (i = 0; statusDescArray[i].stringName; i++)
      {
        if (statusDescArray[i].stringVal == statusCode)
        {
          sprintf (errMessage, "%s", statusDescArray[i].stringName);
          return (VI_SUCCESS);
        }
      }
    
      sprintf (errMessage, "Unknown Error 0x%X", statusCode);
      
      return (VI_WARN_UNKNOWN_STATUS);
    }

    ifr2975_status = VI_SUCCESS;
    
    return ifr2975_status;
}

/******************************************************************************
* Function: Revision Query                                                  
* Purpose:  This function returns the driver and instrument revisions.      
*           If the revision query function is not supported by the           
*           instrument, this function returns the warning                   
*           VI_WARN_NSUP_REV_QUERY.                                         
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Revision_Query (ViSession instrumentHandle,
                                         ViChar driverRev[], ViChar intrRev[])
{
    ViStatus ifr2975_status = VI_SUCCESS;
	ViChar rdbuf[BUFFER_SIZE];
	
    /*=CHANGE:=============================================================*/
    /* Query instrument firmware revision.                                 */
    /* Change the ID Query command to the ID Query command for your device.*/
    /*=====================================================================*/
	// CHECKERR (viWrite(instrumentHandle,"*idn? \n",strlen ("*idn? \n"),0));
	CHECKERR (viWrite(instrumentHandle, (ViBuf) "*idn? \n",strlen ("*idn? \n"),0));
	
	// CHECKERR (viRead (instrumentHandle, rdbuf, 10000, 0));
	CHECKERR (viRead (instrumentHandle, (ViBuf) rdbuf, 10000, 0));

    //  CHECKERR (viPrintf (instrSession, "*IDN?"));

	sscanf (rdbuf, "%s,%s,%s,%s", rdbuf, rdbuf, rdbuf, intrRev);
    
    /*=CHANGE:=============================================================*/
    /* This code makes the following assumption: The String returned from  */
    /* the instrument is in the format set forth in the SCPI 1994 Command  */
    /* Reference                                                           */
    /*=====================================================================*/
    //CHECKERR (viScanf (instrSession, "%*[^,],%*[^,],%*[^,],%[^\r\n]", instrRev));

    /*=CHANGE: ==============================================================*/
    /* If the Instrument Does Not support a Revision Query, delete the above */
    /* section that sends the Revision Query command to the instrument and   */
    /* parses the response and use the following line of code to return the  */ 
    /* proper warning status code.                                           */
    /*                                                                       */
    
    //ifr2975_status = VI_WARN_NSUP_REV_QUERY;
    
    /*                                                                       */
    /* If a Revision Query is supported delete this section of comment and   */
    /* code.                                                                 */
    /*=======================================================================*/
    
    sprintf (driverRev, "%s", ifr2975_REVISION_DRIVER);
    sprintf (intrRev, "%s", rdbuf);
    
    return ifr2975_status;
}


/*===========================================================================*/
/* Function: Write To Instrument                                             */
/* Purpose:  This function writes a command string to the instrument.        */
/*===========================================================================*/
ViStatus _VI_FUNC ifr2975_writeInstrData (ViSession instrumentHandle,
                                         ViChar writeBuffer[])
{
    ViStatus ifr2975_status = VI_SUCCESS;
    
    CHECKERR (viPrintf (instrumentHandle, "%s", writeBuffer));
    
    return ifr2975_status;
}

/*===========================================================================*/
/* Function: Read Instrument Buffer                                          */
/* Purpose:  This function reads the output buffer of the instrument.        */
/*===========================================================================*/
ViStatus _VI_FUNC ifr2975_readInstrData (ViSession instrumentHandle,
                                        ViInt32 numberBytesToRead,
                                        ViChar readBuffer[],
                                        ViInt32 *numBytesRead)
{
    ViStatus    ifr2975_status = VI_SUCCESS;
    ViUInt32    tmpCnt;

    if (numBytesRead)
    {
      *numBytesRead = 0L;
    }
        
    CHECKERR (viRead (instrumentHandle, (ViBuf)readBuffer, numberBytesToRead, &tmpCnt));
    
    if (numBytesRead)
    {
      *numBytesRead = (ViInt32)tmpCnt;
    }
    
    return ifr2975_status;
}

/******************************************************************************
* Function: ifr2975_Read_Write_From_Instrument                              
* Purpose:  This function commands writes to the instrument.
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Read_Write_From_Instrument (ViSession instrumentHandle,
                                                     ViChar writeData[],
                                                     ViChar readData[])
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViString (instrumentHandle, writeData, readData, 10000));
    
    return ifr2975_status;
}

/******************************************************************************
* Function: Self-Test                                                       
* Purpose:  This function executes the instrument self-test and returns     
*           the result. If the self test function is not supported by the   
*           instrument, this function returns the warning                   
*           VI_WARN_NSUP_SELF_TEST.                                         
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Self_Test (ViSession instrumentHandle,
                                    ViInt16 *testResult, ViChar testMessage[])
{
    ViStatus ifr2975_status = VI_SUCCESS;
	
    /*=CHANGE:=============================================================*/
    /* Change the *TST? command to the self-test command for your device.  */
    /*=====================================================================*/

    //CHECKERR (viPrintf (instrSession, "*TST?"));

    /*=CHANGE:=============================================================*/
    /* Test the test_message to see if a self-test error occurred. Assign  */
    /* the result to *test_result and return. Zero (0) means that the test */
    /* executed successfully any other number means selftest failed see    */
    /* control help for more details.                                      */
    /* This code makes the following assumption: The String returned from  */
    /* the instrument is in the format set forth in the SCPI 1994 Command  */
    /* Reference                                                           */
    /*=====================================================================*/
    
    //CHECKERR (viScanf (instrSession, "%hd,\"%[^\"]", testResult, testMessage));
    
    /*=CHANGE: ==============================================================*/
    /* If the Instrument Does Not support a Self Test, delete the above      */
    /* section that sends the Self Test command to the instrument and parses */
    /* the response and use the following line of code to return the proper  */ 
    /* warning status code.                                                  */
    /*                                                                       */
    
    ifr2975_status = VI_WARN_NSUP_SELF_TEST;                             
    
    /*                                                                       */ 
    /* If a Self Test is supported delete this section of comment and code.  */
    /*=======================================================================*/

    return ifr2975_status;
}

/******************************************************************************
*= UTILITY ROUTINES (Non-Exportable Functions) ================================
******************************************************************************/

/******************************************************************************
* Function: Boolean Value Out Of Range - ViBoolean                          
* Purpose:  This function checks a Boolean to see if it is equal to VI_TRUE 
*           or VI_FALSE. If the value is out of range, the return value is  
*           VI_TRUE, otherwise the return value is VI_FALSE.                
*****************************************************************************/
ViBoolean ifr2975_invalidViBooleanRange (ViBoolean val)
{
    return ((val != VI_FALSE && val != VI_TRUE) ? VI_TRUE : VI_FALSE);
}

/******************************************************************************
* Function: Short Signed Integer Value Out Of Range - ViInt16               
* Purpose:  This function checks a short signed integer value to see if it    
*           lies between a minimum and maximum value.  If the value is out  
*           of range, the return value is VI_TRUE, otherwise the return     
*           value is VI_FALSE.           
******************************************************************************/
ViBoolean ifr2975_invalidViInt16Range (ViInt16 val, ViInt16 min, ViInt16 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/******************************************************************************
* Function: Long Signed Integer Value Out Of Range - ViInt32                
* Purpose:  This function checks a long signed integer value to see if it     
*           lies between a minimum and maximum value.  If the value is out  
*           of range, the return value is VI_TRUE, otherwise the return     
*           value is VI_FALSE.           
******************************************************************************/
ViBoolean ifr2975_invalidViInt32Range (ViInt32 val, ViInt32 min, ViInt32 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/******************************************************************************
* Function: Unsigned Char Value Out Of Range - ViUInt8                      
* Purpose:  This function checks an unsigned char value to see if it          
*           lies between a minimum and maximum value.  If the value is out  
*           of range, the return value is VI_TRUE, otherwise the return     
*           value is VI_FALSE.           
******************************************************************************/
ViBoolean ifr2975_invalidViUInt8Range (ViUInt8 val, ViUInt8 min, ViUInt8 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/******************************************************************************
* Function: Short Unsigned integer Value Out Of Range - ViUInt16            
* Purpose:  This function checks a short unsigned integer value to see if it  
*           lies between a minimum and maximum value.  If the value is out  
*           of range, the return value is VI_TRUE, otherwise the return     
*           value is VI_FALSE.           
******************************************************************************/
ViBoolean ifr2975_invalidViUInt16Range (ViUInt16 val, ViUInt16 min, ViUInt16 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/******************************************************************************
* Function: Long Unsigned integer Value Out Of Range - ViUInt32             
* Purpose:  This function checks a long unsigned integer value to see if it   
*           lies between a minimum and maximum value.  If the value is out  
*           of range, the return value is VI_TRUE, otherwise the return     
*           value is VI_FALSE.           
******************************************************************************/
ViBoolean ifr2975_invalidViUInt32Range (ViUInt32 val, ViUInt32 min, ViUInt32 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/******************************************************************************
* Function: Real (Float) Value Out Of Range - ViReal32                      
* Purpose:  This function checks a real (float) value to see if it lies      
*           between a minimum and maximum value.  If the value is out of    
*           range, the return value is VI_TRUE, otherwise the return value  
*           is VI_FALSE.                 
******************************************************************************/
ViBoolean ifr2975_invalidViReal32Range (ViReal32 val, ViReal32 min, ViReal32 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/******************************************************************************
* Function: Real (Double) Value Out Of Range - ViReal64                     
* Purpose:  This function checks a real (double) value to see if it lies      
*           between a minimum and maximum value.  If the value is out of    
*           range, the return value is VI_TRUE, otherwise the return value  
*           is VI_FALSE.                 
******************************************************************************/
ViBoolean ifr2975_invalidViReal64Range (ViReal64 val, ViReal64 min, ViReal64 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}


/******************************************************************************
* Function: ifr2975_Query_Ver_RLMETER         
* Purpose:  This function queries the 2975 for the version.         
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Query_Ver_RLMETER (ViSession instrumentHandle,
                                            ViInt16 system, ViChar version[])
{
    ViStatus ifr2975_status = VI_SUCCESS; 
    ViChar wrtbuf[BUFFER_SIZE];
    
    // Verify system range
    // 1=Driver
    // 2=Engine
    if (ifr2975_invalidViInt16Range (system, 0, 1))
    {
      return VI_ERROR_PARAMETER2;
    }
    
    // Formats string to be transmitted 
    sprintf ((char*) wrtbuf, "RLmeter VER %i ",system);
    
    // This writes command to instrument 
    CHECKERR (ifr2975_getViString (instrumentHandle, wrtbuf, version, 100));
    
    return ifr2975_status;
}

/******************************************************************************
* Function: ifr2975_Get_Trace_SA         
* Purpose:  This function returns the current trace of the Spectrum         
*           Analyzer.                    
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Get_Trace_SA (ViSession instrumentHandle,
                        ViInt32 *length_array, ViReal64 *TEMP_DATA)
{
    ViStatus ifr2975_status = VI_SUCCESS;
    ViByte rdbuf[10000];
    ViByte wrtbuf[10000];
    ViInt16 i;
    char *End_Pointer;
    
    ifr2975_delay (1);
    
    sprintf((char *) wrtbuf, "Analyzer trace");

    // This writes command to instrument 
    CHECKERR (ifr2975_getViString (instrumentHandle, (ViString) wrtbuf, (ViString) rdbuf, 10000));

    // Gets the length of the array value from the string */
    sscanf((const char *) rdbuf, "%d", length_array);

    // Removes the first value in the returned string (length value) */
    strtod ((const char *) rdbuf, &End_Pointer);
    
    // Process Oscilloscope Trace Data */
    for (i = 0; i <=(*length_array-1); i++) 
    {
      sscanf(End_Pointer, "%lf", &TEMP_DATA[i]);
      strtod (End_Pointer, &End_Pointer);
    };
    
    return ifr2975_status;
}


/*****************************************************************************
* Function: ifr2975_Close_Conn           
* Purpose:  This function closes the 2975 instrument session.               
*
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Close_Conn (ViSession instrumentHandle)
{
    ifr2975_instrRange instrPtr;
    ViSession rmSession;
    ViStatus ifr2975_status = VI_SUCCESS;

    if ((ifr2975_status = viGetAttribute (instrumentHandle, VI_ATTR_RM_SESSION, &rmSession)) < 0)
    {
      return ifr2975_status;
    }
    
    ifr2975_initCleanUp (rmSession, &instrumentHandle, ifr2975_status); 
    
    return ifr2975_status;
}

/*****************************************************************************
* Function: ifr2975_Close           
* Purpose:  This function closes the 2975 instrument session.               
*
******************************************************************************/
ViStatus _VI_FUNC ifr2975_Close (ViSession instrumentHandle)
{
    ifr2975_instrRange instrPtr;
    ViSession rmSession;
    ViStatus ifr2975_status = VI_SUCCESS;

    if ((ifr2975_status = viGetAttribute (instrumentHandle, VI_ATTR_RM_SESSION, &rmSession)) < 0)
    {
      return ifr2975_status;
    }
    
    ifr2975_initCleanUp (rmSession, &instrumentHandle, ifr2975_status); 
    
    return ifr2975_status;
}

/******************************************************************************
* Function: Initialize Clean Up                                             
* Purpose:  This function is used only by the ifr2975_init function.  When  
*           an error is detected this function is called to close the       
*           open resource manager and instrument object sessions and to     
*           set the instrumentHandle that is returned from ifr2975_init to      
*           VI_NULL.                                                        
******************************************************************************/
ViStatus ifr2975_initCleanUp (ViSession openRMSession,
                             ViPSession openInstrSession,
                             ViStatus currentStatus)
{
    ViStatus ifr2975_status = VI_SUCCESS;
    ifr2975_instrRange instrPtr;
    ViByte rdbuf[10000];

    if (viGetAttribute (*openInstrSession, VI_ATTR_USER_DATA, &instrPtr) >= 0)
    {
      if (instrPtr != NULL)
      {
        free (instrPtr);
      }
    }
            
    viClose (*openInstrSession);
    viClose (openRMSession);
    *openInstrSession = VI_NULL;
    
    return currentStatus;
}

/******************************************************************************
* Function: ifr2975_delay                
* Purpose : With a call to ifr2975_delay, the current program is suspended  
*           from execution for the number of seconds specified by the         
*           argument seconds. ifr2975_delay is accurate to a millisecond.   
*           This function is platform independent.                             
******************************************************************************/
void ifr2975_delay (ViReal64 numberOfSeconds)
{
    clock_t StartTime = clock();
    
    while ((((double)(clock() - StartTime))/CLOCKS_PER_SEC) < (numberOfSeconds));

  return;
}


/******************************************************************************
* Function: Set String                      
* Purpose:  This function sets command with string parameter.               
******************************************************************************/
ViStatus ifr2975_setViString (ViSession instrumentHandle, ViString cmd, 
                              ViString value)
{
	ViStatus	ifr2975_status;
	ViByte		rdbuf[256];
	ViByte		buffer[256];
	ViStatus Status;
	
	if (value == 0)
	{
  	 
  	  sprintf ((char *) buffer, "%s \n ", cmd);
	
	  CHECKERR (Status = viWrite (instrumentHandle, buffer, strlen ((const char *) buffer), 0));
	  
	  ifr2975_delay(1.0);
		
	  CHECKERR (viRead (instrumentHandle,rdbuf, 256, 0));

	} 
	else if (value[0] == '2')
	{
  	  sprintf ((char *) buffer, "%s \n ", cmd);
	
	  CHECKERR (Status = viWrite(instrumentHandle,buffer,strlen ((const char *) buffer),0));

	  ifr2975_delay(1.0);
	  
	  // No viRead call
	  // These RCI commands do not return a value
	} 
	else 
	{
	  sprintf ((char *) buffer, "%s %s \n ", cmd, value);
	  
	  CHECKERR (Status = viWrite(instrumentHandle,buffer,strlen ((const char *) buffer),0));

	  ifr2975_delay(1.0);
	  
	  CHECKERR (viRead (instrumentHandle,rdbuf, 256, 0));
	  
	}
    
	CHECKERR (ifr2975_sysStatus (instrumentHandle));
	
	return ifr2975_status;

}

/******************************************************************************
* Function: Get String                      
* Purpose:  This function gets command with string parameter.               
******************************************************************************/
ViStatus ifr2975_getViString (ViSession instrumentHandle, ViString cmd, 
                              ViString value, ViInt32 string_length)
{
	ViStatus	ifr2975_status;
	ViByte		buffer[256];
	ViStatus	Status;
	
	if (value == VI_NULL)
	{
	  return VI_ERROR_PARAMETER2;
	}
	
	sprintf ((char *) buffer, "%s \n ", cmd);
	
	CHECKERR (Status = viWrite(instrumentHandle,buffer,strlen ((const char *) buffer),0));

	ifr2975_delay(1.0);
	
	CHECKERR (viRead (instrumentHandle,(ViPBuf) value, string_length, 0));
	

	CHECKERR (ifr2975_sysStatus (instrumentHandle));
	
	return ifr2975_status;
}


/******************************************************************************
* Function: Status                     
* Purpose:  This function reads the system status byte.                   
******************************************************************************/
ViStatus ifr2975_sysStatus (ViSession instrumentHandle)
{
    ViStatus ifr2975_status = VI_SUCCESS;
    ViInt32 esrByte;
    ifr2975_instrRange instrPtr;

/*    CHECKERR (viGetAttribute (instrumentHandle, VI_ATTR_USER_DATA, &instrPtr));

    if (instrPtr -> errorChecking) { /* Is Error Checking Enabled? */
        /*--------------- Read the Standard Event Register ---------------*/
/*        CHECKERR (viQueryf (instrumentHandle, "*ESR?", "%ld", &esrByte));

        /*--------------- Check if any error or message bit was asserted -*/
/*        if ((esrByte & 0x38) != 0)
            ifr2975_status = ifr2975_ERROR_INSTRUMENT_ERROR;
        else
           ifr2975_status = VI_SUCCESS;
    }
*/

    return ifr2975_status;
}


/******************************************************************************
* Function: Set ViReal64                  
* Purpose:  This function sets command with double parameter.               
******************************************************************************/
ViStatus ifr2975_setViReal64 (ViSession instrumentHandle, ViString cmd, 
                             ViReal64 value, ViReal64 min, ViReal64 max, ViString unit)
{
    ViStatus    ifr2975_status;
    ViByte        rdbuf[256];
    ViByte        buffer[256];
    ViStatus	Status;

    if (((max != 0)||(max != 0)))
    {
      if (ifr2975_invalidViReal64Range (value, min, max))
      {
        return VI_ERROR_PARAMETER2;
      }
    }

    if (unit != 0)
    {
      CHECKERR (sprintf ((char *) buffer, "%s %Lf %s \n ", (char *) cmd, value, (char *) unit));

      CHECKERR (Status = viWrite(instrumentHandle,buffer,strlen ((const char *) buffer),0));
  
      ifr2975_delay(1.0);
      
      CHECKERR (viRead (instrumentHandle,rdbuf, 256, 0));
      
    } 
    else
    {
      CHECKERR (sprintf ((char *) buffer, "%s %Lf \n ", cmd, value));
      
      CHECKERR (Status = viWrite(instrumentHandle,buffer,strlen ((const char *) buffer),0));

      ifr2975_delay(1.0);
      
      CHECKERR (viRead (instrumentHandle,rdbuf, 256, 0));

    }

    CHECKERR (ifr2975_sysStatus (instrumentHandle));
    
    return ifr2975_status;
}

/******************************************************************************
* Function: Get ViReal64                  
* Purpose:  This function gets command with double parameter.               
******************************************************************************/
ViStatus ifr2975_getViReal64 (ViSession instrumentHandle, ViString cmd, 
                             ViReal64 *value)
{
	int formatted;
	ViStatus	ifr2975_status;
	ViByte		rdbuf[256];
	ViByte		buffer[256];
	ViStatus	Status;
	
	if (value == VI_NULL)
	{
	  return VI_ERROR_PARAMETER2;
	}

	CHECKERR (sprintf ((char *) buffer, "%s \n", cmd));
	
	CHECKERR (Status = viWrite(instrumentHandle,buffer,strlen ((const char *)buffer),0));

	ifr2975_delay(1.0);	
	
	CHECKERR (viRead (instrumentHandle, rdbuf, 256, 0));
	
	CHECKERR (ifr2975_sysStatus (instrumentHandle));
	
	return ifr2975_status;
}


/******************************************************************************
* Function: Get Boolean                  
* Purpose:  This function gets command with boolean parameter.                 
******************************************************************************/
ViStatus ifr2975_getViBoolean (ViSession instrumentHandle, ViString cmd,
                             ViBoolean *value)
{
    ViStatus    ifr2975_status;
    ViByte        rdbuf[256];
    ViByte        buffer[256];
    ViStatus	Status;
    
    int formatted;
    
    if (value == VI_NULL)
    {
      return VI_ERROR_PARAMETER2;
    }

    CHECKERR (sprintf ((char *) buffer, "%s \n ", (char *) cmd));
    
    CHECKERR (Status = viWrite(instrumentHandle,buffer,strlen ((const char *) buffer),0));

    ifr2975_delay(1.0);
    
    CHECKERR (viRead (instrumentHandle,rdbuf, 256, 0));
    
    CHECKERR (ifr2975_sysStatus (instrumentHandle));
    
    return ifr2975_status;
}

/******************************************************************************
* Function: Set ViInt32                  
* Purpose:  This function sets command with integer parameter.               
******************************************************************************/
ViStatus ifr2975_setViInt32 (ViSession instrumentHandle, ViString cmd, 
                             ViInt32 value, ViInt32 min, ViInt32 max)
{
    ViStatus    ifr2975_status;
    ViByte        rdbuf[256];
    ViByte        buffer[256];
    ViStatus	Status;
    
    if (((max != 0)||(max != 0)))
    {
      if (ifr2975_invalidViInt32Range (value, min, max))
      {
        return VI_ERROR_PARAMETER2;
      }
    }
    
    // Formats string to be transmitted 
    CHECKERR (sprintf ((char *) buffer, "%s %d \n ", cmd, value));

    CHECKERR (Status = viWrite(instrumentHandle,buffer,strlen ((const char *) buffer),0));

    ifr2975_delay(1.0);
    
    CHECKERR (viRead (instrumentHandle,rdbuf, 256, 0));

    CHECKERR (ifr2975_sysStatus (instrumentHandle));
    
    return ifr2975_status;
}

/******************************************************************************
* Function: Get ViInt32                  
* Purpose:  This function gets command with integer parameter.               
******************************************************************************/
ViStatus ifr2975_getViInt32 (ViSession instrumentHandle, ViString cmd, 
                             ViInt32 *value)
{
    ViStatus    ifr2975_status;
    ViByte        rdbuf[256];
    ViByte        buffer[256];
    ViStatus	Status;
    int formatted;
    
    if (value == VI_NULL)
    {
      return VI_ERROR_PARAMETER2;
    }

    CHECKERR (sprintf ((char *) buffer, "%s \n ", cmd));
    
    CHECKERR (Status = viWrite(instrumentHandle,buffer,strlen ((const char *)buffer),0));

    ifr2975_delay(1.0);
    
    CHECKERR (viRead (instrumentHandle,rdbuf, 256, 0));
    
    CHECKERR (ifr2975_sysStatus (instrumentHandle));
    
    return ifr2975_status;
}


/******************************************************************************
* Function: Set ViInt16                  
* Purpose:  This function sets command with integer parameter.              
******************************************************************************/
ViStatus ifr2975_setViInt16 (ViSession instrumentHandle, ViString cmd, 
                             ViInt16 value, ViInt16 min, ViInt16 max)
{
    ViStatus    ifr2975_status;
    ViByte        rdbuf[256];
    ViByte        buffer[256];
    ViStatus	Status;
    
    if ( (max != 0) || (max != 0) )
    {
      if (ifr2975_invalidViInt16Range (value, min, max))
      {
        return VI_ERROR_PARAMETER2;
      }
    }
    
    // Formats string to be transmitted 
    CHECKERR (sprintf ((char *) buffer, "%s %d \n ", cmd, value));
    
    CHECKERR (Status = viWrite(instrumentHandle,buffer,strlen ((const char *) buffer),0));

    ifr2975_delay(1.0);
    
    CHECKERR (viRead (instrumentHandle,rdbuf, 256, 0));
    
    CHECKERR (ifr2975_sysStatus (instrumentHandle));
    
    return ifr2975_status;
}

/******************************************************************************
* Function: Get ViInt16                  
* Purpose:  This function gets command with integer parameter.               
******************************************************************************/
ViStatus ifr2975_getViInt16 (ViSession instrumentHandle, ViString cmd, 
                             ViInt16 *value)
{
    ViStatus    ifr2975_status;
    ViByte        rdbuf[256];
    ViByte        buffer[256];
    ViInt32        temp;
    int formatted;
    ViStatus	Status;
    
    if (value == VI_NULL)
    {
      return VI_ERROR_PARAMETER2;
    }

    CHECKERR (sprintf ((char *) buffer, "%s \n ", cmd));
    
    CHECKERR (Status = viWrite(instrumentHandle,buffer,strlen ((const char *) buffer),0));

    ifr2975_delay(1.0);
    
    CHECKERR (viRead (instrumentHandle,rdbuf, 256, 0));
    
    *value = temp;
    
    CHECKERR (ifr2975_sysStatus (instrumentHandle));
    
    return ifr2975_status;
}

/******************************************************************************
===== END INSTRUMENT DRIVER SOURCE CODE =======================================
******************************************************************************/


/*===========================================================================*/
/* Function: Default Instrument Setup                                        */
/* Purpose:  This function sends a default setup to the instrument.  This    */
/*           function is called by the ifr2975_reset operation and by the     */
/*           ifr2975_init function if the reset option has not been           */
/*           selected.  This function is useful for configuring any          */
/*           instrument settings that are required by the rest of the        */
/*           instrument driver functions such as turning headers ON or OFF   */
/*           or using the long or short form for commands, queries, and data.*/                                    
/*===========================================================================*/
ViStatus ifr2975_defaultInstrSetup (ViSession instrumentHandle)
{
    ViStatus            ifr2975_status = VI_SUCCESS;
    ifr2975_instrRange   instrPtr;

    /* Determine if the structure has been initialized for the current VISA  */
    /* Session and malloc if it has not.                                     */
    //CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    /*=CHANGE:=============================================================*/
    /* Change to reflect the global status variables that your driver needs*/
    /* to keep track of.  For example trigger mode of each session         */
    /*=====================================================================*/
    //instrPtr -> triggerMode = 0;
    //instrPtr -> val2 = 0;
    //instrPtr -> val3 = 0;
    //strcpy (instrPtr -> instrDriverRevision, ifr2975_REVISION);

    /*=CHANGE:=============================================================*/
    /* Change "HEADERS OFF" to be the default setup command(s) you want to */
    /* send to your device.                                                */
    /*                                                                     */
    /* WARNING: This must either be changed to a valid command for your    */
    /* instrument or deleted.                                              */
    /*=====================================================================*/
    //CHECKERR (viPrintf (instrSession, ":HEADERS OFF"));
	CHECKERR (ifr2975_setViString (instrumentHandle, "DEFAULT", 0));

    return ifr2975_status;
}
